;+
; Project     : SOHO - CDS     
;                   
; Name        : BP_SEEK_POS
;               
; Purpose     : Display and look at BP_SEEK study data
;               
; Explanation : Plots an image formed from the 3 wide slit locations in the 3 lines
;               OV, MgIX and FeXVI.  Allows the user to select an object location
;               using the cursor in the FeXVI image and then suggests the command
;               line for the commanding Perl script  RUN_SCI_ALT.
;               
; Use         : IDL> bp_seek_pos, filename (and/or qlds=qlds)
;   
;               eg
;
;               IDL> bp_seek_pos,'s11454r00'
;               will read, calibrate and display that file
;
;               IDL> bp_seek_pos,'s11454r00',qlds=qlds
;               will read, calibrate and display that file and return the qlds
;               so that it is subsequently quicker just to do
;
;               IDL> bp_seek_pos, qlds
;             
; Inputs      : filename  - name of a FITS file containing BP_SEEK data
;              
;               
; Opt. Inputs : None
;               
; Outputs     : Prints results to screen 
;               
; Opt. Outputs: None
;               
; Keywords    : QLDS - can supply a previously read qlds or return one if the filename
;                      was supplied
;
; Calls       : 
;
; Common      : None
;               
; Restrictions: None
;               
; Side effects: None
;               
; Category    : 
;               
; Prev. Hist. : None
;
; Written     : C D Pike, RAL, 
;               
; Modified    : 
;
; Version     : Version 1, 23-Jun-98 
;-            

pro bp_seek_pos, file, qlds=qlds

if n_params() lt 1 and not keyword_set(qlds) then begin
   print,'Use:  bp_seek_pos, filename, qlds=qlds'
   return
endif

if datatype(qlds,1) ne 'Structure' then begin
   print,'Reading and calibrating file: ',file
   qlds = readcdsfits(file)
   if datatype(qlds,1) ne 'Structure' then begin
      print,'Ooops, no data of that name'
      bell
      return
   endif
endif

vds_debias,qlds,[374,374,261,261]
vds_calib,qlds

ov = gt_windata(qlds,2)
mg = gt_windata(qlds,1)
fe = gt_windata(qlds,0)

ov = reform(ov,135,143)
mg = reform(mg,135,143)
fe = reform(fe,135,143)

window,0,xs=1000,ys=500
!p.multi=[0,3,1]
setflag,miss=-100
plot_image,sigrange(ov),tit='OV',chars=2.5
plot_image,sigrange(mg),tit='Mg IX',chars=2.5
plot_image,sigrange(fe),tit='Fe XVI',chars=2.5

xcen = qlds.header.xcen
ycen = qlds.header.ycen

print,'Use cursor to identify Bright Point'
circle_sym
xc = 1
while xc ge 0 and xc le 134 do begin
   cursor,xc,yc,3,/data
   if xc ge 0 then begin
      xc = fix(xc)
      yc = fix(yc)
      oplot,[xc,xc],[yc,yc],psym=8,syms=6,th=4
      ypos = fix(ycen + (yc-71.5)*1.68)
      xpos = fix(xcen + (xc-67.0)*1.68)
      lr = cdsxy2lr(xpos,ypos)
      pix = fmt_vect([xpos,ypos])
      lrc = fmt_vect([lr(0),lr(1)])
      print,' '
      print,'Solar (X,Y) = '+pix
      print,'Suggested uplink command:'
      print,'     >> run RUN_SCI_ALT ',lr(0),lr(1),'  119  xxxx (= PBP_DIAG cdhs study_id)' 
      print,' '
   endif
endwhile
!p.multi=0

end


