	PRO CDS_MOSAIC, FILE_NAMES, WINDOW, MOSAIC, ORIGIN, SCALE=SCALE, $
		LABEL=LABEL, COUNTDOWN=COUNTDOWN, NOCLEAN=NOCLEAN,	$
		DEBIAS=DEBIAS, NOMISSING=NOMISSING,PIX1=PIX1,PIX2=PIX2
;+
; Project     :	SOHO - CDS
;
; Name	:	CDS_MOSAIC
;
; Purpose     :	Forms a mosaic of CDS images.
;
; Category    :	Class2, CDS, Image_processing
;
; Explanation :	Reads in a series of CDS FITS files, and forms an image which
;		is a mosaic of the selected data window.
;
; Syntax      :	CDS_MOSAIC, FILENAMES, WINDOW, MOSAIC, ORIGIN
;
; Examples    :	CDS_MOSAIC, 's2707r*.fits', 'HE_2_303_78', MOSAIC, ORIGIN
;		CDS_MOSAIC, 2707, 'HE_2_303_78', MOSAIC, ORIGIN
;		CDS_MOSAIC, 's3084r00.fits,s3085r00.fits', 1, MOSAIC, ORIGIN
;		CDS_MOSAIC, [3084, 3085], 1, MOSAIC, ORIGIN
;
; Inputs      :	FILENAMES = A comma separated list of the names of the FITS
;			    files to use in forming the mosaic.  Wildcards are
;			    allowed, but take some time to resolve.
;
;			    Alternatively, one can pass an integer array
;			    containing the study numbers to process.  This is
;			    much faster than using wildcards.
;
;			    One can also pass in a quicklook data structure.
;			    This option is used for rasters formed with the
;			    90x240 arcsecond slit.  The individual exposures
;			    then each form an image, and all the exposures
;			    together form a larger image.
;
;		WINDOW	  = The number of the window to form the mosaic from.
;			    Can be from 0 to N-1.
;
; Opt. Inputs :	None.
;
; Outputs     :	MOSAIC	= An array containing the mosaic.
;
;		ORIGIN	= The origin of the mosaic, i.e. the coordinates of the
;			  lower left pixel.  This parameter can be used in
;			  routines such as PLOT_IMAGE, e.g.
;
;				PLOT_IMAGE, MOSAIC, ORIGIN=ORIGIN, SCALE=2
;
; Opt. Outputs:	None.
;
; Keywords    :	SCALE	  = The scale in arcseconds to use in forming the
;			    mosaic.  The default is 2.
;
;		LABEL	  = Returns the label of the window used in making the
;			    mosaic.
;
;		COUNTDOWN = If set, then a countdown is displayed as the FITS
;			    files are processed.
;
;		NOCLEAN	  = If set, then CDS_CLEAN_IMAGE is not called.
;
;		DEBIAS	  = If set, then VDS_DEBIAS is called.
;
;		NOMISSING = Passed to CDS_CLEAN_IMAGE to clean cosmic rays, but
;			    not fill in missing pixels.
;               PIX1, PIX2  = If present these represent the spectral pixels within 
;                             the chosen data window that are to be averaged.
;
; Calls       :	FXHREAD, FXPAR, COUNTDOWN, READCDSFITS, GT_WINDATA, GT_WINDESC,
;		AVERAGE, CDS_CLEAN_IMAGE
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Prev. Hist. :	None.
;
; History     :	Version 1, William Thompson, GSFC, 14 June 1996
;		Version 2, William Thompson, GSFC, 13 November 1996
;			Added calls to UPD_CDS_POINT
;			Added keyword DEBIAS
;		Version 3, William Thompson, GSFC, 3 February 1998
;			Changed to call GT_WINDATA/GT_WINDESC instead of
;			DETDATA/DETDESC.  This means that windows are specified
;			from 0 to N-1 instead of 1 to N.
;			Only read in the window that will be used.
;		Version 4, C D Pike, RAL, 23-Feb-98.  Skip find_files search
;			if explicit names given
;		Version 5, William Thompson, GSFC, 24-Aug-1999
;			Allow study numbers to be passed instead of filenames.
;			Added keyword NOMISSING
;			Average together overlaid pixels.
;		Version 6, 02-Nov-1999, William Thompson, GSFC
;			Form mosaics out of slit 6 images.
;			Allow quicklook data structures to be passed.
;		Version 7, 17-Feb-2000, Willliam Thompson, GSFC
;			Modify to work with GIS rasters.
;               Version 8, 21-Mar-2000, CDP, Add PIX1/PIX2 keywords
;               Version 9, 11-Jul-2003, William Thompson, GSFC
;                       Handle upside-down images.
;
; Contact     :	WTHOMPSON
;-
;
;
	ON_ERROR, 2
;
	IF N_PARAMS() NE 4 THEN MESSAGE,	$
		'Syntax:  CDS_MOSAIC, FILENAMES, WINDOW, MOSAIC, ORIGIN'
;
;  Determine the scale to use.  The default is 2 arcseconds.
;
	IF N_ELEMENTS(SCALE) NE 1 THEN SCALE = 2.0
;
;  If a quicklook structure was passed, then skip parsing the filenames.
;
	IF DATATYPE(FILE_NAMES,1) NE 'Structure' THEN BEGIN
;
;  If FILENAME is passed as a series of study numbers, then find all the files
;  that go with it.
;
	    IF DATATYPE(FILE_NAMES,1) EQ 'String' THEN FILENAMES=FILE_NAMES $
		    ELSE BEGIN
		FILENAMES = ''
		FOR I=0,N_ELEMENTS(FILE_NAMES)-1 DO BEGIN
		    LIST_EXPER, FILE_NAMES(I), OBS, N_OBS
		    FOR J=0,N_OBS-1 DO IF FILENAMES EQ '' THEN		$
			    FILENAMES = STRTRIM(OBS(J).FILENAME) ELSE	$
			    FILENAMES = FILENAMES + ',' +		$
			    STRTRIM(OBS(J).FILENAME)
		    ENDFOR
	    ENDELSE
;
;  Find all the FITS files specified.
;
	    IF STRPOS(FILENAMES,'*') GE 0 THEN BEGIN
		NAMES = STR_SEP(FILENAMES, ',')
		FILES = FIND_FILES(NAMES(0), '$CDS_FITS_DATA')
		FOR I_NAME = 1,N_ELEMENTS(NAMES)-1 DO FILES =	$
			[FILES, FIND_FILES(NAMES(I_NAME), '$CDS_FITS_DATA')]
	    ENDIF ELSE BEGIN
		FILES = STR_SEP(FILENAMES,',')
	    ENDELSE
	    N_FILES = N_ELEMENTS(FILES)
	END ELSE N_FILES = 1
;
;  Step through each of the FITS files, and extract the pointing information.
;
	FOR I_FILE = 0,N_FILES-1 DO BEGIN
	    IF DATATYPE(FILE_NAMES,1) EQ 'Structure' THEN	$
		    A = COPY_QLDS(FILE_NAMES) ELSE		$
		    A = READCDSFITS(FILES(I_FILE), /HEADER)
	    UPD_CDS_POINT, A
	    XCEN = A.HEADER.XCEN
	    YCEN = A.HEADER.YCEN
	    IXWIDTH = A.HEADER.IXWIDTH
	    IYWIDTH = A.HEADER.IYWIDTH
;
	    IF I_FILE EQ 0 THEN BEGIN
		XMIN = XCEN - IXWIDTH / 2.
		XMAX = XCEN + IXWIDTH / 2.
		YMIN = YCEN - IYWIDTH / 2.
		YMAX = YCEN + IYWIDTH / 2.
	    END ELSE BEGIN
		XMIN = XMIN < (XCEN - IXWIDTH / 2.)
		XMAX = XMAX > (XCEN + IXWIDTH / 2.)
		YMIN = YMIN < (YCEN - IYWIDTH / 2.)
		YMAX = YMAX > (YCEN + IYWIDTH / 2.)
	    ENDELSE
	ENDFOR
;
;  Create an array of the right size.
;
	XMIN = ROUND(XMIN/SCALE)  &  XMAX = ROUND(XMAX/SCALE)
	YMIN = ROUND(YMIN/SCALE)  &  YMAX = ROUND(YMAX/SCALE)
	NNX = XMAX - XMIN + 1
	NNY = YMAX - YMIN + 1
	MOSAIC   = FLTARR(NNX, NNY)
	N_MOSAIC = INTARR(NNX, NNY)
	ORIGIN = [XMIN, YMIN] * SCALE
;
;  Read each of the files, and extract the selected window.
;
	IF KEYWORD_SET(COUNTDOWN) THEN COUNTDOWN, N_ELEMENTS(FILES), /OPEN
	FOR I_FILE = 0,N_FILES-1 DO BEGIN
	    IF KEYWORD_SET(COUNTDOWN) THEN COUNTDOWN, I_FILE
	    IF DATATYPE(FILE_NAMES,1) EQ 'Structure' THEN	$
		    A = COPY_QLDS(FILE_NAMES) ELSE		$
		    A = READCDSFITS(FILES(I_FILE), PRESELECT=WINDOW)
	    IF KEYWORD_SET(DEBIAS) THEN VDS_DEBIAS,A
	    AA = GT_WINDATA(A, WINDOW)
	    DD = GT_WINDESC(A, WINDOW)
;
;  If SOHO is upside-down, then rotate the image.
;
            IF DD.SPACING(1) LT 0 THEN BEGIN
                SZ = SIZE(AA)
                FOR II=0,SZ(1)-1 DO BEGIN
                    AAA = REFORM(AA(II,*,*))
                    AAA = ROTATE(AAA,2)
                    AA(II,*,*) = AAA
                ENDFOR
            ENDIF
;
;  Average over dimensions other than the spatial ones.
;
	    SZ = SIZE(AA)
	    IF SZ(0) EQ 4 THEN AA = AVERAGE(AA,4,MISSING=DD.MISSING)
	    IF A.HEADER.SLIT_NUM NE 6 THEN BEGIN
                IF N_ELEMENTS(PIX1) GT 0 THEN BEGIN
                   AA = AA(PIX1:PIX2,*,*)
                ENDIF
		AA = AVERAGE(AA,1,MISSING=DD.MISSING)
		N_IMAGES = 1
	    END ELSE BEGIN
		N_IMAGES = SZ(2)
		IF N_IMAGES EQ 1 THEN AA = REFORM(AA,/OVERWRITE)
	    ENDELSE
;
;  Get the image size, and resample the image.
;
	    INS_X = A.INS_XDATA
            IF DD.SPACING(1) LT 0 THEN INS_X = -INS_X
	    FILL_MISSING, INS_X, A.INS_XDESC.MISSING, 1, /EXTRAPOLATE
	    FOR I_IMAGE = 0,N_IMAGES-1 DO BEGIN
		YCEN = A.HEADER.YCEN
		IYWIDTH = A.HEADER.IYWIDTH
		IF N_IMAGES EQ 1 THEN BEGIN
		    XCEN = A.HEADER.XCEN
		    IXWIDTH = A.HEADER.IXWIDTH
		    AAA = AA
		END ELSE BEGIN
		    XCEN = INS_X(I_IMAGE)
		    IXWIDTH = (SZ(1)*1.68*DD.BINX) < 90
		    AAA = REFORM(AA(*,I_IMAGE,*))
		    IF SZ(1) EQ 1024 THEN BEGIN
			IF DD.WAVEBAND EQ 1 THEN I0 = WAVE2PIX('N1',368.1) $
				ELSE I0 = WAVE2PIX('N2',584.4)
		    END ELSE I0 = SZ(1) / 2
		    I1 = ROUND(I0 - 27) > 0
		    I2 = ROUND(I0 + 27) < (SZ(1)-1)
		    AAA = AAA(I1:I2,*)
		ENDELSE
;
;  If NIS data, then remove cosmic rays.
;
		IF (A.HEADER.DETECTOR EQ 'NIS') AND (NOT KEYWORD_SET(NOCLEAN))$
			THEN CDS_CLEAN_IMAGE, AAA, MISSING=DD.MISSING,	$
				NOMISSING=NOMISSING
;
		XMIN = ROUND((XCEN - IXWIDTH / 2.) / SCALE)
		XMAX = ROUND((XCEN + IXWIDTH / 2.) / SCALE)
		YMIN = ROUND((YCEN - IYWIDTH / 2.) / SCALE)
		YMAX = ROUND((YCEN + IYWIDTH / 2.) / SCALE)
		NX = XMAX-XMIN+1
		NY = YMAX-YMIN+1
		AAA = CONGRID(AAA, NX, NY)
;
;  Put the image in the correct place in the mosaic.
;
		I = (XMIN - ORIGIN(0) / SCALE) > 0
		J = (YMIN - ORIGIN(1) / SCALE) > 0
		I2 = (I+NX-1) < (NNX-1)
		J2 = (J+NY-1) < (NNY-1)
		N_MOSAIC(I,J) = N_MOSAIC(I:I2,J:J2) +	$
			(AAA(0:I2-I,0:J2-J) NE DD.MISSING)
		W = WHERE(AAA EQ DD.MISSING, COUNT)
		IF COUNT GT 0 THEN AAA(W) = 0
		MOSAIC(I,J) = MOSAIC(I:I2,J:J2) + AAA(0:I2-I,0:J2-J)
	    ENDFOR
	ENDFOR
	IF KEYWORD_SET(COUNTDOWN) THEN COUNTDOWN, /CLOSE
	W = WHERE(N_MOSAIC NE 0, COUNT)
	IF COUNT GT 0 THEN MOSAIC(W) = MOSAIC(W) / N_MOSAIC(W)
	W = WHERE(N_MOSAIC EQ 0, COUNT)
	IF COUNT GT 0 THEN MOSAIC(W) = DD.MISSING
	LABEL = DD.LABEL
;
	RETURN
	END
