	FUNCTION CDS_WAVE_LABEL, LABEL
;+
; Project     :	SOHO - CDS
;
; Name        :	CDS_WAVE_LABEL()
;
; Purpose     :	Converts CDS wavelength labels into a nicer format
;
; Category    :	CDS, Class4, String
;
; Explanation :	Takes CDS labels such as "SI_12_520_69" and converts them into
;		a nicer format (e.g. "Si XII 520.69 A") for use in plot titles
;		and the like.  Alternate forms supported are "BB_*" (broad
;		band) and "WW_*" (wavelength).
;
; Syntax      :	Result = CDS_WAVE_LABEL( LABEL )
;
; Examples    :	A = READCDSFITS('s2707r00.fits')
;		AA = REFORM(DETDATA(A,1))
;		EXPTV, AA
;		DD = DETDESC(A,1)
;		LABEL_IMAGE, CDS_WAVE_LABEL(DD.LABEL)
;
; Inputs      :	LABEL = The window label from the CDS FITS file, stored in a
;			TTYPEn keyword.
;
; Opt. Inputs :	None.
;
; Outputs     :	The result of the function is the modified label.
;
; Opt. Outputs:	None.
;
; Keywords    :	None.
;
; Calls       :	ROMAN
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Prev. Hist. :	Based partially on a routine called cnv_label by Bob Bentley.
;
; History     :	Version 1, 03-Jul-1996, William Thompson, GSFC
;		Version 2, 19-Aug-1996, William Thompson, GSFC
;			Made more robust.
;		Version 3, 20-Jan-1998, Vincenzo Andretta, NRC/GSFC
;			Handles case of concatenated labels. 
;
; Contact     :	WTHOMPSON
;-
;
   ON_ERROR, 2
;
;  Check the number of parameters.
;
   IF N_PARAMS() NE 1 THEN MESSAGE, $
      'Syntax:  Result = CDS_WAVE_LABEL( LABEL )'
;
;  Separate the input string into parts based on the separator "+".
;
   SEP_C='+'
   LABEL_C = STR_SEP(LABEL,SEP_C)
   N_C = N_ELEMENTS(LABEL_C)
;
;  Go through all components. 
;
   RESULT=''
   FOR I_C=0,N_C-1 DO BEGIN
;
;  Separate the input string into parts based on the separator "_".
;
   SS = STR_SEP(LABEL_C(I_C), '_')
   N = N_ELEMENTS(SS)

   CASE 1 OF
;
;  If the first part is "BB", then use the broadband format.
;
   SS(0) EQ 'BB' : BEGIN
      RESULT = RESULT + 'Broadband'
      FOR I = 1, N-1 DO RESULT = RESULT + ' ' + SS(I)
   END
;
;  If the first part is "WW", then use the generic wavelength format.
;
   (SS(0) EQ 'WW') AND (N GE 3): $
      RESULT = RESULT + 'Wavelength ' + SS(1) + '.' + SS(2) + ' A'
;
;  If the first part is more than two characters, or there are not four parts,
;  then don't attempt to interpret the label any further, except to interpret
;  the last two parts as the wavelength.
;
   (STRLEN(SS(0)) GT 2) OR (N NE 4) : BEGIN
      RESULT = RESULT + SS(0)
      FOR I = 1, N-3 DO RESULT = RESULT + ' ' + SS(I)
      IF N GT 2 THEN RESULT = RESULT + ' ' + SS(N-2) + '.' + SS(N-1) + ' A'
   END
;
;  Otherwise, assume that the correct line format is used.
;
   ELSE: RESULT = RESULT + STRMID(SS(0),0,1) + $
      STRLOWCASE(STRMID(SS(0),1,1)) + ' ' + $
      ROMAN(SS(1)) + ' ' + SS(2) + '.' + SS(3) + ' A'

   ENDCASE

;
;  Otherwise, assume that the correct line format is used.
;

   IF I_C LT N_C-1 THEN RESULT=RESULT+' '+SEP_C+' '

   ENDFOR

   RETURN,RESULT
   END
