;+
; Project     : SOHO - CDS     
;                   
; Name        : MK_SYNOPTIC
;               
; Purpose     : Puts together the CDS Meridian NIS synoptic scans.
;               
; Explanation : Each day the CDS takes 9 4'x4' rasters running down the 
;               solar meridian.  This routine takes those data and rearranges
;               them to form images at the 4 wavelengths observed (8 if
;               SYNOP_HI is used during high telemetry rate periods). It then
;               writes them to summary FITS files.
;               
; Use         : IDL> mk_synoptic, file_number
;    
; Inputs      : file_number - the study (or equivalently FITS file) number
;                             from which the data are to be read.  There
;                             are 9 rasters associated with each study.
;               
; Opt. Inputs : None
;               
; Outputs     : None
;               
; Opt. Outputs: None
;               
; Keywords    : SHOW  - if present the image data are displayed
;               COMP  - check for compressed data files
;               CLEAN - apply cosmic ray removal routine
; 
; Calls       : None
;
; Common      : None
;               
; Restrictions: None
;               
; Side effects: None
;               
; Category    : Data analysis
;               
; Prev. Hist. : None
;
; Written     : C D Pike, RAL, 7-May-96
;               
; Modified    : Version 2, 7-May-1996, William Thompson, GSFC
;			Write out to FITS files.
;               Added more FITS headers etc..   CDP, 8-May-96
;               Pad array if rasters missing, sort display.  CDP, 9-May-96
;               Cut out debugging 'stop'    CDP, 16-May-96
;		Version 6, 21-May-1996, William Thompson, GSFC
;			Added FITS keywords TELESCOP and INSTRUME, as well as
;			standard FITS pointing keywords.
;		Version 7, 31-May-1996, William Thompson, GSFC
;			Removed pointing keywords, as not really applicable to
;			synoptic data.
;               Version 8, Fixed typo in use of missing value. CDP, 24-Jul-96
;               Version 9, SV Haugan, UiO, 30 July 1996
;                       Using gt_windata instead of direct detdata access.
;		Version 10, William Thompson, GSFC, 16 December 1996
;			Fixed bug where wrong files could be found.
;		Version 11, William Thompson, GSFC, 14 September 1997
;			Accommodate study SYNOP_HI
;	        Verison 12, Jeff Payne, RAL, 16 October 1997
;			Modified to handle standard 4 lines from
;			SYNOP or 8 lines with HI_SYNOP
;	        Verison 13, Jeff Payne, RAL, 27 November 1997
;			Bug fixed when generating filename for line 600
;		Version 14, 23-Dec-1997, William Thompson, GSFC
;			Handle only 1 line from SYNOP_LP
;		Version 15, 20-Jan-1998, William Thompson, GSFC
;			Fixed sorting bug where input files are on several
;			disks.
;		Version 16, 29-Jun-1998, William Thompson, GSFC
;			Support version 4 of SYNOP_F
;			Call FIND_WITH_DEF instead of FIND_FILES
;               Version 17, 26-Apr-2000, CDP, add COMP & CLEAN keywords.
;                                             Relax start time to avoid
;                                             late start problems
;               Version 18, 11-Jul-2003, William Thompson, GSFC
;                       Handle upside-down images.
;
; Version     : Version 18, 11-Jul-2003
;-            

pro mk_synoptic, fnum ,show=show, comp=comp, clean=clean

;
;  insist on a parameter
;
if n_params() lt 1 then begin
   print,'Use:  IDL> mk_synoptic, file_number [,/show]'
   return
endif

if not valid_num(fnum,/int) then begin
   print,'Parameter is not a valid study/file number'
   return
endif

;
;  check output directory
;
if getenv('CDS_SUMMARY_DATA_W') eq '' then begin
   print,'$CDS_SUMMARY_DATA_W is not defined'
   return
endif

;
;  get names of all raster files involved
;
ss = ''
if not keyword_set(comp) then exten = '.fits'  else exten = '.fits.Z'
for i=0,8 do ss = [ss, find_with_def('s'+trim(fnum)+'r0'+ntrim(i)+exten, $
	'$CDS_FITS_DATA')]
ww = where(ss ne '', count)
if count gt 0 then begin
    ss=ss(ww)
    if n_elements(ss) gt 1 then begin
	break_file,ss,ss_disk,ss_dir,ss_name
	ss = ss(sort(ss_name))
    endif
endif

if ss(0) eq '' then begin
   print,'No files found with that number'
   return
endif

if n_elements(ss) lt 9 then begin
   print,'Warning: Incomplete series of synoptic files.'
endif

;
;  loop through all existing files (allow for missing ones), read, calibrate
;
nf = n_elements(ss)
j = 0
for i=0,nf-1 do begin
   break_file, ss(i), disk, dir, file, ext
   expectf = 's'+trim(fnum)+'r'+strmid(trim(j+100),1,2)
   a = readcdsfits(file)
;   if i eq 0 then begin
      syn_start = a.header.date_obs
      obt_time = a.header.obt_time
      angle = a.header.angle
      ixwidth = a.header.ixwidth
      iywidth = a.header.iywidth
;   endif
;   if i eq nf-1 then begin
      date_end = a.header.date_end
      obt_end = a.header.obt_end
;   endif   
   pst = a.detdesc.ixstart(0)
   pend = a.detdesc.ixstop(0)

   vds_debias,a
   if keyword_set(clean) then cds_new_spike,a
   vds_calib,a
   missing=a.detdesc(0).missing
   d = gt_dimension(a)
   iimage = replicate(float(missing), d.ssolar_x, d.ssolar_y)
   if file ne expectf then begin
      while file ne expectf do begin
         if j eq 0 then begin
	    for ii = 1,n_elements(a.detdesc) do test =	$
		    execute('image' + ntrim(ii) + ' = iimage')
         endif else begin
	    for ii = 1,n_elements(a.detdesc) do test =	$
		    execute('image' + ntrim(ii) + ' = concat2d(iimage,image' +$
		    ntrim(ii) + ')')
         endelse
         j = j+1
         expectf = 's'+trim(fnum)+'r'+strmid(trim(j+100),1,2)
      endwhile
   endif
;
;  Sort the windows so that the normal 4 windows come first.
;
   if a.header.obs_prog eq 'SYNOP_HI' then begin
	ww = [2,4,5,7,0,1,3,6]
   end else if n_elements(a.detdesc) eq 5 then begin
	ww = [0,1,2,4,3]
   end else ww = [0,1,2,3]
;
;  Determine if the images need to be rotated.
;
   rot_images = a.detdesc(0).spacing(1) lt 0
;
   for ii = 1,n_elements(a.detdesc) do begin
       temp = average(gt_windata(a,ww(ii-1)),1,missing=missing)
       if rot_images then temp = rotate(temp,2)
       test = execute('iimage' + ntrim(ii) + ' = temp')
   endfor
   if j eq 0 then begin
      for ii = 1,n_elements(a.detdesc) do test =	$
	      execute('image' + ntrim(ii) + ' = iimage' + ntrim(ii))
   endif else begin
      for ii = 1,n_elements(a.detdesc) do test =	$
	      execute('image' + ntrim(ii) + ' = concat2d(iimage' +	$
	      ntrim(ii) + ',image' + ntrim(ii) + ')')
   endelse
;
;  display if requested
;
   nplts = n_elements(a.detdesc)
   if keyword_set(show) then begin
      erase
      for ii = 1,n_elements(a.detdesc) do begin
          setimage,ii,nplts
	  test = execute('image = image' + ntrim(ii))
          exptv,sigrange(image,miss=missing),missing=missing,/nores
      endfor
      print,'Image posn: '+fmt_vect([a.header.xcen,a.header.ycen])
   endif
   j = j+1
endfor

;
;  if files were missing off the end then pad out to full array
;
sz = size(image1)
if sz(2) lt 1287 then begin
   dummy = fltarr(sz(1),1287-sz(2))
   for ii = 1,n_elements(a.detdesc) do test =	$
	   execute('image' + ntrim(ii) + ' = concat2d(dummy,image' +	$
	   ntrim(ii) + ')')
endif


;
;  Save all the images.  Start with the the 584 image.  Form the filename.
;
trailer = '_me_' + anytim2cal(syn_start,form=8,/date) + '_' +	$
	anytim2cal(syn_start,form=8,/time) + '.fts'
if a.header.obs_prog eq 'SYNOP_LP' then		$
	filename = 'scds_00630' + trailer else	$
	filename = 'scds_00584' + trailer
;
;  Create the FITS header.  Add the various keywords pertaining to the
;  observation.
;
fxhmake, h, image1
fxaddpar, h, 'TELESCOP', 'SOHO', 'Solar and Heliospheric Observatory'
fxaddpar, h, 'INSTRUME', 'CDS', 'Coronal Diagnostic Spectrometer'
fxaddpar, h, 'FILENAME', filename, 'Name of the FITS file'
fxaddpar, h, 'DATE_OBS', syn_start,	$
	'Start date/time of observation'
fxaddpar, h, 'DATE_END', date_end,	$
	'End date/time of observation'
fxaddpar, h, 'OBT_TIME', obt_time,	$
	'Onboard start time (TAI)'
fxaddpar, h, 'OBT_END', obt_end,	$
	'Onboard end time (TAI)'
;
;  The standard FITS pointing keywords are commented out--they're not really
;  applicable to these data.
;
;fxaddpar, h, 'CRPIX1', 1, 'Reference pixel along X axis'
;fxaddpar, h, 'CRPIX2', 1, 'Reference pixel along Y axis'
;fxaddpar, h, 'CRVAL1', -122.936, 'Reference value along X axis'
;fxaddpar, h, 'CRVAL2', -1081.92, 'Reference value along Y axis'
;fxaddpar, h, 'CDELT1', 2.032, 'Pixel size along X axis'
;fxaddpar, h, 'CDELT2', 1.68,  'Pixel size along Y axis'
;
fxaddpar, h, 'XCEN', 0.0,	$
	'Preliminary center in X of field-of-view
fxaddpar, h, 'YCEN', 0.0,	$
	'Preliminary center in Y of field-of-view
fxaddpar, h, 'ANGLE', angle,	$
	'Preliminary orientation of field-of-view
fxaddpar, h, 'IXWIDTH', ixwidth,	$
	'Preliminary width in X of field-of-view
fxaddpar, h, 'IYWIDTH', iywidth*9.0,	$
	'Preliminary width in Y of field-of-view
fxaddpar, h, 'WAVELNTH', a.detdesc(ww(0)).wavelength,	$
	'Principal wavelength in Angstroms'
;
;  Write out the image.
;
fxwrite, concat_dir(getenv('CDS_SUMMARY_DATA_W'), filename), h, image1,	$
	nanvalue=missing
;
;  If we're running the SYNOP_LP procedure, then we're done.
;
if a.header.obs_prog ne 'SYNOP_LP' then begin
;
;  Next, do the 361 image.  Replace the filename and wavelength in the header.
;
    filename = 'scds_00361' + trailer
    fxaddpar, h, 'FILENAME', filename
    fxaddpar, h, 'WAVELNTH', a.detdesc(ww(1)).wavelength
    fxwrite, concat_dir(getenv('CDS_SUMMARY_DATA_W'), filename), h, image2, $
	    nanvalue=missing
;
;  Next, do the 368 image.
;
    filename = 'scds_00368' + trailer
    fxaddpar, h, 'FILENAME', filename
    fxaddpar, h, 'WAVELNTH', a.detdesc(ww(2)).wavelength
    fxwrite, concat_dir(getenv('CDS_SUMMARY_DATA_W'), filename), h, image3, $
	    nanvalue=missing
;
;  Next, do the 630 image.
;
    filename = 'scds_00630' + trailer
    fxaddpar, h, 'FILENAME', filename
    fxaddpar, h, 'WAVELNTH', a.detdesc(ww(3)).wavelength
    fxwrite, concat_dir(getenv('CDS_SUMMARY_DATA_W'), filename), h, image4, $
	    nanvalue=missing
;
;  If there are 5 images, then do the 625 image.
;
    if n_elements(a.detdesc) eq 5 then begin
	filename = 'scds_00625' + trailer
	fxaddpar, h, 'FILENAME', filename
	fxaddpar, h, 'WAVELNTH', a.detdesc(ww(4)).wavelength
	fxwrite, concat_dir(getenv('CDS_SUMMARY_DATA_W'), filename), h,	$
		image5, nanvalue=missing
    endif
;
; 
    if a.header.obs_prog eq 'SYNOP_HI' then begin
;
	;  Next , do the 521 image.
	;
	filename = 'scds_00521' + trailer
	fxaddpar, h, 'FILENAME', filename
	fxaddpar, h, 'WAVELNTH', a.detdesc(ww(4)).wavelength
	fxwrite, concat_dir(getenv('CDS_SUMMARY_DATA_W'), filename), h,	$
		image5, nanvalue=missing
	;

	;  Next , do the 563 image.
	;
	filename = 'scds_00563' + trailer
	fxaddpar, h, 'FILENAME', filename
	fxaddpar, h, 'WAVELNTH', a.detdesc(ww(5)).wavelength
	fxwrite, concat_dir(getenv('CDS_SUMMARY_DATA_W'), filename), h,	$
		image6, nanvalue=missing
	;
	;  Next , do the 600 image.
	;
	filename = 'scds_00600' + trailer
	fxaddpar, h, 'FILENAME', filename
	fxaddpar, h, 'WAVELNTH', a.detdesc(ww(6)).wavelength
	fxwrite, concat_dir(getenv('CDS_SUMMARY_DATA_W'), filename), h,	$
		image7, nanvalue=missing
	;
	;  Finally , do the 625 image.
	;
	filename = 'scds_00625' + trailer
	fxaddpar, h, 'FILENAME', filename
	fxaddpar, h, 'WAVELNTH', a.detdesc(ww(7)).wavelength
	fxwrite, concat_dir(getenv('CDS_SUMMARY_DATA_W'), filename), h,	$
		image8, nanvalue=missing
    endif
endif
;
end
