;+
; Project     :	SOHO - CDS
;
; Name        :	XCDS_SNAPSHOT
;
; Purpose     :	Widget interface to CDS_SNAPSHOT
;
; Category    :	CDS, Class3, Quicklook
;
; Explanation :	Puts up a small widget which allows one to control how
;		CDS_SNAPSHOT is run.  Also, one can redirect the output to a
;		PostScript file or printer.
;
; Syntax      :	XCDS_SNAPSHOT, FILENAME
;		XCDS_SNAPSHOT, DATA
;
; Examples    :	XCDS_SNAPSHOT, 's3042r00.fits'
;
;		A = READCDSFITS('s3042r00.fits')
;		XCDS_SNAPSHOT, A
;
; Inputs      :	FILENAME = Name of CDS FITS file to read in.
;
; Opt. Inputs :	DATA	 = A quicklook data structure as returned by
;			   READCDSFITS.
;
; Outputs     :	None.
;
; Opt. Outputs:	None.
;
; Keywords    :	REVERSE = If set, then plot is done in inverse video.  In other
;			  words, white areas on the screen will appear dark,
;			  and visa versa.  This saves toner on greyscale
;			  printers.
;
;		NOCLEAN = If set, then CDS_CLEAN_IMAGE is not called.
;
;		WINDOWS = An array of windows to display.  The windows can be
;			  passed as either the NUMBERS of the data windows
;			  (NOTE: numbering from 0...N-1) or as text with the
;			  LABELS of the desired spectral windows.  The windows
;			  will be displayed in the order given by the WINDOWS
;			  array, unless the WAVESORT keyword is passed.
;
;		WAVESORT = If set, then the windows are displayed in wavelength
;			   order.  (Note that second order lines will be
;			   displayed according to their first order
;			   equivalent.)
;
;		QUASI_FIT= If set, then the routine CDS_QUASI_FIT is called to
;			   extract the images.  See the documentation on that
;			   routine for more information.
;
;		SPECTRA	 = If set, then spectra are shown instead of images.
;			   For some data sets, spectra are shown by default.
;
;		LOG	 = If set, then spectra are plotted logarithmically.
;			   Only used for line plots.
;
;		MOSAIC	 = If set, then a mosaic is formed out of the
;			   individual exposures.  This option is only used for
;			   rasters made with slit #6, and is otherwise ignored.
;
;		FRACTION = Fraction to pass to SIGRANGE function.  The default
;			   is 0.99.
;
;		OWN_WINDOW = If set, then XCDS_SNAPSHOT will always display
;			     data in its own window, rather than the default
;			     graphics window.
;
;		ERRMSG = If defined and passed, then any error messages will be
;			 returned to the user in this parameter rather than
;			 depending on the MESSAGE routine in IDL.  If no errors
;			 are encountered, then a null string is returned.  In
;			 order to use this feature, ERRMSG must be defined
;			 first, e.g.
;
;				ERRMSG = ''
;				XCDS_SNAPSHOT, INPUT, ERRMSG=ERRMSG
;				IF ERRMSG NE '' THEN ...
;
;		GROUP	= The widget ID of the widget that calls XCDS_SNAPSHOT.
;			  When this ID is specified, a death of the caller
;			  results in a death of XCDS_SNAPSHOT.
;
;		In addition, XCDS_SNAPSHOT supports any keyword used by the
;		XMANAGER routine, e.g. /MODAL, through the _EXTRA mechanism.
;
; Calls       :	CDS_SNAPSHOT, SETWINDOW, XPS_SETUP, PS, PSPLOT, PSCLOSE,
;		POPUP_MSG
;
; Common      :	Uses the internal common block XCDS_SNAPSHOT.
;
; Restrictions:	Can only be used for rasters which build up images via mirror
;		(and slit for GIS) movements.
;
; Side effects:	None.
;
; Prev. Hist. :	None.
;
; History     :	Version 1, 18-Jul-1996, William Thompson, GSFC
;		Version 2, 21-Aug-1996, William Thompson, GSFC
;			Added keywords SPECTRA and WO_CR
;		Version 3, 28-Oct-1996, William Thompson, GSFC
;			Added keyword FRACTION.
;		Version 4, 22-Nov-1996, William Thompson, GSFC
;			Make widget reflect keywords.
;		Version 5, 14-Jan-1997, William Thompson, GSFC
;			Added keyword QUASI_FIT
;		Version 6, 21-Mar-1997, William Thompson, GSFC
;			Added keyword OWN_WINDOW
;		Version 7, 19-Mar-1999, William Thompson, GSFC
;			Added keyword /LOG.
;		Version 8, 02-Nov-1999, William Thompson, GSFC
;			Added keyword /MOSAIC.
;			Removed extremely slow WO_CR option.
;			Make window numbering from 0,...,N-1
;		Version 9, 24-Jul-2002, William Thompson, GSFC
;			Create windows with RETAIN=2
;
; Contact     :	WTHOMPSON
;-
;
;==============================================================================
;
;  First, we have the event handler for the selection subwidget.
;
	PRO XCDS_SNAP_SEL_EVENT, EVENT
;
	COMMON XCDS_SNAPSHOT, A, SNAP_WINDOW, ALLOC_WINDOW, REVERSE, NOCLEAN, $
		WINDOWS, WAVESORT, QFIT, SPECTRA, MOSAIC, SEL_LIST, FRACTION, $
		LOG
;
;  Get the widget event, and act accordingly.
;
	WIDGET_CONTROL,EVENT.ID,GET_UVALUE=EV
	CASE EV OF 
;
;  If one of the window buttons was selected, then add that button to the list.
;
	    'WINDOW': BEGIN
		IWINDOW = EVENT.VALUE
		IF WINDOWS(0) LE -1 THEN BEGIN
			WINDOWS = IWINDOW
		END ELSE BEGIN
			W = WHERE(WINDOWS EQ IWINDOW, COUNT)
			IF COUNT EQ 0 THEN WINDOWS = [WINDOWS, IWINDOW]
		ENDELSE
		WIDGET_CONTROL, EVENT.ID, GET_VALUE=VALUES
		VALUES(EVENT.VALUE) = 1
		WIDGET_CONTROL, EVENT.ID, SET_VALUE=VALUES
		END
;
;  If the "Restart" button was pressed, then reset the WINDOWS array.  The
;  special value of -2 means that, barring any further selection, all the
;  windows will be selected.
;
	    'RESTART': BEGIN
		WINDOWS = -2
		WIDGET_CONTROL, SEL_LIST, GET_VALUE=VALUES
		VALUES(*) = 0
		WIDGET_CONTROL, SEL_LIST, SET_VALUE=VALUES
		END
;
;  If the "Cancel" button was pressed, then reset the WINDOWS array and return.
;
	    'CANCEL': BEGIN
		WINDOWS = -1
		WIDGET_CONTROL, EVENT.TOP, /DESTROY
		END
;
;  If the "Commit button was pressed, then simply return.
;
	    'COMMIT': WIDGET_CONTROL, EVENT.TOP, /DESTROY
	ENDCASE
	END
;
;==============================================================================
;
;  Next, we have the widget setup for the selection subwidget.
;
	PRO XCDS_SNAP_SEL, LABELS, GROUP=GROUP
;
	COMMON XCDS_SNAPSHOT
;
	IF N_ELEMENTS(GROUP) EQ 0 THEN GROUP=0
;
	MAIN = WIDGET_BASE(GROUP_LEADER=GROUP, /COLUMN, /MAP,	$
		TITLE='CDS Data Window Selection')
;
;  Put some help text at the top of the widget.
;
	TEXT = [ $
		'Pick the data windows that', $
		'you wish to display, in', $
		'the order that they should', $
		'be placed on the screen.', $
		'', $
		'Using "Restart" and "Commit"', $
		'in succession will select', $
		'all the windows' ]
	DUMMY = WIDGET_TEXT(MAIN, VALUE=TEXT, XSIZE=MAX(STRLEN(TEXT)),	$
		YSIZE=N_ELEMENTS(TEXT))
;
;  Put up a list of the possible window labels.
;
	VALUES = INTARR(N_ELEMENTS(LABELS))
	IF N_ELEMENTS(WINDOWS) NE N_ELEMENTS(VALUES) THEN VALUES(WINDOWS) = 1
	SEL_LIST = CW_BGROUP(MAIN, LABELS, ROW=N_ELEMENTS(LABELS),	$
		/NONEXCLUSIVE, UVALUE='WINDOW', SET_VALUE=VALUES)
;
;  Add the buttons at the bottom of the widget.
;
	BUTTONS = WIDGET_BASE(MAIN, /ROW, /MAP)
	DUMMY = WIDGET_BUTTON(BUTTONS, UVALUE='RESTART', VALUE='Restart')
	DUMMY = WIDGET_BUTTON(BUTTONS, UVALUE='CANCEL',  VALUE='Cancel')
	DUMMY = WIDGET_BUTTON(BUTTONS, UVALUE='COMMIT',  VALUE='Commit')
;
;  Start it all going.
;
	WSAVE = WINDOWS
	IF N_ELEMENTS(WINDOWS) EQ N_ELEMENTS(LABELS) THEN WINDOWS = -1
	WIDGET_CONTROL, MAIN, /REALIZE
	XMANAGER, 'XCDS_SNAP_SEL', MAIN, /MODAL
	IF WINDOWS(0) EQ -1 THEN WINDOWS = WSAVE
	IF WINDOWS(0) EQ -2 THEN WINDOWS = INDGEN(N_ELEMENTS(LABELS))
;
	END
;
;==============================================================================
;
;  Next, we have the event handler for the main snapshot widget.
;
	PRO XCDS_SNAPSHOT_EVENT, EVENT
;
	COMMON XCDS_SNAPSHOT
;
;  Get the widget event, and act accordingly.
;
	WIDGET_CONTROL,EVENT.ID,GET_UVALUE=EV

	CASE EV OF
	    'EXIT': WIDGET_CONTROL, EVENT.TOP, /DESTROY
;
;  If the "Display" button was pressed, then display the snapshot.  Start off
;  by switching to the correct window (if it still exists).
;
	    'DISPLAY': BEGIN
		WIDGET_CONTROL, /HOURGLASS
		DEVICE, WINDOW_STATE=WIN_STATE
		IF SNAP_WINDOW GE 0 THEN BEGIN
		    IF WIN_STATE(SNAP_WINDOW) THEN BEGIN
			SAVE_WINDOW = !D.WINDOW
			SETWINDOW, SNAP_WINDOW
		    END ELSE BEGIN
			SNAP_WINDOW = -1
			SAVE_WINDOW = -1
		    ENDELSE
		END ELSE SAVE_WINDOW = -1
;
;  If the /USE_WINDOW keyword was used (ALLOC_WINDOW > -1), then make sure that
;  the right window is used.
;
		IF (ALLOC_WINDOW GE 0) AND (ALLOC_WINDOW NE SNAP_WINDOW) $
			THEN BEGIN
		    SAVE_WINDOW = !D.WINDOW
		    WINDOW, /FREE, RETAIN=2
		    SNAP_WINDOW = !D.WINDOW
		    ALLOC_WINDOW = !D.WINDOW
		ENDIF
;
;  Run CDS_SNAPSHOT.
;
		ERRMSG = ''
		CDS_SNAPSHOT, A, ERRMSG=ERRMSG, REVERSE=REVERSE,	$
			NOCLEAN=NOCLEAN, WINDOWS=WINDOWS, WAVESORT=WAVESORT, $
			QUASI_FIT=QFIT, SPECTRA=SPECTRA, MOSAIC=MOSAIC,	$
			FRACTION=FRACTION, LOG=LOG
		IF !D.WINDOW GE 0 THEN WSHOW
		IF ERRMSG NE '' THEN XACK, ERRMSG
;
;  Switch back to the previous window.
;
		IF SAVE_WINDOW GE 0 THEN SETWINDOW, SAVE_WINDOW
		END
;
;  If the "Adjust Color" button was pressed, then call XLOADCT.
;
	    'ADJUST': XLOADCT
;
;  If the "New window" button was pressed, then create a new graphics window.
;
	    'WINDOW': BEGIN
		SAVE_WINDOW = !D.WINDOW
		WINDOW, /FREE, RETAIN=2
		SNAP_WINDOW = !D.WINDOW
		IF SAVE_WINDOW GE 0 THEN SETWINDOW, SAVE_WINDOW
		IF ALLOC_WINDOW GE 0 THEN ALLOC_WINDOW = SNAP_WINDOW
		END
;
;  If the "Select" button was pressed, then select which lines to display.
;
	    'SELECT': XCDS_SNAP_SEL, A.DETDESC.LABEL, GROUP=EVENT.TOP
;
;  If the "Print" button was pressed, then call XPS_SETUP to control how to
;  write out the output.
;
	    'PRINT': BEGIN
		XPS_SETUP, PS_STC, GROUP=EVENT.TOP, STATUS=STATUS
		IF STATUS THEN BEGIN
		    WIDGET_CONTROL, /HOURGLASS
		    PS, PS_STC.FILENAME, COLOR=PS_STC.COLOR, COPY=PS_STC.COPY,$
			    ENCAPSULATED=PS_STC.ENCAPSULATED,	$
			    INTERPOLATE=PS_STC.INTERPOLATE,	$
			    PORTRAIT=PS_STC.PORTRAIT
;
;  Write the snapshot to the PostScript file.
;
		    ERRMSG = ''
		    CDS_SNAPSHOT, A, ERRMSG=ERRMSG, REVERSE=REVERSE,	$
			    NOCLEAN=NOCLEAN, WINDOWS=WINDOWS,		$
			    WAVESORT=WAVESORT, QUASI_FIT=QFIT,		$
			    SPECTRA=SPECTRA, MOSAIC=MOSAIC, LOG=LOG,	$
			    FRACTION=FRACTION
		    IF ERRMSG NE '' THEN XACK, ERRMSG
;
;  Either send the PostScript file to the printer, or display a message showing
;  the name of the file that was created.
;
		    IF PS_STC.HARD THEN BEGIN
			PSPLOT, DELETE=PS_STC.DELETE, QUEUE=PS_STC.PRINTER
			POPUP_MSG, 'Plot has been sent to printer ' +	$
				PS_STC.PRINTER + '.', GROUP=EVENT.TOP
		    ENDIF ELSE BEGIN
			PSCLOSE
			CD, CURRENT=CURR_PATH
			POPUP_MSG, 'Plot saved to PS file: ' +	$
				FORM_FILENAME(PS_STC.FILENAME, '.ps',	$
				DIRECTORY=CURR_PATH), GROUP=EVENT.TOP
		    ENDELSE
		ENDIF
		END
;
;  If either of the options events was selected, then set the value of that
;  option to either on or off.
;
	    'OPTIONS': BEGIN
		IF EVENT.SELECT THEN SEL = 1 ELSE SEL = 0
		CASE EVENT.VALUE OF
		    0: REVERSE = SEL
		    1: NOCLEAN = 1 - SEL
		    2: WAVESORT = SEL
		ENDCASE
		END
	    'OPTIONS2': BEGIN
		IF EVENT.SELECT THEN SEL = 1 ELSE SEL = 0
		CASE EVENT.VALUE OF
		    0: QFIT = SEL
		    1: SPECTRA = SEL
		    2: LOG     = SEL
		    3: MOSAIC  = SEL
		ENDCASE
		END
	    'FRACTION': WIDGET_CONTROL, EVENT.ID, GET_VALUE=FRACTION
	ENDCASE
	END
;
;==============================================================================
;
;  Finally, we have the widget setup for the main snapshot widget.
;
	PRO XCDS_SNAPSHOT, INPUT, GROUP=GROUP, ERRMSG=ERRMSG,	$
		REVERSE=K_REVERSE, NOCLEAN=K_NOCLEAN, WINDOWS=WINS,	$
		WAVESORT=K_WAVESORT, QUASI_FIT=K_QFIT, SPECTRA=K_SPECTRA, $
		FRACTION=K_FRACTION, OWN_WINDOW=OWN_WINDOW, LOG=K_LOG,	$
		MOSAIC=K_MOSAIC, _EXTRA=_EXTRA
;
	ON_ERROR, 2
	COMMON XCDS_SNAPSHOT
;
;  Check the number of parameters passed.
;
	IF N_PARAMS() NE 1 THEN BEGIN
	    MESSAGE = 'Syntax:  XCDS_SNAPSHOT, FILENAME or ' +	$
		    'XCDS_SNAPSHOT, DATA'
	    GOTO, HANDLE_ERROR
	ENDIF
;
;  If already registered, then inform the user and return.
;
	IF XREGISTERED('XCDS_SNAPSHOT') THEN BEGIN
		XACK, 'Only one copy XCDS_SNAPSHOT can be run at a time'
		RETURN
	ENDIF
;
;  If a data structure was passed, then use it.  Otherwise, read the FITS file
;  whose name was passed.
;
	IF DATATYPE(INPUT,1) EQ 'Structure' THEN BEGIN
	    A = INPUT
	END ELSE BEGIN
	    IF DATATYPE(INPUT,1) NE 'String' THEN BEGIN
		MESSAGE = 'INPUT must be either a structure or ' +	$
			'a character string'
		GOTO, HANDLE_ERROR
	    ENDIF
;
	    A = READCDSFITS(INPUT)
	    IF DATATYPE(A,1) NE 'Structure' THEN BEGIN
		MESSAGE ='Unable to read FITS file ' + INPUT
		GOTO, HANDLE_ERROR
	    ENDIF
	ENDELSE
;
;  Get the value of the GROUP parameter.
;
	IF N_ELEMENTS(GROUP) EQ 0 THEN GROUP=0
;
;  If the WINDOWS array was passed, then store it in the common block.
;
	IF N_ELEMENTS(WINS) GT 0 THEN WINDOWS = WINS ELSE	$
		WINDOWS = INDGEN(N_ELEMENTS(A.DETDESC))
;
;  Determine what window will be used.  If /OWN_WINDOW is passed, then make
;  sure that a special window has been specifically allocated.
;
	IF N_ELEMENTS(SNAP_WINDOW) EQ 0 THEN BEGIN
	    IF KEYWORD_SET(OWN_WINDOW) THEN BEGIN
		SAVE_WINDOW = !D.WINDOW
		WINDOW, /FREE, RETAIN=2
		SNAP_WINDOW = !D.WINDOW
		ALLOC_WINDOW = !D.WINDOW
		IF SAVE_WINDOW GE 0 THEN SETWINDOW, SAVE_WINDOW
	    END ELSE BEGIN
		SNAP_WINDOW = !D.WINDOW
		ALLOC_WINDOW = -1
	    ENDELSE
	ENDIF
;
;  Use the ALLOC_WINDOW parameter to signal to the event handler that the
;  /OWN_WINDOW keyword was passed.
;
	IF NOT KEYWORD_SET(OWN_WINDOW) THEN ALLOC_WINDOW = -1 ELSE $
		ALLOC_WINDOW = ALLOC_WINDOW > 32
;
;  Set up the initial values of the other parameters.
;
	REVERSE  = KEYWORD_SET(K_REVERSE)
	NOCLEAN  = KEYWORD_SET(K_NOCLEAN)
	WAVESORT = KEYWORD_SET(K_WAVESORT)
	QFIT	 = KEYWORD_SET(K_QFIT)
	SPECTRA  = KEYWORD_SET(K_SPECTRA)
	LOG	 = KEYWORD_SET(K_LOG)
	MOSAIC   = KEYWORD_SET(K_MOSAIC)
;
;  Create the main widget base.
;
	MAIN = WIDGET_BASE(GROUP_LEADER=GROUP, /COLUMN, /MAP,	$
		TITLE='CDS Snapshot')
;
;  Put the DISPLAY button across the top.
;
	DUMMY = WIDGET_BUTTON(MAIN, UVALUE='DISPLAY', VALUE='DISPLAY', FRAME=1)
;
;  Create a base to hold the buttons, and define the buttons.
;
	TOP = WIDGET_BASE(MAIN, /ROW, FRAME=1)
	DUMMY = WIDGET_BUTTON(TOP, UVALUE='ADJUST', VALUE='Adjust Color')
	DUMMY = WIDGET_BUTTON(TOP, UVALUE='WINDOW', VALUE='New window')
	DUMMY = WIDGET_BUTTON(TOP, UVALUE='SELECT', VALUE='Select')
	DUMMY = WIDGET_BUTTON(TOP, UVALUE='PRINT',  VALUE='Print')
	DUMMY = WIDGET_BUTTON(TOP, FRAME=1, UVALUE='EXIT', VALUE='Exit')
;
;  Create a base to hold the options, and define the options menu.
;
	OPTIONS = WIDGET_BASE(MAIN, /COLUMN, FRAME=1)
	MIDDLE = WIDGET_BASE(OPTIONS, /ROW)
	TEXT = ['Reverse', 'Clean','Wave sort']
	DUMMY = CW_BGROUP(MIDDLE, TEXT, COLUMN=3, /NONEXCLUSIVE,	$
		UVALUE='OPTIONS', LABEL_LEFT='Options:',		$
		SET_VALUE=[REVERSE,1-NOCLEAN,WAVESORT])
	BOTTOM = WIDGET_BASE(OPTIONS, /ROW)
	TEXT = ['Quasi-fit', 'Spectra', 'Log', 'Mosaic']
	DUMMY = CW_BGROUP(BOTTOM, TEXT, COLUMN=4, /NONEXCLUSIVE,	$
		UVALUE='OPTIONS2', LABEL_LEFT='        ',		$
		SET_VALUE=[QFIT,SPECTRA,LOG,MOSAIC])
;
;  Put in a field to control the SIGRANGE fraction.
;
	IF N_ELEMENTS(K_FRACTION) EQ 1 THEN FRACTION = K_FRACTION
	IF N_ELEMENTS(FRACTION) EQ 0 THEN FRACTION = 0.99
	DUMMY = CW_FIELD(OPTIONS, VALUE=FRACTION, /ROW, /FLOAT, /ALL_EVENTS, $
		TITLE='SIGRANGE fraction', UVALUE='FRACTION')
;
;  Put in an explanatory text window.
;
	TEXT = [ $
		'                    BUTTONS', $
		'', $
		'Use the "Display" button to show the snapshot.', $
		'Use "Adust Color" to adjust the color table.', $
		'Use "New Window" to create a new graphics window.', $
		'Use "Select" to select which data to display.', $
		'Use "Print" to make a hardcopy of the snapshot,', $
		'    or to write it to a file.', $
		'', $
		'                    OPTIONS', $
		'', $
		'The "Reverse" option allows the images to be', $
		'    displayed in reverse video.', $
		'The "Clean" option selects automatic removal of', $
		'    cosmic rays from NIS data.', $
		'The "Wave sort" option selects automatic sort', $
		'    by wavelength', $
		'The "Quasi-fit" option uses the CDS_QUASI_FIT', $
		'    routine', $
		'The "Spectra" option displays spectra instead', $
		'    of images---automatic for some data sets', $
		'The "Log" option displays spectra logarithmically', $
		'The "Mosaic" option forms an image out of the', $
		'    individual exposures.  Only used for rasters ', $
		'    made with slit #6, and is otherwise ignored.', $
		'The "SIGRANGE fraction" field contains the value', $
		'    to be passed to the SIGRANGE function.', $
		'    Default is 0.99.  Set to 1 for the entire range.']
	DUMMY = WIDGET_TEXT(MAIN, VALUE=TEXT, XSIZE=MAX(STRLEN(TEXT)),	$
		YSIZE=N_ELEMENTS(TEXT))
;
;  Start it all going.
;
	WIDGET_CONTROL, MAIN, /REALIZE
	XMANAGER, 'XCDS_SNAPSHOT', MAIN, _EXTRA=_EXTRA
	GOTO, FINISH
;
;  Error handling point.
;
HANDLE_ERROR:
	IF N_ELEMENTS(ERRMSG) NE 0 THEN ERRMSG = 'CDS_SNAPSHOT: ' + MESSAGE $
		ELSE MESSAGE, MESSAGE, /CONTINUE
;
FINISH:
	RETURN
	END
