;+
; Project     : SOHO - CDS     
;                   
; Name        : CDS_CORR_GRADIENT
;               
; Purpose     : Apply local intensity gradient correction to line position.
;               
; Explanation : For NIS wavelength band 2, there is a tendency of the line
;               position to show a marked correlation with the local intensity
;               gradient along the slit (see Haugan 1999, "Anomalous line
;               shifts from local intensity gradients on the SOHO/CDS NIS
;               detector", Sol. Phys., in press. A preprint is available at
;               http://www.uio.no/~steinhh/CDS/anomalous.ps.gz)
;
;               Given maps of the total line intensity and the line position,
;               this routine offers a tentative "repair" procedure to
;               eliminate (at least partially) the induced line shift.
;
;               Statistically, the (asymmetric part of the) correlation
;               appears to be a shift of 0.23 pixels * d0, where d0 is the
;               normalized derivative of the line intensity along the
;               slit. Correcting for this, the residual correlations (with
;               pixels further away) are at least an order of magnitude
;               smaller.
;
;               However, it should be emphasized that the correction is only
;               strictly appropriate in a statistical sense. Some profiles
;               could end up with the *physcially correct* line position, even
;               after being affected by the tilted PSF. This correction will
;               then of course produce the "wrong" result for that line
;               profile.
;
;               Conversely, it is of course also possible (and quite likely)
;               that the correction should be larger than average for some
;               profiles.  To test the effect of correcting more/less, use the
;               keyword FACTOR, which says how many times the default
;               correction should be applied (default is 1, 2 means correct
;               twice the normal amount, etc).
;
;               The recommendation is to use this routine with care. In my
;               personal opinion, the results after applying the correction
;               are *probably* more correct than before applying the
;               correction, but that doesn't mean they are completely correct.
;
;               A very good exercise is to blink an uncorrected and a
;               corrected velocity map against each other.
;
;               Personally, I would be very careful about discussing physical
;               interpretations based on uncorrected velocities that disappear
;               when applying this routine with FACTOR <= 2. Velocities that
;               are *strengthened* by this correction should be very safe,
;               though!
;
;               
; Use         : CDS_CORR_GRADIENT,INTENSITY,POSITION,MISSING
;
;               or
;               
;               CDS_CORR_GRADIENT,ANALYSIS
;
; Inputs      : INTENSITY : Data array containing the total line intensity
;                           (i.e. Gaussian amplitude times width) at each
;                           position of a raster (or series of rasters).
;
;                           The array may have 2 or more dimensions, but the
;                           important thing is that the "slit direction" is
;                           oriented along the SECOND dimension.
;
;                           INTENSITY is usually a two-dimensional image,
;                           though a time series with dimensions (1,M,N) can
;                           be used as well.
;
;               POSITION : Data array (of same size as INTENSITY) containing
;                          the line position (in angstrom) to be
;                          corrected. The line position is assumed to be first
;                          order NIS2 band.
;
;               MISSING : The (scalar) value of MISSING data points.
;
;
;               Called with a single parameter:
;
;               
;               ANALYSIS : CFIT analysis structure where
;
;                 RESULT(0,*,*) is the amplitude,
;                 RESULT(1,*,*) is the position, and
;                 RESULT(2,*,*) is the width...
;
;                 .. of a Gaussian which has been fitted to CDS data in the
;                 NIS2 band. Pixels where the intensity gradient cannot be
;                 calculated are flagged as MISSING.
;
;                 If the line of interest is not the first component in the
;                 fit, use the RESULT_OFFSET keword to indicate the index of
;                 the first parameter of the Gaussian.
;
; Opt. Inputs : None.
;
; Outputs     : Modifies the analysis result block (or POSITION input array).
;               
; Opt. Outputs: None.
;               
; Keywords    : FACTOR : The correction applied is multiplied with FACTOR.
;                        Default value 1.
;
;               RESULT_OFFSET : If the parameters of the Gaussian to be
;                               corrected are not the first parameters in the
;                               fit - e.g. if they are store in
;                               RESULT(3:5,*,*) - this must give the offset (3
;                               in this case) into the result array for the
;                               first parameter of the Gaussian.
;
; Calls       : default, make_sfit_stc
;
; Common      : None.
;               
; Restrictions: This routine *only* applies to data taken before the
;               recommissioning.
;
;               Make sure the appropriate wavelength calibration is loaded
;               (needed to calculate the delta-lambda per pixel).
;
;               The data are assumed to have positions corresponding to
;               first-order wavelengths within NIS2.
;               
; Side effects: Flags the line fits for which the correction cannot be
;               calculated (due to edge effects, neighbouring missing points
;               etc) as FAILED (all parameters MISSING, constant, and all
;               components excluded).
;
;               The parameter values will no longer correspond to a data model
;               that is the best fit to the data!
;               
; Category    : Analysis.
;               
; Prev. Hist. : None.
;
; Written     : S.V.H.Haugan, UiO, 3 February 1999
;               
; Modified    : Version 2, SVHH, 22 February 1999
;                       Added INTENSITY,POSITION,MISSING calling convention.
;
; Version     : 2, 22 February 1999
;-            

PRO cds_corr_gradient,ana,position,missing, $
                      result_offset=res_offset,factor=factor
  
  default,res_offset,0
  default,factor,1.0
  
  IF n_params() EQ 3 THEN BEGIN
     
     INTENSITY = TEMPORARY(ANA)
     
  END ELSE IF n_params() EQ 1 THEN BEGIN
     
     ;; If called with an ANALYSIS:
     ;; 
     ;; Get INTENSITY, POSITION & MISSING as if called with 3 params.
     
     missing = ana.missing
     handle_value,ana.result_h,res
     
     
     ;; Get original size - when we reform things later, we don't want to
     ;; loose a singular X dimension.
     ;; 
     sz = size(res)
     datasize = sz(2:sz(0))
     
     amplitude = res(res_offset,*,*,*,*,*,*)
     width = res(res_offset+2,*,*,*,*,*,*)
     
     intensity = amplitude * width
     
     ix = where(temporary(amplitude) EQ missing OR temporary(width) EQ missing)
     
     intensity(ix) = missing
     
     ;; Now, reform to match size specs.
     ;; 
     intensity = dimreform(intensity,datasize,/overwrite)
     
     position = dimreform(res(res_offset+1,*,*,*,*,*,*),datasize,/overwrite)
     
     ;; Get linear transformation values from fit structure
     handle_value,ana.fit_h,fit
     sfit = make_sfit_stc(fit)
     
     tr_a = sfit.trans_a(res_offset+1)
     tr_b = sfit.trans_b(res_offset+1)
     
     ;; Actual = nominal * tr_a + tr_b
     
     position = temporary(position) * tr_a + tr_b
     
  END ELSE BEGIN
     message,"Use: CDS_CORR_GRADIENT,INTENSITY,POSITION,MISSING",/CONT
     message,"or:  CSD_CORR_GRADIENT,ANALYSIS"
  END
  
  ;; Remember, we could have: INTENSITY(X,Y,T,J,K,L,M,N)
  ;;                          POSITION(X,Y,T,J,K,L,M,N)
  
  
  d0 = 0.5*(shift(intensity,0,-1)-shift(intensity,0,1))/ (intensity > 1e-7)
  
  ;; Bad points: Where intensity is missing or zero
  
  bad = intensity EQ missing OR intensity EQ 0.0
  
  ;; Or neighbours of such points.
  
  bad = bad OR shift(bad,0,1) OR shift(bad,0,-1)
  
  ;; Or border points.
  
  sz = size(bad)
  bad(*,[0,1],*,*,*,*,*,*) = 1b
  bad(*,sz(2)-[1,2],*,*,*,*,*,*) = 1b
  
  ;; Handy:
  
  badix = where(bad)
  goodix = where(1b-bad)
  
  mpos = median(position(goodix))
  
  IF mpos LT 400 THEN BEGIN
     message,"NIS 1 gradient correction not established",/continue
     return
  END
  
  ;; Find dispersion from median position
  
  px = wave2pix('NIS2',mpos)
  lam = pix2wave('NIS2',[px,px+1])
  dispersion = lam(1)-lam(0)
  
  ;;
  ;; This is where the correction's done.
  ;;
  correction = dispersion*(-1 > d0 < 1) * 0.23 * factor
  position = temporary(position)-correction
  
  ;; If called with 3 parameters, just flag bad pixels and return.
  ;; 
  IF n_params() EQ 3 THEN BEGIN
     ;; Patch missing points
     position(badix) = missing 
     ;; Put back this one..
     ana = temporary(intensity)
     return
  END
  
  ;;
  ;; Now convert back to nominal values
  ;;
  position = (temporary(position)-tr_b)/tr_a
  
  ;; Put back values
  
  res(res_offset+1,0,0,0,0,0,0) = dimreform(position,[1,datasize],/overwrite)
  handle_value,ana.result_h,res,/set
  
  cfit_apatch,ana,"FAIL",badix
  
  return
END







