;+
; Project     : SOHO - CDS     
;                   
; Name        : FIT_CDS_QL
;               
; Purpose     : Gaussian fit of CDS QL spectra
;               
; Category    : Analysis
;               
; Explanation : 
;               
; Syntax      : IDL> fit_cds_ql,ql
;    
; Examples    : 
;
; Inputs      : QL = QL structure to fit
;               
; Opt. Inputs : None
;               
; Outputs     : See Keywords
;
; Opt. Outputs: None
;               
; Keywords    : WINDOW = window to fit (input)
;               WCEN = first guess of centroid (input)
;               WBACK = first guess of background (input)
;               WDOPP = first guess of Doppler width (input)
;               FIXP  = parameter indicies to fix (input)
;               WRANGE = wavelength range for fit (input)
;               SUB = subarray to limit to fit (input)
;               CENT = 2d array of fitted centroids (output)
;               WIDTH = 2d array of fitted Doppler widths (output)
;               INTEN = 2d array of fitted intensities (output)
;
; Common      : None.
;               
; Restrictions: Limited to Gaussian
;               
; Side effects: None.
;               
; History     : Version 1,  17-Jan-1998,  D M Zarro.  Written
;
; Contact     : DZARRO@SOLAR.STANFORD.EDU
;-            


pro fit_cds_ql,ql,window=window,cent=cent,inten=inten,width=width,plot=plot,$
 wcen=wcen,wback=wback,wrange=wrange,wdopp=wdopp,sub=sub,fixp=fixp,$
 missing=missing,debug=debug
               
qlmgr,ql,valid
if not valid then return

if not exist(window) then window=gt_cds_window(ql)
if window lt 0 then return
fdata=gt_windata(ql,window,/quick)

sz=size(fdata)                       ;-- data array
nx=sz(2) & ny=sz(3)
width=fltarr(nx,ny)
cent=fltarr(nx,ny)
inten=fltarr(nx,ny)

i1=0 & i2=nx-1
j1=0 & j2=ny-1
if n_elements(sub) eq 4 then begin
 i1=sub(0) & i2=sub(1) & j1=sub(2) & j2=sub(3)
 dprint,'using sub..'
 if exist(missing) then begin
  width(*)=missing
  cent(*)=missing
  inten(*)=missing
 endif
endif

if not exist(fixp) then fixp=[1,2]
chk=where(fixp eq 5,count)
fix_w=(count eq 1)

cwave=cds_tilt_cor(ql,window)
for i=i1,i2 do for j=j1,j2 do begin
 spec=gt_spectrum(ql,xix=i,yix=j,wind=window,/nocheck)
 lam=cwave(*,i,j)

 if exist(wrange) then begin
  pick=where((lam ge min(wrange)) and (lam le max(wrange)),count)
  if count gt 0 then begin
   wave=lam(pick)
   spec=spec(pick)
  endif
 endif else wave=lam

 if exist(wave) then begin

;-- substitute user guesses

  if exist(a) then begin
   if exist(wback) then begin
    a(0)=wback 
    if exist(fixp) then a(fixp)=0
   endif
   if exist(wcen) then begin
    a(4)=wcen
    diff=abs(wave-wcen)
    loc=min(diff,k)
    a(3)=(spec(k)-a(0)) > 0
   endif
   if exist(wdopp) then a(5)=wdopp
  endif

  f=gauss_fit(wave,spec,a,/last,status=status,fixp=fixp)
  if keyword_set(plot) then begin
   back=a(0)+a(1)*wave+a(2)*wave^2
   plot,wave,(spec-back) > 0.,psym=10
   oplot,wave,(f-back) > 0.
   t=''
   if keyword_set(debug) then read,t
   if (t eq 'q') then plot=0
  endif
  if status then begin
   cent(i,j)=a(4)
   inten(i,j)=a(3)
   width(i,j)=a(5)
   if fix_w and exist(wdopp) then begin
    if (a(5) ne wdopp) then begin
     print,'%non-fixed width value: ',a(5)
     print,' at ',i,j
    endif
   endif
  endif
 endif
endfor

return & end
