;+
; Project     : SOHO - CDS     
;                   
; Name        : CAT_FITS()
;               
; Purpose     : Cataslogues available FITS files and optional info.
;               
; Explanation : Lists the available FITS files in the official directory or 
;               user's own directory.
;               
; Use         : IDL> cat_fits([,filter, /quiet, ascii=hardcopy_file, info=info])
;    
; Inputs      : None
;               
; Opt. Inputs : filter - string which returned file names must contain.
;               
; Outputs     : Function returns a string array
;               
; Opt. Outputs: None
;               
; Keywords    : QUIET -  if present the results are not listed to the screen
;
;               ASCII - if present specifies the ascii file in which results
;                          are saved.
;
;               INFO     - if given as an output keyword variable the header
;                          information is returned in it.
;
;               TIMERANGE - allows selection of files by date.  If only one 
;                           value is given, it is taken as a 'start" date/time
;
; Calls       : None
;
; Common      : None
;               
; Restrictions: None
;               
; Side effects: None
;               
; Category    : Help, FITS
;               
; Prev. Hist. : Based on LIST_FITS
;
; Written     : C D Pike, RAL, 20-Mar-96
;               
; Modified    : Fudge FINDFILE to cope with many files.  CDP, 21-Mar-96
;               Use LOC_FILE for above. Separate HARDCOPY and ASCII keywords
;                                                        CDP, 25-Mar-96
;               Cut any wildcard characters from input (they are assumed 
;               anyway).  CDP, 28-Mar-96
;               Refix file finding.    CDP, 1-Apr-96
;               Use find_files instead of above fix.  CDP, 8-May-96
;
; Version     : Version 6, 8-May-96
;-            

pro cat_fits, filter, quiet=quiet, ascii=hfile, hardcopy=hardcopy, info=info, $
                      timerange=timerange
;
;  check have correct evar
;
if getenv('CDS_FITS_DATA') eq '' then begin
   print,'$CDS_FITS_DATA is not defined.'
   return
endif

;
;  get files
;
ff = find_files('*.fits','$CDS_FITS_DATA')


;
;  apply filter if given
;
if n_params() eq 1 then begin
   if datatype(filter,1) eq 'String' then begin
      ffilt = strcompress(repchar(filter,'*',' '),/rem)
      n = where(strpos(ff,ffilt) ge 0, count)
      if count gt 0 then ff = ff(n) else ff = ''
   endif else begin
      bell
      print,'File name filter must be a string - ignored.'
   endelse
endif

;
;  check have some files
;
n = where(ff ne '',count)
if count eq 0 then begin
   bell
   print,'No matched files found.'
   return
endif

;
;  if a time range was required then wait until the time in the file is
;  known
;
case n_elements(timerange) of
  0: begin
        trange = dblarr(2)
        trange(0) = utc2tai(str2utc('1995-01-01'))
        trange(1) = utc2tai(str2utc('2005-01-01'))
        tgiven = 0
     end
  1: begin
        temp = utc2tai(anytim2utc(timerange))
        trange = dblarr(2)
        trange(0) = temp
        trange(1) = utc2tai(str2utc('2005-01-01'))
        tgiven = 1
     end
  2: begin
        trange = utc2tai(anytim2utc(timerange))
        tgiven = 1
     end
 else: begin
         print,'Invalid time range'
         return
       end
endcase

;
;  inform of time range
;
if not keyword_set(quiet) then begin
   t = anytim2cal(tai2utc(trange))
   print,' '
   print,'Locating files generated between: '+t(0)+' and '+t(1)
   print,' '
endif
   
;
;  gather headers
;
info = strarr(n_elements(ff))
for i=0,n_elements(ff)-1 do begin
   h = headfits(ff(i))       
   ndet = where(strpos(h,'DETECTOR') ge 0)
   ndet = ndet(0)

   ngset = where(strpos(h,'GSET_ID') ge 0)
   ngset = ngset(0)
   
   nfil = where(strpos(h,'FILENAME') ge 0)
   nfil = nfil(0)
   
   ndate1 = where(strpos(h,'DATE_OBS') ge 0)
   ndate1 = ndate1(0)

   ndate2 = where(strpos(h,'DATE_END') ge 0)
   ndate2 = ndate2(0)

   nobj = where(strpos(h,'SCI_OBJ') ge 0)
   nobj = nobj(0)

   nspec = where(strpos(h,'SCI_SPEC') ge 0)
   nspec = nspec(0)


   det = strtrim(str_pick(h(ndet),"'","'"),2)
   if ngset ge 0 then gs='('+strtrim(str_pick(h(ngset),"=","/"),2)+')' $
                 else gs='    '
   fil = strpad(str_pick(strtrim(str_pick(h(nfil),"'","'"),2),'','.'),8,/aft)
   obs = strtrim(str_pick(h(ndate1),"'","'"),2)
   obs = repstr(obs,'T','  ')
   obs = repchar(obs,'Z',' ')
   obe = strtrim(str_pick(h(ndate2),"'","'"),2)
   obe = repstr(obe,'T','  ')
   obe = repchar(obe,'Z',' ')
   obe = strmid(obe,12,8)
   obj = strpad(strtrim(str_pick(h(nobj),"'","'"),2),30,/after)
   spec = strtrim(str_pick(h(nspec),"'","'"),2)

   b = '  '
   if not tgiven then begin
      sobs = strmid(obs,0,20)
      info(i) = fil+b+det+gs+b+sobs+b+obe+b+obj+b+spec 
   endif else begin
      tobs = utc2tai(str2utc(obs))
      sobs = strmid(obs,0,20)
      if (tobs ge trange(0)) and (tobs le trange(1)) then begin
         info(i) = fil+b+det+gs+b+sobs+b+obe+b+obj+b+spec 
      endif else begin
         info(i) = ''
         ff(i) = ''
      endelse
   endelse
endfor
   
   
;
;  re-sort file and info list
;
n = where(ff ne '',count)
if count gt 0 then begin
   ff = ff(n)
   info = info(n)
endif

;
;  sort into time order
;
tinfo = strmid(info,15+strlen(gs),20)
if n_elements(tinfo) gt 1 then begin
   tinfo = utc2tai(str2utc(tinfo))
   nt = sort(tinfo)
   info = info(nt)
endif 

;
;  add titles
;
info = ['  File    Det         Date      Start      End         '+$
        '           Objective                    ',info]  

;
;  present the header info if needed
; 
if not keyword_set(quiet) then print_str,info

;
;  save to file and/or print 
;
if keyword_set(hardcopy) then hcopy = 1 else hcopy=0
if keyword_set(ascii) then kcopy = 1 else kcopy=0

print_str,info,/quiet,hard=hcopy,keep=kcopy,file=hfile

;
;  return 
;
return

end

