;+
; Project     : SOHO - CDS     
;                   
; Name        : CDS_FILL_MISSING
;               
; Purpose     : Fills MISSING pixels in QLDS
;               
; Explanation : This procedure goes through all data in a QLDS, filling in
;               pixels marked as MISSING.
;
;               For NIS data (withouth pixel binning) it tries to fill the
;               missing pixels by calculating the local median of neighbouring
;               pixels inside a rectangular box (3 by 9 pixels) on the same
;               exposure.
;
;               For those pixels where the above method fails, or for data
;               sets where the above method is not applied, the routine
;               FILL_MISSING is used to interpolate a given pixel's value
;               between neighbouring *exposures*.
;
; Use         : CDS_FILL_MISSING,QLDS
;    
; Inputs      : QLDS : A Quicklook Data Structure
;               
; Opt. Inputs : None.
;               
; Outputs     : None.
;               
; Opt. Outputs: None.
;               
; Keywords    : INTERPOLATE : Set this keyword to always use the FILL_MISSING
;                             routine to interpolate between exposures. This 
;                             is the default for GIS spectra.
;
; Calls       : fill_missing, fmedian(), gt_dimension(), gt_windata()
;               qlmgr, st_windata, trim()
;
; Common      : None
;               
; Restrictions: 
;               
; Side effects: None.
;               
; Category    : ?
;               
; Prev. Hist. : None.
;
; Written     : S.V.H. Haugan, UiO, 4 January 1997
;               
; Modified    : Version 2, V. Andretta (NASA/GSFC), 19 March 1998
;                       Fixed a problem with GIS one-dimensional sequences.
;                       No informational message is issued if FILL_MISSING is 
;                       used and no call to FMEDIAN was possible/requested.
;
; Version     : 2, 19 March 1998
;-            

PRO cds_fill_missing,qlds,interpolate=interpolate
  
  on_error,2
  IF !debug NE 0 THEN on_error,0
  
  IF n_params() NE 1 THEN message,"Use: CDS_FILL_MISSING,QLDS"
  
  qlmgr,qlds,valid
  
  IF NOT valid THEN MESSAGE,"Parameter is not a valid QLDS"
  
  dim = gt_dimension(qlds)
  
  use_median = 0
  
  IF qlds.header.detector EQ 'NIS' THEN BEGIN
     IF qlds.detdesc(0).binx LE 1 AND qlds.detdesc(0).biny LE 1 THEN $
        use_median = 1
  END
  
  IF keyword_set(interpolate) THEN use_median = 0
  
  validwin = where(qlds.detdesc(*).ixstop(0) GE 0,count)
  
  FOR w = 0,count-1 DO BEGIN
     ww = validwin(w)
     
     catch,error
     IF error NE 0 THEN BEGIN
        catch,/cancel
        message,!err_string,/continue
        message,"Caught error",/continue
        IF n_elements(da) NE 0 THEN BEGIN
           message,"Saving data",/continue
           st_windata,qlds,ww,da
           message,"Aborting"
        END
     END
        
     da = gt_windata(qlds,ww,/no_copy)
     missing = qlds.detdesc(ww).missing
     
     IF use_median THEN BEGIN
        
        FOR tix = 0,(dim.sdel_time-1) > 0 DO BEGIN
           FOR xix = 0,(dim.ssolar_x-1) > 0 DO BEGIN
              ex = reform(da(*,xix,*,tix),/overwrite)
              missix = where(ex EQ missing,nmiss)
              IF nmiss GT 0 THEN BEGIN
                 med = fmedian(ex,3,9,missing=missing)
                 ex(missix) = med(missix)
                 da(*,xix,*,tix) = ex
              ENDIF
           ENDFOR
        ENDFOR
        
     ENDIF ; use_median
     
     leftix = where(da EQ missing,nmiss)
     
     IF nmiss GT 0 THEN BEGIN
        IF use_median THEN print,"Using fill_missing, window="+trim(ww)
        IF qlds.header.detector EQ 'NIS' THEN BEGIN     ; NIS
          FOR yix = 0,(dim.ssolar_y-1) > 1 DO BEGIN
             FOR lix = 0,(size(da))(1)-1 DO BEGIN
                line = reform(da(lix,*,yix,*))
                missix = where(line EQ missing,nmiss_line)
                IF nmiss_line GT 0 THEN fill_missing,line,missing
                da(lix,*,yix,*) = line
             END
          END
        ENDIF ELSE BEGIN                                ; GIS
          FOR lix = 0,(size(da))(1)-1 DO BEGIN
             line = reform(da(lix,*,*,*))
             missix = where(line EQ missing,nmiss_line)
             IF nmiss_line GT 0 THEN fill_missing,line,missing,(size(line))(0)
             da(lix,*,*,*) = line
          END
        ENDELSE
     END
     
     st_windata,qlds,ww,da,/no_copy
     catch,/cancel
  END
  
END
