
PRO CDS_NEW_SPIKE, QLDS, INFO=INFO, NO_NEIGHBOURS=NO_NEIGHBOURS, $
                     NOFILL=NOFILL

;+
; PROJECT:	SOHO - CDS
;
; NAME: 	CDS_NEW_SPIKE
;
; PURPOSE: 
;
;	Removes cosmic rays from CDS quicklook data structures
;
; CATEGORY:
;
;	Utility
;
; EXPLANATION:
;
;	Takes each set of window data from the QLDS and cleans them
;	with the NEW_SPIKE routine. If the data has not been de-biased 
;	then VDS_DEBIAS is called first. Unless the keyword NOFILL is set, 
;	the cosmic ray pixels are filled in, otherwise they are given the 
;	MISSING value.
;
;	For more details on the cleaning process see the header for the 
;	NEW_SPIKE routine.
;
; EXAMPLES: 
;
;	CDS_NEW_SPIKE, QLDS
;
;	CDS_NEW_SPIKE, QLDS, /NOFILL, /INFO
;
; INPUTS:
;
;	QLDS -	CDS quicklook data structure.
;
; KEYWORDS:
;
;	INFO -		Display number of flagged pixels for each window
;
;	NO_NEIGHBOURS -	Don't flag neighbours of cosmic rays
;
;	NOFILL -	Change cosmic rays to missing value of QLDS
;
; CALLS:
;
;	REARRANGE, GT_NUMWIN, GT_DIMENSION, NEW_SPIKE, VDS_DEBIAS,
;	GT_WINDATA, ST_WINDATA, GT_WLABEL
;
; RESTRICTIONS: 
;
;	The QLDS data should not be calibrated.
;
; HISTORY:
;
;	Ver.1, PRY 18-Aug-98
;	Ver.2, PRY 24-Nov-98, cosmetic changes
;       Ver.3, PRY 20-Feb-03, caught error when array dimensions are too 
;                 small
;       Ver.4, PRY 15-Aug-03, added ybin= when calling new_spike to allow
;                 for data binned in Y-direction.
;
;
; WRITTEN:
;
;	P.R. Young, Rutherford Appleton Laboratory, p.r.young@rl.ac.uk
;-

IF n_params() LT 1 THEN BEGIN
  print,'Use: IDL> cds_new_spike, qlds [,/info, /no_neighbours, /nofill]'
  return
ENDIF


IF qlds.detdesc(0).units NE 'DEBIASED-ADC' THEN BEGIN
  MESSAGE,"Removing bias",/informational
  vds_debias,qlds
END

n_win=gt_numwin(qlds)
dim=gt_dimension(qlds)

fill = KEYWORD_SET(nofill) - 1b
neighbours=KEYWORD_SET(no_neighbours) - 1b

FOR i=1,n_win DO BEGIN
  missing=qlds.detdesc(i-1).missing
  arr=gt_windata(qlds,i-1)
;  -----
;   If the raster consists of repeated exposures at the same slit position,
;   then the fourth dimension is classed as the X direction instead of the
;   second dimension.
;
  IF dim.ssolar_x EQ 1 THEN BEGIN
    arr_sub=reform(arr(*,0,*,*))
    arr_sub=rearrange(arr_sub,[1,3,2])
  ENDIF ELSE BEGIN
    arr_sub=arr
  ENDELSE
 ;
  IF KEYWORD_SET(info) THEN $
       print,STRCOMPRESS(i),'. ', gt_wlabel(qlds,i-1),' --'
 ;
 ; Clean data; arr_clean contains the cleaned data
 ;
 ;  ybin= is a keyword (on or off). qlds.detdesc.ybin can be 1 (no binning)
 ;  or more. new_spike applies a slightly different method if the data is
 ;  y-binned.
 ;
  new_spike,arr_sub,arr_clean,neighbours=neighbours,info=info, $
              missing=missing, fill=fill,ybin=qlds.detdesc[i-1].biny-1
  arr_sub=0
  IF n_elements(arr_clean) EQ 1 AND arr_clean[0] EQ -1. THEN BEGIN
    print,'** CDS_NEW_SPIKE Aborting...'
    print,''
    return
  ENDIF
 ;
  IF KEYWORD_SET(info) THEN print,''
;  -----
;   Need to reset the array back to its original form
;
  IF dim.ssolar_x EQ 1 THEN BEGIN
    arr_clean=rearrange(temporary(arr_clean),[1,3,2])
    arr(*,0,*,*)=arr_clean
  ENDIF ELSE BEGIN
    arr=arr_clean
  ENDELSE
  arr_clean=0
;
  st_windata,qlds,i-1,arr
  arr=0
ENDFOR

END
