	FUNCTION CDS_QUASI_FIT, QLDS, WINDOW, QUIET=QUIET, ERRMSG=ERRMSG
;+
; Project     :	SOHO - CDS
;
; Name        :	CDS_QUASI_FIT()
;
; Purpose     :	Quasi-fits line profile to extract image
;
; Category    :	Class3, CDS, Analysis
;
; Explanation :	CDS/NIS spectral line data sit on top of a scattered light
;		component due to small irregularities in the grating.  This
;		scattered light component is spatially structured, mimicking
;		the behavior of continuum radiation.  In order to extract the
;		correct spatial behavior of the line being observed, then some
;		analysis must be applied to the data to remove the background
;		component.
;
;		This routine applies some crude analysis to remove the
;		scattered light component.  It does this by taking the average
;		of the three leftmost and rightmost pixels in the wavelength
;		direction.  If there appears to be significant data on one side
;		that isn't on the other side, as recognized by the statistics
;		in the two average images, then only the other side is used.
;
; Syntax      :	Result = CDS_QUASI_FIT( QLDS, WINDOW )
;		Result = CDS_QUASI_FIT( ARRAY, MISSING )
;
; Examples    :	QLDS = READCDSFITS('s3300r00.fits')
;		IMAGE = CDS_QUASI_FIT(QLDS, 0)
;
; Inputs      :	QLDS	= A CDS quicklook data structure as returned by
;			  READCDSFITS.  Alternatively, this could be an array
;			  containing a single data window.
;
;		WINDOW	= The index number of the desired window, with values
;			  from 0 to N-1.  Alternatively, if an array is passed
;			  instead of a QLDS, then this is the missing pixel
;			  flag value.
;
; Opt. Inputs :	None.
;
; Outputs     :	The extracted image is returned as the result of the function.
;
; Opt. Outputs:	None.
;
; Keywords    :	QUIET	= If the wavelength coverage is less than nine pixels,
;			  then a straight average is applied.  When the /QUIET
;			  keyword is used, the warning messages about this are
;			  turned off.
;
;		ERRMSG = If defined and passed, then any error messages will be
;			 returned to the user in this parameter rather than
;			 depending on the MESSAGE routine in IDL.  If no errors
;			 are encountered, then a null string is returned.  In
;			 order to use this feature, ERRMSG must be defined
;			 first, e.g.
;
;				ERRMSG = ''
;				A = CDS_QUASI_FIT( ERRMSG=ERRMSG, ... )
;				IF ERRMSG NE '' THEN ...
;
; Calls       :	GT_WINDATA, GT_WINDESC, AVERAGE, FILL_MISSING, STDEV, 
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	Missing pixels are filled in.
;
; Prev. Hist. :	None.
;
; History     :	Version 1, 13-Jan-1997, William Thompson, GSFC
;		Version 2, 13-Mar-1997, William Thompson, GSFC
;			Improved handling of missing pixels
;
; Contact     :	WTHOMPSON
;-
;
;  Unless the !DEBUG option was requested, then return to the calling routine
;  if an error is encountered.
;
	IF !DEBUG EQ 0 THEN ON_ERROR, 2 ELSE ON_ERROR, 0
;
;  Check the input parameters.
;
	IF N_PARAMS() NE 2 THEN BEGIN
		MESSAGE = 'Syntax:  Result = CDS_QUASI_FIT( QLDS, WINDOW )'
		GOTO, HANDLE_ERROR
	ENDIF
;
;  Extract the requested data window and the corresponding description.
;
	IF DATATYPE(QLDS,1) EQ 'Structure' THEN BEGIN
		MESSAGE = ''
		A = GT_WINDATA(QLDS, WINDOW, ERRMSG=MESSAGE)
		IF MESSAGE NE '' THEN GOTO, HANDLE_ERROR
		D = GT_WINDESC(QLDS, WINDOW, ERRMSG=MESSAGE)
		IF MESSAGE NE '' THEN GOTO, HANDLE_ERROR
		MISSING = D.MISSING
;
;  If only an array was passed rather than a structure, then the second
;  parameter is the missing pixel flag value.
;
	END ELSE BEGIN
		A = QLDS
		MISSING = WINDOW
	ENDELSE
;
;  Check the size of the image.  If there aren't enough wavelength points, then
;  simply return the estimated total.
;
	SZ = SIZE(A)
	IF SZ(0) LT 2 THEN BEGIN
		MESSAGE = 'Array must be at least two-dimensional'
		GOTO, HANDLE_ERROR
	END ELSE IF SZ(1) EQ 1 THEN BEGIN
		IF NOT KEYWORD_SET(QUIET) THEN MESSAGE, /INFORMATIONAL,	$
			'No wavelength coverage'
		RETURN, REFORM(A, SZ(2:SZ(0)), /OVERWRITE)
	END ELSE IF SZ(1) LT 9 THEN BEGIN
		IF NOT KEYWORD_SET(QUIET) THEN MESSAGE, /INFORMATIONAL,	$
			'Insufficient wavelength coverage to analyze -- ' + $
			'using total'
		RETURN, AVERAGE(TEMPORARY(A), 1, MISSING=MISSING) * SZ(1)
	ENDIF
;
;  Fill in any missing pixels from other exposures.
;
	IF SZ(0) EQ 4 THEN D_EXP = 4 ELSE D_EXP = 2
	ASAVE = A
	FILL_MISSING, A, MISSING, D_EXP
;
;  Extract the average left and right image.
;
	AL = AVERAGE(A(0:2,*,*,*), 1)
	AR = AVERAGE(A(SZ(1)-3:*,*,*,*), 1)
;
;  Clean any cosmic rays from the image.
;
	CDS_CLEAN_IMAGE, AL
	CDS_CLEAN_IMAGE, AR
;
;  Use the statistics of the images to determine which image to use as the
;  estimate of the scatter, or whether the average of both images should be
;  used.
;
	S_AL = STDEV(AL)
	S_AR = STDEV(AR)
	IF S_AL GE 1.5*S_AR THEN ALR = AR ELSE	$
		IF S_AR GE 1.5*S_AL THEN ALR = AL ELSE ALR = (AL + AR) / 2.
;
;  Subtract the estimated scatter, and sum over wavelength.
;
	A = (AVERAGE(TEMPORARY(A), 1) - ALR) * SZ(1)
;
;  Find all the pixels which contain no profiles at all.
;
	ASAVE = AVERAGE(TEMPORARY(ASAVE), 1, MISSING=MISSING)
	W = WHERE(ASAVE EQ MISSING, COUNT)
	IF COUNT GT 0 THEN A(W) = MISSING
	RETURN, A
;
;  Error handling point.
;
HANDLE_ERROR:
	IF N_ELEMENTS(ERRMSG) NE 0 THEN ERRMSG = 'CDS_QUASI_FIT: ' + MESSAGE $
		ELSE MESSAGE, MESSAGE
	RETURN, 0
	END
