;+
; Project     : SOHO - CDS     
;                   
; Name        : CDS_READ_MISSING
;               
; Purpose     : Read list of missing pixels and apply to QLDS
;               
; Explanation : The procedure CDS_SAVE_MISSING can be used to save a list of
;               MISSING pixels (cosmic rays etc) in a QLDS to an IDL SAVE
;               file. This routine reads such a save file and flags those
;               pixels as MISSING in the supplied QLDS.
;               
; Use         : CDS_READ_MISSING,QLDS,FILENAME
;    
; Inputs      : QLDS : Quick Look Data Structure
;
;
; Opt. Inputs : FILENAME : Name of save-file containing the list of MISSING
;                          pixels. Must have been written by CDS_SAVE_MISSING.
;                          If this parameter is not present, then the STATUS
;                          keyword must contain the missing status (see
;                          CDS_SAVE_MISSING).
;               
; Outputs     : Alters the QLDS
;               
; Opt. Outputs: None.
;               
; Keywords    : STATUS : The missing status of a QLDS, as ouput from
;                        CDS_SAVE_MISSING.
;
; Calls       : gt_windata(), parcheck, qlmgr, st_windata, test_open(), typ()
;
; Common      : None.
;               
; Restrictions: None.
;               
; Side effects: None.
;               
; Category    : CDS utility
;               
; Prev. Hist. : None.
;
; Written     : S. V. H. Haugan, UiO/GSFC, 26 November 1996
;               
; Modified    : Version 2, SVHH, 3 January 1997
;                       Added STATUS keyword.
;               Version 3, SVHH, 23 March 1998
;                       Corrected spelling of ST_WINDATA (pointed out by
;                       V. Andretta) and added CATCH,/CANCEL.
;                       
; Version     : 3, 23 March 1998
;-            

pro cds_read_missing,qlds,filename,status=status
  
  on_error,2
  
  IF !debug NE 0 THEN on_error,0
  
  IF n_params() LT 1 THEN $
     message,"Use: CDS_SAVE_MISSING,QLDS [,FILENAME]"
  
  qlmgr,qlds,valid
  IF NOT valid THEN $
     message,"First parameter must be a valid QLDS"
  
  IF n_params() EQ 2 THEN BEGIN
     parcheck,filename,2,typ(/str),0,'FILENAME'
     
     IF NOT test_open(filename) THEN $
        message,"Cannot open file "+filename
     
     restore,filename           ; Gets MISSIX, WINS
  END ELSE BEGIN
     IF n_elements(status) EQ 0 THEN $
        message,"Must have either FILENAME or STATUS"
     missix = status.missix
     wins = status.wins
  END
  
  FOR w = 0,N_elements(qlds.detdesc)-1 DO BEGIN
     IF qlds.detdesc(w).ixstop(0) GE 0 THEN BEGIN
        missing = qlds.detdesc(w).missing
        
        missing_ix_ix = where(wins EQ w,count)
        IF count GT 0 THEN BEGIN
           missing_ix = missix(missing_ix_ix)
           
           catch,error
           IF error NE 0 THEN BEGIN
              catch,/cancel
              print,!err_string
              IF n_elements(winda) GT 0 THEN BEGIN
                 st_windata,qlds,w,winda ; Don't use no_copy here
                 message,"Saved window data",/continue
              END
              message,"Aborting"
           END
        
           winda = gt_windata(qlds,w,/no_copy)
           winda(missing_ix) = missing
           st_windata,qlds,w,winda,/no_copy
           catch,/cancel
           
        END
     END
  END
  
END
