;+
; Project     : SOHO - CDS     
;                   
; Name        : CDS_REPAIR_TOP
;               
; Purpose     : "Repair" the tops of saturated lines ( 4095 counts )
;               
; Explanation : This routine, as it is, should be applied to NIS data only,
;               *before* any calibration/debiasing routines have been applied.
;
;               CDS_REPAIR_TOP goes through all single spectra in the QLDS and
;               replaces saturated pixel values with values matching a
;               Gaussian fit to the surrounding, non-saturated pixels.
;
;               The large amount of assumptions necessary to justify such a
;               procedure should be obvious, so *BE CAREFUL* about analyzing
;               data after processing them with this program.
;
;               Of course, if your data contains anything else than a single
;               gaussian plus background, this routine may fill the saturated
;               pixels with stuff that's even more wrong than the saturated
;               values.
;
; Use         : CDS_REPAIR_TOP,QLDS
;    
; Inputs      : QLDS : Quick Look Data Structure.
;               
; Opt. Inputs : K : The number of coefficients included in the fit of the
;                   background level in CDS_GAUSS. Default is 1 (i.e., a
;                   constant background).
;               
; Outputs     : The data in the QLDS are altered.
;               
; Opt. Outputs: None.
;               
; Keywords    : None.
;
; Calls       : default, gt_dimension(), gt_windata(), qlmgr, st_windata
;
; Common      : None.
;               
; Restrictions: See Explanation.
;               
; Side effects: None.
;               
; Category    : 
;               
; Prev. Hist. : None.
;
; Written     : S. V. H. Haugan, UiO/GSFC, 26 November 1996
;               
; Modified    : Not yet.
;
; Version     : 1, 26 November 1996
;-            



pro cds_repair_singletop,spec,missing,k,topvalue
  
  top = spec EQ topvalue
  miss = spec EQ missing
  
  IF total(top) EQ 0 THEN return
  
  good = (top-1b) AND (miss-1b) ;; Ends up as either 255b or 0b
  
  goodix = where(good,ngood)
  IF ngood LT (4 + k) THEN BEGIN
     message,"Too few points to make fit",/continue
     return
  END
     
  badix = where(NOT good)
  
  ;;
  initial_intens = topvalue
  
  ;; Average position of saturated pixels
  
  initial_center = total(top * findgen(n_elements(top)))/total(top)
  
  ;; Just a guess.., for the NIS
  initial_wid = 2.0
  
  print,initial_center,initial_wid
  
  initial_a = [initial_intens,initial_center,initial_wid]
  
  dummy = cds_gauss(goodix,spec(goodix),a,k,initial_a = initial_a)
  
  ;; Filling in:
  ;; It's crazy that cds_gfunct is non-callable
  
  aa = dblarr(6)
  aa(0) = a
  z=(badix-aa(1))/aa(2)
  
  spec(badix) = aa(0)*exp(-z^2/2) + aa(3) + aa(4)*badix + aa(5)*badix^2
  
  return
end

pro cds_repair_top,qlds,k,topvalue=topvalue
  
  on_error,2
  IF !debug NE 0 THEN on_error,0
  
  ;; Various checks
  
  qlmgr,qlds,valid
  IF NOT valid THEN message,"Parameter must be a valid QLDS"
  
  IF qlds.header.detector NE 'NIS' THEN BEGIN
     message,"Can only apply CDS_REPAIR_TOP to NIS data",/continue
     return
  END
  
  default,topvalue,4095
  
  wgoodix = where(qlds.detdesc(*).ixstop(0) GT 0)
  IF wgoodix(0) EQ -1 THEN return
  
  IF qlds.detdesc(wgoodix(0)).units NE 'ADC' THEN BEGIN
     message,"Can only apply CDS_REPAIR_TOP *before* debiasing",/continue
     return
  END
  
  ;; We should be able to proceed now...
  
  ;; Default is to have constant background
  
  default,k,1
  
  ;; Get info on QLDS
  
  dims = gt_dimension(qlds)
  
  FOR w = 0,n_elements(qlds.detdesc)-1 DO BEGIN
     IF qlds.detdesc(w).ixstop(0) GT 0 THEN BEGIN
        winda = gt_windata(qlds,w)
        IF total(winda EQ topvalue) GT 0 THEN BEGIN
           missing = qlds.detdesc(w).missing
           FOR tix = 0,(dims.sdel_time-1) > 0 DO BEGIN 
              FOR yix = 0,(dims.ssolar_y-1) > 0 DO BEGIN
                 FOR xix = 0,(dims.ssolar_x-1) > 0 DO BEGIN
                    spec = reform(winda(*,xix,yix,tix))
                    cds_repair_singletop,spec,missing,k,topvalue
                    winda(*,xix,yix,tix) = spec
                 END
              END
           END
           st_windata,qlds,w,winda,/no_copy
        END
     END
  END
  
  
  return
end
