;+
; Project     : SOHO - CDS     
;                   
; Name        : CDS_SAVE_MISSING
;               
; Purpose     : Save a list of MISSING pixels to a file
;               
; Explanation : To avoid e.g., removing cosmic rays manually more than once,
;               this routine will save a list of those pixels marked as
;               MISSING. CDS_READ_MISSING will be able to read that list and
;               mark the corresponding pixels as missing during another IDL
;               session.
;
;               It is possible to use CDS_SAVE_MISSING/CDS_READ_MISSING
;               without writing/reading a file, through the keyword STATUS,
;               e.g.:
;
;               CDS_SAVE_MISSING,QLDS,STATUS=NAMED
;               ....
;               CDS_READ_MISSING,QLDS,STATUS=NAMED
;               
; Use         : CDS_SAVE_MISSING,QLDS,FILENAME
;    
; Inputs      : QLDS : Quick Look Data Structure
;
;                          
; Opt. Inputs : FILENAME:  The name of the file in which the list of MISSING
;                          pixels will be saved.
;               
; Outputs     : None.
;               
; Opt. Outputs: None.
;               
; Keywords    : STATUS : A named variable to receive the status structure
;                        containing the list of missing pixels.
;
; Calls       : gt_windata(), parcheck, qlmgr, st_windata, test_open(), typ()
;
; Common      : None.
;               
; Restrictions: None.
;               
; Side effects: None.
;               
; Category    : CDS utility
;               
; Prev. Hist. : None.
;
; Written     : S. V. H. Haugan, UiO/GSFC, 27 November 1996
;               
; Modified    : Version 2, SVHH, 3 January 1997
;                       Corrected bug, added STATUS keyword
;               Version 3, SVHH, 23 March 1998
;                       Corrected spelling of ST_WINDATA (pointed out by 
;                       V. Andretta), added catch,/cancel
;                       
; Version     : 3, 23 March 1998
;-            
PRO cds_save_missing,qlds,filename,status=status
  
  on_error,2
  
  IF !debug NE 0 THEN on_error,0
  
  IF n_params() LT 1 THEN $
     message,"Use: CDS_SAVE_MISSING,QLDS [,FILENAME]"
  
  qlmgr,qlds,valid
  IF NOT valid THEN $
     message,"First parameter must be a valid QLDS"
  
  IF n_params() EQ 2 THEN BEGIN
     parcheck,filename,2,typ(/str),0,'FILENAME'
     IF NOT test_open(filename,/write) THEN $
        message,"Cannot write to file "+filename
  END
  
  missix = [-1]
  wins = [-1]
  
  FOR w = 0,N_elements(qlds.detdesc)-1 DO BEGIN
     IF qlds.detdesc(w).ixstop(0) GE 0 THEN BEGIN
        missing = qlds.detdesc(w).missing
        
        catch,error
        IF error NE 0 THEN BEGIN
           catch,/cancel
           print,!err_string
           IF n_elements(winda) GT 0 THEN BEGIN
              st_windata,qlds,w,winda ; Don't use no_copy here
              message,"Saved window data",/continue
           END
           message,"Aborting"
        END
        
        winda = gt_windata(qlds,w,/no_copy)
        new_ix = where(winda EQ missing,count)
        st_windata,qlds,w,winda,/no_copy
        catch,/cancel
        
        IF count GT 0 THEN BEGIN
           missix = [missix,new_ix]
           wins = [wins,replicate(byte(w),count)]
        END
     END
  END
  
  status = {wins:wins,missix:missix}
  
  IF n_params() EQ 2 THEN save,missix,wins,filename=filename
  
END
