;+
; Project     : SOHO - CDS     
;                   
; Name        : CDS_WAVE_CAT()
;               
; Purpose     : Find data extraction windows that match a given input.
;               
; Explanation : Looks up windows for the given element.  If an ionisation
;               number is specified, it will pick out the matches for both
;               the given element and ionization number.  If a wavelength
;               is also specified the windows which match all three will be
;               found along with the general extraction windows (WW_...)
;               whose wavelengths match the specified wavelength.
;               If just a wavelength is specified it will find windows that
;               match solely in wavelength.

; Use         : IDL> win = cds_wave_cat(elem, ion, wave)
;    
; Inputs      : elem - The element being searched for.  1 or 2 characters 
;               e.g. 'Fe'
;               
; Opt. Inputs : ion  - The ionization stage being searched for.  An integer.
;               wave - The wavelength being searched for. Between 300 & 
;                      650 Angstroms
;               
; Outputs     : Function returns Window names which are a match for the 
;               given input data.
;               
; Opt. Outputs: None
;               
; Keywords    : Range - wavelength search range (default +/-  1.5A)
;               Quiet - if set, no screen output. 
;               
;
; Calls       : None
;
; Common      : None
;               
; Restrictions: None
;               
; Side effects: None
;               
; Category    : Catalogues
;               
; Prev. Hist. : None
;
; Written     : A N Pike, RAL, 21/07/98 
;               
; Modified    : Fix IDL v4 bug, ANP, 3/9/98  
;
; Version     : Version 2, 3/09/98 
;-            

function cds_wave_cat,elem,ion,wave,range=range,quiet=quiet


;
;  check necessary files exist
;
if not file_exist(concat_dir('$CDS_HEAD_CAT','CDS_lines_all')) then begin
   print,'File: $CDS_HEAD_CAT/CDS_lines_all does not exist'
   return,''
endif

if not file_exist(concat_dir('$CDS_HEAD_CAT','CDS_WW_all')) then begin
   print,'File: $CDS_HEAD_CAT/CDS_WW_all does not exist'
   return,''
endif


;
;  Set default range.
;

if not keyword_set(range) then range = 1.5

;
;  Setup header line for output.
;

headerline = 'Window       Files    Comments'
headerww= 'Window       Files     Min      Max'

printed_line= 0
printed_ww= 0
out1= ''

;
;  If no parameters are specified, all windows are printed.
;


if n_params() eq 0 then begin
   txt= rd_ascii(concat_dir('$CDS_HEAD_CAT','CDS_lines_all'),lines=lines)
   if not keyword_set(quiet) then begin
      print,' '
      print_str, txt
   endif
   return, txt
endif


;
;  If one parameter is specified, it is checked to see if it is a string.
;

if n_params() eq 1 then begin
   k= elem
   jw= datatype(k,0)
   
   if jw ne 'STR' then begin

;
;  If it is not a string, it will be a wavelength that has been typed in.
;  This wavelength must be between 300 and 634 Angstroms.
;

      k= float(k)

      if k le 300. or k ge 634. then begin
         print,'Wavelength must be between 300 and 634 Angstroms.'
      endif

;
;  The 'CDS_WW_all' file is now read to find any windows that match the
;  specified wavelength.
;

 
      txt= rd_ascii(concat_dir('$CDS_HEAD_CAT','CDS_WW_all'),lines=lines)
      text= txt
      out= ''
  
      text= strcompress(text)
      nl= n_elements(text)
      for i= 6, nl-1 do begin
         z= str2arr(text(i),' ')
         wmin= float(z(2))
         wmax= float(z(3))
         if k ge wmin and k le wmax then begin
            out= [out,txt(i)]

            if printed_ww eq 0 and not keyword_set(quiet) then begin
               print,' '
               print,[headerww] 
               printed_ww= 1
            endif
            if not keyword_set(quiet) then print_str,txt(i) 
         endif
      endfor

;
;  The 'CDS_lines_all' file is now read to find any windows that match the
;  specified wavelength.
;

      tixt= rd_ascii(concat_dir('$CDS_HEAD_CAT','CDS_lines_all'),lines=lines)
      tixt= tixt(3:*)
      g= where(tixt ne ' ' and tixt ne '')
      tixt= tixt(g)
      j= strmid(tixt,0,12)
      for i=0,n_elements(j)-1 do begin
         m= str2arr(j(i),'_')
         wav= float(m(2)) + float(m(3))/100.
         v= k-wav
         if abs(v) le range then begin 
            out= [out,tixt(i)]

            if not keyword_set(quiet) then begin
               if printed_line eq 0 then begin
                  print,' '
                  print,[headerline]
                  printed_line= 1  
               endif
               print_str,tixt(i)
            endif 
         endif     
      endfor
      if n_elements(out) eq 1 then return,'' else return, out(1:*)
   endif
endif

;
;  Only one input parameter, element or wavelength?.
;

if n_params() ge 1 then begin
   f= elem
   a= datatype(f,0)

   if a eq 'STR' then begin


      elem= strcompress(elem, /remove_all)
      elem= strtrim(elem, 2)

      leng= strlen(elem)

;
;  Returns if elem consists of more than 2 characters.  
;

      if leng gt 2 then begin 
         print,'Too many characters in element name.' 
         return,''
      endif

;
;  Makes first character upper case, and second lower case so as to be in
;  same form as in the windows.
;

      cap= strmid(elem,0,1)
      cap= strupcase(cap)
      low= strmid(elem,1,1)
      low= strlowcase(low)

      elem= cap + low

;
;  Checks specified element is valid.
;

      name= ['Al', 'Ar', 'C', 'Ca', 'Cr', 'Fe', 'He', 'Mg', 'N', 'Ne', $ 
             'Ni', 'O', 'Si']
      maxion= [13,18,6,20,24,26,2,12,7,10,28,8,14]

      n= where(name eq elem,count)
      if count gt 0 then begin 
         imax= maxion(n(0))
      endif else begin 
         print,'Element not known.  Use: f = cds_wave_cat(elem, ion, wave)'
         return,''
      endelse


;
;  Checks ion number is an integer greater than 0 but less than the maximum
;  for the given element.
;
 
      if n_elements(ion) gt 0 then begin

         type= datatype(ion,0)

         if type ne 'INT' then begin
            print,'Ionization number must be an integer.'
            return,''
         endif


         if ion le 0 then begin
            print,'Ionization number must be greater than 0.  Use: ' + $
                  'w = cds_wave_cat(elem, ion, wave)'
            return,''
         endif


         if ion gt imax then begin
            print,'Ionization number too large for this element.  Use: ' + $  
                  'w = cds_wave_cat(elem, ion, wave)'
            return,''
         endif

      endif else begin
         if n_params() gt 1 then begin
            print,' Ionization number must be an integer'
            return,''
         endif
      endelse

;
;  Reading file which contains specific named windows.
;


      txt= rd_ascii(concat_dir('$CDS_HEAD_CAT','CDS_lines_all'),lines=lines)

;
;  Cuts off headers and blank lines of 'CDS_lines_all' which do not contain
;  data windows.
;

      txt= txt(3:*)
      n= where(txt ne ' ')
      txt= txt(n)

;
;  Sets elem and ion to the same form as they appear in the data windows.
;

      if n_elements(ion) gt 0 then begin
         p= elem + '_' + trim(ion) + '_'
      endif else begin
         p= elem + '_'
      endelse

      a= where(strpos(txt,p) ge 0,count)

;
;  If windows have been found to match the element and ion number, these
;  windows are printed on screen, along with the header.
;

      if count gt 0 then begin   
         if n_elements(ion) gt 0 then begin      
            txt= txt(a)
         endif else begin

            if not keyword_set(quiet) then begin

               if printed_line eq 0 then begin
                  print,' '
                  print,headerline
                  printed_line= 1
               endif

               print_str,txt(a)
            endif
            return,txt(a)
         endelse


;
;  If no matches have been found, 'Nothing found' is printed.
;

      endif else begin
         print,'Nothing found for: ',p
         if n_elements(wave) eq 0 then return,''
         endelse

;
;  If a wavelength is specified it checks this value is between 300 and 650
;  Angstroms.
;

      if n_elements(wave) gt 0 then begin
          
         if count gt 0 then begin
            w= wave

            if w le 300 or w ge 650 then begin 
               print,'Wavelength must be between 300 and 650 Angstroms.'
            endif

;
;  Checks for matches at specified wavelength.
;

            z= strlen(elem) + strlen(trim(ion)) +1
            y= fix(strmid(txt,z+1,3)) 

            x= y-w
  
            h= where(abs(x) le range,count) 
            if count gt 0 then begin
               out1= txt(h)
               if not keyword_set(quiet) then begin
                  if printed_line eq 0 then begin
                     print,' '
                     print,headerline
                     printed_line= 1
                  endif
                  print_str,txt(h)
               endif
            endif else begin 
               print,'Nothing found for specified element at this wavelength.'
            endelse  
         endif

;
;  Checks for matches with wavelengths in general data extraction windows.
;

         txt= rd_ascii(concat_dir('$CDS_HEAD_CAT','CDS_WW_all'),lines=lines)
   
         text= txt

         text= strcompress(text)
         nl= n_elements(text)
         out = ''
         for i= 6, nl-1 do begin
            z= str2arr(text(i),' ')
            wmin= float(z(2))
            wmax= float(z(3))
            
            wave= float(wave) 
            if wave ge wmin and wave le wmax then begin
               out = [out,txt(i)]
               if not keyword_set(quiet) then begin
 
                  if printed_ww eq 0 then begin
                     print,' '
                     print,headerww
                     printed_ww= 1
                  endif
  
                  print,txt(i)
               endif
            endif
         endfor
         if n_elements(out) gt 1 then return, [out1,out(1:*)] else return,out1
      endif else begin
         
         if not keyword_set(quiet) then begin

            if printed_line eq 0 then begin
               print,' '
               print,headerline
               printed_line= 1
            endif
            print_str,txt
         endif
         return, txt
      endelse  
   endif else begin
      print,' '
      print,'If the number of parameters entered is greater than 1,'
      print,'the first must be the element name.'   
      print,' '
   endelse
endif

end
