;+
; Project     : SOHO - CDS     
;                   
; Name        : LIST_FITS()
;               
; Purpose     : Produce a list of available FITS files and optional info.
;               
; Explanation : Returns and optionally lists the available FITS files in 
;               the official directory or user's own directory.
;               
; Use         : IDL> ff = list_fits([filter, /own, /ql, /quiet, /hard, $
;                                            info=info])
;    
; Inputs      : None
;               
; Opt. Inputs : filter - string which returned file names must contain.
;               
; Outputs     : Function returns a string array
;               
; Opt. Outputs: None
;               
; Keywords    : OWN   -  if present the current directory is searched rather
;                        than the official $CDS_FITS_DATA
;               QL    -  if present a list of FITS files in the official QLI
;                        directory ($CDS_FITS_QLI) is returned.
;               QUIET -  if present the results are not listed to the screen
;
;               HARDCOPY - if present a hardcopyo fthe output is sent to 
;                          the default printer
;
;               INFO     - if given as a output keytword variable the header
;                          information is returned in it.
;               NOINFO   - suppresses the header information
;
;               TIMERANGE - allows selection of files by date.  If only one 
;                           value is given, it is taken as a 'start" date/time
;
; Calls       : None
;
; Common      : None
;               
; Restrictions: None
;               
; Side effects: None
;               
; Category    : Help, FITS
;               
; Prev. Hist. : None
;
; Written     : C D Pike, RAL, 1-Nov-95
;               
; Modified    : Include filter string.   CDP, 16-Nov-95
;               Collect and present FITS header info.  CDP, 27-Jan-96
;               Add NOINFO and TIMERANGE keywords  2-Feb-96
;               Sort files for presentation of header.  4-Mar-96
;               Added GSET_ID for GIS files.    13-Mar-96
;               Added OBJECT and OBJ_ID.        14-Mar-96
;               Replace FINDFILE with LOC_FILE.  CDP, 25-Mar-96
;               Fixed bug associated with last change.  CDP, 28-Mar-96
;               Refix file finding.    CDP, 1-Apr-96
;               Use find_files instead of previous fix.  CDP, 8-May-96
;               Sort file list.  CDP, 14-May-96
;
; Version     : Version 12, 14-May-96
;-            

function list_fits, filter, own=own, qli=qli, quiet=quiet, $
                            hardcopy=hardcopy, info=info, noinfo=noinfo, $
                            timerange=timerange
;
;  check have correct evar
;
if (not keyword_set(own)) and (getenv('CDS_FITS_DATA') eq '') then begin
   print,'$CDS_FITS_DATA is not defined.'
   return,''
endif

if keyword_set(qli) and  (getenv('CDS_FITS_QLI') eq '') then begin
   print,'$CDS_FITS_QLI is not defined.'
   return,''
endif

;
;  get files
;
if keyword_set(own) then begin
   if keyword_set(qli) then begin
      ff = loc_file('cds_*.fits')
   endif else begin
      ff = loc_file('s*.fits')
   endelse
endif else begin
   if keyword_set(qli) then begin
      ff = find_files('*.fits','$CDS_FITS_QLI')
   endif else begin
      ff = find_files('*.fits','$CDS_FITS_DATA')
   endelse
endelse   

;
;  apply filter if given
;
if n_params() eq 1 then begin
   if datatype(filter,1) eq 'String' then begin
      n = where(strpos(ff,filter) ge 0, count)
      if count gt 0 then ff = ff(n)
   endif else begin
      bell
      print,'File name filter must be a string - ignored.'
   endelse
endif

;
;  sort them alphabetically
;
ff = ff(sort(ff))

;
;  if a time range was required then wait until the time in the file is
;  known
;
case n_elements(timerange) of
  0: begin
        trange = dblarr(2)
        trange(0) = utc2tai(str2utc('1995-01-01'))
        trange(1) = utc2tai(str2utc('2005-01-01'))
        tgiven = 0
     end
  1: begin
        temp = utc2tai(anytim2utc(timerange))
        trange = dblarr(2)
        trange(0) = temp
        trange(1) = utc2tai(str2utc('2005-01-01'))
        tgiven = 1
     end
  2: begin
        trange = utc2tai(anytim2utc(timerange))
        tgiven = 1
     end
 else: begin
         print,'Invalid time range'
         return,''
       end
endcase

;
;  inform of time range
;
if not keyword_set(quiet) then begin
   t = anytim2cal(tai2utc(trange))
   print,' '
   print,'Locating files generated between: '+t(0)+' and '+t(1)
   print,' '
endif
   
;
;  report if not silenced
;
if not tgiven then begin 
   if ff(0) ne '' then begin
      if not keyword_set(quiet) then begin
         print,'Available files are:'
         print,' '
         print_str,ff,/num
         print,' '
      endif
   endif else begin
      if not keyword_set(quiet) then begin
         bell
         print,'No files found.'
      endif
      return,''
   endelse
endif

;
;  gather headers
;
if not keyword_set(noinfo) then begin
   
   info = strarr(n_elements(ff))
   for i=0,n_elements(ff)-1 do begin
      h = headfits(ff(i))       
      ndet = where(strpos(h,'DETECTOR') ge 0)
      ndet = ndet(0)
   
      ngset = where(strpos(h,'GSET_ID') ge 0)
      ngset = ngset(0)
      
      nfil = where(strpos(h,'FILENAME') ge 0)
      nfil = nfil(0)
      
      ndate = where(strpos(h,'DATE_OBS') ge 0)
      ndate = ndate(0)
      
      nobs = where(strpos(h,'OBS_PROG') ge 0)
      nobs = nobs(0)
      
      ntit = where(strpos(h,'TITLE ') ge 0)
      ntit = ntit(0)
      
      nsid = where(strpos(h,'STUDY_ID') ge 0)
      nsid = nsid(0)
      
      nsvar = where(strpos(h,'STUDYVAR') ge 0)
      nsvar = nsvar(0)
      
      nrid = where(strpos(h,'RAS_ID') ge 0)
      nrid = nrid(0)
      
      nrv = where(strpos(h,'RAS_VAR') ge 0)
      nrv = nrv(0)

      nobj = where(strpos(h,'OBJECT') ge 0)
      nobj = nobj(0)

      noid = where(strpos(h,'OBJ_ID') ge 0)
      noid = noid(0)

   
      det = strtrim(str_pick(h(ndet),"'","'"),2)
      if ngset ge 0 then gs='('+strtrim(str_pick(h(ngset),"=","/"),2)+')' $
                    else gs='    '
      fil = strpad(str_pick(strtrim(str_pick(h(nfil),"'","'"),2),'','.'),8,/aft)
      obs = strtrim(str_pick(h(ndate),"'","'"),2)
      obs = repstr(obs,'T','  ')
      obs = repchar(obs,'Z',' ')
      prg = strpad(strtrim(str_pick(h(nobs),"'","'"),2),8,/after)
      obj = strpad(strtrim(str_pick(h(nobj),"'","'"),2),8,/after)
      oid = strpad(strtrim(str_pick(h(noid),"'","'"),2),8,/after)
      tit = strmid(strpad(strtrim(str_pick(h(ntit),"'","'"),2),50,/after),0,35)
      sid = strtrim(str_pick(h(nsid),"=","/"),2)
      svar = strtrim(str_pick(h(nsvar),"=","/"),2)
      rid = strtrim(str_pick(h(nrid),"=","/"),2)
      rv = strtrim(str_pick(h(nrv),"=","/"),2)
   
      b = '  '
      if oid eq '        ' then oid = ' ---'
      if obj eq '        ' then obj = '--- '
      otype = strpad(strcompress(obj+'/'+oid,/remove),16,/after)
      if not tgiven then begin
         info(i) = fil+b+det+gs+b+obs+b+prg+b+tit+b+otype+b+$
                                           sid+'/'+svar+b+rid+'/'+rv
      endif else begin
         tobs = utc2tai(str2utc(obs))
         if (tobs ge trange(0)) and (tobs le trange(1)) then begin
            info(i) = fil+b+det+gs+b+obs+b+prg+b+tit+b+otype+b+$
                                           sid+'/'+svar+b+rid+'/'+rv
         endif else begin
            info(i) = ''
            ff(i) = ''
         endelse
      endelse
   endfor
   
   
;
;  resort file and info list
;
   n = where(ff ne '',count)
   if count gt 0 then begin
      ff = ff(n)
      info = info(n)
   endif

;
;  if time range was given give the file list now
;
   if tgiven then begin 
      if not keyword_set(quiet) then print_str,ff,/num
      if not keyword_set(quiet) then print,' ' 
   endif

;
;  sort into time order
;
   tinfo = strmid(info,15+strlen(gs),24)
   if n_elements(tinfo) gt 1 then begin
      tinfo = utc2tai(str2utc(tinfo))
      nt = sort(tinfo)
      info = info(nt)
   endif 

;
;  add titles
;
   info = ['  File    Det         Date        Time        Program          '+$
           '       Title              Object (Type/ID)      Study Raster',info]  

;
;  present the header info if needed
; 
   if not keyword_set(quiet) then print_str,info
   if keyword_set(hardcopy) then begin
      print_str,info,/quiet,/hard,/keep,file='FITS_list'
   endif
endif

;
;  return with list
;
return,ff

end

