	PRO RAY_NEIGHBOUR, V, U, ARB_AMOUNT, ANY_DROPOUTS
;+
; Project     : SOHO - CDS
;
; Name        : RAY_NEIGHBOUR
;
; Purpose     : A routine called by COSMIC_RAY_ZAPPER.
;
; Category    : Analysis
;
; Explanation : On given a row vector V creates a vector U such that:
;
;		U(i) = ( 1/2 * (V(i-1) + V(i+1))    i != 0 or n
;	       	       (
;	       	       ( V(i+1)			    i=0
;              	       (
;              	       ( V(i-1)			    i=n
;		
;		where n is the number of elements of V. By doing this one can
;		then do a (V/U - 1) gt cutoff to check for the existence of
;		cosmic rays (see COSMIC_RAY_ZAPPER).
;
;		Obviously there's problems if any element of U is zero, so I
; 		use ARB_AMOUNT to replace any zero values.
;
;		For elements U(0) and U(n) there is only one neighbour and so
;		it is this single value that goes into V.
;
;		Data drop-outs cause problems. I assume they have the value
;		-100 and set them to zero for calculation of V. Allowing for
;		drop-outs slows the routine down, so I have the flag 
;		ANY_DROPOUTS which tells the routine if there are drop-outs 
;		in the data.
;
; Syntax      : RAY_NEIGHBOUR, V, U, ARB_AMOUNT, ANY_DROPOUTS
;
; Inputs      : V	= a vector of dimension greater than 2
;
;		U	= a vector of same dimension as V
;
;		ARB_AMOUNT	= set in COSMIC_RAY_ZAPPER and used to replace
;				  any zero values in U
;
;		ANY_DROPOUTS	= 0 if no drop-outs in the data array
;				  1 if drop-outs in data array
;
; Calls       : None.
;
; Restrictions: V must have dimension greater than 1
;		
; Side effects: 
;
; Prev. Hist. : 
;
; History     : Version 1, 15/9/96 	PRY 
;
; Contact     : Peter Young,  Cambridge University
;-


n=n_elements(v) - 1 & u=v

; -------------------
; The use of lower and upper is a device to generate u
;
if n eq 1 then begin
  lower=[v(1),v(0)]
  upper=lower
endif else begin
  lower=[v(1),v(0:n-2),v(n-1)]
  upper=[v(1),v(2:n),v(n-1)]
endelse
; -------------------

; -------------------
; Special case if there are drop-outs in the data
;
if any_dropouts eq 1 then begin
  ind=where( (lower eq -100) and (upper eq -100) )
  if ind(0) ne -1 then begin
    lower(ind)=0 & upper(ind)=0
  endif

  ind=where( lower eq -100 )
  if ind(0) ne -1 then lower(ind)=upper(ind)

  ind=where( upper eq -100 )
  if ind(0) ne -1 then upper(ind)=lower(ind)
endif
; --------------------

u=(lower + upper)/2

; Any zero values of u will cause problems, so...
;
ind=where(u le 0)
if ind(0) ne -1 then u(ind)=arb_amount

end

