;
;+
; Project     : SOHO - CDS     
;                   
; Name        : RD_SERTS
;               
; Purpose     : Read the SERTS Active Region line list 
;               
; Explanation : The Active Region SERTS line list items are put into
;               a strucure variable which returns all the information
;               in line list.
;               The returned structure contains the following tags:
;
;
;               .list - Table header info
;               .cent - Centroid pixel
;               .wavelength   - Observed wavelength
;               .wavsigm      - Wavelength sigma
;               .intensity    - Observed intensity (erg/cm2/s/sr)
;               .int_sig      - Intensity sigma
;               .FWHM         - FWHM
;               .FWHMs        - FWHM sigma
;               .fit          - 
;               .behr_wav     - Published wavelength - Behr
;               .kelly_wav    - Published wavelength - Kelly
;               .dere_wav     - Published wavelength - Dere
;               .blend        - Blend flag
;               .element      - Element
;               .ion          - Ionization stage
;               .tmax         - Tempertur of formation
;               .ord          - Spectral order
;               .seq          - Sequence
;               .XX           - ID code (see tag .INFO for references) 
;               .IES          - Isoelectronic Sequence
;               .TM           - Transition number from Mewe 
;               .conf         - Configuration (upper and lower level)
;               .terms        - Terms (upper and lower level)
;               .info         - Information block at the end of SERTS table
; 
;               Not that the .conf and .terms are 2-elements arrays
;               containing both lower and upper levels.
;               To extract the configuration of the lower level of
;               line number i: low_lev(i)=sertslist.conf(i,0)
;
; Use         : RD_SERTS_LINES, serts_list
;    
; Inputs      : none
;               
; Opt. Inputs : None.
;               
; Outputs     : Returns a structure with the information in the line list
;               
; Opt. Outputs: None.
;               
; Keywords    : None.
;
; Calls       : CONCAT_DIR()
;
; Common      : None.
;               
; Restrictions: Needs $CDS_ATOMIC/serts_cat89.wave
;               
; Side effects: 
;               
; Category    : CDS QL
;               
; Prev. Hist. : None.
;
; Written     : P. Brekke, 22 January 1996 
;               
; Modified    : Upper/lower configuration levels split into
;               seperate arrays                  P. Brekke, 30 January 1996
;
; Version     : Version 1.1, 30-January-1996
;-            



;
; Returns the roman literal for a number in the range [1,..,39]
;
FUNCTION roman_literal,no
  IF no eq 0 THEN return,'?'
  t = [ 'I', 'II', 'III', 'IV', 'V', 'VI', 'VII', 'VIII',$
	'IX', 'X']
  WHILE no gt 10 DO BEGIN
      t = 'X'+t
      no = no - 10
  EndWHILE
  return,t(no-1)
END


PRO rd_serts,sertslist
   
file = concat_dir('$CDS_ATOMIC','serts_cat89.dat')
OPENR,unit,file,/get_lun


; number of lines in the SERTS table
sl=286
;   
; create structure to hold the entire table contents
;
sertslist = {list:strarr(1),        $ ; Table header info
             cent:fltarr(sl),       $ ; Centroid pixel
             wavelength:fltarr(sl), $ ; Observed wavelength
             wavsigm:fltarr(sl),    $ ; Wavelength sigma
             intensity:fltarr(sl),  $ ; Observed intensity (erg/cm2/s/sr)
             int_sig:fltarr(sl),    $ ; Intensity sigma
             FWHM:fltarr(sl),       $ ; FWHM
             FWHMs:fltarr(sl),      $ ; FWHM sigma
             fit:fltarr(sl),        $ ; 
             behr_wav:fltarr(sl),   $ ; Published wavelength - Behr
             kelly_wav:fltarr(sl),  $ ; Published wavelength - Kelly
             dere_wav:fltarr(sl),   $ ; Published wavelength - Dere
             blend:STRARR(sl),      $ ; Blend flag
             element:STRARR(sl),    $ ; Element
             ion:STRARR(sl),        $ ; Ionization stage
             tmax:fltarr(sl),       $ ; Tempertur of formation
             ord:fltarr(sl),        $ ; Spectral order
             seq:fltarr(sl),        $ ; Sequence
             XX:fltarr(sl),         $ ; ID code (see 
             IES:STRARR(sl),        $ ; Isoel. Seq.
             TM:intarr(sl),         $ ; Transition number
             conf:STRARR(sl,2),     $ ; Configuration
             terms:STRARR(sl,2),    $ ; Terms
             info:STRARR(76)}         ; info block


; Read/skip header information
hed1=' '
text = ' '
READF,unit,hed1
sertslist.list=hed1
READF,unit,hed1
READF,unit,hed1
READF,unit,hed1
READF,unit,hed1
READF,unit,hed1


FOR i = 0, sl-1 DO BEGIN
   
   READF,unit,text   

   sertslist.cent(i) = DOUBLE(STRMID(text,0,7))   
   sertslist.wavelength(i) = DOUBLE(STRMID(text,10,7))
   sertslist.wavsigm(i) = FIX(STRMID(text,17,3))
   sertslist.intensity(i) = DOUBLE(STRMID(text,20,8))
   sertslist.int_sig(i) = DOUBLE(STRMID(text,28,7))
   sertslist.FWHM(i) = FIX(STRMID(text,35,5))
   sertslist.FWHMs(i) = DOUBLE(STRMID(text,40,3))
   sertslist.fit(i) = DOUBLE(STRMID(text,43,3))

   IF STRMID(text,49,1) NE '-' THEN $
      sertslist.behr_wav(i) = $
      FIX(sertslist.wavelength(i)) + DOUBLE(STRMID(text,46,5)) $
      ELSE sertslist.behr_wav(i) = 9.999

   IF STRMID(text,55,1) NE '-' THEN $
      sertslist.kelly_wav(i) = $
      FIX(sertslist.wavelength(i)) + DOUBLE(STRMID(text,51,6)) $
      ELSE sertslist.kelly_wav(i) = 9.999

   IF STRMID(text,60,1) NE '-' THEN $
      sertslist.dere_wav(i) = $
      FIX(sertslist.wavelength(i)) + DOUBLE(STRMID(text,57,5)) $
      ELSE sertslist.dere_wav(i) = 9.999

   sertslist.blend(i) = STRMID(text,62,2)
   sertslist.element(i) = STRMID(text,64,2)

   IF STRMID(text,67,1) NE ' ' THEN $
      sertslist.ion(i) = roman_literal(FIX(STRMID(text,66,2))) $
      ELSE sertslist.ion(i) =  ' '

   IF STRMID(text,71,1) NE ' ' THEN $
      sertslist.tmax(i) = DOUBLE(STRMID(text,68,4)) $
      ELSE  sertslist.tmax(i) = 0.0

   sertslist.ord(i) = FIX(STRMID(text,72,2))
   sertslist.seq(i) = FIX(STRMID(text,74,4))
   sertslist.XX(i) = FIX(STRMID(text,78,2))
   sertslist.IES(i) = STRMID(text,80,4)

   IF STRMID(text,86,1) NE ' ' THEN $
      sertslist.TM(i) = FIX(STRMID(text,84,3)) $
      ELSE sertslist.TM(i) = FIX(-1)
   
   
   IF STRMID(text,91,1) NE '' THEN BEGIN 
      txt = str2arr(STRMID(text,91,16),delim='-')
      sertslist.Conf(i,0) = txt(0)
      sertslist.Conf(i,1) = txt(1)
   END ELSE BEGIN 
      sertslist.Conf(i,*) =  ''
   ENDELSE
   
   IF STRMID(text,108,1) NE '' THEN BEGIN
      txt = str2arr(STRMID(text,108,20),delim='-')
      sertslist.Terms(i,0) = txt(0)
      sertslist.Terms(i,1) = txt(1) 
   END ELSE BEGIN
      sertslist.Terms(i,*) =  ''
   ENDELSE
   
   
END


; Read the info section of the SERTS table
;
text = STRARR(76)
READF,unit,text
sertslist.info = text

close,unit
free_lun,unit
END


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; End of 'rd_serts.pro'.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
