;+
; Project     : SOHO - CDS     
;                   
; Name        : XCDS_COSMIC
;               
; Purpose     : Cosmic ray removal - automatic and manual
;               
; Explanation : This is a utility for cosmic ray removal from CDS NIS data.
;
;               The recommended use is to:
;
;               Read the data file, start xcds_cosmic - this will atomatically
;               debias the data if it's not already done, and make a "working
;               copy" of it as well.
;
;               AUTOMATIC DETECTION
;               
;               Select one of the automatic methods for CR removal, and press
;               the button labeled "Execute" to apply that method to the
;               working copy. This will flag all suspected cosmic ray pixels
;               as MISSING.
;
;               MANUAL VERIFICATION
;               
;               To verify manually that all cosmic rays have been detected,
;               use the Manual inspection, on all the windows ("Inspect all
;               windows") or on single windows ("Inspect current window" - the
;               current window may be selected from the pulldown menu labeled
;               "Window: ...").
;
;               The manual inspection (XCR_REMOVE) allows you to browse back
;               and forth between the exposures of one line window at a time,
;               viewing both the original data and the working copy. By
;               zooming into the image the program switches into a "FLIP" mode
;               (both the Mode status button and the mouse cursor will be
;               changed to reflect this) and you may flip the status
;               (MISSING/not MISSING) of individual pixels.
;
;               Note that this allows un-flagging pixels marked MISSING by the
;               automatic routines.
;
;               SAVING/RETRIEVING A LIST OF FLAGGED PIXELS
;               
;               When you are satisfied with the flagging of cosmic ray pixels,
;               you may save a list of the flagged pixels to a file by
;               pressing the button labeled "Save list of flagged pixels".
;
;               If you have set the environment variable CDS_ANALYSIS to point
;               at at directory, this will be used as the default path to save
;               the list of flagged pixels.
;
;               For later sessions you may simply use the corresponding button
;               "Read list of flagged pixels" to avoid the time-consuming task
;               of manually inspecting all exposures.
;
;               Do NOT fill in the bad pixels (see below) before saving the
;               list of bad pixels (since the identity of the filled-in pixels
;               will be lost when they are filled in). You may, however,
;               save a list of bad pixels, fill them in, flag some more pixels
;               as missing and then re-apply the first list of bad pixels.
;
;               FILLING IN BAD PIXELS
;               
;               With all the cosmic ray pixels flagged as MISSING, you have
;               the option of filling them in with data estimated from the
;               local median around the missing pixels. If you are going to
;               use the data for qualitative work, such as viewing images to
;               identify features etc, it is OK to fill in the cosmic ray
;               pixels. If you are doing line fits to the data, you ought to
;               leave the cosmic ray pixels flagged as MISSING and simply
;               leave those pixels out when doing the fitting.
;
;               Selecting "Fill CR pixels only" will fill in only those pixels
;               identified as cosmic rays *during this session* - pixels
;               flagged as missing before starting XCDS_COSMIC will be left
;               untouched.
;               
;               If you have small data gaps in your data set, you may also
;               want to fill those as well, by selecting "Fill all MISSING
;               pixels". If there are large data gaps, however, this is
;               definitely not recommended.
;
;               FINISHING UP
;
;               When you exit this program, you may choose to discard any
;               changes you have made to the working copy of your data, or you
;               may replace the data with the working copy.
;
; Use         : XCDS_COSMIC,QLDS
;    
; Inputs      : QLDS : Quick look data structure. Data should not be
;                      flatfielded or calibrated, but they may be
;                      debiased. Debiasing will be applied in case this has
;                      not been done.
;               
; Opt. Inputs : None.
;               
; Outputs     : None.
;               
; Opt. Outputs: None.
;               
; Keywords    : WINDOW_IX,
; 
;               EXPOSURE_IX : Window/exposure index - set these to cause
;                             automatic pop-up of XCR_REMOVE for that window
;                             and exposure.
; 
;               FILE_MISS   : Default file for the list of flagged pixels.
;                              
;
; Calls       : break_file, cds_clean, cds_read_missing, copy_qlds(),
;               cw_pselect(), cwq_winsel(), default, delete_qlds,
;               find_with_def(), gt_dimension(), handle_create_r(), parcheck,
;               qlmgr, since_version(), st_windata, trim(), typ(), vds_debias,
;               widget_base(), xack, xmessage
;
; Common      : None.
;               
; Restrictions: Must have NIS unbinned data.
;               
; Side effects: None.
;               
; Category    : ?
;               
; Prev. Hist. : None.
;
; Written     : S. V. H. Haugan, UiO, 6 January 1997
;               
; Modified    : Version 2, SVHH, 22 January 1997
;                       Added /NO_NEIGHBOUR choice for CDS_CLEAN_EXP
;               Version 3, SVHH, 31 January 1997
;                       Added use of CDS_ANALYSIS env. var. when present.
;                       Fixed bug about adding directory to file name.
;               Version 4, SVHH, 15 September 1997
;                       Various small changes.
;               Version 5, V. Andretta (NASA/GSFC), 19 March 1998
;                       Added keyword FILE_MISS; if CDS_ANALYSIS is not set, 
;                       uses current directory. 
;
;               Version 6, Giulio Del Zanna, DAMTP, 1-Jun-2001
;                       Added two extra options to the CDS_CLEAN_EXP 
;                       call, that is the possibility to set the parameter
;                       (keyword) max_factor_hi=2.2 or 2.5. 
;                       By default, max_factor_hi=1.7. The new options are
;                       better when cleaning data with strong brightenings in
;                       the exposures.
;                       ALso, added a button to call DSPEXP to inspect groups of
;                       windows, at the same time. This considerably speeds up
;                       the process of visually inspect a large number of
;                       exposures and NIS extraction windows at the same time.
;                       Once the exposures with problems are found, the standard
;                       widget can then be used to manually correct the cosmic
;                       ray corrections applied to the data.
;
;               Version 7, Peter Young, RAL, 23-Sep-2005
;                       substituted cds_new_spike for cds_clean_spike.
;
;
; Version     : 7, 23-Sep-2005
;-            

PRO xcds_cosmic_help,group=group
  
  txt = $
     ['This is a utility for cosmic ray removal from CDS NIS data.',$
      '         ',$
      'The recommended use is to:',$
      '',$
      'Read the data file, start xcds_cosmic - this will',$
      'atomatically debias the data if it''s not already done, and',$
      'make a "working copy" of it as well.',$
      '',$
      'AUTOMATIC DETECTION',$
      '',$
      'Select one of the automatic methods for CR removal, and',$
      'press the button labeled "Execute" to apply that method to',$
      'the working copy. This will flag all suspected cosmic ray',$
      'pixels as MISSING.',$
      '',$
      'MANUAL VERIFICATION',$
      '',$
      'To verify manually that all cosmic rays have been detected,',$
      'use the Manual inspection, on all the windows ("Inspect all',$
      'windows") or on single windows ("Inspect current window" -',$
      'the current window may be selected from the pulldown menu',$
      'labeled "Window: ...").',$
      '',$
      'The manual inspection (XCR_REMOVE) allows you to browse back',$
      'and forth between the exposures of one line window at a',$
      'time, viewing both the original data and the working',$
      'copy. By zooming into the image the program switches into a',$
      '"FLIP" mode (both the Mode status button and the mouse',$
      'cursor will be changed to reflect this) and you may flip the',$
      'status (MISSING/not MISSING) of individual pixels.',$
      '',$
      'Note that this allows un-flagging pixels marked MISSING by',$
      'the automatic routines.',$
      '',$
      'SAVING/RETRIEVING A LIST OF FLAGGED PIXELS',$
      '',$
      'When you are satisfied with the flagging of cosmic ray',$
      'pixels, you may save a list of the flagged pixels to a file',$
      'by pressing the button labeled "Save list of flagged',$
      'pixels".',$
      '',$
      'If you have set the environment variable CDS_ANALYSIS to point',$
      'at a directory, this will be used as the default path for saving the',$
      'file',$
      '',$
      'For later sessions you may simply use the corresponding',$
      'button "Read list of flagged pixels" to avoid the',$
      'time-consuming task of manually inspecting all exposures.',$
      '',$
      'Do NOT fill in the bad pixels (see below) before saving the',$
      'list of bad pixels (since the identity of the filled-in',$
      'pixels will be lost when they are filled in). You may,',$
      'however, save a list of bad pixels, fill them in, flag some',$
      'more pixels as missing and then re-apply the first list of',$
      'bad pixels.',$
      '',$
      'FILLING IN BAD PIXELS',$
      '',$
      'With all the cosmic ray pixels flagged as MISSING, you have',$
      'the option of filling them in with data estimated from the',$
      'local median around the missing pixels. If you are going to',$
      'use the data for qualitative work, such as viewing images to',$
      'identify features etc, it is OK to fill in the cosmic ray',$
      'pixels. If you are doing line fits to the data, you ought to',$
      'leave the cosmic ray pixels flagged as MISSING and simply',$
      'leave those pixels out when doing the fitting.',$
      '',$
      'Selecting "Fill CR pixels only" will fill in only those',$
      'pixels identified as cosmic rays *during this session* -',$
      'pixels flagged as missing before starting XCDS_COSMIC will',$
      'be left untouched.',$
      '',$
      'If you have small data gaps in your data set, you may also',$
      'want to fill those as well, by selecting "Fill all MISSING',$
      'pixels". If there are large data gaps, however, this is',$
      'definitely not recommended.',$
      '',$
      'FINISHING UP',$
      '',$
      'When you exit this program, you may choose to discard any',$
      'changes you have made to the working copy of your data, or',$
      'you may replace the data with the working copy.']
  
  xtext,txt,/just_reg,group=group

END


PRO xcds_cosmic_inspect,info,orig,curr,ww,exposure_ix
  
  widget_control,info.winsel_id,set_value=ww
  
  catch,error
  IF error NE 0 THEN BEGIN
     catch,/cancel
     print,!err_string
     message,"Caught error",/continue
     IF n_elements(d_o) NE 0 THEN BEGIN
        message,"Saving data",/continue
        st_windata,orig,ww,d_o,/no_copy
     END
     
     IF n_elements(d_c) NE 0 THEN BEGIN
        message,"Saving data",/continue
        st_windata,curr,ww,d_c,/no_copy
     END
     
     message,"Returning",/continue
     return
  END
  
  d_o = gt_windata(orig,ww,/no_copy)
  d_c = gt_windata(curr,ww,/no_copy)
  missing = orig.detdesc(ww).missing
  xcr_remove,d_o,d_c,missing,exposure_ix=exposure_ix
  st_windata,orig,ww,d_o,/no_copy
  st_windata,curr,ww,d_c,/no_copy
END


PRO xcds_cosmic_event,ev
 ; common QLSAVE,QLDS,SAVED

  widget_control,ev.top,get_uvalue=info,/no_copy

;  qlds_checkin, 1, widget=ev.top

  widget_control,ev.id,get_uvalue=uvalue
  
  widget_control,/hourglass
  
  CASE uvalue OF 
     
  'DISCARD':BEGIN
     widget_control,ev.top,/destroy
     return
     ENDCASE
     
  'COMMIT':BEGIN
     handle_value,info.orig_h,orig,/no_copy
     handle_value,info.curr_h,curr,/no_copy
     FOR w = 0,n_elements(orig.detdesc)-1 DO BEGIN
        IF orig.detdesc(w).ixstop(0) GE 0 THEN BEGIN
           da = gt_windata(curr,w,/no_copy)
           st_windata,orig,w,da,/no_copy
        ENDIF
     ENDFOR
     handle_value,info.orig_h,orig,/set,/no_copy
     handle_value,info.curr_h,curr,/set,/no_copy
     widget_control,ev.top,/destroy
     return
     ENDCASE
     
  'HELP':BEGIN
     xcds_cosmic_help,group=ev.top
     ENDCASE
     
  'CDS_CLEAN':BEGIN
     info.method = ''
     ENDCASE

   'CDS_CLEAN_EXP_gdz1':BEGIN
     info.method = '_EXP_gdz1'
     ENDCASE

    'CDS_CLEAN_EXP_gdz2':BEGIN
     info.method = '_EXP_gdz2'
     ENDCASE
    

  'CDS_CLEAN_EXP':BEGIN
     info.method = '_EXP'
     ENDCASE

  'CDS_CLEAN_EXP-N':BEGIN
     info.method = '_EXP-N'
     ENDCASE
     
  'CDS_NEW_SPIKE':BEGIN
     info.method = '_SPIKE'
     ENDCASE
     
  'EXECUTE':BEGIN
     handle_value,info.curr_h,curr,/no_copy
     CASE info.method OF 
        "_EXP":BEGIN 
     xmessage,["Executing automatic CR detection, CDS_CLEAN_EXP,/NOFILL please wait"],wbase=wb
   cds_clean_exp,curr,/nofill
END 
        "_EXP_gdz1": BEGIN 
     xmessage,["Executing automatic CR detection with max_factor_hi=2.2 , please wait"],wbase=wb
  cds_clean_exp,curr,/nofill,max_factor_hi=2.2
END 

        "_EXP_gdz2": BEGIN 
     xmessage,["Executing automatic CR detection with max_factor_hi=2.5 , please wait"],wbase=wb
  cds_clean_exp,curr,/nofill,max_factor_hi=2.5
END 

        "_EXP-N": BEGIN 
     xmessage,["Executing automatic CR detection, please wait"],wbase=wb
cds_clean_exp,curr,/nofill,/no_neighbour
END 
        "_SPIKE": BEGIN
     xmessage,["Executing automatic CR detection, please wait"],wbase=wb
; cds_clean_spike,curr,/nofill
 cds_new_spike,curr,/nofill
END 
        ELSE: BEGIN 
     xmessage,["Executing  CR detection with CDS_CLEAN,/NOFILL, please wait"],wbase=wb
cds_clean,curr,/nofill
END 
     END
     handle_value,info.curr_h,curr,/set,/no_copy
     widget_control,wb,/destroy
     ENDCASE
     
  'WINDOW':BEGIN
     info.windowi = ev.windowi
     ENDCASE
     
  'INSP_ALL':BEGIN
     widget_control,ev.top,sensitive=0
     handle_value,info.orig_h,orig,/no_copy
     handle_value,info.curr_h,curr,/no_copy
     
     validwin = where(orig.detdesc(*).ixstop(0) GE 0,count)
     
     FOR w = 0,count-1 DO BEGIN
        xcds_cosmic_inspect,info,orig,curr,validwin(w)
     ENDFOR
     
     handle_value,info.orig_h,orig,/no_copy,/set
     handle_value,info.curr_h,curr,/no_copy,/set
     widget_control,info.winsel_id,set_value=info.windowi
     widget_control,ev.top,sensitive=1
     ENDCASE
     
  'INSP_CUR':BEGIN
     widget_control,ev.top,sensitive=0
     handle_value,info.orig_h,orig,/no_copy
     handle_value,info.curr_h,curr,/no_copy
     
     ww = info.windowi
     xcds_cosmic_inspect,info,orig,curr,ww
     
     handle_value,info.orig_h,orig,/no_copy,/set
     handle_value,info.curr_h,curr,/no_copy,/set
     widget_control,ev.top,sensitive=1
     ENDCASE


     'INSP_WIN':BEGIN
    
     widget_control,ev.top,sensitive=0
     handle_value,info.orig_h,orig,/no_copy
     handle_value,info.curr_h, curr,/no_copy

;    cur = copy_qlds(curr)
     dspexp, curr ,group_leader=ev.top ; ,/nocheck
;    delete_qlds, cur

     handle_value,info.orig_h,orig,/no_copy,/set
     handle_value,info.curr_h,curr,/no_copy,/set
   widget_control,ev.top,sensitive=1
 
     ENDCASE

  'SAVE':BEGIN
     handle_value,info.curr_h,curr,/no_copy
     IF info.file_miss NE '' THEN BEGIN
       break_file,info.file_miss      ,disk,dir,name,ext,fv,nod
       ipath = expand_path(disk+dir)
       fname = name+ext
       if ext ne '' then filtext='*'+ext else filtext='*.miss'
     ENDIF ELSE BEGIN
       break_file,curr.header.filename,disk,dir,name,ext,fv,nod
       ipath = expand_path(getenv("CDS_ANALYSIS"))
       fname = name+'.miss'
       filtext='*.miss'
     ENDELSE
     IF ipath NE '' THEN $
       file = pickfile(file=fname,/write,get_path=path,path=ipath,$
                       filter=filtext) $
     ELSE $
       file = pickfile(file=fname,/write,get_path=path,$
                       filter=filtext)
     IF file NE '' THEN BEGIN
        IF strpos(file,path) EQ -1 THEN file = concat_dir(path,file)
        IF test_open(file,/write) AND NOT chk_dir(file) THEN $
           cds_save_missing,curr,file $
        ELSE xack,'Could not write to '+file
     END
     handle_value,info.curr_h,curr,/set,/no_copy
     ENDCASE
     
  'READ':BEGIN
     handle_value,info.curr_h,curr,/no_copy
     IF info.file_miss NE '' THEN BEGIN
       break_file,info.file_miss      ,disk,dir,name,ext,fv,nod
       ipath = expand_path(disk+dir)
       fname = name+ext
       if ext ne '' then filtext='*'+ext else filtext='*.miss'
     ENDIF ELSE BEGIN
       break_file,curr.header.filename,disk,dir,name,ext,fv,nod
       ipath = expand_path(getenv("CDS_ANALYSIS"))
       fname = name+'.miss'
       filtext='*.miss'
     ENDELSE
     IF ipath NE '' THEN $
       file = pickfile(file=fname,/read,get_path=path,path=ipath,$
                       filter=filtext,$
                       /must_exist) $
     ELSE $
       file = pickfile(file=fname,/read,get_path=path,$
                       filter=filtext,$
                       /must_exist)
     IF file NE '' THEN BEGIN
        IF strpos(file,path) EQ -1 THEN file = concat_dir(path,file)
        IF test_open(file) AND NOT chk_dir(file) THEN  $
           cds_read_missing,curr,file $
        ELSE xack,'Could not open file: '+file
     END
     handle_value,info.curr_h,curr,/set,/no_copy
     ENDCASE
     
  'FILL_CR':BEGIN
     handle_value,info.orig_h,orig,/no_copy
     handle_value,info.curr_h,curr,/no_copy
     
     cds_fill_missing,curr
     cds_save_missing,orig,status=status
     cds_read_missing,curr,status=status
     
     handle_value,info.orig_h,orig,/no_copy,/set
     handle_value,info.curr_h,curr,/no_copy,/set
     ENDCASE
     
  'FILL':BEGIN
     handle_value,info.curr_h,curr,/no_copy
     
     cds_fill_missing,curr
     
     handle_value,info.curr_h,curr,/no_copy,/set
     ENDCASE
     
  END
  
  
  widget_control,ev.top,set_uvalue=info,/no_copy
END


PRO xcds_cosmic,qlds,exposure_ix=exposure_ix,window_ix=window_ix $
               ,file_miss=file_miss
;COMMON QLSAVE, ql_curr_h, saved
  
  on_error,2
  IF !debug NE 0 THEN on_error,0
  
  IF n_params() NE 1 THEN $
     message,"Use: XCDS_COSMIC,QLDS"
  
  default,window_ix,-1
  default,exposure_ix,-1
  default,file_miss,''
  
  qlmgr,qlds,valid
  
  IF NOT valid THEN message,"Parameter is not a valid QLDS"
  
  ;; What windows are valid ?
  
  validwin = where(qlds.detdesc(*).ixstop(0) GE 0)
  IF validwin(0) EQ -1 THEN message,"No data in the supplied QLDS"
  
  maxwin = max(validwin)
  
  ;; Was a window_ix/exposure_ix pair given, and are they valid?
  
  parcheck,window_ix,0,typ(/nat),0,'WINDOW',minval=-1,maxval=maxwin
  IF window_ix NE -1 THEN $
     IF qlds.detdesc(window_ix).ixstop(0) LT 0 THEN $
     message,"Window "+trim(window_ix)+" is not read in"
  
  dim = gt_dimension(qlds)
  nexp = (dim.ssolar_x > 1) * (dim.sdel_time > 1)
  parcheck,exposure_ix,0,typ(/nat),0,'EXPOSURE_IX',minval=-1,maxval=nexp
  
  IF qlds.detdesc(validwin(0)).units EQ 'ADC' THEN BEGIN
     message,"Debiasing",/continue
     vds_debias,qlds
  END
  
  IF qlds.detdesc(validwin(0)).units NE 'DEBIASED-ADC' THEN BEGIN
     xack,["Data must not be flatfielded/calibrated before",$
           "calling XCDS_COSMIC. Returning"]
     return
  END
  
  ;;
  ;; Parameter checking finished - build the widget
  ;;
  base = widget_base(/column,title='XCDS_COSMIC')
  
  lab = widget_label(base,value='File: '+qlds.header.filename)
  
  ;;
  ;; Exit menu
  ;;
  ex_menu = widget_button(base,value='Exit',menu=2)
  dummy = widget_button(ex_menu,value='Commit changes',uvalue='COMMIT')
  dummy = widget_button(ex_menu,value='Discard changes',uvalue='DISCARD')
  
  ;;
  ;; Help button
  ;;
  
  dummy = widget_button(base,value='Help',uvalue='HELP')
  
  
  IF since_version('4.0.1') THEN bleft = {base_align_left:1}
  
  ;;
  ;; Automatic detection section
  ;;
  
  auto_base = widget_base(base,/column,/frame,_extra=bleft)
  lab = widget_label(auto_base,value='Automatic detection method')
  auto_base0 = widget_base(auto_base,/row)
  auto_base1 = widget_base(auto_base0,/column,_extra=bleft)
  auto_base2 = widget_base(auto_base0,/column,_extra=bleft)
  
  dummy = {pselect_s,btext:'',mtext:'',uvalue:'',flags:0}
  cdc = 'CDS_CLEAN'
  auto_menu = [{pselect_s,cdc,cdc,cdc,0},$
               {pselect_s,cdc+'_EXP',cdc+'_EXP',cdc+'_EXP',0},$
               {pselect_s,cdc+'_EXP (*)',cdc+'_EXP (max_factor_hi=2.2 )',cdc+'_EXP_gdz1',0},$
               {pselect_s,cdc+'_EXP (*)',cdc+'_EXP (max_factor_hi=2.5 )',cdc+'_EXP_gdz2',0},$
               {pselect_s,cdc+'_EXP (*)',cdc+'_EXP (/NO_NEIGHBOUR)', cdc+'_EXP-N',0},$
               {pselect_s,'CDS_NEW_SPIKE','CDS_NEW_SPIKE','CDS_NEW_SPIKE',0}]
;               {pselect_s,cdc+'_SPIKE',cdc+'_SPIKE',cdc+'_SPIKE',0}]
  auto_menu_id = cw_pselect(auto_base2,'Method: ',auto_menu)

  widget_control,auto_menu_id,set_value=cdc+'_EXP'

  ;; Remove this when cds_new_spike has been written
  ;; 
  IF find_with_def('cds_new_spike',!path,'.pro') EQ '' THEN $
     widget_control,auto_menu_id,set_value={insensitive:'CDS_NEW_SPIKE'}
  
  dummy = widget_button(auto_base2,value='Execute',uvalue='EXECUTE')
  
  ;;
  ;; Manual verification section
  ;;
  man_base = widget_base(base,/column,/frame,_extra=bleft)
  lab = widget_label(man_base,value='Manual inspection')
  man_base0 = widget_base(man_base,/row)
  man_base1 = widget_base(man_base0,/column,_extra=bleft)
  man_base2 = widget_base(man_base0,/column,_extra=bleft)
  
  dummy = widget_button(man_base2,value='Inspect all windows',$
                        uvalue='INSP_ALL')
  
  ;; Individual window inspection
  winsel_id = cwq_winsel(man_base2,qlds,title='Window: ',uvalue='WINDOW')
  dummy = widget_button(man_base2,value='Inspect current window',$
                        uvalue='INSP_CUR')

;gdz
  dummy = widget_button(man_base2,value='View *only* -- groups of windows ',$
                       uvalue='INSP_WIN')


  ;;
  ;; "File menu" - cds_save/restore_missing, fill, etc.
  ;;
  f_base = widget_base(base,/column,/frame,_extra=bleft)
  lab = widget_label(f_base,value='Other actions')
  f_base0 = widget_base(f_base,/row)
  f_base1 = widget_base(f_base0,/column,_extra=bleft)
  f_base2 = widget_base(f_base0,/column,_extra=bleft)
  
  dummy = widget_button(f_base2,value='Save list of flagged pixels',$
                        uvalue='SAVE')
  dummy = widget_button(f_base2,value='Read list of flagged pixels',$
                        uvalue='READ')
  
  dummy = widget_button(f_base2,value='Fill CR pixels only',uvalue='FILL_CR')
  dummy = widget_button(f_base2,value='Fill all MISSING pixels',uvalue='FILL')

  ;;
  ;; Realize the widget
  ;;
  widget_control,base,/realize
  
  ;;
  ;; Handles to store the original and "current" qlds
  ;; 
  ql_curr_h = handle_create()
  ql_orig_h = handle_create()
  
  info = { curr_h : ql_curr_h,$
           orig_h : ql_orig_h,$
           method : "_EXP",$
           winsel_id : winsel_id,$
           windowi : window_ix,$
           file_miss: file_miss}
  ;;
  ;; Make working copy
  ;; 
  ql_curr = copy_qlds(qlds)
  
  ;;
  ;; If (window_ix,exposure_ix) has been passed, pop up xcr_remove first
  ;; 
  
  IF window_ix NE -1 AND exposure_ix NE -1 THEN BEGIN
     widget_control,base,sensitive=0
     xcds_cosmic_inspect,info,qlds,ql_curr,window_ix,exposure_ix
     widget_control,base,sensitive=1
  END

  IF window_ix EQ -1 THEN window_ix = validwin(0)
  
  info.windowi = window_ix
  widget_control,winsel_id,set_value=window_ix
  
  handle_value,ql_curr_h,ql_curr,/set,/no_copy
  handle_value,ql_orig_h,qlds,/set
  
  widget_control,base,set_uvalue=info
  
  xmanager,"xcds_cosmic",base,/modal
  
  ;; The exit procedure should have acted on the user's exit
  ;; choice to switch the values of the two handles if necessary
  
  handle_value,ql_curr_h,ql_curr,/no_copy
  handle_value,ql_orig_h,qlds,/no_copy
  print,"Deleting qlds ",ql_curr.ql_no
  
  delete_qlds,ql_curr
  
  handle_free,ql_curr_h
  handle_free,ql_orig_h
  
END

