	FUNCTION ADD_ALT, DEF, ERRMSG=ERRMSG
;+
; Project     :	SOHO - CDS
;
; Name        :	ADD_ALT()
;
; Purpose     :	Adds a CDS alternate science plan record to the database
;
; Explanation :	This procedure takes a CDS alternate science plan entry and
;		adds it to the database "sci_alt".  This database contains a
;		series of such entries, making a historical list.
;
; Use         :	Result = ADD_ALT( DEF )
;
;		IF NOT ADD_ALT( DEF ) THEN ...
;
; Inputs      :	DEF = This is an anonymous structure containing the following
;		      tags:
;
;			PROG_ID	   = Program ID, linking one or more studies
;				     together
;			STUDY_ID   = Number defining the study
;			STUDYVAR   = Number giving the study variation ID
;			SCI_OBJ    = Science objective from the daily science
;				     meeting
;			SCI_SPEC   = Specific science objective from meeting
;			CMP_NO	   = Campaign number
;			OBJECT	   = Code for object planned to be observed
;			OBJ_ID	   = Object identification
;			START_TIME = Date/time of beginning of alternate plan,
;				     in TAI format
;			END_TIME   = Date/time of end of alternate plan, in
;				     TAI format
;			ORIG_DUR   = Original duration of the observation, in
;				     seconds, before being truncated by
;				     following time-tagged observations.
;			N_RASTERS1 = Variable number of rasters parameter
;			TRACKING   = True (1) if feature tracking to be used,
;				     or false (0) otherwise
;			N_POINTINGS= Number of pointings to use with study
;			N_REPEAT_S = Number of times to repeat study
;			GSET_ID	   = GSET ID number
;			GET_RAW	   = True if raw data should be collected.
;
;		      If N_POINTINGS is greater than 0, then there will be the
;		      additional tag POINTINGS.  This is an array of pointing
;		      positions.  It itself is a structure, of type
;		      PLAN_PNT_STRUC, with the following tags:
;
;			INS_X	   = Pointing in X relative to sun center
;			INS_Y	   = Pointing in Y relative to sun center
;			ZONE_ID	   = Pointing zone ID
;
;		      DEF is also allowed to be an array of such structures.
;		      In such a case, the substructure POINTINGS will be
;		      dimensioned based on the largest value of N_POINTINGS.
;
; Opt. Inputs :	None.
;
; Outputs     :	The result of the function is a logical value representing
;		whether or not the operation was successful, where 1 is
;		successful and 0 is unsuccessful.
;
; Opt. Outputs:	None.
;
; Keywords    :	
;       ERRMSG    = If defined and passed, then any error messages will be
;                   returned to the user in this parameter rather than
;                   depending on the MESSAGE routine in IDL.  If no errors are
;                   encountered, then a null string is returned.  In order to
;                   use this feature, ERRMSG must be defined first, e.g.
;
;                       ERRMSG = ''
;                       Result = ADD_ALT( ERRMSG=ERRMSG, ... )
;                       IF ERRMSG NE '' THEN ...
;
;
; Calls       :	DATATYPE, DBOPEN, DBBUILD, DBCLOSE, TRIM, GET_OBJECT,
;		GET_CAMPAIGN, GET_PROGRAM, GET_STUDY
;
; Common      :	None.
;
; Restrictions:	Only this routine or DEL_ALT can be used to add or delete
;		alternate plan descriptions to or from the database.  Modifying
;		the database by hand could corrupt its integrity.
;
;		The data types and sizes of the structure elements must match
;		the definitions in the database.
;
;		!PRIV must be 2 or greater to use this routine.
;
; Side effects:	The dates in the structure are rounded off to millisecond
;		accuracy.
;
; Category    :	Planning, Databases.
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 9 May 1995
;
; Modified    :	Version 1, William Thompson, GSFC, 9 May 1995
;		Version 2, William Thompson, GSFC, 29 June 1995
;			Changed so that times in structure are rounded off to
;			millisecond accuracy.
;		Version 3, William Thompson, GSFC, 31 August 1995
;			Allow DEF to be an array
;		Version 4, William Thompson, GSFC, 28 September 1995
;			Added tags ORIG_DUR, GSET_ID, GET_RAW
;		Version 5, William Thompson, GSFC, 11 December 1995
;			Check validity of GSET_ID when using GIS.
;
; Version     :	Version 5, 11 December 1995
;-
;
	ON_ERROR, 2
;
;  Initialize RESULT to represent non-success.  If the routine is successful,
;  this value will be updated below.
;
	RESULT = 0
;
;  Check the input parameters
;
        IF N_PARAMS() NE 1 THEN BEGIN
           MESSAGE = 'Syntax:  Result = ADD_ALT(DEF)'
	   GOTO, HANDLE_ERROR
        ENDIF
;
;  Make sure that the user has privilege to write into the database.
;
	IF !PRIV LT 2 THEN BEGIN $
           MESSAGE = '!PRIV must be 2 or greater to write into the database'
           GOTO, HANDLE_ERROR
	ENDIF
;
;  Check each of the structure components to verify that it is of the correct
;  type and size.
;
	DEF0 = DEF(0)
;
	IF DATATYPE(DEF0.PROG_ID,1) NE 'Integer' THEN BEGIN
           MESSAGE = 'Tag PROG_ID must be a short integer'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(DEF0.PROG_ID) NE 1 THEN BEGIN
           MESSAGE = 'Tag PROG_ID must be a scalar'
           GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(DEF0.STUDY_ID,1) NE 'Integer' THEN BEGIN
           MESSAGE = 'Tag STUDY_ID must be a short integer'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(DEF0.STUDY_ID) NE 1 THEN BEGIN
           MESSAGE = 'Tag STUDY_ID must be a scalar'
           GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(DEF0.STUDYVAR,1) NE 'Integer' THEN BEGIN
           MESSAGE = 'Tag STUDYVAR must be a short integer'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(DEF0.STUDYVAR) NE 1 THEN BEGIN
           MESSAGE = 'Tag STUDYVAR must be a scalar'
           GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(DEF0.SCI_OBJ,1) NE 'String' THEN BEGIN
           MESSAGE = 'Tag SCI_OBJ must be a character string'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(DEF0.SCI_OBJ) NE 1 THEN BEGIN
           MESSAGE = 'Tag SCI_OBJ must be a scalar'
           GOTO, HANDLE_ERROR
	END ELSE IF MAX(STRLEN(DEF.SCI_OBJ)) GT 50 THEN BEGIN $
           MESSAGE = 'Tag SCI_OBJ must be 50 characters or less'
           GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(DEF0.SCI_SPEC,1) NE 'String' THEN BEGIN
           MESSAGE = 'Tag SCI_SPEC must be a character string'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(DEF0.SCI_SPEC) NE 1 THEN BEGIN
           MESSAGE = 'Tag SCI_SPEC must be a scalar'
           GOTO, HANDLE_ERROR
	END ELSE IF MAX(STRLEN(DEF.SCI_SPEC)) GT 50 THEN BEGIN
           MESSAGE = 'Tag SCI_SPEC must be 50 characters or less'
           GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(DEF0.CMP_NO,1) NE 'Integer' THEN BEGIN
           MESSAGE = 'Tag CMP_NO must be a short integer'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(DEF0.CMP_NO) NE 1 THEN BEGIN
           MESSAGE = 'Tag CMP_NO must be a scalar'
           GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(DEF0.OBJECT,1) NE 'String' THEN BEGIN
           MESSAGE = 'Tag OBJECT must be a character string'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(DEF0.OBJECT) NE 1 THEN BEGIN
           MESSAGE = 'Tag OBJECT must be a scalar'
           GOTO, HANDLE_ERROR
	END ELSE IF MAX(STRLEN(DEF.OBJECT)) GT 3 THEN BEGIN
           MESSAGE = 'Tag OBJECT must be 3 characters or less'
           GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(DEF0.OBJ_ID,1) NE 'String' THEN BEGIN
           MESSAGE = 'Tag OBJ_ID must be a character string'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(DEF0.OBJ_ID) NE 1 THEN BEGIN
           MESSAGE = 'Tag OBJ_ID must be a scalar'
           GOTO, HANDLE_ERROR
	END ELSE IF MAX(STRLEN(DEF.OBJ_ID)) GT 6 THEN BEGIN
           MESSAGE = 'Tag OBJ_ID must be 6 characters or less'
           GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(DEF0.START_TIME,1) NE 'Double' THEN BEGIN
           MESSAGE = 'Tag START_TIME must be a double precision number'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(DEF0.START_TIME) NE 1 THEN BEGIN
           MESSAGE = 'Tag START_TIME must be a scalar'
           GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(DEF0.END_TIME,1) NE 'Double' THEN BEGIN
           MESSAGE = 'Tag END_TIME must be a double precision number'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(DEF0.END_TIME) NE 1 THEN BEGIN
           MESSAGE = 'Tag END_TIME must be a scalar'
           GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(DEF0.ORIG_DUR,1) NE 'Double' THEN BEGIN
           MESSAGE = 'Tag ORIG_DUR must be a double precision number'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(DEF0.ORIG_DUR) NE 1 THEN BEGIN
           MESSAGE = 'Tag ORIG_DUR must be a scalar'
           GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(DEF0.N_RASTERS1,1) NE 'Integer' THEN BEGIN
           MESSAGE = 'Tag N_RASTERS1 must be a short integer'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(DEF0.N_RASTERS1) NE 1 THEN BEGIN
           MESSAGE = 'Tag N_RASTERS1 must be a scalar'
           GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(DEF0.TRACKING,1) NE 'Byte' THEN BEGIN
           MESSAGE = 'Tag TRACKING must be a byte value'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(DEF0.TRACKING) NE 1 THEN BEGIN
           MESSAGE = 'Tag TRACKING must be a scalar'
           GOTO, HANDLE_ERROR
	END ELSE IF MAX(DEF.TRACKING) GT 1 THEN BEGIN
           MESSAGE = 'Tag TRACKING must be either 0 or 1'
           GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(DEF0.N_POINTINGS,1) NE 'Integer' THEN BEGIN
           MESSAGE = 'Tag N_POINTINGS must be a short integer value'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(DEF0.N_POINTINGS) NE 1 THEN BEGIN
           MESSAGE = 'Tag N_POINTINGS must be a scalar'
           GOTO, HANDLE_ERROR
	END ELSE IF MIN(DEF.N_POINTINGS) LT 0 THEN BEGIN
           MESSAGE = 'Tag N_POINTINGS must not be negative'
           GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(DEF0.N_REPEAT_S,1) NE 'Integer' THEN BEGIN
           MESSAGE = 'Tag N_REPEAT_S must be a short integer value'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(DEF0.N_REPEAT_S) NE 1 THEN BEGIN
           MESSAGE = 'Tag N_REPEAT_S must be a scalar'
           GOTO, HANDLE_ERROR
	END ELSE IF MIN(DEF.N_REPEAT_S) LE 0 THEN BEGIN
           MESSAGE = 'Tag N_REPEAT_S must be positive'
           GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(DEF0.GSET_ID,1) NE 'Integer' THEN BEGIN
           MESSAGE = 'Tag GSET_ID must be a short integer'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(DEF0.GSET_ID) NE 1 THEN BEGIN
           MESSAGE = 'Tag GSET_ID must be a scalar'
           GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(DEF0.GET_RAW,1) NE 'Byte' THEN BEGIN
           MESSAGE = 'Tag GET_RAW must be a byte value'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(DEF0.GET_RAW) NE 1 THEN BEGIN
           MESSAGE = 'Tag GET_RAW must be a scalar'
           GOTO, HANDLE_ERROR
	END ELSE IF MAX(DEF.GET_RAW) GT 1 THEN BEGIN
           MESSAGE = 'Tag GET_RAW must be either 0 or 1'
           GOTO, HANDLE_ERROR
	ENDIF
;
;  Extract the first instance of the structure in the POINTINGS tag, and check
;  it's tags.
;
	TEST = (DEF0.POINTINGS)(0)
;
	IF DATATYPE(TEST.INS_X,1) NE 'Float' THEN BEGIN
           MESSAGE = 'Tag INS_X must be a floating point number'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(TEST.INS_X) NE 1 THEN BEGIN
           MESSAGE = 'Tag INS_X must be a scalar'
           GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(TEST.INS_Y,1) NE 'Float' THEN BEGIN
           MESSAGE = 'Tag INS_Y must be a floating point number'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(TEST.INS_Y) NE 1 THEN BEGIN
           MESSAGE = 'Tag INS_Y must be a scalar'
           GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(TEST.ZONE_ID,1) NE 'Integer' THEN BEGIN
           MESSAGE = 'Tag ZONE_ID must be a short integer'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(TEST.ZONE_ID) NE 1 THEN BEGIN
           MESSAGE = 'Tag ZONE_ID must be a scalar'
           GOTO, HANDLE_ERROR
	ENDIF
;
;  The number of pointings supplied must match the number specified.
;
	IF MAX(DEF.N_POINTINGS) GT N_ELEMENTS(DEF0.POINTINGS) THEN BEGIN
		MESSAGE = 'Number of pointings does not match array size'
		GOTO, HANDLE_ERROR
	ENDIF
;
;  Make sure that the program ID matches an entry in the CDS program database.
;
	FOR I = 0, N_ELEMENTS(DEF)-1 DO BEGIN
	    IF DEF(I).PROG_ID NE 0 THEN BEGIN
		GET_PROGRAM, DEF(I).PROG_ID, TEMP
		IF TEMP.PROG_ID LE 0 THEN BEGIN
		    MESSAGE = 'Program ID ' + STRTRIM(DEF(I).PROG_ID,2) + $
			    ' not found in database' 
		    GOTO, HANDLE_ERROR
		ENDIF
	    ENDIF
;
;  Make sure that the campaign number matches an entry in the SoHO campaign
;  database.
;
	    IF DEF(I).CMP_NO NE 0 THEN BEGIN
		GET_CAMPAIGN, DEF(I).CMP_NO, TEMP
		IF TEMP.CMP_NO LE 0 THEN BEGIN 
                   MESSAGE = 'Campaign number ' + STRTRIM(DEF(I).CMP_NO,2) + $
                      ' not found in database'                   
                   GOTO, HANDLE_ERROR
		ENDIF
	    ENDIF
;
;  Make sure that the object code matches an entry in the database.
;
	    GET_OBJECT, DEF(I).OBJECT, TEMP
	    IF TEMP.OBJECT EQ '' THEN BEGIN
		MESSAGE = 'Object code ' + DEF(I).OBJECT + $
			' not found in database'
		GOTO, HANDLE_ERROR
	    ENDIF
	    IF I EQ 0 THEN OBJ = REPLICATE(TEMP,N_ELEMENTS(DEF)) ELSE	$
		OBJ(I) = TEMP
;
;  Make sure that the study ID and study variation index match an entry in the
;  CDS study database.
;
	    GET_STUDY, DEF(I).STUDY_ID, DEF(I).STUDYVAR, TEMP
	    IF TEMP.STUDY_ID LE -1 THEN BEGIN
		MESSAGE = 'Study ID/variation ' + TRIM(DEF(I).STUDY_ID) + $
			',' + TRIM(DEF(I).STUDYVAR) + ' not found in database'
		GOTO, HANDLE_ERROR
	    ENDIF
;
;  If any of the rasters use the GIS detector, then make sure that GSET_ID is
;  valid.
;
	    IF TEMP.N_RASTER_DEF GT 0 THEN BEGIN
		USE_GIS = 0
		FOR IR = 0, N_ELEMENTS(TEMP.RASTERS)-1 DO BEGIN
		    GET_F_RASTER, TEMP.RASTERS(IR).RAS_ID, TEMP_R
		    USE_GIS = USE_GIS OR (TEMP_R.DETECTOR EQ 'G')
		ENDFOR
		IF USE_GIS THEN BEGIN
		    GET_GSET, DEF(I).GSET_ID, TEMP_G
		    IF TEMP_G.GSET_ID LE 0 THEN BEGIN
			MESSAGE = 'GSET ID ' + TRIM(DEF(I).GSET_ID) +	$
				' not found in database'
			GOTO, HANDLE_ERROR
		    ENDIF
		ENDIF
	    ENDIF
;
;  Make sure that the stop date is greater than the start date.
;
	    IF DEF(I).END_TIME LE DEF(I).START_TIME THEN BEGIN
		MESSAGE = 'The stop date must be after the start date'
		GOTO, HANDLE_ERROR
	    ENDIF
	ENDFOR
;
;  Reformat the times to millisecond accuracy.  This is necessary so that
;  the times are written out in a controlled way.
;
	DEF.START_TIME = DOUBLE( STRING( DEF.START_TIME, FORMAT='(F15.3)' ))
	DEF.END_TIME   = DOUBLE( STRING( DEF.END_TIME,   FORMAT='(F15.3)' ))
;
;  Make sure that the proposed entries don't overlap with each other.
;
	FOR I = 0, N_ELEMENTS(DEF)-1 DO BEGIN
	    W = WHERE((DEF(I).START_TIME LT DEF.END_TIME) AND	$
		    (DEF(I).END_TIME GT DEF.START_TIME), COUNT)
	    IF COUNT GT 1 THEN BEGIN
		MESSAGE = 'Proposed entries at time ' + $
			TAI2UTC(DEF(I).START_TIME,/ECS,/TR) + ' to ' +	$
			TAI2UTC(DEF(I).END_TIME,/ECS,/TR) + ' and ' +	$
			TAI2UTC(DEF(W(1)).START_TIME,/ECS,/TR) + ' to ' + $
			TAI2UTC(DEF(W(1)).END_TIME,/ECS,/TR) +		$
			' overlap each other.'
		GOTO, HANDLE_ERROR
	    ENDIF
	ENDFOR
;
;  Open the alternate plan database for write access.
;
	DBOPEN, 'sci_alt', 1
;
;  Make sure that the proposed entries do not overlap any of the existing
;  entries.
;
	MIN_START_TIME = MIN(DEF(W).START_TIME)
	MAX_END_TIME   = MAX(DEF(W).END_TIME)
	ENTRIES = DBFIND('END_TIME>' + TRIM(MIN_START_TIME,'(F15.3)') + $
		',START_TIME<' + TRIM(MAX_END_TIME,'(F15.3)') +	$
		',DELETED=N', /SILENT)
	IF !ERR EQ 0 THEN BEGIN
		N_FOUND = 0
	END ELSE BEGIN
		ENTRIES = ENTRIES(UNIQ(ENTRIES))
		N_FOUND = N_ELEMENTS(ENTRIES)
		DBEXT, ENTRIES, 'start_time,end_time', START_TIME, END_TIME
	ENDELSE
;
;  It's okay if the entries in the database only touches the time range of the
;  new data.  Start by sorting the entries by the start time, so that one only
;  has to check the first and last entry.
;
	IF N_FOUND GT 0 THEN BEGIN
		IF N_FOUND GT 1 THEN BEGIN
			S = SORT(START_TIME)
			ENTRIES    = ENTRIES(S)
			START_TIME = START_TIME(S)
			END_TIME   = END_TIME(S)
		ENDIF
		IF START_TIME(N_FOUND-1) EQ MAX_END_TIME THEN	$
			N_FOUND = N_FOUND - 1
		IF END_TIME(0) EQ MIN_START_TIME THEN N_FOUND = N_FOUND - 1
		IF N_FOUND GT 0 THEN BEGIN
		    MESSAGE = 'Definition would overlap one in the database'
		    GOTO, HANDLE_ERROR
		ENDIF
	ENDIF
;
;  Add the entries.
;
	DBBUILD, DEF.PROG_ID, DEF.STUDY_ID, DEF.STUDYVAR, DEF.SCI_OBJ,	$
		DEF.SCI_SPEC, DEF.CMP_NO, OBJ.OBJECT, DEF.OBJ_ID,	$
		DEF.START_TIME, DEF.END_TIME, DEF.ORIG_DUR, DEF.N_RASTERS1, $
		DEF.TRACKING, DEF.N_POINTINGS, DEF.N_REPEAT_S, DEF.GSET_ID, $
		DEF.GET_RAW, REPLICATE('N', N_ELEMENTS(DEF)), STATUS=STATUS
	IF STATUS EQ 0 THEN BEGIN
           MESSAGE = 'Write to alternate plan database was not successful'
           GOTO, HANDLE_ERROR
	ENDIF
;
;  Write the associated pointings.  First, collect them into arrays.
;
	N_POINTINGS = 0
	FOR I = 0, N_ELEMENTS(DEF)-1 DO BEGIN
	    N = DEF(I).N_POINTINGS
	    IF N GT 0 THEN BEGIN
		IF N_POINTINGS EQ 0 THEN BEGIN
		    START_TIME = REPLICATE(DEF(I).START_TIME, N)
		    POINT_IND  = INDGEN(N)
		    POINTINGS  = DEF(I).POINTINGS(0:N-1)
		END ELSE BEGIN
		    START_TIME = [START_TIME, REPLICATE(DEF(I).START_TIME, N)]
		    POINT_IND  = [POINT_IND,  INDGEN(N)]
		    POINTINGS  = [POINTINGS,  DEF(I).POINTINGS(0:N-1)]
		ENDELSE
		N_POINTINGS = N_POINTINGS + N
	    ENDIF
	ENDFOR
;
	IF N_POINTINGS GT 0 THEN BEGIN
	    DBOPEN, 'point_alt', 1
	    DBBUILD, START_TIME, POINT_IND, POINTINGS.INS_X, POINTINGS.INS_Y, $
		    POINTINGS.ZONE_ID, REPLICATE('N', N_POINTINGS)
	ENDIF
;
;  Signal success.
;
	RESULT = 1
	GOTO, FINISH
;
;  Error handling point.
;
HANDLE_ERROR:
	IF N_ELEMENTS(ERRMSG) NE 0 THEN ERRMSG = 'ADD_ALT: ' + MESSAGE $
		ELSE MESSAGE, MESSAGE, /CONTINUE
;
;  Close the database, and return whether the routine was successful or not.
;
FINISH:
	DBCLOSE
;
	RETURN, RESULT
	END
