	PRO ADD_CDS_POINT, DATE_EFF, EQ_NUM, COEFF
;+
; Project     :	SOHO - CDS
;
; Name        :	ADD_CDS_POINT
;
; Purpose     :	Adds a pointing calibration curve to database
;
; Category    :	CDS, Calibration, Class3
;
; Explanation :	This routine adds a functional expression representing the CDS
;		pointing as a function of OPS position to the database.
;
; Syntax      :	ADD_CDS_POINT, DATE_EFF, EQ_NUM, COEFF
;
; Examples    :	ADD_CDS_POINT, '1995/12/02', 'N2', [4.81,-0.0068]
;
; Inputs      :	DATE_EFF = The date, and optionally time, that the curve
;			   becomes effective.  In other words, the calibration
;			   curve should be applied to all science data taken on
;			   or after that date.  Can be in any CDS time format.
;
;			   The current date is also stored along with the data.
;			   When the database is read in, the most current
;			   version for a given date is used.
;
;			   For example, if there were two values of DATE_EFF in
;			   the database,
;
;				DATE_EFF = 1995/12/02
;				DATE_EFF = 1996/09/16
;
;			   then the first would be used for all data taken
;			   between 1995/12/02 and 1996/09/16, and the second
;			   would be used for all data from 1996/09/16 on.  If
;			   there was more than one entry in the database with
;			   an effective date of 1996/09/16, then the most
;			   recent one would be used.
;
;			   There is one thing to watch out for in maintaining
;			   the database.  Taking the above example, if one
;			   added a calibration with an effective date of
;			   1996/05/01, then it would be applied to data between
;			   that date and the next date in the database,
;			   i.e. 1996/09/16.  If one wanted the new calibration
;			   to apply to all data after 1996/05/01, then one
;			   would have to also add a new entry for 1996/09/16.
;
;		EQ_NUM	 = An identification number for the equation format
;			   being used.  See the documention for GET_CDS_POINT
;			   for a discussion of what the various possible
;			   equations are.
;
;		COEFF	 = The coefficients of a function giving the pointing
;			   calibration curve, expressed as a function of OPS
;			   positions.  Can have up to 10 elements.
;
; Opt. Inputs :	None.
;
; Outputs     :	None.
;
; Opt. Outputs:	None.
;
; Keywords    :	None.
;
; Calls       :	PRIV_ZDBASE, ANYTIM2UTC, GET_UTC, UTC2TAI, DBOPEN, DBBUILD,
;		DBCLOSE
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Prev. Hist. :	None.
;
; History     :	Version 1, 25-Sep-1996, William Thompson, GSFC
;
; Contact     :	WTHOMPSON
;-
;
	ON_ERROR, 2
;
;  Check the input parameters.
;
	IF N_PARAMS() NE 3 THEN MESSAGE,	$
		'Syntax:  ADD_CDS_POINT, DATE_EFF, EQ_NUM, COEFF'
;
	IF (EQ_NUM NE FIX(EQ_NUM)) OR (EQ_NUM LE 0) THEN MESSAGE,	$
		'EQ_NUM must be a positive integer'
;
	IF N_ELEMENTS(COEFF) GT 10 THEN MESSAGE,	$
		'Only up to 10 coefficients are allowed.
;
;  Make sure that one has privilege to write into the calibration database.
;
	MESSAGE = ''
	IF NOT PRIV_ZDBASE(/CALIB,ERRMSG=MESSAGE) THEN MESSAGE, MESSAGE
;
;  Convert the effective date to TAI format.  Reformat the time to millisecond
;  accuracy.  This is necessary so that the times are written out in a
;  controlled way.
;
	TAI_EFF = DOUBLE( STRING( ANYTIM2TAI(DATE_EFF), FORMAT='(F15.3)' ))
;
;  Get the current date.
;
	GET_UTC, DATE
	DATE = UTC2TAI(DATE)
;
;  Pad the coefficients out to 10 values.
;
	C = FLTARR(10)
	C(0) = COEFF(*)
;
;  Store the data in the database.
;
	PRIV = !PRIV  &  !PRIV = 2
	DBOPEN, 'pointing', 1
	DBBUILD, DATE, TAI_EFF, FIX(EQ_NUM), C
	DBCLOSE
	!PRIV = PRIV
;
	RETURN
	END
