	FUNCTION ADD_DATAWIN, DEF, ERRMSG=ERRMSG, DW_ID=DBDW_ID,	$
		CHECK_ONLY=CHECK_ONLY
;+
; Project     :	SOHO - CDS
;
; Name        :	ADD_DATAWIN()
;
; Purpose     :	Adds a data window list definition to the database
;
; Explanation :	This procedure takes a data extraction window list definition
;		and adds it to the data window list databases "datawin" and
;		"datawin2".  The ID number for the data window list is
;		assigned.
;
; Use         :	Result = ADD_DATAWIN( DEF )
;
;		IF NOT ADD_DATAWIN( DEF ) THEN ...
;
; Inputs      :	DEF = This is an anonymous structure containing the following
;		      tags:
;
;			DW_ID	 = Data window list ID number.  Initially, this
;				   is zero.  It is updated by this routine to
;				   reflect the ID number in the database.
;			DETECTOR = Either "G" for GIS or "N" for NIS.
;			DW_DESC	 = A short description of the data window list
;				   beyond what is given in the associated line
;				   list description, e.g. "Full slit, 10 pixels
;				   wide".  Can be up to 65 characters.
;			W_WIDTH  = Width in pixels used to generate the
;				   windows.
;			W_HEIGHT = The height in pixels used to generate the
;				   windows.  VDS only, for GIS this is set to
;				   unity.
;			VDS_BACK = Either 0 for off, or 1 for on,
;				   representing whether or not VDS background
;				   windows are being used.  For GIS window
;				   lists this must be 0.  If VDS_BACK is 1,
;				   then at least one of the window names must
;				   start with the characters "BACK" (case
;				   insensitive).
;			LL_ID	 = The line list ID number for the line list
;				   that this window list was derived from.  An
;				   appropriate entry must exist in the line
;				   list database.  Alternatively, this can be
;				   set to zero to signal that the data window
;				   list is not connected to a line list.
;			WINS	 = An array containing the window definitions.
;
;		      The window definitions themselves are structures, of type
;		      "cds_windows", with the following tags:
;
;			WIN_NAME = Window name, derived from the line name in
;				   the line list database.  Can be up to 40
;				   characters.
;			WIN_DEF	 = The window definition.  This is four
;				   numbers, although GIS window definitions
;				   only use two of them.
;			WIN_FLAG = Whether or not the window can be used to
;				   generate a flag.  Either 0 (false) or 1
;				   (true).
;
; Opt. Inputs :	None.
;
; Outputs     :	The result of the function is a logical value representing
;		whether or not the operation was successful, where 1 is
;		successful and 0 is unsuccessful.
;
;		The tag DW_ID in the input structure is updated to reflect
;		the ID number assigned in the database.
;
; Opt. Outputs:	None.
;
; Keywords    :	
;       ERRMSG    = If defined and passed, then any error messages will be
;                   returned to the user in this parameter rather than
;                   depending on the MESSAGE routine in IDL.  If no errors are
;                   encountered, then a null string is returned.  In order to
;                   use this feature, ERRMSG must be defined first, e.g.
;
;                       ERRMSG = ''
;                       Result = ADD_DATAWIN( ERRMSG=ERRMSG, ... )
;                       IF ERRMSG NE '' THEN ...
;
;	DW_ID	  = If the definition matches one that is already in the
;                   database, then this keyword returns the ID of the
;                   matching data window list.  Otherwise, a -1 is returned.
;
;	CHECK_ONLY= If this keyword is set, then the routine only checks to see
;		    if the proposed entry is valid, but does not attempt to
;		    place it in the database.
;
; Calls       :	DATATYPE, DBOPEN, DB_INFO, DBBUILD, DBCLOSE, GET_LINELIST,
;		DBFIND, DBMATCH
;
; Common      :	None.
;
; Restrictions:	Only this routine can be used to add window list definitions to
;		the database.  Modifying the database by hand could corrupt its
;		integrity.
;
;		The data types and sizes of the structure elements must match
;		the definitions in the database.  The string lengths must not
;		exceed the lengths defined in the database.
;
;		!PRIV must be 2 or greater to use this routine.
;
; Side effects:	Once a window list definition has been added to the database,
;		it must never be removed.
;
; Category    :	Planning, Databases.
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 31 March 1994
;
; Modified    :	Version 1, William Thompson, GSFC, 1 August 1994
;               Version 2, Liyun Wang, GSFC/ARC, September 21, 1994
;                  Added the ERRMSG keyword
;		Version 3, William Thompson, GSFC, 24 October 1994
;			Changed W_HEIGHT to 1 for GIS data.
;               Version 4, Liyun Wang, GSFC/ARC, November 4, 1994
;                  CHK_DATAWIN is called to check the existence of the 
;                  given data window
;		Version 5, William Thompson, GSFC, 8 November 1994
;			Added keyword DW_ID
;		Version 6, William Thompson, GSFC, 29 December 1994
;			Fixed bug when inputs are bytes.
;		Version 7, William Thompson, GSFC, 2 March 1995
;			Enforce uniqueness for DW_DESC.
;			Added keyword CHECK_ONLY.
;		Version 8, William Thompson, GSFC, 14 March 1995
;			Changed DBFIND to DBMATCH for DW_DESC
;		Version 9, William Thompson, GSFC, 15 March 1995
;			Changed so that DW_DESC only needs to be unique for a
;			given detector.
;		Version 10, William Thompson, GSFC, 20 June 1995
;			Added tag WIN_FLAG.
;		Version 11, Stein Vidar Haugan, UiO, 25 September 1995
;			Fixed bug with DBEXT always returning arrays.
;		Version 12, William Thompson, GSFC, 16 May 1997
;			Allow for possibility of CHK_DATAWIN returning an
;			array.
;
; Version     :	Version 12, 16 May 1997
;-
;
	ON_ERROR, 2
;
;  Make sure that the DW_ID keyword is initialized, in case the program
;  exits before the CHK_DATAWIN routine is reached.
;
	DBDW_ID = -1
;
;  Initialize RESULT to represent non-success.  If the routine is successful,
;  this value will be updated below.
;
	RESULT = 0
;
;  Check the input parameters
;
        IF N_PARAMS() NE 1 THEN BEGIN
           MESSAGE = 'Syntax:  Result = ADD_DATAWIN(DEF)'
	   GOTO, HANDLE_ERROR
        ENDIF
;
;  Make sure that the user has privilege to write into the database.
;
	IF !PRIV LT 2 THEN BEGIN
           MESSAGE = '!PRIV must be 2 or greater to write into the database'
           GOTO, HANDLE_ERROR
	ENDIF
;
;  Make sure that the data window list ID number is less than or equal to zero.
;
	IF DEF.DW_ID GT 0 THEN BEGIN
           MESSAGE= 'The window list ID number must be LE 0. '+$
                 'The final value is assigned by this routine'
           GOTO, HANDLE_ERROR
	ENDIF
;
;  Check each of the structure components to verify that it is of the correct
;  type and size.  Start off with the parameters that will be placed in the
;  "datawin" database.
;
	IF DATATYPE(DEF.DETECTOR,1) NE 'String' THEN BEGIN
           MESSAGE = 'Tag DETECTOR must be a character string'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(DEF.DETECTOR) NE 1 THEN BEGIN
           MESSAGE = 'Tag DETECTOR must be a scalar'
           GOTO, HANDLE_ERROR
	END ELSE IF STRLEN(DEF.DETECTOR) NE 1 THEN BEGIN
           MESSAGE = 'Tag DETECTOR must be a single character'
           GOTO, HANDLE_ERROR
	END ELSE IF (DEF.DETECTOR NE 'G') AND (DEF.DETECTOR NE 'N') THEN BEGIN
           MESSAGE = 'Tag DETECTOR must be either "G" or "N"'
           GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(DEF.DW_DESC,1) NE 'String' THEN BEGIN
           MESSAGE = 'Tag DW_DESC must be a character string'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(DEF.DW_DESC) NE 1 THEN BEGIN
           MESSAGE = 'Tag DW_DESC must be a scalar'
           GOTO, HANDLE_ERROR
	END ELSE IF STRLEN(DEF.DW_DESC) GT 65 THEN BEGIN
           MESSAGE = 'Tag DW_DESC must be 65 characters or less'
           GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(DEF.W_WIDTH,1) NE 'Integer' THEN BEGIN
           MESSAGE = 'Tag W_WIDTH must be a short integer'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(DEF.W_WIDTH) NE 1 THEN BEGIN
           MESSAGE = 'Tag W_WIDTH must be a scalar'
           GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(DEF.W_HEIGHT,1) NE 'Integer' THEN BEGIN
           MESSAGE = 'Tag W_HEIGHT must be a short integer'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(DEF.W_HEIGHT) NE 1 THEN BEGIN
           MESSAGE = 'Tag W_HEIGHT must be a scalar'
           GOTO, HANDLE_ERROR
	END ELSE IF (DEF.DETECTOR EQ 'G') AND (DEF.W_HEIGHT NE 1) THEN BEGIN $
           MESSAGE = 'Tag W_HEIGHT must be 1 for the GIS detector'
           GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(DEF.VDS_BACK,1) NE 'Integer' THEN BEGIN
           MESSAGE = 'Tag VDS_BACK must be a short integer'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(DEF.VDS_BACK) NE 1 THEN BEGIN
           MESSAGE = 'Tag VDS_BACK must be a scalar'
           GOTO, HANDLE_ERROR
	END ELSE IF (DEF.VDS_BACK NE 0) AND (DEF.VDS_BACK NE 1) THEN BEGIN
           MESSAGE = 'Tag VDS_BACK must be either 0 or 1'
           GOTO, HANDLE_ERROR
	END ELSE IF (DEF.DETECTOR EQ 'G') AND (DEF.VDS_BACK EQ 1) THEN BEGIN $
           MESSAGE = 'Tag VDS_BACK must be 0 for the GIS detector'
           GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(DEF.LL_ID,1) NE 'Integer' THEN BEGIN
           MESSAGE = 'Tag LL_ID must be a short integer'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(DEF.LL_ID) NE 1 THEN BEGIN
           MESSAGE = 'Tag LL_ID must be a scalar'
           GOTO, HANDLE_ERROR
	ENDIF
;
;  Extract the first instance of the structure in the WINS tag, and check its
;  tags.
;
	TEST = (DEF.WINS)(0)
;
	IF DATATYPE(TEST.WIN_NAME,1) NE 'String' THEN BEGIN
           MESSAGE = 'Tag WINS.WIN_NAME must be a character string'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(TEST.WIN_NAME) NE 1 THEN BEGIN
           MESSAGE = 'Tag WINS.WIN_NAME must be a scalar'
           GOTO, HANDLE_ERROR
	END ELSE IF STRLEN(TEST.WIN_NAME) GT 40 THEN BEGIN
           MESSAGE = 'Tag WINS.WIN_NAME must be 40 characters or less'
           GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(TEST.WIN_DEF,1) NE 'Integer' THEN BEGIN
           MESSAGE = 'Tag WINS.WIN_DEF must be a short integer'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(TEST.WIN_DEF) NE 4 THEN BEGIN
           MESSAGE = 'Tag WINS.WIN_DEF must have four elements'
           GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(TEST.WIN_FLAG,1) NE 'Integer' THEN BEGIN
           MESSAGE = 'Tag WINS.WIN_FLAG must be a short integer'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(TEST.WIN_FLAG) NE 1 THEN BEGIN
           MESSAGE = 'Tag WINS.WIN_FLAG must be a scalar'
           GOTO, HANDLE_ERROR
	END ELSE IF (TEST.WIN_FLAG NE 0) AND (TEST.WIN_FLAG NE 1) THEN BEGIN
	   MESSAGE = 'Tag WINS.WIN_FLAG must be either 0 or 1'
	   GOTO, HANDLE_ERROR
	ENDIF
;
;  Make sure that the line list ID exists in the database.
;
	IF DEF.LL_ID NE 0 THEN BEGIN
		GET_LINELIST, DEF.LL_ID, LL_DEF
		DBOPEN, 'linelist'
		LINELIST_PTR = DBFIND('ll_id=' + STRTRIM(LONG(DEF.LL_ID),2), $
			/SILENT)
		IF !ERR EQ 0 THEN BEGIN
                   MESSAGE = 'Line list ' + STRTRIM(DEF.LL_ID,2) + $
                      ' does not exist in database'
                   GOTO, HANDLE_ERROR
		ENDIF
;
;  Make sure that the DETECTOR tag matches the entry in the line list database.
;
		DBEXT, LINELIST_PTR, 'detector', LL_DETECTOR
		LL_DETECTOR = LL_DETECTOR(0)
		IF DEF.DETECTOR NE LL_DETECTOR THEN BEGIN
                   MESSAGE = 'Tag DETECTOR should be "' + $
                      LL_DETECTOR + '" for this line list ID'
                   GOTO, HANDLE_ERROR
		ENDIF
	END ELSE LINELIST_PTR = 0
	LINELIST_PTR = LONG(LINELIST_PTR(0))
;
;  If VDS_BACK is set, then there has to be at least one window whose name
;  starts with "BACK"
;
	IF DEF.VDS_BACK THEN BEGIN
		FIRST_FOUR = STRUPCASE(STRMID(DEF.WINS.WIN_NAME,0,4))
		IF NOT MAX(FIRST_FOUR EQ 'BACK') THEN BEGIN 
                   MESSAGE = 'At least one of the windows ' + $
                      'must have a name starting with BACK'
                   GOTO, HANDLE_ERROR
		ENDIF
	ENDIF
;----------------------------------------------------------------------
;  Check the existence of the data window
;----------------------------------------------------------------------
        dbdw_id = chk_datawin(def)
	dbdw_id = dbdw_id(0)
        IF dbdw_id GE 0 THEN BEGIN
           message = 'Data base entry for this data window already ' + $
              'exists. ID = '+num2str(dbdw_id)
           GOTO, HANDLE_ERROR
        ENDIF
;
;  Open the primary data window database for write access.
;
	DBOPEN, 'datawin', 1
;
;  Check for an entry with the same description.
;
	ENTRIES = DBFIND('DETECTOR='+DEF.DETECTOR, /SILENT)
	ENTRY = DBMATCH('DW_DESC', STRTRIM(DEF.DW_DESC,2), ENTRIES,	$
		/FULLSTRING)
	IF ENTRY(0) GT 0 THEN BEGIN
           MESSAGE = 'Data window list with same description already exists'
           GOTO, HANDLE_ERROR
	ENDIF
;
;  Find the largest window list ID number currently in the database, and add
;  one to it.    Also calculate the pointer to the new entry.
;
	N_ENTRIES = DB_INFO('entries','datawin')
	IF N_ENTRIES EQ 0 THEN DW_ID = 1 ELSE BEGIN
		DBEXT, -1, 'dw_id', DW_IDS
		DW_ID = MAX(DW_IDS) + 1
	ENDELSE
	DATAWIN_PTR = LONG(N_ENTRIES + 1)
;
;  If the CHECK_ONLY keyword was set, then return now as if the routine were
;  successful.  However, don't modify the data window ID in the definition.
;
	IF KEYWORD_SET(CHECK_ONLY) THEN BEGIN
		RESULT = 1
		GOTO, FINISH
	ENDIF
;
;  Append the data to the opened database.
;
	N_WINDOWS = FIX(N_ELEMENTS(DEF.WINS))
	DUMMY = WHERE(DEF.WINS.WIN_FLAG EQ 1, N_FLAGS)
	DBBUILD, DW_ID, DEF.DETECTOR, STRTRIM(DEF.DW_DESC,2), LINELIST_PTR, $
		DEF.LL_ID, DEF.W_WIDTH, DEF.W_HEIGHT, DEF.VDS_BACK,	$
		N_WINDOWS, FIX(N_FLAGS), STATUS=STATUS
	IF STATUS EQ 0 THEN BEGIN
           MESSAGE = 'Write to datawin database was not successful'
           GOTO, HANDLE_ERROR
        ENDIF
;
;  Open the secondary data window list database, and append the data in that
;  database.
;
	DBOPEN, 'datawin2', 1
	IND = INDGEN(N_WINDOWS) + 1
	PTR = REPLICATE(DATAWIN_PTR, N_WINDOWS)
	DW_IDS = REPLICATE(DW_ID, N_WINDOWS)
	DBBUILD, PTR, DW_IDS, IND, DEF.WINS.WIN_NAME, DEF.WINS.WIN_DEF,	$
		DEF.WINS.WIN_FLAG, STATUS=STATUS
;
;  If the second write was unsuccessful, then delete the entry inserted above
;  into "datawin".
;
	IF STATUS EQ 0 THEN BEGIN
           MESSAGE = 'Write to datawin2 database was not successful.  ' + $
                 'Returned datawin to its previous state'
           PRIV = !PRIV  &  !PRIV = 3
           DBDELETE, DATAWIN_PTR, 'datawin'
           !PRIV = PRIV
           GOTO, HANDLE_ERROR
	ENDIF
;
;  Update the data window list ID number in the structure.  Signal success.
;
	DEF.DW_ID = DW_ID
	RESULT = 1
	GOTO, FINISH
;
;  Error handling point.
;
HANDLE_ERROR:
	IF N_ELEMENTS(ERRMSG) NE 0 THEN ERRMSG = 'ADD_DATAWIN: ' + MESSAGE $
		ELSE MESSAGE, MESSAGE, /CONTINUE
;
;  Close the database, and return whether the routine was successful or not.
;
FINISH:
	DBCLOSE
;
	RETURN, RESULT
	END
