	FUNCTION ADD_EXP_COMM, COMM, ERRMSG=ERRMSG
;+
; Project     :	SOHO - CDS
;
; Name        :	ADD_EXP_COMM()
;
; Purpose     :	Adds a CDS experiment comment entry to the as-run database
;
; Explanation :	This procedure takes a comment pertaining to the CDS
;		raster-level (experiment) as-run entry and adds it to the
;		database "exper_comment".  This database contains a series of
;		such entries, making a historical list.
;
; Use         :	Result = ADD_EXP_COMM( COMM )
;
;		IF NOT ADD_EXP_COMM( COMM ) THEN ...
;
; Inputs      :	COMM = This is an anonymous structure containing the following
;		      tags:
;
;			SEQ_NUM	   = Experiment sequence number.  This serves
;				     as a link to the experiment database.
;			COMM_NO	   = Comment ID number.  Initially, this is -1.
;				     It is updated by this routine to reflect
;				     the ID number in the database.
;			COMMENT	   = A comment.
;
; Opt. Inputs :	None.
;
; Outputs     :	The result of the function is a logical value representing
;		whether or not the operation was successful, where 1 is
;		successful and 0 is unsuccessful.
;
; Opt. Outputs:	None.
;
; Keywords    :
;       ERRMSG    = If defined and passed, then any error messages will be
;                   returned to the user in this parameter rather than
;                   depending on the MESSAGE routine in IDL.  If no errors are
;                   encountered, then a null string is returned.  In order to
;                   use this feature, ERRMSG must be defined first, e.g.
;
;                       ERRMSG = ''
;                       Result = ADD_EXP_COMM( ERRMSG=ERRMSG, ... )
;                       IF ERRMSG NE '' THEN ...
;
;
; Calls       :	DATATYPE, DBOPEN, DBBUILD, DBCLOSE, TRIM, GET_OBJECT,
;		GET_CAMPAIGN, GET_PROGRAM, GET_STUDY
;
; Common      :	None.
;
; Restrictions:	Only this routine can be used to add entries to the
;		EXPER_COMMENT database.  Modifying the database by hand could
;		corrupt its integrity.
;
;		The data types and sizes of the structure elements must match
;		the definitions in the database.
;
;		!PRIV must be 2 or greater to use this routine.
;
; Side effects:	Once an entry has been added to the database, it must never be
;		removed.
;
;		The dates in the structure are rounded off to millisecond
;		accuracy.
;
; Category    :	Planning, Databases.
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 7 March 1996
;
; Modified    :	Version 1, William Thompson, GSFC, 7 March 1996
;		Version 2, William Thompson, GSFC, 11 April 1996
;			Call LOCK_DATABASE and UNLOCK_DATABASE
;
; Version     :	Version 2, 11 April 1996
;-
;
	ON_ERROR, 2
;
;  Initialize RESULT to represent non-success.  If the routine is successful,
;  this value will be updated below.
;
	RESULT = 0
;
;  Check the input parameters
;
        IF N_PARAMS() NE 1 THEN BEGIN
           MESSAGE = 'Syntax:  Result = ADD_EXP_COMM(COMM)'
	   GOTO, HANDLE_ERROR
        ENDIF
;
;  Make sure that the user has privilege to write into the database.
;
	IF !PRIV LT 2 THEN BEGIN
           MESSAGE = '!PRIV must be 2 or greater to write into the database'
           GOTO, HANDLE_ERROR
	ENDIF
;
;  Make sure that the comment ID number is negative.
;
	IF COMM.COMM_NO GE 0 THEN BEGIN
           MESSAGE = 'The comment ID number must be negative.  ' +	$
		   'The final value is assigned by this routine'
           GOTO, HANDLE_ERROR
	ENDIF
;
;  Check each of the structure components to verify that it is of the correct
;  type and size.
;
	IF DATATYPE(COMM.SEQ_NUM,1) NE 'Long' THEN BEGIN
	   MESSAGE = 'Tag SEQ_NUM must be a long integer'
	   GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(COMM.SEQ_NUM) NE 1 THEN BEGIN
	   MESSAGE = 'Tag SEQ_NUM must be a scalar'
	   GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(COMM.COMM_NO,1) NE 'Long' THEN BEGIN
	   MESSAGE = 'Tag COMM_NO must be a long integer'
	   GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(COMM.COMM_NO) NE 1 THEN BEGIN
	   MESSAGE = 'Tag COMM_NO must be a scalar'
	   GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(COMM.COMMENT,1) NE 'String' THEN BEGIN
	   MESSAGE = 'Tag COMMENT must be a character string'
	   GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(COMM.COMMENT) NE 1 THEN BEGIN
	   MESSAGE = 'Tag COMMENT must be a scalar'
	   GOTO, HANDLE_ERROR
	END ELSE IF STRLEN(COMM.COMMENT) GT 60 THEN BEGIN
	   MESSAGE = 'Tag COMMENT must be 60 characters or less'
	   GOTO, HANDLE_ERROR
	ENDIF
;
;  Get the corresponding experiment entry.
;
	GET_EXPER, COMM.SEQ_NUM, EXPER
	IF EXPER.SEQ_NUM LT 0 THEN BEGIN
		MESSAGE = 'SEQ_NUM=' + TRIM(COMM.SEQ_NUM) + ' not found'
		GOTO, HANDLE_ERROR
	ENDIF
;
;  Lock the database for write.
;
	LOCK_DATABASE, 'exper_comment', LOCKFILE
;
;  Open the exper_comment as-run database for write access.
;
	DBOPEN, 'exper_comment', 1
;
;  Find the largest comment ID number currently in the database, and add one to
;  it.  If the database is empty, then set the COMM_NO to one.  Find the number
;  of entries and add one to it--that will be the pointer for the new entry.
;
	N_ENTRIES = DB_INFO('entries','exper_comment')
	IF N_ENTRIES EQ 0 THEN COMM_NO = 1L ELSE BEGIN
		DBEXT, -1, 'comm_no', COMM_NOS
		COMM_NO = MAX(COMM_NOS) + 1
	ENDELSE
;
;  Get the modification date.
;
	GET_UTC, DATE_MOD
	DATE_MOD = UTC2TAI(DATE_MOD)
;
;  Add the entry.
;
	DBBUILD, COMM.SEQ_NUM, COMM_NO, COMM.COMMENT, DATE_MOD, STATUS=STATUS
	IF STATUS EQ 0 THEN BEGIN
	   MESSAGE = 'Write to exper_comment database was not successful.'
	   GOTO, HANDLE_ERROR
	ENDIF
;
;  If necessary, modify the experiment entry to signal that it contains
;  comments.
;
	IF EXPER.COMMENTS EQ 'N' THEN BEGIN
		MESSAGE = ''
		EXPER.COMMENTS = 'Y'
		IF NOT MOD_EXPER(EXPER,ERRMSG=MESSAGE) THEN GOTO, HANDLE_ERROR
	ENDIF
;
;  Update the comment ID number in the structure.  Signal success.
;
	COMM.COMM_NO = COMM_NO
	RESULT = 1
	GOTO, FINISH
;
;  Error handling point.
;
HANDLE_ERROR:
	IF N_ELEMENTS(ERRMSG) NE 0 THEN ERRMSG = 'ADD_EXP_COMM: ' + MESSAGE $
		ELSE MESSAGE, MESSAGE, /CONTINUE
;
;  Close the database, and return whether the routine was successful or not.
;
FINISH:
	DBCLOSE
	IF N_ELEMENTS(LOCKFILE) NE 0 THEN UNLOCK_DATABASE, LOCKFILE
;
	RETURN, RESULT
	END
