	FUNCTION ADD_EXPER, OBS, ERRMSG=ERRMSG
;+
; Project     :	SOHO - CDS
;
; Name        :	ADD_EXPER()
;
; Purpose     :	Adds a CDS experiment entry to the as-run database
;
; Explanation :	This procedure takes a CDS raster-level (experiment) as-run
;		entry and adds it to the database "experiment".  This database
;		contains a series of such entries, making a historical list.
;
; Use         :	Result = ADD_EXPER( OBS )
;
;		IF NOT ADD_EXPER( OBS ) THEN ...
;
; Inputs      :	OBS = This is an anonymous structure containing the following
;		      tags:
;
;			PROG_NUM   = Study counter number.
;			SEQ_NUM	   = Unique sequence number.  This serves as
;				     the primary key to the database.  Note
;				     that SEQ_NUM is *not* assigned by this
;				     routine.
;			OBS_SEQ	   = A character string derived from the raster
;				     number and the variation number,
;				     e.g. "R4V3".
;			COUNT	   = The number of repeated observations making
;				     up this experiment.  Normally 1.
;			SEQ_IND	   = The raster counter
;			RAS_ID	   = The raster ID number
;			RAS_VAR	   = The raster variation index
;			EXPTIME	   = The exposure time, in seconds
;			OBS_MODE   = The observing mode.
;			DW_ID	   = The data window ID number
;			DATE_OBS   = Date/time of beginning of observation,
;				     in TAI format
;			DATE_END   = Date/time of end of observation, in TAI
;				     format
;			OBT_TIME   = Onboard date/time of beginning of
;				     observation, in TAI format
;			OBT_END	   = Onboard date/time of end of observation,
;				     in TAI format
;			XCEN	   = Center of instrument field-of-view in
;				     solar X
;			YCEN	   = Center of instrument field-of-view in
;				     solar Y
;			ANGLE	   = Angle of instrument relative to solar
;				     north
;			IXWIDTH	   = Width of field-of-view in instrument X
;			IYWIDTH	   = Width of field-of-view in instrument Y
;			INS_X0	   = The instrument origin pointing in X
;			INS_Y0	   = The instrument origin pointing in Y
;			INS_ROLL   = The instrument roll angle
;			SC_X0	   = The spacecraft pointing in X
;			SC_Y0	   = The spacecraft pointing in Y
;			SC_ROLL	   = The spacecraft roll
;			WAVEMIN	   = The minimum wavelength of observation, in
;				     Angstroms.
;			WAVEMAX	   = The maximum wavelength, in Angstroms
;			TRACKING   = True if tracking is on
;			SER_ID	   = The series ID number
;			OPSLBITS   = The OPS-L status bits at the start of the
;				     observation.
;			OPSRBITS   = The OPS-R status bits at the start of the
;				     observation.
;			SLIT_POS   = The slit position at the start of the
;				     observation.
;			MIR_POS	   = The mirror position at the start of the
;				     observation.
;			EV_ENAB	   = True if event recognition enabled
;			COMP_ERR   = True if a compression error was
;				     encountered.
;			VDS_PMCP   = The VDS MCP programmable voltage setting
;			VDS_MODE   = The VDS readout mode.
;			VDS_ORI	   = True if the VDS telemetry data is oriented
;				     by columns
;			VDS_ACC	   = True if VDS is operated in accumulate mode
;			GSET_ID	   = GIS Setup ID
;			DETECTOR   = Either N or G for NIS/GIS.
;			ZONE_ID	   = Pointing zone ID
;			SLIT_NUM   = Slit number
;			FILENAME   = FITS file name
;			SEQVALID   = True for valid science data
;			DATASRC	   = Data source, either Q(uicklook) or F(inal
;				     distribution).
;
;		      It can also be an array of such structures.
;
; Opt. Inputs :	None.
;
; Outputs     :	The result of the function is a logical value representing
;		whether or not the operation was successful, where 1 is
;		successful and 0 is unsuccessful.
;
; Opt. Outputs:	None.
;
; Keywords    :
;       ERRMSG    = If defined and passed, then any error messages will be
;                   returned to the user in this parameter rather than
;                   depending on the MESSAGE routine in IDL.  If no errors are
;                   encountered, then a null string is returned.  In order to
;                   use this feature, ERRMSG must be defined first, e.g.
;
;                       ERRMSG = ''
;                       Result = ADD_EXPER( ERRMSG=ERRMSG, ... )
;                       IF ERRMSG NE '' THEN ...
;
;
; Calls       :	DATATYPE, DBOPEN, DBBUILD, DBCLOSE, TRIM, GET_OBJECT,
;		GET_CAMPAIGN, GET_PROGRAM, GET_STUDY
;
; Common      :	None.
;
; Restrictions:	Only this routine can be used to add entries to the EXPERIMENT
;		database.  Modifying the database by hand could corrupt its
;		integrity.
;
;		The data types and sizes of the structure elements must match
;		the definitions in the database.
;
;		!PRIV must be 2 or greater to use this routine.
;
; Side effects:	Once an entry has been added to the database, it must never be
;		removed.
;
;		The dates in the structure are rounded off to millisecond
;		accuracy.
;
; Category    :	Planning, Databases.
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 30 June 1995
;
; Modified    :	Version 1, William Thompson, GSFC, 30 June 1995
;		Version 2, William Thompson, GSFC, 4 August 1995
;			Changed so that SEQ_NUM is not assigned by this
;			routine.
;		Version 3, William Thompson, GSFC, 21 February 1996
;			Modified to be more robust.  It will complain about not
;			finding IDs in the databases, but will insert the
;			catalog entry anyway.
;		Version 4, William Thompson, GSFC, 23 February 1996
;			Modified for version 2 of the experiment databases.
;		Version 5, William Thompson, GSFC, 1 April 1996
;			Added database locking capability.
;		Version 6, William Thompson, GSFC, 1 April 1996
;			Added parameter DATASRC
;		Version 7, William Thompson, GSFC, 9 April 1996
;			Split off locking mechanism to separate routines
;			LOCK_DATABASE, UNLOCK_DATABASE
;		Version 8, William Thompson, GSFC, 25 March 1997
;			Added check to make sure entry didn't appear while the
;			routine was running.
;
; Version     :	Version 8, 25 March 1997
;-
;
	ON_ERROR, 2
;
;  Initialize RESULT to represent non-success.  If the routine is successful,
;  this value will be updated below.
;
	RESULT = 0
;
;  Check the input parameters
;
        IF N_PARAMS() NE 1 THEN BEGIN
           MESSAGE = 'Syntax:  Result = ADD_EXPER(OBS)'
	   GOTO, HANDLE_ERROR
        ENDIF
;
;  Make sure that the user has privilege to write into the database.
;
	IF !PRIV LT 2 THEN BEGIN
           MESSAGE = '!PRIV must be 2 or greater to write into the database'
           GOTO, HANDLE_ERROR
	ENDIF
;
;  Initialize the MAIN_PTR array.
;
	MAIN_PTR = LONARR(N_ELEMENTS(OBS))
;
;  Check each of the structure components to verify that it is of the correct
;  type and size.
;
	FOR I = 0,N_ELEMENTS(OBS)-1 DO BEGIN
	    IF DATATYPE(OBS(I).PROG_NUM,1) NE 'Long' THEN BEGIN
	       MESSAGE = 'Tag PROG_NUM must be a long integer'
	       GOTO, HANDLE_ERROR
	    END ELSE IF N_ELEMENTS(OBS(I).PROG_NUM) NE 1 THEN BEGIN
	       MESSAGE = 'Tag PROG_NUM must be a scalar'
	       GOTO, HANDLE_ERROR
	    ENDIF
;
	    IF DATATYPE(OBS(I).SEQ_NUM,1) NE 'Long' THEN BEGIN
	       MESSAGE = 'Tag SEQ_NUM must be a long integer'
	       GOTO, HANDLE_ERROR
	    END ELSE IF N_ELEMENTS(OBS(I).SEQ_NUM) NE 1 THEN BEGIN
	       MESSAGE = 'Tag SEQ_NUM must be a scalar'
	       GOTO, HANDLE_ERROR
	    ENDIF
;
	    IF DATATYPE(OBS(I).OBS_SEQ,1) NE 'String' THEN BEGIN
	       MESSAGE = 'Tag OBS_SEQ must be a character string'
	       GOTO, HANDLE_ERROR
	    END ELSE IF N_ELEMENTS(OBS(I).OBS_SEQ) NE 1 THEN BEGIN
	       MESSAGE = 'Tag OBS_SEQ must be a scalar'
	       GOTO, HANDLE_ERROR
	    END ELSE IF STRLEN(OBS(I).OBS_SEQ) GT 12 THEN BEGIN
	       MESSAGE = 'Tag OBS_SEQ must be 12 characters or less'
	       GOTO, HANDLE_ERROR
	    ENDIF
;
	    IF DATATYPE(OBS(I).COUNT,1) NE 'Integer' THEN BEGIN
	       MESSAGE = 'Tag COUNT must be a short integer'
	       GOTO, HANDLE_ERROR
	    END ELSE IF N_ELEMENTS(OBS(I).COUNT) NE 1 THEN BEGIN
	       MESSAGE = 'Tag COUNT must be a scalar'
	       GOTO, HANDLE_ERROR
	    ENDIF
;
	    IF DATATYPE(OBS(I).SEQ_IND,1) NE 'Integer' THEN BEGIN
	       MESSAGE = 'Tag SEQ_IND must be a short integer'
	       GOTO, HANDLE_ERROR
	    END ELSE IF N_ELEMENTS(OBS(I).SEQ_IND) NE 1 THEN BEGIN
	       MESSAGE = 'Tag SEQ_IND must be a scalar'
	       GOTO, HANDLE_ERROR
	    ENDIF
;
	    IF DATATYPE(OBS(I).RAS_ID,1) NE 'Integer' THEN BEGIN
	       MESSAGE = 'Tag RAS_ID must be a short integer'
	       GOTO, HANDLE_ERROR
	    END ELSE IF N_ELEMENTS(OBS(I).RAS_ID) NE 1 THEN BEGIN
	       MESSAGE = 'Tag RAS_ID must be a scalar'
	       GOTO, HANDLE_ERROR
	    ENDIF
;
	    IF DATATYPE(OBS(I).RAS_VAR,1) NE 'Integer' THEN BEGIN
	       MESSAGE = 'Tag RAS_VAR must be a short integer'
	       GOTO, HANDLE_ERROR
	    END ELSE IF N_ELEMENTS(OBS(I).RAS_VAR) NE 1 THEN BEGIN
	       MESSAGE = 'Tag RAS_VAR must be a scalar'
	       GOTO, HANDLE_ERROR
	    ENDIF
;
	    IF DATATYPE(OBS(I).EXPTIME,1) NE 'Float' THEN BEGIN
	       MESSAGE = 'Tag EXPTIME must be a floating point number'
	       GOTO, HANDLE_ERROR
	    END ELSE IF N_ELEMENTS(OBS(I).EXPTIME) NE 1 THEN BEGIN
	       MESSAGE = 'Tag EXPTIME must be a scalar'
	       GOTO, HANDLE_ERROR
	    ENDIF
;
	    IF DATATYPE(OBS(I).OBS_MODE,1) NE 'String' THEN BEGIN
	       MESSAGE = 'Tag OBS_MODE must be a character string'
	       GOTO, HANDLE_ERROR
	    END ELSE IF N_ELEMENTS(OBS(I).OBS_MODE) NE 1 THEN BEGIN
	       MESSAGE = 'Tag OBS_MODE must be a scalar'
	       GOTO, HANDLE_ERROR
	    END ELSE IF STRLEN(OBS(I).OBS_MODE) GT 20 THEN BEGIN
	       MESSAGE = 'Tag OBS_MODE must be 20 characters or less'
	       GOTO, HANDLE_ERROR
	    ENDIF
;
	    IF DATATYPE(OBS(I).DW_ID,1) NE 'Integer' THEN BEGIN
	       MESSAGE = 'Tag DW_ID must be a short integer'
	       GOTO, HANDLE_ERROR
	    END ELSE IF N_ELEMENTS(OBS(I).DW_ID) NE 1 THEN BEGIN
	       MESSAGE = 'Tag DW_ID must be a scalar'
	       GOTO, HANDLE_ERROR
	    ENDIF
;
	    IF DATATYPE(OBS(I).DATE_OBS,1) NE 'Double' THEN BEGIN
	       MESSAGE = 'Tag DATE_OBS must be a double precision number'
	       GOTO, HANDLE_ERROR
	    END ELSE IF N_ELEMENTS(OBS(I).DATE_OBS) NE 1 THEN BEGIN
	       MESSAGE = 'Tag DATE_OBS must be a scalar'
	       GOTO, HANDLE_ERROR
	    ENDIF
;
	    IF DATATYPE(OBS(I).DATE_END,1) NE 'Double' THEN BEGIN
	       MESSAGE = 'Tag DATE_END must be a double precision number'
	       GOTO, HANDLE_ERROR
	    END ELSE IF N_ELEMENTS(OBS(I).DATE_END) NE 1 THEN BEGIN
	       MESSAGE = 'Tag DATE_END must be a scalar'
	       GOTO, HANDLE_ERROR
	    ENDIF
;
	    IF DATATYPE(OBS(I).OBT_TIME,1) NE 'Double' THEN BEGIN
	       MESSAGE = 'Tag OBT_TIME must be a double precision number'
	       GOTO, HANDLE_ERROR
	    END ELSE IF N_ELEMENTS(OBS(I).OBT_TIME) NE 1 THEN BEGIN
	       MESSAGE = 'Tag OBT_TIME must be a scalar'
	       GOTO, HANDLE_ERROR
	    ENDIF
;
	    IF DATATYPE(OBS(I).OBT_END,1) NE 'Double' THEN BEGIN
	       MESSAGE = 'Tag OBT_END must be a double precision number'
	       GOTO, HANDLE_ERROR
	    END ELSE IF N_ELEMENTS(OBS(I).OBT_END) NE 1 THEN BEGIN
	       MESSAGE = 'Tag OBT_END must be a scalar'
	       GOTO, HANDLE_ERROR
	    ENDIF
;
	    IF DATATYPE(OBS(I).XCEN,1) NE 'Float' THEN BEGIN
	       MESSAGE = 'Tag XCEN must be a floating point number'
	       GOTO, HANDLE_ERROR
	    END ELSE IF N_ELEMENTS(OBS(I).XCEN) NE 1 THEN BEGIN
	       MESSAGE = 'Tag XCEN must be a scalar'
	       GOTO, HANDLE_ERROR
	    ENDIF
;
	    IF DATATYPE(OBS(I).YCEN,1) NE 'Float' THEN BEGIN
	       MESSAGE = 'Tag YCEN must be a floating point number'
	       GOTO, HANDLE_ERROR
	    END ELSE IF N_ELEMENTS(OBS(I).YCEN) NE 1 THEN BEGIN
	       MESSAGE = 'Tag YCEN must be a scalar'
	       GOTO, HANDLE_ERROR
	    ENDIF
;
	    IF DATATYPE(OBS(I).ANGLE,1) NE 'Float' THEN BEGIN
	       MESSAGE = 'Tag ANGLE must be a floating point number'
	       GOTO, HANDLE_ERROR
	    END ELSE IF N_ELEMENTS(OBS(I).ANGLE) NE 1 THEN BEGIN
	       MESSAGE = 'Tag ANGLE must be a scalar'
	       GOTO, HANDLE_ERROR
	    ENDIF
;
	    IF DATATYPE(OBS(I).IXWIDTH,1) NE 'Float' THEN BEGIN
	       MESSAGE = 'Tag IXWIDTH must be a floating point number'
	       GOTO, HANDLE_ERROR
	    END ELSE IF N_ELEMENTS(OBS(I).IXWIDTH) NE 1 THEN BEGIN
	       MESSAGE = 'Tag IXWIDTH must be a scalar'
	       GOTO, HANDLE_ERROR
	    ENDIF
;
	    IF DATATYPE(OBS(I).IYWIDTH,1) NE 'Float' THEN BEGIN
	       MESSAGE = 'Tag IYWIDTH must be a floating point number'
	       GOTO, HANDLE_ERROR
	    END ELSE IF N_ELEMENTS(OBS(I).IYWIDTH) NE 1 THEN BEGIN
	       MESSAGE = 'Tag IYWIDTH must be a scalar'
	       GOTO, HANDLE_ERROR
	    ENDIF
;
	    IF DATATYPE(OBS(I).INS_X0,1) NE 'Float' THEN BEGIN
	       MESSAGE = 'Tag INS_X0 must be a floating point number'
	       GOTO, HANDLE_ERROR
	    END ELSE IF N_ELEMENTS(OBS(I).INS_X0) NE 1 THEN BEGIN
	       MESSAGE = 'Tag INS_X0 must be a scalar'
	       GOTO, HANDLE_ERROR
	    ENDIF
;
	    IF DATATYPE(OBS(I).INS_Y0,1) NE 'Float' THEN BEGIN
	       MESSAGE = 'Tag INS_Y0 must be a floating point number'
	       GOTO, HANDLE_ERROR
	    END ELSE IF N_ELEMENTS(OBS(I).INS_Y0) NE 1 THEN BEGIN
	       MESSAGE = 'Tag INS_Y0 must be a scalar'
	       GOTO, HANDLE_ERROR
	    ENDIF
;
	    IF DATATYPE(OBS(I).INS_ROLL,1) NE 'Float' THEN BEGIN
	       MESSAGE = 'Tag INS_ROLL must be a floating point number'
	       GOTO, HANDLE_ERROR
	    END ELSE IF N_ELEMENTS(OBS(I).INS_ROLL) NE 1 THEN BEGIN
	       MESSAGE = 'Tag INS_ROLL must be a scalar'
	       GOTO, HANDLE_ERROR
	    ENDIF
;
	    IF DATATYPE(OBS(I).SC_X0,1) NE 'Float' THEN BEGIN
	       MESSAGE = 'Tag SC_X0 must be a floating point number'
	       GOTO, HANDLE_ERROR
	    END ELSE IF N_ELEMENTS(OBS(I).SC_X0) NE 1 THEN BEGIN
	       MESSAGE = 'Tag SC_X0 must be a scalar'
	       GOTO, HANDLE_ERROR
	    ENDIF
;
	    IF DATATYPE(OBS(I).SC_Y0,1) NE 'Float' THEN BEGIN
	       MESSAGE = 'Tag SC_Y0 must be a floating point number'
	       GOTO, HANDLE_ERROR
	    END ELSE IF N_ELEMENTS(OBS(I).SC_Y0) NE 1 THEN BEGIN
	       MESSAGE = 'Tag SC_Y0 must be a scalar'
	       GOTO, HANDLE_ERROR
	    ENDIF
;
	    IF DATATYPE(OBS(I).SC_ROLL,1) NE 'Float' THEN BEGIN
	       MESSAGE = 'Tag SC_ROLL must be a floating point number'
	       GOTO, HANDLE_ERROR
	    END ELSE IF N_ELEMENTS(OBS(I).SC_ROLL) NE 1 THEN BEGIN
	       MESSAGE = 'Tag SC_ROLL must be a scalar'
	       GOTO, HANDLE_ERROR
	    ENDIF
;
	    IF DATATYPE(OBS(I).WAVEMIN,1) NE 'Float' THEN BEGIN
	       MESSAGE = 'Tag WAVEMIN must be a floating point number'
	       GOTO, HANDLE_ERROR
	    END ELSE IF N_ELEMENTS(OBS(I).WAVEMIN) NE 1 THEN BEGIN
	       MESSAGE = 'Tag WAVEMIN must be a scalar'
	       GOTO, HANDLE_ERROR
	    ENDIF
;
	    IF DATATYPE(OBS(I).WAVEMAX,1) NE 'Float' THEN BEGIN
	       MESSAGE = 'Tag WAVEMAX must be a floating point number'
	       GOTO, HANDLE_ERROR
	    END ELSE IF N_ELEMENTS(OBS(I).WAVEMAX) NE 1 THEN BEGIN
	       MESSAGE = 'Tag WAVEMAX must be a scalar'
	       GOTO, HANDLE_ERROR
	    ENDIF
;
	    IF DATATYPE(OBS(I).TRACKING,1) NE 'Byte' THEN BEGIN
	       MESSAGE = 'Tag TRACKING must be a short integer'
	       GOTO, HANDLE_ERROR
	    END ELSE IF N_ELEMENTS(OBS(I).TRACKING) NE 1 THEN BEGIN
	       MESSAGE = 'Tag TRACKING must be a scalar'
	       GOTO, HANDLE_ERROR
	    END ELSE IF (OBS(I).TRACKING NE 0) AND (OBS(I).TRACKING NE 1) $
		    THEN BEGIN
	       MESSAGE = 'Tag TRACKING must be either 0 or 1'
	       GOTO, HANDLE_ERROR
	    ENDIF
;
	    IF DATATYPE(OBS(I).SER_ID,1) NE 'Integer' THEN BEGIN
	       MESSAGE = 'Tag SER_ID must be a short integer'
	       GOTO, HANDLE_ERROR
	    END ELSE IF N_ELEMENTS(OBS(I).SER_ID) NE 1 THEN BEGIN
	       MESSAGE = 'Tag SER_ID must be a scalar'
	       GOTO, HANDLE_ERROR
	    ENDIF
;
	    IF DATATYPE(OBS(I).OPSLBITS,1) NE 'String' THEN BEGIN
	       MESSAGE = 'Tag OPSLBITS must be a character string'
	       GOTO, HANDLE_ERROR
	    END ELSE IF N_ELEMENTS(OBS(I).OPSLBITS) NE 1 THEN BEGIN
	       MESSAGE = 'Tag OPSLBITS must be a scalar'
	       GOTO, HANDLE_ERROR
	    END ELSE IF STRLEN(OBS(I).OPSLBITS) NE 4 THEN BEGIN
	       MESSAGE = 'Tag OPSLBITS must be 4 characters'
	       GOTO, HANDLE_ERROR
	    ENDIF
;
	    IF DATATYPE(OBS(I).OPSRBITS,1) NE 'String' THEN BEGIN
	       MESSAGE = 'Tag OPSRBITS must be a character string'
	       GOTO, HANDLE_ERROR
	    END ELSE IF N_ELEMENTS(OBS(I).OPSRBITS) NE 1 THEN BEGIN
	       MESSAGE = 'Tag OPSRBITS must be a scalar'
	       GOTO, HANDLE_ERROR
	    END ELSE IF STRLEN(OBS(I).OPSRBITS) NE 4 THEN BEGIN
	       MESSAGE = 'Tag OPSRBITS must be 4 characters'
	       GOTO, HANDLE_ERROR
	    ENDIF
;
	    IF DATATYPE(OBS(I).SLIT_POS,1) NE 'Integer' THEN BEGIN
	       MESSAGE = 'Tag SLIT_POS must be a short integer'
	       GOTO, HANDLE_ERROR
	    END ELSE IF N_ELEMENTS(OBS(I).SLIT_POS) NE 1 THEN BEGIN
	       MESSAGE = 'Tag SLIT_POS must be a scalar'
	       GOTO, HANDLE_ERROR
	    ENDIF
;
	    IF DATATYPE(OBS(I).MIR_POS,1) NE 'Integer' THEN BEGIN
	       MESSAGE = 'Tag MIR_POS must be a short integer'
	       GOTO, HANDLE_ERROR
	    END ELSE IF N_ELEMENTS(OBS(I).MIR_POS) NE 1 THEN BEGIN
	       MESSAGE = 'Tag MIR_POS must be a scalar'
	       GOTO, HANDLE_ERROR
	    ENDIF
;
	    IF DATATYPE(OBS(I).EV_ENAB,1) NE 'Byte' THEN BEGIN
	       MESSAGE = 'Tag EV_ENAB must be a short integer'
	       GOTO, HANDLE_ERROR
	    END ELSE IF N_ELEMENTS(OBS(I).EV_ENAB) NE 1 THEN BEGIN
	       MESSAGE = 'Tag EV_ENAB must be a scalar'
	       GOTO, HANDLE_ERROR
	    END ELSE IF (OBS(I).EV_ENAB NE 0) AND (OBS(I).EV_ENAB NE 1) $
		    THEN BEGIN
	       MESSAGE = 'Tag EV_ENAB must be either 0 or 1'
	       GOTO, HANDLE_ERROR
	    ENDIF
;
	    IF DATATYPE(OBS(I).COMP_ERR,1) NE 'Byte' THEN BEGIN
	       MESSAGE = 'Tag COMP_ERR must be a short integer'
	       GOTO, HANDLE_ERROR
	    END ELSE IF N_ELEMENTS(OBS(I).COMP_ERR) NE 1 THEN BEGIN
	       MESSAGE = 'Tag COMP_ERR must be a scalar'
	       GOTO, HANDLE_ERROR
	    END ELSE IF (OBS(I).COMP_ERR NE 0) AND (OBS(I).COMP_ERR NE 1) $
		    THEN BEGIN
	       MESSAGE = 'Tag COMP_ERR must be either 0 or 1'
	       GOTO, HANDLE_ERROR
	    ENDIF
;
	    IF DATATYPE(OBS(I).VDS_PMCP,1) NE 'Integer' THEN BEGIN
	       MESSAGE = 'Tag VDS_PMCP must be a short integer'
	       GOTO, HANDLE_ERROR
	    END ELSE IF N_ELEMENTS(OBS(I).VDS_PMCP) NE 1 THEN BEGIN
	       MESSAGE = 'Tag VDS_PMCP must be a scalar'
	       GOTO, HANDLE_ERROR
	    ENDIF
;
	    IF DATATYPE(OBS(I).VDS_MODE,1) NE 'Integer' THEN BEGIN
	       MESSAGE = 'Tag VDS_MODE must be a short integer'
	       GOTO, HANDLE_ERROR
	    END ELSE IF N_ELEMENTS(OBS(I).VDS_MODE) NE 1 THEN BEGIN
	       MESSAGE = 'Tag VDS_MODE must be a scalar'
	       GOTO, HANDLE_ERROR
	    ENDIF
;
	    IF DATATYPE(OBS(I).VDS_ORI,1) NE 'Byte' THEN BEGIN
	       MESSAGE = 'Tag VDS_ORI must be a byte value'
	       GOTO, HANDLE_ERROR
	    END ELSE IF N_ELEMENTS(OBS(I).VDS_ORI) NE 1 THEN BEGIN
	       MESSAGE = 'Tag VDS_ORI must be a scalar'
	       GOTO, HANDLE_ERROR
	    END ELSE IF (OBS(I).VDS_ORI NE 0) AND (OBS(I).VDS_ORI NE 1) $
		    THEN BEGIN
	       MESSAGE = 'Tag VDS_ORI must be either 0 or 1'
	       GOTO, HANDLE_ERROR
	    ENDIF
;
	    IF DATATYPE(OBS(I).VDS_ACC,1) NE 'Byte' THEN BEGIN
	       MESSAGE = 'Tag VDS_ACC must be a byte value'
	       GOTO, HANDLE_ERROR
	    END ELSE IF N_ELEMENTS(OBS(I).VDS_ACC) NE 1 THEN BEGIN
	       MESSAGE = 'Tag VDS_ACC must be a scalar'
	       GOTO, HANDLE_ERROR
	    END ELSE IF (OBS(I).VDS_ACC NE 0) AND (OBS(I).VDS_ACC NE 1) $
		    THEN BEGIN
	       MESSAGE = 'Tag VDS_ACC must be either 0 or 1'
	       GOTO, HANDLE_ERROR
	    ENDIF
;
	    IF DATATYPE(OBS(I).GSET_ID,1) NE 'Integer' THEN BEGIN
	       MESSAGE = 'Tag GSET_ID must be an integer'
	       GOTO, HANDLE_ERROR
	    END ELSE IF N_ELEMENTS(OBS(I).GSET_ID) NE 1 THEN BEGIN
	       MESSAGE = 'Tag GSET_ID must be a scalar'
	       GOTO, HANDLE_ERROR
	    ENDIF
;
	    IF DATATYPE(OBS(I).DETECTOR,1) NE 'String' THEN BEGIN
	       MESSAGE = 'Tag DETECTOR must be a character string'
	       GOTO, HANDLE_ERROR
	    END ELSE IF N_ELEMENTS(OBS(I).DETECTOR) NE 1 THEN BEGIN
	       MESSAGE = 'Tag DETECTOR must be a scalar'
	       GOTO, HANDLE_ERROR
	    END ELSE IF STRLEN(OBS(I).DETECTOR) NE 1 THEN BEGIN
	       MESSAGE = 'Tag DETECTOR must be a single character'
	       GOTO, HANDLE_ERROR
	    END ELSE IF (OBS(I).DETECTOR NE 'N') AND (OBS(I).DETECTOR NE 'G') $
		    THEN BEGIN
	       MESSAGE = 'Tag DETECTOR must be either G or N'
	       GOTO, HANDLE_ERROR
	    ENDIF
;
	    IF DATATYPE(OBS(I).ZONE_ID,1) NE 'Integer' THEN BEGIN
	       MESSAGE = 'Tag ZONE_ID must be an integer'
	       GOTO, HANDLE_ERROR
	    END ELSE IF N_ELEMENTS(OBS(I).ZONE_ID) NE 1 THEN BEGIN
	       MESSAGE = 'Tag ZONE_ID must be a scalar'
	       GOTO, HANDLE_ERROR
	    ENDIF
;
	    IF DATATYPE(OBS(I).SLIT_NUM,1) NE 'Integer' THEN BEGIN
	       MESSAGE = 'Tag SLIT_NUM must be an integer'
	       GOTO, HANDLE_ERROR
	    END ELSE IF N_ELEMENTS(OBS(I).SLIT_NUM) NE 1 THEN BEGIN
	       MESSAGE = 'Tag SLIT_NUM must be a scalar'
	       GOTO, HANDLE_ERROR
	    ENDIF
;
	    IF DATATYPE(OBS(I).FILENAME,1) NE 'String' THEN BEGIN
	       MESSAGE = 'Tag FILENAME must be a character string'
	       GOTO, HANDLE_ERROR
	    END ELSE IF N_ELEMENTS(OBS(I).FILENAME) NE 1 THEN BEGIN
	       MESSAGE = 'Tag FILENAME must be a scalar'
	       GOTO, HANDLE_ERROR
	    END ELSE IF STRLEN(OBS(I).FILENAME) GT 15 THEN BEGIN
               MESSAGE = 'Tag FILENAME must be no more than 15 characters'
               GOTO, HANDLE_ERROR
	    ENDIF
;
	    IF DATATYPE(OBS(I).SEQVALID,1) NE 'Byte' THEN BEGIN
	       MESSAGE = 'Tag SEQVALID must be a short integer'
	       GOTO, HANDLE_ERROR
	    END ELSE IF N_ELEMENTS(OBS(I).SEQVALID) NE 1 THEN BEGIN
	       MESSAGE = 'Tag SEQVALID must be a scalar'
	       GOTO, HANDLE_ERROR
	    END ELSE IF (OBS(I).SEQVALID NE 0) AND (OBS(I).SEQVALID NE 1) $
		    THEN BEGIN
	       MESSAGE = 'Tag SEQVALID must be either 0 or 1'
	       GOTO, HANDLE_ERROR
	    ENDIF
;
	    IF DATATYPE(OBS(I).DATASRC,1) NE 'String' THEN BEGIN
               MESSAGE = 'Tag DATASRC must be a character string'
               GOTO, HANDLE_ERROR
	    END ELSE IF N_ELEMENTS(OBS(I).DATASRC) NE 1 THEN BEGIN
               MESSAGE = 'Tag DATASRC must be a scalar'
               GOTO, HANDLE_ERROR
	    END ELSE IF STRLEN(OBS(I).DATASRC) GT 1 THEN BEGIN
               MESSAGE = 'Tag DATASRC must be a single character'
               GOTO, HANDLE_ERROR
	    END ELSE IF (OBS(I).DATASRC NE 'Q') AND (OBS(I).DATASRC NE 'F') $
		    THEN BEGIN
	       MESSAGE = 'Tag DATASRC must be either Q or F'
	       GOTO, HANDLE_ERROR
	    ENDIF
;
;  Make sure that the raster ID and raster variation index match an entry in
;  the CDS raster database.
;
	    GET_RASTER, OBS(I).RAS_ID, OBS(I).RAS_VAR, TEMP
	    IF TEMP.RAS_ID LE -1 THEN CDS_MESSAGE, /CONTINUE, /ATTENTION, $
		    'Raster ID/variation ' + TRIM(OBS(I).RAS_ID) + ',' + $
		    TRIM(OBS(I).RAS_VAR) + ' not found in database'
;
;  Make sure that the data window ID number matches an entry in the main
;  database.
;
	    GET_DATAWIN, OBS(I).DW_ID, TEMP
	    IF TEMP.DW_ID LE 0 THEN CDS_MESSAGE, /CONTINUE, /ATTENTION,	$
		    'Data window ID ' + STRTRIM(OBS(I).DW_ID,2) + $
		    ' not found in database'
;
;  Make sure that the proposed entry is not already in the database.
;
	    GET_EXPER, OBS(I).PROG_NUM, OBS(I).SEQ_IND, TEMP
	    IF TEMP.SEQ_NUM GE 0 THEN BEGIN
	       MESSAGE = 'Program number ' + TRIM(OBS(I).PROG_NUM) +	$
		   ' and sequence index ' + TRIM(OBS(I).SEQ_IND) +	$
		   ' are already in the database'
	       GOTO, HANDLE_ERROR
	    ENDIF
;
	    GET_EXPER, OBS(I).SEQ_NUM, TEMP
	    IF TEMP.SEQ_NUM GE 0 THEN BEGIN
	       MESSAGE = 'Sequence number ' + TRIM(OBS(I).SEQ_NUM) +	$
		   ' is already in the database'
	       GOTO, HANDLE_ERROR
	    ENDIF
;
;  Make sure that the stop date is greater than the start date.
;
	    IF OBS(I).DATE_END LE OBS(I).DATE_OBS THEN BEGIN
	       MESSAGE = 'The stop date must be after the start date'
	       GOTO, HANDLE_ERROR
	    ENDIF
;
	    IF OBS(I).OBT_END LE OBS(I).OBT_TIME THEN BEGIN
	       MESSAGE = 'The OBT stop date must be after the start date'
	       GOTO, HANDLE_ERROR
	    ENDIF
;
;  Find the entry number in the main database.
;
	    DBOPEN, 'main'
	    TEMP = DBFIND('PROG_NUM='+TRIM(OBS(I).PROG_NUM),/SILENT)
	    IF TEMP(0) LT 0 THEN BEGIN
	       MESSAGE = 'Program number ' + TRIM(OBS(I).PROG_NUM) +	$
		       ' not found in the main database'
	       GOTO, HANDLE_ERROR
	    ENDIF
	    MAIN_PTR(I) = TEMP(0)
	ENDFOR
;
;  Make sure that none of the proposed entries use the same program number and
;  sequence index combination.
;
	PROG_NUM = OBS.PROG_NUM
	SEQ_IND  = OBS.SEQ_IND
	MAX_SEQ_IND = MAX(SEQ_IND)
	TEST = (MAX_SEQ_IND+1)*PROG_NUM + SEQ_IND	;Sort on both
	IF N_ELEMENTS(TEST) GT 1 THEN BEGIN
	    S = SORT(TEST)
	    PROG_NUM = PROG_NUM(S)
	    SEQ_IND = SEQ_IND(S)
	    W = WHERE((PROG_NUM(1:*) - PROG_NUM EQ 0) AND	$
		    (SEQ_IND(1:*) - SEQ_IND EQ 0), COUNT)
	    IF COUNT GT 0 THEN BEGIN
	       MESSAGE = 'Proposed entries use the same PROG_NUM and SEQ_IND'
	       GOTO, HANDLE_ERROR
	    ENDIF
	ENDIF
;
;  Make sure that none of the proposed entries use the same sequence number.
;
	SEQ_NUM = OBS.SEQ_NUM
	IF N_ELEMENTS(SEQ_NUM) GT 1 THEN BEGIN
	    SEQ_NUM = SEQ_NUM(SORT(SEQ_NUM))
	    W = WHERE(SEQ_NUM(1:*) - SEQ_NUM EQ 0, COUNT)
	    IF COUNT GT 0 THEN BEGIN
	       MESSAGE = 'Proposed entries use the same SEQ_NUM'
	       GOTO, HANDLE_ERROR
	    ENDIF
	ENDIF
;
;  Reformat the times to millisecond accuracy.  This is necessary so that
;  the times are written out in a controlled way.
;
	OBS.DATE_OBS = DOUBLE( STRING( OBS.DATE_OBS, FORMAT='(F15.3)' ))
	OBS.DATE_END = DOUBLE( STRING( OBS.DATE_END, FORMAT='(F15.3)' ))
	OBS.OBT_TIME = DOUBLE( STRING( OBS.OBT_TIME, FORMAT='(F15.3)' ))
	OBS.OBT_END  = DOUBLE( STRING( OBS.OBT_END,  FORMAT='(F15.3)' ))
;
;  Get the modification date.  Make a dummy array for the COMMENTS field.
;
	GET_UTC, DATE_MOD
	DATE_MOD = REPLICATE(UTC2TAI(DATE_MOD), N_ELEMENTS(OBS))
	COMMENTS = REPLICATE('N', N_ELEMENTS(OBS))
;
;  Lock the database for write.
;
	LOCK_DATABASE, 'experiment', LOCKFILE
;
;  Make sure once again that the proposed entry is not already in the database.
;
	FOR I = 0,N_ELEMENTS(OBS)-1 DO BEGIN
	    GET_EXPER, OBS(I).PROG_NUM, OBS(I).SEQ_IND, TEMP
	    IF TEMP.SEQ_NUM GE 0 THEN BEGIN
		MESSAGE = 'Program number ' + TRIM(OBS(I).PROG_NUM) +	$
			' and sequence index ' + TRIM(OBS(I).SEQ_IND) +	$
			' are already in the database'
		GOTO, HANDLE_ERROR
	    ENDIF
;
	    GET_EXPER, OBS(I).SEQ_NUM, TEMP
	    IF TEMP.SEQ_NUM GE 0 THEN BEGIN
		MESSAGE = 'Sequence number ' + TRIM(OBS(I).SEQ_NUM) +	$
			' is already in the database'
		GOTO, HANDLE_ERROR
	    ENDIF
	ENDFOR
;
;  Open the experiment as-run database for write access.
;
	DBOPEN, 'experiment', 1
;
;  Calculate the pointers to the new entries.
;
	N_ENTRIES = DB_INFO('entries','experiment')
	EXPER_PTR = LONG(N_ENTRIES + 1 + INDGEN(N_ELEMENTS(OBS)))
;
;  Add the entry.
;
	DBBUILD, MAIN_PTR, OBS.SEQ_NUM, OBS.OBS_SEQ, OBS.COUNT, OBS.SEQ_IND, $
		OBS.RAS_ID, OBS.RAS_VAR, OBS.EXPTIME, OBS.OBS_MODE, OBS.DW_ID,$
		OBS.OBT_TIME, OBS.OBT_END, OBS.XCEN, OBS.YCEN, OBS.ANGLE, $
		OBS.IXWIDTH, OBS.IYWIDTH, OBS.INS_X0, OBS.INS_Y0,	$
		OBS.INS_ROLL, OBS.SC_X0, OBS.SC_Y0, OBS.SC_ROLL, OBS.WAVEMIN, $
		OBS.WAVEMAX, STATUS=STATUS
	IF STATUS EQ 0 THEN BEGIN
	   MESSAGE = 'Write to experiment database was not successful'
	   GOTO, HANDLE_ERROR
	ENDIF
;
;  Do the same for experiment1.  (The database was so large, it had to split up
;  between three tables to get around the 25 item limit imposed by DBBUILD.
;
	DBOPEN, 'experiment1', 1
	N_ENTRIES = DB_INFO('entries','experiment1')
	EXPER1_PTR = LONG(N_ENTRIES + 1 + INDGEN(N_ELEMENTS(OBS)))
;
	DBBUILD, OBS.PROG_NUM, OBS.SEQ_NUM, EXPER_PTR, OBS.DATASRC,	$
		STATUS=STATUS
	IF STATUS EQ 0 THEN BEGIN
	   MESSAGE = 'Write to experiment1 database was not successful.  ' + $
		'Returned experiment to its previous state.'
	   PRIV = !PRIV  &  !PRIV = 3
	   DBDELETE, EXPER_PTR, 'experiment'
	   !PRIV = PRIV
	   GOTO, HANDLE_ERROR
	ENDIF
;
;  Do the same for experiment2.
;
	DBOPEN, 'experiment2', 1
	DBBUILD, OBS.PROG_NUM, OBS.SEQ_NUM, EXPER1_PTR, OBS.DATE_OBS,	$
		OBS.DATE_END, OBS.TRACKING, OBS.SER_ID, OBS.OPSLBITS,	$
		OBS.OPSRBITS, OBS.SLIT_POS, OBS.MIR_POS, OBS.EV_ENAB,	$
		OBS.COMP_ERR, OBS.VDS_PMCP, OBS.VDS_MODE, OBS.VDS_ORI,	$
		OBS.VDS_ACC, OBS.SEQVALID, COMMENTS, OBS.GSET_ID,	$
		OBS.DETECTOR, OBS.ZONE_ID, OBS.SLIT_NUM, OBS.FILENAME,	$
		DATE_MOD, STATUS=STATUS
	IF STATUS EQ 0 THEN BEGIN
	   MESSAGE = 'Write to experiment2 database was not successful.  ' + $
		'Returned experiment and experiment1 to their previous state.'
	   PRIV = !PRIV  &  !PRIV = 3
	   DBDELETE, EXPER_PTR,  'experiment'
	   DBDELETE, EXPER1_PTR, 'experiment1'
	   !PRIV = PRIV
	   GOTO, HANDLE_ERROR
	ENDIF
;
;  Signal success.
;
	RESULT = 1
	GOTO, FINISH
;
;  Error handling point.
;
HANDLE_ERROR:
	IF N_ELEMENTS(ERRMSG) NE 0 THEN ERRMSG = 'ADD_EXPER: ' + MESSAGE $
		ELSE MESSAGE, MESSAGE, /CONTINUE
;
;  Close and unlock the database, and return whether the routine was successful
;  or not.
;
FINISH:
	DBCLOSE
	IF N_ELEMENTS(LOCKFILE) NE 0 THEN UNLOCK_DATABASE, LOCKFILE
;
	RETURN, RESULT
	END
