	FUNCTION ADD_F_RASTER, DEF, ERRMSG=ERRMSG, RAS_ID=DBRAS_ID,	$
		CHECK_ONLY=CHECK_ONLY
;+
; Project     :	SOHO - CDS
;
; Name        :	ADD_F_RASTER()
;
; Purpose     :	Adds a fundamental raster definition to the database
;
; Explanation :	This procedure takes the fundamental parameters of a raster
;		definition and adds them to the database "raster".  The ID
;		number for the raster is assigned.
;
; Use         :	Result = ADD_F_RASTER( DEF )
;
;		IF NOT ADD_F_RASTER( DEF ) THEN ...
;
; Inputs      :	DEF = This is a structure of type "cds_f_raster" containing the
;		      following tags:
;
;			RAS_ID	 = Raster ID number.  Initially, this is
;				   zero.  It is updated by this routine to
;				   reflect the ID number in the database.
;			DETECTOR = Either "G" for GIS or "N" for NIS.
;			RAS_DESC = A short description of the raster, giving
;				   its purpose.
;			SLIT_NUM = The ID number for the slit to be used.
;			XSTEP	 = The step size in the X direction, in arcsec.
;			YSTEP	 = The step size in the Y direction, in arcsec.
;			NX	 = The number of exposure positions in X.
;			NY	 = The number of exposure positions in Y.
;
; Opt. Inputs :	None.
;
; Outputs     :	The result of the function is a logical value representing
;		whether or not the operation was successful, where 1 is
;		successful and 0 is unsuccessful.
;
;		The tag RAS_ID in the input structure is updated to reflect
;		the ID number assigned in the database.
;
; Opt. Outputs:	None.
;
; Keywords    :
;       ERRMSG    = If defined and passed, then any error messages will be
;                   returned to the user in this parameter rather than
;                   depending on the MESSAGE routine in IDL.  If no errors are
;                   encountered, then a null string is returned.  In order to
;                   use this feature, ERRMSG must be defined first, e.g.
;
;                       ERRMSG = ''
;                       Result = ADD_F_RASTER( ERRMSG=ERRMSG, ... )
;                       IF ERRMSG NE '' THEN ...
;
;       RAS_ID	  = If the definition matches one that is already in the
;                   database, then this keyword returns the ID of the
;                   matching fundamental raster.  Otherwise, a -1 is returned.
;
;	CHECK_ONLY= If this keyword is set, then the routine only checks to see
;		    if the proposed entry is valid, but does not attempt to
;		    place it in the database.
;
; Calls       :	DATATYPE, DBOPEN, DB_INFO, DBBUILD, DBCLOSE, NUM2STR, DBFIND,
;		DBMATCH
;
; Common      :	None.
;
; Restrictions:	Only this routine can be used to add fundamental raster
;		definitions to the database.  Modifying the database by hand
;		could corrupt its integrity.
;
;		The data types and sizes of the structure elements must match
;		the definitions in the database.  The string lengths must not
;		exceed the lengths defined in the database.
;
;		!PRIV must be 2 or greater to use this routine.
;
; Side effects:	Once a raster definition has been added to the database, it
;		must never be removed.
;
; Category    :	Planning, Databases.
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 4 April 1994
;
; Modified    :	Version 1.0, William Thompson, GSFC, 4 May 1994
;               Version 2, Liyun Wang, GSFC/ARC, September 21, 1994
;                  Added the keyword ERRMSG.
;               Version 3, Liyun Wang, GSFC/ARC, November 4, 1994
;                  CHK_F_RASTER is called to check the existence of the 
;                  given fundamental raster
;		Version 4, William Thompson, GSFC, 8 November 1994
;			Added keyword RAS_ID
;		Version 5, William Thompson, GSFC, 29 December 1994
;			Fixed bug when inputs are bytes.
;		Version 6, William Thompson, GSFC, 2 March 1995
;			Enforce uniqueness for RAS_DESC.
;			Added keyword CHECK_ONLY.
;		Version 7, William Thompson, GSFC, 14 March 1995
;			Changed DBFIND to DBMATCH for RAS_DESC
;
; Version     :	Version 7, 14 March 1995
;-
;
	ON_ERROR, 2
;
;  Make sure that the RAS_ID keyword is initialized, in case the program
;  exits before the CHK_F_RASTER routine is reached.
;
	DBRAS_ID = -1
;
;  Check the input parameters
;
        IF N_PARAMS() NE 1 THEN BEGIN
           MESSAGE = 'Syntax:  Result = ADD_F_RASTER(DEF)'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN BEGIN
              ERRMSG = MESSAGE 
              RETURN, 0
           ENDIF ELSE MESSAGE, MESSAGE
        ENDIF
;
;  Initialize RESULT to represent non-success.  If the routine is successful,
;  this value will be updated below.
;
	RESULT = 0
;
;  Make sure that the user has privilege to write into the database.
;
	IF !PRIV LT 2 THEN BEGIN $
           MESSAGE = '!PRIV must be 2 or greater to write into the database'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	ENDIF
;
;  Make sure that the raster ID number is less than or equal to zero.
;
	IF DEF.RAS_ID GT 0 THEN BEGIN
           IF N_ELEMENTS(ERRMSG) NE 0 THEN BEGIN
              ERRMSG = 'The raster ID number must be LE 0. '+$
                 'The final value is assigned by this routine'	
           ENDIF ELSE BEGIN
              MESSAGE, 'The raster ID number must be LE 0', /CONTINUE
              MESSAGE, 'The final value is assigned by this routine',$
                 /CONTINUE
           ENDELSE
           GOTO, FINISH
	ENDIF
;
;  Check each of the structure components to verify that it is of the correct
;  type and size.
;
	IF DATATYPE(DEF.DETECTOR,1) NE 'String' THEN BEGIN
           MESSAGE = 'Tag DETECTOR must be a character string'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	END ELSE IF N_ELEMENTS(DEF.DETECTOR) NE 1 THEN BEGIN
           MESSAGE = 'Tag DETECTOR must be a scalar'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	END ELSE IF STRLEN(DEF.DETECTOR) NE 1 THEN BEGIN
           MESSAGE = 'Tag DETECTOR must be a single character'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	END ELSE IF (DEF.DETECTOR NE 'G') AND (DEF.DETECTOR NE 'N') THEN BEGIN
           MESSAGE = 'Tag DETECTOR must be either "G" or "N"'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	ENDIF
;
	IF DATATYPE(DEF.RAS_DESC,1) NE 'String' THEN BEGIN
           MESSAGE = 'Tag RAS_DESC must be a character string'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	END ELSE IF N_ELEMENTS(DEF.RAS_DESC) NE 1 THEN BEGIN
           MESSAGE = 'Tag RAS_DESC must be a scalar'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	END ELSE IF STRLEN(DEF.RAS_DESC) GT 65 THEN BEGIN
           MESSAGE = 'Tag RAS_DESC must be no longer than 65 characters'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	ENDIF
;
	IF DATATYPE(DEF.SLIT_NUM,1) NE 'Integer' THEN BEGIN
           MESSAGE = 'Tag SLIT_NUM must be a short integer'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	END ELSE IF N_ELEMENTS(DEF.SLIT_NUM) NE 1 THEN BEGIN
           MESSAGE = 'Tag SLIT_NUM must be a scalar'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	ENDIF
;
	IF DATATYPE(DEF.XSTEP,1) NE 'Integer' THEN BEGIN
           MESSAGE = 'Tag XSTEP must be a short integer'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	END ELSE IF N_ELEMENTS(DEF.XSTEP) NE 1 THEN BEGIN
           MESSAGE = 'Tag XSTEP must be a scalar'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	ENDIF
;
	IF DATATYPE(DEF.YSTEP,1) NE 'Integer' THEN BEGIN
           MESSAGE = 'Tag YSTEP must be a short integer'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	END ELSE IF N_ELEMENTS(DEF.YSTEP) NE 1 THEN BEGIN
           MESSAGE = 'Tag YSTEP must be a scalar'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	ENDIF
;
	IF DATATYPE(DEF.NX,1) NE 'Integer' THEN BEGIN
           MESSAGE = 'Tag NX must be a short integer'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	END ELSE IF N_ELEMENTS(DEF.NX) NE 1 THEN BEGIN
           MESSAGE = 'Tag NX must be a scalar'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	ENDIF
;
	IF DATATYPE(DEF.NY,1) NE 'Integer' THEN BEGIN
           MESSAGE = 'Tag NY must be a short integer'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	END ELSE IF N_ELEMENTS(DEF.NY) NE 1 THEN BEGIN
           MESSAGE = 'Tag NY must be a scalar'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	ENDIF
;----------------------------------------------------------------------
;  Check the existence of the fundamental raster
;----------------------------------------------------------------------
        dbras_id = chk_f_raster(def)
        IF dbras_id GE 0 THEN BEGIN
           message = 'Data base entry for this fundamental raster already ' + $
              'exists. ID = '+num2str(dbras_id)
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG =  MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE
           GOTO, FINISH
        ENDIF
;
;  Open the primary raster database for write access.
;
	DBOPEN, 'raster', 1
;
;  First, check to see if there are any rasters with the same definition,
;  ignoring the description.
;
	ENTRY = DBFIND('DETECTOR=' + DEF.DETECTOR +		$
		',SLIT_NUM='	+ NUM2STR(DEF.SLIT_NUM) +	$
		',XSTEP='	+ NUM2STR(DEF.XSTEP) +		$
		',YSTEP='	+ NUM2STR(DEF.YSTEP) +		$
		',NX='		+ NUM2STR(DEF.NX) +		$
		',NY='		+ NUM2STR(DEF.NY), /SILENT)
	IF !ERR NE 0 THEN BEGIN
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              errmsg = 'Raster with same definition already exists '+ $
              'Raster number ' + NUM2STR(ENTRY(0)) $
           ELSE BEGIN
              MESSAGE, 'Raster with same definition already exists', $
                 /CONTINUE
              MESSAGE, 'Raster number ' + NUM2STR(ENTRY(0)), /CONTINUE
           ENDELSE
           GOTO, FINISH
	ENDIF
;
;  Also see if there are any with the same description.
;
	ENTRY = DBMATCH('RAS_DESC', STRTRIM(DEF.RAS_DESC,2), /FULLSTRING)
	IF ENTRY(0) GT 0 THEN BEGIN
           MESSAGE = 'Raster with same description already exists'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	ENDIF
;
;  Find the largest raster ID number currently in the database, and add one to
;  it.
;
	N_ENTRIES = DB_INFO('entries','raster')
	IF N_ENTRIES EQ 0 THEN RAS_ID = 1 ELSE BEGIN
		DBEXT, -1, 'ras_id', RAS_IDS
		RAS_ID = MAX(RAS_IDS) + 1
	ENDELSE
;
;  If the CHECK_ONLY keyword was set, then return now as if the routine were
;  successful.  However, don't modify the raster ID in the definition.
;
	IF KEYWORD_SET(CHECK_ONLY) THEN BEGIN
		RESULT = 1
		GOTO, FINISH
	ENDIF
;
;  Append the data to the opened database.
;
	DBBUILD, RAS_ID, DEF.DETECTOR, STRTRIM(DEF.RAS_DESC,2), DEF.SLIT_NUM, $
		DEF.XSTEP, DEF.YSTEP, DEF.NX, DEF.NY, STATUS=STATUS
	IF STATUS EQ 0 THEN BEGIN
           MESSAGE = 'Write to raster database was not successful'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	ENDIF
;
;  Update the raster ID number in the structure.  Signal success.
;
	DEF.RAS_ID = RAS_ID
	RESULT = 1
;
;  Close the database, and return whether the routine was successful or not.
;
FINISH:
	DBCLOSE
;
	RETURN, RESULT
	END
