	FUNCTION ADD_F_STUDY, DEF, ERRMSG=ERRMSG, STUDY_ID=DBSTUDY_ID,	$
		CHECK_ONLY=CHECK_ONLY
;+
; Project     :	SOHO - CDS
;
; Name        :	ADD_F_STUDY()
;
; Purpose     :	Adds a fundamental study definition to the database
;
; Explanation :	This procedure takes the fundamental parameters of a study
;		definition and adds them to the databases "study",
;		"study_title", and "study2".  The ID numbers for the study and
;		title are assigned.
;
; Use         :	Result = ADD_F_STUDY( DEF )
;
;		IF NOT ADD_F_STUDY( DEF ) THEN ...
;
; Inputs      :	DEF = This is an anonymous structure containing the following
;		      tags:
;
;			STUDY_ID    = Study ID number.  Initially, this is -1.
;				      It is updated by this routine to reflect
;				      the ID number in the database.
;			TITLE_ID    = Study title ID number.  Initially, this
;				      is -1.  It is updated by this routine to
;				      reflect the ID number in the database.
;			OBS_PROG    = Name of the study
;			TITLE	    = A descriptive title for the study
;			CATEGORY    = The study category, either "T"est,
;				      "S"cience or "C"alibration.
;			N_RASTER_DEF= The number of rasters in the definition.
;			RASTERS	    = A array listing the rasters to be used
;				      during the study.
;
;		      The raster descriptions themselves are structures, of
;		      type "cds_f_st_ras", with the following tags:
;
;			RAS_ID	    = Raster ID number, pointing to the
;				      definition of the raster.
;			POINTING    = How to handle the pointing.  Valid
;				      entries are:
;
;					 1 = No pointing, supply in plan
;					 0 = Absolute pointing
;					-1 = Offset value from first raster
;
;			INS_X	    = Together with INS_Y, the pointing to use
;				      when user-supplied values are not
;				      allowed.  Only valid when POINTING=0
;				      (absolute) or POINTING=-1 (relative to
;				      1st raster).
;			INS_Y	    = ...
;
;		      If N_RASTER_DEF is 0, then the RASTERS tag can be
;		      omitted.  However, in that case, an additional tag must
;		      be added,
;
;			VAR_POINT   = Either Y or N depending on whether the
;				      study has variable pointing or fixed
;				      pointing.
;
; Opt. Inputs :	None.
;
; Outputs     :	The result of the function is a logical value representing
;		whether or not the operation was successful, where 1 is
;		successful and 0 is unsuccessful.
;
;		The tags STUDY_ID and TITLE_ID in the input structure are
;		updated to reflect the ID numbers assigned in the database.
;
; Opt. Outputs:	None.
;
; Keywords    :	
;       ERRMSG    = If defined and passed, then any error messages will be
;                   returned to the user in this parameter rather than
;                   depending on the MESSAGE routine in IDL.  If no errors are
;                   encountered, then a null string is returned.  In order to
;                   use this feature, ERRMSG must be defined first, e.g.
;
;                       ERRMSG = ''
;                       Result = ADD_F_STUDY( ERRMSG=ERRMSG, ... )
;                       IF ERRMSG NE '' THEN ...
;
;       STUDY_ID  = If the definition matches one that is already in the
;                   database, then this keyword returns the ID of the
;                   matching study.  Otherwise, a -1 is returned.
;
;	CHECK_ONLY= If this keyword is set, then the routine only checks to see
;		    if the proposed entry is valid, but does not attempt to
;		    place it in the database.
;
; Calls       :	DATATYPE, DBOPEN, DB_INFO, DBBUILD, DBCLOSE, DBFIND, DBMATCH
;
; Common      :	None.
;
; Restrictions:	Only this routine can be used to add fundamental study
;		definitions to the database.  Modifying the database by hand
;		could corrupt its integrity.
;
;		The data types and sizes of the structure elements must match
;		the definitions in the database.  The string lengths must not
;		exceed the lengths defined in the database.
;
;		!PRIV must be 2 or greater to use this routine.
;
; Side effects:	Once a study definition has been added to the database, it
;		must never be removed.
;
; Category    :	Planning, Databases.
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 28 April 1994
;
; Modified    :	Version 1, William Thompson, GSFC, 3 August 1994
;               Version 2, Liyun Wang, GSFC/ARC, September 21, 1994
;                  Added the keyword ERRMSG.
;               Version 3, Liyun Wang, GSFC/ARC, November 4, 1994
;                  CHK_F_STUDY is called to check the existence of the 
;                  given fundamental study
;               Version 3.1, Zarro GSFC/ARC, November 7, 1994
;                  Added DBSTUDY_ID
;		Version 4, William Thompson, GSFC/ARC, 8 November 1994
;			Renamed DBSTUDY_ID keyword to STUDY_ID.
;			Changed how it is determined.
;			Fixed bug with duplicate name check.
;		Version 5, William Thompson, GSFC, 21 November 1994
;			Removed item N_RASTERS0
;			Added item N_RASTER_DEF
;		Version 6, William Thompson, GSFC, 29 December 1994
;			Fixed bug when inputs are bytes.
;		Version 7, William Thompson, GSFC, 10 February 1995
;			Changed N_POINTINGS to VAR_POINT.
;			Changed definition of POINTING.
;			Removed VAR_RASTERS.
;		Version 8, William Thompson, GSFC, 2 March 1995
;			Enforce uniqueness for TITLE.
;			Added keyword CHECK_ONLY.
;		Version 9, William Thompson, GSFC, 14 March 1995
;			Used DBMATCH instead of DBFIND for TITLE.
;		Version 10, William Thompson, GSFC, 12 May 1995
;			Modified to split off separate study_title database
;		Version 11, William Thompson, GSFC, 6 October 1995
;			Modified to allow for cases with N_RASTER_DEF=0.
;
; Version     :	Version 11, 6 October 1995
;-
;
	ON_ERROR, 2
;
;  Make sure that the STUDY_ID keyword is initialized, in case the program
;  exits before the CHK_F_STUDY routine is reached.
;
	DBSTUDY_ID = -1
;
;  Initialize RESULT to represent non-success.  If the routine is successful,
;  this value will be updated below.
;
	RESULT = 0
;
;  Check the input parameters
;
        IF N_PARAMS() NE 1 THEN BEGIN
           MESSAGE = 'Syntax:  Result = ADD_F_STUDY(DEF)'
	   GOTO, HANDLE_ERROR
        ENDIF
;
;  Make sure that the user has privilege to write into the database.
;
	IF !PRIV LT 2 THEN BEGIN $
           MESSAGE = '!PRIV must be 2 or greater to write into the database'
           GOTO, HANDLE_ERROR
	ENDIF
;
;  Make sure that the study ID number is negative.
;
	IF DEF.STUDY_ID GE 0 THEN BEGIN
           MESSAGE = 'The study ID number must be negative.  ' +	$
		   'The final value is assigned by this routine'
           GOTO, HANDLE_ERROR
	ENDIF
;
;  Make sure that the title ID number is negative.
;
	IF DEF.TITLE_ID GE 0 THEN BEGIN
	   MESSAGE = 'The title ID number must be negative.  ' +	$
		   'The final value is assigned by this routine'
	   GOTO, HANDLE_ERROR
	ENDIF
;
;  Check each of the structure components to verify that it is of the correct
;  type and size.
;
	IF DATATYPE(DEF.OBS_PROG,1) NE 'String' THEN BEGIN
           MESSAGE = 'Tag OBS_PROG must be a character string'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(DEF.OBS_PROG) NE 1 THEN BEGIN
           MESSAGE = 'Tag OBS_PROG must be a scalar'
           GOTO, HANDLE_ERROR
	END ELSE IF STRLEN(DEF.OBS_PROG) GT 8 THEN BEGIN
           MESSAGE = 'Tag OBS_PROG must be no longer than 8 characters'
           GOTO, HANDLE_ERROR
	END ELSE IF DEF.OBS_PROG NE STRUPCASE(DEF.OBS_PROG) THEN BEGIN
           MESSAGE = 'Tag OBS_PROG must be uppercase'
           GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(DEF.TITLE,1) NE 'String' THEN BEGIN
           MESSAGE = 'Tag TITLE must be a character string'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(DEF.TITLE) NE 1 THEN BEGIN
           MESSAGE = 'Tag TITLE must be a scalar'
           GOTO, HANDLE_ERROR
	END ELSE IF STRLEN(DEF.TITLE) GT 80 THEN BEGIN
           MESSAGE = 'Tag TITLE must be no longer than 80 characters'
           GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(DEF.CATEGORY,1) NE 'String' THEN BEGIN
           MESSAGE = 'Tag CATEGORY must be a character string'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(DEF.CATEGORY) NE 1 THEN BEGIN
           MESSAGE = 'Tag CATEGORY must be a scalar'
           GOTO, HANDLE_ERROR
	END ELSE IF STRLEN(DEF.CATEGORY) NE 1 THEN BEGIN
           MESSAGE = 'Tag CATEGORY must be a single character'
           GOTO, HANDLE_ERROR
	END ELSE IF (DEF.CATEGORY NE 'T') AND (DEF.CATEGORY NE 'S') AND	$
			(DEF.CATEGORY NE 'C') THEN BEGIN
           MESSAGE = 'Tag CATEGORY must be "T", "S", or "C"'
           GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(DEF.N_RASTER_DEF,1) NE 'Integer' THEN BEGIN
           MESSAGE = 'Tag N_RASTER_DEF must be a short integer'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(DEF.N_RASTER_DEF) NE 1 THEN BEGIN
           MESSAGE = 'Tag N_RASTER_DEF must be a scalar'
           GOTO, HANDLE_ERROR
	ENDIF
;
;  If N_RASTER_DEF is not zero, then some additional checks are made.
;
	IF DEF.N_RASTER_DEF NE 0 THEN BEGIN
	    IF DEF.N_RASTER_DEF NE N_ELEMENTS(DEF.RASTERS) THEN BEGIN
		MESSAGE = 'Tag N_RASTER_DEF not consistent with tag RASTERS'
		GOTO, HANDLE_ERROR
	    ENDIF
;
;  Extract the first instance of the structure in the RASTERS tag, and check
;  it's tags.
;
	    TEST = (DEF.RASTERS)(0)
;
	    IF DATATYPE(TEST.RAS_ID,1) NE 'Integer' THEN BEGIN
	       MESSAGE = 'Tag RASTERS.RAS_ID must be a short integer'
	       GOTO, HANDLE_ERROR
	    END ELSE IF N_ELEMENTS(TEST.RAS_ID) NE 1 THEN BEGIN
	       MESSAGE = 'Tag RASTERS.RAS_ID must be a scalar'
	       GOTO, HANDLE_ERROR
	    ENDIF
;
	    IF DATATYPE(TEST.POINTING,1) NE 'Integer' THEN BEGIN
	       MESSAGE = 'Tag RASTERS.POINTING must be a short integer'
	       GOTO, HANDLE_ERROR
	    END ELSE IF N_ELEMENTS(TEST.POINTING) NE 1 THEN BEGIN
	       MESSAGE = 'Tag RASTERS.POINTING must be a scalar'
	       GOTO, HANDLE_ERROR
	    ENDIF
;
	    IF DATATYPE(TEST.INS_X,1) NE 'Float' THEN BEGIN
	       MESSAGE = 'Tag RASTERS.INS_X must be a floating point number'
	       GOTO, HANDLE_ERROR
	    END ELSE IF N_ELEMENTS(TEST.INS_X) NE 1 THEN BEGIN
	       MESSAGE = 'Tag RASTERS.INS_X must be a scalar'
	       GOTO, HANDLE_ERROR
	    ENDIF
;
	    IF DATATYPE(TEST.INS_Y,1) NE 'Float' THEN BEGIN
	       MESSAGE = 'Tag RASTERS.INS_Y must be a floating point number'
	       GOTO, HANDLE_ERROR
	    END ELSE IF N_ELEMENTS(TEST.INS_Y) NE 1 THEN BEGIN
	       MESSAGE = 'Tag RASTERS.INS_Y must be a scalar'
	       GOTO, HANDLE_ERROR
	    ENDIF
;
;  Make sure that the POINTING parameters are correct.
;
	    PMIN = MIN(DEF.RASTERS.POINTING, MAX=PMAX)
	    IF (PMIN LT -1) OR (PMAX GT 1) THEN BEGIN
	       MESSAGE = 'Tag RASTERS.POINTING must have values between -1 and 1'
	       GOTO, HANDLE_ERROR
	    ENDIF
;
;  The first value of pointing cannot be -1.
;
	    IF DEF.RASTERS(0).POINTING EQ -1 THEN BEGIN
	       MESSAGE = 'The first pointing cannot be relative'
	       GOTO, HANDLE_ERROR
	    ENDIF
;
;  Determine VAR_POINT based on the values of POINTING.
;
	    IF PMAX EQ 1 THEN VAR_POINT = 'Y' ELSE VAR_POINT = 'N'
;
;  Make sure that all the raster IDs correspond to legitimate entries in the
;  raster table.  Save the entry numbers for later.
;
	    DBOPEN, 'raster'
	    RAS_ID = DEF.RASTERS.RAS_ID
	    RASTER_PTR = REPLICATE(0L, N_ELEMENTS(RAS_ID))
	    FOR I = 0,N_ELEMENTS(RAS_ID)-1 DO BEGIN
		TEST = DBFIND('RAS_ID=' + STRTRIM(LONG(RAS_ID(I)),2), /SILENT)
		IF !ERR EQ 0 THEN BEGIN
		   MESSAGE = 'Raster ID does not exist in database'
		   GOTO, HANDLE_ERROR
		ENDIF
		RASTER_PTR(I) = TEST(0)
	    ENDFOR
;
;  Otherwise, if N_RASTER_DEF is zero, then check for a VAR_POINT tag.
;
	END ELSE BEGIN
	    IF DATATYPE(DEF.VAR_POINT,1) NE 'String' THEN BEGIN
	       MESSAGE = 'Tag VAR_POINT must be a character string'
	       GOTO, HANDLE_ERROR
	    END ELSE IF N_ELEMENTS(DEF.VAR_POINT) NE 1 THEN BEGIN
	       MESSAGE = 'Tag VAR_POINT must be a scalar'
	       GOTO, HANDLE_ERROR
	    END ELSE IF STRLEN(DEF.VAR_POINT) NE 1 THEN BEGIN
	       MESSAGE = 'Tag VAR_POINT must be a single character'
	       GOTO, HANDLE_ERROR
	    END ELSE IF (DEF.VAR_POINT NE 'Y') AND (DEF.VAR_POINT NE 'N') $
		    THEN BEGIN
	       MESSAGE = 'Tag VAR_POINT must be either "Y" or "N"'
	       GOTO, HANDLE_ERROR
	    ENDIF
;
	    VAR_POINT = DEF.VAR_POINT
	ENDELSE
;
;  Check the existence of the fundamental study.
;
	DBSTUDY_ID = CHK_F_STUDY(DEF)
	IF DBSTUDY_ID GE 0 THEN BEGIN
	   MESSAGE = 'Data base entry for this fundamental study already ' + $
	      'exists, ID = ' + NUM2STR(DBSTUDY_ID)
	   GOTO, HANDLE_ERROR
	ENDIF
;
;  Check the existence of the study name or title.
;
	TEST = CHK_T_STUDY(DEF, /EITHER)
	IF TEST GE 0 THEN BEGIN
	   MESSAGE = 'Data base entry for this study name or title ' + $
	      'already exists, title ID = ' + NUM2STR(TEST)
	   GOTO, HANDLE_ERROR
	ENDIF
;
;  Open the primary study database for write access.
;
	DBOPEN, 'study', 1
;
;  Find the largest study ID number currently in the database, and add one to
;  it.  If the database is empty, then set the STUDY_ID to zero.  Find the
;  number of entries and add one to it--that will be the pointer for the new
;  entry.
;
	N_ENTRIES = DB_INFO('entries','study')
	STUDY_PTR = N_ENTRIES + 1
	IF N_ENTRIES EQ 0 THEN STUDY_ID = 0 ELSE BEGIN
		DBEXT, -1, 'study_id', STUDY_IDS
		STUDY_ID = MAX(STUDY_IDS) + 1
	ENDELSE
;
;  If the CHECK_ONLY keyword was set, then return now as if the routine were
;  successful.  However, don't modify the study ID or title ID in the
;  definition.
;
	IF KEYWORD_SET(CHECK_ONLY) THEN BEGIN
		RESULT = 1
		GOTO, FINISH
	ENDIF
;
;  Append the data to the opened database.
;
	DBBUILD, STUDY_ID, VAR_POINT, DEF.N_RASTER_DEF, STATUS=STATUS
	IF STATUS EQ 0 THEN BEGIN
	   MESSAGE = 'Write to study database was not successful'
	   GOTO, HANDLE_ERROR
	ENDIF
;
;  Open the study title data for write access.
;
	DBOPEN, 'study_title', 1
;
;  Find the largest study title ID number currently in the database, and add
;  one to it.  If the database is empty, then set the TITLE_ID to one.  Find
;  the number of entries and add one to it--that will be the pointer for the
;  new entry.
;
	N_ENTRIES = DB_INFO('entries','study_title')
	TITLE_PTR = N_ENTRIES + 1
	IF N_ENTRIES EQ 0 THEN TITLE_ID = 1 ELSE BEGIN
		DBEXT, -1, 'title_id', TITLE_IDS
		TITLE_ID = MAX(TITLE_IDS) + 1
	ENDELSE
;
;  Append the data to the opened database.
;
	DBBUILD, LONG(STUDY_PTR), STUDY_ID, TITLE_ID,	$
		STRTRIM(DEF.OBS_PROG,2), STRTRIM(DEF.TITLE,2), DEF.CATEGORY, $
		STATUS=STATUS
;
;  If the second write was unsuccessful, then delete the entry inserted above
;  into "study".
;
	IF STATUS EQ 0 THEN BEGIN
	   MESSAGE = 'Write to study_title database was not successful.  ' + $
	      'Returned study to its previous state'
	   PRIV = !PRIV  &  !PRIV = 3
	   DBDELETE, STUDY_PTR, 'study'
	   !PRIV = PRIV
	   GOTO, HANDLE_ERROR
	ENDIF
;
;  If N_RASTER_DEF is nonzero, then open the secondary study database, and
;  append the data in that database.
;
	IF DEF.N_RASTER_DEF GT 0 THEN BEGIN
	    DBOPEN, 'study2', 1
	    N_RASTERS = N_ELEMENTS(DEF.RASTERS)
	    IND = INDGEN(N_RASTERS) + 1
	    PTR = REPLICATE(LONG(STUDY_PTR),N_RASTERS)
	    STUDY_IDS = REPLICATE(STUDY_ID,N_RASTERS)
	    DBBUILD, STATUS=STATUS, PTR, STUDY_IDS, IND, RASTER_PTR,	$
		DEF.RASTERS.RAS_ID, DEF.RASTERS.POINTING, DEF.RASTERS.INS_X, $
		DEF.RASTERS.INS_Y
;
;  If the third write was unsuccessful, then delete the entries inserted above
;  into "study" and "study_title".
;
	    IF STATUS EQ 0 THEN BEGIN
	       MESSAGE = 'Write to study2 database was not successful.  ' + $
	          'Returned study and study_title to their previous states'
	       PRIV = !PRIV  &  !PRIV = 3
	       DBDELETE, STUDY_PTR, 'study'
	       DBDELETE, TITLE_PTR, 'study_title'
	       !PRIV = PRIV
	       GOTO, HANDLE_ERROR
	    ENDIF
	ENDIF
;
;  Update the study and title ID numbers in the structure.  Signal success.
;
	DEF.STUDY_ID = STUDY_ID
	DEF.TITLE_ID = TITLE_ID
	RESULT = 1
	GOTO, FINISH
;
;  Error handling point.
;
HANDLE_ERROR:
	IF N_ELEMENTS(ERRMSG) NE 0 THEN ERRMSG = 'ADD_F_STUDY: ' + MESSAGE $
		ELSE MESSAGE, MESSAGE, /CONTINUE
;
;  Close the database, and return whether the routine was successful or not.
;
FINISH:
	DBCLOSE
;
	RETURN, RESULT
	END
