	FUNCTION ADD_LINELIST, DEF, ERRMSG=ERRMSG, LL_ID=DBLL_ID,	$
		CHECK_ONLY=CHECK_ONLY
;+
; Project     :	SOHO - CDS
;
; Name        :	ADD_LINELIST()
;
; Purpose     :	Adds a line list definition to the database
;
; Explanation :	This procedure takes a line list definition and adds it to line
;		list databases "linelist" and "linelist2".  The ID number for
;		the linelist is assigned.
;
; Use         :	Result = ADD_LINELIST( DEF )
;
;		IF NOT ADD_LINELIST( DEF ) THEN ...
;
; Inputs      :	DEF = This is an anonymous structure containing the following
;		      tags:
;
;			LL_ID	 = Line list ID number.  Initially, this is
;				   zero.  It is updated by this routine to
;				   reflect the ID number in the database.
;			DETECTOR = Either "G" for GIS or "N" for NIS.
;			LL_DESC	 = A description of the line list, e.g.
;				   "Temperature sensitive line pairs".  Can be
;				   up to 65 characters.
;			LINES	 = An array containing the line definitions.
;
;		      The line definitions themselves are structures, of type
;		      "cds_lines", with the following tags:
;
;			LINENAME = Line name, e.g. "He II 304".  Can be up to
;				   12 characters.
;			WAVELNTH = The wavelength of the line, in Angstroms.
;			GR_ORDER = The grating order
;			CEN_PIX	 = The central detector pixel.
;			WAVEBAND = The wavelength band, 1 or 2 for NIS, 1-4 for
;				   GIS.
;
; Opt. Inputs :	None.
;
; Outputs     :	The result of the function is a logical value representing
;		whether or not the operation was successful, where 1 is
;		successful and 0 is unsuccessful.
;
;		The tag LL_ID in the input structure is updated to reflect
;		the ID number assigned in the database.
;
; Opt. Outputs:	None.
;
; Keywords    :
;       ERRMSG    = If defined and passed, then any error messages will be
;                   returned to the user in this parameter rather than
;                   depending on the MESSAGE routine in IDL.  If no errors are
;                   encountered, then a null string is returned.  In order to
;                   use this feature, ERRMSG must be defined first, e.g.
;
;                       ERRMSG = ''
;                       Result = ADD_LINELIST( ERRMSG=ERRMSG, ... )
;                       IF ERRMSG NE '' THEN ...
;
;       LL_ID	  = If the definition matches one that is already in the
;                   database, then this keyword returns the ID of the
;                   matching linelist.  Otherwise, a -1 is returned.
;
;	CHECK_ONLY= If this keyword is set, then the routine only checks to see
;		    if the proposed entry is valid, but does not attempt to
;		    place it in the database.
;
; Calls       :	DATATYPE, DBOPEN, DB_INFO, DBBUILD, DBCLOSE, DBFIND, DBMATCH
;
; Common      :	None.
;
; Restrictions:	Only this routine can be used to add line list definitions to
;		the database.  Modifying the database by hand could corrupt its
;		integrity.
;
;		The data types and sizes of the structure elements must match
;		the definitions in the database.  The string lengths must not
;		exceed the lengths defined in the database.
;
;		!PRIV must be 2 or greater to use this routine.
;
; Side effects:	Once a line list definition has been added to the database, it
;		must never be removed.
;
; Category    :	Planning, Databases.
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 30 March 1994
;
; Modified    :	Version 1.0, William Thompson, GSFC, 2 June 1994
;               Version 2, Liyun Wang, GSFC/ARC, September 21, 1994
;                  Added the keyword ERRMSG.
;               Version 3, Liyun Wang, GSFC/ARC, November 4, 1994
;                  CHK_LINELIST is called to check the existence of the 
;                  given line list
;		Version 4, William Thompson, GSFC, 8 November 1994
;			Added keyword LL_ID
;		Version 5, William Thompson, GSFC, 2 March 1995
;			Enforce uniqueness for LL_DESC.
;			Added keyword CHECK_ONLY.
;		Version 6, William Thompson, GSFC, 14 March 1995
;			Changed DBFIND to DBMATCH for LL_DESC.
;		Version 7, William Thompson, GSFC, 15 March 1995
;			Removed LL_TITLE.  Changed so that LL_DESC only needs
;			to be unique for a given detector.
;		Version 8, William Thompson, GSFC, 16 May 1997
;			Allow for possibility of CHK_LINELIST returning an
;			array.
;
; Version     :	Version 8, 16 May 1997
;-
;
	ON_ERROR, 2
;
;  Make sure that the LL_ID keyword is initialized, in case the program
;  exits before the CHK_LINELIST routine is reached.
;
	DBLL_ID = -1
;
;  Check the input parameters
;
        IF N_PARAMS() NE 1 THEN BEGIN
           MESSAGE = 'Syntax:  Result = ADD_LINELIST(DEF)'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN BEGIN
              ERRMSG = MESSAGE
              RETURN, 0
           ENDIF ELSE MESSAGE, MESSAGE
        ENDIF
;
;  Initialize RESULT to represent non-success.  If the routine is successful,
;  this value will be updated below.
;
	RESULT = 0
;
;  Make sure that the user has privilege to write into the database.
;
	IF !PRIV LT 2 THEN BEGIN $
           MESSAGE = '!PRIV must be 2 or greater to write into the database'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	ENDIF
;
;  Make sure that the line list ID number is less than or equal to zero.
;
	IF DEF.LL_ID GT 0 THEN BEGIN
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = 'The line list ID number must be LE 0. '+$
              'The final value is assigned by this routine' $
           ELSE BEGIN
              MESSAGE, 'The line list ID number must be LE 0', /CONTINUE
              MESSAGE, 'The final value is assigned by this routine',	$
                 /CONTINUE
           ENDELSE
           GOTO, FINISH
	ENDIF
;
;  Check each of the structure components to verify that it is of the correct
;  type and size.  Start off with the parameters that will be placed in the
;  "linelist" database.
;
	IF DATATYPE(DEF.DETECTOR,1) NE 'String' THEN BEGIN
           MESSAGE = 'Tag DETECTOR must be a character string'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	END ELSE IF N_ELEMENTS(DEF.DETECTOR) NE 1 THEN BEGIN
           MESSAGE = 'Tag DETECTOR must be a scalar'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	END ELSE IF STRLEN(DEF.DETECTOR) NE 1 THEN BEGIN
           MESSAGE = 'Tag DETECTOR must be a single character'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	END ELSE IF (DEF.DETECTOR NE 'G') AND (DEF.DETECTOR NE 'N') THEN BEGIN
           MESSAGE = 'Tag DETECTOR must be either "G" or "N"'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	ENDIF
;
	IF DATATYPE(DEF.LL_DESC,1) NE 'String' THEN BEGIN
           MESSAGE = 'Tag LL_DESC must be a character string'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	END ELSE IF N_ELEMENTS(DEF.LL_DESC) NE 1 THEN BEGIN
           MESSAGE = 'Tag LL_DESC must be a scalar'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	END ELSE IF STRLEN(DEF.LL_DESC) GT 65 THEN BEGIN
           MESSAGE = 'Tag LL_DESC must be 65 characters or less'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	ENDIF
;
;  Extract the first instance of the structure in the LINES tag, and check it's
;  tags.
;
	TEST = (DEF.LINES)(0)
;
	IF DATATYPE(TEST.LINENAME,1) NE 'String' THEN BEGIN
           MESSAGE = 'Tag LINES.LINENAME must be a character string'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	END ELSE IF N_ELEMENTS(TEST.LINENAME) NE 1 THEN BEGIN
           MESSAGE = 'Tag LINES.LINENAME must be a scalar'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	END ELSE IF STRLEN(TEST.LINENAME) GT 12 THEN BEGIN
           MESSAGE = 'Tag LINES.LINENAME must be 12 characters or less'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	ENDIF
;
	IF DATATYPE(TEST.WAVELNTH,1) NE 'Float' THEN BEGIN
           MESSAGE = 'Tag LINES.WAVELNTH must be a floating point number'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	END ELSE IF N_ELEMENTS(TEST.WAVELNTH) NE 1 THEN BEGIN
           MESSAGE = 'Tag LINES.WAVELNTH must be a scalar'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	ENDIF
;
	IF DATATYPE(TEST.GR_ORDER,1) NE 'Integer' THEN BEGIN
           MESSAGE = 'Tag LINES.GR_ORDER must be a short integer'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	END ELSE IF N_ELEMENTS(TEST.GR_ORDER) NE 1 THEN BEGIN
           MESSAGE = 'Tag LINES.GR_ORDER must be a scalar'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	ENDIF
;
	IF DATATYPE(TEST.CEN_PIX,1) NE 'Integer' THEN BEGIN
           MESSAGE = 'Tag LINES.CEN_PIX must be a short integer'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	END ELSE IF N_ELEMENTS(TEST.CEN_PIX) NE 1 THEN BEGIN
           MESSAGE = 'Tag LINES.CEN_PIX must be a scalar'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	ENDIF
;
	IF DATATYPE(TEST.WAVEBAND,1) NE 'Integer' THEN BEGIN
           MESSAGE = 'Tag LINES.WAVEBAND must be a short integer'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	END ELSE IF N_ELEMENTS(TEST.WAVEBAND) NE 1 THEN BEGIN
           MESSAGE = 'Tag LINES.WAVEBAND must be a scalar'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	ENDIF

;----------------------------------------------------------------------
;  Check the existence of the line list
;----------------------------------------------------------------------
        dbll_id = chk_linelist(def)
	dbll_id = dbll_id(0)
        IF dbll_id GE 0 THEN BEGIN
           message = 'Data base entry for this line list already ' + $
              'exists. ID = '+num2str(dbll_id)
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG =  MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE
           GOTO, FINISH
        ENDIF
;
;  Open the primary line list database for write access.
;
	DBOPEN, 'linelist', 1
;
;  First, check to see if there are any lines with the same description.
;
	ENTRIES = DBFIND('DETECTOR='+DEF.DETECTOR, /SILENT)
	ENTRY = DBMATCH('LL_DESC', STRTRIM(DEF.LL_DESC,2), ENTRIES,	$
		/FULLSTRING)
	IF ENTRY(0) GT 0 THEN BEGIN
           MESSAGE = 'Line list with same description already exists'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	ENDIF
;
;  Find the largest line list ID number currently in the database, and add one
;  to it.  Also calculate the pointer to the new entry.
;
	N_ENTRIES = DB_INFO('entries','linelist')
	IF N_ENTRIES EQ 0 THEN LL_ID = 1 ELSE BEGIN
		DBEXT, -1, 'll_id', LL_IDS
		LL_ID = MAX(LL_IDS) + 1
	ENDELSE
	LINELIST_PTR = LONG(N_ENTRIES + 1)
;
;  If the CHECK_ONLY keyword was set, then return now as if the routine were
;  successful.  However, don't modify the linelist ID in the definition.
;
	IF KEYWORD_SET(CHECK_ONLY) THEN BEGIN
		RESULT = 1
		GOTO, FINISH
	ENDIF
;
;  Append the data to the opened database.
;
	N_LINES = FIX(N_ELEMENTS(DEF.LINES))
	DBBUILD, LL_ID, DEF.DETECTOR, STRTRIM(DEF.LL_DESC,2), N_LINES,	$
		STATUS=STATUS
	IF STATUS EQ 0 THEN BEGIN
           MESSAGE = 'Write to linelist database was not successful'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	ENDIF
;
;  Open the secondary line list database, and append the data in that database.
;
	DBOPEN, 'linelist2', 1
	IND = INDGEN(N_LINES) + 1
	PTR = REPLICATE(LINELIST_PTR, N_LINES)
	LL_IDS = REPLICATE(LL_ID, N_LINES)
	DBBUILD, PTR, LL_IDS, DEF.LINES.LINENAME, IND, DEF.LINES.WAVELNTH, $
		DEF.LINES.GR_ORDER, DEF.LINES.CEN_PIX, DEF.LINES.WAVEBAND, $
		STATUS=STATUS
;
;  If the second write was unsuccessful, then delete the entry inserted above
;  into "linelist".
;
	IF STATUS EQ 0 THEN BEGIN
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = 'Write to linelist2 database was not successful. '+$
              'Returning linelist to its previous state'$
           ELSE BEGIN
              MESSAGE, 'Write to linelist2 database was not successful', $
                 /CONTINUE
              MESSAGE, 'Returning linelist to its previous state', /CONTINUE
           ENDELSE
           PRIV = !PRIV  &  !PRIV = 3
           DBDELETE, LINELIST_PTR, 'linelist'
           !PRIV = PRIV
           GOTO, FINISH
	ENDIF
;
;  Update the line list ID number in the structure.  Signal success.
;
	DEF.LL_ID = LL_ID
	RESULT = 1
;
;  Close the database, and return whether the routine was successful or not.
;
FINISH:
	DBCLOSE
;
	RETURN, RESULT
	END
