	FUNCTION ADD_MAIN, OBS, ERRMSG=ERRMSG
;+
; Project     :	SOHO - CDS
;
; Name        :	ADD_MAIN()
;
; Purpose     :	Adds a CDS main entry to the as-run database
;
; Explanation :	This procedure takes a CDS study-level (main) as-run entry and
;		adds it to the database "main".  This database contains
;		a series of such entries, making a historical list.
;
; Use         :	Result = ADD_MAIN( OBS )
;
;		IF NOT ADD_MAIN( OBS ) THEN ...
;
; Inputs      :	OBS = This is an anonymous structure containing the following
;		      tags:
;
;			PROG_NUM   = Study counter number.  This forms the
;				     primary key for this database.
;			PROG_ID	   = Program ID, linking one or more studies
;				     together
;			PROG_IND   = An index, reflecting one's position within
;				     a repeated sequence of studies via the
;				     N_REPEAT_S mechanism.
;			STUDY_ID   = Number defining the study
;			STUDYVAR   = Number giving the study variation ID
;			OBS_PROG   = Name of the study
;			DETECTOR   = Detector, "N", "G", or "B"(oth).
;			SCI_OBJ    = Science objective from the daily science
;				     meeting
;			SCI_SPEC   = Specific science objective from meeting
;			CMP_NO	   = Campaign number
;			OBJECT	   = Code for object planned to be observed
;			OBJ_ID	   = Object identification
;			DATE_OBS   = Date/time of beginning of observation,
;				     in TAI format
;			DATE_END   = Date/time of end of observation, in TAI
;				     format
;			OBT_TIME   = Onboard date/time of beginning of
;				     observation, in TAI format
;			OBT_END	   = Onboard date/time of end of observation,
;				     in TAI format
;			XCEN	   = Center of instrument field-of-view in
;				     solar X
;			YCEN	   = Center of instrument field-of-view in
;				     solar Y
;			ANGLE	   = Angle of instrument relative to solar
;				     north
;			IXWIDTH	   = Width of field-of-view in instrument X
;			IYWIDTH	   = Width of field-of-view in instrument Y
;			SEQ_FROM   = Start of range of observing sequence
;				     numbers.
;			SEQ_TO	   = End of range of observing sequence
;				     numbers.
;
;		      It can also be an array of such structures.
;
; Opt. Inputs :	None.
;
; Outputs     :	The result of the function is a logical value representing
;		whether or not the operation was successful, where 1 is
;		successful and 0 is unsuccessful.
;
; Opt. Outputs:	None.
;
; Keywords    :
;       ERRMSG    = If defined and passed, then any error messages will be
;                   returned to the user in this parameter rather than
;                   depending on the MESSAGE routine in IDL.  If no errors are
;                   encountered, then a null string is returned.  In order to
;                   use this feature, ERRMSG must be defined first, e.g.
;
;                       ERRMSG = ''
;                       Result = ADD_MAIN( ERRMSG=ERRMSG, ... )
;                       IF ERRMSG NE '' THEN ...
;
;
; Calls       :	DATATYPE, DBOPEN, DBBUILD, DBCLOSE, TRIM, GET_OBJECT,
;		GET_CAMPAIGN, GET_PROGRAM, GET_STUDY
;
; Common      :	None.
;
; Restrictions:	Only this routine can be used to add entries to the MAIN
;		database.  Modifying the database by hand could corrupt its
;		integrity.
;
;		The data types and sizes of the structure elements must match
;		the definitions in the database.
;
;		!PRIV must be 2 or greater to use this routine.
;
; Side effects:	Once an entry has been added to the database, it must never be
;		removed.
;
;		The dates in the structure are rounded off to millisecond
;		accuracy.
;
; Category    :	Planning, Databases.
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 30 June 1995
;
; Modified    :	Version 1, William Thompson, GSFC, 30 June 1995
;		Version 2, William Thompson, GSFC, 21 February 1996
;			Modified to be more robust.  It will complain about not
;			finding IDs in the databases, but will insert the
;			catalog entry anyway.
;		Version 3, William Thompson, GSFC, 2 April 1996
;			Modified for version 3 of the catalog.
;		Version 4, William Thompson, GSFC, 9 April 1996
;			Added calls to LOCK_DATABASE, UNLOCK_DATABASE
;		Version 5, William Thompson, GSFC, 25 March 1997
;			Added check to make sure that the entry didn't appear
;			while the routine was running.
;
; Version     :	Version 5, 25 March 1997
;-
;
	ON_ERROR, 2
;
;  Initialize RESULT to represent non-success.  If the routine is successful,
;  this value will be updated below.
;
	RESULT = 0
;
;  Check the input parameters
;
        IF N_PARAMS() NE 1 THEN BEGIN
           MESSAGE = 'Syntax:  Result = ADD_MAIN(OBS)'
	   GOTO, HANDLE_ERROR
        ENDIF
;
;  Make sure that the user has privilege to write into the database.
;
	IF !PRIV LT 2 THEN BEGIN
           MESSAGE = '!PRIV must be 2 or greater to write into the database'
           GOTO, HANDLE_ERROR
	ENDIF
;
;  Check each of the structure components to verify that it is of the correct
;  type and size.
;
	FOR I = 0,N_ELEMENTS(OBS)-1 DO BEGIN
	    IF DATATYPE(OBS(I).PROG_NUM,1) NE 'Long' THEN BEGIN
	       MESSAGE = 'Tag PROG_NUM must be a long integer'
	       GOTO, HANDLE_ERROR
	    END ELSE IF N_ELEMENTS(OBS(I).PROG_NUM) NE 1 THEN BEGIN
	       MESSAGE = 'Tag PROG_NUM must be a scalar'
	       GOTO, HANDLE_ERROR
	    ENDIF
;
	    IF DATATYPE(OBS(I).PROG_ID,1) NE 'Integer' THEN BEGIN
	       MESSAGE = 'Tag PROG_ID must be a short integer'
	       GOTO, HANDLE_ERROR
	    END ELSE IF N_ELEMENTS(OBS(I).PROG_ID) NE 1 THEN BEGIN
	       MESSAGE = 'Tag PROG_ID must be a scalar'
	       GOTO, HANDLE_ERROR
	    ENDIF
;
	    IF DATATYPE(OBS(I).PROG_IND,1) NE 'Integer' THEN BEGIN
	       MESSAGE = 'Tag PROG_IND must be a short integer'
	       GOTO, HANDLE_ERROR
	    END ELSE IF N_ELEMENTS(OBS(I).PROG_IND) NE 1 THEN BEGIN
	       MESSAGE = 'Tag PROG_IND must be a scalar'
	       GOTO, HANDLE_ERROR
	    ENDIF
;
	    IF DATATYPE(OBS(I).STUDY_ID,1) NE 'Integer' THEN BEGIN
	       MESSAGE = 'Tag STUDY_ID must be a short integer'
	       GOTO, HANDLE_ERROR
	    END ELSE IF N_ELEMENTS(OBS(I).STUDY_ID) NE 1 THEN BEGIN
	       MESSAGE = 'Tag STUDY_ID must be a scalar'
	       GOTO, HANDLE_ERROR
	    ENDIF
;
	    IF DATATYPE(OBS(I).STUDYVAR,1) NE 'Integer' THEN BEGIN
	       MESSAGE = 'Tag STUDYVAR must be a short integer'
	       GOTO, HANDLE_ERROR
	    END ELSE IF N_ELEMENTS(OBS(I).STUDYVAR) NE 1 THEN BEGIN
	       MESSAGE = 'Tag STUDYVAR must be a scalar'
	       GOTO, HANDLE_ERROR
	    ENDIF
;
	    IF DATATYPE(OBS(I).OBS_PROG,1) NE 'String' THEN BEGIN
               MESSAGE = 'Tag OBS_PROG must be a character string'
               GOTO, HANDLE_ERROR
	    END ELSE IF N_ELEMENTS(OBS(I).OBS_PROG) NE 1 THEN BEGIN
               MESSAGE = 'Tag OBS_PROG must be a scalar'
               GOTO, HANDLE_ERROR
	    END ELSE IF STRLEN(OBS(I).OBS_PROG) GT 8 THEN BEGIN $
               MESSAGE = 'Tag OBS_PROG must be 8 characters or less'
               GOTO, HANDLE_ERROR
	    ENDIF
;
	    IF DATATYPE(OBS(I).DETECTOR,1) NE 'String' THEN BEGIN
               MESSAGE = 'Tag DETECTOR must be a character string'
               GOTO, HANDLE_ERROR
	    END ELSE IF N_ELEMENTS(OBS(I).DETECTOR) NE 1 THEN BEGIN
               MESSAGE = 'Tag DETECTOR must be a scalar'
               GOTO, HANDLE_ERROR
	    END ELSE IF STRLEN(OBS(I).DETECTOR) GT 1 THEN BEGIN $
               MESSAGE = 'Tag DETECTOR must be 1 characters or less'
               GOTO, HANDLE_ERROR
	    ENDIF
;
	    IF DATATYPE(OBS(I).SCI_OBJ,1) NE 'String' THEN BEGIN
	       MESSAGE = 'Tag SCI_OBJ must be a character string'
	       GOTO, HANDLE_ERROR
	    END ELSE IF N_ELEMENTS(OBS(I).SCI_OBJ) NE 1 THEN BEGIN
	       MESSAGE = 'Tag SCI_OBJ must be a scalar'
	       GOTO, HANDLE_ERROR
	    END ELSE IF STRLEN(OBS(I).SCI_OBJ) GT 50 THEN BEGIN $
	       MESSAGE = 'Tag SCI_OBJ must be 50 characters or less'
	       GOTO, HANDLE_ERROR
	    ENDIF
;
	    IF DATATYPE(OBS(I).SCI_SPEC,1) NE 'String' THEN BEGIN
	       MESSAGE = 'Tag SCI_SPEC must be a character string'
	       GOTO, HANDLE_ERROR
	    END ELSE IF N_ELEMENTS(OBS(I).SCI_SPEC) NE 1 THEN BEGIN
	       MESSAGE = 'Tag SCI_SPEC must be a scalar'
	       GOTO, HANDLE_ERROR
	    END ELSE IF STRLEN(OBS(I).SCI_SPEC) GT 50 THEN BEGIN
	       MESSAGE = 'Tag SCI_SPEC must be 50 characters or less'
	       GOTO, HANDLE_ERROR
	    ENDIF
;
	    IF DATATYPE(OBS(I).CMP_NO,1) NE 'Integer' THEN BEGIN
	       MESSAGE = 'Tag CMP_NO must be a short integer'
	       GOTO, HANDLE_ERROR
	    END ELSE IF N_ELEMENTS(OBS(I).CMP_NO) NE 1 THEN BEGIN
	       MESSAGE = 'Tag CMP_NO must be a scalar'
	       GOTO, HANDLE_ERROR
	    ENDIF
;
	    IF DATATYPE(OBS(I).OBJECT,1) NE 'String' THEN BEGIN
	       MESSAGE = 'Tag OBJECT must be a character string'
	       GOTO, HANDLE_ERROR
	    END ELSE IF N_ELEMENTS(OBS(I).OBJECT) NE 1 THEN BEGIN
	       MESSAGE = 'Tag OBJECT must be a scalar'
	       GOTO, HANDLE_ERROR
	    END ELSE IF STRLEN(OBS(I).OBJECT) GT 3 THEN BEGIN
	       MESSAGE = 'Tag OBJECT must be 3 characters or less'
	       GOTO, HANDLE_ERROR
	    ENDIF
;
	    IF DATATYPE(OBS(I).OBJ_ID,1) NE 'String' THEN BEGIN
	       MESSAGE = 'Tag OBJ_ID must be a character string'
	       GOTO, HANDLE_ERROR
	    END ELSE IF N_ELEMENTS(OBS(I).OBJ_ID) NE 1 THEN BEGIN
	       MESSAGE = 'Tag OBJ_ID must be a scalar'
	       GOTO, HANDLE_ERROR
	    END ELSE IF STRLEN(OBS(I).OBJ_ID) GT 6 THEN BEGIN
	       MESSAGE = 'Tag OBJ_ID must be 6 characters or less'
	       GOTO, HANDLE_ERROR
	    ENDIF
;
	    IF DATATYPE(OBS(I).DATE_OBS,1) NE 'Double' THEN BEGIN
	       MESSAGE = 'Tag DATE_OBS must be a double precision number'
	       GOTO, HANDLE_ERROR
	    END ELSE IF N_ELEMENTS(OBS(I).DATE_OBS) NE 1 THEN BEGIN
	       MESSAGE = 'Tag DATE_OBS must be a scalar'
	       GOTO, HANDLE_ERROR
	    ENDIF
;
	    IF DATATYPE(OBS(I).DATE_END,1) NE 'Double' THEN BEGIN
	       MESSAGE = 'Tag DATE_END must be a double precision number'
	       GOTO, HANDLE_ERROR
	    END ELSE IF N_ELEMENTS(OBS(I).DATE_END) NE 1 THEN BEGIN
	       MESSAGE = 'Tag DATE_END must be a scalar'
	       GOTO, HANDLE_ERROR
	    ENDIF
;
	    IF DATATYPE(OBS(I).OBT_TIME,1) NE 'Double' THEN BEGIN
	       MESSAGE = 'Tag OBT_TIME must be a double precision number'
	       GOTO, HANDLE_ERROR
	    END ELSE IF N_ELEMENTS(OBS(I).OBT_TIME) NE 1 THEN BEGIN
	       MESSAGE = 'Tag OBT_TIME must be a scalar'
	       GOTO, HANDLE_ERROR
	    ENDIF
;
	    IF DATATYPE(OBS(I).OBT_END,1) NE 'Double' THEN BEGIN
	       MESSAGE = 'Tag OBT_END must be a double precision number'
	       GOTO, HANDLE_ERROR
	    END ELSE IF N_ELEMENTS(OBS(I).OBT_END) NE 1 THEN BEGIN
	       MESSAGE = 'Tag OBT_END must be a scalar'
	       GOTO, HANDLE_ERROR
	    ENDIF
;
	    IF DATATYPE(OBS(I).XCEN,1) NE 'Float' THEN BEGIN
	       MESSAGE = 'Tag XCEN must be a floating point number'
	       GOTO, HANDLE_ERROR
	    END ELSE IF N_ELEMENTS(OBS(I).XCEN) NE 1 THEN BEGIN
	       MESSAGE = 'Tag XCEN must be a scalar'
	       GOTO, HANDLE_ERROR
	    ENDIF
;
	    IF DATATYPE(OBS(I).YCEN,1) NE 'Float' THEN BEGIN
	       MESSAGE = 'Tag YCEN must be a floating point number'
	       GOTO, HANDLE_ERROR
	    END ELSE IF N_ELEMENTS(OBS(I).YCEN) NE 1 THEN BEGIN
	       MESSAGE = 'Tag YCEN must be a scalar'
	       GOTO, HANDLE_ERROR
	    ENDIF
;
	    IF DATATYPE(OBS(I).ANGLE,1) NE 'Float' THEN BEGIN
	       MESSAGE = 'Tag ANGLE must be a floating point number'
	       GOTO, HANDLE_ERROR
	    END ELSE IF N_ELEMENTS(OBS(I).ANGLE) NE 1 THEN BEGIN
	       MESSAGE = 'Tag ANGLE must be a scalar'
	       GOTO, HANDLE_ERROR
	    ENDIF
;
	    IF DATATYPE(OBS(I).IXWIDTH,1) NE 'Float' THEN BEGIN
	       MESSAGE = 'Tag IXWIDTH must be a floating point number'
	       GOTO, HANDLE_ERROR
	    END ELSE IF N_ELEMENTS(OBS(I).IXWIDTH) NE 1 THEN BEGIN
	       MESSAGE = 'Tag IXWIDTH must be a scalar'
	       GOTO, HANDLE_ERROR
	    ENDIF
;
	    IF DATATYPE(OBS(I).IYWIDTH,1) NE 'Float' THEN BEGIN
	       MESSAGE = 'Tag IYWIDTH must be a floating point number'
	       GOTO, HANDLE_ERROR
	    END ELSE IF N_ELEMENTS(OBS(I).IYWIDTH) NE 1 THEN BEGIN
	       MESSAGE = 'Tag IYWIDTH must be a scalar'
	       GOTO, HANDLE_ERROR
	    ENDIF
;
	    IF DATATYPE(OBS(I).SEQ_FROM,1) NE 'Long' THEN BEGIN
	       MESSAGE = 'Tag SEQ_FROM must be a long integer'
	       GOTO, HANDLE_ERROR
	    END ELSE IF N_ELEMENTS(OBS(I).SEQ_FROM) NE 1 THEN BEGIN
	       MESSAGE = 'Tag SEQ_FROM must be a scalar'
	       GOTO, HANDLE_ERROR
	    ENDIF
;
	    IF DATATYPE(OBS(I).SEQ_TO,1) NE 'Long' THEN BEGIN
	       MESSAGE = 'Tag SEQ_TO must be a long integer'
	       GOTO, HANDLE_ERROR
	    END ELSE IF N_ELEMENTS(OBS(I).SEQ_TO) NE 1 THEN BEGIN
	       MESSAGE = 'Tag SEQ_TO must be a scalar'
	       GOTO, HANDLE_ERROR
	    ENDIF
;
;  Make sure that the program ID matches an entry in the CDS program database.
;
	    IF OBS(I).PROG_ID GT 0 THEN BEGIN
		GET_PROGRAM, OBS(I).PROG_ID, TEMP
		IF TEMP.PROG_ID LE 0 THEN CDS_MESSAGE, /CONTINUE, /ATTENTION, $
			'Program ID ' + STRTRIM(OBS(I).PROG_ID,2) + $
			' not found in database'
	    ENDIF
;
;  Make sure that the campaign number matches an entry in the SOHO campaign
;  database.
;
	    IF OBS(I).CMP_NO GT 0 THEN BEGIN
		GET_CAMPAIGN, OBS(I).CMP_NO, TEMP
		IF TEMP.CMP_NO LE 0 THEN CDS_MESSAGE, /CONTINUE, /ATTENTION, $
			'Campaign number ' + STRTRIM(OBS(I).CMP_NO,2) +	$
			' not found in database'
	    ENDIF
;
;  Make sure that the object code matches an entry in the database.
;
	    GET_OBJECT, OBS(I).OBJECT, TEMP
	    IF TEMP.OBJECT EQ '' THEN BEGIN
	       CDS_MESSAGE, /CONTINUE, /ATTENTION, 'Object code ' +	$
		       OBS(I).OBJECT + ' not found in database'
	       CDS_MESSAGE, /CONTINUE, 'Resetting to blank'
	       OBS(I).OBJECT = ''
	    ENDIF
;
;  Make sure that the study ID and study variation index match an entry in the
;  CDS study database.
;
	    GET_STUDY, OBS(I).STUDY_ID, OBS(I).STUDYVAR, TEMP
	    IF TEMP.STUDY_ID LE -1 THEN CDS_MESSAGE, /CONTINUE, /ATTENTION, $
		    'Study ID/variation ' + TRIM(OBS(I).STUDY_ID) + ',' + $
		    TRIM(OBS(I).STUDYVAR) + ' not found in database'
;
;  Make sure that the proposed entry is not already in the database.
;
	    GET_MAIN, OBS(I).PROG_NUM, TEMP
	    IF TEMP.PROG_NUM GE 0 THEN BEGIN
	       MESSAGE = 'Program number ' + TRIM(OBS(I).PROG_NUM) +	$
		   ' is already in the database'
	       GOTO, HANDLE_ERROR
	    ENDIF
;
;  Make sure that the stop date is greater than the start date.
;
	    IF OBS(I).DATE_END LE OBS(I).DATE_OBS THEN BEGIN
	       MESSAGE = 'The stop date must be after the start date'
	       GOTO, HANDLE_ERROR
	    ENDIF
;
	    IF OBS(I).OBT_END LE OBS(I).OBT_TIME THEN BEGIN
	       MESSAGE = 'The OBT stop date must be after the start date'
	       GOTO, HANDLE_ERROR
	    ENDIF
	ENDFOR
;
;  Make sure that none of the proposed entries use the same program number.
;
	PROG_NUM = OBS.PROG_NUM
	IF N_ELEMENTS(PROG_NUM) GT 1 THEN BEGIN
	    PROG_NUM = PROG_NUM(SORT(PROG_NUM))
	    W = WHERE(PROG_NUM(1:*)-PROG_NUM EQ 0, COUNT)
	    IF COUNT GT 0 THEN BEGIN
	       MESSAGE = 'Proposed entries use the same program number'
	       GOTO, HANDLE_ERROR
	    ENDIF
	ENDIF
;
;  Reformat the times to millisecond accuracy.  This is necessary so that
;  the times are written out in a controlled way.
;
	OBS.DATE_OBS = DOUBLE( STRING( OBS.DATE_OBS, FORMAT='(F15.3)' ))
	OBS.DATE_END = DOUBLE( STRING( OBS.DATE_END, FORMAT='(F15.3)' ))
	OBS.OBT_TIME = DOUBLE( STRING( OBS.OBT_TIME, FORMAT='(F15.3)' ))
	OBS.OBT_END  = DOUBLE( STRING( OBS.OBT_END,  FORMAT='(F15.3)' ))
;
;  Get the modification date.  Make a dummy array for the COMMENTS field.
;
	GET_UTC, DATE_MOD
	DATE_MOD = REPLICATE(UTC2TAI(DATE_MOD), N_ELEMENTS(OBS))
	COMMENTS = REPLICATE('N', N_ELEMENTS(OBS))
;
;  Lock the database.
;
	LOCK_DATABASE, 'main', LOCKFILE
;
;  Make sure once again that MAIN does not already contain this entry.
;
	
	FOR I = 0,N_ELEMENTS(OBS)-1 DO BEGIN
	    GET_MAIN, OBS(I).PROG_NUM, TEMP
	    IF TEMP.PROG_NUM GE 0 THEN BEGIN
	       MESSAGE = 'Program number ' + TRIM(OBS(I).PROG_NUM) +	$
		   ' is already in the database'
	       GOTO, HANDLE_ERROR
	    ENDIF
	ENDFOR
;
;  Open the main as-run database for write access, and add the entry.
;
	DBOPEN, 'main', 1
	DBBUILD, OBS.PROG_NUM, OBS.PROG_ID, OBS.PROG_IND, OBS.STUDY_ID, $
		OBS.STUDYVAR, OBS.OBS_PROG, OBS.DETECTOR, OBS.SCI_OBJ,	$
		OBS.SCI_SPEC, OBS.CMP_NO, OBS.OBJECT, OBS.OBJ_ID,	$
		OBS.DATE_OBS, OBS.DATE_END, OBS.OBT_TIME, OBS.OBT_END,	$
		OBS.XCEN, OBS.YCEN, OBS.ANGLE, OBS.IXWIDTH, OBS.IYWIDTH, $
		OBS.SEQ_FROM, OBS.SEQ_TO, COMMENTS, DATE_MOD, STATUS=STATUS
	IF STATUS EQ 0 THEN BEGIN
	   MESSAGE = 'Write to main database was not successful'
	   GOTO, HANDLE_ERROR
	ENDIF
;
;  Signal success.
;
	RESULT = 1
	GOTO, FINISH
;
;  Error handling point.
;
HANDLE_ERROR:
	IF N_ELEMENTS(ERRMSG) NE 0 THEN ERRMSG = 'ADD_MAIN: ' + MESSAGE $
		ELSE MESSAGE, MESSAGE, /CONTINUE
;
;  Close the database, and return whether the routine was successful or not.
;
FINISH:
	DBCLOSE
;
	IF N_ELEMENTS(LOCKFILE) EQ 1 THEN UNLOCK_DATABASE, LOCKFILE
	RETURN, RESULT
	END
