	FUNCTION ADD_PROGRAM, DEF, ERRMSG=ERRMSG
;+
; Project     :	SOHO - CDS
;
; Name        :	ADD_PROGRAM()
;
; Purpose     :	Adds a CDS program definition to the database
;
; Explanation :	This procedure takes a CDS multi-observation program definition
;		and adds it to the database "program".
;
; Use         :	Result = ADD_PROGRAM( DEF )
;
;		IF NOT ADD_PROGRAM( DEF ) THEN ...
;
; Inputs      :	DEF = This is an anonymous structure containing the following
;		      tags:
;
;			PROGNAME = Name of the program.
;			PROG_ID	 = Unique identifier number.  Initially this is
;				   zero.  It is updated by this routine to
;				   reflect the ID number in the database.
;			PROGDESC = Up to five lines of text describing the
;				   program.
;
; Opt. Inputs :	None.
;
; Outputs     :	The result of the function is a logical value representing
;		whether or not the operation was successful, where 1 is
;		successful and 0 is unsuccessful.
;
; Opt. Outputs:	None.
;
; Keywords    :
;       ERRMSG    = If defined and passed, then any error messages will be
;                   returned to the user in this parameter rather than
;                   depending on the MESSAGE routine in IDL.  If no errors are
;                   encountered, then a null string is returned.  In order to
;                   use this feature, ERRMSG must be defined first, e.g.
;
;                       ERRMSG = ''
;                       Result = ADD_PROGRAM( ERRMSG=ERRMSG, ... )
;                       IF ERRMSG NE '' THEN ...
;
;
; Calls       :	DATATYPE, DBOPEN, DB_INFO, DBBUILD, DBCLOSE
;
; Common      :	None.
;
; Restrictions:	Only this routine can be used to add program descriptions to
;		the database.  Modifying the database by hand could corrupt its
;		integrity.
;
;		The data types and sizes of the structure elements must match
;		the definitions in the database.  The string lengths must not
;		exceed the lengths defined in the database.
;
;		!PRIV must be 2 or greater to use this routine.
;
; Side effects:	Once a program description has been added to the database, it
;		must never be removed.
;
; Category    :	Planning, Databases.
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 21 July 1994
;
; Modified    :	Version 1.0, William Thompson, GSFC, 21 July 1994
;               Version 2, Liyun Wang, GSFC/ARC, September 22, 1994
;                  Added the keyword ERRMSG.
;
; Version     :	Version 2, September 22, 1994
;-
;
	ON_ERROR, 2
;
;  Check the input parameters
;
        IF N_PARAMS() NE 1 THEN BEGIN
           MESSAGE = 'Syntax:  Result = ADD_PROGRAM(DEF)'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN BEGIN
              ERRMSG = MESSAGE
              RETURN, 0
           ENDIF ELSE MESSAGE, MESSAGE
        ENDIF

;
;  Initialize RESULT to represent non-success.  If the routine is successful,
;  this value will be updated below.
;
	RESULT = 0
;
;  Make sure that the user has privilege to write into the database.
;
	IF !PRIV LT 2 THEN BEGIN $
           MESSAGE = '!PRIV must be 2 or greater to write into the database'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	ENDIF
;
;  Make sure that the program ID number is less than or equal to zero.
;
	IF DEF.PROG_ID GT 0 THEN BEGIN
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = 'The program ID number must be LE 0. '+$
              'The final value is assigned by this routine' $
           ELSE BEGIN 
              MESSAGE, 'The program ID number must be LE 0',/CONTINUE
              MESSAGE, 'The final value is assigned by this routine',$
                 /CONTINUE
           ENDELSE
           GOTO, FINISH
	ENDIF
;
;  Check each of the structure components to verify that it is of the correct
;  type and size.
;
	IF DATATYPE(DEF.PROGNAME,1) NE 'String' THEN BEGIN
           MESSAGE = 'Tag PROGNAME must be a character string'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	END ELSE IF N_ELEMENTS(DEF.PROGNAME) NE 1 THEN BEGIN
           MESSAGE = 'Tag PROGNAME must be a scalar'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	END ELSE IF STRLEN(DEF.PROGNAME) GT 20 THEN BEGIN
           MESSAGE = 'Tag PROGNAME must be 20 characters or less'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	ENDIF
;
	IF DATATYPE(DEF.PROGDESC,1) NE 'String' THEN BEGIN
           MESSAGE = 'Tag PROGDESC must be a character string'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	END ELSE IF N_ELEMENTS(DEF.PROGDESC) GT 5 THEN BEGIN
           MESSAGE = 'Tag PROGDESC must have no more than 5 elements'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	END ELSE IF MAX(STRLEN(DEF.PROGDESC)) GT 80 THEN BEGIN
           MESSAGE = 'Each element of tag PROGDESC must be 80 ' +$
              'characters or less'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	ENDIF
;
;  Open the program database for write access.
;
	DBOPEN, 'program', 1
;
;  Find the largest program ID number currently in the database, and add one
;  to it.
;
	N_ENTRIES = DB_INFO('entries','program')
	IF N_ENTRIES EQ 0 THEN PROG_ID = 1 ELSE BEGIN
		DBEXT, -1, 'prog_id', PROG_IDS
		PROG_ID = MAX(PROG_IDS) + 1
	ENDELSE
;
;  Concatenate the multiline program description into a single string.
;
	PROGDESC = STRING(REPLICATE(32B,400))
	FOR I=0,N_ELEMENTS(DEF.PROGDESC)-1 DO	$
		STRPUT,PROGDESC,DEF.PROGDESC(I),80*I
;
;  Add the program to the database.
;
	DBBUILD, DEF.PROGNAME, PROG_ID, PROGDESC, STATUS=STATUS
	IF STATUS EQ 0 THEN BEGIN
           MESSAGE = 'Write to program database was not successful'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	ENDIF
;
;  Update the line list ID number in the structure.  Signal success.
;
	DEF.PROG_ID = PROG_ID
	RESULT = 1
;
;  Close the database, and return whether the routine was successful or not.
;
FINISH:
	DBCLOSE
;
	RETURN, RESULT
	END
