	FUNCTION ADD_RAW, DEF, ERRMSG=ERRMSG
;+
; Project     :	SOHO - CDS
;
; Name        :	ADD_RAW()
;
; Purpose     :	Adds an entry to the GIS raw data file catalog.
;
; Explanation :	This procedure adds an entry to the GIS raw data file catalog.
;		The ID number for the raw data file is assigned.
;
; Use         :	Result = ADD_RAW( DEF )
;
;		IF NOT ADD_RAW( DEF ) THEN ...
;
; Inputs      :	DEF = This is an anonymous structure containing the following
;		      tags:
;
;			RAW_ID	   = Raw data file ID number.  Initially, this
;				     is zero.  It is updated by this routine to
;				     reflect the ID number in the database.
;			RAW_DESC   = Free format description
;			START_TIME = Start time of data taking, in TAI format.
;			DET_NUM	   = GIS detector number: 1-4
;			SLIT_NUM   = Slit number
;			HVOLT	   = High voltage value
;			LLD	   = Low level discriminator value
;			CNT_RATE   = Count rate range (min, max)
;			RAW_FILE   = Raw data file name
;			FFB	   = Front face bias
;			FIL_ID	   = Filament ID number
;			FIL_CUR	   = Filament current
;			ZONE_ID	   = Zone ID number
;
; Opt. Inputs :	None.
;
; Outputs     :	The result of the function is a logical value representing
;		whether or not the operation was successful, where 1 is
;		successful and 0 is unsuccessful.
;
;		The tag RAW_ID in the input structure is updated to reflect
;		the ID number assigned in the database.
;
; Opt. Outputs:	None.
;
; Keywords    :	
;       ERRMSG    = If defined and passed, then any error messages will be
;                   returned to the user in this parameter rather than
;                   depending on the MESSAGE routine in IDL.  If no errors are
;                   encountered, then a null string is returned.  In order to
;                   use this feature, ERRMSG must be defined first, e.g.
;
;                       ERRMSG = ''
;                       Result = ADD_RAW( ERRMSG=ERRMSG, ... )
;                       IF ERRMSG NE '' THEN ...
;
; Calls       :	DATATYPE, DBOPEN, DB_INFO, DBBUILD, DBCLOSE, DBFIND, DBMATCH
;
; Common      :	None.
;
; Restrictions:	Only this routine can be used to add raw data file entries to
;		the database.  Modifying the database by hand could corrupt its
;		integrity.
;
;		The data types and sizes of the structure elements must match
;		the definitions in the database.  The string lengths must not
;		exceed the lengths defined in the database.
;
;		!PRIV must be 2 or greater to use this routine.
;
; Side effects:	Once a raw data file definition has been added to the database,
;		it must never be removed.
;
;		The date in the structure is rounded off to millisecond
;		accuracy.
;
;		There is currently no safeguard to prevent the same data being
;		added to the database twice.
;
; Category    :	Planning, Databases.
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 7 September 1995
;
; Modified    :	Version 1, William Thompson, GSFC, 7 September 1995
;		Version 2, William Thompson, GSFC, 14 September 1995
;			Added RAW_DESC, rename FF_BIAS to FFB to match GSET
;			database.
;
; Version     :	Version 2, 14 September 1995
;-
;
	ON_ERROR, 2
;
;  Initialize RESULT to represent non-success.  If the routine is successful,
;  this value will be updated below.
;
	RESULT = 0
;
;  Check the input parameters
;
        IF N_PARAMS() NE 1 THEN BEGIN
           MESSAGE = 'Syntax:  Result = ADD_RAW(DEF)'
	   GOTO, HANDLE_ERROR
        ENDIF
;
;  Make sure that the user has privilege to write into the database.
;
	IF !PRIV LT 2 THEN BEGIN
           MESSAGE = '!PRIV must be 2 or greater to write into the database'
           GOTO, HANDLE_ERROR
	ENDIF
;
;  Make sure that the raw data file ID number is less than or equal to zero.
;
	IF DEF.RAW_ID GT 0 THEN BEGIN
           MESSAGE= 'The raw data file ID number must be LE 0. '+$
                 'The final value is assigned by this routine'
           GOTO, HANDLE_ERROR
	ENDIF
;
;  Check each of the structure components to verify that it is of the correct
;  type and size.
;
	IF DATATYPE(DEF.RAW_DESC,1) NE 'String' THEN BEGIN
	   MESSAGE = 'Tag RAW_DESC must be a character string'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(DEF.RAW_DESC) NE 1 THEN BEGIN
           MESSAGE = 'Tag RAW_DESC must be a scalar'
           GOTO, HANDLE_ERROR
	END ELSE IF STRLEN(DEF.RAW_DESC) GT 80 THEN BEGIN
	   MESSAGE = 'Tag RAW_DESC must be 80 characters or less'
	   GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(DEF.START_TIME,1) NE 'Double' THEN BEGIN
           MESSAGE = 'Tag START_TIME must be a double precision number'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(DEF.START_TIME) NE 1 THEN BEGIN
           MESSAGE = 'Tag START_TIME must be a scalar'
           GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(DEF.DET_NUM,1) NE 'Integer' THEN BEGIN
           MESSAGE = 'Tag DET_NUM must be a short integer'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(DEF.DET_NUM) NE 1 THEN BEGIN
           MESSAGE = 'Tag DET_NUM must be a scalar'
           GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(DEF.SLIT_NUM,1) NE 'Integer' THEN BEGIN
           MESSAGE = 'Tag SLIT_NUM must be a short integer'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(DEF.SLIT_NUM) NE 1 THEN BEGIN
           MESSAGE = 'Tag SLIT_NUM must be a scalar'
           GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(DEF.HVOLT,1) NE 'Integer' THEN BEGIN
           MESSAGE = 'Tag HVOLT must be a short integer'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(DEF.HVOLT) NE 1 THEN BEGIN
           MESSAGE = 'Tag HVOLT must be a scalar'
           GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(DEF.LLD,1) NE 'Integer' THEN BEGIN
           MESSAGE = 'Tag LLD must be a short integer'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(DEF.LLD) NE 1 THEN BEGIN
           MESSAGE = 'Tag LLD must be a scalar'
           GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(DEF.CNT_RATE,1) NE 'Integer' THEN BEGIN
           MESSAGE = 'Tag CNT_RATE must be a short integer'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(DEF.CNT_RATE) NE 2 THEN BEGIN
           MESSAGE = 'Tag CNT_RATE must have two values (min, max)'
           GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(DEF.RAW_FILE,1) NE 'String' THEN BEGIN
           MESSAGE = 'Tag RAW_FILE must be a character string'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(DEF.RAW_FILE) NE 1 THEN BEGIN
           MESSAGE = 'Tag RAW_FILE must be a scalar'
           GOTO, HANDLE_ERROR
	END ELSE IF STRLEN(DEF.RAW_FILE) GT 30 THEN BEGIN
	   MESSAGE = 'Tag RAW_FILE must be 30 characters or less'
	   GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(DEF.FFB,1) NE 'Integer' THEN BEGIN
           MESSAGE = 'Tag FFB must be a short integer'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(DEF.FFB) NE 1 THEN BEGIN
           MESSAGE = 'Tag FFB must be a scalar'
           GOTO, HANDLE_ERROR
	END ELSE IF (DEF.FFB NE -1) AND (DEF.FFB NE +1) THEN BEGIN
	   MESSAGE = 'Tag FFB must be either -1 or +1'
	   GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(DEF.FIL_ID,1) NE 'Integer' THEN BEGIN
           MESSAGE = 'Tag FIL_ID must be a short integer'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(DEF.FIL_ID) NE 1 THEN BEGIN
           MESSAGE = 'Tag FIL_ID must be a scalar'
           GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(DEF.FIL_CUR,1) NE 'Integer' THEN BEGIN
           MESSAGE = 'Tag FIL_CUR must be a short integer'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(DEF.FIL_CUR) NE 1 THEN BEGIN
           MESSAGE = 'Tag FIL_CUR must be a scalar'
           GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(DEF.ZONE_ID,1) NE 'Integer' THEN BEGIN
           MESSAGE = 'Tag ZONE_ID must be a short integer'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(DEF.ZONE_ID) NE 1 THEN BEGIN
           MESSAGE = 'Tag ZONE_ID must be a scalar'
           GOTO, HANDLE_ERROR
	ENDIF
;
;  Reformat the time to millisecond accuracy.  This is necessary so that
;  the times are written out in a controlled way.
;
	DEF.START_TIME = DOUBLE( STRING( DEF.START_TIME, FORMAT='(F15.3)' ))
;
;  Open the raw data file database for write access.
;
	DBOPEN, 'raw_files', 1
;
;  Find the largest raw data file ID number currently in the database, and add
;  one to it.
;
	N_ENTRIES = DB_INFO('entries','raw_files')
	IF N_ENTRIES EQ 0 THEN RAW_ID = 1 ELSE BEGIN
		DBEXT, -1, 'raw_id', RAW_IDS
		RAW_ID = MAX(RAW_IDS) + 1
	ENDELSE
;
;  Append the data to the opened database.
;
	DBBUILD, RAW_ID, DEF.RAW_DESC, DEF.START_TIME, DEF.DET_NUM,	$
		DEF.SLIT_NUM, DEF.HVOLT, DEF.LLD, DEF.CNT_RATE, DEF.RAW_FILE, $
		DEF.FFB, DEF.FIL_ID, DEF.FIL_CUR, DEF.ZONE_ID, STATUS=STATUS
	IF STATUS EQ 0 THEN BEGIN
           MESSAGE = 'Write to raw_files database was not successful'
           GOTO, HANDLE_ERROR
        ENDIF
;
;  Update the raw data file ID number in the structure.  Signal success.
;
	DEF.RAW_ID = RAW_ID
	RESULT = 1
	GOTO, FINISH
;
;  Error handling point.
;
HANDLE_ERROR:
	IF N_ELEMENTS(ERRMSG) NE 0 THEN ERRMSG = 'ADD_RAW: ' + MESSAGE $
		ELSE MESSAGE, MESSAGE, /CONTINUE
;
;  Close the database, and return whether the routine was successful or not.
;
FINISH:
	DBCLOSE
;
	RETURN, RESULT
	END
