	FUNCTION ADD_T_STUDY, DEF, ERRMSG=ERRMSG, TITLE_ID=DBTITLE_ID,	$
		CHECK_ONLY=CHECK_ONLY
;+
; Project     :	SOHO - CDS
;
; Name        :	ADD_T_STUDY()
;
; Purpose     :	Adds a fundamental study definition to the title database
;
; Explanation :	This procedure takes the name, title, and category parts of the
;		fundamental parameters of a study definition and adds them to
;		the database "study_title"., and "study2".  The ID
;		number for the study title is assigned.
;
; Use         :	Result = ADD_T_STUDY( DEF )
;
;		IF NOT ADD_T_STUDY( DEF ) THEN ...
;
; Inputs      :	DEF = This is an anonymous structure containing the following
;		      tags:
;
;			STUDY_ID    = Study ID number.  Must match an entry in
;				      the fundamental study database.
;			TITLE_ID    = Study title ID number.  Initially, this
;				      is -1.  It is updated by this routine to
;				      reflect the ID number in the database.
;			OBS_PROG    = Name of the study
;			TITLE	    = A descriptive title for the study
;			CATEGORY    = The study category, either "T"est,
;				      "S"cience or "C"alibration.
;
; Opt. Inputs :	None.
;
; Outputs     :	The result of the function is a logical value representing
;		whether or not the operation was successful, where 1 is
;		successful and 0 is unsuccessful.
;
;		The tag TITLE_ID in the input structure is updated to reflect
;		the ID number assigned in the database.
;
; Opt. Outputs:	None.
;
; Keywords    :	
;       ERRMSG    = If defined and passed, then any error messages will be
;                   returned to the user in this parameter rather than
;                   depending on the MESSAGE routine in IDL.  If no errors are
;                   encountered, then a null string is returned.  In order to
;                   use this feature, ERRMSG must be defined first, e.g.
;
;                       ERRMSG = ''
;                       Result = ADD_T_STUDY( ERRMSG=ERRMSG, ... )
;                       IF ERRMSG NE '' THEN ...
;
;	TITLE_ID  = If the name and title matches one that is already in the
;		    database for the given study ID, then this keyword returns
;		    the title ID of the matching study.  Otherwise, a -1 is
;		    returned.
;
;	CHECK_ONLY= If this keyword is set, then the routine only checks to see
;		    if the proposed entry is valid, but does not attempt to
;		    place it in the database.
;
; Calls       :	DATATYPE, DBOPEN, DB_INFO, DBBUILD, DBCLOSE, DBFIND, DBMATCH
;
; Common      :	None.
;
; Restrictions:	Only this routine and ADD_F_STUDY can be used to add
;		fundamental study names and titles to the database.  Modifying
;		the database by hand could corrupt its integrity.
;
;		The data types and sizes of the structure elements must match
;		the definitions in the database.  The string lengths must not
;		exceed the lengths defined in the database.
;
;		This routine must not be called for studies where
;		N_RASTER_DEF=0.
;
;		!PRIV must be 2 or greater to use this routine.
;
; Side effects:	Once a study title has been added to the database, it must
;		never be removed.
;
; Category    :	Planning, Databases.
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 12 May 1995
;
; Modified    :	Version 1, William Thompson, GSFC, 12 May 1995
;		Version 2, William Thompson, GSFC, 2 June 1995
;			Changed way that error messages are handled.
;
; Version     :	Version 2, 2 June 1995
;-
;
	ON_ERROR, 2
;
;  Make sure that the TITLE_ID keyword is initialized, in case the program
;  exits before the CHK_T_STUDY routine is reached.
;
	DBTITLE_ID = -1
;
;  Initialize RESULT to represent non-success.  If the routine is successful,
;  this value will be updated below.  Also initialize MESSAGE.
;
	RESULT = 0
	MESSAGE = ''
;
;  Check the input parameters
;
        IF N_PARAMS() NE 1 THEN BEGIN
           MESSAGE = 'Syntax:  Result = ADD_T_STUDY(DEF)'
	   GOTO, HANDLE_ERROR
        ENDIF
;
;  Make sure that the user has privilege to write into the database.
;
	IF !PRIV LT 2 THEN BEGIN $
           MESSAGE = '!PRIV must be 2 or greater to write into the database'
           GOTO, HANDLE_ERROR
	ENDIF
;
;  Make sure that the title ID number is negative.
;
	IF DEF.TITLE_ID GE 0 THEN BEGIN
	   MESSAGE = 'The title ID number must be negative.  ' +	$
		   'The final value is assigned by this routine'
           GOTO, HANDLE_ERROR
	ENDIF
;
;  Check each of the structure components to verify that it is of the correct
;  type and size.
;
	IF DATATYPE(DEF.STUDY_ID,1) NE 'Integer' THEN BEGIN
           MESSAGE = 'Tag STUDY_ID must be a short integer'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(DEF.STUDY_ID) NE 1 THEN BEGIN
           MESSAGE = 'Tag STUDY_ID must be a scalar'
           GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(DEF.OBS_PROG,1) NE 'String' THEN BEGIN
           MESSAGE = 'Tag OBS_PROG must be a character string'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(DEF.OBS_PROG) NE 1 THEN BEGIN
           MESSAGE = 'Tag OBS_PROG must be a scalar'
           GOTO, HANDLE_ERROR
	END ELSE IF STRLEN(DEF.OBS_PROG) GT 8 THEN BEGIN
           MESSAGE = 'Tag OBS_PROG must be no longer than 8 characters'
           GOTO, HANDLE_ERROR
	END ELSE IF DEF.OBS_PROG NE STRUPCASE(DEF.OBS_PROG) THEN BEGIN
           MESSAGE = 'Tag OBS_PROG must be uppercase'
           GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(DEF.TITLE,1) NE 'String' THEN BEGIN
           MESSAGE = 'Tag TITLE must be a character string'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(DEF.TITLE) NE 1 THEN BEGIN
           MESSAGE = 'Tag TITLE must be a scalar'
           GOTO, HANDLE_ERROR
	END ELSE IF STRLEN(DEF.TITLE) GT 80 THEN BEGIN
           MESSAGE = 'Tag TITLE must be no longer than 80 characters'
           GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(DEF.CATEGORY,1) NE 'String' THEN BEGIN
           MESSAGE = 'Tag CATEGORY must be a character string'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(DEF.CATEGORY) NE 1 THEN BEGIN
           MESSAGE = 'Tag CATEGORY must be a scalar'
           GOTO, HANDLE_ERROR
	END ELSE IF STRLEN(DEF.CATEGORY) NE 1 THEN BEGIN
           MESSAGE = 'Tag CATEGORY must be a single character'
           GOTO, HANDLE_ERROR
	END ELSE IF (DEF.CATEGORY NE 'T') AND (DEF.CATEGORY NE 'S') AND	$
			(DEF.CATEGORY NE 'C') THEN BEGIN
           MESSAGE = 'Tag CATEGORY must be "T", "S", or "C"'
           GOTO, HANDLE_ERROR
	ENDIF
;
;  Make sure that STUDY_ID points to a valid entry in the fundamental study
;  database.
;
	DBOPEN, 'study'
	STUDY_PTR = DBFIND('STUDY_ID=' + STRTRIM(LONG(DEF.STUDY_ID),2),/SILENT)
	IF !ERR EQ 0 THEN BEGIN
           MESSAGE = 'Study ID does not exist in database'
           GOTO, HANDLE_ERROR
	ENDIF
	STUDY_PTR = LONG(STUDY_PTR(0))
;
;  Check to see if the OBS_PROG and TITLE are already in the database or not.
;
	DBTITLE_ID = CHK_T_STUDY(DEF)
        IF DBTITLE_ID GE 0 THEN BEGIN
           MESSAGE = 'Data base entry for this study name and title ' + $
              'already exists, title ID = ' + NUM2STR(DBTITLE_ID)
           GOTO, HANDLE_ERROR
        ENDIF
;
;  Also check the OBS_PROG and TITLE parameters individually.
;
	TEST = CHK_T_STUDY(DEF, /EITHER, ERRMSG=MESSAGE)
        IF TEST GE 0 THEN GOTO, HANDLE_ERROR
;
;  Open the study title data for write access.
;
	DBOPEN, 'study_title', 1
;
;  Find the largest study title ID number currently in the database, and add
;  one to it.  If the database is empty, then set the TITLE_ID to ond.  Find
;  the number of entries and add one to it--that will be the pointer for the
;  new entry.
;
	N_ENTRIES = DB_INFO('entries','study_title')
	TITLE_PTR = N_ENTRIES + 1
	IF N_ENTRIES EQ 0 THEN TITLE_ID = 1 ELSE BEGIN
		DBEXT, -1, 'title_id', TITLE_IDS
		TITLE_ID = MAX(TITLE_IDS) + 1
	ENDELSE
;
;  If the CHECK_ONLY keyword was set, then return now as if the routine were
;  successful.  However, don't modify the study ID or title ID in the
;  definition.
;
	IF KEYWORD_SET(CHECK_ONLY) THEN BEGIN
		RESULT = 1
		GOTO, FINISH
	ENDIF
;
;  Append the data to the opened database.
;
	DBBUILD, LONG(STUDY_PTR), DEF.STUDY_ID, TITLE_ID,	$
		STRTRIM(DEF.OBS_PROG,2), STRTRIM(DEF.TITLE,2), DEF.CATEGORY, $
		STATUS=STATUS
	IF STATUS EQ 0 THEN BEGIN
           MESSAGE = 'Write to study database was not successful'
           GOTO, HANDLE_ERROR
	ENDIF
;
;  Update the title ID number in the structure.  Signal success.
;
	DEF.TITLE_ID = TITLE_ID
	RESULT = 1
	GOTO, FINISH
;
;  Error handling point.
;
HANDLE_ERROR:
	IF N_ELEMENTS(ERRMSG) NE 0 THEN ERRMSG = 'ADD_T_STUDY: ' + MESSAGE $
		ELSE MESSAGE, MESSAGE, /CONTINUE
;
;  Close the database, and return whether the routine was successful or not.
;
FINISH:
	DBCLOSE
;
	RETURN, RESULT
	END
