	FUNCTION ADD_V_RASTER, DEF, ERRMSG=ERRMSG, RASTER=DBRASTER,	$
		CHECK_ONLY=CHECK_ONLY
;+
; Project     :	SOHO - CDS
;
; Name        :	ADD_V_RASTER()
;
; Purpose     :	Adds a raster variation definition to the database
;
; Explanation :	This procedure takes the parameters that define a raster
;		variation and adds them to the database "raster_var".  The
;		raster variation index number is assigned.
;
; Use         :	Result = ADD_V_RASTER( DEF )
;
;		IF NOT ADD_V_RASTER( DEF ) THEN ...
;
; Inputs      :	DEF = This is a structure of type "cds_v_raster" containing the
;		      following tags:
;
;			RAS_ID	   = Raster ID number.  Must match an entry in
;				     the fundamental raster database.
;			RAS_VAR	   = Raster variation index.  Initially, this
;				     is zero.  It is updated by this routine to
;				     reflect the index in the database.
;			RV_DESC	   = A short description of the raster
;				     variation beyond what is given in the
;				     associated fundamental raster description.
;			EXPTIME	   = Exposure time in seconds, to millisecond
;				     accuracy.
;			LL_ID	   = Line list ID.  Must match an entry in the
;				     line list database.  Must also be
;				     consistent with the corresponding DETECTOR
;				     parameter in the fundamental raster
;				     database.  Alternatively, this can be set
;				     to zero to signal that the raster is not
;				     connected to a line list.
;			COMP_ID	   = Compression method ID.
;			COMP_OPT   = Compression option parameter.
;			DW_ID	   = Data extraction window list ID.  Must
;				     match an entry in the data window list
;				     table.  Must also be consistent with the
;				     LL_ID parameter, and with the DETECTOR
;				     parameter from the fundamental raster
;				     definition.
;			IEF_ID	   = Inter-instrument event flag ID.
;			VDS_ORIENT = VDS orientation, either 0 (row) or 1
;				     (column).
;			VDS_MAP	   = VDS mapping mode: 2=Normal, 3=Accumulate.
;			TEL_RATE   = Estimated required telemetry rate, as a
;				     character representing Low/Medium/High
;			DURATION   = Estimated duration of the raster, in
;				     seconds.
;			USABLE	   = Either "Y" or "N" to signal whether or not
;				     the raster is usable.  Normally "Y".
;
; Opt. Inputs :	None.
;
; Outputs     :	The result of the function is a logical value representing
;		whether or not the operation was successful, where 1 is
;		successful and 0 is unsuccessful.
;
;		The tag RAS_VAR in the input structure is updated to reflect
;		the index number assigned in the database.
;
; Opt. Outputs:	None.
;
; Keywords    :
;       ERRMSG    = If defined and passed, then any error messages will be
;                   returned to the user in this parameter rather than
;                   depending on the MESSAGE routine in IDL.  If no errors are
;                   encountered, then a null string is returned.  In order to
;                   use this feature, ERRMSG must be defined first, e.g.
;
;                       ERRMSG = ''
;                       Result = ADD_V_RASTER( ERRMSG=ERRMSG, ... )
;                       IF ERRMSG NE '' THEN ...
;
;	RASTER	  = If the definition matches one already in the database,
;		    then this keyword returns a vector containing the RAS_ID
;		    and RAS_VAR of the match.  Otherwise, it returns the
;		    single value -1.
;
;	CHECK_ONLY= If this keyword is set, then the routine only checks to see
;		    if the proposed entry is valid, but does not attempt to
;		    place it in the database.
;
; Calls       :	DATATYPE, DBOPEN, DBFIND, DBBUILD, DBCLOSE, DBMATCH
;
; Common      :	None.
;
; Restrictions:	Only this routine can be used to add raster variation
;		definitions to the database.  Modifying the database by hand
;		could corrupt its integrity.
;
;		The data types and sizes of the structure elements must match
;		the definitions in the database.  The string lengths must not
;		exceed the lengths defined in the database.
;
;		!PRIV must be 2 or greater to use this routine.
;
; Side effects:	Once a raster definition has been added to the database, it
;		must never be removed.
;
; Category    :	Planning, Databases.
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 4 April 1994
;
; Modified    :	Version 1.0, William Thompson, GSFC, 21 July 1994
;		Version 1.1, William Thompson, GSFC, 22 August 1994
;			Changed definition of VDS_MAP.
;               Version 2, Liyun Wang, GSFC/ARC, September 22, 1994
;                  Added the keyword ERRMSG.
;		Version 3, William Thompson, GSFC, 24 October 1994
;			Change VDS_MAP to be zero for GIS data
;               Version 4, Liyun Wang, GSFC/ARC, November 4, 1994
;                  CHK_V_RASTER is called to check the existence of the 
;                  given raster variation
;		Version 5, William Thompson, GSFC, 8 November 1994
;			Added keyword RASTER
;		Version 6, William Thompson, GSFC, 29 December 1994
;			Fixed bug when inputs are bytes.
;		Version 7, William Thompson, GSFC, 27 February 1995
;			Added check for valid COMP_ID.
;		Version 8, William Thompson, GSFC, 2 March 1995
;			Enforce uniqueness for RV_DESC.
;			Added keyword CHECK_ONLY.
;		Version 9, William Thompson, GSFC, 14 March 1995
;			Changed to use DBMATCH instead of DBFIND for RV_DESC.
;		Version 10, William Thompson, GSFC, 15 March 1995
;			Changed TEL_RATE to character
;		Version 11, Stein Vidar Haugan, UiO, 25 September 1995
;			Fixed bug with DBEXT always returning arrays.
;		Version 12, William Thompson, GSFC, 27 October 1995
;			Added item IEF_ID.
;		Version 13, William Thompson, GSFC, 11 December 1997
;			Removed TEL_RATE value check, to allow for additional
;			values besides L/M/H.
;
; Version     :	Version 13, 11 December 1997
;-
;
	ON_ERROR, 2
;
;  Make sure that the RASTER keyword is initialized, in case the program
;  exits before the CHK_V_RASTER routine is reached.
;
	DBRASTER = -1
;
;  Check the input parameters
;
        IF N_PARAMS() NE 1 THEN BEGIN
           MESSAGE = 'Syntax:  Result = ADD_V_RASTER(DEF)'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN BEGIN
              ERRMSG = MESSAGE
              RETURN, 0
           ENDIF ELSE MESSAGE, MESSAGE
        ENDIF
;
;  Initialize RESULT to represent non-success.  If the routine is successful,
;  this value will be updated below.
;
	RESULT = 0
;
;  Make sure that the user has privilege to write into the database.
;
	IF !PRIV LT 2 THEN BEGIN $
           MESSAGE = '!PRIV must be 2 or greater to write into the database'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	ENDIF
;
;  Make sure that the raster variation index is less than or equal to zero.
;
        IF DEF.RAS_VAR GT 0 THEN BEGIN
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = 'The raster variation index must be LE 0 '+$
              'The final value is assigned by this routine'$
           ELSE BEGIN
              MESSAGE, 'The raster variation index must be LE 0', /CONTINUE
              MESSAGE, 'The final value is assigned by this routine',	$
                 /CONTINUE
           ENDELSE
           GOTO, FINISH
	ENDIF
;
;  Check each of the structure components to verify that it is of the correct
;  type and size.
;
	IF DATATYPE(DEF.RAS_ID,1) NE 'Integer' THEN BEGIN
           MESSAGE = 'Tag RAS_ID must be a short integer'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	END ELSE IF N_ELEMENTS(DEF.RAS_ID) NE 1 THEN BEGIN
           MESSAGE = 'Tag RAS_ID must be a scalar'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	ENDIF
;
	IF DATATYPE(DEF.RV_DESC,1) NE 'String' THEN BEGIN
           MESSAGE = 'Tag RV_DESC must be a character string'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	END ELSE IF N_ELEMENTS(DEF.RV_DESC) NE 1 THEN BEGIN
           MESSAGE = 'Tag RV_DESC must be a scalar'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	END ELSE IF STRLEN(DEF.RV_DESC) GT 65 THEN BEGIN
           MESSAGE = 'Tag RV_DESC must be no longer than 65 characters'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	ENDIF
;
	IF DATATYPE(DEF.EXPTIME,1) NE 'Float' THEN BEGIN
           MESSAGE = 'Tag EXPTIME must be a floating point number'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	END ELSE IF N_ELEMENTS(DEF.EXPTIME) NE 1 THEN BEGIN
           MESSAGE = 'Tag EXPTIME must be a scalar'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	ENDIF
;
	IF DATATYPE(DEF.LL_ID,1) NE 'Integer' THEN BEGIN
           MESSAGE = 'Tag LL_ID must be a short integer'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	END ELSE IF N_ELEMENTS(DEF.LL_ID) NE 1 THEN BEGIN
           MESSAGE = 'Tag LL_ID must be a scalar'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	ENDIF
;
	IF DATATYPE(DEF.COMP_ID,1) NE 'Integer' THEN BEGIN
           MESSAGE = 'Tag COMP_ID must be a short integer'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	END ELSE IF N_ELEMENTS(DEF.COMP_ID) NE 1 THEN BEGIN
           MESSAGE = 'Tag COMP_ID must be a scalar'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	END ELSE IF DEF.COMP_ID LE 0 THEN BEGIN
	   MESSAGE = 'Tag COMP_ID must be 1 or greater'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	ENDIF
;
	IF DATATYPE(DEF.COMP_OPT,1) NE 'Integer' THEN BEGIN
           MESSAGE = 'Tag COMP_OPT must be a short integer'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	END ELSE IF N_ELEMENTS(DEF.COMP_OPT) NE 1 THEN BEGIN
           MESSAGE = 'Tag COMP_OPT must be a scalar'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	ENDIF
;
	IF DATATYPE(DEF.DW_ID,1) NE 'Integer' THEN BEGIN
           MESSAGE = 'Tag DW_ID must be a short integer'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	END ELSE IF N_ELEMENTS(DEF.DW_ID) NE 1 THEN BEGIN
           MESSAGE = 'Tag DW_ID must be a scalar'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	ENDIF
;
	IF DATATYPE(DEF.IEF_ID,1) NE 'Integer' THEN BEGIN
           MESSAGE = 'Tag IEF_ID must be a short integer'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	END ELSE IF N_ELEMENTS(DEF.IEF_ID) NE 1 THEN BEGIN
           MESSAGE = 'Tag IEF_ID must be a scalar'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	ENDIF
;
	IF DATATYPE(DEF.VDS_ORIENT,1) NE 'Integer' THEN BEGIN
           MESSAGE = 'Tag VDS_ORIENT must be a short integer'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	END ELSE IF N_ELEMENTS(DEF.VDS_ORIENT) NE 1 THEN BEGIN
           MESSAGE = 'Tag VDS_ORIENT must be a scalar'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	END ELSE IF (DEF.VDS_ORIENT NE 0) AND (DEF.VDS_ORIENT NE 1) THEN BEGIN
           MESSAGE = 'Tag VDS_ORIENT must be either 0 or 1'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	ENDIF
;
	IF DATATYPE(DEF.VDS_MAP,1) NE 'Integer' THEN BEGIN
           MESSAGE = 'Tag VDS_MAP must be a short integer'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	END ELSE IF N_ELEMENTS(DEF.VDS_MAP) NE 1 THEN BEGIN
           MESSAGE = 'Tag VDS_MAP must be a scalar'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	ENDIF
;
	IF DATATYPE(DEF.TEL_RATE,1) NE 'String' THEN BEGIN
           MESSAGE = 'Tag TEL_RATE must be a character string'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	END ELSE IF N_ELEMENTS(DEF.TEL_RATE) NE 1 THEN BEGIN
           MESSAGE = 'Tag TEL_RATE must be a scalar'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	END ELSE IF STRLEN(DEF.TEL_RATE) NE 1 THEN BEGIN
           MESSAGE = 'Tag TEL_RATE must be a single character'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	ENDIF
;
	IF DATATYPE(DEF.DURATION,1) NE 'Float' THEN BEGIN
           MESSAGE = 'Tag DURATION must be a floating point number'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	END ELSE IF N_ELEMENTS(DEF.DURATION) NE 1 THEN BEGIN
           MESSAGE = 'Tag DURATION must be a scalar'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	ENDIF
;
	IF DATATYPE(DEF.USABLE,1) NE 'String' THEN BEGIN
           MESSAGE = 'Tag USABLE must be a character string'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	END ELSE IF N_ELEMENTS(DEF.USABLE) NE 1 THEN BEGIN
           MESSAGE = 'Tag USABLE must be a scalar'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	END ELSE IF STRLEN(DEF.USABLE) NE 1 THEN BEGIN
           MESSAGE = 'Tag USABLE must be a single character'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	END ELSE IF (DEF.USABLE NE 'Y') AND (DEF.USABLE NE 'N') THEN BEGIN
           MESSAGE = 'Tag USABLE must be either "Y" or "N"'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	ENDIF
;----------------------------------------------------------------------
;       Check the existence of the raster variation
;----------------------------------------------------------------------
        dbraster = chk_v_raster(def)
        IF N_ELEMENTS(dbraster) GT 1 THEN BEGIN
           MESSAGE = 'Raster variation definition already exists. '+$
              'Raster ID = '+num2str(dbraster(0))+', variation ID = '+$
              num2str(dbraster(1))
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG =  MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE
           GOTO, FINISH
        ENDIF
;
;  Make sure that the raster ID number matches an entry in the fundamental
;  raster database.  Save the pointer for later.
;
	DBOPEN, 'raster'
	RASTER_PTR = DBFIND('RAS_ID=' + NUM2STR(DEF.RAS_ID), /SILENT)
	IF !ERR EQ 0 THEN BEGIN
           MESSAGE = 'Raster ID number does not exist in database'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	ENDIF
	RASTER_PTR = LONG(RASTER_PTR(0))
;
;  Get the DETECTOR parameter from the fundamental raster database.
;
	DBEXT, RASTER_PTR(0), 'detector', DETECTOR
        DETECTOR = DETECTOR(0)
;
;  Depending on the detector, check the value of VDS_MAP.
;
	IF DETECTOR EQ 'N' THEN BEGIN
	   IF (DEF.VDS_MAP NE 2) AND (DEF.VDS_MAP NE 3) THEN BEGIN
		MESSAGE = 'Tag VDS_MAP must be either 2 or 3 for NIS'
		IF N_ELEMENTS(ERRMSG) NE 0 THEN $
		    ERRMSG = MESSAGE $
		ELSE MESSAGE, MESSAGE, /CONTINUE 
		GOTO, FINISH
	   ENDIF
	END ELSE BEGIN
	   IF DEF.VDS_MAP NE 0 THEN BEGIN
		MESSAGE = 'Tag VDS_MAP must be 0 for GIS'
		IF N_ELEMENTS(ERRMSG) NE 0 THEN $
		    ERRMSG = MESSAGE $
		ELSE MESSAGE, MESSAGE, /CONTINUE 
		GOTO, FINISH
	   ENDIF
	ENDELSE
;
;  Make sure that the line list ID number (if not zero) matches an entry in the
;  line list database, and that the DETECTOR parameter is consistent with the
;  entry in the raster definition.  Save the pointer for later.
;
	IF DEF.LL_ID NE 0 THEN BEGIN
		DBOPEN, 'linelist'
		LINELIST_PTR = DBFIND('LL_ID=' + NUM2STR(DEF.LL_ID), /SILENT)
		IF !ERR EQ 0 THEN BEGIN
                   MESSAGE = 'Line list ID does not exist in database'
                   IF N_ELEMENTS(ERRMSG) NE 0 THEN $
                      ERRMSG = MESSAGE $
                   ELSE MESSAGE, MESSAGE, /CONTINUE 
                   GOTO, FINISH
		ENDIF
		DBEXT, LINELIST_PTR, 'detector', LL_DETECTOR
		IF DETECTOR NE LL_DETECTOR(0) THEN BEGIN
                   MESSAGE = 'Raster and line list DETECTOR parameters '+$
                      'DO NOT match'
                   IF N_ELEMENTS(ERRMSG) NE 0 THEN $
                      ERRMSG = MESSAGE $
                   ELSE MESSAGE, MESSAGE, /CONTINUE 
                   GOTO, FINISH
		ENDIF
	END ELSE LINELIST_PTR = 0
	LINELIST_PTR = LONG(LINELIST_PTR(0))
;
;  Make sure that the data extraction window list ID number matches an entry in
;  the database, and that the LL_ID and DETECTOR parameters are consistent
;  with the rest of the definition.  Save the pointer for later.
;
	DBOPEN, 'datawin,linelist'
	DATAWIN_PTR = DBFIND('DW_ID=' + NUM2STR(DEF.DW_ID), /SILENT)
	IF !ERR EQ 0 THEN BEGIN
           MESSAGE = 'Data window list ID does not exist in database'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	ENDIF
;
	DATAWIN_PTR = LONG(DATAWIN_PTR(0))
	DBEXT, DATAWIN_PTR, 'll_id, detector', DW_LL_ID, DW_DETECTOR
;
	IF DW_LL_ID(0) NE DEF.LL_ID THEN BEGIN $
           MESSAGE = 'Line list and data window list IDs are inconsistent'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	END ELSE IF DETECTOR NE DW_DETECTOR(0) THEN BEGIN
           MESSAGE = 'Data window list ID not consistent with DETECTOR'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	ENDIF
;
;  First, check to see if there are any rasters with the same definition
;  (ignoring the description and those parameters which can be modified by
;  MOD_V_RASTER).
;
	DBOPEN, 'raster_var,raster'
	ENTRY = DBFIND('RAS_ID='+ NUM2STR(DEF.RAS_ID) +		$
		',EXPTIME='	+ NUM2STR(DEF.EXPTIME) +	$
		',LL_ID='	+ NUM2STR(DEF.LL_ID) +		$
		',COMP_ID='	+ NUM2STR(DEF.COMP_ID) +	$
		',COMP_OPT='	+ NUM2STR(DEF.COMP_OPT) +	$
		',DW_ID='	+ NUM2STR(DEF.DW_ID) +		$
		',VDS_ORIENT='	+ NUM2STR(DEF.VDS_ORIENT) +	$
		',VDS_MAP='	+ NUM2STR(DEF.VDS_MAP), /SILENT)
	IF !ERR NE 0 THEN BEGIN
           MESSAGE = 'Raster variation with same definition already exists'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	ENDIF
;
;  Find the total number of currently defined raster variations for the
;  selected raster ID number in the database, and add one to it.
;
	ENTRIES = DBFIND('RAS_ID=' + NUM2STR(DEF.RAS_ID), /SILENT)
	IF !ERR EQ 0 THEN RAS_VAR = 1 ELSE BEGIN
		RAS_VAR = FIX(N_ELEMENTS(ENTRIES) + 1)
;
;  Make sure that RV_DESC does not match one already in the database for that
;  study ID.  Use DBMATCH instead of DBFIND because there are problems with
;  special characters in DBFIND.
;
		ENTRIES = DBMATCH('RV_DESC',STRTRIM(DEF.RV_DESC,2), ENTRIES, $
			/FULLSTRING)
		IF ENTRIES(0) GT 0 THEN BEGIN
		    MESSAGE = 'Variation description already in database'
		    IF N_ELEMENTS(ERRMSG) NE 0 THEN $
			ERRMSG = MESSAGE $
		    ELSE MESSAGE, MESSAGE, /CONTINUE
		    GOTO, FINISH
		ENDIF
	ENDELSE
;
;  If the CHECK_ONLY keyword was set, then return now as if the routine were
;  successful.  However, don't modify the raster variation index in the
;  definition.
;
	IF KEYWORD_SET(CHECK_ONLY) THEN BEGIN
		RESULT = 1
		GOTO, FINISH
	ENDIF
;
;  Open the raster variation database for write access.
;
	DBOPEN, 'raster_var', 1
;
;  Append the data to the opened database.
;
	DBBUILD, RASTER_PTR, DEF.RAS_ID, RAS_VAR, STRTRIM(DEF.RV_DESC,2), $
		DEF.EXPTIME, LINELIST_PTR, DEF.LL_ID, DEF.COMP_ID,	$
		DEF.COMP_OPT, DATAWIN_PTR, DEF.DW_ID, DEF.IEF_ID,	$
		DEF.VDS_ORIENT, DEF.VDS_MAP, DEF.TEL_RATE, DEF.DURATION, $
		DEF.USABLE, STATUS=STATUS
	IF STATUS EQ 0 THEN BEGIN
           MESSAGE = 'Write to raster variation database was not successful'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	ENDIF
;
;  Update the raster variation index number in the structure.  Signal success.
;
	DEF.RAS_VAR = RAS_VAR
	RESULT = 1
;
;  Close the database, and return whether the routine was successful or not.
;
FINISH:
	DBCLOSE
;
	RETURN, RESULT
	END
