	FUNCTION ADD_V_STUDY, DEF, ERRMSG=ERRMSG, STUDY=DBSTUDY,	$
		CHECK_ONLY=CHECK_ONLY
;+
; Project     :	SOHO - CDS
;
; Name        :	ADD_V_STUDY()
;
; Purpose     :	Adds a study variation definition to the database
;
; Explanation :	This procedure takes the parameters that define a study
;		variation, and adds them to the databases "study_var" and
;		"study_var2".  The study variation index number is assigned.
;
; Use         :	Result = ADD_V_STUDY( DEF )
;
;		IF NOT ADD_V_STUDY( DEF ) THEN ...
;
; Inputs      :	DEF = This is an anonymous structure containing the following
;		      tags:
;
;			STUDY_ID    = Study ID number.  Must match an entry in
;				      the fundamental study database.
;			TITLE_ID    = Study title ID number.  Must match an
;				      entry in the study title database.
;			STUDYVAR    = Study variation index.  Initially, this
;				      is zero.  It is updated by this routine
;				      to reflect the index in the database.
;			SV_DESC	    = A short description of the study
;				      variation beyond what is given in the
;				      associated fundamental study description.
;				      Must be no more than 65 characters.
;			N_RASTERS0  = Fixed part of the number of rasters in
;				      the study.
;			DURATION0   = Fixed part of the study duration, in
;				      seconds.
;			DURATION1   = Number of seconds/raster for the variable
;				      part of the study duration.
;			ZONE_ID	    = Pointing zone ID.
;			USABLE	    = Either "Y" or "N" to signal whether or
;				      not the study is usable.  Normally "Y".
;			RASTERS	    = A array listing the raster variations to
;				      be used during the study.
;
;		      The raster descriptions themselves are structures, of
;		      type "cds_v_st_ras", with the following tags:
;
;			RAS_VAR	    = The raster variation index to be used
;				      during the study.  The index refers to
;				      the corresponding raster ID in the
;				      "study2" table.
;			N_REPEAT_R  = Number of times to repeat this raster.
;
;		      If N_RASTER_DEF in the corresponding fundamental study is
;		      0, then the RASTERS tag can be omitted.
;
; Opt. Inputs :	None.
;
; Outputs     :	The result of the function is a logical value representing
;		whether or not the operation was successful, where 1 is
;		successful and 0 is unsuccessful.
;
;		The tag STUDYVAR in the input structure is updated to reflect
;		the ID number assigned in the database.
;
; Opt. Outputs:	None.
;
; Keywords    :
;       ERRMSG    = If defined and passed, then any error messages will be
;                   returned to the user in this parameter rather than
;                   depending on the MESSAGE routine in IDL.  If no errors are
;                   encountered, then a null string is returned.  In order to
;                   use this feature, ERRMSG must be defined first, e.g.
;
;                       ERRMSG = ''
;                       Result = ADD_V_STUDY( ERRMSG=ERRMSG, ... )
;                       IF ERRMSG NE '' THEN ...
;
;	STUDY	  = If the definition matches one already in the database,
;		    then this keyword returns a vector containing the STUDY_ID
;		    and STUDYVAR of the match.  Otherwise, it returns the
;		    single value -1.
;
;	CHECK_ONLY= If this keyword is set, then the routine only checks to see
;		    if the proposed entry is valid, but does not attempt to
;		    place it in the database.
;
; Calls       :	DATATYPE, DBOPEN, DB_INFO, DBBUILD, DBCLOSE, DBEXT, DBFIND,
;		DBMATCH
;
; Common      :	None.
;
; Restrictions:	Only this routine can be used to add study variation
;		definitions to the database.  Modifying the database by hand
;		could corrupt its integrity.
;
;		The data types and sizes of the structure elements must match
;		the definitions in the database.  The string lengths must not
;		exceed the lengths defined in the database.
;
;		!PRIV must be 2 or greater to use this routine.
;
; Side effects:	Once a study variation definition has been added to the
;		database, it must never be removed.
;
; Category    :	Planning, Databases.
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 29 April 1994
;
; Modified    :	Version 1, William Thompson, GSFC, 3 May 1994
;               Version 2, Liyun Wang, GSFC/ARC, September 22, 1994
;                  Added the keyword ERRMSG.
;               Version 3, Liyun Wang, GSFC/ARC, November 4, 1994
;                  CHK_V_STUDY is called to check the existence of the 
;                  given study variation
;		Version 4, William Thompson, GSFC, 8 November 1994
;			Added keyword STUDY
;		Version 5, William Thompson, GSFC, 21 November 1994
;			Added items N_RASTERS0 and N_REPEAT_R
;			Changed RAS_VAR into a structure containing RAS_VAR and
;			N_REPEAT_R.
;		Version 6, William Thompson, GSFC, 29 December 1994
;			Fixed bug when inputs are bytes.
;		Version 7, William Thompson, GSFC, 2 March 1995
;			Enforce uniqueness for SV_DESC.
;			Added keyword CHECK_ONLY.
;		Version 8, William Thompson, GSFC, 14 March 1995
;			Use DBMATCH instead of DBFIND for SV_DESC
;			Check values of DURATION0 and DURATION1.
;		Version 9, William Thompson, GSFC, 29 March 1995
;			Added tag ZONE_ID
;		Version 10, William Thompson, GSFC, 12 May 1995
;			Modified to split off separate study_title database
;		Version 11, William Thompson, GSFC, 20 June 1995
;			Determines FLAGABLE parameter
;		Version 11, William Thompson, GSFC, 6 October 1995
;			Modified to allow for cases with N_RASTER_DEF=0.
;		Version 12, William Thompson, GSFC, 27 October 1995.
;			Removed tag flagable.  Any study can produce a flag if
;			given an IEF_ID.
;		Version 13, William Thompson, GSFC, 26 February 1996
;			Removed requirements that DURATION0 and DURATION1 must
;			be positive.  Negative values indicate that the
;			durations have been impirically determined.
;
; Version     :	Version 13, 26 February 1996
;-
;
	ON_ERROR, 2
;
;  Make sure that the STUDY keyword is initialized, in case the program
;  exits before the CHK_V_STUDY routine is reached.
;
	DBSTUDY = -1
;
;  Initialize RESULT to represent non-success.  If the routine is successful,
;  this value will be updated below.
;
	RESULT = 0
;
;  Check the input parameters
;
        IF N_PARAMS() NE 1 THEN BEGIN
           MESSAGE = 'Syntax:  Result = ADD_V_STUDY(DEF)'
	   GOTO, HANDLE_ERROR
        ENDIF
;
;  Make sure that the user has privilege to write into the database.
;
	IF !PRIV LT 2 THEN BEGIN $
           MESSAGE = '!PRIV must be 2 or greater to write into the database'
           GOTO, HANDLE_ERROR
	ENDIF
;
;  Make sure that the study variation index is less than or equal to zero.
;
	IF DEF.STUDYVAR GT 0 THEN BEGIN
           MESSAGE = 'The study variation index must be LE 0.  '+$
              'The final value is assigned by this routine'
           GOTO, HANDLE_ERROR
	ENDIF
;
;  Check each of the structure components to verify that it is of the correct
;  type and size.
;
	IF DATATYPE(DEF.STUDY_ID,1) NE 'Integer' THEN BEGIN
           MESSAGE = 'Tag STUDY_ID must be a short integer'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(DEF.STUDY_ID) NE 1 THEN BEGIN
           MESSAGE = 'Tag STUDY_ID must be a scalar'
           GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(DEF.TITLE_ID,1) NE 'Integer' THEN BEGIN
           MESSAGE = 'Tag TITLE_ID must be a short integer'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(DEF.TITLE_ID) NE 1 THEN BEGIN
           MESSAGE = 'Tag TITLE_ID must be a scalar'
           GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(DEF.SV_DESC,1) NE 'String' THEN BEGIN
           MESSAGE = 'Tag SV_DESC must be a character string'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(DEF.SV_DESC) NE 1 THEN BEGIN
           MESSAGE = 'Tag SV_DESC must be a scalar'
           GOTO, HANDLE_ERROR
	END ELSE IF STRLEN(DEF.SV_DESC) GT 65 THEN BEGIN
           MESSAGE = 'Tag SV_DESC must have no more than 65 characters'
           GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(DEF.N_RASTERS0,1) NE 'Integer' THEN BEGIN
           MESSAGE = 'Tag N_RASTERS0 must be a short integer'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(DEF.N_RASTERS0) NE 1 THEN BEGIN
           MESSAGE = 'Tag N_RASTERS0 must be a scalar'
           GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(DEF.DURATION0,1) NE 'Float' THEN BEGIN
           MESSAGE = 'Tag DURATION0 must be a floating point number'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(DEF.DURATION0) NE 1 THEN BEGIN
           MESSAGE = 'Tag DURATION0 must be a scalar'
           GOTO, HANDLE_ERROR
;	END ELSE IF DEF.DURATION0 LT 0 THEN BEGIN
;	   MESSAGE = 'Tag DURATION0 must not be negative'
;	   GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(DEF.DURATION1,1) NE 'Float' THEN BEGIN
           MESSAGE = 'Tag DURATION1 must be a floating point number'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(DEF.DURATION1) NE 1 THEN BEGIN
           MESSAGE = 'Tag DURATION1 must be a scalar'
           GOTO, HANDLE_ERROR
;	END ELSE IF DEF.DURATION1 LE 0 THEN BEGIN
;	   MESSAGE = 'Tag DURATION1 must be positive'
;	   GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(DEF.ZONE_ID,1) NE 'Integer' THEN BEGIN
           MESSAGE = 'Tag ZONE_ID must be a short integer'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(DEF.ZONE_ID) NE 1 THEN BEGIN
           MESSAGE = 'Tag ZONE_ID must be a scalar'
           GOTO, HANDLE_ERROR
	END ELSE IF DEF.ZONE_ID LE 0 THEN BEGIN
	   MESSAGE = 'Tag ZONE_ID must be positive'
           GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(DEF.USABLE,1) NE 'String' THEN BEGIN
           MESSAGE = 'Tag USABLE must be a character string'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(DEF.USABLE) NE 1 THEN BEGIN
           MESSAGE = 'Tag USABLE must be a scalar'
           GOTO, HANDLE_ERROR
	END ELSE IF STRLEN(DEF.USABLE) NE 1 THEN BEGIN
           MESSAGE = 'Tag USABLE must be a single character'
           GOTO, HANDLE_ERROR
	END ELSE IF (DEF.USABLE NE 'Y') AND (DEF.USABLE NE 'N') THEN BEGIN
           MESSAGE = 'Tag USABLE must be either "Y" or "N"'
           GOTO, HANDLE_ERROR
	ENDIF
;
;  Make sure that STUDY_ID points to a valid entry in the fundamental study
;  database, and get the value of N_RASTER_DEF.
;
	DBOPEN, 'study'
	ENTRY = DBFIND('STUDY_ID=' + STRTRIM(LONG(DEF.STUDY_ID),2),/SILENT)
	IF !ERR EQ 0 THEN BEGIN
           MESSAGE = 'Study ID does not exist in database'
           GOTO, HANDLE_ERROR
	ENDIF
	DBEXT, ENTRY, 'N_RASTER_DEF', N_RASTER_DEF
	N_RASTER_DEF = N_RASTER_DEF(0)
;
;  If N_RASTER_DEF is nonzero, then extract the first instance of the structure
;  in the RASTERS tag, and check its tags.
;
	IF N_RASTER_DEF NE 0 THEN BEGIN
	    TEST = (DEF.RASTERS)(0)
;
	    IF DATATYPE(TEST.RAS_VAR,1) NE 'Integer' THEN BEGIN $
	       MESSAGE = 'Tag RAS_VAR must be of type short integer'
	       GOTO, HANDLE_ERROR
	    END ELSE IF N_ELEMENTS(TEST.RAS_VAR) NE 1 THEN BEGIN
	       MESSAGE = 'Tag RAS_VAR must be a scalar'
	       GOTO, HANDLE_ERROR
	    ENDIF
;
	    IF DATATYPE(TEST.N_REPEAT_R,1) NE 'Integer' THEN BEGIN $
	       MESSAGE = 'Tag N_REPEAT_R must be of type short integer'
	       GOTO, HANDLE_ERROR
	    END ELSE IF N_ELEMENTS(TEST.N_REPEAT_R) NE 1 THEN BEGIN
	       MESSAGE = 'Tag N_REPEAT_R must be a scalar'
	       GOTO, HANDLE_ERROR
	    ENDIF
	ENDIF
;----------------------------------------------------------------------
;       Check the existence of the study variation
;----------------------------------------------------------------------
	dbstudy = chk_v_study(def)
	IF N_ELEMENTS(dbstudy) GT 1 THEN BEGIN
	   MESSAGE = 'Study variation definition already exists. '+$
	      'Study ID = '+num2str(dbstudy(0))+', variation ID = '+$
	      num2str(dbstudy(1))
	   GOTO, HANDLE_ERROR
	ENDIF
;
;  Make sure that TITLE_ID points to a valid entry in the fundamental study
;  title database.
;
	DBOPEN, 'study_title'
	TITLE_PTR = DBFIND('TITLE_ID=' + STRTRIM(DEF.TITLE_ID,2),/SILENT)
	IF !ERR EQ 0 THEN BEGIN
	   MESSAGE = 'Title ID does not exist in database'
	   GOTO, HANDLE_ERROR
	ENDIF
	TITLE_PTR = LONG(TITLE_PTR(0))
;
;  Make sure that the STUDY_ID matches that in the database for the TITLE_ID.
;
	DBEXT, TITLE_PTR, 'study_id', STUDY_ID
	IF DEF.STUDY_ID NE STUDY_ID(0) THEN BEGIN
	   MESSAGE = 'Tag STUDY_ID is not consistent with TITLE_ID'
	   GOTO, HANDLE_ERROR
	ENDIF
;
;  Make sure that the number of raster variation indices match the number of
;  corresponding entries in the study2 database, and that each variation index
;  is valid.  Save the entry numbers and RAS_ID for later.
;
	IF N_RASTER_DEF NE 0 THEN BEGIN
	    DBOPEN, 'study2,study,raster'
	    STUDY2_PTR = DBFIND('STUDY_ID='+STRTRIM(LONG(DEF.STUDY_ID),2), $
		    /SILENT)
	    IF N_ELEMENTS(DEF.RASTERS) NE N_ELEMENTS(STUDY2_PTR) THEN BEGIN
	       MESSAGE = 'Number of raster variations does not match ' + $
	          'number of raster IDs'
	       GOTO, HANDLE_ERROR
	    ENDIF
	    DBEXT, STUDY2_PTR, 'ras_id', RAS_ID
;
;  At the same time one is checking to make sure that the raster variations
;  exist, also extract the data window ID numbers.
;
	    DBOPEN, 'raster_var,raster'
	    RASTER_VAR_PTR = STUDY2_PTR
	    FOR I = 0,N_ELEMENTS(DEF.RASTERS)-1 DO BEGIN
		TEST = DBFIND('RAS_ID=' + STRTRIM(LONG(RAS_ID(I)),2) +	$
			',RAS_VAR='+STRTRIM(LONG(DEF.RASTERS(I).RAS_VAR),2), $
			/SILENT)
		IF !ERR EQ 0 THEN BEGIN
		   MESSAGE = 'Raster variation does not exist'
		   GOTO, HANDLE_ERROR
		ENDIF
		RASTER_VAR_PTR(I) = TEST(0)
		DBEXT, TEST(0), 'dw_id', DW_ID
		IF I EQ 0 THEN DW_IDS = DW_ID ELSE DW_IDS = [DW_IDS, DW_ID]
	    ENDFOR
	ENDIF
;
;  Find the total number of currently defined study variations for the
;  selected study ID number in the database, and add one to it.
;
	DBOPEN, 'study_var,study_title,study'
	ENTRIES = DBFIND('STUDY_ID=' + STRTRIM(LONG(DEF.STUDY_ID),2), /SILENT)
	IF !ERR EQ 0 THEN STUDYVAR = 1 ELSE BEGIN
		STUDYVAR = FIX(N_ELEMENTS(ENTRIES) + 1)
;
;  Make sure that SV_DESC does not match one already in the database for that
;  study ID.
;
		ENTRIES = DBMATCH('SV_DESC', STRTRIM(DEF.SV_DESC,2),	$
			ENTRIES, /FULLSTRING)
		IF ENTRIES(0) GT 0 THEN BEGIN
		    MESSAGE = 'Variation description already in database'
		    GOTO, HANDLE_ERROR
		ENDIF
	ENDELSE
;
;  If the CHECK_ONLY keyword was set, then return now as if the routine were
;  successful.  However, don't modify the study variation index in the
;  definition.
;
	IF KEYWORD_SET(CHECK_ONLY) THEN BEGIN
		RESULT = 1
		GOTO, FINISH
	ENDIF
;
;  Open the primary study variation database for write access.  Find the number
;  of entries and add one to it--that will be the pointer for the new entry.
;
	DBOPEN, 'study_var', 1
	STUDY_VAR_PTR = DB_INFO('entries','study_var') + 1
;
;  Append the data to the opened database.
;
	DBBUILD, DEF.STUDY_ID, TITLE_PTR, DEF.TITLE_ID, STUDYVAR,	$
		STRTRIM(DEF.SV_DESC,2), DEF.N_RASTERS0, DEF.DURATION0,	$
		DEF.DURATION1, DEF.ZONE_ID, DEF.USABLE, STATUS=STATUS
	IF STATUS EQ 0 THEN BEGIN
	   MESSAGE = 'Write to study_var database was not successful'
	   GOTO, HANDLE_ERROR
	ENDIF
;
;  Open the secondary study variation database, and append the data in that
;  database.
;
	IF N_RASTER_DEF NE 0 THEN BEGIN
	    DBOPEN, 'study_var2', 1
	    N_RASTERS = N_ELEMENTS(DEF.RASTERS)
	    IND = INDGEN(N_RASTERS) + 1
	    STUDY_VAR_PTR = REPLICATE(LONG(STUDY_VAR_PTR), N_RASTERS)
	    STUDY_ID = REPLICATE(DEF.STUDY_ID, N_RASTERS)
	    SVAR = REPLICATE(STUDYVAR, N_RASTERS)
	    DBBUILD, STATUS=STATUS, STUDY_VAR_PTR, STUDY_ID, SVAR,	$
		LONG(STUDY2_PTR), IND, LONG(RASTER_VAR_PTR), RAS_ID,	$
		DEF.RASTERS.RAS_VAR, DEF.RASTERS.N_REPEAT_R
;
;  If the second write was unsuccessful, then delete the entry inserted above
;  into "study_var".
;
	    IF STATUS EQ 0 THEN BEGIN
	       MESSAGE = 'Write to study_var2 database was not successful.  '+$
	          'Returned study_var to its previous state'
	       PRIV = !PRIV  &  !PRIV = 3
	       DBDELETE, STUDY_VAR_PTR(0), 'study_var'
	       !PRIV = PRIV
	       GOTO, HANDLE_ERROR
	    ENDIF
	ENDIF
;
;  Update the study variation index in the structure.  Signal success.
;
	DEF.STUDYVAR = STUDYVAR
	RESULT = 1
	GOTO, FINISH
;
;  Error handling point.
;
HANDLE_ERROR:
	IF N_ELEMENTS(ERRMSG) NE 0 THEN ERRMSG = 'ADD_V_STUDY: ' + MESSAGE $
		ELSE MESSAGE, MESSAGE, /CONTINUE
;
;  Close the database, and return whether the routine was successful or not.
;
FINISH:
	DBCLOSE
;
	RETURN, RESULT
	END
