	FUNCTION ADD_WAVECAL, DATE_EFF, DETECTOR, COEFF, GSET_ID=GSET_ID, $
		FILENAME=FILENAME, ERRMSG=ERRMSG
;+
; Project     :	SOHO - CDS
;
; Name        :	ADD_WAVECAL()
;
; Purpose     :	Register wavelength calibrations in the database.
;
; Category    :	Calibration, GIS, NIS, Wavelength
;
; Explanation :	Adds a wavelength calibration to the database.  The calibration
;		is marked with the current date/time.
;
; Syntax      :	Result = ADD_WAVECAL( DATE_EFF, DETECTOR, COEFF  $
;			[, GSET_ID=GSET_ID ] )
;
; Examples    :	IF NOT ADD_WAVECAL('1995-12-2','N',COEFF) THEN ...
;
; Inputs      :	DATE_EFF = The date, and optionally time, that the value
;			   becomes effective.  In other words, the calibration
;			   value should be applied to all science data taken on
;			   or after that date.  Can be in any CDS time format.
;
;			   The current date is also stored along with the data.
;			   When the database is read in, the most current
;			   version for a given date is used.
;
;			   For example, if there were two values of DATE_EFF in
;			   the database,
;
;				DATE_EFF = 1995/12/02
;				DATE_EFF = 1996/09/16
;
;			   then the first would be used for all data taken
;			   between 1995/12/02 and 1996/09/16, and the second
;			   would be used for all data from 1996/09/16 on.  If
;			   there was more than one entry in the database with
;			   an effective date of 1996/09/16, then the most
;			   recent one would be used.
;
;			   There is one thing to watch out for in maintaining
;			   the database.  Taking the above example, if one
;			   added a calibration with an effective date of
;			   1996/05/01, then it would be applied to data between
;			   that date and the next date in the database,
;			   i.e. 1996/09/16.  If one wanted the new calibration
;			   to apply to all data after 1996/05/01, then one
;			   would have to also add a new entry for 1996/09/16.
;
;		DETECTOR = Either "NIS" or "GIS" (can be abbreviated).
;
;		COEFF	 = The wavelength calibration coefficients.  The
;			   dimensions depend on the spectrometer used.
;
;			   For the NIS spectrometer, the dimensions are (2,2,2)
;			   with the following meanings:
;
;				Dim1 = NIS1 (0) or NIS2 (1)
;				Dim2 = Linear or quadratic coefficients
;				Dim3 = Coefficients are for wave->pix (0) or
;				       for pix->wave (1)
;
;			   For the GIS spectrometer, the dimensions are (4,3,2)
;			   with the following meanings:
;
;				Dim1 = Detector number (0-3)
;				Dim2 = Quadratic coefficients
;				Dim3 = Coefficients are for wave->pix (0) or
;				       for pix->wave (1)
;
; Opt. Inputs :	None.
;
; Outputs     :	The result of the function is a logical value representing
;		whether or not the operation was successful, where 1 is
;		successful and 0 is unsuccessful.
;
; Opt. Outputs:	None.
;
; Keywords    :	GSET_ID	 = The GIS Setup ID that the wavelength calibration
;			   refers to.  This is a required keyword for adding
;			   GIS wavelength calibrations.
;
;		FILENAME = The name of the file that the calibration is based
;			   on.  If not passed, then the null string is used.
;
;		ERRMSG	 = If defined and passed, then any error messages will
;			   be returned to the user in this parameter rather
;			   than depending on the MESSAGE routine in IDL.  If no
;			   errors are encountered, then a null string is
;			   returned.  In order to use this feature, ERRMSG must
;			   be defined first, e.g.
;
;				ERRMSG = ''
;				Result = ADD_WAVECAL( ERRMSG=ERRMSG, ... )
;				IF ERRMSG NE '' THEN ...
;
; Calls       :	DATATYPE, GET_UTC, UTC2TAI, DBOPEN, DBBUILD, DBCLOSE
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Prev. Hist. :	None.
;
; History     :	Version 1, 05-March-1996, William Thompson, GSFC
;               Version 2, 15-March-1996, CDP, RAL 
;                          Allow gset_id=0 input as a special case not
;                          requiring corresponding entry in GSET database.
;		Version 3, 05-Jul-1996, William Thompson, GSFC
;			Change NIS coefficients to second order.
;		Version 4, 18-Apr-1997, William Thompson, GSFC
;			Added DATE_EFF parameter.
;			Corrected call to PRIV_ZDBASE.
;
; Contact     :	WTHOMPSON
;-
;
	ON_ERROR, 2
;
;  Initialize RESULT to represent non-success.  If the routine is successful,
;  this value will be updated below.
;
	RESULT = 0
;
;  Check the number of parameters.
;
	IF N_PARAMS() NE 3 THEN BEGIN
		MESSAGE = 'Syntax:  Result = '	+ $
			'ADD_WAVECAL( DATE_EFF, DETECTOR, COEFF )'
		GOTO, HANDLE_ERROR
	ENDIF
;
;  Make sure that the user has privilege to write into the database.
;
	IF NOT PRIV_ZDBASE(/CALIB) THEN BEGIN
	    MESSAGE = 'No write privilege to database'
	    GOTO, HANDLE_ERROR
	END ELSE IF !PRIV LT 2 THEN BEGIN $
            MESSAGE = '!PRIV must be 2 or greater to write into the database'
            GOTO, HANDLE_ERROR
	ENDIF
;
;  Make sure that the input parameters have the right datatypes.
;
	IF DATATYPE(DETECTOR,1) NE 'String' THEN BEGIN
		MESSAGE = 'DETECTOR must be of type string'
		GOTO, HANDLE_ERROR
	END ELSE IF DATATYPE(COEFF,1) NE 'Double' THEN BEGIN
		MESSAGE = 'COEFF must be double precision'
		GOTO, HANDLE_ERROR
	ENDIF
;
;  Make sure that DETECTOR is a scalar.
;
	IF N_ELEMENTS(DETECTOR) NE 1 THEN BEGIN
		MESSAGE = 'DETECTOR must be a scalar'
		GOTO, HANDLE_ERROR
	ENDIF
;
;  Determine which detector is being considered.
;
	DET = STRUPCASE(STRMID(DETECTOR,0,1))
	IF (DET NE 'N') AND (DET NE 'G') THEN BEGIN
		MESSAGE = 'DETECTOR must be either N or G'
		GOTO, HANDLE_ERROR
	ENDIF
;
;  Check the dimensions of COEFF, depending on the detector.
;
	SZ = SIZE(COEFF)
	IF SZ(0) NE 3 THEN BEGIN
		MESSAGE = 'COEFF must be three-dimensional'
		GOTO, HANDLE_ERROR
	ENDIF
;
	CASE DET OF
	    'N': IF (SZ(1) NE 2) OR (SZ(2) NE 3) OR (SZ(3) NE 2) THEN BEGIN
		MESSAGE = 'COEFF must be 2x3x2 for the NIS'
		GOTO, HANDLE_ERROR
		END
	    'G': IF (SZ(1) NE 4) OR (SZ(2) NE 3) OR (SZ(3) NE 2) THEN BEGIN
		MESSAGE = 'COEFF must be 4x3x2 for the GIS'
		GOTO, HANDLE_ERROR
		END
	ENDCASE
;
;  If the GIS detector was selected, then make sure that GSET_ID was passed,
;  and that it points to a valid entry in the database.
;
	IF DET EQ 'G' THEN BEGIN
		IF N_ELEMENTS(GSET_ID) NE 1 THEN BEGIN
			MESSAGE = 'Keyword GSET_ID must be passed for the GIS'
			GOTO, HANDLE_ERROR
		ENDIF
                IF GSET_ID GT 0 THEN BEGIN
	    	   GET_GSET, GSET_ID, DEF
		   IF DEF.GSET_ID LT 0 THEN BEGIN
		  	   MESSAGE = 'GSET_ID=' + TRIM(GSET_ID) +	$
				   ' not found in database'
			   GOTO, HANDLE_ERROR
		   ENDIF
                ENDIF
	ENDIF
;
;  Convert the effective date to TAI format.  Reformat the time to millisecond
;  accuracy.  This is necessary so that the times are written out in a
;  controlled way.
;
	TAI_EFF = DOUBLE( STRING( ANYTIM2TAI(DATE_EFF), FORMAT='(F15.3)' ))
;
;  Get the current date, and convert to a TAI value.
;
	GET_UTC, UTC
	TAI = UTC2TAI(UTC)
;
;  If the FILENAME hasn't been defined, then define it to the null string.
;  Make sure that it has the right definition.
;
	IF N_ELEMENTS(FILENAME) EQ 0 THEN FILE = '' ELSE BEGIN
		FILE = FILENAME
		IF DATATYPE(FILE,1) NE 'String' THEN BEGIN
			MESSAGE = 'FILENAME must be a string'
			GOTO, HANDLE_ERROR
		END ELSE IF N_ELEMENTS(FILE) GT 1 THEN BEGIN
			MESSAGE = 'FILENAME must be a scalar'
			GOTO, HANDLE_ERROR
		ENDIF
	ENDELSE
;
;  Open the appropriate database and insert the data.
;
	CASE DET OF
	    'N': BEGIN
		DBOPEN, 'nis_wave', 1
		DBBUILD, TAI, TAI_EFF, FILE, COEFF(*), STATUS=STATUS
		IF STATUS EQ 0 THEN BEGIN
		    MESSAGE = 'Write to nis_wave database was not successful'
		    GOTO, HANDLE_ERROR
		ENDIF
		END
	    'G': BEGIN
		DBOPEN, 'gis_wave', 1
		DBBUILD, TAI, TAI_EFF, GSET_ID, FILE, COEFF, STATUS=STATUS
		IF STATUS EQ 0 THEN BEGIN
		    MESSAGE = 'Write to gis_wave database was not successful'
		    GOTO, HANDLE_ERROR
		ENDIF
		END
	ENDCASE
;
;  If we got this far, then we must have been successful.
;
	RESULT = 1
	GOTO, FINISH
;
;  Error handling point.
;
HANDLE_ERROR:
	IF N_ELEMENTS(ERRMSG) NE 0 THEN ERRMSG = 'ADD_WAVECAL: ' + MESSAGE $
		ELSE MESSAGE, MESSAGE, /CONTINUE
;
;  Close the database, and return whether the routine was successful or not.
;
FINISH:
	DBCLOSE
;
	RETURN, RESULT
	END
