;+
; Project     : SOHO - CDS     
;                   
; Name        : CALC_RAS_DUR()
;                
; Purpose     : Calculate the duration of a raster.
;               
; Explanation : Uses the raster definition to calciulate the expected duration
;               of the raster.  Takes account of exposure time, telemetry load
;               and mechanism motion.
;
; Use         : IDL> dur = calc_ras_dur(tp_raster_str)    or
;                    dur = calc_ras_dur(raster_str)       or
;                    dur = calc_ras_dur(ras_id, ras_var_id)
;
; Inputs      : tp_raster_str  - structure as used by technical  planning
;                                mk_raster (ie for use in mk_raster or
;                                with the structure returned by mk_raster)
;                    or
;               raster_str     - the raster structure returned by get_raster
;                    or
;               ras_id         -  the fundamental raster ID and
;               ras_var_id     -  the raster variation ID
;               
; Opt. Inputs : None
;               
; Outputs     : Function returns the duration in seconds.
;               
; Opt. Outputs: None
;               
; Keywords    : TIME2DUMP - returns the telemetry time per exposure in secs.
;
; Calls       : 
;               
; Restrictions: None
;               
; Side effects: None
;               
; Category    : Technical planning.
;               
; Prev. Hist. : None
;
; Written     : C D Pike, RAL, 9-May-95
;               
; Modified    : Make so that exposure time and VDS readout are added to
;               telemetry dump time for first exposure. Thereafter take the
;               greater of the two.    CDP, 2-Jun-95 
;
;               Isolate numerical variables used and add overall fudge
;               factor.   CDP, 2-Jun-95 
;
;               Change state database call to cp_get_state.  CDP, 07-Sep-95
;
;               Fix typo in sum over window compression. CDP, 25-Sep-95
;               Fix duration calculated for sum over windows. CDP, 26-Sep-95
;               Fix bug when input is get_raster output. CDP, 6-Oct-95
;
;               Change compression factor for VBLW (mode 3) from 2.0 to
;               1.25 (based on results of SYNOP study).  CDP, 13-Oct-95
; 
;               Update sum compression scheme IDs.  CDP, 2-Nov-95
;
;               Cut out fudge factor and add 1.0 second overhead per
;               exposure.                           CDP, 22-Nov-95
;
;               Include extra factor for NIS pixel scale change and
;               when default NIS extraction is used assume the 
;               default cover-all extraction (because of slope) 
;                                                   CDP, 16-Feb-96
;
;               Change telemetry rate used to be actual 11.14 Kbits/s rather
;               than estimated 10.0.                CDP, 19-Feb-96
;              
;               Change call to cp_get_state to get_vds_state.  CDP, 23-Feb-96
;
;               Allow for window ht variation when default NIS extraction
;               used.                               CDP, 28-Mar-96
;
;
; Version     : Version 13, 28-Mar-96
;-            

function calc_ras_dur, p1, p2, time2dump=time2dump

;
;  setup hardcoded variables.  These should come from the state database
;  eventually.

tlm_rate = 11140L          ;tlm bits/sec
dtime    = 14.0            ;VDS CCD time for 1 pixel digitization (microsecs)
ndtime   = 2.0             ;VDS CCD time for 1 pixel non-digit....
smtime   = 33.0            ;stepper motor msecs/step
ovhtime  = 120.0           ;motor overhead time per step (msecs)
fudge    = 1.0             ;0% fudge factor

;
;  what is the input variable?
;
;
;  Two parameters given so assume they were fundamental and variation IDs
;
if n_params() eq 2 then begin
      if datatype(p1,1) eq 'Integer' and datatype(p2,1) eq 'Integer' then begin
         errmsg = ''
         get_raster,p1,p2,raster,errmsg=errmsg
         if errmsg ne '' then begin
            print,errmsg
            return, -1
         endif
         expose     = raster.exptime
         nwin       = raster.n_windows
         compid     = raster.comp_id
         vds_orient = raster.vds_orient
         detector   = raster.detector
         nx         = raster.nx
         ny         = raster.ny
         xstep      = raster.xstep
         ystep      = raster.ystep

         get_datawin,raster.dw_id, dw
         if dw.dw_id lt 0 then begin
            time2dump = 0.0
            return, -1
         endif
      endif else begin
         print,'Two parameters must be integer IDs.'
         return, -1
      endelse
endif

;
;  if a single parameter then must be a structure of either ...
;
if n_params() eq 1 then begin
   if datatype(p1,1) ne 'Structure' then begin
      print,'Single parameter must be a structure.'
      return, -1
   endif
   case n_tags(p1) of
;
;  internal type used in mk_raster or ...
;
      10: begin
             expose     = p1.raster_v.exptime
             compid     = p1.raster_v.comp_id
             vds_orient = p1.raster_v.vds_orient
             detector   = p1.raster_p.detector
             nx         = p1.raster_p.nx
             ny         = p1.raster_p.ny
             xstep      = p1.raster_p.xstep
             ystep      = p1.raster_p.ystep
             dwid       = p1.raster_v.dw_id
             nwin       = p1.data_win.n_windows
             dw         = {wins:p1.data_win2}
          end
;
; ...type returned by get_raster
;
      28: begin
             expose     = p1.exptime
             compid     = p1.comp_id
             vds_orient = p1.vds_orient
             detector   = p1.detector
             nx         = p1.nx
             ny         = p1.ny
             xstep      = p1.xstep
             ystep      = p1.ystep
             nwin       = p1.n_windows

             get_datawin, p1.dw_id, dw
             if dw.dw_id lt 0 then begin
                time2dump = 0.0
                return, -1
             endif
          end
    else: begin
             print,'Unrecognised structure.'                       
             return, -1
          end
   endcase
endif


;
;  calculate rough telemetry requirements. 
;
if detector eq 'N' then pix_fac = 1.2 else pix_fac = 1.0
totpix = 0L
if nwin gt 0 then begin
   for i=0,nwin-1 do begin
      new =  (long(dw.wins(i).win_def(2))*long(dw.wins(i).win_def(3)))*pix_fac
      if strmid(dw.wins(i).win_name,0,6) eq 'BB_NIS' then begin
         if dw.wins(i).win_def(3) eq 120 then begin
            new = 1024L*157L
         endif else begin
            new = 1024L*dw.wins(i).win_def(3)
         endelse
      endif
      totpix = totpix + long(new)
   endfor
endif

nbits = totpix*16L
time2dump = float(nbits/tlm_rate)

;
;  allow for compression scheme (approximate factors only)
;
case compid of
   1: time2dump = time2dump
   2: time2dump = time2dump/1.33
   3: time2dump = time2dump/1.25
   6: begin
         if vds_orient eq 0 then begin
            totpix = 0
            for i=0,nwin-1 do begin
               totpix = totpix + long(dw.wins(i).win_def(3))
            endfor
            nbits = totpix*16L
            time2dump = float(nbits/tlm_rate)
         endif else begin
            totpix = 0
            for i=0,nwin-1 do begin
               totpix = totpix + long(dw.wins(i).win_def(2))
            endfor
            nbits = totpix*16L
            time2dump = float(nbits/tlm_rate)
         endelse
      end
   7: begin
         n = where(dw.wins.win_name ne '')
         if n(0) ge 0 then begin
            totpix = 4 * n_elements(n)
         endif else begin
            totpix = 0
         endelse
         nbits = totpix*16L
         time2dump = float(nbits/tlm_rate)
      end
   else:
endcase

;
;  hence time to dump in telemetry
;
time2dump = round_off(time2dump,0.1)

;
;  raster duration (many assumptions may be wrong) 
;
;  for exposures only  - take which ever is the greater of dump or
;                        (exposure time + readout overheads)
;

;  For VDS
;  read out 2 microsecs per skipped pixel, 14 microsecs per digitised pixel
;  plus 0.5 secs recovery.  (See setup variables for current definitions)
;
if detector eq 'N' then begin
   get_utc,utc
   get_vds_state,utc2tai(utc),s
   vdsew = get_vds_win(rotate(dw.wins.win_def,1),s.vds_read) 
;
;  depending on the readout mode, different numbers of pixels can be 
;  digitized simultaneously
;
   case s.vds_read of
      1: sumpix = 512L*512L
      2: sumpix = 512L*1024L
      3: sumpix = 512L*1024L
      4: sumpix = 1024L*1024L
      5: sumpix = 1024L*1024L
      6: sumpix = 1024L*1024L
      7: sumpix = 1024L*1024L
   else: begin
            print,'VDS readout mode not known.'
            return, -1
         end
   endcase
   
   nv = where(vdsew(*,3) gt 0)
   if nv(0) ge 0 then begin
      dig = 0L
      for i=0,n_elements(nv)-1 do begin
         dig = dig + long(vdsew(i,2)) * long(vdsew(i,3))
      endfor
      nodig = sumpix - dig
      readout_overhead = (ndtime/1000.d0/1000.d0)*nodig+$
                          (dtime/1000.d0/1000.d0)*dig+0.5
   endif else begin
      readout_overhead = 0.0
   endelse
endif else begin
   readout_overhead = 0.0
endelse

;
; add 1.0 seconds as extra inter-exposure delay
;
readout_overhead = readout_overhead + 1.0



;
;  For first exposure, integration time and telemetry are sequential operations
;  for subsequent exposures the exposure can happen while telemetry dump
;  of previous exposure is happening
;
maxtime1 = time2dump + (expose + readout_overhead)
maxtime  = time2dump > (expose + readout_overhead)
rasdur = maxtime1+ maxtime * ( ( float(nx)*float(ny) ) - 1)

;
;  overheads per move (120 msec/move)
;  x = mirror, y = slit movement
;
if xstep gt 0 then begin
   nxmoves = (nx-1)*ny
   nflyback = ny-1
endif else begin
   nxmoves = 0
   nflyback = 0
endelse

if ystep gt 0 then begin
   nymoves = ny-1
endif else begin
   nymoves = 0
endelse
 
overhead = (long(nxmoves)+nflyback+nymoves)*ovhtime/1000.d0


;
;  33 msec per stepper motor step
;
xmove  = nxmoves * xstep/2.0d0 * smtime/1000.d0
fbmove = (nx-1) * (xstep/2.0d0) * nflyback *  smtime/1000.0d0
ymove  = nymoves * ystep * smtime/1000.0d0
overhead = overhead + xmove + fbmove + ymove

rasdur = long((rasdur + overhead + 0.5)*fudge)

return,rasdur

end
