;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Document name: chk_datawin.pro
; Created by:    Liyun Wang, GSFC/ARC, September 28, 1994
;
; Last Modified: Tue Oct 25 17:54:00 1994 (lwang@orpheus.gsfc.nasa.gov)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
FUNCTION CHK_DATAWIN, datawin, errmsg=errmsg
;+
; PROJECT:
;       SOHO - CDS
;
; NAME:
;       CHK_DATAWIN()
;
; PURPOSE:
;       Check the existence in CDS database of a given datawin
;
; EXPLANATION:
;       A valid datawin structure should contain the following tags:
;
;             DW_ID    = Data window list ID number.  Initially, this
;                        is zero.  It is updated by this routine to
;                        reflect the ID number in the database.
;             DETECTOR = Either "G" for GIS or "N" for NIS.
;             DW_DESC  = A short description of the data window list
;                        beyond what is given in the associated line
;                        list description, e.g. "Full slit, 10 pixels
;                        wide".  Can be up to 50 characters.
;             W_WIDTH  = Width in pixels used to generate the
;                        windows.
;             W_HEIGHT = The height in pixels used to generate the
;                        windows.  VDS only, for GIS this is set to
;                        unity.
;             VDS_BACK = Either 0 for off, or 1 for on,
;                        representing whether or not VDS background
;                        windows are being used.  For GIS window
;                        lists this must be 0.  If VDS_BACK is 1,
;                        then at least one of the window names must
;                        start with the characters "BACK" (case
;                        insensitive).
;             LL_ID    = The line list ID number for the line list
;                        that this window list was derived from.  An
;                        appropriate entry must exist in the line
;                        list database.  Alternatively, this can be
;                        set to zero to signal that the data window
;                        list is not connected to a line list.
;             WINS     = An array containing the window definitions.
;
;           The window definitions themselves are structures, of type
;           "cds_windows", with the following tags:
;
;             WIN_NAME = Window name, derived from the line name in
;                        the line list database.  Can be up to 40
;                        characters.
;             WIN_DEF  = The window definition.  This is a 4-element
;                        integer array, although GIS window definitions
;                        only use two of them.
;	      WIN_FLAG = Either 1 (true) or 0 (false) if window can generate a
;			 flag.
;
;       This routine will check ALL tags of the given datawin, except
;       DW_ID and DW_DESC, against entries found from the database to see if
;       value of each tag matches. 
;
;       This routine does NOT check to see if the given datawin
;       structure is valid. It should, therefore, be called after such
;       checkings are made.
;
; CALLING SEQUENCE:
;       Result = CHK_DATAWIN(datawin [,errmsg=errmsg])
;
; INPUTS:
;       DATAWIN -- Strcuture containing the data window definition
;
; OPTIONAL INPUTS:
;       None.
;
; OUTPUTS:
;       Result -- -1 if the given datawin does not exist in the database; 
;                 -2 indication of syntax error or wrong data type;
;                 integer scalar or vector, indicating the datawin IDs
;                 (DW_ID) of the matched datawins.
;
; OPTIONAL OUTPUTS:
;       None.
;
; KEYWORD PARAMETERS:
;       ERRMSG -- If defined and passed, then any error messages will be
;                 returned to the user in this parameter rather than
;                 depending on the MESSAGE routine in IDL.  If no errors are
;                 encountered, then a null string is returned.  In order to
;                 use this feature, ERRMSG must be defined first, e.g.
;
;                    ERRMSG = ''
;                    RESULT = CHK_DATAWIN(datawin, ERRMSG=ERRMSG, ...)
;                    IF ERRMSG NE '' THEN ...
;
; CALLS:
;       DATATYPE, DBOPEN, DBFIND, DBEXT, NUM2STR
;
; COMMON BLOCKS:
;       None.
;
; RESTRICTIONS:
;       None.
;
; SIDE EFFECTS:
;       None.
;
; CATEGORY:
;       Planning, Databases
;
; PREVIOUS HISTORY:
;       Written September 28, 1994, by Liyun Wang, GSFC/ARC
;
; MODIFICATION HISTORY:
;       Version 2, Liyun Wang, GSFC/ARC, October 11, 1994
;          ALL tags of a given datawin structure are checked now.
;          Cater for empty database.  CDP, 3-Nov-94
;	Version 4, William Thompson, GSFC, 20 June 1995
;		Add tag WIN_FLAG
;	Version 5, 16-May-1997, William Thompson, GSFC
;		Fix bug involving strings starting with blank.
;
; VERSION:
;       Version 5, 16-May-1997
;-
;
   ON_ERROR, 2

   IF N_ELEMENTS(datawin) EQ 0 THEN BEGIN
      MESSAGE = 'Syntax: Result = CHK_DATAWIN(datawin [,errmsg=errmsg])'
      GOTO, HANDLE_ERROR
   ENDIF
   IF datatype(datawin) NE 'STC' THEN BEGIN
      MESSAGE = 'The input datawin must be a structure.'
      GOTO, HANDLE_ERROR
   ENDIF

   n_wins =  N_ELEMENTS(datawin.wins) 
   dbopen, 'datawin2,datawin'
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;  Search for data window entries which have the same detector name and 
;  the same number of data windows as the given data window
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
   entries = dbfind('DETECTOR='+STRTRIM(datawin.detector,2)+$
                    ',W_WIDTH='+num2str(datawin.w_width)+$
                    ',W_HEIGHT='+num2str(datawin.w_height)+$
                    ',VDS_BACK='+num2str(datawin.vds_back)+$
                    ',LL_ID='+num2str(datawin.ll_id)+$
                    ',N_WINDOWS='+num2str(n_wins),$
                    /silent)

   IF entries(0) LE 0 THEN RETURN, -1

   given_names = STRTRIM(datawin.wins.win_name,2)
   IF N_ELEMENTS(given_names) GT 1 THEN ib = SORT(given_names) ELSE ib = 0

;  Get entry number and datawin ID:
   dbext, entries, ['ENTRY','DW_ID'], new_entry, id_list
  
;  Figure out how many data windows are found
   uniq_id = uniq(id_list)

;  For each window list, we have to check all windows in it to see
;  if all of them are identical to the given window list.
   offset = 0
   FOR i = 0, N_ELEMENTS(uniq_id)-1 DO BEGIN
      sub_entry = new_entry(offset:uniq_id(i))
      dbext,sub_entry,['WIN_NAME','WIN_DEF','WIN_FLAG'], win_names, win_def, $
		win_flags
      n_tags = 0
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;     Make a sorting so that orders of windows do not matter
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; 
      win_names = STRTRIM(win_names,2)
      IF N_ELEMENTS(win_names) GT 1 THEN ia = SORT(win_names) ELSE ia = 0
      ii = WHERE(win_names(ia) EQ given_names(ib), count)
      IF count EQ n_wins THEN n_tags = n_tags+1
      temp = 0
      FOR j =  0, n_wins-1 DO BEGIN
         jj = WHERE(win_def(*,ia(j)) EQ datawin.wins(ib(j)).win_def, count) 
         IF count EQ 4 THEN temp = temp+1 
      ENDFOR
      IF temp EQ n_wins THEN n_tags = n_tags+1
      ii = WHERE(STRTRIM(win_flags(ia),2) EQ datawin.wins(ib).win_flag, count)
      IF count EQ n_wins THEN n_tags = n_tags+1
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;     All tags have been checked so far. Now if n_tags equals to 3,
;     it means that the current datawin is identical to the given one
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
      IF n_tags EQ 3 THEN BEGIN
         IF N_ELEMENTS(match) EQ 0 THEN match = id_list(uniq_id(i)) $
         ELSE match =  [match,id_list(uniq_id(i))]
      ENDIF
      offset = uniq_id(i)+1
   ENDFOR
   IF N_ELEMENTS(match) NE 0 THEN RETURN, match ELSE RETURN, -1
;
;  Error handling point.
;
HANDLE_ERROR:
	IF N_ELEMENTS(ERRMSG) NE 0 THEN ERRMSG = 'CHK_DATAWIN: ' + MESSAGE $
		ELSE MESSAGE, MESSAGE, /CONTINUE
	RETURN, -2
;
END

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; End of 'chk_datawin.pro'.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
