;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Document name: chk_f_raster.pro
; Created by:    Liyun Wang, GSFC/ARC, October 13, 1994
;
; Last Modified: Tue Oct 25 17:39:31 1994 (lwang@orpheus.gsfc.nasa.gov)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
FUNCTION CHK_F_RASTER, f_raster, errmsg=errmsg
;+
; PROJECT:
;       SOHO - CDS
;
; NAME:
;       CHK_F_RASTER()
;
; PURPOSE:
;       Check existence in CDS database of a given raster structure
;
; EXPLANATION:
;       Currently a f_raster structure has the following tags:
;
;          RAS_ID   = Raster ID number.  Initially, this is
;                     zero.  It is updated by this routine to
;                     reflect the ID number in the database.
;          DETECTOR = Either "G" for GIS or "N" for NIS.
;          RAS_DESC = A short description of the raster, giving
;                     its purpose.
;          SLIT_NUM = The ID number for the slit to be used.
;          XSTEP    = The step size in the X direction, in arcsec.
;          YSTEP    = The step size in the Y direction, in arcsec.
;          NX       = The number of exposure positions in X.
;          NY       = The number of exposure positions in Y.
;
;       This routine will check ALL tags of the given f_raster, except
;       RAS_ID and RAS_DESC against entries found from the database to
;       see if value of each tag matches.
;
;       This routine does NOT check to see if the given f_raster
;       structure is valid. It should, therefore, be called after such
;       checkings are made.
;
; CALLING SEQUENCE:
;       Result = CHK_F_RASTER(f_raster [,errmsg=errmsg])
;
; INPUTS:
;       F_RASTER -- A strcuture containing the f_raster definition
;
; OPTIONAL INPUTS:
;       None.
;
; OUTPUTS:
;       Result -- -1 if the given f_raster does not exist in the database;
;                 -2 for syntax error or wrong type of input parameter;
;                 Integer scalar or vector indicating the ID number of existed
;                    f_raster if found in the database 
;
; OPTIONAL OUTPUTS:
;       None.
;
; KEYWORD PARAMETERS:
;       ERRMSG -- If defined and passed, then any error messages will be
;                 returned to the user in this parameter rather than
;                 depending on the MESSAGE routine in IDL.  If no errors are
;                 encountered, then a null string is returned.  In order to
;                 use this feature, ERRMSG must be defined first, e.g.
;
;                    ERRMSG = ''
;                    RESULT = CHK_F_RASTER(f_raster, ERRMSG=ERRMSG, ...)
;                    IF ERRMSG NE '' THEN ...
;
; CALLS:
;       DATATYPE, DBOPEN, DBFIND, NUM2STR
;
; COMMON BLOCKS:
;       None.
;
; RESTRICTIONS:
;       None.
;
; SIDE EFFECTS:
;       None.
;
; CATEGORY:
;       Planning, Databases
;
; PREVIOUS HISTORY:
;       Written October 13, 1994, by Liyun Wang, GSFC/ARC
;
; MODIFICATION HISTORY:
;       Cater for empty database.  CDP, 3-Nov-94
;
; VERSION:
;       Version 2, 3-Nov-94
;-
;
   ON_ERROR, 2

   IF N_ELEMENTS(f_raster) EQ 0 THEN BEGIN
      MESSAGE = 'Syntax: Result = CHK_F_RASTER(f_raster [,errmsg=errmsg])'
      IF KEYWORD_SET(errmsg) THEN BEGIN
         errmsg = MESSAGE
         RETURN, -2
      ENDIF ELSE $
         MESSAGE, MESSAGE
   ENDIF

   IF datatype(f_raster) NE 'STC' THEN BEGIN
      MESSAGE = 'The input f_raster must be a structure.'
      IF KEYWORD_SET(errmsg) THEN BEGIN
         errmsg = MESSAGE
         RETURN, -2
      ENDIF ELSE $
         MESSAGE, MESSAGE
   ENDIF

   dbopen, 'raster'
;----------------------------------------------------------------------
;  Unimportant items should be removed (i.e., commented out) from the
;  following searching
;----------------------------------------------------------------------
   entries = dbfind('DETECTOR='+STRTRIM(f_raster.DETECTOR,2)+$
                    ',SLIT_NUM='+num2str(f_raster.SLIT_NUM)+$
                    ',XSTEP='+STRTRIM(f_raster.XSTEP,2)+$
                    ',YSTEP='+STRTRIM(f_raster.YSTEP,2)+$
                    ',NX='+STRTRIM(f_raster.nx,2)+$
                    ',NY='+STRTRIM(f_raster.ny,2),$
                    /silent)
   IF entries(0) LE 0 THEN RETURN, -1

;  Get entry number and program ID from the above entries:
   dbext, entries, ['ENTRY','RAS_ID'], new_entry, id_list

;  Figure out how many lists are found (should be only 1 in reality)
   uniq_id = uniq(id_list)

   RETURN, id_list(uniq_id)

END

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; End of 'chk_f_raster.pro'.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
