FUNCTION CHK_F_STUDY, f_study, errmsg=errmsg
;+
; PROJECT:
;       SOHO - CDS
;
; NAME:
;       CHK_F_STUDY()
;
; PURPOSE:
;       Check existence in CDS database of a given fundamental study
;
; EXPLANATION:
;       Currently a fundamental study structure has 8 tags. They are:
;          STUDY_ID    -- Study ID
;          OBS_PROG    -- Name of the study
;          TITLE       -- Title of the study
;          CATEGORY    -- The study category
;	   VAR_POINT   -- Boolean parameter that indicates whether the pointing
;			  of the study is controllable
;	   N_RASTER_DEF-- Number of rasters in the definition.
;          RASTERS     -- Definition of rasters in the study (if
;			   N_RASTER_DEF>0). It itself is a structure that has
;			   the following tags:
;             RAS_ID   -- Raster ID number 
;             POINTING -- How to treat pointing parameters INS_X, INS_Y
;             INS_X    -- X position of the instrument pointing
;             INS_Y    -- Y position of the instrument pointing
;       
;       This routine will check ALL tags of the given f_study, except
;       STUDY_ID, OBS_PROG, TITLE and CATEGORY, against entries found from the
;       database to see if value of each tag matches. 
;
;       Two fundamental studies are said to be identical only if
;       values of all tags of the two studies are identical.
;
;       This routine does NOT check to see if the given f_study
;       structure is valid. It should, therefore, be called after such
;       checkings are made.
;
; CALLING SEQUENCE:
;       Result = CHK_F_STUDY(f_study [,errmsg=errmsg])
;
; INPUTS:
;       F_STUDY -- A strcuture containing the study definition
;
; OPTIONAL INPUTS:
;       None.
;
; OUTPUTS:
;       Result -- -1 if the given f_study does not exist in the database;
;                 -2 for syntax error or wrong type of input parameter;
;                 Integer scalar or vector indicating the ID number of existed
;                    f_study if found in the database 
;
; OPTIONAL OUTPUTS:
;       None.
;
; KEYWORD PARAMETERS:
;       ERRMSG -- If defined and passed, then any error messages will be
;                 returned to the user in this parameter rather than
;                 depending on the MESSAGE routine in IDL.  If no errors are
;                 encountered, then a null string is returned.  In order to
;                 use this feature, ERRMSG must be defined first, e.g.
;
;                    ERRMSG = ''
;                    RESULT = CHK_F_STUDY(f_study, ERRMSG=ERRMSG,..)
;                    IF ERRMSG NE '' THEN ...
;
; CALLS:
;       DATATYPE, DBOPEN, DBFIND, DBEXT, NUM2STR
;
; COMMON BLOCKS:
;       None.
;
; RESTRICTIONS:
;       CHK_F_STUDY must not be called if N_RASTER_DEF=0.
;
; SIDE EFFECTS:
;       None.
;
; CATEGORY:
;       Planning, Databases
;
; PREVIOUS HISTORY:
;       Written October 11, 1994, by Liyun Wang, GSFC/ARC
;
; MODIFICATION HISTORY:
;       Cater for empty database.  CDP, 3-Nov-94         
;	Version 3, William Thompson, GSFC, 7 November 1994
;		Maintain original order of rasters, don't sort.
;	Version 4, William Thompson, GSFC, 21 November 1994
;		Removed item N_RASTERS0
;		Added item N_RASTER_DEF
;	Version 5, William Thompson, GSFC, 10 February 1995
;		Changed N_POINTINGS to VAR_POINT.
;		Removed VAR_RASTERS.
;
; VERSION:
;       Version 5, 10 February 1995
;-
;
   ON_ERROR, 2

   IF N_ELEMENTS(f_study) EQ 0 THEN BEGIN
      MESSAGE = 'Syntax: Result = CHK_F_STUDY(f_study [,errmsg=errmsg])'
      IF KEYWORD_SET(errmsg) THEN BEGIN
         errmsg = MESSAGE
         RETURN, -2
      ENDIF ELSE $
         MESSAGE, MESSAGE
   ENDIF

   IF datatype(f_study) NE 'STC' THEN BEGIN
      MESSAGE = 'The input f_study must be a structure.'
      IF KEYWORD_SET(errmsg) THEN BEGIN
         errmsg = MESSAGE
         RETURN, -2
      ENDIF ELSE $
         MESSAGE, MESSAGE
   ENDIF

   dbopen, 'study2,study'
   entries = dbfind('N_RASTER_DEF='+num2str(f_study.n_raster_def),$
                    /silent)

   IF entries(0) LE 0 THEN RETURN, -1

   n_rasters = N_ELEMENTS(f_study.rasters)
   given_names = f_study.rasters.ras_id
;  IF N_ELEMENTS(given_names) GT 1 THEN ib = SORT(given_names) ELSE ib = 0
   ib = lindgen(n_elements(given_names))

;  Get entry number and f_study ID from the above entries:
   dbext, entries, ['ENTRY','STUDY_ID'], new_entry, id_list

;  Figure out how many f_studys are found
   uniq_id = uniq(id_list)

;  For each study, we have to check all lines in it to see
;  if all of them are identical to the given study.
   offset = 0
   FOR i = 0, N_ELEMENTS(uniq_id)-1 DO BEGIN
      sub_entry = new_entry(offset:uniq_id(i))
      dbext,sub_entry,['RAS_ID','POINTING','INS_X','INS_Y'],$
                       ras_id, pointing, ins_x, ins_y
      n_tags = 0
;     IF N_ELEMENTS(ras_id) GT 1 THEN ia = SORT(ras_id) ELSE ia = 0
      ia = lindgen(n_elements(ras_id))
      ii = WHERE(ras_id(ia) EQ given_names(ib), count)
      IF count EQ n_rasters THEN n_tags = n_tags+1
      ii = WHERE(pointing(ia) EQ f_study.rasters(ib).pointing, count)
      IF count EQ n_rasters THEN n_tags = n_tags+1
      ii = WHERE(ins_x(ia) EQ f_study.rasters(ib).ins_x, count)
      IF count EQ n_rasters THEN n_tags = n_tags+1
      ii = WHERE(ins_y(ia) EQ f_study.rasters(ib).ins_y, count)
      IF count EQ n_rasters THEN n_tags = n_tags+1
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;     All tags have been checked so far. Now if n_tags equals to 4,
;     it means that the current f_study is identical to the given one
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
      IF n_tags EQ 4 THEN BEGIN
         IF N_ELEMENTS(match) EQ 0 THEN match = id_list(uniq_id(i)) $
         ELSE match =  [match,id_list(uniq_id(i))]
      ENDIF
      offset = uniq_id(i)+1
   ENDFOR
   IF N_ELEMENTS(match) NE 0 THEN RETURN, match ELSE RETURN, -1
END

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; End of 'chk_f_study.pro'.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
