;+
; PROJECT:
;       SOHO - CDS
;
; NAME:
;       CHK_LINELIST()
;
; PURPOSE:
;       Check the existence in CDS database of a given linelist
;
; EXPLANATION:
;       Currently a linelist structure has 5 tags. They are:
;          LL_ID    -- Line list ID number
;          DETECTOR -- Either 'G' or 'N'
;          LL_DESC  -- Description of the line list
;          LINES    -- A structure array for all lines in the
;                       list. It has 5 tags for each line:
;             LINES.LINENAME -- Line name
;             LINES.WAVELNTH -- Wavelength in Angstroms
;             LINES.GR_ORDER -- Grating order
;             LINES.CEN_PIX  -- Central detector pixel
;             LINES.WAVEBAND -- Wavelength band
;
;       This routine will check ALL tags of the given linelist, except LL_ID
;       and LL_DESC, against entries found from the database to see if value of
;       each tag matches.
;
;       Two line lists are said to be identical only if values of all checked
;       tags of the two line lists are identical.
;
;       This routine does NOT check to see if the given linelist
;       structure is valid. It should, therefore, be called after such
;       checkings are made.
;
; CALLING SEQUENCE:
;       Result = CHK_LINELIST(linelist [,errmsg=errmsg])
;
; INPUTS:
;       LINELIST -- A strcuture containing the line list definition
;
; OPTIONAL INPUTS:
;       None.
;
; OUTPUTS:
;       Result -- -1 if the given linelist does not exist in the database; 
;                 -2 indication of syntax error or wrong data type;
;                 integer scalar or vector, indicating the linelist IDs
;                 (LL_ID) of the matched linelists.
;
; OPTIONAL OUTPUTS:
;       None.
;
; KEYWORD PARAMETERS:
;       ERRMSG -- If defined and passed, then any error messages will be
;                 returned to the user in this parameter rather than
;                 depending on the MESSAGE routine in IDL.  If no errors are
;                 encountered, then a null string is returned.  In order to
;                 use this feature, ERRMSG must be defined first, e.g.
;
;                    ERRMSG = ''
;                    RESULT = CHK_LINELIST(linelist, ERRMSG=ERRMSG, ...)
;                    IF ERRMSG NE '' THEN ...
;
; CALLS:
;       DATATYPE, DBOPEN, DBFIND, DBEXT, NUM2STR
;
; COMMON BLOCKS:
;       None.
;
; RESTRICTIONS:
;       None.
;
; SIDE EFFECTS:
;       None.
;
; CATEGORY:
;       Planning, Databases
;
; PREVIOUS HISTORY:
;       Written September 28, 1994, by Liyun Wang, GSFC/ARC
;
; MODIFICATION HISTORY:
;       Version 2, Liyun Wang, GSFC/ARC, October 6, 1994
;          Now returns ID of the linelist found in the database. 
;       Version 3, Liyun Wang, GSFC/ARC, October 11, 1994
;          ALL tags of a given linelist structure are checked now.
;       Version 4  CDP, cater for empty databases.  3-Nov-94
;       Version 5, CDP, exclude central pixel from uniqueness test.
;	Version 6, 16-May-1997, William Thompson, GSFC
;		Fix bug involving strings starting with blank.
;	Version 7, 05-Oct-1999, William Thompson, GSFC
;		Fixed bug involving sub-milliAngstrom wavelength differences
;		Tighten way sorting is done.
;
; VERSION:
;       Version 7, 05-Oct-1999
;-
;
FUNCTION CHK_LINELIST, linelist, errmsg=errmsg

   ON_ERROR, 2

   IF N_ELEMENTS(linelist) EQ 0 THEN BEGIN
      MESSAGE = 'Syntax: Result = CHK_LINELIST(linelist [,errmsg=errmsg])'
      IF KEYWORD_SET(errmsg) THEN BEGIN
         errmsg = MESSAGE
         RETURN, -2
      ENDIF ELSE $
         MESSAGE, MESSAGE
   ENDIF

   IF datatype(linelist) NE 'STC' THEN BEGIN
      MESSAGE = 'The input linelist must be a structure.'
      IF KEYWORD_SET(errmsg) THEN BEGIN
         errmsg = MESSAGE
         RETURN, -2
      ENDIF ELSE $
         MESSAGE, MESSAGE
   ENDIF

   n_lines = N_ELEMENTS(linelist.lines)
   dbopen, 'linelist2,linelist'
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;  Search for linelist entries which have the same detector name and 
;  number of lines as the given line list
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
   entries = dbfind('DETECTOR='+STRTRIM(linelist.detector,2)+$
                    ',N_LINES='+num2str(n_lines),$
                    /silent)

   IF entries(0) LE 0 THEN RETURN, -1

   given_names = STRTRIM(linelist.lines.linename,2)
   IF N_ELEMENTS(given_names) GT 1 THEN ib = SORT(given_names +	$
	   STRTRIM(linelist.lines.wavelnth,2)) ELSE ib = 0
   
;  Get entry number and linelist ID from the above entries:
   dbext, entries, ['ENTRY','LL_ID'], new_entry, id_list

;  Figure out how many linelists are found
   uniq_id = uniq(id_list)

;  For each line list, we have to check all lines in it to see
;  if all of them are identical to the given line list.
   offset = 0
   FOR i = 0, N_ELEMENTS(uniq_id)-1 DO BEGIN
      sub_entry = new_entry(offset:uniq_id(i))
      dbext,sub_entry,['LINENAME','WAVELNTH','GR_ORDER','CEN_PIX',$
                       'WAVEBAND'],line_names,wavelnth,gr_order,cen_pix,$
                       waveband

      n_tags = 0
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;     Make a sorting so that orders of lines do not matter
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; 
      line_names = STRTRIM(line_names,2)
      IF N_ELEMENTS(line_names) GT 1 THEN ia = SORT(line_names +	$
	   STRTRIM(wavelnth,2)) ELSE ia = 0
      ii = WHERE(line_names(ia) EQ given_names(ib), count)
      IF count EQ n_lines THEN n_tags = n_tags+1
      ii = WHERE(round(1000*wavelnth(ia)) EQ	$
	      round(1000*linelist.lines(ib).wavelnth), count)
      IF count EQ n_lines THEN n_tags = n_tags+1
      ii = WHERE(gr_order(ia) EQ linelist.lines(ib).gr_order, count)
      IF count EQ n_lines THEN n_tags = n_tags+1
;      ii = WHERE(cen_pix(ia) EQ linelist.lines(ib).cen_pix, count)
;      IF count EQ n_lines THEN n_tags = n_tags+1
      ii = WHERE(waveband(ia) EQ linelist.lines(ib).waveband, count)
      IF count EQ n_lines THEN n_tags = n_tags+1
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;     All tags have been checked so far. Now if n_tags equals to 4,
;     it means that the current linelist is identical to the given one
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
      IF n_tags EQ 4 THEN BEGIN
         IF N_ELEMENTS(match) EQ 0 THEN match = id_list(uniq_id(i)) $
         ELSE match =  [match,id_list(uniq_id(i))]
      ENDIF
      offset = uniq_id(i)+1
   ENDFOR
   IF N_ELEMENTS(match) NE 0 THEN RETURN, match ELSE RETURN, -1
END

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; End of 'chk_linelist.pro'.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
