	FUNCTION CHK_T_STUDY, F_STUDY, EITHER=EITHER, ERRMSG=ERRMSG
;+
; Project     :	SOHO - CDS
;
; Name        :	CHK_T_STUDY()
;
; Purpose     :	Check existence in CDS database of a given study title
;
; Explanation :	This routine will check the STUDY_ID, OBS_PROG, and TITLE tags 
;		against entries found from the database to see if a match is
;		found.  If the EITHER keyword is set, then the STUDY_ID tag is
;		ignored, and a positive value is returned if either the
;		OBS_PROG or TITLE tags match.
;
;		Use CHK_T_STUDY without /EITHER to see if a given STUDY_ID,
;		OBS_PROG, TITLE combination already exists in the database or
;		needs to be added.
;
;		Use CHK_T_STUDY with /EITHER to check if the STUDY_ID,
;		OBS_PROG, TITLE combination can be added.
;
; Use         :	Result = CHK_T_STUDY(f_study [,errmsg=errmsg])
;
; Inputs      :	F_STUDY = A structure containing the study definition
;
; Opt. Inputs :	None.
;
; OUTPUTS     :	Result = -1 if the given study does not exist in the database,
;			 -2 for syntax error or wrong type of input parameter,
;			 Integer scalar or vector indicating the ID number of
;			 existing f_study if found in the database 
;
; Opt. Outputs:	None.
;
; Keywords    :	EITHER = If set, then the software will ignore the value of
;			 STUDY_ID, and check if either the OBS_PROG or TITLE
;			 tags already exists in the database.  When used with
;			 this keyword, a non-negative result does not mean that
;			 both the name and title are the same, but only that
;			 one or the other is the same.
;
;		ERRMSG = If defined and passed, then any error messages will be
;			 returned to the user in this parameter rather than
;			 depending on the MESSAGE routine in IDL.  If no errors
;			 are encountered, then a null string is returned.  In
;			 order to use this feature, ERRMSG must be defined
;			 first, e.g.
;
;				ERRMSG = ''
;				Result = CHK_T_STUDY(f_study, ERRMSG=ERRMSG,..)
;				IF ERRMSG NE '' THEN ...
;
;			 When used with the EITHER keyword, the value of ERRMSG
;			 will specify whether the name or the title was in
;			 conflict.
;
; Calls       :	DATATYPE, DBOPEN, DBFIND, DBEXT, NUM2STR
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side Effects:	None.
;
; Category    :	Planning, Databases
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 12 May 1995
;
; Modified    : Version 1, William Thompson, GSFC, 12 May 1995
;		Version 2, William Thompson, GSFC, 2 June 1995
;			Changed way ERRMSG is handled when /EITHER is used.
;
; Version     :	Version 2, 2 June 1995
;-
;
	ON_ERROR, 2
;
;  Initialize MESSAGE.
;
	MESSAGE = ''
;
	IF N_ELEMENTS(f_study) EQ 0 THEN BEGIN
		MESSAGE = 'Syntax:  Result = CHK_T_STUDY(F_STUDY)'
		GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(F_STUDY) NE 'STC' THEN BEGIN
		MESSAGE = 'The input F_STUDY must be a structure.'
		GOTO, HANDLE_ERROR
	ENDIF
;
	DBOPEN, 'study_title'
;
;  First, check to see if there are any studies with the same name.  If the
;  EITHER keyword was set, then return if any entry was found.  Otherwise,
;  return -1 if no entries were found and the EITHER keyword was not set.
;
	ENTRY = DBFIND('OBS_PROG=' + STRTRIM(F_STUDY.OBS_PROG,2), /SILENT, $
		/FULLSTRING)
	IF !ERR NE 0 THEN BEGIN
		IF KEYWORD_SET(EITHER) THEN BEGIN
			RESULT = ENTRY(0)
			MESSAGE = 'Conflicting study name, title ID = ' + $
				NUM2STR(RESULT)
			GOTO, FINISH
		ENDIF
	END ELSE BEGIN
		IF NOT KEYWORD_SET(EITHER) THEN BEGIN
			RESULT = -1
			GOTO, FINISH
		END ELSE ENTRY = -1
	ENDELSE
;
;  Next check for entries with the same title.  If one is found and the EITHER
;  keyword is set, then return.  Likewise, if none is found, then return.
;
	ENTRY = DBMATCH('TITLE', STRTRIM(F_STUDY.TITLE,2), ENTRY, /FULLSTRING)
	IF ENTRY(0) GT 0 THEN BEGIN
		IF KEYWORD_SET(EITHER) THEN BEGIN
			RESULT = ENTRY(0)
			MESSAGE = 'Conflicting study title, title ID = ' + $
				NUM2STR(RESULT)
			GOTO, FINISH
		ENDIF
	END ELSE BEGIN
		RESULT = -1
		GOTO, FINISH
	ENDELSE
;
;  Finally, if the EITHER keyword was not set, then check the STUDY_ID as well.
;
	ENTRY = DBFIND('STUDY_ID=' + NUM2STR(F_STUDY.STUDY_ID), ENTRY,	$
		/SILENT)
	IF !ERR NE 0 THEN RESULT = ENTRY(0) ELSE RESULT = -1
	GOTO, FINISH
;
;  Error handling point.
;
HANDLE_ERROR:
	IF N_ELEMENTS(ERRMSG) EQ 0 THEN MESSAGE, MESSAGE, /CONTINUE
	RESULT = -2
;
;  Exit point.  Even if an error didn't occur, then ERRMSG might be set.
;
FINISH:
	IF N_ELEMENTS(ERRMSG) NE 0 THEN ERRMSG = 'CHK_T_STUDY: ' + MESSAGE
	DBCLOSE
	RETURN, RESULT
	END
