;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Document name: chk_v_raster.pro
; Created by:    Liyun Wang, GSFC/ARC, October 13, 1994
;
; Last Modified: Tue Oct 25 17:44:56 1994 (lwang@orpheus.gsfc.nasa.gov)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
FUNCTION CHK_V_RASTER, v_raster, errmsg=errmsg
;+
; PROJECT:
;       SOHO - CDS
;
; NAME:
;       CHK_V_RASTER()
;
; PURPOSE:
;       Check the existence in CDS database of a raster variation
;
; EXPLANATION:
;       Currently a raster variation structure has the following tags:
;
;          RAS_ID     = Raster ID number.  Must match an entry in
;                       the fundamental raster database.
;          RAS_VAR    = Raster variation index.  Initially, this
;                       is zero.  It is updated by this routine to
;                       reflect the index in the database.
;          RV_DESC    = A short description of the raster
;                       variation beyond what is given in the
;                       associated fundamental raster description.
;          EXPTIME    = Exposure time in seconds, to millisecond
;                       accuracy.
;          LL_ID      = Line list ID.  Must match an entry in the
;                       line list database.  Must also be
;                       consistent with the corresponding DETECTOR
;                       parameter in the fundamental raster
;                       database.  Alternatively, this can be set
;                       to zero to signal that the raster is not
;                       connected to a line list.
;          COMP_ID    = Compression method ID.
;          COMP_OPT   = Compression option parameter.
;          DW_ID      = Data extraction window list ID.  Must
;                       match an entry in the data window list
;                       table.  Must also be consistent with the
;                       LL_ID parameter, and with the DETECTOR
;                       parameter from the fundamental raster
;                       definition.
;	   IEF_ID     = Inter-instrument event flag ID.
;          VDS_ORIENT = VDS orientation, either 0 (row) or 1
;                       (column).
;          VDS_MAP    = VDS mapping mode: 2=Normal, 3=Accumulate.
;	   TEL_RATE   = Estimated required telemetry rate, as a character
;			representing Low/Medium/High
;          DURATION   = Estimated duration of the raster, in
;                       seconds.
;          USABLE     = Either "Y" or "N" to signal whether or not
;                       the raster is usable.  Normally "Y".
;
;       This routine will check all tags of the given v_raster, except
;       RAS_VAR, RV_DESC, TEL_RATE and DURATION, against entries 
;       found from the database to see if value of each tag matches.
;
;       This routine does NOT check to see if the given v_raster
;       structure is valid. It should, therefore, be called after such
;       checkings are made.
;
; CALLING SEQUENCE:
;       Result = CHK_V_RASTER(v_raster [,errmsg=errmsg])
;
; INPUTS:
;       V_RASTER -- A strcuture containing the raster variation definition
;
; OPTIONAL INPUTS:
;       None.
;
; OUTPUTS:
;       Result -- -1 if the given v_raster does not exist in the database
;                 -2 for syntax error or wrong type of input parameter
;                 A 2D integer array containing raster IDs and
;                    variation numbers of associated raster variations
;                    found in the data base that have the same
;                    definition as the given raster var structure.
;
; OPTIONAL OUTPUTS:
;       None.
;
; KEYWORD PARAMETERS:
;       ERRMSG -- If defined and passed, then any error messages will be
;                 returned to the user in this parameter rather than
;                 depending on the MESSAGE routine in IDL.  If no errors are
;                 encountered, then a null string is returned.  In order to
;                 use this feature, ERRMSG must be defined first, e.g.
;
;                    ERRMSG = ''
;                    RESUTL = CHK_V_RASTER(v_raster, ERRMSG=ERRMSG, ...)
;                    IF ERRMSG NE '' THEN ...
;
; CALLS:
;       DATATYPE, DBOPEN, DBFIND, NUM2STR
;
; COMMON BLOCKS:
;       None.
;
; RESTRICTIONS:
;       None.
;
; SIDE EFFECTS:
;       None.
;
; CATEGORY:
;       Planning, Databases
;
; PREVIOUS HISTORY:
;       Written October 13, 1994, by Liyun Wang, GSFC/ARC
;
; MODIFICATION HISTORY:
;       Cater for empty database.  CDP, 3-Nov-94
;	Version 3, William Thompson, GSFC, 19 May 1995
;		Changed way DBFIND is called, to speed up.
;
; VERSION:
;       Version 3, 19 May 1995
;-
;
   ON_ERROR, 2

   IF N_ELEMENTS(v_raster) EQ 0 THEN BEGIN
      MESSAGE = 'Syntax: Result = CHK_V_RASTER(v_raster [,errmsg=errmsg])'
      IF KEYWORD_SET(errmsg) THEN BEGIN
         errmsg = MESSAGE
         RETURN, -2
      ENDIF ELSE $
         MESSAGE, MESSAGE
   ENDIF

   IF datatype(v_raster) NE 'STC' THEN BEGIN
      MESSAGE = 'The input v_raster must be a structure.'
      IF KEYWORD_SET(errmsg) THEN BEGIN
         errmsg = MESSAGE
         RETURN, -2
      ENDIF ELSE $
         MESSAGE, MESSAGE
   ENDIF

   dbopen, 'raster_var,raster'
;----------------------------------------------------------------------
;  Unimportant items should be removed (i.e., commented out) from the
;  following searching
;----------------------------------------------------------------------
   entries = dbfind('RAS_ID='+num2str(v_raster.ras_id)+$
                    ',EXPTIME='+num2str(v_raster.EXPTIME,FORMAT='(f15.3)')+$
                    ',LL_ID='+num2str(v_raster.LL_ID)+$
                    ',COMP_ID='+num2str(v_raster.COMP_ID)+$
                    ',COMP_OPT='+num2str(v_raster.COMP_OPT)+$
                    ',DW_ID='+num2str(v_raster.DW_ID)+$
                    ',VDS_ORIENT='+num2str(v_raster.VDS_ORIENT)+$
                    ',VDS_MAP='+num2str(v_raster.VDS_MAP)+$
                    ',USABLE='+STRTRIM(v_raster.USABLE,2),$
                    /silent)

   IF entries(0) LE 0 THEN RETURN, -1

;  Get entry number and program ID from the above entries:
   dbext, entries, ['ENTRY','RAS_ID','RAS_VAR'], new_entry, id_list, var_list

;  Figure out how many lists are found (should be only 1 in reality)
   uniq_id = uniq(id_list)

   result = [[id_list(uniq_id)],[var_list(uniq_id)]]

   RETURN, TRANSPOSE(result)

END

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; End of 'chk_v_raster.pro'.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
