FUNCTION CHK_V_STUDY, study, errmsg=errmsg
;+
; PROJECT:
;       SOHO - CDS
;
; NAME:
;       CHK_V_STUDY()
;
; PURPOSE:
;       Check existence in CDS database of a given study variation
;
; EXPLANATION:
;       Currently a study variation structure has the following tags:
;          STUDY_ID    -- Study ID
;	   TITLE_ID    -- Title ID
;          STUDYVAR    -- Study variation number
;          SV_DESC     -- Description of the study
;          N_RASTERS0  -- Total number of rasters in the study
;          DURATION0   -- Total duration of the study 
;          DURATION1   -- Number of seconds per raster for
;                         user-supplied rasters
;	   ZONE_ID     -- Pointing zone ID
;          USABLE      -- A signal of whether the study variation is usable
;	   RASTERS     -- A array listing the raster variations to be used
;			  during the study.
;
;	The raster descriptions themselves are structures, of type
;	"cds_v_st_ras", with the following tags:
;
;	   RAS_VAR     -- The raster variation index to be used during the
;			  study.  The index refers to the corresponding raster
;			  ID in the "study2" table.
;	   N_REPEAT_R  -- Number of times to repeat this raster.
;       
;	This routine will check the tags STUDY_ID, TITLE_ID, RAS_VAR, and
;	N_REPEAT_R of the given study against entries found from the database
;	to see if the value of each tag matches.
;
;       This routine does NOT check to see if the given study
;       structure is valid. It should, therefore, be called after such
;       checkings are made.
;
; CALLING SEQUENCE:
;       Result = CHK_V_STUDY(study [,errmsg=errmsg])
;
; INPUTS:
;       STUDY -- A strcuture containing the study definition
;
; OPTIONAL INPUTS:
;       None.
;
; OUTPUTS:
;       Result -- -1 if the given study does not exist in the database
;                 -2 for syntax error or wrong type of input parameter
;                 A 2D integer array containing study IDs and
;                    variation numbers of associated study variations
;                    found in the data base that have the same
;                    definition as the given study var structure.
;
; OPTIONAL OUTPUTS:
;       None.
;
; KEYWORD PARAMETERS:
;       ERRMSG -- If defined and passed, then any error messages will be
;                 returned to the user in this parameter rather than
;                 depending on the MESSAGE routine in IDL.  If no errors are
;                 encountered, then a null string is returned.  In order to
;                 use this feature, ERRMSG must be defined first, e.g.
;
;                    ERRMSG = ''
;                    RESULT = CHK_V_STUDY(study, ERRMSG=ERRMSG, ...)
;                    IF ERRMSG NE '' THEN ...
;
; CALLS:
;       DATATYPE, DBOPEN, DBFIND, DBEXT, NUM2STR, GET_F_STUDY
;
; COMMON BLOCKS:
;       None.
;
; RESTRICTIONS:
;       None.
;
; SIDE EFFECTS:
;       Always returns -1 if N_RASTER_DEF=0.
;
; CATEGORY:
;       Planning, Databases
;
; PREVIOUS HISTORY:
;       Written October 11, 1994, by Liyun Wang, GSFC/ARC
;
; MODIFICATION HISTORY:
;       Cater for empty database.  CDP, 3-Nov-94
;	Version 3, William Thompson & Liyun Wang, GSFC/ARC, 7 November 1994
;		Modified to use the same input structure as ADD_V_STUDY, and to
;		check only STUDY_ID and RAS_VAR.
;		Added call to DBCLOSE.
;	Version 4, William Thompson, GSFC, 21 November 1994
;		Added items N_RASTERS0, N_REPEAT_R.
;		Changed RAS_VAR into a structure containing RAS_VAR and
;		N_REPEAT_R.
;	Version 5, William Thompson, GSFC, 12 May 1995
;		Modified to split off separate study_title database
;	Version 6, William Thompson, GSFC, 17 May 1995
;		Modified to make TITLE_ID one of the parameters needed to make
;		a study variation unique.
;	Version 7, William Thompson, GSFC, 19 May 1995
;		Changed way DBFIND is called, to speed up.
;	Version 8, William Thompson, GSFC, 23 May 1995
;		Fixed bug where wrong raster variation number was returned.
;	Version 9, William Thompson, GSFC, 1 December 1995
;		Always return -1 if N_RASTER_DEF=0.
;
; Version     :
;	Version 9, 1 December 1995
;-
;
   ON_ERROR, 2

   IF N_ELEMENTS(study) EQ 0 THEN BEGIN
      MESSAGE = 'Syntax: Result = CHK_V_STUDY(study [,errmsg=errmsg])'
      IF KEYWORD_SET(errmsg) THEN BEGIN
         errmsg = MESSAGE
         RETURN, -2
      ENDIF ELSE $
         MESSAGE, MESSAGE
   ENDIF

   IF datatype(study) NE 'STC' THEN BEGIN
      MESSAGE = 'The input study must be a structure.'
      IF KEYWORD_SET(errmsg) THEN BEGIN
         errmsg = MESSAGE
         RETURN, -2
      ENDIF ELSE $
         MESSAGE, MESSAGE
   ENDIF

   dbopen, 'study_var2,study2,study_var,study_title,study'
   entries = dbfind('STUDY_ID='+num2str(study.study_id) +	$
		    ',TITLE_ID='+num2str(study.title_id) +	$
		    ',N_RASTERS0='+num2str(study.n_rasters0), /silent)

   IF entries(0) LE 0 THEN RETURN, -1
;
;  Get the fundamental study definition.
;
   get_f_study, study.study_id, fstudy, errmsg=errmsg
   IF N_ELEMENTS(ERRMSG) NE 0 THEN RETURN, -1
   IF FSTUDY.N_RASTER_DEF EQ 0 THEN RETURN, -1
   dbopen, 'study_var2,study2,study_var,study_title,study'
;
   n_rasters = N_ELEMENTS(fstudy.rasters)
   given_names = fstudy.rasters.ras_id
   ib = lindgen(n_elements(given_names))

;  Get entry number and study ID from the above entries:
   dbext, entries, ['ENTRY','STUDY_ID','STUDYVAR'], new_entry, $
      id_list, var_list

;  Figure out how many studys are found
   uniq_id = uniq(var_list)

;  For each study, we have to check all lines in it to see
;  if all of them are identical to the given study.
   offset = 0
   FOR i = 0, N_ELEMENTS(uniq_id)-1 DO BEGIN
      sub_entry = new_entry(offset:uniq_id(i))
      dbext,sub_entry,'RAS_VAR,N_REPEAT_R', ras_var, n_repeat_r
      n_tags = 0
      ia = lindgen(n_elements(ras_var))

      ii = WHERE((ras_var(ia) EQ study.rasters(ib).ras_var) and	$
		(n_repeat_r(ia) EQ study.rasters(ib).n_repeat_r), count)
      IF count EQ n_rasters THEN BEGIN
         IF N_ELEMENTS(match) EQ 0 THEN $
            match = [[id_list(uniq_id(i))], [var_list(uniq_id(i))]] $
         ELSE match = [match, [[id_list(uniq_id(i))], [var_list(uniq_id(i))]]]
      ENDIF
      offset = uniq_id(i)+1
   ENDFOR
   dbclose
   IF N_ELEMENTS(match) NE 0 THEN RETURN, TRANSPOSE(match) ELSE RETURN, -1
END

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; End of 'chk_v_study.pro'.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
