	PRO CONV_CAT_2
;+
; Project     :	SOHO - CDS
;
; Name        :	CONV_CAT_2
;
; Purpose     :	Converts format of catalog databases from 2 to 3
;
; Explanation :	Converts the catalog databases to a new format.  The following
;		changes are made:
;
;			MAIN:		Add new parameters OBS_PROG, DETECTOR
;					Correct OBJECT to three characters
;
;			EXPERIMENT:	Change indices to speed up
;			EXPERIMENT1:	New database between the other
;					two--contains DATASRC
;			EXPERIMENT2:	Change indices to speed up
;
; Use         :	One should use the following procedure to convert the databases
;		to the new formats:
;
;			1.  Make sure that the title lines of the old
;			    experiment.dbd and experiment2.dbd database files
;			    contain the phrase "Version 2", and that the new
;			    versions contain the phrase "Version 3".
;
;			2.  Make sure that the title line of the new main.dbd
;			    database file contains the phrase "Version 3"--the
;			    old title should contain no version information.
;
;			2.  Change to the directory containing the database
;			    files to be converted.  *** Very important ***
;
;			3.  Replace the old "main.dbd", "experiment.dbd" and
;			    "experiment2.dbd" files with the new versions.  You
;			    may wish to rename the old files to "old_main.dbd",
;			    etc.  Place the new "experiment1.dbd" file in the
;			    same directory.
;
;			4.  Start up IDL and compile this procedure with the
;			    command ".run conv_cat_2.prg".  Naming the file
;			    with the .prg extension is used to ensure that the
;			    routine is not executed accidently.
;
;			5.  Make sure that the software is enabled by entering
;			    the command "!priv = 5".  This is another
;			    protective step.
;
;			6.  Execute the procedure with the command
;			    "conv_cat_2".
;
;		During the conversion, the software will move the old
;		"main.dbf", ".dbh", and ".dbx" files to "old_main.dbf", etc.
;		The same is done with "experiment" and "experiment2".  When the
;		conversion is complete, you will be asked if you want to delete
;		these files or not.
;
; Category    :	This is not part of the CDS software tree.
;
; Written     :	William Thompson, GSFC, 1 February 1996
;-
;
	ON_ERROR, 2
;
;  Make sure that the user has the privilege set correctly to run this routine.
;
	IF !PRIV LT 5 THEN MESSAGE,	$
		'!PRIV must be set to 5 to run this routine.'
;
;  Make sure that the user really wants to convert the database.
;
	PRINT, 'Converting catalog databases to version 3.'
	ASK, 'Are you sure you want to do this? ', ANSWER
	IF ANSWER NE 'Y' THEN BEGIN
		PRINT, 'Execution aborted'
		RETURN
	ENDIF
;
;  Check to make sure that the proper database files are in the current
;  directory.
;
	MESSAGE = 'Database files not in current directory'
	ON_IOERROR, HANDLE_ERROR
	OPENR, UNIT, 'experiment2.dbf', /GET_LUN
	FREE_LUN, UNIT
;
;  Determine the correct commands for renaming and deletion.
;
	CASE OS_FAMILY() OF
		'vms': BEGIN
			RENAME = 'RENAME'
			DELETE = 'DELETE'
			END
		'unix': BEGIN
			RENAME = 'mv'
			DELETE = 'rm'
			END
		ELSE: MESSAGE, 'Only supported in VMS or Unix'
	ENDCASE
;
;  Preserve the old database files by renaming them.
;
	MESSAGE = 'Error occured'
;
	SPAWN, RENAME + ' main.dbf old_main.dbf'
	SPAWN, RENAME + ' main.dbx old_main.dbx'
	SPAWN, RENAME + ' main.dbh old_main.dbh'
;
	SPAWN, RENAME + ' experiment.dbf old_experiment.dbf'
	SPAWN, RENAME + ' experiment.dbx old_experiment.dbx'
	SPAWN, RENAME + ' experiment.dbh old_experiment.dbh'
;
	SPAWN, RENAME + ' experiment2.dbf old_experiment2.dbf'
	SPAWN, RENAME + ' experiment2.dbx old_experiment2.dbx'
	SPAWN, RENAME + ' experiment2.dbh old_experiment2.dbh'
;
;  Create empty versions of the new database files.
;
	DBCREATE, 'main',	 1, 1, /EXTERNAL
	DBCREATE, 'experiment',	 1, 1, /EXTERNAL
	DBCREATE, 'experiment1', 1, 1, /EXTERNAL
	DBCREATE, 'experiment2', 1, 1, /EXTERNAL
;
;******************************************************************************
;
;  Open the old main database file, and read in the data.
;
	DBOPEN, 'old_main'
	DBEXT, -1, 'PROG_NUM,PROG_ID,PROG_IND,STUDY_ID,STUDYVAR,SCI_OBJ', $
		PROG_NUM, PROG_ID, PROG_IND, STUDY_ID, STUDYVAR, SCI_OBJ
	DBEXT, -1, 'SCI_SPEC,CMP_NO,OBJECT,OBJ_ID,DATE_OBS,DATE_END',	$
		SCI_SPEC, CMP_NO, OBJECT, OBJ_ID, DATE_OBS, DATE_END
	DBEXT, -1, 'OBT_TIME,OBT_END,XCEN,YCEN,ANGLE,IXWIDTH,IYWIDTH',	$
		OBT_TIME, OBT_END, XCEN, YCEN, ANGLE, IXWIDTH, IYWIDTH
	DBEXT, -1, 'SEQ_FROM,SEQ_TO,COMMENTS,DATE_MOD',		$
		SEQ_FROM, SEQ_TO, COMMENTS, DATE_MOD
;
;  Correct some mistakes in the OBJECT information.
;
	W = WHERE(OBJECT EQ 'CO', COUNT)
	IF COUNT GT 0 THEN OBJECT(W) = 'COR'
	W = WHERE(OBJECT EQ 'FI', COUNT)
	IF COUNT GT 0 THEN OBJECT(W) = 'FIL'
	W = WHERE(OBJECT EQ 'LM', COUNT)
	IF COUNT GT 0 THEN OBJECT(W) = 'LMB'
	W = WHERE(OBJECT EQ 'NE', COUNT)
	IF COUNT GT 0 THEN OBJECT(W) = 'NET'
	W = WHERE(OBJECT EQ 'PL', COUNT)
	IF COUNT GT 0 THEN OBJECT(W) = 'PLG'
;
;  Determine the new information to put in the database.
;
	NN = N_ELEMENTS(PROG_NUM)
	OBS_PROG = STRARR(NN)
	DETECTOR = STRARR(NN)
	COUNTDOWN, NN, /OPEN
	FOR I=0,NN-1 DO BEGIN
	    GET_STUDY, STUDY_ID(I), STUDYVAR(I), S
	    IF S.STUDY_ID GE 0 THEN BEGIN
		OBS_PROG(I) = S.OBS_PROG
		FOR J = 0,S.N_RASTER_DEF-1 DO BEGIN
		    GET_RASTER, S.RASTERS(J).RAS_ID, S.RASTERS(J).RAS_VAR, R
		    IF DETECTOR(I) EQ '' THEN BEGIN
			DETECTOR(I) = R.DETECTOR
		    END ELSE IF DETECTOR(I) NE R.DETECTOR THEN BEGIN
			DETECTOR(I) = 'B'
		    ENDIF
		ENDFOR
	    ENDIF
	    COUNTDOWN, I+1
	ENDFOR
	COUNTDOWN, /CLOSE
;
;  Put the data into the new database.
;
	DBOPEN, 'main', 1
	DBBUILD, PROG_NUM, PROG_ID, PROG_IND, STUDY_ID, STUDYVAR, OBS_PROG, $
		DETECTOR, SCI_OBJ, SCI_SPEC, CMP_NO, OBJECT, OBJ_ID,	$
		DATE_OBS, DATE_END, OBT_TIME, OBT_END, XCEN, YCEN, ANGLE, $
		IXWIDTH, IYWIDTH, SEQ_FROM, SEQ_TO, COMMENTS, DATE_MOD
	DBCLOSE
;
;******************************************************************************
;
;  Open the old experiment database file, and read in the data.
;
	DBOPEN, 'old_experiment'
	DBEXT, -1, 'MAIN_PTR,SEQ_NUM,OBS_SEQ,COUNT,SEQ_IND,RAS_ID,RAS_VAR', $
		MAIN_PTR, SEQ_NUM, OBS_SEQ, COUNT, SEQ_IND, RAS_ID, RAS_VAR
	DBEXT, -1, 'EXPTIME,OBS_MODE,DW_ID,OBT_TIME,OBT_END,XCEN,YCEN',	$
		EXPTIME, OBS_MODE, DW_ID, OBT_TIME, OBT_END, XCEN, YCEN
	DBEXT, -1, 'ANGLE,IXWIDTH,IYWIDTH,INS_X0,INS_Y0,INS_ROLL,SC_X0', $
		ANGLE, IXWIDTH, IYWIDTH, INS_X0, INS_Y0, INS_ROLL, SC_X0
	DBEXT, -1, 'SC_Y0,SC_ROLL,WAVEMIN,WAVEMAX',	$
		SC_Y0, SC_ROLL, WAVEMIN, WAVEMAX
;
;  Get the OBS_MODE parameters.
;
	NN = N_ELEMENTS(SEQ_NUM)
	COUNTDOWN, NN, /OPEN
	FOR I=0,NN-1 DO IF TRIM(OBS_MODE(I)) EQ '' THEN BEGIN
	    GET_RASTER, RAS_ID(I), RAS_VAR(I), R
	    IF R.RAS_ID GE 0 THEN BEGIN
		OBS_MODE(I) = R.DETECTOR + '-S' + TRIM(R.SLIT_NUM) + '-R' + $
			TRIM(R.RAS_ID) + '-E' + TRIM(R.EXPTIME)
	    ENDIF
	    COUNTDOWN, I+1
	ENDIF
	COUNTDOWN, /CLOSE
;
;  Put the data into the new database.
;
	DBOPEN, 'experiment', 1
	DBBUILD, MAIN_PTR, SEQ_NUM, OBS_SEQ, COUNT, SEQ_IND, RAS_ID, RAS_VAR, $
		EXPTIME, OBS_MODE, DW_ID, OBT_TIME, OBT_END, XCEN, YCEN, $
		ANGLE, IXWIDTH, IYWIDTH, INS_X0, INS_Y0, INS_ROLL, SC_X0, $
		SC_Y0, SC_ROLL, WAVEMIN, WAVEMAX
	DBCLOSE
;
;******************************************************************************
;
;  Open the old experiment2 database file, and read in the data.
;
	DBOPEN, 'old_experiment2'
	DBEXT, -1, 'PROG_NUM,SEQ_NUM,EXPER_PTR,DATE_OBS,DATE_END,TRACKING', $
		PROG_NUM, SEQ_NUM, EXPER_PTR, DATE_OBS, DATE_END, TRACKING
	DBEXT, -1, 'SER_ID,OPSLBITS,OPSRBITS,SLIT_POS,MIR_POS,EV_ENAB',	$
		SER_ID, OPSLBITS, OPSRBITS, SLIT_POS, MIR_POS, EV_ENAB
	DBEXT, -1, 'COMP_ERR,VDS_PMCP,VDS_MODE,VDS_ORI,VDS_ACC,SEQVALID', $
		COMP_ERR, VDS_PMCP, VDS_MODE, VDS_ORI, VDS_ACC, SEQVALID
	DBEXT, -1, 'COMMENTS,GSET_ID,DETECTOR,ZONE_ID,SLIT_NUM,FILENAME', $
		COMMENTS,GSET_ID,DETECTOR,ZONE_ID,SLIT_NUM,FILENAME
	DBEXT, -1, 'DATE_MOD', DATE_MOD
	DBCLOSE
;
;  Generate the additional data to be put into the new databases.
;
	NN = N_ELEMENTS(PROG_NUM)
	DATASRC = REPLICATE('Q',NN)
	EXPER1_PTR = LINDGEN(NN) + 1
;
;  Insert the data into the new experiment1 database.
;
	DBOPEN, 'experiment1', 1
	DBBUILD, PROG_NUM, SEQ_NUM, EXPER_PTR, DATASRC
	DBCLOSE
;
;  Insert the data into the new experiment2 database.
;
	DBOPEN, 'experiment2', 1
	DBBUILD, PROG_NUM, SEQ_NUM, EXPER1_PTR, DATE_OBS, DATE_END, TRACKING, $
		SER_ID, OPSLBITS, OPSRBITS, SLIT_POS, MIR_POS, EV_ENAB,	$
		COMP_ERR, VDS_PMCP, VDS_MODE, VDS_ORI, VDS_ACC, SEQVALID, $
		COMMENTS, GSET_ID, DETECTOR, ZONE_ID, SLIT_NUM, FILENAME, $
		DATE_MOD
	DBCLOSE
;
;******************************************************************************
;
;  Clean up.
;
	ASK ,'Conversion complete, do you wish to delete the old files? ', $
		ANSWER
	IF ANSWER EQ 'Y' THEN SPAWN, DELETE +	$
		' old_main.db* old_experiment.db* old_experiment2.db*'
	RETURN
;
;  Error handling point.
;
HANDLE_ERROR:
	MESSAGE, MESSAGE
;
	END
