	PRO EXPORT_PLAN, START_TIME, END_TIME,	$
		FILENAME=FILENAME, ERRMSG=ERRMSG, PATH=PATH
;+
; Project     :	SOHO - CDS
;
; Name        :	EXPORT_PLAN
;
; Purpose     :	Exports a CDS science plan from the database
;
; Explanation :	This routine exports a CDS science plan covering a given period
;		of time from the database to a text file in a specific export
;		format.  This text file can then be used by the routine
;		IMPORT_PLAN to import the plan into another database.  This
;		allows the plan to be moved between different sites, or from a
;		user's personal database to the official CDS database.
;
; Use         :	EXPORT_PLAN, START_TIME, END_TIME
;
; Inputs      :	START_TIME = The start time of the period, in any CDS time
;			     format.
;
;		END_TIME   = The end time of the period.
;
; Opt. Inputs :	None.
;
; Outputs     :	The complete plan definition, including any study definitions,
;		is written to a file.  The filename is derived from the start
;		date, includes a version number, and has the extension ".plan";
;		for example, "19970410_123456_001.plan".
;
; Opt. Outputs:	None.
;
; Keywords    :	PATH	  = The directory to write the file to.  If not
;			    defined, then the file will be written to the
;			    current directory.
;
;		FILENAME  = Returns the name of the output file.
;
;		ERRMSG	  = If defined and passed, then any error messages will
;			    be returned to the user in this parameter rather
;			    than depending on the MESSAGE routine in IDL.  If
;			    no errors are encountered, then a null string is
;			    returned.  In order to use this feature, ERRMSG
;			    must be defined first, e.g.
;
;				ERRMSG = ''
;				EXPORT_PLAN, ERRMSG=ERRMSG, ...
;				IF ERRMSG NE '' THEN ...
;
; Calls       :	GET_STUDY, GET_RASTER, GET_LINELIST, GET_DATAWIN, TRIM
;
; Common      :	None.
;
; Restrictions:	The user must have write privilege in the current default
;		directory.
;
; Side effects:	There are certain assumptions which are made in order for this
;		software to be able to work correctly:
;
;		1.  It is assumed that the GSET database is exactly the same on
;		    both the export and import sides.  There's no checking of
;		    the validity of the GSET_ID number.
;
;		2.  There's no checking of the durations of the various
;		    studies.  In particular, any implied assumptions about
;		    whether or not GSETs need to be loaded are carried over
;		    without checking.
;
;		3.  It is assumed that any engineering studies (STUDY_ID=0) are
;		    defined identically on both the export and import sides.
;
;		4.  Campaign numbers are assumed to have exactly the same
;		    meaning on both the export and import sides.
;
; Category    :	Planning, Databases
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 10-Apr-1997
;
; Modified    :	Version 1, William Thompson, GSFC, 10-Apr-1997
;             :	Version 2, Zarro, GSFC, 22-Apr-1997 -- fixed ERRMSG
;		Version 3, William Thompson, GSFC, 5 May 1997
;			Changed filename convention to include a version
;			number, and to put an "_" character between the date
;			and the time.
;		Version 4, William Thompson, GSFC, 17 May 1999
;			Fixed bug involving TRACKING and GET_GSET
;
; Version     :	Version 3, 5 May 1997
;-
;
	ON_ERROR, 2
	MESSAGE = ''
;
;  Check the number of parameters.
;
	IF N_PARAMS() NE 2 THEN BEGIN
	    MESSAGE = 'Syntax:  EXPORT_PLAN, START_TIME, END_TIME'
	    GOTO, HANDLE_ERROR
	ENDIF
;
;  Check the input and output times.
;

	TAI_START = ANYTIM2TAI(START_TIME, ERRMSG=MESSAGE)
	IF TRIM(MESSAGE) NE '' THEN GOTO, HANDLE_ERROR
	TAI_END   = ANYTIM2TAI(END_TIME,   ERRMSG=MESSAGE)
	IF TRIM(MESSAGE) NE '' THEN GOTO, HANDLE_ERROR
	IF TAI_END LE TAI_START THEN BEGIN
	    MESSAGE = 'End time must be greater than start time'
	    GOTO, HANDLE_ERROR
	ENDIF
;
;  Form the first part of the name of the output file.
;
	FILENAME = ANYTIM2CAL(START_TIME, FORM=8, /DATE) + '_' +	$
		ANYTIM2CAL(START_TIME, FORM=8, /TIME) + '_'
	IF N_ELEMENTS(PATH) EQ 1 THEN FILENAME=CONCAT_DIR(PATH,FILENAME)
;
;  Find all the plan files with names starting with those characters.  Get the
;  highest version number and add one to it.
;
	FOUND = FINDFILE(FILENAME + '*.plan', COUNT=COUNT)
	IF COUNT EQ 0 THEN VERSION = 1 ELSE BEGIN
		FOUND = FOUND(SORT(FOUND))
		BREAK_FILE, FOUND(COUNT-1), DISK, DIR, NAME, EXT
		VERSION = FIX(STRMID(NAME,16,3)) + 1
	ENDELSE
	FILENAME = FILENAME + STRMID(TRIM(1000+VERSION),1,3) + '.plan'
;
;  Open the output file.
;
	OPENW, UNIT, FILENAME, /GET_LUN, ERR=ERR
	IF ERR NE 0 THEN BEGIN
	    MESSAGE = !ERR_STRING
	    GOTO, HANDLE_ERROR
	ENDIF
;
;  Write out the file header.
;
	PRINTF, UNIT, 'DATATYPE = CDS PLAN DEFINITION'
	PRINTF, UNIT, 'FILENAME = ' + FILENAME
	PRINTF, UNIT, 'INSTRUME = CDS'
	GET_UTC, UTC, /ECS, /TRUNCATE
	PRINTF, UNIT, 'DATE_CRE = ' + UTC
	PRINTF, UNIT, 'STARTIME = ' + ANYTIM2UTC(START_TIME, /ECS)
	PRINTF, UNIT, 'ENDTIME  = ' + ANYTIM2UTC(END_TIME,   /ECS)
	PRINTF, UNIT, 'END'
;
;  Keep track of what definitions have already been written out to the file.
;
	S_STUDY_VARS = ''
;
;  Get the CDS general science plan.
;
	LIST_PLAN, START_TIME, END_TIME, PLAN, N_PLAN, INSTRUMENT='CDS', $
		ERRMSG=MESSAGE
	IF TRIM(MESSAGE) NE '' THEN GOTO, HANDLE_ERROR
;
;  Step through the plan, and write it to the file.
;
	FOR I_PLAN = 0, N_PLAN-1 DO BEGIN
	    PRINTF, UNIT, ''
	    PRINTF, UNIT, 'SCI_PLAN'
	    PRINTF, UNIT, '    SCI_OBJ  = ' + TRIM(PLAN(I_PLAN).SCI_OBJ)
	    PRINTF, UNIT, '    SCI_SPEC = ' + TRIM(PLAN(I_PLAN).SCI_SPEC)
	    PRINTF, UNIT, '    NOTES    = ' + TRIM(PLAN(I_PLAN).NOTES)
	    PRINTF, UNIT, '    STARTIME = ' + TRIM(PLAN(I_PLAN).START_TIME, $
		    '(F15.3)')
	    PRINTF, UNIT, '    END_TIME = ' + TRIM(PLAN(I_PLAN).END_TIME, $
		    '(F15.3)')
	    PRINTF, UNIT, '    OBJECT   = ' + TRIM(PLAN(I_PLAN).OBJECT)
	    PRINTF, UNIT, '    OBJ_ID   = ' + TRIM(PLAN(I_PLAN).OBJ_ID)
	    PRINTF, UNIT, '    PROG_ID  = ' + TRIM(PLAN(I_PLAN).PROG_ID)
	    PRINTF, UNIT, '    CMP_NO   = ' + TRIM(PLAN(I_PLAN).CMP_NO)
	    PRINTF, UNIT, '    XCEN     = ' + TRIM(PLAN(I_PLAN).XCEN)
	    PRINTF, UNIT, '    YCEN     = ' + TRIM(PLAN(I_PLAN).YCEN)
	    PRINTF, UNIT, '    DISTURB  = ' + TRIM(PLAN(I_PLAN).DISTURBANCES)
	ENDFOR
;
;  Get the CDS specific science plan.
;
	LIST_DETAIL, START_TIME, END_TIME, DETAILS, N_DETAILS, ERRMSG=MESSAGE
	IF TRIM(MESSAGE) NE '' THEN GOTO, HANDLE_ERROR
;
;  Step through the plan and get the full definition.
;
	FOR I_DETAIL = 0, N_DETAILS-1 DO BEGIN
	    GET_DETAIL, DETAILS(I_DETAIL).DATE_OBS, DET, ERRMSG=MESSAGE
	    IF TRIM(MESSAGE) NE '' THEN GOTO, HANDLE_ERROR
	    S_STUDY_VAR = TRIM(DET.STUDY_ID) + '.' + TRIM(DET.STUDYVAR)
	    TEST = WHERE(S_STUDY_VAR EQ S_STUDY_VARS, COUNT)
	    IF (COUNT EQ 0) AND (DET.STUDY_ID NE 0) THEN BEGIN
		EXPORT_STUDY, DET.STUDY_ID, DET.STUDYVAR, UNIT=UNIT,	$
			ERRMSG=MESSAGE
		IF TRIM(MESSAGE) NE '' THEN GOTO, HANDLE_ERROR
		S_STUDY_VARS = [S_STUDY_VARS, S_STUDY_VAR]
	    ENDIF
;
;  Print out the detailed plan entry.
;
	    PRINTF, UNIT, ''
	    PRINTF, UNIT, 'PROGRAM'
	    PRINTF, UNIT, '    PROG_ID  = ' + TRIM(DET.PROG_ID)
	    PRINTF, UNIT, '    STUDY_ID = ' + TRIM(DET.STUDY_ID)
	    PRINTF, UNIT, '    STUDYVAR = ' + TRIM(DET.STUDYVAR)
	    PRINTF, UNIT, '    SCI_OBJ  = ' + TRIM(DET.SCI_OBJ)
	    PRINTF, UNIT, '    SCI_SPEC = ' + TRIM(DET.SCI_SPEC)
	    PRINTF, UNIT, '    CMP_NO   = ' + TRIM(DET.CMP_NO)
	    PRINTF, UNIT, '    OBJECT   = ' + TRIM(DET.OBJECT)
	    PRINTF, UNIT, '    OBJ_ID   = ' + TRIM(DET.OBJ_ID)
	    PRINTF, UNIT, '    DATE_OBS = ' + TRIM(DET.DATE_OBS, '(F15.3)')
	    PRINTF, UNIT, '    DATE_END = ' + TRIM(DET.DATE_END, '(F15.3)')
	    PRINTF, UNIT, '    ORIG_DUR = ' + TRIM(DET.ORIG_DUR, '(F15.3)')
	    PRINTF, UNIT, '    N_RAS1   = ' + TRIM(DET.N_RASTERS1)
	    PRINTF, UNIT, '    TIMETAG  = ' + TRIM(FIX(DET.TIME_TAGGED))
	    PRINTF, UNIT, '    TRACKING = ' + TRIM(FIX(DET.TRACKING))
	    PRINTF, UNIT, '    N_REP_S  = ' + TRIM(DET.N_REPEAT_S)
	    PRINTF, UNIT, '    FLAGMSTR = ' + TRIM(DET.FLAG_MASTER)
	    PRINTF, UNIT, '    GSET_ID  = ' + TRIM(DET.GSET_ID)
	    PRINTF, UNIT, '    GET_RAW  = ' + TRIM(FIX(DET.GET_RAW))
	    PRINTF, UNIT, '    N_POINT  = ' + TRIM(DET.N_POINTINGS)
	    N_POINTINGS = DET.N_POINTINGS
;
;  The pointings appear immediately after the element N_POINTINGS.
;
	    FOR I = 0, N_POINTINGS-1 DO BEGIN
		P = DET.POINTINGS(I)
		PRINTF, UNIT, '    PNT_IND  = ' + TRIM(I+1)
		PRINTF, UNIT, '        INS_X    = ' + TRIM(P.INS_X)
		PRINTF, UNIT, '        INS_Y    = ' + TRIM(P.INS_Y)
		PRINTF, UNIT, '        ZONE_ID  = ' + TRIM(P.ZONE_ID)
	    ENDFOR
	ENDFOR
;
;  Get the complete name of the file, and print out a message signifying
;  completion.
;
	TEST = FSTAT(UNIT)
	FILENAME = TEST.NAME
	MESSAGE, 'Wrote file ' + FILENAME, /INFORMATIONAL
	GOTO, FINISH
;
;  Error handling point.
;
HANDLE_ERROR:
        IF DATATYPE(ERRMSG) NE 'STR' THEN BEGIN
         IF TRIM(MESSAGE) NE '' THEN MESSAGE, MESSAGE, /CONTINUE 
        ENDIF ELSE BEGIN
         IF TRIM(MESSAGE) NE '' THEN ERRMSG = 'EXPORT_PLAN: ' + MESSAGE 
        ENDELSE
;
;  Exit point.
;
FINISH:
	IF N_ELEMENTS(UNIT) NE 0 THEN FREE_LUN, UNIT
;
	RETURN
	END
