	PRO EXPORT_STUDY, INPUT, STUDYVAR, USE_TITLE=USE_TITLE,	$
		FILENAME=FILENAME, ERRMSG=ERRMSG, PATH=PATH, UNIT=K_UNIT
;+
; Project     :	SOHO - CDS
;
; Name        :	EXPORT_STUDY
;
; Purpose     :	Exports a study definition from the database
;
; Explanation :	This routine exports a study definition from the database to a
;		text file in a specific export format.  This text file can then
;		be used by the routine IMPORT_STUDY to import the software into
;		another database.  This allows study definitions to be moved
;		between different sites, or from a user's personal database to
;		the official CDS database.
;
; Use         :	EXPORT_STUDY, STUDY_ID, STUDYVAR
;		EXPORT_STUDY, TITLE_ID, STUDYVAR, /USE_TITLE
;
; Inputs      :	STUDY_ID
;
; Opt. Inputs :	STUDY_ID = Study ID number, or
;		TITLE_ID = Title ID number
;
;		STUDYVAR = Study variation index
;
; Outputs     :	The complete study definition, including raster description,
;		linelists, and data window lists, is written to a file.  The
;		filename is derived from the study name (OBS_PROG) and the
;		study variation number, and has the extension ".def"; for
;		example, "synop_1.def".
;
; Opt. Outputs:	None.
;
; Keywords    :	USE_TITLE = If set, then the input parameter is the title ID
;			    rather than the study ID.  Not required if OBS_PROG
;			    is passed.
;
;		PATH	  = The directory to write the file to.  If not
;			    defined, then the file will be written to the
;			    current directory.
;
;		FILENAME  = Returns the name of the output file.
;
;		UNIT	  = If passed, then contains the unit number of an
;			    already opened file.  This option is used by the
;			    routine export_plan.  When used, the header is not
;			    written to the file, and the file is left open when
;			    the routine is done.
;
;		ERRMSG	  = If defined and passed, then any error messages will
;			    be returned to the user in this parameter rather
;			    than depending on the MESSAGE routine in IDL.  If
;			    no errors are encountered, then a null string is
;			    returned.  In order to use this feature, ERRMSG
;			    must be defined first, e.g.
;
;				ERRMSG = ''
;				EXPORT_STUDY, ERRMSG=ERRMSG, ...
;				IF ERRMSG NE '' THEN ...
;
; Calls       :	GET_STUDY, GET_RASTER, GET_LINELIST, GET_DATAWIN, TRIM
;
; Common      :	None.
;
; Restrictions:	The user must have write privilege in the current default
;		directory.
;
; Side effects:	None.
;
; Category    :	Planning, Databases
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 16 May 1995
;
; Modified    :	Version 1, William Thompson, GSFC, 23 May 1995
;		Version 2, William Thompson, GSFC, 20 June 1995
;			Added item WIN_FLAG.
;		Version 3, William Thompson, GSFC, 27 October 1995
;			Added item IEF_ID.
;		Version 4, William Thompson, GSFC, 2 October 1996
;			Added keyword PATH
;		Version 5, William Thompson, GSFC, 10 April 1997
;			Added keyword UNIT
;		Version 6, William Thompson, GSFC, 18 April 1997
;			Fixed bug introduced in version 5
;
; Version     :	Version 6, 18 April 1997
;-
;
	ON_ERROR, 2
	MESSAGE = ''
;
;  Check the number of parameters.
;
	IF N_PARAMS() NE 2 THEN BEGIN
	    MESSAGE = 'Syntax:  EXPORT_STUDY, STUDY_ID, STUDYVAR'
	    GOTO, HANDLE_ERROR
	ENDIF
;
;  Get the study definition.
;
	GET_STUDY, INPUT, STUDYVAR, STUDY, USE_TITLE=USE_TITLE, ERRMSG=MESSAGE
	IF MESSAGE NE '' THEN GOTO, HANDLE_ERROR
;
;  Form the output filename.  Convert anything which isn't a letter or a number
;  to the underscore "_" character.
;
	B = BYTE(STRLOWCASE(STRTRIM(STUDY.OBS_PROG,2)))
	B0 = (BYTE('0'))(0)
	B9 = (BYTE('9'))(0)
	BA = (BYTE('a'))(0)
	BZ = (BYTE('z'))(0)
	W = WHERE( ((B LT B0) OR (B GT B9)) AND ((B LT BA) OR (B GT BZ)), NW)
	IF NW GT 0 THEN B(W) = BYTE('_')
	FILENAME = STRING(B)
;
;  Unless already open, then open the output file.
;
	IF N_ELEMENTS(K_UNIT) EQ 1 THEN BEGIN
	    UNIT = K_UNIT
	    TEST = FSTAT(UNIT)
	    IF NOT TEST.WRITE THEN BEGIN
		MESSAGE = 'Unit ' + TRIM(FIX(UNIT)) + ' is not open for write'
		GOTO, HANDLE_ERROR
	    ENDIF
	END ELSE BEGIN
	    FILENAME = FILENAME + '_' + TRIM(STUDY.STUDYVAR) + '.def'
	    IF N_ELEMENTS(PATH) EQ 1 THEN FILENAME=CONCAT_DIR(PATH,FILENAME)
	    OPENW, UNIT, FILENAME, /GET_LUN, ERR=ERR
	    IF ERR NE 0 THEN BEGIN
		MESSAGE = !ERR_STRING
		GOTO, HANDLE_ERROR
	    ENDIF
;
;  Write out the file header.
;
	    PRINTF, UNIT, 'DATATYPE = CDS STUDY DEFINITION'
	    PRINTF, UNIT, 'FILENAME = ' + FILENAME
	    PRINTF, UNIT, 'INSTRUME = CDS'
	    GET_UTC, UTC, /ECS, /TRUNCATE
	    PRINTF, UNIT, 'DATE_CRE = ' + UTC
	    PRINTF, UNIT, 'END'
	ENDELSE
;
;  Keep track of what definitions have already been written out to the file.
;
	LL_IDS   = -1
	DW_IDS   = -1
	RAS_IDS  = -1
	S_RAS_VARS = ''
;
;  Step through the rasters and get the raster definitions.
;
	FOR I_RASTER = 0, STUDY.N_RASTER_DEF-1 DO BEGIN
	    RAS_ID  = STUDY.RASTERS(I_RASTER).RAS_ID
	    RAS_VAR = STUDY.RASTERS(I_RASTER).RAS_VAR
	    GET_RASTER, RAS_ID, RAS_VAR, RASTER
;
;  Unless the linelist has already been written out to the file, read in the
;  line list and write it to the file.  If there is no linelist (signalled by
;  LL_ID LE 0) then ignore this step.
;
	    LL_ID = RASTER.LL_ID
	    TEST = WHERE(LL_ID EQ LL_IDS, COUNT)
	    IF (COUNT EQ 0) AND (LL_ID GT 0) THEN BEGIN
		GET_LINELIST, LL_ID, LINELIST
		PRINTF, UNIT, ''
		PRINTF, UNIT, 'LINELIST'
		PRINTF, UNIT, '    LL_ID    = ' + TRIM(LINELIST.LL_ID)
		PRINTF, UNIT, '    DETECTOR = ' + TRIM(LINELIST.DETECTOR)
		PRINTF, UNIT, '    LL_DESC  = ' + TRIM(LINELIST.LL_DESC)
		N_LINES = N_ELEMENTS(LINELIST.LINES)
		PRINTF, UNIT, '    N_LINES  = ' + TRIM(N_LINES)
;
;  The lines themselves appear immediately after the element N_LINES.
;
		FOR I = 0, N_LINES-1 DO BEGIN
		    L = LINELIST.LINES(I)
		    PRINTF, UNIT, '    LL_IND   = ' + TRIM(I+1)
		    PRINTF, UNIT, '        LINENAME = ' + TRIM(L.LINENAME)
		    PRINTF, UNIT, '        WAVELNTH = ' + TRIM(L.WAVELNTH)
		    PRINTF, UNIT, '        GR_ORDER = ' + TRIM(L.GR_ORDER)
		    PRINTF, UNIT, '        CEN_PIX  = ' + TRIM(L.CEN_PIX)
		    PRINTF, UNIT, '        WAVEBAND = ' + TRIM(L.WAVEBAND)
		ENDFOR
		LL_IDS = [LL_IDS, LL_ID]
	    ENDIF
;
;  Unless the data window list has already been written out to the file, read
;  in the data window list and write it to the file.
;
	    DW_ID = RASTER.DW_ID
	    TEST = WHERE(DW_ID EQ DW_IDS, COUNT)
	    IF COUNT EQ 0 THEN BEGIN
		GET_DATAWIN, DW_ID, DATAWIN
		PRINTF, UNIT, ''
		PRINTF, UNIT, 'DATAWIN'
		PRINTF, UNIT, '    DW_ID    = ' + TRIM(DATAWIN.DW_ID)
		PRINTF, UNIT, '    DETECTOR = ' + TRIM(DATAWIN.DETECTOR)
		PRINTF, UNIT, '    DW_DESC  = ' + TRIM(DATAWIN.DW_DESC)
		PRINTF, UNIT, '    W_WIDTH  = ' + TRIM(DATAWIN.W_WIDTH)
		PRINTF, UNIT, '    W_HEIGHT = ' + TRIM(DATAWIN.W_HEIGHT)
		PRINTF, UNIT, '    VDS_BACK = ' + TRIM(DATAWIN.VDS_BACK)
		PRINTF, UNIT, '    LL_ID    = ' + TRIM(DATAWIN.LL_ID)
		N_WINS = N_ELEMENTS(DATAWIN.WINS)
		PRINTF, UNIT, '    N_WINS   = ' + TRIM(N_WINS)
;
;  The data windows themselves appear immediately after the element N_WINS.
;
		FOR I = 0, N_WINS-1 DO BEGIN
		    W = DATAWIN.WINS(I)
		    PRINTF, UNIT, '    DW_IND   = ' + TRIM(I+1)
		    PRINTF, UNIT, '        WIN_NAME = ' + W.WIN_NAME
		    PRINTF, UNIT, '        WIN_DEF  = (' +	$
			TRIM(W.WIN_DEF(0)) + ',' + TRIM(W.WIN_DEF(1)) +	',' + $
			TRIM(W.WIN_DEF(2)) + ',' + TRIM(W.WIN_DEF(3)) + ')'
		    PRINTF, UNIT, '        WIN_FLAG = ' + TRIM(W.WIN_FLAG)
		ENDFOR
		DW_IDS = [DW_IDS, DW_ID]
	    ENDIF
;
;  Write out the information about the fundamental part of the raster (unless
;  already written).
;
	    TEST = WHERE(RAS_ID EQ RAS_IDS, COUNT)
	    IF COUNT EQ 0 THEN BEGIN
		PRINTF, UNIT, ''
		PRINTF, UNIT, 'RASTER'
		PRINTF, UNIT, '    RAS_ID   = ' + TRIM(RASTER.RAS_ID)
		PRINTF, UNIT, '    DETECTOR = ' + TRIM(RASTER.DETECTOR)
		PRINTF, UNIT, '    RAS_DESC = ' + TRIM(RASTER.RAS_DESC)
		PRINTF, UNIT, '    SLIT_NUM = ' + TRIM(RASTER.SLIT_NUM)
		PRINTF, UNIT, '    XSTEP    = ' + TRIM(RASTER.XSTEP)
		PRINTF, UNIT, '    YSTEP    = ' + TRIM(RASTER.YSTEP)
		PRINTF, UNIT, '    NX       = ' + TRIM(RASTER.NX)
		PRINTF, UNIT, '    NY       = ' + TRIM(RASTER.NY)
		RAS_IDS = [RAS_IDS, RAS_ID]
	    ENDIF
;
;  Write out the information about the variation part of the raster (unless
;  already written).
;
	    S_RAS_VAR = TRIM(RAS_ID) + '.' + TRIM(RAS_VAR)
	    TEST = WHERE(S_RAS_VAR EQ S_RAS_VARS, COUNT)
	    IF COUNT EQ 0 THEN BEGIN
		PRINTF, UNIT, ''
		PRINTF, UNIT, 'RASTER_VAR'
		PRINTF, UNIT, '    RAS_ID   = ' + TRIM(RASTER.RAS_ID)
		PRINTF, UNIT, '    RAS_VAR  = ' + TRIM(RASTER.RAS_VAR)
		PRINTF, UNIT, '    RV_DESC  = ' + TRIM(RASTER.RV_DESC)
		PRINTF, UNIT, '    EXPTIME  = ' + TRIM(RASTER.EXPTIME)
		PRINTF, UNIT, '    LL_ID    = ' + TRIM(RASTER.LL_ID)
		PRINTF, UNIT, '    COMP_ID  = ' + TRIM(RASTER.COMP_ID)
		PRINTF, UNIT, '    COMP_OPT = ' + TRIM(RASTER.COMP_OPT)
		PRINTF, UNIT, '    DW_ID    = ' + TRIM(RASTER.DW_ID)
		PRINTF, UNIT, '    IEF_ID   = ' + TRIM(RASTER.IEF_ID)
		PRINTF, UNIT, '    VDS_ORIENT = ' + TRIM(RASTER.VDS_ORIENT)
		PRINTF, UNIT, '    VDS_MAP  = ' + TRIM(RASTER.VDS_MAP)
		PRINTF, UNIT, '    TEL_RATE = ' + TRIM(RASTER.TEL_RATE)
		PRINTF, UNIT, '    DURATION = ' + TRIM(RASTER.DURATION)
		PRINTF, UNIT, '    USABLE   = ' + TRIM(RASTER.USABLE)
		S_RAS_VARS = [S_RAS_VARS, S_RAS_VAR]
	    ENDIF
;
;  End of raster loop.
;
	ENDFOR
;
;  Write out the information about the fundamental part of the study.
;
	PRINTF, UNIT, ''
	PRINTF, UNIT, 'STUDY'
	PRINTF, UNIT, '    STUDY_ID = ' + TRIM(STUDY.STUDY_ID)
	PRINTF, UNIT, '    TITLE_ID = ' + TRIM(STUDY.TITLE_ID)
	PRINTF, UNIT, '    OBS_PROG = ' + TRIM(STUDY.OBS_PROG)
	PRINTF, UNIT, '    TITLE    = ' + TRIM(STUDY.TITLE)
	PRINTF, UNIT, '    CATEGORY = ' + TRIM(STUDY.CATEGORY)
	PRINTF, UNIT, '    VAR_POINT = ' + TRIM(STUDY.VAR_POINT)
	PRINTF, UNIT, '    N_RASTER_DEF = ' + TRIM(STUDY.N_RASTER_DEF)
;
;  The rasters themselves appear immediately after the element N_RASTER_DEF.
;
	FOR I = 0, STUDY.N_RASTER_DEF-1 DO BEGIN
	    R = STUDY.RASTERS(I)
	    PRINTF, UNIT, '    RAS_IND   = ' + TRIM(I+1)
	    PRINTF, UNIT, '        RAS_ID   = ' + TRIM(R.RAS_ID)
	    PRINTF, UNIT, '        POINTING = ' + TRIM(R.POINTING)
	    PRINTF, UNIT, '        INS_X    = ' + TRIM(R.INS_X)
	    PRINTF, UNIT, '        INS_Y    = ' + TRIM(R.INS_Y)
	ENDFOR
;
;  Write out the information about the variation part of the study.
;
	PRINTF, UNIT, ''
	PRINTF, UNIT, 'STUDY_VAR'
	PRINTF, UNIT, '    STUDY_ID = ' + TRIM(STUDY.STUDY_ID)
	PRINTF, UNIT, '    TITLE_ID = ' + TRIM(STUDY.TITLE_ID)
	PRINTF, UNIT, '    STUDYVAR = ' + TRIM(STUDY.STUDYVAR)
	PRINTF, UNIT, '    SV_DESC  = ' + TRIM(STUDY.SV_DESC)
	PRINTF, UNIT, '    N_RASTERS0 = ' + TRIM(STUDY.N_RASTERS0)
	PRINTF, UNIT, '    DURATION0 = ' + TRIM(STUDY.DURATION0)
	PRINTF, UNIT, '    DURATION1 = ' + TRIM(STUDY.DURATION1)
	PRINTF, UNIT, '    ZONE_ID  = ' + TRIM(STUDY.ZONE_ID)
	PRINTF, UNIT, '    USABLE   = ' + TRIM(STUDY.USABLE)
	PRINTF, UNIT, '    N_RASTER_DEF = ' + TRIM(STUDY.N_RASTER_DEF)
;
;  The rasters themselves appear immediately after the element N_RASTER_DEF.
;
	FOR I = 0, STUDY.N_RASTER_DEF-1 DO BEGIN
	    R = STUDY.RASTERS(I)
	    PRINTF, UNIT, '    RAS_IND   = ' + TRIM(I+1)
	    PRINTF, UNIT, '        RAS_ID   = ' + TRIM(R.RAS_ID)
	    PRINTF, UNIT, '        RAS_VAR  = ' + TRIM(R.RAS_VAR)
	    PRINTF, UNIT, '        N_REPEAT_R = ' + TRIM(R.N_REPEAT_R)
	ENDFOR
;
;  Get the complete name of the file, and print out a message signifying
;  completion.
;
	IF N_ELEMENTS(K_UNIT) NE 1 THEN BEGIN
	    TEST = FSTAT(UNIT)
	    FILENAME = TEST.NAME
	    MESSAGE, 'Wrote file ' + FILENAME, /INFORMATIONAL
	ENDIF
	GOTO, FINISH
;
;  Error handling point.
;
HANDLE_ERROR:
	IF N_ELEMENTS(ERRMSG) NE 0 THEN ERRMSG = 'EXPORT_STUDY: ' + MESSAGE $
		ELSE MESSAGE, MESSAGE, /CONTINUE
;
;  Exit point.
;
FINISH:
	IF (N_ELEMENTS(UNIT) NE 0) AND (N_ELEMENTS(K_UNIT) NE 1) THEN	$
		FREE_LUN, UNIT
;
	RETURN
	END
