	PRO FIX_CATALOG, PROG_NUM, SEQ_IND, SEQ_NUM
;+
; Project     :	SOHO - CDS
;
; Name        :	FIX_CATALOG
;
; Purpose     :	Fix an entry in the CDS catalog.
;
; Category    :	Class5, Operations, Catalog
;
; Explanation :	Place a missing entry in the catalog.  Optionally, can change
;		the sequence number from that in the header to avoid conflicts
;		with entries already in the database.
;
;		The corresponding FITS file is found, and the FITS header is
;		read in.  Optionally, the SEQ_NUM is changed.  Then INS_CATALOG
;		is called to place the entry in the catalog.
;
;		Only for use by experts.
;
; Syntax      :	FIX_CATALOG, PROG_NUM, SEQ_IND  [, SEQ_NUM ]
;
; Examples    :	
;
; Inputs      :	PROG_NUM, SEQ_IND = The unique identifier for the data set.
;		The FITS filename is formed from these parameters.
;
; Opt. Inputs :	SEQ_NUM	= A new sequence number to apply to the data, to
;			  replace the sequence number in the FITS header.
;
; Outputs     :	None.
;
; Opt. Outputs:	None.
;
; Keywords    :	None.
;
; Calls       :	FXHREAD, FXADDPAR, INS_CATALOG
;
; Common      :	None.
;
; Restrictions:	Cannot change the SEQ_NUM of an entry that is already in the
;		database.  That can only be done by hand.
;
; Side effects:	None.
;
; Prev. Hist. :	None.
;
; History     :	Version 1, 01-Apr-1996, William Thompson, GSFC
;		Version 2, 09-Apr-1996, William Thompson, GSFC
;			Free LUN after using.
;		Version 3, 12-Apr-1996, William Thompson, GSFC
;			Check !PRIV
;
; Contact     :	WTHOMPSON
;-
;
	ON_ERROR, 2
;
	If N_PARAMS() LT 2 THEN MESSAGE,	$
		'Syntax:  FIX_CATALOG, PROG_NUM, SEQ_IND  [, SEQ_NUM ]'
;
;  Make sure that !PRIV is set correctly.
;
	IF !PRIV LT 2 THEN MESSAGE,	$
		'!PRIV must be 2 or greater to write into the database'
;
;  Form the name of the FITS file.
;
	FILENAME = 's' + TRIM(PROG_NUM) + 'r'
	RASTERCOUNTER = TRIM(SEQ_IND)
	IF STRLEN(RASTERCOUNTER) EQ 1 THEN RASTERCOUNTER = '0' + RASTERCOUNTER
	FILENAME = FILENAME + RASTERCOUNTER + '.fits'
;
;  Find the FITS file and read the header
;
	FILE = FIND_WITH_DEF(FILENAME, '$CDS_FITS_DATA')
	IF FILE EQ '' THEN MESSAGE, 'Unable to open file ' + FILENAME
	OPENR, UNIT, FILE, /GET_LUN
	FXHREAD, UNIT, HEADER
	FREE_LUN, UNIT
;
;  If SEQ_NUM was passed, then change it in the header.
;
	IF N_PARAMS() EQ 3 THEN FXADDPAR, HEADER, 'SEQ_NUM', SEQ_NUM
;
;  Insert the data in the catalog.
;
	INS_CATALOG, HEADER
;
	RETURN
	END
