	PRO FLUSH_CATALOG, ERRMSG=ERRMSG, YEAR=YEAR
;+
; Project     :	SOHO - CDS
;
; Name        :	FLUSH_CATALOG
;
; Purpose     :	Flush the CDS catalog databases.
;
; Category    :	Level4, CDS, Operations, Catalog, Database
;
; Explanation :	This routine moves all the entries currently in the experiment*
;		databases to fexperiment*, where the "f" stands for "full".
;		This way, the databases being appended to are kept small, so
;		that FITSGEN runs more efficiently.
;
; Syntax      :	FLUSH_CATALOG
;
; Examples    :	
;
; Inputs      :	None.
;
; Opt. Inputs :	None.
;
; Outputs     :	None.
;
; Opt. Outputs:	None.
;
; Keywords    :	YEAR   = If set, then all entries with a start date within
;			 that year are moved to a database specifically set
;			 aside for that year.  The use of this keyword requires
;			 !PRIV=3.
;
;		ERRMSG = If defined and passed, then any error messages will be
;			 returned to the user in this parameter rather than
;			 depending on the MESSAGE routine in IDL.  If no errors
;			 are encountered, then a null string is returned.  In
;			 order to use this feature, ERRMSG must be defined
;			 first, e.g.
;
;				ERRMSG = ''
;				FLUSH_CATALOG, ERRMSG=ERRMSG
;				IF ERRMSG NE '' THEN ...
;
; Calls       :	
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	Backup copies are made of the database files, called
;		old_experiment.dbf, old_fexperiment.dbf, etc.
;
; Prev. Hist. :	None.
;
; History     :	Version 1, 11-Apr-1996, William Thompson, GSFC
;		Version 2, 1-June-1998, William Thompson, GSFC
;			Added YEAR keyword
;
; Contact     :	WTHOMPSON
;-
;
	ON_ERROR, 2
;
	BACKUPS_MADE = 0
	FBACKUPS_MADE = 0
;
;  Make sure that one has privilege to write into the database.
;
	IF NOT PRIV_ZDBASE(/CAT) THEN BEGIN
	    MESSAGE = 'You have no privilege to write into the catalog'
	    GOTO, HANDLE_ERROR
	END ELSE IF (N_ELEMENTS(YEAR) EQ 1) AND (!PRIV LT 3) THEN BEGIN
	    MESSAGE = '!PRIV must be 3 or greater to use the YEAR keyword'
	    GOTO, HANDLE_ERROR
	END ELSE IF !PRIV LT 2 THEN BEGIN
	    MESSAGE = '!PRIV must be 2 or greater to write into the catalog'
	    GOTO, HANDLE_ERROR
	ENDIF
;
;  Determine the correct commands for copying and renaming.
;
	CASE OS_FAMILY() OF
		'vms': BEGIN
			COPY   = 'COPY'
			RENAME = 'RENAME'
			END
		'unix': BEGIN
			COPY   = 'cp'
			RENAME = 'mv'
			END
		ELSE: BEGIN
			MESSAGE = 'Only supported in VMS or Unix'
			GOTO, HANDLE_ERROR
			END
	ENDCASE
;
;  Lock the databases, so that no other processes can write to the catalog
;  while the flush is going on.
;
	LOCK_DATABASE, 'experiment', EXPER_LOCKFILE
;
;  Find the directory containing the file experiment.dbf.
;
	FILENAME = FIND_WITH_DEF('experiment.dbf','$ZDBASE')
	BREAK_FILE, FILENAME, DISK, DIR
	PATH = DISK+DIR
;
;  Save the current directory, and move to the database directory.
;
	CD, CURRENT=CURRENT
	IF PATH NE '' THEN CD, PATH
;
;  Make backup copies of the databases.
;
	SPAWN, COPY + ' experiment.dbf old_experiment.dbf'
	SPAWN, COPY + ' experiment.dbx old_experiment.dbx'
	SPAWN, COPY + ' experiment.dbh old_experiment.dbh'
;
	SPAWN, COPY + ' experiment1.dbf old_experiment1.dbf'
	SPAWN, COPY + ' experiment1.dbx old_experiment1.dbx'
	SPAWN, COPY + ' experiment1.dbh old_experiment1.dbh'
;
	SPAWN, COPY + ' experiment2.dbf old_experiment2.dbf'
	SPAWN, COPY + ' experiment2.dbx old_experiment2.dbx'
	SPAWN, COPY + ' experiment2.dbh old_experiment2.dbh'
;
	BACKUPS_MADE = 1
;
;  Check to see if the "f" databases exist.  If not, then simply move the
;  regular databases to their "f" counterparts, and create empty versions of
;  the regular files.
;
	FILENAME = FIND_WITH_DEF('fexperiment.dbf','$ZDBASE')
	IF FILENAME EQ '' THEN BEGIN
;
;  Move the .dbf, .dbx, and .dbh files to their "f" counterparts.
;
		SPAWN, RENAME + ' experiment.dbf fexperiment.dbf'
		SPAWN, RENAME + ' experiment.dbx fexperiment.dbx'
		SPAWN, RENAME + ' experiment.dbh fexperiment.dbh'
;
		SPAWN, RENAME + ' experiment1.dbf fexperiment1.dbf'
		SPAWN, RENAME + ' experiment1.dbx fexperiment1.dbx'
		SPAWN, RENAME + ' experiment1.dbh fexperiment1.dbh'
;
		SPAWN, RENAME + ' experiment2.dbf fexperiment2.dbf'
		SPAWN, RENAME + ' experiment2.dbx fexperiment2.dbx'
		SPAWN, RENAME + ' experiment2.dbh fexperiment2.dbh'
;
		GOTO, FINISH
	ENDIF
;
;  Otherwise, the "f" databases need to be backed up as well.
;
	SPAWN, COPY + ' fexperiment.dbf old_fexperiment.dbf'
	SPAWN, COPY + ' fexperiment.dbx old_fexperiment.dbx'
	SPAWN, COPY + ' fexperiment.dbh old_fexperiment.dbh'
;
	SPAWN, COPY + ' fexperiment1.dbf old_fexperiment1.dbf'
	SPAWN, COPY + ' fexperiment1.dbx old_fexperiment1.dbx'
	SPAWN, COPY + ' fexperiment1.dbh old_fexperiment1.dbh'
;
	SPAWN, COPY + ' fexperiment2.dbf old_fexperiment2.dbf'
	SPAWN, COPY + ' fexperiment2.dbx old_fexperiment2.dbx'
	SPAWN, COPY + ' fexperiment2.dbh old_fexperiment2.dbh'
;
	FBACKUPS_MADE = 1
;
;  Move the data from the regular databases to the "f" counterparts.  Start
;  with the experiment catalog.
;
	DBOPEN, 'experiment'
	IF (DB_INFO('entries'))(0) NE 0 THEN BEGIN
	    DBEXT,-1,'MAIN_PTR,SEQ_NUM,OBS_SEQ,COUNT,SEQ_IND,RAS_ID,RAS_VAR', $
		    MAIN_PTR, SEQ_NUM, OBS_SEQ, COUNT, SEQ_IND, RAS_ID, RAS_VAR
	    DBEXT, -1, 'EXPTIME,OBS_MODE,DW_ID,OBT_TIME,OBT_END,XCEN,YCEN', $
		    EXPTIME, OBS_MODE, DW_ID, OBT_TIME, OBT_END, XCEN, YCEN
	    DBEXT, -1, 'ANGLE,IXWIDTH,IYWIDTH,INS_X0,INS_Y0,INS_ROLL,SC_X0', $
		    ANGLE, IXWIDTH, IYWIDTH, INS_X0, INS_Y0, INS_ROLL, SC_X0
	    DBEXT, -1, 'SC_Y0,SC_ROLL,WAVEMIN,WAVEMAX',	$
		    SC_Y0, SC_ROLL, WAVEMIN, WAVEMAX
;
;  Open the "f" counterpart, and get the number of records already in the
;  database.  Calculate the pointers.
;
	    DBOPEN, 'fexperiment', 1
	    N_ENTRIES = (DB_INFO('entries'))(0)
	    FEXPER0_PTR = LONG(N_ENTRIES + 1 + LINDGEN(N_ELEMENTS(MAIN_PTR)))
;
;  Write to the database.
;
	    DBBUILD, MAIN_PTR, SEQ_NUM, OBS_SEQ, COUNT, SEQ_IND, RAS_ID, $
		    RAS_VAR, EXPTIME, OBS_MODE, DW_ID, OBT_TIME, OBT_END, $
		    XCEN, YCEN, ANGLE, IXWIDTH, IYWIDTH, INS_X0, INS_Y0, $
		    INS_ROLL, SC_X0, SC_Y0, SC_ROLL, WAVEMIN, WAVEMAX,	$
		    STATUS=STATUS
	    IF STATUS EQ 0 THEN BEGIN
		MESSAGE = 'Write to fexperiment database was not successful'
		GOTO, HANDLE_ERROR
	    ENDIF
	ENDIF
;
;  Do the same with the experiment1 database.
;
	DBOPEN, 'experiment1'
	IF (DB_INFO('entries'))(0) NE 0 THEN BEGIN
	    DBEXT, -1, 'PROG_NUM,SEQ_NUM,EXPER0_PTR,DATASRC',	$
		    PROG_NUM, SEQ_NUM, EXPER0_PTR, DATASRC
;
	    DBOPEN, 'fexperiment1', 1
	    N_ENTRIES = (DB_INFO('entries'))(0)
	    FEXPER_PTR = LONG(N_ENTRIES + 1 + LINDGEN(N_ELEMENTS(PROG_NUM)))
	    EXPER0_PTR = FEXPER0_PTR(EXPER0_PTR-1)
;
	    DBBUILD, PROG_NUM, SEQ_NUM, EXPER0_PTR, DATASRC, STATUS=STATUS
	    IF STATUS EQ 0 THEN BEGIN
		MESSAGE = 'Write to fexperiment1 database was not successful'
		GOTO, HANDLE_ERROR
	    ENDIF
	ENDIF
;
;  Finally, do the same with the experiment2 database.
;
	DBOPEN, 'experiment2'
	IF (DB_INFO('entries'))(0) NE 0 THEN BEGIN
	    DBEXT,-1,'PROG_NUM,SEQ_NUM,EXPER_PTR,DATE_OBS,DATE_END,TRACKING', $
		    PROG_NUM, SEQ_NUM, EXPER_PTR, DATE_OBS, DATE_END, TRACKING
	    DBEXT, -1, 'SER_ID,OPSLBITS,OPSRBITS,SLIT_POS,MIR_POS,EV_ENAB', $
		    SER_ID, OPSLBITS, OPSRBITS, SLIT_POS, MIR_POS, EV_ENAB
	    DBEXT, -1, 'COMP_ERR,VDS_PMCP,VDS_MODE,VDS_ORI,VDS_ACC,SEQVALID', $
		    COMP_ERR, VDS_PMCP, VDS_MODE, VDS_ORI, VDS_ACC, SEQVALID
	    DBEXT, -1, 'COMMENTS,GSET_ID,DETECTOR,ZONE_ID,SLIT_NUM,FILENAME', $
		    COMMENTS, GSET_ID, DETECTOR, ZONE_ID, SLIT_NUM, FILENAME
	    DBEXT, -1, 'DATE_MOD', DATE_MOD
;
	    DBOPEN, 'fexperiment2', 1
	    EXPER_PTR = FEXPER_PTR(EXPER_PTR-1)
;
	    DBBUILD, PROG_NUM, SEQ_NUM, EXPER_PTR, DATE_OBS, DATE_END,	$
		    TRACKING, SER_ID, OPSLBITS, OPSRBITS, SLIT_POS, MIR_POS, $
		    EV_ENAB, COMP_ERR, VDS_PMCP, VDS_MODE, VDS_ORI, VDS_ACC, $
		    SEQVALID, COMMENTS, GSET_ID, DETECTOR, ZONE_ID, SLIT_NUM, $
		    FILENAME, DATE_MOD, STATUS=STATUS
	    IF STATUS EQ 0 THEN BEGIN
		MESSAGE = 'Write to fexperiment2 database was not successful'
		GOTO, HANDLE_ERROR
	    ENDIF
	ENDIF
;
;  Create new experiment catalogs.
;
FINISH:
	DBCREATE, 'experiment',	 1, 1, /external
	DBCREATE, 'experiment1', 1, 1, /external
	DBCREATE, 'experiment2', 1, 1, /external
;
;  If the year keyword was set, then create the new year database.
;
	IF N_ELEMENTS(YEAR) EQ 1 THEN BEGIN
	    TEMP = ANYTIM2UTC(TRIM(YEAR) + '-1-1', /EXT)
	    TYEAR = TEMP.YEAR
	    FILE = 'fexperiment2_' + TRIM(TYEAR)
	    IF FILE_EXIST(FILE+'.dbf') THEN BEGIN
		MESSAGE = 'Database ' + FILE + ' already exists'
		GOTO, SHOW_ERROR
	    ENDIF
;
;  Recreate the index files, to pick up any changes in the size parameter.
;
	    DATABASES = ['fexperiment','fexperiment1','fexperiment2']
	    FOR I=0,2 DO BEGIN
		DBCLOSE
		IF NOT FILE_EXIST(DATABASES(I)+'.dbd') THEN BEGIN
		    MESSAGE = 'File ' + DATABASES(I) + '.dbd must exist ' + $
			    'to use YEAR keyword'
		    GOTO, SHOW_ERROR
		ENDIF
		DBCREATE, DATABASES(I), 1, /EXTERNAL
		DBOPEN, DATABASES(I), 1
		DBINDEX
	    ENDFOR
;
;  Open the databases, and determine which entries fall within that year.
;
	    DATE1 = UTC2TAI(TRIM(TYEAR)   + '-1-1')
	    DATE2 = UTC2TAI(TRIM(TYEAR+1) + '-1-1')
	    DBOPEN, 'fexperiment2,fexperiment1,fexperiment'
	    DBEXT,-1,'exper_ptr,exper0_ptr,date_obs',	$
		    EXPER_PTR, EXPER0_PTR, DATE_OBS
	    W_IN = WHERE((DATE_OBS GE DATE1) AND (DATE_OBS LT DATE2), C_IN)
	    W_OUT = WHERE((DATE_OBS LT DATE1) OR (DATE_OBS GE DATE2), C_OUT)
	    IF C_IN EQ 0 THEN BEGIN
		MESSAGE = 'No entries for the target year'
		GOTO, SHOW_ERROR
	    ENDIF
;	    
;  Copy each database in turn, and remove all entries that are not in that
;  year.  Fix any pointers.
;
	    SYEAR = TRIM(TYEAR)
	    IF C_OUT GT 0 THEN BEGIN
		SPAWN, COPY + ' fexperiment.dbd fexperiment_'+SYEAR+'.dbd'
		SPAWN, COPY + ' fexperiment.dbf fexperiment_'+SYEAR+'.dbf'
		SPAWN, COPY + ' fexperiment.dbx fexperiment_'+SYEAR+'.dbx'
		SPAWN, COPY + ' fexperiment.dbh fexperiment_'+SYEAR+'.dbh'
		DBOPEN, 'fexperiment'
		POS = EXPER0_PTR(W_IN)
		DBEXT, POS, 'MAIN_PTR,SEQ_NUM,OBS_SEQ,COUNT,SEQ_IND,RAS_ID', $
			MAIN_PTR, SEQ_NUM, OBS_SEQ, COUNT, SEQ_IND, RAS_ID
		DBEXT, POS, 'RAS_VAR,EXPTIME,OBS_MODE,DW_ID,OBT_TIME,OBT_END',$
			RAS_VAR, EXPTIME, OBS_MODE, DW_ID, OBT_TIME, OBT_END
		DBEXT, POS, 'XCEN,YCEN,ANGLE,IXWIDTH,IYWIDTH,INS_X0,INS_Y0', $
			XCEN, YCEN, ANGLE, IXWIDTH, IYWIDTH, INS_X0, INS_Y0
		DBEXT, POS, 'INS_ROLL,SC_X0,SC_Y0,SC_ROLL,WAVEMIN,WAVEMAX', $
			INS_ROLL, SC_X0, SC_Y0, SC_ROLL, WAVEMIN, WAVEMAX
		DBOPEN, 'fexperiment_' + SYEAR, 1
		DBDELETE, LINDGEN((DB_INFO('entries'))(0)+1)
		DBBUILD, MAIN_PTR, SEQ_NUM, OBS_SEQ, COUNT, SEQ_IND, RAS_ID, $
			RAS_VAR, EXPTIME, OBS_MODE, DW_ID, OBT_TIME, OBT_END, $
			XCEN, YCEN, ANGLE, IXWIDTH, IYWIDTH, INS_X0, INS_Y0, $
			INS_ROLL, SC_X0, SC_Y0, SC_ROLL, WAVEMIN, WAVEMAX
		DBCLOSE
		DBCOMPRESS, 'fexperiment_' + SYEAR
;
		DBOPEN, 'fexperiment', 1
		POS = EXPER0_PTR(W_OUT)
		DBEXT, POS, 'MAIN_PTR,SEQ_NUM,OBS_SEQ,COUNT,SEQ_IND,RAS_ID', $
			MAIN_PTR, SEQ_NUM, OBS_SEQ, COUNT, SEQ_IND, RAS_ID
		DBEXT, POS, 'RAS_VAR,EXPTIME,OBS_MODE,DW_ID,OBT_TIME,OBT_END',$
			RAS_VAR, EXPTIME, OBS_MODE, DW_ID, OBT_TIME, OBT_END
		DBEXT, POS, 'XCEN,YCEN,ANGLE,IXWIDTH,IYWIDTH,INS_X0,INS_Y0', $
			XCEN, YCEN, ANGLE, IXWIDTH, IYWIDTH, INS_X0, INS_Y0
		DBEXT, POS, 'INS_ROLL,SC_X0,SC_Y0,SC_ROLL,WAVEMIN,WAVEMAX', $
			INS_ROLL, SC_X0, SC_Y0, SC_ROLL, WAVEMIN, WAVEMAX
		DBDELETE, LINDGEN((DB_INFO('entries'))(0)+1)
		DBBUILD, MAIN_PTR, SEQ_NUM, OBS_SEQ, COUNT, SEQ_IND, RAS_ID, $
			RAS_VAR, EXPTIME, OBS_MODE, DW_ID, OBT_TIME, OBT_END, $
			XCEN, YCEN, ANGLE, IXWIDTH, IYWIDTH, INS_X0, INS_Y0, $
			INS_ROLL, SC_X0, SC_Y0, SC_ROLL, WAVEMIN, WAVEMAX
		DBCLOSE
		DBCOMPRESS, 'fexperiment'
	    END ELSE BEGIN
		DBCLOSE
		SPAWN, COPY + ' fexperiment.dbd fexperiment_'+SYEAR+'.dbd'
		SPAWN, COPY + ' fexperiment.dbf fexperiment_'+SYEAR+'.dbf'
		SPAWN, COPY + ' fexperiment.dbx fexperiment_'+SYEAR+'.dbx'
		SPAWN, COPY + ' fexperiment.dbh fexperiment_'+SYEAR+'.dbh'
		DBCREATE, 'fexperiment', 1, 1, /EXTERNAL
	    ENDELSE
;
	    IF C_OUT GT 0 THEN BEGIN
		SPAWN, COPY + ' fexperiment1.dbd fexperiment1_'+SYEAR+'.dbd'
		SPAWN, COPY + ' fexperiment1.dbf fexperiment1_'+SYEAR+'.dbf'
		SPAWN, COPY + ' fexperiment1.dbx fexperiment1_'+SYEAR+'.dbx'
		SPAWN, COPY + ' fexperiment1.dbh fexperiment1_'+SYEAR+'.dbh'
		DBOPEN, 'fexperiment1'
		POS = EXPER_PTR(W_IN)
		DBEXT, POS, 'PROG_NUM,SEQ_NUM,EXPER0_PTR,DATASRC',	$
			PROG_NUM, SEQ_NUM, EXPER0_PTR, DATASRC
		DBOPEN, 'fexperiment1_' + SYEAR, 1
		EXPER0_PTR = LINDGEN(N_ELEMENTS(EXPER0_PTR)) + 1
		DBDELETE, LINDGEN((DB_INFO('entries'))(0)+1)
		DBBUILD, PROG_NUM, SEQ_NUM, EXPER0_PTR, DATASRC
		DBCLOSE
		DBCOMPRESS, 'fexperiment1_' + SYEAR
;
		DBOPEN, 'fexperiment1', 1
		POS = EXPER_PTR(W_OUT)
		DBEXT, POS, 'PROG_NUM,SEQ_NUM,EXPER0_PTR,DATASRC',	$
			PROG_NUM, SEQ_NUM, EXPER0_PTR, DATASRC
		DBDELETE, LINDGEN((DB_INFO('entries'))(0)+1)
		EXPER0_PTR = LINDGEN(N_ELEMENTS(EXPER0_PTR)) + 1
		DBBUILD, PROG_NUM, SEQ_NUM, EXPER0_PTR, DATASRC
		DBCLOSE
		DBCOMPRESS, 'fexperiment1'
	    END ELSE BEGIN
		DBCLOSE
		SPAWN, COPY + ' fexperiment1.dbd fexperiment1_'+SYEAR+'.dbd'
		SPAWN, COPY + ' fexperiment1.dbf fexperiment1_'+SYEAR+'.dbf'
		SPAWN, COPY + ' fexperiment1.dbx fexperiment1_'+SYEAR+'.dbx'
		SPAWN, COPY + ' fexperiment1.dbh fexperiment1_'+SYEAR+'.dbh'
		DBCREATE, 'fexperiment1', 1, 1, /EXTERNAL
	    ENDELSE
;
	    IF C_OUT GT 0 THEN BEGIN
		SPAWN, COPY + ' fexperiment2.dbd fexperiment2_'+SYEAR+'.dbd'
		SPAWN, COPY + ' fexperiment2.dbf fexperiment2_'+SYEAR+'.dbf'
		SPAWN, COPY + ' fexperiment2.dbx fexperiment2_'+SYEAR+'.dbx'
		SPAWN, COPY + ' fexperiment2.dbh fexperiment2_'+SYEAR+'.dbh'
		DBOPEN, 'fexperiment2'
		POS = W_IN + 1
		DBEXT, POS, 'PROG_NUM,SEQ_NUM,EXPER_PTR,DATE_OBS,DATE_END', $
			PROG_NUM, SEQ_NUM, EXPER_PTR, DATE_OBS, DATE_END
		DBEXT, POS, 'TRACKING,SER_ID,OPSLBITS,OPSRBITS,SLIT_POS', $
			TRACKING, SER_ID, OPSLBITS, OPSRBITS, SLIT_POS
		DBEXT, POS, 'MIR_POS,EV_ENAB,COMP_ERR,VDS_PMCP,VDS_MODE', $
			MIR_POS, EV_ENAB, COMP_ERR, VDS_PMCP, VDS_MODE
		DBEXT, POS, 'VDS_ORI,VDS_ACC,SEQVALID,COMMENTS,GSET_ID', $
			VDS_ORI, VDS_ACC, SEQVALID, COMMENTS, GSET_ID
		DBEXT, POS, 'DETECTOR,ZONE_ID,SLIT_NUM,FILENAME,DATE_MOD', $
			DETECTOR, ZONE_ID, SLIT_NUM, FILENAME, DATE_MOD
		DBOPEN, 'fexperiment2_' + SYEAR, 1
		EXPER_PTR = LINDGEN(N_ELEMENTS(EXPER_PTR)) + 1
		DBDELETE, LINDGEN((DB_INFO('entries'))(0)+1)
		DBBUILD, PROG_NUM, SEQ_NUM, EXPER_PTR, DATE_OBS, DATE_END, $
			TRACKING, SER_ID, OPSLBITS, OPSRBITS, SLIT_POS, $
			MIR_POS, EV_ENAB, COMP_ERR, VDS_PMCP, VDS_MODE, $
			VDS_ORI, VDS_ACC, SEQVALID, COMMENTS, GSET_ID, $
			DETECTOR, ZONE_ID, SLIT_NUM, FILENAME, DATE_MOD
		DBCLOSE
		DBCOMPRESS, 'fexperiment2_' + SYEAR
;
		DBOPEN, 'fexperiment2', 1
		POS = W_OUT + 1
		DBEXT, POS, 'PROG_NUM,SEQ_NUM,EXPER_PTR,DATE_OBS,DATE_END', $
			PROG_NUM, SEQ_NUM, EXPER_PTR, DATE_OBS, DATE_END
		DBEXT, POS, 'TRACKING,SER_ID,OPSLBITS,OPSRBITS,SLIT_POS', $
			TRACKING, SER_ID, OPSLBITS, OPSRBITS, SLIT_POS
		DBEXT, POS, 'MIR_POS,EV_ENAB,COMP_ERR,VDS_PMCP,VDS_MODE', $
			MIR_POS, EV_ENAB, COMP_ERR, VDS_PMCP, VDS_MODE
		DBEXT, POS, 'VDS_ORI,VDS_ACC,SEQVALID,COMMENTS,GSET_ID', $
			VDS_ORI, VDS_ACC, SEQVALID, COMMENTS, GSET_ID
		DBEXT, POS, 'DETECTOR,ZONE_ID,SLIT_NUM,FILENAME,DATE_MOD', $
			DETECTOR, ZONE_ID, SLIT_NUM, FILENAME, DATE_MOD
		DBDELETE, LINDGEN((DB_INFO('entries'))(0)+1)
		EXPER_PTR = LINDGEN(N_ELEMENTS(EXPER_PTR)) + 1
		DBBUILD, PROG_NUM, SEQ_NUM, EXPER_PTR, DATE_OBS, DATE_END, $
			TRACKING, SER_ID, OPSLBITS, OPSRBITS, SLIT_POS, $
			MIR_POS, EV_ENAB, COMP_ERR, VDS_PMCP, VDS_MODE, $
			VDS_ORI, VDS_ACC, SEQVALID, COMMENTS, GSET_ID, $
			DETECTOR, ZONE_ID, SLIT_NUM, FILENAME, DATE_MOD
		DBCLOSE
		DBCOMPRESS, 'fexperiment2'
	    END ELSE BEGIN
		DBCLOSE
		SPAWN, COPY + ' fexperiment2.dbd fexperiment2_'+SYEAR+'.dbd'
		SPAWN, COPY + ' fexperiment2.dbf fexperiment2_'+SYEAR+'.dbf'
		SPAWN, COPY + ' fexperiment2.dbx fexperiment2_'+SYEAR+'.dbx'
		SPAWN, COPY + ' fexperiment2.dbh fexperiment2_'+SYEAR+'.dbh'
		DBCREATE, 'fexperiment2', 1, 1, /EXTERNAL
	    ENDELSE
;
	ENDIF
;
;  In VMS, purge the directory.
;
	IF OS_FAMILY() EQ 'vms' THEN SPAWN, 'PURGE'
	GOTO, EXIT_POINT
;
;  Error handling point.  Return the databases to their original state.
;
HANDLE_ERROR:
	IF BACKUPS_MADE THEN BEGIN
	    SPAWN, RENAME + ' old_experiment.dbf experiment.dbf'
	    SPAWN, RENAME + ' old_experiment.dbx experiment.dbx'
	    SPAWN, RENAME + ' old_experiment.dbh experiment.dbh'
;
	    SPAWN, RENAME + ' old_experiment1.dbf experiment1.dbf'
	    SPAWN, RENAME + ' old_experiment1.dbx experiment1.dbx'
	    SPAWN, RENAME + ' old_experiment1.dbh experiment1.dbh'
;
	    SPAWN, RENAME + ' old_experiment2.dbf experiment2.dbf'
	    SPAWN, RENAME + ' old_experiment2.dbx experiment2.dbx'
	    SPAWN, RENAME + ' old_experiment2.dbh experiment2.dbh'
;
	    IF FBACKUPS_MADE THEN BEGIN
		SPAWN, RENAME + ' old_fexperiment.dbf fexperiment.dbf'
		SPAWN, RENAME + ' old_fexperiment.dbx fexperiment.dbx'
		SPAWN, RENAME + ' old_fexperiment.dbh fexperiment.dbh'
;
		SPAWN, RENAME + ' old_fexperiment1.dbf fexperiment1.dbf'
		SPAWN, RENAME + ' old_fexperiment1.dbx fexperiment1.dbx'
		SPAWN, RENAME + ' old_fexperiment1.dbh fexperiment1.dbh'
;
		SPAWN, RENAME + ' old_fexperiment2.dbf fexperiment2.dbf'
		SPAWN, RENAME + ' old_fexperiment2.dbx fexperiment2.dbx'
		SPAWN, RENAME + ' old_fexperiment2.dbh fexperiment2.dbh'
	    ENDIF
	ENDIF
;
;  Determine what do with the error message.
;
SHOW_ERROR:
	IF N_ELEMENTS(ERRMSG) NE 0 THEN ERRMSG = 'FLUSH_CATALOG: ' + MESSAGE $
		ELSE MESSAGE, MESSAGE, /CONTINUE
;
;  Unlock the databases, and go back to the original directory.
;
EXIT_POINT:
	IF N_ELEMENTS(EXPER_LOCKFILE) EQ 1 THEN UNLOCK_DATABASE, EXPER_LOCKFILE
	IF N_ELEMENTS(CURRENT) EQ 1 THEN CD, CURRENT
;
	RETURN
	END
