	PRO GET_ALT, DATE, STATUS, ERRMSG=ERRMSG
;+
; Project     :	SOHO - CDS
;
; Name        :	GET_ALT
;
; Purpose     :	Extracts a CDS alternate entry from the database
;
; Explanation :	This routine extracts parameters which describe the CDS
;		alternate science plan status at a particular point in time.
;
; Use         :	GET_ALT, DATE, STATUS
;
; Inputs      :	DATE	= The date/time value that the user wishes the
;			  alternate plan entry for.  This can be in any of the
;			  standard CDS time formats.
;
; Opt. Inputs :	None.
;
; Outputs     :	STATUS	= Structure containing the alternate science plan
;			  entry.  It contains the following tags:
;
;			STRUCT_TYPE= The character string 'CDS-ALT'
;			PROG_ID	   = Program ID, linking one or more studies
;				     together
;			STUDY_ID   = Number defining the study
;			STUDYVAR   = Number giving the study variation ID
;			SCI_OBJ    = Science objective from the daily science
;				     meeting
;			SCI_SPEC   = Specific science objective from meeting
;			CMP_NO	   = Campaign number
;			OBJECT	   = Code for object planned to be observed
;			OBJ_ID	   = Object identification
;			START_TIME = Date/time of beginning of alternate plan,
;				     in TAI format
;			END_TIME   = Date/time of end of alternate plan, in
;				     TAI format
;			ORIG_DUR   = Original duration of the observation, in
;				     seconds, before being truncated by
;				     following time-tagged observations.
;			N_RASTERS1 = Variable number of rasters parameter
;			TRACKING   = True (1) if feature tracking to be used,
;				     or false (0) otherwise
;			N_POINTINGS= Number of pointings to apply to the study.
;			N_REPEAT_S = Number of times to repeat study
;			GSET_ID	   = GSET ID number
;			GET_RAW	   = True if raw data should be collected.
;			POINTINGS  = An array of pointings to use with the
;				     study.
;
;		If no entry is found, then a simpler structure is returned with
;		STUDY_ID set to -1.  POINTINGS is itself a structure of type
;		PLAN_PNT_STRUC with the following tags:
;
;			INS_X	   = Pointing in X relative to commanded pos.
;			INS_Y	   = Pointing in Y relative to commanded pos.
;			ZONE_ID	   = Pointing zone ID
;
; Opt. Outputs:	None.
;
; Keywords    :
;       ERRMSG    = If defined and passed, then any error messages will be
;                   returned to the user in this parameter rather than
;                   depending on the MESSAGE routine in IDL.  If no errors are
;                   encountered, then a null string is returned.  In order to
;                   use this feature, ERRMSG must be defined first, e.g.
;
;                       ERRMSG = ''
;                       GET_ALT, ERRMSG=ERRMSG, ... 
;                       IF ERRMSG NE '' THEN ...
;
;
; Calls       :	DATATYPE, DBOPEN, DBFIND, DBEXT, DBCLOSE, TRIM
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Category    :	Planning, Databases.
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 9 May 1995
;
; Modified    :	Version 1, William Thompson, GSFC, 9 May 1995
;		Version 2, William Thompson, GSFC, 22 May 1995
;			Changed way DBFIND is called, to speed up.
;		Version 3, William Thompson, GSFC, 28 September 1995
;			Added tags ORIG_DUR, GSET_ID, GET_RAW
;
; Version     :	Version 3, 28 September 1995
;-
;
	ON_ERROR, 2
;
;  Initialize the output structure, in case any errors occur.
;
	STATUS = {STUDY_ID: -1}
;
;  Check the number of parameters.
;
        IF N_PARAMS() NE 2 THEN BEGIN
           MESSAGE = 'Syntax:  GET_ALT, DATE, STATUS'
	   GOTO, HANDLE_ERROR
        ENDIF
;
;  Check the input parameter.
;
	IF N_ELEMENTS(DATE) NE 1 THEN BEGIN
           MESSAGE = 'DATE must be a scalar'
           GOTO, HANDLE_ERROR
	ENDIF
;
;  Convert the date to TAI format.
;
	IF DATATYPE(DATE,1) EQ 'Double' THEN TAI = DATE ELSE	$
		TAI = UTC2TAI(DATE)
;
;  Open the database.
;
	DBOPEN, 'sci_alt'
;
;  Search for entries with dates spanning that given by the user.
;
	ENTRIES = DBFIND('END_TIME>' + TRIM(TAI,'(F15.3)') + ',START_TIME<' + $
		TRIM(TAI,'(F15.3)') + ',DELETED=N', /SILENT)
;
;  If no entries were found, then return immediately.
;
	IF !ERR EQ 0 THEN BEGIN
		MESSAGE = 'Alternate science plan entry not found'
		GOTO, HANDLE_ERROR
	ENDIF
;
;  Extract the entry.
;
	DBEXT, ENTRIES, 'prog_id,study_id,studyvar,sci_obj,sci_spec,cmp_no', $
		PROG_ID, STUDY_ID, STUDYVAR, SCI_OBJ, SCI_SPEC, CMP_NO
	DBEXT, ENTRIES, 'object,obj_id,start_time,end_time,n_rasters1', $
		OBJECT, OBJ_ID, START_TIME, END_TIME, N_RASTERS1
	DBEXT, ENTRIES, 'tracking,n_pointings,n_repeat_s', $
		TRACKING, N_POINTINGS, N_REPEAT_S
	DBEXT, ENTRIES, 'orig_dur,gset_id,get_raw', ORIG_DUR, GSET_ID, GET_RAW
;
;  If more than one entry has been found, then use the one with the closest
;  start time to the requested time.  This avoids problems with times which are
;  right on the boundaries between entries.
;
	DIFF = ABS(START_TIME - TAI)
	I = (WHERE(DIFF EQ MIN(DIFF)))(0)
;
;  Define the structure to be used with the pointings.
;
	POINTINGS = {PLAN_PNT_STRUC, INS_X: 0.0, INS_Y: 0.0, ZONE_ID: 0}
;
;  Extract the pointings associated with this entry.
;
	N_POINTINGS = N_POINTINGS(I)
	IF N_POINTINGS GT 0 THEN BEGIN
		DBOPEN, 'point_alt'
		ENTRIES = DBFIND('START_TIME=' + TRIM(START_TIME(I),	$
			'(F15.3)') + ',DELETED=N', /SILENT)
		DBEXT, ENTRIES, 'pt_ind,ins_x,ins_y,zone_id',	$
			PT_IND, INS_X, INS_Y, ZONE_ID
;
;  Put the data into the structure.
;
		IF N_POINTINGS EQ 1 THEN BEGIN
			POINTINGS.INS_X   = INS_X(0)
			POINTINGS.INS_Y   = INS_Y(0)
			POINTINGS.ZONE_ID = ZONE_ID(0)
		END ELSE BEGIN
			POINTINGS = REPLICATE(POINTINGS, N_POINTINGS)
			S = SORT(PT_IND)
			POINTINGS.INS_X   = INS_X(S)
			POINTINGS.INS_Y   = INS_Y(S)
			POINTINGS.ZONE_ID = ZONE_ID(S)
		ENDELSE
	ENDIF
;
;  Define the output structure as a whole.
;
	STATUS = {STRUCT_TYPE: 'CDS-ALT',	$
		PROG_ID: PROG_ID(I),		$
		STUDY_ID: STUDY_ID(I),		$
		STUDYVAR: STUDYVAR(I),		$
		SCI_OBJ: SCI_OBJ(I),		$
		SCI_SPEC: SCI_SPEC(I),		$
		CMP_NO: CMP_NO(I),		$
		OBJECT: OBJECT(I),		$
		OBJ_ID: OBJ_ID(I),		$
		START_TIME: START_TIME(I),	$
		END_TIME: END_TIME(I),		$
		ORIG_DUR: ORIG_DUR(I),		$
		N_RASTERS1: N_RASTERS1(I),	$
		TRACKING: TRACKING(I),		$
		N_POINTINGS: N_POINTINGS,	$
		N_REPEAT_S: N_REPEAT_S(I),	$
		GSET_ID: GSET_ID(I),		$
		GET_RAW: GET_RAW(I),		$
		POINTINGS: POINTINGS}
	GOTO, FINISH
;
;  Error handling point.
;
HANDLE_ERROR:
	IF N_ELEMENTS(ERRMSG) NE 0 THEN ERRMSG = 'GET_ALT: ' + MESSAGE $
		ELSE MESSAGE, MESSAGE, /CONTINUE
;
;  Close the database, and return.
;
FINISH:
	DBCLOSE
;
	RETURN
	END
