;+
; Project     :	SOHO - CDS
;
; Name        :	GET_CDHS_SERIES()
;
; Purpose     :	Returns the parameters of an internal CDHS series ID.
;
; Explanation : Interrogates the series_id database to get the entry 
;               corresponding to this CDHS ID. If none found returns structure
;               with .ID = -1 else returns structure containing parameters.
;
; Use         : < st = get_cdhs_series( cdhs_id, /DTYPE ) >
;
; Inputs      : cdhs_id : Integer giving the CDHS series ID.
;
; Opt. Inputs : None.
;
; Outputs     : Structure array containing database information on this CDHS ID.
;               Tags :
;                 DATE        C*20  String containing UTC date entry created
;                 SW_VERSION  C*4   String containing software version used to create table
;                 CDHS_ID     I*2   Internal CDHS ID number
;                 ID          I*2   Database ID number
;                 VAR         I*2   Database variation number
;                 REUS        C*1   Fundamental series
;                 SFT         C*1   Solar feature tracking Y/N
;                 SOLARX      I*2   Initial solar x coord of study 
;                 SOLARY      I*2   Initial solar y coord of study 
;                 SLITN       I*2   Initial slit number of study
;                 GSETID      I*2   Initial GSET ID of study
;                 RPT         C*1   Repeat study indefinitely
;                 REPN        I*2   No. of times to repeat last raster and IEF flags
;
; Opt. Outputs:	None.
;
; Keywords    : DTYPE : Indicates should use the dseries_id database.
;
; Calls       :	dbopen, db_info, dbfind, dbext, dbclose,
;               get_utc, rem_dup.
;                
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Category    :	Command preparation.
;
; Prev. Hist. :	None.
;
; Written     :	Version 0.0, Martin Carter, RAL, 9/6/95
;
; Modified    :	Version 2. Minor change to error return. CDP, 9-Jun-95
;               Version 3  Changed database naming scheme added new tags
;                          MKC 9/11/95
;               Version 4 MKC 15/12/95
;                         Renamed fund reus
;               Version 5 Put in test for rseries db available. CDP, 17-Jan-96
;               Version 6 MKC, 18/1/96
;                         Put in infinite loop to keep accessing database until successful.
;               Version 7 MKC, 3/10/96
;                         Added DTYPE keyword.
;               Version 8 Ron Yurow 03/14/2008
;                         Added section to handle finding multiple
;                         entries in the database table.  Latest entry
;                         will now be selected.
;
; Version     :	Debug version ONLY
;-
;**********************************************************

FUNCTION get_cdhs_series, cdhs_id, DTYPE=dtype

  ; check if table defined 

  ; set up database name

  IF KEYWORD_SET(dtype) THEN database = 'dseries_id' ELSE database = 'rseries_id'

  ; open database for reading

  unavail = 1
  WHILE unavail DO BEGIN
    dbopen, database, unavail=unavail
    IF unavail THEN BEGIN
      PRINT,'Database ' + database + ' unavailable'
      WAIT, 1
    ENDIF
  ENDWHILE

  ; check if table exists 

  ; check database has some entries
  ; else dbfind will flag error

  nentries = db_info ( 'entries' )

  IF nentries(0) EQ 0 THEN RETURN, {id: -1}

  ; set up search criteria

  search = 'CDHS_ID=' + STRING(cdhs_id) 

  list = dbfind ( search, /SILENT)

  ; list is array of entries in database
  ; if no entries then list contains a zero
  ; valid entries start at 1 

  ; if no entries found, return

  IF list(0) EQ 0 THEN RETURN, {id: -1}      

  ; if more than one entry found then report error

  ; The following line was removed 3/14/08 Ron Yurow
  ; IF N_ELEMENTS(list) GT 1 THEN MESSAGE, 'Invalid no. of entries for CDHS ID' + STRTRIM(cdhs_id,1)
  

  ; The following section was added 3/14/08 Ron Yurow
  ; Check for more then one entry.  If we find multiple entries, then we will
  ; use the entry with latest date.

  IF N_ELEMENTS (list) GT 1 THEN BEGIN

  ; More then one entry, we will choose the entry with the latest date to use.

     MESSAGE, 'More then one entry for CDHS_ID: ' + STRTRIM (cdhs_id, 1), /CONT

  ; Get the dates for each of the entries that we found.

     dbext, list, 'DATE', date

  ; Convert the dates to SECONDS

     s = UTC2SEC (ANYTIM2UTC (date))

  ; Find the latest entry.

     last = REVERSE (SORT (s))
     last = last [0]

  ; OK, got it.  Reset last to the cronologically last entry.

     list = list [last]

     MESSAGE, 'Using entry with date: ' + date [last], /CONT

  ENDIF
  ;**** End of section added on 3/14/08

  ; extract entry

  dbext, list, 'DATE,SW_VERSION,ID,VAR,REUS,SFT,SOLARX,SOLARY,SLITN,GSETID,RPT,REPN', $
                date,sw_version,id,var,reus,sft,solarx,solary,slitn,gsetid,rpt,repn

  ; close database

;
;  debug
; ********
;
  if n_elements(sw_version) eq 0 then help,list,date,id,cdhs_id

  dbclose, database

  ; return structure containing results

  RETURN, { date:date(0), sw_version:sw_version(0), cdhs_id:cdhs_id(0), id:id(0), var:var(0), reus:reus(0), $
            sft:sft(0), solarx:solarx(0), solary:solary(0), slitn:slitn(0), gsetid:gsetid(0), rpt:rpt(0),   $
            repn:repn(0) }

END
