;+
; Project     :	SOHO - CDS
;
; Name        :	GET_CDHS_VDSWIN()
;
; Purpose     :	Returns the parameters of an internal CDHS vdswin ID.
;
; Explanation : Interrogates the vdswin_id database to get the entry 
;               corresponding to this CDHS ID. If none found it returns
;               a structure with .ID =  -1 else returns structure containing 
;               parameters.
;
; Use         : IDL>  st = get_cdhs_vdswin( cdhs_id )
;
; Inputs      : cdhs_id : Integer giving the CDHS vdswin ID.
;
; Opt. Inputs : None.
;
; Outputs     : Structure array containing database information on this CDHS ID.
;               Tags :
;                 DATE        C*20  String containing UTC date entry created
;                 SW_VERSION  C*4   String containing software version used to create table
;                 CDHS_ID     I*2   Internal CDHS ID number
;                 ID          I*2   Database ID number
;                 VERSION     I*2   Calibration version number
;                 IEFID       I*2   ID of flag master IEF scheme
;
; Opt. Outputs:	None.
;
; Keywords    : None.
;
; Calls       :	dbopen, db_info, dbfind, dbext, dbclose,
;                
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Category    :	Command preparation.
;
; Prev. Hist. :	None.
;
; Written     :	Version 0.0, Martin Carter, RAL, 21/11/95
;
; Modified    :	Version 1 MKC, 18/1/96
;                         Put in infinite loop to keep accessing database until successful.
;               Version 2 MKC 19/2/96
;                         Changed some of the parameters in the database.
;
; Version     :	Version 2, 19/02/96
;-
;**********************************************************

FUNCTION get_cdhs_vdswin, cdhs_id

  ; check if table defined 

  ; open database for reading

  unavail = 1
  WHILE unavail DO BEGIN
    dbopen, 'rvdswin_id', unavail=unavail
    IF unavail THEN BEGIN
      PRINT,'Database rvdswin_id unavailable'
      WAIT, 1
    ENDIF
  ENDWHILE

  ; check if table exists 

  ; check database has some entries
  ; else dbfind will flag error

  nentries = db_info ( 'entries' )

  IF nentries(0) EQ 0 THEN RETURN, {id: -1}

  ; set up search criteria

  search = 'CDHS_ID=' + STRING(cdhs_id) 

  list = dbfind ( search, /SILENT)

  ; list is array of entries in database
  ; if no entries then list contains a zero
  ; valid entries start at 1 

  ; if no entries found, return

  IF list(0) EQ 0 THEN RETURN,  {id: -1}

  ; if more than one entry found then report error

  IF N_ELEMENTS(list) GT 1 THEN MESSAGE, 'Invalid no. of entries for CDHS ID' + STRTRIM(cdhs_id,1)

  ; extract entry

  dbext, list, 'DATE,SW_VERSION,ID,VERSION,IEFID', $
                date,sw_version,id,version,iefid

  ; close database

  dbclose, 'rvdswin_id'

  ; return structure containing results

  RETURN, { date:date(0), sw_version:sw_version(0), cdhs_id:cdhs_id(0), $
            id:id(0), version:version(0), iefid:iefid(0) }

END


