	PRO GET_COMPRESS, COMP_ID, DESC, ERRMSG=ERRMSG
;+
; Project     :	SOHO - CDS
;
; Name        :	GET_COMPRESS
;
; Purpose     :	Extracts a SOHO compression description from the database
;
; Explanation :	This routine extracts parameters which describes a SOHO
;		compression method.
;
; Use         :	GET_COMPRESS, COMP_ID, DESC
;
; Inputs      :	COMP_ID	= The unique compression ID.
;
; Opt. Inputs :	None.
;
; Outputs     :	DESC = Structure containing the compression description.  It
;		       contains the following tags:
;
;			COMP_ID	 = Compression ID number.  If a match is
;				   not found, then a simpler structure is
;				   returned with this set to 0.
;			COMPDESC = A short description of the compression
;				   method.
;
; Opt. Outputs:	None.
;
; Keywords    :
;       ERRMSG    = If defined and passed, then any error messages will be
;                   returned to the user in this parameter rather than
;                   depending on the MESSAGE routine in IDL.  If no errors are
;                   encountered, then a null string is returned.  In order to
;                   use this feature, ERRMSG must be defined first, e.g.
;
;                       ERRMSG = ''
;                       GET_COMPRESS, ERRMSG=ERRMSG, ...
;                       IF ERRMSG NE '' THEN ...
;
;
; Calls       :	DATATYPE, DBOPEN, DBFIND, DBEXT, DBCLOSE
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Category    :	Planning, Databases.
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 25 July 1994
;
; Modified    :	Version 1, William Thompson, GSFC, 27 July 1994
;               Version 2, Liyun Wang, GSFC/ARC, September 22, 1994
;                  Added the keyword ERRMSG.
;		Version 3, William Thompson, GSFC, 29 December 1994
;			Fixed bug when inputs are bytes.
;		Version 4, William Thompson, GSFC, 28 April 1995
;			Returns an error string when no entry is found.
;
; Version     :	Version 4, 28 April 1995
;-
;
	ON_ERROR, 2
;
;  Define a null result.  If the routine is successful, this will be updated
;  later.
;
	DESC = {COMP_ID: 0}
;
;  Check the number of parameters.
;
        IF N_PARAMS() NE 2 THEN BEGIN
           MESSAGE = 'Syntax:  GET_COMPRESS, COMP_ID, DESC'
	   GOTO, HANDLE_ERROR
        ENDIF
;
;  Check the input parameter COMP_ID.
;
	TYPE = DATATYPE(COMP_ID,2)
	IF TYPE EQ 0 THEN BEGIN
           MESSAGE = 'COMP_ID is undefined'
           GOTO, HANDLE_ERROR
	END ELSE IF TYPE GE 4 THEN BEGIN
           MESSAGE = 'COMP_ID must be an integer'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(COMP_ID) NE 1 THEN BEGIN
           MESSAGE = 'COMP_ID must be a scalar'
           GOTO, HANDLE_ERROR
	ENDIF
;
;  Open the database.
;
	DBOPEN, 'compression'
;
;  Search on the COMP_ID field.
;
	ENTRIES = DBFIND('COMP_ID=' + STRTRIM(LONG(COMP_ID),2), /SILENT)
;
;  If no entries were found, then return immediately.
;
	IF !ERR EQ 0 THEN BEGIN
		MESSAGE = 'Compression ID ' + TRIM(LONG(COMP_ID)) +	$
			' not found'
		GOTO, HANDLE_ERROR
	ENDIF
;
;  Extract the relevant entry from the database.
;
	DBEXT, ENTRIES(0), 'compdesc', COMPDESC
;
;  Define the output structure.
;
	DESC = {COMP_ID: COMP_ID(0),	$
		COMPDESC: COMPDESC(0)}
	GOTO, FINISH
;
;  Error handling point.
;
HANDLE_ERROR:
	IF N_ELEMENTS(ERRMSG) NE 0 THEN ERRMSG = 'GET_COMPRESS: ' + MESSAGE $
		ELSE MESSAGE, MESSAGE, /CONTINUE
;
;  Close the database, and return.
;
FINISH:
	DBCLOSE
;
	RETURN
	END
