	PRO GET_DATAWIN, DW_ID, DEF, ERRMSG=ERRMSG
;+
; Project     :	SOHO - CDS
;
; Name        :	GET_DATAWIN
;
; Purpose     :	Extracts a data window list definition from the database
;
; Explanation :	This routine extracts a complete data extraction window list
;		definition from the database for a requested data window list.
;
; Use         :	GET_DATAWIN, DW_ID, DEF
;
; Inputs      :	DW_ID = Data extraction window list ID number
;
; Opt. Inputs :	None.
;
; Outputs     :	DEF	= Structure containing the linelist definition.  It
;			  contains the following tags:
;
;			DW_ID	 = Data window list ID number.  If the
;				   requested data window list is not found,
;				   then a simpler structure is returned, with
;				   this set to -1.
;			DETECTOR = Either "G" for GIS or "N" for NIS.
;			DW_DESC  = A short description of the data window list
;				   beyond what is given in the associated line
;				   list description, e.g. "Full slit, 10 pixels
;				   wide".
;			W_WIDTH  = Width in pixels used to generate the
;				   windows.
;			W_HEIGHT = The height in pixels used to generate the
;				   windows.  VDS only, for GIS this is set to
;				   unity.
;			VDS_BACK = Either 0 for off, or 1 for on,
;				   representing whether or not VDS background
;				   windows are being used.  For GIS window
;				   lists this is 0.
;			N_FLAGS	 = The number of flag windows.
;			LL_ID	 = The line list ID number for the line list
;				   that this window list was derived from, or
;				   zero to signal that the data window list is
;				   not connected to a line list.
;			LL_DESC  = A description of the line list, e.g.
;				   "Temperature sensitive line pairs".
;			N_LINES	 = The number of lines in the line list.
;			WINS	 = An array containing the window definitions.
;
;			  The window definitions themselves are structures, of
;			  type "cds_windows", with the following tags:
;
;			WIN_NAME = Window name, derived from the line name in
;				   the line list database.  Can be up to 40
;				   characters.
;			WIN_DEF	 = The window definition.  This is four
;				   numbers, although GIS window definitions
;				   only use two of them.
;			WIN_FLAG = Whether or not the window can be used to
;				   generate a flag.  Either 0 (false) or 1
;				   (true).
;
; Opt. Outputs:	None.
;
; Keywords    :
;       ERRMSG    = If defined and passed, then any error messages will be
;                   returned to the user in this parameter rather than
;                   depending on the MESSAGE routine in IDL.  If no errors are
;                   encountered, then a null string is returned.  In order to
;                   use this feature, ERRMSG must be defined first, e.g.
;
;                       ERRMSG = ''
;                       GET_DATAWIN, ERRMSG=ERRMSG, ... 
;                       IF ERRMSG NE '' THEN ...
;
;
; Calls       :	DATATYPE, DBOPEN, DBFIND, DBEXT, DBCLOSE
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Category    :	Planning, Databases.
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 1 April 1994
;
; Modified    :	Version 1, William Thompson, GSFC, 1 August 1994
;               Version 2, Liyun Wang, GSFC/ARC, September 22, 1994
;                  Added the keyword ERRMSG.
;		Version 3, William Thompson, GSFC, 29 December 1994
;			Fixed bug when inputs are bytes.
;		Version 4, William Thompson, GSFC, 15 March 1995
;			Removed LL_TITLE
;		Version 5, William Thompson, GSFC, 28 April 1995
;			Returns an error string when no entry is found.
;		Version 6, William Thompson, GSFC, 20 June 1995
;			Added tags N_FLAGS and WIN_FLAG
;
; Version     :	Version 6, 20 June 1995
;-
;
	ON_ERROR, 2
;
;  Define a null result.  If the routine is successful, this will be updated
;  later.
;
	DEF = {DW_ID: -1}
;
;  Check the number of parameters.
;
        IF N_PARAMS() NE 2 THEN BEGIN
           MESSAGE = 'Syntax:  GET_DATAWIN, DW_ID, DEF'
	   GOTO, HANDLE_ERROR
        ENDIF
;
;  Check the input parameter DW_ID.
;
	TYPE = DATATYPE(DW_ID,2)
	IF TYPE EQ 0 THEN BEGIN
           MESSAGE = 'DW_ID is undefined'
           GOTO, HANDLE_ERROR
	END ELSE IF TYPE GE 4 THEN BEGIN
           MESSAGE = 'DW_ID must be an integer'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(DW_ID) NE 1 THEN BEGIN
           MESSAGE = 'DW_ID must be a scalar'
           GOTO, HANDLE_ERROR
	ENDIF
;
;  Open the database.
;
	DBOPEN, 'datawin2,datawin'
;
;  Search on the DW_ID field.
;
	ENTRIES = DBFIND('DW_ID='+STRTRIM(LONG(DW_ID),2), /SILENT)
;
;  If no entries were found, then return immediately.
;
	IF !ERR EQ 0 THEN BEGIN
		MESSAGE = 'Data window ID ' + TRIM(LONG(DW_ID)) + ' not found'
		GOTO, HANDLE_ERROR
	ENDIF
;
;  Extract the linelist pointer to determine if it's zero or not.
;
	DBEXT, ENTRIES(0), 'linelist_ptr', LINELIST_PTR
;
;  Depending on the value of LINELIST_PTR, extract the relevant entries from
;  the database.  Note that when the linelist database is included in the
;  search there's some ambiguity about which "DETECTOR" field is being read,
;  but since ADD_DATAWIN enforces that they match, this shouldn't be a problem.
;
	IF LINELIST_PTR(0) NE 0 THEN BEGIN
		DBOPEN, 'datawin2,datawin,linelist'
		DBEXT, ENTRIES, 'win_name,win_def,detector,dw_desc,' +	$
			'vds_back,ll_id,ll_desc,n_lines', WIN_NAME, $
			WIN_DEF, DETECTOR, DW_DESC, VDS_BACK, LL_ID,	$
			LL_DESC, N_LINES
		DBEXT, ENTRIES, 'w_width,w_height,win_flag,n_flags',	$
			W_WIDTH, W_HEIGHT, WIN_FLAG, N_FLAGS
	END ELSE BEGIN
		DBEXT, ENTRIES, 'win_name,win_def,detector,dw_desc,vds_back', $
			WIN_NAME, WIN_DEF, DETECTOR, DW_DESC, VDS_BACK
		DBEXT, ENTRIES, 'w_width,w_height,win_flag,n_flags',	$
			W_WIDTH, W_HEIGHT, WIN_FLAG, N_FLAGS
		LL_ID    = 0
		LL_DESC  = ''
		N_LINES  = 0
	ENDELSE
;
;  Define the structure for the window definitions.
;
	WINS = {CDS_WINDOWS,		$
		WIN_NAME: '',		$
		WIN_DEF: [0,0,0,0],	$
		WIN_FLAG: 0}
;
	IF N_ELEMENTS(ENTRIES) EQ 1 THEN BEGIN
		WINS.WIN_NAME = WIN_NAME(0)
		WINS.WIN_DEF  = WIN_DEF
		WINS.WIN_FLAG = WIN_FLAG(0)
	END ELSE BEGIN
		WINS = REPLICATE(WINS, N_ELEMENTS(ENTRIES))
		WINS.WIN_NAME = WIN_NAME
		WINS.WIN_DEF  = WIN_DEF
		WINS.WIN_FLAG = WIN_FLAG
	ENDELSE
;
;  Define the output structure as a whole.
;
	DEF = {DW_ID: DW_ID(0),		$
		DETECTOR: DETECTOR(0),	$
		DW_DESC: DW_DESC(0),	$
		W_WIDTH: W_WIDTH(0),	$
		W_HEIGHT: W_HEIGHT(0),	$
		VDS_BACK: VDS_BACK(0),	$
		N_FLAGS: N_FLAGS(0),	$
		LL_ID: LL_ID(0),	$
		LL_DESC: LL_DESC(0),	$
		N_LINES: N_LINES(0),	$
		WINS: WINS}
	GOTO, FINISH
;
;  Error handling point.
;
HANDLE_ERROR:
	IF N_ELEMENTS(ERRMSG) NE 0 THEN ERRMSG = 'GET_DATAWIN: ' + MESSAGE $
		ELSE MESSAGE, MESSAGE, /CONTINUE
;
;  Close the database, and return.
;
FINISH:
	DBCLOSE
;
	RETURN
	END
