	PRO GET_EXPER, P1, P2, P3, ERRMSG=ERRMSG
;+
; Project     :	SOHO - CDS
;
; Name        :	GET_EXPER
;
; Purpose     :	Extracts a CDS raster entry from the as-run database
;
; Explanation :	This routine extracts parameters which describe the CDS as-run
;		database at the raster (experiment) level for a given SEQ_NUM
;		value.  Optionally, one can pass PROG_NUM and SEQ_IND instead.
;
; Use         :	GET_EXPER, SEQ_NUM, OBS
;		GET_EXPER, PROG_NUM, SEQ_IND, OBS
;
; Inputs      :	SEQ_NUM	 = The sequence number which serves as the primary
;			   index into the EXPERIMENT database.
;
;		PROG_NUM = The program number (study counter).
;		SEQ_IND	 = The raster index within the study.
;
; Opt. Inputs :	None.
;
; Outputs     :	OBS	= Structure containing the detailed science plan
;			  record.  It contains the following tags:
;
;			STRUCT_TYPE= The character string 'CDS-EXPERIMENT'
;			PROG_NUM   = Study counter number.  This forms the
;				     primary key for this database.
;			PROG_ID	   = Program ID, linking one or more studies
;				     together
;			PROG_IND   = An index, reflecting one's position within
;				     a repeated sequence of studies via the
;				     N_REPEAT_S mechanism.
;			STUDY_ID   = Number defining the study
;			STUDYVAR   = Number giving the study variation ID
;			SCI_OBJ    = Science objective from the daily science
;				     meeting
;			SCI_SPEC   = Specific science objective from meeting
;			CMP_NO	   = Campaign number
;			OBJECT	   = Code for object planned to be observed
;			OBJ_ID	   = Object identification
;			SEQ_NUM	   = Unique sequence number.  This serves as
;				     the primary key to the database.
;			OBS_SEQ	   = A character string derived from the raster
;				     number and the variation number,
;				     e.g. "R4V3".
;			COUNT	   = The number of repeated observations making
;				     up this experiment.  Normally 1.
;			SEQ_IND	   = The raster counter
;			RAS_ID	   = The raster ID number
;			RAS_VAR	   = The raster variation index
;			EXPTIME	   = The exposure time, in seconds
;			OBS_MODE   = The observing mode.
;			DW_ID	   = The data window ID number
;			DATE_OBS   = Date/time of beginning of observation,
;				     in TAI format
;			DATE_END   = Date/time of end of observation, in TAI
;				     format
;			OBT_TIME   = Onboard date/time of beginning of
;				     observation, in TAI format
;			OBT_END	   = Onboard date/time of end of observation,
;				     in TAI format
;			XCEN	   = Center of instrument field-of-view in
;				     solar X
;			YCEN	   = Center of instrument field-of-view in
;				     solar Y
;			ANGLE	   = Angle of instrument relative to solar
;				     north
;			IXWIDTH	   = Width of field-of-view in instrument X
;			IYWIDTH	   = Width of field-of-view in instrument Y
;			INS_X0	   = The instrument origin pointing in X
;			INS_Y0	   = The instrument origin pointing in Y
;			INS_ROLL   = The instrument roll angle
;			SC_X0	   = The spacecraft pointing in X
;			SC_Y0	   = The spacecraft pointing in Y
;			SC_ROLL	   = The spacecraft roll
;			WAVEMIN	   = The minimum wavelength of observation, in
;				     Angstroms.
;			WAVEMAX	   = The maximum wavelength, in Angstroms
;			TRACKING   = True if tracking is on
;			SER_ID	   = The series ID number
;			OPSLBITS   = The OPS-L status bits at the start of the
;				     observation.
;			OPSRBITS   = The OPS-R status bits at the start of the
;				     observation.
;			SLIT_POS   = The slit position at the start of the
;				     observation.
;			MIR_POS	   = The mirror position at the start of the
;				     observation.
;			EV_ENAB	   = True if event recognition enabled
;			COMP_ERR   = True if a compression error was
;				     encountered.
;			VDS_PMCP   = The VDS MCP programmable voltage setting
;			VDS_MODE   = The VDS readout mode.
;			VDS_ORI	   = True if the VDS telemetry data is oriented
;				     by columns
;			VDS_ACC	   = True if VDS is operated in accumulate mode
;			GSET_ID	   = GIS Setup ID
;			DETECTOR   = Either N or G for NIS/GIS.
;			ZONE_ID	   = Pointing zone ID
;			SLIT_NUM   = Slit number
;			FILENAME   = FITS file name
;			SEQVALID   = True for valid science data
;			DATASRC	   = Data source, either Q(uicklook) or F(inal
;				     distribution).
;			COMMENTS   = Either Y or N
;			DATE_MOD   = Date and time when catalog record was last
;				     modified.
;
;		If no entry is found, then a simpler structure is returned with
;		SEQ_NUM and STUDY_ID both set to -1.
;
; Opt. Outputs:	None.
;
; Keywords    :
;       ERRMSG    = If defined and passed, then any error messages will be
;                   returned to the user in this parameter rather than
;                   depending on the MESSAGE routine in IDL.  If no errors are
;                   encountered, then a null string is returned.  In order to
;                   use this feature, ERRMSG must be defined first, e.g.
;
;                       ERRMSG = ''
;                       GET_EXPER, ERRMSG=ERRMSG, ... 
;                       IF ERRMSG NE '' THEN ...
;
;
; Calls       :	DATATYPE, DBOPEN, DBFIND, DBEXT, DBCLOSE, TRIM
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Category    :	Planning, Databases.
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 30 June 1995
;
; Modified    :	Version 1, William Thompson, GSFC, 30 June 1995
;		Version 2, William Thompson, GSFC, 23 February 1996
;			Modified for version 2 of the experiment databases.
;		Version 3, William Thompson, GSFC, 2 April 1996
;			Added parameters DATASRC and OBS_PROG
;		Version 4, William Thompson, GSFC, 12 April 1996
;			Look in both experiment* and fexperiment*
;		Version 5, William Thompson, GSFC, 25 April 1996
;			Fixed bug in version 6 related to environment name
;		Version 6, William Thompson, GSFC, 1 June 1998
;			Look for entries in "_year" files
;
; Version     :	Version 6, 1 June 1998
;-
;
	ON_ERROR, 2
;
;  Initialize the output structure, in case any errors occur.
;
	OBS = {SEQ_NUM: -1, STUDY_ID: -1}
;
;  Check the number of parameters.
;
        IF N_PARAMS() LT 2 THEN BEGIN
           MESSAGE = 'Syntax:  GET_EXPER, SEQ_NUM, OBS  or  ' +	$
		'GET_EXPER, PROG_NUM, SEQ_IND, OBS'
	   GOTO, HANDLE_ERROR
        ENDIF
;
;  Check the input parameters.
;
	IF N_ELEMENTS(P1) NE 1 THEN BEGIN
           MESSAGE = 'P1 must be a scalar'
           GOTO, HANDLE_ERROR
	END ELSE IF DATATYPE(P1,2) GT 3 THEN BEGIN
	   MESSAGE = 'P1 must be an integer value'
	   GOTO, HANDLE_ERROR
	ENDIF
;
	IF N_PARAMS() EQ 3 THEN BEGIN
	    IF N_ELEMENTS(P2) NE 1 THEN BEGIN
               MESSAGE = 'P2 must be a scalar'
	       GOTO, HANDLE_ERROR
	    END ELSE IF DATATYPE(P2,2) GT 3 THEN BEGIN
	       MESSAGE = 'P2 must be an integer value'
	       GOTO, HANDLE_ERROR
	    ENDIF
	ENDIF
;
;  Open the database.
;
	VERSION = ''
	TRAILER = ''
	DBOPEN, 'experiment2,experiment1,experiment'
;
;  Search for the requested entry.
;
FIND_ENTRY:
	IF N_PARAMS() EQ 2 THEN BEGIN
		ENTRY = DBFIND('SEQ_NUM=' + TRIM(P1), /SILENT)
	END ELSE IF N_PARAMS() EQ 3 THEN BEGIN
		ENTRY = DBFIND('PROG_NUM=' + TRIM(P1) + ',SEQ_IND=' +	$
			TRIM(P2), /SILENT)
	ENDIF
;
;  If no entries were found, then either try the "f" version, or return
;  immediately.
;
	IF !ERR EQ 0 THEN BEGIN
	    IF VERSION EQ '' THEN BEGIN
		VERSION = 'f'
	    END ELSE IF TRAILER EQ '' THEN BEGIN
		ITRAILER = 1996
		TRAILER = '_' + TRIM(ITRAILER)
	    END ELSE BEGIN
		ITRAILER = ITRAILER + 1
		TRAILER = '_' + TRIM(ITRAILER)
	    ENDELSE
;
	    IF FIND_WITH_DEF('fexperiment2'+TRAILER+'.dbf','$ZDBASE') NE '' $
		    THEN BEGIN
		DBOPEN, 'fexperiment2' + TRAILER + ',fexperiment1' + TRAILER +$
			',fexperiment' + TRAILER
		GOTO, FIND_ENTRY
	    ENDIF
	    MESSAGE = 'No experiment as-run entry found meeting search criteria'
	    GOTO, HANDLE_ERROR
	ENDIF
;
;  Extract the entry.
;
	DBEXT, ENTRY(0), 'seq_num,obs_seq,count,seq_ind,ras_id,ras_var,' + $
		'exptime,obs_mode,dw_id,date_obs,date_end,obt_time', SEQ_NUM, $
		OBS_SEQ, COUNT, SEQ_IND, RAS_ID, RAS_VAR, EXPTIME, OBS_MODE, $
		DW_ID, DATE_OBS, DATE_END, OBT_TIME
	DBEXT, ENTRY(0), 'obt_end,xcen,ycen,angle,ixwidth,iywidth,ins_x0,' + $
		'ins_y0,ins_roll,sc_x0,sc_y0,sc_roll', OBT_END, XCEN, YCEN, $
		ANGLE, IXWIDTH, IYWIDTH, INS_X0, INS_Y0, INS_ROLL, SC_X0, $
		SC_Y0, SC_ROLL
	DBEXT, ENTRY(0), 'wavemin,wavemax,tracking,ser_id,opslbits,opsrbits,'+$
		'slit_pos,mir_pos,ev_enab,comp_err,vds_mode,vds_ori', $
		WAVEMIN, WAVEMAX, TRACKING, SER_ID, OPSLBITS, OPSRBITS,	$
		SLIT_POS, MIR_POS, EV_ENAB, COMP_ERR, VDS_MODE, VDS_ORI
	DBEXT, ENTRY(0),'vds_acc,gset_id,seqvalid,comments,date_mod,datasrc,'+$
		'prog_num,vds_pmcp,detector,zone_id,slit_num,filename', $
		VDS_ACC, GSET_ID, SEQVALID, COMMENTS, DATE_MOD, DATASRC, $
		PROG_NUM, VDS_PMCP, DETECTOR, ZONE_ID, SLIT_NUM, FILENAME
;
;  Also get the part from the MAIN database.
;
	GET_MAIN, PROG_NUM, MAIN
;
;  Define the output structure.
;
	OBS = { STRUCT_TYPE: 'CDS-EXPERIMENT',	$
		PROG_NUM: PROG_NUM(0),	$
		PROG_ID: MAIN.PROG_ID,	$
		PROG_IND: MAIN.PROG_IND,$
		STUDY_ID: MAIN.STUDY_ID,$
		STUDYVAR: MAIN.STUDYVAR,$
		OBS_PROG: MAIN.OBS_PROG,$
		SCI_OBJ: MAIN.SCI_OBJ,	$
		SCI_SPEC: MAIN.SCI_SPEC,$
		CMP_NO: MAIN.CMP_NO,	$
		OBJECT: MAIN.OBJECT,	$
		OBJ_ID: MAIN.OBJ_ID,	$
		SEQ_NUM: SEQ_NUM(0),	$
		OBS_SEQ: OBS_SEQ(0),	$
		COUNT: COUNT(0),	$
		SEQ_IND: SEQ_IND(0),	$
		RAS_ID: RAS_ID(0),	$
		RAS_VAR: RAS_VAR(0),	$
		EXPTIME: EXPTIME(0),	$
		OBS_MODE: OBS_MODE(0),	$
		DW_ID: DW_ID(0),	$
		DATE_OBS: DATE_OBS(0),	$
		DATE_END: DATE_END(0),	$
		OBT_TIME: OBT_TIME(0),	$
		OBT_END: OBT_END(0),	$
		XCEN: XCEN(0),		$
		YCEN: YCEN(0),		$
		ANGLE: ANGLE(0),	$
		IXWIDTH: IXWIDTH(0),	$
		IYWIDTH: IYWIDTH(0),	$
		INS_X0: INS_X0(0),	$
		INS_Y0: INS_Y0(0),	$
		INS_ROLL: INS_ROLL(0),	$
		SC_X0: SC_X0(0),	$
		SC_Y0: SC_Y0(0),	$
		SC_ROLL: SC_ROLL(0),	$
		WAVEMIN: WAVEMIN(0),	$
		WAVEMAX: WAVEMAX(0),	$
		TRACKING: TRACKING(0),	$
		SER_ID: SER_ID(0),	$
		OPSLBITS: OPSLBITS(0),	$
		OPSRBITS: OPSRBITS(0),	$
		SLIT_POS: SLIT_POS(0),	$
		MIR_POS: MIR_POS(0),	$
		EV_ENAB: EV_ENAB(0),	$
		COMP_ERR: COMP_ERR(0),	$
		VDS_PMCP: VDS_PMCP(0),	$
		VDS_MODE: VDS_MODE(0),	$
		VDS_ORI: VDS_ORI(0),	$
		VDS_ACC: VDS_ACC(0),	$
		GSET_ID: GSET_ID(0),	$
		DETECTOR: DETECTOR(0),	$
		ZONE_ID: ZONE_ID(0),	$
		SLIT_NUM: SLIT_NUM(0),	$
		FILENAME: FILENAME(0),	$
		SEQVALID: SEQVALID(0),	$
		DATASRC: DATASRC(0),	$
		COMMENTS: COMMENTS(0),	$
		DATE_MOD: DATE_MOD(0)}
	GOTO, FINISH
;
;  Error handling point.
;
HANDLE_ERROR:
	IF N_ELEMENTS(ERRMSG) NE 0 THEN ERRMSG = 'GET_EXPER: ' + MESSAGE $
		ELSE MESSAGE, MESSAGE, /CONTINUE
;
;  Return the parameter OBS, close the database, and return.
;
FINISH:
	IF N_PARAMS() EQ 2 THEN P2 = OBS ELSE P3 = OBS
	DBCLOSE
;
	RETURN
	END
