	PRO GET_F_RASTER, RAS_ID, DEF, ERRMSG=ERRMSG
;+
; Project     :	SOHO - CDS
;
; Name        :	GET_F_RASTER
;
; Purpose     :	Extracts a fundamental raster definition from the database
;
; Explanation :	This routine extracts the fundamental parameters of a raster
;		from the database for the requested raster ID number.
;
; Use         :	GET_F_RASTER, RAS_ID, DEF
;
; Inputs      :	RAS_ID	= Raster ID number
;
; Opt. Inputs :	None.
;
; Outputs     :	DEF	= Structure of type "cds_f_raster" containing the
;			  fundamental parameters of a raster definition.  It
;			  contains the following tags:
;
;			RAS_ID	 = Raster ID number.  If the requested raster
;				   is not found, then a simpler structure is
;				   returned, with this set to -1.
;			DETECTOR = Either "G" for GIS or "N" for NIS.
;			RAS_DESC = A short description of the raster, giving
;				   its purpose.
;			SLIT_NUM = The ID number for the slit to be used.
;			XSTEP	 = The step size in the X direction, in arcsec.
;			YSTEP	 = The step size in the Y direction, in arcsec.
;			NX	 = The number of exposure positions in X.
;			NY	 = The number of exposure positions in Y.
;
; Opt. Outputs:	None.
;
; Keywords    :
;       ERRMSG    = If defined and passed, then any error messages will be
;                   returned to the user in this parameter rather than
;                   depending on the MESSAGE routine in IDL.  If no errors are
;                   encountered, then a null string is returned.  In order to
;                   use this feature, ERRMSG must be defined first, e.g.
;
;                       ERRMSG = ''
;                       GET_F_RASTER, ERRMSG=ERRMSG, ... 
;                       IF ERRMSG NE '' THEN ...
;
;
; Calls       :	DATATYPE, DBOPEN, DBFIND, DBEXT, DBCLOSE
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Category    :	Planning, Databases.
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 4 April 1994
;
; Modified    :	Version 1, William Thompson, GSFC, 28 July 1994
;               Version 2, Liyun Wang, GSFC/ARC, September 22, 1994
;                  Added the keyword ERRMSG.
;		Version 3, William Thompson, GSFC, 29 December 1994
;			Fixed bug when inputs are bytes.
;		Version 4, William Thompson, GSFC, 28 April 1995
;			Returns an error string when no entry is found.
;
; Version     :	Version 4, 28 April 1995
;-
;
	ON_ERROR, 2
;
;  Define a null result.  If the routine is successful, this will be updated
;  later.
;
	DEF = {RAS_ID: -1}
;
;  Check the number of parameters.
;
        IF N_PARAMS() NE 2 THEN BEGIN
           MESSAGE = 'Syntax:  GET_F_RASTER, RAS_ID, DEF'
	   GOTO, HANDLE_ERROR
        ENDIF
;
;  Check the input parameter.
;
	TYPE = DATATYPE(RAS_ID,2)
	IF TYPE EQ 0 THEN BEGIN
           MESSAGE = 'RAS_ID is undefined'
           GOTO, HANDLE_ERROR
	END ELSE IF TYPE GE 4 THEN BEGIN
           MESSAGE = 'RAS_ID must be an integer'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(RAS_ID) NE 1 THEN BEGIN
           MESSAGE = 'RAS_ID must be a scalar'
           GOTO, HANDLE_ERROR
	ENDIF
;
;  Open the database.
;
	DBOPEN, 'raster'
;
;  Search on the RAS_ID field.
;
	ENTRIES = DBFIND('RAS_ID='+STRTRIM(LONG(RAS_ID),2), /SILENT)
;
;  If no entries were found, then return immediately.
;
	IF !ERR EQ 0 THEN BEGIN
		MESSAGE = 'Raster ID ' + TRIM(LONG(RAS_ID)) + ' not found'
		GOTO, HANDLE_ERROR
	ENDIF
;
;  Extract the relevant parameters from the database.
;
	DBEXT, ENTRIES(0), 'detector,ras_desc,slit_num,xstep,ystep,nx,ny', $
		DETECTOR, RAS_DESC, SLIT_NUM, XSTEP, YSTEP, NX, NY
;
;  Define the output structure.
;
	DEF = {CDS_F_RASTER,		$
		RAS_ID: RAS_ID(0),	$
		DETECTOR: DETECTOR(0),	$
		RAS_DESC: RAS_DESC(0),	$
		SLIT_NUM: SLIT_NUM(0),	$
		XSTEP: XSTEP(0),	$
		YSTEP: YSTEP(0),	$
		NX: NX(0),		$
		NY: NY(0)}
	GOTO, FINISH
;
;  Error handling point.
;
HANDLE_ERROR:
	IF N_ELEMENTS(ERRMSG) NE 0 THEN ERRMSG = 'GET_F_RASTER: ' + MESSAGE $
		ELSE MESSAGE, MESSAGE, /CONTINUE
;
;  Close the database, and return.
;
FINISH:
	DBCLOSE
;
	RETURN
	END
