	PRO GET_F_STUDY, INPUT, DEF, USE_TITLE=USE_TITLE, ERRMSG=ERRMSG
;+
; Project     :	SOHO - CDS
;
; Name        :	GET_F_STUDY
;
; Purpose     :	Extracts a fundamental study definition from the database
;
; Explanation :	This routine extracts the fundamental parameters of a study
;		from the database for the requested study ID number.
;
; Use         :	GET_F_STUDY, STUDY_ID, DEF
;		GET_F_STUDY, TITLE_ID, DEF, /USE_TITLE
;		GET_F_STUDY, OBS_PROG, DEF
;
; Inputs      :	STUDY_ID = Study ID number, or
;		TITLE_ID = Title ID number, or
;		OBS_PROG = Name of the study
;
; Opt. Inputs :	None.
;
; Outputs     :	DEF = Anonymous structure variable containing the fundamental
;		      parameters of a study definition.  It contains the
;		      following tags:
;
;			STUDY_ID    = Study ID number.  If the requested study
;				      is not found, a simpler structure is
;				      returned with this set to -1.
;			TITLE_ID    = Title ID number.
;			OBS_PROG    = Name of the study
;			TITLE	    = A descriptive title for the study
;			CATEGORY    = The study category, either "T"est,
;				      "S"cience or "C"alibration.
;			VAR_POINT   = Either "Y" or "N" representing whether or
;				      not the pointing can be controlled by the
;				      user.  If "Y" then the pointing is
;				      defined in the plan.  If "N" then the
;				      pointing is embedded within the study.
;			N_RASTER_DEF= Number of rasters in the definition.
;			RASTERS	    = A array listing the rasters to be used
;				      during the study.
;
;		      The raster descriptions themselves are structures, of
;		      type "cds_f_st_ras", with the following tags:
;
;			RAS_ID	    = Raster ID number, pointing to the
;				      definition of the raster.
;			POINTING    = How to handle the pointing.  Valid
;				      entries are:
;
;					 1 = No pointing, supply in plan
;					 0 = Absolute pointing
;					-1 = Offset value from first raster
;
;			INS_X	    = Together with INS_Y, the pointing to use
;				      when user-supplied values are not
;				      allowed.  Only valid when POINTING=0
;				      (absolute) or POINTING=-1 (relative to
;				      1st raster).
;			INS_Y	    = ...
;
; Opt. Outputs:	None.
;
; Keywords    :
;	USE_TITLE = If set, then the input parameter is the title ID rather
;		    than the study ID.  Not required if OBS_PROG is passed.
;
;       ERRMSG    = If defined and passed, then any error messages will be
;                   returned to the user in this parameter rather than
;                   depending on the MESSAGE routine in IDL.  If no errors are
;                   encountered, then a null string is returned.  In order to
;                   use this feature, ERRMSG must be defined first, e.g.
;
;                       ERRMSG = ''
;                       GET_F_STUDY, ERRMSG=ERRMSG, ...
;                       IF ERRMSG NE '' THEN ...
;
;
; Calls       :	DATATYPE, DBOPEN, DBFIND, DBEXT, DBCLOSE
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	If the input parameter is the STUDY_ID, then the TITLE_ID,
;		OBS_PROG, TITLE, and CATEGORY fields will be for one and only
;		one of the entries in the study_title database that matches
;		that STUDY_ID.
;
; Category    :	Planning, Databases.
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 28 April 1994
;
; Modified    :	Version 1, William Thompson, GSFC, 26 July 1994
;               Version 2, Liyun Wang, GSFC/ARC, September 22, 1994
;                  Added the keyword ERRMSG.
;		Version 3, William Thompson, GSFC, 21 November 1994
;			Removed item N_RASTERS0
;			Added item N_RASTER_DEF
;		Version 4, William Thompson, GSFC, 29 December 1994
;			Fixed bug when inputs are bytes.
;		Version 5, William Thompson, GSFC, 10 February 1995
;			Changed N_POINTINGS to VAR_POINT.
;			Removed VAR_RASTERS.
;		Version 6, William Thompson, GSFC, 28 April 1995
;			Returns an error string when no entry is found.
;		Version 7, William Thompson, GSFC, 12 May 1995
;			Modified to split off separate study_title database
;		Version 8, William Thompson, GSFC, 17-Mar-1997
;			Fixed bug involving finding multiple entries if only
;			study ID was passed.
;
; Version     :	Version 8, 17-Mar-1997
;-
;
	ON_ERROR, 2
;
;  Define a null result.  If the routine is successful, this will be updated
;  later.
;
	DEF = {STUDY_ID: -1}
;
;  Check the number of parameters.
;
	IF N_PARAMS() NE 2 THEN BEGIN
           MESSAGE = 'Syntax:  GET_F_STUDY, ID, DEF; or ' +	$
                   'GET_F_STUDY, OBS_PROG, DEF'
	   GOTO, HANDLE_ERROR
	ENDIF
;
;  Check the input parameter.
;
	IF N_ELEMENTS(INPUT) NE 1 THEN BEGIN
           MESSAGE = 'Input parameter must be a scalar'
           GOTO, HANDLE_ERROR
	END ELSE IF DATATYPE(INPUT,1) EQ 'Structure' THEN BEGIN
           MESSAGE = 'Input parameter must not be a structure'
           GOTO, HANDLE_ERROR
	END ELSE IF DATATYPE(INPUT,1) EQ 'Complex' THEN BEGIN
           MESSAGE = 'Input parameter must not be complex'
           GOTO, HANDLE_ERROR
	ENDIF
;
;  Open the primary databases.
;
	DBOPEN, 'study_title,study'
;
;  If the first input parameter is a character string, then search on the
;  OBS_PROG field.
;
	TYPE = DATATYPE(INPUT,1)
	IF TYPE EQ 'String' THEN BEGIN
		ENTRIES = DBFIND('OBS_PROG='+INPUT, /SILENT, /FULLSTRING)
;
;  Otherwise, if the USE_TITLE keyword was set, then search on the TITLE_ID.
;
	END ELSE IF KEYWORD_SET(USE_TITLE) THEN BEGIN
		ENTRIES = DBFIND('TITLE_ID='+STRTRIM(LONG(INPUT),2), /SILENT)
;
;  Otherwise, search on the STUDY_ID field.
;
	END ELSE BEGIN
		ENTRIES = DBFIND('STUDY_ID='+STRTRIM(LONG(INPUT),2), /SILENT)
	ENDELSE
;
;  If no entries were found, then return immediately.
;
	IF !ERR EQ 0 THEN BEGIN
		MESSAGE = 'Study ID ' + TRIM(LONG(INPUT)) + ' not found'
		GOTO, HANDLE_ERROR
	ENDIF
;
;  Extract the relevant parameters from the database.
;
	ENTRIES = ENTRIES(0)
	DBEXT, ENTRIES, 'study_id,title_id,obs_prog,title,category,' +	$
		'var_point,n_raster_def', STUDY_ID, TITLE_ID, OBS_PROG,	$
		TITLE, CATEGORY, VAR_POINT, N_RASTER_DEF
;
;  Now open the raster databases and extract the remaining parameters.
;
	DBOPEN, 'study2,raster'
	ENTRIES = DBFIND('STUDY_ID='+STRTRIM(STUDY_ID,2), /SILENT)
	DBEXT, ENTRIES, 'ras_id,pointing,ins_x,ins_y', RAS_ID, POINTING, $
		INS_X, INS_Y
;
;  Define the structure for the raster definitions.
;
	RASTERS = {CDS_F_ST_RAS,	$
		RAS_ID: 0,		$
		POINTING: 0,		$
		INS_X: 0.0,		$
		INS_Y: 0.0}
;
	IF N_ELEMENTS(ENTRIES) EQ 1 THEN BEGIN
		RASTERS.RAS_ID   = RAS_ID(0)
		RASTERS.POINTING = POINTING(0)
		RASTERS.INS_X	 = INS_X(0)
		RASTERS.INS_Y	 = INS_Y(0)
	END ELSE BEGIN
		RASTERS = REPLICATE(RASTERS, N_ELEMENTS(ENTRIES))
		RASTERS.RAS_ID   = RAS_ID
		RASTERS.POINTING = POINTING
		RASTERS.INS_X	 = INS_X
		RASTERS.INS_Y	 = INS_Y
	ENDELSE
;
;  Define the output structure as a whole.
;
	DEF = { STUDY_ID: STUDY_ID(0),		$
		TITLE_ID: TITLE_ID(0),		$
		OBS_PROG: OBS_PROG(0),		$
		TITLE: TITLE(0),		$
		CATEGORY: CATEGORY(0),		$
		VAR_POINT: VAR_POINT(0),	$
		N_RASTER_DEF: N_RASTER_DEF(0),	$
		RASTERS: RASTERS}
	GOTO, FINISH
;
;  Error handling point
;
HANDLE_ERROR:
	IF N_ELEMENTS(ERRMSG) NE 0 THEN ERRMSG = 'GET_F_STUDY: ' + MESSAGE $
		ELSE MESSAGE, MESSAGE, /CONTINUE
;
;  Close the database, and return.
;
FINISH:
	DBCLOSE
;
	RETURN
	END
