	PRO GET_FLAG, DATE, STATUS, ERRMSG=ERRMSG
;+
; Project     :	SOHO - CDS
;
; Name        :	GET_FLAG
;
; Purpose     :	Extracts a CDS flag receiver status from the database
;
; Explanation :	This routine extracts parameters which describe the CDS flag
;		receiver science plan status at a particular point in time.
;
; Use         :	GET_FLAG, DATE, STATUS
;
; Inputs      :	DATE	= The date/time value that the user wishes the flag
;			  receiver status for.  This can be in any of the
;			  standard CDS time formats.
;
; Opt. Inputs :	None.
;
; Outputs     :	STATUS	= Structure containing the flag receiver status.  It
;			  contains the following tags:
;
;			STRUCT_TYPE= The character string 'CDS-FLAG'
;			PROG_ID	   = Program ID, linking one or more studies
;				     together
;			STUDY_ID   = Number defining the study
;			STUDYVAR   = Number giving the study variation ID
;			SCI_OBJ    = Science objective from the daily science
;				     meeting
;			SCI_SPEC   = Specific science objective from meeting
;			CMP_NO	   = Campaign number
;			OBJECT	   = Code for object planned to be observed
;			OBJ_ID	   = Object identification
;			RCVR_START = Date/time of beginning of receiver status,
;				     in TAI format
;			RCVR_STOP  = Date/time of end of receiver status, in
;				     TAI format
;			ORIG_DUR   = Original duration of the observation, in
;				     seconds, before being truncated by
;				     following time-tagged observations.
;			N_RASTERS1 = Variable number of rasters parameter
;			TRACKING   = True (1) if feature tracking to be used,
;				     or false (0) otherwise
;			N_POINTINGS= Number of pointings to apply to the study.
;				     Can be either 0 or 1.
;			REPOINT	   = Either 1 to allow repoints or 0 to
;				     disallow.
;			GSET_ID	   = GSET ID number
;			POINTINGS  = An array of pointings to use with the
;				     study.
;
;		If no entry is found, then a simpler structure is returned with
;		STUDY_ID set to -1.  POINTINGS is itself a structure of type
;		PLAN_PNT_STRUC with the following tags:
;
;			INS_X	   = Pointing in X relative to flag position
;			INS_Y	   = Pointing in Y relative to flag position
;			ZONE_ID	   = Pointing zone ID
;
; Opt. Outputs:	None.
;
; Keywords    :
;       ERRMSG    = If defined and passed, then any error messages will be
;                   returned to the user in this parameter rather than
;                   depending on the MESSAGE routine in IDL.  If no errors are
;                   encountered, then a null string is returned.  In order to
;                   use this feature, ERRMSG must be defined first, e.g.
;
;                       ERRMSG = ''
;                       GET_FLAG, ERRMSG=ERRMSG, ... 
;                       IF ERRMSG NE '' THEN ...
;
;
; Calls       :	DATATYPE, DBOPEN, DBFIND, DBEXT, DBCLOSE, TRIM
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Category    :	Planning, Databases.
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 25 July 1994
;
; Modified    :	Version 1, William Thompson, GSFC, 25 July 1994
;               Version 2, Liyun Wang, GSFC/ARC, September 22, 1994
;                  Added the keyword ERRMSG.
;		Version 3, William Thompson, GSFC, 17 November 1994
;			Fixed so that if time is on boundary, then the plan
;			which starts then is picked.
;		Version 4, William Thompson, GSFC, 26 January 1995
;			Added tag STRUCT_TYPE
;		Version 5, William Thompson, GSFC, 13 February 1995
;			Changed the way that pointing is treated.
;		Version 6, William Thompson, GSFC, 28 April 1995
;			Returns an error string when no entry is found.
;		Version 7, William Thompson, GSFC, 8 May 1995
;			Modified to pay attention to DELETED field in database
;		Version 8, William Thompson, GSFC, 22 May 1995
;			Changed way DBFIND is called, to speed up.
;		Version 9, William Thompson, GSFC, 5 July 1995
;			Added tag REPOINT.
;		Version 10, William Thompson, GSFC, 6 October 1995
;			Added tags ORIG_DUR, GSET_ID
;			Removed tag N_REPEAT_S
;
; Version     :	Version 10, 6 October 1995
;-
;
	ON_ERROR, 2
;
;  Initialize the output structure, in case any errors occur.
;
	STATUS = {STUDY_ID: -1}
;
;  Check the number of parameters.
;
        IF N_PARAMS() NE 2 THEN BEGIN
           MESSAGE = 'Syntax:  GET_FLAG, DATE, STATUS'
	   GOTO, HANDLE_ERROR
        ENDIF
;
;  Check the input parameter.
;
	IF N_ELEMENTS(DATE) NE 1 THEN BEGIN
           MESSAGE = 'DATE must be a scalar'
           GOTO, HANDLE_ERROR
	ENDIF
;
;  Convert the date to TAI format.
;
	IF DATATYPE(DATE,1) EQ 'Double' THEN TAI = DATE ELSE	$
		TAI = UTC2TAI(DATE)
;
;  Open the database.
;
	DBOPEN, 'sci_flag'
;
;  Search for entries with dates spanning that given by the user.
;
	ENTRIES = DBFIND('RCVR_STOP>' + TRIM(TAI,'(F15.3)') + ',RCVR_START<' +$
		TRIM(TAI,'(F15.3)') + ',DELETED=N', /SILENT)
;
;  If no entries were found, then return immediately.
;
	IF !ERR EQ 0 THEN BEGIN
		MESSAGE = 'Flag receiver entry not found'
		GOTO, HANDLE_ERROR
	ENDIF
;
;  Extract the entry.
;
	DBEXT, ENTRIES, 'prog_id,study_id,studyvar,sci_obj,sci_spec,cmp_no', $
		PROG_ID, STUDY_ID, STUDYVAR, SCI_OBJ, SCI_SPEC, CMP_NO
	DBEXT, ENTRIES, 'object,obj_id,rcvr_start,rcvr_stop,n_rasters1', $
		OBJECT, OBJ_ID, RCVR_START, RCVR_STOP, N_RASTERS1
	DBEXT, ENTRIES, 'tracking,n_pointings,orig_dur,gset_id,repoint', $
		TRACKING, N_POINTINGS, ORIG_DUR, GSET_ID, REPOINT
;
;  If more than one entry has been found, then use the one with the closest
;  start time to the requested time.  This avoids problems with times which are
;  right on the boundaries between entries.
;
	DIFF = ABS(RCVR_START - TAI)
	I = (WHERE(DIFF EQ MIN(DIFF)))(0)
;
;  Define the structure to be used with the pointings.
;
	POINTINGS = {PLAN_PNT_STRUC, INS_X: 0.0, INS_Y: 0.0, ZONE_ID: 0}
;
;  Extract the pointings associated with this entry.
;
	N_POINTINGS = N_POINTINGS(I)
	IF N_POINTINGS GT 0 THEN BEGIN
		DBOPEN, 'point_flag'
		ENTRIES = DBFIND('RCVR_START=' + TRIM(RCVR_START(I),	$
			'(F15.3)') + ',DELETED=N', /SILENT)
		DBEXT, ENTRIES, 'pt_ind,ins_x,ins_y,zone_id',	$
			PT_IND, INS_X, INS_Y, ZONE_ID
;
;  Put the data into the structure.
;
		IF N_POINTINGS EQ 1 THEN BEGIN
			POINTINGS.INS_X   = INS_X(0)
			POINTINGS.INS_Y   = INS_Y(0)
			POINTINGS.ZONE_ID = ZONE_ID(0)
		END ELSE BEGIN
			POINTINGS = REPLICATE(POINTINGS, N_POINTINGS)
			S = SORT(PT_IND)
			POINTINGS.INS_X   = INS_X(S)
			POINTINGS.INS_Y   = INS_Y(S)
			POINTINGS.ZONE_ID = ZONE_ID(S)
		ENDELSE
	ENDIF
;
;  Define the output structure as a whole.
;
	STATUS = {STRUCT_TYPE: 'CDS-FLAG',	$
		PROG_ID: PROG_ID(I),		$
		STUDY_ID: STUDY_ID(I),		$
		STUDYVAR: STUDYVAR(I),		$
		SCI_OBJ: SCI_OBJ(I),		$
		SCI_SPEC: SCI_SPEC(I),		$
		CMP_NO: CMP_NO(I),		$
		OBJECT: OBJECT(I),		$
		OBJ_ID: OBJ_ID(I),		$
		RCVR_START: RCVR_START(I),	$
		RCVR_STOP: RCVR_STOP(I),	$
		ORIG_DUR: ORIG_DUR(I),		$
		N_RASTERS1: N_RASTERS1(I),	$
		TRACKING: TRACKING(I),		$
		N_POINTINGS: N_POINTINGS,	$
		REPOINT: REPOINT(I),		$
		GSET_ID: GSET_ID(I),		$
		POINTINGS: POINTINGS}
	GOTO, FINISH
;
;  Error handling point.
;
HANDLE_ERROR:
	IF N_ELEMENTS(ERRMSG) NE 0 THEN ERRMSG = 'GET_FLAG: ' + MESSAGE $
		ELSE MESSAGE, MESSAGE, /CONTINUE
;
;  Close the database, and return.
;
FINISH:
	DBCLOSE
;
	RETURN
	END
