	PRO GET_GSET, GSET_ID, DEF, ERRMSG=ERRMSG
;+
; Project     :	SOHO - CDS
;
; Name        :	GET_GSET
;
; Purpose     :	Extracts a GIS raw data file entry from the database
;
; Explanation :	This routine extracts a GIS raw data file entry from the
;		database for a requested raw data file ID number.
;
; Use         :	GET_GSET, GSET_ID, DEF
;
; Inputs      :	GSET_ID = GIS setup ID number
;
; Opt. Inputs :	None.
;
; Outputs     :	DEF	= Structure containing the GIS setup entry.  It
;			  contains the following tags:
;
;			GSET_ID	   = GIS setup ID number.  If the requested
;				     GIS setup entry is not found, then a
;				     simpler structure is returned, with this
;				     set to -1.
;			HVOLT	   = High voltages (4)
;			LLD	   = Low level discriminators (4)
;			LUT_CHKSUM = LUT checksum in hexidecimal notation (4)
;			LUT_PAR	   = LUT parameters (4,11)
;			RAW_ID	   = RAW GIS science data file ID (4)
;			SLIT_NUM   = Slit number
;			ZONE_ID	   = Zone ID number
;			FIL_ID	   = Filament ID number, or 0 for science data
;			FFB	   = Front face bias, either -1 or +1
;			DET_USED   = Whether or not detector is used, either 0
;				     or 1 (4)
;			GSET_DESC  = Free format descriptions (4)
;
; Opt. Outputs:	None.
;
; Keywords    :
;       ERRMSG    = If defined and passed, then any error messages will be
;                   returned to the user in this parameter rather than
;                   depending on the MESSAGE routine in IDL.  If no errors are
;                   encountered, then a null string is returned.  In order to
;                   use this feature, ERRMSG must be defined first, e.g.
;
;                       ERRMSG = ''
;                       GET_GSET, ERRMSG=ERRMSG, ... 
;                       IF ERRMSG NE '' THEN ...
;
;
; Calls       :	DATATYPE, DBOPEN, DBFIND, DBEXT, DBCLOSE
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Category    :	Planning, Databases.
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 7 September 1995
;
; Modified    :	Version 1, William Thompson, GSFC, 7 September 1995
;		Version 2, William Thompson, GSFC, 14 September 1995
;			Replace FIL_CUR with FFB
;		Version 3, William Thompson, GSFC, 28 September 1995
;			Make LUT_CHKSUM an array.
;		Version 4, William Thompson, GSFC, 4 October 1995
;			Modified to reflect change in way LUT_PAR, LUT_CHKSUM,
;			and GSET_DESC are stored.
;		Version 5, William Thompson, GSFC, 25 January 1996
;			Modified definition of LUT_PAR and added DET_USED.
;
; Version     :	Version 5, 25 January 1996
;-
;
	ON_ERROR, 2
;
;  Define a null result.  If the routine is successful, this will be updated
;  later.
;
	DEF = {GSET_ID: -1}
;
;  Check the number of parameters.
;
        IF N_PARAMS() NE 2 THEN BEGIN
           MESSAGE = 'Syntax:  GET_GSET, GSET_ID, DEF'
	   GOTO, HANDLE_ERROR
        ENDIF
;
;  Check the input parameter GSET_ID.
;
	TYPE = DATATYPE(GSET_ID,2)
	IF TYPE EQ 0 THEN BEGIN
           MESSAGE = 'GSET_ID is undefined'
           GOTO, HANDLE_ERROR
	END ELSE IF TYPE GE 4 THEN BEGIN
           MESSAGE = 'GSET_ID must be an integer'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(GSET_ID) NE 1 THEN BEGIN
           MESSAGE = 'GSET_ID must be a scalar'
           GOTO, HANDLE_ERROR
	ENDIF
;
;  Open the database.
;
	DBOPEN, 'gset'
;
;  Search on the GSET_ID field.
;
	ENTRIES = DBFIND('GSET_ID='+STRTRIM(LONG(GSET_ID),2), /SILENT)
;
;  If no entries were found, then return immediately.
;
	IF !ERR EQ 0 THEN BEGIN
		MESSAGE = 'GIS setup ID ' + TRIM(LONG(GSET_ID)) + ' not found'
		GOTO, HANDLE_ERROR
	ENDIF
;
;  Extract the data from the GIS setup database.
;
	DBEXT, ENTRIES, 'hvolt,lld,lut_chksum,lut_par,raw_id,slit_num,' + $
		'zone_id,fil_id,ffb,det_used,gset_desc', HVOLT, LLD, CHKSUM, $
		LUT_PAR, RAW_ID, SLIT_NUM, ZONE_ID, FIL_ID, FFB, DET_USED, DESC
;
;  Rearrange the checksum and description each into a series of lines.
;
	LUT_CHKSUM = STRARR(4)
	FOR I = 0,3 DO LUT_CHKSUM(I) = STRTRIM(STRMID(CHKSUM,4*I,4))
	GSET_DESC = STRARR(4)
	FOR I = 0,3 DO GSET_DESC(I) = STRTRIM(STRMID(DESC,80*I,80))
;
;  Define the output structure.  Note that LUT_PAR must be reformatted because
;  the UIT database software cannot handle multidimensional arrays.
;
	DEF = { GSET_ID: GSET_ID(0),			$
		HVOLT: HVOLT(*,0),			$
		LLD: LLD(*,0),				$
		LUT_CHKSUM: LUT_CHKSUM(*,0),		$
		LUT_PAR: REFORM(LUT_PAR(*,0),4,11),	$
		RAW_ID: RAW_ID(*,0),			$
		SLIT_NUM: SLIT_NUM(0),			$
		ZONE_ID: ZONE_ID(0),			$
		FIL_ID: FIL_ID(0),			$
		FFB: FFB(0),				$
		DET_USED: DET_USED(*,0),		$
		GSET_DESC: GSET_DESC(*,0)}
	GOTO, FINISH
;
;  Error handling point.
;
HANDLE_ERROR:
	IF N_ELEMENTS(ERRMSG) NE 0 THEN ERRMSG = 'GET_GSET: ' + MESSAGE $
		ELSE MESSAGE, MESSAGE, /CONTINUE
;
;  Close the database, and return.
;
FINISH:
	DBCLOSE
;
	RETURN
	END
