	PRO GET_LINELIST, LL_ID, DEF, ERRMSG=ERRMSG
;+
; Project     :	SOHO - CDS
;
; Name        :	GET_LINELIST
;
; Purpose     :	Extracts a line list definition from the database
;
; Explanation :	This routine extracts a complete line list definition from the
;		database for the requested line list.
;
; Use         :	GET_LINELIST, LL_ID, DEF
;
; Inputs      :	LL_ID	= Line list ID number
;
; Opt. Inputs :	None.
;
; Outputs     :	DEF	= Structure containing the line list definition.  It
;			  contains the following tags:
;
;			LL_ID	 = Line list ID number.  If the requested line
;				   list is not found, then a simpler structure
;				   is returned, with this set to -1.
;			DETECTOR = Either "G" for GIS or "N" for NIS.
;			LL_DESC	 = A description of the line list, e.g.
;				   "Temperature sensitive line pairs".
;			LINES	 = An array containing the line definitions.
;
;		      The line definitions themselves are structures, of type
;		      "cds_lines", with the following tags:
;
;			LINENAME = Line name, e.g. "He II 304".  Can be up to
;				   12 characters.
;			WAVELNTH = The wavelength of the line, in Angstroms.
;			GR_ORDER = The grating order.
;			CEN_PIX	 = The central detector pixel.
;			WAVEBAND = The wavelength band, 1 or 2 for NIS, 1-4 for
;				   GIS.
;
; Opt. Outputs:	None.
;
; Keywords    :
;       ERRMSG    = If defined and passed, then any error messages will be
;                   returned to the user in this parameter rather than
;                   depending on the MESSAGE routine in IDL.  If no errors are
;                   encountered, then a null string is returned.  In order to
;                   use this feature, ERRMSG must be defined first, e.g.
;
;                       ERRMSG = ''
;                       GET_LINELIST, ERRMSG=ERRMSG, ...
;                       IF ERRMSG NE '' THEN ...
;
;
; Calls       :	DATATYPE, DBOPEN, DBFIND, DBEXT, DBCLOSE
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Category    :	Planning, Databases.
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 31 March 1994
;
; Modified    :	Version 1, William Thompson, GSFC, 26 July 1994
;               Version 2, Liyun Wang, GSFC/ARC, September 22, 1994
;                  Added the keyword ERRMSG.
;		Version 3, William Thompson, GSFC, 29 December 1994
;			Fixed bug when inputs are bytes.
;		Version 4, William Thompson, GSFC, 15 March 1995
;			Removed LL_TITLE
;		Version 5, William Thompson, GSFC, 28 April 1995
;			Returns an error string when no entry is found.
;
; Version     :	Version 5, 28 April 1995
;-
;
	ON_ERROR, 2
;
;  Define a null result.  If the routine is successful, this will be updated
;  later.
;
	DEF = {LL_ID: -1}
;
;  Check the number of parameters.
;
        IF N_PARAMS() NE 2 THEN BEGIN
           MESSAGE = 'Syntax:  GET_LINELIST, LL_ID, DEF'
	   GOTO, HANDLE_ERROR
        ENDIF
;
;  Check the input parameter.
;
	TYPE = DATATYPE(LL_ID,2)
	IF TYPE EQ 0 THEN BEGIN
           MESSAGE = 'LL_ID is undefined'
           GOTO, HANDLE_ERROR
	END ELSE IF TYPE GE 4 THEN BEGIN
           MESSAGE = 'LL_ID must be an integer'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(LL_ID) NE 1 THEN BEGIN
           MESSAGE = 'LL_ID must be a scalar'
           GOTO, HANDLE_ERROR
	ENDIF
;
;  Open the database, and search on the LL_ID field.
;
	DBOPEN, 'linelist2,linelist'
	ENTRIES = DBFIND('LL_ID='+STRTRIM(LONG(LL_ID),2), /SILENT)
;
;  If no entries were found, then return immediately.
;
	IF !ERR EQ 0 THEN BEGIN
		MESSAGE = 'Line list ID ' + TRIM(LONG(LL_ID)) + ' not found'
		GOTO, HANDLE_ERROR
	ENDIF
;
;  Extract the relevant entries from the database.
;
	DBEXT, ENTRIES, 'linename,wavelnth,gr_order,cen_pix,waveband,' + $
		'detector,ll_desc', LINENAME, WAVELNTH, GR_ORDER, CEN_PIX, $
		WAVEBAND, DETECTOR, LL_DESC
;
;  Define the structure for the line definitions.
;
	LINES = {CDS_LINES,		$
		LINENAME: 'String',	$
		WAVELNTH: 1.0,		$
		GR_ORDER: 1,		$
		CEN_PIX: 1,		$
		WAVEBAND: 1}
;
	IF N_ELEMENTS(ENTRIES) EQ 1 THEN BEGIN
		LINES.LINENAME = LINENAME(0)
		LINES.WAVELNTH = WAVELNTH(0)
		LINES.GR_ORDER = GR_ORDER(0)
		LINES.CEN_PIX  = CEN_PIX(0)
		LINES.WAVEBAND = WAVEBAND(0)
	END ELSE BEGIN
		LINES = REPLICATE(LINES, N_ELEMENTS(ENTRIES))
		LINES.LINENAME = LINENAME
		LINES.WAVELNTH = WAVELNTH
		LINES.GR_ORDER = GR_ORDER
		LINES.CEN_PIX  = CEN_PIX
		LINES.WAVEBAND = WAVEBAND
	ENDELSE
;
;  Define the output structure as a whole.
;
	DEF = {LL_ID: LL_ID(0),		$
		DETECTOR: DETECTOR(0),	$
		LL_DESC: LL_DESC(0),	$
		LINES: LINES}
	GOTO, FINISH
;
;  Error handling point.
;
HANDLE_ERROR:
	IF N_ELEMENTS(ERRMSG) NE 0 THEN ERRMSG = 'GET_LINELIST: ' + MESSAGE $
		ELSE MESSAGE, MESSAGE, /CONTINUE
;
;  Close the database, and return.
;
FINISH:
	DBCLOSE
;
	RETURN
	END
