	PRO GET_MAIN, PROG_NUM, OBS, ERRMSG=ERRMSG
;+
; Project     :	SOHO - CDS
;
; Name        :	GET_MAIN
;
; Purpose     :	Extracts a CDS main entry from the as-run database
;
; Explanation :	This routine extracts parameters which describe the CDS as-run
;		database at the study level for a given PROG_NUM (study
;		counter) value.
;
; Use         :	GET_MAIN, PROG_NUM, OBS
;
; Inputs      :	PROG_NUM = The program number (study counter) which serves as
;			   the primary index into the MAIN database.
;
; Opt. Inputs :	None.
;
; Outputs     :	OBS	= Structure containing the detailed science plan
;			  record.  It contains the following tags:
;
;			STRUCT_TYPE= The character string 'CDS-MAIN'
;			PROG_NUM   = Study counter number.  This forms the
;				     primary key for this database.
;			PROG_ID	   = Program ID, linking one or more studies
;				     together
;			PROG_IND   = An index, reflecting one's position within
;				     a repeated sequence of studies via the
;				     N_REPEAT_S mechanism.
;			STUDY_ID   = Number defining the study
;			STUDYVAR   = Number giving the study variation ID
;			OBS_PROG   = Name of the study
;			DETECTOR   = Detector, "N", "G", or "B"(oth).
;			SCI_OBJ    = Science objective from the daily science
;				     meeting
;			SCI_SPEC   = Specific science objective from meeting
;			CMP_NO	   = Campaign number
;			OBJECT	   = Code for object planned to be observed
;			OBJ_ID	   = Object identification
;			DATE_OBS   = Date/time of beginning of observation,
;				     in TAI format
;			DATE_END   = Date/time of end of observation, in TAI
;				     format
;			OBT_TIME   = Onboard date/time of beginning of
;				     observation, in TAI format
;			OBT_END	   = Onboard date/time of end of observation,
;				     in TAI format
;			XCEN	   = Center of instrument field-of-view in
;				     solar X
;			YCEN	   = Center of instrument field-of-view in
;				     solar Y
;			ANGLE	   = Angle of instrument relative to solar
;				     north
;			IXWIDTH	   = Width of field-of-view in instrument X
;			IYWIDTH	   = Width of field-of-view in instrument Y
;			SEQ_FROM   = Start of range of observing sequence
;				     numbers.
;			SEQ_TO	   = End of range of observing sequence
;				     numbers.
;			COMMENTS   = Either Y or N
;			DATE_MOD   = Date and time when catalog record was last
;				     modified.
;
;		If no entry is found, then a simpler structure is returned with
;		PROG_NUM and STUDY_ID both set to -1.
;
; Opt. Outputs:	None.
;
; Keywords    :
;       ERRMSG    = If defined and passed, then any error messages will be
;                   returned to the user in this parameter rather than
;                   depending on the MESSAGE routine in IDL.  If no errors are
;                   encountered, then a null string is returned.  In order to
;                   use this feature, ERRMSG must be defined first, e.g.
;
;                       ERRMSG = ''
;                       GET_MAIN, ERRMSG=ERRMSG, ... 
;                       IF ERRMSG NE '' THEN ...
;
;
; Calls       :	DATATYPE, DBOPEN, DBFIND, DBEXT, DBCLOSE, TRIM
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Category    :	Planning, Databases.
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 30 June 1995
;
; Modified    :	Version 1, William Thompson, GSFC, 30 June 1995
;		Version 2, William Thompson, GSFC, 2 April 1996
;			Modified for version 3 of the catalog.
;		Version 3, William Thompson, GSFC, 6 March 1998
;			Fixed bug where PROG_NUM converted to 1-element array
;
; Version     :	Version 3, 6 March 1998
;-
;
	ON_ERROR, 2
;
;  Initialize the output structure, in case any errors occur.
;
	OBS = {PROG_NUM: -1, STUDY_ID: -1}
;
;  Check the number of parameters.
;
        IF N_PARAMS() NE 2 THEN BEGIN
           MESSAGE = 'Syntax:  GET_MAIN, PROG_NUM, OBS'
	   GOTO, HANDLE_ERROR
        ENDIF
;
;  Check the input parameter.
;
	IF N_ELEMENTS(PROG_NUM) NE 1 THEN BEGIN
           MESSAGE = 'PROG_NUM must be a scalar'
           GOTO, HANDLE_ERROR
	END ELSE IF DATATYPE(PROG_NUM,2) GT 3 THEN BEGIN
	   MESSAGE = 'PROG_NUM must be an integer value'
	   GOTO, HANDLE_ERROR
	ENDIF
;
;  Open the database.
;
	DBOPEN, 'main'
;
;  Search for the requested entry.
;
	ENTRY = DBFIND('PROG_NUM=' + TRIM(PROG_NUM(0)), /SILENT)
;
;  If no entries were found, then return immediately.
;
	IF !ERR EQ 0 THEN BEGIN
		MESSAGE = 'No main as-run entry found meeting search criteria'
		GOTO, HANDLE_ERROR
	ENDIF
;
;  Extract the entry.
;
	DBEXT, ENTRY(0), 'prog_num,prog_id,prog_ind,study_id,studyvar,' + $
		'sci_obj,sci_spec,cmp_no,object,obj_id,date_obs,date_end', $
		PROG_NUM0, PROG_ID, PROG_IND, STUDY_ID, STUDYVAR, SCI_OBJ, $
		SCI_SPEC, CMP_NO, OBJECT, OBJ_ID, DATE_OBS, DATE_END
	DBEXT, ENTRY(0), 'obt_time,obt_end,xcen,ycen,angle,ixwidth,' +	$
		'iywidth,seq_from,seq_to,comments,date_mod', OBT_TIME,	$
		OBT_END, XCEN, YCEN, ANGLE, IXWIDTH, IYWIDTH, SEQ_FROM,	$
		SEQ_TO, COMMENTS, DATE_MOD
	DBEXT, ENTRY(0), 'obs_prog,detector', OBS_PROG, DETECTOR
;
;  Define the output structure.
;
	OBS = { STRUCT_TYPE: 'CDS-MAIN',$
		PROG_NUM: PROG_NUM0(0),	$
		PROG_ID: PROG_ID(0),	$
		PROG_IND: PROG_IND(0),	$
		STUDY_ID: STUDY_ID(0),	$
		STUDYVAR: STUDYVAR(0),	$
		OBS_PROG: OBS_PROG(0),	$
		DETECTOR: DETECTOR(0),	$
		SCI_OBJ: SCI_OBJ(0),	$
		SCI_SPEC: SCI_SPEC(0),	$
		CMP_NO: CMP_NO(0),	$
		OBJECT: OBJECT(0),	$
		OBJ_ID: OBJ_ID(0),	$
		DATE_OBS: DATE_OBS(0),	$
		DATE_END: DATE_END(0),	$
		OBT_TIME: OBT_TIME(0),	$
		OBT_END: OBT_END(0),	$
		XCEN: XCEN(0),		$
		YCEN: YCEN(0),		$
		ANGLE: ANGLE(0),	$
		IXWIDTH: IXWIDTH(0),	$
		IYWIDTH: IYWIDTH(0),	$
		SEQ_FROM: SEQ_FROM(0),	$
		SEQ_TO: SEQ_TO(0),	$
		COMMENTS: COMMENTS(0),	$
		DATE_MOD: DATE_MOD(0)}
	GOTO, FINISH
;
;  Error handling point.
;
HANDLE_ERROR:
	IF N_ELEMENTS(ERRMSG) NE 0 THEN ERRMSG = 'GET_MAIN: ' + MESSAGE $
		ELSE MESSAGE, MESSAGE, /CONTINUE
;
;  Close the database, and return.
;
FINISH:
	DBCLOSE
;
	RETURN
	END
