	PRO GET_PROGRAM, PROG_ID, DEF, ERRMSG=ERRMSG
;+
; Project     :	SOHO - CDS
;
; Name        :	GET_PROGRAM
;
; Purpose     :	Extracts a CDS program definition from the database
;
; Explanation :	This routine extracts parameters which defines a CDS
;		multi-observation program.
;
; Use         :	GET_PROGRAM, PROG_ID, DEF
;
; Inputs      :	PROG_ID	= The unique program ID.
;
; Opt. Inputs :	None.
;
; Outputs     :	DEF	= Structure containing the program definition.  It
;			  contains the following tags:
;
;			PROGNAME = Name of the program.
;			PROG_ID	 = Unique identifier number.
;			PROGDESC = Up to five lines of text describing the
;				   program.
;
; Opt. Outputs:	None.
;
; Keywords    :
;       ERRMSG    = If defined and passed, then any error messages will be
;                   returned to the user in this parameter rather than
;                   depending on the MESSAGE routine in IDL.  If no errors are
;                   encountered, then a null string is returned.  In order to
;                   use this feature, ERRMSG must be defined first, e.g.
;
;                       ERRMSG = ''
;                       GET_PROGRAM, ERRMSG=ERRMSG, ... 
;                       IF ERRMSG NE '' THEN ...
;
;
; Calls       :	DATATYPE, DBOPEN, DBFIND, DBEXT, DBCLOSE
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Category    :	Planning, Databases.
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 21 July 1994
;
; Modified    :	Version 1, William Thompson, GSFC, 27 July 1994
;               Version 2, Liyun Wang, GSFC/ARC, September 22, 1994
;                  Added the keyword ERRMSG.
;		Version 3, William Thompson, GSFC, 29 December 1994
;			Fixed bug when inputs are bytes.
;		Version 4, William Thompson, GSFC, 28 April 1995
;			Returns an error string when no entry is found.
;
; Version     :	Version 4, 28 April 1995
;-
;
	ON_ERROR, 2
;
;  Define a null result.  If the routine is successful, this will be updated
;  later.
;
	DEF = {PROG_ID: 0}
;
;  Check the number of parameters.
;
        IF N_PARAMS() NE 2 THEN BEGIN
           MESSAGE = 'Syntax:  GET_PROGRAM, PROG_ID, DEF'
	   GOTO, HANDLE_ERROR
        ENDIF
;
;  Check the input parameter PROG_ID.
;
	TYPE = DATATYPE(PROG_ID,2)
	IF TYPE EQ 0 THEN BEGIN
           MESSAGE = 'PROG_ID is undefined'
           GOTO, HANDLE_ERROR
	END ELSE IF TYPE GE 4 THEN BEGIN
           MESSAGE = 'PROG_ID must be an integer'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(PROG_ID) NE 1 THEN BEGIN
           MESSAGE = 'PROG_ID must be a scalar'
           GOTO, HANDLE_ERROR
	ENDIF
;
;  Open the database.
;
	DBOPEN, 'program'
;
;  Search on the PROG_ID field.
;
	ENTRIES = DBFIND('PROG_ID='+STRTRIM(LONG(PROG_ID),2), /SILENT)
;
;  If no entries were found, then return immediately.
;
	IF !ERR EQ 0 THEN BEGIN
		MESSAGE = 'Program ID ' + TRIM(LONG(PROG_ID)) + ' not found'
		GOTO, HANDLE_ERROR
	ENDIF
;
;  Extract the relevant entry from the database.
;
	DBEXT, ENTRIES(0), 'progname,progdesc', PROGNAME, DESC
;
;  Rearrange the program description into a series of lines.
;
	PROGDESC = STRARR(5)
	FOR I = 0,4 DO PROGDESC(I) = STRTRIM(STRMID(DESC,80*I,80))
;
;  Define the output structure.
;
	DEF = {PROGNAME: PROGNAME(0),	$
		PROG_ID: PROG_ID(0),	$
		PROGDESC: PROGDESC}
	GOTO, FINISH
;
;  Error handling point.
;
HANDLE_ERROR:
	IF N_ELEMENTS(ERRMSG) NE 0 THEN ERRMSG = 'GET_PROGRAM: ' + MESSAGE $
		ELSE MESSAGE, MESSAGE, /CONTINUE
;
;  Close the database, and return.
;
FINISH:
	DBCLOSE
;
	RETURN
	END
