	PRO GET_RASTER, RAS_ID, RAS_VAR, DEF, ERRMSG=ERRMSG
;+
; Project     :	SOHO - CDS
;
; Name        :	GET_RASTER
;
; Purpose     :	Extracts a complete raster definition from the database
;
; Explanation :	This routine extracts all the parameters of a raster from the
;		database for the requested raster ID number and variation
;		index.
;
; Use         :	GET_RASTER, RAS_ID, RAS_VAR, DEF
;
; Inputs      :	RAS_ID	= Raster ID number
;		RAS_VAR	= Raster variation index
;
; Opt. Inputs :	None.
;
; Outputs     :	DEF	= Structure of type "cds_raster" containing the
;			  complete raster definition.  It contains the
;			  following tags:
;
;			RAS_ID	   = Raster ID number.  If the requested raster
;				     is not found, then a simpler structure is
;				     returned, with this set to -1.
;			DETECTOR   = Either "G" for GIS or "N" for NIS.
;			RAS_DESC   = A short description of the raster, giving
;				     its purpose.
;			SLIT_NUM   = The ID number for the slit to be used.
;			XSTEP	   = The step size in the X direction, in
;				     arcsec.
;			YSTEP	   = The step size in the Y direction, in
;				     arcsec.
;			NX	   = The number of exposure positions in X.
;			NY	   = The number of exposure positions in Y.
;			RAS_VAR	   = Raster variation index.
;			RV_DESC	   = A short description of the raster
;				     variation beyond what is given in the
;				     associated fundamental raster description.
;			EXPTIME	   = Exposure time in seconds, to millisecond
;				     accuracy.
;			LL_ID	   = Line list ID.  Alternatively, this can be
;				     zero to signal that the raster is not
;				     connected to a line list.
;			LL_DESC	   = A description of the line list, e.g.
;				     "Temperature sensitive line pairs".
;			N_LINES	   = The number of lines.
;			COMP_ID	   = Compression method ID.
;			COMP_OPT   = Compression option parameter.
;			DW_ID	   = Data extraction window list ID.
;			IEF_ID	   = Inter-instrument event flag ID.
;			VDS_ORIENT = VDS orientation, either 0 (row) or 1
;				     (column).
;			VDS_MAP	   = VDS mapping mode: 2=Normal, 3=Accumulate.
;			DW_DESC    = A short description of the data window
;				     list beyond what is given in the
;				     associated line list description, e.g.
;				     "Full slit, 10 pixels wide".
;			VDS_BACK   = Either 0 for off, or 1 for on,
;				     representing whether or not VDS background
;				     windows are being used.  For GIS window
;				     lists this is 0.
;			N_WINDOWS  = The number of windows.
;			N_FLAGS	   = The number of flag windows.
;			TEL_RATE   = Estimated required telemetry rate, as a
;				     character representing Low/Medium/High
;			DURATION   = Estimated duration of the raster, in
;				     seconds.
;			USABLE	   = Either "Y" or "N" to signal whether or not
;				     the raster is usable.  Normally "Y".
;
; Opt. Outputs:	None.
;
; Keywords    :
;       ERRMSG    = If defined and passed, then any error messages will be
;                   returned to the user in this parameter rather than
;                   depending on the MESSAGE routine in IDL.  If no errors are
;                   encountered, then a null string is returned.  In order to
;                   use this feature, ERRMSG must be defined first, e.g.
;
;                       ERRMSG = ''
;                       GET_RASTER, ERRMSG=ERRMSG, ... 
;                       IF ERRMSG NE '' THEN ...
;
;
; Calls       :	DATATYPE, DBOPEN, DBFIND, DBEXT, DBCLOSE
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Category    :	Planning, Databases.
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 5 April 1994
;
; Modified    :	Version 1, William Thompson, GSFC, 1 August 1994
;               Version 2, Liyun Wang, GSFC/ARC, September 22, 1994
;                  Added the keyword ERRMSG.
;		Version 3, William Thompson, GSFC, 29 December 1994
;			Fixed bug when inputs are bytes.
;		Version 4, William Thompson, GSFC, 15 March 1995
;			Removed LL_TITLE
;		Version 5, William Thompson, GSFC, 28 April 1995
;			Returns an error string when no entry is found.
;		Version 6, William Thompson, GSFC, 20 June 1995
;			Added tag N_FLAGS.
;		Version 5, William Thompson, GSFC, 27 October 1995
;			Added item IEF_ID.
;
; Version     :	Version 5, 27 October 1995
;-
;
	ON_ERROR, 2
;
;  Define a null result.  If the routine is successful, this will be updated
;  later.
;
	DEF = {RAS_ID: -1}
;
;  Check the number of parameters.
;
        IF N_PARAMS() NE 3 THEN BEGIN
           MESSAGE = 'Syntax:  GET_RASTER, RAS_ID, RAS_VAR, DEF'
	   GOTO, HANDLE_ERROR
        ENDIF
;
;  Check the input parameters.
;
	TYPE = DATATYPE(RAS_ID,2)
	IF TYPE EQ 0 THEN BEGIN
           MESSAGE = 'RAS_ID is undefined'
           GOTO, HANDLE_ERROR
	END ELSE IF TYPE GE 4 THEN BEGIN
           MESSAGE = 'RAS_ID must be an integer'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(RAS_ID) NE 1 THEN BEGIN
           MESSAGE = 'RAS_ID must be a scalar'
           GOTO, HANDLE_ERROR
	ENDIF
;
	TYPE = DATATYPE(RAS_VAR,2)
	IF TYPE EQ 0 THEN BEGIN
           MESSAGE = 'RAS_VAR is undefined'
           GOTO, HANDLE_ERROR
	END ELSE IF TYPE GE 4 THEN BEGIN
           MESSAGE = 'RAS_VAR must be an integer'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(RAS_VAR) NE 1 THEN BEGIN
           MESSAGE = 'RAS_VAR must be a scalar'
           GOTO, HANDLE_ERROR
	ENDIF
;
;  Open the database.
;
	DBOPEN, 'raster_var,raster'
;
;  Search on the RAS_ID and RAS_VAR fields.
;
	ENTRY = DBFIND('RAS_ID=' + STRTRIM(LONG(RAS_ID),2) +	$
		',RAS_VAR=' + STRTRIM(LONG(RAS_VAR),2), /SILENT)
;
;  If no entries were found, then return immediately.
;
	IF !ERR EQ 0 THEN BEGIN
		MESSAGE = 'Raster ID/variation ' + TRIM(LONG(RAS_ID)) + '/' + $
			TRIM(LONG(RAS_VAR)) + ' not found'
		GOTO, HANDLE_ERROR
	ENDIF
;
;  Extract the relevant parameters from the database.
;
	DBEXT, ENTRY(0), 'detector,ras_desc,slit_num,xstep,ystep,nx,ny', $
		DETECTOR, RAS_DESC, SLIT_NUM, XSTEP, YSTEP, NX, NY
	DBEXT, ENTRY(0), 'rv_desc,exptime,linelist_ptr,comp_id,comp_opt,' + $
		'datawin_ptr,tel_rate,duration,usable', RV_DESC, $
		EXPTIME, LINELIST_PTR, COMP_ID, COMP_OPT, DATAWIN_PTR,	$
		TEL_RATE, DURATION, USABLE
	DBEXT, ENTRY(0), 'ief_id,vds_orient,vds_map', IEF_ID, VDS_ORIENT, $
		VDS_MAP
;
;  Get the information about the line list, if appropriate.
;
	IF LINELIST_PTR(0) NE 0 THEN BEGIN
		DBOPEN, 'linelist'
		DBEXT, LINELIST_PTR, 'll_id,ll_desc,n_lines', $
			LL_ID, LL_DESC, N_LINES
	END ELSE BEGIN
		LL_ID = 0
		LL_DESC = ''
		N_LINES = 0
	ENDELSE
;
;  Get the information about the data extraction window list.
;
	DBOPEN, 'datawin'
	DBEXT, DATAWIN_PTR, 'dw_id,dw_desc,vds_back,n_windows,n_flags',	$
		DW_ID, DW_DESC,	VDS_BACK, N_WINDOWS, N_FLAGS
	DBEXT, DATAWIN_PTR, 'w_width,w_height', W_WIDTH, W_HEIGHT
;
;  Define the output structure.
;
	DEF = {CDS_RASTER,			$
		RAS_ID: RAS_ID,			$
		DETECTOR: DETECTOR(0),		$
		RAS_DESC: RAS_DESC(0),		$
		SLIT_NUM: SLIT_NUM(0),		$
		XSTEP: XSTEP(0),		$
		YSTEP: YSTEP(0),		$
		NX: NX(0),			$
		NY: NY(0),			$
		RAS_VAR: RAS_VAR(0),		$
		RV_DESC: RV_DESC(0),		$
		EXPTIME: EXPTIME(0),		$
		LL_ID: LL_ID(0),		$
		LL_DESC: LL_DESC(0),		$
		N_LINES: N_LINES(0),		$
		COMP_ID: COMP_ID(0),		$
		COMP_OPT: COMP_OPT(0),		$
		DW_ID: DW_ID(0),		$
		DW_DESC: DW_DESC(0),		$
		W_WIDTH: W_WIDTH(0),		$
		W_HEIGHT: W_HEIGHT(0),		$
		VDS_BACK: VDS_BACK(0),		$
		N_WINDOWS: N_WINDOWS(0),	$
		N_FLAGS: N_FLAGS(0),		$
		IEF_ID: IEF_ID(0),		$
		VDS_ORIENT: VDS_ORIENT(0),	$
		VDS_MAP: VDS_MAP(0),		$
		TEL_RATE: TEL_RATE(0),		$
		DURATION: DURATION(0),		$
		USABLE: USABLE(0)}
	GOTO, FINISH
;
;  Error handling point.
;
HANDLE_ERROR:
	IF N_ELEMENTS(ERRMSG) NE 0 THEN ERRMSG = 'GET_RASTER: ' + MESSAGE $
		ELSE MESSAGE, MESSAGE, /CONTINUE
;
;  Close the database, and return.
;
FINISH:
	DBCLOSE
;
	RETURN
	END
