	PRO GET_RAW, RAW_ID, DEF, ERRMSG=ERRMSG
;+
; Project     :	SOHO - CDS
;
; Name        :	GET_RAW
;
; Purpose     :	Extracts a GIS raw data file entry from the database
;
; Explanation :	This routine extracts a GIS raw data file entry from the
;		database for a requested raw data file ID number.
;
; Use         :	GET_RAW, RAW_ID, DEF
;
; Inputs      :	RAW_ID = GIS raw data file ID number
;
; Opt. Inputs :	None.
;
; Outputs     :	DEF	= Structure containing the raw data file entry.  It
;			  contains the following tags:
;
;			RAW_ID	   = Raw data file ID number.  If the requested
;				     raw data file entry is not found, then a
;				     simpler structure is returned, with this
;				     set to -1.
;			RAW_DESC   = Free format description.
;			START_TIME = Start time of data taking, in TAI format.
;			DET_NUM	   = GIS detector number: 1-4
;			SLIT_NUM   = Slit number
;			HVOLT	   = High voltage value
;			LLD	   = Low level discriminator value
;			CNT_RATE   = Count rate range (min, max)
;			RAW_FILE   = Raw data file name
;			FFB	   = Front face bias
;			FIL_ID	   = Filament ID number
;			FIL_CUR	   = Filament current
;			ZONE_ID	   = Zone ID number
;
; Opt. Outputs:	None.
;
; Keywords    :
;       ERRMSG    = If defined and passed, then any error messages will be
;                   returned to the user in this parameter rather than
;                   depending on the MESSAGE routine in IDL.  If no errors are
;                   encountered, then a null string is returned.  In order to
;                   use this feature, ERRMSG must be defined first, e.g.
;
;                       ERRMSG = ''
;                       GET_RAW, ERRMSG=ERRMSG, ... 
;                       IF ERRMSG NE '' THEN ...
;
;
; Calls       :	DATATYPE, DBOPEN, DBFIND, DBEXT, DBCLOSE
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Category    :	Planning, Databases.
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 7 September 1995
;
; Modified    :	Version 1, William Thompson, GSFC, 7 September 1995
;		Version 2, William Thompson, GSFC, 14 September 1995
;			Added RAW_DESC, rename FF_BIAS to FFB to match GSET
;			database.
;		Version 3, William Thompson, GSFC, 1 March 1996
;			Fixed bug
;
; Version     :	Version 3, 1 March 1996
;-
;
	ON_ERROR, 2
;
;  Define a null result.  If the routine is successful, this will be updated
;  later.
;
	DEF = {RAW_ID: -1}
;
;  Check the number of parameters.
;
        IF N_PARAMS() NE 2 THEN BEGIN
           MESSAGE = 'Syntax:  GET_RAW, RAW_ID, DEF'
	   GOTO, HANDLE_ERROR
        ENDIF
;
;  Check the input parameter RAW_ID.
;
	TYPE = DATATYPE(RAW_ID,2)
	IF TYPE EQ 0 THEN BEGIN
           MESSAGE = 'RAW_ID is undefined'
           GOTO, HANDLE_ERROR
	END ELSE IF TYPE GE 4 THEN BEGIN
           MESSAGE = 'RAW_ID must be an integer'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(RAW_ID) NE 1 THEN BEGIN
           MESSAGE = 'RAW_ID must be a scalar'
           GOTO, HANDLE_ERROR
	ENDIF
;
;  Open the database.
;
	DBOPEN, 'raw_files'
;
;  Search on the RAW_ID field.
;
	ENTRIES = DBFIND('RAW_ID='+STRTRIM(LONG(RAW_ID),2), /SILENT)
;
;  If no entries were found, then return immediately.
;
	IF !ERR EQ 0 THEN BEGIN
		MESSAGE = 'Raw data file ID ' + TRIM(LONG(RAW_ID)) +	$
			' not found'
		GOTO, HANDLE_ERROR
	ENDIF
;
;  Extract the data from the raw data file database.
;
	DBEXT, ENTRIES, 'start_time,det_num,slit_num,hvolt,lld,cnt_rate,' + $
		'raw_file,ffb,fil_id,fil_cur,zone_id,raw_desc', START_TIME, $
		DET_NUM, SLIT_NUM, HVOLT, LLD, CNT_RATE, RAW_FILE, FFB, $
		FIL_ID, FIL_CUR, ZONE_ID, RAW_DESC
;
;  Define the output structure.
;
	DEF = { RAW_ID: RAW_ID(0),		$
		RAW_DESC: RAW_DESC(0),		$
		START_TIME: START_TIME(0),	$
		DET_NUM: DET_NUM(0),		$
		SLIT_NUM: SLIT_NUM(0),		$
		HVOLT: HVOLT(0),		$
		LLD: LLD(0),			$
		CNT_RATE: CNT_RATE(*,0),	$
		RAW_FILE: RAW_FILE(0),		$
		FFB: FFB(0),			$
		FIL_ID: FIL_ID(0),		$
		FIL_CUR: FIL_CUR(0),		$
		ZONE_ID: ZONE_ID(0)}
	GOTO, FINISH
;
;  Error handling point.
;
HANDLE_ERROR:
	IF N_ELEMENTS(ERRMSG) NE 0 THEN ERRMSG = 'GET_RAW: ' + MESSAGE $
		ELSE MESSAGE, MESSAGE, /CONTINUE
;
;  Close the database, and return.
;
FINISH:
	DBCLOSE
;
	RETURN
	END
