	PRO GET_RAW_FIL, DET_NUM, FIL_ID, DATE, DEF, ERRMSG=ERRMSG
;+
; Project     :	SOHO - CDS
;
; Name        :	GET_RAW_FIL
;
; Purpose     :	Extracts GIS raw filament data file entry from database
;
; Explanation :	This routine extracts a GIS raw data file entry from the
;		database for a requested detector number, filament ID number,
;		and target time.  The entry in the database with a start time
;		closest to the target time is returned.
;
; Use         :	GET_RAW_FIL, DET_NUM, FIL_ID, DATE, DEF
;
; Inputs      :	DET_NUM = GIS raw data file ID number
;		FIL_ID	= Filament ID number.
;		DATE	= Target date.  Can be in any standard CDS time format.
;
; Opt. Inputs :	None.
;
; Outputs     :	DEF	= Structure containing the raw data file entry.  It
;			  contains the following tags:
;
;			RAW_ID	   = Raw data file ID number.  If the requested
;				     raw data file entry is not found, then a
;				     simpler structure is returned, with this
;				     set to -1.
;			RAW_DESC   = Free format description
;			START_TIME = Start time of data taking, in TAI format.
;			DET_NUM	   = GIS detector number: 1-4
;			SLIT_NUM   = Slit number
;			HVOLT	   = High voltage value
;			LLD	   = Low level discriminator value
;			CNT_RATE   = Count rate range (min, max)
;			RAW_FILE   = Raw data file name
;			FFB	   = Front face bias
;			FIL_ID	   = Filament ID number
;			FIL_CUR	   = Filament current
;			ZONE_ID	   = Zone ID number
;
; Opt. Outputs:	None.
;
; Keywords    :
;       ERRMSG    = If defined and passed, then any error messages will be
;                   returned to the user in this parameter rather than
;                   depending on the MESSAGE routine in IDL.  If no errors are
;                   encountered, then a null string is returned.  In order to
;                   use this feature, ERRMSG must be defined first, e.g.
;
;                       ERRMSG = ''
;                       GET_RAW_FIL, ERRMSG=ERRMSG, ... 
;                       IF ERRMSG NE '' THEN ...
;
;
; Calls       :	DATATYPE, DBOPEN, DBFIND, DBEXT, DBCLOSE
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Category    :	Planning, Databases.
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 7 September 1995
;
; Modified    :	Version 1, William Thompson, GSFC, 7 September 1995
;		Version 2, William Thompson, GSFC, 14 September 1995
;			Added RAW_DESC, rename FF_BIAS to FFB to match GSET
;			database.
;
; Version     :	Version 2, 14 September 1995
;-
;
	ON_ERROR, 2
;
;  Define a null result.  If the routine is successful, this will be updated
;  later.
;
	DEF = {RAW_ID: -1}
;
;  Check the number of parameters.
;
        IF N_PARAMS() NE 4 THEN BEGIN
           MESSAGE = 'Syntax:  GET_RAW_FIL, DET_NUM, FIL_ID, DATE, DEF'
	   GOTO, HANDLE_ERROR
        ENDIF
;
;  Check the input parameters.
;
	TYPE = DATATYPE(DET_NUM,2)
	IF TYPE EQ 0 THEN BEGIN
           MESSAGE = 'DET_NUM is undefined'
           GOTO, HANDLE_ERROR
	END ELSE IF TYPE GE 4 THEN BEGIN
           MESSAGE = 'DET_NUM must be an integer'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(DET_NUM) NE 1 THEN BEGIN
           MESSAGE = 'DET_NUM must be a scalar'
           GOTO, HANDLE_ERROR
	ENDIF
;
	IF N_ELEMENTS(DATE) NE 1 THEN BEGIN
           MESSAGE = 'DATE must be a scalar'
           GOTO, HANDLE_ERROR
	ENDIF
;
;  Convert the date to TAI format.
;
	IF DATATYPE(DATE,1) EQ 'Double' THEN TAI = DATE ELSE	$
		TAI = UTC2TAI(DATE)
;
;  Open the database.
;
	DBOPEN, 'raw_files'
;
;  Search on the DET_NUM field.
;
	ENTRIES = DBFIND('DET_NUM='+STRTRIM(LONG(DET_NUM),2), /SILENT)
;
;  If no entries were found, then return immediately.
;
	IF !ERR EQ 0 THEN BEGIN
		MESSAGE = 'Raw data file ID not found'
		GOTO, HANDLE_ERROR
	ENDIF
;
;  Extract the dates, and find the one closest in time to the start time.
;
	DBEXT, ENTRIES, 'start_time', START_TIME
	DIFF = ABS(START_TIME-TAI)
	W = WHERE(DIFF EQ MIN(DIFF))
	ENTRY = ENTRIES(W(0))
;
;  Extract the data from the raw data file database.
;
	DBEXT, ENTRY, 'raw_id,raw_desc,start_time,slit_num,hvolt,lld,' + $
		'cnt_rate,raw_file,ffb,fil_cur,zone_id', RAW_ID, RAW_DESC, $
		START_TIME, SLIT_NUM, HVOLT, LLD, CNT_RATE, RAW_FILE, FFB, $
		FIL_ID, FIL_CUR, ZONE_ID
;
;  Define the output structure.
;
	DEF = { RAW_ID: RAW_ID(0),		$
		RAW_DESC: RAW_DESC(0),		$
		START_TIME: START_TIME(0),	$
		DET_NUM: DET_NUM(0),		$
		SLIT_NUM: SLIT_NUM(0),		$
		HVOLT: HVOLT(0),		$
		LLD: LLD(0),			$
		CNT_RATE: CNT_RATE(*,0),	$
		RAW_FILE: RAW_FILE(0),		$
		FFB: FFB(0),			$
		FIL_ID: FIL_ID(0),		$
		FIL_CUR: FIL_CUR(0),		$
		ZONE_ID: ZONE_ID(0)}
	GOTO, FINISH
;
;  Error handling point.
;
HANDLE_ERROR:
	IF N_ELEMENTS(ERRMSG) NE 0 THEN ERRMSG = 'GET_RAW_FIL: ' + MESSAGE $
		ELSE MESSAGE, MESSAGE, /CONTINUE
;
;  Close the database, and return.
;
FINISH:
	DBCLOSE
;
	RETURN
	END
