	PRO GET_STUDY, INPUT, STUDYVAR, DEF, USE_TITLE=USE_TITLE,	$
		ERRMSG=ERRMSG
;+
; Project     :	SOHO - CDS
;
; Name        :	GET_STUDY
;
; Purpose     :	Extracts a complete study definition from the database
;
; Explanation :	This routine extracts all the parameters of a study from the
;		database for the requested study ID number and variation index.
;
; Use         :	GET_STUDY, STUDY_ID, STUDYVAR, DEF
;		GET_STUDY, TITLE_ID, STUDYVAR, DEF, /USE_TITLE
;
; Inputs      :	STUDY_ID	= Study ID number
;		STUDYVAR	= Study variation index
;
; Opt. Inputs :	TITLE_ID	= Study title ID number.  This can be passed
;				  instead of the study ID, but this must be
;				  signalled by passing the /USE_TITLE flag.
;
; Outputs     :	DEF = Anonymous structure containing the complete study
;		      definition.  It contains the following tags:
;
;			STRUCT_TYPE = The character string 'CDS-STUDY'
;			STUDY_ID    = Study ID number.  If the requested study
;				      is not found, then a simpler structure is
;				      returned, with this set to -1.
;			TITLE_ID    = Study title ID number.
;			OBS_PROG    = Name of the study
;			TITLE	    = A descriptive title for the study
;			CATEGORY    = The study category, either "T"est,
;				      "S"cience or "C"alibration.
;			N_RASTERS0  = Fixed part of the number of rasters in
;				      the study.
;			VAR_POINT   = Either "Y" or "N" representing whether or
;				      not the pointing can be controlled by the
;				      user.  If "Y" then the pointing is
;				      defined in the plan.  If "N" then the
;				      pointing is embedded within the study.
;			N_RASTER_DEF= Number of rasters in the definition.
;			STUDYVAR    = Study variation index.  Initially, this
;				      is zero.  It is updated by this routine
;				      to reflect the index in the database.
;			SV_DESC	    = A short description of the study
;				      variation beyond what is given in the
;				      associated fundamental study description.
;			DURATION0   = Fixed part of the study duration, in
;				      seconds.
;			DURATION1   = Number of seconds/raster for the variable
;				      part of the study duration.
;			ZONE_ID	    = Pointing zone ID
;			USABLE	    = Either "Y" or "N" to signal whether or
;				      not the study is usable.  Normally "Y".
;			RASTERS	    = A array listing the rasters to be used
;				      during the study.
;
;		      The raster descriptions themselves are structures, of
;		      type "cds_study_ras", with the following tags:
;
;			RAS_ID	    = Raster ID number, pointing to the
;				      definition of the raster.
;			RAS_VAR	    = Raster variation index.
;			POINTING    = How to handle the pointing.  Valid
;				      entries are:
;
;					 1 = No pointing, supply in plan
;					 0 = Absolute pointing
;					-1 = Offset value from first raster
;
;			INS_X	    = Together with INS_Y, the pointing to use
;				      when user-supplied values are not
;				      allowed.  Only valid when POINTING=0
;				      (absolute) or POINTING=-1 (relative to
;				      1st raster).
;			INS_Y	    = ...
;			N_REPEAT_R  = Number of times to repeat raster.
;
; Opt. Outputs:	None.
;
; Keywords    :
;	USE_TITLE = If set, then the input parameter is the title ID rather
;		    than the study ID.  Not required if OBS_PROG is passed.
;
;       ERRMSG    = If defined and passed, then any error messages will be
;                   returned to the user in this parameter rather than
;                   depending on the MESSAGE routine in IDL.  If no errors are
;                   encountered, then a null string is returned.  In order to
;                   use this feature, ERRMSG must be defined first, e.g.
;
;                       ERRMSG = ''
;                       GET_STUDY, ERRMSG=ERRMSG, ...
;                       IF ERRMSG NE '' THEN ...
;
;
; Calls       :	DATATYPE, DBOPEN, DBFIND, DBEXT, DBCLOSE
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Category    :	Planning, Databases.
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 29 April 1994
;
; Modified    :	Version 1, William Thompson, GSFC, 2 August 1994
;               Version 2, Liyun Wang, GSFC/ARC, September 22, 1994
;                  Added the keyword ERRMSG.
;		Version 3, William Thompson, GSFC, 21 November 1994
;			Added items N_REPEAT_R and N_RASTER_DEF
;		Version 4, William Thompson, GSFC, 29 December 1994
;			Fixed bug when inputs are bytes.
;		Version 5, William Thompson, GSFC, 26 January 1995
;			Added tag STRUCT_TYPE
;		Version 6, William Thompson, GSFC, 10 February 1995
;			Changed N_POINTINGS to VAR_POINT.
;			Removed VAR_RASTERS.
;		Version 7, William Thompson, GSFC, 29 March 1995
;			Added tag ZONE_ID
;		Version 8, William Thompson, GSFC, 28 April 1995
;			Returns an error string when no study is found.
;		Version 9, William Thompson, GSFC, 12 May 1995
;			Modified to split off separate study_title database
;		Version 10, William Thompson, GSFC, 20 June 1995
;			Added tag FLAGABLE.
;		Version 11, William Thompson, GSFC, 27 October 1995.
;			Removed tag flagable.  Any study can produce a flag if
;			given an IEF_ID.
;		Version 12, William Thompson, GSFC, 1 December 1995
;			Allow case where N_RASTER_DEF=0.
;
; Version     :	Version 12, 1 December 1995
;-
;
	ON_ERROR, 2
;
;  Define a null result.  If the routine is successful, this will be updated
;  later.
;
	DEF = {STUDY_ID: -1}
;
;  Check the number of parameters.
;
        IF N_PARAMS() NE 3 THEN BEGIN
           MESSAGE = 'Syntax:  GET_STUDY, STUDY_ID, STUDYVAR, DEF'
	   GOTO, HANDLE_ERROR
        ENDIF
;
;  Check the input parameters.
;
	TYPE = DATATYPE(INPUT,2)
	IF TYPE EQ 0 THEN BEGIN
           MESSAGE = 'STUDY_ID is undefined'
	   GOTO, HANDLE_ERROR
	END ELSE IF TYPE GE 4 THEN BEGIN
           MESSAGE = 'STUDY_ID must be an integer'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(INPUT) NE 1 THEN BEGIN
           MESSAGE = 'STUDY_ID must be a scalar'
           GOTO, HANDLE_ERROR
	ENDIF
;
	TYPE = DATATYPE(STUDYVAR,2)
	IF TYPE EQ 0 THEN BEGIN
           MESSAGE = 'STUDYVAR is undefined'
           GOTO, HANDLE_ERROR
	END ELSE IF TYPE GE 4 THEN BEGIN
           MESSAGE = 'STUDYVAR must be an integer'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(STUDYVAR) NE 1 THEN BEGIN
           MESSAGE = 'STUDYVAR must be a scalar'
           GOTO, HANDLE_ERROR
	ENDIF
;
;  First open the top-level databases.
;
	DBOPEN, 'study_var,study_title,study'
;
;  Search on the STUDY_ID/TITLE_ID and STUDYVAR fields.
;
	IF KEYWORD_SET(USE_TITLE) THEN BEGIN
		ENTRIES = DBFIND('TITLE_ID=' + STRTRIM(LONG(INPUT),2) +	$
			',STUDYVAR=' + STRTRIM(LONG(STUDYVAR),2), /SILENT)
	END ELSE BEGIN
		ENTRIES = DBFIND('STUDY_ID=' + STRTRIM(LONG(INPUT),2) +	$
			',STUDYVAR=' + STRTRIM(LONG(STUDYVAR),2), /SILENT)
	ENDELSE
;
;  If no entries were found, then return immediately.
;
	IF !ERR EQ 0 THEN BEGIN
	   MESSAGE = 'Study ID/variation ' + TRIM(LONG(INPUT)) + '/' + $
		   TRIM(LONG(STUDYVAR)) + ' not found'
	   GOTO, HANDLE_ERROR
	ENDIF
;
;  Extract the relevant parameters from the database.
;
	DBEXT, ENTRIES(0), 'study_id,title_id,obs_prog,title,category,' + $
		'n_rasters0,n_raster_def,var_point,sv_desc,duration0,' + $
		'duration1,zone_id', STUDY_ID, TITLE_ID, OBS_PROG, TITLE, $
		CATEGORY, N_RASTERS0, N_RASTER_DEF, VAR_POINT, SV_DESC,	$
		DURATION0, DURATION1, ZONE_ID
	DBEXT, ENTRIES, 'usable', USABLE
;
;  If N_RASTER_DEF is zero, then skip reading in the remaining parameters.
;
	IF N_RASTER_DEF(0) EQ 0 THEN GOTO, FORM_OUTPUT
;
;  Open all the databases.
;
	DBOPEN, 'study_var2,study2,study_var,study_title,study'
;
;  Search on the STUDY_ID/TITLE_ID and STUDYVAR fields.
;
	IF KEYWORD_SET(USE_TITLE) THEN BEGIN
		ENTRIES = DBFIND('TITLE_ID=' + STRTRIM(LONG(INPUT),2) +	$
			',STUDYVAR=' + STRTRIM(LONG(STUDYVAR),2), /SILENT)
	END ELSE BEGIN
		ENTRIES = DBFIND('STUDY_ID=' + STRTRIM(LONG(INPUT),2) +	$
			',STUDYVAR=' + STRTRIM(LONG(STUDYVAR),2), /SILENT)
	ENDELSE
;
;  If no entries were found, then return immediately.
;
	IF !ERR EQ 0 THEN BEGIN
	   MESSAGE = 'Study ID/variation ' + TRIM(LONG(INPUT)) + '/' + $
		   TRIM(LONG(STUDYVAR)) + ' not found'
	   GOTO, HANDLE_ERROR
	ENDIF
;
;  Extract the relevant parameters from the database.
;
	DBEXT, ENTRIES, 'ras_id,ras_var,pointing,ins_x,ins_y,n_repeat_r', $
		RAS_ID, RAS_VAR, POINTING, INS_X, INS_Y, N_REPEAT_R
;
;  Define the structure for the raster definitions.
;
FORM_OUTPUT:
	RASTERS = {CDS_STUDY_RAS,	$
		RAS_ID: 0,	$
		RAS_VAR: 0,	$
		POINTING: 0,	$
		INS_X: 1.0,	$
		INS_Y: 1.0,	$
		N_REPEAT_R: 0}
;
	IF N_RASTER_DEF(0) NE 0 THEN BEGIN
		IF N_ELEMENTS(ENTRIES) EQ 1 THEN BEGIN
			RASTERS.RAS_ID     = RAS_ID(0)
			RASTERS.RAS_VAR    = RAS_VAR(0)
			RASTERS.POINTING   = POINTING(0)
			RASTERS.INS_X      = INS_X(0)
			RASTERS.INS_Y      = INS_Y(0)
			RASTERS.N_REPEAT_R = N_REPEAT_R(0)
		END ELSE BEGIN
			RASTERS = REPLICATE(RASTERS, N_ELEMENTS(ENTRIES))
			RASTERS.RAS_ID     = RAS_ID
			RASTERS.RAS_VAR    = RAS_VAR
			RASTERS.POINTING   = POINTING
			RASTERS.INS_X      = INS_X
			RASTERS.INS_Y      = INS_Y
			RASTERS.N_REPEAT_R = N_REPEAT_R
		ENDELSE
	ENDIF
;
;  Define the output structure as a whole.
;
	DEF = { STRUCT_TYPE: 'CDS-STUDY',	$
		STUDY_ID: STUDY_ID(0),		$
		TITLE_ID: TITLE_ID(0),		$
		OBS_PROG: OBS_PROG(0),		$
		TITLE: TITLE(0),		$
		CATEGORY: CATEGORY(0),		$
		N_RASTERS0: N_RASTERS0(0),	$
		VAR_POINT: VAR_POINT(0),	$
		N_RASTER_DEF: N_RASTER_DEF(0),	$
		STUDYVAR: STUDYVAR(0),		$
		SV_DESC: SV_DESC(0),		$
		DURATION0: DURATION0(0),	$
		DURATION1: DURATION1(0),	$
		ZONE_ID: ZONE_ID(0),		$
		USABLE: USABLE(0),		$
		RASTERS: RASTERS}
	GOTO, FINISH
;
;  Error handling point.
;
HANDLE_ERROR:
	IF N_ELEMENTS(ERRMSG) NE 0 THEN ERRMSG = 'GET_STUDY: ' + MESSAGE $
		ELSE MESSAGE, MESSAGE, /CONTINUE
;
;  Close the database, and return.
;
FINISH:
	DBCLOSE
;
	RETURN
	END
