	PRO GET_WAVECAL, DETECTOR, REQ_DATE, WAVECAL, GSET_ID=GSET_ID,	$
		ERRMSG=ERRMSG
;+
; Project     :	SOHO - CDS
;
; Name        :	GET_WAVECAL
;
; Purpose     :	Get the wavelength calibration for a given date.
;
; Category    :	Calibration, GIS, NIS, Wavelength
;
; Explanation :	Looks up the wavelength calibration in the database for a given
;		detector, date, and GSET_ID combination.  First, the software
;		finds the largest effective date which is on or earlier than
;		the observation date.  If there are multiple entries with the
;		same effective date, then the one with the latest entry date is
;		used.
;
; Syntax      :	GET_WAVECAL, DETECTOR, REQ_DATE, WAVECAL  [, GSET_ID=GSET_ID ]
;
; Examples    :	GET_WAVECAL, 'N', DATE, WAVECAL
;		IF WAVECAL.DATE NE '' THEN ...
;
; Inputs      :	DETECTOR = Either "NIS" or "GIS" (can be abbreviated).
;
;		REQ_DATE = The date to use in searching for a wavelength
;			   calibration.
;
; Opt. Inputs :	None.
;
; Outputs     :	WAVECAL	 = A structure containing the information about the
;			   wavelength calibration.  It contains the following
;			   tags:
;
;			DATE	 = The registration date for the calibration,
;				   in truncated CCSDS format.  If unsuccessful,
;				   then a simpler structure is returned with
;				   DATE set to the null string.
;
;			GSET_ID	 = GIS Setup ID number.  Not included for NIS
;				   data.
;
;			FILENAME = The name of the file that the calibration is
;				   based on.
;
;			COEFF	 = The wavelength calibration coefficients.
;				   The dimensions depend on the spectrometer
;				   used.
;
;				   For the NIS spectrometer, the dimensions are
;				   (2,2,2) with the following meanings:
;
;					Dim1 = NIS1 (0) or NIS2 (1)
;					Dim2 = Linear or quadratic coefficients
;					Dim3 = Coeff. are for wave->pix (0) or
;					       for pix->wave (1)
;
;				   For the GIS spectrometer, the dimensions are
;				   (4,3,2) with the following meanings:
;
;					Dim1 = Detector number (0-3)
;					Dim2 = Quadratic coefficients
;					Dim3 = Coeff. are for wave->pix (0) or
;					       for pix->wave (1)
;
; Opt. Outputs:	None.
;
; Keywords    :	GSET_ID	 = The GIS Setup ID that the wavelength calibration
;			   refers to.  This is a required keyword for
;			   retrieving GIS wavelength calibrations.
;
;		ERRMSG	 = If defined and passed, then any error messages will
;			   be returned to the user in this parameter rather
;			   than depending on the MESSAGE routine in IDL.  If no
;			   errors are encountered, then a null string is
;			   returned.  In order to use this feature, ERRMSG must
;			   be defined first, e.g.
;
;				ERRMSG = ''
;				GET_WAVECAL, ERRMSG=ERRMSG, ...
;				IF ERRMSG NE '' THEN ...
;
; Calls       :	
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Prev. Hist. :	None.
;
; History     :	Version 1, 05-Mar-1996, William Thompson, GSFC
;		Version 2, 12-Mar-1996, William Thompson, GSFC
;			Add /ATTENTION to call to CDS_MESSAGE
;			Return named structures.
;		Version 3, 19-Mar-1996, William Thompson, GSFC
;			Removed call to CDS_MESSAGE
;		Version 4, 05-Jul-1996, William Thompson, GSFC
;			Change NIS coefficients to second order.
;		Version 5, 18-Apr-1997, William Thompson, GSFC
;			Use DATE_EFF in database.
;
; Contact     :	WTHOMPSON
;-
;
	ON_ERROR, 2
;
;  Define a null result.  If the routine is successful, this will be updated
;  later.  Also define TRIED_AGAIN for later use.
;
	WAVECAL = {DATE: ''}
	TRIED_AGAIN = 0
;
;  Check the number of parameters.
;
	IF N_PARAMS() NE 3 THEN BEGIN
		MESSAGE = 'Syntax:  GET_WAVECAL, DETECTOR, REQ_DATE, WAVECAL'
		GOTO, HANDLE_ERROR
	ENDIF
;
;  Make sure that DETECTOR is defined properly.
;
	IF DATATYPE(DETECTOR,1) NE 'String' THEN BEGIN
		MESSAGE = 'DETECTOR must be of type string'
		GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(DETECTOR) NE 1 THEN BEGIN
		MESSAGE = 'DETECTOR must be a scalar'
		GOTO, HANDLE_ERROR
	ENDIF
;
;  Determine which detector is being considered.
;
	DET = STRUPCASE(STRMID(DETECTOR,0,1))
	IF (DET NE 'N') AND (DET NE 'G') THEN BEGIN
		MESSAGE = 'DETECTOR must be either N or G'
		GOTO, HANDLE_ERROR
	ENDIF
;
;  If the GIS detector was selected, then make sure that GSET_ID was passed,
;  and that it points to a valid entry in the database.
;
	IF DET EQ 'G' THEN BEGIN
		IF N_ELEMENTS(GSET_ID) NE 1 THEN BEGIN
			MESSAGE = 'Keyword GSET_ID must be passed for the GIS'
			GOTO, HANDLE_ERROR
		ENDIF
	ENDIF
;
;  Convert the requested date into CCSDS format.
;
	IF N_ELEMENTS(REQ_DATE) NE 1 THEN BEGIN
		MESSAGE = 'REQ_DATE must be a scalar'
		GOTO, HANDLE_ERROR
	ENDIF
;
	MESSAGE = ''
	UTC = ANYTIM2UTC(REQ_DATE, ERRMSG=MESSAGE)
;
TRY_AGAIN:
	TAI = UTC2TAI(UTC)
	IF MESSAGE NE '' THEN GOTO, HANDLE_ERROR
	DATE_STR = STRING(TAI, FORMAT='(F15.3)')
;
;  Open the appropriate database, and find all the entries prior to or equal to
;  the requested date.
;
	CASE DET OF
	    'N': BEGIN
		DBOPEN, 'nis_wave'
		ENTRIES = DBFIND('DATE_EFF<' + DATE_STR, /SILENT, /FULLSTRING)
		END
	    'G': BEGIN
		DBOPEN, 'gis_wave'
		ENTRIES = DBFIND('DATE_EFF<' + DATE_STR + ',GSET_ID=' +	$
			NTRIM(GSET_ID), /SILENT, /FULLSTRING)
		END
	ENDCASE
;
;  If no entries were found, then try again with the current date/time value.
;  Otherwise return an error message.
;
	IF !ERR EQ 0 THEN BEGIN
	    IF TRIED_AGAIN THEN BEGIN
		MESSAGE = 'No wavelength calibration data found'
		GOTO, HANDLE_ERROR
	    ENDIF ELSE BEGIN
		GET_UTC, UTC
		TRIED_AGAIN = 1
		MESSAGE, /CONTINUE, 'No wavelength calibration data found'
		MESSAGE, /CONTINUE, 'Trying again with current date'
		BELL
		DBCLOSE
		GOTO, TRY_AGAIN
	    ENDELSE
	ENDIF
;
;  Select the last entry.  (However, if had to try again, then select the first
;  entry instead.)
;
	ENTRIES = DBSORT(ENTRIES, 'DATE_EFF', REVERSE=[1-TRIED_AGAIN])
;
;  Find all entries with the selected effective date, and sort on the entry
;  date.
;
	DBEXT, ENTRIES(0), 'date_eff', DATE_EFF
	ENTRIES = DBFIND('DATE_EFF=' + TRIM(DATE_EFF,'(F15.3)'), ENTRIES, $
		/SILENT)
	IF N_ELEMENTS(ENTRIES) GT 1 THEN	$
		ENTRIES = DBSORT(ENTRIES, 'DATE', REVERSE=[1])
	ENTRY = ENTRIES(0)
;
;  Depending on the detector, read in the wavelength calibration data.
;
	CASE DET OF
	    'N': BEGIN
		DBEXT, ENTRY, 'date,filename,coeff', DATE, FILENAME, COEFF
		COEFF = REFORM(COEFF,2,3,2)
		WAVECAL = {NIS_WAVECAL,		$
			DATE: DATE(0),		$
			DATE_EFF: DATE_EFF(0),	$
			FILENAME: FILENAME(0),	$
			COEFF: COEFF}
		END
	    'G': BEGIN
		DBEXT, ENTRY, 'date,gset_id,filename,coeff',	$
			DATE, GSET, FILENAME, COEFF
		COEFF = REFORM(COEFF,4,3,2)
		WAVECAL = {GIS_WAVECAL,		$
			DATE: DATE(0),		$
			DATE_EFF: DATE_EFF(0),	$
			GSET_ID: GSET(0),	$
			FILENAME: FILENAME(0),	$
			COEFF: COEFF}
		END
	ENDCASE
;
	GOTO, FINISH
;
;  Error handling point.
;
HANDLE_ERROR:
	IF N_ELEMENTS(ERRMSG) NE 0 THEN ERRMSG = 'GET_WAVECAL: ' + MESSAGE $
		ELSE MESSAGE, MESSAGE, /CONTINUE
;
;  Close the database, and return.
;
FINISH:
	DBCLOSE
;
	RETURN
	END
