	FUNCTION IMP_DATAWIN, UNIT, DEF, ERRMSG=ERRMSG
;+
; Project     :	SOHO - CDS
;
; Name        :	IMP_DATAWIN()
;
; Purpose     :	Reads data window definition from database export file
;
; Explanation :	Reads a data extraction window definition from a database
;		export file.  Called from IMPORT_STUDY.
;
; Use         :	Result = IMP_DATAWIN( UNIT, DEF )
;
;		IF NOT IMP_DATAWIN( UNIT, DEF ) THEN ...
;
; Inputs      :	UNIT = The logical unit number of the opened input file.
;
; Opt. Inputs :	None.
;
; Outputs     :	DEF  = The data extraction window list definition.  This is a
;		       structure of a form suitable for use with the routine
;		       ADD_DATAWIN.
;
; Opt. Outputs:	None.
;
; Keywords    :	ERRMSG	  = If defined and passed, then any error messages will
;			    be returned to the user in this parameter rather
;			    than depending on the MESSAGE routine in IDL.  If
;			    no errors are encountered, then a null string is
;			    returned.  In order to use this feature, ERRMSG
;			    must be defined first, e.g.
;
;				ERRMSG = ''
;				Result = IMP_DATAWIN( ERRMSG=ERRMSG, ... )
;				IF ERRMSG NE '' THEN ...
;
; Calls       :	None.
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Category    :	Planning, Databases.
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 23 May 1995
;
; Modified    :	Version 1, William Thompson, GSFC, 23 May 1995
;		Version 2, William Thompson, GSFC, 20 June 1995
;			Added tag WIN_FLAG.
;
; Version     :	Version 2, 20 June 1995
;-
;
	ON_ERROR, 2
;
;  Check the number of parameters.
;
	IF N_PARAMS() NE 2 THEN BEGIN
		MESSAGE = 'Syntax:  IMP_DATAWIN, UNIT, DEF'
		GOTO, HANDLE_ERROR
	ENDIF
;
;  Set up some default values for various parameters, in case they're not found
;  in the file.
;
	DW_ID    = -1
	DETECTOR = ''
	DW_DESC  = ''
	W_WIDTH  = 0
	W_HEIGHT = 0
	VDS_BACK = 0
	LL_ID    = 0
	N_WINS   = 0
;
;  Keep reading until the first DW_IND line is found.
;
	KEYWORD = ''
	LINE = ''
	WHILE (KEYWORD NE 'DW_IND') AND (NOT EOF(UNIT)) DO BEGIN
		READF, UNIT, LINE
;
;  Parse the line to extract the keyword and the value.
;
		SEP = STRPOS(LINE,'=')
		IF (SEP LE 0) OR (SEP GE STRLEN(LINE)) THEN BEGIN
			MESSAGE = 'Line is not in keyword=value format'
			GOTO, HANDLE_ERROR
		ENDIF
		KEYWORD = STRTRIM( STRMID(LINE,0,SEP), 2)
		VALUE   = STRTRIM( STRMID(LINE,SEP+1, (STRLEN(LINE)-SEP-1)), 2)
;
;  Interpret the keyword and value.
;
		CASE STRUPCASE(KEYWORD) OF
			'DW_ID':	DW_ID    = FIX(VALUE)
			'DETECTOR':	DETECTOR = VALUE
			'DW_DESC':	DW_DESC  = VALUE
			'W_WIDTH':	W_WIDTH  = FIX(VALUE)
			'W_HEIGHT':	W_HEIGHT = FIX(VALUE)
			'VDS_BACK':	VDS_BACK = FIX(VALUE)
			'LL_ID':	LL_ID    = FIX(VALUE)
			'N_WINS':	N_WINS   = FIX(VALUE)
			'DW_IND':	DW_IND   = FIX(VALUE)
			ELSE:	BEGIN
				MESSAGE = 'Unrecognized keyword ' + KEYWORD
				GOTO, HANDLE_ERROR
				END
		ENDCASE
	ENDWHILE
;
;  If N_WINS is zero, then it probably wasn't found.  In any case, it's an
;  error.
;
	IF N_WINS LE 0 THEN BEGIN
		MESSAGE = 'No data windows found'
		GOTO, HANDLE_ERROR
	ENDIF
;
;  Define the structure for the line definitions.
;
	WINS = {CDS_WINDOWS,		$
		WIN_NAME: '',		$
		WIN_DEF: [0,0,0,0],	$
		WIN_FLAG: 0}
	IF N_WINS GT 1 THEN WINS = REPLICATE(WINS, N_WINS)
;
;  Read through the window definitions.
;
	FOR I = 0,N_WINS-1 DO BEGIN
	    KEYWORD = 'Keyword'
	    WHILE (KEYWORD NE 'DW_IND') AND (STRTRIM(LINE,2) NE '') AND	$
			(NOT EOF(UNIT)) DO BEGIN
		READF, UNIT, LINE
;
;  Parse the line to extract the keyword and the value.
;
		IF STRTRIM(LINE) NE '' THEN BEGIN
		    SEP = STRPOS(LINE,'=')
		    IF (SEP LE 0) OR (SEP GE STRLEN(LINE)) THEN BEGIN
			MESSAGE = 'Line is not in keyword=value format'
			GOTO, HANDLE_ERROR
		    ENDIF
		    KEYWORD = STRTRIM( STRMID(LINE,0,SEP), 2)
		    VALUE   = STRTRIM( STRMID(LINE,SEP+1,	$
			    (STRLEN(LINE)-SEP-1)), 2)
;
;  Interpret the keyword and value.
;
		    CASE STRUPCASE(KEYWORD) OF
			'WIN_NAME':	WINS(I).WIN_NAME = VALUE
			'WIN_DEF':	WINS(I).WIN_DEF  = FIX(FXBTDIM(VALUE))
			'WIN_FLAG':	WINS(I).WIN_FLAG = FIX(VALUE)
			'DW_IND':	DW_IND   = FIX(VALUE)
			ELSE:	BEGIN
				MESSAGE = 'Unrecognized keyword ' + KEYWORD
				GOTO, HANDLE_ERROR
				END
		    ENDCASE
		ENDIF
	    ENDWHILE
	ENDFOR
;
;  Define the output structure as a whole.
;
	DEF = { DW_ID: DW_ID,		$
		DETECTOR: DETECTOR,	$
		DW_DESC: DW_DESC,	$
		W_WIDTH: W_WIDTH,	$
		W_HEIGHT: W_HEIGHT,	$
		VDS_BACK: VDS_BACK,	$
		LL_ID: LL_ID,		$
		WINS: WINS}
	RESULT = 1
	GOTO, FINISH
;
;  Error handling point.
;
HANDLE_ERROR:
	IF N_ELEMENTS(ERRMSG) NE 0 THEN ERRMSG = 'IMP_DATAWIN: ' + MESSAGE $
		ELSE MESSAGE, MESSAGE
	RESULT = 0
;
;  Exit point.
;
FINISH:
	RETURN, RESULT
	END
