	FUNCTION IMP_F_RASTER, UNIT, DEF, ERRMSG=ERRMSG
;+
; Project     :	SOHO - CDS
;
; Name        :	IMP_F_RASTER()
;
; Purpose     :	Reads fundamental raster def. from database export file
;
; Explanation :	Reads a fundamental raster definition from a database export
;		file.  Called from IMPORT_STUDY.
;
; Use         :	Result = IMP_F_RASTER( UNIT, DEF )
;
;		IF NOT IMP_F_RASTER( UNIT, DEF ) THEN ...
;
; Inputs      :	UNIT = The logical unit number of the opened input file.
;
; Opt. Inputs :	None.
;
; Outputs     :	DEF  = The raster definition.  This is a structure of a form
;		       suitable for use with the routine ADD_F_RASTER.
;
; Opt. Outputs:	None.
;
; Keywords    :	ERRMSG	  = If defined and passed, then any error messages will
;			    be returned to the user in this parameter rather
;			    than depending on the MESSAGE routine in IDL.  If
;			    no errors are encountered, then a null string is
;			    returned.  In order to use this feature, ERRMSG
;			    must be defined first, e.g.
;
;				ERRMSG = ''
;				Result = IMP_F_RASTER( ERRMSG=ERRMSG, ... )
;				IF ERRMSG NE '' THEN ...
;
; Calls       :	None.
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Category    :	Planning, Databases.
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 23 May 1995
;
; Modified    :	Version 1, William Thompson, GSFC, 23 May 1995
;
; Version     :	Version 1, 23 May 1995
;-
;
	ON_ERROR, 2
;
;  Check the number of parameters.
;
	IF N_PARAMS() NE 2 THEN BEGIN
		MESSAGE = 'Syntax:  IMP_F_RASTER, UNIT, DEF'
		GOTO, HANDLE_ERROR
	ENDIF
;
;  Set up some default values for various parameters, in case they're not found
;  in the file.
;
	RAS_ID   = -1
	DETECTOR = ''
	RAS_DESC = ''
	SLIT_NUM = 0
	XSTEP    = 0
	YSTEP	 = 0
	NX	 = 0
	NY	 = 0
;
;  Keep reading until a blank line is found.
;
	KEYWORD = ''
	LINE = 'String'
	WHILE (LINE NE '') AND (NOT EOF(UNIT)) DO BEGIN
	    READF, UNIT, LINE
;
;  Parse the line to extract the keyword and the value.
;
	    IF STRTRIM(LINE) NE '' THEN BEGIN
		SEP = STRPOS(LINE,'=')
		IF (SEP LE 0) OR (SEP GE STRLEN(LINE)) THEN BEGIN
			MESSAGE = 'Line is not in keyword=value format'
			GOTO, HANDLE_ERROR
		ENDIF
		KEYWORD = STRTRIM( STRMID(LINE,0,SEP), 2)
		VALUE   = STRTRIM( STRMID(LINE,SEP+1, (STRLEN(LINE)-SEP-1)), 2)
;
;  Interpret the keyword and value.
;
		CASE STRUPCASE(KEYWORD) OF
			'RAS_ID':	RAS_ID   = FIX(VALUE)
			'DETECTOR':	DETECTOR = VALUE
			'RAS_DESC':	RAS_DESC = VALUE
			'SLIT_NUM':	SLIT_NUM = FIX(VALUE)
			'XSTEP':	XSTEP    = FIX(VALUE)
			'YSTEP':	YSTEP    = FIX(VALUE)
			'NX':		NX       = FIX(VALUE)
			'NY':		NY       = FIX(VALUE)
			ELSE:	BEGIN
				MESSAGE = 'Unrecognized keyword ' + KEYWORD
				GOTO, HANDLE_ERROR
				END
		ENDCASE
	    ENDIF
	ENDWHILE
;
;  Define the output structure.
;
	DEF = {CDS_F_RASTER,		$
		RAS_ID: RAS_ID,		$
		DETECTOR: DETECTOR,	$
		RAS_DESC: RAS_DESC,	$
		SLIT_NUM: SLIT_NUM,	$
		XSTEP: XSTEP,		$
		YSTEP: YSTEP,		$
		NX: NX,			$
		NY: NY}
	RESULT = 1
	GOTO, FINISH
;
;  Error handling point.
;
HANDLE_ERROR:
	IF N_ELEMENTS(ERRMSG) NE 0 THEN ERRMSG = 'IMP_F_RASTER: ' + MESSAGE $
		ELSE MESSAGE, MESSAGE
	RESULT = 0
;
;  Exit point.
;
FINISH:
	RETURN, RESULT
	END
