	FUNCTION IMP_LINELIST, UNIT, DEF, ERRMSG=ERRMSG
;+
; Project     :	SOHO - CDS
;
; Name        :	IMP_LINELIST()
;
; Purpose     :	Reads a linelist definition from a database export file
;
; Explanation :	Reads a linelist definition from a database export file.
;		Called from IMPORT_STUDY.
;
; Use         :	Result = IMP_LINELIST( UNIT, DEF )
;
;		IF NOT IMP_LINELIST( UNIT, DEF ) THEN ...
;
; Inputs      :	UNIT = The logical unit number of the opened input file.
;
; Opt. Inputs :	None.
;
; Outputs     :	DEF  = The linelist definition.  This is a structure of a form
;		       suitable for use with the routine ADD_LINELIST.
;
; Opt. Outputs:	None.
;
; Keywords    :	ERRMSG	  = If defined and passed, then any error messages will
;			    be returned to the user in this parameter rather
;			    than depending on the MESSAGE routine in IDL.  If
;			    no errors are encountered, then a null string is
;			    returned.  In order to use this feature, ERRMSG
;			    must be defined first, e.g.
;
;				ERRMSG = ''
;				Result = IMP_LINELIST( ERRMSG=ERRMSG, ... )
;				IF ERRMSG NE '' THEN ...
;
; Calls       :	None.
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Category    :	Planning, Databases.
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 23 May 1995
;
; Modified    :	Version 1, William Thompson, GSFC, 23 May 1995
;
; Version     :	Version 1, 23 May 1995
;-
;
	ON_ERROR, 2
;
;  Check the number of parameters.
;
	IF N_PARAMS() NE 2 THEN BEGIN
		MESSAGE = 'Syntax:  IMP_LINELIST, UNIT, DEF'
		GOTO, HANDLE_ERROR
	ENDIF
;
;  Set up some default values for various parameters, in case they're not found
;  in the file.
;
	LL_ID    = -1
	DETECTOR = ''
	LL_DESC  = ''
	N_LINES  = 0
;
;  Keep reading until the first LL_IND line is found.
;
	KEYWORD = ''
	LINE = ''
	WHILE (KEYWORD NE 'LL_IND') AND (NOT EOF(UNIT)) DO BEGIN
		READF, UNIT, LINE
;
;  Parse the line to extract the keyword and the value.
;
		SEP = STRPOS(LINE,'=')
		IF (SEP LE 0) OR (SEP GE STRLEN(LINE)) THEN BEGIN
			MESSAGE = 'Line is not in keyword=value format'
			GOTO, HANDLE_ERROR
		ENDIF
		KEYWORD = STRTRIM( STRMID(LINE,0,SEP), 2)
		VALUE   = STRTRIM( STRMID(LINE,SEP+1, (STRLEN(LINE)-SEP-1)), 2)
;
;  Interpret the keyword and value.
;
		CASE STRUPCASE(KEYWORD) OF
			'LL_ID':	LL_ID    = FIX(VALUE)
			'DETECTOR':	DETECTOR = VALUE
			'LL_DESC':	LL_DESC  = VALUE
			'N_LINES':	N_LINES  = FIX(VALUE)
			'LL_IND':	LL_IND   = FIX(VALUE)
			ELSE:	BEGIN
				MESSAGE = 'Unrecognized keyword ' + KEYWORD
				GOTO, HANDLE_ERROR
				END
		ENDCASE
	ENDWHILE
;
;  If N_LINES is zero, then it probably wasn't found.  In any case, it's an
;  error.
;
	IF N_LINES LE 0 THEN BEGIN
		MESSAGE = 'No lines found'
		GOTO, HANDLE_ERROR
	ENDIF
;
;  Define the structure for the line definitions.
;
	LINES = {CDS_LINES,	$
		LINENAME: '',	$
		WAVELNTH: 0.0,	$
		GR_ORDER: 0,	$
		CEN_PIX: 0,	$
		WAVEBAND: 0}
	IF N_LINES GT 1 THEN LINES = REPLICATE(LINES, N_LINES)
;
;  Read through the line definitions.
;
	FOR I = 0,N_LINES-1 DO BEGIN
	    KEYWORD = 'Keyword'
	    WHILE (KEYWORD NE 'LL_IND') AND (STRTRIM(LINE,2) NE '') AND	$
			(NOT EOF(UNIT)) DO BEGIN
		READF, UNIT, LINE
;
;  Parse the line to extract the keyword and the value.
;
		IF STRTRIM(LINE) NE '' THEN BEGIN
		    SEP = STRPOS(LINE,'=')
		    IF (SEP LE 0) OR (SEP GE STRLEN(LINE)) THEN BEGIN
			MESSAGE = 'Line is not in keyword=value format'
			GOTO, HANDLE_ERROR
		    ENDIF
		    KEYWORD = STRTRIM( STRMID(LINE,0,SEP), 2)
		    VALUE   = STRTRIM( STRMID(LINE,SEP+1,	$
			    (STRLEN(LINE)-SEP-1)), 2)
;
;  Interpret the keyword and value.
;
		    CASE STRUPCASE(KEYWORD) OF
			'LINENAME':	LINES(I).LINENAME = VALUE
			'WAVELNTH':	LINES(I).WAVELNTH = FLOAT(VALUE)
			'GR_ORDER':	LINES(I).GR_ORDER = FIX(VALUE)
			'CEN_PIX':	LINES(I).CEN_PIX  = FIX(VALUE)
			'WAVEBAND':	LINES(I).WAVEBAND = FIX(VALUE)
			'LL_IND':	LL_IND = FIX(VALUE)
			ELSE:	BEGIN
				MESSAGE = 'Unrecognized keyword ' + KEYWORD
				GOTO, HANDLE_ERROR
				END
		    ENDCASE
		ENDIF
	    ENDWHILE
	ENDFOR
;
;  Define the output structure as a whole.
;
	DEF = { LL_ID: LL_ID,		$
		DETECTOR: DETECTOR,	$
		LL_DESC: LL_DESC,	$
		LINES: LINES}
	RESULT = 1
	GOTO, FINISH
;
;  Error handling point.
;
HANDLE_ERROR:
	IF N_ELEMENTS(ERRMSG) NE 0 THEN ERRMSG = 'IMP_LINELIST: ' + MESSAGE $
		ELSE MESSAGE, MESSAGE, /CONTINUE
	RESULT = 0
;
;  Exit point.
;
FINISH:
	RETURN, RESULT
	END
