	FUNCTION IMP_SCI_PLAN, UNIT, DEF, ERRMSG=ERRMSG
;+
; Project     :	SOHO - CDS
;
; Name        :	IMP_SCI_PLAN()
;
; Purpose     :	Reads a SCI_PLAN definition from an export file.
;
; Explanation :	Reads a science plan definition from a database export file.
;		Called from IMPORT_PLAN.
;
; Use         :	Result = IMP_SCI_PLAN( UNIT, DEF )
;
;		IF NOT IMP_SCI_PLAN( UNIT, DEF ) THEN ...
;
; Inputs      :	UNIT = The logical unit number of the opened input file.
;
; Opt. Inputs :	None.
;
; Outputs     :	DEF  = The linelist definition.  This is a structure of a form
;		       suitable for use with the routine ADD_PLAN.
;
; Opt. Outputs:	None.
;
; Keywords    :	ERRMSG	  = If defined and passed, then any error messages will
;			    be returned to the user in this parameter rather
;			    than depending on the MESSAGE routine in IDL.  If
;			    no errors are encountered, then a null string is
;			    returned.  In order to use this feature, ERRMSG
;			    must be defined first, e.g.
;
;				ERRMSG = ''
;				Result = IMP_SCI_PLAN( ERRMSG=ERRMSG, ... )
;				IF ERRMSG NE '' THEN ...
;
; Calls       :	None.
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Category    :	Planning, Databases.
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 10 April 1997
;
; Modified    :	Version 1, William Thompson, GSFC, 10 April 1997
;
; Version     :	Version 1, 10 April 1997
;-
;
	ON_ERROR, 2
;
;  Check the number of parameters.
;
	IF N_PARAMS() NE 2 THEN BEGIN
		MESSAGE = 'Syntax:  IMP_SCI_PLAN, UNIT, DEF'
		GOTO, HANDLE_ERROR
	ENDIF
;
;  Keep reading until a blank line is found.
;
	KEYWORD = ''
	LINE = 'String'
	WHILE (LINE NE '') AND (NOT EOF(UNIT)) DO BEGIN
	    READF, UNIT, LINE
;
;  Parse the line to extract the keyword and the value.
;
	    IF STRTRIM(LINE) NE '' THEN BEGIN
		SEP = STRPOS(LINE,'=')
		IF (SEP LE 0) OR (SEP GE STRLEN(LINE)) THEN BEGIN
			MESSAGE = 'Line is not in keyword=value format'
			GOTO, HANDLE_ERROR
		ENDIF
		KEYWORD = STRTRIM( STRMID(LINE,0,SEP), 2)
		VALUE   = STRTRIM( STRMID(LINE,SEP+1, (STRLEN(LINE)-SEP-1)), 2)
;
;  Interpret the keyword and value.
;
		CASE STRUPCASE(KEYWORD) OF
			'SCI_OBJ':	SCI_OBJ	     = VALUE
			'SCI_SPEC':	SCI_SPEC     = VALUE
			'NOTES':	NOTES	     = VALUE
			'STARTIME':	START_TIME   = DOUBLE(VALUE)
			'END_TIME':	END_TIME     = DOUBLE(VALUE)
			'OBJECT':	OBJECT	     = VALUE
			'OBJ_ID':	OBJ_ID	     = VALUE
			'PROG_ID':	PROG_ID	     = FIX(VALUE)
			'CMP_NO':	CMP_NO	     = FIX(VALUE)
			'XCEN':		XCEN	     = VALUE
			'YCEN':		YCEN	     = VALUE
			'DISTURB':	DISTURBANCES = VALUE
			ELSE:	BEGIN
				MESSAGE = 'Unrecognized keyword ' + KEYWORD
				GOTO, HANDLE_ERROR
				END
		ENDCASE
	    ENDIF
	ENDWHILE
;
;  Define the output structure.
;
	DEF = { SCI_OBJ:	SCI_OBJ,	$
		SCI_SPEC:	SCI_SPEC,	$
		NOTES:		NOTES,		$
		START_TIME:	START_TIME,	$
		END_TIME:	END_TIME,	$
		OBJECT:		OBJECT,		$
		OBJ_ID:		OBJ_ID,		$
		PROG_ID:	PROG_ID,	$
		CMP_NO:		CMP_NO,		$
		XCEN:		XCEN,		$
		YCEN:		YCEN,		$
		DISTURBANCES:	DISTURBANCES}
	RESULT = 1
	GOTO, FINISH
;
;  Error handling point.
;
HANDLE_ERROR:
	IF N_ELEMENTS(ERRMSG) NE 0 THEN ERRMSG = 'IMP_SCI_PLAN: ' + MESSAGE $
		ELSE MESSAGE, MESSAGE, /CONTINUE
	RESULT = 0
;
;  Exit point.
;
FINISH:
	RETURN, RESULT
	END
