	FUNCTION IMP_V_RASTER, UNIT, DEF, ERRMSG=ERRMSG
;+
; Project     :	SOHO - CDS
;
; Name        :	IMP_V_RASTER()
;
; Purpose     :	Reads raster variation def. from database export file
;
; Explanation :	Reads a raster variation definition from a database export
;		file.  Called from IMPORT_STUDY.
;
; Use         :	Result = IMP_V_RASTER( UNIT, DEF )
;
;		IF NOT IMP_V_RASTER( UNIT, DEF ) THEN ...
;
; Inputs      :	UNIT = The logical unit number of the opened input file.
;
; Opt. Inputs :	None.
;
; Outputs     :	DEF  = The raster definition.  This is a structure of a form
;		       suitable for use with the routine ADD_V_RASTER.
;
; Opt. Outputs:	None.
;
; Keywords    :	ERRMSG	  = If defined and passed, then any error messages will
;			    be returned to the user in this parameter rather
;			    than depending on the MESSAGE routine in IDL.  If
;			    no errors are encountered, then a null string is
;			    returned.  In order to use this feature, ERRMSG
;			    must be defined first, e.g.
;
;				ERRMSG = ''
;				Result = IMP_V_RASTER( ERRMSG=ERRMSG, ... )
;				IF ERRMSG NE '' THEN ...
;
; Calls       :	None.
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Category    :	Planning, Databases.
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 23 May 1995
;
; Modified    :	Version 1, William Thompson, GSFC, 23 May 1995
;		Version 2, William Thompson, GSFC, 27 October 1995
;			Added item IEF_ID.
;
; Version     :	Version 2, 27 October 1995
;-
;
	ON_ERROR, 2
;
;  Check the number of parameters.
;
	IF N_PARAMS() NE 2 THEN BEGIN
		MESSAGE = 'Syntax:  IMP_V_RASTER, UNIT, DEF'
		GOTO, HANDLE_ERROR
	ENDIF
;
;  Set up some default values for various parameters, in case they're not found
;  in the file.
;
	RAS_ID     = -1
	RAS_VAR    = -1
	RV_DESC    = ''
	EXPTIME    = 0.0
	LL_ID      = 0
	COMP_ID	   = 0
	COMP_OPT   = 0
	DW_ID	   = 0
	IEF_ID	   = 0
	VDS_ORIENT = 0
	VDS_MAP    = 0
	TEL_RATE   = ''
	DURATION   = 0.0
	USABLE     = ''
;
;  Keep reading until a blank line is found.
;
	KEYWORD = ''
	LINE = 'String'
	WHILE (LINE NE '') AND (NOT EOF(UNIT)) DO BEGIN
		READF, UNIT, LINE
;
;  Parse the line to extract the keyword and the value.
;
		IF STRTRIM(LINE) NE '' THEN BEGIN
		    SEP = STRPOS(LINE,'=')
		    IF (SEP LE 0) OR (SEP GE STRLEN(LINE)) THEN BEGIN
			MESSAGE = 'Line is not in keyword=value format'
			GOTO, HANDLE_ERROR
		    ENDIF
		    KEYWORD = STRTRIM( STRMID(LINE,0,SEP), 2)
		    VALUE   = STRTRIM( STRMID(LINE,SEP+1,	$
			    (STRLEN(LINE)-SEP-1)), 2)
;
;  Interpret the keyword and value.
;
		    CASE STRUPCASE(KEYWORD) OF
			'RAS_ID':	RAS_ID     = FIX(VALUE)
			'RAS_VAR':	RAS_VAR    = FIX(VALUE)
			'RV_DESC':	RV_DESC    = VALUE
			'EXPTIME':	EXPTIME    = FLOAT(VALUE)
			'LL_ID':	LL_ID      = FIX(VALUE)
			'COMP_ID':	COMP_ID    = FIX(VALUE)
			'COMP_OPT':	COMP_OPT   = FIX(VALUE)
			'DW_ID':	DW_ID      = FIX(VALUE)
			'IEF_ID':	IEF_ID	   = FIX(VALUE)
			'VDS_ORIENT':	VDS_ORIENT = FIX(VALUE)
			'VDS_MAP':	VDS_MAP	   = FIX(VALUE)
			'TEL_RATE':	TEL_RATE   = VALUE
			'DURATION':	DURATION   = FLOAT(VALUE)
			'USABLE':	USABLE     = VALUE
			ELSE:	BEGIN
				MESSAGE = 'Unrecognized keyword ' + KEYWORD
				GOTO, HANDLE_ERROR
				END
		    ENDCASE
		ENDIF
	ENDWHILE
;
;  Define the output structure.
;
	DEF = { RAS_ID: RAS_ID,		$
		RAS_VAR: RAS_VAR,	$
		RV_DESC: RV_DESC,	$
		EXPTIME: EXPTIME,	$
		LL_ID: LL_ID,		$
		COMP_ID: COMP_ID,	$
		COMP_OPT: COMP_OPT,	$
		DW_ID: DW_ID,		$
		IEF_ID: IEF_ID,		$
		VDS_ORIENT: VDS_ORIENT,	$
		VDS_MAP: VDS_MAP,	$
		TEL_RATE: TEL_RATE,	$
		DURATION: DURATION,	$
		USABLE: USABLE}
	RESULT = 1
	GOTO, FINISH
;
;  Error handling point.
;
HANDLE_ERROR:
	IF N_ELEMENTS(ERRMSG) NE 0 THEN ERRMSG = 'IMP_V_RASTER: ' + MESSAGE $
		ELSE MESSAGE, MESSAGE, /CONTINUE
	RESULT = 0
;
;  Exit point.
;
FINISH:
	RETURN, RESULT
	END
