	FUNCTION IMP_V_STUDY, UNIT, DEF, ERRMSG=ERRMSG
;+
; Project     :	SOHO - CDS
;
; Name        :	IMP_V_STUDY()
;
; Purpose     :	Reads study variation def. from database export file
;
; Explanation :	Reads a study variation definition from a database export file.
;		Called from IMPORT_STUDY.
;
; Use         :	Result = IMP_V_STUDY( UNIT, DEF )
;
;		IF NOT IMP_V_STUDY( UNIT, DEF ) THEN ...
;
; Inputs      :	UNIT = The logical unit number of the opened input file.
;
; Opt. Inputs :	None.
;
; Outputs     :	DEF  = The study definition.  This is a structure of a form
;		       suitable for use with the routine ADD_V_STUDY.
;
; Opt. Outputs:	None.
;
; Keywords    :	ERRMSG	  = If defined and passed, then any error messages will
;			    be returned to the user in this parameter rather
;			    than depending on the MESSAGE routine in IDL.  If
;			    no errors are encountered, then a null string is
;			    returned.  In order to use this feature, ERRMSG
;			    must be defined first, e.g.
;
;				ERRMSG = ''
;				Result = IMP_V_STUDY( ERRMSG=ERRMSG, ... )
;				IF ERRMSG NE '' THEN ...
;
; Calls       :	None.
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Category    :	Planning, Databases.
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 23 May 1995
;
; Modified    :	Version 1, William Thompson, GSFC, 23 May 1995
;
; Version     :	Version 1, 23 May 1995
;-
;
	ON_ERROR, 2
;
;  Check the number of parameters.
;
	IF N_PARAMS() NE 2 THEN BEGIN
		MESSAGE = 'Syntax:  IMP_V_STUDY, UNIT, DEF'
		GOTO, HANDLE_ERROR
	ENDIF
;
;  Set up some default values for various parameters, in case they're not found
;  in the file.
;
	STUDY_ID     = -1
	TITLE_ID     = -1
	STUDYVAR     = -1
	SV_DESC	     = ''
	N_RASTERS0   = 0
	DURATION0    = 0.0
	DURATION1    = 0.0
	ZONE_ID      = 0
	USABLE       = ''
	N_RASTER_DEF = 0
;
;  Keep reading until the first RAS_IND line is found.
;
	KEYWORD = ''
	LINE = ''
	WHILE (KEYWORD NE 'RAS_IND') AND (NOT EOF(UNIT)) DO BEGIN
		READF, UNIT, LINE
;
;  Parse the line to extract the keyword and the value.
;
		SEP = STRPOS(LINE,'=')
		IF (SEP LE 0) OR (SEP GE STRLEN(LINE)) THEN BEGIN
			MESSAGE = 'Line is not in keyword=value format'
			GOTO, HANDLE_ERROR
		ENDIF
		KEYWORD = STRTRIM( STRMID(LINE,0,SEP), 2)
		VALUE   = STRTRIM( STRMID(LINE,SEP+1, (STRLEN(LINE)-SEP-1)), 2)
;
;  Interpret the keyword and value.
;
		CASE STRUPCASE(KEYWORD) OF
			'STUDY_ID':	STUDY_ID     = FIX(VALUE)
			'TITLE_ID':	TITLE_ID     = FIX(VALUE)
			'STUDYVAR':	STUDYVAR     = FIX(VALUE)
			'SV_DESC':	SV_DESC	     = VALUE
			'N_RASTERS0':	N_RASTERS0   = FIX(VALUE)
			'DURATION0':	DURATION0    = FLOAT(VALUE)
			'DURATION1':	DURATION1    = FLOAT(VALUE)
			'ZONE_ID':	ZONE_ID	     = FIX(VALUE)
			'USABLE':	USABLE	     = VALUE
			'N_RASTER_DEF':	N_RASTER_DEF = FIX(VALUE)
			'RAS_IND':	RAS_IND      = FIX(VALUE)
			ELSE:	BEGIN
				MESSAGE = 'Unrecognized keyword ' + KEYWORD
				GOTO, HANDLE_ERROR
				END
		ENDCASE
	ENDWHILE
;
;  If N_RASTER_DEF is negative, then it probably wasn't found.  In any case,
;  it's an error.  (Note: it is legal for N_RASTER_DEF to be zero.)
;
	IF N_RASTER_DEF LT 0 THEN BEGIN
		MESSAGE = 'No rasters found'
		GOTO, HANDLE_ERROR
	ENDIF
;
;  Define the structure for the raster definitions.
;
	RASTERS = {CDS_I_ST_RAS,	$
		RAS_ID: 0,		$
		RAS_VAR: 0,		$
		N_REPEAT_R: 0}
	IF N_RASTER_DEF GT 1 THEN RASTERS = REPLICATE(RASTERS, N_RASTER_DEF)
;
;  Read through the raster definitions.
;
	FOR I = 0,N_RASTER_DEF-1 DO BEGIN
	    KEYWORD = 'Keyword'
	    WHILE (KEYWORD NE 'RAS_IND') AND (STRTRIM(LINE,2) NE '') AND $
			(NOT EOF(UNIT)) DO BEGIN
		READF, UNIT, LINE
;
;  Parse the line to extract the keyword and the value.
;
		IF STRTRIM(LINE) NE '' THEN BEGIN
		    SEP = STRPOS(LINE,'=')
		    IF (SEP LE 0) OR (SEP GE STRLEN(LINE)) THEN BEGIN
			MESSAGE = 'Line is not in keyword=value format'
			GOTO, HANDLE_ERROR
		    ENDIF
		    KEYWORD = STRTRIM( STRMID(LINE,0,SEP), 2)
		    VALUE   = STRTRIM( STRMID(LINE,SEP+1,	$
			    (STRLEN(LINE)-SEP-1)), 2)
;
;  Interpret the keyword and value.
;
		    CASE STRUPCASE(KEYWORD) OF
			'RAS_ID':	RASTERS(I).RAS_ID     = FIX(VALUE)
			'RAS_VAR':	RASTERS(I).RAS_VAR    = FIX(VALUE)
			'N_REPEAT_R':	RASTERS(I).N_REPEAT_R = FIX(VALUE)
			'RAS_IND':	RAS_IND = FIX(VALUE)
			ELSE:	BEGIN
				MESSAGE = 'Unrecognized keyword ' + KEYWORD
				GOTO, HANDLE_ERROR
				END
		    ENDCASE
		ENDIF
	    ENDWHILE
	ENDFOR
;
;  Define the output structure as a whole.
;
;
;  Define the output structure as a whole.
;
	DEF = { STUDY_ID: STUDY_ID,	$
		TITLE_ID: TITLE_ID,	$
		STUDYVAR: STUDYVAR,	$
		SV_DESC: SV_DESC,	$
		N_RASTERS0: N_RASTERS0,	$
		DURATION0: DURATION0,	$
		DURATION1: DURATION1,	$
		ZONE_ID: ZONE_ID,	$
		USABLE: USABLE,		$
		RASTERS: RASTERS}
	RESULT = 1
	GOTO, FINISH
;
;  Error handling point.
;
HANDLE_ERROR:
	IF N_ELEMENTS(ERRMSG) NE 0 THEN ERRMSG = 'IMP_V_STUDY: ' + MESSAGE $
		ELSE MESSAGE, MESSAGE, /CONTINUE
	RESULT = 0
;
;  Exit point.
;
FINISH:
	RETURN, RESULT
	END
