	PRO IMPORT_PLAN, FILENAME, ERRMSG=ERRMSG
;+
; Project     :	SOHO - CDS
;
; Name        :	IMPORT_PLAN
;
; Purpose     :	Imports a science plan file written by EXPORT_PLAN.
;
; Explanation :	This procedure reads a text file written by EXPORT_PLAN, and
;		imports it into the currently selected database.  The text file
;		will contain a complete plan definition, including study
;		definitions.  This allows plan definitions to be moved between
;		different sites, or from a user's personal database to the
;		official CDS database.
;
;		Note that the various ID numbers may change as the studies are
;		moved from one database to another.  This procedure keeps track
;		of all that information, and replaces the old study numbers
;		with the new study numbers
;
; Use         :	IMPORT_PLAN, FILENAME
;
; Inputs      :	FILENAME = The name of the file to read.  The default extension
;			   is ".plan"
;
; Opt. Inputs :	None.
;
; Outputs     :	None.
;
; Opt. Outputs:	None.
;
; Keywords    :	ERRMSG	 = If defined and passed, then any error messages will
;			   be returned to the user in this parameter rather
;			   than depending on the MESSAGE routine in IDL.  If no
;			   errors are encountered, then a null string is
;			   returned.  In order to use this feature, ERRMSG must
;			   be defined first, e.g.
;
;				ERRMSG = ''
;				IMPORT_PLAN, ERRMSG=ERRMSG, ...
;				IF ERRMSG NE '' THEN ...
;
; Calls       :	PRIV_ZDBASE, FIND_WITH_DEF, IMP_LINELIST, IMP_DATAWIN,
;		IMP_F_RASTER, IMP_V_RASTER, IMP_F_STUDY, IMP_V_STUDY
;
; Common      :	None.
;
; Restrictions:	One must have write privilege in the currently selected
;		database.
;
;		!PRIV must be 2 or greater to use this routine.
;
; Side effects:	There are certain assumptions which are made in order for this
;		software to be able to work correctly:
;
;		1.  It is assumed that the GSET database is exactly the same on
;		    both the export and import sides.  There's no checking of
;		    the validity of the GSET_ID number.
;
;		2.  There's no checking of the durations of the various
;		    studies.  In particular, any implied assumptions about
;		    whether or not GSETs need to be loaded are carried over
;		    without checking.
;
;		3.  It is assumed that any engineering studies (STUDY_ID=0) are
;		    defined identically on both the export and import sides.
;
;		4.  Campaign numbers are assumed to have exactly the same
;		    meaning on both the export and import sides.
;
;		This routine does not check to see if the definitions are
;		viable.
;
;		Generally speaking, the various ID numbers and variation
;		indices describing the studies will be completely different in
;		the new database.
;
; Category    :	Planning, Databases.
;
; Prev. Hist. :	None
;
; Written     :	William Thompson, GSFC, 17-Apr-1997
;
; Modified    :	Version 1, William Thompson, GSFC, 17-Apr-1997
;             :	Version 2, Zarro, GSFC, 22-Apr-1997 -- fixed ERRMSG
;		Version 3, 27-May-1997, William Thompson, GSFC
;			Fixed bug when more than one variation of a study in
;			the plan.
;		Version 4, 17-May-1999, William Thompson, GSFC
;			Fixed bug where STUDY_ID changes.
;
; Version     :	Version 4, 17-May-1999
;-
;
	IF !DEBUG EQ 0 THEN ON_ERROR, 2 ELSE ON_ERROR, 0
	MESSAGE = ''
;
;  Check the input parameters.
;
	IF N_PARAMS() NE 1 THEN BEGIN
		MESSAGE = 'Syntax:  IMPORT_PLAN, FILENAME'
		GOTO, HANDLE_ERROR
	ENDIF
;
;  Check the current value of !PRIV.
;
	IF !PRIV LT 2 THEN BEGIN $
           MESSAGE = '!PRIV must be 2 or greater to write into the database'
           GOTO, HANDLE_ERROR
	ENDIF
;
;  Check to see if one has write privilege in the definitions database files.
;
	IF NOT PRIV_ZDBASE(/DEFINITION) THEN BEGIN
		MESSAGE = 'No write privilege for definitions databases.'
		GOTO, HANDLE_ERROR
	ENDIF
;
;  Check to see if one has write privilege in the planning database files.
;
	IF NOT PRIV_ZDBASE(/DAILY) THEN BEGIN
		MESSAGE = 'No write privilege for planning databases.'
		GOTO, HANDLE_ERROR
	ENDIF
;
;  Open up the input file.
;
	FILE = FIND_WITH_DEF(FILENAME, '', '.plan')
	IF FILE EQ '' THEN BEGIN
		MESSAGE = 'Unable to find file ' + FILENAME + '[.plan]'
		GOTO, HANDLE_ERROR
	ENDIF
	OPENR, UNIT, FILE, /GET_LUN
;
;  Read in the header.
;
	LINE = ''
	FIRST = 1
	WHILE (STRTRIM(LINE,2) NE 'END') AND (NOT EOF(UNIT)) DO BEGIN
		READF, UNIT, LINE
		IF FIRST THEN HEADER = LINE ELSE HEADER = [HEADER,LINE]
		FIRST = 0
	ENDWHILE
;
;  If the last line read was not 'END' then signal an error.
;
	IF STRTRIM(LINE,2) NE 'END' THEN BEGIN
		MESSAGE = 'End of file reached before end of header'
		GOTO, HANDLE_ERROR
	ENDIF
;
;  Make sure that the datatype in the header is correct.
;
	DATATYPE = ''
	KEYWORD = STRMID(HEADER,0,8)
	W = WHERE(KEYWORD EQ 'DATATYPE', COUNT)
	IF COUNT GT 0 THEN BEGIN
	    LINE = HEADER(W(0))
	    DATATYPE = TRIM(STRMID(LINE,11,STRLEN(LINE)-11))
	ENDIF
	IF DATATYPE NE 'CDS PLAN DEFINITION' THEN BEGIN
	    MESSAGE = 'File does not contain a CDS plan definition'
	    GOTO, HANDLE_ERROR
	ENDIF
;
;  Extract the start and end times.
;
	W = WHERE(KEYWORD EQ 'STARTIME', COUNT)
	IF COUNT EQ 0 THEN BEGIN
	    MESSAGE = 'Keyword STARTIME not found in header'
	    GOTO, HANDLE_ERROR
	ENDIF
	LINE = HEADER(W(0))
	START_TIME = TRIM(STRMID(LINE,11,STRLEN(LINE)-11))
;
	W = WHERE(KEYWORD EQ 'ENDTIME ', COUNT)
	IF COUNT EQ 0 THEN BEGIN
	    MESSAGE = 'Keyword ENDTIME not found in header'
	    GOTO, HANDLE_ERROR
	ENDIF
	LINE = HEADER(W(0))
	END_TIME = TRIM(STRMID(LINE,11,STRLEN(LINE)-11))
;
;  Delete all the database entries in the specified time range.
;
	LIST_PLAN, START_TIME, END_TIME, PLAN, N_PLAN, INSTRUMENT='CDS', $
		ERRMSG=MESSAGE
	IF TRIM(MESSAGE) NE '' THEN GOTO, HANDLE_ERROR
	FOR I_PLAN = 0, N_PLAN-1 DO IF NOT DEL_PLAN( PLAN(I_PLAN).START_TIME, $
		INSTRUMENT='CDS', ERRMSG=MESSAGE) THEN GOTO, HANDLE_ERROR
;
	LIST_DETAIL, START_TIME, END_TIME, DETAILS, N_DETAILS, ERRMSG=MESSAGE
	IF TRIM(MESSAGE) NE ''  THEN GOTO, HANDLE_ERROR
	FOR I_DETAIL = 0, N_DETAILS-1 DO IF NOT DEL_DETAIL(	$
		DETAILS(I_DETAIL).DATE_OBS, ERRMSG=MESSAGE) THEN	$
		GOTO, HANDLE_ERROR
;
;  Keep track of the old and new ID numbers.
;
	OLD_LL_ID   = -1	&  NEW_LL_ID   = -1
	OLD_DW_ID   = -1	&  NEW_DW_ID   = -1
	OLD_RAS_ID  = -1	&  NEW_RAS_ID  = -1
	OLD_RAS_IDV = -1				;To go with RAS_VAR
	OLD_RAS_VAR = -1	&  NEW_RAS_VAR = -1
	OLD_STUDY_ID  = -1	&  NEW_STUDY_ID  = -1
	OLD_TITLE_ID  = -1	&  NEW_TITLE_ID  = -1
	OLD_STUDY_IDV = -1				;To go with STUDY_VAR
	OLD_STUDY_VAR = -1	&  NEW_STUDY_VAR = -1
;
;  Read in the file item by item until the end is reached.
;
	TYPE = 'Test'
	WHILE TYPE NE '' DO BEGIN
;
;  Keep reading lines from the KAP file until a recognizable item is found.
;
	    TYPE = ''
	    WHILE (TYPE EQ '') AND (NOT EOF(UNIT)) DO BEGIN
		READF, UNIT, TYPE
		TYPE = STRTRIM(TYPE,2)
	    ENDWHILE
;
;  If the item is a linelist description, and hasn't been added yet, then add
;  it to the database system.
;
	    CASE TYPE OF
		'LINELIST': BEGIN
		    IF NOT IMP_LINELIST(UNIT, DEF, ERRMSG=MESSAGE) THEN $
			    GOTO, HANDLE_ERROR
		    W = WHERE(DEF.LL_ID EQ OLD_LL_ID, COUNT)
		    IF COUNT EQ 0 THEN BEGIN
			OLD_LL_ID = [OLD_LL_ID, DEF.LL_ID]
			DEF.LL_ID = -1
;
;  If unable to add it to the database, it may be because it's already in the
;  database.
;
			IF NOT ADD_LINELIST(DEF,LL_ID=LL_ID,ERRMSG=MESSAGE) $
				THEN BEGIN
			    IF LL_ID GT 0 THEN DEF.LL_ID = LL_ID ELSE	$
				    GOTO, HANDLE_ERROR
			ENDIF
;
;  Keep track of the correspondence between the old and new ID numbers.
;
			NEW_LL_ID = [NEW_LL_ID, DEF.LL_ID]
		    ENDIF
		    END
;
;  If the item is a data window list description, and hasn't been added yet,
;  then add it to the database system.  Make sure to convert any linelist ID
;  numbers to their new values first.
;
		'DATAWIN': BEGIN
		    IF NOT IMP_DATAWIN(UNIT, DEF, ERRMSG=MESSAGE) THEN	$
			    GOTO, HANDLE_ERROR
		    IF DEF.LL_ID GT 0 THEN BEGIN
			W = WHERE(DEF.LL_ID EQ OLD_LL_ID)
			DEF.LL_ID = NEW_LL_ID(W(0))
		    ENDIF
;
		    W = WHERE(DEF.DW_ID EQ OLD_DW_ID, COUNT)
		    IF COUNT EQ 0 THEN BEGIN
			OLD_DW_ID = [OLD_DW_ID, DEF.DW_ID]
			DEF.DW_ID = -1
;
			IF NOT ADD_DATAWIN(DEF,DW_ID=DW_ID,ERRMSG=MESSAGE) $
				THEN BEGIN
			    IF DW_ID GT 0 THEN DEF.DW_ID = DW_ID ELSE	$
				    GOTO, HANDLE_ERROR
			ENDIF
;
			NEW_DW_ID = [NEW_DW_ID, DEF.DW_ID]
		    ENDIF
		    END
;
;  If the item is a fundamental raster description, and hasn't been added yet,
;  then add it to the database system.
;
		'RASTER': BEGIN
		    IF NOT IMP_F_RASTER(UNIT, DEF, ERRMSG=MESSAGE) THEN $
			    GOTO, HANDLE_ERROR
		    W = WHERE(DEF.RAS_ID EQ OLD_RAS_ID, COUNT)
		    IF COUNT EQ 0 THEN BEGIN
			OLD_RAS_ID = [OLD_RAS_ID, DEF.RAS_ID]
			DEF.RAS_ID = -1
;
			IF NOT ADD_F_RASTER(DEF,RAS_ID=RAS_ID,ERRMSG=MESSAGE) $
				THEN BEGIN
			    IF RAS_ID GT 0 THEN DEF.RAS_ID = RAS_ID ELSE $
				    GOTO, HANDLE_ERROR
			ENDIF
;
			NEW_RAS_ID = [NEW_RAS_ID, DEF.RAS_ID]
		    ENDIF
		    END
;
;  If the item is a raster variation description, and hasn't been added yet,
;  then add it to the database system.  Make sure to convert any linelist and
;  data window ID numbers to their new values first.
;
		'RASTER_VAR': BEGIN
		    IF NOT IMP_V_RASTER(UNIT, DEF, ERRMSG=MESSAGE) THEN $
			    GOTO, HANDLE_ERROR
		    IF DEF.LL_ID GT 0 THEN BEGIN
			W = WHERE(DEF.LL_ID EQ OLD_LL_ID)
			DEF.LL_ID = NEW_LL_ID(W(0))
		    ENDIF
		    W = WHERE(DEF.DW_ID EQ OLD_DW_ID)
		    DEF.DW_ID = NEW_DW_ID(W(0))
;
;  Also edit the raster variation description to include the new linelist and
;  data window list IDs.
;
		    DEF.RV_DESC = STRMID(DEF.RV_DESC, 0,	$
			    STRPOS(DEF.RV_DESC, 'IDs-(')) + 'IDs-(' +	$
			    TRIM(DEF.LL_ID) + ',' + TRIM(DEF.DW_ID) + ')'
;
		    W = WHERE((DEF.RAS_ID EQ OLD_RAS_IDV) AND	$
			    (DEF.RAS_VAR EQ OLD_RAS_VAR), COUNT)
		    IF COUNT EQ 0 THEN BEGIN
			OLD_RAS_IDV = [OLD_RAS_IDV, DEF.RAS_ID]
			OLD_RAS_VAR = [OLD_RAS_VAR, DEF.RAS_VAR]
;
;  Before adding the raster variation to the database, convert the old raster
;  ID number to the new value.
;
			W = WHERE(DEF.RAS_ID EQ OLD_RAS_ID)
			DEF.RAS_ID = NEW_RAS_ID(W(0))
			DEF.RAS_VAR = -1
;
			IF NOT ADD_V_RASTER(DEF,RASTER=R,ERRMSG=MESSAGE) $
				THEN BEGIN
			    IF R(0) GT 0 THEN DEF.RAS_VAR = R(1) ELSE	$
				    GOTO, HANDLE_ERROR
			ENDIF
;
			NEW_RAS_VAR = [NEW_RAS_VAR, DEF.RAS_VAR]
		    ENDIF
		    END
;
;  If the item is a fundamental study description, and hasn't been added yet,
;  then add it to the database system.  Make sure to convert the old raster ID
;  numbers to their new values.
;
		'STUDY': BEGIN
		    IF NOT IMP_F_STUDY(UNIT, DEF, ERRMSG=MESSAGE) THEN	$
			    GOTO, HANDLE_ERROR
		    W = WHERE((DEF.STUDY_ID EQ OLD_STUDY_ID) AND	$
			    (DEF.TITLE_ID EQ OLD_TITLE_ID), COUNT)
		    IF COUNT EQ 0 THEN BEGIN
			N_RASTER_DEF = N_ELEMENTS(DEF.RASTERS)
			FOR I = 0, N_RASTER_DEF-1 DO BEGIN
			    W = WHERE(DEF.RASTERS(I).RAS_ID EQ OLD_RAS_ID)
			    DEF.RASTERS(I).RAS_ID = NEW_RAS_ID(W(0))
			ENDFOR
;
			OLD_STUDY_ID = [OLD_STUDY_ID, DEF.STUDY_ID]
			OLD_TITLE_ID = [OLD_TITLE_ID, DEF.TITLE_ID]
			DEF.STUDY_ID = -1
			DEF.TITLE_ID = -1
;
;  If the study definition is already in the database, then try adding the
;  title.
;
		        IF NOT ADD_F_STUDY(DEF, STUDY_ID=STUDY_ID,	$
				ERRMSG=MESSAGE) THEN BEGIN
			    IF STUDY_ID LT 0 THEN GOTO, HANDLE_ERROR
			    DEF.STUDY_ID = STUDY_ID
			    IF NOT ADD_T_STUDY(DEF, TITLE_ID=TITLE_ID,	$
				    ERRMSG=MESSAGE) THEN BEGIN
				IF TITLE_ID GT 0 THEN DEF.TITLE_ID = TITLE_ID $
					ELSE GOTO, HANDLE_ERROR
			    ENDIF
			ENDIF
;
		        NEW_STUDY_ID = [NEW_STUDY_ID, DEF.STUDY_ID]
			NEW_TITLE_ID = [NEW_TITLE_ID, DEF.TITLE_ID]
		    ENDIF
		    END
;
;  If the item is a study variation description, and it hasn't been added yet,
;  then add it to the database system.  Make sure to convert all the old ID
;  numbers to their new values.
;
		'STUDY_VAR': BEGIN
		    IF NOT IMP_V_STUDY(UNIT, DEF, ERRMSG=MESSAGE) THEN	$
			    GOTO, HANDLE_ERROR
		    W = WHERE((DEF.STUDY_ID EQ OLD_STUDY_IDV) AND	$
			    (DEF.STUDYVAR EQ OLD_STUDY_VAR), COUNT)
		    IF COUNT EQ 0 THEN BEGIN
			OLD_STUDY_IDV = [OLD_STUDY_IDV, DEF.STUDY_ID]
			OLD_STUDY_VAR = [OLD_STUDY_VAR, DEF.STUDYVAR]
;
			W = WHERE((DEF.STUDY_ID EQ OLD_STUDY_ID) AND	$
				(DEF.TITLE_ID EQ OLD_TITLE_ID), COUNT)
			DEF.STUDY_ID = NEW_STUDY_ID(W(0))
			DEF.TITLE_ID = NEW_TITLE_ID(W(0))
			N_RASTER_DEF = N_ELEMENTS(DEF.RASTERS)
			FOR I = 0, N_RASTER_DEF-1 DO BEGIN
			    W = WHERE((DEF.RASTERS(I).RAS_ID EQ OLD_RAS_IDV) $
				   AND (DEF.RASTERS(I).RAS_VAR EQ OLD_RAS_VAR))
			    DEF.RASTERS(I).RAS_VAR = NEW_RAS_VAR(W(0))
			    W = WHERE(DEF.RASTERS(I).RAS_ID EQ OLD_RAS_ID)
			    DEF.RASTERS(I).RAS_ID = NEW_RAS_ID(W(0))
		        ENDFOR
;
			DEF.STUDYVAR = -1
;
			IF NOT ADD_V_STUDY(DEF, STUDY=S, ERRMSG=MESSAGE) $
				THEN BEGIN
			   IF S(0) GT 0 THEN DEF.STUDYVAR = S(1) ELSE	$
				    GOTO, HANDLE_ERROR
			ENDIF
;
		        NEW_STUDY_VAR = [NEW_STUDY_VAR, DEF.STUDYVAR]
		    ENDIF
		    END
;
;  If the item is a science plan, then add it to the database system.
;
		'SCI_PLAN': BEGIN
		    IF NOT IMP_SCI_PLAN(UNIT, DEF, ERRMSG=MESSAGE) THEN	$
			    GOTO, HANDLE_ERROR
		    IF NOT ADD_PLAN(DEF, ERRMSG=MESSAGE) THEN	$
			    GOTO, HANDLE_ERROR
		    END
;
;  If the item is a detailed science plan, then add it to the database system.
;  Change the study ID and variation index.
;
		'PROGRAM': BEGIN
		    IF NOT IMP_DETAIL(UNIT, DEF, ERRMSG=MESSAGE) THEN	$
			    GOTO, HANDLE_ERROR
		    IF DEF.STUDY_ID NE 0 THEN BEGIN
			SAVE_STUDY_ID  = DEF.STUDY_ID
			SAVE_STUDY_VAR = DEF.STUDYVAR
			W = WHERE(DEF.STUDY_ID EQ OLD_STUDY_ID, COUNT)
			DEF.STUDY_ID = NEW_STUDY_ID(W(0))
			W = WHERE((SAVE_STUDY_ID EQ OLD_STUDY_IDV) AND	$
				(DEF.STUDYVAR EQ OLD_STUDY_VAR), COUNT)
			DEF.STUDYVAR = NEW_STUDY_VAR(W(0))
;
;  If the SCI_SPEC parameter is the study mnemonic plus the variation index,
;  then change the variation index.
;
			GET_STUDY, DEF.STUDY_ID, DEF.STUDYVAR, STUDY
			TEST = TRIM(STUDY.OBS_PROG) + '/v' +	$
				TRIM(SAVE_STUDY_VAR)
			IF DEF.SCI_SPEC EQ TEST THEN DEF.SCI_SPEC = $
				TRIM(STUDY.OBS_PROG) + '/v' +	$
				TRIM(DEF.STUDYVAR)
		    ENDIF
		    IF NOT ADD_DETAIL(DEF, ERRMSG=MESSAGE) THEN	$
			    GOTO, HANDLE_ERROR
		    END
;
;  Otherwise, if the item type has not been recognized yet, and isn't the null
;  string, then an error must have occured.
;
		ELSE: IF TYPE NE '' THEN BEGIN
		    MESSAGE = 'Unable to recognize item ' + TYPE
		    GOTO, HANDLE_ERROR
		    END
;
	    ENDCASE
	ENDWHILE
	GOTO, FINISH
;
;  Error handling point.
;
HANDLE_ERROR:

        IF DATATYPE(ERRMSG) NE 'STR' THEN BEGIN
         IF TRIM(MESSAGE) NE '' THEN MESSAGE, MESSAGE, /CONTINUE 
        ENDIF ELSE BEGIN
         IF TRIM(MESSAGE) NE '' THEN ERRMSG = 'IMPORT_PLAN: ' + MESSAGE 
        ENDELSE
	STUDY = {STUDY_ID: -1}
;
;  Exit point.
;
FINISH:
	IF N_ELEMENTS(UNIT) NE 0 THEN FREE_LUN, UNIT
;
	RETURN
	END
