	PRO IMPORT_STUDY, FILENAME, STUDY, ERRMSG=ERRMSG
;+
; Project     :	SOHO - CDS
;
; Name        :	IMPORT_STUDY
;
; Purpose     :	Imports a study definition file written by EXPORT_STUDY.
;
; Explanation :	This procedure reads a text file written by EXPORT_STUDY, and
;		imports it into the currently selected database.  The text file
;		will contain a complete study definition, including linelists
;		and data window lists.  This allows study definitions to be
;		moved between different sites, or from a user's personal
;		database to the official CDS database.
;
;		Note that the various ID numbers may change as the study is
;		moved from one database to another.  This procedure keeps track
;		of all that information, and replaces the old study numbers
;		with the new study numbers
;
; Use         :	IMPORT_STUDY, FILENAME, STUDY
;
; Inputs      :	FILENAME = The name of the file to read.  The default extension
;			   is ".def"
;
; Opt. Inputs :	None.
;
; Outputs     :	STUDY    = The study definition as returned by GET_STUDY once
;			   it has been imported into the new database.  If
;			   unable to import the study, then a simpler structure
;			   is returned, with the single tag STUDY_ID set to -1.
;
; Opt. Outputs:	None.
;
; Keywords    :	ERRMSG	 = If defined and passed, then any error messages will
;			   be returned to the user in this parameter rather
;			   than depending on the MESSAGE routine in IDL.  If no
;			   errors are encountered, then a null string is
;			   returned.  In order to use this feature, ERRMSG must
;			   be defined first, e.g.
;
;				ERRMSG = ''
;				IMPORT_STUDY, ERRMSG=ERRMSG, ...
;				IF ERRMSG NE '' THEN ...
;
; Calls       :	PRIV_ZDBASE, FIND_WITH_DEF, IMP_LINELIST, IMP_DATAWIN,
;		IMP_F_RASTER, IMP_V_RASTER, IMP_F_STUDY, IMP_V_STUDY
;
; Common      :	None.
;
; Restrictions:	One must have write privilege in the currently selected
;		database.
;
;		!PRIV must be 2 or greater to use this routine.
;
; Side effects:	This routine does not check to see if the definitions are
;		viable.
;
;		Generally speaking, the various ID numbers and variation
;		indices describing the study will be completely different in
;		the new database.
;
; Category    :	Planning, Databases.
;
; Prev. Hist. :	None
;
; Written     :	William Thompson, GSFC, 17 May 1995
;
; Modified    :	Version 1, William Thompson, GSFC, 23 May 1995
;		Version 2, William Thompson, GSFC, 15 January 1996
;			Modify RV_DESC to reflect new ID numbers
;
; Version     :	Version 2, 15 January 1996
;-
;
	ON_ERROR, 2
	MESSAGE = ''
;
;  Check the input parameters.
;
	IF N_PARAMS() NE 2 THEN BEGIN
		MESSAGE = 'Syntax:  IMPORT_STUDY, FILENAME, STUDY'
		GOTO, HANDLE_ERROR
	ENDIF
;
;  Check the current value of !PRIV.
;
	IF !PRIV LT 2 THEN BEGIN $
           MESSAGE = '!PRIV must be 2 or greater to write into the database'
           GOTO, HANDLE_ERROR
	ENDIF
;
;  Check to see if one has write privilege in the definitions database files.
;
	IF NOT PRIV_ZDBASE(/DEFINITION) THEN BEGIN
		MESSAGE = 'No write privilege for definitions databases.'
		GOTO, HANDLE_ERROR
	ENDIF
;
;  Open up the input file.
;
	FILE = FIND_WITH_DEF(FILENAME, '', '.def')
	IF FILE EQ '' THEN BEGIN
		MESSAGE = 'Unable to find file ' + FILENAME + '[.def]'
		GOTO, HANDLE_ERROR
	ENDIF
	OPENR, UNIT, FILE, /GET_LUN
;
;  Read in the header.
;
	LINE = ''
	FIRST = 1
	WHILE (STRTRIM(LINE,2) NE 'END') AND (NOT EOF(UNIT)) DO BEGIN
		READF, UNIT, LINE
		IF FIRST THEN HEADER = LINE ELSE HEADER = [HEADER,LINE]
		FIRST = 0
	ENDWHILE
;
;  If the last line read was not 'END' then signal an error.
;
	IF STRTRIM(LINE,2) NE 'END' THEN BEGIN
		MESSAGE = 'End of file reached before end of header'
		GOTO, HANDLE_ERROR
	ENDIF
;
;  Keep track of the old and new ID numbers.
;
	OLD_LL_ID   = -1	&  NEW_LL_ID   = -1
	OLD_DW_ID   = -1	&  NEW_DW_ID   = -1
	OLD_RAS_ID  = -1	&  NEW_RAS_ID  = -1
	OLD_RAS_IDV = -1				;To go with RAS_VAR
	OLD_RAS_VAR = -1	&  NEW_RAS_VAR = -1
;
;  Read in the file item by item until the end is reached.
;
	TYPE = 'Test'
	WHILE TYPE NE '' DO BEGIN
;
;  Keep reading lines from the KAP file until a recognizable item is found.
;
	    TYPE = ''
	    WHILE (TYPE EQ '') AND (NOT EOF(UNIT)) DO BEGIN
		READF, UNIT, TYPE
		TYPE = STRTRIM(TYPE,2)
	    ENDWHILE
;
;  If the item is a linelist description, and hasn't been added yet, then add
;  it to the database system.
;
	    CASE TYPE OF
		'LINELIST': BEGIN
		    IF NOT IMP_LINELIST(UNIT, DEF, ERRMSG=MESSAGE) THEN $
			    GOTO, HANDLE_ERROR
		    W = WHERE(DEF.LL_ID EQ OLD_LL_ID, COUNT)
		    IF COUNT EQ 0 THEN BEGIN
			OLD_LL_ID = [OLD_LL_ID, DEF.LL_ID]
			DEF.LL_ID = -1
;
;  If unable to add it to the database, it may be because it's already in the
;  database.
;
			IF NOT ADD_LINELIST(DEF,LL_ID=LL_ID,ERRMSG=MESSAGE) $
				THEN BEGIN
			    IF LL_ID GT 0 THEN DEF.LL_ID = LL_ID ELSE	$
				    GOTO, HANDLE_ERROR
			ENDIF
;
;  Keep track of the correspondence between the old and new ID numbers.
;
			NEW_LL_ID = [NEW_LL_ID, DEF.LL_ID]
		    ENDIF
		    END
;
;  If the item is a data window list description, and hasn't been added yet,
;  then add it to the database system.  Make sure to convert any linelist ID
;  numbers to their new values first.
;
		'DATAWIN': BEGIN
		    IF NOT IMP_DATAWIN(UNIT, DEF, ERRMSG=MESSAGE) THEN	$
			    GOTO, HANDLE_ERROR
		    IF DEF.LL_ID GT 0 THEN BEGIN
			W = WHERE(DEF.LL_ID EQ OLD_LL_ID)
			DEF.LL_ID = NEW_LL_ID(W(0))
		    ENDIF
;
		    W = WHERE(DEF.DW_ID EQ OLD_DW_ID, COUNT)
		    IF COUNT EQ 0 THEN BEGIN
			OLD_DW_ID = [OLD_DW_ID, DEF.DW_ID]
			DEF.DW_ID = -1
;
			IF NOT ADD_DATAWIN(DEF,DW_ID=DW_ID,ERRMSG=MESSAGE) $
				THEN BEGIN
			    IF DW_ID GT 0 THEN DEF.DW_ID = DW_ID ELSE	$
				    GOTO, HANDLE_ERROR
			ENDIF
;
			NEW_DW_ID = [NEW_DW_ID, DEF.DW_ID]
		    ENDIF
		    END
;
;  If the item is a fundamental raster description, and hasn't been added yet,
;  then add it to the database system.
;
		'RASTER': BEGIN
		    IF NOT IMP_F_RASTER(UNIT, DEF, ERRMSG=MESSAGE) THEN $
			    GOTO, HANDLE_ERROR
		    W = WHERE(DEF.RAS_ID EQ OLD_RAS_ID, COUNT)
		    IF COUNT EQ 0 THEN BEGIN
			OLD_RAS_ID = [OLD_RAS_ID, DEF.RAS_ID]
			DEF.RAS_ID = -1
;
			IF NOT ADD_F_RASTER(DEF,RAS_ID=RAS_ID,ERRMSG=MESSAGE) $
				THEN BEGIN
			    IF RAS_ID GT 0 THEN DEF.RAS_ID = RAS_ID ELSE $
				    GOTO, HANDLE_ERROR
			ENDIF
;
			NEW_RAS_ID = [NEW_RAS_ID, DEF.RAS_ID]
		    ENDIF
		    END
;
;  If the item is a raster variation description, and hasn't been added yet,
;  then add it to the database system.  Make sure to convert any linelist and
;  data window ID numbers to their new values first.
;
		'RASTER_VAR': BEGIN
		    IF NOT IMP_V_RASTER(UNIT, DEF, ERRMSG=MESSAGE) THEN $
			    GOTO, HANDLE_ERROR
		    IF DEF.LL_ID GT 0 THEN BEGIN
			W = WHERE(DEF.LL_ID EQ OLD_LL_ID)
			DEF.LL_ID = NEW_LL_ID(W(0))
		    ENDIF
		    W = WHERE(DEF.DW_ID EQ OLD_DW_ID)
		    DEF.DW_ID = NEW_DW_ID(W(0))
;
;  Also edit the raster variation description to include the new linelist and
;  data window list IDs.
;
		    DEF.RV_DESC = STRMID(DEF.RV_DESC, 0,	$
			    STRPOS(DEF.RV_DESC, 'IDs-(')) + 'IDs-(' +	$
			    TRIM(DEF.LL_ID) + ',' + TRIM(DEF.DW_ID) + ')'
;
		    W = WHERE((DEF.RAS_ID EQ OLD_RAS_IDV) AND	$
			    (DEF.RAS_VAR EQ OLD_RAS_VAR), COUNT)
		    IF COUNT EQ 0 THEN BEGIN
			OLD_RAS_IDV = [OLD_RAS_IDV, DEF.RAS_ID]
			OLD_RAS_VAR = [OLD_RAS_VAR, DEF.RAS_VAR]
;
;  Before adding the raster variation to the database, convert the old raster
;  ID number to the new value.
;
			W = WHERE(DEF.RAS_ID EQ OLD_RAS_ID)
			DEF.RAS_ID = NEW_RAS_ID(W(0))
			DEF.RAS_VAR = -1
;
			IF NOT ADD_V_RASTER(DEF,RASTER=R,ERRMSG=MESSAGE) $
				THEN BEGIN
			    IF R(0) GT 0 THEN DEF.RAS_VAR = R(1) ELSE	$
				    GOTO, HANDLE_ERROR
			ENDIF
;
			NEW_RAS_VAR = [NEW_RAS_VAR, DEF.RAS_VAR]
		    ENDIF
		    END
;
;  If the item is a fundamental study description, then add it to the database
;  system.  It is assumed that there is only one of these per file.  Make sure
;  to convert the old raster ID numbers to their new values.
;
		'STUDY': BEGIN
		    IF NOT IMP_F_STUDY(UNIT, DEF, ERRMSG=MESSAGE) THEN	$
			    GOTO, HANDLE_ERROR
		    N_RASTER_DEF = N_ELEMENTS(DEF.RASTERS)
		    FOR I = 0, N_RASTER_DEF-1 DO BEGIN
			W = WHERE(DEF.RASTERS(I).RAS_ID EQ OLD_RAS_ID)
			DEF.RASTERS(I).RAS_ID = NEW_RAS_ID(W(0))
		    ENDFOR
;
		    DEF.STUDY_ID = -1
		    DEF.TITLE_ID = -1
;
;  If the study definition is already in the database, then try adding the
;  title.
;
		    IF NOT ADD_F_STUDY(DEF, STUDY_ID=STUDY_ID,		$
			    ERRMSG=MESSAGE) THEN BEGIN
			IF STUDY_ID LT 0 THEN GOTO, HANDLE_ERROR
			DEF.STUDY_ID = STUDY_ID
			IF NOT ADD_T_STUDY(DEF, TITLE_ID=TITLE_ID,	$
				ERRMSG=MESSAGE) THEN BEGIN
			    IF TITLE_ID GT 0 THEN DEF.TITLE_ID = TITLE_ID $
				    ELSE GOTO, HANDLE_ERROR
			ENDIF
		    ENDIF
;
		    NEW_STUDY_ID = DEF.STUDY_ID
		    NEW_TITLE_ID = DEF.TITLE_ID
		    END
;
;  If the item is a study variation description, then add it to the database
;  system.  It is assumed that there is only one of these per file.  Make sure
;  to convert all the old ID numbers to their new values.
;
		'STUDY_VAR': BEGIN
		    IF NOT IMP_V_STUDY(UNIT, DEF, ERRMSG=MESSAGE) THEN	$
			    GOTO, HANDLE_ERROR
		    DEF.STUDY_ID = NEW_STUDY_ID
		    DEF.TITLE_ID = NEW_TITLE_ID
		    N_RASTER_DEF = N_ELEMENTS(DEF.RASTERS)
		    FOR I = 0, N_RASTER_DEF-1 DO BEGIN
			W = WHERE((DEF.RASTERS(I).RAS_ID EQ OLD_RAS_IDV) AND $
				(DEF.RASTERS(I).RAS_VAR EQ OLD_RAS_VAR))
			DEF.RASTERS(I).RAS_VAR = NEW_RAS_VAR(W(0))
			W = WHERE(DEF.RASTERS(I).RAS_ID EQ OLD_RAS_ID)
			DEF.RASTERS(I).RAS_ID = NEW_RAS_ID(W(0))
		    ENDFOR
;
		    DEF.STUDYVAR = -1
;
		    IF NOT ADD_V_STUDY(DEF, STUDY=S, ERRMSG=MESSAGE) THEN BEGIN
			IF S(0) GT 0 THEN DEF.STUDYVAR = S(1) ELSE	$
				GOTO, HANDLE_ERROR
		    ENDIF
;
		    NEW_STUDYVAR = DEF.STUDYVAR
		    END
;
;  Otherwise, if the item type has not been recognized yet, and isn't the null
;  string, then an error must have occured.
;
		ELSE: IF TYPE NE '' THEN BEGIN
		    MESSAGE = 'Unable to recognize item ' + TYPE
		    GOTO, HANDLE_ERROR
		    END
;
	    ENDCASE
	ENDWHILE
;
;  Retrieve the study definition as stored in the database.
;
	GET_STUDY, NEW_STUDY_ID, NEW_STUDYVAR, STUDY
	GOTO, FINISH
;
;  Error handling point.
;
HANDLE_ERROR:
	IF N_ELEMENTS(ERRMSG) NE 0 THEN ERRMSG = 'IMPORT_STUDY: ' + MESSAGE $
		ELSE MESSAGE, MESSAGE, /CONTINUE
	STUDY = {STUDY_ID: -1}
;
;  Exit point.
;
FINISH:
	IF N_ELEMENTS(UNIT) NE 0 THEN FREE_LUN, UNIT
;
	RETURN
	END
