	PRO LIST_ALT, START, EEND, PERIODS, N_FOUND, ERRMSG=ERRMSG
;+
; Project     :	SOHO - CDS
;
; Name        :	LIST_ALT
;
; Purpose     :	List the CDS alternate science plan for a date range.
;
; Explanation :	Extracts all the CDS alternate science plan entries which
;		intersect an input time range.
;
; Use         :	LIST_ALT, START, END, PERIODS, N_FOUND
;
; Inputs      :	START, END = The range of date/time values to use in searching
;			     the database.  Can be in any standard CDS time
;			     format.
;
; Opt. Inputs :	None.
;
; Outputs     :	PERIODS	= A structure variable containing the following tags
;			  for each alternate science plan entry:
;
;			STRUCT_TYPE= The character string 'CDS-ALT-LIST'
;			PROG_ID	   = Program ID, linking one or more studies
;				     together
;			STUDY_ID   = Number defining the study
;			STUDYVAR   = Number giving the study variation ID
;			SCI_OBJ    = Science objective from the daily science
;				     meeting
;			SCI_SPEC   = Specific science objective from meeting
;			CMP_NO	   = Campaign number
;			OBJECT	   = Code for object planned to be observed
;			OBJ_ID	   = Object identification
;			START_TIME = Date/time of beginning of alternate plan,
;				     in TAI format
;			END_TIME   = Date/time of end of alternate plan, in
;				     TAI format
;			ORIG_DUR   = Original duration of the observation, in
;				     seconds, before being truncated by
;				     following time-tagged observations.
;			N_RASTERS1 = Variable number of rasters parameter
;			TRACKING   = True (1) if feature tracking to be used,
;				     or false (0) otherwise
;			N_POINTINGS= Number of pointings to use with study.
;			N_REPEAT_S = Number of times to repeat study
;			GSET_ID	   = GSET ID number
;			GET_RAW	   = True if raw data should be collected.
;
;		N_FOUND	= Number of periods found.
;
; Opt. Outputs:	None.
;
; Keywords    :
;       ERRMSG    = If defined and passed, then any error messages will be
;                   returned to the user in this parameter rather than
;                   depending on the MESSAGE routine in IDL.  If no errors are
;                   encountered, then a null string is returned.  In order to
;                   use this feature, ERRMSG must be defined first, e.g.
;
;                       ERRMSG = ''
;                       LIST_ALT, ERRMSG=ERRMSG, ... 
;                       IF ERRMSG NE '' THEN ...
;
;
; Calls       :	DATATYPE, DBOPEN, DBFIND, DBEXT, DBCLOSE, TRIM
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	If the number of periods found is zero, then the output
;		parameter PERIODS is not modified.
;
; Category    :	Planning, Database.
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 9 May 1995
;
; Modified    :	Version 1, William Thompson, GSFC, 9 May 1995
;		Version 2, William Thompson, GSFC, 28 September 1995
;			Added tags ORIG_DUR, GSET_ID, GET_RAW
;
; Version     :	Version 2, 28 September 1995
;-
;
	ON_ERROR, 2
;
;  Check the number of parameters.
;
	N_FOUND = 0
        IF N_PARAMS() LT 4 THEN BEGIN
           MESSAGE = 'Syntax:  LIST_ALT, START, END, PERIODS, N_FOUND'
	   GOTO, HANDLE_ERROR
        ENDIF
;
;  Check the input parameters.
;
	IF N_ELEMENTS(START) NE 1 THEN BEGIN
           MESSAGE = 'START must be a scalar'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(EEND) NE 1 THEN BEGIN
           MESSAGE = 'END must be a scalar'
           GOTO, HANDLE_ERROR
	ENDIF
;
;  Convert the input dates to TAI format.
;
	IF DATATYPE(START,1) EQ 'Double' THEN TAI_START = START ELSE	$
		TAI_START = UTC2TAI(START)
	IF DATATYPE(EEND,1) EQ 'Double' THEN TAI_END = EEND ELSE	$
		TAI_END = UTC2TAI(EEND)
;
;  Open the alternate science plan database.
;
	DBOPEN, 'sci_alt'
;
;  Find all the entries in the requested time range.
;
	ENTRIES = DBFIND('END_TIME>' + TRIM(TAI_START,'(F15.3)') +	$
		',START_TIME<' + TRIM(TAI_END,'(F15.3)') + ',DELETED=N', $
		/SILENT)
	IF !ERR EQ 0 THEN BEGIN
		N_FOUND = 0
		GOTO, FINISH
	END ELSE BEGIN
		ENTRIES = ENTRIES(UNIQ(ENTRIES))
		N_FOUND = N_ELEMENTS(ENTRIES)
	ENDELSE
;
;  Extract the requested entries, sorted by start times.
;
	ENTRIES = DBSORT(ENTRIES,'start_time')
	DBEXT, ENTRIES, 'prog_id,study_id,studyvar,sci_obj,sci_spec,cmp_no', $
		PROG_ID, STUDY_ID, STUDYVAR, SCI_OBJ, SCI_SPEC, CMP_NO
	DBEXT, ENTRIES, 'object,obj_id,start_time,end_time,n_rasters1', $
		OBJECT, OBJ_ID, START_TIME, END_TIME, N_RASTERS1
	DBEXT, ENTRIES, 'tracking,n_pointings,n_repeat_s', $
		TRACKING, N_POINTINGS, N_REPEAT_S
	DBEXT, ENTRIES, 'orig_dur,gset_id,get_raw', ORIG_DUR, GSET_ID, GET_RAW
;
;  Remove any entries that only touch the requested time range.
;
	W = WHERE((END_TIME NE TAI_START) AND (START_TIME NE TAI_END),	$
		N_FOUND)
	IF N_FOUND EQ 0 THEN GOTO, FINISH
;
;  Define the structure that the data will be returned in.
;
	PERIODS = {STRUCT_TYPE: 'CDS-ALT-LIST',	$
		PROG_ID: 0,		$
		STUDY_ID: 0,		$
		STUDYVAR: 0,		$
		SCI_OBJ: 'String',	$
		SCI_SPEC: 'String',	$
		CMP_NO: 0,		$
		OBJECT: 'String',	$
		OBJ_ID: 'String',	$
		START_TIME: 0.0D0,	$
		END_TIME: 0.0D0,	$
		ORIG_DUR: 0.0D0,	$
		N_RASTERS1: 0,		$
		TRACKING: 0B,		$
		N_POINTINGS: 0,		$
		N_REPEAT_S: 0,		$
		GSET_ID: 0,		$
		GET_RAW: 0B}
;
	IF N_FOUND EQ 1 THEN BEGIN
		PERIODS.PROG_ID	   = PROG_ID(W(0))
		PERIODS.STUDY_ID   = STUDY_ID(W(0))
		PERIODS.STUDYVAR   = STUDYVAR(W(0))
		PERIODS.SCI_OBJ	   = SCI_OBJ(W(0))
		PERIODS.SCI_SPEC   = SCI_SPEC(W(0))
		PERIODS.CMP_NO	   = CMP_NO(W(0))
		PERIODS.OBJECT	   = OBJECT(W(0))
		PERIODS.OBJ_ID	   = OBJ_ID(W(0))
		PERIODS.START_TIME = START_TIME(W(0))
		PERIODS.END_TIME   = END_TIME(W(0))
		PERIODS.ORIG_DUR   = ORIG_DUR(W(0))
		PERIODS.N_RASTERS1 = N_RASTERS1(W(0))
		PERIODS.TRACKING   = TRACKING(W(0))
		PERIODS.N_POINTINGS= N_POINTINGS(W(0))
		PERIODS.N_REPEAT_S = N_REPEAT_S(W(0))
		PERIODS.GSET_ID	   = GSET_ID(W(0))
		PERIODS.GET_RAW	   = GET_RAW(W(0))
	END ELSE BEGIN
		PERIODS = REPLICATE(PERIODS, N_FOUND)
		PERIODS.PROG_ID	   = PROG_ID(W)
		PERIODS.STUDY_ID   = STUDY_ID(W)
		PERIODS.STUDYVAR   = STUDYVAR(W)
		PERIODS.SCI_OBJ	   = SCI_OBJ(W)
		PERIODS.SCI_SPEC   = SCI_SPEC(W)
		PERIODS.CMP_NO	   = CMP_NO(W)
		PERIODS.OBJECT	   = OBJECT(W)
		PERIODS.OBJ_ID	   = OBJ_ID(W)
		PERIODS.START_TIME = START_TIME(W)
		PERIODS.END_TIME   = END_TIME(W)
		PERIODS.ORIG_DUR   = ORIG_DUR(W)
		PERIODS.N_RASTERS1 = N_RASTERS1(W)
		PERIODS.TRACKING   = TRACKING(W)
		PERIODS.N_POINTINGS= N_POINTINGS(W)
		PERIODS.N_REPEAT_S = N_REPEAT_S(W)
		PERIODS.GSET_ID	   = GSET_ID(W)
		PERIODS.GET_RAW	   = GET_RAW(W)
	ENDELSE
	GOTO, FINISH
;
;  Error handling point.
;
HANDLE_ERROR:
	IF N_ELEMENTS(ERRMSG) NE 0 THEN ERRMSG = 'LIST_ALT: ' + MESSAGE $
		ELSE MESSAGE, MESSAGE, /CONTINUE
;
;  Close the database and return.
;
FINISH:
	DBCLOSE
;
	RETURN
	END
