	PRO LIST_CDROM, P1, P2, LIST, N_FOUND, ERRMSG=ERRMSG, PRINT=PRNT, $
		OUTPUT=OUTPUT
;+
; Project     :	SOHO - CDS
;
; Name        :	LIST_CDROM
;
; Purpose     :	List the contents of CDS level-0 CDROMs
;
; Category    :	Class4, Operations, Database, Catalog
;
; Explanation :	Searches the CDROM database for information about what dates
;		are on which level-0 CDROM.  Two kinds of requests can be made.
;		You can ask which dates are on a given CDROM by giving the
;		sequence and volume number.  Alternatively, you can give a
;		range of dates, and the software will return the CDROMS that
;		contain those dates.
;
; Syntax      :	LIST_CDROM, SEQUENCE, VOLUME, LIST, N_FOUND
;		LIST_CDROM, START, END, LIST, N_FOUND
;
; Examples    :	LIST_CDROM, 2, 4, LIST, N_FOUND
;		LIST_CDROM, '1996-01-10', '1996-01-15', LIST, N_FOUND
;
; Inputs      :	One of the two following pairs of input parameters must be
;		passed.
;
;		SEQUENCE = The sequence number of the CDROM.  For example, if
;			   the CDROM is identified as SO_CDS_LZ_0002, then its
;			   sequence number is 2.
;		VOLUME	 = The volume number of the CDROM.  For example, if the
;			   CDROM is identified as "4 of 9", then the volume
;			   number is 4.
;
;		or
;
;		START	 = A starting date to search for.
;		END	 = An end date to search for.  The start and end dates
;			   can be in any standard CDS time format.
;
; Opt. Inputs :	None.
;
; Outputs     :	LIST	 = A list of entries.  This is a structure with the
;			   following tags:
;
;				SEQUENCE = The sequence number
;				VOLUME	 = The volume number
;				DATE	 = The date, e.g. "1995-12-14"
;
;		N_FOUND	 = The number of entries found.
;
; Opt. Outputs:	None.
;
; Keywords    :	PRINT  = If set, then the entries found are printed to the
;			 terminal.
;
;		OUTPUT = When used with /PRINT, specifies a filename to write
;			 the output to instead of writing to the terminal.
;
;		ERRMSG = If defined and passed, then any error messages will be
;			 returned to the user in this parameter rather than
;			 depending on the MESSAGE routine in IDL.  If no errors
;			 are encountered, then a null string is returned.  In
;			 order to use this feature, ERRMSG must be defined
;			 first, e.g.
;
;				ERRMSG = ''
;				LIST_CDROM, ERRMSG=ERRMSG, ...
;				IF ERRMSG NE '' THEN ...
;
; Calls       :	
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	If the number of CDROM entries found is zero, then the output
;		parameter LIST is not modified.
;
; Prev. Hist. :	None.
;
; History     :	Version 1, 23-Apr-1996, William Thompson, GSFC
;		Version 2, 24-Apr-1996, William Thompson, GSFC
;			Added keywords PRINT and OUTPUT
;		Version 3, 25-Apr-1997, William Thompson, GSFC
;			Changed way printing is done to terminal, to correct
;			problem with IDL 4.0
;
; Contact     :	WTHOMPSON
;-
;
	ON_ERROR, 2
	N_FOUND = 0
;
;  Check the number of parameters.
;
	IF N_PARAMS() NE 4 THEN BEGIN
	    MESSAGE = 'Syntax: LIST_CDROM, SEQ, VOL or LIST_CDROM, START, END'
	    GOTO, HANDLE_ERROR
	ENDIF
;
;  If the parameters are integers, then the sequence number and volume were
;  passed.
;
	IF (DATATYPE(P1,2) LE 3) AND (DATATYPE(P2,2) LE 3) AND	$
		(N_ELEMENTS(P1) EQ 1) AND (N_ELEMENTS(P2) EQ 1) THEN BEGIN
	    SEQUENCE = P1
	    VOLUME = P2
;
;  Otherwise times were passed.
;
	END ELSE BEGIN
	    MESSAGE = ''
	    START_TIME = ANYTIM2UTC(P1, ERRMSG=MESSAGE)
	    IF MESSAGE NE '' THEN GOTO, HANDLE_ERROR
	    START_TIME = START_TIME.MJD
	    END_TIME = ANYTIM2UTC(P2, ERRMSG=MESSAGE)
	    IF MESSAGE NE '' THEN GOTO, HANDLE_ERROR
	    END_TIME = END_TIME.MJD
	ENDELSE
;
;  Open the CDROM database.
;
	DBOPEN, 'cdrom'
;
;  Depending on what was requested, search for the relevant entries.
;
	IF N_ELEMENTS(SEQUENCE) EQ 1 THEN BEGIN
		ENTRIES = DBFIND('SEQUENCE=' + TRIM(SEQUENCE) + ',VOLUME=' + $
			TRIM(VOLUME), /SILENT)
	END ELSE BEGIN
		ENTRIES = DBFIND('MJD>'+TRIM(START_TIME) + ',MJD<' +	$
			TRIM(END_TIME), /SILENT)
	ENDELSE
;
	IF !ERR EQ 0 THEN BEGIN
		N_FOUND = 0
		GOTO, FINISH
	END ELSE BEGIN
		ENTRIES = ENTRIES(UNIQ(ENTRIES))
		N_FOUND = N_ELEMENTS(ENTRIES)
	ENDELSE
;
;  Extract the requested entries.
;
	ENTRIES = DBSORT(ENTRIES, 'sequence,volume,mjd')
	DBEXT, ENTRIES, 'sequence,volume,mjd', SEQUENCE, VOLUME, MJD
;
;  Form a structure to contain the data.
;
	LIST = {SEQUENCE: 0L, VOLUME: 0L, DATE: 'String'}
	DATE = {MJD: 0L, TIME: 0L}
	IF N_FOUND EQ 1 THEN BEGIN
		DATE.MJD = MJD(0)
		LIST.SEQUENCE = SEQUENCE(0)
		LIST.VOLUME   = VOLUME(0)
		LIST.DATE     = UTC2STR(DATE, /DATE_ONLY)
	END ELSE BEGIN
		LIST = REPLICATE(LIST, N_FOUND)
		DATE = REPLICATE(DATE, N_FOUND)
		DATE.MJD = MJD
		LIST.SEQUENCE = SEQUENCE
		LIST.VOLUME   = VOLUME
		LIST.DATE     = UTC2STR(DATE, /DATE_ONLY)
	ENDELSE
	GOTO, FINISH
;
;  Error handling point.
;
HANDLE_ERROR:
	IF N_ELEMENTS(ERRMSG) NE 0 THEN ERRMSG = 'LIST_CDROM: ' + MESSAGE $
		ELSE MESSAGE, MESSAGE, /CONTINUE
;
;  Close the database and return.  If a printout was selected, then print out
;  any entries found.
;
FINISH:
	DBCLOSE
	IF KEYWORD_SET(PRNT) AND (N_FOUND GT 0) THEN BEGIN
	    IF N_ELEMENTS(OUTPUT) EQ 1 THEN BEGIN
		OPENW, UNIT, OUTPUT, /GET_LUN
		PRINTF, UNIT, ''
		PRINTF, UNIT, 'Sequence Volume    Date'
		PRINTF, UNIT, ''
		FOR I=0,N_FOUND-1 DO PRINTF, UNIT, FORMAT='(2(I5,3X),A)', $
			LIST(I)
		PRINTF, UNIT, ''
		FREE_LUN, UNIT
	    END ELSE BEGIN
		PRINT, ''
		PRINT, 'Sequence Volume    Date'
		PRINT, ''
		FOR I=0,N_FOUND-1 DO PRINT, FORMAT='(2(I5,3X),A)', LIST(I)
		PRINT, ''
	    ENDELSE
	ENDIF
;
	RETURN
	END
