	PRO LIST_DATAWIN, LL_ID, LISTS, N_FOUND, ERRMSG=ERRMSG
;+
; Project     :	SOHO - CDS
;
; Name        :	LIST_DATAWIN
;
; Purpose     :	List the available data window lists for a given line list.
;
; Explanation :	Extracts the descriptions of all the data window lists from
;		the database, for a given line list ID number.
;
; Use         :	LIST_DATAWIN, LL_ID, LISTS, N_FOUND
;
; Inputs :	LL_ID = The line list ID number to search on.  Setting this to
;			zero will search for lists that are not connected to
;			line lists.
;
; Opt. Inputs :	None.
;
; Outputs     :	LISTS	= A structure variable containing the following tags
;			  for each data window list:
;
;			DW_ID	  = Data window list ID number.
;			DETECTOR  = Either "G" for GIS or "N" for NIS.
;			DW_DESC	  = A short description of the data window list
;				    beyond what is given in the associated line
;				    list description, e.g. "Full slit, 10
;				    pixels wide".
;			W_WIDTH	  = Width in pixels used to generate the
;				    windows.
;			W_HEIGHT  = The height in pixels used to generate the
;				    windows.  VDS only, for GIS this is set to
;				    unity.
;			VDS_BACK  = Either 0 for off, or 1 for on,
;				    representing whether or not VDS background
;				    windows are being used.  For GIS window
;				    lists this is 0.
;			N_WINDOWS = The number of windows.
;			N_FLAGS	  = The number of flag windows.
;			LL_ID	  = The line list ID number for the line list
;				    that this window list was derived from, or
;				    zero for no line list.
;			LL_DESC	  = A description of the line list, e.g.
;				    "Temperature sensitive line pairs".
;			N_LINES	  = The number of lines in the line list
;
;		N_FOUND	= Number of lists found.
;
; Opt. Outputs:	None.
;
; Keywords    :
;       ERRMSG    = If defined and passed, then any error messages will be
;                   returned to the user in this parameter rather than
;                   depending on the MESSAGE routine in IDL.  If no errors are
;                   encountered, then a null string is returned.  In order to
;                   use this feature, ERRMSG must be defined first, e.g.
;
;                       ERRMSG = ''
;                       LIST_DATAWIN, ERRMSG=ERRMSG, ... 
;                       IF ERRMSG NE '' THEN ...
;
;
; Calls       :	DATATYPE, DBOPEN, DBFIND, DBEXT, DBCLOSE
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	If the number of window lists found is zero, then the output
;		parameter LISTS is not modified.
;
; Category    :	Planning, Database.
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 31 March 1994
;
; Modified    :	Version 1, William Thompson, GSFC, 1 August 1994
;               Version 2, Liyun Wang, GSFC/ARC, September 22, 1994
;                  Added the keyword ERRMSG.
;		Version 3, William Thompson, GSFC, 29 December 1994
;			Fixed bug when inputs are bytes.
;		Version 4, William Thompson, GSFC, 15 March 1995
;			Removed LL_TITLE
;		Version 5, William Thompson, GSFC, 20 June 1995
;			Modified to include N_FLAGS tag.
;
; Version     :	Version 5, 20 June 1995
;-
;
	ON_ERROR, 2
;
;  Initialize N_FOUND to 0 in case any errors are found.  It will be updated
;  later.
;
	N_FOUND = 0
;
;  Check the number of parameters.
;
        IF N_PARAMS() LT 3 THEN BEGIN
           MESSAGE = 'Syntax:  LIST_DATAWIN, LL_ID, LISTS, N_FOUND'
	   GOTO, HANDLE_ERROR
        ENDIF
;
;  Check the input parameter LL_ID.
;
	TYPE = DATATYPE(LL_ID,2)
	IF TYPE EQ 0 THEN BEGIN
           MESSAGE = 'LL_ID is undefined'
           GOTO, HANDLE_ERROR
	END ELSE IF TYPE GE 4 THEN BEGIN
           MESSAGE = 'LL_ID must be an integer'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(LL_ID) NE 1 THEN BEGIN
           MESSAGE = 'LL_ID must be a scalar'
           GOTO, HANDLE_ERROR
	ENDIF
;
;  Open the window and line list database (if LL_ID is not zero).
;
	IF LL_ID EQ 0 THEN DBOPEN, 'datawin' ELSE DBOPEN, 'datawin,linelist'
;
;  If LL_ID is nonzero, then search for entries with that line list ID
;  number.  Otherwise, search for entries with LINELIST_PTR set to zero.
;
	IF LL_ID NE 0 THEN TEST = 'LL_ID='+STRTRIM(LONG(LL_ID),2) ELSE $
		TEST = 'LINELIST_PTR=0'
	ENTRIES = DBFIND(TEST, /SILENT)
	IF !ERR EQ 0 THEN BEGIN
		N_FOUND = 0
		GOTO, FINISH
	END ELSE N_FOUND = N_ELEMENTS(ENTRIES)
;
;  Depending on whether or not LL_ID is zero, treat the database
;  differently.
;
	IF LL_ID NE 0 THEN BEGIN
;
;  Extract the data from the data extraction window list database.  Note that
;  there's some ambiguity about which "DETECTOR" field is being read, but since
;  ADD_DATAWIN enforces that they match, this shouldn't be a problem.
;
		DBEXT, ENTRIES, 'dw_id,detector,dw_desc,vds_back,' +	$
			'n_windows,n_flags,ll_desc,n_lines', DW_ID,	$
			DETECTOR, DW_DESC, VDS_BACK, N_WINDOWS, N_FLAGS,$
			LL_DESC, N_LINES
		DBEXT, ENTRIES, 'w_width,w_height', W_WIDTH, W_HEIGHT
;
;  If LL_ID is zero, then the data extraction is a little simpler.
;
	END ELSE BEGIN
		DBEXT, ENTRIES, 'dw_id,detector,dw_desc,vds_back,' +	$
			'n_windows,n_flags', DW_ID, DETECTOR, DW_DESC,	$
			VDS_BACK, N_WINDOWS, N_FLAGS
		DBEXT, ENTRIES, 'w_width,w_height', W_WIDTH, W_HEIGHT
		LL_DESC = ''
		N_LINES = 0
	ENDELSE
;
;  Define the structure that the data will be returned in.
;
	LISTS = {DW_ID: 0,		$
		DETECTOR: 'D',		$
		DW_DESC: 'String',	$
		W_WIDTH: 0,		$
		W_HEIGHT: 0,		$
		VDS_BACK: 0,		$
		N_WINDOWS: 0,		$
		N_FLAGS: 0,		$
		LL_ID: 0,		$
		LL_DESC: 'String',	$
		N_LINES: 0}
;
	IF N_FOUND EQ 1 THEN BEGIN
		LISTS.DW_ID	= DW_ID(0)
		LISTS.DETECTOR	= DETECTOR(0)
		LISTS.DW_DESC	= DW_DESC(0)
		LISTS.W_WIDTH	= W_WIDTH(0)
		LISTS.W_HEIGHT	= W_HEIGHT(0)
		LISTS.VDS_BACK	= VDS_BACK(0)
		LISTS.N_WINDOWS	= N_WINDOWS(0)
		LISTS.N_FLAGS	= N_FLAGS(0)
		LISTS.LL_ID	= LL_ID(0)
		LISTS.LL_DESC	= LL_DESC(0)
		LISTS.N_LINES	= N_LINES(0)
	END ELSE BEGIN
		LISTS = REPLICATE(LISTS, N_FOUND)
		LISTS.DW_ID	= DW_ID
		LISTS.DETECTOR	= DETECTOR
		LISTS.DW_DESC	= DW_DESC
		LISTS.W_WIDTH	= W_WIDTH
		LISTS.W_HEIGHT	= W_HEIGHT
		LISTS.VDS_BACK	= VDS_BACK
		LISTS.N_WINDOWS	= N_WINDOWS
		LISTS.N_FLAGS	= N_FLAGS
		LISTS.LL_ID	= LL_ID
		LISTS.LL_DESC	= LL_DESC
		LISTS.N_LINES	= N_LINES
	ENDELSE
	GOTO, FINISH
;
;  Error handling point.
;
HANDLE_ERROR:
	IF N_ELEMENTS(ERRMSG) NE 0 THEN ERRMSG = 'LIST_DATAWIN: ' + MESSAGE $
		ELSE MESSAGE, MESSAGE, /CONTINUE
;
;  Close the database and return.
;
FINISH:
	DBCLOSE
;
	RETURN
	END
