	PRO LIST_EXP_COMM, P1, COMM, N_FOUND, ERRMSG=ERRMSG,	$
		DATE_MOD=MOD_DATE, END_DATE=END_DATE
;+
; Project     :	SOHO - CDS
;
; Name        :	LIST_EXP_COMM
;
; Purpose     :	List the comments on a CDS raster as-run entry
;
; Explanation :	Extracts all the comments for a CDS raster-level (experiment)
;		as-run entry.
;
; Use         :	LIST_EXP_COMM, SEQ_NUM, COMM, N_FOUND
;		LIST_EXP_COMM, DATE_MOD, COMM, N_FOUND, /DATE_MOD
;
; Inputs      :	SEQ_NUM  = The sequence number to use in searching the
;			   database.
;
; Opt. Inputs :	DATE_MOD = The modification date to use in searching the
;			   database.  This parameter is used in place of
;			   SEQ_NUM if the DATE_MOD keyword is set.
;
; Outputs     :	COMM	= A structure variable containing the following tags
;			  for each planned observation:
;
;			STRUCT_TYPE= The character string 'CDS-EXP-COMM-LIST'
;			SEQ_NUM	   = Experiment sequence number.  This serves
;				     as a link to the experiment database.
;			COMM_NO	   = Comment ID number.
;			COMMENT	   = The comment.
;			DATE_MOD   = Date and time when catalog record was last
;				     modified.
;			END_DATE   = The end date to search over.  This is used
;				     to support the END_DATE keyword in
;				     WRITE_CIF.
;
;		N_FOUND	= Number of observations found.
;
; Opt. Outputs:	None.
;
; Keywords    :	DATE_MOD = If set, then the search is done on the modification
;			   date rather than the start and stop dates.
;
;		ERRMSG   = If defined and passed, then any error messages will
;			   be returned to the user in this parameter rather
;			   than depending on the MESSAGE routine in IDL.  If no
;			   errors are encountered, then a null string is
;			   returned.  In order to use this feature, ERRMSG must
;			   be defined first, e.g.
;
;				ERRMSG = ''
;				LIST_EXP_COMM, ERRMSG=ERRMSG, ...
;				IF ERRMSG NE '' THEN ...
;
;
; Calls       :	DATATYPE, DBOPEN, DBFIND, DBEXT, DBCLOSE, TRIM
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	If the number of comments found is zero, then the output
;		parameter COMM is not modified.
;
; Category    :	Planning, Database.
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 7 March 1996
;
; Modified    :	Version 1, William Thompson, GSFC, 7 March 1996
;		Version 2, William Thompson, GSFC, 2 April 1996
;			Added keyword DATE_MOD
;		Version 3, William Thompson, GSFC, 12-Mar-1997
;			Added keyword END_DATE
;		Version 4, William Thompson, GSFC, 17-Mar-1997
;			Add ERRMSG in call to DBFIND
;
; Version     :	Version 4, 17-Mar-1997
;-
;
	ON_ERROR, 2
;
;  Check the number of parameters.
;
	N_FOUND = 0
        IF N_PARAMS() NE 3 THEN BEGIN
           MESSAGE = 'Syntax:  LIST_EXP_COMM, SEQ_NUM, COMM, N_FOUND'
	   GOTO, HANDLE_ERROR
        ENDIF
;
;  Check the input parameter.
;
	IF N_ELEMENTS(P1) NE 1 THEN BEGIN
	   MESSAGE = 'SEQ_NUM must be a scalar'
	   GOTO, HANDLE_ERROR
	ENDIF
;
;  Open the as-run comment database.
;
	DBOPEN, 'exper_comment'
;
;  Find all the entries.
;
	IF KEYWORD_SET(MOD_DATE) THEN BEGIN
		TAI = UTC2TAI(ANYTIM2UTC(P1))
		TEST = 'DATE_MOD>' + TRIM(TAI,'(F15.3)')
		IF N_ELEMENTS(END_DATE) EQ 1 THEN BEGIN
			END_TIME = ANYTIM2UTC(END_DATE)
			END_TIME.MJD = END_TIME.MJD + 1
			END_TIME.TIME = 0
			TEST = TEST + ',DATE_MOD<' +	$
				TRIM(UTC2TAI(END_TIME),'(F15.3)')
		ENDIF
		ENTRIES = DBFIND(TEST, /SILENT, ERRMSG=ERRMSG)
	END ELSE BEGIN
		ENTRIES = DBFIND('SEQ_NUM=' + TRIM(P1), /SILENT, ERRMSG=ERRMSG)
	ENDELSE
;
	IF N_ELEMENTS(ERRMSG) NE 0 THEN IF ERRMSG NE '' THEN BEGIN
		MESSAGE = ERRMSG
		GOTO, HANDLE_ERROR
	ENDIF
;
	IF !ERR EQ 0 THEN BEGIN
		N_FOUND = 0
		GOTO, FINISH
	END ELSE BEGIN
		ENTRIES = ENTRIES(UNIQ(ENTRIES))
		N_FOUND = N_ELEMENTS(ENTRIES)
	ENDELSE
;
;  Extract the requested entries.
;
	DBEXT, ENTRIES, 'seq_num,comm_no,comment,date_mod',	$
		SEQ_NUM, COMM_NO, COMMENT, DATE_MOD
;
;  Define the structure that the data will be returned in.
;
	COMM = { STRUCT_TYPE: 'CDS-EXP-COMM-LIST',	$
		SEQ_NUM: 0L,		$
		COMM_NO: 0L,		$
		COMMENT: '',		$
		DATE_MOD: 0.0D0}
;
	IF N_FOUND EQ 1 THEN BEGIN
		COMM.SEQ_NUM	= SEQ_NUM(0)
		COMM.COMM_NO	= COMM_NO(0)
		COMM.COMMENT	= COMMENT(0)
		COMM.DATE_MOD	= DATE_MOD(0)
	END ELSE BEGIN
		COMM = REPLICATE(COMM, N_FOUND)
		COMM.SEQ_NUM	= SEQ_NUM
		COMM.COMM_NO	= COMM_NO
		COMM.COMMENT	= COMMENT
		COMM.DATE_MOD	= DATE_MOD
	ENDELSE
	GOTO, FINISH
;
;  Error handling point.
;
HANDLE_ERROR:
	IF N_ELEMENTS(ERRMSG) NE 0 THEN ERRMSG = 'LIST_EXP_COMM: ' + MESSAGE $
		ELSE MESSAGE, MESSAGE, /CONTINUE
;
;  Close the database and return.
;
FINISH:
	DBCLOSE
;
	RETURN
	END
