	PRO LIST_EXPER, P1, P2, P3, P4, ERRMSG=ERRMSG, DATE_MOD=MOD_DATE, $
		SEARCH=SEARCH
;+
; Project     :	SOHO - CDS
;
; Name        :	LIST_EXPER
;
; Purpose     :	List the CDS raster as-run entries for a given date range
;
; Explanation :	Extracts all the CDS raster-level (experiment) as-run entries
;		which intersect an input time range.
;
; Use         :	LIST_EXPER, START, END, OBS, N_FOUND  [, /DATE_MOD ]
;		LIST_EXPER, PROG_NUM, OBS, N_FOUND
;
; Inputs      :	One of the two following sets of input values must be passed.
;
;		START, END = The range of date/time values to use in searching
;			     the database.  Can be in any standard CDS time
;			     format.
;
;		PROG_NUM   = The program number to use in searching the
;			     database.
;
; Opt. Inputs :	None.
;
; Outputs     :	OBS	= A structure variable containing the following tags
;			  for each planned observation:
;
;			STRUCT_TYPE= The character string 'CDS-EXPERIMENT-LIST'
;			PROG_NUM   = Study counter number.  This forms the
;				     primary key for this database.
;			PROG_ID	   = Program ID, linking one or more studies
;				     together
;			PROG_IND   = An index, reflecting one's position within
;				     a repeated sequence of studies via the
;				     N_REPEAT_S mechanism.
;			STUDY_ID   = Number defining the study
;			STUDYVAR   = Number giving the study variation ID
;			OBS_PROG   = Name of the study
;			SCI_OBJ    = Science objective from the daily science
;				     meeting
;			SCI_SPEC   = Specific science objective from meeting
;			CMP_NO	   = Campaign number
;			OBJECT	   = Code for object planned to be observed
;			OBJ_ID	   = Object identification
;			SEQ_NUM	   = Unique sequence number.  This serves as
;				     the primary key to the database.
;			OBS_SEQ	   = A character string derived from the raster
;				     number and the variation number,
;				     e.g. "R4V3".
;			COUNT	   = The number of repeated observations making
;				     up this experiment.  Normally 1.
;			SEQ_IND	   = The raster counter
;			RAS_ID	   = The raster ID number
;			RAS_VAR	   = The raster variation index
;			EXPTIME	   = The exposure time, in seconds
;			OBS_MODE   = The observing mode.
;			DW_ID	   = The data window ID number
;			DATE_OBS   = Date/time of beginning of observation,
;				     in TAI format
;			DATE_END   = Date/time of end of observation, in TAI
;				     format
;			OBT_TIME   = Onboard date/time of beginning of
;				     observation, in TAI format
;			OBT_END	   = Onboard date/time of end of observation,
;				     in TAI format
;			XCEN	   = Center of instrument field-of-view in
;				     solar X
;			YCEN	   = Center of instrument field-of-view in
;				     solar Y
;			ANGLE	   = Angle of instrument relative to solar
;				     north
;			IXWIDTH	   = Width of field-of-view in instrument X
;			IYWIDTH	   = Width of field-of-view in instrument Y
;			INS_X0	   = The instrument origin pointing in X
;			INS_Y0	   = The instrument origin pointing in Y
;			INS_ROLL   = The instrument roll angle
;			SC_X0	   = The spacecraft pointing in X
;			SC_Y0	   = The spacecraft pointing in Y
;			SC_ROLL	   = The spacecraft roll
;			WAVEMIN	   = The minimum wavelength of observation, in
;				     Angstroms.
;			WAVEMAX	   = The maximum wavelength, in Angstroms
;			TRACKING   = True if tracking is on
;			SER_ID	   = The series ID number
;			OPSLBITS   = The OPS-L status bits at the start of the
;				     observation.
;			OPSRBITS   = The OPS-R status bits at the start of the
;				     observation.
;			SLIT_POS   = The slit position at the start of the
;				     observation.
;			MIR_POS	   = The mirror position at the start of the
;				     observation.
;			EV_ENAB	   = True if event recognition enabled
;			COMP_ERR   = True if a compression error was
;				     encountered.
;			VDS_PMCP   = The VDS MCP programmable voltage setting
;			VDS_MODE   = The VDS readout mode.
;			VDS_ORI	   = True if the VDS telemetry data is oriented
;				     by columns
;			VDS_ACC	   = True if VDS is operated in accumulate mode
;			GSET_ID	   = GIS Setup ID
;			DETECTOR   = Either N or G for NIS/GIS.
;			ZONE_ID	   = Pointing zone ID
;			SLIT_NUM   = Slit number
;			FILENAME   = FITS file name
;			SEQVALID   = True for valid science data
;			DATASRC	   = Data source, either Q(uicklook) or F(inal
;				     distribution).
;			COMMENTS   = Either Y or N
;			DATE_MOD   = Date and time when catalog record was last
;				     modified.
;
;		N_FOUND	= Number of observations found.
;
; Opt. Outputs:	None.
;
; Keywords    :	DATE_MOD = If set, then the search is done on the modification
;			   date rather than the start and stop dates.
;
;		SEARCH	 = Allows one to pass in an optional search string, as
;			   used by DBFIND.  For example, if one wants to search
;			   for all instances where the object was a coronal
;			   hole, then set
;
;				SEARCH='OBJECT=CH'
;
;			   Multiple search criteria can be passed as a single
;			   (case-insensitive) string separated by commas, e.g.
;
;				SEARCH='OBJECT=CH,STUDY_ID=11'
;
;		ERRMSG   = If defined and passed, then any error messages will
;			   be returned to the user in this parameter rather
;			   than depending on the MESSAGE routine in IDL.  If no
;			   errors are encountered, then a null string is
;			   returned.  In order to use this feature, ERRMSG must
;			   be defined first, e.g.
;
;				ERRMSG = ''
;				LIST_EXPER, ERRMSG=ERRMSG, ...
;				IF ERRMSG NE '' THEN ...
;
; Calls       :	DATATYPE, DBOPEN, DBFIND, DBEXT, DBCLOSE, TRIM
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	If the number of planned observations found is zero, then the
;		output parameter OBS is not modified.
;
; Category    :	Planning, Database.
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 3 July 1995
;
; Modified    :	Version 1, William Thompson, GSFC, 3 July 1995
;		Version 2, William Thompson, GSFC, 31 January 1996
;			Added capability of calling with PROG_NUM
;		Version 3, William Thompson, GSFC, 23 February 1996
;			Modified for version 2 of the experiment databases.
;		Version 4, William Thompson, GSFC, 2 April 1996
;			Added keyword DATE_MOD
;		Version 5, William Thompson, GSFC, 2 April 1996
;			Added parameters DATASRC and OBS_PROG
;		Version 6, William Thompson, GSFC, 12 April 1996
;			Look in both experiment* and fexperiment*
;		Version 7, William Thompson, GSFC, 25 April 1996
;			Fixed bug in version 6 related to environment name
;		Version 8, William Thompson, GSFC, 03-Mar-1997
;			Added keyword SEARCH
;		Version 9, William Thompson, GSFC, 12-Mar-1997
;			Modified to pay attention to the end date when the
;			/DATE_MOD keyword is passed.  This is so that the
;			END_DATE keyword in WRITE_CIF will work.
;		Version 10, William Thompson, GSFC, 17-Mar-1997
;			Add ERRMSG in call to DBFIND
;		Version 11, William Thompson, GSFC, 30-Apr-1997
;			Fixed bug where DATASRC was not being returned right.
;		Version 12, William Thompson, GSFC, 1 June 1998
;			Look for entries in "_year" files
;
; Version     :	Version 12, 1 June 1998
;-
;
	ON_ERROR, 2
;
;  Check the number of parameters.
;
	N_FOUND = 0
        IF N_PARAMS() LT 3 THEN BEGIN
           MESSAGE = 'Syntax:  LIST_EXPER, START, END, OBS, N_FOUND' +	$
		   ' or LIST_EXPER, PROG_NUM, OBS, N_FOUND'
	   GOTO, HANDLE_ERROR
        ENDIF
;
;  Check the input parameters.
;
	IF N_PARAMS() EQ 3 THEN BEGIN
		IF N_ELEMENTS(P1) NE 1 THEN BEGIN
		   MESSAGE = 'PROG_NUM must be a scalar'
		   GOTO, HANDLE_ERROR
		ENDIF
	END ELSE BEGIN
		IF N_ELEMENTS(P1) NE 1 THEN BEGIN
        	   MESSAGE = 'START must be a scalar'
	           GOTO, HANDLE_ERROR
		END ELSE IF N_ELEMENTS(P2) NE 1 THEN BEGIN
	           MESSAGE = 'END must be a scalar'
        	   GOTO, HANDLE_ERROR
		ENDIF
;
;  Convert the input dates to TAI format.
;
		IF DATATYPE(P1,1) EQ 'Double' THEN TAI_START = P1 ELSE	$
			TAI_START = UTC2TAI(P1)
		IF DATATYPE(P2,1) EQ 'Double' THEN TAI_END = P2 ELSE	$
			TAI_END = UTC2TAI(P2)
	ENDELSE
;
;  Open the as-run databases.
;
	VERSION = ''
	TRAILER = ''
	SAVE_N_FOUND = 0
	DBOPEN, 'experiment2,experiment1,experiment,main'
;
;  Find all the entries in the requested time range.
;
FIND_ALL:
	IF N_PARAMS() EQ 3 THEN BEGIN
		TEST = 'PROG_NUM=' + TRIM(P1)
	END ELSE IF KEYWORD_SET(MOD_DATE) THEN BEGIN
		TEST = 'DATE_MOD>' + TRIM(TAI_START,'(F15.3)') + $
			',DATE_MOD<' + TRIM(TAI_END,'(F15.3)')
	END ELSE BEGIN
		TEST = 'DATE_END>' + TRIM(TAI_START,'(F15.3)') + $
			',DATE_OBS<' + TRIM(TAI_END,'(F15.3)')
	ENDELSE
	IF N_ELEMENTS(SEARCH) EQ 1 THEN TEST = TEST + ',' + SEARCH
	ENTRIES = DBFIND(TEST, /SILENT, ERRMSG=ERRMSG)
	IF N_ELEMENTS(ERRMSG) NE 0 THEN IF ERRMSG NE '' THEN BEGIN
		MESSAGE = ERRMSG
		GOTO, HANDLE_ERROR
	ENDIF
;
	IF !ERR EQ 0 THEN BEGIN
		N_FOUND = 0
		GOTO, FINISH
	END ELSE BEGIN
		ENTRIES = ENTRIES(UNIQ(ENTRIES))
		N_FOUND = N_ELEMENTS(ENTRIES)
	ENDELSE
;
;  Extract the requested entries, sorted by start times.
;
	ENTRIES = DBSORT(ENTRIES,'date_obs')
	DBEXT, ENTRIES, 'prog_num,prog_id,prog_ind,study_id,studyvar,' + $
		'sci_obj,sci_spec,cmp_no,object,obj_id,seq_num,obs_seq', $
		PROG_NUM, PROG_ID, PROG_IND, STUDY_ID, STUDYVAR, SCI_OBJ, $
		SCI_SPEC, CMP_NO, OBJECT, OBJ_ID, SEQ_NUM, OBS_SEQ
	DBEXT, ENTRIES, 'count,seq_ind,ras_id,ras_var,exptime,obs_mode,' + $
		'dw_id,date_obs,date_end,obt_time,obt_end,obs_prog', COUNT, $
		SEQ_IND, RAS_ID, RAS_VAR, EXPTIME, OBS_MODE, DW_ID, DATE_OBS, $
		DATE_END, OBT_TIME, OBT_END, OBS_PROG
	DBEXT, ENTRIES, 'xcen,ycen,angle,ixwidth,iywidth,ins_x0,ins_y0,' + $
		'ins_roll,sc_x0,sc_y0,sc_roll', XCEN, YCEN, ANGLE, IXWIDTH, $
		IYWIDTH, INS_X0, INS_Y0, INS_ROLL, SC_X0, SC_Y0, SC_ROLL
	DBEXT, ENTRIES, 'wavemin,wavemax,tracking,ser_id,opslbits,opsrbits,'+ $
		'slit_pos,mir_pos,ev_enab,comp_err,vds_pmcp,vds_mode',	$
		WAVEMIN, WAVEMAX, TRACKING, SER_ID, OPSLBITS, OPSRBITS, $
		SLIT_POS, MIR_POS, EV_ENAB, COMP_ERR, VDS_PMCP, VDS_MODE
	DBEXT, ENTRIES, 'vds_ori,vds_acc,gset_id,seqvalid,comments,' +	$
		'date_mod,detector,zone_id,slit_num,filename,datasrc',	$
		VDS_ORI, VDS_ACC, GSET_ID, SEQVALID, COMMENTS, DATE_MOD, $
		DETECTOR, ZONE_ID, SLIT_NUM, FILENAME, DATASRC
;
;  Remove any entries that only touch the requested time range, unless the
;  DATE_MOD keyword was set.
;
	IF (N_PARAMS() EQ 4) AND (NOT KEYWORD_SET(MOD_DATE)) THEN BEGIN
		W = WHERE((DATE_END NE TAI_START) AND (DATE_OBS NE TAI_END), $
			N_FOUND)
		IF N_FOUND EQ 0 THEN GOTO, FINISH
	END ELSE W = LINDGEN(N_FOUND)
;
;  Define the structure that the data will be returned in.
;
	OBS = { CDS_EXPER_LIST,		$
		STRUCT_TYPE: 'CDS-EXPERIMENT-LIST',	$
		PROG_NUM: 0L,		$
		PROG_ID: 0,		$
		PROG_IND: 0,		$
		STUDY_ID: 0,		$
		STUDYVAR: 0,		$
		OBS_PROG: 'String',	$
		SCI_OBJ: 'String',	$
		SCI_SPEC: 'String',	$
		CMP_NO: 0,		$
		OBJECT: 'String',	$
		OBJ_ID: 'String',	$
		SEQ_NUM: 0L,		$
		OBS_SEQ: 'String',	$
		COUNT: 0,		$
		SEQ_IND: 0,		$
		RAS_ID: 0,		$
		RAS_VAR: 0,		$
		EXPTIME: 0.0,		$
		OBS_MODE: 'String',	$
		DW_ID: 0,		$
		DATE_OBS: 0.0D0,	$
		DATE_END: 0.0D0,	$
		OBT_TIME: 0.0D0,	$
		OBT_END: 0.0D0,		$
		XCEN: 0.0,		$
		YCEN: 0.0,		$
		ANGLE: 0.0,		$
		IXWIDTH: 0.0,		$
		IYWIDTH: 0.0,		$
		INS_X0: 0.0,		$
		INS_Y0: 0.0,		$
		INS_ROLL: 0.0,		$
		SC_X0: 0.0,		$
		SC_Y0: 0.0,		$
		SC_ROLL: 0.0,		$
		WAVEMIN: 0.0,		$
		WAVEMAX: 0.0,		$
		TRACKING: 0B,		$
		SER_ID: 0,		$
		OPSLBITS: 'String',	$
		OPSRBITS: 'String',	$
		SLIT_POS: 0,		$
		MIR_POS: 0,		$
		EV_ENAB: 0B,		$
		COMP_ERR: 0B,		$
		VDS_PMCP: 0,		$
		VDS_MODE: 0,		$
		VDS_ORI: 0B,		$
		VDS_ACC: 0B,		$
		GSET_ID: 0,		$
		DETECTOR: 'S',		$
		ZONE_ID: 0,		$
		SLIT_NUM: 0,		$
		FILENAME: 'String',	$
		SEQVALID: 0B,		$
		DATASRC: 'S',		$
		COMMENTS: 'S',		$
		DATE_MOD: 0.0D0}
;
	IF N_FOUND EQ 1 THEN BEGIN
		OBS.PROG_NUM	= PROG_NUM(W(0))
		OBS.PROG_ID	= PROG_ID(W(0))
		OBS.PROG_IND	= PROG_IND(W(0))
		OBS.STUDY_ID	= STUDY_ID(W(0))
		OBS.STUDYVAR	= STUDYVAR(W(0))
		OBS.OBS_PROG	= OBS_PROG(W(0))
		OBS.SCI_OBJ	= SCI_OBJ(W(0))
		OBS.SCI_SPEC	= SCI_SPEC(W(0))
		OBS.CMP_NO	= CMP_NO(W(0))
		OBS.OBJECT	= OBJECT(W(0))
		OBS.OBJ_ID	= OBJ_ID(W(0))
		OBS.SEQ_NUM	= SEQ_NUM(W(0))
		OBS.OBS_SEQ	= OBS_SEQ(W(0))
		OBS.COUNT	= COUNT(W(0))
		OBS.SEQ_IND	= SEQ_IND(W(0))
		OBS.RAS_ID	= RAS_ID(W(0))
		OBS.RAS_VAR	= RAS_VAR(W(0))
		OBS.EXPTIME	= EXPTIME(W(0))
		OBS.OBS_MODE	= OBS_MODE(W(0))
		OBS.DW_ID	= DW_ID(W(0))
		OBS.DATE_OBS	= DATE_OBS(W(0))
		OBS.DATE_END	= DATE_END(W(0))
		OBS.OBT_TIME	= OBT_TIME(W(0))
		OBS.OBT_END	= OBT_END(W(0))
		OBS.XCEN	= XCEN(W(0))
		OBS.YCEN	= YCEN(W(0))
		OBS.ANGLE	= ANGLE(W(0))
		OBS.IXWIDTH	= IXWIDTH(W(0))
		OBS.IYWIDTH	= IYWIDTH(W(0))
		OBS.INS_X0	= INS_X0(W(0))
		OBS.INS_Y0	= INS_Y0(W(0))
		OBS.INS_ROLL	= INS_ROLL(W(0))
		OBS.SC_X0	= SC_X0(W(0))
		OBS.SC_Y0	= SC_Y0(W(0))
		OBS.SC_ROLL	= SC_ROLL(W(0))
		OBS.WAVEMIN	= WAVEMIN(W(0))
		OBS.WAVEMAX	= WAVEMAX(W(0))
		OBS.TRACKING	= TRACKING(W(0))
		OBS.SER_ID	= SER_ID(W(0))
		OBS.OPSLBITS	= OPSLBITS(W(0))
		OBS.OPSRBITS	= OPSRBITS(W(0))
		OBS.SLIT_POS	= SLIT_POS(W(0))
		OBS.MIR_POS	= MIR_POS(W(0))
		OBS.EV_ENAB	= EV_ENAB(W(0))
		OBS.COMP_ERR	= COMP_ERR(W(0))
		OBS.VDS_PMCP	= VDS_PMCP(W(0))
		OBS.VDS_MODE	= VDS_MODE(W(0))
		OBS.VDS_ORI	= VDS_ORI(W(0))
		OBS.VDS_ACC	= VDS_ACC(W(0))
		OBS.GSET_ID	= GSET_ID(W(0))
		OBS.DETECTOR	= DETECTOR(W(0))
		OBS.ZONE_ID	= ZONE_ID(W(0))
		OBS.SLIT_NUM	= SLIT_NUM(W(0))
		OBS.FILENAME	= FILENAME(W(0))
		OBS.SEQVALID	= SEQVALID(W(0))
		OBS.DATASRC	= DATASRC(W(0))
		OBS.COMMENTS	= COMMENTS(W(0))
		OBS.DATE_MOD	= DATE_MOD(W(0))
	END ELSE BEGIN
		OBS = REPLICATE(OBS, N_FOUND)
		OBS.PROG_NUM	= PROG_NUM(W)
		OBS.PROG_ID	= PROG_ID(W)
		OBS.PROG_IND	= PROG_IND(W)
		OBS.STUDY_ID	= STUDY_ID(W)
		OBS.STUDYVAR	= STUDYVAR(W)
		OBS.OBS_PROG	= OBS_PROG(W)
		OBS.SCI_OBJ	= SCI_OBJ(W)
		OBS.SCI_SPEC	= SCI_SPEC(W)
		OBS.CMP_NO	= CMP_NO(W)
		OBS.OBJECT	= OBJECT(W)
		OBS.OBJ_ID	= OBJ_ID(W)
		OBS.SEQ_NUM	= SEQ_NUM(W)
		OBS.OBS_SEQ	= OBS_SEQ(W)
		OBS.COUNT	= COUNT(W)
		OBS.SEQ_IND	= SEQ_IND(W)
		OBS.RAS_ID	= RAS_ID(W)
		OBS.RAS_VAR	= RAS_VAR(W)
		OBS.EXPTIME	= EXPTIME(W)
		OBS.OBS_MODE	= OBS_MODE(W)
		OBS.DW_ID	= DW_ID(W)
		OBS.DATE_OBS	= DATE_OBS(W)
		OBS.DATE_END	= DATE_END(W)
		OBS.OBT_TIME	= OBT_TIME(W)
		OBS.OBT_END	= OBT_END(W)
		OBS.XCEN	= XCEN(W)
		OBS.YCEN	= YCEN(W)
		OBS.ANGLE	= ANGLE(W)
		OBS.IXWIDTH	= IXWIDTH(W)
		OBS.IYWIDTH	= IYWIDTH(W)
		OBS.INS_X0	= INS_X0(W)
		OBS.INS_Y0	= INS_Y0(W)
		OBS.INS_ROLL	= INS_ROLL(W)
		OBS.SC_X0	= SC_X0(W)
		OBS.SC_Y0	= SC_Y0(W)
		OBS.SC_ROLL	= SC_ROLL(W)
		OBS.WAVEMIN	= WAVEMIN(W)
		OBS.WAVEMAX	= WAVEMAX(W)
		OBS.TRACKING	= TRACKING(W)
		OBS.SER_ID	= SER_ID(W)
		OBS.OPSLBITS	= OPSLBITS(W)
		OBS.OPSRBITS	= OPSRBITS(W)
		OBS.SLIT_POS	= SLIT_POS(W)
		OBS.MIR_POS	= MIR_POS(W)
		OBS.EV_ENAB	= EV_ENAB(W)
		OBS.COMP_ERR	= COMP_ERR(W)
		OBS.VDS_PMCP	= VDS_PMCP(W)
		OBS.VDS_MODE	= VDS_MODE(W)
		OBS.VDS_ORI	= VDS_ORI(W)
		OBS.VDS_ACC	= VDS_ACC(W)
		OBS.GSET_ID	= GSET_ID(W)
		OBS.DETECTOR	= DETECTOR(W)
		OBS.ZONE_ID	= ZONE_ID(W)
		OBS.SLIT_NUM	= SLIT_NUM(W)
		OBS.FILENAME	= FILENAME(W)
		OBS.SEQVALID	= SEQVALID(W)
		OBS.DATASRC	= DATASRC(W)
		OBS.COMMENTS	= COMMENTS(W)
		OBS.DATE_MOD	= DATE_MOD(W)
	ENDELSE
	GOTO, FINISH
;
;  Error handling point.
;
HANDLE_ERROR:
	IF N_ELEMENTS(ERRMSG) NE 0 THEN ERRMSG = 'LIST_EXPER: ' + MESSAGE $
		ELSE MESSAGE, MESSAGE, /CONTINUE
	GOTO, EXIT_POINT
;
;  If not the first time through, then concatenate the two data arrays
;  together.
;
FINISH:
	IF (VERSION EQ 'f') AND (SAVE_N_FOUND GT 0) THEN BEGIN
	    IF N_FOUND EQ 0 THEN BEGIN
		N_FOUND = SAVE_N_FOUND
		IF N_FOUND GT 0 THEN OBS = SAVE_OBS
	    END ELSE BEGIN
		N_FOUND = N_FOUND + SAVE_N_FOUND
		OBS = [OBS, SAVE_OBS]
		IF N_FOUND GT 1 THEN BEGIN
		    S = SORT(OBS.DATE_OBS)
		    OBS = OBS(S)
		ENDIF
	    ENDELSE
	ENDIF
;
;  If the regular version of the database was searched, then search the "f"
;  version as well.  Also search through the year variations.
;
	IF VERSION EQ '' THEN BEGIN
	    VERSION = 'f'
	END ELSE IF TRAILER EQ '' THEN BEGIN
	    ITRAILER = 1996
	    TRAILER = '_' + TRIM(ITRAILER)
	END ELSE BEGIN
	    ITRAILER = ITRAILER + 1
	    TRAILER = '_' + TRIM(ITRAILER)
	ENDELSE
;
	IF (FIND_WITH_DEF('fexperiment2'+TRAILER+'.dbf','$ZDBASE') NE '') $
		THEN BEGIN
	    SAVE_N_FOUND = N_FOUND
	    IF N_FOUND GT 0 THEN SAVE_OBS = OBS
	    DBOPEN, 'fexperiment2' + TRAILER + ',fexperiment1' + TRAILER + $
		    ',fexperiment' + TRAILER + ',main'
	    GOTO, FIND_ALL
	ENDIF
;
;  Store the output in the correct variables, close the database and return.
;
EXIT_POINT:
	DBCLOSE
	IF N_PARAMS() EQ 3 THEN BEGIN
		IF N_ELEMENTS(OBS) NE 0 THEN P2 = OBS
		P3 = N_FOUND
	END ELSE BEGIN
		IF N_ELEMENTS(OBS) NE 0 THEN P3 = OBS
		P4 = N_FOUND
	ENDELSE
;
	RETURN
	END
