	PRO LIST_F_RASTER, RASTERS, N_FOUND, DETECTOR=DET, ERRMSG=ERRMSG
;+
; Project     :	SOHO - CDS
;
; Name        :	LIST_F_RASTER
;
; Purpose     :	List the fundamental parameters of all available rasters.
;
; Explanation :	Extracts the fundamental parameters of all the rasters from
;		the database.
;
; Use         :	LIST_F_RASTER, RASTERS, N_FOUND
;
; Inputs      :	None.
;
; Opt. Inputs :	None.
;
; Outputs     :	RASTERS	= A structure variable of type "cds_f_raster"
;			  containing the following tags for each raster:
;
;			RAS_ID	 = Raster ID number.
;			DETECTOR = Either "G" for GIS or "N" for NIS.
;			RAS_DESC = A short description of the raster, giving
;				   its purpose.
;			SLIT_NUM = The ID number for the slit to be used.
;			XSTEP	 = The step size in the X direction, in arcsec.
;			YSTEP	 = The step size in the Y direction, in arcsec.
;			NX	 = The number of exposure positions in X.
;			NY	 = The number of exposure positions in Y.
;
;		N_FOUND	= Number of rasters found.
;
; Opt. Outputs:	None.
;
; Keywords    :	
;       DETECTOR = Can be set to either "G" or "N" to extract only
;		   entries for the GIS or NIS detectors.  If not
;		   passed, then all rasters are returned.
;       ERRMSG    = If defined and passed, then any error messages will be
;                   returned to the user in this parameter rather than
;                   depending on the MESSAGE routine in IDL.  If no errors are
;                   encountered, then a null string is returned.  In order to
;                   use this feature, ERRMSG must be defined first, e.g.
;
;                       ERRMSG = ''
;                       LIST_F_RASTER, ERRMSG=ERRMSG, ... 
;                       IF ERRMSG NE '' THEN ...
;
;
; Calls       :	DATATYPE, DBOPEN, DBFIND, DB_INFO, DBEXT, DBCLOSE
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	If the number of rasters found is zero, then the output
;		parameter RASTERS is not modified.
;
; Category    :	Planning, Database.
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 4 April 1994
;
; Modified    :	Version 1.0, William Thompson, GSFC, 22 July 1994
;               Version 2, Liyun Wang, GSFC/ARC, September 22, 1994
;                  Added the keyword ERRMSG.
;
; Version     :	Version 2, September 22, 1994
;-
;
	ON_ERROR, 2
;
;  Check the number of parameters.
;
        IF N_PARAMS() LT 2 THEN BEGIN
           MESSAGE = 'Syntax:  LIST_F_RASTER, RASTERS, N_FOUND'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN BEGIN
              ERRMSG = MESSAGE
              RETURN
           ENDIF ELSE MESSAGE, MESSAGE
        ENDIF
;
;  If passed, then check the keyword parameter DETECTOR.
;
	IF N_ELEMENTS(DET) NE 0 THEN BEGIN
		IF DATATYPE(DET,1) NE 'String' THEN BEGIN
                   MESSAGE = 'DETECTOR must be a character string'
                   IF N_ELEMENTS(ERRMSG) NE 0 THEN $
                      ERRMSG = MESSAGE $
                   ELSE MESSAGE, MESSAGE, /CONTINUE 
                   N_FOUND = 0
                   GOTO, FINISH
		END ELSE IF N_ELEMENTS(DET) NE 1 THEN BEGIN
                   MESSAGE = 'DETECTOR must be a scalar'
                   IF N_ELEMENTS(ERRMSG) NE 0 THEN $
                      ERRMSG = MESSAGE $
                   ELSE MESSAGE, MESSAGE, /CONTINUE 
                   N_FOUND = 0
                   GOTO, FINISH
		END ELSE IF (DET NE 'G') AND (DET NE 'N') THEN BEGIN
                   MESSAGE = 'DETECTOR must be either "G" or "N"'
                   IF N_ELEMENTS(ERRMSG) NE 0 THEN $
                      ERRMSG = MESSAGE $
                   ELSE MESSAGE, MESSAGE, /CONTINUE 
                   N_FOUND = 0
                   GOTO, FINISH
		ENDIF
	ENDIF
;
;  Open the raster database.
;
	DBOPEN, 'raster'
;
;  If the keyword DETECTOR was passed, then search for entries satisfying that
;  condition.  Otherwise, return the entire database.
;
	IF N_ELEMENTS(DET) EQ 1 THEN BEGIN
		ENTRIES = DBFIND('DETECTOR='+DET, /SILENT)
		IF !ERR EQ 0 THEN BEGIN
			N_FOUND = 0
			GOTO, FINISH
		END ELSE N_FOUND = N_ELEMENTS(ENTRIES)
	END ELSE BEGIN
		N_FOUND = DB_INFO('entries','raster')
		IF N_FOUND EQ 0 THEN GOTO, FINISH
		ENTRIES = LINDGEN(N_FOUND) + 1
	ENDELSE
;
;  Extract the data from the raster database.
;
	DBEXT, ENTRIES, 'ras_id,detector,ras_desc,slit_num,xstep,ystep,' + $
		'nx,ny', RAS_ID, DETECTOR, RAS_DESC, SLIT_NUM, XSTEP, YSTEP, $
		NX, NY
;
;  Define the structure that the data will be returned in.
;
	RASTERS = {CDS_F_RASTER,	$
		RAS_ID: 0,		$
		DETECTOR: 'D',		$
		RAS_DESC: 'String',	$
		SLIT_NUM: 0,		$
		XSTEP: 0,		$
		YSTEP: 0, 		$
		NX: 0,			$
		NY: 0}
;
	IF N_FOUND EQ 1 THEN BEGIN
		RASTERS.RAS_ID	 = RAS_ID(0)
		RASTERS.DETECTOR = DETECTOR(0)
		RASTERS.RAS_DESC = RAS_DESC(0)
		RASTERS.SLIT_NUM = SLIT_NUM(0)
		RASTERS.XSTEP    = XSTEP(0)
		RASTERS.YSTEP    = YSTEP(0)
		RASTERS.NX       = NX(0)
		RASTERS.NY       = NY(0)
	END ELSE BEGIN
		RASTERS = REPLICATE(RASTERS, N_FOUND)
		RASTERS.RAS_ID	 = RAS_ID
		RASTERS.DETECTOR = DETECTOR
		RASTERS.RAS_DESC = RAS_DESC
		RASTERS.SLIT_NUM = SLIT_NUM
		RASTERS.XSTEP    = XSTEP
		RASTERS.YSTEP    = YSTEP
		RASTERS.NX       = NX
		RASTERS.NY       = NY
	ENDELSE
;
;  Close the database and return.
;
FINISH:
	DBCLOSE
;
	RETURN
	END
