	PRO LIST_F_STUDY, STUDIES, N_FOUND, ERRMSG=ERRMSG, CATEGORY=CATEGORY
;+
; Project     :	SOHO - CDS
;
; Name        :	LIST_F_STUDY
;
; Purpose     :	List the fundamental parameters of all available studies.
;
; Explanation :	Extracts the fundamental parameters of all the studies from
;		the database.
;
; Use         :	LIST_F_STUDY, STUDIES, N_FOUND
;
; Inputs      :	None.
;
; Opt. Inputs :	None.
;
; Outputs     :	STUDIES	= Structure variable of type "cds_f_study" containing
;			  the following tags for each study:
;
;			STUDY_ID    = Study ID number.
;			TITLE_ID    = Study title ID number.
;			OBS_PROG    = Name of the study
;			TITLE	    = A descriptive title for the study
;			CATEGORY    = The study category, either "T"est,
;				      "S"cience or "C"alibration.
;			VAR_POINT   = Either "Y" or "N" representing whether or
;				      not the pointing can be controlled by the
;				      user.  If "Y" then the pointing is
;				      defined in the plan.  If "N" then the
;				      pointing is embedded within the study.
;			N_RASTER_DEF= Number of rasters in the definition.
;
;		N_FOUND	= Number of studies found.
;
; Opt. Outputs:	None.
;
; Keywords    :
;       ERRMSG    = If defined and passed, then any error messages will be
;                   returned to the user in this parameter rather than
;                   depending on the MESSAGE routine in IDL.  If no errors are
;                   encountered, then a null string is returned.  In order to
;                   use this feature, ERRMSG must be defined first, e.g.
;
;                       ERRMSG = ''
;                       LIST_F_STUDY, ERRMSG=ERRMSG, ... 
;                       IF ERRMSG NE '' THEN ...
;
;	CATEGORY  = If passed, then one of "T", "S", or "C" to select only
;		    those studies with that category.  If not passed, or passed
;		    as the null string, then all studies are returned.
;
; Calls       :	DATATYPE, DBOPEN, DB_INFO, DBEXT, DBCLOSE
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	If the number of studies found is zero, then the output
;		parameter STUDIES is not modified.
;
; Category    :	Planning, Database.
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 28 April 1994
;
; Modified    :	Version 1, William Thompson, GSFC, 4 May 1994
;               Version 2, Liyun Wang, GSFC/ARC, September 22, 1994
;                  Added the keyword ERRMSG.
;		Version 3, William Thompson, GSFC, 21 November 1994
;			Removed item N_RASTERS0
;			Added item N_RASTER_DEF
;		Version 4, William Thompson, GSFC, 10 February 1995
;			Changed N_POINTINGS to VAR_POINT.
;			Removed VAR_RASTERS.
;		Version 5, William Thompson, GSFC, 12 May 1995
;			Modified to split off separate study_title database
;		Version 6, William Thompson, GSFC, 30 November 1995
;			Added keyword CATEGORY
;
; Version     :	Version 6, 30 November 1995
;-
;
	ON_ERROR, 2
;
;  Check the number of parameters.
;
	N_FOUND = 0
        IF N_PARAMS() LT 2 THEN BEGIN
           MESSAGE = 'Syntax:  LIST_F_STUDY, STUDIES, N_FOUND'
	   GOTO, HANDLE_ERROR
        ENDIF
;
;  Open the study database.
;
	DBOPEN, 'study_title,study'
;
;  If the CATEGORY keyord was passed, then return only the studies meeting the
;  requested category.  Otherwise, return the entire database.
;
	IF N_ELEMENTS(CATEGORY) EQ 1 THEN BEGIN
		ENTRIES = DBFIND('CATEGORY='+CATEGORY, /SILENT)
		IF !ERR EQ 0 THEN BEGIN
			N_FOUND = 0
			GOTO, FINISH
		END ELSE N_FOUND = N_ELEMENTS(ENTRIES)
	END ELSE BEGIN
		N_FOUND = DB_INFO('entries','study_title')
		IF N_FOUND EQ 0 THEN GOTO, FINISH
		ENTRIES = LINDGEN(N_FOUND) + 1
	ENDELSE
;
;  Extract the data from the study database.
;
	DBEXT, ENTRIES, 'study_id,title_id,obs_prog,title,category,' + $
		'var_point,n_raster_def', STUDY_ID, TITLE_ID, OBS_PROG, $
		TITLE, CATEGORY, VAR_POINT, N_RASTER_DEF
;
;  Define the structure that the data will be returned in.
;
	STUDIES = {CDS_F_STUDY,		$
		STUDY_ID: 0,		$
		TITLE_ID: 0,		$
		OBS_PROG: 'String',	$
		TITLE: 'String',	$
		CATEGORY: 'C',		$
		VAR_POINT: 'C',		$
		N_RASTER_DEF: 0}
;
	IF N_FOUND EQ 1 THEN BEGIN
		STUDIES.STUDY_ID    = STUDY_ID(0)
		STUDIES.TITLE_ID    = TITLE_ID(0)
		STUDIES.OBS_PROG    = OBS_PROG(0)
		STUDIES.TITLE	    = TITLE(0)
		STUDIES.CATEGORY    = CATEGORY(0)
		STUDIES.VAR_POINT   = VAR_POINT(0)
		STUDIES.N_RASTER_DEF= N_RASTER_DEF(0)
	END ELSE BEGIN
		STUDIES = REPLICATE(STUDIES, N_FOUND)
		STUDIES.STUDY_ID    = STUDY_ID
		STUDIES.TITLE_ID    = TITLE_ID
		STUDIES.OBS_PROG    = OBS_PROG
		STUDIES.TITLE	    = TITLE
		STUDIES.CATEGORY    = CATEGORY
		STUDIES.VAR_POINT   = VAR_POINT
		STUDIES.N_RASTER_DEF= N_RASTER_DEF
	ENDELSE
	GOTO, FINISH
;
;  Error handling point.
;
HANDLE_ERROR:
	IF N_ELEMENTS(ERRMSG) NE 0 THEN ERRMSG = 'LIST_F_STUDY: ' + MESSAGE $
		ELSE MESSAGE, MESSAGE, /CONTINUE
;
;  Close the database and return.
;
FINISH:
	DBCLOSE
;
	RETURN
	END
