	PRO LIST_FLAG, START, EEND, PERIODS, N_FOUND, ERRMSG=ERRMSG
;+
; Project     :	SOHO - CDS
;
; Name        :	LIST_FLAG
;
; Purpose     :	List the CDS flag receiver status for a date range.
;
; Explanation :	Extracts all the CDS flag receiver science plan entries which
;		intersect an input time range.
;
; Use         :	LIST_FLAG, START, END, PERIODS, N_FOUND
;
; Inputs      :	START, END = The range of date/time values to use in searching
;			     the database.  Can be in any standard CDS time
;			     format.
;
; Opt. Inputs :	None.
;
; Outputs     :	PERIODS	= A structure variable containing the following tags
;			  for each flag receiver period:
;
;			STRUCT_TYPE= The character string 'CDS-FLAG-LIST'
;			PROG_ID	   = Program ID, linking one or more studies
;				     together
;			STUDY_ID   = Number defining the study
;			STUDYVAR   = Number giving the study variation ID
;			SCI_OBJ    = Science objective from the daily science
;				     meeting
;			SCI_SPEC   = Specific science objective from meeting
;			CMP_NO	   = Campaign number
;			OBJECT	   = Code for object planned to be observed
;			OBJ_ID	   = Object identification
;			RCVR_START = Date/time of beginning of receiver status,
;				     in TAI format
;			RCVR_STOP  = Date/time of end of receiver status, in
;				     TAI format
;			ORIG_DUR   = Original duration of the observation, in
;				     seconds, before being truncated by
;				     following time-tagged observations.
;			N_RASTERS1 = Variable number of rasters parameter
;			TRACKING   = True (1) if feature tracking to be used,
;				     or false (0) otherwise
;			N_POINTINGS= Number of pointings to use with study.
;				     Can be either 0 or 1.
;			REPOINT	   = Either 1 to allow repoints or 0 to
;				     disallow.
;			GSET_ID	   = GSET ID number
;
;		N_FOUND	= Number of periods found.
;
; Opt. Outputs:	None.
;
; Keywords    :
;       ERRMSG    = If defined and passed, then any error messages will be
;                   returned to the user in this parameter rather than
;                   depending on the MESSAGE routine in IDL.  If no errors are
;                   encountered, then a null string is returned.  In order to
;                   use this feature, ERRMSG must be defined first, e.g.
;
;                       ERRMSG = ''
;                       LIST_FLAG, ERRMSG=ERRMSG, ... 
;                       IF ERRMSG NE '' THEN ...
;
;
; Calls       :	DATATYPE, DBOPEN, DBFIND, DBEXT, DBCLOSE, TRIM
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	If the number of periods found is zero, then the output
;		parameter PERIODS is not modified.
;
; Category    :	Planning, Database.
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 25 July 1994
;
; Modified    :	Version 1, William Thompson, GSFC, 3 August 1994
;               Version 2, Liyun Wang, GSFC/ARC, September 22, 1994
;                  Added the keyword ERRMSG.
;		Version 3, William Thompson, GSFC, 21 November 1994
;			Modified so that items which only touch requested range
;			are excluded.
;			Added item N_REPEAT_S
;		Version 4, William Thompson, GSFC, 26 January 1995
;			Added tag STRUCT_TYPE
;		Version 5, William Thompson, GSFC, 13 February 1995
;			Changed the way that pointing is treated.
;		Version 6, William Thompson, GSFC, 8 May 1995
;			Modified to pay attention to DELETED field in database
;		Version 7, William Thompson, GSFC, 5 July 1995
;			Added tag REPOINT
;		Version 8, William Thompson, GSFC, 6 October 1995
;			Added tags ORIG_DUR, GSET_ID
;			Removed tag N_REPEAT_S
;
; Version     :	Version 8, 6 October 1995
;-
;
	ON_ERROR, 2
;
;  Check the number of parameters.
;
	N_FOUND = 0
        IF N_PARAMS() LT 4 THEN BEGIN
           MESSAGE = 'Syntax:  LIST_FLAG, START, END, PERIODS, N_FOUND'
	   GOTO, HANDLE_ERROR
        ENDIF
;
;  Check the input parameters.
;
	IF N_ELEMENTS(START) NE 1 THEN BEGIN
           MESSAGE = 'START must be a scalar'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(EEND) NE 1 THEN BEGIN
           MESSAGE = 'END must be a scalar'
           GOTO, HANDLE_ERROR
	ENDIF
;
;  Convert the input dates to TAI format.
;
	IF DATATYPE(START,1) EQ 'Double' THEN TAI_START = START ELSE	$
		TAI_START = UTC2TAI(START)
	IF DATATYPE(EEND,1) EQ 'Double' THEN TAI_END = EEND ELSE	$
		TAI_END = UTC2TAI(EEND)
;
;  Open the flag reciever science plan database.
;
	DBOPEN, 'sci_flag'
;
;  Find all the entries in the requested time range.
;
	ENTRIES = DBFIND('RCVR_STOP>' + TRIM(TAI_START,'(F15.3)') +	$
		',RCVR_START<' + TRIM(TAI_END,'(F15.3)') + ',DELETED=N', $
		/SILENT)
	IF !ERR EQ 0 THEN BEGIN
		N_FOUND = 0
		GOTO, FINISH
	END ELSE BEGIN
		ENTRIES = ENTRIES(UNIQ(ENTRIES))
		N_FOUND = N_ELEMENTS(ENTRIES)
	ENDELSE
;
;  Extract the requested entries, sorted by start times.
;
	ENTRIES = DBSORT(ENTRIES,'rcvr_start')
	DBEXT, ENTRIES, 'prog_id,study_id,studyvar,sci_obj,sci_spec,cmp_no', $
		PROG_ID, STUDY_ID, STUDYVAR, SCI_OBJ, SCI_SPEC, CMP_NO
	DBEXT, ENTRIES, 'object,obj_id,rcvr_start,rcvr_stop,n_rasters1', $
		OBJECT, OBJ_ID, RCVR_START, RCVR_STOP, N_RASTERS1
	DBEXT, ENTRIES, 'tracking,n_pointings,orig_dur,gset_id,repoint', $
		TRACKING, N_POINTINGS, ORIG_DUR, GSET_ID, REPOINT
;
;  Remove any entries that only touch the requested time range.
;
	W = WHERE((RCVR_STOP NE TAI_START) AND (RCVR_START NE TAI_END),	$
		N_FOUND)
	IF N_FOUND EQ 0 THEN GOTO, FINISH
;
;  Define the structure that the data will be returned in.
;
	PERIODS = {STRUCT_TYPE: 'CDS-FLAG-LIST',	$
		PROG_ID: 0,		$
		STUDY_ID: 0,		$
		STUDYVAR: 0,		$
		SCI_OBJ: 'String',	$
		SCI_SPEC: 'String',	$
		CMP_NO: 0,		$
		OBJECT: 'String',	$
		OBJ_ID: 'String',	$
		RCVR_START: 0.0D0,	$
		RCVR_STOP: 0.0D0,	$
		ORIG_DUR: 0.0D0,	$
		N_RASTERS1: 0,		$
		TRACKING: 0B,		$
		N_POINTINGS: 0,		$
		REPOINT: 0B,		$
		GSET_ID: 0}
;
	IF N_FOUND EQ 1 THEN BEGIN
		PERIODS.PROG_ID	   = PROG_ID(W(0))
		PERIODS.STUDY_ID   = STUDY_ID(W(0))
		PERIODS.STUDYVAR   = STUDYVAR(W(0))
		PERIODS.SCI_OBJ	   = SCI_OBJ(W(0))
		PERIODS.SCI_SPEC   = SCI_SPEC(W(0))
		PERIODS.CMP_NO	   = CMP_NO(W(0))
		PERIODS.OBJECT	   = OBJECT(W(0))
		PERIODS.OBJ_ID	   = OBJ_ID(W(0))
		PERIODS.RCVR_START = RCVR_START(W(0))
		PERIODS.RCVR_STOP  = RCVR_STOP(W(0))
		PERIODS.ORIG_DUR   = ORIG_DUR(W(0))
		PERIODS.N_RASTERS1 = N_RASTERS1(W(0))
		PERIODS.TRACKING   = TRACKING(W(0))
		PERIODS.N_POINTINGS= N_POINTINGS(W(0))
		PERIODS.REPOINT	   = REPOINT(W(0))
		PERIODS.GSET_ID	   = GSET_ID(W(0))
	END ELSE BEGIN
		PERIODS = REPLICATE(PERIODS, N_FOUND)
		PERIODS.PROG_ID	   = PROG_ID(W)
		PERIODS.STUDY_ID   = STUDY_ID(W)
		PERIODS.STUDYVAR   = STUDYVAR(W)
		PERIODS.SCI_OBJ	   = SCI_OBJ(W)
		PERIODS.SCI_SPEC   = SCI_SPEC(W)
		PERIODS.CMP_NO	   = CMP_NO(W)
		PERIODS.OBJECT	   = OBJECT(W)
		PERIODS.OBJ_ID	   = OBJ_ID(W)
		PERIODS.RCVR_START = RCVR_START(W)
		PERIODS.RCVR_STOP  = RCVR_STOP(W)
		PERIODS.ORIG_DUR   = ORIG_DUR(W)
		PERIODS.N_RASTERS1 = N_RASTERS1(W)
		PERIODS.TRACKING   = TRACKING(W)
		PERIODS.N_POINTINGS= N_POINTINGS(W)
		PERIODS.REPOINT	   = REPOINT(W)
		PERIODS.GSET_ID	   = GSET_ID(W)
	ENDELSE
	GOTO, FINISH
;
;  Error handling point.
;
HANDLE_ERROR:
	IF N_ELEMENTS(ERRMSG) NE 0 THEN ERRMSG = 'LIST_FLAG: ' + MESSAGE $
		ELSE MESSAGE, MESSAGE, /CONTINUE
;
;  Close the database and return.
;
FINISH:
	DBCLOSE
;
	RETURN
	END
