	PRO LIST_GSET, FIL_ID, GSETS, N_FOUND, FFB=SEARCH_FFB, ERRMSG=ERRMSG
;+
; Project     :	SOHO - CDS
;
; Name        :	LIST_GSET
;
; Purpose     :	List GIS setup 
;
; Explanation :	Extracts the descriptions of all the GIS setup descriptions
;		from the database.
;
; Use         :	LIST_GSET, FIL_ID, GSETS, N_FOUND
;
; Inputs      :	FIL_ID = Filament ID, either 0 (science), 1, or 2.
;
; Opt. Inputs :	None.
;
; Outputs     :	GSETS = A structure variable containing the following tags
;			for each GIS setup entry:
;
;			GSET_ID	   = GIS setup ID number.
;			HVOLT	   = High voltages (4)
;			LLD	   = Low level discriminators (4)
;			LUT_CHKSUM = LUT checksum in hexidecimal notation (4)
;			LUT_PAR	   = LUT parameters (4,11)
;			RAW_ID	   = RAW GIS science data file ID (4)
;			SLIT_NUM   = Slit number
;			ZONE_ID	   = Zone ID number
;			FIL_ID	   = Filament ID number, or 0 for science data
;			FFB	   = Front face bias, either -1 or +1
;			DET_USED   = Whether or not detector is used, either 0
;				     or 1 (4)
;			GSET_DESC  = Free format descriptions (4)
;
;		N_FOUND	= Number of GIS setups found.
;
; Opt. Outputs:	None.
;
; Keywords    :	FFB	= Select items with a particular front face bias,
;			  either -1 or +1.
;
;		ERRMSG	= If defined and passed, then any error messages will
;			  be returned to the user in this parameter rather than
;			  depending on the MESSAGE routine in IDL.  If no
;			  errors are encountered, then a null string is
;			  returned.  In order to use this feature, ERRMSG must
;			  be defined first, e.g.
;
;				ERRMSG = ''
;				LIST_GSET, ERRMSG=ERRMSG, ... 
;				IF ERRMSG NE '' THEN ...
;
;
; Calls       :	DATATYPE, DBOPEN, DBFIND, DBEXT, DBCLOSE
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	If the number of raw data files found is zero, then the output
;		parameter GSETS is not modified.
;
; Category    :	Planning, Database.
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 7 September 1995
;
; Modified    :	Version 1, William Thompson, GSFC, 7 September 1995
;		Version 2, William Thompson, GSFC, 14 September 1995
;			Replace FIL_CUR with FFB
;			Add FIL_ID as input parameter
;			Add FFB as keyword parameter
;		Version 3, William Thompson, GSFC, 28 September 1995
;			Make LUT_CHKSUM an array.
;		Version 4, William Thompson, GSFC, 4 October 1995
;			Modified to reflect change in way LUT_PAR, LUT_CHKSUM,
;			and GSET_DESC are stored.
;		Version 5, William Thompson, GSFC, 25 January 1996
;			Modified definition of LUT_PAR and added DET_USED.
;
; Version     :	Version 5, 25 January 1996
;-
;
	ON_ERROR, 2
;
;  Initialize N_FOUND to 0 in case any errors are found.  It will be updated
;  later.
;
	N_FOUND = 0
;
;  Check the number of parameters.
;
        IF N_PARAMS() LT 3 THEN BEGIN
           MESSAGE = 'Syntax:  LIST_GSET, FIL_ID, GSETS, N_FOUND'
	   GOTO, HANDLE_ERROR
        ENDIF
;
;  Check the input parameter FIL_ID.
;
	TYPE = DATATYPE(FIL_ID,2)
	IF TYPE EQ 0 THEN BEGIN
           MESSAGE = 'FIL_ID is undefined'
           GOTO, HANDLE_ERROR
	END ELSE IF TYPE GE 4 THEN BEGIN
           MESSAGE = 'FIL_ID must be an integer'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(FIL_ID) NE 1 THEN BEGIN
           MESSAGE = 'FIL_ID must be a scalar'
           GOTO, HANDLE_ERROR
	ENDIF
;
;  Open the GIS setup catalog.
;
	DBOPEN, 'gset'
;
;  Search for entries with the requested filament ID.  If passed, also test on
;  FFB.
;
	TEST = 'FIL_ID='+STRTRIM(LONG(FIL_ID),2)
	IF N_ELEMENTS(SEARCH_FFB) EQ 1 THEN TEST = TEST +	$
		',FFB='+STRTRIM(LONG(SEARCH_FFB))
	ENTRIES = DBFIND(TEST, /SILENT)
	IF !ERR EQ 0 THEN BEGIN
		N_FOUND = 0
		GOTO, FINISH
	END ELSE N_FOUND = N_ELEMENTS(ENTRIES)
;
;  Extract the data.
;
	DBEXT, ENTRIES, 'gset_id,hvolt,lld,lut_chksum,lut_par,raw_id,' + $
		'slit_num,zone_id,ffb,det_used,gset_desc', GSET_ID, HVOLT, $
		LLD, LUT_CHKSUM, LUT_PAR, RAW_ID, SLIT_NUM, ZONE_ID, FFB, $
		DET_USED, GSET_DESC
;
;  Reformat the checksum and description each into an array of strings.
;
	LUT_CHKSUM = STRTRIM( REFORM( BYTE(LUT_CHKSUM), 4, 4, N_FOUND))
	GSET_DESC  = STRTRIM( REFORM( BYTE(GSET_DESC), 80, 4, N_FOUND))
;
;  Define the structure that the data will be returned in.
;
	GSETS = {GSET_ID: 0,	$
		HVOLT: INTARR(4),	$
		LLD: INTARR(4),		$
		LUT_CHKSUM: STRARR(4),	$
		LUT_PAR: BYTARR(4,11),	$
		RAW_ID: INTARR(4),	$
		SLIT_NUM: 0,		$
		ZONE_ID: 0,		$
		FIL_ID: 0,		$
		FFB: 0,			$
		DET_USED: INTARR(4),	$
		GSET_DESC: STRARR(4)}
;
;  Extract the data.  Note that LUT_PAR must be reformatted because the UIT
;  database software cannot handle multidimensional arrays.
;
	IF N_FOUND EQ 1 THEN BEGIN
		GSETS.GSET_ID	= GSET_ID(0)
		GSETS.HVOLT	= HVOLT(*,0)
		GSETS.LLD	= LLD(*,0)
		GSETS.LUT_CHKSUM= LUT_CHKSUM(*,0)
		GSETS.LUT_PAR	= REFORM(LUT_PAR(*,0),4,11)
		GSETS.RAW_ID	= RAW_ID(*,0)
		GSETS.SLIT_NUM	= SLIT_NUM(0)
		GSETS.ZONE_ID	= ZONE_ID(0)
		GSETS.FIL_ID	= FIL_ID(0)
		GSETS.FFB	= FFB(0)
		GSETS.DET_USED	= DET_USED(*,0)
		GSETS.GSET_DESC	= GSET_DESC(*,0)
	END ELSE BEGIN
		GSETS = REPLICATE(GSETS, N_FOUND)
		GSETS.GSET_ID	= GSET_ID
		GSETS.HVOLT	= HVOLT
		GSETS.LLD	= LLD
		GSETS.LUT_CHKSUM= LUT_CHKSUM
		GSETS.LUT_PAR	= REFORM(LUT_PAR,4,11,N_FOUND)
		GSETS.RAW_ID	= RAW_ID
		GSETS.SLIT_NUM	= SLIT_NUM
		GSETS.ZONE_ID	= ZONE_ID
		GSETS.FIL_ID	= FIL_ID
		GSETS.FFB	= FFB
		GSETS.DET_USED	= DET_USED
		GSETS.GSET_DESC	= GSET_DESC
	ENDELSE
	GOTO, FINISH
;
;  Error handling point.
;
HANDLE_ERROR:
	IF N_ELEMENTS(ERRMSG) NE 0 THEN ERRMSG = 'LIST_GSET: ' + MESSAGE $
		ELSE MESSAGE, MESSAGE, /CONTINUE
;
;  Close the database and return.
;
FINISH:
	DBCLOSE
;
	RETURN
	END
