	PRO LIST_LINELIST, LISTS, N_FOUND, DETECTOR=DET, ERRMSG=ERRMSG
;+
; Project     :	SOHO - CDS
;
; Name        :	LIST_LINELIST
;
; Purpose     :	List all the available line lists.
;
; Explanation :	Extracts the names and descriptions of all the line lists from
;		the database.
;
; Use         :	LIST_LINELIST, LISTS, N_FOUND
;
; Inputs      :	None.
;
; Opt. Inputs :	None.
;
; Outputs     :	LISTS	= A structure variable containing the following tags
;			  for each line list:
;
;			LL_ID	 = Line list ID number.
;			DETECTOR = Either "G" for GIS or "N" for NIS.
;			LL_DESC	 = A description of the line list, e.g.
;				   "Temperature sensitive line pairs".
;			N_LINES	 = The number of lines in the line list.
;
;		N_FOUND	= Number of lists found.
;
; Opt. Outputs:	None.
;
; Keywords    :	
;       DETECTOR = Can be set to either "G" or "N" to extract only
;		   entries for the GIS or NIS detectors.  If not
;		   passed, then all line lists are returned.
;       ERRMSG    = If defined and passed, then any error messages will be
;                   returned to the user in this parameter rather than
;                   depending on the MESSAGE routine in IDL.  If no errors are
;                   encountered, then a null string is returned.  In order to
;                   use this feature, ERRMSG must be defined first, e.g.
;
;                       ERRMSG = ''
;                       LIST_LINELIST, ERRMSG=ERRMSG, ... 
;                       IF ERRMSG NE '' THEN ...
;
;
; Calls       :	DATATYPE, DBOPEN, DBFIND, DB_INFO, DBEXT, DBCLOSE
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	If the number of line lists found is zero, then the output
;		parameter LISTS is not modified.
;
; Category    :	Planning, Database.
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 31 March 1994
;
; Modified    :	Version 1.0, William Thompson, GSFC, 22 July 1994
;               Version 2, Liyun Wang, GSFC/ARC, September 22, 1994
;                  Added the keyword ERRMSG.
;		Version 3, William Thompson, GSFC, 15 March 1995
;			Removed LL_TITLE
;
; Version     :	Version 3, 15 March 1995
;-
;
	ON_ERROR, 2
;
;  Check the number of parameters.
;
        IF N_PARAMS() LT 2 THEN BEGIN
           MESSAGE = 'Syntax:  LIST_LINELIST, LISTS, N_FOUND'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN BEGIN
              ERRMSG = MESSAGE
              RETURN
           ENDIF ELSE MESSAGE, MESSAGE
        ENDIF
;
;  If passed, then check the keyword parameter DETECTOR.
;
	IF N_ELEMENTS(DET) NE 0 THEN BEGIN
		IF DATATYPE(DET,1) NE 'String' THEN BEGIN
                   MESSAGE = 'DETECTOR must be a character string'
                   IF N_ELEMENTS(ERRMSG) NE 0 THEN $
                      ERRMSG = MESSAGE $
                   ELSE MESSAGE, MESSAGE, /CONTINUE 
                   N_FOUND = 0
                   GOTO, FINISH
		END ELSE IF N_ELEMENTS(DET) NE 1 THEN BEGIN
                   MESSAGE = 'DETECTOR must be a scalar'
                   IF N_ELEMENTS(ERRMSG) NE 0 THEN $
                      ERRMSG = MESSAGE $
                   ELSE MESSAGE, MESSAGE, /CONTINUE 
                   N_FOUND = 0
                   GOTO, FINISH
		END ELSE IF (DET NE 'G') AND (DET NE 'N') THEN BEGIN
                   MESSAGE = 'DETECTOR must be either "G" or "N"'
                   IF N_ELEMENTS(ERRMSG) NE 0 THEN $
                      ERRMSG = MESSAGE $
                   ELSE MESSAGE, MESSAGE, /CONTINUE 
                   N_FOUND = 0
                   GOTO, FINISH
		ENDIF
	ENDIF
;
;  Open the line list database.
;
	DBOPEN, 'linelist'
;
;  If the keyword DETECTOR was passed, then search for entries satisfying that
;  condition.  Otherwise, return the entire database.
;
	IF N_ELEMENTS(DET) EQ 1 THEN BEGIN
		ENTRIES = DBFIND('DETECTOR='+DET, /SILENT)
		IF !ERR EQ 0 THEN BEGIN
			N_FOUND = 0
			GOTO, FINISH
		END ELSE N_FOUND = N_ELEMENTS(ENTRIES)
	END ELSE BEGIN
		N_FOUND = DB_INFO('entries','linelist')
		IF N_FOUND EQ 0 THEN GOTO, FINISH
		ENTRIES = LINDGEN(N_FOUND) + 1
	ENDELSE
;
;  Extract the data from the line list database.
;
	DBEXT, ENTRIES, 'll_id,detector,ll_desc,n_lines',	$
		LL_ID, DETECTOR, LL_DESC, N_LINES
;
;  Define the structure that the data will be returned in.
;
	LISTS = {LL_ID: 0,		$
		DETECTOR: 'D',		$
		LL_DESC: 'String',	$
		N_LINES: 0}
;
	IF N_FOUND EQ 1 THEN BEGIN
		LISTS.LL_ID    = LL_ID(0)
		LISTS.DETECTOR = DETECTOR(0)
		LISTS.LL_DESC  = LL_DESC(0)
		LISTS.N_LINES  = N_LINES(0)
	END ELSE BEGIN
		LISTS = REPLICATE(LISTS, N_FOUND)
		LISTS.LL_ID    = LL_ID
		LISTS.DETECTOR = DETECTOR
		LISTS.LL_DESC  = LL_DESC
		LISTS.N_LINES  = N_LINES
	ENDELSE
;
;  Close the database and return.
;
FINISH:
	DBCLOSE
;
	RETURN
	END
