	PRO LIST_MAIN, START, EEND, OBS, N_FOUND, ERRMSG=ERRMSG,	$
		DATE_MOD=MOD_DATE, SHORT_LIST=SHORT_LIST, SEARCH=SEARCH
;+
; Project     :	SOHO - CDS
;
; Name        :	LIST_MAIN
;
; Purpose     :	List the CDS main as-run entries for a given date range
;
; Explanation :	Extracts all the CDS study-level (main) as-run entries which
;		intersect an input time range.
;
; Use         :	LIST_MAIN, START, END, OBS, N_FOUND  [, /DATE_MOD ]
;
; Inputs      :	START, END = The range of date/time values to use in searching
;			     the database.  Can be in any standard CDS time
;			     format.
;
; Opt. Inputs :	None.
;
; Outputs     :	OBS	= A structure variable containing the following tags
;			  for each planned observation:
;
;			STRUCT_TYPE= The character string 'CDS-MAIN-LIST'
;			PROG_NUM   = Study counter number.  This forms the
;				     primary key for this database.
;			PROG_ID	   = Program ID, linking one or more studies
;				     together
;			PROG_IND   = An index, reflecting one's position within
;				     a repeated sequence of studies via the
;				     N_REPEAT_S mechanism.
;			STUDY_ID   = Number defining the study
;			STUDYVAR   = Number giving the study variation ID
;			OBS_PROG   = Name of the study
;			DETECTOR   = Detector, "N", "G", or "B"(oth).
;			SCI_OBJ    = Science objective from the daily science
;				     meeting
;			SCI_SPEC   = Specific science objective from meeting
;			CMP_NO	   = Campaign number
;			OBJECT	   = Code for object planned to be observed
;			OBJ_ID	   = Object identification
;			DATE_OBS   = Date/time of beginning of observation,
;				     in TAI format
;			DATE_END   = Date/time of end of observation, in TAI
;				     format
;			OBT_TIME   = Onboard date/time of beginning of
;				     observation, in TAI format
;			OBT_END	   = Onboard date/time of end of observation,
;				     in TAI format
;			XCEN	   = Center of instrument field-of-view in
;				     solar X
;			YCEN	   = Center of instrument field-of-view in
;				     solar Y
;			ANGLE	   = Angle of instrument relative to solar
;				     north
;			IXWIDTH	   = Width of field-of-view in instrument X
;			IYWIDTH	   = Width of field-of-view in instrument Y
;			SEQ_FROM   = Start of range of observing sequence
;				     numbers.
;			SEQ_TO	   = End of range of observing sequence
;				     numbers.
;			COMMENTS   = Either Y or N
;			DATE_MOD   = Date and time when catalog record was last
;				     modified.
;
;			  However, if the keyword /SHORT_LIST is set, then only
;			  a subset of the parameters are returned.  This speeds
;			  up access when called from XCAT.
;
;		N_FOUND	= Number of observations found.
;
; Opt. Outputs:	None.
;
; Keywords    :	DATE_MOD = If set, then the search is done on the modification
;			   date rather than the start and stop dates.
;
;		SHORT_LIST = If set, then a shortened list of parameters is
;			   returned.
;
;		SEARCH	 = Allows one to pass in an optional search string, as
;			   used by DBFIND.  For example, if one wants to search
;			   for all instances where the object was a coronal
;			   hole, then set
;
;				SEARCH='OBJECT=CH'
;
;			   Multiple search criteria can be passed as a single
;			   (case-insensitive) string separated by commas, e.g.
;
;				SEARCH='OBJECT=CH,STUDY_ID=11'
;
;		ERRMSG   = If defined and passed, then any error messages will
;			   be returned to the user in this parameter rather
;			   than depending on the MESSAGE routine in IDL.  If no
;			   errors are encountered, then a null string is
;			   returned.  In order to use this feature, ERRMSG must
;			   be defined first, e.g.
;
;				ERRMSG = ''
;				LIST_MAIN, ERRMSG=ERRMSG, ...
;				IF ERRMSG NE '' THEN ...
;
;
; Calls       :	DATATYPE, DBOPEN, DBFIND, DBEXT, DBCLOSE, TRIM
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	If the number of planned observations found is zero, then the
;		output parameter OBS is not modified.
;
; Category    :	Planning, Database.
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 3 July 1995
;
; Modified    :	Version 1, William Thompson, GSFC, 3 July 1995
;		Version 2, William Thompson, GSFC, 2 April 1996
;			Added keyword DATE_MOD
;		Version 3, William Thompson, GSFC, 2 April 1996
;			Modified for version 3 of the catalog.
;		Version 4, William Thompson, GSFC, 2 May 1996
;			Added keyword /SHORT_LIST
;		Version 5, William Thompson, GSFC, 03-Mar-1997
;			Added keyword SEARCH
;		Version 6, William Thompson, GSFC, 12-Mar-1997
;			Modified to pay attention to the end date when the
;			/DATE_MOD keyword is passed.  This is so that the
;			END_DATE keyword in WRITE_CIF will work.
;		Version 7, William Thompson, GSFC, 17-Mar-1997
;			Add ERRMSG in call to DBFIND
;		Version 8, William Thompson, GSFC, 24-Jul-1997
;			Filter out negative PROG_NUM values.
;
; Version     :	Version 8, 24-Jul-1997
;-
;
	ON_ERROR, 2
;
;  Check the number of parameters.
;
	N_FOUND = 0
        IF N_PARAMS() LT 4 THEN BEGIN
           MESSAGE = 'Syntax:  LIST_MAIN, START, END, OBS, N_FOUND'
	   GOTO, HANDLE_ERROR
        ENDIF
;
;  Check the input parameters.
;
	IF N_ELEMENTS(START) NE 1 THEN BEGIN
           MESSAGE = 'START must be a scalar'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(EEND) NE 1 THEN BEGIN
           MESSAGE = 'END must be a scalar'
           GOTO, HANDLE_ERROR
	ENDIF
;
;  Convert the input dates to TAI format.
;
	IF DATATYPE(START,1) EQ 'Double' THEN TAI_START = START ELSE	$
		TAI_START = UTC2TAI(START)
	IF DATATYPE(EEND,1) EQ 'Double' THEN TAI_END = EEND ELSE	$
		TAI_END = UTC2TAI(EEND)
;
;  Open the main as-run database.
;
	DBOPEN, 'main'
;
;  Find all the entries in the requested time range.
;
	IF KEYWORD_SET(MOD_DATE) THEN BEGIN
		TEST = 'DATE_MOD>' + TRIM(TAI_START,'(F15.3)') + $
			',DATE_MOD<' + TRIM(TAI_END,'(F15.3)')
	END ELSE BEGIN
		TEST = 'DATE_END>' + TRIM(TAI_START,'(F15.3)') + $
			',DATE_OBS<' + TRIM(TAI_END,'(F15.3)')
	ENDELSE
	TEST = TEST + ',PROG_NUM>0'
	IF N_ELEMENTS(SEARCH) EQ 1 THEN TEST = TEST + ',' + SEARCH
	ENTRIES = DBFIND(TEST, /SILENT, ERRMSG=ERRMSG)
	IF N_ELEMENTS(ERRMSG) NE 0 THEN IF ERRMSG NE '' THEN BEGIN
		MESSAGE = ERRMSG
		GOTO, HANDLE_ERROR
	ENDIF
;
	IF !ERR EQ 0 THEN BEGIN
		N_FOUND = 0
		GOTO, FINISH
	END ELSE BEGIN
		ENTRIES = ENTRIES(UNIQ(ENTRIES))
		N_FOUND = N_ELEMENTS(ENTRIES)
	ENDELSE
;
;  Extract the requested entries, sorted by start times.
;
	ENTRIES = DBSORT(ENTRIES,'date_obs')
	DBEXT,ENTRIES,'prog_num,date_obs,date_end,obs_prog,detector,sci_obj', $
		PROG_NUM, DATE_OBS, DATE_END, OBS_PROG, DETECTOR, SCI_OBJ
	IF NOT KEYWORD_SET(SHORT_LIST) THEN BEGIN
		DBEXT, ENTRIES, 'prog_id,prog_ind,study_id,studyvar,' + $
			'sci_obj,sci_spec,cmp_no,object,obj_id,',	$
			PROG_ID, PROG_IND, STUDY_ID, STUDYVAR, SCI_OBJ, $
			SCI_SPEC, CMP_NO, OBJECT, OBJ_ID
		DBEXT, ENTRIES, 'obt_time,obt_end,xcen,ycen,angle,ixwidth,' + $
			'iywidth,seq_from,seq_to,comments,date_mod',	$
			OBT_TIME, OBT_END, XCEN, YCEN, ANGLE, IXWIDTH,	$
			IYWIDTH, SEQ_FROM, SEQ_TO, COMMENTS, DATE_MOD
	ENDIF
;
;  Remove any entries that only touch the requested time range, unless the
;  DATE_MOD keyword was set.
;
	W = WHERE((DATE_END NE TAI_START) AND (DATE_OBS NE TAI_END), N_FOUND)
	IF KEYWORD_SET(MOD_DATE) THEN BEGIN
		N_FOUND = N_ELEMENTS(ENTRIES)
		W = LINDGEN(N_FOUND)
	ENDIF
	IF N_FOUND EQ 0 THEN GOTO, FINISH
;
;  Define the structure that the data will be returned in.
;
	IF KEYWORD_SET(SHORT_LIST) THEN BEGIN
		OBS = {	STRUCT_TYPE: 'CDS-MAIN-SHORTLIST',	$
			PROG_NUM: 0L,		$
			DATE_OBS: 0.0D0,	$
			DATE_END: 0.0D0,	$
			OBS_PROG: 'String',	$
			DETECTOR: 'S',		$
			SCI_OBJ: 'String'}
	END ELSE BEGIN
		OBS = { STRUCT_TYPE: 'CDS-MAIN-LIST',	$
			PROG_NUM: 0L,		$
			PROG_ID: 0,		$
			PROG_IND: 0,		$
			STUDY_ID: 0,		$
			STUDYVAR: 0,		$
			OBS_PROG: 'String',	$
			DETECTOR: 'S',		$
			SCI_OBJ: 'String',	$
			SCI_SPEC: 'String',	$
			CMP_NO: 0,		$
			OBJECT: 'String',	$
			OBJ_ID: 'String',	$
			DATE_OBS: 0.0D0,	$
			DATE_END: 0.0D0,	$
			OBT_TIME: 0.0D0,	$
			OBT_END: 0.0D0,		$
			XCEN: 0.0,		$
			YCEN: 0.0,		$
			ANGLE: 0.0,		$
			IXWIDTH: 0.0,		$
			IYWIDTH: 0.0,		$
			SEQ_FROM: 0L,		$
			SEQ_TO: 0L,		$
			COMMENTS: 'C',		$
			DATE_MOD: 0.0D0}
	ENDELSE
;
	IF N_FOUND EQ 1 THEN BEGIN
		OBS.PROG_NUM	= PROG_NUM(W(0))
		OBS.DATE_OBS	= DATE_OBS(W(0))
		OBS.DATE_END	= DATE_END(W(0))
		OBS.OBS_PROG	= OBS_PROG(W(0))
		OBS.DETECTOR	= DETECTOR(W(0))
		OBS.SCI_OBJ	= SCI_OBJ(W(0))
		IF NOT KEYWORD_SET(SHORT_LIST) THEN BEGIN
			OBS.PROG_ID	= PROG_ID(W(0))
			OBS.PROG_IND	= PROG_IND(W(0))
			OBS.STUDY_ID	= STUDY_ID(W(0))
			OBS.STUDYVAR	= STUDYVAR(W(0))
			OBS.SCI_SPEC	= SCI_SPEC(W(0))
			OBS.CMP_NO	= CMP_NO(W(0))
			OBS.OBJECT	= OBJECT(W(0))
			OBS.OBJ_ID	= OBJ_ID(W(0))
			OBS.OBT_TIME	= OBT_TIME(W(0))
			OBS.OBT_END	= OBT_END(W(0))
			OBS.XCEN	= XCEN(W(0))
			OBS.YCEN	= YCEN(W(0))
			OBS.ANGLE	= ANGLE(W(0))
			OBS.IXWIDTH	= IXWIDTH(W(0))
			OBS.IYWIDTH	= IYWIDTH(W(0))
			OBS.SEQ_FROM	= SEQ_FROM(W(0))
			OBS.SEQ_TO	= SEQ_TO(W(0))
			OBS.COMMENTS	= COMMENTS(W(0))
			OBS.DATE_MOD	= DATE_MOD(W(0))
		ENDIF
	END ELSE BEGIN
		OBS = REPLICATE(OBS, N_FOUND)
		OBS.PROG_NUM	= PROG_NUM(W)
		OBS.DATE_OBS	= DATE_OBS(W)
		OBS.DATE_END	= DATE_END(W)
		OBS.OBS_PROG	= OBS_PROG(W)
		OBS.DETECTOR	= DETECTOR(W)
		OBS.SCI_OBJ	= SCI_OBJ(W)
		IF NOT KEYWORD_SET(SHORT_LIST) THEN BEGIN
			OBS.PROG_ID	= PROG_ID(W)
			OBS.PROG_IND	= PROG_IND(W)
			OBS.STUDY_ID	= STUDY_ID(W)
			OBS.STUDYVAR	= STUDYVAR(W)
			OBS.SCI_SPEC	= SCI_SPEC(W)
			OBS.CMP_NO	= CMP_NO(W)
			OBS.OBJECT	= OBJECT(W)
			OBS.OBJ_ID	= OBJ_ID(W)
			OBS.OBT_TIME	= OBT_TIME(W)
			OBS.OBT_END	= OBT_END(W)
			OBS.XCEN	= XCEN(W)
			OBS.YCEN	= YCEN(W)
			OBS.ANGLE	= ANGLE(W)
			OBS.IXWIDTH	= IXWIDTH(W)
			OBS.IYWIDTH	= IYWIDTH(W)
			OBS.SEQ_FROM	= SEQ_FROM(W)
			OBS.SEQ_TO	= SEQ_TO(W)
			OBS.COMMENTS	= COMMENTS(W)
			OBS.DATE_MOD	= DATE_MOD(W)
		ENDIF
	ENDELSE
	GOTO, FINISH
;
;  Error handling point.
;
HANDLE_ERROR:
	IF N_ELEMENTS(ERRMSG) NE 0 THEN ERRMSG = 'LIST_MAIN: ' + MESSAGE $
		ELSE MESSAGE, MESSAGE, /CONTINUE
;
;  Close the database and return.
;
FINISH:
	DBCLOSE
;
	RETURN
	END
