	PRO LIST_RAW, DET_NUM, FIL_ID, RAW_FILES, N_FOUND, ERRMSG=ERRMSG
;+
; Project     :	SOHO - CDS
;
; Name        :	LIST_RAW
;
; Purpose     :	List raw data files for given GIS detector
;
; Explanation :	Extracts the descriptions of all the GIS raw data files from
;		the database, for a given detector number and filament ID.  Set
;		the filament ID to 0 to select raw science data.
;
; Use         :	LIST_RAW, DET_NUM, FIL_ID, RAW_FILES, N_FOUND
;
; Inputs      :	DET_NUM	= The GIS detector number to search on, 1-4
;		FIL_ID	= The filament ID, 0 (science), 1, 2.
;
; Opt. Inputs :	None.
;
; Outputs     :	RAW_FILES = A structure variable containing the following tags
;			    for each raw data file entry:
;
;			RAW_ID	   = Raw data file ID number.
;			RAW_DESC   = Free format description
;			START_TIME = Start time of data taking, in TAI format.
;			DET_NUM	   = GIS detector number: 1-4
;			SLIT_NUM   = Slit number
;			HVOLT	   = High voltage value
;			LLD	   = Low level discriminator value
;			CNT_RATE   = Count rate range (min, max)
;			RAW_FILE   = Raw data file name
;			FFB	   = Front face bias
;			FIL_ID	   = Filament ID number, or 0 for science data
;			FIL_CUR	   = Filament current
;			ZONE_ID	   = Zone ID number
;
;		N_FOUND	= Number of raw file entries found.
;
; Opt. Outputs:	None.
;
; Keywords    :
;       ERRMSG    = If defined and passed, then any error messages will be
;                   returned to the user in this parameter rather than
;                   depending on the MESSAGE routine in IDL.  If no errors are
;                   encountered, then a null string is returned.  In order to
;                   use this feature, ERRMSG must be defined first, e.g.
;
;                       ERRMSG = ''
;                       LIST_RAW, ERRMSG=ERRMSG, ... 
;                       IF ERRMSG NE '' THEN ...
;
;
; Calls       :	DATATYPE, DBOPEN, DBFIND, DBEXT, DBCLOSE
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	If the number of raw data files found is zero, then the output
;		parameter RAW_FILES is not modified.
;
; Category    :	Planning, Database.
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 7 September 1995
;
; Modified    :	Version 1, William Thompson, GSFC, 7 September 1995
;		Version 2, William Thompson, GSFC, 14 September 1995
;			Added RAW_DESC, rename FF_BIAS to FFB to match GSET
;			database.
;
; Version     :	Version 2, 14 September 1995
;-
;
	ON_ERROR, 2
;
;  Initialize N_FOUND to 0 in case any errors are found.  It will be updated
;  later.
;
	N_FOUND = 0
;
;  Check the number of parameters.
;
        IF N_PARAMS() LT 4 THEN BEGIN
           MESSAGE = 'Syntax:  LIST_RAW, DET_NUM, FIL_ID, RAW_FILES, N_FOUND'
	   GOTO, HANDLE_ERROR
        ENDIF
;
;  Check the input parameter DET_NUM.
;
	TYPE = DATATYPE(DET_NUM,2)
	IF TYPE EQ 0 THEN BEGIN
           MESSAGE = 'DET_NUM is undefined'
           GOTO, HANDLE_ERROR
	END ELSE IF TYPE GE 4 THEN BEGIN
           MESSAGE = 'DET_NUM must be an integer'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(DET_NUM) NE 1 THEN BEGIN
           MESSAGE = 'DET_NUM must be a scalar'
           GOTO, HANDLE_ERROR
	ENDIF
;
;  Check the input parameter FIL_ID.
;
	TYPE = DATATYPE(FIL_ID,2)
	IF TYPE EQ 0 THEN BEGIN
           MESSAGE = 'FIL_ID is undefined'
           GOTO, HANDLE_ERROR
	END ELSE IF TYPE GE 4 THEN BEGIN
           MESSAGE = 'FIL_ID must be an integer'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(FIL_ID) NE 1 THEN BEGIN
           MESSAGE = 'FIL_ID must be a scalar'
           GOTO, HANDLE_ERROR
	ENDIF
;
;  Open the raw data file catalog.
;
	DBOPEN, 'raw_files'
;
;  Search for entries with the requested detector number.
;
	ENTRIES = DBFIND('DET_NUM='+STRTRIM(LONG(DET_NUM),2) +	$
		',FIL_ID='+STRTRIM(LONG(FIL_ID),2), /SILENT)
	IF !ERR EQ 0 THEN BEGIN
		N_FOUND = 0
		GOTO, FINISH
	END ELSE N_FOUND = N_ELEMENTS(ENTRIES)
;
;  Extract the data from the raw data file database.
;
	DBEXT, ENTRIES, 'raw_id,raw_desc,start_time,slit_num,hvolt,lld,' + $
		'cnt_rate,raw_file,ffb,fil_cur,zone_id', RAW_ID, RAW_DESC, $
		START_TIME, SLIT_NUM, HVOLT, LLD, CNT_RATE, RAW_FILE, FFB, $
		FIL_CUR, ZONE_ID
;
;  Define the structure that the data will be returned in.
;
	RAW_FILES = {RAW_ID: 0,		$
		RAW_DESC: 'String',	$
		START_TIME: 0.0D0,	$
		DET_NUM: 0,		$
		SLIT_NUM: 0,		$
		HVOLT: 0,		$
		LLD: 0,			$
		CNT_RATE: [0,0],	$
		RAW_FILE: 'String',	$
		FFB: 0,			$
		FIL_ID: 0,		$
		FIL_CUR: 0,		$
		ZONE_ID: 0}
;
	IF N_FOUND EQ 1 THEN BEGIN
		RAW_FILES.RAW_ID	= RAW_ID(0)
		RAW_FILES.RAW_DESC	= RAW_DESC(0)
		RAW_FILES.START_TIME	= START_TIME(0)
		RAW_FILES.DET_NUM	= DET_NUM(0)
		RAW_FILES.SLIT_NUM	= SLIT_NUM(0)
		RAW_FILES.HVOLT		= HVOLT(0)
		RAW_FILES.LLD		= LLD(0)
		RAW_FILES.CNT_RATE	= CNT_RATE(*,0)
		RAW_FILES.RAW_FILE	= RAW_FILE(0)
		RAW_FILES.FFB		= FFB(0)
		RAW_FILES.FIL_ID	= FIL_ID(0)
		RAW_FILES.FIL_CUR	= FIL_CUR(0)
		RAW_FILES.ZONE_ID	= ZONE_ID(0)
	END ELSE BEGIN
		RAW_FILES = REPLICATE(RAW_FILES, N_FOUND)
		RAW_FILES.RAW_ID	= RAW_ID
		RAW_FILES.RAW_DESC	= RAW_DESC
		RAW_FILES.START_TIME	= START_TIME
		RAW_FILES.DET_NUM	= DET_NUM
		RAW_FILES.SLIT_NUM	= SLIT_NUM
		RAW_FILES.HVOLT		= HVOLT
		RAW_FILES.LLD		= LLD
		RAW_FILES.CNT_RATE	= CNT_RATE
		RAW_FILES.RAW_FILE	= RAW_FILE
		RAW_FILES.FFB		= FFB
		RAW_FILES.FIL_ID	= FIL_ID
		RAW_FILES.FIL_CUR	= FIL_CUR
		RAW_FILES.ZONE_ID	= ZONE_ID
	ENDELSE
	GOTO, FINISH
;
;  Error handling point.
;
HANDLE_ERROR:
	IF N_ELEMENTS(ERRMSG) NE 0 THEN ERRMSG = 'LIST_RAW: ' + MESSAGE $
		ELSE MESSAGE, MESSAGE, /CONTINUE
;
;  Close the database and return.
;
FINISH:
	DBCLOSE
;
	RETURN
	END
