	PRO LIST_V_RASTER, RAS_ID, RASTERS, N_FOUND, USABLE=USE_ABLE,	$
		ALL=ALL, ERRMSG=ERRMSG
;+
; Project     :	SOHO - CDS
;
; Name        :	LIST_V_RASTER
;
; Purpose     :	List the defined variations for a given raster ID.
;
; Explanation :	Extracts the variable parameters of all the rasters in the
;		database with a specific raster ID.
;
; Use         :	LIST_V_RASTER, RAS_ID, RASTERS, N_FOUND
;
; Inputs      :	RAS_ID = Raster ID number.
;
; Opt. Inputs :	None.
;
; Outputs     :	RASTERS	= A structure variable of type "cds_v_raster"
;			  containing the following tags for each raster:
;
;			RAS_ID	   = Raster ID number.
;			RAS_VAR	   = Raster variation index.
;			RV_DESC	   = A short description of the raster
;				     variation beyond what is given in the
;				     associated fundamental raster description.
;			EXPTIME	   = Exposure time in seconds, to millisecond
;				     accuracy.
;			LL_ID	   = Line list ID.  Alternatively, this can be
;				     zero to signal that the raster is not
;				     connected to a line list.
;			COMP_ID	   = Compression method ID.
;			COMP_OPT   = Compression option parameter.
;			DW_ID	   = Data extraction window list ID.
;			IEF_ID	   = Inter-instrument event flag ID.
;			VDS_ORIENT = VDS orientation, either 0 (row) or 1
;				     (column).
;			VDS_MAP	   = VDS mapping mode: 2=Normal, 3=Accumulate
;			TEL_RATE   = Estimated required telemetry rate, as a
;				     character representing Low/Medium/High
;			DURATION   = Estimated duration of the raster, in
;				     seconds.
;			USABLE	   = Either "Y" or "N" to signal whether or not
;				     the raster is usable.
;
;		N_FOUND	= Number of rasters found.
;
; Opt. Outputs:	None.
;
; Keywords    :	
;       USABLE	  = If set, then only those entries with USABLE=Y will be
;		    returned.  This is the default behavior.
;
;	ALL	  = If set, then all entries matching the search criteria will
;		    be returned, regardless of the value of USABLE.
;
;       ERRMSG    = If defined and passed, then any error messages will be
;                   returned to the user in this parameter rather than
;                   depending on the MESSAGE routine in IDL.  If no errors are
;                   encountered, then a null string is returned.  In order to
;                   use this feature, ERRMSG must be defined first, e.g.
;
;                       ERRMSG = ''
;                       LIST_V_RASTER, ERRMSG=ERRMSG, ... 
;                       IF ERRMSG NE '' THEN ...
;
;
; Calls       :	DATATYPE, DBOPEN, DBFIND, DBEXT, DBCLOSE
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	If the number of rasters found is zero, then the output
;		parameter RASTERS is not modified.
;
; Category    :	Planning, Database.
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 5 April 1994
;
; Modified    :	Version 1, William Thompson, GSFC, 29 July 1994
;               Version 2, Liyun Wang, GSFC/ARC, September 22, 1994
;                  Added the keyword ERRMSG.
;		Version 3, William Thompson, GSFC, 29 December 1994
;			Fixed bug when inputs are bytes.
;		Version 4, William Thompson, GSFC, 15 March 1995
;			Changed TEL_RATE to character
;		Version 5, William Thompson, GSFC, 27 October 1995
;			Added item IEF_ID.
;		Version 6, William Thompson, GSFC, 26 April 1996
;			Added keyword /ALL.  Default behavior is to now only
;			list usable rasters.
;
; Version     :	Version 6, 26 April 1996
;-
;
	ON_ERROR, 2
;
;  Check the number of parameters.
;
        IF N_PARAMS() LT 3 THEN BEGIN
           MESSAGE = 'Syntax:  LIST_V_RASTER, RAS_ID, RASTERS, N_FOUND'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN BEGIN
              ERRMSG = MESSAGE
              RETURN
           ENDIF ELSE MESSAGE, MESSAGE
        ENDIF
;
;  Check the input parameter RAS_ID.
;
	TYPE = DATATYPE(RAS_ID,2)
	IF TYPE EQ 0 THEN BEGIN
           MESSAGE = 'RAS_ID is undefined'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           N_FOUND = 0
           GOTO, FINISH
	END ELSE IF TYPE GE 4 THEN BEGIN
           MESSAGE = 'RAS_ID must be an integer'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           N_FOUND = 0
           GOTO, FINISH
	END ELSE IF N_ELEMENTS(RAS_ID) NE 1 THEN BEGIN
           MESSAGE = 'RAS_ID must be a scalar'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           N_FOUND = 0
           GOTO, FINISH
	ENDIF
;
;  Open the raster variation database.
;
	DBOPEN, 'raster_var,raster'
;
;  Search for entries with the requested raster ID number.  Unless the ALL
;  keyword was set, then only select entries with USABLE=Y.
;
	TEST = 'RAS_ID=' + STRTRIM(LONG(RAS_ID),2)
	IF NOT KEYWORD_SET(ALL) THEN TEST = TEST + ',USABLE=Y'
	ENTRIES = DBFIND(TEST, /SILENT)
	IF !ERR EQ 0 THEN BEGIN
		N_FOUND = 0
		GOTO, FINISH
	END ELSE N_FOUND = N_ELEMENTS(ENTRIES)
;
;  Extract the data from the raster database.
;
	DBEXT, ENTRIES, 'ras_var,rv_desc,exptime,ll_id,comp_id,comp_opt,' + $
		'dw_id,tel_rate,duration,usable', RAS_VAR, RV_DESC, $
		EXPTIME, LL_ID, COMP_ID, COMP_OPT, DW_ID, TEL_RATE, $
		DURATION, USABLE
	DBEXT, ENTRIES, 'ief_id,vds_orient,vds_map', IEF_ID, VDS_ORIENT, $
		VDS_MAP
;
;  Define the structure that the data will be returned in.
;
	RASTERS = {CDS_V_RASTER,	$
		RAS_ID: 0,		$
		RAS_VAR: 0,		$
		RV_DESC: 'String',	$
		EXPTIME: 0.0,		$
		LL_ID: 0,		$
		COMP_ID: 0,		$
		COMP_OPT: 0,		$
		DW_ID: 0,		$
		IEF_ID: 0,		$
		VDS_ORIENT: 0,		$
		VDS_MAP: 0,		$
		TEL_RATE: 'C',		$
		DURATION: 0.0,		$
		USABLE: 'C'}
;
	IF N_FOUND EQ 1 THEN BEGIN
		RASTERS.RAS_ID     = RAS_ID(0)
		RASTERS.RAS_VAR    = RAS_VAR(0)
		RASTERS.RV_DESC	   = RV_DESC(0)
		RASTERS.EXPTIME    = EXPTIME(0)
		RASTERS.LL_ID	   = LL_ID(0)
		RASTERS.COMP_ID	   = COMP_ID(0)
		RASTERS.COMP_OPT   = COMP_OPT(0)
		RASTERS.DW_ID	   = DW_ID(0)
		RASTERS.IEF_ID	   = IEF_ID(0)
		RASTERS.VDS_ORIENT = VDS_ORIENT(0)
		RASTERS.VDS_MAP	   = VDS_MAP(0)
		RASTERS.TEL_RATE   = TEL_RATE(0)
		RASTERS.DURATION   = DURATION(0)
		RASTERS.USABLE     = USABLE(0)
	END ELSE BEGIN
		RASTERS = REPLICATE(RASTERS, N_FOUND)
		RASTERS.RAS_ID     = RAS_ID
		RASTERS.RAS_VAR    = RAS_VAR
		RASTERS.RV_DESC	   = RV_DESC
		RASTERS.EXPTIME    = EXPTIME
		RASTERS.LL_ID	   = LL_ID
		RASTERS.COMP_ID	   = COMP_ID
		RASTERS.COMP_OPT   = COMP_OPT
		RASTERS.DW_ID	   = DW_ID
		RASTERS.IEF_ID	   = IEF_ID
		RASTERS.VDS_ORIENT = VDS_ORIENT
		RASTERS.VDS_MAP	   = VDS_MAP
		RASTERS.TEL_RATE   = TEL_RATE
		RASTERS.DURATION   = DURATION
		RASTERS.USABLE     = USABLE
	ENDELSE
;
;  Close the database and return.
;
FINISH:
	DBCLOSE
;
	RETURN
	END
