	PRO LIST_V_STUDY, INPUT, STUDIES, N_FOUND, USABLE=USE_ABLE, ALL=ALL, $
		USE_TITLE=USE_TITLE, ERRMSG=ERRMSG
;+
; Project     :	SOHO - CDS
;
; Name        :	LIST_V_STUDY
;
; Purpose     :	List the defined variations for a given study.
;
; Explanation :	Extracts the variable parameters of all the studies in the
;		database with a specific study ID.
;
; Use         :	LIST_V_STUDY, STUDY_ID, STUDIES, N_FOUND
;		LIST_V_STUDY, TITLE_ID, STUDIES, N_FOUND, /USE_TITLE
;		LIST_V_STUDY, OBS_PROG, STUDIES, N_FOUND
;
; Inputs      :	STUDY_ID = Study ID number, or
;		TITLE_ID = Title ID number, or
;		OBS_PROG = Study name.
;
; Opt. Inputs :	None.
;
; Outputs     :	STUDIES	= A structure variable of type "cds_v_study"
;			  containing the following tags for each study:
;
;			STUDY_ID    = Study ID number.
;			TITLE_ID    = Study title ID number.
;			OBS_PROG    = Name of the study
;			TITLE	    = A descriptive title for the study
;			CATEGORY    = The study category, either "T"est,
;				      "S"cience or "C"alibration.
;			N_RASTERS0  = Fixed part of the number of rasters in
;				      the study.
;			VAR_POINT   = Either "Y" or "N" representing whether or
;				      not the pointing can be controlled by the
;				      user.  If "Y" then the pointing is
;				      defined in the plan.  If "N" then the
;				      pointing is embedded within the study.
;			N_RASTER_DEF= Number of rasters in the definition.
;			STUDYVAR    = Study variation index.
;			SV_DESC	    = A short description of the study
;				      variation beyond what is given in the
;				      associated fundamental study description.
;			DURATION0   = Fixed part of the study duration, in
;				      seconds.
;			DURATION1   = Number of seconds/raster for the variable
;				      part of the study duration.
;			ZONE_ID	    = Pointing zone ID.
;			USABLE	    = Either "Y" or "N" to signal whether or
;				      not the study is usable.  Normally "Y".
;
;		N_FOUND	= Number of studies found.
;
; Opt. Outputs:	None.
;
; Keywords    :	
;	USE_TITLE = If set, then the input parameter is the title ID rather
;		    than the study ID.  Not required if OBS_PROG is passed.
;
;       USABLE	  = If set, then only those entries with USABLE=Y will be
;		    returned.  This is the default behavior.
;
;	ALL	  = If set, then all entries matching the search criteria will
;		    be returned, regardless of the value of USABLE.
;
;       ERRMSG    = If defined and passed, then any error messages will be
;                   returned to the user in this parameter rather than
;                   depending on the MESSAGE routine in IDL.  If no errors are
;                   encountered, then a null string is returned.  In order to
;                   use this feature, ERRMSG must be defined first, e.g.
;
;                       ERRMSG = ''
;                       LIST_V_STUDY, ERRMSG=ERRMSG, ... 
;                       IF ERRMSG NE '' THEN ...
;
; Calls       :	DATATYPE, DBOPEN, DBFIND, DBEXT, DBCLOSE
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	If the number of studies found is zero, then the output
;		parameter STUDIES is not modified.
;
; Category    :	Planning, Database.
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 29 April 1994
;
; Modified    :	Version 1, William Thompson, GSFC, 29 July 1994
;               Version 2, Liyun Wang, GSFC/ARC, September 22, 1994
;                  Added the keyword ERRMSG.
;		Version 3, William Thompson, GSFC, 21 November 1994
;			Added item N_RASTER_DEF
;		Version 4, William Thompson, GSFC, 29 December 1994
;			Fixed bug when inputs are bytes.
;		Version 5, William Thompson, GSFC, 10 February 1995
;			Changed N_POINTINGS to VAR_POINT.
;			Removed VAR_RASTERS.
;		Version 6, William Thompson, GSFC, 29 March 1995
;			Added tag ZONE_ID
;		Version 7, William Thompson, GSFC, 12 May 1995
;			Modified to split off separate study_title database
;		Version 8, William Thompson, GSFC, 20 June 1995
;			Added tag FLAGABLE.
;		Version 9, William Thompson, GSFC, 27 October 1995.
;			Removed tag flagable.  Any study can produce a flag if
;			given an IEF_ID.
;		Version 10, William Thompson, GSFC, 26 April 1996
;			Added keyword /ALL.  Default behavior is to now only
;			list usable studies.
;
; Version     :	Version 10, 26 April 1996
;-
;
	ON_ERROR, 2
;
;  Check the number of parameters.
;
	N_FOUND = 0
        IF N_PARAMS() LT 3 THEN BEGIN
           MESSAGE = 'Syntax:  LIST_V_STUDY, STUDY_ID, STUDIES, N_FOUND'
	   GOTO, HANDLE_ERROR
        ENDIF
;
;  Check the input parameter.
;
	IF N_ELEMENTS(INPUT) NE 1 THEN BEGIN
           MESSAGE = 'Input parameter must be a scalar'
           GOTO, HANDLE_ERROR
	END ELSE IF DATATYPE(INPUT,1) EQ 'Structure' THEN BEGIN
           MESSAGE = 'Input parameter must not be a structure'
           GOTO, HANDLE_ERROR
	END ELSE IF DATATYPE(INPUT,1) EQ 'Complex' THEN BEGIN
           MESSAGE = 'Input parameter must not be complex'
           GOTO, HANDLE_ERROR
	ENDIF
;
;  Open the study variation database.
;
	DBOPEN, 'study_var,study_title,study'
;
;  If the first input parameter is a character string, then search on the
;  OBS_PROG field.
;
	TYPE = DATATYPE(INPUT,1)
	IF TYPE EQ 'String' THEN BEGIN
		TEST = 'OBS_PROG=' + INPUT
;
;  Otherwise, if the USE_TITLE keyword was set, then search on the TITLE_ID.
;
	END ELSE IF KEYWORD_SET(USE_TITLE) THEN BEGIN
		TEST = 'TITLE_ID=' + STRTRIM(LONG(INPUT),2)
;
;  Otherwise, search on the STUDY_ID field.
;
	END ELSE BEGIN
		TEST = 'STUDY_ID=' + STRTRIM(LONG(INPUT),2)
	ENDELSE
;
;  Unless the ALL keyword was set, then only select entries with USABLE=Y.
;
	IF NOT KEYWORD_SET(ALL) THEN TEST = TEST + ',USABLE=Y'
	ENTRIES = DBFIND(TEST, /SILENT, /FULLSTRING)
;
;  If no entries were found, then return immediately.
;
	IF !ERR EQ 0 THEN BEGIN
		N_FOUND = 0
		GOTO, FINISH
	END ELSE N_FOUND = N_ELEMENTS(ENTRIES)
;
;  Extract the data from the study database.
;
	DBEXT, ENTRIES, 'studyvar,sv_desc,duration0,duration1,usable,' + $
		'study_id,title_id,obs_prog,title,category,n_rasters0', $
		STUDYVAR, SV_DESC, DURATION0, $
		DURATION1, USABLE, STUDY_ID, TITLE_ID, OBS_PROG, TITLE,	$
		CATEGORY, N_RASTERS0
	DBEXT, ENTRIES, 'var_point,n_raster_def,zone_id',	$
		VAR_POINT, N_RASTER_DEF, ZONE_ID
;
;  Define the structure that the data will be returned in.
;
	STUDIES = {CDS_V_STUDY,		$
		STUDY_ID: 0,		$
		TITLE_ID: 0,		$
		OBS_PROG: 'String',	$
		TITLE: 'String',	$
		CATEGORY: 'C',		$
		N_RASTERS0: 0,		$
		VAR_POINT: 'C',		$
		N_RASTER_DEF: 0,	$
		STUDYVAR: 0,		$
		SV_DESC: 'String',	$
		DURATION0: 0.0,		$
		DURATION1: 0.0,		$
		ZONE_ID: 0,		$
		USABLE: 'C'}
;
	IF N_FOUND EQ 1 THEN BEGIN
		STUDIES.STUDY_ID    = STUDY_ID(0)
		STUDIES.TITLE_ID    = TITLE_ID(0)
		STUDIES.OBS_PROG    = OBS_PROG(0)
		STUDIES.TITLE       = TITLE(0)
		STUDIES.CATEGORY    = CATEGORY(0)
		STUDIES.N_RASTERS0  = N_RASTERS0(0)
		STUDIES.VAR_POINT   = VAR_POINT(0)
		STUDIES.N_RASTER_DEF= N_RASTER_DEF(0)
		STUDIES.STUDYVAR    = STUDYVAR(0)
		STUDIES.SV_DESC     = SV_DESC(0)
		STUDIES.DURATION0   = DURATION0(0)
		STUDIES.DURATION1   = DURATION1(0)
		STUDIES.ZONE_ID	    = ZONE_ID(0)
		STUDIES.USABLE	    = USABLE(0)
	END ELSE BEGIN
		STUDIES = REPLICATE(STUDIES, N_FOUND)
		STUDIES.STUDY_ID    = STUDY_ID
		STUDIES.TITLE_ID    = TITLE_ID
		STUDIES.OBS_PROG    = OBS_PROG
		STUDIES.TITLE       = TITLE
		STUDIES.CATEGORY    = CATEGORY
		STUDIES.N_RASTERS0  = N_RASTERS0
		STUDIES.VAR_POINT   = VAR_POINT
		STUDIES.N_RASTER_DEF= N_RASTER_DEF
		STUDIES.STUDYVAR    = STUDYVAR
		STUDIES.SV_DESC     = SV_DESC
		STUDIES.DURATION0   = DURATION0
		STUDIES.DURATION1   = DURATION1
		STUDIES.ZONE_ID	    = ZONE_ID
		STUDIES.USABLE	    = USABLE
	ENDELSE
	GOTO, FINISH
;
;  Error handling point.
;
HANDLE_ERROR:
	IF N_ELEMENTS(ERRMSG) NE 0 THEN ERRMSG = 'LIST_V_STUDY: ' + MESSAGE $
		ELSE MESSAGE, MESSAGE, /CONTINUE
;
;  Close the database and return.
;
FINISH:
	DBCLOSE
;
	RETURN
	END
