	FUNCTION MOD_CATALOG, OLD, NEW, ERRMSG=ERRMSG
;+
; Project     :	SOHO - CDS
;
; Name        :	MOD_CATALOG()
;
; Purpose     :	Modify a catalog entry
;
; Category    :	Catalog
;
; Explanation :	Called from XCAT to modify a catalog entry.  The routines
;		MOD_MAIN and MOD_EXPER are called to modify the database
;		entries.
;
; Syntax      :	Result = MOD_CATALOG( OLD, NEW )
;
; Examples    :	IF NOT MOD_CATALOG( OLD, NEW ) THEN ...
;
; Inputs      :	OLD = Old data structure, as returned by GET_EXPER or
;		      LIST_EXPER.
;
;		NEW = New data structure, as modified by the XSTRUCT program.
;
; Opt. Inputs :	None.
;
; Outputs     :	The result of the function is either 1 for success or 0 for
;		failure.
;
;		NEW: If the modification is unsuccessful, then the NEW
;		structure is replaced with the OLD structure.  This can then be
;		redisplayed with XSTRUCT.
;
;		OLD: If the modification is successful, then the OLD structure
;		is reread from the database.  This can then be put back into
;		the array stored in the XCAT common block.
;
; Opt. Outputs:	None.
;
; Keywords    :	ERRMSG = If defined and passed, then any error messages will be
;			 returned to the user in this parameter rather than
;			 depending on the MESSAGE routine in IDL.  If no errors
;			 are encountered, then a null string is returned.  In
;			 order to use this feature, ERRMSG must be defined
;			 first, e.g.
;
;				ERRMSG = ''
;				Result = MOD_CATALOG( ERRMSG=ERRMSG, ... )
;				IF ERRMSG NE '' THEN ...
;

;
; Calls       :	MOD_MAIN, MOD_
;
; Common      :	None.
;
; Restrictions:	Not all parameters can be modified.  The parameters PROG_NUM,
;		SEQ_NUM, and SEQ_IND cannot be changed, because they refer to
;		the location of the data in the database.  This routine also
;		cannot be used to change the COMMENTS or DATE_MOD
;		parameter--any changes in the structure will be ignore.
;
;		Must have write privilege into catalog database files.
;
; Side effects:	None.
;
; Prev. Hist. :	None.
;
; History     :	Version 1, 06-Mar-1996, William Thompson, GSFC
;		Version 2, 11-Mar-1996, William Thompson, GSFC
;			Times are TAI in both new and old structures.
;		Version 3, 02-Apr-1996, William Thompson, GSFC
;			Modified for version 3 of the catalog databases.
;
; Contact     :	WTHOMPSON
;-
;
	ON_ERROR, 2
;
;  Initialize RESULT to represent non-success.  If the routine is successful,
;  this value will be updated below.  Also initialize MESSAGE.
;
	RESULT = 0
	MESSAGE = ''
;
;  Make sure that one isn't trying to change PROG_NUM, SEQ_NUM, or SEQ_IND.
;
	IF OLD.PROG_NUM NE NEW.PROG_NUM THEN BEGIN
		MESSAGE = 'Cannot change PROG_NUM'
		GOTO, HANDLE_ERROR
	END ELSE IF OLD.SEQ_NUM NE NEW.SEQ_NUM THEN BEGIN
		MESSAGE = 'Cannot change SEQ_NUM'
		GOTO, HANDLE_ERROR
	END ELSE IF OLD.SEQ_IND NE NEW.SEQ_IND THEN BEGIN
		MESSAGE = 'Cannot change SEQ_IND'
		GOTO, HANDLE_ERROR
	ENDIF
;
;  Extract those parameters which are specific to the MAIN catalog.
;
	MAIN1 ={PROG_NUM:	NEW.PROG_NUM,		$
		PROG_ID:	FIX(NEW.PROG_ID),	$
		PROG_IND:	FIX(NEW.PROG_IND),	$
		STUDY_ID:	FIX(NEW.STUDY_ID),	$
		STUDYVAR:	FIX(NEW.STUDYVAR),	$
		OBS_PROG:	STRTRIM(NEW.OBS_PROG,2),$
		DETECTOR:	STRMID(NEW.DETECTOR,0,1),$
		SCI_OBJ:	NEW.SCI_OBJ,		$
		SCI_SPEC:	NEW.SCI_SPEC,		$
		CMP_NO:		FIX(NEW.CMP_NO),	$
		OBJECT:		STRTRIM(NEW.OBJECT,2),	$
		OBJ_ID:		STRTRIM(NEW.OBJ_ID,2),	$
		DATE_OBS:	NEW.DATE_OBS,		$
		DATE_END:	NEW.DATE_END,		$
		OBT_TIME:	NEW.OBT_TIME,		$
		OBT_END:	NEW.OBT_END,		$
		XCEN:		FLOAT(NEW.XCEN),	$
		YCEN:		FLOAT(NEW.YCEN),	$
		ANGLE:		FLOAT(NEW.ANGLE),	$
		IXWIDTH:	FLOAT(NEW.IXWIDTH),	$
		IYWIDTH:	FLOAT(NEW.IYWIDTH),	$
		SEQ_FROM:	NEW.SEQ_NUM,		$
		SEQ_TO:		NEW.SEQ_NUM,		$
		COMMENTS:	'N'}
;
;  Get the corresponding entry from the MAIN database.
;
	GET_MAIN, MAIN1.PROG_NUM, MAIN0
	IF MAIN0.PROG_NUM LT 0 THEN BEGIN
		MESSAGE = 'PROG_NUM=' + TRIM(MAIN1.PROG_NUM) + ' not found'
		GOTO, HANDLE_ERROR
	ENDIF
;
;  Check the old entry against the new to see if anything needs to be updated.
;
	MODIFY = 0
	IF MAIN0.PROG_ID NE MAIN1.PROG_ID THEN MODIFY = 1
	IF MAIN0.PROG_IND NE MAIN1.PROG_IND THEN MODIFY = 1
	IF MAIN0.STUDY_ID NE MAIN1.STUDY_ID THEN MODIFY = 1
	IF MAIN0.STUDYVAR NE MAIN1.STUDYVAR THEN MODIFY = 1
	IF TRIM(MAIN0.OBS_PROG) NE TRIM(MAIN1.OBS_PROG) THEN MODIFY = 1
	IF (TRIM(MAIN0.DETECTOR) NE TRIM(MAIN1.DETECTOR)) AND	$
		(MAIN0.DETECTOR NE 'B') THEN MODIFY = 1
	IF TRIM(MAIN0.SCI_OBJ) NE TRIM(MAIN1.SCI_OBJ) THEN MODIFY = 1
	IF TRIM(MAIN0.SCI_SPEC) NE TRIM(MAIN1.SCI_SPEC) THEN MODIFY = 1
	IF MAIN0.CMP_NO NE MAIN1.CMP_NO THEN MODIFY = 1
	IF TRIM(MAIN0.OBJECT) NE TRIM(MAIN1.OBJECT) THEN MODIFY = 1
	IF TRIM(MAIN0.OBJ_ID) NE TRIM(MAIN1.OBJ_ID) THEN MODIFY = 1
;
;  Calculate the smallest box which encompasses all the data.  The assumption
;  is made that ANGLE does not change.  First convert the data to the
;  instrument coordinate system.
;
	A = MAIN0.ANGLE / !RADEG
	COS_A = COS(A)
	SIN_A = SIN(A)
	X0 =  MAIN0.XCEN * COS_A + MAIN0.YCEN * SIN_A
	Y0 = -MAIN0.XCEN * SIN_A + MAIN0.YCEN * COS_A
	X1 =  MAIN1.XCEN * COS_A + MAIN1.YCEN * SIN_A
	Y1 = -MAIN1.XCEN * SIN_A + MAIN1.YCEN * COS_A
;
;  Calculate the old and new boundaries, and see if a change needs to be made.
;
	XMIN0 = X0 - MAIN0.IXWIDTH/2  &  XMAX0 = X0 + MAIN0.IXWIDTH/2
	XMIN1 = X1 - MAIN1.IXWIDTH/2  &  XMAX1 = X1 + MAIN1.IXWIDTH/2
	YMIN0 = Y0 - MAIN0.IYWIDTH/2  &  YMAX0 = Y0 + MAIN0.IYWIDTH/2
	YMIN1 = Y1 - MAIN1.IYWIDTH/2  &  YMAX1 = Y1 + MAIN1.IYWIDTH/2
	MODIFY_BOX = 0
	IF (XMIN1 LT XMIN0) OR (XMAX1 GT XMAX0) THEN BEGIN
		MODIFY_BOX = 1
		XMIN = XMIN0 < XMIN1
		XMAX = XMAX0 > XMAX1
		MAIN1.IXWIDTH = XMAX - XMIN
		X0 = (XMIN + XMAX) / 2
	ENDIF ELSE BEGIN
		MAIN1.XCEN    = MAIN0.XCEN
		MAIN1.IXWIDTH = MAIN0.IXWIDTH
	ENDELSE
	IF (YMIN1 LT YMIN0) OR (YMAX1 GT YMAX0) THEN BEGIN
		MODIFY_BOX = 1
		YMIN = YMIN0 < YMIN1
		YMAX = YMAX0 > YMAX1
		MAIN1.IYWIDTH = YMAX - YMIN
		Y0 = (YMIN + YMAX) / 2
	ENDIF ELSE BEGIN
		MAIN1.YCEN    = MAIN0.YCEN
		MAIN1.IYWIDTH = MAIN0.IYWIDTH
	ENDELSE
;
;  If the box needs to be modified, then calculate the new X and Y positions in
;  solar coordinates.
;
	IF MODIFY_BOX THEN BEGIN
		MODIFY = 1
		MAIN1.XCEN = X0 * COS_A - Y0 * SIN_A
		MAIN1.YCEN = X0 * SIN_A + Y0 * COS_A
	ENDIF
;
;  Use the smallest range of times which includes all the values.
;
	IF MAIN0.DATE_OBS GT MAIN1.DATE_OBS THEN MODIFY = 1 ELSE $
		MAIN1.DATE_OBS = MAIN0.DATE_OBS
	IF MAIN0.DATE_END LT MAIN1.DATE_END THEN MODIFY = 1 ELSE $
		MAIN1.DATE_END = MAIN0.DATE_END
;
	IF MAIN0.OBT_TIME GT MAIN1.OBT_TIME THEN MODIFY = 1 ELSE $
		MAIN1.OBT_TIME = MAIN0.OBT_TIME
	IF MAIN0.OBT_END  LT MAIN1.OBT_END  THEN MODIFY = 1 ELSE $
		MAIN1.OBT_END  = MAIN0.OBT_END
;
;  Use the smallest range of sequence numbers which includes all the values.
;
	IF MAIN0.SEQ_FROM GT MAIN1.SEQ_FROM THEN MODIFY = 1 ELSE $
		MAIN1.SEQ_FROM = MAIN0.SEQ_FROM
	IF MAIN0.SEQ_TO LT MAIN1.SEQ_TO THEN MODIFY = 1 ELSE	$
		MAIN1.SEQ_TO = MAIN0.SEQ_TO
;
;  Look at the old entry to determine if there are any comments or not.
;
	MAIN1.COMMENTS = MAIN0.COMMENTS
;
;  If MODIFY was set, then call MOD_MAIN.
;
	IF MODIFY THEN BEGIN
		PRINT, 'Calling MOD_MAIN'
		IF MOD_MAIN(MAIN1, ERRMSG=MESSAGE) LT 0 THEN	$
			GOTO, HANDLE_ERROR
	ENDIF
;
;  Extract the information for the EXPERIMENT database.
;
	EXP1 = {PROG_NUM:	NEW.PROG_NUM,		$
		SEQ_NUM:	NEW.SEQ_NUM,		$
		OBS_SEQ:	NEW.OBS_SEQ,		$
		COUNT:		1,			$
		SEQ_IND:	FIX(NEW.SEQ_IND),	$
		RAS_ID:		FIX(NEW.RAS_ID),	$
		RAS_VAR:	FIX(NEW.RAS_VAR),	$
		EXPTIME:	FLOAT(NEW.EXPTIME),	$
		OBS_MODE:	NEW.OBS_MODE,		$
		DW_ID:		FIX(NEW.DW_ID),		$
		DATE_OBS:	NEW.DATE_OBS,		$
		DATE_END:	NEW.DATE_END,		$
		OBT_TIME:	NEW.OBT_TIME,		$
		OBT_END:	NEW.OBT_END,		$
		XCEN:		FLOAT(NEW.XCEN),	$
		YCEN:		FLOAT(NEW.YCEN),	$
		ANGLE:		FLOAT(NEW.ANGLE),	$
		IXWIDTH:	FLOAT(NEW.IXWIDTH),	$
		IYWIDTH:	FLOAT(NEW.IYWIDTH),	$
		INS_X0:		FLOAT(NEW.INS_X0),	$
		INS_Y0:		FLOAT(NEW.INS_Y0),	$
		INS_ROLL:	FLOAT(NEW.INS_ROLL),	$
		SC_X0:		FLOAT(NEW.SC_X0),	$
		SC_Y0:		FLOAT(NEW.SC_Y0),	$
		SC_ROLL:	FLOAT(NEW.SC_ROLL),	$
		WAVEMIN:	FLOAT(NEW.WAVEMIN),	$
		WAVEMAX:	FLOAT(NEW.WAVEMAX),	$
		TRACKING:	BYTE(NEW.TRACKING),	$
		SER_ID:		FIX(NEW.SER_ID),	$
		OPSLBITS:	STRTRIM(NEW.OPSLBITS,2),$
		OPSRBITS:	STRTRIM(NEW.OPSRBITS,2),$
		SLIT_POS:	FIX(NEW.SLIT_POS),	$
		MIR_POS:	FIX(NEW.MIR_POS),	$
		EV_ENAB:	BYTE(NEW.EV_ENAB),	$
		COMP_ERR:	BYTE(NEW.COMP_ERR),	$
		VDS_PMCP:	FIX(NEW.VDS_PMCP),	$
		VDS_MODE:	FIX(NEW.VDS_MODE),	$
		VDS_ORI:	BYTE(NEW.VDS_ORI),	$
		VDS_ACC:	BYTE(NEW.VDS_ACC),	$
		GSET_ID:	FIX(NEW.GSET_ID),	$
		DETECTOR:	STRMID(NEW.DETECTOR,0,1),$
		ZONE_ID:	FIX(NEW.ZONE_ID),	$
		SLIT_NUM:	FIX(NEW.SLIT_NUM),	$
		FILENAME:	STRTRIM(NEW.FILENAME,2),$
		SEQVALID:	BYTE(NEW.SEQVALID),	$
		DATASRC:	STRMID(NEW.DATASRC,0,1),$
		COMMENTS:	'N'}
;
;  Get the corresponding entry, if any, from the EXPERIMENT database.
;
	GET_EXPER, EXP1.SEQ_NUM, EXP0
	IF EXP0.SEQ_NUM LT 0 THEN BEGIN
		MESSAGE = 'SEQ_NUM=' + TRIM(SEQ_NUM) + ' not found'
		GOTO, HANDLE_ERROR
	ENDIF
;
;  Check the old entry against the new to see if anything needs to be updated.
;
	MODIFY = 0
	IF TRIM(EXP0.OBS_SEQ) NE TRIM(EXP1.OBS_SEQ) THEN MODIFY = 1
	IF EXP0.COUNT NE EXP1.COUNT THEN MODIFY = 1
	IF EXP0.RAS_ID NE EXP1.RAS_ID THEN MODIFY = 1
	IF EXP0.RAS_VAR NE EXP1.RAS_VAR THEN MODIFY = 1
	IF EXP0.EXPTIME NE EXP1.EXPTIME THEN MODIFY = 1
	IF TRIM(EXP0.OBS_MODE) NE TRIM(EXP1.OBS_MODE) THEN MODIFY = 1
	IF EXP0.DW_ID NE EXP1.DW_ID THEN MODIFY = 1
	IF EXP0.DATE_OBS NE EXP1.DATE_OBS THEN MODIFY = 1
	IF EXP0.DATE_END NE EXP1.DATE_END THEN MODIFY = 1
	IF EXP0.OBT_TIME NE EXP1.OBT_TIME THEN MODIFY = 1
	IF EXP0.OBT_END NE EXP1.OBT_END THEN MODIFY = 1
	IF EXP0.XCEN NE EXP1.XCEN THEN MODIFY = 1
	IF EXP0.YCEN NE EXP1.YCEN THEN MODIFY = 1
	IF EXP0.ANGLE NE EXP1.ANGLE THEN MODIFY = 1
	IF EXP0.IXWIDTH NE EXP1.IXWIDTH THEN MODIFY = 1
	IF EXP0.IYWIDTH NE EXP1.IYWIDTH THEN MODIFY = 1
	IF EXP0.INS_X0 NE EXP1.INS_X0 THEN MODIFY = 1
	IF EXP0.INS_Y0 NE EXP1.INS_Y0 THEN MODIFY = 1
	IF EXP0.INS_ROLL NE EXP1.INS_ROLL THEN MODIFY = 1
	IF EXP0.SC_X0 NE EXP1.SC_X0 THEN MODIFY = 1
	IF EXP0.SC_Y0 NE EXP1.SC_Y0 THEN MODIFY = 1
	IF EXP0.SC_ROLL NE EXP1.SC_ROLL THEN MODIFY = 1
	IF EXP0.WAVEMIN NE EXP1.WAVEMIN THEN MODIFY = 1
	IF EXP0.WAVEMAX NE EXP1.WAVEMAX THEN MODIFY = 1
	IF EXP0.TRACKING NE EXP1.TRACKING THEN MODIFY = 1
	IF EXP0.SER_ID NE EXP1.SER_ID THEN MODIFY = 1
	IF EXP0.OPSLBITS NE EXP1.OPSLBITS THEN MODIFY = 1
	IF EXP0.OPSRBITS NE EXP1.OPSRBITS THEN MODIFY = 1
	IF EXP0.SLIT_POS NE EXP1.SLIT_POS THEN MODIFY = 1
	IF EXP0.MIR_POS NE EXP1.MIR_POS THEN MODIFY = 1
	IF EXP0.EV_ENAB NE EXP1.EV_ENAB THEN MODIFY = 1
	IF EXP0.COMP_ERR NE EXP1.COMP_ERR THEN MODIFY = 1
	IF EXP0.VDS_PMCP NE EXP1.VDS_PMCP THEN MODIFY = 1
	IF EXP0.VDS_MODE NE EXP1.VDS_MODE THEN MODIFY = 1
	IF EXP0.VDS_ORI NE EXP1.VDS_ORI THEN MODIFY = 1
	IF EXP0.VDS_ACC NE EXP1.VDS_ACC THEN MODIFY = 1
	IF EXP0.GSET_ID NE EXP1.GSET_ID THEN MODIFY = 1
	IF EXP0.DETECTOR NE EXP1.DETECTOR THEN MODIFY = 1
	IF EXP0.ZONE_ID NE EXP1.ZONE_ID THEN MODIFY = 1
	IF EXP0.SLIT_NUM NE EXP1.SLIT_NUM THEN MODIFY = 1
	IF TRIM(EXP0.FILENAME) NE TRIM(EXP1.FILENAME) THEN MODIFY = 1
	IF EXP0.SEQVALID NE EXP1.SEQVALID THEN MODIFY = 1
	IF EXP0.DATASRC NE EXP1.DATASRC THEN MODIFY = 1
;
;  Look at the old entry to determine if there are any comments or not.
;
	EXP1.COMMENTS = EXP0.COMMENTS
;
;  If MODIFY was set, then call MOD_EXPER.
;
	IF MODIFY THEN BEGIN
		PRINT, 'Calling MOD_EXPER'
		IF MOD_EXPER(EXP1, ERRMSG=MESSAGE) LT 0 THEN	$
			GOTO, HANDLE_ERROR
	ENDIF
;
;  If we reached here, then we must be successful.  Reread the old structure
;  from the database.
;
	GET_EXPER, EXP0.SEQ_NUM, OLD
	GOTO, FINISH
;
;  Error handling point.  Replace the NEW structure with the contents of OLD.
;
HANDLE_ERROR:
	IF N_ELEMENTS(ERRMSG) NE 0 THEN ERRMSG = 'MOD_EXPER: ' + MESSAGE $
		ELSE MESSAGE, MESSAGE, /CONTINUE
	NEW = OLD
;
;  Return whether the routine was successful or not.
;
FINISH:
	RETURN, RESULT
	END
