	FUNCTION MOD_DETAIL, OLD_DEF, NEW_DEF, PROG_NUM=PROG_NUM, ERRMSG=ERRMSG
;+
; Project     :	SOHO - CDS
;
; Name        :	MOD_DETAIL()
;
; Purpose     :	Modifies an entry in the science plan database.
;
; Explanation :	Modifies an entry in the science plan database.  There are two
;		possible calling sequences.  One way to call this is to pass
;		the old database entry, and the new PROG_NUM value through the
;		keyword.  In this mode, the old entry remains, and only the
;		PROG_NUM value is updated.
;
;		The second way this routine can be called is to pass an old
;		entry and a new entry, without a PROG_NUM keyword.  In this
;		mode, the old entry is removed, and the new entry is added.
;
; Use         :	Result = MOD_DETAIL( OLD_DEF, PROG_NUM=PROG_NUM)
;		Result = MOD_DETAIL( OLD_DEF, NEW_DEF )
;
;		IF NOT MOD_DETAIL(OLD_DEF,NEW_DEF) THEN ...
;
; Inputs      :	OLD_DEF	= Previous entry.  This is a structure such as that
;			  returned by GET_DETAIL or LIST_DETAIL.
;
; Opt. Inputs :	NEW_DEF = New entry, as used by ADD_DETAIL.  It has the tags
;
;			PROG_ID	   = Program ID, linking one or more studies
;				     together
;			STUDY_ID   = Number defining the study
;			STUDYVAR   = Number giving the study variation ID
;			SCI_OBJ    = Science objective from the daily science
;				     meeting
;			SCI_SPEC   = Specific science objective from meeting
;			CMP_NO	   = Campaign number
;			OBJECT	   = Code for object planned to be observed
;			OBJ_ID	   = Object identification
;			DATE_OBS   = Date/time of beginning of observation,
;				     in TAI format
;			DATE_END   = Date/time of end of observation, in TAI
;				     format
;			ORIG_DUR   = Original duration of the observation, in
;				     seconds, before being truncated by
;				     following time-tagged observations.
;			N_RASTERS1 = Variable number of rasters parameter
;			TIME_TAGGED= True (1) if the start of the study is to
;				     be a time-tagged event.  Otherwise, the
;				     study will begin immediately after the
;				     previous study.
;			TRACKING   = True (1) if feature tracking to be used,
;				     or false (0) otherwise
;			N_POINTINGS= Number of pointings to use with study.
;			N_REPEAT_S = Number of times to repeat study.
;			FLAG_MASTER= Nonzero if flag master.
;			GSET_ID	   = GSET ID number
;			GET_RAW	   = True if raw data should be collected.
;			PROG_NUM   = Study counter number (negative is
;				     predicted).  This parameter is optional.
;
;		      If N_POINTINGS is greater than 0, then there will be the
;		      additional tag POINTINGS.  This is an array of pointing
;		      positions.  It itself is a structure, of type
;		      PLAN_PNT_STRUC, with the following tags:
;
;			INS_X	   = Pointing in X relative to sun center
;			INS_Y	   = Pointing in Y relative to sun center
;			ZONE_ID	   = Pointing zone ID
;
;		      Any field can be changed, including the start and end
;		      times.  If the PROG_NUM keyword is not passed, then
;		      NEW_DEF is a required input parameter.
;
; Outputs     :	The result of the function is a logical value representing
;		whether or not the operation was successful, where 1 is
;		successful and 0 is unsuccessful.
;
; Opt. Outputs:	None.
;
; Keywords    :	PROG_NUM = The new value for the PROG_NUM parameter.  When
;			   PROG_NUM is passed, then the NEW_DEF parameter must
;			   be omitted.
;
;		ERRMSG = If defined and passed, then any error messages will be
;			 returned to the user in this parameter rather than
;			 depending on the MESSAGE routine in IDL.  If no errors
;			 are encountered, then a null string is returned.  In
;			 order to use this feature, ERRMSG must be defined
;			 first, e.g.
;
;				ERRMSG = ''
;				Result = MOD_DETAIL( ERRMSG=ERRMSG, ... )
;				IF ERRMSG NE '' THEN ...
;
; Calls       :	DEL_DETAIL, ADD_DETAIL, DBOPEN, DBFIND, DBUPDATE
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Category    :	Planning, Databases.
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 17 November 1994
;
; Modified    :	Version 1, William Thompson, GSFC, 17 November 1994
;
; Version     :	Version 1, 17 November 1994
;-
;
	ON_ERROR, 2
;
;  The behavior of the program depends on whether or not the PROG_NUM keyword
;  was passed.  First, the option where it is.
;
	IF N_ELEMENTS(PROG_NUM) EQ 1 THEN BEGIN
;
;  Check the number of parameters.
;
	    IF N_PARAMS() NE 1 THEN BEGIN
		MESSAGE = 'Syntax: MOD_DETAIL, OLD_DEF, PROG_NUM=PROG_NUM'
		GOTO, HANDLE_ERROR
	    ENDIF
;
;  Make sure that the user has privilege to write into the database.
;
	    IF !PRIV LT 2 THEN BEGIN
		MESSAGE = '!PRIV must be 2 or greater to write into the ' + $
			'database'
		GOTO, HANDLE_ERROR
	    ENDIF
;
;  Open the database for update.
;
	    DBOPEN, 'sci_details', 1
;
;  Search for the entry with the start date/time given by the user.
;
	    ENTRIES = DBFIND('DATE_OBS=' + TRIM(OLD_DEF.DATE_OBS,'(F15.3)') + $
		    ',DELETED=N', /SILENT)
;
;  If no entries were found, then return immediately.
;
	    IF !ERR EQ 0 THEN BEGIN
		MESSAGE = 'No such entry in the database'
		GOTO, HANDLE_ERROR
	    ENDIF
;
;  Change the PROG_NUM value.
;
	    DBUPDATE, ENTRIES, 'prog_num', LONG(PROG_NUM)
;
;  Next, the option where PROG_NUM wasn't passed.  This time, the program needs
;  two input parameters.
;
	END ELSE BEGIN
	    IF N_PARAMS() NE 2 THEN BEGIN
		MESSAGE = 'Syntax:  MOD_DETAIL, OLD_DEF, NEW_DEF'
		GOTO, HANDLE_ERROR
	    ENDIF
;
;  First delete the old entry.
;
	    MESSAGE = ''
	    IF NOT DEL_DETAIL( OLD_DEF.DATE_OBS, ERRMSG=MESSAGE) THEN	$
		    GOTO, HANDLE_ERROR
;
;  Next add in the new entry.
;
	    IF NOT ADD_DETAIL( NEW_DEF, ERRMSG=MESSAGE) THEN BEGIN
;
;  If you can't add in the new definition, put back the old definition.
;
		MSG = ''
		IF NOT ADD_DETAIL( OLD_DEF, ERRMSG=MSG) THEN MESSAGE =	$
			"Something's wrong--can't even put back old entry"
		GOTO, HANDLE_ERROR
	    ENDIF
	ENDELSE
;
;  If you got this far, you must be successful.
;
	MESSAGE = ''
	RETURN, 1
;
;  Error handling point.
;
HANDLE_ERROR:
	IF N_ELEMENTS(ERRMSG) NE 0 THEN ERRMSG = 'MOD_DETAIL: ' + MESSAGE $
		ELSE MESSAGE, MESSAGE
	RETURN, 0
	END
