	FUNCTION MOD_EXPER, OBS, ERRMSG=ERRMSG
;+
; Project     :	SOHO - CDS
;
; Name        :	MOD_EXPER()
;
; Purpose     :	Modifies a CDS experiment entry in the as-run database
;
; Explanation :	This procedure takes a CDS raster-level (experiment) as-run
;		entry, compares it against that in the database "exper", and
;		makes any necessary modifications.  Any parameter can be
;		modified, except PROG_NUM, SEQ_NUM, and SEQ_IND.  (Note that
;		COMMENTS can only be changed from "N"(o) to "Y"(es).)
;
; Use         :	Result = MOD_EXPER( OBS )
;
;		IF MOD_EXPER( OBS ) LT 0 THEN ...
;
; Inputs      :	OBS = This is an anonymous structure containing the following
;		      tags:
;
;			PROG_NUM   = Study counter number.
;			SEQ_NUM	   = Unique sequence number.  This serves as
;				     the primary key to the database.
;			OBS_SEQ	   = A character string derived from the raster
;				     number and the variation number,
;				     e.g. "R4V3".
;			COUNT	   = The number of repeated observations making
;				     up this experiment.  Normally 1.
;			SEQ_IND	   = The raster counter
;			RAS_ID	   = The raster ID number
;			RAS_VAR	   = The raster variation index
;			EXPTIME	   = The exposure time, in seconds
;			OBS_MODE   = The observing mode.
;			DW_ID	   = The data window ID number
;			DATE_OBS   = Date/time of beginning of observation,
;				     in TAI format
;			DATE_END   = Date/time of end of observation, in TAI
;				     format
;			OBT_TIME   = Onboard date/time of beginning of
;				     observation, in TAI format
;			OBT_END	   = Onboard date/time of end of observation,
;				     in TAI format
;			XCEN	   = Center of instrument field-of-view in
;				     solar X
;			YCEN	   = Center of instrument field-of-view in
;				     solar Y
;			ANGLE	   = Angle of instrument relative to solar
;				     north
;			IXWIDTH	   = Width of field-of-view in instrument X
;			IYWIDTH	   = Width of field-of-view in instrument Y
;			INS_X0	   = The instrument origin pointing in X
;			INS_Y0	   = The instrument origin pointing in Y
;			INS_ROLL   = The instrument roll angle
;			SC_X0	   = The spacecraft pointing in X
;			SC_Y0	   = The spacecraft pointing in Y
;			SC_ROLL	   = The spacecraft roll
;			WAVEMIN	   = The minimum wavelength of observation, in
;				     Angstroms.
;			WAVEMAX	   = The maximum wavelength, in Angstroms
;			TRACKING   = True if tracking is on
;			SER_ID	   = The series ID number
;			OPSLBITS   = The OPS-L status bits at the start of the
;				     observation.
;			OPSRBITS   = The OPS-R status bits at the start of the
;				     observation.
;			SLIT_POS   = The slit position at the start of the
;				     observation.
;			MIR_POS	   = The mirror position at the start of the
;				     observation.
;			EV_ENAB	   = True if event recognition enabled
;			COMP_ERR   = True if a compression error was
;				     encountered.
;			VDS_PMCP   = The VDS MCP programmable voltage setting
;			VDS_MODE   = The VDS readout mode.
;			VDS_ORI	   = True if the VDS telemetry data is oriented
;				     by columns
;			VDS_ACC	   = True if VDS is operated in accumulate mode
;			GSET_ID	   = GIS Setup ID
;			DETECTOR   = Either N or G for NIS/GIS.
;			ZONE_ID	   = Pointing zone ID
;			SLIT_NUM   = Slit number
;			FILENAME   = FITS file name
;			SEQVALID   = True for valid science data
;			DATASRC	   = Data source, either Q(uicklook) or F(inal
;				     distribution).
;			COMMENTS   = Either Y or N
;
; Opt. Inputs :	None.
;
; Outputs     :	The result of the function is the number of parameters that
;		were modified.  If an error occurs, then -1 is returned
;		instead.
;
; Opt. Outputs:	None.
;
; Keywords    :
;       ERRMSG    = If defined and passed, then any error messages will be
;                   returned to the user in this parameter rather than
;                   depending on the MESSAGE routine in IDL.  If no errors are
;                   encountered, then a null string is returned.  In order to
;                   use this feature, ERRMSG must be defined first, e.g.
;
;                       ERRMSG = ''
;                       Result = MOD_EXPER( ERRMSG=ERRMSG, ... )
;                       IF ERRMSG NE '' THEN ...
;
;
; Calls       :	DATATYPE, DBOPEN, DBBUILD, DBCLOSE, TRIM, GET_OBJECT,
;		GET_CAMPAIGN, GET_PROGRAM, GET_STUDY
;
; Common      :	None.
;
; Restrictions:	Only this routine can be used to modify entries in the MAIN
;		database.  Modifying the database by hand could corrupt its
;		integrity.
;
;		The data types and sizes of the structure elements must match
;		the definitions in the database.
;
;		!PRIV must be 2 or greater to use this routine.
;
; Side effects:	The dates in the structure are rounded off to millisecond
;		accuracy.
;
; Category    :	Planning, Databases.
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 3 July 1995
;
; Modified    :	Version 1, William Thompson, GSFC, 3 July 1995
;		Version 2, William Thompson, GSFC, 21 February 1996
;			Modified to be more robust.  It will complain about not
;			finding IDs in the databases, but will modify the
;			catalog entry anyway.
;		Version 3, William Thompson, GSFC, 23 February 1996
;			Modified for version 2 of the experiment databases.
;		Version 4, William Thompson, GSFC, 27 March 1996
;			Added parameter DATASRC
;		Version 5, William Thompson, GSFC, 9 April 1996
;			Added calls to LOCK_DATABASE, UNLOCK_DATABASE
;		Version 6, William Thompson, GSFC, 12 April 1996
;			Look in both experiment* and fexperiment*
;		Version 7, William Thompson, GSFC, 26 April 1996
;			Fixed bug introduced in version 6 for MOD_DATE
;		Version 8, William Thompson, GSFC, 1 June 1998
;			Look for entries in "_year" files
;
; Version     :	Version 8, 1 June 1998
;-
;
	ON_ERROR, 2
;
;  Initialize RESULT to represent non-success.  If the routine is successful,
;  this value will be updated below.
;
	RESULT = -1
;
;  Check the input parameters
;
        IF N_PARAMS() NE 1 THEN BEGIN
           MESSAGE = 'Syntax:  Result = MOD_EXPER(OBS)'
	   GOTO, HANDLE_ERROR
        ENDIF
;
;  Make sure that the user has privilege to write into the database.
;
	IF !PRIV LT 2 THEN BEGIN
           MESSAGE = '!PRIV must be 2 or greater to write into the database'
           GOTO, HANDLE_ERROR
	ENDIF
;
;  Check each of the structure components to verify that it is of the correct
;  type and size.
;
	IF DATATYPE(OBS.PROG_NUM,1) NE 'Long' THEN BEGIN
	   MESSAGE = 'Tag PROG_NUM must be a long integer'
	   GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(OBS.PROG_NUM) NE 1 THEN BEGIN
	   MESSAGE = 'Tag PROG_NUM must be a scalar'
	   GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(OBS.SEQ_NUM,1) NE 'Long' THEN BEGIN
	   MESSAGE = 'Tag SEQ_NUM must be a long integer'
	   GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(OBS.SEQ_NUM) NE 1 THEN BEGIN
	   MESSAGE = 'Tag SEQ_NUM must be a scalar'
	   GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(OBS.OBS_SEQ,1) NE 'String' THEN BEGIN
	   MESSAGE = 'Tag OBS_SEQ must be a character string'
	   GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(OBS.OBS_SEQ) NE 1 THEN BEGIN
	   MESSAGE = 'Tag OBS_SEQ must be a scalar'
	   GOTO, HANDLE_ERROR
	END ELSE IF STRLEN(OBS.OBS_SEQ) GT 12 THEN BEGIN
	   MESSAGE = 'Tag OBS_SEQ must be 12 characters or less'
	   GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(OBS.COUNT,1) NE 'Integer' THEN BEGIN
	   MESSAGE = 'Tag COUNT must be a short integer'
	   GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(OBS.COUNT) NE 1 THEN BEGIN
	   MESSAGE = 'Tag COUNT must be a scalar'
	   GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(OBS.SEQ_IND,1) NE 'Integer' THEN BEGIN
	   MESSAGE = 'Tag SEQ_IND must be a short integer'
	   GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(OBS.SEQ_IND) NE 1 THEN BEGIN
	   MESSAGE = 'Tag SEQ_IND must be a scalar'
	   GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(OBS.RAS_ID,1) NE 'Integer' THEN BEGIN
	   MESSAGE = 'Tag RAS_ID must be a short integer'
	   GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(OBS.RAS_ID) NE 1 THEN BEGIN
	   MESSAGE = 'Tag RAS_ID must be a scalar'
	   GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(OBS.RAS_VAR,1) NE 'Integer' THEN BEGIN
	   MESSAGE = 'Tag RAS_VAR must be a short integer'
	   GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(OBS.RAS_VAR) NE 1 THEN BEGIN
	   MESSAGE = 'Tag RAS_VAR must be a scalar'
	   GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(OBS.EXPTIME,1) NE 'Float' THEN BEGIN
	   MESSAGE = 'Tag EXPTIME must be a floating point number'
	   GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(OBS.EXPTIME) NE 1 THEN BEGIN
	   MESSAGE = 'Tag EXPTIME must be a scalar'
	   GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(OBS.OBS_MODE,1) NE 'String' THEN BEGIN
	   MESSAGE = 'Tag OBS_MODE must be a character string'
	   GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(OBS.OBS_MODE) NE 1 THEN BEGIN
	   MESSAGE = 'Tag OBS_MODE must be a scalar'
	   GOTO, HANDLE_ERROR
	END ELSE IF STRLEN(OBS.OBS_MODE) GT 20 THEN BEGIN
	   MESSAGE = 'Tag OBS_MODE must be 20 characters or less'
	   GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(OBS.DW_ID,1) NE 'Integer' THEN BEGIN
	   MESSAGE = 'Tag DW_ID must be a short integer'
	   GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(OBS.DW_ID) NE 1 THEN BEGIN
	   MESSAGE = 'Tag DW_ID must be a scalar'
	   GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(OBS.DATE_OBS,1) NE 'Double' THEN BEGIN
	   MESSAGE = 'Tag DATE_OBS must be a double precision number'
	   GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(OBS.DATE_OBS) NE 1 THEN BEGIN
	   MESSAGE = 'Tag DATE_OBS must be a scalar'
	   GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(OBS.DATE_END,1) NE 'Double' THEN BEGIN
	   MESSAGE = 'Tag DATE_END must be a double precision number'
	   GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(OBS.DATE_END) NE 1 THEN BEGIN
	   MESSAGE = 'Tag DATE_END must be a scalar'
	   GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(OBS.OBT_TIME,1) NE 'Double' THEN BEGIN
	   MESSAGE = 'Tag OBT_TIME must be a double precision number'
	   GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(OBS.OBT_TIME) NE 1 THEN BEGIN
	   MESSAGE = 'Tag OBT_TIME must be a scalar'
	   GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(OBS.OBT_END,1) NE 'Double' THEN BEGIN
	   MESSAGE = 'Tag OBT_END must be a double precision number'
	   GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(OBS.OBT_END) NE 1 THEN BEGIN
	   MESSAGE = 'Tag OBT_END must be a scalar'
	   GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(OBS.XCEN,1) NE 'Float' THEN BEGIN
	   MESSAGE = 'Tag XCEN must be a floating point number'
	   GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(OBS.XCEN) NE 1 THEN BEGIN
	   MESSAGE = 'Tag XCEN must be a scalar'
	   GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(OBS.YCEN,1) NE 'Float' THEN BEGIN
	   MESSAGE = 'Tag YCEN must be a floating point number'
	   GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(OBS.YCEN) NE 1 THEN BEGIN
	   MESSAGE = 'Tag YCEN must be a scalar'
	   GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(OBS.ANGLE,1) NE 'Float' THEN BEGIN
	   MESSAGE = 'Tag ANGLE must be a floating point number'
	   GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(OBS.ANGLE) NE 1 THEN BEGIN
	   MESSAGE = 'Tag ANGLE must be a scalar'
	   GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(OBS.IXWIDTH,1) NE 'Float' THEN BEGIN
	   MESSAGE = 'Tag IXWIDTH must be a floating point number'
	   GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(OBS.IXWIDTH) NE 1 THEN BEGIN
	   MESSAGE = 'Tag IXWIDTH must be a scalar'
	   GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(OBS.IYWIDTH,1) NE 'Float' THEN BEGIN
	   MESSAGE = 'Tag IYWIDTH must be a floating point number'
	   GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(OBS.IYWIDTH) NE 1 THEN BEGIN
	   MESSAGE = 'Tag IYWIDTH must be a scalar'
	   GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(OBS.INS_X0,1) NE 'Float' THEN BEGIN
	   MESSAGE = 'Tag INS_X0 must be a floating point number'
	   GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(OBS.INS_X0) NE 1 THEN BEGIN
	   MESSAGE = 'Tag INS_X0 must be a scalar'
	   GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(OBS.INS_Y0,1) NE 'Float' THEN BEGIN
	   MESSAGE = 'Tag INS_Y0 must be a floating point number'
	   GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(OBS.INS_Y0) NE 1 THEN BEGIN
	   MESSAGE = 'Tag INS_Y0 must be a scalar'
	   GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(OBS.INS_ROLL,1) NE 'Float' THEN BEGIN
	   MESSAGE = 'Tag INS_ROLL must be a floating point number'
	   GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(OBS.INS_ROLL) NE 1 THEN BEGIN
	   MESSAGE = 'Tag INS_ROLL must be a scalar'
	   GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(OBS.SC_X0,1) NE 'Float' THEN BEGIN
	   MESSAGE = 'Tag SC_X0 must be a floating point number'
	   GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(OBS.SC_X0) NE 1 THEN BEGIN
	   MESSAGE = 'Tag SC_X0 must be a scalar'
	   GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(OBS.SC_Y0,1) NE 'Float' THEN BEGIN
	   MESSAGE = 'Tag SC_Y0 must be a floating point number'
	   GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(OBS.SC_Y0) NE 1 THEN BEGIN
	   MESSAGE = 'Tag SC_Y0 must be a scalar'
	   GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(OBS.SC_ROLL,1) NE 'Float' THEN BEGIN
	   MESSAGE = 'Tag SC_ROLL must be a floating point number'
	   GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(OBS.SC_ROLL) NE 1 THEN BEGIN
	   MESSAGE = 'Tag SC_ROLL must be a scalar'
	   GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(OBS.WAVEMIN,1) NE 'Float' THEN BEGIN
	   MESSAGE = 'Tag WAVEMIN must be a floating point number'
	   GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(OBS.WAVEMIN) NE 1 THEN BEGIN
	   MESSAGE = 'Tag WAVEMIN must be a scalar'
	   GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(OBS.WAVEMAX,1) NE 'Float' THEN BEGIN
	   MESSAGE = 'Tag WAVEMAX must be a floating point number'
	   GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(OBS.WAVEMAX) NE 1 THEN BEGIN
	   MESSAGE = 'Tag WAVEMAX must be a scalar'
	   GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(OBS.TRACKING,1) NE 'Byte' THEN BEGIN
	   MESSAGE = 'Tag TRACKING must be a short integer'
	   GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(OBS.TRACKING) NE 1 THEN BEGIN
	   MESSAGE = 'Tag TRACKING must be a scalar'
	   GOTO, HANDLE_ERROR
	END ELSE IF (OBS.TRACKING NE 0) AND (OBS.TRACKING NE 1) $
		THEN BEGIN
	   MESSAGE = 'Tag TRACKING must be either 0 or 1'
	   GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(OBS.SER_ID,1) NE 'Integer' THEN BEGIN
	   MESSAGE = 'Tag SER_ID must be a short integer'
	   GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(OBS.SER_ID) NE 1 THEN BEGIN
	   MESSAGE = 'Tag SER_ID must be a scalar'
	   GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(OBS.OPSLBITS,1) NE 'String' THEN BEGIN
	   MESSAGE = 'Tag OPSLBITS must be a character string'
	   GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(OBS.OPSLBITS) NE 1 THEN BEGIN
	   MESSAGE = 'Tag OPSLBITS must be a scalar'
	   GOTO, HANDLE_ERROR
	END ELSE IF STRLEN(OBS.OPSLBITS) NE 4 THEN BEGIN
	   MESSAGE = 'Tag OPSLBITS must be 4 characters'
	   GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(OBS.OPSRBITS,1) NE 'String' THEN BEGIN
	   MESSAGE = 'Tag OPSRBITS must be a character string'
	   GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(OBS.OPSRBITS) NE 1 THEN BEGIN
	   MESSAGE = 'Tag OPSRBITS must be a scalar'
	   GOTO, HANDLE_ERROR
	END ELSE IF STRLEN(OBS.OPSRBITS) NE 4 THEN BEGIN
	   MESSAGE = 'Tag OPSRBITS must be 4 characters'
	   GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(OBS.SLIT_POS,1) NE 'Integer' THEN BEGIN
	   MESSAGE = 'Tag SLIT_POS must be a short integer'
	   GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(OBS.SLIT_POS) NE 1 THEN BEGIN
	   MESSAGE = 'Tag SLIT_POS must be a scalar'
	   GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(OBS.MIR_POS,1) NE 'Integer' THEN BEGIN
	   MESSAGE = 'Tag MIR_POS must be a short integer'
	   GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(OBS.MIR_POS) NE 1 THEN BEGIN
	   MESSAGE = 'Tag MIR_POS must be a scalar'
	   GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(OBS.EV_ENAB,1) NE 'Byte' THEN BEGIN
	   MESSAGE = 'Tag EV_ENAB must be a short integer'
	   GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(OBS.EV_ENAB) NE 1 THEN BEGIN
	   MESSAGE = 'Tag EV_ENAB must be a scalar'
	   GOTO, HANDLE_ERROR
	END ELSE IF (OBS.EV_ENAB NE 0) AND (OBS.EV_ENAB NE 1) $
		THEN BEGIN
	   MESSAGE = 'Tag EV_ENAB must be either 0 or 1'
	   GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(OBS.COMP_ERR,1) NE 'Byte' THEN BEGIN
	   MESSAGE = 'Tag COMP_ERR must be a short integer'
	   GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(OBS.COMP_ERR) NE 1 THEN BEGIN
	   MESSAGE = 'Tag COMP_ERR must be a scalar'
	   GOTO, HANDLE_ERROR
	END ELSE IF (OBS.COMP_ERR NE 0) AND (OBS.COMP_ERR NE 1) $
		THEN BEGIN
	   MESSAGE = 'Tag COMP_ERR must be either 0 or 1'
	   GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(OBS.VDS_PMCP,1) NE 'Integer' THEN BEGIN
	   MESSAGE = 'Tag VDS_PMCP must be a short integer'
	   GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(OBS.VDS_PMCP) NE 1 THEN BEGIN
	   MESSAGE = 'Tag VDS_PMCP must be a scalar'
	   GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(OBS.VDS_MODE,1) NE 'Integer' THEN BEGIN
	   MESSAGE = 'Tag VDS_MODE must be a short integer'
	   GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(OBS.VDS_MODE) NE 1 THEN BEGIN
	   MESSAGE = 'Tag VDS_MODE must be a scalar'
	   GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(OBS.VDS_ORI,1) NE 'Byte' THEN BEGIN
	   MESSAGE = 'Tag VDS_ORI must be a byte value'
	   GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(OBS.VDS_ORI) NE 1 THEN BEGIN
	   MESSAGE = 'Tag VDS_ORI must be a scalar'
	   GOTO, HANDLE_ERROR
	END ELSE IF (OBS.VDS_ORI NE 0) AND (OBS.VDS_ORI NE 1) $
		THEN BEGIN
	   MESSAGE = 'Tag VDS_ORI must be either 0 or 1'
	   GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(OBS.VDS_ACC,1) NE 'Byte' THEN BEGIN
	   MESSAGE = 'Tag VDS_ACC must be a byte value'
	   GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(OBS.VDS_ACC) NE 1 THEN BEGIN
	   MESSAGE = 'Tag VDS_ACC must be a scalar'
	   GOTO, HANDLE_ERROR
	END ELSE IF (OBS.VDS_ACC NE 0) AND (OBS.VDS_ACC NE 1) $
		THEN BEGIN
	   MESSAGE = 'Tag VDS_ACC must be either 0 or 1'
	   GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(OBS.GSET_ID,1) NE 'Integer' THEN BEGIN
	   MESSAGE = 'Tag GSET_ID must be an integer'
	   GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(OBS.GSET_ID) NE 1 THEN BEGIN
	   MESSAGE = 'Tag GSET_ID must be a scalar'
	   GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(OBS.DETECTOR,1) NE 'String' THEN BEGIN
	   MESSAGE = 'Tag DETECTOR must be a character string'
	   GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(OBS.DETECTOR) NE 1 THEN BEGIN
	   MESSAGE = 'Tag DETECTOR must be a scalar'
	   GOTO, HANDLE_ERROR
	END ELSE IF STRLEN(OBS.DETECTOR) NE 1 THEN BEGIN
	   MESSAGE = 'Tag DETECTOR must be a single character'
	   GOTO, HANDLE_ERROR
	END ELSE IF (OBS.DETECTOR NE 'N') AND (OBS.DETECTOR NE 'G') THEN BEGIN
	   MESSAGE = 'Tag DETECTOR must be either G or N'
	   GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(OBS.ZONE_ID,1) NE 'Integer' THEN BEGIN
	   MESSAGE = 'Tag ZONE_ID must be an integer'
	   GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(OBS.ZONE_ID) NE 1 THEN BEGIN
	   MESSAGE = 'Tag ZONE_ID must be a scalar'
	   GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(OBS.SLIT_NUM,1) NE 'Integer' THEN BEGIN
	   MESSAGE = 'Tag SLIT_NUM must be an integer'
	   GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(OBS.SLIT_NUM) NE 1 THEN BEGIN
	   MESSAGE = 'Tag SLIT_NUM must be a scalar'
	   GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(OBS.FILENAME,1) NE 'String' THEN BEGIN
	   MESSAGE = 'Tag FILENAME must be a character string'
	   GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(OBS.FILENAME) NE 1 THEN BEGIN
	   MESSAGE = 'Tag FILENAME must be a scalar'
	   GOTO, HANDLE_ERROR
	END ELSE IF STRLEN(OBS.FILENAME) GT 15 THEN BEGIN
           MESSAGE = 'Tag FILENAME must be no more than 15 characters'
           GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(OBS.SEQVALID,1) NE 'Byte' THEN BEGIN
	   MESSAGE = 'Tag SEQVALID must be a short integer'
	   GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(OBS.SEQVALID) NE 1 THEN BEGIN
	   MESSAGE = 'Tag SEQVALID must be a scalar'
	   GOTO, HANDLE_ERROR
	END ELSE IF (OBS.SEQVALID NE 0) AND (OBS.SEQVALID NE 1) $
		THEN BEGIN
	   MESSAGE = 'Tag SEQVALID must be either 0 or 1'
	   GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(OBS.DATASRC,1) NE 'String' THEN BEGIN
           MESSAGE = 'Tag DATASRC must be a character string'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(OBS.DATASRC) NE 1 THEN BEGIN
           MESSAGE = 'Tag DATASRC must be a scalar'
           GOTO, HANDLE_ERROR
	END ELSE IF STRLEN(OBS.DATASRC) GT 1 THEN BEGIN
           MESSAGE = 'Tag DATASRC must be a single character'
           GOTO, HANDLE_ERROR
	END ELSE IF (OBS.DATASRC NE 'Q') AND (OBS.DATASRC NE 'F') THEN BEGIN
	   MESSAGE = 'Tag DATASRC must be either Q or F'
	   GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(OBS.COMMENTS,1) NE 'String' THEN BEGIN
           MESSAGE = 'Tag COMMENTS must be a character string'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(OBS.COMMENTS) NE 1 THEN BEGIN
           MESSAGE = 'Tag COMMENTS must be a scalar'
           GOTO, HANDLE_ERROR
	END ELSE IF STRLEN(OBS.COMMENTS) GT 1 THEN BEGIN
           MESSAGE = 'Tag COMMENTS must be a single character'
           GOTO, HANDLE_ERROR
	END ELSE IF (OBS.COMMENTS NE 'N') AND (OBS.COMMENTS NE 'Y') THEN BEGIN
	   MESSAGE = 'Tag COMMENTS must be either Y or N'
	   GOTO, HANDLE_ERROR
	ENDIF
;
;  Make sure that the raster ID and raster variation index match an entry in
;  the CDS raster database.
;
	GET_RASTER, OBS.RAS_ID, OBS.RAS_VAR, TEMP
	IF TEMP.RAS_ID LE -1 THEN CDS_MESSAGE, /CONTINUE, /ATTENTION,	$
		'Raster ID/variation ' + TRIM(OBS.RAS_ID) + ',' + $
		TRIM(OBS.RAS_VAR) + ' not found in database'
;
;  Make sure that the data window ID number matches an entry in the main
;  database.
;
	GET_DATAWIN, OBS.DW_ID, TEMP
	IF TEMP.DW_ID LE 0 THEN CDS_MESSAGE, /CONTINUE, /ATTENTION,	$
		'Data window ID ' + STRTRIM(OBS.DW_ID,2) +	$
		' not found in database'
;
;  Make sure that the stop date is greater than the start date.
;
	IF OBS.DATE_END LE OBS.DATE_OBS THEN BEGIN
           MESSAGE = 'The stop date must be after the start date'
           GOTO, HANDLE_ERROR
	ENDIF
;
	IF OBS.OBT_END LE OBS.OBT_TIME THEN BEGIN
           MESSAGE = 'The OBT stop date must be after the start date'
           GOTO, HANDLE_ERROR
	ENDIF
;
;  Reformat the times to millisecond accuracy.  This is necessary so that
;  the times are written out in a controlled way.
;
	OBS.DATE_OBS = DOUBLE( STRING( OBS.DATE_OBS, FORMAT='(F15.3)' ))
	OBS.DATE_END = DOUBLE( STRING( OBS.DATE_END, FORMAT='(F15.3)' ))
	OBS.OBT_TIME = DOUBLE( STRING( OBS.OBT_TIME, FORMAT='(F15.3)' ))
	OBS.OBT_END  = DOUBLE( STRING( OBS.OBT_END,  FORMAT='(F15.3)' ))
;
;  Make sure that the proposed entry is already in the database.
;
	GET_EXPER, OBS.SEQ_NUM, OLD
	IF OLD.SEQ_NUM LT 0 THEN BEGIN
	   MESSAGE = 'Sequence number ' + TRIM(OBS.SEQ_NUM) +	$
		   ' is not in the database'
	   GOTO, HANDLE_ERROR
	ENDIF
;
;  Make sure that one isn't trying to change the program number or the sequence
;  index.
;
	IF OBS.PROG_NUM NE OLD.PROG_NUM THEN BEGIN
	   MESSAGE = 'Cannot change PROG_NUM'
	   GOTO, HANDLE_ERROR
	ENDIF
;
	IF OBS.SEQ_IND NE OLD.SEQ_IND THEN BEGIN
	   MESSAGE = 'Cannot change SEQ_IND'
	   GOTO, HANDLE_ERROR
	ENDIF
;
;  Lock the database.
;
	LOCK_DATABASE, 'experiment', LOCKFILE
;
;  Open the database for write access, and find the exact record to modify.
;  Start with the second database, and then do the first.  (Because of the
;  pointers, and because of the need to do COMMENTS first.)
;
	VERSION = ''
	TRAILER = ''
FIND_SEQ_NUM:
	DBOPEN, VERSION+'experiment2'+TRAILER, 1
	ENTRY = DBFIND('SEQ_NUM=' + TRIM(OBS.SEQ_NUM), /SILENT)
	ENTRY = ENTRY(0)
;
;  If nothing was found, then look in the "f" version of the database.
;
	IF ENTRY LE 0 THEN BEGIN
	    IF VERSION EQ '' THEN BEGIN
		VERSION = 'f'
	    END ELSE IF TRAILER EQ '' THEN BEGIN
		ITRAILER = 1996
		TRAILER = '_' + TRIM(ITRAILER)
	    END ELSE BEGIN
		ITRAILER = ITRAILER + 1
		TRAILER = '_' + TRIM(ITRAILER)
	    ENDELSE
	    IF FIND_WITH_DEF('fexperiment2'+TRAILER+'.dbf','$ZDBASE') NE '' $
		    THEN GOTO, FIND_SEQ_NUM
	ENDIF
;
;  Extract the pointer to experiment1, for use later.
;
	DBEXT, ENTRY, 'exper_ptr', EXPER_PTR
;
;  For each entry, check to see if it has changed.  If it has, then update the
;  database.
;
	MODS = 0
;
;  Start with COMMENTS, because it has an extra check.  That way, if an error
;  occurs, then no changes will be made.
;
	IF OBS.COMMENTS NE OLD.COMMENTS THEN BEGIN
		IF OBS.COMMENTS EQ 'N' THEN BEGIN
			MESSAGE = "Can't change COMMENTS from Y to N"
			GOTO, HANDLE_ERROR
		END ELSE BEGIN
			DBUPDATE, ENTRY, 'comments', OBS.COMMENTS
			PRINT, 'Modified parameter COMMENTS'
			MODS = MODS + 1
		ENDELSE
	ENDIF
;
	IF OBS.DATE_OBS NE OLD.DATE_OBS THEN BEGIN
		DBUPDATE, ENTRY, 'date_obs', OBS.DATE_OBS
		PRINT, 'Modified parameter DATE_OBS'
		MODS = MODS + 1
	ENDIF
;
	IF OBS.DATE_END NE OLD.DATE_END THEN BEGIN
		DBUPDATE, ENTRY, 'date_end', OBS.DATE_END
		PRINT, 'Modified parameter DATE_END'
		MODS = MODS + 1
	ENDIF
;
	IF OBS.TRACKING NE OLD.TRACKING THEN BEGIN
		DBUPDATE, ENTRY, 'tracking', OBS.TRACKING
		PRINT, 'Modified parameter TRACKING'
		MODS = MODS + 1
	ENDIF
;
	IF OBS.SER_ID NE OLD.SER_ID THEN BEGIN
		DBUPDATE, ENTRY, 'ser_id', OBS.SER_ID
		PRINT, 'Modified parameter SER_ID'
		MODS = MODS + 1
	ENDIF
;
	IF OBS.OPSLBITS NE OLD.OPSLBITS THEN BEGIN
		DBUPDATE, ENTRY, 'opslbits', OBS.OPSLBITS
		PRINT, 'Modified parameter OPSLBITS'
		MODS = MODS + 1
	ENDIF
;
	IF OBS.OPSRBITS NE OLD.OPSRBITS THEN BEGIN
		DBUPDATE, ENTRY, 'opsrbits', OBS.OPSRBITS
		PRINT, 'Modified parameter OPSRBITS'
		MODS = MODS + 1
	ENDIF
;
	IF OBS.SLIT_POS NE OLD.SLIT_POS THEN BEGIN
		DBUPDATE, ENTRY, 'slit_pos', OBS.SLIT_POS
		PRINT, 'Modified parameter SLIT_POS'
		MODS = MODS + 1
	ENDIF
;
	IF OBS.MIR_POS NE OLD.MIR_POS THEN BEGIN
		DBUPDATE, ENTRY, 'mir_pos', OBS.MIR_POS
		PRINT, 'Modified parameter MIR_POS'
		MODS = MODS + 1
	ENDIF
;
	IF OBS.EV_ENAB NE OLD.EV_ENAB THEN BEGIN
		DBUPDATE, ENTRY, 'ev_enab', OBS.EV_ENAB
		PRINT, 'Modified parameter EV_ENAB'
		MODS = MODS + 1
	ENDIF
;
	IF OBS.COMP_ERR NE OLD.COMP_ERR THEN BEGIN
		DBUPDATE, ENTRY, 'comp_err', OBS.COMP_ERR
		PRINT, 'Modified parameter COMP_ERR'
		MODS = MODS + 1
	ENDIF
;
	IF OBS.VDS_PMCP NE OLD.VDS_PMCP THEN BEGIN
		DBUPDATE, ENTRY, 'vds_pmcp', OBS.VDS_PMCP
		PRINT, 'Modified parameter VDS_PMCP'
		MODS = MODS + 1
	ENDIF
;
	IF OBS.VDS_MODE NE OLD.VDS_MODE THEN BEGIN
		DBUPDATE, ENTRY, 'vds_mode', OBS.VDS_MODE
		PRINT, 'Modified parameter VDS_MODE'
		MODS = MODS + 1
	ENDIF
;
	IF OBS.VDS_ORI NE OLD.VDS_ORI THEN BEGIN
		DBUPDATE, ENTRY, 'vds_ori', OBS.VDS_ORI
		PRINT, 'Modified parameter VDS_ORI'
		MODS = MODS + 1
	ENDIF
;
	IF OBS.VDS_ACC NE OLD.VDS_ACC THEN BEGIN
		DBUPDATE, ENTRY, 'vds_acc', OBS.VDS_ACC
		PRINT, 'Modified parameter VDS_ACC'
		MODS = MODS + 1
	ENDIF
;
	IF OBS.GSET_ID NE OLD.GSET_ID THEN BEGIN
		DBUPDATE, ENTRY, 'gset_id', OBS.GSET_ID
		PRINT, 'Modified parameter GSET_ID'
		MODS = MODS + 1
	ENDIF
;
	IF OBS.DETECTOR NE OLD.DETECTOR THEN BEGIN
		DBUPDATE, ENTRY, 'detector', OBS.DETECTOR
		PRINT, 'Modified parameter DETECTOR'
		MODS = MODS + 1
	ENDIF
;
	IF OBS.ZONE_ID NE OLD.ZONE_ID THEN BEGIN
		DBUPDATE, ENTRY, 'zone_id', OBS.ZONE_ID
		PRINT, 'Modified parameter ZONE_ID'
		MODS = MODS + 1
	ENDIF
;
	IF OBS.SLIT_NUM NE OLD.SLIT_NUM THEN BEGIN
		DBUPDATE, ENTRY, 'slit_num', OBS.SLIT_NUM
		PRINT, 'Modified parameter SLIT_NUM'
		MODS = MODS + 1
	ENDIF
;
	IF TRIM(OBS.FILENAME) NE TRIM(OLD.FILENAME) THEN BEGIN
		DBUPDATE, ENTRY, 'filename', OBS.FILENAME
		PRINT, 'Modified parameter FILENAME'
		MODS = MODS + 1
	ENDIF
;
	IF OBS.SEQVALID NE OLD.SEQVALID THEN BEGIN
		DBUPDATE, ENTRY, 'seqvalid', OBS.SEQVALID
		PRINT, 'Modified parameter SEQVALID'
		MODS = MODS + 1
	ENDIF
;
;  Now open the intermediate database, and make any necessary modifications in
;  there.  Save the previous entry point.
;
	ENTRY2 = ENTRY
	DBOPEN, VERSION+'experiment1'+TRAILER, 1
	ENTRY = EXPER_PTR
	ENTRY = ENTRY(0)
;
;  Extract the pointer to experiment1, for use later.
;
	DBEXT, ENTRY, 'exper0_ptr', EXPER0_PTR
;
	IF OBS.DATASRC NE OLD.DATASRC THEN BEGIN
		DBUPDATE, ENTRY, 'datasrc', OBS.DATASRC
		PRINT, 'Modified parameter DATASRC'
		MODS = MODS + 1
	ENDIF
;
;  Now open the first experiment database, and make any necessary modifications
;  in there.  Save the previous entry point.
;
	DBOPEN, VERSION+'experiment'+TRAILER, 1
	ENTRY = EXPER0_PTR
	ENTRY = ENTRY(0)
;
	IF OBS.OBS_SEQ NE OLD.OBS_SEQ THEN BEGIN
		DBUPDATE, ENTRY, 'obs_seq', OBS.OBS_SEQ
		PRINT, 'Modified parameter OBS_SEQ'
		MODS = MODS + 1
	ENDIF
;
	IF OBS.COUNT NE OLD.COUNT THEN BEGIN
		DBUPDATE, ENTRY, 'count', OBS.COUNT
		PRINT, 'Modified parameter COUNT'
		MODS = MODS + 1
	ENDIF
;
	IF OBS.SEQ_IND NE OLD.SEQ_IND THEN BEGIN
		DBUPDATE, ENTRY, 'seq_ind', OBS.SEQ_IND
		PRINT, 'Modified parameter SEQ_IND'
		MODS = MODS + 1
	ENDIF
;
	IF OBS.RAS_ID NE OLD.RAS_ID THEN BEGIN
		DBUPDATE, ENTRY, 'ras_id', OBS.RAS_ID
		PRINT, 'Modified parameter RAS_ID'
		MODS = MODS + 1
	ENDIF
;
	IF OBS.RAS_VAR NE OLD.RAS_VAR THEN BEGIN
		DBUPDATE, ENTRY, 'ras_var', OBS.RAS_VAR
		PRINT, 'Modified parameter RAS_VAR'
		MODS = MODS + 1
	ENDIF
;
	IF OBS.EXPTIME NE OLD.EXPTIME THEN BEGIN
		DBUPDATE, ENTRY, 'exptime', OBS.EXPTIME
		PRINT, 'Modified parameter EXPTIME'
		MODS = MODS + 1
	ENDIF
;
	IF OBS.OBS_MODE NE OLD.OBS_MODE THEN BEGIN
		DBUPDATE, ENTRY, 'obs_mode', OBS.OBS_MODE
		PRINT, 'Modified parameter OBS_MODE'
		MODS = MODS + 1
	ENDIF
;
	IF OBS.DW_ID NE OLD.DW_ID THEN BEGIN
		DBUPDATE, ENTRY, 'dw_id', OBS.DW_ID
		PRINT, 'Modified parameter DW_ID'
		MODS = MODS + 1
	ENDIF
;
	IF OBS.OBT_TIME NE OLD.OBT_TIME THEN BEGIN
		DBUPDATE, ENTRY, 'obt_time', OBS.OBT_TIME
		PRINT, 'Modified parameter OBT_TIME'
		MODS = MODS + 1
	ENDIF
;
	IF OBS.OBT_END NE OLD.OBT_END THEN BEGIN
		DBUPDATE, ENTRY, 'obt_end', OBS.OBT_END
		PRINT, 'Modified parameter OBT_END'
		MODS = MODS + 1
	ENDIF
;
	IF OBS.XCEN NE OLD.XCEN THEN BEGIN
		DBUPDATE, ENTRY, 'xcen', OBS.XCEN
		PRINT, 'Modified parameter XCEN'
		MODS = MODS + 1
	ENDIF
;
	IF OBS.YCEN NE OLD.YCEN THEN BEGIN
		DBUPDATE, ENTRY, 'ycen', OBS.YCEN
		PRINT, 'Modified parameter YCEN'
		MODS = MODS + 1
	ENDIF
;
	IF OBS.ANGLE NE OLD.ANGLE THEN BEGIN
		DBUPDATE, ENTRY, 'angle', OBS.ANGLE
		PRINT, 'Modified parameter ANGLE'
		MODS = MODS + 1
	ENDIF
;
	IF OBS.IXWIDTH NE OLD.IXWIDTH THEN BEGIN
		DBUPDATE, ENTRY, 'ixwidth', OBS.IXWIDTH
		PRINT, 'Modified parameter IXWIDTH'
		MODS = MODS + 1
	ENDIF
;
	IF OBS.IYWIDTH NE OLD.IYWIDTH THEN BEGIN
		DBUPDATE, ENTRY, 'iywidth', OBS.IYWIDTH
		PRINT, 'Modified parameter IYWIDTH'
		MODS = MODS + 1
	ENDIF
;
	IF OBS.INS_X0 NE OLD.INS_X0 THEN BEGIN
		DBUPDATE, ENTRY, 'ins_x0', OBS.INS_X0
		PRINT, 'Modified parameter INS_X0'
		MODS = MODS + 1
	ENDIF
;
	IF OBS.INS_Y0 NE OLD.INS_Y0 THEN BEGIN
		DBUPDATE, ENTRY, 'ins_y0', OBS.INS_Y0
		PRINT, 'Modified parameter INS_Y0'
		MODS = MODS + 1
	ENDIF
;
	IF OBS.INS_ROLL NE OLD.INS_ROLL THEN BEGIN
		DBUPDATE, ENTRY, 'ins_roll', OBS.INS_ROLL
		PRINT, 'Modified parameter INS_ROLL'
		MODS = MODS + 1
	ENDIF
;
	IF OBS.SC_X0 NE OLD.SC_X0 THEN BEGIN
		DBUPDATE, ENTRY, 'sc_x0', OBS.SC_X0
		PRINT, 'Modified parameter SC_X0'
		MODS = MODS + 1
	ENDIF
;
	IF OBS.SC_Y0 NE OLD.SC_Y0 THEN BEGIN
		DBUPDATE, ENTRY, 'sc_y0', OBS.SC_Y0
		PRINT, 'Modified parameter SC_Y0'
		MODS = MODS + 1
	ENDIF
;
	IF OBS.SC_ROLL NE OLD.SC_ROLL THEN BEGIN
		DBUPDATE, ENTRY, 'sc_roll', OBS.SC_ROLL
		PRINT, 'Modified parameter SC_ROLL'
		MODS = MODS + 1
	ENDIF
;
	IF OBS.WAVEMIN NE OLD.WAVEMIN THEN BEGIN
		DBUPDATE, ENTRY, 'wavemin', OBS.WAVEMIN
		PRINT, 'Modified parameter WAVEMIN'
		MODS = MODS + 1
	ENDIF
;
	IF OBS.WAVEMAX NE OLD.WAVEMAX THEN BEGIN
		DBUPDATE, ENTRY, 'wavemax', OBS.WAVEMAX
		PRINT, 'Modified parameter WAVEMAX'
		MODS = MODS + 1
	ENDIF
;
;  If anything has been modified, then update the modification date.
;
	IF MODS GT 0 THEN BEGIN
		GET_UTC, DATE_MOD
		DATE_MOD = UTC2TAI(DATE_MOD)
		DBOPEN, VERSION+'experiment2'+TRAILER, 1
		DBUPDATE, ENTRY2, 'date_mod', DATE_MOD
	ENDIF
;
;  Signal success.
;
	RESULT = MODS
	GOTO, FINISH
;
;  Error handling point.
;
HANDLE_ERROR:
	IF N_ELEMENTS(ERRMSG) NE 0 THEN ERRMSG = 'MOD_EXPER: ' + MESSAGE $
		ELSE MESSAGE, MESSAGE, /CONTINUE
;
;  Close the database, and return whether the routine was successful or not.
;
FINISH:
	DBCLOSE
;
	IF N_ELEMENTS(LOCKFILE) EQ 1 THEN UNLOCK_DATABASE, LOCKFILE
	RETURN, RESULT
	END
