	FUNCTION MOD_FLAG, OLD_DEF, NEW_DEF, ERRMSG=ERRMSG
;+
; Project     :	SOHO - CDS
;
; Name        :	MOD_FLAG()
;
; Purpose     :	Modify entry in the flag receiver science plan database.
;
; Explanation :	Modifies an entry in the flag receiver science plan database.
;		The old entry is removed, and the new entry is added.
;
; Use         :	Result = MOD_FLAG( OLD_DEF, NEW_DEF )
;
;		IF NOT MOD_FLAG(OLD_DEF,NEW_DEF) THEN ...
;
; Inputs      :	OLD_DEF	= Previous entry.  This is a structure such as that
;			  returned by GET_FLAG, and has the tags:
;
;			PROG_ID	   = Program ID, linking one or more studies
;				     together
;			STUDY_ID   = Number defining the study
;			STUDYVAR   = Number giving the study variation ID
;			SCI_OBJ    = Science objective from the daily science
;				     meeting
;			SCI_SPEC   = Specific science objective from meeting
;			CMP_NO	   = Campaign number
;			OBJECT	   = Code for object planned to be observed
;			OBJ_ID	   = Object identification
;			RCVR_START = Date/time of beginning of receiver status,
;				     in TAI format
;			RCVR_STOP  = Date/time of end of receiver status, in
;				     TAI format
;			ORIG_DUR   = Original duration of the observation, in
;				     seconds, before being truncated by
;				     following time-tagged observations.
;			N_RASTERS1 = Variable number of rasters parameter
;			TRACKING   = True (1) if feature tracking to be used,
;				     or false (0) otherwise
;			N_POINTINGS= Number of pointings to use with study.
;				     Can be either 0 or 1.
;			REPOINT	   = Either 1 to allow repoints or 0 to
;				     disallow.
;			GSET_ID	   = GSET ID number
;
;		      If N_POINTINGS is greater than 0, then there will be the
;		      additional tag POINTINGS.  This is an array of pointing
;		      positions.  It itself is a structure, of type
;		      PLAN_PNT_STRUC, with the following tags:
;
;			INS_X	   = Pointing in X relative to sun center
;			INS_Y	   = Pointing in Y relative to sun center
;			ZONE_ID	   = Pointing zone ID
;
;		NEW_DEF = New entry.  It has the same structure as OLD_DEF.
;			  Any field can be changed, including the start and end
;			  times.
;
; Opt. Inputs :	None.
;
; Outputs     :	The result of the function is a logical value representing
;		whether or not the operation was successful, where 1 is
;		successful and 0 is unsuccessful.
;
; Opt. Outputs:	None.
;
; Keywords    :	ERRMSG = If defined and passed, then any error messages will be
;			 returned to the user in this parameter rather than
;			 depending on the MESSAGE routine in IDL.  If no errors
;			 are encountered, then a null string is returned.  In
;			 order to use this feature, ERRMSG must be defined
;			 first, e.g.
;
;				ERRMSG = ''
;				Result = MOD_FLAG( ERRMSG=ERRMSG, ... )
;				IF ERRMSG NE '' THEN ...
;
; Calls       :	DEL_FLAG, ADD_FLAG
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Category    :	Planning, Databases.
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 13 February 1995
;
; Modified    :	Version 1, William Thompson, GSFC, 13 February 1995
;
; Version     :	Version 1, 13 February 1995
;-
;
	ON_ERROR, 2
;
;  Check the number of parameters.
;
        IF N_PARAMS() NE 2 THEN BEGIN
		MESSAGE = 'Syntax:  MOD_FLAG, OLD_DEF, NEW_DEF'
		IF N_ELEMENTS(ERRMSG) NE 0 THEN BEGIN
			ERRMSG = MESSAGE
			RETURN, 0
		ENDIF ELSE MESSAGE, MESSAGE
        ENDIF
;
;  First delete the old entry.
;
	MESSAGE = ''
	IF NOT DEL_FLAG( OLD_DEF.RCVR_START, ERRMSG=MESSAGE) THEN BEGIN
		IF N_ELEMENTS(ERRMSG) NE 0 THEN BEGIN
			ERRMSG = MESSAGE
			RETURN, 0
		ENDIF ELSE MESSAGE, MESSAGE
	ENDIF
;
;  Next add in the new entry.
;
	IF NOT ADD_FLAG( NEW_DEF, ERRMSG=MESSAGE) THEN BEGIN
;
;  If you can't add in the new definition, put back the old definition.
;
		MSG = ''
		IF NOT ADD_FLAG( OLD_DEF, ERRMSG=MSG) THEN MESSAGE =	$
			"Something's wrong--can't even put back old entry"
		IF N_ELEMENTS(ERRMSG) NE 0 THEN BEGIN
			ERRMSG = MESSAGE
			RETURN, 0
		ENDIF ELSE MESSAGE, MESSAGE
	ENDIF
;
;  If you got this far, you must be successful.
;
	MESSAGE = ''
	RETURN, 1
	END
