	FUNCTION MOD_GSET, GSET_ID, RAW_ID=RAW_ID, GSET_DESC=GSET_DESC,	$
		ERRMSG=ERRMSG
;+
; Project     :	SOHO - CDS
;
; Name        :	MOD_GSET()
;
; Purpose     :	Modifies a GIS setup definition in the database
;
; Explanation :	This procedure allows one to modify certain parameters in the
;		definition of a GIS setup.
;
; Use         :	Result = MOD_GSET( GSET_ID, keywords=... )
;
;		IF NOT MOD_GSET( GSET_ID, keywords=... ) THEN ...
;
; Inputs      :	GSET_ID = GIS setup ID. This parameter simply allows the
;			  relevant entry to be found in the database.  The
;			  modifications are passed through keyword parameters.
;
; Opt. Inputs :	None.
;
; Outputs     :	The result of the function is a logical value representing
;		whether or not the operation was successful, where 1 is
;		successful and 0 is unsuccessful.
;
; Opt. Outputs:	None.
;
; Keywords    :	
;       RAW_ID	  = ID number of associated raw science files (4)
;	GSET_DESC = Free format descriptions (4)
;
;		The above keywords specify the modifications to be made to the
;		database.
;
;       ERRMSG    = If defined and passed, then any error messages will be
;                   returned to the user in this parameter rather than
;                   depending on the MESSAGE routine in IDL.  If no errors are
;                   encountered, then a null string is returned.  In order to
;                   use this feature, ERRMSG must be defined first, e.g.
;
;                       ERRMSG = ''
;                       Result = MOD_GSET( ERRMSG=ERRMSG, ... )
;                       IF ERRMSG NE '' THEN ...
;
; Calls       :	DATATYPE, DBOPEN, DBFIND, DBBUILD, DBCLOSE
;
; Common      :	None.
;
; Restrictions:	Only this routine can be used to modify GIS setup definitions
;		in the database.  Modifying the database by hand could corrupt
;		its integrity.
;
;		The data types and sizes of the modifications must match the
;		definitions in the database.  The string lengths must not
;		exceed the lengths defined in the database.
;
;		!PRIV must be 2 or greater to use this routine.
;
; Side effects:	None.
;
; Category    :	Planning, Databases.
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 8 September 1995
;
; Modified    :	Version 1, William Thompson, GSFC, 8 September 1995
;		Version 2, William Thompson, GSFC, 7 February 1996
;			Fixed several bugs.
;
; Version     :	Version 2, 7 February 1996
;-
;
	ON_ERROR, 2
;
;  Initialize RESULT to represent non-success.  If the routine is successful,
;  this value will be updated below.
;
	RESULT = 0
;
;  Check the input parameters
;
        IF N_PARAMS() NE 1 THEN BEGIN
           MESSAGE = 'Syntax:  Result = ' +	$
		'MOD_GSET( GSET_ID, keywords=... )'
	   GOTO, HANDLE_ERROR
        ENDIF
;
;  Make sure that the user has privilege to write into the database.
;
	IF !PRIV LT 2 THEN BEGIN $
           MESSAGE = '!PRIV must be 2 or greater to write into the database'
           GOTO, HANDLE_ERROR
	ENDIF
;
;  Check each of the input parameters to verify that it is of the correct type
;  and size.
;
	IF DATATYPE(GSET_ID,1) NE 'Integer' THEN BEGIN
           MESSAGE = 'GSET_ID must be a short integer'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(GSET_ID) NE 1 THEN BEGIN
           MESSAGE = 'GSET_ID must be a scalar'
           GOTO, HANDLE_ERROR
	ENDIF
;
	IF N_ELEMENTS(RAW_ID) NE 0 THEN BEGIN
		IF DATATYPE(RAW_ID,1) NE 'Integer' THEN BEGIN
        	   MESSAGE = 'Tag RAW_ID must be a short integer array'
	           GOTO, HANDLE_ERROR
		END ELSE IF N_ELEMENTS(RAW_ID) NE 4 THEN BEGIN
	           MESSAGE = 'Tag RAW_ID must have four elements'
        	   GOTO, HANDLE_ERROR
		ENDIF
	ENDIF
;
	IF N_ELEMENTS(GSET_DESC) NE 0 THEN BEGIN
		IF DATATYPE(GSET_DESC,1) NE 'String' THEN BEGIN
        	   MESSAGE = 'Tag GSET_DESC must be a character string'
	           GOTO, HANDLE_ERROR
		END ELSE IF N_ELEMENTS(GSET_DESC) NE 4 THEN BEGIN
	           MESSAGE = 'Tag GSET_DESC must have four elements'
        	   GOTO, HANDLE_ERROR
		END ELSE IF MAX(STRLEN(GSET_DESC)) GT 80 THEN BEGIN
		   MESSAGE = 'Tag GSET_DESC must be 80 characters or less'
		   GOTO, HANDLE_ERROR
		ENDIF
	ENDIF
;
;  Open the GIS setup database for read access, and find the entry in
;  question.
;
	DBOPEN, 'gset'
	ENTRY = DBFIND('GSET_ID='+ STRTRIM(LONG(GSET_ID),2), /SILENT)
	IF !ERR EQ 0 THEN BEGIN
           MESSAGE = 'GIS setup ID ' + TRIM(LONG(GSET_ID)) +	$
		   ' not found in database'
           GOTO, HANDLE_ERROR
	ENDIF
;
;  Open the GIS setup database for write access.
;
	DBOPEN, 'gset', 1
;
;  For each keyword passed, update the database.  Signal success.
;
	IF N_ELEMENTS(RAW_ID) NE 0 THEN DBUPDATE, ENTRY, 'raw_id', RAW_ID
;
;  Concatenate the multiline description into a single strings.
;
	IF N_ELEMENTS(GSET_DESC) NE 0 THEN BEGIN
		GSETDESC = STRING(REPLICATE(32B,320))
		FOR I = 0,3 DO STRPUT, GSETDESC, GSET_DESC(I), 80*I
		DBUPDATE, ENTRY, 'gset_desc', GSETDESC
	ENDIF
;
	RESULT = 1
	GOTO, FINISH
;
;  Error handling point.
;
HANDLE_ERROR:
	IF N_ELEMENTS(ERRMSG) NE 0 THEN ERRMSG = 'MOD_GSET: ' + MESSAGE $
		ELSE MESSAGE, MESSAGE, /CONTINUE
;
;  Close the database, and return whether the routine was successful or not.
;
FINISH:
	DBCLOSE
;
	RETURN, RESULT
	END
