	FUNCTION MOD_LINELIST, LL_ID, CEN_PIX=CEN_PIX, ERRMSG=ERRMSG
;+
; Project     :	SOHO - CDS
;
; Name        :	MOD_LINELIST()
;
; Purpose     :	Modifies a linelist definition in the database
;
; Explanation :	This procedure allows one to modify certain parameters in the
;		definition of a linelist.
;
; Use         :	Result = MOD_LINELIST( LL_ID, keywords=... )
;
;		IF NOT MOD_LINELIST( LL_ID, keywords=... ) THEN ...
;
; Inputs      :	LL_ID = Linelist ID number.
;
;		The above parameters simply allow the relevant entry to be
;		found in the database.  The modifications are passed through
;		keyword parameters.
;
; Opt. Inputs :	None.
;
; Outputs     :	The result of the function is a logical value representing
;		whether or not the operation was successful, where 1 is
;		successful and 0 is unsuccessful.
;
; Opt. Outputs:	None.
;
; Keywords    :	
;       CEN_PIX = Array of central pixel positions for the lines in the
;		  linelist.
;
;		The keywords specify the modifications to be made to the
;		database.  Currently, only modifications of CEN_PIX is
;		supported.
;
;       ERRMSG    = If defined and passed, then any error messages will be
;                   returned to the user in this parameter rather than
;                   depending on the MESSAGE routine in IDL.  If no errors are
;                   encountered, then a null string is returned.  In order to
;                   use this feature, ERRMSG must be defined first, e.g.
;
;                       ERRMSG = ''
;                       Result = MOD_LINELIST( ERRMSG=ERRMSG, ... )
;                       IF ERRMSG NE '' THEN ...
;
; Calls       :	DATATYPE, DBOPEN, DBFIND, DBBUILD, DBCLOSE
;
; Common      :	None.
;
; Restrictions:	Only this routine can be used to modify linelist definitions in
;		the database.  Modifying the database by hand could corrupt its
;		integrity.
;
;		The data types and sizes of the structure elements must match
;		the definitions in the database.
;
;		!PRIV must be 2 or greater to use this routine.
;
; Side effects:	Once a linelist definition has been added to the database, it
;		must never be removed.
;
; Category    :	Planning, Databases.
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 20 November 1995
;
; Modified    :	Version 1, William Thompson, GSFC, 20 November 1995
;               Version 2, Fix typo on parameter checking.  CDP, 27-Nov-95
;
; Version     :	Version 2, 27 November 1995
;-
;
	ON_ERROR, 2
;
;  Initialize RESULT to represent non-success.  If the routine is successful,
;  this value will be updated below.
;
	RESULT = 0
;
;  Check the input parameters
;
        IF N_PARAMS() NE 1 THEN BEGIN
           MESSAGE = 'Syntax:  Result = ' +	$
		'MOD_LINELIST( LL_ID, keywords=... )'
	   GOTO, HANDLE_ERROR
        ENDIF
;
;  Make sure that the user has privilege to write into the database.
;
	IF !PRIV LT 2 THEN BEGIN $
           MESSAGE = '!PRIV must be 2 or greater to write into the database'
           GOTO, HANDLE_ERROR
	ENDIF
;
;  Check each of the input parameters to verify that it is of the correct type
;  and size.
;
	IF DATATYPE(LL_ID,1) NE 'Integer' THEN BEGIN
           MESSAGE = 'LL_ID must be a short integer'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(LL_ID) NE 1 THEN BEGIN
           MESSAGE = 'LL_ID must be a scalar'
           GOTO, HANDLE_ERROR
	ENDIF
;
;  Get the current linelist definition.
;
	MESSAGE = ''
	GET_LINELIST, LL_ID, LL, ERRMSG=MESSAGE
	IF LL_ID LE 0 THEN GOTO, HANDLE_ERROR
	N_LINES = N_ELEMENTS(LL.LINES)
;
	IF N_ELEMENTS(CEN_PIX) NE 0 THEN BEGIN
		IF DATATYPE(CEN_PIX,1) NE 'Integer' THEN BEGIN
                   MESSAGE = 'CEN_PIX must be of type short integer'
                   GOTO, HANDLE_ERROR
		END ELSE IF N_ELEMENTS(CEN_PIX) NE N_LINES THEN BEGIN
                   MESSAGE = 'CEN_PIX must have ' + TRIM(N_LINES) + ' elements'
                   GOTO, HANDLE_ERROR
		ENDIF
	ENDIF
;
;  Open the linelist2 database for write access.
;
	DBOPEN, 'linelist2', 1
;
;  For each keyword passed, update the database.  Signal success.
;
	IF N_ELEMENTS(CEN_PIX) NE 0 THEN BEGIN
	    FOR LINE_IND = 1, N_LINES DO BEGIN
		ENTRY = DBFIND('LL_ID='+ STRTRIM(LONG(LL_ID),2) +	$
			',LINE_IND=' + STRTRIM(LONG(LINE_IND),2), /SILENT)
		DBUPDATE, ENTRY, 'cen_pix', CEN_PIX(LINE_IND-1)
	    ENDFOR
	ENDIF
;
	RESULT = 1
	GOTO, FINISH
;
;  Error handling point.
;
HANDLE_ERROR:
	IF N_ELEMENTS(ERRMSG) NE 0 THEN ERRMSG = 'MOD_LINELIST: ' + MESSAGE $
		ELSE MESSAGE, MESSAGE, /CONTINUE
;
;  Close the database, and return whether the routine was successful or not.
;
FINISH:
	DBCLOSE
;
	RETURN, RESULT
	END
