	FUNCTION MOD_MAIN, OBS, ERRMSG=ERRMSG
;+
; Project     :	SOHO - CDS
;
; Name        :	MOD_MAIN()
;
; Purpose     :	Modifies a CDS main entry in the as-run database
;
; Explanation :	This procedure takes a CDS study-level (main) as-run entry,
;		compares it against that in the database "main", and makes any
;		necessary modifications.  Any parameter can be modified, except
;		PROG_NUM.  (Note that COMMENTS can only be changed from "N"(o)
;		to "Y"(es).)
;
; Use         :	Result = MOD_MAIN( OBS )
;
;		IF MOD_MAIN( OBS ) LT 0 THEN ...
;
; Inputs      :	OBS = This is an anonymous structure containing the following
;		      tags:
;
;			PROG_NUM   = Study counter number.  This forms the
;				     primary key for this database.
;			PROG_ID	   = Program ID, linking one or more studies
;				     together
;			PROG_IND   = An index, reflecting one's position within
;				     a repeated sequence of studies via the
;				     N_REPEAT_S mechanism.
;			STUDY_ID   = Number defining the study
;			STUDYVAR   = Number giving the study variation ID
;			OBS_PROG   = Name of the study
;			DETECTOR   = Detector, "N", "G", or "B"(oth).
;			SCI_OBJ    = Science objective from the daily science
;				     meeting
;			SCI_SPEC   = Specific science objective from meeting
;			CMP_NO	   = Campaign number
;			OBJECT	   = Code for object planned to be observed
;			OBJ_ID	   = Object identification
;			DATE_OBS   = Date/time of beginning of observation,
;				     in TAI format
;			DATE_END   = Date/time of end of observation, in TAI
;				     format
;			OBT_TIME   = Onboard date/time of beginning of
;				     observation, in TAI format
;			OBT_END	   = Onboard date/time of end of observation,
;				     in TAI format
;			XCEN	   = Center of instrument field-of-view in
;				     solar X
;			YCEN	   = Center of instrument field-of-view in
;				     solar Y
;			ANGLE	   = Angle of instrument relative to solar
;				     north
;			IXWIDTH	   = Width of field-of-view in instrument X
;			IYWIDTH	   = Width of field-of-view in instrument Y
;			SEQ_FROM   = Start of range of observing sequence
;				     numbers.
;			SEQ_TO	   = End of range of observing sequence
;				     numbers.
;			COMMENTS   = Either Y or N
;
; Opt. Inputs :	None.
;
; Outputs     :	The result of the function is the number of parameters that
;		were modified.  If an error occurs, then -1 is returned
;		instead.
;
; Opt. Outputs:	None.
;
; Keywords    :
;       ERRMSG    = If defined and passed, then any error messages will be
;                   returned to the user in this parameter rather than
;                   depending on the MESSAGE routine in IDL.  If no errors are
;                   encountered, then a null string is returned.  In order to
;                   use this feature, ERRMSG must be defined first, e.g.
;
;                       ERRMSG = ''
;                       Result = MOD_MAIN( ERRMSG=ERRMSG, ... )
;                       IF ERRMSG NE '' THEN ...
;
;
; Calls       :	DATATYPE, DBOPEN, DBBUILD, DBCLOSE, TRIM, GET_OBJECT,
;		GET_CAMPAIGN, GET_PROGRAM, GET_STUDY
;
; Common      :	None.
;
; Restrictions:	Only this routine can be used to modify entries in the MAIN
;		database.  Modifying the database by hand could corrupt its
;		integrity.
;
;		The data types and sizes of the structure elements must match
;		the definitions in the database.
;
;		!PRIV must be 2 or greater to use this routine.
;
; Side effects:	The dates in the structure are rounded off to millisecond
;		accuracy.
;
; Category    :	Planning, Databases.
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 30 June 1995
;
; Modified    :	Version 1, William Thompson, GSFC, 30 June 1995
;		Version 2, William Thompson, GSFC, 21 February 1996
;			Modified to be more robust.  It will complain about not
;			finding IDs in the databases, but will modify the
;			catalog entry anyway.
;		Version 3, William Thompson, GSFC, 2 April 1996
;			Modified for version 3 of the catalog.
;		Version 4, William Thompson, GSFC, 9 April 1996
;			Added calls to LOCK_DATABASE, UNLOCK_DATABASE
;		Version 5, William Thompson, GSFC, 16 September 1997
;			Converted TRIM to STRTRIM in tests (remove trailing
;				blanks only).
;			Added STRTRIM to test for OBS_PROG.
;
; Version     :	Version 5, 16 September 1997
;-
;
	ON_ERROR, 2
;
;  Initialize RESULT to represent non-success.  If the routine is successful,
;  this value will be updated below.
;
	RESULT = -1
;
;  Check the input parameters
;
        IF N_PARAMS() NE 1 THEN BEGIN
           MESSAGE = 'Syntax:  Result = MOD_MAIN(OBS)'
	   GOTO, HANDLE_ERROR
        ENDIF
;
;  Make sure that the user has privilege to write into the database.
;
	IF !PRIV LT 2 THEN BEGIN
           MESSAGE = '!PRIV must be 2 or greater to write into the database'
           GOTO, HANDLE_ERROR
	ENDIF
;
;  Check each of the structure components to verify that it is of the correct
;  type and size.
;
	IF DATATYPE(OBS.PROG_NUM,1) NE 'Long' THEN BEGIN
           MESSAGE = 'Tag PROG_NUM must be a long integer'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(OBS.PROG_NUM) NE 1 THEN BEGIN
           MESSAGE = 'Tag PROG_NUM must be a scalar'
           GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(OBS.PROG_ID,1) NE 'Integer' THEN BEGIN
           MESSAGE = 'Tag PROG_ID must be a short integer'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(OBS.PROG_ID) NE 1 THEN BEGIN
           MESSAGE = 'Tag PROG_ID must be a scalar'
           GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(OBS.PROG_IND,1) NE 'Integer' THEN BEGIN
           MESSAGE = 'Tag PROG_IND must be a short integer'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(OBS.PROG_IND) NE 1 THEN BEGIN
           MESSAGE = 'Tag PROG_IND must be a scalar'
           GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(OBS.STUDY_ID,1) NE 'Integer' THEN BEGIN
           MESSAGE = 'Tag STUDY_ID must be a short integer'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(OBS.STUDY_ID) NE 1 THEN BEGIN
           MESSAGE = 'Tag STUDY_ID must be a scalar'
           GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(OBS.STUDYVAR,1) NE 'Integer' THEN BEGIN
           MESSAGE = 'Tag STUDYVAR must be a short integer'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(OBS.STUDYVAR) NE 1 THEN BEGIN
           MESSAGE = 'Tag STUDYVAR must be a scalar'
           GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(OBS.OBS_PROG,1) NE 'String' THEN BEGIN
           MESSAGE = 'Tag OBS_PROG must be a character string'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(OBS.OBS_PROG) NE 1 THEN BEGIN
           MESSAGE = 'Tag OBS_PROG must be a scalar'
           GOTO, HANDLE_ERROR
	END ELSE IF STRLEN(OBS.OBS_PROG) GT 8 THEN BEGIN $
           MESSAGE = 'Tag OBS_PROG must be 8 characters or less'
           GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(OBS.DETECTOR,1) NE 'String' THEN BEGIN
           MESSAGE = 'Tag DETECTOR must be a character string'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(OBS.DETECTOR) NE 1 THEN BEGIN
           MESSAGE = 'Tag DETECTOR must be a scalar'
           GOTO, HANDLE_ERROR
	END ELSE IF STRLEN(OBS.DETECTOR) GT 1 THEN BEGIN $
           MESSAGE = 'Tag DETECTOR must be 1 characters or less'
           GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(OBS.SCI_OBJ,1) NE 'String' THEN BEGIN
           MESSAGE = 'Tag SCI_OBJ must be a character string'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(OBS.SCI_OBJ) NE 1 THEN BEGIN
           MESSAGE = 'Tag SCI_OBJ must be a scalar'
           GOTO, HANDLE_ERROR
	END ELSE IF STRLEN(OBS.SCI_OBJ) GT 50 THEN BEGIN $
           MESSAGE = 'Tag SCI_OBJ must be 50 characters or less'
           GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(OBS.SCI_SPEC,1) NE 'String' THEN BEGIN
           MESSAGE = 'Tag SCI_SPEC must be a character string'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(OBS.SCI_SPEC) NE 1 THEN BEGIN
           MESSAGE = 'Tag SCI_SPEC must be a scalar'
           GOTO, HANDLE_ERROR
	END ELSE IF STRLEN(OBS.SCI_SPEC) GT 50 THEN BEGIN
           MESSAGE = 'Tag SCI_SPEC must be 50 characters or less'
           GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(OBS.CMP_NO,1) NE 'Integer' THEN BEGIN
           MESSAGE = 'Tag CMP_NO must be a short integer'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(OBS.CMP_NO) NE 1 THEN BEGIN
           MESSAGE = 'Tag CMP_NO must be a scalar'
           GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(OBS.OBJECT,1) NE 'String' THEN BEGIN
           MESSAGE = 'Tag OBJECT must be a character string'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(OBS.OBJECT) NE 1 THEN BEGIN
           MESSAGE = 'Tag OBJECT must be a scalar'
           GOTO, HANDLE_ERROR
	END ELSE IF STRLEN(OBS.OBJECT) GT 3 THEN BEGIN
           MESSAGE = 'Tag OBJECT must be 3 characters or less'
           GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(OBS.OBJ_ID,1) NE 'String' THEN BEGIN
           MESSAGE = 'Tag OBJ_ID must be a character string'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(OBS.OBJ_ID) NE 1 THEN BEGIN
           MESSAGE = 'Tag OBJ_ID must be a scalar'
           GOTO, HANDLE_ERROR
	END ELSE IF STRLEN(OBS.OBJ_ID) GT 6 THEN BEGIN
           MESSAGE = 'Tag OBJ_ID must be 6 characters or less'
           GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(OBS.DATE_OBS,1) NE 'Double' THEN BEGIN
           MESSAGE = 'Tag DATE_OBS must be a double precision number'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(OBS.DATE_OBS) NE 1 THEN BEGIN
           MESSAGE = 'Tag DATE_OBS must be a scalar'
           GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(OBS.DATE_END,1) NE 'Double' THEN BEGIN
           MESSAGE = 'Tag DATE_END must be a double precision number'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(OBS.DATE_END) NE 1 THEN BEGIN
           MESSAGE = 'Tag DATE_END must be a scalar'
           GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(OBS.OBT_TIME,1) NE 'Double' THEN BEGIN
           MESSAGE = 'Tag OBT_TIME must be a double precision number'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(OBS.OBT_TIME) NE 1 THEN BEGIN
           MESSAGE = 'Tag OBT_TIME must be a scalar'
           GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(OBS.OBT_END,1) NE 'Double' THEN BEGIN
           MESSAGE = 'Tag OBT_END must be a double precision number'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(OBS.OBT_END) NE 1 THEN BEGIN
           MESSAGE = 'Tag OBT_END must be a scalar'
           GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(OBS.XCEN,1) NE 'Float' THEN BEGIN
           MESSAGE = 'Tag XCEN must be a floating point number'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(OBS.XCEN) NE 1 THEN BEGIN
           MESSAGE = 'Tag XCEN must be a scalar'
           GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(OBS.YCEN,1) NE 'Float' THEN BEGIN
           MESSAGE = 'Tag YCEN must be a floating point number'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(OBS.YCEN) NE 1 THEN BEGIN
           MESSAGE = 'Tag YCEN must be a scalar'
           GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(OBS.ANGLE,1) NE 'Float' THEN BEGIN
           MESSAGE = 'Tag ANGLE must be a floating point number'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(OBS.ANGLE) NE 1 THEN BEGIN
           MESSAGE = 'Tag ANGLE must be a scalar'
           GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(OBS.IXWIDTH,1) NE 'Float' THEN BEGIN
           MESSAGE = 'Tag IXWIDTH must be a floating point number'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(OBS.IXWIDTH) NE 1 THEN BEGIN
           MESSAGE = 'Tag IXWIDTH must be a scalar'
           GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(OBS.IYWIDTH,1) NE 'Float' THEN BEGIN
           MESSAGE = 'Tag IYWIDTH must be a floating point number'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(OBS.IYWIDTH) NE 1 THEN BEGIN
           MESSAGE = 'Tag IYWIDTH must be a scalar'
           GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(OBS.SEQ_FROM,1) NE 'Long' THEN BEGIN
           MESSAGE = 'Tag SEQ_FROM must be a long integer'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(OBS.SEQ_FROM) NE 1 THEN BEGIN
           MESSAGE = 'Tag SEQ_FROM must be a scalar'
           GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(OBS.SEQ_TO,1) NE 'Long' THEN BEGIN
           MESSAGE = 'Tag SEQ_TO must be a long integer'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(OBS.SEQ_TO) NE 1 THEN BEGIN
           MESSAGE = 'Tag SEQ_TO must be a scalar'
           GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(OBS.COMMENTS,1) NE 'String' THEN BEGIN
           MESSAGE = 'Tag COMMENTS must be a character string'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(OBS.COMMENTS) NE 1 THEN BEGIN
           MESSAGE = 'Tag COMMENTS must be a scalar'
           GOTO, HANDLE_ERROR
	END ELSE IF STRLEN(OBS.COMMENTS) GT 1 THEN BEGIN
           MESSAGE = 'Tag COMMENTS must be a single character'
           GOTO, HANDLE_ERROR
	END ELSE IF (OBS.COMMENTS NE 'N') AND (OBS.COMMENTS NE 'Y') THEN BEGIN
	   MESSAGE = 'Tag COMMENTS must be either Y or N'
	   GOTO, HANDLE_ERROR
	ENDIF
;
;  Make sure that the program ID matches an entry in the CDS program database.
;
	IF OBS.PROG_ID GT 0 THEN BEGIN
		GET_PROGRAM, OBS.PROG_ID, TEMP
		IF TEMP.PROG_ID LE 0 THEN CDS_MESSAGE, /CONTINUE, /ATTENTION, $
			'Program ID ' + STRTRIM(OBS.PROG_ID,2) +	$
			' not found in database'
	ENDIF
;
;  Make sure that the campaign number matches an entry in the SOHO campaign
;  database.
;
	IF OBS.CMP_NO GT 0 THEN BEGIN
		GET_CAMPAIGN, OBS.CMP_NO, TEMP
		IF TEMP.CMP_NO LE 0 THEN CDS_MESSAGE, /CONTINUE, /ATTENTION, $
			'Campaign number ' + STRTRIM(OBS.CMP_NO,2) + $
			' not found in database'
	ENDIF
;
;  Make sure that the object code matches an entry in the database.
;
	GET_OBJECT, OBS.OBJECT, TEMP
	IF TEMP.OBJECT EQ '' THEN BEGIN
           CDS_MESSAGE, /CONTINUE, /ATTENTION, 'Object code ' + OBS.OBJECT + $
              ' not found in database'
	   CDS_MESSAGE, /CONTINUE, 'Resetting to blank'
	   OBS.OBJECT = ''
	ENDIF
;
;  Make sure that the study ID and study variation index match an entry in the
;  CDS study database.
;
	GET_STUDY, OBS.STUDY_ID, OBS.STUDYVAR, STUDY
	IF STUDY.STUDY_ID LE -1 THEN CDS_MESSAGE, /CONTINUE, /ATTENTION, $
		'Study ID/variation ' + TRIM(OBS.STUDY_ID) + ',' + $
		TRIM(OBS.STUDYVAR) + ' not found in database'
;
;  Make sure that the stop date is greater than the start date.
;
	IF OBS.DATE_END LE OBS.DATE_OBS THEN BEGIN
           MESSAGE = 'The stop date must be after the start date'
           GOTO, HANDLE_ERROR
	ENDIF
;
	IF OBS.OBT_END LE OBS.OBT_TIME THEN BEGIN
           MESSAGE = 'The OBT stop date must be after the start date'
           GOTO, HANDLE_ERROR
	ENDIF
;
;  Reformat the times to millisecond accuracy.  This is necessary so that
;  the times are written out in a controlled way.
;
	OBS.DATE_OBS = DOUBLE( STRING( OBS.DATE_OBS, FORMAT='(F15.3)' ))
	OBS.DATE_END = DOUBLE( STRING( OBS.DATE_END, FORMAT='(F15.3)' ))
	OBS.OBT_TIME = DOUBLE( STRING( OBS.OBT_TIME, FORMAT='(F15.3)' ))
	OBS.OBT_END  = DOUBLE( STRING( OBS.OBT_END,  FORMAT='(F15.3)' ))
;
;  Make sure that the proposed entry is already in the database.
;
	GET_MAIN, OBS.PROG_NUM, OLD
	IF OLD.PROG_NUM LT 0 THEN BEGIN
	   MESSAGE = 'Program number ' + TRIM(OBS.PROG_NUM) +	$
		   ' is not in the database'
	   GOTO, HANDLE_ERROR
	ENDIF
;
;  Lock the database.
;
	LOCK_DATABASE, 'main', LOCKFILE
;
;  Open the database for write access, and find the exact record to modify.
;
	DBOPEN, 'main', 1
	ENTRY = DBFIND('PROG_NUM=' + TRIM(OBS.PROG_NUM), /SILENT)
	ENTRY = ENTRY(0)
;
;  For each entry, check to see if it has changed.  If it has, then update the
;  database.
;
	MODS = 0
;
;  Start with COMMENTS, because it has an extra check.  That way, if an error
;  occurs, then no changes will be made.
;
	IF OBS.COMMENTS NE OLD.COMMENTS THEN BEGIN
		IF OBS.COMMENTS EQ 'N' THEN BEGIN
			MESSAGE = "Can't change COMMENTS from Y to N"
			GOTO, HANDLE_ERROR
		END ELSE BEGIN
			DBUPDATE, ENTRY, 'comments', OBS.COMMENTS
			PRINT, 'Modified parameter COMMENTS'
			MODS = MODS + 1
		ENDELSE
	ENDIF
;
	IF OBS.PROG_ID NE OLD.PROG_ID THEN BEGIN
		DBUPDATE, ENTRY, 'prog_id', OBS.PROG_ID
		PRINT, 'Modified parameter PROG_ID'
		MODS = MODS + 1
	ENDIF
;
	IF OBS.PROG_IND NE OLD.PROG_IND THEN BEGIN
		DBUPDATE, ENTRY, 'prog_ind', OBS.PROG_IND
		PRINT, 'Modified parameter PROG_IND'
		MODS = MODS + 1
	ENDIF
;
	IF OBS.STUDY_ID NE OLD.STUDY_ID THEN BEGIN
		DBUPDATE, ENTRY, 'study_id', OBS.STUDY_ID
		PRINT, 'Modified parameter STUDY_ID'
		MODS = MODS + 1
	ENDIF
;
	IF OBS.STUDYVAR NE OLD.STUDYVAR THEN BEGIN
		DBUPDATE, ENTRY, 'studyvar', OBS.STUDYVAR
		PRINT, 'Modified parameter STUDYVAR'
		MODS = MODS + 1
	ENDIF
;
	IF STRTRIM(OBS.OBS_PROG) NE STRTRIM(OLD.OBS_PROG) THEN BEGIN
		DBUPDATE, ENTRY, 'obs_prog', OBS.OBS_PROG
		PRINT, 'Modified parameter OBS_PROG'
		MODS = MODS + 1
	ENDIF
;
	IF (OBS.DETECTOR NE OLD.DETECTOR) AND (OLD.DETECTOR NE 'B') THEN BEGIN
		IF TRIM(OLD.DETECTOR) EQ '' THEN DETECTOR = OBS.DETECTOR ELSE $
			DETECTOR = 'B'
		DBUPDATE, ENTRY, 'detector', DETECTOR
		PRINT, 'Modified parameter DETECTOR'
		MODS = MODS + 1
	ENDIF
;
	IF STRTRIM(OBS.SCI_OBJ) NE STRTRIM(OLD.SCI_OBJ) THEN BEGIN
		DBUPDATE, ENTRY, 'sci_obj', OBS.SCI_OBJ
		PRINT, 'Modified parameter SCI_OBJ'
		MODS = MODS + 1
	ENDIF
;
	IF STRTRIM(OBS.SCI_SPEC) NE STRTRIM(OLD.SCI_SPEC) THEN BEGIN
		DBUPDATE, ENTRY, 'sci_spec', OBS.SCI_SPEC
		PRINT, 'Modified parameter SCI_SPEC'
		MODS = MODS + 1
	ENDIF
;
	IF OBS.CMP_NO NE OLD.CMP_NO THEN BEGIN
		DBUPDATE, ENTRY, 'cmp_no', OBS.CMP_NO
		PRINT, 'Modified parameter CMP_NO'
		MODS = MODS + 1
	ENDIF
;
	IF OBS.OBJECT NE OLD.OBJECT THEN BEGIN
		DBUPDATE, ENTRY, 'object', OBS.OBJECT
		PRINT, 'Modified parameter OBJECT'
		MODS = MODS + 1
	ENDIF
;
	IF STRTRIM(OBS.OBJ_ID) NE STRTRIM(OLD.OBJ_ID) THEN BEGIN
		DBUPDATE, ENTRY, 'obj_id', OBS.OBJ_ID
		PRINT, 'Modified parameter OBJ_ID'
		MODS = MODS + 1
	ENDIF
;
	IF OBS.DATE_OBS NE OLD.DATE_OBS THEN BEGIN
		DBUPDATE, ENTRY, 'date_obs', OBS.DATE_OBS
		PRINT, 'Modified parameter DATE_OBS'
		MODS = MODS + 1
	ENDIF
;
	IF OBS.DATE_END NE OLD.DATE_END THEN BEGIN
		DBUPDATE, ENTRY, 'date_end', OBS.DATE_END
		PRINT, 'Modified parameter DATE_END'
		MODS = MODS + 1
	ENDIF
;
	IF OBS.OBT_TIME NE OLD.OBT_TIME THEN BEGIN
		DBUPDATE, ENTRY, 'obt_time', OBS.OBT_TIME
		PRINT, 'Modified parameter OBT_TIME'
		MODS = MODS + 1
	ENDIF
;
	IF OBS.OBT_END NE OLD.OBT_END THEN BEGIN
		DBUPDATE, ENTRY, 'obt_end', OBS.OBT_END
		PRINT, 'Modified parameter OBT_END'
		MODS = MODS + 1
	ENDIF
;
	IF OBS.XCEN NE OLD.XCEN THEN BEGIN
		DBUPDATE, ENTRY, 'xcen', OBS.XCEN
		PRINT, 'Modified parameter XCEN'
		MODS = MODS + 1
	ENDIF
;
	IF OBS.YCEN NE OLD.YCEN THEN BEGIN
		DBUPDATE, ENTRY, 'ycen', OBS.YCEN
		PRINT, 'Modified parameter YCEN'
		MODS = MODS + 1
	ENDIF
;
	IF OBS.ANGLE NE OLD.ANGLE THEN BEGIN
		DBUPDATE, ENTRY, 'angle', OBS.ANGLE
		PRINT, 'Modified parameter ANGLE'
		MODS = MODS + 1
	ENDIF
;
	IF OBS.IXWIDTH NE OLD.IXWIDTH THEN BEGIN
		DBUPDATE, ENTRY, 'ixwidth', OBS.IXWIDTH
		PRINT, 'Modified parameter IXWIDTH'
		MODS = MODS + 1
	ENDIF
;
	IF OBS.IYWIDTH NE OLD.IYWIDTH THEN BEGIN
		DBUPDATE, ENTRY, 'iywidth', OBS.IYWIDTH
		PRINT, 'Modified parameter IYWIDTH'
		MODS = MODS + 1
	ENDIF
;
	IF OBS.SEQ_FROM NE OLD.SEQ_FROM THEN BEGIN
		DBUPDATE, ENTRY, 'seq_from', OBS.SEQ_FROM
		PRINT, 'Modified parameter SEQ_FROM'
		MODS = MODS + 1
	ENDIF
;
	IF OBS.SEQ_TO NE OLD.SEQ_TO THEN BEGIN
		DBUPDATE, ENTRY, 'seq_to', OBS.SEQ_TO
		PRINT, 'Modified parameter SEQ_TO'
		MODS = MODS + 1
	ENDIF
;
;  If anything has been modified, then update the modification date.
;
	IF MODS GT 0 THEN BEGIN
		GET_UTC, DATE_MOD
		DATE_MOD = UTC2TAI(DATE_MOD)
		DBUPDATE, ENTRY, 'date_mod', DATE_MOD
	ENDIF
;
;  Signal success.
;
	RESULT = MODS
	GOTO, FINISH
;
;  Error handling point.
;
HANDLE_ERROR:
	IF N_ELEMENTS(ERRMSG) NE 0 THEN ERRMSG = 'MOD_MAIN: ' + MESSAGE $
		ELSE MESSAGE, MESSAGE, /CONTINUE
;
;  Close the database, and return whether the routine was successful or not.
;
FINISH:
	DBCLOSE
;
	IF N_ELEMENTS(LOCKFILE) EQ 1 THEN UNLOCK_DATABASE, LOCKFILE
	RETURN, RESULT
	END
