FUNCTION MOD_V_RASTER, RAS_ID, RAS_VAR, TEL_RATE=TEL_RATE,	$
		DURATION=DURATION, USABLE=USABLE, ERRMSG=ERRMSG
;+
; Project     :	SOHO - CDS
;
; Name        :	MOD_V_RASTER()
;
; Purpose     :	Modifies a raster variation definition in the database
;
; Explanation :	This procedure allows one to modify certain parameters in the
;		definition of a raster variation.
;
; Use         :	Result = MOD_V_RASTER( RAS_ID, RAS_VAR, keywords=... )
;
;		IF NOT MOD_V_RASTER( RAS_ID, RAS_VAR, keywords=... ) THEN ...
;
; Inputs      :	RAS_ID	 = Raster ID number.
;		RAS_VAR	 = Raster variation index.
;
;		The above parameters simply allow the relevant entry to be
;		found in the database.  The modifications are passed through
;		keyword parameters.
;
; Opt. Inputs :	None.
;
; Outputs     :	The result of the function is a logical value representing
;		whether or not the operation was successful, where 1 is
;		successful and 0 is unsuccessful.
;
; Opt. Outputs:	None.
;
; Keywords    :	
;       TEL_RATE = Estimated required telemetry rate, Low/Medium/High
;	DURATION = Estimated duration of the raster, in seconds.
;	USABLE	 = Either "Y" or "N" to signal whether or not the
;		   raster is usable.  Normally "Y".
;
;		The keywords specify the modifications to be made to the
;		database.
;
;       ERRMSG    = If defined and passed, then any error messages will be
;                   returned to the user in this parameter rather than
;                   depending on the MESSAGE routine in IDL.  If no errors are
;                   encountered, then a null string is returned.  In order to
;                   use this feature, ERRMSG must be defined first, e.g.
;
;                       ERRMSG = ''
;                       Result = MOD_V_RASTER( ERRMSG=ERRMSG, ... )
;                       IF ERRMSG NE '' THEN ...
;
; Calls       :	DATATYPE, DBOPEN, DBFIND, DBBUILD, DBCLOSE
;
; Common      :	None.
;
; Restrictions:	Only this routine can be used to modify raster variation
;		definitions in the database.  Modifying the database by hand
;		could corrupt its integrity.
;
;		The data types and sizes of the structure elements must match
;		the definitions in the database.  The string lengths must not
;		exceed the lengths defined in the database.
;
;		!PRIV must be 2 or greater to use this routine.
;
; Side effects:	Once a raster definition has been added to the database, it
;		must never be removed.
;
; Category    :	Planning, Databases.
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 5 April 1994
;
; Modified    :	Version 1, William Thompson, GSFC, 4 May 1994
;               Version 2, Liyun Wang, GSFC/ARC, September 22, 1994
;                  Added the keyword ERRMSG.
;		Version 3, William Thompson, GSFC, 29 December 1994
;			Fixed bug when inputs are bytes.
;		Version 4, William Thompson, GSFC, 15 March 1995
;			Changed TEL_RATE to character
;		Version 5, William Thompson, GSFC, 11 December 1997
;			Removed TEL_RATE value check, to allow for additional
;			values besides L/M/H.
;
; Version     :	Version 5, 11 December 1997
;-
;
	ON_ERROR, 2
;
;  Check the input parameters
;
        IF N_PARAMS() NE 2 THEN BEGIN
           MESSAGE = $
              'Syntax:  Result = MOD_V_RASTER( RAS_ID, RAS_VAR, keywords=... )'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN BEGIN
              ERRMSG = MESSAGE
              RETURN, 0
           ENDIF ELSE MESSAGE, MESSAGE
        ENDIF
;
;  Initialize RESULT to represent non-success.  If the routine is successful,
;  this value will be updated below.
;
	RESULT = 0
;
;  Make sure that the user has privilege to write into the database.
;
	IF !PRIV LT 2 THEN BEGIN $
           MESSAGE = '!PRIV must be 2 or greater to write into the database'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	ENDIF
;
;  Check each of the input parameters to verify that it is of the correct type
;  and size.
;
	IF DATATYPE(RAS_ID,1) NE 'Integer' THEN BEGIN
           MESSAGE = 'RAS_ID must be a short integer'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	END ELSE IF N_ELEMENTS(RAS_ID) NE 1 THEN BEGIN
           MESSAGE = 'RAS_ID must be a scalar'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	ENDIF
;
	IF DATATYPE(RAS_VAR,1) NE 'Integer' THEN BEGIN
           MESSAGE = 'RAS_VAR must be a short integer'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	END ELSE IF N_ELEMENTS(RAS_VAR) NE 1 THEN BEGIN
           MESSAGE = 'RAS_VAR must be a scalar'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	ENDIF
;
	IF N_ELEMENTS(TEL_RATE) NE 0 THEN BEGIN
		IF DATATYPE(TEL_RATE,1) NE 'String' THEN BEGIN
                   MESSAGE = 'TEL_RATE must be a character string'
                   IF N_ELEMENTS(ERRMSG) NE 0 THEN $
                      ERRMSG = MESSAGE $
                   ELSE MESSAGE, MESSAGE, /CONTINUE 
                   GOTO, FINISH
		END ELSE IF N_ELEMENTS(TEL_RATE) NE 1 THEN BEGIN
                   MESSAGE = 'TEL_RATE must be a scalar'
                   IF N_ELEMENTS(ERRMSG) NE 0 THEN $
                      ERRMSG = MESSAGE $
                   ELSE MESSAGE, MESSAGE, /CONTINUE 
                   GOTO, FINISH
		END ELSE IF STRLEN(TEL_RATE) NE 1 THEN BEGIN
                   MESSAGE = 'TEL_RATE must be a single character'
                   IF N_ELEMENTS(ERRMSG) NE 0 THEN $
                      ERRMSG = MESSAGE $
                   ELSE MESSAGE, MESSAGE, /CONTINUE 
                   GOTO, FINISH
		ENDIF
	ENDIF
;
	IF N_ELEMENTS(DURATION) NE 0 THEN BEGIN
		IF DATATYPE(DURATION,1) NE 'Float' THEN BEGIN
                   MESSAGE = 'DURATION must be a floating point number'
                   IF N_ELEMENTS(ERRMSG) NE 0 THEN $
                      ERRMSG = MESSAGE $
                   ELSE MESSAGE, MESSAGE, /CONTINUE 
                   GOTO, FINISH
		END ELSE IF N_ELEMENTS(DURATION) NE 1 THEN BEGIN
                   MESSAGE = 'DURATION must be a scalar'
                   IF N_ELEMENTS(ERRMSG) NE 0 THEN $
                      ERRMSG = MESSAGE $
                   ELSE MESSAGE, MESSAGE, /CONTINUE 
                   GOTO, FINISH
		ENDIF
	ENDIF
;
	IF N_ELEMENTS(USABLE) NE 0 THEN BEGIN
		IF DATATYPE(USABLE,1) NE 'String' THEN BEGIN
                   MESSAGE = 'USABLE must be a character string'
                      IF N_ELEMENTS(ERRMSG) NE 0 THEN $
                      ERRMSG = MESSAGE $
                   ELSE MESSAGE, MESSAGE, /CONTINUE 
                   GOTO, FINISH
		END ELSE IF N_ELEMENTS(USABLE) NE 1 THEN BEGIN
                   MESSAGE = 'USABLE must be a scalar'
                   IF N_ELEMENTS(ERRMSG) NE 0 THEN $
                      ERRMSG = MESSAGE $
                   ELSE MESSAGE, MESSAGE, /CONTINUE 
                   GOTO, FINISH
		END ELSE IF STRLEN(USABLE) NE 1 THEN BEGIN
                   MESSAGE = 'USABLE must be a single character'
                   IF N_ELEMENTS(ERRMSG) NE 0 THEN $
                      ERRMSG = MESSAGE $
                   ELSE MESSAGE, MESSAGE, /CONTINUE 
                   GOTO, FINISH
		END ELSE IF (USABLE NE 'Y') AND (USABLE NE 'N') THEN BEGIN 
                   MESSAGE = 'USABLE must be either "Y" or "N"'
                   IF N_ELEMENTS(ERRMSG) NE 0 THEN $
                      ERRMSG = MESSAGE $
                   ELSE MESSAGE, MESSAGE, /CONTINUE 
                   GOTO, FINISH
		ENDIF
	ENDIF
;
;  Find the entry in question.
;
	DBOPEN, 'raster_var,raster'
	ENTRY = DBFIND('RAS_ID='+ STRTRIM(LONG(RAS_ID),2) + ',RAS_VAR=' + $
		STRTRIM(LONG(RAS_VAR),2), /SILENT)
	IF !ERR EQ 0 THEN BEGIN
           MESSAGE = 'Raster ID/variation ' + TRIM(LONG(RAS_ID)) + '/' + $
		   TRIM(LONG(RAS_VAR)) + ' not found in database'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	ENDIF
;
;  Open the raster variation database for write access.
;
	DBOPEN, 'raster_var', 1
;
;  For each keyword passed, update the database.  Signal success.
;
	IF N_ELEMENTS(TEL_RATE) NE 0 THEN DBUPDATE, ENTRY, 'tel_rate', TEL_RATE
	IF N_ELEMENTS(DURATION) NE 0 THEN DBUPDATE, ENTRY, 'duration', DURATION
	IF N_ELEMENTS(USABLE)   NE 0 THEN DBUPDATE, ENTRY, 'usable',   USABLE
;
	RESULT = 1
;
;  Close the database, and return whether the routine was successful or not.
;
FINISH:
	DBCLOSE
;
	RETURN, RESULT
	END
