	FUNCTION MOD_V_STUDY, STUDY_ID, STUDYVAR, DURATION0=DURATION0,	$
		DURATION1=DURATION1, USABLE=USABLE, ZONE_ID=ZONE_ID,	$
		ERRMSG=ERRMSG
;+
; Project     :	SOHO - CDS
;
; Name        :	MOD_V_STUDY()
;
; Purpose     :	Modifies a study variation definition in the database
;
; Explanation :	This procedure allows one to modify certain parameters in the
;		definition of a study variation.
;
; Use         :	Result = MOD_V_STUDY( STUDY_ID, STUDYVAR, keywords=... )
;
;		IF NOT MOD_V_STUDY( STUDY_ID, STUDYVAR, keywords=... ) THEN ...
;
; Inputs      :	STUDY_ID = Study ID number.
;		STUDYVAR = Study variation index.
;
;		The above parameters simply allow the relevant entry to be
;		found in the database.  The modifications are passed through
;		keyword parameters.
;
; Opt. Inputs :	None.
;
; Outputs     :	The result of the function is a logical value representing
;		whether or not the operation was successful, where 1 is
;		successful and 0 is unsuccessful.
;
; Opt. Outputs:	None.
;
; Keywords    :	
;       DURATION0 = Fixed part of the study duration, in seconds.
;	DURATION1 = Number of seconds/raster for the variable part of
;		    the study duration.
;	USABLE	  = Either "Y" or "N" to signal whether or not the
;		    study is usable.  Normally "Y".
;	ZONE_ID	  = Pointing zone ID.
;
;		The above keywords specify the modifications to be made to the
;		database.
;
;       ERRMSG    = If defined and passed, then any error messages will be
;                   returned to the user in this parameter rather than
;                   depending on the MESSAGE routine in IDL.  If no errors are
;                   encountered, then a null string is returned.  In order to
;                   use this feature, ERRMSG must be defined first, e.g.
;
;                       ERRMSG = ''
;                       Result = MOD_V_STUDY( ERRMSG=ERRMSG, ... )
;                       IF ERRMSG NE '' THEN ...
;
; Calls       :	DATATYPE, DBOPEN, DBFIND, DBBUILD, DBCLOSE
;
; Common      :	None.
;
; Restrictions:	Only this routine can be used to modify study variation
;		definitions in the database.  Modifying the database by hand
;		could corrupt its integrity.
;
;		The data types and sizes of the structure elements must match
;		the definitions in the database.  The string lengths must not
;		exceed the lengths defined in the database.
;
;		!PRIV must be 2 or greater to use this routine.
;
; Side effects:	Once a study definition has been added to the database, it
;		must never be removed.
;
; Category    :	Planning, Databases.
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 29 April 1994
;
; Modified    :	Version 1, William Thompson, GSFC, 3 May 1994
;               Version 2, Liyun Wang, GSFC/ARC, September 22, 1994
;                  Added the keyword ERRMSG.
;		Version 3, William Thompson, GSFC, 29 December 1994
;			Fixed bug when inputs are bytes.
;		Version 4, William Thompson, GSFC, 12 May 1995
;			Modified to split off separate study_title database
;		Version 5, William Thompson, GSFC, 8 May 1996
;			Added keyword ZONE_ID
;
; Version     :	Version 5, 8 May 1996
;-
;
	ON_ERROR, 2
;
;  Initialize RESULT to represent non-success.  If the routine is successful,
;  this value will be updated below.
;
	RESULT = 0
;
;  Check the input parameters
;
        IF N_PARAMS() NE 2 THEN BEGIN
           MESSAGE = 'Syntax:  Result = ' +	$
		'MOD_V_STUDY( STUDY_ID, STUDYVAR, keywords=... )'
	   GOTO, HANDLE_ERROR
        ENDIF
;
;  Make sure that the user has privilege to write into the database.
;
	IF !PRIV LT 2 THEN BEGIN $
           MESSAGE = '!PRIV must be 2 or greater to write into the database'
           GOTO, HANDLE_ERROR
	ENDIF
;
;  Check each of the input parameters to verify that it is of the correct type
;  and size.
;
	IF DATATYPE(STUDY_ID,1) NE 'Integer' THEN BEGIN
           MESSAGE = 'STUDY_ID must be a short integer'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(STUDY_ID) NE 1 THEN BEGIN
           MESSAGE = 'STUDY_ID must be a scalar'
           GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(STUDYVAR,1) NE 'Integer' THEN BEGIN
           MESSAGE = 'STUDYVAR must be a short integer'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(STUDYVAR) NE 1 THEN BEGIN
           MESSAGE = 'STUDYVAR must be a scalar'
           GOTO, HANDLE_ERROR
	ENDIF
;
	IF N_ELEMENTS(DURATION0) NE 0 THEN BEGIN
		IF DATATYPE(DURATION0,1) NE 'Float' THEN BEGIN
                   MESSAGE = 'DURATION0 must be a floating point number'
                   GOTO, HANDLE_ERROR
		END ELSE IF N_ELEMENTS(DURATION0) NE 1 THEN BEGIN
                   MESSAGE = 'DURATION0 must be a scalar'
                   GOTO, HANDLE_ERROR
		ENDIF
	ENDIF
;
	IF N_ELEMENTS(DURATION1) NE 0 THEN BEGIN
		IF DATATYPE(DURATION1,1) NE 'Float' THEN BEGIN
                   MESSAGE = 'DURATION1 must be a floating point number'
                   GOTO, HANDLE_ERROR
		END ELSE IF N_ELEMENTS(DURATION1) NE 1 THEN BEGIN
                   MESSAGE = 'DURATION1 must be a scalar'
                   GOTO, HANDLE_ERROR
		ENDIF
	ENDIF
;
	IF N_ELEMENTS(USABLE) NE 0 THEN BEGIN
		IF DATATYPE(USABLE,1) NE 'String' THEN BEGIN
                   MESSAGE = 'USABLE must be a character string'
                   GOTO, HANDLE_ERROR
		END ELSE IF N_ELEMENTS(USABLE) NE 1 THEN BEGIN
                   MESSAGE = 'USABLE must be a scalar'
                   GOTO, HANDLE_ERROR
		END ELSE IF STRLEN(USABLE) NE 1 THEN BEGIN
                   MESSAGE = 'USABLE must be a single character'
                   GOTO, HANDLE_ERROR
		END ELSE IF (USABLE NE 'Y') AND (USABLE NE 'N') THEN BEGIN $
                   MESSAGE = 'USABLE must be either "Y" or "N"'
                   GOTO, HANDLE_ERROR
		ENDIF
	ENDIF
;
	IF N_ELEMENTS(ZONE_ID) NE 0 THEN BEGIN
		IF DATATYPE(ZONE_ID,1) NE 'Integer' THEN BEGIN
        	   MESSAGE = 'ZONE_ID must be a short integer'
	           GOTO, HANDLE_ERROR
		END ELSE IF N_ELEMENTS(ZONE_ID) NE 1 THEN BEGIN
	           MESSAGE = 'ZONE_ID must be a scalar'
        	   GOTO, HANDLE_ERROR
		END ELSE IF ZONE_ID LE 0 THEN BEGIN
		   MESSAGE = 'ZONE_ID must be positive'
	           GOTO, HANDLE_ERROR
		ENDIF
	ENDIF
;
;  Open the study variation database for read access, and find the entry in
;  question.
;
	DBOPEN, 'study_var,study_title,study'
	ENTRY = DBFIND('STUDY_ID='+ STRTRIM(LONG(STUDY_ID),2) +	$
		',STUDYVAR=' + STRTRIM(LONG(STUDYVAR),2), /SILENT)
	IF !ERR EQ 0 THEN BEGIN
           MESSAGE = 'Study ID/variation ' + TRIM(LONG(STUDY_ID)) + '/' + $
		   TRIM(LONG(STUDYVAR)) + ' not found in database'
           GOTO, HANDLE_ERROR
	ENDIF
;
;  Open the study variation database for write access.
;
	DBOPEN, 'study_var', 1
;
;  For each keyword passed, update the database.  Signal success.
;
	IF N_ELEMENTS(DURATION0) NE 0 THEN	$
		DBUPDATE, ENTRY, 'duration0', DURATION0
	IF N_ELEMENTS(DURATION1) NE 0 THEN	$
		DBUPDATE, ENTRY, 'duration1', DURATION1
	IF N_ELEMENTS(USABLE)    NE 0 THEN	$
		DBUPDATE, ENTRY, 'usable',    USABLE
	IF N_ELEMENTS(ZONE_ID)   NE 0 THEN	$
		DBUPDATE, ENTRY, 'zone_id',   ZONE_ID
;
	RESULT = 1
	GOTO, FINISH
;
;  Error handling point.
;
HANDLE_ERROR:
	IF N_ELEMENTS(ERRMSG) NE 0 THEN ERRMSG = 'MOD_V_STUDY: ' + MESSAGE $
		ELSE MESSAGE, MESSAGE, /CONTINUE
;
;  Close the database, and return whether the routine was successful or not.
;
FINISH:
	DBCLOSE
;
	RETURN, RESULT
	END
