	FUNCTION PRED_PROG_NUM, TIME, ERRMSG=ERRMSG, MOD_PLAN=MOD_PLAN
;+
; Project     :	SOHO - CDS
;
; Name        :	PRED_PROG_NUM()
;
; Purpose     :	Predict the study counter for a given date/time.
;
; Category    :	Class3, CDS, Planning
;
; Explanation :	Looks up the information in the SCI_DETAILS planning database,
;		and uses it to predict what the study counter will be for a
;		given date/time.  The predicted numbers are also optionally
;		stored in the planning database.
;
; Syntax      :	Result = PRED_PROG_NUM( [TIME] )
;
; Examples    :	PRINT, PRED_PROG_NUM()
;		PRINT, PRED_PROG_NUM('5-June-1997 12:30')
;
; Inputs      :	None required.
;
; Opt. Inputs :	TIME	= A date/time value, in any CDS format.  If omitted,
;			  then the current time is used.
;
; Outputs     :	The result of the function is the predicted program number.  If
;		unsuccessful, then 0 will be returned.
;
; Opt. Outputs:	None.
;
; Keywords    :	MOD_PLAN = If set, then predicted study counters are also
;			   stored in the planning database, if one has write
;			   privilege there.
;
;		ERRMSG = If defined and passed, then any error messages will be
;			 returned to the user in this parameter rather than
;			 depending on the MESSAGE routine in IDL.  If no errors
;			 are encountered, then a null string is returned.  In
;			 order to use this feature, ERRMSG must be defined
;			 first, e.g.
;
;				ERRMSG = ''
;				Result = PRED_PROG_NUM( ERRMSG=ERRMSG, ... )
;				IF ERRMSG NE '' THEN ...
;
; Calls       :	PRIV_ZDBASE, ANYTIM2TAI, UTC2TAI, LIST_DETAIL, MOD_DETAIL
;
; Common      :	None.
;
; Restrictions:	There are several factors which could affect the accuracy of
;		this prediction.  First of all, it depends on the accuracy of
;		FITSGEN in associating the telemetry with the science plan.
;		This process is fairly robust, but occasionally makes mistakes.
;		Second, the prediction can also be thrown off if the science
;		plan has been changed since the last time the instrument was
;		commanded, as sometimes happens during the recovery from an
;		instrument crash.  It's best to add a safety margin to the
;		result of this function.
;
; Side effects:	None.
;
; Prev. Hist. :	None.
;
; History     :	Version 1, 05-Jun-1997, William Thompson, GSFC
;
; Contact     :	WTHOMPSON
;-
;
	IF !DEBUG EQ 0 THEN ON_ERROR, 2 ELSE ON_ERROR, 0
	MESSAGE = ''
;
;  Determine whether or not one has privilege to write into the planning
;  databases.
;
	OLD_PRIV = !PRIV
	IF KEYWORD_SET(MOD_PLAN) THEN BEGIN
	    TEST_PRIV = PRIV_ZDBASE(/DAILY)
	    IF TEST_PRIV THEN !PRIV = 2 > OLD_PRIV
	END ELSE TEST_PRIV = 0
;
;  Check the input parameters.  Add one millisecond to the requested time, to
;  catch entries which start exactly on the requested time.
;
	IF N_PARAMS() EQ 0 THEN GET_UTC, TIME
	TAI2 = ANYTIM2TAI(TIME, ERRMSG=MESSAGE) + 0.001
	IF MESSAGE(0) NE '' THEN GOTO, HANDLE_ERROR
;
;  Search the planning database until a positive program number is found, or
;  one has gone back before the beginning of the mission.
;
	TAI0 = UTC2TAI('1995-12-2')
	TDELTA = 86400.D0
;
SEARCH:
	TAI1 = TAI2 - TDELTA
	IF TAI1 LT TAI0 THEN BEGIN
	    MESSAGE = 'No program numbers found before selected time'
	    GOTO, HANDLE_ERROR
	ENDIF
;
;  List all the entries in the catalog for the selected period, and find the
;  last one with a recorded program number.
;
	LIST_DETAIL, TAI1, TAI2, OBS, N_FOUND
	IF N_FOUND EQ 0 THEN BEGIN
	    TDELTA = 2 * TDELTA
	    GOTO, SEARCH
	ENDIF
;
	W = WHERE(OBS.PROG_NUM GT 0, COUNT)
	IF COUNT EQ 0 THEN BEGIN
	    TDELTA = 2 * TDELTA
	    GOTO, SEARCH
	ENDIF
;
;  Fill in program numbers for all unrecorded studies which follow.  Set them
;  to be negative to show that they're predictive.
;
	W = MAX(W)
	PROG_NUM = OBS(W).PROG_NUM
	FOR I = W+1, N_FOUND-1 DO BEGIN
;
;  Don't increment the study counter for the QCM check or the "One hour break"
;  study.
;
	    IF (OBS(I).STUDY_ID NE 0) OR (OBS(I).STUDYVAR GT 2) THEN	$
		    PROG_NUM = PROG_NUM + 1
	    OBS(I).PROG_NUM = -PROG_NUM
	    IF TEST_PRIV AND KEYWORD_SET(MOD_PLAN) THEN			$
		    IF NOT MOD_DETAIL(OBS(I), PROG_NUM=PROG_NUM, 	$
			    ERRMSG=MESSAGE) THEN GOTO, HANDLE_ERROR
	ENDFOR
;
;  Return the last program number in the series.
;
	RESULT = ABS(OBS(N_FOUND-1).PROG_NUM)
	GOTO, FINISH
;
;  Error handling point.
;
HANDLE_ERROR:
	IF N_ELEMENTS(ERRMSG) NE 0 THEN ERRMSG = 'PRED_PROG_NUM: ' + MESSAGE $
		ELSE MESSAGE, MESSAGE
	RESULT = 0
;
FINISH:
	!PRIV = OLD_PRIV
	RETURN, RESULT
	END
