	FUNCTION REG_CDROM, SEQUENCE, VOLUME, MJD, ERRMSG=ERRMSG
;+
; Project     :	SOHO - CDS
;
; Name        :	REG_CDROM
;
; Purpose     :	Register information about the level zero CDROMs.
;
; Category    :	Class4, Operations, Catalog
;
; Explanation :	Puts an entry in the CDROM database about a CDROM disk
;		containing CDS level-zero telemetry data.  The sequence number
;		and volume number are stored, together with the start and end
;		times of the telemetry packets.
;
;		Called from the routine LOAD_CDROM
;
; Syntax      :	Result = REG_CDROM( SEQUENCE, VOLUME, MJD )
;
; Examples    :	IF NOT REG_CDROM( ... ) THEN ...
;
; Inputs      :	SEQUENCE = The sequence number of the CDROM, taken from the
;			   file voldesc.sfd.  For example, if the sequence
;			   number was 2, then the CDROM would have the name
;			   "SO_CDS_LZ_0002" on the cover.
;
;		VOLUME	 = The volume number of the CDROM, taken from the
;			   file voldesc.sfc.  For example, if the volume
;			   number was 4, then the CDROM cover might say
;			   something like "4 of 9" on the cover.
;
;		MJD      = The modified julian day number of a date contained
;			   on the CDROM.
;
; Opt. Inputs :	None.
;
; Outputs     :	The result of the function is 1 if successful, or 0 if
;		unsuccessful.
;
; Opt. Outputs:	None.
;
; Keywords    :	ERRMSG = If defined and passed, then any error messages will be
;			 returned to the user in this parameter rather than
;			 depending on the MESSAGE routine in IDL.  If no errors
;			 are encountered, then a null string is returned.  In
;			 order to use this feature, ERRMSG must be defined
;			 first, e.g.
;
;				ERRMSG = ''
;				Result = REG_CDROM( ERRMSG=ERRMSG, ... )
;				IF ERRMSG NE '' THEN ...
;
; Calls       :	
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Prev. Hist. :	None.
;
; History     :	Version 1, 04-Apr-1996, William Thompson, GSFC
;		Version 2, 10-Apr-1996,	William Thompson, GSFC
;			Modified for version 2 of the CDROM database
;
; Contact     :	WTHOMPSON
;-
;
	ON_ERROR, 2
;
;  Initialize RESULT to represent non-success.  If the routine is successful,
;  this value will be updated below.
;
	RESULT = 0
;
;  Check the input parameters
;
	IF N_PARAMS() NE 3 THEN BEGIN
		MESSAGE = 'Syntax:  Result = REG_CDROM( SEQUENCE, VOLUME, MJD )'
		GOTO, HANDLE_ERROR
	ENDIF
;
;  Check each of the input parameters to verify that it is of the correct
;  type and size.
;
	IF DATATYPE(SEQUENCE,1) NE 'Long' THEN BEGIN
		MESSAGE = 'Parameter SEQUENCE must be a long integer'
		GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(SEQUENCE) NE 1 THEN BEGIN
		MESSAGE = 'Parameter SEQUENCE must be a scalar'
		GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(VOLUME,1) NE 'Long' THEN BEGIN
		MESSAGE = 'Parameter VOLUME must be a long integer'
		GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(VOLUME) NE 1 THEN BEGIN
		MESSAGE = 'Parameter VOLUME must be a scalar'
		GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(MJD,1) NE 'Long' THEN BEGIN
		MESSAGE = 'Parameter MJD must be a long integer'
		GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(MJD) NE 1 THEN BEGIN
		MESSAGE = 'Parameter MJD must be a scalar'
		GOTO, HANDLE_ERROR
	ENDIF
;
;  Open the CDROM database for write access.
;
	DBOPEN, 'cdrom', 1
;
;  Look for the sequence number, volume number, and MJD.  If found, then exit
;  immediately--the data has already been registered.
;
	ENTRY = DBFIND('SEQUENCE=' + TRIM(SEQUENCE) + ',VOLUME=' +	$
		TRIM(VOLUME) + ',MJD=' + TRIM(MJD), /SILENT)
	IF ENTRY(0) GE 1 THEN BEGIN
		RESULT = 1
		GOTO, FINISH
	ENDIF
;
;  Otherwise, add the entry.
;
	DBBUILD, SEQUENCE, VOLUME, MJD, STATUS=STATUS
	IF STATUS EQ 0 THEN BEGIN
		MESSAGE = 'Write to cdrom database was not successful'
		GOTO, HANDLE_ERROR
	ENDIF
;
;  Signal success.
;
	RESULT = 1
	GOTO, FINISH
;
;  Error handling point.
;
HANDLE_ERROR:
	IF N_ELEMENTS(ERRMSG) NE 0 THEN ERRMSG = 'REG_CDROM: ' + MESSAGE $
		ELSE MESSAGE, MESSAGE, /CONTINUE
;
;  Close the database, and return whether the routine was successful or not.
;
FINISH:
	DBCLOSE
;
	RETURN, RESULT
	END
