;+
; Project     : SOHO - CDS     
;                   
; Name        : SHOW_CDHS
;               
; Purpose     : Interprets CDHS series and raster IDs
;               
; Explanation : The series and raster IDs used by the CDHS are not the same
;               as the study and raster IDs in the ground based databases.
;               The correspondence between them is kept in the series_id and
;               raster_id databases.  This routine allows the user to see which 
;               groundbased studies and rasters are associated with a given
;               CDHS ID. If neither a series ID nor a raster ID are given then
;               a summary of all CDHS IDs is produced.
;               
; Use         : IDL> show_cdhs [,raster=raster, series=series, /hardcopy, $
;                                file=file, /quiet, /keep] 
;    
; Inputs      : None
;               
; Opt. Inputs : None
;               
; Outputs     : Information is optionally printed to the screen and hardcopy.
;               
; Opt. Outputs: None
;               
; Keywords    : RASTER   - specifies the CDHS raster ID
;               SERIES   - specifies the CDHS series ID
;               HARDCOPY - send copy of output to printer
;               KEEP     - inhibits deletion of hardcopy file created in the
;                          user's home directory.
;               FILE     - if null on entry then the hardcopy file name 
;                          created is returned in this.  If non-null on
;                          entry then that file name is used for the output.
;               QUIET    - if present, no information is output to the screen
;
; Calls       : get_cdhs_series, get_cdhs_raster, show_raster, show_study
;
; Common      : None
;               
; Restrictions: None
;               
; Side effects: None
;               
; Category    : Telemetry, planning
;               
; Prev. Hist. : None
;
; Written     : C D Pike, RAL, 9-Jun-95
;               
; Modified    : Added message when output KEEPed.  CDP, 27-Jul-95
;
; Version     : Version 2, 27-Jul-95
;-            

pro show_cdhs, raster=raster, series=series, file=file, keep=keep, $
               quiet=quiet,   hardcopy=hardcopy

;
;  OS?
;
vms=(!version.os eq 'VMS')


;                     
;  to terminal?
;
if keyword_set(quiet) then to_screen = 0 else to_screen = 1
if to_screen then openw, screen, filepath(/terminal), /more, /get_lun

;
;  was it a raster ID...?
;
case 1 of 
   keyword_set(raster): begin
        r = get_cdhs_raster(raster)
        if r.id ge 0 then begin
           if not keyword_set(file) then begin
              hfile = 'cdhs_raster_'+strtrim(string(raster,form='(i3)'),2)
              if vms then home='sys$login' else home=getenv('HOME')
              hfile=concat_dir(home,hfile)
           endif else begin
              hfile = file
           endelse
      
           show_raster, r.id, r.var, /keep, /quiet, file=hfile
           cdhs_text = [' ','          Translation of CDHS raster ID #'+$
                        strtrim(string(raster,form='(i3)'),2),$
                        '          ----------------------------------']
           text = rd_ascii(hfile)
           text = [cdhs_text,text(3:*)]
           if to_screen then print_str,text
     
           if keyword_set(file) or keyword_set(keep) $
                                or keyword_set(hardcopy) then begin
              print_str, text, file=hfile, /quiet
           endif
        endif else begin
           print,'No CDHS ID exists'
           return
        endelse
   end 

;
;  ... or a series ID...?
;
   keyword_set(series): begin
        r = get_cdhs_series(series)
        if r.id ge 0 then begin
           if not keyword_set(file) then begin
              hfile = 'cdhs_series_'+strtrim(string(series,form='(i3)'),2)
              if vms then home='sys$login' else home=getenv('HOME')
              hfile=concat_dir(home,hfile)
           endif else begin
              hfile = file
           endelse
      
           show_study, r.id, r.var, /keep, /quiet, file=hfile
           cdhs_text = [' ','          Translation of CDHS series ID #'+$
                        strtrim(string(series,form='(i3)'),2),$
                        '          ----------------------------------',' ']
           text = rd_ascii(hfile)
           text = [cdhs_text,text]
           if to_screen then print_str,text
     
           if keyword_set(file) or keyword_set(keep) $
                                or keyword_set(hardcopy) then begin
              print_str, text, file=hfile, /quiet
           endif
        endif else begin
           print,'No CDHS ID exists'
           return
        endelse
   end

;
;  was it neither and therefore a summary is required..?
;  Use the same code for series and rasters therefore just loop twice
;  with a few changes en route. ie sr = 0 means series and sr=1 means rasters
;
   else: begin
        for sr = 0,1 do begin
           s = {id:0}
           if sr eq 0 then series = 12 else series = 2
           sum_series = intarr(3,500) & sum_series(*,*) = -1
           while s.id ge 0 do begin
              if sr eq 0 then begin
                 s = get_cdhs_series(series)
              endif else begin
                 s = get_cdhs_raster(series)
              endelse
              if s.id ge 0 then begin
                 sum_series(1,series) = s.id
                 sum_series(2,series) = s.var
                 sum_series(0,series) = series
                 series = series + 1
              endif
           endwhile
           n = where(sum_series(0,*) gt 0)
           if n(0) ge 0 then begin
              sum_series = sum_series(*,n)
           endif else begin
              if sr eq 0 then begin
                 print,'No CDHS series found.'
              endif else begin
                 print,'No CDHS rasters found.'
              endelse
              return
           endelse
   
           if sr eq 0 then begin
              if not keyword_set(file) then begin
                 get_utc,utc
                 hfile = 'cdhs_summary_'+anytim2cal(utc,form=8)
                 if vms then home='sys$login' else home=getenv('HOME')
                 hfile=concat_dir(home,hfile)
              endif else begin
              hfile = file
           endelse
   
              openw,lun,hfile,/get_lun
           endif
           ns = n_elements(sum_series)/3
           text = ' '
           if to_screen then printf,screen,text
           printf,lun,text
           if sr eq 0 then begin
              text = [' ',' Translation of CDHS series IDs #1 - '+$
                           strtrim(string(ns,form='(i3)'),2)]
           endif else begin
              text = [' ',' Translation of CDHS raster IDs #1 - '+$
                           strtrim(string(ns,form='(i3)'),2)]
           endelse
           if to_screen then printf,screen,text
           printf,lun,text
   
           if sr eq 0 then begin
              text = ' CDHS Series   Study ID  Study Variation'
           endif else begin
              text = ' CDHS Raster   RasterID  Raster Variation'
           endelse
           if to_screen then printf,screen,text
           printf,lun,text
           text = ' -----------   --------  ---------------'
           if to_screen then printf,screen,text
           printf,lun,text
           text = ' '
           if to_screen then printf,screen,text
           printf,lun,text
   
           for i=0,ns-1 do begin
              if to_screen then printf,screen,sum_series(*,i),$
                                       form='(2x,i5,7x,i5,8x,i5)'
              printf,lun,sum_series(*,i),$
                                       form='(2x,i5,7x,i5,8x,i5)'
           endfor
        endfor
        free_lun,lun
   end
endcase

;
;  close unit
;
if to_screen then free_lun, screen
      
;
;  hardcopy required?
;
if keyword_set(hardcopy) then begin
   vms=(!version.os eq 'VMS')
   if vms then com = 'print '+hfile else com = 'lpr '+hfile
   spawn, com
   bell
   print,'Hardcopy sent to printer'
endif

;
;  Keep file?
;
if (not keyword_set(keep)) and (not  keyword_set(file)) then begin
   s = delete_file(hfile,/noconfirm)
endif else begin
   print,'Output saved in file: ',hfile
endelse

;
;  return the file name used
;
file = hfile

end
