;+
; Project     : SOHO - CDS     
;                   
; Name        : SHOW_LINELIST
;               
; Purpose     : To list the contents of the linelist database.
;               
; Explanation : Uses the linelist database access routines to present the
;               user with a listing of the requested line list(s).
;               
; Use         : IDL> show_linelist [, num_list, /hardcopy, etc.]
;    
; Inputs      : num_list   -  Integer or integer array giving ID numbers
;                             of specific line list(s) required.  
;                   or
;               line_struct - line list structure as returned by get_linelist
;
;               
; Opt. Inputs : None
;               
; Outputs     : Listing is given to screen or printer if required.
;               
; Opt. Outputs: None
;               
; Keywords    : DETECTOR   - specify detector to which line lists relate.
;               HARDCOPY   - print hardcopy of output.
;               QUIET      - no output to screen
;               KEEP       - keep output file
;               FILE       - specify output file, or retrieve file used if
;                            file = '' on entry.  If file is omitted or blank
;                            on entry, then a standard file is written to
;                            user's 'home' directory.  Otherwise the file
;                            name supplied is used unchanged.
;               SUMMARY    - summary of all lists (no details of individual
;                            lines).
;               ERRMSG     - return error in this if it is defined on entry
;
; Calls       : LIST_LINELIST
;               GET_LINELIST
;
; Common      : None
;               
; Restrictions: None
;               
; Side effects: None
;               
; Category    : Database
;               
; Prev. Hist. : None
;
; Written     : C D Pike, RAL, 14-Aug-94
;               
; Modified    : Version 1, C D Pike, RAL, 14-Aug-94
;		Version 2, William Thompson, GSFC, 16 September 1994
;			Output to screen with /MORE
;		Version 3, William Thompson, GSFC, 15 March 1995
;			Removed LL_TITLE
;               Version 4, CDP, 24-May-95.  Major revision to bring into line
;                                           with other show_xxx routines.
;               Version 5, CDP, 2-Jun-95.  Rework file name handling.
;               Version 6, CDP, 08-Aug-96. Use FIND_ALL_DIR.
;
; Version     :	Version 6, 08-Aug-96
;-            

pro show_linelist, num_list, detector=detector, hardcopy=hardcopy,$
                             quiet=quiet, summary=summary, keep=keep,$
                             errmsg=errmsg, file=file

;
;  Open a logical unit to the screen with the /MORE switch.
;
if not keyword_set(quiet) then to_screen = 1 else to_screen = 0
if to_screen then openw, screen, filepath(/TERMINAL), /MORE, /GET_LUN


;
;  was summary mode selected?
;
if n_params() eq 0 and keyword_set(summary) then begin


;
;  create filename (in home directory - unless directory specified) 
;  for hardcopy
;
   vms=(!version.os eq 'VMS')
   if not keyword_set(file) then begin
      hfile = 'line_lists'
      if vms then home='sys$login' else home=getenv('HOME')
      hfile=concat_dir(home,hfile)
   endif else begin
      hfile = file
   endelse

;
;  get basic line list details
;
   list_linelist, ll, nll
   if keyword_set(detector) then begin
      n = where(ll.detector eq strupcase(strmid(detector,0,1)))
      if n(0) ge 0 then begin
         ll = ll(n)
         nll = n_elements(ll)
      endif else begin
         if n_elements(errmsg) gt 0 then begin
            errmsg = 'No line lists found.'
         endif else begin
            print,'No line lists found.'
         endelse
         if to_screen then free_lun, screen
         return
      endelse
   endif 


;
;  where from and when?
;
   g = find_all_dir('ZDBASE')
   t = systime()

;
;  something to handle so open hardcopy file
;
   openw,lun,hfile,/get_lun

;
;  begin output, echoing to hardcopy if needed
;
   if to_screen then begin
      printf,screen,' '
      printf,screen,'Information from line list databases in: '
      for i=0,n_elements(g)-1 do printf,screen,g(i),form='(20x,a)'
      printf,screen,' '
      printf,screen,' Printed on: ' + t
      printf,screen,' '
   endif

   printf,lun,' '
   printf,lun,'Information from line list databases in: '
   for i=0,n_elements(g)-1 do printf,lun,g(i),form='(20x,a)'
   printf,lun,' '
   printf,lun,' Printed on: '+ t
   printf,lun,' '

   if to_screen then begin
      printf,screen,'  ID   Detector   # lines               Description'
   endif
   printf,lun,      '  ID   Detector   # lines               Description'

;
;  loop for each line list found
;
   if to_screen then printf,screen,' '
   printf,lun,' '
   for i= 0,nll-1 do begin
      text = string(ll(i).ll_id,ll(i).detector,ll(i).n_lines,ll(i).ll_desc,$
                    form='(i4,6x,a1,8x,i4,8x,a40)')
      if to_screen then printf,screen,text
      printf,lun,text
   endfor

;
;  print hard copy
;
   free_lun,lun
   if to_screen then free_lun,screen
   
   if keyword_set(hardcopy) then begin
      if vms then com = 'print '+hfile else com = 'lpr '+hfile
      spawn,com
      bell                        
      print,'Hard copy sent to printer'
   endif

;
;  want to keep the output?
;
   if (not keyword_set(keep)) and (not keyword_set(file)) then begin
      status = delete_file(hfile,/noconfirm)
   endif

;
;  return file name
;
   if not keyword_set(file) then file = hfile
   return
endif

;
;  wanted a specific line list
;  ---------------------------
;

;
;  if no specific linelist wanted then set up for all of them
;
if n_params() eq 0 then begin
   print,' '
   print,'Use: IDL> show_linelist, ID_array'
   print,'           or'
   print,'     IDL> show_linelist, structure'
   print,'           or'
   print,'     IDL> show_linelist, /summary'
   print,' '
   if to_screen then free_lun,screen
   return
endif

;
;  create filename (in home directory - unless directory specified) 
;  for hardcopy
;
vms=(!version.os eq 'VMS')
if not keyword_set(file) then begin
   hfile = 'line_list_details'
   if vms then home='sys$login' else home=getenv('HOME')
   hfile=concat_dir(home,hfile)
endif else begin
   hfile = file
endelse


;
;  open disk file
;
openw,lun,hfile,/get_lun,error=err
if err ne 0 then begin
   if n_elements(errmsg) eq 0 then begin
      print,'Error opening output file ',hfile
   endif else begin
      errmsg = 'Error opening output file '+ hfile
   endelse
   return
endif

;
;  check type of first parameter
;
;  was it a get_linelist structure?  If so then set up the details ready 
;  for use as if an ID was entered.
;
if datatype(num_list,1) eq 'Structure' then begin
   if n_tags(num_list) ne 4 then begin
      if n_elements(errmsg) gt 0 then begin
         errmsg = 'Invalid structure input.'
      endif else begin
         print,'Invalid structure input.'
      endelse
      if to_screen then free_lun, screen
      return
   endif else begin
      use_list = num_list.ll_id
   endelse
endif else begin
   use_list = num_list
endelse

;
;  can now proceed as if only IDs were input
;
;
;  retrieve the linelists from the database
;
list_linelist,ll,n

;
;  loop through the linelists
;

for i=0,n_elements(use_list)-1 do begin
   nuse = where(ll.ll_id eq use_list(i))
   if nuse(0) ge 0 then begin
      nuse = nuse(0)
      nlines = ll(nuse).n_lines
      if to_screen then begin
         printf,screen,' '
         printf,screen,'Line list #',ll(nuse).ll_id,' for detector ',$
                                     ll(nuse).detector,'IS',$
                               '  - ',nlines,' lines.',form='(a,i3,4a,i2,a)'
         printf,screen,'Description: ',strtrim(ll(nuse).ll_desc,2)
      endif

;
;  repeat for hardcopy
;
      printf,lun,' '
      printf,lun,'Line list #',ll(nuse).ll_id,' for detector ',$
                               ll(nuse).detector,'IS',$
                             '  - ',nlines,' lines.',form='(a,i3,4a,i2,a)'
      printf,lun,'Description: ',strtrim(ll(nuse).ll_desc,2)

;
;  retrieve details of line list
;
      get_linelist,ll(nuse).ll_id,details
      if to_screen then begin
         printf,screen,'    Wavelength          Line name      Order     Band     Pixel'
      endif
      printf,lun,'    Wavelength          Line name      Order     Band     Pixel'
      for j=0,nlines-1 do begin
         if to_screen then printf,screen,details.lines(j).wavelnth,$
                     '          (',details.lines(j).linename,')',$
                                   details.lines(j).gr_order,$
                                   details.lines(j).waveband,$
                                   details.lines(j).cen_pix,$
                                   format='(6x,f6.2,3a,5x,i1,9x,i1,6x,i4)'
         printf,lun,details.lines(j).wavelnth,$
                        '          (',details.lines(j).linename,')',$
                                      details.lines(j).gr_order,$
                                      details.lines(j).waveband,$
                                      details.lines(j).cen_pix,$
                                      format='(6x,f6.2,3a,5x,i1,9x,i1,6x,i4)'
      endfor
   endif else begin
      if to_screen then begin
         printf,screen,' '
         printf,screen,' Line list ID # ',use_list(i),' does not exist.'
         printf,screen,' '
      endif
      printf,lun,' '
      printf,lun,' Line list ID # ',use_list(i),' does not exist.'
      printf,lun,' '
      errmsg =  string(' Line list ID # ',use_list(i),' does not exist.',$
                         form='(a,i3,a)')
   endelse
endfor

;
;   Close output to terminal and file
;
if to_screen then free_lun,screen	
free_lun,lun

;
;  deal with hardcopy
;
if keyword_set(hardcopy) then begin
   if vms then com = 'print '+hfile else com = 'lpr '+hfile
   spawn,com
endif

;
;  keep file?
;
if (not keyword_set(keep)) and (not keyword_set(file)) then begin
   status = delete_file(hfile,/noconfirm)
endif else begin
   print,'Output saved in file: ',hfile
endelse

;
;  return file name
;
file = hfile

end

