;+
; Project     : SOHO - CDS     
;                   
; Name        : SHOW_RASTER
;               
; Purpose     : Show details of a raster definition.
;               
; Explanation : Sets down on paper the details contained within the
;               raster structure. Output is stored in files 'raster_list'
;               for summary data or 'raster_{raster id}_{variation id}' for
;               details of specific rasters.  All file are written to the
;               users home directory (unless the directory is specified by
;               the FILE keyword on entry). 
;               
; Use         : IDL> show_raster, ras_id, var_id
;                    or
;               IDL> show_raster, ras_struct
;    
; Inputs      : ras_id      -  fundamental raster ID number
;               var_id      -  raster variation number
;                   or
;               ras_struct  - structure returned by get_raster
;               
; Opt. Inputs : None
;               
; Outputs     : None
;               
; Opt. Outputs: None
;               
; Keywords    : QUIET     -  no output to screen
;               HARDCOPY  -  send details to printer
;               KEEP      -  keep disk file of details
;               DETECTOR  -  select from rasters of a given detector (only in
;                            conjunction with SUMMARY keyword.
;               SUMMARY   -  give a summary of all rasters.
;               FILE      -  specify particular file name for output or 
;                            receive file name used if this is '' on entry.
;                            If this keyword is omitted or is blank on entry
;                            then output is written to a standard file in the
;                            user's 'home' directory.
;               ERRMSG    -  if defined on entry, return error messages 
;                            in this variable
;
; Calls       : GET_RASTER
;
; Common      : None
;               
; Restrictions: None
;               
; Side effects: None
;               
; Category    : Planning, databases
;               
; Prev. Hist. : Derived from tp_dump_setup used in mk_raster
;
; Written     : C D Pike, RAL, 24-May-95
;               
; Modified    : Added spectrograph ID to summary listing.  CDP, 17-Jul-95
;               Added message when file KEEPed.  CDP, 27-Jul-95
;               Update for compression scheme IDs.  CDP, 27-Nov-95
;               Use FIND_ALL_DIR.                   CDP, 09-Aug-96
;               Take note of new LOW++ telemetry mode. CDP, 12-Dec-97
;
; Version     : Version 6, 12-Dec-97
;-            

pro show_raster, p1, p2, summary=summary, file=file, quiet=quiet,$
                         hardcopy=hardcopy, keep=keep, detector=detector,$
                         errmsg=errmsg

;
;  to terminal?
;
if keyword_set(quiet) then to_screen = 0 else to_screen = 1
if to_screen then openw, screen, filepath(/terminal), /more, /get_lun


;
;  was summary mode selected?
;
if n_params() eq 0 and keyword_set(summary) then begin


;
;  create filename (in home directory - unless directory specified) 
;  for hardcopy
;
   vms=(!version.os eq 'VMS')
   if not keyword_set(file) then begin
      hfile = 'raster_list'
      if vms then home='sys$login' else home=getenv('HOME')
      hfile=concat_dir(home,hfile)
   endif else begin
      hfile = file
   endelse

;
;  get fundamental raster details
;
   if keyword_set(detector) then begin
      list_f_raster, fr, n, detector=detector
   endif else begin
      list_f_raster, fr, n
   endelse

   if n gt 0 then begin

;
;  where from and when?
;
      g = find_all_dir('ZDBASE')
      t = systime()

;
;  something to handle so open screen and hardcopy files if needed
;
      if to_screen then openw,screen, filepath(/terminal), /more, /get_lun
      openw,lun,hfile,/get_lun
;
;  begin output, echoing to hardcopy if needed
;
      if to_screen then begin
         printf,screen,' '
         printf,screen,'Information from raster databases in: '
         for i=0,n_elements(g)-1 do printf,screen,g(i),form='(20x,a)'
         printf,screen,' '
         printf,screen,' Printed on: ' + t
         printf,screen,' '
      endif

      printf,lun,' '
      printf,lun,'Information from raster databases in: '
      for i=0,n_elements(g)-1 do printf,lun,g(i),form='(20x,a)'
      printf,lun,' '
      printf,lun,' Printed on: '+ t
      printf,lun,' '

      if to_screen then printf,screen,'  ID              Description/Variations'
      printf,lun,'  ID              Description/Variations'
;
;  loop for each fundamental raster found
;
      for i= 0,n_elements(fr)-1 do begin
         if to_screen then printf,screen,' '
         printf,lun,' '
         id = strtrim(string(fr(i).ras_id,form='(i4)'),2)+' '+$
                      fmt_vect([fr(i).detector])
         if to_screen then printf,screen,'  #'+id+'   '+fr(i).ras_desc
         printf,lun,'  #'+id+'   '+fr(i).ras_desc
         list_v_raster,fr(i).ras_id,rv,nv
;
;  list any variations found
; 
        if nv gt 0 then begin
            for j=0,n_elements(rv)-1 do begin
               id = strtrim(string(rv(j).ras_var,form='(i4)'),2)+'>'
               if to_screen then printf,screen,'                      <v'+id+$
                                               '   '+rv(j).rv_desc
               printf,lun,'                      <v'+id+'   '+rv(j).rv_desc
            endfor
         endif
      endfor
   endif

;
;  print hard copy
;
   free_lun,lun
   if to_screen then free_lun,screen

   if keyword_set(hardcopy) then begin
      if vms then com = 'print '+hfile else com = 'lpr '+hfile
      spawn,com
      bell                        
      print,'Hard copy sent to printer'
   endif

;
;  wanna keep the output?
;
   if (not keyword_set(keep)) and (not keyword_set(file)) then begin
      status = delete_file(hfile,/noconfirm)
   endif

;
;  return file name
;
   if not keyword_set(file) then file = hfile
   return
endif

                                                          
;
;  specific details on particular raster/variation required
;  --------------------------------------------------------

;
;  check input
;
if n_params() eq 0 then begin
   print,'Use: IDL> show_raster, id, var_id [,/keywords]'
   print,'       or'
   print,'Use: IDL> show_raster, raster_struct'
   return
endif


;
;  check input parameters
;
if n_params() eq 1 and (datatype(p1,1) ne 'Structure') then begin
   print,'Single parameter must be raster definition structure.'
   return
endif

if n_params() eq 2 then begin
   if datatype(p1,1) eq 'Integer' and datatype(p2,1) eq 'Integer' then begin
      get_raster,p1,p2,str
      if str.ras_id eq -1 then begin
         if n_elements(errmsg) gt 0 then begin
            errmsg = 'No raster/variation with that ID found.'
         endif else begin
            print,'No raster/variation with that ID found.'
         endelse
         return
      endif
   endif else begin
      print,'Two parameters must be integer IDs'
      return
   endelse
endif else begin
   str = p1
endelse

;
;  create filename (in home directory - unless directory specified) 
;  for hardcopy 
;
rid = strtrim(string(str.ras_id,form='(i5)'),2)
vid = strtrim(string(str.ras_var,form='(i5)'),2)
vms=(!version.os eq 'VMS')
if not keyword_set(file) then begin
   hfile = 'raster_'+rid+'_'+vid
   if vms then home='sys$login' else home=getenv('HOME')
   hfile=concat_dir(home,hfile)
endif else begin
   hfile = file
endelse

;
;  open the disk file
;
openw,lun,hfile,/get_lun,error=err
if err ne 0 then begin
   if n_elements(errmsg) eq 0 then begin
      print,'Error opening show file ',hfile
   endif else begin
      errmsg = 'Error opening show file '+hfile
   endelse
   return
endif



;
;  header first
;
printf,lun,' '
printf,lun,'             CDS raster definition for '+str.detector+'IS'
printf,lun,'             -----------------------------'
printf,lun,' '

if to_screen then begin
   printf,screen,' '
   printf,screen,'             CDS raster definition for '+str.detector+'IS'
   printf,screen,'             -----------------------------'
   printf,screen,' '
endif

;
;  Details of FUNDAMENTAL/VARIATION raster
;  =======================================
;
slit = ['(2 x 2)','(4 x 4)','(8 x 51)','(2 x 240)','(4 x 240)','(90 x 240)']
if to_screen then printf,screen,'FUNDAMENTAL raster ID: #',rid,'  {',$
                         strtrim(str.ras_desc,2),'}' 
printf,lun,'FUNDAMENTAL raster ID: #',rid,'  {',$
            strtrim(str.ras_desc,2),'}' 

if str.slit_num gt 0 then begin
if to_screen then    printf,screen,string('Slit: #',str.slit_num,' - ',$
                            slit(str.slit_num-1),' arcsecs',form='(a,i1,3a)')
   printf,lun,string('Slit: #',str.slit_num,' - ',$
              slit(str.slit_num-1),' arcsecs',form='(a,i1,3a)')
endif else begin
   if to_screen then printf,screen,'Slit: None defined'
   printf,lun,'Slit: None defined'
endelse

if to_screen then printf,screen,string('Size of mirror steps: ',str.xstep,$
                                    ' arcsecs',form='(a,i3,a)')
printf,lun,string('Size of mirror steps: ',str.xstep,$
                  ' arcsecs',form='(a,i3,a)')

if str.detector eq 'G' then begin
   if to_screen then printf,screen,string('Size of slit steps:   ',str.ystep,$
                                       ' arcsecs',form='(a,i3,a)')
   printf,lun,string('Size of slit steps:   ',str.ystep,$
                     ' arcsecs',form='(a,i3,a)')
endif

if to_screen then printf,screen,string('Number of mirror locations: ',str.nx,$
                                     form='(a,i3)')
printf,lun,string('Number of mirror locations: ',str.nx,$
                   form='(a,i3)')

if str.detector eq 'G' then begin
   if to_screen then printf,screen,string('Number of slit locations:   ',str.ny,$
                                        form='(a,i3)')
   printf,lun,string('Number of slit locations:   ',str.ny,$
                   form='(a,i3)')
endif

;
; Raster Variation details
; ------------------------
;

if to_screen then printf,screen,'  '
printf,lun,'  '
if to_screen then printf,screen,'Raster VARIATION ID: #',vid,'  {',$
                              strtrim(str.rv_desc,2),'}' 
printf,lun,'Raster VARIATION ID: #',vid,'  {',$
            strtrim(str.rv_desc,2),'}' 


et = string(str.exptime,form='(f7.1)')
if to_screen then printf,screen,'Exposure time:      '+strtrim(et,2)+' secs'
printf,lun,'Exposure time:      '+strtrim(et,2)+' secs'


optnum = ['0','8','16','32','64','126','256','512']
scheme = [' ','Straight copy','Truncate to 12 bits','VBWL',' ',' ',$
              'Sum over lines','Sum over windows','USER defined']

if str.comp_id gt 0 then begin
   text = 'Compression scheme: '+scheme(str.comp_id)
   if str.comp_id eq 3 then begin
      if str.comp_opt eq 0 then  begin
         text = text + '  Default option'
      endif else begin
         text = text + '  option: Blocksize ' + optnum(str.comp_opt)
      endelse
   endif
   if str.comp_id eq 6 then begin
      if str.vds_orient eq 0 then  begin
         text = 'Compression scheme: Sum over wavelength'
      endif else begin
         text = 'Compression scheme: Sum over spatial dimension'
      endelse
   endif
endif else begin
   text = 'Compression scheme: Not defined'
endelse
if to_screen then printf,screen,text
printf,lun,text

orient = ['Row','Column']
map = [' ',' ','Normal','Accumulate']
if str.detector eq 'N' then begin
   if to_screen then printf,screen,'VDS readout orientation: ',$
               orient(str.vds_orient)
   if to_screen then printf,screen,'VDS mapping mode: ',map(str.vds_map)
   printf,lun,'VDS readout orientation: ',$
               orient(str.vds_orient)
   printf,lun,'VDS mapping mode: ',map(str.vds_map)
endif

case str.tel_rate of
   'L': tlm = 'Low'
   'P': tlm = 'Low++'
   'M': tlm = 'Medium'
   'H': tlm = 'High'
  else: tlm = 'Medium'
endcase
if to_screen then printf,screen,'Telemetry required: ',tlm
printf,lun,'Telemetry required: ',tlm


dur = string(str.duration,form='(f9.1)')
if to_screen then printf,screen,'Raster duration:    ', + strtrim(dur,2) + ' seconds'
if to_screen then printf,screen,'Raster usable?:     ',str.usable
printf,lun,'Raster duration:    ', + strtrim(dur,2) + ' seconds'
printf,lun,'Raster usable?:     ',str.usable



;
;  close file(s) 
;
free_lun, lun
if to_screen then free_lun, screen

;
; hardcopy?
;
if keyword_set(hardcopy) then begin
   if vms then com = 'print '+hfile else com = 'lpr '+hfile
   spawn,com
   print,'Hardcopy sent to printer.'
endif

;
;  keep file?
;
if (not keyword_set(keep)) and (not keyword_set(file)) then begin
   status = delete_file(hfile,/noconfirm)
endif else begin
   print,'Output saved in file: ',hfile
endelse

;
;  return file name
;
if not keyword_set(file) then file = hfile

end
