;+
; Project     : SOHO - CDS     
;                   
; Name        : SHOW_STUDY
;               
; Purpose     : Produce a listing of a CDS study definition.
;               
; Explanation : For those who like to read paper, this routine displays
;               the salient points in the definition of a CDS study.
;               
; Use         : IDL> show_study, str    
;                          or
;               IDL> show_study, study_id, study_var_id
;
;    
; Inputs      : str          -  a valid CDS study definition structure
;                               as returned by get_study.
;                   OR
;
;               study_id     -  the study ID in the current (ie pointed to
;                               by env. var. ZDBASE) database
;               study_var_id -  the study variation ID
;
;               
; Opt. Inputs : None
;               
; Outputs     : If required, sends to file and prints and outputs to screen.
;               
; Opt. Outputs: None
;               
; Keywords    : KEEP    -  keep output file.  Default ile name is 
;                          study_ss_vv.dmp  where ss and vv are the ID numbers
;                          for study-specific output or study_list for 
;                          summary information (both in $HOME directory).
;               QUIET   -  no information to screen
;               ERRMSG  -  if defined on entry will be returned with any
;                          error messages.
;               FILE    -  specify file to be written or if blank on entry
;                          file name written is returned.  If omitted or
;                          blank on entry then data are written to a file
;                          in users 'home' directory.
;               SUMMARY  - give summary of all entries in study database.
;               HARDCOPY - send output to printer.
;
; Calls       : get_study
;
; Common      : None
;               
; Restrictions: None
;               
; Side effects: None
;               
; Category    : Planning, databases
;               
; Prev. Hist. : Based on original show_study and dump_study
;
; Written     : C D Pike, RAL, 23-May-95
;               
; Modified    : Fix bug not returning file name.  CDP, 2-Jun-95
;               Include raster duration.          CDP, 9-Jun-95
;               Added messgae when output KEEPed. CDP, 27-Jul-95
;               Fixed calculation of study duration to take account
;               of n_repeat_r=1 meaning just do it once.  CDP, 13-Oct-95
;               Refix it.                                 CDP, 10-Nov-95
;               Trap failure to find raster ID            CDP, 22-Nov-95
;               Trap cranky zone_ids.                     CDP, 18-Jan-96
;               Trap no rasters.                          CDP, 25-Jan-96
;               Add detector to raster output.            CDP, 26-Feb-96
;               Use FIND_ALL_DIR.                         CDP, 08-Aug-96
;               Sort on study ID in summary mode.         CDP, 23-Sep-97
;
; Version     : Version 12, 23-Sep-97
;-            

pro show_study, p1, p2, quiet=quiet, keep=keep, hardcopy=hardcopy, $
                        file=file, errmsg=errmsg, summary=summary
                        

;
;  to terminal?
;
if keyword_set(quiet) then to_screen = 0 else to_screen = 1
if to_screen then openw, screen, filepath(/terminal), /more, /get_lun


;
;  immediately check if want summary mode (ie no parameters supplied)
;
if n_params() eq 0 and keyword_set(summary) then begin

;
;  create filename (in home directory) for hardcopy or use user-supplied
;
   vms=(!version.os eq 'VMS')
   if not keyword_set(file) then begin
      hfile = 'study_list'
      if vms then home='sys$login' else home=getenv('HOME')
      hfile=concat_dir(home,hfile)
   endif else begin
      hfile = file
   endelse


;
;  get fundamental study details
;
   list_f_study, study, n


   if n gt 0 then begin

;
;  sort by study ID rather than title ID
;
      sid = study.study_id
      ns = sort(sid)
      study = study(ns)

;
;  where from and when?
;
      g = find_all_dir('ZDBASE')
      
      t = systime()

;
;  something to handle so open screen and hardcopy files if needed
;
      openw,screen, filepath(/terminal), /more, /get_lun
      openw,lun,hfile,/get_lun

;
;  begin output, echoing to hardcopy if needed
;
      if to_screen then begin
         printf,screen,' Information from study databases in: '
         for i=0,n_elements(g)-1 do printf,screen,'   ',g(i)
         printf,screen,' '
         printf,screen,' Printed on: ' + t
         printf,screen,' '
      endif

      printf,lun,' Information from study databases in: '
      for i=0,n_elements(g)-1 do printf,lun,'   ',g(i)
      printf,lun,' '
      printf,lun,' Printed on: '+ t
      printf,lun,' '

      if to_screen then printf,screen,$
         'Study ID    Name            Study Title                       Variations'
      printf,lun,$
         'Study ID    Name            Study Title                       Variations'

;
;  loop for each fundamental study found
;
      for i= 0,n_elements(study)-1 do begin
         if to_screen then printf,screen,' '
         printf,lun,' '
         prog = strpad(strtrim(study(i).obs_prog,2),10,/after)
         id = strtrim(string(study(i).study_id,prog,$
                          form='(i4,6x,a10)'),2)
         if to_screen then printf,screen,'  #'+id+'        '+study(i).title
         printf,lun,'  #'+id+'        '+study(i).title
         find_study,sv,study=study(i).study_id
;
;  list any variations found
; 
        if n_elements(sv) gt 0 then begin
            for j=0,n_elements(sv)-1 do begin
               if sv(j).title_id eq study(i).title_id then begin
                  id = strtrim(string(sv(j).studyvar,form='(i4)'),2)
                  if to_screen then begin
                     printf,screen,strtrim('v'+id+'   '+sv(j).sv_desc),form='(50x,a)'
                  endif
                  printf,lun,strtrim('v'+id+'   '+sv(j).sv_desc),form='(50x,a)'
               endif
            endfor
         endif
      endfor
   endif

;
;  release units
;
   free_lun,lun
   if to_screen then free_lun,screen

;
;  print hard copy
;
   if keyword_set(hardcopy) then begin
      if vms then com = 'print '+hfile else com = 'lpr '+hfile
      spawn,com
      bell
      print,'Hard copy sent to printer'
   endif
   if (not keyword_set(keep)) and (not keyword_set(file)) then begin
      status = delete_file(hfile,/noconfirm)
   endif
;
;  return file name
;
   if not keyword_set(file) then file = hfile
   return
endif

;
;  get details of a particular study
;  ---------------------------------
;

;
;  check form of input
;
if n_params() eq 0 then begin
   print,'Use: IDL> show_study, study_structure'
   print,'   or'
   print,'     IDL> show_study, study_ID, study_var_ID'
   print,' '
   return
endif

if n_params() eq 1 and (datatype(p1,1) ne 'Structure') then begin
   print,'Single parameter must be study definition structure.'
   return
endif

if n_params() eq 2 then begin
   if datatype(p1,1) eq 'Integer' and datatype(p2,1) eq 'Integer' then begin
      get_study,p1,p2,str
      if str.study_id eq -1 then begin
         if n_elements(errmsg) gt 0 then begin
            errmsg = 'No study/variation with that ID found.'
         endif else begin
            print,'No study/variation with that ID found.'
         endelse
         return
      endif
   endif else begin
      print,'Two parameters must be integer IDs'
      return
   endelse
endif else begin
   str = p1
endelse



;
;  have the structure now so display as needed.
;
sid = str.study_id
vid = str.studyvar
tid = str.title_id

;
;  create filename (in home directory) for hardcopy or use user-supplied
;
vms=(!version.os eq 'VMS')
if not keyword_set(file) then begin
   hfile = 'study_'+strtrim(string(sid,form='(i3)'),2)+'_'+$
                   strtrim(string(vid,form='(i3)'),2)
   if vms then home='sys$login' else home=getenv('HOME')
   hfile=concat_dir(home,hfile)
endif else begin
   hfile = file
endelse



openw,lun,hfile,/get_lun,error=err
if err ne 0 then begin
   if n_elements(errmsg) eq 0 then begin
      print,'Error opening show file ',hfile
   endif else begin
      errmsg = 'Error opening show file '+hfile
   endelse
   return
endif


;
;  pretty format
;
printf,lun,' '
if to_screen then printf,screen,' '

printf,lun,'Study Title: ',strmid(str.title,0,65)
if to_screen then printf,screen,'Study Title: ',strmid(str.title,0,65)

printf,lun,'Obs prog:    ',str.obs_prog
if to_screen then printf,screen,'Obs prog:    ',str.obs_prog

case str.category of
   'S': cat = '    Science'
   'C': cat = '    Calibration'
   'T': cat = '    Test'
 else : cat = '    Unknown'
endcase

;
;  category and zone
;

printf,lun,'Category:',cat,format='(a9,a15)'
if to_screen then printf,screen,'Category:',cat,format='(a9,a15)'


zone_type = ['','CORONA','LIMB','CORONAL HOLE','QUIET','ACTIVE','FLARE',$
             'GIS_FIL1_NEG','GIS_FIL1_POS','GIS_FIL2_NEG','GIS_FIL2_POS']

if str.zone_id le 10 then text = zone_type(str.zone_id) else text = 'Unknown'
printf,lun,'Zone ID :  ',str.zone_id,' (',text,')',$
                  format='(a11,i1,a2,a,a1)'
if to_screen then printf,screen,'Zone ID :  ',$
                                str.zone_id,' (',zone_type(str.zone_id),')',$
                                format='(a11,i1,a2,a,a1)'

;
;  study, variation and title IDs
;
csid = strtrim(string(str.study_id,form='(i3)'),2)
svid = strtrim(string(str.studyvar,form='(i3)'),2)
tid  = strtrim(string(str.title_id,form='(i3)'),2)

printf,lun,'Study ID #'+csid,'Variation ID #'+svid,'Title ID #'+tid,$
           format='(a,5x,a,5x,a)'
if to_screen then printf,screen,'Study ID #'+csid,$
                                'Variation ID #'+svid,'Title ID #'+tid,$
                                 format='(a,5x,a,5x,a)'

;
;  detail each raster
;
nr = str.n_raster_def

if nr gt 0 then begin
   printf,lun,' '
   if to_screen then printf,screen,' '
   printf,lun,'Number of rasters: ',nr,form='(a,i2)'
   if to_screen then printf,screen,'Number of rasters: ',nr,form='(a,i2)'
   
   printf,lun,' '
   if to_screen then printf,screen,' '
   
   title = '  Raster      ID    Var. ID    Pointing   Xpos   Ypos    Duration  Repeat'
   printf,lun,title
   if to_screen then printf,screen,title
   printf,lun,' '
   if to_screen then printf,screen,' '
   
   pc = ['Offset', 'Fixed', 'Deferred']
   for i=0,nr-1 do begin
      get_raster,str.rasters(i).ras_id,str.rasters(i).ras_var,rast
      if rast.ras_id gt -1 then begin 
         rasn = trim(i+1)+'('+rast.detector+')'
         printf,lun,rasn,str.rasters(i).ras_id,str.rasters(i).ras_var,$
                        pc(str.rasters(i).pointing+1), fix(str.rasters(i).ins_x),$
                        fix(str.rasters(i).ins_y), rast.duration,$  
                        str.rasters(i).n_repeat_r,$
                        format='(3x,a5,5x,i3,5x,i3,5x,a10,2i7,5x,i5,5x,i3)'
         if to_screen then begin
            printf,screen,rasn,str.rasters(i).ras_id,str.rasters(i).ras_var,$
                              pc(str.rasters(i).pointing+1), $
                              fix(str.rasters(i).ins_x),$
                              fix(str.rasters(i).ins_y), rast.duration,$
                              str.rasters(i).n_repeat_r,$
                              format='(3x,a5,5x,i3,5x,i3,5x,a10,2i7,5x,i5,5x,i3)'
         endif
      endif else begin
         printf,lun,i+1,'No such ID and variation'
         if to_screen then printf,screen,i+1,'No such ID and variation'
      endelse
   endfor
   
   printf,lun,' '
   if to_screen then printf,screen,' '
   
   nras = n_elements(str.rasters)
   dur_sec = long(str.duration0+str.duration1*(str.rasters(nr-1).n_repeat_r))
   ut = {mjd:0L, time:dur_sec*1000L}
   hms = ' ('+strmid(utc2str(ut,/time,/trun),0,8)+')'
   cdur = strtrim(string(dur_sec,$
                  form='(i8)'),2)+' seconds.'
   printf,lun,'Duration of study as defined above: '+cdur+hms
   if to_screen then printf,screen,'Duration of study as defined above: '+cdur+hms
endif else begin
   if to_screen then printf,screen,' '
   if to_screen then printf,screen,'No rasters defined for this study.'
   printf,lun,' '
   printf,lun,'No rasters defined for this study.'
endelse

;
;  tidy up, print file
;
if to_screen then free_lun, screen
free_lun, lun
if vms then com = 'print '+hfile else com= 'lpr '+hfile
if keyword_set(hardcopy) then spawn,com

if (not keyword_set(keep)) and (not keyword_set(file)) then begin
   status = delete_file(hfile,/noconfirm)
endif else begin
   print,'Output saved in file: ',hfile
endelse

;
; return file variable in case needed
;
if not keyword_set(file) then file = hfile

end
