;+
; Project     : SOHO - CDS     
;                   
; Name        : UPDATE_DEX()
;               
; Purpose     : Updates CDS data extraction windows for current setup
;               
; Explanation : The CDS data extraction windows depend not only on the 
;               wavelength specified by the planner of the raster but also
;               upon the current wavelength calibration and setup (eg VDS
;               offsets) of the spectrographs.  Since these may have changed
;               since the original planning was performed, this routine is
;               called by CPT to update the extraction windows immediately
;               prior to uplinking the commands.  Note that the datawin
;               database (And mk-raster which writes it) use the convention
;               that NIS1 uses CCD rows 0-511 while NIS2 uses CCD rows 512-1023
;               That is the reverse of real operations and so this routine
;               automatically performs the switch required.  Use of the 
;               routine plot_dex on a raw structure returned by get_datawin
;               will show the wrong orientation so always use...
;
;               IDL> get_datawin, ID, dw
;               IDL> plot_dex, update_dex(dw)
;
;               
; Use         : IDL> new_dex = update(old_dex,version=version)
;    
; Inputs      : old_dex  - the datawin structure as returned by GET_DATAWIN
;               
; Opt. Inputs : None
;               
; Outputs     : Function returns same structure as input with window definition
;               values updated.
;
; Opt. Outputs: None               
;
; Keywords    : VERSION - returns the version number of the calibration
;                         parameters.
;
; Calls       : 
;
; Common      : None
;               
; Restrictions: None
;               
; Side effects: None
;               
; Category    : Planning, command
;               
; Prev. Hist. : None
;
; Written     : C D Pike, RAL, 6-Feb-96
;               
; Modified    : Added offsets from STATE db.  CDP, 16-Feb-96
;               Cut out use of w_height tag so it will handle 
;               IEF window structure as well.   CDP, 20-Feb-96
;               Trap new windows going beyond upper pixel limit. CDP, 23-Feb-96
;               Sort out NIS1/2 and use NIS_HT parameter. CDP, 1-Mar-96
;               Updated to use new wavecal system.  CDP, 14-Mar-96
;               Use X centre of window to calculate Y location,
;               and fix grating order blindness. CDP, 20-Mar-96
;               Allow NIS default extraction to have a window height.
;                                                CDP, 28-Mar-96
;               Fix bug associated with second order lines.  CDP, 25-Mar-97
;		Version 10, 07-Dec-1998, William Thompson, GSFC
;			Round off vertical positions on window, instead of
;			simply rounding off.
;
; Version     : Version 10, 07-Dec-1998
;-
function update_dex, old_str, gset_id=gset_id, version=version

;
;  If there is no NIS wavecal loaded, the first call to wave2pix will load
;  the default one, so no need to bother here.  The GIS wavecal is specific
;  to the GSET so store any current GIS wavecal and restore it at the end
;  so that the specific wavecal for the gset_id can be used here.
;
if old_str.detector eq 'G' then begin
   if n_elements(gset_id) eq 0 then begin
      bell
      print,'UPDATE_DEX: Error, no GSET_ID specified.'
      return, old_str
   endif
   g_wcal = save_wavecal('g')
   if not load_wavecal('g',gset=gset_id) then begin
      bell,2
      print,'*** Warning default GIS wavelength calibration used. ***'
      if not load_wavecal('g',gset=!def_gset_id) then begin
         bell,3
         print,'*** Error loading default GIS wavecal - windows not updated.'
         s = load_wavecal('g',cal_struct=g_wcal)
         return, old_str
      endif
   endif
endif


;
;  retrieve VDS offsets from STATE database
;
val = cp_get_entry('N1_CEN_S',0)  & nis1_cen_s=val.active 
val = cp_get_entry('N1_CEN_L',0)  & nis1_cen_l=val.active
val = cp_get_entry('N2_CEN_S',0)  & nis2_cen_s=val.active 
val = cp_get_entry('N2_CEN_L',0)  & nis2_cen_l=val.active 
val = cp_get_entry('NIS_HT',0)    & nis_ht    =val.active 

;
;  pick up the associated linelist (to check dispersion order)
;
if old_str.ll_id gt 0 then begin
   get_linelist, old_str.ll_id, linelist
endif else begin
   linelist = 0
endelse




;
;  derive scale factor from quoted maximum window height
;
scale = float(nis_ht)/120.

;
;  get the calibration (wavelength etc) version number
;
val = cp_get_entry('CAL_VERS',0)  & version=val.active 


;
;  copy old to new since most is unchanged
;
new_str = old_str

;
;  make sure there was some data
;
n = where(old_str.wins.win_name ne '', count)
if count eq 0 then goto, finished

;
;  only want to reread Backgroud definitions once
;
have_back = 0

;
;  loop over each window definition
;
for i=0, count-1 do begin

   old_str.wins(i).win_name = strtrim(old_str.wins(i).win_name,2)
   case strmid(old_str.wins(i).win_name,0,3) of
    'All': goto, finished

    'BB_': begin
            height = old_str.wins(i).win_def(3)
            case old_str.wins(i).win_name of
             'BB_NIS_2': begin
                            if height eq 120 then begin
                               new_str.wins(i).win_def(1) = $
                                   nis2_cen_s - (nis_ht-1)/2
                               new_str.wins(i).win_def(3) = $
                                   (nis2_cen_l + (nis_ht-1)/2) - $
                                    new_str.wins(i).win_def(1) + 1  
                            endif else begin
                               mid = (nis2_cen_s+nis2_cen_l)/2
                               new_str.wins(i).win_def(1) = $
                                      mid - fix(height*scale)/2
                               new_str.wins(i).win_def(3) = fix(height*scale)
                            endelse
                         end
             'BB_NIS_1': begin
                            if height eq 120 then begin
                               new_str.wins(i).win_def(1) = $
                                   nis1_cen_s - (nis_ht-1)/2
                               new_str.wins(i).win_def(3) = $
                                   (nis1_cen_l + (nis_ht-1)/2) - $
                                   new_str.wins(i).win_def(1) + 1  
                            endif else begin
                               mid = (nis1_cen_s+nis1_cen_l)/2
                               new_str.wins(i).win_def(1) = $
                                      mid - fix(height*scale)/2
                               new_str.wins(i).win_def(3) = fix(height*scale)
                            endelse
                         end
                   else:
            endcase
           end

    'Bac': begin
              if not have_back then begin
                 back = cp_get_entry('VDS_BACK',indgen(4,4))

                 if n_elements(back) eq 16 then begin
                    back = back.active
                 endif else begin
                    print,'*** Error reading background definition.'
                 endelse
                 nb = where(strtrim(old_str.wins.win_name,2) eq $
                                                         'Background', cb)
                 if cb ne 4 then begin
                    print,'*** Error: Only ',cb,' background windows'
                 endif else begin
                    new_str.wins(nb).win_def = back
                 endelse
                 have_back = 1
              endif
           end

    'WW_': begin
              w = str2arr(old_str.wins(i).win_name, delim='_')
              if n_elements(w) ne 3 then begin
                 print,'*** Error: inconsistent window name - '+$
                       old_str.wins(i).win_name
                 w = 0.0
              endif else begin
                 w = float(w(1)) + float(w(2))/100.0
              endelse

;
;  check the grating order for this window
;
              if datatype(linelist,1) eq 'Structure' then begin
                 ng = where(abs(linelist.lines.wavelnth - w) lt 0.02, count)
                 if count gt 0 then begin
                    ng = ng(0)
                    w = w * linelist.lines(ng).gr_order
                 endif
              endif




              case old_str.detector of
                 'N': begin
                         if w lt 500.0 then begin
                            cpix = wave2pix('N1',w,/limit)
                         endif else begin
                            cpix = wave2pix('N2',w,/limit)
                         endelse
                      end
                 'G': begin
                         iband = which_gis_band(w)
                         band  = trim(iband)
                         cpix = 2047 - wave2pix('G'+band,w,/limit)
                         cpix = cpix + ((iband-1)*2048)
                      end
              endcase
              new_str.wins(i).win_def(0) = $
                                 (cpix - old_str.wins(i).win_def(2)/2.0) > 0
;
;  check the new window doesn't extend beyond the upper pixel limit
;               
              if old_str.detector eq 'N' then begin
                 if new_str.wins(i).win_def(0)+$
                          new_str.wins(i).win_def(2) gt 1023 then begin
                    new_str.wins(i).win_def(0) = 1024 - new_str.wins(i).win_def(2)
                 endif
              endif else begin
                 band = which_gis_band(w)
                 maxpix = (band-1)*2048 + 2047
                 if new_str.wins(i).win_def(0)+$
                          new_str.wins(i).win_def(2) gt maxpix then begin
                    new_str.wins(i).win_def(0) = maxpix+1-new_str.wins(i).win_def(2)
                 endif
              endelse   
;
;  now adjust the height and ystart parameter for NIS sloping spectra
;
              if old_str.detector eq 'N' then begin
                 height = old_str.wins(i).win_def(3)
                 if w lt 500.0 then begin
                    xpix = new_str.wins(i).win_def(0)+$
                               new_str.wins(i).win_def(2)/2.0
                    ylen = fix(height * scale)
                    ypix = fix(round((nis1_cen_l - nis1_cen_s)/1023.*xpix +$
                               nis1_cen_s))
                    new_str.wins(i).win_def(1) = ypix-(ylen/2)
                    new_str.wins(i).win_def(3) = ylen
                 endif else begin
                    xpix = new_str.wins(i).win_def(0)+$
                               new_str.wins(i).win_def(2)/2.0
                    ylen = fix(height * scale)
                    ypix = fix(round((nis2_cen_l - nis2_cen_s)/1023.*xpix +$
                               nis2_cen_s))
                    new_str.wins(i).win_def(1) = ypix-(ylen/2)
                    new_str.wins(i).win_def(3) = ylen
                 endelse
              endif
           end
; 
;  if not one of them it must be a real spectral line window
;
     else: begin
               w = str2arr(old_str.wins(i).win_name, delim='_')
               if n_elements(w) ne 4 then begin
                  print,'*** Error: inconsistent window name - '+$
                        old_str.wins(i).win_name
                  w = 0.0
               endif else begin
                  w = float(w(2)) + float(w(3))/100.0
               endelse

;
;  check the grating order for this window
;
              if datatype(linelist,1) eq 'Structure' then begin
                 ng = where(abs(linelist.lines.wavelnth - w) lt 0.02, count)
                 if count gt 0 then begin
                    ng = ng(0)
                    w = w * linelist.lines(ng).gr_order
                 endif
              endif

               case old_str.detector of
                  'N': begin
                          if w lt 500.0 then begin
                             cpix = wave2pix('N1',w,/limit)
                          endif else begin
                             cpix = wave2pix('N2',w,/limit)
                          endelse
                       end
                  'G': begin
                         iband = which_gis_band(w)
                         band  = trim(iband)
                         cpix = 2047 - wave2pix('G'+band,w,/limit)
                         cpix = cpix + ((iband-1)*2048)
                       end
               endcase
               new_str.wins(i).win_def(0) = $
                        (cpix - (old_str.wins(i).win_def(2)/2)) > 0
; 
; check the new window doesn't extend beyond the upper pixel limit
;               
              if old_str.detector eq 'N' then begin
                 if new_str.wins(i).win_def(0)+$
                          new_str.wins(i).win_def(2) gt 1023 then begin
                    new_str.wins(i).win_def(0) = 1024 - new_str.wins(i).win_def(2)
                 endif
              endif else begin
                 band = which_gis_band(w)
                 maxpix = (band-1)*2048 + 2047
                 if new_str.wins(i).win_def(0)+$
                          new_str.wins(i).win_def(2) gt maxpix then begin
                    new_str.wins(i).win_def(0) = maxpix+1-new_str.wins(i).win_def(2)
                 endif
              endelse   
;  
;  now adjust the height and ystart parameter for NIS sloping spectra
;
               if old_str.detector eq 'N' then begin
                  height = old_str.wins(i).win_def(3)
                  if w lt 500.0 then begin
                     xpix = new_str.wins(i).win_def(0)+$
                                new_str.wins(i).win_def(2)/2.0
                     ylen = fix(height * scale)
                     ypix = fix(round((nis1_cen_l - nis1_cen_s)/1023.*xpix +$
                                nis1_cen_s))
                     new_str.wins(i).win_def(1) = ypix-(ylen/2)
                     new_str.wins(i).win_def(3) = ylen
                  endif else begin
                     xpix = new_str.wins(i).win_def(0)+$
                                new_str.wins(i).win_def(2)/2.0
                     ylen = fix(height * scale)
                     ypix = fix(round((nis2_cen_l - nis2_cen_s)/1023.*xpix +$
                                nis2_cen_s))
                     new_str.wins(i).win_def(1) = ypix-(ylen/2)
                     new_str.wins(i).win_def(3) = ylen
                  endelse
               endif
           end
   endcase
endfor

;
;  should always get here
;   
finished:
if old_str.detector eq 'G' then begin
   if not load_wavecal('g',cal_struct=g_wcal) then begin
      bell,2
      print,'** UPDATE_DEX - Warning: unable to reset GIS wavecal.'
   endif 
endif
return, new_str

end
