;+
; Project     : SOHO - CDS     
;                   
; Name        : WHAT_USES
;               
; Purpose     : Informs of entries in planning databases
;               
; Explanation : Given one of a number of selection criteria, this routine
;               reports on  which studies or rasters perform an observation
;               with a parameter matching the search value.  Then use the 
;               show_xxx routines for more detailed information.
;               
; Use         : IDL> what_uses [+ keywords]
;
;                    eg
;
;                    what_uses,ras=10  --> list of studies using raster 10
;
;                    what_uses,ll='Synoptic Line Selection' --> returns
;                                                               raster ID using
;                                                               that line list
;
; Inputs      : None
;               
; Opt. Inputs : None
;               
; Outputs     : Info is printed to screen
;               
; Opt. Outputs: None
;               
; Keywords    : RASTER  -  specify a raster ID to retrieve which studies use
;                          that raster.
;               VARIATION - used in conjunction with raster.
;
;               MLOC    -  specify to retrieve which rasters use that number
;                          of mirror locations
;               SLOC    -  specify to retrieve which rasters use that number
;                          of slit locations
;               EXPTIME -  specify to retrieve which rasters use this 
;                          exposure time
;               ACRONYM -  specify which study has this 'acronym'
;
;               DW_ID   -  specify which rasters/varns use this datawin ID
;
;               LL_ID   -  specify which rasters/varns use this line list ID
;
;               LLDESC  -  several formats
;                       -->  '?' will return numbered list of line list
;                                descriptions            
;                       -->  '#nn' will return rasters using line list desc
;                                  number nn in the above list.
;                       -->  plain string will try to match that as a line list
;                            description
;
; Calls       : DBOPEN, DBFIND, DBEXT, DBCLOSE, LIST_LINELIST
;
; Common      : None
;               
; Restrictions: None
;               
; Side effects: None
;               
; Category    : Databases, planning
;               
; Prev. Hist. : None
;
; Written     : C D Pike, RAL, 1-Dec-95
;               
; Modified    : Add raster variation.  CDP, 19-Dec-95
;               Give study variation on 'acronym'.  CDP, 16-Jan-96
;               Add search for DW and LL IDs.       CDP, 20-Feb-96
;               Add more help when acronym is ambiguous. CDP, 28-Feb-96
;
; Version     : Version 5, 28-Feb-96
;-            

pro what_uses, raster=raster, variation=variation, mloc=mloc, sloc=sloc,$
               exptime=exptime,lldesc=lldesc, acronym=acronym, dw_id=dw_id,$
               ll_id=ll_id

;
;  check for logic
;
if keyword_set(variation) and not keyword_set(raster) then begin
   print,'Must supply raster and variation IDs or just raster ID'
   return
endif

;
;  handle each case on its own merits
;

case 1 of

;
;  What study has this acronym?
;
   keyword_set(acronym): begin
         acro = strtrim(strupcase(acronym),2)
         dbopen,'study_title'
         stext = 'obs_prog='+acro
         val = dbfind(stext,/silent)
         if val(0) gt 0 then begin
            dbext,val,['study_id','title_id','obs_prog'],sid,tid,obsprog
            n = where(strtrim(obsprog,2) eq acro,count)
            if count gt 0 then begin
               sid = sid(0)
               tid = tid(0)
               dbclose
               dbopen,'study_var'
               stext = 'study_id='+trim(sid)+',title_id='+trim(tid)
               val = dbfind(stext,/silent)
               if val(0) gt 0 then begin
                  dbext,val,['studyvar'],vid
                  vid = vid(0)
               endif else vid = 0
            endif else begin
               print,'No exact match found for <'+acronym+'>'
               dbext,val,['obs_prog'],obsp
               print,'Choose from ...'
               print_str,obsp,/num
               dbclose
               return
            endelse 
            print,'That is study ID #'+strtrim(string(sid),2)+$
                  '/'+trim(vid)+$
                  '    (with title ID = '+strtrim(string(tid),2)+')'
            dbclose
         endif else begin
            print,'Not a known acronym.'
            dbclose
            return
         endelse
   end

;
;  What studies use raster ID #n and variation #n?
;
   keyword_set(raster) and keyword_set(variation): begin
         rc = strtrim(string(raster),2)
         rv = strtrim(string(variation),2)
         dbopen,'study_var2'
         stext = 'ras_id='+rc+',ras_var='+rv
         val = dbfind(stext,/silent)
         if val(0) gt 0 then begin
            dbext,val,['study_id','studyvar'],sid,sv
            print,'Following study/varn. combinations use raster #'+rc+$
                  '/'+rv
            for i=0,n_elements(sid)-1 do begin
               print,'     '+fmt_vect([sid(i),sv(i)],delim='/',/no_par)
            endfor   
         endif
         dbclose
   end

;
;  What studies use raster ID #n?
;
   keyword_set(raster): begin
         rc = strtrim(string(raster),2)
         dbopen,'study_var2'
         stext = 'ras_id='+rc
         val = dbfind(stext,/silent)
         if val(0) gt 0 then begin
            dbext,val,['study_id','studyvar'],sid,sv
            print,'Following study/varn. combinations use raster #'+rc
            for i=0,n_elements(sid)-1 do begin
               print,'     '+fmt_vect([sid(i),sv(i)],delim='/',/no_par)
            endfor   
         endif
         dbclose
   end

;
;  What rasters use number of mirror locations = NX?
;
   keyword_set(mloc): begin
         rc = strtrim(string(mloc),2)
         dbopen,'raster'
         stext = 'nx='+rc
         val = dbfind(stext,/silent)
         if val(0) gt 0 then begin
            dbext,val,['detector','ras_id','ras_desc'],rdet,rid,rdesc
            rid= strtrim(string(rid),2)
            rdesc = strtrim(rdesc,2)
            print,'Following rasters use '+rc+' mirror locations.'
            for i=0,n_elements(rid)-1 do begin
               print,rdet(i)+'IS --> #'+rid(i)+'  <'+rdesc(i)+'>'
            endfor   
         endif
         dbclose
   end
   
;
;  What rasters use number of slit locations = NY?
;
   keyword_set(sloc): begin
         rc = strtrim(string(sloc),2)
         dbopen,'raster'
         stext = 'ny='+rc
         val = dbfind(stext,/silent)
         if val(0) gt 0 then begin
            dbext,val,['detector','ras_id','ras_desc'],rdet,rid,rdesc
            rid= strtrim(string(rid),2)
            rdesc = strtrim(rdesc,2)
            print,'Following rasters use '+rc+' slit locations.'
            for i=0,n_elements(rid)-1 do begin
               print,rdet(i)+'IS --> #'+rid(i)+'  <'+rdesc(i)+'>'
            endfor   
         endif
         dbclose
   end

;
;  What raster variations use integration time xxx secs?
;
   keyword_set(exptime): begin
         rc = strtrim(string(exptime,form='(f7.2)'),2)
         dbopen,'raster_var'
         stext = 'exptime='+rc
         val = dbfind(stext,/silent)
         if val(0) gt 0 then begin
            dbext,val,['ras_id','ras_var'],rid,rv
            print,'Following rasters/varn. use an exposure time of '+rc+' secs.'
            for i=0,n_elements(rid)-1 do begin
               print,'     '+fmt_vect([rid(i),rv(i)],delim='/',/no_par)
            endfor   
         endif
         dbclose
   end

;
;  What raster variations use datawin ID xx?
;
   keyword_set(dw_id): begin
         rc = trim(dw_id)
         dbopen,'raster_var'
         stext = 'dw_id='+rc
         val = dbfind(stext,/silent)
         if val(0) gt 0 then begin
            dbext,val,['ras_id','ras_var'],rid,rv
            print,'Following rasters/varn. use datawin ID of '+rc
            for i=0,n_elements(rid)-1 do begin
               print,'     '+fmt_vect([rid(i),rv(i)],delim='/',/no_par)
            endfor   
         endif
         dbclose
   end

;
;  What raster variations use line list ID xx?
;
   keyword_set(ll_id): begin
         rc = trim(ll_id)
         dbopen,'raster_var'
         stext = 'll_id='+rc
         val = dbfind(stext,/silent)
         if val(0) gt 0 then begin
            dbext,val,['ras_id','ras_var'],rid,rv
            print,'Following rasters/varn. use line list ID of '+rc
            for i=0,n_elements(rid)-1 do begin
               print,'     '+fmt_vect([rid(i),rv(i)],delim='/',/no_par)
            endfor   
         endif
         dbclose
   end

;
;  What rasters use line list description LLDESC?
;
   keyword_set(lldesc): begin
         case 1 of
            lldesc eq '?':  begin
               list_linelist,ll,n
               if n eq 0 then begin
                  print,'No line lists found in this database.'
               endif else begin
                  print_str,ll.ll_desc,/num
               endelse
               return
            end

            datatype(lldesc,1) eq 'String': begin
               if strmid(lldesc,0,1) eq '#' then  begin
                  nid = fix(strmid(lldesc,1,100))
                  list_linelist,ll,n
                  if n ge nid then begin
                     ll_desc = strtrim(ll(nid).ll_desc,2)
                  endif else begin
                     print,'Not that many line lists in the database.'
                     return
                  endelse
               endif else begin
                  ll_desc = lldesc
               endelse
               dbopen,'linelist'
               stext = 'll_desc='+ll_desc
               val = dbfind(stext,/silent)
               if val(0) gt 0 then begin
                  dbext,val,'ll_id',lid
                  dbclose
               endif else begin
                  print,'No line list with description found.'
                  print,'<< '+ll_desc+' >>'
                  dbclose
                  return
               endelse
               dbopen,'raster_var'
               stext = 'll_id='+'['+fmt_vect(lid,/no_p)+']'
               val = dbfind(stext,/silent)
               if val(0) gt 0 then begin
                  dbext,val,['ras_id','ras_var'], rid, rv
                  dbclose
                  print,'Following rasters/varn. use Line list with '+$
                        'description '+ll_desc
                  for i=0,n_elements(rid)-1 do begin
                     print,'     '+fmt_vect([rid(i),rv(i)],delim='/',/no_par)
                  endfor   
               endif else begin
                  print,'No rasters with that line list description'
                  dbclose
                  return
               endelse
            end
            else: 
         endcase
      end   
   else: begin
            print,'Use keywords to specify the search item:'
            print,' '
            print,'RASTER  -  specify a raster ID to retrieve which studies use
            print,'                          that raster.
            print,'VARIATION - to specify a raster variation as well (must'
            print,'            only be used with RASTER keyword).'
            print,'MLOC    -  specify to retrieve which rasters use that number
            print,'                          of mirror locations
            print,'SLOC    -  specify to retrieve which rasters use that number
            print,'                          of slit locations
            print,'EXPTIME -  specify to retrieve which rasters use this 
            print,'                          exposure time
            print,'DW_ID   - to search for rasters using that datawin ID'
            print,'LL_ID   - to search for rasters using that line list ID'
            print,'ACRONYM -  specify which study has this ''acronym'''
            print,'
            print,'LLDESC  -  several formats
            print,'      -->  "?" will return numbered list of line list
            print,'               descriptions            
            print,'      -->  "#nn" will return rasters using line list desc
            print,'                 number nn in the above list.
            print,'      -->  plain string will try to match that as a line list
            print,'                 description
         end
endcase

end
