;+
; Project     : SOHO - CDS
;
; Name        : CDS_STACK
;
; Purpose     : store CDS QL structures in a pointer stack
;
; Category    : CDS planning
;
; Syntax      : IDL> cds_stack,ql,[/set,/get,file=file,/clear]
;
; Inputs      : QL = CDS QL data structure (if /GET)
;
; Outputs     : QL = retrieved structure (if /SET and FILE)
;
; Keywords    : SET = insert QL in stack
;               GET = retrieve filename in keyword FILE from stack
;               CLEAR = empty stack
;               FILE = string filename to retrieve
;               STATUS =1/0 if success/failure
;               MAX_SIZE= maximum size of stack [def=25]
;               CHECK = check if QL is saved in stack
;
; History     : Version 1,  1-Sept-1999,  D.M. Zarro.  Written
;
; Contact     : DZARRO@SOLAR.STANFORD.EDU
;-

pro show_stack,stack                  ;-- show stack contents

if datatype(stack) eq 'STC' then begin
 if tag_exist(stack,'pointer') then begin
  nstack=n_elements(stack)
  for i=0,nstack-1 do begin
   temp=get_pointer(stack(i).pointer)
   if valid_ql(temp) then begin
    file=trim(get_tag_value(temp,/filename))
    break_file,file,dsk,dir,name
    print,num2str(i)+') '+trim(name)+' ('+stack(i).file+')'
   endif
  endfor
 endif
endif else message,'stack is empty',/cont

return & end

;--------------------------------------------------------------------------

pro free_stack,stack                 ;-- free-up stack memory

if datatype(stack) eq 'STC' then begin
 if tag_exist(stack,'pointer') then begin
  nstack=n_elements(stack)
  for i=0,nstack-1 do begin
   temp=get_pointer(stack(i).pointer,/no_copy)
   if valid_ql(temp) then delete_qlds,temp
   free_pointer,stack(i).pointer
  endfor 
 endif
endif
delvarx,stack

return & end

;------------------------------------------------------------------------------
            
pro cds_stack,ql,clear=clear,file=file,set=set,get=get,status=status,$
              max_size=max_size,show=show,check=check

common cds_stack,stack
status=0b
stack_exists=datatype(stack) eq 'STC'

;-- clear stack 

if keyword_set(clear) then begin
 free_stack,stack
 status=1b
 return
endif
    
;-- show stack
        
if keyword_set(show) then begin
 show_stack,stack
 status=1b
 return
endif

;-- insert into stack

curr_size=n_elements(stack)
if not exist(max_size) then max_size=25

if keyword_set(set) then begin

 if (not valid_ql(ql)) then begin
  status=0b
  return
 endif

 file=trim(get_tag_value(ql,/filename))
 break_file,file,dsk,dir,name,ext
 name=trim(name)
 in_stack=0b

 if stack_exists then begin
  slook=where(stack.file eq name,count)
  in_stack=count gt 0 
 endif

;-- if already in stack, just update pointer value and return

 if in_stack then begin
  ptr=stack(slook(0)).pointer 
  set_pointer,ptr,ql
  status=1b
  return
 endif

;-- if not in stack, append a new record if max_size is not reached
;-- if max_size is reached, recycle pointer from first record to conserve
;-- memory

 recycle=curr_size ge max_size
 need_pointer=1b
 if recycle then begin
  ptr=stack(0).pointer 
  if valid_pointer(ptr) then need_pointer=0b
 endif
 if need_pointer then make_pointer,ptr

 if not valid_pointer(ptr) then begin
  status=0b
  return
 endif

 set_pointer,ptr,ql
 record={cds_stack,file:name,pointer:ptr}

 if recycle then begin 
  nstart=curr_size-max_size+1
  if nstart gt 1 then begin
   old_stack=stack(1:nstart-1)
   free_stack,old_stack
  endif
  stack=stack(nstart:curr_size-1)
 endif

 stack=merge_struct(stack,record)
 dprint,'% CDS_STACK: stack size = '+num2str(n_elements(stack))   
 status=1b
 return

endif
 
;-- check or retrieve from stack

if (datatype(file) ne 'STR') or (1-stack_exists) then begin
 status=0b
 return
endif

if keyword_set(get) or keyword_set(check) then begin
 break_file,file,dsk,dir,name,ext
 name=trim(name)
 slook=where(stack.file eq name,count)

 if count eq 0 then begin
  status=0b
  return
 endif

 if keyword_set(check) then return

 ql=get_pointer(stack(slook(0)).pointer,status=status)
 if not status then begin
  dprint,'% CDS_STACK: '+file+' has corrupted pointer.'
  status=0b
  return
 endif
endif

;-- error checks

valid=valid_ql(ql,/delete)
if (not valid) then begin
 status=0b
 return
endif

status=1b
dprint,'% CDS_STACK: '+file+' retrieved from stack.'

return & end

