;+
; Project     :	SOHO - CDS
;
; Name        :	FIND_CDS_STUDIES
;
; Purpose     :	Find CDS studies or programs matching search criteria
;
; Category    :	CDS, Database, Class2
;
; Explanation :	This widget program allows the user to search for studies which
;		match certain search criteria.  Alternatively, one can search
;		for 
;
; Syntax      :	FIND_CDS_STUDIES, OUTPUT  [, GROUP=GROUP ]  [, /PROGRAM ]
;
; Examples    :	FIND_CDS_STUDIES, OUTPUT
;		FIND_CDS_STUDIES, OUTPUT, GROUP=EVENT.TOP, /PROGRAM
;
; Inputs      :	None.
;
; Opt. Inputs :	None.
;
; Outputs     :	OUTPUT	= An output structure array.  The format of OUTPUT
;			  depends on the value of the /PROGRAM keyword.
;			  Without the /PROGRAM keyword set, the tags are
;
;				OBS_PROG = The name of the observing program
;				STUDY_ID = The study ID number
;				STUDYVAR = The study variation number
;
;			  With the /PROGRAM keyword set, all the above tags are
;			  returned, plus the additional tags
;
;				DATE_OBS = The observation date (TAI seconds)
;				PROG_NUM = The observation counter number
;
; Opt. Outputs:	None.
;
; Keywords    :	GROUP	= The group ID of the calling widget.  If passed, and
;			  greater than zero, then FIND_CDS_STUDIES is run as a
;			  modal widget.
;
;		PROGRAM	= If set, then the program takes an additional step and
;			  looks for actual runs of each study found.
;
; Calls       :	DELVARX, STR_SEP, VALID_NUM, WAVE2PIX, NTRIM, DBOPEN, DBFIND,
;		DBEXT, DBCLOSE, UNIQ, XACK, XANSWER, WIDG_HELP
;
; Common      :	Uses the internal common block FIND_CDS_STUDIES
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Prev. Hist. :	None.
;
; History     :	Version 1, 27-Dec-1999, William Thompson, GSFC
;
; Contact     :	WTHOMPSON
;-
;
	PRO FIND_CDS_STUDIES_EVENT, EVENT
;
	COMMON FIND_CDS_STUDIES, OBS_PROG, STUDY_ID, STUDYVAR, FOUND, $
		DETECTOR, SLIT_NUM, EXPTIME_W, NX_W, NY_W, XSTEP_W, YSTEP_W, $
		COMP_ID, VDS_ORIENT, MSG_WIDG, WIDTH_W, HEIGHT_W, WAVE_W, $
		DURATION_W, INDEX, PROGRAM
;
;  Get the widget event, and act accordingly.
;
	WIDGET_CONTROL, EVENT.ID, GET_UVALUE=EV
	CASE EV OF
	    'HELP': WIDG_HELP, 'find_cds_studies', GROUP=EVENT.TOP,	$
		    TITLE='Help: FIND_CDS_STUDIES', /MODAL
;
;  If the FIND button was pressed, then start the process of finding the
;  studies which match the search criteria.  First, make sure that a detector
;  has been selected.
;
	    'FIND': BEGIN
		WIDGET_CONTROL, /HOURGLASS
		WIDGET_CONTROL, MSG_WIDG, SET_VALUE=''
		DELVARX, INDEX
		FOUND = 1
		IF N_ELEMENTS(DETECTOR) EQ 0 THEN BEGIN
		    ERROR_MESSAGE = 'Must select a detector'
		    GOTO, ERROR_POINT
		ENDIF
		SEARCH = 'DETECTOR=' + STRTRIM(DETECTOR,2) + ',USABLE=Y'
;
;  If wavelengths have been typed in, then parse the string.
;
		WIDGET_CONTROL, WAVE_W, GET_VALUE=WAVES
		WAVES = STRCOMPRESS(WAVES(0), /REMOVE_ALL)
		IF WAVES EQ '' THEN DELVARX, WAVES ELSE BEGIN
		    WAVES = STR_SEP(WAVES,',')
		    ERROR_MESSAGE = ['Invalid wavelength field.',	$
			    'Must be one or more numbers',		$
			    'separated by commas.']
		    FOR I=0,N_ELEMENTS(WAVES)-1 DO		$
			    IF NOT VALID_NUM(WAVES(I)) THEN GOTO, ERROR_POINT
		    WAVES = FLOAT(WAVES)
;
;  Check the wavelengths are in the correct ranges by converting them to
;  detector pixels.
;
		    FOR I = 0,N_ELEMENTS(WAVES)-1 DO BEGIN
			IF DETECTOR EQ 'N' THEN BEGIN
			    IF WAVES(I) LT 400 THEN BEGIN
				PIX = WAVE2PIX('N1',WAVES(I),/NOLIMIT)
			    END ELSE PIX = WAVE2PIX('N2',WAVES(I),/NOLIMIT)
			    IF (PIX LT 0) OR (PIX GT 1023) THEN BEGIN
				ERROR_MESSAGE = 'Bad wavelength ' +	$
					NTRIM(WAVES(I))
				GOTO, ERROR_POINT
			    ENDIF
			END ELSE BEGIN
			    IF WAVES(I) LT 240 THEN BEGIN
				PIX = WAVE2PIX('G1',WAVES(I),/NOLIMIT)
			    END ELSE IF WAVES(I) LT 350 THEN BEGIN
				PIX = WAVE2PIX('G2',WAVES(I),/NOLIMIT)
			    END ELSE IF WAVES(I) LT 500 THEN BEGIN
				PIX = WAVE2PIX('G3',WAVES(I),/NOLIMIT)
			    END ELSE PIX = WAVE2PIX('G4',WAVES(I),/NOLIMIT)
			    IF (PIX LT 0) OR (PIX GT 2047) THEN BEGIN
				ERROR_MESSAGE = 'Bad wavelength ' +	$
					NTRIM(WAVES(I))
				GOTO, ERROR_POINT
			    ENDIF
			ENDELSE
		    ENDFOR
		ENDELSE
;
;  Interpret the exposure time input string.
;
		WIDGET_CONTROL, EXPTIME_W, GET_VALUE=EXPTIME
		EXPTIME = STRCOMPRESS(EXPTIME(0), /REMOVE_ALL)
		IF EXPTIME NE '' THEN BEGIN
		    EXPTIME = STR_SEP(EXPTIME,'-')
		    ERROR_MESSAGE = ['Invalid exposure time field.',	$
			    'Must be a number, or two numbers',	$
			    'separated by a dash.']
		    IF N_ELEMENTS(EXPTIME) GT 2 THEN GOTO, ERROR_POINT
		    IF NOT VALID_NUM(EXPTIME(0)) THEN GOTO, ERROR_POINT
		    IF N_ELEMENTS(EXPTIME) EQ 2 THEN	$
			    IF NOT VALID_NUM(EXPTIME(1)) THEN GOTO, ERROR_POINT
		    IF N_ELEMENTS(EXPTIME) EQ 1 THEN SEARCH = SEARCH +	$
			    ',EXPTIME='+EXPTIME(0) ELSE SEARCH = SEARCH + $
			    ',EXPTIME>' + EXPTIME(0) + ',EXPTIME<' + EXPTIME(1)
		ENDIF
;
;  Interpret the duration input string.
;
		WIDGET_CONTROL, DURATION_W, GET_VALUE=DURATION
		DURATION = STRCOMPRESS(DURATION(0), /REMOVE_ALL)
		IF DURATION EQ '' THEN DELVARX, DURATION ELSE BEGIN
		    ERROR_MESSAGE = ['Invalid DURATION field.',	$
			    'Must be a number.']
		    IF NOT VALID_NUM(DURATION) THEN GOTO, ERROR_POINT
		    DURATION = FLOAT(DURATION)
		ENDELSE
;
;  Interpret the raster width input string.
;
		WIDGET_CONTROL, WIDTH_W, GET_VALUE=WIDTH
		WIDTH = STRCOMPRESS(WIDTH(0), /REMOVE_ALL)
		IF WIDTH EQ '' THEN DELVARX, WIDTH ELSE BEGIN
		    WIDTH = STR_SEP(WIDTH,'-')
		    ERROR_MESSAGE = ['Invalid WIDTH field.',	$
			    'Must be a number, or two numbers',	$
			    'separated by a dash.']
		    IF N_ELEMENTS(WIDTH) GT 2 THEN GOTO, ERROR_POINT
		    IF NOT VALID_NUM(WIDTH(0)) THEN GOTO, ERROR_POINT
		    IF N_ELEMENTS(WIDTH) EQ 2 THEN	$
			    IF NOT VALID_NUM(WIDTH(1)) THEN GOTO, ERROR_POINT
		    WIDTH = FLOAT(WIDTH)
		ENDELSE
;
;  Interpret the raster height input string.
;
		WIDGET_CONTROL, HEIGHT_W, GET_VALUE=HEIGHT
		HEIGHT = STRCOMPRESS(HEIGHT(0), /REMOVE_ALL)
		IF HEIGHT EQ '' THEN DELVARX, HEIGHT ELSE BEGIN
		    HEIGHT = STR_SEP(HEIGHT,'-')
		    ERROR_MESSAGE = ['Invalid HEIGHT field.',	$
			    'Must be a number, or two numbers',	$
			    'separated by a dash.']
		    IF N_ELEMENTS(HEIGHT) GT 2 THEN GOTO, ERROR_POINT
		    IF NOT VALID_NUM(HEIGHT(0)) THEN GOTO, ERROR_POINT
		    IF N_ELEMENTS(HEIGHT) EQ 2 THEN	$
			    IF NOT VALID_NUM(HEIGHT(1)) THEN GOTO, ERROR_POINT
		    HEIGHT = FLOAT(HEIGHT)
		ENDELSE
;
;  Interpret the NX input string.
;
		WIDGET_CONTROL, NX_W, GET_VALUE=NX
		NX = STRCOMPRESS(NX(0), /REMOVE_ALL)
		IF NX NE '' THEN BEGIN
		    NX = STR_SEP(NX,'-')
		    ERROR_MESSAGE = ['Invalid NX field.',	$
			    'Must be a number, or two numbers',	$
			    'separated by a dash.']
		    IF N_ELEMENTS(NX) GT 2 THEN GOTO, ERROR_POINT
		    IF NOT VALID_NUM(NX(0)) THEN GOTO, ERROR_POINT
		    IF N_ELEMENTS(NX) EQ 2 THEN	$
			    IF NOT VALID_NUM(NX(1)) THEN GOTO, ERROR_POINT
		    IF N_ELEMENTS(NX) EQ 1 THEN SEARCH = SEARCH +	$
			    ',NX='+NX(0) ELSE SEARCH = SEARCH + $
			    ',NX>' + NX(0) + ',NX<' + NX(1)
		ENDIF
;
;  Interpret the NY input string.
;
		WIDGET_CONTROL, NY_W, GET_VALUE=NY
		NY = STRCOMPRESS(NY(0), /REMOVE_ALL)
		IF (NY NE '') AND (DETECTOR EQ 'G') THEN BEGIN
		    NY = STR_SEP(NY,'-')
		    ERROR_MESSAGE = ['Invalid NY field.',	$
			    'Must be a number, or two numbers',	$
			    'separated by a dash.']
		    IF N_ELEMENTS(NY) GT 2 THEN GOTO, ERROR_POINT
		    IF NOT VALID_NUM(NY(0)) THEN GOTO, ERROR_POINT
		    IF N_ELEMENTS(NY) EQ 2 THEN	$
			    IF NOT VALID_NUM(NY(1)) THEN GOTO, ERROR_POINT
		    IF N_ELEMENTS(NY) EQ 1 THEN SEARCH = SEARCH +	$
			    ',NY='+NY(0) ELSE SEARCH = SEARCH + $
			    ',NY>' + NY(0) + ',NY<' + NY(1)
		ENDIF
;
;  Interpret the XSTEP input string.
;
		WIDGET_CONTROL, XSTEP_W, GET_VALUE=XSTEP
		XSTEP = STRCOMPRESS(XSTEP(0), /REMOVE_ALL)
		IF XSTEP NE '' THEN BEGIN
		    XSTEP = STR_SEP(XSTEP,'-')
		    ERROR_MESSAGE = ['Invalid Xstep field',	$
			    'Must be a number, or two numbers',	$
			    'separated by a dash.']
		    IF N_ELEMENTS(XSTEP) GT 2 THEN GOTO, ERROR_POINT
		    IF NOT VALID_NUM(XSTEP(0)) THEN GOTO, ERROR_POINT
		    IF N_ELEMENTS(XSTEP) EQ 2 THEN	$
			    IF NOT VALID_NUM(XSTEP(1)) THEN GOTO, ERROR_POINT
		    IF N_ELEMENTS(XSTEP) EQ 1 THEN SEARCH = SEARCH +	$
			    ',XSTEP='+XSTEP(0) ELSE SEARCH = SEARCH + $
			    ',XSTEP>' + XSTEP(0) + ',XSTEP<' + XSTEP(1)
		ENDIF
;
;  Interpret the YSTEP input string.
;
		WIDGET_CONTROL, YSTEP_W, GET_VALUE=YSTEP
		YSTEP = STRCOMPRESS(YSTEP(0), /REMOVE_ALL)
		IF (YSTEP NE '') AND (DETECTOR EQ 'G') THEN BEGIN
		    YSTEP = STR_SEP(YSTEP,'-')
		    ERROR_MESSAGE = ['Invalid Ystep field.',	$
			    'Must be a number, or two numbers',	$
			    'separated by a dash.']
		    IF N_ELEMENTS(YSTEP) GT 2 THEN GOTO, ERROR_POINT
		    IF NOT VALID_NUM(YSTEP(0)) THEN GOTO, ERROR_POINT
		    IF N_ELEMENTS(YSTEP) EQ 2 THEN	$
			    IF NOT VALID_NUM(YSTEP(1)) THEN GOTO, ERROR_POINT
		    IF N_ELEMENTS(YSTEP) EQ 1 THEN SEARCH = SEARCH +	$
			    ',YSTEP='+YSTEP(0) ELSE SEARCH = SEARCH + $
			    ',YSTEP>' + YSTEP(0) + ',YSTEP<' + YSTEP(1)
		ENDIF
;
;  If a specific slit was selected, then search on the slit number.
;
		IF SLIT_NUM GT 0 THEN SEARCH = SEARCH + ',SLIT_NUM=' +	$
			STRTRIM(SLIT_NUM,2)
;
;  If a specific compression mode was selected, then search on the compression
;  ID number.
;
		IF N_ELEMENTS(COMP_ID) EQ 1 THEN SEARCH = SEARCH +	$
			',COMP_ID='+STRTRIM(COMP_ID,2)
;
;  Depending on the compression mode, the VDS orientation parameter may also
;  need to be searched on.
;
		IF N_ELEMENTS(VDS_ORIENT) EQ 1 THEN SEARCH = SEARCH +	$
			',VDS_ORIENT='+STRTRIM(VDS_ORIENT,2)
;
;  Look for rasters which satisfy the basic search criteria.
;
		DBOPEN, 'raster_var,raster'
		ENTRY = DBFIND(SEARCH,/SILENT)
		IF ENTRY(0) LE 0 THEN BEGIN
		    ERROR_MESSAGE = 'No rasters found'
		    GOTO, ERROR_POINT
		ENDIF
		DBEXT, ENTRY, 'RAS_ID, RAS_VAR, DW_ID', RAS_ID, RAS_VAR, DW_ID
;
;  If the user entered in a maximum duration, then filter out rasters with
;  longer durations.
;
		IF N_ELEMENTS(DURATION) GT 0 THEN BEGIN
		    DBEXT, ENTRY , 'DURATION', DURATIONS
		    DURATIONS = ABS(DURATIONS)
		    W = WHERE(DURATION(0) GE DURATIONS, COUNT)
		    IF COUNT EQ 0 THEN BEGIN
			ERROR_MESSAGE = 'No rasters found'
			GOTO, ERROR_POINT
		    END ELSE BEGIN
			ENTRY   = ENTRY(W)
			RAS_ID  = RAS_ID(W)
			RAS_VAR = RAS_VAR(W)
			DW_ID   = DW_ID(W)
		    ENDELSE
		ENDIF
;
;  If selected, then filter out rasters with the wrong width.
;
		IF N_ELEMENTS(WIDTH) GT 0 THEN BEGIN
		    DBEXT, ENTRY , 'NX, XSTEP', NX, XSTEP
		    WIDTHS = NX*XSTEP
		    IF N_ELEMENTS(WIDTH) EQ 1 THEN BEGIN
			W = WHERE(WIDTH(0) EQ WIDTHS, COUNT)
			IF COUNT EQ 0 THEN BEGIN
			    ERROR_MESSAGE = 'No rasters found'
			    GOTO, ERROR_POINT
			END ELSE BEGIN
			    ENTRY   = ENTRY(W)
			    RAS_ID  = RAS_ID(W)
			    RAS_VAR = RAS_VAR(W)
			    DW_ID   = DW_ID(W)
			ENDELSE
		    END ELSE IF N_ELEMENTS(WIDTH) EQ 2 THEN BEGIN
			W = WHERE((WIDTH(0) LE WIDTHS) AND	$
				(WIDTH(1) GE WIDTHS), COUNT)
			IF COUNT EQ 0 THEN BEGIN
			    ERROR_MESSAGE = 'No rasters found'
			    GOTO, ERROR_POINT
			END ELSE BEGIN
			    ENTRY   = ENTRY(W)
			    RAS_ID  = RAS_ID(W)
			    RAS_VAR = RAS_VAR(W)
			    DW_ID   = DW_ID(W)
			ENDELSE
		    ENDIF
		ENDIF
;
;  If selected, and the GIS detector was selected, then filter out rasters with
;  the wrong height.  Ignored for NIS.
;
		IF (DETECTOR EQ 'G') AND (N_ELEMENTS(HEIGHT) GT 0) THEN BEGIN
		    DBEXT, ENTRY, 'NY, YSTEP', NY, YSTEP
		    HEIGHTS = NY*YSTEP
		    IF N_ELEMENTS(HEIGHT) EQ 1 THEN BEGIN
			W = WHERE(HEIGHT(0) EQ HEIGHTS, COUNT)
			IF COUNT EQ 0 THEN BEGIN
			    ERROR_MESSAGE = 'No rasters found'
			    GOTO, ERROR_POINT
			END ELSE BEGIN
			    ENTRY   = ENTRY(W)
			    RAS_ID  = RAS_ID(W)
			    RAS_VAR = RAS_VAR(W)
			    DW_ID   = DW_ID(W)
			ENDELSE
		    END ELSE IF N_ELEMENTS(HEIGHT) EQ 2 THEN BEGIN
			W = WHERE((HEIGHT(0) LE HEIGHTS) AND	$
				(HEIGHT(1) GE HEIGHTS), COUNT)
			IF COUNT EQ 0 THEN BEGIN
			    ERROR_MESSAGE = 'No rasters found'
			    GOTO, ERROR_POINT
			END ELSE BEGIN
			    ENTRY   = ENTRY(W)
			    RAS_ID  = RAS_ID(W)
			    RAS_VAR = RAS_VAR(W)
			    DW_ID   = DW_ID(W)
			ENDELSE
		    ENDIF
		ENDIF
;
;  If wavelengths were entered, then filter out rasters which do not observe
;  the selected wavelengths.  First extract out the information about the data
;  window and line lists.
;
		IF N_ELEMENTS(WAVES) GT 0 THEN BEGIN
		    DBOPEN, 'datawin'
		    DBEXT, -1, 'dw_id,ll_id,w_width', DW_IDS, LL_IDS, W_WIDTHS
		    DBOPEN, 'datawin2,datawin'
		    DBEXT, -1, 'dw_id,win_name', DW_IDS2, WIN_NAMES2
		    DBOPEN, 'linelist2,linelist'
		    DBEXT, -1, 'll_id,wavelnth,gr_order,waveband',	$
			    LL_IDS2, WAVELNTHS2, GR_ORDERS2, WAVEBANDS2
		    DWAVN = [0.07,0.117]
		    DWAVG = [0.034, 0.0415, 0.0485, 0.0615]
;
;  Step through the rasters which have been found so far.  From the data window
;  ID, determine the line list ID and the window widths.
;
		    GOOD = BYTARR(N_ELEMENTS(RAS_ID))
		    FOR I = 0,N_ELEMENTS(RAS_ID)-1 DO BEGIN
			GOOD_WAV = BYTARR(N_ELEMENTS(WAVES))
			W = WHERE(DW_ID(I) EQ DW_IDS)
			LL_ID = LL_IDS(W(0))
			W_WIDTH = W_WIDTHS(W(0))
;
;  If a line list is used, then use the window widths to determine if the
;  selected wavelengths fall in the data windows.
;
			IF LL_ID GT 0 THEN BEGIN
			    W = WHERE(LL_ID EQ LL_IDS2)
			    WAVELNTH = WAVELNTHS2(W) * GR_ORDERS2(W)
			    WAVEBAND = WAVEBANDS2(W)
			    IF DETECTOR EQ 'N' THEN DWAV=DWAVN ELSE DWAV=DWAVG
			    WAV1 = WAVELNTH - DWAV(WAVEBAND-1)*W_WIDTH/2
			    WAV2 = WAVELNTH + DWAV(WAVEBAND-1)*W_WIDTH/2
			    FOR J = 0,N_ELEMENTS(WAVES)-1 DO BEGIN
				W = WHERE((WAVES(J) GE WAV1) AND	$
					(WAVES(J) LE WAV2), COUNT)
				IF COUNT GT 0 THEN GOOD_WAV(J) = 1
			    ENDFOR
			ENDIF
;
;  Look for cases where the full wavebands are read out.
;
			W = WHERE(DW_ID(I) EQ DW_IDS2)
			WIN_NAMES = STRTRIM(WIN_NAMES2(W),2)
			FOR J = 0,N_ELEMENTS(WAVES)-1 DO BEGIN
			    IF DETECTOR EQ 'N' THEN BEGIN
				IF WAVES(J) LT 400 THEN	NAME = '1' $
					ELSE NAME = '2'
				NAME = 'BB_NIS_' + NAME
			    END ELSE BEGIN
				IF WAVES(J) LT 240 THEN NAME = '1' ELSE	$
					IF WAVES(I) LT 350 THEN NAME='2' ELSE $
					IF WAVES(I) LT 500 THEN NAME='3' ELSE $
					NAME = '4'
				NAME = 'BB_GIS_' + NAME
			    ENDELSE
			    W = WHERE(NAME EQ WIN_NAMES, COUNT)
			    IF COUNT GT 0 THEN GOOD_WAV(J) = 1
			ENDFOR
;
;  If each line is either found in a linelist window or in a full waveband
;  readout, then mark the raster as good.
;
			IF TOTAL(GOOD_WAV) EQ N_ELEMENTS(GOOD_WAV) THEN	$
				GOOD(I) = 1
		    ENDFOR
;
;  Filter out rasters which didn't pass the wavelength test.
;
		    W = WHERE(GOOD, COUNT)
		    IF COUNT GT 0 THEN BEGIN
			ENTRY   = ENTRY(W)
			RAS_ID  = RAS_ID(W)
			RAS_VAR = RAS_VAR(W)
			DW_ID   = DW_ID(W)
		    END ELSE BEGIN
			ERROR_MESSAGE = 'No rasters found'
			GOTO, ERROR_POINT
		    ENDELSE
		ENDIF
;
;  Extract the studies which use the selected rasters.
;
		DBOPEN, 'study_var2,study_var,study2,study_title,study'
		DBEXT, -1, 'ras_id,ras_var,usable', RAS_IDS, RAS_VARS, USABLE
		DELVARX, ENTRY
		FOR I=0,N_ELEMENTS(RAS_ID)-1 DO BEGIN
		    TEMP = WHERE((RAS_ID(I) EQ RAS_IDS) AND	$
			    (RAS_VAR(I) EQ RAS_VARS) AND (USABLE EQ 'Y'), $
			    COUNT) + 1
		    IF COUNT GT 0 THEN BEGIN
			IF N_ELEMENTS(ENTRY) EQ 0 THEN ENTRY = TEMP $
				ELSE ENTRY = [ENTRY, TEMP]
		    ENDIF
		ENDFOR
		IF N_ELEMENTS(ENTRY) EQ 0 THEN BEGIN
		    ERROR_MESSAGE = 'No studies found'
		    GOTO, ERROR_POINT
		ENDIF
		DBEXT, ENTRY, 'OBS_PROG, STUDY_ID, STUDYVAR',	$
			OBS_PROG, STUDY_ID, STUDYVAR
		DBCLOSE
;
;  Define the program label from the study name and the variation.  Filter out
;  repeats.  Show the result in the widget.
;
		PROGRAM = STRTRIM(OBS_PROG,2) + '/v' +	$
			STRTRIM(STUDYVAR,2)
		S = SORT(PROGRAM)
		PROGRAM = PROGRAM(S)
		OBS_PROG = OBS_PROG(S)
		STUDY_ID = STUDY_ID(S)
		STUDYVAR = STUDYVAR(S)
		U = UNIQ(PROGRAM)
		PROGRAM = PROGRAM(U)
		OBS_PROG = OBS_PROG(U)
		STUDY_ID = STUDY_ID(U)
		STUDYVAR = STUDYVAR(U)
		WIDGET_CONTROL, MSG_WIDG, SET_VALUE=PROGRAM
		END
;
;  Set up the detector variable.
;
	    'DETECTOR': BEGIN
		IF EVENT.VALUE EQ 0 THEN DETECTOR = 'N' ELSE DETECTOR = 'G'
		FOUND = 0
		END
;
;  Set up the slit number variable.
;
	    'SLIT': BEGIN
		SLIT_NUM = EVENT.INDEX
		FOUND = 0
		END
;
;  From the compression mode selected, set up the COMP_ID and VDS_ORIENT
;  parameters.
;
	    'COMP_ID': BEGIN
		DELVARX, COMP_ID, VDS_ORIENT
		CASE EVENT.INDEX OF
		    0: DUMMY = 1
		    1: COMP_ID = 1
		    2: COMP_ID = 2
		    3: COMP_ID = 3
		    4: BEGIN & COMP_ID=6 & VDS_ORIENT=0 & END
		    5: BEGIN & COMP_ID=6 & VDS_ORIENT=1 & END
		    6: COMP_ID = 7
		    7: COMP_ID = 8
		ENDCASE
		FOUND = 0
		END
;
;  Keep track of which results were selected.
;
	    'RESULTS': BEGIN
		I = EVENT.INDEX
		IF N_ELEMENTS(INDEX) EQ 0 THEN INDEX = I ELSE BEGIN
		    W = WHERE(I EQ INDEX, COUNT)
		    IF COUNT EQ 0 THEN INDEX = [INDEX, I]
		ENDELSE
		END
;
;  If the RESET button was pressed, then undo any selection in the RESULTS
;  widget.
;
	    'RESET': BEGIN
		DELVARX, INDEX
		WIDGET_CONTROL, MSG_WIDG, SET_VALUE=PROGRAM
		END
;
;  If the EXIT button was pressed, then check to see if the FIND button has
;  been pressed since the last change in any of the search parameters.  If not,
;  then see if the user wants to exit anyway.
;
	    'EXIT': BEGIN
		IF NOT FOUND THEN FOUND = XANSWER(			$
			['Input parameters have changed since the',	$
			'last time the FIND button was pressed,',	$
			'or else they have not yet been entered.',	$
			'', 'Do you still want to exit?'])
		IF FOUND THEN WIDGET_CONTROL, EVENT.TOP, /DESTROY
		END
;
;  Any other widget events will be in one of the text widgets.  Mark that the
;  FIND button needs to be used before exiting.
;
	    ELSE: FOUND = 0
	ENDCASE
	GOTO, EXIT_POINT
;
;  Target point for any errors found.
;
ERROR_POINT:
        XACK, ERROR_MESSAGE
	FOUND = 0
;
EXIT_POINT:
	END
;
;==============================================================================
;
	PRO FIND_CDS_STUDIES, OUTPUT, GROUP=GROUP, PROGRAM=K_PROGRAM,	$
		_EXTRA=_EXTRA
;
	ON_ERROR, 2
	COMMON FIND_CDS_STUDIES
;
;  If already registered, then inform the user and return.
;
	IF XREGISTERED('FIND_CDS_STUDIES') THEN BEGIN
	    XACK, 'Only one copy FIND_CDS_STUDIES can be run at a time'
	    RETURN
	ENDIF
;
;  Check the input parameters.
;
	IF N_PARAMS() NE 1 THEN BEGIN
	    XACK, 'Syntax: FIND_CDS_STUDIES, OUTPUT'
	    RETURN
	ENDIF
;
;  Get the value of the GROUP parameter.
;
	IF N_ELEMENTS(GROUP) EQ 0 THEN GROUP=0
;
;  Create the main widget base.  Create a base on the left for storing the 
;
	MODAL = GROUP(0) GT 0
	MAIN = WIDGET_BASE(GROUP_LEADER=GROUP, /ROW, /MAP, MODAL=MODAL, $
		TITLE='Find CDS Studies')
	LEFT = WIDGET_BASE(MAIN, /COLUMN, FRAME=1)
;
;  Create various widgets for controlling the different things.  Start off with
;  the FIND button.
;
	DUMMY = WIDGET_BUTTON(LEFT, UVALUE='HELP', VALUE='Help')
;
;  Define widgets for the detector, slit, exposure time, and duration.
;
	DUMMY1 = WIDGET_BASE(LEFT, /ROW)
	DUMMY2 = WIDGET_BASE(DUMMY1, /COLUMN)
	DUMMY = CW_BGROUP(DUMMY2, ['NIS', 'GIS'], COLUMN=2, /EXCLUSIVE,	$
		UVALUE='DETECTOR', LABEL_LEFT='Detector: ')
;
	TEXT = ['All', '#1: 2x2', '#2: 4x4', '#3: 8x50', '#4: 2x240',	$
		'#5: 4x240', '#6: 90x240']
	TEXT(1:*) = TEXT(1:*) + ' arcsec'
	DUMMY = WIDGET_DROPLIST(DUMMY2, VALUE=TEXT, UVALUE='SLIT',	$
		TITLE='Slit:')
	DUMMY = WIDGET_BUTTON(DUMMY1, VALUE='        FIND        ',	$
		UVALUE='FIND', FRAME=1)
;
	EXPTIME_W = CW_FIELD(LEFT, TITLE='Exposure time (seconds):',	$
		UVALUE='EXPTIME', /ALL_EVENTS)
;
	DURATION_W = CW_FIELD(LEFT, TITLE='Maximum duration (seconds):', $
		UVALUE='DURATION', /ALL_EVENTS)
;
;  Define widgets for the raster width and height.
;
	SUBBASE = WIDGET_BASE(LEFT, /COLUMN, FRAME=1)
	DUMMY = WIDGET_LABEL(SUBBASE, VALUE='Raster size (arcsec):',	$
		/ALIGN_LEFT)
	TEMP = WIDGET_BASE(SUBBASE, /ROW)
	WIDTH_W = CW_FIELD(TEMP, TITLE='Width:', UVALUE='WIDTH',	$
		/ALL_EVENTS, XSIZE=15)
	HEIGHT_W = CW_FIELD(TEMP, TITLE='Height:', UVALUE='HEIGHT',	$
		/ALL_EVENTS, XSIZE=15)
;
;  Define widgets for the raster parameters NX,NY,XSTEP,YSTEP.
;
	SUBBASE = WIDGET_BASE(LEFT, /COLUMN, FRAME=1)
	DUMMY = WIDGET_LABEL(SUBBASE, VALUE='Raster parameters:', /ALIGN_LEFT)
	TEMP = WIDGET_BASE(SUBBASE, /ROW)
	NX_W = CW_FIELD(TEMP, TITLE='NX:', UVALUE='NX', /ALL_EVENTS, XSIZE=15)
	XSTEP_W = CW_FIELD(TEMP, TITLE='Xstep:', UVALUE='XSTEP',	$
		/ALL_EVENTS, XSIZE=15)
	TEMP = WIDGET_BASE(SUBBASE, /ROW)
	NY_W = CW_FIELD(TEMP, TITLE='NY:', UVALUE='NY', /ALL_EVENTS, XSIZE=15)
	YSTEP_W = CW_FIELD(TEMP, TITLE='Ystep:', UVALUE='YSTEP',	$
		/ALL_EVENTS, XSIZE=15)
;
;  Define widgets for the compression mode and wavelength.
;
	TEXT = ['All', 'Straight copy', 'Trucate to 12 bits',	$
		'Variable Block Word Length (VBWL)', 'Sum over wavelength', $
		'Sum over total spatial dimension', 'Sum over windows',	$
		'Sum over N rows in spatial dimension']
	DUMMY = WIDGET_DROPLIST(LEFT, VALUE=TEXT, UVALUE='COMP_ID',	$
		TITLE='Compression:')
;
	WAVE_W = CW_FIELD(LEFT, TITLE='Wavelengths:', UVALUE='WAVELENGTH', $
		/ALL_EVENTS, XSIZE=40)
;
;  Depending on whether or not the /PROGRAM keyword was set, the last button
;  will be either CONTINUE or EXIT.
;
	IF KEYWORD_SET(K_PROGRAM) THEN VALUE='Continue' ELSE VALUE='Exit'
	DUMMY = WIDGET_BUTTON(LEFT, FRAME=1, UVALUE='EXIT', VALUE=VALUE)
;
;  Make the result widgets on the right.
;
	DUMMY = WIDGET_BASE(MAIN, /COLUMN, /FRAME, YOFFSET=-100)
	DUMMY_WIDG = WIDGET_LABEL(DUMMY, VALUE='Results:')
	MSG_WIDG = WIDGET_LIST(DUMMY, FRAME=1, VALUE='',	$
		UVALUE='RESULTS', XSIZE=20, YSIZE=25)
	DUMMY_WIDG = WIDGET_BUTTON(DUMMY, UVALUE='RESET', VALUE='Reset')
;
;  Start it all going.
;
	WIDGET_CONTROL, MAIN, /REALIZE
	DELVARX, OBS_PROG, STUDY_ID, STUDYVAR, DETECTOR, COMP_ID, VDS_ORIENT, $
		INDEX
	SLIT_NUM = 0
	FOUND = 0
	XMANAGER, 'FIND_CDS_STUDIES', MAIN, _EXTRA=_EXTRA
;
;  If particular studies were selected out of the result list, then filter out
;  the unselected studies.
;
	IF N_ELEMENTS(INDEX) GT 0 THEN BEGIN
	    OBS_PROG = OBS_PROG(INDEX)
	    STUDY_ID = STUDY_ID(INDEX)
	    STUDYVAR = STUDYVAR(INDEX)
	ENDIF
;
;  If the /PROGRAM keyword was set, then find the study counter numbers which
;  represent data generated with the selected studies.  Start by opening up the
;  main database and finding out what studies were run.
;
	IF N_ELEMENTS(OBS_PROG) GT 0 THEN BEGIN
	    IF KEYWORD_SET(K_PROGRAM) THEN BEGIN
		DBOPEN, 'main'
		DBEXT, -1, 'study_id,studyvar', STUDY_IDS, STUDYVARS
;
;  Find all the studies from the as-run database which match the selected
;  studies.
;
		DELVARX, ENTRY
		FOR I=0,N_ELEMENTS(STUDY_ID)-1 DO BEGIN
		    TEMP = WHERE((STUDY_ID(I) EQ STUDY_IDS) AND	$
			    (STUDYVAR(I) EQ STUDYVARS), COUNT) + 1
		    IF COUNT GT 0 THEN BEGIN
			IF N_ELEMENTS(ENTRY) EQ 0 THEN ENTRY = TEMP $
				ELSE ENTRY = [ENTRY, TEMP]
		    ENDIF
		ENDFOR
;
		IF N_ELEMENTS(ENTRY) EQ 0 THEN BEGIN
		    XACK, 'No observations found'
		    DELVARX, OUTPUT
		    GOTO, FINISH
		END
;
;  Extract the information about the selected observations.
;
		DBEXT, ENTRY, 'date_obs,prog_num,obs_prog,study_id,studyvar', $
			DATE_OBS, PROG_NUM, OBS_PROG, STUDY_ID, STUDYVAR
		W = WHERE(PROG_NUM GT 0, COUNT)
		IF COUNT GT 0 THEN BEGIN
		    DATE_OBS = DATE_OBS(W)
		    PROG_NUM = PROG_NUM(W)
		    OBS_PROG = OBS_PROG(W)
		    STUDY_ID = STUDY_ID(W)
		    STUDYVAR = STUDYVAR(W)
		END ELSE BEGIN
		    DELVARX, OUTPUT
		    XACK, 'No observations found'
		    GOTO, FINISH
		ENDELSE
;
;  Sort the observations by date.
;
		IF COUNT GT 1 THEN BEGIN
		    S = SORT(DATE_OBS)
		    DATE_OBS = DATE_OBS(S)
		    PROG_NUM = PROG_NUM(S)
		    OBS_PROG = OBS_PROG(S)
		    STUDY_ID = STUDY_ID(S)
		    STUDYVAR = STUDYVAR(S)
		ENDIF
;
;  Format the output structure with the information about the observations.
;
		OUTPUT = {DATE_OBS: 0.0D0, PROG_NUM: 0L, OBS_PROG: '',	$
			STUDY_ID: 0L, STUDYVAR: 0L}
		OUTPUT = REPLICATE(OUTPUT, N_ELEMENTS(OBS_PROG))
		OUTPUT.DATE_OBS = DATE_OBS
		OUTPUT.PROG_NUM = PROG_NUM
		OUTPUT.OBS_PROG = OBS_PROG
		OUTPUT.STUDY_ID = STUDY_ID
		OUTPUT.STUDYVAR = STUDYVAR
;
;  If /PROGRAM was not set, then format the output based on the studies found.
;
	    END ELSE BEGIN
		OUTPUT = {OBS_PROG: '', STUDY_ID: 0L, STUDYVAR: 0L}
		OUTPUT = REPLICATE(OUTPUT, N_ELEMENTS(OBS_PROG))
		OUTPUT.OBS_PROG = OBS_PROG
		OUTPUT.STUDY_ID = STUDY_ID
		OUTPUT.STUDYVAR = STUDYVAR
	    ENDELSE
;
;  If nothing was found, then undefine the OUTPUT array.
;
	END ELSE DELVARX, OUTPUT
;
FINISH:
	RETURN
	END
