;+
; Project     : SOHO - CDS
;
; Name        : GET_CDS_OH
;
; Purpose     : computes time after a set of un-tagged
;               CDS plans before a time-tagged plan can start.
;
; Category    : planning
;
; Explanation : 5% overhead is often necessary to allow for uncertainties
;               in study durations causing a series of consecutive studies
;               to run overtime.
;
; Syntax      : IDL>oh_time=get_cds_oh(time,plans,status=status)
;
; Inputs      : TIME = planned start time of plan entry 
;               (can also be a plan structure, in which case
;                PLAN.DATE_OBS is used)
;             : PLANS = current set of plans
;
; Opt. Inputs : None
;
; Outputs     : OH_TIME= computed start time of time-tagged study (with OH added)
;
; Opt. Outputs: None
;
; Keywords    : STATUS = 1 if input time is greater than OH time
;                      = 0 if input time is less than OH time
;
; Common      : None
;
; Restrictions: None
;
; Side effects: None
;
; History     : Version 1,  14-Feb-1997,  D.M. Zarro.  Written
;
; Contact     : DZARRO@SOLAR.STANFORD.EDU
;-

function get_cds_oh,time,plans,status=status,err=err,tgap=tgap

status=1
err=''
if n_elements(tgap) eq 0 then tgap=1.d-2
oh=.05

;-- PLAN or TIME input?

if datatype(time) eq 'STC' then begin
 if get_plan_type(time) eq 0 then begin
  itime=get_plan_itime(time)
  ptime=time.(itime)
 endif
endif else begin
 ttime=anytim2tai(time,err=err)
 if err eq '' then ptime=ttime
endelse

if not exist(ptime) then return,0
if not exist(plans) then return,ptime

;-- compute duration since last time-tagged entry up to and including input time

plast=mk_plan_where(ptime,plans,status=in_db)
if in_db then plast=plast-1
if plast lt 0 then return,ptime
plan_dur=0. & itime=get_plan_itime(plans)
last_time=plans(plast).(itime+1)
is_tt=where(plans(0:plast).time_tagged,count)
if count gt 0 then begin
 last_tt=plans(is_tt(count-1))
 plan_dur=(last_time-last_tt.(itime))
 dprint,anytim2utc(last_tt.(itime),/ecs),' : ',anytim2utc(last_time,/ecs)
endif 

odur=oh*plan_dur
odur=odur > tgap
oh_time=last_time+odur

status=str_format(ptime) ge str_format(oh_time)

return,oh_time
end
