;+
; Project     : SOHO - CDS
;
; Name        : GET_CDS_XY
;
; Purpose     : get central (X,Y) coordinates CDS rasters in a plan
;
; Category    : planning
;
; Explanation : 
;
; Syntax      : IDL> get_cds_xy,plan,x,y
;
; Inputs      : PLAN = plan structure
;
; Opt. Inputs : None
;
; Outputs     : X, Y = vector of X and Y positions in arcsecs
;
; Opt. Outputs: None
;
; Keywords    : FIRST = set to just return pointing of first raster in plan
;               LAST = set to just return pointing of last raster in plan
;               STUDY = study structure associated with PLAN (optional)
;               STRING_FORM = output pointings as strings
;               ONE_DIM = force output vectors to be 1-d
;               ERR = error string
;               FIXED = vector of 1/0 if fixed/deferred pointing
;
; Common      : None
;
; Restrictions: 
;
; Side effects: None
;
; History     : Version 1,  29-Dec-1995,  D.M. Zarro.  Written
;               Version 2,  2-Jun-1998, Zarro, added FIXED keyword
;               Version 3,  12-Dec-1998, Zarro (SM&A), added check for null plan
;
; Contact     : DZARRO@SOLAR.STANFORD.EDU
;-

pro get_cds_xy,plan,x,y,first=first,last=last,study=study,$
               string_form=string_form,err=err,one_dim=one_dim,fixed=fixed

on_error,1

x=0 & y=0 & err=''
fixed=0

;-- fake a PLAN structure if one doesn't exist

if not exist(plan) and (datatype(study) eq 'STC') then begin
 def_cds_detail,plan
 plan.study_id=study.study_id
 plan.studyvar=study.studyvar
endif

type=get_plan_type(plan)
if (type lt 0) then begin
 pr_syntax,'GET_CDS_XY,PLAN,X,Y'
 return
endif

;-- check for valid study tags

if (not tag_exist(plan,'STUDY_ID')) or $
   (not tag_exist(plan,'STUDYVAR')) then begin
 message,'No STUDY_ID/STUDYVAR associated with PLAN',/cont
 return
endif

;-- check for rasters

read_study=1
if (datatype(study) eq 'STC') then $
 read_study=(study.study_id ne plan.study_id) or (study.studyvar ne plan.studyvar)
if read_study then study=get_cds_study(plan.study_id,plan.studyvar,err=err)
if err ne '' then return

;-- if no rasters, then assume an engineering study and take deferred pointing

npoints=plan.n_pointings
if (not tag_exist(study,'RASTERS')) or (study.n_raster_def eq 0) then begin
 if npoints eq -1 then begin  ;-- if no deferred pointing then check HOME pos
  home_pos=get_cds_home()
  x=float(home_pos.solarx)
  y=float(home_pos.solary)
  fixed=0
 endif else begin
  x=plan.pointings(0).ins_x
  y=plan.pointings(0).ins_y
  fixed=fix(npoints eq 0)
 endelse
 goto,done
endif

;-- check STUDY.RASTERS.POINTING. If,
;  -1 --> offset and add to pointing in first raster
;   0 --> fixed and use .INS values in STUDY
;   1 --> deferred and use .INS values in PLAN.POINTINGS


rasters=study.rasters
npoints= npoints > 1
nrast=n_elements(rasters)
point_type=rasters.pointing
first_x=rasters(0).ins_x
first_y=rasters(0).ins_y

x=fltarr(npoints,nrast)
y=x
fixed=fix(x)

for j=0,npoints-1 do begin
 def_x=plan.pointings(j,0).ins_x
 def_y=plan.pointings(j,0).ins_y
 if point_type(0) eq 0 then begin  ;-- fixed pointing
  firstp=1
  zer_x=first_x & zer_y=first_y
 endif else begin
  firstp=0
  zer_x=def_x & zer_y=def_y
 endelse
 for i=0,nrast-1 do begin
  cur_x=rasters(i).ins_x
  cur_y=rasters(i).ins_y  
  case point_type(i) of
    -1: begin x(j,i)=zer_x+cur_x & y(j,i)=zer_y+cur_y & fixed(j,i)=firstp & end
     0: begin x(j,i)=cur_x & y(j,i)=cur_y & fixed(j,i)=1 & end
     1: begin x(j,i)=def_x & y(j,i)=def_y & fixed(j,i)=0 & end
   else:message,'invalid pointing type',/cont
  endcase
  quit= (keyword_set(first) and (i eq 0) and (j eq 0)) or $
        (keyword_set(last)  and (i eq (nrast-1)) and (j eq (npoints-1)))
  if quit then begin
   x=x(j,i) & y=y(j,i) & fixed=fixed(j,i) & goto,done
  endif
 endfor
endfor

done: 

if keyword_set(string_form) or keyword_set(one_dim) then begin
 sx=size(x)
 if sx(0) eq 2 then x=reform(x,sx(1)*sx(2))
 sy=size(y)
 if sy(0) eq 2 then y=reform(y,sy(1)*sy(2))
 sf=size(fixed)
 if sf(0) eq 2 then fixed=reform(fixed,sf(1)*sf(2))
endif

if keyword_set(string_form) then begin
 x=arr2str(trim(str_format(x,'(i6)')),delim=',')
 y=arr2str(trim(str_format(y,'(i6)')),delim=',')
 fixed=arr2str(trim(str_format(fixed,'(i1)')),delim=',')
endif else begin
 if n_elements(x) eq 1 then x=x(0)
 if n_elements(y) eq 1 then y=y(0)
 if n_elements(fixed) eq 1 then fixed=fixed(0)
endelse

return & end

 
