;+
; Project     :	SOHO - CDS
;
; Name        :	GET_PLAN_TT
;
; Purpose     :	Find start time of next time-tagged study in a series of plans
;
; Explanation : Useful for speeding up editing in MK_PLAN
;
; Use         :	TNEXT=GET_PLAN_TT(TIME,PLANS)
;
; Inputs      :
;               TIME = input time 
;               PLANS = structure array of plan entries to search
;
; Opt. Inputs : None.
;
; Outputs     :	TNEXT = index of next time-tagged study
;
; Opt. Outputs:	None.
;
; Keywords    : GSET = if > -1 then find first time-tagged study after 
;                      GIS study with this GSET 
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Category    :	Planning
;
; Prev. Hist. :	None.
;
; Written     :	Dominic Zarro (ARC)
;               
; Modified    : April 25 1997, Zarro, added check for undefined plans input
;
; Version     :	2
;       
;-

function get_plan_tt,time,plans,err=err,gset=gset

err=''
on_error,1
index=-1 
type=get_plan_type(plans)
if type eq -1 then return,index
if not exist(time) then return,index

;-- get tag indicies for PLANS start and end times

temp=anytim2utc(time,err=err)
if err ne '' then return,index
ptime=utc2tai(temp)

itime=get_plan_itime(plans) 
pstart=(plans.(itime))
pend=(plans.(itime+1))
nplans=n_elements(plans)
last_plan=plans(nplans-1)

;-- format times

ptime=str_format(ptime)
pstart=str_format(pstart)
pend=str_format(pend)

;-- find all GIS studies

next_gis_index=-1
if exist(gset) and tag_exist(plans,'GSET_ID') then begin
 if gset gt -1 then begin
  dprint,'% GET_PLAN_TT: checking for GIS studies'
  use_gis=(plans.gset_id gt -1)
  glook=where( (ptime lt pstart) and use_gis, cnt)
  if cnt gt 0 then next_gis_index=glook(0)
  gsame=where( (ptime lt pstart) and use_gis and (plans.gset_id eq gset), scnt)
  if scnt gt 0 then next_gis_index=gsame(0)
 endif
endif

stime=ptime
flook=where(pstart gt ptime,cnt)

;-- if TIME-TAGGED field doesn't exist then return start of next plan, or 
;   end of last plan (if no next plan)

if not tag_exist(plans,'TIME_TAGGED') then begin
 if cnt gt 0 then tnext=plans(flook(0)).(itime) else tnext=last_plan.(itime+1)
 return,tnext
endif

;-- examine first study after PTIME. If time-tagged, then we skip it

if cnt gt 0 then begin
 first=plans(flook(0)) 
 if first.time_tagged then begin
  stime=first.(itime+1)
 endif
endif

;-- find first time-tagged study after STIME
;-- if GSET is entered then find first time-tagged study after first GIS study
;   after STIME with this GSET value. If not found, then take next available
;   GIS study

time_tagged=plans.time_tagged
pindex=indgen(nplans)

;-- default to last plan if no match found

tlook=where( (stime lt pstart) and time_tagged and (pindex gt next_gis_index),cnt)
if cnt gt 0 then  tnext=plans(tlook(0)).(itime) else tnext=last_plan.(itime+1)

if tlook(0) eq (nplans-1) then tnext=last_plan.(itime+1)

return,tnext & end

