;+
; Project     : SOHO - CDS
;
; Name        : GET_RASTER_PAR
;
; Purpose     : to get CDS raster parameter information
;
; Category    : database, planning
;
; Explanation : 
;
; Syntax      : IDL> get_raster_par,raster
;
; Inputs      :	rasters = CDS raster structure
;
; Opt. Inputs :	None.
;
; Outputs     :	See keywords.
;
; Opt. Outputs:	None.
;
; Keywords    :	
;               ERR = any error string
;               RDUR  = duration (secs) of individual rasters composing study 
;               REPS  = # of repeats per raster
;               DETS  = detectors per raster (e.g. ['N','G','N'....])
;               SLITS = slit number per raster
;               SDUR = total duration of study from summing rasters
;                      (not taking account of slit and OPS motions)
;               OH  =  % overhead to add to duration
;               TEL = telemetry rate of each raster (L/M/H)
;               VERBOSE = verbose output 
;
; Common      : None
;
; Restrictions: None
;
; Side effects: None
;
; History     : Version 1,  1-Jan-1996,  D.M. Zarro.  Written
;
; Contact     : DZARRO@SOLAR.STANFORD.EDU
;-

pro get_raster_par,rasters,rdur=rdur,sdur=sdur,dets=dets,slits=slits,oh=oh,$
                   reps=reps,err=err,telem=telem,verbose=verbose

err=''
dets='' & reps=0. & dur=0. & rdur=0. & sdur=0.

if not exist(rasters) then begin
 message,'Syntax: GET_CDS_RASTER,RASTER,[SDUR=SDUR]',/cont
 return
endif

rid=rasters.ras_id
rv=rasters.ras_var
reps=float(rasters.n_repeat_r)

;-- sum raster durations 

sdur=0.
if n_elements(oh) eq 0 then oh=0.
overhead=(1.+oh/100.)
nrast=n_elements(rasters)
rdur=findgen(nrast)
dets=strarr(nrast)
slits=intarr(nrast)
telem=strarr(nrast)
for i=0,nrast-1 do begin
 ras_stc=get_cds_raster(rid(i),rv(i))
 if (ras_stc.ras_id  lt 0) then begin
  message,'no raster info for '+num2str(rid(i))+','+num2str(rv(i)),/contin
 endif else begin
  duration=abs(ras_stc.duration)
  rdur(i)=duration*reps(i)
  dets(i)=ras_stc.detector
  slits(i)=ras_stc.slit_num
  telem(i)=trim(ras_stc.tel_rate)
  sdur=sdur+rdur(i)
 endelse
endfor

sdur=sdur*overhead
rdur=rdur*overhead

count=nrast
if count eq 1 then begin
 dets=dets(0) & reps=reps(0) & rdur=rdur(0) & slits=slits(0)
 telem=telem(0)
endif

return & end
