	PRO OPS_POINT, OPSLCOORD, OPSRCOORD, SOLAR_X, SOLAR_Y,	$
		DATE_OBS=DATE_OBS
;+
; Project     :	SOHO - CDS
;
; Name        :	OPS_POINT
;
; Purpose     :	Calculate pointing from OPS data in the telemetry.
;
; Explanation :	Calculates the pointing based on the OPS data in the telemetry.
;		The effects of the slit and scan mirror mechanisms are not
;		taken into account.  Nor are the effects of the pixel selection
;		on the (VDS) detector.
;
; Use         :	OPS_POINT, OPSLCOORD, OPSRCOORD, SOLAR_X, SOLAR_Y
;
; Inputs      :	OPSLCOORD = OPS L coordinate telemetry word, from either the
;			    raster or exposure headers.  Can be either a scalar
;			    or an array.
;		OPSRCOORD = OPS R coordinate telemetry word.
;
; Opt. Inputs :	None.
;
; Outputs     :	SOLAR_X	  = Pointing in the X direction, in arcsec.
;		SOLAR_Y	  = Pointing in the Y direction, in arcsec.
;
; Opt. Outputs:	None.
;
; Keywords    :	DATE_OBS  = If passed, then contains the date that should be
;			    used when determining the conversion.  When
;			    DATE_OBS is passed, then GET_CDS_PNTCAL is called.
;
; Calls       :	OPS_DECODE, CP_GET_MNEMONIC, CP_GET_INDECES, GET_CDS_PNTCAL
;
; Common      :	None.
;
; Restrictions:	The formulas used to calculate the pointing are based on
;		equations 11 and 12 in "Notes on Pointing Calculation in CDS",
;		Version 2.0, MKC, 1/3/94.  They should be replaced with a more
;		exact calculation later.
;
;		Also, this routine should be upgraded to incorporate
;		information about the spacecraft pointing and roll.
;
; Side effects:	None.
;
; Category    :	Data handling, telemetry.
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 29 August 1994
;
; Modified    :	Version 1, William Thompson, GSFC, 23 June 1995
;               Version 2, MKC, RAL, 15/8/95
;                            Added calls to CDHS state database to retrieve 
;                            pointing parameters.
;                            Noted Lx and Ly parameters were swapped.
;               Version 3, MKC, RAL, 5/1/96
;                            Modified state database storage of cb2point 
;                            values to reflect proper command format so 
;                            have to change handling of indices.
;               Version 4, CDP, RAL, 28-06-96
;                            Use common block to eliminate unnecessary
;                            database access.
;		Version 5, William Thompson, GSFC, 22 August 1996
;			     Only call OPS_DECODE if short integer values are
;			     passed.  Otherwise, take values as they are.  This
;			     allows fractional values to be passed.
;		Version 6, William Thompson, GSFC, 8 May 1997
;			     Add keyword DATE_OBS
;
; Version     :	Version 6, 8 May 1997
;-
;
	ON_ERROR,2
;
;  Common block for internal storage.
;
        COMMON cds_lrxy, s, ss
;
;  Check the number of arguments.
;
	IF N_PARAMS() NE 4 THEN MESSAGE,	$
		'Syntax:  OPS_POINT, OPSLCOORD, OPSRCOORD, SOLAR_X, SOLAR_Y'
;
;  Extract the values for the offset pointing system.
;
	IF DATATYPE(OPSLCOORD,1) EQ 'Integer' THEN BEGIN
		OPS_DECODE, OPSLCOORD, BITS, OPSL
		OPS_DECODE, OPSRCOORD, BITS, OPSR
	END ELSE BEGIN
		OPSL = OPSLCOORD
		OPSR = OPSRCOORD
	ENDELSE
;
;  If the DATE_OBS keyword was passed, then call GET_CDS_PNTCAL and return.
;
	IF N_ELEMENTS(DATE_OBS) EQ 1 THEN BEGIN
		GET_CDS_PNTCAL, DATE_OBS, OPSL, OPSR, SOLAR_X, SOLAR_Y
		RETURN
	ENDIF
;
;  Get ops pointing parameters, either from common (if been here before)
;  or directly from database.
;
        IF N_ELEMENTS(s) EQ 0 THEN BEGIN
           S = CP_GET_MNEMONIC ( 'CB2POINT', /QUIET )
;
;  Get structure elements corresponding to relevant ops pointing parameters.
;
           SS = CP_GET_INDECES ( S, [0,2,4,6,7,8,10] )
;
;  Check that values are OK.
;
           IF N_ELEMENTS(ss) EQ 0 THEN $
             MESSAGE, 'INVALID OPS POINTING PARAMETERS IN CDHS STATE DATABASE'
        ENDIF


; set up constants
; NB values stored as 16 bit values for cb2point command
;    only OPS R may be bigger than 65536

  ops_l_zero = ss(0).active                            ; ops l position corresponding to sun centre
  ops_r_zero = ss(1).active                            ; ops r position corresponding to sun centre
  ops_K      = ss(2).active                            ; ops K value
  ops_R      = ss(3).active + ISHFT(ss(4).active,16)   ; ops R value
  ops_LTHE   = ss(5).active                            ; ops L theta value
  ops_LPHI   = ss(6).active                            ; ops L phi value
;
;  The following constants are used in the calculation.
;
        LX = FLOAT ( ops_LPHI )	;X pivot length in mm
	LY = FLOAT ( ops_LTHE )	;Y pivot length in mm
	K  = FLOAT ( ops_K )
	R  = FLOAT ( ops_R )
;
;  Calculate the pointing in arcsec (using floating point arithmetic).
;  NB should probably calculate dx,dy for sun centre to central position and add
;     to dx,dy for l,r to central position but won't be much different
;
	DEL_P  = 1.*OPSL  + OPSR - ( ops_l_zero + ops_r_zero )
	DEL_M  = 1.*OPSL  - OPSR - ( ops_l_zero - ops_r_zero )

;  2nd order calculation

	SOLAR_X  = -(K/LX) * (DEL_M  + DEL_M * DEL_P /R)
	SOLAR_Y  = -(K/LY) * (DEL_P  - (DEL_M^2  + DEL_P^2 )/(2*R))

;
	RETURN
	END
