;+
; Project     : SOHO - CDS     
;                   
; Name        : ROT_CDS_XY
;               
; Purpose     : solar rotate deferred CDS pointing
;               
; Category    : Planning
;               
; Explanation : computes pointing in CDS planning structures by
;               computing solar rotation during time interval since first
;               pointing.
;               
; Syntax      : IDL> rplan=rot_cds_xy(plan)
;    
; Examples    : 
;
; Inputs      : PLAN = plan structure (can be array)
;               
; Opt. Inputs : None
;               
; Outputs     : RPLAN = rotated plan structures
;
; Opt. Outputs: None
;               
; Keywords    : RPLAN = reference plan coordinates to rotate from
;               MATCH = rotate relative to matching pointing entries
;                      (if RPLAN has same number of pointing as PLAN)
;               WARN = warning if rotation steps are less than 2"
;
; Common      : None
;               
; Restrictions: Input plans must have deferred pointing
;               
; Side effects: None.
;               
; History     : Version 1,  17-July-1996,  D M Zarro.  Written
;               Version 2,   6-June-1997, Zarro, modified to only
;               accept rotation steps greater than 2" since this
;               is the basic OPS step size.
;               Version 3,   20-May-1998, Zarro, fixed small bug in
;               timing for multiple repeats of last raster 
;
; Contact     : DZARRO@SOLAR.STANFORD.EDU
;-            

function rot_cds_xy,plan,err=err,rplan=rplan,match=match,warn=warn
err=''

dwarn=$
['Difference between rotated coordinates is less than 2 arcsecs.',$
 'In the current mode, any pointing changes will be limited to those where ',$
 'the calculated change between rasters is >= 2.',$
 '',$
 'For more accurate control, you should consider using the scan mirror pointing mode.',$
 'Please ask the Operations staff for further advice.']

dwarn=$

['You have asked for a small motion to adjust for solar rotation.',$
 'You should be aware that the accuracy of the CDS Offset Pointing System ',$
 '(OPS) may not be consistent with such small movements.  There are three ',$
 'options open to you:',$
 '',$
 '1. Do not compensate for solar rotation.',$
 '',$
 '2. Adjust the "REPEAT LAST RASTER" and "# OF POINTINGS" parameters',$
 '   to make the same total number of observations, but compensate',$
 '   for rotation less often.  ',$
 '   (Depends on the study definition containing only a single raster.)',$
 '',$
 '3. Select the option: ',$
 '   "Allow adjustment of mirror and slit positions to avoid OPS movement" in the XCPT control panel. ',$
 '   (Note: This will work only if there is enough range in the mirror and slit mechanisms.) ']

if datatype(plan) ne 'STC' then if exist(plan) then return,plan else return,-1
np=n_elements(plan)
type=get_plan_type(plan)
if type eq -1 then return,plan
tplan=plan
itime=get_plan_itime(plan)
have_pointings=tag_exist(plan,'N_POINTINGS')
if not have_pointings then begin
 err='Plan does not have pointing parameters'
 return,plan
endif
have_reps=tag_exist(plan,'N_REPEAT_S')

;-- if no reference plan, rotate from first pointing
;   in current set of plans
;-- if reference plan entered then rotate coordinates relative to
;   last pointing in reference plan. If MATCH set, then rotate relative
;   to matching pointing index

ref_entered=(get_plan_type(rplan) gt -1)
if ref_entered then begin
 if keyword_set(match) and (plan.n_pointings ne rplan.n_pointings) then begin
  err='Cannot rotate. Previous plan has different number of pointings.'
  return,plan
 endif
 dprint,'%ROT_CDS_XY: using last plan'
 get_study_par,plan=rplan(0),rdur=rdur,reps=reps
 nr=n_elements(rdur)
 bduration=0.
 if (nr gt 0) then bduration=rdur(nr-1)
 if tag_exist(rplan,'n_rasters1') then $
  bduration=bduration*rplan.n_rasters1/reps(nr-1)

 rdate=rplan(0).(itime+1)-bduration
 get_cds_xy,rplan(0),ref_x,ref_y,last=1-keyword_set(match)
endif else begin
 rdate=tplan(0).(itime)
 get_cds_xy,tplan(0),ref_x,ref_y,/first
endelse

delvarx,warn
tol=2.0
for i=0,np-1 do begin
 n_pointings=tplan(i).n_pointings
 if (n_pointings gt 0) then begin
  bplan=tplan(i)
  bplan.n_pointings=1
  if have_reps then bplan.n_repeat_s=1
  duration=get_cds_dur(bplan,point=0)
  interval=(tplan(i).(itime)-rdate)
  dprint,'%ROT_CDS_XY: interval ',i,interval
  for j=0,n_pointings-1 do begin
   if (keyword_set(match)) and ref_entered then begin
    xp=ref_x(j,0) & yp=ref_y(j,0)
   endif else begin
    xp=ref_x & yp=ref_y
   endelse
   rot_cor=rot_xy(xp,yp,interval+duration*j,date=rdate,$
                  /soho,/keep)
   new_x=rot_cor(0)   
   new_y=rot_cor(1)
   perr=''
   get_ops_pos,new_x,new_y,err=perr
   if perr ne '' then begin
    new_x=xp & new_y=yp
   endif

   if (j gt 0) then begin
    old_x=tplan(i).pointings(j-1).ins_x
    old_y=tplan(i).pointings(j-1).ins_y
    if abs(new_x-old_x) le tol then begin
     new_x=old_x
     if not exist(warn) then warn=dwarn
    endif
    if abs(new_y-old_y) le tol then new_y=old_y
   endif
   tplan(i).pointings(j).ins_x=new_x
   tplan(i).pointings(j).ins_y=new_y
  endfor
 endif
endfor

;-- if reference plan not entered then set first pointing back

if not ref_entered then begin
 tplan(0).pointings(0).ins_x=ref_x
 tplan(0).pointings(0).ins_y=ref_y
endif

if (np eq 1) then begin
 if (n_pointings eq 0) then err='Plan entry does not have deferred pointing'
 if not ref_entered and (n_pointings eq 1) then err='Plan entry has only one pointing'
endif

return,tplan & end


