;+
; Project     : SOHO - CDS     
;                   
; Name        : VALID_CDS_POINT
;               
; Purpose     : validate that CDS pointing is within acceptable limits
;               
; Category    : Planning
;               
; Explanation : CDS is capable of pointing within a 4080 arcsec square
;               centered approx. on the Sun. The square is rotated 45 degs
;               from N/S owing to the orientation of the OPS legs.
;               
; Syntax      : IDL> valid=valid_cds_point(x,y)
;    
; Examples    : 
;
; Inputs      : x,y = coordinate pair to check (e.g. -200, 300) in units of
;               arcsecs (N and W being positive)
;               
; Opt. Inputs : None
;               
; Outputs     : 1/0 = for valid/invalid pointing. If coordinates are
;               arrays, then valid is an array.
;
; Opt. Outputs: None
;               
; Keywords    : None
;
; Common      : None
;               
; Restrictions: None
;               
; Side effects: None.
;               
; History     : Version 1,  17-Nov-1995,  D M Zarro.  Written
;               Version 2,  28-Jun-1996   C D Pike    Rewrite to only take
;                                                     account of OPL limits
;
; Contact     : DZARRO@SOLAR.STANFORD.EDU
;-            
;==============================================================================


function valid_cds_point,x,y,err=err

on_error,1
valid=0b

if not have_proc('get_ops_pos') then return,1b

if (n_elements(x) ne n_elements(y)) or $
   (n_elements(x) eq 0) or (n_elements(y) eq 0) then begin
 message,'Input coordinates must either both scalar, or equal sized vectors',/cont
 return,valid
endif

;
;  Hardcode OPS limits
;
opsl_low = '240'x
opsl_hi  = 'e00'x
opsr_low = '200'x
opsr_hi  = 'e00'x

;
;  calculate OPS positions for (x,y) input
;
np=n_elements(x)
opsl=fltarr(np) & opsr=fltarr(np) & valid = bytarr(np)

for i=0,np-1 do begin

;
;  calculate individual OPS coordinates
;
 err=''
 get_ops_pos,x(i),y(i),l,r,err=err
 opsl(i)=l
 opsr(i)=r
;
;  check validity
;
 valid(i) = (opsl(i) gt opsl_low) and $
            (opsl(i) lt opsl_hi)  and $
            (opsr(i) gt opsr_low) and $
            (opsr(i) lt opsr_hi) 
     
endfor

if np eq 1 then valid=valid(0)

return,valid & end


