;+
; Project     : SOHO - CDS
;
; Name        : XPROGRAM_ADD
;
; Purpose     : widget interface to add program ID to database
;
; Category    : planning,operations,widgets
;
; Explanation : 
;
; Syntax      : IDL> XPROGRAM_ADD
;
; Inputs      : None
;
; Opt. Inputs : PROG_ID = if valid PROG_ID is entered, then it is displayed
;
; Outputs     : None
;
; Opt. Outputs: None
;
; Keywords    : GROUP = widget ID of any calling widget
;               MODAL = set to freeze calling widget
;               SAVED = 1 if program successfully added
;
; Common      : None
;
; Restrictions: None
;
; Side effects: None
;
; History     : Version 1,  4-Jan-1996,  D.M. Zarro.  Written
;
; Contact     : DZARRO@SOLAR.STANFORD.EDU
;-

pro xprogram_add_event,event

widget_control, event.id, get_uvalue = uservalue
if (n_elements(uservalue) eq 0) then uservalue=''
child=widget_info(event.top,/child)
widget_control,child, get_uvalue = unseen

;-- next get pointer values

info=get_pointer(unseen,/no_copy)
if datatype(info) ne 'STC' then return 
nx=info.nx
ny=info.ny

;-- button event

bname=strtrim(uservalue,2)
case bname of 

 'desc': begin
   widget_control,event.id,get_value=in_string
   info.program.progdesc=strmid(in_string(0:ny-1),0,nx)
  end

 'name': begin
   widget_control,event.id,get_value=in_string
   info.program.progname=strmid(strtrim(in_string(0),2),0,20)
  end
 
 'return': begin
   progdesc=strtrim(info.program.progdesc,2)
   progname=strtrim(info.program.progname,2)
   prog_id=info.program.prog_id
   chk=where(progdesc ne '',count)
   stat=1
   if which_inst() eq 'C' then stat=call_function('priv_zdbase',/sealed)
   ok_to_save=(progname ne '') and (count gt 0) and (prog_id eq -1) and stat
   save_it=0 & quit=1
   if ok_to_save then begin
    save_it=xanswer('Current program not saved. Do you wish to save it?',$
                     group=event.top)
    if save_it then begin
     xprogram_save,info,event
     if (not info.saved) then quit=0
    endif
   endif
   if quit then begin
    xtext_reset,info
    xkill,event.top
   endif
  end

 'clear': begin
   def_program,program
   info=rep_tag_value(info,program,'program')
   widget_control,info.wdesc,set_value=program.progdesc
   widget_control,info.wname,set_value=''
   widget_control,info.wid,set_value='-1'
   info.saved=0
  end

 'save': xprogram_save,info,event

  else: do_nothing=1

endcase

if info.saved then begin
 if xalive(info.wid) then widget_control,info.wid,set_value=fix(info.program.prog_id)
endif

set_pointer,unseen,info,/no_copy

return & end

;------------------------------------------------------------------------

pro xprogram_save,info,event  ;-- save in DB 

progdesc=strtrim(info.program.progdesc,2)
progname=strtrim(info.program.progname,2)
chk=where(progdesc ne '',count)
print,count,progdesc
ok_to_save=(progname ne '') and (count gt 0)

;-- check for valid fields

if (not ok_to_save) then begin
 xack,'Please fill out both Program name and Description',group=event.top
 return
endif

;-- check if already saved

if info.program.prog_id gt -1 then begin
 xack,'Program already saved',group=event.top
 return
endif

;-- go ahead and save it

info.saved=0

;-- check write priviledge

if which_inst() eq 'C'  then begin
 status=call_function('priv_zdbase',/sealed)
 if not status then begin
  err='Priviledge to update program DB denied. Try switching DB.'
  xack,err,group=event.top,/icon
  return
 endif
endif

err=''                                                      
program=info.program
s=add_program(program,err=err)
if (err ne '') then begin
 xack,err,group=event.top
endif else begin
 info.saved=1
 info=rep_tag_value(info,program,'program')
endelse

return & end

;------------------------------------------------------------------------

pro xprogram_add,prog_id,group=group,modal=modal,saved=saved

on_error,1

;-- defaults

saved=0
if not have_widgets() then begin
 message,'widgets unavailable',/cont
 return
endif

mk_dfont,bfont=bfont

;-- check if user entered valid PROG_ID.
;   If so, then display it.

user_input=0
def_program,entry
if exist(prog_id) then begin
 err=''
 get_program,prog_id,program,err=err
 if err eq '' then begin
  entry=program 
  user_input=1
 endif
endif

wbase=widget_base(/column,title='XPROGRAM_ADD')

row1=widget_base(wbase,row=1,/frame)

;-- operation buttons

returnb=widget_button(row1,value='EXIT',uvalue='return',font=bfont,/no_rel)
if not user_input then begin
 clearb=widget_button(row1,value='CLEAR VALUES',uvalue='clear',font=bfont,/no_rel)
 saveb=widget_button(row1,value='SAVE TO DB',uvalue='save',font=bfont,/no_rel)
endif

;-- text widgets

pdesc=entry.progdesc
pname=entry.progname
pid=strtrim(string(entry.prog_id),2)

ny=n_elements(pdesc)
nx=80

wid=cw_field(wbase, Title=   'Program ID:          ',value=pid,$
                      xsize = 10, font = bfont,/noedit)


wname=cw_field(wbase, Title= 'Program Name:        ',value=pname,$
             noedit=user_input,/all,uvalue='name', xsize = 20, font = bfont)

temp=widget_base(wbase,/row,/frame)
t1=widget_label(temp,value=  'Program Description: ',font=bfont)
wdesc=widget_text(wbase,xsize=nx,value=pdesc,ysize=ny,uvalue='desc',$
                  edit=(1-user_input),/scroll,all=(1-user_input))
      
;-- realize and center main base

xrealize,wbase,group=group,/center

;-- stuff info structure into pointer and store in child of main base

make_pointer,unseen
info={wid:wid,program:entry,$
      wname:wname,wdesc:wdesc,$
      saved:saved,ny:ny,nx:nx}

set_pointer,unseen,info,/no_copy
child=widget_info(wbase,/child)
widget_control,child,set_uvalue=unseen

xmanager,'xprogram_add',wbase,group=group,/modal
if xalive(wbase) then xmanager

info=get_pointer(unseen,/no_copy)
free_pointer,unseen
if datatype(info) eq 'STC'  then saved=info.saved else saved=0
if saved then prog_id=info.program.prog_id
xshow,group
 
return & end


