	FUNCTION ZONE_DEF_ID, MODE, LABEL
;+
; Project     :	SOHO - CDS
;
; Name        :	ZONE_DEF_ID()
;
; Purpose     :	Associates zone ID's with widget event values.
;
; Explanation :	When used in conjunction with the pulldown menu defined by the
;		data file newdata_selection.pdm, this function can be used to
;		determine the unique zone ID.
;
; Use         :	ZONE_ID = ZONE_DEF_ID( MODE  [, LABEL ] )
;
; Inputs      :	MODE = Four character code for the zone.  This is the first
;		       four characters after the NDATA_ in the user value
;		       defined in the pulldown menu file newdata_selection.pdm.
;		       Arrays are allowed.
;
;		This routine also uses the input file zone_definitions.dat in
;		the same directory as newdata_selection.pdm ($CDS_PLAN_TECH).
;		This is a text file, with each line having the following
;		format:
;
;			Zone ID:	I3, 1X		(3 chars & a space)
;			Mode:		A4, 1X		(4 chars & a space)
;			Label:		A		(unlimited # of chars)
;
;		The modes in this file must match those in the file
;		newdata_selection.pdm.  For every item in newdata_selection.pdm
;		there must be a corresponding item in zone_definitions.dat,
;		although the converse does not necessarily have to be true.
;
;		It is not necessary that the zone IDs match the order of the
;		line in either zone_definitions.dat or newdata_selection.pdm.
;		However, the zone ID must be uniquely associated with a given
;		zone over the lifetime of the mission; i.e. zone ID number 5
;		will always be associated with active regions no matter how the
;		list of zones is reorganized.
;
; Opt. Inputs :	None.
;
; Outputs     :	The result of the function is the unique zone ID associated
;		with the zone.  If no match is found, the number 0 is returned
;		instead.
;
; Opt. Outputs:	LABEL = The label associated with the ID number.
;
; Keywords    :	None.
;
; Calls       :	CONCAT_DIR, DATATYPE
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Category    :	Planning, technical.
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 28 March 1995
; Modified    :	Version 1, William Thompson, GSFC, 28 March 1995
;               Version 1.1, Dominic Zarro, GSFC, 25 October 1995
;               - Added check for scalar input ID. IDL version 4 seems 
;               to have an erratic problem with statements like A(W)=C, 
;               when A is initialized to a scalar string and W is a vector.
;               Version 2, Zarro (SAC/GSFC) - added check for ZONE Def file
;
; Version     :	Version 2
;-
;
	ON_ERROR, 2
;
;  Make arrays to hold the output IDs and labels.
;
	SZ = SIZE(MODE)
	IF SZ(0) EQ 0 THEN BEGIN
		ID = [0]
		LABEL = ['']
	END ELSE BEGIN
		ID = INTARR(N_ELEMENTS(MODE))
		ID = REFORM(ID, SZ(1:SZ(0)), /OVERWRITE)
		LABEL = STRARR(N_ELEMENTS(MODE))
		LABEL = REFORM(LABEL, SZ(1:SZ(0)), /OVERWRITE)
	ENDELSE
;
;  Check the input parameter.
;
	IF N_PARAMS() LT 1 THEN BEGIN
		MESSAGE, 'Syntax:  Result = ZONE_DEF_ID( MODE  [, LABEL ])', $
			/CONTINUE
		RETURN, ID
	ENDIF
;
	IF DATATYPE(MODE,1) NE 'String' THEN BEGIN
		MESSAGE, 'The input parameter must be a character string', $
			/CONTINUE
		RETURN, ID
	ENDIF
;
;  Open the input file.
;
        ZFILE=CONCAT_DIR('$CDS_PLAN_TECH', 'zone_definitions.dat')
        CHK=LOC_FILE(ZFILE,COUNT=COUNT)
        IF COUNT EQ 0 THEN BEGIN
         MESSAGE,'Cannot locate "zone_definitions.dat"',/CONT
         RETURN,ID
        ENDIF

	OPENR, UNIT,ZFILE, /GET_LUN
;
;  Read in the file line by line, and update any zone IDs in the output array.
;
	ZONE_ID = 0
	ZONE_MODE = ''
	ZONE_LABEL = ''
	WHILE NOT EOF(UNIT) DO BEGIN
		READF, UNIT, ZONE_ID, ZONE_MODE, ZONE_LABEL,	$
			FORMAT='(I3,1X,A4,1X,A)'
		W = WHERE(MODE EQ ZONE_MODE, COUNT)
		IF COUNT GE 1 THEN BEGIN
			ID(W) = ZONE_ID
			LABEL(W) = ZONE_LABEL
		ENDIF
	ENDWHILE
;
;  Close the input file and return.
;
	FREE_LUN, UNIT
	IF SZ(0) EQ 0 THEN BEGIN
         ID=ID(0) & LABEL=LABEL(0)
        ENDIF
	RETURN, ID
	END
