	FUNCTION ZONE_DEF_LABEL, ID
;+
; Project     :	SOHO - CDS
;
; Name        :	ZONE_DEF_LABEL
;
; Purpose     :	Associates zone labels with ID numbers.
;
; Explanation :	Reads in the file zone_definitions.dat to retrieve labels.
;
; Use         :	ZONE_ID = ZONE_DEF_LABEL(ID)
;
; Inputs      :	ID = Unique ID number for the zone.
;
;		This routine also uses the input file zone_definitions.dat in
;		the same directory as newdata_selection.pdm ($CDS_PLAN_TECH).
;		This is a text file, with each line having the following
;		format:
;
;			Zone ID:	I3, 1X		(3 chars & a space)
;			Mode:		A4, 1X		(4 chars & a space)
;			Label:		A		(unlimited # of chars)
;
;		The modes in this file must match those in the file
;		newdata_selection.pdm.  For every item in newdata_selection.pdm
;		there must be a corresponding item in zone_definitions.dat,
;		although the converse does not necessarily have to be true.
;
;		It is not necessary that the zone IDs match the order of the
;		line in either zone_definitions.dat or newdata_selection.pdm.
;		However, the zone ID must be uniquely associated with a given
;		zone over the lifetime of the mission; i.e. the zone ID 5 will
;		always be associated with active regions no matter how the list
;		of zones is reorganized.
;
; Opt. Inputs :	None.
;
; Outputs     :	The result of the function is the label associated with the
;		unique zone ID.  If no match is found, the null string is
;		returned instead.
;
; Opt. Outputs:	None.
;
; Keywords    :	None.
;
; Calls       :	CONCAT_DIR, DATATYPE
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Category    :	Planning, technical.
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 28 March 1995
;
; Modified    :	Version 1, William Thompson, GSFC, 28 March 1995
;               Version 1.1, Dominic Zarro, GSFC, 25 October 1995
;               - Added check for scalar input ID. IDL version 4 seems 
;               to have an erratic problem with statements like A(W)=C, 
;               when A is initialized to a scalar string and W is a vector.
;               Version 2, Zarro (SAC/GSFC) - added check for ZONE Def file
;
; Version     :	Version 2
;-
;
	ON_ERROR, 2
;
;  Make an array to hold the output IDs.
;
	SZ = SIZE(ID)
	IF SZ(0) EQ 0 THEN LABEL = [''] ELSE BEGIN
		LABEL = STRARR(N_ELEMENTS(ID))
		LABEL = REFORM(LABEL, SZ(1:SZ(0)))
	ENDELSE
;
;  Check the input parameter.
;
	IF N_PARAMS() NE 1 THEN BEGIN
		MESSAGE, /CONTINUE, 'Syntax:  Result = ZONE_DEF_LABEL(ID)'
		RETURN, LABEL
	ENDIF
;
	TYPE = DATATYPE(ID,2)
	IF (TYPE EQ 0) OR (TYPE GT 3) THEN BEGIN
		MESSAGE, 'The input parameter must be an integer', /CONTINUE
		RETURN, LABEL
	ENDIF
;
;  Open the input file.
;
        ZFILE=CONCAT_DIR('$CDS_PLAN_TECH', 'zone_definitions.dat')
        CHK=LOC_FILE(ZFILE,COUNT=COUNT)
        IF COUNT EQ 0 THEN BEGIN
         MESSAGE,'Cannot locate "zone_definitions.dat"',/CONT
         RETURN,LABEL
        ENDIF

	OPENR, UNIT, ZFILE, /GET_LUN
;
;  Read in the file line by line, and update any zone IDs in the output array.
;
	ZONE_ID = 0
	ZONE_MODE = ''
	ZONE_LABEL = ''
	WHILE NOT EOF(UNIT) DO BEGIN
		READF, UNIT, ZONE_ID, ZONE_MODE, ZONE_LABEL,	$
			FORMAT='(I3,1X,A4,1X,A)'
		W = WHERE(ID EQ ZONE_ID, COUNT)
		IF COUNT GE 1 THEN LABEL(W) = ZONE_LABEL
	ENDWHILE
;
;  Close the input file and return.
;
	FREE_LUN, UNIT
	IF SZ(0) EQ 0 THEN LABEL=LABEL(0)
	RETURN, LABEL
	END
