;+
; Project     : SOHO - CDS     
;                   
; Name        : DISP_DEFERRED
;               
; Purpose     : Display contents of a deferred command store dump.
;               
; Explanation : When a special packet is detected in the CDS telemetry with the
;               format id '04'x, the contents relate to a dump of the deferred
;               command store and they can be displayed in the appropriate
;               widget by this routine. The widgets used are set up in the
;               routine SP_DEF_MENU.
;               
; Use         : disp_deferred, packet, counter
;    
; Inputs      : packet  - an int(141) array containing packet data, read by the
;                         routine read_sp_body
;
;               counter - special packet counter (byte). If it has the MSB set
;                         it is the last packet in the dump.
;               
; Opt. Inputs : None
;               
; Outputs     : None
;               
; Opt. Outputs: None
;               
; Keywords    : None
;
; Calls       : None
;               
; Restrictions: Restricted to use within the EGSE science analysis programs.
;               Uses internal COMMON block for passing the number of packets
;               received in the dump so far and for the widget id's within
;               which to display the results.
;               
; Side effects: Common variable sp_def_cnt is set to -1 if this is the last
;               packet in the dump.
;               
; Category    : EGSE
;               
; Prev. Hist. : From a PC routine by M K Carter
;
; Written     : EGSE version by C D Pike, RAL, 7-Oct-93
;               
; Modified    : 
;
; Version     : Version 1, 7-Oct-93
;-            

PRO disp_deferred, packet, sp_pkt_counter

;
;  COMMON block for communication of widget id's and counter
;
@SpecialWidgets.com

;
;  output format can be hex or decimal
;
if special_format eq 1 then fmt = '(6(1x,i5))' else fmt = '(6(1x,z4))'

;
;  if this is the first packet in the dump, then display header information
;
if sp_def_cnt eq 0 then begin
   if special_format eq 1 then begin
     widget_control,sp_def_exp(0),set_val=string((packet(0) and '0F'x),form=fmt)
     widget_control,sp_def_exp(2),set_val=string(unsign(packet(3)),form=fmt)
     widget_control,sp_def_exp(3),set_val=string(unsign(packet(4)),form=fmt)
     widget_control,sp_def_exp(4),set_val=string(unsign(packet(5)),form=fmt)
     widget_control,sp_def_exp(5),set_val=string(unsign(packet(6)),form=fmt)
   endif else begin
     hexout,sp_def_exp(0), (packet(0) and '0F'x)
     hexout,sp_def_exp(2), packet(3)
     hexout,sp_def_exp(3), packet(4)
     hexout,sp_def_exp(4), packet(5)
     hexout,sp_def_exp(5), packet(6)
   endelse

;
;  output the times
;  s/c time
;
   temp = LONG(packet(1:2),0,1)
   dt = utc2str(tai2utc(double(temp(0))))
   widget_control,sp_def_exp(1),set_val=strmid(dt,11,10)

;
;  earliest time in store
;
   temp = LONG(packet(7:8),0,1)
   if temp(0) eq 0 then begin
      dt = '           00:00:00.0'
   endif else begin
      dt = utc2str(tai2utc(double(temp(0))))
   endelse
   widget_control,sp_def_exp(6),set_val=strmid(dt,11,10)

;
;  latest time in store
;
   temp = LONG(packet(9:10),0,1)
   if temp(0) eq 0 then begin
      dt = '           00:00:00.0'
   endif else begin
      dt = utc2str(tai2utc(double(temp(0))))
   endelse
   widget_control,sp_def_exp(7),set_val=strmid(dt,11,10)
;
;  for info
;
   widget_control,sp_def_exp(8),set_val=' ',/append
   widget_control,sp_def_exp(8),set_val='------  Packet 0  ------ ',/append
   widget_control,sp_def_exp(8),set_val=' ',/append

;
;  now dump the rest of the data
;
   for i=0,31 do begin
      if special_format eq 1 then begin
         widget_control,sp_def_exp(8), /append,$
                set_val=string(unsign(packet(11+i*4:11+3+i*4)),form=fmt)
      endif else begin
         hexout,sp_def_exp(8), packet(11+i*4:11+3+i*4),/append
      endelse
   endfor
   if special_format eq 1 then begin
      widget_control,sp_def_exp(8), /append,$
             set_val=string(unsign(packet(139:140)),form=fmt)
   endif else begin
      hexout,sp_def_exp(8), packet(139:140),/append
   endelse


;
;  Nth packet. This is not the first packet, so it contains pure dump data
;  ==========
;
endif else begin
;
;  for info
;
   widget_control,sp_def_exp(8),set_val=' ',/append
   txt = string((sp_pkt_counter and '7f'x),format='(i1)')
   txt = '----- Packet '+txt+' -----'
   widget_control,sp_def_exp(8),set_val=txt,/append
   widget_control,sp_def_exp(8),set_val=' ',/append


   for i=0,34 do begin
      if special_format eq 1 then begin
         widget_control,sp_def_exp(8),/append,$
                set_val=string(unsign(packet(i*4:(3+i*4))),form=fmt)
      endif else begin
         hexout,sp_def_exp(8),packet(i*4:(3+i*4)),/append
      endelse
   endfor
   if special_format eq 1 then begin
      widget_control,sp_def_exp(8),/append,$
             set_val=string(unsign(packet(140)),form=fmt)
   endif else begin
      hexout,sp_def_exp(8),packet(140),/append
   endelse

endelse

;
;  last packet received (may have been first as well) so reset local counter
;
if (sp_pkt_counter and '80'x)  eq '80'x then begin
   sp_def_cnt = -1
   widget_control,sp_def_exp(8),set_val=' ',/append
   widget_control,sp_def_exp(8),set_val='======== END ======== ',/append
   widget_control,sp_def_exp(8),set_val=' ',/append
endif

end

