;+
; Project     : SOHO - CDS     
;                   
; Name        : GIS_EXT_PLOT
;               
; Purpose     : Extract GIS science stream data into its defined windows. 
;               
; Explanation : Extract GIS science data into the windows requested for the
;               observation. This module decodes the science data stream 
;               as 2 bytes per pixel
;               
; Use         : IDL> gis_ext_plot, buffer, raster, gis_plot, exp_no, msg_widg
;
;               but only designed for use internally to the EGSE science
;               analysis program.
;
; Inputs      : buffer   - a byte array of raw telemetry science data
;               raster   - structure of type GISrasterhdr which contains the
;                          data windows definition.
;               gis_plot - 3-d array (pixel, window, exposure number) in 
;                          which to store the extracted data.
;               exp_no   - the exposure number these data represent.
;               msg_widg - widget id for the widget holding information
;                          messages.
;
; Opt. Inputs : None
;               
; Outputs     : gis_plot - as above on input.
;               
; Opt. Outputs: None
;               
; Keywords    : None
;
; Calls       : None
;               
; Restrictions: Usually for internal use of EGSE science analysis programs.
;               
; Side effects: None
;               
; Category    : EGSE
;               
; Prev. Hist. : EM version by Arnulf, Oct 92
;
; Written     : FM version by C D Pike, RAL, 7-Oct-93
;               
; Modified    : New data structure, CDP, 14-Dec-1993
;
; Version     : Version 2, 14-Dec-93
;-            

PRO GIS_Ext_plot, sci_buffer, raster, gis_plot, exp_no, msg_widg

;
;  issue encouragement in message widget
;
WIDGET_CONTROL, msg_widg, /append, $
                set_val="Extracting GIS windows from science stream."

;
;  check we have valid data first
;
if (raster.NumDataWindows le 0) or (exp_no lt 0) then begin
   bell
   WIDGET_CONTROL, msg_widg, /append, $
                set_val="No valid header information for data extraction."
   return
endif

;
;  create temporary store
;
plot = INTARR(8192)
SciencePointer = LONG(0)

;
;  loop over all the windows defined and extract the data as needed
;
FOR J = 0, raster.NumDataWindows-1 DO BEGIN

   Start  = raster.GISWin(J).Start
   Length = raster.GISWin(J).Length
;   mess   = 'Extracting window '+STRCOMPRESS(STRING(J+1), /REMOVE_ALL)+ $
;      ' of '+STRCOMPRESS(STRING(raster.NumDataWindows), /REMOVE_ALL)+ $
;      ' Start/Length = '+string(Start,"/",Length,format="(i4,a,i4)")
;   WIDGET_CONTROL,msg_widg,set_val=mess, /append
                            
;
;  extract data
;
   ScienceLength = LONG(Length)
   temp = sci_buffer( SciencePointer:SciencePointer+ScienceLength-1)
   gis_plot(0:length-1,j,exp_no) = byteswap(temp)

;
;  increment pointer for next time
;
   SciencePointer = SciencePointer + ScienceLength

ENDFOR

END

