;+
; Project     : SOHO - CDS     
;                   
; Name        : HEXOUT
;               
; Purpose     : Formats an unsigned 16 bit integer for hexadecimal output.
;               
; Explanation : Many 16 bit numbers in the CDS telemetry are held in IDL as
;               signed INT 16-bit numbers but are actually meant to be treated
;               as unsigned integers.  Printing them out in hex format will
;               give an error for those where the sign bit is set.  This 
;               routine avoids that messiness.
;               
; Use         : hexout, widgetID, int16 [, /append, format=format]     
;    
; Inputs      : widget_id  - id of widget where output is to be directed
;               int16      - the IDL 16-bit integer
;               
; Opt. Inputs : None
;               
; Outputs     : None
;               
; Opt. Outputs: None
;               
; Keywords    : append - if set the output id appended to the widget, otherwise
;                        output overwrites the widget contents.
;               format - character string specifying the format for the output
;                        useful for multi-line output which widget control
;                        is very bad at.  eg format='(5(4(2z2.2,2x),/))'
;                        will write 5 lines of 4 values.
;
; Calls       : None
;               
; Restrictions: Only for use with widget applications.
;               
; Side effects: None
;               
; Category    : EGSE
;               
; Prev. Hist. : Originally from M K Carter
;
; Written     : C D Pike, RAL, 22-Oct-93
;               
; Modified    : Version 1, C D Pike, RAL, 22-Oct-93
;		Version 2, William Thompson, GSFC, 12 November 1993.
;			Added /NTOHS to BYTEORDER for platform independence.
;               Version 3, CDP added format keyword
;
; Version     : Version 3, 2 December 1993.
;-            

PRO hexout, widgetID, int16, append=append, format=format     

;
;  byteswap to get display order correct
;
bs16 = int16
byteorder,bs16,/ntohs

;
;  pick up format required, or use default
;
if not keyword_set(format) then fmt = '(6(2z2.2,2x))' else fmt = format

;
;  convert output to byte array and then output that in hex format
;  
if keyword_set(append) then begin
   WIDGET_CONTROL, widgetID, $
                SET_VALUE=STRING( $
                BYTE(bs16,0,2*N_ELEMENTS(bs16)),$
                FORMAT=fmt),$
                /APPEND
endif else begin
   WIDGET_CONTROL, widgetID, $
                SET_VALUE=STRING( $
                BYTE(bs16,0,2*N_ELEMENTS(bs16)),$
                FORMAT=fmt)
endelse

END

