;+
; Project     : SOHO - CDS     
;                   
; Name        : PLOT_GIS
;               
; Purpose     : Plot the GIS spectra from all four detectors.
;               
; Explanation : Used within the EGSE science analysis programs this routine
;               plots the GIS data received.  All four channels are plotted.
;               If called with the INIT_GIS keyword then a dummy plot is
;               performed to initialise the plotting widget area.
;               
; Use         : plot_gis, buff, init_gis=init_gis
;    
; Inputs      : buff      -  the data array (8192)          
;               
; Opt. Inputs : None
;               
; Outputs     : None
;               
; Opt. Outputs: None
;               
; Keywords    : init    -  if set a dummy plot is performed to initialise the
;                            plot window
;               xmin    -  first data point in input array to plot
;               xmax    -  last data point in input array to plot
;
;
; Calls       : None
;               
; Restrictions: Only for use within EGSE science analysis programs.
;               
; Side effects: None
;               
; Category    : EGSE
;               
; Prev. Hist. : EM version by Arnulf, Oct-92
;
; Written     : FM version by C D Pike, RAL, 7-Oct-93
;               
; Modified    : Major update on plot capabilities, CDP, 15-Dec-1993
;               Reverse data array prior to plotting, CDP, 7-Mar-94
;               Reversing data wasn't correct, now done in gis_deselect.
;                                                       CDP, 9-Mar-94
;
; Version     : Version 3, 9-Mar-94
;-            

PRO Plot_GIS, buf, INIT=INIT, xmin=xmin, xmax=xmax

;
;  COMMON block for message widget id and display choices
;
@MainWidgets.com
@GISWidgets.com
@GISData.com
@Cal_Lines.com
@CalFits.com


;
;  retrieve the window in use and set focus to it
;

;
;  set number of plots in x-direction per page
;
if gis_data_str.gis_select eq 'A' then begin
   if !d.name ne 'PS' then begin
      WIDGET_CONTROL, gis_awidg, GET_VALUE = window
      WSET, window
   endif
   if gis_data_str.agis_plot_choice(4) then begin
      if !d.name eq 'PS' then !p.multi = [0,2,2] else  !p.multi=[0,4,1] 
      plot_band = [1,1,1,1]
   endif else begin
      !P.MULTI=[0,fix(total(gis_data_str.agis_plot_choice(0:3))),1]
      plot_band = gis_data_str.agis_plot_choice(0:3)
   endelse
endif else begin
   if !d.name ne 'PS' then begin
      WIDGET_CONTROL, gis_bwidg, GET_VALUE = window
      WSET, window
   endif
   if gis_data_str.bgis_plot_choice(4) then begin
      if !d.name eq 'PS' then !p.multi = [0,2,2] else  !p.multi=[0,4,1] 
      plot_band = [1,1,1,1]
   endif else begin
      !P.MULTI=[0,fix(total(gis_data_str.bgis_plot_choice(0:3))),1]
      plot_band = gis_data_str.bgis_plot_choice(0:3)
   endelse
endelse

;
;  define range of pixels to plot
;
if not keyword_set(xmin) then xmin = 0
if not keyword_set(xmax) then xmax = 2047
 
;
;  loop through the plots
;
FOR p=1,4 DO BEGIN

;
;  define range (initialise here cos reset if wavelengths used)
;
   x_array = indgen(xmax-xmin+1)+xmin

   if plot_band(p-1) then begin
      title = string('GIS band #',p,format='(a,i1)')

      if gis_data_str.gis_plot_wave then begin
         xtitle = 'Wavelength (A)' 
         x_array = pix2wave('GIS'+string(p,form='(i1)'),x_array, /limit)
      endif else begin
         xtitle = 'Pixels' 
      endelse

      xr = [min(x_array),max(x_array)]
;
;  if init requested then do dummy plot
;
      IF KEYWORD_SET(INIT) THEN BEGIN
         if gis_data_str.gis_plot_linear then begin
            PLOT,x_array,indgen(n_elements(x_array)), $
                    /NODATA, CHARSIZE=1.75,title=title,xtitle=xtitle, $
                    xrange=xr, xstyle=1
         endif else begin
            PLOT_IO,x_array,indgen(n_elements(x_array))+1, $
                    /NODATA, CHARSIZE=1.75,title=title,xtitle=xtitle, $
                    xrange=xr, xstyle=1
         endelse
      ENDIF $
;
;  do normal plot
;
      ELSE BEGIN
         y_array = buf(xmin+(p-1)*2048:xmax+(p-1)*2048)
         if max(y_array) eq min(y_array) then begin
            plot,x_array,indgen(n_elements(x_array)), $
                    /nodata, charsize=1.75,title='NO DATA',xtitle=xtitle, $
                    xrange=xr, xstyle=1
            widget_control,gis_msg_widg,$
                      set_val=string('GIS detector #',p,$
                      ':   Data max = min, so no data plotted.',form='(a,i1,a)')
         endif else begin
            if gis_data_str.gis_plot_linear then begin
               PLOT, x_array, y_array, XRANGE=xr, xstyle=1,$
                           CHARSIZE=1.75,psym=10,title=title,xtitle=xtitle 
            endif else begin
               PLOT_IO, x_array, y_array, XRANGE=xr, xstyle=1,$
                           CHARSIZE=1.75,psym=10,title=title,xtitle=xtitle 
            endelse
;
;  overlay lines if requested
;
            if disp_gis_lines then begin
               n = where(strmid(gis_lines.band,3,1) eq p)
               if n(0) ge 0 then begin
                  if gis_data_str.gis_plot_wave then begin
                     xst = pix2wave(gis_lines(n(0)).band,$
                                    gis_lines(n).x_pos-10, /limit)
                     xe  = pix2wave(gis_lines(n(0)).band,$
                                    gis_lines(n).x_pos+10, /limit)
                  endif else begin
                     xst = gis_lines(n).x_pos-10
                     xe  = gis_lines(n).x_pos+10
                  endelse
                  for i=0,n_elements(n)-1 do begin
                     oplot,[xst(i),xst(i)],[0,100000], linestyle=2
                     oplot,[xe(i),xe(i)],  [0,100000], linestyle=2
                  endfor
               endif
            endif
         endelse
      ENDELSE
;
;  save scaling
;
      if gis_data_str.gis_select eq 'A' then begin
         gax = !x & gay = !y & gad = !d
      endif else begin
         gbx = !x & gby = !y & gbd = !d
      endelse

   endif
ENDFOR

;
;  reset plotting variables
;
!P.MULTI= 0

END
