;+
; Project     : SOHO - CDS     
;                   
; Name        : PLOT_MONITOR
;               
; Purpose     : Plots monitor data from EGSE calibration program.
;               
; Explanation : For the calibration of CDS a 'monitor' mode might prove
;               useful.  This consists of a simple plot of the total count
;               received in an exposure.  The idea is to give fast feedback
;               to the persons setting up the instrument and source.
;               
; Use         : Only from within the calibration program.
;    
; Inputs      : None - done via common block
;               
; Opt. Inputs : None
;               
; Outputs     : None
;               
; Opt. Outputs: None
;               
; Keywords    : Hard - just plot hardcopy of complete current data.
;
; Calls       : PS
;               PSPLOT
;               EXT_RANGE()
;               CIRCLE_SYM
;
; Common      : Monitor.com
;               
; Restrictions: None
;               
; Side effects: None
;               
; Category    : EGSE
;               
; Prev. Hist. : None
;
; Written     : C D Pike, RAL, 10-Jan-94
;               
; Modified    : Add hardcopy option,  CDP, 1-Mar-94
;               Add mechanism data plotting, CDP, 6-May-94
;               Use ps/psplot and ext_range.  CDP, 23-Jun-94
;
; Version     : Version 4, 23-Jun-94
;-            

pro  plot_monitor, hard=hard

;
;  common blocks
;
@Monitor.com

;
;  which window is the draw widget?
;
widget_control, mon_draw, get_val=wind
wset, wind

;
;  define filled circle as plot symbol
;
circle_sym,/fill

;
;  plotting for mechanisms
;
if m_monitor_mode then begin

;
;  deal with hardcopy version if requested
;
   if keyword_set(hard) then ps
   !p.multi = [0,2,3]

;
;  number of points to plot
;

   n = slitp_count

;
;  first time in ...
;
   if n eq 0 then begin
      plot,[0.9*slitp_monitor(0),1.1*slitp_monitor(0)],$
           [0.9*mirrp_monitor(0),1.1*mirrp_monitor(0)],$
           xtit='Slit position', ytit='Mirror position',/nodata, charsize=2.0
      oplot,[slitp_monitor(0),slitp_monitor(0)],[mirrp_monitor(0),mirrp_monitor(0)],$
           psym=8

      plot,[0.9*opsl_monitor(0),1.1*opsl_monitor(0)],$
           [0.9*opsr_monitor(0),1.1*opsr_monitor(0)],$
           xtit='OPSL position', ytit='OPSR position',/nodata, charsize=2.0
      oplot,[opsl_monitor(0),opsl_monitor(0)],[opsr_monitor(0),opsr_monitor(0)],$
           psym=8

      plot,[-1,1], [0.9*opsl_monitor(0),1.1*opsl_monitor(0)],$
           xtit='Exposure number', ytit='OPSL position',/nodata, charsize=2.0
      oplot,[0,0],[opsl_monitor(0),opsl_monitor(0)],$
           psym=8

      plot,[-1,1], [0.9*opsr_monitor(0),1.1*opsr_monitor(0)],$
           xtit='Exposure number', ytit='OPSR position',/nodata, charsize=2.0
      oplot,[0,0],[opsr_monitor(0),opsr_monitor(0)],$
           psym=8

      plot,[-1,1], [0.9*slitp_monitor(0),1.1*slitp_monitor(0)],$
           xtit='Exposure number', ytit='Slit position',/nodata, charsize=2.0
      oplot,[0,0],[slitp_monitor(0),slitp_monitor(0)],$
           psym=8

      plot,[-1,1], [0.9*mirrp_monitor(0),1.1*mirrp_monitor(0)],$
           xtit='Exposure number', ytit='Mirror position',/nodata, charsize=2.0
      oplot,[0,0],[mirrp_monitor(0),mirrp_monitor(0)],$
           psym=8
   endif else begin
      
      xmin = (ext_range(slitp_monitor(0:slitp_count),10))(0)
      xmax = (ext_range(slitp_monitor(0:slitp_count),10))(1)
      ymin = (ext_range(mirrp_monitor(0:mirrp_count),10))(0)
      ymax = (ext_range(mirrp_monitor(0:mirrp_count),10))(1)

      plot,slitp_monitor(0:slitp_count),mirrp_monitor(0:mirrp_count),/ynoz,$
            xtit = 'Slit position', ytit = 'Mirror position', psym=8, $
            charsize=2.0,xrange=[xmin,xmax],yrange=[ymin,ymax]

      xmin = (ext_range(opsl_monitor(0:opsl_count),10))(0)
      xmax = (ext_range(opsl_monitor(0:opsl_count),10))(1)
      ymin = (ext_range(opsr_monitor(0:opsr_count),10))(0)
      ymax = (ext_range(opsr_monitor(0:opsr_count),10))(1)
      plot,opsl_monitor(0:opsl_count),opsr_monitor(0:opsr_count),/ynoz,$
            xtit = 'OPSL position', ytit = 'OPSR position', psym=8, $
            charsize=2.0,xrange=[xmin,xmax],yrange=[ymin,ymax]

      xmin = (ext_range(indgen(opsl_count+1),10))(0)
      xmax = (ext_range(indgen(opsl_count+1),10))(1)
      ymin = (ext_range(opsl_monitor(0:opsl_count),10))(0)
      ymax = (ext_range(opsl_monitor(0:opsl_count),10))(1)
      plot,indgen(opsl_count+1),opsl_monitor(0:opsl_count),/ynoz,$
            xtit = 'Exposure number', ytit = 'OPSL position', psym=8, $
            charsize=2.0,xrange=[xmin,xmax],yrange=[ymin,ymax]

      xmin = (ext_range(indgen(opsr_count+1),10))(0)
      xmax = (ext_range(indgen(opsr_count+1),10))(1)
      ymin = (ext_range(opsr_monitor(0:opsr_count),10))(0)
      ymax = (ext_range(opsr_monitor(0:opsr_count),10))(1)
      plot,indgen(opsr_count+1),opsr_monitor(0:opsr_count),/ynoz,$
            xtit = 'Exposure number', ytit = 'OPSR position', psym=8, $
            charsize=2.0,xrange=[xmin,xmax],yrange=[ymin,ymax]

      xmin = (ext_range(indgen(slitp_count+1),10))(0)
      xmax = (ext_range(indgen(slitp_count+1),10))(1)
      ymin = (ext_range(slitp_monitor(0:slitp_count),10))(0)
      ymax = (ext_range(slitp_monitor(0:slitp_count),10))(1)
      plot,indgen(slitp_count+1),slitp_monitor(0:slitp_count),/ynoz,$
            xtit = 'Exposure number', ytit = 'Slit position', psym=8, $
            charsize=2.0,xrange=[xmin,xmax],yrange=[ymin,ymax]

      xmin = (ext_range(indgen(mirrp_count+1),10))(0)
      xmax = (ext_range(indgen(mirrp_count+1),10))(1)
      ymin = (ext_range(mirrp_monitor(0:mirrp_count),10))(0)
      ymax = (ext_range(mirrp_monitor(0:mirrp_count),10))(1)
      plot,indgen(mirrp_count+1),mirrp_monitor(0:mirrp_count),/ynoz,$
            xtit = 'Exposure number', ytit = 'Mirror position', psym=8, $
            charsize=2.0,xrange=[xmin,xmax],yrange=[ymin,ymax]

   endelse
;
;  print hardcopy version
;
   if keyword_set(hard) then psplot
   !p.multi = 0
   return
endif


;
;  must be intensity monitoring so which instrument?
;
if v_monitor_mode then begin
   if vm_count lt 0 then return
   data = vds_monitor(0:vm_count)
   text = 'VDS monitor' 
   n = vm_count
endif else begin
   if gm_count lt 0 then return
   data = gis_monitor(0:gm_count)
   text = 'GIS monitor'
   n = gm_count
endelse

 
;
;  first time in plot axes and overplot first data point
;
if n eq 0 then begin
   plot,[0,19], [0,1.5*data(0)],/nodata, xtit='Exposure number.', $
                                         ytit = 'Total count.'
   oplot,[1,1],[data(0),data(0)],psym=1
   mxmax = 19
   mymax = 1.5*data(0)
   monx = !x & mony = !y & monp = !p 
   return
endif

;
;  are we within limits of plot?
;
redraw = 0
if (mymax - data(n)) lt 0.1*mymax then begin
   mymax = data(n) * 1.5
   redraw = 1 
endif

if (mxmax - n) lt 5 then begin
   mxmax = mxmax + (20 > 0.2*mxmax)
   redraw = 1
endif

;
;  replot whole lot or just add latest
;
x = indgen(mxmax)
if redraw then begin
   plot, [0,x(mxmax-1)],[0,mymax],/nodata, xtit='Exposure number.', $
                                           ytit = 'Total count.'
   for i=0,n do begin
      oplot,[i+1,i+1],[data(i),data(i)], psym=1
   endfor
   monx = !x & mony = !y & monp = !p 
endif else begin
   !x = monx & !y = mony & !p = monp
   oplot,[n+1,n+1],[data(n),data(n)], psym=1
endelse

;
;  deal with hardcopy version if requested
;
if keyword_set(hard) then begin
   ps
   plot, [0,x(mxmax-1)],[0,mymax],/nodata, xtit='Exposure number.', $
                                           ytit = 'Total count.',$
                                           title=text+' Plotted: '+systime()
   if n ge 0 then begin
      for i=0,n do begin
         oplot,[i+1,i+1],[data(i),data(i)], psym=1
      endfor
   endif
   psplot
endif


end
