;+
; Project     : SOHO - CDS     
;                   
; Name        : RD_VDS_LINES
;               
; Purpose     : Read the VDS calibration line list into the line structure.
;               
; Explanation : Reads a line list file into the structure required by the 
;               calibration procedure.  The line list file is assumed
;               to have the following contents:
;               Line 1 - header text
;               Line 2 - blank
;               Line 2 - line identification, wavelength, order, band, pixel
;               .
;               .
;               .
;               Line N - ditto
;
;               
; Use         : IDL> rd_vds_lines, order
;    
; Inputs      : order  - which selection of orders to read:
;                         'F' = first order
;                         'S' = second order
;                         'A' = all or both
;               
; Opt. Inputs : None
;               
; Outputs     : None
;               
; Opt. Outputs: None
;               
; Keywords    : None
;
; Calls       : None
;               
; Restrictions: EGSE calibration use only.
;               
; Side effects: None
;               
; Category    : EGSE
;               
; Prev. Hist. : None
;
; Written     : C D Pike, 6-Jan-94
;               
; Modified    : 
;
; Version     : Version 1, 6-Jan-94
;-            

pro rd_vds_lines, order

;
;  insert common blocks to pass results
;
@Cal_Lines.com
@VDSWidgets.com

;
;  trap no parameters
;
if n_params() eq 0 then order = 'A'

;
;  set selected order
;
if order eq 'F' then req_order = 1
if order eq 'S' then req_order = 2
if order eq 'A' then req_order = 3

;
;  set up structure for line details
;
vlr = {vds_lines_rec, $
          id: ' ',$
          lam: 0.0, $
          order: 0, $
          x_pos: 0, $
          y_pos: 0}

vds_lines = replicate(vlr, 50)

;
;  open file for reading.
;
ff = findfile(concat_dir('$CDS_PLAN_TECH','VDS_cal_lines'), count=count)
if count eq 0 then begin
   bell
   flash_msg, vds_msg_widg,'Error: no VDS lines file found.'
   disp_vds_lines = 0
   return
endif else begin
   openr,lun,ff(0),/get_lun

;
;  read header and spacer lines
;
   header = ' '
   readf,lun,header
   text = ''
   readf,lun,text

;
;  the third token in the header must be the line id number (ie usually
;  entered as "Line list nn").  If not, return.
;
tok = str2arr(header,' ',/nomult)

if not valid_num(tok(2),/int) then begin
   bell
   flash_msg, info_line,'** Invalid line list ID number in file.**'
   free_lun,lun
   return
endif

;
;  read first line of data to check id format
;
   readf,lun,text
   in = str2arr(text,' ',/nomult)
   case 1 of
      (in(3) eq '(1)') or (in(3) eq '(2)'): offset = 1
      (in(2) eq '(1)') or (in(2) eq '(2)'): offset = 0
      else: begin
               flash_msg, vds_msg_widg, 'Error in line file format.'
               free_lun,lun
               num_windows = 0
               return
            end
   endcase      
;
;  all ok so start again
;
   openr,lun,ff(0),/get_lun

;
;  read header and spacer lines
;
   header = ' '
   readf,lun,header
   text = ''
   readf,lun,text
      

;
;  read data, assumed in form: identification, wavelength, order, band, pixel
;  set defaults for the rest.  First assumption is that the id is of the form
;  Fe XXV ie with a space but allow for it with no space - otherwise error.
;
   id = ' '
   num = 0
   while not eof(lun) do begin
      readf,lun,text
      in = str2arr(text,' ',/nomult)
      if in(0) ne '' then begin
         if (req_order eq strmid(in(2+offset),1,1)) or req_order eq 3 then begin
            vds_lines(num).x_pos   = in(4+offset)
            vds_lines(num).lam     = in(1+offset)
            vds_lines(num).order   = strmid(in(2+offset),1,1) 
            obs_lam = vds_lines(num).lam * vds_lines(num).order
            vds_lines(num).y_pos   = (obs_lam lt 400.) * 128 + 22
            if offset eq 0 then begin
               vds_lines(num).id  = in(0)
            endif else begin
               vds_lines(num).id  = in(0)+' '+in(1)
            endelse            
            num = num + 1
         endif
      endif
   endwhile
   free_lun,lun

;
;  tidy the structure
;
   num_windows = max(where(vds_lines.x_pos gt 0)) + 1
endelse

;
;  check for most basic of errors
;
big = where(vds_lines.x_pos gt 1023)
if big(0) ge 0 then begin
   flash_msg,vds_msg_widg,'Line pixel value too large for NIS'
endif

;
;  set flag to inform display is required
;
disp_vds_lines = 1

end
