;+
; Project     : SOHO - CDS     
;                   
; Name        : READ_EXP
;               
; Purpose     : Read and display the exposure packet header. 
;               
; Explanation : The exposure packet comes after the raster header packet and
;               before the science data packets per se.  The contents are
;               identical for both the VDS and GIS.  This routine reads a 
;               structure of type ExpHeaderDef as defined in the main program
;               SCI_ANA and displays the contents in the relevant header
;               information widgets. The COMMON variable 'header_select'
;               is used to flag which of the header display widgets is
;               currently active and what choice of display was chosen.  
;               The values are as follows:
;               header_select = -2  --> display VDS raster header only
;                             = -1  --> display VDS raster + exposure headers
;                             =  1  --> display GIS raster header only
;                             =  2  --> display GIS raster + exposure headers
;               A choice of output formats is provided by the setting of the 
;               COMMON variable 'header_format' (=1 for decimal, =2 for hex).
;               
; Use         : read_exp, unit, Header, instrument
;    
; Inputs      : unit       - the logical unit from which to read the data.
;               Header     - the structure in which to load the data, hence 
;                            it defines the amount of data to be read.
;               instrument - the instrument identification.  This is actually
;                            the id given in the raster header packet and so 
;                            is '11'x for GIS and '12'x for VDS.
;               
; Opt. Inputs : None
;               
; Outputs     : Return value - Function returns a value of 1 for 
;                              success, -1 for failure.
;               Header       - as above for input.
;               
; Opt. Outputs: None
;               
; Keywords    : None
;
; Calls       : None
;               
; Restrictions: Only for use within EGSE science analysis program.
;               
; Side effects: None
;               
; Category    : EGSE
;               
; Prev. Hist. : EM version by Arnulf, Oct-92
;
; Written     : FM version by C D Pike, 11-Oct-93
;               
; Modified    : Insert conversion of exposure time to date/time string
;               CDP, 21/10/93
;
;               Change to v3.0 of packet format, CDP, 27-Oct-93
;               Update output formats, CDP, 28-Oct-93
;               Decode slit position, CDP, 2-Mar-94
;               Change exposure number display (1...n), fix solar X/Y display,
;                                                          CDP, 9-May-94
;
; Version     : Version 6, 2-May-94
;-            

FUNCTION read_exp, unit, Header, instrument

;
;  COMMON for communication of variables header_select and header_format
;  and date/time for each exposure.
;
@HeaderWidgets.com
@Packets.com

;
;  skip on error
;
ON_IOERROR, ERR_exp

;
;  read data from logical unit (realtime or replay)
;
READU,unit,Header

;
;  load individual widget id's depending on which header information widget
;  is currently visible
;
if header_select gt 0 then h = header_g_exp 
if header_select lt 0 then h = header_v_exp

;
; Byte swap all two byte values
;                                   
Header.SolarX            = ByteSwap (Header.SolarX)
Header.ExposureNumber    = ByteSwap (Header.ExposureNumber)
Header.OPSLCoordinate    = ByteSwap (Header.OPSLCoordinate)
Header.OPSRCoordinate    = ByteSwap (Header.OPSRCoordinate)
;
; deal with coded slit position
;
Header.SlitPosition      = ByteSwap (Header.SlitPosition)
slitpos_decode, Header.SlitPosition, slitbits, slitpos
Header.SlitPosition = slitpos

Header.SunSensorPitch    = ByteSwap (Header.SunSensorPitch)
Header.SunSensorYaw      = ByteSwap (Header.SunSensorYaw)
Header.SolarY            = ByteSwap (Header.SolarY)
Header.EventLevel        = ByteSwap (Header.EventLevel)
Header.StudyCounter      = ByteSwap (Header.StudyCounter)
Header.RasterCounter     = ByteSwap (Header.RasterCounter)
Header.ChkSum            = ByteSwap (Header.ChkSum)

;
;  convert time bytes to date/time string
;
fmt_cdhs_time, Header, date,time

;
;  save date/time
;

if instrument eq '11'x then begin
   if Header.ExposureNumber eq 0 then GIS_date_time(*) = ' '
   GIS_date_time(Header.ExposureNumber) = date + ' ' + time
endif
if instrument eq '12'x then begin
   if Header.ExposureNumber eq 0 then VDS_date_time(*) = ' '
   VDS_date_time(Header.ExposureNumber) = date + ' ' + time
endif

;
; Display the contents of the GIS header
;
if instrument eq '11'x and header_select eq 2 then begin
   widget_control,h(19),set_val=date
   widget_control,h(20),set_val=time
   widget_control,h(21),$
             set_val = fmt_vect([Header.ExposureNumber+1,total_exp],$
                          delim='/')
   if header_format eq 2 then begin
      hexout,h(24),Header.SolarX
      hexout,h(09),Header.OPSLCoordinate
      hexout,h(10),Header.OPSRCoordinate
      hexout,h(11),Header.SlitPosition
      hexout,h(22),Header.SunSensorPitch
      hexout,h(23),Header.SunSensorYaw
      hexout,h(25),Header.SolarY
      hexout,h(13),fix(Header.MirrorPosition)
      hexout,h(26),fix(Header.EventStatus)
      hexout,h(27),Header.EventLevel
      hexout,h(15),Header.StudyCounter
      hexout,h(16),Header.RasterCounter
   endif else begin
      fmt = '(5x,i5)'
      widget_control,h(24),$
             set_val=string(Header.SolarX,format=fmt)
      widget_control,h(09),$
             set_val=string(unsign(Header.OPSLCoordinate),format=fmt)
      widget_control,h(10),$
             set_val=string(unsign(Header.OPSRCoordinate),format=fmt)
      widget_control,h(11),$
             set_val=string((Header.SlitPosition),format=fmt)
      widget_control,h(22),$
             set_val=string(unsign(Header.SunSensorPitch),format=fmt)
      widget_control,h(23),$
             set_val=string(unsign(Header.SunSensorYaw),format=fmt)
      widget_control,h(25),$
             set_val=string(Header.SolarY,format=fmt)
      widget_control,h(13),$
             set_val=string(fix(Header.MirrorPosition),format=fmt)
      widget_control,h(26),$
             set_val=string(fix(Header.EventStatus),format=fmt)
      widget_control,h(27),$
             set_val=string(unsign(Header.EventLevel),format=fmt)
      widget_control,h(15),$
             set_val=string(unsign(Header.StudyCounter),format=fmt)
      widget_control,h(16),$
             set_val=string(unsign(Header.RasterCounter),format=fmt)
   endelse
endif

;
;  ... and the VDS
;
if instrument eq '12'x and header_select eq -1 then begin
   widget_control,h(21),set_val=date
   widget_control,h(22),set_val=time
   widget_control,h(23),$
             set_val = fmt_vect([Header.ExposureNumber+1,total_exp],$
                          delim='/')
   if header_format eq 2 then begin
      hexout,h(26),Header.SolarX
      hexout,h(09),Header.OPSLCoordinate
      hexout,h(10),Header.OPSRCoordinate
      hexout,h(11),Header.SlitPosition
      hexout,h(24),Header.SunSensorPitch
      hexout,h(25),Header.SunSensorYaw
      hexout,h(27),Header.SolarY
      hexout,h(13),fix(Header.MirrorPosition)
      hexout,h(28),fix(Header.EventStatus)
      hexout,h(29),Header.EventLevel
      hexout,h(15),Header.StudyCounter
      hexout,h(16),Header.RasterCounter
   endif else begin
      fmt = '(5x,i5)' 
      widget_control,h(26),$
             set_val=string(Header.SolarX,form=fmt)
      widget_control,h(09),$
             set_val=string(unsign(Header.OPSLCoordinate),form=fmt)
      widget_control,h(10),$
             set_val=string(unsign(Header.OPSRCoordinate),form=fmt)
      widget_control,h(11),$
             set_val=string((Header.SlitPosition),form=fmt)
      widget_control,h(24),$
             set_val=string(unsign(Header.SunSensorPitch),form=fmt)
      widget_control,h(25),$
             set_val=string(unsign(Header.SunSensorYaw),form=fmt)
      widget_control,h(27),$
             set_val=string(Header.SolarY,form=fmt)
      widget_control,h(13),$
             set_val=string(fix(Header.MirrorPosition),form=fmt)
      widget_control,h(28),$
             set_val=string(fix(Header.EventStatus),form=fmt)
      widget_control,h(29),$
             set_val=string(unsign(Header.EventLevel),form=fmt)
      widget_control,h(15),$
             set_val=string(unsign(Header.StudyCounter),form=fmt)
      widget_control,h(16),$
             set_val=string(unsign(Header.RasterCounter),form=fmt)
   endelse
endif

;
;  success and error returns
;
RETURN,1
ERR_exp: RETURN,-1

END
