;+
; Project     : SOHO - CDS     
;                   
; Name        : READ_GIS_RASTER()
;               
; Purpose     : Read the GIS raster header from a unit. 
;               
; Explanation : This function reads the packet header for a GIS raster.  It
;               is a structure of type GISrasterdef (see sci_ana.pro) and
;               consists of the science header structure (SciheaderDef) which
;               is common to GIS and VDS with additional items specific to the
;               GIS ie the number of data windows and the structure defining
;               the windows. This function calls swap_scihead which is used
;               to byte-swap the header variables and also to write to the
;               header information display widget if it is realized.
;               
; Use         : header = read_gis_raster(unit, gis_raster)
;    
; Inputs      : unit       - the logical unit from which to read data
;               
; Opt. Inputs : None
;               
; Outputs     : gis_raster - GIS raster header structure
;               Function value returned = -1 for error, +1 for read success
;               and -100 for number of windows exceeding the expected maximum.
;
; Opt. Outputs: None
;               
; Keywords    : None
;
; Calls       : swap_scihead   -   byteswap [and display] header variables.
;               
; Restrictions: Use only in EGSE sci_ana program
;               
; Side effects: None
;               
; Category    : EGSE
;               
; Prev. Hist. : EM version by Arnulf, Oct-92
;
; Written     : FM version by C D Pike, RAL, 18-Oct-93
;               
; Modified    : For v3.0 of packet format, CDP, 27-Oct-93
;
; Version     : Version 2, 27-Oct-1993
;-            

FUNCTION read_GIS_raster, unit, gis_raster

;
;  load COMMON block to communicate widget id's for display widgets
;
@HeaderWidgets.com
@MainWidgets.com

;
;  return value -1 on i/o error
;
ON_IOERROR, ERR_GIS_raster

;
; Read from unit, will either be realtime pipe or replay file unit
;
readu,unit,gis_raster
   
;
; Swap science header variables
;
swap_scihead, gis_raster, 1

;
;  byteswap variable containing number of GIS windows
;
gis_raster.numdatawindows = byteswap(gis_raster.numdatawindows)

;
; If requested, display in header info widget in selected format
;
if header_select gt 0 then begin
;
;  decode science process status to bits and display
;
   dec2bin,gis_raster.scihead.SciProcStatus,bits,/quiet
   widget_control,header_g_exp(18),set_val=string(bits(8:15),format='(1x,8i1)')

endif


;
;  return with value -100 if number of windows exceeds 32.  This is
;  redefinable presumably.
;
if (gis_raster.NumDataWindows GT 32) then return,-100

;
;  if displaying header info then clear out window size display
;
if header_select gt 0 then begin
   for i=0,31 do begin
      widget_control,header_g_w_sx(i),set_val=' '
      widget_control,header_g_w_lx(i),set_val=' '
   endfor
endif

;
; Swap window definitions
;

for i=0,gis_raster.NumDataWindows-1 do begin

   gis_raster.GISWin(i).Start  = ByteSwap (gis_raster.GISWin(i).Start)
   gis_raster.GISWin(i).Length = ByteSwap (gis_raster.GISWin(i).Length)

;
;  ...and display in header widget if selected...
;
   if header_select gt 0 then begin 
      widget_control,header_g_w_sx(i),$
                     set_val=string(gis_raster.GISWin(i).Start,format='(i4)')
      widget_control,header_g_w_lx(i),$
                     set_val=string(gis_raster.GISWin(i).Length,format='(i4)')
   endif else begin
;
;  ..... or in general message window otherwise
;
      widget_control, msg_widg, /append, set_val=$
            string('Window number: ', i, '  Start: ', $
                    gis_raster.GISWin(i).Start,$
                    '  Length: ', gis_raster.GISWin(i).Length,$
                    form="(a,i2,a,i4,a,i4)")
   endelse
endfor

;
;  byteswap checksum variable (and ignore!)
;
gis_raster.Chksum = ByteSwap (gis_raster.Chksum)


;
;  success and failure returns
;
RETURN,1
ERR_GIS_raster: RETURN,-1

END
