;+
; Project     : SOHO - CDS     
;                   
; Name        : READ_PKT_HEADER()
;               
; Purpose     : Read science packet header which contains ID.
;               
; Explanation : The EGSE sci_ana task assumes it is receiving only science
;               stream packets from the telemetry stream.  In order to decide
;               on the action necessary to interpret these packets, it first
;               reads 20 bytes from the packet.  These bytes include the 
;               packet ID.  For a definition of the 20 bytes see the structure
;               PacketHeaderDef in sci_ana.pro.  The ID definitions are given
;               in the latest "CDS science data stream packet format"
;               document by DJ Parker. This function also displays the header
;               information if the display widgets are realized.
;               
; Use         : status = read_pkt_header( input_mode, unit, header_str)
;    
; Inputs      : input_mode  -  mode switch (1 = realtime, else replay mode)
;               unit        -  logical unit from which data are read
;                          
; Opt. Inputs : None
;               
; Outputs     : header_str  -  20 byte structure of form PacketHeaderDef
;                              defined in sci_ana.pro
;               function returns a value -1 for i/o errors
;                                         0 for no realtime data available
;                                         1 for success0

; Opt. Outputs: None
;               
; Keywords    : None
;
; Calls       : None
;               
; Restrictions: For use within EGSE sci_ana program only.
;               
; Side effects: None
;               
; Category    : EGSE
;               
; Prev. Hist. : EM version by Arnulf, Oct-92
;
; Written     : FM version by C D Pike, RAL, 18-Oct-93
;               
; Modified    : For v3.0 of packet definition, CDP 27-Oct-93
;               To include writing of packet ID when id = '21'x or '22'x
;                                              CDP 12-Nov-93
;
; Version     : Version 3, 12-Nov-93
;-            

 FUNCTION read_pkt_header, input_mode, unit, packet_header


;
;  COMMON block for information display widgetsa
;
@HeaderWidgets.com

;
; variable to store response from realtime data pipe
;
response = BYTE(0)

;
;  return function value -1 on i/o error
;
ON_IOERROR, ERR_packet_hdr

;
;  if mode = REALTIME
; 
IF (input_mode EQ 1) THEN BEGIN
;
; See if child process has anything for me
;
   WRITEU,unit,'R'
   READU,unit,response

   IF (response GT 0) THEN BEGIN
;
; A packet is available. Read the header 
;
      READU,unit,packet_header
   ENDIF ELSE BEGIN
;
;  nothing there so slow things down to avoid hogging CPU
;
         wait,0.5
         RETURN,0
   ENDELSE

ENDIF ELSE BEGIN

;
; input mode = REPLAY
;
   READU,unit,packet_header
ENDELSE

;
;  if the packet was a GIS raster header packet or an exposure packet and
;  display of their header information is requested then oblige
;  
 
if (packet_header.id eq '1'x and header_select gt 0) or $
   (packet_header.id eq '11'x and header_select eq 2) then begin

   out = string(packet_header.ap_id,format='(6x,2z2,$)')
   widget_control, header_g_exp(0),set_value=out

   if header_format eq 1 then begin
      out = string(ByteSwap(packet_header.length))
   endif else begin
      out = string(byteswap(packet_header.length),format='(6x,z4)')
   endelse
   widget_control, header_g_exp(1),set_value=out


   out = string(fix(packet_header.id),format='(6x,z4)')
   widget_control, header_g_exp(2),set_value=out

   if header_format eq 1 then begin
      out = string(fix(packet_header.StreamPacketCounter))
   endif else begin
      out = string(fix(packet_header.StreamPacketCounter),format='(6x,z4)')
   endelse
   widget_control, header_g_exp(3),set_value=out

endif

;
;  write out ID even for science data packets
;
if (packet_header.id eq '21'x and header_select gt 0) then begin
   out = string(fix(packet_header.id),format='(6x,z4)')
   widget_control, header_g_exp(2),set_value=out
endif


;
;  similarly for VDS packets
;
if (packet_header.id eq '2'x and header_select lt 0) or $
   (packet_header.id eq '12'x and header_select eq -1) then begin

   out = string(packet_header.ap_id,format='(6x,2z2,$)')
   widget_control, header_v_exp(0),set_value=out

   if header_format eq 1 then begin
      out = string(ByteSwap(packet_header.length))
   endif else begin
      out = string(byteswap(packet_header.length),format='(6x,z4)')
   endelse
   widget_control, header_v_exp(1),set_value=out

   out = string(fix(packet_header.id),format='(6x,z4)')
   widget_control, header_v_exp(2),set_value=out

   if header_format eq 1 then begin
      out = string(fix(packet_header.StreamPacketCounter))
   endif else begin
      out = string(fix(packet_header.StreamPacketCounter),format='(6x,z4)')
   endelse
   widget_control, header_v_exp(3),set_value=out

endif

;
;  and again the science packet ID
;
if (packet_header.id eq '22'x and header_select lt 0) then begin
   out = string(fix(packet_header.id),format='(6x,z4)')
   widget_control, header_v_exp(2),set_value=out
endif

;
;  success and error returns
;
RETURN,1
ERR_packet_hdr: RETURN,-1

END
