;+
; Project     : SOHO - CDS     
;                   
; Name        : READ_VDS_RASTER()
;               
; Purpose     : Read the VDS raster header from a unit. 
;               
; Explanation : This function reads the packet header for a VDS raster.  It
;               is a structure of type VDSrasterdef (see sci_ana.pro) and
;               consists of the science header structure (SciheaderDef) which
;               is common to VDS and GIS with additional items specific to the
;               VDS eg the number of data windows and the structure defining
;               the windows. This function calls swap_scihead which is used
;               to byte-swap the header variables and also to write to the
;               header information display widget if it is realized.
;               
; Use         : header = read_vds_raster(unit, vds_raster)
;    
; Inputs      : unit       - the logical unit from which to read data
;               
; Opt. Inputs : None
;               
; Outputs     : vds_raster - VDS raster header structure
;               Function value returned = -1 for error, +1 for read success
;               and -100 for number of windows exceeding the expected maximum.
;
; Opt. Outputs: None
;               
; Keywords    : None
;
; Calls       : swap_scihead   -   byteswap [and display] header variables.
;               
; Restrictions: Use only in EGSE sci_ana program
;               
; Side effects: None
;               
; Category    : EGSE
;               
; Prev. Hist. : EM version by Arnulf, Oct-92
;
; Written     : FM version by C D Pike, RAL, 18-Oct-93
;               
; Modified    : For v3.0 of packet format, CDP, 27-Oct-93
;               Include calculation of XY binning,  9-Nov-93
;
; Version     : Version 3, 9-Nov-1993
;-            
FUNCTION read_VDS_raster, unit, vds_raster

;
;  COMMON blocks to communicate widgets id's
;
@HeaderWidgets.com
@MainWidgets.com
@Images.com


;
;  on i/o error return -1
;
ON_IOERROR, ERR_VDS_raster

;
; Read structure variable from data unit
;
READU,unit,vds_raster

;   
; Swap science header variables
;
swap_scihead, vds_raster, 2

; 
;  deduce the data orientation
;
dec2bin,vds_raster.scihead.SciProcStatus,bits,/quiet
vds_orientation = bits(12)

;
; .. and the binning factors
;
vds_xbin = ishft(vds_raster.BinFactorXY,-4)+1
vds_ybin = (vds_raster.BinFactorXY and '0f'x) + 1

;
;  if requested, display in header info widget
;
if header_select lt 0 then begin

;
;  display all science process status bits
;
   widget_control,header_v_exp(19),$
                  set_val=string(bits(8:15),format='(1x,8i1)')
   
;
;  display data orientation 
;
   widget_control,msg_widg,/append,$
          set_val=string('Orientation flag = ',vds_orientation,form = '(a,i1)')
   orient = ['   Row','   Column']
   orientation = orient(vds_orientation)
   widget_control,header_v_exp(18),set_val=orientation

;
;  finally the XY bin factor
;   
    widget_control,header_v_exp(20),set_val=fmt_vect([vds_xbin,vds_ybin])


endif

;
;  trap unacceptable numbers of windows
;
IF (vds_raster.NumDataWindows GT 32) THEN RETURN,-100

;
;  if displaying header then clear out old window display
;
if header_select lt 0 then begin 
   for i=0,31 do begin
      widget_control,header_v_w_sx(i),set_val=' '
      widget_control,header_v_w_lx(i),set_val=' '
      widget_control,header_v_w_sy(i),set_val=' '
      widget_control,header_v_w_ly(i),set_val=' '
   endfor
endif

;
; Byteswap window definitions.....
;

FOR i=0,vds_raster.NumDataWindows-1 DO BEGIN
     
  vds_raster.VDSWin(i).StartX  = ByteSwap (vds_raster.VDSWin(i).StartX)
  vds_raster.VDSWin(i).StartY  = ByteSwap (vds_raster.VDSWin(i).StartY)
  vds_raster.VDSWin(i).LengthX = ByteSwap (vds_raster.VDSWin(i).LengthX)
  vds_raster.VDSWin(i).LengthY = ByteSwap (vds_raster.VDSWin(i).LengthY)

;
;  ...and display in header widget if selected...
;
   if header_select lt 0 then begin 
      widget_control,header_v_w_sx(i),$
                     set_val=string(vds_raster.VDSWin(i).StartX,format='(i4)')
      widget_control,header_v_w_lx(i),$
                     set_val=string(vds_raster.VDSWin(i).LengthX,format='(i4)')
      widget_control,header_v_w_sy(i),$
                     set_val=string(vds_raster.VDSWin(i).StartY,format='(i4)')
      widget_control,header_v_w_ly(i),$
                     set_val=string(vds_raster.VDSWin(i).LengthY,format='(i4)')
   endif else begin
;
;  ..... or in general message window otherwise
;
      widget_control, msg_widg, /append, $
         set_val=string('Window number: ', i, '  Start X = ', $
                         vds_raster.VDSWin(i).StartX,$
                        '  Start Y = ', vds_raster.VDSWin(i).StartY,$
                         format="(a,i2,a,i4,a,i4)")
      widget_control, msg_widg, /append, $
         set_val=string('Length X = ', vds_raster.VDSWin(i).LengthX,$
                '  Length Y = ', vds_raster.VDSWin(i).LengthY,$
                   format="(a,i4,a,i4)")
   endelse
endfor

;
;  byteswap the checksum
;
vds_raster.Chksum = ByteSwap (vds_raster.Chksum)


;
;  success and error return
;
RETURN,1
ERR_VDS_raster: RETURN,-1

END
