;+
; Project     : SOHO - CDS     
;                   
; Name        : WORKPROC
;               
; Purpose     : Background task to perform telemetry reading.
;               
; Explanation : The WORKPROC procedure is registered with the IDL widget-
;               handling routine XMANAGER as a background task.  This means
;               it is called as often as possible while the main widget is
;               idling ie waiting for user input.  WORKPROC performs the basic
;               reading of telemetry packets.  It is designed to handle various
;               packets depending on the packet identification it finds within
;               the packet header (first 20 bytes - read by read_pkt_header).
;               Only science packets and special packets are dealt with here.
;               Any other packets are read but disregarded.
;               
; Use         : workproc, widget_base
;    
; Inputs      : widget_base  - the top level widget identification.
;               
; Opt. Inputs : None
;               
; Outputs     : None
;               
; Opt. Outputs: None
;               
; Keywords    : None
;
; Calls       : read_pkt_header    -  read 20 byte packet header
;               read_VDS_raster    -  read VDS specific packet header
;               read_GIS_raster    -  read GIS specific packet header
;               read_data          -  read general unidentified packet
;               handle_io_err      -  perform close of input unit on error
;               handle_tm_err      -  perform close of input unit on error
;               read_sci_cnt       -  read science data packet counter from the
;                                     exposure header
;               read_exp           -  read the exposure header packet
;               Update_Expinfo     -  update display of exposure number
;               dec2bin            -  convert decimal integer to binary
;               VDS_Ext_image      -  extract VDS windows from science buffer
;               vds_deselect       -  to deselect VDS data into 1k array
;               gis_deselect       -  to deselect GIS data into spectral array
;               Plot_VDS           -  display the VDS image.
;               GIS_Ext_plot       -  extract GIS data windows from buffer
;               Plot_GIS           -  plot the GIS data
;               read_sp_id         -  read special packet format id
;               read_sp_body       -  read body od a special packet
;               disp_deferred      -  display the contents of a special 
;                                     deferred command store dump
;               disp_imif          -  display the contents of the special
;                                     IMIF (MCU) interface module dump
;               disp_gen_table     -  display contents of special general table
;
;               
; Restrictions: For use only with EGSE science analysis.
;               
; Side effects: None
;               
; Category    : EGSE
;               
; Prev. Hist. : EM version by Arnulf, Oct 92
;
; Written     : FM version by C D Pike, RAL, 12-Oct-93
;               
; Modified    : 
;
; Version     : Version 1, 12-Oct-93
;-            

PRO workproc,widget_base

;
;  load all the possible COMMON blocks for communications
;
@Images.com
@HeaderWidgets.com
@MainWidgets.com
@VDSWidgets.com
@GISWidgets.com
@Packets.com
@Rawscience.com
@IO_units.com
@Mode.com
@Decomp.com
@SpecialWidgets.com
@ReviewWidgets.com
@VDSData.com
@GISData.com
@RawData.com
@Limits.com
@CalFits.com
@GAS_Details.com
@Monitor.com
@Posn_Details.com

;
;  initialise the counters and data arrays
;
id = FIX(0)
result = FIX(0)
sci_counter = FIX(0)
packet_header = {PacketHeaderDef}

;
;  set up look up tables for translation of packet id's in review mode
;
normals = ['0a'x,'0b'x,'01'x,'02'x,'03'x,'11'x,'12'x,'21'x,'22'x,$
           '04'x,'05'x,'06'x,'07'x]
specials = indgen(14) + 1
  

;
;  determine mode of operation.
;   sci_mode = 1 --> realtime
;   sci_mode = 0 --> replay
;  the logical unit used for replay is set to 1 in main_event when the 
;  telemetry reading is started and that for realtime is a variable returned
;  by the SPAWN command also in main_event
;
if (sci_mode EQ 1) then unit = realtime_unit else unit = replay_unit

;
;  read the first 20 bytes of every packet header.  The variable packet_header
;  is of type PacketheaderDef (see sci_ana.pro)
;
result = read_pkt_header(sci_mode, unit, packet_header)

;
;  if the return value is zero, this is an error but can only happen for
;  realtime mode and indicates that the handshake with the C program
;  read_science failed so presumably there are no data to be read.
;
if (result EQ 0) then begin
   return

;
;  possible returns are still -1 (error) and +1 (success)
;
endif else begin

;
;  on error see if there is another data file because error may be due to EOF
;
   if (result EQ -1) then begin
;
;  realtime i/o error (not just lack of data)
;
      if sci_mode eq 1 then begin
         handle_io_err, widget_base, msg_widg, unit
         return
      endif else begin
;
; replay mode i/o error so try another data file if appropriate
;
         if file_seq_no lt (num_data_files-1) then begin
            free_lun,replay_unit
            file_seq_no = file_seq_no + 1
            openr,replay_unit,file_name(file_seq_no),err=ioerr
;
;  new file opened successfully so read the packet header 
;
            if ioerr eq 0 then begin
               widget_control,msg_widg,/append,set_v='Opening new data file.'
               break_file,file_name(file_seq_no),disk,dir,rfile,rext
               widget_control,rep_widg,set_val=rfile+rext
               result = read_pkt_header(sci_mode, unit, packet_header)
;
;  trap i/o error even at this late stage
;
               if result lt 0 then begin
                  handle_io_err, widget_base, msg_widg, unit
                  return
               endif
            endif else begin
;
;  error on opening new data file
;
               bell
               widget_control,msg_widg,/append, $
                              set_v='Error opening new data file.'
               handle_io_err, widget_base, msg_widg, unit
               return
            endelse
         endif else begin
;
;  no more data files to be had so it really is the end
;
            widget_control,msg_widg,/append, $
                           set_val='End of replay data.'
            handle_io_err, widget_base, msg_widg, unit
            return
         endelse
      endelse
   endif

;
;  record the number of packets (of all kinds) received so far
;   
   packet_no = packet_no + LONG(1)

;
;  every 10th packet update the display to encourage onlookers
;
   if ((packet_no MOD 10) EQ 0) then begin 
      WIDGET_CONTROL, packet_widg, SET_VALUE=STRING(packet_no,form='(i9)')
   endif
   
;
;  before we proceed, check it wasn't an abort packet (id = '03'x)
;  if it was then warn and act as if last packet received.
;
   if packet_header.ID eq '03'x then begin
      bell,3
      widget_control,msg_widg,/append, $
                     set_val=systime()+' ***  ABORT packet received. ***'
;
;  check if both raster and exposure header packets were received.  If so
;  then we have at least the data in the exposure header packet so 
;  attempt to process it.  The body of the abort packet will be read at the
;  end as any unwanted packet is.
;
      if check_headers('VDS') then begin
        dcmp_win    = VDSrasterhdr.VDSwin(0:VDSrasterhdr.NumDataWindows-1)
        scheme      = VDSrasterhdr.Scihead.DataCompScheme 
        option      = VDSrasterhdr.Scihead.DataCompOption
        detector    = 0
        print,'Decompressing VDS data with...'
        print,'Scheme/Data comp method: ',scheme
        print,'Option/Data comp parameter: ',option
        print,'Data orientation: ',vds_orientation
        buff16_sz = n_elements(sci_buffer)/2
        sci_buff16 = dcmpdata(fix(sci_buffer,0,buff16_sz),dcmp_win,detector,$
                              scheme,option,vds_orientation)
        VDS_Ext_image, sci_buff16, VDSrasterhdr, $
                                   vds_data_str.vds_exp, msg_widg
        if vds_display then begin
           vds_data_str.exp_in_a = vds_data_str.vds_exp
           vds_data_str.curr_a_img = $
                    vds_deselect(vdsrasterhdr,vds_data_str.exp_in_a - $
                                              vds_data_str.first_in_mem)
           Plot_VDS, vds_awidg, vds_data_str.curr_a_img
        endif
;
;  if needed, write out data to FITS file
;                               =========
;
        if writing_fits then begin
           write_calfits,'NIS',vds_exphdr(vds_data_str.vds_exp),$
                               vds_image(*,*,*,vds_data_str.vds_exp - $
                                               vds_data_str.first_in_mem)
        endif

        have_v_rst_hdr = 0
        have_v_exp_hdr = 0
      endif 

      if check_headers('GIS') then begin
        dcmp_win = replicate({StartX:0,StartY:0,LengthX:0,LengthY:0},$
                             GISrasterhdr.NumDataWindows)
        for i=0,GISrasterhdr.NumDataWindows-1 do begin
           dcmp_win(i).StartX  = GISrasterhdr.GISwin(i).Start
           dcmp_win(i).StartY  = 0
           dcmp_win(i).LengthX = GISrasterhdr.GISwin(i).Length
           dcmp_win(i).LengthY = 1
        endfor
        scheme = GISrasterhdr.Scihead.DataCompScheme 
        option = GISrasterhdr.Scihead.DataCompOption
        vds_orientation = 0
        detector = 1
        print,'Decompressing GIS data with...'
        print,'Scheme/Data comp method: ',scheme
        print,'Option/Data comp parameter: ',option
        print,'Data orientation: ',vds_orientation
        buff16_sz = n_elements(sci_buffer)/2
        sci_buff16 = dcmpdata(fix(sci_buffer,0,buff16_sz),dcmp_win,detector,$
                              scheme,option,vds_orientation)
      
        GIS_Ext_plot, sci_buff16, GISrasterhdr, gis_plot, $
                                  gis_data_str.gis_exp, msg_widg       
        if gis_display then begin
           gis_data_str.exp_in_a = gis_data_str.gis_exp
           gis_data_str.curr_a_data = gis_deselect(gisrasterhdr,gis_plot,$
                                                   gis_data_str.gis_exp, $
                                                   msg_widg)
           Plot_GIS, gis_data_str.curr_a_data
        endif

;
;  if needed, write out data to FITS file
;                               =========
;
        if writing_fits then begin
           write_calfits,'GIS',gis_exphdr(gis_data_str.gis_exp),$
                               gis_plot(*,*,gis_data_str.gis_exp)
        endif

        have_g_rst_hdr = 0
        have_g_exp_hdr = 0
      endif
   endif

;
;  should have a valid packet header ID so save and display it
;
;   if last_pkt_id ne packet_header.ID then begin
;      widget_control,cpacket_widg,set_val=string(packet_header.ID,form='(z2.2)')
;   endif
   last_pkt_id = packet_header.ID

;
;  Deal with statistics (just in case required later
;  if it's a special packet then wait until we know the format id
;
   if last_pkt_id ne '30'x then begin
      index = where(normals eq last_pkt_id)
      index = index(0)
      if index ge 0 then begin
         review_stats(index) = review_stats(index) + 1
         new_rev_stat = index
      endif
   endif

;
;  now act according to packet header ID
;
   CASE 1 of 

;
;  VDS raster header
;===================
       (packet_header.ID EQ '2'X): $
           begin
              widget_control,msg_widg,/append, $
                             set_val=systime()+' Reading VDS raster HEADER'
              result = read_VDS_raster (unit, VDSrasterhdr)
              if (result EQ -1) then begin
                 handle_io_err, widget_base, msg_widg, unit
                 return
              endif else begin 
                 if (result EQ -100) then begin
                    handle_tm_error, widget_base, msg_widg, unit,$
                                     "Too many windows defined in VDS raster"
                    return
                 endif

;
;  have a valid raster header so sort out the size of arrays needed to store
;  the incoming data
;
                 slitnum = VDSrasterhdr.scihead.slitnumber
                 nw = VDSrasterhdr.NumDataWindows
                 nx = max(VDSrasterhdr.VDSWin.lengthX)
                 ny = VDSrasterhdr.VDSWin(0).lengthY
                 vds_data_str.max_in_mem = fix(MAX_VDS_MEMORY/$
                                               (long(nx)*long(ny)*long(nw)*2L))
                 vds_date_time = strarr(total_exp)
                 no_exp = total_exp < vds_data_str.max_in_mem
                 if no_exp lt total_exp then begin
                    bell
                    text = string('Only ',no_exp,$
                     ' exposures can be held in memory.',form='(a,i3,a)')
                    widget_control,msg_widg,/append,set_v=text
                    text = 'Memory will cycle the exposures after that.'
                    widget_control,msg_widg,/append,set_v=text
                 endif
                 vds_image = intarr(nx,ny,nw,no_exp)
                 sci_buffer = bytarr(long(nx)*long(ny)*long(nw)*2L+276)
                 VDS_exphdr = replicate(Exphdr,total_exp)
                 have_v_rst_hdr = 1
                 WIDGET_CONTROL, msg_widg, /append, $
                             SET_VALUE=systime()+" VDS raster header received"
;
;  reset data structure and set current operation mode
;  cds_mode = 1 (VDS) 2 (GIS)
;
                 vds_data_str = {vds_data_rec}
                 vds_data_str.(0) = -1
                 vds_data_str.(2) = -1
                 vds_data_str.(3) = -1
                 vds_data_str.(5) = -1
                 cds_mode = 1
;
;  Open the FITS file if required and log the file name
;
                 if writing_fits then begin
                    rasnum = strmid($
                             string(vdsrasterhdr.scihead.rastercounter+1000,$
                                    format='(i4)'),1,3)
                    strput,cal_fits_file,rasnum,10
                    widget_control,fitsfile_widg,set_val=cal_fits_file
                    check_fits_name, cal_fits_file
                    open_calfits, concat_dir('$CDS_CAL_FITS',cal_fits_file),$
                                  'NIS', vdsrasterhdr, calhdr

                    if cal_fits_file ne 'None_0000_000.00' then begin
                       openu,clun,concat_dir('$CDS_CAL_FITS','cal_fits.log'),$
                             /get_lun, /append
                       printf,clun,cal_fits_file
                       free_lun,clun
                    endif

                 endif

              endelse
           end 

;
;  GIS raster header
;===================     
       (packet_header.ID EQ '1'X): $
           begin
              widget_control,msg_widg,/append,$
                             set_val=systime()+' Reading GIS raster header'
              result = read_GIS_raster (unit, GISrasterhdr)
              if (result EQ -1) then begin
                 handle_io_err, widget_base, msg_widg, unit
                 return
              endif else begin 
                 if (result EQ -100) then begin
                    handle_tm_error, widget_base, msg_widg, unit,$
                                     "Too many windows defined in GIS raster"
                    return
                 endif
                 
                 WIDGET_CONTROL, msg_widg, /append, $
                                 SET_VAL=systime()+" GIS raster header received"
;
;  have a valid raster header so sort out the size of arrays needed to store
;  the incoming data
;
                 slitnum = GISrasterhdr.scihead.slitnumber
                 have_g_rst_hdr = 1
                 nw = GISrasterhdr.NumDataWindows
                 nx = max(GISrasterhdr.GISWin.length)
                 no_exp = total_exp 
                 gis_plot = intarr(nx,nw,no_exp)
                 gis_date_time = strarr(no_exp)
                 gis_exphdr = replicate(exphdr,no_exp)
                 sci_buffer = bytarr(long(nx)*long(nw)*long(no_exp)*2L+276)


;
;  reset data structure and set operational mode
;    cds_mode = 1 (VDS) 2 (GIS)
;
                 gis_data_str = {gis_data_rec}
                 gis_data_str.(0) = -1
                 gis_data_str.(1) = -1
                 gis_data_str.(2) = -1
                 gis_data_str.(4) =  1
                 gis_data_str.(7) = 'A'
                 gis_data_str.(8)(4) = 1
                 gis_data_str.(9)(4) = 1
                 cds_mode = 2

;
;  Open the FITS file if required, update the log and adjust file name for
;  raster counter
;
                 if writing_fits then begin
                    rasnum = strmid($
                             string(gisrasterhdr.scihead.rastercounter+1000,$
                                    format='(i4)'),1,3)
                    strput,cal_fits_file,rasnum,10
                    widget_control,fitsfile_widg,set_val=cal_fits_file
                    check_fits_name,cal_fits_file
                    open_calfits, concat_dir('$CDS_CAL_FITS',cal_fits_file),$
                                  'GIS', gisrasterhdr, calhdr
                    openu,clun,concat_dir('$CDS_CAL_FITS','cal_fits.log'),$
                          /get_lun, /append
                    printf,clun,cal_fits_file
                    free_lun,clun
                 endif

              endelse
            end 
 
;
;  Read exposure header for either instrument received
;=====================================================    
        ((packet_header.ID EQ '11'X) OR $
          (packet_header.ID EQ '12'X)): $
           begin
;
; Ignore if no raster has been read yet
;
              if (cds_mode EQ 0) then begin
                 widget_control,msg_widg,/append, $
                      set_v='No raster has been read so ignoring, cds_mode = 0'
                 result = read_data(unit,Any_Packet)
                 return
              endif

;
;  read science data packet counter (2 bytes)
;
              sci_counter = read_sci_cnt (unit, packet_header.ID) 
              if (sci_counter EQ -1) then begin
                 widget_control,msg_widg,/append, set_val='Sci counter = -1'
                 handle_io_err, widget_base, msg_widg, unit
                 return
              endif     


;
;  check if this packet - even though exposure header - is the last
;
              check_for_last,sci_counter,last_packet

;
;  read the rest of the exposure packet (some header info and some data)
;
              widget_control,msg_widg,/append, $
                             set_v=systime()+' Reading Exposure header.'
              result = read_exp(unit, Exphdr, packet_header.ID)
              if (result EQ -1) then begin
                 handle_io_err, widget_base, msg_widg, unit
                 return
              endif    
;
;  if monitoring the mechanisms, load the arrays and counters
;
              if m_monitor_mode then begin
                 opsl_count = opsl_count + 1
                 if opsl_count gt 999 then begin
                    opsl_count = 0
                    opsl_monitor(*) = 0
                 endif
                 opsl_monitor(opsl_count) = exphdr.opslcoordinate

                 opsr_count = opsr_count + 1
                 if opsr_count gt 999 then begin
                    opsr_count = 0
                    opsr_monitor(*) = 0
                 endif
                 opsr_monitor(opsr_count) = exphdr.opsrcoordinate

                 slitn_count = slitn_count + 1
                 if slitn_count gt 999 then begin
                    slitn_count = 0
                    slitn_monitor(*) = 0
                 endif
                 slitn_monitor(slitn_count) = slitnum

                 slitp_count = slitp_count + 1
                 if slitp_count gt 999 then begin
                    slitp_count = 0
                    slitp_monitor(*) = 0
                 endif
                 slitp_monitor(slitp_count) = exphdr.slitposition

                 mirrp_count = mirrp_count + 1
                 if mirrp_count gt 999 then begin
                    mirrp_count = 0
                    mirrp_monitor(*) = 0
                 endif
                 mirrp_monitor(mirrp_count) = exphdr.mirrorposition
                 
                 plot_monitor

              endif
     

;
;  load exposure header info according to instrument.  
;
              if (packet_header.ID EQ '12'X) then begin
                 have_v_exp_hdr = 1
                 vds_data_str.vds_exp = Exphdr.ExposureNumber
                 VDS_exphdr(vds_data_str.vds_exp) = Exphdr
                 if vds_display then Update_ExpInfo, $
                                     vds_exphdr(vds_data_str.vds_exp),$
                                     VDSrasterhdr,  vds_exp_awidg
              endif else begin
                 have_g_exp_hdr = 1
                 gis_data_str.gis_exp = Exphdr.ExposureNumber
                 GIS_exphdr(gis_data_str.gis_exp) = Exphdr
                 if gis_display then $
                          Update_ExpInfo, GIS_exphdr(gis_data_str.gis_exp),$
                                                GISrasterhdr,  gis_exp_awidg
              endelse

;
;  sci_writeptr points to the start location in the science data buffer (1-d) 
;  where the current data are to be stored
;
              sci_writeptr = LONG(0)               

;
;Extract science from packet and put in big buffer store
;
              sci_buffer(0) = Exphdr.Science(*)
                

;
; If it was also the last packet (eg in some drastic compression scheme)
; then decompress, extract and plot VDS window
;
              if (packet_header.ID EQ '12'X and last_packet) then begin

                status_ok = check_headers('VDS')
                if status_ok then begin
                  dcmp_win = VDSrasterhdr.VDSwin(0:VDSrasterhdr.NumDataWindows-1)
                  scheme    = VDSrasterhdr.Scihead.DataCompScheme 
                  option     = VDSrasterhdr.Scihead.DataCompOption
                  detector    = 0
                  print,'Decompressing VDS data with...'
                  print,'Scheme/Data comp method: ',scheme
                  print,'Option/Data comp parameter: ',option
                  print,'Data orientation (read): ',vds_orientation
                  buff16_sz = n_elements(sci_buffer)/2
                  sci_buff16 = dcmpdata(fix(sci_buffer,0,buff16_sz),dcmp_win,$
                                        detector, scheme,option,vds_orientation)
                  VDS_Ext_image, sci_buff16, VDSrasterhdr, $
                                                 vds_data_str.vds_exp, msg_widg
                  if vds_display then begin
                     vds_data_str.exp_in_a = vds_data_str.vds_exp
                     vds_data_str.curr_a_img = $
                      vds_deselect(vdsrasterhdr,vds_data_str.exp_in_a - $
                                                vds_data_str.first_in_mem)
                     Plot_VDS, vds_awidg, vds_data_str.curr_a_img
                  endif

;
;  if required, write out FITS data
;
                  if writing_fits then begin
                     write_calfits,'NIS',vds_exphdr(vds_data_str.vds_exp),$
                                      vds_image(*,*,*,vds_data_str.vds_exp - $
                                                      vds_data_str.first_in_mem)
                  endif

                endif else begin
                  bell
                  widget_control,msg_widg,/append,$
                         set_val='Raster or exposure header packet not seen.'
                  widget_control,msg_widg,/append,$
                         set_val='Science data not available.'
                endelse
;
;  if this was the last exposure then clear raster header flag and close the
;  FITS file
;
                if vds_data_str.vds_exp eq $
                      (total_exp-1) then begin
                   have_v_rst_hdr = 0
                   if writing_fits then begin
                      close_calfits
                      widget_control,msg_widg,/append,set_val=' '
                      widget_control,msg_widg,/append,$
                        set_val='** FITS file <'+cal_fits_file+'> written OK **'
                      widget_control,msg_widg,/append,set_val=' '
                   endif
                endif
                have_v_exp_hdr = 0
;
;  monitor mode?
;               
                if v_monitor_mode then begin
                   vm_count = vm_count + 1
                   if vm_count gt 999 then begin
                      vm_count = 0
                      vds_monitor(*) = 0.0
                   endif
                   vds_monitor(vm_count) = total(byteswap(sci_buff16)) > 0
                   plot_monitor
                endif
              endif 
             
;
; If last science packet then decompress, extract and plot GIS window
;
              if (packet_header.ID EQ '11'X and last_packet) then begin
                status_ok = check_headers('GIS')
                if status_ok then begin
;
;  need to force GIS windows into 2-d form same as VDS
;
                  dcmp_win = replicate({StartX:0,StartY:0,LengthX:0,LengthY:0},$
                                       GISrasterhdr.NumDataWindows)
                  for i=0,GISrasterhdr.NumDataWindows-1 do begin
                     dcmp_win(i).StartX  = GISrasterhdr.GISwin(i).Start
                     dcmp_win(i).StartY  = 0
                     dcmp_win(i).LengthX = GISrasterhdr.GISwin(i).Length
                     dcmp_win(i).LengthY = 1
                  endfor
                  scheme = GISrasterhdr.Scihead.DataCompScheme 
                  option = GISrasterhdr.Scihead.DataCompOption
                  vds_orientation = 0
                  detector = 1
                  print,'Decompressing GIS data with...'
                  print,'Scheme/Data comp method: ',scheme
                  print,'Option/Data comp parameter: ',option
                  print,'Data orientation: ',vds_orientation
                  buff16_sz = n_elements(sci_buffer)/2
                  sci_buff16 = dcmpdata(fix(sci_buffer,0,buff16_sz),$
                               dcmp_win,detector,scheme,option,vds_orientation)
       
                  GIS_Ext_plot, sci_buff16, GISrasterhdr, gis_plot, $
                                    gis_data_str.gis_exp, msg_widg       
                  if gis_display then begin
                     gis_data_str.exp_in_a = gis_data_str.gis_exp
                     gis_data_str.curr_a_data = $
                             gis_deselect(gisrasterhdr,gis_plot,$
                                          gis_data_str.gis_exp, msg_widg)
                     Plot_GIS, gis_data_str.curr_a_data
                  endif

;                 help,gis_data_str
;  if required, write out FITS data
;
                  if writing_fits then begin
                     write_calfits,'GIS',gis_exphdr(gis_data_str.gis_exp),$
                                         gis_plot(*,*,gis_data_str.gis_exp)
                  endif

                endif else begin
                  bell
                  widget_control,msg_widg,/append,$
                         set_val='Raster or exposure header packet not seen.'
                  widget_control,msg_widg,/append,$
                         set_val='Science data not available.'
                endelse
;
;  if this the last exposure then clear raster header flag and close the FITS
;  file
;
                if gis_data_str.gis_exp eq $
                      (total_exp-1) then begin
                   have_g_rst_hdr = 0
                   if writing_fits then begin
                      close_calfits
                      widget_control,msg_widg,/append,set_val=' '
                      widget_control,msg_widg,/append,$
                        set_val='** FITS file <'+cal_fits_file+'> written OK **'
                      widget_control,msg_widg,/append,set_val=' '
                   endif
                endif
                have_g_exp_hdr = 0

;
;  monitor mode?
;               
                if g_monitor_mode then begin
                   gm_count = gm_count + 1
                   if gm_count gt 999 then begin
                      gm_count = 0
                      gis_monitor(*) = 0.0
                   endif
                   gis_monitor(gm_count) = total(byteswap(sci_buff16)) > 0L
                   plot_monitor
                endif
              endif
           end

;
;  pure science data from either instrument
;==========================================
        ((packet_header.ID EQ '21'X) OR $
          (packet_header.ID EQ '22'X)): $
            begin
;
;  read the packet counter (for this exposure record)  - its value will be
;  displayed in the header information widget
;
              sci_counter = read_sci_cnt (unit, packet_header.ID) 
;
;  check if it is the last packet. Check for missing ie non-consecutive science
;  packets.
;
              check_for_last,sci_counter,last_packet
              if last_packet then begin
                 last_sci_cnt = 0
              endif else begin
                 if (sci_counter - last_sci_cnt) ne 1 then begin
                    bell,3
                    widget_control,msg_widg,$
                       set_val=systime()+' ** Missed science packet **',/append
                    widget_control,msg_widg,/append,$
                       set_val=string('Last packet counter was: ',$
                                       last_sci_cnt,format="(a,i4)") 
                    print,'Current/last counters =  ',sci_counter, last_sci_cnt
                 endif
                 last_sci_cnt = sci_counter
              endelse

;
;  read the science data.  The structure returned is of type SciPacketDef
;  which contains 6 bytes of counters, 276 bytes of data and 2 bytes of 
;  checksum - see sci_ana.pro
;
              result = read_data(unit, Sci_Packet)
              if (result EQ -1) then begin
                 handle_io_err, widget_base, msg_widg, unit
                 return
              endif     
;
; Extract science - 252 bytes are already there from the exposure packet ie
;                   locations (0-251) so for the first science packet start
;                   at location 252 and thereafter each science packet is 
;                   adding 276 bytes of data.
;
              sci_writeptr = LONG(252) + LONG(276) * (sci_counter-1)
              if (sci_writeptr ge (n_elements(sci_buffer)-277)) or $
                 (sci_writeptr lt 0) then begin
;                 bell
                 widget_control,msg_widg,/append,$
                                set_v='Science counter too large.'
                 return
              endif
              sci_buffer(sci_writeptr) = Sci_Packet.Data(*)

;
; If it was the last packet then decompress, extract and plot VDS window
;
              if (packet_header.ID EQ '22'X and last_packet) then begin
                status_ok = check_headers('VDS')
                if status_ok then begin
                  dcmp_win = VDSrasterhdr.VDSwin(0:VDSrasterhdr.NumDataWindows-1)
                  scheme    = VDSrasterhdr.Scihead.DataCompScheme 
                  option     = VDSrasterhdr.Scihead.DataCompOption
                  detector    = 0
                  print,'Decompressing VDS data with...'
                  print,'Scheme/Data comp method: ',scheme
                  print,'Option/Data comp parameter: ',option
                  print,'Data orientation (read): ',vds_orientation
                  buff16_sz = n_elements(sci_buffer)/2
                  sci_buff16 = dcmpdata(fix(sci_buffer,0,buff16_sz),dcmp_win,$
                                        detector, scheme,option,vds_orientation)
                  VDS_Ext_image, sci_buff16, VDSrasterhdr, $
                                                 vds_data_str.vds_exp, msg_widg
                  if vds_display then begin
                     vds_data_str.exp_in_a = vds_data_str.vds_exp
                     vds_data_str.curr_a_img = $
                                  vds_deselect(vdsrasterhdr,$
                                               vds_data_str.exp_in_a - $
                                               vds_data_str.first_in_mem)
                     Plot_VDS, vds_awidg, vds_data_str.curr_a_img
                  endif

;
;  if required, write out FITS data
;
                  if writing_fits then begin
                     write_calfits,'NIS',vds_exphdr(vds_data_str.vds_exp),$
                                      vds_image(*,*,*,vds_data_str.vds_exp - $
                                                      vds_data_str.first_in_mem)
                  endif

                endif else begin
                  bell
                  widget_control,msg_widg,/append,$
                         set_val='Raster or exposure header packet not seen.'
                  widget_control,msg_widg,/append,$
                         set_val='Science data not available.'
                endelse
;
;  if this the last exposure then clear raster header flag
;
                if vds_data_str.vds_exp eq $
                      (total_exp-1) then begin
                   have_v_rst_hdr = 0
                   if writing_fits then begin
                      close_calfits
                      widget_control,msg_widg,/append,set_val=' '
                      widget_control,msg_widg,/append,$
                        set_val='** FITS file <'+cal_fits_file+'> written OK **'
                      widget_control,msg_widg,/append,set_val=' '
                   endif
                endif
                have_v_exp_hdr = 0

;
;  if we have missed either the raster or exposure header packets then 
;  sci_buff16 will not be defined, but we want to have a monitor count (set to 
;  zero to indicate the omission) so create a dummy sci_buff16 here.
                
                if n_elements(sci_buff16) eq 0 then sci_buff16 = 0

;
;  monitor mode?
;               
                if v_monitor_mode then begin
                   vm_count = vm_count + 1
                   if vm_count gt 999 then begin
                      vm_count = 0
                      vds_monitor(*) = 0.0
                   endif
                   vds_monitor(vm_count) = total(byteswap(sci_buff16)) > 0
                   plot_monitor
                endif
              endif 
             
;
; If last science packet then decompress, extract and plot GIS window
;
              if (packet_header.ID EQ '21'X and last_packet) then begin
                status_ok = check_headers('GIS')
                if status_ok then begin
;
;  need to force GIS windows into 2-d form same as VDS
;
                  dcmp_win = replicate({StartX:0,StartY:0,LengthX:0,LengthY:0},$
                                       GISrasterhdr.NumDataWindows)
                  for i=0,GISrasterhdr.NumDataWindows-1 do begin
                     dcmp_win(i).StartX  = GISrasterhdr.GISwin(i).Start
                     dcmp_win(i).StartY  = 0
                     dcmp_win(i).LengthX = GISrasterhdr.GISwin(i).Length
                     dcmp_win(i).LengthY = 1
                  endfor
                  scheme = GISrasterhdr.Scihead.DataCompScheme 
                  option = GISrasterhdr.Scihead.DataCompOption
                  vds_orientation = 0
                  detector = 1
                  print,'Decompressing GIS data with...'
                  print,'Scheme/Data comp method: ',scheme
                  print,'Option/Data comp parameter: ',option
                  print,'Data orientation: ',vds_orientation
                  buff16_sz = n_elements(sci_buffer)/2
                  sci_buff16 = dcmpdata(fix(sci_buffer,0,buff16_sz),$
                               dcmp_win,detector,scheme,option,vds_orientation)
       
                  GIS_Ext_plot, sci_buff16, GISrasterhdr, gis_plot, $
                                gis_data_str.gis_exp, msg_widg       

                  if gis_display then begin
                     gis_data_str.exp_in_a = gis_data_str.gis_exp
                     gis_data_str.curr_a_data = $
                             gis_deselect(gisrasterhdr,gis_plot,$
                                          gis_data_str.gis_exp, msg_widg)
                     Plot_GIS, gis_data_str.curr_a_data
                  endif

;
;  if required, write out FITS data
;
                  if writing_fits then begin
                     write_calfits,'GIS',gis_exphdr(gis_data_str.gis_exp),$
                                         gis_plot(*,*,gis_data_str.gis_exp)
                  endif

                endif else begin
                  bell
                  widget_control,msg_widg,/append,$
                         set_val='Raster or exposure header packet not seen.'
                  widget_control,msg_widg,/append,$
                         set_val='Science data not available.'
                endelse
;
;  if this the last exposure then clear raster header flag and close the FITS
;  file
;
                if gis_data_str.gis_exp eq $
                     (total_exp-1) then begin
                   have_g_rst_hdr = 0
                   if writing_fits then begin
                      close_calfits
                      widget_control,msg_widg,/append,set_val=' '
                      widget_control,msg_widg,/append,$
                        set_val='** FITS file <'+cal_fits_file+'> written OK **'
                      widget_control,msg_widg,/append,set_val=' '
                   endif
                endif
                have_g_exp_hdr = 0

;
;  if we have missed either the raster or exposure header packets then 
;  sci_buff16 will not be defined, but we want to have a monitor count (set to 
;  zero to indicate the omission) so create a dummy sci_buff16 here.
                
                if n_elements(sci_buff16) eq 0 then sci_buff16 = 0

;
;  monitor mode?
;               
                if g_monitor_mode then begin
                   gm_count = gm_count + 1
                   if gm_count gt 999 then begin
                      gm_count = 0
                      gis_monitor(*) = 0.0
                   endif
                   gis_monitor(gm_count) = total(byteswap(sci_buff16)) > 0L
                   plot_monitor
                endif
              endif
            end

;
;  try trap of mode
;
;        (packet_header.ID EQ '0A'X and have_v_rst_hdr):  begin
;             result =  read_data (unit, Any_Packet)
;              if (result EQ -1) then begin
;                 handle_io_err, widget_base, msg_widg, unit
;                 return
;              endif
;             print,'--'
;             for i=0,11 do begin
;                print,any_packet.data(i*25:(i*25+24 < 285)),form='(25(1x,z2.2))'
;             endfor
;           end

;
;  read special packets - all special packets have the packet id '30'x
;=====================================================================
        (packet_header.ID EQ '30'X):  begin

;
;  special packets are distinguished by their special packet FORMAT id
;  so read it and the counter (2bytes)
;
           temp = read_sp_id(unit)
           sp_pkt_id  = temp(1)
           sp_pkt_cnt = temp(0)

;
;  Deal with statistics of special packets
;
           index = where(specials eq sp_pkt_id)
           index = index(0)
           if index ge 0 then begin
              review_stats(index+13) = review_stats(index+13) + 1
              new_rev_stat = index+13
           endif
;
;  act according to packet format specification if the display window has
;  been requested
;
           case 1 of

;
;  DEFERRED command store, read and display
;
              (sp_pkt_id eq '02'x) and (sp_request eq '02'x): begin
                 packet = read_sp_body(unit, checksum)
                 sp_def_cnt = sp_def_cnt + 1
                 disp_deferred, packet, sp_pkt_cnt
              end

;
;   HEALTH module, read and display
;
              (sp_pkt_id eq '08'x) and (sp_request eq '08'x): begin
                 packet = read_sp_body(unit, checksum)
                 disp_health, packet, sp_pkt_cnt
              end

;
;   IEF module, read and display
;
              (sp_pkt_id eq '0e'x) and (sp_request eq '0e'x): begin
                 packet = read_sp_body(unit, checksum)
                 disp_ief, packet, sp_pkt_cnt
              end

;
;  MCU (IMIF interface module - MCU support) dump, read and display
;
              (sp_pkt_id eq '01'x) and (sp_request eq '01'x): begin
                 packet = read_sp_body(unit, checksum)
                 disp_imif, packet, sp_pkt_cnt
              end
;
;  MCU (IMIF interface module) dump, read and display
;
              (sp_pkt_id eq '09'x) and (sp_request eq '09'x): begin
                 packet = read_sp_body(unit, checksum)
                 disp_imifim, packet
              end
;
;  ERROR module dump, read and display
;
              (sp_pkt_id eq '07'x) and (sp_request eq '07'x): begin
                 packet = read_sp_body(unit, checksum)
                 disp_err, packet
              end
;
;  ANY table dump that is currently requested
;
              sp_pkt_id eq sp_request: begin
                 packet = read_sp_body(unit, checksum)
                 disp_gen_table, packet, sp_pkt_cnt, sp_pkt_id
              end

;
;  not done yet (assume general will do)
;
              else:  begin
                       packet = read_sp_body(unit, checksum)
                     end
           endcase    ; end of special packet format case

        end           ; end of packet.ID eq '30'x

;
;  deal with GIS raw data if requested
;  ===================================
;
        (packet_header.ID eq '07'x)  and $
         raw_gis_request: begin
                             status = read_data(unit,raw_gis_data)
                             if status then begin
                                disp_raw_gis
                             endif else begin
                                flash_msg,msg_widg,$
                                      'Error reading Raw GIS data packet.'
                             endelse
                          end

;
;  deal with VDS raw data if requested
;  ===================================
;
        (packet_header.ID eq '06'x)  and $
         raw_vds_request: begin
                             status = read_data(unit,raw_vds_data)
                             if status then begin
                                disp_raw_vds
                             endif else begin
                                flash_msg,msg_widg,$
                                      'Error reading Raw VDS data packet.'
                             endelse
                          end


       else:begin
;
; Read other type of packets in which we have no vested interest 
; ==============================================================
             result =  read_data (unit, Any_Packet)
              if (result EQ -1) then begin
                 handle_io_err, widget_base, msg_widg, unit
                 return
              endif
           end
       endcase     ; packet.ID
;
;  all dealt with so check if want statistics display
;
      if reviewing and (new_rev_stat ge 0) then begin
         disp_review
         new_rev_stat = -1
      endif

   endelse         ; ok return from reading packet header
end

