;+
; Project     : SOHO - CDS     
;                   
; Name        : CDS_CW_FILE_IN()
;               
; Purpose     : Data file selection for a single input dataset.
;               
; Explanation : This function creates a compound widget consisting of the 
;               compound widget cds_cw_infile.pro, and a 'Done' button.
;	        The done button is automatically de-sensitised
;	        until valid input datasets have been selected.
;
;	        The value of this widget is the settings structure of the
;	        cds_cw_infile widget.  This widget only generates events
;	        when the 'Done' button is pressed.
;	        The event structure returned is;
;	        {ID:0L, TOP:0L, HANDLER:0L, ACTION:''}
;	        ACTION has the value 'Done'.
;
;               
; Use         : See routine cds_file_in.pro for an example.
;    
; Inputs      : PARENT	- Long integer; the ID of the parent widget.
;               
; Opt. Inputs : None
;               
; Outputs     : This function returns the ID of the compound widget created.
;               
; Opt. Outputs: None
;               
; Keywords    : VALUE - A structure which determines the initial settings of
;                       the dataset selection widget cds_cw_infile.  The
;		        structure must be;
;		        {ROOTPATH:'', FILE:'', CENTROOT:'', USERROOT:'' }
;		        The elements of the structure are as follows;
;
;		ROOTPATH - Current data directory e.g '/usr/fred/adas/'
;		FILE     - Current data file in ROOTPATH e.g 'input.dat'
;		CENTROOT - Default central data store e.g '/usr/adas/'
;		USERROOT - Default user data store e.g '/usr/fred/adas/'
;
;		The data file selected by the user is obtained by
;		appending ROOTPATH and FILE.  In the above example
;		the full name of the data file is;
;		/usr/fred/adas/input.dat
;
;		Path names may be supplied with or without the trailing
;		'/'.  The underlying routines add this character where
;		required so that USERROOT will always end in '/' on output.
;
;		The default value is;
;		{ROOTPATH:'./', FILE:'', CENTROOT:'', USERROOT:''}
;		i.e ROOTPATH is set to the user's current directory.
;
;	        TITLE  - The title to be included in the input file widget, 
;                        used to indicate exactly what the required input 
;                        dataset is, e.g 'Input VDS Dataset'
;
;	        UVALUE - A user value for the widget.  Defaults to 0.
;
;	        FONT -   Supplies the font to be used for the interface 
;                        widgets. Defaults to the current system font.
;
; Calls       : CW_LOADSTATE	Recover compound widget state.
;	        CW_SAVESTATE	Save compound widget state.
;      	        CDS_CW_INFILE	Dataset selection widget.
;	        FILE_ACC        Determine filetype and access permissions.
;	        IN_GET_VAL()    Widget management routine in this file.
;               IN_EVENT()      Widget management routine in this file.
;               
; Restrictions: None
;               
; Side effects: A COMMON BLOCK: CW_IN_BLK is used which is private to
;               this widget.
;
;               
; Category    : Util, widgets
;               
; Prev. Hist. : ADAS version by Andrew Bowen, 
;                               Tessella Support Services plc, 12-May-1993

;
; Written     : CDS version for use in EGSE science analysis by
;               C D Pike, RAL, 7-Oct-93
;               
; Modified    : Added cancel button.  CDP, 18-Jan-95
;
; Version     : Version 2, 18-Jan-95
;-            

FUNCTION in_get_val, id

  COMMON cw_in_blk, state_base, state_stash, state

                ;**** Return to caller on error ****
  ON_ERROR, 2

                ;**** Retrieve the state ****
  CW_LOADSTATE, id, state_base, state_stash, state

		;**** Get settings ****
  widget_control,state.fileid,get_value=inset

  RETURN, inset

END

;-----------------------------------------------------------------------------

FUNCTION in_event, event

  COMMON cw_in_blk, state_base, state_stash, state

                ;**** Base ID of compound widget ****
  parent=event.handler

                ;**** Retrieve the state ****
  CW_LOADSTATE, parent, state_base, state_stash, state

		;*********************************
		;**** Clear previous messages ****
		;*********************************
  widget_control,state.messid,set_value=''

		;**** Default output no event ****
  new_event = 0L
                ;************************
                ;**** Process events ****
                ;************************
  CASE event.id OF

		;***********************************
		;**** Event from file selection ****
		;***********************************
    state.fileid: begin
	  if event.action eq 'newfile' then begin
	    widget_control,state.doneid,/sensitive
	  end else begin
	    widget_control,state.doneid,sensitive=0
	  end
	end
		;*********************
		;**** Done button ****
		;*********************
    state.doneid: begin

	  new_event = {ID:parent, TOP:event.top, HANDLER:0L, ACTION:'Done'}

        end
		;*********************
		;**** Cancel button ****
		;*********************
    state.cancelid: begin

	  new_event = {ID:parent, TOP:event.top, HANDLER:0L, ACTION:'Cancel'}

        end

    ELSE:

  ENDCASE

  RETURN, new_event
END

;-----------------------------------------------------------------------------

FUNCTION cds_cw_file_in, parent, $
		VALUE=value, TITLE=title, UVALUE=uvalue, FONT=font


  COMMON cw_in_blk, state_base, state_stash, state

  IF (N_PARAMS() EQ 0) THEN MESSAGE, 'Must specify parent for cds_cw_file_in'
  ON_ERROR, 2					;return to caller

		;**** Set defaults for keywords ****
  IF NOT (KEYWORD_SET(value)) THEN begin
    inset = {   ROOTPATH:'./', FILE:'', $
		CENTROOT:'', USERROOT:'' }
  END ELSE BEGIN
    inset = {   ROOTPATH:value.rootpath, FILE:value.file, $
		CENTROOT:value.centroot, USERROOT:value.userroot }
    if strtrim(inset.rootpath) eq '' then begin
      inset.rootpath = './'
    end else if $
	    strmid(inset.rootpath,strlen(inset.rootpath)-1,1) ne '/' then begin
      inset.rootpath = inset.rootpath+'/'
    end
    if strmid(inset.file,0,1) eq '/' then begin
      inset.file = strmid(inset.file,1,strlen(inset.file)-1)
    end
  END
  IF NOT (KEYWORD_SET(title)) THEN title = ''
  IF NOT (KEYWORD_SET(uvalue)) THEN uvalue = 0
  IF NOT (KEYWORD_SET(font)) THEN font = ''

		;*********************************
		;**** Create the Input widget ****
		;*********************************

		;**** create base widget ****
  cwid = widget_base( parent, UVALUE = uvalue, $
			EVENT_FUNC = "in_event", $
			FUNC_GET_VALUE = "in_get_val", $
			/COLUMN)

		;*************************************
		;**** Input file selection widget ****
		;*************************************
  fileid = cds_cw_infile(cwid,value=inset,title=title,font=font)

		;*****************
		;**** Buttons ****
		;*****************
  base = widget_base(cwid, /row, space=50)

;
;  special font
;
cfont = "-adobe-helvetica-bold-r-normal--14-100-100-100-p-82-iso8859-1"
		;**** Done Button ****
  doneid = widget_button(base,value='Done',font=cfont)

		;**** Cancel Button ****
  cancelid = widget_button(base,value='Cancel',font=cfont)

		;**** Error message ****
  messid = widget_label(parent,font=font,value='*',xsize=40)

		;**** Check filename and desnsitise buttons if it ****
		;**** is a directory or it is a file without read ****
		;**** access.					  ****
  filename = inset.rootpath+inset.file
  file_acc,filename,fileexist,read,write,execute,filetype
  if filetype ne '-' then begin
    widget_control,doneid,sensitive=0
  end else begin
    if read eq 0 then begin
      widget_control,doneid,sensitive=0
    end
  end

		;**** create a state structure for the widget ****
  new_state = { FILEID:fileid,$
		DONEID:doneid, CANCELID:cancelid, MESSID:messid, $
		FONT:font }

                ;**** Save initial state structure ****
  CW_SAVESTATE, cwid, state_base, new_state

  RETURN, cwid

END

