;+
; Project     : SOHO - CDS     
;                   
; Name        : CDS_CW_FILE_SEL()
;               
; Purpose     : Allows UNIX system directory/file selection.
;               
; Explanation : 
;      This function produces a compound widget which allows the
;      user to step through a UNIX file system selecting directories
;      and files.  The widget consists of a non-editable text widget
;      which displays the name of the currently selected file/directory
;      and below it a selection list of all available choices
;      in the currently selected directory.  The '..' symbol, meaning
;      parent directory, may also appear in the selection list to allow
;      the user to back-track through the file system to explore
;      another directory path.  The file selection process starts
;      from a root directory which is specified by the caller.  The
;      user is not able select the parent of this 'root' directory.
;      This is useful for confining the user to a particular part
;      of the UNIX file-system hierarchy.
;
;      This widget generates events as the user interacts with it.
;      The event structure returned is;
;      {ID:0L, TOP:0L, HANDLER:0L, ACTION:''}
;      ACTION has one of two values indicating the state of the file
;      selection process, either 'nofile' when the current selection
;      is a directory or 'newfile' when the current selection is file.
;               
; Use         : An example of how to use this widget;
;
;               value = {ROOT:'/', FILE:''}
;               base = widget_base()
;               id = cds_cw_file_sel(base,VALUE=value)
;               widget_control,base,/realize
;
;               .... process events using widget_event or xmanager ....
;               rc = widget_event()
;
;               .... get the selected file ....
;               widget_control,id,get_value=value
;               filename = value.root+value.file
;
;               Note: this widget generates events at every user interaction.
;    
; Inputs      : PARENT - The ID of the parent widget.
;               
; Opt. Inputs : None
;               
; Outputs     : The return value of this function is the ID of the compound
;               widget which is of type LONG integer.
;               
; Opt. Outputs: None
;               
; Keywords    : VALUE      - A structure which must be;   {ROOT:'', FILE:''}
;               The elements of the structure are as follows;
;
;               ROOT - Root directory e.g '/usr/fred/adas/'
;               FILE - Current data file in ROOT e.g 'adf/input.dat'
;
;                      The caller must ensure that ROOT and FILE are valid
;                      names otherwise this widget may cause an error.
;                      Both ROOT and FILE may be set to '' on entry.  This
;                      will cause ROOT to be set to the user's current
;                      directory './'.
;
;                      The default value is;  fselval = {ROOT:'./', FILE:''}
;
;               TITLE  - A title to be added to the left of the selection list.
;
;               XSIZE  - The width in characters of the current selection text
;                        widget.
;
;               YSIZE  - The height of the selection list in rows of characters.
;
;               FONT   - A font to use for all text in this widget.
;
;               UVALUE - A user value for this widget.
;
;
; Calls       : FILE_ACC      Determine file type and access.
;               EXTRACT_PATH      Extract directory path to current file from
;                  full name (Routine in this file).
;               CW_LOADSTATE    Recover compound widget state.
;               CW_SAVESTATE    Save compound widget state.
;
;               Three other routines are included which are used to manage the
;               widget;
;
;               FILESEL_SET_VAL
;               FILESEL_GET_VAL()
;               FILESEL_EVENT()
;
;               One extra utility routine is included;
;
;               EXTRACT_PATH()
;
;               
; Restrictions: None
;               
; Side effects: This widget uses a COMMON BLOCKs: 
;
;               CW_FILESEL_BLK to hold the widget state.
;               CDS_MULTI_FILES to hold the file selections.
;
; Category    : Util, widgets
;               
; Prev. Hist. : ADAS version by Andrew Bowen, 
;                               Tessella Support Services plc, 13-May-1993
;
; Written     : CDS multi-file version for EGSE by C D Pike, RAL, 7-Oct-93
;               
; Modified    : Version 1, 07-Oct-1993, C D Pike, RAL
;		Version 2, 05-Jan-2000, William Thompson, GSFC
;			Handle both formats of tm.* filenames
;
; Version     : Version 2, 05-Jan-2000
;-            


FUNCTION extract_path, filename

;**** Find the end of the directory path ***
  pathend = -1
  i = strlen(filename) - 1
  while (i gt 0 and pathend eq -1) do begin
    if strmid(filename,i,1) eq '/' then pathend = i
    i = i - 1
  end

;**** Extract directory path ****
  if pathend gt 0 then begin
    path = strmid(filename,0,pathend)
  end else begin
    path = ''
  end

  RETURN, path
END

;-----------------------------------------------------------------------------

PRO filesel_set_val, id, value

  COMMON cw_filesel_blk, state_base, state_stash, state

;**** Return to caller on error ****
  ON_ERROR, 2

;**** Retrieve the state ****
  CW_LOADSTATE, id, state_base, state_stash, state

;**** Tidy pathname and rootname to conform to ****
;**** routine pathname convention             ****
  if strtrim(value.root) eq '' then begin
    value.root = './'
  end else if strmid(value.root,strlen(value.root)-1,1) ne '/' then begin
    value.root = value.root+'/'
  end
  if strmid(value.file,0,1) eq '/' then begin
    value.file = strmid(value.file,1,strlen(value.file)-1)
  end

;**** File name ****
  widget_control,state.fileid,set_value=value.file

;**** See if the new 'file' is a file or a directory ****
  file = value.root+value.file
  file_acc,file,fileexist,read,write,execute,filetype

  if filetype ne 'd' then begin
;**** Extract directory path ****
    file = extract_path(file)
  end

;**** Get list of files in directory ****
  list = findfile(file)
  seelist=list
  ntm = where(strmid(seelist,0,3) eq 'tm.')
  if ntm(0) ge 0 then begin
     test = strmid(seelist(ntm),0,4)
     ntm0 = where(test ne 'tm.1')
     if ntm0(0) ge 0 then begin
	ntm0 = ntm(ntm0)
        year = fix(strmid(list(ntm0),3,2))
	year = ((year + 50) mod 100) + 1950
	doy = fix(strmid(list(ntm0),6,3))
	date = ' ('+strmid(anytim2cal(doy2utc(doy,year),f=1),0,6)+')'
	seelist(ntm0) = seelist(ntm0)+date
     endif
     ntm1 = where(test eq 'tm.1')
     if ntm1(0) ge 0 then begin
	ntm1 = ntm(ntm1)
	year = fix(strmid(list(ntm1),4,2))
	year = ((year + 50) mod 100) + 1950
	doy = fix(strmid(list(ntm1),7,3))
	date = ' ('+strmid(anytim2cal(doy2utc(doy,year),f=1),0,6)+')'
	seelist(ntm1) = seelist(ntm1)+date
     endif
  endif

;**** If the current directory is not 'root' ****
;**** add the '..' symbol to the list.       ****
  if strtrim(value.file) ne '' then begin
    if strtrim(list(0)) ne '' then begin
      list = ['..',list]
    end else begin
      list = ['..']
    end
  end

;**** Set selection list ****
  widget_control,state.selid,set_value=seelist

  fselval = {fselval, ROOT:value.root, FILE:value.file }
;**** Recreate state structure to hold new list ****
  new_state =  { FILEID:state.fileid, SELID:state.selid, $
             LIST:list, FSELVAL:fselval }

;**** Save the new state ****
  CW_SAVESTATE, id, state_base, new_state

END

;-----------------------------------------------------------------------------


FUNCTION filesel_get_val, id

  COMMON cw_filesel_blk, state_base, state_stash, state

;**** Return to caller on error ****
  ON_ERROR, 2

;**** Retrieve the state ****
  CW_LOADSTATE, id, state_base, state_stash, state

  RETURN, state.fselval

END

;-----------------------------------------------------------------------------

FUNCTION filesel_event, event

  COMMON cw_filesel_blk, state_base, state_stash, state
  COMMON cds_multi_files, multi_file_names, num_mfn


;**** Base ID of compound widget ****
  base=event.handler

;**** Retrieve the state ****
  CW_LOADSTATE, base, state_base, state_stash, state

;**** Desensitise selection list to prevent ****
;**** double click.                         ****
  widget_control,state.selid,sensitive=0

;**** The default event is no file selected ****
  action = 'nofile'

;************************
;**** Process Events ****
;************************
  CASE event.id OF

;******************************
;**** Selection list event ****
;******************************
    state.selid: begin


;**** See if the current 'file' is a file or directory ****
      file = state.fselval.root+state.fselval.file
      file_acc,file,fileexist,read,write,execute,filetype

      if state.list(event.index) eq '..' then begin
      ;*****************************
      ;**** Move up a directory ****
      ;*****************************
        if filetype ne 'd' then begin
;**** strip off two names for a file ***
          state.fselval.file = extract_path(state.fselval.file)
        end
        state.fselval.file = extract_path(state.fselval.file)

        widget_control,state.fileid,set_value=state.fselval.file
        file = state.fselval.root+state.fselval.file
        list = findfile(file)
        seelist=list
        ntm = where(strmid(seelist,0,3) eq 'tm.')
        if ntm(0) ge 0 then begin
           test = strmid(seelist(ntm),0,4)
	   ntm0 = where(test ne 'tm.1')
	   if ntm0(0) ge 0 then begin
	      ntm0 = ntm(ntm0)
	      year = fix(strmid(list(ntm0),3,2))
	      year = ((year + 50) mod 100) + 1950
	      doy = fix(strmid(list(ntm0),6,3))
	      date = ' ('+strmid(anytim2cal(doy2utc(doy,year),f=1),0,6)+')'
	      seelist(ntm0) = seelist(ntm0)+date
	   endif
	   ntm1 = where(test eq 'tm.1')
	   if ntm1(0) ge 0 then begin
	      ntm1 = ntm(ntm1)
	      year = fix(strmid(list(ntm1),4,2))
	      year = ((year + 50) mod 100) + 1950
	      doy = fix(strmid(list(ntm1),7,3))
	      date = ' ('+strmid(anytim2cal(doy2utc(doy,year),f=1),0,6)+')'
	      seelist(ntm1) = seelist(ntm1)+date
	   endif
        endif

        if strtrim(state.fselval.file) ne '' then begin
          if strtrim(list(0)) ne '' then begin
            list = ['..',list]
          end else begin
            list = ['..']
          end
        end
        widget_control,state.selid,set_value=seelist


      end else begin
      ;*******************************************
      ;**** Down a directory or select a file ****
      ;*******************************************
;**** Determine the new file name ****
        if filetype eq 'd' then begin
          if strtrim(state.fselval.file) eq '' then begin
            state.fselval.file = state.list(event.index)
          end else begin
            state.fselval.file = $
                        state.fselval.file+'/'+state.list(event.index)
          end
        end else begin
          path = extract_path(state.fselval.file)
          if strtrim(path) eq '' then begin
            state.fselval.file = state.list(event.index)
          end else begin
            state.fselval.file = path+'/'+state.list(event.index)
          end
        end

;**** Display new name ****
        widget_control,state.fileid,get_value=fred
          if fred(0) eq '' then insert = ' ' else insert = ', '
        widget_control,state.fileid,set_value=fred+insert+state.fselval.file

;**** See if the new 'file' is a file or directory ****
        file = state.fselval.root+state.fselval.file
        file_acc,file,fileexist,read,write,execute,filetype
  
;**** Update list if new selection is a directory ****
        if filetype eq 'd' then begin
          list = findfile(file)
          seelist=list
          ntm = where(strmid(seelist,0,3) eq 'tm.')
          if ntm(0) ge 0 then begin
           test = strmid(seelist(ntm),0,4)
	   ntm0 = where(test ne 'tm.1')
	   if ntm0(0) ge 0 then begin
	      ntm0 = ntm(ntm0)
	      year = fix(strmid(list(ntm0),3,2))
	      year = ((year + 50) mod 100) + 1950
	      doy = fix(strmid(list(ntm0),6,3))
	      date = ' ('+strmid(anytim2cal(doy2utc(doy,year),f=1),0,6)+')'
	      seelist(ntm0) = seelist(ntm0)+date
	   endif
	   ntm1 = where(test eq 'tm.1')
	   if ntm1(0) ge 0 then begin
	      ntm1 = ntm(ntm1)
	      year = fix(strmid(list(ntm1),4,2))
	      year = ((year + 50) mod 100) + 1950
	      doy = fix(strmid(list(ntm1),7,3))
	      date = ' ('+strmid(anytim2cal(doy2utc(doy,year),f=1),0,6)+')'
	      seelist(ntm1) = seelist(ntm1)+date
	   endif
          endif

          if strtrim(state.fselval.file) ne '' then begin
            if strtrim(list(0)) ne '' then begin
            list = ['..',list]
            end else begin
            list = ['..']
            end
          end
          widget_control,state.selid,set_value=seelist
        end else begin
          list = state.list
          action = 'newfile'
        end

      end
      end

    ELSE:

  ENDCASE

;**** recreate state structure to fit in new list ****
  new_state =  { FILEID:state.fileid, SELID:state.selid, $
             LIST:list, FSELVAL:state.fselval }

  
;
;  store file name in array
;
  num_mfn = num_mfn + 1
  multi_file_names(num_mfn) = new_state.fselval.root+new_state.fselval.file


;**** Save the new state structure ****
  CW_SAVESTATE, base, state_base, new_state

;**** Re-sensitise selection list ****
  widget_control,state.selid,/sensitive

;**** Generate event indicating if file is selected ****
  RETURN, {ID:base, TOP:event.top, HANDLER:0L, ACTION:action}

END

;-----------------------------------------------------------------------------

FUNCTION cds_cw_file_sel, parent, VALUE=value, TITLE=title, $
            XSIZE=xsize, YSIZE=ysize, FONT=font, UVALUE=uvalue


  COMMON cw_filesel_blk, state_base, state_stash, state

  IF (N_PARAMS() LT 1) THEN MESSAGE, 'Must specify PARENT for cds_cw_file_sel'

;**** Set defaults for keywords ****
  IF NOT (KEYWORD_SET(value)) THEN begin
    fselval = {fselval, ROOT:'./', FILE:''}
  END ELSE BEGIN
    fselval = {fselval, ROOT:value.root, FILE:value.file }
    if strtrim(fselval.root) eq '' then begin
      fselval.root = './'
   end else if strmid(fselval.root,strlen(fselval.root)-1,1) ne '/' then begin
      fselval.root = fselval.root+'/'
    end
    if strmid(fselval.file,0,1) eq '/' then begin
      fselval.file = strmid(fselval.file,1,strlen(fselval.file)-1)
    end
  END

  IF NOT (KEYWORD_SET(title)) THEN title = 'Telemetry File'
  IF NOT (KEYWORD_SET(xsize)) THEN xsize = 60
  IF NOT (KEYWORD_SET(ysize)) THEN ysize = 15
  IF NOT (KEYWORD_SET(font)) THEN font = ''
  IF NOT (KEYWORD_SET(uvalue))  THEN uvalue = 0

;**** Create the main base for the widget ****
  cwid = WIDGET_BASE(parent, UVALUE = uvalue, $
            EVENT_FUNC = "filesel_event", $
            FUNC_GET_VALUE = "filesel_get_val", $
            PRO_SET_VALUE = "filesel_set_val", $
            /ROW)

;**** Title ****
  rc = widget_label(cwid,value=title,font=font)

  base = widget_base(cwid,/column)

;**** File name ****
  fileid = widget_text(base,value=fselval.file,xsize=xsize,font=font)

;**** Selection list ****
  selid = widget_list(base,ysize=ysize,font=font)

;**** See if the current 'file' is a file ****
;**** or a directory.                     ****
  listdir = fselval.root+fselval.file
  file_acc,listdir,fileexist,read,write,execute,filetype

  if filetype ne 'd' then begin
;**** Extract directory path ****
    listdir = extract_path(listdir)
  end

;**** Get list of files in directory ****
  list = findfile(listdir)
  seelist=list
  ntm = where(strmid(seelist,0,3) eq 'tm.')
  if ntm(0) ge 0 then begin
     test = strmid(seelist(ntm),0,4)
     ntm0 = where(test ne 'tm.1')
     if ntm0(0) ge 0 then begin
	ntm0 = ntm(ntm0)
	year = fix(strmid(list(ntm0),3,2))
	year = ((year + 50) mod 100) + 1950
	doy = fix(strmid(list(ntm0),6,3))
	date = ' ('+strmid(anytim2cal(doy2utc(doy,year),f=1),0,6)+')'
	seelist(ntm0) = seelist(ntm0)+date
     endif
     ntm1 = where(test eq 'tm.1')
     if ntm1(0) ge 0 then begin
	ntm1 = ntm(ntm1)
	year = fix(strmid(list(ntm1),4,2))
	year = ((year + 50) mod 100) + 1950
	doy = fix(strmid(list(ntm1),7,3))
	date = ' ('+strmid(anytim2cal(doy2utc(doy,year),f=1),0,6)+')'
	seelist(ntm1) = seelist(ntm1)+date
     endif
  endif

;**** If the current directory is not 'root' ****
;**** add the '..' symbol to the list.       ****
  if strtrim(fselval.file) ne '' then begin
    if strtrim(list(0)) ne '' then begin
      list = ['..',list]
    end else begin
      list = ['..']
    end
  end

;**** Set selection list ****

  widget_control,selid,set_value=seelist

;**** Create state structure ****
  new_state =  { FILEID:fileid, SELID:selid, LIST:list, FSELVAL:fselval }

;**** Save initial state structure ****
  CW_SAVESTATE, cwid, state_base, new_state

  RETURN, cwid

END
