;+
; Project     : SOHO - CDS     
;                   
; Name        : CDS_CW_INFILE()
;               
; Purpose     : Data file selection for an input dataset.
;               
; Explanation : This function creates a compound widget consisting of the
;               compound widget cds_root, compound widget cds_cw_file_sel,
;               a title and a message.  This widget provides its own error
;               messages.
;
;               This widget generates events as the user interacts with it.
;               The event structure returned is;
;               {ID:0L, TOP:0L, HANDLER:0L, ACTION:''}
;               ACTION has one of two values 'nofile' or 'newfile' indicating
;               the current state of the file selection process.
;
;               
; Use         : See routine cds_cw_file_in.pro for an example. 
;    
; Inputs      : PARENT   - Long integer; the ID of the parent widget.  
;               
; Opt. Inputs : 
;               
; Outputs     : The return value of this function is the ID of the compound
;               widget which is of type LONG integer.
;               
; Opt. Outputs: None
;               
; Keywords    : VALUE -  A structure which determines the initial settings of
;                        widgets cw_root and cds_cw_file_sel.
;                        The structure must be;
;                        {ROOTPATH:'', FILE:'', CENTROOT:'', USERROOT:'' }
;                        The elements of the structure are as follows;
;                        
;                        ROOTPATH - Current data directory e.g '/usr/fred/'
;                        FILE     - Current data file in ROOTPATH e.g 'in.dat'
;                        CENTROOT - Default central data store e.g '/usr/adas/'
;                        USERROOT - Default user data store e.g '/usr/fred/'
;                        
;                        The data file selected by the user is obtained by
;                        appending ROOTPATH and FILE.  In the above example
;                        the full name of the data file is;
;                        /usr/fred/adas/input.dat
;
;                        Path names may be supplied with or without the trailing
;                        '/'.  The widget cds_root will add this where
;                        required so that USERROOT will always end in '/' on
;                        output.
;
;                        The default value is;
;                            {ROOTPATH:'./', FILE:'', CENTROOT:'', USERROOT:''}
;                        i.e ROOTPATH is set to the user's current directory.
;
;               TITLE -  The title to be included in this widget, used
;                        to indicate exactly what the required input dataset is,
;                        e.g 'Input VDS Dataset'
;
;               FONT   - A font to use for all text in this widget.
;
;               UVALUE - A user value for this widget.
;
; Calls       : CW_LOADSTATE           Recover compound widget state.
;               CW_SAVESTATE           Save compound widget state.
;               CDS_ROOT               Used to select root data directory.
;               CDS_CW_FILE_SEL        Used to select a UNIX file.
;
;               Three other routines are included which are used to manage the
;               widget;
;
;               INFILE_GET_VAL()
;               INFILE_EVENT()
;               
; Restrictions: 
;               
; Side effects: This widget uses a COMMON BLOCK: CW_INFILE_BLK to hold the
;               widget state.
;               
; Category    : Util, widgets
;               
; Prev. Hist. : ADAS version by Andrew Bowen, 
;                               Tessella Support Services plc, 17-May-1993   
;
; Written     : CDS EGSE version by C D Pike, RAL, 7-Oct-93
;               
; Modified    : 
;
; Version     : Version 1, 7-Oct-93
;-            
;-----------------------------------------------------------------------------


FUNCTION infile_get_val, id

  COMMON cw_infile_blk, state_base, state_stash, state

;**** Return to caller on error ****
  ON_ERROR, 2

;**** Retrieve the state ****
  CW_LOADSTATE, id, state_base, state_stash, state

;**** Get latest root and file name ****
  widget_control,state.fileid,get_value=fselval

;**** Update state ****
  state.infileval.file = fselval.file
  state.infileval.rootpath = fselval.root


  RETURN, state.infileval

END

;-----------------------------------------------------------------------------

FUNCTION infile_event, event

  COMMON cw_infile_blk, state_base, state_stash, state

;**** Base ID of compound widget ****
  base=event.handler

;**** Retrieve the state ****
  CW_LOADSTATE, base, state_base, state_stash, state

;**** Clear any existing error message ****
  widget_control,state.messid,set_value=' '

;************************
;**** Process Events ****
;************************
  CASE event.id OF

    state.rootid: begin
         if event.action eq 'newroot' then begin

;**** Reset the value of the file selection widget ****
           widget_control,state.rootid,get_value=rootval
           fselval = { ROOT:rootval.rootpath, FILE:'' }
           widget_control,state.fileid,set_value=fselval

;**** Update the state ****
           state.infileval.file = ''
           state.infileval.rootpath = rootval.rootpath

;**** Resensitise file selection ****
           widget_control,state.fileid,/sensitive

         end else if event.action eq 'rootedit' then begin

;**** When path is being edited desensitise file selection ****
           widget_control,state.fileid,sensitive=0

         end
         action = 'nofile'
       end

    state.fileid: begin
         if event.action eq 'nofile' then begin
           action = event.action
         end else begin
;**** Get latest root and file name ****
           widget_control,state.fileid,get_value=fselval
           filename=fselval.root+fselval.file

;**** Test for file read access ****
           file_acc,filename,exist,read,write,execute,filetype
           if read eq 0 then begin
             action = 'nofile'
             widget_control,state.messid,set_value='No read access for file'
           end else begin
             action = 'newfile'
           end
         end
       end

    ELSE:

  ENDCASE

;**** Save the new state structure ****
  CW_SAVESTATE, base, state_base, state

  RETURN, {ID:base, TOP:event.top, HANDLER:0L, ACTION:action}

END

;-----------------------------------------------------------------------------

FUNCTION cds_cw_infile, parent, $
              VALUE=value, TITLE=title, FONT=font, UVALUE=uvalue


  COMMON cw_infile_blk, state_base, state_stash, state

  IF (N_PARAMS() LT 1) THEN MESSAGE, 'Must specify PARENT for cds_root'

;**** Set defaults for keywords ****
  IF NOT (KEYWORD_SET(value)) THEN BEGIN
    infileval = {infileval, ROOTPATH:'./', $
                         FILE:'', $
                         CENTROOT:'', $
                         USERROOT:'' }
  END ELSE BEGIN
    infileval = {infileval, ROOTPATH:value.rootpath, $
                         FILE:value.file, $
                         CENTROOT:value.centroot, $
                         USERROOT:value.userroot }
    if strtrim(infileval.rootpath) eq '' then begin
      infileval.rootpath = './'
    end else if strmid(infileval.rootpath,strlen(infileval.rootpath)-1,1) $
                                                      ne '/' then begin
      infileval.rootpath = infileval.rootpath+'/'
    end
    if strmid(infileval.file,0,1) eq '/' then begin
      infileval.file = strmid(infileval.file,1,strlen(infileval.file)-1)
    end
  END

  IF NOT (KEYWORD_SET(title)) THEN title = ''
  IF NOT (KEYWORD_SET(font)) THEN font = ''
  IF NOT (KEYWORD_SET(uvalue))  THEN uvalue = 0

;**** Create the main base for the widget ****
  cwid = WIDGET_BASE(parent, UVALUE = uvalue, $
              EVENT_FUNC = "infile_event", $
              FUNC_GET_VALUE = "infile_get_val", $
              /COLUMN)

;**** Title for input file ****
  rc = widget_label(cwid,value=title,font=font)

;**** Root path name widget ****
  rootval = {       ROOTPATH:infileval.rootpath, $
              CENTROOT:infileval.centroot, $
              USERROOT:infileval.userroot }
  rootid = cds_root(cwid,value=rootval,font=font)

 ;**** File selection widget ****
  fselval = {       ROOT:infileval.rootpath, FILE:infileval.file }
  fileid = cds_cw_file_sel(cwid,value=fselval,font=font)

;**** Message ****
  messid = widget_label(cwid,value=' ',xsize=40,font=font)

;**** Test for file read access ****
  filename = infileval.rootpath+infileval.file
  file_acc,filename,exist,read,write,execute,filetype
  if filetype eq '-' and read eq 0 then begin
    widget_control,messid,set_value='No read access for file'
  end
  
;**** Create state structure ****
  new_state = { ROOTID:rootid, FILEID:FILEID, MESSID:messid, $
              INFILEVAL:infileval, FONT:font }

;**** Save initial state structure ****
  CW_SAVESTATE, cwid, state_base, new_state

  RETURN, cwid

END
