;+
; Project     : SOHO - CDS     
;                   
; Name        : CDS_FILES_IN
;               
; Purpose     : Collect an array of user-selected file names.
;               
; Explanation : This routine creates and manages a pop-up window which allows
;               the user to select files from the UNIX file system.  It is
;               intended as a general routine and hence the widget titles may
;               be passed into this routine using keywords.
;
;               
; Use         : IDL> cds_files_in, files, value
;    
; Inputs      : None although if the environment variable CDS_TM_DATA is
;               defined it should point to a directory which can be referenced
;               within the program as the "Central data" location.
;               
; Opt. Inputs : value - a structure of the form
;                       {rootpath:'',file:'',centroot:'',userroot:''}
;                       If this is omitted the tag 'centroot' is set to
;                       the value of CDS_TM_DATA and if this env. var. is
;                       not set then all directories default to the current.
;               
; Outputs     : files - a string array containing the path and names of the
;                       selected files.
;               
; Opt. Outputs: None
;               
; Keywords    : WINTITLE - A title to be used on the banner for the pop-up 
;                          window. Intended to indicate which application 
;                          is running e.g 'CDS replay telemetry'
;
;               TITLE    - The title to be included in the input file widget, 
;                          used to indicate exactly what the required input 
;                          dataset is,  e.g 'Input CDS Dataset'
;
;               FONT     - Supplies the font to be used for the interface 
;                          widgets.
;
; Calls       : CDS_CW_FILE_IN       Dataset selection widget creation.
;               XMANAGER             Manages the pop=up window.
;               CDS_FILE_IN_EV       Event manager, called indirectly during 
;                                    XMANAGER event management.
;               
; Restrictions: None
;               
; Side effects: XMANAGER is called in /modal mode. Any other widget become
;               inactive.
;               
; Category    : Util, files
;               
; Prev. Hist. : ADAS version by Andrew Bowen, 
;                               Tessella Support Services plc, 17-May-1993
;
; Written     : CDS EGSE version by C D Pike, RAL, 7-Oct-93
;               
; Modified    : Include cancel button.  CDP, 18-Jan-95
;               Make CDS_TM_DATA default start directory.  CDP, 26-May-95
;
; Version     : Version 3, 26-May-95
;-            
;-----------------------------------------------------------------------------

PRO cds_file_in_ev, event

  COMMON in_blk,action,value

;**** Find the event type and copy to common ****
    action = event.action

    CASE action OF

;**** 'Done' button ****
       'Done'  : begin

       ;**** Get the output widget value ****
              widget_control,event.id,get_value=value 
              widget_control,event.top,/destroy

          end


;**** 'Cancel' button ****
       'Cancel': begin
                    widget_control,event.top,/destroy
                 end
    END

END

;-----------------------------------------------------------------------------


PRO cds_files_in, files, val, WINTITLE = wintitle, TITLE = title, FONT = font

  COMMON in_blk,action,value
  COMMON cds_multi_files, multi_file_names, num_mfn


;
; if directory/file structure not supplied then try env. var.
; if that no good then set to zero which forces default to current directory.
;
  if n_params() lt 2 then begin
     dir = getenv("CDS_TM_DATA") 
     if dir eq "" then begin
        val = 0
     endif else begin
        val = {rootpath:dir,file:'',centroot:dir,userroot:''}
     endelse
   endif


;**** Copy value to common ****
  value = val

;
;  set up array and counter for multi-file names
;
   multi_file_names = strarr(50)
   num_mfn = -1

;**** Set defaults for keywords ****
  IF NOT (KEYWORD_SET(wintitle)) THEN $
                      wintitle = 'Science analysis input data files.'
  IF NOT (KEYWORD_SET(title)) THEN title = ''
  IF NOT (KEYWORD_SET(font)) THEN font = ''

;***************************************
;**** Pop-up a new widget if needed ****
;***************************************

  ;**** create base widget ****
  inid = widget_base(TITLE=wintitle,XOFFSET=100,YOFFSET=100)

;**** Declare output options widget ****
  cwid = cds_cw_file_in(inid, VALUE=value, TITLE=title, FONT=font)

;**** Realize the new widget ****
  widget_control,inid,/realize

;**** make widget modal ****
  xmanager,'cds_file_in',inid,event_handler='cds_file_in_ev',/modal,/just_reg
 
;
;  Return the output file list (removing duplicates which are easy to generate
;  when the system response is slow).
;
   
   files = multi_file_names(rem_dup(multi_file_names))
   n = where(files ne '')
   if n(0) ge 0 then begin
      files = files(n)
   endif else begin
      files = ''
   endelse

END

