;+
; Project     : SOHO - CDS     
;                   
; Name        : CDS_ROOT()
;               
; Purpose     : Root directory selection for use in input file selection.
;               
; Explanation : 
;       This function creates a compound widget consisting of;
;       1. A text widget with the current root directory
;          path name.
;       2. A button to set the root to a user default root.
;       3. A button to set the root to a central default root.
;       4. A toggle button to enable/disable direct keyboard entry
;          of the root.
;       This widget performs is own checking and validation of the
;       path and issues its own error messages.
;
;       This widget generates events as the user interacts with it.
;       The event structure returned is;
;       {ID:0L, TOP:0L, HANDLER:0L, ACTION:''}
;       ACTION has one of two values indicating the state of the root
;       selection process, either 'rootedit' when the root is being
;       edited and is not verified or 'newroot' when a new verified
;       root has been selected.
;
;       To operate the widget the user can press the 'User Data' or
;       'Central Data' buttons to copy those default roots to the text
;       widget showing the current root.  Alternatively the 'Edit Path Name'
;       toggle can be switched on to allow direct keyboard entry of the
;       current root.  To complete keyboard entry the user either unsets
;       the toggle button or presses the return key.  Whenever a new root
;       name is set, either with defaults buttons or by direct entry,
;       the path name is checked.  Any error is flagged with a message
;       and the widget is automatically put into 'Edit Path Name' mode
;       until a valid path is entered.  Every time the path is changed
;       an event is generated as described above.  This is to notify
;       any dependent IDL routines of the change.
;               
; Use         : See cds_cw_file_in.pro for an example of use. 
;    
; Inputs      : PARENT   - The ID of the parent widget.
;               
; Opt. Inputs : None
;               
; Outputs     : The return value of this function is the ID of the compound
;               widget which is of type LONG integer.
;               
; Opt. Outputs: None
;               
; Keywords    : VALUE - A structure which must be;
;                       {ROOTPATH:'', CENTROOT:'', USERROOT:'' }
;                       The elements of the structure are as follows;
;
;               ROOTPATH - Current data directory e.g '/usr/fred/adas/'
;               CENTROOT - Default central data store e.g '/usr/adas/'
;               USERROOT - Default user data store e.g '/usr/fred/adas/'
;
;                          Path names may be supplied with or without the 
;                          trailing '/'.  This widget will add this where 
;                          required so that USERROOT will always end in '/' 
;                          on output.  The default value is;
;                          {ROOTPATH:'./', CENTROOT:'', USERROOT:''}
;                          i.e ROOTPATH is set to the user's current directory.
;
;               FONT     - A font to use for all text in this widget.
;
;               UVALUE   - A user value for this widget.
;
; Calls       : FILE_ACC       Used to verify path name.
;               CW_LOADSTATE    Recover compound widget state.
;               CW_SAVESTATE    Save compound widget state.
;
;               Three other routines are included which are used to 
;               manage the widget;
;
;               ROOT_SET_VAL
;               ROOT_GET_VAL()
;               ROOT_EVENT()
;
;               
; Restrictions: None
;               
; Side effects: This widget uses a COMMON BLOCK: CW_ROOT_BLK to hold the
;               widget state.
;               
; Category    : Util, widgets
;               
; Prev. Hist. : ADAS version by Andrew Bowen, 
;                               Tessella Support Services plc, 13-May-1993  
;
; Written     : CDS EGSE version by C D Pike, RAL, 7-Oct-93
;               
; Modified    : 
;
; Version     : Version 1, 7-Oct-93
;-            
;-----------------------------------------------------------------------------

PRO root_set_val, id, value

  COMMON cw_root_blk, state_base, state_stash, state

;**** Return to caller on error ****
  ON_ERROR, 2

;**** Retrieve the state ****
  CW_LOADSTATE, id, state_base, state_stash, state

;**** Tidy path name to conform to routine convention ****
  if strtrim(value.rootpath) eq '' then begin
    value.rootpath = './'
  end else if strmid(value.rootpath,strlen(value.rootpath)-1,1) ne $
                                                        '/' then begin
    value.rootpath = value.rootpath+'/'
  end
;**** Copy new rootname to widget ****
  widget_control,state.rootid,set_value=value.rootpath

;**** Copy the new value to state structure ****
  state.rootval.rootpath = value.rootpath
  state.rootval.centroot = value.centroot
  state.rootval.userroot = value.userroot

;**** Save the new state ****
  CW_SAVESTATE, id, state_base, state

END

;-----------------------------------------------------------------------------


FUNCTION root_get_val, id

  COMMON cw_root_blk, state_base, state_stash, state

;**** Return to caller on error ****
  ON_ERROR, 2

;**** Retrieve the state ****
  CW_LOADSTATE, id, state_base, state_stash, state


  RETURN, state.rootval

END

;-----------------------------------------------------------------------------

FUNCTION root_event, event

  COMMON cw_root_blk, state_base, state_stash, state

;**** Base ID of compound widget ****
  base=event.handler

;**** Retrieve the state ****
  CW_LOADSTATE, base, state_base, state_stash, state

;************************
;**** Process Events ****
;************************
  CASE event.id OF


    state.centid: begin
         widget_control,state.rootid,set_value=state.rootval.centroot
         state.rootval.rootpath = state.rootval.centroot
         if state.editon eq 1 then begin
           action = ''
         end else begin
           action = 'newroot'
         end
       end

    state.userid: begin
         widget_control,state.rootid,set_value=state.rootval.userroot
         state.rootval.rootpath = state.rootval.userroot
         if state.editon eq 1 then begin
           action = ''
         end else begin
           action = 'newroot'
         end
       end

    state.editid: if state.editon eq 0 then begin
         state.editon = 1
         widget_control,state.rootid,/sensitive,/input_focus
         action = 'rootedit'
       end else begin
         state.editon = 0
         widget_control,state.rootid,sensitive=0
         widget_control,state.rootid,get_value=rootpath
         state.rootval.rootpath = rootpath(0)
         action = 'newroot'
       end

    state.rootid: begin
         state.editon = 0
         widget_control,state.rootid,sensitive=0
         widget_control,state.editid,set_button=0
         widget_control,state.rootid,get_value=rootpath
         state.rootval.rootpath = rootpath(0)
         action = 'newroot'
       end

    ELSE:

  ENDCASE

;**** Clear any existing error message ****
  widget_control,state.messid,set_value=' '

;**** Check if the supplied name is okay     ****
;**** Force edit mode in the event of error. ****
;**** Tidy path name if it is okay.          ****
  if action eq 'newroot' then begin
    file_acc,state.rootval.rootpath,fileexist,read,write,execute,filetype
    if fileexist ne 1 then begin
      error = 1
      message = 'Error: The supplied name is invalid.'
      widget_control,state.messid,set_value=message
      state.editon = 1
      widget_control,state.editid,set_button=1
      widget_control,state.rootid,/sensitive,/input_focus
      action = 'rootedit'
    end else begin
      if strtrim(state.rootval.rootpath) eq '' then begin
        state.rootval.rootpath = './'
      end else if $
         strmid(state.rootval.rootpath,strlen(state.rootval.rootpath)-1,1) $
                                                 ne '/' then begin
        state.rootval.rootpath = state.rootval.rootpath+'/'
      end
      widget_control,state.rootid,set_value=state.rootval.rootpath
    end
  end

;**** Save the new state structure ****
  CW_SAVESTATE, base, state_base, state

  if action ne '' then begin
    RETURN, {ID:base, TOP:event.top, HANDLER:0L, ACTION:action}
  end else begin
    RETURN, 0L
  end

END

;-----------------------------------------------------------------------------

FUNCTION cds_root, parent, VALUE=value, FONT=font, UVALUE=uvalue


COMMON cw_root_blk, state_base, state_stash, state

  IF (N_PARAMS() LT 1) THEN MESSAGE, 'Must specify PARENT for cds_root'

;**** Set defaults for keywords ****
  IF NOT (KEYWORD_SET(value)) THEN BEGIN
    rootval = {rootval, ROOTPATH:'./', $
                     CENTROOT:'', $
                     USERROOT:'' }
  END ELSE BEGIN
    rootval = {rootval, ROOTPATH:value.rootpath, $
                     CENTROOT:value.centroot, $
                     USERROOT:value.userroot }
    if strtrim(rootval.rootpath) eq '' then begin
      rootval.rootpath = './'
    end else if strmid(rootval.rootpath,strlen(rootval.rootpath)-1,1) ne $
                                                        '/' then begin
      rootval.rootpath = rootval.rootpath+'/'
    end
  END
  IF NOT (KEYWORD_SET(font)) THEN font = ''
  IF NOT (KEYWORD_SET(uvalue))  THEN uvalue = 0

;**** Create the main base for the widget ****
  cwid = WIDGET_BASE(parent, UVALUE = uvalue, $
              EVENT_FUNC = "root_event", $
              FUNC_GET_VALUE = "root_get_val", $
              PRO_SET_VALUE = "root_set_val", $
              /COLUMN)

;**** Root path name for data ****
  base = widget_base(cwid,/row)
  rc = widget_label(base,value='Data Root ',font=font)
  rootid = widget_text(base,value=rootval.rootpath,/edit,xsize=40,font=font)
  widget_control,rootid,sensitive=0

;**** Base for all buttons ****
  base = widget_base(cwid,/row)

;**** Default path name buttons ****
  rc = widget_base(base,/row)
  centid = widget_button(rc,value='Central Data',font=font)
  userid = widget_button(rc,value='User Data',font=font)

;**** Pathname override switch ****
  rc = widget_base(base,/row,/nonexclusive)
  editid = widget_button(rc,value='Edit Path Name',font=font)

;**** Message ****
  messid = widget_label(cwid,value=' ',xsize=40,font=font)

;**** Create state structure ****
  new_state = { ROOTID:rootid, CENTID:centid, USERID:userid, $
              EDITID:editid, MESSID:messid, EDITON:0, $
              ROOTVAL:rootval, FONT:font }

;**** Save initial state structure ****
  CW_SAVESTATE, cwid, state_base, new_state

  RETURN, cwid

END
