;+
; Project     : SOHO - CDS     
;                   
; Name        : DCMPDATA()
;               
; Purpose     : Decompress CDS telemetry data.
;               
; Explanation : The on board CDS software can compress the science telemtery
;               data in a number of ways.  This routine applies the relevant
;               decompression algorithm to the incoming data.  
;               
; Use         : new_data = dcmpdata(indata, windows, detector,scheme,$
;                                   option, orientation)
;    
; Inputs      : in_data -  a byte array of compressed science data.
;               windows -  a structure giving the data windows with which
;                          the data were extracted.  It has the form:
;                          {StartX:0, StartY:0, LengthX:0, LengthY:0}
;                          For the GIS, tags StartY and LengthY are set to
;                          0 and 1.
;               detector - set to 0/1 for the NIS/GIS
;               scheme   - the decompression scheme used (range 1-7)
;               option   - ??
;               orient   - the orientation of the data extraction 0/1 for
;                          spectral or spatial direction. For GIS, set to 0.
;                         
;                          All of the above are extracted from the science
;                          packet headers.
;               
; Opt. Inputs : None
;               
; Outputs     : Function returns a 16-bit integer array of decompressed data.
;               
; Opt. Outputs: None
;               
; Keywords    : None
;
; Calls       : The code for the following decompression routines are 
;               included in this file:
;               
;               copy
;               quasil
;               sumline
;               sumwin
;               trunk
;               user
;               vbwl
;               sumrows
;
;               Also uses : unsign, extbits.
;
; Restrictions: None
;               
; Side effects: None
;               
; Category    : EGSE
;               
; Prev. Hist. : Original by M K Carter, RAL
;
; Written     : EGSE version by C D Pike, RAL 7-Oct-93
;               
; Modified    : Switched reorientation flag, CDP 27-Oct-93
;
;		Version 3, William Thompson, GSFC, 12 November 1993.
;			Added /NTOHS to BYTEORDER for platform independence.
;
;               Confirm option values for blocksize in VBWL, CDP, 23-Aug-94
;
;               Delete the functions EXTBITS & EXTVBWLBLOCK from this file
;               as they are now separate files,    CDP, 21-Nov-95
;
;		Version 6, William Thompson, GSFC, 29 December 1995
;			Also delete UNMAP--this is also a separate file.
;
;		Version 7, MKC, 2/10/97
;			Added SUMROWS compression scheme.
;
; Version     : Version 7, 1/10/97
;-            

FUNCTION trunk, data, size

  wordpos = 0L
  bitpos  = 0

  exposure = INTARR(size)

  FOR i = 0L, size-1 DO BEGIN

    exposure(i) = extbits ( 12, data, wordpos, bitpos)      

  ENDFOR

  RETURN, exposure

END

FUNCTION vbwl, data, size, detector, option, orientation,$
               windows_st 

; local variables

  wordpos = 0L
  bitpos  = 0

  pixelpos = 0L

  exposure = INTARR(size)

; loop over windows in exposure

  FOR window = 0, N_ELEMENTS(windows_st)-1 DO BEGIN

;   set up blocksize

    opt = option AND '7'X

    IF  opt EQ 0 THEN BEGIN

;     default option

      IF detector EQ 1 THEN BEGIN

;       GIS detector

        blocksize = 16        

      ENDIF ELSE BEGIN

;       NIS detector

        IF orientation EQ 0 THEN BEGIN

;         spectral orientation

          blocksize = windows_st(window).LengthX

        ENDIF ELSE BEGIN

;         spatial orientation

          blocksize = windows_st(window).LengthY

        ENDELSE          

      ENDELSE

    ENDIF ELSE BEGIN

;     valid option
;
;  note that value of 126 is correct here to correspond to the CDHS s/w
;

      CASE opt OF
        1 : blocksize = 8
        2 : blocksize = 16
        3 : blocksize = 32
        4 : blocksize = 64
        5 : blocksize = 126
        6 : blocksize = 256
        7 : blocksize = 512
      ENDCASE

    ENDELSE

;   loop over blocks in window

    area = long(windows_st(window).LengthX) * $
           long(windows_st(window).LengthY)

    FOR blocks = 0, (area/blocksize)-1 DO BEGIN

;     decompress pixels in block 

      exposure(pixelpos:pixelpos+blocksize-1) = $
        extvbwlblock ( blocksize, data, wordpos, bitpos)

      pixelpos = pixelpos + LONG(blocksize)

    ENDFOR

;   decompress remaining pixels in window 

    remainder = area MOD blocksize

    IF remainder GT 0 THEN BEGIN

      exposure(pixelpos:pixelpos+remainder-1) = $
        extvbwlblock ( remainder, data, wordpos, bitpos)

      pixelpos = pixelpos + LONG(remainder)

    ENDIF

  ENDFOR

  RETURN, exposure

END

FUNCTION sumline, data, size, windows_st, orientation

  exposure = INTARR(size)

; find no. of lines in exposure

  lines = 0

  FOR i = 0, N_ELEMENTS(windows_st)-1 DO BEGIN

    IF orientation THEN BEGIN

;     spatial orientation  ( count no. of spectral columns)

      lines = lines + windows_st(i).LengthX

    ENDIF ELSE BEGIN

;     spectral orientation ( count no. of spatial rows)

      lines = lines + windows_st(i).LengthY

    ENDELSE        
  
  ENDFOR

; reformat data into 32 bit values

  vals = LONG ( data, 0, lines)

; set up exposure 

  pixelpos = 0L
  line     = 0

  FOR i = 0, N_ELEMENTS(windows_st)-1 DO BEGIN

    IF orientation THEN BEGIN

;     spatial orientation  

      pixels = windows_st(i).LengthY
      rows   = windows_st(i).LengthX

    ENDIF ELSE BEGIN

;     spectral orientation 

      pixels = windows_st(i).LengthX
      rows   = windows_st(i).LengthY

    ENDELSE        

    FOR p = 0, rows-1 DO BEGIN

      temp = INTARR(pixels)
      val  = FIX(vals(line)/pixels)
      temp (*) = val
      exposure(pixelpos:pixelpos+pixels-1) = temp

      pixelpos = pixelpos + LONG(pixels)
      line = line + 1

    ENDFOR

  ENDFOR

  RETURN, exposure

END

FUNCTION sumwin, data, size, windows_st

  exposure = INTARR(size)

  NumDataWindows = N_ELEMENTS(windows_st)

; reformat data into 32 bit values

  vals = LONG ( data, 0, 2, NumDataWindows)

; set up exposure 

  pixelpos = 0L

  FOR i = 0, NumDataWindows-1 DO BEGIN

    area = LONG(windows_st(i).LengthX)*LONG(windows_st(i).LengthY)

    val  = FIX ( (DOUBLE(vals(1,i))*DOUBLE('40000000'XL)*4.0D + $
                  DOUBLE(vals(0,i))) $
                  / DOUBLE(area) )

    exposure(pixelpos:pixelpos+area-1) = val

    pixelpos = pixelpos + area

  ENDFOR

  RETURN, exposure

END

FUNCTION quasil, data, size, detector, option

  COMMON quasiblock, qutab12, qutab16

  wordpos = 0L
  bitpos  = 0

  exposure = INTARR(size)

; find quasilog table

  IF option EQ 1 THEN BEGIN
    table = qutab12
  ENDIF ELSE IF option EQ 2 THEN BEGIN
    table = qutab16
  ENDIF ELSE BEGIN
    IF detector EQ 0 THEN BEGIN
      table = qutab12
    ENDIF ELSE BEGIN
      table = qutab16
    ENDELSE
  ENDELSE  

  FOR i = 0L, size-1 DO BEGIN

    exposure(i) = table( extbits ( 8, data, wordpos, bitpos) )     

  ENDFOR

  RETURN, exposure

END

FUNCTION copy, data, total_size

  RETURN, data(0:total_size-1L)

END

FUNCTION sumrows, data, size, windows_st, detector, option, orientation

; SUMROWS compression scheme :
;    No. of pixels in column direction is unchanged.
;    option rows are summed together to give a single line.
;    Odd rows left are summed to give an extra line.
;    If option = 0, all rows are summed
;    If option > number of rows, all rows are summed

; treat GIS data differently since GIS uses all 16 bits 
; ie in principle can use same code as NIS but then have problem
;    of putting unsigned 16 bit number stored as a LONG into INTARR.
;    COPY is faster anyway.

  IF detector EQ 1 THEN RETURN, copy ( data, size)

  ; set up array to hold expanded, decompressed data

  exposure = INTARR(size)

; Direction of extraction is controlled by orientation parameter.
; Get orientation independent definition of window rows and columns.
; NB make LONARR to avoid possible INT overflow for no. of pixels

  IF orientation THEN BEGIN

;     spatial orientation 

      cols = LONG(windows_st.LengthY)
      rows = LONG(windows_st.LengthX)

  ENDIF ELSE BEGIN

;     spectral orientation 

      cols = LONG(windows_st.LengthX)
      rows = LONG(windows_st.LengthY)

  ENDELSE        

; set up pointers to current position in exposure and position in input data array

  exposure_pos = 0L
  data_pos = 0L

; loop through windows in exposure

  FOR i = 0, N_ELEMENTS(windows_st)-1 DO BEGIN

;   get no of rows compressed to make a line of compressed data

    IF option EQ 0 THEN nrows = rows(i) ELSE nrows = option

;   find no. of lines of compressed data in window

    lines = (rows(i)+nrows-1) / nrows

;   get unsigned 16 bit data for current window into LONARR

    window_data = unsign ( data(data_pos:data_pos+lines*cols(i)-1) )
    data_pos = data_pos + lines*cols(i)

;   convert window data to 2-D array

    window_data = REFORM ( window_data, cols(i), lines)

;   get data average as FLTARR

    window_data = window_data/FLOAT(nrows)

;   correct last line average for odd rows at bottom of window
;   NB this upsets the statistics of the last line of data

    IF (rows(i) MOD nrows) NE 0 THEN window_data(*,lines-1) = (window_data(*,lines-1) * nrows) / (rows(i) MOD nrows)

;   expand window data

    window_data = REBIN(window_data, cols(i), lines*nrows, /SAMPLE)

;   extract appropriate data

    window_data = window_data ( *, 0:rows(i)-1 )

;   form 1-D array

    window_data = REFORM ( window_data, rows(i)*cols(i) )

;   convert data to INTARR 
;   NB This relies on average for NIS data only being 12 bits.
;      This process throws away some information but is done here for
;      compatability with following SW.
;      This may not be what is required elsewhere.

    window_data = NINT(window_data)
   
;   put into exposure array

    exposure ( exposure_pos:exposure_pos+N_ELEMENTS(window_data)-1) = window_data
    exposure_pos = exposure_pos + N_ELEMENTS(window_data)

  ENDFOR

  RETURN, exposure

END

FUNCTION user, data, total_size, windows_st, detector, scheme, option, orientation

  CASE scheme OF

    8  :  RETURN,  sumrows ( data, total_size, windows_st, detector, option, orientation)

    ELSE : RETURN, data

  ENDCASE

END

;
;  main routine
;

FUNCTION dcmpdata, data, external_windows_st, detector, scheme, option, orientation

;
;  Common for info widget id
;
@MainWidgets.com
@Images.com

; adjust windows structure for xbin and ybin

  windows_st = REPLICATE({LengthX:0,LengthY:0,VDS_xbin:vds_xbin,VDS_ybin:vds_ybin}, N_ELEMENTS(external_windows_st) )

  FOR i = 0, N_ELEMENTS(windows_st)-1 DO BEGIN
    windows_st(i).LengthX = external_windows_st(i).LengthX/windows_st(i).VDS_xbin
    windows_st(i).LengthY = external_windows_st(i).LengthY/windows_st(i).VDS_ybin
  ENDFOR
;
;  byteswap input data for consistency with IDL format
;
byteorder,data,/ntohs

;
; find no. of data elements
;

  total_size = 0L
  FOR i = 0, N_ELEMENTS(windows_st) -1 DO BEGIN 
     total_size = total_size + $
       LONG(windows_st(i).LengthX)*LONG(windows_st(i).LengthY)
  ENDFOR

; decompress data

  CASE scheme OF
    0 : begin
          widget_control,msg_widg,/append,$
             set_val="INVALID SCHEME for decompression."
        end
    1 : BEGIN
          widget_control,msg_widg,/append,$
             set_val="Decompressing data for <COPY SCHEME> Please wait...."
          exposure = copy ( data, total_size)
        END
    2 : BEGIN
          widget_control,msg_widg,/append,$
             set_val="Decompressing data for <TRUNK SCHEME> Please wait...."
          exposure = trunk( data, total_size )
        END
    3 : BEGIN
          widget_control,msg_widg,/append,$
             set_val="Decompressing data for <VBWL SCHEME> Please wait...."
          exposure = vbwl( data, total_size, $
            detector, option, orientation, windows_st )
        END
    4 : BEGIN
          widget_control,msg_widg,/append,$
             set_val="Decompressing data for <QUASIL SCHEME> Please wait...."
          exposure = quasil( data, total_size, detector, option )
        END
    5 : BEGIN
         widget_control,msg_widg,/append,$
             set_val="Decompressing data for <COPY SCHEME 5> Please wait...."
          exposure = data(0:total_size-1L)
        END
    6 : BEGIN
          widget_control,msg_widg,/append,$
             set_val="Decompressing data for <SUMLINE SCHEME> Please wait...."
          exposure = sumline( data, total_size, windows_st, orientation )
        END
    7 :BEGIN
          widget_control,msg_widg,/append,$
             set_val="Decompressing data for <SUMWIN SCHEME> Please wait...."
          exposure = sumwin ( data, total_size, windows_st)
        END
    ELSE : BEGIN
          widget_control,msg_widg,/append,$
             set_val=string('Decompressing data for SCHEME #',$
                            scheme,'> Please wait....',form='(a,i2,a)')
          exposure = user( data, total_size, windows_st, detector, scheme, $
                              option, orientation)
        END
  ENDCASE

;
; find no. of data elements in final image
;
  external_total_size = 0L
  FOR i = 0, N_ELEMENTS(external_windows_st) -1 DO BEGIN 
     external_total_size = external_total_size + $
       LONG(external_windows_st(i).LengthX)*LONG(external_windows_st(i).LengthY)
  ENDFOR
 
; set up final exposure array

  external_exposure = INTARR(external_total_size)

; expand out to final dimensions of image

  internal_pos = 0
  external_pos = 0
  FOR i = 0, N_ELEMENTS(windows_st) -1 DO BEGIN

    ; get internal 1-D window array

    temp = exposure(internal_pos:internal_pos+LONG(windows_st(i).LengthX)*LONG(windows_st(i).LengthY)-1)
    internal_pos = internal_pos + N_ELEMENTS(temp) 

    ; get internal 2-D window array in row by column format

    IF orientation THEN BEGIN

      ; data in SPATIAL = column by row format
      ; transpose rows and columns

      temp = TRANSPOSE ( REFORM ( temp, windows_st(i).LengthY, windows_st(i).LengthX) )

    ENDIF ELSE BEGIN

      ; data in SPECTRAL = row by column format

      temp = REFORM ( temp, windows_st(i).LengthX, windows_st(i).LengthY)

    ENDELSE

    ; expand to external size

    ext = INTARR(external_windows_st(i).LengthX,external_windows_st(i).LengthY)

    ext(0:windows_st(i).LengthX*windows_st(i).VDS_xbin-1,0:windows_st(i).LengthY*windows_st(i).VDS_ybin-1) = $ 
      REBIN ( temp, windows_st(i).LengthX*windows_st(i).VDS_xbin, windows_st(i).LengthY*windows_st(i).VDS_ybin, /SAMPLE)
   
    ; set up external exposure 1-D window array

    external_exposure(external_pos:external_pos+N_ELEMENTS(ext)-1) = REFORM(ext)
    external_pos = external_pos + N_ELEMENTS(ext)

;!order=1
;window,i,xsize=external_windows_st(i).lengthx,ysize=external_windows_st(i).lengthy
;tvscl,ext

  ENDFOR
;
;  re-byte swap
;
byteorder, external_exposure, /ntohs


;
;  return with decompressed data
;
RETURN, external_exposure

END

