;+
; Project     : SOHO - CDS     
;                   
; Name        : DCS_SEL_CLEAR
;               
; Purpose     : Creates list of entries to be selectively cleared from DCS.
;               
; Explanation : When CPT is run, it is assumed that the DCS is clear of
;               entries for the time period covered, otherwise duplicate 
;               entries can occur in the DCS.  This routine reads the last 
;               DCS dump file and checks for entries that must be deleted
;               before a new time period can be loaded.
;               
; Use         : IDL> dcs_sel_clear, t_start, t_end
;    
; Inputs      : t_start -  UTC of start of time period (CDS internal format)
;               t_end   -  UTC of end of period
;               
; Opt. Inputs : None
;               
; Outputs     : None
;               
; Opt. Outputs: None
;               
; Keywords    : None
;
; Calls       : None
;               
; Restrictions: Generally only to be run by MASTER account.  Requires 
;               CDS_CP_TTCFILES_W to be defined.  Works on the file created
;               by STM when the deferred command store is dumped eg in Perl
;               script LOADPLAN (or DFLOAD).  
;               
; Side effects: None
;               
; Category    : EGSE
;               
; Prev. Hist. : None
;
; Written     : C D Pike, RAL, 30-Jul-96
;               
; Modified    : Delete file if nothing to uplink.  CDP, 16-Dec-96
;               Added debug type info on times to be deleted.  CDP, 28-May-97
;
; Version     : Version 3, 28-May-97
;-            

pro dcs_sel_clear, t_start, t_end

;
;  check parameters
;
if n_params() ne 2 then begin
   bell,2
   print,'Use: IDL> dcs_sel_clear, t_start, t_end'
   return
endif

;
;  Time must be internal UTC
;
if datatype(t_start,1) ne 'Structure' or $
                           datatype(t_end,1) ne 'Structure' then begin
   bell,2
   print,'Times must be of internal CDS structure type.'
   return
endif

;
;  User must have correct evar setup
;
if getenv('CDS_CP_TTCFILES_W') eq '' then begin
   bell
   print,'$CDS_CP_TTCFILES_W is not defined'
   return
endif

;
;  even if it is, check have write privilege for output file
;
outfile = concat_dir('$CDS_CP_TTCFILES_W','dcs_sel_clear.times')
openw,lun,outfile,/get_lun,error=err
if err ne 0 then begin
   bell
   print,'Unable to open output file: ',outfile
   return
endif else begin
   free_lun,lun
endelse

;
; prepare the DCS contents
;
dcsf = ''
show_dcs,/quiet,edit_list,file=dcsf
if dcsf eq 'Error' then begin
   bell,3
   print,'No DCS dump file available'
   return
endif

;
;  possible output file
;
   outfile = concat_dir('$CDS_CP_TTCFILES_W','dcs_sel_clear.times')

;
;  tidy the list
;
if n_elements(edit_list) gt 3 then begin
   edit_list = edit_list(3:*)
   edit_list = strmid(edit_list,6,18)
   n = where(edit_list ne '',count)
   if count gt 0 then edit_list = edit_list(n)

;
;  filter out for time period required
;
   t0 = utc2tai(t_start)
   t1 = utc2tai(t_end)
   tai_edit = utc2tai(str2utc(edit_list))
   n = where(tai_edit ge t0 and tai_edit le t1, count)
   if count gt 0 then edit_list = edit_list(n)
   num_edit = n_elements(edit_list)

;
;  anything to do?
;
   if count eq 0 then begin
      print,'Nothing to clear from DCS'
      stat = delete_file(outfile,/noconfirm)
      return
   endif else begin 
      print,'Deleting the following DCS entries'
      print_str,edit_list
   endelse

;
;  Write out values to be cleared
; 
   openw,lun,outfile,/get_lun,error=err
   if err ne 0 then begin
      bell
      print,'Unable to open output file: ',outfile
      return
   endif
   printf,lun,'# TIMES TO BE CLEARED FROM THE DEFERRED COMMAND STORE'
   get_utc,utc
   utc = anytim2cal(utc)
   printf,lun,'# Created '+utc
   printf,lun,'#' 
   printf,lun,'#' 
   for i=0,num_edit-1 do begin
      time = '"'+anytim2cal(edit_list(i),form=11,/msec)+'"'
      printf,lun,time
   endfor
   free_lun,lun
endif else begin
   stat = delete_file(outfile,/noconfirm)
endelse

end
