;+
; Project     : SOHO - CDS     
;                   
; Name        : DISP_DEFERRED
;               
; Purpose     : Display contents of a deferred command store dump.
;               
; Explanation : When a special packet is detected in the CDS telemetry with the
;               format id '04'x, the contents relate to a dump of the deferred
;               command store and they can be displayed in the appropriate
;               widget by this routine. The widgets used are set up in the
;               routine SP_DEF_MENU.
;               
; Use         : disp_deferred, packet, counter
;    
; Inputs      : packet  - an int(141) array containing packet data, read by the
;                         routine read_sp_body
;
;               counter - special packet counter (byte). If it has the MSB set
;                         it is the last packet in the dump.
;               
; Opt. Inputs : None
;               
; Outputs     : None
;               
; Opt. Outputs: None
;               
; Keywords    : None
;
; Calls       : None
;               
; Restrictions: Restricted to use within the EGSE science analysis programs.
;               Uses internal COMMON block for passing the number of packets
;               received in the dump so far and for the widget id's within
;               which to display the results.
;               
; Side effects: Common variable sp_def_cnt is set to -1 if this is the last
;               packet in the dump.
;               
; Category    : EGSE
;               
; Prev. Hist. : From a PC routine by M K Carter
;
; Written     : EGSE version by C D Pike, RAL, 7-Oct-93
;               
; Modified    : Include date in earliest/latest time display.  CDP, 20-Apr-95
;               Major revamp of display format.  CDP, 24-Apr-95
;               Fix bug in multi-packet handling.  CDP, 7-Jun-95
;               Change time stamp on output file to UT.  CDP, 7-Jun-95
;               Update evar name.  CDP< 20-Oct-95
;               Increase size of storage variable def_save.  CDP, 14-Mar-96
;
; Version     : Version 7, 14-Mar-96
;-            

PRO disp_deferred, packet, sp_pkt_counter

;
;  COMMON block for communication of widget id's and counter
;
@SpecialWidgets.com
@MainWidgets.com

;
;  output format can be hex or decimal
;
if special_format eq 1 then fmt = '(20(1x,i5))' else fmt = '(20(1x,z4))'

;
;  if this is the first packet in the dump, then display header information
;

if sp_def_cnt eq 0 then begin
   if special_format eq 1 then begin
     widget_control,sp_def_exp(0),set_val=string((packet(0) and '0F'x),form=fmt)
     widget_control,sp_def_exp(2),set_val=string(unsign(packet(3)),form=fmt)
     widget_control,sp_def_exp(3),set_val=string(unsign(packet(4)),form=fmt)
     widget_control,sp_def_exp(4),set_val=string(unsign(packet(5)),form=fmt)
     widget_control,sp_def_exp(5),set_val=string(unsign(packet(6)),form=fmt)
   endif else begin
     hexout,sp_def_exp(0), (packet(0) and '0F'x)
     hexout,sp_def_exp(2), packet(3)
     hexout,sp_def_exp(3), packet(4)
     hexout,sp_def_exp(4), packet(5)
     hexout,sp_def_exp(5), packet(6)
   endelse

; 
;  save length of store for later dump to file
;
   def_length = (2048 - (unsign(packet(5))*2))/2
;   def_length = unsign(packet(4)) - unsign(packet(3)) -1

;
;  output the times
;  s/c time
;
   temp = LONG(packet(1:2),0,1)
   dt = utc2str(tai2utc(double(temp(0))))
   widget_control,sp_def_exp(1),set_val=strmid(dt,0,10)+'  '+strmid(dt,11,10)

;
;  earliest time in store
;
   temp = LONG(packet(7:8),0,1)
   if temp(0) eq 0 then begin
      dt = '           00:00:00.0'
   endif else begin
      dt = utc2str(tai2utc(double(temp(0))))
   endelse
   widget_control,sp_def_exp(6),set_val=strmid(dt,0,10)+'  '+strmid(dt,11,10)
   def_start = dt

;
;  latest time in store
;
   temp = LONG(packet(9:10),0,1)
   if temp(0) eq 0 then begin
      dt = '           00:00:00.0'
   endif else begin
      dt = utc2str(tai2utc(double(temp(0))))
   endelse
   widget_control,sp_def_exp(7),set_val=strmid(dt,0,10)+'  '+strmid(dt,11,10)
;
;  for info
;
   widget_control,sp_def_exp(8),set_val=' ',/append
   widget_control,sp_def_exp(8),set_val='-------- Start --------- ',/append
   widget_control,sp_def_exp(8),set_val=' ',/append

;
;  now keep the rest of the data for later interpretation
;
   def_save = intarr(2000)
   def_save(0:129) = packet(11:140)
   ndefs = 130

endif else begin
;
;  Nth packet. This is not the first packet, so it contains pure dump data
;  ==========
;
   def_save(ndefs:ndefs+140) = packet
   ndefs = ndefs + 141
endelse


;
;  if this was the last packet (may have been first as well) then display the
;  information and reset local counter so that first packet can be recognised
;  next time
;
if (sp_pkt_counter and '80'x)  eq '80'x then begin
   sp_def_cnt = -1
   i = 0
   while i lt ndefs do begin
      temp = long(def_save(i:i+1),0,1)
      dt = utc2str(tai2utc(double(temp(0))))
      i = i + 2
      np = def_save(i) and '7f'x
      if np gt 0 then begin
         if special_format eq 1 then begin
            widget_control,sp_def_exp(8),/append,$
                           set_val=strmid(dt,11,10)+' --> '+$
                                   string(unsign(def_save(i:i+np)),form=fmt)
         endif else begin
            hexout,sp_def_exp(8),def_save(i:i+np),$
                   label=strmid(dt,11,8)+' --> ',/append
         endelse
      endif
      i = i + np + 1
   endwhile

   widget_control,sp_def_exp(8),set_val=' ',/append
   widget_control,sp_def_exp(8),set_val='======== END ======== ',/append
   widget_control,sp_def_exp(8),set_val=' ',/append

;
;  dump to file for comparison with uplink
;  use earliest time in store for dump file ID
;

;
;  check env. var defined first
;
   evar = getenv('CDS_STM_DUMP_W')
   if evar ne '' then begin
   
      get_utc,ut
      st = anytim2cal(ut,form=8)
      fname = 'deferred_' + st + '.STM'
      
      openw,lun,concat_dir('$CDS_STM_DUMP_W',fname),/get_lun,error=err
   
      if err ne 0 then begin
         bell
         widget_control, msg_widg,/append,$
                set_val='Error opening file '+concat_dir('$CDS_STM_DUMP_W',fname)
      endif else begin
         widget_control, msg_widg,/append,$
                set_val='Writing DCS to file '+concat_dir('$CDS_STM_DUMP_W',fname)
         for i=0,def_length do begin
            bs16 = def_save(i)
            byteorder,bs16,/ntohs
            printf,lun,string(byte(bs16,0,2),format='(2z2.2)')
         endfor
         free_lun,lun
         def_length = 0
      endelse 
   endif else begin
      widget_control, msg_widg,/append,$
             set_val='NOT writing DCS to file ($CDS_STM_DUMP_W not defined).'
   endelse
endif  
end

