;+
; Project     : SOHO - CDS     
;                   
; Name        : DISP_GEN_TABLE
;               
; Purpose     : Display contents of a general store dump.
;               
; Explanation : When a special packet is detected in the CDS telemetry with the
;               format id indicating a table dump, they can be displayed in 
;               the appropriate widget by this routine. The widgets used are 
;               set up in the routine SP_GEN_MENU.
;               
; Use         : disp_gen_table, packet, counter, format_id
;    
; Inputs      : packet    - an int(141) array containing packet data, read 
;                           by the routine read_sp_body
;
;               counter   - special packet counter (byte). If it has the 
;                           MSB set it is the last packet in the dump.
;
;               format_id - special packet format, needed because Sequence
;                           table is different from rest (2 bytes normally
;                           containing the length have study_id + length)
;               
; Opt. Inputs : None
;               
; Outputs     : None
;               
; Opt. Outputs: None
;               
; Keywords    : None
;
; Calls       : None
;               
; Restrictions: Restricted to use within the EGSE science analysis programs.
;               
; Side effects: None
;               
; Category    : EGSE
;               
; Prev. Hist. : From a PC routine by M K Carter
;
; Written     : EGSE version by C D Pike, RAL, 7-Oct-93
;               
; Modified    : To display Study ID,                      CDP, 4-Nov-93
;               Add writing out to file for certain special packets.  
;                                                         CDP, 24-Apr-95
;               Fix error return for bad table length.    CDP, 22-May-95
;               Fix error in checksum calculation.        CDP, 23-May-95
;               Check for success on opening dump file and time
;               stamp the dump files.                     CDP, 8-Jun-95
;               Update evar used.  CDP, 20-Oct-95
;               Update dump file format to match Perl output and implement
;               command verification checking.
;                                                         CDP, 21-Nov-95
;               Fix checksum checking.      CDP, 15-jan-96
;               Use xack to halt when uplink error found. CDP, 29-Feb-96
;               Cater for IDL5.2 hex bug.  CDP, 1-6-99
;
; Version     : Version 11, 1-Jun-99
;-            

PRO disp_gen_table, packet, sp_pkt_counter, format_id

;
;  Common block for widget id's
;
@SpecialWidgets.com
@MainWidgets.com


;
;  output format can be decimal or hex
;

if special_format eq 1 then fmt = '(5x,i5)' else fmt = '(5x,z4)'

;
;  display information in the widgets defined in sp_gen_menu.
;
if special_format eq 2 then begin
   hexout,sp_gen_exp(0),packet(0)
endif else begin
   temp = unsign(packet(0))
   widget_control,sp_gen_exp(0),set_val=string(temp,form=fmt)
endelse

if special_format eq 2 then begin
   hexout,sp_gen_exp(1),packet(1)
endif else begin
   temp = unsign(packet(1))
   widget_control,sp_gen_exp(1),set_val=string(temp,form=fmt)
endelse

;
;  sort out study ID (only applicable for sequence table) 
;
if gen_title eq 'Sequence Table Dump' then begin
   study_id = fix(byte(byteswap(packet(2))))
   if special_format eq 2 then begin
      hexout,sp_gen_exp(2),study_id
   endif else begin
      widget_control,sp_gen_exp(2),set_val=string(study_id,form=fmt)
   endelse
endif else begin
      widget_control,sp_gen_exp(2),set_val='n/a'
endelse
;
;  and the table length.
;
tab_len = fix(byte(packet(2)))
if special_format eq 2 then begin
   hexout,sp_gen_exp(3),tab_len
endif else begin
   widget_control,sp_gen_exp(3),set_val=string(tab_len,form=fmt)
endelse

;
;  put out title spacer
;
temp_mess = strtrim(repstr(gen_title,'Table Dump',' '),2)
widget_control,sp_gen_exp(4),set_val=' ',/append
widget_control,sp_gen_exp(4),set_val='-- Start of '+temp_mess+' --',/append
widget_control,sp_gen_exp(4),set_val=' ',/append

;
;  repeat 'header' info for clarity
;
if special_format eq 2 then begin
   hexout,sp_gen_exp(4),packet(0),label='Table index: ',/append 
endif else begin
   temp = unsign(packet(0))
   widget_control,sp_gen_exp(4),/append ,$
                  set_val='Table index: '+string(temp,form=fmt)
endelse

if special_format eq 2 then begin
   hexout,sp_gen_exp(4),packet(1),label='Table ID: ',/append 
endif else begin
   temp = unsign(packet(1))
   widget_control,sp_gen_exp(4),/append ,$
                  set_val='Table ID: '+string(temp,form=fmt)
endelse

;
;  sort out study ID (only applicable for sequence table) 
;
if gen_title eq 'Sequence Table Dump' then begin
   study_id = fix(byte(byteswap(packet(2))))
   if special_format eq 2 then begin
      hexout,sp_gen_exp(4),study_id,label='Study ID: ',/append 
   endif else begin
      widget_control,sp_gen_exp(4),/append ,set_val='Study ID: '+$
                                            string(study_id,form=fmt)
   endelse
endif 
;
;  and the table length.
;
tab_len = fix(byte(packet(2)))
list_len = fix(byte(packet(2)))

if special_format eq 2 then begin
   hexout,sp_gen_exp(4),tab_len,label='Table length: ',/append 
endif else begin
   widget_control,sp_gen_exp(4),/append ,set_val='Table length: '+$
                                         string(tab_len,form=fmt)
endelse
widget_control,sp_gen_exp(4),set_val=' ',/append


;
;  display the available data, first 2 values then 4 values/row
;
if tab_len ge 2 then begin
   hexout,sp_gen_exp(4),packet(3:4),/append
   tab_len = tab_len - 2
   tab_len = tab_len < 136
   nl = fix(tab_len/4)+1
   if (tab_len mod 4) eq 0 then nl = nl - 1
   st = 5
   for i=0,(34<nl)-1 do begin
      hexout,sp_gen_exp(4),packet(st:(st+3 < (tab_len+4))),/append
      st = st + 4
   endfor
endif  else begin
   bell
   widget_control,msg_widg,set_val='**Invalid table length**',/append
   widget_control,sp_gen_exp(4),set_val='**Invalid table length**',/append
   widget_control,sp_gen_exp(4),set_val=' ',/append
   widget_control,sp_gen_exp(4),set_val='-- End of '+temp_mess+' --',/append
   widget_control,sp_gen_exp(4),set_val=' ',/append
   xack,'Invalid table length',group=main_base,/modal,/icon         
   return
endelse

;
;  put out spacer
;
widget_control,sp_gen_exp(4),set_val=' ',/append
widget_control,sp_gen_exp(4),set_val='-- End of '+temp_mess+' --',/append
widget_control,sp_gen_exp(4),set_val=' ',/append


;=========================================
;
;  dump to file for comparison with uplink
;
;=========================================
case strmid(gen_title,0,4) of
   'Data': fname = 'dexwin_'
   'Rast': fname = 'raster_'
   'Sequ': fname = 'series_'
   'VDS ': fname = 'vdswin_'
   'Mode': fname = 'mode_'
   else:   fname = 'Err'
endcase

;
;  if a dump to file is required...
;
if strmid(fname,0,3) ne 'Err' then begin

;
; check env. var. is defined
;
   evar = getenv('CDS_STM_DUMP_W')
   if evar ne '' then begin
;
;  get index number and compose file name
;
      index = strtrim(string(packet(0),format='(i5)'),2)
      get_utc, utc
      froot = 'down_'+fname + index +'.STM'      
      fname = concat_dir('$CDS_STM_DUMP_W',froot)
      widget_control, msg_widg,/append,$
                   set_val='Writing to file '+fname
   
;
;  check checksum
;
      checksum = long(total(packet(3:list_len+2))) and 'ffff'xl
      temp = unsign(packet(list_len+3)) 
      if checksum ne unsign(packet(list_len+3)) then begin
         flash_msg,msg_widg,'Check sum error on special packet:  '+$
                             gen_title,/append
         widget_control, msg_widg,/append,$
                         set_val=string('Total = '+shorthex(checksum))
         widget_control, msg_widg,/append,$
                         set_val=string('Checksum = ', $
                                 unsign(packet(list_len+3)),$
                                      form='(a,z4.4)')
         xack,'Check sum error on special packet',group=main_base,/modal,/icon
      endif else begin
         widget_control, msg_widg,/append,$
                         set_val='Table dump internal checksum OK'
      endelse
   
      openw,lun,fname,/get_lun,error=err
      if err ne 0 then begin
         flash_msg,msg_widg,'Error opening dump file: '+fname,/append
         return
      endif else begin
         for i=1,list_len+2 do begin
            bs16 = packet(i)
            byteorder,bs16,/ntohs
            printf,lun,'0X'+strupcase(string(byte(bs16,0,2),format='(2z2.2)'))
         endfor
         free_lun,lun
      endelse
   endif else begin
      bell
      widget_control, msg_widg,/append,$
             set_val='NOT writing dumps to disk ($CDS_STM_DUMP_W not defined).'
      return
   endelse

;
;  now do the comparison with what the Perl script PSLOAD has written
;
   evar_up = getenv('CDS_CP_STMFILES_W')
   if evar_up ne '' then begin
      fup = concat_dir('$CDS_CP_STMFILES',strmid(froot,5,40))
      
      if not file_exist(fup) then begin
         bell
         widget_control, msg_widg,/append,$
             set_val='Error opening uplink command file.'+fup
      endif else begin
         tup = strtrim(strupcase(rd_ascii(fup)),2)
         if not file_exist(fname) then begin
            widget_control, msg_widg,/append,$
                set_val='Error opening STM dump file.'+fname
         endif else begin
            tdown = strtrim(strupcase(rd_ascii(fname)),2)
            if not same_data(tup, tdown) then begin
               bell,3
               widget_control, msg_widg,/append,$
                   set_val='MISMATCH in command files '+fup
               xack,'MISMATCH in command files',group=main_base,/modal,/icon
            endif else begin
               widget_control, msg_widg,/append,$
                   set_val='Command files verified.
            endelse
         endelse
      endelse  
   endif 
endif
end
