;+
; Project     : SOHO - CDS     
;                   
; Name        : DUMP_RAW_GIS
;               
; Purpose     : Write GIS raw data to file automatically.
;               
; Explanation : As well as having the user realtime option of dumping GIS
;               raw data or not, this routine is now called every time a raw
;               GIS data packet is detected in order to sort the raw and PHA
;               data and write the info automatically to disk.
;               
; Use         : Only from within STM
;    
; Inputs      : head  - 20 byte packet header
;               body  - the 286 byte body of the packet read from the 
;                       telemetry and identified as a raw GIS data packet.
;                       The first 4 bytes are a counter which are discarded.
;
; Opt. Inputs : None
;               
; Outputs     : Data written to disk file if evar CDS_GIS_RAW_W is defined..
;               
; Opt. Outputs: None
;               
; Keywords    : None
;
; Calls       : None
;
; Common      : MainWidgets.com - for message widget ID
;               
; Restrictions: None
;               
; Side effects: None
;               
; Category    : Telemetry
;               
; Prev. Hist. : None
;
; Written     : C D Pike, RAL, 29-Aug-95
;               
; Modified    : Append detector ID to file names.              CDP, 4-Sep-95
;               Use input file name for output file name root. CDP, 5-Sep-95
;               Add filament data to dump records.             CDP, 08-Sep-95
;               Add check for detector ID before dump.         CDP, 11-Sep-95
;               Sort confusion over env. var. names.           CDP, 26-Sep-95
;               Major rewrite to work in Eva's processing.     CDP, 11-Oct-95
;               Update evar name and skip if not defined.      CDP, 20-Oct-95
;               Give message when new dump file started.       CDP, 13-Nov-95
;               Warn first time if _W evar is not set.         CDP, 13-Nov-95
;               Increased jitter range on FFB, filament current and
;               HVs.                                           CDP, 19-Dec-95
;               Change which parameters are used for HV and FFB 
;               to avoid jitter problem.                       CDP, 20-Dec-95
;               Update error message when invalid packet.      CDP, 11-Jan-96
;               Include more debugging output                  CDP, 24-Jan-96
;               Trap check sum errors.                         CDP, 25-Jan-96
;               Fix bug in ffb variable reference.             CDP, 29-Jan-96
;
; Version     : Version 16, 29-Jan-96
;-            

pro dump_raw_gis, head, body

;
;  for message widget ID
;
@MainWidgets.com
@IO_units.com
@Mode.com
@RawData.com
@PHA_data.com

;
;  if the Write evar is not defined then forget it
;
if getenv('CDS_GIS_RAW_W') eq '' then begin
   if pha_warn eq 0 then begin
      pha_warn = 1
      widget_control,msg_widg,/append,$
                set_v='$CDS_GIS_RAW_W not defined, no dumping possible.'
      bell,3
   endif
   return
endif   

;
;  check checksum to eliminate baddies
;
b = [byte(head.msgid,0,2),$
     byte(head.msglen,0,2),$
     byte(head.msgreqid,0,2),$
     head.ap_id,$
     byte(head.counter,0,2),$
     byte(head.length,0,2),$
     head.time,head.id,head.streampacketcounter,$
     body]  
chksum_ok = check_sum(b,6,151) eq (byteswap(fix(b,304,1)))(0)
if not chksum_ok then begin
   bell 
   pkt_time = anytim2cal(tai2utc(obt2tai(head.time)))
   widget_control,msg_widg,/append,$
                set_v=pkt_time + ' Chksum error in raw dump - packet not saved.'
   return
endif
   

;
;  time of this packet rounded to minutes
;
pkt_time = strmid(anytim2cal(tai2utc(obt2tai(head.time)),form=8),0,12)

;
;  add the detector ID
;
det = strtrim(string(body(7),form='(i1)'),2)
if det ne '1' and det ne '2' and det ne '3' and det ne '4' then begin
   widget_control, msg_widg, /append, $
      set_val='Detector ID error in raw packet.'
   print,'First 13 bytes of packet were: ',body(0:12),form='(a,13i4)'
   return
endif

;
; make up possible file names from the packet time, detector and slit
;
new_rawfile = pkt_time+'_d'+det+'_s'+string(slit_num,form='(i1)')+'.raw'
new_phafile = pkt_time+'_d'+det+'_s'+string(slit_num,form='(i1)')+'.pha'

;
;  prepend std directory if available
;
new_rawfile = concat_dir('$CDS_GIS_RAW_W',new_rawfile)
new_phafile = concat_dir('$CDS_GIS_RAW_W',new_phafile)

;
;  make up current parameter specification
;
raw_current.detector = det
raw_current.slit     = slit_num
raw_current.hv       = fix(body(15))
raw_current.lld      = fix(body(14))
raw_current.ffb      = gis_ffb(fix(det-1))
raw_current.fil_id   = fil_sel
raw_current.fil_curr = fil_curr
raw_current.slit_pos = slit_pos
raw_current.mirr_pos = mirr_pos

;
;  fiddle to allow filament current to flicker a little 
;  (may need to apply to other parameters later)
;

if abs(raw_current.fil_curr - raw_previous.fil_curr) le 10 then begin
   raw_previous.fil_curr = raw_current.fil_curr
endif

;
; ***** include this line if HV continues to cause problems *****
;
;raw_previous.hv = raw_current.hv


;
;  is current spec same as previous?
;
if same_data(raw_current, raw_previous) then begin
   gis_rawdata = [[gis_rawdata],[body(4:243)]]
   rawfile = old_rawfile
   phafile = old_phafile
endif else begin
   print,'*******NOW IS******************'
   ptime = anytim2cal(tai2utc(obt2tai(head.time)))
   print,'Packet time ',ptime
   help,raw_current,/str
   print,'*******LAST WAS******************'
   help,raw_previous,/str
   gis_rawdata = body(4:243)
   rawfile = new_rawfile
   phafile = new_phafile
   widget_control, msg_widg, /append,$
                   set_val='Started new GIS RAW file: '+rawfile
endelse

raw_previous = raw_current
old_rawfile = rawfile
old_phafile = phafile


;
;  every 50th packet, process it
;
if ((size(gis_rawdata))(2) mod 50) eq 0 then begin
   process_raw_gis, gis_rawdata, rawfile, phafile
endif
   
end
