;+
; Project     : SOHO - CDS     
;                   
; Name        : GIS_DESELECT
;               
; Purpose     : Deselects GIS data into a complete 8192 pixel spectrum. 
;               
; Explanation : Extract GIS science data from the array in which it is stored
;               as (pixel,window,exposure) into a full 8192 pixel array
;               representing a spectrum from all 4 detectors.
;               
; Use         : data = gis_deselect( rasterhdr, gis_plot, exp_no, msg_widg)
;
;               but only designed for use internally to the EGSE science
;               analysis program.
;
; Inputs      : rasterhdr - structure of type GISrasterhdr which contains the
;                           data windows definition.
;               gis_plot  - 3-d array (spectrum,exposure number) from which to
;                           extract the data.
;               exp_no    - the exposure number these data represent.
;
;               msg_widg  - widget id for the widget holding information
;                           messages.
;
; Opt. Inputs : None
;               
; Outputs     : Function returns the deselected spectrum.
;               
; Opt. Outputs: None
;               
; Keywords    : None
;
; Calls       : None
;               
; Restrictions: Usually for internal use of EGSE science analysis programs.
;               
; Side effects: None
;               
; Category    : EGSE
;               
; Prev. Hist. : None
;
; Written     : C D Pike, RAL, 14-Dec-93
;               
; Modified    : Reverse the data within the individual detector parts of the
;               overall data array.   CDP, 9-Mar-94
;
; Version     : Version 2, 9-Mar-94
;-            

Function GIS_deselect, raster, gis_plot, exp_no, msg_widg


;
;  check we have valid data first
;
if (raster.NumDataWindows le 0) or (exp_no lt 0) then begin
   bell
   WIDGET_CONTROL, msg_widg, /append, $
                set_val="No valid header information for data extraction."
   return,0
endif

;
;  create temporary store
;
spectrum  = INTARR(8192)

;
;  loop over all the windows defined and extract the data as needed
;
FOR J = 0, raster.NumDataWindows-1 DO BEGIN

   spectrum(raster.GISWin(J).Start) = $
                                 gis_plot(0:raster.GISWin(J).Length-1,J,exp_no)

ENDFOR

;
;  reverse each detector portion
;
for i=0,3 do spectrum(i*2048:i*2048+2047)=reverse(spectrum(i*2048:i*2048+2047))

;
;  return spectral array
;
return, spectrum

END

