;+
; Project     : SOHO - CDS
;
; Name        : PROCESS_RAW_GIS()
;
; Purpose     : Performs the data processing on the raw GIS data. 
;
; Explanation : Extracts the raw and PHA data from the packets sent down in
;               GIS 'raw' data mode and writes them out to separate files.
;
; Use         : IDL>  process_raw_gis, gis_data, rawfile, phafile
;
; Inputs      : gis_data - the basic raw packet array
;               rawfile  - name of file in which raw data are dumped
;               phafile  - name of file in which pha data are dumped
;
; Opt. Inputs : None
;
; Outputs     : Two files containing RAW and PHA data are written.
;
; Opt. Outputs: None
;
; Keywords    : None
;
; Calls       : None
;
; Restrictions: None
;
; Side effects: None
;
; Category    : Telemetry, GIS
;
; Prev. Hist. : Based on PROC_RAW_GIS by E Alam, MSSL.
;
; Written     : C D Pike, RAL, 11-Oct-95
;
; Modified    : Replace use of UNIQ with own and call HEAD_ENGINEER.  
;                                                     CDP, 12-Oct-95
;               Put total counts on plot.             CDP, 13-Nov-95
;		Change algorithm to detect valid data, ERB, 17-Nov-95
;               Trap empty PHA array   CDP, 11-Jan-96
;		Rescale PHA plot, ERB, 14 Apr 96
;               Trap error on opening dump files.  CDP, 30-Jul-96
;
; Version     : Version 7, 30-Jul-96
;-

pro process_raw_gis, rgis_data, rawfile, phafile

;
;  common for widget IDs
;
@PHA_data.com

;
; Change dimensions of GIS_DATA to look at each GIS PKT per row
; (Only works when gis_data is 1-d because it's byte data!!)
;

gis_data = reform(rgis_data,(size(rgis_data))(1)/2,2*((size(rgis_data))(2)))

;
; Check GIS RAW DATA index for only RAW GIS DATA
;
index = where(gis_data(0,*) eq 2, count)
if count ne 0 then gis_data = gis_data(0:117,[index])

;
;  check for valid packets remaining
;
index = where(gis_data(1,*) gt 0, count_pkt)
if count_pkt gt 0 then gis_data = gis_data(*, [index])

;
; Extract Eng header, PHA & Raw data 
;

start_eng = 2 & start_pha = 14 & start_raw = 22


;
;  packet contains raw data if its length is > start_pack
;
index_raw = where(gis_data(1,*) gt start_raw, count_raw)

;
;  if there are raw data packets, then extract them
;
if count_raw gt 0 then begin
	raw = gis_data(start_raw:*, [index_raw])
endif

;
;  extract PHA
;
index_pha = where((gis_data(1,*) gt start_pha), count_pha)

if count_pha gt 0 then begin
   pha = gis_data(start_pha:start_raw-1,[index_pha])
endif

;
;  extract engineering data
;
eng = gis_data(start_eng:start_raw-1,*)


;
; Process ENG array
;

;
; Create Engineering Header (I/P: ENG, O/P: H_ENG)
; This will call AVG_ENG(I/P: ENG, O/P: A_ENG) & add strings as reqd 
; (ie DET NO: etc)
;

head = head_engineer(eng)

;
; Write out data to O/P RAW & PHA files
;
openw, lunraw, rawfile, /get_lun, error=err
if err ne 0 then begin
   print,'*** Error ***  Cannot open RAW data file for write <'+rawfile+'>'
   free_lun, lunraw
   return
endif

openw, lunpha, phafile, /get_lun, error=err
if err ne 0 then begin
   print,'*** Error ***  Cannot open PHA data file for write <'+phafile+'>'
   free_lun, lunpha
   return
endif

if count_raw gt 0 then begin
   writeu, lunraw, head, raw
   writeu, lunpha, head, pha
endif else if count_pha gt 0 then writeu, lunpha, head, pha

free_lun, lunraw
free_lun, lunpha

;
; If requested Plot PHA
;
     
if xregistered('pha') then begin
   if n_elements(pha) gt 1 then begin
      widget_control, pha_draw, get_val=wind
      wset, wind
      np = n_elements(pha)
      h = smooth((histogram(pha,min=1)),11)	; smoothed pha histogram 
      ymax = max(h(50:200)) + 1			; rescale to ignore high energy peak
      plot,h,yrange=[0,ymax],psym=10,title='Total # of counts: '+strtrim(string(np),2)
   endif else begin
      print,'Warning: no PHA data'
   endelse
endif       

end

