;+
; Project     : SOHO - CDS     
;                   
; Name        : READ_EXP
;               
; Purpose     : Read and display the exposure packet header. 
;               
; Explanation : The exposure packet comes after the raster header packet and
;               before the science data packets per se.  The contents are
;               identical for both the VDS and GIS.  This routine reads a 
;               structure of type ExpHeaderDef as defined in the main program
;               SCI_ANA and displays the contents in the relevant header
;               information widgets. The COMMON variable 'header_select'
;               is used to flag which of the header display widgets is
;               currently active and what choice of display was chosen.  
;
;               The values are as follows:
;
;               header_select > 0 for GIS
;                             < 0 for VDS
;
;               A choice of output formats is provided by the setting of the 
;               COMMON variable 'header_format' (=1 for decimal, =2 for hex).
;               
; Use         : read_exp, unit, Header, instrument
;    
; Inputs      : unit       - the logical unit from which to read the data.
;               Header     - the structure in which to load the data, hence 
;                            it defines the amount of data to be read.
;               instrument - the instrument identification.  This is actually
;                            the id given in the raster header packet and so 
;                            is '11'x for GIS and '12'x for VDS.
;               
; Opt. Inputs : None
;               
; Outputs     : Return value - Function returns a value of 1 for 
;                              success, -1 for failure.
;               Header       - as above for input.
;               
; Opt. Outputs: None
;               
; Keywords    : None
;
; Calls       : None
;               
; Restrictions: Only for use within EGSE science analysis program.
;               
; Side effects: None
;               
; Category    : EGSE
;               
; Prev. Hist. : EM version by Arnulf, Oct-92
;
; Written     : FM version by C D Pike, 11-Oct-93
;               
; Modified    : Insert conversion of exposure time to date/time string
;               CDP, 21/10/93
;
;               Change to v3.0 of packet format, CDP, 27-Oct-93
;               Update output formats, CDP, 28-Oct-93
;               Decode slit position, CDP, 2-Mar-94
;               Change exposure number display (1...n), fix solar X/Y display,
;                                                          CDP, 9-May-94
;               Update for operations version. CDP, 11-Aug-94
;               Mask off 12 bit OPS values.  CDP, 26-May-95
;
; Version     : Version 8, 26-May-95
;-            

FUNCTION read_exp, unit, Header, instrument

;
;  COMMON for communication of variables header_select and header_format
;  and date/time for each exposure.
;
@HeaderWidgets.com
@Packets.com
@MainWidgets.com
@Mode.com

;
;  skip on error
;
ON_IOERROR, ERR_exp

;
;  read data from logical unit (realtime or replay)
;
READU,unit,Header


;
; Byte swap all two byte values
;                                   
Header.SolarX            = ByteSwap (Header.SolarX)
Header.ExposureNumber    = ByteSwap (Header.ExposureNumber)
Header.OPSLCoordinate    = (ByteSwap (Header.OPSLCoordinate)) and 'fff'x
Header.OPSRCoordinate    = (ByteSwap (Header.OPSRCoordinate)) and 'fff'x
;
; deal with coded slit position
;
Header.SlitPosition      = ByteSwap (Header.SlitPosition)
slitpos_decode, Header.SlitPosition, slitbits, slitpos
Header.SlitPosition = slitpos

Header.SunSensorPitch    = ByteSwap (Header.SunSensorPitch)
Header.SunSensorYaw      = ByteSwap (Header.SunSensorYaw)
Header.SolarY            = ByteSwap (Header.SolarY)
Header.EventLevel        = ByteSwap (Header.EventLevel)
Header.StudyCounter      = ByteSwap (Header.StudyCounter)
Header.RasterCounter     = ByteSwap (Header.RasterCounter)
Header.ChkSum            = ByteSwap (Header.ChkSum)

;
;  convert time bytes to date/time string
;
fmt_cdhs_time, Header, date,time

;
;  save date/time
;

if instrument eq '11'x then begin
   if Header.ExposureNumber gt (n_elements(GIS_date_time)-1) then begin
      flash_msg, msg_widg, 'Error on number of exposures in this raster'
      cds_mode = 0
      return, -1
   endif    
   if Header.ExposureNumber eq 0 then GIS_date_time(*) = ' '
   GIS_date_time(Header.ExposureNumber) = date + ' ' + time
endif
if instrument eq '12'x then begin
   if Header.ExposureNumber gt (n_elements(VDS_date_time)-1) then begin
      flash_msg, msg_widg, 'Error on number of exposures in this raster'
      cds_mode = 0
      return, -1
   endif    
   if Header.ExposureNumber eq 0 then VDS_date_time(*) = ' '
   VDS_date_time(Header.ExposureNumber) = date + ' ' + time
endif


;
;  success and error returns
;
RETURN,1
ERR_exp: RETURN,-1

END
