;+
; Project     : SOHO - CDS     
;                   
; Name        : READ_GIS_RASTER()
;               
; Purpose     : Read the GIS raster header from a unit. 
;               
; Explanation : This function reads the packet header for a GIS raster.  It
;               is a structure of type GISrasterdef (see sci_ana.pro) and
;               consists of the science header structure (SciheaderDef) which
;               is common to GIS and VDS with additional items specific to the
;               GIS ie the number of data windows and the structure defining
;               the windows. This function calls swap_scihead which is used
;               to byte-swap the header variables.
;               
; Use         : header = read_gis_raster(unit, gis_raster)
;    
; Inputs      : unit       - the logical unit from which to read data
;               
; Opt. Inputs : None
;               
; Outputs     : gis_raster - GIS raster header structure
;               Function value returned = -1 for error, +1 for read success
;               and -100 for number of windows exceeding the expected maximum.
;
; Opt. Outputs: None
;               
; Keywords    : None
;
; Calls       : swap_scihead   -   byteswap header variables.
;               
; Restrictions: Use only in EGSE sci_ana program
;               
; Side effects: None
;               
; Category    : EGSE
;               
; Prev. Hist. : EM version by Arnulf, Oct-92
;
; Written     : FM version by C D Pike, RAL, 18-Oct-93
;               
; Modified    : For v3.0 of packet format, CDP, 27-Oct-93
;               Modify call to swap_scihead and cut out displays.
;                                          CDP, 12-AUg-94
;
; Version     : Version 3, 12-Aug-94           
;-            

FUNCTION read_GIS_raster, unit, gis_raster

;
;  load COMMON block to communicate widget id's for display widgets
;
@HeaderWidgets.com
@MainWidgets.com

;
;  return value -1 on i/o error
;
ON_IOERROR, ERR_GIS_raster

;
; Read from unit, will either be realtime pipe or replay file unit
;
readu,unit,gis_raster
   
;
; Swap science header variables
;
swap_scihead, gis_raster

;
;  byteswap variable containing number of GIS windows
;
gis_raster.numdatawindows = byteswap(gis_raster.numdatawindows)


;
;  return with value -100 if number of windows exceeds 32.  This is
;  redefinable presumably.
;
if (gis_raster.NumDataWindows GT 32) then return,-100

;
; Swap window definitions
;

for i=0,gis_raster.NumDataWindows-1 do begin
   gis_raster.GISWin(i).Start  = ByteSwap (gis_raster.GISWin(i).Start)
   gis_raster.GISWin(i).Length = ByteSwap (gis_raster.GISWin(i).Length)
endfor

;
;  byteswap checksum variable (and ignore!)
;
gis_raster.Chksum = ByteSwap (gis_raster.Chksum)


;
;  success and failure returns
;
RETURN,1
ERR_GIS_raster: RETURN,-1

END
