;+
; Project     : SOHO - CDS     
;                   
; Name        : READ_IWS_PKT
;               
; Purpose     : Read one data packet from an EGSE telemetry file.
;               
; Explanation : This function reads one data packet from a telemetry file 
;		created by the Spacetek EGSE software (tm. file).  The
;		packet may either be a science packet or a housekeeping
;		packet.  In either case the packet will be 306 bytes long, 
;		housekeeping packets will be zero extended to this length.
;		If the packet is a science packet, then it is checked 
;		against its internal checksum.  No such check is performed
;		on housekeeping packets.  Finally, a status code is 
;		returned to indicate the success or failure of the
;		read operation.
;
;		All packets will be organized as follows:  The first six
;		bytes of the packet of will be capsule data attached by 
;		the Spacetek software.  The next six bytes are the packet
;		header with the packet body filling out rest of the packet.
;
;		This function returns a status code to indicate the result
;		of the read operation.  When reading from telemetry file
;		which is being created in realtime, the result No Packet
;		Available or Incomplete Packet may indicate data which is
;		currently being processed by the EGSE software.  In this
;		case, calling the function again may result in a complete
;		packet.
;
;		The status code should be checked carefully before using
;		a packet returned by this function, as a packet is only
;		valid if the status code is set to Success or Housekeeping 
;		Packet.
;
;		To process packets faster, and to allow packets to be read
;		in realtime from a file, a certain amount of data is kept
;		in static variables inside a common block.  This data is
;		is initialized whenever the function is called with the 
;		RESET option set.  It is therefore important to set this
;		option when using this function for the first time.
;		Failure to do this may result in run time errors.   
;               
; Use         :	status = READ_IWS_PKT (dev, pkt, [/RESET])
;    
; Inputs      : dev	The logical unit number returned when the telemetry
;			file was opened.
;
; Opt. Inputs : None.
;               
; Outputs     : pkt	A 306 byte array containing the packet which was read
;			from the telemetry file.  Note that the data in the 
;			array is only valid if the status code returned by 
;			the function indicates success.
;               
; Opt. Outputs: None.
;               
; Keywords    :	RESET	When set, causes the static variables, which are 
;			stored in a common block, to be reset to there 
;			initial values.  The first time this fucntion is 
;			used, it should be called with the /RESET option.
;
; Return Value: A status code representing the success or failure of the 
;		of the read operation.
;
;		Status Codes Are:
;
;		0	Success.
;		1	Checksum Failure.
;		2       Housekeeping Packet
;		3	No Packet Available.
;		4	Incomplete Packet.
;		5	I/O error.
;
;
; Calls       : CHECK_SUM
;		BYTESWAP
;
; Common      : readiwspkt.
;               
; Restrictions: Must be called on a previously open telemetry file.  The
;		file must have been opened with the /NOSTDIO option.  The
;		first time the procedure is used, it should be called with
;		the reset option.
;               
; Side effects: None.
;               
; Category    : Engineering.
;               
; Prev. Hist. : None.
;
; Written     : Ron Yurow, SM&A, 12 May 1999
;               
; Modified    : Version 1, Initial Release		RY, 12 May 1999
;
; Version     : Version 1, 12 May 1999
;-            

	FUNCTION READ_IWS_PKT, LUN, PKT, RESET = reset

;
; Common block for static variables.
;

	COMMON readiwspkt,		$
           PKTSIZE,			$
	   STOK,			$
	   STCHKSUM,			$
	   STHKPKT,			$
	   STNOPKT,			$
	   STINCPKT,			$
	   STIOERR,			$
	   target,			$
	   count, 			$
	   buffer			
	   
;
; Set jump point to handle IO errors.
;

	ON_IOERROR, read_error

;
; Check if the keyword RESET is set.  If it is, then we will have to 
; initialize all of the static variables.
;

	IF KEYWORD_SET (reset) THEN BEGIN

	   PKTSIZE = 306

	   STOK = 0
	   STCHKSUM = 1
	   STHKPKT = 2
	   STNOPKT = 3
	   STINCPKT = 4
	   STIOERR = 5

	   target = PKTSIZE ;
	   count = 0 ;
	   buffer = BYTARR (PKTSIZE) ;

	ENDIF

;
; Force PKT to be a PKTSIZE Byte array.
;

	pkt = BYTARR (PKTSIZE, /NOZERO)

;
; Check if target is set to PKTSIZE, if it is, then we can go ahead and read
; a new packet.  Otherwise we must complete the packet that we were reading 
; previously.
;

	IF target EQ PKTSIZE THEN BEGIN 

;
; Read up to PKTSIZE bytes of data from the file specified by LUN
;

	   READU, lun, pkt, TRANSFER_COUNT = count

;
; Check if count is PKTSIZE, if it is then we have a packet.
;

           IF count EQ PKTSIZE THEN BEGIN 

;
; Check if this is a housekeeping packet.  If it is then there is no need
; to calculate a checksum on it.
;

	      IF pkt (7) EQ '63'X THEN RETURN, STHKPKT

;
; Calculate the checksum of the packet.
;

	      chk = CHECK_SUM (pkt, 6, 151)

;
; Check if the checksum is what we expect it to be.  If it is, then return 0.
;

	      IF chk EQ  (BYTESWAP (FIX (pkt, 304, 1))) (0) THEN RETURN, STOK

;
; Checksum failure, return 1.
;

	       RETURN, STCHKSUM

	   ENDIF

;
; Check if count is 0, if it is then no packet was available.
;

	   IF count EQ 0 THEN RETURN, STNOPKT

;
; Otherwise, we have an incomplete packet.  Now we will have to do a bit of
; extra work to get the rest of it.
;

;
; Set target to the number of bytes left to recieve.
;

	   target = PKTSIZE - count

;
; Copy however much of the packet we have so far into buffer.
;

	   buffer = pkt

;
; Return a status code to indicate that we received a partial packet.
;

	   RETURN, STINCPKT

	ENDIF

;
; Create a temporary array.  The size of this array will be the same size as
; number byte we need to complete the packet.
;

	temp = BYTARR (target)

;
; Try to read in the rest of the packet.
;

	READU, lun, temp, TRANSFER_COUNT = count

;
; Copy any new bytes that we managed to read back into the buffer.  Only do 
; this if we have recieved at least 1 new byte. 
;

	IF count NE 0 THEN buffer (PKTSIZE - target:305) = temp

;
; Copy whatever we have so far into pkt to return to the calling procedure.
;

	pkt = buffer

;
; Check if the packet is complete. 
;

	IF count EQ target THEN BEGIN

;
; Reset target to the size of a packet.  This will insure we try to get a new
; new packet the next time this procedure is called.
;

	   target = PKTSIZE

;
; Check if this is a housekeeping packet.  If it is then there is no need
; to calculate a checksum on it.
;

	   IF pkt (7) EQ '63'X THEN RETURN, STHKPKT

;
; Calculate the checksum of the packet.
;

	   chk = CHECK_SUM (pkt, 6, 151)

;
; Check if the checksum is what we expect it to be.  If it is, then return 0.
;

	   IF chk EQ  (BYTESWAP (FIX (pkt, 304, 1))) (0) THEN RETURN, STOK

;
; Checksum failure, return 1.
;

	   RETURN, STCHKSUM

	ENDIF

;
; Otherwise, we still have an incomplete packet.
;

	RETURN, STINCPKT

;
; Handle Errors.
;

READ_ERROR:

;
; If we can set STATUS to I/0 error, do it.  Otherwise just return.
;

	RETURN, STIOERR

	END






