;+
; Project     : SOHO - CDS     
;                   
; Name        : SHOW_DCS
;               
; Purpose     : Shows current contents of CDS deferred command store.
;               
; Explanation : Whenever instructions are uplinked to the CDS CDHS deferred
;               command store, the store is read and dumped in the science 
;               telemetry stream.  If STM is running with the evar
;               CDS_STM_DUMP_W defined, it will dump that
;               telemetry into a time-stamped file in the directory
;               CDS_STM_DUMP_W.  This routine picks up the latest such file
;               (which should represent the current DCS contents) and
;               displays it in a readable form.
;               
; Use         : IDL> show_dcs [text, file=file, /hardcopy, /quiet, /keep]
;    
; Inputs      : None
;               
; Opt. Inputs : None
;               
; Outputs     : Can print to screen and/or hardcopy
;               
; Opt. Outputs: TEXT     - contains text of output
;               
; Keywords    : HARDCOPY - create and print hardcopy.
;               QUIET    - no output to screen
;               KEEP     - keep output file
;               FILE     - if present but null then the default file used is
;                          returned, otherwise the file specified is used. If
;                          an error occurs on finding this file then a value
;                          of 'Error' is returned in this variable.
;
; Calls       : None
;
; Common      : None
;               
; Restrictions: Required CDS_STM_DUMP (ie the read version) to be defined.
;               
; Side effects: None
;               
; Category    : Telemetry
;               
; Prev. Hist. : None
;
; Written     : C D Pike, RAL, 31-May-95
;               
; Modified    : Include interpretation of cb5abort.  CDP, 6-Jun-95
;               Include error message in file keyword.  CDP, 12-Jun-95
;               Update header.  CDP, 20-Oct-95
;               Interpret GIS LUT load. CDP, 15-Dec-95
;               Update for change in OPS movement code. CDP, 08-Jan-96
;               Translate GIS commissioning type events.  CDP, 11-Jan-96
;               Increase size of storage for reading DCS file. CDP, 14-Mar-96
;               Add interpretation of Parking GIS HV.  CDP, 8-May-96
;               Fix bug introduced in v9.  CDP, 10-May-96
;               Added TEXT command line output.        CDP, 29-Jul-96
;               Added MCU turn on/off and slit step.   CDP, 19-Nov-96
;               Corrected OPS left/right interpretation 
;               and included series ID interpretation.  CDP, 17-Jan-98
;               Use FIND_FILE for long list.  CDP, 26-Jan-98
;               Include interpretation of OPS L/R, 3-Jun-98
;     
; Version     : Version 15, 3-Jun-98
;-            
pro show_dcs, text, file=file, hardcopy=hardcopy, quiet=quiet, keep=keep

;
;  to terminal?
;
if keyword_set(quiet) then to_screen = 0 else to_screen = 1
if to_screen then openw, screen, filepath(/terminal), /more, /get_lun

;
;  force to send to file, even temporarily
;
to_file=1


;
;  create filename (in home directory - unless directory specified) 
;  for hardcopy
;
if not keyword_set(file)  then begin
   f = find_file(concat_dir('$CDS_STM_DUMP','deferred*.STM'),count=count)
   if count eq 0 then begin
      print,'No DCS dump files found.'
      file = 'Error'
      return
   endif else begin
      infile = last_item(f)
   endelse
endif else begin
   infile = file
   if not file_exist(infile) then begin
      print,'Specified file {',infile,'} does not exist.'
      file = 'Error'
      return
   endif
endelse

;
;  something to handle so open input file
;
openr,lun1,infile,/get_lun

;
;  read input file
;
dcs = strarr(2000)
text = ''
i = 0
while not eof(lun1) do begin
   readf,lun1,text
   dcs(i) = text
   i = i + 1                    
endwhile

;
;  Is there anything there?
;
nd = where(dcs ne '')               
if nd(0) ge 0 then begin
   dcs = dcs(nd)
endif else begin
   free_lun, lun1
   if to_screen then free_lun, screen
   print,'Error: Deferred command dump file is empty.'
   bell
   return
endelse

;
;  create filename in home directory for hardcopy
;
hfile = 'current_dcs'
vms=(!version.os eq 'VMS')
if vms then home='sys$login' else home=getenv('HOME')
hfile=concat_dir(home,hfile)

;
;  open hardcopy file if needed
;
if to_file then openw,lun2,hfile,/get_lun

;
;  put out title
;
break_file,infile,disk,dir,ff,ext
if to_screen then begin
   printf,screen,' '
   printf,screen,'CDS DCS contents inferred from file ',ff
   printf,screen,' '
endif
if to_file then begin
   printf,lun2,' '
   printf,lun2,'CDS DCS contents inferred from file ',ff
   printf,lun2,' '
endif

;
;  format the output
;
ndefs = n_elements(dcs)
idcs = intarr(ndefs)
for i=0,ndefs-1 do begin
   hex2dec,dcs(i),temp,/quiet
   idcs(i) = temp
endfor
i = 0
item = 1
while i lt (ndefs-1) do begin
   temp = long(idcs(i:i+1),0,1)
   dt = anytim2cal(utc2str(tai2utc(double(temp(0)))))
   i = i + 2
   np = idcs(i) and '7f'x
   if np gt 0 then begin
      case dcs(i) of
         '2501': begin
                    text1 = '  Move mechanism '
                    if strmid(dcs(i+1),0,1) eq 'b' then begin
                       text2 = '(OPS right to '
                       hex2dec,strmid(dcs(i+1),1,3),val,/quiet
                       val = strtrim(string(val,form='(i4)'),2)
                       text2 = text2 + val + ') }'
                       opsrv = fix(val)
                    endif
                    if strmid(dcs(i+1),0,1) eq 'a' then begin
                       text2 = '(OPS left to  '
                       hex2dec,strmid(dcs(i+1),1,3),val,/quiet
                       val = strtrim(string(val,form='(i4)'),2)
                       text2 = text2 + val + ')  }'
                       opslv = fix(val)
                    endif
                    if strmid(dcs(i+1),0,2) eq '92' then begin
                       text2 = '(Slit number to '
                       hex2dec,strmid(dcs(i+1),3,1),val,/quiet
                       val = strtrim(string(val,form='(i4)'),2)
                       text2 = text2 + val + ')'
                    endif
                    if strmid(dcs(i+1),0,2) eq '96' then begin
                       text2 = '(Step slit position DOWN)'
                    endif
                    if strmid(dcs(i+1),0,2) eq '97' then begin
                       text2 = '(Step slit position UP)'
                    endif
                 end
         '2401': begin
                    text1 = '  GIS -  '
                    if dcs(i+1) eq '8d8d' then begin
                       text2 = 'Filaments OFF'
                    endif
                    if dcs(i+1) eq '9898' then begin
                       text2 = 'HV1 relay OFF'
                    endif
                    if dcs(i+1) eq 'a9a9' then begin
                       text2 = 'HV2 relay OFF'
                    endif
                    if dcs(i+1) eq 'baba' then begin
                       text2 = 'HV3 relay OFF'
                    endif
                    if dcs(i+1) eq 'cbcb' then begin
                       text2 = 'HV4 relay OFF'
                    endif
                    if dcs(i+1) eq 'd9d9' then begin
                       text2 = '12v OFF'
                    endif
                    if dcs(i+1) eq 'c9c9' then begin
                       text2 = '5v OFF'
                    endif
                 end
         '2301': begin
                    text1 = '  GIS -  '
                    case strmid(dcs(i+1),0,2) of
                        'bb': text2 = 'HV1 to parking voltage '
                        'bc': text2 = 'HV2 to parking voltage '
                        'bd': text2 = 'HV3 to parking voltage '
                        'be': text2 = 'HV4 to parking voltage '
                        else: text2 = 'LUT parameter load'
                    endcase
                    hex2dec,strmid(dcs(i+1),2,2),val,/quiet 
                    if strmid(text2,0,3) ne 'LUT' then text2 = text2 + trim(val)
                 end
         '2101': begin
                    text1 = '  Turn MCU backup '
                    case strmid(dcs(i+1),3,1) of
                        '1': text2 = 'ON'
                        '2': text2 = 'OFF'
                        else: text2 = '??'
                    endcase
                 end
         '5e82': begin
                    text1 = '  Move mechanism '
                    if strmid(dcs(i+2),0,1) eq 'a' then begin
                       text2 = '(OPS right to  '
                       hex2dec,strmid(dcs(i+2),1,3),val,/quiet
                       val = strtrim(string(val,form='(i4)'),2)
                       text2 = text2 + val + ')  }'
                       opsrv = fix(val)

                    endif
                    if strmid(dcs(i+2),0,1) eq 'b' then begin
                       text2 = '(OPS left to '
                       hex2dec,strmid(dcs(i+2),1,3),val,/quiet
                       val = strtrim(string(val,form='(i4)'),2)
                       text2 = text2 + val + ') }'
                       opslv = fix(val)
                    endif
                 end
         '5381': begin
                    text1 = '  Run sequence ID 0x'
                    hex2dec,dcs(i+1),seq,/quiet
                    seq1 = get_cdhs_series(seq)
                    if seq1.id ne -1 then begin
                      get_study,seq1.id,seq1.var,seq
                      sname = strtrim(seq.obs_prog,2)
                    endif else sname = ''
                    text2 = dcs(i+1) + ' ('+sname+')'
                 end
         '5081': begin
                    if strmid(dcs(i+1),3,1) eq 'a' then begin
                       text1 = '  Abort any previous sequences.'
                       text2 = ' '
                    endif else begin
                       text1 = '  '+dcs(i)
                       text2 = '  '+dcs(i+1)
                    endelse
                 end
         '5e8d': begin
                    text1 = '  Load GIS LUT for detector #'
                    text2 = trim(fix(strmid(dcs(i+9),2,2)))
                 end
           else: begin
                    text1 = '  '+dcs(i)
                    text2 = '  '+dcs(i+1)
                 end
      endcase
                 
      if to_screen then begin
         if (np eq 1) or (dcs(i) eq '5e8d') or (dcs(i) eq '5e82')  then begin
            if strpos(text2,'(OPS right') ge 0 then begin
               opsxy = fix(cdslr2xy(opslv,opsrv))
               printf,screen,item,')  '+dt+text1+text2,$
                             '-->(X,Y) = '+fmt_vect([opsxy(0),opsxy(1)]),$
                             format='(i3,a,a)'
            endif else begin
               printf,screen,item,')  '+dt+text1+text2,$
                             format='(i3,a)'
            endelse
         endif else begin
            if strpos(text2,'(OPS right') ge 0 then begin
               opsxy = fix(cdslr2xy(opslv,opsrv))
               printf,screen,item,')  '+dt+text1+text2,dcs(i+2:i+np),$
                             '-->(X,Y) '+fmt_vect([opsxy(0),opsxy(1)])
                             format='(i3,a,10(2x,a4,a))'
            endif else begin
               printf,screen,item,')  '+dt+text1+text2,dcs(i+2:i+np),$
                             format='(i3,a,10(2x,a4))'
            endelse
         endelse
         if strmid(text1,0,5) eq '  Run' then printf,screen,' '
      endif
      if to_file then begin
         if (np eq 1) or (dcs(i) eq '5e8d') or (dcs(i) eq '5e82')  then begin
            if strpos(text2,'(OPS right') ge 0 then begin
               opsxy = fix(cdslr2xy(opslv,opsrv))
               printf,lun2,item,')  '+dt+text1+text2,$
                             ' -->(X,Y) = '+fmt_vect([opsxy(0),opsxy(1)]),$
                             format='(i3,a,a)'
            endif else begin
               printf,lun2,item,')  '+dt+text1+text2,$
                             format='(i3,a)'
            endelse
         endif else begin
            if strpos(text2,'(OPS right') ge 0 then begin
               printf,lun2,item,')  '+dt+text1+text2,dcs(i+2:i+np),$
                             '--> (X,Y) '+fmt_vect([opslv,opsrv])
                             format='(i3,a,10(2x,a4,a))'
            endif else begin
               printf,lun2,item,')  '+dt+text1+text2,dcs(i+2:i+np),$
                             format='(i3,a,10(2x,a4))'
            endelse
         endelse
         if strmid(text1,0,5) eq '  Run' then printf,lun2,' '
      endif
   endif
   i = i+ np + 1
   item = item + 1
endwhile

;
;  close file(s) 
;
free_lun, lun1
if to_screen then free_lun, screen
if to_file then free_lun, lun2

;
; hardcopy?
;
if keyword_set(hardcopy) then begin
   if vms then com = 'print '+hfile else com = 'lpr '+hfile
   spawn,com
   print,'Hardcopy sent to printer.'
endif

;
;  in case text was to be returned then get it back
;
text = rd_ascii(hfile)

;
;  keep file?
;
if not keyword_set(keep) then begin
   status = delete_file(hfile,/noconfirm)
endif

;
;  what input file was used?
;
file = infile
end
