	PRO STCNT_UPDATE, STCNT_UNIT, STCNT_MAX, STUDYCOUNTER
;+
; Project     :	SOHO - CDS
;
; Name        :	STCNT_UPDATE
;
; Purpose     :	Update the value in the file max_study_counter.dat
;
; Category    :	Class4, CDS, Telemetry
;
; Explanation :	Updates the value which is stored in the file
;		$CDS_CP_MISC_W/max_study_counter.dat with a larger value.
;		Called from within FITSGEN in the subroutine FIND_RASTER.
;
;		If the study counter value jumps by more than 100, then a
;		message is generated.
;
;		Normally, one calls STCNT_OPEN once at the beginning of
;		processing telemetry, and then calls STCNT_UPDATE to maintain
;		the value in the file.
;
; Syntax      :	STCNT_UPDATE, STCNT_UNIT, STCNT_MAX, STUDYCOUNTER
;
; Examples    :	STATUS = FSTAT(STCNT_UNIT)
;		IF STATUS.OPEN THEN BEGIN
;			STUDYCOUNTER = UNSIGN(PACKET.PKT.SCI.STUDYCOUNTER)
;			STCNT_UPDATE, STCNT_UNIT, STCNT_MAX, STUDYCOUNTER
;		ENDIF
;
; Inputs      :	STCNT_UNIT   = Logical unit number of the opened file.
;		STCNT_MAX    = The current value stored in the file.
;		STUDYCOUNTER = The new study counter value.  The file is only
;			       modified if this is larger than the old value.
;
; Opt. Inputs :	None
;
; Outputs     :	STCNT_MAX  = The current value stored in the file (updated).
;
; Opt. Outputs:	None.
;
; Keywords    :	None.
;
; Calls       :	CDS_MESSAGE
;
; Common      :	None.
;
; Restrictions:	The file must be opened with STCNT_OPEN.
;
; Side effects:	None.
;
; Prev. Hist. :	Extracted from FIND_RASTER
;
; History     :	Version 1, 29-May-1997, William Thompson, GSFC
;
; Contact     :	WTHOMPSON
;-
;
;
	IF !DEBUG EQ 0 THEN ON_ERROR, 2 ELSE ON_ERROR, 0
;
;  Check the calling sequence.
;
	IF N_PARAMS() NE 3 THEN MESSAGE,	$
		'Syntax:  STCNT_UPDATE, STCNT_UNIT, STCNT_MAX, STUDYCOUNTER'
;
;  Check the input parameters.
;
	IF N_ELEMENTS(STCNT_MAX) NE 1 THEN MESSAGE,	$
		'STCNT_MAX must be a scalar'
	IF N_ELEMENTS(STUDYCOUNTER) NE 1 THEN MESSAGE,	$
		'STUDYCOUNTER must be a scalar'
	IF STUDYCOUNTER LE STCNT_MAX THEN RETURN
;
;  Make sure that STCNT_UNIT points to an open file.
;
	IF N_ELEMENTS(STCNT_UNIT) NE 1 THEN MESSAGE,	$
		'STCNT_UNIT must be a scalar'
	STATUS = FSTAT(STCNT_UNIT)
	IF NOT STATUS.OPEN THEN MESSAGE,	$
		'STCNT_UNIT must point to an open file'
;
;  Close and reopen the file, and reread the counter value, in case it's been
;  written to since the last time the file was opened.
;
	CLOSE, STCNT_UNIT
	OPENU, STCNT_UNIT, STATUS.NAME
	READF, STCNT_UNIT, STCNT_MAX
;
	IF STUDYCOUNTER GT STCNT_MAX THEN BEGIN
	    DIFF = STUDYCOUNTER - STCNT_MAX
	    IF DIFF GT 100 THEN CDS_MESSAGE, /ATTENTION, /CONTINUE, $
		    'Study counter jumped by ' + TRIM(DIFF), /ADD_TIME
	    STCNT_MAX = STUDYCOUNTER
	    POINT_LUN, STCNT_UNIT, 0	;Beg. of file
	    PRINTF, STCNT_UNIT, STCNT_MAX
;
;  Do a close and reopen to force a flush to the disk.
;
	    CLOSE, STCNT_UNIT
	    OPENU, STCNT_UNIT, STATUS.NAME
	ENDIF
;
	RETURN
	END
