;+
; Project     : SOHO - CDS     
;                   
; Name        : STM
;               
; Purpose     : Science Telemetry Monitor.
;               
; Explanation : A procedure to set up the structures and declare and initialise
;               the variables used in the telemetry monitoring programs.
;               
; Use         : IDL>  stm
;    
; Inputs      : None
;               
; Opt. Inputs : None
;               
; Outputs     : None                  
;               
; Opt. Outputs: None
;               
; Keywords    : None
;
; Calls       : None
;               
; Restrictions: None
;               
; Side effects: None
;               
; Category    : Telemetry
;               
; Prev. Hist. : Based on "calibration" and "sci_ana"
;
; Written     : C D Pike, RAL, 27-Jul-94
;               
; Modified    : Added vds_offset to data structure.  CDP, 24-Feb-95
;               Added DCS list widget.  CDP, 6-Jun-95
;               Added parameters to do with GIS raw data. CDP, 11-Oct-95
;               Added vds_centre to vds_data-str structure. CDP, 20-Oct-95
;               Add DCS status checking parameters. CDP, 18-Dec-95
;               Add gis_ffb to initialisation.      CDP, 20-Dec-95
;               Add last_event_no to initialisation CDP, 11-Jan-96
;               Initialise gis_ffb to an array      CDP, 24-Jan-96
;               Add secondary SEU variable to init. CDP, 19-Feb-96
;               Initialise vds_tick to -1.0         CDP, 27-Feb-96
;               Read VDS slit position from state db. CDP, 01-Mar-96
;               Include binning initialisation.        CDP, 16-Oct-97
;               Add initial value for sp_gen_base widget ID CDP, 18-Jun-98
;               Cater for IDL5.2 hex bug.  CDP, 1-6-99
;
; Version     : Version 15, 1-Jun-99
;-            

pro stm

;
;  load all the COMMON blocks used throughout the program
;
@Decomp.com
@GISWidgets.com
@HeaderWidgets.com
@IO_units.com       
@Images.com
@Mode.com
@Packets.com
@Rawscience.com
@ReviewWidgets.com
@SpecialWidgets.com
@VDSWidgets.com
@MainWidgets.com
@VDSData.com
@GISData.com
@RawData.com
@Profiles.com
@Monitor.com
@Base_Offsets.com
@PHA_data.com

;-----------------------------------------------------------------------------
; Define structures for version 3.0 of CDS science data stream packet format
;-----------------------------------------------------------------------------
; Logic of packet reading is .....
;
; a) Read packet_header (20 bytes)
; b) Test value of .ID
;    if '2'X then read_VDS_raster  (286 bytes)  ie total = 20+286   = 306 bytes
;    if '1'X then read_GIS_raster  (286 bytes)  ie total = 20+286   = 306 bytes
;    if '11'X or '12'X then read_sci_cnt (2 bytes) 
;       then read_exp_hdr (284 bytes)              total = 20+2+284 = 306 bytes
;    if '21'x or '22'x then read_sci_cnt (2 bytes) 
;       then read_science_packet (284 bytes)       total = 20+2+284 = 306 bytes
;    else read_data (286 bytes)                    total = 20+286   = 306 bytes 
;-----------------------------------------------------------------------------
;
;General packet header structure
;
PacketHeader = {PacketHeaderDef,                $
                Msgid:               FIX(0),    $
		MsgLen:              FIX(0),    $
		MsgReqid:            FIX(0),    $    
		ap_id:               BYTARR(2), $
		counter:             FIX(0),    $
		length:              FIX(0),    $
		Time:                BYTARR(6), $
		ID:                  BYTE(0),   $
		StreamPacketCounter: BYTE(0)}

;
;Define Science data header
;
SciHeader = {SciHeaderDef, $
		StudyID:               BYTE(0),   $
		DataCompScheme:        BYTE(0),   $ 
		DataExtWindowID:       FIX(0),    $
		SequenceID:            FIX(0),    $
		RasterLengthM:         BYTE(0),   $
		RasterLengthS:         BYTE(0),   $
		OPSLCoordinate:        FIX(0),    $
		OPSRCoordinate:        FIX(0),    $
                SlitPosition:          FIX(0),    $
		SlitNumber:            BYTE(0),   $
		MirrorPosition:        BYTE(0),   $
		ExposureTime:          FIX(0),    $
		StudyCounter:          FIX(0),    $
		RasterCounter:         FIX(0),    $
                DataCompOption:        FIX(0),    $
                SciProcStatus:         FIX(0)}

;
;  Science packet counter
;
Scnt_str = {scntdef,$
                count:    FIX(0),$
                last:     FIX(0)}

;
;Exposure header
;
;  remember the 2 byte packet counter which could be in here at the beginning
;  is actually read in read_sci_cnt on its own
;
Exphdr= {ExpHeaderDef, $
		SolarX:                FIX(0),      $
		ExposureStart:         BYTARR(6),   $
		ExposureNumber:        FIX(0),      $
		OPSLCoordinate:        FIX(0),      $
		OPSRCoordinate:        FIX(0),      $
		SlitPosition:          FIX(0),      $
                SunSensorPitch:        FIX(0),      $
                SunSensorYaw:          FIX(0),      $
                SolarY:                FIX(0),      $
                MirrorPosition:        BYTE(0),     $
                EventStatus:           BYTE(0),     $
                EventLevel:            FIX(0),      $
		StudyCounter:          FIX(0),      $
		RasterCounter:         FIX(0),      $
                Science:               BYTARR(252), $
		ChkSum:                FIX(0)}

;
; VDS Window Data.
;
VDSWindow = {VDSWdw, $
                StartX:   FIX(0),$
                StartY:   FIX(0),$
                LengthX:  FIX(0),$
                LengthY:  FIX(0)}

;
;GIS Window Data
;
GISWindow = {GISWdw,            $
		Start:  FIX(0), $
		Length: FIX(0)}


;
;VDS raster packet
;
VDSrasterhdr = {VDSrasterdef,                                 $
		Scihead:              {SciHeaderDef},         $
                BinFactorXY:          BYTE(0),                $
                NumDataWindows:       BYTE(0),                $
		VDSWin :              REPLICATE({VDSWdw},32), $
		Chksum:               FIX(0)}

;				
;GIS raster packet 
;
GISrasterhdr = {GISrasterdef,                                 $
		Scihead:              {SciHeaderDef},         $
                Spare1:               BYTARR(12),             $
                NumDataWindows:       FIX(0),                 $
		GISWin :              REPLICATE({GISWdw},61), $
		Chksum :              FIX(0)}
	


;
;  science packet - remember again that the packet counter is read separately
;  and is therefore not included here. 
;
Sci_Packet = {SciPacketDef, $
              ExpNumber:     FIX(0),            $
              StudyCounter:  FIX(0),            $
              RasterCounter: FIX(0),            $
              Data:          BYTARR(276),       $
	      Chksum:        FIX(0)}	

;
;  any packet ie a dummy read for any unwanted packets
;
Any_Packet = {AnyPacketDef, $
              Data: BYTARR(286)}


;
;  a structure to keep check of the VDS data and its display
;  NB don't change this structure except by adding tags atthe end because
;  the current tags are referenced by number in some places.
;
vds_data_str = {vds_data_rec,$
            vds_exp:        fix(-1), $           ; last exposure read
            max_in_mem:     fix(0),  $           ; max number of exp in memory
            exp_in_a:       fix(-1), $           ; exp displayed in window A
            exp_in_b:       fix(-1), $           ; exp displayed in window B
            first_in_mem:   fix(0),  $           ; first exp in memory
            last_in_mem:    fix(-1), $           ; last exp in memory
            curr_a_img:     intarr(1024,1024),$  ; image array in A
            curr_b_img:     intarr(1024,1024),$  ; image array in B
            vds_centre:     fix(511),$           ; centre of gravity of VDS sp.
            vds_offset:     intarr(2)}           ; extraction offsets
            

;
;  a structure to keep check of the GIS data and its display
;
gis_data_str = {gis_data_rec,$
            gis_exp:        fix(-1), $           ; last exposure read
            exp_in_a:       fix(-1), $           ; exp displayed in window A
            exp_in_b:       fix(-1), $           ; exp displayed in window B
            gis_plot_wave:  fix(0),  $           ; pixel(0) or lambda(1) plot
            gis_plot_linear:fix(1),  $           ; linear(1) or log(0) plot
            agis_last_x:    fltarr(2),  $        ; last window in A
            bgis_last_x:    fltarr(2),  $        ; last window in B
            gis_select:           'A',  $        ; display selected
            agis_plot_choice: bytarr(5),$        ; which bands in A selected
            bgis_plot_choice: bytarr(5),$        ; which bands in B selected
            curr_a_data:    intarr(8192),$       ; spectrum in A
            curr_b_data:    intarr(8192)}        ; spectrum in B

;
; Initialize buffers and flags
;
sci_writettr = LONG(0)
sci_readptr  = LONG(0)
sci_mode     = FIX(1)
sci_pause    = FIX(0)
sci_going    = FIX(0)
cds_mode     = FIX(0)
ops_mode     = FIX(0)
reviewing    = FIX(0)
review_stats = LONARR(23)
new_rev_stat = FIX(-1)
raw_gis_data = bytarr(286)
raw_vds_data = bytarr(286)
;
; initialize images and plots 
;
have_v_rst_hdr  =  0
have_v_exp_hdr =  0
have_g_rst_hdr =  0
have_g_exp_hdr =  0
header_select  =  0
gis_display    =  0
vds_display    =  0
header_format  =  1
special_format =  2
raw_format     =  2
raw_gis_request = 0
raw_vds_request = 0
raw_gis_file   = ''
raw_gis_dump   =  0
raw_gis_display = 1
raw_vds_file   = ''
raw_vds_dump   =  0
sp_request     =  0
packet_no      = LONG(0)
last_packet    =  0
last_pkt_id    =  0
last_sci_cnt   =  0
sp_def_cnt     = -1
writing_fits   =  1
gasid = 'None'
gasvoltage = 0
gascurrent = 0.0
gaspressure = 0
seq_list_min = 16
seq_list_max = 0
cal_fits_file = 'None_0000_000.00'
plotting_prof = 0
plen_fac = 1
maxvala = 0
maxvalb = 0
last_window = ' '
disp_vds_lines = 0
disp_gis_lines = 0
vm_count = -1
gm_count = -1
opsl_count = -1
opsr_count = -1
slitn_count = -1
slitp_count = -1
mirrp_count = -1
v_monitor_mode = 0
g_monitor_mode = 0
m_monitor_mode = 0
monitor_dcs    = 0
bk4nis         = 0
vds_monitor = fltarr(1000)
gis_monitor = fltarr(1000)
opsl_monitor = intarr(1000)
opsr_monitor = intarr(1000)
slitn_monitor = bytarr(1000)
slitp_monitor = intarr(1000)
mirrp_monitor = bytarr(1000)
cal_hor_posn  = 0.0
cal_vert_posn = 0.0
vds_disp_yst  = 362
vds_disp_yend = 661
vds_back_sub  = 0
vds_tick      = -1.0
last_header = ' '
last_study_id = 0
dcs_status = 0
dcs_warned = 1
study_info = ''
last_event_no = -1
seen_address = strarr(50)
last_sevent_no = -1
sseen_address = strarr(50)
vds_xbin = 1
vds_ybin = 1
vds_orientation = 0
sp_gen_base       = -1
gis_header_off    = [765,26,631,774]
gis_menu_off      = [196,37,1073,788]
vds_header_off    = [598,35,671,737]
mon_menu_off      = [304,45,610,873]
pha_menu_off      = [304,45,610,873]
prof_menu_off     = [574,496,669,458]
gis_raw_off       = [673,36,595,623]
vds_raw_off       = [752,343,517,670]
review_menu_off   = [742,35,435,571]
sp_def_off        = [14,177,650,610]
sp_err_off        = [200,200,300,300]
sp_gen_off        = [905,404,390,640]
sp_health_off     = [862,35,420,740]
sp_ief_off        = [731,35,538,736]
sp_imif_off       = [707,37,615,905]
sp_imifim_off     = [776,530,495,530]
stm_menu_off      = [39,153,550,393]
vds_menu_off      = [189,37,1080,750]
dcs_menu_off      = [500,100,530,730]
ops_plan_menu_off = [500,100,530,730]

;
;  read VDS slit position
;
get_utc, utc
get_vds_slitpos, utc, vds_sp

;
;  in case we don't see any Eng packets before raw comes
;
fil_sel = 0
filon_off = 0
fil_curr = 0
slit_num = 0
slit_pos = 0
mirr_pos = 0
gis_ffb = intarr(4)
gis_lut_load = 1
raw_pkt_time = 0
raw_current = {detector: 0, $
               slit:     0, $
               hv:       0, $
               lld:      0, $
               ffb:      0, $
               fil_id:   0, $
               fil_curr: 0, $
               slit_pos: 0, $
               mirr_pos: 0}

raw_previous = raw_current
old_rawfile = ''
old_phafile = ''
pha_warn    = 0

;
;  data decompression setup
;
bits = ['0000'xl, '0001'xl, '0003'xl, '0007'xl, '000F'xl, $
                 '001F'xl, '003F'xl, '007F'xl, '00FF'xl, $
                 '01FF'xl, '03FF'xl, '07FF'xl, '0FFF'xl, $
                 '1FFF'xl, '3FFF'xl, '7FFF'xl, 'FFFF'xl]

; quasilog tables

qutab12 = settab ( 3, 5)
qutab16 = settab ( 4, 4) 

;
;  for study ID etc need access to official databases.
;
status = fix_zdbase(/cds)

;
;Let's go
;
stm_menu


END
