	FUNCTION TM_ENOUGH, UNIT, NEEDED, CHECKSUM, MAX_BYTE, I1=I1, I2=I2
;+
; Project     :	SOHO - CDS
;
; Name        :	TM_ENOUGH()
;
; Purpose     :	Returns true if enough bytes in file to read.
;
; Explanation :	This routine determines whether or not there are enough bytes
;		to read in a file which is open for read access.  It is
;		particularly useful when the file is open for write by another
;		process, such as when processing realtime telemetry.
;
; Use         :	Result = TM_ENOUGH( UNIT, NEEDED  [, CHECKSUM] )
;
;		IF TM_ENOUGH(UNIT, NEEDED) THEN ...
;
; Inputs      :	UNIT	= Logical unit number of the opened file.
;		NEEDED	= The number of bytes which need to be read.
;
; Opt. Inputs :	None.
;
; Outputs     :	The result of the function is true if there are enough bytes,
;		or false otherwise.
;
; Opt. Outputs:	CHECKSUM = If passed, then the software calculates a checksum
;			   value as the XOR of all the 16-bit words starting
;			   from the time stamp and up to, but not including,
;			   the last.
;
;		MAX_BYTE = If passed, then the software finds the maximum byte
;			   value in all the words used in calculating the
;			   checksum.  This is useful for determining whether or
;			   not the packet is empty.
;
; Keywords    :	I1,I2	 = Normally, the software will calculate for itself
;			   which words will be used in the checksum.  However,
;			   the user can explicitly pass in the start and stop
;			   values.  For a 153 word (306 byte) packet, these
;			   would default to I1=6, I2=151.
;
; Calls       :	None.
;
; Common      :	None.
;
; Restrictions:	This routine may not work except in Unix.  The file must not be
;		opened with any optional keywords (other than GET_LUN).
;
; Side effects:	None.
;
; Category    :	Data handling, Telemetry
;
; Prev. Hist. :	Based partially on TM_GET_SIZE by C. D. Pike.
;
; Written     :	William Thompson, GSFC, 7 June 1995
;
; Modified    :	Version 1, William Thompson, GSFC, 7 June 1995
;		Version 2, William Thompson, GSFC, 13 June 1995
;			Return false if file is not open.
;			Modified to try to read the bytes and reset, rather
;			than closing and reopening the file.  Seems to be less
;			stress on the system.
;		Version 3, William Thompson, GSFC, 28 June 1995
;			Added optional parameter CHECKSUM
;		Version 4, William Thompson, GSFC, 4 June 1997
;			Added optional parameter MAX_BYTE
;
; Version     :	Version 4, 4 June 1997
;-
;
	ON_ERROR, 2
;
;  Check the number of parameters.
;
	IF N_PARAMS() LT 2 THEN MESSAGE,	$
		'Syntax:  Result = TM_ENOUGH( UNIT, NEEDED  ' +	$
		'[, CHECKSUM  [, MAX_BYTE ]] )'
;
;  Save the current pointer.  If the file isn't open, then return immediately.
;
	STATUS = FSTAT(UNIT)
	IF STATUS.OPEN EQ 0 THEN RETURN, 0
;
;  Try to read in the correct number of bytes.
;
	TEMP = BYTARR(NEEDED)
	ON_IOERROR, ERROR_POINT
	READU, UNIT, TEMP
;
;  If requested, calculate the checksum and the maximum number of bytes.
;
	IF N_PARAMS() GE 3 THEN BEGIN
		IF N_ELEMENTS(I1) NE 1 THEN I1 = (NEEDED - 294) / 2
		IF N_ELEMENTS(I2) NE 1 THEN I2 = (NEEDED -   4) / 2
		CHECKSUM = CHECK_SUM(TEMP, I1, I2)
		IF N_PARAMS() EQ 4 THEN MAX_BYTE = MAX(TEMP(2*I1:2*I2))
	ENDIF
;
;  If we didn't trap an error, then there must be enough bytes.
;
	RESULT = 1
	GOTO, FINISH
;
;  Didn't read all the bytes, so return 0 as the result.
;
ERROR_POINT:
	RESULT = 0
	MAX_BYTE = 0
;
;  Reset the pointer and return.
;
FINISH:
	POINT_LUN, UNIT, STATUS.CUR_PTR
	RETURN, RESULT
	END
