;+
; Project     : SOHO - CDS     
;                   
; Name        : TP_OFFSETS
;               
; Purpose     : To read/write the base widgets offsets and sizes.
;               
; Explanation : Since the user may redesign the location and size of the 
;               various base widgets used in STM, this procedure reads and
;               writes the last used values to a file in the current
;               directory.  The design should then remain constant across
;               invokations of the widgets and across IDL sessions (when run
;               from the same directory).
;               
; Use         : tp_offsets [,/write]
;    
; Inputs      : None
;               
; Opt. Inputs : None
;               
; Outputs     : Changes values in common and writes to file if requested.
;               
; Opt. Outputs: None
;               
; Keywords    : WRITE - if present the values of all the base widget offsets
;                       and sizes are written to the file Base_Offsets in a
;                       format useful for reading and translating later.
;
; Calls       : None
;
; Common      : base_off_size
;               
; Restrictions: For use only within the STM suite of programs.
;               
; Side effects: User satisfaction.
;               
; Category    : Telemetry
;               
; Prev. Hist. : None, responding to user requests.
;
; Written     : C D Pike, RAL, 24-Jan-95
;               
; Modified    : Cutout setting default values on read (done in stm.po)
;                                         CDP, 27-Apr-95
;               Send offsets file to home directory.  CDP, 16-May-95
;               Include DCS widget.       CDP, 6-Jun=95
;               Add PHA display.  CDP, 11-Oct-95
;               Add Plan listing.  CDP, 15-Nov-95
;
; Version     : Version  6, 15-Nov-95
;-            

pro tp_offsets, write=write

;
;  common for all variable storage
;
@Base_Offsets.com

;
;  form home directory file name
;
vms = (!version.os eq 'VMS')
if vms then home = 'sys$login' else home = getenv('HOME')
bo_file = concat_dir(home,'Base_Offsets')

;
;  READ only mode
;  ====
;
if not keyword_set(write) then begin
   if file_exist(bo_file) then begin
      openr,lun,bo_file,/get_lun
      text = '#'
      while (strmid(text,0,1) eq '#') or $
            (strmid(text,0,1) eq ' ') do readf,lun,text
      if text ne '' then status = execute(text)
      while not eof(lun) do begin
         readf,lun,text
         text = strtrim(text,2)
         if text ne '' then status = execute(text)
      endwhile
      free_lun,lun
   endif 
endif else begin

;
;  WRITE the updated values
;  =====
;
   it = -1
   if file_exist(bo_file) then begin
      text = strarr(1000)
      temp = '                     '
      openr,lun,bo_file,/get_lun
      while not eof(lun) do begin
         readf,lun,temp
         if strmid(temp,0,1) eq '#' then begin
            it = it + 1
            if strmid(temp,0,16) eq '#  File created:' then begin
               text(it) = strmid(temp,0,16) + ' ' + systime()
            endif else begin   
               text(it) = temp
            endelse
         endif   
      endwhile
      free_lun,lun
   endif
   openw,lun,bo_file,/get_lun,error=err
   if err eq 0 then begin
      if it gt -1 then begin
         for i=0,it do begin
            printf,lun,text(i)
         endfor
      endif else begin
         printf,lun,'#'
         printf,lun,'#  File to store STM base widget positions and sizes'
         printf,lun,'#'
         printf,lun,'#  File created: '+systime()
         printf,lun,'#'
      endelse
      value = fmt_vect(gis_header_off)
      value = repchar(value,'(','[')
      value = repchar(value,')',']')
      printf,lun,'gis_header_off = '+value
   
      value = fmt_vect(gis_menu_off)
      value = repchar(value,'(','[')
      value = repchar(value,')',']')
      printf,lun,'gis_menu_off = '+value
   
      value = fmt_vect(vds_header_off)
      value = repchar(value,'(','[')
      value = repchar(value,')',']')
      printf,lun,'vds_header_off = '+value
   
      value = fmt_vect(mon_menu_off)
      value = repchar(value,'(','[')
      value = repchar(value,')',']')
      printf,lun,'mon_menu_off = '+value
   
      value = fmt_vect(pha_menu_off)
      value = repchar(value,'(','[')
      value = repchar(value,')',']')
      printf,lun,'pha_menu_off = '+value
   
      value = fmt_vect(prof_menu_off)
      value = repchar(value,'(','[')
      value = repchar(value,')',']')
      printf,lun,'prof_menu_off = '+value
   
      value = fmt_vect(gis_raw_off)
      value = repchar(value,'(','[')
      value = repchar(value,')',']')
      printf,lun,'gis_raw_off = '+value
   
      value = fmt_vect(vds_raw_off)
      value = repchar(value,'(','[')
      value = repchar(value,')',']')
      printf,lun,'vds_raw_off = '+value
   
      value = fmt_vect(review_menu_off)
      value = repchar(value,'(','[')
      value = repchar(value,')',']')
      printf,lun,'review_menu_off = '+value
   
      value = fmt_vect(sp_def_off)
      value = repchar(value,'(','[')
      value = repchar(value,')',']')
      printf,lun,'sp_def_off = '+value
   
      value = fmt_vect(sp_err_off)
      value = repchar(value,'(','[')
      value = repchar(value,')',']')
      printf,lun,'sp_err_off = '+value
   
      value = fmt_vect(sp_gen_off)
      value = repchar(value,'(','[')
      value = repchar(value,')',']')
      printf,lun,'sp_gen_off = '+value
   
      value = fmt_vect(sp_health_off)
      value = repchar(value,'(','[')
      value = repchar(value,')',']')
      printf,lun,'sp_health_off = '+value
   
      value = fmt_vect(sp_ief_off)
      value = repchar(value,'(','[')
      value = repchar(value,')',']')
      printf,lun,'sp_ief_off = '+value
   
      value = fmt_vect(sp_imif_off)
      value = repchar(value,'(','[')
      value = repchar(value,')',']')
      printf,lun,'sp_imif_off = '+value
   
      value = fmt_vect(sp_imifim_off)
      value = repchar(value,'(','[')
      value = repchar(value,')',']')
      printf,lun,'sp_imifim_off = '+value
   
      value = fmt_vect(stm_menu_off)
      value = repchar(value,'(','[')
      value = repchar(value,')',']')
      printf,lun,'stm_menu_off = '+value
   
      value = fmt_vect(vds_menu_off)
      value = repchar(value,'(','[')
      value = repchar(value,')',']')
      printf,lun,'vds_menu_off = '+value
   
      value = fmt_vect(dcs_menu_off)
      value = repchar(value,'(','[')
      value = repchar(value,')',']')
      printf,lun,'dcs_menu_off = '+value
   
      value = fmt_vect(ops_plan_menu_off)
      value = repchar(value,'(','[')
      value = repchar(value,')',']')
      printf,lun,'ops_plan_menu_off = '+value
   
      free_lun,lun 
   endif else begin
      print,'Error opening Base_Offset file for writing.'
   endelse
endelse

end
