;+
; Project     : SOHO - CDS     
;                   
; Name        : VDS_EXT_IMAGE
;               
; Purpose     : Loads VDS data stream into VDS image array.
;               
; Explanation : As the VDS telemetry is received, it is stored in a vector.
;               This routine reformats it into a 4-d array with dimensions
;               (width,height,window,exposure).
;               
; Use         : VDS_Ext_image, sci_buffer, raster, exp_no, msg_widg
;    
; Inputs      : sci_buffer  -  1-d buffer with science data
;               raster      -  raster header structure
;               exp_no      -  exposure number
;               msg_widg    -  general message widget
;              
;               
; Opt. Inputs : None
;               
; Outputs     : vds_image is updated and held in COMMON
;               
; Opt. Outputs: None
;               
; Keywords    : None
;
; Calls       : None
;               
; Restrictions: Only for use within EGSE sci_ana programs.
;               
; Side effects: None
;               
; Category    : EGSE
;               
; Prev. Hist. : EM version by Arnulf, Oct-92
;
; Written     : FM version by C D Pike, RAL, 27-Oct-93
;               
; Modified    : Version 1, C D Pike, RAL, 27-Oct-93
;		Version 2, William Thompson, GSFC, 12 November 1993.
;			Added /NTOHS to BYTEORDER for platform independence.
;               Version 3  CDP, 13-Dec 1993 
;                          To incorporate new data array format. 
;               Version 4  Rework handling of binned data.  CDP, 3-Oct-97
;
; Version     : Version 4, 3-Oct-97
;-            

PRO VDS_Ext_image, sci_buffer, raster, exp_no, msg_widg

;
;  COMMON for binning factors and limits on data array size
;
@Images.com
@VDSData.com


;
; message because this can take a while
;
WIDGET_CONTROL, msg_widg, /append, $
                SET_VALUE="Extracting VDS windows from science stream."

;
;  make sure we have valid parameters first
;
if (raster.NumDataWindows le 0) or (exp_no lt 0) then begin
   bell
   WIDGET_CONTROL, msg_widg, /append, $
       SET_VALUE="No valid header parameters, aborting image extraction."
   WIDGET_CONTROL, msg_widg, /append, $
       SET_VALUE="(Number of windows <=0 or exposure number < 1)"
   return
endif 

;
;  pointer within the incoming 1-d array
;
SciencePointer = LONG(0)

;
;  check which exposure number location to load in, if over the top then
;  recycle.
;
s = size(vds_image)
max_exp = s(4)

;
;  limit reached, so inform
;
if exp_no eq (max_exp) then begin
   text1 = fmt_vect([max_exp])
   text = string('Maximum number of exposures ',text1,$
                 ' reached, cycling exposures',form='(3a)')
   bell,2
   widget_control,msg_widg,set_val=text,/append
endif

;
;  when at or over the top, then recycle
;
if exp_no ge max_exp then begin
   load_exp_no = max_exp - 1
   vds_image = shift(vds_image,0,0,0,-1)
   vds_data_str.first_in_mem = vds_data_str.first_in_mem + 1
endif else begin
   load_exp_no = exp_no
endelse

;
;  store which is last exposure in memory
;
vds_data_str.last_in_mem = exp_no

;
; loop for each window definition. Note that the window value are 'full size'
; even if the binning is applied, so correct local variables according to 
; binning
;
for j = 0, raster.NumDataWindows-1 do begin

   StartX  = raster.VDSWin(J).StartX
   StartY  = raster.VDSWin(J).StartY
   LengthX = raster.VDSWin(J).LengthX
   LengthY = raster.VDSWin(J).LengthY
   

;
;  length in words of this window
;
   ScienceLength = LONG(LengthX) * LONG(LengthY)

;
;  cut out this window
;
   temp = sci_buffer(SciencePointer:SciencePointer+Sciencelength-1)

;
;  turn cut-out into 2-d array and byteswap it
;
   temp = REFORM (temp, LengthX, LengthY, /OVERWRITE)
   byteorder,temp,/ntohs

;
;  .. and insert it into output array
;
  
   vds_image(0:LengthX-1,0:LengthY-1,j,load_exp_no) = temporary(temp)
  
;
;  increment pointer in 1-d array for next time
;
   SciencePointer = SciencePointer + ScienceLength

;
;  kill duff column data (as notified by WTT) ie first and last couple of 
;  columns in master 1024x1024 array.
;
   if StartX eq 0 then vds_image(0:1,*,j,load_exp_no) = 0
   if StartX eq 1 then vds_image(0,*,j,load_exp_no) = 0
   if StartX+LengthX-1  eq 1022 then vds_image(LengthX-1,*,j,load_exp_no) = 0
   if StartX+LengthX-1  eq 1023 then $
                           vds_image(LengthX-2:LengthX-1,*,j,load_exp_no) = 0

endfor

;
;  inform of completion
;
WIDGET_CONTROL, msg_widg, /append, $
       SET_VALUE=systime()+" Finished data extraction."

END
