	PRO VDS_MEM_ANAL, FILE
;+
; Project     :	SOHO - CDS
;
; Name        :	VDS_MEM_ANAL
;
; Purpose     :	Analyze a NIS Memory Dump
;
; Explanation :	This program examines the contents of a NIS memory dump and
;		and checks for the existence certain structures such as 
;		program modules and the jump table.  In addition a set of 
;		consistency checks are performed in order to assess the state
;		of the NIS software when the memory dump was performed.
;		Finally a list of every module, code and pattern, found in the
;		memory space is produced along with the contents of key memory
;		locations.   
;
; Use         :	VDS_MEM_ANAL, FILE 
;
; Inputs      :	FILE:	The name of the memory dump file to analyze.  The
;			memory dump file is produced by STM when CDS completes
;			a NIS memory dump.
;
; Opt. Inputs :	None.
;
; Outputs     :	None.
;
; Opt. Outputs:	None.
;
; Keywords    :	None.
;
; Calls       :	None.
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Category    :	Engineering.
;
; Prev. Hist. :	None.
;
; Written     :	Ron Yurow, GSFC, 30 October 1997
;
; Modified    :	Version 1, Ron Yurow, GSFC, 30 October 1997
;               Cater for IDL5.2 hex bug.  CDP, 1-6-99
;
; Version     :	Version 2, 1-Jun-99
;-
;

;
; Open the file that contians the VDS memory dump
; 

	OPENR, LUN, FILE, /GET_LUN

;
; Create a byte array to hold 256 bytes of data from the VDS memory dump file.
; We will use this array to read the memory dump file into the larger array
; VDS, which will hold the entire memory dump. 
;

	TEMP1 = BYTARR (256)

;
; Create a byte array to hold 256 bytes of data from the VDS memory dump file.
; This will be same data stored in TEMP1, only in word reversed format to 
; correct the byte order that CDS stores the memory dump in.
;

	TEMP2 = BYTARR (256)

;
; Create a byte array to hold the entire VDS memory dump.
;

	VDS = BYTARR (32768)

;
; Loop to read in the VDS memory dump file into the array VDS. The memory dump
; file is read blocks of 256 bytes.
;

	FOR I = 0, 127 DO BEGIN

;
; Read 256 Bytes from the VDS memory dump file.
;

	   READU, LUN, TEMP1

;
; Copy the bytes from TEMP1 to TEMP2.  As we do this, reverse the byte order
; inside each 16 bit word.
;

	   FOR J = 0, 127 DO BEGIN

	      TEMP2 (J * 2) = TEMP1 (J * 2 + 1)
	      TEMP2 (J * 2 + 1) = TEMP1 (J * 2)

	   ENDFOR

;
; Copy the bytes that we just read into the array VDS.
;

	   VDS (I * 256) = TEMP2

	ENDFOR

;
; Create a table of all the listing all the error strings for all the error
; conditions that the program can detect.  Each error condition will be given
; a unique number which will be the same as the subsript which references its
; string in this table.
;

	ERRTBL = 							$
	["No Error.",							$
	 "Module does not end with module termination string 'ZZZZ'.",	$
	 "Jump Table not found at expected address.",			$
	 "Null Error."]
	
;
; Write a 2 lines telling the user what program is running.
;

	PRINT, "VDS Memory Analysis Program"
	PRINT, "==========================="

;
; Write a line telling the user that we are examineing the Page 0 Memory
; addresses.
;

	PRINT, "Examining the contents of page 0 memory addresses:"

;
; Set the variable WORKSPACE from address #0070 of the memory map
; (workspace address). 
;

	WORKSPACE = LONG (VDS ('70'xl:'73'xl), 0)

;
; Set the variable MODRELOAD from address #0074 of the memory map
; (number of modules reloaded by the kernel). 
;

	MODRELOAD = LONG (VDS ('74'xl:'77'xl), 0)

;
; Set the variable MEMORYMAP from address #0078 of the memory map
; (address of memory map). 
;

	MEMORYMAP = LONG (VDS ('78'xl:'7B'xl), 0)

;
; Set the variable JUMPTABLE from address #007C of the memory map
; (address of the jumptable). 
;

	JUMPTABLE = LONG (VDS ('7C'xl:'7F'xl), 0)

;
; Set WS_ADR to the address of the workspace within the memory dump that
; we received from CDS.
;

	WS_ADR = WORKSPACE AND 'FFFF'xl

;
; Set MM_ADR to the address of the memory map within the memory dump that
; we received from CDS
;

	MM_ADR = MEMORYMAP AND 'FFFF'xl

;
; Set JT_ADR to the address of the jump table within the memory dump that
; we received from CDS
;

	JT_ADR = JUMPTABLE AND 'FFFF'xl

;
; Write out the results of examineing the Page 0 memory addresses.
;

	PRINT, WS_ADR, FORMAT = '("Workspace:", T20, Z4.4, " H")'
	PRINT, MM_ADR, FORMAT = '("Memory Map:", T20, Z4.4, " H")'
	PRINT, JT_ADR, FORMAT = '("Jump Table:", T20, Z4.4, " H")'
	PRINT, MODRELOAD, FORMAT = '(I0, " Modules reloaded from EPROM.")'


;
; Print out a blank line.
;

	PRINT

;
; Define the structure to hold data on modules as we find them.  Note that
; type is set 1 for code modules and 2 for pattern modules.
;

        MODL = { VDS_MODULE_LST,		$
	         ERROR: 0,			$
                 ADDRESS: 0,			$
                 TYPE: 0,			$
                 NAME: "",			$
                 ID: 0,				$
                 BYTLEN: 0,			$
                 SEGLEN: 0,			$
                 OFFSET: 0}

;
; Write a line telling the user that we are about to begin searching the
; RAM space for modules.
;

	PRINT, "Searching RAM for Modules:


;
; Main loop to search for modules in the entire VDS memory dump.  As each
; module is found, the information from the module header is extracted and
; added to list of modules.
;


	FOR I = 0L, '7FFC'xl, 4 DO BEGIN

;
; Define TEMP to be the value of the 32 bit word at address I in the VDS
; memory dump.
;

	   TEMP = LONG (VDS (I:I+3), 0)

;
; Check if TEMP has the value of the ascii string "MDL*".  This mark the 
; beginning of a mudule in the VDS memory space.
;

	   IF TEMP EQ '2A4C444D'xl THEN BEGIN

;
; We found a module!  Begin to process it by making a copy of the module
; structure VDS_MODULE_LST in TMOD. 
;

	      TMOD = {VDS_MODULE_LST}

;
; Now set the fields of TMOD as appropiate.  Begin by setting address to the
; address in the VDS memory dump where we encountered the module.
;

	      TMOD.ADDRESS = I

;
; Check if word 1 of the module header has the value of the ascii string 
; "CODE".  If it does, then set type field of TMOD to 1.
;

	      IF LONG (VDS (I+4:I+7), 0) EQ '45444F43'xl THEN BEGIN
	         TMOD.TYPE = 1
	      ENDIF

;
; Otherwise, check if word 1 of the module header has the value of the ascii 
; string "PTRN".  If it does, then set type field of TMOD to 2.
;

	      IF LONG (VDS (I+4:I+7), 0) EQ '4E525450'xl THEN BEGIN
	         TMOD.TYPE = 2
	      ENDIF

;
; Check that module is either a pattern module or a code module.  If it was
; found to be neither, then we will assume this really wasn't a module after
; all.  In this case we will force another iteration of the loop.
;

	      IF TMOD.TYPE EQ 0 THEN GOTO, CONTINUE

;
; Set the name field of TMOD to word 2 of the module header.
;

	      TMOD.NAME = STRING (VDS (I+8:I+11))

;
; Set the ID field of TMOD to word 3 of the module header.
;

	      TMOD.ID = FIX (LONG (VDS (I+12:I+15), 0))

;
; Set the BYTLEN field of TMOD to word 5 of the module header.
;

	      TMOD.BYTLEN = FIX (LONG (VDS (I+20:I+23), 0))

;
; Set the SEGLEN field of TMOD to word 6 of the module header.
;

	      TMOD.SEGLEN = FIX (LONG (VDS (I+24:I+27), 0))

;
; Set the OFFSET field of TMOD to word 7 of the module header.
;

	      TMOD.OFFSET = FIX (LONG (VDS (I+28:I+31), 0))

;
; Move the current address being examined (I) to the end of the module, where
; the module termination string should be. 
;

	      I = I + '20'xl + TMOD.BYTLEN

;
; Check for the module termination string 'ZZZZ'.  If it does not exist, then
; we will have to mark an error condition. 
;

	      IF LONG (VDS (I:I+3), 0) NE '5A5A5A5A'xl THEN BEGIN
	         TMOD.ERROR = 1
	      ENDIF

;
; Move the current address being examined (I) to the next word after the 
; module termination string 'ZZZZ'.
;

	      I = I + 4

;
; Check the array MODLST already exists.  If it does, then add the record 
; into the array.  If it does not already exist, then create it.
;

	      IF N_ELEMENTS (MODLST) EQ 0 THEN BEGIN 
	         MODLST = TMOD 
	      ENDIF ELSE BEGIN
	         MODLST = [MODLST, TMOD]
	      ENDELSE

	   ENDIF

;
; Continue label to force a loop iteration.
;

	   CONTINUE:

	ENDFOR

;
; Set NMOD to the number of modules we found in RAM.
;

	NMOD = N_ELEMENTS (MODLST)

;
; Sort the module list by ID's.  Store the array subscripts which sort the
; module list by ID's in the array IDSORT
;

	IDSORT = SORT (MODLST.ID)

;
; Write a message telling the user how many modules we found in RAM.
;

	PRINT, NMOD, FORMAT = '("Found ", I0, " Modules in RAM.")'

;
; Print a three line header for a table listing the modules that we found
; in RAM.
;

	PRINT
	PRINT, "Name    Address   ID      Length  Status  Type"
	PRINT, "-------------------------------------------------"

;
; Loop to print out information on each of the modules that we found in RAM.
; 

	FOR I = 0, NMOD - 1 DO BEGIN

;
; Print the module name in the first column.
;

	   PRINT, MODLST (IDSORT (I)).NAME, FORMAT = '(A4, T9, $)'

;
; Print the module address in the VDS ram space in the second column.
;

           PRINT, MODLST (IDSORT (I)).ADDRESS, FORMAT = '(Z4.4, " H", T11, $)'

;
; Print the module ID in the third column.
;

           PRINT, MODLST (IDSORT (I)).ID, FORMAT = '(I0, T9, $)'

;
; Print the length of the module in bytes in the forth column.
;

	   PRINT, MODLST (IDSORT (I)).BYTLEN + 36, FORMAT = '(I0, T9, $)'

;
; Print the error status code for the module in the fifth column.
;

	   PRINT, MODLST (IDSORT (I)).ERROR, FORMAT = '(I2.2, T9, $)'

;
; Check the module type code.  If it is 1, then print 'Code' in the sixth
; column.  If it is 2, then print 'Pattern' in the sixth column. 
;

	   IF MODLST (IDSORT (I)).TYPE EQ 1 THEN PRINT, "Code"
	   IF MODLST (IDSORT (I)).TYPE EQ 2 THEN PRINT, "Pattern"

	ENDFOR  

;
; Print out a blank line.
;

	PRINT

;
; Set JWORD0 to the first word of the jump table.  Set JWORD1 to the second
; word of the jump table 
;

	JWORD0 = LONG (VDS (JT_ADR:JT_ADR+3), 0)
	JWORD1 = LONG (VDS (JT_ADR+4:JT_ADR+7), 0)

;
; Make sure that the jump table is at its expected position.  That is JWORD0
; has the value of the ascii string 'JMP*' and JWORD1 has the value of the
; ascii string 'TBLE'
;

	IF JWORD0 NE '2A504D4A'xl OR JWORD1 NE '454C4254'xl THEN BEGIN

;
; Set the variable JTFOUND to 0 to indicate that the jump table was 
; not found at the correct address.
;

	   JTFOUND = 0

;
; Write a message telling the user that we could not find the Jump Tabe.
;

	   PRINT, "Jump Table was not found at expected address!"

;
; Nothing more that we can do, so lets just get out of this section.
;

	   GOTO, END_JT_ANAL

	ENDIF

;
; Set JTFOUND to 1 to indicate that we found a jump table.
;

	JTFOUND = 1

;
; Write a message telling the user that we found the Jump Table.
;

	PRINT, "Found Jump Table."

;
; Set NENTRIES to the number of entries in the jump table, taken from the
; jump table header.
; 

	NENTRIES = LONG (VDS (JT_ADR+16:JT_ADR+19), 0)

;
; Set JLENBYTE to the length of the jump table in bytes
;

	JLENBYTE = LONG (VDS (JT_ADR+20:JT_ADR+23), 0)

;
; Set JLENSEG to the length of the jump table in memory segments.
;

	JLENSEG = LONG (VDS (JT_ADR+24:JT_ADR+27), 0)

;
; Write a message telling the user how many entries we found in the jump
; table (from the header).
;

        PRINT, NENTRIES, FORMAT = '("Number of Jump Table entries: ", I0)'

;
; Set NSLOTS to the number of slots which may contian entries in the jump
; table.  We base this number on the size of the jump table.
;

	NSLOTS = (JLENBYTE - 16) / 8

;
; Set JMODL to an array to hold a list of modules which have pointers in the 
; jump table.  We use this array to compare with each module that was found
; in memory. 
;

	JMODL = INTARR (2, NSLOTS)

;
; Set NFOUND to 0. We will use this to keep track of the number of entries that
; actually find in the jump table, in case this is different from the number 
; that we expect to find.   
;

	NFOUND = 0

;
; Set S0_ADR to the address of slot 0 in the vds memory dump. 
;

	S0_ADR = JT_ADR + 32

;
; Loop to search through all the jump table and look slots which used to hold
; a module pointer.  As we find modules with pointers in the jump table, we
; will add them to the array JMODL.
;

	FOR I = 0, NSLOTS - 1 DO BEGIN

;
; Set MODLADR to the contents of word 1 of the slot we are currently 
; examineing.  If the slot points to a module, then this word will contain the
; the address of that module in the Transputers memory space.
;

	   MODLADR = LONG (VDS (S0_ADR+(I*8)+4:S0_ADR+(I*8)+7), 0) 

;
; Check if slot I in the jump table is occupied.  If it is, then MODLADR will
; be nonzero.
;

	   IF MODLADR NE 0 THEN BEGIN

;
; Set the first element of row NFOUND (where NFOUND is number of occupied 
; slots that we have found so far) to slot number (and module) number of the
; module we just found.  Set the second element of the row to the address of
; the module we just found.
;

	      JMODL (0, NFOUND) = I
	      JMODL (1, NFOUND) = MODLADR AND 'FFFF'xl

;
; Increment NFOUND by one because we just found another module.
;

	      NFOUND = NFOUND + 1

	   ENDIF

	ENDFOR

;
; Write a message telling the user how many modules we actually found in the
; jump table.
;

	PRINT, NFOUND, FORMAT = '("Found ", I0, " active modules", $)
	PRINT, " in the Jump Table.

;
; Print out a blank line.
;

	PRINT

;
; Check if the number modules in the Jump Table according to the header 
; matches the number that are actually being used.  If there is a discrepency
; the write a warning message.
;

	IF NFOUND NE NENTRIES THEN BEGIN

	   PRINT, "Warning: The number of modules found in the Jump Table"
	   PRINT, "         does not match the number expected from reading"
	   PRINT, "         the Jump Table header.

	ENDIF 

;
; Set IMIF_RD_BUF to the contents of the IMIF Read Buffer, which is stored
; in the jump table.
;

	IMIF_RD_BUF = LONG (VDS (JT_ADR+484:JT_ADR+487), 0)

;
; Set IMIF_WR_BUF to the contents of the IMIF Read Buffer, which is stored
; in the jump table.
;

	IMIF_WR_BUF = LONG (VDS (JT_ADR+488:JT_ADR+491), 0)

;
; Set CMD_BUF to the contents of the Command Buffer, which is stored in the
; jump table.
;

	CMD_BUF = LONG (VDS (JT_ADR+492:JT_ADR+495), 0)

;
; Set IMIF_CHAN to the contents of the current IMIF channel, which is stored
; in the jump table.
;

	IMIF_CHAN = LONG (VDS (JT_ADR+496:JT_ADR+499), 0)

;
; Write out the values of the static variables which are store in the
; Jump Table.
;

	FMT = '("IMIF Read Buffer:  ", Z4.4, " H")'
        PRINT, FIX (IMIF_RD_BUF), FORMAT = FMT

	FMT = '("IMIF Write Buffer: ", Z4.4, " H")'
	PRINT, FIX (IMIF_WR_BUF), FORMAT = FMT  

	FMT = '("Command Buffer:    ", Z4.4, " H")'
	PRINT, FIX (CMD_BUF), FORMAT = FMT

	FMT = '("IMIF Channel is:   ", Z4.4, " H")'
	PRINT, FIX (IMIF_CHAN), FORMAT =  FMT 

;
; Loop to Check that all of the module listed in the Jump Table match up
; with actual modules found by searching the memory space.
;  

	FOR I = 0, NFOUND - 1 DO BEGIN

;
; Set J equal to 0.  This will be used as a loop control variable as we 
; search through the array MODLST, the modules found in RAM.
;

	   J = 0

;
; Set the flag MDLFND to 0.  We will use this flag to keep track of when we
; have encountered the module that we are searching for.
;

	   MDLFND = 0

;
; Loop to search the array of structures MODLST, which contains one sturcture
; for every module that was found in RAM.  The loop looks for structure with 
; the same module ID as the module described by row I of the array JMODL.  
; Once this structure is found and processed, the loop control variable J
; is set to the value of NMOD, which forces an immediate exit.
;
	   WHILE (J LT NMOD) AND NOT MDLFND  DO BEGIN

	      IF MODLST (J).ID EQ JMODL (0, I) THEN BEGIN


;
; Check that the module described by this structure has the same address as 
; the module that is described by by row I of the array JMODL.  If it is not
; then print a warning.
;

	         IF MODLST (J).ADDRESS NE JMODL (1, I) THEN BEGIN

	            PRINT, "Warning: Address of module ", FORMAT = '(A, $)'
	            PRINT, MODLST (J).ID, FORMAT = '(I0, " ", $)'
		    PRINT, "does not match Jump Table."

	         ENDIF	

;
; Set the flag MDLFND to 1 (TRUE) to indicate that we found the module that
; we were looking for.
;

	        MDLFND = 1

	      ENDIF

;
; Increment J by 1 so that the next structure in the array will be checked.   
;

	      J = J + 1	      

	   ENDWHILE

;
; Check if found the module that we were looking for.  If did not, that means
; that there is a module listed in the Jump Table with no corresponding module
; in memory, so print out a warning message.
;

	   IF NOT MDLFND THEN BEGIN

	      PRINT, "Warning: Could not find module ", FORMAT = '(A, $)'
	      PRINT, JMODL (0, I), FORMAT = '(I0, " in memory.")'

	   ENDIF      

	ENDFOR


;
; Label to end the jump table analysis section.
;

	END_JT_ANAL:

;
; Print out a blank line.
;

	PRINT

;
; Print out a message to the user telling them that we are starting to 
; display memory map data:
;

	PRINT, "Examining the Memory Map:"

;
; Loop to cycle through the Memory Map and examine each word.  The 
; Memory Map consists of 32 16 bit words, where every bit in each word 
; represents one segment or 64 bytes.  The loop ends when all the words
; in the memory map have been examined.
;

	FOR I = 0, 15 DO BEGIN

;
; Set ADRWRD to the address of the next word from the Memory Map to examine
; in the VDS memory space.
;

	   ADRWRD = MM_ADR + (I * 4)

;
; Set MAPWRD to the Ith word of the Memory Map.
;

	   MAPWRD = LONG (VDS (ADRWRD:ADRWRD+3), 0)

;
; Set MASK to a long integer with a value of 1.  MASK will be used to mask
; successive bits in MAPWRD so that they may be examined one at a time.
;

	   MASK = 1L

;
; Loop to examine every bit in the word MAPWRD.  Each bit represents one
; segment or 64 bytes of the VDS address space.  The loop ends when all
; 32 bits have been examined.
;

	   FOR J = 0, 31 DO BEGIN

;
; Check if the bit in MAPWRD at position I is SET (equal to 1).  If it is not
; then that means that the seqment represent by that bit is unusable and we
; will have to print out a message.
;

	      IF (MAPWRD AND MASK) EQ 0 THEN BEGIN

;
; Set SEGMENT to the memory segment represented by the bit that we just found
; to be invalid.
;

	         SEGMENT = I * 32 + J

;
; Write an error message telling the user which segment is invalid.
;

	         PRINT, "Memory Segment ", SEGMENT, FORMAT = '(A, I0, $)'
	         PRINT, SEGMENT * 64, FORMAT = '(" (at location ", Z4.4, $)'
	         PRINT, " H) is invalid."

	      ENDIF

;
; Left shift the contents of MASK by 1. This will allow MASK to be used to 
; examine the next bit in MAPWRD on the next iteration of this loop.
;

	      MASK = ISHFT (MASK, 1)

	   ENDFOR 

	ENDFOR

;
; Print out a blank line.
;

	PRINT

;
; Print a message telling the user that we are going to print out the values
; of selected system variables.
;

	PRINT, "Values of Selected System Variables:"

;
; Create an anonymous structure to hold the name and offset of the system
; variables.
;

	STRUCT = {NAME:		"",		$
	          OFFSET:	0,		$
	          FORMAT:	""}
;
; Replicate the stucture STRUCT into an array of structures.  Each structure
; in the array will hold the name and offset of one system variable.
;

	SYSDEF = REPLICATE (STRUCT, 13)

;
; Set the name element of all the structures in the array SYSDEF.
; 

	SYSDEF.NAME = ["STATUS",		$
	               "SCANLINE",		$
	               "SIZE.SCAN.LINE",	$
	               "NO.SCAN.LINES",		$
	               "DATA.POINT",		$
	               "MODE",			$
	               "EXPOSURE",		$
	               "NO.EXPOSURES",		$
	               "VALUE",			$
	               "COMMAND.RECEIVED",	$
	               "READOUT.OVER",		$
	               "READOUT.TYPE",		$
	               "SLOW.PORT.MASK"]

;
; Set the offset element of all the structures in the array SYSDEF.  The 
; offset element is set the offset of the system variable described by the
; structure to the top the Work Space area.  
;

	SYSDEF.OFFSET = [16,			$
	                 20,			$
	                 24,			$
	                 28,			$
	                 32,			$
	                 36,			$
	                 60,			$
	                 64,			$
	                 92,			$
	                 96,			$
	                 100,			$
	                 120,			$
	                 144]
	                 
;
; Set the format element of all the structures in the array SYSDEF. The 
; format element controls how the system variable will be displayed.
;              

	SYSDEF.FORMAT = ['Z4.4, " H"',		$
	                 'I0',			$
	                 'I0',			$
	                 'I0',			$
	                 'I0',			$
	                 'I0',			$
	                 'I0',			$
	                 'I0',			$
	                 'Z4.4, " H"',		$
	                 'Z4.4, " H"',		$
	                 'Z4.4, " H"',		$
	                 'I0',			$
	                 'Z4.4, " H"']

;
; Loop to print out the name and values of the system variables that we have
; decided to display.  The loop ends when the system variables written to the
; screen.
;

	FOR I = 0, 12 DO BEGIN

;
; Write the name of the system variable.  This is stored in the Ith structure
; in the array SYSDEF.
;

	   PRINT, SYSDEF (I).NAME, FORMAT = '(A, T20, $)'

;
; Set VARADR to the address of the Ith system variable described in the array
; SYSDEF.
;

	   VARADR = WS_ADR - SYSDEF (I).OFFSET

;
; Set VAL to the value of the system variable at address VARADR.
;

	   VAL = LONG (VDS (VARADR:VARADR+3), 0)

;
; Write out the value of the system variable which is stored in VAL.
;

	   PRINT, VAL, FORMAT = '(' + SYSDEF (I).FORMAT + ')'


	ENDFOR

;
; Print out a blank line.
;

	PRINT

;
; Write a message telling the user that we are done.
;

	PRINT, "All checks complete."

;
; Were done. Yeahh!!!
;

	RETURN

	END
