	PRO VDS_MEM_DISP, FILE, LOW, HIGH, SEGMENT=SEGMENT
;+
; Project     :	SOHO - CDS
;
; Name        :	VDS_MEM_DISP
;
; Purpose     :	Display the contents of a NIS Memory Dump
;
; Explanation :	This program displays the contents of a NIS memory dump in
;		both hex format and as ascii characters.  Eight bytes of 
;		memory are displayed one line, with the display optionally
;		broken up into segments (64 byte sections of memory).  The
;		default behavior is for the entire memory dump to be
;		displayed, however a smaller range can be selected by useing
;		the optional parameters LOW and HIGH.   
;
; Use         :	VDS_MEM_ANAL, FILE 
;
; Inputs      :	FILE:	The name of the memory dump file to analyze.  The
;			memory dump file is produced by STM when CDS completes
;			a NIS memory dump.
;
; Opt. Inputs :	LOW:	The lowest address to include in the memory dump.  
;			Addresses are specified as bytes from the beginning
;			of the memory dump, not as Transputer addresses.
;
;		HIGH:	The highest address to include in the memory dump.
;			Addresses are specified as bytes from the beginning
;			of the memory dump, not as Transputer addresses.
;
; Outputs     :	None.
;
; Opt. Outputs:	None.
;
; Keywords    :	SEGMENT:If defined and non-zero, the memory dump display will
;			be broken up into 64 byte segments.  Each segment will
;			also be numbered.  Segments are the unit at which the
;			NIS on board software manipulates memory. 
;
; Calls       :	None.
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Category    :	Engineering.
;
; Prev. Hist. :	None.
;
; Written     :	Ron Yurow, GSFC, 30 October 1997
;
; Modified    :	Version 1, Ron Yurow, GSFC, 30 October 1997
;
; Version     :	Version 1, 30 October 1997
;-
;

;
; Open the file that contians the VDS memory dump
; 

	OPENR, LUN, FILE, /GET_LUN

;
; Create a byte array to hold 256 bytes of data from the VDS memory dump file.
; We will use this array to read the memory dump file into the larger array
; VDS, which will hold the entire memory dump. 
;

	TEMP1 = BYTARR (256)

;
; Create a byte array to hold 256 bytes of data from the VDS memory dump file.
; This will be same data stored in TEMP1, only in word reversed format to 
; correct the byte order that CDS stores the memory dump in.
;

	TEMP2 = BYTARR (256)

;
; Create a byte array to hold the entire VDS memory dump.
;

	VDS = BYTARR (32768)

;
; Loop to read in the VDS memory dump file into the array VDS. The memory dump
; file is read blocks of 256 bytes.
;

	FOR I = 0, 127 DO BEGIN

;
; Read 256 Bytes from the VDS memory dump file.
;

	   READU, LUN, TEMP1

;
; Copy the bytes from TEMP1 to TEMP2.  As we do this, reverse the byte order
; inside each 16 bit word.
;

	   FOR J = 0, 127 DO BEGIN

	      TEMP2 (J * 2) = TEMP1 (J * 2 + 1)
	      TEMP2 (J * 2 + 1) = TEMP1 (J * 2)

	   ENDFOR

;
; Copy the bytes that we just read into the array VDS.
;

	   VDS (I * 256) = TEMP2

	END

;
; Set the OFFSET counter to the position within one display line that the 
; current byte will occupy.  Since each display line consists of eight bytes
; it will also tell when the entire line has been displayed.
; 

	OFFSET = LOW MOD 8

;
; Set LINE to a 8 byte array.  We will use line to hold the text portion
; of our display line.
;

	LINE = BYTARR (8)

;
; Check if the keyword SEGMENT is defined.  If it is then print the 
; segment number of the first byte to display.
;

	IF KEYWORD_SET (SEGMENT) THEN BEGIN
	   PRINT, FIX (LOW / 64), FORMAT = '("Segment ", I3)'
	ENDIF

;
; Print the address of the first line  to the display.
;

	PRINT, LOW, FORMAT = '(Z4.4, ":  ", $)'

;
; Print out a number of spaces, so that we start off displaying hex values in
; the correct position
;

	FOR I = 0, OFFSET - 1 DO PRINT, "   ", FORMAT = '(A, $)'

;
; Main loop to print out a memory dump of the entire VDS memory space.
; Each line will be 8 bytes from the dump, displayed in both Hex and ascii
; formats.  A total of 4096 lines will be displayed.

	
	FOR I = LOW, HIGH DO BEGIN

;
; Print the next byte at address I in hex format.
;

	   PRINT, VDS (I), FORMAT = '(Z2.2, " ", $)'

;
; Copy the byte we just displayed to the array LINE at subscript OFFSET.  This
; will be useful when we display the line in text format.
;

	   LINE (OFFSET) = VDS (I)

;
; Increment OFFSET by one.
;

	   OFFSET = OFFSET + 1

;
; Check if OFFSET is equal to 8.  If it is, then it is time to finish off the
; current line and begin a new line.
;

	   IF OFFSET EQ 8 THEN BEGIN

;
; Reset OFFSET to zero.
;

	      OFFSET = 0

;
; Convert any character in line that is less then '20'XB to '20'XB
;

	      LINE = LINE > '20'XB

;
; Set the array REPLACE to the subsripts of the array  LINE which reference
; any character that is greater then '79'XB.
;

	      REPLACE = WHERE (LINE GT '79'XB)

;
; Check if REPLACE is equal to -1.  If it is not, then replace the elements in
; LINE which are referenced by the subscripts in REPLACE with the value
; '20'XB.
;

	      IF REPLACE (0) NE -1 THEN LINE (REPLACE) = '20'XB

;
; Print out the array LINE as a string.
;

	      PRINT, STRING (LINE)

;
; Check if there is another line following this one.
;

	      IF I LT HIGH THEN BEGIN 

;
; Check if the next line starts a new segment.
;

	         IF (I + 1) MOD 64 EQ 0 AND KEYWORD_SET (SEGMENT) THEN BEGIN

;
; Print a blank line and then the segment number of the first byte of the 
; next byte that will be displayed.
;

	            PRINT
	            PRINT, FIX ((I + 1) / 64), FORMAT = '("Segment ", I3)'
	            
	         ENDIF

;
; Print the address of the first byte that will be displayed on the next line.
;

	         PRINT, I + 1, FORMAT = '(Z4.4, ":  ", $)'

	      ENDIF

	   ENDIF

	ENDFOR

;
; Check if OFFSET is any value but 0.  If is then we have to finish off the
; line we were writing when the loop ended.
;

	IF OFFSET NE 0 THEN BEGIN

;
; Loop to print out spaces where hex values would have been displayed, if we
; had not run out of memory dump values.  In addition, unused elements in 
; the array line are set to 0.
;

	   FOR I = OFFSET, 7 DO BEGIN
	      PRINT, "   ", FORMAT = '(A, $)'
	      LINE (I) = 0
	   ENDFOR

;
; Convert any character in line that is less then '20'XB to '20'XB
;

	   LINE = LINE > '20'XB

;
; Set the array REPLACE to the subsripts of the array  LINE which reference
; any character that is greater then '79'XB.
;

	   REPLACE = WHERE (LINE GT '79'XB)

;
; Check if REPLACE is equal to -1.  If it is not, then replace the elements in
; LINE which are referenced by the subscripts in REPLACE with the value
; '20'XB.
;

	   IF REPLACE (0) NE -1 THEN LINE (REPLACE) = '20'XB

;
; Print out the array LINE as a string.
;

	   PRINT, STRING (LINE)


	ENDIF


	RETURN

	END