;+
; Project     : SOHO - CDS     
;                   
; Name        : VDS_STATS
;               
; Purpose     : To calculate the statistics within a VDS image.
;               
; Explanation : Within the EGSE science analysis display of VDS data an option
;               is provided to select a region of the image using a box cursor.
;               The cursor position and size can be varied.  Some basic
;               statistics are reported for the region of data selected.
;               
; Use         : vds_stats, image, sx, sy, px, py, avg, sigma, imax, imin
;               vds_stats, image, /all
;    
; Inputs      : image  - the full VDS image array
;               sx, sy - the x,y size in pixels of the region chosen
;               px, py - the start pixel in x,y of the region chosen.
;               
; Opt. Inputs : None
;               
; Outputs     : avg   - the mean value within the region
;               sigma - the standard deviation within the region
;               imin  - the minimum value within the region
;               imax  - the maximum value within the region
;               
; Opt. Outputs: None
;               
; Keywords    : All  -  do statsa for all non-zero pixels in image
;
; Calls       : None
;               
; Restrictions: None
;               
; Side effects: None
;               
; Category    : EGSE
;               
; Prev. Hist. : None
;
; Written     : C D Pike, RAL, 10-Nov-1993
;               
; Modified    : Add /all keyword, CDP, 17-Feb-94
;               Update std dev output format.  CDP, 01-Mar-96
;
; Version     : version 3, 01-Mar-96
;-            

pro vds_stats, image, sx, sy, px, py, avg, sigma, vmax, vmin, all=all

;
;  COMMON for widget IDs
;
@VDSWidgets.com

;
;  check for /all keyword
;
if keyword_set(all) then begin
   n = where(image gt 0)
   if n(0) eq -1 then begin
      flash_msg, vds_msg_widg,'All data are zero.'
      return
   endif
   temp = image(n)
   avg = total(temp)/n_elements(temp)
   vmax = max(temp) & vmin = min(temp)
   if n_elements(temp) gt 1 then begin
      sigma = sqrt(total((temp-avg)^2)/(n_elements(temp)-1))
   endif else begin
      sigma = 0.0
   endelse
   px = 0 & py = 0
   sx = 1024 & sy = 1024

endif else begin

;
;  extract the region of interest to a temporary array
;
   temp = image(px:px+sx-1,py:py+sy-1)

;
;  the mean value
;
   avg = total(temp)/(sx*sy)

;
;  min and max values in subarray
;
   vmax = max(temp) & vmin = min(temp)

;
;  standard deviation
;
   if sx*sy gt 1 then begin
      sigma = sqrt(total((temp-avg)^2)/(sx*sy-1)) 
   endif else begin
      sigma = 0.0
   endelse

endelse

;
;  display in statistics widgets
;
widget_control,vds_exp_swidg(0),set_val=fmt_vect([px,py])
widget_control,vds_exp_swidg(1),set_val=fmt_vect([sx,sy])
widget_control,vds_exp_swidg(2),set_val=string(avg,format='(2x,f6.1)')
widget_control,vds_exp_swidg(3),set_val=string(sigma,format='(3x,f6.1)')
widget_control,vds_exp_swidg(4),set_val=fmt_vect([vmin,vmax])


end
