;+
; Project     : SOHO - CDS     
;                   
; Name        : WORKPROC
;               
; Purpose     : Background task to perform telemetry reading.
;               
; Explanation : The WORKPROC procedure is registered with the IDL widget-
;               handling routine XMANAGER as a background task.  This means
;               it is called as often as possible while the main widget is
;               idling ie waiting for user input.  WORKPROC performs the basic
;               reading of telemetry packets.  It is designed to handle various
;               packets depending on the packet identification it finds within
;               the packet header (first 20 bytes - read by read_pkt_header).
;               Only science packets and special packets are dealt with here.
;               Any other packets are read but disregarded.
;               
; Use         : workproc, widget_base
;    
; Inputs      :  widget_base  - the top level widget identification.
;               
; Opt. Inputs : None
;               
; Outputs     : None
;               
; Opt. Outputs: None
;               
; Keywords    : None
;
; Calls       : read_pkt_header    -  read 20 byte packet header
;               read_VDS_raster    -  read VDS specific packet header
;               read_GIS_raster    -  read GIS specific packet header
;               read_data          -  read general unidentified packet
;               handle_io_err      -  perform close of input unit on error
;               handle_tm_err      -  perform close of input unit on error
;               read_sci_cnt       -  read science data packet counter from the
;                                     exposure header
;               read_exp           -  read the exposure header packet
;               Update_Expinfo     -  update display of exposure number
;               dec2bin            -  convert decimal integer to binary
;               VDS_Ext_image      -  extract VDS windows from science buffer
;               vds_deselect       -  to deselect VDS data into 1k array
;               gis_deselect       -  to deselect GIS data into spectral array
;               Plot_VDS           -  display the VDS image.
;               GIS_Ext_plot       -  extract GIS data windows from buffer
;               Plot_GIS           -  plot the GIS data
;               read_sp_id         -  read special packet format id
;               read_sp_body       -  read body od a special packet
;               disp_deferred      -  display the contents of a special 
;                                     deferred command store dump
;               disp_imif          -  display the contents of the special
;                                     IMIF (MCU) interface module dump
;               disp_gen_table     -  display contents of special general table
;
;               
; Restrictions: For use only with EGSE science analysis.
;               
; Side effects: None
;               
; Category    : EGSE
;               
; Prev. Hist. : EM version by Arnulf, Oct 92
;
; Written     : FM version by C D Pike, RAL, 12-Oct-93
;               
; Modified    : Operations version.       CDP, RAL, 12-Aug-94
;               Fix bugs.                 CDP, 8-Dec-94
;               Changed for realtime operation from tm file.  
;                                         CDP, 6-Jan-95
;               Fix lack of outout of packet counter.  CDP, 10-Jan-95
;               Added PAUSE facility.     CDP, 11-Jan-95
;               Make it always displays special packets regardless
;                  of mode - eases dumping to files.  
;                                         CDP, 24-Apr-95
;               Fix for timing problems on Ultrix w/station
;                                         CDP, 11-May-95
;               Zero packet counter on replay file change. 
;                                         CDP, 22-May-95
;               Unsign monitor GIS data.  CDP, 26-May-95
;               Added facility to wind back to find beginning of raster.
;                                         CDP, 30-May-95
;               Make lookback mode optional. CDP, 5-Jun-95
;               Trap empty deferred command dump file. CDP, 23-Jun-95
;               Added message in main window with study number. CDP, 26-Jun-95
;               Added message re change of realtime file to main window.
;                                                               CDP, 20-Jul-95
;               Limit number of loops for empty packets. CDP, 28-Jul-95
;               Add automatic dump of GIS raw data. CDP, 29-Aug-95
;               Pick out filament data from every EngA packet.  CDP, 8-Sep-95
;               Re-sensitise the file selection after eof. CDP, 11-Sep-95
;               Fix bug in filament data select.  CDP, 13-Sep-95
;
;               Change philosophy of handling no data in realtime mode.
;               If no new data are available after the timeout period
;               a 'loss of telemetry' message is given and STM keeps
;               waiting for more.        CDP, 15-Sep-95
;  
;               Put out packet time in main widget title.  CDP, 18-Sep-95
; 
;               Rework GIS raw data handling.  CDP, 11-Oct-95
;               Include memory dumping.  CDP, 12-Oct-95
;               Changed realtime packet reading delay to 0.05s from 0.1s. 
;                                        CDP, 13-Oct-95
;               Undo bug introduced when GIS raw data dumping was 
;               revamped.                CDP, 18-Oct-95
;
;               Add vds_centre to vds_data_str.  CDP, 20-Oct-95
;
;               Make all filament parameters int16.  CDP, 23-Oct-95
;
;               Desensitize the DONE buttons of header and data display.
;                                                    CDP, 14-Nov-95
;               Add study ID display feature.        CDP, 15-Nov-95
;               Slight tweak to above.               CDP, 16-Nov-95
;
;               Include pickup of VDS timer unit from EngA. CDP, 21-Nov-95
;
;               Add auto switch off of memory dump.  CDP, 28-Nov-95
;
;               Act on seeing Engineering studies and display study titles.
;                                                    CDP, 12-Dec-95
;               Keep check on DCS status and report errors.  CDP, 18-Dec-95
;
;               Fix bug in engineering ID display.  CDP, 20-Dec-95
;               Save GIS ffb value from EngB packet. CDP, 20-Dec-95
;               Add notification of GIS LUT loading and Warm Start table
;                                                    CDP, 22-Dec-95
;               Change 'running WARM START' message. CDP, 08-Jan-96
;               Include SEU detection warning.       CDP, 11-Jan-96
;               Change SEU detection address format. CDP, 15-Jan-96
;               Change gis_ffb to array.             CDP, 24-Jan-96
;               Added SEU detection to secondary memory. CDP, 19-Feb-96
;               Get VDS exposure time directly from AKVEXPTM parameter.
;                                                        CDP, 27-Feb-96
;               Cut out call to cp_get_state.            CDP, 01-Mar-96
;               Fix bug that was not recording last 
;               study ID seen when idling.               CDP, 15-Apr-96
;               Cut out 'science counter too large' message
;               and trap silly seu address message
;                                                        CDP, 26-Jul-96
;               Add realtime check on packet header contents
;               to deal with .REL source file problem.   CDP, 14-Apr-98
;            
;               Changed call to handle error routines.  CDP, 19-Jun-98
;               Cater for IDL5.2 hex bug.  CDP, 1-6-99
;               Kill bell warning on loss of real time telemetry.  CDP 15-May-00
;
; Version     : Version 51, 15-May-2000
;-            

PRO workproc,widget_base

;
;  load all the possible COMMON blocks for communications
;
@Images.com
@HeaderWidgets.com
@MainWidgets.com
@VDSWidgets.com
@GISWidgets.com
@Packets.com
@Rawscience.com
@IO_units.com
@Mode.com
@Decomp.com
@SpecialWidgets.com
@ReviewWidgets.com
@VDSData.com
@GISData.com
@RawData.com
@Limits.com
@Monitor.com
@PHA_data.com
@Mem_dump.com

;
;  initialise the counters and data arrays
;
id = FIX(0)
result = FIX(0)
sci_counter = FIX(0)
packet_header = {PacketHeaderDef}

;
;  set up look up tables for translation of packet id's in review mode
;
normals = ['0a'xl,'0b'xl,'01'xl,'02'xl,'03'xl,'11'xl,'12'xl,'21'xl,'22'xl,$
           '04'xl,'05'xl,'06'xl,'07'xl]

specials = ['01'xl,'02'xl,'03'xl,'04'xl,'05'xl,'08'xl,'09'xl,'0a'xl,'0b'xl,'0e'xl]
  
;
;  If paused then wait and exit.
;
if sci_pause then begin
   wait,0.5
   return
endif


;
;  determine mode of operation.
;   sci_mode = 1 --> realtime
;   sci_mode = 0 --> replay
;  the logical unit used for replay is allocated in main_event when the 
;  telemetry reading is started and that for realtime similarly.
;
if (sci_mode EQ 1) then unit = realtime_unit else unit = replay_unit

;
;  read the first 20 bytes of every packet header.  The variable packet_header
;  is of type PacketheaderDef (see stm.pro).  This also the return point for
;  realtime ops when a read error occurs.
;

empty_count = 0
read_header:

;
;  slow things down in realtime to give chance for disk i/o
;
if sci_mode and not catching_up then wait,0.05
if catching_up and (packet_no ge to_catchup) then catching_up = 0

;
;  read packet header which will show up any i/o errors and give packet ID
;
result = read_pkt_header(sci_mode, unit, packet_header)
;print,anytim2cal(tai2utc(obt2tai(packet_header.time)),form=1), packet_header.ID

;
;  timing etc delay to cope with problems caused by Ultrix workstation
;  i/o idiosyncrasies.  On Ultrix, packets would appear to be ready for
;  reading judged by the file size, but when read had nothing in them
;  ie they had not really been written.  So if they appear to read OK
;  (result = 1) but are empty then wait a bit and go back and try again.
;  It was also found necessary to release the file by closing it and
;  reopening in order to allow the complete update.
;
;  (added restriction of being in realtime because it was having problems with 
;   files generated from .REL files - which have msgid == 0)
;
if (packet_header.msgid eq 0) and (result eq 1) and (sci_mode eq 1) then begin
   free_lun,realtime_unit
   wait,2
   openr, realtime_unit,curr_file,/get_lun
   pkt_read = packet_no + start_pack  
   point_lun, realtime_unit, long(pkt_read)*306L
   print,'Empty packet in realtime disk file, trying again.....'
   empty_count = empty_count + 1
   if empty_count gt 10 then begin
      widget_control,msg_widg,/append,$
         set_val='Giving up - too many empty packets.'
      handle_io_err, widget_base, msg_widg, dummy_label, unit
      return
   endif
   goto, read_header
endif

;
;  if the return value is zero, this indicates an error in realtime
;  mode and indicates that the reading has reached the current end of file
;  (or possibly the real end of file).
;
if (result EQ 0) then begin
 
   pkt_read = packet_no + start_pack  
;
;  is it worth trying again? A 10 second timeout is allowed
; 
   waited = 0
   pkt_avail = 0
   while ((pkt_avail - pkt_read) lt 1) and (waited lt 20) do begin
      wait,0.5
      waited = waited + 1
      tm_size = tm_get_size(curr_file)
      pkt_avail = tm_size/306L
   endwhile

;
;  time out? then probably onto new file
;  if error reading new file or new file is in fact same as old file
;  (ie haven't switched tm files) then treat as unrecoverable error.
;
   if waited eq 20 then begin
      old_file = curr_file
      free_lun,unit
      curr_file = concat_dir('$CDS_TM_DATA',gt_rt_file())
;
;  if it's not a file switch then must have lost telemetry in realtime
;
      if curr_file eq old_file then begin
         widget_control, msg_widg, /append, $
                         set_val='Loss of realtime telemetry'
;         bell
         free_lun,realtime_unit
         openr, realtime_unit,curr_file,/get_lun
         pkt_read = packet_no + start_pack  
         point_lun, realtime_unit, long(pkt_read)*306L
         return
      endif
      openr,realtime_unit,curr_file,/get_lun,error=err
      if err ne 0 then begin
         handle_io_err, widget_base, msg_widg, dummy_label, unit 
         widget_control, pbase(1), sens=1
         return
      endif else begin
         widget_control, msg_widg, /append, $
              set_val='Accessing new realtime file '+curr_file
         unit=realtime_unit
         start_pack = 0
         packet_no = 0L
         goto, read_header
      endelse        
   endif else begin
;
;  ok more packets have appeared, make sure of file positioning
;  and try again
;
      goto, read_header
   endelse
endif


;
;  possible returns are still -1 (error) and +1 (success)
;

;
;  on error see if there is another data file because error may be due to EOF
;
if (result EQ -1) then begin
;
;  realtime i/o error (not just lack of data)
;
   if sci_mode eq 1 then begin
      handle_io_err, widget_base, msg_widg, dummy_label, unit
      widget_control, pbase(1), sens=1
      return
   endif else begin
;
; replay mode i/o error so try another data file if appropriate
;
      if file_seq_no lt (num_data_files-1) then begin
         free_lun,replay_unit
         file_seq_no = file_seq_no + 1
         curr_fsize = tm_get_size(file_name(file_seq_no))/306L
         openr,replay_unit,file_name(file_seq_no),err=ioerr,/get_lun
;
;  new file opened successfully so read the packet header 
;
         if ioerr eq 0 then begin
            unit = replay_unit
            widget_control,msg_widg,/append,set_v='Opening new data file.'
            break_file,file_name(file_seq_no),disk,dir,rfile,rext
            year = fix(strmid(rfile+rext,3,2)) + 1900
            doy = fix(strmid(rfile+rext,6,3))
            date = ' ('+strmid(anytim2cal(doy2utc(doy,year),f=1),0,6)+')'
            widget_control,rep_widg,set_val=rfile+rext+date
            result = read_pkt_header(sci_mode, unit, packet_header)
            packet_no = 0L

;
;  trap i/o error even at this late stage
;
            if result lt 0 then begin
               handle_io_err, widget_base, msg_widg, dummy_label, unit
               curr_fsize = 0L
               widget_control, pbase(1), sens=1
               return
            endif
         endif else begin
;
;  error on opening new data file
;
            bell
            widget_control,msg_widg,/append, $
                           set_v='Error opening new data file.'
            handle_io_err, widget_base, msg_widg, dummy_label, unit
            widget_control, pbase(1), sens=1
            return
         endelse
      endif else begin
;
;  no more data files to be had so it really is the end
;
         widget_control,msg_widg,/append, $
                        set_val='End of replay data.'
         handle_io_err, widget_base, msg_widg, dummy_label, unit
         widget_control, pbase(1), sens=1
         return
      endelse
   endelse
endif

;
;  if reached this stage then reading of packet header was OK
;  Check if in the middle of a raster, if so then go search for beginning
;  if look back mode is on.
;

if first_packet and lback_mode then begin
   if sci_mode eq 0 then lfile = file_name(file_seq_no) else lfile = curr_file
   if packet_header.ID eq '12'xl or $
         packet_header.ID eq '22'xl then begin
      widget_control, msg_widg, set_val=' ',/app
      widget_control, msg_widg, set_val=' ',/app
      flash_msg, msg_widg, 'Searching for VDS raster start',/app
      if sci_mode eq 0 then begin
         tlm_summary,lfile,[2],found_st,/quiet,packs=0,packe=skip_pack
      endif else begin
         tlm_summary,lfile,[2],found_st,/quiet,packs=0,packe=start_pack
      endelse
      if max(found_st) ge 0 then begin
         found_st = max(found_st)
         point_lun,unit,found_st*306L
         if sci_mode eq 0 then begin
            to_catchup = skip_pack-1
            packet_no = long(found_st)
            widget_control,spacket_widg,$
                           set_val=strtrim(string(found_st,form='(i5)'),2)
            widget_control, msg_widg, set_val='Start packet has been reset.',$
                            /append
            skip_pack = found_st
         endif else begin
            to_catchup = start_pack - found_st
            packet_no = 0L
            skip_pack = 0
            catching_up = 1
            start_pack = found_st
         endelse
;        print,'to catch = ',to_catchup
         goto, read_header
      endif else begin
         flash_msg,msg_widg,'No VDS RASTER beginning found in current file.',$
                  /append
      endelse
   endif

   if packet_header.ID eq '11'xl or $
      packet_header.ID eq '21'xl then begin
      widget_control, msg_widg, set_val=' ',/app
      widget_control, msg_widg, set_val=' ',/app
      flash_msg, msg_widg, 'Searching for GIS raster start',/app
      if sci_mode eq 0 then begin
         tlm_summary,lfile,[1],found_st,/quiet,packs=0,packe=skip_pack
      endif else begin
         tlm_summary,lfile,[1],found_st,/quiet,packs=0,packe=start_pack
      endelse
      if max(found_st) ge 0 then begin
         found_st = max(found_st)
         point_lun,unit,found_st*306L
         if sci_mode eq 0 then begin
            to_catchup = skip_pack-1
            packet_no = long(found_st)
            widget_control,spacket_widg,$
                           set_val=strtrim(string(found_st,form='(i5)'),2)
            widget_control, msg_widg, set_val='Start packet has been reset.',$
                            /append
            skip_pack = found_st
         endif else begin
            to_catchup = start_pack - found_st
            packet_no = 0L
            skip_pack = 0
            catching_up = 1
            start_pack = found_st
         endelse
         goto, read_header
      endif else begin
         flash_msg,msg_widg,'No GIS RASTER beginning found in current file.',$
                  /append
      endelse
   endif
;
;  if this was the first science data packet then switch off the trap
;
   if packet_header.ID eq '1'xl or $  
      packet_header.ID eq '11'xl or $  
      packet_header.ID eq '21'xl or $  
      packet_header.ID eq '2'xl or $  
      packet_header.ID eq '12'xl or $  
      packet_header.ID eq '22'xl then first_packet = 0

endif



;
;  record the number of packets (of all kinds) received so far
;   
packet_no = packet_no + LONG(1)

;
;  in replay update counter display every 10th packet,  update the display 
;  every packet in realtime mode to encourage onlookers
;

if sci_mode eq 0 then begin
   if ((packet_no MOD 10) EQ 0) then begin 
      pc = (float(packet_no)/float(curr_fsize))*100.0
      percent = '  ('+strtrim(string(fix(pc),form='(i3)'),2)$
                                                      +'%)'
      WIDGET_CONTROL, packet_widg, SET_VALUE=STRING(packet_no,form='(i8)')$
                                                      +percent   
      ut = anytim2cal(tai2utc(obt2tai(packet_header.time)))
      widget_control, main_base, tlb_set_title='CDS STM - '+ut  
   endif
endif else begin
   ut = anytim2cal(tai2utc(obt2tai(packet_header.time)))
   widget_control, main_base, tlb_set_title='CDS STM - '+ut  
   widget_control, packet_widg, set_val=string(packet_no,form='(i8)')
endelse

   
;
;  before we proceed, check it wasn't an ABORT PACKET (id = '03'xl)
;                                        ============
;  if it was then warn and act as if last packet received.
;

   if packet_header.ID eq '03'xl then begin
      bell,3
      widget_control,msg_widg,/append, $
                     set_val=systime()+' ***  ABORT packet received. ***'
;
;  check if both raster and exposure header packets were received.  If so
;  then we have at least the data in the exposure header packet so 
;  attempt to process it.  The body of the abort packet will be read at the
;  end as any unwanted packet is.
;
      if check_headers('VDS') then begin
        dcmp_win    = VDSrasterhdr.VDSwin(0:VDSrasterhdr.NumDataWindows-1)
        scheme      = VDSrasterhdr.Scihead.DataCompScheme 
        option      = VDSrasterhdr.Scihead.DataCompOption
        detector    = 0
        print,'Decompressing VDS data with...'
        print,'Scheme/Data comp method: ',scheme
        print,'Option/Data comp parameter: ',option
        print,'Data orientation: ',vds_orientation
        buff16_sz = n_elements(sci_buffer)/2
        sci_buff16 = dcmpdata(fix(sci_buffer,0,buff16_sz),dcmp_win,detector,$
                              scheme,option,vds_orientation)
        VDS_Ext_image, sci_buff16, VDSrasterhdr, $
                                   vds_data_str.vds_exp, msg_widg
        if vds_display then begin
           vds_data_str.exp_in_a = vds_data_str.vds_exp
           vds_data_str.curr_a_img = $
                    vds_deselect(vdsrasterhdr,vds_data_str.exp_in_a - $
                                              vds_data_str.first_in_mem)
           Plot_VDS, vds_awidg, vds_data_str.curr_a_img
        endif
        have_v_rst_hdr = 0
        have_v_exp_hdr = 0
      endif 

      if check_headers('GIS') then begin
        dcmp_win = replicate({StartX:0,StartY:0,LengthX:0,LengthY:0},$
                             GISrasterhdr.NumDataWindows)
        for i=0,GISrasterhdr.NumDataWindows-1 do begin
           dcmp_win(i).StartX  = GISrasterhdr.GISwin(i).Start
           dcmp_win(i).StartY  = 0
           dcmp_win(i).LengthX = GISrasterhdr.GISwin(i).Length
           dcmp_win(i).LengthY = 1
        endfor
        scheme = GISrasterhdr.Scihead.DataCompScheme 
        option = GISrasterhdr.Scihead.DataCompOption
        vds_orientation = 0
        detector = 1
        print,'Decompressing GIS data with...'
        print,'Scheme/Data comp method: ',scheme
        print,'Option/Data comp parameter: ',option
        print,'Data orientation: ',vds_orientation
        buff16_sz = n_elements(sci_buffer)/2
        sci_buff16 = dcmpdata(fix(sci_buffer,0,buff16_sz),dcmp_win,detector,$
                              scheme,option,vds_orientation)
      
        GIS_Ext_plot, sci_buff16, GISrasterhdr, gis_plot, $
                                  gis_data_str.gis_exp, msg_widg       
        if gis_display then begin
           gis_data_str.exp_in_a = gis_data_str.gis_exp
           gis_data_str.curr_a_data = gis_deselect(gisrasterhdr,gis_plot,$
                                                   gis_data_str.gis_exp, $
                                                   msg_widg)
           Plot_GIS, gis_data_str.curr_a_data
        endif

        have_g_rst_hdr = 0
        have_g_exp_hdr = 0
        cds_mode = 0
      endif
   endif

;
;  should have a valid packet header ID so save it
;
   last_pkt_id = packet_header.ID

;
;  Deal with statistics (just in case required later
;  if it's a special packet then wait until we know the format id
;
   if last_pkt_id ne '30'xl then begin
      index = where(normals eq last_pkt_id)
      index = index(0)
      if index ge 0 then begin
         review_stats(index) = review_stats(index) + 1
         new_rev_stat = index
      endif
   endif
;
;  now act according to packet header ID
;
   CASE 1 of 

;
;  Primary memory dump
;  ===================
;
       (packet_header.ID eq '04'xl and mdump_mode): begin
             body = bytarr(286)
             result =  read_data (unit, body)
             loc = (byteswap(long(body,0,1)))(0)
             info = byteswap(long(body,4,70))
             if loc+70 le 131071 then begin
                pmdump_data(loc:loc+69) = info
             endif else begin
                n = 131071 - loc
                pmdump_data(loc:loc+n) = info(0:n)
             endelse

             if ploc_start lt 0 then begin
                ploc_start = loc
                ploc_end = loc + 131071L + 210
             endif 
             now = loc
             if (not pcycle) and (now lt ploc_start) then pcycle = 1 
             now = now + pcycle*131071L
             percent = string(float(now-ploc_start)/float(ploc_end-ploc_start)*$
                       100.0,'%',format='(f6.2,a)')
             widget_control,mdump_widg,set_val=percent          

             if now gt ploc_end then begin
                widget_control,mdump_base,send_event=dump_ev
             endif

       end


;
;  Secondary memory dump
;  =====================
;
       (packet_header.ID eq '05'xl and mdump_mode): begin
             body = bytarr(286)
             result =  read_data (unit, body)
             loc = (byteswap(long(body,0,1)))(0)
             info = byteswap(long(body,4,70))
             if loc+70 le 131071 then begin
                smdump_data(loc:loc+69) = info
             endif else begin
                n = 131071 - loc
                smdump_data(loc:loc+n) = info(0:n)
             endelse

             if sloc_start lt 0 then begin
                sloc_start = loc
                sloc_end = loc + 131071L + 210
             endif 
             now = loc
             if (not scycle) and (now lt sloc_start) then scycle = 1 
             now = now + scycle*131071L
             if now gt sloc_end then begin
                widget_control,mdump_base,send_event=dump_ev
             endif

       end


;
;  VDS raster header
;  =================
       (packet_header.ID EQ '2'xl): $
           begin

              vds_image = 0
              sci_buffer = 0
              last_header = 'VDS'
;
;  in data driven mode?
;
              if ops_mode then begin
;
;  delete GIS displays if present
;
                 if xregistered('header_g') then begin
                    widget_control,header_g_base,/destroy
                    gis_display=0
                 endif
                 if xregistered('gis') then begin
                    widget_control,gis_base,/destroy
                 endif

;
;  create VDS displays
;
                 vds_display = 1
                 header_select = -1
                 if not xregistered('header_v') then begin
                    header_vds_menu
                    xmanager,"header_v",header_v_base
                    vds_menu
                    widget_control,vds_base,map=1
                    xmanager,"vds",vds_base
                 endif

;
;  desensitize the DONE buttons in data driven mode
;

                 widget_control,vds_i_done,sens=0
                 widget_control,vds_h_done,sens=0


              endif   
              widget_control,msg_widg,/append, $
                             set_val=systime()+' Reading VDS raster HEADER'
              result = read_VDS_raster (unit, VDSrasterhdr)
              if (result EQ -1) then begin
                 handle_io_err, widget_base, msg_widg, dummy_label, unit
                 return
              endif else begin 
                 if (result EQ -100) then begin
                    handle_tm_error, widget_base, msg_widg, dummy_label, unit,$
                                     "Too many windows defined in VDS raster"
                    return
                 endif

;
;  have a valid raster header so sort out the size of arrays needed to store
;  the incoming data
;
                 slitnum = VDSrasterhdr.scihead.slitnumber
                 nw = VDSrasterhdr.NumDataWindows
                 nx = max(VDSrasterhdr.VDSWin.lengthX)
                 ny = max(VDSrasterhdr.VDSWin.lengthY)
                 vds_data_str.max_in_mem = fix(MAX_VDS_MEMORY/$
                                               (long(nx)*long(ny)*long(nw)*2L))
                 VDS_date_time = strarr(total_exp)
                 no_exp = total_exp < vds_data_str.max_in_mem
                 if no_exp lt total_exp then begin
                    bell
                    text = string('Only ',no_exp,$
                     ' exposures can be held in memory.',form='(a,i3,a)')
                    widget_control,msg_widg,/append,set_v=text
                    text = 'Memory will cycle the exposures after that.'
                    widget_control,msg_widg,/append,set_v=text
                 endif
;
;  rough check we have sensible values
;
                 if long(nw)*long(nx)*long(ny)*long(no_exp) le 0 then begin
                    flash_msg,msg_widg,'Error in VDS raster header values'
                    print,'Number of windows   = ',nw
                    print,'Window width        = ',nx
                    print,'Window height       = ',ny
                    print,'Number of exposures = ',no_exp
                    
                    have_v_rst_hdr = 0
                    return
                 endif   
                 vds_image = intarr(nx,ny,nw,no_exp)
                 sci_buffer = bytarr(long(nx)*long(ny)*long(nw)*2L+276)
                 VDS_exphdr = replicate(Exphdr,total_exp)
                 WIDGET_CONTROL, msg_widg, /append, $
                             SET_VALUE=systime()+" VDS raster header received"
;
;  ok header so display if wanted
;             
                 if header_select lt 0 then begin
                    disp_vds_hdr, packet_header
                    disp_vds_hdr, VDSrasterhdr
                 endif else begin
                    a = strtrim(string(VDSrasterhdr.Scihead.StudyCounter,f='(i6)'),2)
                    b = strtrim(string(VDSrasterhdr.Scihead.RasterCounter,f='(i6)'),2)
                    widget_control,msg_widg,/append,$
                                   set_val='Study: '+a+', Raster: '+b
                 endelse
;
;  reset data structure and set current operation mode
;  cds_mode = 1 (VDS) 2 (GIS)
;  and flag that have valid raster header
;
                 vds_data_str = {vds_data_rec}
                 vds_data_str.(0) = -1
                 vds_data_str.(2) = -1
                 vds_data_str.(3) = -1
                 vds_data_str.(5) = -1
                 cds_mode = 1
                 have_v_rst_hdr = 1
              endelse
           end 

;
;  GIS raster header
;===================     
       (packet_header.ID EQ '1'xl): $
           begin

              gis_plot = 0
              sci_buffer = 0
              last_header = 'GIS'
;
;  in data driven mode?
;
              if ops_mode then begin
;
;  delete VDS displays if present
;
                 if xregistered('header_v') then begin
                    widget_control,header_v_base,/destroy
                 endif
                 if xregistered('vds') then begin
                    widget_control,vds_base,/destroy
                    vds_display=0
                    plotting_prof = 0
                    prof_area = 0
                    last_window = ' '
                 endif

                 gis_display = 1
                 header_select = 1
                 if not xregistered('header_g') then begin
                    header_gis_menu
                    xmanager,"header_g",header_g_base
                    gis_menu
                    widget_control,gis_base,map=1
                    xmanager,"gis",gis_base
                 endif

;
;  desensitize the DONE buttons in data driven mode
;

                 widget_control,gis_i_done,sens=0
                 widget_control,gis_h_done,sens=0


              endif   
              widget_control,msg_widg,/append,$
                             set_val=systime()+' Reading GIS raster header'
              result = read_GIS_raster (unit, GISrasterhdr)
              if (result EQ -1) then begin
                 handle_io_err, widget_base, msg_widg, dummy_label, unit
                 return
              endif else begin 
                 if (result EQ -100) then begin
                    handle_tm_error, widget_base, msg_widg, dummy_label, unit,$
                                     "Too many windows defined in GIS raster"
                    return
                 endif
                 
                 WIDGET_CONTROL, msg_widg, /append, $
                                 SET_VAL=systime()+" GIS raster header received"
;
;  ok header so display if wanted
;             
                 if header_select gt 0 then begin
                    disp_gis_hdr, packet_header
                    disp_gis_hdr, GISrasterhdr
                 endif
;
;  have a valid raster header so sort out the size of arrays needed to store
;  the incoming data
;
                 slitnum = GISrasterhdr.scihead.slitnumber
                 nw = GISrasterhdr.NumDataWindows
                 nx = max(GISrasterhdr.GISWin.length)
                 no_exp = total_exp 
;
;  rough check we have sensible values
;
                 if long(nw)*long(nx)*long(no_exp) le 0 then begin
                    flash_msg,msg_widg,'Error in GIS raster header values'
                    print,'Number of windows = ',nw
                    print,'Window width = ',nx
                    print,'Number of exposures = ',no_exp
                    have_g_rst_hdr = 0
                    return
                 endif   
                 gis_plot = intarr(nx,nw,no_exp)
                 GIS_date_time = strarr(no_exp)
                 gis_exphdr = replicate(exphdr,no_exp)
;                 sci_buffer = bytarr(long(nx)*long(nw)*long(no_exp)*2L+276)
                 sci_buffer = bytarr(long(nx)*long(nw)*2L+276)


;
;  reset data structure and set operational mode
;    cds_mode = 1 (VDS) 2 (GIS)
;
                 gis_data_str = {gis_data_rec}
                 gis_data_str.(0) = -1
                 gis_data_str.(1) = -1
                 gis_data_str.(2) = -1
                 gis_data_str.(4) =  1
                 gis_data_str.(7) = 'A'
                 gis_data_str.(8)(4) = 1
                 gis_data_str.(9)(4) = 1
                 cds_mode = 2
                 have_g_rst_hdr = 1

              endelse
            end 
 
;
;  Read exposure header for either instrument received
;=====================================================    
        ((packet_header.ID EQ '11'xl) OR $
          (packet_header.ID EQ '12'xl)): $
           begin
;
; Ignore if no raster has been read yet
;
              if (cds_mode EQ 0) then begin
                 widget_control,msg_widg,/append, $
                      set_v='No raster has been read so ignoring exposure hdr.'
                 result = read_data(unit,Any_Packet)
                 return
              endif

;
;  read science data packet counter (2 bytes)
;
              scnt_str = read_sci_cnt (unit, packet_header.ID) 
              sci_counter = scnt_str.count
              last_packet = scnt_str.last
              if (sci_counter EQ -1) then begin
                 widget_control,msg_widg,/append, set_val='Sci counter = -1'
                 handle_io_err, widget_base, msg_widg, dummy_label, unit
                 return
              endif     

;
;  display science counter
;
              if header_select gt 0 then begin
                 disp_gis_hdr, packet_header
                 disp_gis_hdr,scnt_str
              endif
              if header_select lt 0 then begin
                 disp_vds_hdr, packet_header
                 disp_vds_hdr, scnt_str
              endif

;
;  read the rest of the exposure packet (some header info and some data)
;
              widget_control,msg_widg,/append, $
                             set_v=systime()+' Reading Exposure header.'
              result = read_exp(unit, Exphdr, packet_header.ID)
              if (result EQ -1) then begin
                 handle_io_err, widget_base, msg_widg, dummy_label, unit
                 return
              endif    
;
;  display the header info if needed
;
              if header_select gt 0 then begin
                 disp_gis_hdr, packet_header
                 disp_gis_hdr, ExpHdr
              endif
              if header_select lt 0 then begin
                 disp_vds_hdr, packet_header
                 disp_vds_hdr, ExpHdr
              endif

;
;  if monitoring the mechanisms, load the arrays and counters
;
              if m_monitor_mode then begin
                 opsl_count = opsl_count + 1
                 if opsl_count gt 999 then begin
                    opsl_count = 0
                    opsl_monitor(*) = 0
                 endif
                 opsl_monitor(opsl_count) = exphdr.opslcoordinate

                 opsr_count = opsr_count + 1
                 if opsr_count gt 999 then begin
                    opsr_count = 0
                    opsr_monitor(*) = 0
                 endif
                 opsr_monitor(opsr_count) = exphdr.opsrcoordinate

                 slitn_count = slitn_count + 1
                 if slitn_count gt 999 then begin
                    slitn_count = 0
                    slitn_monitor(*) = 0
                 endif
                 slitn_monitor(slitn_count) = slitnum

                 slitp_count = slitp_count + 1
                 if slitp_count gt 999 then begin
                    slitp_count = 0
                    slitp_monitor(*) = 0
                 endif
                 slitp_monitor(slitp_count) = exphdr.slitposition

                 mirrp_count = mirrp_count + 1
                 if mirrp_count gt 999 then begin
                    mirrp_count = 0
                    mirrp_monitor(*) = 0
                 endif
                 mirrp_monitor(mirrp_count) = exphdr.mirrorposition
                 
                 plot_monitor

              endif
     

;
;  load exposure header info according to instrument.  
;
              if (packet_header.ID EQ '12'xl) then begin
                 have_v_exp_hdr = 1
                 vds_data_str.vds_exp = Exphdr.ExposureNumber
                 VDS_exphdr(vds_data_str.vds_exp) = Exphdr
                 if vds_display then Update_ExpInfo, $
                                     vds_exphdr(vds_data_str.vds_exp),$
                                     VDSrasterhdr,  vds_exp_awidg
              endif else begin
                 have_g_exp_hdr = 1
                 gis_data_str.gis_exp = Exphdr.ExposureNumber
                 GIS_exphdr(gis_data_str.gis_exp) = Exphdr
                 if gis_display then $
                          Update_ExpInfo, GIS_exphdr(gis_data_str.gis_exp),$
                                                GISrasterhdr,  gis_exp_awidg
              endelse

;
;  sci_writeptr points to the start location in the science data buffer (1-d) 
;  where the current data are to be stored
;
              sci_writeptr = LONG(0)               

;
;Extract science from packet and put in big buffer store
;
              sci_buffer(0) = Exphdr.Science(*)
                

;
; If it was also the last packet (eg in some drastic compression scheme)
; then decompress, extract and plot VDS window
;
              if (packet_header.ID EQ '12'xl and last_packet) then begin

                status_ok = check_headers('VDS')
                if status_ok then begin
                  dcmp_win = VDSrasterhdr.VDSwin(0:VDSrasterhdr.NumDataWindows-1)
                  scheme    = VDSrasterhdr.Scihead.DataCompScheme 
                  option     = VDSrasterhdr.Scihead.DataCompOption
                  detector    = 0
                  print,'Decompressing VDS data with...'
                  print,'Scheme/Data comp method: ',scheme
                  print,'Option/Data comp parameter: ',option
                  print,'Data orientation (read): ',vds_orientation
                  buff16_sz = n_elements(sci_buffer)/2
                  sci_buff16 = dcmpdata(fix(sci_buffer,0,buff16_sz),dcmp_win,$
                                        detector, scheme,option,vds_orientation)
                  VDS_Ext_image, sci_buff16, VDSrasterhdr, $
                                                 vds_data_str.vds_exp, msg_widg
                  if vds_display then begin
                     vds_data_str.exp_in_a = vds_data_str.vds_exp
                     vds_data_str.curr_a_img = $
                      vds_deselect(vdsrasterhdr,vds_data_str.exp_in_a - $
                                                vds_data_str.first_in_mem)
                     Plot_VDS, vds_awidg, vds_data_str.curr_a_img
                  endif

                endif else begin
                  widget_control,msg_widg,/append,$
                         set_val='No science - no raster/exp header'
                endelse
;
;  if this was the last exposure then clear raster header flag 
;
                if vds_data_str.vds_exp eq (total_exp-1) then begin
                   have_v_rst_hdr = 0
                   cds_mode = 0
                endif
                have_v_exp_hdr = 0
;
;  intensity monitor mode?
;               
                if v_monitor_mode then begin
                   vm_count = vm_count + 1
                   if vm_count gt 999 then begin
                      vm_count = 0
                      vds_monitor(*) = 0.0
                   endif
                   vds_monitor(vm_count) = total(byteswap(sci_buff16)) > 0
                   plot_monitor
                endif
              endif 
             
;
; If last science packet then decompress, extract and plot GIS window
;
              if (packet_header.ID EQ '11'xl and last_packet) then begin
                status_ok = check_headers('GIS')
                if status_ok then begin
;
;  need to force GIS windows into 2-d form same as VDS
;
                  dcmp_win = replicate({StartX:0,StartY:0,LengthX:0,LengthY:0},$
                                       GISrasterhdr.NumDataWindows)
                  for i=0,GISrasterhdr.NumDataWindows-1 do begin
                     dcmp_win(i).StartX  = GISrasterhdr.GISwin(i).Start
                     dcmp_win(i).StartY  = 0
                     dcmp_win(i).LengthX = GISrasterhdr.GISwin(i).Length
                     dcmp_win(i).LengthY = 1
                  endfor
                  scheme = GISrasterhdr.Scihead.DataCompScheme 
                  option = GISrasterhdr.Scihead.DataCompOption
                  vds_orientation = 0
                  detector = 1
                  buff16_sz = n_elements(sci_buffer)/2
                  sci_buff16 = dcmpdata(fix(sci_buffer,0,buff16_sz),$
                               dcmp_win,detector,scheme,option,vds_orientation)
       
                  GIS_Ext_plot, sci_buff16, GISrasterhdr, gis_plot, $
                                    gis_data_str.gis_exp, msg_widg       
                  if gis_display then begin
                     gis_data_str.exp_in_a = gis_data_str.gis_exp
                     gis_data_str.curr_a_data = $
                             gis_deselect(gisrasterhdr,gis_plot,$
                                          gis_data_str.gis_exp, msg_widg)
                     Plot_GIS, gis_data_str.curr_a_data
                  endif

                endif else begin
                  widget_control,msg_widg,/append,$
                    set_val='No science data - no raster/exp header.'
                endelse
;
;  if this the last exposure then clear raster header flag 
;
                if gis_data_str.gis_exp eq $
                      (total_exp-1) then begin
                   have_g_rst_hdr = 0
                   cds_mode = 0
                endif
                have_g_exp_hdr = 0

;
;  monitor mode?
;               
                if g_monitor_mode then begin
                   gm_count = gm_count + 1
                   if gm_count gt 999 then begin
                      gm_count = 0
                      gis_monitor(*) = 0.0
                   endif
                   gis_monitor(gm_count) = $
                        total(unsign(byteswap(sci_buff16)))
                   plot_monitor
                endif
              endif
           end

;
;  pure science data from either instrument
;==========================================
        ((packet_header.ID EQ '21'xl) OR $
          (packet_header.ID EQ '22'xl)): $
            begin
;
;  read the packet counter (for this exposure record)  - its value will be
;  displayed in the header information widget
;
              scnt_str = read_sci_cnt (unit, packet_header.ID) 
              sci_counter = scnt_str.count
              last_packet = scnt_str.last
;
;  display science counter
;
              if header_select gt 0 then begin
                 disp_gis_hdr, packet_header
                 disp_gis_hdr,scnt_str
              endif
              if header_select lt 0 then begin
                 disp_vds_hdr, packet_header
                 disp_vds_hdr, scnt_str
              endif

;
;  Check for missing ie non-consecutive science packets.
;
              if last_packet then begin
                 last_sci_cnt = 0
              endif else begin
                 if (sci_counter - last_sci_cnt) ne 1 then begin
                    widget_control,msg_widg,$
                       set_val=systime()+' ** Missed science packet **',/append
                    widget_control,msg_widg,/append,$
                       set_val=string('Last packet counter was: ',$
                                       last_sci_cnt,format="(a,i4)") 
                    print,'Current/last counters =  ',sci_counter, last_sci_cnt
                 endif
                 last_sci_cnt = sci_counter
              endelse

;
;  read the science data.  The structure returned is of type SciPacketDef
;  which contains 6 bytes of counters, 276 bytes of data and 2 bytes of 
;  checksum - see sci_ana.pro
;
              result = read_data(unit, Sci_Packet)
              if (result EQ -1) then begin
                 handle_io_err, widget_base, msg_widg, dummy_label, unit
                 return
              endif     
;
; Extract science - 252 bytes are already there from the exposure packet ie
;                   locations (0-251) so for the first science packet start
;                   at location 252 and thereafter each science packet is 
;                   adding 276 bytes of data.
;
              sci_writeptr = LONG(252) + LONG(276) * (sci_counter-1)
              if (sci_writeptr ge (n_elements(sci_buffer)-277)) or $
                 (sci_writeptr lt 0) then begin
;                 bell
;                 widget_control,msg_widg,/append,$
;                                set_v='Science counter too large.'
                 return
              endif
              sci_buffer(sci_writeptr) = Sci_Packet.Data(*)

;
; If it was the last packet then decompress, extract and plot VDS window
;
              if (packet_header.ID EQ '22'xl and last_packet) then begin
                status_ok = check_headers('VDS')
                if status_ok then begin
                  dcmp_win = VDSrasterhdr.VDSwin(0:VDSrasterhdr.NumDataWindows-1)
                  scheme    = VDSrasterhdr.Scihead.DataCompScheme 
                  option     = VDSrasterhdr.Scihead.DataCompOption
                  detector    = 0
                  buff16_sz = n_elements(sci_buffer)/2
                  sci_buff16 = dcmpdata(fix(sci_buffer,0,buff16_sz),dcmp_win,$
                                        detector, scheme,option,vds_orientation)
                  VDS_Ext_image, sci_buff16, VDSrasterhdr, $
                                                 vds_data_str.vds_exp, msg_widg
                  if vds_display then begin
                     vds_data_str.exp_in_a = vds_data_str.vds_exp
                     vds_data_str.curr_a_img = $
                                  vds_deselect(vdsrasterhdr,$
                                               vds_data_str.exp_in_a - $
                                               vds_data_str.first_in_mem)
                     Plot_VDS, vds_awidg, vds_data_str.curr_a_img
                  endif

                endif else begin
                  widget_control,msg_widg,/append,$
                         set_val='Raster or exposure header packet not seen.'
                  widget_control,msg_widg,/append,$
                         set_val='No science data - no raster/exp header'
                endelse
;
;  if this the last exposure then clear raster header flag
;
                if vds_data_str.vds_exp eq $
                      (total_exp-1) then begin
                   have_v_rst_hdr = 0
                   cds_mode = 0
                endif
                have_v_exp_hdr = 0

;
;  if we have missed either the raster or exposure header packets then 
;  sci_buff16 will not be defined, but we want to have a monitor count (set to 
;  zero to indicate the omission) so create a dummy sci_buff16 here.
                
                if n_elements(sci_buff16) eq 0 then sci_buff16 = 0

;
;  monitor mode?
;               
                if v_monitor_mode then begin
                   vm_count = vm_count + 1
                   if vm_count gt 999 then begin
                      vm_count = 0
                      vds_monitor(*) = 0.0
                   endif
                   vds_monitor(vm_count) = total(byteswap(sci_buff16)) > 0
                   plot_monitor
                endif
              endif 
             
;
; If last science packet then decompress, extract and plot GIS window
;
              if (packet_header.ID EQ '21'xl and last_packet) then begin
                status_ok = check_headers('GIS')
                if status_ok then begin
;
;  need to force GIS windows into 2-d form same as VDS
;
                  dcmp_win = replicate({StartX:0,StartY:0,LengthX:0,LengthY:0},$
                                       GISrasterhdr.NumDataWindows)
                  for i=0,GISrasterhdr.NumDataWindows-1 do begin
                     dcmp_win(i).StartX  = GISrasterhdr.GISwin(i).Start
                     dcmp_win(i).StartY  = 0
                     dcmp_win(i).LengthX = GISrasterhdr.GISwin(i).Length
                     dcmp_win(i).LengthY = 1
                  endfor
                  scheme = GISrasterhdr.Scihead.DataCompScheme 
                  option = GISrasterhdr.Scihead.DataCompOption
                  vds_orientation = 0
                  detector = 1
                  buff16_sz = n_elements(sci_buffer)/2
                  sci_buff16 = dcmpdata(fix(sci_buffer,0,buff16_sz),$
                               dcmp_win,detector,scheme,option,vds_orientation)
       
                  GIS_Ext_plot, sci_buff16, GISrasterhdr, gis_plot, $
                                gis_data_str.gis_exp, msg_widg       

                  if gis_display then begin
                     gis_data_str.exp_in_a = gis_data_str.gis_exp
                     gis_data_str.curr_a_data = $
                             gis_deselect(gisrasterhdr,gis_plot,$
                                          gis_data_str.gis_exp, msg_widg)
                     Plot_GIS, gis_data_str.curr_a_data
                  endif

                endif else begin
                  widget_control,msg_widg,/append,$
                         set_val='No science data - no raster/exp header.'
                endelse
;
;  if this the last exposure then clear raster header flag 
;
                if gis_data_str.gis_exp eq $
                     (total_exp-1) then begin
                   have_g_rst_hdr = 0
                   cds_mode = 0
                endif
                have_g_exp_hdr = 0

;
;  if we have missed either the raster or exposure header packets then 
;  sci_buff16 will not be defined, but we want to have a monitor count (set to 
;  zero to indicate the omission) so create a dummy sci_buff16 here.
                
                if n_elements(sci_buff16) eq 0 then sci_buff16 = 0

;
;  monitor mode?
;               
                if g_monitor_mode then begin
                   gm_count = gm_count + 1
                   if gm_count gt 999 then begin
                      gm_count = 0
                      gis_monitor(*) = 0.0
                   endif
                   gis_monitor(gm_count) = $
                        total(unsign(byteswap(sci_buff16))) 
                   plot_monitor
                endif
              endif
            end


;
;  read special packets - all special packets have the packet id '30'xl
;=====================================================================
        (packet_header.ID EQ '30'xl):  begin

;
;  special packets are distinguished by their special packet FORMAT id
;  so read it and the counter (2bytes)
;
           temp = read_sp_id(unit)
           sp_pkt_id  = temp(1)
           sp_pkt_cnt = temp(0)

;
;  Deal with statistics of special packets
;
           index = where(specials eq sp_pkt_id)
           index = index(0)
           if index ge 0 then begin
              review_stats(index+13) = review_stats(index+13) + 1
              new_rev_stat = index+13
           endif

;
;  Always make sure correct display is present for specials
;
           sp_request = sp_pkt_id
           case sp_request of

;
;  switch on  display of contents of special "IMIF (MCU support)" packet
;                                             ===================

             '01'xl: begin
                       sp_imif_menu
                       WIDGET_CONTROL, sp_imif_base, MAP=1
                       XMANAGER, "sp_imif",sp_imif_base
                    end

;
;  switch on  display of contents of special "DEFERRED" packet
;                                             ========

             '02'xl: begin
                      sp_def_menu
                      dcs_warned = 0
                      WIDGET_CONTROL, sp_def_base, MAP=1
                      XMANAGER, "sp_def",sp_def_base
                    end

;
;  switch on  display of contents of special "LINE LIST TABLE" packet
;                                             ===============

             '03'xl: begin
                        gen_title = 'Data Extraction Table Dump'
                        sp_gen_menu 
                        WIDGET_CONTROL, sp_gen_base, MAP=1
                        XMANAGER, "sp_gen",sp_gen_base
                     end


;
;  switch on  display of contents of special "RASTER TABLE" packet
;                                             ============

             '04'xl: begin
                        gen_title = 'Raster Table Dump'
                        sp_gen_menu 
                        WIDGET_CONTROL, sp_gen_base, MAP=1
                        XMANAGER, "sp_gen",sp_gen_base
                     end

;
;  switch on  display of contents of special "SEQUENCE TABLE" packet
;                                             ==============

             '05'xl: begin
                        gen_title = 'Sequence Table Dump'
                        sp_gen_menu 
                        WIDGET_CONTROL, sp_gen_base, MAP=1
                        XMANAGER, "sp_gen",sp_gen_base
                     end


;
;  switch on  display of contents of special "HEALTH" packet
;                                             ======

             '08'xl: begin
                       sp_health_menu
                       WIDGET_CONTROL, sp_health_base, MAP=1
                       XMANAGER, "sp_health",sp_health_base
                    end


;
;  switch on  display of contents of special "IMIF interface module" packet
;                                             ======================

             '09'xl: begin
                       sp_imifim_menu
                       WIDGET_CONTROL, sp_imifim_base, MAP=1
                       XMANAGER, "sp_imifim",sp_imifim_base
                    end


;
;  switch on  display of contents of special "HARDWARE TABLE" packet
;                                             ==============

             '0a'xl: begin
                        gen_title = 'VDS Hardware Table Dump'
                        sp_gen_menu 
                        WIDGET_CONTROL, sp_gen_base, MAP=1
                        XMANAGER, "sp_gen",sp_gen_base
                     end

;
;  switch on  display of contents of special "MODE TABLE" packet
;                                             ==========

             '0b'xl: begin
                        gen_title = 'Mode Table Dump'
                        sp_gen_menu 
                        WIDGET_CONTROL, sp_gen_base, MAP=1
                        XMANAGER, "sp_gen",sp_gen_base
                     end

;
;  switch on  display of contents of special "IEF" packet
;                                             ===
                    
             '0e'xl: begin
                       sp_ief_menu
                       WIDGET_CONTROL, sp_ief_base, MAP=1
                       XMANAGER, "sp_ief",sp_ief_base
                    end
             else:  widget_control,msg_widg,set_val=$
                          'Unknown special packet ID received'
           endcase


;
;  act according to packet format specification if the display window has
;  been requested
;
           case 1 of

;
;  DEFERRED command store, read and display
;
              (sp_pkt_id eq '02'xl) and (sp_request eq '02'xl): begin
                 packet = read_sp_body(unit, checksum)
                 sp_def_cnt = sp_def_cnt + 1
                 disp_deferred, packet, sp_pkt_cnt
              end

;
;   HEALTH module, read and display
;
              (sp_pkt_id eq '08'xl) and (sp_request eq '08'xl): begin
                 packet = read_sp_body(unit, checksum)
                 disp_health, packet, sp_pkt_cnt
              end

;
;   IEF module, read and display
;
              (sp_pkt_id eq '0e'xl) and (sp_request eq '0e'xl): begin
                 packet = read_sp_body(unit, checksum)
                 disp_ief, packet, sp_pkt_cnt
              end

;
;  MCU (IMIF interface module - MCU support) dump, read and display
;
              (sp_pkt_id eq '01'xl) and (sp_request eq '01'xl): begin
                 packet = read_sp_body(unit, checksum)
                 disp_imif, packet, sp_pkt_cnt
              end
;
;  MCU (IMIF interface module) dump, read and display
;
              (sp_pkt_id eq '09'xl) and (sp_request eq '09'xl): begin
                 packet = read_sp_body(unit, checksum)
                 disp_imifim, packet
              end
;
;  ERROR module dump, read and display
;
              (sp_pkt_id eq '07'xl) and (sp_request eq '07'xl): begin
                 packet = read_sp_body(unit, checksum)
                 disp_err, packet
              end
;
;  ANY table dump that is currently requested
;
              sp_pkt_id eq sp_request: begin
                 packet = read_sp_body(unit, checksum)
                 disp_gen_table, packet, sp_pkt_cnt, sp_pkt_id
              end

;
;  not done yet (assume general will do)
;
              else:  begin
                       packet = read_sp_body(unit, checksum)
                     end
           endcase                        ; end of special packet format case

        end                               ; end of (packet_header.ID eq '30'xl)
                                          ; ie end of specials

;
;  deal with ENGINEERING A packets
;  ===============================
;
;  since they contain the filament data, sequence ID and the VDS exposure
;  unit, all of which we need.  Note that the offsets
;  given in the database are from the start of the packet.  We need to
;  subtract 20 bytes from those to reference the location within the
;  packet body as read here.
;
        (packet_header.ID eq '0a'xl): begin
                             status = read_data(unit,Any_Packet)
                             if status then begin
                                fil_sel = fix(Any_Packet.data(96) and '0008'xl)
                                filon_off = fix(Any_Packet.data(96) and '0004'xl)
                                fil_curr = fix(Any_Packet.data(141))
;
;  sort so that saved values fil_sel = 1/2 for filament selected. and filon_off
;  = 0/1 for off/on.  Curent is left in raw uncalibrated values.
;
                                if fil_sel eq 0 then begin
                                   fil_sel = 2
                                endif else begin
                                   fil_sel = 1
                                endelse
                                if filon_off gt 0 then filon_off = 1

;
;  Is GIS 5v on? (for use in conjuction with LUT loading parameter - EngB)
;
                                gis_5v_on = fix(Any_Packet.data(96) and '0020'xl)
                     
;
;  collect the Primary SEU detection parameters
;
                                par_err = intarr(4)
                                event_no = unsign(byteswap((fix(Any_Packet.data,0,1))(0)))
                                par_err(0) = fix(Any_Packet.data(3) and '0040'xl)
                                par_err(1) = fix(Any_Packet.data(3) and '0080'xl)
                                par_err(2) = fix(Any_Packet.data(2) and '0001'xl)
                                par_err(3) = fix(Any_Packet.data(2) and '0002'xl)
                           
                                seu_addr = strtrim(string(Any_Packet.data(4:7),form='(4z2.2)'),2)
                                seu_addr = strcompress(seu_addr,/rem)

                                if last_event_no ge 0 then begin
                                   if event_no ne last_event_no then begin
                                      last_event_no = event_no
                                      na = where(seen_address eq seu_addr,count)
                                      if count eq 0 then begin
                                         na = min(where(seen_address eq ''))
                                         seen_address(na) = seu_addr
                                         if total(par_err) gt 0 then begin
                                            nseu = where(par_err ne 0)  
                                            text = ['Primary memory',$
                                                    'New event detected with parity',$
                                                    'error on byte '+fmt_vect(nseu),$
                                                    'Address: '+seu_addr] 
                                            if seu_addr ne '696969' then begin
                                               xack, text, group=main_base,/icon
                                            endif
                                         endif
                                      endif
                                   endif
                                endif else begin
                                   last_event_no = event_no
                                endelse      

;
;  collect the sequence ID and sequence status (0 == running, 161 == waiting)
;  If sid=04  it's the Warm Start table
;  If sid=05  it's the QCM procedure
;
                                temp = byteswap((fix(Any_Packet.data,54,1))(0))
                                sid = unsign(temp)
                                sst = fix(Any_Packet.data(56))
                                if (sst eq 0) or (sst eq 161) then begin
                                   if sid ne last_study_id then begin
                                      last_study_id = sid
                                      status = get_cdhs_series(sid)
                                      if status.id ne -1 then begin
                                         get_study,status.id,status.var,ss
                                         if ss.study_id ne -1 then begin
                                            ss = strtrim(ss.title,2)+$
                                                 ' - '+strtrim(ss.sv_desc,2)
                                         endif else begin
                                            ss = ''
                                         endelse         
                                         sc = fmt_vect([status.id,status.var],$
                                                       del='/',/no_par)
                                         sc = '#'+sc + ' ('+ss+')'
                                         widget_control,study_widg,set_v='  '+sc
                                         study_info = sc
                                      endif else begin
                                         case sid of
                                             4: begin
                                                  widget_control, study_widg,$
                                                  set_v=' Running UPDATE procedure'+$
                                                        ' (mostly mechanisms to HOME)'
                                                end
                                             5: begin
                                                  widget_control, study_widg,$
                                                  set_v=' Running QCM procedure'
                                                end
                                          else: begin
                                                  widget_control, study_widg,$
                                                  set_v=' Unknown study ID: '+$
                                                          strtrim(string(sid),2)
                                                end
                                         endcase
                                      endelse
                                   endif 
                                endif else begin
;
;  sequence isn't active but was it a "Realtime ONLY" Engineering 'study' 
;
;
                                   if sid ne last_study_id then begin
                                      last_study_id = sid
                                      cdhs = get_cdhs_series(sid)
                                      if cdhs.id eq 0 then begin
                                         get_study,cdhs.id,cdhs.var,gs
                                         if gs.study_id ne -1 then begin
                                            if gs.title_id eq 1 then begin
                                               gss = strtrim(gs.title,2)+$
                                                     ' - '+strtrim(gs.sv_desc,2)
                                               xack,'Daily plan requires Realtime '+$
                                                    'engineering now',group=main_base,$
                                                    /icon
                                               sc = fmt_vect([cdhs.id,cdhs.var],$
                                                          del='/',/no_par)
                                               sc = '#'+sc + ' ('+gss+')'
                                               widget_control,study_widg,set_v='  '+sc
                                               study_info = sc
                                            endif else begin
                                               gs = 'Idling...'
                                            endelse 
                                         endif else begin
                                            gs = 'Idling...'
                                         endelse 
                                      endif else begin
                                         widget_control,study_widg,set_v='Idling...'
                                      endelse  
                                   endif else begin
                                      if (gis_lut_load eq 0) and $
                                               (gis_5v_on gt 0) then begin
                                         widget_control,study_widg,$
                                            set_v='Loading GIS Look Up Table'
                                      endif else begin
                                         if strpos(study_info,'Realtime ONLY') eq -1 then begin
                                            widget_control,study_widg,set_v='Idling...'
                                         endif
                                      endelse
                                   endelse
                                endelse

;
;  collect the VDS exposure time value NB vds_tick now defined to be the exposure
;  time, so don't rely on scihead.exposuretime when writing VDS header info in
;  disp_vds_hdr.pro
;
                                temp = byteswap((fix(Any_Packet.data,238,1))(0))
                                temp = fix(unsign(temp) and long('3fff'xl)) ;mask bits 0-13
                                vexpt = fix(temp and '3ff'xl)               ;mask bits 0-9
                                temp = fix('3c00'xl and temp)               ;mask bits 10-13
                                if temp eq 1024 then vds_tick = 1L
                                if temp eq 2048 then vds_tick = 10L
                                if temp eq 4096 then vds_tick = 100L
                                if temp eq 8192 then vds_tick = 1000L
                                vds_tick = float(vds_tick*vexpt)/1000.
                                if xregistered('header_v') then begin
                                   widget_control,header_v_exp(8),$
                                     set_val=string(vds_tick,f='(f7.2)')
                                endif
                             endif else begin
                                flash_msg,msg_widg,$
                                      'Error reading EngA packet.'
                             endelse
                         end


;
;  deal with GIS raw data if requested
;  ===================================
;
;  Note that this now dumps raw data to file regardless of what the user is
;  doing in disp_raw_gis
;
        (packet_header.ID eq '07'xl): begin
                             status = read_data(unit,raw_gis_data)
                             if status then begin
                                if xregistered('raw_gis') then disp_raw_gis
                                dump_raw_gis, packet_header, raw_gis_data
                             endif else begin
                                flash_msg,msg_widg,$
                                      'Error reading Raw GIS data packet.'
                             endelse
                          end

;
;  deal with VDS memory dump
;  =========================
;
        (packet_header.ID eq '06'xl): $
                           begin
                             status = read_data(unit,raw_vds_data)
                             if status then begin
                                if xregistered('raw_vds') then disp_raw_vds
                             endif else begin
                                flash_msg,msg_widg,$
                                      'Error reading Raw VDS data packet.'
                             endelse
                          end

;
;  deal with ENGINEERING B packets
;  ===============================
;
        (packet_header.ID eq '0b'xl): begin
                          status = read_data(unit,Any_Packet)
                          if status then begin
;
;
;  since they contain the slit, mirror and FFB information we require for the 
;  raw GIS data analysis (note slit and mirror positions are uncalibrated)
;  and also the deferred command store contents
;
                             slit_num = (Any_Packet.data(0) and '000e'xl)/2
                             temp = byteswap((fix(Any_Packet.data,0,1))(0))
                             slit_pos = unsign(temp) and long('01ff'xl)
                             mirr_pos = Any_Packet.data(3) and 'ffff'xl
                             gis_ffb = intarr(4)
                             gis_ffb(0)  = Any_Packet.data(117) and 'ffff'xl
                             gis_ffb(1)  = Any_Packet.data(116) and 'ffff'xl
                             gis_ffb(2)  = Any_Packet.data(119) and 'ffff'xl
                             gis_ffb(3)  = Any_Packet.data(118) and 'ffff'xl

                             gis_lut_load  = Any_Packet.data(193) and '0008'xl



;
;  collect the Secondary SEU detection parameters
;
                             par_err = intarr(4)
                             sevent_no = unsign(byteswap((fix(Any_Packet.data,278,1))(0)))
                             par_err(0) = fix(Any_Packet.data(265) and '0040'xl)
                             par_err(1) = fix(Any_Packet.data(265) and '0080'xl)
                             par_err(2) = fix(Any_Packet.data(264) and '0001'xl)
                             par_err(3) = fix(Any_Packet.data(264) and '0002'xl)
                        
                             seu_addr = strtrim(string(Any_Packet.data(266:269),form='(4z2.2)'),2)

                             if last_sevent_no ge 0 then begin
                                if sevent_no ne last_sevent_no then begin
                                   last_sevent_no = sevent_no
                                   na = where(sseen_address eq seu_addr,count)
                                   if count eq 0 then begin
                                      na = min(where(sseen_address eq ''))
                                      sseen_address(na) = seu_addr
                                      if total(par_err) gt 0 then begin
                                         nseu = where(par_err ne 0)  
                                         text = ['Secondary memory',$
                                                 'New event detected with parity',$
                                                 'error on byte '+fmt_vect(nseu),$
                                                 'Address: '+seu_addr] 
                                         xack, text, group=main_base,/icon
                                      endif
                                   endif
                                endif
                             endif else begin
                                last_sevent_no = sevent_no
                             endelse      

;
;  then deal with DCS contents display if needed
;  extract the BK4NIS parameter
;                            
                             if monitor_dcs then begin
                                oldbk = bk4nis
                                bk4nis = $
                                  unsign(byteswap((fix(Any_Packet.data,248,1))(0)))
                                if xregistered('dcs',/noshow) and $
                                              (oldbk ne bk4nis) then begin
                                   show_dcs,/quiet,/keep
                                   dcsfile = 'current_dcs'
                                   vms=(!version.os eq 'VMS')
                                   if vms then home='sys$login' else home=getenv('HOME')
                                   dcsfile=concat_dir(home,dcsfile)
   
                                   dcs_pointer = 0
                                   mess = rd_ascii(dcsfile)
                                   mess = strpad(mess,65,/after)
                                   np = where(strpos(mess,')') ge 0)
                                   if np(0) ge 0 then begin
                                      n = n_elements(np)
                                      if n eq bk4nis then begin
                                         mess(np(0)) = mess(np(0)) + '<-- NEXT'
                                      endif else begin
                                         k = 0
                                         for i=0,n_elements(mess)-1 do begin
                                            if (strpos(mess(i),')') ge 0) then begin
                                               k = k + 1
                                               if ((n-k) eq bk4nis) and (bk4nis ne 0) then begin
                                                  mess(i) = mess(i) + '<-- CURRENT'
                                                  dcs_pointer = i
                                               endif
                                            endif
                                         endfor
                                      endelse
                                         p = (0 > (dcs_pointer-5)) < (n_elements(mess)-15)
                                         if bk4nis eq 0 then begin
                                         mess = [mess,'  ',$
                                                   '*** Final command is either in '+$
                                                   'progress or has finished ***']
                                         p = n_elements(mess) - 15
   
                                      endif
                                      mess = ['Loaded contents of DCS were: ','  ',mess,'   ',' ======  END ======']
                                      widget_control, dcs_list_id, set_value=mess
                                      widget_control, dcs_list_id, set_list_top = p 
                                   endif else begin
                                      widget_control, dcs_list_id, $
                                            set_value='Dump file was empty'
                                   endelse
                                endif 
                             endif
;
;  always get the Deferred Command Store status
;
                             dcs_status = $
                             unsign(byteswap((fix(Any_Packet.data,246,1))(0)))
                             if not dcs_warned then stm_warn_dcs
                          endif else begin
                             flash_msg,msg_widg,$
                                   'Error reading EngB packet.'
                             handle_io_err, widget_base, msg_widg, dummy_label, unit
                          endelse
                       end 

       else:begin
;
; Read other type of packets in which we have no vested interest 
; ==============================================================
             if result eq 1 then begin
                result =  read_data (unit, Any_Packet)
                if (result EQ -1) then begin
                   handle_io_err, widget_base, msg_widg, dummy_label, unit
                   return
                endif
             endif
           end
       endcase     ; packet.ID
;
;  all dealt with so check if want statistics display
;
      if reviewing and (new_rev_stat ge 0) then begin
         disp_review
         new_rev_stat = -1
      endif

end



