;+
; Project     :	SOHO - CDS
;
; Name        :	MAGTAPE
;
; Purpose     :	Driver program for SFDU/FITS magnetic tape I/O.
;
; Explanation :	Driver program to read and write magnetic tapes in FITS
;		format using the Standard Formatted Data Unit protocol (the
;		default).  This procedure has been designed to work on any 
;		operating system (currently only Unix and VMS are supported).
;		This procedure will work for any IDL version later than 2.0.
;		
;		************************************************************
;		The user is encouraged to use XWINTAPE to drive this program.
;		XWINTAPE is an IDL/WIDGET interface to this procedure.
;		************************************************************
;
;		The SOHO/SOWG has selected two standards relating the storage
;		of scientific data: the Standard Formatted Data Unit (SFDU)
;		and the Flexible Image Transport System (FITS).  FITS is a
;		standard for formatting data into files in a computer-
;		independent and self-describing fashion, whereas SFDU is a 
;		standard for documenting the contents and format of a data
;		file, and for outlining the interdependencies of collections
;		of files.
;
;		SFDU works by associating a 20 byte label with each file which 
;		serves as a reference to documentation about the file format.
;		This label can be prepended to the file, in which case it is
;		known as an "attached" SFDU label.  Otherwise, it can be 
;		contained in a separate file on the archive medium, with a 
;		reference to the file it applies to, which is a "detached"
;		label.  In addition, each file can be associated with one or 
;		more additional files which describes its contents.
;
;		The FITS standard not only specifies how data is organized
;		within a FITS file, but also how a FITS file is stored on disk
;		or tape.  As such, it is incompatible with attached SFDU 
;		labels, so that detached labels are recommended and is the 
;		format used here (see CDS Software Note #28).
;
; Use         :	MAGTAPE, UNIT, TYPE, SFDU, [, BLFAC, FILES, XWSTR]
;
; Inputs      :	UNIT:	Tape unit number.  Must be set by the following
;			commands prior to entering IDL:
;			VMS:	ALLOCATE MUA0: (assuming MUA0: is a tape drive)
;				MOUNT/FOR MUA0:
;				DEFINE MT1 MUA0: (in this example, UNIT = 1,
;						  possible values: MT0 ... MT9,
;						  relate to UNIT = 0 ... 9)
;			Unix:	setenv MT0 /dev/nrmt0  (assuming /dev/nrmt0
;							is a tape drive, in 
;							this example, UNIT=0)
;
;		TYPE:	'write' - Write FITS files to the tape.
;			'read'  - Retrieve FITS files from the tape.
;
; Opt. Inputs :	BLFAC:	Blocking factor (1-10) = # of 2880 byte records per 
;			block [DEFAULT = 1].
;
;		FILES:	Can contain either:
;			(1) For TYPE = 'read':
;				A vector of tape file numbers [INT] (or names 
;				[STR]) to be read off of the tape.  
;
;				   (a)	If tape file numbers [INT] are given, 
;					then the disk file names are 
;					constructed as described under the 
;					keyword "NAME_TYPE".
;					***************************************
;					NOTE: The first file may contain the 
;					SFDU Header File.  If so, consider its
;					Tape File Number to be "0", so that the
;					first FITS file is Tape File #1 (i.e.,
;					input your tape file location integers
;					as though this header file didn't 
;					exist -> FITS File #1 = Tape File #1).
;					If NO SFDU header file exists on the
;					tape, the first file (now a FITS file)
;					is STILL Tape File #1.
;					***************************************
;
;				   (b)	If tape file names [STR] are given,
;					then the names supplied are compared
;					with the names in the SFDU header file
;					Form: SFDU_LG for Unix, VMS, and MacOS.
;					Form: SFDU_SM for DOS.
;
;				    (c)	If not supplied, the program will read 
;					the entire tape and place the contents 
;					in the directory associated with DISK
;					using disk names as described by the
;					keyword NAME_TYPE.
;
;			(2) For TYPE = 'write': 
;				A string vector that contains all of the file
;				names from disk that will be written to tape.
;				If not given, the program writes all files in 
;				the directory assigned to DISK and named 
;				'*.fits' to the tape.
;
;		XWSTR:	A string array that contains informational text that
;			is either displayed to the screen or to the FILENAME 
;			widget (set to XWIDGET internally) if XWINTAPE drives 
;			this routine.
;
; Outputs     :	SFDU:	String array that contains the SFDU header created for
;			or read from the tape.  Set to 'NONE' if the /NOSFDU
;			keyword has been set.
;
; Opt. Outputs:	XWSTR:	A string array that contains informational text that
;			is either displayed to the screen or to the FILENAME 
;			widget (set to XWIDGET internally) if XWINTAPE drives 
;			this routine.  Note that MAGTAPE adds text to this 
;			array.
;
; Keywords    :	NOSFDU:	Do not use the Standard Formatted Data Unit labels.
;				MAGTAPE, 1, 'write', /NOSFDU
;			The default is to use them!
;
;		DISK:	Name of the directory where the files are to be read
;			or written.
;				MAGTAPE, 1, 'write', DISK='/cds/data'
;
;		NAME_TYPE:  Naming convention for TYPE 'read' only.  Four
;			possibilities exist:
;			'kword:XXX': Use information from the specified FITS
;				   keyword XXX, (i.e., if NAME_TYPE = 
;				   'kword:filename', then whatever name that is
;				   associated with the FILENAME keyword in
;				   the FITS header is used).  If this keyword
;				   is used, the input filenames in FILES (if
;				   supplied) must contain integers of the file
;				   location on the tape.
;			'sfdu_sm': Use the "short name" from the SFDU header
;				   file [DEFAULT for reads in DOS].
;			'sfdu_lg': Use the "long name" from the SFDU header
;				   file [DEFAULT for reads in Unix, VMS, and 
;				   MacOS].
;			'tapeloc': Use 'tapennnn.fits', where "nnnn" is the 
;				   tape file number.  Used if this keyword is
;				   not supplied.
;
;		OUTPUT:	File name of an output file that contains a log of all
;			events that occur from this procedure.  If one simply
;			uses "/OUTPUT", then MAGTAPE creates its own unique
;			(based upon the current time) log filename.
;
;		UNIXMT:	Override the IDL tape reading routines and simply use
;			the Unix "mt" and "dd" commands to access the tape
;			(Unix systems only).  Note that this is the default
;			for IDL/Unix versions earlier than 3.1.
;
;		CKFITS:	If set, check the first file to see if the file
;			conforms to the CDS level-1 binary FITS prorocol.  If
;			so, the FITS file (and it is assumed that all other
;			FITS files in this directory) is valid.  The default
;			is to check the first file for validity.
;
;		XWIDGET: If set, the widget program XWINTAPE is driving this
;			procedure and any text that is normally printed to the
;			screen will instead be printed to the widget.  If 
;			used, this keyword passes the value of the FILENAME 
;			(set internally to XWIDGET) widget.  Note that 
;			messages sent to the widget are done in reverse order
;			due to the manner in which WIDGET_TEXT displays
;			messages (note that including the "/APPEND" keyword
;			has no effect on this protocol).
;
;		ERRMSG:	If defined and passed, then any error messages will 
;			be returned to the user in this parameter rather than
;			being handled by the IDL MESSAGE utility.  If no 
;			errors are encountered, then a null string is 
;			returned.  In order to use this feature, the string 
;			ERRMSG must be defined first, e.g.,
;
;				ERRMSG = ''
;				MAGTAPE, 1, 'read', SFDU, ERRMSG=ERRMSG
;				IF ERRMSG(0) NE '' THEN ...
;
; Calls       :	ANYTIM2CAL, CHECKCDSFITS, DATATYPE, FXTAPEREAD, FXTAPEWRITE, 
;		GET_UTC, NUM2LET, SFDU_2FHD, SFDU_BYTES, SFDU_FITSHD, 
;		SFDU_FSIZE, SFDU_MAKE, SFDU_OS, SFDU_READ, SFDU_WRITE, UNIXDD
;
; Common      :	SFDU_BYTES, SFDU_FITSHD
;
; Restrictions:	FITS files can only be read from (or written to) a single
;		subdirectory.
;
; Side effects:	None.
;
; Category    :	Data Handling, I/O, FITS, SFDU, Generic
;
; Prev. Hist. :	Basic FITS components fashioned after the VMS FITS procedures
;		written by Don Lindler and William Thompson.  For Unix 
;		environments, uses the IDL/Unix tapes procedures written by 
;		William Thompson for IDL Versions 3.1 and later.   For IDL 
;		Versions earlier than 3.1, this routine uses the Unix command 
;		"dd". 
;
; Written     :	Donald G. Luttermoser, GSFC/ARC, 05 April 1995
;
; Modified    :	Version 1, Donald G. Luttermoser, GSFC/ARC, 05 April 1995
;			Initial program.
;		Version 2, Donald G. Luttermoser, GSFC/ARC, 19 April 1995
;			Modified the SFDU I/O.
;		Version 3, Donald G. Luttermoser, GSFC/ARC, 16 May 1995
;			Added call to SFDU_BYTES, SFDU_FITSHD and added 
;			XWIDGET keyword.
;		Version 4, Donald G. Luttermoser, GSFC/ARC, 26 May 1995
;			Modified the document header and the length of the
;			"long" version of the filenames.
;		Version 5, Donald G. Luttermoser, GSFC/ARC, 29 November 1995
;			Fixed the file "short-name" generator so that identical
;			names are not created on the same volume (e.g., tape).
;		Version 6, Donald G. Luttermoser, GSFC/ARC, 13 December 1995
;			Fixed the output text such that the SFDU header file
;			is now called tape file #1 (instead of #0) and the 
;			first FITS file on tape is now called tape file #2
;			(instead of #1).  Added the CKFITS keyword.
;
; Version     :	Version 6,  13 December 1995.
;
;-
;
PRO MAGTAPE, UNIT, TYPE, SFDU, BLFAC, FILES, XWSTR, NOSFDU=NOSFDU, DISK=DISK, $
	NAME_TYPE=NAME_TYPE, OUTPUT=OUTPUT, UNIXMT=UNIXMT, CKFITS=CKFITS, $
	XWIDGET=XWIDGET, ERRMSG=ERRMSG
;
COMMON SFDU_BYTES, NSFDU_BYTE, NREC_PER_LINE, NTAPE_BYTE, NLINE_BYTE
COMMON SFDU_FITSHD, FITSHD, PVLHEAD
;
; Internal parameter definitions:
;
;	INFILES	= String array containing the inputted file names to be read
;			from or written to tape.
;	INLIST	= Integer array that contains the tape file positions of the
;			FITS files you wish to read off of the tape.
;	NFILE	= Number of files inputted in FILES.
;	NTAPE	= Number of files on the tape.
;	NMTYPE	= Internal version of NAME_TYPE.
;	XWSTR	= String array of file access (or error) messages that either
;			appear in a widget (if XWINTAPE called MAGTAPE) or
;			or on the screen.  Can be passed by XWINTAPE.
;
ON_ERROR, 2   ; Return to caller if error is encountered.
MESSAGE = ''  ; Set to non-null string if error is encountered.
EXPOINT = '!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!'
;
; Set the blocking factor of the FITS files if not passed.
;
IF N_PARAMS() LT 4 THEN BLFAC = 1
;
; Check input parameters.
;
IF N_PARAMS() LT 3 THEN BEGIN
	MESSAGE = 'Syntax:  MAGTAPE, UNIT, TYPE, SFDU, [, BLFAC, FILES, XWSTR]'
ENDIF ELSE BEGIN
	IF DATATYPE(UNIT,1) NE 'Integer' THEN MESSAGE = $
		'UNIT must be an integer in the range: 0 - 9.'
	IF MESSAGE NE '' THEN GOTO, HANDLE_ERROR
;
	IF (UNIT LT 0) OR (UNIT GT 9) THEN MESSAGE = $
	  'UNIT must be an integer in the range: 0 - 9.'
;
	IF DATATYPE(TYPE,1) NE 'String' THEN MESSAGE = $
	  "TYPE must be a string: 'read' or 'write'."
	IF MESSAGE NE '' THEN GOTO, HANDLE_ERROR
;
	IF (STRUPCASE(TYPE) NE 'READ') AND (STRUPCASE(TYPE) NE 'WRITE') THEN $
	  MESSAGE = "TYPE must be a string: 'read' or 'write'."
	IF MESSAGE NE '' THEN GOTO, HANDLE_ERROR
;
	IF DATATYPE(BLFAC,1) NE 'Integer' THEN MESSAGE = $
	  'BLFAC must be an integer (1-10) -- FITS blocking factor.'
ENDELSE
IF MESSAGE NE '' THEN GOTO, HANDLE_ERROR
;
; Set various parameter byte sizes.
;
SFDU_BYTES, BLFAC, ERRMSG=ERRMSG
;
; See if the SFDU has been sent (i.e., if XWINTAPE is driving this routine).
; If so, there's no need to reread the header.
;
IF N_ELEMENTS(SFDU) EQ 0 THEN BEGIN
	SFDU = STRARR(1)
	SFDU(*) = 'NONE'	;  This gets returned if /NOSFDU is set.
ENDIF
;
; Check operating system to see which is used.
;
SFDU_OS, OS_USED, DIR_COM, DIR_DELIM, SUFFIX, QVERS, TYPERW, ERRMSG=ERRMSG
IF N_ELEMENTS(ERRMSG) GT 0 THEN IF ERRMSG(0) NE '' THEN RETURN
;
; Check to see if the XWINTAPE widget is running this program.
;
IF KEYWORD_SET(XWIDGET) THEN QWIDGET = 1 ELSE QWIDGET = 0
IF N_ELEMENTS(XWSTR) EQ 0 THEN XWSTR = ' ' ELSE XWSTR = [' ', XWSTR]
;
; Check to see if the operator wants to override the value in TYPERW.
;
IF KEYWORD_SET(UNIXMT) THEN BEGIN
	IF OS_USED NE 'UNIX' THEN BEGIN
		IF QWIDGET EQ 1 THEN WIDGET_CONTROL, XWIDGET, SET_VALUE=[' ',$
			'UNIXMT keyword ignored -- your current operating '+$
			'system is not UNIX!'] $
		ELSE PRINT, 'UNIXMT keyword ignored -- your current '+$
			'operating system is not UNIX!'
	ENDIF ELSE TYPERW = 'UNIXDD'
ENDIF
;
; Check to see if operator wants to check to see if the FITS files conforms to
; the CDS level-1 binary FITS file protocol by checking the first file in a 
; directory.  This is only done for type "write".  Default is to do this.
;
QCKFITS = 1
IF KEYWORD_SET(CKFITS) THEN BEGIN
	IF CKFITS EQ 0 THEN QCKFITS = 0
ENDIF
;
; Input file names are either stored internally in INFILES (if string) or in
; INLIST (if integer).
;
INFILES = 'None'  &  INLIST = -1  &  NFILE = 0
IF N_ELEMENTS(FILES) GT 0 THEN BEGIN
	IF DATATYPE(FILES,1) EQ 'String' THEN BEGIN
		IF FILES(0) NE '' THEN BEGIN
; Type  WRITE:  Input filenames.
			INFILES = FILES
			NFILE = N_ELEMENTS(INFILES)
		ENDIF
	ENDIF ELSE BEGIN
; Type  READ:  Input tape location numbers.
		INLIST = FILES
		NFILE = N_ELEMENTS(INLIST)
	ENDELSE
ENDIF
;
; Check to see if Standard Formatted Data Unit labels are to be used (this is
; the default).
;
IF KEYWORD_SET(NOSFDU) THEN QSFDU=0 ELSE QSFDU=1
;
; Where are (will) the FITS (be) located?
;
IF NOT KEYWORD_SET(DISK) THEN CD,CURRENT=DISK
;
; Get tape drive name.
;
TAPENM = GETENV('MT'+STRING(UNIT,FORMAT='(I1)'))  &  TAPENM = TAPENM(0)
IF TAPENM EQ '' THEN BEGIN
	MESSAGE = 'Exit IDL and set the proper environment (or logical) '+$
		  'variable that names the tape drive.'
	GOTO, HANDLE_ERROR
ENDIF
;
; Make an output file that lists files read/written from/to tape if wanted.
;
IF KEYWORD_SET(OUTPUT) THEN BEGIN
	QOUT = 'Y'
	GET_UTC, UTC				; Get universal time.
	UNIQ_NAME = ANYTIM2CAL(UTC, FORM=8)	; Convert it to string.
	IF OS_USED EQ 'DOS' THEN UNIQ_NAME = STRMID(UNIQ_NAME, 10, 4)
	IF DATATYPE(OUTPUT,1) EQ 'String' THEN OUTNAME = OUTPUT ELSE $
		OUTNAME = 'tape'+UNIQ_NAME+'.log'
	XWSTR = ['The name of the log file for this process is: '+OUTNAME, $
		XWSTR]
	IF QWIDGET EQ 1 THEN WIDGET_CONTROL, XWIDGET, SET_VALUE=XWSTR ELSE $
		PRINT, XWSTR(0)
	OPENW, UWR, OUTNAME, /GET_LUN
	IF QSFDU EQ 1 THEN BEGIN
		IF STRUPCASE(TYPE) EQ 'WRITE' THEN PRINTF,UWR, $
			'SFDU header file will be written to the tape.' ELSE $
			PRINTF, UWR, 'SFDU header file will be read from '+$
				'the tape.'
		PRINTF,UWR, ' '
	ENDIF
	IF STRUPCASE(TYPE) EQ 'WRITE' THEN BEGIN
		PRINTF, UWR, 'This tape was written on ' + SYSTIME()
		PRINTF, UWR, 'FITS files located in ' + DISK + ' will be ' + $
			'written to tape drive ' + TAPENM
	ENDIF ELSE BEGIN
		PRINTF, UWR, 'The following disk files were created from ' + $
		'tape on ' + SYSTIME()
		PRINTF, UWR, 'FITS files will be written to ' + DISK
	ENDELSE
	PRINTF, UWR, ' '
ENDIF ELSE BEGIN
	QOUT = 'N'
	UWR = 0
ENDELSE
;
; Determine naming convention for files
;
IF KEYWORD_SET(NAME_TYPE) THEN BEGIN
	IF STRUPCASE(TYPE) EQ 'WRITE' THEN MESSAGE = $
		'NAME_TYPE keyword not allowed when writing a tape.'
	CASE STRMID(STRUPCASE(NAME_TYPE),0,6) OF
		'KWORD:':  BEGIN
			KLEN = STRLEN(NAME_TYPE)
			KEYWORD = STRUPCASE(STRMID(NAME_TYPE,6,KLEN-6))
			NMTYPE = 'KEYWORD'
			END
		'SFDU_S':  NMTYPE = 'SFDU_SHORT'
		'SFDU_L':  NMTYPE = 'SFDU_LONG'
		'TAPELO':  NMTYPE = 'TAPELOC'
		ELSE:  MESSAGE = "Invalid NAME_TYPE.  Use: 'kword:XXX',"+$
			"'sfdu_sm','sfdu_lg', or 'tapeloc'."
	ENDCASE
ENDIF ELSE BEGIN
	IF OS_USED EQ 'DOS' THEN NMTYPE = 'SFDU_SHORT' $
		ELSE NMTYPE = 'SFDU_LONG'
	IF QSFDU EQ 0 THEN NMTYPE = 'TAPELOC'
ENDELSE
IF MESSAGE NE '' THEN GOTO, HANDLE_ERROR
;
; Check for inconsistancy.
;
IF (STRMID(NMTYPE,0,4) EQ 'SFDU') AND (QSFDU EQ 0) THEN BEGIN
	MESSAGE = $
	 '/NOSFDU keyword not compatible with NAME_TYPE='+NAME_TYPE+'.'
	GOTO, HANDLE_ERROR
ENDIF
;
; Get file names out of the directory assigned to DISK if FILES not supplied.
; Create the "short name" and "long name" for the file names for the SFDU 
; header.
;
IF STRUPCASE(TYPE) EQ 'WRITE' THEN BEGIN
	IF STRUPCASE(INFILES(0)) EQ 'NONE' THEN BEGIN
		INFILES = FINDFILE(DISK+DIR_DELIM+'*'+SUFFIX, COUNT=NFILE)
		IF NFILE EQ 0 THEN BEGIN
			MESSAGE = 'No files found with "'+SUFFIX+'" in its '+$
			 'name in directory: '+DISK
			GOTO, HANDLE_ERROR
		ENDIF ELSE BEGIN
; Trim the directory name off of the file name.
			IDELIM = STRPOS(INFILES(0), DIR_DELIM)
			IF IDELIM GT -1 THEN BEGIN
				FOR I=0,NFILE-1 DO BEGIN
				  FITLOC = STRPOS(INFILES(I), SUFFIX)
				  NMLEN = STRLEN(INFILES(I))
				  FOR J=FITLOC,0,-1 DO BEGIN
				    IF STRMID(INFILES(I),J,1) EQ DIR_DELIM $
				      THEN GOTO, GOTNAME
				  ENDFOR
GOTNAME:
				  INFILES(I) = STRMID(INFILES(I),J+1,NMLEN-J-1)
				ENDFOR
			ENDIF
		ENDELSE
	ENDIF ELSE NFILE = N_ELEMENTS(INFILES) > N_ELEMENTS(INLIST)
;
; Follow $CCSDS1 specification for short names and $CCSDS2 for long names.
;
	SHORTNM = STRARR(NFILE)  &  LONGNM = SHORTNM
	FOR I=0,NFILE-1 DO BEGIN
		FLEN = STRLEN(INFILES(I))
		DOTPOS = STRPOS(INFILES(I),'.')
		IF DOTPOS EQ -1 THEN DOTPOS = FLEN
; Long names (Unix, VMS, & MacOS), maximum length including suffix is 31 chars
; (26+5).
		LNLEN = DOTPOS < 26  ; Max length of long name preceeding dot.
		LONGNM(I)  = STRMID(INFILES(I),0,LNLEN)
; Short names (DOS), 8 is max length of short name preceeding dot.
		IF DOTPOS LE 8 THEN $
		   SHORTNM(I) = STRMID(INFILES(I),0,DOTPOS) $
		ELSE $
		   SHORTNM(I) = STRMID(INFILES(I),0,4)+$
				STRMID(INFILES(I),DOTPOS-4,4)
	ENDFOR
; Make sure the short names are unique.
	NMCHECK = INTARR(NFILE)		; =-1 (unique), =n (same name as "n")
	NMCHECK(*) = -1
	ICKMAX = -1
	NMSAME = STRARR(NFILE)
	FOR I=0,NFILE-1 DO BEGIN
		ICHECK = WHERE(SHORTNM(I) EQ SHORTNM)
		NCHECK = N_ELEMENTS(ICHECK)
		IF NCHECK EQ 1 THEN BEGIN
			IF ICHECK(0) EQ -1 THEN BEGIN
				MESSAGE = 'Error in checking uniqueness of '+$
					'the "short" filenames.'
				GOTO, HANDLE_ERROR
			ENDIF
		ENDIF ELSE BEGIN
			IF NMCHECK(ICHECK(NCHECK-1)) EQ -1 THEN BEGIN
				NMCHECK(ICHECK) = I
				ICKMAX = MAX([ICKMAX, I])
			ENDIF
		ENDELSE
	ENDFOR
	IF ICKMAX GT -1 THEN BEGIN
		FOR I = 0, ICKMAX DO BEGIN
			ISAME = WHERE(I EQ NMCHECK)
			IF ISAME(0) GT -1 THEN BEGIN
				NSAME = N_ELEMENTS(ISAME)
				IF NSAME GT 26 THEN PRINT, '**** Warning!  '+$
					'Non-uniquenes of "short" filenames!'
				INMSAME = INDGEN(NSAME) + 1
; Create unique IDs:
				NMSAME(ISAME) = NUM2LET(INMSAME)
			ENDIF
		ENDFOR
; Modify the non-unique short filenames.
		FOR I=0,NFILE-1 DO BEGIN
			NMLEN = STRLEN(SHORTNM(I))
			IF NMCHECK(I) NE -1 THEN BEGIN
				IF NMLEN LT 8 THEN BEGIN
					SHORTNM(I) = SHORTNM(I) + NMSAME(I)
				ENDIF ELSE BEGIN
					USPOS = STRPOS(SHORTNM(I), '_')
					IF USPOS EQ -1 THEN USPOS = 7
					TMPNM = SHORTNM(I)
					STRPUT, TMPNM, NMSAME(I), USPOS
					SHORTNM(I) = TMPNM
				ENDELSE
			ENDIF
		ENDFOR
	ENDIF
; Attach suffix.
	LONGNM  = LONGNM  + '.fits'
	SHORTNM = SHORTNM + '.fit'
ENDIF
;
; If TYPE = 'READ', get the SFDU header file (1st file on tape) off of the 
; tape and determine which files are to be read.  If TYPE = 'WRITE', create 
; the SFDU header and write it to tape.
;
NTAPE = -1  ;  Set the number of files on the tape to -1 --> no SFDU file.
IF INLIST(0) NE -1 THEN NTAPE = INLIST(NFILE-1)
CASE STRUPCASE(TYPE) OF
;
; SFDU reading section.
;
	'READ': BEGIN
		IF QWIDGET EQ 1 THEN WIDGET_CONTROL, XWIDGET, SET_VALUE=XWSTR $
			ELSE PRINT, XWSTR
		IF QSFDU EQ 0 THEN BEGIN
			XWSTR = ['Assuming tape does not contain SFDU header '+$
				'file.','Performing standard FITS read.', XWSTR]
			IF QWIDGET EQ 1 THEN WIDGET_CONTROL, XWIDGET, $
				SET_VALUE=XWSTR $
			ELSE BEGIN $
				PRINT, XWSTR(0)
				PRINT, XWSTR(1)
			ENDELSE
		ENDIF ELSE BEGIN
			IF SFDU(0) EQ 'NONE' THEN BEGIN
				XWSTR = ['Reading the SFDU Header file.', $
					XWSTR]
				IF QWIDGET EQ 1 THEN WIDGET_CONTROL, XWIDGET, $
					SET_VALUE=XWSTR ELSE $
					PRINT, XWSTR(0)
			ENDIF
; This variable has the tape location name on output. 
			TPLOCNM = SUFFIX
			IF QWIDGET EQ 1 THEN BEGIN
				SFDU_READ, UNIT, TAPENM, TYPERW, SFDU, $
					SHORTNM, LONGNM, TPLOCNM, XWSTR, $
					XWIDGET=XWIDGET, ERRMSG=ERRMSG
; Note that SFDU_2FHD has already been called in XWINTAPE if QWIDGET=1.
			ENDIF ELSE BEGIN
				SFDU_READ, UNIT, TAPENM, TYPERW, SFDU, $
					SHORTNM, LONGNM, TPLOCNM, ERRMSG=ERRMSG
				SFDU_2FHD, SFDU, ERRMSG=ERRMSG
			ENDELSE
			IF N_ELEMENTS(ERRMSG) GT 0 THEN $
				IF ERRMSG(0) NE '' THEN RETURN
			NTAPE = N_ELEMENTS(LONGNM)
;
			XWSTR = [' ', 'There is 1 SFDU header file and '+$
				STRTRIM(STRING(NTAPE),2)+$
				' FITS files on the tape mounted on drive: '+$
				TAPENM, XWSTR]
			IF QWIDGET EQ 1 THEN WIDGET_CONTROL, XWIDGET, $
				SET_VALUE=XWSTR $
			ELSE BEGIN
				PRINT, XWSTR(0)
				PRINT, XWSTR(1)
			ENDELSE
			IF QOUT EQ 'Y' THEN BEGIN
				PRINTF, UWR, 'There is 1 SFDU header file ', $
			 	 'and ',STRTRIM(STRING(NTAPE),2), $
			 	 ' FITS files on this tape.'
				PRINTF, UWR, ' '
			ENDIF
		ENDELSE
		CASE STRUPCASE(NMTYPE) OF
;
; List of tape file numbers should have been supplied for this option, if not
; all of the files from the tape will be read.
;
			'KEYWORD':  BEGIN
				XWSTR = ['Disk filenames will be derived '+$
					'from the FITS keyword: '+KEYWORD, $
					XWSTR]
				IF QWIDGET EQ 1 THEN WIDGET_CONTROL, XWIDGET, $
					SET_VALUE=XWSTR $
					ELSE PRINT, XWSTR(0)
				IF INLIST(0) EQ -1 THEN INLIST=INDGEN(NTAPE)+1
				END
			'SFDU_SHORT':  BEGIN
				XWSTR = ['SFDU short name will be used for '+$
					'the file names.', XWSTR]
				IF QWIDGET EQ 1 THEN WIDGET_CONTROL, XWIDGET, $
					SET_VALUE=XWSTR $
					ELSE PRINT, XWSTR(0)
				IF INLIST(0) EQ -1 THEN BEGIN
				  IF STRMID(STRUPCASE(INFILES(0)),0,4) NE $
				    'NONE' THEN BEGIN
					INLIST = -1
					FOR I=0,NTAPE-1 DO INLIST = $
					 [INLIST,WHERE(INFILES EQ SHORTNM(I))]
					IUSE = WHERE(INLIST GT -1, NLIST)
					IF NLIST EQ 0 THEN BEGIN
					 MESSAGE = 'Cannot find the inputted'+$
					 ' short file names in the SFDU '+$
					 'header file.'
					 GOTO, HANDLE_ERROR
					ENDIF
					INLIST = INLIST(SORT(INLIST(IUSE)))
				  ENDIF ELSE INLIST = INDGEN(NTAPE) + 1
				ENDIF
				END
			'SFDU_LONG':  BEGIN
				XWSTR = ['SFDU long name will be used for '+$
					'the file names.', XWSTR]
				IF QWIDGET EQ 1 THEN WIDGET_CONTROL, XWIDGET, $
					SET_VALUE=XWSTR $
					ELSE PRINT, XWSTR(0)
				IF INLIST(0) EQ -1 THEN BEGIN
				  IF STRMID(STRUPCASE(INFILES(0)),0,4) NE $
				    'NONE' THEN BEGIN
					INLIST = -1
					FOR I=0,NTAPE-1 DO INLIST = $
					 [INLIST,WHERE(INFILES EQ LONGNM(I))]
					IUSE = WHERE(INLIST GT -1, NLIST)
					IF NLIST EQ 0 THEN BEGIN
					 MESSAGE = 'Cannot find the inputted'+$
					 ' long file names in the SFDU '+$
					 'header file.'
					 GOTO, HANDLE_ERROR
					ENDIF
					INLIST = INLIST(SORT(INLIST(IUSE)))
				  ENDIF ELSE INLIST = INDGEN(NTAPE) + 1
				ENDIF
				END
;
; List of tape file numbers should have been supplied for this option, if not
; all of the files from the tape will be read.
;
			'TAPELOC':  BEGIN
				XWSTR = ['File names that are read from the '+$
					'tape will be labelled:  '+$
					'tapennnn.fit(s).', XWSTR]
				IF QWIDGET EQ 1 THEN WIDGET_CONTROL, XWIDGET, $
					SET_VALUE=XWSTR $
					ELSE PRINT, XWSTR(0)
				IF INLIST(0) EQ -1 THEN INLIST=INDGEN(NTAPE)+1
				END
			ELSE:  MESSAGE = 'Error in determining tape files '+$
				'to be read.'
		ENDCASE
		PRINT, ' '
		IF MESSAGE NE '' THEN GOTO, HANDLE_ERROR
		END
;
; SFDU writing section.
;
	'WRITE': BEGIN
;
; Double check to make sure files were found.
;
		IF STRMID(LONGNM(0),0,4) EQ 'NONE' THEN BEGIN
			MESSAGE = 'No FITS files found in '+DISK+'.'
			GOTO, HANDLE_ERROR
		ENDIF
;
; Check to make sure the FITS files conform to the CDS standard.  Note that 
; only the first file is checked (to save time) and it is assumed that if the
; first file in a directory is valid, that all of the FITS files in that
; directory are valid.
;
		IF QCKFITS EQ 1 THEN BEGIN
			XWSTR = ['Checking to see that the FITS file is '+$
				'valid for the CDS.', XWSTR]
			IF QWIDGET EQ 1 THEN WIDGET_CONTROL, XWIDGET, $
				SET_VALUE=XWSTR ELSE PRINT, XWSTR(0)
			CKFITS_RESULT = CHECKCDSFITS(LONGNM(0), MESSG, $
				DIR=DISK+DIR_DELIM, ERRMSG=MESSAGE)
			IF MESSAGE(0) NE '' THEN GOTO, HANDLE_ERROR
;
			IF STRUPCASE(CKFITS_RESULT) NE 'VALID' THEN BEGIN
				CKF_TEXT = [EXPOINT,$
				    'WARNING!  The FITS files in '+DISK,$
				    'do not conform to the SOHO/CDS standard!',$
				    EXPOINT]
				IF QWIDGET EQ 1 THEN BEGIN
				    BASE0 = WIDGET_BASE(TITLE='Invalid FITS '+$
					'Files', /COLUMN, /FRAME)
				    WH0 = WIDGET_TEXT(BASE0, VALUE=[CKF_TEXT,$
					' ',MESSG,' ','Do you wish to '+$
					'continue?'], $
					XSIZE=MAX(STRLEN(CKF_TEXT))+15, $
					YSIZE=N_ELEMENTS(MESSG)+7)
				    WA0 = WIDGET_BUTTON(BASE0, VALUE="Yes, "+$
					"please continue!", /FRAME)
				    WB0 = WIDGET_BUTTON(BASE0, VALUE=$
					"Abort, do not write the tape!", /FRAME)
				    WIDGET_CONTROL, BASE0, /REALIZE
				    CKF0 = ' '
				    WHILE CKF0 NE 'WIDGET_BUTTON' DO BEGIN
					EVENT0 = WIDGET_EVENT(BASE0)
					CKF0 = TAG_NAMES(EVENT0, /STRUCTURE)
				    ENDWHILE
				    WIDGET_CONTROL, EVENT0.ID, GET_VALUE=VAL0
				    WIDGET_CONTROL, BASE0, /DESTROY
				    IF VAL0 EQ 'Abort, do not write the tape!'$
					THEN BEGIN
					MESSAGE = 'Aborting archive due to '+$
					    'FITS CDS validity error.'
					GOTO, HANDLE_ERROR
				    ENDIF
				ENDIF ELSE BEGIN
				    FOR ICF=0,3 DO PRINT, CKF_TEXT(ICF)
				    PRINT, ' '
				    NMESSG = N_ELEMENTS(MESSG)
				    FOR ICF=0,NMESSG-1 DO PRINT, MESSG(ICF)
				    PRINT, ' '
				    ASK = 'Y'
				    READ, 'Do you wish to continue [<return>]'+$
					' or abort [enter "a"]? ', ASK
				    IF STRUPCASE(STRMID(STRTRIM(ASK,2), 0, 1))$
					EQ 'A' THEN BEGIN
					MESSAGE = 'Aborting archive due to '+$
					    'FITS CDS validity error.'
					GOTO, HANDLE_ERROR
				    ENDIF
				ENDELSE
			ENDIF ELSE BEGIN
				XWSTR = ['  >>> FITS files conforms to CDS ' +$
					'standard.', XWSTR]
				IF QWIDGET EQ 1 THEN WIDGET_CONTROL, XWIDGET, $
					SET_VALUE=XWSTR ELSE PRINT, XWSTR(0)
			ENDELSE
		ENDIF
;
		IF QSFDU EQ 0 THEN BEGIN
			XWSTR = ['NO SFDU header file will be generated.',$
				'Performing standard FITS write.', XWSTR]
			IF QWIDGET EQ 1 THEN WIDGET_CONTROL, XWIDGET, $
				SET_VALUE=XWSTR $
			ELSE BEGIN
				PRINT, XWSTR(0)
				PRINT, XWSTR(1)
			ENDELSE
		ENDIF ELSE BEGIN
; Tell the public what we're doing.
			XWSTR = ['Generating an SFDU Header file.', XWSTR]
			IF QWIDGET EQ 1 THEN WIDGET_CONTROL, XWIDGET, $
				SET_VALUE=XWSTR $
				ELSE PRINT, XWSTR(0)
; Get total byte size of the FITS files.
			FILE_SIZE = SFDU_FSIZE(DISK, INFILES, ERRMSG=ERRMSG)
			IF N_ELEMENTS(ERRMSG) GT 0 THEN IF ERRMSG(0) NE '' $
				THEN RETURN
; Get the FITS files headers out of the files.
			IF N_ELEMENTS(FITSHD) EQ 0 THEN SFDU_FITSHD, DISK, $
				INFILES, ERRMSG=ERRMSG
; In case someone runs the program again without leaving IDL.
			IF FITSHD(0) EQ '' THEN SFDU_FITSHD, DISK, $
				INFILES, ERRMSG=ERRMSG
			IF N_ELEMENTS(ERRMSG) GT 0 THEN IF ERRMSG(0) NE '' $
				THEN RETURN
; Make SFDU header file.
			SFDU = SFDU_MAKE( SHORTNM, LONGNM, FILE_SIZE, $
				ERRMSG=ERRMSG)
; If an error occured in SFDU_MAKE, then return.
			IF N_ELEMENTS(ERRMSG) GT 0 THEN IF ERRMSG(0) NE '' $
				THEN RETURN
; Now write it to tape.
			XWSTR = ['Preparing SFDU Header for the tape.',XWSTR]
			IF QWIDGET EQ 1 THEN BEGIN
				WIDGET_CONTROL, XWIDGET, SET_VALUE=XWSTR
				SFDU_WRITE, UNIT, TAPENM, TYPERW, SFDU, XWSTR, $
					XWIDGET=XWIDGET, ERRMSG=ERRMSG
			ENDIF ELSE BEGIN
				PRINT, XWSTR(0)
				SFDU_WRITE, UNIT, TAPENM, TYPERW, SFDU, $
					ERRMSG=ERRMSG
			ENDELSE
			IF N_ELEMENTS(ERRMSG) GT 0 THEN IF ERRMSG(0) NE '' $
				THEN RETURN
			NTAPE = N_ELEMENTS(LONGNM)
			IF QOUT EQ 'Y' THEN BEGIN
				PRINTF, UWR, 'There are ', $
			 	STRTRIM(STRING(NTAPE),2),' disk FITS files '+$
					'that will be written to this tape.'
				PRINTF, UWR, ' '
			ENDIF
		ENDELSE
		PRINT, ' '
		END
;
; If neither READ or WRITE, something is wrong.
;
	ELSE:   ;  Routine should never get here.
ENDCASE
;
; At this point the tape should be at the start of the 2nd file (unless /NOSFDU
; was set), which should be the 1st FITS file on the tape if 'read' is set.
;
; This next step is necessary for the FXTAPEREAD FITS routines.
;
IF (NTAPE EQ -1) AND (STRUPCASE(TYPE) EQ 'READ') THEN BEGIN
	XWSTR = [' ','***************************************',$
		'Answer the question in the TEXT window!',$
		'***************************************', ' ', XWSTR]
	IF QWIDGET EQ 1 THEN WIDGET_CONTROL, XWIDGET, SET_VALUE=XWSTR
	READ, ' How many FITS files are on this tape? ',NTAPE
	PRINT, ' '
	INLIST = INDGEN(NTAPE) + 1
	TPLOCNM = 'tape' + STRMID(STRING(INLIST+10000,FORMAT='(I5)'),1,4) + $
		SUFFIX
ENDIF
;
; NFILE should now contain the total number of files to be accessed to/from the
; tape.  This next line makes sure this is the case.
;
IF STRUPCASE(TYPE) EQ 'READ' THEN NFILE = N_ELEMENTS(INLIST) $
	ELSE NFILE = N_ELEMENTS(INFILES)
;
; Read or Write the appropriate FITS files.  Change to the appropriate 
; directory first.
;
CD,DISK,CURRENT=OLD_DIR
;
; *** Note: FXTAPEREAD automatically rewinds the tape.  However, I have 
; installed a switch in FXTAPEREAD that will skip the first file if SFDU 
; keyword is set.  (Note: UNIXDD does not rewind the tape before reading 
; the FITS files.)
;
CASE STRUPCASE(TYPE) OF
;
; **********  Tape reading section.  **************
;
	'READ':	 BEGIN
		IF INLIST(0) EQ -1 THEN BEGIN
			MESSAGE = 'Error in determining which files to read '+$
				'off of the tape.'
			GOTO, HANDLE_ERROR
		ENDIF
		IF QWIDGET NE 1 THEN PRINT, $
			'FITS files will be written to directory: '+DISK
		PRINT, ' '
		IF QSFDU EQ 1 THEN BEGIN
			IF TYPERW EQ 'IDLTP' THEN BEGIN
				XWSTR = ['   sfdu.head', 'File:        1', $
					' ', XWSTR]
				IF QWIDGET EQ 1 THEN WIDGET_CONTROL, XWIDGET, $
					SET_VALUE=XWSTR $
				ELSE BEGIN
					PRINT, XWSTR(1)
					PRINT, XWSTR(0)
					PRINT, XWSTR(2)
				ENDELSE
			ENDIF ELSE BEGIN
				XWSTR = ['Tape file #   1 -- SFDU header '+$
					'file.', XWSTR]
				IF QWIDGET EQ 1 THEN WIDGET_CONTROL, XWIDGET, $
					SET_VALUE=XWSTR $
					ELSE PRINT, XWSTR(0)
			ENDELSE
			IF QOUT EQ 'Y' THEN BEGIN
				PRINTF, UWR, 'Tape file #   1 is the SFDU '+ $
					'header file.'
			ENDIF
		ENDIF
;
		CASE NMTYPE OF
;
		   'SFDU_SHORT':  BEGIN
			CASE TYPERW OF
;
; Use the IDL tape accessing commands.
;
			   'IDLTP': BEGIN
				IF QWIDGET EQ 1 THEN $
					FXTAPEREAD, UNIT, INLIST, '', '', $
					SHORTNM(INLIST-1), XWSTR, /NOSUFFIX, $
					SFDU=QSFDU, XWIDGET=XWIDGET, $
					ERRMSG=ERRMSG $
				ELSE $
					FXTAPEREAD, UNIT, INLIST, '', '', $
					SHORTNM(INLIST-1), /NOSUFFIX, $
					SFDU=QSFDU, ERRMSG=ERRMSG
				IF N_ELEMENTS(ERRMSG) GT 0 THEN $
					IF ERRMSG(0) NE '' THEN RETURN
; Write to log file.
				IF QOUT EQ 'Y' THEN BEGIN
					FOR I=0,NFILE-1 DO BEGIN
					  PRINTF, UWR, 'Tape file # ', $
			 		    STRING(FORMAT='(I3)',INLIST(I)+1),$
					    ' being written to disk file ', $
					    SHORTNM(INLIST(I)-1)
					ENDFOR
				ENDIF
				END
;
; Use the Unix tape accessing commands.
;
			   'UNIXDD': BEGIN
				IF QWIDGET EQ 1 THEN $
				   QUNIXDD = UNIXDD( TAPENM, TYPE, OLD_DIR, $
					INLIST, SHORTNM, XWSTR, BLFAC=BLFAC, $
					OUTPUT=UWR, XWIDGET=XWIDGET, $
					ERRMSG=ERRMSG) $
				ELSE $
				   QUNIXDD = UNIXDD( TAPENM, TYPE, OLD_DIR, $
					INLIST, SHORTNM, BLFAC=BLFAC, $
					OUTPUT=UWR, ERRMSG=ERRMSG)
				IF N_ELEMENTS(ERRMSG) GT 0 THEN $
					IF ERRMSG(0) NE '' THEN RETURN
				END
;
; This next case should never happen.
;
			   ELSE:  MESSAGE = 'Unable to determine appropriate'+$
					' tape read type for SHORTNM.'
			ENDCASE
			END
;
		   'SFDU_LONG':  BEGIN
			CASE TYPERW OF
;
; Use the IDL tape accessing commands.
;
			   'IDLTP': BEGIN
				IF QWIDGET EQ 1 THEN $
					FXTAPEREAD, UNIT, INLIST, '', '', $
					LONGNM(INLIST-1), XWSTR, /NOSUFFIX, $
					SFDU=QSFDU, XWIDGET=XWIDGET, $
					ERRMSG=ERRMSG $
				ELSE $
					FXTAPEREAD, UNIT, INLIST, '', '', $
					LONGNM(INLIST-1), /NOSUFFIX, $
					SFDU=QSFDU, ERRMSG=ERRMSG
				IF N_ELEMENTS(ERRMSG) GT 0 THEN $
					IF ERRMSG(0) NE '' THEN RETURN
; Write to log file.
				IF QOUT EQ 'Y' THEN BEGIN
					FOR I=0,NFILE-1 DO BEGIN
					  PRINTF, UWR, 'Tape file # ', $
			 		    STRING(FORMAT='(I3)',INLIST(I)+1),$
					    ' being written to disk file ',$
					    LONGNM(INLIST(I)-1)
					ENDFOR
				ENDIF
				END
;
; Use the Unix tape accessing commands.
;
			   'UNIXDD': BEGIN
				IF QWIDGET EQ 1 THEN $
				   QUNIXDD = UNIXDD( TAPENM, TYPE, OLD_DIR, $
					INLIST, LONGNM, XWSTR, BLFAC=BLFAC, $
					OUTPUT=UWR, XWIDGET=XWIDGET, $
					ERRMSG=ERRMSG) $
				ELSE $
				   QUNIXDD = UNIXDD( TAPENM, TYPE, OLD_DIR, $
					INLIST, LONGNM, BLFAC=BLFAC, $
					OUTPUT=UWR, ERRMSG=ERRMSG)
				IF N_ELEMENTS(ERRMSG) GT 0 THEN $
					IF ERRMSG(0) NE '' THEN RETURN
				END
;
; This next case should never happen.
;
			   ELSE:  MESSAGE = 'Unable to determine appropriate'+$
					' tape read type for LONGNM.'
			ENDCASE
			END
;
		   'KEYWORD':  BEGIN
			CASE TYPERW OF
;
; Use the IDL tape accessing commands.
;
			   'IDLTP': BEGIN
				IF QWIDGET EQ 1 THEN $
					FXTAPEREAD, UNIT, INLIST, KEYWORD, '', $
					KEYNAMES, XWSTR, SFDU=QSFDU, $
					XWIDGET=XWIDGET, ERRMSG=ERRMSG $
				ELSE $
					FXTAPEREAD, UNIT, INLIST, KEYWORD, '', $
					KEYNAMES, SFDU=QSFDU, ERRMSG=ERRMSG
				IF N_ELEMENTS(ERRMSG) GT 0 THEN $
					IF ERRMSG(0) NE '' THEN RETURN
; Write to log file.
				IF QOUT EQ 'Y' THEN BEGIN
					FOR I=0,NFILE-1 DO BEGIN
					  PRINTF, UWR, 'Tape file # ', $
			 		    STRING(FORMAT='(I3)',INLIST(I)+1),$
					    ' being written to disk file ',$
					    KEYNAMES(I)+SUFFIX
					ENDFOR
				ENDIF
				END
;
; Use the Unix tape accessing commands.
;
			   'UNIXDD': BEGIN
; YET TO BE INSTALLED!!!
				MESSAGE = 'Keyword read has not been set '+$
					'yet for UNIX.'
				END
;
; This next case should never happen.
;
			   ELSE:  MESSAGE = 'Unable to determine appropriate'+$
					' tape read type for KEYWORD.'
			ENDCASE
			END
;
		   'TAPELOC':  BEGIN
			CASE TYPERW OF
;
; Use the IDL tape accessing commands.
;
			   'IDLTP': BEGIN
				IF QWIDGET EQ 1 THEN $
					FXTAPEREAD, UNIT, INLIST, '', '', $
					TPLOCNM(INLIST-1), XWSTR, SFDU=QSFDU, $
					/NOSUFFIX, XWIDGET=XWIDGET, $
					ERRMSG=ERRMSG $
				ELSE $
					FXTAPEREAD, UNIT, INLIST, '', '', $
					TPLOCNM(INLIST-1), SFDU=QSFDU, $
					/NOSUFFIX, ERRMSG=ERRMSG
				IF N_ELEMENTS(ERRMSG) GT 0 THEN $
					IF ERRMSG(0) NE '' THEN RETURN
; Write to log file.
				IF QOUT EQ 'Y' THEN BEGIN
					FOR I=0,NFILE-1 DO BEGIN
					  PRINTF, UWR, 'Tape file # ', $
			 		    STRING(FORMAT='(I3)',INLIST(I)+1),$
					    ' being written to disk file ',$
					    TPLOCNM(INLIST(I)-1)+SUFFIX
					ENDFOR
				ENDIF
				END
;
; Use the Unix tape accessing commands.
;
			   'UNIXDD': BEGIN
				IF QWIDGET EQ 1 THEN $
				   QUNIXDD = UNIXDD( TAPENM, TYPE, OLD_DIR, $
					INLIST, TPLOCNM, XWSTR, BLFAC=BLFAC, $
					OUTPUT=UWR, XWIDGET=XWIDGET, $
					ERRMSG=ERRMSG) $
				ELSE $
				   QUNIXDD = UNIXDD( TAPENM, TYPE, OLD_DIR, $
					INLIST, TPLOCNM, BLFAC=BLFAC, $
					OUTPUT=UWR, ERRMSG=ERRMSG)
				IF N_ELEMENTS(ERRMSG) GT 0 THEN $
					IF ERRMSG(0) NE '' THEN RETURN
				END
;
; This next case should never happen.
;
			   ELSE:  MESSAGE = 'Unable to determine appropriate'+$
					' tape read type for TAPELOC.'
			ENDCASE
			END
		ENDCASE
		END
;
; **********  Tape writing section.  **************
;
	'WRITE': BEGIN
		XWSTR =['The following printed FITS files are being written '+$
			'to tape.', XWSTR]
		IF QWIDGET EQ 1 THEN WIDGET_CONTROL, XWIDGET, SET_VALUE=$
			XWSTR ELSE PRINT, XWSTR(0)
		IF QSFDU EQ 1 THEN BEGIN
			XWSTR = ['SFDU header file has been written to tape'+$
				' file #   1.', XWSTR]
			IF QWIDGET EQ 1 THEN WIDGET_CONTROL, XWIDGET, $
				SET_VALUE=XWSTR ELSE PRINT, $
				XWSTR(0)
			IF QOUT EQ 'Y' THEN BEGIN
				PRINTF, UWR, 'File (#   1) is the SFDU '+ $
					'header file and is being written '+$
					'to tape.'
			ENDIF
		ENDIF
		CASE TYPERW OF
;
; Use the IDL tape accessing commands.  Assume that 'FILENAME' already exists
; in the FITS header.
;
		   'IDLTP':  BEGIN
			IF QWIDGET EQ 1 THEN $
				FXTAPEWRITE, UNIT, BLFAC, INFILES, '', XWSTR, $
				XWIDGET=XWIDGET, SFDU=QSFDU, ERRMSG=ERRMSG $
			ELSE $
				FXTAPEWRITE, UNIT, BLFAC, INFILES, '', $
				SFDU=QSFDU, ERRMSG=ERRMSG
			IF N_ELEMENTS(ERRMSG) GT 0 THEN IF ERRMSG(0) NE '' $
				THEN RETURN
; Write to log file.
			IF QOUT EQ 'Y' THEN BEGIN
				FOR I=0,NFILE-1 DO PRINTF, UWR, 'File (# ', $
					STRING(I+2,FORMAT='(I3)') + ') ' + $
					INFILES(I) + ' being written to tape.'
			ENDIF
; Note that an <End-of-Tape> mark (= double <End-of-File> marks) is 
; automatically written to the tape in FXTPIO_WRITE which is called in 
; FXTAPEWRITE.
			END
;
; Use the Unix tape accessing commands.
;
		   'UNIXDD': BEGIN
			IF QWIDGET EQ 1 THEN $
				QUNIXDD = UNIXDD( TAPENM, TYPE, OLD_DIR, $
				INFILES, XWSTR, BLFAC=BLFAC, OUTPUT=UWR, $
				XWIDGET=XWIDGET, ERRMSG=ERRMSG) $
			ELSE $
				QUNIXDD = UNIXDD( TAPENM, TYPE, OLD_DIR, $
				INFILES, BLFAC=BLFAC, OUTPUT=UWR, ERRMSG=ERRMSG)
			IF N_ELEMENTS(ERRMSG) GT 0 THEN $
				IF ERRMSG(0) NE '' THEN RETURN
			END
;
; This next case should never happen.
;
		   ELSE:  MESSAGE = 'DOS & MacOS not supported yet.'
		ENDCASE
		END
	ELSE:	 MESSAGE = $
		  "Invalid TYPE sent to MAGTAPE: 'read' or 'write' only."
ENDCASE
;
; Change back to the original directory, close log file, and rewind the tape.
;
CD,OLD_DIR
IF QOUT EQ 'Y' THEN FREE_LUN, UWR
XWSTR = [' ', 'Rewinding tape ...', ' ', XWSTR]
IF QWIDGET EQ 1 THEN BEGIN
	WIDGET_CONTROL, XWIDGET, SET_VALUE=XWSTR
ENDIF ELSE BEGIN
	PRINT, XWSTR(0)
	PRINT, XWSTR(1)
	PRINT, XWSTR(2)
ENDELSE
CASE TYPERW OF
	'IDLTP':  DISMOUNT, UNIT, /NOUNLOAD
	'UNIXDD': SPAWN,'mt -f '+TAPENM+' rewind'
	ELSE:   MESSAGE = 'Error in rewinding the tape.'
ENDCASE
;
; In case someone runs the program again without leaving IDL.
;
IF QWIDGET EQ 0 THEN FITSHD = ''
;
IF MESSAGE NE '' THEN GOTO, HANDLE_ERROR
IF N_ELEMENTS(ERRMSG) GT 0 THEN ERRMSG=MESSAGE
RETURN
;
; Error handling portion of the procedure.
;
HANDLE_ERROR:
;
; In case someone runs the program again without leaving IDL.
;
IF QWIDGET EQ 0 THEN FITSHD = ''
;
IF N_ELEMENTS(QOUT) GT 0 THEN BEGIN
	IF QOUT EQ 'Y' THEN BEGIN
		PRINTF, UWR, MESSAGE
		FREE_LUN, UWR
	ENDIF
ENDIF
IF N_ELEMENTS(ERRMSG) EQ 0 THEN MESSAGE, MESSAGE
ERRMSG = MESSAGE
RETURN
;
END
