;+
; Project     :	SOHO - CDS
;
; Name        :	SFDU_2FHD
;
; Purpose     :	Program to extract primary FITS headers from the SFDU.
;
; Explanation :	The SoHO/SOWG has selected two standards relating the storage
;		of scientific data: the Standard Formatted Data Unit (SFDU)
;		and the Flexible Image Transport System (FITS).  FITS is a
;		standard for formatting data into files in a computer-
;		independent and self-describing fashion, whereas SFDU is a 
;		standard for documenting the contents and format of a data
;		file, and for outlining the interdependencies of collections
;		of files.
;
;		SFDU works by associating a 20 byte label with each file which 
;		serves as a reference to documentation about the file format.
;		The primary FITS header for each file has been written to tape 
;		in the SFDU header file under the PVL object name FITS_HEADER.
;
; Use         :	SFDU_2FHD, SFDU [, XWSTR]
;
; Inputs      :	SFDU:	The Standard Formatted Data Unit string array.
;
; Opt. Inputs :	XWSTR:	A string array that contains informational text that 
;			is either displayed to the screen or to the FILENAME
;			widget (set internally to XWIDGET) if XWINTAPE drives
;			this routine.
;
; Outputs     :	None.  Output is stored in FITSHD and PVLHEAD in the common 
;		block SFDU_FITSHD.
;
; Opt. Outputs:	XWSTR:	A string array that contains informational text that 
;			is either displayed to the screen or to the FILENAME
;			widget (set internally to XWIDGET) if XWINTAPE drives
;			this routine.  Note that SFDU_2FHD adds text to this
;			array.
;
; Keywords    :	ERRMSG:	If defined and passed, then any error messages will 
;			be returned to the user in this parameter rather than
;			being handled by the IDL MESSAGE utility.  If no 
;			errors are encountered, then a null string is 
;			returned.  In order to use this feature, the string 
;			ERRMSG must be defined first, e.g.,
;
;				ERRMSG = ''
;				SFDU_2FHD, SFDU, ERRMSG=ERRMSG
;				IF ERRMSG(0) NE '' THEN ...
;
;		XWIDGET: If set, the widget program XWINTAPE is driving this
;			procedure and any text that is normally printed to the
;			screen will instead be printed to the widget.  If
;			used, this keyword passes the ID of the FILENAME (set
;			internally to XWIDGET) widget.  This keyword requires
;			the optional input/output variable XWSTR.
;
; Calls       :	DATATYPE
;
; Common      :	SFDU_FITSHD
;
; Restrictions:	None.
;
; Side effects:	If an error is encountered, FITSHD and PVLHEAD are returned 
;		with the value of 'NONE'.
;
; Category    :	Data Handling, I/O, FITS, SFDU, Generic
;
; Prev. Hist. :	None.
;
; Written     :	Donald G. Luttermoser, GSFC/ARC, 19 May 1995
;
; Modified    :	Version 1, Donald G. Luttermoser, GSFC/ARC, 19 May 1995
;			Initial program.
;
; Version     :	Version 1,  19 May 1995
;-
;
PRO SFDU_2FHD, SFDU, XWSTR, XWIDGET=XWIDGET, ERRMSG=ERRMSG
;
COMMON SFDU_FITSHD, FITSHD, PVLHEAD
;
ON_ERROR, 2		; Return to caller if error is encountered.
MESSAGE = ''		; Set to non-null string if error is encountered.
FITSHD = 'NONE'		; Returned this if error is encountered.
PVLHEAD = 'NONE'	; Returned this if error is encountered.
;
; Check input parameters.
;
IF N_PARAMS() LT 1 THEN BEGIN
	MESSAGE = 'Syntax:  SFDU_2FHD, SFDU [, XWSTR]'
ENDIF ELSE BEGIN
	IF DATATYPE(SFDU,1) NE 'String' THEN MESSAGE = 'SFDU must be a '+$
		'string array containing the SFDU.'
ENDELSE
IF MESSAGE NE '' THEN GOTO, HANDLE_ERROR
;
IF N_ELEMENTS(XWSTR) EQ 0 THEN XWSTR = ' ' ELSE XWSTR = [' ', XWSTR]
;
; Find out how many FITS headers are in the SFDU.
;
NSFDU = N_ELEMENTS(SFDU)
HDLINES = INTARR(NSFDU)
CNTHD = 0  &  MAXLN = 0  &  INHEAD = 0
FOR I=0,NSFDU-1 DO BEGIN
	IF INHEAD EQ 1 THEN CNTLN = CNTLN + 1
	IF STRPOS(SFDU(I),'BEGIN_OBJECT = FITS_HEADER;') GT -1 THEN BEGIN
		CNTLN = 0
		CNTHD = CNTHD + 1
		INHEAD = 1
	ENDIF
	IF (STRPOS(SFDU(I),'END_OBJECT;') GT -1) AND (INHEAD EQ 1) THEN BEGIN
		CNTLN = CNTLN - 1
		HDLINES(CNTHD-1) = CNTLN
		MAXLN = MAX([MAXLN,CNTLN])
		INHEAD = 0
	ENDIF
ENDFOR
;
IF CNTHD LE 0 THEN XWSTR = ['No FITS headers found in SFDU.', XWSTR]
IF MAXLN LE 0 THEN XWSTR = ['No FITS headers contain no information in '+$
	'SFDU.', XWSTR]
IF XWSTR(0) NE ' ' THEN BEGIN
	IF KEYWORD_SET(XWIDGET) THEN WIDGET_CONTROL, XWIDGET, SET_VALUE = $
		XWSTR $
	ELSE BEGIN
		PRINT, XWSTR(0)
		PRINT, XWSTR(1)
	ENDELSE
	RETURN
ENDIF
;
; Set up arrays and read the FITS headers from the SFDU.
;
HDLINES = HDLINES(WHERE(HDLINES GT 0))
PVLHEAD = STRARR(CNTHD, MAXLN)
INHEAD = 0
II = 0  &  JJ = 0
;
XWSTR = ['Searching SFDU for "FITS_HEADER" PVL objects.', XWSTR]
IF KEYWORD_SET(XWIDGET) THEN WIDGET_CONTROL, XWIDGET, SET_VALUE = XWSTR $
	ELSE PRINT, XWSTR(0)
;
FOR K=0,NSFDU-1 DO BEGIN
	QSTART = STRPOS(SFDU(K),'BEGIN_OBJECT = FITS_HEADER;')
; Found start of FITS header.
	IF QSTART GT -1 THEN INHEAD = 1
; Found end of FITS header.
	IF (STRPOS(SFDU(K),'END_OBJECT;') GT -1) AND (INHEAD EQ 1) THEN BEGIN
		II = II + 1
		JJ = 0
		INHEAD = 0
	ENDIF
; Store FITS header.
	IF (INHEAD EQ 1) AND (QSTART EQ -1) THEN BEGIN
		IF II GE CNTHD THEN BEGIN
			MESSAGE = 'Error in storing HEAD array in PVLHEAD.'
			GOTO, HANDLE_ERROR
		ENDIF
		IF JJ GE MAXLN THEN BEGIN
			MESSAGE = 'Error in storing LINE in PVLHEAD.'
			GOTO, HANDLE_ERROR
		ENDIF
		PVLHEAD(II, JJ) = SFDU(K)	; Store the header line.
		JJ = JJ + 1
	ENDIF
ENDFOR
;
; Reformat the PVLHEAD array so that it looks like a standard FITS header.
;
XWSTR = ['Constructing FITS headers from SFDU/PVLs', XWSTR]
IF KEYWORD_SET(XWIDGET) THEN WIDGET_CONTROL, XWIDGET, SET_VALUE = XWSTR $
	ELSE PRINT, XWSTR(0)
;
FITSHD = STRARR(CNTHD, MAXLN+1)
FOR K=0,CNTHD-1 DO BEGIN
;
	TMPHEAD = STRARR(HDLINES(K)+1)
	TMPHEAD(0:HDLINES(K)-1) = STRTRIM(STRCOMPRESS(PVLHEAD(K, $
		0:HDLINES(K)-1)),2)
; Add FITS header ending mark.
	TMPHEAD(HDLINES(K)) = 'END'
	FOR J=0,HDLINES(K)-1 DO BEGIN
		LINESTR = TMPHEAD(J)
		QCOM1 = STRPOS(LINESTR, '/*')
		QCOM2 = STRPOS(LINESTR, '*/')
		QHIST = STRPOS(LINESTR, 'HISTORY')
		QSEMI = STRPOS(LINESTR, ';')
; Get rid of PVL ending marks.
		IF QSEMI GT -1 THEN STRPUT, LINESTR, ' ', QSEMI
; Change comments from PVL to FITS style.
		IF QCOM1 GT -1 THEN BEGIN
			STRPUT, LINESTR, '  ', QCOM1
			STRPUT, LINESTR, '  ', QCOM2
;
			IF QHIST GT -1 THEN BEGIN
; Reformat HISTORY lines.
				LINESTR = STRTRIM(LINESTR,2)
			ENDIF ELSE BEGIN
; Reformat other commented lines.
				HDINFO = STRTRIM(STRMID(LINESTR, 0, QCOM1),$
						2)
				HDCOMM = STRTRIM(STRMID(LINESTR, QCOM1, $
						QCOM2-QCOM1),2)
; Put "=" at byte 8 position.
				QEQUA = STRPOS(HDINFO, '=')
				ILEN = STRLEN(HDINFO)
				IF QEQUA EQ -1 THEN BEGIN
					MESSAGE = 'Error in locating keyword'+$
						' in: '+HDINFO
					GOTO, HANDLE_ERROR
				ENDIF
				KEYW = STRTRIM(STRMID(HDINFO, 0, QEQUA))
				VALW = STRMID(HDINFO, QEQUA+1, ILEN-QEQUA-1)
				JLEN = STRLEN(KEYW)
				IF JLEN LT 8 THEN BEGIN
					BLANKS = 8 - JLEN
					FOR I=0,BLANKS-1 DO KEYW = KEYW + ' '
				ENDIF
				HDINFO = KEYW + '=' + VALW
; Reformat HDINFO so that the comment "/" is put in the proper location.
				ILEN = STRLEN(HDINFO)
				IF ILEN LT 33 THEN BEGIN
					BLANKS = 33 - ILEN
					FOR I=0,BLANKS-1 DO HDINFO = HDINFO+' '
				ENDIF
				HDINFO = HDINFO + ' / '
; Now put back the reformatted string in TMPHEAD.
				LINESTR = HDINFO + HDCOMM
			ENDELSE
		ENDIF ELSE BEGIN
			LINESTR = TMPHEAD(J)
; Get rid of PVL ending marks.
			IF QSEMI GT -1 THEN STRPUT, LINESTR, ' ', QSEMI
			QEQUA = STRPOS(LINESTR, '=')
; Put "=" at byte 8 position.
			ILEN = STRLEN(LINESTR)
			IF QEQUA EQ -1 THEN BEGIN
				MESSAGE = 'Error in locating keyword in: '+$
					LINESTR
				GOTO, HANDLE_ERROR
			ENDIF
			KEYW = STRTRIM(STRMID(LINESTR, 0, QEQUA))
			VALW = STRMID(LINESTR, QEQUA+1, ILEN-QEQUA-1)
			JLEN = STRLEN(KEYW)
			IF JLEN LT 8 THEN BEGIN
				BLANKS = 8 - JLEN
				FOR I=0,BLANKS-1 DO KEYW = KEYW + ' '
			ENDIF
			LINESTR = KEYW + '=' + VALW
; Reformat LINESTR so that the comment "/" is put in the proper location.
			ILEN = STRLEN(LINESTR)
			IF ILEN LT 33 THEN BEGIN
				BLANKS = 33 - ILEN
				FOR I=0,BLANKS-1 DO LINESTR = LINESTR + ' '
			ENDIF
			LINESTR = LINESTR + ' / '
		ENDELSE
; ********** Now put back the reformatted string in TMPHEAD. **************
		TMPHEAD(J) = LINESTR
	ENDFOR
	FITSHD(K, 0:HDLINES(K)) = TMPHEAD(*)
ENDFOR
;
XWSTR = ['FITS headers are now generated.', XWSTR]
IF KEYWORD_SET(XWIDGET) THEN WIDGET_CONTROL, XWIDGET, SET_VALUE = XWSTR $
	ELSE PRINT, XWSTR(0)
;
IF N_ELEMENTS(ERRMSG) GT 0 THEN ERRMSG=MESSAGE
RETURN		;  Return to the calling program with no errors.  <<<<<<<<<<<<
;
; Error handling portion of the procedure.
;
HANDLE_ERROR:
IF N_ELEMENTS(ERRMSG) EQ 0 THEN MESSAGE, MESSAGE
ERRMSG = MESSAGE
RETURN
;
END
