;+
; Project     :	SOHO - CDS
;
; Name        :	SFDU_BYTES
;
; Purpose     :	Program to set various byte size parameters for SFDU tapes.
;
; Explanation :	The SoHO/SOWG has selected two standards relating the storage
;		of scientific data: the Standard Formatted Data Unit (SFDU)
;		and the Flexible Image Transport System (FITS).  FITS is a
;		standard for formatting data into files in a computer-
;		independent and self-describing fashion, whereas SFDU is a 
;		standard for documenting the contents and format of a data
;		file, and for outlining the interdependencies of collections
;		of files.
;
;		SFDU works by associating a 20 byte label with each file which 
;		serves as a reference to documentation about the file format.
;		This label can be prepended to the file, in which case it is
;		known as an "attached" SFDU label.  Otherwise, it can be 
;		contained in a separate file on the archive medium, with a 
;		reference to the file it applies to, which is a "detached"
;		label.  Note that comment and keyword (PVL = Parameter Value
;		Language) lines in the SFDU header can extend out to 80 bytes 
;		(i.e., characters).  Such lines end with a ";" (i.e., PVL) or 
;		are enclosed by "/* */" (i.e., comments) similar to C.  All 
;		other lines are standard 20-byte SFDU labels and begin with 
;		either "CCSD" or "NSSD".
;
; Use         :	SFDU_BYTES, BLFAC
;
; Inputs      :	BLFAC:	Blocking factor (n x 2880 bytes/record).  Must lie
;			between 1 and 10.
;
; Opt. Inputs :	None.
;
; Outputs     :	None.  Output is stored in the SFDU_BYTES common block.
;
; Opt. Outputs:	None.
;
; Keywords    :	ERRMSG:	If defined and passed, then any error messages will 
;			be returned to the user in this parameter rather than
;			being handled by the IDL MESSAGE utility.  If no 
;			errors are encountered, then a null string is 
;			returned.  In order to use this feature, the string 
;			ERRMSG must be defined first, e.g.,
;
;				ERRMSG = ''
;				SFDU_BYTES, BLFAC, ERRMSG=ERRMSG
;				IF ERRMSG(0) NE '' THEN ...
;
; Calls       :	DATATYPE
;
; Common      :	SFDU_BYTES
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Category    :	Data Handling, I/O, FITS, SFDU, Generic
;
; Prev. Hist. :	None.
;
; Written     :	Donald G. Luttermoser, GSFC/ARC, 4 May 1995
;
; Modified    :	Version 1, Donald G. Luttermoser, GSFC/ARC, 4 May 1995
;			Initial program.
;
; Version     :	Version 1,  4 May 1995
;-
;
PRO SFDU_BYTES, BLFAC, ERRMSG=ERRMSG
;
COMMON SFDU_BYTES, NSFDU_BYTE, NREC_PER_LINE, NTAPE_BYTE, NLINE_BYTE
;
ON_ERROR, 2	  ; Return to caller if error is encountered.
MESSAGE = ''	  ; Set to non-null string if error is encountered.
;
; Check input parameters.
;
IF N_PARAMS() NE 1 THEN BEGIN
	MESSAGE = 'Syntax:  SFDU_BYTES, BLFAC'
ENDIF ELSE BEGIN
	IF DATATYPE(BLFAC,1) NE 'Integer' THEN MESSAGE = $
		'Blocking factor must be an integer in the range: 1 - 10.'
	IF MESSAGE NE '' THEN GOTO, HANDLE_ERROR
;
	IF (BLFAC LT 1) OR (BLFAC GT 10) THEN MESSAGE = $
	  'Blocking factor must be an integer in the range: 1 - 10.'
ENDELSE
IF MESSAGE NE '' THEN GOTO, HANDLE_ERROR
;
; Initialize internal parameters.
;
NSFDU_BYTE = 20				; Bytes/record of SFDU blocks.
NREC_PER_LINE = 4			; Possible # of 20-byte SFDUs per line.
NLINE_BYTE = NSFDU_BYTE * NREC_PER_LINE	; Max bytes per ASCII line (incl. <CR>).
NTAPE_BYTE = 2880*BLFAC			; Bytes/record on the tape.
;
IF N_ELEMENTS(ERRMSG) GT 0 THEN ERRMSG=MESSAGE
RETURN  ;  Return to the calling program with no errors.  <<<<<<<<<<<<<<<<<<<
;
; Error handling portion of the procedure.
;
HANDLE_ERROR:
IF N_ELEMENTS(ERRMSG) EQ 0 THEN MESSAGE, MESSAGE
ERRMSG = MESSAGE
RETURN
;
END
