;+
; Project     :	SOHO - CDS
;
; Name        :	SFDU_FITSHD
;
; Purpose     :	Program to retrieve all primary FITS headers from disk files.
;
; Explanation :	The SoHO/SOWG has selected two standards relating the storage
;		of scientific data: the Standard Formatted Data Unit (SFDU)
;		and the Flexible Image Transport System (FITS).  FITS is a
;		standard for formatting data into files in a computer-
;		independent and self-describing fashion, whereas SFDU is a 
;		standard for documenting the contents and format of a data
;		file, and for outlining the interdependencies of collections
;		of files.
;
;		SFDU works by associating a 20 byte label with each file which 
;		serves as a reference to documentation about the file format.
;		The primary FITS header for each file written to tape will be
;		written to the SFDU under the object name FITS_HEADER.
;
; Use         :	SFDU_FITSHD, DISK, FILES
;
; Inputs      :	DISK:	The directory name where the FITS files are kept.
;			A passed '' in this parameter indicates that the 
;			current directory is to be used.
;
;		FILES:	The name of the FITS files on the disk that will be 
;			written to tape for which the primary headers are to 
;			be retrieved.
;
; Opt. Inputs :	None.
;
; Outputs     :	None.  Output is stored in FITSHD and PVLHEAD in the common 
;		block SFDU_FITSHD.
;
; Opt. Outputs:	None.
;
; Keywords    :	ERRMSG:	If defined and passed, then any error messages will 
;			be returned to the user in this parameter rather than
;			being handled by the IDL MESSAGE utility.  If no 
;			errors are encountered, then a null string is 
;			returned.  In order to use this feature, the string 
;			ERRMSG must be defined first, e.g.,
;
;				ERRMSG = ''
;				SFDU_FITSHD, DISK, FILES, ERRMSG=ERRMSG
;				IF ERRMSG(0) NE '' THEN ...
;
; Calls       :	DATATYPE, HEADFITS, SFDU_OS
;
; Common      :	SFDU_FITSHD
;
; Restrictions:	All of the FITS files must be in the same directory.
;
; Side effects:	If an error is encountered, FITSHD and PVLHEAD are returned 
;		with the value of 'NONE'.
;
; Category    :	Data Handling, I/O, FITS, SFDU, Generic
;
; Prev. Hist. :	None.
;
; Written     :	Donald G. Luttermoser, GSFC/ARC, 16 May 1995
;
; Modified    :	Version 1, Donald G. Luttermoser, GSFC/ARC, 16 May 1995
;			Initial program.
;		Version 2, Donald G. Luttermoser, GSFC/ARC, 28 December 1995
;			Fixed bug to insure that PVL length is less than 80
;			characters.
;
; Version     :	Version 2,  28 December 1995
;-
;
PRO SFDU_FITSHD, DISK, FILES, ERRMSG=ERRMSG
;
COMMON SFDU_FITSHD, FITSHD, PVLHEAD
;
ON_ERROR, 2		; Return to caller if error is encountered.
MESSAGE = ''		; Set to non-null string if error is encountered.
FITSHD = 'NONE'		; Returned this if error is encountered.
PVLHEAD = 'NONE'	; Returned this if error is encountered.
;
; Check input parameters.
;
IF N_PARAMS() NE 2 THEN BEGIN
	MESSAGE = 'Syntax:  SFDU_FITSHD, DISK, FILES'
ENDIF ELSE BEGIN
	IF DATATYPE(DISK,1) NE 'String' THEN MESSAGE = 'DISK must be a '+$
		'scalar string containing the directory name of the FITS files.'
	IF MESSAGE NE '' THEN GOTO, HANDLE_ERROR
;
	IF DATATYPE(FILES,1) NE 'String' THEN MESSAGE = 'FILES must be a '+$
		'string array containing the names of the FITS files.'
ENDELSE
IF MESSAGE NE '' THEN GOTO, HANDLE_ERROR
;
; Get operating system information.
;
SFDU_OS, OS_USED, DIR_COM, DIR_DELIM, SUFFIX, QVERS, TYPE, ERRMSG=ERRMSG
IF N_ELEMENTS(ERRMSG) GT 0 THEN IF ERRMSG(O) NE '' THEN RETURN
;
; Set the full path of the file names.
;
IF OS_USED EQ 'VMS' THEN DIR_DELIM = ''
IF DISK NE '' THEN INFILES = DISK+DIR_DELIM+INFILES ELSE INFILES = FILES
;
; Set internal parameters.
;
NF = N_ELEMENTS(INFILES)
HDNUM = INTARR(NF)	; Number of lines in each header.
MAXHL = 200		; Assume the maximum FITS header lines is MAXHL.
RESETCNT = 0		; Count the number of times we reset MAXHL.
;
RESETHD:		; Come back here and reset MAXHD if need to.
FITSHD = 'NONE'		; Returned this if error is encountered.
RESETCNT = RESETCNT + 1
IF RESETCNT GE 100 THEN BEGIN
	MESSAGE = 'Unable to figure out the maximum number of FITS header '+$
		'lines.'
	GOTO, HANDLE_ERROR
ENDIF
FHEAD = STRARR(NF, MAXHL)
;
; Read the headers.
;
FOR I=0,NF-1 DO BEGIN
	HEAD = HEADFITS(INFILES(I))
	HDNUM(I) = N_ELEMENTS(HEAD)
	IF HDNUM(I) GT MAXHL THEN BEGIN
		MAXHL = HDNUM(I)
		GOTO, RESETHD
	ENDIF
	FHEAD(I, 0:HDNUM(I)-1) = HEAD
ENDFOR
;
; Make the final FITS header array.
;
FITSHD = STRARR(NF, MAX([0,HDNUM]))
FOR I=0,NF-1 DO FITSHD(I, 0:HDNUM(I)-1) = FHEAD(I, 0:HDNUM(I)-1)
;
; Now modify this FITS header array so that it conforms to the PVL standards.
;
PVLHEAD = STRARR(NF, MAX([0,HDNUM-1]))
PVLINDENT = '     '
;
FOR I=0,NF-1 DO BEGIN
	TMPHEAD = STRARR(HDNUM(I))
	TMPHEAD(0:HDNUM(I)-1) = FITSHD(I,0:HDNUM(I)-1)
	TMPHEAD = STRTRIM(STRCOMPRESS(TMPHEAD),2)
	COMHEAD = TMPHEAD
	FOR J=0,HDNUM(I)-1 DO BEGIN
; Get rid of the final END line.
		IF TMPHEAD(J) EQ 'END' THEN TMPHEAD(J) = ''
; Separate the comments from the FITS header (i.e., find ' /').
		SLASH = STRPOS(COMHEAD(J), ' /')
		SLEN = STRLEN(COMHEAD(J))
		IF SLASH GT -1 THEN BEGIN
			COMHEAD(J) = STRTRIM(STRMID(COMHEAD(J), SLASH+2, $
					SLEN-SLASH-2),2)
			TMPHEAD(J) = STRTRIM(STRMID(TMPHEAD(J), 0, SLASH),2)
		ENDIF ELSE BEGIN
			COMHEAD(J) = ''
		ENDELSE
;
; >>>> COMHEAD now has the comments to tack on at the end of the line!
;      TMPHEAD has the header string preceeding the comment (keyword + value).
;
; Delete line ending slashes (i.e., those lines without comments).
		SLEN = STRLEN(TMPHEAD(J))
		IF STRMID(TMPHEAD(J), SLEN-1, 1) EQ '/' THEN $
			TMPHEAD(J) = STRMID(TMPHEAD(J), 0, SLEN-1)
; Separate keywords and values.
		KEYHEAD = TMPHEAD
		EQUAL = STRPOS(KEYHEAD(J), '=')
		SLEN = STRLEN(KEYHEAD(J))
		IF EQUAL GT -1 THEN BEGIN
			TMPHEAD(J) = STRTRIM(STRMID(TMPHEAD(J), EQUAL+1, $
					SLEN-EQUAL-1),2) + ';'
			KEYHEAD(J) = STRTRIM(STRMID(KEYHEAD(J), 0, EQUAL),2)
		ENDIF ELSE BEGIN
; If no equal sign, then it's a HISTORY or END (= '') line.  Comment HISTORY.
			IF TMPHEAD(J) NE '' THEN BEGIN
				IF SLEN GT 68 THEN TMPHEAD(J) = $
					STRMID(TMPHEAD(J), 0, 68)
				TMPHEAD(J) = PVLINDENT + '/* ' + TMPHEAD(J) +$
					' */'
			ENDIF
			KEYHEAD(J) = ''
		ENDELSE
;
; >>>> KEYHEAD now contains the FITS keyword alone!
; >>>> TMPHEAD now contains the value of the FITS keyword!
;
; Format the keywords into 8-character strings (= max FITS keyword size).
		IF KEYHEAD(J) NE '' THEN BEGIN
			KLEN = STRLEN(KEYHEAD(J))
			IF KLEN LT 8 THEN FOR K=KLEN,7 DO KEYHEAD(J) = $
				KEYHEAD(J) + ' '
; Add PVL indent and equal sign to keywords.
			KEYHEAD(J) = PVLINDENT + KEYHEAD(J) + ' = '
		ENDIF
;
CALCLEN:
		PVLLEN = STRLEN(KEYHEAD(J))
		TMPLEN = STRLEN(TMPHEAD(J))
		COMLEN = STRLEN(COMHEAD(J)) + 6
;
		BLANKS = 78 - (PVLLEN + TMPLEN + COMLEN)
		IF BLANKS LT 1 THEN BEGIN
			IF COMLEN LE 6 THEN BEGIN
				MESSAGE = 'FITS header line > 79 characters.'
				GOTO, HANDLE_ERROR
			ENDIF
			COMHEAD(J) = STRMID(COMHEAD(J), 0, COMLEN-6+BLANKS)
			BLANKS = 0
		ENDIF
; Add blanks between the FITS header info and comments -> comments on RHS.
		IF COMHEAD(J) NE '' THEN COMHEAD(J) = '/* ' + COMHEAD(J) + $
			' */'
		FOR K=0,BLANKS DO COMHEAD(J) = ' ' + COMHEAD(J)
; Make final PVL FITS header line at 79 characters long.
		IF TMPHEAD(J) NE '' THEN TMPHEAD(J) = KEYHEAD(J) + $
			TMPHEAD(J) + COMHEAD(J)
	ENDFOR
;
	TMPHEAD = TMPHEAD(WHERE(TMPHEAD NE ''))
	NTMP = N_ELEMENTS(TMPHEAD)
	IF NTMP GT HDNUM(I)-1 THEN BEGIN
		MESSAGE = 'Error in constructing PVL/FITS header.'
		GOTO, HANDLE_ERROR
	ENDIF
	PVLHEAD(I, 0:NTMP-1) = TMPHEAD
ENDFOR
;
IF N_ELEMENTS(ERRMSG) GT 0 THEN ERRMSG=MESSAGE
RETURN		;  Return to the calling program with no errors.  <<<<<<<<<<<<
;
; Error handling portion of the procedure.
;
HANDLE_ERROR:
IF N_ELEMENTS(ERRMSG) EQ 0 THEN MESSAGE, MESSAGE
ERRMSG = MESSAGE
RETURN
;
END
