;+
; Project     :	SOHO - CDS
;
; Name        :	SFDU_FSIZE
;
; Purpose     :	Program to ascertain total bytes in FITS files for tape I/O.
;
; Explanation :	The SoHO/SOWG has selected two standards relating the storage
;		of scientific data: the Standard Formatted Data Unit (SFDU)
;		and the Flexible Image Transport System (FITS).  FITS is a
;		standard for formatting data into files in a computer-
;		independent and self-describing fashion, whereas SFDU is a 
;		standard for documenting the contents and format of a data
;		file, and for outlining the interdependencies of collections
;		of files.
;
;		SFDU works by associating a 20 byte label with each file which 
;		serves as a reference to documentation about the file format.
;		This procedure ascertains the total bytes in all FITS files 
;		to be loaded to tape.  This number, along with the size of the
;		SFDU header file, is written to the SFDU header via the PVL 
;		keywords "TOTAL_FITS_BYTES" and "TOTAL_TAPE_BYTES" (which also
;		includes the size of the SFDU header).
;
; Use         :	FILE_SIZE = SFDU_FSIZE( DISK, FILES)
;
; Inputs      :	DISK:	The directory name where the FITS files are kept.
;
;		FILES:	The name of the FITS files on the disk that will be 
;			written to tape.
;
; Opt. Inputs :	None.
;
; Outputs     :	FILE_SIZE:  A structure (FSIZE) that contains the tags:
;			BYTES:	Total number of bytes associated with all of
;				the FITS disk files to be read to tape.
;			TAPE_BLOCKS: Total number of tape blocks associated 
;				with the FITS files on the tape.  Note that
;				1 tape block = BLFAC*2880 (=NTAPE_BYTE) bytes.
;			VMS_BLOCKS:  Total number of VMS blocks associated 
;				with all of the FITS disk files.  Note that 
;				1 VMS block = 512 bytes.
;
; Opt. Outputs:	None.
;
; Keywords    :	ERRMSG:	If defined and passed, then any error messages will 
;			be returned to the user in this parameter rather than
;			being handled by the IDL MESSAGE utility.  If no 
;			errors are encountered, then a null string is 
;			returned.  In order to use this feature, the string 
;			ERRMSG must be defined first, e.g.,
;
;				ERRMSG = ''
;				RESULT = SFDU_FSIZE(DISK, FILES, ERRMSG=ERRMSG)
;				IF ERRMSG(0) NE '' THEN ...
;
; Calls       :	DATATYPE, SFDU_OS
;
; Common      :	SFDU_BYTES
;
; Restrictions:	All of the FITS files must be in the same directory.
;
; Side effects:	If an error is encountered, all tags in the structure are
;		returned with values of -1.
;
; Category    :	Data Handling, I/O, FITS, SFDU, Generic
;
; Prev. Hist. :	None.
;
; Written     :	Donald G. Luttermoser, GSFC/ARC, 16 May 1995
;
; Modified    :	Version 1, Donald G. Luttermoser, GSFC/ARC, 16 May 1995
;			Initial program.
;
; Version     :	Version 1,  16 May 1995
;-
;
FUNCTION SFDU_FSIZE, DISK, FILES, ERRMSG=ERRMSG
;
COMMON SFDU_BYTES, NSFDU_BYTE, NREC_PER_LINE, NTAPE_BYTE, NLINE_BYTE
;
ON_ERROR, 2	  ; Return to caller if error is encountered.
MESSAGE = ''	  ; Set to non-null string if error is encountered.
;
; Set up output structure.
;
FILE_SIZE = {FSIZE, BYTES: -1L, TAPE_BLOCKS: -1L, VMS_BLOCKS: -1L}
;
; Check input parameters.
;
IF N_PARAMS() NE 2 THEN BEGIN
	MESSAGE = 'Syntax:  Result = SFDU_FSIZE( DISK, FILES)'
ENDIF ELSE BEGIN
	IF N_ELEMENTS(NSFDU_BYTE) EQ 0 THEN MESSAGE = 'The procedure '+$
		'SFDU_BYTES must be ran before SFDU_FSIZE is run.'
	IF MESSAGE NE '' THEN GOTO, HANDLE_ERROR
;
	IF DATATYPE(DISK,1) NE 'String' THEN MESSAGE = 'DISK must be a '+$
		'scalar string containing the directory name of the FITS files.'
	IF MESSAGE NE '' THEN GOTO, HANDLE_ERROR
;
	IF DATATYPE(FILES,1) NE 'String' THEN MESSAGE = 'FILES must be a '+$
		'string array containing the names of the FITS files.'
ENDELSE
IF MESSAGE NE '' THEN GOTO, HANDLE_ERROR
;
; Define internal parameters.
;
MON = [' jan ', ' feb ', ' mar ', ' apr ', ' may ', ' jun ', ' jul ', $
	' aug ', ' sep ', ' oct ', ' nov ', ' dec ']
;
; Get operating system information.
;
SFDU_OS, OS_USED, DIR_COM, DIR_DELIM, SUFFIX, QVERS, TYPE, ERRMSG=ERRMSG
IF N_ELEMENTS(ERRMSG) GT 0 THEN IF ERRMSG(0) NE '' THEN RETURN, FILE_SIZE
;
; Get a full listing (including sizes) of the FITS files.
;
CASE OS_USED OF
	'VMS':		DIR_COM = DIR_COM + '/size=all'
	'UNIX':		DIR_COM = DIR_COM + ' -l'
	'DOS':		MESSAGE = 'SFDU_FSIZE does not yet support the DOS '+$
				'operating system.'
	'MACOS':	MESSAGE = 'SFDU_FSIZE does not yet support the MacOS'+$
				' operating system.'
	ELSE:		MESSAGE = 'Unknown operating system encountered:  '+$
				OS_USED
ENDCASE
IF MESSAGE NE '' THEN GOTO, HANDLE_ERROR
;
IF OS_USED EQ 'VMS' THEN DIR_DELIM = ''
SPAWN,DIR_COM+' '+DISK+DIR_DELIM+'*'+SUFFIX, FOUND
;
; Trim off any unwanted text.
;
FOUND = STRTRIM(STRCOMPRESS(STRLOWCASE(FOUND)))
USE = STRPOS(FOUND, SUFFIX)
IUSE = WHERE(USE GT -1)
IF IUSE(0) EQ -1 THEN BEGIN
	MESSAGE = 'No *'+SUFFIX+' files found in '+DISK
	GOTO, HANDLE_ERROR
ENDIF
FOUND = FOUND(IUSE)
;
; Find which files from the disk will be written to tape
;
INFILES = STRLOWCASE(FILES)
USE = INTARR(N_ELEMENTS(FOUND))  &  USE(*) = -1
FOR I=0,N_ELEMENTS(FOUND)-1 DO BEGIN
	FOR J=0,N_ELEMENTS(INFILES)-1 DO BEGIN
		IF STRPOS(FOUND(I), INFILES(J)) GT -1 THEN USE(I) = 1
	ENDFOR
ENDFOR
IUSE = WHERE(USE GT -1) 
FOUND = FOUND(IUSE)
IF N_ELEMENTS(FOUND) NE N_ELEMENTS(INFILES) THEN BEGIN
	MESSAGE = 'SFDU_FSIZE could not find all of the tape input files '+$
		'located in '+DISK
	GOTO, HANDLE_ERROR
ENDIF
;
; Find the location of the file byte size on Unix machines.  Note that it is 
; never found as the first character.
;
IF OS_USED EQ 'UNIX' THEN BEGIN
	BYTLOC = INTARR(2,N_ELEMENTS(FOUND))
	FOR I=0,N_ELEMENTS(FOUND)-1 DO BEGIN
		FOR J=0,11 DO BEGIN
			QMON = STRPOS(FOUND(I), MON(J))
			IF QMON GT -1 THEN BEGIN
				BYTLOC(1,I) = QMON
				FOR K=QMON-1,1,-1 DO BEGIN
					IF STRMID(FOUND(I), K, 1) EQ ' ' $
					  THEN BEGIN
						BYTLOC(0,I) = K
						GOTO, NEXTFILE
					ENDIF
				ENDFOR
				MESSAGE = 'Cannot find file size for: '+$
					FOUND(I)
				GOTO, HANDLE_ERROR
			ENDIF
		ENDFOR
		MESSAGE = 'Cannot find the date for: '+FOUND(I)
		GOTO, HANDLE_ERROR
NEXTFILE:
	ENDFOR
ENDIF
;
; Add up the sizes of the files.
;
VMSBL = 0L
BYTES = 0L
FOR I=0,N_ELEMENTS(FOUND)-1 DO BEGIN
	CASE OS_USED OF
		'VMS':	BEGIN
				SLASH = STRPOS(FOUND(I),'/')
				IF SLASH EQ -1 THEN BEGIN
					MESSAGE = 'Could not find the size '+$
						'of file: '+FOUND(I)
					GOTO, HANDLE_ERROR
				ENDIF
				LEN = STRLEN(FOUND(I))
				VMSBL = VMSBL + LONG(STRMID(FOUND(I), SLASH+1,$
					LEN-SLASH-1))
			END
		'UNIX':	BYTES = BYTES + LONG(STRMID(FOUND(I), BYTLOC(0,I), $
				BYTLOC(1,I)-BYTLOC(0,I)))
	ENDCASE
ENDFOR
;
IF OS_USED EQ 'VMS' THEN BYTES = VMSBL*512L ELSE VMSBL = BYTES/512L
;
; Input the calculated values into the output structure.
;
FILE_SIZE.BYTES = BYTES
IF BYTES MOD NTAPE_BYTE NE 0 THEN FILE_SIZE.TAPE_BLOCKS = BYTES/NTAPE_BYTE+1 $
	ELSE FILE_SIZE.TAPE_BLOCKS = BYTES/NTAPE_BYTE
FILE_SIZE.VMS_BLOCKS = VMSBL
;
IF N_ELEMENTS(ERRMSG) GT 0 THEN ERRMSG=MESSAGE
RETURN, FILE_SIZE  ;  Return to the calling program with no errors.  <<<<<<<<
;
; Error handling portion of the procedure.
;
HANDLE_ERROR:
IF N_ELEMENTS(ERRMSG) EQ 0 THEN MESSAGE, MESSAGE
ERRMSG = MESSAGE
RETURN, FILE_SIZE
;
END
