;+
; Project     :	SOHO - CDS
;
; Name        :	SFDU_GETREG
;
; Purpose     :	Determine the name and location of the SFDU registration files.
;
; Explanation :	Various SFDU procedures need to access the latest SFDU
;		registration files.  This procedure finds the names of these
;		files and their ADIDNAMEs.
;
; Use         :	SFDU_GETREG, ADID_SFDU
;
; Inputs      :	None.
;
; Opt. Inputs :	None.
;
; Outputs     :	ADID_SFDU:  A structure that contains information concerning
;			the SFDU registration files.  Tags:
;			NAME:	The keyword name identifying the file.
;				VOLM:	The Volumn Keyword Description.
;				CATG:	SOHO/CDS Catalog Object: FITS/PVL
;					Description.
;				FITS:	SOHO/CDS FITS File Description.
;				DESC:	SOHO/CDS Instrument Description.
;			CAID:	The Control Authority Identifier.
;			DDID:	The Data Description Identifier (note that the
;				ADID = CAID+DDID).
;			FILE:	The filename (including directory and file
;				suffix) of the latest registration file.
;			VERS:	The version number of the registration file.
;
; Opt. Outputs:	None.
;
; Keywords    :	ERRMSG:	If defined and passed, then any error messages will 
;			be returned to the user in this parameter rather than
;			being handled by the IDL MESSAGE utility.  If no 
;			errors are encountered, then a null string is 
;			returned.  In order to use this feature, the string 
;			ERRMSG must be defined first, e.g.,
;
;				ERRMSG = ''
;				SFDU_GETREG, ADID_SFDU, ERRMSG=ERRMSG)
;				IF ERRMSG(0) NE '' THEN ...
;
; Calls       :	FIND_WITH_DEF()
;
; Common      :	None.
;
; Restrictions:	So as not to make this procedure "hardwired" to a given 
;		machine on which it runs, SFDU_GETREG reads the directory
;		name where the SFDU registration files are kept from the data
;		file sfdu_getreg.dat.
;
; Side effects:	None.
;
; Category    :	Data Handling, I/O, FITS, SFDU, Generic
;
; Prev. Hist. :	None.
;
; Written     :	Donald G. Luttermoser, GSFC/ARC, 20 December 1995
;
; Modified    :	Version 1, Donald G. Luttermoser, GSFC/ARC, 20 December 1995
;			Initial program.
;
; Version     :	Version 1,  20 December 1995
;-
;
PRO SFDU_GETREG, ADID_SFDU, ERRMSG=ERRMSG
;
ON_ERROR, 2		; Return to caller if error is encountered.
MESSAGE = ''		; Set to non-null string if error is encountered.
;
; Check input parameters.
;
IF N_PARAMS() NE 1 THEN BEGIN
	MESSAGE = 'Syntax:  SFDU_GETREG, ADID_SFDU'
	IF MESSAGE NE '' THEN GOTO, HANDLE_ERROR
ENDIF
;
; The SFDU ADID/file structure array:
;  Note that the original version of the SFDU registration files has a suffix
;  of ".reg0".  Any subsequent revisions to the SFDU registration files will
;  be suffixed with ".reg1", ".reg2", etc.
;
ADID_SFDU = REPLICATE({ADID, NAME: '', CAID: 'NSSD', DDID: '0000', $
	FILE: 'sfdu', VERS: 0}, 4)
; ADID for the "Volume Keyword Description".
ADID_SFDU(0) = {ADID, 'VOLM', 'NSSD', '0227', 'sfdu_vol_dict', 0}
; ADID for the "SOHO/CDS Catalog Object".
ADID_SFDU(1) = {ADID, 'CATG', 'NSSD', '0228', 'sfdu_cds_cat_obj', 0}
; ADID for the "SOHO/CDS FITS File Descritpion".
ADID_SFDU(2) = {ADID, 'FITS', 'NSSD', '0229', 'sfdu_cds_fits_dict', 0}
; ADID for the "SOHO/CDS Instrument Description".
ADID_SFDU(3) = {ADID, 'DESC', 'NSSD', '0238', 'sfdu_cds_desc', 0}
;
FILE_SUFFIX = '.reg'
;
; Find the file that contains the SFDU registration files directory name.
;
REGDAT = FIND_WITH_DEF('sfdu_getreg.dat', !PATH)
IF REGDAT(0) EQ '' THEN BEGIN
	MESSAGE = 'Cannot find the file that contains the directory name ' + $
		'where the SFDU registration files are kept.'
	GOTO, HANDLE_ERROR
ENDIF
;
; Get the directory name where the SFDU registration files are kept.
;
SFDU_DIR = ''	; Directory with the SFDU registration files.
DUMMY = ' '
OPENR, URD, REGDAT(0), /GET_LUN
	WHILE NOT EOF(URD) DO BEGIN
		READF, URD, DUMMY
		COLPOS = STRPOS(DUMMY, ':')
		LEN = STRLEN(DUMMY)
		IF COLPOS NE -1 THEN BEGIN
			SFDU_DIR = STRTRIM(STRMID(DUMMY, COLPOS+1, $
				LEN-COLPOS-1), 2) + ' '
			BLKPOS = STRPOS(SFDU_DIR, ' ')
			IF BLKPOS EQ -1 THEN BEGIN
				MESSAGE = 'Error in reading the location '+$
					' of the SFDU registration files.'
				FREE_LUN, URD
				GOTO, HANDLE_ERROR
			ENDIF
			SFDU_DIR = STRMID(SFDU_DIR, 0, BLKPOS)
		ENDIF
	ENDWHILE
FREE_LUN, URD
;
; ***************************************************************************
;
; Get the names of the latest versions of the SFDU registration files.
;
GOT_FILE = INTARR(4)
GOT_FILE(*) = 0
HOLD_NAME = 'error'
FOR I = 0, 3 DO BEGIN
	IVERS = ADID_SFDU(I).VERS
GET_REG_VERS:
	STRVCT = STRTRIM(STRING(IVERS+100),2)
	IF STRMID(STRVCT, 1, 1) NE 0 THEN STRVCT = STRMID(STRVCT, 1, 2) $
		ELSE STRVCT = STRMID(STRVCT, 2, 1)
	TEST_FILE = FIND_WITH_DEF(ADID_SFDU(I).FILE, SFDU_DIR, $
		+FILE_SUFFIX+STRVCT)
	IF TEST_FILE NE '' THEN BEGIN
		HOLD_NAME = TEST_FILE
		GOT_FILE(I) = 1
		IVERS = IVERS + 1
		IF IVERS GT 99 THEN BEGIN
			MESSAGE = 'Too many SFDU registration versions of ' + $
				'file ' + ADID_SFDU(I).FILE + $
				' found in directory ' + SFDU_DIR
			GOTO, HANDLE_ERROR
		ENDIF
		GOTO, GET_REG_VERS
	ENDIF ELSE BEGIN
; The initial (and previous) version(s) of the SFDU registration file has been 
; deleted.  Continue anyway until we find one.
		IF GOT_FILE(I) EQ 0 THEN BEGIN
			IVERS = IVERS + 1
			IF IVERS GT 99 THEN BEGIN
				MESSAGE = 'Could not find the SFDU ' + $
					'registration file ' + $
					ADID_SFDU(I).FILE + $
					' in directory ' + SFDU_DIR
				GOTO, HANDLE_ERROR
			ENDIF
			GOTO, GET_REG_VERS
		ENDIF
	ENDELSE
	ADID_SFDU(I).VERS = IVERS - 1
	ADID_SFDU(I).FILE = HOLD_NAME
ENDFOR
;
IF N_ELEMENTS(ERRMSG) GT 0 THEN ERRMSG=MESSAGE
RETURN		;  Return to the calling program with no errors.  <<<<<<<<<<<<<
;
; Error handling portion of the procedure.
;
HANDLE_ERROR:
IF N_ELEMENTS(ERRMSG) EQ 0 THEN MESSAGE, MESSAGE
ERRMSG = MESSAGE
RETURN
;
END
