;+
; Project     :	SOHO - CDS
;
; Name        :	SFDU_MAKE
;
; Purpose     :	Program to generate a SFDU header file.
;
; Explanation :	The SOHO/SOWG has selected two standards relating the storage
;		of scientific data: the Standard Formatted Data Unit (SFDU)
;		and the Flexible Image Transport System (FITS).  FITS is a
;		standard for formatting data into files in a computer-
;		independent and self-describing fashion, whereas SFDU is a 
;		standard for documenting the contents and format of a data
;		file, and for outlining the interdependencies of collections
;		of files.
;
;		The SFDU protocol generated here was registered with the CSSDS
;		(through the NNSDC) in early December 1995.  The files are 
;			sfdu_vol_dict.reg (NSSD0227):
;				Volume keyword dictionary,
;			sfdu_cds_cat_obj.reg (NSSD0228):
;				Describes how the SFDU/PVL relates to the FITS 
;				headers,
;			sfdu_cds_fits_dict.reg (NSSD0229):
;				SOHO/CDS FITS header dictionary,
;			sfdu_cds_desc.reg (NSSD0238):
;				SOHO/CDS instrument description.
;		The numbers in the parentheses are the ADID names of the files.
;
;		SFDU works by associating a 20 byte label with each file which 
;		serves as a reference to documentation about the file format.
;		This label can be prepended to the file, in which case it is
;		known as an "attached" SFDU label.  Otherwise, it can be 
;		contained in a separate file on the archive medium, with a 
;		reference to the file it applies to, which is a "detached"
;		label.  Note that comment and keyword lines in the SFDU header
;		can extend out to 80 bytes (i.e., characters).  Such lines end
;		with a ";" (i.e., keywords) or have "/* */" delimiters (i.e.,
;		comments) similar to C comments.  All other lines are standard 
;		20-byte SFDU labels and begin with either "CCSD" or "NSSD".
;
;		A standard 20-byte SFDU label might look like this:
;				CCSD3ZF0000100000001
;		which has the following meaning:
;		 ------------------------------------------------------------
;		|   CCSD    | 3| Z| F| 0|    0001    |     00000001          |
;		 ------------------------------------------------------------
;		  1  2  3  4  5  6  7  8  9  10 11 12 13 14 15 16 17 18 19 20
;
;		Bytes 1-4, 9-12:	ADID (Authority [CAID] and Description 
;					      Identifier [DDID])  (= CCSD0001)
;			Bytes 1-4:	[CAID]  (= CCSD, can also be NSSD)
;			Bytes 9-12:	[DDID]	(= 0001)
;		Byte 5:			Version of SFDU label  (= 3)
;			1 = Length in ASCII
;			2 = Length in Binary Integer
;			3 = Other Delimitation Techniques
;		Byte 6:			SFDU Class Identification  (= Z)
;				 --
;		  Registered	|  C	SFDU Identifier Service
;		  Metadata	|  D	Data Description Record
;		  Objects	|  E	Data Element Dictionary
;				|  F	Description Data Unit (DDU)
;				 --
;				 --
;				|  I	(Primary) Data (of Interest)
;		  Data		|  K	Instance Identifier/Attributes
;		  Objects	|  R	SFDU Reference Service
;				|  S	Supplementary Data
;				|  V	Volumn Preparation Data
;				 --
;				 --
;		  SFDU Object	|  U	Related, Labeled Data Objects follows
;		  Identifiers	|  Z	"I'm an SFDU"; Labeled Object follows
;				 --
;		Byte 7:			Delimitation Type  (= F)
;			0 = Spare (in Version 1 & 2, the rest are Version 3)
;			S = Marker
;			E = Sequential EOF(s)
;			C = Consecutive EOF(s)
;			F = System EOF
;		Byte 8:			Spare  (= 0)
;		Bytes 13-20:		Description  (= 00000001)
;			Length of Object to follow (Version 1 & 2 Label)
;			Marker (Version 3, Type S)
;			EOF Count (Version 3, Type E & C)
;			00000001 (Version 3, Type F)
;
;		Short Glossary:
;			ADID:	Authority Description Identifier.
;			LVO:	Label Value Object.
;			PVL:	Parameter Value Language.
;
;		Information concerning the files is stored in the SFDU 
;		following the PVL formalism.  PVL's are very similar to FITS
;		headers and the PVL's associated with the SOHO data will be
;		obtained from the individual FITS files.  For further 
;		information, see CDS Software Note #28.
;
;		NOTE: This procedure is designed to be accessed only from 
;		MAGTAPE!
;
; Use         :	SFDU = SFDU_MAKE( SHORTNM, LONGNM [, FILE_SIZE])
;
; Inputs      :	SHORTNM: A string array that contains all of the "short" file
;			names in the SFDU header.  These names are meant to
;			be used in the DOS environments.  According to $CCSDS1
;			specifications, the short file name must be of the form
;			xxxxx.yyy, where the filename "xxxxx" must be "A-Z", 
;			"0-9", or "_" symbols 1-8 characters long, and the 
;			extension "yyy" must be made of the same symbols 1-3 
;			characters long.  The SOHO/CDS team has adopted the 
;			form of "filename.fit" for the short name, where 
;			"filename" is truncated to 8 characters when necessary.
;
;		LONGNM:	A string array that contains all of the "long" file
;			names in the SFDU header.  These names are meant to
;			be used in the UNIX, VMS, and MacOS environments.  
;			According to $CCSDS2 specifications, the long file 
;			name must be composed of the same symbols as mentioned
;			for the short name, but the total length of the string
;			must lie between 1-31.  The SOHO/CDS team has adopted
;			the form of "filename.fits" for the long name, where 
;			"filename" can range between 1-26 characters.
;
; Opt. Inputs :	FILE_SIZE:  A structure (FSIZE) that contains information
;			concerning the total byte size of the files to be 
;			loaded to tape.  This number, along with the byte size 
;			of the SFDU header are written to the header.  This 
;			information can then be used to check to see if 
;			sufficient space is available on the tape or disk.
;
; Outputs     :	SFDU:	A string array that contains the entire SFDU header.
;
; Opt. Outputs:	None.
;
; Keywords    :	ERRMSG:	If defined and passed, then any error messages will 
;			be returned to the user in this parameter rather than
;			being handled by the IDL MESSAGE utility.  If no 
;			errors are encountered, then a null string is 
;			returned.  In order to use this feature, the string 
;			ERRMSG must be defined first, e.g.,
;
;				ERRMSG = ''
;				SFDU = SFDU_MAKE( SHRTNM, LNGNM, FILE_SIZE, $
;					ERRMSG=ERRMSG)
;				IF ERRMSG(0) NE '' THEN ...
;
; Calls       :	DATATYPE, GET_UTC, SFDU_GETREG, SFDU_VOLHD, UTC2STR
;
; Common      :	SFDU_BYTES, SFDU_FITSHD
;
; Restrictions:	This procedure is setup only for the Unix environment and
;		specifically for the CDS machines.
;
; Side effects:	Returns a string of value '-1' if an error is encountered and 
;		the keyword ERRMSG has been set.  This routine assumes that
;		the maximum number of printable characters in an SFDU header
;		line is 79.  The 80th character is a new line feed <CR>.
;
; Category    :	Data Handling, I/O, FITS, SFDU, Generic
;
; Prev. Hist. :	None.
;
; Written     :	Donald G. Luttermoser, GSFC/ARC, 20 March 1995
;
; Modified    :	Version 1, Donald G. Luttermoser, GSFC/ARC, 20 March 1995
;			Initial program.
;		Version 2, Donald G. Luttermoser, GSFC/ARC, 25 April 1995
;			Modified SFDU line so that each ASCII line has 80
;			characters in it with the last character being a <CR>.
;		Version 3, Donald G. Luttermoser, GSFC/ARC, 16 May 1995
;			Included PVL data in the SFDU.
;		Version 4, Donald G. Luttermoser, GSFC/ARC, 26 May 1995
;			Modified the document header and the length of the
;			"long" version of the FITS filename.
;		Version 5, Donald G. Luttermoser, GSFC/ARC, 11 December 1995
;			Modified the format of the SFDU header to reflect the
;			the changes incorporated into the initial (i.e.,
;			original) SFDU registration files.  Created a new 
;			procedure called SFDU_VOLHD to make the size of this
;			procedure smaller.
;
; Version     :	Version 5,  11 December 1995
;-
;
FUNCTION SFDU_MAKE, SHORTNM, LONGNM, FILE_SIZE, ERRMSG=ERRMSG
;
COMMON SFDU_BYTES, NSFDU_BYTE, NREC_PER_LINE, NTAPE_BYTE, NLINE_BYTE
COMMON SFDU_FITSHD, FITSHD, PVLHEAD
;
ON_ERROR, 2		; Return to caller if error is encountered.
MESSAGE = ''		; Set to non-null string if error is encountered.
SFDU = '-1'		; Return this string if initial error encountered.
;
; Check input parameters.
;
IF (N_PARAMS() LT 2) OR (N_PARAMS() GT 3) THEN BEGIN
	MESSAGE = 'Syntax:  SFDU = SFDU_MAKE( SHORTNM, LONGNM [, FILE_SIZE])'
ENDIF ELSE BEGIN
	IF DATATYPE(SHORTNM,1) NE 'String' THEN MESSAGE = $
		'SHORTNM must be a string array.'
	IF MESSAGE NE '' THEN GOTO, HANDLE_ERROR
;
	IF DATATYPE(LONGNM,1) NE 'String' THEN MESSAGE = $
		'LONGNM must be a string array.'
	IF MESSAGE NE '' THEN GOTO, HANDLE_ERROR
;
	IF N_ELEMENTS(LONGNM) NE N_ELEMENTS(SHORTNM) THEN MESSAGE = $
		'SHORTNM & LONGNM must be string arrays of equal size.'
	IF MESSAGE NE '' THEN GOTO, HANDLE_ERROR
;
	FOR I=0,N_ELEMENTS(SHORTNM)-1 DO BEGIN
		DSPOS = STRPOS(SHORTNM(I), '.')
		DLPOS = STRPOS(LONGNM(I), '.')
		SHLEN = STRLEN(SHORTNM(I))  &  LNLEN = STRLEN(LONGNM(I))
;
		IF DSPOS GT -1 THEN BEGIN
			IF DSPOS EQ 0 THEN MESSAGE = 'Short Filename '+$
			 'nonexistent, must be 1-8 characters long.'
			IF DSPOS GT 8 THEN MESSAGE = 'Short Filename too '+$
			 'long, must be 1-8 characters as per $CCSDS1 '+$
			 'specification.'
			IF SHLEN-DSPOS-1 EQ 0 THEN MESSAGE = 'Short Filename'+$
			 ' extension nonexistent, must be 1-3 characters long.'
			IF SHLEN-DSPOS-1 GT 3 THEN MESSAGE = 'Short Filename'+$
			 ' extension too long, must be 1-3 characters as per'+$
			 ' $CCSDS1 specification.'
		ENDIF ELSE MESSAGE = 'SHORTNM must have a "." in its name.'
;
		IF DLPOS GT -1 THEN BEGIN
			IF DLPOS EQ 0 THEN MESSAGE = 'Long Filename '+$
			 'nonexistent, must be 1-31 characters long.'
			IF DLPOS GT 27 THEN MESSAGE = 'Long Filename too '+$
			 'long, must be 1-26 characters as per $CCSDS2 '+$
			 'specification.'
			IF LNLEN GT 31 THEN MESSAGE = 'Long Filename (plus '+$
			 'extension) too long, must be 1-31 characters as '+$
			 'per $CCSDS2 specification.'
		ENDIF ELSE MESSAGE = 'Recommend using extensions (i.e., '+$
			 '".fits") in the LONGNM name.'
;
		IF MESSAGE NE '' THEN GOTO, HANDLE_ERROR
	ENDFOR
;
	IF N_ELEMENTS(NSFDU_BYTE) EQ 0 THEN MESSAGE = 'The procedure SFDU_'+$
		'BYTES must be ran before running SFDU_MAKE.'
	IF MESSAGE NE '' THEN GOTO, HANDLE_ERROR
;
ENDELSE
IF MESSAGE NE '' THEN GOTO, HANDLE_ERROR
;
; SFDU header information defaults.
;
; ***************************************************************************
; The 3 parameters below tell user that "Only Labelled Objects" will follow.
; ***************************************************************************
CAID       = 'CCSD' ; The Control Authority Identifier (CAID) for the CCSDS.
DDID       = '0001' ; The Data Description ID (DDID): Exchange Data Unit.
DDID_PVL   = '0006' ; The Data Description ID (DDID): PVL description.
DDID_FMAIN = '0009' ; The Data Description ID (DDID): Application Data Unit.
DDID_FNAME = '0003' ; The Data Description ID (DDID): Replacement service.
DDID_FHEAD = '0006' ; The Data Description ID (DDID): PVL description.
CLASS       = 'Z'   ; SFDU Identification Class ("I'm an SFDU"; labels follow).
CLASS_VOLM  = 'V'   ; SFDU ID Class: Volumn preparation data.
CLASS_FMAIN = 'U'   ; SFDU ID Class: Related data objects follow.
CLASS_FNAME = 'R'   ; SFDU ID Class: SFDU reference service: file name.
CLASS_FHEAD = 'K'   ; SFDU ID Class: Instance attributes.
CLASS_DATA  = 'I'   ; SFDU ID Class: Primary Data of Interest.
; ***************************************************************************
SFDU_VERS = '3'     ; Version of SFDU label: Other Delimitation Technique used.
DELIM_TYP_EOF = 'F' ; Delimitation Type: SFDU label ends with a system EOF.
DELIM_TYP_MARK = 'S'; Delimitation Type: FITS file attributes end with MARKER.
SPARE = '0'         ; Currently unused byte position.
MARKER = '00000001' ; Mandatory marker for Version 3, Type F.
; ***************************************************************************
;
; Get information concerning the SFDU registration files.
;
SFDU_GETREG, ADID_SFDU, ERRMSG=MESSAGE
IF MESSAGE NE '' THEN GOTO, HANDLE_ERROR
;
; ***************************************************************************
;
MARK = 'MRK**'			; Prefix for the numeric marker string.
MARK_CNT = 0        		; Label marker counter.
NFILES = N_ELEMENTS(SHORTNM)	; Number of FITS files to be written to tape.
;
; <<<<<<<<<<<<<<<<<<<<<<<<<<<<  INITIAL LINE  >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
;
; Write the initial SFDU line -> I am an SFDU and will end with an EOF.
;
SFDU = CAID + SFDU_VERS + CLASS + DELIM_TYP_EOF + SPARE + DDID + MARKER
;
; Now attach the beginning marker for the "Volume Description" to the end of 
; the previous line.
;
; +++ VOLUME marker (1st in SFDU file):
MARK_CNT = MARK_CNT + 1
STRMCT = STRTRIM(STRING(MARK_CNT+1000),2)
STRMCT = STRMID(STRMCT, 1, 3)
MARK_VOL = MARK + STRMCT
;
IVOLM = -1
FOR I = 0, N_ELEMENTS(ADID_SFDU)-1 DO $		; Find the Volume ADID.
	IF ADID_SFDU(I).NAME EQ 'VOLM' THEN IVOLM = I
SFDU = SFDU + ADID_SFDU(IVOLM).CAID + SFDU_VERS + CLASS_VOLM + $
	DELIM_TYP_MARK + SPARE + ADID_SFDU(IVOLM).DDID + MARK_VOL
;
; <<<<<<<<<<<<<<<<<<<<<<<<<<<  INITIAL COMMENT  >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
;
; Comment concerning the Volume Description that is put at the top of each 
; tape/CD SFDU file.
;
; Retrieve this comment from the SFDU 'VOLM' file.
;
SFDU_VOLHD, ADID_SFDU, VOLMCOM, VOLKWRD
;
; Add the "Volume" comment to the SFDU header.
;
SFDU = [SFDU, VOLMCOM]
;
; <<<<<<<<<<<<<<<<<<<<<<<<<<<  TAPE_INFORMATION  >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
;
; Create a PVL group that tells global information about the tape.
;
IF N_ELEMENTS(NTAPE_BYTE) EQ 0 THEN BEGIN
	MESSAGE = 'The SFDU_BYTES procedure must be run before SFDU_MAKE is '+$
		'run.'
	GOTO, HANDLE_ERROR
ENDIF
; Make a string of the tape file blocking record size.
TAPE_BYTE = STRTRIM(STRING(NTAPE_BYTE),2)
; Get the current UT.
GET_UTC, UTC, ERRMSG=ERRMSG
IF N_ELEMENTS(ERRMSG) GT 0 THEN IF ERRMSG(0) NE '' THEN RETURN, SFDU
; Express the time as a string.
TCREATE = UTC2STR(UTC, ERRMSG=ERRMSG)
IF N_ELEMENTS(ERRMSG) GT 0 THEN IF ERRMSG(0) NE '' THEN RETURN, SFDU
;
; Check that the Volume keywords printed from this file match what is in the
; Volume SFDU registration file.
;
VOLUME_KEYWD = ['VOL_CREATION_TIME', 'VOL_BYTES_RECORD', 'TOTAL_VOL_BYTES', $
	'TOTAL_FITS_FILES', 'TOTAL_FITS_BYTES', 'TOTAL_VMS_BLOCKS']
NVKWD = N_ELEMENTS(VOLKWRD)
FOR I = 0, NVKWD-1 DO BEGIN
	KEYOK = WHERE(VOLKWRD(I) EQ VOLUME_KEYWD)
	IF KEYOK(0) EQ -1 THEN BEGIN
		MESSAGE = 'Volume keyword consistancy check error!  SFDU '+$
			'registered keyword '+VOLKWRD(I)+' not found in '+$
			'SFDU_MAKE.PRO.'
		GOTO, HANDLE_ERROR
	ENDIF
ENDFOR
;
INDENT = '     '
PVLTAPE = [ 'BEGIN_OBJECT = VOLUME_INFORMATION;', $
		INDENT+VOLUME_KEYWD(0)+" = '"+TCREATE+"';", $
		INDENT+VOLUME_KEYWD(1)+' = '+TAPE_BYTE+';']
; Include total file sizes if they were passed.
IF N_ELEMENTS(FILE_SIZE) GT 0 THEN BEGIN
	PVLTAPE = [ PVLTAPE, INDENT+VOLUME_KEYWD(2)+' = ' + $
		STRTRIM(STRING(FILE_SIZE.BYTES),2) + ';', $
		INDENT+VOLUME_KEYWD(3)+' = ' + $
		STRTRIM(STRING(NFILES),2) + ';', $
		INDENT+VOLUME_KEYWD(4)+' = ' + $
		STRTRIM(STRING(FILE_SIZE.BYTES),2) + ';', $
		INDENT+VOLUME_KEYWD(5)+' = ' + $
		STRTRIM(STRING(FILE_SIZE.VMS_BLOCKS),2) + ';']
ENDIF ELSE BEGIN
	PVLTAPE = [ PVLTAPE, INDENT+VOLUME_KEYWD(4)+' = ' + $
		STRTRIM(STRING(NFILES),2) + ';']
ENDELSE
PVLTAPE = [ PVLTAPE, 'END_OBJECT;']
;
; Close volume marker, INITIAL open main file (names + FITS header) marker, 
; open file name marker.
;
MARKIT = CAID + '$$MARKER' + MARK_VOL	; Close Volume marker.
;
; +++ MAIN file marker:
MARK_CNT = MARK_CNT + 1
ASTRMCT = STRTRIM(STRING(MARK_CNT+1000),2)
ASTRMCT = STRMID(ASTRMCT, 1, 3)
MARK_MAIN = MARK + ASTRMCT
MARKIT = MARKIT + CAID + SFDU_VERS + CLASS_FMAIN + DELIM_TYP_MARK + SPARE + $
	DDID_FMAIN + MARK_MAIN		; Open MAIN file marker.
;
; +++ File NAME marker:
MARK_CNT = MARK_CNT + 1
BSTRMCT = STRTRIM(STRING(MARK_CNT+1000),2)
BSTRMCT = STRMID(BSTRMCT, 1, 3)
MARK_NAME = MARK + BSTRMCT
MARKIT = MARKIT + CAID + SFDU_VERS + CLASS_FNAME + DELIM_TYP_MARK + SPARE + $
	DDID_FNAME + MARK_NAME		; Open filename marker.
;
; Add the tape information PVL and the above markers to the SFDU.
;
SFDU = [SFDU, PVLTAPE, MARKIT]
;
; <<<<<<<<<<<<<<<<<<<<<<<<<<<  FILE_INFORMATION  >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
;
; Create the SFDU lines that contain the file information.
;
IFITS = -1
FOR I = 0, N_ELEMENTS(ADID_SFDU)-1 DO $		; Find the FITS File ADID.
	IF ADID_SFDU(I).NAME EQ 'FITS' THEN IFITS = I
BLANK = ' '
TYPELINE = 'REFERENCETYPE = CCSDS0;'
FILE_LAB = 'LABEL = ' + ADID_SFDU(IFITS).CAID + SFDU_VERS + CLASS_DATA + $
	DELIM_TYP_EOF + SPARE + ADID_SFDU(IFITS).DDID + MARKER + ';'
;
ICATG = -1
FOR I = 0, N_ELEMENTS(ADID_SFDU)-1 DO $		; Find Catalog Object ADID.
	IF ADID_SFDU(I).NAME EQ 'CATG' THEN ICATG = I
;
FOR I = 0, NFILES-1 DO BEGIN
;
; Make the file name SFDU header line.  Note that the first file on the tape
; is the SFDU header file and that this file is labelled as tape file #1.
; Hence the first FITS file on the tape is tape file #2.  Capitalize the short
; name for DOS applications.
;
	FILELINE = STRARR(4)
	FILELINE(0) = 'REFERENCE = ( "CCSDS1 = ' + $
		STRUPCASE(STRTRIM(SHORTNM(I),2))  + '"'
	FILELINE(1) = '            , "CCSDS2 = ' + STRTRIM(LONGNM(I),2)   + '"'
	FILELINE(2) = '            , "CCSDS3 = ' + STRTRIM(STRING(I+2),2) + '"'
	FILELINE(3) = '            );'
;
; Close name marker, open FITS header marker.
;
	MARK_SEP = CAID + '$$MARKER' + MARK_NAME		; Close name marker.
;
; +++ FITS file HEADER marker:
	MARK_CNT = MARK_CNT + 1
	CSTRMCT = STRTRIM(STRING(MARK_CNT+1000),2)
	CSTRMCT = STRMID(CSTRMCT, 1, 3)
	MARK_FHEAD = MARK + CSTRMCT
; Open FITS header marker.
	MARK_SEP = MARK_SEP + ADID_SFDU(ICATG).CAID + SFDU_VERS + $
		CLASS_FHEAD + DELIM_TYP_MARK + SPARE + $
		ADID_SFDU(ICATG).DDID + MARK_FHEAD
;
; Get FITS header!
;
	TMPHEAD = PVLHEAD(I,*)
	TMPHEAD = REFORM(TMPHEAD)
	IHEAD = WHERE(TMPHEAD NE '')
	PVLHEADPR = TMPHEAD(IHEAD)
	PVLHEADPR = ['BEGIN_OBJECT = FITS_HEADER;', PVLHEADPR, 'END_OBJECT;']
;
; Close FITS header marker, close main marker.
;
	MARK_END = CAID + '$$MARKER' + MARK_FHEAD	; Close FITS HD marker.
	MARK_END = MARK_END + CAID + '$$MARKER' + MARK_MAIN  ; Close MAIN.
;
; Open next main marker and open next name marker (DO NOT OPEN THESE IF ON 
; LAST FILE).
;
	IF I LT NFILES-1 THEN BEGIN
; +++ MAIN file markers:
		MARK_CNT = MARK_CNT +1
		ASTRMCT = STRTRIM(STRING(MARK_CNT+1000),2)
		ASTRMCT = STRMID(ASTRMCT, 1, 3)
		MARK_MAIN = MARK + ASTRMCT
; Open file MAIN marker.
		MARK_NEW = CAID + SFDU_VERS + CLASS_FMAIN + DELIM_TYP_MARK + $
			SPARE + DDID_FMAIN + MARK_MAIN
;
; +++ File NAME markers:
		MARK_CNT = MARK_CNT +1
		BSTRMCT = STRTRIM(STRING(MARK_CNT+1000),2)
		BSTRMCT = STRMID(BSTRMCT, 1, 3)
		MARK_NAME = MARK + BSTRMCT
; Open file NAME marker.
		MARK_NEW = MARK_NEW + CAID + SFDU_VERS + CLASS_FNAME + $
			DELIM_TYP_MARK + SPARE + DDID_FNAME + MARK_NAME
	ENDIF ELSE MARK_NEW = ''
;
; Add all to the output string array [NOTE: the ordering is important!].
;
	SFDU = [SFDU, TYPELINE, FILE_LAB, FILELINE, MARK_SEP, PVLHEADPR, $
		MARK_END, MARK_NEW]
ENDFOR
SFDU = SFDU(WHERE(SFDU NE ''))
;
; NOTE:  The EOF marker ends the SFDU header.
;
; ****************************************************************************
;
; Figure out how many bytes this SFDU header file will take and re-insert this
; number back into the global tape PVL.
;
NBY_SFDU = N_ELEMENTS(SFDU)*80
NBL_SFDU = NBY_SFDU/NTAPE_BYTE
IF NBY_SFDU MOD NTAPE_BYTE GT 0 THEN NBL_SFDU = NBL_SFDU + 1
NBY_SFDU = NTAPE_BYTE*NBL_SFDU
;
FOR I=0,N_ELEMENTS(SFDU)-1 DO BEGIN
	IF STRPOS(SFDU(I),'TOTAL_TAPE_BYTES = ') GT -1 THEN SFDU(I) = $
		'     TOTAL_TAPE_BYTES = ' + $
		STRTRIM(STRING(FILE_SIZE.BYTES+NBY_SFDU),2) + ';'
ENDFOR
;
; Make each line extend out to 79 bytes (note that a <CR> is automatically 
; attached to the end of each line, i.e., character 80, when the header is
; written to the tape (IDLTP) or file (UNIXDD).  Note that according to CCSDS
; protocol, "standard" SFDU labels (i.e., those beginning with CCSD or NSSD)
; can have printable characters out to 80 bytes with no <CR> closing the line.
; Since the array nature of an IDL string array will always attach a "<CR>"
; at the end of each string, I don't allow more than 3 (= 60 bytes) of these 
; "standard" labels on one line and fill this line with blanks out to column
; 79 (with byte 80 containing a <CR>).
;
FOR I=0,N_ELEMENTS(SFDU)-1 DO BEGIN
	FLEN = STRLEN(SFDU(I))
	IF FLEN LT NLINE_BYTE-1 THEN BEGIN
		FOR J=FLEN,NLINE_BYTE-2 DO SFDU(I) = SFDU(I) + BLANK
	ENDIF
ENDFOR
;
;
IF N_ELEMENTS(ERRMSG) GT 0 THEN ERRMSG=MESSAGE
RETURN, SFDU  ;  Return to the calling program with no errors.  <<<<<<<<<<<<<
;
; Error handling portion of the procedure.
;
HANDLE_ERROR:
IF N_ELEMENTS(ERRMSG) EQ 0 THEN MESSAGE, MESSAGE
ERRMSG = MESSAGE
RETURN, SFDU
;
END
