;+
; Project     :	SOHO - CDS
;
; Name        :	SFDU_OS
;
; Purpose     :	Program to load the operating system info into variables.
;
; Explanation :	Load the operating system ID and supplemental OS parameters 
;		into convenient parameters.  These parameters are needed for 
;		the SFDU and MAGTAPE procedures.  Eliminates redundancies in 
;		determining parameters concerning the operating system 
;		currently being used that was introduced in the various 
;		procedures written for the SFDU tape archiving.
;
; Use         :	SFDU_OS, OS_USED, DIR_COM, DIR_DELIM, SUFFIX, QVERS, TYPE
;
; Inputs      :	None.
;
; Opt. Inputs :	None.
;
; Outputs     :	OS_USED: A string containing one of the following operating
;			 systems: 'VMS', 'UNIX', 'DOS', 'MACOS'.
;		DIR_COM: A string containing the operating system command to
;			 do a directory listing of files.
;		DIR_DELIM: A string that contains a delimiter that is unique
;			 to a given operating system.
;		SUFFIX:	 A string that contains the FITS file name suffix for 
;			 the various operating systems ('.fits' for VMS & Unix,
;			 '.fit' for DOS & MacOS).
;		QVERS:	 A floating real value that contains the version of IDL
;			 that is being used currently.
;		TYPE:	 Tape read/write access type:
;				'IDLTP':  Use the standard IDL tape reading
;					  and writing commands.
;				'UNIXDD': Use the Unix "dd" command to read
;					  and write the tape.
;
;
; Opt. Outputs:	None.
;
; Keywords    :	ERRMSG:	If defined and passed, then any error messages will 
;			be returned to the user in this parameter rather than
;			being handled by the IDL MESSAGE utility.  If no 
;			errors are encountered, then a null string is 
;			returned.  In order to use this feature, the string 
;			ERRMSG must be defined first, e.g.,
;
;				ERRMSG = ''
;				SFDU_OS, OS_USED, DIR_COM, DIR_DELIM, SUFFIX, $
;					QVERS, TYPE, ERRMSG=ERRMSG)
;				IF ERRMSG(0) NE '' THEN ...
;
; Calls       :	None.
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Category    :	Data Handling, I/O, FITS, SFDU, Generic
;
; Prev. Hist. :	None.
;
; Written     :	Donald G. Luttermoser, GSFC/ARC, 13 March 1995
;
; Modified    :	Version 1, Donald G. Luttermoser, GSFC/ARC, 13 March 1995
;			Initial program.
;		Version 2, Donald G. Luttermoser, GSFC/ARC, 26 May 1995
;			Modified document header.
;
; Version     :	Version 1,  26 May 1995
;-
;
PRO SFDU_OS, OS_USED, DIR_COM, DIR_DELIM, SUFFIX, QVERS, TYPE, ERRMSG=ERRMSG
;
ON_ERROR, 2	  ; Return to caller if error is encountered.
MESSAGE = ''	  ; Set to non-null string if error is encountered.
;
; Check input parameters.
;
IF N_PARAMS() EQ 0 THEN BEGIN
	MESSAGE = 'Syntax:  SFDU_OS, OS_USED, DIR_COM, DIR_DELIM, SUFFIX, '+$
		'QVERS, TYPE'
	GOTO, HANDLE_ERROR
ENDIF
;
OPERSYS = ['VMS',       'UNIX',  'DOS',  'MACOS']
TYPETP  = ['IDLTP',     'IDLTP', 'NONE', 'NONE' ]
DRDELIM = [']',         '/',     '\'  ,  '?'    ]
SUFFARR = ['.fits',     '.fits', '.fit', '.fit' ]
DIRCOM  = ['dir/col=1', 'ls',    'dir',  '?'    ]
;
; Check operating system to see which is used.
;
OS_USED=STRUPCASE(!VERSION.OS)
QOS=WHERE(OPERSYS EQ OS_USED)
; Assume unrecognized OS is Unix-based.
IF QOS(0) EQ -1 THEN QOS = 1 ELSE QOS = QOS(0)
;
OS_USED = OPERSYS(QOS)
DIR_DELIM = DRDELIM(QOS)
DIR_COM = DIRCOM(QOS)
SUFFIX = SUFFARR(QOS)
;
; Check the IDL version number.  If less than Version 3.1, the tape is read
; with the Unix command:  dd if=/dev/nrmt0 of=file.name bs=(bytes/record).
;
QVERS=FLOAT(STRMID(!VERSION.RELEASE,0,3))
IF (QVERS LT 3.1) AND (OS_USED EQ 'UNIX') THEN TYPE = 'UNIXDD' ELSE $
	TYPE = TYPETP(QOS)
;
IF (OS_USED EQ 'DOS') OR (OS_USED EQ 'MACOS') THEN BEGIN
  MESSAGE = 'The '+OS_USED+' operating system is currently not supported.'
  GOTO, HANDLE_ERROR
ENDIF
;
; Return with no errors.
;
IF N_ELEMENTS(ERRMSG) GT 0 THEN ERRMSG = MESSAGE
RETURN
;
; Error handling portion of the procedure.
;
HANDLE_ERROR:
IF N_ELEMENTS(ERRMSG) EQ 0 THEN MESSAGE, MESSAGE
ERRMSG = MESSAGE
RETURN
;
END
