;+
; Project     :	SOHO - CDS
;
; Name        :	SFDU_READ
;
; Purpose     :	Program to read the SFDU file off of tape and return filenames.
;
; Explanation :	Program to read the Standard Formatted Data Unit file (1st
;		file off of the tape).  This procedure is designed to be 
;		accessed only from the procedure MAGTAPE.
;
;		The SoHO/SOWG has selected two standards relating the storage
;		of scientific data: the Standard Formatted Data Unit (SFDU)
;		and the Flexible Image Transport System (FITS).  FITS is a
;		standard for formatting data into files in a computer-
;		independent and self-describing fashion, whereas SFDU is a 
;		standard for documenting the contents and format of a data
;		file, and for outlining the interdependencies of collections
;		of files.
;
;		SFDU works by associating a 20 byte label with each file which 
;		serves as a reference to documentation about the file format.
;		This label can be prepended to the file, in which case it is
;		known as an "attached" SFDU label.  Otherwise, it can be 
;		contained in a separate file on the archive medium, with a 
;		reference to the file it applies to, which is a "detached"
;		label.  Note that each line in the SFDU header is formatted
;		into 80 byte (i.e., character) lengths, with the 80th byte
;		containing a <new line> character.  Besided the standard SFDU
;		label (20-byte) lines, there are also comment and keyword 
;		(PVL = Parameter Value Language) lines in the SFDU header.  
;		Such lines end with a ";" (i.e., PVL) or are enclosed by 
;		"/* */" (i.e., comments) similar to C.  All of the standard 
;		20-byte SFDU labels either "CCSD" or "NSSD".
;
; Use         :	SFDU_READ, UNIT, TAPENM, TYPEREAD, SFDU, SHORTNM, LONGNM, $
;			TPLOCNM [, XWSTR]
;
; Inputs      :	UNIT:	Tape unit number.  Must be set by the following
;			commands prior to entering IDL:
;			VMS:	ALLOCATE MUA0: (assuming MUA0: is a tape drive)
;				MOUNT/FOR MUA0:
;				DEFINE MT1 MUA0: (in this example, UNIT = 1,
;						  possible values: MT0 ... MT9,
;						  relate to UNIT = 0 ... 9)
;			Unix:	setenv MT0 /dev/nrmt0  (assuming /dev/nrmt0
;							is a tape drive, in 
;							this example, UNIT=0)
;		TAPENM:	Tape drive name as determined from the GETENV command
;			in the procedure MAGTAPE.
;
;		TYPEREAD: Tape accessing procedure:
;			IDLTP:	Use the IDL style tape accessing commands.
;			UNIXDD:	Use the Unix "mt" & "dd" commands to drive the
;				tape.
;
;		TPLOCNM:  On input, contains the SUFFIX (".fits" or ".fit") to
;			  be used with the the "tape location names", which 
;			  are returned through this variable on output.
;
; Opt. Inputs :	SFDU:	Sent from previous call to SFDU_READ from routine
;			XWINTAPE which drives MAGTAPE.
;
;		XWSTR:	A string array that contains informational text 
;			concerning the tape access.  These string are printed
;			either to the screen or to the FILENAME (set internally
;			to XWIDGET) widget if XWINTAPE is driving this routine.
;
; Outputs     :	SFDU:	  A string array that contains the entire SFDU header.
;		SHORTNM:  A string array that contains all of the "short" file
;			  names in the SFDU header.  These names are meant to
;			  be used in the DOS and MacOS environments.
;		LONGNM:   A string array that contains all of the "long" file
;			  names in the SFDU header.  These names are meant to
;			  be used in the UNIX and VMS environments.
;		TPLOCNM:  A string array that contains the "tape location
;			  name" (i.e. "tapennnn.fit(s)) of the files on the 
;			  tape.
;
; Opt. Outputs:	XWSTR:	A string array that contains informational text 
;			concerning the tape access.  These string are printed
;			either to the screen or to the FILENAME (set internally
;			to XWIDGET) widget if XWINTAPE is driving this routine.
;			Note that SFDU_READ adds text to this array which is
;			passed back to the calling routine.
;
; Keywords    :	XWIDGET: If set, the widget program XWINTAPE is driving this
;			procedure and any text that is normally printed to the
;			screen will instead be printed to the widget.  If
;			used, this keyword passes the value of the FILENAME
;			(set internally to XWIDGET) widget.
;
;		ERRMSG:	If defined and passed, then any error messages will 
;			be returned to the user in this parameter rather than
;			being handled by the IDL MESSAGE utility.  If no 
;			errors are encountered, then a null string is 
;			returned.  In order to use this feature, the string 
;			ERRMSG must be defined first, e.g.,
;
;				ERRMSG = ''
;				SFDU_READ, 1, TAPENM, SFDU, SNAM, LNAM, $
;					ERRMSG=ERRMSG
;				IF ERRMSG(0) NE '' THEN ...
;
; Calls       :	DATATYPE, REWIND, SKIPF, TAPRD
;
; Common      :	SFDU_BYTES
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Category    :	Data Handling, I/O, FITS, SFDU, Generic
;
; Prev. Hist. :	None.
;
; Written     :	Donald G. Luttermoser, GSFC/ARC, 30 March 1995
;
; Modified    :	Version 1, Donald G. Luttermoser, GSFC/ARC, 30 March 1995
;			Initial program.
;		Version 2, Donald G. Luttermoser, GSFC/ARC, 28 April 1995
;			Modified the tape blocking factor to NTAPE_BYTE =
;			2880*BLFAC.
;		Version 3, Donald G. Luttermoser, GSFC/ARC, 18 May 1995
;			Added keyword XWIDGET, modified how the SFDU is 
;			assembled once it is read off of the tape.
;		Version 4, Donald G. Luttermoser, GSFC/ARC, 19 December 1995
;			Modified the detection software for filenames based
;			on the new CCSDS protocol.
;
; Version     :	Version 4,  19 December 1995
;-
;
PRO SFDU_READ, UNIT, TAPENM, TYPEREAD, SFDU, SHORTNM, LONGNM, TPLOCNM, $
	XWSTR, XWIDGET=XWIDGET, ERRMSG=ERRMSG
;
COMMON SFDU_BYTES, NSFDU_BYTE, NREC_PER_LINE, NTAPE_BYTE, NLINE_BYTE
;
ON_ERROR, 2	  ; Return to caller if error is encountered.
MESSAGE = ''	  ; Set to non-null string if error is encountered.
;
; Check input parameters.
;
IF N_PARAMS() LT 7 THEN BEGIN
	MESSAGE = 'Syntax:  SFDU_READ, UNIT, TAPENM, TYPEREAD, SFDU, '+$
		'SHORTNM, LONGNM, TPLOCNM [, XWSTR]'
ENDIF ELSE BEGIN
	IF DATATYPE(UNIT,1) NE 'Integer' THEN MESSAGE = $
		'UNIT must be an integer in the range: 0 - 9.'
	IF MESSAGE NE '' THEN GOTO, HANDLE_ERROR
;
	IF (UNIT LT 0) OR (UNIT GT 9) THEN MESSAGE = $
	  'UNIT must be an integer in the range: 0 - 9.'
	IF MESSAGE NE '' THEN GOTO, HANDLE_ERROR
;
	IF DATATYPE(TAPENM,1) NE 'String' THEN MESSAGE = $
		'TAPENM must be a string and contain the device name of the '+$
		'tape drive.'
	IF MESSAGE NE '' THEN GOTO, HANDLE_ERROR
;
	IF DATATYPE(TPLOCNM,1) NE 'String' THEN MESSAGE = $
		'TPLOCNM must be a string and contain either ".fits" or '+$
		'".fit".'
	IF MESSAGE NE '' THEN GOTO, HANDLE_ERROR
;
	IF N_ELEMENTS(NSFDU_BYTE) EQ 0 THEN MESSAGE = 'The procedure SFDU_'+$
		'BYTES must be ran before running SFDU_READ.'
	IF MESSAGE NE '' THEN GOTO, HANDLE_ERROR
;
	IF DATATYPE(TYPEREAD,1) NE 'String' THEN MESSAGE = $
		"TYPEREAD must be a string: 'IDLTP' or 'UNIXDD'."
ENDELSE
IF MESSAGE NE '' THEN GOTO, HANDLE_ERROR
;
; Initialize internal parameters from variables passed in SFDU_BYTES.
;
SFDU_BYTE = STRTRIM(STRING(NSFDU_BYTE),2)
LINE_BYTE = STRTRIM(STRING(NLINE_BYTE),2)
TAPE_BYTE = STRTRIM(STRING(NTAPE_BYTE),2)
;
; Is the XWINTAPE widget driving this program?
;
IF KEYWORD_SET(XWIDGET) THEN QWIDGET = 1 ELSE QWIDGET = 0
IF N_ELEMENTS(XWSTR) EQ 0 THEN XWSTR = 'SFDU_READ: Rewinding tape ...' $
	ELSE XWSTR = ['SFDU_READ: Rewinding tape ...', XWSTR]
;
; Read the header file depending on the operating system used.
;
;ON_IOERROR, EOT
SUFFIX = TPLOCNM(0)
IF (SUFFIX NE '.fits') AND (SUFFIX NE '.fit') THEN SUFFIX = '.fit'
;
; If SFDU has already been read from XWINTAPE, then get file names from it.
;
IF N_ELEMENTS(SFDU) NE 0 THEN BEGIN
	IF SFDU(0) NE 'NONE' THEN GOTO, GOT_HEAD
ENDIF
;
CASE TYPEREAD OF
;
; Use IDL tape access commands.
;
	'IDLTP': BEGIN
		NREC = 1000	; Assume NREC 80-byte lines in SFDU header.
		SFDU = STRARR(NREC)
		REC_NUM = 0
		HDBYTE = BYTARR(NTAPE_BYTE)
		IF QWIDGET EQ 1 THEN WIDGET_CONTROL, XWIDGET, SET_VALUE=XWSTR $
			ELSE PRINT, XWSTR(0)
		REWIND, UNIT	; Make sure the tape is at the beginning.
		XWSTR = ['SFDU_READ: Reading SFDU header file from tape ...', $
			XWSTR]
		IF QWIDGET EQ 1 THEN WIDGET_CONTROL, XWIDGET, SET_VALUE=XWSTR $
			ELSE PRINT, XWSTR(0)
BIG_HEAD:
; Read the next NTAPE_BYTE bytes on the tape.
		TAPRD, HDBYTE, UNIT
;
; !ERR = -4 if end-of-file encountered, else !ERR = # of bytes read.
;
		IF !ERR NE NTAPE_BYTE THEN GOTO, GOT_HEAD
;
		FOR J=0,NTAPE_BYTE-1,NLINE_BYTE DO BEGIN
			FOR K=0,NLINE_BYTE-1 DO SFDU(REC_NUM) = $
				SFDU(REC_NUM) + STRING(HDBYTE(J+K))
			REC_NUM = REC_NUM + 1
			IF REC_NUM EQ NREC THEN BEGIN
				SFDU = [SFDU, STRARR(1000)]
				NREC = NREC + 1000
			ENDIF
		ENDFOR
;
; If too many lines are seen, something is wrong.
;
		IF NREC GE 32000 THEN BEGIN
			MESSAGE = 'SFDU header read error -> too many ' + $
				'lines found.'
			GOTO, HANDLE_ERROR
		ENDIF
;
; Read the next tape record.
;
		GOTO, BIG_HEAD
		END
;
; Use Unix "dd" to read first file.
;
	'UNIXDD':  BEGIN
		IF QWIDGET EQ 1 THEN WIDGET_CONTROL, XWIDGET, SET_VALUE=XWSTR $
			ELSE PRINT, XWSTR(0)
		SPAWN,'mt -f ' + TAPENM + ' rewind'
		WAIT, 5
		XWSTR = ['SFDU_READ: Reading SFDU header file from tape ...', $
			XWSTR]
		IF QWIDGET EQ 1 THEN WIDGET_CONTROL, XWIDGET, SET_VALUE=XWSTR $
			ELSE PRINT, XWSTR(0)
		SPAWN,'dd if=' + TAPENM + ' of=sfduhd.tmp ibs=' + TAPE_BYTE +$
			' obs=' + LINE_BYTE
;
		START_REC = 0
		OPENR, URD, 'sfduhd.tmp', /GET_LUN
		   WHILE NOT EOF(URD) DO BEGIN
;
			INLINE = ''
			READF, URD, INLINE
;
			LLEN = STRLEN(INLINE)
			IF LLEN GT NLINE_BYTE THEN BEGIN
				MESSAGE = 'Error in reading SFDU header.'
				GOTO, HANDLE_ERROR
			ENDIF
;
			IF START_REC EQ 0 THEN SFDU = INLINE ELSE $
				SFDU = [SFDU, INLINE]
			START_REC = START_REC + 1
		   ENDWHILE
		FREE_LUN,URD
		WAIT, 4  ; Wait 4 seconds to allow IDL to release the file.
		SPAWN,' rm -f sfduhd.tmp'
		END
;
; Until the DOS and MacOS versions are installed, getting this far results in
; an error.  Note that currently, this procedure should not get to this point.
;
	ELSE:  MESSAGE = 'Error in determining operating system in SFDU_READ.'
;
ENDCASE
IF MESSAGE NE '' THEN GOTO, HANDLE_ERROR
;
GOT_HEAD:
;
; Trim trailing "blanks", "line feeds" (character 80), and blank lines from 
; the SFDU.
;
FOR I=0,N_ELEMENTS(SFDU)-1 DO SFDU(I) = STRMID(SFDU(I),0,79)
SFDU = STRTRIM(SFDU)
SFDU = SFDU(WHERE(SFDU NE '', NHEAD))
;
; Get the filenames on the tape.
;
QSTART = 0
QFNAME = 0
FOR I=0,NHEAD-1 DO BEGIN
	IF STRPOS(SFDU(I), 'REFERENCE = (') GT -1 THEN BEGIN
		QFNAME = 1
		QSTART = QSTART + 1
	ENDIF
	IF (QFNAME EQ 1) AND (STRPOS(SFDU(I), ');') GT -1) THEN QFNAME = 0
; Get the names:
	IF QFNAME EQ 1 THEN BEGIN
		QSH = STRPOS(SFDU(I), '"CCSDS1 =')
		QLG = STRPOS(SFDU(I), '"CCSDS2 =')
		QCT = STRPOS(SFDU(I), '"CCSDS3 =')
; >>> Short name:
		IF QSH GT -1 THEN BEGIN
			QM1 = STRPOS(SFDU(I), '"', QSH+9)
			IF QSTART EQ 1 THEN SHORTNM = $
				STRTRIM(STRMID(SFDU(I), QSH+9, QM1-QSH-9), 2) $
			ELSE SHORTNM = [SHORTNM, $
				STRTRIM(STRMID(SFDU(I), QSH+9, QM1-QSH-9),2) ]
		ENDIF
; >>> Long name:
		IF QLG GT -1 THEN BEGIN
			QM2 = STRPOS(SFDU(I), '"', QLG+9)
			IF QSTART EQ 1 THEN LONGNM = $
				STRTRIM(STRMID(SFDU(I), QLG+9, QM2-QLG-9), 2) $
			ELSE LONGNM  = [LONGNM, $
				STRTRIM(STRMID(SFDU(I), QLG+9, QM2-QLG-9),2) ]
		ENDIF
; >>> File count (1st FITS file is tape file #2):
		IF QCT GT -1 THEN BEGIN
			QM3 = STRPOS(SFDU(I), '"', QCT+9)
			IF QSTART EQ 1 THEN BEGIN
				CNT = FIX(STRMID(SFDU(I), QCT+9, QM3-QCT-9))
				TPLOCNM	= 'tape' + STRMID(STRING(CNT+10000, $
					FORMAT='(I5)'), 1, 4) + SUFFIX
			ENDIF ELSE BEGIN
				CNT0 = FIX(STRMID(SFDU(I), QCT+9, QM3-QCT-9))
				CNT = [CNT, CNT0]
				TPLOCNM	= [TPLOCNM, 'tape' + $
					STRMID(STRING(CNT0+10000, $
					FORMAT='(I5)'),1,4) + SUFFIX]
			ENDELSE
		ENDIF
	ENDIF
ENDFOR
;
; Typically Unix filenames are lower-case.  Since "case" is irrelavent in VMS
; and DOS, make all filenames lower-case.
;
SHORTNM = STRLOWCASE(SHORTNM)
LONGNM  = STRLOWCASE(LONGNM)
TPLOCNM = STRLOWCASE(TPLOCNM)
;
; Check to make sure all of the file names have been read from the SFDU header.
; NOTE: The first FITS file is TAPE file number 2!!!
;
FOR I=0,N_ELEMENTS(CNT)-1 DO IF CNT(I) NE I+2 THEN MESSAGE = $
	'Error in reading filenames from SFDU header at tape file (CCSDS3) #'+$
	STRTRIM(STRING(I+2),2)+'.'
IF MESSAGE NE '' THEN GOTO, HANDLE_ERROR
;
IF N_ELEMENTS(ERRMSG) GT 0 THEN ERRMSG=MESSAGE
RETURN  ;  Return to the calling program with no errors.  <<<<<<<<<<<<<<<<<<<
;
; If End-of-Tape was found while trying to read SFDU header file.
;
;EOT:  MESSAGE = '<End-of-Tape> encountered during SFDU-header file read.'
;
; Error handling portion of the procedure.
;
HANDLE_ERROR:
IF N_ELEMENTS(ERRMSG) EQ 0 THEN MESSAGE, MESSAGE
ERRMSG = MESSAGE
RETURN
;
END
