;+
; Project     :	SOHO - CDS
;
; Name        :	SFDU_WRITE
;
; Purpose     :	Program to write a SFDU file on to a tape.
;
; Explanation :	Program to write a Standard Formatted Data Unit file (1st file
;		on the tape).  This procedure is designed to be accessed only 
;		from the procedure MAGTAPE.  
;
;		The SoHO/SOWG has selected two standards relating the storage
;		of scientific data: the Standard Formatted Data Unit (SFDU)
;		and the Flexible Image Transport System (FITS).  FITS is a
;		standard for formatting data into files in a computer-
;		independent and self-describing fashion, whereas SFDU is a 
;		standard for documenting the contents and format of a data
;		file, and for outlining the interdependencies of collections
;		of files.
;
;		SFDU works by associating a 20 byte label with each file which 
;		serves as a reference to documentation about the file format.
;		This label can be prepended to the file, in which case it is
;		known as an "attached" SFDU label.  Otherwise, it can be 
;		contained in a separate file on the archive medium, with a 
;		reference to the file it applies to, which is a "detached"
;		label.  Note that comment and keyword lines in the SFDU header
;		can extend out to 80 bytes (i.e., characters including 
;		terminating line feed <CR>).  Such lines end with a ";<CR>" 
;		(i.e., keywords) or are enclosed within "/* */" (with ending
;		<CR>) delimiters (i.e., comments) similar to C.  All other 
;		lines are standard 20-byte SFDU labels and begin with either 
;		"CCSD" or "NSSD".
;
;		A standard 20-byte SFDU label might look like this:
;				CCSD3ZF0000100000001
;		See SFDU_MAKE.PRO for a complete description of this format.
;
;		NOTE: This procedure prepares the SDFU for tape writing in
;		80-byte records.
;
; Use         :	SFDU_WRITE, UNIT, TAPENM, TYPEWRITE, SFDU [, XWSTR]
;
; Inputs      :	UNIT:	Tape unit number.  Must be set by the following
;			commands prior to entering IDL:
;			VMS:	ALLOCATE MUA0: (assuming MUA0: is a tape drive)
;				MOUNT/FOR MUA0:
;				DEFINE MT1 MUA0: (in this example, UNIT = 1,
;						  possible values: MT0 ... MT9,
;						  relate to UNIT = 0 ... 9)
;			Unix:	setenv MT0 /dev/nrmt0  (assuming /dev/nrmt0
;							is a tape drive, in 
;							this example, UNIT=0)
;		TAPENM:	String containing the name of the tape drive (needed
;			for UNIXDD type.
;
;		TYPEWRITE: Tape accessing procedure:
;			IDLTP:	Use the IDL style tape accessing commands.
;			UNIXDD:	Use the Unix "mt" & "dd" commands to drive the
;				tape.
;
;		SFDU:	A string array that contains the entire SFDU header.
;
; Opt. Inputs :	XWSTR:	A string array that contains informational text 
;			concerning the tape accessing.  These strings are 
;			printed either to the screen or to the FILENAME widget
;			(internally called XWIDGET) if XWINTAPE is driving
;			this procedure.
;
; Outputs     :	None.
;
; Opt. Outputs:	XWSTR:	A string array that contains informational text 
;			concerning the tape accessing.  These strings are 
;			printed either to the screen or to the FILENAME widget
;			(internally called XWIDGET) if XWINTAPE is driving
;			this procedure.  Note that SFDU_WRITE adds strings to
;			this array and passes it back to the caller.
;
; Keywords    :	XWIDGET: If set, the widget program XWINTAPE is driving this
;			procedure and any text that is normally printed to the
;			screen will instead be printed to the widget.  If used,
;			this keyword passes the value of the FILENAME 
;			(set internally to XWIDGET) widget.
;
;		ERRMSG:	If defined and passed, then any error messages will 
;			be returned to the user in this parameter rather than
;			being handled by the IDL MESSAGE utility.  If no 
;			errors are encountered, then a null string is 
;			returned.  In order to use this feature, the string 
;			ERRMSG must be defined first, e.g.,
;
;				ERRMSG = ''
;				SFDU_WRITE, 1, SFDU, ERRMSG=ERRMSG
;				IF ERRMSG(0) NE '' THEN ...
;
; Calls       :	DATATYPE, SFDU_OS, REWIND, SKIPF, TAPWRT
;
; Common      :	SFDU_BYTES
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Category    :	Data Handling, I/O, FITS, SFDU, Generic
;
; Prev. Hist. :	None.
;
; Written     :	Donald G. Luttermoser, GSFC/ARC, 30 March 1995
;
; Modified    :	Version 1, Donald G. Luttermoser, GSFC/ARC, 30 March 1995
;			Initial program.
;		Version 2, Donald G. Luttermoser, GSFC/ARC, 28 April 1995
;			Modified SFDU blocking structure to NTAPE_BYTE =
;			2880*BLFAC.
;		Version 3, Donald G. Luttermoser, GSFC/ARC, 9 May 1995
;			Added the XWINTAPE keyword.
;
; Version     :	Version 3,  9 May 1995
;-
;
PRO SFDU_WRITE, UNIT, TAPENM, TYPEWRITE, SFDU, XWSTR, XWIDGET=XWIDGET, $
	ERRMSG=ERRMSG
;
COMMON SFDU_BYTES, NSFDU_BYTE, NREC_PER_LINE, NTAPE_BYTE, NLINE_BYTE
;
ON_ERROR, 2	  ; Return to caller if error is encountered.
MESSAGE = ''	  ; Set to non-null string if error is encountered.
;
; Check input parameters.
;
IF N_PARAMS() LT 4 THEN BEGIN
	MESSAGE = 'Syntax:  SFDU_WRITE, UNIT, TAPENM, TYPEWRITE, SFDU [, XWSTR]'
ENDIF ELSE BEGIN
	IF DATATYPE(UNIT,1) NE 'Integer' THEN MESSAGE = $
		'UNIT must be an integer in the range: 0 - 9.'
	IF MESSAGE NE '' THEN GOTO, HANDLE_ERROR
;
	IF (UNIT LT 0) OR (UNIT GT 9) THEN MESSAGE = $
	  'UNIT must be an integer in the range: 0 - 9.'
	IF MESSAGE NE '' THEN GOTO, HANDLE_ERROR
;
	IF DATATYPE(TAPENM,1) NE 'String' THEN MESSAGE = 'Tape Drive name '+$
		' error -- must be a string scalar.'
	IF MESSAGE NE '' THEN GOTO, HANDLE_ERROR
;
	IF N_ELEMENTS(TAPENM) NE 1 THEN MESSAGE = 'Tape Drive name '+$
		' error -- passed string must be a scalar.'
	IF MESSAGE NE '' THEN GOTO, HANDLE_ERROR
;
	IF N_ELEMENTS(TYPEWRITE) NE 1 THEN MESSAGE = 'TYPEWRITE must be a '+$
		" string: 'IDLTP' or 'UNIXDD'."
	IF MESSAGE NE '' THEN GOTO, HANDLE_ERROR
;
	IF DATATYPE(SFDU,1) NE 'String' THEN MESSAGE = 'Improper SFDU header'+$
		' array sent -- must be a string array.'
	IF MESSAGE NE '' THEN GOTO, HANDLE_ERROR
;
	IF N_ELEMENTS(NSFDU_BYTE) EQ 0 THEN MESSAGE = 'The procedure SFDU_'+$
		'BYTES must be ran before running SFDU_WRITE.'
	IF MESSAGE NE '' THEN GOTO, HANDLE_ERROR
;
	IF N_ELEMENTS(SFDU) LE 1 THEN MESSAGE = 'SFDU must be a string array.'
ENDELSE
IF MESSAGE NE '' THEN GOTO, HANDLE_ERROR
;
; Initialize internal parameters based on variables passed in SFDU_BYTE.
;
SFDU_BYTE = STRTRIM(STRING(NSFDU_BYTE),2)
LINE_BYTE = STRTRIM(STRING(NLINE_BYTE),2)
TAPE_BYTE = STRTRIM(STRING(NTAPE_BYTE),2)
;
; See if XWINTAPE is driving this procedure.
;
IF KEYWORD_SET(XWIDGET) THEN QWIDGET = 1 ELSE QWIDGET = 0
IF N_ELEMENTS(XWSTR) EQ 0 THEN XWSTR = '  Rewinding tape ...' $
	ELSE XWSTR = ['  Rewinding tape ...', XWSTR]
;
ON_IOERROR, EOT
CASE TYPEWRITE OF
	'IDLTP':  MESSAGE = ''	; Use IDL tape access commands.
;
	'UNIXDD': MESSAGE = ''	; Use the UNIX tape access commands.
;
; Until the DOS and MacOS versions are installed, getting this far results in
; an error.  Note that currently, this procedure should not get to this point.
;
	ELSE:  MESSAGE = TYPEWRITE+' operating system not supported in '+$
		'SFDU_WRITE.'
;
ENDCASE
IF MESSAGE NE '' THEN GOTO, HANDLE_ERROR
;
; Make sure the tape is at the beginning.
;
IF QWIDGET EQ 1 THEN WIDGET_CONTROL, XWIDGET, SET_VALUE=XWSTR ELSE $
	PRINT, XWSTR(0)
CASE TYPEWRITE OF
	'IDLTP':  REWIND, UNIT  
;
	'UNIXDD': BEGIN
		SPAWN, 'mt -f '+TAPENM+' rewind'
		OPENW, UDD, 'unixdd.sfdu', /GET_LUN ; Open file for Unix write.
		END
ENDCASE
;		
NHEAD = N_ELEMENTS(SFDU)
SFDU_CNT = 0			; ASCII line count (UNIXDD)/char count (IDLTP).
TOT_SEG = NTAPE_BYTE/NLINE_BYTE	; Total number of line segments/tape record.
CR_BYTE = 10B			; Line feed in bytes.
CR_STRG = STRING(CR_BYTE)	; Line feed as string.
SFDU_TAPE = BYTARR(NTAPE_BYTE)	; Data byte array to be loaded to tape.
;
XWSTR = ['  Writing the SFDU header to the tape ...', XWSTR]
IF QWIDGET EQ 1 THEN WIDGET_CONTROL, XWIDGET, SET_VALUE=XWSTR ELSE $
	PRINT, XWSTR(0)
FOR I=0,NHEAD-1 DO BEGIN
	NSFDU = STRLEN(SFDU(I))
	SFDU_TMP = STRARR(NSFDU)
	FOR J=0,NSFDU-1 DO SFDU_TMP(J) = STRMID(SFDU(I),J,1)
;
; Make sure SFDU array line has NLINE_BYTE-1 (=79) bytes in it (byte 80 will
; be a line feed <CR>).  This should never happen since sfdu_make constructs 
; the SFDU header that way.
;
	IF NSFDU NE NLINE_BYTE-1 THEN BEGIN
		MESSAGE = 'Record size in SFDU header (=' + $
			STRTRIM(STRING(NSFDU),2) + ') not equal to ' +$
			STRTRIM(STRING(NLINE_BYTE-1),2) + '.'
		GOTO, HANDLE_ERROR
	ENDIF
;
; Write it to the tape (IDLTP) or to an 80-byte/record file (UNIXDD).
;
	CASE TYPEWRITE OF
		'IDLTP':  BEGIN
;
; SFDU_TMP is an 79-byte (= character) array at this point.  Add a line feed
; <CR> at this point so that the total length is 80 bytes.  Once a new array
; of length NTAPE_BYTE is filled write it to tape.
;
			SFDU_TMP = [SFDU_TMP, CR_STRG]
			SFDU_TAPE(SFDU_CNT:SFDU_CNT+NLINE_BYTE-1) = $
				BYTE(SFDU_TMP(0:NLINE_BYTE-1))
			SFDU_CNT = SFDU_CNT + NLINE_BYTE
; Write the filled array to tape.
			IF SFDU_CNT EQ NTAPE_BYTE THEN BEGIN
				TAPWRT, SFDU_TAPE, UNIT
				SFDU_CNT = 0
			ENDIF
			IF SFDU_CNT GT NTAPE_BYTE THEN BEGIN
				MESSAGE = 'Error in constructing tape ' + $
					' data variable.'
				GOTO, HANDLE_ERROR
			ENDIF
			END
;
		'UNIXDD':  BEGIN
; Dump SFDU line to ASCII file.
			PRINTF, UDD, SFDU(I)
			SFDU_CNT = SFDU_CNT + 1
			END
	ENDCASE
ENDFOR
;
; Add extra blank lines to fill up NTAPE_BYTE buffer.
;
CASE TYPEWRITE OF
	'IDLTP': BEGIN
		NFILL = NTAPE_BYTE - SFDU_CNT
		IF (NFILL MOD NLINE_BYTE) NE 0 THEN BEGIN
			MESSAGE = 'Error in determining blank padding ' + $
				'for SFDU header in UNIXDD mode.'
			GOTO, HANDLE_ERROR
		ENDIF
		IF NFILL NE NTAPE_BYTE THEN BEGIN
			NFILL = NFILL/NLINE_BYTE
			BLNKARR = STRARR(NLINE_BYTE)
			BLNKARR(*) = ' '  &  BLNKARR(NLINE_BYTE-1) = CR_STRG
			FOR I=0,NFILL-1 DO BEGIN
				SFDU_TAPE(SFDU_CNT:SFDU_CNT+NLINE_BYTE-1) = $
					BYTE(BLNKARR(0:NLINE_BYTE-1))
				SFDU_CNT = SFDU_CNT + NLINE_BYTE
			ENDFOR
; Write it to tape.
			TAPWRT, SFDU_TAPE, UNIT
		ENDIF
		END
	'UNIXDD': BEGIN
		NPARTIAL = NLINE_BYTE*SFDU_CNT MOD NTAPE_BYTE
		IF NPARTIAL GT 0 THEN BEGIN
			NFILL = NTAPE_BYTE - NPARTIAL
			IF (NFILL MOD NLINE_BYTE) NE 0 THEN BEGIN
				MESSAGE = 'Error in determining blank ' + $
					'padding for SFDU header in ' + $
					'UNIXDD mode.'
				GOTO, HANDLE_ERROR
			ENDIF
			NFILL = NFILL/NLINE_BYTE
			BLANKS = ' '
			FOR K=1,NLINE_BYTE-2 DO BLANKS = BLANKS + ' '
;
; Add blanks to the Unix script file to fill buffer.
;
			FOR I=0,NFILL-1 DO PRINTF, UDD, BLANKS
		ENDIF
		END
ENDCASE
;
; Close the UNIXDD file and write it to tape or write an <End-of-File> marker 
; on the tape for IDLTP type.
;
CASE TYPEWRITE OF
	'IDLTP':  WEOF, UNIT  ;  Write <End-of-File> marker on tape.
;
	'UNIXDD': BEGIN
		FLUSH, UDD
		FREE_LUN, UDD
		SPAWN, 'dd if=unixdd.sfdu of=' + TAPENM + ' ibs=' + LINE_BYTE $
			+ ' obs=' + TAPE_BYTE + ' ; rm -f unixdd.sfdu'
		END
ENDCASE
;
; Get rid of the trailing blanks (needed for the tape write) for each line of 
; the SFDU and return to caller.
;
SFDU = STRTRIM(SFDU)
IF N_ELEMENTS(ERRMSG) GT 0 THEN ERRMSG=MESSAGE
RETURN  ;  Return to the calling program with no errors.  <<<<<<<<<<<<<
;
; If <End-of-Tape> was found while trying to write SFDU header file.
;
EOT:  MESSAGE = '<End-of-Tape> encountered during writing of SFDU-header file.'
;
; Error handling portion of the procedure.
;
HANDLE_ERROR:
;
; EOF mark so tape is readable despite error (IDLTP) or close the UNIXDD file.
;
CASE TYPEWRITE OF
	'IDLTP':  WEOF, UNIT  ;  Write <End-of-File> marker on tape.
;
	'UNIXDD': FREE_LUN, UDD
ENDCASE
;
IF N_ELEMENTS(ERRMSG) EQ 0 THEN MESSAGE, MESSAGE
ERRMSG = MESSAGE
RETURN
;
END
