;+
; Project     :	SOHO - CDS
;
; Name        :	UNIXDD
;
; Purpose     :	Function for FITS tape I/O using Unix "dd" and "mt" commands.
;
; Explanation :	This program is called from the MAGTAPE procedure, which
;		reads or writes a magnetic tape containing FITS files and
;		(possibly) an SFDU header file.  It is designed to be used
;		for Unix environments running versions of IDL earlier than
;		V3.1.  With IDL Version 3.1 or later, use the TAPRD, TAPWRT,
;		etc. routines written for Unix by Bill Thompson.
;
; Use         :	RESULT = UNIXDD( TAPENM, TYPE, OLD_DIR, FILES, NAMES [, XWSTR] )
;
; Inputs      :	TAPENM:	A string containing the name of the tape drive.
;
;		TYPE:	Tape access type: 'read' or 'write'.
;
;		OLD_DIR: Name of the directory where the process started.
;			Required for the creation of the Unix script file.
;
;		FILES:  For type 'read', contains the tape file numbers 
;			[integer] where the first FITS file number on the tape 
;			is "1".
;			For type 'write', contains the names [string] of the 
;			disk files.
;
;		NAMES:	File names of the FITS files read off of the tape.
;			Required only for TYPE 'read'.  Note that 
;			N_ELEMENTS(NAMES) >= N_ELEMENTS(FILES).
;
; Opt. Inputs :	XWSTR:	A string array that contains informational text 
;			concerning the tape I/O.  These strings are either
;			printed to the screen or to the FILENAME widget (set
;			internally to XWIDGET) if the XWINTAPE procedure is
;			driving this procedure.
;
; Outputs     :	An integer greater than 0 if successful, -1 if an error occurs.
;
; Opt. Outputs:	XWSTR:	A string array that contains informational text 
;			concerning the tape I/O.  These strings are either
;			printed to the screen or to the FILENAME widget (set
;			internally to XWIDGET) if the XWINTAPE procedure is
;			driving this procedure.  Note that UNIXDD adds 
;			additional strings to this array.
;
; Keywords    :	BLFAC:	Blocking factor (1-10) for the 2880 bytes/record files.
;			Default is "1".
;
;		OUTPUT:	If set, information will be sent to the file associated
;			with the unit number OUTPUT.
;
;		KEYWORD: Add the filename to FITS header with the label 
;			KEYWORD.  Note that for TYPE 'write', the disk file 
;			is rewritten with this new label in the header.
;
;		XWIDGET: Tells UNIXDD that XWINTAPE is driving this procedure.
;			If so, messages stored in XWSTR will be printed to the
;			FILENAME widget (called XWIDGET internally) instead of
;			screen.  NOTE: Much of the informational text output
;			is written from a UNIX shell script.  Since this 
;			script knows nothing of the widget, it is unable to
;			dump the output to the widget.  Instead UNIXDD informs
;			the operator through the widget that informational
;			text is being written to the text screen.
;
;		ERRMSG:	If defined and passed, then any error messages will 
;			be returned to the user in this parameter rather than
;			being handled by the IDL MESSAGE utility.  If no 
;			errors are encountered, then a null string is 
;			returned.  In order to use this feature, the string 
;			ERRMSG must be defined first, e.g.,
;
;				ERRMSG = ''
;				UNIXDD, TAPENM, 'read', OLD_DIR, FILES, NAMES,$
;					ERRMSG=ERRMSG
;				IF ERRMSG(0) NE '' THEN ...
;
; Calls       :	DATATYPE
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	If the KEYWORD keyword is set for TYPE 'write', then the
;		current FITS disk file is rewritten with the included KEYWORD.
;
; Category    :	Data Handling, I/O, FITS, SFDU, Unix-based
;
; Prev. Hist. :	None.
;
; Written     :	Donald G. Luttermoser, GSFC/ARC, 22 March 1995
;
; Modified    :	Version 1, Donald G. Luttermoser, GSFC/ARC, 22 March 1995
;			Initial program.
;		Version 2, Donald G. Luttermoser, GSFC/ARC, 19 April 1995
;			Modified the Unix script file format.
;		Version 3, Donald G. Luttermoser, GSFC/ARC, 9 May 1995
;			Added the XWIDGET keyword.
;
; Version     :	Version 3,  9 May 1995.
;
;-
;
FUNCTION UNIXDD, TAPENM, TYPE, OLD_DIR, FILES, NAMES, XWSTR, BLFAC=BLFAC, $
	OUTPUT=OUTPUT, KEYWORD=KEYWORD, XWIDGET=XWIDGET, ERRMSG=ERRMSG
;
ON_ERROR, 2   ; Return to caller if error is encountered.
MESSAGE = ''  ; Set to non-null string if error is encountered.
;
; Check keywords.
;
IF NOT KEYWORD_SET(BLFAC) THEN BLFAC = 1
IF KEYWORD_SET(OUTPUT) THEN QOUT = 'Y' ELSE QOUT = 'N'
IF QOUT EQ 'Y' THEN UWR = OUTPUT
IF KEYWORD_SET(XWIDGET) THEN QWIDGET = 1 ELSE QWIDGET = 0
;
; Check input parameters.
;
;
IF N_PARAMS() LT 4 THEN BEGIN
; Note that NAMES only has to be sent for type READ!
	MESSAGE = 'Syntax:  RESULT = UNIXDD(TAPENM, TYPE, OLD_DIR, FILES, '+$
		'NAMES [, XWSTR])'
ENDIF ELSE BEGIN
	IF DATATYPE(TYPE,1) NE 'String' THEN MESSAGE = $
		"TYPE must be a string: 'read' or 'write'."
	IF MESSAGE NE '' THEN GOTO, HANDLE_ERROR
;
	IF DATATYPE(TAPENM,1) NE 'String' THEN MESSAGE = $
		'TAPENM must be a string containing the name of the tape drive.'
	IF MESSAGE NE '' THEN GOTO, HANDLE_ERROR
;
	IF (STRUPCASE(TYPE) NE 'READ') AND (STRUPCASE(TYPE) NE 'WRITE') THEN $
	  MESSAGE = "TYPE must be a string: 'read' or 'write'."
	IF MESSAGE NE '' THEN GOTO, HANDLE_ERROR
;
	IF (STRUPCASE(TYPE) EQ 'READ') AND (DATATYPE(FILES,1) NE 'Integer') $
	  THEN MESSAGE = "For TYPE = 'read', input file names must be an " + $
		'integer array.'
	IF MESSAGE NE '' THEN GOTO, HANDLE_ERROR
;
	IF (STRUPCASE(TYPE) EQ 'WRITE') AND (DATATYPE(FILES,1) NE 'String') $
	  THEN MESSAGE = "For TYPE = 'write', input file names must be a " + $
		'string array.'
	IF MESSAGE NE '' THEN GOTO, HANDLE_ERROR
;
	IF DATATYPE(OLD_DIR,1) NE 'String' THEN MESSAGE = $
		'OLD_DIR must be a string containing the directory where ' + $
		'the process started.'
	IF MESSAGE NE '' THEN GOTO, HANDLE_ERROR
;
	IF DATATYPE(BLFAC,1) NE 'Integer' THEN MESSAGE = $
	  'BLFAC must be an integer (1-10) -- FITS blocking factor.'
ENDELSE
IF MESSAGE NE '' THEN GOTO, HANDLE_ERROR
;
; Set parameters needed for making the Unix script.
;
NFILE = N_ELEMENTS(FILES)  ;  Number of files to be accessed from/to the tape.
BS = 2880*BLFAC            ;  Blocking factor for files.
OPENW, UNX, OLD_DIR+'/unixdd_tape', /GET_LUN	;  Open the script file.
;
; The output information string array.
;
IF N_ELEMENTS(XWSTR) EQ 0 THEN XWSTR = ' '
;
; Access the tape.
;
CASE STRUPCASE(TYPE) OF
;
; **********  Tape reading section.  **************
;
	'READ':	 BEGIN
;
; Move tape to proper file.
		IF FILES(0) NE 1 THEN BEGIN
			PRINTF, UNX, 'echo "Skipping the first ' + $
					 STRTRIM(STRING(FILES(0)-1),2) + $
					' files on the tape."'
			PRINTF, UNX, 'mt -f ' + TAPENM + ' fsf ' + $
				STRTRIM(STRING(FILES(0)-1),2)
		ENDIF
		NSKIP = 0
		FOR I=0,NFILE-1 DO BEGIN
			IF I GT 0 THEN NSKIP = FILES(I) - FILES(I-1) - 1
			IF NSKIP GT 0 THEN BEGIN
				PRINTF, UNX, 'echo "Skipping next ' + $
					STRTRIM(STRING(NSKIP),2) + ' files ' +$
					'on the tape."'
				PRINTF, UNX, 'mt -f ' + TAPENM + ' fsf ' + $
					STRTRIM(STRING(NSKIP),2)
			ENDIF
;
			PRINTF, UNX, 'echo "Reading file (# '+$
				STRING(FORMAT="(I3,') ')",FILES(I))+$
				NAMES(FILES(I)-1)+' from tape drive '+$
				TAPENM+'."'
			PRINTF, UNX, 'dd if='+TAPENM+' of=' + $
				NAMES(FILES(I)-1)+' bs='+STRTRIM(STRING(BS),2)
; Write to log file.
			IF QOUT EQ 'Y' THEN PRINTF, UWR, 'Tape file # ', $
			 	STRING(FORMAT='(I3)',FILES(I)), $
				' written to disk file ', $
				NAMES(FILES(I)-1)
		ENDFOR
		END
;
; **********  Tape writing section.  **************
;
	'WRITE': BEGIN
		FOR I=0,NFILE-1 DO BEGIN
;
; Add the filename to the FITS header before writing the file to tape.
;
			IF KEYWORD_SET(KEYWORD) THEN BEGIN
				FITSDAT = READFITS(FILES(I), HEAD)
				KEYLEN = STRLEN(KEYWORD)
				ISTHERE = WHERE(STRMID(HEAD,0,KEYLEN) EQ $
					KEYWORD)
				IF ISTHERE(0) EQ -1 THEN BEGIN
					XWSTR = ['Adding the '+KEYWORD+$
						' keyword to the FITS '+$
						'header.', XWSTR]
					IF QWIDGET EQ 1 THEN WIDGET_CONTROL, $
						XWIDGET, SET_VALUE=XWSTR $
						ELSE PRINT, XWSTR(0)
					FXADDPAR, HEAD, KEYWORD, FILES(I)
					WRITEFITS, FILES(I), FITSDAT, HEAD
				ENDIF
			ENDIF
;
; Write the tape writing command to the Unix script file.
;
			PRINTF, UNX, 'echo "Writing file '+FILES(I)+$
				' to tape drive '+TAPENM+' (# '+$
				STRING(FORMAT='(I3)',I+1)+')."'
			PRINTF, UNX, 'dd if=' + FILES(I) + ' of=' + TAPENM + $
					' bs=' + STRTRIM(STRING(BS),2)
; Write to log file.
			IF QOUT EQ 'Y' THEN PRINTF, UWR, 'File (# ', $
				STRING(FORMAT='(I3)',I+1) + ') ' + $
				FILES(I), ' being written to tape.'
		ENDFOR
; Write additional <End-of-File> marker 
;	-> 2 <End-of-File> markers = 1 <End-of-Tape> marker.
		PRINTF, UNX, 'mt -f ' + TAPENM + ' weof 1'
;
		END
	ELSE:	 MESSAGE = $
		  "Invalid TYPE sent to UNIXDD: 'read' or 'write' only."
ENDCASE
;
; Close the Unix script file and run it.
;
FREE_LUN, UNX
XWSTR = [' ', 'Running the Unix script to '+TYPE+' the tape.', ' ', XWSTR]
IF QWIDGET EQ 1 THEN XWSTR = [' ', $
	'Script status being dumped to the TEXT window.', XWSTR]
IF QWIDGET EQ 1 THEN BEGIN
	WIDGET_CONTROL, XWIDGET, SET_VALUE=XWSTR
ENDIF ELSE BEGIN
	PRINT, XWSTR(0)
	PRINT, XWSTR(1)
	PRINT, XWSTR(2)
ENDELSE
;
SPAWN,'source '+OLD_DIR+'/unixdd_tape ; rm -f '+OLD_DIR+'/unixdd_tape'
;
XWSTR = [' ', '<<<<<<  Unix Process Complete  >>>>>>', ' ', XWSTR]
IF QWIDGET EQ 1 THEN BEGIN
	WIDGET_CONTROL, XWIDGET, SET_VALUE=XWSTR
ENDIF ELSE BEGIN
	PRINT, XWSTR(0)
	PRINT, XWSTR(1)
	PRINT, XWSTR(2)
ENDELSE
;
IF MESSAGE NE '' THEN GOTO, HANDLE_ERROR
IF N_ELEMENTS(ERRMSG) GT 0 THEN ERRMSG=MESSAGE
RETURN, 1	;  Return with no errors.
;
; Error handling portion of the procedure.
;
HANDLE_ERROR:
IF N_ELEMENTS(QOUT) GT 0 THEN IF QOUT EQ 'Y' THEN FREE_LUN, UWR
IF N_ELEMENTS(ERRMSG) EQ 0 THEN MESSAGE, MESSAGE
ERRMSG = MESSAGE
RETURN, -1	;  Return with an error.
;
END
