;+
; Project     :	SOHO - CDS
;
; Name        :	XWINTAPE
;
; Purpose     :	Widget driver program to read/write SFDU/FITS archive tapes.
;
; Explanation :	A window/menu driver program that runs the MAGTAPE procedure.
;		MAGTAPE is a driver program to read and write magnetic tapes
;		in FITS format using the Standard Formatted Data Unit protocol.
;		An interactive window is generated with various menu options
;		that help you read a magnetic tape.  Once the options have 
;		been selected, this routine runs MAGTAPE.
;
; Use         :	XWINTAPE [, SFDU]
;
; Inputs      :	None.
;
; Opt. Inputs :	None.
;
; Outputs     :	None.
;
; Opt. Outputs:	SFDU:	A string array containing the SFDU header.
;
; Keywords    :	ERRMSG:	If defined and passed, then any error messages will 
;			be returned to the user in this parameter rather than
;			being handled by the IDL MESSAGE utility.  If no 
;			errors are encountered, then a null string is 
;			returned.  In order to use this feature, the string 
;			ERRMSG must be defined first, e.g.,
;
;				ERRMSG = ''
;				XWINTAPE, ERRMSG=ERRMSG
;				IF ERRMSG(0) NE '' THEN ...
;
;			Note that ERRMSG can sometimes be returned as a string
;			array instead of a string scalar.
;
; Calls       :	ANYTIM2CAL, CHK_DIR, CW_PDMENU, MAGTAPE, GET_UTC, SFDU_2FHD, 
;		SFDU_BYTES, SFDU_FITSHD, SFDU_OS, SFDU_READ, SFDU_QWIG, XMENU, 
;		XGETFILES (internal to this routine)
;
; Common      :	SFDU_BYTES, SFDU_FITSHD, XTAPE_INTL, XWINTAPE
;
; Restrictions:	None.
;
; Side effects:	Tape I/O messages are sent to the widget FILENAME instead of
;		screen.  These widget messages are printed in reverse order
;		with respect to normal screen dumps.  This is due to the
;		manner in which WIDGET_TEXT displays new messages (even if one
;		uses the "/APPEND" keyword) -- message #1 is on top and always
;		visible whenever a new message is attached to the bottom.
;
; Category    :	Data Handling, I/O, FITS, SFDU, Generic
;
; Prev. Hist. :	None.
;
; Written     :	Donald G. Luttermoser, GSFC/ARC, 5 April 1995
;
; Modified    :	Version 1, Donald G. Luttermoser, GSFC/ARC, 5 April 1995
;			Initial program.
;		Version 2, Donald G. Luttermoser, GSFC/ARC, 19 April 1995
;			Modified passage of the SFDU header to MAGTAPE.
;			Added more option buttons and changed appearance of
;			widget.
;		Version 3, Donald G. Luttermoser, GSFC/ARC, 18 May 1995
;			Added "Accept Directory" widget button and modified 
;			"<< Begin >>" widget.  Added SFDU_FITSHD call.
;		Version 4, Donald G. Luttermoser, GSFC/ARC, 29 November 1995
;			Modified the SFDU header generator (sfdu_make.pro).
;			Fixed a bug in the "FITS Files" widget "CLOSE" button.
;			Fixed a bug in the "Short Name" file convention such
;			that two identical "short" file names for two different
;			files are not constructed for input into the SFDU 
;			header.  Installed a sensor that deactivates the 
;			"Accept Directory" button has successfully been 
;			engaged (i.e., a valid directory was found after the 
;			button was pushed).  This will prevent an inadvertant 
;			button click after directory parameters have been set.
;		Version 5, Donald G. Luttermoser, GSFC/ARC, 19 December 1995
;			Added a new widget button "Valid CDS FITS?"
;			The default for this button is "Yes".
;		Version 6, 24-October-2000, William Thompson, GSFC
;			Use PNG instead of GIF for SOHO logo,
;			But only for version 5.2 or above.
;
; Version     :	Version 6, 24-October-2000
;-
;
; ****************************************************************************
; **                 Get FITS file names in directory DISK.                 **
; ****************************************************************************
PRO XGETFILES, DISK, FILES
;
COMMON SFDU_FITSHD, FITSHD, PVLHEAD
;
; Get operating system information.
;
SFDU_OS, OS_USED, DIR_COM, DIR_DELIM, SUFFIX, QVERS, TYPERW
;
; Get the file names.
;
CD, DISK, CURRENT=OLD_DIR
FILES = FINDFILE('*'+SUFFIX)
IFIL = STRPOS(FILES, SUFFIX)
IFIL = WHERE(IFIL GT -1, NFILE)
IF NFILE EQ 0 THEN BEGIN
	FILES = 'No FITS files found.' 
ENDIF ELSE BEGIN
	FILES = STRTRIM(FILES(IFIL),2)
; Retrieve the FITS primary headers.
	SFDU_FITSHD, '', FILES
ENDELSE
;
; Go back to original directory.
;
CD, OLD_DIR
;
END
;
; ****************************************************************************
; **          Event handling procedure for the XWINTAPE procedure.          **
; ****************************************************************************
PRO XWINTAPE_EVENT, EV
;
COMMON SFDU_BYTES, NSFDU_BYTE, NREC_PER_LINE, NTAPE_BYTE, NLINE_BYTE
COMMON SFDU_FITSHD, FITSHD, PVLHEAD
COMMON XTAPE_INTL, DISKMESS, DISKNAME, FILENAME, GETFILE, CUR_FILE, WINMESS, $
	MESSCNT, TMPSFDU, MESSAGE, ABORT, WMESS, WDIR, WNAME, WOVERRIDE, $
	WBEGIN, WACCESS, TP_MENU, BL_MENU, WLOGF, WSFDU, WSAVESFDU, WCKFITS, $
	WHELP, WFITS, WABORT, WACCDIR, WEXIT, QSUCCESS
COMMON XWINTAPE, UNIT, TAPETYPE, BLFAC, DISK, FILES, QSFDU, UNIXMT, NAME_TYPE,$
	OUTPUT, SAVHEAD, CKFITS
;
; Obtain name of the event structure.
;
TYPE = TAG_NAMES(EV, /STRUCTURE)
;
; The event CW_PDMENU does not have a "named" structure.  ID such events.
;
IF TYPE EQ '' THEN BEGIN
	TAGS = TAG_NAMES(EV)  &  NTAGS = N_TAGS(EV)
	IF NTAGS EQ 4 THEN IF TAGS(3) EQ 'VALUE' THEN TYPE = 'CW_PDMENU'
ENDIF
;
; Process the widget BUTTON event.
;
CASE TYPE OF
;
; ############################################################################
;
    'WIDGET_BUTTON': BEGIN
	WIDGET_CONTROL, EV.ID, GET_VALUE=VALUE
;
	CASE STRUPCASE(STRTRIM(VALUE,2)) OF
;
; $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
;
; Leave the input portion of the code and start MAGTAPE.
;
	    '<< BEGIN >>':  BEGIN
;
		MESSCNT = 4
; Print the new message.
		WIDGET_CONTROL, DISKMESS, SET_VALUE=WINMESS(MESSCNT,0:2)
;
; Double check to make sure this is what the user wants.
;
		BASE1 = WIDGET_BASE(TITLE='Double Check', /COLUMN, /FRAME)
		WH1 = WIDGET_TEXT(BASE1, VALUE=['Continuing this process '+$
		    'will access the tape drive.','Are you sure you wish '+$
		    'to continue?'], XSIZE=52, YSIZE=3)
		WA1 = WIDGET_BUTTON(BASE1, VALUE="YES, please continue!", $
		    /FRAME)
		WA2 = WIDGET_BUTTON(BASE1, VALUE=$
		    "NO, please return to main widget!", /FRAME)
		WIDGET_CONTROL, BASE1, /REALIZE
		TYPE1 = ' '
		WHILE TYPE1 NE 'WIDGET_BUTTON' DO BEGIN
		    EVENT1 = WIDGET_EVENT(BASE1)
		    TYPE1 = TAG_NAMES(EVENT1, /STRUCTURE)
		ENDWHILE
		WIDGET_CONTROL, EVENT1.ID, GET_VALUE=VALUE1
		WIDGET_CONTROL, BASE1, /DESTROY
		IF VALUE1 EQ 'NO, please return to main widget!' THEN $
		    GOTO, NOT_YET
;
; Resume normal tape I/O.
;
		IF TAPETYPE EQ '?' THEN BEGIN
		    BASE6 = WIDGET_BASE(TITLE='ERROR!', /COLUMN, /FRAME)
		    WH6 = WIDGET_TEXT(BASE6, VALUE=[' ', $
			'Error in determining tape accessing command.', $
			'Click one the "Tape Access" buttons and continue.'], $
			XSIZE=65, YSIZE=4)
		    WB6 = WIDGET_BUTTON(BASE6, VALUE="<<< CLOSE >>>", /FRAME)
		    WIDGET_CONTROL, BASE6, /REALIZE
		    TYPE6 = ' '
		    WHILE TYPE6 NE 'WIDGET_BUTTON' DO BEGIN
			EVENT6 = WIDGET_EVENT(BASE6)
			TYPE6 = TAG_NAMES(EVENT6, /STRUCTURE)
		    ENDWHILE
		    WIDGET_CONTROL, BASE6, /DESTROY
;
; Allow access to the disk name widgets again.
;
		    WIDGET_CONTROL, DISKNAME, SENSITIVE=1
		    WIDGET_CONTROL, WACCDIR, SENSITIVE=1
		    GOTO, NOT_YET
		ENDIF
;
; Set input parameters for reading/writing the tape.
;
		IF QSFDU EQ 0 THEN NOSFDU = 1 ELSE NOSFDU = 0
		IF TMPSFDU(0) NE 'NONE' THEN SFDU = TMPSFDU
;
; Check that files have been selected.
;
		IF CUR_FILE(0) EQ -1 THEN BEGIN
		    BASE7 = WIDGET_BASE(TITLE='ERROR!', /COLUMN, /FRAME)
		    WH7 = WIDGET_TEXT(BASE7, VALUE=[' ', $
			'Error in determining files to access.', 'Select '+$
			'the desired FITS files from the top window first.'], $
			XSIZE=80, YSIZE=4)
		    WB7 = WIDGET_BUTTON(BASE7, VALUE="<<< CLOSE >>>", /FRAME)
		    WIDGET_CONTROL, BASE7, /REALIZE
		    TYPE7 = ' '
		    WHILE TYPE7 NE 'WIDGET_BUTTON' DO BEGIN
			EVENT7 = WIDGET_EVENT(BASE7)
			TYPE7 = TAG_NAMES(EVENT7, /STRUCTURE)
		    ENDWHILE
		    WIDGET_CONTROL, BASE7, /DESTROY
		    GOTO, NOT_YET
		ENDIF
; Certain procedures require integers:
		CUR_FILE = FIX(CUR_FILE)
		IF TAPETYPE EQ 'write' THEN FILES = FILES(CUR_FILE) $
			ELSE FILES = CUR_FILE
;
; Make all subwidgets insensitive (do this instead of just using BASE since
; I don't want the logo to dissappear).  However leave the FILENAME widget
; alive since we want to see the messages.
;
		SFDU_QWIDG, 'OFF'
		WIDGET_CONTROL, FILENAME, SENSITIVE=1
;
; Inform the user that the tape is being accessed.
;
		BASE2 = WIDGET_BASE(TITLE='Status')
		IF STRUPCASE(STRMID(TAPETYPE,0,4)) EQ 'READ' THEN $
		    STVERB='from' ELSE STVERB='to'
		WS2 = WIDGET_TEXT(BASE2, VALUE=STRUPCASE(STRMID(TAPETYPE,0,1))$
		    + STRMID(TAPETYPE,1,3)+'ing files ' + STVERB + $
		    ' tape mounted on MT' + STRTRIM(STRING(UNIT),2), $
		    XSIZE=STRLEN(TAPETYPE)+34+3)
		WIDGET_CONTROL, BASE2, /REALIZE
;
; Print the inputted parameters in the widget text box.
;
		ACCMESS = STRARR(9)
		ACCMESS(0) = ' '
		ACCMESS(1) = '>>> Tape being accessed.  Widget will be ' + $
		    'inactive during this time. <<<'
		ACCMESS(2) = ' '
		ACCMESS(3) = 'Tape Unit Number ' + STRTRIM(STRING(UNIT),2) + $
		    ' has been set.'
		ACCMESS(4) = 'Tape Access: '+TAPETYPE
		ACCMESS(5) = 'File Naming Convention: ' + NAME_TYPE
		IF TAPETYPE EQ 'write' THEN ACCMESS(6) = 'FITS files will ' + $
		    'be written to tape from directory: '+DISK $
		ELSE ACCMESS(6) = 'FITS files will be placed in directory: '+$
		    DISK
		IF QSFDU EQ 0 THEN ACCMESS(7) = 'SFDU Header File will NOT '+$
		    'be used.' $
		ELSE ACCMESS(7) = 'SFDU Header File will be used.'
		IF UNIXMT EQ 1 THEN ACCMESS(8) = 'Unix tape commands will '+$
		    'be used exclusively.' $
		ELSE ACCMESS(8) = ' '
		WIDGET_CONTROL, FILENAME, SET_VALUE=ACCMESS
;
; Set cursor to hour glass for waiting.
;
		WIDGET_CONTROL, /HOURGLASS
;
; *************** Access the files !!!!!!!!!!!!!!!!!!!!!!!
;
		ERRMSG = MESSAGE
		IF STRUPCASE(NAME_TYPE) EQ 'DEFAULT' THEN $
		    MAGTAPE, UNIT, TAPETYPE, SFDU, BLFAC, FILES, ACCMESS, $
		    DISK=DISK, NOSFDU=NOSFDU, UNIXMT=UNIXMT, OUTPUT=OUTPUT, $
		    CKFITS=CKFITS, XWIDGET=FILENAME, ERRMSG=ERRMSG $
		ELSE $
		    MAGTAPE, UNIT, TAPETYPE, SFDU, BLFAC, FILES, ACCMESS, $
		    DISK=DISK, NOSFDU=NOSFDU, NAME_TYPE=NAME_TYPE, $
		    UNIXMT=UNIXMT, OUTPUT=OUTPUT, CKFITS=CKFITS, $
		    XWIDGET=FILENAME, ERRMSG=ERRMSG
;
; Save the SFDU header file before leaving if error occurred in MAGTAPE.
;
		IF SAVHEAD EQ 1 THEN BEGIN
; Get operating system to determine filename length.
		    SFDU_OS, OS_USED, DIR_COM, DIR_DELIM, SUFFIX, QVERS, TYPERW
		    GET_UTC, UTC	; Get current UT.
		    DTNAME = ANYTIM2CAL(UTC,FORM=8)
		    IF N_ELEMENTS(OS_USED) GT 0 THEN $
			IF OS_USED EQ 'DOS' THEN DTNAME = STRMID(DTNAME,10,4)
		    FHEADNM = 'sfdu'+DTNAME+'.head'
		    ACCMESS = [' ', 'Saving the SFDU Header in disk file:  '+$
			FHEADNM, ACCMESS]
		    WIDGET_CONTROL, FILENAME, SET_VALUE=ACCMESS
		    OPENW, UHD, FHEADNM, /GET_LUN
			FOR I=0,N_ELEMENTS(SFDU)-1 DO PRINTF, UHD, SFDU(I)
		    FREE_LUN, UHD
		ENDIF
		TMPSFDU = SFDU
;
; Did an error occur in MAGTAPE?
;
		IF ERRMSG(0) NE '' THEN BEGIN
		    MESSAGE = ERRMSG
		    ABORT = 2
		    WIDGET_CONTROL, BASE2, /DESTROY
		    WIDGET_CONTROL, EV.TOP, /DESTROY
		    RETURN
		ENDIF
;
		QSUCCESS = 1
;
		ACCMESS = [' ', '***********************************', $
		    '        MAGTAPE is finished!', $
		    '***********************************', ACCMESS]
		WIDGET_CONTROL, FILENAME, SET_VALUE=ACCMESS
;
		WIDGET_CONTROL, BASE2, /DESTROY
; Print the new message.
		MESSCNT = 5
		WIDGET_CONTROL, DISKMESS, SET_VALUE=WINMESS(MESSCNT,0:2)
;
; Make "## Exit ##" and "Help" widgets sensitive.
;
		WIDGET_CONTROL, WHELP, SENSITIVE=1
		WIDGET_CONTROL, WEXIT, SENSITIVE=1
;
NOT_YET:
	    END
;
; ############################################################################
;
; Leave the input portion of the code and abort the process.
;
	    '!! ABORT !!':  BEGIN
		ABORT = 1
		WIDGET_CONTROL, EV.TOP, /DESTROY
	    END
;
; $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
;
; Leave the widget.
;
	    '## EXIT ##':  WIDGET_CONTROL, EV.TOP, /DESTROY
;
; $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
;
; Give detailed help in a separate widget.
;
	    'HELP':  BEGIN
		BASE3 = WIDGET_BASE(TITLE='Help!', /COLUMN, /FRAME)
		PASSCNT = MESSCNT
		SFDU_HELP, PASSCNT, HELPTEXT
		YHELP = N_ELEMENTS(HELPTEXT) + 1
		WH3 = WIDGET_TEXT(BASE3, VALUE=HELPTEXT, XSIZE=80, YSIZE=YHELP)
		WB3 = WIDGET_BUTTON(BASE3, VALUE="<<< CLOSE >>>", /FRAME)
		WIDGET_CONTROL, BASE3, /REALIZE
		TYPE3 = ' '
		WHILE TYPE3 NE 'WIDGET_BUTTON' DO BEGIN
		    EVENT3 = WIDGET_EVENT(BASE3)
		    TYPE3 = TAG_NAMES(EVENT3, /STRUCTURE)
		ENDWHILE
		WIDGET_CONTROL, BASE3, /DESTROY
	    END
;
; $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
;
; Accept the entered directory name.
;
	    'ACCEPT DIRECTORY':  BEGIN
; In case this widget is accidentally hit.
		WIDGET_CONTROL, DISKNAME, GET_VALUE=VALUE
		MESSCNT = 3
		IF VALUE(0) NE '' THEN DISK = VALUE(0)
; Check to see if the entered directory is valid.
		IF CHK_DIR(DISK) EQ 0 THEN BEGIN
		    FILES = 'No such directory exists.'
		    GOTO, PRINTIT
		ENDIF
		CASE TAPETYPE OF
		    'read':  NEWMESS = 'read to ' + DISK + '.'
		    'write': NEWMESS = 'write from ' + DISK + '.'
		    ELSE: NEWMESS ='ERROR!  Reclick "Read" or "Write" buttons.'
		ENDCASE
; Print the error message if one occurs.
		IF STRMID(NEWMESS,0,3) EQ 'ERR' THEN BEGIN
		    WIDGET_CONTROL, DISKMESS, SET_VALUE=[' ', NEWMESS, ' ']
		    RETURN
		ENDIF
; Print the new message.
		WIDGET_CONTROL, DISKMESS, SET_VALUE=[WINMESS(MESSCNT,0)+$
		    NEWMESS, WINMESS(MESSCNT,1), WINMESS(MESSCNT,2)]
		XWSTR = 'Retrieving file names and reading FITS headers.  '+$
		    'Please wait ...'
		WIDGET_CONTROL, FILENAME, SET_VALUE=XWSTR
					
; Set cursor to hour glass for waiting.
		WIDGET_CONTROL, /HOURGLASS
; Make all subwidgets insensitive (do this instead of just using BASE since
; I don't want the logo to dissappear).  Leave FILENAME widget on so user can
; read the I/O messages.
		SFDU_QWIDG, 'OFF'
		WIDGET_CONTROL, FILENAME, SENSITIVE=1
;
; List the FITS files in/for the new directory.
;
		IF TAPETYPE EQ 'write' THEN XGETFILES, DISK, FILES $
		ELSE BEGIN
; Get operating system dependent filename suffix.
		    SFDU_OS, OS_USED, DIR_COM, DIR_DELIM, SUFFIX, QVERS, TYPERW
		    IF UNIXMT EQ 1 THEN TYPERW = 'UNIXDD'
		    IF QSFDU EQ 0 THEN BEGIN
; If no SFDU file is on the tape, then ask user how many files are there.
			NTAPE = 0
			READ, 'How many FITS files are on the tape: ', NTAPE
			FILES = 'tape' + STRMID(STRING(INDGEN(NTAPE)+10001,$
			    FORMAT='(I5)'),1,4) + SUFFIX
		    ENDIF ELSE BEGIN
			IF TMPSFDU(0) EQ 'NONE' THEN BEGIN
			    XWSTR = ['Begin accessing tape to retrieve '+$
				'SFDU header -- please wait ...', XWSTR]
			    WIDGET_CONTROL, FILENAME, SET_VALUE=XWSTR
			    TPNM = SUFFIX
			    TAPENM = GETENV('MT'+STRING(UNIT, FORMAT='(I1)'))
			    TAPENM = TAPENM(0)
;
; Read SFDU header file to get listings of files and FITS headers.
;
			    SFDU_BYTES, BLFAC
			    SFDU_READ, UNIT, TAPENM, TYPERW, TMPSFDU, SHNM, $
				LGNM, TPNM, XWSTR, XWIDGET=FILENAME
			    NTP = N_ELEMENTS(LGNM)
;
; Put the 3 types of filenames in one variable and format this array string
; such that the filenames appear in a straight column.
;
; Add and format short filenames.
			    FILES = STRING(INDGEN(NTP)+1,FORMAT='(I4)')+'  '+$
				SHNM
			    NMLMAX = MAX(STRLEN(FILES))
			    FOR JK = 0, NTP-1 DO BEGIN
				NMLEN = STRLEN(FILES(JK))
				BLNK = ' '
				FOR KK = 0, NMLMAX-NMLEN DO BLNK = BLNK + ' '
				FILES(JK) = FILES(JK) + BLNK
			    ENDFOR
; Add and format long filenames.
			    FILES = FILES + LGNM
			    NMLMAX = MAX(STRLEN(FILES))
			    FOR JK = 0, NTP-1 DO BEGIN
				NMLEN = STRLEN(FILES(JK))
				BLNK = ' '
				FOR KK = 0, NMLMAX-NMLEN DO BLNK = BLNK + ' '
				FILES(JK) = FILES(JK) + BLNK
			    ENDFOR
; Add tape location filenames.
			    FILES = FILES + TPNM
;
			    SFDU_2FHD, TMPSFDU, XWSTR, XWIDGET=FILENAME
			ENDIF
		    ENDELSE
		ENDELSE
; Make all subwidgets sensitive.
		SFDU_QWIDG, 'ON'
;
		IF FILES(0) NE 'No FITS files found.' THEN BEGIN
		    FILES = ['<Select All Displayed Files>', FILES]
; Don't allow further access to the disk name widgets.
		    WIDGET_CONTROL, DISKNAME, SENSITIVE=0
		    WIDGET_CONTROL, WACCDIR, SENSITIVE=0
; Make "FITS Header" widget sensitive.
		    WIDGET_CONTROL, WFITS, SENSITIVE=1
; If type "write", make File Naming Convention inactive.
		    IF TAPETYPE EQ 'write' THEN WIDGET_CONTROL, WNAME, $
			SENSITIVE=0
		ENDIF
PRINTIT:
		WIDGET_CONTROL, FILENAME, SET_VALUE=FILES
	    END
;
; $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
;
; Display the FITS header.
;
	    'FITS HEADER': BEGIN
; >>> File list widget.
		BASE4 = WIDGET_BASE(TITLE='FITS Files', /COLUMN, /FRAME)
		Y4FITS = N_ELEMENTS(FILES) + 1
		IF Y4FITS LE 2 THEN LST4FILES = FILES $
		    ELSE LST4FILES = FILES(1:Y4FITS-2)
		WI4 = WIDGET_TEXT(BASE4, VALUE=$
		    ["Select one of the files below.", $
		    "After examining the text in the 'FITS Header' window,", $
		    "select a different file or CLOSE the 'FITS Files' "+$
		    "window."], XSIZE=60, YSIZE=3)
		WF4 = WIDGET_LIST(BASE4, VALUE=LST4FILES, XSIZE=80, $
		    YSIZE=Y4FITS<20)
		WB4 = WIDGET_BUTTON(BASE4, VALUE="<<< CLOSE >>>", /FRAME)
		WIDGET_CONTROL, BASE4, /REALIZE
		TYPE4 = ' '
		WHILE TYPE4 NE 'WIDGET_BUTTON' DO BEGIN
		    EVENT4 = WIDGET_EVENT(BASE4)
		    TYPE4 = TAG_NAMES(EVENT4, /STRUCTURE)
; >>> FITS header widget.
		    IF TYPE4 EQ 'WIDGET_LIST' THEN BEGIN
			CREATE5 = 0
			IF N_ELEMENTS(BASE5) GT 0 THEN BEGIN
			    IF WIDGET_INFO(BASE5, /VALID_ID) NE 1 THEN BEGIN
				BASE5 = WIDGET_BASE(TITLE='FITS Header', $
				    /FRAME)
				CREATE5 = 1
			    ENDIF
			ENDIF ELSE BEGIN
			    BASE5 = WIDGET_BASE(TITLE='FITS Header', /FRAME)
			    CREATE5 = 1
			ENDELSE
			IF CREATE5 EQ 1 THEN BEGIN
			    FHSIZE = SIZE(FITSHD)
			    Y5FITS = FHSIZE(2) + 1
			    WT5 = WIDGET_TEXT(BASE5, VALUE=$
				FITSHD(EVENT4.INDEX,*), /SCROLL, XSIZE=80, $
				YSIZE=Y5FITS<50)
			    WIDGET_CONTROL, BASE5, /REALIZE
			ENDIF ELSE BEGIN
			    WIDGET_CONTROL, WT5, SET_VALUE=$
				FITSHD(EVENT4.INDEX,*)
			ENDELSE
		    ENDIF
		ENDWHILE
		IF N_ELEMENTS(BASE5) GT 0 THEN IF WIDGET_INFO(BASE5, $
		    /VALID_ID) EQ 1 THEN WIDGET_CONTROL, BASE5, /DESTROY
		WIDGET_CONTROL, BASE4, /DESTROY
	    END
;
; $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
;
; Set the tape drive to read.
;
	    'READ':  BEGIN
		TAPETYPE = 'read'
		MESSCNT = 1
; Print the new message.
		WIDGET_CONTROL,DISKMESS,SET_VALUE=WINMESS(MESSCNT,0:2)
	    END
;
; $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
;
; Set the tape drive to write.
;
	    'WRITE': BEGIN
		TAPETYPE = 'write'
		MESSCNT = 1
; Print the new message.
		WIDGET_CONTROL,DISKMESS,SET_VALUE=WINMESS(MESSCNT,0:2)
	    END
;
; $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
;
; Output log file?
;
	    'LOG FILE':	OUTPUT = 1
	    'NO FILE':	OUTPUT = 0
;
; $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
;
; Save SFDU header in an external file?
;
	    'YES':	SAVHEAD = 1
	    'NO':	SAVHEAD = 0
;
; $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
;
; Check FITS file for CDS level-1 validity?
;
	    'CHECK FITS VALIDITY':	CKFITS = 1
	    'IGNORE FITS CHECK':	CKFITS = 0
;
; $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
;
; SFDU Set?
;
	    'SFDU':  BEGIN
		QSFDU = 1
		MESSCNT = 1
	    END
	    'NO SFDU': BEGIN
		QSFDU = 0
		MESSCNT = 1
	    END
;
; $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
;
; Unix Override?
;
	    'IDL STYLE COMMANDS':  BEGIN
		UNIXMT = 0
		MESSCNT = 2
	    END
	    'UNIX MT & DD COMMANDS': BEGIN
		UNIXMT = 1
		MESSCNT = 2
	    END
;
; $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
;
; File naming convention:
;
	    'DEFAULT':		NAME_TYPE = 'default'
	    'SFDU_SMALL':	NAME_TYPE = 'sfdu_sm'
	    'SFDU_LARGE':	NAME_TYPE = 'sfdu_lg'
	    'KEYWORD':		NAME_TYPE = 'kword:FILENAME'
	    'TAPE LOCATION':	NAME_TYPE = 'tapeloc'
; Get unit number from the tape drive button.
	    ELSE:  WIDGET_CONTROL, FILENAME, SET_VALUE=$
		[' ','>>>>> ERROR!',' ','Unknown button!']
	ENDCASE
    END
;
; ############################################################################
;
; Process the Pull Down Menu widget.
;
    'CW_PDMENU': BEGIN
	IF STRMID(EV.VALUE,0,2) EQ 'MT' THEN BEGIN
	    UNIT = FIX(STRMID(EV.VALUE,2,1))
; Print the new message.
	    MESSCNT = 2
	    WIDGET_CONTROL,DISKMESS,SET_VALUE=WINMESS(MESSCNT,0:2)
	ENDIF
	IF STRPOS(EV.VALUE,'bytes/record') GT -1 THEN BEGIN
	    BLFAC = FIX(STRMID(EV.VALUE,0,2))
	    MESSCNT = 3
	ENDIF
    END
;
; ############################################################################
;
; Process the widget LIST event.  This gets the FITS file names from the disk
; or tape.
;
    'WIDGET_LIST': BEGIN
	IF QSUCCESS EQ 1 THEN GOTO, NO_LIST	; Don't access this when done.
	IF MESSCNT LT 3 THEN GOTO, NO_LIST	; Don't access this until ready.
	WIDGET_CONTROL, EV.ID, GET_UVALUE=UVALUE
	CASE UVALUE OF
;
; $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
;
	    "FILES": BEGIN
		IF CUR_FILE(0) EQ -1 THEN CUR_FILE = EV.INDEX ELSE $
		    CUR_FILE = [CUR_FILE, EV.INDEX]
		NCUR = N_ELEMENTS(CUR_FILE)
; Sort the filename in the proper order.
		IF NCUR GT 1 THEN BEGIN
; Sort the filename in the proper order.
		    CUR_FILE = CUR_FILE(SORT(CUR_FILE))
; In case someone double clicks on the same file, assume this was a mistake.
		    FOR IC=1,NCUR-1 DO IF CUR_FILE(IC-1) EQ CUR_FILE(IC) THEN $
			CUR_FILE(IC-1) = -1
		ENDIF
		ICUR = WHERE(CUR_FILE GT -1, NCUR)
		IF NCUR GE 1 THEN CUR_FILE = CUR_FILE(ICUR) ELSE $
		    WIDGET_CONTROL, FILENAME, SET_VALUE=[' ','>>>>> ERROR!',$
		    ' ','Error in file selection.']
; Select all files if FILES(CUR_FILE(0)) eq '<Select All Displayed Files>':
		IF FILES(CUR_FILE(0)) EQ '<Select All Displayed Files>' THEN $
		    CUR_FILE = INDGEN(N_ELEMENTS(FILES)-1) + 1
		WIDGET_CONTROL, GETFILE,SET_VALUE=['<Click on an Entry below'+$
		    ' to Remove it from the "Selected File" List>', $
		    FILES(CUR_FILE)]
	    END
;
; $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
;
	    "GETFILE": BEGIN
		IF CUR_FILE(0) EQ -1 THEN BEGIN
		    WIDGET_CONTROL, GETFILE, SET_VALUE='No files yet '+$
			'selected -- select files from upper window ...'
		ENDIF ELSE BEGIN
		    SUB_FILE = EV.INDEX
		    IF SUB_FILE GT 0 THEN BEGIN
			SUB_FILE = CUR_FILE(SUB_FILE-1)
			CUR_FILE = CUR_FILE(WHERE(CUR_FILE NE SUB_FILE))
		    ENDIF
		    WIDGET_CONTROL, GETFILE, SET_VALUE=['<Click on an '+$
			'Entry below to Remove it from this "Selected File" '+$
			'List>', FILES(CUR_FILE)]
		ENDELSE
	    END
;
; $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
;
	    ELSE: WIDGET_CONTROL, FILENAME, SET_VALUE=[' ','>>>>> ERROR!',' ',$
		'Unknown Widget List entered.']
	ENDCASE
NO_LIST:
    END
;
; ############################################################################
;
; Process the widget TEXT event.  This gets the subdirectory name where FITS
; files are accessed.
;
    'WIDGET_TEXT_CH': BEGIN
	WIDGET_CONTROL, EV.ID, GET_UVALUE=UVALUE
	CASE UVALUE OF
	    "NAME":  BEGIN
		WIDGET_CONTROL, DISKMESS, SET_VALUE=[' ', $
		    'Click on the "Accept Directory" box.', ' ']
	    END
	    ELSE:
	ENDCASE
    END
;
; ############################################################################
;
; Warn the user that an unknown widget was used.
;
    ELSE: WIDGET_CONTROL, FILENAME, SET_VALUE=[' ','>>>>> ERROR!',' ',$
	'Unknown widget has been selected!']
ENDCASE
;
; ############################################################################
;
RETURN
END
; ****************************************************************************
; **                       Procedure to drive MAGTAPE.                      **
; ****************************************************************************
PRO XWINTAPE, SFDU, ERRMSG=ERRMSG
;
COMMON SFDU_FITSHD, FITSHD, PVLHEAD
COMMON XTAPE_INTL, DISKMESS, DISKNAME, FILENAME, GETFILE, CUR_FILE, WINMESS, $
	MESSCNT, TMPSFDU, MESSAGE, ABORT, WMESS, WDIR, WNAME, WOVERRIDE, $
	WBEGIN, WACCESS, TD_MENU, BL_MENU, WLOGF, WSFDU, WSAVESFDU, WCKFITS, $
	WHELP, WFITS, WABORT, WACCDIR, WEXIT, QSUCCESS
COMMON XWINTAPE, UNIT, TAPETYPE, BLFAC, DISK, FILES, QSFDU, UNIXMT, NAME_TYPE,$
	OUTPUT, SAVHEAD, CKFITS
;
ON_ERROR, 2   ; Return to caller if error is encountered.
MESSAGE = ''  ; Set to non-null string if error is encountered.
;
; Check to be sure that X-window is the default terminal.
;
IF STRUPCASE(!D.NAME) NE 'X' THEN BEGIN
	MESSAGE = ['X-windows required for this procedure.   IDL says your '+$
		'device is '+!D.NAME+'.', $
		"Try the IDL command  set_plot,'x'  before rerunning.", $
		'If this fails, you must run MAGTAPE by itself (see CDS '+$
		'SOFTWARE NOTE #28).', 'X-Window error.  Returning ...']
	GOTO, HANDLE_ERROR
ENDIF
;
; Defaults for the input parameters.
;
ABORT = 0		; Do not abort the process.
BLFAC = 1		; Blocking factor (BLFAC*2880 bytes/record).
CKFITS = 1		; Check the FITS file for CDS validity.
CUR_FILE = -1		; File selection parameter.
NAME_TYPE = 'default'	; Use the default naming convention for the files.
OUTPUT = 1		; Generate an output log file named tapelog.lis.
QSFDU = 1		; Use the Standard Formatted Data Unit header file.
QSUCCESS = 0		; Set to "1" when tape has been successfully accessed.
SAVHEAD = 0		; Do not save the SFDU header in a disk file.
TAPETYPE = '?'		; Tape will be read -- set to "?" initially.
TMPSFDU = 'NONE'	; Temporary SFDU array.
UNIT = 0		; Tape drive "MT0" will be accessed.
UNIXMT = 0		; Do not override the tape access derivation.
;
; Step-by-step instructions that are displayed to the window.
;
MESSCNT = 0		; The "Window Message" count.
WINMESS = STRARR(6,3)
WINMESS(0,0:2) = [$
	'If the tape does/will NOT contain an SFDU header file, click the',$
	'appropriate button in the "Tape Header" box before proceeding.  Then',$
	'click on the "Read" or "Write" button in the "Tape Access" box.']
WINMESS(1,0:2) = [$
	'To override default tape access commands, click in "Access '+$
	'Override" box.', ' ', 'Select the "Tape Drive".']
WINMESS(2,0:2) = [$
	'Enter the directory for FITS-file/tape I/O in the box below and ',$
	'click the "Accept Directory" box.', $
	'Once the names appear, the "FITS Header" button can display the '+$
		'headers.']
WINMESS(3,0:2) = [$
	'Select the files you wish to ', ' ', $
	'When done, change any defaults you wish, then hit "<< Begin >>".']
WINMESS(4,0:2) = [$
	' ', $
	'Widget will remain inactive while tape I/O is being performed.', $
	' ']
WINMESS(5,0:2) = [$
	'************************************************', $
	'Click the  "## Exit ##"  button to leave widget.', $
	'************************************************']
;
CD,CURRENT=DISK		; Files will be written to (or read from) current dir.
;
; Get FITS files from the current directory.
;
XGETFILES,DISK,FILES
;
; Open the workspace window.
;
WIDGET_TITLE = 'XWINTAPE:  Read/Write FITS/SFDU Tapes'
BASE = WIDGET_BASE(TITLE=WIDGET_TITLE, /ROW, SPACE=20)
LCOL = WIDGET_BASE(BASE, /FRAME, /COLUMN, SPACE=10)
RCOL = WIDGET_BASE(BASE, /FRAME, /COLUMN, SPACE=10)
;
; ****************************************************************************
;  Left side.
; ****************************************************************************
;
; Create a message widget that gives concise operational information.
;
WMESS = WIDGET_BASE(LCOL, /ROW, /FRAME)
DMLAB = WIDGET_LABEL(WMESS, VALUE='Message:        ')
DISKMESS = WIDGET_TEXT(WMESS, XSIZE=74, YSIZE=3, $
	VALUE=WINMESS(MESSCNT,0:2), UVALUE='MESSAGE')
;
; Create a widget that displays the directory name and create button to accept 
; the directory.
;
WDIR = WIDGET_BASE(LCOL, /ROW, /FRAME)
DNLAB = WIDGET_LABEL(WDIR, VALUE='Directory Name: ')
DISKNAME = WIDGET_TEXT(WDIR, /EDITABLE, XSIZE=55, YSIZE=1, VALUE=DISK, $
	UVALUE='NAME')
WACCDIR = WIDGET_BUTTON(WDIR, VALUE="Accept Directory", /FRAME)
;
; Make FileName table, fill with the SOHO/CDS logo first.
;
SC_LOGO = STRARR(17)
FLOGO = FIND_WITH_DEF('soho_cds.logo',!PATH)
IF FLOGO(0) EQ '' THEN BEGIN
	MESSAGE = 'Cannot find file: soho_cds.logo!'
	GOTO, HANDLE_ERROR
ENDIF
OPENR,ULOGO,FLOGO,/GET_LUN
	READF, ULOGO, SC_LOGO
FREE_LUN,ULOGO
FILENAME = WIDGET_LIST(LCOL, XSIZE=30, YSIZE=17, VALUE=SC_LOGO, $
	UVALUE='FILES', FONT='fixed')
XMENU, ['Default', 'SFDU_small', 'SFDU_large', 'Keyword', 'Tape Location'], $
	LCOL, TITLE='File Naming Convention', /ROW, BUTTONS=FNC_ID, $
	/EXCLUSIVE, /FRAME, /NO_RELEASE, BASE=WNAME
;
; Must use the Unix tape accessing commands!
; Check FITS file for CDS validity?
;
WMISC = WIDGET_BASE(LCOL, /ROW, /FRAME)
XMENU, ['IDL Style Commands', 'Unix mt & dd Commands'], WMISC, $
	TITLE='Access Override', /ROW, /EXCLUSIVE, BUTTONS=UNIX_ID, /FRAME, $
	/NO_RELEASE, BASE=WOVERRIDE
XMENU, ['Check FITS Validity', 'Ignore FITS Check'], WMISC, $
	TITLE='Valid CDS FITS?', /ROW, /EXCLUSIVE, BUTTONS=CKFITS_ID, /FRAME, $
	/NO_RELEASE, BASE=WCKFITS
;
; This window contains the selected files.
;
GET_TEXT = STRARR(1)  &  GET_TEXT(0) = 'Select files from upper window ...'
GETFILE = WIDGET_LIST(LCOL, XSIZE=30, YSIZE=13, VALUE=GET_TEXT, $
	UVALUE='GETFILE')
;
; ****************************************************************************
;  Right side.
; ****************************************************************************
;
; Start accessing the tape.
;
WBEGIN = WIDGET_BUTTON(RCOL, VALUE="<< Begin >>", /FRAME)
;
; Read or Write the tape.
;
XMENU, ['Read', 'Write'], RCOL, TITLE='Tape Access', /COLUMN, /EXCLUSIVE, $
	/FRAME, BASE=WACCESS
;
; Which tape drive?
;
TAPE_DRIVE = {CW_PDMENU_S, FLAGS:0, NAME:''}
TAPE_DRIVE = REPLICATE(TAPE_DRIVE, 11)
TAPE_DRIVE(0).NAME = 'Tape Drive'
FOR I=1,10 DO TAPE_DRIVE(I).NAME = 'MT'+STRTRIM(STRING(I)-1,2)
TAPE_DRIVE(0).FLAGS = 3
TAPE_DRIVE(10).FLAGS = 2
TD_MENU = CW_PDMENU(RCOL, TAPE_DRIVE, /RETURN_NAME)
;
; Which Blocking Factor?
; 
BL_FAC = {CW_PDMENU_S, FLAGS:0, NAME:''}
BL_FAC = REPLICATE(BL_FAC, 11)
BL_FAC(0).NAME = 'Blocking Factor'
FOR I=1,10 DO BL_FAC(I).NAME = STRING(I,FORMAT='(I2)')+'*2880 bytes/record'
BL_FAC(0).FLAGS = 3
BL_FAC(10).FLAGS = 2
BL_MENU = CW_PDMENU(RCOL, BL_FAC, /RETURN_NAME)
;
; Generate an output log of the tape seesion?
;
XMENU, ['Log File', 'No File'], RCOL, TITLE='Session Log', /COLUMN, $
	/EXCLUSIVE, BUTTONS=LOG_ID, /FRAME, /NO_RELEASE, BASE=WLOGF
;
; No SFDU header?
;
XMENU, ['SFDU', 'No SFDU'], RCOL, TITLE='Tape Header', /COLUMN, /EXCLUSIVE, $
	BUTTONS=SFDU_ID, /FRAME, /NO_RELEASE, BASE=WSFDU
;
; Save SFDU header in file?
;
XMENU, ['Yes', 'No'], RCOL, TITLE='Save SFDU File?', /COLUMN, /EXCLUSIVE, $
	BUTTONS=SAVHEAD_ID, /FRAME, /NO_RELEASE, BASE=WSAVESFDU
;
; Display the FITS headers.
;
WFITS = WIDGET_BUTTON(RCOL, VALUE="FITS Header", /FRAME)
;
; HELP!
;
WHELP = WIDGET_BUTTON(RCOL, VALUE="Help", /FRAME)
;
; Official SoHO logo widget.  See the note by the actual drawing part of this
; widget below.  Note that Sun workstations do not support the same fonts as 
; the DEC Alpha machines.  As such, 2 different soho_logo.png pictures have 
; been made to make the main widget pretty.
;   Sun version:  soho_smlogo.png    byte array (131, 89)
;   DEC version:  soho_lglogo.png    byte array (144, 89)
;
IF STRUPCASE(!VERSION.OS) EQ 'SUNOS' THEN BEGIN
	XS_PIC = 131
	LOGO_FILE = 'soho_smlogo'
ENDIF ELSE BEGIN
	XS_PIC = 144
	LOGO_FILE = 'soho_lglogo'
ENDELSE
IF !VERSION.RELEASE GE '5.4' THEN LOGO_EXT = '.png' ELSE LOGO_EXT = '.gif'
DRAW = WIDGET_DRAW(RCOL, XSIZE=XS_PIC, YSIZE=89)
;
; Abort the session!
;
WABORT = WIDGET_BUTTON(RCOL, VALUE="!! Abort !!", /FRAME)
;
; Leave the widget, process complete!
;
WEXIT = WIDGET_BUTTON(RCOL, VALUE="## Exit ##", /FRAME)
;
; Set the SFDU button to ON, output log file ON, and the Unix Override button 
; to OFF, since these are the defaults.  Set the File Naming Convention to the 
; Default.  Set check FITS validity button to ON.
;
WIDGET_CONTROL, SFDU_ID(0),    /SET_BUTTON
WIDGET_CONTROL, SAVHEAD_ID(1), /SET_BUTTON
WIDGET_CONTROL, LOG_ID(0),     /SET_BUTTON
WIDGET_CONTROL, UNIX_ID(0),    /SET_BUTTON
WIDGET_CONTROL, FNC_ID(0),     /SET_BUTTON
WIDGET_CONTROL, CKFITS_ID(0),  /SET_BUTTON
; "Exit" button only activated after tape is accessed.
WIDGET_CONTROL, WEXIT,         SENSITIVE=0
; "FITS Header" button only activated after FITS files are selected.
WIDGET_CONTROL, WFITS,         SENSITIVE=0
;
; >>>>>>>>>>>>>> Make the widget window.
;
WIDGET_CONTROL, BASE, /REALIZE
;
; Draw the official SOHO logo widget.  Note that Sun workstations do not 
; support the same fonts as the DEC Alpha machines.  As such, 2 different 
; soho_logo.png pictures have been made to make the main widget pretty.
;
WIDGET_CONTROL, GET_VALUE=LOGO_WIN, DRAW
WSET, LOGO_WIN
SLOGO = FIND_WITH_DEF(LOGO_FILE+LOGO_EXT,!PATH)
IF SLOGO(0) EQ '' THEN WIDGET_CONTROL, FILENAME, SET_VALUE=[' ',$
	'>>>>> ERROR!',' ','Cannot find file: '+LOGO_FILE+LOGO_EXT] $
ELSE BEGIN
    IF LOGO_EXT EQ '.gif' THEN BEGIN
	READ_GIF, SLOGO, IM_LOGO, RLOGO, GLOGO, BLOGO
    END ELSE BEGIN
	IM_LOGO = READ_PNG(SLOGO, RLOGO, GLOGO, BLOGO)
	IF !VERSION.RELEASE LT '5.4' THEN IM_LOGO = REVERSE(IM_LOGO,2)
    ENDELSE
    TVLCT, RLOGO, GLOGO, BLOGO
    TV, IM_LOGO
ENDELSE
;
; Start the widget.
;
XMANAGER, 'XWINTAPE', BASE
QSUCCESS = 0	; Reset in case a new tape is to be accessed.
;
SFDU = TMPSFDU			; TMPSFDU is passed between internal procs.
IF ABORT EQ 1 THEN BEGIN
	MESSAGE = 'Operator aborted without reading the tape.'
	GOTO, HANDLE_ERROR
ENDIF
IF ABORT GT 1 THEN GOTO, HANDLE_ERROR	; Error occurred in XWINTAPE_EVENT.
;
; In case somebody reruns this without leaving IDL.
;
FITSHD = ''
;
IF MESSAGE(0) NE '' THEN GOTO, HANDLE_ERROR
IF N_ELEMENTS(ERRMSG) GT 0 THEN ERRMSG=MESSAGE
RETURN
;
; Error handling portion of the procedure.
;
HANDLE_ERROR:
FITSHD = ''
IF N_ELEMENTS(ERRMSG) EQ 0 THEN BEGIN
	NMESS = N_ELEMENTS(MESSAGE)
	IF NMESS GT 1 THEN FOR I = 0, NMESS-2 DO PRINT, MESSAGE(I)
	MESSAGE, MESSAGE(NMESS-1)
ENDIF
ERRMSG = MESSAGE
RETURN
;
END
