	PRO CHK_CD_STATUS, MAXSC = MAX_STUDY_COUNT, MAXSEQ = MAX_SEQ
;+
; Project     :	SOHO - CDS
;
; Name        :	CHK_CD_STATUS
;
; Purpose     :	Check the processing status of all CDROMs.
;
; Explanation :	This procedure extracts information about every CDROM which
;		was listed by the procedure GET_CDROM_LST.  This information
;		is not returned to the calling procedure, but instead it is 
;		added to an array of structures which holds information about
;		all the CDROMs which have currently been registered.  The 
;		specific information which is extracted by this procedure 
;		includes whether each CDROM has been processed, and what 
;		studies are included in each day covered by the CDROM.  
;
; Use         :	CHK_CD_STATUS, MAXSC = MAXSC, MAXSEQ = MAXSEQ
;
; Inputs      :	None.
;
; Opt. Inputs :	None.
;
; Outputs     :	None.
;
; Opt. Outputs:	None.
;
; Keywords    :	MAXSC	The maximum study counter which was used by any
;			of the CDROMs which this procedure examined.
;
;		MAXSEQ	The largest sequence number in use.  A sequences is
;			a group CDROMs which were created by the DDF at the
;			same time.
;
; Calls       :	CDSHEADFITS,
;		FXPAR
;
; Common      :	CDTBL,
;		CD_TBL_SIZE,
;		TBL,
;		TBLSIZE
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Category    :	Planning, Database.
;
; Prev. Hist. :	None.
;
; Written     :	Ron Yurow, GSFC, 21 October 1997
;
; Modified    :	Version 1, Ron Yurow, GSFC, 21 October 1997
;
; Version     :	Version 1, 21 October 1997
;-
;

	COMMON CDROM_TBL, CDTBL, CD_TBL_SIZE
	COMMON FITS_NAME_TBL, TBL, TBLSIZE

;
; Set SECPERDAY to the number of seconds in one day.  We will use this latter.
;

	SECPERDAY = 3600.0D0 * 24.0D0

;
; Get the largest CDROM sequence number that has so far been registered.
;
	
	MAX_SEQ = MAX (CDTBL.SEQUENCE)

;
; Set MAX_STUDY_COUNTER to 0.  We will use it to keep track of the maximum
; study counter that is used by a FITS file.
;

	MAX_STUDY_COUNT = 0 

;
; Main loop to search through all the CDROM's that have so far been 
; registered.  This loop searchs through each set of CDROMs (sequence),
; while the inner loop searches through each CDROM in the current sequence
; (volume).
;

        FOR SEQ = 1, MAX_SEQ DO BEGIN

;
; Set MAX_VOL to the largest volume number in the current CDROM sequence.
;

	   MAX_VOL = MAX (CDTBL (WHERE (CDTBL.SEQUENCE EQ SEQ)).VOLUME)

;
; Inner loop to search through each CDROM (volumen) in the current sequence.
; 

	   FOR VOL = 1, MAX_VOL DO BEGIN

;
; Set LST to an array where every element corresponds to index in the table
; CDTBL for CDROM sequence SEQ and volume VOL.
;

	      LST = WHERE (CDTBL.SEQUENCE EQ SEQ AND CDTBL.VOLUME EQ VOL)

;
; Check if no entries in CDTBL were found for CDROM sequence SEQ and volume
; VOL.  If this was the case, then LST will be a scaler set to -1.  Otherwise
; we go ahead and process all the entries that we found.
;

	      IF LST (0) NE -1 THEN BEGIN

;
; Set TMP_MAX_SC to 0.  TMP_MAX_SC is used to contain the maximum value for 
; the study counter that we found for the series of dates which is included
; in the current CD.  If data for this CD is determined to be unprocessed, 
; then this value will be discarded.  Otherwise it will be used to update 
; MAXS_STUDY_COUNT 
;

	         TMP_MAX_SC = 0

; 
; Set NO_DATES to the number of entries we found in the table CDTBL.  Since 
; each entry corresponds to a different date, this is the number dates we
; have to process.
;

	         NO_DATES = N_ELEMENTS (LST)

;
; Initialize DAY to 0.  We will use DAY as an index counter while processing
; all of the dates for CDROM sequence SEQ and volume VOL.
;

	         DAY = 0	
;
; Set PROCESSED to 1.  If we find that any of the FITS files which belong to
; to the days recorded on this CDROM were no processed from CDROM data, then
; we will PROCESSED to 0.
;

	         PROCESSED = 1

;
; Loop to check through all of the days encompassed by CDROM sequence SEQ
; and volume VOL.  Each day is checked to see if the last FITS file of that
; day was processed from CDROM data.
;

	         WHILE DAY LT NO_DATES DO BEGIN

;
; Set TMP1 to the elements of the FITS file name table TBL which reference
; FITS files that were created after the start of the current day which is 
; being processed.
;

	            TMP1 = TBL.DATE_END GT CDTBL (LST (DAY)).DATE

;
; Set TMP2 to the elements of the FITS file name table TBL which reference
; FITS files that were created prior to 24 hours after the start of the 
; current day which is being processed.
;

	            TMP2 = TBL.DATE_END LT CDTBL (LST (DAY)).DATE + SECPERDAY

;
; Set IND to the array of elements of the FITS file name table TBL which 
; reference FITS that were created during the current day being processed.
;


 	            IND = WHERE (TMP1 AND TMP2)

;
; Check if there were any FITS files created on current day being processed.
; If there were no FITS files created, then we don't have to do anything.
;

	            IF IND (0) NE -1 THEN BEGIN

;
; Set FITS_LIST to a string array containing the names of all the FITS
; FILES that were created on the current day.  Set SDY_LIST to an integer
; array containing the study counter values associated with all the FITS files
; created on the current day.  Set RAS_LIST to an integer array contianing 
; the raster numbers associated with all the FITS files created on the current
; day.
;

	               FITS_LIST = TBL (IND).FILENAME
	               SDY_LIST = LONG (TBL (IND).PROG_NUM)
	               RAS_LIST = LONG (TBL (IND).SEQ_IND)

;
; Create a sort list of index's into the arrays FITS_LIST, SDY_LIST, and 
; RAS_LIST.
;

	               SIND = SORT ((SDY_LIST * 1000L) + RAS_LIST)
 
;
; Sort the list of FITS file names.
;

	               FITS_LIST = FITS_LIST (SIND)

;
; Sort the list of study counters
;

	               SDY_LIST = SDY_LIST (SIND)

;
; Set the name of the lowest (in alphebetical order) FITS file to the first
; FITS file from FITS_LIST.
;

                       CDTBL (LST (DAY)).LFITS = FITS_LIST (0)

;
; Set the name fo the highest (in alphabetical order) FITS file to the last
; last FITS file from FITS_LIST
;

	               NO_IND = N_ELEMENTS (IND)
                       CDTBL (LST (DAY)).HFITS = FITS_LIST (NO_IND - 1)

;
; Update the value in TMP_MAX_SC so that includes the highest study counter
; value for the current day being processed.
;

	               IF SDY_LIST (NO_IND - 1) GT TMP_MAX_SC THEN BEGIN
	                  TMP_MAX_SC = SDY_LIST (NO_IND - 1)
	               ENDIF

;
; Set the flag FITSEXIST to 1 (TRUE) in the entry in the table CDTBL which
; references the current day being processed.
;

	               CDTBL (LST (DAY)).FITSEXIST = 1

;
; Set HDR to the FITS file header for the last file that was produced on
; on the current day being processed.
;

	               HDR = CDSHEADFITS (CDTBL (LST (DAY)).HFITS)

;
; Set SRC to the string from the FITS file header in HDR which corresponds to
; the tag DATASRC.
;

	               SRC = FXPAR (HDR, "DATASRC")

;
; Check if the substring "CDROM" appears somewhere in the string in STR.  If
; it does not, then this means that at least one FITS file from CDROM sequence
; SEQ and volume VOL was not processed from CDROM data.  Consequently, we 
; will set the flag processed to 0.
;

	               IF STRPOS (SRC, "CDROM") EQ -1 THEN BEGIN
	                  PROCESSED = 0
	               ENDIF

	            ENDIF

;
; Increment DAY by 1 to look at the next day included in CDROM sequence SEQ
; volume VOL.
;

	            DAY = DAY + 1

	         ENDWHILE

;
; For all the days in the table CDTBL which refer to CDROM sequence SEQ and 
; volume VOL, set their processed field to value of PROCESSED.
;

	         CDTBL (LST).PROCESSED = PROCESSED

;
; Check if PROCESSED is set to 1 (TRUE).  If it is the update MAX_STUDY_COUNT
; to include the value in TMP_MAX_SC.
;

	         IF PROCESSED EQ 1 THEN BEGIN
	            IF TMP_MAX_SC GT MAX_STUDY_COUNT THEN BEGIN
	               IF TMP_MAX_SC LT 13000 THEN BEGIN
	                  MAX_STUDY_COUNT = TMP_MAX_SC
	               ENDIF
	            ENDIF
	         ENDIF

	      ENDIF

	   ENDFOR

        ENDFOR

;
; Print a message telling the user what the highest study counter value that
; we found was.
;

	PRINT, "Largest Study ID was: ", MAX_STUDY_COUNT

;
; Were Done !
;

	RETURN

	END