	PRO GET_CDROM_LST, P1, P2, ERRMSG=ERRMSG
;+
; Project     :	SOHO - CDS
;
; Name        :	GET_CDROM_LST
;
; Purpose     :	Get the dates from all the CDROM's that have been registered.
;
; Explanation :	Extracts the time in TAI format of the first second of all 
;		the days which have telemetry data stored on a CDROM.  The 
;		time is returned as part of a  structure which also contains
;		the sequence number and volume of the CDROM which contains the
;		data from that day.  Note that if more than one CDROM contains
;		the data from any particular date, then one structure will be
;		returned for every CDROM that contains data from that date.
;
; Use         :	GET_CDROM_LST, TBL, N_FOUND, ERRMSG = ERRMSG
;
; Inputs      :	None.
;
; Opt. Inputs :	None.
;
; Outputs     :	TBL	A structure variable containing the following tags
;			for each day:
;
;			STRUCT_TYPE= The character string 'CDS-CDROM-LIST'
;			DATE       = Time in TAI format of the first second
;				     of the day which the structure contains
;				     information (CDROM sequence/volume)
;				     about.
;			SEQUENCE   = The sequence number of the CDROM that 
;				     contains the data from the day in DATE.
;			VOLUME	   = The volume number of the CDROM that 
;				     contains the data from the day in DATE.
;			PROCESSED  = Set to 1 (TRUE) if the data in the CDROM
;				     indicated by SEQUENCE and VOLUME has been
;				     processed.
;				     This tag is not set by this procedure.
;			LFITS      = The name of the FITS file with the lowest
;				     study counter number created on the day
;				     in DATE. 
;				     This tag is not set by this procedure.
;			HFITS      = The name of the FITS file with the highest
;				     study counter number created on the day
;				     in DATE.
;				     This tag is not set by this procedure.
;			MINSC      = The minimum study counter number which was
;				     used on the day in DATE.
;				     This tag is not set by this procedure.
;			MAXSC      = The maximum study counter number which was
;				     used on the day in DATE.
;				     This tag is not set by this procedure.
;			FITSEXIST  = Set to 1 (TRUE) if there were any FITS
;				     files created on the day in DATE.
;
;		N_FOUND	Number of days found.
;
; Opt. Outputs:	None.
;
; Keywords    :	ERRMSG   = If defined and passed, then any error messages will
;			   be returned to the user in this parameter rather
;			   than depending on the MESSAGE routine in IDL.  If no
;			   errors are encountered, then a null string is
;			   returned.  In order to use this feature, ERRMSG must
;			   be defined first, e.g.
;
;				ERRMSG = ''
;				LIST_EXPER, ERRMSG=ERRMSG, ...
;				IF ERRMSG NE '' THEN ...
;
; Calls       :	DATATYPE, DBOPEN, DBFIND, DBEXT, DBCLOSE
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Category    :	Planning, Database.
;
; Prev. Hist. :	Based on LIST_EXPER Version 11, 30-Apr-1997
;		Written by William Thompson, GSFC.
;
; Written     :	Ron Yurow, GSFC, 19 August 1997
;
; Modified    :	Version 1, Ron Yurow, GSFC, 19 August 1997
;
; Version     :	Version 1, 19 August 1997
;-
;
	ON_ERROR, 2

;
; Set the parameter N_FOUND to 0.
;

	N_FOUND = 0

;
; Check the number of parameters.
;

        IF N_PARAMS() LT 2 THEN BEGIN
           MESSAGE = 'Syntax:  GET_FITS_FILES, OBS, N_FOUND'
	   GOTO, HANDLE_ERROR
        ENDIF

;
; Open the CDROM databases.
;

	DBOPEN, 'cdrom'
;
; Find all the entries in the currently open database.
;

	DBEXT, -1, 			$
               'sequence, volume, mjd', $
               SEQUENCE, 		$
	       VOLUME,			$
               MJD

;
; Set N_FOUND to the number of records which we extracted from the CDROM
; database.
; 

	N_FOUND = N_ELEMENTS (SEQUENCE)

;
; Set W to a long integer array of indexes in ascending order.  This will be
; used to copy the results of our database query into the array of structures.
;  

	W = LINDGEN (N_FOUND)

;
;  Define the structure that the data will be returned in.
;

	REC = { CDS_CDROM_LST,			$
		STRUCT_TYPE: 'CDS-CDROM-LST',	$
		SEQUENCE: 0,			$
                VOLUME: 0,			$
		PROCESSED: 0,			$
	        LFITS: "",			$
	        HFITS: "",			$
	        MINSC: 0,			$
	        MAXSC: 0,			$
		FITSEXIST: 0,			$
		DATE: 0.0D0}

;
; Set UTCDATE to UTC time structure with both tags set to 0.  This is used
; as a tempory value when converting MJD into TAI format. 
;

	UTCDATE  = {MJD: 0L, TIME: 0L}

; 
; Check if only one record was retrieved from the database.  If it was, then
; we can just copy the information from the fields into the single structure
; in REC.
;

	IF N_FOUND EQ 1 THEN BEGIN
		REC.SEQUENCE	= SEQUENCE (W(0))
                REC.VOLUME	= VOLUME (W(0))
		UTCDATE.MJD	= MJD (W(0))
		REC.DATE	= UTC2TAI (UTCDATE)

;
;
; Otherwise, replicate the structure REC as many times as we found records 
; from the database.  Also replicate the structure UTCDATE the same number of
; times.  Except for the date field,  copy all the fields into the appropiate
; tags in the array of structures we just created.  The date field is first
; copied to the MJD tag in the UTCDATE array of structures.  Then this array
; is copied to the DATE tag in REC array of structures after being coverted
; to TAI format.
;


	END ELSE BEGIN
		REC =     REPLICATE(REC, N_FOUND)
		UTCDATE = REPLICATE(UTCDATE, N_FOUND)
		REC.SEQUENCE	= SEQUENCE (W)
                REC.VOLUME	= VOLUME (W)
		UTCDATE.MJD	= MJD (W)     
                REC.DATE	= UTC2TAI (UTCDATE)
	ENDELSE

;
; Were done, so go to the exit point.
;

	GOTO, EXIT_POINT

;
;  Error handling point.
;

HANDLE_ERROR:

	IF N_ELEMENTS(ERRMSG) NE 0 THEN BEGIN 
           ERRMSG = 'GET_CDROM_LST: ' + MESSAGE 
        ENDIF ELSE MESSAGE, MESSAGE, /CONTINUE

;
;  Store the output in the correct variables, close the database and return.
;

EXIT_POINT:

;
; Close all the databases that we used.
;

	DBCLOSE

	P1 = REC
	P2 = N_FOUND

	RETURN
	END
