	PRO GET_FITS_FILES, P1, P2, ERRMSG=ERRMSG
;+
; Project     :	SOHO - CDS
;
; Name        :	GET_FITS_FILES
;
; Purpose     :	List the names of all the CDS FITS files in existence.
;
; Explanation :	Extracts the names of all the FITS files currently in existence
;		from the AS-RUN database.  The names of each FITS file are 
;		stored in a structure variable along with some other related
;		information
;
; Use         :	GET_FITS_FILES, OBS, N_FOUND, ERRMSG = ERRMSG
;
; Inputs      :	None.
;
; Opt. Inputs :	None.
;
; Outputs     :	OBS	A structure variable containing the following tags
;			for each planned observation:
;
;			STRUCT_TYPE= The character string 'CDS-FILE-LIST'
;			PROG_NUM   = Study counter number.  This forms the
;				     primary key for this database.
;			SEQ_IND    = The raster counter.
;			FILENAME   = FITS file name
;			DATE_OBS   = Date/time of beginning of observation,
;				     in TAI format
;			DATE_END   = Date/time of end of observation, in TAI
;				     format.
;			DATE_MOD   = Date and time when catalog record was last
;				     modified.
;
;		N_FOUND	= Number of observations found.
;
; Opt. Outputs:	None.
;
; Keywords    :	ERRMSG   = If defined and passed, then any error messages will
;			   be returned to the user in this parameter rather
;			   than depending on the MESSAGE routine in IDL.  If no
;			   errors are encountered, then a null string is
;			   returned.  In order to use this feature, ERRMSG must
;			   be defined first, e.g.
;
;				ERRMSG = ''
;				LIST_EXPER, ERRMSG=ERRMSG, ...
;				IF ERRMSG NE '' THEN ...
;
; Calls       :	DATATYPE, DBOPEN, DBFIND, DBEXT, DBCLOSE, TRIM
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Category    :	Planning, Database.
;
; Prev. Hist. :	Based on LIST_EXPER Version 11, 30-Apr-1997
;		Written by William Thompson, GSFC.
;
; Written     :	Ron Yurow, GSFC, 19 August 1997
;
; Modified    :	Version 1, Ron Yurow, GSFC, 19 August 1997
;
; Version     :	Version 1, 19 August 1997
;-
;
	ON_ERROR, 2

;
; Set the parameter N_FOUND to 0.
;

	N_FOUND = 0

;
;  Check the number of parameters.
;

        IF N_PARAMS() LT 2 THEN BEGIN
           MESSAGE = 'Syntax:  GET_FITS_FILES, OBS, N_FOUND'
	   GOTO, HANDLE_ERROR
        ENDIF

;
;  Open the as-run databases.
;

	VERSION = ''
	SAVE_N_FOUND = 0
	DBOPEN, 'experiment2, experiment1, experiment, main'

;
; Check if there are actually any entries in the experiment database.  If
; it is empty, then we will just ahead and look at the fexperiment database.
;

	LST = DBFIND ("PROG_NUM > 0")
	IF LST (0) EQ -1 THEN GOTO, FINISH

;
;  Find all the entries in the currently open database.
;

FIND_ALL:

;
; Extract the requested entries, sorted by start times.
;
	DBEXT, -1, 			$
               'prog_num, seq_ind, filename, date_obs, date_end, date_mod', $
               PROG_NUM, 		$
	       SEQ_IND,			$
               FILENAME,		$ 
	       DATE_OBS,		$
	       DATE_END,		$ 
	       DATE_MOD

;
; Set N_FOUND to the number of entries we found in the ASRUN database.
;

	N_FOUND = N_ELEMENTS (FILENAME)

;
; Set W to a long integer array of indexes in ascending order.  This will be
; used to copy the results of our database query into the array of structures.
;  

	W = LINDGEN (N_FOUND)
;
; Define the structure that the data will be returned in.  
;
	OBS = { CDS_FILE_LIST,		$
		STRUCT_TYPE: 'CDS-FILE-LIST',	$
		PROG_NUM: 0L,		$
                SEQ_IND: 0,             $
		FILENAME: 'String',	$
                DATE_OBS: 0.0D0,        $
                DATE_END: 0.0D0,        $
		DATE_MOD: 0.0D0}

; 
; Check if only one record was retrieved from the database.  If it was, then
; we can just copy the information from the fields into the single structure
; in OBS.
;

	IF N_FOUND EQ 1 THEN BEGIN
		OBS.PROG_NUM	= PROG_NUM(W(0))
                OBS.SEQ_IND     = SEQ_IND(W(0))
		OBS.FILENAME	= FILENAME(W(0))
                OBS.DATE_OBS    = DATE_OBS(W(0))
                OBS.DATE_END    = DATE_END(W(0))
		OBS.DATE_MOD	= DATE_MOD(W(0))

;
; Otherwise, replicate the structure OBS as many times as we found records 
; from the database.  Then copy all the fields into the appropiate tags in the 
; array of structures we just created. 
;

	END ELSE BEGIN
		OBS = REPLICATE(OBS, N_FOUND)
		OBS.PROG_NUM	= PROG_NUM(W)
                OBS.SEQ_IND     = SEQ_IND(W)
		OBS.FILENAME	= FILENAME(W)
                OBS.DATE_OBS    = DATE_OBS(W)
                OBS.DATE_END    = DATE_END(W)
		OBS.DATE_MOD	= DATE_MOD(W)
	ENDELSE

;
; Jump ahead to see if we have to switch databases and do this all over again.
;

	GOTO, FINISH

;
;  Error handling point.
;

HANDLE_ERROR:
	IF N_ELEMENTS(ERRMSG) NE 0 THEN ERRMSG = 'LIST_EXPER: ' + MESSAGE $
		ELSE MESSAGE, MESSAGE, /CONTINUE
	GOTO, EXIT_POINT
;
;  If the regular version of the database was searched, then search the "f"
;  version as well.
;

FINISH:
	IF (FIND_WITH_DEF('fexperiment2.dbf','$ZDBASE') NE '') AND $
		(VERSION EQ '') THEN BEGIN
	    VERSION = 'f'
	    SAVE_N_FOUND = N_FOUND
	    IF N_FOUND GT 0 THEN SAVE_OBS = OBS
	    DBOPEN, 'fexperiment2, fexperiment1, fexperiment, main'
	    GOTO, FIND_ALL
	ENDIF
;
;  Concatenate the two data arrays together.
;
	IF (VERSION EQ 'f') AND (SAVE_N_FOUND GT 0) THEN BEGIN
		IF N_FOUND EQ 0 THEN BEGIN
			N_FOUND = SAVE_N_FOUND
			IF N_FOUND GT 0 THEN OBS = SAVE_OBS
		END ELSE BEGIN
			N_FOUND = N_FOUND + SAVE_N_FOUND
			OBS = [OBS, SAVE_OBS]
			IF N_FOUND GT 1 THEN BEGIN
				S = SORT(OBS.DATE_OBS)
				OBS = OBS(S)
			ENDIF
		ENDELSE
	ENDIF
;
;  Store the output in the correct variables, close the database and return.
;

EXIT_POINT:

	DBCLOSE

	P1 = OBS
	P2 = N_FOUND

	RETURN
	END
