	PRO MK_CDROM_WEB, ERRMSG=ERRMSG
;+
; Project     :	SOHO - CDS
;
; Name        :	MK_CDROM_WEB
;
; Purpose     :	Create a set of WWW pages which describe the CDROM processing.
;
; Explanation :	Create a set of HTML formated documents that describes the 
;		current processing status of level 0 final distribution data
;		which is distributed on CDROMs.  The set of pages include a
;		top level document which is called 'cdrom_processing.html' 
;		and CDROM processing status table which is broken up over 
;		one or more subpages.  The top level page includes links to
;		all the subpages as well as a basic search engine. 
;
; Use         :	MK_CDROM_WEB, ERRMSG = ERRMSG
;
; Inputs      :	None.
;
; Opt. Inputs :	None.
;
; Outputs     :	None.
;
; Opt. Outputs:	None.
;
; Keywords    :	ERRMSG   = If defined and passed, then any error messages will
;			   be returned to the user in this parameter rather
;			   than depending on the MESSAGE routine in IDL.  If no
;			   errors are encountered, then a null string is
;			   returned.  In order to use this feature, ERRMSG must
;			   be defined first, e.g.
;
;				ERRMSG = ''
;				LIST_EXPER, ERRMSG=ERRMSG, ...
;				IF ERRMSG NE '' THEN ...
;
; Calls       :	FIX_ZDBASE
;		GET_FITS_FILES
;		GET_CDROM_LST
;		CHK_CD_STATUS
;		ROWSORT
;		WR_WELCM_PG
;		WR_TBL_HDR
;		WR_CD_HTML
;
; Common      :	TBL, 
;		TBLSIZE
;		CDTBL, 
;		CD_TBL_SIZE
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Category    :	Planning, Database.
;
; Prev. Hist. :	none.
;
; Written     :	Ron Yurow, GSFC, 18 September 1997
;
; Modified    :	Version 1, Ron Yurow, GSFC, 18 September 1997
;
; Version     :	Version 1, 18 September 1997
;-
;

;
; Common Block for variables used both by this procedure and by WR_CD_HTML.
;

	COMMON FITS_NAME_TBL, TBL, TBLSIZE

;
; Common Block for variables used both by this procedure and by CHK_CD_STATUS.
;

	COMMON CDROM_TBL, CDTBL, CD_TBL_SIZE

;
; Set the error handling procedure to return to the caller of this procedure
; if an error occurs.
;

	ON_ERROR, 2

;
; Make sure that we are useing the official CDS database.
;

	STATUS = FIX_ZDBASE (/CDS)

;
; Tell the user that we are going to start compiling a table which lists
; which FITS files were created on which day.  This will take a while...
;

	PRINT, "Searching the AS-RUN Database for FITS filenames..."

;
; Call the procedure GET_FITS_FILES to compile a table which list which
; which FITS files were created on which day.  This will be stored in the
; array TBL.
;

	GET_FITS_FILES, TBL, TBLSIZE, ERRMSG = ERRMSG

;
; Print a message telling the user that we are done.
;

	PRINT, "Done."

;
; Tell the user that we are going to start compiling a table which list every
; CDROM that has been registered so far and what days each CDROM covers.  This
; will take while...
;

	PRINT, "Searching the CDROM Database..."

;
; Call the procedure GET_CDROM_LST to compile a table which lists every CDROM
; that has been registered so far and what days each CDROM covers. This will
; be stored in the array CDTBL.  
; 

	GET_CDROM_LST, CDTBL, CD_TBL_SIZE, ERRMSG = ERRMSG

;
; Print a message telling the user that we are done.
;

	PRINT, "Done."

;
; Tell the user that we are going to start running a procedure to calculate 
; which FITS files were run on which days and whether those FITS files were
; generated from telemetry data or from CDROMs.  This also may take a while...
;

	PRINT, "Verifing the processing status of all CDs..."

;
; Call the procedure CHK_CD_STATUS to calculate which FITS files were run on
; which days and whether those FITS files were generated from telemetry data
; or from CDROMs.  This information is stored in the array CDTBL which was 
; created by the procedure GET_CDROM_LST and passed to this procedure as 
; part of a common block.
;

	CHK_CD_STATUS, MAXSC = MAXSC, MAXSEQ = MAX_SEQ

;
; Print a message telling the user that we are done.
;

	PRINT, "Done."

;
; Create an array to hold both the mission days and the CDROMs that go with 
; those days in the same order that they appear in array of structures, 
; CDTBL.
;

	CDDATE = INTARR (2, CD_TBL_SIZE)

;
; Copy the date elements from the array of structures CDTBL into the 
; array UT.  As they are copied, convert them from TAI format to UTC.
;

	UT = TAI2UTC (CDTBL.DATE)

;
; Extract the modified julian day from the array of UTC times CDDATE.  Then 
; subtract 50053, which is the modified julian day when SoHO was launched.
; This gives us the SoHO mission day.  Store it back in the array UT.
;

 	UT = UT.MJD - 50053

;
; Copy the array UT into column 0 of the array CDDATE.
;

	CDDATE (0, *) = UT

;
; Copy the sequence elements from the array of structures CDTBL into column
; 1 of the array CDDATE.
;

	CDDATE (1, *) = CDTBL.SEQUENCE

;
; Set INDEX to the array of indices which will result in UT being sorted
; in ascending order.
;

	INDEX = ROWSORT (CDDATE, 0, 1)


;
; Reset INDEX to the array of indices which will result in the unique elements
; of the array UT being sorted in ascending order.
;

	INDEX = INDEX (UNIQ (UT (INDEX)))

;
; Create a 3 column integer array LST which has one row for every unique 
; element of UT.
; 

	LST = INTARR (N_ELEMENTS (INDEX), 3)


;
; Set up the array LST to contain a sorted list of SoHO mission days in the
; first column.  In the second will the CDROM sequence number where the 
; telemetry data for that mission day is stored and the third column will be
; the CDROM volume within that sequence.
;

	LST (*, 0) = UT (INDEX)
	LST (*, 1) = CDTBL (INDEX).SEQUENCE
	LST (*, 2) = CDTBL (INDEX).VOLUME

;
; Call the procedure WR_WELCM_PG to create a top level web page with links
; to each of the subpages containing CDROM processing information.
;

	WR_WELCM_PG, MAX_SEQ, MAXSC, LST

;
; Set NO_FILES to the number of files to create.  Each file will contain a
; HTML table describeing the CDROM processing status for 20 sequences of 
; CDROMs.  This is set intitially to the number of sequences divided by 20.
;

	NO_FILES = FIX (MAX_SEQ / 20)

;
; Adjust NO_FILES, or the number of files to create, to handle the case where
; the number of sequences is not evenly divisible by 20.
;

	IF NO_FILES * 20 LT MAX_SEQ THEN NO_FILES = NO_FILES + 1

;
; Print a message to the user telling him/her what the highest sequence number
; that exists in the catalog so far.
;

	PRINT, "Highest CD Volume in catalog is ", MAX_SEQ

;
; Set SEQ to 1.  We will use this variable to keep track of which sequence of
; CDROMs we are currently processing.
;

	SEQ = 1

;
; Main loop to create each of the WWW pages that, when taken together, will 
; describe the processing status of the CDROMs.  These pages will all be 
; accessable the top level page creater earlier.
;

	FOR FILE = 1, NO_FILES DO BEGIN

;
; Set FILENAME to string conaining the name of the file which contain the 
; current WWW page being created. 
;

	   FILENAME = STRING (FILE, FORMAT = '("cd_status_", I0, ".html")')

;
; Tell the user which file where writeing the CDROM status WWW page to.
;

	   PRINT, "Writeing CD status data to ", FILENAME

;
; Open the file to write the WWW page to.
;

           OPENW, LUN, FILENAME, /GET_LUN 

;
; Call the procedure WR_TBL_HDR to write the initial HTML text to start the
; HTML table which will contian the CDROM processing information.
;

	   WR_TBL_HDR, LUN

;
; Set LIMIT to the last CDROM sequence number which will be included in the
; current WWW page being generated.  
;

	   LIMIT = SEQ + 20

;
; Check if LIMIT is set to the last CDROM sequence number in existence.  If it
; is, then increment LIMIT by one so this sequence number will be included on
; on the current page.
;

	   IF LIMIT GT MAX_SEQ THEN LIMIT = MAX_SEQ + 1

;
; Loop to add the data for all the CDROMs with sequence numbers between the
; current value of SEQ and LIMIT to the WWW page we are createing.  As the
; information on each sequence of CDROMs is added to the web page, SEQ is
; is incremented by 1.
;

           WHILE SEQ LT LIMIT DO BEGIN

;
; Set the MAX_VOL the highest CDROM volume number in the sequence which is
; currently being processed.
;

	      MAX_VOL = MAX (CDTBL (WHERE (CDTBL.SEQUENCE EQ SEQ)).VOLUME)

;
; Loop to add information on each CDROM volume in the current sequence being 
; processed to the HTML table.  CDROM volumes are assumed to begin at 1 and
; go up to the highest volume number in the sequence, which is contained in
; MAX_VOL.
;

	      FOR VOL = 1, MAX_VOL DO BEGIN

;
; Set LST to a vector of subscripts where each subscript points to a element
; in the array CDTBL which refers to the sequence SEQ and volumne VOL.  The
; elements differ in which day they refer to.
;

	         LST = WHERE (CDTBL.SEQUENCE EQ SEQ AND CDTBL.VOLUME EQ VOL)

;
; Check that LST contians at least one subsript of CDTBL.  If no subscipts were
; found (no elements refer to sequence SEQ and volume VOL), then LST will be
; set to -1.
;

	         IF LST (0) NE -1 THEN BEGIN

;
; Call the function WR_CD_HTML to add the data for CDROM sequence SEQ volume
; VOL to the HTML table being created.  
;

	            WR_CD_HTML, SEQ, VOL, MAX_VOL, CDTBL (LST), LUN

	         ENDIF

	      ENDFOR

;
; Flush the buffer which attached to the currently open file.  This will allow
; the file to be examined as it is created.
;

	      FLUSH, LUN

;
; Increment SEQ by 1.  We now move on to process the next sequence.
;

	      SEQ = SEQ + 1

           ENDWHILE

;
; At this point were done with the writeing the HTML WWW page that we are
; currently generating.  Write the last HTML code to end the table.
;

	   PRINTF, LUN, "</table>"

;
; Write the HTML codes to end the WWW page.
;

	   PRINTF, LUN, "</center>"	   
	   PRINTF, LUN, "</body>"
	   PRINTF, LUN, "</html>"

;
; Close the file that we are writeing the HTML WWW page to.  This in the end
; of our outermost loop.
;

           CLOSE, LUN
        
	ENDFOR      

;
; Were done generateing web pages, so just get out of here!!
;
        
        RETURN

;
;  Error handling point.
;

HANDLE_ERROR:
	IF N_ELEMENTS(ERRMSG) NE 0 THEN ERRMSG = 'MK_CDROM_WEB: ' + MESSAGE $
		ELSE MESSAGE, MESSAGE, /CONTINUE
	RETURN
        

        END
          