	PRO WR_CD_HTML, SEQ, VOL, MVOL, LST, LUN
;+
; Project     :	SOHO - CDS
;
; Name        :	WR_CD_HTML
;
; Purpose     :	Write the processing status of one CDROM to an HTML table.
;
; Explanation :	This procedure write the processing status of one CDROM to
;		a table in an HTML document.  The file containing the HTML
;		document must have previously been opened for writeing and 
;		the HTML format codes to begin a table already written to 
;		it.  This procedure writes the rows nescesary to describe the
;		processing status for one CDROM to the table, one row for each
;		day which is included in the CDROM.  
;
; Use         :	WR_CD_HTML, SEQ, VOL, MVOL, LST, LUN 
;
; Inputs      :	SEQ	The sequence number of the CDROM to be added 
;			to the processing status table.
; 
;		VOL	The volume number of the CDROM to be added 
;			to the processing status table.
;
;		MVOL	The highest volume number in CDROM sequence SEQ.
;
;		TBL	A array of structures, one structure for each day
;			included in CDROM sequence SEQ volume VOL.  Each 
;			structure has the following tags:
;
;			STRUCT_TYPE= The character string 'CDS-CDROM-LIST'
;			DATE       = Time in TAI format of the first second
;				     of the day which the structure contains
;				     information (CDROM sequence/volume)
;				     about.
;			SEQUENCE   = The sequence number of the CDROM that 
;				     contains the data from the day in DATE.
;			VOLUME	   = The volume number of the CDROM that 
;				     contains the data from the day in DATE.
;			PROCESSED  = Set to 1 (TRUE) if the data in the CDROM
;				     indicated by SEQUENCE and VOLUME has been
;				     processed.
;			LFITS      = The name of the FITS file with the lowest
;				     study counter number created on the day
;				     in DATE. 
;			HFITS      = The name of the FITS file with the highest
;				     study counter number created on the day
;				     in DATE.
;			MINSC      = The minimum study counter number which was
;				     used on the day in DATE.
;			MAXSC      = The maximum study counter number which was
;				     used on the day in DATE.
;			FITSEXIST  = Set to 1 (TRUE) if there were any FITS
;				     files created on the day in DATE.
;
;		LUN	The logical unit number of the file to write the
;			HTML format codes to.  The file must have been 
;			opened for writeing by the calling procedure.
;
; Opt. Inputs : None.
;
; Outputs     :	None.
;
; Opt. Outputs:	None.	
;
; Keywords    :	None.
;
; Calls       :	UTC2STR,
;		TAI2UTC
;
; Common      :	None.
;
; Restrictions:	The procedure is only designed to be called by the procedure 
;		MK_CDROM_WEB.PRO
;
; Side effects:	Writes HTML format codes to the file referenced by the logical
;		unit number passed to the procedure as the parameter LUN.
;
; Category    :	Planning, Database.
;
; Prev. Hist. :	none.
;
; Written     :	Ron Yurow, GSFC, 24 October 1997
;
; Modified    :	Version 1, Ron Yurow, GSFC, 24 October 1997
;
; Version     :	Version 1, 24 October 1997
;-

;
; Create an array of strings, where each string contains one of the dates
; which is included in the CDROM being processed.
;

	DATE = UTC2STR (TAI2UTC (LST.DATE), /DATE_ONLY)

;
; Set NO_DATES to the number of dates included in the CDROM being processed.
;

	NO_DATES = N_ELEMENTS (LST)

;
; Set SEPERATER to a string which will be used to visually seperate the HTML
; format codes for each row in the table we are createing.
;

        SEPERATER = " ------------------------------------------->"

;
; Write an HTML comment line describeing the sequence and volume number of 
; of the CDROM being processed.
;

	PRINTF, LUN, SEQ, FORMAT = '("<!--***** Sequence No. ", I3, $)'
        PRINTF, LUN, VOL, SEPERATER, FORMAT = '(" Volume ", I3, A)'

;
; Write the HTML format codes to begin the first row describeing the CDROM
; which is currently being processed.  This row is different then the other
; rows which may follow it since contain an extra colum.  The extra column
; is used to hold the name of the CDROM.  Here we will only begin the row
; by setting the font size and some other formatting information.  
;

	PRINTF, LUN, "<tr> ", FORMAT = '(A, $)
	PRINTF, LUN, NO_DATES, FORMAT = '("<td rowspan = ", I0, " ", $)'
	PRINTF, LUN, "valign = top>", FORMAT = '(A, $)'
	PRINTF, LUN, "<font size = +1 color = #FF0000>"

;
; Create an unique anchor based on the sequence and volume number of the 
; CDROM being processed.  This is stored in the string variable ANCHOR.
;

	ANCHOR = STRING (SEQ, VOL, FORMAT = '("S", I0, "V", I0)')

;
; Write the HTML format codes for the first column of the first row 
; describeing the CDROM which currently being processed.  This field will hold
; the sequence and volume number of the CDROM.  It will be defined so that it
; span all the rows for this CDROM, the following rows will begin after this
; column.  
;

	PRINTF, LUN, '<a name = "', ANCHOR, '">', FORMAT = '(A, A, A, $)'
	PRINTF, LUN, SEQ, FORMAT = '("Volume: ", I0, " <br>", $)'
        PRINTF, LUN, VOL, MVOL, FORMAT = '(I0, " of ", I0, $)'
	PRINTF, LUN, "</a>"
        PRINTF, LUN, "</font> </td>"
	
;
; Loop to add the HTML format codes for the rest of the first row and all
; sucessor rows that will be used to describe the processing status of the
; current CDROM.  Since each row will now have the same number of cells 
; (the format codes for the extra cell in the first row have already been
; written), the loop can take care of writeing all the rows.  This loop
; is executed once for every date included in the CDROM being processed. 
;

        FOR I = 0, NO_DATES - 1 DO BEGIN 

;
; Write an HTML comment line describeing the date which the current row
; will cover. 
;

	   PRINTF, LUN, DATE (I), FORMAT = '("<!--      DAY: ", A, $)'
           PRINTF, LUN, SEPERATER, FORMAT = '("            ", A)'

;
; Check if this is the first iteration of the loop.  If it is, then HTML forat
; codes to begin a table row has already been written (the first row in fact
; already has one cell in it).  In this case will just write some spaces to
; make everything look pretty.  Otherwise, write the HTML format code to 
; begin a new table row.
;

	   IF I NE 0 THEN BEGIN 
              PRINTF, LUN, "<tr> ", FORMAT = '(A, $)'
	   ENDIF ELSE BEGIN
	      PRINTF, LUN, "     ", FORMAT = '(A, $)'
	   ENDELSE
 
;
; Write the HTML format codes to create a cell with the date that the data 
; in the row is for.
;

           PRINTF, LUN, "<td>",  FORMAT = '(A, $)
	   PRINTF, LUN, DATE (I), FORMAT = '(A, $)'
	   PRINTF, LUN, "</td>", FORMAT = '(A, $)

;
; Check if the CDROM has been processed.  If the CDROM has not been processed
; then the flag PROCESSED will be set to zero in each of the structures that
; make up the array of structures LST.  If the CDROM has not processed, then
; rest of the row will be made of one big cell with the words.
; "CD Not Processed".  
;

	   IF LST (I).PROCESSED EQ 0 THEN BEGIN

;
; Check if this the first iteration of the loop.  If it is, then we will have
; Write the HTML format codes to create one big cell that will span all the
; remaining columns in the row as well as the rest of the rows that will be
; created for the current CDROM.  
;
	      IF I EQ 0 THEN BEGIN
                 PRINTF, LUN, "<td colspan = 3 ", FORMAT = '(" ", A, $)'
                 PRINTF, LUN, NO_DATES, FORMAT = '("rowspan = ", I0, " ", $)'
                 PRINTF, LUN, "align = center>",  FORMAT = '(" ", A, $)'
	         PRINTF, LUN, "CD Not Processed.", FORMAT = '(A, $)'      
	         PRINTF, LUN, "</td> </tr>"

;
; Otherwise, just write the HTML format code to end the row.
;

	      ENDIF ELSE PRINTF, LUN, " </tr>"

;
; Otherwise, we assume that the CEROM has been processed.  Now we check to
; see if any FITS files were created on the date which this row is listing
; information for.  If no FITS files were created on this date, then the
; rest of the row will consist of one cell with the words " No Studies This
; Day". 
;

	   ENDIF ELSE IF LST (I).FITSEXIST EQ 0 THEN BEGIN

              PRINTF, LUN, "<td colspan = 3 ", FORMAT = '(" ", A, $)'
              PRINTF, LUN, "align = center>",  FORMAT = '(" ", A, $)'
	      PRINTF, LUN, "No Studies This Day.", FORMAT = '(A, $)'      
	      PRINTF, LUN, "</td> </tr>"

;
; Otherwise, FITS were created on this date.  Therefore, lets complete 
; the row normally.
;
	   ENDIF ELSE BEGIN

;
; Write the HTML format codes to create a cell with the first FITS file
; created on the day covered by this row.
;

              PRINTF, LUN, "<td>",  FORMAT = '(" ", A, $)     
	      PRINTF, LUN, LST (I).LFITS, FORMAT = '(A, $)'
	      PRINTF, LUN, "</td>", FORMAT = '(A, " ", $)

;
; Write the HTML format codes to create a cell with the last FITS file
; created on the day covered by this row.
;


              PRINTF, LUN, "<td>", FORMAT = '(A, $)     
	      PRINTF, LUN, LST (I).HFITS, FORMAT = '(A, $)'
	      PRINTF, LUN, "</td>"
;
; Write the HTML format codes to create a cell with the words no comment,
; written in the same color as the backround.  Comments are not implemented
; in the current version of this software, although they may be implemented
; in a future release.  This cell ends the current row.
;

              PRINTF, LUN, '<td><font color = "FFFFFF" >'
              PRINTF, LUN, "<!-- Place Comment Here --> no comment."
	      PRINTF, LUN, "</font> </td> </tr>"

	   ENDELSE

	ENDFOR

;
; Write two blank lines the file that contains the HTML table we are 
; createing.
;                          

	PRINTF, LUN
	PRINTF, LUN
;
; Were done!!!
;

	RETURN

	END
