;+
; Project     : SOHO - CDS     
;                   
; Name        : EMON_GIS_HV_CNT
;               
; Purpose     : Extract and plot GIS HV and counts from CDS tm file.
;               
; Explanation : Packages EMON in batch mode to extract the GIS HV and raw
;               counts from a telemetry file.
;               
; Use         : IDL> emon_gis_hv_cnt, tm_file [, a,b,c, and various keywords]
;    
; Inputs      : tm_file  -  telemetry file(s) name.  Searches in current
;                           (or specifed) directory first and then if no
;                           directory was specified it looks in  $CDS_TM_DATA
;               
; Opt. Inputs : None
;               
; Outputs     : None
;               
; Opt. Outputs: a,b,c  -  standard outputs as from EMON q.v. If an array
;                         of telemetry files is used, these output parameters
;                         refer only to the last file used.
;               
; Keywords    : HV       -  extract only High Voltage data
;               COUNTS   -  extract only raw counts
;               HARDCOPY - print hardcopy of plots 
;
; Calls       : EMON
;
; Common      : None
;               
; Restrictions: None
;               
; Side effects: None
;               
; Category    : Telemetry, instrument, CDS
;               
; Prev. Hist. : None
;
; Written     : C D Pike, 22-Jun-95
;               
; Modified    : 
;
; Version     : Version 1, 22-Jun-95
;-            

pro emon_gis_hv_cnt, tmfile, a, b, c, hv=hv, counts=counts, hardcopy=hardcopy

;
;  check parameters
;
if n_params() eq 0 then begin
   print,'Use: emon_gis_hv_cnt, tmfile [,a,b,c + keywords'
   return
endif

;
;  if file specification is not an array, let it be a wildcard
;
break_file,tmfile(0),disk,dir,f,e
usefile = findfile(tmfile,count=count)
if (count eq 0) and (dir eq '') then begin
   usefile = findfile(concat_dir('$CDS_TM_DATA',tmfile),count=count)
   if count eq 0 then begin
      print,'No telemetry files found. ',tmfile
      return
   endif
endif

;
;  warn if multiple files
;
if n_elements(usefile) gt 1 then begin
   print,'Input translates into ',n_elements(usefile),' telemetry files.',$
          form='(a,i3,a)' 
endif

;
;  which parameters were required?
;
parms=['BUGHV1','BUGHV2','BUGHV3','BUGHV4','BKGRCD1','BKGRCD2','BKGRCD3',$
       'BKGRCD4']
if keyword_set(hv) then parms = parms(0:3)
if keyword_set(counts) then parms = parms(4:7)


;
;  was hardcopy requested?
;
if keyword_set(hardcopy) then ps

;
;  set up 4 plots per page
;
!p.multi = [0,1,4]

;
;  loop in case file spec is an array
;
for i=0,n_elements(usefile)-1 do begin
   
   emon,a,b,c,fname=usefile(i),pname=parms,/batch
   for i=0,n_elements(parms)-1 do begin
      cd = tm_calib(c(i),b(i,*),words)
      plot_tm_calib,a,cd,words
      ta = time_average(str2utc(a,/dmy),cd,min=15,top=top,bottom=bottom)
      n = n_elements(ta)
      for j=0,n-1 do begin
        print,anytim2cal(ta(j).date),' ',ta(j).data
        ypos = (!y.crange(1)-!y.crange(0))/10. + !y.crange(0)
        utstring,ta(j).date,ypos,strtrim(string(ta(j).data,form='(f8.3)'),2)
      endfor
      print, ' '
      print, ' '
   endfor
endfor

;
;  print hardcopy
;
if keyword_set(hardcopy) then psplot

;
;  reset plot
;
!p.multi = 0

end
