;+
; Project     : SOHO - CDS     
;                   
; Name        : EMON_LUT
;               
; Purpose     : To extract GIS LUT details from telemetry.
;               
; Explanation : Extracts the details of LUTs loaded for the GIS
;               
; Use         : IDL> emon_lut [,doy, hours, year=year, /hard, /rt]
;
;               eg.  IDL> emon_lut,244,[13,14],year=94,/hard
;
;                    would extract data for the 2 hrs 13:00-15:00 on 1/Sep/94
;
;                    IDL> emon_lut,/rt
;
;                    would extract data from current realtime session.
;    
; Inputs      : One of DOY or /rt must be supplied.
;               
; Opt. Inputs : DOY   - day of year for file name.  Only one day at a time is
;                       allowed.
;
;               HOURS - scalar or vector giving the UT hours required (as in
;                       file names). If not realtime and not specified then
;                       hours 0-23 are assumed.
;         
;               
; Outputs     : None
;               
; Opt. Outputs: None
;
;               
; Keywords    : YEAR    - specify the year IF NOT current year.
;
;               HARD    - if want hard copy of plots.
;
;               RT      - if want to pick up only current realtime session.
;                         Note this will only read the current file to the end.
;                         It will not sit and wait for more data to come in
;                         as STM does in realtime mode.
;
;               KEEP    - if present the emon extracted parameters are SAVEd
;                         to a file. ('time'.save)
;
;
; Calls       :
;
; Common      : None
;               
; Restrictions: Data files must be in $CDS_TM_DATA directory.
;               String time specifications are assumed in European format
;               ie day,month,year order.
;               
; Side effects: None
;               
; Category    : Util, telemetry
;               
; Prev. Hist. : None
;
; Written     : C D Pike, RAL, 2-Feb-95
;               
; Modified    : 
;
; Version     : Version 1, 2-Feb-95
;-            

pro emon_lut, doy, hour, year=year, hard=hard, rt=rt, keep=keep

;
;  gather data from file
;
;  check input
;
if n_params() eq 0 then begin
   if not keyword_set(rt) then begin
      print,'Use:  IDL> emon_lut, [doy, hour, /hard, /rt]'
      print,' '
      print,'NB either doy OR /rt must be set.'
      print,' ' 
      return
   endif
endif
   
;
;  either old file or current realtime
;
if n_params() eq 1 and keyword_set(rt) then begin
   print,'Cannot specify doy AND /rt.'
   return
endif

;
;  only one day allowed
;
if n_elements(doy) gt 1 then begin
   print,'Only one day at a time please!'
   return
endif

;
;  if want realtime session...
;
if keyword_set(rt) then begin
   tmfiles = concat_dir('$CDS_TM_DATA',gt_rt_file())
endif else begin

;
;  else select old TM files
;

;
;  format day number
;
   cday = strmid(string(doy+1000,form='(i4)'),1,3)

;
;  if not current year...
;
   if keyword_set(year) then begin
      if year gt 1000 then yr = year - 1900 else yr = year
      yr = string(yr,form='(i2)')
   endif else begin
      yr = strmid(systime(),22,2)
   endelse

;
;  depending on how many hours requested, form file names
;
   if n_params() eq 1 then hour = indgen(24)
   nf = n_elements(hour)
   tmfiles = strarr(nf)
   for i=0,nf-1 do begin
      chr = strmid(string(hour(i)+100,form='(i3)'),1,2)
      fdum = 'tm.'+yr+'_'+cday+'_'+chr
      tmfiles(i) = concat_dir('$CDS_TM_DATA',fdum)
   endfor
endelse

print,'Using data files.....'
print_str,tmfiles
break_file,tmfiles(0),disk,dir,fil,ext
out_file = 'LUT'+strmid(fil+ext,2,10)
                            

;
;  after the GIS heart, LUT loading and LUT parameters
;
name = ['BSGHEART', 'BSGLUT', 'BKGTAB1', 'BKGTAB2', 'BKGTAB3', 'BKGTAB4',$
        'BKGTAB5', 'BKGTAB6', 'BKGTAB7', 'BKGTAB8', 'BKGTAB9', 'BKGTAB10',$
        'BKGTAB11']



;
;  call emon to do the work
;
emon,ut_data,parm_data,parms,fname=tmfiles,pname=name,/batch

;
;  trap error return
;
if ut_data(0) eq 0 then return



;
;  check on LUTs in order
;
if keyword_set(hard) then begin
   openw,lun,out_file,/get_lun
endif else begin
   hard = 0 
endelse

;
;  print search times as header
;

print,'LUT search.....'
print,'Files used: '
if hard then printf,lun,'Files used: '
for i=0,n_elements(tmfiles)-1 do begin
   print,' ',tmfiles(i)
   if hard then printf,lun,tmfiles(i)
endfor
print,' '
print,'Start time: ',ut_data(0)
print,'End time:   ',ut_data(n_elements(ut_data)-1)
print,' '
if hard then begin
   printf,lun,' '
   printf,lun,'LUT search.....'
   printf,lun,'Start time: ',ut_data(0)
   printf,lun,'End time:   ',ut_data(n_elements(ut_data)-1)
   printf,lun,' '
endif

for j=1,4 do begin
   print,' '
   if hard then printf,lun,' '
   n = where(parm_data(0,*) eq 2 and $ ; heart beating
             parm_data(1,*) eq 0 and $ ; LUT loading
             parm_data(9,*) eq j)      ; GIS LUT #1

   if n(0) ge 0 then begin
      t1 = ut_data(n)
      for i=0,10 do begin
         if i eq 0 then print,'Time: ',ut_data(n(0))
         if i eq 0 and hard then printf,lun,'Time: ',ut_data(n(0))
         parm = mode_val(parm_data(i+2,n))
         if (i eq 1) or (i eq 2) then parm = parm/10.
         if (i eq 3) or (i eq 8) or (i eq 9) or (i eq 10) then begin
            parm = parm/100.
         endif
         if i eq 4 then parm = parm * 2.
         if i eq 5 then parm = parm / 40.        
         print,'LUT #',j,': Parameter ',i+1,$
               ' value:',parm,form='(a,i1,a,i2,a,f6.2)'
         if hard then printf,lun,'LUT #',j,': Parameter ',i+1,$
               ' value:',parm,form='(a,i1,a,i2,a,f6.2)'
      endfor
   endif else begin
      print,'No LUT load for LUT #',j
      if hard then printf,lun,'No LUT load for LUT #',j
   endelse
endfor


;
;  close hardcopy
;
if hard then free_lun,lun

;
;  save if required
;
if keyword_set(keep) then begin
   save,ut_data,parm_data,parms,$
        file=out_file+'.save'
endif

end





