;+
; Project     : SOHO - CDS     
;                   
; Name        : EMON_MECH_LOG
;               
; Purpose     : To summarize CDS mechanism movements.
;               
; Explanation : Reads a telemetry files, packet by packet and records 
;               the mechanism movements found in the file. Note that it only
;               logs the movements recorded in raster and exposure header
;               packets, so it will not see any engineering sequences (eg the
;               lubrication sequence).  Since the vast majority of movements
;               are expected to be performed as integral parts of rasters, this
;               should give a good approximation to the mechanism history.
;               
; Use         : IDL> emon_mech_log, input
;    
; Inputs      : input - can be one of:  DOY number (all files read)
;                                       Specific date - in CDS string format
;                                                  (all files read)
;                                       CDS UTC structure
;                                       Specific telemetry file name (or part
;                                       thereof - eg 'tm.95_123'  will pick
;                                       all files from DOY 123
;
; Opt. Inputs : None     
;               
; Outputs     : Save file is updated
;               
; Opt. Outputs: None
;               
; Keywords    : QUIET     - no screen output.
;               ALL_CDROM - use all TM files in $CDS_CDROM_DATA
;               STATUS    - set to 0/1 on exit for error/success
;
; Calls       : None
;
; Common      : None
;               
; Restrictions: The file selection will fall over when the year changes!
;               Requires TM files from $CDS_CDROM_DATA (first) or 
;               $CDS_TM_DATA (backup) unless ALL_CDROM keyword is set in 
;               which case all the (and only those) tm files in CDS_CDROM_DATA
;               are used.
;
; Side effects: None
;               
; Category    : Telemetry, mechanisms
;               
; Prev. Hist. : None
;
; Written     : C D Pike, RAL, 23-Nov-95
;               
; Modified    : Trap insufficient data and rework file input.  CDP, 29-Jan-96
;               Fix typo bug, add CDS_CDROM_DATA and add QUIET keyword.
;                                                              CDP, 10-Apr-96
;               Add ALL_CDROM keyword.                         CDP, 11-Apr-96
;               Kill any duplicate entries caused by reruns.   CDP, 12-Apr-96
;               Increased mirror posn array size and trimmed others.
;                                                              CDP, 10-Jun-96
;               Added check for no telemetry files.  CDP, 29-Apr-97
;               Set on_error,0  for debugging.  CDP, 8-May-97
;               Fix integer round off bug.  CDP, 14-May-97
;               Increase ops_left etc. array size.  CDP, 18-Jun-97
;
; Version     : Version 10, 18-Jun-97
;-            

pro emon_mech_log, file, quiet=quiet, all_cdrom=all_cdrom, status=status

        
on_error,0

;
;  does user have privilege to write data?
;
direct = getenv('CDS_ENG_DATA_W')
if direct eq '' then begin
   if not keyword_set(quiet) then begin
      print,'Environment variable CDS_ENG_DATA_W is not defined.'
   endif
   status = 0
   return
endif


    
;
;  input can be
;               none (+ ALL_CDROM keyword) - use all files in $CDS_CDROM_DATA 
;               doy    (specific day of year number, all files)
;               CDS  UTC structure (use all files from that day)
;               CDS  date string (use all files from it)
;               Specific file name (use only it)
;

if n_params() eq 0  then begin
   if keyword_set(all_cdrom) then begin
      ff = findfile(concat_dir('$CDS_CDROM_DATA','tm.*'),count=count)
      if count eq 0 then begin
         if not keyword_set(quiet) then begin
            print,'No telemetry files found in CDS_CDROM_DATA '
         endif
         status = 0
         return
      endif
   endif else begin
      if not keyword_set(quiet) then begin
         print,'USE: emon_mech_log, DOY or CDS date or telemetry file name.'
      endif
      status = 0
      return
   endelse
endif else begin


;
;  DOY specified
;

   if datatype(file(0),1) eq 'Integer' or datatype(file(0),1) eq 'Long' then begin
      cdoy = strmid(string(file+1000,form='(i4)'),1,3)
      date = anytim2cal(doy2utc(file))
      year = strmid(date,7,2)
      ff = findfile(concat_dir('$CDS_CDROM_DATA',$
                    'tm.'+year+'_'+cdoy+'*'),count=count)
      if count eq 0 then ff = findfile(concat_dir('$CDS_TM_DATA',$
                                       'tm.'+year+'_'+cdoy+'*'),count=count)
      if count eq 0 then begin
         print,'No telemetry files found for DOY ',file
         status = 0
         return
      endif
   endif

;
;  CDS UTC structure
;
   if datatype(file(0),1) eq 'Structure' then begin
      date = anytim2cal(file)
      doy = utc2doy(file)
      cdoy = strmid(string(file+1000,form='(i4)'),1,3)
      year = strmid(date,7,2)
      ff = findfile(concat_dir('$CDS_CDROM_DATA',$
                    'tm.'+year+'_'+cdoy+'*'),count=count)
      if count eq 0 then ff = findfile(concat_dir('$CDS_TM_DATA',$
                                       'tm.'+year+'_'+cdoy+'*'),count=count)
      if count eq 0 then begin
         print,'No telemetry files found for UTC ',file
         status = 0
         return
      endif
   endif
   

                   
;
;  CDS date/time string or telemetry file name (check for directory already
;  being there)
;
   if datatype(file(0),1) eq 'String' then begin
      break_file, file, disk, direct, f, ext
      if direct(0) eq '' then begin
          if strmid(file(0),0,2) ne 'tm' then begin
             date = anytim2cal(file)
             doy = utc2doy(str2utc(file))
             cdoy = strmid(string(doy+1000,form='(i4)'),1,3)
             year = strmid(date,7,2)
             ff = findfile(concat_dir('$CDS_CDROM_DATA',$
                           'tm.'+year+'_'+cdoy+'*'),count=count)
             if count eq 0 then ff = findfile(concat_dir('$CDS_TM_DATA',$
                                              'tm.'+year+'_'+cdoy+'*'),count=count)
             if count eq 0 then begin
                print,'No telemetry files found for date ',file
                status = 0
                return
             endif
          endif else begin
             for i=0,n_elements(file)-1 do begin
                 if i eq 0 then begin
                    ff = findfile(concat_dir('$CDS_CDROM_DATA',$
                                  file(0)+'*'),count=count)
                    if count eq 0 then ff = findfile(concat_dir('$CDS_TM_DATA',$
                                                     file(0)+'*'),count=count)
                 endif else begin
                    ff = [ff,findfile(concat_dir('$CDS_CDROM_DATA',$
                                  file(i)+'*'),count=count)]
                    if count eq 0 then ff = [ff,findfile(concat_dir('$CDS_TM_DATA',$
                                             file(i)+'*'),count=count)]
                 endelse
             endfor
             if count eq 0 then begin
                print,'No telemetry files of that name found.',file
                status = 0
                return
             endif
          endelse
      endif else begin
          ff = file
      endelse
   endif

endelse
;
;  try and make sure no funnies got through
;
break_file,ff,disk,dir,f,ext
n = where(strlen(ext) eq 10,count)
if count gt 0 then begin
   ff = ff(n)
endif else begin
   print,'No valid files found.'
   status = 0
   return
endelse



;
;  Define general packet header structure
;
PacketHeader = {PacketHeaderDef,                $
                Msgid:               FIX(0),    $
		MsgLen:              FIX(0),    $
		MsgReqid:            FIX(0),    $    
		ap_id:               BYTARR(2), $
		counter:             FIX(0),    $
		length:              FIX(0),    $
		Time:                BYTARR(6), $
		ID:                  BYTE(0),   $
		StreamPacketCounter: BYTE(0)}

;
; Define Science data header
;
SciHeader = {SciHeaderDef, $
		StudyID:               BYTE(0),   $
		DataCompScheme:        BYTE(0),   $ 
		DataExtWindowID:       FIX(0),    $
		SequenceID:            FIX(0),    $
		RasterLengthM:         BYTE(0),   $
		RasterLengthS:         BYTE(0),   $
		OPSLCoordinate:        FIX(0),    $
		OPSRCoordinate:        FIX(0),    $
                SlitPosition:          FIX(0),    $
		SlitNumber:            BYTE(0),   $
		MirrorPosition:        BYTE(0),   $
		ExposureTime:          FIX(0),    $
		StudyCounter:          FIX(0),    $
		RasterCounter:         FIX(0),    $
                DataCompOption:        FIX(0),    $
                SciProcStatus:         FIX(0)}

;
;  counter
;
count = fix(0)

;
;Exposure header
;
;
Exphdr= {ExpHeaderDef, $
		SolarX:                FIX(0),      $
		ExposureStart:         BYTARR(6),   $
		ExposureNumber:        FIX(0),      $
		OPSLCoordinate:        FIX(0),      $
		OPSRCoordinate:        FIX(0),      $
		SlitPosition:          FIX(0),      $
                SunSensorPitch:        FIX(0),      $
                SunSensorYaw:          FIX(0),      $
                SolarY:                FIX(0),      $
                MirrorPosition:        BYTE(0),     $
                EventStatus:           BYTE(0),     $
                EventLevel:            FIX(0),      $
		StudyCounter:          FIX(0),      $
		RasterCounter:         FIX(0),      $
                Science:               BYTARR(252), $
		ChkSum:                FIX(0)}

;
;  dummy data arrays
;
b260 = bytarr(260)
b286 = bytarr(286)
b284 = bytarr(284)

;
;  arrays to accumulate movements
;
mirr_pos  = replicate({time:0.0d0, data:0},50000)
mpos = 0L

slit_pos  = replicate({time:0.0d0, data:0},50000)
spos = 0L

slit_num  = replicate({time:0.0d0, data:0},10000)
snum = 0L

ops_left  = replicate({time:0.0d0, data:0},10000)
opsl = 0L

ops_right  = replicate({time:0.0d0, data:0},10000)
opsr = 0L


                        
;
;  now LOOP for every tm file to be read
;

nf = n_elements(ff)
for kk = 0L,nf-1 do begin

   tfile = ff(kk)
   if not keyword_set(quiet) then begin
      print,' '
      print,'Processing telemetry file: ',tfile
   endif

;
;  open input file 
;
   openr, unit, tfile, /get_lun


;
;  for incomplete files...
;
   on_ioerror, duff_file


;
;  Packets IDS we are interested in (raster and exposure headers)
;
   packet_ids = ['01'x,'02'x,'11'x,'12'x]
   counter = 0L
;
;  begin reading file
;
   while not eof(unit) do begin
      readu, unit, PacketHeader

;
;  was ID wanted?
;
      hit = where(packet_ids eq PacketHeader.id)
      if hit(0) ge 0 then begin
         case packetheader.id of
            '1'x: begin
                     readu,unit,sciheader
                     readu,unit,b260
                  end
            '2'x: begin
                     readu,unit,sciheader
                     readu,unit,b260
                  end
           '11'x: begin
                     readu,unit,count
                     readu,unit,exphdr
                  end
           '12'x: begin
                     readu,unit,count
                     readu,unit,exphdr
                  end
            else: readu,unit,b286
         endcase
         
;
;  RASTER headers only use OPS and slit number values but also assume that
;                 the mirror and slit positions have come from their central
;                 location ie (128,0)
;
         if packetheader.id eq '1'x or packetheader.id eq '2'x then begin
            tai = obt2tai(packetheader.time)

            opslc = byteswap(sciheader.opslcoordinate) and 'fff'x
            opsrc = byteswap(sciheader.opsrcoordinate) and 'fff'x

            ops_left(opsl).data = opslc
            ops_left(opsl).time  = tai
            opsl = opsl + 1

            ops_right(opsr).data = opsrc
            ops_right(opsr).time  = tai
            opsr = opsr + 1

            slit_num(snum).data = sciheader.slitnumber
            slit_num(snum).time  = tai
            snum = snum + 1

            mirr_pos(mpos).data = 128
            mirr_pos(mpos).time  = tai
            mpos = mpos + 1

            slit_pos(spos).data = 0
            slit_pos(spos).time  = tai
            spos = spos + 1

            if not keyword_set(quiet) then begin
               time = anytim2cal(tai2utc(tai),form=1)
               print,'RASTER   @ pkt ',strpad(trim(counter),5,/after), '  ',$
                      time, $
                     ' Seq ID: ',trim(byteswap(sciheader.sequenceid)), $
                     '  OPS: ',fmt_vect([opslc, opsrc])
            endif
         endif

;
;  EXPOSURE headers use mirror/slit positions
;
         if packetheader.id eq '11'x or packetheader.id eq '12'x then begin
            slitpos_decode,byteswap(exphdr.slitposition),sb,sp
            tai = obt2tai(packetheader.time)

            mirr_pos(mpos).data = exphdr.mirrorposition
            mirr_pos(mpos).time  = tai
            mpos = mpos + 1

            slit_pos(spos).data = sp
            slit_pos(spos).time  = tai
            spos = spos + 1

            if not keyword_set(quiet) then begin
               time = anytim2cal(tai2utc(tai),form=1)
               print,'EXPOSURE @ pkt ',strpad(trim(counter),5,/after),$
                     '  ',time, ' Mirror/slit: ',$
                     fmt_vect([exphdr.mirrorposition,sp])
            endif
         endif
   
      endif else begin
         readu,unit,b286
      endelse

;
;  next packet
;
      counter = counter + 1
   
   endwhile



;
;  trap empty files, after recent experience
;
   if counter eq 0 then begin
      bell
      print,'Warning: Telemetry file appears to be empty.'
      free_lun,unit
      status = 0
      return
   endif


  
;
;  close files
;
   duff_file:
   free_lun, unit

endfor

;
;  trim arrays
;
n = where(ops_left.time gt 0.0,nc)
if nc gt 0 then ops_left = ops_left(n) else ops_left = {time:0.0d0, data:0} 

n = where(ops_right.time gt 0.0,nc)
if nc gt 0 then ops_right = ops_right(n) else ops_right = {time:0.0d0, data:0} 

n = where(slit_pos.time gt 0.0,nc)
if nc gt 0 then slit_pos = slit_pos(n) else slit_pos = {time:0.0d0, data:0} 

n = where(mirr_pos.time gt 0.0,nc)
if nc gt 0 then mirr_pos = mirr_pos(n) else mirr_pos = {time:0.0d0, data:0} 

n = where(slit_num.time gt 0.0,nc)
if nc gt 0 then slit_num = slit_num(n) else slit_num = {time:0.0d0, data:0} 

;
;  and kill identical consecutive values
;
if n_elements(slit_pos.data) gt 1 then begin
   n = where(slit_pos.data - shift(slit_pos.data,1) ne 0, ns)
   if ns gt 0 then slit_pos  = slit_pos(n) else slit_pos = slit_pos(0)
endif

if n_elements(mirr_pos.data) gt 1 then begin
   n = where(mirr_pos.data - shift(mirr_pos.data,1) ne 0, ns)
   if ns gt 0 then mirr_pos  = mirr_pos(n) else mirr_pos = mirr_pos(0)
endif


if n_elements(slit_num.data) gt 1 then begin
   n = where(slit_num.data - shift(slit_num.data,1) ne 0, ns)
   if ns gt 0 then  slit_num = slit_num(n) else slit_num = slit_num(0)
endif


if n_elements(ops_left.data) gt 1 then begin
   n = where(ops_left.data - shift(ops_left.data,1) ne 0, ns)
   if ns gt 0 then  ops_left = ops_left(n) else ops_left = ops_left(0)
endif


if n_elements(ops_right.data) gt 1 then begin
   n = where(ops_right.data - shift(ops_right.data,1) ne 0, ns)
   if ns gt 0 then  ops_right = ops_right(n) else ops_right = ops_right(0)
endif



;
;  does log file already exist?
;
if file_exist(concat_dir('$CDS_ENG_DATA_W','mechanism_log')) then begin
   cds_mech_mpos = 0
   cds_mech_spos = 0
   cds_mech_snum = 0
   cds_mech_opsl = 0
   cds_mech_opsr = 0
   restore,concat_dir('$CDS_ENG_DATA_W','mechanism_log')

;
;  for each of the recorded items, work through the existing data
;
   for i=0,4 do begin

      if i eq 0 then data = cds_mech_mpos
      if i eq 1 then data = cds_mech_spos
      if i eq 2 then data = cds_mech_snum
      if i eq 3 then data = cds_mech_opsl
      if i eq 4 then data = cds_mech_opsr

      if i eq 0 then data_new = mirr_pos
      if i eq 1 then data_new = slit_pos
      if i eq 2 then data_new = slit_num
      if i eq 3 then data_new = ops_left
      if i eq 4 then data_new = ops_right

      nsamp = n_elements(data.time)
      nsamp_new = n_elements(data_new.time)


;
;  put new and old together
;
      d1 = concat_struct(data, data_new)

;
;  kill duplicates caused by reruns
;
      nnd = sort(d1.time)
      d1 = d1(nnd)
      nnd = uniq(d1.time)
      if nnd(0) ge 0 then d1 = d1(nnd)

;
;  switch it to variable that will be saved
;
      if i eq 0 then cds_mech_mpos = d1
      if i eq 1 then cds_mech_spos = d1
      if i eq 2 then cds_mech_snum = d1
      if i eq 3 then cds_mech_opsl = d1
      if i eq 4 then cds_mech_opsr = d1

   endfor

endif else begin
      if not keyword_set(quiet) then print,'Starting new mechanism log'
      for i=0,4 do begin
         if i eq 0 then cds_mech_mpos = mirr_pos
         if i eq 1 then cds_mech_spos = slit_pos
         if i eq 2 then cds_mech_snum = slit_num
         if i eq 3 then cds_mech_opsl = ops_left
         if i eq 4 then cds_mech_opsr = ops_right
      endfor
endelse

if n_elements(cds_mech_mpos.data) gt 1 or $ 
    n_elements(cds_mech_spos.data) gt 1 or $ 
     n_elements(cds_mech_snum.data) gt 1 or $ 
      n_elements(cds_mech_opsl.data) gt 1 or $ 
       n_elements(cds_mech_opsr.data) gt 1 then begin
   save,/xdr,file=concat_dir('$CDS_ENG_DATA_W','mechanism_log'),$
                              cds_mech_mpos, $
                              cds_mech_spos, $
                              cds_mech_snum, $
                              cds_mech_opsl, $
                              cds_mech_opsr
endif 
status = 1
     

end
   
