;+
; Project     : SOHO - CDS     
;                   
; Name        : EMON_QCM_LOG
;               
; Purpose     : Accumulates CDS QCM logs.
;               
; Explanation : Reads all the QCM-related telemetry values within a telemetry 
;               file.  These values are added to the QCM log, which is a save
;               file 'qcm_log' kept in $CDS_ENG_DATA.
;               
; Use         : IDL> emon_qcm_log, input
;    
; Inputs      : input - can be one of:  DOY number (all files read)
;                                       Specific date - in CDS string format
;                                                  (all files read)
;                                       CDS UTC structure
;
;                                       Specific telemetry file name (or part
;                                       thereof, but MUST begin with tm)
;                                       eg 'tm.95_123'  will pick all files 
;                                       from DOY 123
;                                       
;               
; Opt. Inputs : None
;               
; Outputs     : Save file is updated.
;                    
; Opt. Outputs: None
;               
; Keywords    : None
;
; Calls       : None
;
; Common      : None
;               
; Restrictions: Requires CDS_ENG_DATA_W to be defined.
;               
; Side effects: None
;               
; Category    : Telemetry
;               
; Prev. Hist. : None
;
; Written     : C D Pike, RAL, 20-Nov-95
;               
; Modified    : Increase temporary storage array size.  CDP, 16-Jan-96
;               Changed QCM seqid to 05 from 10.  CDP, 26-Mar-96
;               Made ID checking more general to cope with running
;               as engineering study.             CDP, 27-Mar-96
;
;
; Version     : Version 4, 27-Mar-96
;-            

pro emon_qcm_log, file


;
;  does user have privilege to write data?
;
direct = getenv('CDS_ENG_DATA_W')
if direct eq '' then begin
   print,'Environment variable CDS_ENG_DATA_W is not defined.'
   return
endif


;
;  input can be 
;               doy    (specific day of year number, all files)
;               CDS  UTC structure (use all files from that day)
;               CDS  date string (use all files from it)
;               Specific file name (use only it)
;
if (n_params() eq 0)  then begin
   print,'USE: emon_qcm_log, doy or CDS date or telemetry file name.'
   return
endif

;
;  DOY specified
;

if datatype(file,1) eq 'Integer' or datatype(file,1) eq 'Long' then begin
   cdoy = strmid(string(file+1000,form='(i4)'),1,3)
   date = anytim2cal(doy2utc(file))
   year = strmid(date,7,2)
   ff = findfile(concat_dir('$CDS_TM_DATA',$
                 'tm.'+year+'_'+cdoy+'*'),count=count)
   if count eq 0 then begin
      print,'No telemetry files found for DOY ',file
      return
   endif
endif

;
;  CDS UTC structure
;
if datatype(file,1) eq 'Structure' then begin
   date = anytim2cal(file)
   doy = utc2doy(file)
   cdoy = strmid(string(file+1000,form='(i4)'),1,3)
   year = strmid(date,7,2)
   ff = findfile(concat_dir('$CDS_TM_DATA',$
                 'tm.'+year+'_'+cdoy+'*'),count=count)
   if count eq 0 then begin
      print,'No telemetry files found for UTC ',file
      return
   endif
endif
 
;
;  CDS date/time string or telemetry file name
;  
if datatype(file,1) eq 'String' then begin
   if strmid(file,0,2) ne 'tm' then begin
      date = anytim2cal(file)
      doy = utc2doy(str2utc(file))
      cdoy = strmid(string(doy+1000,form='(i4)'),1,3)
      year = strmid(date,7,2)
      ff = findfile(concat_dir('$CDS_TM_DATA',$
                    'tm.'+year+'_'+cdoy+'*'),count=count)
      if count eq 0 then begin
         print,'No telemetry files found for date ',file
         return
      endif
   endif else begin
      ff = findfile(concat_dir('$CDS_TM_DATA',$
                    file+'*'),count=count)
      if count eq 0 then begin
         print,'No telemetry files of that name found.',file
         return
      endif
   endelse
endif

;
;  try and make sure no funnies got through
;
break_file,ff,disk,dir,f,ext
n = where(strlen(ext) eq 10,count)
if count gt 0 then begin
   ff = ff(n) 
endif else begin
   print,'No valid files found.'
   return
endelse 

;
;  OK to continue?
;
if ff(0) eq '' then begin
   print,'No valid files.'
   return
endif
nf = n_elements(ff)

   
;
;  pick up the names
;
get_mask_etc, mask

bnames = ['BKMQCMC','BTMQCMT1','BTMQCMT2','BTMQCMG1','BTMQCMG2']
pnum = intarr(5)
for i=0,4 do pnum(i) = where(mask(0,*) eq bnames(i))
idnum = intarr(2)
anames = ['AK5SID','AK5SSTAT']
for i=0,1 do idnum(i) = where(mask(0,*) eq anames(i))

;
;  now LOOP for every tm file to be read
;

for kk = 0,nf-1 do begin

   tfile = ff(kk)
   print,'Processing telemetry file: ',tfile

;
;  reset the sequence ID flag
;
   seqid = 0
   finished = 1

;
;  Assume max of 2000 QCM readings per file
;
   new = 2000
   d1 = {time:double(0.0),$
         freq:long(0),$
         t1:0.0,$
         t2:0.0,$
         g1:'',$
         g2:''}

   qcm_new = replicate(d1,new)

;
;  read temperature data from telemetry file
;
   openr,lun,tfile,/get_lun,error=err
   if err ne 0 then begin
      print,'Error opening file ',tfile
      return
   endif

   packet   = bytarr(306)
   ipack = -1
   last_id = -1
   new_id = 0
   ss = ''

   on_ioerror, duff_pkt

   while (not eof(lun)) do begin
      readu,lun,packet
      case 1 of

;
;  Eng A packet, collect the sequence ID and the status
;
        (packet(18) eq '0A'x) : begin
                    for i=0,1 do begin
                       temp = (strmid(mask(5,idnum(i)),2,4))(0) 
                       hex2dec,temp,pmask,/quiet
                       offset = fix(mask(3,idnum(i)))
                       type = mask(2,idnum(i))
                       case type of
                         'B': begin
                                if pmask gt 0 then begin
                                  data = fix(packet(offset) and pmask)
                                endif else begin
                                  data = fix(packet(offset))
                                endelse
                              end
                         'W': begin
                                temp = byteswap((fix(packet,offset,1))(0))
                                if pmask gt 0 then begin
                                  data = unsign(temp) and long(pmask)
                                endif else begin
                                  data = unsign(temp)
                                endelse
                              end
                       endcase
                       if i eq 0 then begin
                          seqid = data 
                          if seqid eq last_id then begin
                             new_id = 0
                          endif else begin
                             last_id = seqid
                             new_id = 1
                             ss = get_cdhs_series(seqid)
                             if ss.id ne -1 then begin
                                ss = fmt_vect([ss.id,ss.var],delim='/')
                             endif else begin
                                ss = ''
                             endelse
                          endelse
                       endif else begin
                          finished = (data eq 162)
                       endelse
                    endfor
                                end
;
;  Eng B packet, collect the QCM parameters
;  but only if the sequence ID is correct (either 5 for running manually or
;  id/var= 0/2 for planned engineering) and it hasn't finished
;
        (packet(18) eq '0b'x) : begin
                    if ((seqid eq '05'x) or (ss eq '(0/2)')) $
                                            and (not finished) then begin
                       ipack = ipack + 1
                       qcm_new(ipack).time = obt2tai(packet(12:17))
                              
                       for i=0,4 do begin
                          temp = (strmid(mask(5,pnum(i)),2,4))(0) 
                          hex2dec,temp,pmask,/quiet
                          offset = fix(mask(3,pnum(i)))
                          type = mask(2,pnum(i))
                          case type of
                            'B': begin
                                   if pmask gt 0 then begin
                                     data = fix(packet(offset) and pmask)
                                   endif else begin
                                     data = fix(packet(offset))
                                   endelse
                                 end
                            'W': begin
                                   temp = byteswap((fix(packet,offset,1))(0))
                                   if pmask gt 0 then begin
                                     data = unsign(temp) and long(pmask)
                                   endif else begin
                                     data = unsign(temp)
                                   endelse
                                 end
                          endcase
                          if i lt 3 then begin
                             qcm_new(ipack).(i+1) = data
                          endif else begin
                             hilow = ['HIGH','LOW']
                             qcm_new(ipack).(i+1) = hilow(data gt 0)
                          endelse
                       endfor
                    endif
                                end
   
                          else:
      endcase
   endwhile

   duff_pkt:
   on_ioerror, null

;
;  was anything found in this file?
;
   if ipack eq -1 then begin
      print,'Nothing in this file'
   endif else begin   

;
;  trim new array
;
      qcm_new = qcm_new(0:ipack)

;
;  kill duplicates as most will be just same value repeated until next
;  new one comes along
;
      qcm_new = qcm_new(rem_dup(qcm_new.freq))

;
;  and re-sort in time
;
      if n_elements(qcm_new) gt 1 then qcm_new = qcm_new(sort(qcm_new.time))


      print,'Adding',n_elements(qcm_new),' new values.'

;
;  calibrate the temperatures
;
      for i=2,3 do begin
         temp = qcm_new.(i)
         temp = tm_calib(bnames(i-1),temp)
         if n_elements(temp) eq 1 then temp = temp(0)
         qcm_new.(i) = temp
      endfor

;
;  does data file already exist?
;
      if file_exist(concat_dir('$CDS_ENG_DATA_W','qcm_log')) then begin
         cds_qcm_log = 0
         restore,concat_dir('$CDS_ENG_DATA_W','qcm_log')
         nsamp = n_elements(cds_qcm_log.time)

;
;  check new times are not within current range
;
         mintime = min(cds_qcm_log.time)
         maxtime = max(cds_qcm_log.time)
         minntime = min(qcm_new.time)
         maxntime = max(qcm_new.time)
         if ((minntime ge mintime) and (minntime le maxtime)) or $
            ((maxntime ge mintime) and (maxntime le maxtime)) then begin
            bell
            print,' '
            print,'*** Warning:'
            print,' '
            print,'New data has a time within the range of existing data.'
            print,'Existing times are: ',anytim2cal(mintime),' to '
            print,'                    ',anytim2cal(maxtime)
            print,' '
            print,'New times are:      ',anytim2cal(minntime),' to '
            print,'                    ',anytim2cal(maxntime)
            print,' '
            yes_no,'OK to continue? ',ok,'Yes'
            if not ok then return
         endif   


;
;  add new to old
;
         cds_qcm_log = concat_struct(cds_qcm_log,qcm_new)


      endif else begin
         bell
         print,'Starting new log'
         cds_qcm_log = qcm_new
      endelse              
   
      save,/xdr,cds_qcm_log,file=concat_dir('$CDS_ENG_DATA_W','qcm_log')
   endelse
   free_lun, lun
endfor

end
