;+
; Project     : SOHO - CDS
;
; Name        : SHOW_CDS_MECH
;
; Purpose     : Plot CDS mechanism statistics.
;
; Explanation : Reads mechanism log and allows any parameter to be plotted.
;
; Use         : IDL> show_cds_mech [,/slit_pos, ..., timerange=...]
;
; Inputs      : None
;
; Opt. Inputs : None
;
; Outputs     : None
;
; Opt. Outputs: See keywords FORWARD/BACWARD/POS
;
; Keywords    : TIMERANGE - A 2-element string array giving the start and
;                           stop times for plotting.  Elements are CDS string
;                           format.
;               SLIT_POS - plot slit position data
;               SLIT_NUM - plot slit number data
;               MIRROR   - plot mirror position data
;               OPSL     - plot OPS left data
;               OPSR     - plot OPS right data
;               FB       - show on the cumulative plot the forward and backward
;                          movements separately.
;               HARDCOPY - send copy of plots to printer.
;               FORWARD  - returns forward movement accumulated log
;               BACKWARD - returns backward accumulated log
;               POS      - returns location value for forward/backward log
;                          (note last three only useful if only one mechanism
;                           at a time is plotted)
;
;
; Calls       : CDS_UTPLOT
;
; Common      : None
;
; Restrictions: Requires $CDS_ENG_DATA
;
; Side effects:
;
; Category    : Mechanisms, telemetry
;
; Prev. Hist. : None
;
; Written     : C D Pike, RAL, 24-Nov-95
;
; Modified    : Trap insufficient data.  CDP, 29-Jan-96
;               Fixed bug and added return variables.  CDP, 2-Jul-96
;               Fixed typo.  CDP, 3-Jul-96
;               Change UTPLOT name.  CDP, 2-Oct-96
;
; Version     : Version 5, 2-Oct-96
;-

pro show_cds_mech, timerange=timerange, slit_pos=slit_pos, slit_num=slit_num,$
                   mirror=mirror, opsl=opsl, opsr=opsr,fb=fb, $
                   hardcopy=hardcopy, forward=accum_f,back=accum_b,pos=x

                                           ;
;  has the correct evar been defined
;
if getenv('CDS_ENG_DATA') eq '' then begin
   bell
   print,'$CDS_ENG_DATA is not defined.'
   return
endif                                      

;
;  read the mechanism log data
;                                 
cds_mech_mpos = 0
cds_mech_spos = 0
cds_mech_snum = 0
cds_mech_opsl = 0
cds_mech_opsr = 0

if file_exist(concat_dir('$CDS_ENG_DATA','mechanism_log')) then begin
   restore,concat_dir('$CDS_ENG_DATA','mechanism_log')
endif else begin
   bell
   print,concat_dir('$CDS_ENG_DATA','mechanism_log')
   print,'Mechanism log file does not exist.'
   return
endelse

;
;  which parameters were required?
;
wanted = intarr(5)
if keyword_set(mirror)    then wanted(0) = 1
if keyword_set(slit_pos)  then wanted(1) = 1
if keyword_set(slit_num)  then wanted(2) = 1
if keyword_set(opsl)      then wanted(3) = 1
if keyword_set(opsr)      then wanted(4) = 1

;
;  default for all
;
if total(wanted) eq 0 then wanted(*)=1

;
;  loop over all possible parameters
;
!p.multi= [0,1,2]

done = 0

for i=0,4 do begin

   if wanted(i) then begin

      offset = 0

      if i eq 0 then begin
         time = tai2utc(cds_mech_mpos.time)
         data = cds_mech_mpos.data
         y1_tit = 'Mirror location'
         y2_tit = 'Cumulative total'
         x1_tit = 'Date/time (UT)'
         x2_tit = 'Mirror location'
         tit    = 'CDS scan mirror movement log'
      endif
  
      if i eq 1 then begin
         time = tai2utc(cds_mech_spos.time)
         data = cds_mech_spos.data
         offset = 256
         y1_tit = 'Slit location'
         y2_tit = 'Cumulative total'
         x1_tit = 'Date/time (UT)'
         x2_tit = 'Slit location'
         tit    = 'CDS slit position log'
      endif
  
      if i eq 2 then begin
         time = tai2utc(cds_mech_snum.time)
         data = cds_mech_snum.data
         y1_tit = 'Slit number'
         y2_tit = 'Cumulative total'
         x1_tit = 'Date/time (UT)'
         x2_tit = 'Slit number'
         tit    = 'CDS slit number usage'
      endif
  
      if i eq 3 then begin
         time = tai2utc(cds_mech_opsl.time)
         data = cds_mech_opsl.data
         y1_tit = 'OPS Left location'
         y2_tit = 'Cumulative total'
         x1_tit = 'Date/time (UT)'
         x2_tit = 'OPS Left location'
         tit    = 'CDS OPS Left movement log'
      endif
  
      if i eq 4 then begin
         time = tai2utc(cds_mech_opsr.time)
         data = cds_mech_opsr.data
         y1_tit = 'OPS Right location'
         y2_tit = 'Cumulative total'
         x1_tit = 'Date/time (UT)'
         x2_tit = 'OPS Right location'
         tit    = 'CDS OPS Right movement log'
      endif
  
      data = data + offset
;
;  set up default time range
;

      if not keyword_set(timerange) then begin
         tplot_min = utc2tai(time(0))
         tplot_max = utc2tai(time(n_elements(time)-1))
      endif else begin
         if datatype(timerange(0),1) eq 'String' and $
                n_elements(timerange) eq 2 then begin
            tplot_min = utc2tai(str2utc(timerange(0)))
            tplot_max = utc2tai(str2utc(timerange(1)))
         endif else begin
            bell
            print,'Invalid time range specification'
            print,"Timerange: ['start time','end time']"
            return
         endelse
      endelse

;
;  trim data if needed
;
      if keyword_set(timerange) then begin
         tai = utc2tai(time)
         nit = where(tai ge tplot_min and tai le tplot_max,tc)
         if tc gt 0 then begin
            data = data(nit)
            time = time(nit)
         endif
      endif
;
;  check time axis label
;
      nt = n_elements(time.mjd)
      if (time(nt-1).mjd - time(0).mjd) le 2 then begin
         x1_tit = 'Start date: '+strmid(anytim2cal(time(0)),0,9)+' (UT)'
      endif

;
;  pause for reflection
;
      if done gt 0 then begin
         ok = 0
         while not ok do yes_no,'Ready for next plot?',ok,'Yes'
      endif
    
;
;  prepare plotting area
;
      if i ne 1 then begin
         cds_utplot,[time(0),time(n_elements(time)-1)],ytit=y1_tit,xtit=x1_tit,$
                ext_range([min(data),max(data)],50),$
                psym=3,/ynoz,chars=1.5,tit=tit
      endif else begin
         cds_utplot,[time(0),time(n_elements(time)-1)],ytit=y1_tit,$
                xtit=x1_tit,tit=tit,$
                ext_range([min(data)-128,max(data)-128],50),$
                psym=3,/ynoz,chars=1.5
      endelse
      accum_f = intarr(max(data)+1)
      accum_b = intarr(max(data)+1)
   
      if n_elements(data) gt 2 then begin
      
         for j=0L,n_elements(data)-2 do begin
            cds_outplot,[time(j),time(j)],[data(j)-offset,data(j+1)-offset]
            if data(j) ne data(j+1) then begin
               if data(j) lt data(j+1) then begin
                  accum_f(data(j):data(j+1)-1) = $
                                  accum_f(data(j):data(j+1)-1) + 1
               endif else begin
                  accum_b(data(j+1)+1:data(j)) = $
                                  accum_b(data(j+1)+1:data(j)) + 1
               endelse
            endif
         endfor
   
         accum = accum_f + accum_b
         accum = [accum,0]
         x = indgen(n_elements(accum))-offset
         plot,x,accum,psym=10,chars=1.5,ytit=y2_tit,xtit=x2_tit,$
                xr=ext_range([min(data-offset),max(data-offset)],10),$
                yr=ext_range([min(accum),max(accum)],10,/pos)

;
;  display forward and backward separately?
;
         if keyword_set(fb) then begin
            accum_b = [accum_b,0]
            accum_f = [accum_f,0]
            x = indgen(n_elements(accum_b))-offset
            oplot,x,accum_b,psym=10,line=1
            x1 = (!x.crange(1)-!x.crange(0))*0.05 + !x.crange(0)
            x2 = x1 + (!x.crange(1)-!x.crange(0))*0.05
            y1 = !y.crange(1) - (!y.crange(1)-!y.crange(0))*0.1 
            oplot,[x1,x2],[y1,y1],line=1
            xyouts,x2,y1,'Backwards'
   
            x = indgen(n_elements(accum_f))-offset
            oplot,x,accum_f,psym=10,line=2
            oplot,[x1,x2],[y1*0.92,y1*0.92],line=2
            xyouts,x2,y1*0.92,'Forwards'
         endif

;
;  repeat for hardcopy?
;
         if keyword_set(hardcopy) then begin
            ps
            if i ne 1 then begin
               cds_utplot,[time(0),time(n_elements(time)-1)],ytit=y1_tit,xtit=x1_tit,$
                      ext_range([min(data),max(data)],50),$
                      psym=3,/ynoz,chars=1.5,tit=tit
            endif else begin
               cds_utplot,[time(0),time(n_elements(time)-1)],ytit=y1_tit,$
                      xtit=x1_tit,tit=tit,$
                      ext_range([min(data)-128,max(data)-128],50),$
                      psym=3,/ynoz,chars=1.5
            endelse
            accum_f = intarr(max(data)+1)
            accum_b = intarr(max(data)+1)
         
            for j=0L,n_elements(data)-2 do begin
               cds_outplot,[time(j),time(j)],[data(j)-offset,data(j+1)-offset]
               if data(j) ne data(j+1) then begin
                  if data(j) lt data(j+1) then begin
                     accum_f(data(j):data(j+1)-1) = $
                                     accum_f(data(j):data(j+1)-1) + 1
                  endif else begin
                     accum_b(data(j+1)+1:data(j)) = $
                                     accum_b(data(j+1)+1:data(j)) + 1
                  endelse
               endif
            endfor
   
            accum = accum_f + accum_b
            accum = [accum,0]
            x = indgen(n_elements(accum))-offset
            plot,x,accum,psym=10,chars=1.5,ytit=y2_tit,xtit=x2_tit,$
                   xr=ext_range([min(data-offset),max(data-offset)],10),$
                   yr=ext_range([min(accum),max(accum)],10,/pos)
   
   
            if keyword_set(fb) then begin
               accum_b = [accum_b,0]
               accum_f = [accum_f,0]
               x = indgen(n_elements(accum_b))-offset
               oplot,x,accum_b,psym=10,line=1
               x1 = (!x.crange(1)-!x.crange(0))*0.05 + !x.crange(0)
               x2 = x1 + (!x.crange(1)-!x.crange(0))*0.05
               y1 = !y.crange(1) - (!y.crange(1)-!y.crange(0))*0.1 
               oplot,[x1,x2],[y1,y1],line=1
               xyouts,x2,y1,'Backwards'
      
               x = indgen(n_elements(accum_f))-offset
               oplot,x,accum_f,psym=10,line=2
               oplot,[x1,x2],[y1*0.92,y1*0.92],line=2
               xyouts,x2,y1*0.92,'Forwards'
            endif
   
            psplot
         endif
      endif else begin
         print,'Insufficient data to plot'
      endelse
      done = done + 1
   endif
endfor

!p.multi=0

end

