;+
; Project     : SOHO - CDS     
;                   
; Name        : SHOW_TM_PKT
;               
; Purpose     : To show details of packets in a CDS telemetry file.
;               
; Explanation : Reads a packet from the given tm file and displays details
;               of the contents.
;               
; Use         : IDL> show_tm_pkt, filename, packets [, various keywords]
;    
; Inputs      : filename - the telemetry file name
;
; Opt. Inputs : packets  - the packet numbers to be dumped. If a 2-element
;                          array then the elements are assumed to be
;                          the start and stop of a range of packets.  If this
;                          is not supplied then all packets are dumped.
;               
; Outputs     : Optionally outputs information to screen and file.
;               
; Opt. Outputs: None
;               
; Keywords    : HARDCOPY  - print hardcopy of output
;               QUIET     - no  display to screen
;               KEEP      - keep output disk file
;               MORE      - if present the screen output is 'paged' as in Unix
;                           'more' command
;               DUMP      - will dump a listing of the packet values
;               DECIMAL   - dump format is decimal (default IS hex)
;               BYTEARRAY - dump as byte array (default is INT16)
;               TS_SAVE   - array to return times of science packets.
;               THK_SAVE  - array to return times of HK packets
;               HK_PKTS   - numbers of HK packets corresponding to entries
;                           in THK_SAVE
;               S_PKTS    - numbers of science packets corresponding to 
;                           entries in TC_SAVE
;
;
; Calls       : None
;
; Common      : None
;               
; Restrictions: None
;               
; Side effects: None
;               
; Category    : Telemetry
;               
; Prev. Hist. : None
;
; Written     : C D Pike, RAL, 18-Jun-95
;             
; Modified    : Trap packets requested beyond end of file.  CDP, 14/9/95
;               Put out time to msec accuracy and make default packets
;               be all packets. Also include keywords to return specific
;               times of packets. CDP, 2-Oct-95
;               Generalize hardcopy file name.  CDP, 3-Oct-95
;
; Version     : Version 4, 03-Oct-95
;-            

pro show_tm_pkt, file, packets, hardcopy=hardcopy, quiet=quiet, keep=keep, $
                                more=more,dump=dump, decimal=decimal, $
                                bytearray=bytearray, thk_save=thk_save,$
                                ts_save=ts_save, hk_pkts=hk_pkts,$
                                s_pkts=s_pkts

;
;  insist on correct input
;
if n_params() lt 1 then begin
   print,'Use:  show_tm_pkt, filename [, packets]'
   return
endif


;
;  Define general packet header structure
;
PacketHeader = {PacketHeaderDef,                $
                Msgid:               FIX(0),    $
		MsgLen:              FIX(0),    $
		MsgReqid:            FIX(0),    $    
		ap_id:               BYTARR(2), $
		counter:             FIX(0),    $
		length:              FIX(0),    $
		Time:                BYTARR(6), $
		ID:                  BYTE(0),   $
		StreamPacketCounter: BYTE(0)}

;
;  science or HK?
;
type = strarr(225)
type(*) = ''
type(16) = 'HK'
type(32) = 'HK'
type(64) = 'HK'
type(80) = 'HK'
type(96) = 'HK'
type(128) = 'HK'
type(144) = 'HK'
type(224) = 'HK'

;
;  human readable interpretation of ID
;
name = strarr(225)
name(1) = 'GIS-raster'
name(2) = 'VDS-raster'
name(3) = 'Abort'
name(17) = 'GIS-exp'
name(18) = 'VDS-exp'
name(33) = 'GIS-data'
name(34) = 'VDS-data'
name(48) = 'Special'
name(16) = 'HK Snooze mode'
name(32) = 'HK Standby mode'
name(64) = 'HK Ready-GIS mode'
name(80) = 'HK Ready-VDS mode'
name(96) = 'HK Ready-Both mode'
name(128) = 'HK Execute-GIS mode'
name(144) = 'HK Execute-VDS mode'
name(224) = 'HK Emergency Standby mode'

;
;  and of special packet formats
;
sp_name = strarr(20)
sp_name(1)  = '(MCU)'
sp_name(2)  = '(Deferred)'
sp_name(3)  = '(Data line list)'
sp_name(4)  = '(Raster table)'
sp_name(5)  = '(Sequence table)'
sp_name(6)  = '(CDS h/k table)'
sp_name(7)  = '(Error)'
sp_name(8)  = '(Health)'
sp_name(9)  = '(IMIF)'
sp_name(10)  = '(Hardware list)'
sp_name(11)  = '(Mode table)'
sp_name(12)  = '(Engineering table)'
sp_name(13)  = '(Sequence controller table)'
sp_name(14) = '(IEF)'


pname = strarr(225)
pname(*) = '          ?           '
pname(10) = 'Engineering A'
pname(11) = 'Engineering B'
pname(1)  = 'GIS Raster header'
pname(2)  = 'VDS Raster header'
pname(3)  = 'Raster abort'
pname(17)  = 'GIS Exposure header'
pname(18)  = 'VDS Exposure header'
pname(33)  = 'GIS Science data'
pname(34)  = 'VDS Science data'
pname(4)  = 'Primary   Proc. Mem. Dump'
pname(5)  = 'Secondary Proc. Mem. Dump'
pname(6)  = 'VDS Memory Dump'
pname(7)  = 'GIS raw data dump'
pname(48)  = 'Special Packet'
pname(16) = 'HK Snooze mode'
pname(32) = 'HK Standby mode'
pname(64) = 'HK Ready-GIS mode'
pname(80) = 'HK Ready-VDS mode'
pname(96) = 'HK Ready-Both mode'
pname(128) = 'HK Execute-GIS mode'
pname(144) = 'HK Execute-VDS mode'
pname(224) = 'HK Emergency Standby mode'

;
;  allow user to have own version of telemetry files
;
tfile = ''
if  file_exist(file) then begin
   tfile = file(0)
endif else begin
   if file_exist(concat_dir('$CDS_TM_DATA',file(0))) then begin
      tfile = concat_dir('$CDS_TM_DATA',file(0))
   endif
endelse

;
;  telemetry file doesn't exist in user or std directory
;
if tfile eq '' then begin
   print,'show_tm_pkt:  Input file does not exist.'
   return
endif

;
;  try to open telemetry file
;
openr,tmlun,tfile,/get_lun,error=err
if err ne 0 then begin
   print,'Cannot open telemetry file ',tfile
   return
endif else begin
   f = fstat(tmlun)
   maxpax = f.size/306L
endelse

;
;  were packets specified?
;
if n_elements(packets) eq 0 then packets = [0,maxpax-1]

;
;  dummy data arrays
;
b286 = bytarr(286)
b284 = bytarr(284)

;
;  create filename (in home directory) for hardcopy
;
vms=(!version.os eq 'VMS')
hfile = 'show_tm_pkt_'
if vms then home='sys$login' else home=getenv('HOME')
hfile=concat_dir(home,hfile)

;
;  any other similar files already exist?
;
fc = findfile(hfile+'*',count=count)
if count gt 0 then begin
   hfile = hfile + strmid(string(count+1000,format='(i4)'),1,3)
endif else begin
   hfile = hfile + '000'
endelse

;
;  attempt to open output file
;
openw,lun,hfile,/get_lun,error=err
if err ne 0 then begin
   print,'Error opening output file ',hfile
   free_lun,tmlun
   return
endif


;
;  to terminal?
;
if keyword_set(quiet) then to_screen = 0 else to_screen = 1
if to_screen then begin
   if keyword_set(more) then begin
      openw, screen, filepath(/terminal), /get_lun ,/more
   endif else begin
      openw, screen, filepath(/terminal), /get_lun
   endelse
endif

;
;  input packets can be an array so sort out
;
if n_elements(packets) eq 2 then begin
   packets = packets(sort(packets))
   packs = indgen(packets(1)-packets(0)+1)+packets(0)
endif else begin
   packs = packets
endelse

;
;  save time for output if needed
;
ts_save  = dblarr(n_elements(packs))
thk_save = dblarr(n_elements(packs))

;
;  ... and loop around
;
for i=0,n_elements(packs)-1 do begin

   pack = packs(i)
   if pack ge maxpax then begin
      bell
      print,'Requested packet is beyond end of file.'
      print,'Max packet number allowed = ',maxpax-1
      return
   endif
   if pack gt 0 then point_lun,tmlun, pack*306L
   readu,tmlun, packetheader
   if packetheader.id gt 224 or packetheader.id lt 0 then packetheader.id = 0
   packc = strpad(strcompress('#'+string(pack,form='(i6)'),/rem),6,/after)
   tt = obt2tai(packetheader.time)
   ptime = '   @ '+anytim2cal(tai2utc(tt),form=9)
   title = strpad('Packet '+packc+' is '+pname(packetheader.id),43,/after)+ptime

;
;  save time in appropriate array
;
   if type(packetheader.id) eq 'HK' then thk_save(i)=tt else ts_save(i)=tt
  
;
;  give input file name for reference
;
   if i eq 0 then begin
      if to_screen then printf,screen,' '
      if to_screen then printf,screen,'Telemetry file: ',tfile
      printf,lun,' '
      printf,lun,'Telemetry file: ',tfile
   endif
;
;  separate packets if dump requested
;
   if keyword_set(dump) then begin
      text = '-'
      for dash=0,78 do text = text + '-'
      if to_screen then printf,screen,' '
      if to_screen then printf,screen,text
      print,' '
      printf,lun,' '
      printf,lun,text
   endif
;
;  print title if not Special packet
;
   if packetheader.id ne '30'x then begin
      if to_screen then print,screen,title
      printf,lun,title
   endif
   case packetheader.id of
     '30'x: begin
               temp = read_sp_id(tmlun)
               sp_id = temp(1) 
               readu,tmlun,b284 
            end
      else: readu,tmlun,b286
   endcase
         
;
;  if special packet we have waited to get the 'Special ID'
;
   if packetheader.id eq '30'x then begin
      if (sp_id lt 50) and (sp_id ge 0) then begin
          title = title + '  '+sp_name(sp_id)
       endif else begin
          title = title + ' (Bad SPECIAL packet ID)'
       endelse
       if to_screen then printf,screen,title
       printf,lun,title
   endif

;
;  dump requested
;
   barray = bytarr(306)
   if keyword_set(dump) then begin
      if keyword_set(bytearray) then begin
         fdec = '(20(16i4,/))'
         fhex = '(20(16(1x,z2.2),/))'
         if keyword_set(decimal) then begin
            text = '---0---1---2---3---4---5---6---7---8---9--10'+$
                   '--11--12--13--14--15'
         endif else begin
            text = ' 00-01-02-03-04-05-06-07-08-09-0a'+$
                  '-0b-0c-0d-0e-0f'
         endelse
      endif else begin
         fdec = '(20(8(1x,i6),/))'
         fhex = '(20(16(1x,z4.4),/))'
         if keyword_set(decimal) then begin
            text = '    0  --  1  --  2  --  3  --  4  --  5  --  6  --  7'
         endif else begin
            text = '  00 - 01 - 02 - 03 - 04 - 05 - 06 - 07 - 08 - 09 - 0a'+$
                  ' - 0b - 0c - 0d - 0e - 0f'
         endelse
      endelse

;
;  reread the packet
;
      if pack gt 0 then point_lun,tmlun, pack*306L
      readu,tmlun,barray

      if keyword_set(bytearray) then begin
         p = barray
      endif else begin
         p  = unsign(byteswap(fix(barray,0,153)))
      endelse

      if to_screen then begin
         printf,screen,' '
         printf,screen,text
         if keyword_set(decimal) then begin
            printf,screen,p,form=fdec
         endif else begin
            printf,screen,p,form=fhex
         endelse
      endif
      printf,lun,' '
      printf,lun,text
      if keyword_set(decimal) then begin
         printf,lun,p,form=fdec
      endif else begin
         printf,lun,p,form=fhex
      endelse
;
;  slight delay to help screen dump output
;
      wait,0.5
   endif
endfor

;
;  tidy the time arrays
;
n = where(thk_save gt 0.0)
if n(0) ge 0 then begin
   thk_save = thk_save(n)
   hk_pkts = n
endif else begin
   thk_save = -1
   hk_pkts = -1
endelse

n = where(ts_save gt 0.0)
if n(0) ge 0 then begin
   ts_save = ts_save(n)
   s_pkts = n
endif else begin
   s_save = -1
   s_pkts = -1
endelse


;
;  release units
;
free_lun,tmlun
free_lun,lun
if to_screen then free_lun,screen


;
;  print hard copy
;
if keyword_set(hardcopy) then begin
   if vms then com = 'print '+hfile else com = 'lpr '+hfile
   spawn,com
   bell
   print,'Hard copy sent to printer'
endif
if not keyword_set(keep) then begin
   status = delete_file(hfile,/noconfirm)
endif else begin
   if not keyword_set(quiet) then bell
   print,'Dump saved in ',hfile
endelse

end
   
