;+
; Project     : SOHO - CDS     
;                   
; Name        : TLM_SUMMARY
;               
; Purpose     : To summarize a telemetry file.
;               
; Explanation : Reads a telemetry files, packet by packet and records 
;               the details of every packet whose ID matches that requested.
;               
; Use         : IDL> tlm_summary, filename, id_spec $
;                                 [,found=found,packs=packs,packe=packe,/quiet]
;    
; Inputs      : filename - the telemetry file name
;               
; Opt. Inputs : id_spec  - integer array giving ID wanted eg [1,2] to pick
;                          GIS/NIS raster header packets 
;                          (watch decimal/hex switch! - the input values are
;                          DECIMAL).  Defaults to [1,2] ie raster headers.
;                          See also keywords for alternative method of
;                          specifying these.
;               
; Outputs     : Outputs information to screen and file 'filename'_summ in 
;               current directory. 
;               
; Opt. Outputs: found - returns packets where specified IDs found.  Returned
;                       as an array (N,M) where N is the number of IDs
;                       requested and M is the total number of finds for
;                       any ID. (Values not used for a particular ID are set
;                       to -1).
;               
; Keywords    : packs          -  the packet at which to start logging
;               packe          -  the packet at which to stop logging
;               quiet          -  no reporting
;               vras,vexp,vdat - select only VDS raster, exposure and/or 
;                                 data packets
;               gras,gexp,gdat - select only GIS raster, exposure and/or 
;                                 data packets
;               msec           - put out time msec values as well
;
; Calls       : None
;
; Common      : None
;               
; Restrictions: None
;               
; Side effects: None
;               
; Category    : Telemetry
;               
; Prev. Hist. : None
;
; Written     : C D Pike, RAL, 23-May-1994
;               
; Modified    : Make it write summary file in current directory and change
;               extension added to be _summ.  CDP, 15-Jun-94
;
;               Make structures strictly same as used in science analysis
;               programs.  CDP, 11-Aug-94
;
;               Added found parameter and quiet keyword.  CDP, 30-May-95
;               Trap empty files.  CDP, 31-May-95
;
;               Extended arrays to catch all HK IDs as well, and
;               make IDs requested default to raster headers. CDP, 15-Jun-95
;
;               Enlarge return array to cope with complete file if needed.
;                                                             CDP, 17-Nov-95
;               Trap incomplete packets if realtime file is being accessed.
;                                                             CDP, 17-nov-95
;               Added extra keywords vras, vexp etc.     CDP, 23-Nov-95
;               Print out file name in summary.  CDP, 13-Dec-95
;
;               Add msec keyword.  CDP, 15-Jan-97
;
; Version     : Version 11, 15-Jan-97
;-            

pro tlm_summary, file, pkt_ids, found, quiet=quiet, packs=packs, $
                       packe=packe, vras=vras, vexp=vexp, vdat=vdat,$
                       gras=gras, gexp=gexp, gdat=gdat, msec=msec

;
;  insist on correct input
;
if n_params() lt 1 then begin
   print,'Use:  tlm_summary, filename [, packet_ids or keywords]
   found = -1
   return
endif

;
;  collect keywords
;
pids = -1
if keyword_set(vras) then pids = [pids,'02'x]
if keyword_set(vexp) then pids = [pids,'12'x]
if keyword_set(vdat) then pids = [pids,'22'x]
if keyword_set(gras) then pids = [pids,'01'x]
if keyword_set(gexp) then pids = [pids,'11'x]
if keyword_set(gdat) then pids = [pids,'21'x]

;
;  msec timing?
;
if keyword_set(msec) then form=9 else form=1

;
;  are defaults required for IDs?
;
if n_elements(pkt_ids) gt 0 then begin
   if n_elements(pids) gt 1 then packet_ids = [pkt_ids,pids(1:*)] $
                            else packet_ids = pkt_ids
endif else begin
   if n_elements(pids) gt 1 then packet_ids = pids(1:*) else packet_ids=[1,2]
endelse

;
;  sort 
;
packet_ids = packet_ids(sort(packet_ids))

;
;  allow user to have own version of telemetry files
;
tfile = ''
if  file_exist(file) then begin
   tfile = file
endif else begin
   if file_exist(concat_dir('$CDS_TM_DATA',file)) then begin
      tfile = concat_dir('$CDS_TM_DATA',file)
   endif
endelse

;
;  telemetry file doesn't exist in user or std directory
;
if tfile eq '' then begin
   print,'TLM_SUMMARY:  Input file does not exist.'
   found = -1
   return
endif

;
;  Define general packet header structure
;
PacketHeader = {PacketHeaderDef,                $
                Msgid:               FIX(0),    $
		MsgLen:              FIX(0),    $
		MsgReqid:            FIX(0),    $    
		ap_id:               BYTARR(2), $
		counter:             FIX(0),    $
		length:              FIX(0),    $
		Time:                BYTARR(6), $
		ID:                  BYTE(0),   $
		StreamPacketCounter: BYTE(0)}

;
; Define Science data header
;
SciHeader = {SciHeaderDef, $
		StudyID:               BYTE(0),   $
		DataCompScheme:        BYTE(0),   $ 
		DataExtWindowID:       FIX(0),    $
		SequenceID:            FIX(0),    $
		RasterLengthM:         BYTE(0),   $
		RasterLengthS:         BYTE(0),   $
		OPSLCoordinate:        FIX(0),    $
		OPSRCoordinate:        FIX(0),    $
                SlitPosition:          FIX(0),    $
		SlitNumber:            BYTE(0),   $
		MirrorPosition:        BYTE(0),   $
		ExposureTime:          FIX(0),    $
		StudyCounter:          FIX(0),    $
		RasterCounter:         FIX(0),    $
                DataCompOption:        FIX(0),    $
                SciProcStatus:         FIX(0)}

;
;  counter
;
count = fix(0)

;
;Exposure header
;
;
Exphdr= {ExpHeaderDef, $
		SolarX:                FIX(0),      $
		ExposureStart:         BYTARR(6),   $
		ExposureNumber:        FIX(0),      $
		OPSLCoordinate:        FIX(0),      $
		OPSRCoordinate:        FIX(0),      $
		SlitPosition:          FIX(0),      $
                SunSensorPitch:        FIX(0),      $
                SunSensorYaw:          FIX(0),      $
                SolarY:                FIX(0),      $
                MirrorPosition:        BYTE(0),     $
                EventStatus:           BYTE(0),     $
                EventLevel:            FIX(0),      $
		StudyCounter:          FIX(0),      $
		RasterCounter:         FIX(0),      $
                Science:               BYTARR(252), $
		ChkSum:                FIX(0)}

;
;  dummy data arrays
;
b260 = bytarr(260)
b286 = bytarr(286)
b284 = bytarr(284)
;
;  accumulative totals for info
;
tot = intarr(225)
sp_tot = intarr(50)


;
;  open input file 
;
openr, unit, tfile, /get_lun

;
;  open output file in the current directory
;
if not keyword_set(quiet) then begin
   break_file,tfile,disk,direct,ff,ext
   openw, lun, ff+ext+'_summ',/get_lun,width=130
endif

;
;  human readable interpretation of ID
;
name = strarr(225)
name(1) = 'GIS-raster'
name(2) = 'VDS-raster'
name(3) = 'Abort'
name(17) = 'GIS-exp'
name(18) = 'VDS-exp'
name(33) = 'GIS-data'
name(34) = 'VDS-data'
name(48) = 'Special'
name(16) = 'HK Snooze mode'
name(32) = 'HK Standby mode'
name(64) = 'HK Ready-GIS mode'
name(80) = 'HK Ready-VDS mode'
name(96) = 'HK Ready-Both mode'
name(128) = 'HK Execute-GIS mode'
name(144) = 'HK Execute-VDS mode'
name(224) = 'HK Emergency Standby mode'

;
;  and of special packet formats
;
sp_name = strarr(20)
sp_name(1)  = '(MCU)'
sp_name(2)  = '(Deferred)'
sp_name(3)  = '(Data line list)'
sp_name(4)  = '(Raster table)'
sp_name(5)  = '(Sequence table)'
sp_name(6)  = '(CDS h/k table)'
sp_name(7)  = '(Error)'
sp_name(8)  = '(Health)'
sp_name(9)  = '(IMIF)'
sp_name(10)  = '(Hardware list)'
sp_name(11)  = '(Mode table)'
sp_name(12)  = '(Engineering table)'
sp_name(13)  = '(Sequence controller table)'
sp_name(14) = '(IEF)'

;
;  print statistics summary
;

pname = strarr(225)
pname(*) = '          ?           '
pname(10) = 'Engineering A'
pname(11) = 'Engineering B'
pname(1)  = 'GIS Raster header'
pname(2)  = 'VDS Raster header'
pname(3)  = 'Raster abort'
pname(17)  = 'GIS Exposure header'
pname(18)  = 'VDS Exposure header'
pname(33)  = 'GIS Science data'
pname(34)  = 'VDS Science data'
pname(4)  = 'Primary Proc. Mem. Dump'
pname(5)  = 'Secondary Proc. Mem. Dump'
pname(6)  = 'VDS Memory Dump'
pname(7)  = 'GIS raw data dump'
pname(48)  = 'Special Packet'
pname(16) = 'HK Snooze mode'
pname(32) = 'HK Standby mode'
pname(64) = 'HK Ready-GIS mode'
pname(80) = 'HK Ready-VDS mode'
pname(96) = 'HK Ready-Both mode'
pname(128) = 'HK Execute-GIS mode'
pname(144) = 'HK Execute-VDS mode'
pname(224) = 'HK Emergency Standby mode'



;
;  title
;
if not keyword_set(quiet) then begin
   print,'================================================'
   print,'From telemetry file: ',tfile
   print,'================================================'

   printf,lun,'================================================'
   printf,lun,'From telemetry file: ',tfile
   printf,lun,'================================================'

  
   print,' Packet      Type             Time            Seq-id   Raster-count  '+$
         'S-step M-step   Exp #   Slit Posn    Mirror Posn'
   printf,lun,' Packet      Type              Time           Seq-id   Raster-count  '+$
              'S-step M-step   Exp #   Slit Posn    Mirror Posn'
endif

;
;  any packets to be skipped?
;
if not keyword_set(packs) then packs = 0L
if not keyword_set(packe) then packe = 1000000L

counter = packs

;
;  skip to start place
;
if packs gt 0 then point_lun, unit, long(packs)*306L

;
;  initialise the return variable
;
fid = -1
found = intarr(n_elements(packet_ids),20000)
found(*,*) = -1

;
;  for incomplete files...
;
on_ioerror, duff_file


;
;  begin reading file
;
while not eof(unit) and (counter le packe) do begin
   readu, unit, PacketHeader
;
;  produce totals
;
   if packetheader.id lt 225 then tot(packetheader.id) = tot(packetheader.id)+1
   if packetheader.id eq '30'x then begin
      temp = read_sp_id(unit) 
      if (temp(1) lt 50) and (temp(1) gt 0) then begin
         sp_tot(temp(1)) = sp_tot(temp(1))+1
      endif else begin
         if not keyword_set(quiet) then begin
            print,counter,'Special packet format ID out of range. Value = ',temp(1),$
                           form='(1x,i5,4x,a,i4)'
            printf,lun,counter,$
                          'Special packet format ID out of range. Value = ',temp(1),$
                           form='(1x,i5,4x,a,i4)'
         endif
      endelse
      readu,unit,b284
   endif

;
;  was ID wanted?
;
   hit = where(packet_ids eq PacketHeader.id)
   if hit(0) ge 0 then begin
      fid = fid + 1
      found(hit(0),fid) = counter
      case packetheader.id of
         '1'x: begin
                  readu,unit,sciheader
                  readu,unit,b260
               end
         '2'x: begin
                  readu,unit,sciheader
                  readu,unit,b260
               end
        '11'x: begin
                  readu,unit,count
                  readu,unit,exphdr
               end
        '12'x: begin
                  readu,unit,count
                  readu,unit,exphdr
               end
        '30'x:
         else: readu,unit,b286
      endcase
         
      if packetheader.id eq '1'x or packetheader.id eq '2'x then begin
         time = anytim2cal(tai2utc(obt2tai(packetheader.time)),form=form)
         if not keyword_set(quiet) then begin
            print,counter, name(packetheader.id), time, $
                           byteswap(sciheader.sequenceid),$
                           byteswap(sciheader.rastercounter),$
                           sciheader.rasterlengths,$
                           sciheader.rasterlengthm,$
                           form='(1x,i5,4x,a10,2x,a24,1x,z4.4,5x,i6,4x,i6,1x,i6)'
            printf,lun,counter, name(packetheader.id), time, $
                           byteswap(sciheader.sequenceid),$
                           byteswap(sciheader.rastercounter),$
                           sciheader.rasterlengths,$
                           sciheader.rasterlengthm,$
                           form='(1x,i5,4x,a10,2x,a24,1x,z4.4,5x,i6,4x,i6,1x,i6)'
         endif
      endif

      if packetheader.id eq '11'x or packetheader.id eq '12'x then begin
         time = anytim2cal(tai2utc(obt2tai(exphdr.exposurestart)),form=form)
         slitpos_decode,byteswap(exphdr.slitposition),sb,sp
         if not keyword_set(quiet) then begin
            print,counter, name(packetheader.id), time, $
                           byteswap(exphdr.exposurenumber),$
                           sp,$
                           exphdr.mirrorposition,$
                           format='(1x,i5,4x,a10,2x,a24,32x,i10,7x,i4,9x,i4)'
            printf,lun,counter, name(packetheader.id), time, $
                           byteswap(exphdr.exposurenumber),$
                           sp,$
                           exphdr.mirrorposition,$
                           format='(1x,i5,4x,a10,2x,a24,32x,i10,7x,i4,9x,i4)'
         endif
      endif

      if packetheader.id eq '21'x or packetheader.id eq '22'x then begin
         time = anytim2cal(tai2utc(obt2tai(packetheader.time)),form=form)
         if not keyword_set(quiet) then begin
            print,counter, name(packetheader.id),$
                                         time,form='(1x,i5,4x,a10,2x,a24)'
            printf,lun,counter, $
                   name(packetheader.id),time,form='(1x,i5,4x,a10,2x,a24)'
         endif
      endif

      if packetheader.id eq '30'x then begin
          sp_id = temp(1) 
          time = anytim2cal(tai2utc(obt2tai(packetheader.time)),form=form)
          if (sp_id lt 50) and (sp_id ge 0) then begin
             if not keyword_set(quiet) then begin
                print,counter, name(packetheader.id),$
                          time,sp_name(sp_id),form='(1x,i5,4x,a10,2x,a24,2x,a30)'
                printf,lun,counter, name(packetheader.id),$
                          time,sp_name(sp_id),form='(1x,i5,4x,a10,2x,a24,2x,a30)'
             endif
          endif else begin
             if not keyword_set(quiet) then begin
                print,counter, name(packetheader.id),$
                  time,'(Incorrect format ID)',form='(1x,i5,4x,a10,2x,a24,2x,a30)'
                printf,lun,counter, name(packetheader.id),$
                  time,'(Incorrect format ID)',form='(1x,i5,4x,a10,2x,a24,2x,a30)'
             endif
          endelse
      endif

   endif else begin
      if packetheader.id ne '30'x then readu,unit,b286
   endelse

;
;  next packet
;
   counter = counter + 1

endwhile

;
;  jump to here if io error
;
duff_file:

;
;  trap empty files, after recent experience
;
if counter eq 0 then begin
   bell
   print,'Warning: Telemetry file appears to be empty.'
   found = -1
   if not keyword_set(quiet) then free_lun,lun
   free_lun,unit
   return
endif

;
;  all was well so continue with output
;
if not keyword_set(quiet) then begin
   print,' '
   print,'   Packet statistics.'
   print,'   -----------------'
   print,' '
   print,'For packets ',fmt_vect([packs,counter-1],delim=' to ')
   print,' '
   print,'Packet ID               Type                Number of packets'
   print,'  (hex)
   for i=0,224 do begin
      if tot(i) gt 0 then print,i,pname(i),tot(i),form='(3x,z2.2,7x,a25,15x,i5)
   endfor
   print,' '
   if total(sp_tot) gt 0 then begin
      print,'Specials....'
      for i=0,49 do begin
         if sp_tot(i) gt 0 then print,i,sp_name(i),sp_tot(i),$
                                                    form='(3x,z2.2,7x,a25,15x,i5)
      endfor
   endif

;
;  hard copy
;

   printf,lun,' '
   printf,lun,'   Packet statistics.'
   printf,lun,'   -----------------'
   printf,lun,' '
   printf,lun,'For packets ',fmt_vect([packs,counter-1],delim=' to ')
   printf,lun,' '
   printf,lun,'Packet ID               Type                Number of packets'
   printf,lun,'  (hex)
   for i=0,224 do begin
      if tot(i) gt 0 then begin
         printf,lun,i,pname(i),tot(i),form='(3x,z2.2,7x,a25,15x,i5)
      endif
   endfor
   printf,lun,' '
   if total(sp_tot) gt 0 then begin
      printf,lun,'Specials....'
      for i=0,49 do begin
         if sp_tot(i) gt 0 then printf,lun,i,sp_name(i),sp_tot(i),$
                                                    form='(3x,z2.2,7x,a25,15x,i5)
      endfor
   endif


  
;
;  close files
;
   free_lun, lun 
endif
free_lun, unit

;
;  trim found array in case needed
;
if fid ge 0 then found = found(*,0:fid) else found = -1



end
   
