;+
; Project     : SOHO - CDS     
;                   
; Name        : TM_CALIB()
;               
; Purpose     : To calibrate the CDS engineering telemetry data.
;               
; Explanation : The procedure emon can be used to extract engineering data
;               from the telemetry stream.  This routine takes the raw values
;               and 'calibrates' them according to the menu given in the
;               telemetry database.  The form of the result will depend on
;               the parameter being calibrated.
;               
; Use         : cal_data = tm_calib(name, raw_data)
;    
; Inputs      : name     - the telemetry parameter name, needed to lookup
;                          the calibration details.
;
;               raw_data - the raw telemetry values to be calibrated 
;               
; Opt. Inputs : None
;               
; Outputs     : Words - for calibration type 1 then there are two
;                       associated words for the values 0/1.  These are
;                       returned in this variable if it is given
;    
;               Function itself returns vector of calibrated data values.
;               
; Opt. Outputs: None
;               
; Keywords    : QUIET - suppresses warning message.
;
; Calls       : GET_MASK_ETC - to get calibration details.
;
; Common      : TM_CALIB is an internal common block used to store information
;		between calls.
;               
; Restrictions: None
;               
; Side effects: None
;               
; Category    : Telemetry
;               
; Prev. Hist. : None
;
; Written     : C D Pike, RAL, 22-Dec-94
;               
; Modified    : Fix interpretation of +/- signs in parameter names.  CDP
;                                                          6-Jan-95
;
;               Fix bug in calib type 8.  CDP, 10-Jan-95
;               And again, another one.  CDP, 23-Jan-95
;               Add type 6 calibration.  CDP, 30-Jan-95
;               Add text_list for string plotting. CDP, 3-Feb-95
;               Add type 7 calibration (log plot). CDP, 18-Sep-95
;		Version 8, William Thompson, GSFC, 7 November 1995
;			Added common block to speed up.
;               Added /quiet keyword.  CDP, 18-Dec-95
;		Version 10, William Thompson, GSFC, 19 April 1996
;			Changed loop to type long.
;
; Version     : Version 10, 19 April 1996
;-            

function tm_calib, name, raw_data, words, quiet=quiet
common tm_calib, details
;
;  list of calibration (type 6) files that contain text strings rather than 
;  numerical data.  This must be updated if more files are added.
;
text_list = [4,5,6,7,8,9,10]

;
;  If not already read in, access the database details.
;
if n_elements(details) eq 0 then begin
    get_mask_etc, details

;
;  parameter names returned have the +/- signs as in original database, got
;  to replace those with the internal version
;
    names = reform(details(0,*))
    for i=0,n_elements(names)-1 do names(i) = repstr(names(i),'+','_POS_')
    for i=0,n_elements(names)-1 do names(i) = repstr(names(i),'-','_NEG_')
    details(0,*) = names

;
;  also the numerical code doesn't like null strings so replace with zeros
;
    n = where(details eq '')
    if n(0) ge 0 then details(n) = '0' 
endif

;
;  select parameter in question
;
n = where(details(0,*) eq strupcase(name))
if n(0) ge 0 then begin
   detail = details(*,n(0))
endif else begin
   bell
   print,'Parameter name not recognised. Data not calibrated.
   words = ['Uncalibrated',name,' ']
   return, raw_data
endelse

;
;  what type of calibration? (7 = 8 for this purpose)
;
ctype = fix(detail(8))
if ctype eq 7 then ctype = 8


case ctype of

;
;  straight through
;
   0: begin
         words = ['title',strtrim(detail(17),2),' ']
         return, raw_data
      end

;
;  on/off type variable
;
   1: begin
         n = where(raw_data gt 0)
         temp = raw_data
         if n(0) ge 0 then temp(n) = 1
         words = [detail(9), detail(10), strtrim(detail(17),2)]
         return, temp
      end
;
;  calibration in a world of its own
;
   6: begin
         cd = fltarr(n_elements(raw_data))
         number = fix(detail(9))
         caldata = tm_read_cal(number)
         if n_elements(caldata) eq 1 then begin
            bell
            words = ['Uncalibrated',name,' ']
            return, raw_data
         endif else begin
            s = find_common([number],text_list)
            if s(0) lt 0 then begin
               x = caldata(0,*)
               y = caldata(1,*)
               for i=0,n_elements(raw_data)-1 do begin
                  n = where(x ge raw_data(i))
                  if n(0) ge 1 then begin
                     n = min(n)
                     cd(i) = (raw_data(i)-x(n-1))/(x(n)-x(n-1))*(y(n)-y(n-1)) + y(n-1)
                  endif else begin
                     if (raw_data(i) lt x(0)) and $
                                             (not keyword_set(quiet)) then begin 
                        print,'Data outside calibration.'
                     endif
                     cd(i) = y(0)
                  endelse
               endfor
               words = ['title',strtrim(detail(17),2),' ']
               return,cd
            endif else begin
               words = 'Label'
               n = where(raw_data eq 0) 
               if n(0) ge 0 then cd(n) = 0
               for i=0,(n_elements(caldata)/2)-1 do begin
                  n = where(raw_data eq caldata(0,i))   
                  if n(0) ge 0 then cd(n) = i+1
                  words = [words,caldata(1,i)]
               endfor
               words = [words,strtrim(detail(17),2)]
               return, cd
            endelse
         endelse
      end

;
;  linear interpolation between 2 (x,y) pairs
;
   8: begin
         for i=9,16 do begin
            if not valid_num(detail(i)) then begin
               bell
               print,'Illegal calibration value (type 8). Data not calibrated.'
               words = ['Uncalibrated',name,' ']
               return, raw_data
            endif
         endfor

         x = fltarr(4)  &  y = fltarr(4)
         n = indgen(4)*2+9
         x = float(detail(n))
         n = n+1
         y = float(detail(n))
         n = where( (x+y) ne 0 )
;
;  a (0,0) pair at the beginning is a valid calibration pair
;  but not elsewhere.
;
         if x(0)+y(0) eq 0 then begin
            x = [x(0),x(n)] 
            y = [y(0),y(n)]
         endif else begin 
            x = x(n) 
            y = y(n)
         endelse

         cd = fltarr(n_elements(raw_data))
         for i=0L,n_elements(raw_data)-1 do begin
            n = where(x ge raw_data(i))
            if n(0) ge 1 then begin
               n = min(n)
               cd(i) = (raw_data(i)-x(n-1))/(x(n)-x(n-1))*(y(n)-y(n-1)) + y(n-1)
            endif else begin
                     if (raw_data(i) lt x(0)) and $
                                             (not keyword_set(quiet)) then begin 
                        print,'Data outside calibration.'
                     endif
               cd(i) = y(0)
            endelse
         endfor
         if fix(detail(8)) eq 7 then begin
            words = ['Ltitle',strtrim(detail(17),2),' ']
         endif else begin
            words = ['title',strtrim(detail(17),2),' ']
         endelse
         return,cd
      end
  else: print,'Calibration type ', ctype, ' not handled.  
endcase
       

end
   
