;+
; Project     : SOHO - CDS     
;                   
; Name        : WHERE_RASTER()
;               
; Purpose     : Locates start packets of GIS or VDS rasters in tm file.
;               
; Explanation : Searches packet headers in a telemetry file for GIS/VDS
;               raster headers and reports the packet numbers where rasters
;               start. If neither detector is specified, the starts of both
;               are returned in a (2,N) array.   The size of N is determined
;               by the numbers of rasters in either case (inapplicable entries
;               have a value -1)
;               
; Use         : IDL> print,where_raster(file [,/gis, /vds, /rt])
;    
; Inputs      : file - telemetry file name (assumed in CDS_TM_DATA unless
;                      directory specified).
;               
; Opt. Inputs : None
;               
; Outputs     : Function returns raster start packet numbers (-1 if none)
;               
; Opt. Outputs: None
;               
; Keywords    : VDS  -  look for VDS rasters
;               GIS  -  look for GIS rasters
;               RT   -  use current realtime telemetry file rather than a
;                       replay file.
;
; Calls       : tlm_summary
;
; Common      : None
;               
; Restrictions: None
;               
; Side effects: None
;               
; Category    : Telemetry
;               
; Prev. Hist. : None
;
; Written     : C D Pike, RAL, 30-May-95
;               
; Modified    : 
;
; Version     : Version 1, 30-May-95
;-            

function where_raster, file, vds=vds, gis=gis, rt=rt

;
;  check have sufficuent input
;
if (n_params() eq 0) or (file eq '') then begin
   print,' '
   print,'Use:    IDL> print,where_raster(file [,/vds, /gis])
   print,' '
   return, -1
endif

;
;  did file have directory attached?
;
if not keyword_set(rt) then begin
   break_file, file, disk, dir, f, ext
   if dir eq '' then $
               use_file = concat_dir('$CDS_TM_DATA',file) else use_file=file
endif else begin
   use_file = gt_rt_file()
endelse

;
;  call tlm_summary in quiet mode to do the work
;
tlm_summary,use_file,[1,2],found,/quiet

;
;    if nothing found then return
;
if max(found) eq -1 then return,found


;
;  was only GIS requested?
;
if keyword_set(gis) and (not keyword_set(vds)) then begin
   found = found(0,*)
   found = found(where(found ge 0))
   return,found
endif

;
;  was only VDS requested?
;
if keyword_set(vds) and (not keyword_set(gis)) then begin
   found = found(1,*)
   found = found(where(found ge 0))
   return,found
endif

;
;  if neither keyword specified or both were then return all
;
return, found

end

