	PRO CDS_MESSAGE, MSG, CONTINUE=CONT, INFORMATIONAL=INF,	$
		ATTENTION=ATTN, ADD_TIME=ADD_TIME, BELL=BELLS
;+
; Project     : SOHO - CDS
;
; Name        : 
;	CDS_MESSAGE
; Purpose     : 
;	Writes error messages to the log file.
; Explanation : 
;	Writes an informational or error message to the telemetry processing
;	log file.  Emulates the standard IDL routine MESSAGE.  Currently, for
;	test purposes, the message is only written to the screen, using either
;	the text widget in FITSGEN, or the standard IDL routine MESSAGE.
; Use         : 
;	CDS_MESSAGE, MESSAGE
; Inputs      : 
;	MESSAGE		= Message to be written to the log file.
; Opt. Inputs : 
;	None.
; Outputs     : 
;	None.
; Opt. Outputs: 
;	None.
; Keywords    : 
;	CONTINUE	= Continue after writing out the error message.
;	INFORMATIONAL	= Informational message only, not error.  The message
;			  is not printed to the screen if !QUIET is set.
;	ATTENTION	= Also call CDS_ATTENTION to bring this error to the
;			  attention of the CDS operators.
;	BELL		= If set, then ring the bell.  This is less obtrusive
;			  than using /ATTENTION.
;
;	If neither CONTINUE or INFORMATIONAL are set, then an error condition
;	will be signalled, CDS_ATTENTION will be called automatically, and the
;	user will be returned to the calling procedure in a stopped state.
; Calls       : 
;	CDS_ATTENTION, DATATYPE
; Common      : 
;	FITSGEN from the include file "fitsgen.cmn"
; Restrictions: 
;	Because CDS_MESSAGE does not a priori know what routine called it, the
;	name of the routine should be evident from the text of the message.
; Side effects: 
;	May set the system variables !ERR, !ERROR and !ERR_STRING.  If
;	CDS_MESSAGE is called incorrectly, then a message describing the error
;	will be printed instead, and the IDL program will be brought to a halt.
; Category    : 
;	Data Handling, Telemetry.
; Prev. Hist. : 
;	None.
; Written     : 
;	William Thompson, GSFC, February 1993.
; Modified    : 
;	Version 1, William Thompson, GSFC, February 1993.
;	Version 2, William Thompson, GSFC, 17 September 1993.
;		Modified to allow multi-line messages.
;	Version 3, William Thompson, GSFC, 31 August 1994
;		Modified to use FITSGEN text widget when available.
;	Version 4, William Thompson, GSFC, 1 September 1994
;		Modified to write to LOG_UNIT when available.
;	Version 5, William Thompson, GSFC, 1 November 1995
;		Added keyword ADD_TIME
;	Version 6, William Thompson, GSFC, 22 November 1995
;		Also add packet number to ADD_TIME messages.
;	Version 7, William Thompson, GSFC, 2 May 1996
;		Added keyword /BELL
; Version     : 
;	Version 6, 22 November 1995
;-
;
	ON_ERROR, 2
;
;  Include the common block to support the FITSGEN routine.
;
@fitsgen.cmn
;
;  Get the values of the keywords.
;
	CONTINUE = KEYWORD_SET(CONT)
	INFORMATIONAL = KEYWORD_SET(INF)
	ATTENTION = KEYWORD_SET(ATTN)
;
;  Test the input parameter MESSAGE.  If CDS_MESSAGE is called improperly, then
;  the program will be brought to a complete halt.
;
	IF N_PARAMS() EQ 0 THEN BEGIN
		MESSAGE = "Syntax:  CDS_MESSAGE, MESSAGE"
		CONTINUE = 0
		INFORMATIONAL = 0
	END ELSE IF N_ELEMENTS(MSG) EQ 0 THEN BEGIN
		MESSAGE = "CDS_MESSAGE:  MESSAGE not defined"
		CONTINUE = 0
		INFORMATIONAL = 0
	END ELSE BEGIN
		MESSAGE = MSG
		IF DATATYPE(MESSAGE,1) NE 'String' THEN BEGIN
			MESSAGE = "CDS_MESSAGE:  MESSAGE must be a string"
			CONTINUE = 0
			INFORMATIONAL = 0
		ENDIF
	ENDELSE
;
;  If requested, add the current packet time to the message:
;
	IF KEYWORD_SET(ADD_TIME) AND (DATATYPE(LPACKET,1) EQ 'Structure') $
		THEN MESSAGE = [TAI2UTC(OBT2TAI(LPACKET.HDR.TIME),/ECS) + $
		' (packet ' + TRIM(I_PACK) + '): ', MESSAGE]
;
;  If the error message is severe--i.e. both CONTINUE and INFORMATIONAL are
;  zero, or ATTENTION is set--then bring the error to the attention of the CDS
;  operators.
;
	IF (NOT CONTINUE) AND (NOT INFORMATIONAL) THEN ATTENTION = 1
	IF ATTENTION THEN CDS_ATTENTION, MESSAGE
;
;  Write out the message to the screen.
;
	N = N_ELEMENTS(MESSAGE)
;
;  If FITSGEN is running, then write the message to the scrolling text window
;  in that widget.  Also write it into the log file.
;
	IF KEYWORD_SET(MSG_WIDG) THEN BEGIN
		IF NOT (KEYWORD_SET(INFORMATIONAL) AND !QUIET) THEN	$
			FOR I=0,N-1 DO BEGIN
				WIDGET_CONTROL, MSG_WIDG, /APPEND, $
					SET_VALUE=MESSAGE(I)
				IF KEYWORD_SET(LOG_UNIT) THEN	$
					PRINTF, LOG_UNIT, MESSAGE(I)
			ENDFOR
;
;  Otherwise, call the MESSAGE facility to write the message.  The way the
;  program is written, at least one of CONTINUE, ATTENTION or INFORMATIONAL has
;  to be set.  Thus, this allows the preliminary lines to be written.
;
	END ELSE BEGIN
		CONT0 = CONTINUE OR ATTENTION
		FOR I = 0, N-2 DO MESSAGE, MESSAGE(I), CONTINUE=CONT0,	$
			INFORMATIONAL=INFORMATIONAL
		MESSAGE, MESSAGE(N-1), CONTINUE=CONTINUE,	$
			INFORMATIONAL=INFORMATIONAL
;
;  If the BELL keyword was set, then ring the bell.
;
		IF KEYWORD_SET(BELLS) THEN BELL, BELLS
	ENDELSE
;
	RETURN
	END
