	PRO CDS_POINT, RASTER, TIMES, POINTINGS
;+
; Project     :	SOHO - CDS
;
; Name        :	CDS_POINT
;
; Purpose     :	Calculate CDS pointing information for FITS header
;
; Explanation :	This procedure uses the pointing information embedded in the
;		telemetry to calculate the values of the pointing parameters to
;		be placed in the CDS FITS header.
;
; Use         :	CDS_POINT, RASTER, TIMES, POINTINGS
;
; Inputs      :	RASTER	  = Structure variable from READ_RASTER.
;		TIMES	  = Structure variable from GETCDSTIMES.
;
; Opt. Inputs :	None.
;
; Outputs     :	POINTINGS = A structure containing the following tags:
;
;			SC_X0	= Spacecraft pointing in X from orbit file
;			SC_Y0	= Spacecraft pointing in Y from orbit file
;			SC_ROLL = Spacecraft roll from orbit file
;			INS_X0	= Instrument pointing in X from OPS system
;			INS_Y0	= Instrument pointing in Y from OPS system
;			INS_ROLL= Instrument roll (presumed zero).
;			INS_X	= Array of instrument pointings in X with
;				  mirror and slit positions thrown in.
;			INS_Y	= Array of instrument pointings in Y with
;				  mirror and slit positions thrown in.
;			XCEN	= Center of field-of-view in the X direction
;			YCEN	= Center of field-of-view in the Y direction
;			ANGLE	= Angle of instrument field-of-view relative to
;				  true Solar X and Y directions, measured
;				  counter-clockwise
;			IXWIDTH	= Width of field-of-view in the instrument X
;				  direction
;			IYWIDTH	= Width of field-of-view in the instrument Y
;				  direction
;			XSTART	= Position of first pixel in the X direction
;				  for each data window
;			XDELTA	= Delta between pixels in the X direction for
;				  each data window
;			YSTART	= Position of first pixel in the Y direction
;				  for each data window
;			YDELTA	= Delta between pixels in the Y direction for
;				  each data window
;
; Opt. Outputs:	None.
;
; Keywords    :	None.
;
; Calls       :	MECH_POINT, OPS_POINT, GET_VDS_SLIT, BASE_INTERP
;
; Common      :	None.
;
; Restrictions:	At this point the software assumes that ANGLE is zero.
;
;		Currently, the software uses OPS mechanism positions to
;		determine the pointing.  Later the sun sensor data may be used
;		instead of the OPS mechanism positions, or else the SOLAR_X,
;		SOLAR_Y values in the telemetry.
;
;		The values used for the slit and pixel sizes are only to the
;		nearest arcsec.  More precise values need to be used.
;
; Side effects:	None.
;
; Category    :	Data_handling, Telemetry, I/O, FITS, CDS.
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 28 December 1994
;
; Modified    :	Version 1, William Thompson, GSFC, 26 September 1995
;		Version 2, William Thompson, GSFC, 17 October 1995
;			Corrected way that VDS_OFFSET is handled.
;		Version 3, William Thompson, GSFC, 1 November 1995
;			Changed to use VDS_CENTRE.
;		Version 4, William Thompson, GSFC, 15 November 1995
;			Corrected calculation of YSTART for VDS.
;		Version 5, William Thompson, GSFC, 16 November 1995
;			Split off routine GET_VDS_SLIT
;		Version 6, William Thompson, GSFC, 4 December 1995
;			Add call to GET_SC_ATT.  Modify XCEN, YCEN accordingly.
;		Version 7, William Thompson, GSFC, 7 December 1995
;			Call GET_SC_POINT instead of GET_SC_ATT
;		Version 8, William Thompson, GSFC, 21 February 1996
;			Corrected VDS pixel scale.
;		Version 9, William Thompson, GSFC, 22 February 1996
;			Corrected NIS slit length.
;		Version 10, William Thompson, GSFC, 23 February 1996
;			Corrected calculation of window position's effect on
;			pointing.  Resolved problems with differing VDS and
;			datawindow coordinate systems.
;		Version 11, William Thompson, GSFC, 4 March 1996
;			Correct for behavior of slit mechanism.
;		Version 12, William Thompson, GSFC, 17 June 1996
;			Make more robust when the initial slit, mirror position
;			is bad.
;		Version 13, William Thompson, GSFC, 18 June 1996
;			Made even more robust by calling BASE_INTERP
;		Version 14, William Thompson, GSFC, 12 August 1996
;			Corrected YSTART for GIS data.
;		Version 15, William Thompson, GSFC, 16 April 1997
;			Made more sophisticated when OPS positions change
;			during raster.
;		Version 16, William Thompson, GSFC, 8 May 1997
;			Added keyword DATE_OBS to call to OPS_POINT
;		Version 17, 29-Jul-1997, William Thompson, GSFC
;			Make even more sophisticated when OPS moves during
;			observation.
;		Version 18, 24-Nov-1997, William Thompson, GSFC
;			Correctly handle binning.
;		Version 19, 23-Dec-1997, William Thompson, GSFC
;			Correctly handle mechanism positions for GIS
;		Version 20, 31-Dec-1997, William Thompson, GSFC
;			Don't use slit positions for NIS data
;
; Version     :	Version 20, 31-Dec-1997
;-
;
	ON_ERROR, 2
;
;  Check the number of paramters.
;
	IF N_PARAMS() NE 3 THEN MESSAGE,	$
		'Syntax:  CDS_POINT, RASTER, TIMES, POINTINGS'
;
;  Get the spacecraft pointing information.
;
	MESSAGE = ''
	ATT = GET_SC_POINT(TIMES.OBT_TIME, ERRMSG=MESSAGE)
	IF MESSAGE NE '' THEN CDS_MESSAGE, 'CDS_POINT: ' + MESSAGE,	$
		/CONTINUE, /ATTENTION
;
;  Get the pointing information, either from the exposure headers if they
;  exist, or from the raster header.
;
	NX = RASTER.HEADER.SCI.NUMMIRRORPOS > 1
	NY = RASTER.HEADER.SCI.NUMSLITPOS   > 1
	IF TOTAL(RASTER.HEADERFOUND) GT 0 THEN BEGIN
		W = WHERE(RASTER.HEADERFOUND)
		OPSLPOSITION   = (RASTER.OPSLPOSITION)(W)
		OPSRPOSITION   = (RASTER.OPSRPOSITION)(W)
;
;  If the data don't change by more than two, then simply average them.
;  Otherwise, extrapolate to W=0.
;
		IF ((MAX(OPSLPOSITION)-MIN(OPSLPOSITION)) LE 2) AND	$
			((MAX(OPSRPOSITION)-MIN(OPSRPOSITION)) LE 2)	$
			THEN BEGIN
		    OPSLPOSITION = AVERAGE(OPSLPOSITION)
		    OPSRPOSITION = AVERAGE(OPSRPOSITION)
		END ELSE BEGIN
		    WW = INDGEN(N_ELEMENTS(W))
		    REPEAT BEGIN
			N = N_ELEMENTS(WW)
			W = W(WW)
			OPSLPOSITION = OPSLPOSITION(WW)
			OPSRPOSITION = OPSRPOSITION(WW)
			PL = POLY_FIT(W, OPSLPOSITION, 1)
			PR = POLY_FIT(W, OPSRPOSITION, 1)
			DL = OPSLPOSITION - PL(0) - PL(1)*W  &  SL = STDEV(DL)
			DR = OPSRPOSITION - PR(0) - PR(1)*W  &  SR = STDEV(DR)
			WW = WHERE((ABS(DL) LT 3*SL) AND (ABS(DR) LT 3*SR))
		    ENDREP UNTIL (N EQ N_ELEMENTS(WW)) OR (WW(0) EQ -1)
		    OPSLPOSITION = PL(0)
		    OPSRPOSITION = PR(0)
		ENDELSE
;
		IF NX*NY GT 1 THEN BEGIN
		    IF RASTER.INSTRUMENT EQ 'GIS' THEN BEGIN
			SLITPOSITION   = REFORM(RASTER.SLITPOSITION,   NY, NX)
			MIRRORPOSITION = REFORM(RASTER.MIRRORPOSITION, NY, NX)
			HEADERFOUND    = REFORM(RASTER.HEADERFOUND,    NY, NX)
			SLITPOSITION   = TRANSPOSE(SLITPOSITION)
			MIRRORPOSITION = TRANSPOSE(MIRRORPOSITION)
			HEADERFOUND    = TRANSPOSE(HEADERFOUND)
		    END ELSE BEGIN
			SLITPOSITION   = REFORM(RASTER.SLITPOSITION,   NX, NY)
			MIRRORPOSITION = REFORM(RASTER.MIRRORPOSITION, NX, NY)
			HEADERFOUND    = REFORM(RASTER.HEADERFOUND,    NX, NY)
		    ENDELSE
		END ELSE BEGIN
			SLITPOSITION   = (RASTER.SLITPOSITION)(0)
			MIRRORPOSITION = (RASTER.MIRRORPOSITION)(0)
			HEADERFOUND    = (RASTER.HEADERFOUND)(0)
			SLITPOS_USED = -1
			MIRPOS_USED  = -1
		ENDELSE
	END ELSE BEGIN
		OPSLPOSITION   = RASTER.HEADER.SCI.OPSLPOSITION
		OPSRPOSITION   = RASTER.HEADER.SCI.OPSRPOSITION
		SLITPOSITION   = RASTER.HEADER.SCI.SLITPOSITION
		MIRRORPOSITION = RASTER.HEADER.SCI.MIRRORPOSITION
		SLITPOS_USED = 0
		MIRPOS_USED  = 0
	ENDELSE
;
;  Calculate the pointing from the OPS mechanism positions.  Assume the
;  instrument roll is zero.
;
	OPS_POINT, OPSLPOSITION, OPSRPOSITION, INS_X0, INS_Y0,	$
		DATE_OBS=TIMES.DATE_OBS
	INS_ROLL = 0.0
;
;  Modify the pointing based on the slit and mirror positions.
;
	XCEN = INS_X0
	YCEN = INS_Y0
	IF RASTER.INSTRUMENT EQ 'NIS' THEN SLITPOSITION = 0*SLITPOSITION
	MECH_POINT, SLITPOSITION, MIRRORPOSITION, XCEN, YCEN
;
;  If an array of centers were calculated, then calculate XCEN by first
;  interpolating to the middle position in the slit direction, and then to the
;  middle position in the mirror direction.
;
	IF N_ELEMENTS(XCEN) GT 1 THEN BEGIN
		IF NY GT 1 THEN BEGIN
		    TEMP = XCEN(*,0)
		    XFOUND = REPLICATE(0,NX)
		    FOR IX = 0,NX-1 DO BEGIN
			XTEMP = XCEN(IX,*)
			HFTEMP = HEADERFOUND(IX,*)
			W = WHERE(HFTEMP, COUNT)
			IF COUNT EQ 1 THEN BEGIN
			    TEMP(IX) = XTEMP(W(0))
			    XFOUND(IX) = 1
			END ELSE IF COUNT GT 1 THEN BEGIN
			    TEMP(IX) = BASE_INTERP(XTEMP(W), W, (NY-1)/2., $
				    MINSIG=1)
			    XFOUND(IX) = 1
			ENDIF
		    ENDFOR
		    XCEN = TEMP
		END ELSE XFOUND = HEADERFOUND
;
		W = WHERE(XFOUND, COUNT)
		IF COUNT EQ 1 THEN BEGIN
			XCEN = XCEN(W(0))
			MIRPOS_USED = W(0)
		END ELSE BEGIN
			XCEN = BASE_INTERP(XCEN(W), W, (NX-1)/2., MINSIG=1)
			MIRPOS_USED = -1
		ENDELSE
;
;  Do the opposite for YCEN.
;
		TEMP = REFORM(YCEN(0,*))
		YFOUND = REPLICATE(0,NY)
		FOR IY = 0,NY-1 DO BEGIN
		    YTEMP = YCEN(*,IY)
		    HFTEMP = HEADERFOUND(*,IY)
		    W = WHERE(HFTEMP, COUNT)
		    IF COUNT EQ 1 THEN BEGIN
			TEMP(IY) = YTEMP(W(0))
			YFOUND(IY) = 1
		    END ELSE IF COUNT GT 1 THEN BEGIN
			TEMP(IY) = BASE_INTERP(YTEMP(W), W, (NY-1)/2., $
				MINSIG=1)
			YFOUND(IY) = 1
		    ENDIF
		ENDFOR
		YCEN = TEMP
;
		W = WHERE(YFOUND, COUNT)
		IF COUNT EQ 1 THEN BEGIN
			YCEN = YCEN(W(0))
			SLITPOS_USED = W(0)
		END ELSE BEGIN
			YCEN = BASE_INTERP(YCEN(W), W, (NY-1)/2., MINSIG=1)
			SLITPOS_USED = -1
		ENDELSE
	END ELSE BEGIN
		MIRPOS_USED  = -1
		SLITPOS_USED = -1
	ENDELSE
;
;  Make sure that XCEN and YCEN end up as scalars.
;
	XCEN = XCEN(0)
	YCEN = YCEN(0)
;
;  Calculate the width due to the slit.
;
	CASE RASTER.HEADER.SCI.SLITNUMBER OF
		1:  BEGIN  &  SXWIDTH =  2  &  SYWIDTH =   2	& END
		2:  BEGIN  &  SXWIDTH =  4  &  SYWIDTH =   4	& END
		3:  BEGIN  &  SXWIDTH =  8  &  SYWIDTH =  50.8	& END
		4:  BEGIN  &  SXWIDTH =  2  &  SYWIDTH = 240	& END
		5:  BEGIN  &  SXWIDTH =  4  &  SYWIDTH = 240	& END
		6:  BEGIN  &  SXWIDTH = 90  &  SYWIDTH = 240	& END
		ELSE: BEGIN
			CDS_MESSAGE, 'CDS_POINT: Unrecognized slit number', $
				/ATTENTION, /CONTINUE
			SXWIDTH = 2
			SYWIDTH = 2
			END
	ENDCASE
;
;  Calculate the relative fields-of-view for each of the data windows depending
;  on the detector.  Also calculate the starting pixel and step size for each
;  window.  First for the GIS.
;
	IF RASTER.INSTRUMENT EQ 'GIS' THEN BEGIN
		IYWIDTH = (NY - 1.) * RASTER.SLITSTEP + SYWIDTH
		IF SLITPOS_USED LT 0 THEN RYCEN = 0 ELSE RYCEN =	$
			(0.5*(NY-1) - SLITPOS_USED) * RASTER.SLITSTEP
		YSTART = REPLICATE(YCEN, RASTER.HEADER.NWINDOWS) -	$
			(IYWIDTH - SYWIDTH) / 2
		YDELTA = REPLICATE(RASTER.SLITSTEP > 1.016,	$
			RASTER.HEADER.NWINDOWS)
;
;  The VDS is more complicated.  One needs to take into account the positions
;  of the data windows relative to the total extent of the slit.  It is assumed
;  that the pointing calculated so far refers to the center of the slit, and
;  that this is SLENGTH/2 pixels above the values for the VDS offset in the
;  state database.  (It is assumed that the NIS is only used with one of the
;  tall slits.)
;
	END ELSE BEGIN
		SCALE = 1.68				;Arcsec/pixel
		SLENGTH = ROUND(240./SCALE)		;NIS slit length (pix)
		GET_VDS_SLIT, TIMES, SY1, SY2, SLENGTH=SLENGTH, RASTER=RASTER
;
;  Extract the data window positions and determine the relevant spectral band.
;  Note that BAND as defined here is 1 less than the number that would appear
;  in the data window database.
;
		Y1 = RASTER.HEADER.DATA_WIN.STARTY		;1st pixel
		Y2 = Y1 + RASTER.HEADER.DATA_WIN.LENGTHY - 1	;last pixel
		IBAND = REPLICATE(0,N_ELEMENTS(Y1))
		W = WHERE( ((Y1+Y2)/2.) LT 511.5, COUNT)
		IF COUNT GT 0 THEN IBAND(W) = 1
;
;  If SY1 and SY2 are two dimensional, then convert IBAND to correctly address
;  these arrays.
;
		IF N_ELEMENTS(SY1) GT 2 THEN IBAND = IBAND +	$
			2*INDGEN(N_ELEMENTS(SY1)/2)
;
;  Determine the effective height of the pixels.
;
		IF RASTER.SP_STATUS.VDS_ORIENT EQ 0 THEN BEGIN
			WIDTH_Y = RASTER.WIDTH2
			PIX_SIZE = RASTER.BIN2
		END ELSE BEGIN
			WIDTH_Y = RASTER.WIDTH1
			PIX_SIZE = RASTER.BIN1
		ENDELSE
;
;  Calculate the starting point and delta for each window.  Remember that the
;  solar image is inverted on the VDS detector.
;
		SYCEN = 0.5 * (SY1(IBAND) + SY2(IBAND))
		YSTART = YCEN - SCALE*(Y2 - SYCEN - 0.5*(PIX_SIZE-1))
		YDELTA = SCALE * PIX_SIZE
;
;  Ignore any data windows that fall outside the slit.  If all of them fall
;  outside the slit, then simply use the default values.
;
		W = WHERE((Y1 LE SY2(IBAND)) AND (Y2 GE SY1(IBAND)), COUNT)
		IF COUNT EQ 0 THEN BEGIN
			Y1 = SY1(0)
			Y2 = SY2(0)
			IBAND = 0
		END ELSE BEGIN
			Y1 = Y1(W)
			Y2 = Y2(W)
			IBAND = IBAND(W)
		ENDELSE
;
;  Calculate the minimum and maximum data window positions relative to the
;  slit.  Ignore any points outside the slit.
;
		Y1 = MIN( (Y1 - SY1(IBAND)) > 0 )
		Y2 = MAX( (Y2 - SY1(IBAND)) < (SLENGTH-1) )
		RYCEN = SCALE * (SLENGTH - 1 - (Y1 + Y2)) / 2
		IYWIDTH = SCALE * (Y2 - Y1 + 1)
	ENDELSE
;
;  The relative width in X is calculated in the same way for either
;  spectrograph.
;
	IXWIDTH = (NX - 1.) * RASTER.MIRRORSTEP + SXWIDTH
;
;  Modify the pointing based on the detector information.  Note that the scan
;  mirror moves backwards.
;
	IF MIRPOS_USED GE 0 THEN XCEN = XCEN - (0.5*(NX-1) - MIRPOS_USED) * $
		RASTER.MIRRORSTEP
	YCEN = YCEN + RYCEN
;
;  Store the starting point and delta in the X direction.
;
	XSTART = XCEN - (IXWIDTH - SXWIDTH) / 2
	XDELTA = RASTER.MIRRORSTEP > 1.016
;
;  Assume that ANGLE is the spacecraft roll.
;
	ANGLE = ATT.SC_ROLL
;
;  Modify the pointing based on ANGLE, and the spacecraft pointing.
;
	XX = XCEN  &  YY = YCEN
	COS_A = COS(ANGLE / !RADEG)
	SIN_A = SIN(ANGLE / !RADEG)
	XCEN = XX * COS_A  -  YY * SIN_A  +  ATT.SC_X0
	YCEN = XX * SIN_A  +  YY * COS_A  +  ATT.SC_Y0
;
	XX = XSTART  &  YY = YSTART
	XSTART = XX * COS_A  -  YY * SIN_A  +  ATT.SC_X0
	YSTART = XX * SIN_A  +  YY * COS_A  +  ATT.SC_Y0
;
;  Calculate the array of instrument pointings from all the mechanism
;  movements.
;
	OPS_POINT,  RASTER.OPSLPOSITION, RASTER.OPSRPOSITION,   INS_X, INS_Y
	SLITPOSITION = RASTER.SLITPOSITION
	IF RASTER.INSTRUMENT EQ 'NIS' THEN SLITPOSITION = 0*SLITPOSITION
	MECH_POINT, slitposition, RASTER.MIRRORPOSITION, INS_X, INS_Y
;
;  Store the information in the output structure and return.
;
	POINTINGS = {SC_X0:	ATT.SC_X0,	$
		     SC_Y0:	ATT.SC_Y0,	$
		     SC_ROLL:	ATT.SC_ROLL,	$
		     INS_X0:	INS_X0,		$
		     INS_Y0:	INS_Y0,		$
		     INS_ROLL:	INS_ROLL,	$
		     INS_X:	INS_X,		$
		     INS_Y:	INS_Y,		$
		     XCEN:	XCEN,		$
		     YCEN:	YCEN,		$
		     ANGLE:	ANGLE,		$
		     IXWIDTH:	IXWIDTH,	$
		     IYWIDTH:	IYWIDTH,	$
		     XSTART:	XSTART,		$
		     XDELTA:	XDELTA,		$
		     YSTART:	YSTART,		$
		     YDELTA:	YDELTA}
	RETURN
	END
