	FUNCTION CDS_TCUNI, TDESC
;+
; Project     :	SOHO - CDS
;
; Name        :	CDS_TCUNI()
;
; Purpose     :	Forms TCUNI based on TDESC.
;
; Explanation :	Forms the TCUNI string for a FITS binary table column based on
;		the TDESC string for the same column.  This information is
;		extracted from the axis labels based on the following table:
;
;			TDESC		TCUNI
;
;			WAVELNTH	ANGSTROM
;			SOLAR_X		ARCSEC
;			SOLAR_Y		ARCSEC
;			DEL_TIME	SECOND
;
;		Called from CDSADDCOL.
;
; Use         :	Result = CDS_TCUNI(TDESC)
;
; Inputs      :	TDESC = Character string containing the axis labels, e.g.
;
;			"DEL_TIME"		;One-dimensional
;			"(SOLAR_X,SOLAR_Y)	;Multi-dimensional
;
; Opt. Inputs :	None.
;
; Outputs     :	The result of the function are the units corresponding to the
;		axes labels derived from the TDESC string, formatted in the
;		same way as a character string.
;
; Opt. Outputs:	None.
;
; Keywords    :	None.
;
; Calls       :	FXBTDIM
;
; Common      :	None.
;
; Restrictions:	This routine will only work if the labels in TDESC are
;		restricted to the above table.  Any unrecognized axes will show
;		up as a replication of the axis label in the output array.
;
; Side effects:	None.
;
; Category    :	Data_handling, Telemetry, I/O, FITS, CDS.
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 28 December 1994
;
; Modified    :	Version 1, William Thompson, GSFC, 28 December 1994
;
; Version     :	Version 1, 28 December 1994
;-
;
	ON_ERROR, 2
;
;  Check the number of parameters.
;
	IF N_PARAMS() NE 1 THEN MESSAGE, 'Syntax:  Result = CDS_TCUNI( TDESC )'
;
;  Parse the TDESC string into an array of axis labels.
;
	LABELS = FXBTDIM(TDESC)
;
;  Find the corresponding units associated with the axes.
;
	AXES  = ['WAVELNTH','SOLAR_X','SOLAR_Y','DEL_TIME']
	UNITS = ['ANGSTROM','ARCSEC','ARCSEC','SECOND']
	CUNI = LABELS
	FOR I = 0,N_ELEMENTS(LABELS)-1 DO BEGIN
		W = WHERE(LABELS(I) EQ AXES, COUNT)
		IF COUNT NE 0 THEN CUNI(I) = UNITS(W(0))
	ENDFOR
;
;  Reformat the units into the proper string format.
;
	IF N_ELEMENTS(LABELS) EQ 1 THEN TCUNI = CUNI(0) ELSE BEGIN
		TCUNI = '(' + CUNI(0)
		FOR I = 1,N_ELEMENTS(LABELS)-1 DO	$
			TCUNI = TCUNI + ',' + CUNI(I)
		TCUNI = TCUNI + ')'
	ENDELSE
;
	RETURN, TCUNI
	END
