	PRO CDS_TRPIX, TDESC, I_WINDOW, TRPIX, TRVAL, TDELT, TROTA
;+
; Project     :	SOHO - CDS
;
; Name        :	CDS_TRPIX
;
; Purpose     :	Forms pixel reference keywords for CDS binary table.
;
; Explanation :	Forms the pixel reference keywords TRPIX, TRVAL, TDELT, TROTA
;		for a data window in the binary table based on information from
;		the telemetry.
;
; Use         :	CDS_TRPIX, TDESC, I_WINDOW, TRPIX, TRVAL, TDELT, TROTA
;
; Inputs      :	TDESC	    = Column labels from CDSWINEXTR
;		I_WINDOW    = Window that reference information refers to.  If
;			      called for one of the auxilliary data columns,
;			      then this number will be set to -1.
;
; Opt. Inputs :	None.
;
; Outputs     :	TRPIX = The reference pixel position.
;		TRVAL = The axis values at the reference pixel
;		TDELT = The axis increments
;		TROTA = The rotation elements.
;
;		All the output parameters are character strings in the format
;
;			"(Val_1,Val_2,...)'
;
; Opt. Outputs:	None.
;
; Keywords    :	None.
;
; Calls       :	None.
;
; Common      :	FITSGEN contains the arrays WAVELENGTHS, POINTINGS, TIMES.
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Category    :	Data_handling, Telemetry, I/O, FITS, CDS.
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 1 March 1995
;
; Modified    :	Version 1, William Thompson, GSFC, 10 March 1995
;		Version 2, William Thompson, GSFC, 16 October 1995
;			Added DETECTOR as possible array dimension.
;		Version 3, William Thompson, GSFC, 5 December 1995
;			Added TROTA as an output parameter.
;
; Version     :	Version 3, 5 December 1995
;-
;
	ON_ERROR, 2
@fitsgen.cmn
;
;  Check the number of parameters.
;
	IF N_PARAMS() NE 6 THEN MESSAGE, 'Syntax: CDS_TRPIX, TDESC, ' + $
		'I_WINDOW, TRPIX, TRVAL, TDELT, TROTA'
;
;  Extract the axis descriptions, and calculate the number of axes.
;
	DESC = FXBTDIM(TDESC)
	N_AXES = N_ELEMENTS(DESC)
;
;  If there's more than one axis, then use the format "(val,val,...)".
;
	IF N_AXES GT 1 THEN BEGIN
		TRPIX = '('
		TRVAL = '('
		TDELT = '('
		TROTA = '('
	END ELSE BEGIN
		TRPIX = ''
		TRVAL = ''
		TDELT = ''
		TROTA = ''
	ENDELSE
;
;  For each axis, determine the value at the reference pixel, and the
;  increment.  The reference pixel is always the first pixel in the array.
;
	FOR I = 0,N_AXES-1 DO BEGIN
		IF I GT 0 THEN BEGIN
			TRPIX = TRPIX + ','
			TRVAL = TRVAL + ','
			TDELT = TDELT + ','
			TROTA = TROTA + ','
		ENDIF
;
		TRPIX = TRPIX + '1'
;
		CASE DESC(I) OF
		    'WAVELNTH': BEGIN
			TRVAL = TRVAL + TRIM(WAVELENGTHS.TRVAL(I_WINDOW))
			TDELT = TDELT + TRIM(WAVELENGTHS.TDELT(I_WINDOW))
			TROTA = TROTA + '0'
			END
		    'SOLAR_X': BEGIN
			TRVAL = TRVAL + TRIM(POINTINGS.XSTART(I_WINDOW>0))
			TDELT = TDELT + TRIM(POINTINGS.XDELTA)
			TROTA = TROTA + TRIM(POINTINGS.ANGLE)
			END
		    'SOLAR_Y': BEGIN
			TRVAL = TRVAL + TRIM(POINTINGS.YSTART(I_WINDOW>0))
			TDELT = TDELT + TRIM(POINTINGS.YDELTA(I_WINDOW>0))
			TROTA = TROTA + TRIM(POINTINGS.ANGLE)
			END
		    'DEL_TIME': BEGIN
			TRVAL = TRVAL + '0'
			TDELT = TDELT + TRIM(TIMES.PARAM(1))
			TROTA = TROTA + '0'
			END
		    'DETECTOR': BEGIN
			TRVAL = TRVAL + '1'
			TDELT = TDELT + '1'
			TROTA = TROTA + '0'
			END
		ENDCASE
	ENDFOR
;
	IF N_AXES GT 1 THEN BEGIN
		TRPIX = TRPIX + ')'
		TRVAL = TRVAL + ')'
		TDELT = TDELT + ')'
		TROTA = TROTA + ')'
	ENDIF
;
	RETURN
	END
