	PRO CDS_WAVE, RASTER, TIMES, WAVELENGTHS
;+
; Project     :	SOHO - CDS
;
; Name        :	CDS_WAVE
;
; Purpose     :	Calculate CDS wavelength information for FITS header
;
; Explanation :	This procedure uses the information embedded in the telemetry
;		to calculate the values of the wavelength parameters to be
;		placed in the CDS FITS header.
;
; Use         :	CDS_WAVE, RASTER, TIMES, WAVELENGTHS
;
; Inputs      :	RASTER	= Structure variable from READ_RASTER.
;		TIMES	= Structure variable from GETCDSTIMES.
;
; Opt. Inputs :	None.
;
; Outputs     :	WAVELENGTHS = A structure containing the following tags:
;
;			WAVEMIN	 = Minimum wavelength overall
;			WAVEMAX	 = Maximum wavelength overall
;			TWMIN	 = Minimum for each window
;			TWMAX	 = Maximum for each window
;			TWAVE	 = Principal wavelength for each window
;			TRVAL	 = First wavelength in each window
;			TDELT	 = Wavelength increment in each window
;			BAND	 = Wavelength band for each window
;			GR_ORDER = Grating order for each window
;
; Opt. Outputs:	None.
;
; Keywords    :	None.
;
; Calls       :	GET_VDS_SLIT
;
; Common      :	None.
;
; Restrictions:	At this point the software assumes that ANGLE is zero.
;
;		Currently, the software uses OPS mechanism positions to
;		determine the pointing.  Later the sun sensor data may be used
;		instead of the OPS mechanism positions, or else the SOLAR_X,
;		SOLAR_Y values in the telemetry.
;
;		The values used for the slit and pixel sizes are only to the
;		nearest arcsec.  More precise values need to be used.
;
; Side effects:	The start pixel for GIS data windows is modified to agree with
;		the working of PIX2WAVE.
;
; Category    :	Data_handling, Telemetry, I/O, FITS, CDS.
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 1 March 1995
;
; Modified    :	Version 1, William Thompson, GSFC, 3 July 1995
;		Version 2, William Thompson, GSFC, 7 July 1995
;			Corrected bug when linelist ID is zero.
;		Version 3, William Thompson, GSFC, 6 October 1995
;			Corrected so that calculated wavelength band agrees
;			with definition in data window database.
;		Version 4, William Thompson, GSFC, 16 November 1995
;			Added TIMES as an input parameter.
;			Rewrote to ignore background windows.
;		Version 5, William Thompson, GSFC, 12 December 1995
;			Redefine pixel positions.
;			Modify start pixel for GIS windows.
;		Version 6, William Thompson, GSFC, 23 January 1996
;			Fixed bug when only when data window in list.
;		Version 7, William Thompson, GSFC, 12 March 1996
;			Add call to LOAD_WAVECAL
;		Version 8, William Thompson, GSFC, 2 May 1996
;			Use /BELL instead of /ATTENTION
;		Version 9, William Thompson, GSFC, 19-Mar-1997
;			Corrected problem with N_LINES NE N_WINDOWS when
;			background windows are used.
;
; Version     :	Version 9, 19-Mar-1997
;-
;
	ON_ERROR, 2
;
;  Check the number of paramters.
;
	IF N_PARAMS() NE 3 THEN MESSAGE,	$
		'Syntax:  CDS_WAVE, RASTER, TIMES, WAVELENGTHS'
;
;  Load the appropriate wavelength calibration.
;
	IF NOT LOAD_WAVECAL(RASTER.INSTRUMENT, TIMES.OBT_TIME,	$
			GSET_ID=RASTER.GSET_ID) THEN BEGIN
		CDS_MESSAGE, /CONTINUE, /BELL,	$
			['Unable to load wavelength calibration',	$
			'Using current time and GSET_ID=15']
		GET_UTC, UTC
		STATUS = LOAD_WAVECAL(RASTER.INSTRUMENT, UTC, GSET_ID=15)
	ENDIF
;
;  Extract the data window positions, and the relevant spectral band.  Also
;  extract the width of the windows after compression.
;
	IF RASTER.INSTRUMENT EQ 'GIS' THEN BEGIN
		PREFIX = 'GIS'
		X1 = RASTER.HEADER.DATA_WIN.START
		X2 = X1 + RASTER.HEADER.DATA_WIN.LENGTH - 1
		BAND = FIX( X1 / 2048 ) + 1
		X1 = X1 MOD 2048
		X2 = X2 MOD 2048
		WIDTH_X = RASTER.WIDTH1
;
;  Reverse the GIS parameters, and reset the START value.
;
		TEMP = X1
		X1 = 2047 - X2
		X2 = 2047 - TEMP
		RASTER.HEADER.DATA_WIN.START = X1 + (BAND-1)*2048
;
;  NIS section.
;
	END ELSE BEGIN
		PREFIX = 'NIS'
		X1 = RASTER.HEADER.DATA_WIN.STARTX		;1st pixel
		Y1 = RASTER.HEADER.DATA_WIN.STARTY
		X2 = X1 + RASTER.HEADER.DATA_WIN.LENGTHX - 1	;last pixel
		Y2 = Y1 + RASTER.HEADER.DATA_WIN.LENGTHY - 1
		BAND = REPLICATE(1,N_ELEMENTS(Y1))
		W = WHERE( ((Y1+Y2)/2.) LT 511.5, COUNT)
		IF COUNT GT 0 THEN BAND(W) = 2
		IF RASTER.SP_STATUS.VDS_ORIENT EQ 0 THEN	$
			WIDTH_X = RASTER.WIDTH1 ELSE WIDTH_X = RASTER.WIDTH2
	END
;
;  Make the dimensions of X1, X2, and BAND agree with the actual number of data
;  windows.
;
	N_WINDOWS = RASTER.HEADER.NWINDOWS
	X1 = X1(0:N_WINDOWS-1)
	X2 = X2(0:N_WINDOWS-1)
	BAND = BAND(0:N_WINDOWS-1)
;
;  Determine the effective width of the pixels.
;
	PIX_SIZE = (X2 - X1 + 1) / WIDTH_X
;
;  Calculate the minimum and maximum wavelengths, and the wavelength of the
;  first pixel in each window.
;
	PIXELS = [ [X1-0.5], [X2+0.5], [X1 + 0.5*(PIX_SIZE - 1)] ]
	WAVES = 0.*PIXELS
;
;  The calculation must be done band-by-band.
;
	FOR I_BAND = 1,4 DO BEGIN
		W = WHERE(BAND EQ I_BAND, COUNT)
		IF COUNT GT 0 THEN WAVES(W,*) =		$
			PIX2WAVE(PREFIX+TRIM(I_BAND), PIXELS(W,*), /NOLIMIT)
	ENDFOR
;
;  Extract the information, and calculate the average wavelength.
;
	TWMIN = WAVES(*,0)
	TWMAX = WAVES(*,1)
	TRVAL = WAVES(*,2)
	TWAVE = (TWMIN + TWMAX) / 2.
;
;  Calculate the wavelength scale.
;
	TDELT = 2 * (TRVAL - TWMIN)
	W = WHERE(TDELT EQ 0, COUNT)
	IF COUNT GT 0 THEN TDELT(W) = TWMAX(W) - TWMIN(W)
;
;  Make a dummy GR_ORDER array.
;
	GR_ORDER = REPLICATE(1,N_WINDOWS)
;
;  From the data window ID, get the linelist ID.
;
	GET_DATAWIN, RASTER.HEADER.SCI.DATAWINDOWID, DW
	IF DW.DW_ID GT 0 THEN BEGIN
;
;  If there is a linelist, then get the information from there.
;
	    IF DW.LL_ID GT 0 THEN BEGIN
		GET_LINELIST, DW.LL_ID, LL
		IF LL.LL_ID GT 0 THEN BEGIN
		    N_LINES = N_ELEMENTS(LL.LINES)
		    IF (N_LINES EQ N_WINDOWS) OR ((N_LINES+4) EQ N_WINDOWS) $
			    THEN BEGIN
;
;  If everything's okay, then replace the wavelength, waveband, and grating
;  order information.
;
			TWAVE(0)    = LL.LINES.WAVELNTH
			BAND(0)     = LL.LINES.WAVEBAND
			GR_ORDER(0) = LL.LINES.GR_ORDER
;
;  Also, modify the other information to take into account the grating order.
;
			W = WHERE(GR_ORDER GT 1, COUNT)
			IF COUNT GE 1 THEN BEGIN
			    TWMIN(W) = TWMIN(W) / GR_ORDER(W)
			    TWMAX(W) = TWMAX(W) / GR_ORDER(W)
			    TRVAL(W) = TRVAL(W) / GR_ORDER(W)
			    TDELT(W) = TDELT(W) / GR_ORDER(W)
			ENDIF
		    ENDIF
		ENDIF
	    ENDIF
	ENDIF
;
;  Calculate the overall minimum and maximum wavelengths.  For the VDS camera,
;  ignore any windows which are outside the slit--these are probably background
;  windows.  If all of them fall outside the slit, then use all the windows.
;
	IF RASTER.INSTRUMENT EQ 'GIS' THEN BEGIN
		W = INDGEN(N_ELEMENTS(TWMIN))
	END ELSE BEGIN
		Y1 = RASTER.HEADER.DATA_WIN.STARTY		;1st pixel
		Y2 = Y1 + RASTER.HEADER.DATA_WIN.LENGTHY - 1	;last pixel
		IBAND = REPLICATE(0,N_ELEMENTS(Y1))
		W = WHERE( ((Y1+Y2)/2.) LT 511.5, COUNT)
		IF COUNT GT 0 THEN IBAND(W) = 1
		GET_VDS_SLIT, TIMES, SY1, SY2
		W = WHERE((Y1 LE SY2(IBAND)) AND (Y2 GE SY1(IBAND)), COUNT)
		IF COUNT EQ 0 THEN W = INDGEN(N_ELEMENTS(TWMIN))
	ENDELSE
;
	WAVEMIN = MIN(TWMIN(W))
	WAVEMAX = MAX(TWMAX(W))
;
;  Store the information in the output structure and return.
;
	WAVELENGTHS = {			$
		WAVEMIN:  WAVEMIN,	$
		WAVEMAX:  WAVEMAX,	$
		TWMIN:	  TWMIN,	$
		TWMAX:	  TWMAX,	$
		TWAVE:	  TWAVE,	$
		TRVAL:	  TRVAL,	$
		TDELT:	  TDELT,	$
		BAND:	  BAND,		$
		GR_ORDER: GR_ORDER}
;
	RETURN
	END
