	PRO CDSADDCOL, COL, HEADER, DATA, NAME, COMMENT, TUNIT, TDESC, TNULL, $
		TDETX=DETX, TDETY=DETY, DCOMPLEX=DCOMPLEX, BIT=BIT,	$
		LOGICAL=LOGICAL, NOMINMAX=NOMINMAX, INDEX=INDEX
;+
; Project     :	SOHO - CDS
;
; Name        :	CDSADDCOL
;
; Purpose     :	Add a column to a CDS FITS binary table.
;
; Explanation :	Adds a column to a CDS FITS binary table.  Called from
;		MAKECDSFITS.
;
; Use         :	CDSADDCOL, COL, HEADER, DATA, NAME, COMMENT, TUNIT, TDESC, $
;			TNULL  [,TDETX=TDETX  [, TDETY=TDETY ]]
;
; Inputs      :	HEADER	= String array containing FITS extension header.
;		DATA	= IDL variable used to determine the data size and type
;			  associated with the column.
;		NAME	= Column label
;		COMMENT	= Comment associated with the column label.
;		TUNIT	= Units associated with the data in the column.
;		TDESC	= Description of the axes in the column.
;
; Opt. Inputs :	None.
;
; Outputs     :	COL	= Index (1-999) of the created column.
;
; Opt. Outputs:	None.
;
; Keywords    :	TDETX	= If passed and non-negative, then the value to give
;			  for the detector start column.
;		TDETY	= If passed and non-negative, then the value to give
;			  for the detector start row.  If only TDETX is passed
;			  with a non-negative value, then GIS data is assumed.
;
;		DCOMPLEX= If set, and ARRAY is complex, with the first
;			  dimension being two (real and imaginary parts), then
;			  the column is defined as double-precision complex
;			  (type "M").
;		BIT	= If passed, and ARRAY is of type byte, then the column
;			  is defined as containing bit mask arrays (type "X"),
;			  with the value of BIT being equal to the number of
;			  mask bits.
;		LOGICAL	= If set, and array is of type byte, then the column is
;			  defined as containing logical arrays (type "L").
;
;		NOMINMAX= If set, then the TDMIN and TDMAX keywords are not
;			  written to the header.  Setting the keyword LOGICAL
;			  or BIT implies NOMINMAX.
;
;		INDEX	= The index of the data window that the column pertains
;			  to.
;
; Calls       :	CDS_TCUNI, CDS_TRPIX
;
; Common      :	FITSGEN contains the arrays WAVELENGTHS, POINTINGS, TIMES.
;
; Restrictions:	This routine decides whether or not the column being created is
;		for a data window or for auxiliary data based on whether or
;		not the TDETX keyword is passed as a non-negative value.
;
; Side effects:	None.
;
; Category    :	Data_handling, Telemetry, I/O, FITS, CDS.
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 27 December 1994
;
; Modified    :	Version 1, William Thompson, GSFC, 3 July 1995
;		Version 2, William Thompson, GSFC, 5 December 1995
;			Added TROTA to header.
;		Version 3, William Thompson, GSFC, 6 March 1998
;			Improve comments in FITS header.
;
; Version     : Version 3, 6 March 1998
;-
;
	ON_ERROR, 2
@fitsgen.cmn
;
;  Check the input parameters.
;
	IF N_PARAMS() NE 8 THEN MESSAGE, 'Syntax:  CDSADDCOL, '	+ $
		'COL, HEADER, DATA, NAME, COMMENT, TUNIT, TDESC, TNULL'
;
;  Extract the minimum and maximum values, paying attention to any missing
;  values.  Make sure that byte values are converted to integer so that they're
;  not treated as characters.
;
	WGOOD = WHERE(DATA NE TNULL, COUNT)
	IF COUNT EQ 0 THEN BEGIN
		TDMIN = 0
		TDMAX = 0
	END ELSE TDMIN = MIN(DATA(WGOOD), MAX=TDMAX)
	IF DATATYPE(DATA,1) EQ 'Byte' THEN BEGIN
		TDMIN = FIX(TDMIN)
		TDMAX = FIX(TDMAX)
	ENDIF
;
;  Create the column.
;
	FXBADDCOL, COL, HEADER, DATA, NAME, COMMENT, TUNIT=TUNIT,	$
		DCOMPLEX=DCOMPLEX, BIT=BIT, LOGICAL=LOGICAL
;
;  If the data type is integer, then add the TNULL field.
;
	IF DATATYPE(DATA,2) LE 3 THEN FXADDPAR, HEADER, 'TNULL'+TRIM(COL), $
		TNULL, 'Null value for column ' + TRIM(COL)
;
;  Unless the array is a scalar (or an array with the single dimension of 1)
;  add the TDESC field.  Also add the corresponding TCUNI field.
;
	SZ = SIZE(DATA)
	IF (SZ(0) NE 0) AND ((SZ(0) NE 1) OR (SZ(1) NE 1)) THEN	BEGIN
		FXADDPAR, HEADER, 'TDESC'+TRIM(COL), TDESC,	$
			'Axis labels for column ' + TRIM(COL)
		FXADDPAR, HEADER, 'TCUNI'+TRIM(COL), CDS_TCUNI(TDESC),	$
			'Axis units for column ' + TRIM(COL)
	ENDIF
;
;  If the NOMINMAX keyword was not set, then add the TDMIN and TDMIN fields.
;  Setting the keyword LOGICAL or BIT implies NOMINMAX.
;
	IF NOT (KEYWORD_SET(LOGICAL) OR KEYWORD_SET(BIT) OR	$
			KEYWORD_SET(NOMINMAX)) THEN BEGIN
		FXADDPAR, HEADER, 'TDMIN'+TRIM(COL), TDMIN,	$
			'Minimum value in column ' + TRIM(COL)
		FXADDPAR, HEADER, 'TDMAX'+TRIM(COL), TDMAX,	$
			'Maximum value in column ' + TRIM(COL)
	ENDIF
;
;  If TDETX or TDETX,TDETY are passed, then add these to the header.
;
	IF N_ELEMENTS(DETX) EQ 1 THEN TDETX = DETX ELSE TDETX = -1
	IF N_ELEMENTS(DETY) EQ 1 THEN TDETY = DETY ELSE TDETY = -1
	IF TDETX GE 0 THEN BEGIN
		IF TDETY GE 0 THEN BEGIN
			FXADDPAR, HEADER, 'TDETX'+TRIM(COL), TDETX,	$
				'Detector start column'
			FXADDPAR, HEADER, 'TDETY'+TRIM(COL), TDETY,	$
				'Detector start row'
		END ELSE BEGIN
			FXADDPAR, HEADER, 'TDETX'+TRIM(COL), TDETX,	$
				'Detector start position'
		ENDELSE
	ENDIF
;
;  Add the reference pixel and wavelength information.
;
	IF N_ELEMENTS(INDEX) EQ 1 THEN I = INDEX ELSE I = -1
	IF (SZ(0) NE 0) AND ((SZ(0) NE 1) OR (SZ(1) NE 1)) THEN	BEGIN
		CDS_TRPIX, TDESC, I, TRPIX, TRVAL, TDELT, TROTA
		FXADDPAR, HEADER, 'TRPIX' + TRIM(COL), TRPIX,	$
			'Reference pixel for column ' + TRIM(COL)
		FXADDPAR, HEADER, 'TRVAL' + TRIM(COL), TRVAL,	$
			'Prelim. reference values, col. ' + TRIM(COL)
		FXADDPAR, HEADER, 'TDELT' + TRIM(COL), TDELT,	$
			'Prelim. axis increments, col. ' + TRIM(COL)
		FXADDPAR, HEADER, 'TROTA' + TRIM(COL), TROTA,	$
			'Prelim. axis rotations, col. ' + TRIM(COL) + $
			' (deg CCW)'
	ENDIF
;
;  If this is a data window, then add in the wavelength information.
;
	IF I GE 0 THEN BEGIN
		FXADDPAR, HEADER, 'TWAVE' + TRIM(COL), WAVELENGTHS.TWAVE(I), $
			'Principal wavelength in Angstroms'
		FXADDPAR, HEADER, 'TWMIN' + TRIM(COL), WAVELENGTHS.TWMIN(I), $
			'Preliminary minimum wavelength in Angstroms'
		FXADDPAR, HEADER, 'TWMAX' + TRIM(COL), WAVELENGTHS.TWMAX(I), $
			'Preliminary maximum wavelength in Angstroms'
		FXADDPAR, HEADER, 'TWBND' + TRIM(COL), WAVELENGTHS.BAND(I), $
			'Wavelength band'
		FXADDPAR, HEADER, 'TGORD' + TRIM(COL),	$
			WAVELENGTHS.GR_ORDER(I), 'Grating order'
	ENDIF
;
	RETURN
	END
