	FUNCTION CDSAUXEXTR, RASTER, AUX, DIMEN2, DIM_ORDER2, TNULL,	$
		NOREARRANGE=NOREARRANGE
;+
; Project     :	SOHO - CDS
;
; Name        :	CDSAUXEXTR()
;
; Purpose     :	Extract auxilliary data windows with proper dimensions
;
; Explanation :	This routine extracts auxilliary data such as mechanism
;		positions and reformats them into the proper dimensions to be
;		put into FITS files.  Called from MAKECDSFITS.
;
; Use         :	Result = CDSAUXEXTR(RASTER, AUX, DIMEN2, DIM_ORDER2 [, TNULL])
;
; Inputs      :	RASTER	   = Raster structure from FORM_RASTER.
;		AUX	   = The auxilliary data.
;
;		The following parameters are taken from the routine CDSWINEXTR
;
;		DIMEN2	   = The dimensions to use for the auxilliary data.
;		DIM_ORDER2 = The order to use for the dimensions for the
;			     auxilliary data.
;
; Opt. Inputs :	None.
;
; Outputs     :	The output of the function is the selected auxilliary data,
;		properly reformatted.
;
; Opt. Outputs:	TNULL	   = Null value to use for the reformatted data.
;
; Keywords    :	NOREARRANGE = If set, then an array with the proper data type
;			      and dimensions will be returned, but the data
;			      will not be rearranged into the correct order.
;			      This saves time when one is merely setting up the
;			      structure of the FITS binary table.
;
; Calls       :	None.
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Category    :	Data_handling, Telemetry, I/O, FITS, CDS.
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 22 December 1994
;
; Modified    :	Version 1, William Thompson, GSFC, 29 December 1994
;		Version 2, William Thompson, GSFC, 3 July 1995
;			Fixed bug when maximum value of floating point array is
;			negative.
;		Version 3, William Thompson, GSFC, 29 February 1996
;			Fixed bug when values are missing.
;		Version 4, William Thompson, GSFC, 4 March 1996
;			Fixed bug when values are missing and NOREARRANGE
;			keyword is missing.
;
; Version     :	Version 4, 4 March 1996
;-
;
	ON_ERROR, 2
;
;  Check the number of parameters.
;
	IF N_PARAMS() LT 4 THEN MESSAGE, 'Syntax:  Result = ' +	$
		'CDSAUXEXTR( RASTER, AUX, DIMEN2, DIM_ORDER2  [, TNULL] )'
;
;  Find the missing and non-missing values.
;
	WGOOD = WHERE(RASTER.HEADERFOUND EQ 1, GOOD_COUNT)
	WBAD  = WHERE(RASTER.HEADERFOUND NE 1, BAD_COUNT)
;
;  Extract the minimum and maximum values, paying attention to any missing
;  values.
;
	IF GOOD_COUNT EQ 0 THEN BEGIN
		TDMIN = 0
		TDMAX = 0
	END ELSE TDMIN = MIN(AUX(WGOOD), MAX=TDMAX)
;
;  Determine the best value to use for the missing values, depending on the
;  datatype and the range of values in the data.  For byte values, either use
;  255B, or 0B.  Failing either of those, set it to the nonsense value of 256.
;
	TYPE = DATATYPE(AUX,1)
	IF TYPE EQ 'Byte' THEN BEGIN
		TDMIN = FIX(TDMIN)
		TDMAX = FIX(TDMAX)
		IF TDMAX LT 255 THEN BEGIN
			TNULL = 255
		END ELSE IF TDMIN GT 0 THEN BEGIN
			TNULL = 0
		END ELSE BEGIN
			TNULL = 256
		ENDELSE
;
;  For floating point numbers, either use -1 or a simple power of 10 which is
;  larger than anything in the array.
;
	END ELSE IF (TYPE EQ 'Double') OR (TYPE EQ 'Float') THEN BEGIN
		IF TDMIN GE 0 THEN TNULL = -1.0 ELSE	$
			TNULL = 10.^(FIX(ALOG10(TDMAX>1)+1) > 1)
;
;  For integers, either use -1 or the largest possible integer (either positive
;  or negative) for that data type.  If all else fails, use a nonsense value.
;
	END ELSE BEGIN
		IF TDMIN GE 0 THEN TNULL = -1 ELSE BEGIN
			IF TYPE EQ 'Integer' THEN BEGIN
				TNULL = '7FFF'X
				IF TDMAX EQ TNULL THEN TNULL=FIX('8000'X)
				IF TDMIN EQ TNULL THEN TNULL='7FFFFFFF'X
			END ELSE BEGIN			;Long integer
				TNULL = '7FFFFFFF'X
				IF TDMAX EQ TNULL THEN TNULL='80000000'X
				IF TDMIN EQ TNULL THEN TNULL=1E10
			ENDELSE
		ENDELSE
	ENDELSE
;
;  Reformat the input array into the proper dimensions.  If rearrangement is
;  done, then also set any missing data points to the null value.
;
	IF KEYWORD_SET(NOREARRANGE) THEN BEGIN
		DATA = REFORM(AUX(*),DIMEN2(ABS(DIM_ORDER2)-1))
		IF BAD_COUNT GT 0 THEN DATA(WBAD) = TNULL
	END ELSE BEGIN
		DATA = REFORM(AUX(*),DIMEN2)
		IF BAD_COUNT GT 0 THEN DATA(WBAD) = TNULL
		DATA = REARRANGE(DATA, DIM_ORDER2)
	ENDELSE
;
	RETURN, DATA
	END
