	PRO CDSWINEXTR, RASTER, DATAWIN, I_DATAWIN, TDESC, DATA,	$
		TDESC2, DIMEN2, DIM_ORDER2, NOREARRANGE=NOREARRANGE
;+
; Project     :	SOHO - CDS
;
; Name        :	CDSWINEXTR
;
; Purpose     :	Extract data windows with proper dimensions
;
; Explanation :	This routine extracts data windows and reformats them into the
;		proper dimensions to be put into FITS files.
;
; Use         :	CDSWINEXTR, RASTER, DATAWIN, I_DATAWIN, TDESC, DATA
;			[, TDESC2, DIMEN2, DIM_ORDER2 ]
;
; Inputs      :	RASTER	  = Structure variable from READ_RASTER, containing all
;			    the metadata extracted from the science telemetry
;			    stream pertaining to the raster.
;		DATAWIN   = Array from READ_RASTER, containing all the data
;			    windows.
;		I_DATAWIN = Index into the DATAWIN array.
;
; Opt. Inputs :	None.
;
; Outputs     :	TDESC	  = A string containing the dimension descriptors in
;			    the form used by the FITS keyword TDESC, e.g.
;
;				'(WAVELNTH,SOLAR_X,SOLAR_Y)'
;
;		DATA	  = The selected data window, properly reformatted.
;
; Opt. Outputs:	TDESC2	  = The descriptor string for the dimensions of
;			    auxilliary data, such as mechanism positions.
;		DIMEN2	  = The dimensions to use for the auxilliary data.
;		DIM_ORDER2= The order to use for the dimensions for the
;			    auxilliary data.
;
; Keywords    :	NOREARRANGE = If set, then an array with the proper data type
;			      and dimensions will be returned, but the data
;			      will not be rearranged into the correct order.
;			      This saves time when one is merely setting up the
;			      structure of the FITS binary table.
;
; Calls       :	None.
;
; Common      :	None.
;
; Restrictions:	May not work correctly if GIS data windows span detectors.
;
; Side effects:	None.
;
; Category    :	Data_handling, Telemetry, I/O, FITS, CDS.
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 19 October 1994
;
; Modified    :	Version 1, William Thompson, GSFC, 29 December 1994
;		Version 2, William Thompson, GSFC, 25 October 1995
;			Always have a wavelength dimension, even if trivial.
;		Version 3, William Thompson, GSFC, 21 February 1996
;			Correct orientation of VDS data.
;		Version 4, William Thompson, GSFC, 4 March 1996
;			Correct for behavior of slit mechanism.
;		Version 5, William Thompson, GSFC, 13 March 1996
;			Correct bug when only one window.
;		Version 6, William Thompson, GSFC, 20 October 1997
;			If COMPRESSION=8, then adjust the data for partial
;			pixels.
;		Version 7, William Thompson, GSFC, 27 April 1998
;			Fix bug involving adjusting missing pixels.
;
; Version     :	Version 7, 27 April 1998
;-
;
	ON_ERROR, 2
;
;  Check the number of parameters.
;
	IF N_PARAMS() LT 5 THEN MESSAGE, 'Syntax:  CDSWINEXTR, RASTER, ' + $
	    'DATAWIN, I_DATAWIN, TDESC, DATA  [, TDESC2, DIMEN2, DIM_ORDER2 ]'
;
;  Calculate the total number of exposures.
;
	N_MIRROR = RASTER.HEADER.SCI.NUMMIRRORPOS > 1
	N_SLIT   = RASTER.HEADER.SCI.NUMSLITPOS   > 1
	N_EXPOSURES = N_MIRROR * N_SLIT
	N1 = (RASTER.WIDTH1)(I_DATAWIN)
	N2 = (RASTER.WIDTH2)(I_DATAWIN)
;
;  The action of this routine depends on which spectrograph is being used.
;  Start with the Normal Incidence Spectrometer.
;		  =============================
;
	IF RASTER.INSTRUMENT EQ 'NIS' THEN BEGIN
;
;  Start with the default configuration.  Begin by making DESC an array.  It
;  will be converted to a single string later.  Take into account the fact that
;  the scan mirror moves backwards.
;
		DESC = ['WAVELNTH','SOLAR_Y','SOLAR_X']
		DIM_ORDER = [1,-3,2]
		DIMEN = [N1, N2, N_EXPOSURES]
		DESC2 = 'SOLAR_X'
		DIMEN2 = N_EXPOSURES
		DIM_ORDER2 = -1
;
;  If the VDS readout orientation is by column, then reverse the first two
;  dimensions.
;
		IF RASTER.SP_STATUS.VDS_ORIENT EQ 1 THEN BEGIN
			DIM_ORDER = [2,-3,1]
			TEMP = DESC(0)
			DESC(0) = DESC(1)
			DESC(1) = TEMP
		ENDIF
;
;  If the mirror step size is zero, and N_EXPOSURES is greater than 1, then the
;  third dimension is really time.  Insert a dummy dimension for SOLAR_X.
;  Convert the dimension orders back to positive, since the scan mirror does
;  not move.
;
		IF (RASTER.MIRRORSTEP EQ 0) AND (N_EXPOSURES GT 1) THEN BEGIN
			DIMEN = [DIMEN(0:1), 1, DIMEN(2)]
			DESC = [DESC, 'DEL_TIME']
			DIM_ORDER = [ABS(DIM_ORDER), 4]
			DESC2 = 'DEL_TIME'
			DIM_ORDER2 = ABS(DIM_ORDER2)
		ENDIF
;
;  The other case is the Grazing Incidence Spectrometer.
;			 ==============================
;
	END ELSE BEGIN
		DESC = ['WAVELNTH','SOLAR_Y','SOLAR_X']
		DIM_ORDER = [-1,-3,2]
		DIMEN = [N1, N_SLIT, N_MIRROR]
		DESC2 = ['SOLAR_Y','SOLAR_X']
		DIMEN2 = [N_SLIT, N_MIRROR]
		DIM_ORDER2 = [-2,1]
;
;  If either the mirror or slit step size is zero, than that is a time
;  dimension.  If both are zero, then combine the two into a single dimension.
;  Only do this if the time dimension would have more than one element in it.
;
		IF (RASTER.MIRRORSTEP EQ 0) AND (RASTER.SLITSTEP EQ 0) AND $
				(N_EXPOSURES GT 1) THEN BEGIN
			DESC = [DESC,'DEL_TIME']
			DIMEN = [N1,1,1,N_EXPOSURES]
			DIM_ORDER = [-1,3,2,4]
			DESC2 = 'DEL_TIME'
			DIMEN2 = N_EXPOSURES
			DIM_ORDER2 = 1
		END ELSE IF (RASTER.MIRRORSTEP EQ 0) AND (N_MIRROR GT 1) THEN $
				BEGIN
			DESC = [DESC,'DEL_TIME']
			DIMEN = [DIMEN(0:1),1,DIMEN(2)]
			DIM_ORDER = [-1,3,2,4]
			DESC2  = [DESC(1),DESC(3)]
			DIM_ORDER2 = [1,2]
		END ELSE IF (RASTER.SLITSTEP EQ 0) AND (N_SLIT GT 1) THEN BEGIN
			DESC = [DESC(0:1),'DEL_TIME',DESC(2)]
			DIMEN = [DIMEN(0),1,DIMEN(1:*)]
			DIM_ORDER = [-1,-4,2,3]
			DESC2 = DESC(2:3)
;
;  If the number of exposures is 1, then the auxilliary columns are really
;  scalars.  Signal this by setting DESC2 to "DEL_TIME".
;
		END ELSE IF N_EXPOSURES EQ 1 THEN BEGIN
			DESC2 = 'DEL_TIME'
			DIMEN2 = N_EXPOSURES
			DIM_ORDER2 = 1
		ENDIF
	ENDELSE
;
;  Rearrange DESC into the proper order, and convert the DESC array into a
;  single character string.
;
	DESC = DESC(ABS(DIM_ORDER)-1)
	TDESC = '(' + STRTRIM(DESC(0),2)
	FOR I = 1, N_ELEMENTS(DESC)-1 DO	$
		TDESC = TDESC + ',' + STRTRIM(DESC(I),2)
	TDESC = TDESC + ')'
;
;  Do the same for the DESC2 array.
;
	DESC2 = DESC2(ABS(DIM_ORDER2)-1)
	IF N_ELEMENTS(DESC2) EQ 1 THEN TDESC2 = DESC2 ELSE BEGIN
		TDESC2 = '(' + STRTRIM(DESC2(0),2)
		FOR I = 1, N_ELEMENTS(DESC2)-1 DO	$
			TDESC2 = TDESC2 + ',' + STRTRIM(DESC2(I),2)
		TDESC2 = TDESC2 + ')'
	ENDELSE
;
;  Extract the data from the DATAWIN array and reformat it.
;
	IF RASTER.INSTRUMENT EQ 'NIS' THEN	$
		DATA = DATAWIN(0:N1-1,0:N2-1,I_DATAWIN,*) ELSE	$
		DATA = DATAWIN(0:N1-1,I_DATAWIN,*)
;
	IF KEYWORD_SET(NOREARRANGE) THEN BEGIN
	    DATA = REFORM(DATA,DIMEN(ABS(DIM_ORDER)-1),/OVERWRITE)
	END ELSE BEGIN
	    DATA = REARRANGE( REFORM(DATA,DIMEN,/OVERWRITE), DIM_ORDER)
;
;  If the compression mode is 8, then assume that binning along the slit was
;  used, and correct the topmost row for any partial bins.
;
	    IF (RASTER.INSTRUMENT EQ 'NIS') AND		$
		    (RASTER.HEADER.SCI.COMPRESSION EQ 8) THEN BEGIN
		HEIGHT = RASTER.HEADER.DATA_WIN(I_DATAWIN).LENGTHY
		IF RASTER.SP_STATUS.VDS_ORIENT THEN		$
			BINY = RASTER.BIN1(I_DATAWIN) ELSE	$
			BINY = RASTER.BIN2(I_DATAWIN)
;
		PARTIAL = HEIGHT MOD BINY
		IF PARTIAL GT 0 THEN BEGIN
		    SZ = SIZE(DATA)
		    DATA(*,*,SZ(3)-1,*) = -1 > ROUND(DATA(*,*,SZ(3)-1,*) *	$
			    FLOAT(BINY) / PARTIAL)
		ENDIF
	    ENDIF
	ENDELSE
;
;  Find the minimum necessary data type for the data.  Make sure that no
;  trivial (one-element) dimensions are lost.
;
	DATAMAX = MAX(DATA)
	IF (DATATYPE(DATA,1) EQ 'Double') AND (LONG(DATAMAX) EQ DATAMAX) THEN $
		DATA = LONG(DATA)
	IF DATAMAX LE 32767 THEN DATA = FIX(DATA)
	DATA = REFORM(DATA,DIMEN(ABS(DIM_ORDER)-1),/OVERWRITE)
;
	RETURN
	END
