	FUNCTION ENGB_AVERAGE, INPUT
;+
; Project     :	SOHO - CDS
;
; Name        :	ENGB_AVERAGE()
;
; Purpose     :	Averages together CDS Engineering B packets.
;
; Category    :	Telemetry, Science
;
; Explanation :	Averages together CDS Engineering B packets to extract
;		information about exposures to be put into the FITS binary
;		table.  ENGB_STORE should be used to store and retrieve the
;		relevant packets.
;
; Syntax      :	Result = ENGB_AVERAGE( INPUT )
;
; Examples    :	
;
; Inputs      :	INPUT	= The decoded Engineering B packets, in the structure
;			  format {ENGB_PKT} from ENGB_DECODE.
;
; Opt. Inputs :	None.
;
; Outputs     :	The result of the function is an output structure containing
;		various housekeeping parameters.  Each tag in the structure is
;		a four-element array, with one value for each GIS detector.
;
; Opt. Outputs:	None.
;
; Keywords    :	None.
;
; Calls       :	None.
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Prev. Hist. :	None.
;
; History     :	Version 1, 13-Oct-1995, William Thompson, GSFC
;		Version 2, 17-Oct-1995, William Thompson, GSFC
;			Added check for BUGHV values.
;
; Contact     :	WTHOMPSON
;-
;
	ON_ERROR, 2
;
;  Check the number of parameters.
;
	IF N_PARAMS() NE 1 THEN MESSAGE,	$
		'Syntax:  Result = ENGB_AVERAGE( INPUT )'
;
;  Decide if a single packet, or multiple packets have been passed.
;
	N_INPUT = N_ELEMENTS(INPUT)
;
;  Look at the voltages to make sure that at least some of them are above the
;  parking voltage.
;
	BUGHV = FLOAT(INPUT.BUGHV)
	BUGHV(0,*) = TM_CALIB('BUGHV1', BUGHV(0,*))
	BUGHV(1,*) = TM_CALIB('BUGHV2', BUGHV(1,*))
	BUGHV(2,*) = TM_CALIB('BUGHV3', BUGHV(2,*))
	BUGHV(3,*) = TM_CALIB('BUGHV4', BUGHV(3,*))
	TEST = (BUGHV GT 3.4) OR (INPUT.BUGHV EQ -1)
	W = WHERE( TOTAL(TEST, 1) EQ 0, COUNT)
	IF COUNT GT 0 THEN BEGIN
	    MESSAGE =	$
		'Some engineering B packets appear to be outside the exposure'
	    CDS_MESSAGE, /CONTINUE, MESSAGE
	    PRINT, MESSAGE
	    PRINT, INPUT.BUGHV
	ENDIF
;
;  Detector processed event count.  Since the AVERAGE function converts to
;  float, always return floating point values.
;
	IF N_INPUT GT 1 THEN BKGPCD = AVERAGE(INPUT.BKGPCD,2) ELSE	$
		BKGPCD = FLOAT(INPUT.BKGPCD)
;
;  Detector raw event count
;
	IF N_INPUT GT 1 THEN BKGRCD = AVERAGE(INPUT.BKGRCD,2) ELSE	$
		BKGRCD = FLOAT(INPUT.BKGRCD)
;
;  Detector upper level discriminator count
;
	IF N_INPUT GT 1 THEN BKGULCD = AVERAGE(INPUT.BKGULCD,2) ELSE	$
		BKGULCD = FLOAT(INPUT.BKGULCD)
;
;  Form the output structure.
;
	RETURN,{ENGB_EXP,		$
		BKGRCD:  BKGRCD,	$
		BKGPCD:  BKGPCD,	$
		BKGULCD: BKGULCD}
;
	END
