	FUNCTION ENGB_CALIB, INPUT
;+
; Project     :	SOHO - CDS
;
; Name        :	ENGB_CALIB()
;
; Purpose     :	Calibrates CDS Engineering B packets.
;
; Category    :	Telemetry, Science
;
; Explanation :	Applies calibration to Engineering B parameters.  ENGB_STORE
;		should be used to store and retrieve the relevant packets.
;
; Syntax      :	Result = ENGB_CALIB( INPUT )
;
; Examples    :	
;
; Inputs      :	INPUT	= The decoded Engineering B packets, in the structure
;			  format {ENGB_PKT} from ENGB_DECODE.
;
; Opt. Inputs :	None.
;
; Outputs     :	The result of the function is an output structure containing
;		the calibrated values of the engineering parameters.
;
; Opt. Outputs:	None.
;
; Keywords    :	None.
;
; Calls       :	None.
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Prev. Hist. :	None.
;
; History     :	Version 1, 19-Oct-1995, William Thompson, GSFC
;
; Contact     :	WTHOMPSON
;-
;
	ON_ERROR, 2
;
;  Check the number of parameters.
;
	IF N_PARAMS() NE 1 THEN MESSAGE, 'Syntax:  ENGB_CALIB, INPUT'
;
;  Define the output structure.
;
	OUTPUT = {ENGB_CAL,		$
		BUGHV:   FLTARR(4),	$
		BKGCHK:  STRARR(4),	$
		BIGMCP:  FLTARR(4),	$
		BUGMCPB: FLTARR(4),	$
		BKGRCD:  INTARR(4),	$
		BKGPCD:  INTARR(4),	$
		BKGULCD: INTARR(4),	$
		BKGHV_L: FLTARR(4),	$
		BKGLDLD: INTARR(4)}
	N_INPUT = N_ELEMENTS(INPUT)
	IF N_INPUT GT 1 THEN OUTPUT = REPLICATE(OUTPUT, N_INPUT)
;
;  High voltage
;
	OUTPUT(*).BUGHV(0) = TM_CALIB('BUGHV1', INPUT(*).BUGHV(0))
	OUTPUT(*).BUGHV(1) = TM_CALIB('BUGHV2', INPUT(*).BUGHV(1))
	OUTPUT(*).BUGHV(2) = TM_CALIB('BUGHV3', INPUT(*).BUGHV(2))
	OUTPUT(*).BUGHV(3) = TM_CALIB('BUGHV4', INPUT(*).BUGHV(3))
;
;  Lookup table checksum
;
	OUTPUT.BKGCHK = STRING(UNSIGN(INPUT.BKGCHK),FORMAT='(Z4)')
;
;  Detector processed event count.
;
	OUTPUT.BKGPCD = INPUT.BKGPCD
;
;  High voltage current
;
	OUTPUT(*).BIGMCP(0) = TM_CALIB('BIGMCP1', INPUT(*).BIGMCP(0))
	OUTPUT(*).BIGMCP(1) = TM_CALIB('BIGMCP2', INPUT(*).BIGMCP(1))
	OUTPUT(*).BIGMCP(2) = TM_CALIB('BIGMCP3', INPUT(*).BIGMCP(2))
	OUTPUT(*).BIGMCP(3) = TM_CALIB('BIGMCP4', INPUT(*).BIGMCP(3))
;
;  MCP F/face detector
;
	OUTPUT(*).BUGMCPB(0) = TM_CALIB('BUGMCPB1', INPUT(*).BUGMCPB(0))
	OUTPUT(*).BUGMCPB(1) = TM_CALIB('BUGMCPB2', INPUT(*).BUGMCPB(1))
	OUTPUT(*).BUGMCPB(2) = TM_CALIB('BUGMCPB3', INPUT(*).BUGMCPB(2))
	OUTPUT(*).BUGMCPB(3) = TM_CALIB('BUGMCPB4', INPUT(*).BUGMCPB(3))
;
;  Detector raw event count
;
	OUTPUT.BKGRCD = INPUT.BKGRCD
;
;  Detector upper level discriminator count
;
	OUTPUT.BKGULCD = INPUT.BKGULCD
;
;  High voltage last set value
;
	OUTPUT(*).BKGHV_L(0) = TM_CALIB('BKGHV1L', INPUT(*).BKGHV_L(0))
	OUTPUT(*).BKGHV_L(1) = TM_CALIB('BKGHV2L', INPUT(*).BKGHV_L(1))
	OUTPUT(*).BKGHV_L(2) = TM_CALIB('BKGHV3L', INPUT(*).BKGHV_L(2))
	OUTPUT(*).BKGHV_L(3) = TM_CALIB('BKGHV4L', INPUT(*).BKGHV_L(3))
;
;  Detector last setting low level discriminator
;
	OUTPUT.BKGLDLD = INPUT.BKGLDLD
;
	RETURN, OUTPUT
	END
