	PRO ENGB_EXPOSURE, RASTER, EXPOSURE, RESET=RESET
;+
; Project     :	SOHO - CDS
;
; Name        :	ENGB_EXPOSURE
;
; Purpose     :	Extract Engineering B data for the current exposure
;
; Category    :	Data_handling, Telemetry, Science
;
; Explanation :	Uses ENGB_STORE and ENGB_AVERAGE to extract the Engineering B
;		data associated with the current exposure.
;
;		When the raster header is received, the routine should be
;		called with the syntax
;
;			ENGB_EXPOSURE, RASTER, /RESET
;
;		to initialize the stored array.  After each exposure, call with
;		the syntax
;
;			ENGB_EXPOSURE, RASTER, EXPOSURE
;
;		to store the information.  At the end of the raster, call with
;		the syntax
;
;			ENGB_EXPOSURE, RESULT
;
;		to return the stored information.
;
; Syntax      :	ENGB_EXPOSURE, RASTER, /RESET		;To init. stored array
;		ENGB_EXPOSURE, RASTER, EXPOSURE		;To insert exp. entry
;		ENGB_EXPOSURE, RESULT			;To return stored array
;
; Examples    :	
;
; Inputs      :	RASTER	 = Raster structure, as defined by FORM_RASTER
;		EXPOSURE = Current exposure index
;
; Opt. Inputs :	None.
;
; Outputs     :	None.
;
; Opt. Outputs:	RESULT of the function is an output structure containing
;		various housekeeping parameters.  Each tag in the structure is
;		a four-element array, with one value for each GIS detector.
;
; Keywords    :	RESET	 = If set, then the stored array is reinitialized.
;
; Calls       :	ENGB_STORE, ENGB_AVERAGE
;
; Common      :	ENGB_EXPOSURE is a private common block used to save
;		information between calls.
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Prev. Hist. :	If no appropriate Engineering B packets are found, then all
;		returned values are set to -1.
;
; History     :	Version 1, 25-Aug-1995, William Thompson, GSFC
;		Version 2, 13-Oct-1995, William Thompson, GSFC
;			Modified to call ENGB_AVERAGE instead of ENGB_DECODE
;		Version 3, 17-Oct-1995, William Thompson, GSFC
;			Allow for a 2 second delay in the engineering data
;
; Contact     :	WTHOMPSON
;-
;
	ON_ERROR, 2
	COMMON ENGB_EXPOSURE, ENGB_EXP
;
;  Check the number of parameters.
;
	IF N_PARAMS() LT 1 THEN BEGIN
		MESSAGE, 'Syntax:  ENGB_EXPOSURE, RASTER, /RESET', /CONTINUE
		MESSAGE, '  or     ENGB_EXPOSURE, RASTER, EXPOSURE', /CONTINUE
		MESSAGE, '  or     ENGB_EXPOSURE, RESULT'
	ENDIF
;
;  If the RESET keyword was set, then reinitialize the stored array.
;
	IF KEYWORD_SET(RESET) THEN BEGIN
		ENGB_EXP = ENGB_DECODE({ANYPACKETPKT})
		ENGB_EXP = ENGB_AVERAGE(ENGB_EXP)
		ENGB_EXP = REPLICATE(ENGB_EXP, N_ELEMENTS(RASTER.HEADERFOUND))
		RETURN
	ENDIF
;
;  If only one parameter was passed, then return the result.
;
	IF N_PARAMS() EQ 1 THEN BEGIN
		RASTER = ENGB_EXP	;Really "RESULT"
		RETURN
	ENDIF
;
;  Get the exposure start and end times.  Allow for a 0-2 second delay in the
;  time.
;
	TAI1 = OBT2TAI(RASTER.STARTTIME(*,EXPOSURE)) + 2
	TAI2 = TAI1 + RASTER.HEADER.SCI.EXPOSURETIME*0.1
;
;  Retrieve the packets within these times.
;
	ENGB_STORE, PACKETS, /EXTRACT, RANGE=[TAI1,TAI2]
;
;  Derive and store the parameters.
;
	ENGB_EXP(EXPOSURE) = ENGB_AVERAGE(PACKETS)
;
	END
