	PRO ENGB_STORE, PACKET, RESET=RESET, EXTRACT=EXTRACT, RANGE=RANGE, $
		TIME=TIME
;+
; Project     :	SOHO - CDS
;
; Name        :	ENGB_STORE
;
; Purpose     :	Store CDS Engineering B packets for later use.
;
; Category    :	Telemetry, Science
;
; Explanation :	Decodes and stores the last 2000 CDS Engineering B packets for
;		later use in extracting information to be put into the FITS
;		header.  Also stores the first Engineering B packet after the
;		receipt of the raster header packet.
;
;		Each time an Engineering B packet is encountered, this routine
;		should be called with the syntax
;
;			ENGB_STORE, PACKET
;
;		When a raster header packet is encountered, this routine should
;		be called with the syntax
;
;			ENGB_STORE, /RESET
;
;		After each exposure, the following format should be used to
;		extract all packets taking during (or around) the range of time
;		given by the keyword RANGE.  (Done in ENGB_EXPOSURE.)
;
;			ENGB_STORE, PACKET, /EXTRACT, RANGE=[TAI1,TAI2]
;
;		When one wants to write out the FITS file, this routine should
;		be called with the syntax
;
;			ENGB_STORE, PACKET, /EXTRACT
;
;		To extract the first packet that was seen after the last
;		/RESET.
;
; Syntax      :	ENGB_STORE, PACKET		To decode a packet
;		ENGB_STORE, /RESET		To start looking for next pkt
;		ENGB_STORE, PACKET, /EXTRACT	To extract packet
;		ENGB_STORE, PACKET, /EXTRACT, RANGE=RANGE
;						To extract pkts in time range
;
; Examples    :	IF PACKET.HDR.ID EQ '0B'X THEN ENGB_STORE, PACKET
;
;		IF (PACKET.HDR.ID EQ '01'X) OR (PACKET.HDR.ID EQ '02'X) $
;			THEN ENGB_STORE, /RESET
;
;		ENGB_STORE, ENG_B, /EXTRACT
;		OUTPUT = ENGB_DECODE( ENG_B )
;
; Inputs      :	PACKET	= The Engineering B packet, in the structure format
;			  {ANYPACKETPKT}.
;
; Opt. Inputs :	None.
;
; Outputs     :	PACKET	= When used with the /EXTRACT keyword, this is an
;			  output parameter rather than an input parameter.
;			  Then the appropriate stored packet is returned.
;
; Opt. Outputs:	None.
;
; Keywords    :	RESET	= If set, then the software assumes that the next
;			  Engineering B packet will be the first one after the
;			  raster header.
;		EXTRACT	= If set, then data is extracted from the common block
;			  rather than stored.
;		RANGE	= If passed, then will contain a range of TAI times to
;			  use in extracting the packets.  If not passed, then
;			  the first packet after the raster header is returned
;			  instead.
;		TIME	= An array containing the TAI times associated with
;			  each packet returned.  Only valid if both /EXTRACT
;			  and RANGE are passed.
;
; Calls       :	None.
;
; Common      :	Common block ENGB_STORE is a private common block used to store
;		information between calls.
;
; Restrictions:	The routine TM_DEFINE_STRUC must have been called previously to
;		define the structures used in telemetry processing.
;
; Side effects:	If no suitable packet was found, then an empty packet is
;		returned instead.  Such packets are identifiable by the fact
;		that all the values are set to -1.
;
; Prev. Hist. :	None.
;
; History     :	Version 1, 24-Aug-1995, William Thompson, GSFC
;		Version 2, 13-Oct-1995, William Thompson, GSFC
;			Increased NUM_SAVE to 2000.
;			Store only decoded information from packets.
;			Added keyword TIME
;		Version 3, 17-Oct-1995, William Thompson, GSFC
;			Tightened restriction on range.
;
; Contact     :	WTHOMPSON
;-
;
	ON_ERROR, 2
	COMMON ENGB_STORE, ENGB_FIRST, ENGB_FOUND, ENGB_LAST, ENGB_TAI,	$
		ENGB_INDEX
;
;  Make sure that the common block has been initialized.
;
	NUM_SAVE = 2000
	IF N_ELEMENTS(ENGB_LAST) EQ 0 THEN BEGIN
		ENGB_LAST = ENGB_DECODE({ANYPACKETPKT})
		ENGB_LAST = REPLICATE(ENGB_LAST, NUM_SAVE)
		ENGB_TAI  = REPLICATE(0.0D0, NUM_SAVE)
		ENGB_INDEX = -1
	ENDIF
;
;  If the RESET keyword was passed, then unset ENGB_FOUND and return.
;
	IF KEYWORD_SET(RESET) THEN BEGIN
		ENGB_FOUND = 0
		RETURN
	ENDIF
;
;  Otherwise check the number of parameters.
;
	IF N_PARAMS() NE 1 THEN BEGIN
		MESSAGE, 'Syntax:  ENGB_STORE, PACKET', /CONTINUE
		MESSAGE, '  or     ENGB_STORE, /RESET', /CONTINUE
		MESSAGE, '  or     ENGB_STORE, PACKET, /EXTRACT', /CONTINUE
		MESSAGE, '  or     ENGB_STORE, PACKET, /EXTRACT, RANGE=RANGE'
	ENDIF
;
;  If the EXTRACT keyword was passed, then return the information that has been
;  stored.  First see if a time range was selected.  If it was, then look for
;  packets in that time range.  If none are found, return an empty packet.
;
	IF KEYWORD_SET(EXTRACT) THEN BEGIN
		IF N_ELEMENTS(RANGE) EQ 2 THEN BEGIN
			W = WHERE((RANGE(0) LE ENGB_TAI) AND	$
				(ENGB_TAI LE RANGE(1)), COUNT)
			IF COUNT EQ 0 THEN BEGIN
				PACKET = ENGB_DECODE({ANYPACKETPKT})
				TIME = 0.0D0
				RETURN
			ENDIF
			PACKET = ENGB_LAST(W)
			TIME = ENGB_TAI(W)
			RETURN
;
;  A time range was not selected.  If the parameter ENGB_FOUND is set, then
;  return the first packet found after the raster header packet.  If not set,
;  then return the last packet found.  As a last resort, return an empty
;  packet.
;
		END ELSE BEGIN
			IF KEYWORD_SET(ENGB_FOUND) THEN BEGIN
				PACKET = ENGB_FIRST
			END ELSE IF ENGB_INDEX GE 0 THEN BEGIN
				PACKET = ENGB_LAST(ENGB_INDEX)
			END ELSE BEGIN
				PACKET = ENGB_DECODE({ANYPACKETPKT})
			ENDELSE
			RETURN
		ENDELSE
	ENDIF
;
;  No keywords were set.  Decode and store the current packet in the common
;  block.
;
	ENGB_INDEX = (ENGB_INDEX + 1) MOD NUM_SAVE
	DECODED = ENGB_DECODE(PACKET)
	ENGB_LAST(ENGB_INDEX) = DECODED
	ENGB_TAI(ENGB_INDEX)  = OBT2TAI(PACKET.HDR.TIME)
;
;  If ENGB_FOUND is not set, then assume this is the first packet after the
;  raster header.
;
	IF NOT KEYWORD_SET(ENGB_FOUND) THEN BEGIN
		ENGB_FIRST = DECODED
		ENGB_FOUND = 1
	ENDIF
;
	RETURN
	END
