	PRO FIND_CDROM_GAPS, DRIVE=K_DRIVE, ERRMSG=ERRMSG
;+
; Project     :	SOHO - CDS
;
; Name        :	FIND_CDROM_GAPS
;
; Purpose     :	Find all data gaps on a CDROM.
;
; Category    :	Class2, CDS, Operations, Telemetry
;
; Explanation :	Opens up all the .d01 files on a CDROM containing CDS level-0
;		telemetry, and prints out a message about the data time
;		coverage, and any significant gaps.
;
; Syntax      :	FIND_CDROM_GAPS
;
; Examples    :	FIND_CDROM_GAPS
;		FIND_CDROM_GAPS, DRIVE='SYS$CDROM:[000000]'
;
; Inputs      :	None.
;
; Opt. Inputs :	None.
;
; Outputs     :	Information about the CDROM is printed to the terminal
;
; Opt. Outputs:	None.
;
; Keywords    :	DRIVE  = A character string giving the same of the mount
;			 point for the CDROM.  The default is "/cdrom" on
;			 Unix systems, and "CDROM:[000000]" on VMS
;			 systems.
;
;		ERRMSG = If defined and passed, then any error messages will be
;			 returned to the user in this parameter rather than
;			 depending on the MESSAGE routine in IDL.  If no errors
;			 are encountered, then a null string is returned.  In
;			 order to use this feature, ERRMSG must be defined
;			 first, e.g.
;
;				ERRMSG = ''
;				FIND_CDROM_GAPS, ERRMSG=ERRMSG, ...
;				IF ERRMSG NE '' THEN ...
;
; Calls       :	FIND_GAPS
;
; Common      :	None.
;
; Restrictions:	A CDROM must be mounted.
;
; Side effects:	None.
;
; Prev. Hist. :	None.
;
; History     :	Version 1, 1-May-1997, William Thompson, GSFC
;		Version 2, 1-May-1997, William Thompson, GSFC
;			Added keyword ERRMSG
;		Version 3, 8-Mar-2002, William Thompson, GSFC
;			Made sure that input DRIVE keyword not modified.
;
; Contact     :	WTHOMPSON
;-
;
	ON_ERROR, 2
;
;  Determine the value of the DRIVE parameter.
;
	IF N_ELEMENTS(K_DRIVE) EQ 1 THEN DRIVE = K_DRIVE ELSE BEGIN
		CASE OS_FAMILY() OF
			'unix':	DRIVE = '/cdrom'
			'vms':	DRIVE = 'CDROM:[000000]'
			ELSE:	BEGIN
				MESSAGE = 'Only supported in VMS or Unix'
				GOTO, HANDLE_ERROR
				END
		ENDCASE
	ENDELSE
;
;  Decide whether or not upper- or lowercase filenames are being used.
;
	VOLDESC = FINDFILE(CONCAT_DIR(DRIVE,'voldesc.sfd*'), COUNT=COUNT)
	LOWER = COUNT NE 0
	IF NOT LOWER THEN BEGIN
		VOLDESC = FINDFILE(CONCAT_DIR(DRIVE,'VOLDESC.SFD*'),	$
			COUNT=COUNT)
		UPPER = COUNT NE 0
		IF NOT UPPER THEN BEGIN
			MESSAGE = 'Incorrect CDROM mounted'
			GOTO, HANDLE_ERROR
		ENDIF
	ENDIF
;
;  Set up the name of the base directory for CDS telemetry data.
;
	DIRS = ['data', 'so']
	IF NOT LOWER THEN DIRS = STRUPCASE(DIRS)
	BASE = CONCAT_DIR(DRIVE, /DIR, DIRS(0))
	BASE = CONCAT_DIR(BASE,  /DIR, DIRS(1))
;
;  Step through the three CDS telemetry rates, and collect all the filenames.
;
	RATES = ['g026', 'g027', 'g028']
	LZ = 'lz'
	DAT_FILES = '*.d*'
	IF NOT LOWER THEN BEGIN
		RATES = STRUPCASE(RATES)
		LZ = STRUPCASE(LZ)
		DAT_FILES = STRUPCASE(DAT_FILES)
	ENDIF
	FILES = ''
	FOR I_RATE = 0, 2 DO BEGIN
;
;  Set up the name of the directory containing the telemetry data.
;
	    PATH = CONCAT_DIR(BASE, /DIR, RATES(I_RATE))
	    PATH = CONCAT_DIR(PATH, /DIR, LZ)
;
;  Find all the data files in this directory.  Make sure that only the highest
;  version number is used.
;
	    TEMP = FINDFILE(CONCAT_DIR(PATH, DAT_FILES), COUNT=COUNT)
	    IF COUNT GT 0 THEN BEGIN
		BREAK_FILE, TEMP, DISK, DIR, NAME
		S = REVERSE(SORT([NAME]))
		TEMP = TEMP(S)
		NAME = NAME(S)
		LAST = ''
		FOR I = 0, COUNT-1 DO BEGIN
		    TEST = STRMID(NAME(I),0,4)
		    IF TEST NE LAST THEN FILES = [FILES, TEMP(I)]
		    LAST = TEST
		ENDFOR
	    ENDIF
	ENDFOR
;
;  Remove the dummy first filename.
;
	N_FILES = N_ELEMENTS(FILES) - 1
	IF N_FILES EQ 0 THEN BEGIN
		MESSAGE = 'No data files found on the CDROM'
		GOTO, HANDLE_ERROR
	ENDIF
	FILES = FILES(1:*)
;
;  Step through all the data files, and find any gaps.
;
	IF N_FILES GT 1 THEN FILES = FILES(SORT(FILES))
	FOR I_FILE = 0, N_FILES-1 DO FIND_GAPS, FILES(I_FILE)
	RETURN
;
;  Error handling point.
;
HANDLE_ERROR:
	IF N_ELEMENTS(ERRMSG) NE 0 THEN ERRMSG = 'LOAD_CDROM: ' + MESSAGE $
		ELSE MESSAGE, MESSAGE, /CONTINUE
;
	END
