	PRO FIND_GAPS, FILENAME, TIME
;+
; Project     :	SOHO - CDS
;
; Name        :	FIND_GAPS
;
; Purpose     :	Find all gaps in telemetry files of 1 minute or more.
;
; Category    :	Class3, CDS, Operations, Telemetry
;
; Explanation :	Opens up a telemetry file, reads in all the packet times,
;		(except for housekeeping packets), and finds all gaps of 1
;		minute or more in length.
;
; Syntax      :	FIND_GAPS, FILENAME  [, TIME ]
;
; Examples    :	FIND_GAPS, '60960404.d01'
;		FIND_GAPS, 'tm.97_112_21'
;
;		FILES = FINDFILE('*.d01')
;		FOR I=0,N_ELEMENTS(FILES)-1 DO FIND_GAPS, FILES(I)
;
; Inputs      :	FILENAME = The name of the telemetry file.  Any kind of
;			   telemetry file is allowed, including directly off of
;			   CDROMs.
;
; Opt. Inputs :	None.
;
; Outputs     :	A message about the file is written to the screen.
;
; Opt. Outputs:	TIME	 = Returns the packet times, in TAI format.
;
; Keywords    :	None.
;
; Calls       :	OPEN_TM_FILE, OBT2TAI, TAI2UTC
;
; Common      :	Uses fitsgen.cmn
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Prev. Hist. :	None.
;
; History     :	Version 1, 25-Apr-1997, William Thompson, GSFC
;
; Contact     :	WTHOMPSON
;-
;
@fitsgen.cmn
	ON_ERROR, 2
;
;  Check the input parameters.
;
	IF N_PARAMS() LT 1 THEN MESSAGE,	$
		'Syntax:  FIND_GAPS, FILENAME  [, TIME ]'
;
;  Open the telemetry file.
;
	GET_LUN, UNIT
	IF NOT OPEN_TM_FILE(UNIT, FILENAME, TEL_MODE) THEN BEGIN
		FREE_LUN, UNIT
		MESSAGE, 'Unable to open file ' + FILENAME
	ENDIF
;
;  Determine the packet size, and the number of packets.
;
	IF TEL_MODE LT 2 THEN BEGIN
		BSIZE = 306
		STAT = FSTAT(UNIT)
		NUM_PACK = STAT.SIZE / BSIZE
	END ELSE BSIZE = 300
;
;  Read in the packets, 10000 at a time.
;
	N_LEFT = NUM_PACK
	N_TIME = 0
	N_STEP = 10000
	WHILE N_LEFT GT 0 DO BEGIN
		B = BYTARR(BSIZE, N_LEFT < N_STEP)
		READU, UNIT, B
;
;  Extract the packet times and APIDs.  Filter out any packets with the
;  housekeeping APID.
;
		O = BSIZE - 300
		T = OBT2TAI(B(O+6:O+11,*))
		APID = B(O+1,*)
		W = WHERE(APID NE 99, COUNT)
		IF COUNT GT 0 THEN BEGIN
			IF COUNT NE NUM_PACK THEN T = T(W)
			IF N_TIME EQ 0 THEN TIME = T ELSE TIME = [TIME, T]
			N_TIME = N_TIME + N_ELEMENTS(T)
		ENDIF
		N_LEFT = N_LEFT - N_STEP
	ENDWHILE
	FREE_LUN, UNIT
;
;  Print the time range covered.
;
	PRINT, 'File: ' + FILENAME
	PRINT, 'Packets run from ' + TAI2UTC(TIME(0),/VMS,/TRUNC) + ' to ' + $
		TAI2UTC(TIME(N_ELEMENTS(TIME)-1),/VMS,/TRUNC)
;
;  Find all gaps of 1 minute or more.
;
	TDIFF = LONG((TIME(1:*) - TIME) / 60)
	W = WHERE(TDIFF GT 0, COUNT)
	IF COUNT GT 0 THEN HPRINT, 'Gap from ' +	$
			TAI2UTC(TIME(W),/ECS,/TIME,/TRUNC) + ' to ' +	$
			TAI2UTC(TIME(W+1),/ECS,/TIME,/TRUNC) + ' (' +	$
			STRTRIM(TDIFF(W),2) + ' minutes)'
	PRINT, ''
;
	END
