	PRO FIND_IN_PLAN, RASTER, PLAN, RASTER_DEF
;+
; Project     :	SOHO - CDS
;
; Name        :	FIND_IN_PLAN
;
; Purpose     :	Finds the current raster in the science plan
;
; Explanation :	This procedure compares the data in the raster header in the
;		telemetry, and compares it against the science plan so that a
;		correspondence can be made.
;
; Use         :	FIND_IN_PLAN, RASTER, PLAN, RASTER_DEF
;
; Inputs      :	RASTER	= Structure variable from READ_RASTER
;
; Opt. Inputs :	None.
;
; Outputs     :	PLAN	   = Structure variable as returned from GET_DETAIL.
;		RASTER_DEF = Structure variable from GET_RASTER.
;
; Opt. Outputs:	None.
;
; Keywords    :	None.
;
; Calls       :	MATCH_PLAN, TAG_EXIST
;
; Common      :	FIND_PLAN_CMN is an internal common block used to keep track of
;		information between calls.
;
;		Also uses the FITSGEN common block from the file fitsgen.cmn.
;
; Restrictions:	None.
;
; Side effects:	The parameter GSET_ID is added to the RASTER structure.
;
; Category    :	Data_handling, Telemetry, I/O, FITS, CDS.
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 28 December 1994
;
; Modified    :	Version 1, William Thompson, GSFC, 27 June 1995
;		Version 2, William Thompson, GSFC, 3 August 1995
;			Add PROG_IND to PLAN output structure.
;			Assign SEQ_NUM if not already exists.
;			Add FITSGEN common block
;			Remove TIMES and I_REPEAT_S from parameter list.
;		Version 3, William Thompson, GSFC, 21 August 1995
;			Fixed bugs in interaction with MAIN and EXPERIMENT
;			catalogs.
;		Version 4, William Thompson, GSFC, 18 October 1995
;			Added check if SQNUM_UNIT is open.
;		Version 5, William Thompson, GSFC, 1 November 1995
;			Use TAG_EXIST for N_REPEAT_S
;		Version 6, William Thompson, GSFC, 2 November 1995
;			Use routine GET_FLAG_ITIME
;		Version 7, William Thompson, GSFC, 28 February 1996
;			Made allocation of SEQ_NUM more robust
;		Version 8, William Thompson, GSFC, 6 March 1996
;			Pay attention to N_POINTINGS.
;			Put ZONE_ID in the output structure.
;			Use TOP_LEVEL keyword with TAG_EXIST
;		Version 9, William Thompson, GSFC, 12 March 1996
;			Add GSET_ID to RASTER structure.
;		Version 10, William Thompson, GSFC, 28 March 1996
;			Close and reopen sequence number file to force a flush
;			to the disk.
;		Version 11, William Thompson, GSFC, 16 April 1996
;			Don't reset RASTER_DEF if a match can't be found.
;		Version 12, William Thompson, GSFC, 12 September 1996
;			Treat STUDYCOUNTER as an unsigned integer.
;		Version 13, William Thompson, GSFC, 4 June 1997
;			Insert PROG_NUM into SCI_DETAILS database.
;		Version 14, William Thompson, GSFC, 13 June 1997
;			Make sure that SQNUM_MAX is a long integer
;		Version 15, William Thompson, GSFC, 11 August 1997
;			Added /TOP_LEVEL to ADD_TAG calls
;		Version 16, William Thompson, GSFC, 11 September 1997
;			Always set ZONE_ID=0 for NIS data
;		Version 17, 19-Mar-1999, William Thompson, GSFC
;			Check both ALT and FLAG databases
;
; Version     : Version 17, 19-Mar-1999
;-
;
	ON_ERROR, 2
	COMMON FIND_PLAN_CMN, CURR_PLAN, CURR_STUDY, N_REPEATS
@fitsgen.cmn
;
;  Check the number of parameters.
;
	IF N_PARAMS() NE 3 THEN MESSAGE, 'Syntax:  FIND_IN_PLAN, ' +	$
		'RASTER, PLAN, RASTER_DEF'
;
;  Make sure the common block is initialized.
;
	IF N_ELEMENTS(CURR_PLAN) EQ 0 THEN BEGIN
		CURR_PLAN = {STUDY_ID: -1}
		CURR_STUDY = -1
		N_REPEATS = -1
	ENDIF
;
;  Make sure that RASTER_DEF is defined.
;
	RASTER_DEF = {RAS_ID: -1, RAS_VAR: -1, LL_ID: -1}
;
;  From the CDHS sequence ID, get the STUDY_ID and STUDYVAR parameters.
;
	ST = GET_CDHS_SERIES( RASTER.HEADER.SCI.SEQUENCEID )
	IF ST.ID NE -1 THEN BEGIN
		RASTER.STUDY_ID = ST.ID
		RASTER.STUDYVAR = ST.VAR
	ENDIF
;
;  Get the start and stop times.  A distinction should be made between
;  DATE_OBS/DATE_END and OBT_TIME/OBT_END.  Not only are they in different
;  formats, but DATE_OBS/DATE_END may have a correction applied to it for the
;  light travel time between the earth and the SOHO spacecraft.
;
	DATE_OBS = TIMES.DATE_OBS
	DATE_END = TIMES.DATE_END
	OBT_TIME = TIMES.OBT_TIME
	OBT_END  = TIMES.OBT_END
;
;  First check to see if any entry has already been made in the CDS data
;  catalog for this observation.
;
	GET_EXPER, UNSIGN(RASTER.HEADER.SCI.STUDYCOUNTER),	$
		RASTER.HEADER.SCI.RASTERCOUNTER, OBS
;
;  If something was found, then get the raster information, and the repeat
;  index.
;
	FOUND_IN_EXPER = OBS.SEQ_NUM GE 0
	IF FOUND_IN_EXPER THEN BEGIN
		PLAN = OBS
		GET_RASTER, PLAN.RAS_ID, PLAN.RAS_VAR, RASTER_DEF
		N_REPEATS = OBS.PROG_IND
		GOTO, FINISH
	ENDIF
;
;  If not, then check to see if one is still processing the same study as
;  previously.  If so, then the parameters should be consistent with where one
;  should be in that study.
;
	IF UNSIGN(RASTER.HEADER.SCI.STUDYCOUNTER) EQ CURR_STUDY THEN BEGIN
		PLAN = CURR_PLAN
		IF NOT MATCH_PLAN(RASTER, PLAN, RASTER_DEF) THEN BEGIN
			IF PLAN.STUDYVAR GE 0 THEN CDS_MESSAGE,		$
				'Previous match in science plan ' +	$
				'apparently incorrect', /ATTENTION, /CONTINUE
			CURR_PLAN = {STUDY_ID: -1}
			CURR_STUDY = -1
			N_REPEATS = -1
		END ELSE GOTO, FINISH
	ENDIF
;
;  Perhaps, this is simply a repeat of the same study.
;
	IF CURR_PLAN.STUDY_ID GE 0 THEN BEGIN
		IF TAG_EXIST(CURR_PLAN, 'N_REPEAT_S', /TOP_LEVEL) THEN	$
			N_REPEAT_S = CURR_PLAN.N_REPEAT_S ELSE N_REPEAT_S = 1
		IF TAG_EXIST(CURR_PLAN, 'N_POINTINGS', /TOP_LEVEL) THEN	$
			N_POINTINGS = CURR_PLAN.N_POINTINGS ELSE N_POINTINGS=1
		IF N_REPEATS LT (N_REPEAT_S*N_POINTINGS) THEN BEGIN
			IF MATCH_PLAN(RASTER, CURR_PLAN, RASTER_DEF) THEN BEGIN
				PLAN = CURR_PLAN
				N_REPEATS = N_REPEATS + 1
				GOTO, FINISH
			ENDIF
		ENDIF
;
;  If not the same study as before, perhaps it's the next study.  Search up to
;  one day for the next study in sequence.
;
		CURR_PLAN_DATE_END = CURR_PLAN.(GET_PLAN_ITIME(CURR_PLAN)+1)
		LIST_DETAIL, CURR_PLAN_DATE_END+0.001,	$
			CURR_PLAN_DATE_END+86400, LIST, N_FOUND
		IF N_FOUND GE 1 THEN BEGIN
			LIST_DATE_OBS = LIST.(GET_PLAN_ITIME(LIST))
			IF N_FOUND GT 1 THEN BEGIN
				S = SORT(LIST_DATE_OBS)
				LIST = LIST(S(0))
			END ELSE LIST = LIST(0)
			GET_DETAIL, LIST_DATE_OBS, OBS
			IF MATCH_PLAN(RASTER, OBS, RASTER_DEF) THEN BEGIN
				PLAN = OBS
				N_REPEATS = 1
				GOTO, FINISH
			ENDIF
		ENDIF
	ENDIF
;
;  From here on out, one is searching the planning databases.  The first pass
;  one searches the details database, and the second pass one searches the flag
;  database.
;
	DATABASE = 'DETAIL'
START_LOOKING:
;
;  Taking the date of the beginning of the observation, find the study in
;  science plan which best corresponds to that time.
;
	CALL_PROCEDURE, 'GET_'+DATABASE, OBT_TIME, OBS
	IF OBS.STUDY_ID GE 0 THEN BEGIN
		IF MATCH_PLAN(RASTER, OBS, RASTER_DEF) THEN BEGIN
			PLAN = OBS
			N_REPEATS = 1
			GOTO, FINISH
		ENDIF
	ENDIF
;
;  If the information doesn't match, then there may be a small mismatch in the
;  actual start time and the planned start time.  If the raster counter is
;  zero, then find the study with the closest start time within 12 hours of the
;  observation time.
;
	DT = 12.D0 * 60.D0^2
	N_FOUND = 0
	IF RASTER.HEADER.SCI.RASTERCOUNTER EQ 0 THEN BEGIN
		CALL_PROCEDURE,'LIST_'+DATABASE, OBT_TIME-DT, OBT_TIME+DT, $
			LIST, N_FOUND
		IF N_FOUND GT 0 THEN BEGIN
			LIST_DATE_OBS = LIST.(GET_PLAN_ITIME(LIST))
			DIFF = ABS(LIST_DATE_OBS - OBT_TIME)
			W = WHERE(DIFF EQ MIN(DIFF))
			CALL_PROCEDURE, 'GET_'+DATABASE,	$
				(LIST_DATE_OBS)(W(0)), OBS
			IF MATCH_PLAN(RASTER, OBS, RASTER_DEF) THEN BEGIN
				PLAN = OBS
				N_REPEATS = 1
				GOTO, FINISH
			ENDIF
		ENDIF
	ENDIF
;
;  If the information still doesn't match, then the previous study may not have
;  completed yet.  Check the telemetry against the previous study.
;
	IF N_FOUND GT 0 THEN BEGIN
		IF W(0) EQ 0 THEN BEGIN
			DT = 2*DT
			CALL_PROCEDURE,'LIST_'+DATABASE, OBT_TIME-DT,	$
				OBT_TIME+DT, LIST, N_FOUND
			LIST_DATE_OBS = LIST.(GET_PLAN_ITIME(LIST))
			DIFF = ABS(LIST_DATE_OBS - OBT_TIME)
			W = WHERE(DIFF EQ MIN(DIFF))
		ENDIF
		IF W(0) NE 0 THEN BEGIN
			CALL_PROCEDURE, 'GET_'+DATABASE,	$
				(LIST_DATE_OBS)(W(0)-1), OBS
			IF MATCH_PLAN(RASTER, OBS, RASTER_DEF) THEN BEGIN
				PLAN = OBS
				N_REPEATS = 1
				GOTO, FINISH
			ENDIF
		ENDIF
	ENDIF
;
;  Or, the previous study may have completed earlier than expected.  Check the
;  telemetry against the next study.
;
	IF N_FOUND GT 0 THEN BEGIN
		IF W(0) EQ (N_FOUND-1) THEN BEGIN
			DT = 2*DT
			CALL_PROCEDURE,'LIST_'+DATABASE, OBT_TIME-DT,	$
				OBT_TIME+DT, LIST, N_FOUND
			LIST_DATE_OBS = LIST.(GET_PLAN_ITIME(LIST))
			DIFF = ABS(LIST_DATE_OBS - OBT_TIME)
			W = WHERE(DIFF EQ MIN(DIFF))
		ENDIF
		IF W(0) NE (N_FOUND-1) THEN BEGIN
			CALL_PROCEDURE, 'GET_'+DATABASE,	$
				(LIST_DATE_OBS)(W(0)+1), OBS
			IF MATCH_PLAN(RASTER, OBS, RASTER_DEF) THEN BEGIN
				PLAN = OBS
				N_REPEATS = 1
				GOTO, FINISH
			ENDIF
		ENDIF
	ENDIF
;
;  If a match has not been found, then an alternate plan may have been run, or
;  the spacecraft may have reacted to a flag.  Reapply the above steps to the
;  alternate and flag plan.
;
	IF DATABASE EQ 'DETAIL' THEN BEGIN
		DATABASE = 'ALT'
		GOTO, START_LOOKING
	END ELSE IF DATABASE EQ 'ALT' THEN BEGIN
		DATABASE = 'FLAG'
		GOTO, START_LOOKING
	ENDIF
;
;  If no match can be found, then set all the database information to nulls,
;  and flag the error to the operator.
;
	PLAN = {STUDY_ID: RASTER.STUDY_ID,	$
		STUDYVAR: RASTER.STUDYVAR,	$
		DATE_OBS: OBT_TIME,	$
		DATE_END: OBT_END,	$
		PROG_ID: -1,		$
		SCI_OBJ: '',		$
		SCI_SPEC: '',		$
		CMP_NO: -1,		$
		OBJECT: '',		$
		OBJ_ID: '',		$
		TRACKING: 0,		$
		N_REPEAT_S: 1,		$
		N_POINTINGS: 1}
	N_REPEATS = 1
	CDS_MESSAGE, 'Unable to match raster to science plan', /ATTENTION, $
		/CONTINUE
FINISH:
;
;  If the raster does not yet have a SEQ_NUM value, then assign one by adding
;  one to the last number used.
;
	SEQ_NUM = -1
	IF FOUND_IN_EXPER THEN SEQ_NUM = PLAN.SEQ_NUM
	IF SEQ_NUM LE 0 THEN BEGIN
TRY_AGAIN:
		SQNUM_MAX = SQNUM_MAX + 1L
		IF N_ELEMENTS(SQNUM_UNIT) EQ 1 THEN BEGIN
		    STATUS = FSTAT(SQNUM_UNIT)
		    IF STATUS.OPEN THEN BEGIN
;
;  Close and reopen the file, and reread the counter value, in case it's been
;  written to since the last time the file was opened.
;
			CLOSE, SQNUM_UNIT
			OPENU, SQNUM_UNIT, STATUS.NAME
			TEMP = SQNUM_MAX
			READF, SQNUM_UNIT, TEMP
			SQNUM_MAX = SQNUM_MAX > (TEMP + 1L)
;
			POINT_LUN, SQNUM_UNIT, 0	;Beginning of file
			PRINTF, SQNUM_UNIT, SQNUM_MAX
;
;  Do a close and reopen to force a flush to the disk.
;
			CLOSE, SQNUM_UNIT
			OPENU, SQNUM_UNIT, STATUS.NAME
		    ENDIF
		ENDIF
;
;  Make sure that the sequence number is not already being used.
;
		GET_EXPER, SQNUM_MAX, TEMP
		IF TEMP.SEQ_NUM GE 0 THEN GOTO, TRY_AGAIN
;
;  Otherwise, add it to the plan structure.
;
		IF TAG_EXIST(PLAN, 'SEQ_NUM', /TOP_LEVEL) THEN	$
			PLAN.SEQ_NUM = SQNUM_MAX ELSE	$
			PLAN = ADD_TAG(PLAN, SQNUM_MAX, 'SEQ_NUM', /TOP_LEVEL)
	ENDIF
;
;  Make sure that N_REPEATS is also in the PLAN output structure under the
;  name PROG_IND.
;
	IF TAG_EXIST(PLAN, 'PROG_IND', /TOP_LEVEL) THEN		$
		PLAN.PROG_IND = N_REPEATS ELSE $
		PLAN = ADD_TAG(PLAN, N_REPEATS, 'PROG_IND', /TOP_LEVEL)
;
;  If possible, make sure that ZONE_ID is also in the PLAN output.
;
	IF TAG_EXIST(PLAN, 'POINTINGS', /TOP_LEVEL) THEN BEGIN
	    IF RASTER.INSTRUMENT EQ 'NIS' THEN ZONE_ID = 0 ELSE BEGIN
		I_POINT = (N_REPEATS > 0) MOD (PLAN.N_POINTINGS > 1)
		ZONE_ID = PLAN.POINTINGS(I_POINT).ZONE_ID
	    ENDELSE
	    IF TAG_EXIST(PLAN, 'ZONE_ID', /TOP_LEVEL) THEN	$
		    PLAN.ZONE_ID = ZONE_ID ELSE	$
		    PLAN = ADD_TAG(PLAN, ZONE_ID, 'ZONE_ID', /TOP_LEVEL)
	ENDIF
;
;  Finish up by storing the current plan in the common block.
;
	CURR_PLAN = PLAN
	CURR_STUDY = UNSIGN(RASTER.HEADER.SCI.STUDYCOUNTER)
;
;  Put the GSET_ID in the raster header.
;
	IF TAG_EXIST(PLAN, 'GSET_ID', /TOP_LEVEL) THEN	$
		GSET_ID = PLAN.GSET_ID ELSE GSET_ID = -1
	RASTER = ADD_TAG(RASTER, GSET_ID, 'GSET_ID', /TOP_LEVEL)
;
;  If something was found, then put that information back into the plan
;  database.
;
	IF TAG_EXIST(PLAN, 'STRUCT_TYPE') THEN BEGIN
	    IF PLAN.STRUCT_TYPE EQ 'CDS-DETAIL' THEN BEGIN
		MESSAGE = ''
		PROG_NUM = UNSIGN(RASTER.HEADER.SCI.STUDYCOUNTER)
		IF NOT MOD_DETAIL(PLAN, PROG_NUM=PROG_NUM, ERRMSG=MESSAGE) $
			THEN CDS_MESSAGE, MESSAGE, /CONTINUE
	    ENDIF
	ENDIF
;
	RETURN
	END
