	FUNCTION FIND_RASTER, TEL_MODE, UNIT, PACKET
;+
; Project     : SOHO - CDS
;
; Name        : FIND_RASTER()
;
; Purpose     : Finds the next raster header in the telemetry.
;
; Explanation : This routine searches through the CDS science telemetry stream
;		until it finds a GIS or NIS raster header packet.
;
; Use         : Result = FIND_RASTER( TEL_MODE, UNIT, PACKET )
;
; Inputs      : TEL_MODE = Input telemetry mode, as used by READ_PACKET.
;		UNIT	 = Unit number input was opened with.
;		PACKET	 = Current telemetry packet.
;
; Opt. Inputs : None.
;
; Outputs     : PACKET	 = First telemetry packet that contains a raster
;			   header.  This may be the same as the packet that was
;			   passed to this routine.
;
;		The result of the function is either true (1) if successful, or
;		false (0) if unsuccessful.
;
; Opt. Outputs: None.
;
; Keywords    : None.
;
; Calls       : GET_SCI_PKT
;
; Common      : Uses common block FITSGEN to keep track of the largest study
;		counter used.
;
; Restrictions: None.
;
; Side effects: If science telemetry packets (other than engineering packets,
;		etc.) are skipped, then a summary of the number of packets
;		skipped is written to the telemetry log.
;
; Category    : Data Handling, Telemetry.
;
; Prev. Hist. : Although there is an older routine with the same name, this
;		routine represents a complete rewrite.
;
; Written     : William Thompson, GSFC, 16 April 1993.
;
; Modified    : Version 1, William Thompson, GSFC, 16 April 1993
;		Version 2, William Thompson, GSFC, 3 March 1995
;			Modified to write out to file max_study_counter.dat if
;			open for write.
;		Version 3, William Thompson, GSFC, 23 June 1995
;			First make sure that first packet is a science packet.
;			Modified to accomodate change in READ_EXPOSURE
;		Version 4, William Thompson, GSFC, 3 July 1995
;			Signal error if STCNT_MAX jumps by more than 100
;		Version 5, William Thompson, GSFC, 1 November 1995
;			Add time to error messages.
;			Don't count raster abort packets as skipped packets.
;		Version 6, William Thompson, GSFC, 28 March 1996
;			Close and reopen the study counter file to force a
;			flush to the disk.
;		Version 7, William Thompson, GSFC, 12 September 1996
;			Treat STUDYCOUNTER as an unsigned integer.
;		Version 8, William Thompson, GSFC, 29 May 1997
;			Call STCNT_UPDATE
;
; Version     : Version 8, 29 May 1997
;-
;
	ON_ERROR, 2
@fitsgen.cmn
;
;  Check the number of parameters.
;
	IF N_PARAMS() NE 3 THEN MESSAGE,	$
		'Syntax:  Result = FIND_RASTER( TEL_MODE, UNIT, PACKET )'
;
;  First, make sure that we have a science packet.
;
	IF GET_SCI_PKT(TEL_MODE,UNIT,PACKET) NE 1 THEN RETURN,0
;
;  Keep reading packets until a raster header packet is found.  Don't count
;  raster abort packets as skipped packets.
;
	N_SKIPPED = 0
	WHILE (PACKET.HDR.ID NE '01'X) AND (PACKET.HDR.ID NE '02'X) DO BEGIN
		IF GET_SCI_PKT(TEL_MODE,UNIT,PACKET,/NEXT) NE 1 THEN RETURN, 0
		IF PACKET.HDR.ID NE '03'X THEN N_SKIPPED = N_SKIPPED + 1
	ENDWHILE
;
;  If packets were skipped, then write a message to the log file about it.
;
	IF N_SKIPPED GT 0 THEN CDS_MESSAGE, /CONTINUE, 'FIND_RASTER:  ' + $
		'Skipped total of ' + STRTRIM(N_SKIPPED,2) + ' records', $
		/ADD_TIME
;
;  If the file max_study_counter.dat is open, and the current study counter is
;  larger than the value in the file, then replace the value in the file with
;  the current value.
;
	IF N_ELEMENTS(STCNT_UNIT) EQ 1 THEN BEGIN
	    STATUS = FSTAT(STCNT_UNIT)
	    IF STATUS.OPEN THEN BEGIN
		STUDYCOUNTER = UNSIGN(PACKET.PKT.SCI.STUDYCOUNTER)
		STCNT_UPDATE, STCNT_UNIT, STCNT_MAX, STUDYCOUNTER
	    ENDIF
	ENDIF
;
;  Signal successful conclusion.
;
	RETURN, 1
	END
