	PRO FIND_TM_FILES, TSTART, TEND, FILES, START_TIMES, END_TIMES, $
		REPORT=REPORT, N_FOUND=N_FOUND, ERRMSG=ERRMSG
;+
; Project     :	SOHO - CDS
;
; Name        :	FIND_TM_FILES
;
; Purpose     :	Find all telemetry files within a given date range
;
; Category    :	Class4, CDS, Operations, Telemetry
;
; Explanation :	Searches the directories CDS_TM_DATA, CDS_QKLSCI_DATA and
;		CDS_RELSCI_DATA for files which cover a given date range.  The
;		filenames are returned sorted in time order.
;
; Syntax      :	FIND_TM_FILES, TSTART, TEND, FILES  [, START_TIMES, END_TIMES ]
;
; Examples    :	FIND_TM_FILES, '1996/05/12 02:55', '1996/05/12 11:28', FILES
;
; Inputs      :	TSTART	    = Start of time interval to search for.
;		TEND	    = End of time interval to search for.
;
; Opt. Inputs :	None.
;
; Outputs     :	FILES	    = The names of the files returned.
;
; Opt. Outputs:	START_TIMES = The start times corresponding to FILES
;		END_TIMES   = The end times corresponding to FILES
;
; Keywords    :	N_FOUND	= Returns the number of telemetry files found.
;
;		REPORT	= If set, then a report on the files found is printed
;			  to the terminal.
;
;		ERRMSG  = If defined and passed, then any error messages will
;			  be returned to the user in this parameter rather than
;			  depending on the MESSAGE routine in IDL.  If no
;			  errors are encountered, then a null string is
;			  returned.  In order to use this feature, ERRMSG must
;			  be defined first, e.g.
;
;				ERRMSG = ''
;				FIND_TM_FILES, ERRMSG=ERRMSG, ... 
;				IF ERRMSG NE '' THEN ...
;
; Calls       :	ANYTIM2TAI, FIND_FILES, BREAK_FILE, UTC2TAI, OBT2TAI
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Prev. Hist. :	None.
;
; History     :	Version 1, 22-May-1996, William Thompson, GSFC
;		Version 2, 20-Sep-1996, William Thompson, GSFC
;			Handle cases where quicklook or realtime directories
;			are empty
;
; Contact     :	WTHOMPSON
;-
;
	ON_ERROR, 2
;
;  Check the number of parameters.
;
	IF N_PARAMS() LT 3 THEN BEGIN
		MESSAGE = 'Syntax:  FIND_TM_FILES, TSTART, TEND, FILES'
		GOTO, HANDLE_ERROR
	ENDIF
;
;  Convert the input parameters to TAI format.
;
	START_TIME = ANYTIM2TAI(TSTART)
	END_TIME   = ANYTIM2TAI(TEND)
;
;  List all the files in CDS_TM_DATA, and extract those which are within the
;  requested time range.  Use one hour margins on either side to be safe.
;
	N_FOUND = 0
	TM_FILES = FIND_FILES('tm.*', '$CDS_TM_DATA')
	IF TM_FILES(0) EQ '' THEN COUNT = 0 ELSE BEGIN
	    BREAK_FILE, TM_FILES, DISK, DIR, NAME, EXT
	    DATE = UTC2TAI(STRMID(EXT,1,2) + '-' + STRMID(EXT,4,3) + ' ' + $
		    STRMID(EXT,8,2))
	    W = WHERE((DATE GE (START_TIME - 3600.D0)) AND	$
		    (DATE LE (END_TIME + 3600.D0)), COUNT)
	ENDELSE
	IF COUNT GT 0 THEN BEGIN
;
;  Open each tm file in turn, and read the first and last packet.  Extract the
;  start and stop times.
;
	    FOR I_FILE = 0,COUNT-1 DO BEGIN
		OPENR, UNIT, TM_FILES(W(I_FILE)), /GET_LUN
		B = BYTARR(306)
		READU, UNIT, B
		TM_START = OBT2TAI(B(12:17))
		STAT = FSTAT(UNIT)
		N_PACK = STAT.SIZE / 306
		POINT_LUN, UNIT, (N_PACK-1)*306
		READU, UNIT, B
		TM_END = OBT2TAI(B(12:17))
		FREE_LUN, UNIT
		IF (TM_START LE END_TIME) AND (TM_END GE START_TIME) THEN BEGIN
		    IF N_FOUND EQ 0 THEN BEGIN
			N_FOUND = 1
			FILES       = TM_FILES(W(I_FILE))
			START_TIMES = TM_START
			END_TIMES   = TM_END
		    END ELSE BEGIN
			N_FOUND = N_FOUND + 1
			FILES       = [FILES,       TM_FILES(W(I_FILE))]
			START_TIMES = [START_TIMES, TM_START]
			END_TIMES   = [END_TIMES,   TM_END]
		    ENDELSE
		ENDIF
	    ENDFOR
	ENDIF
;
;  List all the files in CDS_QKLSCI_DATA and CDS_RELSCI_DATA, and extract those
;  which are within the requested time range.  Use a one day margin at the
;  beginning to be safe.
;
	PATH = '$CDS_QKLSCI_DATA'
	EXT = '.QKL'
	FOR I_PATH = 0,1 DO BEGIN
	    TM_FILES = FIND_FILES('CDS*'+EXT, PATH)
	    IF TM_FILES(0) EQ '' THEN TM_FILES='CDSSCM_000000_000000.REL'
	    BREAK_FILE, TM_FILES, DISK, DIR, NAME
	    DATE = UTC2TAI(STRMID(NAME,7,2) + '-' + STRMID(NAME,9,2) + '-' + $
		    STRMID(NAME,11,2) + ' ' + STRMID(NAME,14,2) + ':' +	$
		    STRMID(NAME,16,2) + ':' + STRMID(NAME,18,2))
	    W = WHERE((DATE GE (START_TIME - 86400.D0)) AND	$
		    (DATE LE END_TIME), COUNT)
;
;  Read in the headers and extract the start and end times.
;
	    IF COUNT GT 0 THEN FOR I_FILE = 0,COUNT-1 DO BEGIN
		OPENR, UNIT, TM_FILES(W(I_FILE)), /GET_LUN
		LINE = 'String'
		WHILE STRTRIM(LINE,2) NE 'END' DO BEGIN
		    READF, UNIT, LINE
		    IF STRMID(LINE,0,8) EQ 'STARTIME' THEN	$
			    TM_START=ANYTIM2TAI(STRMID(LINE,9,STRLEN(LINE)-9))
		    IF STRMID(LINE,0,7) EQ 'ENDTIME' THEN	$
			    TM_END  =ANYTIM2TAI(STRMID(LINE,9,STRLEN(LINE)-9))
		ENDWHILE
		FREE_LUN, UNIT
		IF (TM_START LE END_TIME) AND (TM_END GE START_TIME) THEN BEGIN
		    IF N_FOUND EQ 0 THEN BEGIN
			N_FOUND = 1
			FILES       = TM_FILES(W(I_FILE))
			START_TIMES = TM_START
			END_TIMES   = TM_END
		    END ELSE BEGIN
			N_FOUND = N_FOUND + 1
			FILES       = [FILES,       TM_FILES(W(I_FILE))]
			START_TIMES = [START_TIMES, TM_START]
			END_TIMES   = [END_TIMES,   TM_END]
		    ENDELSE
		ENDIF
	    ENDFOR
;
;  On the second iteration, look in the REL directory.
;
	    PATH = '$CDS_RELSCI_DATA'
	    EXT = '.REL'
	ENDFOR
;
;  Sort the files by start times.
;
	IF N_FOUND GT 1 THEN BEGIN
		S = SORT(START_TIMES)
		FILES       = FILES(S)
		START_TIMES = START_TIMES(S)
		END_TIMES   = END_TIMES(S)
	ENDIF
;
;  If /REPORT was requested, the write out a report on the files.
;
	IF KEYWORD_SET(REPORT) THEN BEGIN
	    PRINT, ''
	    PRINT, FORMAT="('Filename',T30,'Start time',11X,'End time')"
	    PRINT, ''
	    FOR I=0,N_FOUND-1 DO BEGIN
		BREAK_FILE, FILES(I), DISK, DIR, NAME, EXT
		PRINT, NAME+EXT, TAI2UTC(START_TIMES(I),/ECS,/TRUNCATE), $
			TAI2UTC(END_TIMES(I),/ECS,/TRUNCATE),		$
			FORMAT='(A,T30,A,2X,A)'
	    ENDFOR
	    PRINT, ''
	ENDIF
;
	GOTO, FINISH
;
;  Error handling point.
;
HANDLE_ERROR:
	IF N_ELEMENTS(ERRMSG) NE 0 THEN ERRMSG = 'FIND_TM_FILES: ' + MESSAGE $
		ELSE MESSAGE, MESSAGE, /CONTINUE
;
;  Exit point.
;
FINISH:
	RETURN
	END
