;+
; Project     :	SOHO - CDS     
;
; Name        :	FITSGEN
;
; Purpose     :	Widget interface to FITS generation from telemetry
;
; Explanation :	This procedure creates a widget that controls the process to
;		generate FITS files from the telemetry.  From this widget one
;		can select to process real-time telemetry, or replay stored
;		telemetry.
;
; Use         :	FITSGEN
;
; Inputs      :	None
;
; Opt. Inputs :	None
;
; Outputs     :	None
;
; Opt. Outputs:	None
;
; Keywords    : None.
;
; Env. vars.  :	CDS_TM_LOG defines where the log file will be written.  If not
;		defined, then the log file is written in the current directory.
;		The filename has the form "YYYYMMDD_HHMM.tmlog" based on the
;		current (UTC) date and time.  A new log file is created each
;		time processing is started.
;
; Calls       :	CONCAT_DIR, FIX_ZDBASE, PRIV_ZDBASE, DELVARX
;
; Common      : FITSGEN from the include file "fitsgen.cmn"
;		FIND_PLAN_CMN, shared with routine FIND_IN_PLAN
;
; Restrictions:	None.
;
; Side effects:	None
;
; Category    :	Data handling, Telemetry
;
; Prev. Hist. :	None, but heavily influenced by STM_MENU by Arnulf and C D
;		Pike.
;
; Written     :	William Thompson, GSFC, 31 August 1994
;
; Modified    :	Version 1, William Thompson, GSFC, 3 February 1995
;		Version 2, William Thompson, GSFC, 3 March 1995
;			Modified to open and close max_study_counter.dat file.
;		Version 3, William Thompson, GSFC, 21 April 1995
;			Modified to include a stop button.
;		Version 4, William Thompson, GSFC, 3 August 1995
;			Modified to open and close max_seq_num.dat file.
;			Make sure that both .dat files are read in as integers.
;		Version 5, William Thompson, GSFC, 21 August 1995
;			Added common block FIND_PLAN_CMN so that FIND_IN_PLAN
;			would not be confused by any previous calls to FITSGEN
;			in the same IDL session.
;			Added calls to FIX_ZDBASE and PRIV_ZDBASE.
;		Version 6, William Thompson, GSFC, 18 October 1995
;			Changed CDS_CP_MISC to CDS_CP_MISC_W.  Added check if
;			environment variable is not defined.
;               Version 7, C D Pike, RAL, 20_Oct-95
;                       Corrected typo in environment variable. 
;		Version 8, William Thompson, GSFC, 16 November 1995
;			Separated START and STOP buttons.
;			Made message widget bigger.
;		Version 9, William Thompson, GSFC, 21 December 1995
;			Added autostop button
;		Version 10, William Thompson, GSFC, 6 March 1996
;			Fixed call to PRIV_ZDBASE to test for /CATALOG rather
;			than /DAILY
;		Version 11, William Thompson, GSFC, 11 March 1996
;			Added widgets for controlling CDS_OPERATOR, and for
;			start and end study counter values.
;		Version 12, William Thompson, GSFC, 12 March 1996
;			Added widgets for controlling the start and end raster
;			index values.
;		Version 13, William Thompson, GSFC, 28 March 1996
;			Close and reopen the study counter and sequence number
;			files to force a flush to the disk.
;		Version 14, William Thompson, GSFC, 10 April 1996
;			Made some of the text widgets bigger
;		Version 15, William Thompson, GSFC, 30 April 1996
;			Make sure that FINALDIST starts out disabled.
;		Version 16, William Thompson, GSFC, 2 May 1996
;			Use /BELL instead of /ATTENTION
;		Version 17, William Thompson, GSFC, 9 September 1996
;			Increased width of message window
;		Version 18, William Thompson, GSFC, 29-May-1997
;			Modified to call STCNT_OPEN
;		Version 19, William Thompson, GSFC, 13 June 1997
;			Make sure that SQNUM_MAX is a long integer
;
; Version     : Version 19, 13 June 1997
;-            
; 
	PRO FITSGEN
;
;  Define the error return status, and include the common block.
;
	ON_ERROR, 2
	COMMON FIND_PLAN_CMN, CURR_PLAN, CURR_STUDY, N_REPEATS
@fitsgen.cmn
;
;  If already running, then return immediately.
;
	IF XREGISTERED('fitsgen') THEN RETURN
;
;  Make sure that FIND_IN_PLAN is running with a clean slate.
;
	CURR_PLAN = {STUDY_ID: -1}
	CURR_STUDY = -1
	N_REPEATS = -1
;
;  Test to see if the CDS_CP_MISC_W directory exists.  If not, then undefine
;  STCNT_UNIT and SQNUM_UNIT.
;
	IF GETENV('CDS_CP_MISC_W') EQ '' THEN BEGIN
	    DELVARX, STCNT_UNIT  &  STCNT_MAX = 0L
	    DELVARX, SQNUM_UNIT  &  SQNUM_MAX = 0L
	END ELSE BEGIN
;
;  Open the file to keep track of the highest study counter entry, and read in
;  the current value.
;
	    STCNT_OPEN, STCNT_UNIT, STCNT_MAX
;
;  Open the file to keep track of the highest sequence number entry, and read
;  in the current value.  If the file doesn't exist yet, then initialize it to
;  contain the value 0.
;
	    SQNUM_FILE = CONCAT_DIR('$CDS_CP_MISC_W','max_seq_num.dat')
	    SQNUM_MAX = 0L
	    OPENU, SQNUM_UNIT, SQNUM_FILE, /GET_LUN, ERROR=ERROR
	    IF ERROR NE 0 THEN BEGIN
		OPENW, SQNUM_UNIT, SQNUM_FILE, /GET_LUN
		PRINTF, SQNUM_UNIT, SQNUM_MAX
;
;  Do a close and reopen to force a flush to the disk.
;
		CLOSE, SQNUM_UNIT
		OPENU, SQNUM_UNIT, SQNUM_FILE
	    END ELSE READF, SQNUM_UNIT, SQNUM_MAX
	ENDELSE
;
;  Initialize some variables.  Make sure that the packet structures are
;  defined.
;
	SCI_MODE = 1
	TM_DEFINE_STRUC
;
;  Define the main widget.  Give it the user value of "Timer" to give the timer
;  program something to hook on to.
;
	MAIN_BASE = WIDGET_BASE(TITLE='CDS FITS Generation Software',	$
		/FRAME, /COLUMN, UVALUE='Timer')
;
;  Define a base that will contain everything but the message window.
;
	TOP_BASE = WIDGET_BASE(MAIN_BASE, /COLUMN)
;
;  Put in an informational title.
;
	TITLE_WIDG = WIDGET_TEXT(TOP_BASE, XSIZE=30, YSIZE=1)
;
;  Create a menu base for the buttons.
;
	DUMMY = WIDGET_BASE(TOP_BASE, /ROW)
;
;  Create the "Start" button and "Stop" buttons.
;
	START_BUTTON = WIDGET_BUTTON(DUMMY, VALUE='Start', UVALUE='Start')
	STOP_BUTTON = WIDGET_BUTTON(DUMMY, VALUE='Stop', UVALUE='Stop')
;
;  Create a menu base for the rest of the buttons.  This way, these can be
;  sensitized and desensitized as a unit.
;
	MENU_BASE = WIDGET_BASE (DUMMY, /ROW)
;
;  Create the exit "Done" button.
;
	DUMMY = WIDGET_BUTTON(MENU_BASE, VALUE='EXIT', UVALUE='Done')
;
;  Create the "Mode" button.
;
	XPDMENU, ['"Mode" {',					$
		  '"Real time"		Realtime',		$
		  '"Replay (TM file)"	Replay',	'}'],	$
		  MENU_BASE, BASE=MODE_BASE
;
;  Create the file selection button.  This needs to be in a base of its own, so
;  that it can be mapped and unmapped.
;
	PBASE = LONARR(2)
	DUMMY = WIDGET_BASE(MENU_BASE)
	PBASE(0) = WIDGET_BASE(DUMMY,MAP=0)
	PBASE(1) = WIDGET_BASE(DUMMY,MAP=0,/ROW)
	DUMMY = WIDGET_BUTTON(PBASE(1), VALUE='Select TM file',	$
			UVALUE='Get_TM')
;
;  The "Limited/Unlimited Run" menu shares the same base as the file selection
;  button, so it to can be mapped and unmapped.
;
	XMENU,['Limited Run','Unlimited Run'],PBASE(1),/COLUMN,/EXCLUSIVE, $
		/FRAME,/NO_RELEASE,UVALUE=['Limited','Unlimited'],	$
		BUTTONS=RUN_BUTTONS
	RUN_STATE = 0
	WIDGET_CONTROL, RUN_BUTTONS(RUN_STATE), SET_BUTTON=1
;
;  Create the "Help" button.
;
	DUMMY = WIDGET_BUTTON(MENU_BASE, VALUE='Help', UVALUE='Help')
;
;  Allow the user to modify the value of CDS_OPERATOR.
;
	DUMMY = CW_FIELD(TOP_BASE, VALUE=GETENV('CDS_OPERATOR'), /ROW,	$
		/STRING, /ALL_EVENTS, TITLE='CDS Operator',		$
		UVALUE='Operator', XSIZE=50)
;
;  Define a menu base for the second layer.
;
	PACKET_BASE= WIDGET_BASE(TOP_BASE,/COLUMN)
;
;  Make a display for the packet number.
;
	PKT_BASE = WIDGET_BASE(PACKET_BASE,/ROW)
	DUMMY = WIDGET_BASE(PKT_BASE, /ROW)
	DUMMY_WIDG =  WIDGET_LABEL(DUMMY, VALUE='Packet number:')
	PACKET_WIDG = WIDGET_TEXT(DUMMY, XSIZE = 6, YSIZE=1)
;
;  Make a text widget to display the current log file.
;
	DUMMY = WIDGET_LABEL(PKT_BASE, VALUE='Log file:')
	LOG_WIDG = WIDGET_TEXT(PKT_BASE, XSIZE=30, YSIZE=1)
;
;  Make another base below this, with a blank level for when in real time.
;
	PFBASE = LONARR(2)
	DUMMY = WIDGET_BASE(PACKET_BASE)
	PFBASE(0) = WIDGET_BASE(DUMMY,MAP=0)
	PFBASE(1) = WIDGET_BASE(DUMMY,MAP=0,/COLUMN)
	ROW_BASE = WIDGET_BASE(PFBASE(1),/ROW)
;
;  Make a text widget to display the current telemetry file, and another to set
;  the start packet.
;
	DUMMY = WIDGET_LABEL(ROW_BASE, VALUE='Current file: ')
	REP_WIDG = WIDGET_TEXT(ROW_BASE, XSIZE=28, YSIZE=1)
	WIDGET_CONTROL, REP_WIDG, SET_VAL='   None'
;
	SP_WIDG      = WIDGET_LABEL(ROW_BASE, VALUE='Start Pkt: ')
	SPACKET_WIDG = WIDGET_TEXT(ROW_BASE, XSIZE=6, YSIZE=1, /EDITABLE)
	WIDGET_CONTROL,SPACKET_WIDG,SET_VAL='0'
;
;  Make widgets to select a start and stop study, and raster index.
;
	COL_BASE = WIDGET_BASE(PFBASE(1),/COLUMN,/FRAME)
	ROW_BASE = WIDGET_BASE(COL_BASE,/ROW)
	DUMMY = CW_FIELD(ROW_BASE, VALUE=0, /ROW, /INTEGER, /ALL_EVENTS, $
		TITLE='Start study', UVALUE='StartCnt')
	DUMMY = CW_FIELD(ROW_BASE, VALUE=-1, /ROW, /INTEGER, /ALL_EVENTS, $
		TITLE='Start index', UVALUE='StartInd')
;
	ROW_BASE = WIDGET_BASE(COL_BASE,/ROW)
	DUMMY = CW_FIELD(ROW_BASE, VALUE=0, /ROW, /INTEGER, /ALL_EVENTS, $
		TITLE='End study', UVALUE='EndCnt')
	DUMMY = CW_FIELD(ROW_BASE, VALUE=-1, /ROW, /INTEGER, /ALL_EVENTS, $
		TITLE='End index', UVALUE='EndInd')
	START_CNT = 0
	START_IND = -1
	END_CNT = 0
	END_IND = -1
;
;  Make a button group to select automatic stop.
;
	DUMMY = CW_BGROUP(PFBASE(1), ['Generate stop on mode change'], ROW=1, $
		/NONEXCLUSIVE, UVALUE='Autostop')
;
;  Make a button group to select final distribution.
;
	TEXT = ['Processing final distribution (CDROMs)']
	DUMMY = CW_BGROUP(PFBASE(1), TEXT, ROW=1, /NONEXCLUSIVE,	$
		UVALUE='FinalDist')
	FINALDIST = 0
;
;  Make the message window widget.
;
	DUMMY = WIDGET_BASE(MAIN_BASE, /COLUMN, /FRAME, YOFFSET=-100)
	DUMMY_WIDG = WIDGET_LABEL(DUMMY, VALUE=' Messages   : ')
	MSG_WIDG = WIDGET_TEXT(DUMMY, XSIZE=70, YSIZE=20, /SCROLL)
;
;  Realise the main widget to make it all happen.
;
	WIDGET_CONTROL, /REALIZE , MAIN_BASE
	WIDGET_CONTROL, MAIN_BASE, /CLEAR_EVENT
	WIDGET_CONTROL, STOP_BUTTON, SENSITIVE=0
;
;  Select the official CDS database.
;
	IF NOT FIX_ZDBASE(/CDS) THEN BEGIN
		CDS_MESSAGE, /BELL, /CONTINUE,	$
			'*** Unable to select official CDS database'
		!PRIV = 0
	END ELSE CDS_MESSAGE, /CONTINUE, 'Using official CDS database'
;
;  Determine whether or not one has privilege to write into the database.
;
	IF NOT PRIV_ZDBASE(/CATALOG) THEN BEGIN
		CDS_MESSAGE, /BELL, /CONTINUE,	$
			'*** Unable to write into CDS database'
		!PRIV = 0
	END ELSE !PRIV = 3
;
;  If the "_W" environment variables are not defined, then signal this to the
;  operator.
;
	IF GETENV('CDS_OPERATOR') EQ '' THEN CDS_MESSAGE, /BELL,	$
		/CONTINUE, '*** CDS_OPERATOR not defined'
	IF GETENV('CDS_CP_MISC_W') EQ '' THEN CDS_MESSAGE, /BELL,	$
		/CONTINUE, ['*** CDS_CP_MISC_W not defined',	$
		'Unable to access sequence and study counter files']
	IF GETENV('CDS_TM_LOG_W') EQ '' THEN CDS_MESSAGE, /BELL,	$
		/CONTINUE, ['*** CDS_TM_LOG_W not defined.',	$
		'Log files will be written to current directory']
	IF GETENV('CDS_FITS_DATA_W') EQ '' THEN CDS_MESSAGE, /BELL, $
		/CONTINUE, ['*** CDS_FITS_DATA_W not defined.',	$
		'FITS files will be written to current directory']
	IF GETENV('CDS_FITS_QLI_W') EQ '' THEN CDS_MESSAGE, /BELL,	$
		/CONTINUE, ['*** CDS_FITS_QLI_W not defined.',	$
		'FITS QLI files will be written to current directory']
;
;  Set it all going.
;
	XMANAGER, 'fitsgen', MAIN_BASE
;
;  Clean up.  Close the study counter and sequence number files.  Delete the
;  unit number parameters to signal that they are closed.
;
	IF N_ELEMENTS(STCNT_UNIT) EQ 1 THEN FREE_LUN, STCNT_UNIT
	IF N_ELEMENTS(SQNUM_UNIT) EQ 1 THEN FREE_LUN, SQNUM_UNIT
	DELVARX, STCNT_UNIT
	DELVARX, SQNUM_UNIT
;
	END
