;+
; Project     :	SOHO - CDS     
;
; Name        :	FITSGEN_EVENT
;
; Purpose     :	Widget interface to FITS generation from telemetry
;
; Explanation :	This procedure creates a widget that controls the process to
;		generate FITS files from the telemetry.  From this widget one
;		can choose to process real-time telemetry, or reprocess stored
;		real-time telemetry.
;
; Use         :	Called from XMANAGER
;
; Inputs      :	None
;
; Opt. Inputs :	None
;
; Outputs     :	None
;
; Opt. Outputs:	None
;
; Keywords    :	None
;
; Env. vars.  :	CDS_TM_LOG_W defines where the log file will be written.  If
;		not defined, then the log file is written in the current
;		directory.  The filename has the form "YYYYMMDD_HHMM.tmlog"
;		based on the current (UTC) date and time.  A new log file is
;		created each time processing is started.
;
;		CDS_FITS_DATA_W defines where the FITS files will be written.
;		If not defined, then the FITS files are written in the current
;		directory.  If it is defined, then CDS_FITS_DATA is searched
;		first to make sure that the file doesn't already exist, in
;		which case it would be overwritten.
;
; Calls       :	BREAK_FILE, CDS_MESSAGE, TRIM, MAKECDSFITS, FLASH_MSG,
;		OPEN_TM_FILE, GET_SCI_PKT, READ_RASTER
;
; Common      :	FITSGEN from the include file "fitsgen.cmn".
;
; Restrictions:	None
;
; Side effects:	None
;
; Category    :	Data handling, Telemetry
;
; Prev. Hist. :	None, but heavily influenced by STM_EVENT by Arnulf and C D
;		Pike.
;
; Written     :	William Thompson, GSFC, 31 August 1994
;
; Modified    :	Version 1, William Thompson, GSFC, 3 February 1995
;		Version 2, William Thompson, GSFC, 28 March 1995
;			Make sure that SKIP_PACK, I_PACK and NUM_PACK are
;			always long.
;		Version 3, William Thompson, GSFC, 21 April 1995
;			Modified to include a stop button.
;			FITS files written to CDS_FITS_DATA directory if
;			exists.
;		Version 4, William Thompson, GSFC, 24 April 1995
;			Fixed bug where no science packets found in telemetry
;		Version 5, William Thompson, GSFC, 18 May 1995
;			Fixed bug when event has no user value.
;		Version 6, William Thompson, GSFC, 26 June 1995
;			Fixed bug with packet counter not resetting to zero.
;		Version 7, William Thompson, GSFC, 18 October 1995
;			Changed environment variables to CDS_FITS_DATA_W and
;			CDS_TM_LOG_W.
;		Version 8, William Thompson, GSFC, 16 November 1995
;			Separated START and STOP buttons.
;		Version 9, William Thompson, GSFC, 21 December 1995
;			Added autostop button
;		Version 10, William Thompson, GSFC, 5 March 1996
;			Call SEL_TM_FILES instead of CDS_FILES_IN
;		Version 11, William Thompson, GSFC, 11 March 1996
;			Added support for the widgets for controlling
;			CDS_OPERATOR, and for start and end study counter
;			values.
;		Version 12, William Thompson, GSFC, 18 March 1996
;			If processing CDROM files, and have reached the end of
;			telemetry before the end of the raster, then don't
;			write out the FITS file.
;		Version 13, William Thompson, GSFC, 30 April 1996
;			If processing CDROM files, then also call EMON_MECH_LOG
;			and EMON_RELAYS_LOG.
;		Version 14, William Thompson, GSFC, 2 May 1996
;			Use /BELL instead of /ATTENTION in some places
;		Version 15, William Thompson, GSFC, 20 May 1996
;			Call CDS_MESSAGE with /CONTINUE so attention message is
;			not generated.
;		Version 16, William Thompson, GSFC, 13 June 1996
;			Automatically call MERGE_FITS_DATA
;		Version 17, William Thompson, GSFC, 26 July 1996
;			Call TLM_SCIENCE
;		Version 18, William Thompson, GSFC, 12 September 1996
;			Treat STUDYCOUNTER as an unsigned integer.
;		Version 19, William Thompson, GSFC, 21 November 1996
;			Added call to REGEN_SYNOP
;		Version 20, William Thompson, GSFC, 13 December 1996
;			Overwrite files if they already exist.
;			Remove call to MERGE_FITS_DATA.
;		Version 21, William Thompson, GSFC, 29 May 1997
;			Read file $CDS_CP_MISC/study_counter_anomaly.dat
;		Version 22, William Thompson, GSFC, 16 March 1998
;			Delete packet variable before calling GET_SCI_PKT.
;			Needed for modified time test in GET_SCI_PKT.
;
; Version     : Version 22, 16 March 1998
;-
;
	PRO FITSGEN_EVENT, EV
;
;  Define the error return status, and include the common block.
;
	IF !DEBUG EQ 0 THEN ON_ERROR, 2 ELSE ON_ERROR, 0
@fitsgen.cmn
;
;  Get the user value for the widget event (careful it is called plain
;  'value'!).
;
	WIDGET_CONTROL, EV.ID, GET_UVALUE=VALUE
	IF N_ELEMENTS(VALUE) EQ 0 THEN VALUE = ''
;
;  Begin to act depending on the identification of the widget.  If a top level
;  DONE was requested, everything dies.  Set PACKET_WIDG and MSG_WIDG to 0 to
;  signal that they don't exist anymore.  Make sure the log file is closed, and
;  set its unit number to zero to signal that it is closed.
;
	IF (VALUE EQ 'Done') THEN BEGIN
	    ON_IOERROR, GOING
	    IF N_ELEMENTS(TELEM_UNIT) GT 0 THEN FREE_LUN, TELEM_UNIT
GOING:	    WIDGET_CONTROL, /DESTROY, EV.TOP
	    MSG_WIDG = 0
	    PACKET_WIDG = 0
	    IF KEYWORD_SET(LOG_UNIT) THEN FREE_LUN, LOG_UNIT
	    LOG_UNIT = 0
	ENDIF
;
;  Request was to START receipt of telemetry.  First, form the name of the log
;  file.  If the environment variable CDS_TM_LOG_W is defined, then write the
;  log file into that directory, otherwise write it into the current directory.
;  The log filename is formed from the current (UTC) time as
;  'YYYYMMDD_HHMM.tmlog".
;
	IF (VALUE EQ 'Start') THEN BEGIN
	    GET_UTC,UTC,/EXTERNAL
	    LOG_FILE_NAME = TRIM(UTC.YEAR) +		$
	    	    STRMID(TRIM(100+UTC.MONTH),1,2) +	$
		    STRMID(TRIM(100+UTC.DAY),1,2) + '_' +	$
		    STRMID(TRIM(100+UTC.HOUR),1,2) +	$
		    STRMID(TRIM(100+UTC.MINUTE),1,2) + '.tmlog'
	    LOG_FILE = CONCAT_DIR(GETENV('CDS_TM_LOG_W'), LOG_FILE_NAME)
;
;  Actions are different for different ingest modes--here REPLAY mode.  (I.e.
;  any mode other than realtime.)
;
	    IF (SCI_MODE NE 1) THEN BEGIN
;
;  Start replay mode was requested so we need some data files.  Check that they
;  are defined.  If so, open the first.
;
		FILE_SEQ_NO = 0
		NUM_DATA_FILES = N_ELEMENTS(FILE_NAME)
		IF NUM_DATA_FILES GT 0 THEN BEGIN
		    TITLE = "Reprocessing stored (realtime) telemetry"
		    BREAK_FILE,FILE_NAME(FILE_SEQ_NO),DISK,DIR,RFILE,REXT
		    WIDGET_CONTROL,REP_WIDG,SET_VALUE=RFILE+REXT
		    GET_LUN, TELEM_UNIT
		    IF NOT OPEN_TM_FILE(TELEM_UNIT, FILE_NAME(FILE_SEQ_NO), $
			    SCI_MODE) THEN RETURN
;
;  Open the log file, and initialize the message window.
;
		    IF KEYWORD_SET(LOG_UNIT) THEN FREE_LUN, LOG_UNIT
		    OPENW, LOG_UNIT, LOG_FILE, /GET_LUN
		    PRINTF, LOG_UNIT, 'FITSGEN report file ' + LOG_FILE
		    WIDGET_CONTROL, LOG_WIDG, SET_VALUE=LOG_FILE_NAME
		    WIDGET_CONTROL, MSG_WIDG, SET_VALUE=''
;
		    CDS_MESSAGE, /CONTINUE,	$
			    'Opened file ' + FILE_NAME(FILE_SEQ_NO)
		ENDIF ELSE BEGIN
;
;  If no data files were defined, then warn and exit.
;
		    WIDGET_CONTROL, MSG_WIDG, /APPEND, $
			    SET_VALUE='No data files defined'
		    RETURN
		ENDELSE
;
;  Were we meant to skip to a particular packet?
;
		WIDGET_CONTROL,SPACKET_WIDG,GET_VALUE=SKIP_PACK
		IF VALID_NUM(SKIP_PACK(0),/INTEGER) THEN BEGIN
		    CDS_MESSAGE, 'Skipping to packet '+SKIP_PACK(0), /CONTINUE
;
;  Get the current position within the file, and set the position relative to
;  that.  Depending on the file format, the first packet may not be at the
;  beginning of the file.  The size of a packet depends on what the telemetry
;  mode is.
;
		    POINT_LUN, -TELEM_UNIT, CURR_POINT
		    IF SCI_MODE LE 1 THEN PACKETLENGTH = 306 ELSE	$
			    PACKETLENGTH = 300
		    POINT_LUN, TELEM_UNIT, CURR_POINT +	$
			    LONG(SKIP_PACK(0)) * PACKETLENGTH
		ENDIF ELSE BEGIN
		    BELL
		    WIDGET_CONTROL,MSG_WIDG, /APPEND, SET_VALUE=	$
			    '** Error - invalid packet skip requested. **'
		    SKIP_PACK = 0L
		    WIDGET_CONTROL,SPACKET_WIDG,SET_VALUE='0'
		ENDELSE
		I_PACK = LONG(SKIP_PACK(0))
	    ENDIF ELSE BEGIN
;
;  REALTIME mode selected so open the current realtime file.
;
		CURR_FILE = CONCAT_DIR('$CDS_TM_DATA', GT_RT_FILE())
		IF NOT FILE_EXIST(CURR_FILE) THEN BEGIN
		    FLASH_MSG, MSG_WIDG,	$
			    'No current realtime session in progress (' + $
			    CURR_FILE + ')'
		    RETURN
		END ELSE BEGIN
		    IF N_ELEMENTS(TELEM_UNIT) GT 0 THEN FREE_LUN,TELEM_UNIT
		    OPENR, TELEM_UNIT, CURR_FILE, ERROR=IOERR, /GET_LUN
;
		    IF (IOERR NE 0) THEN BEGIN
			CDS_MESSAGE, !ERR_STRING, /CONTINUE
			FLASH_MSG, MSG_WIDG, /APPEND,	$
				'Unable to open current telemetry file'
			RETURN
		    END ELSE BEGIN
;
;  Open the log file, and initialize the message window.
;
			IF KEYWORD_SET(LOG_UNIT) THEN FREE_LUN, LOG_UNIT
			OPENW, LOG_UNIT, LOG_FILE, /GET_LUN
			PRINTF, LOG_UNIT, 'FITSGEN report file ' + LOG_FILE
			WIDGET_CONTROL, LOG_WIDG, SET_VALUE=LOG_FILE_NAME
			WIDGET_CONTROL, MSG_WIDG, SET_VALUE=''
;
			CDS_MESSAGE, /CONTINUE, 'Opened file ' + CURR_FILE
		    ENDELSE
;
		    TITLE = "Processing realtime telemetry"
		    SKIP_PACK = 0L
		    I_PACK = 0L
		ENDELSE
	    ENDELSE
;
;  In either mode we should now be ready so start processing telemetry files.
;  First, turn on the hourglass and desensitize the widget (except for the
;  message windows) so that the user knows that widget events will not be
;  processed.  Display the title, and write out a message.
;
	    WIDGET_CONTROL, /HOURGLASS
	    WIDGET_CONTROL, MENU_BASE, SENSITIVE=0
	    WIDGET_CONTROL, START_BUTTON, SENSITIVE=0
	    WIDGET_CONTROL, STOP_BUTTON, SENSITIVE=1
	    WIDGET_CONTROL, SPACKET_WIDG, SENSITIVE=0
	    WIDGET_CONTROL, TITLE_WIDG, SET_VALUE=TITLE
	    CDS_MESSAGE, TITLE, /CONTINUE
	    CDS_MESSAGE, SYSTIME()+" Data acquisition started", /CONTINUE
;
;  Read in the first science packet.
;
	    DELVARX, LPACKET
	    IF NOT GET_SCI_PKT(SCI_MODE,TELEM_UNIT,LPACKET,/NEXT) THEN BEGIN
;
;  If no science packets were found, then simulate a "Timer" event to force the
;  widget to return to it's original state.
;
		CDS_MESSAGE, 'Unable to read first packet', /CONTINUE, $
			/ATTENTION
		VALUE = 'Timer'
;
;  Keep processing while still able to do rasters.
;
	    END ELSE RUNNING = 1
	ENDIF
;
;  If the stop button was pressed, then stop processing as soon as the current
;  file is finished.
;
	IF VALUE EQ 'Stop' THEN RUNNING = 0
;
;  If the timer event was seen, then process a FITS file.
;
	IF VALUE EQ 'Timer' THEN BEGIN
	    IF KEYWORD_SET(RUNNING) THEN BEGIN
		IF READ_RASTER(SCI_MODE,TELEM_UNIT,LPACKET,RASTER, DATAWIN) $
			THEN BEGIN
;
;  If the study counter is greater than the END_CNT value, then stop
;  processing.  Also stop processing if the STUDYCOUNTER/RASTERCOUNTER
;  combination is after the END_CNT/END_IND combination.
;
		    STUDYCOUNTER = UNSIGN(RASTER.HEADER.SCI.STUDYCOUNTER)
		    RASTERCOUNTER = RASTER.HEADER.SCI.RASTERCOUNTER
		    IF (END_CNT GT 0) AND ((STUDYCOUNTER GT END_CNT) OR	$
			    ((END_IND GE 0) AND (STUDYCOUNTER EQ END_CNT) AND $
			    (RASTERCOUNTER GT END_IND))) THEN BEGIN
			RUNNING = 0
;
;  If the study counter is less than the START_CNT value, then don't create the
;  FITS file.
;
		    END ELSE IF (STUDYCOUNTER GT START_CNT) OR	$
			    ((STUDYCOUNTER EQ START_CNT) AND	$
			    (RASTERCOUNTER GE START_IND)) THEN BEGIN
;
;  Read in the file study_counter_anomaly.dat.  If the current study is found,
;  then remap the study counter number.
;
			ANOMALY_FILE = CONCAT_DIR('$CDS_CP_MISC',	$
				'study_counter_anomaly.dat')
			IF FILE_EXIST(ANOMALY_FILE) THEN BEGIN
			    OPENR, AUNIT, ANOMALY_FILE, /GET_LUN
			    LINE = 'String'
			    WHILE NOT EOF(AUNIT) DO BEGIN
				READF, AUNIT, LINE
				LINE = STRCOMPRESS(STRTRIM(LINE,2))
				VALUES = STR_SEP(LINE, ' ')
				IF N_ELEMENTS(VALUES) EQ 3 THEN BEGIN
				    OLD_COUNTER = LONG(VALUES(0))
				    TEST_DATE = VALUES(1)
				    NEW_COUNTER = LONG(VALUES(2))
				    IF STUDYCOUNTER EQ OLD_COUNTER THEN BEGIN
					GETCDSTIMES, RASTER, TIMES
					IF TIMES.DATE_OBS GE TEST_DATE	$
						THEN BEGIN
					    CDS_MESSAGE, /CONTINUE,	$
						'Remapping study counter ' + $
						TRIM(STUDYCOUNTER) +	$
						' to ' + TRIM(NEW_COUNTER)
					    STUDYCOUNTER = NEW_COUNTER
					    RASTER.HEADER.SCI.STUDYCOUNTER = $
						FIX(STUDYCOUNTER)
					ENDIF
				    ENDIF
				ENDIF
			    ENDWHILE
			    FREE_LUN, AUNIT
			ENDIF
;
;  Form the filename from the study and raster counters, and write out the FITS
;  file.
;
			FILENAME = 's' + TRIM(STUDYCOUNTER) + 'r'
			RASTERCOUNTER = TRIM(RASTER.HEADER.SCI.RASTERCOUNTER)
			IF STRLEN(RASTERCOUNTER) EQ 1 THEN RASTERCOUNTER = $
				'0' + RASTERCOUNTER
			FILENAME = FILENAME + RASTERCOUNTER + '.fits'
			PATH = GETENV('CDS_FITS_DATA_W')
			IF PATH NE '' THEN BEGIN
			    TEST = FIND_WITH_DEF(FILENAME,'$CDS_FITS_DATA')
			    IF TEST NE '' THEN FILENAME = TEST ELSE	$
				    FILENAME = CONCAT_DIR(PATH, FILENAME)
			ENDIF
;
;  If we're processing CDROM data, and the telemetry file has been closed, then
;  don't create the FITS file.
;
			FILESTAT = FSTAT(TELEM_UNIT)
			IF FILESTAT.OPEN OR (NOT KEYWORD_SET(FINALDIST)) $
				THEN BEGIN
			    CDS_MESSAGE, 'Please wait, writing FITS file ...',$
				    /CONTINUE
			    MAKECDSFITS, FILENAME, RASTER, DATAWIN
			    CDS_MESSAGE, 'Wrote FITS file ' + FILENAME,	$
				    /CONTINUE
			END ELSE CDS_MESSAGE, 'Incomplete FITS file ' +	$
				FILENAME + ' not written', /CONTINUE
;
;  If we've just processed the last raster of the last study, then generate a
;  stop.
;
			IF (STUDYCOUNTER EQ END_CNT) AND	$
				(RASTERCOUNTER EQ END_IND) THEN RUNNING = 0
		    ENDIF
		END ELSE RUNNING = 0
	    ENDIF
;
;  If no longer running, either because the user has requested a stop, or we've
;  run out of data, then check to see if the final distribution was being
;  processed.  If it was, then run some engineering monitoring routines.
;
	    IF NOT KEYWORD_SET(RUNNING) THEN BEGIN
		IF KEYWORD_SET(FINALDIST) THEN BEGIN
		    CDS_MESSAGE, 'Running EMON_MECH_LOG', /CONTINUE
		    EMON_MECH_LOG, /QUIET, /ALL_CDROM
;
		    CDS_MESSAGE, 'Running EMON_RELAYS_LOG', /CONTINUE
		    EMON_RELAYS_LOG, /QUIET, /ALL_CDROM
;
		    CDS_MESSAGE, 'Running TLM_SCIENCE', /CONTINUE
		    TLM_SCIENCE, /QUIET
;
		    CDS_MESSAGE, 'Running REGEN_SYNOP', /CONTINUE
		    REGEN_SYNOP	
		ENDIF		
;
;  We're done.  Resensitize the widget, and signal that the data acquisition
;  has stopped.  The title is removed from the top, but placed in the message
;  widget in parentheses so that the user knows what the program was doing.
;  Close the log file.
;
		WIDGET_CONTROL, START_BUTTON, SENSITIVE=1
		WIDGET_CONTROL, STOP_BUTTON, SENSITIVE=0
		WIDGET_CONTROL, MENU_BASE, SENSITIVE=1
		WIDGET_CONTROL, SPACKET_WIDG, SENSITIVE=1
		WIDGET_CONTROL, TITLE_WIDG, GET_VALUE=TITLE
		CDS_MESSAGE, [' ','('+TITLE+')',SYSTIME()+	$
			" Data acquisition stopped"], /CONTINUE, BELL=3
		WIDGET_CONTROL, TITLE_WIDG, SET_VALUE=''
		FREE_LUN, LOG_UNIT
		LOG_UNIT = 0
		WIDGET_CONTROL, MSG_WIDG, /APPEND, SET_VALUE=	$
			"Closed log file "+LOG_FILE
		WIDGET_CONTROL, LOG_WIDG, SET_VALUE=''
	    ENDIF
	ENDIF
;
;  Want to SELECT data files.
;
	IF (VALUE EQ 'Get_TM') THEN BEGIN
	    SEL_TM_FILES, FILE_NAME
	    FILE_SEQ_NO = 0
	    NUM_DATA_FILES = N_ELEMENTS(FILE_NAME)
	    BREAK_FILE,FILE_NAME(FILE_SEQ_NO),DISK,DIR,RFILE,REXT
	    WIDGET_CONTROL,REP_WIDG,SET_VALUE=RFILE+REXT
	ENDIF
;
;  Request change to REPLAY mode.  Turn on the widgets for selecting the
;  telemetry files.  Note that a value of SCI_MODE other than 1 may be changed
;  by OPEN_TM_FILE.
;
	IF (VALUE EQ 'Replay') THEN BEGIN
	    SCI_MODE = 0
	    WIDGET_CONTROL,PBASE(0),MAP=0
	    WIDGET_CONTROL,PBASE(1),MAP=1
	    WIDGET_CONTROL,PFBASE(0),MAP=0
	    WIDGET_CONTROL,PFBASE(1),MAP=1
	ENDIF
;
;  Request change to REALTIME mode.  Turn off the widgets for selecting the
;  telmetry files.
;
	IF (VALUE EQ 'Realtime') THEN BEGIN       
	    SCI_MODE = 1
	    WIDGET_CONTROL,PBASE(1),MAP=0
	    WIDGET_CONTROL,PBASE(0),MAP=1
	    WIDGET_CONTROL,PFBASE(1),MAP=0
	    WIDGET_CONTROL,PFBASE(0),MAP=1
	ENDIF
;
;  Request change to limited or unlimited run.
;
	IF (VALUE EQ 'Limited')   THEN RUN_STATE=0
	IF (VALUE EQ 'Unlimited') THEN RUN_STATE=1
;
;  Request for HELP.
;
	IF (VALUE EQ 'Help') THEN WIDG_HELP, 'fitsgen',	$
		TITLE='Help: FITSGEN', GROUP=EV.TOP
;
;  The CDS operator value was modified.
;
	IF (VALUE EQ 'Operator') THEN BEGIN
		WIDGET_CONTROL, EV.ID, GET_VALUE=OPERATOR
		SETENV, 'CDS_OPERATOR=' + TRIM(OPERATOR(0))
	ENDIF
;
;  The start or end study counter or raster index was modified.
;
	IF (VALUE EQ 'StartCnt') THEN WIDGET_CONTROL, EV.ID,	$
		GET_VALUE=START_CNT
	IF (VALUE EQ 'StartInd') THEN WIDGET_CONTROL, EV.ID,	$
		GET_VALUE=START_IND
	IF (VALUE EQ 'EndCnt') THEN WIDGET_CONTROL, EV.ID,	$
		GET_VALUE=END_CNT
	IF (VALUE EQ 'EndInd') THEN WIDGET_CONTROL, EV.ID,	$
		GET_VALUE=END_IND
;
;  Toggle of autostop requested.
;
	IF (VALUE EQ 'Autostop') THEN AUTOSTOP = EV.SELECT
;
;  Toggle of autostop requested.
;
	IF (VALUE EQ 'FinalDist') THEN FINALDIST = EV.SELECT
;
;  If one is running, then start the timer.
;
	IF KEYWORD_SET(RUNNING) THEN WIDGET_CONTROL, MAIN_BASE, TIMER=0.1
;
	END
