	PRO FIX_GIS_POINT, FILENAME, NOWRITE=NOWRITE
;+
; Project     :	SOHO - CDS
;
; Name        :	FIX_GIS_POINT
;
; Purpose     :	Repair GIS pointing XCEN,YCEN values
;
; Category    :	Class5, CDS, Coordinates
;
; Explanation :	There was a bug in the routine CDS_POINT which occasionally
;		caused bad XCEN, YCEN pointing values in the FITS header.  This
;		routine updates the header to remove the bug.
;
; Syntax      :	FIX_GIS_POINT, FILENAME  [, /NOWRITE ]
;
; Examples    :	FIX_GIS_POINT, 's10066r00.fits'
;		FIX_GIS_POINT, '*.fits'
;
; Inputs      :	FILENAME = Name of file to repair.  Can include a wildcard to
;			   process a group of files.  However, including a
;			   path does not work with a wildcard.  NIS files are
;			   automatically ignored, as are files where the
;			   correction is under an arcsecond.
;
; Opt. Inputs :	None.
;
; Outputs     :	The names of the files processed and the correction amounts are
;		printed to the screen.
;
; Opt. Outputs:	None.
;
; Keywords    :	NOWRITE = If set, then the correction factor is calculated, but
;			  not applied.  All GIS filenames are printed, even if
;			  the correction factor is less than 1 arcsec.
;
; Calls       :	FIND_FILE, FXBOPEN, FXBMHEADER, FXPAR, FXBCOLNUM, FXBREAD,
;		MECH_POINT, BASE_INTERP, FXADDPAR, FXBTDIM, PRIV_ZDBASE,
;		INS_CATALOG, FXBCLOSE
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Prev. Hist. :	None.
;
; History     :	Version 1, 24-Dec-1997, William Thompson, GSFC
;
; Contact     :	WTHOMPSON
;-
;
	ON_ERROR, 2
;
	IN  = -1
	OUT = -1
;
	FILES = FIND_FILE(FILENAME, COUNT=COUNT)
	IF COUNT GT 1 THEN BEGIN
	    FOR I = 0,COUNT-1 DO FIX_GIS_POINT, FILES(I), NOWRITE=NOWRITE
	    GOTO, FINISH
	ENDIF
;
;  Open the input and output files.
;
	FXBOPEN, IN, FILENAME, 1, EXTHEADER
	HEADER = FXBMHEADER(IN)
	NHEAD0 = FIX((N_ELEMENTS(HEADER)+35)/36)
	NEXHD0 = FIX((N_ELEMENTS(EXTHEADER)+35)/36)
	NANVALUE = -1E6
;
	DETECTOR = STRTRIM(FXPAR(HEADER, 'DETECTOR'))
	IF DETECTOR NE 'GIS' THEN GOTO, FINISH
;
	INS_X0  = FXPAR(HEADER, 'INS_X0')
	INS_Y0  = FXPAR(HEADER, 'INS_Y0')
	XCEN0   = FXPAR(HEADER, 'XCEN')
	YCEN0   = FXPAR(HEADER, 'YCEN')
	NX      = FXPAR(HEADER, 'NX')
	NY      = FXPAR(HEADER, 'NY')
	SC_X0   = FXPAR(HEADER, 'SC_X0')
	SC_Y0   = FXPAR(HEADER, 'SC_Y0')
	SC_ROLL = FXPAR(HEADER, 'SC_ROLL')
	XSTEP   = FXPAR(HEADER, 'XSTEP')
	YSTEP   = FXPAR(HEADER, 'YSTEP')
;
	NWINDOWS = FXPAR(HEADER, 'NWINDOWS')
	SLIT_NUM = FXPAR(HEADER, 'SLIT_NUM')
;
	MIRRORSTEP = FXPAR(HEADER, 'XSTEP')
	SLITSTEP   = FXPAR(HEADER, 'YSTEP')
;
	MIR_COL = FXBCOLNUM(IN, 'MIR_POS')
	FXBREAD, IN, MIRRORPOSITION, MIR_COL
	MIR_NULL = FXPAR(EXTHEADER, 'TNULL' + TRIM(MIR_COL))
	HEADERFOUND = MIRRORPOSITION NE MIR_NULL
;
	FXBREAD, IN, SLITPOSITION, 'SLIT_POS'
;
	IF (XSTEP EQ 0) AND (YSTEP EQ 0) AND (NX*NY GT 1) THEN BEGIN
	    MIRRORPOSITION = REFORM(MIRRORPOSITION, NX, NY)
	    SLITPOSITION   = REFORM(SLITPOSITION,   NX, NY)
	    HEADERFOUND    = REFORM(HEADERFOUND,    NX, NY)
	ENDIF
;
;  Modify the pointing based on the slit and mirror positions.
;
	XCEN = INS_X0
	YCEN = INS_Y0
	MECH_POINT, SLITPOSITION, MIRRORPOSITION, XCEN, YCEN
;
;  If an array of centers were calculated, then calculate XCEN by first
;  interpolating to the middle position in the slit direction, and then to the
;  middle position in the mirror direction.
;
	IF N_ELEMENTS(XCEN) GT 1 THEN BEGIN
		IF NY GT 1 THEN BEGIN
		    TEMP = XCEN(*,0)
		    XFOUND = REPLICATE(0,NX)
		    FOR IX = 0,NX-1 DO BEGIN
			XTEMP = XCEN(IX,*)
			HFTEMP = HEADERFOUND(IX,*)
			W = WHERE(HFTEMP, COUNT)
			IF COUNT EQ 1 THEN BEGIN
			    TEMP(IX) = XTEMP(W(0))
			    XFOUND(IX) = 1
			END ELSE IF COUNT GT 1 THEN BEGIN
			    TEMP(IX) = BASE_INTERP(XTEMP(W), W, (NY-1)/2., $
				    MINSIG=1)
			    XFOUND(IX) = 1
			ENDIF
		    ENDFOR
		    XCEN = TEMP
		END ELSE XFOUND = HEADERFOUND
;
		W = WHERE(XFOUND, COUNT)
		IF COUNT EQ 1 THEN BEGIN
			XCEN = XCEN(W(0))
			MIRPOS_USED = W(0)
		END ELSE BEGIN
			XCEN = BASE_INTERP(XCEN(W), W, (NX-1)/2., MINSIG=1)
			MIRPOS_USED = -1
		ENDELSE
;
;  Do the opposite for YCEN.
;
		TEMP = REFORM(YCEN(0,*))
		YFOUND = REPLICATE(0,NY)
		FOR IY = 0,NY-1 DO BEGIN
		    YTEMP = YCEN(*,IY)
		    HFTEMP = HEADERFOUND(*,IY)
		    W = WHERE(HFTEMP, COUNT)
		    IF COUNT EQ 1 THEN BEGIN
			TEMP(IY) = YTEMP(W(0))
			YFOUND(IY) = 1
		    END ELSE IF COUNT GT 1 THEN BEGIN
			TEMP(IY) = BASE_INTERP(YTEMP(W), W, (NY-1)/2., $
				MINSIG=1)
			YFOUND(IY) = 1
		    ENDIF
		ENDFOR
		YCEN = TEMP
;
		W = WHERE(YFOUND, COUNT)
		IF COUNT EQ 1 THEN BEGIN
			YCEN = YCEN(W(0))
			SLITPOS_USED = W(0)
		END ELSE BEGIN
			YCEN = BASE_INTERP(YCEN(W), W, (NY-1)/2., MINSIG=1)
			SLITPOS_USED = -1
		ENDELSE
	END ELSE BEGIN
		MIRPOS_USED  = -1
		SLITPOS_USED = -1
	ENDELSE
;
;  Make sure that XCEN and YCEN end up as scalars.
;
	XCEN = XCEN(0)
	YCEN = YCEN(0)
;
;  Calculate the width due to the slit.
;
	CASE SLIT_NUM OF
		1:  BEGIN  &  SXWIDTH =  2  &  SYWIDTH =   2	& END
		2:  BEGIN  &  SXWIDTH =  4  &  SYWIDTH =   4	& END
		3:  BEGIN  &  SXWIDTH =  8  &  SYWIDTH =  50.8	& END
		4:  BEGIN  &  SXWIDTH =  2  &  SYWIDTH = 240	& END
		5:  BEGIN  &  SXWIDTH =  4  &  SYWIDTH = 240	& END
		6:  BEGIN  &  SXWIDTH = 90  &  SYWIDTH = 240	& END
		ELSE: BEGIN
			CDS_MESSAGE, 'CDS_POINT: Unrecognized slit number', $
				/ATTENTION, /CONTINUE
			SXWIDTH = 2
			SYWIDTH = 2
			END
	ENDCASE
;
;  Calculate the relative fields-of-view for each of the data windows depending
;  on the detector.  Also calculate the starting pixel and step size for each
;  window.
;
	IYWIDTH = (NY - 1.) * SLITSTEP + SYWIDTH
	IF SLITPOS_USED LT 0 THEN RYCEN = 0 ELSE RYCEN =	$
		(0.5*(NY-1) - SLITPOS_USED) * SLITSTEP
	YSTART = REPLICATE(YCEN, NWINDOWS) - (IYWIDTH - SYWIDTH) / 2
	YDELTA = REPLICATE(SLITSTEP > 1.016, NWINDOWS)
;
;  The relative width in X is calculated in the same way for either
;  spectrograph.
;
	IXWIDTH = (NX - 1.) * MIRRORSTEP + SXWIDTH
;
;  Modify the pointing based on the detector information.  Note that the scan
;  mirror moves backwards.
;
	IF MIRPOS_USED GE 0 THEN XCEN = XCEN - (0.5*(NX-1) - MIRPOS_USED) * $
		MIRRORSTEP
	YCEN = YCEN + RYCEN
;
;  Store the starting point and delta in the X direction.
;
	XSTART = XCEN - (IXWIDTH - SXWIDTH) / 2
	XDELTA = MIRRORSTEP > 1.016
;
;  Assume that ANGLE is the spacecraft roll.
;
	ANGLE = SC_ROLL
;
;  Modify the pointing based on ANGLE, and the spacecraft pointing.
;
	XX = XCEN  &  YY = YCEN
	COS_A = COS(ANGLE / !RADEG)
	SIN_A = SIN(ANGLE / !RADEG)
	XCEN = XX * COS_A  -  YY * SIN_A  +  SC_X0
	YCEN = XX * SIN_A  +  YY * COS_A  +  SC_Y0
;
;  If the NOWRITE keyword was set, then print out an informative message.
;
	IF KEYWORD_SET(NOWRITE) THEN BEGIN
	    PRINT, FILENAME, XCEN-XCEN0, YCEN-YCEN0, FORMAT='(A20,2F8.1)'
	    GOTO, FINISH
	ENDIF
;
;  If the position hasn't changed by more than an arcsecond, then don't rewrite
;  the FITS file.
;
	IF ((XCEN-XCEN0)^2 + (YCEN-YCEN0)^2) LE 1 THEN GOTO, FINISH
	PRINT, FILENAME, XCEN-XCEN0, YCEN-YCEN0, FORMAT='(A20,2F8.1)'
;
;  Calculate the start positions.
;
	XX = XSTART  &  YY = YSTART
	XSTART = XX * COS_A  -  YY * SIN_A  +  SC_X0
	YSTART = XX * SIN_A  +  YY * COS_A  +  SC_Y0
;
;  Calculate the array of instrument pointings from all the mechanism
;  movements.
;
	INS_X = INS_X0
	INS_Y = INS_Y0
	MECH_POINT, SLITPOSITION, MIRRORPOSITION, INS_X, INS_Y
	W = WHERE(HEADERFOUND NE 1, COUNT)
	IF COUNT GT 0 THEN BEGIN
	    INS_X(W) = NANVALUE
	    INS_Y(W) = NANVALUE
	ENDIF
;
;  Update information in the FITS headers.
;
	FXADDPAR, HEADER, 'XCEN', XCEN, FORMAT='(F7.1)'
	FXADDPAR, HEADER, 'YCEN', YCEN, FORMAT='(F7.1)'
	FXADDPAR, EXTHEADER, 'XCEN', XCEN, FORMAT='(F7.1)'
	FXADDPAR, EXTHEADER, 'YCEN', YCEN, FORMAT='(F7.1)'
;
;  Update the origins in the headers.
;
	N_COLUMNS = FXPAR(EXTHEADER, 'TFIELDS')
	FOR I = 1, N_COLUMNS DO BEGIN
	    AXES = FXPAR(EXTHEADER, 'TDESC' + TRIM(I), COUNT=COUNT)
	    IF COUNT EQ 1 THEN BEGIN
		AXES = FXBTDIM(AXES)
		ORIGIN = FLOAT(FXBTDIM(FXPAR(EXTHEADER, 'TRVAL' + TRIM(I))))
		ROTATION = FXPAR(EXTHEADER, 'TROTA' + TRIM(I))
;
;  Find where the SOLAR_X axis is, and modify its origin.  Do the same for
;  SOLAR_Y.  If the NOROLL keyword is set, then modify the rotation field.
;
		W = WHERE(AXES EQ 'SOLAR_X', COUNT)
		IF COUNT GT 0 THEN ORIGIN(W,*) = XSTART(0)
		W = WHERE(AXES EQ 'SOLAR_Y', COUNT)
		IF COUNT GT 0 THEN ORIGIN(W,*) = YSTART(0)
;
		N_AXES = N_ELEMENTS(AXES)
		IF N_AXES EQ 1 THEN TRVAL = '' ELSE TRVAL = '('
		FOR J = 0,N_AXES-1 DO BEGIN
		    IF J GT 0 THEN TRVAL = TRVAL + ','
		    TRVAL = TRVAL + TRIM(ORIGIN(J))
		ENDFOR
		IF N_AXES GT 1 THEN TRVAL = TRVAL + ')'
		FXADDPAR, EXTHEADER, 'TRVAL' + TRIM(I), TRVAL
	    ENDIF
	ENDFOR
;
;  Reopen the file for update, and write the headers.
;
	NHEAD = FIX((N_ELEMENTS(HEADER)+35)/36)
	NEXHD = FIX((N_ELEMENTS(EXTHEADER)+35)/36)
	IF (NHEAD NE NHEAD0) OR (NEXHD NE NEXHD0) THEN BEGIN
	    MESSAGE, 'Header lengths not the same for file ' +	$
		    FILENAME, /CONTINUE
	    GOTO, FINISH
	ENDIF
	FXBCLOSE, IN  &  IN = -1
	OPENU, OUT, FILENAME, /GET_LUN
;
	LEN = N_ELEMENTS(HEADER)
	BHDR = REPLICATE(32B, 80, 36*NHEAD)
	FOR N = 0,LEN-1 DO BHDR(0,N) = BYTE( STRMID(HEADER(N),0,80) )
	WRITEU, OUT, BHDR
;
	LEN = N_ELEMENTS(EXTHEADER)
	BHDR = REPLICATE(32B, 80, 36*NEXHD)
	FOR N = 0,LEN-1 DO BHDR(0,N) = BYTE( STRMID(EXTHEADER(N),0,80) )
	WRITEU, OUT, BHDR
;
;  Update the entry in the catalog.
;
	IF PRIV_ZDBASE(/CAT,/QUIET) THEN BEGIN
	    !PRIV = 2
	    INS_CATALOG, HEADER
	ENDIF
;
FINISH:
	IF IN GE 0 THEN FXBCLOSE, IN
	IF OUT GE 0 THEN FREE_LUN, OUT
;
	RETURN
	END
