	FUNCTION GET_SCI_PKT, TEL_MODE, UNIT, PACKET, NEXT=NEXT
;+
; Project     : SOHO - CDS
;
; Name        : GET_SCI_PKT()
;
; Purpose     : Get a science packet from the telemetry stream.
;
; Explanation : Keeps reading the telemetry stream until a packet is
;		encountered that is directly related to the science data, i.e.
;		not an engineering packet, etc.
;
; Use         : Result = GET_SCI_PKT( TEL_MODE, UNIT, PACKET [, /NEXT ] )
;
; Inputs      : TEL_MODE = Input mode.  The following modes have been defined.
;
;				0:	Replay stored telemetry
;				1:	Realtime telemetry
;				2:	Quicklook tape recorder dumps
;				3:	Realtime telemetry archived by ECS
;				4:	Final distribution
;
;		UNIT	 = Logical unit number of input stream.
;		PACKET	 = Current telemetry packet.
;
; Opt. Inputs : None.
;
; Outputs     : PACKET	= First telemetry packet that satisfies the test
;			  condition.  This may be the same as the packet that
;			  was passed to this routine.
;
;		The return value for the function can be one of the following:
;
;			 1	Packet successfully read.
;			 0	End-of-file encountered.
;			-1	Error during read.
;
; Opt. Outputs: None.
;
; Keywords    : NEXT	= Setting this keyword forces the routine to start with
;			  the next packet in the telemetry stream instead of
;			  testing the current packet first.
;
; Calls       : CDS_MESSAGE, OPEN_NEXT, READ_PACKET, TEST_SCI_PKT
;
; Common      : GET_SCI_PKT is a private common block used to maintain
;		information between calls.
;
;		Also uses FITSGEN from the include file "fitsgen.cmn"
;
; Restrictions: PACKET must contain a valid packet.
;
; Side effects: If an error is discovered, then an error message is written to
;		the telemetry log, and in some cases the error is brought to
;		the attention of the CDS operator.
;
; Category    : Data Handling, Telemetry
;
; Prev. Hist. : None.
;
; Written     : William Thompson, GSFC, 16 April 1993.
;
; Modified    : Version 1, William Thompson, GSFC, 16 April 1993.
;		Version 2, William Thompson, GSFC, 31 January 1995
;			Modified to call OPEN_NEXT for realtime telemetry.
;		Version 3, William Thompson, GSFC, 2 February 1995
;			Modified to notify only once when one can no longer
;			read packets.
;		Version 4, William Thompson, GSFC, 29 June 1995
;			Added test for CHECKSUM, and common block FITSGEN
;		Version 5, William Thompson, GSFC, 1 November 1995
;			Add time to error messages produced.
;		Version 6, William Thompson, GSFC, 22 December 1995
;			Add test for packet time when opening a new file.
;		Version 7, William Thompson, GSFC, 18 January 1996
;			Added retry when checksum error encountered during
;			realtime telemetry.
;		Version 8, William Thompson, GSFC, 18 March 1996
;			Close telemetry unit at end of run.  This signals
;			FITSGEN to not try to generate a file when processing
;			CDROM data.
;		Version 9, William Thompson, GSFC, 2 May 1996
;			Use /BELL instead of /ATTENTION
;		Version 10, William Thompson, GSFC, 15 May 1996
;			Don't close telemetry unit if realtime data
;		Version 11, William Thompson, GSFC, 2 August 1996
;			Make sure telemetry file is open if realtime data
;		Version 12, William Thompson, GSFC, 9 July 1997
;			Close and reopen realtime telemetry file on error
;		Version 13, William Thompson, GSFC, 16 March 1998
;			Apply test for packet time to all packets
;
; Version     : Version 13, 16 March 1998
;-
;
	ON_ERROR, 2
	COMMON GET_SCI_PKT, ALREADY_NOTIFIED
@fitsgen.cmn
;
;  Initialize the parameter LAST_TIME.
;
	IF DATATYPE(PACKET,1) EQ 'Structure' THEN	$
		LAST_TIME = OBT2TAI(PACKET.HDR.TIME) ELSE LAST_TIME = 0.0D0
;
;  If the NEXT keyword was not set, then test the current packet.
;
	IF NOT KEYWORD_SET(NEXT) THEN	$
		IF TEST_SCI_PKT(PACKET) THEN GOTO, TEST_CHECKSUM
;
;  Otherwise keep reading until a correct packet is found.
;
READ_PACKET:
	TEST = READ_PACKET( TEL_MODE, UNIT, PACKET )
;
;  Signal any errors, unless the user has already been notified
;
	IF TEST EQ -1 THEN BEGIN
		IF NOT KEYWORD_SET(ALREADY_NOTIFIED) THEN BEGIN
			CDS_MESSAGE, /BELL, /CONTINUE, /ADD_TIME,	$
			    'GET_SCI_PKT:  Error encountered reading packet'
			ALREADY_NOTIFIED = 1
		ENDIF
		RETURN, -1
	ENDIF
;
;  If no packet was found, then try to open the next telemetry file.
;
	IF TEST EQ 0 THEN BEGIN
		IF NOT OPEN_NEXT( TEL_MODE, UNIT ) THEN BEGIN
			CDS_MESSAGE, /CONTINUE, 'End of data'
			IF TEL_MODE NE 1 THEN CLOSE, UNIT
			RETURN, 0
		ENDIF
		GOTO, READ_PACKET
	ENDIF
;
;  If the packet is not a science packet, then try again.
;
	IF NOT TEST_SCI_PKT(PACKET) THEN GOTO, READ_PACKET
;
;  Test the checksum.  If reading realtime telemetry, then wait a bit and try
;  again before giving up completely.
;
TEST_CHECKSUM:
	IF CHECKSUM NE PACKET.PKT.CHKSUM THEN BEGIN
	    IF TEL_MODE EQ 1 THEN BEGIN
		FILESTAT = FSTAT(UNIT)
		IF FILESTAT.OPEN THEN BEGIN
			WAIT, 0.1
			POINT_LUN, -UNIT, CUR_POS
			CLOSE, UNIT
			OPENR, UNIT, FILESTAT.NAME
			POINT_LUN, UNIT, CUR_POS-306
			I_PACK = I_PACK - 1L
			TEST = READ_PACKET( TEL_MODE, UNIT, PACKET )
			IF CHECKSUM EQ PACKET.PKT.CHKSUM THEN GOTO, TEST_TIME
		ENDIF
	    ENDIF
	    CDS_MESSAGE, /CONTINUE, 'Packet ' + TRIM(I_PACK) +	$
		    ' does not have the correct checksum', /ADD_TIME
	    GOTO, READ_PACKET
	ENDIF
;
;  Test the packet time against the last time.
;
TEST_TIME:
	IF KEYWORD_SET(NEXT) THEN BEGIN
	    IF OBT2TAI(PACKET.HDR.TIME) LE LAST_TIME THEN GOTO, READ_PACKET
	ENDIF
;
;  Unset ALREADY_NOTIFIED and return 1 for success.
;
	ALREADY_NOTIFIED = 0
	RETURN, 1
	END
