	PRO GET_VDS_SLIT, TIMES, SY1, SY2, RASTER=RASTER, SLENGTH=SLENGTH
;+
; Project     :	SOHO - CDS
;
; Name        :	GET_VDS_SLIT
;
; Purpose     :	Get the limits of the VDS slit on the detector.
;
; Category    :	
;
; Explanation :	Extracts information from the state database to get information
;		about the VDS slit.
;
; Syntax      :	GET_VDS_SLIT, TIMES, SY1, SY2
;
; Examples    :	
;
; Inputs      :	TIMES	= A structure variable from GETCDSTIMES.
;
; Opt. Inputs :	None.
;
; Outputs     :	SY1	= The bottom of the slit in the two spectral bands.
;		SY2	= The top of the slit in the two spectral bands.
;
; Opt. Outputs:	None.
;
; Keywords    :	RASTER	= Structure variable from READ_RASTER.  When passed,
;			  the 
;		SLENGTH	= The length of the NIS slits, in pixels.  The default
;			  is 143.
;
; Calls       :	GET_VDS_SLITPOS, GET_UTC
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Prev. Hist. :	Used to be part of the routine CDS_POINT.
;
; History     :	Version 1, 16-Oct-1995, William Thompson, GSFC
;		Version 2, 22-Feb-1996, William Thompson, GSFC
;			Added keyword SLENGTH.
;		Version 3, 23-Feb-1996, William Thompson, GSFC
;			Corrected bug where positions were reversed on CCD.
;		Version 4, 02-May 1996, William Thompson, GSFC
;			Use /BELL instead of /ATTENTION
;		Version 5, William Thompson, GSFC, 20 May 1996
;			Call CDS_MESSAGE with /CONTINUE so attention message is
;			not generated.
;
; Version     :	Version 5, 20 May 1996
;
; Contact     :	WTHOMPSON
;-
;
	ON_ERROR, 2
;
;  Get the VDS state.
;
	ERRMSG = ''
	GET_VDS_SLITPOS, TIMES.OBT_TIME, DESC, ERRMSG=ERRMSG
	IF (ERRMSG NE '') OR (DESC.DATE EQ 0) THEN BEGIN
		ERRMSG = [ERRMSG, 'Unable to get state for time ' + $
			TAI2UTC(TIMES.OBT_TIME,/CCSDS),	$
			'Using current time instead']
		CDS_MESSAGE, ERRMSG, /BELL, /CONTINUE
		GET_UTC, UTC
		GET_VDS_SLITPOS, UTC, DESC
	ENDIF
;
;  If the RASTER structure was passed, then extract the centers of the data
;  windows.  Otherwise, assume 511.5
;
	IF N_ELEMENTS(RASTER) EQ 1 THEN BEGIN
		XCEN = RASTER.HEADER.DATA_WIN.STARTX +	$
			RASTER.HEADER.DATA_WIN.LENGTHX / 2.
		XCEN = XCEN(WHERE(RASTER.HEADER.DATA_WIN.LENGTHX NE 0))
	END ELSE XCEN = 511.5
;
;  If XCEN is scalar, then force it to be a vector for the array multiplication
;  below to work.
;
	IF N_ELEMENTS(XCEN) EQ 1 THEN XCEN = XCEN(0:0)
;
;  Calculate the centers of the slit.  The result will have dimensions (2,NW).
;  Subtract it from 1023 to convert from the CDHS coordinate system to the VDS
;  coordinate system.
;
	YS = [DESC.N1_CEN_S, DESC.N2_CEN_S]
	YL = [DESC.N1_CEN_L, DESC.N2_CEN_L]
	YCEN = YS # REPLICATE(1,N_ELEMENTS(XCEN)) + (YL-YS) # (XCEN / 1023.)
;
;  If the SLENGTH parameter is not passed, then 143 is assumed.  Use the length
;  to determine the bottoms and tops of the slit.
;
	IF N_ELEMENTS(SLENGTH) EQ 0 THEN SLENGTH = 143
	SY1 = ROUND(YCEN - SLENGTH/2.)
	SY2 = SY1 + SLENGTH - 1
;
	RETURN
	END
