	PRO GUESS_STEP, RASTER, MIRRORSTEP, SLITSTEP
;+
; Project     :	SOHO - CDS
;
; Name        :	GUESS_STEP
;
; Purpose     :	Guess the step size from the telemetry.
;
; Explanation :	This routine takes the mirror and step positions from the
;		exposure headers in the telemetry, and tries to determine what
;		the step sizes are.
;
; Use         :	GUESS_STEP, RASTER, MIRRORSTEP, SLITSTEP
;
; Inputs      :	RASTER	 = Structure variable from READ_RASTER, containing all
;			   the metadata extracted from the science telemetry
;			   stream pertaining to the raster.
;
; Opt. Inputs :	None.
;
; Outputs     :	MIRRORSTEP = The determined mirror step size, multiplied by 2
;			     to roughly convert to arcsec.
;		SLITSTEP   = The determined slit step size.
;
; Opt. Outputs:	None.
;
; Keywords    :	None.
;
; Calls       :	None.
;
; Common      :	None.
;
; Restrictions:	Returns zero if the exposure headers are missing from the
;		telemetry.
;
; Side effects:	None.
;
; Category    :	Data_handling, Telemetry, I/O, FITS, CDS.
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 18 October 1994
;
; Modified    :	Version 1, William Thompson, GSFC, 18 October 1994
;		Version 2, William Thompson, GSFC, 19 March 1996
;			Made more robust
;
; Version     :	Version 2, 19 March 1996
;-
;
	ON_ERROR,2
;
;  Check the number of parameters.
;
	IF N_PARAMS() NE 3 THEN MESSAGE,	$
		'Syntax:  GUESS_STEP, RASTER, MIRRORSTEP, SLITSTEP'
;
;  Reformat the step positions into properly dimensioned arrays.
;
	N_MIRROR = RASTER.HEADER.SCI.NUMMIRRORPOS > 1
	N_SLIT	 = RASTER.HEADER.SCI.NUMSLITPOS > 1
	SLITPOS  = REFORM((RASTER.SLITPOSITION)(*),   N_SLIT, N_MIRROR)
	MIRPOS   = REFORM((RASTER.MIRRORPOSITION)(*), N_SLIT, N_MIRROR)
	HEADFND	 = REFORM((RASTER.HEADERFOUND)(*),    N_SLIT, N_MIRROR)
;
;  Calculate the differences in the scan mirror and slit directions.  In the
;  mirror direction, assume that the minimum is the step size, since solar
;  tracking should only serve to increase this parameter.
;
	IF N_MIRROR EQ 1 THEN MIRRORSTEP = 0 ELSE BEGIN
		MIRRORSTEP = MIRPOS(*,1:*)  -   MIRPOS(*,0:N_MIRROR-2)
		VALID     = HEADFND(*,1:*) AND HEADFND(*,0:N_MIRROR-2)
		IF TOTAL(VALID) EQ 0 THEN MIRRORSTEP = 0 ELSE	$
			MIRRORSTEP = MIN(MIRRORSTEP(WHERE(VALID EQ 1)))
	ENDELSE
;
;  In the slit direction, take the most common value, since solar tracking
;  could either increase or decrease this parameter, depending on the solar B0
;  angle, and on the initial solar latitude and longitude of the target.
;
	IF N_SLIT EQ 1 THEN SLITSTEP = 0 ELSE BEGIN
		D_SLIT = SLITPOS(1:*,*)  -  SLITPOS(0:N_SLIT-2,*)
		VALID  = HEADFND(1:*,*) AND HEADFND(0:N_SLIT-2,*)
		W = WHERE(VALID EQ 1, COUNT)
		IF COUNT EQ 0 THEN BEGIN
			SLITSTEP = 0
		END ELSE IF MAX(D_SLIT(W)) EQ MIN(D_SLIT(W)) THEN BEGIN
			SLITSTEP = D_SLIT(W(0))
		END ELSE IF COUNT EQ 1 THEN BEGIN
			SLITSTEP = D_SLIT(W(0))
		END ELSE BEGIN
			DMIN = MIN(D_SLIT(W))
			H = HISTOGRAM(D_SLIT(W),MIN=DMIN)
			SLITSTEP = (WHERE(H EQ MAX(H)))(0) + DMIN
		ENDELSE
	ENDELSE
;
;  Multiply MIRRORSTEP by two to put into rough units of arcseconds.
;
	MIRRORSTEP = 2 * MIRRORSTEP
;
	RETURN
	END
