	PRO INS_CATALOG, HEADER, ERRMSG=ERRMSG
;+
; Project     :	SOHO - CDS
;
; Name        :	INS_CATALOG
;
; Purpose     :	Insert as-run catalog records based on the FITS header
;
; Category    :	Data handling, Telemetry, FITS
;
; Explanation :	This routine parses the FITS header, and derives the entries
;		for the MAIN and EXPERIMENT "as-run" catalogs.  If the
;		information is not already in the catalog, then it is added.
;
; Syntax      :	INS_CATALOG, HEADER
;
; Examples    :	
;
; Inputs      :	HEADER = FITS header
;
; Opt. Inputs :	None.
;
; Outputs     :	None.
;
; Opt. Outputs:	None.
;
; Keywords    :	ERRMSG = If defined and passed, then any error messages will be
;			 returned to the user in this parameter rather than
;			 depending on the MESSAGE routine in IDL.  If no errors
;			 are encountered, then a null string is returned.  In
;			 order to use this feature, ERRMSG must be defined
;			 first, e.g.
;
;				ERRMSG = ''
;				INS_CATALOG, HEADER, ERRMSG=ERRMSG
;				IF ERRMSG NE '' THEN ...
;
;
; Calls       :	FXPAR, GET_MAIN, ADD_MAIN, MOD_MAIN, GET_EXPER, ADD_EXPER,
;		MOD_EXPER.
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Prev. Hist. :	None.
;
; History     :	Version 1, 04-Aug-1995, William Thompson, GSFC
;		Version 2, 02-Nov-1995, William Thompson, GSFC
;			Force type declaration of tags.
;		Version 3, 21-Feb-1995, William Thompson, GSFC
;			Call CDS_MESSAGE.  Fixed bug with pointing box
;			calculation.
;		Version 4, 23-Feb-1996, William Thompson, GSFC
;			Modified for version 2 of the experiment databases.
;		Version 5, 06-Mar-1996, William Thompson, GSFC
;			Get SC parameters from the header now.
;		Version 6, 12-Mar-1996, William Thompson, GSFC
;			Get OBS_MODE and OBS_SEQ from the header, if there.
;		Version 7, 02-Apr-1996, William Thompson, GSFC
;			Modified for version 3 of the experiment databases.
;		Version 8, 08-Apr-1996, William Thompson, GSFC
;			Fixed bug introduced in version 7 where the DETECTOR
;			field in the MAIN structure had too many characters.
;		Version 9, 23-Sep-2002, William Thompson, GSFC
;			Improved way that observation box is calculated for
;			the MAIN database.
;
; Contact     :	WTHOMPSON
;-
;
	ON_ERROR, 2
;
;  Initialize MESSAGE to the null string.  If an error occurs, it will be
;  updated below.
;
	MESSAGE = ''
;
;  Check the number of parameters.
;
	IF N_PARAMS() NE 1 THEN BEGIN
		MESSAGE = 'Syntax:  INS_CATALOG, HEADER'
		GOTO, HANDLE_ERROR
	ENDIF
;
;  Get the sequence number for this observation.
;
	SEQ_NUM = FXPAR(HEADER, 'SEQ_NUM')
;
;  Extract the information for the MAIN database from the header.
;
	MAIN1 ={PROG_NUM: FXPAR(HEADER, 'PROG_NUM'),		$
		PROG_ID: FIX(FXPAR(HEADER, 'PROG_ID')),		$
		PROG_IND: FIX(FXPAR(HEADER, 'PROG_IND')),	$
		STUDY_ID: FIX(FXPAR(HEADER, 'STUDY_ID')),	$
		STUDYVAR: FIX(FXPAR(HEADER, 'STUDYVAR')),	$
		OBS_PROG: STRTRIM(FXPAR(HEADER, 'OBS_PROG')),	$
		DETECTOR: STRMID(FXPAR(HEADER, 'DETECTOR'),0,1),$
		SCI_OBJ: FXPAR(HEADER, 'SCI_OBJ'),		$
		SCI_SPEC: FXPAR(HEADER, 'SCI_SPEC'),		$
		CMP_NO: FIX(FXPAR(HEADER, 'CMP_NO')),		$
		OBJECT: STRTRIM(FXPAR(HEADER, 'OBJECT'),2),	$
		OBJ_ID: STRTRIM(FXPAR(HEADER, 'OBJ_ID'),2),	$
		DATE_OBS: UTC2TAI(FXPAR(HEADER, 'DATE_OBS')),	$
		DATE_END: UTC2TAI(FXPAR(HEADER, 'DATE_END')),	$
		OBT_TIME: DOUBLE(FXPAR(HEADER, 'OBT_TIME')),	$
		OBT_END: DOUBLE(FXPAR(HEADER, 'OBT_END')),	$
		XCEN: FLOAT(FXPAR(HEADER, 'XCEN')),		$
		YCEN: FLOAT(FXPAR(HEADER, 'YCEN')),		$
		ANGLE: FLOAT(FXPAR(HEADER, 'ANGLE')),		$
		IXWIDTH: FLOAT(FXPAR(HEADER, 'IXWIDTH')),	$
		IYWIDTH: FLOAT(FXPAR(HEADER, 'IYWIDTH')),	$
		SEQ_FROM: SEQ_NUM,				$
		SEQ_TO: SEQ_NUM,				$
		COMMENTS: 'N'}
;
;  Get the corresponding entry, if any, from the MAIN database.
;
	GET_MAIN, MAIN1.PROG_NUM, MAIN0
;
;  If none was found, then add a new entry into the database.
;
	IF MAIN0.PROG_NUM LT 0 THEN BEGIN
		print,'Calling ADD_MAIN'
		IF NOT ADD_MAIN(MAIN1, ERRMSG=MESSAGE) THEN GOTO, HANDLE_ERROR
;
;  Otherwise, check the old entry against the new to see if anything needs to
;  be updated.
;
	END ELSE BEGIN
		MODIFY = 0
		IF MAIN0.PROG_ID NE MAIN1.PROG_ID THEN MODIFY = 1
		IF MAIN0.PROG_IND NE MAIN1.PROG_IND THEN MODIFY = 1
		IF MAIN0.STUDY_ID NE MAIN1.STUDY_ID THEN MODIFY = 1
		IF MAIN0.STUDYVAR NE MAIN1.STUDYVAR THEN MODIFY = 1
		IF TRIM(MAIN0.SCI_OBJ) NE TRIM(MAIN1.SCI_OBJ) THEN MODIFY = 1
		IF TRIM(MAIN0.SCI_SPEC) NE TRIM(MAIN1.SCI_SPEC) THEN MODIFY = 1
		IF MAIN0.CMP_NO NE MAIN1.CMP_NO THEN MODIFY = 1
		IF TRIM(MAIN0.OBJECT) NE TRIM(MAIN1.OBJECT) THEN MODIFY = 1
		IF TRIM(MAIN0.OBJ_ID) NE TRIM(MAIN1.OBJ_ID) THEN MODIFY = 1
;
;  Calculate the smallest box which encompasses all the data.  The assumption
;  is made that ANGLE does not change.  First convert the data to the
;  instrument coordinate system.
;
		A = MAIN0.ANGLE / !RADEG
		COS_A = COS(A)
		SIN_A = SIN(A)
		X0 =  MAIN0.XCEN * COS_A + MAIN0.YCEN * SIN_A
		Y0 = -MAIN0.XCEN * SIN_A + MAIN0.YCEN * COS_A
		X1 =  MAIN1.XCEN * COS_A + MAIN1.YCEN * SIN_A
		Y1 = -MAIN1.XCEN * SIN_A + MAIN1.YCEN * COS_A
;
;  Calculate the old and new boundaries, and see if a change needs to be made.
;
		XMIN0 = X0 - MAIN0.IXWIDTH/2  &  XMAX0 = X0 + MAIN0.IXWIDTH/2
		XMIN1 = X1 - MAIN1.IXWIDTH/2  &  XMAX1 = X1 + MAIN1.IXWIDTH/2
		YMIN0 = Y0 - MAIN0.IYWIDTH/2  &  YMAX0 = Y0 + MAIN0.IYWIDTH/2
		YMIN1 = Y1 - MAIN1.IYWIDTH/2  &  YMAX1 = Y1 + MAIN1.IYWIDTH/2
;
		LIST_EXPER, MAIN0.PROG_NUM, EXPER, N_EXPER
		IF N_EXPER GT 0 THEN BEGIN
		    XX0 =  EXPER.XCEN * COS_A + EXPER.YCEN * SIN_A
		    YY0 = -EXPER.XCEN * SIN_A + EXPER.YCEN * COS_A
		    XMIN1 = XMIN1 < MIN(XX0 - EXPER.IXWIDTH/2)
		    XMAX1 = XMAX1 > MAX(XX0 + EXPER.IXWIDTH/2)
		    YMIN1 = YMIN1 < MIN(YY0 - EXPER.IYWIDTH/2)
		    YMAX1 = YMAX1 > MAX(YY0 + EXPER.IYWIDTH/2)
		ENDIF
		MODIFY_BOX = 0
		IF (XMIN1 NE XMIN0) OR (XMAX1 NE XMAX0) THEN BEGIN
			MODIFY_BOX = 1
			XMIN = XMIN1
			XMAX = XMAX1
			MAIN1.IXWIDTH = XMAX - XMIN
			X0 = (XMIN + XMAX) / 2
		ENDIF ELSE BEGIN
			MAIN1.XCEN    = MAIN0.XCEN
			MAIN1.IXWIDTH = MAIN0.IXWIDTH
		ENDELSE
		IF (YMIN1 NE YMIN0) OR (YMAX1 NE YMAX0) THEN BEGIN
			MODIFY_BOX = 1
			YMIN = YMIN1
			YMAX = YMAX1
			MAIN1.IYWIDTH = YMAX - YMIN
			Y0 = (YMIN + YMAX) / 2
		ENDIF ELSE BEGIN
			MAIN1.YCEN    = MAIN0.YCEN
			MAIN1.IYWIDTH = MAIN0.IYWIDTH
		ENDELSE
;
;  If the box needs to be modified, then calculate the new X and Y positions in
;  solar coordinates.
;
		IF MODIFY_BOX THEN BEGIN
			MODIFY = 1
			MAIN1.XCEN = X0 * COS_A - Y0 * SIN_A
			MAIN1.YCEN = X0 * SIN_A + Y0 * COS_A
		ENDIF
;
;  Use the smallest range of times which includes all the values.
;
		IF MAIN0.DATE_OBS GT MAIN1.DATE_OBS THEN MODIFY = 1 ELSE $
			MAIN1.DATE_OBS = MAIN0.DATE_OBS
		IF MAIN0.DATE_END LT MAIN1.DATE_END THEN MODIFY = 1 ELSE $
			MAIN1.DATE_END = MAIN0.DATE_END
;
		IF MAIN0.OBT_TIME GT MAIN1.OBT_TIME THEN MODIFY = 1 ELSE $
			MAIN1.OBT_TIME = MAIN0.OBT_TIME
		IF MAIN0.OBT_END  LT MAIN1.OBT_END  THEN MODIFY = 1 ELSE $
			MAIN1.OBT_END  = MAIN0.OBT_END
;
;  Use the smallest range of sequence numbers which includes all the values.
;
		IF MAIN0.SEQ_FROM GT MAIN1.SEQ_FROM THEN MODIFY = 1 ELSE $
			MAIN1.SEQ_FROM = MAIN0.SEQ_FROM
		IF MAIN0.SEQ_TO LT MAIN1.SEQ_TO THEN MODIFY = 1 ELSE	$
			MAIN1.SEQ_TO = MAIN0.SEQ_TO
;
;  Look at the old entry to determine if there are any comments or not.
;
		MAIN1.COMMENTS = MAIN0.COMMENTS
;
;  If MODIFY was set, then call MOD_MAIN.
;
		IF MODIFY THEN BEGIN
			print,'Calling MOD_MAIN'
			IF MOD_MAIN(MAIN1, ERRMSG=MESSAGE) LT 0 THEN	$
				GOTO, HANDLE_ERROR
		ENDIF
	ENDELSE
;
;  Extract the information for the EXPERIMENT database from the header.  Some
;  information does not yet appear in the header.
;
	OBS_SEQ = FXPAR(HEADER, 'OBS_SEQ')
	IF DATATYPE(OBS_SEQ,1) NE 'String' THEN OBS_SEQ = ''
	OBS_MODE = FXPAR(HEADER, 'OBS_MODE')
	IF DATATYPE(OBS_MODE,1) NE 'String' THEN OBS_MODE = ''
;
	EXP1 = {PROG_NUM: FXPAR(HEADER, 'PROG_NUM'),		$
		SEQ_NUM: FXPAR(HEADER, 'SEQ_NUM'),		$
		OBS_SEQ: OBS_SEQ,				$
		COUNT: 1,					$
		SEQ_IND: FIX(FXPAR(HEADER, 'SEQ_IND')),		$
		RAS_ID: FIX(FXPAR(HEADER, 'RAS_ID')),		$
		RAS_VAR: FIX(FXPAR(HEADER, 'RAS_VAR')),		$
		EXPTIME: FLOAT(FXPAR(HEADER, 'EXPTIME')),	$
		OBS_MODE: OBS_MODE,				$
		DW_ID: FIX(FXPAR(HEADER, 'DW_ID')),		$
		DATE_OBS: UTC2TAI(FXPAR(HEADER, 'DATE_OBS')),	$
		DATE_END: UTC2TAI(FXPAR(HEADER, 'DATE_END')),	$
		OBT_TIME: DOUBLE(FXPAR(HEADER, 'OBT_TIME')),	$
		OBT_END: DOUBLE(FXPAR(HEADER, 'OBT_END')),	$
		XCEN: FLOAT(FXPAR(HEADER, 'XCEN')),		$
		YCEN: FLOAT(FXPAR(HEADER, 'YCEN')),		$
		ANGLE: FLOAT(FXPAR(HEADER, 'ANGLE')),		$
		IXWIDTH: FLOAT(FXPAR(HEADER, 'IXWIDTH')),	$
		IYWIDTH: FLOAT(FXPAR(HEADER, 'IYWIDTH')),	$
		INS_X0: FLOAT(FXPAR(HEADER, 'INS_X0')),		$
		INS_Y0: FLOAT(FXPAR(HEADER, 'INS_Y0')),		$
		INS_ROLL: FLOAT(FXPAR(HEADER, 'INS_ROLL')),	$
		SC_X0: FLOAT(FXPAR(HEADER, 'SC_X0')),		$
		SC_Y0: FLOAT(FXPAR(HEADER, 'SC_Y0')),		$
		SC_ROLL: FLOAT(FXPAR(HEADER, 'SC_ROLL')),	$
		WAVEMIN: FLOAT(FXPAR(HEADER, 'WAVEMIN')),	$
		WAVEMAX: FLOAT(FXPAR(HEADER, 'WAVEMAX')),	$
		TRACKING: BYTE(FXPAR(HEADER, 'TRACKING')),	$
		SER_ID: FIX(FXPAR(HEADER, 'SER_ID')),		$
		OPSLBITS: STRTRIM(FXPAR(HEADER, 'OPSLBITS'),2),	$
		OPSRBITS: STRTRIM(FXPAR(HEADER, 'OPSRBITS'),2),	$
		SLIT_POS: FIX(FXPAR(HEADER, 'SLIT_POS')),	$
		MIR_POS: FIX(FXPAR(HEADER, 'MIR_POS')),		$
		EV_ENAB: BYTE(FXPAR(HEADER, 'EV_ENAB')),	$
		COMP_ERR: BYTE(FXPAR(HEADER, 'COMP_ERR')),	$
		VDS_PMCP: FIX(FXPAR(HEADER, 'VDS_PMCP')),	$
		VDS_MODE: FIX(FXPAR(HEADER, 'VDS_MODE')),	$
		VDS_ORI: BYTE(FXPAR(HEADER, 'VDS_ORI')),	$
		VDS_ACC: BYTE(FXPAR(HEADER, 'VDS_ACC')),	$
		GSET_ID: FIX(FXPAR(HEADER, 'GSET_ID')),		$
		DETECTOR: STRMID(FXPAR(HEADER, 'DETECTOR'),0,1),$
		ZONE_ID: FIX(FXPAR(HEADER, 'ZONE_ID')),		$
		SLIT_NUM: FIX(FXPAR(HEADER, 'SLIT_NUM')),	$
		FILENAME: STRTRIM(FXPAR(HEADER, 'FILENAME'),2),	$
		SEQVALID: BYTE(FXPAR(HEADER, 'SEQVALID')),	$
		DATASRC: STRMID(FXPAR(HEADER, 'DATASRC'), 0, 1),$
		COMMENTS: 'N'}
;
;  Get the corresponding entry, if any, from the EXPERIMENT database.
;
	GET_EXPER, SEQ_NUM, EXP0
;
;  If none was found, then add a new entry into the database.
;
	IF EXP0.SEQ_NUM LT 0 THEN BEGIN
		print,'Calling ADD_EXPER'
		IF NOT ADD_EXPER(EXP1, ERRMSG=MESSAGE) THEN GOTO, HANDLE_ERROR
;
;  Otherwise, check the old entry against the new to see if anything needs to
;  be updated.
;
	END ELSE BEGIN
		MODIFY = 0
		IF TRIM(EXP0.OBS_SEQ) NE TRIM(EXP1.OBS_SEQ) THEN MODIFY = 1
		IF EXP0.COUNT NE EXP1.COUNT THEN MODIFY = 1
		IF EXP0.RAS_ID NE EXP1.RAS_ID THEN MODIFY = 1
		IF EXP0.RAS_VAR NE EXP1.RAS_VAR THEN MODIFY = 1
		IF EXP0.EXPTIME NE EXP1.EXPTIME THEN MODIFY = 1
		IF TRIM(EXP0.OBS_MODE) NE TRIM(EXP1.OBS_MODE) THEN MODIFY = 1
		IF EXP0.DW_ID NE EXP1.DW_ID THEN MODIFY = 1
		IF EXP0.DATE_OBS NE EXP1.DATE_OBS THEN MODIFY = 1
		IF EXP0.DATE_END NE EXP1.DATE_END THEN MODIFY = 1
		IF EXP0.OBT_TIME NE EXP1.OBT_TIME THEN MODIFY = 1
		IF EXP0.OBT_END NE EXP1.OBT_END THEN MODIFY = 1
		IF EXP0.XCEN NE EXP1.XCEN THEN MODIFY = 1
		IF EXP0.YCEN NE EXP1.YCEN THEN MODIFY = 1
		IF EXP0.ANGLE NE EXP1.ANGLE THEN MODIFY = 1
		IF EXP0.IXWIDTH NE EXP1.IXWIDTH THEN MODIFY = 1
		IF EXP0.IYWIDTH NE EXP1.IYWIDTH THEN MODIFY = 1
		IF EXP0.INS_X0 NE EXP1.INS_X0 THEN MODIFY = 1
		IF EXP0.INS_Y0 NE EXP1.INS_Y0 THEN MODIFY = 1
		IF EXP0.INS_ROLL NE EXP1.INS_ROLL THEN MODIFY = 1
		IF EXP0.SC_X0 NE EXP1.SC_X0 THEN MODIFY = 1
		IF EXP0.SC_Y0 NE EXP1.SC_Y0 THEN MODIFY = 1
		IF EXP0.SC_ROLL NE EXP1.SC_ROLL THEN MODIFY = 1
		IF EXP0.WAVEMIN NE EXP1.WAVEMIN THEN MODIFY = 1
		IF EXP0.WAVEMAX NE EXP1.WAVEMAX THEN MODIFY = 1
		IF EXP0.TRACKING NE EXP1.TRACKING THEN MODIFY = 1
		IF EXP0.SER_ID NE EXP1.SER_ID THEN MODIFY = 1
		IF EXP0.OPSLBITS NE EXP1.OPSLBITS THEN MODIFY = 1
		IF EXP0.OPSRBITS NE EXP1.OPSRBITS THEN MODIFY = 1
		IF EXP0.SLIT_POS NE EXP1.SLIT_POS THEN MODIFY = 1
		IF EXP0.MIR_POS NE EXP1.MIR_POS THEN MODIFY = 1
		IF EXP0.EV_ENAB NE EXP1.EV_ENAB THEN MODIFY = 1
		IF EXP0.COMP_ERR NE EXP1.COMP_ERR THEN MODIFY = 1
		IF EXP0.VDS_PMCP NE EXP1.VDS_PMCP THEN MODIFY = 1
		IF EXP0.VDS_MODE NE EXP1.VDS_MODE THEN MODIFY = 1
		IF EXP0.VDS_ORI NE EXP1.VDS_ORI THEN MODIFY = 1
		IF EXP0.VDS_ACC NE EXP1.VDS_ACC THEN MODIFY = 1
		IF EXP0.GSET_ID NE EXP1.GSET_ID THEN MODIFY = 1
		IF EXP0.DETECTOR NE EXP1.DETECTOR THEN MODIFY = 1
		IF EXP0.ZONE_ID NE EXP1.ZONE_ID THEN MODIFY = 1
		IF EXP0.SLIT_NUM NE EXP1.SLIT_NUM THEN MODIFY = 1
		IF EXP0.FILENAME NE EXP1.FILENAME THEN MODIFY = 1
		IF EXP0.SEQVALID NE EXP1.SEQVALID THEN MODIFY = 1
		IF EXP0.DATASRC NE EXP1.DATASRC THEN MODIFY = 1
;
;  Look at the old entry to determine if there are any comments or not.
;
		EXP1.COMMENTS = EXP0.COMMENTS
;
;  If MODIFY was set, then call MOD_EXPER.
;
		IF MODIFY THEN BEGIN
			print,'Calling MOD_EXPER'
			IF MOD_EXPER(EXP1, ERRMSG=MESSAGE) LT 0 THEN	$
				GOTO, HANDLE_ERROR
		ENDIF
	ENDELSE
	GOTO, FINISH
;
;  Error handling point.
;
HANDLE_ERROR:
	IF N_ELEMENTS(ERRMSG) NE 0 THEN ERRMSG = 'INS_CATALOG: ' + MESSAGE $
		ELSE CDS_MESSAGE, MESSAGE, /CONTINUE
;
FINISH:
	RETURN
	END
