	FUNCTION MATCH_PLAN, RASTER, PLAN, RASTER_DEF
;+
; Project     :	SOHO - CDS
;
; Name        :	MATCH_PLAN()
;
; Purpose     :	Matches telemetry against the science plan.
;
; Explanation :	Determines whether or not a raster from the CDS telemetry
;		matches an entry in the science plan.
;
; Use         :	Result = MATCH_PLAN( RASTER, PLAN, RASTER_DEF )
;
;		IF MATCH_PLAN( RASTER, PLAN, RASTER_DEF ) THEN ...
;
; Inputs      :	RASTER	= Structure variable from READ_RASTER
;		PLAN	= Structure variable from GET_DETAIL
;
; Opt. Inputs :	None.
;
; Outputs     :	RASTER_DEF = Structure variable from GET_RASTER
;
;		The result of the function is either true or false, depending
;		on whether or not the raster matches the plan entry.
;
; Opt. Outputs:	None.
;
; Keywords    :	None.
;
; Calls       :	None.
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	If the raster counter is larger than the number of rasters
;		defined for this study, then the routine will assume that
;		either the N_REPEAT_S or N_POINTINGS mechanism was used, and
;		will try each of the raster definitions in the study in turn.
;
; Category    :	Data_handling, Telemetry, I/O, FITS, CDS.
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 29 December 1994
;
; Modified    :	Version 1, William Thompson, GSFC, 27 June 1995
;		Version 2, William Thompson, GSFC, 21 February 1996
;			Changed default RASTER_DEF structure, when no match can
;			be made, to include tags RAS_VAR and LL_ID.
;		Version 3, William Thompson, GSFC, 16 April 1996
;			Return valid RASTER_DEF, even if a match cannot be
;			made.
;		Version 4, William Thompson, GSFC, 17 January 1997
;			Take into account N_REPEAT_S/N_POINTINGS to reflect
;			change in behavior of CPT.
;		Version 5, William Thompson, GSFC, 12 April 1999
;			Fix bug where GET_RASTER doesn't return anything.
;
; Version     :	Version 5, 12 April 1999
;-
;
	ON_ERROR, 2
;
;  Check the number of parameters.
;
	IF N_PARAMS() NE 3 THEN MESSAGE,	$
		'Syntax:  Result = MATCH_PLAN( RASTER, PLAN, RASTER_DEF )'
;
;  Check to see if the appropriate elements in the raster match those in the
;  science plan.  Keep checking individual items until a discrepency is found.
;
	RASTER_DEF = {RAS_ID: -1, RAS_VAR: -1, LL_ID: -1}
	SCIHEAD = RASTER.HEADER.SCI
;
;  Determine the raster ID and variation index.
;
	GET_STUDY, RASTER.STUDY_ID, RASTER.STUDYVAR, STUDY
	IF STUDY.STUDY_ID LT 0 THEN RETURN, 0
;
;  Use the N_REPEAT_R parameters to determine which raster in the study
;  definition this raster should be.  Take into account the fact that the study
;  may be repeated via the N_REPEAT_S or N_POINTINGS mechanism.
;
	I = 0
	TOT_RASTERS = 0
	REPEAT BEGIN
		TOT_RASTERS = TOT_RASTERS + STUDY.RASTERS(I).N_REPEAT_R
		I = I + 1
	ENDREP UNTIL (SCIHEAD.RASTERCOUNTER LT TOT_RASTERS) OR	$
		(I GE STUDY.N_RASTER_DEF)
	I_FOUND = I
;
;  Try the currently selected raster.
;
TRY_RASTER:
	RAS_ID  = STUDY.RASTERS(I-1).RAS_ID
	RAS_VAR = STUDY.RASTERS(I-1).RAS_VAR
;
;  Get the raster definition.  Continue checking against the information in the
;  science plan.
;
	GET_RASTER, RAS_ID, RAS_VAR, RASDEF
	IF RASDEF.RAS_ID        EQ -1              THEN GOTO, FAILED
	IF SCIHEAD.NUMMIRRORPOS	NE RASDEF.NX       THEN GOTO, FAILED
	IF SCIHEAD.NUMSLITPOS	NE RASDEF.NY       THEN GOTO, FAILED
	IF SCIHEAD.COMPRESSION	NE RASDEF.COMP_ID  THEN GOTO, FAILED
	IF SCIHEAD.COMP_OPTION	NE RASDEF.COMP_OPT THEN GOTO, FAILED
	IF SCIHEAD.SLITNUMBER	NE RASDEF.SLIT_NUM THEN GOTO, FAILED
	IF SCIHEAD.DATAWINDOWID	NE RASDEF.DW_ID	   THEN GOTO, FAILED
;
;  The exposure time in the telemetry is in units of 10^N seconds, where N can
;  be an integer from -3 to 0.
;
	IF (SCIHEAD.EXPOSURETIME       NE RASDEF.EXPTIME) AND	$
	   (SCIHEAD.EXPOSURETIME*0.1   NE RASDEF.EXPTIME) AND	$
	   (SCIHEAD.EXPOSURETIME*0.01  NE RASDEF.EXPTIME) AND	$
	   (SCIHEAD.EXPOSURETIME*0.001 NE RASDEF.EXPTIME) THEN GOTO, FAILED
;
;  The raster appears to check out.  Check it against the plan.
;
	RASTER_DEF  = RASDEF
	IF RASTER.STUDY_ID NE PLAN.STUDY_ID THEN GOTO, FAILED
	IF RASTER.STUDYVAR NE PLAN.STUDYVAR THEN GOTO, FAILED
;
;  If we've got this far, then the match must be successful.
;
	RETURN, 1
;
;  If we've failed to find a match, but we've reached the last raster in the
;  study, then maybe the study has started to repeat itself via the N_REPEAT_S
;  or N_POINTINGS mechanism.  Try looking at the other rasters in the study
;  definition.
;
FAILED:
	IF (I_FOUND GE STUDY.N_RASTER_DEF) AND (I GT 1) THEN BEGIN
		I = I - 1
		GOTO, TRY_RASTER
	ENDIF
;
;  Otherwise, signal failure by returning 0.
;
	RETURN, 0
	END
