	PRO MECH_POINT, SLITPOS, MIRRORPOS, SOLAR_X, SOLAR_Y
;+
; Project     :	SOHO - CDS
;
; Name        :	MECH_POINT
;
; Purpose     :	Calculate pointing mods from slit and mirror mechanisms
;
; Explanation :	Modifies the pointing based on the slit and scan mirror
;		mechanism position data in the telemetry.
;
;		The formulas used to calculate the pointing are based on
;		equations 17 and 18 in "Notes on Pointing Calculation in CDS",
;		Version 2.0, MKC, 1/3/94.
;
; Use         :	MECH_POINT, SLITPOS, MIRRORPOS, SOLAR_X, SOLAR_Y
;
; Inputs      :	SLITPOS	  = Slit position word from the telemetry.
;		MIRRORPOS = Mirror position byte from the telemetry.
;		SOLAR_X	  = Pointing in the X direction from either the OPS
;			    data or from the sun sensor data.
;		SOLAR_Y	  = Pointing in the Y direction.
;
; Opt. Inputs :	None.
;
; Outputs     :	SOLAR_X	  = Modified pointing in the X direction with the scan
;			    mirror position taken into account.
;		SOLAR_Y	  = Modified pointing in the Y direction with the slit
;			    position taken into account.
;
; Opt. Outputs:	None.
;
; Keywords    :	None.
;
; Calls       :	SLITPOS_DECODE
;
; Common      :	None.
;
; Restrictions:	Either OPS_POINT or SENSOR_POINT should be called before this
;		to calculate the basic pointing of CDS.
;
;		This routine should be upgraded to incorporate information
;		about the spacecraft roll.
;
; Side effects:	None.
;
; Category    :	Data handling, telemetry.
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 29 August 1994
;
; Modified    :	Version 1, William Thompson, GSFC, 29 August 1994
;		Version 2, William Thompson, GSFC, 4 March 1996
;			Correct for behavior of slit mechanism.
;
; Version     :	Version 2, 4 March 1996
;-
;
	ON_ERROR,2
;
;  Check the number of arguments.
;
	IF N_PARAMS() NE 4 THEN MESSAGE,	$
		'Syntax:  MECH_POINT, SLITPOS, MIRRORPOS, SOLAR_X, SOLAR_Y'
;
;  The following constants are used in the calculation.
;
	SLITSTEP = 1.016	;Slit step size in arcsec
	MIRSTEP	 = 2.032	;Mirror step size in arcsec
;
;  Calculate the updated pointing in arcsec.  Take into account that the scan
;  mirror moves backwards.
;
	SLITPOS_DECODE, SLITPOS, BITS, S_POS
	SOLAR_X  = SOLAR_X - (MIRRORPOS - '80'X)*MIRSTEP
	SOLAR_Y  = SOLAR_Y + S_POS*SLITSTEP
;
	RETURN
	END
