	PRO MERGE_FITS_DATA, CHECK=CHECK
;+
; Project     :	SOHO - CDS
;
; Name        :	MERGE_FITS_DATA
;
; Purpose     :	Selects most recent version of CDS FITS files.
;
; Category    :	Class3, Operations, CDS
;
; Explanation :	This routine looks in the directories given by the environment
;		variable CDS_FITS_DATA and looks for any duplicate files.  If
;		it finds any, then it opens them up to see which is the most
;		recent version, based on the DATE keyword in the FITS header.
;		The file with the most recent date is then moved to replace the
;		older file, so that only one version remains.
;
; Syntax      :	MERGE_FITS_DATA  [, /CHECK ]
;
; Examples    :	
;
; Inputs      :	None.
;
; Opt. Inputs :	None.
;
; Outputs     :	None.
;
; Opt. Outputs:	None.
;
; Keywords    :	CHECK = If set, then the program will print out the necessary
;			commands, but will not execute them.
;
; Calls       :	BREAK_PATH, BREAK_FILE, FXHREAD, STR2UTC, UTC2TAI
;
; Common      :	None.
;
; Restrictions:	Can only be run from the master account
;
; Side effects:	None.
;
; Prev. Hist. :	None.
;
; History     :	Version 1, 24-May-1996, William Thompson, GSFC
;		Version 2, 29-May-1996, William Thompson, GSFC
;			Fixed bug
;		Version 3, 24-Jan-1997, William Thompson, GSFC
;			Added keyword CHECK
;
; Contact     :	WTHOMPSON
;-
;
	ON_ERROR, 2
;
;  Find the names of all the FITS files in the various paths.  Ignore any
;  directories on the soho archive system.
;
	PATHS = BREAK_PATH(GETENV('CDS_FITS_DATA'), /NOCURRENT)
	FOR I_PATH = 0,N_ELEMENTS(PATHS)-1 DO BEGIN
	    IF STRMID(PATHS(I_PATH),0,5) NE '/soho' THEN BEGIN
		TEMP = FIND_FILES('*.fits',PATHS(I_PATH))
		IF TEMP(0) NE '' THEN BEGIN
		    IF I_PATH EQ 0 THEN FILES = TEMP ELSE FILES = [FILES, TEMP]
		ENDIF
	    ENDIF
	ENDFOR
;
;  Break the filenames apart into their various components.  Sort by name.
;
	BREAK_FILE, FILES, DISKS, DIRS, NAMES
	S = SORT(NAMES)
	FILES = FILES(S)
	NAMES = NAMES(S)
;
;  If the CHECK keyword was set, then print a message reminding the user that
;  the commands are not actually being executed.
;
	IF KEYWORD_SET(CHECK) THEN BEGIN
	    PRINT, 'Call without /CHECK to execute the following commands:'
	    PRINT, ''
	ENDIF
;
;  Step through each file.
;
	I_NAME = 0L
	LAST_NAME = ''
	LAST_FILE = ''
	WHILE I_NAME LT N_ELEMENTS(NAMES) DO BEGIN
;
;  If a duplicate filename is detected, then extract the dates from the FITS
;  headers.
;
	    IF NAMES(I_NAME) EQ LAST_NAME THEN BEGIN
		GET_LUN, UNIT
		OPENR, UNIT, FILES(I_NAME)
		FXHREAD, UNIT, HEADER
		DATE = UTC2TAI(STR2UTC(FXPAR(HEADER, 'DATE'), /DMY))
		CLOSE, UNIT
;
		OPENR, UNIT, LAST_FILE
		FXHREAD, UNIT, HEADER
		LAST_DATE = UTC2TAI(STR2UTC(FXPAR(HEADER, 'DATE'), /DMY))
		FREE_LUN, UNIT
;
;  Move the newer file on top of the older file.  Make sure that LAST_NAME and
;  LAST_FILE point to the newer file.
;
		IF DATE GT LAST_DATE THEN BEGIN
		    COMMAND = 'mv ' + FILES(I_NAME) + ' ' + LAST_FILE
		    PRINT, COMMAND
		    IF NOT KEYWORD_SET(CHECK) THEN SPAWN, COMMAND
		END ELSE BEGIN
		    COMMAND = 'mv ' + LAST_FILE + ' ' + FILES(I_NAME)
		    PRINT, COMMAND
		    IF NOT KEYWORD_SET(CHECK) THEN SPAWN, COMMAND
		    LAST_NAME = NAMES(I_NAME)
		    LAST_FILE = FILES(I_NAME)
		ENDELSE
	    END ELSE BEGIN
		LAST_NAME = NAMES(I_NAME)
		LAST_FILE = FILES(I_NAME)
	    ENDELSE
	    I_NAME = I_NAME + 1
	ENDWHILE
;
	RETURN
	END
