	FUNCTION OPEN_TM_FILE, UNIT, FILENAME, TEL_MODE
;+
; Project     :	SOHO - CDS
;
; Name        :	OPEN_TM_FILE()
;
; Purpose     :	Opens a stored telemetry file.
;
; Explanation :	This routine opens a stored telemetry file.  For realtime
;		telemetry files stored by the CDS EGSE system (the "tm."
;		files), this is straightforward since the files are just raw
;		dumps of the telemetry packets.  For tape recorder dumps and
;		realtime telemetry archives from the ECS, the header must be
;		read, and the total number of telemetry packets in the file
;		determined.
;
; Use         :	Result = OPEN_TM_FILE( UNIT, FILENAME, TEL_MODE )
;
;		IF NOT OPEN_TM_FILE( UNIT, FILENAME, TEL_MODE ) THEN ...
;
; Inputs      :	UNIT	 = The logical unit number to use when opening the
;			   file.
;		FILENAME = The name of the telemetry file to open.
;
; Opt. Inputs :	None.
;
; Outputs     :	TEL_MODE = The telemetry mode to use for the file.  If equal to
;			   1 for realtime, then it unchanged.  Otherwise, it is
;			   based on the filename extension, using the following
;			   list:
;
;				Ext.	Mode	Meaning
;
;				.QKL	2	Tape recorder dump
;				.REL	3	RT telemetry stored by ECS
;				.DAT1	4	Final distribution
;				.D01	4	  "       "
;
;			   Otherwise, mode 0, replay stored telemetry, is
;			   assumed.
;
;		Also, the result of the function is a status code, with 0 for
;		unsuccessful and 1 for successful.
;
; Opt. Outputs:	None.
;
; Keywords    :	None.
;
; Calls       :	None.
;
; Common      :	FITSGEN from the include file "fitsgen.cmn".
;
; Restrictions:	Will only work if the telemetry file is properly named.
;
; Side effects:	None.
;
; Category    :	Data Handling, Telemetry, Science.
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 3 February 1995
;
; Modified    :	Version 1, William Thompson, GSFC, 3 February 1995
;		Version 2, William Thompson, GSFC, 6 February 1995
;			Incorporated final distribution data format.
;		Version 3, William Thompson, GSFC, 28 March 1995
;			Make sure that NUM_PACK and I_PACK are always long.
;		Version 4, William Thompson, GSFC, 17 April 1995
;			Make detection of final distribution telemetry files
;			more robust.
;		Version 5, William Thompson, GSFC, 23 August 1995
;			Fixed bug introduced in version 4
;		Version 6, William Thompson, GSFC, 30 January 1996
;			Recognize CDROM filenames which end in .D01.
;		Version 7, William Thompson, GSFC, 26 February 1996
;			Added routine name to error message.
;
; Version     :	Version 7, 26 February 1996
;-
;
	ON_ERROR, 2
@fitsgen.cmn
;
;  Check the number of parameters.
;
	IF N_PARAMS() NE 3 THEN MESSAGE,	$
		'Syntax: Result = OPEN_TM_FILE( UNIT, FILENAME, TEL_MODE )'
;
;  Try to open the file.
;
	CLOSE, UNIT
	OPENR, UNIT, FILENAME, ERROR=IOERR
	IF IOERR NE 0 THEN BEGIN
		CDS_MESSAGE, 'OPEN_TM_FILE: ' + !ERR_STRING, /CONTINUE
		RETURN, 0
	ENDIF
;
;  Determine what telemetry mode to use for the file.
;
	IF N_ELEMENTS(TEL_MODE) EQ 0 THEN TEL_MODE = 0
	IF TEL_MODE NE 1 THEN BEGIN
		BREAK_FILE, FILENAME, DISK, DIR, RFILE, REXT
		CASE STRUPCASE(STRMID(REXT,0,4)) OF
			'.QKL':  TEL_MODE = 2
			'.REL':  TEL_MODE = 3
			'.DAT':	 TEL_MODE = 4
			'.D01':	 TEL_MODE = 4
			ELSE:	 TEL_MODE = 0
		ENDCASE
	ENDIF
;
;  If the telemetry mode is either 2 or 3, then read in the ASCII header and
;  determine the number of packets in the file.
;
	IF (TEL_MODE EQ 2) OR (TEL_MODE EQ 3) THEN BEGIN
		HEADER = ''
		LINE = ''
		WHILE STRTRIM(STRUPCASE(LINE),2) NE 'END' DO BEGIN
			READF,UNIT,LINE
			HEADER = [HEADER,LINE]
		ENDWHILE
		NUM_PACK = LONG(FXPAR(HEADER,'NUM_PACK'))
;
;  If the telemetry mode is 4, then read in the binary header.  Again,
;  determine the number of packets in the file.  Most of the information in the
;  header is ignored.
;
	END ELSE IF TEL_MODE EQ 4 THEN BEGIN
		SC_ID = 0
		SC_CLOCK = INTARR(5,2)
		NUM_PACK = 0L
		PR_OPT = 0B
		DT_FLAG = 0B
		ON_CLOCK = INTARR(5)
		SEL_OPT = 0B
		N_GROUPS = 0B
		READU, UNIT, SC_ID, SC_CLOCK, NUM_PACK, PR_OPT, DT_FLAG, $
			ON_CLOCK, SEL_OPT, N_GROUPS
		IEEE_TO_HOST, NUM_PACK
;
		APID = INTARR(N_GROUPS)
		DUMMY = 0B
		N_QAC = 0B
		READU, UNIT, APID, DUMMY, N_QAC
;
		QAC_OFFSET = LONARR(N_QAC)
		READU, UNIT, QAC_OFFSET
	ENDIF
;
;  Set the number of packets read in to 0, and return 1 for success.
;
	I_PACK = 0L
	RETURN, 1
	END
