	PRO READ_EXPOSURE, TEL_MODE, UNIT, EXPOSURE, PACKET, RASTER, DATAWIN
;+
; Project     : SOHO - CDS
;
; Name        : READ_EXPOSURE
;
; Purpose     : Reads the packets making up a NIS or GIS exposure.
;
; Explanation : This procedure keeps reading and processing CDS science
;		telemetry packets until it encounters one that belongs to
;		a different exposure or raster.
;
; Use         : READ_EXPOSURE, TEL_MODE, UNIT, EXPOSURE, PACKET, RASTER, DATAWIN
;
; Inputs      : TEL_MODE = Telemetry input mode, as used by READ_PACKET.
;		UNIT	 = Unit number telemetry input was opened with.
;		EXPOSURE = Exposure number to read in.
;		PACKET	 = Last packet read in.
;		RASTER	 = Raster structure from FORM_RASTER.
;		DATAWIN	 = Data window array from FORM_RASTER.
;
; Opt. Inputs : None.
;
; Outputs     : PACKET	 = The last packet read in by this routine is returned
;			   in this parameter.  This is the next science packet
;			   after the end of the current exposure.
;		RASTER	 = The exposure header information and the science data
;			   stream are stored within the raster structure.
;
; Opt. Outputs: None.
;
; Keywords    : None.
;
; Calls       : DECOMP_SCIENCE, GET_SCI_PKT, LOAD_DATA, LOAD_EXPOSURE,
;		LOAD_SCIENCE, TEST_CURR_EXP
;
; Common      : None.
;
; Restrictions: None.
;
; Side effects: None.
;
; Category    : Data Handling, Telemetry.
;
; Prev. Hist. : Although there was a previous routine with this name, this is a
;		completely new routine.
;
; Written     : William Thompson, GSFC, 16 April 1993.
;
; Modified    : Version 1, William Thompson, GSFC, 20 April 1993.
;		Version 2, William Thompson, GSFC, 16 November 1993.
;			Split off DATAWIN from RASTER structure.
;		Version 3, William Thompson, GSFC, 23 June 1995
;			Modified to react to last packet bit in exposure
;			records
;		Version 4, William Thompson, GSFC, 24-Oct-1997
;			If the exposure ends at precisely the end of the file,
;			then open the next file.
;
; Version     : Version 4, 24-Oct-1997
;-
;
	ON_ERROR, 2
;
;  Check the number of parameters.
;
	IF N_PARAMS() NE 6 THEN MESSAGE,	$
		'Syntax:  READ_EXPOSURE, TEL_MODE, UNIT, EXPOSURE, ' +	$
		'PACKET, RASTER, DATAWIN'
;
;  Skip past any telemetry packets that are not directly related to the CDS
;  science data, e.g. engineering packets, etc.  If none are found, then
;  return.
;
	IF GET_SCI_PKT(TEL_MODE,UNIT,PACKET) NE 1 THEN RETURN
;
;  Initialize the variables used in the loop below.  Make the arrays big enough
;  to *probably* contain all the data, to speed the processing, although they
;  will be able to increase in size if necessary (see LOAD_SCIENCE).
;
	COUNTER = -1
	SCIENCE = BYTARR(2*RASTER.N_DATA)
	VALID   = BYTARR(2*RASTER.N_DATA)
;
;  Test to see if the packet is part of the current exposure.  Keep doing this
;  for each subsequent packet, until the last packet in the exposure is
;  reached, or another exposure is found.
;
	DONE = 0
	WHILE TEST_CURR_EXP( RASTER, PACKET, EXPOSURE, COUNTER)	AND NOT DONE $
			DO BEGIN
;
;  If the packet is the exposure header, then load this into the raster
;  structure.
;
		IF (PACKET.HDR.ID EQ '11'X) OR (PACKET.HDR.ID EQ '12'X) THEN $
			LOAD_EXPOSURE, RASTER, PACKET
;
;  Load the science data into temporary arrays.
;
		LOAD_SCIENCE, PACKET, SCIENCE, VALID
;
;  Test to see if the packet was the last one in the exposure, by checking to
;  see if the most significant bit in the packetcounter parameter is set.  If
;  it is, then read the next packet, whether science or not.  If no packets are
;  available, then exit the loop.
;
		IF (PACKET.PKT.PACKETCOUNTER AND '8000'X) NE 0 THEN BEGIN
			DONE = 1
			TEST_TIME = 0
READ_PACKET:
			TEST = READ_PACKET(TEL_MODE,UNIT,PACKET)
;
;  If one is at the end of the file, then open the next file.  Keep reading
;  packets until a packet with a higher time is found.
;
			IF TEST EQ 0 THEN BEGIN
			    IF NOT TEST_TIME THEN	$
				    LAST_TIME = OBT2TAI(PACKET.HDR.TIME)
			    TEST = OPEN_NEXT( TEL_MODE, UNIT )
			    IF TEST THEN GOTO, READ_PACKET
			ENDIF
;
			IF TEST NE 1 THEN GOTO, CONTINUE
;
;  Make sure, if a new file was opened, that the packet has a higher time than
;  the previously used packet.
;
			IF TEST_TIME THEN	$
				IF OBT2TAI(PACKET.HDR.TIME) LE LAST_TIME THEN $
					GOTO, READ_PACKET
;
;  Otherwise, look for the next science packet.  If an error occurs, or no
;  packets are available, then exit the loop.
;
		END ELSE BEGIN
			IF GET_SCI_PKT(TEL_MODE,UNIT,PACKET,/NEXT) NE 1 THEN $
				GOTO, CONTINUE
		ENDELSE
	ENDWHILE
;
;  Decompress the science data.
;
CONTINUE:
	DECOMP_SCIENCE, RASTER, SCIENCE, VALID, DATA
;
;  Load the data in the proper places in the raster structure.
;
	LOAD_DATA, RASTER, DATAWIN, EXPOSURE, DATA
;
	RETURN
	END
