	PRO REPAIR_SC_Y0, FILENAME
;+
; Project     :	SOHO - CDS
;
; Name        :	REPAIR_SC_Y0
;
; Purpose     :	Repairs the values of SC_X0, SC_Y0 in FITS headers
;
; Category    :	
;
; Explanation :	The headers are read in, and the values of SC_X0 and SC_Y0 are
;		compared against those from 
;
; Syntax      :	REPAIR_SC_Y0, FILENAME
;
; Examples    :	REPAIR_SC_Y0, 's3169r00.fits'
;
; Inputs      :	FILENAME = The name of the file to repair.  The complete
;			   filename must be given, including path if different
;			   from the current directory.  This routine will not
;			   search for it in $CDS_FITS_DATA.
;
; Opt. Inputs :	None.
;
; Outputs     :	The FITS file headers are modified.
;
; Opt. Outputs:	None.
;
; Keywords    :	None.
;
; Calls       :	FXBOPEN, FXPAR, READCDSCOL, FXBCLOSE, FXBTDIM, BASE_INTERP,
;		ASK, INS_CATALOG
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Prev. Hist. :	None.
;
; History     :	Version 1, 08-Jul-1996, William Thompson, GSFC
;		Version 2, 23-Jul-1996, William Thompson, GSFC
;			Add call to FXBCLOSE
;			Corrected problem with CASE statement
;		Version 3, 20-Aug-1996, William Thompson, GSFC
;			Handle TRVAL more robustly.
;
; Contact     :	WTHOMPSON
;-
;
	ON_ERROR, 2
;
;  Check the number of parameters.
;
	IF N_PARAMS() NE 1 THEN MESSAGE, 'Syntax:  REPAIR_SC_Y0, FILENAME'
;
;  Make sure that !PRIV is set correctly.
;
	IF !PRIV LT 2 THEN MESSAGE,	$
		'!PRIV must be at least 2 to run this routine'
;
;  Open the FITS file, and read in the date and spacecraft pointings.
;
	FXBOPEN, UNIT, FILENAME, 1, HEADER
	FXBCLOSE, UNIT
	DATE_OBS= FXPAR(HEADER, 'DATE_OBS')
	XCEN    = FXPAR(HEADER, 'XCEN')
	YCEN    = FXPAR(HEADER, 'YCEN')
	SC_X0   = FXPAR(HEADER, 'SC_X0')
	SC_Y0   = FXPAR(HEADER, 'SC_Y0')
;
;  Get the spacecraft pointing from the attitude file.  If it agrees with the
;  values in the header, then no corrections need to be made.  The filter is
;  fairly large--we're looking for big discrepencies here, on the order of 3.3
;  arcminutes.
;
	SC_POINT = GET_SC_POINT( DATE_OBS )
	IF (ABS(SC_POINT.SC_X0 - SC_X0) LE 5) AND	$
		(ABS(SC_POINT.SC_Y0 - SC_Y0) LE 5) THEN BEGIN
	    MESSAGE, /INFORMATIONAL, 'Correction too small -- not applying'
	    RETURN
	ENDIF
;
;  Calculate the correction factors.
;
	XCOR = SC_POINT.SC_X0 - SC_X0
	YCOR = SC_POINT.SC_Y0 - SC_Y0
	PRINT, 'Applying corrections ' + TRIM(XCOR) + ',' + TRIM(YCOR) + $
		' to file ' + TRIM(FILENAME)
;
;  Reopen the file for update.
;
	OPENU, UNIT, FILENAME, /GET_LUN
;
;  Read in the main header for the FITS file.
;
	POINT_LUN, UNIT, 0
	FXHREAD, UNIT, HEADER
;
;  Correct the values of SC_X0, SC_Y0, XCEN, YCEN in the header.
;
	FXADDPAR, HEADER, 'SC_X0', SC_POINT.SC_X0, FORMAT='(F7.1)'
	FXADDPAR, HEADER, 'SC_Y0', SC_POINT.SC_Y0, FORMAT='(F7.1)'
	FXADDPAR, HEADER, 'XCEN',  XCEN + XCOR,    FORMAT='(F7.1)'
	FXADDPAR, HEADER, 'YCEN',  YCEN + YCOR,    FORMAT='(F7.1)'
;
;  Write it back out again, and insert it in the catalog.
;
	POINT_LUN, UNIT, 0
	WRITEU, UNIT, HEADER
	INS_CATALOG, HEADER
;
;  Save the current file pointer position, and read in the binary table header.
;
	POINT_LUN, UNIT, 0
	FXHREAD, UNIT, HEADER
	POINT_LUN, -UNIT, POS
	FXHREAD, UNIT, HEADER
;
;  Correct the values of SC_X0, SC_Y0, XCEN, YCEN in the header.
;
	FXADDPAR, HEADER, 'SC_X0', SC_POINT.SC_X0, FORMAT='(F7.1)'
	FXADDPAR, HEADER, 'SC_Y0', SC_POINT.SC_Y0, FORMAT='(F7.1)'
	FXADDPAR, HEADER, 'XCEN',  XCEN + XCOR,    FORMAT='(F7.1)'
	FXADDPAR, HEADER, 'YCEN',  YCEN + YCOR,    FORMAT='(F7.1)'
;
;  Get the number of columns.
;
	N_COLUMNS = FXPAR(HEADER, 'TFIELDS')
;
;  For each column, extract the starting values, and the dimension labels.
;
	FOR I_COL = 1, N_COLUMNS DO BEGIN
	    TRVAL = FXBTDIM(FXPAR(HEADER, 'TRVAL' + TRIM(I_COL), COUNT=COUNT))
	    TDESC = FXBTDIM(FXPAR(HEADER, 'TDESC' + TRIM(I_COL)))
	    IF COUNT GT 0 THEN BEGIN
;
;  Replace the TRVALn line in the header.
;
		VALUE = TRVAL
		IF N_ELEMENTS(VALUE) GT 1 THEN TRVAL = '(' ELSE TRVAL = ''
;
		FOR I = 0,N_ELEMENTS(VALUE)-1 DO BEGIN
			IF I GT 0 THEN BEGIN
				TRVAL = TRVAL + ','
			ENDIF
;
			CASE TDESC(I) OF
			    'SOLAR_X':  TRVAL = TRVAL + TRIM(VALUE(I) + XCOR)
			    'SOLAR_Y':  TRVAL = TRVAL + TRIM(VALUE(I) + YCOR)
			    ELSE:	TRVAL = TRVAL + TRIM(VALUE(I))
			ENDCASE
		ENDFOR
;
		IF N_ELEMENTS(VALUE) GT 1 THEN TRVAL = TRVAL + ')'
		FXADDPAR, HEADER, 'TRVAL' + TRIM(I_COL), TRVAL
	    ENDIF
	ENDFOR
;
;  Write out the modified header.
;
	POINT_LUN, UNIT, POS
	WRITEU, UNIT, HEADER
;
;  Close the file and return.
;
	FREE_LUN, UNIT
	RETURN
	END
