	PRO REPAIR_XYCEN, FILENAME
;+
; Project     :	SOHO - CDS
;
; Name        :	REPAIR_XYCEN
;
; Purpose     :	Repairs the value of XCEN, YCEN in FITS headers
;
; Category    :	
;
; Explanation :	The mirror and slit positions are read in from the FITS file,
;		and the header is modified to correct XCEN, YCEN and the
;		various TRVALn keywords, to take into account the change in the
;		way that the mechanisms are treated.
;
; Syntax      :	REPAIR_XYCEN, FILENAME
;
; Examples    :	REPAIR_XYCEN, 's3169r00.fits'
;
; Inputs      :	FILENAME = The name of the file to repair.  The complete
;			   filename must be given, including path if different
;			   from the current directory.  This routine will not
;			   search for it in $CDS_FITS_DATA.
;
; Opt. Inputs :	None.
;
; Outputs     :	The FITS file headers are modified.
;
; Opt. Outputs:	None.
;
; Keywords    :	None.
;
; Calls       :	FXBOPEN, FXPAR, READCDSCOL, FXBCLOSE, FXBTDIM, BASE_INTERP,
;		ASK, INS_CATALOG
;
; Common      :	None.
;
; Restrictions:	Can only be applied to a FITS file once.  Applying twice will
;		then put erroneous results in the FITS file.
;
; Side effects:	None, unless applied more than once.
;
; Prev. Hist. :	None.
;
; History     :	Version 1, 18-Jun-1996, William Thompson, GSFC
;		Version 2, 08-Jul-1996, William Thompson, GSFC
;			Corrected bugs involving MIR_POS.AXES, MECH_POINT, and
;			INSTRUMENT.  Check for !PRIV
;		Version 3, 20-Aug-1996, William Thompson, GSFC
;			Handle TRVAL more robustly.
;
; Contact     :	WTHOMPSON
;-
;
	ON_ERROR, 2
;
;  Check the number of parameters.
;
	IF N_PARAMS() NE 1 THEN MESSAGE, 'Syntax:  REPAIR_XYCEN, FILENAME'
;
;  Make sure that !PRIV is set correctly.
;
	IF !PRIV LT 2 THEN MESSAGE,	$
		'!PRIV must be at least 2 to run this routine'
;
;  Open the FITS file, and determine the raster dimensions.
;
	FXBOPEN, UNIT, FILENAME, 1, HEADER
	NX = FXPAR(HEADER, 'NX')
	NY = FXPAR(HEADER, 'NY')
	MIRRORSTEP = FXPAR(HEADER, 'XSTEP')
	SLITSTEP   = FXPAR(HEADER, 'YSTEP')
	INSTRUMENT = TRIM(FXPAR(HEADER, 'DETECTOR'))
;
;  Read in the mirror and slit positions.  Reverse them in X to return them to
;  the original orientation.
;
	READCDSCOL, UNIT, MIR_POS, 'MIR_POS'
	READCDSCOL, UNIT, SLIT_POS, 'SLIT_POS'
	FXBCLOSE, UNIT
	IF (MIR_POS.AXES)(0) EQ 'SOLAR_X' THEN BEGIN
		MIR_POS.ARRAY  = REVERSE(MIR_POS.ARRAY,  1)
		SLIT_POS.ARRAY = REVERSE(SLIT_POS.ARRAY, 1)
	ENDIF
	HEADERFOUND = MIR_POS.ARRAY NE MIR_POS.MISSING
;
;  Start off with the old algorithm, and calculate the center based on
;  INS_X0=0, INS_Y0=0.  Get the initial pointing information, either from the
;  first exposure header if it exists, or from the raster header.
;
	IF HEADERFOUND(0) THEN BEGIN
		SLITPOSITION   = (SLIT_POS.ARRAY)(0)
		MIRRORPOSITION = (MIR_POS.ARRAY)(0)
	END ELSE BEGIN
		SLITPOSITION   = SLIT_POS.ARRAY
		MIRRORPOSITION = MIR_POS.ARRAY
	ENDELSE
;
;  Modify the pointing based on the slit and mirror positions.
;
	XCEN = 0.
	YCEN = 0.
	MECH_POINT, SLITPOSITION, MIRRORPOSITION, XCEN, YCEN
;
;  For the GIS detector, calculate the effect of the slit movement.
;
	IF INSTRUMENT EQ 'GIS' THEN BEGIN
		RYCEN = (NY - 1.) * SLITSTEP / 2
	END ELSE RYCEN = 0
;
;  Modify the pointing based on the detector information.  Note that the scan
;  mirror moves backwards.
;
	XCEN = XCEN - (NX - 1.) * MIRRORSTEP / 2
	YCEN = YCEN + RYCEN
;
;  Reverse the values of XCEN and YCEN, and then use the new algorithm.  The
;  result will be the correction factor that needs to be applied.
;
	XCEN = -XCEN
	YCEN = -YCEN
;
;  Get the initial pointing information, either from the exposure headers
;  if they exist, or from the raster header.
;
	IF TOTAL(HEADERFOUND) GT 0 THEN BEGIN
		W = WHERE(HEADERFOUND)
		IF NX*NY GT 1 THEN BEGIN
			SLITPOSITION   = REFORM(SLIT_POS.ARRAY, NX, NY)
			MIRRORPOSITION = REFORM(MIR_POS.ARRAY,  NX, NY)
			HEADERFOUND    = REFORM(HEADERFOUND,    NX, NY)
		END ELSE BEGIN
			SLITPOSITION   = (SLIT_POS.ARRAY)(0)
			MIRRORPOSITION = (MIR_POS.ARRAY)(0)
			HEADERFOUND    = HEADERFOUND(0)
			SLITPOS_USED = -1
			MIRPOS_USED  = -1
		ENDELSE
	END ELSE BEGIN
		SLITPOSITION   = FXPAR(HEADER, 'SLIT_POS')
		MIRRORPOSITION = FXPAR(HEADER, 'MIR_POS')
		SLITPOS_USED = 0
		MIRPOS_USED  = 0
	ENDELSE
;
;  Modify the pointing based on the slit and mirror positions.
;
	MECH_POINT, FIX(SLITPOSITION), MIRRORPOSITION, XCEN, YCEN
;
;  If an array of centers were calculated, then calculate XCEN by first
;  interpolating to the middle position in the slit direction, and then to the
;  middle position in the mirror direction.
;
	IF N_ELEMENTS(XCEN) GT 1 THEN BEGIN
		IF NY GT 1 THEN BEGIN
		    TEMP = XCEN(*,0)
		    XFOUND = REPLICATE(0,NX)
		    FOR IX = 0,NX-1 DO BEGIN
			XTEMP = XCEN(IX,*)
			HFTEMP = HEADERFOUND(IX,*)
			W = WHERE(HFTEMP, COUNT)
			IF COUNT EQ 1 THEN BEGIN
			    TEMP(IX) = XTEMP(W(0))
			    XFOUND(IX) = 1
			END ELSE IF COUNT GT 1 THEN BEGIN
			    TEMP(IX) = BASE_INTERP(XTEMP(W), W, (NY-1)/2., $
				    MINSIG=1)
			    XFOUND(IX) = 1
			ENDIF
		    ENDFOR
		    XCEN = TEMP
		END ELSE XFOUND = HEADERFOUND
;
		W = WHERE(XFOUND, COUNT)
		IF COUNT EQ 1 THEN BEGIN
			XCEN = XCEN(W(0))
			MIRPOS_USED = W(0)
		END ELSE BEGIN
			XCEN = BASE_INTERP(XCEN(W), W, (NX-1)/2., MINSIG=1)
			MIRPOS_USED = -1
		ENDELSE
;
;  Do the opposite for YCEN.
;
		TEMP = REFORM(YCEN(0,*))
		YFOUND = REPLICATE(0,NY)
		FOR IY = 0,NY-1 DO BEGIN
		    YTEMP = YCEN(*,IY)
		    HFTEMP = HEADERFOUND(*,IY)
		    W = WHERE(HFTEMP, COUNT)
		    IF COUNT EQ 1 THEN BEGIN
			TEMP(IY) = YTEMP(W(0))
			YFOUND(IY) = 1
		    END ELSE IF COUNT GT 1 THEN BEGIN
			TEMP(IY) = BASE_INTERP(YTEMP(W), W, (NY-1)/2., $
				MINSIG=1)
			YFOUND(IY) = 1
		    ENDIF
		ENDFOR
		YCEN = TEMP
;
		W = WHERE(YFOUND, COUNT)
		IF COUNT EQ 1 THEN BEGIN
			YCEN = YCEN(W(0))
			SLITPOS_USED = W(0)
		END ELSE BEGIN
			YCEN = BASE_INTERP(YCEN(W), W, (NY-1)/2., MINSIG=1)
			SLITPOS_USED = -1
		ENDELSE
	END ELSE BEGIN
		MIRPOS_USED  = -1
		SLITPOS_USED = -1
	ENDELSE
;
;  Make sure that XCEN and YCEN end up as scalars.
;
	XCEN = XCEN(0)
	YCEN = YCEN(0)
;
;  For the GIS detector, calculate the effect of the slit movement.
;
	IF INSTRUMENT EQ 'GIS' THEN BEGIN
		IF SLITPOS_USED LT 0 THEN RYCEN = 0 ELSE RYCEN =	$
			(0.5*(NY-1) - SLITPOS_USED) * SLITSTEP
	END ELSE RYCEN = 0
;
;  Modify the pointing based on the detector information.  Note that the scan
;  mirror moves backwards.
;
	IF MIRPOS_USED GE 0 THEN XCEN = XCEN - (0.5*(NX-1) - MIRPOS_USED) * $
		MIRRORSTEP
	YCEN = YCEN + RYCEN
;
;  Store these correction factors for use later.
;
	XCOR = XCEN
	YCOR = YCEN
;
;  Get the original values of XCEN, YCEN.  Also get the spacecraft pointing.
;
	XCEN    = FXPAR(HEADER, 'XCEN')  &  XCEN0 = XCEN
	YCEN    = FXPAR(HEADER, 'YCEN')  &  YCEN0 = YCEN
	SC_X0   = FXPAR(HEADER, 'SC_X0')
	SC_Y0   = FXPAR(HEADER, 'SC_Y0')
	SC_ROLL = FXPAR(HEADER, 'SC_ROLL')
;
;  Remove the spacecraft pointing from XCEN, YCEN
;
	ANGLE = SC_ROLL
	XX = XCEN - SC_X0
	YY = YCEN - SC_Y0
	COS_A = COS(ANGLE / !RADEG)
	SIN_A = SIN(ANGLE / !RADEG)
	XCEN =  XX * COS_A  +  YY * SIN_A
	YCEN = -XX * SIN_A  +  YY * COS_A
;
;  Apply the correction factor, and reapply the spacecraft pointing.
;
	XX = XCEN + XCOR
	YY = YCEN + YCOR
	XCEN = XX * COS_A  -  YY * SIN_A  +  SC_X0
	YCEN = XX * SIN_A  +  YY * COS_A  +  SC_Y0
;
;  The difference between the old and new values of XCEN, YCEN are the complete
;  correction factors.
;
	XCOR = XCEN - XCEN0
	YCOR = YCEN - YCEN0
;
;  If the corrections are less than an arcsecond, then don't apply them.
;
	IF (ABS(XCOR) LT 1) AND (ABS(YCOR) LT 1) THEN BEGIN
		MESSAGE, /INFORMATIONAL, 'Correction too small -- not applying'
		RETURN
	ENDIF
;
;  Make sure that the user wishes to proceed.
;
	PRINT, 'Changing XCEN=' + TRIM(XCEN0) + ' to ' + TRIM(XCEN) +	$
		' compared against INS_X0=' + TRIM(FXPAR(HEADER,'INS_X0'))
	PRINT, 'Changing YCEN=' + TRIM(YCEN0) + ' to ' + TRIM(YCEN) +	$
		' compared against INS_Y0=' + TRIM(FXPAR(HEADER,'INS_Y0'))
	ASK, 'Proceed? ', ANSWER
	IF ANSWER NE 'Y' THEN RETURN
;
;  Reopen the file for update.
;
	OPENU, UNIT, FILENAME, /GET_LUN
;
;  Read in the main header for the FITS file.
;
	POINT_LUN, UNIT, 0
	FXHREAD, UNIT, HEADER
;
;  Correct the values of XCEN, YCEN in the header.
;
	FXADDPAR, HEADER, 'XCEN', XCEN, FORMAT='(F7.1)'
	FXADDPAR, HEADER, 'YCEN', YCEN, FORMAT='(F7.1)'
;
;  Write it back out again, and insert it in the catalog.
;
	POINT_LUN, UNIT, 0
	WRITEU, UNIT, HEADER
	INS_CATALOG, HEADER
;
;  Save the current file pointer position, and read in the binary table header.
;
	POINT_LUN, UNIT, 0
	FXHREAD, UNIT, HEADER
	POINT_LUN, -UNIT, POS
	FXHREAD, UNIT, HEADER
;
;  Correct the values of XCEN, YCEN in the header.
;
	FXADDPAR, HEADER, 'XCEN', XCEN, FORMAT='(F7.1)'
	FXADDPAR, HEADER, 'YCEN', YCEN, FORMAT='(F7.1)'
;
;  Get the number of columns.
;
	N_COLUMNS = FXPAR(HEADER, 'TFIELDS')
;
;  For each column, extract the starting values, and the dimension labels.
;
	FOR I_COL = 1, N_COLUMNS DO BEGIN
	    TRVAL = FXBTDIM(FXPAR(HEADER, 'TRVAL' + TRIM(I_COL), COUNT=COUNT))
	    TDESC = FXBTDIM(FXPAR(HEADER, 'TDESC' + TRIM(I_COL)))
	    IF COUNT GT 0 THEN BEGIN
;
;  Replace the TRVALn line in the header.
;
		VALUE = TRVAL
		IF N_ELEMENTS(VALUE) GT 1 THEN TRVAL = '(' ELSE TRVAL = ''
;
		FOR I = 0,N_ELEMENTS(VALUE)-1 DO BEGIN
			IF I GT 0 THEN BEGIN
				TRVAL = TRVAL + ','
			ENDIF
;
			CASE TDESC(I) OF
			    'WAVELNTH': TRVAL = TRVAL + TRIM(VALUE(I))
			    'SOLAR_X':  TRVAL = TRVAL + TRIM(VALUE(I) + XCOR)
			    'SOLAR_Y':  TRVAL = TRVAL + TRIM(VALUE(I) + YCOR)
			    'DEL_TIME': TRVAL = TRVAL + TRIM(VALUE(I))
			    'DETECTOR': TRVAL = TRVAL + TRIM(VALUE(I))
			ENDCASE
		ENDFOR
;
		IF N_ELEMENTS(VALUE) GT 1 THEN TRVAL = TRVAL + ')'
		FXADDPAR, HEADER, 'TRVAL' + TRIM(I_COL), TRVAL
	    ENDIF
	ENDFOR
;
;  Write out the modified header.
;
	POINT_LUN, UNIT, POS
	WRITEU, UNIT, HEADER
;
;  Close the file and return.
;
	FREE_LUN, UNIT
	RETURN
	END
