	PRO REPAIR_YSTART, FILENAME
;+
; Project     :	SOHO - CDS
;
; Name        :	REPAIR_YSTART
;
; Purpose     :	Repairs the value of TRVAL for GIS data in FITS headers
;
; Category    :	
;
; Explanation :	The value of TRVAL for the Y axis in GIS data was calculated
;		incorrectly for some FITS files.  This routine corrects that
;		error.
;
; Syntax      :	REPAIR_YSTART, FILENAME
;
; Examples    :	REPAIR_YSTART, 's3169r00.fits'
;
; Inputs      :	FILENAME = The name of the file to repair.  The complete
;			   filename must be given, including path if different
;			   from the current directory.  This routine will not
;			   search for it in $CDS_FITS_DATA.
;
; Opt. Inputs :	None.
;
; Outputs     :	The FITS file headers are modified.
;
; Opt. Outputs:	None.
;
; Keywords    :	None.
;
; Calls       :	FXBOPEN, FXPAR, READCDSCOL, FXBCLOSE, FXBTDIM, BASE_INTERP,
;		ASK, INS_CATALOG
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	None, unless applied more than once.
;
; Prev. Hist. :	None.
;
; History     :	Version 1, 12-Aug-1996, William Thompson, GSFC
;		Version 2, 20-Aug-1996, William Thompson, GSFC
;			Handle TRVAL more robustly.
;
; Contact     :	WTHOMPSON
;-
;
	ON_ERROR, 2
;
;  Check the number of parameters.
;
	IF N_PARAMS() NE 1 THEN MESSAGE, 'Syntax:  REPAIR_YSTART, FILENAME'
;
;  Open the FITS file, and determine the instrument.  If not GIS, then return.
;
	FXBOPEN, UNIT, FILENAME, 1, HEADER
	INSTRUMENT = TRIM(FXPAR(HEADER, 'DETECTOR'))
	IF INSTRUMENT NE 'GIS' THEN BEGIN
		MESSAGE, /INFORMATIONAL, 'NIS data does not require processing'
		FXBCLOSE, UNIT
		RETURN
	ENDIF
;
;  Get the raster dimensions.  If NY=1, then the file doesn't require
;  processing.
;
	NX = FXPAR(HEADER, 'NX')
	NY = FXPAR(HEADER, 'NY')
	IF NY EQ 1 THEN BEGIN
		MESSAGE, /INFORMATIONAL, 'NY=1, no processing needed'
		FXBCLOSE, UNIT
		RETURN
	ENDIF
;
;  Get the pointing information.
;
	MIRRORSTEP = FXPAR(HEADER, 'XSTEP')
	SLITSTEP   = FXPAR(HEADER, 'YSTEP')
	XCEN       = FXPAR(HEADER, 'XCEN')
	YCEN       = FXPAR(HEADER, 'YCEN')
	SC_X0      = FXPAR(HEADER, 'SC_X0')
	SC_Y0      = FXPAR(HEADER, 'SC_Y0')
	SC_ROLL    = FXPAR(HEADER, 'SC_ROLL')
;
;  Get the first YSTART value.  If it's okay, then we don't need to process the
;  file.  Problems are recognized by the fact that YSTART and YCEN are the same
;  within 0.1 arcseconds.  The X and Y axes are always second and third.
;
	TRVAL = FLOAT( FXBTDIM( FXPAR(HEADER, 'TRVAL1') ))
	FXBCLOSE, UNIT
	XSTART = TRVAL(1)  &  XSTART0 = XSTART
	YSTART = TRVAL(2)  &  YSTART0 = YSTART
	IF ABS(YSTART-YCEN) GT 0.1 THEN BEGIN
		MESSAGE, /INFORMATIONAL, 'File does not require correction'
		RETURN
	ENDIF
;
;  Remove the spacecraft pointing from XSTART, YSTART.
;
	ANGLE = SC_ROLL
	XX = XSTART - SC_X0
	YY = YSTART - SC_Y0
	COS_A = COS(ANGLE / !RADEG)
	SIN_A = SIN(ANGLE / !RADEG)
	XSTART =  XX * COS_A  +  YY * SIN_A
	YSTART = -XX * SIN_A  +  YY * COS_A
;
;  Apply the correction factor, and reapply the spacecraft pointing.
;
	XX = XSTART
	YY = YSTART - (NY - 1.) * SLITSTEP / 2
	XSTART = XX * COS_A  -  YY * SIN_A  +  SC_X0
	YSTART = XX * SIN_A  +  YY * COS_A  +  SC_Y0
;
;  Make sure that the user wishes to proceed.
;
	PRINT, 'Changing XSTART=' + TRIM(XSTART0) + ' to ' + TRIM(XSTART)
	PRINT, 'Changing YSTART=' + TRIM(YSTART0) + ' to ' + TRIM(YSTART)
	ASK, 'Proceed? ', ANSWER
	IF ANSWER NE 'Y' THEN RETURN
;
;  Reopen the file for update.
;
	OPENU, UNIT, FILENAME, /GET_LUN
;
;  Read in the main header for the FITS file.
;
	FXHREAD, UNIT, HEADER
;
;  Save the current file pointer position, and read in the binary table header.
;
	POINT_LUN, -UNIT, POS
	FXHREAD, UNIT, HEADER
;
;  Get the number of columns.
;
	N_COLUMNS = FXPAR(HEADER, 'TFIELDS')
;
;  For each column, extract the starting values, and the dimension labels.
;
	FOR I_COL = 1, N_COLUMNS DO BEGIN
	    TRVAL = FXBTDIM(FXPAR(HEADER, 'TRVAL' + TRIM(I_COL), COUNT=COUNT))
	    TDESC = FXBTDIM(FXPAR(HEADER, 'TDESC' + TRIM(I_COL)))
	    IF COUNT GT 0 THEN BEGIN
;
;  Replace the TRVALn line in the header.
;
		VALUE = TRVAL
		IF N_ELEMENTS(VALUE) GT 1 THEN TRVAL = '(' ELSE TRVAL = ''
;
		FOR I = 0,N_ELEMENTS(VALUE)-1 DO BEGIN
			IF I GT 0 THEN BEGIN
				TRVAL = TRVAL + ','
			ENDIF
;
			CASE TDESC(I) OF
			    'SOLAR_X':  TRVAL = TRVAL + TRIM(XSTART)
			    'SOLAR_Y':  TRVAL = TRVAL + TRIM(YSTART)
			    ELSE: TRVAL = TRVAL + TRIM(VALUE(I))
			ENDCASE
		ENDFOR
;
		IF N_ELEMENTS(VALUE) GT 1 THEN TRVAL = TRVAL + ')'
		FXADDPAR, HEADER, 'TRVAL' + TRIM(I_COL), TRVAL
	    ENDIF
	ENDFOR
;
;  Write out the modified header.
;
	POINT_LUN, UNIT, POS
	WRITEU, UNIT, HEADER
;
;  Close the file and return.
;
	FREE_LUN, UNIT
	RETURN
	END
