;+
; Project     :	SOHO - CDS
;
; Name        :	SEL_CDROM_DRIVE
;
; Purpose     :	Selects a SOHO DPS directory containing a CDROM imagej
;
; Category    :	Class1, Operations, Telemetry
;
; Explanation :	Lists SOHO DPS directories in the /soho_dps/data tree of the
;		form YYYYMMDD, e.g. "20020101".
;
; Syntax      :	SEL_CDROM_DRIVE, DRIVE
;
; Examples    :	SEL_CDROM_DRIVE, DRIVE, GROUP=EVENT.TOP, /MODAL
;
; Inputs      :	None.
;
; Opt. Inputs :	None.
;
; Outputs     :	DRIVE	= The name of the directory selected.
;
; Opt. Outputs:	None.
;
; Keywords    :	GROUP	= The group leader of the calling widget.
;		MODAL	= Suspend the calling widget until this routine returns
;			  with an answer.
;
; Calls       :	DELVARX, FIND_FILE
;
; Common      :	Uses the internal common block SEL_CDROM_DRIVE
;
; Restrictions:	Must have /soho_dps mounted.
;
; Side effects:	None.
;
; Prev. Hist. :	None.
;
; History     :	Version 1, 08-Mar-2002, William Thompson, GSFC
;               Version 2, 20-Oct-2004, William Thompson, GSFC
;                       Corrected some widget errors for IDL 6.1
;
; Contact     :	WTHOMPSON
;-
;
	PRO SEL_CDROM_DRIVE_EVENT, EVENT
;
	COMMON SEL_CDROM_DRIVE, LIST_WID, DONE_WID, LIST, SELECTED
;
;  Get the event value.  Depending on the value, take the appropriate action.
;
	WIDGET_CONTROL, EVENT.ID, GET_UVALUE=EV
	CASE EV OF
	    'LIST': BEGIN
		WIDGET_CONTROL, DONE_WID, SENSITIVE=1
		SELECTED = LIST(EVENT.INDEX)
		END
;
;  If the DONE button has been pressed, then kill the widget.
;
	    'DONE': WIDGET_CONTROL, EVENT.TOP, /DESTROY
;
;  If the CANCEL button has been pressed, then remove any selected files and
;  kill the widget.
;
	    'CANCEL': BEGIN
		DELVARX, SELECTED
		WIDGET_CONTROL, EVENT.TOP, /DESTROY
		END
;
	ENDCASE
	END
;
;==============================================================================
;
	PRO SEL_CDROM_DRIVE, DRIVE, GROUP=GROUP, MODAL=MODAL
;
	COMMON SEL_CDROM_DRIVE
	ON_ERROR, 2
;
;  Check the number of arguments.
;
	IF N_PARAMS() NE 1 THEN MESSAGE, 'Syntax:  SEL_CDROM_DRIVE, DRIVE'
;
;  If a copy of SEL_CDROM_DRIVE is already running, then exit.
;
	IF (XREGISTERED("sel_cdrom_drive") NE 0) THEN RETURN
;
;  Make sure that GROUP is defined.
;
	IF N_ELEMENTS(Group) EQ 0 THEN GROUP=0
;
;  Define the main base.
;
	MAIN = WIDGET_BASE(GROUP_LEADER=GROUP, /COLUMN, MODAL=MODAL)
	DUMMY = WIDGET_LABEL(MAIN, VALUE='SOHO DPS directories')
	LIST_WID = WIDGET_LIST(MAIN, VALUE='', UVALUE='LIST', XSIZE=40, $
		YSIZE=15)
;
;  Finally, put the done and cancel buttons at the bottom.
;
	BASE4 = WIDGET_BASE(MAIN, /ROW)
;
	DONE_WID = WIDGET_BUTTON(BASE4, UVALUE='DONE', VALUE='Done')
	DUMMY = WIDGET_BUTTON(BASE4, UVALUE='CANCEL', VALUE='Cancel')
;
;  Realize the widget.
;
	WIDGET_CONTROL, MAIN, /REALIZE
;
;  Densitize the Done button.
;
	WIDGET_CONTROL, DONE_WID, SENSITIVE=0
;
;  Select the realtime path, and display all the drives.
;
	WIDGET_CONTROL, /HOURGLASS
	LIST = FIND_FILE('/soho_dps/data', COUNT=COUNT)
	IF COUNT EQ 0 THEN BEGIN
	    WIDGET_CONTROL, LIST_WID, SET_VALUE='', SENSITIVE=0
	END ELSE BEGIN
	    W = WHERE((STRMID(LIST,0,2) EQ '20') AND (STRLEN(LIST) EQ 8), $
		    COUNT)
	    IF COUNT EQ 0 THEN BEGIN
		WIDGET_CONTROL, LIST_WID, SET_VALUE='', SENSITIVE=0
	    END ELSE BEGIN
		LIST = '/soho_dps/data/' + LIST(W)
		WIDGET_CONTROL, LIST_WID, SET_VALUE=LIST, SENSITIVE=1
	    ENDELSE
	ENDELSE
;
;  Make sure that the SELECTED list starts out empty.
;
	DELVARX, SELECTED
;
;  Set it all going.
;
	XMANAGER, 'SEL_CDROM_DRIVE', MAIN
;
;  If a drive was selected, then return it.
;
	IF N_ELEMENTS(SELECTED) GT 0 THEN DRIVE = SELECTED
;
	RETURN
	END
