;+
; Project     :	SOHO - CDS
;
; Name        :	SEL_TM_FILES
;
; Purpose     :	Select telemetry files for use within FITSGEN.
;
; Category    :	Data handling, telemetry
;
; Explanation :	Allows the user to select telemetry files to process.  The user
;		can select from one of three directories:
;
;			Realtime	($CDS_TM_DATA)
;			Quicklook	($CDS_QKLSCI_DATA)
;			CDROM		($CDS_CDROM_DATA)
;
;		or from a personal directory.  One can also mix and match,
;		e.g. to put together data from realtime and quicklook.
;
; Syntax      :	SEL_TM_FILES, FILES
;
; Examples    :	
;
; Inputs      :	None.
;
; Opt. Inputs :	None.
;
; Outputs     :	FILES	= A list of selected filenames.
;
; Opt. Outputs:	None.
;
; Keywords    :	None.
;
; Calls       :	DOY2UTC, ANYTIM2CAL
;
;		Also, the routines LIST_TM_FILES and SEL_TM_FILES_EVENT are
;		included within this file.
;
; Common      :	The file sel_tm_files.cmn contains the 
;
; Restrictions:	Only one copy of this widget can be running at any one time.
;
; Side effects:	None.
;
; Prev. Hist. :	Inspired by CDS_FILES_IN by Andrew Bowen and C. D. Pike.
;
; History     :	Version 1, 05-Mar-1996, William Thompson, GSFC
;		Version 2, 06-Mar-1996, William Thompson, GSFC
;			Fixed bug when CANCEL was selected.
;		Version 3, 15-Mar-1996, William Thompson, GSFC
;			Allow selection of both kinds of realtime files
;		Version 4, 18-Jun-1996, William Thompson, GSFC
;			Call FIND_FILE instead of FINDFILE
;		Version 5, 05-Jan-2000, William Thompson, GSFC
;			Handle both formats of tm.* filenames
;
; Contact     :	WTHOMPSON
;-
;
	PRO LIST_TM_FILES
;
@sel_tm_files.cmn
;
;  Find all the files in the selected directory.  If no files are found, then
;  densitize the list widget.
;
	WIDGET_CONTROL, /HOURGLASS
	LIST = FIND_FILE(PATH, COUNT=COUNT)
	IF COUNT EQ 0 THEN BEGIN
		WIDGET_CONTROL, LIST_WID, SET_VALUE='', SENSITIVE=0
		RETURN
	ENDIF
;
;  Append the calendar date for all "tm." files.
;
	SEELIST = LIST
	NTM = WHERE(STRMID(SEELIST,0,3) EQ 'tm.', COUNT)
	IF COUNT GT 0 THEN BEGIN
	    TEST = STRMID(SEELIST(NTM),0,4)
	    NTM0 = WHERE(TEST NE 'tm.1', COUNT)
	    IF COUNT GT 0 THEN BEGIN
		NTM0 = NTM(NTM0)
		YEAR = FIX(STRMID(LIST(NTM0),3,2))
		YEAR = ((YEAR + 50) MOD 100) + 1950
		DOY  = FIX(STRMID(LIST(NTM0),6,3))
		ERRMSG = ''
		DATE = DOY2UTC(DOY,YEAR,ERRMSG=ERRMSG)
		ERRMSG = ''
		DATE = ANYTIM2CAL(DATE,F=1,ERRMSG=ERRMSG)
		DATE = ' (' + STRMID(DATE,0,6) + ')'
		SEELIST(NTM0) = SEELIST(NTM0) + DATE
	    ENDIF
;
	    NTM1 = WHERE(TEST EQ 'tm.1', COUNT)
	    IF COUNT GT 0 THEN BEGIN
		NTM1 = NTM(NTM1)
		YEAR = FIX(STRMID(LIST(NTM1),4,2))
		YEAR = ((YEAR + 50) MOD 100) + 1950
		DOY  = FIX(STRMID(LIST(NTM1),7,3))
		ERRMSG = ''
		DATE = DOY2UTC(DOY,YEAR,ERRMSG=ERRMSG)
		ERRMSG = ''
		DATE = ANYTIM2CAL(DATE,F=1,ERRMSG=ERRMSG)
		DATE = ' (' + STRMID(DATE,0,6) + ')'
		SEELIST(NTM1) = SEELIST(NTM1) + DATE
	    ENDIF
	ENDIF
;
;  Show the list.
;
	WIDGET_CONTROL, LIST_WID, SET_VALUE=SEELIST, SENSITIVE=1
;
	END
;
;==============================================================================
;
	PRO SEL_TM_FILES_EVENT, EVENT
;
@sel_tm_files.cmn
;
;  Get the event value.  Depending on the value, take the appropriate action.
;
	WIDGET_CONTROL, EVENT.ID, GET_UVALUE=EV
	CASE EV OF
;
;  If the user's path has been changed, then change the path.
;
	    'PATH': BEGIN
		WIDGET_CONTROL, EVENT.ID, GET_VALUE=USERPATH
		USERPATH = USERPATH(0)
		PATH = USERPATH
		LIST_TM_FILES
		END
;
;  If the realtime button was selected, then change the path to where the
;  realtime files are.
;
	    'REALTIME': BEGIN
		CASE EVENT.VALUE OF
		    'TM files': BEGIN
			WIDGET_CONTROL, PATH_WID, SENSITIVE=0
			PATH = GETENV('CDS_TM_DATA')
			LIST_TM_FILES
			END
		    'REL files': BEGIN
			WIDGET_CONTROL, PATH_WID, SENSITIVE=0
			PATH = GETENV('CDS_RELSCI_DATA')
			LIST_TM_FILES
			END
		ENDCASE
		END
;
;  If the quicklook button was pressed, then change the path to where the
;  quicklook files are.
;
	    'QUICKLOOK': BEGIN
		WIDGET_CONTROL, PATH_WID, SENSITIVE=0
		PATH = GETENV('CDS_QKLSCI_DATA')
		LIST_TM_FILES
		END
;
;  If the CDROM button was pressed, then change the path to where the CDROM
;  files are.
;
	    'CDROM': BEGIN
		WIDGET_CONTROL, PATH_WID, SENSITIVE=0
		PATH = GETENV('CDS_CDROM_DATA')
		LIST_TM_FILES
		END
;
;  If the user data button was pressed, then change the path to where the
;  user's files are.
;
	    'USERDATA': BEGIN
		PATH = USERPATH
		WIDGET_CONTROL, PATH_WID, SENSITIVE=1
		LIST_TM_FILES
		END
;
;  If a file has been selected from the list, then add it to the list of
;  selected files.
;
	    'LIST': BEGIN
		WIDGET_CONTROL, DONE_WID, SENSITIVE=1
;
		SELECT = LIST(EVENT.INDEX)
		WIDGET_CONTROL, SELECT_WID, GET_VALUE=LINE
		LINE = LINE(0)
		IF STRTRIM(LINE) EQ '' THEN LINE = SELECT ELSE	$
			LINE = LINE + ',' + SELECT
		WIDGET_CONTROL, SELECT_WID, SET_VALUE=LINE
;
		SELECT = CONCAT_DIR(PATH,SELECT)
		IF N_ELEMENTS(SELECTED) EQ 0 THEN SELECTED = SELECT ELSE $
			SELECTED = [SELECTED, SELECT]
		END
;
;  If the DONE button has been pressed, then kill the widget.
;
	    'DONE': WIDGET_CONTROL, EVENT.TOP, /DESTROY
;
;  If the CANCEL button has been pressed, then remove any selected files and
;  kill the widget.
;
	    'CANCEL': BEGIN
		DELVARX, SELECTED
		WIDGET_CONTROL, EVENT.TOP, /DESTROY
		END
;
	ENDCASE
	END
;
;==============================================================================
;
	PRO SEL_TM_FILES, FILES, GROUP=GROUP
;
@sel_tm_files.cmn
ON_ERROR, 2
;
;  Check the number of arguments.
;
	IF N_PARAMS() NE 1 THEN MESSAGE, 'Syntax:  SEL_TM_FILES, FILES'
;
;  If a copy of SEL_TM_FILES is already running, then exit.
;
	IF (XREGISTERED("sel_tm_files") NE 0) THEN RETURN
;
;  Make sure that GROUP is defined.
;
	IF N_ELEMENTS(Group) EQ 0 THEN GROUP=0
;
;  Define the main base.
;
	MAIN = WIDGET_BASE(GROUP_LEADER=GROUP, /COLUMN, /MAP)
;
;  Define the widget to contain the user's path.
;
	PATH_WID = CW_FIELD(MAIN, VALUE='', /ROW, /STRING, /RETURN_EVENTS, $
		TITLE='Data Root', UVALUE='PATH', XSIZE=50)
;
;  Define a base for the path selection buttons, and define buttons for
;  selecting realtime data, quicklook data, and user data.
;
	BASE1 = WIDGET_BASE(MAIN, /ROW, /MAP)
;
;  The realtime button is a pulldown menu.
;
	JUNK = {CW_PDMENU_S, FLAGS: 0, NAME: ''}
	DESC = [{CW_PDMENU_S, 3, 'Realtime Data'},	$
		{CW_PDMENU_S, 0, 'TM files'},		$
		{CW_PDMENU_S, 2, 'REL files'}]
	DUMMY = CW_PDMENU(BASE1, DESC, /RETURN_NAME, UVALUE='REALTIME')
;
;	DUMMY = WIDGET_BUTTON(BASE1,UVALUE='REALTIME', VALUE='Realtime Data')
	DUMMY = WIDGET_BUTTON(BASE1,UVALUE='QUICKLOOK',VALUE='Quicklook Data')
	DUMMY = WIDGET_BUTTON(BASE1,UVALUE='CDROM',    VALUE='CDROM Data')
	DUMMY = WIDGET_BUTTON(BASE1,UVALUE='USERDATA', VALUE='User Data')
;
;  Define a base for the telemetry files, and put a label at the left.
;
	BASE2 = WIDGET_BASE(MAIN, /ROW, /MAP)
	DUMMY = WIDGET_LABEL(BASE2, VALUE='Telemetry File')
;
;  Define another base to the right of the label.
;
	BASE3 = WIDGET_BASE(BASE2, /COLUMN, /MAP)
;
;  Put the list of selected files at the top, and those files which can be
;  selected at the bottom.
;
	SELECT_WID = WIDGET_TEXT(BASE3, VALUE='', UVALUE='SELECTED',	$
		XSIZE=50, YSIZE=1)
	LIST_WID = WIDGET_LIST(BASE3, VALUE='', UVALUE='LIST', YSIZE=15)
;
;  Finally, put the done and cancel buttons at the bottom.
;
	BASE4 = WIDGET_BASE(MAIN, /ROW, /MAP)
;
	DONE_WID = WIDGET_BUTTON(BASE4, UVALUE='DONE', VALUE='Done')
	DUMMY = WIDGET_BUTTON(BASE4, UVALUE='CANCEL', VALUE='Cancel')
;
;  Realize the widget.
;
	WIDGET_CONTROL, MAIN, /REALIZE
;
;  If the user path hasn't been defined yet, then set it equal to the current
;  directory.  Put the path in the path widget.
;
	IF N_ELEMENTS(USERPATH) EQ 0 THEN USERPATH = ''
	WIDGET_CONTROL, PATH_WID, SET_VALUE=USERPATH
;
;  Densitize the user path and the Done button.
;
	WIDGET_CONTROL, PATH_WID, SENSITIVE=0
	WIDGET_CONTROL, DONE_WID, SENSITIVE=0
;
;  Select the realtime path, and display all the files.
;
	PATH = GETENV('CDS_TM_DATA')
	LIST_TM_FILES
;
;  Make sure that the SELECTED list starts out empty.
;
	DELVARX, SELECTED
	DELVARX, FILES
;
;  Set it all going.
;
	XMANAGER, 'SEL_TM_FILES', MAIN, /MODAL
;
;  If any files were selected, then return them.
;
	IF N_ELEMENTS(SELECTED) GT 0 THEN FILES = SELECTED ELSE FILES = ''
;
	RETURN
	END
