	PRO SLITPOS_DECODE, SLITPOS, BITS, POSITION, MAP=MAP
;+
; Project     :	SOHO - CDS
;
; Name        :	SLITPOS_DECODE
;
; Purpose     :	Decode MCU telemetry format for slit position.
;
; Explanation :	This procedure takes the slit position in MCU telemetry format,
;		and extracts the status bits and position value.
;
; Use         :	SLITPOS_DECODE, SLITPOS, BITS, POSITION  [, /MAP ]
;
; Inputs      :	SLITPOS	 = Short integer input scalar or array containing the
;			   telemetry bits corresponding to the slit position.
;
; Opt. Inputs :	None.
;
; Outputs     :	BITS	 = The status bits in one of two formats:
;
;				* Character string containing the status bits
;				  as ones and zeros.  This is the default, and
;				  is suitable for use in FITS header.
;
;				* Byte variable with the status bits shifted
;				  all the way to the right.  Requires the /MAP
;				  keyword.  Suitable for use with FITS binary
;				  table bit array fields.
;
;		POSITION = The slit position.
;
; Opt. Outputs:	None.
;
; Keywords    :	MAP	 = If set, then the format of the BITS output variable
;			   is changed.
;
; Calls       :	None.
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Category    :	Data_handling, I/O, Telemetry.
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 16 December 1993.
;
; Modified    :	Version 1, William Thompson, GSFC, 16 December 1993.
;		Version 2, William Thompson, GSFC, 28 February 1994.
;			Fixed bug with negative numbers.
;
; Version     :	Version 2, 28 February 1994.
;-
;
	ON_ERROR, 2
;
;  Check the number of parameters.
;
	IF N_PARAMS() NE 3 THEN MESSAGE,	$
		'Syntax:  SLITPOS_DECODE, SLITPOS, BITS, POSITION'
;
;  Make sure that the input is of the correct type.
;
	IF DATATYPE(SLITPOS,1) NE 'Integer' THEN MESSAGE,	$
		'SLITPOS must be a short integer'
;
;  Extract the status bits, 0 through 6.
;
	IF KEYWORD_SET(MAP) THEN BEGIN
		BITS = BYTE(ISHFT(SLITPOS,-9))
	ENDIF ELSE BEGIN
		DEC2BIN, SLITPOS, SLITPOSBITS, /QUIET
		BITS = STRTRIM(FIX(SLITPOSBITS(0)),2)
		FOR I = 1,6 DO BITS = BITS + STRTRIM(FIX(SLITPOSBITS(I)),2)
	ENDELSE
;
;  Extract the position, bits 7 through 15.  Make sure it has the correct sign
;  associated with it.
;
	POSITION = SLITPOS AND '1FF'X
	W = WHERE(POSITION GE 256, COUNT)
	IF COUNT GT 0 THEN POSITION(W) = POSITION(W) OR 'FE00'X
;
	RETURN
	END
