	FUNCTION TEST_CURR_EXP, RASTER, PACKET, EXPOSURE, COUNTER
;+
; Project     : SOHO - CDS
;
; Name        : TEST_CURR_EXP()
;
; Purpose     : Tests if packet is part of the current exposure.
;
; Explanation : This routine tests whether or not a packet is part of the
;		current exposure.  If not, then presumably it is part of the
;		next exposure or raster.
;
; Use         : Result = TEST_CURR_EXP( RASTER, PACKET, EXPOSURE, COUNTER )
;
; Inputs      : RASTER		= Raster structure defined by FORM_RASTER.
;		PACKET		= Packet to be tested.
;		EXPOSURE	= Number of current exposure, starting with 0.
;		COUNTER		= The value of the packet counter for the last
;				  packet read in for this exposure.  Should be
;				  set to -1 if no packets have been read in
;				  yet.
;
; Opt. Inputs : None
;
; Outputs     : The result of the function is either true (1) if the packet is
;		part of the current exposure, or false (0) otherwise.
;
; Opt. Outputs: None.
;
; Keywords    : None.
;
; Calls       : None.
;
; Common      : None.
;
; Restrictions: It is assumed that TEST_SCI_PKT has already been used to filter
;		out non-science packets.
;
; Side effects: None.
;
; Category    : Data Handling, Telemetry.
;
; Prev. Hist. : Taken from an earlier routine called READ_EXPOSURE by William
;		Thompson, GSFC, November-February 1993.
;
; Written     : William Thompson, GSFC, 23 March 1993.
;
; Modified    : Version 1, William Thompson, GSFC, 5 April 1993.
;			Modified for version 2.4 of the packet format document.
;		Version 2, William Thompson, GSFC, 7 June 1993.
;			Modified for version 2.8 of the packet format document.
;		Version 2.1, William Thompson, GSFC, 16 November 1993.
;			Commented out some of the tests to overcome problems in
;			some telemetry files.
;		Version 3, William Thompson, GSFC, 6 February 1995
;			Restored to complete functionality.
;
; Version     : Version 3, 6 February 1995
;-
;
	ON_ERROR, 2
;
;  Check the number of parameters.
;
	IF N_PARAMS() NE 4 THEN MESSAGE, 'Syntax:  Result = ' +	$
		'TEST_CURR_EXP(RASTER,PACKET,EXPOSURE,COUNTER)'
;
;  Extract out the needed data from the RASTER structure.
;
	INSTRUMENT = RASTER.INSTRUMENT
	HEADER	   = RASTER.HEADER
;
;  First check to see if the packet ID is consistent with the raster header.
;
	PKTID = PACKET.HDR.ID
	IF INSTRUMENT EQ 'GIS' AND PKTID NE '11'X AND PKTID NE '21'X THEN $
		RETURN, 0
	IF INSTRUMENT EQ 'NIS' AND PKTID NE '12'X AND PKTID NE '22'X THEN $
		RETURN, 0
;
;  The packet counter must be larger than the previous packet counter.
;
	PACKETCOUNTER = PACKET.PKT.PACKETCOUNTER AND '7FFF'X
	IF PACKETCOUNTER LE COUNTER THEN RETURN, 0
;
;  Check the exposure number, study counter and raster counter.
;
	IF EXPOSURE                 NE PACKET.PKT.EXPOSURENUMBER THEN RETURN, 0
	IF HEADER.SCI.STUDYCOUNTER  NE PACKET.PKT.STUDYCOUNTER   THEN RETURN, 0
	IF HEADER.SCI.RASTERCOUNTER NE PACKET.PKT.RASTERCOUNTER  THEN RETURN, 0
;
;  Otherwise, we have to assume that the packet is part of the current
;  exposure.  Update COUNTER and return.
;
	COUNTER = PACKETCOUNTER
	RETURN, 1
	END
