	PRO UNLOAD_CDROM, INPUT, DESTINATION=DESTINATION
;+
; Project     :	SOHO - CDS
;
; Name        :	UNLOAD_CDROM
;
; Purpose     :	Unloads files loaded by LOAD_CDROM
;
; Category    :	Class3, I/O
;
; Explanation :	Removes CDS level-0 telemetry files loaded by the routine
;		LOAD_CDROM.  Files are deleted up to, but not past, a certain
;		point.  This is so that some continuity can be provided when
;		loading the next set of CDROMs.
;
; Syntax      :	UNLOAD_CDROM, DATE
;		UNLOAD_CDROM, FILENAME		
;
; Examples    :	UNLOAD_CDROM, '1995/12/20 11:57:00'
;		UNLOAD_CDROM, 'tm.95_254_11'
;
; Inputs      :	One of the following input parameters must be entered.
;
;		DATE	 = A date/time value.  Can be in any one of the CDS
;			   time formats.  All files which have times less than
;			   this value, *except* for the last at each of the
;			   three CDS telemetry rates, will be deleted.
;
;		FILENAME = The name of a level-0 telemetry file.  All files
;			   with dates prior to this file are deleted.  The
;			   specified file itself is *not* deleted.
;
;			   When called with the FILENAME option, the telemetry
;			   time is extracted from the filename, so that it is
;			   is equivalent to using the date option.
;
; Opt. Inputs :	None.
;
; Outputs     :	None.
;
; Opt. Outputs:	None.
;
; Keywords    :	DESTINATION = The directory containing the CDROM files.  (The
;			      name of this keyword was chosen to be consistent
;			      with LOAD_CDROM.)  The default is given by the
;			      environment variable CDS_CDROM_DATA_W.
;
; Calls       :	DATATYPE, ANYTIM2UTC, CONCAT_DIR
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Prev. Hist. :	None.
;
; History     :	Version 1, 23-Jan-1996, William Thompson, GSFC
;		Version 2, 29-Jan-1996, William Thompson, GSFC
;			Treat each telemetry mode separately.
;		Version 3, 31-Jan-1996, William Thompson, GSFC
;			Rewrote because LOAD_CDROM now writes out "tm." format
;			files.
;			Use environment variable CDS_CDROM_DATA_W to avoid
;			affecting realtime data.
;		Version 4, 10-May-1996, William Thompson, GSFC
;			Changed typo involving environment variable
;		Version 5, 18-Jun-1996, William Thompson, GSFC
;			Call FIND_FILE instead of FINDFILE
;
; Contact     :	WTHOMPSON
;-
;
	ON_ERROR, 2
;
;  Check the input parameter.
;
	IF N_PARAMS() NE 1 THEN MESSAGE,	$
		'Syntax:  UNLOAD_CDROM, FILENAME or UNLOAD_CDROM, DATE'
;
;  If the input parameter is a string, decide whether or not it is a filename
;  or a date string.
;
	IF DATATYPE(INPUT,1) EQ 'String' THEN BEGIN
		IF STRLOWCASE(STRMID(INPUT,0,3)) EQ 'tm.' THEN BEGIN
			DATE = STRMID(INPUT,3,2) + '-' +	$
				STRMID(INPUT,6,3) + ' ' +	$
				STRMID(INPUT,10,2)
		ENDIF ELSE DATE = INPUT
	ENDIF ELSE DATE = INPUT
;
;  Convert the DATE into CCSDS format.
;
	DATE = ANYTIM2UTC(DATE, /CCSDS, /TRUNCATE)
;
;  Determine the destination directory.
;
	IF N_ELEMENTS(DESTINATION) EQ 0 THEN BEGIN
		DESTINATION = GETENV('CDS_CDROM_DATA_W')
		IF DESTINATION EQ '' THEN MESSAGE,	$
			'Environment variable CDS_CDROM_DATA_W not defined'
	ENDIF
;
;  Determine the delete command to use.
;
	CASE OS_FAMILY() OF
		'unix':	DELETE = 'rm'
		'vms':	DELETE = 'DELETE'
		ELSE:	MESSAGE, 'Only supported in VMS or Unix'
	ENDCASE
;
;  Find all the level-0 telemetry files in the destination directory, and
;  extract the time parts of the dates.
;
	FILES = FIND_FILE(CONCAT_DIR(DESTINATION, 'tm.*'), COUNT=COUNT)
	IF COUNT EQ 0 THEN MESSAGE,	$
		'No level-0 telemetry files found in ' + DESTINATION
	BREAK_FILE, FILES, DISKS, DIRS, NAMES, EXTS
	DATES = STRMID(EXTS,1,2) + '-' + STRMID(EXTS,4,3) + ' ' +	$
		STRMID(EXTS,8,2)
	DATES = ANYTIM2UTC(DATES, /CCSDS, /TRUNCATE)
;
;  Determine which files have timestamps up to or including the specified time.
;  Filter out the last one.
;
	W = WHERE(DATES LE DATE, COUNT)
	IF COUNT LE 1 THEN MESSAGE, 'No files to unload'
	W = W(0:COUNT-2)
;
;  Remove the files.
;
	FOR I = 0, N_ELEMENTS(W)-1 DO BEGIN
		FILE = FILES(W(I))
		BREAK_FILE, FILE, DISK, DIR, NAME, EXT
		PRINT, NAME + EXT
		SPAWN, DELETE + ' ' + FILE
	ENDFOR
;
	RETURN
	END
