;+
; Project     :	SOHO - CDS
;
; Name        :	ADD_IEFTAB
;
; Purpose     :	Adds an IEF flag table parameter data set to the IEFTAB database.
;
; Explanation : Adds the parameters which define the flag generation scheme 
;               to the IEFTAB database. Assigns the ID number.
;
; Use         : < result = add_ieftab ( def ) >
;
; Inputs      : def = anonymous structure containing IEF table database values.
;                  def.ieftab_id  = I*2      IEF database ID.
;                                            Initially, this is zero.  It is updated by this routine to
;				             reflect the ID number in the database.
;                  def.desc       = C*65     Description of event generation scheme.
;                  def.upp_era    = 4(I*2)   Upper limit for each ERA measure (unsigned INT16)
;                  def.low_era    = 4(I*2)   Lower limit for each ERA measure (unsigned INT16)
;                  def.discrim    = I*2      Discriminator level (unsigned INT16)
;                  def.ev_type    = I*2      Event type (4 bits)
;                  def.pmode      = I*2      Pointing mode : NIS=0, VF=1, GIS=2 (2 bits)
;                  def.cmode      = I*2      Centroid mode : POS=0, NEG=1, ABS=2 (2 bits)
;                  def.c_banda    = I*2      Slit centre band A (INT16)
;                  def.c_bandb    = I*2      Slit centre band B (INT16)
;                  def.c_row      = I*2      Central row (INT16)
;                  def.detector   = C*1      'N'= NIS, 'G'= GIS.
;                  def.n_iefwins  = I*2      No. of IEF windows (<=3)
;                  def.iefwin_ids = 3(I*2)   IEFWIN database IDs
;
; Outputs     :	Returns database ID no. in input structure.
;        	The result of the function is a logical value representing
;		whether or not the operation was successful, where 1 is
;		successful and 0 is unsuccessful.
;
; Keywords    :	
;       ERRMSG    = If defined and passed, then any error messages will be
;                   returned to the user in this parameter rather than
;                   depending on the MESSAGE routine in IDL.  If no errors are
;                   encountered, then a null string is returned.  In order to
;                   use this feature, ERRMSG must be defined first, e.g.
;
;                       ERRMSG = ''
;                       Result = ADD_IEFTAB( ERRMSG=ERRMSG, ... )
;                       IF ERRMSG NE '' THEN ...
;
; Restrictions:	Only this routine can be used to add IEF table definitions to
;		the database.  Modifying the database by hand could corrupt its
;		integrity.
;
;		The data types and sizes of the structure elements must match
;		the definitions in the database.  The string lengths must not
;		exceed the lengths defined in the database.
;
; Side effects:	Once a window list definition has been added to the database,
;		it must never be removed.
;
; Calls       :	dbopen, dbext, dbinfo, dbbuild, dbclose, unsign.
;                
; Common      :	None.
;
; Category    :	Command preparation.
;
; Prev. Hist. :	Adapted from add_datawin.
;
; Written     :	Version 0.0, Martin Carter, RAL, 18/10/95
;
; Modified    :	
;
; Version     :	Version 0.0, 18/10/95
;-
;**********************************************************

FUNCTION add_ieftab, def, ERRMSG=errmsg
;
	ON_ERROR, 2
;
;  Make sure that the ID keyword is initialized
;
	ieftab_id = -1
;
;  Initialize RESULT to represent non-success.  If the routine is successful,
;  this value will be updated below.
;
	result = 0
;
;  Check the input parameters
;
        IF N_PARAMS() NE 1 THEN BEGIN
           message = 'Syntax:  Result = ADD_IEFTAB ( def )'
	   GOTO, HANDLE_ERROR
        ENDIF
;
;  Make sure that the user has privilege to write into the database.
;
	IF !PRIV LT 2 THEN BEGIN
           message = '!PRIV must be 2 or greater to write into the database'
           GOTO, HANDLE_ERROR
	ENDIF
;
;  Make sure that the data window list ID number is less than or equal to zero.
;
	IF def.ieftab_id GT 0 THEN BEGIN
           message= 'The window list ID number must be LE 0. '+$
                    'The final value is assigned by this routine'
           GOTO, HANDLE_ERROR
	ENDIF
;
;  Check each of the structure components to verify that it is of the correct
;  type and size.  
;
	IF DATATYPE ( def.desc, 1 ) NE 'String' THEN BEGIN
           message = 'Tag DESC must be a character string'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS (  def.desc ) NE 1 THEN BEGIN
           message = 'Tag DESC must be a scalar'
           GOTO, HANDLE_ERROR
	END ELSE IF STRLEN ( def.desc ) GT 65 THEN BEGIN
           message = 'Tag DESC must be 65 characters or less'
           GOTO, HANDLE_ERROR
	ENDIF

	IF DATATYPE ( def.upp_era, 1 ) NE 'Integer' THEN BEGIN
           message = 'Tag UPP_ERA must be a short integer'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS ( def.upp_era ) NE 4 THEN BEGIN
           message = 'Tag UPP_ERA must have four elements'
           GOTO, HANDLE_ERROR
	ENDIF

	IF DATATYPE ( def.low_era, 1 ) NE 'Integer' THEN BEGIN
           message = 'Tag LOW_ERA must be a short integer'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS ( def.low_era ) NE 4 THEN BEGIN
           message = 'Tag LOW_ERA must have four elements'
           GOTO, HANDLE_ERROR
	ENDIF

        dummy = WHERE ( unsign(def.low_era) GE unsign(def.upp_era), count )

        IF count NE 0 THEN BEGIN
           message = 'Tag LOW_ERA must be less than UPP_ERA'
           GOTO, HANDLE_ERROR
        ENDIF

	IF DATATYPE ( def.discrim, 1 ) NE 'Integer' THEN BEGIN
           message = 'Tag DISCRIM must be a short integer'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS ( def.discrim ) NE 1 THEN BEGIN
           message = 'Tag DISCRIM must be a scalar'
           GOTO, HANDLE_ERROR
	ENDIF

	IF DATATYPE ( def.ev_type, 1 ) NE 'Integer' THEN BEGIN
           message = 'Tag EV_TYPE must be a short integer'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS ( def.ev_type ) NE 1 THEN BEGIN
           message = 'Tag EV_TYPE must be a scalar'
           GOTO, HANDLE_ERROR
        END ELSE IF (def.ev_type AND NOT 'F'X ) NE 0 THEN BEGIN
           message = 'Tag EV_TYPE must be only 4 bits'
           GOTO, HANDLE_ERROR
	ENDIF

	IF DATATYPE ( def.pmode, 1 ) NE 'Integer' THEN BEGIN
           message = 'Tag PMODE must be a short integer'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS ( def.pmode ) NE 1 THEN BEGIN
           message = 'Tag PMODE must be a scalar'
           GOTO, HANDLE_ERROR
        END ELSE IF (def.pmode LT 0 OR def.pmode GT 2 ) NE 0 THEN BEGIN
           message = 'Tag PMODE must be in range 0-2'
           GOTO, HANDLE_ERROR
	ENDIF

	IF DATATYPE ( def.cmode, 1 ) NE 'Integer' THEN BEGIN
           message = 'Tag CMODE must be a short integer'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS ( def.cmode ) NE 1 THEN BEGIN
           message = 'Tag CMODE must be a scalar'
           GOTO, HANDLE_ERROR
        END ELSE IF (def.cmode LT 0 OR def.cmode GT 2 ) NE 0 THEN BEGIN
           message = 'Tag CMODE must be in range 0-2'
           GOTO, HANDLE_ERROR
	ENDIF

	IF DATATYPE ( def.c_banda, 1 ) NE 'Integer' THEN BEGIN
           message = 'Tag C_BANDA must be a short integer'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS ( def.c_banda ) NE 1 THEN BEGIN
           message = 'Tag C_BANDA must be a scalar'
           GOTO, HANDLE_ERROR
        END ELSE IF (def.c_banda LT 0 OR def.c_banda GT 1023 ) NE 0 THEN BEGIN
           message = 'Tag C_BANDA must be in range 0-1023'
           GOTO, HANDLE_ERROR
	ENDIF

	IF DATATYPE ( def.c_bandb, 1 ) NE 'Integer' THEN BEGIN
           message = 'Tag C_BANDB must be a short integer'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS ( def.c_bandb ) NE 1 THEN BEGIN
           message = 'Tag C_BANDB must be a scalar'
           GOTO, HANDLE_ERROR
        END ELSE IF (def.c_bandb LT 0 OR def.c_bandb GT 1023 ) NE 0 THEN BEGIN
           message = 'Tag C_BANDB must be in range 0-1023'
           GOTO, HANDLE_ERROR
	ENDIF

	IF DATATYPE ( def.c_row, 1 ) NE 'Integer' THEN BEGIN
           message = 'Tag C_ROW must be a short integer'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS ( def.c_row ) NE 1 THEN BEGIN
           message = 'Tag C_ROW must be a scalar'
           GOTO, HANDLE_ERROR
        END ELSE IF (def.c_row LT 0 OR def.c_row GT 1023 ) NE 0 THEN BEGIN
           message = 'Tag C_ROW must be in range 0-1023'
           GOTO, HANDLE_ERROR
	ENDIF

        IF def.c_banda LE def.c_bandb OR def.c_row LE def.c_bandb OR def.c_row GE def.c_banda THEN BEGIN
           message = 'Tags C_ROW, C_BANDA, C_BANDB in wrong order'
           GOTO, HANDLE_ERROR
        ENDIF

	IF DATATYPE ( def.detector, 1 ) NE 'String' THEN BEGIN
           message = 'Tag DETECTOR must be a character string'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS ( def.detector )  NE 1 THEN BEGIN
           message = 'Tag DETECTOR must be a scalar'
           GOTO, HANDLE_ERROR
	END ELSE IF STRLEN ( def.detector ) NE 1 THEN BEGIN
           message = 'Tag DETECTOR must be a single character'
           GOTO, HANDLE_ERROR
	END ELSE IF ( def.detector NE 'G') AND ( def.detector NE 'N') THEN BEGIN
           message = 'Tag DETECTOR must be either "G" or "N"'
           GOTO, HANDLE_ERROR
	ENDIF

	IF DATATYPE ( def.n_iefwins, 1 ) NE 'Integer' THEN BEGIN
           message = 'Tag N_IEFWINS must be a short integer'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS ( def.n_iefwins ) NE 1 THEN BEGIN
           message = 'Tag N_IEFWINS must be a scalar'
           GOTO, HANDLE_ERROR
        END ELSE IF (def.n_iefwins LT 1 OR def.n_iefwins GT 3 ) NE 0 THEN BEGIN
           message = 'Tag N_IEFWINS must be in range 1-3'
           GOTO, HANDLE_ERROR
	ENDIF

	IF DATATYPE ( def.iefwin_ids, 1 ) NE 'Integer' THEN BEGIN
           message = 'Tag IEFWIN_IDS must be a short integer'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS ( def.iefwin_ids ) NE 3 THEN BEGIN
           message = 'Tag IEFWIN_IDS must have three elements'
           GOTO, HANDLE_ERROR
	ENDIF
;
;  Open the IEF table database for write access.
;
	dbopen, 'ieftab', 1
;
;  Find the largest ID number currently in the database, and add
;  one to it. 
;
	n_entries = DB_INFO ( 'entries', 'ieftab')
	IF n_entries EQ 0 THEN ieftab_id = 1 ELSE BEGIN
		dbext, -1, 'ieftab_id', ieftab_ids
		ieftab_id = MAX (ieftab_ids ) + 1
	ENDELSE

;  Append the data to the opened database.

        dbbuild, ieftab_id, def.desc, def.upp_era, def.low_era, def.discrim, $
                 def.ev_type, def.pmode, def.cmode, def.c_banda, def.c_bandb, $
                 def.c_row, def.detector, def.n_iefwins, def.iefwin_ids, STATUS=status
	IF status EQ 0 THEN BEGIN
           message = 'Write to ieftab database was not successful'
           GOTO, HANDLE_ERROR
        ENDIF

;  Update the ID number in the structure.  Signal success.
;
	def.ieftab_id = ieftab_id
	result = 1
	GOTO, FINISH
;
;  Error handling point.
;
HANDLE_ERROR:
	IF N_ELEMENTS ( errmsg ) NE 0 THEN errmsg = 'ADD_IEFTAB: ' + message $
		ELSE MESSAGE, message, /CONTINUE
;
;  Close the database, and return whether the routine was successful or not.
;
FINISH:
	dbclose
;
	RETURN, result

	END