;+
; Project     :	SOHO - CDS
;
; Name        :	ADD_IEFWIN
;
; Purpose     :	Adds an IEF window flag parameter data set to the IEFWIN database.
;
; Explanation : Adds the parameters which define the flag generation scheme for a window
;               to the IEFWIN database. Assigns the ID number.
;
; Use         : < result = add_iefwin ( def ) >
;
; Inputs      : def = structure containing IEF window database values.
;                 def.iefwin_id  = I*2    IEF window ID.
;                                         Initially, this is zero.  It is updated by this routine to
;				          reflect the ID number in the database.
;                 def.win_name   = C*40   Window name
;                 def.ev_desc    = C*65   Description of event detection scheme employed
;                 def.detector   = C*1    'N' = NIS, 'G' = GIS
;                 def.win_def    = 4(I*2) Window definition ( xstart, ystart, xsize, ysize )
;                 def.px_offset  = I*2    Pixel offset defining ROI around peak pixel.
;                 def.exp_era    = 4(I*2) Expected values for each ERA in window.
;                 def.dev_era    = 4(I*2) Deviations for each ERA in window.
;                 def.wgt_era    = 4(I*2) Weights for each direct ERA in window.
;                 def.wgt_dera   = 4(I*2) Weights for each differential ERA in window.
;
; Outputs     :	Returns database ID no. in input structure.
;        	The result of the function is a logical value representing
;		whether or not the operation was successful, where 1 is
;		successful and 0 is unsuccessful.
;
; Keywords    :	
;       ERRMSG    = If defined and passed, then any error messages will be
;                   returned to the user in this parameter rather than
;                   depending on the MESSAGE routine in IDL.  If no errors are
;                   encountered, then a null string is returned.  In order to
;                   use this feature, ERRMSG must be defined first, e.g.
;
;                       ERRMSG = ''
;                       Result = ADD_IEFWIN( ERRMSG=ERRMSG, ... )
;                       IF ERRMSG NE '' THEN ...
;
; Restrictions:	Only this routine can be used to add window list definitions to
;		the database.  Modifying the database by hand could corrupt its
;		integrity.
;
;		The data types and sizes of the structure elements must match
;		the definitions in the database.  The string lengths must not
;		exceed the lengths defined in the database.
;
; Side effects:	Once a window list definition has been added to the database,
;		it must never be removed.
;
; Calls       :	dbopen, dbext, dbinfo, dbbuild, dbclose.
;                
; Common      :	None.
;
; Category    :	Command preparation.
;
; Prev. Hist. :	Adapted from add_datawin.
;
; Written     :	Version 0.0, Martin Carter, RAL, 18/10/95
;
; Modified    :	
;
; Version     :	Version 0.0, 18/10/95
;-
;**********************************************************

FUNCTION add_iefwin, def, ERRMSG=errmsg
;
	ON_ERROR, 2
;
;  Make sure that the ID keyword is initialized.
;
	iefwin_id = -1
;
;  Initialize RESULT to represent non-success.  If the routine is successful,
;  this value will be updated below.
;
	result = 0
;
;  Check the input parameters
;
        IF N_PARAMS() NE 1 THEN BEGIN
           message = 'Syntax:  Result = ADD_IEFWIN ( def )'
	   GOTO, HANDLE_ERROR
        ENDIF
;
;  Make sure that the user has privilege to write into the database.
;
	IF !PRIV LT 2 THEN BEGIN
           message = '!PRIV must be 2 or greater to write into the database'
           GOTO, HANDLE_ERROR
	ENDIF
;
;  Make sure that the data window list ID number is less than or equal to zero.
;
	IF def.iefwin_id GT 0 THEN BEGIN
           message= 'The window list ID number must be LE 0. '+$
                    'The final value is assigned by this routine'
           GOTO, HANDLE_ERROR
	ENDIF
;
;  Check each of the structure components to verify that it is of the correct
;  type and size. 
;
	IF DATATYPE ( def.win_name, 1 ) NE 'String' THEN BEGIN
           message = 'Tag WIN_NAME must be a character string'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS (  def.win_name ) NE 1 THEN BEGIN
           message = 'Tag WIN_NAME must be a scalar'
           GOTO, HANDLE_ERROR
	END ELSE IF STRLEN ( def.win_name ) GT 40 THEN BEGIN
           message = 'Tag WIN_NAME must be 40 characters or less'
           GOTO, HANDLE_ERROR
	ENDIF

	IF DATATYPE ( def.ev_desc, 1 ) NE 'String' THEN BEGIN
           message = 'Tag EV_DESC must be a character string'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS (  def.ev_desc ) NE 1 THEN BEGIN
           message = 'Tag EV_DESC must be a scalar'
           GOTO, HANDLE_ERROR
	END ELSE IF STRLEN ( def.ev_desc ) GT 65 THEN BEGIN
           message = 'Tag EV_DESC must be 65 characters or less'
           GOTO, HANDLE_ERROR
	ENDIF

	IF DATATYPE ( def.detector, 1 ) NE 'String' THEN BEGIN
           message = 'Tag DETECTOR must be a character string'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS ( def.detector )  NE 1 THEN BEGIN
           message = 'Tag DETECTOR must be a scalar'
           GOTO, HANDLE_ERROR
	END ELSE IF STRLEN ( def.detector ) NE 1 THEN BEGIN
           message = 'Tag DETECTOR must be a single character'
           GOTO, HANDLE_ERROR
	END ELSE IF ( def.detector NE 'G') AND ( def.detector NE 'N') THEN BEGIN
           message = 'Tag DETECTOR must be either "G" or "N"'
           GOTO, HANDLE_ERROR
	ENDIF

	IF DATATYPE ( def.win_def, 1 ) NE 'Integer' THEN BEGIN
           message = 'Tag WIN_DEF must be a short integer'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS ( def.win_def ) NE 4 THEN BEGIN
           message = 'Tag WIN_DEF must have four elements'
           GOTO, HANDLE_ERROR
        END ELSE IF def.detector EQ 'N' THEN BEGIN

            ; NIS detector

            IF def.win_def(0) LT 0 OR def.win_def(1) LT 0 OR $
               def.win_def(2) LE 0 OR def.win_def(3) LE 0 OR $
               def.win_def(0) GT 1023 OR def.win_def(1) GT 1023 OR $
               (def.win_def(0)+def.win_def(2)) GT 1023 OR (def.win_def(1)+def.win_def(3)) GT 1023 $
               THEN BEGIN
                 message = 'Tag WIN_DEF has invalid values'
                 GOTO, HANDLE_ERROR
            ENDIF

        ENDIF ELSE BEGIN

            ; GIS detector

            IF def.win_def(0) LT 0 OR def.win_def(1) LE 0 OR $
               def.win_def(1) GT 1023 OR (def.win_def(0)+def.win_def(1)) GT 1023 $
               THEN BEGIN
                 message = 'Tag WIN_DEF has invalid values'
                 GOTO, HANDLE_ERROR
            ENDIF

	ENDELSE

	IF DATATYPE ( def.px_offset, 1 ) NE 'Integer' THEN BEGIN
           message = 'Tag PX_OFFSET must be a short integer'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS ( def.px_offset ) NE 1 THEN BEGIN
           message = 'Tag PX_OFFSET must be a scalar'
           GOTO, HANDLE_ERROR
        END ELSE IF (def.px_offset AND NOT '1F'X ) NE 0 THEN BEGIN
           message = 'Tag PX_OFFSET must be only 5 bits'
           GOTO, HANDLE_ERROR
	ENDIF

	IF DATATYPE ( def.exp_era, 1 ) NE 'Integer' THEN BEGIN
           message = 'Tag EXP_ERA must be a short integer'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS ( def.exp_era ) NE 4 THEN BEGIN
           message = 'Tag EXP_ERA must have four elements'
           GOTO, HANDLE_ERROR
        ENDIF 

	IF DATATYPE ( def.dev_era, 1 ) NE 'Integer' THEN BEGIN
           message = 'Tag DEV_ERA must be a short integer'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS ( def.dev_era ) NE 4 THEN BEGIN
           message = 'Tag DEV_ERA must have four elements'
           GOTO, HANDLE_ERROR
	ENDIF

	IF DATATYPE ( def.wgt_era, 1 ) NE 'Integer' THEN BEGIN
           message = 'Tag WGT_ERA must be a short integer'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS ( def.wgt_era ) NE 4 THEN BEGIN
           message = 'Tag WGT_ERA must have four elements'
           GOTO, HANDLE_ERROR
        ENDIF ELSE BEGIN

           dummy = WHERE ( (def.wgt_era AND NOT 'FF'X) NE 0, count )

           IF count GT 0 THEN BEGIN
             message = 'Tag WGT_ERA must be only 8 bits'
             GOTO, HANDLE_ERROR
           ENDIF

        ENDELSE

	IF DATATYPE ( def.wgt_dera, 1 ) NE 'Integer' THEN BEGIN
           message = 'Tag WGT_DERA must be a short integer'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS ( def.wgt_dera ) NE 4 THEN BEGIN
           message = 'Tag WGT_DERA must have four elements'
           GOTO, HANDLE_ERROR
        ENDIF ELSE BEGIN

           dummy = WHERE ( (def.wgt_dera AND NOT 'FF'X) NE 0, count )

           IF count GT 0 THEN BEGIN
             message = 'Tag WGT_DERA must be only 8 bits'
             GOTO, HANDLE_ERROR
           ENDIF

        ENDELSE

        ; check weights defined only where deviations defined

        list = WHERE ( def.dev_era EQ 0, count )

        IF count GT 0 THEN BEGIN

           IF TOTAL ( (def.wgt_era(list) NE 0) OR (def.wgt_dera(list) NE 0) ) NE 0 THEN BEGIN
             message = 'Tag DEV_ERA not defined, but weights defined'
             GOTO, HANDLE_ERROR
           ENDIF

        ENDIF
;
;  Open the IEF window database for write access.
;
	dbopen, 'iefwin', 1
;
;  Find the ID number currently in the database, and add
;  one to it.
;
	n_entries = DB_INFO ( 'entries', 'iefwin')
	IF n_entries EQ 0 THEN iefwin_id = 1 ELSE BEGIN
		dbext, -1, 'iefwin_id', iefwin_ids
		iefwin_id = MAX (iefwin_ids ) + 1
	ENDELSE

;  Append the data to the opened database.
;
        dbbuild, iefwin_id, def.win_name, def.ev_desc, def.detector, def.win_def, $
                 def.px_offset, def.exp_era, def.dev_era, def.wgt_era, def.wgt_dera, STATUS=status
	IF status EQ 0 THEN BEGIN
           message = 'Write to iefwin database was not successful'
           GOTO, HANDLE_ERROR
        ENDIF

;  Update the data window list ID number in the structure.  Signal success.
;
	def.iefwin_id = iefwin_id
	result = 1
	GOTO, FINISH
;
;  Error handling point.
;
HANDLE_ERROR:
	IF N_ELEMENTS ( errmsg ) NE 0 THEN errmsg = 'ADD_IEFWIN: ' + message $
		ELSE MESSAGE, message, /CONTINUE
;
;  Close the database, and return whether the routine was successful or not.
;
FINISH:
	dbclose
;
	RETURN, result

	END