;+
; Project     :	SOHO - CDS
;
; Name        :	CP_STATE
;
; Purpose     :	Widget tool for accessing the CDHS state database.
;
; Explanation : The CDHS state database contains information on the current settings of various 
;               quasi-static CDS parameters. This information is used by various routines in the 
;               planning tools and to create the list of commands for changing the CDS defaults.
;               This tool allows parameters to be added to the database and current settings to be 
;               changed; it allows the information in the database to be displayed and allows a file
;               containing the list of CDS default update commands to be generated.
;              
;               Each entry in the CDHS state database contains information on a CDS parameter in the 
;               following form :
;                  DATE        C*20  String containing UTC date entry created
;                  MNEMONIC    C*8   String containing command mnemonic
;                  PNUMBER     I*2   Parameter number associated with this entry
;                  NUMBERP     I*2   Number of parameters associated with this command mnemonic
;                  LOAD        I*2   Flag indicating parameter should be loaded
;                  DELAY       I*2   Delay in seconds associated with command.
;                  ACTIVE      I*4   Current setting for parameter
;                  DEFAULT     I*4   Default setting for parameter
;                  COMMENT     C*80  String containing comment
;
;               This tool provides a number of function buttons and text boxes for entering the 
;               values for each of the above database entries.
;               Functions :
;      
;                    EXIT : Exit tool.
;                   
;                    SAVE : Save current text box entries.
;                           Note that the date saved with the entry is the current time rather than the 
;                           time in the date text box and the mnemonic is converted to uppercase. 
;                           The display is updated with what was written to the database.
;                           At present the date text box is non-editable.
;                           Checks that pnumber and numberp are consistent.
;
;                    RETRIEVE : Retrieves the latest entries in the database with the command
;                           mnemonic in the mnemonic text box. If more than one pnumber in stored
;                           entries then displays all of these in a separate text window and 
;                           updates the tool display with either the entry corresponding to the 
;                           pnumber box or the first entry if the latter does not exist. 
;                           If no entry can be found with the given mnemonic then returns the
;                           mnemonic : "Unknown" and a warning message.
;                           
;                    ALL  : Retrieves all the latest entries in the database and displays these in
;                           a separate text window. 
;                           If no entries can be found then produces a popup message.
;                           Result may be printed to a file or default printer.
;   
;                    LOAD : Retrieves all the latest loadable entries from the database. 
;                           Produces an update.dt output file containing the commands to be 
;                           loaded and displays the commands in a text window. If any problem
;                           found with any command then produces message and does not output
;                           command to file (though command is displayed in text window).
;
;                    HISTORY :  Retrieves the history for a given command mnemonic and parameter
;                           number. Result may be printed to a file or default printer.
;
;               On detection of any errors within the software the IDL MESSAGE function is used 
;               to pinpoint the error. Since this application is widget based ON_ERROR=2 is used
;               to return control to the IDL command line.
;                  
; Use         : <cp_state>
;
; Inputs      : None.
;
; Opt. Inputs : None.
;
; Outputs     : Updates CDHS state database and produces update.dt command file.
;
; Opt. Outputs:	None.
;
; Keywords    : None.
;
; Calls       :	blok_cp_state, setw_cp_state, getw_cp_state, show_cp_state,
;               save_cp_state, retr_cp_state, disp_cp_state, cp_state_event.
;                
; Common      :	cpinputwdblock.
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Category    :	Command preparation.
;
; Prev. Hist. :	None.
;
; Written     :	Version 0.0, Martin Carter, RAL, 24/5/94
;
; Modified    :	Version 0.1, MKC, 11/8/95
;                            Changed active and default to I*4.
;                            Added comment field.
;                            Stopped history being displayed if no entries.
;                            Added print option.
;               Version 0.2, MKC, 25/9/95
;                            Added non fatal treatment of error conditions to avoid
;                            IDL hanging up on errors.
;               Version 0.3, MKC, 18/10/95
;                            Changed name from cp_input_wd to cp_state.
;                            Added check that pnumber and numberp are conistent when data 
;                            saved and retrieved.
;               Version 0.4, MKC, 18/12/95
;                            Added delay field to database and extended the comment field.
;		Version 5, William Thompson, GSFC, 9 September 1996
;			Made compatible with IDL version 4.
;      
; Version     :	Version 5, 9 September 1996
;-
;**********************************************************

;**********************************************************
; blok_cp_state : sets up common block
;**********************************************************

PRO blok_cp_state

  COMMON cpinputwdblock, struct, stids

  ; setup parameter structure

  s = cp_get_mnemonic ( 'Unknown', /QUIET )

  struct = s(0)

  tags   = TAG_NAMES(struct)

  stids  = LONARR(N_ELEMENTS(tags))

END

;**********************************************************
; setw_cp_state : sets up tool label and text widgets
;**********************************************************

PRO setw_cp_state, mainbase

  COMMON cpinputwdblock, struct, stids

  tags = TAG_NAMES(struct)   ; get tag names

  lsize = 300
  tsize = 22

  base  = WIDGET_BASE ( mainbase, /COLUMN)

  FOR k = 0, N_TAGS(struct)-1 DO BEGIN

    row      = WIDGET_BASE ( base, /ROW)

    label    = WIDGET_LABEL ( row, VALUE=tags(k) )

    ; make date tag non editable
    
    IF tags(k) EQ 'DATE' THEN $
      stids(k) = WIDGET_TEXT  ( row, XSIZE=tsize, /FRAME, $
                                VALUE = STRING(struct.(k)), UVALUE='') ELSE $
      stids(k) = WIDGET_TEXT  ( row, XSIZE=tsize, /EDITABLE, /FRAME, $
                                VALUE = STRING(struct.(k)), UVALUE='')

  ENDFOR

END

;**********************************************************
; getw_cp_state : retrieves setting of text widgets into current values
;**********************************************************

PRO getw_cp_state

  COMMON cpinputwdblock, struct, stids

  ; get parameters

  FOR k = 0, N_TAGS(struct)-1 DO BEGIN

     WIDGET_CONTROL, stids(k), GET_VALUE = value
     struct.(k) = value(0)

  ENDFOR

  ; force mnemonic to be upper case

  struct.mnemonic = STRUPCASE ( struct.mnemonic )

END

;**********************************************************
; show_cp_state : displays current values in text widgets
;**********************************************************

PRO show_cp_state

  COMMON cpinputwdblock, struct, stids

  FOR k = 0, N_TAGS(struct)-1 DO  WIDGET_CONTROL, stids(k), SET_VALUE=STRING(struct.(k))

END

;**********************************************************
; save_cp_state : saves current values to database
;**********************************************************

PRO save_cp_state

  COMMON cpinputwdblock, struct, stids

  ; Get current utc time in string format
  ; NB scalars passed by value

  get_utc, date, /ECS

  struct.date = date

  ; save into database

  cp_add_mnemonic, struct

END

;**********************************************************
; retr_cp_state : retrieves entry for current mnemonic 
;                    from database and puts into current 
;                    values.
;**********************************************************

PRO retr_cp_state, GROUP=group

  COMMON cpinputwdblock, struct, stids

  ; retrieve mnemonic
  ; NB may get array 

  s = cp_get_mnemonic ( struct.mnemonic, GROUP=group )

  ; see if pnumbers correspond anywhere

  list = WHERE ( s.pnumber EQ struct.pnumber, count)

  IF count EQ 0 THEN struct = s(0) ELSE struct = s(list(0))

  ; display all items if more than one

  IF N_ELEMENTS(s) GT 1 THEN disp_cp_state, s, struct.mnemonic, GROUP=group

END

;**********************************************************
; disp_cp_state : displays array of database entries in 
;                    separate text widget.
;**********************************************************

PRO disp_cp_state, struct, title, GROUP=group

  ; do not display unknown entries

  IF struct(0).mnemonic EQ 'Unknown' THEN RETURN

  ; get tag names

  tags = TAG_NAMES(struct)

  ntags = N_TAGS(struct)

  nstrs = N_ELEMENTS(struct)

  ; create string array 

  strs = STRARR(1+nstrs)

  ; create entries

  FOR j = 1, nstrs DO $
    FOR k = 0, ntags-1 DO strs(j) = strs(j) + STRING(struct(j-1).(k)) + '  '

  ; create header

  ; get lengths of fields

  lens = INTARR(ntags)

  FOR k = 0, ntags-1 DO lens(k) = MAX ( STRLEN ( [ STRING(struct.(k)) + '  ', tags(k) ] ) )

  ; create blank string NB does not like "blank" to be used as variable name

  blnk = BYTARR ( MAX(lens) )
  
  blnk(*) = BYTE ( ' ' )

  ; create array of blank strings

  blnks = STRARR(ntags)

  FOR k = 0, ntags-1 DO blnks(k) = STRMID ( blnk, 0, lens(k) )

  ; put tags into blank strings

  FOR k = 0, ntags-1 DO BEGIN
    temp = blnks(k)
    STRPUT, temp, tags(k) 
    blnks(k) = temp
  ENDFOR

  ; create single string

  FOR k = 0, ntags-1 DO strs(0) = strs(0) +  blnks(k)

  ; display string array

  cp_dsp_strarr, strs, title=title, GROUP=group

END

;**********************************************************
; cp_state_event : event handler for main routine
;**********************************************************

PRO cp_state_event, event

  COMMON cpinputwdblock, struct, stids

  WIDGET_CONTROL, event.id, GET_UVALUE = uvalue

  CASE uvalue OF

   'EXIT' : WIDGET_CONTROL, event.top, /DESTROY

   'SAVE' : BEGIN
               getw_cp_state
               save_cp_state
               show_cp_state
            END

   'RETR' : BEGIN
               getw_cp_state
               retr_cp_state, GROUP=group
               show_cp_state
            END

   'ALLP' : BEGIN
               s = cp_get_all ( GROUP=event.top )
               disp_cp_state, s, 'Current CDHS database entries', GROUP=event.top
            END

   'LOAD' : BEGIN
               s = cp_get_all ( GROUP=event.top )
               s = cp_get_load ( s, GROUP=event.top )
               disp_cp_state, s, 'Loadable CDHS database entries', GROUP=event.top
            END

   'HIST' : BEGIN
               getw_cp_state
               s = cp_get_history ( struct, GROUP=event.top )
               disp_cp_state, s, 'History for command mnemonic', GROUP=event.top
            END

    ELSE  : RETURN

  ENDCASE

END

;**********************************************************
; main routine for widget
;**********************************************************

PRO cp_state, GROUP=group

  ; set up error treatment to be return to calling routine on error

  ON_ERROR, 2

  ; check if already running

  IF XREGISTERED ("cp_state") THEN RETURN

  ; set up common block 

  blok_cp_state

  ; set up main widget

  items = [  ' "EXIT"                     EXIT', $
             ' "SAVE"                     SAVE', $
             ' "ALL"                      ALLP', $
             ' "LOAD"                     LOAD', $
             ' "RETRIEVE"                 RETR', $
             ' "HISTORY"                  HIST'  ]
 
  base = WIDGET_BASE ( TITLE = 'CDHS STATE DATABASE TOOL',/COLUMN)

  XPDMENU, items, base

  setw_cp_state, base
   
  WIDGET_CONTROL, base, /REALIZE

  XMANAGER, "cp_state", base, GROUP_LEADER=group

END
