;+
; Project     :	SOHO - CDS
;
; Name        :	DO_FOLLOWING
;
; Purpose     :	Adds commands to take to initial state of following study.
;
; Explanation :	If the following study is not time-tagged it follows on sequentially from the 
;               current study. This routine inserts commands to move to the initial position
;               of the following study into the series tables associated with the current
;               study.
;               The jump to the series tables associated with the following study is added later.
;
;               Perl script uses format required by CB5FILS i.e token list for getpar :
;
;               1) COMMAND 1 2 3 4                    # simple scc
;
;               2) SCRIPT                             # script
;                  0x1 
;                  0x2
;                  TERMINATE
;
;               NB All commands or scripts must be to macro and so begin CB5* ...
;              
; Use         : <do_following, fseries>
;
; Inputs      : series   = structure array describing current state
;               fseries  = structure array describing following study
;
; Opt. Inputs : None.
;
; Outputs     : Writes commands to command array. Updates series.fstate. 
;
; Opt. Outputs:	None.
;
; Keywords    : None.
;
; Calls       :	get_ops_pos, get_ops_delay, get_slitn_delay, shorthex, fileout, 
;               do_gset_init, macro_wait, safe_slit.
;                
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Category    :	Command preparation.
;
; Prev. Hist. :	Created from do_raster.
;
; Written     :	Version 0.0, Martin Carter, RAL, 7/11/95
;
; Modified    :	Version 0.1, MKC, 24/11/95
;                            Changed so that CVT file shave .dt extension
;                       0.2, MKC, 12/12/95
;                            Added gset initialization.
;                       0.3, MKC, 14/12/95
;                            Adapted from do_previous.
;                       0.4, 20/12/95
;                            Added a bit more printout.
;                       0.5, 16/1/96
;                            Added macro_wait.
;                       0.6, 18/1/96
;                            Only update slit number and OPS position 
;                            if necessary.
;                       0.7, 18/10/96
;                            Added safe slit commands.
;                       0.8, 24/2/97
;                            Added check for special study.
;                       0.9, MKC, 21/4/97
;                            Prevent SW from writing out slitn=0 command explicitly.
;                            Use slitn=0 to signal no MCU commands for special study.
;                            Removed check for special study.
;                            Changed printout to reflect above.
;                             
; Version     :	Version 0.9, 21/4/97
;-
;**********************************************************

PRO do_following, series, fseries

  ; NB series may be an array, fseries will not be an array

  ; series.istate   contains the settings as they were at the start of this study
  ; series.fstate   contains the current settings i.e how this study has left them
  ; fseries.istate  contains the settings required for the next study
  ; The GSET ID is complicated because a NIS study has gsetid = 0 and does not change
  ; the gsetid parameters. However, the gsetid does not change during a study. 
  ; The gset id for the current study is contained in series.istate.gsetid.
  ; The current gset id is contained in series.fstate.gsetid.
  ; The current state (including gsetid) is updated in this routine.

  ; get initial pointing of following study

  new_solarx = fseries.istate.solarx
  new_solary = fseries.istate.solary
  new_slitn  = fseries.istate.slitn
  new_gsetid = fseries.istate.gsetid

  ; get current ops positions

  get_ops_pos, series(0).fstate.solarx, series(0).fstate.solary, current_opsl, current_opsr

  ; get new ops positions

  get_ops_pos, new_solarx, new_solary, new_opsl, new_opsr

  ; work out delays in ms

  opsl_delay  = get_ops_delay   ( current_opsl, new_opsl, 0 )  

  opsr_delay  = get_ops_delay   ( current_opsr, new_opsr, 1 )

  slitn_delay = get_slitn_delay ( series(0).fstate.slitn, new_slitn )

  If !debug THEN PRINT, 'Following study not time-tagged, appending commands to study'

  ; add commands to current study

  fileout, '# Next study follows sequentially'
  fileout, ''

  ; change gsetid if necessary

  IF (new_gsetid GT 0) AND (series(0).fstate.gsetid NE new_gsetid) THEN BEGIN

    do_gset_init, new_gsetid

    ; only update gsetid if changed in study

    series(*).fstate.gsetid = new_gsetid

  ENDIF

  fileout, '# Move to start of next study'
  fileout, '#   Current Solar x  = ' + STRTRIM ( series(0).fstate.solarx, 1 ) + ' arcsecs'
  fileout, '#   Current Solar y  = ' + STRTRIM ( series(0).fstate.solary, 1 ) + ' arcsecs'
  fileout, '#   Current slit no. = ' + STRTRIM ( series(0).fstate.slitn, 1 ) 
  fileout, '#   Current gset_id  = ' + STRTRIM ( series(0).fstate.gsetid, 1 ) 
  fileout, '#   New Solar x      = ' + STRTRIM ( new_solarx, 1 ) + ' arcsecs'
  fileout, '#   New Solar y      = ' + STRTRIM ( new_solary, 1 ) + ' arcsecs'
  fileout, '#   New slit no.     = ' + STRTRIM ( new_slitn, 1 ) 
  fileout, '#   New gset_id      = ' + STRTRIM ( new_gsetid, 1 ) 
  fileout, ''

  ; check explicitly for slitn=0

  IF new_slitn NE 0 THEN BEGIN

    ; check if solarx/y need changing

    IF series(0).fstate.solarx NE new_solarx OR $
       series(0).fstate.solary NE new_solary THEN BEGIN 

      fileout, words=3, 'CB5SIC' + '          # tell macro to tell mcu to move OPS left' 
      fileout, 'CBMOPSLP ' + shorthex ( new_opsl ) + ' # OPS left = ' + STRTRIM ( new_opsl, 1 )
      fileout, 'TERMINATE'
      fileout, ''

      macro_wait, opsl_delay
      fileout, ''

      fileout, words=3, 'CB5SIC' + '          # tell macro to tell mcu to move OPS right'
      fileout, 'CBMOPSRP ' + shorthex ( new_opsr ) + ' # OPS right = ' + STRTRIM ( new_opsr, 1 ) 
      fileout, 'TERMINATE'
      fileout, ''

      macro_wait, opsr_delay
      fileout, ''

    ENDIF

    ; check if slit number needs changing

    IF new_slitn NE series(0).fstate.slitn THEN BEGIN 

      fileout, words=3, 'CB5SIC' + '          # tell macro to tell mcu to change slit number'
      fileout, 'CBMSLITN ' + shorthex ( new_slitn ) + $
               ' # slit number = ' + STRTRIM ( new_slitn, 1 ) 
      fileout, 'TERMINATE'
      fileout, ''

      macro_wait, slitn_delay
      fileout, ''

      ; ensure slit motor turned off

      safe_slit, /SERIES

    ENDIF

    ; update current mechanisms state

    series(*).fstate.slitn  = new_slitn
    series(*).fstate.solarx = new_solarx
    series(*).fstate.solary = new_solary

  ENDIF ELSE BEGIN

    ; slitn=0 

    fileout, '#  No following science studies'
    fileout, ''

  ENDELSE

END

