;+
; Project     :	SOHO - CDS
;
; Name        :	DO_RASTER
;
; Purpose     :	Writes out raster part of CDS SERIES tables.
;
; Explanation :	Writes out the part of a study associated with a raster 
;               in the format required by the Perl script files for 
;               loading onto the CDS.
;               The initial pointing in istate is the pointing of the first raster in the study 
;               whether deferred or fixed. The deferred pointing is taken from the point_details
;               entry if present else is zero. The pointing in fstate is the current pointing.
;               The OPS pointing is output at the start of each raster only if the solarx/y
;               position changes.
;
;               Perl script uses format required by CB5FILS i.e token list for getpar :
;
;               1) COMMAND 1 2 3 4                    # simple scc
;
;               2) SCRIPT                             # script
;                  0x1 
;                  0x2
;                  TERMINATE
;
;               NB All commands or scripts must be to macro and so begin CB5* ...
;               Perl script need format:
;              
; Use         : <do_raster, raster, series, flag_study>
;
; Inputs      : raster              = structure giving information on raster
;                                     table;
;               series              = structure array describing state parameters
;
; Opt. Inputs : None.
;
; Outputs     : Writes raster commands to file. Updates series.fstate.
;               NB must pass series by reference.
;
; Opt. Outputs:	None.
;
; Keywords    : ADJUST_ALLOWED : allow adjustment of mirror and slit positions to avoid 
;                                OPS movement if possible. If ADJUST_ALLOWED=2 then 
;                                explicit movement of the slit is allowed for NIS rasters.
;               ALTSCI         : indicates alternative study which does not want mirror and slit
;                                initial positions commanded.
;		MAX_SLIT_POS   : Use the number passed through this keyword as the maximum
;				 allowed slit position.  This value is used to offset GIS rasters
;				 in the N/S direction in such a way that they do not exceed 
;				 this value.

;
; Calls       :	get_ops_pos, get_ops_delay, get_slitn_delay, shorthex, fileout, macro_wait, get_slitp_delay, safe_slit.
;                
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Category    :	Command preparation.
;
; Prev. Hist. :	None.
;
; Written     :	Version 0.00, Martin Carter, RAL, 14/10/94
;
; Modified    :	Version 0.01, Martin Carter, RAL, 28/11/94
;                            Added proforma.
;                       0.1, MKC, 16/2/95
;                            Changed comments slightly
;                       0.2, MKC, 20/2/95
;                            Changed comments slightly
;                       0.3, MKC, 2/3/95
;                            Changed ntimes to repn
;                       0.4, MKC, 6/3/95
;                            Added raster.exptime tag 
;                            Moved changing detector into routine.
;                            Added state structure.
;                            Implemented solar feature tracking switch.
;                            Implemented calculation of delay times.
;                            No slit movement if NIS detector.
;                       0.5, MKC, 15/3/95
;                            Corrected VDS exposure command to tell macro to set
;                            VDS exposure.
;                       0.6, MKC, 23/3/95
;                            Correct VDS exposure command.
;                       0.7, MKC, 27/3/95
;                            Added command mnemonic for CB5SIC
;                       0.8, MKC, 19/4/95
;                            Changed state structure.
;                            Added new pointing logic.
;                            Changed output lay out.
;                            Removed fundamental flag.
;                            Used STRTRIM in display.
;                       0.9  MKC, 28/4/95
;                            Fiddled with CB5SIC -nosend. Perl script does not seem to need it.
;                       1.0, MKC, 17/5/95
;                            Added mirror and slit offsets to raster start position.
;                            Added initial study state and current state structures to argument list.
;                       1.1, MKC, 1/6/95
;                            Added more printout. Changed argument list so that structures not passed 
;                            by value.
;                       1.2, MKC, 10/8/95
;                            Adjusted relative pointing to be relative to istate rather than fstate.
;                       1.3, MKC, 14/8/95
;                            Modified get_ops_delay argument list.
;                       1.4, MKC, 26/9/95
;                            Removed SFT commands to study level.
;                       1.5, MKC, 29/9/95
;                            Removed sciplan_details from argument list.
;                            Output slit position for NIS raster also.
;                       1.6, MKC, 12/10/95
;                            Added flag_study flag.
;                       1.7, MKC, 6/11/95
;                            Added CB5SIC to CBMSLITN command.
;                       1.8, MKC, 7/11/95
;                            Added gset_id to state structure.
;                            Modified treatment of pointing.
;                       1.9, MKC, 24/1195
;                            Modified series file format.
;                       2.0, MKC, 5/12/95
;                            Added range logic.
;                            Changed so that does not send OPS or slit number commands
;                            if already at required loaction.
;                       2.1, MKC, 12/12/95
;                            Added do_gset_on and do_gset_off.
;                       2.2, MKC, 18/1295
;                            Sent OPS commands even if does not need moving because needed by macro.
;                       2.3, MKC, 2/1/96
;                            Added GIS detector check.
;                            Added flag study logic.
;                            Corrected deferred pointing logic.
;                            Only send OPS commands if necessary again.
;                       2.4, MKC, 16/1/96
;                            Added macro_wait.
;                       2.5, MKC, 29/1/96
;                            Added adjust_allowed keyword.
;                       2.6, MKC, 15/5/96
;                            Added SFT turn on/off every raster.
;                       2.7, MKC, 20/5/96
;                            Corrected bug using series.sft rather than series(0).sft.
;                       2.8, MKC, 8/8/96
;                            Corrected mirror adjust for wrong mirror direction.
;                       2.9, MKC, 5/9/96
;                            Added printout to indicate when mirror/slit adjust is used.
;                            Allowed slit movement pointing for NIS rasters.
;                       3.0, MKC, 7/10/96
;                            Set up raster pointing in cpt_details.
;                            Removed study_raster_detail from argument list.
;                            Only set slit initial slit position in macro if GIS detector.
;                            Added slit step up and down to cure slit oscillation problem in
;                            prime mode. Note this requires turning off MCU backup mode.
;                       3.1, MKC, 21/4/97
;                            Prevent SW from writing out slitn=0 command explicitly.
;                            Now send slit initial slit position to macro if NIS detector also.
;                       3.2, MKC, 9/10/97
;                            Corrected problem with -ve slitp commands -- need to be coded as 9 bit
;                            two's complement.
;                       3.3, MKC, 18/6/98
;                            Added ALTSCI keyword to comment out mirror and slit positions.     
;                       4.0, William Thompson, GSFC, 01-Jul-2003
;                            Look at SOHO_ORIENT environment variable
;                       4.1, REY, 08/05/03
;                            Added code to support GIS rasters along the Y axis (using the slit)
;                            that are not centered in the middle of the slit range.  Added the 
;                            MAX_SLIT_POS keyword to support this mode of operation.      
;                             
; Version     :	Version 4.1, 08/05/03
;-
;**********************************************************

PRO do_raster, raster, series, ADJUST_ALLOWED=adjust_allowed, ALTSCI=altsci, MAX_SLIT_POS = max_slit_pos

  ; NB series may be an array at this point

  ; get current positions
  ; fstate gives current OPS positions and slit number on entry and returns with
  ; updated OPS positions and slit number
  ; istate gives OPS positions and slit number of first raster of study whether
  ; fixed or deferred
  ; series.istate.gsetid gives the gsetid used by this study
  ; series.fstate.gsetid gives the current gsetid
  ; if the study is a GIS study these should be the same since 
  ; gsetid set up before study and is not changed by study
  ; If a NIS study the istate and fstate gsetids may be different and
  ; the fstate is not updated.

  current_solarx = series(0).fstate.solarx
  current_solary = series(0).fstate.solary
  current_slitn  = series(0).fstate.slitn

  ; update current slit state

  new_solarx = raster.solx
  new_solary = raster.soly
  new_slitn  = raster.slitn

  ; output raster header in series table

  fileout, '# Detector = ' + raster.detector
  fileout, '#   Current Solar x  = ' + STRTRIM ( current_solarx, 1 ) + ' arcsecs'
  fileout, '#   Current Solar y  = ' + STRTRIM ( current_solary, 1 ) + ' arcsecs'
  fileout, '#   Current slit no. = ' + STRTRIM ( current_slitn, 1 ) 
  fileout, '#   New Solar x      = ' + STRTRIM ( new_solarx, 1 ) + ' arcsecs'
  fileout, '#   New Solar y      = ' + STRTRIM ( new_solary, 1 ) + ' arcsecs'
  fileout, '#   New slit no.     = ' + STRTRIM ( new_slitn, 1 ) 
  fileout, '#   no. of rasters   = ' + STRTRIM ( raster.repn, 1 ) 
  IF raster.detector EQ 'G' THEN $
    fileout, '#   Gset_id          = ' + STRTRIM ( series(0).istate.gsetid, 1 ) 

  fileout, ''

  ; output raster commands

  ; determine whether any x movement required and whether raster offset may be 
  ; accomplished with mirror movements
  ; NB spans are in steps, one mirror step is 2 arcsecs
  ; increasing mirror moves in -ve x direction

  IF KEYWORD_SET(adjust_allowed) AND $
     (new_solarx NE current_solarx) AND $
     (new_solarx + raster.mirrorspan) LT ( current_solarx + series(0).range ) AND $
     (new_solarx - raster.mirrorspan) GT ( current_solarx - series(0).range ) THEN BEGIN

     ; use mirror movements to perform raster position offset
     ; NB mirror will return to home position at end of raster
     ; If the enviroment variable SOHO_ORIENT is 180, then invert the correction

     delta_solarx = new_solarx - current_solarx
     if getenv('SOHO_ORIENT') eq 180 then delta_solarx = -delta_solarx
     mirror_pos = '80'X - delta_solarx/2 - raster.mirrorspan/2

     ; change new OPS positions

     new_solarx = current_solarx

     ; add comment to printout

     fileout, '# NB. Mirror start position adjusted to give raster position offset'
     fileout, ''
  
  ENDIF ELSE BEGIN

     ; offset mirror position so that raster centred on the home position

     mirror_pos = '80'X - raster.mirrorspan/2

  ENDELSE

  ; If the max_slit_pos keyword is not set, then use the default values to set
  ; the maximum slit position and minimum slit position

  IF NOT KEYWORD_SET (max_slit_pos) THEN BEGIN

     slit_max_pos =   series(0).range
     slit_min_pos = - series(0).range

  ; Otherwise, use parameters derived from the max_slit_pos keyword.

  ENDIF ELSE BEGIN

     slit_max_pos =    max_slit_pos
     slit_min_pos =  - series(0).range

  ENDELSE

  ; determine whether any y movement required and whether raster offset may be 
  ; accomplished with slit movements
  ; only allow for GIS unless adjust_allowed=2
  ; NB spans are in steps, one slit step is 1 arcsec

  IF NOT KEYWORD_SET(adjust_allowed) THEN adjust_allowed = 0

  IF ( ((raster.detector EQ 'G') AND KEYWORD_SET(adjust_allowed)) OR (adjust_allowed EQ 2) ) AND $
     (new_solary NE current_solary ) AND $
     (new_solary + raster.slitspan/2) LT ( current_solary + slit_max_pos ) AND $
     (new_solary - raster.slitspan/2) GT ( current_solary + slit_min_pos ) THEN BEGIN

     ; use slit movements to perform raster position offset
     ; If the enviroment variable SOHO_ORIENT is 180, then invert the correction

     IF raster.detector EQ 'G' THEN BEGIN

       ; NB slit will return to home position at end of GIS raster

       delta_solary = new_solary - current_solary
       if getenv('SOHO_ORIENT') eq 180 then delta_solary = -delta_solary
       slit_pos   = delta_solary - raster.slitspan/2
  
       ; change new OPS positions

       new_solary = current_solary
   
       ; add comment to printout

       fileout, '# NB. Slit start position adjusted to give GIS raster position offset'
       fileout, ''

     ENDIF ELSE BEGIN

       ; explicitly move slit for NIS

       ; NB slit will not return to home position at end of NIS raster

       new_slitp   = new_solary - current_solary
  
       ; change new OPS positions

       new_solary = current_solary
   
       ; get slit position movement delay

       slitp_delay = get_slitp_delay ( 0, new_slitp )

       ; make 9 bit twos complement

       IF new_slitp LT 0 THEN new_slitp = '200'X + new_slitp

       ; set slit position explicitly

       fileout, '# NB. Slit start position set explicitly for NIS raster position offset'
       fileout, ''

       fileout, words=3, 'CB5SIC' + '          # tell macro to tell mcu to move slit position' 
       fileout, 'CBMSLITP ' + shorthex ( new_slitp ) + ' # SLIT position = ' + STRTRIM ( new_slitp, 1 )
       fileout, 'TERMINATE'
       fileout, ''

       macro_wait, slitp_delay
       fileout, ''

       ; ensure slit motor turned off

       safe_slit, /SERIES

       ; offset slit position so that raster centred on the home position
       ; doesn't actually do anything for NIS raster on-board

       slit_pos   = - raster.slitspan/2
   
     ENDELSE    

  ENDIF ELSE BEGIN

    ; check if this is a GIS raster and the MAX_SLIT_POS keyword has been set.

    IF (raster.detector EQ 'G') 	AND	$ 
       KEYWORD_SET (max_slit_pos)       AND     $
       (raster.slitspan GT 0)		THEN BEGIN

       ; Calculate a new slit range and slit center based on the new maximum slit position.

;       new_slit_range  = max_slit_pos + series(0).range
;       new_slit_center = -series(0).range + new_slit_range/2.0
       new_slit_center = max_slit_pos - raster.slitspan/2.0

       fileout, '# NB. Initial position adjusted to allow GIS raster about position: '   $
                + STRTRIM (STRING (new_slit_center), 2)
       fileout, ''

       ; offset slit position so that raster centred on the home position

       slit_pos   = - raster.slitspan/2

       ; adjust the initial slit position for slit movements that are not centered
       ; about the slit home position.

       slit_pos   = slit_pos + new_slit_center
        
    ENDIF ELSE BEGIN

       ; offset slit position so that raster centred on the home position

       slit_pos   = - raster.slitspan/2

    ENDELSE
   
  ENDELSE

  ; get current ops positions

  get_ops_pos, current_solarx, current_solary, current_opsl, current_opsr

  ; get new ops positions

  get_ops_pos, new_solarx, new_solary, new_opsl, new_opsr

  ; work out delays in ms

  opsl_delay  = get_ops_delay   ( current_opsl, new_opsl, 0 )  

  opsr_delay  = get_ops_delay   ( current_opsr, new_opsr, 1 )

  slitn_delay = get_slitn_delay ( current_slitn, new_slitn )

  slitp_delay = get_slitp_delay ( 0, slit_pos)

  ; update new state OPS positions and slit number

  series.fstate.solarx = new_solarx
  series.fstate.solary = new_solary
  series.fstate.slitn  = new_slitn

  ; set mirror and slit slit initial possition 
  ; NB The slit value goes into the raster header packet for both detectors but is
  ;    only used on board for the GIS detector. However, this will reflect a
  ;    non-zero slit position for the VDS if ADJUST_ALLOWED.
  ; For alternative studies these commands are commented out and will now be done by operator.

  IF KEYWORD_SET(altsci) THEN BEGIN

    fileout, '# Mirror and slit initial positions must be commanded by operator.
    fileout, ''

    ; set slit initial position 

    fileout, '# CB5SLIT ' + shorthex ( slit_pos ) + $
           '  # tell macro slit initial position : ' + STRTRIM ( slit_pos, 1 )
    fileout, ''

    ; set mirror initial position

    fileout, '# CB5MIR ' + shorthex ( mirror_pos ) + $
             '   # tell macro mirror initial position : ' + STRTRIM ( mirror_pos, 1 )
    fileout, ''

  ENDIF ELSE BEGIN

    ; set slit initial position 

    fileout, words=2, 'CB5SLIT ' + shorthex ( slit_pos ) + $
           '  # tell macro slit initial position : ' + STRTRIM ( slit_pos, 1 )
    fileout, ''

    ; set mirror initial position

    fileout, words=2, 'CB5MIR ' + shorthex ( mirror_pos ) + $
             '   # tell macro mirror initial position : ' + STRTRIM ( mirror_pos, 1 )
    fileout, ''

  ENDELSE

  ; check if solarx/y need changing
  ; NB This requires that when OPs is set up in the deferred command store it goes through the
  ;    macro process so that macro knows about it.
  ;    This allows flag studies to be treated normally else there would be OPS commands
  ;    in the flag study itself.

  IF new_solarx NE current_solarx OR new_solary NE current_solary THEN BEGIN 

    fileout, words=3, 'CB5SIC' + '          # tell macro to tell mcu to move OPS left' 
    fileout, 'CBMOPSLP ' + shorthex ( new_opsl ) + ' # OPS left = ' + STRTRIM ( new_opsl, 1 )
    fileout, 'TERMINATE'
    fileout, ''

    macro_wait, opsl_delay
    fileout, ''

    fileout, words=3, 'CB5SIC' + '          # tell macro to tell mcu to move OPS right'
    fileout, 'CBMOPSRP ' + shorthex ( new_opsr ) + ' # OPS right = ' + STRTRIM ( new_opsr, 1 ) 
    fileout, 'TERMINATE'
    fileout, ''

    macro_wait, opsr_delay
    fileout, ''

  ENDIF

  ; check if slit number needs changing

  IF new_slitn NE current_slitn THEN BEGIN 

    ; check explicitly for slitn=0

    IF new_slitn NE 0 THEN BEGIN

      fileout, words=3, 'CB5SIC' + '          # tell macro to tell mcu to change slit number'
      fileout, 'CBMSLITN ' + shorthex ( new_slitn ) + ' # slit number = ' + STRTRIM ( new_slitn, 1 ) 
      fileout, 'TERMINATE'
      fileout, ''

      macro_wait, slitn_delay
      fileout, ''

      ; ensure slit motor turned off (not really needed if GIS raster)

      safe_slit, /SERIES

    ENDIF ELSE BEGIN

      ; slitn=0 error

      MESSAGE, 'ERROR : TRIED TO WRITE SLITN=0 COMMAND', /TRACEBACK, /CONT

      fileout, '#  ERROR : SLITN = 0 COMMAND FORBIDDEN'
      fileout, ''

    ENDELSE

  ENDIF

  ; turn on SFT if required

  IF series(0).sft EQ 'Y' THEN BEGIN

    fileout, words=3, 'CB5SIC'            + '          # enable solar feature tracking'
    fileout, 'CB2FTEN'
    fileout, 'TERMINATE'                 
    fileout, ''

  ENDIF 

  ; test whether NIS or GIS detector

  IF raster.detector EQ 'N' THEN BEGIN

    ; NIS detector

    ; set exposure time units if NIS

    fileout, words=3, 'CB5SIC'                + '          # tell macro to set VDS Exposure units'
    fileout, 'CBVEXPTI ' + shorthex ( raster.expunits ) 
    fileout, 'TERMINATE'
    fileout, ''

  ENDIF ELSE do_gset_on, series(0).istate.gsetid, raster.gis

  fileout, words=2, 'CB5RUNR ' + shorthex ( raster.cdhsid ) + '  # run raster, index : ' + STRTRIM ( raster.cdhsx, 1 )
  fileout, ''

  ; turn off SFT if required

  IF series(0).sft EQ 'Y' THEN BEGIN

    fileout, words=3, 'CB5SIC'            + '          # disable solar feature tracking'
    fileout, 'CB2FTDIS'
    fileout, 'TERMINATE'         
    fileout, ''

  ENDIF

  IF raster.detector EQ 'G' THEN BEGIN

    ; turn off HV etc

    do_gset_off, series(0).istate.gsetid, raster.gis 

    ; ensure slit motor turned off

    safe_slit, /SERIES

  ENDIF

  ; return to slit home position if required i.e if slit position has been moved
  ; Note that CDHS has slit home position = 0 hardcoded and will return to home
  ; after a GIS raster.

  IF N_ELEMENTS(new_slitp) GT 0 THEN BEGIN

    fileout, '# Return slit to home position'
    fileout, ''

    fileout, words=3, 'CB5SIC' + '          # tell macro to tell mcu to move slit position' 
    fileout, 'CBMSLITP 0x0000 # SLIT position = 0' 
    fileout, 'TERMINATE'
    fileout, ''

    macro_wait, slitp_delay
    fileout, ''

    ; ensure slit motor turned off

    safe_slit, /SERIES

  ENDIF

END






