; Project     :	SOHO - CDS
;
; Name        :	GET_IEF
;
; Purpose     :	Retrieves an IEF flag parameter and windows data set from the IEFTAB and IEFWIN databases.
;
; Explanation : Extracts the parameters which define the flag generation scheme and flag windows
;               from the IEFTAB and IEFWIN database. 
;
; Use         : < get_ief, ieftab_id, def >
;
; Inputs      : ieftab_id = IEF table ID number.
;
; Outputs     : def = structure containing IEF table database values.
;                  def.ieftab_id  = I*2      IEF table ID. 
;                                            If the requested IEF table is not found,
;				             then a simpler structure is returned, with
;				             this set to -1.
;                  def.desc       = C*65     Description of event generation scheme.
;                  def.upp_era    = 4(I*2)   Upper limit for each ERA measure (unsigned INT16)
;                  def.low_era    = 4(I*2)   Lower limit for each ERA measure (unsigned INT16)
;                  def.discrim    = I*2      Discriminator level (unsigned INT16)
;                  def.ev_type    = I*2      Event type (4 bits)
;                  def.pmode      = I*2      Pointing mode : NIS=0, VF=1, GIS=2 (2 bits)
;                  def.cmode      = I*2      Centroid mode : POS=0, NEG=1, ABS=2 (2 bits)
;                  def.c_banda    = I*2      Slit centre band A (INT16)
;                  def.c_bandb    = I*2      Slit centre band B (INT16)
;                  def.c_row      = I*2      Central row (INT16)
;                  def.detector   = C*1      'N'= NIS, 'G'= GIS.
;                  def.wins       =  structure array containing window info
;                    wins.iefwin_id  = I*2    Window ID.
;                    wins.win_name   = C*40   Window name
;                    wins.ev_desc    = C*65   Description of event detection scheme employed
;                    wins.win_def    = 4(I*2) Window definition ( xstart, ystart, xsize, ysize )
;                    wins.px_offset  = I*2    Pixel offset defining ROI around peak pixel.
;                    wins.exp_era    = 4(I*2) Expected values for each ERA in window.
;                    wins.dev_era    = 4(I*2) Deviations for each ERA in window.
;                    wins.wgt_era    = 4(I*2) Weights for each direct ERA in window.
;                    wins.wgt_dera   = 4(I*2) Weights for each differential ERA in window.
;
; Keywords    :
;       ERRMSG    = If defined and passed, then any error messages will be
;                   returned to the user in this parameter rather than
;                   depending on the MESSAGE routine in IDL.  If no errors are
;                   encountered, then a null string is returned.  In order to
;                   use this feature, ERRMSG must be defined first, e.g.
;
;                       ERRMSG = ''
;                       GET_DATAWIN, ERRMSG=ERRMSG, ... 
;                       IF ERRMSG NE '' THEN ...

; Calls       :	datatype, get_ieftab, get_iefwin, copy_struct.
;                
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Category    :	Command preparation.
;
; Prev. Hist. :	Adapted from get_datawin.
;
; Written     :	Version 0.0, Martin Carter, RAL, 18/10/95
;
; Modified    :	
;
; Version     :	Version 0.0, 18/10/95
;-
;**********************************************************

PRO get_ief, ieftab_id, def, ERRMSG=errmsg

	ON_ERROR, 2

;  Define a null result.  If the routine is successful, this will be updated
;  later.
;
	def = { ieftab_id : -1 }
;
;  Check the number of parameters.
;
        IF N_PARAMS() NE 2 THEN BEGIN
           MESSAGE = 'Syntax:  GET_IEF, IEFTAB_ID, DEF'
	   GOTO, HANDLE_ERROR
        ENDIF

;
;  Check the input parameter iefwin_id
;
	type = datatype ( ieftab_id, 2 )
	IF type EQ 0 THEN BEGIN
           MESSAGE = 'IEFTAB_ID is undefined'
           GOTO, HANDLE_ERROR
	END ELSE IF type GE 4 THEN BEGIN
           MESSAGE = 'IEFTAB_ID must be an integer'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS ( ieftab_id ) NE 1 THEN BEGIN
           MESSAGE = 'IEFTAB_ID must be a scalar'
           GOTO, HANDLE_ERROR
	ENDIF

; get IEF table

  get_ieftab, ieftab_id, ieftab 

  IF ieftab.ieftab_id LT 0 THEN BEGIN
     MESSAGE = 'Get IEF table failed'
     GOTO, HANDLE_ERROR
  ENDIF

; get first IEF window

  get_iefwin, ieftab.iefwin_ids(0), iefwin

  IF iefwin.iefwin_id LT 0 THEN BEGIN
     MESSAGE = 'Get IEF window failed'
     GOTO, HANDLE_ERROR
  ENDIF 

;  Set up return structure

   win = { IEF_WINDOWS,                      $
           iefwin_id : ieftab.iefwin_ids(0), $
           win_name  : iefwin.win_name,      $
           ev_desc   : iefwin.ev_desc,       $
           win_def   : iefwin.win_def,       $
           px_offset : iefwin.px_offset,     $
           exp_era   : iefwin.exp_era,       $
           dev_era   : iefwin.dev_era,       $
           wgt_era   : iefwin.wgt_era,       $
           wgt_dera  : iefwin.wgt_dera       }

   def = { ieftab_id  : ieftab_id,                 $
           desc       : ieftab.desc,               $
           upp_era    : ieftab.upp_era,            $
           low_era    : ieftab.low_era,            $
           discrim    : ieftab.discrim,            $
           ev_type    : ieftab.ev_type,            $
           pmode      : ieftab.pmode,              $
           cmode      : ieftab.cmode,              $
           c_banda    : ieftab.c_banda,            $
           c_bandb    : ieftab.c_bandb,            $
           c_row      : ieftab.c_row,              $
           detector   : ieftab.detector,           $
           wins       : REPLICATE ( win, ieftab.n_iefwins ) }

; get other windows

  FOR w = 1, ieftab.n_iefwins - 1 DO BEGIN

    get_iefwin, ieftab.iefwin_ids(w), iefwin

    IF iefwin.iefwin_id LT 0 THEN BEGIN
       MESSAGE = 'Get IEF window failed'
       GOTO, HANDLE_ERROR
    ENDIF 

    ; copy to output structure
    ; NB be careful about passing by value

    copy_struct, iefwin, win

    def.wins(w) = win    

  ENDFOR

  RETURN
;
;  Error handling point.
;
HANDLE_ERROR:
	IF N_ELEMENTS(ERRMSG) NE 0 THEN ERRMSG = 'GET_IEF: ' + MESSAGE $
		ELSE MESSAGE, MESSAGE, /CONTINUE

END


