;+
; Project     :	SOHO - CDS
;
; Name        :	GET_RASTER_ID()
;
; Purpose     :	Returns an internal CDHS ID for a table.
;
; Explanation : Interrogates the raster_id database to find whether 
;               table has already been used. If already used then returns the CDHS ID assigned to
;               it else gives the table a new ID and enters it into the database.
;
; Use         : <get_raster_id, initial_table>
;
; Inputs      : initial_table = structure containing table information.
;               Input tags used :
;                    table.vs     = character string indicating the software version
;                                   of the code used to create the table.
;                    table.id     = integer planning database ID for the table.
;                    table.var    = integer planning database variation for the table.
;                    table.repn   = an integer indicating how many times the RASTER is to be 
;                                   repeated. 
;
; Opt. Inputs : None.
;
; Outputs     : Structure array containing table information with CDHS ID and table OK
;               entries set.
;                    table.cdhsid = the internal CDHS ID used to reference the table
;                    table.ok     = an integer flag indicating whether table has already 
;                                   been loaded onto the CDS.
;
; Opt. Outputs:	None.
;
; Keywords    : None.
;
; Calls       :	dbopen, db_info, dbbuild, dbfind, dbext, dbclose,
;               get_utc, rem_dup.
;                
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Category    :	Command preparation.
;
; Prev. Hist. :	None.
;
; Written     :	Version 0.0, Martin Carter, RAL, 14/10/94
;
; Modified    :	Version 0.1, Martin Carter, RAL, 28/11/94
;                            Added proforma.
;                       0.2, Martin Carter, RAL, 15/02/95
;                            Started CDHS IDs from default offsets and 
;                            added max. ID check
;                       0.3, MKC, 2/3/95
;                            Added repn and sft to table info,
;                            Removed the dexwin and vdswin tables from the tableID database.
;                       0.4, MKC, 18/4/95
;                            Adapted from get_cdhs_id.
;                            Removed duplicate keyword.
;                       0.5, MKC, 25/9/95
;                            Pointed SW at duplicate database.
;                       0.6, MKC, 24/10/95
;                            Added /silent keyword to dbbuild.
;                       0.7, MKC, 27/4/98
;                            Current implementation of alternative studies allows 2 versions of
;                            a table to be active on CDHS. Changed checks to reflect this.
;
; Version     :	Version 0.7, 27/4/98
;-
;**********************************************************

FUNCTION get_raster_id, initial_table

  ; check if table defined and if not return new CDHS ID

  ; get local copy of table

  table = initial_table

  ; define first cdhs ID 
  ; NB First 2 raster tables are reserved or modified on power up
  ;     Tables start at 0.
  ; The raster IDs are 0xFFFE and 0xFFFF.

  ; Start the CDHS Ids so that they keep track with the table index.

  first_id = 2  

  ; set priviledge to update database

  !PRIV = 2

  ; open raster ID database for updating

  dbopen, 'draster_id', 1

  ; check if table exists already

  ; check database has some entries
  ; else dbfind will flag error

  nentries = db_info ( 'entries' )

  IF nentries(0) EQ 0 THEN BEGIN

    ; database is empty

    ; table does not exist

    table.ok = 0

    ; set first entry

    table.cdhsid = first_id

    ; Get current utc time in string format

    get_utc, timenow, /ECS

    ; create new entry in database

    dbbuild, timenow, table.vs, table.cdhsid, table.id, table.var, table.repn, /SILENT

  ENDIF ELSE BEGIN

    ; set up search criteria

    search = 'ID=' + STRING ( table.id ) + ',VAR=' + STRING ( table.var ) 

    list = dbfind ( search, /SILENT)

    ; list is array of entries in database
    ; if no entries then list contains a zero
    ; valid entries start at 1 

    IF list(0) GE 1 THEN BEGIN

      ; refine search criteria

      search = 'SW_VERSION=' + table.vs + $
               ',REPN=' + STRING ( table.repn ) 

      list = dbfind ( search, list, /SILENT)

    ENDIF

    ; does table not exist

    IF ( list(0) LT 1 ) THEN BEGIN

      ; table does not exist, create new table ID entry

      table.ok = 0

      ; get next CDHS ID

      ; extract last  cdhs ID s

      dbext, [ LONG ( nentries ) ], 'CDHS_ID', cdhsids   

      table.cdhsid = cdhsids(0) + 1   

      IF table.cdhsid EQ 32768 THEN MESSAGE, 'MAXIMUM CDHSID REACHED'

      ; Get current utc time in string format

      get_utc, timenow, /ECS

      ; create new entry in database

      dbbuild, timenow, table.vs, table.cdhsid, table.id, table.var, table.repn, /SILENT

    ENDIF ELSE BEGIN

      ; table does exist, extract table entry

      table.ok = 1

      ; remove any duplicates

      list = list ( rem_dup ( list ) )
    
      ; list may have 2 entries

      IF N_ELEMENTS ( list ) GT 2 THEN $
        MESSAGE, 'Invalid number of entries : ' + STRTRIM ( N_ELEMENTS(list), 1 )

      ; get table entry

      dbext, list(0), 'CDHS_ID', cdhsids
 
      ; set ID

      table.cdhsid = cdhsids(0)

    ENDELSE

  ENDELSE

  ; close table ID database

  dbclose, 'draster_id'

  ; reset priviledges

  !PRIV = 1  

  ; return results

  RETURN, table

END


