;+
; Project     :	SOHO - CDS
;
; Name        :	GETNEXTPERIOD()
;
; Purpose     :	Returns start/end times for next CDS science plan period.
;
; Explanation :	Defines the next CDS science plan period. Unless otherwise specified this 
;               begins from the start of the next UTC day and lasts for 24 hours.
;               NB Any study that overlaps this interval will be selected from the science
;                  plan.
;
; Use         :	<period = GETNEXTPERIOD()>
;
; Inputs      :	None.
;
; Opt. Inputs : None.
;
; Outputs     : Anonymous structure variable containing the following tags :
;
;			STARTOFPERIOD = String containing UTC start time of next CDS period
;			ENDOFPERIOD   = String containing UTC end   time of next CDS period
;
; Opt. Outputs:	None.
;
; Keywords    : STARTOFPERIOD = start time of CDS science plan period in any CDS format;
;               ENDOFPERIOD   = stop  time of CDS science plan period in any CDS format.
;               NOW           = 1, indicates that CDS science plan period runs from
;                                  current time to end of the day (NB ENDOFPERIOD keyword
;                                  overrides end time).
;
; Calls       :	get_utc, anytim2utc, utc2str.
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Category    :	Command preparation.
;
; Prev. Hist. :	None.
;
; Written     :	Martin Carter, RAL, 14/10/94
;
; Modified    :	Version 1, Martin Carter, RAL, 14/10/94
;		Version 2, William Thompson, GSFC, 14 November 1994
;			Modified .DAY to .MJD
;               Version 3, Martin Carter, RAL, 28/11/94
;                       Updated comments. Avoided modifying keyword inputs.
;               Version 4, Martin Carter, RAL, 6/2/95
;                       Corrected bug when endofperiod not specified.
;               Version 5, MKC, RAL, 6/3/95
;                       Added NOW keyword.
;
; Version     : Version 5, 6/3/95
;-
;**********************************************************

FUNCTION getnextperiod, STARTOFPERIOD=startofperiod, ENDOFPERIOD=endofperiod, NOW=now

  ; check if now keyword set

  IF KEYWORD_SET(now) THEN BEGIN

    ; Get current utc time in internal format

    get_utc, startofp

  ENDIF ELSE BEGIN

    ; check if start of period defined

    IF NOT KEYWORD_SET(startofperiod) THEN BEGIN

      ; Get current utc time in internal format

      get_utc, startofp

      ; Assume next period begins at end of UTC day 

      startofp.mjd  = startofp.mjd + 1
      startofp.time = 0

    ENDIF ELSE BEGIN

      ; start time explicitly defined
      ; convert to internal format

      startofp = anytim2utc ( startofperiod )

    ENDELSE

  ENDELSE

  ; check if end of period defined

  IF NOT KEYWORD_SET(endofperiod) THEN BEGIN

    ; Assume end of next period is 24 hours later and starts at 0 hours

    endofp = startofp
    endofp.mjd = endofp.mjd + 1
    endofp.time = 0

  ENDIF ELSE BEGIN

    ; end time explicitly defined
    ; convert to internal format

    endofp = anytim2utc ( endofperiod )

  ENDELSE

  ; Convert to string format and put into structure

  period = { STARTOFPERIOD : utc2str ( startofp ), ENDOFPERIOD: utc2str ( endofp )}

  RETURN, period

END
