; Project     :	SOHO - CDS
;
; Name        :	LIST_IEFTAB
;
; Purpose     :	Lists the available IEF table database entries.
;
; Explanation : Extracts the descriptions of all the flag generation schemes
;               from the IEFTAB database. 
;
; Use         : < list_ieftab, list, n_found >
;
; Inputs      : None.
;
; Outputs     : list = an array of structures containing the IEF table database values.
;                  Tags are :
;                  .ieftab_id  = I*2      IEF table ID. 
;                  .desc       = C*65     Description of event generation scheme.
;                  .upp_era    = 4(I*2)   Upper limit for each ERA measure (unsigned INT16)
;                  .low_era    = 4(I*2)   Lower limit for each ERA measure (unsigned INT16)
;                  .discrim    = I*2      Discriminator level (unsigned INT16)
;                  .ev_type    = I*2      Event type (4 bits)
;                  .pmode      = I*2      Pointing mode : NIS=0, VF=1, GIS=2 (2 bits)
;                  .cmode      = I*2      Centroid mode : POS=0, NEG=1, ABS=2 (2 bits)
;                  .c_banda    = I*2      Slit centre band A (INT16)
;                  .c_bandb    = I*2      Slit centre band B (INT16)
;                  .c_row      = I*2      Central row (INT16)
;                  .detector   = C*1      'N'= NIS, 'G'= GIS.
;                  .n_iefwins  = I*2      No. of IEF windows (<=3)
;                  .iefwin_ids = 3(I*2)   IEFWIN database IDs
;               n_found : no. of available database entries found
;
; Keywords    :
;       ERRMSG    = If defined and passed, then any error messages will be
;                   returned to the user in this parameter rather than
;                   depending on the MESSAGE routine in IDL.  If no errors are
;                   encountered, then a null string is returned.  In order to
;                   use this feature, ERRMSG must be defined first, e.g.
;
;                       ERRMSG = ''
;                       LIST_IEFTAB, ERRMSG=ERRMSG, ... 
;                       IF ERRMSG NE '' THEN ...
;
; Calls       :	dbopen, db_info, dbext, dbclose.
;                
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Category    :	Command preparation.
;
; Prev. Hist. :	Adapted from listt_datawin.
;
; Written     :	Version 0.0, Martin Carter, RAL, 27/11/95
;
; Modified    :	
;
; Version     :	Version 0.0, 27/11/95
;-
;**********************************************************

PRO list_ieftab, list, n_found, ERRMSG=errmsg

  ON_ERROR, 2

;  Define a null result.  If the routine is successful, this will be updated
;  later.
;

  n_found = 0

;
;  Check the number of parameters.
;
  IF N_PARAMS() NE 2 THEN BEGIN
    MESSAGE = 'Syntax:  LIST_IEFTAB, LIST, N_FOUND'
    GOTO, HANDLE_ERROR
  ENDIF

; open database 

        dbopen, 'ieftab'

; check database has some entries
; else dbfind will flag error

  nentries = db_info ( 'entries' )

;
;  If no entries were found, then return immediately.
;
  IF nentries(0) LT 1 THEN BEGIN
    MESSAGE = 'No IEF tables found'
    GOTO, HANDLE_ERROR
  ENDIF

; read entries from database (NB at most 12 at a time)

  dbext, LINDGEN(nentries(0)), 'IEFTAB_ID,DESC,UPP_ERA,LOW_ERA,DISCRIM,EV_TYPE,PMODE,CMODE,C_BANDA,C_BANDB,C_ROW', $
                                ieftab_id,desc,upp_era,low_era,discrim,ev_type,pmode,cmode,c_banda,c_bandb,c_row

  dbext, LINDGEN(nentries(0)), 'DETECTOR,N_IEFWINS,IEFWIN_IDS', $
                                detector,n_iefwins,iefwin_ids


  ; set up results

  n_found = nentries(0)

  ; form first element of structure array 

  list = { ieftab_id : ieftab_id(0), desc : desc(0), upp_era : upp_era(*,0), low_era : low_era(*,0),       $
           discrim : discrim(0), ev_type : ev_type(0), pmode : pmode(0), cmode : cmode(0),                 $
           c_banda : c_banda(0), c_bandb : c_bandb(0), c_row : c_row(0), detector : detector(0),           $
           n_iefwins : n_iefwins(0), iefwin_ids : iefwin_ids(*,0) }                                    

  ; form structure array

  list = REPLICATE ( list, n_found )

  ; set up structure array elements

  list.ieftab_id  = ieftab_id
  list.desc       = desc
  list.upp_era    = upp_era
  list.low_era    = low_era
  list.discrim    = discrim
  list.ev_type    = ev_type
  list.pmode      = pmode
  list.cmode      = cmode
  list.c_banda    = c_banda
  list.c_bandb    = c_bandb
  list.c_row      = c_row
  list.detector   = detector
  list.n_iefwins  = n_iefwins
  list.iefwin_ids = iefwin_ids

  GOTO, FINISH
;
;  Error handling point.
;
HANDLE_ERROR:
	IF N_ELEMENTS(ERRMSG) NE 0 THEN ERRMSG = 'LIST_IEFTAB: ' + MESSAGE $
		ELSE MESSAGE, MESSAGE, /CONTINUE
;
;  Close the database, and return.
;
FINISH:
	dbclose, 'ieftab
;
	RETURN

END


