;+
; Project     :	SOHO - CDS
;
; Name        :	MK_ENG()
;
; Purpose     :	Adds an engineering study entry to the study_var database.
;
; Explanation : This routine sets up the entries in the studyvar database for a new 
;               engineering study. It also creates a file to contain the engineering
;               study. The file name is returned.
;               Engineering studies are handled in the same way as science studies
;               in the planning SW and have a study database entriy associated 
;               with them along with a studyvar database entry which uses some of the existing
;               fields to describe the engineering study. 
;               The engineering studies are either coded as series tables which will be  
;               loaded and ran by CPT or as perl scripts which will be run by the operator
;               in real time.  
;               A series table is still associated with a PERL script engineering table.
;               This will contain something to identify it to the operator as the start
;               of a realtime period.       
;
; Use         : < filename = mk_eng ( type, duration, description ) >
;
; Inputs      : type        : Defines type of study : either 'SCRIPT' or 'SERIES' 
;               description : description of the engineering study.
;               duration    : Float giving duration of the engineering study in seconds.
;
; Opt. Inputs : None.
;
; Outputs     : Adds an entry to the studyvar database. 
;               The fields of studyvar are used in the following way :
;                 STUDY_ID   : This always has the value 0 indicating an engineering study                
;                 STUDYVAR   : This gives the ID no. of the engineering study.
;                 SV_DESC    : This describes the purpose of the study.
;                 N_RASTERS0 : This always has the value 0.
;                 DURATION0  : This contains the duration of the study.
;                 DURATION1  : This always has the value 1.0
;                 ZONEID     : This always has the value 1.
;                 USABLE     : This is used to indicate whether the study is currently usable.
;                              This should always be 'Y'.
;               Returns the filename for the engineering study.
;
; Opt. Outputs:	None.
;
; Keywords    : ERRMSG : If defined and passed, then any error messages will be
;                        returned to the user in this parameter rather than
;                        depending on the MESSAGE routine in IDL.  If no errors are
;                        encountered, then a null string is returned.  In order to
;                        use this feature, ERRMSG must be defined first, e.g.
;
;                         ERRMSG = ''
;                         MK_ENG, ERRMSG=ERRMSG, ... 
;                         IF ERRMSG NE '' THEN ...
;
; Calls       :	concat_dir, add_v_study, shorthex
;                
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Category    :	Command preparation.
;
; Prev. Hist. :	None.
;
; Written     :	Version 0.0, Martin Carter, RAL, 29/11/95
;
; Modified    :	Version 0.1, MKC, 23/1/96.
;                 Removed suggestion that undefined parameters be prompted for.
;                 This can be done by a calling routine if necessary. 
;                 Removed popup keywords.
;               Version 0.2, MKC, 1/2/96
;                 Added filename and variation to file contents.
;               Version 0.3, MKC, 24/2/97
;                 Removed CB5ENDS commands from end of file.
;
; Version     :	Version 0.3, 24/2/97
;-
;**********************************************************

FUNCTION mk_eng, type, duration, description, ERRMSG=ERRMSG
;
   ON_ERROR, 2
;
;  Make sure that the filename is initialized.
;
   filename = ''
;
;  Check the input parameters
;
   IF N_ELEMENTS(type) GT 0 THEN BEGIN

      IF DATATYPE ( type, 1 ) NE 'String' THEN BEGIN
          message = 'Parameter TYPE must be a character string'
          GOTO, HANDLE_ERROR
      END ELSE IF N_ELEMENTS (  type ) NE 1 THEN BEGIN
           message = 'Parameter TYPE must be a scalar'
           GOTO, HANDLE_ERROR
      END ELSE IF STRUPCASE ( type ) NE 'SERIES' AND $
                  STRUPCASE ( type ) NE 'SCRIPT' THEN BEGIN
           message = "Parameter TYPE  must be either 'SCRIPT or 'SERIES'"
           GOTO, HANDLE_ERROR
      ENDIF

   ENDIF ELSE type = ''

   IF N_ELEMENTS(duration) GT 0 THEN BEGIN

      IF DATATYPE ( duration, 1 ) NE 'Float' THEN BEGIN
           message = 'Parameter DURATION must be a floating point number'
           GOTO, HANDLE_ERROR
      END ELSE IF N_ELEMENTS ( duration ) NE 1 THEN BEGIN
           message = 'Parameter DURATION must be a scalar'
           GOTO, HANDLE_ERROR
      END ELSE IF duration LT 0.0 THEN BEGIN
           message = 'Parameter DURATION must be positive'
           GOTO, HANDLE_ERROR
      ENDIF

   ENDIF ELSE duration = 0.0

   IF N_ELEMENTS(description) GT 0 THEN BEGIN

      IF DATATYPE ( description, 1 ) NE 'String' THEN BEGIN
           message = 'Parameter DESCRIPTION must be a character string'
           GOTO, HANDLE_ERROR
      END ELSE IF N_ELEMENTS ( description ) NE 1 THEN BEGIN
           message = 'Parameter DESCRIPTION must be a scalar'
           GOTO, HANDLE_ERROR
      END ELSE IF STRLEN ( description ) GT 65 THEN BEGIN
           message = 'Parameter DESCRIPTION must be less than 65 characters'
           GOTO, HANDLE_ERROR
      ENDIF

   ENDIF ELSE description = ''

;  check parameters set up

   IF type EQ '' OR duration EQ 0 OR description EQ '' THEN BEGIN

      message = 'Input parameter not set up'
      GOTO, HANDLE_ERROR

   ENDIF

;  determine value of title ID

   IF STRUPCASE(type) EQ 'SCRIPT' THEN title_id = 1 ELSE title_id = 2 

;  set up database entry structure
;  NB rasters tag does not need to be defined because study_id zero has n_rasters_def=0

   struct = { study_id:0, title_id:title_id, studyvar:0, sv_desc:description, n_rasters0:0, $
              duration0:duration, duration1:1.0, zone_id:1, usable:'Y'}

;  add entry to database

   !priv = 2

   message = ''

   result = add_v_study (struct, ERRMSG=message)

   !priv = 1

   IF message NE '' THEN BEGIN
     GOTO, HANDLE_ERROR
   ENDIF

   IF result NE 1 THEN BEGIN
     message = 'Could not add study variation'
     GOTO, HANDLE_ERROR
   ENDIF

;  set up file name based on studyvar ID

   filename = concat_dir ( GETENV ( 'CDS_CP_ENG_W' ), 'engineering_' + STRTRIM ( struct.studyvar, 1) + '.dt' )

;  create file for study

   OPENW, unit, filename, /GET_LUN

;  write file header

   get_utc, utc, /ECS
   
   PRINTF, unit, '# ENGINEERING SERIES TABLE : ' + 'engineering_' + STRTRIM ( struct.studyvar, 1) + '.dt'
   PRINTF, unit, '# Created ', utc
   PRINTF, unit, '# Study id : 0
   PRINTF, unit, '# Study variation : ' +  STRTRIM ( struct.studyvar, 1)
   PRINTF, unit
   PRINTF, unit, '# ' + type + ' format'
   PRINTF, unit, '# ' + description
   PRINTF, unit, '# Duration = ' + STRTRIM ( duration, 1)
   PRINTF, unit

   IF STRUPCASE(type) EQ 'SERIES' THEN BEGIN

     PRINTF, unit, '# Enter body here 
     PRINTF, unit, '# NB omit CDHS ID, start with study ID/length
     PRINTF, unit, '#    omit CB5ENDS 
     PRINTF, unit, '#    do not change pointing
     PRINTF, unit, '#    do not change slit number
     PRINTF, unit, '#    do not change GSETID parameters
     PRINTF, unit

   ENDIF ELSE BEGIN

     PRINTF, unit, '0x0002    # study ID / length
     PRINTF, unit

   ENDELSE

;  close file and free logical unit

   FREE_LUN, unit

;  return filename

   RETURN, filename

;  Error handling point.
;
HANDLE_ERROR:
   IF N_ELEMENTS ( errmsg ) NE 0 THEN errmsg = 'MK_ENG : ' + message $
     ELSE MESSAGE, message

   RETURN, filename

END