;+
; Project     :	SOHO - CDS
;
; Name        :	WR_RASTER_FILE
;
; Purpose     :	Writes out a CDS RASTER table to file.
;
; Explanation :	Writes out the raster parameters in the format required by 
;               the Perl script files for loading onto the CDS.
;
; Use         : <wr_raster_file, raster, raster_details>
;                 or if version number required
;               <wr_raster_file, raster, VERSION=version>.
;
; Inputs      : raster          = structure array giving information on raster
;                                 tables;
;               raster_details  = structure containing raster information 
;                                 from raster planning database.
;
; Opt. Inputs : None.
;
; Outputs     : Writes raster data to file. 
;
; Opt. Outputs:	version = software version number if keyword set.
;
; Keywords    : VERSION = Character string containing software version number.
;
; Calls       :	get_utc, shorthex, concat_dir, cp_get_entry.
;                
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Category    :	Command preparation.
;
; Prev. Hist. :	None.
;
; Written     :	Version 0.00, Martin Carter, RAL, 14/10/94
;
; Modified    :	Version 0.01,Martin Carter, RAL, 28/11/94
;                            Added proforma.
;               Version 0.1, MKC, RAL, 2/3/95
;                            Changed .ntimes to .repn
;               Version 0.2, MKC, 6/3/95
;                            Added raster.exptm tag.
;               Version 0.3, MKC, 23/3/95
;                            Removed ll_title from display.
;               Version 0.4, MKC, 21/4/95
;                            Modified argument list.
;               Version 0.5, MKC, 17/5/95
;                            Stored mirror and slit offsets from home positions in
;                            raster structure. Removed this to cpt level.
;               Version 0.6, MKC, 23/5/95
;                            Switched GIS banks 1 and 2 around.
;               Version 0.7, MKC, 4/9/95
;                            Retrieved a number of previosuly hard-coded parameters
;                            from the CDHS state database ie. VDS mode.
;               Version 0.8, MKC, 23/10/95
;                            Renamed environment variables and used concat_dir.
;               Version 0.9, MKC, 7/11/95
;                            Used cp_get_entry.
;                            Modified prinout.
;                            Changed so that will write out table if already loaded.
;               Version 1.0, MKC, 24/11/95
;                            Changed so that CVT files have .dt extension.
;               Version 1.1, MKC, 29/3/96
;                            Hardcoded the VDS map and HTR status which should never change since the
;                            other modes are not supported in the on-board software.
;                            Modified so that now picks up the mode by reference to the VDS readout mode
;                            parameter in the CBVCAM command.
;
; Version     :	Version 1.1, 29/3/96
;-
;**********************************************************

PRO wr_raster_file, raster, raster_details, VERSION=version

  ; set version number 

  sw_version = '1.1'

  ; check if only version required

  IF KEYWORD_SET(version) THEN BEGIN

    raster.vs = sw_version
    RETURN

  ENDIF

  ; check if raster loaded already

  IF !debug THEN IF raster.ok THEN PRINT, 'Raster table already loaded'

  ; need to load raster

  ; Output raster CVT file 
  ; overwrite any existing file 

  ; get file name

  filename = concat_dir ( '$CDS_CP_CVTFILES_W' , 'raster_' + STRTRIM ( raster.cdhsx, 1) + '.dt' )

  IF !debug THEN PRINT, 'WRITING FILE : ', filename

  ; open file

  OPENW, unit, filename, /GET_LUN

  ; output raster table

  ; file header info

  get_utc, utc, /ECS

  PRINTF, unit, '# RASTER TABLE FILE
  PRINTF, unit, '# Created ', utc
  PRINTF, unit, '# Version ', sw_version
  PRINTF, unit, '# CDHS id ', raster.cdhsid
  PRINTF, unit, '# CDHS index ', raster.cdhsx
  PRINTF, unit, '# RASTER ID  = ', raster.id
  PRINTF, unit, '# RASTER VAR = ', raster.var
  PRINTF, unit

 ; output raster parameters

  PRINTF, unit, '# detector  = ', raster_details.detector
  PRINTF, unit, '# rasdesc   = ', STRTRIM(raster_details.ras_desc)
  PRINTF, unit, '# rv_desc   = ', STRTRIM(raster_details.rv_desc)
  PRINTF, unit, '# ll_desc   = ', STRTRIM(raster_details.ll_desc)
  PRINTF, unit, '# dw_desc   = ', STRTRIM(raster_details.dw_desc)
  PRINTF, unit

  PRINTF, unit, shorthex ( raster.cdhsid ), $
                '    # CDHS Raster ID'

  IF raster_details.detector EQ 'G' THEN $
    PRINTF, unit, shorthex(9 ), '    # GIS table length' ELSE $
    PRINTF, unit, shorthex(12), '    # VDS table length :', 12

  PRINTF, unit

  comp_value = ISHFT(raster_details.comp_id,8) OR ISHFT(raster_details.slit_num,4) 

  IF raster_details.detector EQ 'G' THEN comp_value = comp_value OR 1

  PRINTF, unit, '# Compression scheme :', raster_details.comp_id
  PRINTF, unit, '# Slit number        :', raster_details.slit_num
  PRINTF, unit
  PRINTF, unit, shorthex(comp_value), $
                '    # Compression scheme(byte), slitn(nibble), detector(nibble)
  PRINTF, unit

  PRINTF, unit, '# Respond to IEF events    : ', (raster.repn AND '1000'x) NE 0
  PRINTF, unit, '# Generate   IEF events    : ', (raster.repn AND '2000'x) NE 0
  PRINTF, unit, '# Allow OPS repointing     : ', (raster.repn AND '4000'x) NE 0
  PRINTF, unit, '# Number of runs of raster : ', raster.repn AND '0FFF'x
  PRINTF, unit
  PRINTF, unit, shorthex(raster.repn), $
                '    # IEF control (nibble) / Number of runs of raster (12 bits) '
  PRINTF, unit

  PRINTF, unit, '# Number of mirror locations :', raster_details.nx
  PRINTF, unit, '# Number of slit   locations :', raster_details.ny
  PRINTF, unit
  PRINTF, unit, shorthex(ISHFT(raster_details.nx,8) OR raster_details.ny), $
                '    # Number of mirror/slit locations
  PRINTF, unit

  PRINTF, unit, '# Size of mirror steps :', raster_details.xstep, ' arcsecs'
  PRINTF, unit, '# Size of slit   steps :', raster_details.ystep, ' arcsecs'
  PRINTF, unit
  PRINTF, unit, shorthex(ISHFT(raster_details.xstep/2,8) OR raster_details.ystep), $
                '    # Size of mirror/slit steps'
  PRINTF, unit

  PRINTF, unit, shorthex ( raster.exptm ), $
                '    # Exposure time :', raster_details.exptime,' secs'
  PRINTF, unit

  PRINTF, unit, shorthex ( raster.dexwid ), $
                '    # CDHS Data extract window ID'
  PRINTF, unit

  PRINTF, unit, shorthex(raster_details.comp_opt), $
                '    # Data compression option :', raster_details.comp_opt
  PRINTF, unit

  PRINTF, unit
                 
  IF raster_details.detector EQ 'G' THEN BEGIN

    ; GIS detector

    ; retrieve GIS parameters from CDHS state database

    ss = cp_get_entry ( 'GISBANKS', [0,1] )

    ; output values

    PRINTF, unit, '# GIS specific commands
    PRINTF, unit

    PRINTF, unit, shorthex ( ss(0).active ), $
                '    # GIS bank switch 1'
    PRINTF, unit

    PRINTF, unit, shorthex ( ss(1).active ), $
                '    # GIS bank switch 2'
    PRINTF, unit

  ENDIF ELSE BEGIN

    ; VDS detector

    ; hard code the CDHS map and HTR values and binning mode
    ; NB HTR not supported on board
    ;    status of binning is unclear on board

    ; set up CDHS modes corresponding to VDS mode - 1

    cdhs_modes = ISHFT ( [ 0, 9, 10, 5, 6, 7, 8], 4)
   
    ; retrieve VDS readout mode from CDHS state database

    ss = cp_get_entry ( 'CBVCAM', [4] )

    vdsmode = ss(0).active

    ; check vds mode

    IF vdsmode LT 1 OR vdsmode GT 7 THEN MESSAGE, 'INVALID VDS READOUT MODE : ' + STRTRIM(vdsmode,2)

    ; output values

    PRINTF, unit, '# VDS specific commands
    PRINTF, unit

    PRINTF, unit, shorthex(raster.vdswid), $
                '    # CDHS VDS extraction window ID'
    PRINTF, unit

    PRINTF, unit, '# VDS orientation :', raster_details.vds_orient
    PRINTF, unit
    PRINTF, unit, shorthex( ISHFT(raster_details.vds_orient,12) OR '0200'X OR cdhs_modes(vdsmode-1) ), $
                '    # Orientation, CDHS map=2, CDHS mode, spare'
    PRINTF, unit

    PRINTF, unit, '0x0000', $
                '    # Spare'
    PRINTF, unit

    PRINTF, unit, '0x0000', $
                '    # Spare'
    PRINTF, unit

    PRINTF, unit, '0x0000', $
                  '    # VDS on-chip binning : off/on, xfac, yfac ( BYTE, NIBBLE, NIBBLE)
    PRINTF, unit
     
  ENDELSE

  ; close file and free logical unit 

  FREE_LUN, unit

END
