;+
; Project     : SOHO - CDS     
;                   
; Name        : COMP_GIS_DOUBLE
;               
; Purpose     : Evaluate double Gaussian for use in CURVEFIT/CFIT/MCURVEFIT
;               
; Explanation : Evaluates a double Gaussian for fitting to GIS data.
;		Especially useful for bright lines in detector 1.
;
;               
;                       F(x) = 	A0 * EXP(-((x-A1)/A2)^2/2) +
;				A3 * EXP(-((x-A4)/A5)^2/2) + 
;				A6
;
;		Suggested parameter values to improve the fit when 
;		using CURVEFIT etc.
;			A0 = A3: initially the max. height of the combined 
;				line
;			A2 = A5: initially 2/3 of the total width of the 
;				combined line
;			A4-A1: initial separation of 0.17 Angstroms for 
;				detector 1 and 0.13 Angstroms for all other 
;				detectors
;			A6: initial background of zero will usually suffice
;
; Use         : COMP_GIS_DOUBLE,X,A,F [,PDER]
;    
; Inputs      : As usual for any CURVEFIT function
;               
; Opt. Inputs : PDER : Partial derivatives are calculated if parameter is
;                      present 
;               
; Outputs     : F : The evaluated broadened Gaussian at the given points
;               
; Opt. Outputs: PDER
;               
; Keywords    : None.
;
; Calls       : None.
;
; Common      : None.
;               
; Restrictions: None.
;               
; Side effects: None.
;               
; Category    : Analysis
;               
; Prev. Hist. : Modified from COMP_GAUSS by S.V.H. Haugan.
;
; Written     : Eddie Breeveld, MSSL/UCL, 12 Jan 2000
;               
; Modified    : 
;
; Version     : Version 1, 12-Jan-2000
;-

pro comp_gis_double, x, a, f, pder
;
; avoid divide by zero
;
        if a[2] ne 0.0 then begin
            z1 = (x-a[1])/a[2]          ; get z
            ez1 = exp(-(z1)^2/2.)   	; gaussian part
        endif else begin
            z1 = 100.
            ez1 = 0.0
        endelse
;
        if a[5] ne 0.0 then begin
            z2 = (x-a[4])/a[5]          ; get z
            ez2 = exp(-(z2)^2/2.)   	; gaussian part
        endif else begin
            z2 = 100.
            ez2 = 0.0
        endelse
;
; evaluate the function
;
      	f = a[0]*ez1 + a[3]*ez2 + a[6]
;
        if n_params(0) le 3 then return		; need partial?
;
; calculate the partials
;
        pder = fltarr(n_elements(x),7) 		; yes, make array.
;
; First gaussian
;
        pder[*,0] = ez1       
        if a[2] ne 0. then pder[*,1] = a[0] * ez1 * z1/a[2]
        pder[*,2] = pder[*,1] * z1
;
; second gaussian
;
        pder[*,3] = ez2     
        if a[5] ne 0. then pder[*,4] = a[3] * ez2 * z2/a[5]
        pder[*,5] = pder[*,4] * z2
;
; constant background
;
	pder[*,6] = 1.
;
        return
end








