;+
; Project     : SOHO - CDS     
;                   
; Name        : GIS_RAW_HEADER.PRO
;               
; Purpose     : Decipher and print out the header of a GIS raw/pha file
;               
; Explanation : 
;               
; Use         :  
;		IDL>	head = bytarr (512)
;		IDL>	openr, lun, /get_lun, 'raw_or_phafilename'
;		IDL>	readu, lun, head	
;		IDL>	result = gis_raw_header (head)
;
; Inputs      : head	a 512 byte array with the file header
;
; Opt. Inputs : None
;               
; Outputs     : 1) Prints out the header in "English"
;		2) returns "result" a structure with the raw file header 
;		information
;  
;	result = $
;		{	comment:comment,	$	; string comment
;			detector:detector,	$	; I*2 detector
;			bughv:bughv,		$	; I*2 HV reading
;			bugmcpb:bugmcpb,	$	; I*2 mcp bias
;			bigmcp:bigmcp,		$	; i*2 mcp current
;			bkgrcd:bkgrcd,		$	; I*2 raw count rate
;			bkgulcd:bkgulcd,	$	; I*2 upper level rate
;			bkgldld:bkgldld,	$	; I*2 lower level setting
;			bkghvl:bkghvl,		$	; I*2 HV LAST SETTING
;			filament:filament,	$	; I*2 filament number
;			fcurrent:fcurrent,	$	; I*2 filament current
;			slitn:slitn,		$	; I*2 slit number
;			slitp:slitp,		$	; I*2 slit position
;			mirror:mirror	}		; I*2 mirror position
;
; Opt. Outputs: None
;               
; Keywords    : QUIET	do not print out the header, just return the structure
;
; Calls       : tm_calib, dec2hex
;
; Common      : None
;               
; Restrictions: None
;               
; Side effects: !
;               
; Category    : Telemetry
;               
; Prev. Hist. : None
;
; Written     : E Breeveld (erb@mssl.ucl.ac.uk) 6 March 1996
;			Cut from  new_raw_gis.pro
;               
; Modified    : ERB added decimal HV setting, 19 April 1996
;		ERB added quiet keyword; return hex and calibrated params
;			11 Jan 2000
;
; Version     : 2
;-            

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

function gis_raw_header, head, quiet=quiet

;
;  Decipher and print out the header of a GIS raw/pha file
;  Takes the header (head) as input
;  Returns a structure (head_str) with the data in variables
;
	header = head > 32b		; make sure it's printable!
	header = head < 128b

;
;  Translate data into 'Detector-speak'
;
	comment = string(header (0:79))

	detector = 0
	reads, string(header (92:99)), detector
	bughv = 0
	reads, string(header (170:179)), bughv
	bugmcpb = 0
	reads, string(header (190:199)), bugmcpb
	bigmcp = 0
	reads, string(header (210:219)), bigmcp
	bkgrcd = 0
	reads, string(header (250:259)), bkgrcd
	bkgulcd = 0
	reads, string(header (290:299)), bkgulcd
	bkgldld = 0
	reads, string(header (330:339)), bkgldld
	bkghvl = 0
	reads, string(header (350:359)), bkghvl
	filament = 0
	reads, string(header (410:419)), filament
	fcurrent = 0
	reads, string(header (430:439)), fcurrent
	slitn = 0
	reads, string(header (450:459)), slitn
	slitp = 0
	reads, string(header (470:479)), slitp
	mirror = 0
	reads, string(header (490:499)), mirror
;
;  Convert to hex all those required
;
	dec2hex, fcurrent, hfcurrent,  /quiet, nchar=2, /upper
	dec2hex, bughv, hbughv,  /quiet, nchar=2, /upper
	dec2hex, bkghvl, hbkghvl,  /quiet, nchar=2, /upper

;
;  Calibrate parameters to be calibrated
;
	cbughv = tm_calib('BUGHV1', bughv, /quiet)
	cbigmcp = tm_calib('BIGMCP1', bigmcp, /quiet)
	cbugmcpb = tm_calib('BUGMCPB1', bugmcpb, /quiet)

	head_str = $
		{	comment:comment,	$
			detector:detector,	$
			bughv:bughv,		$
			bugmcpb:bugmcpb,	$
			bigmcp:bigmcp,		$
			bkgrcd:bkgrcd,		$
			bkgulcd:bkgulcd,	$
			bkgldld:bkgldld,	$
			bkghvl:bkghvl,		$
			filament:filament,	$
			fcurrent:fcurrent,	$
			slitn:slitn,		$
			slitp:slitp,		$
			mirror:mirror,		$
			hex_fcurrent:hfcurrent,	$
			hex_bughv:hbughv,	$
			hex_bkghvl:hbkghvl,	$
			cal_bughv:cbughv,	$
			cal_bigmcp:cbigmcp,	$
			cal_bugmcpb:cbugmcpb		}
;
	if not keyword_set (quiet) then begin
		print, $
'--------------------------------------------------------------------------------'
		print, comment
		print,  'Detector      =', detector, $
			'     Filament =', filament, $
			'              @    0x', hfcurrent, ' current', $
				format = '(a, i8, a, i8, a, a, a)'
	
		print,  'HV read       =    0x', hbughv, $
			'              =', cbughv, $
			' kV           @', cbigmcp, ' microAmps', $
				format = '(a, a, a, f8.2, a, f8.2, a)'
	
		print,  'HV set        =    0x', hbkghvl, ' (', bkghvl, $
		   ' decimal)       Front Face Bias =', cbugmcpb, ' Volts', $
				format = '(a, a, a, i3, a, f8.2, a) 
	
		print,	'Raw counts    =', bkgrcd, $
			'   ULD counts =', bkgulcd, $
			'      LLD set =', bkgldld
	
		print,	'Mirror        =', mirror, $
			'     Slit no. =', slitn, $
			'    Slit pos. =', slitp
	
		print, $
'--------------------------------------------------------------------------------'
	endif
;
	return, head_str
end
