;+
; Project     : SOHO - CDS
;
; Name        : GHOST_AMOUNT
;
; Purpose     : Find amount of ghosting
;
; Explanation : 
;
; Use         : ghost_array = GHOST_AMOUNT (CHAN, GSET_ID, ARM_POS [,/QUIET])
;
; Inputs      : CHAN =      GIS channel (1-4)
;               GSET_ID = 	gset for the observation
;               ARM_POS =   array of arm positions for each pixel - needed
;                               only for the default processing
;
; Opt. Inputs : None.
;
; Outputs     : Returns amount of ghosting for each pixel address
;
; Opt. Outputs: None.
;
; Keywords    : QUIET =     Do not report informational messages
;
; Calls       :
;
; Common      : None.
;
; Restrictions: None.
;
; Side effects: None.
;
; Category    : Calibration, GIS, Intensity.
;
; Prev. Hist. : 
;
; Written     : Eddie Breeveld, erb@mssl.ucl.ac.uk  8 July 1998
;
; Modified    : v1.1,  8 Jul 98, erb, Better messages
;               v1.2, 10 Jul 98, erb, Check local directory too
;               v1.3, 30 Nov 98, erb, Changed default regions slightly
;               v1.4,  5 Jan 99, erb, Made better use of /quiet keyword
;               v1.5, 15 Mar 99, erb, Bug in /quiet processing!
;
; Version     : v1.4
;
;-
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;
function ghost_amount, chan, gset_id, arm_pos, quiet=quiet
;
;
    if (n_params() ne 3) then begin
        message, 'ERROR - Usage:', /continue
        chkarg, 'ghost_amount'
        retall
    endif
;
    ghost_array = fltarr(2048, 2)
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Open the relevant file, if possible
;
    dirname  = getenv('CDS_GIS_CAL_INT')
    filename = 'ghostfile_' + trim(gset_id) + '_' + trim(chan)
    fullname = dirname + '/' + filename 
;
    if not keyword_set(quiet) then $
            message, 'Trying ghost information file: ' + fullname, /continue
;
    if not file_exist(fullname) then begin
        if not keyword_set(quiet) then message, $
               'File not found, trying current working directory', /continue
        fullname = './' + filename
        if not file_exist(fullname) then begin
            message, $
                'Ghost Information File not found, using default values', /continue
;
; default processing for non-defined gset_ids, arm fractions below 0.1 and
;   above 0.6 are suspect
;
            arm_frac = arm_pos - fix(arm_pos)          ; position within arm
;
            ghost_array[(where ((arm_frac le 0.125) or $
                                (arm_frac ge 0.575))), *] = 0.5
;
            return, ghost_array
         endif
    endif
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; ghost amount/position file available:
;   read in the file with the data
;
    openr, lun, fullname, /get_lun
    if not keyword_set(quiet) then message, $
         'File found', /continue
;
; first the number of blocks in the position array
;
    readf, lun, n_blocks
;
; define and read the postion array:
;   [*, 0] - pixel positions
;   [*, 1] - up ghosting amount
;   [*, 2] - down ghosting amount
;
    position_array = fltarr (3, n_blocks) 
    readf, lun, position_array
    free_lun, lun
;
; now work out the amounts of ghosting for every pixel
;
    for i=0, n_blocks-2 do begin
        for j=position_array[0,i], position_array[0, i+1] do begin
            ghost_array [j, *] = position_array[1:2, i]
        endfor
    endfor
;
    return, ghost_array
end
