;+
; Project     : SOHO - CDS
;
; Name        : GHOST_BUSTER
;
; Purpose     : Move GIS detector ghosts
;
; Explanation : This procedure takes GIS data read with READCDSFITS and
;               removes ghosts.  It uses three different methods:
;
;               o   By default, ghost buster returns the qlds with all
;                   the ghosting regions marked as uncalibratable.
;                   i.e., ghost_buster returns only the ghost free regions.
;
;               o   Optionally (by adding the detector number) it will plot the
;                   data from one detector and prompt for manual ghost
;		    correction.  It then asks the user to bracket
;                   known ghosts; these can be moved back where they came
;                   from.  Other information plotted includes left and right
;                   shifted spectra with likely correllations and an
;                   (optional) theoretical spectrum.
;
;		    In manual mode areas of the spectrum can also be marked as
;		    unusable.
;
;		It is also possible to save and restore manually corrected
;		information; for instance if the same corrections have to
;		be applied to more than one data set.
;
;		o   To save manually corrections for ghosts, add the
;		    detector number and the keyword SAVE=save_struct.  The
;		    program will plot and prompt for ghosts as in the manual
;		    correction above.
;
;		o   To restore saved ghost corrections, add the detector
;		    number and the keyword RESTORE=save_struct.  Note that
;		    the qlds must use the same detector and GIS setup as in
;		    the original SAVE.
;
; Use         : For ghost free regions or automatic corrections:
;
;                   GHOST_BUSTER, QLDS [,/AUTOMATIC] [,/PLOT] $
;				[,/HARDCOPY] [,/ANGSTROMS] [,/NM]
;
;               or for manual corrections add the detector number:
;
;                   GHOST_BUSTER, QLDS, DETNUMBER [,/PLOT] $
;                       [,/HARDCOPY] [,/ANGSTROMS] [,/NM] [,/SAMPLE]
;			[,/LOGSCALE] [,/QUIET] [,SAVE=SAVE_STRUCT]
;
;		to restore previously saved ghosts:
;
;		    GHOST_BUSTER, QLDS, DETNUMBER, RESTORE=SAVE_STRUCT [,/PLOT]
;
; Inputs      : QLDS =      Structure variable of GIS data as read by
;                           READCDSFITS.
;
; Opt. Inputs : DETNUMBER = The detector number (1-4) to ghost correct manually
;
; Outputs     : QLDS =      returned with the ghosts corrected
;
; Opt. Outputs: None.
;
; Keywords    : HARDCOPY =  if set will plot the original and ghost busted
;                           spectra to printer
;
;		PLOT =	    Plot the original and reslutant data
;
;               ANGSTROMS = plot in Angstroms
;
;               NM =        plot in nanometers
;
;               SAMPLE =    Plot a sample theoretical spectrum
;
;               LOGSCALE =  Plot the log of the spectra
;
;               AUTOMATIC = Make a 'best guess' at restoring ghosts (the
;                           default is to return only ghost-free spectra)
;
;               SAVE =      If set to an empty variable then will return an
;                           array of structures with saved ghosting parameters.
;                           If not empty, then new ghosting parameters are
;                           appended to the array.  Note that the input array
;                           must be from data with the same GSET and detector
;                           number.
;
;			    Alternatively, if set to a string, then this string
;			    is interpreted as a filename, and the save
;			    structure is written to the file.  This is mainly
;			    for internal use, to produce the files needed for
;			    automatic ghost processing.
;
;		RESTORE =   If supplied, will use the correction data in the
;			    structure to move the ghosts.
;
;               QUIET =     Do not report informational messages.  All warnings
;                           and errors are still reported.
;
; Calls       : gt_windesc, ghost_plot_one, yes_no, ghost_test,
;               ghost_plot_sample, ghost_restore, xycursor, gt_windata
;
; Common      : None.
;
; Restrictions: None.
;
; Side effects: None.
;
; Still To Do : Add more complex automatic processing
;		Marking unusable areas
;
; Category    : Calibration, GIS, Intensity.
;
; Prev. Hist. : None.
;
; Written     : Eddie Breeveld, erb@mssl.ucl.ac.uk  13 November 1997
;
; Modified    : v2:     19 Nov 97, erb, Use ghost_test to plot correlation
;                               information too
;               v2.1:    1 Feb 98, erb, added hardcopy and angstrom switches
;                               and processing
;               v3:      2 Feb 98, erb, allowed user to re-process already
;                               ghost-corrected data
;               v3.1:   24 Feb 98, erb, add SAMPLE keyword
;               v4.0:    3 Mar 98, erb, Added LOGSCALE keyword
;               v4.1:   10 Mar 98, erb, Use ghost_cursor cursor
;               v4.2:   12 Mar 98, erb, improved ghost_cursor
;               v4.3:    3 Jul 98, erb, IDLv5, use [] not ()
;               v5:     26 Nov 98, erb, Default to returning only ghost free
;                               regions.
;                               Also add /quiet to repeated ghost_amount calls
;               v5.1:   13 Jan 98, erb, Make gis_calib to work properly with
;                               ghost free data - push all 'ghosting' data
;                               into pixel 0 - thus maintain total detector
;                               count rates
;               v5.2:   15 Jan 99, erb, minor improvement to GSET_ID
;                               processing
;               v5.3:   03 Feb 99, erb, limit count rates for quiet sun zone,
;                               plot thicker cursors, add nm switch
;               v5.4:   02 Mar 99, erb, remove redundant ghost_information_file
;                               messages
;               v5.5:   15 Mar 99, erb, bug in amounts of ghosting!  Also
;                               default to Angstroms
;               v5.6:   25 May 99, erb, Issue warning when quiet sun count
;		                rates very high; added QUIET keyword;
;		                check for GIS data
;	        v6:     28 Jun 99, erb, renamed 'ghost_restore' to
;		                'ghost_move'; added RESTORE keyword processing;
;	        	        print wavelengths of the cursors at the
;		                bottom of the manual mode plot; plot input vs.
;		                output data (via /PLOT and /HARDCOPY); added
;		                some automatic processing
;		v6.1	22 Jul 99, erb, default to moving data to the Right;
;				added /cross_cor keyword; use sensible colour
;				table
;               v7         Aug 03, caf, Implimented autmomatic ghost correction so
;                                       now automatically corrects some ghosts.
;
;
; Version     : 7
;
;-
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
pro ghost_buster_free, qlds, detno
;
; returns only ghost free regions (the default)
; works on one detector at a time
;
    windata = gt_windata(qlds, detno-1)
    windesc = gt_windesc(qlds, detno-1)
;
; first find where the ghosts are
;
    gis_ghosting, qlds.header.gset_id, detno, ghosts
;
; GIS_GHOSTING returns:
; Outputs     : GHOSTS  = an array containing ghosting information:
;                   0,* = lower ghost pixel address
;                   1,* = upper ghost pixel address
;                   2,* = position in arm (0.0 - 1.0)
;
; Find out the likely ghosting regions.
; Function ghost_amount returns the amounts of ghosting for each pixel
;
    amount = ghost_amount (detno, qlds.header.gset_id, ghosts[2,*], /quiet)
;
; ghost_amount returns the expected fractions of left and right ghosting
;   per pixel, if zero then no ghosting at that location
;
    ghost_pos = [where (total(amount, 2) gt 0.1)]         ; original positions
;
    ghost_low       = where (amount[*, 0] gt 0.0)       ; left ghosting
    ghost_low_pos   = ghosts[0, ghost_low>0]
;
    ghost_high      = where (amount[*, 1] gt 0.0)       ; right ghosting
    ghost_high_pos  = ghosts[1, ghost_high>0]
;
; put them together, MAKE INTO INTEGERS, and add one pixel either side
;
;;;    ghost_pixels = fix([ghost_pos[*], ghost_low_pos[*], ghost_high_pos[*]])
    ghost_pixels = fix(ghost_pos[*])
    ghost_pixels = [ghost_pixels, ghost_pixels+1, ghost_pixels-1] >0 <2047
;
; sort and remove duplicate pixels
;
    ghost_pixels = ghost_pixels[uniq(ghost_pixels, sort(ghost_pixels))]
;
; pick up totals per scan position in ghosted areas, ignoring missing data
;
    ghost_totals = total(windata[ghost_pixels, *, *, *]>0.0, 1)
;
; and set all ghosted areas to uncalibratable
;
    windata[ghost_pixels, *, *, *] = windesc.missing
;
; put removed data into pixel zero
;
    windata[0, *, *, *] =  ghost_totals
;
; restore the data
;
    st_windata, qlds, detno-1, windata
;
    return
end

;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
pro ghost_buster_automatic, qlds, detno
;
; set up the structure
;
    new_save = {save, pix_left:0, pix_right:0, direction:'a'}
;
; read in the information file
;
;

    
    g1=rd_ascii(concat_dir('$CDS_GIS_CAL_INT','ghostfile_'+strcompress(string(qlds.header.gset_id),/remove_all)+'_1'))
    if g1(0) eq '' then     g1=rd_ascii(concat_dir('$CDS_GIS_CAL_INT','ghostfile_42_1'))
    gh1=fltarr(n_elements(g1)-1,3)
    for a=1,n_elements(g1)-1 do gh1(a-1,*)=(str_sep(g1(a),'    '))(1:3)*1.

    g2=rd_ascii(concat_dir('$CDS_GIS_CAL_INT','ghostfile_'+strcompress(string(qlds.header.gset_id),/remove_all)+'_2'))
    if g2(0) eq '' then     g2=rd_ascii(concat_dir('$CDS_GIS_CAL_INT','ghostfile_42_2'))
    gh2=fltarr(n_elements(g2)-1,3)
    for a=1,n_elements(g2)-1 do gh2(a-1,*)=(str_sep(g2(a),'    '))(1:3)*1.

    g3=rd_ascii(concat_dir('$CDS_GIS_CAL_INT','ghostfile_'+strcompress(string(qlds.header.gset_id),/remove_all)+'_3'))
    if g3(0) eq '' then     g3=rd_ascii(concat_dir('$CDS_GIS_CAL_INT','ghostfile_42_3'))
    gh3=fltarr(n_elements(g3)-1,3)
    for a=1,n_elements(g3)-1 do gh3(a-1,*)=(str_sep(g3(a),'    '))(1:3)*1.
    
    g4=rd_ascii(concat_dir('$CDS_GIS_CAL_INT','ghostfile_'+strcompress(string(qlds.header.gset_id),/remove_all)+'_4'))
    if g4(0) eq '' then     g4=rd_ascii(concat_dir('$CDS_GIS_CAL_INT','ghostfile_42_4'))
    gh4=fltarr(n_elements(g4)-1,3)
    for a=1,n_elements(g4)-1 do gh4(a-1,*)=(str_sep(g4(a),'    '))(1:3)*1.
    
    wave1=pix2wave('G1',indgen(2048))
    wave2=pix2wave('G2',indgen(2048))
    wave3=pix2wave('G3',indgen(2048))
    wave4=pix2wave('G4',indgen(2048))
    
    corr=qlds.detdesc.units
    pw1=where(qlds.detdesc.waveband eq 1)
    pw2=where(qlds.detdesc.waveband eq 2)
    pw3=where(qlds.detdesc.waveband eq 3)
    pw4=where(qlds.detdesc.waveband eq 4)
    
if (detno eq 1 and ((qlds.detdesc.units)(pw1>0))(0) ne "de-Ghosted counts per pixel") then begin
  printf,1,' '
  printf,1,'----------------------------------------------------'
  printf,1,'--    GIS Detector 1 - Likely Ghosting Regions     --'
  printf,1,'----------------------------------------------------'
  printf,1,''
  c=where(gh1(*,1) gt 0.)
  for a=0,n_elements(c)-1 do printf,1,'                  '+strcompress(string(wave1(gh1(c(a),0))),/remove_all)+'-'+strcompress(string(wave1(gh1(c(a)+1,0))),/remove_all)
  printf,1,''
  printf,1,''  
  ll=cf_180lg(qlds,tcr=tcr)
  printf,1,'Fe XI 180.2 A line ghosts to 160 A, '+strcompress(string(fix(tcr)),/remove_all)+' counts recovered'
  ll=cf_177lg(qlds,tcr=tcr)
  printf,1,'Fe X 177.1 A line ghosts to 157 A, '+strcompress(string(fix(tcr)),/remove_all)+' counts recovered'
  ll=cf_174lg(qlds,tcr=tcr)
  printf,1,'Fe X 174.0 A line ghosts to 154 A, '+strcompress(string(fix(tcr)),/remove_all)+' counts recovered'
  printf,1,'Fe XII 193.4 A line ghosts into 175 A --- Blend / unrecovered'
  printf,1,'Fe XII 192. A line ghosts into 174.0 A --- Blend / unrecovered'
  printf,1,''
  corr(pw1)="de-Ghosted counts per pixel"

endif
if (detno eq 2 and (qlds.detdesc.units)(pw2>0) ne "de-Ghosted counts per pixel") then begin
  printf,1,'----------------------------------------------------'
  printf,1,'--    GIS Detector 2 - Likely Ghosting Regions     --'
  printf,1,'----------------------------------------------------'
  printf,1,''
  c=where(gh2(*,1) gt 0.)
  for a=0,n_elements(c)-1 do printf,1,'                  '+strcompress(string(wave2(gh2(c(a),0))),/remove_all)+'-'+strcompress(string(wave2(gh2(c(a)+1,0))),/remove_all)
  printf,1,''
  printf,1,'Fe XVI 335.4 A line ghosts into Si VIII 319.8 A --- Blend / unrecovered'
  printf,1,''
  c=where(gh2(*,1) gt 0.)
  corr(pw2)="de-Ghosted counts per pixel"
endif
if (detno eq 3 and (qlds.detdesc.units)(pw3>0) ne "de-Ghosted counts per pixel") then begin
  printf,1,'----------------------------------------------------'
  printf,1,'--    GIS Detector 3 - Likely Ghosting Regions     --'
  printf,1,'----------------------------------------------------'
  printf,1,''
  c=where(gh3(*,1) gt 0.)
  for a=0,n_elements(c)-1 do printf,1,'                  '+strcompress(string(wave3(gh3(c(a),0))),/remove_all)+'-'+strcompress(string(wave3(gh3(c(a)+1,0))),/remove_all)
  printf,1,''
  corr(pw3)="de-Ghosted counts per pixel"
endif
if (detno eq 4 and (qlds.detdesc.units)(pw4>0) ne "de-Ghosted counts per pixel") then begin
  c=where(gh4(*,1) gt 0.)
;  for a=0,n_elements(c)-1 do print,'                  '+  strcompress(string(wave4(gh4(c(a),0))),/remove_all)+'-'+strcompress(string(wave4(gh4(c(a)+1,0))),/remove_all)
  printf,1,'----------------------------------------------------'
  printf,1,'--    GIS Detector 4 - Likely Ghosting Regions     --'
  printf,1,'----------------------------------------------------'
  printf,1,''
  c=where(gh4(*,1) gt 0.)
  for a=0,n_elements(c)-1 do printf,1,'                  '+  strcompress(string(wave4(gh4(c(a),0))),/remove_all)+'-'+strcompress(string(wave4(gh4(c(a)+1,0))),/remove_all)
  printf,1,''
  printf,1,''
  ll=cf_765lg(qlds,tcr=tcr)
  printf,1,'N IV 765 A line ghosts to 738.4 A, '+strcompress(string(fix(tcr)),/remove_all)+' counts recovered'
;  printf,1,'? 768.8 A line ghosts to 742.5 A --- Recovered'
  ll=cf_770lg(qlds,tcr=tcr)
  printf,1,'Ne VIII 770 A line ghosts to 744.4 A, '+strcompress(string(fix(tcr)),/remove_all)+' counts recovered'
;    printf,1,'Ne VIII 779.4 A line ghosts into 756.1 A --- Recovered ???? is it a ghost'
    printf,1,''
    printf,1,'Mg IX 360(x2) A line ghosts into O III 703.8 A --- Blend / unrecovered'
    printf,1,''
    printf,1,'     *** O III is a chromospheric line hence this ghost can be ***'
    printf,1,'     *** recovered for offlimb observations.                   ***'    
    printf,1,''
    printf,1,'              ----------------------'        
   corr(pw4)="de-Ghosted counts per pixel"
  endif
  qlds.detdesc.units=corr
    if not keyword_set(quiet) then begin
    print, "Relocating unblended ghosted lines !"
  endif

  end
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
pro ghost_buster_manual, qlds, detno, save=save, cross_cor=cross_cor, $
            angstroms=angstroms, sample=sample, logscale=logscale, nm=nm
;
; load a suitable colour scale.  nb some colours overridden by ghost_plot_one
;
    loadct, 39, /silent
;
; get ghosting parameters
;
    gis_ghosting, qlds.header.gset_id, detno, ghosts
;
; Plot the data from the detector
;
    ghost_plot_one, qlds, detno, angstroms=angstroms, sample=sample, $
          logscale=logscale, nm=nm, cross_cor=cross_cor
;
; loop over the ghosts
;
    contin = ' '
    while contin do begin
;
; Ask user to input cursor position of the LHS of the ghost(s)
;
        print
        print, '    Press and drag the cursor button over a ghost'
;
        ghost_cursor, X1, X2, ghosts, detno, angstroms=angstroms
;
        ghost_left  = X1
        ghost_right = X2
;
; Ask user if the data is to be moved to the Left or the Right
;
        answer = ''
        print, ''
        print, '    Does this ghost need to be moved left or right? (L/R) '
        read, answer, $
            prompt='        Press C to ignore this ghost, ' + $
                            'or E to ignore and exit [Default: R]: '
;
        direction = strupcase(answer)
        if direction eq '' then direction = 'R'                 ; R default
        if direction eq 'E' then return                         ; E for exit
        if (direction eq 'L') or (direction eq 'R') then begin  ; don't ignore
;
; Move 'em.  Check to see if we're using Angst.
;
            if keyword_set(angstroms) then ghost_move, qlds, $
              ghost_left/10.0, ghost_right/10.0, direction, detno, save $
            else ghost_move, qlds, $
              ghost_left, ghost_right, direction, detno, save
;
        endif
;
; replot the data from the detector
;
        ghost_plot_one, qlds, detno, angstroms=angstroms, sample=sample, $
              logscale=logscale, /quiet, cross_cor=cross_cor
;
; Ask user if there are any more ghosts
;
        contin = ''
        print, ''
        yes_no, '    Any more ghosts? ', contin
   endwhile                                     ; contin
;
end
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
pro ghost_plotthick, x, yt, yb, linestyle=linestyle
;
    device, get_graphics=old, set_graphics=6    ; xor lines and fills
;
; convert to device coords (easier to make thicker)
;
    dev = convert_coord([x, yt], /data, /to_device)
    dev_x = dev[0]
    dev_yt = dev[1]
    dev_yb = (convert_coord([x, yb], /data, /to_device))[1]
;
; plot 3 times
;
    for i=-1, 1 do plots, [dev_x+i, dev_x+i], [dev_yt, dev_yb], /device, $
                        linestyle=linestyle, color=!d.table_size/1.5
;
; print wavelength
;
    xyouts, dev_x, 1, trim(x), /device, color=!d.table_size/1.5
;
    device, set_graphics=old
    return
end
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;
pro ghost_cursor, x1, x2, ghosts, detno, angstroms=angstroms
;
; procedure to 'drag' a cursor over the region of interest
;
    adet = 'G' + trim(detno)
    if keyword_set (angstroms) then conv=1 else conv=10
;
; find limits of plot
;
    yb = !y.crange[0]           ; y bottom
    yt = !y.crange[1]           ; y top
;
; set up cursor
;
    cursor, x1, y1, /nowait                         ; where are we now?
;
    lghost = pix2wave (adet, $
      ghosts[0, wave2pix(adet, x1*conv, /real, /limit)], /limit) / conv
    rghost = pix2wave (adet, $
      ghosts[1, wave2pix(adet, x1*conv, /real, /limit)], /limit) / conv
;
; plot current cursor position and ghosts as dotted
;
    ghost_plotthick, x1, yt, yb
    ghost_plotthick, lghost, yt, yb, linestyle=1
    ghost_plotthick, rghost, yt, yb, linestyle=1
;
; Loop until a button is pressed, drawing a line from top to bottom
;
    button = 0L
    while button eq 0L do begin
        old_x = x1
        old_l = lghost
        old_r = rghost
        cursor, x1, y1, /change           ; wait for change
        button = !mouse.button
;
; remove old line and plot new
;
        ghost_plotthick, old_x, yt, yb
        ghost_plotthick, x1, yt, yb
;
; also remove old and plot left and right ghosts as dotted lines
;
        lghost = pix2wave (adet, $
          ghosts[0, wave2pix(adet, x1*conv, /real, /limit)], /limit) / conv
        rghost = pix2wave (adet, $
          ghosts[1, wave2pix(adet, x1*conv, /real, /limit)], /limit) / conv
;
        ghost_plotthick, old_l, yt, yb, linestyle=1
        ghost_plotthick, old_r, yt, yb, linestyle=1
        ghost_plotthick, lghost, yt, yb, linestyle=1
        ghost_plotthick, rghost, yt, yb, linestyle=1
    endwhile
;
; Button depressed, remove old line
;
    ghost_plotthick, x1, yt, yb
;
; Loop until the button is released,
; filling the region with the inverse colour
;
    x2 = x1
    button = 1L

    while button ne 0L do begin
        old_x = x2
        cursor, x2, y2, /change
        button = !mouse.button
;
; do polyfill
;
        device, get_graphics=old, set_graphics=6
        polyfill, [x2, x2, old_x, old_x], [yt, yb, yb, yt]
	device, set_graphics=old
    endwhile
;
    return
end                                                 ; ghost_cursor
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;
pro ghost_buster, qlds, detnumber, save=save, hardcopy=hardcopy, $
    angstroms=angstroms, nm=nm, plot=plot, cross_cor=cross_cor, $
    sample=sample, logscale=logscale, manual=manual, $
    quiet=quiet, restore=restore,free=free;,nosmooth=nosmooth
;
  ;
  ; Firstly read ghost information files to find out likely ghosting regions
  ; and report those to the user, and record in text file.
  ;
  ;; need to do the concat dir ... check the existance of a file for the
  ;; gset etc ...
  sl=[' ','2"x2"','4"x4"','8"x50"','2"x240"','4"x240"','90"x240"']
  ; open ghost file for writing ; need to add check for write permission.
  ww=curdir()  
  openw,1,concat_dir(ww,(str_sep(qlds.header.filename,'.fits'))(0)+'ghost.readme')
  printf,1,qlds.header.title + ' / '+qlds.header.sci_obj
  printf,1,qlds.header.date_obs + ', ' +qlds.header.filename
  ; printf,1,qlds.header.gset_id + ', ' +qlds.header.filename
  line='Gset: '+strcompress(string(qlds.header.gset_id),/remove_all)+' ; Slit no/size: '+strcompress(string(qlds.header.slit_num),/remove_all)+' / '+sl(qlds.header.slit_num)
  line=line+' ; Pointing: ['+strcompress(string(fix(qlds.header.xcen)),/remove_all)+ ','+strcompress(string(fix(qlds.header.ycen)),/remove_all)+']'
  printf,1,line
  printf,1,' '
    
    ;;;;;;;;;;;;;;;;;;;;
    ;
    ; Check input parameters
    ;
    if (n_params() lt 1) or (n_params() gt 2) then begin
        message, 'ERROR - Incorrect number of arguments', /continue
        chkarg, 'ghost_buster'
        retall
    endif
;
; Make sure that it is GIS data we're dealing with.
;
    if QLDS.HEADER.DETECTOR ne 'GIS' then begin
        MESSAGE, /continue, 'ERROR - Cannot use this routine with VDS data'
        return
    endif
;
; Default to Angstroms
;
    if (not keyword_set(nm)) and (not keyword_set(pixels)) then angstroms=1
;
; Check for known duplicate GSETs (just 50 and 52 at the moment)
;
    if qlds.header.gset_id eq 50 then qlds.header.gset_id = 52
;
;  Set up the structure with the GIS processing stages
;
@gis_stage.inc
;
; Get the detector numbers
;
; next line corrected for by caf@mssl
    ;;    if n_params() eq 1 then detno = [1, 2, 3, 4] else detno = [detnumber]
    if n_params() eq 1 then detno = qlds.detdesc.waveband else detno = [detnumber]
ndetno=n_elements(detno)
    ;
; loop over all detectors asked for
;
;;; fixed by carl so works for less than all windows
for i = 0,ndetno-1 do begin
      temp = gt_windesc(qlds, i)
      old_units = temp.units
      ;;qlds.detdesc.units(*)=temp.units

      ;
; Check the data units in each detector, and smooth is required
;
        case old_units of
;
            GIS_STAGE.SMOOTHED:           ; just continue
;
            GIS_STAGE.RAW: begin
;                if not keyword_set(quiet) then $
;message, /continue, 'Data not smoothed, use gis_smooth,qlds to smooth data '
              ;  gis_smooth, qlds
            end
;
            GIS_STAGE.GHOST_CORRECT: begin
                if not keyword_set(quiet) then begin
                    bell
                    message, /continue,  $
'Warning - Some data already ghost corrected in detector ' + trim(detno[i])
                endif
            end
;
            else: begin
                MESSAGE, 'ERROR - Data units = "' + old_units + $
			'" in detector ' + $
                        trim(detno[i]) + ', cannot ghost correct', /continue
		return
            end
;
        endcase
    endfor          ; loop over detectors
;
; save the old qlds
;
    orig_qlds = copy_qlds(qlds)
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
STAGE_2:
;
; check which processing required
;
    case n_params() of
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
        1: begin                    ; implies automatic or ghost_free
;
; check if Quiet Sun and count rate is within limits
;
            if qlds.header.zone_id eq 4 then begin  ; quiet sun
;
; pick up window 0 (i.e., detector 1) and find its average count rate,
;  ignoring missing data
;
                avg_det0 = $
 average(gt_windata(qlds, 0), missing=qlds.detdesc(0).missing) / $
 qlds.header.exptime
;
                slitsize = [0, $			; doesn't exist!
				 2.032 *   2.032, $
				 4.064 *   4.064, $
				 8.128 *  50.8,   $
				 2.032 * 240.24,  $
				 4.064 * 240.24,  $
				90.000 * 240.24 ]
                slitnum = qlds.header.slit_num
                if (slitnum lt 1) or (slitnum gt 6) then begin $
                    message, 'ERROR - not using a valid slit (' $
                            + trim(slitnum) + ')', /continue
		    return
		endif
;
                avg_det0 = avg_det0/slitsize[slitnum]
;
; if the average (in GIS1) is greater than 0.04 counts/sec/(arcsec**2 of slit),
;   then the data rate is high for quiet sun
;
                if avg_det0 gt 0.04 then $
                    message, 'Warning - count rate (' + trim(avg_det0) + $
                            ' counts/sec/arcsec^2 of slit) high for ' + $
                            'quiet sun processing', /continue
            endif
;
; loop over each detector
;
	    for i=0, ndetno-1 do begin

;
	      ; further check for ghost_free or automatic, then do it.
	      ;
	      
	      ;help,i,detno[i]
	      if keyword_set(free) then ghost_buster_free, qlds, detno[i] $
                        else $
                        ghost_buster_automatic, qlds, detno[i]
            endfor
        end
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
        2: begin                ; implies manual processing
;
            case 1 of
	    keyword_set(AUTOMATIC): begin
                    message, /continue, $
 'ERROR - Cannot use automatic keyword on individual detectors'
                    return
	        end
;
	    keyword_set(RESTORE): begin
		    ghost_restore, qlds, detno[0], restore, quiet=quiet
		end
            else: begin
;
; a manual ghost_buster call.  Check if the save structure goes into a file.
;
		    file_save = ''
		    if datatype(save, 1) eq 'String' then $
				file_save = save
;
                    ghost_buster_manual, qlds, detno[0], save=save, nm=nm, $
                        angstroms=angstroms, sample=sample, $
			logscale=logscale, cross_cor=cross_cor
;
; If wanted, write out the saved information
;
		    if file_save ne '' then begin
			filename = file_save + '.' + $
					trim(save[0].(0)) + $
					'_' + trim(save[0].(1))
			openw, lun, filename, /get_lun
			printf, lun, n_elements(save), $
						'   Saved on', systime()
			format = '(i / i / a)'
			printf, lun, save, format=format
			free_lun, lun
;
		    endif	; filesave
                endelse		; case default
	    endcase		; case on keyword_set
        end			; 2 params (manual) processing
;
    endcase 			; n_params()
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Loop over the detectors
;
    for i=0, n_elements(detno)-1 do begin
;
; Mark data as de-ghosted
;

pww=where(qlds.detdesc.waveband eq detno[i])

        qlds.detdesc[pww].units = gis_stage.ghost_correct
;
; check for plot and hardcopy keywords
;
	if keyword_set(plot) then begin
	    !p.multi=[0, 1, 2]
	    window, i
	    gis_plot, qlds, detno[i], logscale=logscale, $
			angstroms=angstroms, nm=nm
	    gis_plot, orig_qlds, detno[i],  $
			logscale=logscale, angstroms=angstroms, nm=nm
	endif
        if keyword_set(hardcopy) then begin
            ps
	    !p.multi=[0, 1, 2]
            gis_plot, qlds, detno[i], logscale=logscale, $
                    angstroms=angstroms, nm=nm
	    gis_plot, orig_qlds, detno[i], $
			logscale=logscale, angstroms=angstroms, nm=nm
            psplot
        endif
	!p.multi = 0
    endfor
;
; All done!
;
;    if not keyword_set(quiet) then print, 'GHOST_BUSTER Done'
    goto, finish
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Exit
;
FINISH:
print,'A ghost information file has been generated :',concat_dir(ww,(str_sep(qlds.header.filename,'.fits'))(0)+'ghost.readme')
close,1

    return
end
