;+
; Project     : SOHO - CDS
;
; Name        : GHOST_INFO
;
; Purpose     : Display the ghosting information associated with a QLDS
;
; Explanation : The ghosting information is kept in a 'ghost information file'
;               on a pixel by pixel basis.  This routine will translate the 
;               information for one detector into wavelength scale, print 
;               it to the screen and/or save the information as a returned 
;               array.
;
; Use         : GHOST_INFO, QLDS, DETNO, [,INFO] [,/ANGSTROMS] [,/NM], $
;			[/NOPRINT]
;
; Inputs      : QLDS =      Structure variable of GIS data as read by
;                           READCDSFITS.
;              
;               DETNO =     The detector number (1-4)
;
; Outputs     : None.
;
; Opt. Outputs: INFO =      The ghosting information stored in an array
;                           The size of the array is 7*2048.
;                               Column 1:       pixel number
;                               Column 2:       Wavelength (nm or A)
;                               Column 3:       Left ghost position (or 0.0)
;                               Column 4:       Approx. left ghost amount (0-1)
;                               Column 5:       Right ghost position (or 0.0)
;                               Column 6:       Approx. right ghost amount(0-1)
;                               Column 7:       Spiral arm position
;
; Keywords    : ANGSTROMS = output information in Angstroms (default)
;
;               NM =        output information in nanometers
;
;               NOPRINT =   do not print the information to the screen
;
; Calls       : pix2wave, restore_wavecal, gis_ghosting, ghost_amount
;
; Common      : None.
;
; Restrictions: None.
;
; Side effects: None.
;
; Category    : Calibration, GIS, Intensity.
;
; Prev. Hist. : None.
;
; Written     : Eddie Breeveld, erb@mssl.ucl.ac.uk 19 July 1999
;
; Modified    : 
;
; Version     : 1
;
;-
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
pro ghost_info, qlds, detno, info, noprint=noprint, nm=nm, angstroms=angstroms
;
; check
;
    if n_params() lt 2 then begin
        message, /continue, 'Insufficient parameters supplied'
        message, /continue, $
 'Usage:    ghost_info, qlds, detno, [info], [/nm] [,/angstroms] [,/noprint]'
        return
    end
;
; use proper wavelength  data
;
    result = restore_wavecal(qlds)
    waves = pix2wave('GIS' + trim(detno), findgen(2048))
;
; get the ghosting info for this qlds and detector number
;
; GIS_GHOSTING returns:
; Outputs     : GHOSTS  = an array containing ghosting information:
;                   0,* = lower ghost pixel address
;                   1,* = upper ghost pixel address
;                   2,* = position in arm
;
    gis_ghosting, qlds.header.gset_id, detno, ghosts
;
; convert pixels to wavelength, getting correct calibration first
;
    ghosts[0,*] = pix2wave('GIS' + trim(detno), ghosts[0,*], /limit)
    ghosts[1,*] = pix2wave('GIS' + trim(detno), ghosts[1,*], /limit)
;
    ghosts[where(ghosts eq waves[0])] = 0.0
;
; get the ghost information file info
;
    amount = ghost_amount (detno, qlds.header.gset_id, ghosts[2,*])
;
    amount = 10*(fix(amount * 10))	; integer percentage, 10% resolution
;
; swap ghost array around to lose extra dimensions
;
    ghosts = transpose(ghosts)
    print
;
    if keyword_set(nm) then begin
        scale = 10.0
        format = '(("pixel",  "  wavelength", ' + $
                '"  Left ghost, percent", ' + $
                '"  Right ghost, percent", ' + $
                '"  (arm position)", / ' + $
                '19(i5, f9.4, " nm", ' + $
                    'f9.3, " nm,", i8, ' + $
                    'f10.3, " nm,", i8, ' + $
                    'f15.3 /)))'
    endif else begin
        scale = 1.0
        format = '(("pixel",  "  wavelength", ' + $
                '"  Left ghost, percent", ' + $
                '"  Right ghost, percent", ' + $
                '"  (arm position)", / ' + $
                     '19(i5, f10.3, " A", ' + $
                        'f10.3, " A,", i8, ' + $
                        'f11.3, " A,", i8, ' + $
                        'f15.3 /)))'
    endelse
;
; save into 'info' and print all information (if wanted)
;
    info = transpose([ [indgen(2048)], [waves/scale], $
                        [ghosts[*,0]/scale], [amount[*, 0]], $
                        [ghosts[*,1]/scale], [amount[*, 1]], $
                        [ghosts[*,2]] ])
;
    if keyword_set(noprint) then return
;
; the following adapted from the routine 'more'
;
    scrn = filepath (/TERMINAL)
    openw, lun, scrn, /more, /get_lun
;
    printf, lun, info, format=format
;
    free_lun, lun
    return
end
