;+
; Project     : SOHO - CDS
;
; Name        : GHOST_MOVE
;
; Purpose     : Manually move GIS detector ghosts
;
; Explanation : This procedure moves GIS ghosts back.  It is intended to be
;               used only by ghost_buster.
;
; Use         : GHOST_MOVE, qlds, ghost_left, ghost_right, direction, $
;                   detno, save [,/pixels]
;
; Inputs      : QLDS        = Structure variable of GIS data as read by 
;                               READCDSFITS.
;               GHOST_LEFT  = LHS of ghost to be moved, in nm.
;               GHOST_RIGHT = RHS of ghost to be moved, in nm.
;               DIRECTION   = 'L' or 'R', for left or right direction of 
;                               movement
;               DETNO       = The detector number to ghost bust
;
; Opt. Inputs : None.
;
; Outputs     : QLDS  = returned with the ghosts corrected
;
; Opt. Outputs: SAVE  = append valid ghosting parameters into a structure
;
; Keywords    : PIXELS  = if set, then the LHS and RHS are in pixels, not nm.
;
; Calls       : gis_ghosting, gt_windata, gt_windesc, congrid, st_windata
;
; Common      : None.
;
; Restrictions: GHOST_PLOT_ONE must be called before using this 
;                       routine, unless /PIXELS is set
;
; Side effects: None.
;
; Category    : Calibration, GIS, Intensity.
;
; Prev. Hist. : None.
;
; Written     : Eddie Breeveld, erb@mssl.ucl.ac.uk  13 November 1997
;
; Modified    : v2:    2 Feb 98; erb; Tidied up and allowed time-series data
;               v2.1: 15 Jan 99; erb; improved gset_id processing
;               v3    22 Jun 99: erb; renamed ghost_move from ghost_restore, 
;                                   added PIXELS keyword, checked for
;                                   save structure
;               v4       Aug 03: caf; removed check on size of moved array so
;                                     it will now move whole LUT arms etc
; Version     : v4
;
;-
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;

pro ghost_move, qlds, ghost_left, ghost_right, direction, detno, $
        save, pixels=pixels
;
;
    spectrum  = 'GIS' + trim(detno)
;
    if keyword_set (pixels) then begin
        pix_right = ghost_right
        pix_left  = ghost_left
    endif else begin
        left      = wave2pix (spectrum, ghost_left*10.0, /limit)
                                        ; n.b. in Angstroms
        right     = wave2pix (spectrum, ghost_right*10.0, /limit)
                                        ; n.b. in Angstroms
        pix_right = max ([right, left])
        pix_left  = min ([right, left])
    endelse
;
; Check the pixels are sensible
;
;;    if (pix_right-pix_left) gt 100 then begin
;;        message, 'ERROR - moving more than 100 pixels, or data' + $
 ;;                       ' out of bounds. Move ignored', /continue
;;        return
;;    endif
;;    if (pix_right-pix_left) lt 4 then begin
;;        message, 'ERROR - moving less than 4 pixels, or data' + $
;;                        ' out of bounds. Move ignored', /continue
;;        return
;;    endif
;
    if direction eq 'L' then dir_index = 0 else dir_index=1
;
; Work out where the ghosts come from
;
    gis_ghosting, qlds.header.gset_id, detno, ghosts
;
; get the moved LHS and RHS positions
;
    moved_left  = fix (ghosts[dir_index, pix_left] + 0.5)
    moved_right = fix (ghosts[dir_index, pix_right] + 0.5)
;
; Check for movement outside the plotting range
;
    if min([moved_left, moved_right]) le 0 then begin
        message, 'ERROR - moved pixel(s) are outside detector range. ' + $
                        'Data ignored', /continue
        return
    endif
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Return data to the original position:
;  First get the data
;
    windata = gt_windata (qlds, detno-1)
    windesc = gt_windesc (qlds, detno-1)
;
; Extract the ghosts from the originals
;
    data_to_move = windata[pix_left:pix_right, *, *, *]
;
; Check for missing pixels in the selected data
;
    mispix = where (data_to_move eq windesc.missing, n)
    if (n gt 0) then begin
        message, /continue, 'Warning: There are ' + trim(n) + $
            ' missing or previously moved pixels in the selected data. ' + $
                        ' These will be not be moved'
        data_to_move(mispix) = 0.0
    endif
;
; Then set to missing the original data
;
    windata(pix_left:pix_right, *, *, *) = windesc.missing
;
; Expand/contract the data to fit into the new spot
;
    new_size = moved_right - moved_left + 1
;
; use congrid differently depending on the number of dimensions of the data
;
    datasize  = size (data_to_move)
    case 1 of
        datasize(0) eq 1:  moved_data = congrid (data_to_move, new_size)
        datasize(0) eq 2:  moved_data = congrid (data_to_move, new_size, $
                             datasize(2))
        datasize(0) eq 3:  moved_data = congrid (data_to_move, new_size, $
                             datasize(2), datasize(3))
        (datasize(0) eq 4) and (datasize(3) eq 1):  begin
;
; data is 4-dimensional (i.e., time series data), collapse one dimension
;  by removing y (dimension 3) data if possible (datasize(3) eq 1)
;
               data_to_move = reform (data_to_move, datasize(1), datasize(2), $
                 datasize(4), /overwrite)
               moved_data = congrid (data_to_move, new_size, datasize(2), $
                 datasize(4))
;
; and back...
;
               moved_data = reform (moved_data, new_size, datasize(2), $
                 datasize(3), datasize(4), /overwrite)
            end
;
     else:  begin
                message, /continue, $
        'ERROR: Cannot cope with this GIS dataset - Too many ' + $
        'dimensions (' + trim(datasize(0)) + ') for CONGRID'
                retall
            endelse
    endcase
;
; Now normalise the data
;
    moved_data = moved_data * total(data_to_move) / total(moved_data)
;
; Check for missing pixels in the new location
;
    mispix = where (windata(moved_left:moved_right, *, *) eq windesc.missing, $
                        n)
    if (n gt 0) then begin
        message, /continue, 'Warning: There are ' + trim(n) + $
                ' missing or previously moved pixels' + $
                ' in the new location.  These will be ignored'
    endif
;
; Add to the new spot.  n.b. the >0.0 construct will remove missing data
;
    if (windesc.missing le 0.0) then $
        windata(moved_left:moved_right, *, *, *) = $
            (windata(moved_left:moved_right, *, *, *) > 0.0) + moved_data $
    else begin
        message, /continue, $
                'ERROR: Cannot cope with a positive flag for missing data.'
        retall
    endelse
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Save information into array
;
    if n_params() eq 6 then begin
        new_save = {save, pix_left:pix_left, $
                   pix_right:pix_right, direction:direction}
        print,'saving data...'
;
        if (datatype(save, 1) ne 'Structure') then begin
;
; Set up structure array, first element with GSET and detector number
;
            save = [{save, qlds.header.gset_id, detno, 'H'}, new_save]
        endif else begin
;
; Check the array header
;
            if (save(0).(0) eq qlds.header.gset_id) and $
              (save(0).(1) eq detno) and (save(0).(2) eq 'H') then $
              save = [save, new_save] $
            else begin
                message, /continue, $
                        'ERROR: Save array inconsistent GSET or DETNO.'
                retall
            endelse
        endelse
    endif
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Put the new data back into the qlds
;
    st_windata, qlds, detno-1, windata
;
    return
end
