;+
; Project     :	SOHO - CDS
;
; Name        :	GHOST_PLOT_ALL
;
; Purpose     :	Plots all 4 GIS detector data with likely ghost regions
;
; Explanation :	This function takes GIS images read with READCDSFITS and
;		plots the average spectra for each detector with:
;			blue shifted spectrum
;			red shifted spectrum
;			likely ghosting regions  
;
;		Further software is being developed to 'move' selected ghosts
;		back to original locations
;
; Use         :	GHOST_PLOT_ALL, QLDS
;
;		Suggested use:
;			Use GHOST_PLOT_ALL to give a rough idea of where
;			the likely ghosts are for all 4 detectors
;			then use GHOST_PLOT_ONE with and without /LogScale
;			to zoom in to the desired features
;		e.g.,
;		    	IDL> qlds = readcdsfits('s7128r00')
;			    IDL> ghost_plot_all, qlds
;		    	IDL> ghost_plot_one, qlds, 3, /logscale
;		    	IDL> ghost_plot_one, qlds, 2
;
; Inputs      :	qlds	= Structure variable as read by READCDSFITS.
;
; Opt. Inputs :	None.
;
; Outputs     :	None.
;
; Opt. Outputs:	None.
;
; Keywords    :	None.
;
; Calls       :	ghost_plot_one
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Category    :	Calibration, GIS, Intensity.
;
; Prev. Hist. :	None
;
; Written     :	Eddie Breeveld, erb@mssl.ucl.ac.uk  7 March 1997
;
; Modified    :	erb,  1 March 1999, added nm and angstrom switches
;               erb, 15 March 1999, default to Angstroms
;
; Version     : v3
;
;-
;
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
pro ghost_plot_all, qlds, angstroms=angstroms, nm=nm
;
	if (n_params() ne 1) then begin
		print, 'ERROR - GHOST_PLOT_ALL - Usage:  IDL> ghost_plot_all, qlds'
		return
	endif
;
    if not (keyword_set(nm)) then angstroms=1
;
;  Plot all 4 GIS detector likely ghost plots
;
	filename  = strmid (qlds.header.filename, 0, strpos (qlds.header.filename,'.'))
	gsetid    = qlds.header.gset_id
;
	title = string ('File: ', filename, ', ', 'GSET_ID: ', trim (gsetid))

	!p.multi = [0,2,2]
;
	for det=1, 4 do ghost_plot_one, qlds, det, /nolabel, /logscale, $
        	angstroms=angstroms, nm=nm
;
	xyouts, 0.5, 0.48, /normal, align=0.5, 'Wavelength'
	xyouts, 0.0, 0.48, /normal, align=0.0, 'GIS 3'
	xyouts, 1.0, 0.48, /normal, align=1.0, 'GIS 4'
;
	xyouts, 0.5, 0.98, /normal, align=0.5, title 
	xyouts, 0.0, 0.98, /normal, align=0.0, 'GIS 1'
	xyouts, 1.0, 0.98, /normal, align=1.0, 'GIS 2'
;
;  Label the first graph (GIS1) with addition info.
;
	xyouts, 0.425, 0.93, /normal, 'log(blue shifted!C  spectrum)', charsize=0.5
	xyouts, 0.025, 0.87, /normal, 'log(red shifted spectrum)', charsize=0.5
	xyouts, 0.025, 0.82, /normal, 'log(original spectrum)', charsize=0.5
	xyouts, 0.050, 0.55, /normal, 'likely ghosting regions', charsize=0.5


	!p.multi = 0
return
end
