;+
; Project     : SOHO - CDS
;
; Name        : GHOST_PLOT_ONE
;
; Purpose     : Plots a GIS detector data with likely ghost regions
;
; Explanation : This procedure takes GIS data read with READCDSFITS and
;               plots the average spectra for each detector with:
;               blue shifted spectrum
;               red shifted spectrum
;               likely ghosting regions  
;
;               GHOST_BUSTER, q.v., will 'move' selected ghosts back to 
;               original locations.
;
;               GHOST_TEST, q.v., will plot correllation coefficients on top
;               of the resulting plot.
;
; Use         : GHOST_PLOT_ONE, QLDS, CHAN [, /NOLABEL] [, /LOGSCALE] $
;                   [, /ANGSTROMS] $
;                   [, WAVERANGE=[min,max] ] [, GHOSTDATA] [, /PIXELS] $
;                   [, SMOOTHSIZE=smoothsize] [,/sample] [/noghost]
;		    [, /CROSS_COR]
;
;               Suggested use:
;                   Use GHOST_PLOT_ALL to give a rough idea of where
;                   the likely ghosts are for all 4 detectors
;                   then use GHOST_PLOT_ONE with or without /LogScale
;                   to zoom in to the desired features
;               e.g.,
;                   IDL> qlds = readcdsfits('s7128r00')
;                   IDL> ghost_plot_all, qlds
;                   IDL> ghost_plot_one, qlds, 1, /logscale
;
; Inputs      : qlds    = Structure variable as read by READCDSFITS.
;               chan    = Detector number (1-4)
;
; Opt. Inputs : None.
;
; Outputs     : None.
;
; Opt. Outputs: GhostData     = Optional output structure of spectrum and
;                               ghost, for use in further processing.  
;                               Contains:
;                   ghostdata.spec          Original Spectrum
;                   ghostdata.blue_spec     left shifted spectrum
;                   ghostdata.red_spec      right shifted spectrum
;                   ghostdata.wave_scale    Wavelength scale used
;                   ghostdata.wave_min      Minimum plotted wavelength
;                   ghostdata.wave_max      Maximum plotted wavelength
;
; Keywords    : NoLabel       = Do not draw labels around the plots -
;                               mainly used by GHOST_PLOT_ALL
;
;               LogScale      = Plot data with a log scale to show fine details
;
;               Angstroms     = Use Angstroms
;
;               nm            = Use nm
;
;               WaveRange     = Array with the wavelength range for plotting
;                               the data:
;                   waverange(0) = minimum plotted wavelength in nm
;                   waverange(1) = maximum plotted wavelength in nm
;                              If /angstroms or /pixels is set, then 
;                              wavelengths are in angstroms or pixels.
;                              The default is to plot over the whole detector.
;
;               pixels        = plot in pixels, not by wavelength
;
;               sample        = plot a theoretical sample spectrum
;
;               quiet         = pass 'quiet' keyword to ghost_amount
;
;               noghost       = do not display ghost information (mainly used
;                               for test purposes)
;
;		cross_cor     = plot a computer guess of ghosts by cross 
;				correlating the shifted and original spectra
;
;		nocolour      = plot data in b&w - useful for b&w printers
;
; Calls       : gis_ghosting, gt_windata, ghost_sample, ghost_amount, gt_hdr,
;                   gt_numexp
;
; Common      : None.
;
; Restrictions: None.
;
; Side effects: None.
;
; Category    : Calibration, GIS, Intensity.
;
; Prev. Hist. : None
;
; Written     : Eddie Breeveld, erb@mssl.ucl.ac.uk  7 March 1997
;
; Modified    : v2,   erb, 21 Mar 97; Minor plotting details changed
;               v3,   erb,  4 Apr 97; removed smoothing
;               v4,   erb, 13 Nov 97; separated gis_ghosting routine
;               v5,   erb, 17 Nov 97; minor data errors trapped
;               v5.1, erb,  2 Feb 98; some labels changed, and tidied up
;               v5.2, erb, 24 Feb 98; add a sample spectrum
;               v5.3, erb,  2 Mar 98; (from Vincenzo Andretta) fixed colour
;                                   table bug;
;               v5.4, erb,  3 Mar 98; improved log output scale
;               v6,   erb,  9 Mar 98; Use GHOST_AMOUNT to get likely ghost
;                                       areas
;               v6.1, erb, 11 Mar 98; Used colour table 12, not 2 (a bug!)
;               v7,   erb, 10 Jul 98; plot ghost amounts in different colours
;               v7.1, erb, 30 Nov 98; include ghost_test funtionality
;               v7.2, erb,  5 Jan 99: include quiet keyword
;               v7.3, erb, 15 Jan 99: Minor correction to gset_id processing
;               v7.4, erb,  1 Mar 99; add nm and noghost switches
;               v7.5, erb,  3 Mar 99; improved noghost function
;               v7.6, erb, 15 Mar 99; Default to angstroms
;               v7.7, erb, 18 May 99; correct a scaling bug with /noghost
;               v7.8, erb, 27 May 99; fixed bug in scaling bug fix!
;               v7.9, erb,  3 Jun 99; added better titles to /noghost output
;		v8,   erb, 19 Jul 99; More scaling corrections (for the bright
;					HeII line); minor plot label changes;
;					added cross_cor keyword
;		v8.1, erb, 22 Jul 99; better /nolabel processing
;		v8.2, erb, 22 Jul 99: allow colour hardcopies
;
; Version     : 8.2
;
;-
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;
pro ghost_plot_one, qlds, chan, ghostdata, $
    nolabel=nolabel, logscale=logscale, nocolor=nocolor, nocolour=nocolour, $
    angstroms=angstroms, waverange=waverange, cross_cor=cross_cor, $
    pixels=pixels, sample=sample, quiet=quiet, nm=nm, noghost=noghost
;
    if (n_params() lt 2) then begin
input_error:
        message, 'ERROR - Usage:', /continue
        chkarg, 'ghost_plot_one'
        return
    endif
;
    if (not keyword_set(nm)) and (not keyword_set(pixels)) then angstroms=1
;
; Get GSET ID for observation
;
    gset_id = qlds.header.gset_id
;
; Get the spectra and combine to a single spectrum, summing over 
; the x, (y), (time) dimensions
;
    spec = gt_windata (qlds, chan-1) > 0.0         ; and force missing to zero
;
    if n_elements(spec) ne 2048 then spec = total (spec, 2)     ; x
    if n_elements(spec) ne 2048 then spec = total (spec, 2)     ; y
    if n_elements(spec) ne 2048 then spec = total (spec, 2)     ; time
;
    if n_elements(spec) ne 2048 then begin
        print, "GHOST_PLOT_ONE - ERROR - GIS data windows not recognised"
        return
    endif
;
    orig_spec = spec
;
; force near zero pixels to minimum away from edges and dropouts
;
    spec = spec > min (spec[where (spec[64:2047-64] gt 0.0) + 64 ])
;
    if keyword_set(logscale) then spec = alog (spec)    ; use log scaling
;
    spec = spec - min(spec)         ; use an offset to improve scaling
;
; Work out maximum, ignoring the brightest line and edge of detector
;
    brightest = max (spec[64:2047-64], position)
    position = position + 64
    cmax = max ([max (spec[64 : (position-28)>64]), $
			max(spec[(position+28)<(2047-64) : 2047-64]) ])
;
; Convert to wavelength scale
;
    schan   = 'GIS' + trim(chan)
    wave_scale = pix2wave (schan, indgen(2048))/10.0    ; in nm
;
    if keyword_set(angstroms) then wave_scale = wave_scale*10.0
    if keyword_set(pixels) then wave_scale = findgen(2048)
;
    wave_min   = wave_scale[0]
    wave_max   = wave_scale[2047]
;
    if keyword_set(waverange) then begin
        if n_elements(waverange) ne 2 then goto, input_error
;
        wave_min = waverange[0]
        wave_max = waverange[1]
;
; Check wavelength scales
;
        case 1 of 
            keyword_set(pixels): $
              cmax = max (spec[wave_min : wave_max])
            keyword_set(angstroms): $
              cmax = max (spec[wave2pix(schan, wave_min) : $
                wave2pix(schan, wave_max)])
            else: cmax = max (spec[wave2pix(schan, wave_min*10.) : $
              wave2pix(schan, wave_max*10.)])
        endcase
    endif
;
; Scale the spectrum using cmax
;
    if cmax le 0.0 then begin
        print, "GHOST_PLOT_ONE - ERROR - no valid data in window"
        return
    endif
    spec = spec / cmax * 10.0
;
; Where are the ghosts?
;
    if gset_id le 0 then begin
        print, "GHOST_PLOT_ONE - ERROR - invalid GSET for observation"
        return
    endif
;
; work out ghosting parameters.  cf GIS_GHOSTING
;
    gis_ghosting, gset_id, chan, ghosts
;
; GIS_GHOSTING returns:
; Outputs     : GHOSTS  = an array containing ghosting information:
;                   0,* = lower ghost pixel address
;                   1,* = upper ghost pixel address
;                   2,* = position in arm (0.0 - 1.0)
;
    wgood0   = where (ghosts[0,*] gt -1)        ; - lower ghost (blue)
    wgood1   = where (ghosts[1,*] gt -1)        ; - upper ghost (red)
;
; Find out the likely ghosting regions.
; Function ghost_amount returns the amounts of ghosting for each pixel
;
    amount = ghost_amount (chan, gset_id, ghosts[2,*], /quiet)
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Set up to plot the main spectrum and 
; Set up the current plot window.  Test for a windowing device, and set
;   the size
;
    if ((!d.flags and 128) eq 128) and (total(!p.multi) eq 0) then $
                                window, !d.window>0<31, xsize=1280, ysize=512
;
; set up a colour table
;
    if keyword_set(nocolour) or keyword_set(nocolor) then begin
	loadct, 0, /silent
	col_red = !d.table_size/2
	col_blue = !d.table_size/2
	col_lred  = !d.table_size/1.1
	col_lblue = !d.table_size/1.1
;
    endif else begin
        r = bytscl([0, 1, 0, 0, 0.6, 1,   1])
        g = bytscl([0, 0, 0, 1, 0.6, 0.6, 1])
        b = bytscl([0, 0, 1, 0, 1,   0.6, 1])
        tvlct, r, g, b
;
    	col_red   = 1
    	col_blue  = 2
    	col_lred  = 5
    	col_lblue = 4
    endelse
;
    scale = '(nm)'
    if keyword_set(angstroms) then scale = '(Angstroms)'
    if keyword_set(pixels) then scale = '(pixels)'
    yscale = ' '
    if keyword_set(logscale) then yscale = '(log scale)'
;
    fileid  = gt_hdr(qlds, 'filename')
    fileid  = strmid (fileid, 0, strpos (fileid,'.'))
;
    tit     = qlds.header.filename + ' ' + qlds.header.obs_prog + ' ' + $
                strmid(qlds.header.date_obs, 0, 10) + ' ' + schan
;
    ytickn  = [  ' ',                 $
                 ' ',                 $
                 ' ',                 $
                 ' ',                 $
                 ' ',                 $
                 ' ',                 $
                 'SPECTRUM!C'+yscale+'!Carb. units', $
                 ' ',                 $
                 ' ',                 $
                 ' ',                 $
                 ' ',                 $
                 'right shifted!Cspectrum', $
                 ' ',                 $
                 'left shifted!Cspectrum',  $
                 ' ',                 $
                 'theoretical!Cquiet Sun!Cspectrum', $
                 ' ' ]
;
    if keyword_set(cross_cor) then begin
	ytickn[10] = '(correlation)'
	ytickn[12] = '(correlation)'
    endif
;
    xtitle  = 'Wavelength ' + scale
    xmargin = [11, 2]
    ymargin = [4, 2]
;
    if keyword_set(nolabel) then begin
        tit = ' '
        ytickn = replicate (' ', 17)
        xtitle = ''
        xmargin = [1, 1]
        ymargin = [2, 2]
    endif
;
; Reserve space for sample spectrum
;
    if keyword_set(sample) then yranmax = 16 else yranmax = 14
;
; Check for 'noghost' keyword
;
    if keyword_set(noghost) then begin
        orig_spec = orig_spec/gt_numexp(qlds)
;
        yrange = [0.0, max ([orig_spec[64 : 1100], orig_spec[1300:2047-64]])]
        if keyword_set(waverange) then yrange = [0, 0] 
;
        plot, wave_scale, orig_spec,        $
            xrange=[wave_min, wave_max],    $
            xstyle=1,                       $
            xtitle = xtitle,                $
            ytitle='(' + qlds.detdesc[chan-1].units + ')/exposure', $
            psym=10,                        $
            yrange = yrange,                $
            title = tit
        return
    endif
;
; Set up axes etc. 
;
    plot, wave_scale, spec,    $
        title=tit,                      $
;
        xrange=[wave_min, wave_max],    $
        xmargin = xmargin,              $
        xstyle=1,                       $
        xtitle = xtitle,                $
;
        yrange=[0,yranmax],             $
        ystyle=1,                       $
        yticks=yranmax,                 $
        ytickname = ytickn,             $
        yticklen = 0.005,               $
        ymargin = ymargin
;
; plot the likely ghosting areas
;
    for i=0, 2046 do begin
        wlb = wave_scale[i]
        wnb = wave_scale[i+1]
;
;  right_shifted
;
        if amount[i, 1] gt 0.0 then begin
            height = 10*amount[i,1]
            polyfill, noclip=0, [wlb, wlb, wnb, wnb], $
                [0, height, height, 0], $
		   color = col_lred
        endif
;
;  left-shifted
;
        if amount[i, 0] gt 0.0 then begin
            height = 10*amount[i,0]
            polyfill, noclip=0, [wlb, wlb, wnb, wnb], $
		[0, height, height, 0], $
		color = col_lblue
        endif

    endfor
;
; replot the r-shifted outline in case it got behind
;
    oplot, wave_scale[indgen(2048)], 10*amount[*,1], $
        color=col_lred, psym=10
;
; replot the x axes, corrupted by the above polyfills
;
    axis, xaxis = 0,                 $
        xrange=[wave_min, wave_max], $
        xmargin = xmargin,           $
        xstyle=1,                    $
        xtickname = replicate(' ', 10)
;
    axis, xaxis = 1,                 $
        xrange=[wave_min, wave_max], $
        xmargin = xmargin,           $
        xstyle=1,                    $
        xtickname = replicate(' ', 10)
;
; Plot the data
;
    oplot, wave_scale, spec
;
; Draw lines to separate regions
;
    plots, [wave_min, wave_max], [10, 10]
    plots, [wave_min, wave_max], [12, 12]
;
; Interpolate all ghosting data to uniform steps 
;
    blue_spec = interpol (spec[wgood0], ghosts[0, wgood0], findgen(2048))
    red_spec  = interpol (spec[wgood1], ghosts[1, wgood1], findgen(2048))
;
; plot the blue-shifted spectrum
;
    oplot, wave_scale, (blue_spec*.2 > 0.0) + 12, color=col_blue, thick=1.5
;
; plot the red-shifted spectrum
;
    oplot, wave_scale, (red_spec*.2 > 0.0) + 10, color=col_red, thick=1.5
;
; Plot the sample spectrum
;
    if keyword_set(sample) then begin
        ghost_plot_sample, schan, sample_data, logscale=logscale, $
          waverange=waverange, pixels=pixels, angstroms=angstroms
;
; Scale the sample spec
;
        sample_spec = sample_data.spec - min (sample_data.spec)
        sample_spec = sample_spec/max(sample_spec)*2.0 + 14.0
        oplot, sample_data.wave, sample_spec
    endif
;
; Fill in the ghostdata structure
;
    ghostdata = {spec:spec, blue_spec:blue_spec, red_spec:red_spec, $
                wave_scale:wave_scale, wave_min:wave_min, wave_max:wave_max}
;
; do a cross-correlation, if keyword_set (cross_cor)
;
    if keyword_set(cross_cor) then begin
        blue_cross = fltarr(2048)
        red_cross  = fltarr(2048)
;
        bin_size=128
        for i=0, 2047-bin_size do begin
            sample_spec = ghostdata.spec(i:i+bin_size)
            sample_blue = ghostdata.blue_spec(i:i+bin_size)
            sample_red  = ghostdata.red_spec(i:i+bin_size)
;
            if total(sample_blue) gt 0.0 then begin
                blue_cross(i+bin_size/2) = $
                       abs(correlate (sample_spec, sample_blue))
            endif
;            
            if total(sample_red) gt 0.0 then begin
                red_cross(i+bin_size/2)  = $
                       abs(correlate (sample_spec, sample_red))
            endif
        endfor
;
; blue(red)_cross now contains the correlation coefficients. Use
;   oplot, ghostdata.wave_scale, blue(red)_cross
; to plot the data directly.
;
; Highlight the coefficients gt 0.4 with an asterisk
;
        big_blue_waves = ghostdata.wave_scale (where ((blue_cross) gt 0.4)>0)
        plots, big_blue_waves, 12, psym = 2, noclip=0
;
        big_red_waves = ghostdata.wave_scale (where ((red_cross) gt 0.4)>0)
        plots, big_red_waves, 10,  psym = 2, noclip=0
;
    endif       ; plot Xcorrs 
;
    return
end
