;+
; Project     : SOHO - CDS
;
; Name        : GHOST_PLOT_SAMPLE
;
; Purpose     : Plots (or returns) a theoretical sample GIS spectrum
;
; Explanation : This routine will plot a sample spectrum covering the GIS
;               detector ranges.  It uses data from a theoretical quiet sun as
;               calculated by Enrico Landi (E.Landi@damtp.cam.ac.uk).
;
;               Second order lines are guessed to be a factor 10 less intense than
;               1st order.
;               The HeII line has been suppressed (as with detector 2).
;
;   Some warnings and information from Enrico:
;
;   1 - the intensities have been calculated using the new version of the 
;   Arcetri Spectral Code (landi and landini 1998 AA submitted) using a quiet 
;   Sun diffrential emission measure.
;
;   2 - Some calcium lines  
;   are calculate assuming level population from the ground level only and 
;   for this reason can be badly overestimated. This is also possibly true for 
;   other ions calculated with this assumption. 
;
;   3 - Fe III lines included in the GIS 4 spectral range need to be taken 
;   with caution since this ions still need some more checking.
;
; Use         : GHOST_PLOT_SAMPLE, DETNO, [SAMPLE_DATA]
;
; Inputs      : DETNO    = Detector number (1-4)
;
;               Reads the save file ghost_plot_sample.save which contains the
;               details of the sample spectrum
;               Data from Enrico Landi, using Chianti
;
; Opt. Inputs : None.
;
; Outputs     : None.
;
; Opt. Outputs: SAMPLE_DATA = Returns the sample spectrum.  The data is not
;                           plotted if this parameter is supplied
;
; Keywords    : ANGSTROMS = use angstroms instead on nanometers
;
;               PIXELS =    use pixels instead of nanometers
;
;               WAVERANGE = 2 element array with the wavelength range to 
;                           be used. In nanometers, unless other keywords set.
;
;               LOGSCALE =  plot using a log scale
;
; Calls       : None.
;
; Common      : None.
;
; Restrictions: Many!  Resulting data to be taken with a pinch of salt.  
;
; Side effects: None.
;
; Category    : Calibration, GIS, Intensity.
;
; Prev. Hist. : None
;
; Written     : 24 Feb 1998, Eddie Breeveld, erb@mssl.ucl.ac.uk
;
; Modified    : v1.1    erb  3 Mar 98   Added LOGSCALE keyword
;
; Version     : v1.1
;
;-
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;
pro ghost_plot_sample, detno, sample_data, logscale=logscale, $
            waverange=waverange, angstroms=angstroms, pixels=pixels, save=save
;
    if ((n_params() gt 2) or (n_params() lt 1)) then begin
        print, 'ERROR - GHOST_PLOT_SAMPLE - Usage:'
        print, '  IDL> ghost_plot_sample, detno, [sample_data]  [,/logscale] $'
        print, '         [waverange=[min, max]] [,/angstroms] [,/pixels]'
        return
    endif
;
; restore the data file
;  contains two arrays: lambda with wavelengths (in nm) and inten with predicted 
;  intensities
;
    restore,'$CDS_GIS_CAL_INT/ghost_plot_sample.save'
;
; 'Plate' scale
;
    scale      = 500.0	            ; points per nm
    smoothsize = 0.010 * scale      ; nm * scale
;
    s_lambda = lambda*scale
;
; set up array with the spectrum and wavelength scale
;
    spec       = fltarr(2000L*scale)         ; cover the range 0-2000 nm
    wave_scale = findgen (2000L*scale) / scale
;
    for i=0,3721 do spec(s_lambda(i)) = spec(s_lambda(i)) + inten(i)
;
; add second order, guess at 1/10th first order intensity
;
    for i=0,3721 do spec(s_lambda(i)*2) = spec(s_lambda(i)*2) + inten(i)/10.0
;
; suppress region around HeII 30.4
;
    HeII_region = where(wave_scale ge 30.1 and wave_scale le 30.75)
    spec(HeII_region) = spec(HeII_region) / 100.0
;
; reduce the size of the array to cover the selected channel
;
    schan      = 'GIS'+trim(detno)
    det_range  = pix2wave (schan, [0, 2047]) / 10.0   ; in nm
    spec       = spec(where(wave_scale ge det_range(0) and $
                            wave_scale le det_range(1)))
    wave_scale = wave_scale(where(wave_scale ge det_range(0) and $
                                  wave_scale le det_range(1)))
;
; normalize and smooth the data with a gaussian of width smoothsize
;
    spec = spec/max(spec) * 1e5
    spec = convol (spec, $
      gauss_put(smoothsize*20, 0, 1e5, smoothsize*10, smoothsize) )
;
; process wavelength scale depending on keywords set
;
    case 1 of
        keyword_set(pixels): begin
;
; Convert wavelengths to pixels
;
            wave_scale = wave2pix(schan, wave_scale*10.0, /nolimit, /real)
            if not keyword_set (waverange) then waverange = [0, 2047]
        end
        keyword_set(angstroms): begin
;
; Convert to Angs
;
            wave_scale = wave_scale*10.0
            if not keyword_set (waverange) then waverange = $
              pix2wave (schan, [0, 2047])
        end
        else: begin
            if not keyword_set (waverange) then waverange = $
              pix2wave (schan, [0, 2047]) / 10.0
        end
    endcase
;
; Check for log scaling
;
    if keyword_set(logscale) then begin
        spec(where(spec lt max(spec)/1e4)) = max(spec)/1e4
        spec = alog10(spec)
    endif
;
; plot it, or pass the spectrum on to the calling routine
;
    if n_params() eq 1 then $
      plot, wave_scale, spec, xrange=waverange, xstyle=1, /ynozero, ystyle=1 $
    else sample_data = {wave:wave_scale, spec:spec}
;
    return
end
