;+
; Project     : SOHO - CDS
;
; Name        : GHOST_RESTORE
;
; Purpose     : Automatically move GIS detector ghosts
;
; Explanation : This procedure moves GIS ghosts back from saved information.  
;		It is intended to be used only by ghost_buster.
;
; Use         : GHOST_RESTORE, qlds, detno, save
;
; Inputs      : QLDS        = Structure variable of GIS data as read by 
;                               READCDSFITS.
;		SAVE        = ghosting parameters to restore
;
; Opt. Inputs : None.
;
; Outputs     : QLDS  	= returned with the saved ghosts moved
;
; Opt. Outputs: None.
;
; Keywords    : QUIET	= run quietly, without informational messages
;
; Calls       : ghost_move
;
; Common      : None.
;
; Restrictions: None, but intended to be called only from GHOST_BUSTER
;
; Side effects: None.
;
; Category    : Calibration, GIS, Intensity.
;
; Prev. Hist. : None.
;
; Written     : Eddie Breeveld, erb@mssl.ucl.ac.uk  9 June 1999
;
; Modified    : 
;
; Version     : 1
;
;-
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
pro ghost_restore, qlds, detno, save, quiet=quiet
;
; first check the save structure
;
    if (save[0].(2) ne 'H') then message, $
		'ABORTING - Save information header corrupted'
    sav_gset  = save[0].(0)
    sav_detno = save[0].(1)
    if not keyword_set(QUIET) then begin
	message, /continue, 'Saved GSET id = ' + trim(sav_gset)
    	message, /continue, 'Saved detector number = ' + trim(sav_detno)
    endif
;
; check the detector number is correct
;
    if detno ne sav_detno then message, $
		'ABORTING - Supplied detno uses is different (' + $
			trim(detno) + ') from the ' + $
			'save information (' + trim(sav_detno) + ')'
;
; check the GSET ID is correct
;
    if qlds.header.gset_id ne sav_gset then message, $
		'ABORTING - Supplied qlds uses a different gset (' + $
			trim(qlds.header.gset_id) + ') from the ' + $
			'save information (' + trim(sav_gset) + ')'
;
; now loop over all the moves in the save structure
;
    for movei = 1, n_elements(save)-1 do begin	    ; element 0 is the header
;
	pix_left = save[movei].pix_left
	pix_right = save[movei].pix_right
	direction = save[movei].direction
;
	if not keyword_set(QUIET) then $
		message, /continue,  'Restore saved ghost ' + trim(movei)
;
	ghost_move, qlds, pix_left, pix_right, direction, sav_detno, /pixels
;
    endfor
;
    return
end
