;+
; Project     : SOHO - CDS
;
; Name        : GHOST_TEST
;
; Purpose     : Add correlation tests to GIS ghost plots
;
; Explanation : This function takes GIS data read with READCDSFITS and
;               tests for ghosts.  It uses known detector parmeters and 
;               computes cross-correlation curves.
;
;               This procedure is intended to be called from GHOST_BUSTER, q.v.
;
; Use         : GHOST_TEST, QLDS, DETNO [,WAVERANGE=WAVERANGE] [,/ANGSROMS] $
;                           [,/LOGSCALE] [,/PIXELS] [,GHOSTDATA], [/SAMPLE]
;
;                   IDL> qlds = readcdsfits('s7128r00')
;                   IDL> ghost_test, qlds, 1
;
; Inputs      : qlds =      Structure variable as read by READCDSFITS.
;
;               DETNO =     Detector number (1-4)
;
; Opt. Inputs : None.
;
; Outputs     : None.
;
; Opt. Outputs: GHOSTDATA = ghosting parameters.  See GHOST_PLOT_ONE
;
; Keywords    : ANGSTROMS = use angstroms instead on nanometers
;
;               PIXELS =    use pixels instead of nanometers
;
;               WAVERANGE = 2 element array with the wavelength range to 
;                           be used. In nanometers, unless other keywords set.
;
;               SAMPLE =    Add a sample theoretical spectrum
;
;               LOGSCALE =  Plot using log scaling
;
; Calls       : ghost_plot_one, correlate
;
; Common      : None.
;
; Restrictions: None.
;
; Side effects: None.
;
; Category    : Calibration, GIS, Intensity.
;
; Prev. Hist. : None.
;
; Written     : 24 Feb 1998, Eddie Breeveld, erb@mssl.ucl.ac.uk
;
; Modified    : v1.1, erb, 3 March 1998, Added LOGSCALE keyword and improved
;                   window access
;               v1.2, erb, 6 April 1998, Fixed window>31 selection bug
;
; Version     : v1.2
;
;-
;
pro ghost_test, qlds, detno, ghostdata, sample=sample, $
            waverange=waverange, angstroms=angstroms, pixels=pixels, $
            logscale=logscale
;
    if (n_params() lt 2) then begin
        print, 'ERROR - GHOST_TEST - Usage:'
        chkarg, 'ghost_test'
        return
    endif
;
; Set up the current plot window.  Test for a windowing device, and set
;   the size
;
    if ((!d.flags and 128) eq 128) then $
                                window, !d.window>0<31, xsize=1280, ysize=512
;
; Do the initial plotting and calculations (returned in ghostdata)
;
    ghost_plot_one, qlds, detno, ghostdata, waverange=waverange, $
        logscale=logscale, angstroms=angstroms,  pixels=pixels, sample=sample
;
; Do a cross-correlation
;
    blue_cross = fltarr(2048)
    red_cross  = fltarr(2048)
;
    bin_size=128
    for i=0, 2047-bin_size do begin
        sample_spec = ghostdata.spec(i:i+bin_size)
        sample_blue = ghostdata.blue_spec(i:i+bin_size)
        sample_red  = ghostdata.red_spec(i:i+bin_size)
;
        if total(sample_blue) gt 0.0 then begin
            blue_cross(i+bin_size/2) = $
                                    abs(correlate (sample_spec, sample_blue))
        endif
;            
        if total(sample_red) gt 0.0 then begin
            red_cross(i+bin_size/2)  = $
                                     abs(correlate (sample_spec, sample_red))
        endif
    endfor
;
; blue(red)_cross now contains the correlation coefficients. Use
;   oplot, ghostdata.wave_scale, blue(red)_cross
; to plot the data directly.
;
; Highlight the coefficients gt 0.4 with an asterisk
;
    big_blue_waves = ghostdata.wave_scale (where ((blue_cross) gt 0.4)>0)
    plots, big_blue_waves, 12, psym = 2, noclip=0
;
    big_red_waves = ghostdata.wave_scale (where ((red_cross) gt 0.4)>0)
    plots, big_red_waves, 10,  psym = 2, noclip=0
;
    return
end
