;+
; Project     : SOHO - CDS
;
; Name        : GIS_CALIB
;
; Purpose     : Applies calibration factors to GIS spectra and records errors
;               into qlds.
;
; Explanation : This procedure corrects GIS data which has been read
;               with READCDSFITS and already smoothed (using gis_smooth)
;               and corrected for ghosts using ghost_buster. 
;
;               Corrections performed include:
;                   FIFO dead time
;                   Simple correction for Quiz-show dead time 
;                   Analogue dead time               
;                   Short term gain depression check
;                   Long term gain depression
;                   Long slit (4, 5 and 6) optical correction
;
;  ERRORS :    
;              The dominant source of error in the gis data are those associated 
;              with the photon statistics and the Long term gain depression.
;              These errors are of the order of a few percent, and are combined 
;              in quadrature and recorded directly into the quick look data structure
;              for later use with gis_fit. This routine takes the combined errors
;              and uses them as weights in the fitting process.
;              The resultant error in the amplitude and width, are also combined
;              in quadrature to determine the final error in the fitted intensities,
;              which are recorded as an addition tag in the returned structure.
;
;               sigma_I = sqrt((sigma_amplitude/amplitude)^2+(sigma_width/width)^2)
;
;              These combined errors can be extracted from the qlds using the 
;              gis_error, if you fit the lines yourself. These errors should  
;              be used as weights in the fitting process, the errors from which
;              should be combined in a similar fashion to determine the errors in
;              the intensities.
;
;
; Use         : GIS_CALIB, DATA [,ERRMSG=ERRMSG] [,/SETMISSING] [,/QUIET] $
;                                   [,/STERADIAN_M2] [,/ARCSEC2_CM2]
;
; Inputs      : DATA =      Structure variable as read by READCDSFITS and
;                           ghost corrected with GHOST_BUSTER.
;                           
; Opt. Inputs : None.
;
; Outputs     : DATA =      The calibrated data will be returned as a 
;                           structure with the same format as the input.
;                           The units of the calibrated data will be
;                           photon-events/pixel/sec.
;
; Opt. Outputs: None.
;
; Keywords    : ERRMSG =    If defined and passed, then any error messages will
;                           be returned to the user in this parameter rather 
;                           than depending on the MESSAGE routine in IDL.  If 
;                           no errors are encountered, then a null string is 
;                           returned.  In order to use this feature, ERRMSG 
;                           must be defined first, e.g.
;
;                                  ERRMSG = ''
;                                  CALIB = GIS_CALIB (DATA, ERRMSG=ERRMSG)
;                                  IF ERRMSG NE '' THEN ...
;
;               SETMISSING = if set, then uncalibrated data will be set to 
;                           missing.   If not set, then the function aborts 
;                           with an error message if there is any 
;                           uncalibratable data
;
;               QUIET =     Reduces the information printed to screen, but 
;                           errors and warnings are still printed.
;
;               HeII =      Allow the HeII 30.4 line through.  By default the
;                           line is marked as missing in all the spectra
;                           because it can't be calibrated.
;                           NOTE: if this switch is used, do NOT assume that
;                           the region around 30.4 nm can be calibrated.  This
;                           area of the spectrum on detector 2 can ONLY be used
;                           for morphology.
;
;               STERADIAN_M2 =  Use best known detector efficiencies to convert
;                               the data into:
;                               photons/second/pixel/steradian/(meter^2)
;
;               ARCSEC2_CM2 =   Use best known detector efficiencies to convert
;                               the data into:
;                               photons/second/pixel/(cm^2)/(meter^2)
;
; Calls       : gt_winsize, gis_calib_extract, gis_calib_fifo, gis_calib_print,
;               gis_calib_quiz, gis_calib_analogue, gis_calib_stgd, 
;               gis_calib_ff_ltgd, gis_calib_heii, st_windata,
;               gis_calib_longslit
;
; Include file: gis_stage.inc        Units for all gis processing stages
;
; Common      : None.
;
; Restrictions: Data must be ghost corrected before invoking this procedure.
;               GIS_CALIB only works with full GIS data windows (whole
;               detectors)
;
;               TO DO LIST:
;                   edge effects
;                   off-axis calibration (vignetting)
;                   stray light
;
;                   any other corrections?
;
; Side effects: None?
;
; Category    : Calibration, GIS, Intensity.
;
; Prev. Hist. : None.
;
; Written     : Eddie Breeveld, erb@mssl.ucl.ac.uk  30 January 1997
;
; Modified    : v2.0, erb 31 Jan 1997: Added smoothing
;               v3.0, erb  4 Feb 1997: Used $CDS_GIS_CAL_INT directory for files
;               v4.0, erb  4 Apr 1997: Remove smoothing and force use of ghost
;                       corrected data.
;               v5.0, erb 20 Jan 1998: Changed to a routine instead of a
;                       function.  Added  GIS_CALIB_AREA routine from 
;                       Giuilio del-Zanna
;               v5.1, erb 28 Jan 1998: Optimised array index ordering;
;                       which also solved an extra returned dimension problem.
;               v5.2, erb 29 Jan 1998: Not all detectors have to be 
;                       ghost-corrected now.
;               v5.3, erb  1 Feb 1998: Separated out GIS_CALIB_AREA (to be
;                       added later though!)
;               v5.4, erb  3 Feb 1998: Split into separate routines
;               v5.5, erb 27 Feb 1998: Corrected header description
;               v6,   erb 22 Dec 1998: Added HeII switch for morphology only
;               v6.1, erb 08 Jan 1999: more info on number of missing pixels
;               v7    erb 25 Jan 1999: Add /steradian_m2 and /arcsec2_cm2 
;					switches to further calibrate the data
;               v7.1  erb 01 Feb 1999: Correct the header information
;               v8.0  erb 25 May 1999: make corrections for long slits,
;                                      use new stgd and ltgd files
;		v8.1  erb 19 Jul 1999: Use calibration databases in 
;			gis_calib_area; use full wavelength
;			calibration
;               v9.0  caf Aug 2003 : Fixed various bugs.
;                                    Now defaults to photons / sec / arcsec2 / cm2
;                                    Added error array to output qlds - extract with gis_error                                
;                                    Now able to calibrate observations obtained with a subset of the
;                                    GIS detectors
;               v10   caf 10 Oct 2003: added the option to return the fifo, and LTGD correction factors
;
;               v11   caf 7 Oct 2004: Passed errors through into qlds, combined in quadrature, 
;                                     for use in gis_fit, and gt_error and added description of errors. 
;
; Version     : 11.0
;
;-
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;
PRO GIS_CALIB, QLDS, ERRMSG=ERRMSG, SETMISSING=SETMISSING, QUIET=QUIET, $
                  HeII=HeII, STERADIAN_M2=STERADIAN_M2, COUNTS=COUNTS,fifo=fifo,ltgd=ltgd
;
;
; Set up some strings for the different stages of GIS processing
;
@gis_stage.inc
;
; See if we can continue...
;
; Check the number of parameters.
;
    message = ''
    if n_params() lt 1 then begin
        MESSAGE = 'ERROR - incorrect calling arguments'
        chkarg, 'GIS_CALIB'
        goto, handle_error
    endif
;
    if not keyword_set (QUIET) then print, 'GIS Calibration...'
;
; Make sure that it is GIS data we're dealing with.
;
    if QLDS.HEADER.DETECTOR ne 'GIS' then begin
        MESSAGE = 'Cannot use this routine with VDS data'
        goto, HANDLE_ERROR
    endif
;
; Check if the full GIS is used
;
    winsize = gt_winsize (qlds)
     waveband=qlds.detdesc.waveband

     for i=0,n_elements(winsize)-1 do begin
        if (winsize(i) ne 2048) then begin
            MESSAGE = 'Only partial GIS windows, cannot be calibrated - Working on a Fix - Aug 2003 caf@mssl.ucl.ac.uk'
            goto, HANDLE_ERROR
        endif
    endfor
;
; Make sure we are using the correct wavelength calibration
;
    qlds=gis_cqlds(qlds)
    success = restore_wavecal(qlds)
    if not (success) then message, 'Restore wavelength calibration failed'
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; extract the relevant data: returns data, info, missing_array
;
    message = ''
    gis_calib_extract, qlds, data, info, missing_array, message, $
                        quiet, setmissing
    if message ne '' then goto, handle_error
;
    if not keyword_set (QUIET) then $
		gis_calib_print, data, qlds.detdesc(0).units,waveband
;
; FIFO dead time
;
    gis_calib_fifo, data, info, quiet,fifo=fifo

    ;
; Quiz show dead time
;
    gis_calib_quiz, data, info, message, missing_array, setmissing, quiet

    if message ne '' then goto, handle_error

;
; 2.6 microsecond analogue dead time correction
;
    gis_calib_analogue, data, info, message, missing_array, $
                          setmissing, quiet
    if message ne '' then goto, handle_error

;
; Calibrate for short term gain depression
; 
    gis_calib_stgd, data,  message, missing_array, setmissing, quiet, $
                    qlds.header.slit_num
    if message ne '' then goto, handle_error

;
; set the units, currently just in photons!
;
    units = gis_stage.photons
    ;
    ; check if using a long slit
    ;
    if qlds.header.slit_num ge 4 then $
	gis_calib_longslit, data, info, qlds.header.slit_num, quiet
    ;
    ; Further calibrations asked for?
    ;
    if not keyword_set(counts) then begin
      if not keyword_set(steradian_m2) then $
            gis_calib_area, data, qlds.header.slit_num, message, units, $
                    qlds.mir_posdata, qlds.header.date_obs,waveband,/arcsec2_cm2 $
    else gis_calib_area, data, qlds.header.slit_num, message, units, $
                    qlds.mir_posdata, qlds.header.date_obs,waveband
endif
      if message ne '' then goto, handle_error

;
; Calibrate for Flat field and long-term gain depression
;
    gis_calib_ff_ltgd, qlds, data, waveband,info, message, quiet, ltgd=ltgd

    if message ne '' then goto, handle_error
;    
; Remove HeII 30.4 line completely
;


   if not keyword_set (HeII) then gis_calib_HeII, missing_array, quiet

   ;
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;
; All calibrations done!
;
    if not keyword_set (QUIET) then print, '    Return data to structure...'
;
; Restore calibrated missing data to 'missing'
;
    list_missing = where (missing_array lt 0, count)
    if count gt 0 then data(list_missing) = -100.0
;
    qlds.DETDESC(*).MISSING = -100.0    ; set missing flag to -100
;
    if not keyword_set (QUIET) then gis_calib_print, data, units,waveband
;
; copy data into the structure
;

winsize=gt_winsize(qlds)
det=waveband-1
for det=0, 2*n_elements(det)-1 do st_windata, qlds, det, data(*, *, *, *, det)
;
; Tell the structure that we've finished
;
    qlds.DETDESC(*).UNITS = units
;
    if not keyword_set (QUIET) then print, 'Calibration complete'
    goto, FINISH
;
; Error handling point.
;
HANDLE_ERROR:
    if N_ELEMENTS(ERRMSG) ne 0 then ERRMSG = 'GIS_CALIB: ' + MESSAGE $
        else MESSAGE, MESSAGE, /continue
;
; Exit
;
FINISH:
    return
end
