;+
; Project     : SOHO - CDS
;
; Name        : GIS_CALIB_ANALOGUE
;
; Purpose     : Applies analogue calibration factors to GIS spectra.
;
; Explanation : This procedure takes GIS images read with READCDSFITS and
;               applies the analogue electronics calibration factors.
;               See GIS_CALIB. Reads $CDS_GIS_CAL_INT/deadtime.dat to get
;               the factors.
;
; Use         : GIS_CALIB_ANALOGUE, data, info, message, missing_array, $
;                         setmissing, quiet
;
; Inputs      : DATA =      Data as supplied by GIS_CALIB
;               INFO =      Information about the data
;               MISSING_ARRAY = Array marking missing data
;               SETMISSING = Flag to set the data missing if unable to calibrate
;               QUIET =     Flag to run quietly
;
; Opt. Inputs : None.
;
; Outputs     : DATA =      The calibrated data will be returned as a
;                           structure with the same format as the input.
;               MESSAGE =   Returns error messages (if any)
;               MISSING_ARRAY = Array marking missing and uncalibratable data
;
; Opt. Outputs: None.
;
; Keywords    : None.
;
; Calls       : None.
;
; Common      : None.
;
; Restrictions: None.
;
; Side effects: None?
;
; Category    : Calibration, GIS, Intensity.
;
; Prev. Hist. : Separated from GIS_CALIB
;
; Written     : Eddie Breeveld, erb@mssl.ucl.ac.uk  3 February 1998
;
; Modified    : v.2, erb, 17 Dec 98, Used spline for better interpolation
;                                   and ignored low count-rate corrections
;                                   because they have a high statistical error
;
;               v.3, caf     Aug 03, fixed bug so now works with partial GIS data, $
;                                    and new error array.
;
; Version     : V.3
;
;-
;
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
pro gis_calib_analogue, data, info, message, missing_array, $
                          setmissing, quiet
;
; 2.6 microsecond analogue dead time correction
; Applies to individual detectors
;
; Read in the deadtime data - extending deadtime
;
    if not keyword_set (QUIET) then print, '    Analogue deadtime...'
;
    deadtime_event = fltarr(10) ; First ten values is enough
    deadtime_uld   = fltarr(10)
    deadtime_valid = fltarr(10)
    deadtime_max = 0.0
;
    openr, unit, concat_dir('$CDS_GIS_CAL_INT','deadtime.dat'), /get_lun
    readf, unit, deadtime_event     ; event counter
    readf, unit, deadtime_uld       ; uld counter
    readf, unit, deadtime_valid     ; valid counter
;
    readf, unit, deadtime_max       ; max count-rate per detector
    free_lun, unit
; caf testing
    deadtime_max=deadtime_max*10.
    ;
; Convert to Hz
;
    deadtime_event = deadtime_event * 1000.0d0
    deadtime_uld   = deadtime_uld   * 1000.0d0
    deadtime_valid = deadtime_valid * 1000.0d0
;
; deadtime_input is the actual count rate (before detector dead-time)
; and deadtime_output observed count rate (including dead-time)
;
    deadtime_input = deadtime_event - deadtime_uld
    deadtime_output = deadtime_valid
;
; correct for very low event rates (measurement errors in the data)
;
    valid = where(deadtime_input gt 10000.0d0)
    deadtime_input  = [0.0, 1.0, deadtime_input[valid]]
    deadtime_output = [0.0, 1.0, deadtime_output[valid]]
;
; Analogue deadtime is corrected on a detector by detector basis;
; loop over the detectors to get the detector count rates
; for each exposure
;
winsize=n_elements(data(0,0,0,0,*))/2
for det=0,winsize-1. do begin
        for tpos = 0, info.sdel_time-1 do begin
            for xpos = 0, info.ssolar_x-1 do begin
                for ypos = 0, info.ssolar_y-1 do begin
                    rate = total(data(*, xpos, ypos, tpos, det))
;
; Check if not too high
;
                    if (rate gt deadtime_max) then begin
                        if keyword_set (setmissing) then begin
                            missing_array(*, xpos, ypos, tpos, det) = -1
                        endif else begin
                            MESSAGE = 'ABORTED - count rate too high for' + $
                              ' analogue deadtime correction'
                            return
                        endelse
                    endif
;
; Work out the real rate
;
                    if (rate gt 0.0) then begin
                        ratio = spline (deadtime_output, $
                          deadtime_input, rate)/rate
                        ratio = ratio(0)    ; this is an array, get 1st element
                    endif else  ratio = 1.0
;
; Force ratio >= 1.0, may be lower because of deadtime measurement errors
;
                    ratio = ratio > 1.0
;
; Correct the spectrum at this point for this detector
;
                    data(*, xpos, ypos, tpos, det) = $
                      data(*, xpos, ypos, tpos, det) * ratio
;
                endfor      ; ypos
            endfor          ; xpos
        endfor              ; tpos
    endfor                  ; det
;
    return
end
;
;
