;+
; Project     :	SOHO - CDS
;
; Name        :	GIS_CALIB_AREA
;
; Purpose     :	Applies effective area calibration to GIS data
;
; Explanation :	This function takes GIS data from the GIS_CALIB routine
;               and translates to  (per steradian per m^2), or optionally
;               to (per arcsec^2 per cm^2)
;              
;
; Use         :	GIS_CALIB_AREA, data, slit, message, units, mir_posdata, $
;			date_obs, [,/arcsec2_cm2]
;
; Inputs      :	DATA	= Array variable as supplied by GIS_CALIB
;
;               SLIT    = The slit used for the observation
;
;		MESSAGE = Error messages, if any
;
;		MIR_POSDATA = mirror positions for each exposure
;
;		DATE_OBS = date of observation
;
; Opt. Inputs :	None.
;
; Outputs     :	DATA    = The calibrated data will be returned as an
;			  array with the same format as the input.
;			  The units of the calibrated data will be
;		  photon-events/pixel/sec/(unit area)/(unit square angle)
;
;		UNITS	= The units of the returned data, as defined by
;			  gis_stage.inc
;
; Opt. Outputs:	None.
;
; Keywords    :	ARCSEC2_CM2 =  convert output to per arcsec^2 per cm^2
;                               default per steradian per m^2
;
; Calls       :	many.
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Category    :	Calibration, GIS, Intensity.
;
; Prev. Hist. :	
;
; Written     :	Eddie Breeveld, erb@mssl.ucl.ac.uk  27 January 1999
;
; Modified    :	v2: 19 Jun 1999, erb, use calibration databases
;
;               v3:    Aug 2003, caf fixed problem which caused crash for sit and stare observations !
;                                    made changes so error array included in qlds, and fixed bug
;                                    so now works with partial GIS data.
; Version     : 3
;
;-
;
PRO GIS_CALIB_AREA, data, slit, message, units, mir_posdata, date_obs,waveband,$ $
	arcsec2_cm2=arcsec2_cm2
;
; processing stages
;
@gis_stage.inc
;
; table of slit sizes
;
    case slit of
;
    1: sw_xy = [ 2.032,   2.032] 
    2: sw_xy = [ 4.064,   4.064]
    3: sw_xy = [ 8.128,  50.80]
    4: sw_xy = [ 2.032, 240.24]
    5: sw_xy = [ 4.064, 240.24]
    6: sw_xy = [91.44,  240.24]
;
    else: begin
 	    MESSAGE = 'ERROR - Unknown slit ' + trim(slit)
	    return
      	end
    endcase
;
    slitw = sw_xy(0)
    slith = sw_xy(1)
;
; define spectrograph details
;
    focal = 257.8     ; focal length (cm)
;
; get areas from the calibration databases (same for all GIS detectors)
;
    get_eff_area, date_obs, 'GIS1', eff_area, errmsg=message
    if strlen(message) gt 1 then return
;
; loop over all detectors
;
    for stp=0,n_elements(waveband)-1 do begin
    ichannel=waveband(stp)
	adetno = 'GIS' + trim(ichannel) 
;
; first calculate the wavelength ranges
;
	lambda = pix2wave (adetno, findgen(2048))
;
; get sensitivities from the databases
;
	get_efficiency, date_obs, adetno, efficiency, wavelength=lambda
; ;                       arcsec2 counts /phot ;;; 1./ = phots /arcsec2 / sec  
; instrumental response = slit_h*slit_w*eff_area*efficiency/(focal_length^2).
; eff_area is mirror position sensitive, and efficiency is wavelength
; sensitive
;
; loop over all exposures
;
	for ix = 0, n_elements(data[0, *, 0, 0, stp])-1 do begin
	    for iy = 0, n_elements(data[0, 0, *, 0, stp])-1 do begin
		for it = 0, n_elements(data[0, 0, 0, *, stp])-1 do begin
;
if (n_elements(data[0, *, 0, 0, stp]) gt 1. or n_elements(data[0, 0, *, 0, stp]) gt 1.) then inst_resp = efficiency * eff_area[mir_posdata[ix, iy, it]] else inst_resp = efficiency * eff_area[mir_posdata[it]]
		    inst_resp = $
		(slitw*0.0000125)*(slith*0.0000125)*inst_resp/(focal^2)
;
; intensity in photons /s /ster /m^2
;
		    ; next line works on error array
        	    data[*, ix, iy, it, stp+n_elements(waveband)] = $
			data[*, ix, iy, it, stp+n_elements(waveband)] / inst_resp

		    ;
        	    data[*, ix, iy, it, stp] = $
			data[*, ix, iy, it, stp] / inst_resp
;
		endfor	; loop over 'it'
	    endfor	; loop over iy
	endfor		; loop over ix
    endfor		; loop over ichannel
;
    units = gis_stage.steradian_m2
;
; intensity in photons /s /arcsec^2 /cm^2 ?
;
    if keyword_set(arcsec2_cm2) then begin
     ;  (!PI/(180.*60.^2))^2
        data = data * ((!PI/(180.0*60.0*60.0))^2) $     ; per arcsec^2
    	              / (100.0^2)                       ; per cm^2
;
    	units = gis_stage.arcsec2_cm2
    endif
;
    RETURN
END
