;+
; Project     : SOHO - CDS
;
; Name        : GIS_CALIB_EXTRACT
;
; Purpose     : Extracts GIS data for use with GIS_CALIB
;
; Explanation : This procedure takes a GIS qlds and prepares data arrays
;               for use in GIS_CALIB, q.v.  
;
;
; Use         : GIS_CALIB_EXTRACT, qlds, data, info, missing_array, message, $
;                        quiet, setmissing
;
; Inputs      : QLDS =      Structure variable as read by READCDSFITS and
;                           ghost corrected with GHOST_BUSTER.
;               QUIET =     Flag to run quietly
;               SETMISSING = Flag to mark uncalibratable data as missing
;                           
; Opt. Inputs : None.
;
; Outputs     : DATA =      The calibrated data will be returned as a 
;                           structure with the same format as the input.
;                           The units of the calibrated data will be
;                           photon-events/pixel/sec.
;               INFO =      Information about the size of the dataset
;               MISSING_ARRAY = array marking missing data and uncalibratable 
;                           data
;               MESSAGE =   Returns error messages (if any)
;
; Opt. Outputs: None.
;
; Keywords    : None.
;
; Calls       : gt_dimension, gt_exptime, gt_windata, anytim2tai,
;               gt_winsize, gt_start, gt_windesc, st_windata, interpol, 
;               find_file, trim
;
; Common      : None.
;
; Restrictions: None.
;
; Side effects: None?
;
; Category    : Calibration, GIS, Intensity.
;
; Prev. Hist. : Separated from GIS_CALIB
;
; Written     : Eddie Breeveld, erb@mssl.ucl.ac.uk  3 February 1998
;
; Modified    : v2, erb, 08 Jan 1999, Pixel 0 used for ghost_buster_free
;                       processing: thus always set to missing
;               v3, erb, 27 Jan 1999, check for STERADIAN or ARCSEC2
;                       calibrated data.  More info with gis_calib_print.
;               v4, erb, 15 Feb 1999, fixed missing data bug
;               v5, erb, 25 May 1999, reduced amout of printing
;               v6, caf,    Aug 2003, changed so works with new error array. 
;                                     fixed bug so works with partial GIS data.
;                                    
; Version     : 6
;
;-
;
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
pro gis_calib_extract, qlds, data, info, missing_array, message, $
                        quiet, setmissing
;
; set up GIS stages strings
;
@gis_stage.inc
;
; extract the relevant data and
; save missing pixels - these will be set to missing again at the end
;
    if not keyword_set (QUIET) then print,'    Extracting data...'
;
; Check if calibrated already
;
    ;carl next 2 lines
    winsize=gt_winsize(qlds)
    for det = 0, n_elements(winsize)-1 do begin
        description = gt_windesc (qlds, det)
;
;  Case statement here for other (future) gis calibration stages
;
        case description.units of
            gis_stage.raw:              ; allow further processing
            gis_stage.smoothed:         ; allow further processing
            gis_stage.ghost_correct:    ; allow further processing
            gis_stage.photons: begin
                MESSAGE = 'ERROR - Some data already calibrated'
                return
            end
            gis_stage.steradian_m2: begin
                MESSAGE = 'ERROR - Some data already calibrated'
                return
            end
            gis_stage.arcsec2_cm2: begin
                MESSAGE = 'ERROR - Some data already calibrated'
                return
            end
            else: begin
                MESSAGE = 'ERROR - Unknown data units for detector ' + $
                        trim(det+1) + ': ' + description.units
                return
            end
        endcase
    endfor
;
    info = gt_dimension (qlds)
    exposure = gt_exptime (qlds)
    info.sdel_time = info.sdel_time > 1	; force at least one exposure!
;
    ndetno=n_elements(winsize)
    data = fltarr(2048, info.ssolar_x, info.ssolar_y, $
                                    info.sdel_time, ndetno)
    for det=0, ndetno-1 do data(*, *, *, *, det) = gt_windata (qlds, det)
;
; check amount of missing data, build up a new missing array
;
    missing_array = fix (data) * 0            ; empty missing array
    missing_list  = [where (data eq description(0).missing, count)]
    not_missing   = [where (data ne description(0).missing)]
;
    if count gt 0 then begin
        missing_array(missing_list) = -1    ; set missing data to -1 in here
;
; set missing data to average of all the non-missing data
; This data will be reset to missing again at the end.
;
        average = total(data(not_missing)) / n_elements(not_missing)
;
        data(missing_list) = average
    endif
;
; Check if ghosts moved
;
    for det=0, n_elements(winsize)-1  do begin
        description = gt_windesc (qlds, det)
;
        if description.units ne gis_stage.ghost_correct then begin
            if not keyword_set (SETMISSING) then begin
                MESSAGE = 'All data must be ghost corrected first, ' + $
                  'or use /SETMISSING to set non-corrected data to missing'
                return
            endif         ; not keyword_set(missing)
;
; /setmissing set, and this detector data not ghost corrected, thus mark this 
; data missing
;
            missing_array(*, *, *, *, det) = -1
        endif
    endfor         ; loop over detectors
;
; set pixel zero to missing - used by ghost_buster_free processing to plonk all
;  data otherwise removed from ghosted areas.  This allows further calibrations
;  to work on whole detector count rates.
;
    missing_array(0, *, *, *, *) = -1
;
; Final check to see if any data left!
;
    if max(missing_array) lt 0 then begin
        MESSAGE = 'No valid ghost-corrected data'
        return
    end
;
; Convert to count rate
;
    data = data / exposure
;
    return
end
