;+
; Project     : SOHO - CDS
;
; Name        : GIS_CALIB_FF_LTGD
;
; Purpose     : Corrects GIS data for flat field and long term gain depression
;
; Explanation : This procedure is called as part of GIS_CALIB, q.v.
;               Reads the calibration file $CDS_GIS_CAL_INT/ff_*.dat nearest
;               in time to the study start time
;
; Use         : GIS_CALIB_FF_LTGD, qlds, data, info, message, quiet
;
; Inputs      : QLDS =      Structure variable as read by READCDSFITS and
;                             ghost corrected with GHOST_BUSTER.
;               DATA =      GIS data as supplied by GIS_CALIB
;               INFO =      QLDS information as supplied by GIS_CALIB
;               QUIET =     Flag for running quietly
;
; Opt. Inputs : None
;
; Outputs     : DATA =      The calibrated data will be returned as a
;                           structure with the same format as the input.
;               MESSAGE =   Error message (if any) returned
;
; Opt. Outputs: None.
;
; Keywords    : None
;
; Calls       : find_file, anytim2tai, gt_start
;
; Common      : None.
;
; Restrictions: None.
;
; Side effects: None.
;
; Category    : Calibration, GIS, Intensity.
;
; Prev. Hist. : Separated from GIS_CALIB
;
; Written     : Eddie Breeveld, erb@mssl.ucl.ac.uk  3 February 1998
;
; Modified    : v.2, 10 May 1999, erb: Use XDR format for the binary file
;               v.3,    Aug 2003, caf: Added Flat field files, and fixed
;                       bug so now works with partial GIS data.
;               v.4,  1 Oct 2003, caf: fixed the way that the error in LTGD is
;                       passed, and statistical error calculated from
;                       non-normalised data.
;               v.5, 10 Oct 2003 added the option to return the ltgd correction
;                       factor
;
;               v.6, 7 Oct 2004  Entered revised LTGD errors in and passed to qlds.
;
; Version     : 6
;
;-
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
FUNCTION GIS_CALIB_NEAREST, INPUT, ARRAY
;
; returns the index of the array element that is closest to the input
;
    subtract = array - input
    absolute = abs (subtract)
    sort_index = sort (absolute)
;
    return, sort_index(0)
end
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;
pro gis_calib_ff_ltgd, qlds, data, waveband,info, message, quiet,ltgd=ltgd
;
; Calibrate for Flat field and long-term gain depression
;
    if not keyword_set (QUIET) then print, $
            '    Flat field and long term gain depression correction...'
    if not keyword_set (QUIET) then print, $
           'This is the new version 10-2004 based on the GIMCPS1 observations'
;
    ; Get time of first exposure
;
    obstime = anytim2tai (gt_start (qlds))
;
; Get time of flat field files
;
    ff_files = find_file (concat_dir('$CDS_GIS_CAL_INT','ff_*.dat'), count=count)
    if count eq 0 then begin
        MESSAGE = 'ABORTED - no GIS flat field files found'
        return
    endif
;
    len = strlen (ff_files(0))
    ff_times = anytim2tai (strmid (ff_files, len-15, 11))
;
; Find nearest time to obs time
;
    nearest_file = ff_files (gis_calib_nearest (obstime, ff_times))
;
    if not keyword_set (QUIET) then print, $
            '        Flat field file chosen: ', nearest_file
;
; Open the nearest file, and read the corrections
;
    ff_mult   = fltarr (2048, 4)
    ff_offset = fltarr (2048, 4)
;
    openr, unit, nearest_file, /get_lun, /xdr
    readu, unit, ff_mult
    readu, unit, ff_offset
    free_lun, unit
  ltgd=ff_mult
    ;
; Apply corrections
;
    ndets= n_elements(waveband)
    ddet=indgen(4)+1
    for det=0, n_elements(waveband)-1 do begin
      for tpos = 0, info.sdel_time-1 do begin
	for ypos = 0, info.ssolar_y-1 do begin
	  for xpos = 0, info.ssolar_x-1 do begin
	    dwa=where(ddet eq waveband(det) )
	    dw=where(waveband(det) eq ddet)
	    ;
	    ; Correction is of the form (ax + b) with separate coefficients for
	    ; each pixel in each detector,
	    ; where a is the multiplier and b the offset,
	    ; and x is the observed data in counts per second
	    ;
	    ; For this to work best, fixed patterning should be removed first
	    ;
	    ;next line adds change in intensity to error array ...
	    ;
	    expos=gt_exptime(qlds)
	    con=[expos,expos,expos,expos]
	    n_data=n_elements(data(*, xpos, ypos, tpos, ndets+det))
	    ; errors in ltgd uncertainty revised down to 2-5%
	    ;
	    alpha=120.
            ;
	    ltce=[0.0198,0.0513,0.0499,0.0377]
	    ltgd_er2= (((ltce(det)*(ff_mult(*,dwa)-1.)))/ff_mult(*,dwa))^2. ;(replicate(ltce(det),n_data))^2.
            phot=alpha*con(det)*data(*, xpos, ypos, tpos, det)*(smooth(ff_mult(*, dwa),6))
	    phots_er2=(sqrt(phot)/phot)^2.
	    data(*, xpos, ypos, tpos, ndets+det) = sqrt(ltgd_er2+phots_er2)*(smooth(ff_mult(*, dwa),6)*data(*, xpos, ypos, tpos, det) )
	    ;%error=sqrt(%a^2+%b^2)
	    ; estimate of error in FF correction is ~2-5%
	    data(*, xpos, ypos, tpos, det) = $
		  (smooth(ff_mult(*, dwa),6) * $
		   data(*, xpos, ypos, tpos, det) ) + $
		  ff_offset (*, dwa)
	    ;
	  endfor      ; xpos
	endfor          ; ypos
      endfor              ; tpos
    endfor                  ; det
    ;

    return
  end

