;+
; Project     : SOHO - CDS
;
; Name        : GIS_CALIB_FIFO
;
; Purpose     : Applies FIFO calibration factors to GIS spectra.
;
; Explanation : This procedure takes GIS images read with READCDSFITS and
;               applies the appropriate calibration factors for the electronics
;               FIFO.  Called from GIS_CALIB, q.v.
;
; Use         : GIS_CALIB_FIFO, data, info, quiet
;
; Inputs      : DATA =      Data supplied by GIS_CALIB
;               INFO =      Information about the data
;               QUIET =     Flag to run quietly
;                           
; Opt. Inputs : None.
;
; Outputs     : DATA =      The calibrated data will be returned as a 
;                           structure with the same format as the input.
;
; Opt. Outputs: None.
;
; Keywords    : None.
;
; Calls       : None.
;
; Common      : None.
;
; Restrictions: None.
;
; Side effects: None?
;
; Category    : Calibration, GIS, Intensity.
;
; Prev. Hist. : Separated from GIS_CALIB
;
; Written     : Eddie Breeveld, erb@mssl.ucl.ac.uk  3 February 1998
;
; Modified    : Carl Foley, caf@mssl.ucl.ac.uk 21 December 2000,
;                           trapped error as suggested by Jim and Dave (RAL).  
;
;               caf, 10 October 2003, revised to return fifo correction factor
;
; Version     : v.2, 10 October 2003
;
;-
;
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
pro gis_calib_fifo, data, info, quiet,fifo=fifo
;
; FIFO dead time
; Applies to all four detectors simultaneously
; Constant 10 microseconds, non-extending
;
; correct_rate = observed_rate * (1/deadtime) / ((1/deadtime) - observed_rate)
; where the observed rate refers to the sum of all four detectors

;setup fifo array
fifo=fltarr(info.ssolar_x,info.ssolar_y,info.sdel_time)
  if not keyword_set (QUIET) then print, '    FIFO deadtime...'
    deadtime = 10.0e-6  ; seconds
;
    for tpos = 0, info.sdel_time-1 do begin
        for xpos = 0, info.ssolar_x-1 do begin
            for ypos = 0, info.ssolar_y-1 do begin
;
                rate = total (data(*, xpos, ypos, tpos, *))
;
                if rate gt 0.0 then begin 
                    corrected_rate = (rate / deadtime) / ((1.0/deadtime) - rate)>0.<1e10   
                    ratio = corrected_rate / rate
                    data(*, xpos, ypos, tpos, *) = $
	  	    data(*, xpos, ypos, tpos, *) * ratio
		    fifo(xpos, ypos, tpos)=ratio
		  endif
;
            endfor  ; ypos
        endfor      ; xpos
    endfor          ; tpos
;

    return
end
