;+
; Project     : SOHO - CDS
;
; Name        : GIS_CALIB_LONGSLIT
;
; Purpose     : Corrects GIS data for using the big slits (4, 5, and 6)
;
; Explanation : Big slits (4, 5 and 6) cover an area bigger than the detectors,
;               this routine will correct the data for this.
;
; The correction is from RAL tech Report RAL-TR-1999-036, The Laboratory
; calibration of the SOHO Coronal Diagnostic Spectrometer, 18 May 1999,
; J. Lang, et. al. p41f
; n.b., this must be the last applied detector calibration, all the others work
;       on the counts as seen by the detectors, this correction extrapolates
;       to the counts that would have been seen by big enough detectors.
;
; Use         : GIS_CALIB_LONGSLIT, data, info, slit_num, quiet
;
; Inputs      : DATA =      Data as supplied by GIS_CALIB
;               INFO =      Information about the data
;               QUIET =     Flag to run quietly
;                           
; Opt. Inputs : None.
;
; Outputs     : DATA =      The calibrated data will be returned as an 
;                           array with the same format as the input.
;
; Opt. Outputs: None.
;
; Keywords    : None.
;
; Calls       : None.
;
; Common      : None.
;
; Restrictions: None.
;
; Side effects: None?
;
; Category    : Calibration, GIS, Intensity.
;
; Prev. Hist. : None
;
; Written     : Eddie Breeveld, erb@mssl.ucl.ac.uk  25 May 1999
;
; Modified    : Not yet
;
; Version     : 1
;
;-
;
pro gis_calib_longslit, data, info, slit_num, quiet
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Work out correction coefficients
; data from RAL-TR-1999-036
;
    wavelength = [ 256,  297,  338,  393,  443,  493,  656,  721,  785]
    factor     = [0.96, 1.02, 1.07, 1.13, 1.18, 1.23, 1.38, 1.44, 1.49]
;
; Fit the data to a polynomial to find the correction coefficients
;
    cor_coeffs = poly_fit(wavelength, factor, 2)
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; loop over detectors
;
    if slit_num ge 4 then begin
;
        if not keyword_set (QUIET) then print, '    Big Slit correction...'
;
        for det = 1, 3 do begin           ; no correction needed for det. 1
                                          ; i.e. index=0
;
            wavescale = pix2wave('GIS'+trim(det+1), findgen(2048))
;
; correction factors for every pixel.  Limit to values > 1 because if
; less that 1 the whole detector is illuminated (no correction needed).
;
            cor_factor = poly(wavescale, cor_coeffs) > 1.0
;
; loop over each exposure
;
            for tpos = 0, info.sdel_time-1 do begin
                for xpos = 0, info.ssolar_x-1 do begin
                    for ypos = 0, info.ssolar_y-1 do begin
;
; Correct the spectrum at this point for this detector
;
                        data(*, xpos, ypos, tpos, det) = $
                          data(*, xpos, ypos, tpos, det) * cor_factor
;
                    endfor      ; ypos
                endfor          ; xpos
            endfor              ; tpos
        endfor                  ; loop over detectors
    endif                       ; check slit number
;
    return
end



