;+
; Project     : SOHO - CDS
;
; Name        : GIS_CALIB_QUIZ
;
; Purpose     : Applies calibration factors for the GIS quiz-show.
;
; Explanation : This procedure takes GIS images read with READCDSFITS and
;               applies the appropriate calibration factors for the GIS
;               electronic quiz show deadtimes.  See GIS_CALIB.
;
; Use         : GIS_CALIB_QUIZ, data, info, message, missing_array, $
;                     setmissing, quiet
;
; Inputs      : DATA =      Input data from GIS_CALIB
;               INFO =      Information about the GIS data
;               MISSING_ARRAY = Marks missing data
;               SETMISSING = Flag to mark uncalibratable data
;               QUIET =     Flag to suppress information messages
;                           
; Opt. Inputs : None.
;
; Outputs     : DATA =      The calibrated data will be returned as a 
;                           structure with the same format as the input.
;                           The units of the calibrated data will be
;                           photon-events/pixel/sec.
;               MISSING_ARRAY = Marks previous and new missing data
;               MESSAGE =   Returns error messages (if any)
;
; Opt. Outputs: None.
;
; Keywords    : None.
;
; Calls       : None.
;
; Common      : None.
;
; Restrictions: None.
;
; Side effects: None?
;
; Category    : Calibration, GIS, Intensity.
;
; Prev. Hist. : Separated from GIS_CALIB
;
; Written     : Eddie Breeveld, erb@mssl.ucl.ac.uk  30 January 1997
;
; Modified    : 
;
; Version     : V.1 30 January 1998
;
;-
;
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
pro gis_calib_quiz, data, info, message, missing_array, setmissing, $
                      quiet
;
;
; Quiz show blocking, just set an upper limit on the rates.
; 600ns is the response time.  If data arrives at greater than 
; 10 times this (i.e. faster than 6 microseconds) then complain.
; The maximum rate is then 1.0/6.0e-6 or approx 1.7e5 over all
; four detectors
;
    if not keyword_set (QUIET) then print, $
                                      '    Quiz show blocking correction ...'
    max_rate = 1.7e5            ; per second
;
    for tpos = 0, info.sdel_time-1 do begin
        for ypos = 0, info.ssolar_y-1 do begin
            for xpos = 0, info.ssolar_x-1 do begin
;
                rate = total (data(*, xpos, ypos, tpos, *))
;
                if rate gt max_rate then begin 
                    if keyword_set (setmissing) then begin
                        missing_array(*, xpos, ypos, tpos, *) = -1
                    endif else begin 
                        MESSAGE = 'ABORTED - count rate too high for ' + $
                            'this quiz show blocking correction'
                        return
                    endelse
                endif
;
            endfor  ; xpos
        endfor      ; ypos
    endfor          ; tpos
;
    return
end
;
