;+
; Project     : SOHO - CDS
;
; Name        : GIS_CALIB_STGD
;
; Purpose     : Calibrates GIS data for short term gain depression
;
; Explanation : This procedure applies short term gain depression calibration 
;               factors to GIS data.  Used as part of GIS_CALIB, q.v.
;               Reads calibration information from 
;               $CDS_GIS_CAL_INT/shortterm.dat
;
; Use         : GIS_CALIB_STGD, data, message, missing_array, setmissing, $
;                      quiet
;
; Inputs      : DATA =      Supplied GIS data from GIS_CALIB
;               MISSING_ARRAY = Array marking missing data
;               SETMISSING = Flag to mark uncalibratable data
;               QUIET =     Flag to run quietly
;                           
; Opt. Inputs : None.
;
; Outputs     : DATA =      The calibrated data will be returned as a 
;                           structure with the same format as the input.
;               MESSAGE =   Returns error messages, if any.
;               MISSING_ARRAY = Array marking missing and uncalibratable data
;
; Opt. Outputs: None.
;
; Keywords    : None.
;
; Calls       : None.
;
; Common      : None.
;
; Restrictions: None.
;
; Side effects: None?
;
; Category    : Calibration, GIS, Intensity.
;
; Prev. Hist. : Separated from GIS_CALIB
;
; Written     : Eddie Breeveld, erb@mssl.ucl.ac.uk  3 February 1998
;
; Modified    : v2, erb, 09 Jan 1999, check only valid data points
;               v3, erb, 21 May 1999, use much better STGD data
;               v4, erb,  4 Jun 1999, correct bug if no valid data
;               v5, caf, August 2003, instead of marking unreliable data as missing
;                                     it is now passed and a suitable error entered into
;                                     the QLDS.
;
; Version     : 5
;
;-
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
pro gis_calib_stgd, data, message, missing_array, setmissing, $
                      quiet, slit_num
;
; Calibrate for short term gain depression
; 
    if not keyword_set (QUIET) then print, $
            '    Short term gain depression correction...'
;
; Maximum percent stgd error allowed
;
;;; carl changed - need to pass to an error as opposed to loosing data
; this high count rate data is interesting..
    
    max_percent = 4.
;;    max_allowed_stgd = (100.-max_percent)/100.
;
 max_allowed_stgd =0.00000001
    
; slit data (size in arcsec)
;
	slit_arcsec = 	[$
			         [ 2.0,   2.0],         $   ; slit 1
			         [ 4.0,   4.0],         $   ; slit 2
			         [ 8.0,  50.8],         $   ; slit 3
			         [ 2.0, 240.0],         $   ; slit 4
			         [ 4.0, 240.0],         $   ; slit 5
			         [60.0, 240.0]]             ; slit 6
;
; mcp data
;
    pixels_per_mm = 2048.0/50.0     ; per mm
    mm_per_pore   = 0.015           ; mm
    pores_per_mm2 = 5132            ; mm2 (Alice's thesis p.142)
;
; slit heights (mm)
;
    slit_heights = slit_arcsec[1, *] * 0.0125
;
; c/r per pore vs gain (Alice Thesis, page 140f)
;
    cr_per_pore = [0.0014, 0.007, 0.016, 0.065, 0.205]
    cpp_gain    = [4.52,   4.46,  4.15,  3.5,   1.6  ] * 1e7
;
; output c/r vs gain  (Alice Thesis, fig 72)
;
    gain   = [1.1, 1.75, 2.33, 2.61, 3.04, 3.6, 3.9, 4.36, 5.0, 5.67] * 1e7
    out_cr = [220, 339,  418,  450,  474,  505, 510, 503,  481, 469] * 1.0
;
; interpolate out_cr at cpp_gain values
;
    interpol_cr = interpol(out_cr, gain, cpp_gain)
;
; Normalise, using maximum value
;
    interpol_cr = interpol_cr/max(interpol_cr)
; 
; fit a 2nd order polynomial through the points to find the gain 
; depression coefficients.
;
    gd_coeffs = poly_fit(cr_per_pore, interpol_cr, 2)
;
    curve_array = findgen(1000)/4000.
;
; Work out <max_allowed_stgd error region
;
    small_error_region = where (poly(curve_array, gd_coeffs) gt max_allowed_stgd)
;
; Maximum counts per pore with less than max_allowed_stgd error
;
    small_stgd = max(curve_array[small_error_region])
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Now work out the slit sizes (lengths) in mm on the detectors
; (From RAL CDS calibration report)
;       z = ll' + LL'
;
; First input l', L', and L 
;
    l_prime = [$                 
                (2.141 + 2.496)/2.0, $      ; average for detector 1
                (2.644 + 2.976)/2.0, $      ; detector 2
                (3.195 + 3.522)/2.0, $      ; detector 3
                (4.005 + 4.358)/2.0  ]      ; detector 4
;
    big_l_prime = [$                  
                (3.058 + 3.423)/2.0, $      ; average for detector 1
                (3.563 + 3.877)/2.0, $      ; detector 2
                (4.083 + 4.389)/2.0, $      ; detector 3
                (4.838 + 5.162)/2.0  ]      ; detector 4
;
    big_l = [2.26, 2.26, 2.26, 2.26, 2.26, 2.26]     ; const. for each slit
;
; Calculate 'z' (the image length on the detector)
;
    image_mm_length = slit_heights##l_prime + big_l##big_l_prime
;
; max image size is 16mm
;
    image_mm_length = image_mm_length < 16.0
;
; now the slit areas (per pixel)
;
    image_area_per_pixel = image_mm_length / pixels_per_mm      ; mm2 per pixel
    pores_per_pixel = image_area_per_pixel * pores_per_mm2      ; pores
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Find max count rates per pixel with less than max_allowed_stgd error
;
    max_cr_per_pixel = pores_per_pixel * small_stgd
;
; This 4*6 array (max_cr_per_pixel) contains the max count rates allowed
; for all 4 detectors for all 6 slits
;
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Short term gain depression operates on a pixel by pixel basis,
; thus we have to check every pixel on every detector in every
; exposure
;
;
; loop over detectors
;
    for det = 0, n_elements(data(0,0,0,0,*))/2-1 do begin

      ;
; get the required maximum value from the array created above
;
        shortterm_max = max_cr_per_pixel[det, slit_num]
;
        det_data = data[*, *, *, *, det]
        det_miss = missing_array[*, *, *, *, det]
; 
        valid = where (det_miss ge 0, count)
        if (count gt 0) then $
		list = where (det_data[valid] gt shortterm_max, count)
;
        if count gt 0 then begin
            if keyword_set (setmissing) then begin
                det_miss[valid[list]] = -1
            endif else begin
                MESSAGE = 'ABORTED - count rate too high for ' + $
                  'short term gain depression correction, use /SETMISSING'
                return
            endelse
;
; return the missing array to the original arrays
;
            missing_array[*, *, *, *, det] = det_miss
        endif
;       
    endfor                  ; loop over detectors
;
    return
end
;
