;+
; Project     : SOHO - CDS
;
; Name        : GIS_ERROR
;
; Purpose     : Extracts GIS errors as stored in the QLDS, by GIS_CALIB.
;
; Explanation : This routine extracts the errors from the QLDS which were
;               stored during calibration. Currently, this error only contains
;               that associated with the flat field correction. But it will shortly
;               contain, ghosting errors and those associated with inaccurate
;               Short term gain depression corrections. Currently, data qhich has
;               a STGD correction of the order of 5% are marked as missing
;               This happens to include data which are most interesting. So in
;               instead of marking this data as missing, a suitable uncertainly will
;               be included.
;
; Use         : error_array=gis_error(qlds, detno)
;               error_array=gis_error(qlds, 1, ltgd=ltgd)
;
;
; Inputs      : QLDS =      Structure variable as read by READCDSFITS and
;                           ghost corrected with GHOST_BUSTER.
;               DETNO =     The Detector number for which the error estimate is wanted.
;
; Opt. Inputs : None.
;
; Outputs     : The error array for each pixel.
;
; Opt. Outputs: ltgd  = This is the percentage error attributed to the LTGD correction.
;
; Keywords    : None.
;
; Calls       :
;
; Common      : None.
;
; Restrictions: None.
;
; Side effects: None?
;
; Category    : Calibration, GIS, Intensity.
;
; Written     : Carl Foley, caf@mssl.ucl.ac.uk  August 2003
;
; Modified    :             caf, 2 Oct 2003. Incorporated LTGD keyword so that the percentage
;                           error attributed to the LTGD can be seen, and combined with other
;                           non-statistical / statistical errors independantly if desired.
;                           Changed notification to user.
;
; Version     : 2.
;
;-
;
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
FUNCTION GIS_CALIB_NEAREST, INPUT, ARRAY
;
; returns the index of the array element that is closest to the input
;
    subtract = array - input
    absolute = abs (subtract)
    sort_index = sort (absolute)
;
    return, sort_index(0)
end
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;
function gis_error, qlds, detno,stat=stat,ltgd=ltgd
;
; set up GIS stages strings
;
@gis_stage.inc
;
; extract the relevant data and
; save missing pixels - these will be set to missing again at the end
;
    if not keyword_set (QUIET) then print,'    Extracting error for GIS'+strcompress(string(detno),/remove_all)
;
; Check if calibrated already
;
;
    info = gt_dimension (qlds)
    exposure = gt_exptime (qlds)
    info.sdel_time = info.sdel_time > 1	; force at least one exposure!
;
    if detno eq 0. then print,'Detectors are numbered: 1-4, Defaulting to Detector 1 !'
    if detno eq 0. then detno=1

    data=gt_windata (qlds, (detno+3))>0.
    ;    if keyword_set(ltgd) then begin
    obstime = anytim2tai (gt_start (qlds))
    ;
    ; Get time of flat field files
    ;
    ff_files = find_file (concat_dir('$CDS_GIS_CAL_INT','ff_*.dat'), count=count)
    if count eq 0 then begin
      MESSAGE = 'ABORTED - no GIS flat field files found'
    endif
    ;
    len = strlen (ff_files(0))
    ff_times = anytim2tai (strmid (ff_files, len-15, 11))
    ;
    ; Find nearest time to obs time
    ;
    nearest_file = ff_files (gis_calib_nearest (obstime, ff_times))
    ;
    ; Open the nearest file
    ;
    ff_mult   = fltarr (2048, 4)
    ff_offset = fltarr (2048, 4)
    ;
    openr, unit, nearest_file, /get_lun, /xdr
    readu, unit, ff_mult
    readu, unit, ff_offset
    free_lun, unit
    ;
    ltce=[0.0198,0.0513,0.0499,0.0377]
    ltgd=ltce(detno-1)*(ff_mult(*,detno-1)-1.)*100.
    ;endif
    return,data
  end


