;+
; Project     : SOHO - CDS
;
; Name        : GIS_FF_WRITE
;
; Purpose     : Write a GIS flat field
;
; Explanation : Write a GIS calibration flat field file ff_*.dat 
;
; Use         : GIS_FF_WRITE, in_date,det1_ff_mult=det1_ff_mult,det1_ff_offset=det1_ff_offset,$
;               det2_ff_mult=det2_ff_mult,det2_ff_offset=det2_ff_offset,$
;		det3_ff_mult=det3_ff_mult,det3_ff_offset=det3_ff_offset,$
;	        det4_ff_mult=det4_ff_mult,det4_ff_offset=det4_ff_offset
;		  
;
; Inputs      : in_date, (prompted for if not supplied): date of flat field 
;		            in any valid time format (for anytim2cal)
;
; Opt. Inputs : None
;
; Outputs     : None
;
; Opt. Outputs: None.
;
; Keywords    : None
;
; Calls       : anytim2cal, anytime2tai, strep
;
; Common      : None.
;
; Restrictions: None.
;
; Side effects: None.
;
; Category    : Calibration, GIS, Intensity.
;
; Prev. Hist. : 
;
; Written     : Eddie Breeveld, erb@mssl.ucl.ac.uk  3 February 1998
;
; Modified    : v.2, 10 May 1999, erb: Use XDR format for the binary file
; Modified    : v.3,    May 2003, caf: Now accepts REAL flat field data
;
; Version     : 3
;
;+
;
pro gis_ff_write, in_date,det1_ff_mult=det1_ff_mult,det1_ff_offset=det1_ff_offset,$
		  det2_ff_mult=det2_ff_mult,det2_ff_offset=det2_ff_offset,$
		  det3_ff_mult=det3_ff_mult,det3_ff_offset=det3_ff_offset,$
		  det4_ff_mult=det4_ff_mult,det4_ff_offset=det4_ff_offset

  ;
	if not keyword_set(in_date) then begin
	    in_date = ''
	    read, 'Type the date, e.g., 5-jan-1996: ', in_date
	endif
;
;  Convert to correct format
;
	in_date = anytim2cal (in_date, form=10, /date)
	in_date = strep (in_date, ' ', '0')	; replace blank with '0'
;
	filename = 'ff_' + strlowcase(in_date) + '.dat'
;
;  Check the date (this is how it will be used in GIS_CALIB_FF_STGD)
;
	ff_time = anytim2tai (strmid (filename, 3, 11))
;
;  See if this is intended
;
;;	yes_no,  'Is ' + strep (anytim2cal (ff_time, form=10, /date), ' ', '0') + $
;;			' the correct file date', answer
;
;	if not answer then begin
;		print, 'Incorrect time, exiting'
;		return
;	endif
;
;  Use dummy data for now.
;  Assume correction of the form (ax + b),
;  where a is the multiplier and b the offset, 
;  and x is the observed data in counts per second
;
if not keyword_set (det1_ff_mult) then det1_ff_mult = replicate (0.0, 2048)
if not keyword_set (det2_ff_mult) then det2_ff_mult = replicate (0.0, 2048)
if not keyword_set (det3_ff_mult) then det3_ff_mult = replicate (0.0, 2048)
if not keyword_set (det4_ff_mult) then det4_ff_mult = replicate (0.0, 2048)

	;
;;;	det4_ff_mult = (findgen (2048) + 1.0) / 204.8
;
if not keyword_set (det1_ff_offset) then det1_ff_offset = replicate (0.0, 2048)
if not keyword_set (det2_ff_offset) then det2_ff_offset = replicate (0.0, 2048)
if not keyword_set (det3_ff_offset) then det3_ff_offset = replicate (0.0, 2048)
if not keyword_set (det4_ff_offset) then det4_ff_offset = replicate (0.0, 2048)
;
;  Write it out
;
	openw, unit, filename, /get_lun, /xdr
;
	writeu, unit, det1_ff_mult
	writeu, unit, det2_ff_mult
	writeu, unit, det3_ff_mult
	writeu, unit, det4_ff_mult
;
	writeu, unit, det1_ff_offset
	writeu, unit, det2_ff_offset
	writeu, unit, det3_ff_offset
	writeu, unit, det4_ff_offset
;
	free_lun, unit
;
    print, 'To make the new file available, move the file ' + trim (filename)
    print, 'from the current directory to the location defined by $CDS_GIS_CAL_INT'
    print, 'on the Goddard CDS directories at the EOF'

;
;  All done
;
return
end
