;+
; Project     : SOHO - CDS
;
; Name        : GIS_FIT
;
; Purpose     : Automatically fits GIS spectra
;
; Explanation : Uses CFIT structures to automatically fit clearly identified lines
;               in the GIS spectra. The structure returned contains the intensities,
;               amplitudes and widths, from the fits along with their associated
;               uncertainties. The errors in the intensity are determined directly
;               from those in the amplitude and width, which are combined in quadrature :
;
;                sigma_I = sqrt((sigma_amplitude/amplitude)^2+(sigma_width/width)^2)
;
;
; Use         : gisfit=gis_fit(qlds, /plot)
;
;
;
; Inputs      : QLDS =      Structure variable of GIS data as read by
;                           READCDSFITS.
;
; Opt. Inputs : PLOT =      Plots progress of fits
;
; Outputs     : GISFIT =    resultant fit structure containing the line intensity
;                           amplitude, width, position, along with other ancillory data.
;
; Opt. Outputs: None.
;
; Keywords    :
;
; Calls       :
;
; Common      : None.
;
; Restrictions: None.
;
; Side effects: None.
;
; Still To Do :
;
; Category    : Calibration, GIS, Intensity.
;
; Prev. Hist. : None.
;
; Written     : Carl Foley, caf@mssl.ucl.ac.uk  August 2003
;
; Modified    : v1.1, caf, Mon Sep 22 2003
;               made structures anonymous to remove comflicts
;               when analysing different data sets.
;
; Version     : 1.1
;
;
;-
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
function gis_fit,qlds,plot=plot,gisfit=gisfit
;
  ;
  ;
  ;
; Set up some strings for the different stages of GIS processing
;
@gis_stage.inc
;
;
;
; Make sure that it is GIS data we're dealing with.
;
  if (size(qlds))(2) ne 8 then begin
    MESSAGE = 'You need to enter a qlds or a gisfit structure'
    goto, HANDLE_ERROR
  endif
  if ((size(qlds.(0)))(0) gt 0) then goto,spc
;  if tag_exist(qlds,'spectra') then goto,spc
  if QLDS.HEADER.DETECTOR ne 'GIS' then begin
    MESSAGE = 'Cannot use this routine with NIS data'
    goto, HANDLE_ERROR
  endif
  ;
  ; Make sure we are using the correct wavelength calibration
  ;
  success = restore_wavecal(qlds)
  if not (success) then message, 'Restore wavelength calibration failed'
  ;

  description = gt_windesc (qlds, 1)
  case description.units of
    gis_stage.raw:     begin         ;
       gis_smooth,qlds
       ghost_buster,qlds
       gis_calib,qlds
       
    end
    ;	  return
    gis_stage.smoothed: begin         ; allow further processing
      ghost_buster,qlds
      gis_calib,qlds
      
    end
    ;	  return
    gis_stage.ghost_correct: begin   ; allow further processing
      gis_calib,qlds
    end
    ;return
    gis_stage.photons:
    gis_stage.steradian_m2:
    gis_stage.arcsec2_cm2:
    ;	  else: begin
    ;	    MESSAGE = 'ERROR - Unknown data units for detector ' + $
    ;	    trim(det+1) + ': ' + description.units
    ;  return
    ;	  end
  endcase


  ;qlds.DETDESC(*).UNITS = units
  ;;;;;;;;;;;;;

  	spc:
  	gisfit=cf_gis1a(qlds,plot=plot)
  	gisfit=cf_gis1b(gisfit,plot=plot)
  	gisfit=cf_gis1c(gisfit,plot=plot)
  	gisfit=cf_gis4a(gisfit,plot=plot)
  	gisfit=cf_gis4b(gisfit,plot=plot)
  	gisfit=cf_gis4c(gisfit,plot=plot)
    gisfit=cf_gis2a(gisfit,plot=plot)

  ;
  if not keyword_set (QUIET) then print, 'Fitting complete'
  goto, FINISH

  HANDLE_ERROR:
    if N_ELEMENTS(ERRMSG) ne 0 then ERRMSG = 'GIS_FIT: ' + MESSAGE $
	        else MESSAGE, MESSAGE, /continue
    ;
    ; Exit
    ;
    FINISH:
  return,gisfit
end
