;+
; Project     : SOHO - CDS
;
; Name        : GIS_GHOSTING
;
; Purpose     : Finds likely GIS ghost regions
;
; Explanation : This procedure takes GIS data read with READCDSFITS and finds
;               the likely ghosting regions  
;
; Use         : GIS_GHOSTING, GSET_ID, CHAN, GHOSTS 
;                   Mainly used within the GIS ghost plotting and correcting
;                   software.
;
; Inputs      : gset_id = GSET that data is wanted for.
;               chan    = Detector number (1-4)
;
; Opt. Inputs : None.
;
; Outputs     : GHOSTS  = an array containing ghosting information:
;                   0,* = lower ghost pixel address
;                   1,* = upper ghost pixel address
;                   2,* = position in arm (0.0 - 1.0)
;
; Opt. Outputs: none.
;
; Keywords    : None.
;
; Calls       : get_gset
;
; Common      : None.
;
; Restrictions: None.
;
; Side effects: None.
;
; Category    : Calibration, GIS, Intensity.
;
; Prev. Hist. : Split from GHOST_PLOT_ONE
;
; Written     : Eddie Breeveld, erb@mssl.ucl.ac.uk  13 Nov 1997
;
; Modified    : v1.1, erb, 10 Mar 1998, improved comments
;               v2,   erb, 15 Jan 1999, improved GSET_ID processing by
;                               using gset_id as input, thus all routines
;                               calling this procedure will need to be
;                               modified
;		v3,   erb, 28 Jun 1999, clarified comments
;
; Version     : 3
;
;-
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;
pro gis_ghosting, gset_id, detector, ghosts
;
;  extract the gset_id and get GSET the definition
;
    get_gset, gset_id,  gset
;
;  form the parameters that build the gset
;
    idet = detector - 1
;
    stop   = gset.lut_par[idet,1]/10.
    start  = gset.lut_par[idet,2]/10.
    length = stop^2 - start^2
    unit   = length/2048.
;
    pixels = indgen(2048)
    pixs   = 2048.0-pixels      ; reverse the array (back to normal)
    ghosts = fltarr(3,2048) - 1 ; prefill array with -1's
;
;  do ghosts
;
    for jpix=0, 2047 do begin
;
;  Now start doing some sums
;  find the line in "arm" space
;
        arm = sqrt(pixs[jpix]*unit + start^2)
;
;   now look at arms either side
;
        g_ap1 = ((arm+1)^2 - start^2)/unit
        g_am1 = ((arm-1)^2 - start^2)/unit
;
        if (g_ap1 ge 0) AND (g_ap1 le 2048) then begin
                ghosts[0,jpix] = 2048.0-(g_ap1) 
        endif
;
        if (g_am1 ge 0) AND (g_am1 le 2048) then begin
                ghosts[1,jpix] = 2048.0-fix(g_am1+0.5)
        endif
;
;  save arm position.
;
        ghosts[2,jpix] = arm
    endfor
return
end
