;+
; Project     : SOHO - CDS
;
; Name        : GIS_HV_CAL()
;
; Purpose     : Rewrite the series tables with updated voltages for each
;               GIS run.  Only works for study GIMCPS1.
;
; Explanation : To calibrate the effect of HV changes, ten rasters are run
;               each with a different voltage.  This routine overwrites the 
;               values created in the series tables by CPT with the 
;               required test values of the GIS HV.  It reads the series 
;               file/table ID numbers from the latest TTC file.
;
; Use         : IDL> gis_hv_cal
;
; Inputs      : None
;
; Opt. Inputs : None
;
; Opt. Outputs: None
;
; Keywords    : None
;
; Calls       : None
;
; Common      : None
;
; Restrictions: User must have write access to the command directory
;               /cs/data/plan/command/cvt
;
; Side effects: None
;
; Category    : Calibration
;
; Prev. Hist. : None
;
; Written     : C D Pike, RAL, 14-Jun-2004
;
; Modified    : Version 2. Corrected ambiguity in finding TTC files.  CDP
;               Version 3. Modified to cope with multiple runs of 
;                          study GIMCPS1.  CDP
; 
; Version     : Version 3, 08-Sep-2004
;-

pro gis_hv_cal

;
;  Calibration HVs to be used as notified by C Foley
;
hv = ['0x00BD',$
      '0x00BF',$
      '0x00C1',$
      '0x00C3',$
      '0x00C5',$
      '0x00C7',$
      '0x00C9',$
      '0x00CB',$
      '0x00CD',$
      '0x00CF']


;
;  find latest TTC file
;
f = find_file('/cs/data/plan/command/ttc/*.TTC')
f = last_item(f)
print,'Looking in TTC file ',f

;
;  read TTC file
;
tin = rd_ascii(f)
n = n_elements(tin)

;
;  find all the references to study GIMCPS1
;
nn = strpos(tin,'# SCI SPEC = GIMCPS1') ge 0
nref = where(nn gt 0,c)
if c le 0 then begin
   bell
   print,'No runs of GIMCPS1 found.'
   return
endif

;
;  find and overwrite all series files involved in each 
;  run of the study
;
nn = n_elements(nref)
for k=0,nn-1 do begin
   t = tin
   if strpos(t(nref(k)+7),'# CDHS series indices') eq 0 then begin
      t = strcompress(t(nref(k)+7))
      t = strtrim(str_pick(t,'='),2)
      nf = str2arr(t,delim=' ')
   endif else begin 
      bell
      print,'Series indices not found, file not updated.'
      return
   endelse

   n = n_elements(nf)
   hvn1 = 0
   hvn2 = 0
   hvn3 = 0
   hvn4 = 0

;
;  read all the series files indicated in the TTC file and update the
;  HV values
;
   print,' '
   print,'------------------------'
   print,' '
   for i=0,n-1 do begin
      print,'Run: '+trim(k+1)+' of study GIMCPS1' 
      t = rd_ascii('/cs/data/plan/command/cvt/series_'+nf(i)+'.dt')

      n1 = where(strpos(t,'set up high voltage for detector 1') ge 0)  
      n2 = where(strpos(t,'set up high voltage for detector 2') ge 0)  
      n3 = where(strpos(t,'set up high voltage for detector 3') ge 0)  
      n4 = where(strpos(t,'set up high voltage for detector 4') ge 0)  
      if n1(0) ge 0 then begin
         t(n1+1) = 'CBGHV1V '+ hv(hvn1:hvn1+n_elements(n1)-1)
         hvn1 = hvn1+n_elements(n1) 
      endif else begin
         print,'No detector 1 voltages found'
      endelse

      if n2(0) ge 0 then begin
         t(n2+1) = 'CBGHV2V '+ hv(hvn2:hvn2+n_elements(n2)-1)
         hvn2 = hvn2+n_elements(n2) 
      endif else begin
         print,'No detector 2 voltages found'
      endelse

      if n3(0) ge 0 then begin
         t(n3+1) = 'CBGHV3V '+ hv(hvn3:hvn3+n_elements(n3)-1)
         hvn3 = hvn3+n_elements(n3) 
      endif else begin
         print,'No detector 3 voltages found'
      endelse

      if n4(0) ge 0 then begin
         t(n4+1) = 'CBGHV4V '+ hv(hvn4:hvn4+n_elements(n4)-1)
         hvn4 = hvn4+n_elements(n4) 
      endif else begin
         print,'No detector 4 voltages found'
      endelse

      yes_no,'Ok to overwrite '+'/cs/data/plan/command/cvt/series_'+nf(i)+'.dt ?',ans
      if ans eq 1 then begin
         wrt_ascii,strtrim(t,2),'/cs/data/plan/command/cvt/series_'+nf(i)+'.dt'
         print,'Written: '+'/cs/data/plan/command/cvt/series_'+nf(i)+'.dt'
      endif else begin
         print,'series_'+nf(i)+'.dt file NOT written.'
      endelse
   endfor
endfor

end
