;+
; Project     : SOHO - CDS
;
; Name        : GIS_PLOT
;
; Purpose     : Plots a summary of GIS detector data
;
; Explanation : This procedure takes GIS data read with READCDSFITS and
;               plots the average spectra for each detector 
;
; Use         : GIS_PLOT, QLDS, CHAN  [, /LOGSCALE] $
;                   [, /ANGSTROMS] [, /nm] [, /NOLABEL] $
;                   [, WAVERANGE=[min,max] ]  [, /PIXELS] $
;
;
; Inputs      : qlds    = Structure variable as read by READCDSFITS.
;               chan    = Detector number (1-4)
;
; Opt. Inputs : None.
;
; Outputs     : None.
;
; Opt. Outputs: None.
;
; Keywords    : LogScale      = Plot data with a log scale to show fine details
;
;               Angstroms     = Use Angstroms
;
;               nm            = Use nm
;
;               WaveRange     = Array with the wavelength range for plotting
;                               the data:
;                   waverange(0) = minimum plotted wavelength in nm
;                   waverange(1) = maximum plotted wavelength in nm
;                              If /angstroms or /pixels is set, then 
;                              wavelengths are in angstroms or pixels.
;                              The default is to plot over the whole detector.
;
;               pixels        = plot in pixels, not by wavelength
;
;		notitle       =	suppress the top title
;
; Calls       : ghost_plot_one
;
; Common      : None.
;
; Restrictions: None.
;
; Side effects: None.
;
; Category    : Calibration, GIS, Intensity.
;
; Prev. Hist. : Developed from ghost_plot_one
;
; Written     : Eddie Breeveld, erb@mssl.ucl.ac.uk  5 March 1999
;
; Modified    : v2, erb, 21 Jul 1999, propagated /nolabel keyword
;
; Version     : 2
;
;-
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;
pro gis_plot, qlds, chan, ghostdata, $
    logscale=logscale, angstroms=angstroms, waverange=waverange, $
    pixels=pixels, nm=nm, nolabel=nolabel
;
;
    if (n_params() lt 2) then begin
input_error:
        message, 'ERROR - Usage:', /continue
        chkarg, 'gis_plot'
        return
    endif
;
    if not keyword_set(nm) and not keyword_set(pixels) then angstroms=1
;
    ghost_plot_one, qlds, chan, /noghost, $
        logscale=logscale, angstroms=angstroms, waverange=waverange, $
        pixels=pixels, nm=nm, nolabel=nolabel
;
    return
end
