;+
; Project     :	SOHO - CDS
;
; Name        :	GIS_SMOOTH
;
; Purpose     :	Smooths GIS spectra to 'remove' fixed patterning
;
; Explanation :	This function takes GIS images read with READCDSFITS and
;		smooths the spectra.  
;
; Use         :	GIS_SMOOTH, QLDS [,ERRMSG=ERRMSG] [,SMOOTHSIZE=SMOOTHSIZE]
;
; Inputs      :	QLDS	= Structure variable as read by READCDSFITS.
;
; Opt. Inputs :	None.
;
; Outputs     :	QLDS =	The smoothed data will be returned as a 
;			structure with the same format as the input.
;			The data will be marked as 'smoothed' for further 
;			processing.
;
; Opt. Outputs:	None.
;
; Keywords    :	ERRMSG = If defined and passed, then any error messages will be
;			returned to the user in this parameter rather than
;			depending on the MESSAGE routine in IDL.  If no errors
;			are encountered, then a null string is returned.  In
;			order to use this feature, ERRMSG must be defined
;			first, e.g.
;
;				ERRMSG = ''
;				CALIB = GIS_CALIB (DATA, ERRMSG=ERRMSG)
;				IF ERRMSG NE '' THEN ...
;
;		SMOOTHSIZE = Size of smoothing function, default 7 pixels,
;			minimum 4 pixels.  Any number less than 4 will result 
;			in no smoothing
;
; Calls       :	Hanning, Convol, gt_winsize, gt_windesc, gt_dimension, gt_detector
;
; Common      :	None.
;
; Restrictions:	GIS_SMOOTH only works with full GIS data windows. 
;
; Side effects:	None.
;
; Category    :	Calibration, GIS, Intensity.
;
; Prev. Hist. :	None.
;
; Written     :	Eddie Breeveld, erb@mssl.ucl.ac.uk  4 April 1997
;
; Modified    :	Version 2: Vincenzo Andretta, VAndretta@solar.stanford.edu, 23 January 1998
;                  Rearranged indexes in some key arrays to fix a problem 
;                  with spurious extra dimensions introduced in the output data.
;               Version 3: Vincenzo Andretta, VAndretta@solar.stanford.edu, 24 March 1998
;                  Fixed a bug: missing pixels were never recognized as such.
;                  Filling missing pixels before smoothing (using CDS_FILL_MISSING).
;                  Use keyword EDGE_TRUNCATE in CONVOL.
;                  Made SMOOTHSIZE=7 the default.
;		Version 4: erb, 24 May 1998; 
;		  Incorporated gis_stage.inc
;               v5.      : caf,    Aug 2003; fix to work with partial GIS
;                                  data.
;             
; Version     : 5
;
;-
;
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;
PRO GIS_SMOOTH, QLDS, ERRMSG=ERRMSG, SMOOTHSIZE=SMOOTHSIZE
;
;
;  See if we can continue...
;  Check the number of parameters.
;
	message = ''
	if n_params() ne 1 then begin
	    MESSAGE = 'Syntax: GIS_SMOOTH,  QLDS [,ERRMSG=ERRMSG] [,SMOOTHSIZE=SMOOTHSIZE]'
	    goto, handle_error
	endif
;
;  Make sure that it is GIS data we're dealing with.
;
	if QLDS.HEADER.DETECTOR ne 'GIS' then begin
	    MESSAGE = 'Cannot use this routine with VDS data'
	    goto, HANDLE_ERROR
	endif
;
;  Set up some strings defining the GIS processing stage
;
@gis_stage.inc
;
;  Check if the full GIS is used 
;
	winsize = gt_winsize (qlds)
;
	if n_elements (winsize) ne 4 then begin
	    MESSAGE = 'Only partial GIS windows, can now be calibrated - carl Sep-2003'
;	    goto, HANDLE_ERROR
	endif
	for i=0, n_elements(qlds.detdesc.waveband)-1 do begin
	    if (winsize(i) ne 2048) then begin
		MESSAGE = 'Only partial GIS windows, can now be calibrated - carl Sep-2003'
		goto, HANDLE_ERROR
	    endif
	endfor
;
;  Check if smoothed already
;
	description = gt_windesc (qlds, 0)
;
	if description.units ne GIS_STAGE.RAW then begin
	    MESSAGE = 'Data not raw, cannot smooth "' + description.units + '" data'
	    goto, HANDLE_ERROR
	endif
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;  extract the relevant data and
;  save missing pixels - these will be set to missing again at the end
;
	info = gt_dimension(qlds)
	corrected_data = fltarr(info.SWAVELNTH, info.ssolar_x, info.ssolar_y, info.sdel_time>1 , n_elements(qlds.detdesc.waveband))
	for i=0, n_elements(qlds.detdesc.waveband)-1 do begin
	    corrected_data(*, *, *, *, i) = gt_windata (qlds, i)
	endfor
;
; check amount of missing data, build up a new missing array
;
	missing_array = fix (corrected_data) * 0			; empty missing array
	missing_list  = [where (corrected_data eq description(0).missing, count)]
	not_missing   = [where (corrected_data ne description(0).missing)]
;
	if count gt 0 then begin
	    missing_array(missing_list) = -1	; set missing data to -1 in this array
;
;  Fill in missing data using CDS_FILL_MISSING
;  These data will be reset to missing again at the end.
;
            qlds_fill=copy_qlds(qlds)
            cds_fill_missing,qlds_fill
            for i=0, n_elements(qlds.detdesc.waveband)-1 do begin
              corrected_data(*, *, *, *, i) = gt_windata (qlds_fill, i)
            endfor
            delete_qlds,qlds_fill
;
	endif
;
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;
;  Smooth the whole dataset with a convolution with a  hanning function
;
	if not keyword_set(smoothsize) then smoothsize = 7
;
	if (smoothsize ge 4) then begin
		func = hanning (smoothsize)
;
		for det=0, n_elements(qlds.detdesc.waveband)-1 do begin
		    for tpos = 0, (info.sdel_time>1)-1 do begin
		        for ypos = 0, info.ssolar_y-1 do begin
		            for xpos = 0, info.ssolar_x-1 do begin
;
			        spec = corrected_data(*, xpos, ypos, tpos, det)
;
			        corrected_data(*, xpos, ypos, tpos, det) = $
				     convol (spec, func, total (func),/edge_tr)
;
		            endfor	; xpos
		        endfor		; ypos
		    endfor		; tpos
		endfor			; det
	endif			; smoothsize ge 4
;
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;
;  Restore calibrated missing data to 'missing'
;
	list_missing = [where (missing_array lt 0)]
	if list_missing(0) ne -1 then corrected_data(list_missing) = -1.0
;
	qlds.DETDESC(*).MISSING = -1.0		; set missing flag to -1
;
;  copy data into the structure
;
	for i=0, n_elements(qlds.detdesc.waveband)-1 do begin
	    st_windata, qlds, i, corrected_data(*, *, *, *, i)
	endfor
;
;  Tell the structure that we've finished
;
	qlds.DETDESC(*).UNITS = GIS_STAGE.SMOOTHED
;
	goto, FINISH
;
;  Error handling point.
;
HANDLE_ERROR:
	if N_ELEMENTS(ERRMSG) ne 0 then ERRMSG = 'GIS_SMOOTH: ' + MESSAGE $
		else MESSAGE, MESSAGE, /continue
;
;  Exit
;
FINISH:
	return
end
