;+
; Project     : SOHO - CDS     
;                   
; Name        : GISPLOT
;               
; Purpose     : Plot engineering data from all four GIS detectors.
;               
; Explanation : This procedure reads engineering data from all four GIS
;		detectors and then generates a single plot that looks kinda
;		like a strip chart.  Engineering data is read from tm files
;		and extracted into a number of arrays.  After all the 
;		requested data has been read, then a plot is created from the
;		stored engineering data.  The program can also be run in
;		realtime mode where data is contiously displayed as it is
;		read in.  The type of the data to be plotted as well as the
;		time range covered by the plot is specified by using keywords.
;               
; Use         : 	GISPLOT, [keywords]
;    
; Inputs      : None.
;
; Opt. Inputs : None.
;               
; Outputs     : None.
;               
; Opt. Outputs: None.
;               
; Keywords    :	START	Start time for the plot.  The plot may actually begin
;			later if this time falls during a data gap.  This 
;			keyword should be set to a UTC time in any acceptable
;			text format.  This keyword can be used in realtime
;			mode to specify a start time earlier then the current
;			time. 
;		STOP	End time for the plot. This keyword should be set to
;			a UTC time in any acceptable text format.  This 
;			option is ignored if the RT keyword (realtime mode) is
;			set.
;		RAW	If set, then the raw counts from all four GIS 
;			is plotted out.  This is the defualt option.
;		ULD	If set, then the upper level discriminator counts from
;			all four GIS detectors is plotted out.
;		PROC	If set, then the processed counts from all four GIS 
;			is plotted out.
;		HVCUR	If set, then the HV current from all four GIS 
;			is plotted out.
;		PRINT	If set, the plot is printed out on the defualt printer
;			instead of being displayed on a terminal.  This option
;			is ingnored if the RT keyword (realtime mode) is set.
;		RT	If set, then realtime plotting will be performed. 
;			In this mode, plots are generated continuously as
;			as data is being read.  Otherwise, a single plot 
;			is created only after all the data has been 
;			processed.
;
; Calls       : GET_UTC
;		UTC2SEC
;		ANYTIM2UTC
;		GET_TM_FILE
;		READ_IWS_PKT
;		NEXT_TM_FILE
;		TAI2UTC
;		OBT2TAI
;		BYTESWAP
;		PS
;		MK_GIS_PLT
;		TM_CALIB
;		PSPLOT
;
; Common      : None.
;               
; Restrictions: None.
;               
; Side effects: There is currently no clean way to exit the program when
;		started in realtime (keyword RT) mode.  Use Ctrl-C instead.
;               
; Category    : Engineering.
;               
; Prev. Hist. : None.
;
; Written     : Ron Yurow, SM&A, 12 May 1999
;               
; Modified    : Version 1, Initial Release		RY, 12 May 1999
;
; Version     : Version 1, 12 May 1999
;-            

	PRO GISPLOT, START = start, 	$
	             STOP = stop,	$
	             RAW = raw,		$
	             ULD = uld,		$
	             PROC = proc,	$
	             HVCUR = hvcur,     $
	             PRINT = print,	$
	             RT = rt

;
; Check if a start time was passed to the program.  If it was not, then
; set the start time to current time.
;

	IF NOT KEYWORD_SET (start) THEN GET_UTC, start

;
; Make sure that we plot something.
;

	IF NOT KEYWORD_SET (raw) AND 		$
	   NOT KEYWORD_SET (uld) AND		$
	   NOT KEYWORD_SET (proc) AND		$
	   NOT KEYWORD_SET (hvcur) THEN raw = 1

;
; Check if a stop time was passed to the program.  If it was not, then
; set the flag sofc to 1.  Otherwise this flag will be set to 0
;

	sofc = 1

	IF KEYWORD_SET (stop) THEN BEGIN 

	   sofc = 0
	   sec_stop = UTC2SEC (ANYTIM2UTC (stop))

	ENDIF

;
; Convert the start time to seconds.  This will allow us to only display 
; data after the start time has been traversed.
;

	sec_start = UTC2SEC (ANYTIM2UTC (start))

;
; Make sure rt is set to 0, if realtime operations are not selected.  If 
; realtime mode is selected, then set print to 0 to disable printing and set
; sofc to 0 to make sure the program continues to the next file.    
;

	IF NOT KEYWORD_SET (rt) THEN BEGIN 
	   rt = 0
        ENDIF ELSE BEGIN
	   print = 0 
	   sofc = 0
	ENDELSE
;
; Set endtime to 0.  This variable will contain the estimated end time of the
; current telemetry file in seconds.
;

	endtime = 0

;
; Set interval to 100.  We will only use this variable if we are running
; in realtime, but it is easier just to set it anyway.
;

	interval = 20

;
; Set datapos to 0.  That is, we will start filling our arrays from the 
; beginning.  
;

	datapos = 0L

;
; Set datasize to the initial size of our dataset that we are going to store
; in our arrays.
;

	datasize = 1L

;
; Set datawinsize to the size of the maximum data window to display.  Note 
; this must be greater then the size specified by interval.  
;

	datawinsize = 1000

;
; Initialize all the arrays we will need.
;

	det_1_raw = intarr (datasize)
	det_2_raw = intarr (datasize)
	det_3_raw = intarr (datasize)
	det_4_raw = intarr (datasize)

	det_1_proc = intarr (datasize)
	det_2_proc = intarr (datasize)
	det_3_proc = intarr (datasize)
	det_4_proc = intarr (datasize)
	   
	det_1_uld = intarr (datasize)
	det_2_uld = intarr (datasize)
	det_3_uld = intarr (datasize)
	det_4_uld = intarr (datasize)

	det_1_cur = fltarr (datasize)
	det_2_cur = fltarr (datasize)
	det_3_cur = fltarr (datasize)
	det_4_cur = fltarr (datasize)

	GET_UTC, utc

	time = REPLICATE (utc, datasize)	

;
; Set file to the tm file that corresponds to start.
;

	path = GET_TM_FILE (ANYTIM2UTC (start), ENDTIME = endtime)

;
; Check if we found a file.  If we didn't, then print out an error message
; and return.
;

	IF path EQ '' THEN BEGIN

	   PRINT, 'Could not find file for time: ', FORMAT = '(a, $)
	   PRINT, UTC2STR (ANYTIME2UTC (start), /ECS, /TRUNCATE)

	   RETURN

	ENDIF


        PRINT, 'Reading file ', path

;
; Open the file.
;

	OPENR, dev, path, /GET_LUN, /NOSTDIO

;
; Reset the routine READ_IWS_PKT
;

        status = READ_IWS_PKT (dev, pkt, /reset)

redo_loop:

;
; Set error_count to 0.
;

	error_count = 0

;
; Set count to 0.  We will use this variable to keep track of where we are in
; in the plotting interval.
;

	count = 0L

;
; Semi-Inifinite loop.  This loop will continue until we hit a read error, we
; run out of files, we pass our stop time, or we come to the end of the data 
; inteval.
;

	WHILE NOT rt OR count LT interval DO BEGIN

loop_continue:

;
; Get a packet.
;

           status = READ_IWS_PKT (dev, pkt)

;
; Check if the packet is a housekeeping packet, if it is then trash it.
;

	   IF status EQ 2 THEN GOTO, loop_continue

;
; Check for an invalid checksum.  Again, trash the packet if we find it.
;

	   IF status EQ 1 THEN GOTO, loop_continue

;
; Check for EOF.  If s = 3 or s = 4 then we should go on to the next file.
;
	
	   IF status EQ 3 OR status EQ 4 THEN BEGIN

;
; Get the current time in UTC.	   
;

	      GET_UTC, utc

;
; Check if the current time is earlier then estimated end time of the file.
; if it is then we can not switch files.  Instead, we will wait a bit, and
; try againg.
;

	      IF UTC2SEC (utc) LT endtime THEN BEGIN

;
; Increment error_count.
;

	         error_count = error_count + 1 

;
; Wait for 5 seconds.
;

	         WAIT, 5.0
;
; Print a message telling the user what we are doing, but only if we have
; been waiting here ahwile.
;

	         IF error_count GT 6 THEN $
	            PRINT, 'Waiting for telemetry to resume...'

;
; Check if error_count is less then 12.  If it is then just go back to the
; beginning of the loop and try again.
;

	         IF error_count LT 12 THEN GOTO, loop_continue

;
; Otherwise, force rt to 0 and and exit the loop.
;

	         rt = 0
	         GOTO, loop_exit

	      ENDIF

;
; Check if the flag sofc (stop on file change) is set.  If it is, then we
; just exit the loop.
;

	      IF sofc THEN GOTO, loop_exit

;
; Close the previous file.
;

	      CLOSE, dev

;
; Set file to the next tm file in sequence.
;

	      path = NEXT_TM_FILE (path, ENDTIME = endtime)

;
; Check if we were able to find another file.  If we were not able to find 
; a new tm file, then force rt to 0 and quit out of the loop.
;


	      IF path EQ '' THEN BEGIN

	         rt = 0
	         GOTO, loop_exit

	      ENDIF

;
; Print a message to the user telling them what file we are reading.


              PRINT, 'Reading file ', path

;
; Open the file.
;

	      OPENR, dev, path, /NOSTDIO

;
; Reset the routine READ_IWS_PKT
;

              status = READ_IWS_PKT (dev, pkt, /reset)

;
; Go back to the beginning of the loop.
;

	      GOTO, loop_continue

	   ENDIF

;
; Check for Read Failure.  Just print an error message and quit in this case. 
;

	   IF status EQ 5 THEN BEGIN

	      PRINT, 'Error ', !ERROR, ' encountered while reading file.'
	      PRINT, !ERR_STRING

	      RETURN

	   ENDIF

;
; Reset error_count to 0.
;

	   error_count = 0

;
; Get the ID of packet that we just read
;

	   id = pkt (18)

;
; Make sure that it is an engineering B packet.  If it isn't then throw it
; away.
; 

	   IF id NE '0B'X THEN GOTO, loop_continue

;
; Get the Instrument Time 
;

	   t_time = TAI2UTC (OBT2TAI (pkt (12:17)))

;
; Convert the Instument Time into seconds.
;

	   s_time = UTC2SEC (t_time)

;
; Check if the Instrument Time is greater then the time to stop looking 
; at data.  If it is, then we will quit out of the loop.
;

	   IF NOT sofc AND NOT rt THEN BEGIN
	
	      IF s_time GT sec_stop THEN GOTO, loop_exit

	   ENDIF

;
; Check if the Instrument Time is less then the time to start looking
; at data.  If it is, then we just go back to the beginning of the loop.
;

	   IF s_time LT sec_start THEN GOTO, loop_continue

;
; Extract the raw events, processed events and ULD events from the packet for
; all four GIS detectors.
;

;
; Raw events.
;

	   t_det_1_raw = BYTESWAP (FIX (pkt, 176))
	   t_det_2_raw = BYTESWAP (FIX (pkt, 178))
	   t_det_3_raw = BYTESWAP (FIX (pkt, 180))
	   t_det_4_raw = BYTESWAP (FIX (pkt, 182))

;
; Processed events.
;

	   t_det_1_proc = BYTESWAP (FIX (pkt, 168))
	   t_det_2_proc = BYTESWAP (FIX (pkt, 170))
	   t_det_3_proc = BYTESWAP (FIX (pkt, 172))
	   t_det_4_proc = BYTESWAP (FIX (pkt, 174))

;
; ULD events.
;

	   t_det_1_uld = BYTESWAP (FIX (pkt, 184))
	   t_det_2_uld = BYTESWAP (FIX (pkt, 186))
	   t_det_3_uld = BYTESWAP (FIX (pkt, 188))
	   t_det_4_uld = BYTESWAP (FIX (pkt, 190))

;
; HV Currents
;

	   t_det_1_cur = pkt (133)
	   t_det_2_cur = pkt (132)
	   t_det_3_cur = pkt (135)
	   t_det_4_cur = pkt (134)

;
; Add the events into their appropiate arrays.  If datapos less then datasize,
; then insert each datapoint at datapos.  Otherwise, append each datapoint
; to the end of the array.
;

	   IF datapos LT datasize THEN BEGIN

	      det_1_raw (datapos) = t_det_1_raw
	      det_2_raw (datapos) = t_det_2_raw
	      det_3_raw (datapos) = t_det_3_raw
	      det_4_raw (datapos) = t_det_4_raw

	      det_1_proc (datapos) = t_det_1_proc
	      det_2_proc (datapos) = t_det_2_proc
	      det_3_proc (datapos) = t_det_3_proc
	      det_4_proc (datapos) = t_det_4_proc
	   
	      det_1_uld (datapos) = t_det_1_uld
	      det_2_uld (datapos) = t_det_2_uld
	      det_3_uld (datapos) = t_det_3_uld
	      det_4_uld (datapos) = t_det_4_uld

	      det_1_cur (datapos) = t_det_1_cur
	      det_2_cur (datapos) = t_det_2_cur
	      det_3_cur (datapos) = t_det_3_cur
	      det_4_cur (datapos) = t_det_4_cur

	      time (datapos) = t_time

;
; Increment datapos by 1.
;

	      datapos = datapos + 1

	   ENDIF ELSE BEGIN 

	      det_1_raw = [det_1_raw, t_det_1_raw]
	      det_2_raw = [det_2_raw, t_det_2_raw]
	      det_3_raw = [det_3_raw, t_det_3_raw]
	      det_4_raw = [det_4_raw, t_det_4_raw]

	      det_1_proc = [det_1_proc, t_det_1_proc]
	      det_2_proc = [det_2_proc, t_det_2_proc]
	      det_3_proc = [det_3_proc, t_det_3_proc]
	      det_4_proc = [det_4_proc, t_det_4_proc]
	   
	      det_1_uld = [det_1_uld, t_det_1_uld]
	      det_2_uld = [det_2_uld, t_det_2_uld]
	      det_3_uld = [det_3_uld, t_det_3_uld]
	      det_4_uld = [det_4_uld, t_det_4_uld]

	      det_1_cur = [det_1_cur, t_det_1_cur]
	      det_2_cur = [det_2_cur, t_det_2_cur]
	      det_3_cur = [det_3_cur, t_det_3_cur]
	      det_4_cur = [det_4_cur, t_det_4_cur]

	      time = [time, t_time]

;
; Increment datasize by 1.
;

	      datasize = datasize + 1

;
; Increment datapos by 1.
;

	      datapos = datapos + 1

	   ENDELSE

;
; Increment count by 1.
;

	   count = count + 1

	ENDWHILE

loop_exit:

;
; Close the current file.  We will do this, however, only if we are not
; running in realtime.
;

	IF rt EQ 0 THEN CLOSE, dev

;
; Check if we are running in real time mode, if we are and datapos is
; greater then or equal to datawinsize, then adjust each array so that 
; first interval elements are shifted to the top of the array and datapos
; is adjust down by the same interval.
;

	IF rt AND datapos GE datawinsize THEN BEGIN

	   sinterval = interval * (-1)

	   det_1_raw = SHIFT (det_1_raw, sinterval)
	   det_2_raw = SHIFT (det_2_raw, sinterval) 
	   det_3_raw = SHIFT (det_3_raw, sinterval)
	   det_4_raw = SHIFT (det_4_raw, sinterval)

	   det_1_proc = SHIFT (det_1_proc, sinterval)
	   det_2_proc = SHIFT (det_2_proc, sinterval)
	   det_3_proc = SHIFT (det_3_proc, sinterval) 
	   det_4_proc = SHIFT (det_4_proc, sinterval) 

	   det_1_uld = SHIFT (det_1_uld, sinterval)
	   det_2_uld = SHIFT (det_2_uld, sinterval) 
	   det_3_uld = SHIFT (det_3_uld, sinterval) 
	   det_4_uld = SHIFT (det_4_uld, sinterval) 

	   det_1_cur = SHIFT (det_1_cur, sinterval) 
	   det_2_cur = SHIFT (det_2_cur, sinterval)
	   det_3_cur = SHIFT (det_3_cur, sinterval) 
	   det_4_cur = SHIFT (det_4_cur, sinterval) 

	   time = SHIFT (time, sinterval) 

	   datapos = datapos - interval

	ENDIF

;
; Check if we should make a hard copy.
;

	IF KEYWORD_SET (print) THEN PS

;
; Do the plots
;

;
; Check if we are supposed to make a plot of GIS raw counts.
;

	IF KEYWORD_SET (raw) THEN BEGIN

	   pltname = 'GIS Raw Counts For All Four Detectors'
	   y_axis_label = 'Raw Counts'

;
; Do the plot.
;

	   MK_GIS_PLT, det_1_raw (0:datapos - 1),  	$
	               det_2_raw (0:datapos - 1),  	$
	               det_3_raw (0:datapos - 1),  	$
	               det_4_raw (0:datapos - 1), 	$
	               TIME = time (0:datapos - 1),	$
	               PLTNAME = pltname,		$
	               Y_AXIS_LABEL = y_axis_label,	$
	               PRINT = print

	ENDIF

;
; Check if we are supposed to make a plot of GIS processed counts.
;

	IF KEYWORD_SET (proc) THEN BEGIN

	   pltname = 'GIS Processed Counts For All Four Detectors'
	   y_axis_label = 'Processed Counts'

;
; Do the plot.
;

	   MK_GIS_PLT, det_1_proc (0:datapos - 1),  	$
	               det_2_proc (0:datapos - 1),  	$
	               det_3_proc (0:datapos - 1),  	$
	               det_4_proc (0:datapos - 1), 	$
	               TIME = time (0:datapos - 1),	$
	               PLTNAME = pltname,		$
	               Y_AXIS_LABEL = y_axis_label,	$
	               PRINT = print

	ENDIF

;
; Check if we are supposed to make a plot of GIS ULD counts.
;

	IF KEYWORD_SET (uld) THEN BEGIN

	   pltname = 'GIS ULD Counts For All Four Detectors'
	   y_axis_label = 'ULD Counts'

;
; Do the plot.
;

	   MK_GIS_PLT, det_1_uld (0:datapos - 1),  	$
	               det_2_uld (0:datapos - 1),  	$
	               det_3_uld (0:datapos - 1),  	$
	               det_4_uld (0:datapos - 1), 	$
	               TIME = time (0:datapos - 1),	$
	               PLTNAME = pltname,		$
	               Y_AXIS_LABEL = y_axis_label,	$
	               PRINT = print

	ENDIF

;
; Check if we supposed to make a plot of GIS MCP current draw.
;

	IF KEYWORD_SET (hvcur) THEN BEGIN

;
; Set the endpoints interval that we wish to calibrate.  
;

	   c0 = datapos - count
	   clast = datapos - 1

;
; Calibrate the raw current values.
;

	   det_1_cur (c0:clast) = TM_CALIB ("BIGMCP1", det_1_cur (c0:clast))  
	   det_2_cur (c0:clast) = TM_CALIB ("BIGMCP2", det_2_cur (c0:clast))  
	   det_3_cur (c0:clast) = TM_CALIB ("BIGMCP3", det_3_cur (c0:clast))  
	   det_4_cur (c0:clast) = TM_CALIB ("BIGMCP4", det_4_cur (c0:clast))  


	   PLTNAME = 'GIS MCP HV Current Draw For All Four Detectors'
	   Y_AXIS_LABEL = 'MCP Current (mA)'

;
; Do the plot.
;

	   MK_GIS_PLT, det_1_cur (0:datapos - 1),  	$
	               det_2_cur (0:datapos - 1),  	$
	               det_3_cur (0:datapos - 1),  	$
	               det_4_cur (0:datapos - 1), 	$
	               TIME = time (0:datapos - 1),	$
	               PLTNAME = pltname,		$
	               Y_AXIS_LABEL = y_axis_label, 	$
	               D1_LABEL = 'BIGMCP1',		$
	               D2_LABEL = 'BIGMCP2',		$
	               D3_LABEL = 'BIGMCP3',		$
	               D4_LABEL = 'BIGMCP4',		$
	               UNITS = 'mAmps',			$
	               PRINT = print

	ENDIF


;
; Check if we should make a hard copy.  If so, print it out.
;

;	IF KEYWORD_SET (print) THEN PSPLOT
	IF KEYWORD_SET (print) THEN PSCLOSE

;
; Check if we exited the last loop due to an error.  If we did then
; print out an error message
;

	IF status EQ 5 THEN BEGIN

	   PRINT, "FATAL I/O Error: ", !ERR
	   PRINT, !ERR_STRING

	   RETURN

	ENDIF

;
; Check if we are in realtime mode.  If we are, then lets go back
; and collect more data.
;

	IF rt THEN GOTO, redo_loop

	RETURN

	END