;+
; Project     : SOHO - CDS     
;                   
; Name        : MK_GISPLOT_LBL
;               
; Purpose     : Create text strings to label tick marks for MK_GIS_PLT
;               
; Explanation : This function, which is only designed to work with MK_GIS_PLT
;		is used to convert time values into text strings which can be
;		used as tick mark labels.  The format to use when creating the
;		text strings is determined by the value of label_type.  The
;		legal values for this parameter are:
;		
;		1:	Seconds
;		2:	Minutes
;		3:	Hours
;		4:	Days
;		5:	Months
;		6:	Years
;
;		Note that Months and Years are not yet implemented.
;
;               
; Use         : 	MK_GISPLOT_LBL, time, label_type
;    
; Inputs      : time		Time value expressed in seconds to convert to
;				text string.
;		label_type	The format to use when creating the text label.
;
; Opt. Inputs : None.
;               
; Outputs     : None.
;               
; Opt. Outputs: None.
;               
; Keywords    :	None.
;
; Return Value: The text string to use as a tick mark label.
;
; Calls       : UTC2STR
;		SEC2UTC
;
; Common      : None.
;               
; Restrictions: None.
;               
; Side effects: None.
;               
; Category    : Engineering.
;               
; Prev. Hist. : None.
;
; Written     : Ron Yurow, SM&A, 12 May 1999
;               
; Modified    : Version 1, Initial Release		RY, 12 May 1999
;
; Version     : Version 1, 12 May 1999
;-            

	FUNCTION MK_GISPLOT_LBL, time, label_type

;
; Convert the time in seconds to a text string.
;

	timestring = UTC2STR (SEC2UTC (time)) 

;
; Determine what type of text label to create.  Base on the label type, 
; create a label from the text string in timestring.
; 

	CASE label_type OF

	   1: BEGIN		; Seconds

	       label = STRMID (timestring, 11, 8) 

	   END

	   2: BEGIN		; Minutes

	       label = STRMID (timestring, 11, 5)

	   END

	   3: BEGIN		; Hours

	       label = STRMID (timestring, 5, 2) 
	       label = label + '/' + STRMID (timestring, 8, 2)
	       label = label + '!C' + STRMID (timestring, 11, 5)
 
 	   END

	   4: BEGIN		; Days

	       label = STRMID (timestring, 5, 2)
	       label = label + '/' + STRMID (timestring, 8, 2)

	   END


	ENDCASE

;
; Were done.  Return the text label.
;

	RETURN, label

	END

;+
; Project     : SOHO - CDS     
;                   
; Name        : MK_GIS_PLT
;               
; Purpose     : Plot engineering data from all four GIS detectors on one plot.
;               
; Explanation : This is a general routine for displaying engineering data from
;		all four GIS detectors on a single plot.  The plot is designed
;		to look like a strip chart, with tick marks on the Y axis
;		being used only to give a relative indication of the actual
;		data values being plotted.  The X axis is marked in regular
;		time increments, which are appropiate to the temporal range
;		of the data set.  Currently, up to 8 days worth of data can
;		be plotted.  Although this routine is designed to work with 
;		data from the GIS detectors, it could be used with other
;		engineering data as well.    
;               
; Use         : 	MK_GIS_PLT, data1, data2, data3, data4, [keywords]
;    
; Inputs      : dt1	Engineering data from GIS detector 1. 
;		dt2	Engineering data from GIS detector 2.
;		dt3	Engineering data from GIS detector 3.
;		dt4	Engineering data from GIS detector 4.
;
; Opt. Inputs : None.
;               
; Outputs     : None, but procedure creates a plot.
;               
; Opt. Outputs: None.
;               
; Keywords    :	TIME		UTC time values to plot detector data against.
;				This must be included in any call to this 
;				procedure.
;		PLTNAME		Text string to use for plot title.
;		Y_AXIS_LABEL	Text string to use as general Y axis label.
;		RANGE		Range of data values to plot.  Data values
;				outside of this range will plotted in another
;				detectors graph space.
;		D1_LABEL	Text string to label data from detector 1.
;		D2_LABEL	Text string to label data from detector 2.
;		D3_LABEL	Text string to label data from detector 3.
;		D4_LABEL	Text string to label data from detector 4.
;		UNITS 		Text string describing the units which the
;				data values represent. 
;		PRINT		When set, spacing between certain elements 
;				is adjusted to allow for differences between
;				plotting to a printer and plotting to the 
;				display.
;
; Calls       : ANYTIM2UTC
;		UTC2SEC
;		MK_GISPLOT_LBL
;
; Common      : None.
;               
; Restrictions: All four datasets must use the same time scale.  The time
;		values must be included in any call to this procedure.  
;               
; Side effects: None
;               
; Category    : Engineering.
;               
; Prev. Hist. : None
;
; Written     : Ron Yurow, SM&A, 13 May 1999
;               
; Modified    : Version 1, Initial Release		RY, 13 May 1999
;
; Version     : Version 1, 13 May 1999
;-            

	PRO MK_GIS_PLT, dt1, 					$
	                dt2, 					$
	                dt3, 					$
	                dt4, 					$
	                TIME = time,				$
	                PLTNAME = pltname,			$
	                Y_AXIS_LABEL = y_axis_label,		$
	                RANGE = range, 				$
	                D1_LABEL = d1_label,			$
	                D2_LABEL = d2_label,			$
	                D3_LABEL = d3_label,			$
	                D4_LABEL = d4_label,			$
	                UNITS = units,				$
	                PRINT = print

;
; Check if label names for each of the graphs have been passed to us.  If any
; have not, then give them default labels.
;

	IF NOT KEYWORD_SET (d1_label) THEN d1_label = 'Detector 1'
	IF NOT KEYWORD_SET (d2_label) THEN d2_label = 'Detector 2'
	IF NOT KEYWORD_SET (d3_label) THEN d3_label = 'Detector 3'
	IF NOT KEYWORD_SET (d4_label) THEN d4_label = 'Detector 4'

;
; Check if string representing the units of whatever we are being asked to 
; plot has been passed to us.  If has not been passed to us, then we will set
; it to a defualt value.
;

	IF NOT KEYWORD_SET (units) THEN units = 'Counts'

;
; Check if a string representing the plot title has been passed to us. If 
; it has not been passed to us, then we will set it to a defualt value.
;

	IF NOT KEYWORD_SET (pltname) THEN pltname = 'GIS Detector Plot'

;
; Check if a string representing the Y axis label has been passed to us. If 
; it has not been passed to us, then we will set it to a defualt value.
;

	IF NOT KEYWORD_SET (y_axis_label) THEN y_axis_label = 'GIS Counts'

;
; Set the position for certain elements of the plot. 
;

	LABEL_X_POS = 0.12

	AXIS_LABEL_X_POS = 0.06
	AXIS_LABEL_Y_POS = 0.5

	LEGEND_X_POS = 0.5
	LEGEND_Y_POS = 0.016
	Y_LABEL_DROP = 0.043
	LABEL_SIZE = 0.8

	IF KEYWORD_SET (print) THEN BEGIN

	   LABEL_X_POS = .08
	   AXIS_LABEL_X_POS = .04
	   Y_LABEL_DROP = 0.025
	   LEGEND_Y_POS = 0.011
	   LABEL_SIZE = 1.0

	ENDIF
;
; Find the maximum values for each of the data sets we supposed to plot.
;

	maxdt1 = MAX (dt1)
	maxdt2 = MAX (dt2)
	maxdt3 = MAX (dt3)
	maxdt4 = MAX (dt4)

;
; Check if the RANGE keyword is set, if it was not then set range to a 
; value based on the data that we are being asked to plot.
;

	IF NOT KEYWORD_SET (range) THEN BEGIN

	   range = MAX ([maxdt1, maxdt2, maxdt3, maxdt4]) * 1.1
	   range = range > 20

	ENDIF

;
; Set the offset to the number of the datapoints that we will offset the 
; plots from the X axis. 
;

	offset =  LONG (range / 4)

;
; Set the maximum range of counts that we will plot.  
;

	plot_range = [0, 4L * range + offset]

;
; Convert time to an array of UTC time structures.
;

	utctime = ANYTIM2UTC (time)

;
; Convert the array of time structures to an array of seconds is MJD0
;

	sectime = UTC2SEC (utctime)

;
; Set start to the number of seconds since MJD0 of the start time of the plot.
; Set stop to the number of secons since MJD0 of the end time of the plot.
; Set len to the number of seconds that will be spanned by the plot.
;

	stop = sectime (N_ELEMENTS (sectime) - 1)
	start = sectime (0) 
	len = (stop - start) + 1

;
; Define our margins for the plot.  This will affect where put tick marks and
; labels, etc.  
;

	lmargin = FIX (len / 9)
	rmargin = len - FIX (len / 9) 

;
; Adjust the array times so that are all offsets from the initial time in the
; array.
;

	sectime = sectime - start

;
; Decide what interval to use for major (labeled) ticks.  We will make this 
; decision based on the length of time we are being asked to plot out.  
;
; Each case will set the following varibles: min_ticks to the number of minor
; tick marks, interval to the interval between tick marks in seconds and 
; label_type to the time unit to use on the label.
;
; The possible values for label_type are as follows:
;		
;		1:	Seconds
;		2:	Minutes
;		3:	Hours
;		4:	Days
;		5:	Months
;		6:	Years
;

	CASE 1 OF

	   (len LT 8):   BEGIN		; Less then 8 seconds.

	      min_ticks = 1
	      interval = 1L
	      label_type = 1

	   END

	   (len LT 16):  BEGIN  	; Less then 16 seconds.

	      min_ticks = 2
	      interval = 2
	      label_type = 1

	   END

	   (len LT 40):  BEGIN  	; Less then 40 seconds.

	      min_ticks = 5
	      interval = 5L
	      label_type = 1

	   END

	   (len LT 80):  BEGIN  	; Less then 80 seconds.

	      min_ticks = 10
	      interval = 10L
	      label_type = 1

	   END

	   (len LT 120):  BEGIN  	; Less then 2 minutes.

	      min_ticks = 3
	      interval = 15L
	      label_type = 1

	   END

	   (len LT 240):  BEGIN		; Less then 4 minutes.

	      min_ticks = 6
	      interval = 30L
	      label_type = 1

	   END

	   (len LT 480):  BEGIN		; Less then 8 minutes.

	      min_ticks = 1
	      interval = 60L
	      label_type = 2

	   END

	   (len LT 960):  BEGIN		; Less then 16 minutes.

	      min_ticks = 2
	      interval = 120L
	      label_type = 2

	   END

	   (len LT 2400):  BEGIN	; Less then 40 minutes.

	      min_ticks = 5
	      interval = 300L
	      label_type = 2

	   END

	   (len LT 4800):  BEGIN	; Less then 80 minutes.

	      min_ticks = 10
	      interval = 600L
	      label_type = 2

	   END

	   (len LT 7200):  BEGIN	; Less then 2 hours.

	      min_ticks = 3
	      interval = 900L
	      label_type = 2

	   END

	   (len LT 14400):  BEGIN	; Less then 4 hours.

	      min_ticks = 6
	      interval = 1800L
	      label_type = 2

	   END

	   (len LT 28800):  BEGIN	; Less then 8 hours.

	      min_ticks = 1
	      interval = 3600L
	      label_type = 3

	   END

	   (len LT 57600):  BEGIN	; Less then 16 hours.

	      min_ticks = 2
	      interval = 7200L
	      label_type = 3

	   END

	   (len LT 115200):  BEGIN	; Less then 32 hours.

	      min_ticks = 4
	      interval = 14400L
	      label_type = 3

	   END

	   (len LT 230400):  BEGIN	; Less then 64 hours.

	      min_ticks = 8
	      interval = 28800L
	      label_type = 3

	   END

	   (len LT 691200):  BEGIN	; Less then 8 days.

	      min_ticks = 1
	      interval = 86400L
	      label_type = 4

	   END

	ENDCASE

;
; Create the array tnames to hold the name of each tick in the plot.  We will
; set the array size to the largest possible number of major ticks for this
; data set.
;

	tnames = strarr (FIX (len / interval) + 3)

;
; Create the array tloc to hold the location (along the x axis) of all of 
; the tick marks.  Create an array tblanknames to hold an array of blank
; strings.
;

	tloc = lonarr (N_ELEMENTS (tnames))
	tnamesblank = REPLICATE (" ", N_ELEMENTS (tnames)) 


;
; Set firsttic to the time (in seconds) that the first tic should be placed
; at.  This may (but probably won't) be the start of the data set that we 
; are plotiing.  The value for firsttic is chosen to ensure that all the 
; time stamps occur on even boundries.
;

	firsttic = start
	sec = LONG (start - UTC2SEC ({MJD: 47892, TIME: 0}))

	IF  sec MOD interval NE 0 THEN BEGIN

	    bump = (((sec + interval - 1) / interval) * interval) - sec
	    firsttic = firsttic + bump  

	ENDIF

;
; Set xloc to the location of the first tick mark as an offset in seconds 
; from the beginning of our data set.  xloc will hold the location of the 
; next tick mark along the x axis.
;

	xloc = firsttic - start

;
; Set mticks to 0.  We will use mticks to keep track of the number of major
; tick marks that we need.
;

        mticks = 0

;
; Make the ticknames for the plot.  Each name will in the form of a timestamp,
; which form depends on the value of label_type.  The loop ends when tick
; marks for the entire plot have been made.
;

	WHILE xloc LT len DO BEGIN

;
; Call MK_GISPLOT_LBL to create the label for the current tick mark.
;

	   label = MK_GISPLOT_LBL (start + xloc, label_type)

;
; Add the location of the tick mark to the array tloc.   Add the label for
; the tick mark to tnames.
;

	   tloc (mticks) = xloc
	   tnames (mticks) = label

;
; Check if the tick mark falls in the margin.  If it does, then set the 
; label to be the empty string.
;

	   IF xloc LT lmargin THEN tnames (mticks) = " "

	   IF xloc GT rmargin THEN tnames (mticks) = " "

;
; Move xloc to the location of the next mark and increment mticks.
; 
	   
	   xloc = xloc + interval
           mticks = mticks + 1	


	ENDWHILE

;
; Make labels for the left and right edges of the graph.  These will always
; have labels.
; 


	ledge = MK_GISPLOT_LBL (start, label_type)
	redge = MK_GISPLOT_LBL (start + len, label_type)

;
; Set the interval between tick marks on the y axis.  The interval size is 
; based on the range values being plotted.
;

	CASE 1 OF

	   (range LT 10) :    yinterval = 1L
	   (range LT 20) :    yinterval = 2L
	   (range LT 50) :    yinterval = 5L
	   (range LT 100) :   yinterval = 10L
	   (range LT 250) :   yinterval = 25L
	   (range LT 500) :   yinterval = 50L
	   (range LT 1000) :  yinterval = 100L
	   (range LT 2000) :  yinterval = 200L
	   (range LT 5000) :  yinterval = 500L
	   (range LT 10000) : yinterval = 1000L
	   (range LT 20000) : yinterval = 2000L
	   (range LT 50000) : yinterval = 5000L
	   ELSE :             yinterval = 10000L

	ENDCASE

;
; Create a string to tell the user how the Y axis is scaled.  We will write
; this string at the bottom of the plot.
;
	
	fmtstr = '("Each Y-Axis Tick Mark = ", I0, " ", A)'
	legend = STRING (yinterval, units, FORMAT = fmtstr)

;
; Calculate the number of tick marks along the y axis for each plot that
; we are making.
;

	yticks = (range / yinterval) - 1L

;
; Do the plotting...
;	

	PLOT, sectime, 					$
	      dt4 + offset + 3L * range, 		$
	      YSTYLE = 5,				$
	      YRANGE = plot_range,			$
	      XSTYLE = 1, 				$
	      XTICKS = mticks - 1,			$
	      XTICKV = tloc,				$
	      XTICKNAME = tnamesblank,			$
	      XMINOR = min_ticks,			$
	      TITLE = pltname

	OPLOT, sectime, dt3 + offset + 2L * range
	OPLOT, sectime, dt2 + offset + range
	OPLOT, sectime, dt1 + offset


;
; Convert range to the normalized coordinate system.  Also covert offset
; as well.
;

	y_axis_pos = (CONVERT_COORD (0, 0, /TO_NORMAL)) (1)

	nrange = (CONVERT_COORD (0, range, /TO_NORMAL)) (1) - y_axis_pos

	noffset = (CONVERT_COORD (0, offset, /TO_NORMAL)) (1)

;
; Draw vertical lines along major tick marks.
;

	FOR i = 0, mticks - 1 DO BEGIN

;
; Draw the dotted line itself.
;

	   PLOTS, [tloc (i), tloc (i)], plot_range, LINESTYLE = 1

;
; Set x_pos to the x position of the line in normal coordinates.
;

	   x_pos = (CONVERT_COORD (tloc (i), 0, /TO_NORMAL)) (0)


;
; Write the correct label at the left edge of the graph.
;

	    XYOUTS, x_pos, 					$
	            y_axis_pos - Y_LABEL_DROP,			$
	            tnames (i),					$
	            ALIGNMENT = 0.5, 				$
	            SIZE = LABEL_SIZE,				$
                    /NORMAL

;
; Draw tick marks along the the line we just drew.  We will do this in 
; four groups, one for the output of each detecotor we are ploting.
;
	   
	   FOR graph = 0, 3 DO BEGIN

	      FOR t = 0, yticks - 1 DO BEGIN

	         y_pos = FLOAT (yinterval * t) * nrange / FLOAT (range) + $ 
			 graph * nrange + 				  $
			 noffset 

	         PLOTS, [x_pos - 0.005, x_pos + .005],		$ 
	                [y_pos, y_pos],				$
	                LINESTYLE = 1,	 			$
	                /NORMAL

	      ENDFOR

	   ENDFOR 

	ENDFOR

;
; Draw vertical lines at the edges of the graphs.
;

	FOR i = 0, 1 DO BEGIN

;
; Determine which end of the graph we are doing and set x_pos appropiately.
; If i = 0, then we are doing the left side, otherwise we are doing the right.
;

	   IF i EQ 0 THEN BEGIN

;
; Set x_pos to the x position of the left edge in normal coordinates.
;

	      x_pos = (CONVERT_COORD (0, 0, /TO_NORMAL)) (0)

;
; Write the correct label at the left edge of the graph.
;

	      XYOUTS, x_pos, 					$
	              y_axis_pos - Y_LABEL_DROP,		$
	              ledge,					$
	              ALIGNMENT = 0.5, 				$
	              SIZE = LABEL_SIZE,			$
                      /NORMAL

	   ENDIF ELSE BEGIN

;
; Set x_pos to the x position of the right edge in normal coordinates.
;

	      x_pos = (CONVERT_COORD (len - 1, 0, /TO_NORMAL)) (0)

;
; Write the correct label at the right edge of the graph.
;

	      XYOUTS, x_pos, 					$
	              y_axis_pos - Y_LABEL_DROP,		$
	              redge,					$
	              ALIGNMENT = 0.5, 				$
	              SIZE = LABEL_SIZE,			$
                      /NORMAL

	   ENDELSE

;
; Draw the line itself.
;

	   PLOTS, [x_pos, x_pos], 				$
	          [y_axis_pos, noffset + 4.0 * nrange], 	$
	          /NORMAL

;
; Draw tick marks along the the line we just drew.  We will do this in 
; four groups, one for the output of each detector we are ploting.
;
	   
	   FOR graph = 0, 3 DO BEGIN

	      FOR t = 0, yticks - 1 DO BEGIN

	         y_pos = FLOAT (yinterval * t) * nrange / FLOAT (range) + $ 
			 graph * nrange + 				  $
			 noffset 

	         PLOTS, [x_pos - 0.005, x_pos + .005],		$ 
	                [y_pos, y_pos],				$
	                LINESTYLE = 0,	 			$
	                /NORMAL

	      ENDFOR

	   ENDFOR 

	ENDFOR

;
; Write a  bunch of text strings onto the graph, ie. the y-axis
; label, the detector labels, etc.
;
	
	XYOUTS, AXIS_LABEL_X_POS, 				$
	        AXIS_LABEL_Y_POS, 				$
	        y_axis_label, 					$
                ALIGNMENT = 0.5, 				$
                /NORMAL, 					$
                ORIENTATION = 90  

	XYOUTS, LEGEND_X_POS, 					$
	        LEGEND_Y_POS, 					$
	        legend, 					$
                ALIGNMENT = 0.5, 				$
                /NORMAL

	XYOUTS, LABEL_X_POS, 					$
	        nrange * 0.4 + noffset, 			$
	        D1_LABEL, 					$
	        ALIGNMENT = 0.5,				$	 
	        /NORMAL, 					$
	        ORIENTATION = 90 

	XYOUTS, LABEL_X_POS, 					$
	        nrange * 1.4 + noffset,				$
	        D2_LABEL, 					$
	        ALIGNMENT = 0.5,				$	 
	        /NORMAL, 					$
	        ORIENTATION = 90 

	XYOUTS, LABEL_X_POS, 					$
                nrange * 2.4 + noffset,				$
	        D3_LABEL, 					$
	        ALIGNMENT = 0.5, 				$
	        /NORMAL, 					$
	        ORIENTATION = 90 

	XYOUTS, LABEL_X_POS, 					$ 
	        nrange * 3.4 + noffset,				$
	        D4_LABEL, 					$
	        ALIGNMENT = 0.5, 				$
	        /NORMAL, 					$
	        ORIENTATION = 90 

;
; Were done.
;

	RETURN

	END




