;+
; Project     : SOHO - CDS     
;                   
; Name        : VIEW_RAW.PRO
;               
; Purpose     : To plot and check raw GIS data and fit LUT parameters
;               
;
; Explanation : Tool to allow the user to analyse GIS raw data and to 
;               create GIS LUT parameters.  These parameters are required
;               by the on board software to process GIS spectral data.
;
; Use         : IDL> view_raw [, file=rawfile, params=LUT_params, $
;				raw_id=raw_id, sumplot, gset_id=gset_id]
;    
; Inputs      : None.
;
; Opt. Inputs : none
;
; Outputs     : LUT_params.txt file with the fitted parameters
;               
; Opt. Outputs: sumplot		= a 2-D histogram array of the spiral plot;
;			no further processing performed, and procedure exits.
;               
; Keywords    : file 		= name of raw GIS data file
;		params		= original LUT params (as an array)
;		raw_id		= raw file ID (use instead of filename)
;		gset_id		= use parameters from this GSET ID
;
; Calls       : gis_raw_header
;
; Common      : VIEW_RAW_MYLUT containing LUT parameter structure for mylut 
;				to use:
;		lut_str 	=  structure LUT_str containing LUT parameters
;               
; Restrictions: None
;               
; Side effects: !
;               
; Category    : GIS, science data
;               
; Prev. Hist. : None
;
; Written     : A Breeveld, MSSL/UCL, 1994
;            
; Modified    : v1.0 ERB, MSSL/UCL, 29 January 1995, Made into a routine, 
;			and tidied up a little
;		     AAB added routine to write in gset LUTs and check them, 
;			March '96
;		     AAB added optional output of LUT params in structure
;		    	March '96
;		     ERB, MSSL/UCL, added file filter option, 24 April 1996
;		     ERB, MSSL/UCL, added xloadct option, 3 June 1996
;		     ERB, MSSL.UCL, added getenv to pickfile
;		v3.5 ERB, MSSL/UCL, 28 Sep 99, added /block to xloadct calls
;		v4   erb, MSSL/UCL, 19 Oct 99, use raw ids and GSET ids
;		v4.1 erb, MSSL/UCL, 11 Jan 2000, released
;		Version 5, 09-May-2003, William Thompson, GSFC
;			Use ssw_strsplit instead of strsplit
;
; Version     : Version 5, 09-May-2003
;-            


;******* VIEW_RAW ******
; A program to plot raw files and fit spiral parameters to the data.
; Modularised!


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;******* DATAIN *******


PRO vraw_datain, whole, rawfile, head_str
	data=bytarr(2,256)
	header=bytarr(512)
	whole=0
	lun = 0

;
; check if filename already supplied
;
	if rawfile eq '' then rawfile = '*raw'
;
	if (strpos(rawfile, '*') ge 0) then begin
		filter = rawfile
		rawfile = ''
		RAWFILE = pickfile(/read, /must_exist, $
				path=getenv('CDS_GIS_RAW'), filter=filter)
		if (rawfile eq '') then return
	endif else begin
		if strpos(rawfile, '/') lt 0 then $
			rawfile = getenv('CDS_GIS_RAW') + '/' + rawfile
	endelse
;
firstread:
	on_ioerror, open_error
	openr, lun, /get_lun, RAWFILE
	on_ioerror, null
	print, rawfile
;
;  read it all in
;
	readu, lun, header	; read header
	head_str = gis_raw_header (header)
;
;  read in first data
;
	on_ioerror, cont
	readu, lun, data
	while (1 eq 1) do begin
		result=size(whole)
		if result(0) eq 0 then whole = transpose(data) else $
			whole=[whole,transpose(data)]
		readu,lun,data
	endwhile
cont:
	on_ioerror, null
	free_lun, lun		; close the file
;
	whole=[whole,transpose(data)]		; add on last bit
;
;  Try to get rid of non zero wholes at end of last block of data
;
	val=0
	i=n_elements(whole(*,0))
	while(val eq 0) do begin
		i=i-1
		val=whole(i,0)+whole(i,1)
	endwhile
	whole=whole(0:i,*)

	RETURN
;
open_error:
	message, /continue, "ERROR - File can't be found/opened"
	retall
END


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;***** VRAW_PRAW *****

PRO vraw_praw, X,Y,rawfile
	print, "Plot of raw data"
	plot,X,Y,psym=3,xrange=[0,256],yrange=[0,256], $
		title='VIEW_RAW normalized plot: '+RAWFILE, $
		charsize=1.0
	RETURN
END

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;***** PRTH *****

PRO vraw_prth, r,dth,title_text
	print,'plotting angle versus radius'
	plot,r,dth,psym=3,xrange=[0,150],yrange=[0,400],title=title_text, $
		charsize = 1.0
	RETURN
END

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;***** PDRTH *****

PRO vraw_pdrth,  r,dth,title_text
; Plots same as prth.pro, but expands angle axis up to 720 degrees.

	edth=dth+360
	ddth=[dth,edth]
	dr=[r,r]
	plot,dr,ddth,psym=3,xrange=[0,150],yrange=[0,720],title=title_text, $
		charsize = 1.0
	RETURN
END

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;***** PXY *****

PRO vraw_pxy, XX,YY,rawfile

	print,"Here is the xy plot"
	plot,XX,YY,psym=3,xrange=[0,0],yrange=[0,0], $
		title='VIEW_RAW spiral plot: '+RAWFILE, $
		charsize = 1.0
RETURN
END

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;***** PRTHCON *****

PRO vraw_prthcon, r,dth,all,intensity 
	print,'generating contour data'

; need to do this for statement in order to do the intensity later on...
	if (intensity eq 'n') then begin
	 for i=0L,(n_elements(r)-1) do all(r(i),dth(i))=all(r(i),dth(i))+1
	endif
	contour,all,/noerase
	intensity='y'
	RETURN
END

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;***** CHGAIN *****

PRO vraw_chgain, xgain,ygain,zgain,sadd,sum,X,Y,XX,YY,r,dth

;	Procedure for changing gains and offsets

	gain=" "
	print,'Please give gain for X (range 0.1 - 2.0): (default = ',xgain,')'
	read, "",gain
	if (strlen(gain) ne 0) then xgain=float(gain) > 0.1  < 2.0

	print,'Please give gain for Y (range 0.1 - 2.0): (default = ',ygain,')'
	read, "",gain
	if (strlen(gain) ne 0) then ygain=float(gain) > 0.1  < 2.0

	print,'Please give gain for Z (range 0.1 - 2.0): (default = ',zgain,')'
	read, "",gain
	if (strlen(gain) ne 0) then zgain=float(gain) > 0.1  < 2.0

	print,$
 'Please give offset for sum (range -460 - +50 even numbers only): ' + $
 '(default = ',sadd,')'
	read, "",gain
	if (strlen(gain) ne 0) then begin
		gain = gain >(-460)  < 50
		if (gain gt 0) then sadd=fix(2*(gain/2)) else $
			sadd=fix(2*((gain-1)/2))
	endif
; check
	print,'gain of X = ',xgain
	print,'gain of Y = ',ygain
	print,'gain of Z = ',zgain
	print,'offset on sum = ',sadd
	print,' '
;	
;	Run procedure to calculate new values based on these gains
	VRAW_CALCGAINS, xgain,ygain,zgain,sadd,sum,X,Y,XX,YY,r,dth

RETURN
END


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;***** CALCGAINS *****


PRO vraw_calcgains, xgain,ygain,zgain,sadd,sum,X,Y,XX,YY,r,dth

; 	Procedure to calculate the transforms for normal and polar plots.
;
; The subscript c implies that the value is 'corrected' for gains
	Zc=float(sum+sadd-X-Y)
; Have to divide by the gain factors because LUT calcs it upside down
	Xc=float(X/xgain)
	Yc=float(Y/ygain)
	Zc=Zc/zgain
	Sumc=(Xc+Yc+Zc)/460.0
	Xc=Xc/Sumc
	Yc=Yc/Sumc
	Zc=Zc/Sumc
;
; Start algorithm
;
;	clockwise
	XX=(Zc-Yc)/sqrt(2.0)
	YY=((2.0*Xc)-Yc-Zc)/sqrt(6.0)
;
	r=sqrt((XX*XX)+(YY*YY))
	rth=atan(XX,YY)
	negs=where(rth lt 0)
	rth(negs)=rth(negs)+(2.0*!dpi)
	dth=rth*180.0/!dpi
	over360 = where(dth ge 360.0)
	if (over360(0) ge 1) then dth(over360) = dth(over360)-360.0
;
; NB put this in to make the bitmap fit later on
; the size of the array is a fudge! Make the max r=150 for ease of use!
;All is the intensity array.
;
	valid= where (r le 150)
	r = r(valid)
	dth = dth(valid)

	RETURN
END

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;***** PRTHINT *****

PRO vraw_prthint,  all_big, OriginDeviceX, OriginDeviceY
; Intensity bit map prepared in intprep.pro

	print,'plotting intensity bit map'

	test_img = alog10(all_big > 0.1)
	tvscl, test_img, OriginDeviceX, OriginDeviceY
;
	RETURN
END


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;***** INTPREP *****

PRO vraw_intprep,  title_text, all, all_big, OriginDeviceX, OriginDeviceY

	print,'Preparing intensity data'
;
	!x.range=[0,150]
	!y.range=[0,360]
	botleft=[0,0]
	topright=[150,450]
; prepare axes
	plot,botleft,topright,  title=title_text, charsize = 1.0
	Origin = [ !x.range(0), !y.range(0)]
	OriginDevice = convert_coord ( Origin, /data, /to_device )
	OriginDeviceX = OriginDevice(0) + 1
	OriginDeviceY = OriginDevice(1) + 1
;	 
	Extent = [ !x.range(1), !y.range(1) ]
	ExtentDevice = convert_coord ( Extent, /data, /to_device )
	ExtentDeviceX = ExtentDevice(0) - 1
	ExtentDeviceY = ExtentDevice(1) - 1
	PlotsizDeviceX = ExtentDeviceX - OriginDeviceX
	PlotsizDeviceY = ExtentDeviceY - OriginDeviceY
;
	all_small = CONGRID(all, 150, 360)
	all_big   = CONGRID(all_small, PlotsizDeviceX, PlotsizDeviceY)

;
RETURN
END


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;***** CHK *****

PRO vraw_chk,  k, dphi, rphi, dalpha, ralpha, all_big, OriginDeviceX, $
		OriginDeviceY
;
;	Procedure for changing k and phi
;	First set up the degree scale - for 1 revs
;
	dalpha=indgen(360)
	ralpha=dalpha*!dpi/180
;
; 1. First take in new parameters
;
	anum = " "
	print, "Please give k (range 0.01 - 2.55): (default = ",k,")"
	read, "",anum
	if (strlen(anum) ne 0) then k = float(anum) > 0.01 < 2.55

	print, "Please give phi (range 0 - 360): (default = ",dphi,")"
	read, "",anum
	if (strlen(anum) ne 0) then dphi = float(anum) > 0  < 360
	rphi=dphi*!dpi/180
;
	vraw_PRTHINT, all_big, OriginDeviceX, OriginDeviceY
;
; 2.  Do the r theta plot
;
	n=0
	while (n le 20) do begin
		rexp=k*(ralpha+(2*n*!dpi)+rphi)
		oplot,rexp,dalpha, linestyle=5
		n=n+1
	endwhile
;
	RETURN
END

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;***** CHSTART *****

PRO vraw_chstart, start,stop,rstart,rstop,k,ralpha,dalpha,rphi,all_big,$
		OriginDeviceX,OriginDeviceY

; Routine for working out start and stop.
;
	anum = " "
	print, $
	  "Please give start (range 1.0 - 20.0): (default = ",start,")"
	read, "",anum
	if (strlen(anum) ne 0) then start = float(anum) > 1.0  < 20.0
;
	print, $
  "Please give stop  (range", start," - 20.0): (default = ",stop,")"
	read, "",anum
	if (strlen(anum) ne 0) then stop = float(anum) > start  < 20.0

	print,'start and stop',start,stop
	rstart = k*((2.0*start*!dpi)+rphi)
	rstop = k*((2.0*stop*!dpi)+rphi)
;
	vraw_PRTHINT, all_big, OriginDeviceX, OriginDeviceY

; Then draw with start and stop in new parameters
; Do the r theta plot
;
	VRAW_PLINES,  k, ralpha, dalpha, rphi, rstart, rstop
;
	RETURN
END


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;***** CHPERCENT *****

PRO vraw_chpercent, percent, ralpha, dalpha, rphi, k, start, stop, rstart, $
		rstop, all_big, OriginDeviceX, OriginDeviceY

; Routine for working out percent used.
; initialise boundaries to start.

	rphilower=rphi-!dpi
	rphiupper=rphi+!dpi
;
	achar="n"

	anum = " "
	print, $
	  "Please give percent (range 1 - 99): (default = ",percent*100,")"
	read, "",anum
	if (strlen(anum) ne 0) then begin
		percent = float(anum) > 1  < 99
		percent=percent/100.0
	endif
;	
;	Run procedure to calculate new values based on this percent
	VRAW_PPERCENT, percent, ralpha, dalpha, rphi, k, start, stop, rstart, $
		rstop, all_big, OriginDeviceX, OriginDeviceY

	RETURN
END

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;***** PPERCENT *****

PRO vraw_ppercent, percent, ralpha, dalpha, rphi, k, start, stop, rstart, $
		rstop, all_big, OriginDeviceX, OriginDeviceY
;
; 	Procedure to calculate the transforms for normal and polar plots.
;
	rphilower= rphi - (percent * !dpi)
	rphiupper= rphi + (percent * !dpi)
	rstartlower = k*((2*start*!dpi)+rphilower)
	rstoplower = k*((2*stop*!dpi)+rphilower)
	rstartupper = k*((2*start*!dpi)+rphiupper)
	rstopupper = k*((2*stop*!dpi)+rphiupper)
	
; refresh bitmap
	vraw_PRTHINT, all_big, OriginDeviceX, OriginDeviceY

; k, phi
	VRAW_PLINES,  k, ralpha, dalpha, rphi, rstart, rstop
	n=0
	while (n le 20) do begin
		rexp=k*(ralpha+(2*n*!dpi)+rphilower)
		ok= where((rexp ge rstartlower) and (rexp le rstoplower))
		if (n_elements(ok) gt 1) then $
			oplot,rexp(ok),dalpha(ok),psym=0
		rexp=k*(ralpha+(2*n*!dpi)+rphiupper)
		ok= where((rexp ge rstartupper) and (rexp le rstopupper))
		if (n_elements(ok) gt 1) then $
			oplot,rexp(ok),dalpha(ok),psym=0
		n=n+1
	endwhile
;
	RETURN
END

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;***** PARAMOUT *****

PRO vraw_paramout, percent,stop,start,k,sadd,dphi,rphi,detno, $
				xgain,ygain,zgain,rejection,rawfile,LUT_str
;
; routine for printing out numbers
; also writes a structure which can be read out for mylut to use directly
;
	print,' '
	print,rawfile
	head = bytarr (512)
	openr, lun, /get_lun, rawfile
	readu, lun, head
	head_str = gis_raw_header (head)
	free_lun, lun
;
	cbugmcpb = tm_calib('BUGMCPB1', head_str.bugmcpb, /quiet)
	if (head_str.fcurrent) eq 0 then $
		    print, 'Science Data, filament off, Slit no.', $
				head_str.slitn $
		else print,  'Filament Number', head_str.filament, $
		' @ 0x', head_str.fcurrent, ' current', $
			format = '(a, i2, a, z2.2, a)'
;
	print,format='("Parameter ",T15,"real",T25,"for LUT:")'
	print,format='(T25,"DEC",T35,"HEX")'
	print, format='("1   percent = ",T16,I3,T25,I3,T35,z2)', $
		(percent*100.0+0.5),(percent*100.0+0.5),(percent*100.0+0.5)
	print,format='("2   stop = ",T15,F4.1,T25,I3,T35,z2)', $
		stop,(stop*10.0+0.5),(stop*10.0+0.5)
	print,format='("3   start = ",T15,F4.1,T25,I3,T35,z2)', $
		start,(start*10.0+0.5),(start*10.0+0.5)
	print,format='("4   k = ",T15,F4.2,T25,I3,T35,z2)', $
		k,(k*100.0+0.5),(k*100.0+0.5)
	print,format='("5   zpln = ",T16,I3,"(",I3,")",T25,I3,T35,z2)', $
		sadd,(460+sadd),((460+sadd)/2),((460+sadd)/2)
	print,format='("6   phi = ",T16,I3,T25,I3,T35,z2)', $
		dphi,(40.0*rphi+0.5),(40.0*rphi+0.5)
	print,format='("7   bits ",T16,I3,T25,I3,T35,z2)',8,8,8
	print,format='("8   detector = ",T16,I3,T25,I3,T35,z2)', $
		detno,detno,detno
	print,format='("9   gain X = ",T15,F4.2,T25,I3,T35,z2)', $
		xgain,(xgain*100.0+0.5),(xgain*100.0+0.5)
	print,format='("10  gain Y = ",T15,F4.2,T25,I3,T35,z2)', $
		ygain,(ygain*100.0+0.5),(ygain*100.0+0.5)
	print,format='("11  gain Z = ",T15,F4.2,T25,I3,T35,z2)', $
		zgain,(zgain*100.0+0.5),(zgain*100.0+0.5)
	print,' '
	print,format='("--  rejection = ",T25,F5.2,T35,"%")',rejection
;
	print,' '
	print,'Structure written containing these parameters'	
;I know it doesn't do the writing just here!
;
	choice=" "
	print,' '
	read, $
  'Do you want to record these values in the file LUT_record.txt?', choice
	choice = strlowcase(choice)
	if (choice eq 'y') then begin
		openu,/append,lun,/get_lun,'LUT_record.txt'

	  printf,lun,rawfile
	  printf, lun, string(head(0:79))
	  if (head_str.fcurrent) eq 0 then $
			printf,lun, 'Science Data, filament off, Slit no.', $
				head_str.slitn $
		else printf,lun,  'Filament Number', head_str.filament, $
		' @ 0x', head_str.fcurrent, ' current', $
			format = '(a, i2, a, z2.2, a)'

	  printf,lun,  'HV set = ', head_str.bkghvl, ' (0x', head_str.bkghvl, $
		')    Front Face Bias =', cbugmcpb, ' Volts', $
		'      LLD set =', head_str.bkgldld, $
			format = '(a, i3, a, Z2.2, a, f8.2, a, a, i2) 
	  printf,lun, ' '
	  printf,lun,format='("Parameter ",T15,"real",T25,"for LUT:")'
	  printf,lun,format='(T25,"DEC",T35,"HEX")'
	  printf,lun,format='("1   percent = ",T16,I3,T25,I3,T35,z2)', $
		(percent*100.0+0.5),(percent*100.0+0.5),(percent*100.0+0.5)
	  printf,lun,format='("2   stop = ",T15,F4.1,T25,I3,T35,z2)', $
		stop,(stop*10.0+0.5),(stop*10.0+0.5)
	  printf,lun,format='("3   start = ",T15,F4.1,T25,I3,T35,z2)', $
		start,(start*10.0+0.5),(start*10.0+0.5)
	  printf,lun,format='("4   k = ",T15,F4.2,T25,I3,T35,z2)', $
		k,(k*100.0+0.5),(k*100.0+0.5)
	  printf,lun, $
		format='("5   zpln = ",T16,I3,"(",I3,")", T25,I3,T35,z2)', $
		sadd,(460+sadd),((460+sadd)/2),((460+sadd)/2)
	  printf,lun,format='("6   phi = ",T16,I3,T25,I3,T35,z2)', $
		dphi,(40.0*rphi+0.5),(40.0*rphi+0.5)
	  printf,lun,format='("7   bits ",T16,I3,T25,I3,T35,z2)',8,8,8
	  printf,lun,format='("8   detector = ",T16,I3,T25,I3,T35,z2)', $
		detno,detno,detno
	  printf,lun,format='("9   gain X = ",T15,F4.2,T25,I3,T35,z2)', $
		xgain,(xgain*100.0+0.5),(xgain*100.0+0.5)
	  printf,lun,format='("10  gain Y = ",T15,F4.2,T25,I3,T35,z2)', $
		ygain,(ygain*100.0+0.5),(ygain*100.0+0.5)
	  printf,lun,format='("11  gain Z = ",T15,F4.2,T25,I3,T35,z2)', $
		zgain,(zgain*100.0+0.5),(zgain*100.0+0.5)
	  printf,lun,' '
	  printf,lun,format='("--  rejection = ",T25,F5.2,T35,"%")',rejection
	  printf,lun,' '
		free_lun,lun
	endif
;	Structure	
	LUT_str = $
		{	percent:(percent*100.0),	$
			stop:stop,			$
			start:start,			$
			k:k,				$
			zpln:(460+sadd),		$
			dphi:dphi,			$
			numbits:8,			$
			detector:detno,			$
			xgain:xgain,			$
			ygain:ygain,			$
			zgain:zgain			$
		}
	RETURN
END

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;***** PARAMIN *****

PRO vraw_paramin, percent,stop,start,k,sadd,dphi,rphi,detno,xgain,ygain, $
		zgain, gset_id, det_selected
;
; routine for converting GSET LUT parameters into defaults for 
;	use in the program.
;
	get_gset, gset_id, def
;
	params = def.lut_par[det_selected-1, *]
;
	percent	= params[0]/100.0
	stop 	= params[1]/10.0
	start 	= params[2]/10.0
	k 	= params[3]/100.0
	sadd 	= (params[4]*2.0)-460.0
	rphi 	= params[5]/40.0
	dphi 	= rphi*180.0/!dpi
	bits	= params[6]
	detno	= params[7]
	xgain 	= params[8]/100.0
	ygain 	= params[9]/100.0
	zgain 	= params[10]/100.0
;
	RETURN
END

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;***** PLINES *****

PRO vraw_plines, k, ralpha, dalpha, rphi, rstart, rstop

; just to plot lines on picture

	if (k ne 0) then begin
		n=0
		while (n le 20) do begin
			rexp=k*(ralpha+(2*n*!dpi)+rphi)
			ok= where((rexp ge rstart) and (rexp le rstop))
			if (n_elements(ok) gt 1) then $
				oplot,rexp(ok),dalpha(ok), linestyle=5
			n=n+1
		endwhile
	endif

	RETURN
END

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;***** PDLINES *****

PRO vraw_pdlines,  k, ralpha, dalpha, rphi, rstart, rstop

; Plots same as plines.pro, but expands angle axis up to 720 degrees.
	if (k ne 0) then begin
		edalpha=dalpha+360
		ddalpha=[dalpha,edalpha]
		eralpha=ralpha+(2.0*!dpi)
		dralpha=[ralpha,eralpha]
		VRAW_PLINES, k,dralpha, ddalpha, rphi, rstart,rstop
	endif

	RETURN
END

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;***** REJECT *****

PRO vraw_reject, percent,stop,start,k,rphi,r,dth,rejection

; To find how many events are slipping through the LUT net
; Use fact that r=k(dth + 2npi + rphi + alpha), where alpha defines 
; distance from fitted r=k(dth + 2npi + rphi) line.
; If alpha falls outside 'percent' margin, then reject point.

	print,' '
	if (start eq 0) AND (stop eq 20) then print, $
			'*** You have not selected start and stop yet'
	if (percent eq 0) then print,'*** You have not selected percent yet'
	if (k ne 0) then begin

; Change dth into radians
		rth = (dth/180.0)*!dpi
;
; Start with percent test
		alphaplus = (r/k)-rphi-rth
		alpha = alphaplus MOD (2*!dpi)
;
		upper = percent*!dpi
		lower = (2*!dpi)-upper
		ok = where((alpha lt upper) OR (alpha gt lower))
		testr = r(ok)
		testdth = dth(ok)

;
; start and stop test - testing 2npi part of equation.
		gtpi=where (alpha gt !dpi)
		alpha(gtpi) = alpha(gtpi) - (2*!dpi) 
;
		test = (testr/k) - alpha(ok) - rphi
		inside=where(( test lt (stop*2.0*!dpi) ) AND $
		     ( test gt (start*2.0*!dpi) ))
		b=indgen(n_elements(r))
		if (n_elements(inside) gt 1) then $
			oplot,testr(inside),testdth(inside),psym=3
;
		rejection = $
		   (1.0-(float(n_elements(inside))/float(n_elements(r))))*100.0
		print,' '
		print,'Percentage rejected = ',rejection
		print,'Percentage included = ',100.0-rejection

	endif
;
	RETURN
END

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

pro view_raw, file=file, params=params, sumplot, raw_id=raw_id, gset_id=gset_id
;
;***** MAIN PROGRAM VIEW_RAW  *****
;
common VIEW_RAW_MYLUT, lut_str
;
;	Program to read in and display spectra from Alec's EGSE like Allsingn
;	Written early 1994 by AAB
; 	fiddled with by mwt late at night
;	and again by AAB
;	New version with sub procedures...and menus....
;
;	NB. 'd' in front of an angle variable means degrees
;	'r' means radians

; 	when fitting k, phi to r, theta plot, 
; 	leave previous guess on screen when inputting new values
; 	no need to hit return when choosing menus
; 	when drawing % zones on k-theta plot, use dashed lines
;	color table 2 is good for this purpose

;
;  Default the filename if not supplied
;
	if keyword_set(file) eq 0 then begin
		rawfile = "" 
		if keyword_set(raw_id) then begin
			get_raw, raw_id, raw_info
			rawfile = trim(raw_info.raw_file)
		endif
	endif else begin
		rawfile=file
	endelse
;
	loadct, 4, /silent

; General parameters
	sum=460
	achar=" "
	choice=" "

; Some parameters for memory window etc. and defaults
	n_xplots = 3
	n_yplots = 3
	plot_number = 1
	intensity='n'
	xgain=1.0
	ygain=1.0
	zgain=1.0
	sadd=0.0
	k=0.0 
	dphi=0.0
	rphi=0.0
	ralpha=0.0
	start=3.1
	stop=7.3
	percent = 0.99
	rejection = 100
	detno = 0
;
; And for plotting..
;;;	old_pcharsize=!p.charsize
;;;	old_xcharsize=!x.charsize
;;;	old_ycharsize=!y.charsize
;;;	!p.charsize=1.4	;was 1.5
;;;	!x.charsize=1.0	;was 0.8
;;;	!y.charsize=1.0	;was 0.8
;
; Get filename from user and read data in
;
	VRAW_DATAIN, whole, rawfile, head_str
	if (rawfile eq '') then return
	rawname = (ssw_strsplit(rawfile, '/', /tail))[0] ; get filename alone
;
	detno = head_str.detector	;use header to set a detector number
			;(may be overwritten if use option 'l' from menu 0)
;
;
	X=whole(*,0)
	Y=whole(*,1)
	if n_params() eq 1 then begin
		sumplot = hist_2d(x, y)
		return
	endif
;
; plot raw data
;
	vraw_PRAW, X,Y,rawname
;
;This is set to 'y' when intensity data has been calc'd
;
	intensity='n'
;	
	if keyword_set(gset_id) then begin
;
		print," "
		print, 'Using parameters from GSET id = ', trim(gset_id)
;
		det_selected = detno
		vraw_PARAMIN,percent,stop,start,k,sadd,dphi,rphi,$
			detno,xgain,ygain,zgain, gset_id, det_selected
;
;Plot data and overplot the fitted LUT
;
;Run procedure to calculate values based on the gains
;
		VRAW_CALCGAINS, xgain,ygain,zgain,sadd,sum,$
			X,Y,XX,YY,r,dth
;need this for all intensity array stuff- needs to be in gain loop
		all=intarr(150,360)
;
		title_text='VIEW_RAW r,theta plot: '+RAWNAME
		VRAW_PRTH, r,dth,title_text
		VRAW_PRTHCON, r,dth,all,intensity
		title_text='VIEW_RAW intensity and r,theta plot: ' + $
			RAWNAME
;
; prepare intensity bitmap to suit LARGE plot window
		VRAW_INTPREP, title_text, all, all_big, $
				OriginDeviceX, OriginDeviceY
;
;have to work out ralpha and dalpha which is normally done in vraw_chk.
;and rstart and rstop which are normally done in vraw_chstart
;
		dalpha=indgen(360)
		ralpha=dalpha*!dpi/180
		rstart = k*((2.0*start*!dpi)+rphi)
		rstop = k*((2.0*stop*!dpi)+rphi)

;Run procedure to plot percent - also includes VRAW_PRTHINT and VRAW_PLINES
		VRAW_PPERCENT, percent, ralpha, dalpha, rphi, k, $
	start, stop, rstart, rstop, all_big, OriginDeviceX, OriginDeviceY
;
		print,' '
		VRAW_REJECT, percent,stop,start,k,rphi,r,dth,$
			rejection 	;update this just in case
		VRAW_PARAMOUT, percent, stop, start, k, sadd, $
				dphi, rphi, detno,$
				xgain, ygain, zgain, rejection, $
				rawfile, LUT_str
;assign structure to the name you gave it
		params=LUT_str	
;
	endif		; gset_id supplied
;
	all=intarr(150,360)
;need this for all intensity array stuff- needs to be in gain loop

;
menu2:
	while (choice ne 'i') do begin 
;
		print," "
		if n_elements(r) le 1 then begin
			print,"*********** Menu 1 ************"
			print, "Select gains and offsets		g"
			print, "Plot raw data				w"
		endif else begin
			print,"*********** Menu 2 ************"
			print, "Select gains and offsets		g"
			print, "Plot raw data				w"
			print, "Plot spiral (XY coords)			x"
			print, "Plot in polar coords			p"
			print, "Plot two wavelengths, polar		d"
			print, "Plot contour in polar (over plot)	c"
			print, "Plot intensity in polar	and FIT to it	i"
			print, "Percentage rejected (2nd time only!)	r"
		endelse
		print, "Quit					q"
;
		read,'which one would you like?    ',choice
		choice = strlowcase(choice)

		case choice of
		"g": begin
			print,"Gain corrections..."
;
			if n_elements(r) gt 1 then begin
				VRAW_PRTH, r, dth, title_text
				VRAW_PLINES,  k, ralpha, dalpha, rphi, $
							rstart, rstop
;
				!p.linestyle = 1
				arrow, 130, 120, 18, 120, /data
				arrow, 130, 240, 18, 240, /data
				arrow, 130, 360, 18, 360, /data
;
				arrow, 135, 300, 40, 300, /data
				arrow, 40, 122, 135, 122, /data
;
				!p.linestyle = 0
;
				xyouts, 10, 120, '+ Z'
				xyouts, 10, 240, '+ Y'
				xyouts, 10, 360, '+ X'
;
				xyouts, 137, 300, '+ zpln'
				xyouts, 137, 122, '+ zpln'
			endif
;
			VRAW_CHGAIN, xgain,ygain,zgain,sadd,sum,X,Y,XX,YY,r,dth
			intensity='n'		
;This is set to 'y' when intensity data has been calc'd
			all=intarr(150,360)	
;need this for all intensity array stuff- needs to be in gain loop
			title_text='VIEW_RAW r,theta plot: '+RAWNAME
			VRAW_PRTH, r, dth, title_text
			VRAW_PLINES,  k, ralpha, dalpha, rphi, rstart, rstop
	     	end
		"w": begin
; 			plot raw data
			vraw_PRAW, X,Y,rawname
	    	 end
		"x": begin
; plot x,y data
			VRAW_PXY, XX,YY,rawname
	     	end
		"p": begin
			title_text='VIEW_RAW r,theta plot: '+RAWNAME
			VRAW_PRTH, r, dth, title_text
			VRAW_PLINES,  k, ralpha, dalpha, rphi, rstart, rstop
	     	end
		"d": begin
			title_text = $
				'VIEW_RAW theta (0 to 720deg),r plot: '+RAWNAME
			VRAW_PDRTH, r,dth,title_text
			VRAW_PDLINES,  k, ralpha, dalpha, rphi, rstart, rstop
	     	end
		"c": begin
			title_text = $
				'VIEW_RAW contour and r,theta plot: '+RAWNAME
			VRAW_PRTH, r,dth,title_text
			VRAW_PRTHCON, r,dth,all,intensity
			VRAW_PLINES,  k, ralpha, dalpha, rphi, rstart, rstop
	     	end
		"i": begin
			title_text = $
				'VIEW_RAW contour and r,theta plot: '+RAWNAME
			VRAW_PRTH, r,dth,title_text
			VRAW_PRTHCON, r,dth,all,intensity
			title_text = $
				'VIEW_RAW intensity and r,theta plot: '+RAWNAME
;
; prepare intensity bitmap to suit LARGE plot window
			VRAW_INTPREP, title_text, all, all_big, $
				OriginDeviceX, OriginDeviceY
;
			VRAW_PRTHINT, all_big, OriginDeviceX, OriginDeviceY
			VRAW_PLINES,  k, ralpha, dalpha, rphi, rstart, rstop
;
	    	 end
		"r": begin
			VRAW_REJECT, percent,stop,start,k,rphi,r,dth,rejection
	     	end
		"q": goto,finish
		else:
		endcase
	endwhile	; end second menu loop
	
;
	if (k eq 0) then begin
		while (choice ne 'k') do begin 
;
			print," "
			print,"*********** Menu 3 ************"
			print,"        	  Fitting"
			print, "Select k and phi			k"
			print, "Return to menu 2			m"
			print, "Change colour table                     t"
			print, "Quit					q"
;
			read,'which one would you like?    ',choice
			choice = strlowcase(choice)

			case choice of
			"k": begin
				VRAW_CHK, k, dphi, rphi, dalpha, ralpha, $
					all_big, OriginDeviceX, OriginDeviceY
;
; need to put in following few lines in case percent is next choice.
				rstart = k*((2*start*!dpi)+rphi)
				rstop = k*((2*stop*!dpi)+rphi)
	    		end
			"m": begin
				goto, menu2
	     		end
			"t": begin
				xloadct, /block
	     		end
			"q": goto,finish
			else:
			endcase
		endwhile	; end third menu loop
	endif

	while (choice ne 'q') do begin 
;
; assume percent = 0.99 (i.e., 99%) always
;
		percent = 0.99
;
		VRAW_PPERCENT, percent, ralpha, dalpha, rphi, k, start, $
			stop, rstart, $
			rstop, all_big, OriginDeviceX, OriginDeviceY
;
		print," "
		print,"*********** Menu 4 ************"
		print,"        	  Fitting"
		print, "Select k and phi			k"
		print, "Select start and stop			s"
;;;		print, "Select arm width			p"
		print, "Output parameters 			o"
		print, "Percentage rejected			r"
		print, "Return to menu 2			m"
		print, "Change colour table                     t"
		print, "Quit					q"
;
		read,'which one would you like?    ',choice
		choice = strlowcase(choice)

		case choice of
		"k": begin
			VRAW_CHK, k, dphi, rphi, dalpha, ralpha, all_big, $
					OriginDeviceX, OriginDeviceY
			rstart = k*((2*start*!dpi)+rphi)
			rstop = k*((2*stop*!dpi)+rphi)
	    	end
		"s": begin
			VRAW_CHSTART, start,stop,rstart,rstop,k,ralpha,$
				dalpha,rphi,all_big,OriginDeviceX,OriginDeviceY
	     	end
		"p": begin
			VRAW_CHPERCENT, percent, ralpha, dalpha, $
				rphi, k, start, stop, rstart, $
				rstop, all_big, OriginDeviceX, OriginDeviceY
	    	end
		"o": begin
			VRAW_REJECT, percent,stop,start,k,rphi,r,dth,$
				rejection 	;update this just in case
			VRAW_PARAMOUT, percent, stop, start, k, sadd, dphi, $
				rphi, detno, xgain, ygain, zgain, rejection, $
				rawfile, LUT_str
			params=LUT_str	
				;assign structure to the name you gave it
	     	end
		"r": begin
			VRAW_REJECT, percent,stop,start,k,rphi,r,dth,rejection
	     	end
		"m": begin
			goto, menu2
	     	end
		"t": begin
			xloadct, /block
	     	end
		"q": goto,finish
		else:
		endcase
	endwhile	;end forth menu loop
;
finish:
	!p.multi=0
	!x.range=0
	!y.range=0
	!p.linestyle=0
;
;;;	!p.charsize=old_pcharsize
;;;	!x.charsize=old_xcharsize
;;;	!y.charsize=old_ycharsize

	return
END
