	PRO CDS_ENG_N1_AN, STUDY_CNTR, RSLT, ERRMSG=ERRMSG
;+
; Project     :	SOHO - CDS
;
; Name        :	CDS_ENG_N1_AN
;
; Purpose     :	Analyze VDS data from CDS engineering test N1.
;
; Category    :	Data analysis, Engineering
;
; Explanation :	Reads in the data generated by CDS engineering test N1.  This
;		tests the NIS CCD dark count by taking a series of measurements
;		with no light falling on the CCD.  The test proceeds in two
;		phases:
;
;			Phase 1.  VDS high voltage powered off.
;			Phase 2.  VDS high voltage powered on, but shuttered
;				  off.
;
;		In each phase, a series of four exposures are made at times of
;		1, 10, 100, and 1000 seconds.  The full CCD is read out.
;
; Syntax      :	CDS_ENG_N1_AN, STUDY_COUNTER  [, RESULT ]
;
; Examples    :	CDS_ENG_N1_AN, 100	;Would read in s100r0.fits,
;					;s100r1.fits, etc.
;
; Inputs      :	STUDY_COUNTER	= The study counter that serves as a unique
;				  identifier to the set of data files.  Also
;				  known as PROG_NUM.
;
; Opt. Inputs :	None.
;
; Outputs     :	None required.
;
; Opt. Outputs:	RESULT	= A structure array containing the following tags:
;
;				AVG	= Average value per quadrant.
;				SIGMA	= Standard deviation per quadrant.
;				MIN	= Minimum value in each quadrant.
;				MAX	= Maximum value in each quadrant.
;
;			  Each tag has four elements.  Also, the tag
;			  STUDY_COUNTER contains the value of STUDY_COUNTER
;			  passed to this routine.  The RESULT array has 4x2
;			  elements representing the four exposure times and the
;			  two phases.
;
; Keywords    :	ERRMSG = If defined and passed, then any error messages will be
;			 returned to the user in this parameter rather than
;			 depending on the MESSAGE routine in IDL.  If no errors
;			 are encountered, then a null string is returned.  In
;			 order to use this feature, ERRMSG must be defined
;			 first, e.g.
;
;				ERRMSG = ''
;				CDS_ENG_N1_AN, ERRMSG=ERRMSG, ...
;				IF ERRMSG NE '' THEN ...
;
; Calls       :	TRIM, FIND_WITH_DEF, CDS_ENG_N1_RD, CDS_ENG_N1_PR
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Prev. Hist. :	None.
;
; History     :	Version 1, 24-Oct-1995, William Thompson, GSFC
;
; Contact     :	WTHOMPSON
;-
;
@cds_eng_n1.cmn
	ON_ERROR, 2
;
;  Check the parameters.
;
	IF N_PARAMS() LT 1 THEN BEGIN
	    MESSAGE = 'Syntax:  CDS_ENG_N1_AN, STUDY_COUNTER  [, RESULT ]'
	    GOTO, HANDLE_ERROR
	ENDIF
;
;  Look for the first raster in the series.
;
	NAME = 's' + TRIM(STUDY_CNTR) + 'r0.fits'
	FILENAME = FIND_WITH_DEF(NAME, getenv('CDS_FITS_DATA'))
	IF FILENAME EQ '' THEN BEGIN
	    MESSAGE = 'File ' + NAME + ' not found'
	    GOTO, HANDLE_ERROR
	ENDIF
;
;  Extract the file path.
;
	BREAK_FILE, FILENAME, DISK, DIR
	PATH = DISK + DIR
;
;  Create the output structure array.
;
	RSLT = {STUDY_COUNTER: STUDY_CNTR,	$
		AVG: FLTARR(4),			$
		SIGMA: FLTARR(4),		$
		MIN: INTARR(4),			$
		MAX: INTARR(4)}
	RSLT = REPLICATE(RSLT, 4, 2)
;
;  Read in the files one by one.
;
	PREFIX = PATH + 's' + TRIM(STUDY_CNTR) + 'r'
	FOR I = 0, 7 DO BEGIN
	    MESSAGE = ''
	    CDS_ENG_N1_RD, PREFIX + TRIM(I) + '.fits', DATA, ERRMSG=MESSAGE
	    IF MESSAGE NE '' THEN GOTO, HANDLE_ERROR
;
;  Calculate the average, standard deviation, and range for each quadrant.
;  Avoid the leftmost and rightmost columns, which are known to be tricky.
;
	    DATA = FLOAT(DATA)
	    A = DATA(1:511,    512:*)
	    B = DATA(512:1022, 512:*)
	    C = DATA(1:511,    0:511)
	    D = DATA(512:1022, 0:511)
;
	    RSLT(I).AVG = [AVERAGE(A), AVERAGE(B), AVERAGE(C), AVERAGE(D)]
	    SQR = [AVERAGE(A^2), AVERAGE(B^2), AVERAGE(C^2), AVERAGE(D^2)]
	    RSLT(I).SIGMA = SQRT(SQR - (RSLT(I).AVG)^2)
	    RSLT(I).MIN = [MIN(A), MIN(B), MIN(C), MIN(D)]
	    RSLT(I).MAX = [MAX(A), MAX(B), MAX(C), MAX(D)]
	ENDFOR
;
;  If the RESULT parameter was not passed, then print the result.
;
	IF N_PARAMS() NE 2 THEN CDS_ENG_N1_PR, RSLT
	RETURN
;
;  Error handling point.
;
HANDLE_ERROR:
	IF N_ELEMENTS(ERRMSG) NE 0 THEN ERRMSG = 'CDS_ENG_N1_AN: ' + MESSAGE $
		ELSE MESSAGE, MESSAGE
	END
