	PRO CDS_ENG_N1_RD, FILENAME, ARRAY, ERRMSG=ERRMSG
;+
; Project     :	SOHO - CDS
;
; Name        :	CDS_ENG_N1_RD
;
; Purpose     :	Read VDS data from CDS engineering test N1.
;
; Category    :	Data analysis, Engineering
;
; Explanation :	Reads in a data file generated by CDS engineering test N1.
;		Called from CDS_ENG_N1_AN.  The array is read in, and the data
;		is returned to its original detector orientation.
;
; Syntax      :	CDS_ENG_N1_RD, FILENAME, ARRAY
;
; Examples    :	CDS_ENG_N1_RD, 's100r0.fits'
;
; Inputs      :	FILENAME = Name of file to read in.
;
; Opt. Inputs :	None.
;
; Outputs     :	ARRAY = Output array of 1024x1024 pixels, in detector
;			orientation.
;
; Opt. Outputs:	None.
;
; Keywords    :	ERRMSG = If defined and passed, then any error messages will be
;			 returned to the user in this parameter rather than
;			 depending on the MESSAGE routine in IDL.  If no errors
;			 are encountered, then a null string is returned.  In
;			 order to use this feature, ERRMSG must be defined
;			 first, e.g.
;
;				ERRMSG = ''
;				CDS_ENG_N1_RD, ERRMSG=ERRMSG, ...
;				IF ERRMSG NE '' THEN ...
;
; Calls       :	READCDSFITS, REARRANGE
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Prev. Hist. :	None.
;
; History     :	Version 1, 24-Oct-1995, William Thompson, GSFC
;
; Contact     :	WTHOMPSON
;-
;
	ON_ERROR, 2
;
;  Check the parameters.
;
	IF N_PARAMS() NE 2 THEN BEGIN
	    MESSAGE = 'Syntax: CDS_ENG_N1_RD, FILENAME, ARRAY'
	    GOTO, HANDLE_ERROR
	ENDIF
;
;  Read in the file, and extract the data.  It should be in column 1.
;
	A = READCDSFITS(FILENAME)
	DATA = DETDATA(A,1)
;
;  Reformat the data array.  It should have 1024x1024 pixels.
;
	DATA = REFORM(DATA)
	SZ = SIZE(DATA)
	IF (SZ(0) NE 2) OR (SZ(1) NE 1024) OR (SZ(2) NE 1024) THEN BEGIN
	    MESSAGE = 'Data array not 1024x1024 - not from CDS eng. test N1'
	    GOTO, HANDLE_ERROR
	ENDIF
;
;  Reverse the Y dimension to put it back in detector orientation.
;
	DATA = REARRANGE(DATA, [1,-2])
	RETURN
;
;  Error handling point.
;
HANDLE_ERROR:
	IF N_ELEMENTS(ERRMSG) NE 0 THEN ERRMSG = 'CDS_ENG_N1_RD: ' + MESSAGE $
		ELSE MESSAGE, MESSAGE
	END
