;+
; Project     :	SOHO - CDS
;
; Name        :	CDS_ENG_N4
;
; Purpose     :	Widget to analyze VDS data from CDS engineering test N4.
;
; Category    :	Data analysis, Engineering
;
; Explanation :	Widget interface to the CDS_ENG_N4 routines.  The user is
;		prompted for a study counter to analyze.  The analysis is then
;		done in the routine CDS_ENG_N4_AN.
;
; Syntax      :	CDS_ENG_N4  [, RESULT]
;
; Examples    :	
;
; Inputs      :	None.
;
; Opt. Inputs :	None.
;
; Outputs     :	None required.
;
; Opt. Outputs:	RESULT	= A structure array containing the following tags:
;
;			STUDY_COUNTER	= The study counter value.
;			AVG	= A 6x2 array containing the average calibrated
;				  value in each of the six exposures and two
;				  spectral bands.  These numbers should be
;				  approximately equal.
;
; Keywords    :	None.
;
; Calls       :	CDS_ENG_N4_AN, CDS_ENG_N4_PR
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Prev. Hist. :	None.
;
; History     :	Version 1, 25-Oct-1995, William Thompson, GSFC
;
; Contact     :	WTHOMPSON
;-
;
;==============================================================================
;  This is the event handler.
;==============================================================================
;
	PRO CDS_ENG_N4_EVENT, EVENT
@cds_eng_n4.cmn
;
;  Get the event user value, and take the appropriate action.
;
	WIDGET_CONTROL, EVENT.ID, GET_UVALUE=EV
	CASE EV OF
;
;  The study counter has been changed.
;
	    'STUDY_COUNTER': BEGIN
		WIDGET_CONTROL, EVENT.ID, GET_VALUE=STUDY_COUNTER
		STUDY_COUNTER = STUDY_COUNTER(0)
	    END
;
;  The "Go" button was pressed.  Call CDS_ENG_N4_AN.
;
	    'GO': BEGIN
		IF STUDY_COUNTER LE 0 THEN BEGIN
			XACK, /MODAL, 'Study counter not defined'
		END ELSE BEGIN
			WIDGET_CONTROL, /HOURGLASS
			MESSAGE = ''
			CDS_ENG_N4_AN, STUDY_COUNTER, RESULT, ERRMSG=MESSAGE
			IF MESSAGE NE '' THEN XACK, /MODAL, MESSAGE ELSE $
				CDS_ENG_N4_PR, RESULT, WIDGET=TEXT
		ENDELSE
	    END
;
;  The "Done" button was pressed.  Exit the program.
;
	    'DONE': WIDGET_CONTROL, EVENT.TOP, /DESTROY
;
;  The "Save" button was pressed.  Call CDS_ENG_N4_PR.
;
	    'SAVE': BEGIN
		IF N_ELEMENTS(RESULT) EQ 0 THEN BEGIN
			XACK, /MODAL, 'No data analyzed yet.'
		END ELSE IF STRLEN(OUTFILE) EQ 0 THEN BEGIN
			XACK, /MODAL, 'Output filename not defined yet.'
		END ELSE BEGIN
			WIDGET_CONTROL, /HOURGLASS
			CDS_ENG_N4_PR, RESULT, OUTFILE
			WIDGET_CONTROL, TEXT, /APPEND, SET_VALUE=''
			WIDGET_CONTROL, TEXT, /APPEND, SET_VALUE=	$
				'Wrote file ' + OUTFILE
		ENDELSE
	    END
;
;  The output filename has been changed.
;
	    'OUTFILE': BEGIN
		WIDGET_CONTROL, EVENT.ID, GET_VALUE=OUTFILE
		OUTFILE = OUTFILE(0)
	    END
	ENDCASE
;
	END
;
;==============================================================================
;  This sets up the widget.
;==============================================================================
;
	PRO CDS_ENG_N4, RSLT
@cds_eng_n4.cmn
;
;  If a copy of CDS_ENG_N4 is already running, then exit.
;
	IF (XREGISTERED("cds_eng_n4") NE 0) THEN RETURN
;
;  Define the main base.
;
	MAIN = WIDGET_BASE(/COLUMN, /MAP)
;
;  Define the top base.  It has an editable label for the study counter, and a
;  "GO" button.
;
	TOP = WIDGET_BASE(MAIN, /ROW, /MAP)
	STUDY_COUNTER = 0
	DUMMY = CW_FIELD(TOP, VALUE=STUDY_COUNTER, /ROW, /LONG,		$
		TITLE='Study counter:', /ALL_EVENTS, UVALUE='STUDY_COUNTER')
	DUMMY = WIDGET_BUTTON(TOP, UVALUE='GO', VALUE='    Go   ', FRAME=3)
;
;  Define the text widget to display the result.
;
	TEXT = WIDGET_TEXT(MAIN, VALUE='', XSIZE=50, YSIZE=15)
;
;  Define the bottom base.  It has a "Done" button, and a base for saving the
;  output.
;
	BOTTOM = WIDGET_BASE(MAIN, /ROW, /MAP)
	DUMMY = WIDGET_BUTTON(BOTTOM, UVALUE='DONE', VALUE='Done')
;
;  Define the base for the save widgets.  It contains a "Save" button, and a
;  text widget for the filename.
;
	SAVE_BASE = WIDGET_BASE(BOTTOM, /ROW, FRAME=2, /MAP)
	DUMMY = WIDGET_BUTTON(SAVE_BASE, UVALUE='SAVE', VALUE='Save')
	OUTFILE = ''
	DUMMY = CW_FIELD(SAVE_BASE, VALUE=OUTFILE, /ROW, /STRING,	$
		/ALL_EVENTS, TITLE='Filename:', UVALUE='OUTFILE')
;
;  Reset the RESULT structure to start with a clean plate.
;
	DELVARX, RESULT
;
;  Realize the widget, and start it going.
;
	WIDGET_CONTROL, MAIN, /REALIZE
	XMANAGER, 'cds_eng_n4', MAIN
;
	END
