	PRO CDS_ENG_N4_AN, STUDY_CNTR, RSLT, ERRMSG=ERRMSG
;+
; Project     :	SOHO - CDS
;
; Name        :	CDS_ENG_N4_AN
;
; Purpose     :	Analyze VDS data from CDS engineering test N4.
;
; Category    :	Data analysis, Engineering
;
; Explanation :	Reads in the data generated by CDS engineering test N4.  This
;		tests the NIS MCP amplification by taking a series of
;		measurements with different voltages:
;
;			Prog. Vol. #14	    Equivalent
;
;				0		603
;			       50		678
;			      100		756
;			      150		834
;			      200		912
;			      250		990
;
; Syntax      :	CDS_ENG_N4_AN, STUDY_COUNTER  [, RESULT ]
;
; Examples    :	CDS_ENG_N4_AN, 100	;Would read in s100r0.fits,
;					;s100r1.fits, etc.
;
; Inputs      :	STUDY_COUNTER	= The study counter that serves as a unique
;				  identifier to the set of data files.  Also
;				  known as PROG_NUM.
;
; Opt. Inputs :	None.
;
; Outputs     :	None required.
;
; Opt. Outputs:	RESULT	= A structure array containing the following tags:
;
;			STUDY_COUNTER	= The study counter value.
;			AVG	= A 6x2 array containing the average calibrated
;				  value in each of the six exposures and two
;				  spectral bands.  These numbers should be
;				  approximately equal.
;
; Keywords    :	ERRMSG = If defined and passed, then any error messages will be
;			 returned to the user in this parameter rather than
;			 depending on the MESSAGE routine in IDL.  If no errors
;			 are encountered, then a null string is returned.  In
;			 order to use this feature, ERRMSG must be defined
;			 first, e.g.
;
;				ERRMSG = ''
;				CDS_ENG_N4_AN, ERRMSG=ERRMSG, ...
;				IF ERRMSG NE '' THEN ...
;
; Calls       :	TRIM, FIND_WITH_DEF, CDS_ENG_N4_RD, CDS_ENG_N4_PR
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Prev. Hist. :	None.
;
; History     :	Version 1, 25-Oct-1995, William Thompson, GSFC
;
; Contact     :	WTHOMPSON
;-
;
@cds_eng_n4.cmn
	ON_ERROR, 2
;
;  Check the parameters.
;
	IF N_PARAMS() LT 1 THEN BEGIN
	    MESSAGE = 'Syntax:  CDS_ENG_N4_AN, STUDY_COUNTER  [, RESULT ]'
	    GOTO, HANDLE_ERROR
	ENDIF
;
;  Look for the first raster in the series.
;
	NAME = 's' + TRIM(STUDY_CNTR) + 'r0.fits'
	FILENAME = FIND_WITH_DEF(NAME, getenv('CDS_FITS_DATA'))
	IF FILENAME EQ '' THEN BEGIN
	    MESSAGE = 'File ' + NAME + ' not found'
	    GOTO, HANDLE_ERROR
	ENDIF
;
;  Extract the file path.
;
	BREAK_FILE, FILENAME, DISK, DIR
	PATH = DISK + DIR
;
;  Create the output array.
;
	RSLT = {STUDY_COUNTER: STUDY_CNTR,	$
		VOLTAGE: INTARR(6),		$
		AVG: FLTARR(6,2)}
;
;  Read in the files one by one.
;
	PREFIX = PATH + 's' + TRIM(STUDY_CNTR) + 'r'
	RSLT.VOLTAGE = [0, 50, 100, 150, 200, 250]
	FOR I = 0, 5 DO BEGIN
	    DATA = READCDSFITS(PREFIX + TRIM(I) + '.fits')
;
	    MESSAGE = ''
	    VDS_EXTR_BIASES, DATA, BIASES, ERRMSG=MESSAGE
	    IF MESSAGE NE '' THEN GOTO, HANDLE_ERROR
;
	    VDS_DEBIAS, DATA, BIASES
	    VDS_CALIB, DATA, RSLT.VOLTAGE(I)
;
;  Calculate the average for each spectral range.  Avoid the leftmost and
;  rightmost columns, which are known to be tricky.
;
	    A = REFORM(DETDATA(DATA, 1))
	    SZ = SIZE(A)
	    IF (SZ(0) NE 2) OR (SZ(1) NE 1024) OR (SZ(2) NE 120) THEN BEGIN
		MESSAGE = 'Data array not 1024x120 - not from CDS eng. test N4'
		GOTO, HANDLE_ERROR
	    ENDIF
	    RSLT.AVG(I,0) = AVERAGE(A(1:1022,*))
;
	    A = REFORM(DETDATA(DATA, 2))
	    SZ = SIZE(A)
	    IF (SZ(0) NE 2) OR (SZ(1) NE 1024) OR (SZ(2) NE 120) THEN BEGIN
		MESSAGE = 'Data array not 1024x120 - not from CDS eng. test N4'
		GOTO, HANDLE_ERROR
	    ENDIF
	    RSLT.AVG(I,1) = AVERAGE(A(1:1022,*))
	ENDFOR
;
;  If the RESULT parameter was not passed, then print the result.
;
	IF N_PARAMS() NE 2 THEN CDS_ENG_N4_PR, RSLT
	RETURN
;
;  Error handling point.
;
HANDLE_ERROR:
	IF N_ELEMENTS(ERRMSG) NE 0 THEN ERRMSG = 'CDS_ENG_N4_AN: ' + MESSAGE $
		ELSE MESSAGE, MESSAGE
	END
