	PRO ADD_EFFICIENCY, DATE_EFF, SPECTRUM, EFFICIENCY
;+
; Project     :	SOHO - CDS
;
; Name        :	ADD_EFFICIENCY
;
; Purpose     :	Adds an overall efficiency value to database
;
; Category    :	CDS, Calibration, Class3
;
; Explanation :	This routine adds an overall efficiency value for one of the
;		CDS spectral ranges to the database.
;
; Syntax      :	ADD_EFFICIENCY, DATE_EFF, SPECTRUM, EFFICIENCY
;
; Examples    :	ADD_EFFICIENCY, '1995/12/02', 'N2', 0.00040
;
; Inputs      :	DATE_EFF = The date, and optionally time, that the value
;			   becomes effective.  In other words, the calibration
;			   value should be applied to all science data taken on
;			   or after that date.  Can be in any CDS time format.
;
;			   The current date is also stored along with the data.
;			   When the database is read in, the most current
;			   version for a given date is used.
;
;			   For example, if there were two values of DATE_EFF in
;			   the database,
;
;				DATE_EFF = 1995/12/02
;				DATE_EFF = 1996/09/16
;
;			   then the first would be used for all data taken
;			   between 1995/12/02 and 1996/09/16, and the second
;			   would be used for all data from 1996/09/16 on.  If
;			   there was more than one entry in the database with
;			   an effective date of 1996/09/16, then the most
;			   recent one would be used.
;
;			   There is one thing to watch out for in maintaining
;			   the database.  Taking the above example, if one
;			   added a calibration with an effective date of
;			   1996/05/01, then it would be applied to data between
;			   that date and the next date in the database,
;			   i.e. 1996/09/16.  If one wanted the new calibration
;			   to apply to all data after 1996/05/01, then one
;			   would have to also add a new entry for 1996/09/16.
;
;		SPECTRUM = A character string representing the spectrum,
;			   e.g. "N1" or "NIS1".
;
;		EFFICIENCY = The overall efficiency value.
;
; Opt. Inputs :	None.
;
; Outputs     :	None.
;
; Opt. Outputs:	None.
;
; Keywords    :	None.
;
; Calls       :	PRIV_ZDBASE, ANYTIM2UTC, GET_UTC, UTC2TAI, DBOPEN, DBBUILD,
;		DBCLOSE
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Prev. Hist. :	None.
;
; History     :	Version 1, 16-Sep-1996, William Thompson, GSFC
;		Version 2, 21-May-2002, William Thompson, GSFC
;			Allow N4 for NIS-2 second order.
;
; Contact     :	WTHOMPSON
;-
;
	ON_ERROR, 2
;
;  Check the input parameters.
;
	IF N_PARAMS() NE 3 THEN MESSAGE,	$
		'Syntax:  ADD_EFFICIENCY, DATE_EFF, SPECTRUM, EFFICIENCY'
;
;  Make sure that one has privilege to write into the calibration database.
;
	MESSAGE = ''
	IF NOT PRIV_ZDBASE(/CALIB,ERRMSG=MESSAGE) THEN MESSAGE, MESSAGE
;
;  Parse the spectrum identifier.
;
	S = STRTRIM(SPECTRUM,2)
	SG = STRUPCASE(STRMID(S,0,1) + STRMID(S,STRLEN(S)-1,1))
	IF (SG NE 'N1') AND (SG NE 'N2') AND (SG NE 'N4') AND (SG NE 'G1') $
		AND (SG NE 'G2') AND (SG NE 'G3') AND (SG NE 'G4') THEN	$
		MESSAGE, 'Unrecognized spectrum identifier'
;
;  Convert the effective date to TAI format.  Reformat the time to millisecond
;  accuracy.  This is necessary so that the times are written out in a
;  controlled way.
;
	TAI_EFF = DOUBLE( STRING( ANYTIM2TAI(DATE_EFF), FORMAT='(F15.3)' ))
;
;  Get the current date.
;
	GET_UTC, DATE
	DATE = UTC2TAI(DATE)
;
;  Store the date in the database.
;
	PRIV = !PRIV  &  !PRIV = 2
	DBOPEN, 'efficiency', 1
	DBBUILD, DATE, TAI_EFF, SG, FLOAT(EFFICIENCY)
	DBCLOSE
	!PRIV = PRIV
;
	RETURN
	END
