	PRO AN_NIMCP_1_2, STUDY1, STUDY2, RESULT, NOREAD=NOREAD
;+
; Project     :	SOHO - CDS
;
; Name        :	AN_NIMCP_1_2
;
; Purpose     :	Analyze data from runs of NIMCP_1 and NIMCP_2
;
; Category    :	Class3, CDS, Analysis, Calibration
;
; Explanation :	Reads in data from runs of NIMCP_1 and NIMCP_2, and plots up a
;		display comparing the average profiles.  The user can then zoom
;		in on a portion of the display and then fit gaussians to the
;		data to determine the amount of loss of sensitivity.
;
;		This routine uses a cruder approach than that of AN_NIMCP, and
;		it's results are less accurate.  In particular, it seems to
;		overestimate the amount of throughput loss.
;
;		Note that this routine is used for the older definitions of
;		NIMCP_1 and NIMCP_2 which were run as separate studies.  The
;		modified NIMCP study should be analyzed with AN_NIMCP or
;		AN_NIMCP_AVG.
;
; Syntax      :	AN_NIMCP_1_2, STUDY1, STUDY2, RESULT
;
; Examples    :	AN_NIMCP_1_2, 122, 123, RESULT
;
; Inputs      :	STUDY1	= The study counter number for the run of NIMCP_1.  If
;			  passed as an array, then the data from the specified
;			  studies are averaged together.
;		STUDY2	= The study counter number for the run of NIMCP_2.  If
;			  passed as an array, then the data from the specified
;			  studies are averaged together.
;
; Opt. Inputs :	RESULT	= The structure returned from an earlier run of this
;			  routine.
;
; Outputs     :	RESULT	= Structure containing the average traces.
;
; Opt. Outputs:	None.
;
; Keywords    :	NOREAD	= If set, then the FITS files are not read.  In this
;			  case, then RESULT must already be defined.
;
; Calls       :	
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Prev. Hist. :	None.
;
; History     :	Version 1, 02-May-1996, William Thompson, GSFC
;		Version 2, 22-May-1996, William Thompson, GSFC
;			Added capability of averaging together several studies.
;			Incorporate gaussian fitting, as in AN_NIMCP_AVG.
;		Version 3, 12-Sep-1996, William Thompson, GSFC
;			Added printing and output options, better scaling
;			Don't call VDS_CALIB
;
; Contact     :	WTHOMPSON
;-
;
	ON_ERROR, 2
;
;  Check the number of parameters.
;
	IF N_PARAMS() NE 3 THEN MESSAGE,	$
		'Syntax: AN_NIMCP_2, STUDY1, STUDY2, RESULT'
	ERASE
;
;  If the NOREAD keyword was set, then skip reading the files.
;
	IF NOT KEYWORD_SET(NOREAD) THEN BEGIN
	    DATES2 = DBLARR(N_ELEMENTS(STUDY2))
	    FOR I=0,N_ELEMENTS(STUDY2)-1 DO BEGIN
;
;  From the study number, form the filename.  Read in the files.
;
		FILENAME2 = 's' + NTRIM(STUDY2(I)) + 'r00.fits'
		A = READCDSFITS(FILENAME2)
		DATES2(I) = UTC2TAI(A.HEADER.DATE_OBS)
;
;  Make sure that these are the correct files.
;
		IF TRIM(A.HEADER.OBS_PROG) NE 'NIMCP_2' THEN MESSAGE,	$
			'Study #' + NTRIM(STUDY2) + ' is not a run of NIMCP_2'
;
;  Calibrate the data.
;
		VDS_DEBIAS, A
		AMISSING = A.DETDESC(0).MISSING
;
;  Extract the average profiles, and normalize them.
;
		AA1 = AVERAGE(REFORM(DETDATA(A,1)),2,MISSING=AMISSING)
		AMIN = MIN(AA1(2:1021),MAX=AMAX)
		AA1 = (AA1 - AMIN) / (AMAX - AMIN)
;
		AA2 = AVERAGE(REFORM(DETDATA(A,2)),2,MISSING=AMISSING)
		AMIN = MIN(AA2(2:1021),MAX=AMAX)
		AA2 = (AA2 - AMIN) / (AMAX - AMIN)
;
		IF I EQ 0 THEN BEGIN
		    A1 = AA1
		    A2 = AA2
		END ELSE BEGIN
		    A1 = [[A1], [AA1]]
		    A2 = [[A2], [AA2]]
		ENDELSE
	    ENDFOR
;
	    IF N_ELEMENTS(STUDY2) GT 1 THEN BEGIN
		A1 = AVERAGE(A1, 2, MISSING=AMISSING)
		A2 = AVERAGE(A2, 2, MISSING=AMISSING)
	    ENDIF
;
;  Do the same for STUDY1.
;
	    DATES1 = DBLARR(N_ELEMENTS(STUDY1))
	    FOR I=0,N_ELEMENTS(STUDY1)-1 DO BEGIN
;
;  From the study number, form the filename.  Read in the files.
;
		FILENAME1 = 's' + NTRIM(STUDY1(I)) + 'r00.fits'
		B = READCDSFITS(FILENAME1)
		DATES1(I) = UTC2TAI(A.HEADER.DATE_OBS)
;
;  Make sure that these are the correct files.
;
		IF TRIM(B.HEADER.OBS_PROG) NE 'NIMCP_1' THEN MESSAGE,	$
			'Study #' + NTRIM(STUDY1) + ' is not a run of NIMCP_1'
;
;  Calibrate the data.
;
		VDS_DEBIAS, B
		BMISSING = B.DETDESC(0).MISSING
;
;  Extract the average profiles, and normalize them.
;
		BB1 = AVERAGE(REFORM(DETDATA(B,1)),2,MISSING=BMISSING)
		BMIN = MIN(BB1(2:1021),MAX=BMAX)
		BB1 = (BB1 - BMIN) / (BMAX - BMIN)
;
		BB2 = AVERAGE(REFORM(DETDATA(B,2)),2,MISSING=BMISSING)
		BMIN = MIN(BB2(2:1021),MAX=BMAX)
		BB2 = (BB2 - BMIN) / (BMAX - BMIN)
;
		IF I EQ 0 THEN BEGIN
		    B1 = BB1
		    B2 = BB2
		END ELSE BEGIN
		    B1 = [[B1], [BB1]]
		    B2 = [[B2], [BB2]]
		ENDELSE
	    ENDFOR
;
	    IF N_ELEMENTS(STUDY1) GT 1 THEN BEGIN
		B1 = AVERAGE(B1, 2, MISSING=BMISSING)
		B2 = AVERAGE(B2, 2, MISSING=BMISSING)
	    ENDIF
;
;  Calculate the wavelength scale.
;
	    W1 = PIX2WAVE('NIS1', INDGEN(1024))
	    W2 = PIX2WAVE('NIS2', INDGEN(1024))
;
;  Store everything in RESULT.
;
	    DATE = TAI2UTC((AVERAGE(DATES1) + AVERAGE(DATES2)) / 2, /CCSDS)
	    RESULT = {A1: A1, A2: A2, B1: B1, B2: B2, W1: W1, W2: W2,	$
		    DATE_OBS: DATE}
	ENDIF
;
;  Plot the data.
;
	TITLE = 'NIMCP studies ' + NTRIM(STUDY1(0))
	FOR I = 1,N_ELEMENTS(STUDY1)-1 DO TITLE = TITLE + ',' +	$
		NTRIM(STUDY1(I))
	TITLE = TITLE + ' and ' + NTRIM(STUDY2(0))
	FOR I = 1,N_ELEMENTS(STUDY2)-1 DO TITLE = TITLE + ',' +	$
		NTRIM(STUDY2(I))
;
PLOT_ALL:
	ERASE
	SETVIEW, 1, 1, 1, 2
	PLOT, RESULT.W1, RESULT.A1, XSTYLE=1, XTITLE='Wavelength (A)',	$
		YTITLE='NIS 1', TITLE=TITLE, YRANGE=[0,1]
	OPLOT, RESULT.W1, RESULT.B1, LINESTYLE=1
;
	SETVIEW, 1, 1, 2, 2
	PLOT, RESULT.W2, RESULT.A2, XSTYLE=1, XTITLE='Wavelength (A)',	$
		YTITLE='NIS 2', YRANGE=[0,1]
	OPLOT, RESULT.W2, RESULT.B2, LINESTYLE=1
;
;  Read in a cursor position, and decide whether or not to zoom.
;
	PRINT, 'Left button to zoom, right button to exit'
	CURSOR,X,Y,3,/NORMAL
	IF !ERR EQ 4 THEN GOTO, FINISH
	IF Y GT 0.5 THEN BEGIN
		DET = 'N1'
		WW = RESULT.W1
		AA = RESULT.A1
		BB = RESULT.B1
		SETVIEW, 1, 1, 1, 2
	END ELSE BEGIN
		DET = 'N2'
		WW = RESULT.W2
		AA = RESULT.A2
		BB = RESULT.B2
		SETVIEW, 1, 1, 2, 2
	ENDELSE
	POS = CONVERT_COORD(X, Y, /NORMAL, /TO_DATA)
	X = WAVE2PIX(DET, POS(0))
	Y = POS(1)
	SETVIEW
	XMIN = (X-50) > 0
	XMAX = (X+50) < 1023
	WW = WW(XMIN:XMAX)
	AA = AA(XMIN:XMAX)
	BB = BB(XMIN:XMAX)
;
RETRY:
	PLOT, WW, AA, XSTYLE=1, XTITLE='Wavelength (A)',		$
		YTITLE = 'NIS ' + STRMID(DET,1,1), TITLE=TITLE
	OPLOT, WW, BB, LINESTYLE=1
;
;  If making a hardcopy, then send it to the printer.
;
	IF !D.NAME EQ 'PS' THEN BEGIN
		PSPLOT, /DELETE
		GOTO, RETRY
	ENDIF
;
;  Pick points to fit, or return to non-zoomed mode.
;
	PRINT, 'Left button (twice) to fit, middle to print, and right to ' + $
		'zoom back out'
	CURSOR,X1,Y1,3,/DATA
;
;  If the middle button was pressed, then make a hardcopy of the screen.
;
	IF !ERR EQ 2 THEN BEGIN
		PS, 'TEMP.PS'
		GOTO, RETRY
	ENDIF
;
;  If the right button was pressed, then go back to the full spectrum.
;
	IF !ERR EQ 4 THEN GOTO, PLOT_ALL
;
;  Read in the second cursor position, and make the same choices.
;
	CURSOR,X2,Y2,3,/DATA
	IF !ERR EQ 2 THEN BEGIN
		PS, 'TEMP.PS'
		GOTO, RETRY
	ENDIF
	IF !ERR EQ 4 THEN GOTO, PLOT_ALL
;
;  Fit the data.
;
	X1 = WAVE2PIX(DET, X1)
	X2 = WAVE2PIX(DET, X2)
	XX = FINDGEN(X2-X1+1) + X1
	YFIT = GAUSSFIT(XX,AA(X1-XMIN:X2-XMIN),PARAM)
	YAVG = PARAM(3) + PARAM(4)*PARAM(1) + PARAM(5)*PARAM(1)^2
	PRINT, DET, PARAM(0)/YAVG, PARAM(1:2)
	WWW = PIX2WAVE(DET, XX)
	OPLOT, WWW, YFIT, PSYM=1
;
;  Decide what to do with the fit.
;
	PRINT, 'Left button to retry, right button to continue'
	CURSOR,X3,Y3,3
	IF !ERR EQ 1 THEN GOTO, RETRY ELSE BEGIN
		OUTFILE = 's' + TRIM(STUDY1(0)) + '_fits.txt'
		OPENU, UNIT, OUTFILE, /GET_LUN, /APPEND
		PRINTF, UNIT, RESULT.DATE_OBS, '  ', DET, PARAM(0)/YAVG, $
			PARAM(1:2)
		FREE_LUN, UNIT
		GOTO, PLOT_ALL
	ENDELSE
;
FINISH:
	SETVIEW
	RETURN
	END
