;+
; Project     :	SOHO - CDS
;
; Name        :	AN_NIMCP_AVG
;
; Purpose     :	Analyze data from runs of NIMCP.
;
; Category    :	Class3, CDS, Analysis, Calibration
;
; Explanation :	Reads in data from runs of NIMCP, and plots up a display
;		comparing the average profiles.  The user can then zoom in on a
;		portion of the display and then fit gaussians to the data to
;		determine the amount of loss of sensitivity.
;
;		This routine uses a cruder approach than that of AN_NIMCP, and
;		it's results are less accurate.  In particular, it seems to
;		overestimate the amount of throughput loss.  However, it is
;		less dependent on the analyzed lines being isolated.
;
; Syntax      :	AN_NIMCP_AVG, STUDY, RESULT
;
; Examples    :	AN_NIMCP_AVG, 2775, RESULT
;
; Inputs      :	STUDY	= The study counter number for the run of NIMCP
;
; Opt. Inputs :	RESULT	= The structure returned from an earlier run of this
;			  routine.
;
; Outputs     :	RESULT	= Structure containing the average traces.
;
;		The parameters from the fits are also stored in a text file
;		with a name formed from the current study number,
;		e.g. "s2775_fits.txt".
;
; Opt. Outputs:	None.
;
; Keywords    :	NOREAD	= If set, then the FITS files are not read.  In this
;			  case, then RESULT must already be defined.
;
; Calls       :	CURVEFIT, PIX2WAVE, WAVE2PIX, SETVIEW
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Prev. Hist. :	None.
;
; History     :	Version 1, 22-May-1996, William Thompson, GSFC
;		Version 2, 09-Sep-1996, William Thompson, GSFC
;			Added printing and output options, better scaling
;		Version 3, 25-Feb-1997, William Thompson, GSFC
;			Add /NOBURNIN to VDS_CALIB call
;
; Contact     :	WTHOMPSON
;-
;
	PRO AN_NIMCP_AVG, STUDY, RESULT, NOREAD=NOREAD
;
	ON_ERROR, 2
;
;  Check the number of parameters.
;
	IF N_PARAMS() NE 2 THEN MESSAGE,	$
		'Syntax: AN_NIMCP_AVG, STUDY, RESULT'
	ERASE
;
;  If the NOREAD keyword was set, then skip reading the files.
;
	IF NOT KEYWORD_SET(NOREAD) THEN BEGIN
;
;  From the study numbers, form the filenames.  Read in the files, and make
;  sure that these are the correct files.
;
		FILENAME1 = 's' + NTRIM(STUDY) + 'r00.fits'
		FILENAME2 = 's' + NTRIM(STUDY) + 'r01.fits'
;
		A = READCDSFITS(FILENAME1)
		IF DATATYPE(A,1) NE 'Structure' THEN MESSAGE,	$
			'FITS file for study #' + NTRIM(STUDY) + ' not found'
		IF TRIM(A.HEADER.OBS_PROG) NE 'NIMCP' THEN MESSAGE,	$
			'Study #' + NTRIM(STUDY) + ' is not a run of NIMCP'
;
		B = READCDSFITS(FILENAME2)
		IF DATATYPE(B,1) NE 'Structure' THEN MESSAGE,	$
			'FITS file for study #' + NTRIM(STUDY) + ' not found'
		IF TRIM(B.HEADER.OBS_PROG) NE 'NIMCP' THEN MESSAGE,	$
			'Study #' + NTRIM(STUDY) + ' is not a run of NIMCP'
;
;  Calibrate the data.
;
		VDS_DEBIAS, A  &  VDS_CALIB, A, /NOBURNIN
		VDS_DEBIAS, B  &  VDS_CALIB, B, /NOBURNIN
		AMISSING = A.DETDESC(0).MISSING
		BMISSING = B.DETDESC(0).MISSING
;
;  Calculate the wavelength scale.
;
		W1 = PIX2WAVE('NIS1', INDGEN(1024))
		W2 = PIX2WAVE('NIS2', INDGEN(1024))
;
;  Extract the average profiles.
;
		X0 = INDGEN(1024) + 12*2.032/1.68
		X1 = INDGEN(1024)
		X2 = INDGEN(1024) - 12*2.032/1.68
;
		A1 = AVERAGE(DETDATA(A,1), 3, MISSING=AMISSING)
		A1 = AVERAGE(A1, 2, MISSING=AMISSING)
		AMIN = MIN(A1(2:1021), MAX=AMAX)
		A1 = (A1 - AMIN) / (AMAX - AMIN)
;
		A2 = AVERAGE(DETDATA(A,2), 3, MISSING=AMISSING)
		A2 = AVERAGE(A2, 2, MISSING=AMISSING)
		AMIN = MIN(A2(2:1021), MAX=AMAX)
		A2 = (A2 - AMIN) / (AMAX - AMIN)
;
;  For the reference profiles with the narrow slit, simply normalize them to a
;  maximum value of unity.
;
		B1 = AVERAGE(REFORM(DETDATA(B,1)), 2, MISSING=BMISSING)
		BMIN = MIN(B1(2:1021), MAX=BMAX)
		B1 = (B1 - BMIN) / (BMAX - BMIN)
;
		B2 = AVERAGE(REFORM(DETDATA(B,2)), 2, MISSING=BMISSING)
		BMIN = MIN(B2(2:1021), MAX=BMAX)
		B2 = (B2 - BMIN) / (BMAX - BMIN)
;
;  Store everything in RESULT.
;
		RESULT = {A1: A1, A2: A2,	$
			  B1: B1, B2: B2,	$
			  W1: W1, W2: W2,	$
			  DATE_OBS: A.HEADER.DATE_OBS}
	ENDIF
;
;  Plot the data.
;
PLOT_ALL:
	ERASE
	SETVIEW, 1, 1, 1, 2
	PLOT, RESULT.W1, RESULT.A1, XSTYLE=1, XTITLE='Wavelength (A)',	$
		YTITLE='NIS 1', TITLE='NIMCP study ' + NTRIM(STUDY),	$
		YRANGE=[0,1.5]
	OPLOT, RESULT.W1, RESULT.B1, LINESTYLE=1
;
	SETVIEW, 1, 1, 2, 2
	PLOT, RESULT.W2, RESULT.A2, XSTYLE=1, XTITLE='Wavelength (A)',	$
		YTITLE='NIS 2', YRANGE=[0,1.5]
	OPLOT, RESULT.W2, RESULT.B2, LINESTYLE=1
;
;  Read in a cursor position, and decide whether or not to zoom.
;
	PRINT, 'Left button to zoom, right button to exit'
	CURSOR,X,Y,3,/NORMAL
	IF !ERR EQ 4 THEN GOTO, FINISH
	IF Y GT 0.5 THEN BEGIN
		DET = 'N1'
		WW = RESULT.W1
		AA = RESULT.A1
		BB = RESULT.B1
		SETVIEW, 1, 1, 1, 2
	END ELSE BEGIN
		DET = 'N2'
		WW = RESULT.W2
		AA = RESULT.A2
		BB = RESULT.B2
		SETVIEW, 1, 1, 2, 2
	ENDELSE
	POS = CONVERT_COORD(X, Y, /NORMAL, /TO_DATA)
	X = WAVE2PIX(DET, POS(0))
	Y = POS(1)
	SETVIEW
	XMIN = (X-50) > 0
	XMAX = (X+50) < 1023
	WW = WW(XMIN:XMAX)
	AA = AA(XMIN:XMAX)
	BB = BB(XMIN:XMAX)
;
RETRY:
	PLOT, WW, AA, XSTYLE=1, XTITLE='Wavelength (A)',	$
		YTITLE = 'NIS ' + STRMID(DET,1,1),		$
		TITLE='NIMCP study ' + NTRIM(STUDY)
	OPLOT, WW, BB, LINESTYLE=1
;
;  If making a hardcopy, then send it to the printer.
;
	IF !D.NAME EQ 'PS' THEN BEGIN
		PSPLOT, /DELETE
		GOTO, RETRY
	ENDIF
;
;  Pick points to fit, or return to non-zoomed mode.
;
	PRINT, 'Left button (twice) to fit, middle to print, and right to ' + $
		'zoom back out'
	CURSOR,X1,Y1,3,/DATA
;
;  If the middle button was pressed, then make a hardcopy of the screen.
;
	IF !ERR EQ 2 THEN BEGIN
		PS, 'TEMP.PS'
		GOTO, RETRY
	ENDIF
;
;  If the right button was pressed, then go back to the full spectrum.
;
	IF !ERR EQ 4 THEN GOTO, PLOT_ALL
;
;  Read in the second cursor position, and make the same choices.
;
	CURSOR,X2,Y2,3,/DATA
	IF !ERR EQ 2 THEN BEGIN
		PS, 'TEMP.PS'
		GOTO, RETRY
	ENDIF
	IF !ERR EQ 4 THEN GOTO, PLOT_ALL
;
;  Fit the data.
;
	IF X1 GT X2 THEN BEGIN
		TEMP = X1
		X1 = X2
		X2 = TEMP
	ENDIF
	X1 = WAVE2PIX(DET, X1)
	X2 = WAVE2PIX(DET, X2)
	XX = FINDGEN(X2-X1+1) + X1
	YFIT = GAUSSFIT(XX,AA(X1-XMIN:X2-XMIN),PARAM)
	YAVG = PARAM(3) + PARAM(4)*PARAM(1) + PARAM(5)*PARAM(1)^2
	PRINT, DET, PARAM(0)/YAVG, PARAM(1:2)
	WWW = PIX2WAVE(DET, XX)
	OPLOT, WWW, YFIT, PSYM=1
;
;  Decide what to do with the fit.
;
	PRINT, 'Left button to retry, right button to continue'
	CURSOR,X3,Y3,3
	IF !ERR EQ 1 THEN GOTO, RETRY ELSE BEGIN
		OUTFILE = 's' + TRIM(STUDY) + '_fits.txt'
		OPENU, UNIT, OUTFILE, /GET_LUN, /APPEND
		PRINTF, UNIT, RESULT.DATE_OBS, '  ', DET, PARAM(0)/YAVG, $
			PARAM(1:2)
		FREE_LUN, UNIT
		GOTO, PLOT_ALL
	ENDELSE
;
FINISH:
	SETVIEW
	RETURN
	END
