	FUNCTION CDS_SLIT6_BURNIN, DATE_IN, WAVE,       $
                ANALYSIS_DATE=ANALYSIS_DATE, ERRMSG=ERRMSG
;+
; Project     :	SOHO - CDS
;
; Name        :	CDS_SLIT6_BURNIN()
;
; Purpose     :	Returns the SLIT6 burn-in data for a given date and wavelength
;
; Explanation :	This routine returns the burn-in due to SLIT6 for a given date
;               and wavelength.
;
; Use         :	Result = CDS_SLIT6_BURNIN( DATE, WAVE )
;
; Inputs      :	DATE = The observation date, in any CDS time format.
;               WAVE = The wavelength, in Angstroms.
;
; Opt. Inputs :	None.
;
; Outputs     :	Messages about the burn-in corrections will be printed to the
;		screen.
;
; Opt. Outputs:	None.
;
; Keywords    :	ANALYSIS_DATE = Date on which the data was analyzed.  This
;                               allows comparison between old and new burn-in
;                               calibrations.  The default is to use the most
;                               up-to-date calibration.
;
;		ERRMSG = If defined and passed, then any error messages will
;			 be returned to the user in this parameter rather than
;			 depending on the MESSAGE routine in IDL.  If no errors
;			 are encountered, then a null string is returned.  In
;			 order to use this feature, ERRMSG must be defined
;			 first, e.g.
;
;				ERRMSG = ''
;				CDS_SLIT6_BURNIN, ERRMSG=ERRMSG, ...
;				IF ERRMSG NE '' THEN ...
;
; Calls       :	
;
; Common      :	None.
;
; Restrictions:	See GET_NIMCP for more information.
;
; Side effects:	The wavelengths displayed will be based on the currently loaded
;		wavelength calibration.  See LOAD_WAVECAL for more information.
;
; Category    :	Class4, Calibration
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 13-November-2003
;
; Modified    :	Version 1, 13-Nov-2003, William Thompson, GSFC
;               Version 2, 09-Jun-2004, William Thompson, GSFC
;                       Corrected bug for pre-accident data
;                       Allow DATE to be an array.
;
; Version     :	Version 2, 09-Jun-2004
;-
;
	ON_ERROR, 2
;
;  Check the number of parameters.
;
	IF N_PARAMS() NE 2 THEN BEGIN
            MESSAGE = 'Syntax:  Result = CDS_SLIT6_BURNIN( DATE, WAVE )'
	    GOTO, HANDLE_ERROR
        ENDIF
;
;  If ANALYSIS_DATE was not passed, then use today.
;
        IF N_ELEMENTS(ANALYSIS_DATE) EQ 0 THEN GET_UTC,ANALYSIS_DATE
;
;  Convert the input dates to TAI.
;
        DATE = ANYTIM2TAI(DATE_IN)
        ADATE = ANYTIM2TAI(ANALYSIS_DATE)
;
;  If DATE is an array, then run this procedure recursively to return an array
;  of results.
;
        IF N_ELEMENTS(DATE) GT 1 THEN BEGIN
            SZ = SIZE(DATE)
            RESULT = MAKE_ARRAY(DIMENSION=SZ[1:SZ[0]],/DOUBLE)
            FOR I=0,N_ELEMENTS(DATE)-1 DO BEGIN
                MESSAGE = ''
                RESULT[I] = CDS_SLIT6_BURNIN(DATE[I],WAVE,ANALYSIS_DATE=ADATE,$
                                          ERRMSG=MESSAGE)
                IF MESSAGE NE '' THEN GOTO, HANDLE_ERROR
            ENDFOR
            RETURN, RESULT
        ENDIF            
;
;  Find all the relevant calibration files.
;
        FILE = FIND_WITH_DEF('burnin_files.dat', '$CDS_VDS_CAL_INT')
        IF FILE EQ '' THEN BEGIN
            MESSAGE = 'Unable to find burnin_files.dat'
            GOTO, HANDLE_ERROR
        ENDIF
        OPENR, UNIT, FILE, /GET_LUN
        LINE = ''
        DELVARX, SYNOP_PRE_NAME,  SYNOP_PRE_TIME
        DELVARX, SYNOP_POST_NAME, SYNOP_POST_TIME
        WHILE NOT EOF(UNIT) DO BEGIN
            READF, UNIT, LINE
            LINE = STRTRIM(STRCOMPRESS(LINE),2)
            IF LINE NE '' THEN BEGIN
                WORDS = STR_SEP(LINE,' ')
                IF STRMID(WORDS[1],0,13) EQ 'synop_cal_pre' THEN BEGIN
                    BOOST_ARRAY, SYNOP_PRE_NAME, WORDS[1]
                    BOOST_ARRAY, SYNOP_PRE_TIME, UTC2TAI(WORDS[0])
                ENDIF
                IF STRMID(WORDS[1],0,14) EQ 'synop_cal_post' THEN BEGIN
                    BOOST_ARRAY, SYNOP_POST_NAME, WORDS[1]
                    BOOST_ARRAY, SYNOP_POST_TIME, UTC2TAI(WORDS[0])
                ENDIF
            ENDIF
        ENDWHILE
        FREE_LUN, UNIT
        SYNOP_PRE_NAME  = CONCAT_DIR('$CDS_VDS_CAL_INT', SYNOP_PRE_NAME)
        SYNOP_POST_NAME = CONCAT_DIR('$CDS_VDS_CAL_INT', SYNOP_POST_NAME)
;
;  Determine which files to read, based on the observation and analysis dates.
;
        POST = DATE GT UTC2TAI('1998-10-01')
        I_SYNOP_PRE  = MAX(WHERE(SYNOP_PRE_TIME  LT ADATE))
        I_SYNOP_POST = MAX(WHERE(SYNOP_POST_TIME LT ADATE))
;
;  Sum up all the depths
;
	DEPTH = 1
;
;  Read in the SYNOP_PRE data.
;
        IF I_SYNOP_PRE GE 0 THEN BEGIN
            OPENR, UNIT, SYNOP_PRE_NAME[I_SYNOP_PRE], /GET_LUN
            LINE = 'String'
            READF, UNIT, LINE
            TAI0 = UTC2TAI(LINE)
            READF, UNIT, LINE
            TAI1 = UTC2TAI(LINE)
            READF, UNIT, LINE
            CORR = DOUBLE(LINE)
            READF, UNIT, LINE
            FREE_LUN, UNIT
            LINE = STRCOMPRESS(STRTRIM(LINE,2))
            PD = DOUBLE(STR_SEP(LINE,' '))
;
;  Adjust based on whether the data is post-recovery
;
            IF POST THEN BEGIN
                IF I_SYNOP_PRE GE 1 THEN PD = 0.813822 * PD
                DTAI = UTC2TAI('1998-Jun-24 11:45')
            END ELSE DTAI = DATE
            IF DTAI GT TAI1 THEN DTAI = TAI1 + (DTAI-TAI1)*CORR
            DTAI = DTAI - TAI0
;
;  Apply the pre-accident slit6 correction.
;
            FILE = FIND_WITH_DEF('avg_slit6.fits', '$CDS_VDS_CAL_INT')
            IF FILE NE '' THEN FXREAD, FILE, AVG_VAL ELSE BEGIN
                MESSAGE, /CONTINUE, 'File avg_slit6.fits not found.' +$
                  '  No SLIT6 correction can be made.'
                AVG_VAL = REPLICATE(0.,1024,2)
            ENDELSE
            AVG_VAL = POLY(AVG_VAL*DTAI, PD) < 0
;
            IF WAVE LT 400 THEN BEGIN
                DEPTH = DEPTH * (1 + AVG_VAL[WAVE2PIX('N1',WAVE),0])
            END ELSE BEGIN
                DEPTH = DEPTH * (1 + AVG_VAL[WAVE2PIX('N2',WAVE),1])
            ENDELSE
        ENDIF
;
;  Read in the SYNOP_POST data.
;
        IF POST THEN BEGIN
	    IF I_SYNOP_POST GE 0 THEN BEGIN
		OPENR, UNIT, SYNOP_POST_NAME[I_SYNOP_POST], /GET_LUN
		LINE = 'String'
		READF, UNIT, LINE
		TAI0 = UTC2TAI(LINE)
		READF, UNIT, LINE
		LINE = STRCOMPRESS(STRTRIM(LINE,2))
		PD0 = DOUBLE(STR_SEP(LINE,' '))
		READF, UNIT, LINE
		TAI1 = UTC2TAI(LINE)
		READF, UNIT, LINE
		LINE = STRCOMPRESS(STRTRIM(LINE,2))
		PD1 = DOUBLE(STR_SEP(LINE,' '))
		IF NOT EOF(UNIT) THEN BEGIN
		    READF, UNIT, LINE
		    TAI2 = UTC2TAI(LINE)
		    READF, UNIT, LINE
		    CORR = DOUBLE(LINE)
		END ELSE BEGIN
		    TAI2 = UTC2TAI('2000-10-28T21:24')
		    CORR = 1
                ENDELSE
                IF NOT EOF(UNIT) THEN BEGIN
                    READF, UNIT, LINE
                    TAI13 = UTC2TAI(LINE)
                    READF, UNIT, LINE
                    TAI3 = UTC2TAI(LINE)
                    READF, UNIT, LINE
                    LINE = STRCOMPRESS(STRTRIM(LINE,2))
                    PD3 = DOUBLE(STR_SEP(LINE,' '))
                END ELSE TAI13 = -1
		FREE_LUN, UNIT
;
;
                DTAI = DATE
                IF DTAI GT TAI2 THEN DTAI = TAI2 + (DTAI-TAI2)*CORR
                IF DTAI GT TAI1 THEN BEGIN
                    DTAI = DTAI - TAI1
                    PD = PD1
                END ELSE BEGIN
                    DTAI = DTAI - TAI0
                    PD = PD0
                ENDELSE
;
;  Apply the post-accident slit6 correction.
;
                FILE = FIND_WITH_DEF('avg_slit6_post.fits', '$CDS_VDS_CAL_INT')
                IF FILE NE '' THEN FXREAD, FILE, AVG_VAL ELSE BEGIN
                    MESSAGE, /CONTINUE, 'File avg_slit6_post.fits not found.'+$
                      '  No post-recovery SLIT6 correction can be made.'
                    AVG_VAL = REPLICATE(0.,1024,2)
                ENDELSE
                DTAI = AVG_VAL*DTAI
                AVG_VAL = NIMCP_DEPTH_FUNCT(DTAI, PD) < 0
                IF TAI13 GT 0 THEN BEGIN
                    W = WHERE(DTAI GT TAI13-TAI3, COUNT)
                    IF COUNT GT 0 THEN      $
                      AVG_VAL(W) = NIMCP_DEPTH_FUNCT(DTAI(W),PD3) < 0
                ENDIF
;
                IF WAVE LT 400 THEN BEGIN
                    DEPTH = DEPTH * (1 + AVG_VAL[WAVE2PIX('N1',WAVE),0])
                END ELSE BEGIN
                    DEPTH = DEPTH * (1 + AVG_VAL[WAVE2PIX('N2',WAVE),1])
                ENDELSE
            ENDIF
        ENDIF
;
	GOTO, FINISH
;
;  Error handling point
;
HANDLE_ERROR:
	IF N_ELEMENTS(ERRMSG) NE 0 THEN ERRMSG = 'CDS_SLIT6_BURNIN: ' + $
                MESSAGE ELSE MESSAGE, MESSAGE, /CONTINUE
;
;  Return.
;
FINISH:
	RETURN, 1 - DEPTH
	END
