	PRO DISPLAY_CDS_BURNIN, DATE_IN, ANALYSIS_DATE=ANALYSIS_DATE, $
                                ERRMSG=ERRMSG
;+
; Project     :	SOHO - CDS
;
; Name        :	DISPLAY_CDS_BURNIN
;
; Purpose     :	Displays the relevant NIS burn-in data for a given date
;
; Explanation :	This routine reads in the burn-in data for a given date,
;		characterizing lines which have been partially burned into the
;		detector.  The total amount of burn-in for each line is then
;		printed to the screen.
;
; Use         :	DISPLAY_CDS_BURNIN, DATE
;
; Inputs      :	DATE = The observation date, in any CDS time format.
;
; Opt. Inputs :	None.
;
; Outputs     :	Messages about the burn-in corrections will be printed to the
;		screen.
;
; Opt. Outputs:	None.
;
; Keywords    :	ANALYSIS_DATE = Date on which the data was analyzed.  This
;                               allows comparison between old and new burn-in
;                               calibrations.  The default is to use the most
;                               up-to-date calibration.
;
;		ERRMSG = If defined and passed, then any error messages will
;			 be returned to the user in this parameter rather than
;			 depending on the MESSAGE routine in IDL.  If no errors
;			 are encountered, then a null string is returned.  In
;			 order to use this feature, ERRMSG must be defined
;			 first, e.g.
;
;				ERRMSG = ''
;				DISPLAY_CDS_BURNIN, ERRMSG=ERRMSG, ...
;				IF ERRMSG NE '' THEN ...
;
; Calls       :	
;
; Common      :	None.
;
; Restrictions:	See GET_NIMCP for more information.
;
; Side effects:	The wavelengths displayed will be based on the currently loaded
;		wavelength calibration.  See LOAD_WAVECAL for more information.
;
; Category    :	Class4, Calibration
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 13-November-2003
;
; Modified    :	Version 1, 13-Nov-2003, William Thompson, GSFC
;
; Version     :	Version 1, 13-Nov-2003
;-
;
	ON_ERROR, 2
;
;  Check the number of parameters.
;
	IF N_PARAMS() NE 1 THEN BEGIN
            MESSAGE = 'Syntax:  DISPLAY_CDS_BURNIN, DATE'
	    GOTO, HANDLE_ERROR
        ENDIF
;
;  If ANALYSIS_DATE was not passed, then use today.
;
        IF N_ELEMENTS(ANALYSIS_DATE) EQ 0 THEN GET_UTC,ANALYSIS_DATE
;
;  Convert the input dates to TAI.
;
        DATE = ANYTIM2TAI(DATE_IN)
        ADATE = ANYTIM2TAI(ANALYSIS_DATE)
;
;  Find all the relevant calibration files.
;
        FILE = FIND_WITH_DEF('burnin_files.dat', '$CDS_VDS_CAL_INT')
        IF FILE EQ '' THEN BEGIN
            MESSAGE = 'Unable to find burnin_files.dat'
            GOTO, HANDLE_ERROR
        ENDIF
        OPENR, UNIT, FILE, /GET_LUN
        LINE = ''
        DELVARX, NIMCP_PRE_NAME,  NIMCP_PRE_TIME
        DELVARX, NIMCP_POST_NAME, NIMCP_POST_TIME
        DELVARX, SYNOP_PRE_NAME,  SYNOP_PRE_TIME
        DELVARX, SYNOP_POST_NAME, SYNOP_POST_TIME
        WHILE NOT EOF(UNIT) DO BEGIN
            READF, UNIT, LINE
            LINE = STRTRIM(STRCOMPRESS(LINE),2)
            IF LINE NE '' THEN BEGIN
                WORDS = STR_SEP(LINE,' ')
                IF STRMID(WORDS[1],0,13) EQ 'synop_cal_pre' THEN BEGIN
                    BOOST_ARRAY, SYNOP_PRE_NAME, WORDS[1]
                    BOOST_ARRAY, SYNOP_PRE_TIME, UTC2TAI(WORDS[0])
                END ELSE IF STRMID(WORDS[1],0,14) EQ 'synop_cal_post' THEN BEGIN
                    BOOST_ARRAY, SYNOP_POST_NAME, WORDS[1]
                    BOOST_ARRAY, SYNOP_POST_TIME, UTC2TAI(WORDS[0])
                END ELSE IF STRMID(WORDS[1],0,14) EQ 'nimcp_cal_post' THEN BEGIN
                    BOOST_ARRAY, NIMCP_POST_NAME, WORDS[1]
                    BOOST_ARRAY, NIMCP_POST_TIME, UTC2TAI(WORDS[0])
                END ELSE BEGIN
                    BOOST_ARRAY, NIMCP_PRE_NAME, WORDS[1]
                    BOOST_ARRAY, NIMCP_PRE_TIME, UTC2TAI(WORDS[0])
                ENDELSE
            ENDIF
        ENDWHILE
        FREE_LUN, UNIT
        SYNOP_PRE_NAME  = CONCAT_DIR('$CDS_VDS_CAL_INT', SYNOP_PRE_NAME)
        SYNOP_POST_NAME = CONCAT_DIR('$CDS_VDS_CAL_INT', SYNOP_POST_NAME)
        NIMCP_PRE_NAME  = CONCAT_DIR('$CDS_VDS_CAL_INT', NIMCP_PRE_NAME)
        NIMCP_POST_NAME = CONCAT_DIR('$CDS_VDS_CAL_INT', NIMCP_POST_NAME)
;
;  Determine which files to read, based on the observation and analysis dates.
;
        IF DATE GT UTC2TAI('1998-10-01') THEN BEGIN
            POST = 1
            CALIBRATION = MAX(WHERE(NIMCP_POST_TIME LT ADATE)) + 1
            IF CALIBRATION LT 1 THEN BEGIN
                MESSAGE = 'Unable to read burn-in data for that analysis date.'
                GOTO, HANDLE_ERROR
            ENDIF
        END ELSE BEGIN
            POST = 0
            CALIBRATION = MAX(WHERE(NIMCP_PRE_TIME LT ADATE)) + 1
            IF CALIBRATION LT 2 THEN BEGIN
                MESSAGE = 'Unable to read burn-in data for that analysis date.'
                GOTO, HANDLE_ERROR
            ENDIF
        ENDELSE
        I_SYNOP_PRE  = MAX(WHERE(SYNOP_PRE_TIME  LT ADATE))
        I_SYNOP_POST = MAX(WHERE(SYNOP_POST_TIME LT ADATE))
;
;  Get the information from GET_NIMCP.
;
 	MESSAGE = ''
	GET_NIMCP, DATE, OUTPUT, ERRMSG=MESSAGE, CALIBRATION=CALIBRATION, $
		POST=POST
	IF MESSAGE NE '' THEN GOTO, HANDLE_ERROR
;
;  Find all the wavelengths.
;
	WAVES = FLTARR(N_ELEMENTS(OUTPUT))
	FOR I = 0,N_ELEMENTS(WAVES)-1 DO WAVES(I) =	$
		PIX2WAVE(OUTPUT(I).DETECTOR, OUTPUT(I).CENTER)
;
;  Sort on the wavelengths, and find all the unique values.
;
	S = SORT(WAVES)
	WAVES = WAVES(S)
	OUTPUT = OUTPUT(S)
	II = UNIQ(WAVES)
;
;  Sum up all the depths
;
	DEPTHS = FLTARR(N_ELEMENTS(II))
	FOR I=0,N_ELEMENTS(II)-1 DO BEGIN
	    W = WHERE(WAVES EQ WAVES(II(I)))
	    DEPTHS(I) = TOTAL(OUTPUT(W).DEPTH) + 1
	ENDFOR
	WAVES = WAVES(II)
;
;  Read in the SYNOP_PRE data.
;
        IF I_SYNOP_PRE GE 0 THEN BEGIN
            OPENR, UNIT, SYNOP_PRE_NAME[I_SYNOP_PRE], /GET_LUN
            LINE = 'String'
            READF, UNIT, LINE
            TAI0 = UTC2TAI(LINE)
            READF, UNIT, LINE
            TAI1 = UTC2TAI(LINE)
            READF, UNIT, LINE
            CORR = DOUBLE(LINE)
            READF, UNIT, LINE
            FREE_LUN, UNIT
            LINE = STRCOMPRESS(STRTRIM(LINE,2))
            PD = DOUBLE(STR_SEP(LINE,' '))
;
;  Adjust based on whether the data is post-recovery
;
            IF POST THEN BEGIN
                IF I_SYNOP_PRE GE 1 THEN PD = 0.813822 * PD
                DTAI = UTC2TAI('1998-Jun-24 11:45')
            END ELSE DTAI = DATE
            IF DTAI GT TAI1 THEN DTAI = TAI1 + (DTAI-TAI1)*CORR
            DTAI = DTAI - TAI0
;
;  Apply the pre-accident slit6 correction.
;
            FILE = FIND_WITH_DEF('avg_slit6.fits', '$CDS_VDS_CAL_INT')
            IF FILE NE '' THEN FXREAD, FILE, AVG_VAL ELSE BEGIN
                MESSAGE, /CONTINUE, 'File avg_slit6.fits not found.' +$
                  '  No SLIT6 correction can be made.'
                AVG_VAL = REPLICATE(0.,1024,2)
            ENDELSE
            AVG_VAL = POLY(AVG_VAL*DTAI, PD) < 0
;
            W1 = WHERE(WAVES LT 400, COMPLEMENT=W2)
            IF W1[0] NE -1 THEN DEPTHS[W1] = DEPTHS[W1] *     $
                    (1 + AVG_VAL[WAVE2PIX('N1',WAVES[W1]),0])
            IF W2[0] NE -1 THEN DEPTHS[W2] = DEPTHS[W2] *     $
                    (1 + AVG_VAL[WAVE2PIX('N2',WAVES[W2]),1])
        ENDIF
;
;  Read in the SYNOP_POST data.
;
        IF POST THEN BEGIN
	    IF I_SYNOP_POST GE 0 THEN BEGIN
		OPENR, UNIT, SYNOP_POST_NAME[I_SYNOP_POST], /GET_LUN
		LINE = 'String'
		READF, UNIT, LINE
		TAI0 = UTC2TAI(LINE)
		READF, UNIT, LINE
		LINE = STRCOMPRESS(STRTRIM(LINE,2))
		PD0 = DOUBLE(STR_SEP(LINE,' '))
		READF, UNIT, LINE
		TAI1 = UTC2TAI(LINE)
		READF, UNIT, LINE
		LINE = STRCOMPRESS(STRTRIM(LINE,2))
		PD1 = DOUBLE(STR_SEP(LINE,' '))
		IF NOT EOF(UNIT) THEN BEGIN
		    READF, UNIT, LINE
		    TAI2 = UTC2TAI(LINE)
		    READF, UNIT, LINE
		    CORR = DOUBLE(LINE)
		END ELSE BEGIN
		    TAI2 = UTC2TAI('2000-10-28T21:24')
		    CORR = 1
                ENDELSE
                IF NOT EOF(UNIT) THEN BEGIN
                    READF, UNIT, LINE
                    TAI13 = UTC2TAI(LINE)
                    READF, UNIT, LINE
                    TAI3 = UTC2TAI(LINE)
                    READF, UNIT, LINE
                    LINE = STRCOMPRESS(STRTRIM(LINE,2))
                    PD3 = DOUBLE(STR_SEP(LINE,' '))
                END ELSE TAI13 = -1
		FREE_LUN, UNIT
;
;
                DTAI = DATE
                IF DTAI GT TAI2 THEN DTAI = TAI2 + (DTAI-TAI2)*CORR
                IF DTAI GT TAI1 THEN BEGIN
                    DTAI = DTAI - TAI1
                    PD = PD1
                END ELSE BEGIN
                    DTAI = DTAI - TAI0
                    PD = PD0
                ENDELSE
;
;  Apply the post-accident slit6 correction.
;
                FILE = FIND_WITH_DEF('avg_slit6_post.fits', '$CDS_VDS_CAL_INT')
                IF FILE NE '' THEN FXREAD, FILE, AVG_VAL ELSE BEGIN
                    MESSAGE, /CONTINUE, 'File avg_slit6_post.fits not found.'+$
                      '  No post-recovery SLIT6 correction can be made.'
                    AVG_VAL = REPLICATE(0.,1024,2)
                ENDELSE
                DTAI = AVG_VAL*DTAI
                AVG_VAL = NIMCP_DEPTH_FUNCT(DTAI, PD) < 0
                IF TAI13 GT 0 THEN BEGIN
                    W = WHERE(DTAI GT TAI13-TAI3, COUNT)
                    IF COUNT GT 0 THEN      $
                      AVG_VAL(W) = NIMCP_DEPTH_FUNCT(DTAI(W),PD3) < 0
                ENDIF
;
                W1 = WHERE(WAVES LT 400, COMPLEMENT=W2)
                IF W1[0] NE -1 THEN DEPTHS[W1] = DEPTHS[W1] *     $
                        (1 + AVG_VAL[WAVE2PIX('N1',WAVES[W1]),0])
                IF W2[0] NE -1 THEN DEPTHS[W2] = DEPTHS[W2] *     $
                        (1 + AVG_VAL[WAVE2PIX('N2',WAVES[W2]),1])
            ENDIF
        ENDIF
;
;  Filter out any entries with a depth of zero.
;
        DEPTHS = DEPTHS - 1
	W = WHERE(DEPTHS NE 0, COUNT)
	IF COUNT EQ 0 THEN BEGIN
	    PRINT, 'No burn-in for date ' + ANYTIM2UTC(DATE,/CCSDS,/DATE)
	    GOTO, FINISH
	ENDIF
	WAVES = WAVES(W)
	DEPTHS = DEPTHS(W)
;
;  Print out the information.
;
	PRINT, ''
	PRINT, '   Wavelength  Burn-in depth'
	PRINT, ''
	FOR I=0,N_ELEMENTS(WAVES)-1 DO PRINT, WAVES(I), -DEPTHS(I)
	PRINT, ''
	GOTO, FINISH
;
;  Error handling point
;
HANDLE_ERROR:
	IF N_ELEMENTS(ERRMSG) NE 0 THEN ERRMSG = 'DISPLAY_CDS_BURNIN: ' + $
                MESSAGE ELSE MESSAGE, MESSAGE, /CONTINUE
;
;  Return.
;
FINISH:
	RETURN
	END
