	PRO DISPLAY_NIMCP, DATE, ERRMSG=ERRMSG, CALIBRATION=CALIBRATION, $
		POST=POST
;+
; Project     :	SOHO - CDS
;
; Name        :	DISPLAY_NIMCP
;
; Purpose     :	Displays the relevant NIMCP data for a given date
;
; Explanation :	This routine reads in the NIMCP data for a given date,
;		characterizing lines which have been partially burned into the
;		detector.  The total amount of burn-in for each line is then
;		printed to the screen.
;
; Use         :	DISPLAY_NIMCP, DATE
;
; Inputs      :	DATE = The observation date, in any CDS time format.
;
; Opt. Inputs :	None.
;
; Outputs     :	Messages about the burn-in corrections will be printed to the
;		screen.
;
; Opt. Outputs:	None.
;
; Keywords    :	CALIBRATION = The number of the calibration to use.  Must be 2
;			      or greater.  (With /POST, it must be 1 or
;			      greater.)  The default is to use the most
;			      up-to-date calibration.
;
;		POST	 = When used with the CALIBRATION keyword, specifies
;			   that the post-recovery databases be searched instead
;			   of the pre-accident databases.  The post-recovery
;			   databases have a completely separate list of version
;			   numbers, starting with 1.
;
;		ERRMSG = If defined and passed, then any error messages will
;			 be returned to the user in this parameter rather than
;			 depending on the MESSAGE routine in IDL.  If no errors
;			 are encountered, then a null string is returned.  In
;			 order to use this feature, ERRMSG must be defined
;			 first, e.g.
;
;				ERRMSG = ''
;				DISPLAY_NIMCP, ERRMSG=ERRMSG, ...
;				IF ERRMSG NE '' THEN ...
;
;
; Calls       :	
;
; Common      :	None.
;
; Restrictions:	See GET_NIMCP for more information.
;
; Side effects:	The wavelengths displayed will be based on the currently loaded
;		wavelength calibration.  See LOAD_WAVECAL for more information.
;
; Category    :	Class4, Calibration
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 10 September 1996
;
; Modified    :	Version 1, 08-Mar-1999, William Thompson, GSFC
;		Version 2, 09-Aug-1999, William Thompson, GSFC
;			Added keyword POST
;
; Version     :	Version 2, 09-Aug-1999
;-
;
	ON_ERROR, 2
;
;  Check the number of parameters.
;
	IF N_PARAMS() NE 1 THEN BEGIN
            MESSAGE = 'Syntax:  DISPLAY_NIMCP, DATE'
	    GOTO, HANDLE_ERROR
	ENDIF
;
;  Get the information from GET_NIMCP.
;
	MESSAGE = ''
	GET_NIMCP, DATE, OUTPUT, ERRMSG=MESSAGE, CALIBRATION=CALIBRATION, $
		POST=POST
	IF MESSAGE NE '' THEN GOTO, HANDLE_ERROR
;
;  Find all the wavelengths.
;
	WAVES = FLTARR(N_ELEMENTS(OUTPUT))
	FOR I = 0,N_ELEMENTS(WAVES)-1 DO WAVES(I) =	$
		PIX2WAVE(OUTPUT(I).DETECTOR, OUTPUT(I).CENTER)
;
;  Sort on the wavelengths, and find all the unique values.
;
	S = SORT(WAVES)
	WAVES = WAVES(S)
	OUTPUT = OUTPUT(S)
	II = UNIQ(WAVES)
;
;  Sum up all the depths
;
	DEPTHS = FLTARR(N_ELEMENTS(II))
	FOR I=0,N_ELEMENTS(II)-1 DO BEGIN
	    W = WHERE(WAVES EQ WAVES(II(I)))
	    DEPTHS(I) = TOTAL(OUTPUT(W).DEPTH)
	ENDFOR
	WAVES = WAVES(II)
;
;  Filter out any entries with a depth of zero.
;
	W = WHERE(DEPTHS NE 0, COUNT)
	IF COUNT EQ 0 THEN BEGIN
	    PRINT, 'No burn-in for date ' + ANYTIM2UTC(DATE,/CCSDS,/DATE)
	    GOTO, FINISH
	ENDIF
	WAVES = WAVES(W)
	DEPTHS = DEPTHS(W)
;
;  Print out the information.
;
	PRINT, ''
	PRINT, '   Wavelength  Burn-in depth'
	PRINT, ''
	FOR I=0,N_ELEMENTS(WAVES)-1 DO PRINT, WAVES(I), -DEPTHS(I)
	PRINT, ''
	GOTO, FINISH
;
;  Error handling point
;
HANDLE_ERROR:
	IF N_ELEMENTS(ERRMSG) NE 0 THEN ERRMSG = 'DISPLAY_NIMCP: ' + MESSAGE $
		ELSE MESSAGE, MESSAGE, /CONTINUE
;
;  Return.
;
FINISH:
	RETURN
	END
