	PRO GET_EFF_AREA, DATE_OBS, SPECTRUM, EFF_AREA, ERRMSG=ERRMSG
;+
; Project     :	SOHO - CDS
;
; Name        :	GET_EFF_AREA
;
; Purpose     :	Gets an overall effective area curve from database
;
; Category    :	CDS, Calibration, Class3
;
; Explanation :	Looks up the correct calibration curve for the effective area
;		as a function of mirror position of one of the CDS spectral
;		ranges.  First, the software finds the largest effect date
;		which is on or earlier than the observation date.  If there are
;		multiple entries with the same effective date, then the one
;		with the latest entry date is used.
;
; Syntax      :	GET_EFF_AREA, DATE_OBS, SPECTRUM, EFF_AREA
;
; Examples    :	GET_EFF_AREA, '1996/09/16', 'N1', EFF_AREA
;
; Inputs      :	DATE_OBS = The date the observation was made.
;
;		SPECTRUM = A character string representing the spectrum,
;			   e.g. "N1" or "NIS1".
;
; Opt. Inputs :	None.
;
; Outputs     :	EFF_AREA = The effective area curve, as a function of mirror
;			   position.
;
; Opt. Outputs:	None.
;
; Keywords    :	ERRMSG = If defined and passed, then any error messages will be
;			 returned to the user in this parameter rather than
;			 depending on the MESSAGE routine in IDL.  If no errors
;			 are encountered, then a null string is returned.  In
;			 order to use this feature, ERRMSG must be defined
;			 first, e.g.
;
;				ERRMSG = ''
;				GET_EFF_AREA, ERRMSG=ERRMSG, ...
;				IF ERRMSG NE '' THEN ...
;
; Calls       :	ANYTIM2TAI, DBOPEN, DBFIND, ANYTIM2UTC, DBSORT, DBEXT, DBCLOSE
;
; Common      :	None
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Prev. Hist. :	None.
;
; History     :	Version 1, 16-Sep-1996, William Thompson, GSFC
;
; Contact     :	WTHOMPSON
;-
;
	ON_ERROR, 2
;
;  Check the input parameters.
;
	IF N_PARAMS() NE 3 THEN BEGIN
	    MESSAGE = 'Syntax:  GET_EFF_AREA, DATE_OBS, SPECTRUM, EFF_AREA'
	    GOTO, HANDLE_ERROR
	ENDIF
;
;  Convert the observation date to TAI format.  Reformat the time to
;  millisecond accuracy.  This is necessary so that the times are written out
;  in a controlled way.
;
	TAI_OBS = DOUBLE( STRING( ANYTIM2TAI(DATE_OBS), FORMAT='(F15.3)' ))
;
;  Parse the spectrum identifier.
;
	S = STRTRIM(SPECTRUM,2)
	SG = STRUPCASE(STRMID(S,0,1) + STRMID(S,STRLEN(S)-1,1))
	IF (SG NE 'N1') AND (SG NE 'N2') AND (SG NE 'G1') AND (SG NE 'G2') $
		AND (SG NE 'G3') AND (SG NE 'G4') THEN BEGIN
	    MESSAGE = 'Unrecognized spectrum identifier'
	    GOTO, HANDLE_ERROR
	ENDIF
;
;  Find all entries in the database matching the search criteria.
;
	DBOPEN, 'eff_area'
	ENTRIES = DBFIND('SPECTRUM=' + SG + ',DATE_EFF<' +	$
		TRIM(TAI_OBS,'(F15.3)'), /SILENT)
;
;  If nothing found, then pick the earliest date in the database.
;
	IF ENTRIES(0) LE 0 THEN BEGIN
	    MESSAGE, /CONTINUE, 'No data found for date ' +	$
		    ANYTIM2UTC(DATE_OBS,/VMS) + ', using earliest date'
	    ENTRIES = DBFIND('SPECTRUM=' + SG, /SILENT)
	    IF ENTRIES(0) LE 0 THEN BEGIN
		MESSAGE = 'No data found for spectrum ' + SPECTRUM
		GOTO, HANDLE_ERROR
	    ENDIF
	    IF N_ELEMENTS(ENTRIES) GT 1 THEN	$
		    ENTRIES = DBSORT(ENTRIES, 'DATE_EFF')
;
;  Otherwise, pick the latest date matching the search criteria.
;
	END ELSE IF N_ELEMENTS(ENTRIES) GT 1 THEN	$
		ENTRIES = DBSORT(ENTRIES, 'DATE_EFF', REVERSE=[1])
;
;  Find all entries with the selected effective date, and sort on the entry
;  date.
;
	DBEXT, ENTRIES(0), 'date_eff', DATE_EFF
	ENTRIES = DBFIND('SPECTRUM=' + SG + ',DATE_EFF=' +	$
		TRIM(DATE_EFF,'(F15.3)'), /SILENT)
	IF N_ELEMENTS(ENTRIES) GT 1 THEN	$
		ENTRIES = DBSORT(ENTRIES, 'DATE', REVERSE=[1])
;
;  Extract the data.
;
	DBEXT, ENTRIES(0), 'eff_area', EFF_AREA
	DBCLOSE
	RETURN
;
;  Error handling point.
;
HANDLE_ERROR:
	IF N_ELEMENTS(ERRMSG) NE 0 THEN ERRMSG = 'GET_EFF_AREA: ' + MESSAGE $
		ELSE MESSAGE, MESSAGE
;
	END
