	PRO GET_NIMCP, DATE, OUTPUT, ERRMSG=ERRMSG, CALIBRATION=CALIBRATION, $
		POST=POST, PRE_ONLY=PRE_ONLY
;+
; Project     :	SOHO - CDS
;
; Name        :	GET_NIMCP
;
; Purpose     :	Gets the relevant NIMCP data for a given date
;
; Explanation :	This routine reads in the NIMCP data for a given date,
;		characterizing lines which have been partially burned into the
;		detector.
;
; Use         :	GET_NIMCP, DATE, OUTPUT
;
; Inputs      :	DATE = The observation date, in any CDS time format.
;
; Opt. Inputs :	None.
;
; Outputs     :	OUTPUT = An array of structures containing the following tags.
;
;			DETECTOR = Either "N1" or "N2".
;			DEPTH	 = The amount of burn-in, e.g. -0.1.
;			CENTER	 = The center pixel of the burn-in.
;			WIDTH	 = The gaussian width of the burn-in.
;			SHAPE	 = A shape parameter, denoting a departure from
;				   Gaussian profiles.
;			POST	 = Set to 1 or 2 if burn-in is post-accident.
;				   The value 2 represents data with broadened
;				   Gaussian shapes.
;
;		Note that some lines will appear multiple times--the total
;		burn-in will be the sum of all the depths for that line.
;
; Opt. Outputs:	None.
;
; Keywords    :	CALIBRATION = The number of the calibration to use.  Must be 2
;			      or greater.  (With /POST, it must be 1 or
;			      greater.)  The default is to use the most
;			      up-to-date calibration.
;
;		POST	 = When used with the CALIBRATION keyword, specifies
;			   that the post-recovery databases be searched instead
;			   of the pre-accident databases.  The post-recovery
;			   databases have a completely separate list of version
;			   numbers, starting with 1.
;
;		PRE_ONLY = If set, and the observation data is post-recovery,
;			   then only the residual burn-in from pre-accident
;			   lines are returned.
;
;		ERRMSG = If defined and passed, then any error messages will
;			 be returned to the user in this parameter rather than
;			 depending on the MESSAGE routine in IDL.  If no errors
;			 are encountered, then a null string is returned.  In
;			 order to use this feature, ERRMSG must be defined
;			 first, e.g.
;
;				ERRMSG = ''
;				GET_NIMCP, ERRMSG=ERRMSG, ...
;				IF ERRMSG NE '' THEN ...
;
;
; Calls       :	READ_NIMCP_CAL, NIMCP_DEPTH_FUNCT, UTC2TAI, FIND_WITH_DEF,
;		NTRIM
;
; Common      :	None.
;
; Restrictions:	Requires the file nimcp_cal2.dat in the directory given by the
;		environment variable CDS_VDS_CAL_INT.  There may also be higher
;		versions, e.g. nimcp_cal3.dat, nimcp_cal4.dat, etc.
;
;		Between the dates 1-Apr-1996 and 12-May-1996, the depths are
;		linearly interpolated so that all are 0 on 1-Apr-1996.
;
; Side effects:	None.
;
; Category    :	Class4, Calibration
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 10 September 1996
;
; Modified    :	Version 1, William Thompson, GSFC, 10 September 1996
;		Version 2, William Thompson, GSFC, 23 September 1996
;			Corrected bug where file was left open.
;		Version 3, William Thompson, GSFC, 25 February 1997
;			Modified to use new nimcp_cal2.dat file.
;		Version 4, William Thompson, GSFC, 11 August 1997
;			Modified to use newest nimcp_cal#.dat file.
;			Added CALIBRATION keyword
;		Version 5, William Thompson, GSFC, 2 June 1998
;			Optionally read shape parameter from data file.
;		Version 6, William Thompson, GSFC, 11 June 1998
;			Split off reading of file to READ_NIMCP_CAL
;		Version 7, William Thompson, GSFC, 17 November 1998
;			As temporary fix, don't extrapolate burn-in past loss
;			of contact on 24-Jun-1998.
;		Version 8, William Thompson, GSFC, 14 January 1999
;			As a temporary fix, reduce all burn-ins by 81% for
;			post-recovery data.
;		Version 9, William Thompson, GSFC, 25 February 1999
;			Fix typo disabling use of CALIBRATION keyword.
;		Version 10, William Thompson, GSFC, 6 August 1999
;			Added keywords POST and PRE_ONLY.
;		Version 11, 30-Sep-1999, William Thompson, GSFC
;			Added POST to the output structure.
;		Version 12, 12-Jan-2000, William Thompson, GSFC
;			Differentiate between ".dat" and "a.dat" files for
;			post-recovery data (POST=1 or 2).
;
; Version     :	Version 12, 12-Jan-2000
;-
;
	ON_ERROR, 2
;
;  Define a null result.  If the routine is successful, this will be updated
;  later.
;
	OUTPUT = {DETECTOR: 'N1', DEPTH: 0.0D0, CENTER: 0.0D0, WIDTH: 1.0D0, $
		SHAPE: 0.0D0, POST: 0}
;
;  Check the number of parameters.
;
	IF N_PARAMS() NE 2 THEN BEGIN
            MESSAGE = 'Syntax:  GET_NIMCP, DATE, OUTPUT'
	    GOTO, HANDLE_ERROR
	ENDIF
;
;  Check the input parameter.
;
	IF N_ELEMENTS(DATE) NE 1 THEN BEGIN
            MESSAGE = 'Input parameter must be a scalar'
            GOTO, HANDLE_ERROR
	ENDIF
;
;  Convert the date into a TAI time.
;
	MESSAGE = ''
	TAI_ACCIDENT = ANYTIM2TAI('1998-Jun-24 11:45',ERRMSG=MESSAGE)
	IF MESSAGE NE '' THEN GOTO, HANDLE_ERROR
	TAI = ANYTIM2TAI(DATE,ERRMSG=MESSAGE)
	IF MESSAGE NE '' THEN GOTO, HANDLE_ERROR
	TAI_POST = TAI GT TAI_ACCIDENT
	TAI_SAVE = TAI
;
;  Check the value of the calibration parameter.  If not passed, or the
;  requested file can't be found, then set FIND=1 to tell READ_NIMCP_CAL to
;  read the most current calibration.
;
	IF N_ELEMENTS(CALIBRATION) EQ 1 THEN BEGIN
	    IF KEYWORD_SET(POST) THEN BEGIN
		IF CALIBRATION LT 1 THEN BEGIN
		    MESSAGE = 'CALIBRATION must be 1 or greater (with /POST)'
		    GOTO, HANDLE_ERROR
		ENDIF
		NAME = 'nimcp_cal_pre' + NTRIM(CALIBRATION) + '.dat'
		FILENAME = FIND_WITH_DEF(NAME, '$CDS_VDS_CAL_INT')
		IF FILENAME NE '' THEN PRE = 1 ELSE PRE = 0
	    END ELSE BEGIN
		IF CALIBRATION LT 2 THEN BEGIN
		    MESSAGE = 'CALIBRATION must be 2 or greater'
		    GOTO, HANDLE_ERROR
		ENDIF
		PRE = 0
		NAME = 'nimcp_cal' + NTRIM(CALIBRATION) + 'a.dat'
		FILENAME = FIND_WITH_DEF(NAME, '$CDS_VDS_CAL_INT')
		IF FILENAME EQ '' THEN BEGIN
		    NAME = 'nimcp_cal' + NTRIM(CALIBRATION) + '.dat'
		    FILENAME = FIND_WITH_DEF(NAME, '$CDS_VDS_CAL_INT')
		ENDIF
	    ENDELSE
;
	    IF FILENAME EQ '' THEN BEGIN
		FIND = 1
		PRE = 0
	    END ELSE BEGIN
		FIND = 0
		TAI_POST = 0
	    ENDELSE
	END ELSE BEGIN
	    FIND = 1
	    PRE = 0
	ENDELSE
;
;  Read in the calibration file.  If an old format file was read, then don't go
;  beyond the loss of ; contact on 24-Jun-1998.
;
	READ_NIMCP_CAL, FILENAME, NIMCP, FIND=FIND, PRE=PRE, POST=TAI_POST
	IF NOT KEYWORD_SET(PRE) THEN TAI = TAI < TAI_ACCIDENT
;
;  If after 12 May 1996, then use the data as is.  Otherwise, interpolate so
;  that the depth is zero for dates earlier than 1 April 1996
;
	LAUNCHDATE = UTC2TAI('02-Dec-1995')
	TAI0 = UTC2TAI('01-Apr-1996')
	TAI1 = UTC2TAI('12-May-1996')
	IF TAI LE TAI0 THEN GOTO, FINISH
	IF TAI GE TAI1 THEN BEGIN
	    IF KEYWORD_SET(PRE) THEN BEGIN
		DEPTH = DBLARR(N_ELEMENTS(NIMCP))
		FOR I=0,N_ELEMENTS(NIMCP)-1 DO DEPTH(I) =	$
			POLY(TAI-LAUNCHDATE, NIMCP(I).PD)
	    END ELSE DEPTH = NIMCP_DEPTH_FUNCT(TAI-LAUNCHDATE, NIMCP.PD)
	END ELSE BEGIN
	    DEPTH = NIMCP_DEPTH_FUNCT(TAI-LAUNCHDATE, NIMCP.PD) * $
		    (TAI-TAI0) / (TAI1-TAI0)
	ENDELSE
	DEPTH = DEPTH < 0
	WIDTH = DBLARR(N_ELEMENTS(NIMCP))  &  CENTER = WIDTH  &  SHAPE = WIDTH
	LPOST = INTARR(N_ELEMENTS(NIMCP))
	FOR I=0,N_ELEMENTS(NIMCP)-1 DO BEGIN
	    WIDTH(I)  = POLY(TAI-LAUNCHDATE, NIMCP(I).PW)
	    CENTER(I) = POLY(TAI-LAUNCHDATE, NIMCP(I).PC)
	    SHAPE(I)  = POLY(TAI-LAUNCHDATE, NIMCP(I).PS)
	ENDFOR
	DET = NIMCP.DET
;
;  For post-recovery data, read in the information about the new burn-ins.
;  Otherwise, if the post-recovery information wasn't found, simply reduce all
;  burn-ins by 81%.
;
	IF ANYTIM2TAI(DATE) GT TAI_ACCIDENT THEN BEGIN
	    IF NOT KEYWORD_SET(PRE) THEN DEPTH = DEPTH * 0.813822 ELSE $
			IF NOT KEYWORD_SET(PRE_ONLY) THEN BEGIN
		MSG = ''
		READ_NIMCP_CAL, FILENAME, NIMCP, /POST, ERRMSG=MSG
		IF MSG NE '' THEN GOTO, CONTINUE
		TAI = TAI_SAVE
		DEP = NIMCP_DEPTH_FUNCT(TAI-LAUNCHDATE, NIMCP.PD) < 0
		DEPTH = [DEPTH(*), DEP(*)]
		WID = DBLARR(N_ELEMENTS(NIMCP))  &  CEN = WID  &  SHP = WID
		FOR I=0,N_ELEMENTS(NIMCP)-1 DO BEGIN
		    WID(I) = POLY(TAI-LAUNCHDATE, NIMCP(I).PW)
		    CEN(I) = POLY(TAI-LAUNCHDATE, NIMCP(I).PC)
		    SHP(I) = POLY(TAI-LAUNCHDATE, NIMCP(I).PS)
		ENDFOR
		WIDTH  = [WIDTH,  WID]
		CENTER = [CENTER, CEN]
		SHAPE  = [SHAPE,  SHP]
		DET    = [DET, NIMCP.DET]
		IF STRMID(FILENAME,STRLEN(FILENAME)-5,5) EQ 'a.dat' THEN $
			LPOST0 = 2 ELSE LPOST0 = 1
		LPOST  = [LPOST, REPLICATE(LPOST0, N_ELEMENTS(NIMCP))]
	    ENDIF
	ENDIF
;
;  Store the data in the output structure.
;
CONTINUE:
	IF N_ELEMENTS(DEPTH) EQ 1 THEN BEGIN
	    OUTPUT.DEPTH  = DEPTH(0)
	    OUTPUT.CENTER = CENTER(0)
	    OUTPUT.WIDTH  = WIDTH(0)
	    OUTPUT.SHAPE  = SHAPE(0)
	    OUTPUT.POST	  = LPOST(0)
	END ELSE BEGIN
	    OUTPUT = REPLICATE(OUTPUT, N_ELEMENTS(DEPTH))
	    OUTPUT.DEPTH  = DEPTH(*)
	    OUTPUT.CENTER = CENTER(*)
	    OUTPUT.WIDTH  = WIDTH(*)
	    OUTPUT.SHAPE  = SHAPE(*)
	    OUTPUT.POST  = LPOST(*)
	ENDELSE
	OUTPUT.DETECTOR = DET(*)
;  
	GOTO, FINISH
;
;  Error handling point
;
HANDLE_ERROR:
	IF N_ELEMENTS(ERRMSG) NE 0 THEN ERRMSG = 'GET_NIMCP: ' + MESSAGE $
		ELSE MESSAGE, MESSAGE, /CONTINUE
;
;  Return.
;
FINISH:
	RETURN
	END
