	PRO GET_VDS_BIAS, UNIT, HEADER, NAMES, DET_COLNO, BACKGROUND
;+
; Project     :	SOHO - CDS
;
; Name        :	GET_VDS_BIAS
;
; Purpose     :	Determines CCD bias levels in the VDS.
;
; Category    :	Calibration, VDS, Intensity
;
; Explanation :	Determines the CCD bias levels to be used with VDS_DEBIAS
;		routine.  The bias levels are determined in one of three ways:
;
;			1.  There are four windows labeled "BACKGROUND".
;			2.  The last four windows are near the edges of the
;			    CCD.
;			3.  The full CCD was read out.
;
;		This routine is called from DETSELECT.
;
; Syntax      :	GET_VDS_BIAS, UNIT, HEADER, NAMES, DET_COLNO, BACKGROUND
;
; Examples    :	
;
; Inputs      :	UNIT	  = The logical unit number of the opened FITS file.
;		HEADER	  = The binary table header.
;		NAMES	  = The names of the windows containing detector data.
;		DET_COLNO = The column numbers of the columns containing
;			    detector data.
;
; Opt. Inputs :	None.
;
; Outputs     :	BACKGROUND = A structure containing the background information,
;			     with the following tags:
;
;			AVERAGE	= The average bias value in each quadrant.
;			STDEV	= The standard deviation in the bias in each
;				  quadrant.
;			MINVAL  = The minimum value in each background window.
;			MAXVAL  = The maximum value in each background window.
;
;		Each tag will have the dimensions of (N_EXP,4) where N_EXP is
;		the number of exposures.
;
; Opt. Outputs:	None.
;
; Keywords    :	None.
;
; Calls       :	BASELINE, AVERAGE, FILL_MISSING
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Prev. Hist. :	Part of this routine was extracted from an earlier version of
;		DETSELECT by SVH Haugan.
;
; History     :	Version 1, 13-Feb-1996, William Thompson, GSFC
;		Version 2, 21-Mar-1996, William Thompson, GSFC
;			Corrected behavior when SUMLINE or SUMWIN compression
;			mode used.
;		Version 3, 26-Mar-1996, William Thompson, GSFC
;			Fill in missing values via interpolation
;		Version 4, 29-Mar-1996, William Thompson, GSFC
;			Don't call FILL_MISSING if only one element.
;		Version 5, 01-Sep-1999, William Thompson, GSFC
;			Filter out background values that have large errors.
;
; Contact     :	WTHOMPSON
;-
;
	ON_ERROR, 2
;
;  This routine should not be executed for GIS data.
;
	IF STRTRIM(FXPAR(HEADER,'DETECTOR')) NE 'NIS' THEN RETURN
;
;  Determine whether or not one can find some background data.  First of all,
;  look for windows named BACKGROUND.
;
	IX = WHERE(NAMES EQ 'BACKGROUND', NBACK)
	IF NBACK EQ 4 THEN BEGIN
		USE_WINDOWS = 1
		GOTO, DEFINE_BACK
	ENDIF
;
;  Next, see if the last four windows are near to the outer edges.
;
	IF N_ELEMENTS(NAMES) GE 4 THEN BEGIN
		FXBFIND, HEADER, 'TDETY', COLNO, TDETY, NCOLS
		TDETY = TDETY(NCOLS-4:*)
		IF (MIN(TDETY) LT 100) AND (MAX(TDETY) GT 756) THEN BEGIN
			IX = INDGEN(4) + NCOLS - 4
			USE_WINDOWS = 1
			GOTO, DEFINE_BACK
		ENDIF
	ENDIF
;
;  Finally, see if a full CCD readout was requested.
;
	IF N_ELEMENTS(NAMES) EQ 1 THEN BEGIN
		FXBFIND, HEADER, 'TDIM', COLNO, TDIM, NCOLS
		TDIM = TDIM(DET_COLNO-1)
		TDIM = FIX(FXBTDIM(TDIM(0)))
		W = WHERE(TDIM EQ 1024, COUNT)
		IF COUNT EQ 2 THEN BEGIN
			USE_WINDOWS = 0
			GOTO, DEFINE_BACK
		ENDIF
	ENDIF
;
;  Otherwise, one can't calculate the background windows.
;
	RETURN
;
;  Set up the BACKGROUND structure.
;
DEFINE_BACK:
	EXPCOUNT=FXPAR(HEADER,'EXPCOUNT')
	BACKGROUND={AVERAGE:	FLTARR(EXPCOUNT,4),	$
		    STDEV:	FLTARR(EXPCOUNT,4),	$
		    MINVAL:	FLTARR(EXPCOUNT,4),	$
		    MAXVAL:	FLTARR(EXPCOUNT,4)}
;
;  If full CCD readout, then read in the data.
;
	IF NOT USE_WINDOWS THEN READCDSCOL, UNIT, WINDOW, DET_COLNO(0)
;
;  Step through the quadrants.  If background windows are present, then read
;  them in.
;
	FOR I=0,3 DO BEGIN
	    IF USE_WINDOWS THEN BEGIN
	        READCDSCOL, UNIT, WINDOW, DET_COLNO(IX(I))
		QUADRANT = (1-(WINDOW.DETY / 512))*2 + (WINDOW.DETX/512)
;
;  Get the binning factors associated with that window.
;
		BINX = FXPAR(HEADER, 'TBINX' + TRIM(DET_COLNO(IX(I))))
		BINY = FXPAR(HEADER, 'TBINY' + TRIM(DET_COLNO(IX(I))))
		BINNING = (BINX > 1) * (BINY > 1)
	    END ELSE BEGIN
		QUADRANT = I
		BINNING = 1
	    ENDELSE
;
;  Step through the exposures, and extract the appropriate background data for
;  that exposure, assuming that 1st dimension is lambda, 2nd is SOLARX, 3rd (if
;  present) is SOLARY and 4th (if present) is DEL_TIME.
;
	    FOR EXPNO=0,EXPCOUNT-1 DO BEGIN
		IF (SIZE(WINDOW.ARRAY))(2) EQ EXPCOUNT THEN BEGIN ; SOLARX
		    BGDATA = WINDOW.ARRAY(*,EXPNO,*,*)
		END ELSE IF WINDOW.AXES(2) EQ 'DEL_TIME' THEN BEGIN
		    BGDATA = WINDOW.ARRAY(*,*,EXPNO)
		END ELSE IF WINDOW.AXES(3) EQ 'DEL_TIME' THEN BEGIN
		    BGDATA = WINDOW.ARRAY(*,*,*,EXPNO)
		ENDIF
;
;  If full CCD readout, then select out the appropriate data.
;
		IF NOT USE_WINDOWS THEN CASE QUADRANT OF
		    0:  BGDATA = BGDATA(0:511,*,0:99,*)
		    1:  BGDATA = BGDATA(512:*,*,0:99,*)
		    2:  BGDATA = BGDATA(0:511,*,924:*,*)
		    3:  BGDATA = BGDATA(512:*,*,924:*,*)
		ENDCASE
;
;  Remove the binning factor.
;
		W = WHERE(BGDATA NE -1, COUNT)
		IF COUNT GT 0 THEN BGDATA(W) = BGDATA(W) / BINNING
;
;  Derive the background data.
;
		BACKGROUND.AVERAGE(EXPNO,QUADRANT) = BASELINE(BGDATA, BSIG, $
			MISSING=-1)
		BACKGROUND.STDEV(EXPNO,QUADRANT)   = BSIG
		BACKGROUND.MINVAL(EXPNO,QUADRANT)  = MIN(BGDATA)
		BACKGROUND.MAXVAL(EXPNO,QUADRANT)  = MAX(BGDATA)
	    ENDFOR
;
;  If the standard deviation is comparable to the background value, then treat
;  it as missing.
;
	    W = WHERE(3*BACKGROUND.STDEV(*,QUADRANT) GT	$
		    BACKGROUND.AVERAGE(*,QUADRANT), COUNT)
	    IF COUNT GT 0 THEN BEGIN
		BACKGROUND.AVERAGE(W,QUADRANT) = -1
		BACKGROUND.STDEV(W,QUADRANT) = -1
	    ENDIF
;
;  Fill in any missing values.
;
	    TEMP = BACKGROUND.AVERAGE(*,QUADRANT)
	    IF N_ELEMENTS(TEMP) GT 1 THEN FILL_MISSING, TEMP, -1
	    BACKGROUND.AVERAGE(*,QUADRANT) = TEMP
	ENDFOR
;
	RETURN
	END
