	PRO GET_WAVE_EFF, DATE_OBS, SPECTRUM, COEFF, ERRMSG=ERRMSG,	$
		DATE_USED=DATE_USED
;+
; Project     :	SOHO - CDS
;
; Name        :	GET_WAVE_EFF
;
; Purpose     :	Gets an efficiency versus wavelength curve from database
;
; Category    :	CDS, Calibration, Class3
;
; Explanation :	Looks up the correct calibration curve for the efficiency as a
;		function of wavelength.  First, the software finds the largest
;		effect date which is on or earlier than the observation date.
;		If there are multiple entries with the same effective date,
;		then the one with the latest entry date is used.
;
;		Note that the actual efficiency is the product of the
;		efficiency returned by this routine, times the wavelength
;		dependence returned by GET_WAVE_EFF.  The latter is a
;		polynomial which will pass through unity at some reference
;		wavelength.  The entire calibration curve would be derived with
;		the following statements:
;
;			WAVE1 = PIX2WAVE('N1', INDGEN(1024))
;			GET_EFFICIENCY, date_obs, 'N1', EFF1
;			GET_WAVE_EFF, date_obs, 'N1', COEFF1
;			EFF1 = EFF1 * POLY(WAVE1, COEFF1)
;
; Syntax      :	GET_WAVE_EFF, DATE_OBS, SPECTRUM, COEFF
;
; Examples    :	GET_WAVE_EFF, '1996/09/16', 'N1', COEFF
;
; Inputs      :	DATE_OBS = The date the observation was made.
;
;		SPECTRUM = A character string representing the spectrum,
;			   e.g. "N1" or "NIS1".
;
; Opt. Inputs :	None.
;
; Outputs     :	COEFF = The polynomial efficients representing the relative
;			efficiency curve, expressed as a function of
;			wavelength.
;
; Opt. Outputs:	None.
;
; Keywords    :	DATE_USED = If a date is passed in through this keyword, then
;			    the most recent efficiency as of this date will be
;			    returned.  This is useful for comparing what the
;			    routine would have returned on a given date against
;			    what it currently returns.  For example,
;
;				GET_WAVE_EFF, '1-Mar-1997', 'N1', OLD_EFF, $
;					DATE_USED='10-Apr-1997'
;				GET_WAVE_EFF, '1-Mar-1997', 'N1', NEW_EFF
;
;		ERRMSG = If defined and passed, then any error messages will be
;			 returned to the user in this parameter rather than
;			 depending on the MESSAGE routine in IDL.  If no errors
;			 are encountered, then a null string is returned.  In
;			 order to use this feature, ERRMSG must be defined
;			 first, e.g.
;
;				ERRMSG = ''
;				GET_WAVE_EFF, ERRMSG=ERRMSG, ...
;				IF ERRMSG NE '' THEN ...
;
; Calls       :	ANYTIM2TAI, DBOPEN, DBFIND, ANYTIM2UTC, DBSORT, DBEXT, DBCLOSE
;
; Common      :	None
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Prev. Hist. :	None.
;
; History     :	Version 1, 16-Sep-1996, William Thompson, GSFC
;		Version 2, 02-Jun-1998, William Thompson, GSFC
;			Added keyword DATE_USED
;		Version 3, 21-May-2002, William Thompson, GSFC
;			Allow N4 for NIS-2 second order.
;
; Contact     :	WTHOMPSON
;-
;
	ON_ERROR, 2
;
;  Check the input parameters.
;
	IF N_PARAMS() NE 3 THEN BEGIN
	    MESSAGE = 'Syntax:  GET_WAVE_EFF, DATE_OBS, SPECTRUM, COEFF'
	    GOTO, HANDLE_ERROR
	ENDIF
;
;  Convert the observation date to TAI format.  Reformat the time to
;  millisecond accuracy.  This is necessary so that the times are written out
;  in a controlled way.
;
	TAI_OBS = DOUBLE( STRING( ANYTIM2TAI(DATE_OBS), FORMAT='(F15.3)' ))
;
;  Parse the spectrum identifier.
;
	S = STRTRIM(SPECTRUM,2)
	SG = STRUPCASE(STRMID(S,0,1) + STRMID(S,STRLEN(S)-1,1))
	IF (SG NE 'N1') AND (SG NE 'N2') AND (SG NE 'N4') AND (SG NE 'G1') $
		AND (SG NE 'G2') AND (SG NE 'G3') AND (SG NE 'G4') THEN	BEGIN
	    MESSAGE = 'Unrecognized spectrum identifier'
	    GOTO, HANDLE_ERROR
	ENDIF
;
;  Find all entries in the database for the requested spectrum.
;
	DBOPEN, 'wave_eff'
	ENTRIES = DBFIND('SPECTRUM=' + SG, /SILENT)
	IF ENTRIES(0) LE 0 THEN BEGIN
	    MESSAGE, 'No data found for spectrum ' + SPECTRUM
	    GOTO, HANDLE_ERROR
	ENDIF
;
;  If the DATE_USED keyword was passed, then reformat the time to millisecond
;  accuracy, and find all the entries which were entered into the database
;  before that time.
;
	IF N_ELEMENTS(DATE_USED) EQ 1 THEN BEGIN
	    TAI_USED = DOUBLE(STRING(ANYTIM2TAI(DATE_USED),FORMAT='(F15.3)'))
	    TEMP = DBFIND('DATE<' + TRIM(TAI_USED,'(F15.3)'), ENTRIES, /SILENT)
	    IF TEMP(0) LE 0 THEN MESSAGE, /CONTINUE,	$
		'No data found for use date ' + ANYTIM2UTC(DATE_USED,/VMS) + $
		', using current data' ELSE ENTRIES = TEMP
	ENDIF
;
;  Find all the entries in the database which match the observation time.  If
;  we don't find any, then use the earliest entry for that observation time.
;
	TEMP = DBFIND('DATE_EFF<' + TRIM(TAI_OBS,'(F15.3)'), ENTRIES, /SILENT)
	IF TEMP(0) LE 0 THEN BEGIN
	    MESSAGE, /CONTINUE, 'No data found for date ' +	$
		    ANYTIM2UTC(DATE_OBS,/VMS) + ', using earliest date'
	    REVERSE = [0]
	END ELSE BEGIN
	    REVERSE = [1]
	    ENTRIES = TEMP
	ENDELSE
;
;  Find all entries with the selected effective date, and sort on the entry
;  date.
;
	IF N_ELEMENTS(ENTRIES) GT 1 THEN	$
		ENTRIES = DBSORT(ENTRIES, 'DATE_EFF', REVERSE=REVERSE)
	DBEXT, ENTRIES(0), 'date_eff', DATE_EFF
	ENTRIES = DBFIND('DATE_EFF=' + TRIM(DATE_EFF,'(F15.3)'), ENTRIES, $
		/SILENT)
	IF N_ELEMENTS(ENTRIES) GT 1 THEN	$
		ENTRIES = DBSORT(ENTRIES, 'DATE', REVERSE=[1])
;
;  Extract the data.
;
	DBEXT, ENTRIES(0), 'n_coeff,coeff', N_COEFF, COEFF
	DBCLOSE
	COEFF = COEFF(0:N_COEFF(0)-1)
	RETURN
;
;  Error handling point.
;
HANDLE_ERROR:
	IF N_ELEMENTS(ERRMSG) NE 0 THEN ERRMSG = 'GET_EFFICIENCY: ' + MESSAGE $
		ELSE MESSAGE, MESSAGE
;
	END
