	FUNCTION GET_WIDTH_CORR, DATE, WAVELENGTH, IDENTITY,	$
		CALIBRATION=CALIBRATION
;+
; Project     :	SOHO - CDS
;
; Name        :	GET_WIDTH_CORR()
;
; Purpose     :	Get adjustments for selected NIS line widths.
;
; Category    :	Class3, Calibration
;
; Explanation :	The narrow-slit burn-in correction applied to the CDS/NIS data
;		does not entirely restore the spectral line shapes to their
;		unburned-in condition.  As a result, the average widths of some
;		lines slowly change over time, as does the integrated
;		brightness over the line.  These changes can be tracked with
;		the weekly NISAT_S study.  Only some lines show significant
;		width variations with time, and the behavior of each spectral
;		line is unique.
;
;		This routine reads in data files storing adjustment factors for
;		the various spectral lines which show significant line width
;		variation, so that the width can be adjusted to a consistent
;		value.  These adjustments can be applied to the fitted
;		intensity, to account for the brightness changes caused by the
;		width changes.
;
;		Although the result of this function is a correction factor for
;		the line width, it is not necessarily true that the adjusted
;		width will be the true width of the line.  The widths are also
;		not necessarily correct for dates extrapolated past the
;		creation date of the data file.
;
;		The line width corrections are critically dependent on the
;		narrow-slit burn-in correction.  Thus, each time the burn-in
;		calibration is changed, the line width calibration also needs
;		to be changed.
;
; Syntax      :	Result = GET_WIDTH_CORR( DATE, WAVELENGTH  [, IDENTITY ] )
;
; Examples    :	Result = GET_WIDTH_CORR('1998-Jan-3', 584.3, 'He I')
;
; Inputs      :	DATE	   = The observation date.  Can also be an array of
;			     dates.
;
;		WAVELENGTH = The wavelength of the line.  Second-order lines
;			     can be specified by either their first- or
;			     second-order wavelengths.  For example, the He II
;			     line at 303.8 A can be specified as either 303.7
;			     or 607.6.
;
;			     It's important to specify the wavelength as
;			     precisely as possible, to avoid confusion with
;			     nearby lines.  For example, trying to specify the
;			     He II line with a wavelength of 608 (or 304)
;			     causes confusion with the O IV line at 608.3 A.
;			     Similar confusion is possible between the He II
;			     line and the Si IX line at 303.3 A.  Since two
;			     lines can be right next to each other, and have
;			     completely different width variations, getting two
;			     nearby lines confused will give the wrong
;			     correction factor.
;
;			     Even where there is no chance of ambiguity, the
;			     wavelength must be specified to within +/- 0.25 A
;			     to find the correct line.
;
; Opt. Inputs :	IDENTITY   = Entering in an identity for a spectral line
;			     removes the ambiguity in line identification, and
;			     allows more leeway (+/- 1 A) in entering in the
;			     wavelength.  For example,
;
;				Result = GET_WIDTH_CORR(Date, 304)
;
;			     is likely to give an incorrect result, while
;
;				Result = GET_WIDTH_CORR(Date, 304, 'He II')
;
;			     will give the correct result.
;
; Outputs     :	The result of the function is the amount to multiply the line
;		width (or integrated intensity) by to correct for the slowly
;		varying width.
;
; Opt. Outputs:	None.
;
; Keywords    :	CALIBRATION = The number of the line width calibration to use.
;			      The default is to use the most up-to-date
;			      calibration.
;
; Calls       :	DATATYPE, STR_SEP, ANYTIM2UTC, NTRIM, FIND_WITH_DEF, ANYTIM2TAI
;
; Common      :	None.
;
; Restrictions:	The data files line_widths_pre*a.dat and _post*a.dat must be in
;		the directory $CDS_VDS_CAL_INT.
;
; Side effects:	None.
;
; Prev. Hist. :	None.
;
; History     :	Version 1, 18-May-2001, William Thompson
;		Version 2, 27-Jun-2001, William Thompson
;			Allow DATE to be an array.
;
; Contact     :	WTHOMPSON
;-
;
;	ON_ERROR, 2
;
	IF N_PARAMS() LT 2 THEN MESSAGE, 'Syntax:  Result = ' +	$
		'GET_WIDTH_CORR( DATE, WAVELENGTH  [, IDENTITY ] )'
;
;  If the line identity is passed, then do some minimal massaging of the
;  format, and use a bigger wavelength filter.
;
	IDENT = ''
	WDELTA = 0.25
	IF DATATYPE(IDENTITY,1) EQ 'String' THEN BEGIN
	    WORDS = STRUPCASE(STR_SEP(STRCOMPRESS(STRTRIM(IDENTITY,2)),' '))
	    IF N_ELEMENTS(WORDS) GE 2 THEN BEGIN
		IDENT = WORDS(0) + ' ' + WORDS(1)
		WDELTA = 1.0
	    ENDIF
	ENDIF
;
;  Convert the date into the proper format.  Depending on the date, open up
;  either the pre- or post-accident data.
;
	SDATE = ANYTIM2UTC(DATE,/CCSDS)
	CORR = REPLICATE(1.0D0, N_ELEMENTS(SDATE))
	W1 = WHERE(SDATE LT '1998-10-01', COUNT1)
	NAME1 = 'line_widths_pre'
	SDATE01 = '1996-04-15'
	W2 = WHERE(SDATE GT '1998-10-01', COUNT2)
	NAME2 = 'line_widths_post'
	SDATE02 = '1998-11-20'
;
	FOR ICOUNT=1,2 DO BEGIN
	    TEST = EXECUTE('COUNT = COUNT'+NTRIM(ICOUNT))
	    TEST = EXECUTE('NAME = NAME'+NTRIM(ICOUNT))
	    TEST = EXECUTE('SDATE0 = SDATE0'+NTRIM(ICOUNT))
	    TEST = EXECUTE('WW = W'+NTRIM(ICOUNT))
	    IF COUNT GT 0 THEN BEGIN
		WL = LINDGEN(COUNT)
		COUNTL = COUNT
		TAI0  = ANYTIM2TAI('2-DEC-1995 00:00:00')
		DTAI0 = ANYTIM2TAI(SDATE0) - TAI0
		DTAI  = (ANYTIM2TAI(DATE(WW))  - TAI0) > DTAI0
;
		FILE = ''
		IF N_ELEMENTS(CALIBRATION) EQ 1 THEN BEGIN
		    FILENAME = NAME + NTRIM(CALIBRATION) + 'a.dat'
		    FILE = FIND_WITH_DEF(FILENAME, '$CDS_VDS_CAL_INT')
		ENDIF
;
		CALIB = 10
		TRY_NEXT = 1
		WHILE (FILE EQ '') AND (CALIB GE 1) DO BEGIN
		    FILENAME = NAME + NTRIM(CALIB) + 'a.dat'
		    FILE = FIND_WITH_DEF(FILENAME, '$CDS_VDS_CAL_INT')
		    IF (FILE EQ '') THEN BEGIN
			CALIB = CALIB - 1
			TRY_NEXT = 0
		    END ELSE IF TRY_NEXT THEN BEGIN
			FILENAME = ''
			CALIB = CALIB + 1
		    ENDIF
		ENDWHILE
		IF FILE EQ '' THEN MESSAGE, 'Unable to open file ' + FILENAME
		OPENR, UNIT, FILE, /GET_LUN
;
;  Keep reading the data file until the correct wavelength (either first- or
;  second-order) is found.
;
		FOUND = 0
		LINE = 'string'
		WHILE (NOT FOUND) AND (NOT EOF(UNIT)) DO BEGIN
		    MAX_PARTS = 0
;
;  Read in the wavelength line, and extract the wavelength and line identities.
;
		    READF, UNIT, LINE
		    WORDS = STRUPCASE( STR_SEP(	$
			    STRCOMPRESS(STRTRIM(LINE,2)), ' ') )
		    WAVE = FLOAT(WORDS(1))
		    IDENTS = STRARR((N_ELEMENTS(WORDS)-2)/2)
		    FOR I=0,N_ELEMENTS(IDENTS)-1 DO IDENTS(I) =	$
			    WORDS(2+2*I) + ' ' + WORDS(3+2*I)
;
;  Is this the correct line?
;
		    IF IDENT EQ '' THEN W = 1 ELSE W = WHERE(IDENT EQ IDENTS)
		    FOUND = (W(0) GE 0) AND ((ABS(WAVE-WAVELENGTH) LE WDELTA) $
			    OR (ABS(WAVE-2*WAVELENGTH) LE WDELTA))
;
;  Read in the number of parts, and step through the parts.
;
		    READF, UNIT, LINE
		    WORDS = STR_SEP(STRCOMPRESS(STRTRIM(LINE,2)),' ')
		    N_PARTS = FIX(WORDS(2))
		    DELVARX, PAR, PAR0
		    FOR I=1,N_PARTS DO BEGIN
;
;  Read in the polynomial parameters.
;
			READF, UNIT, LINE
			WORDS = STR_SEP(STRCOMPRESS(STRTRIM(LINE,2)),' ')
			N_WIDTH = FIX(WORDS(2))
			PAR = DBLARR(N_WIDTH)
			READF, UNIT, PAR
;
;  Read in the next line, either a "BREAK" line if there are more parts to be
;  read, or a blank line between wavelength blocks.
;
			READF, UNIT, LINE
;
;  If a break line was read in, then extract the date.  If appropriate, then
;  use the previously read polynomial parameters.
;
			IF I NE N_PARTS THEN BEGIN
			    WORDS = STR_SEP(STRCOMPRESS(STRTRIM(LINE,2)),' ')
			    BDATE = ANYTIM2UTC(WORDS(2),/CCSDS)
			    IF SDATE0 LE BDATE THEN PAR0 = PAR
			    W = WHERE(SDATE(WW) LE BDATE, COUNT0)
			    IF (COUNT0 GT 0) AND FOUND THEN BEGIN
				WL = WHERE(SDATE(WW) GT BDATE, COUNTL)
				CORR(WW(W)) = POLY(DTAI0, PAR0) /	$
					POLY(DTAI(W), PAR)
			    ENDIF
			ENDIF
		    ENDFOR
;
;  If no parameter array has been selected yet, then use the last one for this
;  block.
;
		    IF N_ELEMENTS(PAR0) EQ 0 THEN PAR0 = PAR
		    IF (COUNTL GT 0) AND FOUND THEN CORR(WW(WL)) =	$
			    POLY(DTAI0, PAR0) / POLY(DTAI(WL), PAR)
		ENDWHILE
		FREE_LUN, UNIT
	    ENDIF
	ENDFOR
;
;  If the input date was scaler, then make the output date scalar as well.
;
	IF N_ELEMENTS(DATE) EQ 0 THEN CORR = CORR(0)
	RETURN, CORR
	END
