	PRO READ_NIMCP_CAL, FILENAME, OUTPUT, FIND=FIND, ERRMSG=ERRMSG, $
		PRE=PRE, POST=POST
;+
; Project     :	SOHO - CDS
;
; Name        :	READ_NIMCP_CAL
;
; Purpose     :	Read in NIMCP burn-in calibration files.
;
; Category    :	Class4, Calibration
;
; Explanation :	Reads in an NIMCP burn-in calibration file.  Two formats are
;		supported.  The older format puts all the parameters on a
;		single line, and has filenames like "nimcp_cal5.dat".
;
;		The newer format is more self-describing, with a header line
;		followed by lines describing the center position, width, shape
;		parameter, and depth functions.  The depth function must come
;		last for each line, and can consist of a series of functions
;		which added together replicate the depth data.  These files
;		have names like "nimcp_cal6a.dat".
;
;		With the instrument changes that resulted from SOHO's accident
;		in the summer of 1998, it was decided to split the burn-in
;		description into three separate databases:
;
;		"Old":	The files discussed above, describing the burn-in
;			history up until the accident.
;
;		"Pre":	Files with names like "nimcp_cal_pre1.dat", which
;			describe how the pre-accident burned-in lines affects
;			post-recovery data.  There is a slight format
;			difference between "Pre" files and the other two
;			databases in how the depth parameters are formatted and
;			interpreted.
;
;		"Post":	Files with names like "nimcp_cal_post1.dat", which
;			describe the new post-recovery burned-in lines.  The
;			"Pre" and "Post" files exist in pairs--e.g. one should
;			use "nimcp_cal_pre3.dat" together with
;			"nimcp_cal_post3.dat".  The version numbers should not
;			be mixed.
;
;		The "Old" database is used for pre-accident data, while the
;		"Pre" and "Post" databases are used for post-recovery data.
;
;		This routine is called from GET_NIMCP.
;
; Syntax      :	READ_NIMCP_CAL, FILENAME, OUTPUT  [, /FIND ]
;
; Examples    :	To read in files explicitly by name
;
;			READ_NIMCP_CAL, 'nimcp_cal3.dat', OUTPUT
;
;			READ_NIMCP_CAL, 'nimcp_cal_pre1.dat', OUTPUT, /PRE
;		
;		To find the most recent "Old" database for pre-accident data:
;
;			READ_NIMCP_CAL, FILENAME, OUTPUT, /FIND
;
;		To find the most recent "Pre" database for post-recovery data:
;
;			READ_NIMCP_CAL, FILENAME, OUTPUT, /FIND, /PRE
;
;		To do the same, but allow failover to an "Old" database if no
;		"Pre" database is found:
;
;			PRE = 0
;			READ_NIMCP_CAL, FILENAME, OUTPUT, /FIND, /POST, PRE=PRE
;
;		The variable PRE will be returned as 1 if a "Pre" database was
;		found, or 0 if it failed over to an "Old" database.
;
;		To read in the "Post" database associated with a previously
;		read-in "Pre" database.
;
;			FILENAME = 'nimcp_cal_pre1.dat'
;			MSG = ''
;			READ_NIMCP_CAL, FILENAME, OUTPUT, /POST, ERRMSG=MSG
;
;		If MSG is modified, then there is no associated "Post"
;		database.
;
; Inputs      :	FILENAME = The name of the file to read in.
;
; Opt. Inputs :	None.
;
; Outputs     :	OUTPUT = An array of structures containing the NIMCP
;		calibration data.  Each structure element contains the
;		following tags:
;
;			DET	= Either "N1" or "N2"
;			CENTER	= The average center pixel position
;			PC	= The polynomial describing the center pixel
;				  position
;			PW	= The polynomial describing the width
;			PS	= The polynomial describing the shape parameter
;			PD	= The polynomial describing the depth
;
; Opt. Outputs:	If FIND is set, then FILENAME is an output parameter rather
;		than an input parameter.
;
; Keywords    :	FIND   = If set, then find the latest burn-in calibration file.
;			 See the examples above for how the FIND keyword
;			 interacts with the PRE and POST keywords.
;
;		PRE    = If set, then read in a file describing the effect of
;			 the pre-accident lines on post-accident data.  When
;			 used with FIND, can also be used as an output
;			 variable.  See the examples above.
;
;		POST   = When used with /FIND, then finds the most up-to-date
;			 calibration file for post-recovery data.  When /FIND
;			 is not set, then reads in the "Post" file associated
;			 with the "Pre" file as identified in the variable
;			 FILENAME.  See the examples above.
;
;		ERRMSG = If defined and passed, then any error messages will
;			 be returned to the user in this parameter rather than
;			 depending on the MESSAGE routine in IDL.  If no errors
;			 are encountered, then the original string is returned.
;			 In order to use this feature, ERRMSG must be defined
;			 first, e.g.
;
;				ERRMSG = ''
;				READ_NIMCP_CA, ERRMSG=ERRMSG, ...
;				IF ERRMSG NE '' THEN ...
;
; Calls       :	DATATYPE, STR_SEP
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Prev. Hist. :	None.
;
; History     :	Version 1, 11-Jun-1998, William Thompson, GSFC
;		Version 2, 06-Aug-1999, William Thompson, GSFC
;			Added keywords PRE and POST
;		Version 3, 12-Jan-2000, William Thompson, GSFC
;			Search for "pre" files with both "a.dat" and ".dat"
;
; Contact     :	WTHOMPSON
;-
;
;
	ON_ERROR, 2
;
;  Check the number of parameters.
;
	IF N_PARAMS() NE 2 THEN BEGIN
	    MESSAGE = 'Syntax:  READ_NIMCP_CAL, FILENAME, OUTPUT'
	    GOTO, HANDLE_ERROR
	ENDIF
;
;  If the /FIND keyword is used, then find the latest calibration file.  If
;  after SOHO's recovery, then first try looking for a "Pre" file.
;
	IF KEYWORD_SET(FIND) THEN BEGIN
	    IF KEYWORD_SET(POST) THEN BEGIN
		MSG = ''
		READ_NIMCP_CAL, FILENAME, OUTPUT, /FIND, /PRE, ERRMSG=MSG
		IF MSG EQ '' THEN BEGIN
		    PRE = 1
		    RETURN
		ENDIF
	    ENDIF
;
;  Otherwise, or if not found, then start with the new format.  If the file for
;  a given calibration value is not found, then try the next lower value.  If
;  it is found, and TRY_NEXT is set, then try the next higher value.
;
	    CALIB = 10
	    TRY_NEXT = 1
	    FILENAME = ''
	    WHILE (FILENAME EQ '') AND (CALIB GE 1) DO BEGIN
		IF KEYWORD_SET(PRE) THEN BEGIN
		    NAME = 'nimcp_cal_pre'
		    EXT = 'a.dat'
		END ELSE BEGIN
		    NAME = 'nimcp_cal'
		    EXT = 'a.dat'
		    PRE = 0
		ENDELSE
		NAME = NAME + NTRIM(CALIB) + EXT
		FILENAME = FIND_WITH_DEF(NAME, '$CDS_VDS_CAL_INT')
		IF (FILENAME EQ '') THEN BEGIN
		    CALIB = CALIB - 1
		    TRY_NEXT = 0
		END ELSE IF TRY_NEXT THEN BEGIN
		    FILENAME = ''
		    CALIB = CALIB + 1
		ENDIF
	    ENDWHILE
;
;  If we didn't find any "nimcp_cal_pre" calibration files with the extension
;  "a.dat", then try it with the extension ".dat"
;
	    IF KEYWORD_SET(PRE) AND (EXT EQ 'a.dat') AND (CALIB EQ 0) THEN $
		    BEGIN
		CALIB = 1
		TRY_NEXT = 1
		FILENAME = ''
		WHILE FILENAME EQ '' DO BEGIN
		    NAME = 'nimcp_cal_pre' + NTRIM(CALIB) + '.dat'
		    FILENAME = FIND_WITH_DEF(NAME, '$CDS_VDS_CAL_INT')
		    IF (FILENAME EQ '') THEN BEGIN
			IF (CALIB EQ 1) THEN BEGIN
			    MESSAGE = 'File ' + NAME + ' not found'
			    GOTO, HANDLE_ERROR
			END ELSE CALIB = CALIB - 1
			TRY_NEXT = 0
		    END ELSE IF TRY_NEXT THEN BEGIN
			FILENAME = ''
			CALIB = CALIB + 1
		    ENDIF
		ENDWHILE
	    ENDIF
;
	    IF NOT KEYWORD_SET(PRE) THEN BEGIN
		CALIB = 5
		TRY_NEXT = 1
		WHILE FILENAME EQ '' DO BEGIN
		    NAME = 'nimcp_cal' + NTRIM(CALIB) + '.dat'
		    FILENAME = FIND_WITH_DEF(NAME, '$CDS_VDS_CAL_INT')
		    IF (FILENAME EQ '') THEN BEGIN
			IF (CALIB EQ 1) THEN BEGIN
			    MESSAGE = 'File ' + NAME + ' not found'
			    GOTO, HANDLE_ERROR
			END ELSE CALIB = CALIB - 1
			TRY_NEXT = 0
		    END ELSE IF TRY_NEXT THEN BEGIN
			FILENAME = ''
			CALIB = CALIB + 1
		    ENDIF
		ENDWHILE
	    ENDIF
;
;  If the /POST keyword is used, without /FIND, then read in the associated
;  "_post" file.
;
	END ELSE IF KEYWORD_SET(POST) THEN BEGIN
	    BREAK_FILE, FILENAME, DISK, DIR, NAME, EXT
	    NAME = 'nimcp_cal_post' + STRMID(NAME,13,STRLEN(NAME)-13)
	    FILENAME = DISK + DIR + NAME + EXT
	    PRE = 0
	ENDIF
;
;  Check the filename parameter.
;
	IF (DATATYPE(FILENAME,1) NE 'String') OR (N_ELEMENTS(FILENAME) NE 1) $
		THEN BEGIN
	    MESSAGE = 'FILENAME must be a single string value'
	    GOTO, HANDLE_ERROR
	ENDIF
;
;  Define the output structure.  Start out with an empty (undefined) OUTPUT
;  array.
;
	OUT_STRUCT = {DET: '',			$
		CENTER: 0.0D0,			$
		PC: REPLICATE(0.0D0, 10),	$
		PW: REPLICATE(0.0D0, 10),	$
		PS: REPLICATE(0.0D0, 10),	$
		PD: REPLICATE(0.0D0, 10)}
	DELVARX, OUTPUT
;
;  Open the file and read it in line by line.  Look for lines that start with
;  the letters "Spectrum", "N1", or "N2".
;
	ON_IOERROR, IO_ERROR_POINT
	OPENR, UNIT, FILENAME, /GET_LUN
	LINE = ''
	WHILE NOT EOF(UNIT) DO BEGIN
	    READF, UNIT, LINE
	    LINE = STRUPCASE(STRTRIM(LINE,2))
;
;  If the line starts with the characters 'SPECTRUM', then read in a burn-in
;  description block.  First, extract the spectral band ("N1" or "N2") and the
;  average line center pixel.
;
	    IF STRMID(LINE,0,8) EQ 'SPECTRUM' THEN BEGIN
		DATA = STR_SEP(STRCOMPRESS(LINE),' ')
		DET = DATA(1)
		CENTER = DOUBLE(DATA(2))
		N_CENTER = 0
		N_WIDTH = 0
		N_SHAPE = 0
;
;  Keep reading, until we reach the N_DEPTH line.
;
		WHILE STRMID(LINE,0,7) NE 'N_DEPTH' DO BEGIN
		    READF, UNIT, LINE
		    LINE = STRUPCASE(STRTRIM(LINE,2))
;
;  If the line starts with N_CENTER, then read in the polynomial describing the
;  line centers.
;
		    IF STRMID(LINE,0,8) EQ 'N_CENTER' THEN BEGIN
			DATA = STR_SEP(STRCOMPRESS(LINE),' ')
			N_CENTER = FIX(DATA(2))
			IF N_CENTER LE 0 THEN BEGIN
			    MESSAGE = 'Entry not allowed: ' + LINE
			    GOTO, HANDLE_ERROR
			ENDIF
			PC = DBLARR(N_CENTER)
			READF, UNIT, PC
;
;  If the line starts with N_WIDTH, then read in the polynomial describing the
;  line widths.
;
		    END ELSE IF STRMID(LINE,0,7) EQ 'N_WIDTH' THEN BEGIN
			DATA = STR_SEP(STRCOMPRESS(LINE),' ')
			N_WIDTH = FIX(DATA(2))
			IF N_WIDTH LE 0 THEN BEGIN
			    MESSAGE = 'Entry not allowed: ' + LINE
			    GOTO, HANDLE_ERROR
			ENDIF
			PW = DBLARR(N_WIDTH)
			READF, UNIT, PW
;
;  If the line starts with N_SHAPE, then read in the polynomial describing the
;  line shapes.
;
		    END ELSE IF STRMID(LINE,0,7) EQ 'N_SHAPE' THEN BEGIN
			DATA = STR_SEP(STRCOMPRESS(LINE),' ')
			N_SHAPE = FIX(DATA(2))
			IF N_SHAPE GT 0 THEN BEGIN
			    PS = DBLARR(N_SHAPE)
			    READF, UNIT, PS
			ENDIF
;
;  Anything else is an incongruous line.
;
		    END ELSE IF (STRMID(LINE,0,7) EQ 'N_SHAPE') AND	$
			    (LINE NE '') THEN BEGIN
			MESSAGE = 'Unrecognized line: ' + LINE
			GOTO, HANDLE_ERROR
		    ENDIF
		ENDWHILE
;
;  Make sure that the center and width polynomials have been read in.  (The
;  shape polynomial is optional.)
;
		IF N_CENTER EQ 0 THEN BEGIN
		    MESSAGE = 'N_CENTER line not found'
		    GOTO, HANDLE_ERROR
		END ELSE IF N_WIDTH EQ 0 THEN BEGIN
		    MESSAGE = 'N_WIDTH line not found'
		    GOTO, HANDLE_ERROR
		ENDIF
;
;  Read in the number of line depth functions.  If the keyword /PRE is set,
;  then this is the number of parameters in a single polynomial expression.
;
		DATA = STR_SEP(STRCOMPRESS(LINE),' ')
		N_DEPTH = FIX(DATA(2))
		IF KEYWORD_SET(PRE) THEN BEGIN
		    ND = N_DEPTH
		    N_DEPTH = 1
		END ELSE ND = 3
;
;  Read in each line depth function, and add an entry to the output structure.
;
		FOR I=1,N_DEPTH DO BEGIN
		    PD = DBLARR(ND)
		    READF, UNIT, PD
		    N_OUT = N_ELEMENTS(OUTPUT)
		    IF N_OUT EQ 0 THEN OUTPUT = OUT_STRUCT ELSE		$
			    OUTPUT = [OUTPUT, OUT_STRUCT]
		    OUTPUT(N_OUT).DET = DET
		    OUTPUT(N_OUT).CENTER = CENTER
		    OUTPUT(N_OUT).PC = PC
		    OUTPUT(N_OUT).PW = PW
		    OUTPUT(N_OUT).PD = PD
		    IF N_SHAPE GT 0 THEN OUTPUT(N_OUT).PS = PS
		ENDFOR
;
;  If the line starts with either "N1" or "N2", then read in an entry from an
;  older-format file.
;
	    END ELSE IF (STRMID(LINE,0,2) EQ 'N1') OR	$
		    (STRMID(LINE,0,2) EQ 'N2') THEN BEGIN
		DATA = STR_SEP(STRCOMPRESS(LINE),' ')
		DET = DATA(0)
		NIMCP = DOUBLE(DATA(1:*))
		N_OUT = N_ELEMENTS(OUTPUT)
		IF N_OUT EQ 0 THEN OUTPUT = OUT_STRUCT ELSE		$
			OUTPUT = [OUTPUT, OUT_STRUCT]
		OUTPUT(N_OUT).DET = DET
		OUTPUT(N_OUT).CENTER = NIMCP(0)
		OUTPUT(N_OUT).PC = NIMCP(0:0)
		OUTPUT(N_OUT).PW = NIMCP(4:5)
		OUTPUT(N_OUT).PD = NIMCP(1:3)
		IF N_ELEMENTS(DATA) EQ 8 THEN OUTPUT(N_OUT).PS = NIMCP(6:6)
;
;  Anything else would be an erroneous line.
;
	    END ELSE IF LINE NE '' THEN BEGIN
		MESSAGE = 'Unrecognized line: ' + LINE
		GOTO, HANDLE_ERROR
	    ENDIF
	ENDWHILE
	FREE_LUN, UNIT
	RETURN
;
;  Error handling points.
;
IO_ERROR_POINT:
	MESSAGE = 'Error opening or reading file ' + FILENAME
;
HANDLE_ERROR:
	IF N_ELEMENTS(UNIT) EQ 1 THEN FREE_LUN, UNIT
	IF N_ELEMENTS(ERRMSG) NE 0 THEN ERRMSG = 'READ_NIMCP_CAL: ' + MESSAGE $
		ELSE MESSAGE, MESSAGE
;
;  Exit point.
;
	END
