	PRO VDS_DEBIAS, DATA, BIASES, ERRMSG=ERRMSG
;+
; Project     :	SOHO - CDS
;
; Name        :	VDS_DEBIAS
;
; Purpose     :	Removes CCD biases from VDS images.
;
; Explanation :	This procedure takes VDS images read with READCDSFITS and
;		removes the CCD bias--which may also include a scattered light
;		component--as a function of readout quadrant.  The resulting
;		output can then be used with VDS_CALIB.
;
; Use         :	VDS_DEBIAS, DATA  [, BIASES ]
;
; Inputs      :	DATA	= Structure variable as read by READCDSFITS.
;
; Opt. Inputs :	BIASES	= Array containing the CCD biases for the quadrants
;
;			     Index	Quadrant
;
;				0	A - Upper left
;				1	B - Upper right
;				2	C - Lower left
;				3	D - Lower right
;
;			  It can also be an array of (N_EXP,4) elements.
;
; Outputs     :	DATA	= The calibrated data will be returned as a new
;			  structure variable with the same format as the input.
;
; Opt. Outputs:	None.
;
; Keywords    :	ERRMSG = If defined and passed, then any error messages will be
;			 returned to the user in this parameter rather than
;			 depending on the MESSAGE routine in IDL.  If no errors
;			 are encountered, then a null string is returned.  In
;			 order to use this feature, ERRMSG must be defined
;			 first, e.g.
;
;				ERRMSG = ''
;				VDS_DEBIAS, ERRMSG=ERRMSG, ...
;				IF ERRMSG NE '' THEN ...
;
; Calls       :	FIND_WITH_DEF, FXREAD, FXPAR, AVERAGE, TRIM, CONCAT_DIR,
;		DATATYPE, REP_TAG_VALUE
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Category    :	Calibration, VDS, Intensity.
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 3 March 1994.
;
; Modified    :	Version 1, William Thompson, GSFC, 3 March 1994.
;		Version 2, William Thompson, GSFC, 19 October 1995
;			Brought up to date with current FITS format
;		Version 3, William Thompson, GSFC, 26 October 1995
;			Allow BIASES to vary with exposure.
;			Make sure that DETDATA has the right dimensions.
;		Version 4, William Thompson, GSFC, 14 November 1995
;			Make BIASES an optional parameter.  Take from DATA
;			structure if not supplied.
;		Version 5, William Thompson, GSFC, 28 December 1995
;			Change units to "DEBIASED-ADC"
;		Version 6, William Thompson, GSFC, 7 February 1996
;			Correct bug when windows go across quadrant boundaries.
;			Keep track of missing pixels, and change the missing
;			pixels flag.
;		Version 7, William Thompson, GSFC, 13 February 1996
;			Correct bug with multiple exposures.
;		Version 8, William Thompson, GSFC, 29 February 1996
;			Check to make sure that this routine has not already
;			been called.
;			Add ERRMSG keyword.
;		Version 9, William Thompson, GSFC, 6 May 1996
;			Average together biases if not in quadrature readout
;			mode.
;		Version 10, William Thompson, GSFC, 8 August 1996
;			Rewrote to process windows one-by-one.  Will make it
;			easier when QL structure is changed.
;               Version 11, S.V.H. Haugan, UiO, 9 August 1996
;                       Windows stored with st_windata, QL data storage system
;                       should be transparent. No references to ixstart/stop.
;               Version 12, S.V.H. Haugan, UiO, 12 August 1996
;                       A reference to ixstop is actually needed to determine
;                       whether data window has been read by readcdsfits.
;		Version 13, William Thompson, GSFC, 28 October 1996
;			Don't stop if data has already been debiased.
;		Version 14, Zarro, GSFC, 28 March 1997
;			Fixed bug in BIASES size check
;		Version 15, William Thompson, GSFC, 4 August 1997
;			Change units of all windows
;
; Version     : Version 15, 4 August 1997
;-
;
	ON_ERROR, 2
;
;  Check the number of parameters.
;
	IF N_PARAMS() LT 1 THEN BEGIN
	    MESSAGE = 'Syntax:  VDS_DEBIAS, DATA  [, BIASES ]'
	    GOTO, HANDLE_ERROR
	ENDIF
;
;  Make sure that it is VDS data we're dealing with.
;
	IF DATA.HEADER.DETECTOR NE 'NIS' THEN BEGIN
	    MESSAGE = 'Cannot debias GIS data'
	    GOTO, HANDLE_ERROR
	ENDIF
;
;  Check to see if the debiasing has already been done.
;
	IF DATA.DETDESC(0).UNITS NE 'ADC' THEN BEGIN
	    MESSAGE = 'Data has already been debiased.'
	    CONTINUE = 1
	    GOTO, HANDLE_ERROR
	ENDIF
;
;  If the biases are not passed, then see if they're in the input structure.
;
	IF N_PARAMS() EQ 1 THEN BEGIN
	    IF DATATYPE(DATA.BACKGROUND, 1) NE 'Structure' THEN BEGIN
		MESSAGE = 'CDS bias values are not in the data structure'
		GOTO, HANDLE_ERROR
	    END ELSE BEGIN
		BIASES = DATA.BACKGROUND.AVERAGE
	    ENDELSE
	ENDIF
;
;  If the datatype of the array in the input structure is not already floating
;  point, then recreate the structure.
;
        OLD_STORAGE = TAG_EXIST(DATA,"DETDATA",/TOP_LEVEL)
        IF OLD_STORAGE THEN BEGIN
	    IF DATATYPE(DATA.DETDATA,2) LT 4 THEN BEGIN
		ARRAY = FLOAT(DATA.ARRAY)
		DATA = REP_TAG_VALUE(DATA,ARRAY,'DETDATA')
            ENDIF
        ENDIF 
;
;  Determine the number of windows, and step through them.
;
	NWIN = N_ELEMENTS(DATA.DETDESC)
        FOR IWIN = 0,NWIN-1 DO $
           IF DATA.DETDESC(IWIN).IXSTOP(0) GE 0 THEN BEGIN
;
;  Extract the data array from the input structure.
;  Matches ST_WINDATA,DATA,IWIN,ARRAY below
;           
	    ARRAY = DETDATA(DATA, IWIN+1)
	    DESC  = DETDESC(DATA, IWIN+1)
	    SA = SIZE(ARRAY)
	    IF DATATYPE(ARRAY,2) LT 4 THEN ARRAY = FLOAT(ARRAY)
	    ARRAY = REFORM(ARRAY, SA(1:SA(0)), /OVERWRITE)
;
;  Determine the number of exposures.
;
	    N_EXP = 1
	    W = WHERE(DESC.AXES EQ 'SOLAR_X', COUNT)
	    IF COUNT EQ 1 THEN N_EXP = N_EXP * SA(W(0)+1)
	    W = WHERE(DESC.AXES EQ 'DEL_TIME', COUNT)
	    IF COUNT EQ 1 THEN N_EXP = N_EXP * SA(W(0)+1)
;
;  Make sure that BIASES is an array with the right number of elements.  It
;  must either have four elements, or four times the number of exposures.
;
	    IF (N_ELEMENTS(BIASES) NE 4) AND (N_ELEMENTS(BIASES) NE 4*N_EXP) $
		    THEN BEGIN
		MESSAGE = 'BIASES must have 4 or ' + TRIM(N_EXP) +'*4 elements'
		GOTO, HANDLE_ERROR
	    ENDIF
	    SB = SIZE(BIASES)
	    IF SB(SB(0)) NE 4 THEN BEGIN
		MESSAGE = 'The last dimension of BIASES must be 4'
		GOTO, HANDLE_ERROR
	    ENDIF
;
;  If not in quadrature readout mode, then use the same bias for quadrants A
;  and B, and for C and D.  If in one of the single quadrant readout modes,
;  then use one bias for all the quadrants.
;
	    BBIASES = BIASES
	    IF DATA.HEADER.VDS_MODE GT 3 THEN BEGIN
		IF N_ELEMENTS(BBIASES) EQ 4 THEN BEGIN
			BBIASES = REPLICATE(AVERAGE(BBIASES),4)
		END ELSE BEGIN
			BBIASES = AVERAGE(BBIASES,2) # REPLICATE(1.,4)
		ENDELSE
	    END ELSE IF DATA.HEADER.VDS_MODE GT 1 THEN BEGIN
		IF N_ELEMENTS(BBIASES) EQ 4 THEN BEGIN
			BIAS_AB = (BIASES(0) + BIASES(1)) / 2.
			BIAS_CD = (BIASES(2) + BIASES(3)) / 2.
			BBIASES(0:1) = BIAS_AB
			BBIASES(2:3) = BIAS_CD
		END ELSE BEGIN
			BIAS_AB = (BIASES(*,0) + BIASES(*,1)) / 2.
			BIAS_CD = (BIASES(*,2) + BIASES(*,3)) / 2.
			BBIASES(*,0) = BIAS_AB
			BBIASES(*,1) = BIAS_AB
			BBIASES(*,2) = BIAS_CD
                        BBIASES(*,3) = BIAS_CD
		ENDELSE
	    ENDIF
;
;  Extract the start and stop indices, and the detector positions.
;  When ST_WINDATA is used to store data, only the size matters
;
	    ISTART = REPLICATE(0,4)
	    ISTOP  = REPLICATE(0,4)  &  ISTOP (0) = SA(1:sa(0))-1
;
;  Subtract the appropriate biases from the array.
;
	    BIAS_ARRAY = MAKE_ARRAY(DIMENSION=SA(1:SA(0)), /FLOAT)
	    I1 = 0  &  NI = ISTOP(0)-ISTART(0)+1  &  I2 = NI-1
	    J1 = 0  &  NJ = ISTOP(1)-ISTART(1)+1  &  J2 = NJ-1
	    K1 = 0  &  NK = ISTOP(2)-ISTART(2)+1  &  K2 = NK-1
	    L1 = 0  &  NL = ISTOP(3)-ISTART(3)+1  &  L2 = NL-1
;
	    BINX = DESC.BINX > 1
	    BINY = DESC.BINY > 1
;
;  Depending on the axis titles, determine the boundaries between the
;  quadrants.  There are two possibilities: either the first (I) axis is
;  wavelength, and the third (K) is solar Y, or the wavelength dimension is
;  missing.  Note that the solar Y is in the opposite direction from that of
;  detector Y.
;
	    AXES = DESC.AXES
	    IF AXES(0) EQ 'WAVELNTH' THEN BEGIN
		DX = DESC.DETX
		DY = 1023 - (DESC.DETY + (K2-K1+1)*BINY - 1)
		I1A = I1  &  I2A = I2 < ((511 - DX)/BINX + I1)
		K1A = K1  &  K2A = K2 < ((511 - DY)/BINY + K1)
		I1D = ((512 - DX)/BINX + I1) > I1  &  I2D = I2
		K1D = ((512 - DY)/BINY + K1) > K1  &  K2D = K2
;
;  The J positions don't change.
;
		J1A = J1  &  J1D = J1  &  J2A = J2  &  J2D = J2
;
;  If the wavelength dimension is not first, the second (J) dimension is
;  assumed to be the solar Y dimension.
;
	    END ELSE BEGIN
		DY = 1023 - (DESC.DETY + (J2-J1+1)*BINY - 1)
		J1A = J1  &  J2A = J2 < ((511 - DY)/BINY + J2)
		J1D = ((512 - DY)/BINY + J1) > J1  &  J2D = J2
;
;  The I and K positions don't change.
;
		I1A = I1  &  I1D = I1  &  I2A = I2  &  I2D = I2
		K1A = K1  &  K1D = K1  &  K2A = K2  &  K2D = K2
	    ENDELSE
;
;  Use symmetry to work out the positions of the B and C quadrants.
;
	    I1C = I1A  &  I2C = I2A  &  I1B = I1D  &  I2B = I2D
	    J1C = J1D  &  J2C = J2D  &  J1B = J1A  &  J2B = J2A
	    K1C = K1D  &  K2C = K2D  &  K1B = K1A  &  K2B = K2A
;
;  Process the window quadrant by quadrant.
;
	    FOR IQUAD = 1, 4 DO BEGIN
		CASE IQUAD OF
;
		    1: BEGIN
			IF (I1A LE I2A) AND (J1A LE J2A) AND (K1A LE K2A) $
				THEN BEGIN
			    II1 = I1A  &  II2 = I2A
			    JJ1 = J1A  &  JJ2 = J2A
			    KK1 = K1A  &  KK2 = K2A
			    IF N_ELEMENTS(BBIASES) EQ 4 THEN	$
				    BIAS = BBIASES(0) ELSE	$
				    BIAS = (REFORM(BBIASES,N_EXP,4))(*,0)
			ENDIF ELSE BIAS = -1
			END
;
		    2: BEGIN
	    		IF (I1B LE I2B) AND (J1B LE J2B) AND (K1B LE K2B) $
				THEN BEGIN
			    II1 = I1B  &  II2 = I2B
			    JJ1 = J1B  &  JJ2 = J2B
			    KK1 = K1B  &  KK2 = K2B
			    IF N_ELEMENTS(BBIASES) EQ 4 THEN	$
				    BIAS = BBIASES(1) ELSE	$
				    BIAS = (REFORM(BBIASES,N_EXP,4))(*,1)
			ENDIF ELSE BIAS = -1
			END
;
		    3: BEGIN
			IF (I1C LE I2C) AND (J1C LE J2C) AND (K1C LE K2C) $
				THEN BEGIN
			    II1 = I1C  &  II2 = I2C
			    JJ1 = J1C  &  JJ2 = J2C
			    KK1 = K1C  &  KK2 = K2C
			    IF N_ELEMENTS(BBIASES) EQ 4 THEN	$
				    BIAS = BBIASES(2) ELSE	$
				    BIAS = (REFORM(BBIASES,N_EXP,4))(*,2)
			ENDIF ELSE BIAS = -1
			END
;
		    4: BEGIN
			IF (I1D LE I2D) AND (J1D LE J2D) AND (K1D LE K2D) $
				THEN BEGIN
			    II1 = I1D  &  II2 = I2D
			    JJ1 = J1D  &  JJ2 = J2D
			    KK1 = K1D  &  KK2 = K2D
			    IF N_ELEMENTS(BBIASES) EQ 4 THEN	$
				    BIAS = BBIASES(3) ELSE	$
				    BIAS = (REFORM(BBIASES,N_EXP,4))(*,3)
			ENDIF ELSE BIAS = -1
			END
		ENDCASE
;
;  If necessary, extend the bias in the WAVELENGTH and/or SOLAR_Y dimensions.
;
		IF BIAS(0) GE 0 THEN BEGIN
		    IF N_ELEMENTS(BIAS) GT 1 THEN BEGIN
			NII = II2 - II1 + 1
			NJJ = JJ2 - JJ1 + 1
			NKK = KK2 - KK1 + 1
			IF AXES(0) EQ 'WAVELNTH' THEN BEGIN
                           IF I1 NE I2 THEN BEGIN
				BIAS = REFORM(BIAS,N_ELEMENTS(BIAS),/OVERWRITE)
				BIAS = REPLICATE(1,NII) # TEMPORARY(BIAS)
                                BIAS = REFORM(BIAS,NII,NJJ,NL, /OVERWRITE)
			    ENDIF
;
			    IF K1 NE K2 THEN BEGIN		;Solar Y
				BIAS = REFORM(BIAS,N_ELEMENTS(BIAS),/OVERWRITE)
				BIAS = TEMPORARY(BIAS) # REPLICATE(1,NKK)
				BIAS = REFORM(BIAS,NII,NJJ,NL,NKK, /OVERWRITE)
				BIAS = REARRANGE(BIAS, [1,2,4,3])
			    ENDIF
			END ELSE BEGIN
			    IF J1 NE J2 THEN BEGIN		;Solar Y
				BIAS = REFORM(BIAS,N_ELEMENTS(BIAS),/OVERWRITE)
				BIAS = TEMPORARY(BIAS) # REPLICATE(1,NJJ)
				BIAS = REFORM(BIAS,NII,NKK,NJJ, /OVERWRITE)
				BIAS = REARRANGE(BIAS, [1,3,2])
			    ENDIF
			ENDELSE
		    ENDIF
;
;  Store the calculated bias in the bias array.
;
		    BIAS_ARRAY(II1:II2,JJ1:JJ2,KK1:KK2,L1:L2) = BIAS
		ENDIF
	    ENDFOR	;IQUAD
;
;  Subtract the bias.  Replace any missing pixels with the value -100.
;
	    MISSING = DESC.MISSING
	    W = WHERE(ARRAY EQ MISSING, N_MISSING)
	    ARRAY = ARRAY - BIAS_ARRAY * BINX * BINY
	    IF N_MISSING GT 0 THEN ARRAY(W) = -100.0
;
;  Put the data back into the structure.
;
;  
            ST_WINDATA,DATA,IWIN,ARRAY,/NO_COPY

;
;  Change the missing pixel flag in the structure.
;
	    DATA.DETDESC(IWIN).MISSING = -100.0
	ENDIF
;
;  Change the label for the units.
;
	DATA.DETDESC.UNITS = 'DEBIASED-ADC'
	GOTO, FINISH
;
;  Error handling point.
;
HANDLE_ERROR:
	IF N_ELEMENTS(ERRMSG) NE 0 THEN ERRMSG = 'VDS_DEBIAS: ' + MESSAGE $
		ELSE MESSAGE, MESSAGE, CONTINUE=KEYWORD_SET(CONTINUE)
;
;  Exit point.
;
FINISH:
	RETURN
	END
