	PRO VDS_READ_FLAT, FLAT, VOLTAGE, MODE, DATE
;+
; Project     :	SOHO - CDS
;
; Name        :	VDS_READ_FLAT
;
; Purpose     :	Reads VDS flat field images.
;
; Category    :	Class4, CDS, Calibration
;
; Explanation :	The VDS flat field images were generated using readout mode 1
;		(quadrature).  Each quadrant has a slightly different
;		amplification level, due to differences in the four amplifier
;		circuits.  When using one of the other VDS readout modes, this
;		difference must be removed.
;
;		Called from VDS_CALIB.
;
; Syntax      :	VDS_READ_FLAT, FLAT, VOLTAGE, MODE  [, DATE]
;
; Examples    :	VDS_READ_FLAT, FLAT, 756, 2
;
; Inputs      :	VOLTAGE	= Voltage value of the flat field image.  Must be one
;			  of the following: 600, 678, 756, 834, 912, 990.
;
;		MODE	= VDS readout mode.  Must be an integer from 1 to 7.
;
; Opt. Inputs :	DATE	= Date/time of the observation.
;
; Outputs     :	FLAT	= Flat field image, modified by the readout mode.
;
; Opt. Outputs:	None.
;
; Keywords    :	None.
;
; Calls       :	
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Prev. Hist. :	None.
;
; History     :	Version 1, 06-May-1996, William Thompson, GSFC
;		Version 2, 15-Jan-1999, William Thompson, GSFC
;			Added parameter DATE, to allow selection between flat
;			fields for before and after the accident.
;		Version 3, 21-Sep-1999, William Thompson, GSFC
;			Provide error message if alternate flat field file not
;			found.
;		Version 4, 27-Sep-2002, William Thompson, GSFC
;			Use FIND_WITH_DEF instead of CONCAT_DIR
;
; Contact     :	WTHOMPSON
;-
;
	ON_ERROR, 2
;
;  Check the number of parameters.
;
	IF N_PARAMS() LT 3 THEN MESSAGE,	$
		'Syntax:  VDS_READ_FLAT, FLAT, VOLTAGE, MODE'
;
;  If the observation date is after loss-of-contact, then try to read in the
;  modified flat field data for the post-recovery data.
;
	FILENAME = ''
	IF N_PARAMS() EQ 4 THEN BEGIN
	    IF ANYTIM2UTC(DATE,/CCSDS) GT '1998-10-01' THEN BEGIN
		FILENAME = 'flat_' + TRIM(VOLTAGE) + 'v2.fits'
		FILENAME = FIND_WITH_DEF(FILENAME, '$CDS_VDS_CAL_INT')
		IF FILENAME EQ '' THEN MESSAGE, /CONTINUE,		$
			'File flat_' + TRIM(VOLTAGE) +			$
			'v2.fits not found.  Using file flat_' +	$
			TRIM(VOLTAGE) + 'v.fits instead.'
	    ENDIF
	ENDIF
;
;  Read in the flat field image.
;
	IF FILENAME EQ '' THEN BEGIN
	    FILENAME = 'flat_' + TRIM(VOLTAGE) + 'v.fits'
	    FILENAME = FIND_WITH_DEF(FILENAME, '$CDS_VDS_CAL_INT')
	ENDIF
	IF FILENAME EQ '' THEN MESSAGE,	$
		'Unable to find flat field image for voltage ' + TRIM(VOLTAGE)
	FXREAD, FILENAME, FLAT
;
;  If quadrature readout mode, then return immediately.
;
	IF MODE EQ 1 THEN RETURN
;
;  Read in the multiplier information.
;
	OPENR, UNIT, FIND_WITH_DEF('multipliers.dat', '$CDS_VDS_CAL_INT'), $
		/GET_LUN
	VOLTS = VOLTAGE - 1
	WHILE VOLTS NE VOLTAGE DO BEGIN
	    IF EOF(UNIT) THEN BEGIN
		FREE_LUN, UNIT
		MESSAGE, 'Unable to modify for voltage ' + TRIM(VOLTAGE)
	    ENDIF
	    READF, UNIT, VOLTS, C_AB, C_CD, C_ABCD
	ENDWHILE
	FREE_LUN, UNIT
;
;  Apply the multipliers to accommodate modes 2 and 3.
;
	FLAT(0:511,512:*) = FLAT(0:511,512:*) / C_AB
	FLAT(512:*,512:*) = FLAT(512:*,512:*) * C_AB
	FLAT(0:511,0:511) = FLAT(0:511,0:511) / C_CD
	FLAT(512:*,0:511) = FLAT(512:*,0:511) * C_CD
;
;  If in one of the single quadrant readout modes, then apply that multiplier
;  as well.
;
	IF MODE GT 3 THEN BEGIN
	    FLAT(*,512:*) = FLAT(*,512:*) / C_ABCD
	    FLAT(*,0:511) = FLAT(*,0:511) * C_ABCD
	ENDIF
;
	RETURN
	END
