;+
; Project     : SOHO - CDS     
;                   
; Name        : VDS_ROTATE
;               
; Purpose     : Corrects full NIS spectral extraction array for rotation
;               
; Explanation : The dispersion axes of the NIS spectra are slanted relative
;               to the CCD detector (x,y) axes.  The track of constant
;               solar Y is therefore not at a constant data array Y value.
;               Under some circumstances it may be appropriate to deslant the
;               raw data array to align solay Y with detector Y.  This
;               routine does that.  Beware of the associated changes in 
;               intensity.
;               
; Use         : IDL> vds_rotate, in, out, band [+keywords]
;
;		IDL> vds_rotate, in, out, 2, /cubic, miss=-100, vel=0.0045
;    
; Inputs      : IN   - input array.  Can be full spectrum or spectral 
;                      extraction window.  If it is 3-d then the middle
;		       dimension is assumed to be the X-scan
;
;               BAND - spectral band (1 or 2). Needed since slants are 
;                      different for NIS1 and NIS2
;               
; Opt. Inputs : None
;               
; Outputs     : OUT - rectified array.
;               
; Opt. Outputs: None
;               
; Keywords    : BILINEAR = Allows bilinear interpolation during rectification
;
;               CUBIC    = Allows cubic interpolation during rectification
;                          (NOTE both of these change the intensity values.
;                           The default is to use nearest neighbour values.)
;
;               VEL_TILT = As well as the slant of the dispersion axis, the
;			   spectral lines are tilted at an angle to the
;			   dispersion direction.  Give a value here if you wish
;			   to correct this tilt as well.  Note the units are
;			   (spectral bins per pixel along the slit).  Note also
;			   that this value may change with wavelength and
;			   spectral band although the value for the OV 629 A
;			   line is well determined to be 0.0045.  It should
;			   also be noted that values of this size have no
;			   effect upon the array unless one of the
;			   interpolation methods (BILINEAR or CUBIC) is used.
;
;			   One can also pass VEL_TILT as a series of
;			   coefficients to a polynomial which expresses the
;			   tilt as a function of pixel position within the
;			   window.  GET_TILTCAL can be used to extract the
;			   proper polynomial coefficients, but only for the
;			   full spectral readout case.  Thus, this option is
;			   only recommended for full spectral readouts.  For
;			   some reason, passing polynomial coefficients seems
;			   to work best with /BILINEAR rather than /CUBIC.
;
;		BOTTOM	 = If set, then the data is aligned so that the bottom
;			   of the slit is aligned with the bottom of the array.
;			   Ignored unless the data contains a full spectral
;			   readout.
;
;		MISSING	 = The value used to flag missing pixels.
;
;		BINY	 = The binning factor in Y.  Used to adjust the tilt
;			   and slant values.  Only needed for data with
;			   compression mode 8 (e.g. USUN_6).
;
;		MIRROR_SHIFT = An array containing the amount of pixels to
;			   shift the spectrum for each X position, to account
;			   for the scan mirror.
;
;		XYOFFSETS = Offsets in the X and Y spatial directions.  This
;			    keyword is used to support the ALIGN keyword in
;			    NIS_ROTATE.
;
; Calls       : FIND_WITH_DEF, GET_UTC, GET_VDS_SLITPOS, POLY_2D
;
; Common      : None
;               
; Restrictions: None
;               
; Side effects: None
;               
; Category    : Data analysis
;               
; Prev. Hist. : None
;
; Written     : C D Pike, RAL, 29-Jul-96
;               
; Modified    : Handle smaller windowed data.  CDP, 9-Sep-96
;               Handle 3-d data input. CDP, 16-Sep-96
;		Version 4, 18-Oct-1996, William Thompson, GSFC
;			Added keyword MISSING
;			Deslant all data, including data windows.
;			Calculate KX and KY analytically, instead of using
;				POLYWARP.
;			Fixed typo involving BILINEAR keyword.
;		Version 5, 21-Oct-1996, William Thompson, GSFC
;			Take into account that full NIS2 data is padded by
;			READCDSFITS.
;			Modify default slant for NIS2.
;			Semi-automatic missing pixel recognition.
;			Improve nearest-neighbor behavior.
;		Version 6, 02-May-1997, William Thompson, GSFC
;			Added keyword BOTTOM
;			Corrected minor bug where wrong Y0 used for full NIS1
;			data.
;		Version 7, 13-Jan-1998, William Thompson, GSFC
;			Added option for VEL_TILT to be polynomial
;			coefficients.
;		Version 8, 22-Jun-1998, William Thompson, GSFC
;			Fixed bug involving time-series data.
;		Version 9, 23-Sep-1998
;			Fixed bug where some pixels would be erroneously marked
;			as missing in full-spectrum NIS-2 data, when the input
;			and output arrays are the same.
;		Version 10, 29-Sep-1999
;			Fixed bug where the bottom-most row would be
;			unaffected.  Added keywords BINY, MIRROR_SHIFT.
;			Fixed bug of wrong pivot point for 4D arrays.
;		Version 11, 30-Aug-2000, William Thompson, GSFC
;			Added keyword XYOFFSETS
;		Version 12, 14-Sep-2000, William Thompson, GSFC
;			Fixed bug in Version 11 that interfered with
;			MIRROR_SHIFT keyword.
;
; Version     : Version 12, 14-Sep-2000
;-            


pro vds_rotate, in_array, out, nis, bilinear=bilinear, cubic=cubic,	$
	vel_tilt=vel_tilt, missing=k_missing, bottom=bottom, biny=biny,	$
	mirror_shift=k_mirror_shift, xyoffsets=xyoffsets

if n_params() lt 3 then begin
   print,'Use: vds_rotate, in_array, out_array, band [+ keywords]'
   return
endif
if n_elements(in_array) eq 0 then message, 'IN_ARRAY not defined'

;
;  Determine what to use for the missing pixel flag value.  If the keyword is
;  passed, then use that.  If the minimum value is either -1 or -100, then use
;  that.  Otherwise, don't use a missing pixel flag value.
;
if n_elements(k_missing) eq 1 then missing = k_missing else begin
	imin = min(in_array)
	if (imin eq -1) or (imin eq -100) then missing = imin
endelse

sz = size(in_array)
def = in_array(0)
if n_elements(missing) eq 1 then def(0) = missing
case sz(0) of
    2:  begin
	in = replicate(def,sz(1),sz(2)+2)
	in(*,1:sz(2)) = in_array
	if n_elements(missing) ne 1 then begin
	    in(*,0) = in_array(*,0)
	    in(*,sz(2)+1) = in_array(*,sz(2)-1)
	endif
	end
    3:  begin
	in = replicate(def,sz(1),sz(2)+2,sz(3)+2)
	in(*,1:sz(2),1:sz(3)) = in_array
	if n_elements(missing) ne 1 then begin
	    in(*,*,0) = in(*,*,1)
	    in(*,*,sz(3)+1) = in(*,*,sz(3))
	    in(*,0,*) = in(*,1,*)
	    in(*,sz(2)+1,*) = in(*,sz(2),*)
	endif
	end
    4:  begin
	in = replicate(def,sz(1),sz(2)+2,sz(3)+2,sz(4))
	in(*,1:sz(2),1:sz(3),*) = in_array
	if n_elements(missing) ne 1 then begin
	    in(*,*,0,*) = in(*,*,1,*)
	    in(*,*,sz(3)+1,*) = in(*,*,sz(3),*)
	    in(*,0,*,*) = in(*,1,*,*)
	    in(*,sz(2)+1,*,*) = in(*,sz(2),*,*)
	endif
	end
endcase

;
;  Setup default slants.  Normally taken from state database, but in
;  case sites do not have that, then hard code here.
;
if find_with_def('cdhsstate.dbf','$ZDBASE') ne '' then begin
	get_utc, date
	get_vds_slitpos, date, desc
	diff1 = desc.n1_cen_l - desc.n1_cen_s
	diff2 = desc.n2_cen_l - desc.n2_cen_s
end else begin
	diff1 = 17
	diff2 = 10
endelse

case nis of
      1: slant = diff1 / 1024.
      2: slant = diff2 / 1024.
endcase
if n_elements(biny) eq 1 then slant = slant / float(biny(0) > 1)
;
;  is velocity tilt wanted as well?
;
if n_elements(vel_tilt) eq 0 then tilt = 0.0 else tilt = vel_tilt
if n_elements(biny) eq 1 then tilt = tilt * float(biny(0) > 1)

;
;  Size of input array, and the center pixels in the wavelength and Solar Y
;  directions.  Don't let Y0 exceed 77 for NIS2--fix to take into account that
;  full NIS2 readout is padded by READCDSFITS.
;
s = size(in)
w0 = (s(1) - 1.) / 2.
y0 = ((s(s(0)<3) - 1.) / 2.)
if nis eq 2 then y0 = y0 < (72 + diff2 / 2.)
;
;  If mirror_shift was passed, then make sure that it matches the number of X
;  positions.  First pad out the first and last positions to match the padding
;  done to the input array.
;
if n_elements(k_mirror_shift) gt 0 then begin
    nn = n_elements(k_mirror_shift)
    mirror_shift = [k_mirror_shift(0),k_mirror_shift,k_mirror_shift(nn-1)]
    if (n_elements(mirror_shift) ne s(2)) and (s(0) gt 2) then begin
	message, 'MIRROR_SHIFT must have ' + NTRIM(S(2)) +	$
		' elements -- ignoring', /CONTINUE
    end else if (n_elements(mirror_shift) ne 1) and (s(0) eq 2) then begin
	message, 'MIRROR_SHIFT must be a scalar -- ignoring', /CONTINUE
    end else mir_shift = mirror_shift
endif

;
;  set interpolation mode
;
case 1 of
   keyword_set(bilinear): mode = 1
   keyword_set(cubic):    mode = 2
   else:                  mode = 0
endcase

;
;  Setup the polynomials for the dewarping routine.
;
kx = replicate(0*tilt(0), n_elements(tilt)+1, n_elements(tilt)+1)
ky = kx
kx(0,1) = 1
for i = 0,n_elements(tilt)-1 do begin
	kx(0,i) = kx(0,i) + tilt(i)*(slant*w0 + y0)
	kx(0,i+1) = kx(0,i+1) - slant*tilt(i)
	kx(1,i) = kx(1,i) - tilt(i)
endfor
ky(0,0) = [[-slant*w0, 1], [slant, 0]]

;
;  If nearest neighbor is to be used, then add 0.5 to the zero-order terms of
;  KX and KY.  This takes into account the difference between truncating and
;  rounding off.
;
if mode eq 0 then begin
	kx(0) = kx(0) + 0.5
	ky(0) = ky(0) + 0.5
endif

;
;  If the /BOTTOM keyword was set, then calculate the offset needed to align
;  the data at the bottom of the array.
;
if keyword_set(bottom) then ky(0) = ky(0) + ((y0-72) > 0)

;
;  If the XYOFFSETS keyword was passed with two values, then include the Y
;  offset.
;
if n_elements(xyoffsets) eq 2 then ky(0) = ky(0) - xyoffsets(1)

;
;  Deslant and detilt the data.  Take into account any missing pixels.
;
kx0 = kx(0)
if s(0) eq 4 then begin
   out = in
   for j=0,s(4)-1 do begin
      for i=0,s(2)-1 do begin
	 if n_elements(mir_shift) gt 0 then kx(0) = kx0 + mir_shift(i)
         temp = reform(in(*,i,*,j))
         tempout = poly_2d(temp, kx, ky, mode)
         if n_elements(missing) eq 1 then begin
	    test = poly_2d(float(temp eq missing), kx, ky, mode)
	    w = where(test ne 0, count)
	    if count gt 0 then tempout(w) = missing
         endif
         out(*,i,*,j) = tempout
      endfor
   endfor
end else if s(0) eq 3 then begin
   out = in
   for i=0,s(2)-1 do begin
      if n_elements(mir_shift) gt 0 then kx(0) = kx0 + mir_shift(i)
      temp = reform(in(*,i,*))
      tempout = poly_2d(temp, kx, ky, mode)
      if n_elements(missing) eq 1 then begin
	 test = poly_2d(float(temp eq missing), kx, ky, mode)
	 w = where(test ne 0, count)
	 if count gt 0 then tempout(w) = missing
      endif
      out(*,i,*) = tempout
   endfor
end else begin
   out = poly_2d(in, kx, ky, mode)
   if n_elements(missing) eq 1 then begin
      if n_elements(mir_shift) gt 0 then kx(0) = kx0 + mir_shift(0)
      test = poly_2d(float(in eq missing), kx, ky, mode)
      w = where(test ne 0, count)
      if count gt 0 then out(w) = missing
   endif
endelse

;
;  If a non-trivial X offset was passed, then apply the X offset to the data.
;
if n_elements(xyoffsets) eq 2 then if xyoffsets(0) ne 0 then begin
    kx = [[-xyoffsets(0),0.],[1.,0.]]
    ky = [[0.,1.],[0.,0.]]
    if s(0) eq 4 then begin
	for j=0,s(4)-1 do begin
	    for i=0,s(1)-1 do begin
		temp = reform(out(i,*,*,j))
		tempout = poly_2d(temp, kx, ky, mode)
		if n_elements(missing) eq 1 then begin
		    test = poly_2d(float(temp eq missing), kx, ky, mode)
		    w = where(test ne 0, count)
		    if count gt 0 then tempout(w) = missing
		endif
		out(i,*,*,j) = tempout
	    endfor
	endfor
    end else if s(0) eq 3 then begin
	for i=0,s(1)-1 do begin
	    temp = reform(out(i,*,*))
	    tempout = poly_2d(temp, kx, ky, mode)
	    if n_elements(missing) eq 1 then begin
		test = poly_2d(float(temp eq missing), kx, ky, mode)
		w = where(test ne 0, count)
		if count gt 0 then tempout(w) = missing
	    endif
	    out(i,*,*) = tempout
	endfor
    endif
endif

case sz(0) of
    2: out = out(*,1:sz(2))
    3: out = out(*,1:sz(2),1:sz(3))
    4: out = out(*,1:sz(2),1:sz(3),*)
endcase
end
