	PRO CONV_WAVECAL3
;+
; Project     :	SOHO - CDS
;
; Name        :	CONV_WAVECAL3
;
; Purpose     :	Converts NIS/GIS wavelength calibration database.
;
; Explanation :	Converts NIS/GIS wavelength calibration database to include an
;		effective date.
;
; Use         :	One should use the following procedure to convert the databases
;		to the new formats:
;
;			1.  The old nis_wave.dbd file should be listed as
;			    version 2 (5 July 1996), and the old gis_wave.dbd
;			    file should be listed as version 1 (5 March 1996).
;			    The new versions of both files should be listed as
;			    version 3 (18 April 1997).
;
;			2.  Change to the directory containing the database
;			    files to be converted.  *** Very important ***
;
;			3.  Replace the old nis_wave.dbd and gis_wave.dbd files
;			    with the new files.  You may wish to rename the old
;			    files to "old_nis_wave.dbd" and "old_gis_wave.dbd".
;
;			4.  Start up IDL and compile this procedure with the
;			    command ".run conv_wavecal3.prg".  Naming the
;			    file with the .prg extension is used to ensure that
;			    the routine is not executed accidently.
;
;			4.  Make sure that the software is enabled by entering
;			    the command "!priv = 5".  This is another
;			    protective step.
;
;			5.  Execute the procedure with the command
;			    "conv_wavecal3".
;
; Category    :	This is not part of the CDS software tree.
;
; Written     :	William Thompson, GSFC, 18 April 1997
;-
;
	ON_ERROR, 2
;
;  Make sure that the user has the privilege set correctly to run this routine.
;
	IF !PRIV LT 5 THEN MESSAGE,	$
		'!PRIV must be set to 5 to run this routine.'
;
;  Make sure that the user really wants to convert the database.
;
	PRINT, 'Converting nis_wave and gis_wave databases to version 3.'
	ASK, 'Are you sure you want to do this? ', ANSWER
	IF ANSWER NE 'Y' THEN BEGIN
		PRINT, 'Execution aborted'
		RETURN
	ENDIF
;
;  Determine the correct commands for renaming and deletion.
;
	CASE OS_FAMILY() OF
		'vms': BEGIN
			RENAME = 'RENAME'
			DELETE = 'DELETE'
			END
		'unix': BEGIN
			RENAME = 'mv'
			DELETE = 'rm'
			END
		ELSE: MESSAGE, 'Only supported in VMS or Unix'
	ENDCASE
;
;  Check to make sure that the proper database files are in the current
;  directory.
;
	MESSAGE = 'Database files not in current directory'
	ON_IOERROR, HANDLE_ERROR
	OPENR, UNIT, 'nis_wave.dbf', /GET_LUN
	FREE_LUN, UNIT
;
;  Preserve the old database files by renaming them.
;
	MESSAGE = 'Error occured'
	SPAWN, RENAME + ' nis_wave.dbf old_nis_wave.dbf'
	SPAWN, RENAME + ' nis_wave.dbx old_nis_wave.dbx'
	SPAWN, RENAME + ' nis_wave.dbh old_nis_wave.dbh'
	SPAWN, RENAME + ' gis_wave.dbf old_gis_wave.dbf'
	SPAWN, RENAME + ' gis_wave.dbx old_gis_wave.dbx'
	SPAWN, RENAME + ' gis_wave.dbh old_gis_wave.dbh'
;
;  Create empty versions of the new database files.
;
	DBCREATE, 'nis_wave', 1, 1, /EXTERNAL
	DBCREATE, 'gis_wave', 1, 1, /EXTERNAL
;
;  Open the old NIS database file, and read in the data.
;
	DBOPEN, 'old_nis_wave'
	DBEXT, -1, 'DATE,FILENAME,COEFF', DATE, FILENAME, COEFF
	DBCLOSE
;
;  Create effective dates for all entries equal to the launch date.
;
	DATE_EFF = REPLICATE(UTC2TAI('1995-12-02'), N_ELEMENTS(DATE))
;
;  Insert the data into the new nis_wave database.
;
	DBOPEN, 'nis_wave', 1
	DBBUILD, DATE, DATE_EFF, FILENAME, COEFF
	DBCLOSE
;
;  Do the same for gis_wave.
;
	DBOPEN, 'old_gis_wave'
	DBEXT, -1, 'DATE,GSET_ID,FILENAME,COEFF', DATE, GSET_ID, FILENAME, $
		COEFF
	DBCLOSE
;
;  Create effective dates for all entries equal to the launch date.
;
	DATE_EFF = REPLICATE(UTC2TAI('1995-12-02'), N_ELEMENTS(DATE))
;
;  Insert the data into the new gis_wave database.
;
	DBOPEN, 'gis_wave', 1
	DBBUILD, DATE, DATE_EFF, GSET_ID, FILENAME, COEFF
	DBCLOSE
;
;  Clean up.
;
	ASK ,'Conversion complete, do you wish to delete the old files? ', $
		ANSWER
	IF ANSWER EQ 'Y' THEN SPAWN, DELETE +	$
		' old_nis_wave.db* old_gis_wave.db*'
	RETURN
;
;  Error handling point.
;
HANDLE_ERROR:
	MESSAGE, MESSAGE
;
	END
