FUNCTION QTILT, WAVE, REQ_DATE
;+
; Project     :	SOHO - CDS
;
; Name        :	QTILT()
;
; Purpose     :	Get the NIS tilt for a given wavelength.
;
; Category    :	Calibration, NIS, Coordinates
;
; Explanation :	Looks up the NIS tilt calibration in the database for a given
;		spectrum and date combination and calculates the tilt for a 
;               specific wavelength.  
;
; Syntax      :	PRINT, QTILTCAL( WAVE, [REQ_DATE])
;
; Examples    :	IDL> print, qtilt(584.3)
;		
;
; Inputs      :	WAVE  - the wavelength of the tilt required
;
;
; Opt. Inputs :	REQ_DATE = The date to use in searching for a wavelength
;			   calibration.
;
; Outputs     :	Returns tilt in pixels per pixel along slit.
;
; Opt. Outputs:	None.
;
; Keywords    :	NONE
;
; Calls       :	GET_TILTCAL
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Prev. Hist. :	
;
; History     :	Version 1, 7-Jun-1999, CDP, RAL
;
;-
;
	on_error, 2
;
;  Check the input parameters.
;
	if n_params() ne 2 then begin
           get_utc, req_date
	endif

;
;   convert date to standard
;
        date = anytim2utc(req_date)

;
;  Check the spectrum value.
;
        spec = 0
        if wave gt 307.1 and wave lt 379.44 then spec = 1
        if wave gt 511.6 and wave lt 632.82 then spec = 2

        if spec eq 0 then begin
           print,'Wavelength out of range'
           return, 0.0
        endif

;
;  get tilt calibration
;
        get_tiltcal,spec,date,coeff

;
;  get tilt for specific wavelength
;
        if spec eq 1 then begin
           pix = wave2pix('n1',wave)
        endif else begin
           pix = wave2pix('n2',wave)
        endelse

;
;  return with tilt
;
        return,poly(pix,coeff)

	end
