;+
; Project     : SOHO - CDS     
;                   
; Name        : CDS_AUTOFIT
;               
; Purpose     : To attempt to automatically detect and characterise a CDS
;               spectrum in terms of a series of gaussian line fits.
;               
; Explanation : Given a structure containing a CDS spectrum (such as returned
;               by PLOT_SPEC) subdivide the spectrum into small segments and
;               search each for "lines" above a certain threshold. If found,
;               fit "lines" with either upto 3 gaussians. Optional return the
;               fitted gaussian parameters and continuum values for use with
;               other routines.
;               
; Use         : IDL> cds_autofit, sp, [ results = gpars ]
;    
; Inputs      : sp   -  a structure containing the input spectrum (wavelengths
;                       and fluxes).
;               
; Opt. Inputs : None.
;
; Outputs     : None.
;               
; Opt. Outputs: Return a structure containing the fitted parameters for each
;               line identified in the input spectrum.
;               
; Keywords    : RESULTS - a structure contain all the relevant gaussian 
;                         parameters pertaining to the fit. It also includes
;                         the CDS subset of lines from the list of Kelly that
;                         lie in the fitted interval.
;
; Calls       : CDS_MULTIG - to do the actual line fitting.
;               CDS_LINES - does the selection of "acceptable" lines. (This
;                           is necessary because the fitting procedure will
;                           find the SAME line multiple times and will also
;                           found "erroneous" lines; eg. too narrow or wide).
;               CDS_SLINE - to get Kelly line list.
;               CDS_PLOT_FIT - to display the results.
;
; Common      : None
;               
; Restrictions: None
;               
; Side effects: None
;               
; Category    : Data_Analysis
;               
; Prev. Hist. : None
;
; Written     : B J Kellett 2/12/95.
;               
; Modified    : 
;
; Version     : Version 1.0, 2-Dec-95
;-            

 PRO cds_autofit, sp, results=results

 cwave = sp.w
 cspec = sp.flux
;
 gwid = 16
 npix = 2 * gwid + 1
 dxpix = 25
;
 thres = 0.75 * sqrt ( min ( cspec ) )
;
 glfit = fltarr ( 750 )
 gxfit = fltarr ( 750 )
 gwfit = fltarr ( 750 )
 ghfit = fltarr ( 750 )
 gbfit = fltarr ( 750 )
 gcfit = fltarr ( 3, 750 )
 elfit = fltarr ( 750 )
 ewfit = fltarr ( 750 )
 ehfit = fltarr ( 750 )
 ecfit = fltarr ( 3, 750 )
 nlines = 0
;
 csperr = sqrt ( cspec )
;
 nfits = n_elements ( cwave ) / ( npix - dxpix ) - 3
;
 !p.multi = [ 0, 6, 3 ]
 !p.charsize = 1.25
;
 loadct, 2, /silent
;
 for i = 0, nfits do begin
;
     iwl = i * ( npix - dxpix )
     iwh = iwl + npix - 1
     iwm = iwl + gwid
;
     if ( iwh gt ( n_elements ( cwave ) - 1 ) ) then begin
         iwh = n_elements ( cwave ) - 1
         iwl = iwh - npix + 1
         iwm = iwl + gwid
     endif 
;
     x = cwave ( iwl:iwh ) - cwave ( iwm )
     y = cspec ( iwl:iwh )
     dy = csperr ( iwl:iwh )
;
     cds_multig, x, y, dy, gpar, gperr, cwave ( iwm ), thres, /quiet
;
     ngpar = n_elements ( gpar )
;
     if ( ngpar lt 5 ) then goto, scont
;
     nfl = ( ngpar - 3 ) / 3
;
     iwl = 3 * indgen ( nfl )
;
     gcf = gpar ( ngpar-3:ngpar-1 )
     gce = gperr ( ngpar-3:ngpar-1 )
;
     xf = gpar ( iwl+1 ) - cwave ( iwm )
;
     if ( nfl eq 1 ) then xf = [ xf, xf ]
;
     bb = poly ( xf, gcf )
;
     if ( nfl eq 1 ) then bb = bb ( 0 )
;
     glfit ( nlines:nlines+nfl-1 ) = gpar ( iwl+1 )
     gxfit ( nlines:nlines+nfl-1 ) = gpar ( iwl+1 ) - cwave ( iwm )
     gwfit ( nlines:nlines+nfl-1 ) = gpar ( iwl+2 ) 
     ghfit ( nlines:nlines+nfl-1 ) = gpar ( iwl )
     gbfit ( nlines:nlines+nfl-1 ) = bb
     for k = nlines, nlines+nfl-1 do gcfit ( 0:2, k ) = gcf
;
     elfit ( nlines:nlines+nfl-1 ) = gperr ( iwl+1 )
     ewfit ( nlines:nlines+nfl-1 ) = gperr ( iwl+2 ) 
     ehfit ( nlines:nlines+nfl-1 ) = gperr ( iwl )
     for k = nlines, nlines+nfl-1 do ecfit ( 0:2, k ) = gce
;
     nlines = nlines + nfl
;
 scont:
;
 endfor
;
 glfit = glfit ( 0:nlines-1 )
 gxfit = gxfit ( 0:nlines-1 )
 gwfit = gwfit ( 0:nlines-1 )
 ghfit = ghfit ( 0:nlines-1 )
 gbfit = gbfit ( 0:nlines-1 )
 gcfit = gcfit ( 0:2, 0:nlines-1 )
 elfit = elfit ( 0:nlines-1 )
 ewfit = ewfit ( 0:nlines-1 )
 ehfit = ehfit ( 0:nlines-1 )
 ecfit = ecfit ( 0:2, 0:nlines-1 )
;
 cds_lines, cwave, cspec, glfit, gwfit, ghfit, gxfit, gbfit, gcfit, elfit,    $
            ewfit, ehfit, ecfit
;
 print, ' '
 print, '     Getting Blue Book line list ... '
;
 wmin = min ( cwave ) - 2.5
 wmax = max ( cwave ) + 2.5
;
 lines = cds_sline ( wmin, wmax, /cds )
;
 ncds = n_elements ( lines ) / 3
;
 wline = fltarr ( ncds )
 welem = strarr ( ncds )
;
 for i = 0, ncds - 1 do begin
     wline ( i ) = float ( lines ( 2, i ) )
     welem ( i ) = strcompress ( lines ( 0, i ) + lines ( 1, i ) )
 endfor
;
 if ( ( wmin lt 605.0 ) and ( wmax gt 609.0 ) ) then begin
     wel_heii = 'HeII(2)'
     ww_heii = 2.0 * 303.78
;
     wline = [ wline, ww_heii ]
     welem = [ welem, wel_heii ]
 endif
;
 nl = n_elements ( glfit )
 nk = n_elements ( wline )
;
 results = { w: fltarr ( nl ), sig: fltarr ( nl ), h: fltarr ( nl ),          $
             x: fltarr ( nl ), bgnd: fltarr ( nl ), cf: fltarr ( 3, nl ),     $
             werr: fltarr ( nl ), sigerr: fltarr ( nl ), herr: fltarr ( nl ), $
             cferr: fltarr ( 3, nl ), kw: fltarr ( nk ), kelem: strarr ( nk ) }
;
 results.w = glfit
 results.sig = gwfit
 results.h = ghfit
 results.x = gxfit
 results.bgnd = gbfit
 results.cf = gcfit
 results.werr = elfit
 results.sigerr = ewfit
 results.herr = ehfit
 results.cferr = ecfit
;
; Add Kelly Lines Selected
;
 results.kw = wline
 results.kelem = welem
;
 cds_plot_fit, sp, results=results, /fill, /kelly, /log
;
 end
; 
