;+
; Project     : SOHO - CDS     
;                   
; Name        : CDS_FIT_WAVE ()
;               
; Purpose     : To fit upto 3 gaussians to a selected wavelength in a CDS spectrum
;               
; Explanation : Having supplied a spectrum to the routine (from PLOT_SPEC for
;               example), the user can supply a wavelength of interest and this
;               routine will select a small region of the spectrum around that
;               wavelength and then attempt to characterise the region with
;               up to 3 gaussians and a quadratic background.
;               
; Use         : IDL> r = cds_fit_wave ( sp, wc [, width = gw, out = gp ] )
;    
; Inputs      : sp   -  a structure containing the input spectrum (wavelengths
;                       and fluxes).
;               wc   -  central wavelength of interest.
;               
; Opt. Inputs : None.
;
; Outputs     : None.
;               
; Opt. Outputs: Return a structure containing the fitted parameters for the 
;               input spectrum.
;               
; Keywords    : WIDTH - Specifies the width of the data region to be fitted (in
;                       pixels). If not specified, 35 is taken as the default.
;
; Calls       : CDS_MULTIG - to do the actual line fitting.
;
; Common      : None
;               
; Restrictions: None
;               
; Side effects: None
;               
; Category    : Data_Analysis
;               
; Prev. Hist. : None
;
; Written     : B J Kellett 2/12/95.
;               
; Modified    : 
;
; Version     : Version 1.0, 2-Dec-95
;-            
;

 FUNCTION cds_fit_wave, sp, wcen0, width = wid, out = out, quiet = quiet,    $
                        threshold = threshold, noplot = noplot, fint = fint

 cwave = sp.w
 cspec = sp.flux
;
;  default is to give verbose output. Use the quiet switch to switch off.
;

 bequiet = 0
 if keyword_set ( quiet ) then bequiet = 1

 iplot = 1
 if keyword_set ( noplot ) then begin
     iplot = 0
     bequiet = 1
 endif

 gwid = 35

 if keyword_set ( wid ) then gwid = wid 
;
 gwid = gwid / 2
;
 npix = 2 * gwid + 1
;
 if keyword_set ( threshold ) then thres = threshold                          $
 else thres = 0.75 * sqrt ( min ( cspec ) )
;
 csperr = sqrt ( cspec )
;
;;; !p.multi = 0
;
 wcen = wcen0
 if ( n_elements ( wcen ) gt 1 ) then mfit = 1 else mfit = 0
 if ( n_elements ( wcen ) eq 1 ) then wcen = [ wcen0, wcen0 ]
;
 nfit = n_elements ( wcen )
;
 ifit = 0
 nlines = 0
;
 fcont:
;
 dw = cwave - wcen ( ifit )
 ww = min ( abs ( dw ) )
;
 ic = !c
;
 iwl = ic - gwid
 iwh = iwl + npix - 1
 iwm = iwl + gwid
;
 if ( iwl lt 0 ) then iwl = 0
 if ( iwh gt ( n_elements ( cwave ) - 1 ) ) then iwh = n_elements ( cwave ) - 1
;
 if (not bequiet) then begin
     print,' '
     print,'   Selected:-', cwave ( iwl ), ' to', cwave ( iwh ),              $
           ': Centered @', cwave ( iwm )
 endif
;
 x = cwave ( iwl:iwh ) - cwave ( iwm )
 dx = 0. * x + ( 0.5 * ( x ( 1 ) - x ( 0 ) ) )
 y = cspec ( iwl:iwh )
 dy = csperr ( iwl:iwh )
;
 imode = 0
 if ( not iplot ) then imode = 1
 if ( bequiet ) then imode = imode + 2

 wc = wcen ( ifit )

 case imode of
       0:  cds_multig, x, y, dy, gpar, gperr, wc, thres, fint = fi,           $
                       ferr = fe, cint = ci, cerr = ce
       1:  cds_multig, x, y, dy, gpar, gperr, wc, thres, fint = fi,           $
                       ferr = fe, cint = ci, cerr = ce, /noprint
       2:  cds_multig, x, y, dy, gpar, gperr, wc, thres, fint = fi,           $
                       ferr = fe, cint = ci, cerr = ce, /quiet
       3:  cds_multig, x, y, dy, gpar, gperr, wc, thres, fint = fi,           $
                       ferr = fe, cint = ci, cerr = ce, /quiet, /noprint
    else: print, '  ILLEGAL Mode', imode
 endcase
;
 ngpar = n_elements ( gpar )
;
 if ( ngpar lt 5 ) then return, 0
;
 nfl = ( ngpar - 3 ) / 3
;
 igl = 3 * indgen ( nfl )
;
 gcf = gpar ( ngpar-3:ngpar-1 )
 gce = gperr ( ngpar-3:ngpar-1 )
;
 xf = gpar ( igl+1 ) - cwave ( iwm )
;
 if ( nfl eq 1 ) then xf = [ xf, xf ]
;
 bb = poly ( xf, gcf )
;
 if ( nfl eq 1 ) then bb = bb ( 0 )
;
 glfit = fltarr ( nfl )
 gcfit = fltarr ( 3, nfl )
 gxfit = glfit 
 ghfit = glfit 
 gwfit = glfit 
 gbfit = glfit 
 elfit = fltarr ( nfl )
 ecfit = fltarr ( 3, nfl )
 ewfit = glfit 
 ehfit = glfit 
;
 glfit ( 0:nfl-1 ) = gpar ( igl+1 )
 gxfit ( 0:nfl-1 ) = gpar ( igl+1 ) - wc
 gwfit ( 0:nfl-1 ) = gpar ( igl+2 ) 
 ghfit ( 0:nfl-1 ) = gpar ( igl )
 gbfit ( 0:nfl-1 ) = bb
 for k = 0, nfl-1 do gcfit ( 0:2, k ) = gcf
;
 elfit ( 0:nfl-1 ) = gperr ( igl+1 )
 ewfit ( 0:nfl-1 ) = gperr ( igl+2 ) 
 ehfit ( 0:nfl-1 ) = gperr ( igl )
 for k = 0, nfl-1 do ecfit ( 0:2, k ) = gce
;
 if ( not bequiet ) then begin
     print, ' '
     print, '     Getting Blue Book line list ... '
;
     if ( mfit eq 0 ) then rwrange = [ cwave ( iwl ), cwave ( iwh ) ]
;
     wmin = cwave ( iwl ) - 0.25
     wmax = cwave ( iwh ) + 0.25
;
     lines = cds_sline ( wmin, wmax, /cds )
;
     ncds = n_elements ( lines ) / 3
;
     wline = fltarr ( ncds )
     welem = strarr ( ncds )
;
     for i = 0, ncds - 1 do begin
         wline ( i ) = float ( lines ( 2, i ) )
         welem ( i ) = strcompress ( lines ( 0, i ) + lines ( 1, i ) )
     endfor
;
     if ( ( wmin lt 605.0 ) and ( wmax gt 609.0 ) ) then begin
         wel_heii = 'HeII(2)'
         ww_heii = 2.0 * 303.78
;
         wline = [ wline, ww_heii ]
         welem = [ welem, wel_heii ]
     endif
 endif else begin
     wline = fltarr ( 4 )
     welem = strarr ( 4 )
 endelse
;
 if ( mfit ne 0 ) then begin
     if ( ifit eq 0 ) then begin
         rlfit = fltarr ( 750 )
         rxfit = fltarr ( 750 )
         rwfit = fltarr ( 750 )
         rhfit = fltarr ( 750 )
         rbfit = fltarr ( 750 )
         rcfit = fltarr ( 3, 750 )
         relfit = fltarr ( 750 )
         rewfit = fltarr ( 750 )
         rehfit = fltarr ( 750 )
         recfit = fltarr ( 3, 750 )

         rkw = wline  
         rkelem = welem
         rfint = fi
         rferr = fe
         rcint = ci
         rcerr = ce
         wr = [ cwave ( iwl ), cwave ( iwh ) ]
         rwrange = wr
     endif
;
     rlfit ( nlines:nlines+nfl-1 ) = glfit
     rxfit ( nlines:nlines+nfl-1 ) = glfit - wc
     rwfit ( nlines:nlines+nfl-1 ) = gwfit
     rhfit ( nlines:nlines+nfl-1 ) = ghfit
     rbfit ( nlines:nlines+nfl-1 ) = gbfit
     for k = nlines, nlines+nfl-1 do rcfit ( 0:2, k ) = gcf
;
     relfit ( nlines:nlines+nfl-1 ) = elfit
     rewfit ( nlines:nlines+nfl-1 ) = ewfit
     rehfit ( nlines:nlines+nfl-1 ) = ehfit
     for k = nlines, nlines+nfl-1 do recfit ( 0:2, k ) = gce
;
     if ( ifit gt 0 ) then begin
         rkw = [ rkw, wline   ]
         rkelem = [ rkelem, welem ]
         rfint = [ rfint, fi ]
         rferr = [ rferr, fe ]
         rcint = [ rcint, ci ]
         rcerr = [ rcerr, ce ]
         wr = [ cwave ( iwl ), cwave ( iwh ) ]
         rwrange = [ rwrange, wr ]
     endif

     ifit = ifit + 1

     nlines = nlines + nfl

     if ( ifit lt nfit ) then wait, 1.25
     if ( ifit lt nfit ) then goto, fcont 
 endif
     
 if ( mfit ne 0 ) then begin
     glfit = rlfit ( 0:nlines - 1 )
     gwfit = rwfit ( 0:nlines - 1 )
     ghfit = rhfit ( 0:nlines - 1 )
     gxfit = rxfit ( 0:nlines - 1 )
     gbfit = rbfit ( 0:nlines - 1 )
     gcfit = rcfit ( 0:2, 0:nlines - 1 )
     elfit = relfit ( 0:nlines - 1 )
     ewfit = rewfit ( 0:nlines - 1 )
     ehfit = rehfit ( 0:nlines - 1 )
     ecfit = recfit ( 0:2, 0:nlines - 1 )

     wline = rkw 
     welem = rkelem 
     fi  = rfint 
     fe = rferr 
     ci = rcint 
     ce = rcerr 
 endif
;
 nl = n_elements ( glfit )
 nk = n_elements ( wline )
 nw = n_elements ( rwrange )
;
 if ( nw eq 0 ) then begin
     rwrange = [ min ( cwave ), max ( cwave ) ]
     nw = n_elements ( rwrange )
 endif
;
 results = { w: fltarr ( nl ), sig: fltarr ( nl ), h: fltarr ( nl ),          $
             x: fltarr ( nl ), bgnd: fltarr ( nl ), cf: fltarr ( 3, nl ),     $
             werr: fltarr ( nl ), sigerr: fltarr ( nl ), herr: fltarr ( nl ), $
             cferr: fltarr ( 3, nl ), kw: fltarr ( nk ), kelem: strarr ( nk ),$
             fint: fltarr ( nl ), ferr: fltarr ( nl ), cint: fltarr ( nl ),   $
             cerr: fltarr ( nl ), wrange: fltarr ( nw ), info: strarr ( 18 ) }
;
; Add Gaussian Fitted Parameters.
;
 results.w = glfit
 results.sig = gwfit
 results.h = ghfit
 results.x = gxfit
 results.bgnd = gbfit
 results.cf = gcfit
 results.werr = elfit
 results.sigerr = ewfit
 results.herr = ehfit
 results.cferr = ecfit
;
; Add Kelly Lines Selected
;
 results.kw = wline
 results.kelem = welem
;
; Add Line Intensity Parameters
;
 results.fint = fi
 results.ferr = fe
 results.cint = ci
 results.cerr = ce
;
; Add Wavelength Range (to assist CDS_PLOT_FIT !!)
;
 results.wrange = rwrange
;
; Add Information on what above structure contains!
;
 results.info = [ 'Gaussian Center', 'Gaussian Width', 'Gaussian Height',     $
                  'Gaussian X', 'Continuum Bgnd.', 'Continuum Polynomial',    $
                  'Error on GC', 'Error on GW', 'Error on GH',                $
                  'Error on Cont. Poly.', 'Kelly W', 'Kelly Elem.',           $
                  'Fitted Line Intensity', 'Error on Fitted Int.',            $
                  'Estimated Counts Intensity', 'Error on Estimated C. Int.', $
                  'Wavelength Interval', 'Information Structure' ]
;
 out = gpar
 fint = fi
;
 return, results
; 
 end
; 
