;+
; Project     : SOHO - CDS     
;                   
; Name        : CDS_LINEFIT_WRAP
;               
; Purpose     : Wrapper routine for several CDS line fitting procedures.
;               
; Explanation : Many different line fitting procedures have been written for
;               the CDS project. This is an attempt to make a coherent
;               interface for some of them.
;
;               Multiple lines are not yet supported.
;               
; Use         : CDS_LINEFIT_WRAP,LAMBDA,INTENS,NLINES,LAB_LAMBDA,A,FIT_TYPE
;    
; Inputs      : (LAMBDA, INTENSITY) The spectrum at hand.
;
;               NLINES : The number of lines to fit. Most fitting routines
;                        only fit one line anyhow.
;               
;               LAB_LAMBDA : The lab wavelength(s) of the line(s) to be
;                            fitted.
;
;               A : Array of coefficients, with same interpretation as
;                   GAUSSFIT. For N lines, an A(6,N)-element vector.
;                   
;                   If this array is defined on entry, it is taken as
;                   initial guesses (for those fitting routines that
;                   do accept them).
;
;               FIT_TYPE : A string indicating the desired fitting procedure.
;                          Options are:
;
;                 'lstsqr': Fixed background  + Gaussian
;                 'curvefit': Fixed background  + Gaussian
;                 'amoeba_c': Fixed background  + Gaussian
;                 'gaussfit': Gaussian, background + linear + quadratic
;                 'cds_fit_wave': Gaussian(s), background + linear + quadr.
;
; Opt. Inputs : None.
;               
; Outputs     : A contains the results of the fit.
;               
; Opt. Outputs: None.
;               
; Keywords    : None.
;
; Calls       : AMOEBA_C, CDS_FIT_WAVE(), CLEANPLOT, CURVEFIT(), LSTSQR
;
; Common      : None.
;               
; Restrictions: Subject to restrictions of the fitting routines.
;               
; Side effects: Calls CLEANPLOT after calling cds_fit_wave.
;               
; Category    : Data_Analysis, Line_fit
;               
; Prev. Hist. : None.
;
; Written     : Stein Vidar H. Haugan, UiO, 13 June 1996
;               
; Modified    : Version 2, 10-Dec-1997, William Thompson, GSFC
;			Use AMOEBA_C instead of AMOEBA, to avoid conflict with
;			IDL/v5 routine of that name.
;
; Version     : 2, 10-Dec-1997
;-            

PRO cds_linefit_wrap,LAMBDA,INTENS,NLINES,LAB_LAMBDA,A,FIT_TYPE

  IF N_PARAMS() LT 6 THEN  $
     MESSAGE,"Use: CDS_LINEFIT,LAMBDA,INTENS,NLINES,LAB_LAMBDA,A,FIT_TYPE"
  
  default,a,[-1.0] ;; Signals not supplied
  
  ;; Parameter checks
  
  parcheck,lambda,    1,typ(/rea),1,'LAMBDA'
  parcheck,intens,    2,typ(/rea),1,'INTENSITY'
  parcheck,nlines,    3,typ(/nat),0,'NLINES',maxval=1,minval=1
  parcheck,lab_lambda,4,typ(/rea),0,'LAB_LAMBDA'
  parcheck,a,         5,typ(/rea),1,'A'
  parcheck,fit_type,  6,typ(/str),0,'FIT_TYPE'
  
  IF N_ELEMENTS(LAMBDA) NE N_ELEMENTS(INTENS) THEN  $
     MESSAGE,("LAMBDA and INTENSITY (parameter 1 and 2) must have same"+ $
              "number of elements")
  
  x = lambda
  
  IF N_ELEMENTS(a) EQ 1 AND a(0) EQ -1.0 THEN BEGIN
     ;; Initial guess: Gaussfit is actually quite good at it.
     IF STRLOWCASE(fit_type) EQ 'gaussfit' THEN BEGIN
        dummy = gaussfit(x,intens,a)
        RETURN
     END
     dummy = gaussfit(x,intens,a)
     a(3) = a(3) + a(4) * total(x)/N_ELEMENTS(x) $
        +a(5) * total(x^2)/N_ELEMENTS(x)
  END
  
  IF N_ELEMENTS(a) NE 3 + 3*NLINES THEN  $
     MESSAGE,("Fitting with "+trim(nlines)+" lines takes "+$
              trim(3+3*Nlines)+" fit parameters")
  
  ;; For those fitting routines with fixed backgrounds
  y = intens - a(3)
  
  catch,error
  IF error NE 0 THEN BEGIN
     catch,/cancel
     a = dblarr(6)
     PRINT,"CAUGHT error in fitting routine, "+fit_type
     RETURN
  END
  
  ;; For those routines with only intensity, shift and width
  aa = a(0:2)
  
  CASE STRLOWCASE(fit_type) OF 
     
  'lstsqr':BEGIN
     lstsqr,x,y,'func_f',aa,/noprint
     a(0:2) = aa
     a(2) = a(2)/SQRT(2.0) ;; Fits into exp(-(x-x0)^2/2w^2)
     ;; A(3) has background
     a(4:*) = 0.0
     ENDCASE 
     
  'curvefit':BEGIN
     w = REPLICATE(1.0,N_ELEMENTS(x))
     res = curvefit(x,y,w,aa,sigma,function_name='functn')
     a(0:2) = aa
     a(2) = a(2)/SQRT(2.0) ;; Fits into exp(-(x-x0)^2/2w^2)
     a(4:*) = 0.0
     ENDCASE
     
  'amoeba_c':BEGIN
     temp = x(0)-0.001 &  aa(1) = aa(1)-temp
     ;; Trick to get it going
     amoeba_c,x-temp,y,'func_f',aa,/noprint,MAX=100
     a(0:2) = aa
     a(1) = a(1)+temp
     a(2) = a(2)/SQRT(2.0)
     a(4:*) = 0.0
     ENDCASE
     
  'gaussfit':BEGIN
     result = gaussfit(x,intens,a)
     ENDCASE
     
  'cds_fit_wave':BEGIN
     res = cds_fit_wave({w:x,flux:intens},a(1),width=N_ELEMENTS(x),out=out, $
                        /quiet,/noplot)
     IF N_ELEMENTS(out) EQ 0 THEN out = dblarr(6)
     nout = N_ELEMENTS(out)
     IF nout GT 6 THEN BEGIN
        PRINT,out
        out = [out(0:2),out(nout-3:*)] ;; First gaussian + bgnd,lin,quadr.
     END
     ;; X Reference shifted from a(1) to zero
     a = out
     a(3) = a(3) + a(5)*a(1)^2 - a(1)*a(4)
     a(4) = a(4) - 2*a(1)*a(5)
     cleanplot ;; ! Necessary, I'm afraid.
     ENDCASE
     
  ELSE:BEGIN
     PRINT,"Unknown fit"
     ENDCASE 
  END
  
END
