;+
; Project     : SOHO - CDS     
;                   
; Name        : CDS_PLOT_FIT
;               
; Purpose     : To display the results from various line fitting routines for
;               CDS spectra.
;               
; Explanation : Routines like CDS_AUTOFIT can return a set of Gaussian fitted
;               components and errors and also continuum estimates. This
;               routine can either simply display the CDS input spectrum with
;               a simple indication of where the lines fitted are, or if the
;               KELLY keyword was used, can indicate the closest matches to
;               known lines (default lambda differences or user defined by
;               WDIFF keyword), and/or indicate the fitted gaussian profile
;               and fitted intensity if the FILL keyword is supplied. The
;               defaults is to display the entire spectrum. However, a WRANGE
;               keyword can be used to plot a restricted wavelength interval.
;               
; Use         : IDL> cds_fit_plot, sp, [ results = gfit, /fill, /kelly,      $
;                                  wdiff = 0.25, wrange = [wave1,wave2],     $
;                                  /logplot ]
;    
; Inputs      : sp   -  a structure containing the input spectrum (wavelengths
;                       and fluxes).
;               
; Opt. Inputs : None.
;
; Outputs     : None.
;               
; Opt. Outputs: None
;               
; Keywords    : RESULTS - a set of gaussian parameters etc. as returned by
;                         eg. CDS_AUTOFIT.
;               KELLY -   indicate full Kelly lines in spectral range within
;                         certain range (default is 1.75 pixels, or user
;                         defined by WDIFF value).
;               BLUEBOOK - indicate CDS Bluebook lines in plotted range.
;
;               FILL - (only valid if RESULTS supplied!) - show fitted gaussians
;                         and continuum and print line intensities.
;               WDIFF - used with the KELLY keyword to select closest lines
;                       in the Kelly CDS list (i.e. within +/- WDIFF Angstroms).
;                       [Default value is 1.75 wavelength pixels].
;               WRANGE - a range of wavelengths over which to display results.
;                        eg WRANGE = [ 333., 336., 355., 360. ] would produce
;                        two plots (333-336 and 355-360).
;               LOGPLOT - use log scaling on the y-axis 
;               ALL -    override WDIFF and plot all KELLY or BLUEBOOK lines.
;
; Calls       : CDS_SLINE - called if KELLY or BLUEBOOK set but no results 
;                           file is given (KELLY would need to always call).
;
; Common      : None
;               
; Restrictions: None
;               
; Side effects: None
;               
; Category    : Data_Analysis
;               
; Prev. Hist. : None
;
; Written     : B J Kellett 2/12/95.
;               
; Modified    : 
;
; Version     : Version 1.0, 2-Dec-95
;-            
;
 PRO cds_plot_fit, sp, results = results, fill = fill, kelly = kelly,        $
                       wdiff = wdiff, wrange = wrange, logplot = logplot,    $
                       bluebook = bluebook, all = all
;
 cwave = sp.w
 cspec = sp.flux
;
 !p.font = -1
 !p.multi = [ 0, 1, 3 ]
 !x.ticklen = 0.0325
 !y.ticklen = 0.0125
 !x.charsize = 1.375
 !y.charsize = 1.375
;
 !p.title = '!17'
 !x.title = 'Wavelength (Ang)'
;
 imode = -1
 ifill = 0
 ilog = 0
 ikelly = 0
 irange = 1
;
 if keyword_set ( results ) then imode = 0
;
 if keyword_set ( bluebook ) then begin
     ikelly = 1
     if ( imode lt 0 ) then begin
         print, ' '
         print, '     Getting Blue Book line list ... '
;
         wmin = min ( sp.w ) - 0.5
         wmax = max ( sp.w ) + 0.5
;
         lines = cds_sline ( wmin, wmax, /cds )
;
         ncds = n_elements ( lines ) / 3
;
         kwlam = fltarr ( ncds )
         kelem = strarr ( ncds )
;
         for i = 0, ncds - 1 do begin
             kwlam ( i ) = float ( lines ( 2, i ) )
             kelem ( i ) = strcompress ( lines ( 0, i ) + lines ( 1, i ) )
         endfor
;
         nkelly = n_elements ( kwlam )
         glfit = fltarr ( 10 ) + 250.
         gbfit = fltarr ( 10 ) + min ( cspec )
         ghfit = fltarr ( 10 )
     endif else begin
         kwlam = results.kw
         kelem = results.kelem
         nkelly = n_elements ( kwlam )
     endelse
;
 endif
;
 if keyword_set ( kelly ) then begin
     print, ' '
     print, '     Getting Kelly line list ... '
;
     wmin = min ( sp.w ) - 0.5
     wmax = max ( sp.w ) + 0.5
;
     lines = cds_sline ( wmin, wmax )
;
     ncds = n_elements ( lines ) / 3
;
     kwlam = fltarr ( ncds )
     kelem = strarr ( ncds )
;
     for i = 0, ncds - 1 do begin
         kwlam ( i ) = float ( lines ( 2, i ) )
         kelem ( i ) = strcompress ( lines ( 0, i ) + lines ( 1, i ) )
     endfor
;
     if ( ( wmin lt 605.0 ) and ( wmax gt 609.0 ) ) then begin
         wel_heii = 'HeII(2)'
         ww_heii = 2.0 * 303.78
;
         kwlam = [ kwlam, ww_heii ]
         kelem = [ kelem, wel_heii ]
     endif
     nkelly = n_elements ( kwlam )
     ikelly = 1
     glfit = fltarr ( 10 ) + 250.
     gbfit = fltarr ( 10 ) + min ( cspec )
     ghfit = fltarr ( 10 )
 endif

 if keyword_set ( fill ) then begin
     if ( imode ge 0 ) then ifill = 1 else begin
         print, ' '
         print, '  CDS_FIT_PLOT: Sorry! Cannot FILL when no results are supplied!'
     endelse
 endif
;
 if keyword_set ( logplot ) then ilog = 1
;
 nplot = 6
 nowait = 1
 kfold = 6
 nprint = 0
 ysf = 1.0
 tsz = 1.25
 tcol = 0
 ipos = -1
 if ( !d.name eq 'PS' ) then tcol = 255
;
 wl = [ 304., 316.5, 329., 341.5, 354., 366.5 ]
 wh = wl + 14.
;
 if keyword_set ( results ) then begin
     glfit = results.w 
     gwfit = results.sig 
     ghfit = results.h 
     gxfit = results.x 
     gbfit = results.bgnd 
     gcfit = results.cf 
     elfit = results.werr 
     ewfit = results.sigerr 
     ehfit = results.herr 
     ecfit = results.cferr 
;
     stags = tag_names ( results )
;
     ipos = max ( strpos ( stags, 'WRANGE' ) )
;
     if ( ipos ne -1 ) then wrange = results.wrange
 endif
;
 nfit = n_elements ( glfit )
;
 if ( n_elements ( glfit ) eq 1 ) then begin
     glfit = [ glfit ( 0 ), glfit ( 0 ) ]
     ghfit = [ ghfit ( 0 ), ghfit ( 0 ) ]
     gbfit = [ gbfit ( 0 ), gbfit ( 0 ) ]
 endif
;
 if ( max ( cwave ) gt 400. ) then begin
     wl = [ 512.5, 532.25, 552.0, 571.75, 591.5, 611.25 ]
     wh = wl + 21.75
 endif
;
 if ( n_elements ( cwave ) lt 1024 ) then begin
     !p.multi = 0
     nplot = 1
     irange = 0
     wl = [ cwave ( 0 ), cwave ( 0 ) ]
     wh = [ max ( cwave ), max ( cwave ) ]
     ysf = 0.525
     tsz = 1.175
 endif
;
 ymin0 = min ( cspec )
 ymax0 = max ( cspec )
;
 if keyword_set ( wrange ) then begin
     !p.multi = 0
     nplot = 1
     irange = 0
;
     wl = [ wrange ( 0 ), wrange ( 0 ) ]
     wh = [ wrange ( 1 ), wrange ( 1 ) ]
     ysf = 0.525
     tsz = 1.175
;
     iw = where ( cwave gt wrange ( 0 ) )
     il = iw ( 0 )
     if ( il lt 1 ) then il = 0
;
     iw = where ( cwave gt wrange ( 1 ) )
     ih = iw ( 0 ) - 1
     if ( ih lt 1 ) then ih = n_elements ( cwave ) - 1
;
     ymin0 = min ( cspec ( il:ih ) )
     ymax0 = max ( cspec ( il:ih ) )
;
     if ( n_elements ( wrange ) gt 2 ) then begin
         nplot = n_elements ( wrange ) / 2
;
         if ( nplot le 12 ) then !p.multi = [ 0, 4, 3 ]
         if ( nplot eq 9 ) then !p.multi = [ 0, 3, 3 ]
         if ( nplot le 8 ) then !p.multi = [ 0, 4, 2 ]
         if ( nplot le 6 ) then !p.multi = [ 0, 3, 2 ]
         if ( nplot eq 4 ) then !p.multi = [ 0, 2, 2 ]
         if ( nplot eq 3 ) then !p.multi = [ 0, 3, 1 ]
         if ( nplot eq 2 ) then !p.multi = [ 0, 2, 1 ]
         nowait = 0
;
         iind = 2 * indgen ( nplot )
         wl = wrange ( iind )
         wh = wrange ( iind+1 )
         ysf = 0.475
         if ( nplot le 12 ) then tsz = 1.075
         if ( nplot eq 9 ) then tsz = 1.075
         if ( nplot le 8 ) then tsz = 1.075
         if ( nplot le 6 ) then tsz = 1.25
         if ( nplot eq 4 ) then tsz = 0.975
         if ( nplot eq 3 ) then tsz = 1.25
         if ( nplot eq 2 ) then tsz = 0.975
;
         if ( nplot lt 6 ) then !y.ticklen = 0.0225
;
         if ( nplot le 3 ) then begin
             !x.ticklen = 0.0125
             !y.ticklen = 0.0225
         endif
;
         if ( nplot gt 6 ) then begin
             !x.ticklen = 0.0325
             !y.ticklen = 0.0325
         endif
;
         ymx = fltarr ( nplot )
         ymn = ymx
;
         for i = 0, nplot - 1 do begin
             iw = where ( cwave gt wrange ( iind ( i ) ) )
             il = iw ( 0 )
             if ( il lt 1 ) then il = 0
;
             iw = where ( cwave gt wrange ( iind ( i ) + 1 ) )
             ih = iw ( 0 ) - 1
             if ( ih lt 1 ) then ih = n_elements ( cwave ) - 1
;
             ymn ( i ) = min ( cspec ( il:ih ) )
             ymx ( i ) = max ( cspec ( il:ih ) )
         endfor
;
         ymin0 = min ( ymn )
         ymax0 = max ( ymx )
     endif
;
 endif
;
 if ( ilog eq 1 ) then begin
     dy = 0.1075 * ( alog10 ( ymax0 ) - alog10 ( ymin0 ) )
;
     if ( nplot le 3 ) then dy = 0.25 * dy
;
     ymin = 10.0 ^ ( alog10 ( ymin0 ) - dy )
     ymax = 10.0 ^ ( alog10 ( ymax0 ) + 2.5 * dy )
;
     ymin0 = ymin0 - 0.675 * dy
;
     if ( nplot le 3 ) then dy = 4.75 * dy
;
 endif else begin
     dy = 0.1075 * ( ymax0 - ymin0 )
;
     ymin = ymin0 - dy
     ymax = ymax0 + 1.25 * dy 
;
     ymin0 = ymin0 - 0.675 * dy
 endelse
;
 xx = [ -0.25, 0.0, 0.25, 0.0, 0.0 ]
 yy = [ 0.50, 0.125, 0.50, 0.125, 1.5 ]
;
 usersym, xx, yy
;
 dlam = 1.75 * ( cwave ( 1 ) - cwave ( 0 ) )
;
 if keyword_set ( wdiff ) then dlam = wdiff
;
 if keyword_set ( all ) then begin
     dlam = 500.
 endif
;
 if ( !d.name eq 'PS' ) then tsz = 0.875 * tsz
;
 ndy = 0
 csiz = 1.175 * tsz
 if ( nplot eq 3 ) then csiz = 0.875 * csiz
 if ( nplot gt 4 ) then csiz = 0.675 * csiz
;
 if ( !d.name eq 'PS' ) then mdy = 4
 if ( !d.name eq 'PS' ) then dy = 0.0625 * ( ymax0 - ymin0 )
 if ( ( ilog eq 1 ) and ( !d.name eq 'PS' ) ) then dy = 0.0625 *              $
                             ( alog10 ( ymax0 ) - alog10 ( ymin0 ) )
;
 gwid = 18
 npix = 2 * gwid + 1
 nxp = 501
;
 dx0 = cwave ( 1 ) - cwave ( 0 )
;
 !p.font = -1
 !p.charsize = 1.325 * tsz
 !x.charsize = 1.075 * tsz
 !y.charsize = 1.075 * tsz
;
 if ( ilog eq 1 ) then begin
     ndec = fix ( alog10 ( ymax ) - alog10 ( ymin ) - 0.275 )
;
     ymn = ymin
     ymx = ymax
;
     if ( ndec lt 1 ) then begin
;
         ymn = 10.0 * ( fix ( ymin / 10. ) )
         ymx = 10.0 * ( fix ( ( ymax / 10.0 ) + 0.625 ) )
;
         np = fix ( ( ymx - ymn ) / 10.0 ) + 1
;
         yt = ymn + findgen ( np + 1 ) * 10.0
         yv = strarr ( np + 1 )
;
         if ( alog10 ( ymx ) gt 3.0 ) then begin
             for j = 0, np - 1 do yv ( j ) = string ( format = '( f7.1 )',   $
                                                      yt ( j ) )
         endif else begin
             for j = 0, np - 1 do yv ( j ) = string ( format = '( f5.1 )',   $
                                                      yt ( j ) )
         endelse
;
;;;         yt = alog10 ( yt )
;
     endif
 endif
;
 for i = 0, nplot - 1 do begin
;
     if keyword_set ( logplot ) then begin
         if ( ndec ge 1 ) then begin
             plot_io, cwave, cspec, xrange = [ wl ( i ), wh ( i ) ],          $
                      xstyle = 1, yrange = [ ymin, ymax ], ystyle = 1,        $
                      /nodata
         endif
         if ( ndec lt 1 ) then begin
             plot_io, cwave, cspec, xrange = [ wl ( i ), wh ( i ) ],          $
                      xstyle = 1, yrange = [ ymin, ymax ], ystyle = 1,        $
                      ytickv = yt, ytickn = yv, yticks = np - 1, /nodata
         endif
     endif else                                                               $
         plot, cwave, cspec, xrange = [ wl ( i ), wh ( i ) ], xstyle = 1,     $
               yrange = [ ymin, ymax ], ystyle = 1, /nodata
;
     dyt = 0.025 * ( !y.crange ( 1 ) - !y.crange ( 0 ) )
;
     yl_range = !y.crange
     if ( ilog eq 1 ) then begin
         yl_range ( 0 ) = 10.0 ^ ( yl_range ( 0 ) + dyt )
         yl_range ( 1 ) = 10.0 ^ ( yl_range ( 1 ) - dyt )
     endif else begin
         yl_range ( 0 ) = yl_range ( 0 ) + dyt
         yl_range ( 1 ) = yl_range ( 1 ) - dyt
     endelse
;
     if ( ( imode lt 0 ) and ( ikelly eq 0 ) ) then goto, nxtplt
     if ( imode lt 0 ) then goto, ckelly
;
     oplot, glfit, ghfit + gbfit + 0.25 * dy, psym = 8, symsize = 2.75 * tsz, $
            thick = 4.0, color = tcol
     oplot, glfit, ghfit + gbfit + 0.25 * dy, psym = 8, symsize = 2.75 * tsz
;
     if ( ifill eq 1 ) then begin
;
         for j = 0, nfit - 1 do begin
             if ( glfit ( j ) lt wl ( i ) ) then goto, fcont
             if ( glfit ( j ) gt wh ( i ) ) then goto, pcont
;
             gcf = [ ghfit ( j ), glfit ( j ), gwfit ( j ) ]
             cf = gcfit ( 0:2, j )
;
             gpar = [ gcf, cf ]
             gperr = [ ehfit ( j ), elfit ( j ), ewfit ( j ), ecfit ( 0:2, j ) ]
;
             wxm = glfit ( j ) - gxfit ( j )
;
             gcf ( 1 ) = gxfit ( j )
;
             dw = cwave - wxm
             ww = min ( abs ( dw ) )
;
             iwl = !c - gwid
             iwh = iwl + npix - 1
             if ( iwl lt 0 ) then iwl = 0
             if ( iwh ge n_elements ( cwave ) ) then iwh = n_elements ( cwave )$
                  - 1
;
             iwm = !c
;
             x = cwave ( iwl:iwh ) - wxm
             y = cspec ( iwl:iwh )
;
             npx = n_elements ( x )
;
             dx = ( x ( npx-1 ) - x ( 0 ) ) / float ( nxp - 1 )
;
             xx = x ( 0 ) + dx * findgen ( nxp )
             wx = xx + wxm
;
             bgnd = poly ( xx, cf )
;
             gg = get_gauss ( xx, gcf ( 1 ), gcf ( 0 ), gcf ( 2 ) )
;
             oplot, wx, bgnd + gg, thick = 4.0, color = 148
;
             for k = 0, nxp-1 do oplot, [ wx ( k ), wx ( k ) ],               $
                     [ bgnd ( k ), bgnd ( k ) + gg ( k ) ], color = 207
;
             oplot, wx, bgnd, thick = 2.0, color = 96
;
             gfit_error, x, y, gpar, gperr, wxm, gfint, gferr, gcint, gcerr,  $
                         /quiet
;
             tint = string ( format = '( f9.3,"+/-",f7.3 )', gfint, gferr )
;
             nyoff = 0.5
             if ( j eq ( 2 * ( j / 2 ) ) ) then nyoff = 1.5
;
             if ( ilog eq 1 ) then begin
                 wy = 10.0 ^ ( alog10 ( ghfit ( j ) + gbfit ( j ) ) +         $
                               nyoff * dyt )
                 if ( ikelly ge 1 ) then wy = 10.0 ^ ( !y.crange ( 0 ) +      $
                                                       nyoff * dyt )
             endif else begin
                 wy = ( ghfit ( j ) + gbfit ( j ) ) + nyoff * dyt
                 if ( ikelly ge 1 ) then wy = !y.crange ( 0 ) + nyoff * dyt
             endelse
;
             xyouts, glfit ( j ), wy, tint, alignment = 0.575, size = 0.875 *  $
                     csiz, charthick = 4.0, color = tcol
             xyouts, glfit ( j ), wy, tint, alignment = 0.575, size = 0.875 *  $
                     csiz
;
 fcont:
         endfor
 pcont:
;
         nw = n_elements ( cwave )
         cw = fltarr ( 2 * nw )
         csp = cw
         iw = 2 * indgen ( nw )
;
         dxpix = 0.5 * ( cwave ( 1 ) - cwave ( 0 ) )
;
         cw ( iw ) = cwave - dxpix
         cw ( iw+1 ) = cwave + dxpix
         csp ( iw ) = cspec
         csp ( iw + 1 ) = cspec
;
         if ( ipos ne -1 ) then cw = cw - dxpix
;
         oplot, cw, csp, thick = 3, color = tcol
         oplot, cw, csp
;
     endif
;
 ckelly:
;
     if ( ikelly eq 1 ) then begin
         ipflag = 0
         for j = 0, nfit - 1 do begin
;
             if ( imode eq -1 ) then goto, jcont
;
             if ( glfit ( j ) lt wl ( i ) ) then goto, hcont
             if ( glfit ( j ) gt wh ( i ) ) then goto, gcont
;
 jcont:
             if ( ( ipflag ge 1 ) and ( dlam gt 50. ) ) then goto, gcont
;
             dw = kwlam - glfit ( j )
;
             wz = where ( abs ( dw ) lt dlam )
;
             if ( wz ( 0 ) eq -1 ) then goto, hcont
;
             nwz = n_elements ( wz )
;
             for k = 0, nwz - 1 do begin
                 if ( kwlam ( wz ( k ) ) lt wl ( i ) ) then goto, icont
                 if ( kwlam ( wz ( k ) )  gt wh ( i ) ) then goto, hcont
;
                 oplot, [ kwlam ( wz ( k ) ), kwlam ( wz ( k ) ) ], yl_range, $
                        linestyle = 1
;
                 wy0 = ( gbfit ( j ) + ghfit ( j ) )
                 if ( wy0 lt ymin0 ) then wy0 = ymin0
;
                 if ( ilog eq 1 ) then                                        $
                     wy = 10.0 ^ ( alog10 ( wy0 ) + 1.675 * dy )              $
                 else wy = wy0 + 1.175 * dy
;
                 if ( wy gt 0.375 * ymax0 ) then begin
                     wy = gbfit ( j )
                     if ( ilog eq 1 ) then                                    $
                         wy = 10.0 ^ ( alog10 ( wy ) + 0.325 * ysf * dy )     $
                     else wy = wy + 0.325 * ysf * dy
                 endif
;
                 rdy = 1.0 + float ( ndy ) / 2
;
                 if ( ( nprint mod 3 ) eq ( nprint mod 6 ) ) then rdy = rdy   $
                                                                      - 0.5
                 if ( irange ) then rdy = rdy / 2.333
;
                 if ( ilog eq 1 ) then                                        $
                     wy = 10.0 ^ ( alog10 ( wy ) + 3.875 * ysf * rdy * dy )   $
                 else wy = wy + 5.875 * ysf * rdy * dy
;
                 ndy = ndy + 2
                 if ( ndy ge kfold ) then ndy = 0
;
                 xyouts, kwlam ( wz ( k ) ), wy, kelem ( wz ( k ) ),         $
                     size = csiz, orientation = 90., charthick = 4.0,        $
                     color = tcol
                 xyouts, kwlam ( wz ( k ) ), wy, kelem ( wz ( k ) ),         $
                     size = csiz, orientation = 90.
;
                 ipflag = 1
                 nprint = nprint + 1
 icont:
             endfor
 hcont:
         endfor
 gcont:
     endif
;
 nxtplt:
;
     if ( not ifill ) then begin
;
         nw = n_elements ( cwave )
         cw = fltarr ( 2 * nw )
         csp = cw
         iw = 2 * indgen ( nw )
;
         dxpix = 0.5 * ( cwave ( 1 ) - cwave ( 0 ) )
;
         cw ( iw ) = cwave - dxpix
         cw ( iw+1 ) = cwave + dxpix
         csp ( iw ) = cspec
         csp ( iw + 1 ) = cspec
;
         if ( ipos ne -1 ) then cw = cw - dxpix
;
         oplot, cw, csp, thick = 3, color = tcol
         oplot, cw, csp
;
     endif
;
     if ( ( nowait ) and ( !d.name ne 'PS' ) and ( i eq 2 ) ) then wait, 5.0
 endfor
;
 !p.position = 0
 !p.multi = 0
 !y.ticklen = 0.020
 !x.charsize = 1.0
 !y.charsize = 1.0
;
 end
; 
