;+
; Project     : SOHO - CDS     
;                   
; Name        : CDS_VEL_SLICE
;               
; Purpose     : To display a single spectrum from a data cube with user
;               selectable inputs.
;               
; Explanation : The procedure takes a data structure (or fits file name) and
;               lists a summary of the data available in it. It then offers
;               the user a variety of options in order to specify the exact
;               details of the spectrum required (eg. band or window, raster,
;               det-y, and wavelength or pixel range).
;               
; Use         : IDL> plot_spec,file_str
;    
; Inputs      : file_str   -  either:-   the name of a CDS FITS file
;                                 or:-   the name of a CDS data structure
;                                (a data structure is the faster option!)
;               
; Opt. Inputs : None
;               
; Outputs     : None (just plots a spectrum in the default window)
;               
; Opt. Outputs: None
;               
; Keywords    : None
;
; Calls       : GT_WLIMITS
;
; Common      : None
;               
; Restrictions: None
;               
; Side effects: None
;               
; Category    : Data_handling, FITS
;               
; Prev. Hist. : None
;
; Written     : B J Kellett, 15-Nov-95.
;               
; Modified    : 
;
; Version     : Version 1, 15-Nov-95.
;-            

 PRO cds_vel_plot, results

 fit_w = results.wfit
 velocity = results.vel
 fit_i = results.fint
 fit_dw = results.werr
 vel_err = results.verr
 fit_di = results.ferr
 sol_y = results.ysol
 sol_x = results.xsol
 tai = results.tai
 ut = results.ut
 dut = results.dut
 wave0 = results.w0
 line_id = results.line_id

 xarr = sol_y
 dxarr = 0. * sol_y + ( 0.5 / 60. )
 pmode = 0

 if ( max ( ut ) ne min ( ut ) ) then begin
     xarr = ut
     dxarr = dut / 3600.0

     ttxt = anytim2cal ( tai2utc ( tai ( 0 ) ) )
     xtxt = 'UT Hours after ' + strmid ( ttxt, 0, 9 )
     pmode = 1
 endif

 !p.font = -1
 !p.charsize = 1.175
 !y.ticklen = 0.01175

 !p.charthick = 1.0

 if ( !d.name eq 'PS' ) then !p.charthick = 2.0

 !x.title = ' '
 !y.title = 'Wavelength (A)'
 !p.title = string ( format =                                                 $
     '( "!17Line Identification: ", a, " @ !7k!17", F7.3 )', line_id, wave0 )

 !p.position = [ 0.125, 0.675, 0.95, 0.925 ]
 if ( pmode ) then !p.position = [ 0.125, 0.70, 0.95, 0.9375 ]
 !x.tickname = replicate ( ' ', 10 )

 xmin = min ( xarr - dxarr )
 xmax = max ( xarr + dxarr )
 dx = 0.0175 * ( xmax - xmin )
 xmin = xmin - dx
 xmax = xmax + dx

 ymin = min ( fit_w - fit_dw )
 ymax = max ( fit_w + fit_dw )
 dy = 0.0875 * ( ymax - ymin )
 ymin = ymin - dy
 ymax = ymax + dy

 plot, xarr, fit_w, xrange = [ xmin, xmax ], xstyle = 1,                     $
       yrange = [ ymin, ymax ], ystyle = 1, /nodata
 oplot, !x.crange, [ wave0, wave0 ], linestyle = 1

 bj_err, xarr, dxarr, fit_w, fit_dw

 !p.title = ' '
 !y.title = 'Velocity (km/s)'
 !p.position = [ 0.125, 0.375, 0.95, 0.675 ]
 if ( pmode ) then !p.position = [ 0.125, 0.4375, 0.95, 0.70 ]

 ymin = min ( velocity - vel_err )
 ymax = max ( velocity + vel_err )
 dy = 0.0875 * ( ymax - ymin )
 ymin = ymin - dy
 ymax = ymax + dy

 plot, xarr, velocity, xrange = [ xmin, xmax ], xstyle = 1,                   $
       yrange = [ ymin, ymax ], ystyle = 1, /nodata, /noerase
 oplot, !x.crange, [ 0.0, 0.0 ], linestyle = 1

 bj_err, xarr, dxarr, velocity, vel_err

 !x.title = 'Solar Y (Arc-Minutes)'

 if ( pmode eq 1 ) then !x.title = xtxt

 !y.title = 'Fitted Line Int.'
 !p.position = [ 0.125, 0.125, 0.95, 0.375 ]
 if ( pmode ) then !p.position = [ 0.125, 0.1875, 0.95, 0.4375 ]
 !x.tickname = replicate ( '', 10 )

 ymin = min ( fit_i - fit_di )
 ymax = max ( fit_i + fit_di )
 dy = 0.0875 * ( ymax - ymin )
 ymin = ymin - dy
 ymax = ymax + dy

 plot, xarr, fit_i, xrange = [ xmin, xmax ], xstyle = 1,                      $
       yrange = [ ymin, ymax ], ystyle = 1, /nodata, /noerase,                $
       xtick_get = xtic

 bj_err, xarr, dxarr, fit_i, fit_di

 if ( pmode ) then begin
     !p.position = [ 0.125, 0.0875, 0.95, 0.125 ]
     !x.title = 'Solar X (Arc-Minutes)
     xx = spline ( xarr, sol_x, xtic )
     np = n_elements ( xx )
     xax = strarr ( np )
     for i = 0, np - 1 do xax ( i ) = string ( format = '(f7.2)', xx ( i ) )

     plot, xarr, sol_x, xrange = [ xmin, xmax ], xstyle = 9, ystyle = 13,     $
           xtickv = xtic, xtickn = xax, /noerase, xticklen = 0.275, /nodata
 endif

 !x.ticklen = 0.020
 !y.ticklen = 0.020
 !p.multi = 0
 !p.position = 0
 !x.tickname = replicate ( '', 10 )

 end

