;+
; Project     : SOHO - CDS     
;                   
; Name        : CDS_VEL_SLICE ()
;               
; Purpose     : To display a single spectrum from a data cube with user
;               selectable inputs.
;               
; Explanation : The procedure takes a data structure (or fits file name) and
;               lists a summary of the data available in it. It then offers
;               the user a variety of options in order to specify the exact
;               details of the spectrum required (eg. band or window, raster,
;               det-y, and wavelength or pixel range).
;               
; Use         : IDL> plot_spec,file_str
;    
; Inputs      : file_str   -  either:-   the name of a CDS FITS file
;                                 or:-   the name of a CDS data structure
;                                (a data structure is the faster option!)
;               
; Opt. Inputs : None
;               
; Outputs     : None (just plots a spectrum in the default window)
;               
; Opt. Outputs: None
;               
; Keywords    : None
;
; Calls       : GT_WLIMITS
;
; Common      : None
;               
; Restrictions: None
;               
; Side effects: None
;               
; Category    : Data_handling, FITS
;               
; Prev. Hist. : None
;
; Written     : B J Kellett, 15-Nov-95.
;               
; Modified    : 
;
; Version     : Version 1, 15-Nov-95.
;-            

 FUNCTION cds_vel_slice, input, hard = hard, debug = debug, solar_y = solar_y, $
                         window = window, dband = dband, exposure = exposure,  $
                         yrange = yrange, wcenter = wcenter, dwidth = dwidth,  $
                         recenter = recenter, batch = batch

;
;  need one parameter
;

if n_params() eq 0 then begin
    print,'USE:   IDL> plot_spec,file or structure
    return, 0
endif

;
;  what was the input?
;
case datatype(input,1) of
   'Structure': begin
                   qlmgr, input, valid
                   if valid then begin
                      cds_s = input
                   endif else begin
                      print,'CDS_VEL_SLICE: Structure supplied is non-standard'
                      return, 0
                   endelse
                end
;
;  allow user to have own version of FITS files
;
      'String': begin
                   tfile = ''
                   if  file_exist(input) then begin
                      tfile = input
                   endif else begin
                     if file_exist(concat_dir('$CDS_CAL_FITS',input)) then begin
                        tfile = concat_dir('$CDS_CAL_FITS',input)
                     endif
                   endelse

;
;  FITS file doesn't exist in user or std directory
;
                   if tfile eq '' then begin
                      print,'CDS_VEL_SLICE:  Input file does not exist.'
                      return, 0
                   endif

                cds_s = readcdsfits ( tfile )

                end
          else: begin
                   print,'Input must be a file name or a CDS data structure.'
                   return, 0
                end
endcase

tmode = 0
pmode = 0

if keyword_set ( debug ) then pmode = 1

ibatch = 0
if keyword_set ( batch ) then ibatch = 1

tx = gt_wlimits ( cds_s, /q )
tw = gt_wlimits ( cds_s, /w, /q )

tx0 = tx

ins = cds_s.header.detector

narray = cds_s.detdesc
nexp =  narray(0).ixstop(1)
nypix =  narray(0).ixstop(2)

ysolar = narray(0).origin(2) + findgen(nypix+1)*narray(0).spacing(2)

nbands = 4
if ( ins eq 'NIS' ) then nbands = 2

nwinds = n_elements ( tx ) / 4

wbands = intarr ( nwinds )

;
;  Print out summary listing of data
;

if ( ibatch ) then goto, zcont

print, ' '
print, '                  CDS_VEL_SLICE - Data Summary.'
print, ' '
print, '    Instrument:        ', ins
print, '    Windows:', nwinds
print, '    Bands:      ', nbands
print, '    Exposures:   ', nexp + 1
print, ' '
print, '      NB: Select Exposure -1 to enable slice in Time direction.'
print, ' '

h1 = ' Window     Label     Band   Det-X Pixels   Det-Y Pixels      Wavelengths'
h2 = ' Window     Label     Band   Det-X Pixels    Solar-Y "        Wavelengths'

if keyword_set ( solar_y ) then begin 
    print, h2
    tx ( 2, 0:nwinds-1 ) = ysolar ( 0 )
    tx ( 3, 0:nwinds-1 ) = ysolar ( nypix )
endif else begin
    print, h1
    tx ( 3, 0:nwinds-1 ) = tx ( 3, 0:nwinds-1 ) - tx ( 2, 0:nwinds-1 )
    tx ( 2, 0:nwinds-1 ) = 0.0
endelse

for i = 0, nwinds - 1 do begin
    if ( tx0 ( 2, i ) lt 512 ) then iband = 2 else iband = 1
    txt = string ( format =                                                   $
                  '( 3x,i2,5x,a,5x,i1,4x,2(i4," - ",i4,4x),f7.3," - ",f7.3)', $
                   i,narray(i).label,iband,tx(0,i),tx(1,i),tx(2,i),tx(3,i),   $
                   tw(0,i),tw(1,i)  )
    print, txt
    wbands ( i ) = iband
endfor

print, ' '

zcont:

iwin = 0
itix = 0
iyix = tx ( 2, iwin )
iband = 1
ipixel = 0
nysp = 1

idefs = [ iwin, iband, itix, iyix, ipixel ]
imins = [ 0, 0, 0, tx ( 2, iwin ), 0 ]
imaxs = [ nwinds - 1, nbands, nexp, tx ( 3, iwin ), 0 ]

tdefs = [ 'Window  ', 'Band    ', 'Exposure', 'Det-Y   ',                     $
          '(P) Pixels or (W) Wavelengths' ]

wdefs = [ 0, 0, 0, 0, 0 ]
xdefs = [ 0, 0, 0, 0, 0 ]

if keyword_set ( window ) then wdefs ( 0 ) = 1
if keyword_set ( dband ) then wdefs ( 1 ) = 1
if keyword_set ( exposure ) then wdefs ( 2 ) = 1
if keyword_set ( yrange ) then wdefs ( 3 ) = 1

if keyword_set ( window ) then begin
    if ( window lt 0 ) then window = 0
    xdefs ( 0 ) = window
endif

if keyword_set ( dband ) then xdefs ( 1 ) = dband

if keyword_set ( exposure ) then begin
    if ( exposure lt -1 ) then exposure = 0
    xdefs ( 2 ) = exposure
endif

if keyword_set ( solar_y ) then begin
    tdefs ( 3 ) = 'Solar-Y '
endif

isel = 0 * idefs

ival = ''

;
;  Get input values for spectrum selection
;

for i = 0, 2 do begin

    if ( ( i eq 1 ) and ( isel ( 0 ) ge 0 ) ) then goto, fcont

    stxt = string ( format = '( 8x,"Select ", a, " [D:", i4, "] " )',         $
                    tdefs ( i ), idefs ( i ) )
    if ( i eq 4 ) then stxt = string ( format =                               $
                               '( 8x,"Select ", a, " [D: Wavelengths] " )',   $
                                       tdefs ( i ) )

 scont:

    if ( wdefs ( i ) eq 0 ) then begin
        print, format = '($,a)', stxt
        read, ival
    endif else ival = xdefs ( i )

;
;  if no value input, set to default
;
    if ( strlen ( ival ) eq 0 ) then val = idefs ( i ) else               $
         val = fix ( ival )

;
;  Trap above max. limit inputs - re-request the input value !
;
    if ( val gt imaxs ( i ) ) then begin
        print, ' '
        print, '     *** ERROR!  Maximum No. of ', tdefs ( i ), ' is ',       $
               imaxs ( i ) 
        print, ' '
        goto, scont
    endif

    if ( ( i eq 0 ) and ( val ge 0 ) ) then begin
        wy = tx ( 2, val )
        if ( wy lt 512 ) then iband = 2 else iband = 1
        isel ( 1 ) = iband
        idefs ( 3 ) = tx ( 2, val )
        imins ( 3 ) = tx ( 2, val )
        imaxs ( 3 ) = tx ( 3, val )
    endif

    if ( i eq 1 ) then begin
        iband = val
        wb = where ( wbands eq iband )

        if ( wb ( 0 ) eq -1 ) then begin
            print, ' '
            print, '   ** Error!   No Band ', iband, ' Data to plot!'
            goto, scont
        endif

        wval = wb ( 0 )

        idefs ( 3 ) = tx ( 2, wval )
        imins ( 3 ) = tx ( 2, wval )
        imaxs ( 3 ) = tx ( 3, wval )
    endif

    if ( ( i eq 2 ) and ( val lt 0 ) ) then begin

        tmode = 1
        isel ( i ) = 0

        stxt = '        Select Y Pixels [D: 0 ] '
        if keyword_set ( solar_y ) then                                       $
           stxt = '        Select Solar_Y Value(s) [D: 0 ] '

        if ( wdefs ( i+1 ) eq 0 ) then begin
            print, format = '($,a)', stxt
            read, ival
        endif else ival = yrange

        cp1 = strpos ( ival, ',' )
        cp2 = strpos ( ival, ':' )
        cp3 = strpos ( ival, ' ' )
        cp = max ( [ cp1, cp2, cp3 ] )

        if ( cp gt 0 ) then begin
            cp = [ cp1, cp2, cp3 ]
            iw = where ( cp ne -1 )
;
            if keyword_set ( solar_y ) then                                  $
                 ysp = ydecode ( ival, ysolar, /solar )                      $
            else ysp = ydecode ( ival, ysolar )
;
            nysp = n_elements ( ysp )
;
            if ( ysp ( 0 ) ge tx0 ( 2, isel ( 0 ) ) ) then ysp = ysp -        $
                                                         tx0 ( 2, isel ( 0 ) )
;
        endif else begin

            if ( strlen ( strcompress ( ival, /remove_all ) ) eq 0 ) then     $
                val = 119                                                     $
            else val = fix ( ival )

            nysp = 1
            ysp = [ val, val ]
            val = 0

            if keyword_set ( solar_y ) then begin
                  dy = ysolar - fix ( ival )
                  dy2 = dy * dy
                  cc = min ( dy2 )
                  isel ( i ) = !c
                  if ( isel ( i ) ge tx0 ( 2, isel ( 0 ) ) ) then             $
                      isel ( i ) = isel ( i ) - tx0 ( 2, isel ( 0 ) )
                  ysp = [ isel ( i ), isel ( i ) ]
            endif
        endelse
    endif

;
;  Trap -ve inputs - reset to zero !?
;
    if ( ( i gt 0 ) and ( val lt imins ( i ) ) ) then val = imins ( i )

    if ( i lt 4 ) then isel ( i ) = val

 fcont:

endfor

if ( isel ( 0 ) ge 0 ) then begin
    if ( tmode eq 0 ) then begin
        sp = gt_spectrum ( cds_s, xix = isel ( 2 ), yix = isel ( 3 ),         $
                           window = isel ( 0 ), lambda = wave )
    endif else begin
        print, ' '
        print, '        Averaging over ', nysp, ' spectra!'
;
        sp = gt_spectrum ( cds_s, xix = isel ( 2 ), yix = ysp ( 0 ),         $
                           window = isel ( 0 ), lambda = wave )
        sp = 0. * sp
;
        for isp = 0, nysp - 1 do begin
            spi = gt_spectrum ( cds_s, xix = isel ( 2 ), yix = ysp ( isp ),  $
                                window = isel ( 0 ) )
            sp = sp + spi
        endfor
    endelse

    wmin = tw ( 0, isel ( 0 ) ) 
    wmax = tw ( 1, isel ( 0 ) )

    xpix = findgen ( n_elements ( wave ) )
    ipmin = min ( xpix )
    ipmax = max ( xpix )
endif else begin
    if ( tmode eq 0 ) then begin
        sp = gt_spectrum ( cds_s, xix = isel ( 2 ), yix = isel ( 3 ),         $
                           band = isel ( 1 ), lambda = wave )
    endif else begin
        print, ' '
        print, '    Averaging over ', nysp, ' spectra!'
;
        sp = gt_spectrum ( cds_s, xix = isel ( 2 ), yix = ysp ( 0 ),         $
                           band = isel ( 1 ), lambda = wave )
        sp = 0. * sp
;
        for isp = 0, nysp - 1 do begin
            spi = gt_spectrum ( cds_s, xix = isel ( 2 ), yix = ysp ( isp ),  $
                                band = isel ( 1 ) )
            sp = sp + spi
        endfor
    endelse

    wmin = min ( wave )
    wmax = max ( wave )

    xpix = findgen ( n_elements ( wave ) )
    ipmin = min ( xpix )
    ipmax = max ( xpix )

endelse

if ( tmode ) then sp = sp / float ( nysp )

;
;  Set up for plotting
;

pcont:

!p.font = -1
!p.charsize = 1.375
!y.ticklen = 0.0125

!p.charthick = 1.0

if ( !d.name eq 'PS' ) then !p.charthick = 2.0

!p.title = '!17SOHO-CDS - Instrument: ' + ins
!y.title = 'Counts'

xarr = wave
xmin = wmin
xmax = wmax

!x.title = 'Wavelength (Ang)'

ymin = min ( sp )
ymax = max ( sp )
dy = 0.0375 * ( ymax - ymin )
ymin = ymin - dy
ymax = ymax + dy

if ( not ibatch ) then plot, xarr, sp, psym = 10, xrange = [ xmin, xmax ],    $
                             xstyle = 1, yrange = [ ymin, ymax ], ystyle = 1

; 
 wfit = 0.0
 gwid = 33
 fw = 0
 sfw = ''
 wmode = 0
;
 wcont:

restart:

if keyword_set ( wcenter ) then begin
    wcen = wcenter
endif else begin
    print, ' '
    read, '        Enter Wavelength to Fit: ', wcen
endelse

 if ( ( wcen lt tw ( 0, isel ( 0 ) ))  or ( wcen gt tw ( 1, isel ( 0 ) ) ) )  $
 then begin
     print, ' '
     print, '     Selected Wavelngth', wcen, ' is outside the data window!'
     print, '     Data Window:', tw ( 0, isel ( 0 ) ), ' to',                 $
            tw ( 1, isel ( 0 ) )

     if ( ibatch gt 0 ) then return, -1

     goto, wcont
 endif

 if ( wmode ) then goto, wcont2

 dw = xarr - wcen
 mw = min ( abs ( dw ) )

 il = !c - gwid
 if ( il lt 0 ) then il = 0
 ih = !c + gwid
 if ( ih gt n_elements ( xarr ) - 1 ) then ih = n_elements ( xarr ) - 1

 nsiz = n_elements ( wave )

 spec = { cds_spec, w: fltarr ( nsiz ), flux: fltarr ( nsiz ) }

 spec.w = wave
 spec.flux = sp

 if ( not ibatch ) then cds_plot_fit, spec, /kelly, /all,                     $
                            wrange = [ xarr ( il ), xarr ( ih ) ]

 lines = cds_sline ( wmin, wmax )
 cds_lines = cds_sline ( wmin, wmax, /cds )

 ncds1 = n_elements ( cds_lines ) / 3

 cwlam = fltarr ( ncds1 )
 celem = strarr ( ncds1 )

 for i = 0, ncds1 - 1 do begin
     cwlam ( i ) = float ( cds_lines ( 2, i ) )
     celem ( i ) = strcompress ( cds_lines ( 0, i ) + cds_lines ( 1, i ) )
 endfor
;
 ncds = n_elements ( lines ) / 3
;
 kwlam = fltarr ( ncds )
 kelem = strarr ( ncds )
;
 for i = 0, ncds - 1 do begin
     kwlam ( i ) = float ( lines ( 2, i ) )
     kelem ( i ) = strcompress ( lines ( 0, i ) + lines ( 1, i ) )
 endfor
;
 if ( ( wmin lt 605.0 ) and ( wmax gt 609.0 ) ) then begin
     wel_heii = 'HeII(2)'
     ww_heii = 2.0 * 303.78
;
     kwlam = [ kwlam, ww_heii ]
     kelem = [ kelem, wel_heii ]
 endif
;
 nkelly = n_elements ( kwlam )
;
 dw = kwlam - wcen
;
 dwave = 4.75 * ( wave ( 1 ) - wave ( 0 ) )
; 
 wz = where ( abs ( dw ) lt dwave )
;
 wave0 = 0.0
 line_id = ''
 id = ''
 dd = 0
;
 if ( wz ( 0 ) ne -1 ) then begin
     nw = n_elements ( wz )

     mdw = min ( abs ( dw ) )
     zlam = kwlam ( !c )

     if ( not ibatch ) then begin
         print, ' '
         print, '     Possible Line IDs from the Kelly List:-'
         print, ' '
         print, '            0)      User Defined Wavelength'
     endif
;
     for i = 0, nw - 1 do begin
;
         flag = ' '
         zz = where ( abs ( cwlam - kwlam ( wz ( i ) ) ) le 0.0475 )
         nz = n_elements ( zz )

         for j = 0, nz - 1 do begin
             if ( zz ( j ) ne -1 ) then begin
                 if ( kelem ( wz ( i ) ) eq celem ( zz ( j ) ) ) then         $
                     flag = '    <<< BLUE BOOK LINE'
             endif
         endfor

         stxt = string ( format =                                             $
                            '( 11x, i2, ") ", a10, "  @", f7.3, " A.", a )',  $
                            i + 1,  kelem ( wz ( i ) ), kwlam ( wz ( i ) ),   $
                            flag )

         if ( not ibatch ) then print, stxt
;
         if ( abs ( kwlam ( wz ( i ) ) - zlam ) lt 0.00025 ) then iz = i + 1

     endfor
;
     if ( ncds1 gt 0 ) then begin
         cwmin = min ( abs ( cwlam - wcen ) ) 
         zz = where ( abs ( kwlam ( wz ) - cwlam ( !c ) ) le 0.0475 )
         nz = n_elements ( zz )
         if ( zz ( 0 ) eq -1 ) then goto, zzcont
         if ( nz eq 1 ) then izz = wz ( zz ( 0 ) )

         for i = 0, nz - 1 do if ( kelem ( wz ( zz ( i ) ) ) eq celem ( !c ) ) $
              then izz = wz ( zz ( i ) )
         wzz = where ( wz eq izz )
         iz = wzz ( 0 ) + 1  
     endif

zzcont:

     if ( not ibatch ) then begin
         print, ' '
         stxt = string ( format = '( "    Enter Choice [D:", i2, "] " )', iz ) 
         print, format = '( $, a )', stxt
         read, id
     endif

     if ( strlen ( strcompress ( id, /remove_all ) ) eq 0 ) then dd = -1 else $
         dd = fix ( id )
;
     if ( dd eq -1 ) then begin
         wave0 = kwlam ( wz ( iz - 1 ) )
         line_id = kelem ( wz ( iz - 1 ) )

         print, ' '
         print, '       Centered on   ', line_id, '  @', wave0, ' Ang.'
     endif
     if ( ( dd gt 0 ) and ( dd lt nw+1 ) ) then begin
         wave0 = kwlam ( wz ( dd - 1 ) )
         line_id = kelem ( wz ( dd - 1 ) )
     endif
;
     if ( dd eq 0 ) then dd = -99
 endif
;
 if ( ( wz ( 0 ) eq -1 ) or ( dd eq -99 ) ) then begin
     if ( wz ( 0 ) eq -1 ) then begin
         print, ' '
         print, '     There are *NO* Kelly Lines close to the specified Wavelength!'
     endif
;
     print, ' '
     read, '      Please Enter The Zero-Velocity Wavelength : ', wave0
     read, '      And the Line Identification [eg. C IV] : ', line_id
 endif
; 
if keyword_set ( recenter ) then begin
    ans = recenter
endif else begin
    ans = ''
    print, ' '
    print, '        You can now fine tune the central fitting wavelength.'
    print, '        (The fitting procedure tends to be more reliable if the line'
    print, '        being fit is centrally located in the fitting window!)'
    print, ' '
    print, '         Enter  Y  to re-enter the central wavelength.'
    print, '  *or*   Enter  K  to use the Kelly Line ID for the central wavelength.'
    print, ' '
    read, '      Would You Like to Re-Center the Fitting Window [y/n D:n] : ', ans
endelse
;
 if ( strupcase ( ans ) eq 'K' ) then wcen = wave0
;
 if ( strupcase ( ans ) eq 'Y' ) then begin
     wmode = 1
     goto, wcont
 endif
;
 wcont2:
;
 if ( ( ibatch ) and keyword_set ( dwidth ) ) then goto, kcont

 dw = xarr - wcen
 mw = min ( abs ( dw ) )

 gwid = 49 / 2

 il = !c - gwid
 if ( il lt 0 ) then il = 0
 ih = !c + gwid
 if ( ih gt n_elements ( xarr ) - 1 ) then ih = n_elements ( xarr ) - 1

 cds_plot_fit, spec, wrange = [ xarr ( il ), xarr ( ih ) ]

 dw = xarr - wcen
 mw = min ( abs ( dw ) )

 ym = total ( sp ( il:ih ) ) / float ( 2 * gwid ) 
 dym = 0.0225 * ( !y.crange ( 1 ) - !y.crange ( 0 ) )

 loadct, 5, /silent


 for k = 0, 5 do begin

     gw = 16 - ( k * 2 )
     il = !c - gw
     if ( il lt 0 ) then il = 0
     ih = !c + gw
     if ( ih gt n_elements ( xarr ) - 1 ) then ih = n_elements ( xarr ) - 1

     ydm = ym + k * dym
     lcol = 220 - k * 32

     oplot, [ xarr ( il ), xarr ( ih ) ], [ ydm, ydm ], color = lcol,         $
            thick = 5.0

     stxt = string ( format = '( i3 )', 2 * gw + 1 )

     xyouts, xarr ( ih ), ym - 0.25 * dym + k * dym, stxt, size = 1.175
 endfor

 print, ' '
 print, '   Bars Indicates Data Window Width in Pixels.'

 gwid = 33

fwcont:

 print, ' '
 stxt = string ( format =                                                     $
      '( 8x, "Enter Data Window Width (Pixels) [D: ", i2, "] " )', gwid )

kcont:

if keyword_set ( dwidth ) then begin
    sfw = dwidth
endif else begin
    print, format = '( $, a )', stxt
    read, sfw
endelse

 if ( strlen ( strcompress ( sfw, /remove_all ) ) eq 0 ) then fw = gwid else  $
      fw = fix ( sfw )

 if ( fw lt 12 ) then begin
     print, ' '
     print, '    Data Window Widths Below  *12*  Pixels are Not Permitted! Sorry!'
     goto, fwcont
 endif

 gwid = fw / 2

 il = !c - gwid
 if ( il lt 0 ) then il = 0
 ih = !c + gwid
 if ( ih gt n_elements ( xarr ) - 1 ) then ih = n_elements ( xarr ) - 1

 nsiz = n_elements ( xarr )

 thres = 1.075 * sqrt ( float ( nysp ) ) * sqrt ( min ( sp ) )
 
 if ( not ibatch ) then begin
     r = cds_fit_wave ( spec, wcen, width = fw, thres = thres )

     print, ' '
     read, '   This is the first spectral fit. OK to continue ? [y/n D: y] : ', ans

     if ( strupcase ( ans ) eq 'N' ) then goto, restart
 endif

 nsp = imaxs ( 3 ) - imins ( 3 ) + 1
 if keyword_set ( solar_y ) then nsp = n_elements ( ysolar )
;
 if ( tmode ) then nsp = nexp + 1
;
 if ( pmode ) then !p.multi = [ 0, 4, 4 ]
;
 fit_w = dblarr ( nsp )
 fit_dw = fit_w
 fit_i = fit_w
 fit_di = fit_w
 tai = fit_w
 ut = fit_w
 dut = 0. * fit_w + ( 0.5 * input.header.exptime )
 sol_x = fit_w
 sol_y = ysolar / 60.0d0

 if ( tmode ) then begin
     m_sol_y = total ( ysolar ( ysp ) ) / float ( nysp )
     if ( nysp eq 1 ) then m_sol_y = ysolar ( ysp ( 0 ) )
     sol_y = fltarr ( nsp ) + ( m_sol_y / 60.0d0 )
 endif

 print, ' '

 for i = 0, nsp - 1 do begin

     if ( isel ( 0 ) ge 0 ) then begin
         if ( tmode eq 0 ) then begin
             sp = gt_spectrum ( cds_s, xix = isel ( 2 ), yix = i,             $
                                window = isel ( 0 ), lambda = wave,           $
                                xsol = xsol, time = tim )
         endif else begin
             sp = gt_spectrum ( cds_s, xix = i, yix = ysp ( 0 ), window =     $
                                isel ( 0 ), lambda = wave, xsol = xsol,       $
                                time = tim )
             sp = 0. * sp

             for isp = 0, nysp - 1 do begin
                 spi = gt_spectrum ( cds_s, xix = i, yix = ysp ( isp ),       $
                                     window = isel ( 0 ) )
                 sp = sp + spi
             endfor
         endelse
     endif else begin
         if ( tmode eq 0 ) then begin
             sp = gt_spectrum ( cds_s, xix = isel ( 2 ), yix = i,             $
                                band = isel ( 1 ), lambda = wave,             $
                                xsol = xsol, time = tim )
         endif else begin
             sp = gt_spectrum ( cds_s, xix = i, yix = ysp ( 0 ),              $
                                band = isel ( 1 ), lambda = wave,             $
                                xsol = xsol, time = tim )
             sp = 0. * sp

             for isp = 0, nysp - 1 do begin
                 spi = gt_spectrum ( cds_s, xix = i, yix = ysp ( isp ),  $
                                     band = isel ( 1 ) )
                 sp = sp + spi
             endfor
         endelse
     endelse

     ptxt = string ( format = '( 8x, "Fitting Spectrum @X,Y:", 2f7.2 )',      $
                     xsol, sol_y ( i ) )
     if ( not pmode ) then print, ptxt

     spec.w = wave
     spec.flux = sp
 
     if ( pmode ) then                                                        $
         r = cds_fit_wave ( spec, wcen, width = fw, thres = thres, /quiet ) $
     else                                                                     $ 
         r = cds_fit_wave ( spec, wcen, width = fw, thres = thres, /noplot )

     lfit = 0

     if ( n_elements ( r.w ) gt 1 ) then begin
          xmin = min ( abs ( r.x ) )
          lfit = !c
     endif

     fit_w ( i ) = r.w ( lfit )
     fit_i ( i ) = r.fint ( lfit )
     fit_dw ( i ) = r.werr ( lfit )
     fit_di ( i ) = r.ferr ( lfit )
     tai ( i ) = tim
     sol_x ( i ) = xsol / 60.0d0
 endfor

 velocity = 29979.246d0 * ( ( fit_w - wave0 ) / wave0 )
 vel_err = 29979.246d0 * ( fit_dw / wave0 )

 cut = int2utc ( utc2int ( tai2utc ( tai ) ) )

 nv = n_elements ( velocity )

 doy = utc2doy ( tai2utc ( tai ) )

 for i = 0, nv - 1 do begin
     fcut = cut ( i )
     fhr = float ( fcut.hour )
     fmn = float ( fcut.minute )
     fsc = float ( fcut.second )
     fms = float ( fcut.millisecond )
     fut = fhr + ( fmn + ( ( fsc + ( fms / 1000. ) ) / 60. ) ) / 60.

     ut ( i ) = ( 24.0 * float ( doy ( i ) - doy ( 0 ) ) ) + fut 
 endfor

 results = { wfit: dblarr ( nv ), vel: dblarr ( nv ), fint: dblarr ( nv ),    $
             werr: dblarr ( nv ), verr: dblarr ( nv ), ferr: dblarr ( nv ),   $
             ysol: dblarr ( nv ), xsol: dblarr ( nv ), tai: dblarr ( nv ),    $
             ut: fltarr ( nv ), dut: fltarr ( nv ), w0: fltarr ( 1 ),         $
             line_id: strarr ( 1 ) , info: strarr ( 14 ) }
;
; Add Results.
;
 results.wfit = fit_w
 results.vel = velocity
 results.fint = fit_i
 results.werr = fit_dw
 results.verr = vel_err
 results.ferr = fit_di
 results.ysol = sol_y
 results.xsol = sol_x
 results.tai = tai
 results.ut = ut
 results.dut = dut
 results.w0 = [ wave0 ]
 results.line_id = [ line_id ]
;
; Add Information on what above structure contains!
;
 results.info = [ 'Fitted Wavelength', 'Velocity', 'Fitted Intensity',        $
                  'Fitted Wavelength Error', 'Velocity Error',                $
                  'Fitted Intensity Error', 'Solar Y', 'Solar X', 'TAI',      $
                  'UT', 'Half Exposure Time', 'Rest Line Wavelength',         $
                  'Line Identification', 'Information Structure' ]

 if ( pmode ) then !p.multi = 0

 if ( not ibatch ) then cds_vel_plot, results

 if keyword_set ( hard ) then begin
     dev_ps
     !p.font = -1
     cds_vel_plot, results
     out_ps

     return, results
 
 endif

 ihard = 0
 ans = ''

 if ( not ibatch ) then begin
     print, ' '
     read, '        Output Plot to Hardcopy Device ? [y/n D: n] : ', ans
 endif

 if ( strupcase ( ans ) eq 'Y' ) then ihard = 1

 if ( ihard eq 1 ) then begin
     dev_ps
     !p.font = -1
     cds_vel_plot, results
     out_ps
 endif

 return, results

 end

