;+
; Project     : SOHO - CDS     
;                   
; Name        : PLOT_SPEC
;               
; Purpose     : To display a single spectrum from a data cube with user
;               selectable inputs.
;               
; Explanation : The procedure takes a data structure (or fits file name) and
;               lists a summary of the data available in it. It then offers
;               the user a variety of options in order to specify the exact
;               details of the spectrum required (eg. band or window, raster,
;               det-y, and wavelength or pixel range).
;               
; Use         : IDL> plot_spec,file_str
;    
; Inputs      : file_str   -  either:-   the name of a CDS FITS file
;                                 or:-   the name of a CDS data structure
;                                (a data structure is the faster option!)
;               
; Opt. Inputs : None
;               
; Outputs     : None (just plots a spectrum in the default window)
;               
; Opt. Outputs: None
;               
; Keywords    : None
;
; Calls       : GT_WLIMITS
;
; Common      : None
;               
; Restrictions: None
;               
; Side effects: None
;               
; Category    : Data_handling, FITS
;               
; Prev. Hist. : None
;
; Written     : B J Kellett, 15-Nov-95.
;               
; Modified    : 
;
; Version     : Version 1, 15-Nov-95.
;-            

 PRO plot_spec, input, info=info, output=output, hardcopy=hardcopy,           $
                solar=solar, spec=spec

;
;  need one parameter
;

if n_params() eq 0 then begin
    print,'USE:   IDL> plot_spec,file or structure
    return
endif

;
;  what was the input?
;
case datatype(input,1) of
   'Structure': begin
                   qlmgr, input, valid
                   if valid then begin
                      cds_s = input
                   endif else begin
                      print,'PLOT_SPEC: Structure supplied is non-standard'
                      return
                   endelse
                end
;
;  allow user to have own version of FITS files
;
      'String': begin
                   tfile = ''
                   if  file_exist(input) then begin
                      tfile = input
                   endif else begin
                     if file_exist(concat_dir('$CDS_CAL_FITS',input)) then begin
                        tfile = concat_dir('$CDS_CAL_FITS',input)
                     endif
                   endelse

;
;  FITS file doesn't exist in user or std directory
;
                   if tfile eq '' then begin
                      print,'PLOT_SPEC:  Input file does not exist.'
                      return
                   endif

                cds_s = readcdsfits ( tfile )

                end
          else: begin
                   print,'Input must be a file name or a CDS data structure.'
                   return
                end
endcase

tx = gt_wlimits ( cds_s, /q )
tw = gt_wlimits ( cds_s, /wave, /q )

tx0 = tx

ins = cds_s.header.detector

narray = cds_s.detdesc
nexp =  narray(0).ixstop(1)
nypix =  narray(0).ixstop(2)

ysolar = narray(0).origin(2) + findgen(nypix+1)*narray(0).spacing(2)

nbands = 4
if ( ins eq 'NIS' ) then nbands = 2

nwinds = n_elements ( tx ) / 4

wbands = intarr ( nwinds )

;
;  Print out summary listing of data
;

print, ' '
print, '                  PLOT_SPEC - Data Summary.'
print, ' '
print, '    Instrument:        ', ins
print, '    Windows:', nwinds
print, '    Bands:      ', nbands
print, '    Exposures:   ', nexp + 1
print, ' '
print, '        (NB: Select Window -1 to enable selection of whole band)'
print, ' '

h1 = ' Window     Label     Band   Det-X Pixels   Det-Y Pixels      Wavelengths'
h2 = ' Window     Label     Band   Det-X Pixels    Solar-Y "        Wavelengths'

if keyword_set(solar) then begin 
    print, h2
    tx ( 2, 0:nwinds-1 ) = ysolar ( 0 )
    tx ( 3, 0:nwinds-1 ) = ysolar ( nypix )
endif else begin
    print, h1
    tx ( 3, 0:nwinds-1 ) = tx ( 3, 0:nwinds-1 ) - tx ( 2, 0:nwinds-1 )
    tx ( 2, 0:nwinds-1 ) = 0.0
endelse

for i = 0, nwinds - 1 do begin
    if ( tx ( 2, i ) lt 512 ) then iband = 2 else iband = 1
    txt = string ( format =                                                   $
                  '( 3x,i2,5x,a,5x,i1,4x,2(i4," - ",i4,4x),f7.3," - ",f7.3)', $
                   i,narray(i).label,iband,tx(0,i),tx(1,i),tx(2,i),tx(3,i),   $
                   tw(0,i),tw(1,i)  )
    print, txt
    wbands ( i ) = iband
endfor

print, ' '

iwin = 0
itix = 0
iyix = tx ( 2, iwin )
iband = 1
ipixel = 0

idefs = [ iwin, iband, itix, iyix, ipixel ]
imins = [ 0, 0, 0, tx ( 2, iwin ), 0 ]
imaxs = [ nwinds - 1, nbands, nexp, tx ( 3, iwin ), 0 ]

tdefs = [ 'Window  ', 'Band    ', 'Exposure', 'Det-Y   ',                     $
          '(P) Pixels or (W) Wavelengths' ]

if keyword_set(solar) then begin
    tdefs ( 3 ) = 'Solar-Y '
endif

isel = 0 * idefs

ival = ''

;
;  Get input values for spectrum selection
;

for i = 0, 4 do begin

    if ( ( i eq 1 ) and ( isel ( 0 ) ge 0 ) ) then goto, fcont

    stxt = string ( format = '( 8x,"Select ", a, " [D:", i4, "] " )',         $
                    tdefs ( i ), idefs ( i ) )
    if ( i eq 4 ) then stxt = string ( format =                               $
                               '( 8x,"Select ", a, " [D: Wavelengths] " )',   $
                                       tdefs ( i ) )

 scont:

    print, format = '($,a)', stxt
    read, ival

;
;  if no value input, set to default
;
    if ( i eq 4 ) then begin
        isel ( i ) = 0
        isea = strpos ( strupcase ( ival ), 'P', 0 )
        if ( isea ne -1 ) then isel ( i ) = 1
    endif else begin
        if ( strlen ( ival ) eq 0 ) then val = idefs ( i )
        cp1 = strpos ( ival, ',' )
        cp2 = strpos ( ival, ':' )
        cp3 = strpos ( ival, ' ' )
        cp = max ( [ cp1, cp2, cp3 ] )
        if ( cp le 0 ) then val = fix ( ival ) else val = imins ( i )
    endelse

;
;  Trap above max. limit inputs - re-request the input value !
;
    if ( ( i lt 4 ) and ( val gt imaxs ( i ) ) ) then begin
        print, ' '
        print, '     *** ERROR!  Maximum No. of ', tdefs ( i ), ' is ',       $
               imaxs ( i ) 
        print, ' '
        goto, scont
    endif

    if ( ( i eq 0 ) and ( val ge 0 ) ) then begin
        wy = tx ( 2, val )
        if ( wy lt 512 ) then iband = 2 else iband = 1
        isel ( 1 ) = iband
        idefs ( 3 ) = tx ( 2, val )
        imins ( 3 ) = tx ( 2, val )
        imaxs ( 3 ) = tx ( 3, val )
    endif

    if ( i eq 1 ) then begin
        iband = val
        wb = where ( wbands eq iband )

        if ( wb ( 0 ) eq -1 ) then begin
            print, ' '
            print, '   ** Error!   No Band ', iband, ' Data to plot!'
            goto, scont
        endif

        wval = wb ( 0 )

        idefs ( 3 ) = tx ( 2, wval )
        imins ( 3 ) = tx ( 2, wval )
        imaxs ( 3 ) = tx ( 3, wval )
    endif

;
;  Trap -ve inputs - reset to zero !?
;
    if ( ( i gt 0 ) and ( val lt imins ( i ) ) ) then val = imins ( i )

    if ( i lt 4 ) then isel ( i ) = val

    if ( i eq 3 ) then begin
        if ( cp gt 0 ) then begin
            cp = [ cp1, cp2, cp3 ]
            iw = where ( cp ne -1 )
;
            if keyword_set(solar) then ysp = ydecode ( ival, ysolar, /solar ) $
            else ysp = ydecode ( ival, ysolar )
;
            ysp = ysp - imins ( 3 )
            nysp = n_elements ( ysp )
;
            if ( ysp ( 0 ) ge tx0 ( 2, isel ( 0 ) ) ) then ysp = ysp -        $
                                                         tx0 ( 2, isel ( 0 ) )
;
            isel ( i ) = -1
        endif else begin
            if keyword_set(solar) then begin
                  dy = ysolar - val
                  dy2 = dy * dy
                  cc = min ( dy2 )
                  isel ( i ) = !c
            endif else isel ( i ) = val - imins ( 3 )
        endelse
    endif

 fcont:

endfor

if ( isel ( 0 ) ge 0 ) then begin
    if ( isel ( 3 ) ne -1 ) then begin
        sp = gt_spectrum ( cds_s, xix = isel ( 2 ), yix = isel ( 3 ),         $
                           window = isel ( 0 ), lambda = wave )

        if keyword_set(info) then begin
            p_x = strcompress ( string ( isel ( 2 ) ), /remove_all )
            p_y = strcompress ( string ( isel ( 3 ) ), /remove_all )
            p_w = strcompress ( string ( isel ( 0 ) ), /remove_all )

            stxt = string ( format =                                          $
  '( "SP = GT_SPECTRUM(QLDS,XIX=",a,",YIX=",a,",WINDOW=",a,",LAMBDA=WAVE)" )',$
                            p_x, p_y, p_w )
;
            print, ' '
            print, '        GT_SPECTRUM command used to extract spectrum ...'
            print, ' '
            print, 'IDL> ', stxt
        endif
    endif else begin
        print, ' '
        print, '        Averaging over ', nysp, ' spectra!'
;
        sp = gt_spectrum ( cds_s, xix = isel ( 2 ), yix = ysp ( 0 ),         $
                           window = isel ( 0 ), lambda = wave )
        sp = 0. * sp
;
        for isp = 0, nysp - 1 do begin
            spi = gt_spectrum ( cds_s, xix = isel ( 2 ), yix = ysp ( isp ),  $
                                window = isel ( 0 ) )
            sp = sp + spi

            if keyword_set(info) then begin
                p_x = strcompress ( string ( isel ( 2 ) ), /remove_all )
                p_y = strcompress ( string ( ysp ( isp ) ), /remove_all )
                p_w = strcompress ( string ( isel ( 0 ) ), /remove_all )

                stxt = string ( format =                                      $
  '( "SP = GT_SPECTRUM(QLDS,XIX=",a,",YIX=",a,",WINDOW=",a,",LAMBDA=WAVE)" )',$
                                p_x, p_y, p_w )
;
                print, ' '
                print, '        GT_SPECTRUM command used to extract spectrum ...'
                print, ' '
                print, 'IDL> ', stxt
            endif
        endfor
    endelse

    wmin = tw ( 0, isel ( 0 ) ) 
    wmax = tw ( 1, isel ( 0 ) )

    xpix = findgen ( n_elements ( wave ) )
    ipmin = min ( xpix )
    ipmax = max ( xpix )
endif else begin
    if ( isel ( 3 ) ne -1 ) then begin
        sp = gt_spectrum ( cds_s, xix = isel ( 2 ), yix = isel ( 3 ),         $
                           band = isel ( 1 ), lambda = wave )

        if keyword_set(info) then begin
            p_x = strcompress ( string ( isel ( 2 ) ), /remove_all )
            p_y = strcompress ( string ( isel ( 3 ) ), /remove_all )
            p_b = strcompress ( string ( isel ( 0 ) ), /remove_all )

            stxt = string ( format =                                          $
  '( "SP = GT_SPECTRUM(QLDS,XIX=",a,",YIX=",a,",WINDOW=",a,",LAMBDA=WAVE)" )',$
                            p_x, p_y, p_b )
;
            print, ' '
            print, '        GT_SPECTRUM command used to extract spectrum ...'
            print, ' '
            print, 'IDL> ', stxt
        endif
    endif else begin
        print, ' '
        print, '    Averaging over ', nysp, ' spectra!'
;
        sp = gt_spectrum ( cds_s, xix = isel ( 2 ), yix = ysp ( 0 ),         $
                           band = isel ( 1 ), lambda = wave )
        sp = 0. * sp
;
        for isp = 0, nysp - 1 do begin
            spi = gt_spectrum ( cds_s, xix = isel ( 2 ), yix = ysp ( isp ),  $
                                band = isel ( 1 ) )
            sp = sp + spi

            if keyword_set(info) then begin
                p_x = strcompress ( string ( isel ( 2 ) ), /remove_all )
                p_y = strcompress ( string ( ysp ( isp ) ), /remove_all )
                p_b = strcompress ( string ( isel ( 1 ) ), /remove_all )

                stxt = string ( format =                                      $
  '( "SP = GT_SPECTRUM(QLDS,XIX=",a,",YIX=",a,",BAND=",a,",LAMBDA=WAVE)" )',$
                                p_x, p_y, p_b )
;
                print, ' '
                print, '        GT_SPECTRUM command used to extract spectrum ...'
                print, ' '
                print, 'IDL> ', stxt
            endif
        endfor
    endelse

    wmin = min ( wave )
    wmax = max ( wave )

    xpix = findgen ( n_elements ( wave ) )
    ipmin = min ( xpix )
    ipmax = max ( xpix )

    print, ' '

    if ( isel ( 4 ) eq 0 ) then begin
        stxt = string ( format =                                              $
               '( 8x,"Select Wavelength Range [D:", f7.3,"-",f7.3, "] " )',   $
                    wmin, wmax )
        print, format = '($,a)', stxt
        read, ival

        if ( strlen ( ival ) gt 0 ) then begin
            wmin = float ( ival )
            isp = strpos ( ival, ' ', 0 )
            wmax = float ( strmid ( ival, isp, strlen ( ival ) - isp ) )
        endif
    endif else begin
        stxt = string ( format =                                              $
               '( 8x,"Select Pixel Range [D:", i4,"-",i4, "] " )',            $
                    fix ( min ( xpix ) ), fix ( max ( xpix + 0.25 ) ) )
        print, format = '($,a)', stxt
        read, ival

        if ( strlen ( ival ) gt 0 ) then begin
            ipmin = float ( ival )
            isp = strpos ( ival, ' ', 0 )
            ipmax = float ( strmid ( ival, isp, strlen ( ival ) - isp ) )
        endif
    endelse

    if keyword_set(info) then begin
        p_x = strcompress ( string ( isel ( 2 ) ), /remove_all )
        p_y = strcompress ( string ( isel ( 3 ) ), /remove_all )
        p_b = strcompress ( string ( isel ( 1 ) ), /remove_all )

        stxt = string ( format =                                              $
   '( "SP = GT_SPECTRUM(QLDS,XIX=",a,",YIX=",a,",BAND=",a,",LAMBDA=WAVE)" )', $
                        p_x, p_y, p_b )
;
        print, ' '
        print, '        GT_SPECTRUM command used to extract spectrum ...'
        print, ' '
        print, 'IDL> ', stxt
    endif

endelse

if ( isel ( 3 ) eq -1 ) then sp = sp / float ( nysp )

;
;  Set up for plotting
;

pcont:

!p.font = -1
!p.charsize = 1.375
!y.ticklen = 0.0125

!p.charthick = 1.0

if ( !d.name eq 'PS' ) then !p.charthick = 2.0

!p.title = '!17SOHO-CDS - Instrument: ' + ins
!y.title = 'Counts'

xarr = wave
xmin = wmin
xmax = wmax

!x.title = 'Wavelength (Ang)'

if ( isel ( 4 ) eq 1 ) then begin
    xarr = xpix
    xmin = ipmin
    xmax = ipmax
    !x.title = 'Pixels'
endif

ymin = min ( sp )
ymax = max ( sp )
dy = 0.0375 * ( ymax - ymin )
ymin = ymin - dy
ymax = ymax + dy

plot, xarr, sp, psym = 10, xrange = [ xmin, xmax ], xstyle = 1,               $
                yrange = [ ymin, ymax ], ystyle = 1

if keyword_set(output) then begin
    file = ''
    print, ' '
    read, '        Enter Output Filename [D: .qdp] : ', file

    iq = strpos ( file, '.', 0 )

    if ( iq eq -1 ) then file = file + '.qdp'

    openw, 9, file

    nw = n_elements ( xarr )

    for i = 0, nw - 1 do printf, 9, format = '(2f16.7)', xarr ( i ), sp ( i )

    close, 9

    print, ' '
    print, '        Spectrum output to ....       ', file
    print, ' '
endif

if keyword_set(hardcopy) then begin
    ihard = 1
endif else begin
    ihard = 0
    print, ' '
    read, '        Output Plot to Hardcopy Device ? [y/n D: n] : ', ival
    if ( strupcase ( ival ) eq 'Y' ) then ihard = 1
endelse

if ( ihard eq 1 ) then begin
    dev_ps
    !p.font = -1
    !p.charthick = 2.0
    plot, xarr, sp, psym = 10, xrange = [ xmin, xmax ], xstyle = 1,           $
                    yrange = [ ymin, ymax ], ystyle = 1
    out_ps
endif

 nsiz = n_elements ( xarr )
 spec = { cds_spec, w: fltarr ( nsiz ), flux: fltarr ( nsiz ) }
;
 spec.w = xarr
 spec.flux = sp
; 

!y.ticklen = 0.020

end

