;+
; Project     : SOHO - CDS     
;                   
; Name        : QLDS_CHECKIN
;               
; Purpose     : Swap the specified QLDS into the cache (QLDS in QLSAVE).
;               
; Explanation : The CDS QuickLook memory management system is designed to
;               minimize the time spent by display routines fetching their
;               data. This is done by having a common block (QLSAVE) that
;               functions as a cache memory, storing the most recently used
;               QuckLook Data Structure. Other QLDS'es are stored on handles.
;               
;               Typically an event routine has the QLSAVE common block, and
;               would call QLDS_CHECKIN before accessing the QLDS in the
;               common block. Needles memory-swapping is avoided by
;               QLDS_CHECKIN by first checking whether the correct QLDS is
;               there already.
;               
; Use         : QLDS_CHECKIN,QL_NO
;    
; Inputs      : QL_NO : The unique number assigned to a QLDS by QLMGR,
;                       on the tag "QL_NO" in the structure.
;               
; Opt. Inputs : None.
;               
; Outputs     : None.
;               
; Opt. Outputs: None.
;               
; Keywords    : WIDGET : Set to a widget base ID that has the QL_NO as
;                       UVALUE. QL_NO need not be supplied explicitly
;                       in this case.
;
;               TEST : Set to avoid a crash if the QLDS could not be
;                       found. Used by e.g. QLDS_INVENTORY
;                       
; Calls       : trim(), datatype(), qlds_store
;
; Common      : QLSAVE: QLDS : The cache slot. The specified QLDS is
;                       stored in this variable (if found).
;                       SAVED : A status variable used in IDL 3.0 to
;                       signify that the QLDS has already been stored
;                       on its storage base.
;
;               QLMGR: See QLMGR
;
; Restrictions: Requires that the QLDS actually can be found
;               
; Side effects: 
;               
; Category    : QuickLook, Management
;               
; Prev. Hist. : None.
;
; Written     : Stein Vidar Hagfors Haugan, ItA/UiO, 11-June-1994
;               
; Modified    : SVHH, 2-March-1995, Removed some CDSNOTIFY calls
;
;               Version 2, SVHH, 21 May 1996
;                       Improved handling of TEST  keyword.
;               Version 3, SVHH, 15 July 1997
;                       Using handles instead of widget bases to store qlds
;                       (immune against e.g., xkill,/all). Also removed some
;                       old-version stuff (IDL 3.0!).
;
; Version     : 3, 15 July 1997
;-            

PRO qlds_checkin,ql_no,widget=widget,test=test
  common QLSAVE,QLDS,saved
  common QLMGR,QL_ID,QL_DATA
  
  On_Error,2
  IF !DEBUG ne 0 THEN On_Error,0
;
; Checking...
;  
  IF N_elements(widget) eq 1 THEN Widget_CONTROL,widget,Get_UVALUE=ql_no
  IF N_elements(ql_no) ne 1 THEN message,'Use: CHECKIN, QL_NO'
  IF N_elements(QL_ID) EQ 0 THEN BEGIN
     IF KEYWORD_SET(test) THEN return
     MESSAGE,'QLMGR not initialized!'
  END
  
  IF ql_no LT 0 OR ql_no GT N_elements(QL_DATA)-1 THEN BEGIN
     IF KEYWORD_SET(test) THEN return
     MESSAGE,'QL_NO must be <0,...,'+trim(N_elements(QL_DATA)-1)+'>'
  END
  
  
;
; Is it there already?
;  
  IF DATATYPE(QLDS) eq 'STC' THEN $
     IF QLDS.QL_NO eq QL_NO THEN return ; All's ok!
  
; If we get here, we need to reload....
  
; But first -- save anything that's there !
  
  qlds_store
  
  QLDS = 0 ;; Default marking.... not a structure
;
; Now see if we can get it!
;
  store_h = QL_DATA(ql_no).storage_h
  
  IF handle_info(store_h,/valid_id) EQ 0 THEN BEGIN 
     IF KEYWORD_SET(test) THEN return
     MESSAGE,'Storage handle for QLDS #'+trim(ql_no)+' is destroyed'
  ENDIF
  
  handle_value,store_h,qlds,/no_copy
  
  IF DATATYPE(QLDS) ne 'STC' and NOT Keyword_SET(test) $
     THEN message,'No QLDS available'
  
  saved = 0
  
;
; If we get here, it should be OK!
;
  return
END

