;+
; Project     : SOHO - CDS     
;                   
; Name        : QLDS_REPORT
;               
; Purpose     : Make a connection between the Widget ID and a managed QLDS
;               
; Explanation : The CDS QuickLook memory management system is designed
;               to minimize the time spent by display routines fetching
;               their data. This is done by having a common block (QLSAVE)
;               that functions as a cache memory, storing the most recently
;               used QuickLook Data Structure. Other QLDS'es are stored
;               on handles.
;
;               Typically an event routine has the QLSAVE common block,
;               and would call QLDS_CHECKIN before accessing the QLDS
;               in the common block. 
;
;               The main routine needs to setup a connection to the
;               specific QLDS, so that the event routine knows which
;               one to ask for. This is normally done by QLDS_REPORT,
;               which sets the UVALUE of a widget base to the QL_NO that
;               is otherwise used in the call to QLDS_CHECKIN.
;
;               Due to the frequent use of this scheme, it is also
;               possible to call QLDS_CHECKIN with just the widget
;               ID of the base carrying the QL_NO, through the
;               keyword WIDGET.
;               
; Use         : QLDS_REPORT,WIDGET_ID,QLDS
;    
; Inputs      : WIDGET_ID : The widget whose UVALUE should be used.
;                           This parameter may be left undefined when e.g.,
;                           using the memory management system outside of
;                           widget programs.
;       
;               QLDS : Standard CDS QLDS.
;               
; Opt. Inputs : None.
;               
; Outputs     : None.
;               
; Opt. Outputs: None.
;               
; Keywords    : STORE : If this is set, the QLDS is actually put into
;                       the storage scheme. If not set, it is assumed
;                       that it is already accessible through a call
;                       to QLDS_CHECKIN.
;
; Calls       : qlds_store, xalive()
;
; Common      : QLSAVE: See QLDS_CHECKIN
;               QLMGR : See QLMGR
;               
; Restrictions: QLDS must be real.
;               
; Side effects: In IDL version 3.1 or later, the supplied QLDS "disappears",
;               but may be retrieved by QLDS_FETCH.
;               
; Category    : QuickLook, Management
;               
; Prev. Hist. : None.
;
; Written     : Stein Vidar Hagfors Haugan, ItA/UiO, 11-June-1994
;               
; Modified    : SVHH, 2-March-1995, Removed CDSNOTIFY calls
;               Version: 2, SVHH, 30 September 1996
;                        Using since_version to determine /no_copy validity.
;               Version 3, SVHH, 15 July 1997
;                       Using handles instead of widget bases for storage.
;                       
; Version     : 3, 15 July 1997
;-            

PRO qlds_report,W_ID,QLDS,store=store
  
  common QLSAVE,sQLDS,saved
  common QLMGR,QL_ID,QL_DATA
  
  IF N_params() lt 2 THEN message,'Use: QLDS_REPORT,WINDOW_ID,QLDS [,/STORE]'
  IF N_elements(QL_ID) eq 0 THEN QLMGR,QLDS
  IF QLDS.QL_ID ne QL_ID THEN QLMGR,QLDS
  
  ql_no = QLDS.QL_NO
  
  IF Keyword_SET(store) THEN  BEGIN
     store_h = QL_DATA(QLDS.QL_NO).storage_h
     
     IF handle_info(store_h,/valid_id) eq 0 THEN BEGIN
        store_h = handle_create()
        QL_DATA(QLDS.QL_NO).storage_h=store_h
     EndIF
     
     ;; Store current
     qlds_store
     sqlds = temporary(qlds)
     saved = 0
  EndIF
  IF xalive(w_id) THEN Widget_CONTROL,W_ID,Set_UVALUE=ql_no
END
