;+
; Project     : SOHO - CDS     
;                   
; Name        : CMOUSE
;               
; Purpose     : Controlling the "Mouse button action" selection & display
;               
; Explanation : In order to make more than three types of actions available
;               for the mouse buttons, a "Mouse button action" display with a
;               pulldown menu for each mouse button is created.
;
;               The pulldown menu has the following appearance when it is not
;               pulled down:
;
;                1:<btext>      2:<btext>     3:<btext>
;               
;               The user selects what type of action is to be hooked to the
;               different mouse buttons at any time by selecting from the
;               pulldown menu of the desired button. The <btext> field on the
;               buttons is changed according to the selected action.
;               
;               The main program then calls CMOUSE_ACTION(EVENT) in order to
;               translate a widget DRAW event into a "uvalue" corresponding to
;               the keypress and the current action selection.
;
;
; Use         : CMOUSE,TOP,BASE,OPTIONS
;    
; Inputs      : TOP:    The widget ID of the top level base 
;                       (used for identification).
;
;               BASE:   The widget base to place the display on.
;               
;               OPTIONS: An array of {CMOUSE_S} (see below) with a 
;                       description of the available mouse keyclick actions.
;
;               The options has the structure
;
;               {CMOUSE_S, btext:'', mtext:'', uvalue:'', avail:0}
;
;               The fields have the following meanings: "btext" is the "button
;               text", i.e., the text that is to be displayed on the pulldown
;               menu if this option is selected.  mtext is the menu text,
;               i.e., the text on the pulldown menu
;
;               The field "avail" is used as a bit field:
;               Bit 0 (value 1) : This option available for button 1
;               Bit 1 (value 2) : This option available for button 2
;               Bit 2 (value 4) : This option available for button 3
;               Bit 4 (value 8) : (Repeat) Pass on motion events when 
;                                 button is pressed continuously.
;
; Opt. Inputs : None
;               
; Outputs     : None.
;               
; Opt. Outputs: Keyword ID
;               
; Keywords    : INIT : An array of 3 integers specifying the initial state
;                       (indices to OPTIONS).
;
;               ID : Returns the identity of this cmouse widget, for the
;                    possibility of having more than one per base.
;
; Calls       : WIDGET_BASE(), ZPARCHECK
;
; Common      : CMOUSE
;               
; Restrictions: The UVALUES must be strings.
;               
; Side effects: 
;               
; Category    : Utility, display
;               
; Prev. Hist. : None.
;
; Written     : Stein Vidar Hagfors Haugan, 11-December-1994
;               
; Modified    : Version 2, SVHH, 9-January-1996
;                          Added output keyword ID, to facilitate
;                          more than one CMOUSE widget per top base.
;               Version 3, SVHH, 31 May 1996
;                          Fixed a bug that gave all buttons title
;                          '1:'+btext instead of trim(button+1)+':'+btext.
;
; Version     : 3, 31 May 1996
;-            


PRO cmouse_event,ev
  COMMON cmouse,ctop,cbase
  
  i = (WHERE(ev.handler EQ cbase))(0)
  Widget_CONTROL,cbase(i),Get_UVALUE=status
  
  v = ev.value
  button = status.buttons(v)
  status.status(button) = v
  button_id = status.button_ID(v)  
  btext = status.btext(v)

  WIDGET_CONTROL,button_ID,set_value= trim(button+1)+':'+btext
  
  Widget_CONTROL,cbase(i),Set_UVALUE=status

  RETURN
  
END





PRO cmouse,top,base,options,init=init,id=id
  COMMON cmouse,ctop,cbase
  
  IF N_params() LT 3 THEN MESSAGE,"Use: 	CMOUSE,TOP,BASE,OPTIONS"
  
  zparcheck,'CMOUSE',options,3,8,1,'CMOUSE OPTIONS'
  
  IF tag_names(options,/struct) NE 'CMOUSE_S' THEN $
     MESSAGE,"OPTIONS must be an array of {CMOUSE_S}"
  
  
  IF N_elements(ctop) EQ 0 THEN BEGIN
     ctop=[top]
     cbase=[0L]
     i=0
  END ELSE BEGIN
     i=N_elements(ctop)
     ctop = [ctop,top]
     cbase = [cbase,0L]
  EndELSE
  
  ibase = Widget_BASE(base,/column,event_pro='cmouse_event',$ ; Internal base
                      space=0, xpad=0, ypad=0, frame=1)  
  label = Widget_LABEL(ibase,value="Editable mouse buttons")
  
  
  cw_pdmenu_s = {CW_PDMENU_S, flags:0, name:''}
  
  
;
; Which options are available on each mouse button
;
  
  ix1 = WHERE((options(*).avail AND 1) NE 0,N1)
  IF N1 EQ 0 THEN $
     MESSAGE,"There MUST be an option for every button (use dummy values)"
  
  mtext1 = REPLICATE({CW_PDMENU_S},N1)
  mtext1(*).name = options(ix1).mtext
  mtext1(N1-1).flags=2

  btext = ['',options(ix1).btext]
  uvalue = ['',options(ix1).uvalue]
  rep = [0,(options(ix1).avail AND 8)/8]
  
  
  
  ix2 = WHERE((options(*).avail AND 2) NE 0,N2)
  IF N2 EQ 0 THEN $
     MESSAGE,"There MUST be an option for every button (use dummy values)"
  
  mtext2 = REPLICATE({CW_PDMENU_S},N2)
  mtext2(*).name = options(ix2).mtext
  mtext2(N2-1).flags=2

  btext = [btext,'',options(ix2).btext]
  uvalue = [uvalue,'',options(ix2).uvalue]
  rep = [rep,0,(options(ix2).avail AND 8)/8]
  
  
  
  ix3 = WHERE((options(*).avail AND 4) NE 0,N3)
  IF N3 EQ 0 THEN $
     MESSAGE,"There MUST be an option for every button (use dummy values)"
  
  mtext3 = REPLICATE({CW_PDMENU_S},N3)
  mtext3(*).name = options(ix3).mtext
  mtext3(N3-1).flags=2

  btext = [btext,'',options(ix3).btext]
  uvalue = [uvalue,'',options(ix3).uvalue]
  rep = [rep,0,(options(ix3).avail AND 8)/8]
  
;
; Decide on the initial display
;
  
  IF N_elements(init) EQ 3 THEN BEGIN
     ini1 = (WHERE(ix1 EQ init(0)))(0)
     ini2 = (WHERE(ix2 EQ init(1)))(0)
     ini3 = (WHERE(ix3 EQ init(2)))(0)
  END ELSE BEGIN
     ini1 = 0
     IF ix2(0) NE ix1(0) THEN ini2=0 $
     ELSE		       ini2= 1 < (N2-1)
     IF ix3(0) NE ix1(ini1) AND ix3(0) NE ix2(ini2) THEN      ini3=0 $
     ELSE IF ix3(1) NE ix1(ini1) AND ix3(1) NE ix2(ini2) THEN ini3=1 $
     ELSE						       ini3=2
  EndELSE
  
  ini3 = ini3 < (N3-1)


  ini1 = ini1 + 1
  ini2 = ini2 + 2 + N1
  ini3 = ini3 + 3 + N1 + N2

  menu = [{CW_PDMENU_S, 1, '1:'+btext(ini1)},$
          mtext1,$
          {CW_PDMENU_S, 1, '2:'+btext(ini2)},$
          mtext2,$
          {CW_PDMENU_S, 1, '3:'+btext(ini3)},$
          mtext3]
  
  pdmenu = cw_pdmenu(ibase,menu)

  pdbutton = Widget_INFO(pdmenu,/child)
  
  buttons = [REPLICATE(0,N1 + 1),$
             REPLICATE(1,N2 + 1),$
             REPLICATE(2,N3 + 1)]

  button_ID=[REPLICATE(pdbutton,N1 + 1),$
             REPLICATE(Widget_INFO(pdbutton,/sibling),N2 + 1),$
             REPLICATE(Widget_INFO(Widget_INFO(pdbutton,/sibling),/sibling),N3+1)]

  
  status = { status: [ini1, ini2, ini3],$
             buttons: buttons,$
             button_ID : button_ID,$
             pressed: 0,$
             mtext  : ['',mtext1.name,'',mtext2.name,'',mtext3.name],$
             btext  : btext,$
             uvalue : uvalue,$
             rep : rep }
  
  
  Widget_CONTROL,ibase,Set_UVALUE=status
  cbase(i)=ibase
  id = ibase
END
