;+
; Project     : SOHO - CDS     
;                   
; Name        : DSP_AUX
;               
; Purpose     : Plot/display auxiliary data from a qlds.
;               
; Explanation : The Quick Look Data Structure contains informations about
;               auxiliary data derived from telemetry data by fitsgen. This
;               routine is designed to plot those data.
;               
; Use         : DSP_AUX,QLDS
;    
; Inputs      : QLDS : Quick Look Data Structure.
;               
; Opt. Inputs : None.
;               
; Outputs     : None.
;               
; Opt. Outputs: None.
;               
; Keywords    : GROUP_LEADER : Widget group leader
;
; Calls       : assign_nocopy, break_file, cleanplot, clipbox, cw_plotz(),
;               default, ps, psclose, psplot, qlds_checkin, qlds_report,
;               qlmgr, widget_base(), xack, xalive(), xkill, xmanager,
;               xps_setup 
;
; Common      : QLSAVE : See e.g., QLDS_CHECKIN.
;               
; Restrictions: QLDS should be valid.
;               
; Side effects: If the QLDS is not managed by the Quick Look memory management
;               system, the call will not return until the widget is dead, to
;               ensure that the QLDS can be returned safely.
;               
; Category    : Quick Look, Display.
;               
; Prev. Hist. : None.
;
; Written     : Stein Vidar H. Haugan, UiO, 19 August 1996
;               
; Modified    : Version 2, SVHH, 28 April 1997
;                       Renamed GROUP -> GROUP_LEADER, started actually
;                       *using* GROUP to make the top level base be killed
;                       when group leader dies.
;                       Added graceful exit with XACK when plotting is not
;                       possible.
;                       
;
; Version     : 2, 28 April 1997
;-            

PRO DSP_AUX_PRINT,EV,info
  
  COMMON qlsave,qlds,saved
  COMMON DSP_AUX_XPS_SETUP,setup,count

  qlds_checkin,info.ql_no
   
;   
; Printer setup/defaults
;
  
  ;; dummy = temporary(setup) ;; Deletes setup
  
  fitsfile = (str_sep(qlds.header.filename,'.'))(0)
  yname = info.tags(info.current)
  yname = STRMID(yname,0,STRLEN(yname)-4)
  IF info.vs_current EQ 100 THEN xname='index' $
  ELSE BEGIN
     xname = info.tags(info.vs_current)
     xname = STRMID(xname,0,STRLEN(xname)-4)
  END
  
  psfile = ("%FILE_aux_%Y_vs_%X") ;; "_t%tix"

  IF datatype(setup) NE 'STC' THEN BEGIN
     xps_setup,setup,/initial
     setup.printer = getenv("PSLASER")
     IF setup.printer EQ '' THEN setup.printer = getenv("PRINTER")
     IF setup.printer EQ '' THEN setup.printer = getenv("PSCOLOR")
     IF setup.printer EQ '' THEN setup.printer = 'lp0'
     setup.color = 1
     break_file,setup.filename,disk,dir,filnam,ext
     setup.filename = disk+dir+psfile+ext
  END
  
  setup.portrait = 1
  
  GROUP = EV.TOP
  
  IF NOT KEYWORD_SET(goahead) THEN BEGIN
     xack,group=group,$
        ["You can use the following special symbols in the PS file name:",$
         "",$
         "  %FILE will be substituted with '"+fitsfile+"'",$
         "  %Y will be substituted with '"+yname+"'",$
         "  %X will be substituted withe '"+xname+"'",$
         "  %COUNT will be substituted with a counter value, and the",$
         "         counter will be incremented"]
     xps_setup,setup,status=status,group=group ;; To avoid XKILL,/ALL!!
     IF NOT status THEN RETURN
  END
  
  filename = repstr(setup.filename,'%FILE',fitsfile)
  filename = repstr(filename,'%Y',yname)
  filename = repstr(filename,'%X',xname)
    
  IF STRPOS(filename,'%COUNT') GE 0 THEN BEGIN
     default,count,1
     filename = repstr(filename,'%COUNT',trim(count))
     count = count+1
  END
  
;
; FINALLY, setup completed
;
  
  ps,filename,encapsulated=setup.encapsulated, $
     portrait=setup.portrait, color=setup.color,copy=setup.copy, $
     interpolate=setup.interpolate
  
  cleanplot
;
; How much space do we have to play with?
;
  
  x0 = 0
  y0 = 0
  x1 = !D.X_SIZE
  y1 = !D.Y_SIZE
  
  !P.clip = [x0,y0,x1,y1]
  clipbox
  
;  
; Want boxratio:1 boxes, with some room at the top
;
  
  boxratio = 1.0
  
  topfrac = 0.15 ;; Top fraction of page used for general info
  xmargin = 0.02
  ymargin = 0.02
  
;
; Reserve room at the top (texts etc):
;  
  topheight = (y1-y0)*topfrac
  x0top = x0 + xmargin*(x1-x0)
  y0top = y1 - topheight + ymargin*topheight
  x1top = x1 - xmargin*(x1-x0)
  y1top = y1 - ymargin*topheight
  
;
; Mark it
;
  !P.clip = [x0top,y0top,x1top,y1top]
  clipbox
  
;
; Texts to be displayed
;
  
  titlel = "Title: "+qlds.header.title
  filel = "File: "+fitsfile+"  "+qlds.header.date_obs
  
  lheight = !D.y_ch_size * 1.5
  
  ;; Descriptive text in top panel
  txt = [titlel, filel]
;
; Where to put it
;
  xtx = x0top + 0.02*(x1top-x0top)
  ytx = y1top - lheight
  
  FOR i=0,N_ELEMENTS(txt)-1 DO BEGIN
     xyouts,xtx,ytx,txt(i),/DEVICE
     ytx = ytx-lheight
  END
  
;  
; Room below the top
;
  botheight = (y1-y0)*(1.0-topfrac)
  x0bot = x0top
  y0bot = y0 + ymargin*botheight
  x1bot = x1top
  y1bot = y0 + botheight - ymargin*botheight
  
  !P.clip = [x0bot,y0bot,x1bot,y1bot]
  clipbox
  
;
; Maximum sizes
;
  xmargin = 0.15
  
  n_items = 1
  maxwidth = (1.0-xmargin)*(x1bot-x0bot)
  maxheight = (1.0-ymargin)*(y1bot-y0bot)
  
;
; Keep the height:width ratios, but within maximum sizes
;
  height = maxheight < boxratio*maxwidth
  width = maxwidth < (height/boxratio)
  
;
; Center positions of each panel (only one)
; 
  xposition = x0bot + 0.5*(x1bot-x0bot)/n_items
  yposition = 0.5*(y0bot+y1bot)
  
;
; Structure that signals a request for postscript plotting.
;
  psset = {replot:1}
  
;
; Tell everyone about their place, and plot.
;
  x0box = xposition-0.5*width
  y0box = yposition-0.5*height
  x1box = xposition+0.5*width
  y1box = yposition+0.5*height
  !P.clip = [x0box,y0box,x1box,y1box]
  clipbox
  WIDGET_CONTROL,info.plotz_id,set_value=psset
  
;
; Close up and return.
;
  
  IF setup.hard THEN $
     psplot,filename,queue=setup.printer,delete=setup.delete $
  ELSE  $
     psclose
  
  RETURN
END



PRO dsp_aux_event,ev
  COMMON QLSAVE,QLDS
  
  WIDGET_CONTROL,ev.top,get_uvalue=info
  
  WIDGET_CONTROL,ev.id,get_uvalu = uvalue
  
  qlds_checkin,info.ql_no
  
  IF datatype(uvalue) EQ 'STR' THEN BEGIN
     IF uvalue EQ 'PRINT' THEN BEGIN
        dsp_aux_print,ev,info 
        RETURN
     END
     WIDGET_CONTROL,ev.top,/destroy
     RETURN
  END ELSE BEGIN
     IF uvalue GT 0 THEN info.current = uvalue $ 
     ELSE                info.vs_current = -uvalue
  END
  
  data = qlds.(info.current)
  missing = qlds.(info.current+1).missing
  title = info.tags(info.current)
  
  IF STRMID(info.tags(info.current),0,3) EQ "BKG" THEN BEGIN
     DATA = TOTAL(DATA,1)
     title = title + ' (averaged over detectors)'
  END
  
  ND = N_ELEMENTS(data)
  
  data = REFORM(data,ND,/overwrite)
  
  IF info.vs_current NE 100 THEN BEGIN
     IF N_ELEMENTS(qlds.(info.vs_current)) EQ ND THEN BEGIN
        X = REFORM(qlds.(info.vs_current),ND)
        ix = SORT(x)
        DATA = [[X(ix)],[DATA(ix)]]
        xtitle = info.tags(info.vs_current)
     END ELSE BEGIN
        PRINT,"Data mismatch"
     END
  END
  
  IF NOT exist(xtitle) THEN BEGIN
     xtitle = 'Index ordered'
  END
  
  
  WIDGET_CONTROL,info.plotz_id,set_value={title:title, missing:missing,$
                                          xtitle:xtitle,zoom:.5}
  WIDGET_CONTROL,info.plotz_id,set_value=data
  
  WIDGET_CONTROL,ev.top,set_uvalue=info
END



PRO dsp_aux,QLDS,group_leader=group
  
  COMMON qlsave,qlds_saved
  
  ON_ERROR,2
  IF !debug NE 0 THEN ON_ERROR,0
  
  help,calls = calls
  interactive = STRMID(calls(1),0,6) EQ '$MAIN$'
  
  IF interactive THEN xkill,/all
  
  IF N_PARAMS() LT 1 THEN MESSAGE,"Use: DSP_AUX,QLDS"
  qlmgr,qlds,valid
  
  IF NOT valid THEN MESSAGE,"Parameter is not a valid QLDS"
  
  tags = tag_names(QLDS)
  ql_no = qlds.ql_no
  
  default,group,0L
  
  fname = (str_sep(qlds.header.filename,'.'))(0)
  
  top = WIDGET_BASE(column=1,title=fname+" auxiliary data",group=group)

  ;; QUIT button, top row
  toprow = WIDGET_BASE(top,/row)
  quit = WIDGET_BUTTON(toprow,value='Quit',uvalue='QUIT')
  prnt = WIDGET_BUTTON(toprow,value='Print',uvalue='PRINT')
  
  ;; ORDINATE list/ ABSCISSA list
  ord_base = WIDGET_BASE(top,frame=2,/column)
  dummy = WIDGET_LABEL(ord_base,value = 'SELECT DATA ITEM TO PLOT')
  list_o_row = WIDGET_BASE(ord_base,/row)
  
  abs_base = widget_base(top,frame=2,/column)
  dummy = WIDGET_LABEL(abs_base,value = 'PLOT DATA AS A FUNCTION OF:')
  list_a_row = WIDGET_BASE(abs_base,/row)
  
  ncols = 5
  lists_o_col = lonarr(ncols)
  lists_a_col = lists_o_col
  
  FOR i = 0,ncols-1 DO BEGIN
     lists_o_col(i) = WIDGET_BASE(list_o_row,/column)
     lists_a_col(i) = WIDGET_BASE(list_a_row,/column)
  END
  
  ix = WHERE(tags EQ "DEL_TIMEDATA")
  
  na = 0
  
  FOR i = ix(0),N_ELEMENTS(tags)-1,2 DO BEGIN
     data = qlds.(i)
     list_o = lists_o_col((i-ix(0))/2/ncols)
     list_a = lists_a_col(na MOD ncols)
     IF N_ELEMENTS(data) GT 1 THEN BEGIN
        name = STRMID(tags(i),0,STRLEN(tags(i))-4)
        roww = WIDGET_BASE(list_o,/row,xpad=0,ypad=0,space=0)
        dummy = WIDGET_BUTTON(roww,value=name,uvalue=i)
        IF MAX(data) NE MIN(data) THEN BEGIN
           na = na+1
           roww = WIDGET_BASE(list_a,/row,xpad=0,ypad=0,space=0)
           dummy = WIDGET_BUTTON(roww,value=name,uvalue=-i)
        END
        IF N_ELEMENTS(value) EQ 0 THEN BEGIN
           value = REFORM(data,N_ELEMENTS(data))
           current = i
           xtitle='"Index ordered"'
           title=tags(i)
        END
     END
  END
  
  IF n_elements(current) EQ 0 THEN BEGIN
     xack,['None of the auxiliary data can be plotted (single points only)',$
           '',$
           'From DSP_MENU use "General (header) information: Auxiliary info"',$
           'to see values','',$
           '(From the command line use "DSP_INFO,qlds,/no_basic,/aux")']
     return
  END
  
  
  list_a = lists_a_col(na MOD ncols)
  roww = widget_base(list_a,/row)
  dummy = WIDGET_BUTTON(roww,value='Index ordered',uvalue=-100)
  
  plotz_id = cw_plotz(top,xwsize=400,ywsize=400,/autonomous,value=value,$
                     psym=-1,linestyle=3,title=title,xtitle=xtitle)
  
  info = {tags:tags,$ 
          current:current,$
          vs_current:100,$
          ql_no:ql_no,$
          plotz_id:plotz_id}
  
  managed = qlds_managed(qlds)
  IF NOT managed THEN qlds_report,top,qlds,store=1
  
  WIDGET_CONTROL,top,set_uvalue=info
  
  WIDGET_CONTROL,top,/realize
  
  XMANAGER,'DSP_AUX',TOP,group_leader=group
  
  IF NOT managed THEN BEGIN
     WHILE xalive(top) DO XMANAGER
     qlds_checkin,ql_no
     assign_nocopy,qlds,qlds_saved
  END
  
END

  
