;+
; Project     : SOHO - CDS     
;                   
; Name        : DSP_CUBE
;               
; Purpose     : Display CDS QLDS data cubes any way you like it...
;               
; Explanation : DSP_CUBE uses the general data cube display widget CW_CUBEVIEW
;               to display the data from a CDS QuickLook Data Structure.
;
;               Each column in the display shows one data cube, corresponding
;               to one data window in the QLDS. You may choose which data
;               window should be shown in which window by choosing from the
;               pulldown menus at the top of each column of data (labeled
;               "Win: xxxx").
;
;               Each column consists of one image and one plot.
;
;               The image is a cross-section of your data along two of the
;               data dimensions. The plot is a cross-section of the data along
;               one dimension. Right above the image/plot windows, there are
;               buttons showing which dimension(s) have been used to form the
;               image or plot. To view the data along different dimensions,
;               click these buttons to change the current dimensions.
;
;               You move around in the data cubes by pressing the middle mouse
;               button to move the focus. When you move along a "physical"
;               dimension such as SOLAR_X, SOLAR_Y or DEL_TIME, the focus of
;               all the display columns will be changed to point to the same,
;               or "corresponding" point. The wavelength dimension is not
;               coupled in this way, and you may move independently along the
;               wavelength dimension in each column.
;
;               To zoom in/out in the image or plot, press the right/left
;               mouse button.
;
;               The color scaling of each image may be adjusted by pressing
;               the "Adjust color scaling" button.
;
;               The plot scaling (min/max etc) may be adjusted by pressing the
;               "Adjust plot scaling" button.
;
; Use         : DSP_CUBE,QLDS [,NWIN]
;    
; Inputs      : QLDS : QuickLook Data Structure
; 
; Opt. Inputs : NWIN : The number of data cubes to be displayed
;                      simultaneously. 
;               
; Outputs     : None.
;               
; Opt. Outputs: None.
;               
; Keywords    : GROUP : Set to the group leader to make the widget die when
;                       the group leader dies.
;
; Calls       : default, qlds_checkin, qlds_managed(), qlds_report, xalive()
;
; Common      : QLSAVE : See qlds_report/qlds_checkin
;               
; Restrictions: ..
;               
; Side effects: ..
;               
; Category    : CDS QuickLook, Display
;               
; Prev. Hist. : None.
;
; Written     : Stein Vidar H. Haugan, UiO, 29 May 1997
;               
; Modified    : Not yet.
;
; Version     : 1, 29 May 1997
;-            

PRO dsp_cube_event,ev
  common QLSAVE,QLDS,saved
  
  widget_control,ev.top,get_uvalue=status,/no_copy
  qlds_checkin,status.ql_no
  
  widget_control,ev.id,get_uvalue=uvalue
  
  CASE uvalue OF 
  'CWQ_CUBE':BEGIN
     focus = ev.focus
     focus(0) = -1 ;; Don't meddle with this one..
     FOR i = 0,status.nwins-1 DO BEGIN
        widget_control,status.viewers(i),set_value=focus
     END
     ENDCASE
     
  'QUIT':BEGIN
     widget_control,ev.top,/destroy
     return
     ENDCASE
     
  ELSE:BEGIN
     ENDCASE
     
  END
  
  widget_control,ev.top,set_uvalue=status,/no_copy
END


PRO dsp_cube,qlds,nwins,group=group
  common QLSAVE,sQLDS,saved
  
  default,nwins,4
  default,group,0L
  
  nwins = (nwins < 6) > 1
  
  main = widget_base(/column,group_leader=group)
  
  bbase = widget_base(main,/row)
  quit1 = widget_button(widget_base(bbase,/column),value='Quit',menu=2)
  quit2 = widget_button(quit1,value='Exit',uvalue='QUIT')
  
  dum = {CW_MOUSE_S, btext:'', mtext:'', action:'', avail:0,flags:0}
  
  m = [{CW_MOUSE_S,'Zoom out','Zoom out','ZOOM-',1,0},$
       {CW_MOUSE_S,'Focus','Focus','ZOOMP',2,0},$
       {CW_MOUSE_S,'Zoom in','Zoom in','ZOOM+',4,0}]
  
  dummy = cw_mouse(bbase,m,title='Mouse buttons')
  
  viewbase = widget_base(main,/row)
  
  validwin = where(qlds.detdesc(*).ixstop(0) GE 0,nvalid)
  
  device,get_screen_size=sz
  
  size = min((sz-[40+20*nwins,360])/[nwins,2])
  
  viewers = lonarr(nwins)
  
  IF nwins EQ 1 THEN row = 1 ELSE row = 0
  
  FOR i = 0,nwins-1 DO BEGIN
     win = validwin(i MOD nvalid)
     viewers(i) = cwq_cube(viewbase,qlds,win,uvalue='CWQ_CUBE',$
                          xsize=size,ysize=size,row=row,frame = 5)
  END
  
  managed = qlds_managed(qlds)
  keeper = $
     (managed EQ 0) AND NOT keyword_set(just_reg) $
     AND NOT keyword_set(nokeep)
  
  ql_no = qlds.ql_no
  
  ;; The qlds needs to be managed 
  qlds_report,main,qlds,store=(managed EQ 0)
  
  widget_control,main,/realize
  
  status = {ql_no:ql_no,$
            nwins:nwins,$
            viewers:viewers}
  widget_control,main,set_uvalue=status
  
  xmanager,'dsp_cube',main,just_reg=just_reg
  
  IF keeper THEN BEGIN
     WHILE xalive(main) DO xmanager
     qlds_checkin,ql_no
     qlds = temporary(sqlds)
  END
END
