;+
; Project     : SOHO - CDS     
;                   
; Name        : DSP_INFO
;               
; Purpose     : Display basic information from a CDS QL Data Structure
;               
; Explanation : Displays basic information in a window about the
;               structure containing data from a CDS FITS level-1 file.
;               Optional display of information concerning each spectral
;               window appears in a separate window, as is also the case
;               with the optional display of auxiliary data information.
;               
; Use         : DSP_INFO, QLDS
;    
; Inputs      : QLDS = A CDS QL Data Structure variable as returned by
;                       READCDSFITS(). See also "CDS QL data format".
;
; Opt. Inputs : None.
;               
; Outputs     : None.
;               
; Opt. Outputs: None.
;               
; Keywords    : ALL:    Set to display all information (basic+window+aux)
;
;               WINDOW: Set to display detector window information
;
;               AUX:    Set to display auxiliary data information
;
;               NO_BASIC:
;                       Set to suppress basic header info display
;                       (to add, e.g., AUX-window without duplicating
;                       basic header information)
;               
;               GROUP_LEADER:
;                       If this invoked by any other CDS QuickLook 
;                       display routine (group leader responsible for 
;                       keeping the QLDS), this keyword should have
;                       the widget ID of the top level base of the
;                       calling routine.
;                       If it is not set, the Xmanager is called to
;                       handle the widgets until they die..
;
;               NOCHECK:
;                       Set this to avoid calls to QLMGR to check
;                       validity of the QLDS.
;
; Calls       : detdesc(), leftlabel, qlmgr, trim()
;
; Common      : QLMGR
;
; Restrictions: The routine is intended solely for CDS QuickLook data,
;               and requires a strict adherence to the QL data format,
;               version 1.1
;
; Side effects: Creates widget windows on the screen. Does not 
;               use xmanager (only "dead" windows).
;               
; Category    : Display_Info, QuickLook, CDS
;               
; Prev. Hist. : None.
;
; Written     : Stein Vidar Hagfors Haugan, Oslo Univ., 27 September 1993
;               
; Modified    : SVHH, 4 October 1993
;                       added QLMGR call & common block to avoid 
;                       multiple window copies & keep window placings
;
;               SVHH, 21 October 1993
;                       added /NOCHECK, GROUP_LEADER keywords
;               SVHH, 29 October 1993
;                       Made the widget active, with "Done" buttons
;                       and CLEANUP keywords
;               SVHH,  3 November 1993
;                       Added display of ORIGIN in spectral window 
;                       information, added Units (Angstrom, etc.)
;                       Fixed bug for 1-dimensional (GIS) data
;               SVHH,  Version 2, 4 October 1995
;                       Changed the handling of auxiliary data.
;               SVHH, Version 2.1, 25-January-1995
;                       Took out LL_TITLE line.
;               Version 3, SVHH, 27 February 1996
;                       Modified output format.
;               Version 4, SVHH, 25 November 1996
;                       Avoid using DETDATA tag. Shrunk widgets,
;                       counteracting RSI's brilliant (NOT!) brand new
;                       interpretation of e.g., XPAD=0.
;               Version 5, SVHH, 28 April 1997
;                       Using NTRIM instead of TRIM when displaying the
;                       min/max values, to cope with byte values.
;                       
; Version     : 5, 28 April 1997
;-            

PRO dsp_info_event,ev
; The only event possible is DONE-buttons..
  Widget_CONTROL,ev.top,/DESTROY
END


;
; Procedure to aid left justification in a column base window.
;
PRO leftlabel,base,value,font
  IF since_version('4.0') THEN sml = 1 ELSE sml = 0
  localbase = Widget_BASE(base,/row,space=sml,ypad=sml,xpad=sml)
  dummy = Widget_LABEL(localbase,value=value,font=font)
  IF since_version('4.0') THEN widget_control,dummy,scr_ysize=13
END




PRO dsp_INFO,f,all=all,no_basic=no_basic,aux=aux,windows=windows,$
             GROUP_LEADER = GROUP_LEADER, font=font, NOCHECK=NOCHECK
  
  COMMON QLMGR,QL_ID,QL_Data
  
  On_Error,2                    ; For checks
  
  IF N_params() LT 1 THEN MESSAGE,"Use: dsp_info, CDS_QL_data_struct"
  
  IF NOT Keyword_SET(NOCHECK) THEN BEGIN
     QLMGR,f,valid
     IF NOT valid THEN MESSAGE,"Parameter is not a CDS QL data structure"
  EndIF
  
  On_Error,0                    ; For debugging
  
  IF NOT Keyword_SET(font) THEN font="fixed"
  IF N_elements(group_leader) EQ 0 THEN group_leader=0L
  
  IF Keyword_SET(all) THEN BEGIN
     aux=1 & windows=1
  EndIF
  
; Shortcuts
;
  h = f.header
  aa = STRING(194B)             ;Angstrom 
  
  sml = 1
  
  IF NOT Keyword_SET(NO_BASIC) THEN BEGIN
;
;   If header info is already on display, keep position 
;   and destroy old copy
;
     IF Widget_INFO(QL_Data(f.QL_no).infohdrbase,/valid_id) THEN BEGIN
        Widget_CONTROL,QL_Data(f.QL_no).infohdrbase,TLB_GET_OFFSET=offset
        Widget_CONTROL,QL_Data(f.QL_no).infohdrbase,/destroy
     END ELSE offset=[0,0]
     
     
     dsp_infobase = Widget_BASE(TITLE = "QuickLook: "+h.filename,$
                                GROUP_LEADER=GROUP_LEADER,$
                                xpad=sml,ypad=sml,space=sml,$
                                /column,xoffset=offset(0),yoffset=offset(1))
     
     QL_DATA(f.QL_NO).infohdrbase = dsp_infobase
     
     topbase = Widget_BASE(dsp_infobase,/row,frame=1,$
                           xpad=sml,ypad=sml,space=sml)
     
     base = Widget_BASE(topbase,/column,space=sml,ypad=sml,xpad=sml)
     
     leftlabel,base,"File:            "+ h.filename,font
     leftlabel,base,"Origin:          "+h.origin,font
     leftlabel,base,"Telescope:       "+h.telescop,font
     leftlabel,base,"Instrument:      "+h.instrume,font
     leftlabel,base,"Detector:        "+h.detector,font
     
     leftlabel,base,"Observation prog:"+ trim(h.obs_prog),font
     leftlabel,base,"Sci. objective:  "+ trim(h.sci_obj),font
     leftlabel,base,"Campaign name:   "+ trim(h.cmp_name),font
     leftlabel,base,"Object ID:       "+ trim(h.obj_id),font
     
     base = Widget_BASE(topbase,/column,space=sml,ypad=sml,xpad=20)
     
     leftlabel,base,"Program number:  "+ trim(h.prog_num),font
     leftlabel,base,"Program ID:      "+ trim(h.prog_id),font
     leftlabel,base,"Sequence ind:    "+ trim(h.seq_ind),font
     leftlabel,base,"Slit number:     "+ trim(h.slit_num),font
;      leftlabel,base,"Linelist:        "+ trim(h.ll_title),font
     leftlabel,base,"Exposure time:   "+ trim(h.exptime)+"s",font
     leftlabel,base,"Campaign number: "+ trim(h.cmp_no),font
     leftlabel,base,"No. of exposures:"+ trim(h.expcount),font
     
     topbase = Widget_BASE(dsp_infobase,/row,frame=1)
     base = Widget_BASE(topbase,/column,space=sml)
     
     leftlabel,base,"Start obs. date: "+ trim(h.date_obs),font
     leftlabel,base,"End obs.   date: "+ trim(h.date_end),font
     
     topbase = Widget_BASE(dsp_infobase,/row,frame=1)
     base = Widget_BASE(topbase,/column,space=sml,ypad=sml,xpad=sml)
     
;      leftlabel,base,"Center to West:  "+ trim(h.ycen)+'"',font
     leftlabel,base,"Width of image:  "+ trim(h.ixwidth)+'"',font
     leftlabel,base,"Angle (Solar N): "+ trim(h.angle),font
     leftlabel,base,"Xcenter          "+ trim(h.xcen),font
     leftlabel,base,"Ycenter          "+ trim(h.ycen),font
     
     
     base = Widget_BASE(topbase,/column,space=sml,ypad=sml,xpad=20)
     
     
     leftlabel,base,"Height of image: "+ trim(h.iywidth)+'"',font
     
     base = Widget_BASE(dsp_infobase,/column,space=sml,ypad=sml,xpad=sml)
     
     N = N_elements(f.DetDesc)
     leftlabel,base,"The file contains " + trim(N) +$
        " detector data windows",font
     
     dummy = Widget_BUTTON(base,value='Done')
     
     Widget_CONTROL,dsp_infobase,/realize
     
     Xmanager,'DSP_INFO',dsp_infobase,CLEANUP='qlmgrclean',/just_reg
     
  EndIF                         ; if NOT keyword_set(NO_BASIC) 
  
  units = f.detdesc(0).axes(*)
  fmt1 = STRMID(units,0,0)
  units(0) = aa
  ix = WHERE(STRMID(units,0,5) EQ 'SOLAR',COUNT)
  IF count GT 0 THEN units(ix) = '"'
  ix = WHERE(units EQ 'DEL_TIME',COUNT)
  IF count GT 0 THEN units(ix) = 's'
  tdim = (WHERE(units EQ 's'))(0)
  
  IF Keyword_SET(windows) THEN BEGIN
;
;   Keep position & destroy old copy
;
     IF Widget_INFO(QL_Data(f.QL_no).infowndbase,/valid_id) THEN BEGIN
        IF WIDGET_INFO(ql_data(f.ql_no).infowndbase,/realized) THEN begin
           Widget_CONTROL,QL_Data(f.QL_no).infowndbase,TLB_GET_OFFSET=offset
           Widget_CONTROL,QL_Data(f.QL_no).infowndbase,/destroy
        END ELSE offset=[400,0]
     END ELSE offset = [400,0]
     
     topbase = Widget_BASE(title="Detector Windows:"+h.filename,/column,$
                           GROUP_LEADER=GROUP_LEADER,$
                           xpad=sml,ypad=sml,space=sml,$
                           xoffset=offset(0),yoffset=offset(1))
     
     QL_DATA(f.QL_NO).infowndbase = topbase
     
     
     N = N_elements(f.DetDesc)
     dims = gt_dimension(f)
     
     Ndims = dims.ndims
     
     type = STRMID(tag_names(f.DetDesc(0),/structure_name),0,1)
     
     FOR w=0,N-1 DO BEGIN
        base = Widget_BASE(topbase,/column,frame=1,space=sml,ypad=sml)
        d = f.DetDesc(w)
        
        wtext = "Window "+trim(w+1,'(I3)',0)+" : "+trim(d.label)
        
        IF type NE 'C' THEN $
           wtext = wtext + "  Line:"+ trim(d.wavelength,'(F7.3)',0)
        
        
        leftlabel,base,wtext,font
        
        IF type NE 'C' THEN BEGIN
           wtext = "          ("
           
           FOR dim = 0,Ndims-1 DO BEGIN
              wtext = wtext+STRMID("    ",0,$
                                   10-STRLEN(f.detdesc(0).axes(dim)))
              wtext = wtext+f.detdesc(0).axes(dim)
              IF dim NE ndims-1 THEN wtext = wtext+","
           END
           wtext = wtext+')'
           leftlabel,base,wtext,font
           
           wtext = "Origin:   ("
           FOR dim = 0,Ndims-1 DO BEGIN
              wtext = wtext + trim(d.origin(dim),'(F9.2)',0)+units(dim)
              IF dim NE ndims-1 THEN wtext = wtext + ","
           END
           wtext = wtext + ")"
           leftlabel,base,wtext,font
           
           wtext = "End:      ("
           FOR dim = 0,Ndims-1 DO BEGIN
              endpoint = d.origin(dim) + $
                 d.spacing(dim)*(d.ixstop(dim)-d.ixstart(dim))
              wtext = wtext + trim(endpoint,'(F9.2)',0) + units(dim)
              IF dim NE Ndims-1 THEN wtext = wtext + ","
           EndFOR
           wtext= wtext + ")"
           leftlabel,base,wtext,font
           
           wtext = "Spacing:  ("
           FOR dim = 0,Ndims-1 DO BEGIN
              wtext = wtext + trim(d.spacing(dim),'(F9.2)',0) + units(dim)
              IF dim NE ndims-1 THEN wtext = wtext + ","
           EndFOR
           wtext= wtext + ")"
           leftlabel,base,wtext,font
           
           wtext = "Dimension:("
           FOR dim = 0,Ndims-1 DO BEGIN
              wtext = wtext + trim(d.ixstop(dim)-d.ixstart(dim)+1,'(I10)',0)
              IF dim NE ndims-1 THEN wtext = wtext + ","
           EndFOR
           wtext = wtext + ")"
           leftlabel,base,wtext,font
        END 
        
     EndFOR
     
     dummy = Widget_BUTTON(topbase,value='Done')
     
     Widget_CONTROL, topbase, /REALIZE ;create the widgets
                                ;that are defined
     
     Xmanager,'DSP_INFO',topbase,CLEANUP='qlmgrclean',/just_reg
     
  EndIF                         ; keyword set windows
  
  
  
  IF Keyword_SET(aux) THEN BEGIN
     IF Widget_INFO(QL_Data(f.QL_no).infoauxbase,/valid_id) THEN BEGIN
        Widget_CONTROL,QL_Data(f.QL_no).infoauxbase,TLB_GET_OFFSET=offset
        Widget_CONTROL,QL_Data(f.QL_no).infoauxbase,/destroy
     END ELSE offset=[800,0]
     
     topbase = Widget_BASE(title="Auxiliary data:"+h.filename,/column,$
                           GROUP_LEADER=GROUP_LEADER,$
                           xpad=sml,ypad=sml,space=sml,$
                           xoffset=offset(0),yoffset=offset(1))
     
     
     QL_DATA(f.QL_NO).infoauxbase = topbase
     
     tagnames = tag_names(f)
     i = (WHERE(tagnames EQ 'DEL_TIMEDATA'))(0)
     tagnames = tagnames(i:*)
     tagnames = tagnames(INDGEN(N_elements(tagnames)/2)*2+1)
     FOR i=0,N_elements(tagnames)-1 DO $
        tagnames(i) = STRMID(tagnames(i),0,STRPOS(tagnames(i),"DESC"))
     
     FOR w=0,N_elements(tagnames)-1 DO BEGIN
        base = Widget_BASE(topbase,/column,frame=1,space=sml,ypad=sml)
        dummy = execute("d = f." + tagnames(w)+"DESC")
        IF N_tags(d) EQ 0 THEN leftlabel,base,tagnames(w)+": N/A",font $
        ELSE BEGIN
           data =0 & dummy=execute("data = f." + tagnames(w) + "DATA")
           wtext = trim(d.label) + ": "
           wtext = wtext + "Value :" + ntrim(MIN(data))
           wtext = wtext + " .... " + ntrim(MAX(data))
           leftlabel,base,wtext,font
           sz = Size(data)
           Ndims = sz(0)
           wtext = "As a function of: ("
           FOR dim = 0,Ndims-2 DO $
              wtext = wtext + d.axes(dim)+","
           wtext = wtext + d.axes(dim-1)+")"
           wtext = wtext + "   Size: ("
           IF ndims GT 0 THEN BEGIN
              FOR dim = 0,Ndims-2 DO $
                 wtext = wtext + trim(sz(dim+1)) + ","
              wtext = wtext + trim(sz(dim)) + ")"
              
              wtext = wtext+"  Spacing: ("
              FOR dim = 0,Ndims-2 DO $
                 wtext = wtext + trim(d.spacing(dim)) + ","
              
              wtext = wtext + trim(d.spacing(Ndims-1)) + ")"
           END ELSE wtext = wtext + '0)'
           leftlabel,base,wtext,font
        EndELSE
     EndFOR
     
     dummy = Widget_BUTTON(topbase,value='Done')
     
     Widget_CONTROL, topbase, /REALIZE ;create the widgets
                                ;that are defined
     Xmanager,'DSP_INFO',topbase,CLEANUP='qlmgrclean',/just_reg
     
  EndIF
  
  IF group_leader EQ 0L THEN Xmanager
  
END
