;+
; Project     : SOHO - CDS     
;                   
; Name        : DSP_MOVIE
;               
; Purpose     : Display a movie of CDS "slit spectrograms" (SCANX/Y/T)
;               
; Explanation : If a QLDS contains data with at least three dimensions
;               it is possible to make a sequence of "slit" spectrograms
;               and play them as a "movie". Note that the sequence needn't
;               be ordered in such a way that it is the physical time that
;               plays the role of time in the movie. It is, however
;               easier to interpret a sequence that represents a succession
;               of time rather than a physical movement of the "slit".
;
;               This routine uses the GT_SCANX/Y/T routines to create
;               a succession of slit spectrograms from the data in the
;               supplied Quick Look Data Structure.
;               
; Use         : dsp_movie,qlds
;    
; Inputs      : QLDS : Quick Look Data Structure as returned by readcdsfits.
;               
; Opt. Inputs : None.
;               
; Outputs     : None.
;               
; Opt. Outputs: None.
;               
; Keywords    : NOCHECK: Set to skip testing the QLDS validity.
;
;               NOSQUARE: Set to allow a size scaling of the data
;                         that does not preserve the axis ratios.
;
; Calls       : ASSIGN_NOCOPY, DEFAULT, GT_DIMENSION(), GT_SCANT(), GT_SCANX()
;               GT_SCANY(), PARCHECK, QLMGR, TRIM(), TYP(), WIDGET_BASE()
;               WIDGET_TEXT(), XMANAGER, XMOVIE, QUICK?
;
; Common      : DSP_MOVIE_COMMON. Used to make the QLDS visible inside
;                                 the event/moviemaking procedures.
;               
; Restrictions: None.
;               
; Side effects: Displays a window while composing the picture.
;               Modal.
;               
; Category    : Quick Look, Display
;               
; Prev. Hist. : Taken partially from DSP_MENU.
;
; Written     : Stein Vidar H. Haugan, UiO, 28 February 1996
;               
; Modified    : Version 2, SVHH, 29 February 1996
;                          Changed name from dsp_scanm->dsp_movie.
;                          Packaged the whole thing into a widget
;                          thing.
;               Version 3, SVHH, 12 November 1996
;                          Added MISSING keyword in XMOVIE call.
;                          
; Version     : 3, 12 November 1996
;-            


PRO dsp_movie_roll,stat
  
  COMMON dsp_movie_common,q
  ON_ERROR,0
  
  default,xix,0
  default,yix,0
  default,tix,0
  
  ;; Strictly speaking, we should test whether the last "free dimension"
  ;; index should be asked for. Saving it for later.
  
  action = STRMID(stat.action,0,7)
  WINDOW = stat.window
  minsize = stat.minsize
  nosquare = stat.nosquare
  memlimit = stat.memlimit
  siz = FIX(STRMID(stat.action,7,15))
  
  WIDGET_CONTROL,stat.frameslider,set_value=1
  WIDGET_CONTROL,stat.frameslider,/sensitive ; Change appearance
  
  CASE STRMID(action,0,5) OF 
     'SCANX': S = gt_scanx(q,WINDOW=WINDOW,yix=yix,tix=tix,/quick)
     'SCANY': S = gt_scany(q,WINDOW=WINDOW,xix=xix,tix=tix,/quick)
     'SCANT': s = gt_scant(q,WINDOW=WINDOW,xix=xix,yix=yix,/quick)
  END
  
  sz = SIZE(s)
  movie = rebin(REFORM(s,sz(1),sz(2),1),sz(1),sz(2),siz,/sample)
  
  FOR frame = 1,siz-1 DO BEGIN
     CASE strmid(action,6,1) OF
        'T': tix = frame
        'X': xix = frame
        'Y': yix = frame
     END
     WIDGET_CONTROL,stat.frameslider,set_value=frame+1
     CASE strmid(action,0,5) OF 
        'SCANX': movie(*,*,frame) = gt_scanx(q,WINDOW=WINDOW, $
                                             yix=yix,tix=tix,/quick)
        'SCANY': movie(*,*,frame) = gt_scany(q,WINDOW=WINDOW, $
                                             xix=xix,tix=tix,/quick)
        'SCANT': movie(*,*,frame) = gt_scant(q,WINDOW=WINDOW, $
                                             xix=xix,yix=yix,/quick)
     END
  END
  
  sz = SIZE(movie)
  
  continue = 1
  WHILE continue DO begin
     
     scale = [1,1]
     IF KEYWORD_SET(nosquare) THEN BEGIN
        IF Sz(1) LT minsize THEN scale(0) = ceil(FLOAT(minsize)/sz(1))
        IF sz(2) LT minsize THEN scale(1) = ceil(FLOAT(minsize)/sz(2))
     END ELSE BEGIN
        IF sz(1) LT minsize THEN scale = scale > ceil(FLOAT(minsize)/sz(1))
        IF sz(2) LT minsize THEN scale = scale > ceil(FLOAT(minsize)/sz(2))
     END
     
     continue = 0
     
     IF total(scale NE 1) GT 0 THEN BEGIN
        m = scale(0)*scale(1)*sz(1)*sz(2)*sz(3)*([0,1,2,4,4,8])(sz(4))
        IF m GT memlimit THEN BEGIN
           minsize = minsize-1
           stat.minsize=minsize
           WIDGET_CONTROL,stat.minsizeslider,set_value=minsize
           continue = 1
        END ELSE BEGIN
           movie = rebin(movie,sz(1)*scale(0),sz(2)*scale(1),sz(3),/sample)
           continue = 0
        END
     END
  END
  
  WIDGET_CONTROL,stat.frameslider,set_value=0
  WIDGET_CONTROL,stat.frameslider,sensitive = 0; Change appearance
  xmovie,movie,missing=q.detdesc(WINDOW).missing
  
END


PRO dsp_movie_event,ev
  COMMON dsp_movie_common,q
  
  WIDGET_CONTROL,ev.top,get_uvalue = stat
  WIDGET_CONTROL,ev.id,get_uvalue = uvalue
  
  CASE STRMID(uvalue,0,8) OF 
     'MINSIZE' : stat.minsize = ev.value
     'NOSQUARE': stat.nosquare = ev.select
     'MEMLIMIT': stat.memlimit = ev.value * 2L^(20)
     'FRAME_NO': PRINT,"Don't touch me!"
     'WINDOW_N': IF ev.select THEN BEGIN
        stat.window = FIX(STRMID(uvalue,8,5))
        WIDGET_CONTROL,stat.windbutton,set_button=0
        stat.windbutton = ev.id
     END
     'ACTION_X': IF ev.select THEN BEGIN 
        stat.action = STRMID(uvalue,8,20)
        WIDGET_CONTROL,stat.modebutton,set_button=0
        stat.modebutton = ev.id
     END
     'ROLL': dsp_movie_roll,stat
     'QUIT': WIDGET_CONTROL,ev.top,/destroy
  END

  IF uvalue NE 'QUIT' THEN BEGIN
     WIDGET_CONTROL,ev.top,set_uvalue = stat
     WIDGET_CONTROL,stat.modebutton,/set_button
     WIDGET_CONTROL,stat.windbutton,/set_button
     WIDGET_CONTROL,stat.nframestext,set_value=STRMID(stat.action,7,15)
     WIDGET_CONTROL,stat.frameslider, $
        set_slider_max=trim(STRMID(stat.action,7,15))
  END
END


PRO dsp_movie,qlds,nocheck=nocheck,nosquare=nosquare,minsize=minsize, $
              memlimit=memlimit
  
  COMMON dsp_movie_common,q
  
  ON_ERROR,2
  IF !debug GT 1 THEN ON_ERROR,0
  
  IF N_PARAMS() EQ 0 THEN MESSAGE,"Use: DSP_MOVIE,QLDS"
  
  IF NOT KEYWORD_SET(nocheck) THEN begin
     qlmgr,qlds,valid
     
     IF NOT valid THEN  $
        MESSAGE,"Parameter is not a valid Quick Look Data Structure"
  END
  
  assign_nocopy,q,qlds
  
  IF N_ELEMENTS(minsize) GT 0 THEN  $
     parcheck,minsize,0,typ(/nat),0,'MINSIZE',minval=1
  
  dim = gt_dimension(q)
  
  scanx_t = 0 & scanx_y = 0
  scany_t = 0 & scany_x = 0
  scant_x = 0 & scant_y = 0
  
  IF dim.ssolar_x GT 1 THEN BEGIN
     ;; X scan is possible, can we make a movie out of it?
     IF dim.ssolar_y GT 1 THEN scanx_y = 1 ELSE scanx_y = 0
     IF dim.sdel_time GT 1 THEN scanx_t = 1 ELSE scanx_t = 0
  END
  
  IF dim.ssolar_y GT 1 THEN BEGIN
     ;; Y scan is possible, can we make a movie?
     IF dim.ssolar_x GT 1 THEN scany_x = 1 ELSE scany_x = 0
     IF dim.sdel_time GT 1 THEN scany_t = 1 ELSE scany_t = 0
  END
  
  IF dim.sdel_time GT 1 THEN BEGIN
     ;; T scan possible, movie?
     IF dim.ssolar_x GT 1 THEN scant_x = 1 ELSE scant_x = 0
     IF dim.ssolar_y GT 1 THEN scant_y = 1 ELSE scant_y = 0
  END
  
  IF NOT (scanx_y OR scanx_t OR scany_x OR  $
          scany_t OR scant_x OR scant_y) THEN BEGIN
        MESSAGE,"Sorry, no scan movie possible"
  END
  
  
  ;; Select what kind of movie to perform
  ;;
  t = ['Time series of SCANX (same point)', $     ; 0
       'Time series of SCANY (same point)', $     ; 1
       'Sequence of SCANX, moving bottom->up',$   ; 2
       'Sequence of SCANY, moving left->right',$  ; 3
       'Sequence of SCANT, moving left->right', $ ; 4
       'Sequence of SCANT, moving bottom->up'] ; 5
  
  act = ['SCANX_T', $
         'SCANY_T', $
         'SCANX_Y', $
         'SCANY_X', $
         'SCANT_X', $
         'SCANT_Y']
  
  siz = [dim.sdel_time, $
         dim.sdel_time, $
         dim.ssolar_y, $
         dim.ssolar_x, $
         dim.ssolar_x, $
         dim.ssolar_y]
  
  opt = [-1]
  IF scanx_t THEN opt = [opt,0]
  IF scanx_y THEN opt = [opt,2]
  IF scany_t THEN opt = [opt,1]
  IF scany_x THEN opt = [opt,3]
  IF scant_x THEN opt = [opt,4]
  IF scant_y THEN opt = [opt,5]
  
  opt = opt(1:*)
  
  base = WIDGET_BASE(/column,xoffset = 400,yoffset=400,title='DSP_MOVIE')
  
  innerbase = WIDGET_BASE(base,/row)
  
  leftbase = WIDGET_BASE(innerbase,/column)
  
  dummy = WIDGET_label(leftbase,value = 'Select mode of operation')
  modebase = WIDGET_BASE(leftbase,/column,/exclusive,/frame)
  FOR i = 0,N_ELEMENTS(opt)-1 DO BEGIN
     dummy = WIDGET_BUTTON(modebase,value=t(opt(i)), $
                           uvalue='ACTION_X'+act(opt(i))+trim(siz(opt(i))))
     IF i EQ 0 THEN modebutton = dummy
  END
  WIDGET_CONTROL,modebutton,/set_button  
  
  action = act(opt(0))+trim(siz(opt(0)))
     
  ;; Now, select which data window to display.
  ;;
  
  rightbase = WIDGET_BASE(innerbase,/column)
  dummy = WIDGET_LABEL(rightbase,value = 'Select wavelength window')
  
  windbase = WIDGET_BASE(rightbase,/column,/exclusive,/frame)
  FOR i = 0,N_ELEMENTS(q.detdesc)-1 DO BEGIN
     dummy = WIDGET_BUTTON(windbase,value=q.detdesc(i).label, $
                           uvalue='WINDOW_N'+trim(i))
     IF i EQ 0 THEN windbutton = dummy
  END
  WIDGET_CONTROL,windbutton,/set_button
  
  nfbase = WIDGET_BASE(leftbase,/row)
  dummy = WIDGET_LABEL(nfbase,value='No. of frames :')
  nframestext = WIDGET_TEXT(nfbase,value=trim(siz(opt(0))))
  
  default,minsize,200
  default,memlimit,4L*2L^20 ;; 1 MB
  
  dummy = WIDGET_SLIDER(base,title='Memory limit (Megabytes)', $
                        minimum=1,maximum=100,uvalue='MEMLIMIT', $
                        value=memlimit/(2L^20))
  
  minsizeslider = WIDGET_SLIDER(base,title='Minimum image size (pixels)',$
                                minimum=1,maximum=1024,uvalue='MINSIZE', $
                                value=minsize)
  
  dummy = WIDGET_BUTTON(WIDGET_BASE(base,/exclusive), $
                        value='Allow resizing to change axis ratio', $
                        uvalue='NOSQUARE')
  WIDGET_CONTROL,dummy,set_button=KEYWORD_SET(nosquare)
  
  
  frameslider = WIDGET_SLIDER(base,title='Extracting frame no', $
                              maximum=siz(opt(0)))
  
  WIDGET_CONTROL,frameslider,sensitive=0
  
  bbase = WIDGET_BASE(base,/row)
  
  dummy = WIDGET_BUTTON(bbase,value='Make movie',uvalue='ROLL')
  dummy = WIDGET_BUTTON(bbase,value='End DSP_MOVIE',uvalue='QUIT')
  
  stat = { minsize:minsize,$
           memlimit:memlimit,$
           nosquare:keyword_set(nosquare),$
           WINDOW:0,$
           windbutton:windbutton,$
           action:action,$
           modebutton:modebutton,$
           nframestext:nframestext,$
           frameslider:frameslider, $
           minsizeslider:minsizeslider}
           
  WIDGET_CONTROL,base,set_uvalue=stat
  WIDGET_CONTROL,base,/realize
  
  XMANAGER,'DSP_MOVIE',base,/modal
  
  assign_nocopy,qlds,q
END

