;+
; Project     : SOHO - CDS     
;                   
; Name        : DSP_POINT
;               
; Purpose     : Show pointing area for QLDS with IMAGE_TOOL
;               
; Explanation : This application can be used to pinpoint the area covered
;               by a CDS raster. Pointing and area information is picked
;               up from the header information in the QLDS, and is put
;               into a pointing structure as used by IMAGE_TOOL.
;
;               In addition, if XIX and/or YIX are supplied, DSP_POINT will
;               make two additional pointings in the same pointing structure,
;               forming a cross at the specified point in the raster.
;
;               The information about the pointings are also used to make
;               a Field Of View structure, FOV, which is also accepted by
;               IMAGE_TOOL. The FOV data cannot be rotation compensated by
;               IMAGE_TOOL, however.
;
;               Both the pointing structure and the FOV structure can be
;               retrieved through the POINTING_STC and FOV output keywords
;               respectively. 
;
;               SHOWING THE CENTER OF EACH EXPOSURE
;
;               If you set the keyword /UNFOLD, the center of each exposure
;               will be pinpointed (like when XIX/YIX is set), but the center
;               coordinates of each exposure will also be rotated to the time
;               of the beginning of the raster, unless the NO_ROTATION keyword
;               is set. Beware that e.g., a 6x6 GIS raster will result in a
;               total of 2x6x6 + 1 = 73 pointings with this keyword set!! That
;               takes a lot of time to plot in image_tool!
;               
;               SHOWING MULTIPLE RASTER AREAS
;               
;               To show the areas of more than one raster together, given
;               QLDS'es A, B and C, use the following sequence:
;
;               IDL> dsp_point,A,point_stc=point_stc,/accumulate
;               IDL> dsp_point,B,point_stc=point_stc,/accumulate
;               IDL> dsp_point,C,point_stc=point_stc
;
;               What DSP_POINT will do is to create a pointing structure
;               during the first call, and place it in point_stc (make sure
;               point_stc is NOT a structure the first time you call dsp_point
;               -- if it is a structure, it must be a pointing structure).
;
;               The /ACCUMULATE keyword tells dsp_point to return after
;               creating the correct pointing structure, skipping the
;               call to IMAGE_TOOL.
;
;               The second call to DSP_POINT adds the pointing of B to the
;               pointing structure given in POINT_STC.
;
;               SOLAR ROTATION COMPENSATION, MULTIPLE RASTERS
;               
;               A pointing structure contains only one observation time, so
;               DSP_POINT will rotate all observations to a common time, the
;               time of the *initial* observation supplied to DSP_POINT.
;
;               If you observe one feature over a considerable time, this is
;               the "common sense" way to treat pointings. As long as the
;               structure is moving according to plan, any pointing centered
;               on the feature will remain pointed at the feature when
;               displayed with IMAGE_TOOL, when IMAGE_TOOL time projection is
;               turned on.
;
;               The FOV structure (which cannot be rotated by IMAGE_TOOL)
;               is composed of the "static" fields of view *after* DSP_POINT
;               has rotated all pointings to a common time.
;
;               Using the fixed FOV or the pointing structure without time
;               projection will thus give you a fairly accurate view of
;               what regions of an *instantaneous* sun will be covered by
;               a sequence of rasters. Try to accumulate a pointing structure
;               with all nine synoptic rasters. If you display this pointing
;               structure without time projection, you will see the region
;               that would be covered by the synoptic *if the synoptic was
;               started at the exact time of this image*.
;
;               To check the *absolute*, *true* pointings of multiple rasters,
;               you should turn off the rotation to a common time by using the
;               /NO_ROTATION keyword in DSP_POINT. Accumulating the pointings
;               of a synoptic sequence of rasters with this keyword will give
;               you a sequence of rasters lined up on the meridian, given that
;               you turn the IMAGE_TOOL time projection off.
;
;               Using /NO_ROTATION in DSP_POINT but time projection in
;               IMAGE_TOOL does not make sense, afaik.
;               
;               
; Use         : DSP_POINT,QLDS
;    
; Inputs      : QLDS: Quick Look Data Structure.
;               
; Opt. Inputs : XIX/YIX : The X and Y indices of a point within the supplied
;                         QLDS. DSP_POINT will make two additional pointings
;                         in the pointing structure to form a "crosshair"
;                         pinpointing the location.
;
;
;               POINT_STC : If this is a structure, it is assumed to be a
;                           pointing structure, and the pointings for the
;                           current QLDS will be added to the pointings in
;                           this structure. Unless /NO_ROTATION is set, the
;                           pointings of the current QLDS will be rotated to
;                           the date_obs of the supplied POINT_STC.
;
;               FOV : If supplied, assumed to be a Field Of View structure
;                     with tags X and Y. ALL pointings in the POINT_STC after
;                     adding the current QLDS pointing will be added to the
;                     supplied FOV.
;
;                     I.e., if you use both FOV *and* POINT_STC
;                     in all calls when accumulating multiple pointings,
;                     you'll end up with a lot of uneccessary entries in
;                     FOV. If you want the correct FOV, get it as an output in
;                     the last call to DSP_PONT.
;
;               
; Outputs     : None.
;               
; Opt. Outputs: POINT_STC : The pointing structure after processing/adding
;                           the current QLDS pointing to any pre-existing
;                           pointings in the POINT_STC.
;
;               FOV : The fixed Fields-Of-View for the pointings in POINT_STC,
;                     added to any entries previously in the FOV
;                     structure. The FOV entries show the pointing coordinates
;                     at the time given by the pointing structure tag DATE_OBS
;                     (in TAI format).
;               
; Keywords    : MODAL: Modality
;
;               GROUP: Group leader. Since IMAGE_TOOL seems to dislike
;                      widget apps that do not pass on any group leader
;                      (xkills them), a dummy base is used when this is
;                      not supplied.
;
;               UNFOLD : Takes the information in the QLDS about the
;                        instrument X/Y values for each exposure and pinpoints
;                        each of them with a "crosshair".
;
;               ACCUMULATE : If set, DSP_POINT will not call IMAGE_TOOL with
;                            the resultant pointing and FOV structures, but
;                            return immediately.
;
;               NO_ROTATION : If set, DSP_POINT will not rotate the current
;                             QLDS pointing to the time of the supplied
;                             pointing structure.
;                             
; Calls       : anytim2tai(), datatype(), default, gt_dimension()
;               gt_spectrum(), image_tool, mk_point_stc, parcheck, qlmgr
;               rot_xy(), typ(), widget_base(), xack
;
; Common      : None.
;               
; Restrictions: None.
;               
; Side effects: Starts XACK/IMAGE_TOOL if /ACCUMULATE is not set.  This means
;               that if GROUP is not set, all widgets will be killed.
;               Create a dummy group leader (must be *realized*) along
;               if you don't want this to happen.
;               
; Category    : Quicklook, Display
;               
; Prev. Hist. : None.
;
; Written     : Stein Vidar H. Haugan, UiO, 23 May 1996
;               
; Modified    : Not yet, but /SHOW_POINTING keyword should be added to
;               Image_tool call when applicable. Using both FOV and
;               POINT_STC for now.
;
;               Version 2, SVHH, 15 August 1996
;                       Added the stuff that accumulates pointings from more
;                       than one raster, and corrected a problem that caused
;                       image_tool to be killed immediately after launch when
;                       it was not modal.
;               Version 3, SVHH, 15 August 1996
;                       Decided not to interfere with xack/image_tool xkill
;                       policy, especially since xack required the group
;                       leader to be realized as well as alive.
;               Version 4, SVHH, 20 August 1996
;                       Added /UNFOLD feature.
;               
; Version     : 4, 20 August 1996
;-            


PRO dsp_point,qlds,modal=modal,group=group,xix=xix,yix=yix,$
              accumulate=accumulate,fov=fov,point_stc=point_stc,$
              no_rotation=no_rotation,soho_view=soho_view,unfold=unfold
  
  ON_ERROR,2
  
  IF N_PARAMS() NE 1 THEN MESSAGE,"Use: DSP_POINT, QLDS"
  
  qlmgr,qlds,valid
  IF NOT valid THEN MESSAGE,"Input parameter must be a valid QLDS"
  
  default,soho_view,1
  unfold = KEYWORD_SET(unfold)
  
  pinpoint = 0
  n_pointings = 1
  
  dim = gt_dimension(qlds)
  IF (N_elements(xix)+N_ELEMENTS(yix)) GT 0 THEN BEGIN
     default,xix,dim.ssolar_x/2
     default,yix,dim.ssolar_y/2
     parcheck,xix,0,typ(/rea),0,'XIX',minval=0,maxval=(dim.ssolar_x-1) > 0
     parcheck,yix,0,typ(/rea),0,'YIX',minval=0,maxval=(dim.ssolar_y-1) > 0
     validwin = (WHERE(qlds.detdesc(*).ixstop(0) GE 0))(0)
     IF validwin EQ -1 THEN PRINT,"Cannot get dummy spectrum without data"
     dummy = gt_spectrum(qlds,WINDOW=validwin,xix=xix,yix=yix, $
                         xsolar=xpt,ysolar=ypt)
     xpt = xpt(0)
     ypt = ypt(0)
     pinpoint = 1
     n_pointings = 3
  END
  
  IF unfold THEN BEGIN
     nexp = N_ELEMENTS(qlds.del_timedata)
     n_pointings = n_pointings + 2*nexp
  END
  
  mk_point_stc,point,n_pointings=n_pointings
  
  point.instrume = qlds.header.instrume
  point.g_label = qlds.header.filename
  point.date_obs = anytim2tai(qlds.header.date_obs)
  point.do_pointing = 0
  pt = point.pointings(0)
  pt.ins_x = qlds.header.xcen
  pt.ins_y = qlds.header.ycen
  pt.width = qlds.header.ixwidth
  pt.height = qlds.header.iywidth
  point.pointings(0) = pt
  
  offset = 1
  
  IF pinpoint THEN BEGIN
     validwin = (WHERE(qlds.detdesc(*).ixstop(0) GE 0))(0)
     spec = gt_spectrum(qlds,window=validwin,xix=xix,yix=yix)
     pt = point.pointings(0)
     pt.ins_x = xpt
     pt.ins_y = ypt
     pt.width = .5
     pt.height = qlds.header.iywidth
     point.pointings(1) = pt
     
     pt.height = .5
     pt.width = qlds.header.ixwidth
     point.pointings(2) = pt
     pt = point.pointings(0)
     offset = 2
  END
  
  IF unfold THEN BEGIN
     
     FOR i = 0,nexp-1 DO begin
        pt = point.pointings(2*i+offset)
        x = qlds.ins_xdata(i)
        y = qlds.ins_ydata(i)
        time = point.date_obs + qlds.del_timedata(i)
        IF NOT KEYWORD_SET(no_rotation) THEN BEGIN
           xy = rot_xy(x,y,tstart=time,tend=point.date_obs,soho=soho_view)
           x = xy(0)
           y = xy(1)
        END
        pt.ins_x = x
        pt.ins_y = y
        pt.width = .5
        pt.height = qlds.header.iywidth
        point.pointings(2*i+offset) = pt
        pt.width = qlds.header.ixwidth
        pt.height = .5
        point.pointings(2*i+offset+1) = pt
     END
     
  END
  
  IF DATATYPE(POINT_STC) EQ 'STC' THEN BEGIN
     n_old = N_ELEMENTS(point_stc.pointings)
     n_new = N_ELEMENTS(point.pointings)
     
     n_pointings = n_old+n_new
     
     mk_point_stc,new_p,N_pointings = n_pointings
     
     new_p.instrume = point_stc.instrume
     new_p.g_label = point_stc.g_label
     new_p.date_obs = point_stc.date_obs
     new_p.do_pointing = 0
     
     new_p.pointings(0:n_old-1) = point_stc.pointings
     
     tstart = point.date_obs
     tend = point_stc.date_obs
     
     FOR i = 0,n_new-1 DO BEGIN
        pt = point.pointings(i)
        IF NOT KEYWORD_SET(no_rotation) THEN $
           xy = rot_xy(pt.ins_x,pt.ins_y,tstart=tstart, $
                       tend=tend,soho=soho_view) $
        ELSE xy = [pt.ins_x,pt.ins_y]
        
        pt.ins_x = xy(0)
        pt.ins_y = xy(1)
        new_p.pointings(n_old+i) = pt
     END
     point = new_p
  END
  
  
  ;; Output parameter
  point_stc = point
  
  pt = point.pointings(0)
  
  x = pt.ins_x + pt.width*0.5* [-1, 1,1,-1,-1]
  y = pt.ins_y + pt.height*0.5*[-1,-1,1, 1,-1]
  
  FOR i=1,N_POINTINGS-1 DO BEGIN
     pt = point.pointings(i)
     x = [x, pt.ins_x + pt.width*0.5* [-1, 1,1,-1,-1]]
     y = [y, pt.ins_y + pt.height*0.5*[-1,-1,1, 1,-1]]
  END
  
  IF NOT KEYWORD_SET(accumulate) THEN BEGIN
     xack,['To display raster area when Image Tool appears, select',$
           '',$
           '   Pointing : Run Pointing Tool (from menu at the top)',$
           '',$
           'and then ',$
           '',$
           '   Plot : All Pointing Areas (inside Pointing Tool box)',$
           '',$
           'Alternatively, you could turn on ',$
           '',$
           '   Pointing : Show Fixed Field of View',$
           '',$
           'To exit press "Exit Pointing Tool" and then File : Quit'],$
        group = group
  END
  
  
     
  IF datatype(fov) EQ 'STC' THEN BEGIN
     PRINT,"Adding this FOV to the existing FOV"
     x = [fov.x,x]
     y = [fov.y,y]
  END
  
  fov = {x:x,y:y}
  
  IF NOT KEYWORD_SET(ACCUMULATE) THEN BEGIN
     image_tool,point_stc = point,modal=modal,group=group,fov=fov
  END
  
END


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; End of 'dsp_point.pro'.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
