;+
; Project     : SOHO - CDS     
;                   
; Name        : DSP_WAV
;               
; Purpose     : Show images at given wavelengths + spectra at given point
;               
; Explanation : The displayed image is taken at the wavelength indicated by
;               the plotting symbol in the spectrum below. The displayed
;               spectrum is taken at the point indicated by the cross-hair
;               in the displayed image.
;
;               You can zoom in and out and move the points by using the mouse
;               buttons inside the plot windows.
;
;               Try anything. *IF* anything should crash, type
;
;                  IDL> @recover
;
;               to avoid having to read in the QLDS over again.
;               
; Use         : dsp_wav,qlds [, NWINDOWS]
;    
; Inputs      : QLDS : Quick look data structure read in by readcdsfits.
;               
; Opt. Inputs : NWINDOWS : The number of line windows to be displayed.
;               
; Outputs     : None.
;               
; Opt. Outputs: None.
;               
; Keywords    : NOCHECK: Don't bother checking whether the QLDS is a QLDS.
;
;               GROUP_LEADER: The group leader.
;
;               NOKEEP: Don't store the quick look data structure in the quick
;                       look memory management system (see qlds_report and
;                       qlds_checkin). The group leader must be responsible
;                       for having the QLDS available during event processing
;                       if this keyword is set.
;
; Calls       : break_file cleanplot clipbox default ps psclose psplot
;               qlds_checkin xack xps_setup qlds_checkin dsp_point break_file
;               cleanplot cw_loadct() cw_mouse() cw_pselect() cwq_dspwav()
;               default detdesc() dsp_wav_point_update* parcheck qlds_fetch
;               qlds_report qlmgr since_version() trim() typ() widget_base()
;               xtvscale()
;
; Common      : QLSAVE : See e.g., QLDS_CHECKIN.
;
;               DSP_WAV_XPS_SETUP : To keep XPS_SETUP stuff between calls.
;               
; Restrictions: None.
;               
; Side effects: None known.
;               
; Category    : QuickLook, Display
;               
; Prev. Hist. : Based on dspwav, completely rewritten, though the layout has
;               not changed a lot.
;
; Written     : S.V.H.Haugan, UiO, June 1996 (s.v.h.haugan@astro.uio.no)
;               
; Modified    : Version 2, SVHH, 18 June 1996
;                       Added keywords NOCHECK, GROUP_LEADER and NOKEEP to fit
;                       the routine in with the rest of QL software.
;
;               Version 3, SVHH, 10 July 1996
;                       Added possibility for fixed pattern smoothing for
;                       GIS data.
;
;               Version 4, SVHH, 2 August 1996
;                       Added some cleanplot statements + buttons, should
;                       be able to refresh image automatically.
;                       Added CW_LOADCT and rehashed layout slightly.
;                       Default XTVSCALE action now uses CDS_CLEAN_IMAGE.
;               Version 5, SVHH, 15 August 1996
;                       Added XIX/YIX keywords in dsp_point call, dropped
;                       making it modal.
;               Version 6, SVHH, 16 October 1996
;                       Modified IDL v 4.0(.1) widgets to save space.
;		Version 7, 10-Dec-1997, William Thompson, GSFC
;			Use AMOEBA_C instead of AMOEBA, to avoid conflict with
;			IDL/v5 routine of that name.
;
; Version     : 7, 10-Dec-1997
;-            


;
; PAPER copy
;
PRO DSP_WAV_PRINT,info,group,goahead=goahead
  COMMON qlsave,qlds,saved
  COMMON DSP_WAV2_XPS_SETUP,setup,count

  qlds_checkin,info.ql_no
   
;   
; Printer setup/defaults
;
  
  ;; Get information about the XIX,YIX, and zoom status.
  WIDGET_CONTROL,info.int.imzoom_id,get_value=zoomt
  
  WIDGET_CONTROL,info.int.xix_id,get_value=xixt
  WIDGET_CONTROL,info.int.yix_id,get_value=yixt
  ;;WIDGET_CONTROL,info.int.tix_id,get_value=tixt
  tixt = '0'
  
  WIDGET_CONTROL,info.int.solx_id,get_value=solxt
  WIDGET_CONTROL,info.int.soly_id,get_value=solyt
  
  ;; dummy = temporary(setup) ;; Deletes setup
  
  fitsfile = (str_sep(qlds.header.filename,'.'))(0)
  psfile = ("%FILE_dsp_wav_z%ZOOM_x%XIX_y%YIX") ;; "_t%tix"

  IF datatype(setup) NE 'STC' THEN BEGIN
     xps_setup,setup,/initial
     setup.printer = getenv("PSLASER")
     IF setup.printer EQ '' THEN setup.printer = getenv("PSCOLOR")
     IF setup.printer EQ '' THEN setup.printer = getenv("PRINTER")
     IF setup.printer EQ '' THEN setup.printer = 'lp0'
     setup.color = 1
     break_file,setup.filename,disk,dir,filnam,ext
     setup.filename = disk+dir+psfile+ext
  END
  
  NWAVES = N_ELEMENTS(info.int.dspwavs)
  
  setup.portrait = 0
  xmargin = 0.02
  
  IF nwaves LE 2 THEN setup.portrait = 1
  
  IF NOT KEYWORD_SET(goahead) THEN BEGIN
     xack,group=group,$
        ["You can use the following special symbols in the PS file name:",$
         "",$
         "  %XIX, %YIX, %TIX, %ZOOM will be substituted with current values",$
         "  %FILE will be substituted with '"+fitsfile+"'",$
         "  %COUNT will be substituted with a counter value, and the",$
         "         counter will be incremented"]
     xps_setup,setup,status=status,group=group ;; To avoid XKILL,/ALL!!
     WIDGET_CONTROL,group,map=1
     IF NOT status THEN RETURN
  END
  
  filename = repstr(setup.filename,'%FILE',fitsfile)
  filename = repstr(filename,'%ZOOM',zoomt)
  filename = repstr(filename,'%XIX',xixt)
  filename = repstr(filename,'%YIX',yixt)
  filename = repstr(filename,'%TIX',tixt)
  
  IF STRPOS(filename,'%COUNT') GE 0 THEN BEGIN
     default,count,1
     filename = repstr(filename,'%COUNT',trim(count))
     count = count+1
  END
  
;
; FINALLY, setup completed
;
  
  ps,filename,encapsulated=setup.encapsulated, $
     portrait=setup.portrait, color=setup.color,copy=setup.copy, $
     interpolate=setup.interpolate
  
  cleanplot
;
; How much space do we have to play with?
;
  
  x0 = 0
  y0 = 0
  x1 = !D.X_SIZE
  y1 = !D.Y_SIZE
  
  !P.clip = [x0,y0,x1,y1]
  clipbox
  
;  
; Want boxratio:1 boxes, with some room at the top
;
  
  boxratio = 2.5
  
  topfrac = 0.15 ;; Top fraction of page used for general info
  xmargin = 0.02
  ymargin = 0.02
  
;
; Reserve room at the top (texts etc):
;  
  topheight = (y1-y0)*topfrac
  x0top = x0 + xmargin*(x1-x0)
  y0top = y1 - topheight + ymargin*topheight
  x1top = x1 - xmargin*(x1-x0)
  y1top = y1 - ymargin*topheight
  
;
; Mark it
;
  !P.clip = [x0top,y0top,x1top,y1top]
  clipbox
  
;
; Texts to be displayed
;
  
  titlel = "Title: "+qlds.header.title
  utc = anytim2utc(qlds.header.date_obs)
  days = ["Mond","Tues","Wednes","Thurs","Fri","Satur","Sun"]+"day"
  day = days((utc.mjd+2) MOD 7)
  filel = "File: "+fitsfile+"  "+day+"  "+qlds.header.date_obs
  curposl = "Current position: XIX="+xixt+"  YIX="+yixt+"  TIX="+tixt
  curposl = curposl + "  Zoom factor: "+zoomt
  solarl = "Solar X: "+solxt+"  Solar Y: "+solyt
  
  lheight = !D.y_ch_size * 1.5
  
  ;; Descriptive text in top panel
  txt = [titlel, filel, curposl, solarl]
;
; Where to put it
;
  xtx = x0top + 0.1*(x1top-x0top)
  ytx = y1top - lheight
  
  FOR i=0,N_ELEMENTS(txt)-1 DO BEGIN
     xyouts,xtx,ytx,txt(i),/DEVICE
     ytx = ytx-lheight
  END
  
;  
; Room below the top
;
  botheight = (y1-y0)*(1.0-topfrac)
  x0bot = x0top
  y0bot = y0 + ymargin*botheight
  x1bot = x1top
  y1bot = y0 + botheight - ymargin*botheight
  
  !P.clip = [x0bot,y0bot,x1bot,y1bot]
  clipbox
  
;
; Distance between the centers of the cwq_dspwav panels
;                                 
  centerdist = (x1bot-x0bot)/nwaves
  
;
; Maximum sizes
;
                                
  maxwidth = (1.0-xmargin)*(x1bot-x0bot)/nwaves
  maxheight = (1.0-ymargin)*(y1bot-y0bot)
  
;
; Keep the height:width ratios, but within maximum sizes
;
  height = maxheight < boxratio*maxwidth
  width = maxwidth < (height/boxratio)
  
;
; Center positions of each panel
; 
  xpositions = x0bot + (FINDGEN(nwaves)+0.5)*(x1bot-x0bot)/nwaves
  yposition = 0.5*(y0bot+y1bot)
  
;
; Structure that signals a request for postscript plotting.
;
  psset = {psplot:1}
  
;
; Tell everyone about their place, and plot.
;
  FOR i = 0,NWAVES-1 DO BEGIN
     x0box = xpositions(i)-0.5*width
     y0box = yposition-0.5*height
     x1box = xpositions(i)+0.5*width
     y1box = yposition+0.5*height
     !P.clip = [x0box,y0box,x1box,y1box]
     clipbox
     WIDGET_CONTROL,info.int.dspwavs(i),set_value=psset
  END
  
;
; Close up and return.
;
  
  IF setup.hard THEN $
     psplot,filename,queue=setup.printer,delete=setup.delete $
  ELSE  $
     psclose
  
  RETURN
END


;
; Update the status information panel with XIX/YIX/SOLAR_X/Y etc.
; 

PRO dsp_wav_point_update,info,event
  COMMON qlsave,qlds,saved
  
  qlds_checkin,info.ql_no
  
  IF exist(event) THEN BEGIN
     FOR i = 0,N_ELEMENTS(info.int.dspwavs)-1 DO $
        WIDGET_CONTROL,info.int.dspwavs(i),set_value=event.set
  END
  
  ;; Get information back
  
  WIDGET_CONTROL,info.int.dspwavs(0),get_value=value
  
  ;; Get xix/yix/tix  values etc.
  
  imzoom = value.imzoom
  xix = value.xix
  yix = value.yix
  tix = 0
  
  solar_x = (qlds.detdesc(0).origin(1) +  $
             qlds.detdesc(0).spacing(1) * xix)
  solar_y = (qlds.detdesc(0).origin(2) +  $
             qlds.detdesc(0).spacing(2) * yix)
  
  WIDGET_CONTROL,info.int.xix_id,set_value=trim(xix)
  WIDGET_CONTROL,info.int.yix_id,set_value=trim(yix)
  ;;WIDGET_CONTROL,into.int.tix_id,set_value=trim(tix)
  WIDGET_CONTROL,info.int.solx_id,set_value=trim(solar_x,'(f7.1)')
  WIDGET_CONTROL,info.int.soly_id,set_value=trim(solar_y,'(f7.1)')
  WIDGET_CONTROL,info.int.imzoom_id,set_value=trim(imzoom)
END


PRO dsp_wav_event,ev
  COMMON qlsave,qlds,saved
  qlds_checkin,widget=ev.top
  WIDGET_CONTROL,ev.id,get_uvalue=uval
  storage = WIDGET_INFO(ev.top,/child)
  
  WIDGET_CONTROL,storage,get_uvalue=info
  
  WIDGET_CONTROL,ev.top,/hourglass
  
  CASE uval OF 
     
     "QUIT": BEGIN 
        WIDGET_CONTROL,EV.TOP,/DESTROY
        RETURN
     END
     
     "BANDWIDTH": BEGIN
        uvs = "BAND_"+["LOW","HIGH"]
        IF ev.select THEN BEGIN
           WIDGET_CONTROL,info.int.cwmouse2,set_value={enable:uvs}
           WIDGET_CONTROL,info.int.cwmouse2,set_value=[uvs(0),'',uvs(1)]
        END ELSE BEGIN
           WIDGET_CONTROL,info.int.cwmouse2,set_value={disable:uvs}
           WIDGET_CONTROL,info.int.cwmouse2,set_value=['ZOOM-','','ZOOM+']
        END
        info.ext.bandwidth = ev.select
        FOR i = 0,N_ELEMENTS(info.int.dspwavs)-1 DO $
           WIDGET_CONTROL,info.int.dspwavs(i),set_value={bandwidth:ev.select}
     END
     
     "PRINT":dsp_wav_print,info,ev.top
     
     "CLEANPLOT":cleanplot ;; Display update should be done as well..
     
     "HELP": widg_help,'dsp_wav',title = 'Help: DSP_WAV',group=ev.top
     
     "IMTOOL": BEGIN
        WIDGET_CONTROL,info.int.dspwavs(0),get_value=value
        dsp_point,qlds,group=ev.top,xix=value.xix,yix=value.yix
     END
  
     "XTVSCALE": dummy = xtvscale(info.int.xtvscale,/map,/show,iconify=0)
     
     "CWQ_DSPWAV": dsp_wav_point_update,info,ev
     
     "LOGARITHMIC": FOR I = 0,N_ELEMENTS(info.int.dspwavs)-1 DO $
        WIDGET_CONTROL,info.int.dspwavs(i),set_value={logarithmic:ev.select}
     
     "SMOOTH": BEGIN
        smooth = ev.value*2 + 1
        IF ev.drag EQ 0 THEN FOR I = 0,N_ELEMENTS(info.int.dspwavs)-1 DO $
           WIDGET_CONTROL,info.int.dspwavs(i),set_value={smooth:smooth}
        WIDGET_CONTROL,info.int.smooth_id,set_value=trim(smooth)
     END
     
; Line fit functions     
     
     "NOFIT": FOR i=0,N_ELEMENTS(info.int.dspwavs)-1 DO $
        WIDGET_CONTROL,info.int.dspwavs(i),set_value={linefit:"NONE"}
     
     "CURVEFIT": FOR i=0,N_ELEMENTS(info.int.dspwavs)-1 DO $
        WIDGET_CONTROL,info.int.dspwavs(i),set_value={linefit:"curvefit"}
     
     "LSTSQR": FOR i=0,N_ELEMENTS(info.int.dspwavs)-1 DO $
        WIDGET_CONTROL,info.int.dspwavs(i),set_value={linefit:"lstsqr"}
     
     "GAUSSFIT": FOR i=0,N_ELEMENTS(info.int.dspwavs)-1 DO $
        WIDGET_CONTROL,info.int.dspwavs(i),set_value={linefit:"gaussfit"}
     
     "CDSFITWAVE": FOR i=0,N_ELEMENTS(info.int.dspwavs)-1 DO $
        WIDGET_CONTROL,info.int.dspwavs(i),set_value={linefit:"cds_fit_wave"}
  
     "AMOEBA_C": FOR i=0,N_ELEMENTS(info.int.dspwavs)-1 DO $
        WIDGET_CONTROL,info.int.dspwavs(i),set_value={linefit:"amoeba_c"}
  END
  
  WIDGET_CONTROL,storage,set_uvalue=info
  
END


;
;
;

PRO dsp_wav,q,nwindows,group_leader=group_leader, $
            nokeep=nokeep,nocheck=nocheck
 
  COMMON qlsave,qlds,saved
  
  cleanplot
  
  IF N_PARAMS() LT 1 THEN MESSAGE,"Use: DSP_WAV,QLDS [,nwindows]"
  
  default,NWINDOWS,4
  default,group_leader,0L
  
  IF NOT KEYWORD_SET(nocheck) THEN BEGIN 
     qlmgr,q,valid
     IF NOT valid THEN MESSAGE,"First parameter must be a QLDS"
  END
  
  parcheck,nwindows,2,typ(/nat),0,'NWINDOWS',MINVAL=1,MAXVAL=7
  
  IF since_version('4.0') THEN sp = 1 ELSE sp = 1
  
  small = {xpad:sp,ypad:sp,space:sp}
  
  IF since_version('4.0') THEN smallr = create_struct(small,'scr_ysize',22) $
  ELSE                         smallr = small
  
  IF since_version('4.0') THEN smalls = create_struct(small,'scr_ysize',19) $
  ELSE                         smalls = small
  
  main = WIDGET_BASE(/column,title=q.header.title,$
                     group_leader=group_leader,_extra=small)
;  WIDGET_CONTROL,main,default_font='fixed'
  
  global = WIDGET_BASE(main,/row,_extra=small)
  
  ;; All cwq_dspwavs go here.
  compound = WIDGET_BASE(main,/row,_extra=small)
     
  ;; Mouse buttons left
  mouseb = WIDGET_BASE(global,/column,_extra=small)
  
  ;; Menu buttons to the right
  buttons = WIDGET_BASE(global,/column,_extra=small)
  slim1 = WIDGET_BASE(buttons,/row,_extra=smallr)
  slim2 = WIDGET_BASE(buttons,/row,_extra=small)
  linefitbase = WIDGET_BASE(slim2)
  logbase = WIDGET_BASE(slim2,/nonexclusive,/column,_extra=small)
  
  
  ;; Status panel below menu buttons, with color bar on the side
  status_color = WIDGET_BASE(buttons,/row,_extra=small)
  status = WIDGET_BASE(status_color,/column,/frame,_extra=small)
  stat1 = WIDGET_BASE(status,/row,_extra=smalls)
  stat2 = WIDGET_BASE(status,/row,_extra=smalls)
  stat3 = WIDGET_BASE(status,/row,_extra=smalls)
  
  IF q.header.detector EQ 'GIS' THEN  $
     stat4 = WIDGET_BASE(status,/row,_extra=small)
  
  dummy = cw_loadct(status_color,frame=0,ysize=15)
  
  
  ;; Mouse buttons on top of each other
  
  dummy = {CW_MOUSE_S,btext:'',mtext:'',action:'',avail:0,flags:0}
  mouse1 = [{cw_mouse_s,'Zoom out','Zoom out','ZOOM-',1,0},$
            {cw_mouse_s,'Set focus','Set focus','ZOOMP',2,0},$
            {cw_mouse_s,'Zoom in','Zoom in','ZOOM+',4,0}]
  cwmouse1 = cw_mouse(mouseb,mouse1,title='Mouse buttons (IMAGE)')
  mouse2 = [mouse1,$
            {cw_mouse_s,'Low limit','Lower wavelength integration limit', $
             'BAND_LOW',1,0},$
            {cw_mouse_s,'Upper limit','Upper wavelength integration limit',$
             'BAND_HIGH',4,0}]
  cwmouse2 = cw_mouse(mouseb,mouse2,title='Mouse buttons (spectral plot)')
  WIDGET_CONTROL,cwmouse2,set_value=['','','ZOOM+']
  WIDGET_CONTROL,cwmouse2,set_value={disable:"BAND_"+["HIGH","LOW"]}
  
  ;; Button menu
  
  quit = WIDGET_BUTTON(slim1,value='Quit',uvalue='QUIT')
  PRINT = WIDGET_BUTTON(slim1,value='Print',uvalue='PRINT')
  help = WIDGET_BUTTON(slim1,value='Help',uvalue='HELP')
  showarea = WIDGET_BUTTON(slim1,value='Image_Tool',uvalue='IMTOOL')
  
  ;; Scaling button
  xtvscaleb = WIDGET_BUTTON(slim1,value='Intensity scaling', $
                            uvalue='XTVSCALE')
  
  clnplotb = WIDGET_BUTTON(slim1,value='Reset !P,!X,!Y',uvalue='CLEANPLOT')
  
  logarithmic = WIDGET_BUTTON(logbase,value='Logarithmic profile plot', $
                              uvalue='LOGARITHMIC')
;  integrate = WIDGET_BUTTON(logbase,value='Bandwidth integrated image',$
;                            uvalue='BANDWIDTH')
  
  smooth_id = 0
  
  IF q.header.detector EQ 'GIS' THEN BEGIN
     label = WIDGET_LABEL(stat4,value='Fix.pattern smooth on spectrum:')
     smooth_id = WIDGET_LABEL(stat4,value='1')
     smooth_slide = WIDGET_SLIDER(stat4,minimum=0,maximum=4,uvalue="SMOOTH",$
                                  /suppress_value,value=0,/DRAG)
  END
  ;; Status panel:
  
  break_file,q.header.filename,disk_log,dir,filnam,ext
  
  dummy = WIDGET_LABEL(stat1,value = 'File : '+filnam)
  dummy = WIDGET_LABEL(stat1,value = '  Image zoom factor:')
  imzoom_id = WIDGET_LABEL(stat1,value = 'xxx')
  
  ;;dummy = WIDGET_LABEL(stat1,value='  Title:'+q.header.title)
  dummy = WIDGET_LABEL(stat2,value='Current position: XIX=')
  xix_id = WIDGET_LABEL(stat2,value='xxx')
  dummy = WIDGET_LABEL(stat2,value=' YIX=')
  yix_id = WIDGET_LABEL(stat2,value='xxx')
  ;;dummy = WIDGET_LABEL(stat2,value=' TIX=')
  ;;yix_id = widget_label(stat2,value='xxx')
  
  dummy = WIDGET_LABEL(stat3,value='Solar X:')
  solx_id = WIDGET_LABEL(stat3,value='xxx')
  dummy = WIDGET_LABEL(stat3,value='  Solar Y:')
  soly_id = WIDGET_LABEL(stat3,value='xxx')

  ;; Line fit menu:
  ;;
  fixed = 'Gaussian + fixed bgnd'
  full =  'Gaussian    +bgnd+lin.+quad.'
  full2 = 'Gaussian(s) +bgnd+lin.+quad.'
  linemen =  $
     [{PSELECT_S,btext:'NONE',mtext:'None',uvalue:'NOFIT',flags:0},$
      {PSELECT_S,'xxxx',fixed,'xxxx',1},$
      {PSELECT_S,'Curvefit','Curvefit','CURVEFIT',0},$
      {PSELECT_S,'Lstsqr',  'Lstsqr  ','LSTSQR',0},$
      {PSELECT_S,'Amoeba_C','Amoeba_C','AMOEBA_C',2},$
      {PSELECT_S,'xxxx',full,'xxxx',1},$
      {PSELECT_S,'Gaussfit','Gaussfit','GAUSSFIT',2},$ $
      {PSELECT_S,'xxxx',full2,'xxxx',1},$
      {PSELECT_S,'Cds_fit_wave','Cds_fit_wave','CDSFITWAVE',2}]
  dummy = cw_pselect(linefitbase,'Line fit:',linemen)
  
  IF nwindows GE 5 THEN pixels = round((220*5.0)/nwindows) $
  ELSE IF nwindows GT 2 THEN pixels = 250 $
  ELSE                  pixels = 300

  ;; Image scaling
  missing = q.detdesc(0).missing
  program = [';cds_clean_image,data',$
            'bscale,data,missing='+trim(missing)]
  xtvscale = xtvscale(map=0,group_leader=main, missing=missing,$
                      auto_missing=1,program=program,  $
                      /cds_clean_image, sigrange=0, $
                      title='Default scaling, all windows')
  
  ;; DSPWAV compound widgets
  
  dspwavs = lonarr(NWINDOWS)
  windowi = 0
  FOR i=0,NWINDOWS-1 DO BEGIN
     dspwavs(i) = cwq_dspwav(compound,q,uvalue='CWQ_DSPWAV', $
                             xtvscale=xtvscale,column=(nwindows GT 1),$
                             number=i+1,pixels=pixels,$
                             windowi=windowi,$
                             cw_mouse_image=cwmouse1,$
                             cw_mouse_plot=cwmouse2)
     windowi = (windowi+1) MOD N_ELEMENTS(q.detdesc)
  END
  
  ;; Get xix/yix/tix  values etc.
  
  
  QL_NO = Q.QL_NO
  
  qlds_report,MAIN,q,store = (KEYWORD_SET(nokeep) EQ 0)
  
  ext = { dummy : 0,$
          bandwidth : 0 $ ;; Default is off
        }
  
  int = { dspwavs: dspwavs,$
          cwmouse2 : cwmouse2,$
          imzoom_id : imzoom_id,$
          xix_id : xix_id,$
          yix_id : yix_id,$
          $;; tix_id : tix_id,$
          solx_id : solx_id,$
          soly_id : soly_id,$
          smooth_id : smooth_id,$
          xtvscale: xtvscale}
  
  info = { ql_no: ql_no, $
           ext : ext, $
           int : int }
  
  dsp_wav_point_update,info
  
  WIDGET_CONTROL,global,set_uvalue=info,/no_copy
  WIDGET_CONTROL,MAIN,/realize
  
  IF KEYWORD_SET(nokeep) THEN BEGIN
     XMANAGER,'DSP_WAV',main,/just_reg
     RETURN
  END 
  
  XMANAGER,'DSP_WAV',MAIN,/MODAL
  
  QLDS_FETCH,ql_no,q
END


