;+
; Project     : SOHO - CDS     
;                   
; Name        : DSPEXP
;               
; Purpose     : Display all exposures of selected Spectral Windows.
;               
; Explanation : A window is created to show all the exposures of
;               the selected Spectral Windows, spread out one after
;               another, and to report the identity of the
;               Spectral Window/exposure number that is clicked on
;               by the user.
;
;       To do:  Logarithmic scaling of data.
;               Interactive selection of windows to be displayed,
;               More information to the status line when clicking.
;               
;               
; Use         : DSPEXP,QLDS,WINDOWS
;    
; Inputs      : QLDS:   CDS QuickLook data structure.
;
; Opt. Inputs : WINDOWS:Array containing the numbers of the
;                       detector data windows to be displayed.
;               
; Outputs     : None.
;               
; Opt. Outputs: None.
;               
; Keywords    : GROUP_LEADER:
;                       If this invoked by any other CDS QuickLook 
;                       display routine (group leader responsible for 
;                       keeping the QLDS), this keyword should have
;                       the widget ID of the top level base of the
;                       calling routine.
;
;               NOCHECK:
;                       Set this to avoid calls to QLMGR to check
;                       validity of the QLDS.
;
; Calls       : AMOEBA_C, ASSIGN_NOCOPY, BELL, BREAK_FILE, CDSCONGRID(), CDSLOG,
;               CDSNOTIFY, CDS_FIT_WAVE(), CLIPBOX, CMOUSE, CMOUSE_ACTION(),
;               CURVEFIT(), DATATYPE(), DEFAULT, EXPSLICE(), GT_DIMENSION(),
;               GT_SCANY(), GT_WINDATA(), LOAD_RED, LSTSQR, NEXPOSURES(),
;               PCONVERT(), PFIND(), PQLPROFILE, PQLZOOM, PS, PSCLOSE, PSPLOT,
;               PSTORE(), QLDS_CHECKIN, QLDS_REPORT, QLMGR, QL_BSCALE(),
;               RESET, SETWINDOW, TRIM(), WAVESIZE(), WIDGET_BASE(),
;               WIDGET_TEXT(), WIDG_HELP, WINDOWNO(), WMAX(), XMANAGER,
;               XPS_SETUP, XSELECT, ZSIZE()
;
; Common      : QLSAVE: See QLDS_CHECKIN
;  
; Restrictions: QLDS must adhere to "CDS QL data format"
;               The number of spectral windows must be gt 0 
;               and le 4
;               
; Side effects: Kills any earlier copy displaying data from the
;               same QLDS.
;               
; Category    : CDS, QuickLook, Display
;               
; Prev. Hist. : Idea/origin: Paal Brekke, September 1993
;
; Written     : Stein Vidar Hagfors Haugan, Medio October 1993
;               
; Modified    : Several times....
;               SVHH, 22 October 1993
;                       Added event handling to report the raster number
;                       and the line name....
;               SVHH, 29 October
;                       Terminology: Raster -> Exposure
;                       Exposure numbers from 1 and up...(consistency)
;                       CLEANUP keyword added in xmanager call.
;               SVHH, 29 October
;                       Added DRAWZOOM functionality
;               SVHH, 20 November
;                       Documentation brush-up to v. 1.0
;               SVHH, March 1994,
;                       Added cdslog calls to save status line.
;               SVHH, Version 2, June 1994
;               PB,   Version 2.1 Aug 24 1994
;                       Added buttons - Help, and Hardcopy
;               PB,   August 26, 1994
;                       Changed dety --> detx
;               PB,   Sept 21 - 94    Added Help using WIDG_HELP
;               SVHH, 10-December 94 -- Added CMOUSE/CMOUSE_ACTION
;                       mouse control.
;               Version 3, SVHH, 8 May 1996
;                          Added velocity/intensity/line width analysis (test)
;               Version 4, SVHH, 9 May 1996
;                          LEGEND (astrolib routine) call taken out
;                          Using XPS_SETUP for printing. Some mixups
;                          about the fitting routines resolved.
;               Version 5, 18 August 1996
;                       Using gt_scany instead of expslice, now compatible with
;                       both storage systems.
;               Version 6, 5 September 1996
;                       Subtracting one from the exposure number after
;                       switching to gt_scany.
;               Version: 7, 30 September 1996
;                        Fixed bug in handling time series.
;               Version 8, SVHH, 27 November 1996
;                        Fixed a bug that made QLDS disappear when called from
;                        prompt. Added cw_loadct.
;               Version 9, SVHH, 5 December 1996
;                       Fixed a bug in solar x/y calculations.
;		Version 10, 10-Dec-1997, William Thompson, GSFC
;			Use AMOEBA_C instead of AMOEBA, to avoid conflict with
;			IDL/v5 routine of that name.
;                        
;               V. 11, 1-June-2001, Giulio Del Zanna, DAMTP.
;                      Allow any number of windows to be plotted.
;
;
; Version     : Version 11,  1-June-2001
;- 


;; Result should contain results of fit for *one* gaussian, 
;; with same interpretation as gaussfit

PRO dspexp_fitvel,x,iy,lam0,a,fit
  
  ;; fit = 'amoeba_c'
  
  y = iy-MIN(iy)
  
  a = dblarr(3)
  
  a(0) = MAX(y)
  a(1) = x((WHERE(y EQ MAX(y)))(0)) ;; Initial guess

  a(2) = total(y)/a(0)*(MAX(x)-MIN(x))/N_ELEMENTS(x)/2.38736
  catch,error
  IF error NE 0 THEN BEGIN
     a = dblarr(3)
     MESSAGE,"CAUGHT error in fitting routine "+fit,/continue
     return
  END
  CASE strlowcase(fit) OF 
     'lstsqr': BEGIN
        lstsqr,x,y,'func_f',a,/noprint
        a(2) = a(2)/SQRT(2.0) ;; Fits into exp(-(x-x0)^2/2w^2)
     END
     'curvefit': BEGIN
        w = REPLICATE(1.0,N_ELEMENTS(x))
        res = curvefit(x,y,w,a,sigma,function_name='functn')
        a(2) = a(2)/SQRT(2.0) ;; Fits into exp(-(x-x0)^2/2w^2)
     END
     'amoeba_c': BEGIN
        temp = x(0)-0.001 &  a(1) = a(1)-temp
        ;; Trick to get it going
        amoeba_c,x-temp,y,'func_f',a,/noprint,MAX=100
        a(1) = a(1)+temp
        a(2) = a(2)/SQRT(2.0)
     END
     'gaussfit':BEGIN
        result = gaussfit(x,y,a)
        a = a(0:2)
     END
     'cds_fit_wave':BEGIN
        res = cds_fit_wave({w:x,flux:y},a(1),width=N_ELEMENTS(x),out=out, $
                          /quiet,/noplot)
        IF N_ELEMENTS(out) EQ 0 THEN out = dblarr(6)
        PRINT,out
        a = out(0:2)
     END
     ELSE: BEGIN
        PRINT,"Unknown fit"
     END
  END
END


PRO dspexp_velocity,q,fit,WINDOW,xix=xix,tix=tix,group=group
  COMMON DSPEXP_VEL_XPS_SETUP,setup
  
  default,WINDOW,0
  default,xix,0
  default,tix,0
  
  strings = str_sep(q.detdesc(WINDOW).label,'_')
  IF N_ELEMENTS(strings) NE 4 THEN BEGIN
     MESSAGE,"Window label doesn't follow standard.",/continue
     return
  END
  
  lam0 = DOUBLE(strings(2)+'.'+strings(3))
  PRINT,"lam0="+trim(lam0)
  
  
  sc = gt_scany(q,xix = xix,tix = tix,WINDOW=window,lambda=lam)
  
  WINDOW,0,xsize=600,ysize=1000
  !P.multi = [0,1,1]
  
  fit_t = (str_sep(fit,':'))(0)
  FOR yix = 0,(SIZE(sc))(2)-1 DO BEGIN
     x = lam(*,yix)
     y = sc(*,yix)
     dspexp_fitvel,x,y,lam0,a,fit_t

     IF N_ELEMENTS(aa) EQ 0 THEN aa = [[a]] $
     ELSE                        aa = [[aa],[a]]
  END
  
  !P.multi = [0,1,3]
  
  c = 300000.0 ;; km/s
  
  npoints = N_ELEMENTS(aa(0,*))
  xrange=[0,npoints]
  intens = aa(0,*)
  vel = c*(lam0-aa(1,*))/lam0
  
  ;; Line widths calculation
  ;; Results from fitting (after modification) should fit
  ;; into exp(-(x-x0)^2/(2w^2))
  ;; ==> fwhm = 2 w sqrt(2 ln 2)
  fwhm = 2.0 * aa(2,*) * SQRT(2.0*alog(2.0))
  
  good = abs(vel) LT 1000.0 AND intens GE 0.0 AND fwhm GE 0.0
  goodix = WHERE(good,count)
  badix = WHERE(NOT good,badcount)
  IF count EQ 0 THEN RETURN
  
  x = FINDGEN(N_ELEMENTS(intens))
  
  x = x(goodix)
  vel = vel(goodix)
  intens = intens(goodix)
  fwhm = fwhm(goodix)
  xrange = [-1,npoints]
  reset
  
  ;; Astrolib isn't part of the CDS path, so we use xyouts instead
  legend = q.detdesc(WINDOW).label +"!C"+ $
     "xix:"+trim(xix)+" tix="+trim(tix) +"!C"+ $
     fit
  
  
  bar = rotate(sc,3)
  plot,x,intens,xrange=xrange,xstyle=1,charsize=2,title='Intensity'
  width = !P.clip(2)-!P.clip(0)
  height = !P.clip(3)-!P.clip(1)
  xyouts,!P.clip(0)+width/10,!P.clip(3)-height/10,legend,/DEVICE,charsize=1.3
  
  
  ;; Figure out size of the bar
  bar = cdscongrid(bar,!P.clip(2)-!P.clip(0),(SIZE(bar))(2))
  
  ;; Plot bar, and oplot the intensity
  tvscl,bar,!P.clip(0),!P.clip(1)
  oplot,x,intens
  oplot,x,intens,psym = 4
  clipbox
  
  yrange = [-1,1] * (max(abs(vel)) < 150)
  plot,x,vel,title='Velocity (km/s)', yrange = yrange,$
     xrange=xrange,xstyle=1,charsize=2
  tvscl,bar,!P.clip(0),!P.clip(1)
  oplot,x,vel
  oplot,x,vel,psym = 4
  clipbox
  
  plot,x,fwhm,title='Line widths', $
     xrange=xrange,xstyle=1,charsize=2
  tvscl,bar,!P.clip(0),!P.clip(1)
  oplot,x,fwhm
  oplot,x,fwhm,psym = 4
  clipbox
  
  IF N_ELEMENTS(str_sep(fit,':')) EQ 1 THEN RETURN ;; No postscript
  
  psfile = (str_sep(q.header.filename,'.'))(0)
  psfile = psfile+"_"+q.detdesc(WINDOW).label+ $
     '_X'+trim(xix)+'_T'+trim(tix)+'_'+fit_t
  
  IF datatype(setup) NE 'STC' THEN BEGIN
     xps_setup,setup,/initial
     setup.printer = getenv("PSLASER")
     IF setup.printer EQ '' THEN setup.printer = getenv("PSCOLOR")
     IF setup.printer EQ '' THEN setup.printer = getenv("PRINTER")
     IF setup.printer EQ '' THEN setup.printer = 'lp0'
  END
  
  break_file,setup.filename,disk,dir,name,ext
  setup.filename = disk+dir+psfile+ext
  
  xps_setup,setup,status=status,group=group ;; To avoid XKILL,/ALL!!
  
  IF NOT status THEN RETURN
  
  psfile = setup.filename
  
  ps,psfile,encapsulated=setup.encapsulated,portrait=setup.portrait, $
     color=setup.color,copy=setup.copy,interpolate=setup.interpolate
     
  bar = rotate(sc,3)
  plot,x,intens,xrange=xrange,xstyle=1,charsize=2,title='Intensity'
  width = !P.clip(2)-!P.clip(0)
  height = !P.clip(3)-!P.clip(1)
  xyouts,!P.clip(0)+width/10,!P.clip(3)-height/10,legend,/DEVICE,charsize=1.3
  
  ;; Pixel scaling of bar image
  xsize=!P.clip(2)-!P.clip(0)
  ysize=(!P.clip(3)-!P.clip(1))/5L

  ;; Plot bar, and oplot the intensity
  tvscl,bar,!P.clip(0),!P.clip(1),xsize=xsize,ysize=ysize
  oplot,x,intens
  oplot,x,intens,psym = 4
  clipbox
  
  plot,x,vel,title='Velocity (km/s)', $
     xrange=xrange,xstyle=1,charsize=2
  tvscl,bar,!P.clip(0),!P.clip(1),xsize=xsize,ysize=ysize
  oplot,x,vel
  oplot,x,vel,psym = 4
  clipbox
  
  plot,x,fwhm,title='Line widths', $
     xrange=xrange,xstyle=1,charsize=2
  tvscl,bar,!P.clip(0),!P.clip(1),xsize=xsize,ysize=ysize
  oplot,x,fwhm
  oplot,x,fwhm,psym = 4
  clipbox
  IF setup.hard THEN BEGIN
     psplot,delete=setup.delete,queue=setup.printer,color=setup.color
  END ELSE BEGIN
     psclose
  END
END

PRO dspexp_event,ev
  common QLSAVE,QLDS,SAVED
  
  storebase = Widget_INFO(ev.top,/child)
  Widget_CONTROL,storebase,Get_UVALUE=info
  
  Widget_CONTROL,ev.id,Get_UVALUE=uvalue
  

  CASE uvalue OF
      "QUIT" : BEGIN
	  Widget_CONTROL,ev.top,/destroy
	  return
      END
      
      "REFRESH": BEGIN
	  QLDS_checkin,widget=ev.top
	  plot_dspexp,info,QLDS
      END
      
      "HELP":BEGIN
         widg_help,'dspexp',$
         title = 'Help: Spectral Window Exposure',group=ev.top
      END
      "HARDCOPY": BEGIN
           cdsnotify,group=ev.top,'Not implemented yet'
      END

      
      "DRAW" : BEGIN
	  action = cmouse_action(ev)
	  Plotreg = Pfind(ev,found,plot_number=N)
	  IF NOT found THEN return
	  exposureno = N mod info.nexp + 1
	  lineno =  N/info.nexp	+ 1
	  win_no = info.windows(lineno-1)
          QLDS_checkin,widget=ev.top
          
          xix = 0 & tix = 0
          
          IF info.timeseries THEN tix = exposureno-1 $
          ELSE                    xix = exposureno-1
          
	  expo = gt_scany(QLDS,window=win_no-1,xix=xix,tix=tix,/quick)
	  pixx = fix(pconvert(Plotreg,ev.x,/DEVICE,/to_pixel))
	  pixy = fix(pconvert(Plotreg,ev.y,/DEVICE,/to_pixel,/Y))
	  
	  origin = QLDS.DetDesc(win_no-1).origin(*)
	  spacing = QLDS.DetDesc(win_no-1).spacing(*)
	  axes = QLDS.DetDesc(win_no-1).axes(*)
	  
	  lambda = origin(0)+spacing(0)*pixx ; NOTE! pix2wave better!
	  Ndims	 = N_elements(QLDS.DetDesc(win_no-1).ixstart)
          
          ;; pos_txt1 & 2 contain axis names & values respectively
          
          pos_txt1 = '(' & pos_txt2='('
	  IF Ndims gt 1	THEN BEGIN
	      IF N_elements(axes) gt 1 $
		 and N_elements(origin)	gt 1 THEN BEGIN
		  pos_txt1 = pos_txt1 +	axes(1)
		  pos_txt2 = pos_txt2 +	trim(origin(1)+spacing(1)*xix)
	      EndIF
	      IF N_elements(axes) gt 2 $
		 and N_elements(origin)	gt 1 THEN BEGIN
		  pos_txt1 = pos_txt1 +', '+axes(2)
		  pos_txt2 = pos_txt2 +', '+trim(origin(2)+$
                                                 spacing(2)*pixy)
	      END
              pos_txt =	'Position:'+pos_txt1+')='+pos_txt2+') '

	  END ELSE pos_txt = ''
	  
	  IF Ndims eq 1	THEN intens = expo(pixx) $
	  ELSE		     intens = expo(pixx,pixy)
	  
	  type = strmid(tag_names(QLDS.DetDesc(win_no-1),/str),0,1)
	  IF type ne 'C' THEN pixx = pixx + QLDS.DetDesc(win_no-1).detx	$
	  ELSE		      pixx = pixx + QLDS.DetDesc(win_no-1).detx
	  
          CASE STRMID(action,0,8) OF
	      'MEASURE': Widget_CONTROL,info.expinfo,Set_VALUE=$
			'Window: '+info.label(lineno-1)	+$
			'   Intensity='+trim(intens)+$
			'   Lambda='+trim(lambda)+string(194b)+$
			'   '+pos_txt+$
			'   Exposure #'+trim(exposureno) +$
			' (DETX,DETY): ('+trim(pixx)+','+trim(pixy)+')'
	      'ZOOM': pqlzoom,expo,event=ev,xysize=300,$
		    title=info.label(lineno-1)+' exposure: '+trim(exposureno),$
		    xtitle='Dispersion pixel',ytitle='Spatial pixel',$
		    xformat='(f5.1)',yformat='(f5.1)'
	      'PROFILE': pqlprofile,expo,event=ev,$
		    title=info.label(lineno-1)+' exposure: '+trim(exposureno),$
		    xtitle='Dispersion pixel',ytitle='Spatial pixel',$
		    xformat='(f6.2)',yformat='(I6)',$
		    datatitle='Counts'
	      'LOG': begin
		  Widget_CONTROL,info.expinfo,Get_VALUE=text
                  cdslog,group=ev.top,text
               END
               
               'VELOCITY': BEGIN
                  dim = gt_dimension(qlds)
                  IF dim.ssolar_x GT 1 THEN BEGIN
                     xix = exposureno-1
                     tix = 0
                  END ELSE BEGIN
                     tix = exposureno-1
                     xix = 0
                  END
                  fit = STRMID(action,8,1000)
                  dspexp_velocity,qlds,fit,win_no-1,xix=xix,tix=tix, $
                     group=ev.top
              END
              
		'': return
	  EndCASE
	  return
      END
  EndCASE
  
  Widget_CONTROL,storebase,Set_UVALUE=info
END



PRO plot_dspexp,info,QLDS,init=init
;
; Display the data..
;
  SetWindow,info.expwindow
  erase
  xix = 0
  tix = 0
  reset
  FOR j=info.nlines-1,0,-1	DO BEGIN
     MAX=Wmax(QLDS,info.windows(j),/NOCHECK)
     NZactive = 0 & Zactive=0
     NPactive = 0 & Pactive=0
     ddt = gt_windata(qlds,info.windows(j)-1,/quick)
     FOR i=1, info.nexp DO BEGIN
        IF info.timeseries THEN tix = i-1 $
        ELSE                    xix = i-1
        dt = REFORM(ddt(*,xix,*,tix),/overwrite)
        sz = Size(dt)	& xrange=[-.5d,sz(1)-.5d]
        jx = info.xstart + (i-1)*(info.xspacing)
        jy = info.ystart + j*info.yspacing
        Plot,[0],/nodata,xrange=xrange,xstyle=5,$
           yrange=[-.5d,sz(2)-.5d],ystyle=5,/noerase,$
           position=[jx,jy,jx+sz(1)-1,jy+sz(2)-1]+0.01,/DEVICE
        tv,ql_bscale(dt),jx,jy
        IF sz(0) EQ 1	THEN p = pstore(j*info.nexp + i-1,sz(1),1,clean=init) $
        ELSE	   p = pstore(j*info.nexp + i-1,sz(1),sz(2),clean=init)
        init=0
        IF sz(0) EQ 2	THEN $
           pqlzoom,P_reg=P,/change,active=Zactive,kill = Nzactive GT 1
        pqlprofile,P_reg=P,/change,active=Pactive,kill = NPactive GT 1
        NPactive = NPactive +	Pactive
        NZactive = NZactive +	Zactive
        clipbox
     EndFOR
     IF sz(0) EQ 1 THEN sizetext = ' ('+trim(sz(1))+') pixels, ' $
     ELSE	   sizetext = '  ('+trim(sz(1))+'x'+trim(sz(2))+') pixels, '
     
     xyouts,10,info.ystart + (j+1)*info.yspacing-25,/dev,charsize=1.5,$
        QLDS.DetDesc(info.windows(j)-1).label +$
        sizetext+$
        '!7k!3=('+trim(QLDS.DetDesc(info.windows(j)-1).origin(0))+$
        ','+trim(QLDS.DetDesc(info.windows(j)-1).origin(0) +$
                 QLDS.DetDesc(info.windows(j)-1).spacing(0)*sz(1))+')'
  ENDFOR
END




PRO dspEXP,GROUP_LEADER=GROUP_LEADER,QLDS,windows,NOCHECK=NOCHECK
  common QLMGR,QL_ID,QL_DATA
  common QLSAVE,saveQLDS,SAVED
  
  On_Error,2
  IF n_params()	lt 1 THEN BEGIN
      message,'Use: dspexp, QL_Data'
  EndIF
;
; Check type with QLmanager
; 
  IF NOT Keyword_SET(NOCHECK) THEN BEGIN
      QLMGR,QLDS,valid
      IF NOT valid THEN	message,'First argument is not a QL Data structure'
  EndIF
  
  IF N_params()	lt 2 THEN BEGIN
      mtext = trim(QLDS.DetDesc(*).wavelength,'(f6.2)')	+$
		'    '+	trim(QLDS.DetDesc(*).label,'(A)')
      selection	= intarr(N_elements(QLDS.DetDesc))
      Xselect,mtext,selection,abort,xoffset=500,yoffset=500,$
		title='Display exposure: Select Spectral Windows (max 4)'
      windows =	where(selection	ne 0,count) + 1
      IF abort or count	eq 0 THEN return
;      IF count gt 4 THEN BEGIN
;	  cdsnotify,group=group_leader,'4 windows maximum'
;	  windows = windows(0:3)
;      EndIF
  EndIF
;
; Parameter checking.
;
  IF datatype(windows) eq 'STR'	THEN BEGIN
      wnd = intarr(N_elements(windows))
      FOR i=0,N_elements(windows)-1 DO wnd(i) =	windowno(QLDS,windows(i))
      windows=wnd
  EndIF
  
  nlines = N_elements(windows)

  windows = [windows]
  
;  IF (nlines gt	4) THEN	BEGIN
;      Bell
;      Print,' Sorry, Too many wavelength windows selcted (Max: 4).'
;      Print,' Deselect some of the windows.'
;      return
;  END
  
  IF (nlines eq	0) THEN	BEGIN
      Bell
      Print,' Sorry, No lines selected...'
      return
  EndIF
  
  On_Error,0
  DEVICE,get_screen_size = scrnsize
  
;
; Find out the size(s) of the data to be displayed...
;
  
  nexp = Nexposures(QLDS,/NOCHECK)
  
  maxwsize=0
  FOR i=0,nlines-1 DO	$
      maxwsize=max([WaveSize(QLDS,windows(i),/NOCHECK),	maxwsize])
  
  xstart = 10
  ystart = 10
  xspacing = maxwsize +	12
  xsize	= xspacing*nexp	+ xstart*2 
  ysz =	Zsize(QLDS,/NOCHECK)
  yspacing = (ysz+30)
  ysize	= yspacing*nlines + ystart
  
;
; Handle any earlier display of the same type for this QLDS
;  - note it's position and kill it.
;
  
  offset=[0,500]
  
  expbase = QL_Data(QLDS.QL_no).dspexpbase
  
  IF Widget_INFO(expbase,/valid_id) THEN BEGIN
      Widget_CONTROL,expbase,TLB_GET_OFFSET=offset
      Widget_CONTROL,expbase,/destroy
  EndIF
  
;
; Setup the windows.
;
  
  
  IF N_elements(GROUP_LEADER) EQ 1 THEN	$
	expbase	= Widget_BASE(title='SPECTRAL WINDOW EXPOSURES',$
			  GROUP_LEADER=GROUP_LEADER,$
			  xoffset=offset(0),yoffset=offset(1),/column) $
  ELSE	expbase	= Widget_BASE(title='SPECTRAL WINDOW EXPOSURES',$
			  xoffset=offset(0),yoffset=offset(1),/column)
  
  opt_base = Widget_BASE(expbase,/row)
  storebase = opt_base	 ; Status storage
  
  
  dummy = {CMOUSE_S, btext:'', mtext:'', uvalue:'', avail:0}
  
  
  fit_methods = ['lstsqr','curvefit','amoeba_c','gaussfit','cds_fit_wave']
  fit_methods = [fit_methods,fit_methods+':PS']
  fit_menu = REPLICATE({cmouse_s},N_ELEMENTS(fit_methods))
  fit_menu(*).btext = 'Velocities ('+fit_methods+')'
  fit_menu(*).mtext = 'Plot int/vel/fwhm in separate window ('+fit_methods+')'
  fit_menu(*).uvalue = 'VELOCITY'+fit_methods
  fit_menu(*).avail = 1+2+4
  
  menu=[{CMOUSE_S, "Measure", "Measuring positions/intensities",$
         "MEASURE",1+8},$
	{CMOUSE_S, "Write to log", "Write the last measurement to the CDSLOG",$
         "LOG",2},$
        {CMOUSE_S, "Zooming", "Zooming in on a window",$
         "ZOOM",2+4},$
        {CMOUSE_S, "Profiling", "Plotting profiles from a window",$
         "PROFILE",2+4}, $
        fit_menu]


  cmouse,expbase,opt_base,menu

  button = Widget_BUTTON(opt_base,value='Refresh screen',uvalue='REFRESH')
  button = Widget_BUTTON(opt_base,value='Help',uvalue="HELP")
  button = Widget_BUTTON(opt_base,value='Hardcopy',uvalue="HARDCOPY")
  button = Widget_BUTTON(opt_base,value='Done',uvalue="QUIT")
  
  cwloadct = cw_loadct(opt_base)
  
  
  expdraw = Widget_DRAW(expbase,/scroll,xsize=xsize,ysize=ysize,$
			x_scroll_size=min([xsize,scrnsize(0)-50]),$
			y_scroll_size=ysize+31,/button_events,$
			uvalue='DRAW',/motion_events)
  
  expinfo = Widget_TEXT(expbase,$
		  value='Please wait...')
  
;
; Register the base with the QLMGR
;
  
  QL_Data(QLDS.QL_no).dspexpbase = expbase
  
  Widget_CONTROL,expbase,/realize
  
  Widget_CONTROL,expdraw,Get_VALUE=expwindow
  
  load_red,0.5
  
  dim = gt_dimension(qlds)
  timeseries = 1
  IF dim.ssolar_x GT 1 THEN timeseries = 0
  
  info = { nexp		: nexp,$
           dim          : dim,$
           timeseries   : timeseries,$
	   nlines	: nlines,$
	   expwindow	: expwindow,$
	   ysize	: Zsize(QLDS,/NOCHECK),$
	   windows	: windows,$
	   maxwsize	: maxwsize,$
	   xspacing	: xspacing,$
	   yspacing	: yspacing,$
	   xstart	: xstart,$
	   ystart	: ystart,$
	   label	: [QLDS.DetDesc(windows-1).label],$
	   expinfo	: expinfo }
  
  plot_dspexp,info,QLDS,/init
  
  widget_control,expinfo,set_value='Click on exposure/line to get number/name'
;
; Set user value of top base to QLDS
;
  ql_no	= QLDS.QL_NO
  qlds_report,expbase,QLDS,store = (N_elements(GROUP_LEADER) eq	0)
  
  Widget_CONTROL,storebase,$
	Set_UVALUE=info
  
  Xmanager,'DSPEXP',expbase,CLEANUP='qlmgrclean'
  
  IF N_elements(GROUP_LEADER) eq 0 and NOT Keyword_SET(nokeep)	THEN BEGIN
      WHILE Widget_INFO(expbase,/valid_id) ne 0 DO Xmanager
      qlds_checkin,ql_no
      assign_nocopy,QLDS,saveQLDS
  END
  
END
