;+
; Project     : SOHO - CDS     
;                   
; Name        : DSPWAV
;       
; Purpose     : Display data slices at specific wavelength +line profiles
;
; Explanation : A window is created with two types of display of the
;               data in four selected spectral windows. 
;
;               The upper display is is a cross section of the 
;               data at a specific wavelength, giving a spatial 
;               view of the region that has been scanned by the CDS NIS/GIS.
;
;               The lower display is the intensity profile along the
;               dispersion direction (line profile), at a selected
;               point in the two-dimensional image displayed in the
;               window above. 
;
;               Although only four spectral windows may be shown
;               on screen at any time, it is possible to select
;               which spectral window is to be shown where from 
;               the pulldown menu between the upper and lower
;               displays.
;
;               To change the spatial views from one wavelength to 
;               another, click on the line profile at the desired
;               wavelength. 
;
;               The point from which the line profile is taken 
;               is selected by clicking the left mouse button in the 
;               spatial view, and it is marked by a cross in the 
;               spatial view. 
;
;               Incremental positioning is done by clicking the left 
;               mouse button outside the axes of the displayed area. 
;               The point/cross is then moved one pixel in the direction 
;               of the mouse marker.
;
;               Cursor action is as follows:
;                       B1: select point/wavelength of interest
;                       B2: extract profiles (profile window appear)
;                       B3: zooming
;
; Use         : DSPWAV,QLDS
;    
; Inputs      : QLDS:   CDS QuickLook data structure.
;
; Opt. Inputs : WINDOWS:Array containing the numbers of the
;                       detector data windows to be displayed.
;               
; Outputs     : None.
;               
; Opt. Outputs: None.
;               
; Keywords    : GROUP_LEADER:
;                       If this invoked by any other CDS QuickLook 
;                       display routine (group leader responsible for 
;                       keeping the QLDS), this keyword should have
;                       the widget ID of the top level base of the
;                       calling routine.
;
;               NOCHECK:
;                       Set this to avoid invoking QLMGR to check
;                       validity of the QLDS.
;
; Calls       : ASSIGN_NOCOPY, BELL, CDSNOTIFY, CDSPICKFILE(), CLIPBOX,
;               CMOUSE, CMOUSE_ACTION(), CW_PSELECT(), DSP_MENU, FUNCT_1,
;               GT_IIMAGE(), GT_SPECTRUM(), INT_FWHM_2, LOAD_RED, LSTSQR,
;               NEXPOSURES(), POINTSLICE(), PQLPROFILE, PQLZOOM, PRESTORE, PS,
;               PSCLOSE, PSPLOT, PSTORE(), QLDS_CHECKIN, QLDS_REPORT, QLMGR,
;               SETWINDOW, SLITPOS_B, SLITPOS_K1, SLITPOS_K2, SLITPOS_Y,
;               TRIM(), WAVESIZE(), WIDGET_BASE(), WIDG_HELP, ZSIZE()
;               (11-Dec-94)
;
; Common      : QLSAVE: See QLDS_CHECKIN
;           
; Restrictions: The QLDS must adhere to the "CDS QL data format"
;               
; Side effects: Kills any previous copy displaying data from the
;               same QLDS, taking its position on the screen.
;               
; Category    : CDS, QuickLook, Display
;               
; Prev. Hist. : Idea/origin: Paal Brekke, September 1993
;
; Written     : Stein Vidar Hagfors Haugan, Medio October 1993
;               
; Modified    : Several times.... cut /no_copy keyword  CDP
;               SVHH, 26 October
;                       BASES are now lonarr, not intarr..
;               SVHH, 29 October
;                       Added DRAWZOOM functionality
;               PB, 8 November
;                       Added wavelength scales in plot window.
;                       Pointing and exposure info diplayed.
;                       Calculates intensity, FWHM and shifts..
;                       Slitposition on Sun etc.
;               SVHH, 21 November
;                       Documentation brush-up
;               SVHH, January -94
;                       Added interactive selection of spectral windows.
;                       Rotated image to have "correct" orientation.
;               SVHH, March -94
;                       Added plot to file/printer routines.
;               SVHH, Version 2, 8-June-1994
;                       Switched to PQLZOOM, added PQLPROFILE for
;                       both image display and the point profile.
;                       
;               PB,   June 29 -94
;                       Added frame around images
;                       Modyfied layout slightly; more full disk images
;                       available, prepared for n(e)/T diagnostics,
;                       Added call to WIDG_HELP <--
;               PB,   Sept 21 - 94    Added Help using WIDG_HELP
;               SVHH, 29-Nov - 94, Fixed POSITION keyword roundoff error
;               SVHH, 10-Dec - 94, Added CMOUSE/CMOUSE_ACTION() Mouse
;                       control functionality. Redesigned the layout and
;                       made the line fit type selection into a pulldown
;                       selection menu (CW_PSELECTION).
;                       Added x/y-format/units in PQLZOOM call.
;               Version 2.6, William Thompson, GSFC, 24 March 1995
;                       Changed so that axes are not reversed, and changed
;                       references from Solar_Y/Z to Solar_X/Y.
;               Version 2.6a, SVHH, 28 July 1995
;                       Removed reversal of axes for PQLZOOM/PROFILE
;                       calls.
;               Version 3, SVHH, 22-December-1995
;                       Linear/logarithmic plot toggle
;                       added. Displaying all rasters inside a 120x120
;                       pixel box.  
;
;               VErsion 3.1, PB,1-February-1996
;                       Changed the GFIT routine to call
;                       CDS_GAUSS(). Background is now subtarcted .
;                       Also removed the Widget_LABEL keywords, 
;                       XSIZE-YSIZE. Exp. No. etc. are now displayed
;                       again.
;
;               Version 4, SVHH, 8 April 1996
;                       Using gt_spectrum to extract spectra. Avoids
;                       crash with rasters w/summed dispersion direction.
;                       Also using gt_iimage instead of waveslice.
;               Version 5, SVHH, 23 May 1996
;                       Using IMAGE_TOOL to show raster location.
;                       
; Version     : 5, 23 May 1996
;- 


PRO dspwav_event,ev
  common QLSAVE,QLDS,SAVED
  
  On_Error,2
  if !DEBUG ne 0 then on_error,0
  
  name = strmid(tag_names(ev, /structure_name), 7,100)
  Widget_CONTROL,ev.top,/hourglass
  child = Widget_INFO(ev.top,/child)
  Widget_CONTROL,child,Get_UVALUE=info; ,/no_copy
  
  Widget_CONTROL,ev.id,Get_UVALUE=UVALUE
  uvalue_str = strmid(trim(uvalue),0,8)
  
  update = 0
  
  CASE uvalue_STR OF
      'WAVDRAW': BEGIN
	  action = cmouse_action(ev)
	  IF action eq '' THEN return
	  i = (where(info(*).wwavTV eq ev.id))(0)   ; What window?
	  IF action eq 'MOVE' THEN BEGIN
	      QLDS_checkin,widget=ev.top
	      selpoint,QLDS,ev,info,i		  ; From the TV window
	      FOR i=0,N_elements(info)-1 DO draw_wave,QLDS,info,i
	  END ELSE BEGIN
	      QLDS_checkin,widget=ev.top
	      i = abs(i)
	      windowno = info(i).window
	      desc = QLDS.DetDesc(windowno-1)
	      lambda=desc.wavemin + info(i).wpixel * desc.spacing(0)
	      title =desc.label + ' ' +trim(lambda)+string(197B)
	      xtitle=desc.axes(1)
	      ytitle=desc.axes(2)
	      wslice = WaveSlice(QLDS,windowno,info(i).wpixel,$
						  valid,/NOCHECK)
	      IF valid THEN BEGIN
		  IF action eq 'PROFILE' THEN	$
			pqlprofile,event=ev,wslice,$
			      title=title,xtitle=xtitle,ytitle=ytitle $
		  ELSE IF action eq 'ZOOM' THEN $
			pqlzoom,event=ev,wslice,$
			      title=title,xtitle=xtitle,ytitle=ytitle,$
			      xformat='(f6.1)',yformat='(f6.1)',$
			      xunits='"',yunits='"'
	      EndIF
	  END
      EndCASE
      

      'PROFDRAW': IF ev.press ne 0 THEN BEGIN
	  i = (where(info(*).wwavPROF eq ev.id))(0)   ; What window?
	  QLDS_checkin,widget=ev.top
	  action = cmouse_action(ev)
	  IF action eq 'MOVE' THEN selwave,QLDS,ev,info,i $; -select wavelength
	  ELSE IF action eq 'PROFILE' THEN BEGIN
	      profile = PointSlice(QLDS,info(i).window,$
		 info(i).coord(1),info(i).coord(0),lambda=lambda,/NOCHECK)
	      pqlprofile,profile,lambda,event=ev ; Details coming..
	  EndIF
	  draw_wave,QLDS,info,i
      EndIF
      
      
      'LOGPLOT': BEGIN
         info(*).log_plot = ev.select
         qlds_checkin,widget=ev.top
         FOR i=0,N_ELEMENTS(info)-1 DO draw_wave,qlds,info,i
      END
      
      'DONE': BEGIN
	  QLDS_checkin,widget=ev.top
	  Widget_CONTROL,ev.top,/destroy
	  return
      EndCASE
      

      'MENU': BEGIN
	  QLDS_checkin,widget=ev.top
	  dsp_MENU,QLDS,group_leader=ev.top
      EndCASE
      

      'PSPRINT': BEGIN
	  QLDS_checkin,widget=ev.top
	  ps
	  FOR i=0,N_elements(info)-1 DO ps_draw_wave,QLDS,info,i
	  psplot
	  cdsnotify,group=ev.top,'Plot sent to printer'
      EndCASE  ; print
      

      'PSFILE': BEGIN
	  QLDS_checkin,widget=ev.top
	  file = cdspickfile(filter='*.ps')
	  IF file ne '' THEN BEGIN
	      ps,file
	      FOR i=0,3 DO ps_draw_wave,QLDS,info,i
	      psclose
	      cdsnotify,group=ev.top,'Plot sent to file:'+file
	  EndIF
      EndCASE
      

      'ISIMPLE': BEGIN
	  info(*).gauss = 1
	  update = 1
      EndCASE
      
      'IGAUSS': BEGIN
	  info(*).gauss=2
	  update = 1
      EndCASE
      
      'IMAGE_TO': BEGIN
         qlds_checkin,widget=ev.top
         dsp_point,qlds,/modal,group=ev.top
      END
      
      'HELP':BEGIN
	  widg_help,'dspwav',$
	  title = 'Help: Wavelength Slice/Profile',group=ev.top
      END
      
      'SETWINDW': BEGIN ; A new spectral window selected......
	  position = fix(strmid(uvalue,8,1))
	  windowno = fix(strmid(uvalue,9,100))
	  info(position-1).window = windowno
	  QLDS_checkin,widget=ev.top
	  info(position-1).wpixel = WaveSize(QLDS,windowno,/NOCHECK)/2
	  draw_wave,QLDS,info,position-1
      END
      
      ELSE: help,ev,/str
      
  EndCASE
  
  IF update THEN BEGIN
      QLDS_checkin,widget=ev.top
      FOR i=0,N_elements(info)-1 DO draw_wave,QLDS,info,i
  EndIF
  
  Widget_CONTROL,child,Set_UVALUE=info; ,/no_copy
  
END



;
; Select a new point in the spatial view.
;
PRO selpoint,QLDS,ev,info,i
  
  prestore,info(i).P_TV
  
  zsz = Zsize(QLDS)
  nexp= Nexposures(QLDS)
  
  ev.x = (ev.x - !P.clip(0))
  ev.y = (ev.y - !P.clip(1))
  
  relflag = 0
  
;
; Incremental positioning in the X-direction?
;
  
  IF ev.x lt 0 THEN BEGIN
      IF info(i).coord(0) gt 0 THEN info(i).coord(0) = info(i).coord(0)-1
      relflag = -1
  END ELSE BEGIN
      IF ev.x gt nexp-1 THEN BEGIN
	  IF info(i).coord(0) lt nexp-1 THEN $
			      info(i).coord(0) = info(i).coord(0)+1
	  relflag=-1
      EndIF
  EndELSE
  
;
; Incremental positioning in the Y-direction?
;
  
  IF ev.y lt 0 THEN BEGIN
      IF info(i).coord(1) gt 0 THEN info(i).coord(1) = info(i).coord(1)-1
      relflag = -1
  END ELSE BEGIN
      IF ev.y gt zsz-1  THEN BEGIN
	  IF info(i).coord(1) lt zsz-1  THEN $
		  info(i).coord(1) = info(i).coord(1)+1
	  relflag = -1
      EndIF
  EndELSE
  
;
; Point selection is distributed to all windows...
;
  IF NOT relflag THEN BEGIN
      info(*).coord(0) = ev.x
      info(*).coord(1) = ev.y
  END ELSE BEGIN
      info(*).coord(0) = info(i).coord(0)
      info(*).coord(1) = info(i).coord(1)
  EndELSE
  
END



;
; Select a new wavelength index to view at
;

PRO selwave,QLDS,ev,info,i
  SetWindow,info(i).wavPROF
; Calculate wavelength scale
;  Should be done once .....
  desc = QLDS.DetDesc(info(i).window-1)
  lambda=desc.wavemin + $
		  findgen(desc.ixstop(0)-desc.ixstart(0)+1) * desc.spacing(0)
  coord = convert_coord(ev.x,ev.y,/DEVICE,/TO_DATA)
  x=coord(0,0)
  dummy = min(abs(lambda-x),wpixel)   ; Find closest pixel
  info(i).wpixel = wpixel
END



;
; Calculate gaussian fit.
;

PRO gfit, QLDS, lambda, prof, ix, info, i, gcoeff, iny, lamdum

  iny=cds_gauss(lambda(ix),prof(ix),gcoeff,1)

  cwave = QLDS.DetDesc(info(i).window-1).wavelength ;
  shift=((gcoeff(1)-cwave)/cwave) * 3.0E+5
  gcoeff(1)=shift
  gcoeff(2) = gcoeff(2) * SQRT(2)
  lamdum=lambda
  iny=iny

END

;
; Refresh screen.
;

PRO ps_draw_wave,QLDS,info,i
  
  offset = i/4.
  
  wslice = gt_iimage(qlds,WINDOW=info(i).window-1, $
                     offset=info(i).wpixel,/quick)
  
  wslice0 = wslice
  IF NOT valid THEN return
  xsize = N_elements(wslice(*,0)) ; Rotation!
  ysize = N_elements(wslice(0,*))
  maxval=max(wslice)
;
; Make crosshairs
;
  wslice(info(i).coord(0),$
	 info(i).coord(1)-10 > 0 : info(i).coord(1)-3 > 0)=maxval
  wslice(info(i).coord(0),$
	 info(i).coord(1)+3 <ysize-1: info(i).coord(1)+10 <ysize-1)=maxval
  wslice(info(i).coord(0)-10 > 0:info(i).coord(0)-3 > 0,$
	 info(i).coord(1))=maxval
  wslice(info(i).coord(0)+3 <xsize-1: info(i).coord(0)+10 <xsize-1,$
	 info(i).coord(1))=maxval
  
  d = QLDS.DetDesc(info(i).window-1)
  
;
  
  sz = [xsize,ysize]
  sz = float(sz)/sz(0) * 0.2
  IF sz(1) gt 0.3 THEN sz = sz/sz(1)*0.3
  
  Plot,[d.origin(1),d.origin(1)+(xsize-1)*d.spacing(1)],$
       [d.origin(2),d.origin(2)+(ysize-1)*d.spacing(2)],$
       xrange=[d.origin(1),d.origin(1)+(xsize-1)*d.spacing(1)],$
       yrange=[d.origin(2),d.origin(2)+(ysize-1)*d.spacing(2)],$
       xstyle=1,ticklen=-.06,yticks=ysize/20,ystyle=1,$
       ytickformat='(I5)',xticks=xsize/30,$
       xtitle=QLDS.DetDesc(info(i).window-1).axes(1)+' (")',$
       ytitle=QLDS.DetDesc(info(i).window-1).axes(2)+' (")',$
       position=[offset+0.025,0.55,offset+0.025+sz(0),0.55+sz(1)],$
	  /NORMAL,/NODATA,/noerase
  
  tvscl,wslice,offset+0.025,0.55,xsize=sz(0),ysize=sz(1),/normal
  
  sol_x = d.origin(1) + info(0).coord(0) * d.spacing(1)
  sol_y = d.origin(2) + info(0).coord(1) * d.spacing(2)
  
  yoff = 0.98
  xyouts,0.2,yoff,d.axes(1)+':'+trim(sol_x)+$
	'     '+d.axes(2)+':'+trim(sol_y)+$
	'     '+'Exposure:'+trim(info(0).coord(0)+1)+$
	'     '+'Along slit:'+trim(info(0).coord(1)+1),/normal
  yoff = yoff - !D.y_ch_size/float(!D.y_size)*1.2
  xyouts,offset,yoff,"Line: "+d.label,/normal
  yoff = yoff - !D.y_ch_size/float(!D.y_size)*1.2
  
;
; Update the profile window
;
  
  profile = PointSlice(QLDS,info(i).window,$
		  info(i).coord(1),info(i).coord(0),lambda=lambda,/NOCHECK)
  
  ix = where(profile gt 0,count)
  IF count gt 0 THEN BEGIN
;
; Should we do some profile analysis?
; 
     IF (info(i).gauss EQ 1) THEN BEGIN
; Find background and subtract from intensity array
        c = poly_fit(lambda(ix),profile(ix),1,yf)
        profile(ix) = profile(ix)-yf 
        int_fwhm_2,lambda(ix),profile(ix),totint,fwhm,shft
        cwave = QLDS.DetDesc(info(i).window-1).wavelength ;
        vel=((shft-cwave)/cwave) * 3.0E+5
        shft=vel
     EndIF
     
     IF (info(i).gauss EQ 2) THEN BEGIN
; Find background and subtract from intensity array
        c = poly_fit(lambda,profile,1,yf)
        profile = profile-yf         
        gfit, QLDS, lambda, profile, ix,info,i,gcoeff,iny,lamdum
        totint=gcoeff(0)*1.77*gcoeff(2)
        fwhm=gcoeff(2)*(2*(SQRT(alog(2))))
        shft=gcoeff(1)
     EndIF
     
     IF (info(i).gauss EQ 0) THEN BEGIN
        totint = 0
        fwhm = 0
        shft  = 0
      END
      
      xyouts,offset,yoff,"Intensity:"+trim(totint(0)),/normal
      yoff = yoff - !D.y_ch_size/FLOAT(!D.y_size)*1.2
      xyouts,offset,yoff,"FWHM:"+trim(fwhm(0)),/normal
      yoff = yoff - !D.y_ch_size/FLOAT(!D.y_size)*1.2
      xyouts,offset,yoff,"Velocity:"+trim(shft(0)),/normal
      yoff = yoff - !D.y_ch_size/FLOAT(!D.y_size)*1.2
      
      
      
;
; Plot the measured data points (to file)
;      
      !Y.style=0
      IF info(i).log_plot THEN begin
         ymin = QDLS.detdesc(info(i).window-1).min > 1
         ymax = QLDS.detdesc(info(i).window-1).max
         Plot_IO,lambda(ix),profile(ix),title='Counts vs Wavelength',$
            position=[offset+0.025,0.025,offset+0.25-0.025,.4],xticks=3,$
            xrange=[MIN(lambda(ix)),MAX(lambda(ix))],xstyle=1,$
            yrange=[ymin,ymax],/noerase,psym=10
      END ELSE BEGIN
         Plot,lambda(ix),profile(ix),title='Counts vs Wavelength',$
            position=[offset+0.025,0.025,offset+0.25-0.025,.4],xticks=3,$
            xrange=[MIN(lambda(ix)),MAX(lambda(ix))],xstyle=1,$ $
            yrange=[1,QLDS.DetDesc(info(i).window-1).max],/noerase,psym=10
         
      END
      
;
; Mark the wavelength we're looking at.
;
      IF info(i).wpixel GE 0 AND info(i).wpixel LT N_elements(profile) THEN $
         OPlot,[lambda(info(i).wpixel)],[profile(info(i).wpixel)],psym=6
      
      ;; Fix background!!!
      IF (info(i).gauss EQ 2) THEN OPlot,lamdum,iny+15,linestyle=3
      
   END ELSE BEGIN
      erase
      xyouts,/normal,.05,.5,'All points Zero',charsize=2
   EndELSE
   !P.noerase = 0
  
END



;
; Update display of window no. i
;

PRO draw_wave,QLDS,info,i

;
; Update TV display
;
  
  SetWindow,info(i).wavTV
  
  wslice = gt_iimage(qlds,WINDOW=info(i).window-1, $
                     offset=info(i).wpixel,/quick)
  
  wslice0 = wslice
  
  xsize = N_elements(wslice(*,0)) ; Rotation!
  ysize = N_elements(wslice(0,*))
  maxval=MAX(wslice)
;
; Make crosshairs
;
  wslice(info(i).coord(0),$
         info(i).coord(1)-10 > 0 : info(i).coord(1)-3 > 0)=maxval
  wslice(info(i).coord(0),$
         info(i).coord(1)+3 <ysize-1: info(i).coord(1)+10 <ysize-1)=maxval
  wslice(info(i).coord(0)-10 > 0:info(i).coord(0)-3 > 0,$
         info(i).coord(1))=maxval
  wslice(info(i).coord(0)+3 <xsize-1: info(i).coord(0)+10 <xsize-1,$
         info(i).coord(1))=maxval
  
  d = QLDS.DetDesc(info(i).window-1)
  
;     
; Plot the axes
;
      
  lambda=d.wavemin + info(i).wpixel * d.spacing(0)
  title =d.label + ' '+trim(lambda)+STRING(197B)
  xtitle=d.axes(1)+' (")'
  ytitle=d.axes(2)+' (")'
  xrange = d.origin(1) + [-d.spacing(1)/2.,(xsize-0.5)*d.spacing(1)]
  yrange = d.origin(2) + [-d.spacing(2)/2.,(ysize-0.5)*d.spacing(2)]
  
  Plot,[0],/NODATA,$
     xrange=xrange,yrange=yrange,$
         xstyle=1+4,ticklen=-.06,yticks=ysize/20,ystyle=1+4,$
     ytickformat='(I5)',xticks=xsize/30,$
         xtitle=xtitle,ytitle=ytitle,$
     position=[info(0).offs(2),info(0).offs(3),$
               xsize-1+info(0).offs(2),ysize-1+info(0).offs(3)],/DEVICE
  
  tvscl,wslice,info(0).offs(2),info(0).offs(3)
  clipbox
  dummy = pstore(1,xsize,ysize)
  
  ;; Make new "dummy" axes 
  
  leftx = round(info(0).offs(2)-info(0).offs(0))
  rightx = leftx + xsize MOD 2
  bottomy = round(info(0).offs(3)-info(0).offs(1))
  topy = bottomy + ysize MOD 2
  
  xrange = d.origin(1) + d.spacing(1)*[-leftx,rightx+xsize]
  yrange = d.origin(2) + d.spacing(2)*[-bottomy,topy+ysize]
  
  plot,[0],/nodata,/noerase,$
     xrange=xrange,yrange=yrange,$
     xstyle=1,ticklen=-.06,yticks=ysize/20,ystyle=1,$
     ytickformat='(I5)',xticks=xsize/30,$
     xtitle=xtitle,ytitle=ytitle,$
     position=[info(0).offs(0)-1,info(0).offs(1)-1,$
               120+info(0).offs(0),120+info(0).offs(1)],/DEVICE
  
  pqlzoom,wslice,P_reg=dummy,/change,$ ; Update calls
     title=title,xtitle=xtitle,ytitle=ytitle
  
  pqlprofile,wslice,P_reg=dummy,/change,$
     title=title,xtitle=xtitle,ytitle=ytitle
  
  sol_x = d.origin(1) + info(0).coord(0) * d.spacing(1)
  sol_y = d.origin(2) + info(0).coord(1) * d.spacing(2)
  Widget_CONTROL,info(i).tw_exp,Set_VALUE=trim(info(0).coord(0)+1)
  Widget_CONTROL,info(i).tw_slitp,Set_VALUE=trim(info(0).coord(1)+1)
  Widget_CONTROL,info(i).tw_sx,Set_VALUE=trim(sol_x)
  Widget_CONTROL,info(i).tw_sy,Set_VALUE=trim(sol_y)
  fulltitle = d.label + ' ' + '(' + trim(sol_x) + '",'+ trim(sol_y)+'")'
    
;
; Update the profile window
;
  
  SetWindow,info(i).wavPROF
  
  profile = gt_spectrum(qlds,WINDOW=info(i).window-1, $
                        xix=info(i).coord(0),yix=info(i).coord(1), $
                        lambda=lambda)
  
  ix = WHERE(profile GT 0,count)
  IF count gt 0 THEN BEGIN
;
; Should we do some profile analysis?
; 
      IF (info(i).gauss eq 1) THEN BEGIN
; Find background and subtract from intensity array
          c = poly_fit(lambda(ix),profile(ix),1,back)
          profile(ix) = profile(ix)-back
          int_fwhm_2,lambda(ix),profile(ix),totint,fwhm,shft
	  cwave = QLDS.DetDesc(info(i).window-1).wavelength ;
	  vel=((shft-cwave)/cwave) * 3.0E+5
          shft=vel
          profile(ix) = profile(ix)+back
      EndIF
      
      IF (info(i).gauss EQ 2) THEN BEGIN
	  gfit, QLDS, lambda, profile, ix,info,i,gcoeff,iny,lamdum
	  totint=gcoeff(0)*1.77*gcoeff(2)
	  fwhm=gcoeff(2)*(2*(sqrt(alog(2))))
	  shft=gcoeff(1)
       EndIF
      
      IF (info(i).gauss eq 0) THEN BEGIN
	  totint = 0
	  fwhm = 0
	  shft  = 0
      END
      
      Widget_CONTROL,info(i).tw_int,$
	       Set_VALUE=string(format='(1f11.0)',totint)
      Widget_CONTROL,info(i).tw_fwhm,$
	       Set_VALUE=string(format='(1f11.3)',fwhm)
      Widget_CONTROL,info(i).tw_vel,$
	       Set_VALUE=string(format='(1f11.3)',shft(0))
;
; Plot the measured data points
;      
      !Y.style=0
      erase
      
      ymin = QLDS.detdesc(info(i).window-1).min
      ymax = QLDS.detdesc(info(i).window-1).max
      IF info(i).log_plot THEN begin
         ymin = ymin > 1
         Plot_IO,lambda(ix),profile(ix),psym=10,title='Counts vs Wavelength',$
            position=[.2,.1,.90,.90],xticks=3,$
            xrange=[MIN(lambda(ix)),MAX(lambda(ix))],xstyle=1,$
            yrange=[ymin,ymax]
      END ELSE BEGIN
         Plot,lambda(ix),profile(ix),psym=10,title='Counts vs Wavelength',$
            position=[.2,.1,.90,.90],xticks=3,$
            xrange=[MIN(lambda(ix)),MAX(lambda(ix))],xstyle=1,$
            yrange=[ymin,ymax]
      END
      
         
      info(i).P_PROF = pstore(1,N_elements(ix),1)
         
      pqlprofile,profile,lambda,/change,$
		P_reg=info(i).P_PROF,$
		title=fulltitle
      
;
; Mark the wavelength we're looking at.
;
      IF info(i).wpixel ge 0 and info(i).wpixel lt N_elements(profile) THEN $
	    OPlot,[lambda(info(i).wpixel)],[profile(info(i).wpixel)],psym=6
      
      ; Fix background!!!
      IF (info(i).gauss eq 2) THEN OPlot,lamdum,iny,linestyle=3
  END ELSE BEGIN
      erase
      xyouts,/normal,.05,.5,'All points Zero',charsize=2
  EndELSE

END





PRO dspWAV,QLDS,windows,GROUP_LEADER=GROUP_LEADER,NOCHECK=NOCHECK
  common QLMGR,QL_ID,QL_DATA
  common QLSAVE,saveQLDS,SAVED
  
  On_Error,2
  if !DEBUG ne 0 then on_error,0
  
  xack,'DSPWAV has been decommissioned, use DSP_WAV instead'
  RETURN
  
  IF N_PARAMS() LT 1 THEN MESSAGE,'Use: DSPWAV, QL_Data [, windows]'
  
;
; Check type with QLmanager
; 
  IF NOT Keyword_SET(NOCHECK) THEN BEGIN
      QLMGR,QLDS,valid
      IF NOT valid THEN message,'First argument is not a QL Data structure'
  EndIF
  
  IF N_elements(QLDS.DetDesc(0).ixstart) lt 3 THEN BEGIN
      cdsnotify,'DSPWAV is not meaningful for one- or two-dimensional data'
      return
  END
;
; Checking parameters...
;
  
  no_lines = N_elements(windows)
  
  IF (no_lines gt 4) THEN BEGIN
      Bell
      Print,' Sorry, Too many wavelength windows selcted (Max: 4).'
      Print,' Deselect some of the windows.'
      return
  END
  
  max_lines = min([ N_elements(QLDS.DetDesc(*)), 4 ])
  IF no_lines eq 0 THEN $
	windows=Indgen(max_lines) +1 $
  ELSE IF no_lines lt max_lines THEN $
	FOR i=1,max_lines DO $
		IF total(windows eq i) eq 0 THEN windows=[windows,i]
  
  DEVICE,get_screen_size = scrnsize
  
  dim = gt_dimension(qlds,/nocheck)
  ysize= 120 + 100	  		; +70 to allow scales/titles.
  xsize= 120 + 100
  
  offset=[0,500]
  
  wavbase = QL_Data(QLDS.QL_no).dspwavbase
  IF Widget_INFO(wavbase,/valid_id) THEN BEGIN
      IF Widget_INFO(wavbase,/realized) THEN BEGIN
	  Widget_CONTROL,wavbase,/show
	  return
      EndIF
  EndIF
  
  IF N_elements(GROUP_LEADER) eq 1 THEN $
	  wavbase = Widget_BASE(title='Wavelength Slice/Profile',/column,$
		  GROUP_LEADER=GROUP_LEADER,uvalue=GROUP_LEADER,$
		  xoffset=offset(0),yoffset=offset(1)) $
  ELSE	  wavbase = Widget_BASE(title='Wavelength Slice/Profile',/column,$
		  xoffset=offset(0),yoffset=offset(1))
  
  upper_row = Widget_BASE(wavbase,/row)
  
  cmouse_s={CMOUSE_S, btext:'', mtext:'', uvalue:'', avail:0}
  
  cmouse,wavbase,upper_row,$
     [{CMOUSE_S, 'Move spot', 'Move the crosshair spot', 'MOVE', 7},$
      {CMOUSE_S, 'Profiling', 'Plot profiles of data','PROFILE', 7},$
      {CMOUSE_S, 'Zooming',   'Zooming in on the data', 'ZOOM', 7}]
  
  control = Widget_BASE(upper_row,/column,space=0,ypad=0)
  ctrl1= Widget_BASE(control,/row,ypad=0)
  ctrl2= Widget_BASE(control,/row)
  
  wbase2 = WIDGET_BASE(wavbase,/row,/nonexclusive)
  dummy = WIDGET_BUTTON(wbase2,value='Logarithmic profile plot',$
                        uvalue='LOGPLOT')
  
  wbase = Widget_BASE(wavbase,/row)
  
  XPdMenu,['"Hardcopy" {','"To file" PSFILE','"To printer" PSPRINT','}'],$
	ctrl1

  
  
  
  dummy = {PSELECT_S, btext:'', mtext:'', uvalue:'', flags:0}
  menu=[{PSELECT_S, 'Simple', 'Simple (straightforward interpolation)',$
							       'ISIMPLE',0},$
       {PSELECT_S, 'Gaussian', 'Gaussian profile', 'IGAUSS',0},$
       {PSELECT_S, 'Amoebe', 'Amoebe fit', 'AMOEBE',0},$
       {PSELECT_S, 'Parabola', 'Parabola fit', 'PARABOLA',0}]
  
  dummy = cw_pselect(ctrl1,'Line fit:',menu)
  
  
  XPdMenu,['"DIAGNOSTICS" {','"Intensity/Velocity Map" INTVEL',$
		   '"Temperature/Density Map" TEMPDEN','}'],ctrl1
  
  XPdMenu,['"Show raster area" {',$
           '"IMAGE_TOOL" IMAGE_TOOL',$
           '}'],ctrl1
  
  hlp = Widget_BUTTON(ctrl1,value='HELP',uvalue='HELP')
  
  done = Widget_BUTTON(ctrl1,value='DONE',uvalue='DONE')
  
  bases = lonarr(N_elements(windows))
  sizes = bases
  
  load_red,0.5
  info = REPLICATE({WINDOW:0,$  ; Detector window
                    $           ; for each label/text needed to display
                    $           ; information, a tag should be added, and
                    $           ; the widget ID of the label/text.... should
                    $           ; be stored
                    top:0L,$    ; Top level base widget ID
                    tw_int:0L,$ ; Line Intensity
                    tw_fwhm:0L, $ ; Line width
                    tw_vel:0L, $ ; Line shift
                    tw_exp:0L,  $ ; Exposure No.
                    tw_slitp:0L,  $ ; Position along slit (pixel)
                    tw_sx:0L,  $ ; Solar_x
                    tw_sy:0L,   $ ; Solar_y
                    gauss:1L,   $ ; Info about integration method
                    wpixel:0,$  ; Wavelength pixel
                    wsel:0, $
                    coord:[0,0],$ ; Spatial coordinates
                    $                    
                    TVbase:0L,$ ; 
                    wavTV:0L,$  ; Window ID of TV window
                    P_TV:0L,$
                    wwavTV:0L,$ ; Widget id.
                    offs:fltarr(4),$
                    $
                    wavPROF:0L,$ ; ID of Profile window
                    P_PROF:0L,$
                    LOG_PLOT:0L,$         ; Profile plot method.
                    wwavprof:0L $ ; Widget id.
		   },N_elements(windows))
  
  twe  = Widget_LABEL(ctrl2,value='Exposure:')

  info(*).tw_exp = Widget_LABEL(ctrl2)
  twe  = Widget_LABEL(ctrl2,value='      Pos. Slit:')
  info(*).tw_slitp = Widget_LABEL(ctrl2)
  twe  = Widget_LABEL(ctrl2,value='      Solar_X:')
  info(*).tw_sx = Widget_LABEL(ctrl2)

  twe  = Widget_LABEL(ctrl2,value='      Solar_Y:')
  info(*).tw_sy = Widget_LABEL(ctrl2)
  
  choices = QLDS.DetDesc(*).label
  
  wnd_texts = replicate('',N_elements(choices))
  FOR i=0,N_elements(choices)-1 DO wnd_texts(i)=trim(i+1)
  
;
; Setting up the row of window displays..
;
  
  d = gt_dimension(qlds)
  
  x0 = 65.0001
  y0 = 60.0001
  xoff = (120-d.ssolar_x)/2 + x0
  yoff = (120-d.ssolar_y)/2 + y0
  
  info(0).offs(*) = [x0,y0,xoff,yoff]
  
  FOR i=0, N_elements(windows)-1 DO BEGIN
      base = Widget_BASE(wbase,/column)
      bases(i) = base
      sizes(i) = WaveSize(QLDS,windows(i),/NOCHECK)
      upper = Widget_BASE(base,/row)
      
      menu = replicate({PSELECT_S},N_elements(wnd_texts))

      menu(*).btext = choices

      menu(*).mtext = choices

      if N_elements(wnd_texts) eq 1 then wnd_texts=wnd_texts(0)
      menu(*).uvalue= 'SETWINDW' + trim(i+1) + wnd_texts

      dummy = cw_pselect(base,'Window: ',menu,initial=windows(i)-1)
      
      
      wavdraw = Widget_DRAW(upper,xsize=xsize,ysize=ysize,$
			  /BUTTON_EVENTS,Uvalue='WAVDRAW',/frame)
      
      profdraw= Widget_DRAW(base,xsize=xsize,ysize=ysize,$
			  /FRAME,/BUTTON_EVENTS,uvalue = 'PROFDRAW')
      
      info(i).top = wavbase
      info(i).TVbase = wavdraw
      info(i).wwavTV = wavdraw
      info(i).wwavPROF=profdraw
      
      output = Widget_BASE(base,/column,/frame)
      
      twi=Widget_BASE(output,/row,xpad=0,ypad=0,space=0)
      
      tl1=Widget_LABEL(twi,value='Intensity:  ')
      info(i).tw_int=Widget_LABEL(twi)
      twf=Widget_BASE(output,/row,xpad=0,ypad=0,space=0)
      tl1=Widget_LABEL(twf,value='FWHM ('+string(197B)+'):   ')
      info(i).tw_fwhm=Widget_LABEL(twf)
      twv=Widget_BASE(output,/row,xpad=0,ypad=0,space=0)
      tl1=Widget_LABEL(twv,value='Velocity:   ')
      info(i).tw_vel=Widget_LABEL(twv)
      
      
   EndFOR
   
   
   QL_Data(QLDS.QL_no).dspwavbase = wavbase
   Widget_CONTROL,wavbase,/realize
   
   FOR i = 0,N_ELEMENTS(windows)-1 DO begin
      info(i).window = windows(i)
      info(i).wpixel = sizes(i)/2
      info(i).coord = [dim.ssolar_x/2,dim.ssolar_y/2]
      Widget_CONTROL,info(i).wwavTV,Get_VALUE=wavTV
      info(i).wavTV = wavTV
      Wset,wavTV
      info(i).P_TV = pstore(1,1,1,/clean) ; Size don't matter here...
      Widget_CONTROL,info(i).wwavPROF,Get_VALUE=wavPROF
      info(i).wavPROF = wavPROF
      Wset,wavPROF
      info(i).P_PROF = pstore(1,1,1,/clean)
      draw_wave,QLDS,info,i
   ENDFOR
   
  Widget_CONTROL,upper_row,Set_UVALUE=info
  
;
; Set user value of top base to QLDS 
;
  ql_no = QLDS.QL_NO
  qlds_report,wavbase,QLDS,store = (N_elements(GROUP_LEADER) eq 0)
  
  Xmanager,'DSPWAV',wavbase,CLEANUP='qlmgrclean'
  
  IF strmid(!VERSION.RELEASE,2,1) eq '0' THEN return
  
  IF N_elements(GROUP_LEADER) eq 0 and NOT Keyword_SET(nokeep)  THEN BEGIN
      WHILE Widget_INFO(base,/valid_id) ne 0 DO Xmanager
      qlds_checkin,ql_no
      assign_nocopy,QLDS,saveQLDS
  END
  
END


