;+
; Project     : SOHO - CDS     
;                   
; Name        : GDSPSPEC
;
; Purpose     : Displays all spectral windows from specific exposures.
;
; Explanation : The individual spectral windows are deselected 
;               into their corresponding positions on the detector.
;               Line selection and windows given in Blue Book
;               can be displayed. Measurements of wavelengths
;               and positions on the detector is possible
;               together with line identification.
;
;               
; Use         : gdspspec,QLDS
;    
; Inputs      : QLDS: A CDS QL data structure
;               
; Opt. Inputs : None.
;               
; Outputs     : None.
;                      
;               
; Opt. Outputs: Should return the displayed data for further analyses
;               and PostScript file. To be written. 
;
; Keywords    : GROUP_LEADER:
;                       If this invoked by any other CDS QuickLook 
;                       display routine (group leader responsible for 
;                       keeping the QLDS), this keyword should have
;                       the widget ID of the top level base of the
;                       calling routine.
;
;               NOCHECK :
;                       Set to skip QLMGR parameter check.
;
; Calls       : ANYTIM2UTC(), BELL, CDSLOG, CDSNOTIFY, CDSPICKFILE(),
;               CONCAT_DIR(), DATATYPE(), GET_QLDS(), LOAD_WAVECAL(),
;               NEXPOSURES(), PCONVERT, PFIND, PICK_LINE(), POINTSLICE,
;               PQLPROFILE, PRESTORE, PS, PIX2WAVE(), PSCLOSE, PSPLOT,
;               PSTORE(), QLHELP, QLMGR(), QL_RD_LLIST(), SETWINDOW, SET_QLDS,
;               TRIM(), WAVESIZE() (Per 7 July 1994)
;
; Common      : QLSAVE
;               
; Restrictions: QLDS must be a valid CDS QL data structure
;               Requires dummy GIS spectrum $CDS_GDS_MODEL/gis_dummy.dat 
;               Looks for line lists in $CDS_PLAN_TECH and $CDS_INFO,
;               with names 'llist_*'.
;               For HELP displays the file $CDS_INFO/gdspspec.hlp
;
; Side effects: 
;               
; Category    : QuickLook, Display, NIS
;               
; Prev. Hist. : From the NIS version + Dave Pike's TP_PAGE1
;
; Written     : Stein Vidar Hagfors Haugan, March 1994
;               
; Modified    : SVHH, 16 May 1994
;                       Added call to CONCAT_DIR('$CDS_GDS_MODEL',..)
;
;               SVHH, 21 May 1994
;                       Switched GEN_WAV(?) --> pix2wave()
;                       Removed some debris from old NDSPSPEC
;               SVHH, Version 2, 7 June 1994
;                       Added Line ID, on-screen line list marking,
;                       PostScript file dump, profiling/measuring.
;                       Added some support for calibration files.
;               PB,   July 6, 1994
;                       Added call to help routine QLHELP
;               PB,    August 26, 1994
;                       Changed dety --> detx in nmake_spect.pro
;               PB,   Sept 21 - 94    Added Help using WIDG_HELP
;               SVHH, Version 3, 25-January-1995
;                     Fixed error in locating band 1 windows.
;               Version 4, SVHH, 27 February 1996
;                     Improved handling of various dimensionality.
;                     (Sort of preliminary facelift)
;               Version 5, SVHH, 14 April 1996
;                     Using LOAD_WAVECAL
;               Version 6, SVHH, 16 April 1996
;                     Plot scaling changed.
;               Version 7, SVHH, 27 November 1996
;                     Fixed disappearing QLDS when called from prompt.
;                      
; Version     : 7, 27 November 1996
;- 


PRO GDSPSPEC_EVENT, EV
  common QLSAVE,QLDS,SAVED
  
;
; Get widget number of first child of top level base
;
  
  status_id = Widget_INFO(ev.top, /child)
  Widget_CONTROL,status_id,Get_UVALUE=STATUS ;,/no_copy
  Widget_CONTROL,ev.id,Get_UVALUE=UVALUE
  Widget_CONTROL,ev.top,/hourglass
  
  CASE strmid(uvalue,0,8) OF
      'POINT': BEGIN
	  status.func =	0
	  Widget_CONTROL,status.func_label,Set_VALUE='Cursor action: Measuring'
      END
      
      'LINE':  BEGIN
	  status.func =	1
	  Widget_CONTROL,status.func_label,Set_VALUE='Cursor action: Line ID'
      END
      
      'PROFILE': BEGIN
	  status.func =	2
	  Widget_CONTROL,status.func_label,Set_VALUE='Cursor action: Profile'
      END
      
      'XIX': BEGIN
         status.xix = ev.value
         qlds_checkin,widget=ev.top
         gmake_spec,qlds,status
      END
      
      'YIX': BEGIN
         status.Yix = ev.value
         qlds_checkin,widget=ev.top
         gmake_spec,qlds,status
      END
      
      'TIX': BEGIN
         status.tix = ev.value
         qlds_checkin,widget=ev.top
         gmake_spec,qlds,status
      END
      
      'DONE':BEGIN
	  Widget_CONTROL,ev.top,/destroy
	  return
      EndCASE
      
      'PSPRINT': BEGIN
	  gdsp_redisplay,STATUS,psfile='_print_'
     EndCASE
      
      'PSFILE':	BEGIN
	  file = cdspickfile(filter='*.ps',/fix_filter,xoffset=500,yoffset=500)
	  IF N_elements(file) gt 1 THEN	BEGIN
	      cdsnotify,ev.top,'Select only one file'
	      return
	  EndIF
	  IF file eq ''	THEN return
	  gdsp_redisplay,STATUS,psfile=file
      EndCASE
      
      'HELP': BEGIN
        widg_help,'gdspspec',$
         title = 'Help: Deselected Spectrum',group=ev.top
      EndCASE
      
      'List 1':	cdsnotify,' To be written.........',group=ev.top
      
      'VDSD':BEGIN
	  openr,lun,concat_dir('$CDS_GDS_MODEL','gis_dummy.dat'),/get_lun
	  adum=assoc(lun,intarr(2048))
	  status.gis(*,0)=adum(0)
	  status.gis(*,1)=adum(1)
	  status.gis(*,2)=adum(2)
	  status.gis(*,3)=adum(3)
          FREE_LUN,lun
          status.dummy = 1
	  gdsp_redisplay,status
      EndCASE
      
      'REAL':BEGIN
         status.dummy = 0
         qlds_checkin,widget=ev.top
         gmake_spec,qlds,status
      END
      
      'XLOADCT': XLoadCT,GROUP=EV.TOP
      
      'LINEID'	:cdsnotify,/Bell,'Not yet implemented...',group=ev.top
      
      'DRAW11':	gdspec_draw,status,ev,1
      'DRAW12':	gdspec_draw,status,ev,2
      'DRAW21':	gdspec_draw,status,ev,3
      'DRAW22':	gdspec_draw,status,ev,4
      
      'DRAW1': gdspec_draw,status,ev,status.view
      
      
      'VIEW5678': BEGIN
	  status.view =	fix(strmid(uvalue,8,1))
	  IF status.view eq 0 THEN txt = 'All' ELSE txt='GIS '+trim(status.view)
	  txt =	"Display detector :"+txt
	  Widget_CONTROL,status.view_label,Set_VALUE=txt
	  gdsp_redisplay,status
      END
      
      'MARKWIND': BEGIN
	  status.markwind=strmid(uvalue,8,1000)
	  gdsp_redisplay,status
      END
      
      ELSE:$
	IF strmid(uvalue,0,1) eq 'M' THEN $
	BEGIN
	  cdsnotify,/Bell,'Not yet implemented....',group=ev.top
      END ELSE IF strmid(uvalue,0,1) eq	'L' THEN $
      BEGIN
	  cdsnotify,/Bell,'Not yet implemented....',group=ev.top
      END
      
  EndCASE
  
  
;
; Put back status information
;
  
  Widget_CONTROL,status_id,Set_UVALUE=STATUS ;,/no_copy
  
END


;
; Write to log file.
;
PRO gdspec_log,status,ev,detector
  Widget_CONTROL,status.infoline,Get_VALUE=info
  cdslog,group=ev.top,info
END




;
;
;
;
PRO gdspec_draw,status,ev,detector
  
  IF ev.press eq 2 THEN	gdspec_log,status,ev,detector
  IF ev.press ne 1 THEN	return
  On_Error,2
  IF !debug ne 0 THEN On_Error,0
  
;
; Is it inside a draw window?
; 
  
  P = pfind(ev,found)
  
  IF NOT found THEN return
  
;
; Set the window...
;
  prestore,P
  
  detstr = trim(detector)
  
  wavl=status.wavl(*,detector-1)
  det =status.gis(*,detector-1)
  
  xpix = pconvert(P,ev.x,/DEVICE,/to_pix)  ; Use P (if it's the big window)
  wl = wavl(xpix)
  
  Ndims	= N_elements(status.origin)
  
  IF Ndims gt 1	THEN $
	axis1 =	status.origin(1) + status.spacing(1) * (status.xix)	$
  ELSE	axis1 =	0.0
  
  IF Ndims gt 2	THEN $
	axis2 =	status.origin(2) + status.spacing(2) * (status.yix)	$
  ELSE axis2 = 0.0
  
  IF Ndims gt 2	THEN $
	postitle = '   ('+status.axes(1)+','+status.axes(2)+$
	') = ('+trim(axis1)+','+trim(axis2)+')'	$
  ELSE IF Ndims	gt 1 THEN $
	postitle = '  ('+status.axes(1)+') = '+trim(axis1) $
  ELSE postitle='POINT'
  
  CASE status.func OF
      0: BEGIN	; Coordinates/Intensity
	  text = 'Wavelength='+trim(wl)+string(197b)
	  text = text +	'  Intensity='+trim(det(xpix))
	  text = text +	' Solar position'+postitle
	  text = text+'  GIS '+trim(detector)
	  text = text+'  Pixeladdress (DISPERSION)'
	  text = text +	'= ('+trim(xpix)+')'
	  Widget_CONTROL,status.infoline,Set_VALUE=text
      END
      
      
      1: BEGIN	; Line identification
	  dummy	= pick_line(wl,delta=0.1)
      EndCASE
      
      2: BEGIN ; Profile
	  pqlprofile,det,wavl,event=ev,xtitle='Wavelength',$
		       ytitle='Pixels along slit',$
		       title='GIS '+trim(detector)+postitle,$
		       datatitle='Counts'
      EndCASE
  EndCASE
  
END


PRO put_glines,status,psfile=psfile
  
  lines	= [0,0,0,0,0]
  IF Keyword_SET(psfile) THEN plotregs = status.ps_reg $
  ELSE			      plotregs = status.plotreg
  
  IF status.markwind ne	'' THEN	BEGIN
      mw = ql_rd_llist(status.markwind)
      mw = mw(sort(mw(*).lam))
      FOR i=0,N_elements(mw)-1 DO BEGIN
	  lam =	mw(i).lam
	  IF lam lt pix2wave('GIS1',2047) THEN detector=1 $
	  ELSE IF lam lt pix2wave('GIS2',2047) THEN detector=2 $
	  ELSE IF lam lt pix2wave('GIS3',2047) THEN detector=3 $
	  ELSE					    detector=4
	  IF lam lt pix2wave('GIS'+trim(detector),0) THEN goto,not_valid_wind
	  line = lines(detector)
	  lines(detector) = lines(detector) + 1
	  IF status.view eq 0 THEN P_reg = plotregs(detector) $
	  ELSE IF status.view eq detector THEN P_reg = plotregs(0) $
	  ELSE goto,not_valid_wind
	  prestore,P_reg
	  block	= bytarr(10,5)
	  IF NOT Keyword_SET(psfile) THEN BEGIN
	      xpos = pconvert(P_reg,lam,/data,/to_dev)
	      tv,block+200,xpos-5,!P.clip(1)-5
	  END ELSE BEGIN
	      xpos = pconvert(P_reg,lam,/data,/to_norm)
	      xstart = xpos - .01
	      xsize = .02
	      ystart = pconvert(P_reg,!P.clip(1),/dev,/to_norm,/Y) - .01
	      ysize  = .01
	      tv,block+200,xstart,ystart,xsize=xsize,ysize=ysize,/normal
	  END
	  Text	= mw(i).id + ' !7k!3'+trim(mw(i).lam,'(f6.2)')
	  IF NOT Keyword_SET(psfile) THEN BEGIN
	      Txpos = !P.clip(0)/2 + 140*(line/3)
	      Typos = !P.clip(1) - 50 -	(line mod 3)*10
	      xyouts,Txpos,Typos,Text,/dev,charsize=1.2,width=width
	      Txendpos = Txpos + pconvert(P_reg,width,/normal,/to_dev) + 4
	  END ELSE BEGIN
	      Txpos = pconvert(P_reg,!P.clip(0),/dev,/to_norm) $
			- .02+0.15*(line/4)
	      Typos = pconvert(P_reg,!P.clip(1),/dev,/to_norm,/Y) $
			- .09 -	(line mod 4)*.025
	      xyouts,Txpos,Typos,Text,/norm,width=width,charsize=0.8
	      Txendpos = Txpos + width +.01
	  END
	  
	  IF NOT Keyword_SET(psfile) THEN BEGIN
	      Txuppos =	Txpos +	120+5*(line mod	3)
	      Tyuppos =	!P.clip(1) - 40
	      IF xpos lt Txuppos THEN Tyuppos =	Tyuppos	+ 4*(line mod 6) $
	      ELSE		       Tyuppos = Tyuppos + 35 -	5*(line	mod 6)
	      xvec = [Txendpos,Txuppos,Txuppos,xpos,xpos]
	      yvec = [Typos+2,Typos+2,Tyuppos,Tyuppos,!P.clip(3)]
	  END ELSE BEGIN
	      Txuppos =	Txpos +	.12 + .008*(line mod 4)
	      Tyuppos =	pconvert(P_reg,!P.clip(1),/dev,/to_norm,/Y) - .07
	      IF xpos lt Txuppos THEN Tyuppos =	Tyuppos	+ .005*(line mod 8) $
	      ELSE		    Tyuppos = Tyuppos +	0.030-.005*(line mod 8)
	      xvec = [Txendpos,Txuppos,Txuppos,xpos,xpos]
	      yvec = [Typos+.005,Typos+.005,Tyuppos,Tyuppos,$
			pconvert(P_reg,!P.clip(3),/dev,/to_norm,/Y)]
	  END
	  IF NOT Keyword_SET(psfile) THEN $
	      plots,xvec,yvec,linestyle=line mod 3,/dev	$
	  ELSE plots,xvec,yvec,linestyle=line mod 3,/norm
	  not_valid_wind:
      EndFOR
  EndIF
  
END





PRO put_gspectra,status,psfile=psfile
  
  Ndims	= N_elements(status.origin)
  
  IF Ndims GT 1	THEN $
	axis1	= status.origin(1) + status.spacing(1) * (status.xix) $
  ELSE	axis1	= 0.0
  
  IF Ndims gt 2	THEN $
	axis2	= status.origin(2) + status.spacing(2) * (status.yix) $
  ELSE axis2 = 0.0
  
  IF Ndims gt 2	THEN $
	postitle = '   ('+status.axes(1)+','+status.axes(2)+$
	') = ('+trim(axis1)+','+trim(axis2)+')'	$
  ELSE IF Ndims	gt 1 THEN $
	postitle = '  ('+status.axes(1)+') = '+trim(axis1) $
  ELSE postitle='POINT'
  
  IF status.view eq 0 THEN BEGIN
      FOR i=1,4	DO BEGIN
	  xrange = [status.wavl(0,i-1),status.wavl(2047,i-1)]
	  position=[.08,.3,.97,.9]
	  noerase=0
	  IF NOT Keyword_SET(psfile) THEN prestore,status.plotreg(i) $
	  ELSE BEGIN
	      noerase=1
	      position([0,2]) =	(position([0,2]) + (i-1) mod 2)/2.0
	      position([1,3]) =	(position([1,3]) + 1 - (i-1)/2)/2.0
	  EndELSE
	  Plot_IO,status.wavl(*,i-1),status.gis(*,i-1) >1,$
		      position=position,noerase=noerase,$
		      title = 'GIS '+trim(i)+postitle,$
		      xrange=xrange,xstyle=1
	  IF Keyword_SET(psfile) THEN P	= pstore(i,2048,1) $
	  ELSE			      P	= pstore(1,2048,1)
	  IF Keyword_SET(psfile) THEN status.ps_reg(i) = P $
	  ELSE			      status.plotreg(i)= P
	  pqlprofile,P_reg=P,status.gis(*,i-1),status.wavl(*,i-1),$
		     title='GIS '+trim(i)+postitle,/change
      EndFOR
      Widget_CONTROL,status.spec_base4,map=1
      Widget_CONTROL,status.spec_base1,map=0
  END ELSE BEGIN
      Widget_CONTROL,status.spec_base1,map=1
      Widget_CONTROL,status.spec_base4,map=0
      position=[.05,.15,.95,.9]
      IF NOT Keyword_SET(psfile) THEN prestore,status.Plotreg(0)
      i	= status.view
      xrange = [status.wavl(0,i-1),status.wavl(2047,i-1)]
      Plot_IO,status.wavl(*,i-1),status.gis(*,i-1) >1,$
		position=position,$
		title =	'GIS detector '+trim(i)+postitle,$
		xrange=xrange,xstyle=1
      IF Keyword_SET(psfile) THEN status.PS_reg(0) = pstore(1,2048,1)
      dummy = pstore(1,2048,1)
      pqlprofile,P_reg=dummy,status.gis(*,i-1),status.wavl(*,i-1),$
	title='GIS '+trim(i)+postitle,/change
  EndELSE
END


PRO gdsp_redisplay,status,psfile=psfile
  IF Keyword_SET(psfile) THEN PS,psfile,/landscape
  put_gspectra,status,psfile=psfile
  put_glines,status,psfile=psfile
  IF Keyword_SET(psfile) THEN BEGIN
      IF psfile	eq '_print_' THEN psplot,/delete $
      ELSE			  psclose
      put_gspectra,status
      put_glines,status
  EndIF
END



PRO gmake_spec,QLDS,status
  
  IF !DEBUG ne 0 THEN On_Error,0
  
  FOR band=1,4 DO BEGIN
     IF gt_valid(qlds,band=band) THEN $
        status.gis(*,band-1) = gt_spectrum(qlds,band=band,xix=status.xix, $
                                           yix=status.yix, tix=status.tix) $
     ELSE status.gis(*,band-1) = 0
  END
  
  gdsp_redisplay,status
  
END




PRO gdspSPEC,QLDS,GROUP_LEADER=GROUP_LEADER,NOCHECK=NOCHECK
;
; Common block to avoid disappearance of QLDS in case this is
; a GROUP LEADER window...
;
  common QLSAVE,saveQLDS,SAVED
  
;
; Quick Look common block to avoid duplication/resetting of window position
;
  common QLMGR,QL_ID,QL_DATA
  
  IF !DEBUG eq 0 THEN On_Error,2
  
  IF n_params()	lt 1 THEN message,'Use: gdspspec, QL_Data'
  
;
; Check type with QLmanager
; 
  IF NOT Keyword_SET(NOCHECK) THEN BEGIN
      QLMGR,QLDS,valid
      IF NOT valid THEN	message,'First argument is not a QL Data structure'
  EndIF
  
  dummy = load_wavecal(qlds.header.detector, $
                       anytim2utc(qlds.header.date_obs),$
                       cal_struct = qlds.wavecal)
  
;------------------------------------------------------------------------
; Standard parameter checking...
;
  IF N_params()	ne 1 THEN $
	  message,'Use: EXAMPLE, QL_Data_structure'
  QLMGR,QLDS,valid
  IF NOT valid THEN message,' Parameter is not a CDS QL data structure'
  
;
; Generate wavelength calibration arrays.
;
  wavl = [ [pix2wave('GIS1',findgen(2048))],$
	   [pix2wave('GIS2',findgen(2048))],$
	   [pix2wave('GIS3',findgen(2048))],$
	   [pix2wave('GIS4',findgen(2048))] ]
  
;-----------------------------------------------------------------------
; If there is already a copy of this window, for this specific QLDS,
; we want to make sure it appears on the same spot as that one.
; NOTE: For each new window application having it's own top level
; base, the structure {QLINFO} (defined in qlmgr.pro) must be
; extended with a tag to hold the top level base number.
;
; Default position set: (x,y)
  
  offset=[0,500]
  xsize=1100	       ; X size draw window (twice the size of the smaller)
  ysize=250*2	       ; Y ------------ == -------------------------------
  
  BASE = QL_DATA(QLDS.QL_NO).GDSPSPECBASE
  
  IF Widget_INFO(BASE,/valid) THEN BEGIN
      IF Widget_INFO(BASE,/realized) THEN BEGIN
	  Widget_CONTROL,BASE,TLB_GET_OFFSET=offset
      EndIF
      Widget_CONTROL,BASE,/DESTROY
  END
  
  title	= 'Full (deselected) GIS spectrum'
;
; Creation of new top level base (alter only title or /row/column etc.)
;
  IF N_elements(GROUP_LEADER) eq 0 THEN	$
	  BASE=Widget_BASE(TITLE=title,/COLUMN,$	  ; This one becomes
		  XOFFSET=offset(0),YOFFSET=offset(1)) $  ; GROUP LEADER
  $
  ELSE	  BASE=Widget_BASE(TITLE=title,$		  ; This one HAS
		  XOFFSET=offset(0),YOFFSET=offset(1), $  ; a group leader
		  UVALUE=GROUP_LEADER,$
		  GROUP_LEADER=GROUP_LEADER,/COLUMN)
;---------------------------------------------------------------------
  
  
;---------------------------------------------------------------------
; Create the rest of the Widgets on screen:
;
  
  firstchild = Widget_BASE(BASE,/frame,/column)	; First child BASE
  innerchild = Widget_BASE(firstchild,frame=0,/row)
  secondchild= Widget_BASE(BASE,/frame,/column)	; Second child BASE
  
  XPdMenu,['"Cursor action: Measuring" {',$
		'"Measuring positions/intensities" POINT',$
		'"Line identification" LINE',$
		'"Profiling" PROFILE',$
	   '}'],innerchild,base=func_base
  func_label = Widget_INFO(func_base,/child)
  
  
  XPdMenu,['"Display detector: All" {',$
		'"ALL "   VIEW56780',$
		'"GIS 1 " VIEW56781',$
		'"GIS 2 " VIEW56782',$
		'"GIS 3 " VIEW56783',$
		'"GIS 4 " VIEW56784',$
	   '}'],innerchild,base=func_base
  view_label = Widget_INFO(func_base,/child)
  
;
; Find the available window/linelists
; PREDEFINED
  
  linelists = findfile(concat_dir('$CDS_PLAN_TECH','llist_*'),count=count)
  IF count gt 0	THEN BEGIN
      headers =	strarr(count)
      hdr = ''
      FOR i=0,N_elements(linelists)-1 DO BEGIN
	  openr,lun,linelists(i),/get_lun
	  readf,lun,hdr
	  headers(i)=hdr
	  close,lun
	  free_lun,lun
      END
      ix = where(strpos(headers,'(GIS)') ne -1,count)
      IF count gt 0 THEN BEGIN
	  linelists = linelists(ix)
	  headers = headers(ix)
      END ELSE BEGIN
	  headers = ['None available']
	  linelists = ['']
      EndELSE
  END ELSE BEGIN
      headers =	['None available']
      linelists	= ['']
  EndELSE
  
  markpredef = '"' + headers + '" MARKWIND' + linelists
  
;
; USER DEFINED
;
  
  linelists = findfile(concat_dir('$CDS_INFO','llist_*'),count=count)
  IF count gt 0	THEN BEGIN
      headers =	strarr(count)
      hdr = ''
      FOR i=0,N_elements(linelists)-1 DO BEGIN
	  openr,lun,linelists(i),/get_lun
	  readf,lun,hdr
	  headers(i)=hdr
	  close,lun
	  free_lun,lun
      END
      ix = where(strpos(headers,'(GIS)') ne -1,count)
      IF count gt 0 THEN BEGIN
	  linelists = linelists(ix)
	  headers = headers(ix)
      END ELSE BEGIN
	  headers = ['None available']
	  linelists = ['']
      EndELSE
  END ELSE BEGIN
      headers =	['None available']
      linelists	= ['']
  EndELSE
  
  markuser = '"' + headers + '" MARKWIND' + linelists
  
  XPdMenu,[ '"Help"		    HELP',$
	    '"Done"  		    DONE',$
	    '"Mark lines" {',$
		'"Clear windows" MARKWIND',$
		'"Predefined lists" {',$
			markpredef,$
		'}',$
		'"User defined list"{',$
			markuser,$
                '}',$
	    '}',$
            '"Observed/Dummy" {',$
                '"Dummy Spectrum"           VDSD',$
                '"Observed Spectrum"        REAL',$
            '}',$
	    '"PostScript"  {',$
		'"To file" PSFILE',$
		'"To printer" PSPRINT',$
	    '}'],innerchild
  
  select_base =	innerchild ;widget_base(firstchild,frame=0,/row)
  
  axes = QLDS.DetDesc(0).axes
  ixsize = QLDS.DetDesc(0).ixstop - QLDS.DetDesc(0).ixstart + 1
  Ndims	= N_elements(ixsize)
  
  dims = gt_dimension(qlds)
  
  print,ndims,ixsize

  IF dims.ssolar_x GT 1 THEN $
     wslider = WIDGET_SLIDER(select_base,minimum=0,maximum=dims.ssolar_x-1,$
                             title='X index',/frame,xsize=2*dims.ssolar_x+50,$
                             uvalue='XIX')
  
  IF dims.ssolar_y GT 1 THEN $
     wslider = WIDGET_SLIDER(select_base,minimum=0,maximum=dims.ssolar_y-1,$
                             title='Y index',/frame,xsize=2*dims.ssolar_y+50,$
                             uvalue='YIX')
  IF dims.sdel_time GT 1 THEN $
     wslider = WIDGET_SLIDER(select_base,minimum=0,maximum=dims.sdel_time-1,$
                             title='Time index',/frame,$
                             xsize=2*dims.sdel_time+50,$
                             uvalue='TIX')
  
  rstinfo = Widget_TEXT(firstchild,value='Information line')
  
  plot_base = Widget_BASE(secondchild)
  
  spec_base4 = Widget_BASE(plot_base,/column)
  
  upper_specbase = Widget_BASE(spec_base4,/row)
  
  specdraw11= Widget_DRAW(upper_specbase,xsize=xsize/2,ysize=ysize/2,$
			 /button_events,$
			  uvalue='DRAW11')
  specdraw12= Widget_DRAW(upper_specbase,xsize=xsize/2,ysize=ysize/2,$
			 /button_events,$
			  uvalue='DRAW12')
  lower_specbase = Widget_BASE(spec_base4,/row)
  specdraw21= Widget_DRAW(lower_specbase,xsize=xsize/2,ysize=ysize/2,$
			 /button_events,$
			  uvalue='DRAW21')
  specdraw22= Widget_DRAW(lower_specbase,xsize=xsize/2,ysize=ysize/2,$
			 /button_events,$
			  uvalue='DRAW22')
  
  spec_base1 = Widget_BASE(plot_base,/column)
  specdraw1 = Widget_DRAW(spec_base1,xsize=xsize,ysize=ysize,$
			 /button_events,$
			  uvalue='DRAW1')
  
  Widget_CONTROL,spec_base1,map=0
  
;
; Register the new base number in QLMGR data
;
  QL_DATA(QLDS.QL_NO).GDSPSPECBASE = BASE
  
  
; Realization of the window
;
  Widget_CONTROL, BASE,	/REALIZE
  
;--------------------------------------------------------------------
  
  
;--------------------------------------------------------------------
; Setting status information for this widget (could have used just
; FIRSTCHILD & SECONDCHILD, but to assure compatibilty with event
; routine, the following approach is recommended (identical to 
; the one in the event handling routine).
;
  status_id = Widget_INFO(base,/child)
;  wait,1
;  widget_control,spec_base4,map=0
;  widget_control,spec_base1,map=1
  
  Widget_CONTROL,specdraw1,Get_VALUE=specwindow1
  SetWindow,specwindow1
  plotreg = intarr(5)
  Plotreg(0) = pstore(1,0,0)
  Widget_CONTROL,specdraw11,Get_VALUE=specwindow11
  SetWindow,specwindow11
  Plotreg(1) = pstore(1,0,0)
  Widget_CONTROL,specdraw12,Get_VALUE=specwindow12
  SetWindow,specwindow12
  Plotreg(2) = pstore(1,0,0)
  Widget_CONTROL,specdraw21,Get_VALUE=specwindow21
  SetWindow,specwindow21
  Plotreg(3) = pstore(1,0,0)
  Widget_CONTROL,specdraw22,Get_VALUE=specwindow22
  SetWindow,specwindow22
  Plotreg(4) = pstore(1,0,0)
  
  
  STATUS = { func: 0,	$	    ; Mouse functionality
	     func_label:func_label, $
             dummy:0,$              ; Dummy flag
             view: 0,		  $ ; View of all four....
	     view_label:view_label, $
	     Plotreg:Plotreg,	  $ ; The Plot Region ID's
	     PS_reg:Plotreg,	  $ ; PostScript plot region ID's
	     spec_base1:spec_base1, $ ; The base of the "one detector view"
	     spec_base4:spec_base4, $ ; The base of the "four detectors view"
             XIX:0,YIX:0,TIX:0,$      ; Pixel Position
	     origin:QLDS.DetDesc(0).origin, $  ;
	     axes:QLDS.DetDesc(0).axes,	$
	     spacing:QLDS.DetDesc(0).spacing, $	;
	     markwind:'',	  $
	     infoline:rstinfo,	  $ ; Information line	
	     length:30,		  $
	     vertical:0,	  $
	     npixels:1,		  $
	     gis: lonarr(2048,4), $
	     wavl :wavl	$
	   }
  
  gmake_spec,QLDS,status
  
  Widget_CONTROL,status_id,Set_UVALUE=STATUS ;,/no_copy
  
;
; Set user value of top base to QLDS 
;
  ql_no	= QLDS.QL_NO
  qlds_report,BASE,QLDS,store =	(N_elements(GROUP_LEADER) eq 0)
  
  
;-----------------------------------------------------------------
; Standard procedure to ensure correct handling of
; GROUP LEADER windows / NON GROUP LEADER windows.
;
; Avoid disappearance of the QLDS parameter.
;
  
  Xmanager,'GDSPSPEC', BASE, CLEANUP='qlmgrclean'
  
  IF N_elements(GROUP_LEADER) eq 0 and NOT Keyword_SET(nokeep) THEN BEGIN
      WHILE Widget_INFO(base,/valid_id)	ne 0 DO	Xmanager
      qlds_checkin,ql_no
      assign_nocopy,QLDS,saveQLDS
  END
  
END

