;+
; Project     : SOHO - CDS
;                   
; Name        : NDSPSPEC()
;
; Purpose     : Displays all Spectral Windows from specific exposures.
;
; Explanation : The individual Spectral Windows are deselected 
;               into their corresponding positions on the detector.
;               Line selection and windows given in Blue Book
;               can be displayed. 
;               Measurements of wavelengths and positions on the 
;               detector is possible together with line identification.
;               
; Use         : ndspspec,QLDS
;    
; Inputs      : QLDS: A CDS QL data structure
;               
; Opt. Inputs : None.
;               
; Outputs     : None.
;                      
;               
; Opt. Outputs: Should return the displayed data for further analyses
;                        and PostScript file. To be written. 
;
; Keywords    : GROUP_LEADER:
;                       If this invoked by any other CDS QuickLook 
;                       display routine (group leader responsible for 
;                       keeping the QLDS), this keyword should have
;                       the widget ID of the top level base of the
;                       calling routine.
;
;               NOCHECK :
;                       Set to skip QLMGR parameter check.
;
; Calls       : ANYTIM2UTC(), ASSIGN_NOCOPY, BELL, CDSLOG, CDSNOTIFY,
;               CDSPICKFILE(), CLIPBOX, CMOUSE, CMOUSE_ACTION(), CONCAT_DIR(),
;               DATATYPE(), EXPSLICE(), LOAD_RED, LOAD_WAVECAL(),
;               NEXPOSURES(), PCONVERT(), PFIND(), PICK_LINE(), PIX2WAVE(),
;               PQLPROFILE, PQLZOOM, PRESTORE, PS, PSCLOSE, PSPLOT, PSTORE(),
;               QLDS_CHECKIN, QLDS_REPORT, QLMGR, QL_DIMENSIONS(),
;               QL_BSCALE(), QL_RD_LLIST(), SETWINDOW, TRIM(), WIDGET_BASE(),
;               WIDGET_TEXT(), WIDG_HELP (11-Dec-94 +20-Feb-95)
;               DSP_RASTER,RD_SERTS
;                
; Common      : QLSAVE: See QLDS_CHECKIN
;               
; Restrictions: QLDS must be a valid CDS QL data structure
;               Requires dummy NIS spectrum $CDS_VDS_MODEL/vds_dummy.dat
;               Looks for line lists in $CDS_PLAN_TECH and $CDS_INFO,
;               with names 'llist_*'.
;               For HELP displays the file $CDS_INFO/ndspspec.hlp
;               
; Side effects: 
;               
; Category    : QuickLook, Display, NIS
;               
; Prev. Hist. : None.
;
; Written     : Paal Brekke, 17 November 1993
;               
; Modified    : SVHH, 20 November 1993
;                       Brushed up documentation.
;               SVHH & PB, Jan-March 1993
;                       Zoom & Profile added, logging of status line info,
;                       Line identification
;               SVHH, 16 May 1994
;                       Added call to CONCAT_DIR('$CDS_VDS_MODEL',..)
;               SVHH, Version 1.5, 30 May 1994
;                       Switched QLPROFILE/QLZOOM -> PQL.../PQL...
;               SVHH, Version 2, 7 June 1994
;                       Modified cursor action selection,
;                       added on-screen plot of LLIST_??
;                       Added PostScript dump.
;               PB,   July 6, 1994
;                       Added call to help routine QLHELP
;               PB,   August 26, 1994
;                       Changed dety --> detx
;                               detz --> dety  in nmake_spect.pro
;               PB,   Sept 21 - 94    Added Help using WIDG_HELP
;               SVHH, Nov 15, Fixed XTICKS/XRANGE conflict for IDL 3.6c
;               SVHH, Dec 10, Added CMOUSE/CMOUSE_ACTION functionality,
;                       and added x/y-format/units in PQLZOOM-call.
;               SVHH, 20-Feb-1995, New data storage format, according
;                       to Bill's note on generating FITS files from
;                       telemetry. Incorporated the use of QL_DIMENSIONS()
;                       to check dimensionality. Should be more robust.
;                       Requires change of EXPSLICE as well.
;               SVHH, 21-Aug-1995, Version 3
;                       Modified call to QL_SIZE
;               Version 4, SVHH, 13-December-1995
;                       Started using GT_SCANY.
;               Version 5, SVHH, 20-December-1995
;                       Added SOLAR_Y axis (left) and moved
;                       the Y index axis to the right. Blanking the
;                       information line when selecting new exposure.
;               Version5.1, P.B, 20-February-1996
;                       Changed the reading procedure for 
;                       VDS_DUMMY now stored in another format.
;                       Line list selection added
;               Version5.2, P.B, 26-February-1996
;                       Added SOLAR_Y axis in PS mode as well
;                       Writing header info to PS file, and
;                       moved locations of plot
;                       Raster location passed on to IMAGE_TOOL
;               Version 6, SVHH, 4 March 1996
;                       Fixed a bug in calling gt_solarx/gt_solary/gt_time.
;               Version 7, SVHH, 12 April 1996
;                       Renamed gt_time -> gt_cds_time
;               Version 8, SVHH, 14 April 1996
;                       Using LOAD_WAVECAL.
;               Version 9, SVHH, 20 May 1996
;                       Fixed bug in calculating XIX/TIX from exposure.
;               Version 10, SVHH, 27 November 1996
;                       Using dsp_point to call IMAGE_TOOL. Changed button
;                       layout etc to have enough space for the exposure
;                       slider even with IDL's standard font. Also improved
;                       color scaling somewhat by taking MISSING into
;                       account.
;               Version 11, SVHH, 29 January 1997
;                       Made necessary modifications to work (sort of)
;                       with SUMLINE compression.
;
; Version     : 11, 29 January 1997
;- 


PRO NDSPSPEC_EVENT, EV
  common QLSAVE,QLDS,SAVED
  
;
; Get widget number of first child of top level base
;
  
  status_id = Widget_INFO(ev.top, /child)
  Widget_CONTROL,status_id,Get_UVALUE=STATUS ;,/no_copy
  Widget_CONTROL,ev.id,Get_UVALUE=UVALUE
  
  CASE strmid(uvalue,0,8) OF

  'EXPOSURE':BEGIN
     status.exposure=ev.value+1
     QLDS_checkin,widget=ev.top
     nmake_spec,QLDS,STATUS
     EndCASE
     
  'DONE':BEGIN
     ;; If this is the GROUP LEADER (check if ev.top.uvalue is a 
     ;; structure instead of a long)  we must ensure to 
     ;; return the QLDS parameter sent to the window creation 
     ;; routine.
     Widget_CONTROL,ev.top,Get_UVALUE=Base_uvalue
     IF datatype(Base_uvalue) eq 'STC' THEN BEGIN
        Widget_CONTROL,ev.top,Get_UVALUE=QLDS ;,/no_copy
     END
     Widget_CONTROL,ev.top,/destroy
     return
     EndCASE
     
  'HARD':BEGIN
     xps_setup,ps_setup,group=ev.top,status=ok
     if ok then ndsp_redisplay,STATUS,psfile=ps_setup
     ENDCASE
     
  'HELP':BEGIN
     widg_help,'ndspspec',$
        title = 'Help: Deselected Spectrum',group=ev.top
     EndCASE
     
  'IMTOOL':BEGIN
     dsp_point,qlds,group=ev.top
     ENDCASE
     
  'VDSD':BEGIN
     vdsdum = intarr(1024,256)
     openr,lun,concat_dir('$CDS_VDS_MODEL','vds_dummy.dat'),/get_lun
     readu,lun,vdsdum
     free_lun,lun
     status.nis2=vdsdum(*,9:128)
     status.nis1=vdsdum(*,129:248)
     ndsp_redisplay,status
     EndCASE
     
     'XLOADCT': XLoadCT,GROUP=EV.TOP
     'KLIST': status.linelist=0      
     'KHLIST': status.linelist=1      
     'CHIANTI': status.linelist=2      
     'CDS': status.linelist=3 
     
     'DRAW1': ndspec_draw,status,ev,1
     'DRAW2': ndspec_draw,status,ev,2
     'RASTER1': dsp_raster,qlds,group=ev.top,source='SYNOPTIC'
     'RASTER2': dsp_raster,qlds,group=ev.top,source='SUMMARY'
     'RASTER3': dsp_raster,qlds,group=ev.top,source='EIT'
     
     'MARKWIND': BEGIN
        status.markwind=strmid(uvalue,8,1000)
        ndsp_redisplay,status
     END
     
     ELSE: HELP,EV,/STR
  EndCASE
  
;
; Put back status information
;
  
  Widget_CONTROL,status_id,Set_UVALUE=STATUS ;,/no_copy
  
END


;
; Write to log file.
;
PRO ndspec_log,status,ev,detector
  Bell
  Widget_CONTROL,status.infoline,Get_VALUE=v
  cdslog,group=ev.top,v
END





;----------------------------------------------------------------------
; Handle all DRAW type events
;----------------------------------------------------------------------

PRO ndspec_draw,status,ev,detector
  common QLSAVE,QLDS,SAVED
  
  on_error,2
  if !DEBUG ne 0 then on_error,0

  QLDS_checkin,widget=ev.top
  
  action = cmouse_action(ev)

  IF action eq '' then return

  IF action eq 'LOG' THEN BEGIN
	ndspec_log,status,ev,detector
	RETURN
  ENDIF

;
; Is it inside a draw window?
; 
  
  P = pfind(ev,found)
  
  IF NOT found THEN return
  
;
; Set the window...
;
  
  prestore,p
  
  IF detector eq 1 THEN	BEGIN
      wavl = status.wavla
      det = status.nis1
  END ELSE BEGIN
      wavl = status.wavlb
      det = status.nis2
  END
  
  detx	= fix(pconvert(p,ev.x,/DEVICE,/to_pix))
  yix	= fix(pconvert(p,ev.y,/DEVICE,/to_pix,/Y))
  wl = wavl(detx)
  
  IF status.timeseries THEN BEGIN
     xix = 0
     tix = status.exposure - 1
  END ELSE BEGIN
     xix = status.exposure - 1
     tix = 0
  END
  
  dety = gt_dety(qlds,yix,detector,/noparcheck)
  solar_x = gt_solarx(qlds,detx,dety,xix,/noparcheck)
  solar_y = gt_solary(qlds,detx,dety,xix,/noparcheck)
  time = gt_cds_time(qlds,xix,yix,tix,/noparcheck)


;  ndsp_redisplay,status
  
  CASE action OF
      'MEASURE': BEGIN	; Coordinates/Intensity
	  text = 'Wavelength='+trim(wl)+string(197b)
	  text = text +	'      Intensity='+trim(det(detx,yix),'(I5)')
          text = text +	'      (SOLAR_X,SOLAR_Y) = (' +TRIM(solar_x)
          text = text + ','+trim(solar_y)+')'
          text = text+'      NIS '+trim(detector)
          text = text+'    Exposure '+trim(status.exposure)
	  text = text+'      Pixeladdress (DETX,DETY)'
	  text = text +	'=('+trim(detx,'(I4)')+','+trim(dety,'(I4)')+')'
	  Widget_CONTROL,status.infoline,Set_VALUE=text
      END
      
      
      'PICKLINE': BEGIN
       print,status.linelist
	  if status.linelist EQ 0 then dummy = pick_line(wl,delta=0.1)
	  if status.linelist EQ 1 then dummy = pick_line(wl,delta=0.1,/serts)
	  if status.linelist EQ 2 then CDSNOTIFY,'Not yet implemented!!'
	  if status.linelist EQ 3 then begin
             list=cds_sline(wl-0.05,wl+0.05)
             if n_elements(list) ge 3 then $
                list=list(0,*)+list(1,*)+list(2,*) else $
                list='No matching lines found in the CDS line list at:'$
                      +string(wl)
             cdslog,list
          end
      EndCASE
      
      'PROFILE': BEGIN ; Profile
	  pqlprofile,det,event=ev,$
		title=	'NIS '+trim(detector)+$
			' Exposure #'+trim(status.exposure)+status.obstime,$
		xtitle='Wavelength',xformat='(f6.2)',$
		ytitle='Pixels along slit',yformat='(I3)',$
		datatitle='Counts'
	  return
      EndCASE
      
      
      'ZOOM': BEGIN ; Zoom request
	  pqlzoom,det,$
		 event=ev,/cursor,$
		 title='Exposure #'+trim(status.exposure)+status.obstime,$
		 xtitle='Wavelength ',xformat='(f7.3)',xunits=string(197b),$
		 ytitle='Y index',yformat='(f5.1)',$
		 subtitle='Nis detector '+trim(detector)
	  return
      EndCASE
      
  EndCASE
  
END


;----------------------------------------------------------------------
; Mark lines/windows on the spectra
;----------------------------------------------------------------------
PRO put_nlines,status,psfile=psfile
  
  lines1 = 0 ;
  lines2 = 0 ;
  
  IF Keyword_SET(psfile) THEN PS_reg=pstore(1,1,1)
  IF status.markwind ne	'' THEN	BEGIN
      mw = ql_rd_llist(status.markwind)
      mw = mw(sort(mw(*).x_pos))
      FOR i=0,N_elements(mw)-1 DO BEGIN
	  IF mw(i).y_pos ge 128	THEN BEGIN
	      detector = 1
	      P_reg = status.plotreg1
	      IF 150*(lines1/3 + 2) lt mw(i).x_pos THEN	$
		      lines1 = lines1 +	3
	      lines = lines1
	      lines1 = lines1+1
	  END ELSE BEGIN
	      detector = 2
	      P_reg = status.plotreg2
	      IF 150*(lines2/3 + 2) lt mw(i).x_pos THEN	$
		      lines2 = lines2 +	3
	      lines = lines2
	      lines2 = lines2+1
	  END
	  prestore,P_reg
	  block	= bytarr(10,5)
	  xpos = mw(i).x_pos+!P.clip(0)
	  IF NOT Keyword_SET(psfile) THEN tv,block+200,xpos-5,!P.clip(1)-5 $
	  ELSE BEGIN
	      x0 = pconvert(P_reg,xpos-5,/dev,/to_norm)
	      y0 = pconvert(P_reg,!P.clip(1)-5,/dev,/to_norm,/Y)
	      x1 = pconvert(P_reg,xpos+5,/dev,/to_norm)
	      y1 = pconvert(P_reg,!P.clip(1),/dev,/to_norm,/Y)
	      y0 = (y0 + (2.-detector))/2.
	      y1 = (y1 + (2.-detector))/2.
	      prestore,PS_reg
	      tv,block+200,x0,y0,xsize=x1-x0,ysize=y1-y0,/norm
	      prestore,P_reg
	  END
	  Txpos	= !P.clip(0) + 150*(lines/3)
	  Typos	= !P.clip(1) - 50 - (lines mod 3)*10
	  Text = mw(i).id + ' !7k!3'+trim(mw(i).lam,'(f6.2)')
	  IF NOT Keyword_SET(psfile) THEN BEGIN
	      xyouts,Txpos,Typos,Text,/dev,charsize=1.2,width=width
	      Txendpos = Txpos + pconvert(P_reg,width,/normal,/to_dev) + 4
	  END ELSE BEGIN
	      prestore,PS_reg
	      xyouts,pconvert(P_reg,Txpos,/dev,/to_norm),$
		    (pconvert(P_reg,Typos,/dev,/to_norm,/Y)+2-detector)/2.,$
		    Text,/norm,width=width
	      prestore,P_reg
	      Txendpos = Txpos + pconvert(P_reg,width,/norm,/to_dev) + 5
	  END
	  
	  Txuppos = Txpos + 130+5*(lines mod 3)
	  Tyuppos = !P.clip(1) - 40
	  IF xpos lt Txuppos THEN Tyuppos = Tyuppos + 5*(lines mod 6) $
	  ELSE			  Tyuppos = Tyuppos + 35 - 5*(lines mod	6)
	  xvec = [Txendpos,Txuppos,Txuppos,xpos,xpos]
	  yvec = [Typos+5,Typos+5,Tyuppos,Tyuppos,!P.clip(3)]
	  IF NOT Keyword_SET(psfile) THEN $
	      plots,xvec,yvec,linestyle=lines mod 3,/dev $
	  ELSE BEGIN
	      prestore,PS_reg
	      plots,pconvert(P_reg,xvec,/dev,/to_norm),$
		    (pconvert(P_reg,yvec,/dev,/to_norm,/Y)+2.-detector)/2.,$
		    linestyle=lines mod	3,/norm
	      prestore,P_reg
	  EndELSE
	  
      EndFOR
  EndIF
END




;----------------------------------------------------------------------
; Plot spectrum (detector 1 or 2)
;----------------------------------------------------------------------


PRO put_nspectra,status,detector,psfile=psfile
  
  IF detector eq 1 THEN	BEGIN
      P_reg=status.plotreg1
      wavl = status.wavla
      det = status.nis1
      xticks = 13
      xtickv = findgen(xticks+1)*5 + 310
  END ELSE BEGIN
      P_reg=status.plotreg2
      wavl = status.wavlb
      det = status.nis2
      xticks = 22
      xtickv = findgen(xticks+1)*5 + 520
  END
  IF NOT Keyword_SET(psfile) THEN BEGIN
      Prestore,P_reg
      erase
  EndIF
  
  jx=75	& jy = 80
  
  sx=(QL_Size(size(det),2))(1)
  sy=(QL_Size(size(det),2))(2)
  
  xdelta = wavl(1) - wavl(0)
  
  IF status.markwind eq	'' THEN	xtitle='Wavelength ('+string(197b)+')' $
  ELSE				xtitle=''
  
  title='!3NIS '+trim(detector) +	' Exposure #'+trim(status.exposure)
               
  xrange=[wavl(0)-xdelta/2.,wavl(sx-1)+xdelta/2.]
  yrange=[.5,sy-.5]
  position = [jx,jy, jx+sx-1,jy+sy-1]     +.01 ; IDL BUG!!!!
  ytitle = 'Y index'
  
  scaled = det
  bscale,scaled,missing=status.missing
  
  IF NOT Keyword_SET(psfile) THEN BEGIN
      Plot,[0],title=title,xrange=xrange,$
         yrange=yrange,position=position,/DEVICE,$
         xstyle=1,ystyle=1+4,/noerase,/nodata,$
         xtitle=xtitle,ytitle=ytitle,$
         xticks=xticks,xtickv=xtickv,$
         xticklen=-.06,yticklen=-.01,xminor=5
      axis,yaxis=1,yrange=yrange,ystyle=1,ytitle=ytitle,yticklen=-0.01
      axis,yaxis=0,yrange=status.sol_yrange,ystyle=1,$
         ytitle='Solar_Y',yticklen=-0.01
      
      tv,scaled,jx,jy
      P	= pstore(1,sx,sy)
      IF detector eq 1 THEN status.plotreg1 = P	$
      ELSE		    status.plotreg2 = P
      
      pqlzoom,det,P_reg=P,/change,/cursor,title=title
      pqlprofile,det,/change,P_reg=P,title=title
  END ELSE BEGIN
      posx = pconvert(P_reg,position([0,2]),/DEVICE,/to_normal)
      posy = pconvert(P_reg,position([1,3]),/DEVICE,/to_normal,/Y)
      position = [posx(0),posy(0),posx(1),posy(1)]
      IF detector eq 1 THEN BEGIN
	  position(1) =	position(1)/2.+0.37          ; moved plot down
	  position(3) =	position(3)/2.+0.37          ; P.B
      END ELSE BEGIN
	  position(1) =	position(1)/2.-0.05
	  position(3) =	position(3)/2.-0.05
      END
      Plot,[0],title=title,xrange=xrange,$
	      yrange=yrange,$
	      position=position,/normal,$
	      xstyle=1,ystyle=1+4,/noerase,/nodata,$
	      xtitle=xtitle,ytitle=ytitle,$
	      xticks=xticks,xtickv=xtickv,$
	      xticklen=-.04,yticklen=-.01,xminor=5
      axis,yaxis=1,yrange=yrange,ystyle=1,ytitle=ytitle,yticklen=-0.01
      axis,yaxis=0,yrange=status.sol_yrange,ystyle=1,$
         ytitle='Solar_Y',yticklen=-0.01
      tv,scaled,position(0),position(1),$
			      xsize=position(2)-position(0),$
			      ysize=position(3)-position(1),/normal


  	dn=0.05
  	plots,[0.03,0.5],[0.99+dn,0.99+dn],/norm
  	plots,[0.03,0.5],[0.85+dn,0.85+dn],/norm
 	plots,[0.03,0.5],[0.97+dn,0.97+dn],/norm
	plots,[0.03,0.03],[0.85+dn,0.99+dn],/norm
	plots,[0.5,0.5],[0.85+dn,0.99+dn],/norm
 
  	xyouts,0.05,0.976+dn,'CDS-NIS observing parameters:',/norm,$
        charsize=0.7 
  	xyouts,0.05,0.955+dn,'Study (program number):    '+$
          strtrim(status.study,2),/norm,charsize=0.7 
  	xyouts,0.05,0.935+dn,'File name:                   '+$
          status.filen,/norm,charsize=0.7 
  	xyouts,0.05,0.915+dn,'Exposure time (s):           '+$
          strtrim(status.expt,2),/norm,charsize=0.7 
  	xyouts,0.05,0.895+dn,'Date/Exp. start time (UT):   '+$
          strtrim(status.expst),/norm,charsize=0.7 
  	xyouts,0.05,0.875+dn,'Slit:                          '+$
          strtrim(status.slitnum,2),/norm,charsize=0.7 
  	xyouts,0.05,0.855+dn,'Xcenter, Ycenter ("):         '+$
          strtrim(status.solarx,2)+'  '+ strtrim(status.solary,2),$
          /norm,charsize=0.7

  EndELSE
  clipbox
  
  return
END


PRO ndsp_redisplay,status,psfile=psfile

;-- check if user passed a postscript setup stucture from XPS_SETUP

  IF Keyword_SET(psfile) THEN BEGIN
   if datatype(psfile) eq 'STC' then begin
    ps,psfile.filename,color=psfile.color,port=psfile.portrait,$
    encap=psfile.encapsulated,$
    copy=psfile.copy,inter=psfile.interpolate 
    t_psfile=psfile.filename
   ENDIF ELSE BEGIN
    PS,psfile,/landscape
    t_psfile=psfile
   ENDELSE
  ENDIF

  WIDGET_CONTROL,status.infoline,set_value=''
  put_nspectra,status,1,psfile=t_psfile
  put_nspectra,status,2,psfile=t_psfile
  put_nlines,status,psfile=t_psfile
  IF Keyword_SET(psfile) THEN BEGIN
   if datatype(psfile) eq 'STC' then begin
    psplot,psfile.filename,que=psfile.printer,del=psfile.delete
   endif else IF psfile eq '#print#' THEN psplot,/delete 
  EndIF
END



PRO nmake_spec,QLDS,status
  
  IF status.timeseries THEN BEGIN
     xix = 0
     tix = status.exposure-1
  END ELSE BEGIN
     tix = 0
     xix = status.exposure-1
  END
  
  status.nis1 =	gt_scany(QLDS,xix=xix,tix=tix,band=1,/quick)
  status.nis2 =	gt_scany(QLDS,xix=xix,tix=tix,band=2,/quick)
  
  status.missing = qlds.detdesc(0).missing

  exps=gt_start(qlds,/exp,/ecs)
  status.expst=exps(status.exposure-1)

  ndsp_redisplay,status
  
END




PRO ndspSPEC,QLDS,GROUP_LEADER=GROUP_LEADER,NOCHECK=NOCHECK
;
; Common block to avoid disappearance of QLDS in case this is
; a GROUP LEADER window...
;
  common QLSAVE,saveQLDS,SAVED
  
;
; Quick Look common block to avoid duplication/resetting of window position
;
  common QLMGR,QL_ID,QL_DATA
  
  On_Error,2
  if !DEBUG ne 0 then on_error,0
  
  IF n_params()	lt 1 THEN message,'Use: ndspspec, QL_Data'
  
;
; Check type with QLmanager
; 
  IF NOT Keyword_SET(NOCHECK) THEN BEGIN
     QLMGR,QLDS,valid
     IF NOT valid THEN	message,'First argument is not a QL Data structure'
  EndIF
  
  dummy = load_wavecal(qlds.header.detector, $
                       anytim2utc(qlds.header.date_obs),$
                       cal_struct = qlds.wavecal)
  
;------------------------------------------------------------------------
; Standard parameter checking...
;
  IF N_params()	ne 1 THEN $
     message,'Use: EXAMPLE, QL_Data_structure'
  QLMGR,QLDS,valid
  IF NOT valid THEN message,' Parameter is not a CDS QL data structure'
  
;
; Generate wavelength calibration arrays.
;
  wavla	= pix2wave('NIS1',findgen(1024))
  wavlb	= pix2wave('NIS2',findgen(1024))
  
;
; Load "Red temperature" color table with gamma
;
  load_red,0.5

;--------------------
; Extract header info for labelling plot


;-----------------------------------------------------------------------
; If there is already a copy of this window, for this specific QLDS,
; we want to make sure it appears on the same spot as that one.
; NOTE: For each new window application having it's own top level
; base, the structure {QLINFO} (defined in qlmgr.pro) must be
; extended with a tag to hold the top level base number.
;
; Default position set: (x,y)
  
  offset=[0,500]
  xsize=1150
  dimensions = ql_dimensions(qlds,/nocheck)
  det_height = dimensions.SSOLAR_Y*qlds.detdesc(0).biny ; A little hack.
  ysize=det_height + 110
  scrnsize=1200
  BASE = QL_DATA(QLDS.QL_NO).NDSPSPECBASE
  
  IF Widget_INFO(BASE,/valid) THEN BEGIN
     IF Widget_INFO(BASE,/realized) THEN BEGIN
        Widget_CONTROL,BASE,TLB_GET_OFFSET=offset
     EndIF
     Widget_CONTROL,BASE,/DESTROY
  END
  
  title	= 'Full (deselected) NIS spectrum'
;
; Creation of new top level base (alter only title or /row/column etc.)
;
  IF N_elements(GROUP_LEADER) eq 0 THEN	$
     BASE=Widget_BASE(TITLE=title,/COLUMN,$ ; This one becomes
                      XOFFSET=offset(0),YOFFSET=offset(1)) $ ; GROUP LEADER
     $
  ELSE	  BASE=Widget_BASE(TITLE=title,$ ; This one HAS
                           XOFFSET=offset(0),YOFFSET=offset(1), $ ; a group leader
                           UVALUE=GROUP_LEADER,$
                           GROUP_LEADER=GROUP_LEADER,/COLUMN)
;---------------------------------------------------------------------
  
  
;---------------------------------------------------------------------
; Create the rest of the Widgets on screen:
;
  
  firstchild = Widget_BASE(BASE,/frame,/column)	; First child BASE
  innerchild = Widget_BASE(firstchild,frame=0,/row)
  secondchild= Widget_BASE(BASE,/frame,/column)	; Second child BASE
  
;
; Find the available window/linelists
; PREDEFINED
  
  linelists = findfile(concat_dir('$CDS_PLAN_TECH','llist_*'),count=count)
  IF count gt 0	THEN BEGIN
     headers =	strarr(count)
     hdr = ''
     FOR i=0,N_elements(linelists)-1 DO BEGIN
        openr,lun,linelists(i),/get_lun
        readf,lun,hdr
        headers(i)=hdr
        close,lun
        free_lun,lun
     END
     ix = where(strpos(headers,'(NIS)') ne -1,count)
     IF count gt 0 THEN BEGIN
        linelists = linelists(ix)
        headers = headers(ix)
     END ELSE BEGIN
        headers = ['None available']
        linelists = ['']
     EndELSE
  END ELSE BEGIN
     headers =	['None available']
     linelists	= ['']
  EndELSE
  
  markpredef = '"' + headers + '" MARKWIND' + linelists
  
;
; USER DEFINED
;
  linelists = findfile(concat_dir('$CDS_INFO','llist_*'),count=count)
  IF count gt 0	THEN BEGIN
     headers =	strarr(count)
     hdr = ''
     FOR i=0,N_elements(linelists)-1 DO BEGIN
        openr,lun,linelists(i),/get_lun
        readf,lun,hdr
        headers(i)=hdr
        close,lun
        free_lun,lun
     END
     ix = where(strpos(headers,'(NIS)') ne -1,count)
     IF count gt 0 THEN BEGIN
        linelists = linelists(ix)
        headers = headers(ix)
     END ELSE BEGIN
        headers = ['None available']
        linelists = ['']
     EndELSE
  END ELSE BEGIN
     headers =	['None available']
     linelists	= ['']
  EndELSE
  
  markuser = '"' + headers + '" MARKWIND' + linelists
  
  cmouse_s = {CMOUSE_S, btext:'', mtext:'', uvalue:'', avail:0}

  menu=[{CMOUSE_S, "Measuring", "Measuring intensities/positions",$
         "MEASURE",1+2+4+8},$
        {CMOUSE_S, "Write to log", "Write last measurement to CDSLOG",$
         "LOG",1+2+4},$
        {CMOUSE_S, "Line ID", "Identification of lines (PICK_LINE)",$
         "PICKLINE",1+2+4},$
        {CMOUSE_S, "Zooming", "Zooming in on part of the display",$
         "ZOOM",1+2+4+8},$
        {CMOUSE_S, "Profiling", "Drawing profiles from the display",$
         "PROFILE",1+2+4+8}]

  cmouse,base,widget_base(innerchild),menu
  
  ;; Create upper row of buttons
  
  column = widget_base(innerchild,/column,/frame)
  menu1 = widget_base(column,/row)
  menu2 = widget_base(column,/row)
  
  dummy = widget_button(menu1,value='Help',uvalue='HELP')
  dummy = widget_button(menu1,value='Exit',uvalue='DONE') 
  dummy = widget_button(menu1,value='Hardcopy',uvalue='HARD')
  dummy = widget_button(menu1,value='Image_Tool',uvalue='IMTOOL')
  
  ;; Create lower row of buttons
  XPdMenu,$
     [ '"Misc. " {',$
       '"Show dummy spectrum" VDSD',$
       '"Show linelists" {',$
       '"No list" MARKWIND',$
       '"Predefined lists" {',$
       markpredef,$
       '}',$
       '"User defined list"{',$
       markuser,$
       '}',$
       '}',$
       '}'],$
     menu2
  

  dummy = {PSELECT_S, btext:'', mtext:'', uvalue:'', flags:0}
  menu=[{PSELECT_S, ' Kelly', 'Kelly all lines','KLIST',0},$
        {PSELECT_S, ' Kelly/SERTS', $
         'Kelly filtered with SERTS line list','KHLIST',0},$
        {PSELECT_S, ' CDS list', $    
         'CDS Blue book list','CDS',0},$
        {PSELECT_S, ' Chianti', 'Chianti line list', 'CHIANTI',0}]
  
  dummy = cw_pselect(menu2,'Line list:',menu)
  
  dummy = cw_loadct(innerchild,/frame)
  dims = ql_dimensions(qlds,/nocheck)
  
  nexp = dims.ssolar_x > dims.sdel_time
  
  timeseries = dims.ssolar_x LT dims.sdel_time
  
  IF timeseries THEN etext = 'TIX' ELSE etext = 'XIX'
  IF nexp gt 1 THEN $
     wslider = Widget_SLIDER(innerchild,minimum=0,maximum=nexp-1,$
                             title=etext,/frame,$
                             xsize=max([nexp*2,70]),uvalue='EXPOSURE')
  

  dety_lo = gt_dety(qlds,0,1,/noparcheck)
  dety_hi = dety_lo + (dims.ssolar_y-1)
  
  sol_yrange = [gt_solary(qlds,0,dety_lo,0,/noparcheck),$
                gt_solary(qlds,0,dety_hi,0,/noparcheck)]

  rstinfo = Widget_TEXT(firstchild,value='Information line')
  
  specdraw1 = Widget_DRAW(secondchild,xsize=xsize,ysize=ysize,$
                          /button_events,/motion_events,$
                          uvalue='DRAW1')
  specdraw2 = Widget_DRAW(secondchild,xsize=xsize,ysize=ysize,$
                          /button_events,/motion_events,$
                          uvalue='DRAW2')
;
; Register the new base number in QLMGR data
;
  QL_DATA(QLDS.QL_NO).NDSPSPECBASE = BASE
  
  
; Realization of the window
;
  Widget_CONTROL, BASE,	/REALIZE
;--------------------------------------------------------------------
  
  
;--------------------------------------------------------------------
; Setting status information for this widget (could have used just
; FIRSTCHILD & SECONDCHILD, but to assure compatibilty with event
; routine, the following approach is recommended (identical to 
; the one in the event handling routine).
;
  status_id = Widget_INFO(base,/child)
  
  Widget_CONTROL,specdraw1,Get_VALUE=specwindow
  SetWindow,specwindow
  Plotreg1 = pstore(1,0,0,/clean) ; Size doesn't count here
  Widget_CONTROL,specdraw2,Get_VALUE=specwindow
  SetWindow,specwindow
  Plotreg2 = pstore(1,0,0,/clean) ; Size doesn't count here
  
  STATUS = { exposure:	1,	  $ ; Which exposure?
             missing: qlds.detdesc(0).missing,$
             timeseries:timeseries,$
             top : base,	  $ ; The top ID - to make dummy events
             Plotreg1:Plotreg1,	  $ ; Plot region 
             Plotreg2:Plotreg2,	  $ ; 
             MarkWind: '',	  $ ; Window marking linelist
             sol_yrange:sol_yrange,$
             origin:QLDS.DetDesc(0).origin, $ ;
             axes:QLDS.DetDesc(0).axes,$
             spacing:QLDS.DetDesc(0).spacing, $	;
             infoline:rstinfo,	  $ ; Information line	
             nis1:lonarr(1024,det_height),$
             nis2:lonarr(1024,det_height),$
             wavla:wavla,	  $
             wavlb:wavlb,	  $
             solarx:qlds.header.ins_x0,            $
             solary:qlds.header.ins_y0,            $
             expt:qlds.header.exptime,              $
             expst:'',		  $
             source:'',           $
             slitnum:qlds.header.slit_num,           $
             filen:qlds.header.filename,            $
             study:qlds.header.prog_num,             $
             linelist:0,           $ ; which line list to use
             obstime:'   Date: '+strmid(QLDS.header.date_obs,0,10)+'  '+ $
             strmid(QLDS.header.date_obs,11,8)+' UT'          $
           }
  

  nmake_spec,QLDS,status
  
  Widget_CONTROL,status_id,Set_UVALUE=STATUS ;,/no_copy
  
;
; Set user value of top base to QLDS 
;
  ql_no	= QLDS.QL_NO
  qlds_report,BASE,QLDS,store =	(N_elements(GROUP_LEADER) eq 0)
  
  
;-----------------------------------------------------------------
; Standard procedure to ensure correct handling of
; GROUP LEADER windows / NON GROUP LEADER windows.
;
; Avoid disappearance of the QLDS parameter.
;
  
  Xmanager,'NDSPSPEC', BASE, CLEANUP='qlmgrclean'
  
  IF N_elements(GROUP_LEADER) eq 0 and NOT Keyword_SET(nokeep)	THEN BEGIN
     WHILE Widget_INFO(base,/valid_id)	ne 0 DO	Xmanager
     qlds_checkin,ql_no
     assign_nocopy,QLDS,saveQLDS
  END
  
END









