;+
; Project     : SOHO - CDS     
;                   
; Name        : PICK_LINE()
;               
; Purpose     : Pick lines from Kelly's line list
;               
; Explanation : Kelly's line list (or part of it) is searched, and a 
;		display pops up with the lines that were within a 
;		range around the given wavelength (default +/- 0.5 Angstrom).
;
;		This is really a whole suite of procedures.
;               
; Use         : Result = PICK_LINE(WAVELENGTH)
;    
; Inputs      : WAVELENGTH
;               
; Opt. Inputs : None.
;               
; Outputs     : Returns a structure with the information in the line list
;               
; Opt. Outputs: None.
;               
; Keywords    : DELTA : Specifies the size of the area to be searched, 
;			around the given wavelength.
;
; 		PICK : 	Set to force a selection before the routine
;			returns.
;
; Calls       : CDSNOTIFY, DATATYPE(), TRIM(), CONCAT_DIR()
;
; Common      : Pick_line
;               
; Restrictions: Needs $CDS_ATOMIC/kelly_lines.dat and kelly_lines.ix
;               
; Side effects: Xmanager...
;               
; Category    : CDS QL
;               
; Prev. Hist. : None.
;
; Written     : SVHH, March 1994
;               
; Modified    : SVHH, 27 April 1994
;			Added calls to HOST_TO_IEEE and vice versa,
;			as well as the VMS /BLOCK keyword to OPENR
;			for platform independence.
;			Lower-cased the second character of two-character
;			element names.
;		SVHH, 16 May 1994
;			Added reference to $CDS_ATOMIC and CONCAT_DIR
;
;		SVHH, 11 November 1994
;			Version 2.0 -- Using Widget_DRAW's for the
;			display for compatibility with IDL v 3.6
;			and for speedup.
;
;               CDP, 24-Mar-95  Update data file names to be specific to Kelly
;
; Version     : Version 3, 24-Mar-95
;-            


; Comments:
; These routines interpret the ASCII format of Kelly's line list that
; we have here. The ASCII file was supplied without any key to interpreting
; the different columns, all rules were inferred by comparison of entries
; in the file with those in the printed version.
;
; I don't have much experience with atomic spectroscopic notation, so I'd like 
; your comments on the following interpretation of the data:
;
; The J values (J - J column) are always coded as one digit - 0, 1, 2 etc. 
; They are interpreted as fractional values (1/2, 3/2, 5/2 etc) if the ion
; has an odd number of electrons. The J subscripts of the parentage terms are
; interpreted the other way around (which also seems logical). 
;
; Now there is also some spectroscopic terms that only consist of numbers,
; i.e. "1[2]*". These exceptions are recognized, and apparently the numbers 
; should be interpreted as fractional if there is an even number of electrons. 
; However, browsing through the printed version several times, I also found 
; an anomaly to this rule - (un)fortunately this is not in the file, so I
; cannot check to see if there's any information in the file that would
; give a pointer to this - it's the lines of AL V around 1600 angstroms.
; Is this a typo in the printed version?
;
; There are also other (in my opinion weird) notations, like "(1/2,3/2)"
; and "F[3/2]" in the printed version that I can't find, so I don't know
; how these would be represented in the ASCII format...
;
; Also, some other peculiarities:
;    Some lines have an "A" in column 82 (where normally a "G" appears
; for the ground state (see C IV, lambda=206.600). ??
;    Some lines have a "G" in column 44 (which is normally "A" to signify
; additive uncertainty in the energy levels). ??
; 
; I'm not sure these are very pressing problems, but they're not
; resolved, anyway.
; 
; Stein Vidar



;
; Put a "token" string to a base. The token language rules are:
; Text between "^" and "-" is placed as superscript
; Text between "_" and "-" is placed as subscript
; If super- and subscript appear (in that order, they are placed
; on top of each other.
;

PRO put_tokens,base,text,xoff,dxoff
  IF N_elements(xoff) eq 0 THEN xoff = 0.0
  txt = text
  WHILE txt ne '' DO BEGIN
      xyot = "!3"
      run1 = strpos(txt,'^') & run2 = strpos(txt,'_')
      IF run1 ne 0 and run2 ne 0 THEN BEGIN
	  IF run1 ge 0 THEN BEGIN
	      IF run2 ge 0 and run2 lt run1 THEN run = run2 $
	      ELSE				 run = run1
	  END ELSE BEGIN
	      IF run2 ge 0 THEN run = run2 $
	      ELSE run = strlen(txt)
	  EndELSE
	  chop_char,nextrun,txt,run
	  xyot = xyot+nextrun
      EndIF
      old_xoff = xoff
      old_dxoff = dxoff
      save=0
      IF strpos(txt,'^') eq 0 THEN BEGIN
	  save = (strpos(txt,'_') ge 0)
	  IF save THEN xyot = xyot + '!S'
	  chop_char,nextc,txt
	  run = strpos(txt,'-')
	  chop_char,nextc,txt,run
	  xyot = xyot + '!U' + nextc
	  IF save THEN xyot = xyot + '!R' $
	  ELSE	       xyot = xyot + '!N'
	  chop_char,nextc,txt			  ; Delete minus sign
      EndIF
      IF strpos(txt,'_') eq 0 THEN BEGIN
	  IF nextc ne '_' THEN chop_char,nextc,txt
	  run = strpos(txt,'-')
	  chop_char,nextc,txt,run
	  xyot = xyot + "!D"+nextc+"!N"
	  chop_char,nextc,txt
	  IF save THEN save=-1			  ; Width adds up the length
      EndIF					  ; of ALL characters!
      xyouts,dxoff,10,xyot,charsize=1.6,/dev,width=width
      dxoff = dxoff + width*!D.X_SIZE $
		- (save eq -1)*!D.X_CH_SIZE	  ; So we subtract...
      xoff = max([old_xoff,xoff])
  EndWHILE
  
END



;
; Utility - chops one or more character off a string.
;
PRO chop_char,char,string,nchars,trim=trim
  IF N_elements(nchars) eq 0 THEN nchars=1
  char = strmid(string,0,nchars) & string = strmid(string,nchars,1000)
  IF Keyword_SET(trim) THEN char = strtrim(char,2)
END


;
; Returns the roman literal for a number in the range [1,..,39]
;
FUNCTION roman_literal,no
  IF no eq 0 THEN return,'?'
  t = [ 'I', 'II', 'III', 'IV', 'V', 'VI', 'VII', 'VIII',$
	'IX', 'X']
  WHILE no gt 10 DO BEGIN
      t = 'X'+t
      no = no - 10
  EndWHILE
  return,t(no-1)
END


;
; Utility 
;
FUNCTION isdigit,c
  return, c ge '0' and c le '9'
END


;
; Analyzes/produces tokens for one term.
;
PRO ana_term,term,tokens=tokens,odd=odd
  chop_char,nextc,term
  IF nextc ne ' ' THEN tokens = tokens+strlowcase(nextc) ; a,b,c ... z,y,x..
  chop_char,firstc,term
  chop_char,nextc,term
  IF isdigit(nextc) THEN BEGIN
      IF Keyword_SET(odd) THEN BEGIN
	  tokens = tokens + "^"+trim(2*fix(firstc)-1)+"-_2-"
	  tokens = tokens + "[^"+trim(2*fix(nextc)-1)+"-_2-]"
      END ELSE BEGIN
	  tokens = tokens + firstc + "[" + nextc + "]"
      END
  END ELSE BEGIN
      IF nextc eq ' ' THEN tokens = tokens + firstc $
      ELSE tokens = tokens + "^"+firstc+"-"+nextc
  EndELSE
  
  chop_char,nextc,term
  IF nextc eq '*' THEN tokens = tokens+"^o-"
  chop_char,nextc,term
  IF isdigit(nextc) THEN BEGIN
      IF Keyword_SET(odd) THEN $
	      tokens = tokens + "_"+trim(2*fix(nextc)-1)+"/2-" $
      ELSE    tokens = tokens + "_"+nextc+"-"
  END
  
END


;
; Analyzes/produces tokens for one subconfiguration
;
PRO subconfig,nextc,config,total_N_e=total_N_e,tokens=tokens
  IF N_elements(total_N_e) eq 0 THEN total_N_e = 0
  IF nextc ne ' ' THEN level = nextc $
  ELSE		       level = ''
  chop_char,nextc,config
  IF nextc eq ' ' THEN chop_char,nextc,config
  IF nextc ge '0' and nextc le '9' THEN BEGIN
      level = level + nextc
      chop_char,nextc,config
  EndIF
  shell = strlowcase(nextc)
  chop_char,N_E,config
  cnf = level+shell
  IF N_E ne ' ' and N_E ne '' THEN BEGIN
      cnf = cnf + "^"+N_E+"-"
      chop_char,nextc,config
      total_N_e = total_N_e + fix(N_e)
  END ELSE BEGIN
      nextc = N_E
      total_N_e = total_N_e + 1
  END
  tokens=tokens+cnf
END




;
; Analyzes/produces tokens for a whole configuration 
;
PRO ana_config,config,total_N_e = total_N_e,tokens=tokens
  cnf = config
  config = strtrim(cnf,2)
  IF N_elements(total_N_e) eq 0 THEN total_N_e=-1
  IF total_N_e eq -1 THEN BEGIN
      N_e = 0
      ana_config,config,total_N_e = N_e,tokens=''
      tokens = ''
      odd = byte(N_e mod 2)
      total_N_e=0
  END ELSE odd = 1
  
  chop_char,nextc,config
  WHILE config ne '' DO BEGIN
      subconf:
      IF isdigit(nextc) THEN $
	    subconfig,nextc,config,total_N_e = total_N_e,tokens=tokens
      finished_subconf:
      IF nextc eq ' ' THEN chop_char,nextc,config
      IF nextc eq '(' THEN BEGIN
	  tokens = tokens + '('
	  chop_char,nextc,config
	  IF isdigit(nextc) THEN goto,subconf
	  chop_char,term,config,4
	  term = nextc+term
	  IF odd THEN ana_term,term,tokens=tokens,odd=0 $
	  ELSE	      ana_term,term,tokens=tokens,odd=1
	  chop_char,nextc,config
	  tokens=tokens+nextc
	  chop_char,nextc,config
	  goto,finished_subconf
      END
      IF nextc eq ')' THEN BEGIN
	  tokens = tokens + nextc
	  chop_char,nextc,config
      END
  EndWHILE
  
  tokens = strtrim(tokens,2)
  config = cnf
END



;
; Displays one line (atomic line) on the base b. The button with
; the wavelength is given the specified uvalue.
;
PRO disp_line,l,b,uvalue=uvalue,show=show
  
  ionpos=10
  levelpos=ionpos + 80
  configpos=levelpos + 70
  termspos=configpos + 250
  jjpos=termspos + 100
  commentpos=jjpos + 50
  endpos = commentpos + 100
  
  IF N_elements(uvalue) eq 0 THEN uvalue=''
  
  IF NOT Keyword_SET(show) THEN BEGIN
      dummy = Widget_BUTTON(Widget_BASE(b,/row),$
			   value=trim(l.lambda,'(f7.3)')+string(197b),$
			   uvalue=uvalue)
      dummy = Widget_DRAW(b,xsize=endpos,ysize=32)
      RETURN
  END
  element_base = Widget_INFO(Widget_INFO(b,/child),/sibling)
  Widget_CONTROL,element_base,Get_VALUE=ddraw
  Wset,ddraw
  
  !P.font = -1
  
  dxoff = ionpos
  xyouts,ionpos,10,"!6"+l.element,width=width,/DEVICE,charsize=2
  xyouts,ionpos+6+width*!D.X_SIZE,10,roman_literal(l.ion_no),$
		width=width2,/dev,charsize=1.5
  plots,ionpos+6+[width,width+width2]*!D.X_SIZE,[10,10],/dev
  plots,ionpos+6+[width,width+width2]*!D.X_SIZE,[10,10]+!D.Y_CH_SIZE-2,/dev
  
  xyouts,levelpos,10,"!3!S!U"+trim(l.level2)+"!R!D"+trim(l.level1)+"!N",/dev,$
	charsize=2.
  
  dxoff = configpos
  
  t='' & N_E = -1
  ana_config,l.config2,tokens=t,total_N_e=N_E
  t = t+'-'
  put_tokens,configb,t,xoff,dxoff
  ana_config,l.config1,tokens=t
  put_tokens,configb,t,xoff,dxoff
  
  t=''
  
  
  dxoff = termspos
  ana_term,l.terms2,tokens=t,odd = (N_E mod 2) ne 1
  IF l.ground eq 'G' THEN t='g'+t
  put_tokens,termb,t+'-',xoff,dxoff
  t=''
  ana_term,l.terms1,tokens=t,odd = (N_E mod 2) ne 1
  put_tokens,termb,t,xoff,dxoff
  
  IF N_E mod 2 THEN t = "^"+trim(2*l.j2-1)+"-_2--^"+trim(2*l.j1-1)+"-_2-" $
  ELSE		    t = trim(l.j2)+"-" + trim(l.j1)
  
  dxoff = jjpos
  put_tokens,jbase,'  '+t,xoff,dxoff
  
  comment=''
  IF l.a THEN comment = comment + 'a1' $
  ELSE	      comment = comment + '  '
  IF l.ground eq 'A' THEN comment = comment + 'a2' $
  ELSE			  comment = comment + '  '
  comment = comment+l.comments
  
  xyouts,commentpos,10,"!6"+comment,/dev,charsize=1.5
END



;
; Makes widget labels to display the heading of the atomic line data.
;
PRO atom_line_heading,b
  headb = Widget_BASE(b,/row)
  heading = Widget_LABEL(headb,value='Lambda     ')
  heading = Widget_LABEL(headb,value='Ion no.    ')
  heading = Widget_LABEL(headb,value='Energy levels          ')
  heading = Widget_LABEL(headb,value='Configurations      '+$
		'            ')
  heading = Widget_LABEL(headb,value='Terms             ')
  heading = Widget_LABEL(headb,value='J-J         ')
  heading = Widget_LABEL(headb,value='Notes     ')
END


;
; Check if line is in SERTS line list
;
FUNCTION serts_filter, lambda
   
   COMMON serts_cat_store, storage
   
   IF N_ELEMENTS(storage) EQ 0 THEN BEGIN
      rd_serts,storage
   END
   ii = WHERE(storage.kelly_wav EQ lambda)
   
   IF ii(0) LT 0 THEN fact = 0
   IF ii(0) GE 0 THEN fact = 1
   
   RETURN,fact
END



;
; What if line not present in Kelly, but exists in SERTS
;
FUNCTION notin_kelly,lambda, delta=delta
      COMMON serts_cat_store, storage
   
   IF N_ELEMENTS(storage) EQ 0 THEN BEGIN
      rd_serts,storage
   END
   IF N_elements(delta) eq 0 THEN delta = 0.5
   
   ix=WHERE(abs(storage.wavelength-lambda) LE delta,count)

   IF count GT 0 THEN BEGIN

      jx = WHERE(abs(storage.kelly_wav(ix)) EQ 9.999,count)
      IF count GT 0 THEN BEGIN
         newlines = jx+ix(0)
      ENDIF ELSE BEGIN 
         newlines = -1
      ENDELSE
      
   ENDIF ELSE newlines = -1

   RETURN,newlines
END





;
; Parses one line of data from Kelly's line list into a structure.
; 
FUNCTION parse_line,line,Print=Print
  x = line
  chop_char,lambda,x,11
  lambda = float(lambda)
  chop_char,element,x,2,/trim
  element = strmid(element,0,1) + strlowcase(strmid(element,1,1))
  chop_char,ion_no,x,2
  IF ion_no eq '  ' THEN ion_no = 0 $
  ELSE			 ion_no = fix(ion_no)
  chop_char,level1,x,14
  IF level1 eq '              ' THEN	level1=0.0 $
  ELSE					level1 = float(level1)
  chop_char,level2,x,14
  IF level2 eq '              ' THEN	level2=0.0 $
  ELSE					level2 = float(level2)
  chop_char,a,x,3
  IF a eq ' A ' THEN a=1 ELSE a=0
  chop_char,config1,x,29
  chop_char,terms1,x,5
  chop_char,j1,x & IF j1 ne ' ' THEN j1 = fix(j1)
  chop_char,ground,x,3
  ground = strmid(ground,1,1)
  chop_char,config2,x,29
  chop_char,terms2,x,5
  chop_char,j2,x & IF j2 ne ' ' THEN j2 = fix(j2)
  comments = x
  l_struct = { $
	lambda:lambda,element:element,ion_no:ion_no,$
	level1:level1,level2:level2,a:a,terms1:terms1,config1:config1,$
	J1:j1,ground:ground,terms2:terms2,config2:config2,j2:j2,$
	comments:comments }
  IF NOT Keyword_SET(Print) THEN return,l_struct
  Print,line
  Print,trim(lambda)+';'+element+';'+trim(ion_no)+';'+trim(level1)+$
		';'+trim(level2)+';'+trim(fix(a))+';'+config1+';'+terms1+$
		';'+trim(j1)+';'+trim(ground)+';'+config2+';'+terms2+';'+$
		trim(j2)+';'+comments
  return,l_struct
END




;
; Looks up the lines in Kelly's line list, returns the ASCII lines for
; each matching line.
;
FUNCTION lookup_line,lambda,delta=delta
  
  common lookup_l,l
  IF N_elements(l) eq 0 THEN BEGIN	; Is the index present in memory?
      file = concat_dir('$CDS_ATOMIC','kelly_lines.ix')
      openr,lun,file,/get_lun,/block	; No - read it 
      fst = fstat(lun)
      Size = fst.Size/4
      l = fltarr(Size)
      readu,lun,l
      ieee_to_host,l
      close,lun & free_lun,lun
  EndIF
  
  IF N_elements(delta) eq 0 THEN delta = 0.5
  
  ix = where(abs(l-lambda) le delta,count)
  IF count eq 0 THEN BEGIN
      return,''
  END ELSE BEGIN
      file = concat_dir('$CDS_ATOMIC','kelly_lines.dat')
      openr,lun,file,/get_lun
      a = assoc(lun,bytarr(125))
      t = [strmid(string(a(ix(0))),0,124)]
      i = 1
      WHILE i lt count DO BEGIN
	  t = [t,strmid(string(a(ix(i))),0,124)]
	  i = i+1
      EndWHILE
      close,lun & free_lun,lun
      return,t
  EndELSE
END



PRO pick_line_event,ev
  common pick_line,line_no,lines
  Widget_CONTROL,ev.id,Get_UVALUE=uvalue
  
  IF datatype(uvalue) ne 'INT' THEN BEGIN
      CASE uvalue OF
	  'QUIT': BEGIN
	      line_no = -1
	      Widget_CONTROL,ev.top,/destroy
	  END
	  'PREFER': BEGIN
	      IF ev.select THEN BEGIN
		  Print,'Preferred selected'
	      END ELSE BEGIN
		  Print,'No preferences'
               ENDELSE
 	  EndCASE
          'WRITELOG':cdslog,lines
          ELSE : BEGIN
	      help,ev,/str
	  EndCASE
      EndCASE
      return
  EndIF
  line_no = uvalue
  Widget_CONTROL,ev.top,/destroy
END



;
; See the header of this file
;
FUNCTION pick_line,lambda,delta=delta,pick=pick,kelly=kelly,serts=serts
  common pick_line,line_no,lines
  COMMON serts_cat_store, storage
  lines = lookup_line(lambda,delta=delta)
  
; If SERTS keyword set, filter Kelly list by matching wavelengths
;
  
  
  IF KEYWORD_SET(serts) THEN BEGIN     

  
  IF lambda GT 171.0 AND lambda LT 448.3 THEN BEGIN

   wave = FLOAT(STRMID(lines,2,8))
   nwav = N_ELEMENTS(wave)
   lines2 = STRARR(nwav)
   j = 0
   FOR i = 0, nwav-1 DO BEGIN
     fact=serts_filter(wave(i))

     IF fact EQ 1 THEN BEGIN
        lines2(j) = lines(i) 
        j = j + 1
     ENDIF 
   END

  IF j GT 0 THEN lines = lines2(0:j-1) ELSE lines = ''

  newlines = notin_kelly(lambda,delta = delta)

  IF newlines(0) GE 0 THEN BEGIN
     bell
     MESSAGE = STRARR(N_ELEMENTS(newlines)+6)
     MESSAGE(0) = 'The following lines were not listed in the KELLY line ' + $
        'list'
     MESSAGE(1) = 'but has been observed with the SERTS instrument.'
     MESSAGE(2) = ' '
     MESSAGE(3) = ' Element   Lambda (obs)  Intensity ' + $
        '    Log T  '
     MESSAGE(4) = '-------------------------------------------------------'    
     MESSAGE(N_ELEMENTS(newlines)+5) = '.'
     FOR i = 0, N_ELEMENTS(newlines)-1 DO BEGIN
        IF storage.element(newlines(i)) EQ '  ' THEN $
           storage.element(newlines(i)) =  ' No ID'
        MESSAGE(i+5) = ' '+STRING(storage.element(newlines(i))+$
                              storage.ion(newlines(i)))+$
           STRING(storage.wavelength(newlines(i)))+'   '+$
           STRING(FIX(storage.intensity(newlines(i))))+$
           STRING(storage.tmax(newlines(i)))
     END
     cdsnotify,message     
     
  END
;
; SERTS filter section end
;  

  ENDIF ELSE BEGIN
     bell
     cdsnotify,'Wavelength outside SERTS ragen!! (171 - 448.3 A)'
     cdsnotify,'Displayed line list was not filtered'
  END



  END

  IF lines(0) eq '' THEN BEGIN
      cdsnotify,'No lines found'
      return,0
  EndIF
  
  
    IF KEYWORD_SET(serts) THEN $     
  b = Widget_BASE(/column,$
       title="Filtered Kelly list (only lines observed with SERTS),  " + $
       "lambda="+trim(lambda)+'+/-'+trim(delta)) $
    ELSE  b = Widget_BASE(/column,$
                  title="From Kelly's line list, " + $
                          "lambda="+trim(lambda)+'+/-'+trim(delta))
   
  bb = Widget_BASE(b,/row)

  dummy = Widget_BUTTON(bb,value='Quit',uvalue='QUIT')
  dummy = Widget_BUTTON(bb,value='Write lines to log',uvalue='WRITELOG')
  dummy = Widget_BUTTON(Widget_BASE(bb,/exclusive),$
			  value='Only show preferred lines',$
			  uvalue='PREFER')
  atom_line_heading,b
  list = Widget_BASE(b,y_scroll_size=400,/scroll,/column)
  
  FOR i=0,N_elements(lines)-1 DO BEGIN
      bb = Widget_BASE(list,/row,/frame)
      disp_line,parse_line(lines(i)),bb,uvalue=i
   EndFOR
  
  Widget_CONTROL,b,/realize
  
  bb = Widget_INFO(list,/child)
  FOR i=0,N_elements(lines)-1 DO BEGIN
      disp_line,parse_line(lines(i)),bb,uvalue=i,/show
      bb = Widget_INFO(bb,/sibling)
  EndFOR
  
  line_no = -2
  Xmanager,"pick_line",b
  IF Keyword_SET(pick) THEN BEGIN
      WHILE line_no eq -2 DO Xmanager
      IF line_no ne -1 THEN return,parse_line(lines(line_no))
  END
  IF line_no ge 0 THEN return,parse_line(lines(line_no))
END


;
; To make the index from the line list
; Optionally returns the index.
;
PRO make_index,file,lambda=lambda
  IF N_elements(file) eq 0 THEN file = concat_dir('$CDS_ATOMIC','lines.dat')
  
  openr,unit,file,/get_lun
  on_ioerror,end_of_file
  
  x = ''
  readf,unit,x
  lambda = [float(strmid(x,0,10))]
  WHILE 1 DO BEGIN
      readf,unit,x
      lambda = [lambda,float(strmid(x,0,10))]
  EndWHILE
  
  end_of_file:
  free_lun,unit
  file = (str_sep(file,'.'))(0)
  openw,unit,file+'.ix',/get_lun
  host_to_ieee,lambda
  writeu,unit,lambda
  close,unit
  free_lun,unit
END






