;+
; Project     : SOHO - CDS     
;                   
; Name        : PQLPROFILE
;               
; Purpose     : General purpose profile-drawing kit.
;               
; Explanation : Passing an event generated in a draw widget inside
;		a stored plot region will cause a new widget to be 
;		created, with a profile of the data in the original 
;		region.
;               
; Use         : PQLPROFILE,DATA
;
; Inputs      : DATA:	The data that is displayed in the plot region.
;
; Opt. Inputs : XSCALE (only for 1D data)
;               
; Outputs     : None.
;               
; Opt. Outputs: None.
;               
; Keywords    : LENGTH: The length of the profiled region in pixels
;			Default is 30 pixels
;
;		VERTICAL: set to plot vertical profiles. No effect for 
;			one-dimensional plots.
;
;		AVERAGE = Number of pixels to average over (only for 2D data).
;			No effect for one-dimensional plots.
;
;		TITLE = title of the plot
;
;		DATATITLE = the title to go on the axis showing the
;			    data values.
;
;		X/YTITLE = X/Y titles.
;		SUBTITLE = subtitle (only for 1D data)
;
;		X/YFORMAT = The format of the displayed X/Y position in the
;			subtitle.
;
;		X/Y_SIZE = size of plot window. Default X:500, Y:250
;
;		OUTSIDE: Set to make the widget appear on the outside
;			of the plot.
;
;		CURSOR: Set to make the widget appear aligned with the
;			plot cursor.
;
;		EVENT:	A WIDGET_DRAW event structure generated in
;			a widget_draw window. See restrictions.
;
;		P_reg: The plot region ID (as returned by pfind())
;			identifying which plot that is affected.
;
;		CHANGE : Set to indicate that a change of the DATA 
;			in the plot region has been made.
;			If there is a profile widget for this plot
;			region then this will update it's display
;			and the markings. If not, nothing happens.
;			If the data has not been changed, just 
;			redisplayed, DATA need not be supplied.
;
;		GROUP = Widget ID for the top level base that is to
;			be group leader (zoom window dies upon death of
;			the group leader).
;
;		KILL: 	Set to kill any profile widget from the specified
;			plot region
;		
;		ACTIVE: Set to a named variable to return 1 if there is
;			a profile window for the specified plot region
;	
;		RESET:  Set to reset the common block
;
; Calls       : CDSNOTIFY, CDSPICKFILE(), CMOUSE, CMOUSE_ACTION(), 
;		DATATYPE(), PCONVERT(), PFIND(), PRESTORE, PS, PSCLOSE, 
;		PSPLOT, PSTORE(), SETWINDOW, TLB_PLACE(), TRIM(), 
;		WIDGET_BASE(), WIDGET_TEXT(), WIDG_HELP, CDS_GAUSS()
;		(11-Dec-94)
;
; Common      : PQLPROFILE:
;			Keeps track of plot region ID's for each PQLPROFILE
;			window that has been generated.
;
; Restrictions: The !P/!D/!X/!Y system variables describing the image
;		or plot axes must have been stored with PSTORE.
;		The EVENT coordinates passed to the routine must be
;		inside the CLIP region of the displayed window - if
;		not - nothing happens.
;		NOTE If XSCALE is not supplied, the routine assumes that 
;		the data array is equispaced and fills the whole plot 
;		region, i.e., plotting x versus y, the xrange must be 
;		set to [min(x),max(x)], and xstyle must be set to 1
;
;
; Side effects: 
;               
; Category    : Display, QuickLook
;               
; Prev. Hist. : Written on the basis of the QuickLook Zoom routine.
;
; Written     : Stein Vidar Hagfors Haugan, Des. -93
;               
; Modified    : SVHH, March 1994
;			Length/vertical/average are not reset upon the next
;			call.
;		SVHH, Version 2, May 1994
;			Using PSTORE/PRESTORE/PFIND instead of SERTS window
;			routines. Profile of 1-D plots are now possible.
;		SVHH, Version 3, 2 June 1994
;			Added incremental positioning a la PQLZOOM
;               PB, 10 July 1994
;                      Added HELP functionality  and Line fit button
;               PB,   Sept 21 - 94    Added Help using WIDG_HELP
;		SVHH, 11-Dec-94, Added CMOUSE/CMOUSE_ACTION mouse
;			control functionality, and zoom in/out buttons.
;               Version 4, SVHH, 21-December-1995
;                       Using histogram mode for plotting.
;               Version 4.1, SVHH, 12-January-1996
;                       Now converting !version.release to float before
;                       using it.
;               Version 4.2, P.B, 6-February-1996
;                       Allows Gassian fit to profiles
;               Version  4.3, P.B, 5-March-1996
;                       Gaussian fit returns FWHM in Pixel and total
;                       total counts in line. Changed yrange to add space
;               Verison 5, SVHH, 17 November 1998
;                       Fixed {WIDGET_DRAW} for IDL v 5
;		Version 6, 29 January 1999, William Thompson, GSFC
;			Fixed {WIDGET_DRAW} for IDL v 4
;		Version 7, 5 January 2001, William Thompson, GSFC
;			Fixed bug for versions 5.3 and 5.4
;
; Version     : Version 7, 5 January 2001
;- 

;
; PQLP_PLOT is the routine that updates the plot of the profile,
; plus markings on the caller's plot region.
;

;
; UPDATE is set if there's a change in PROFILE's parameters (length etc..)
;
; CHANGE is set if there's a change in the source plot region (i.e., source
; window is redrawn).
;

PRO pqlp_plot,ev,which,update=update,change=change,psfile=psfile
; HISTORY:
;
;       Mon Feb 5 20:51:20 1996, Paal Brekke <paalb@rigil.uio.no>
;
;		

;
  common PQLPROFILE,DZ,lastwhich,data,textw
  On_Error,0
  info = DZ(which)
  
;
; Restore the plot region of the caller
;
  
  prestore,info.P_reg,datax,datay,scrnx,scrny,jx,jy
  
  Widget_CONTROL,info.base,/show
  
  IF NOT (Keyword_SET(update) or Keyword_SET(change)) THEN BEGIN
      IF info.ndims eq 1 THEN BEGIN
	  xx = (DATA.X - pconvert(info.P_reg,ev.x,/dev,/to_data))^2 ; Least X^2
	  info.x = (where(xx eq min(xx)))(0)
	  info.y = pconvert(info.P_reg,ev.y,/dev,/to_pix,/Y)
      END ELSE BEGIN
	  info.x = pconvert(info.P_reg,ev.x,/dev,/to_pix)
	  info.y = pconvert(info.P_reg,ev.y,/dev,/to_pix,/Y)
      END
  END
  
;
; Make ready for grab (top/bottom coord's etc.)
; Get the indexes for the "source" area (what region of the original data
; should be plotted)
;
  
  info.x = (info.x < (datax-1)) >0
  info.y = (info.y < (datay-1)) >0
  
  IF info.vertical THEN BEGIN
      height = (info.length-1)
      xtop = info.x + (info.average-1)/2
      ytop = info.y + height/2
      xbot = info.x - (info.average-1)/2 - (info.average-1) mod 2
      ybot = info.y - height/2 - height mod 2
  END ELSE BEGIN
      width = info.length-1
      xtop = info.x + width/2
      ytop = info.y + (info.average-1)/2
      xbot = info.x - width/2 - width mod 2
      ybot = info.y - (info.average-1)/2 - (info.average-1) mod 2
  EndELSE
  
;
; Check limits
;
  
  xtop = (xtop < (datax-1)) >0
  ytop = (ytop < (datay-1)) >0
  xbot = (xbot > 0) < xtop
  ybot = (ybot > 0) < ytop
  
;
; Grab
;  
  
  IF info.ndims eq 2 THEN grab = DATA.D(xbot:xtop,ybot:ytop) $
  ELSE BEGIN
      grab = DATA.D(xbot:xtop)
      grabx= DATA.X(xbot:xtop)
  END
  
  IF info.vertical THEN info.ac = (ybot+ytop)/2 $
  ELSE			info.ac = (xbot+xtop)/2
  
  sz = Size(grab)
  
  
  Widget_CONTROL,info.len_text,Set_VALUE=trim(sz(1))+'/'+trim(info.length)
  
  IF sz(0) gt 1 THEN BEGIN
      IF info.vertical THEN grab = total(grab,1)/info.average	$
      ELSE		    grab = total(grab,2)/info.average
  END
  
;
; Flicker display in source window by inverting the "source box"
;
  
  IF total(info.polyx gt -1) gt 0 $		; Remove any previous markings
	  and NOT Keyword_SET(change) THEN pqlp_plotbar,info
  
  IF info.ndims eq 2 THEN BEGIN
      info.polyx=pconvert(info.P_reg,[xbot,xtop],/pix,/to_dev) ; New position
      info.polyy=pconvert(info.P_reg,[ybot,ytop],/pix,/to_dev,/Y)
  END ELSE BEGIN
      data_xtop = DATA.X(xtop)
      data_xbot = DATA.X(xbot)
      info.polyx=pconvert(info.P_reg,[data_xbot,data_xtop],/data,/to_dev)
      info.polyy(*)=scrny/2+jy  ; Use halfway position for 1-D
  EndELSE
  
  pqlp_plotbar,info
  
;
; Plot the data with correct axes
;
  
  IF info.vertical THEN BEGIN
      x = grab
      y = pconvert(info.P_reg,findgen(N_elements(grab))+ybot,/Y,/pix,/to_data)
      ytitle = info.ytitle
      xtitle = info.dtitle
      IF info.subtitle ne '' THEN stitle=info.subtitle $
      ELSE IF info.ndims gt 1 THEN $
	stitle = 'At ' + info.xtitle + ' ' + $
		trim(pconvert(info.P_reg,(xtop+xbot)/2.,/pix,/to_data),$
								info.xformat) $
      ELSE stitle=''
  END ELSE BEGIN
      IF info.ndims eq 1 THEN x = grabx $
      ELSE x = pconvert(info.P_reg,findgen(N_elements(grab))+xbot,$
								/pix,/to_data)
      y = grab
      xtitle = info.xtitle
      ytitle = info.dtitle
      IF info.subtitle ne '' THEN stitle=info.subtitle $
      ELSE IF info.ndims gt 1 THEN $
	 stitle = 'At ' +info.ytitle + ' '	+ $
		trim(pconvert(info.P_reg,/pix,/to_data,(ytop+ybot)/2.,/Y),$
								info.yformat)$
      ELSE stitle = ''
  END
  IF info.fit EQ 1 then BEGIN
  bk = poly_fit(x,y,1,back)          ;stores background   
  iny1 = cds_gauss(x,y,gcoeff,1)
  lpos = gcoeff(1)
  iny2 = cds_gauss(findgen(N_elements(grab)),y,gcoeff,1)
  gcoeff(2) = gcoeff(2) * SQRT(2)    ; to get Doppler width
  totint = gcoeff(0)*SQRT(!pi)*gcoeff(2)
  fwhm = gcoeff(2)*(2*(SQRT(alog(2))))
  imax1 = gcoeff(0)+(total(back)/N_ELEMENTS(back))
  imax2 = gcoeff(0)
  utext = ' '+$
        'Line pos (A): '+trim(lpos,'(f8.3)')+$
;       '  Imax (c): '+trim(FIX(imax1))+$
       '  Imax-back (C/px): '+trim(FIX(imax2))+'  FWHM (px): '+$
       trim(fwhm,'(f5.3)')+'  Itot (C) :'+trim(FIX(totint))
;    utext =  utext + '  '+$
  WIDGET_CONTROL,textw,Set_VALUE = utext

  END  
  
  prestore,info.Plotreg
  
  IF N_elements(psfile) gt 0 THEN PS,psfile
  
  P = !P
  !P.multi=[0,1,1,1]
  
  IF N_elements(grab) lt 3 or max(grab) eq min(grab) THEN erase $
  ELSE Plot,x,y,linestyle=0,xstyle=1,ystyle=1,xrange=[min(x),max(x)],$
		yrange=[min(y),max(y)+max(y)*0.1],$
		title=info.title,xtitle=xtitle,ytitle=ytitle,$
		subtitle=stitle,position=[0.15,0.2,0.9,0.9],psym=10
  IF info.fit EQ 1 THEN oplot,x,iny1,lines = 3
  
; NB!!
  
  IF N_elements(psfile) eq 0 THEN BEGIN
      IF info.vertical THEN info.plotreg = Pstore(1,1,N_elements(x)) $
      ELSE		    info.plotreg = Pstore(1,N_elements(x),1)
  END
  
  !P = P
  
  IF N_elements(psfile) gt 0 THEN psclose
  
  prestore,info.P_reg
  
  DZ(which) = info		  ; Put back some status info
  
END


PRO pqlp_plotbar,info
  prestore,info.P_reg,datax,datay,scrnx,scrny,jx,jy,plot_number=plotnum
  IF datatype(plotnum) eq 'STR' THEN return
  
  DEVICE,get_graphics_function=oldgraphics
  DEVICE,set_graphics_function=6
  
  xvec = info.polyx
  yvec = info.polyy
  
  IF xvec(0) eq xvec(1) or yvec(0) eq yvec(1) THEN $
		plots,xvec,yvec,/DEVICE $
  ELSE BEGIN
      x = [xvec(0),xvec(0),xvec(1),xvec(1),xvec(0)]
      y = [yvec(0),yvec(1),yvec(1),yvec(0),yvec(0)]
      plots,x+0.5,y+0.5,/DEVICE
  END
  DEVICE,set_graphics_function=oldgraphics
END

;
; Make sure we clean up our act upon dying... (to avoid slaying other
; windows later)
;

PRO pqlp_cleanup,ID
  common PQLPROFILE,DZ,lastwhich,lastdata,textw
  
  Widget_CONTROL,ID,Get_UVALUE=which
  
  pqlp_plotbar,DZ(which)		; Erase markings
  DZ(which)={QLPINFO}
  DZ(which).P_reg=-1
END





;
; Event handling -------------------------------------
;

PRO PQLPROFILE_event, event
  common PQLPROFILE,DZ,lastwhich,lastdata,textw
  
  Widget_CONTROL, event.top, Get_UVALUE = which
  Widget_CONTROL, event.id, Get_UVALUE = eventval	  ;find the user value
  
  i = which
  
;  DZ(i) = DZ(which)
  
;
; Do we have the right data set in the cache?
;
  
  IF lastwhich ne which THEN BEGIN
      lastwhich = which
      Widget_CONTROL,DZ(i).mbase,Get_UVALUE=lastdata
  EndIF

  CASE eventval OF
      
      'DONE': BEGIN
	  Widget_CONTROL,event.top,/destroy
	  return
      END
      
      'AVERAGE': BEGIN
	  DZ(i).average = event.value
	  Widget_CONTROL,DZ(i).avg_txtID,Set_VALUE=trim(DZ(i).average)
	  pqlp_plot,DZ(i).init_ev,which,/update
      END
      
      'LENGTH': BEGIN
	  DZ(i).length = event.value
	  Widget_CONTROL,DZ(i).len_text,Set_VALUE=trim(DZ(i).length)
	  pqlp_plot,DZ(i).init_ev,which,/update
      END
      
      'PSFILE': BEGIN
	  filename=cdspickfile(title='Pick file to plot to')
	  IF filename ne '' THEN BEGIN
	      pqlp_plot,DZ(i).init_ev,which,psfile=filename,/update
	  EndIF
	  psclose
      EndCASE
      
      'PSPRINT': BEGIN
	  pqlp_plot,DZ(i).init_ev,which,psfile='.tmp.ps',/update
	  psplot,'.tmp.ps',/delete
      EndCASE
      
      'SAVE': BEGIN
	  cdsnotify,group=event.top,'To save intensiy/wavelengths to file'
	  cdsnotify,group=event.top,'SAVE not implemented yet!!'
      EndCASE
      
      'HELP': BEGIN
	  widg_help,'pqlprofile',$
	  title = 'Help: Profile Utility',group=event.top
      EndCASE
      
      'IGAUSS': BEGIN
         DZ(i).fit = 1
         WIDGET_CONTROL,textw,Set_VALUE = 'Use RECENTER-button to re-plot'
       ENDCASE
      
      'NOFIT': BEGIN
         DZ(i).fit = 0
         
      EndCASE
      
      'AMOEBE': BEGIN
	  cdsnotify,group=event.top,'AMOEBE not implemented yet!!'
      EndCASE
      
      'OTHER': BEGIN
	  cdsnotify,group=event.top,'OTHER not implemented yet!!'
      EndCASE
      
      'HORIZONTAL': BEGIN
	  DZ(i).vertical=0
	  pqlp_plot,DZ(i).init_ev,which,/update
      EndCASE
      
      'VERTICAL': BEGIN
	  DZ(i).vertical=1
          IF DZ(i).fit EQ 1 THEN BEGIN
             DZ(i).fit = 0
             WIDGET_CONTROL,textw,$
             Set_VALUE = 'Fit turned off! To activate in Hor.mode, reselect FIT'
          ENDIF
	  pqlp_plot,DZ(i).init_ev,which,/update
      EndCASE
      
      'DRAW': CASE cmouse_action(event) OF
	  
	  '': return
	  
	  'RECENTER': BEGIN
	      Preg = pfind(event,found)
	      IF found THEN prestore,Preg,datax,datay
	      IF found and DZ(i).vertical THEN BEGIN
		  DZ(i).y = DZ(i).ac +pconvert(Preg,event.y,/dev,/to_pix,/Y) $
				  - datay/2.0 + 1
	      END ELSE IF found and NOT DZ(i).vertical THEN BEGIN
		  pc = pconvert(Preg,event.x,/dev,/to_pix)
		  DZ(i).x = DZ(i).ac + pconvert(Preg,event.x,/dev,/to_pix) $
				  - datax/2.0 + 1
	      END ELSE BEGIN
		  prestore,DZ(i).Plotreg
		  IF event.y gt !P.clip(3) THEN DZ(i).y = DZ(i).y+1
		  IF event.y lt !P.clip(1) THEN DZ(i).y = DZ(i).y-1
		  IF event.x gt !P.clip(2) THEN DZ(i).x = DZ(i).x+1
		  IF event.x lt !P.clip(0) THEN DZ(i).x = DZ(i).x-1
	      EndELSE
              PRINT,DZ(i).Plotreg,DZ(i).x,DZ(i).init_ev
              pqlp_plot,DZ(i).init_ev,which,/update
	  EndCASE
	  
	  'ZIN': IF DZ(i).length gt 3 THEN BEGIN
	      DZ(i).length = ((DZ(i).length/1.6 - 1) > 3)
	      pqlp_plot,DZ(i).init_ev,which,/update
	  END
	  
	  'ZOUT': BEGIN
	      DZ(i).length = (DZ(i).length*1.6 + 1) < DZ(i).lenmax
	      pqlp_plot,DZ(i).init_ev,which,/update
           END

          'LINE': BEGIN
	  cdsnotify,group=event.top,'LINE SELECT not implemented yet!!'
           EndCASE
          
	  
      EndCASE
      
      ELSE: MESSAGE, "Event User Value Not Found"
  EndCASE
  
END




PRO PQLPROFILE,DAT,XSCALE,length=length,vertical=vertical,average=average,$
		title=title,xtitle=xtitle,ytitle=ytitle,datatitle=datatitle,$
		subtitle=subtitle,$
		xformat=xformat,yformat=yformat,$
		x_size=x_size,y_size=y_size,$
		outside=outside,cursor=cursor,$
		event=ev,P_reg=P_reg,change=change,$
		GROUP=GROUP,reset=reset,kill=kill,active=active
  
  common PQLPROFILE,DZ,lastwhich,lastdata,textw
  
  On_Error,2
  IF !DEBUG ne 0 THEN On_Error,0
  
  IF Keyword_SET(reset) THEN BEGIN
      DZ = 0
      return
  EndIF
  
;
; Check if we have enough information
;
  
  sz = Size(DAT)
  Ndims = sz(0)
  IF (N_elements(DAT) le 1 or Ndims gt 2) and NOT Keyword_SET(change) THEN $
		message,'Data must be either 1D or 2D array'
  
  IF Ndims eq 1 THEN BEGIN
      IF N_elements(XSCALE) ne 0 and N_elements(XSCALE) ne sz(1) THEN $
	      message,'XSCALE must have same number of elements as DATA'
  EndIF
  
  IF N_elements(ev) gt 0 THEN BEGIN
      P_reg = pfind(ev,found)
      IF NOT found THEN return
  END ELSE IF N_elements(P_reg) eq 1 THEN BEGIN
     prestore,P_reg
     IF since_version('5.3') then begin
	ev = {Widget_DRAW}
	ev.x = !P.clip(0)+1
	ev.y = !P.clip(0)+1
     END ELSE begin
	command = ' ev = {Widget_DRAW, ID:0L, TOP:0L, HANDLER:0L, TYPE:0, ' + $
              'X:fix(!P.clip(0)+1), Y:fix(!P.clip(0)+1), press:0b, release:0b'
	IF since_version('5') THEN command = command + ', clicks:0L}' ELSE $
	     command = command + '}'
	test = execute(command)
     ENDELSE
  END ELSE message,'Need to have either event or Plot Region'
  
;
; We now have reasonable event coordinates, and we have the P_region
;
; Defaults
;  
  
  IF N_elements(length) eq 0 THEN length=20
  IF N_elements(vertical) eq 0 THEN vertical=0B
  IF N_elements(average) eq 0 THEN average=1
  IF N_elements(title) eq 0 THEN title=''
  IF N_elements(xtitle) eq 0 THEN xtitle=''
  IF N_elements(ytitle) eq 0 THEN ytitle=''
  IF N_elements(datatitle) eq 0 THEN datatitle=''
  IF N_elements(subtitle) eq 0 THEN subtitle=''
  IF N_elements(xformat) eq 0 THEN xformat=''
  IF N_elements(yformat) eq 0 THEN yformat=''
  
  IF N_elements(x_size) eq 0 THEN BEGIN
      IF N_elements(Y_size) eq 1 THEN x_size = 2*y_size $
      ELSE			      x_size = 500
  EndIF
  
  IF N_elements(y_size) eq 0 THEN y_size = fix(x_size/sqrt(2))
  
  IF Ndims eq 1 THEN BEGIN
      IF N_elements(XSCALE) eq 0 THEN BEGIN
	  span = pconvert(P_reg,sz(1),/pix,/to_dat)$
				-pconvert(P_reg,0,/pix,/to_dat)
	  XSCALE = findgen(sz(1))/(sz(1)-1)*span $
			+ pconvert(P_reg,0,/pix,/to_data)
      END
  END ELSE XSCALE=0
  
  IF N_elements(DAT) gt 0 THEN DATA = { X:XSCALE, D:DAT}
  
; If no Pqlprofile has been invoked before, we need to init common block DZ
; (and the structure type)
  
  IF DATATYPE(DZ) ne 'STC' THEN BEGIN
      info = {QLPINFO,$
		  base	   : 0L,	$ ; Top level base
		  init_ev  : ev,	$ ; (dummy) initial event
		  P_reg	   : -1,	$ ; The source Plot Region
		  mbase	   : 0L,	$ ; Uvalue used to store data.
		  ndims	   : Ndims,	$ ; Data dimensionality
		  length   : length,	$
                  lenmax   : 0,		$
                  fit      : 0,		$ ; Fit switch
		  average  : average,	$
		  vertical : vertical,  $
		  len_text : 0L,	$
		  avg_txtID: 0L,	$
		  x	   : EV.X,	$ ;	Center of source line/box
		  y	   : EV.Y,	$ ;	Center of source line/box
		  ac	   : EV.X,	$ ;   Actual center
		  Plotreg  : -1,	$ ;   Plot region for the profile
		  title	   : title,	$
		  xtitle   : xtitle,	$
		  ytitle   : ytitle,	$
		  dtitle   : datatitle, $
		  subtitle : subtitle,  $
		  xformat  : xformat,	$
		  yformat  : yformat,	$
		  polyx	   : [0,0], $ ;
		  polyy	   : [0,0]  $
	      }
      dz = [info]
      lastwhich = -1
  END
  
  
;
; Is there already a widget corresponding to the call?
;
  
  active = 0
  which = (where(dz(*).P_reg eq P_reg,count))(0)
  
;
; If so, just update it..
;
  
  IF count gt 0 THEN BEGIN
      IF Keyword_SET(kill) THEN BEGIN
	  Widget_CONTROL,dz(which).base,/destroy
	  return
      EndIF
      active = 1
      IF lastwhich ne which or Keyword_SET(change) THEN BEGIN
	  IF N_elements(data) eq 1 THEN BEGIN
	      lastwhich = which
	      lastdata = data
	  END ELSE BEGIN
	      Widget_CONTROL,dz(which).mbase,Get_UVALUE=lastdata
	      lastwhich = which
	  END
	  IF Keyword_SET(change) THEN BEGIN
	      Widget_CONTROL,dz(which).mbase,Set_UVALUE=lastdata
	      IF Keyword_SET(title) THEN dz(which).title=title
	      IF Keyword_SET(xtitle) THEN dz(which).xtitle=xtitle
	      IF Keyword_SET(ytitle) THEN dz(which).ytitle=ytitle
	      IF Keyword_SET(datatitle) THEN dz(which).dtitle=datatitle
	      IF Keyword_SET(subtitle) THEN dz(which).subtitle=subtitle
	      IF Keyword_SET(xformat) THEN dz(which).xformat=xformat
	      IF Keyword_SET(yformat) THEN dz(which).yformat=yformat
	  END
      END
      pqlp_plot,ev,which,change=change
      return
  END
  
;
; If this was just notification of change - don't make new widget.
;  
  
  IF Keyword_SET(change) or Keyword_SET(kill) THEN return
  
;
; Else: Create new widget to show data..
;
  
  which = N_elements(dz)
  dz = [dz,{QLPINFO}]
  
  
;
; Determine where to place top level base....
;
  
  p = tlb_place(x_size+20,y_size+50,pwindow=P_reg)
  xpos = p(0)
  ypos = p(1)
  
;
; Creation of widget base/other widgets....
;
  
  PQLPROFILEbase = Widget_BASE(TITLE = "PQLPROFILE",/column,$
				GROUP_LEADER=ev.top,Uvalue=which,space=0,$
				xoffset=xpos,yoffset=ypos)
  
  qlp = PQLPROFILEBASE ;shorthand
  qcol = Widget_BASE(qlp,/column,/frame)
  
  cmouse_s = {CMOUSE_S, btext:'', mtext:'', uvalue:'', avail:0}
  
  menu=[{CMOUSE_S,"Zoom out", "Zoom out (display more pixels)","ZOUT",7+8},$
        {CMOUSE_S,"Recenter", "Recenter on a pixel","RECENTER",7},$
   {CMOUSE_S,"Zoom in", "Zoom in (display fewer pixels)","ZIN",7},$   ; Paal
        	{CMOUSE_S,"Select line", "Select line","LINE",7+8} ]
  
  cmouse,qlp,qcol,menu
  
;  label1 = Widget_LABEL(qcol,value='Mouse button action:')
;  rowbase0 = Widget_BASE(qcol,/row)
;  label2 = Widget_LABEL(rowbase0,value='1: Recenter',/frame)
;  label2 = Widget_LABEL(rowbase0,value='2: No action',/frame)
;  label2 = Widget_LABEL(rowbase0,value='3: No action',/frame)
  
  rowbase = Widget_BASE(qlp,/row)
  
  b = Widget_BUTTON(rowbase,value='Done',Uvalue='DONE')
  
  XPdMenu,['"Plot" {','"Plot to file" PSFILE','"Print" PSPRINT','}'],rowbase
  inner = Widget_BASE(qlp)
  
  b = Widget_BUTTON(rowbase,value='Save',Uvalue='SAVE')
  b = Widget_BUTTON(rowbase,value='Help',Uvalue='HELP')
  
  
  
  dummy = {PSELECT_S, btext:'', mtext:'', uvalue:'', flags:0}
  menu=[{PSELECT_S, ' None', 'No line fit','NOFIT',0},$
        {PSELECT_S, ' Gaussian', 'Gaussian profile', 'IGAUSS',0}]
    
  dummy = cw_pselect(rowbase,'Line fit:',menu)
  
  
  
  
;  
;  XPdMenu,['"Line Fitting" {','"Gaussian Fit" GAUSS',$
;	   '"Amoebe" AMOEBE','"Other" OTHER','}'],rowbase
;  
  plotwin = Widget_DRAW(inner,xsize=x_size,ysize=y_size,uvalue='DRAW',$
			/button_events,/motion_events)
  
;
; Length slider
; The appearance of slider widgets differs from 3.0 to 3.X
;
  sub_version = float(!version.release)
  IF sub_version gt 3.0 THEN slide_offs=40 $
  ELSE			     slide_offs=0
  
  len_base = Widget_BASE(qlp,/frame)
  len_text_base = Widget_BASE(len_base,/row)
  dummy = Widget_LABEL(len_text_base,value='Length of profile: ')
  len_text_id = Widget_TEXT(len_text_base,value=trim(length),$
				xsize=12,ysize=1)
  dummy = Widget_LABEL(len_text_base,value=' pixels')
  lenmax = max(sz(1:sz(0)))
  length = (length>3)<lenmax
  dummy = Widget_SLIDER(len_base,min=3,max=lenmax,xsize=x_size,$
			yoffset=slide_offs,$
			uvalue='LENGTH',/suppress,/drag,value=length)
      textbase = WIDGET_BASE(qlp,/column)
      textw = WIDGET_TEXT(textbase,xsize = 80,ysize = 1,/frame)
  
  
;
; 2D DATA ONLY:
;
  IF Ndims eq 2 THEN BEGIN
      
;
; Orientation
;
      xmenu,['Horizontal','Vertical'],qlp,/exclusive,/row,$
		      UVALUE=['HORIZONTAL','VERTICAL'], buttons=buttons
      Widget_CONTROL,buttons(0),/set_button
;
; AVERAGE slider (only for 2D data)
;
      avgmax = max(sz(1:2))
      npix_base = Widget_BASE(qlp,/frame)
      avg_txt_base = Widget_BASE(npix_base,/row)
      dummy = Widget_LABEL(avg_txt_base,value='Average over ')
      avg_txtID = Widget_TEXT(avg_txt_base,value=trim(average),$
				xsize=5,ysize=1)
      dummy = Widget_LABEL(avg_txt_base,value=' pixels')
      dummy = Widget_SLIDER(npix_base,min=1,max=avgmax,xsize=x_size,$
			    yoffset=slide_offs,$ $
                            uvalue='AVERAGE',/suppress,/drag,value=average)
      
  END ELSE avg_txtID = 0L
  
;
; That's the setup, now show ourselves...
;
  
  Widget_CONTROL, PQLPROFILEbase, /REALIZE
  
;
; And store vital information in the information block.
;  
  
  Widget_CONTROL,plotwin,Get_VALUE=qlpwnd
  SetWindow,qlpwnd
  Plotreg = pstore(1,0,0,/clean)
  
  dz(which).P_reg = P_reg
  dz(which).x = ev.x			  ;	Center of source box
  dz(which).y = ev.y
  dz(which).ndims = Ndims
  dz(which).length = length
  dz(which).lenmax = lenmax
  dz(which).vertical = vertical
  dz(which).average = average
  dz(which).title = title
  dz(which).xtitle= xtitle
  dz(which).ytitle= ytitle
  dz(which).dtitle=datatitle
  dz(which).xformat=xformat
  dz(which).yformat=yformat
  dz(which).plotreg = plotreg
  dz(which).base = PQLPROFILEbase
  dz(which).mbase=rowbase
  dz(which).len_text =len_text_id
  dz(which).avg_txtID=avg_txtID
  dz(which).polyx(*)=-1
  
  Widget_CONTROL,dz(which).mbase,Set_UVALUE=DATA
  lastdata = DATA
  lastwhich = which
  
;
; Sign up
;
  
  Xmanager, "PQLPROFILE", PQLPROFILEbase, CLEANUP='pqlp_cleanup'
  
;
; Update display..
;
  
  pqlp_plot,ev,which
  
END





















