;+
; Project     : SOHO - CDS     
;                   
; Name        : PQLZOOM
;               
; Purpose     : General purpose draw-window zoom.
;               
; Explanation : Passing an event generated in a draw widget inside
;		a stored plot region will cause a new widget to be 
;		created, with a zoomed image taken from the originating 
;		draw window. 
;               
; Use         : PQLZOOM,DATA
;    
; Inputs      : DATA:	The data that is displayed in the plot region.
;               
; Opt. Inputs : None.
;               
; Outputs     : None.
;               
; Opt. Outputs: None.
;               
; Keywords    : NOSCALE: Set to disable initial rescaling of
;			the data before TV'ing.
;
;		TITLE = title of the zoom plot window
;
;		SUBTITLE = subtitle of zoom plot window.
;
;		X/YTITLE = X/Y titles.
;
;		XYSIZE = size of zoom window display. Default 220.
;
;		MARGIN = Total margin (in device coord's) around the zoom
;			plot window.
;
;		ORIGIN = [x0,y0] The lower left corner (in device coord's)
;			of the zoom plot window.
;
;		EVENT =	A WIDGET_DRAW event structure generated in
;			a widget_draw window.
;
;		P_reg: The plot region ID (as returned by pfind())
;			identifying which plot that is affected.
;
;		CHANGE : Set to indicate that a change of the DATA 
;			in the plot region has been made.
;			If there is a profile widget for this plot
;			region then this will update it's display
;			and the markings. If not, nothing happens.
;			If the data has not been changed, just 
;			redisplayed, DATA need not be supplied.
;
;		ZOOMFACT = Initial zoom factor 
;
;		GROUP = Widget ID for the top level base that is to
;			be group leader (zoom window dies upon death of
;			the group leader).
;
;		KILL: 	Set to kill any zoom widget from the specified
;			plot region.
;
;		ACTIVE: Set to a named variable to return 1 if there is
;			a profile window for the specified plot region
;	
;		RESET:  Set to reset the common block.
;
; Calls       : CDSCONGRID(), CDSNOTIFY, CLIPBOX, CMOUSE, CMOUSE_ACTION(), 
;		DATATYPE(), PCONVERT(), PFIND(), PRESTORE, PSTORE(), 
;		SETWINDOW, TLB_PLACE(), TRIM(), WIDGET_BASE(), WIDGET_TEXT(), 
;		WIDG_HELP
;		(11-Dec-94)
;
; Common      : PQLZOOM:
;			Keeps track of window ID's for each PQLZOOM
;			window that has been generated.
;               
; Restrictions: The correct subwindow must have been selected..
;               
; Side effects: 
;               
; Category    : Display
;               
; Prev. Hist. : None.
;
; Written     : Stein Vidar Hagfors Haugan
;               
; Modified    : SVHH, 17 Nov. -- added setwindow call for non-
;                                interference with Bill T. 
;                                display routines
;		SVHH, 19 Nov. -- Documentation brush-up
;		SVHH, March 1994, Switched to pstore/prestore/pfind routines.
;		SVHH, May/June 1994,
;			Added plot axes and most keywords.
;               PB,   28 Nov. -- added Read Cursor Position functionality,
;				 Text_widget for output,
;                                Plot to file/printer 
;                                Default size increased to 220
;               PB,   Sept 21 - 94    Added Help using WIDG_HELP
;		SVHH, Nov 19 - 94 -- Made "Read position" report
;			continuously when middle button is pressed down
;			and fixed a POSITION keyword roundoff problem.
;		SVHH, Dec 10 - 94 -- Added a calls to CMOUSE/CMOUSE_ACTION,
;			and added x/y-format/units
;               Version 3, SVHH, 17 November 1998
;                       Fixed {WIDGET_DRAW} for IDL v 5
;		Version 4, 29 January 1999, William Thompson, GSFC
;			Fixed {WIDGET_DRAW} for IDL v 4
;		Version 5, 20 December 1999, William Thompson, GSFC
;			Fixed bug preventing compilation in IDL v5.3
;		Version 6, 5 January 2001, William Thompson, GSFC
;			Fixed bug for versions 5.3 and 5.4
;
; Version     : Version 6, 5 January 2001
;- 


PRO pzoom_position,ev,which,update=update,change=change ; UPDATE DISPLAY
  common PQLZOOM,DZ,lastwhich,data,sbox,textw
  On_Error,0
  
  info = DZ(which)
  
  prestore,info.P_reg,datax,datay,mx,my,jx,jy
  
  Widget_CONTROL,info.base,/show
  
;
; Make ready for grab (top/bottom coord's etc.)
;
  
  ssize = info.Zsize/info.zoomFact
  
  IF NOT (Keyword_SET(update) or Keyword_SET(change)) THEN BEGIN
      info.x = float(ev.x - jx)*datax/mx
      info.y = float(ev.y - jy)*datay/my
  END
  
;
; Make ready for grab (top/bottom coord's etc.)
; Get the indexes for the "source" area (what region of the original data
; should be plotted)
;
  
  xtop = long(info.x + ssize/2 - 1 + ssize mod 2)
  ytop = long(info.y + ssize/2 - 1 + ssize mod 2)
  xbot = long(info.x - ssize/2)
  ybot = long(info.y - ssize/2)
  
  x_start = pconvert(info.P_reg,xbot,/pix,/to_data,/lower)
  y_start = pconvert(info.P_reg,ybot,/Y,/pix,/to_data,/lower)
  x_stop  = pconvert(info.P_reg,xtop+1,/pix,/to_data,/lower)
  y_stop  = pconvert(info.P_reg,ytop+1,/Y,/pix,/to_data,/lower)
  
;
; Check limits
;
  
  xtop = (xtop < (datax-1)) >0
  ytop = (ytop < (datay-1)) >0
  xbot = (xbot > 0) < xtop
  ybot = (ybot > 0) < ytop
  
  grab = data(xbot:xtop,ybot:ytop)
  
;
; Flicker display in source window by inverting the "source box"
;
  IF total(info.polyx gt -1) gt 0 $
	  and NOT Keyword_SET(change) THEN pdz_plotpoly,info
  info.polyx=[xbot,xbot,xtop,xtop,xbot]
  info.polyy=[ybot,ytop,ytop,ybot,ybot]
  pdz_plotpoly,info
  
;
; Make ready for rescaling (put source data into a standard size box 
; in case it has been clipped)
;
  
  szgrab = Size(grab)
  
  sbox = 0
  CASE datatype(grab) OF
      'BYT': sbox=bytarr(ssize,ssize)
      'INT': sbox=intarr(ssize,ssize)
      'LON': sbox=lonarr(ssize,ssize)
      'FLO': sbox=fltarr(ssize,ssize)
      'DOU': sbox=dblarr(ssize,ssize)
  EndCASE
  
  startx = xbot - (info.x - ssize/2)
  starty = ybot - (info.y - ssize/2)
  
  stopx = startx + Szgrab(1) - 1
  stopy = starty + Szgrab(2) - 1
  
  sbox(startx:stopx,starty:stopy) = grab
  
  SetWindow,info.zwindow
  
;
; Display the zoomed image.
;
  
  
  interp = info.interp
  
  pos = [info.origin(0),info.origin(1),$
	info.origin(0)+info.Zsize-1,info.origin(1)+info.Zsize-1]+$
	0.0001  ; IDL sometimes chops a pixel.....I'm tired of looking
		; up these bugs!
  
  erase
  Plot,[0,0],/nodata, xrange=[x_start,x_stop],xstyle=1,$
		      yrange=[y_start,y_stop],ystyle=1,$
		      position=pos,/DEVICE,ticklen=-5./!D.X_size,$
		      title=info.title,subtitle=info.stitle,$
		      xtitle=info.xtitle,ytitle=info.ytitle
  
  dummy=pstore(1,ssize,ssize,/clean)
  
  IF info.rescale THEN BEGIN
      tvscl,cdsCongrid(sbox,info.Zsize,info.Zsize,interp=interp),$
	    info.origin(0),info.origin(1)
  END ELSE BEGIN
      max = max(data)
      min = min(data)
      abox = bytscl(sbox,max=max,min=min)
      tv,cdsCongrid(abox,info.Zsize,info.Zsize,interp=info.interp),$
	 info.origin(0),info.origin(1)
  END
  
  clipbox
  
  prestore,info.P_reg
  
  DZ(which) = info		  ; Put back some status info
  
END




PRO pdz_plotpoly,info
  prestore,info.P_reg,sx,sy,mx,my,jx,jy,plot_number=plotnum
  IF datatype(plotnum) eq 'STR' THEN return
  DEVICE,get_graphics_function=oldgraphics
  DEVICE,set_graphics_function=6
  plots,pconvert(info.P_reg,info.polyx,/pix,/to_dev),$
	pconvert(info.P_reg,info.polyy,/pix,/to_dev,/Y),thick=2,/DEVICE
  DEVICE,set_graphics_function=oldgraphics
END


;
; Make sure we clean up our act upon dying... (to avoid slaying other)
; windows later
;

PRO Pdz_cleanup,ID
  common PQLZOOM,DZ,lastwhich,lastdata,sbox,textw
  
  Widget_CONTROL,ID,Get_UVALUE=which
  
  pdz_plotpoly,DZ(which)	; Erase markings
  DZ(which)={DZinf}
  DZ(which).P_reg = -1
END


PRO pqlzoom_readpos,w,event
  common PQLZOOM,DZ,lastwhich,lastdata,sbox,textw
  
  p = pfind(event,found)
  IF NOT found THEN return
  pixx = pconvert(p,event.x,/dev,/to_pix)
  pixy = pconvert(p,event.y,/dev,/to_pix,/Y)
  xsol = pconvert(p,event.x,/dev,/to_data)
  ysol = pconvert(p,event.y,/dev,/to_data,/Y)
;  IF strmid(DZ(w).xtitle,0,1) eq 'W' THEN units=string(197b) $
;  ELSE units=''
  utext = DZ(w).xtitle+'='+trim(xsol,DZ(w).xformat)+DZ(w).xunits
  utext = utext + '  '+$
	  DZ(w).ytitle+'='+trim(ysol,DZ(W).yformat)+DZ(w).yunits
  utext = utext + '  '+$
	  'Value='+trim(sbox(pixx,pixy),DZ(w).format)+DZ(w).units
  Widget_CONTROL,textw,Set_VALUE=utext
END


;
; Event handling -------------------------------------
;

PRO PQLZOOM_event, event
  common PQLZOOM,DZ,lastwhich,lastdata,sbox,textw
  
  Widget_CONTROL, event.top, Get_UVALUE = which
  Widget_CONTROL, event.id, Get_UVALUE = eventval	  ;find the user value

  
;  info = DZ(which)       ; I'm skipping this and replacing info with DZ(w)

  w = which
  oldzoom = DZ(w).zoomFact
  
;
; Do we have the right data set in the cache?
;
  
  IF lastwhich ne which THEN BEGIN
      lastwhich = which
      Widget_CONTROL,DZ(w).mbase,Get_UVALUE=lastdata
  EndIF
  
  redraw = 0
  
  IF eventval eq 'DRAW' THEN eventval = cmouse_action(event)

  CASE eventval OF
      'DONE': BEGIN
	  Widget_CONTROL,event.top,/destroy
	  return
      END
      
      'PSFILE': BEGIN
	  cdsnotify,group=event.top,'Not implemented yet!!'
      END
      
      'PSPRINT': BEGIN
	  cdsnotify,group=event .top,'Not implemented yet!!'
      EndCASE
      
      'MEASURE' : pqlzoom_readpos,w,event

      'ZOUT' : IF DZ(w).zoomFact gt 1 THEN BEGIN
	  redraw=1
	  DZ(w).zoomFact = DZ(w).zoomFact/2
      END

      'CENTER' : BEGIN
	  sz = Size(lastdata)
	  DZ(w).x = (DZ(w).x + (event.x - DZ(w).origin(0) $
		      - DZ(w).Zsize/2)/oldzoom) > 0 <sz(1)
	  DZ(w).y = (DZ(w).y + (event.y - DZ(w).origin(0) $
		      - DZ(w).Zsize/2)/oldzoom) > 0 <sz(2)
	  redraw=1
      ENDcase

      'ZIN' : IF DZ(w).zoomFact lt 128 THEN BEGIN
	  DZ(w).zoomFact=DZ(w).zoomFact*2
	  redraw=1
      ENDif

      '': return
      
      'INTERP': BEGIN
	  DZ(w).interp = event.select & redraw=1
      END
      
      'RESCALE':  BEGIN
	  DZ(w).rescale = event.select & redraw=1
      END
      'READPOS': DZ(w).readpos = event.select
      
      'HELP': BEGIN
	  widg_help,'pqlzoom',$
	  title = 'Help: Zoom Utility',group=event.top
      END
      
      ELSE: MESSAGE, "Event User Value Not Found"
  EndCASE
  
;
; Make sure we report the right position to the Update procedure
;
  
  IF redraw THEN pzoom_position,event,which,/update
  
END




PRO PQLZOOM,DATA,noscale=noscale, $
		title=title,subtitle=subtitle,format=format,units=units,$
		xtitle=xtitle,xformat=xformat,xunits=xunits,$
		ytitle=ytitle,yformat=yformat,yunits=yunits,$
		xysize = xysize, origin=origin, margin=margin,$
		zoomfact=zoomfact, outside=outside,$
		EVENT=ev,P_reg=P_reg,change=change,cursor=cursor,$
		GROUP=GROUP,kill=kill, reset=reset, active=active
  
  common PQLZOOM,DZ,lastwhich,lastdata,sbox,textw
  
  On_Error,2
  IF !DEBUG ne 0 THEN On_Error,0
  
  IF Keyword_SET(reset) THEN BEGIN
      DZ = 0
      return
  EndIF
  
;
; Check if we have enough information
;
  
  sz = Size(DATA)
  Ndims = sz(0)
  IF (N_elements(DATA) le 1 or Ndims ne 2) and NOT Keyword_SET(change) THEN $
		message,'Data must be 2D array'
  
  IF N_elements(ev) eq 1 THEN BEGIN
      P_reg = pfind(ev,found)
      IF NOT found THEN return
  END ELSE IF N_elements(P_reg) eq	1 THEN BEGIN
     prestore,P_reg
     IF since_version('5.3') then begin
	ev = {Widget_DRAW}
	ev.x = !P.clip(0)+1
	ev.y = !P.clip(0)+1
     END ELSE begin
	command = ' ev = {Widget_DRAW, ID:0L, TOP:0L, HANDLER:0L, TYPE:0, ' + $
              'X:fix(!P.clip(0)+1), Y:fix(!P.clip(0)+1), press:0b, release:0b'
	IF since_version('5') THEN command = command + ', clicks:0L}' ELSE $
	     command = command + '}'
	test = execute(command)
     ENDELSE
  END ELSE message,'Must have either EVENT or P-window'
  
;
; Defaults
;
  IF N_elements(xysize) eq 0 THEN xysize = 300	   ;increased from 200
  IF N_elements(origin) eq 0 THEN origin = [60,60]
  IF N_elements(margin) eq 0 THEN margin = 100
  IF N_elements(zoomfact) eq 0 THEN zoomfact=4
  IF N_elements(title) eq 0 THEN title=''
  IF N_elements(subtitle) eq 0 THEN subtitle=''
  IF N_elements(format) eq 0 THEN format=''
  IF N_elements(units) eq 0 THEN units=''
  IF N_elements(xtitle) eq 0 THEN xtitle=''
  IF N_elements(xformat) eq 0 THEN xformat=''
  IF N_elements(xunits) eq 0 THEN xunits=''
  IF N_elements(ytitle) eq 0 THEN ytitle=''
  IF N_elements(yformat) eq 0 THEN yformat=''
  IF N_elements(yunits) eq 0 THEN yunits=''
  
; We now have reasonable event coordinates, and we have the P-window
; If no Pqlprofile has been invoked before, we need to init common block DZ
; (and the structure type)
  
;
; If no Pqlzoom has been invoked before, we need to init DZ
;
  
  IF DATATYPE(DZ) NE 'STC' THEN BEGIN
      info = {DZinf, $
		base	   : 0L,	  $ ; This top-level base
		P_reg	   : -1,	  $ ; Source plot region
		mbase	   : 0L,	  $ ; Uvalue used to store data.
		textw	   : 0L,	   $
		zoomFact : zoomfact,	  $
		interp	 : 0,		  $ ; Interpolation
		rescale  : -1,	  $ ;	
		readpos  : -1,	  $ ; Read cursor position switch
		x	   : EV.X,	  $ ;	Center of source box
		y	   : EV.Y,	  $ ;	Center of source box
		pressed  : 0,	       $; 
		zwindow  : -1L,	      $ ;     Zoom window "WSET" ID
		title	 : title,	      $
		stitle	 : subtitle,	$
		format   : format,	$
		units	 : format,	$
		xtitle	 : xtitle,	$
		xformat  : xformat,	$
		xunits   : xunits,      $
		ytitle	 : ytitle,	$
		yformat  : yformat,	$
		yunits   : yunits,	$
		Zsize	 : xysize,	$ ;	Size of zoom window
		origin	 : origin,	$
		polyx	 : [0,0,0,0,0], $ ;
		polyy	 : [0,0,0,0,0]  $
	}
      dz = [info]
      lastwhich = -1
  END
  
  
  
  
;
; Is there already a widget corresponding to the call?
;
  active = 0
  which = (where(P_reg eq dz(*).P_reg,count))(0)
  
;
; If so, just update it..
;
  w = which
  IF count gt 0 THEN BEGIN
      IF Keyword_SET(kill) THEN BEGIN
	  Widget_CONTROL,dz(w).base,/destroy
	  return
      EndIF
      active = 1
      IF lastwhich ne w or Keyword_SET(change) THEN BEGIN
	  IF N_elements(data) gt 1 THEN BEGIN
	      lastwhich = w
	      lastdata = data
	  END ELSE BEGIN
	      Widget_CONTROL,dz(w).mbase,Get_UVALUE=lastdata
	      lastwhich = w
	  END
	  IF Keyword_SET(change) THEN BEGIN
	      Widget_CONTROL,dz(w).mbase,Set_UVALUE=lastdata
	      IF Keyword_SET(title) THEN dz(w).title=title
	      IF Keyword_SET(subtitle) THEN dz(w).stitle=subtitle
	      IF Keyword_SET(xtitle) THEN dz(w).xtitle=xtitle
	      if keyword_set(xformat) then dz(w).xformat=xformat
	      if keyword_set(xunits) then dz(w).xunits=xunits
	      IF Keyword_SET(ytitle) THEN dz(w).ytitle=ytitle
	      if keyword_set(yformat) then dz(w).yformat=yformat
	      if keyword_set(yunits) then dz(w).yunits=yunits
	  EndIF
      EndIF
      pzoom_position,ev,w,change=change
      return
  END
  
;
; If it was just a notification about change of data - return
;
  
  IF Keyword_SET(change) or Keyword_SET(kill) THEN return
  
;
; Else: Create new widget to show data..
;
  
  which = N_elements(dz)
  dz = [dz,{DZinf}]
  
;
; Determine where to place top level base....
;
  
  p = tlb_place(xysize+50,xysize+50,pwindow=P_reg)
  
  xpos = p(0)
  ypos = p(1)
  
;
; Creation of widget base/other widgets....
;
  
  PQLZOOMbase = Widget_BASE(TITLE = "PQLZOOM",/column,$
				  GROUP_LEADER=ev.top,Uvalue=which,space=0,$
				  xoffset=xpos,yoffset=ypos)
  
  dzb = PQLZOOMBASE ; shorthand
  
  dummy = {CMOUSE_S, btext:'', mtext:'', uvalue:'', avail:0}
  
  menu = [ {CMOUSE_S, "Zoom out", "Zoom out", "ZOUT", 1},$
	   {CMOUSE_S, "Recenter", "Recenter on cursor position","CENTER", 2},$
	   {CMOUSE_S, "Measure",  "Measure intensity/position", "MEASURE",15},$
	   {CMOUSE_S, "Zoom in", "Zoom in", "ZIN", 4}]
  
  cmouse,dzb,dzb,menu,init=[0,1,3]
  
  rowbase = Widget_BASE(dzb,/row,/frame)
  
  b = Widget_BUTTON(rowbase,value='Done',Uvalue='DONE')
  b = Widget_BUTTON(rowbase,value='Help',Uvalue='HELP')
  XPdMenu,['"Plot" {','"Plot to file" PSFILE','"Print" PSPRINT','}'],rowbase
  
  
  nonex = Widget_BASE(rowbase,/nonexclusive,/row)
  b = Widget_BUTTON(nonex,value='Smooth',Uvalue='INTERP')
  b = Widget_BUTTON(nonex,value='Rescale',Uvalue='RESCALE')
  IF NOT Keyword_SET(noscale) THEN Widget_CONTROL,b,/set_button
  
  inner = Widget_BASE(dzb)
  zoomim = Widget_DRAW(inner,xsize=xysize+margin,ysize=xysize+margin,$
		  uvalue='DRAW',/button_ev,/frame,/motion_ev)
  textw=Widget_TEXT(dzb,xsize=30,ysize=1,/frame)
  
;--------------------
; Should use info. but can't get it to work, info not defined here
;help,/str,info
;info.textw=tull
;info.textw = Widget_TEXT(dzb,xsize=30,ysize=1)
;--------------------
  
;
; Show ourselves...
;
  
  Widget_CONTROL, PQLZOOMbase,  /REALIZE
  
  Widget_CONTROL,zoomim,Get_VALUE=zwindow
  
;
; And store vital information in the information block.
;  
  
  dz(which).rescale = NOT Keyword_SET(noscale)
  dz(which).zoomFact = zoomFact
  dz(which).P_reg = P_reg
  dz(which).x = ev.x			  ; Center of source box
  dz(which).y = ev.y
  dz(which).zwindow = -1L		  ; Zoom window "WSET" ID
  dz(which).Zsize = xysize		  ; Size of zoom window
  dz(which).origin = origin		  ; 
  dz(which).zwindow = zwindow
  dz(which).base = PQLZOOMbase
  dz(which).mbase= rowbase
  dz(which).polyx(*)=-1
  dz(which).title=title
  dz(which).stitle=subtitle
  dz(which).format=format
  dz(which).units=units
  dz(which).xtitle=xtitle
  dz(which).xformat=xformat
  dz(which).xunits=xunits
  dz(which).ytitle=ytitle
  dz(which).yformat=yformat
  dz(which).yunits=yunits
  
  Widget_CONTROL,dz(which).mbase,Set_UVALUE=data
  lastdata = data
  lastwhich = which
  
;
; Sign up
;
  
  Xmanager, "PQLZOOM",  PQLZOOMbase, CLEANUP='Pdz_cleanup'
  
;
; Update display..
;
  
  pzoom_position,ev,which
  
END























