;+
; Project     : SOHO - CDS     
;                   
; Name        : QL_RD_LLIST
;               
; Purpose     : Read a CDS line list into the returned structure.
;               
; Explanation : Reads a line list file into the structure required by the 
;               technical planning procedure.  The line list file is assumed
;               to have the following contents:
;               Line 1 - header text
;               Line 2 - blank
;               Line 2 - line identification, wavelength, order, band, pixel
;               .
;               .
;               .
;               Line N - ditto
;
;               
; Use         : IDL> ql_rd_llist, in_file [, header=header]
;    
; Inputs      : in-file    -   the name of the file with the line list.
;               
; Opt. Inputs : None
;               
; Outputs     : None
;               
; Opt. Outputs: header  -  the header line from the line-list file
;               
; Keywords    : None
;
; Calls       : BELL, CDSNOTIFY, RD_LINE_LIST,STR2ARR(),VALID_NUM()
;               
; Restrictions: 
;               
; Side effects: None.
;               
; Category    : QuickLook.
;               
; Prev. Hist. : Taken from C D Pike's RD_LINE_LIST, V 3, 26-Nov 1993
;
; Written     : C D Pike, 14-May-1993
;               
; Modified    : New input file format.  CDP 17-Nov-1993
;               Limit number of lines to 20.  CDP 26-Nov-93
;		SVHH, Version 1, 7-June-1994
;			Adapted for use with QL software (skipped common 
;			blocks, flash_info calls etc.)
;			Changed limit to 50 lines.
;
; Version     : Version 1, 7-June-1994
;-            

FUNCTION ql_rd_llist, in_file, header=header, curr_lwidth,curr_lheight
  
  IF 0 THEN rd_line_list  ; If there are changes to rd_line_list, I want to be
			  ; able to catch it by "Callers"
  
  IF N_elements(curr_lwidth) eq	0 THEN curr_lwidth = 10
  IF N_elements(curr_lheight) eq 0 THEN	curr_lheight = 120
  
  max_num_lwindow = 50
  max_num_fwindow = 4
  
;
;  default header
;
  header = 'CDS VDS line list'
  
;
;  check required parameters
;
  IF n_params()	lt 1 THEN BEGIN
      Bell
      cdsnotify,'Use: ql_rd_llist, in_file [,header=header]'
      return,0
  EndIF
  
;
;  open file for reading.
;
  ff = findfile(in_file, count=count)
  IF count gt 1	THEN BEGIN
      Bell
      cdsnotify,'Warning: more than one lines file found.'
  EndIF
  
  IF count eq 0	THEN BEGIN
      Bell
      cdsnotify,'Error: no lines file found.'
      num_windows = 0
      return,0
  EndIF
  
  openr,lun,ff(0),/get_lun
  
;
;  read header and spacer lines
;
  header = ' '
  readf,lun,header
  text = ''
  readf,lun,text
  
;
;  the third token in the header must be the line id number (ie usually
;  entered as "Line list nn").  If not, return.
;
  tok =	str2arr(header,' ',/nomult)
  IF valid_num(tok(2),/int) THEN BEGIN
      LL_id = tok(2)
  EndIF	ELSE BEGIN
      Bell
      cdsnotify,'** Invalid line list ID number in file.**'
      num_windows = 0
      free_lun,lun
      return,0
  EndELSE
  
;
;  read first line of data to check id format
;
  readf,lun,text
  in = str2arr(text,' ',/nomult)
  CASE 1 OF
      (in(3) eq	'(1)') or (in(3) eq '(2)'): offset = 1
      (in(2) eq	'(1)') or (in(2) eq '(2)'): offset = 0
      ELSE: BEGIN
	  cdsnotify, 'Error in line file format.'
	  free_lun,lun
	  num_windows =	0
	  return,0
      END
  EndCASE
;
;  all ok so start again
;
  close,lun
  openr,lun,ff(0)   ; ,/get_lun We already got a lun
  
;
;  read header and spacer lines
;
  header = ' '
  readf,lun,header
  text = ''
  readf,lun,text
  
  dummy	 = {$
	x_pos:	  0,	 $	     ;  line centre in pixels
	lam:	0.0,	 $	     ;  line wavelength
	order:	  0,	 $	     ;  spectral order
	inten:	0.0,	 $	     ;  intensity (c/s)
	y_pos:	  0,	 $	     ;  line y position (0 or 128)
	left:	  0,	 $	     ;  left edge of line window in pixels
	right:	  0,	 $	     ;  right edge of line window in pixels
	height:	  0,	 $	     ;  window height in arcsec
	id:	 ' '}		     ;  line identification
  
  line_window =	dummy
  
;
;  read data, assumed in form: identification, wavelength, order, band, pixel
;  set defaults for the rest.  First assumption is that the id is of the form
;  Fe XXV ie with a space but allow for it with no space - otherwise error.
;
  id = ' '
  num =	0
  line_window =	replicate(dummy,max_num_lwindow)
  WHILE	NOT eof(lun) and num le	max_num_lwindow-1 DO BEGIN
      readf,lun,text
      in = str2arr(text,' ',/nomult)
      IF in(0) ne '' THEN BEGIN
	  dummy.x_pos	= in(4+offset)
	  dummy.lam	= in(1+offset)
	  dummy.order	= strmid(in(2+offset),1,1)
	  dummy.inten	= 0.0
	  obs_lam = dummy.lam *	dummy.order
	  dummy.y_pos	= (obs_lam lt 400.) * 128
	  dummy.left	= in(4+offset)-(curr_lwidth/2)
	  dummy.right	= dummy.left + curr_lwidth - 1
	  dummy.height	= curr_lheight
	  IF offset eq 0 THEN BEGIN
	      dummy.id	= in(0)
	  EndIF	ELSE BEGIN
	      dummy.id	= in(0)+' '+in(1)
	  EndELSE
	  line_window(num) = dummy
	  num =	num + 1
      EndIF
  EndWHILE
  free_lun,lun
  
;
;  tidy the structure
;
  num_windows =	max(where(line_window(*).lam gt	0.0)) +	1
  
  return,line_window(0:num_windows-1)
END
