;+
; Project     : SOHO - CDS     
;                   
; Name        : CWQ_DSPWAV
;               
; Purpose     : Constitutes one column of the data display in DSPWAV.
;               
; Explanation : This compound widget binds together a CWQ_WINSEL, a CWQ_IMAGE
;               and a CWQ_SPECTR to make up one "column" of the data display
;               area in DSPWAV.
;
;               It always starts up with the image focus point in the middle
;               of the image, and the spectral focus point in the middle of
;               the spectral window.
;
;               These initial values may be retrieved by the main program by
;               using WIDGET_CONTROL,CWQ_ID,GET_VALUE=VALUE, where CWQ_ID is
;               the compound widget ID returned by CWQ_DSPWAV, and VALUE will
;               be returned as a structure with tags WINDOWI, OFFSET, XIX,
;               YIX, TIX, IMZOOM (image zoom), LINEFIT (string describing the
;               current line fitting method, see CDS_LINEFIT_WRAP), and
;               LOGARITHMIC, which is one if CWQ_SPECTR uses a logarithmic Y
;               axis.
;
;               All these values may also be set by the main program, by using
;               WIDGET_CONTROL, CWQ_ID, SET_VALUE={<parameter>:value}. All the
;               changes to these status variables will be reflected in the
;               display.
;
;               There's also a status variable called SMOOTH that is passed on
;               to CWQ_SPECTR to control smoothing of fixed pattern noise for
;               the GIS detector.
;
;               The widget can be made autonomous, in which case it will never
;               return any events to the owner, but self-acknowledge all user
;               actions inside the data windows.
;
;               If it is not operating in the autonomous mode, it generates
;               events whenever the image focus point or zoom value has been
;               changed. The event structure {CWQ_DSPWAV_EVENT} contains the
;               folowing tags:
;
;                    ID :     ID of the compound widget
;                    TOP:     ID of the top of the widget hierarchy
;                    HANDLER: ID of the widget handler receiving the event.
;                    SET : A {CWQ_DSPWAV_SET} structure with tags XIX,
;                          YIX and IMZOOM (image zoom).
;                    OLD : Same as SET, but with old (currently
;                          displayed) values.
;                    EVENT: The original CWQ_IMAGE event
;                    
; Use         : CW_ID = CWQ_DSPWAV(BASE, QLDS [,KEYWORDS])
;    
; Inputs      : BASE : The base to put the compound widget on.
;
;               QLDS : The QLDS from which to take the data to be displayed.
;                      During the active life of the compound widget, it must
;                      be possible to use the QLDS_CHECKIN mechanism to get at
;                      the QLDS.
;               
; Opt. Inputs : None.
;               
; Outputs     : Returns the compound widget ID.
;               
; Opt. Outputs: None.
;               
; Keywords    : UVALUE : The user value to be associated with this widget.
;
;               NO_COPY : Whether to use NO_COPY when setting the UVALUE.
;
;               CW_MOUSE_IMAGE,
;               CW_MOUSE_PLOT : The widget ID OF any CW_MOUSE control box to
;                               be used in the image or spectral plot,
;                               respectively.
;
;               COLUMN : Set this explicitly to zero to stop the compound
;                        widget from appearing with a column layout.
;
;
;               XTVSCALE : The XTVSCALE ID to use when displaying the images.
;
;               NUMBER : For more than one CWQ_DSPWAV in a row, give each one
;                        a number (1,2,3,...). If NUMBER is zero, there is no
;                        point in having a private XTVSCALE button made.
;
;               WINDOWI : The index of the initially selected line
;                         window. Will be set to the chosen value if not
;                         supplied, or if the supplied choice is not valid.
;                         Defaults to (NUMBER-1) > 0
;
;               PIXELS: The size of the widget_draw display windows to be used
;                       for cwq_image and cwq_spectr.
;
;               IMZOOM: The initial image zoom.
;
;               AUTONOMOUS: Set this keyword to make the compound widget
;                           self-acknowledge all events, without sending any
;                           events to the owner.
;                           
; Calls       : clipbox copy_tag_values cwq_dspwav_print cwq_image()
;               cwq_spectr() cwq_winsel() default detdesc() gt_dimension()
;               parcheck since_version() trim() typ() widget_base() xtvscale()
;
; Common      : None, but CWQ_IMAGE/CWQ_SPECTR do use common block QLSAVE, in
;               conjunction with QLDS_CHECKIN.
;               
; Restrictions: None.
;               
; Side effects: None known.
;               
; Category    : QuickLook, Compound_widget, Display
;               
; Prev. Hist. : Based on cw_pzoom
;
; Written     : S.V.H.Haugan, UiO, June 1996
;               
; Modified    : Version 2, SVHH, 10 July 1996
;                       Added fallthrough (to CWQ_SPECTR) status variable
;                       SMOOTH to cope with GIS fixed pattern noise.
;               Version 3, SVHH, 16 October 1996
;                       Saving space with IDL v 4.0(.1)
;                       
; Version     : 3, 16 October 1996
;-            



PRO cwq_dspwav_print,info
  
  
  ;; !P.CLIP should mark an area with approximate height:width ratio of
  ;; approx. 2.5:1
  
  clip = !P.clip  ;; This is the space we have available
  
  number = trim(info.int.number)
  dummy = FINDGEN(2)
  
  width = !p.clip(2)-!p.clip(0)
  height = !P.clip(3)-!P.clip(1)
  
  unit_ht = height/2.5
  
  ;; Top panel: Image, hopefully a square area
  
  x0im = clip(0)
  y0im = clip(3)-unit_ht
  x1im = clip(2)
  y1im = clip(3)
  
  !P.clip = [x0im,y0im,x1im,y1im]
  clipbox
  
  WIDGET_CONTROL,info.int.cwqimag,set_value={psplot:1}
  
  x0spec = clip(0)
  y0spec = clip(1)
  x1spec = clip(2)
  y1spec = clip(1) + 1.5*unit_ht
  
  !P.clip = [x0spec,y0spec,x1spec,y1spec]
  clipbox
  
  WIDGET_CONTROL,info.int.cwqspec,set_value={psplot:1}
END


;
; Return value
;
FUNCTION cwq_dspwav_getv,id
  
  store = WIDGET_INFO(id,/child)
  WIDGET_CONTROL,store,get_uvalue=info,/no_copy
  
  value = info.ext
  
  WIDGET_CONTROL,store,set_uvalue=info,/no_copy
  RETURN,value
END

            
  

PRO cwq_dspwav_setv,ID,VALUE
  
  IF datatype(value) NE 'STC' THEN RETURN ;; No primary value
  
  ;; Get storage/info
  
  store = WIDGET_INFO(id,/child)
  WIDGET_CONTROL,store,get_uvalue=info,/no_copy
  
  ;;
  ;; Setting status variables
  ;;
  
  tags = tag_names(value)
  
  ;; WRITE sensitive stuff
  
  IF total(tags EQ "PSPLOT") NE 0 THEN BEGIN
     ;; PRINT! and return!
     cwq_dspwav_print,info
     WIDGET_CONTROL,store,set_uvalue=info,/no_copy
     return
  END
  
  ext = info.ext
  copy_tag_values,ext,value
  
  ;; CHANGE sensitive stuff
  
  IF info.ext.windowi NE ext.windowi THEN BEGIN
     WIDGET_CONTROL,info.int.cwqwins,set_value=value.windowi
     setboth = {windowi:value.windowi}
  END
  
  IF info.ext.xix NE ext.xix THEN BEGIN
     IF NOT exist(setboth) THEN setboth = {xix:ext.xix} $
     ELSE  setboth = create_struct(setboth,'XIX',ext.xix)
  END
  
  IF info.ext.yix NE ext.yix THEN BEGIN
     IF NOT exist(setboth) THEN setboth = {yix:ext.yix} $
     ELSE setboth = create_struct(setboth,'YIX',ext.yix)
  END
  
  IF info.ext.tix NE ext.tix THEN BEGIN
     IF NOT exist(setboth) THEN setboth = {tix:ext.tix} $
     ELSE setboth = create_struct(setboth,'TIX',ext.tix)
  END
  
  IF info.ext.offset NE ext.offset THEN BEGIN
     IF NOT exist(setboth) THEN setboth = {offset:ext.offset} $
     ELSE setboth = create_struct(setboth,'OFFSET',ext.offset)
  END
  
  ;; Up to here, cwq_image and cwq_spectr have been equal
  
  IF exist(setboth) THEN BEGIN
     img_set = setboth
     spc_set = setboth
  END
  
     
  IF info.ext.xtvscale NE ext.xtvscale THEN begin
     IF NOT exist(img_set) THEN img_set={xtvscale:ext.xtvscale} $
     ELSE  img_set=create_struct(img_set,'XTVSCALE',ext.xtvscale)
  END
  
  IF info.ext.imzoom NE ext.imzoom THEN BEGIN
     IF NOT exist(img_set) THEN img_set={zoom:ext.imzoom} $
     ELSE  img_set=create_struct(img_set,'ZOOM',ext.imzoom)
  END
  
  IF info.ext.logarithmic NE ext.logarithmic THEN BEGIN
     IF NOT exist(spc_set) THEN spc_set = {logarithmic:ext.logarithmic} $
     ELSE spc_set = create_struct(spc_set,'LOGARITHMIC',ext.logarithmic)
  END
  
  IF info.ext.linefit NE ext.linefit THEN BEGIN
     IF NOT exist(spc_set) THEN spc_set = {FIT:ext.linefit} $
     ELSE spc_set=create_struct(spc_set,'FIT',ext.linefit)
  END
  
  IF info.ext.smooth NE ext.smooth THEN BEGIN
     IF NOT exist(spc_set) THEN spc_set = {smooth:ext.smooth} $
     ELSE spc_set=create_struct(spc_set,'SMOOTH',ext.smooth)
  END
     
  ;; Make changes permanent
  
  info.ext = ext
  
  IF exist(img_set) THEN  $
     WIDGET_CONTROL,info.int.cwqimag,set_value=img_set
  
  IF exist(spc_set) THEN $
     WIDGET_CONTROL,info.int.cwqspec,set_value=spc_set
  
  ;; Store updated info structure 
  WIDGET_CONTROL,store,set_uvalue=info,/no_copy
END

;
; EVENT handling
;
  
FUNCTION cwq_dspwav_event,ev

  storage = WIDGET_INFO(ev.handler,/child)
  WIDGET_CONTROL,storage,get_uvalue=info
  
  ;; What's going on?
  
  WIDGET_CONTROL,ev.id,get_uvalue=uval
  
  ;; It may take some time
  WIDGET_CONTROL,ev.top,/hourglass
  
  ;; Default: no event, no autonomous mode
  evnt = 0
  auto = 0
  
  ;; Things that may change and be reported (or stay the same and still be
  ;; reported):
  
  oldxix = info.ext.xix
  oldyix = info.ext.yix
  old_imzoom = info.ext.imzoom
  
  CASE uval OF 
     
; Change of focus/zoom is a public affair.
     
  "CWQ_IMAGE":BEGIN
     newxix = ev.set.xix
     newyix = ev.set.yix
     new_imzoom = ev.set.zoom
     ENDCASE
     
; Change of window is, however, an internal affair. No event!
     
  "CWQ_WINSEL":BEGIN
     WIDGET_CONTROL,ev.id,set_value=ev.windowi
     set = {windowi:ev.windowi}
     Widget_control,info.int.cwqspec,set_value=set
     WIDGET_CONTROL,info.int.cwqimag,set_value=set
     GOTO,NO_EVENT
     ENDCASE
      
; Change of offset is also an internal affair. No event!
     
  "CWQ_SPECTR":BEGIN
     WIDGET_CONTROL,info.int.cwqspec,set_value={offset:ev.set.offset}
     WIDGET_CONTROL,info.int.cwqimag,set_value={offset:ev.set.offset}
     GOTO,NO_EVENT
     ENDCASE
     
; Change of scaling algorithm is also internal. No event.
     
  "XTVSCALE":BEGIN
     IF ev.select THEN BEGIN
        ;; Private XTVSCALE selected. Make it visible.
        IF info.int.xtvscale2 GT 0 THEN  $
           dummy = xtvscale(info.int.xtvscale2,/map,/show,iconify=0)
        WIDGET_CONTROL,info.int.cwqimag, $
           set_value={xtvscale:info.int.xtvscale2}
     END ELSE BEGIN
        ;; Hide the private XTVSCALE
        IF info.int.xtvscale2 GT 0 THEN  $
           dummy = xtvscale(info.int.xtvscale2,map=0)
        WIDGET_CONTROL,info.int.cwqimag, $
           set_value={xtvscale:info.int.xtvscale}
     END
     GOTO,NO_EVENT
     ENDCASE
  
  END

  
MAKE_EVENT:
  
  set = {CWQ_DSPWAV_SET,$
         xix : newxix,$
         yix : newyix,$
         imzoom : new_imzoom $
        }
  
  old = {CWQ_DSPWAV_SET,$
         xix : oldxix,$
         yix : oldyix,$
         imzoom: old_imzoom $
        }
  
  ;; The original event is always coming from cwq_image, so it's always a
  ;; {CWQ_IMAGE_EVENT} structure.
  
  evnt = {CWQ_DSPWAV_EVENT,$
          ID:ev.handler,$
          TOP:ev.top,$
          HANDLER:0L,$
          SET : set,$
          OLD : old,$
          EVENT:EV}
  
  ;; Autonomous mode?
  auto = info.int.auto
  
  ;; Keeping it all inside the array
  
NO_EVENT:
  
  ;; NO replotting so far, just store info structure
  
  WIDGET_CONTROL,STORAGE,set_uvalue=info,/no_copy
  
  ;; Autonomous mode means always acknowledge any event and
  ;; gobble it up.
  IF auto AND datatype(evnt) EQ 'STC' THEN BEGIN
     WIDGET_CONTROL,ev.handler,set_value=evnt.set
     RETURN,0
  END
  
  RETURN,evnt
END



;
; COMPOUND WIGET CREATION
;


FUNCTION cwq_dspwav,on_base, Q, $
                    uvalue=uvalue, no_copy=no_copy, $
                    cw_mouse_image=cw_mouse_image, $
                    cw_mouse_plot=cw_mouse_plot, $
                    column=column,$
                    $
                    $;; Color scaler(s)
                    $
                    xtvscale=xtvscale, $
                    $
                    $;; Miscellaneous
                    $
                    windowi=windowi,$
                    number=number,$
                    pixels=pixels,$
                    imzoom=imzoom,$
                    autonomous=autonomous
;+  
;
; KEYWORD DEFAULTS/EXPLANATIONS
;
; All keywords not marked with * may be altered after widget creation
; through the
;  
;   WIDGET_CONTROL,CW_ID,SET_VALUE={<KEYWORD_NAME>:<keyword_value>}
;
; mechanism.
;
  
  default,uvalue,'CWQ_DSPWAV'   ;*
  default,no_copy,0             ;* For setting uvalue of this Compound widget
  default,CW_MOUSE_image,-1L    ;* CW mouse control box for image display
  default,CW_MOUSE_plot,-1L     ;* CW mouse control box for spectral plot
  default,column,1
  default,frame,1
  
; Color scaler(s)
  default,xtvscale,-1L          ; Color scaling.
  
; Miscellaneous
  default,number,0
  default,windowi,(number-1) > 0
  default,autonomous,0          ;* Auto-redisplay.
  default,imzoom,1.0
  default,pixels,250
  
; Ignore these ACTIONs:
  default,ignore_action,'(IGNORE)(MOTION)(RELEASE1)(RELEASE2)(RELEASE3)'
;
;-  
  
;;
;; Once more: Check variable types
;;
  keyword = 0
  scalar = 0
  real = typ(/rea)
  natural = typ(/nat)
  strng = typ(/str)
  lng = typ(/lon)
  
; Widgety things
  
  no_copy = KEYWORD_SET(no_copy)
  parcheck,cw_mouse_image,keyword, lng,    scalar,'CW_MOUSE_IMAGE'
  parcheck,cw_mouse_plot, keyword, lng,    scalar,'CW_MOUSE_PLOT'
  column = KEYWORD_SET(column)
  row = column XOR 1
  frame = KEYWORD_SET(frame)
  
; Color scaling:
  parcheck,xtvscale,     keyword,lng,    scalar,'XTVSCALE'
  
; Miscellaneous
  parcheck,number, keyword, natural, scalar,'NUMBER'
  autonomous = KEYWORD_SET(autonomous)
  
; Ignore actions
  parcheck,ignore_action,keyword,strng,  scalar,'IGNORE_ACTION'
  
; Get some initial point values:
  
  dim = gt_dimension(Q)
  XIX = dim.ssolar_x/2
  YIX = dim.ssolar_y/2
  tix = dim.sdel_time/2
  ;; Assume all windows equal, and none more equal than others
  offset = (abs(q.detdesc(0).ixstop(0))+1-q.detdesc(0).ixstart(0))/2
  
  
  IF since_version('4.0') then sml=1 else sml=0
  sml2 = 1
  
  small = {xpad:sml,ypad:sml,space:sml}
  
  base = WIDGET_BASE(on_base, _extra=small, $
                     uvalue=uvalue,no_copy=no_copy, $
                     event_func='cwq_dspwav_event', $
                     pro_set_value='cwq_dspwav_setv', $
                     func_get_value='cwq_dspwav_getv')
  
  ;; Internal bases
  ;; 
  
  ;; Visible bases
  
  intbase = WIDGET_BASE(base,column=column, $
                        row=row,xpad=sml,ypad=sml,space=sml2,frame=frame)
  storage = intbase
  
  ;; Upper/left panel
  
  firstbase = WIDGET_BASE(intbase,/column,_extra=small)
  
  IF since_version('4.0') THEN bsize={scr_xsize:pixels+2,scr_ysize:pixels+2}
  
  IF KEYWORD_SET(row) THEN BEGIN
     imgbase = WIDGET_BASE(firstbase,_extra=bsize)
     selectbase = WIDGET_BASE(firstbase,/column,_extra=small)
  END ELSE BEGIN
     selectbase = WIDGET_BASE(firstbase,/column,_extra=small)
     imgbase = WIDGET_BASE(firstbase,_extra=bsize)
  END
  
  ;; DATA WINDOW SELECTOR
  ;;
  IF pixels LT 200 THEN title = 'Wind: ' ELSE title = 'Window: '
  cwqwins = cwq_winsel(selectbase,q,title=title, $
                       uvalue='CWQ_WINSEL',initial=windowi)
  
  ;; If we're anything else than number zero (the only one), we'll make a
  ;; private XTVSCALE button
  ;;
  IF number EQ 0 THEN BEGIN
     xtvscale2 = -2
  END ELSE BEGIN
     IF since_version('4.0') THEN tight = {scr_ysize:19}
     IF pixels GT 200 THEN intens_text = 'Use private intensity scaling' $
     ELSE                  intens_text = 'Priv. intens. scaling'
     nonex = WIDGET_BASE(selectbase,/nonexclusive,/column,_extra=tight)
     xtvscaleb = WIDGET_BUTTON(nonex,value=intens_text, $
                               uvalue='XTVSCALE')
     
     ;; Private XTVSCALEr
     ;;
     missing = q.detdesc(windowi).missing
     program = [';cds_clean_image,data',$
                'bscale,data,missing='+trim(missing)]
     title = 'Scaling for image display no. '+trim(number)
     xtvscale2 = xtvscale(map=0,group_leader=on_base, missing=missing,$
                          auto_missing=0,program=program,$
                          title=title)
  END
  
  ;; IMAGE DISPLAY
  ;;
  cwqimag = cwq_image(imgbase,q,uvalue='CWQ_IMAGE', $
                      xtvscale=[xtvscale,xtvscale2],$ ;; Two scalers
                      windowi=windowi, offset=offset,$
                      xix=xix,yix=yix,tix=tix,zoom=imzoom,$
                      xsize=pixels,ysize=pixels,$
                      cw_mouse=cw_mouse_image)

  ;; SPECTRAL PLOT
  ;;
  cwqspec = cwq_spectr(intbase,q,uvalue='CWQ_SPECTR', $
                       windowi=windowi, offset=offset,$
                       xix=xix,yix=yix,tix=tix,$
                       xsize=pixels,ysize=pixels,$
                       cw_mouse=cw_mouse_plot)
  
  ;; External values
  ;; These can be set by the user "on the fly"
  
  ext = $
     { $ ;; cwq_dspwav_disp,$
       xtvscale         : xtvscale,$ ; Current Xtvscale_id
       $
       $;; State/action variables
       $
       WINDOWI          : windowi,$
       OFFSET           : offset,$
       XIX              : xix,$
       YIX              : yix,$
       TIX              : tix,$
       imzoom           : imzoom,$
       linefit          : 'NONE', $
       logarithmic      : 0, $
       smooth           : 1 $
     }
     
  ;; Internal values
  ;; These are fixed for the lifetime of the compound widget.
  
  int = { $ ;; cwq_dspwav_int,$
          ql_no  : Q.QL_NO, $
          number : number, $
          auto   : autonomous,$ ; Block events?
          cwqspec : cwqspec,$
          cwqimag : cwqimag,$
          cwqwins : cwqwins,$
          xtvscale: xtvscale,$   ;; External xtvscale
          xtvscale2: xtvscale2}  ;; Local xtvscale (!)
  
  info = $
     { $ ;;cwq_dspwav_info,$
       ext  : ext,$
       int: int}
  
  WIDGET_CONTROL,storage,set_uvalue=info,/no_copy
  
  RETURN,base
END

