;+
; Project     : SOHO - CDS     
;                   
; Name        : CWQ_SPECTR
;               
; Purpose     : QuickLook compound widget for spectral plots.
;               
; Explanation : This compound widget displays what is seen in the bottom/right
;               panel of the data display "columns" of DSPWAV.
;
;               Given a QLDS, the variables WINDOWI, XIX, YIX, and TIX,
;               uniquely define a part of the spectrum that may be extracted
;               from the QLDS. This compound widget displays such spectra, but
;               it needs to be told about the QLDS on creation, and the
;               WINDOWI/XIX/YIX/TIX (all default to zero). When realized, it
;               plots the spectrum with a focus point that may be taken as the
;               OFFSET into the spectral window, for extraction of e.g.,
;               images through gt_iimage().
;
;               The spectrum is displayed by using a CW_PLOTZ compound widget,
;               so the user may zoom in and out of the plot. It is possible to
;               use this compound widget in an "autonomous" mode that will
;               never generate events.
;
;               CHANGING STATUS VARIABLES
;
;               The status variables WINDOWI/XIX/YIX/TIX may be changed
;               through the mechanism
;
;                   WIDGET_CONTROL,CWQ_SPECTR_ID,SET_VALUE={<PARAMETER>:VALUE}
;
;               where CWQ_SPECTR_ID is the widget ID returned by CWQ_SPECTR.
;               Other status variables that can be set are:
;
;                   OFFSET : The index of the focus point in the plot.
;                   FIT    : A string describing the line fit method to
;                            use. Possible values are for the moment 'NONE',
;                            'curvefit', 'lstsqr', 'amoeba_c', 'gaussfit' and
;                            'cds_fit_wave'.
;                   LOGARITHMIC : Setting this to one causes the spectral plot
;                                 to have a logarithmic y axis.
;                   SMOOTH : Uses smooth(spectrum,SMOOTH) on spectra, use to
;                            decrease GIS fixed pattern problems.
;
;               All the status variables are "change sensitive". Modifying one
;               of them means that the display is updated (to any degree
;               necessary) to reflect the changes in the status.
;
;               EVENT HANDLING
;
;               If the widget is not in "autonomous" mode, it will generate
;               events whenever the user selects a new "OFFSET" value (focus
;               point in the plot window). The event structure
;               {CWQ_SPECTR_EVENT} contains the following tags.
;
;                  ID : The CWQ_SPECTR ID.
;                  TOP : The ID of the top of the widget hierarchy.
;                  HANDLER : The ID of the event handler.
;                  SET : A structure {CWQ_SPECTR_SET} with only one tag,
;                        OFFSET.
;                  OLD : A {CWQ_SPECTR_SET} structure with the OLD value of
;                        OFFSET.
;                  EVENT : The original {CW_PLOTZ_EVENT}. See CW_PLOTZ for a
;                          description of this.
;               
;               When such an event is generated, the display is NOT updated
;               until the event is acknowledged by setting the OFFSET value
;               through the WIDGET_CONTROL SET_VALUE mechanism, e.g.,
;
;                  WIDGET_CONTROL,EV.ID,SET_VALUE=EV.SET
;
;               will do. When the widget is run in autonomous mode, this is
;               what is done before gobbling up the event.
;               
; Use         : CW_ID = CWQ_SPECTR(BASE, QLDS [,KEYWORDS])
;    
; Inputs      : BASE : The base to put the compound widget on.
;
;               QLDS : The QLDS from which to take the data to be displayed.
;                      During the active life of the compound widget, it must
;                      be possible to use the QLDS_CHECKIN mechanism to get at
;                      the QLDS.
; 
; Opt. Inputs : None.
;               
; Outputs     : Returns the compound widget ID.
;               
; Opt. Outputs: None.
;               
; Keywords    : UVALUE : The user value to be associated with this widget.
;
;               NO_COPY : Whether to use NO_COPY when setting the UVALUE.
;
;               XSIZE/
;               YSIZE : The size of the widget_draw window in which to plot
;                       the spectral data.
;
;               CW_MOUSE : The widget ID of any CW_MOUSE control box to be
;                          used.
;
;               XIX/YIX/TIX : Specifies the point from which the spectrum to
;                             be displayed is taken.
;
;               WINDOWI : The index of the window to take the spectrum
;                         from.
;
;               OFFSET : The offset into the spectral window of the focus
;                        point in the plot.
;
;               AUTONOMOUS : Set this to make the widget self-acknowledge all
;                            user actions, without sending on events to the
;                            owner.
;
; Calls       : cds_linefit_wrap cwq_spectr_fit cwq_spectr_oplotfit
;               qlds_checkin clipbox cwq_spectr_oplotfit copy_tag_values
;               cwq_spectr_print cwq_spectr_update cwq_spectr_oplotfit
;               cwq_spectr_update cw_plotz() default detdesc() gt_spectrum()
;               handle_create() parcheck since_version() trim() typ()
;               widget_base()
;
; Common      : QLSAVE : Contains the QLDS to take the displayed data from,
;                        checking in with QLDS_CHECKIN.
;               
; Restrictions: Needs to be able to get at the QLDS via QLDS_CHECKIN.
;               
; Side effects: None.
;               
; Category    : QuickLook, Compound_widget, Display
;               
; Prev. Hist. : Based on cw_pzoom/cw_plotz.
;
; Written     : S.V.H.Haugan, UiO, June 1996
;               
; Modified    : Version 2, SVHH, 10 July 1996
;                       Added SMOOTH status variable to remove fixed
;                       patterning.
;               Version 3, SVHH, 15 October 1996
;                       Correcting IDL v 4.0(.1) behaviour.
;                       
; Version     : 3, 15 October 1996
;-            

;;
;; Overplot a line fit straight after the spectrum has been
;; replotted
;; 

PRO cwq_spectr_oplotfit,info
  COMMON qlsave,qlds,saved
  
  IF info.int.fit.type EQ 'NONE' THEN RETURN
  
  ;;
  ;; Fit parameters.
  ;; 
  a = info.int.fit.a
  desc = qlds.detdesc(info.ext.windowi)
  
  x = desc.wavemin + FINDGEN(100)/99*(desc.wavemax-desc.wavemin)
  z = (x-a(1))/a(2)
  gauss = a(0)*exp(-z^2*0.5D) + a(3) + a(4)*x + a(5)*x^2
  
  oplot,x,gauss,linestyle=2
END

;;
;; Make a fit to the spectrum, update display of results.
;;

PRO cwq_spectr_fit,info,x,y
  COMMON qlsave,qlds,saved
  
  windowi = info.ext.windowi
  
  ;;
  ;; Find lab wavelength
  ;;
  labelsep = str_sep(qlds.detdesc(windowi).label,'_')
  
  IF N_ELEMENTS(labelsep) NE 4 THEN  $
     lab_lambda = qlds.detdesc(windowi).wavelength $
  ELSE $
     lab_lambda = FLOAT(labelsep(2)+"."+labelsep(3))
  
  info.int.fit.lab_lambda = lab_lambda
  WIDGET_CONTROL,info.int.fit.labw_id,set_value=trim(info.int.fit.lab_lambda)
  
  
  IF info.int.fit.type EQ 'NONE' THEN BEGIN
     WIDGET_CONTROL,info.int.fit.intens_id,set_value=''
     WIDGET_CONTROL,info.int.fit.fwhm_id,set_value=''
     WIDGET_CONTROL,info.int.fit.velocity_id,set_value=''
     RETURN
  END
  
  ;; Find the good data points
  ;;
  goodix = WHERE(y NE qlds.detdesc(info.ext.windowi).missing,ngood)
  
  ;; No fit with less than 6 points!
  IF ngood LT 6 THEN RETURN
  
  xx = x(goodix)
  yy = y(goodix)
  ;;
  ;; Perform fit.
  ;;
  cds_linefit_wrap,xx,yy,1,info.int.fit.lab_lambda,A,info.int.fit.type
  ;;
  ;; Take grating order into account when calculating velocity
  ;;
  order = 1
  IF lab_lambda LT 0.5d*qlds.detdesc(windowi).wavemax THEN order = 2
  
  detec_lambda = info.int.fit.lab_lambda * order
  
  info.int.fit.a = a
  info.int.fit.fwhm = 2.0 * a(2) * SQRT(2.0*alog(2.0))
  info.int.fit.vel = 3e5*(detec_lambda-a(1))/detec_lambda
  
  IF a(0) GT 1000.0 THEN format = '(I8)' ELSE format = '(g8.3)'
  WIDGET_CONTROL,info.int.fit.intens_id,set_value=trim(a(0),format)
  format = '(g8.3)'
  WIDGET_CONTROL,info.int.fit.fwhm_id,set_value=trim(info.int.fit.fwhm,format)
  WIDGET_CONTROL,info.int.fit.velocity_id,set_value=trim(info.int.fit.vel,$
                                                         format)
END

;
; Update display
;

PRO cwq_spectr_update,info
  COMMON qlsave,qlds,saved
  
  qlds_checkin, info.int.ql_no
  
;
; Grab new data?
;
  newdata = info.int.update
  
  IF newdata THEN BEGIN 
     ;; Grab.
     
     spec = gt_spectrum(qlds,lambda=lam,WINDOW=info.ext.windowi,$
                        xix=info.ext.xix,yix=info.ext.yix,tix=info.ext.tix)
     IF info.ext.smooth NE 1 THEN spec = smooth(spec,info.ext.smooth)
     
     ;; Store data
     
     data = [[lam],[spec]]
     handle_value,info.int.lastdata,data,/set,/no_copy
     
     ;; Make sure we have the correct plot title

     WIDGET_CONTROL,info.int.cwplotz, $
        set_value={title:qlds.detdesc(info.ext.windowi).label}
     
     info.int.update = 0
     info.int.replot = 1
     info.int.refit = 1
  END
  
  IF info.int.refit THEN BEGIN
     
     ;; Make the fit
     handle_value,info.int.lastdata,data,/no_copy
     cwq_spectr_fit,info,data(*,0),data(*,1)
     handle_value,info.int.lastdata,data,/no_copy,/set
     
     ;; Make sure we replot
     info.int.replot = 1
  END
  
  IF info.int.replot THEN BEGIN
     
     ;; Set correct focus point
     
     WIDGET_CONTROL,info.int.cwplotz, $
        set_value={focusi:info.ext.offset}
     
     ;; If we got new data, just tell cw_plotz about it 
     IF newdata THEN BEGIN
        handle_value,info.int.lastdata,data,/no_copy
        WIDGET_CONTROL,info.int.cwplotz,set_value=data
        handle_value,info.int.lastdata,data,/no_copy,/set
     END ELSE BEGIN
        ;; Else just replot the old data
        WIDGET_CONTROL,info.int.cwplotz,set_value={replot:1}
     END
     
     ;; Overplot the line fit
     ;;
     cwq_spectr_oplotfit,info
     info.int.replot = 0
  END
  
END


;; Get info structure and free the data handle
;; Note that the ID refers directly to the storage base
PRO cwq_spectr_clean,id
  WIDGET_CONTROL,id,get_uvalue=info,/no_copy
  IF datatype(info) EQ 'STC' THEN BEGIN
     handle_value,info.int.lastdata,data,/no_copy
     handle_free,info.int.lastdata
  END
END

;;
;; PostScript plot.
;;
;; !P.CLIP should mark an area with approximate height:width ratio 1.5:1
;;
PRO cwq_spectr_print,info
  
   ;; This is the space we're allowed to use:
  clip = !P.clip
  width = !P.clip(2)-!P.clip(0)
  height = !P.clip(3)-!P.clip(1)
  
  unit_ht = height/1.5
  
  ;; Top panel: spectral plot
  
  x0top = clip(0) 
  y0top = clip(3)-unit_ht
  x1top = clip(2)
  y1top = clip(3)
  !P.clip = [x0top,y0top,x1top,y1top]
  clipbox
  
  ;; Widths/heights of the top _panel_
  
  topwidth = x1top-x0top
  topheight = y1top-y0top
  
  ;; Margins:
  
  xleftm = 0.2
  xrightm = 0.05
  
  ytopm = 0.05
  ybotm = 0.10
  
  ;; Center of plotting box
  
  xcenter = x0top + 0.5 * ((1.0 + xleftm - xrightm)*topwidth)
  ycenter = y0top + 0.5 * ((1.0 + ybotm - ytopm)*topheight)
  
  ;; Maximum sizes of plot area
  
  maxwidth = (1.0-xleftm-xrightm)*topwidth
  maxheight = (1.0-ytopm-ybotm)*topheight
  
  ;; Adjust height:width ratio
  
  pheight = maxheight < maxwidth*info.int.ratio
  pwidth = maxwidth < pheight/info.int.ratio
  
  ;; Keep
  charsz = !P.charsize
  
  ;; Renormalized character size
  !P.charsize = 0.7*(0.5*(width+height))/10000.0
  
  ;; Final clip size for cw_plotz
  !P.clip = [xcenter-0.5*pwidth,ycenter-0.5*pheight, $
             xcenter+0.5*pwidth,ycenter+0.5*pheight]
  
  WIDGET_CONTROL,info.int.cwplotz,set_value={replot:1}
  
  ;; Overplot some information here.
  cwq_spectr_oplotfit,info
  
  ;; Bottom panel: Intensity/velocity/fwhm texts (if applicable)
  
  txt = ''
  IF info.int.fit.type NE 'NONE' THEN BEGIN
     txt = ("Fitting method: '" + info.int.fit.type + "'!C!C"+ $
            "Intensity : "+trim(info.int.fit.a(0)) + "!C!C" + $
            "Velocity  : "+trim(info.int.fit.vel) + "!C!C" + $
            "FWHM      : "+trim(info.int.fit.fwhm))
     
  END
  xtx = x0top + xleftm*topwidth
  ytx = y0top - 2*ytopm*topwidth
  IF info.ext.smooth NE 1 THEN $
     txt = txt+"!C!CFixed pattern smoothing:"+trim(info.ext.smooth)+"px"
  xyouts,xtx,ytx,txt,/DEVICE,charsize=!P.charsize*1.2
  ;; Set back charisze
  !P.charsize = charsz
END

;
; SETTING the compound widget value  (status parameters)
;
PRO cwq_spectr_setv,ID,VALUE
  
  IF datatype(value) NE 'STC' THEN RETURN ; No primary value (QL_NO?)
  
  ;; Get storage/info
  
  store = WIDGET_INFO(id,/child)
  WIDGET_CONTROL,store,get_uvalue=info,/no_copy
  
  ;;
  ;; Setting status variables
  ;;
  
  tags = tag_names(value)
  
  IF total(tags EQ "PSPLOT") NE 0 THEN BEGIN
     ;; PRINT! and return!
     cwq_spectr_print,info
     WIDGET_CONTROL,store,set_uvalue=info,/no_copy
     return
  END
  
  ;; Change sensitive stuff
  
  newext = info.ext
  copy_tag_values,newext,value,status
  
  IF newext.windowi NE info.ext.windowi OR $
     newext.xix NE info.ext.xix OR $
     newext.yix NE info.ext.yix OR  $
     newext.tix NE info.ext.tix OR $
     newext.smooth NE info.ext.smooth THEN BEGIN
     info.int.update = 1
  END
  
  IF newext.fit NE info.ext.fit THEN BEGIN
     info.int.refit = 1
     info.int.replot = 1
     info.int.fit.type = newext.fit
  END
  
  IF newext.offset NE info.ext.offset OR $
     newext.fit NE  info.ext.fit THEN BEGIN
     info.int.replot = 1
  END
  
  IF newext.logarithmic NE info.ext.logarithmic THEN BEGIN
     WIDGET_CONTROL,info.int.cwplotz,set_value={ytype:value.logarithmic}
     info.int.replot = 1
  END
  
  info.ext = newext
  
  cwq_spectr_update,info
  
  ;; Store updated info structure 
  WIDGET_CONTROL,store,set_uvalue=info,/no_copy
END

;
; Events can only come from the cw_plotz window.
; We should only pass on events that change the offset (focus)
; 

FUNCTION cwq_spectr_event,ev

  ;; Find storage and info
  STORAGE = WIDGET_INFO(ev.handler,/child)
  WIDGET_CONTROL,STORAGE,get_uvalue=info,/no_copy
  
  WIDGET_CONTROL,ev.id,get_uvalue=action
  evnt = 0
  auto = 0
  oldoffset = info.ext.offset
  
  CASE ACTION OF 
     
  'CW_PLOTZ':BEGIN
     newoffset = ev.set.focusi
     ;; Don't pass on pure rezooming events
     IF newoffset EQ oldoffset THEN BEGIN
        WIDGET_CONTROL,EV.ID,SET_VALUE = EV.SET ; Acknowledge change
        cwq_spectr_oplotfit,info                ; Replot fitted line(s)
        GOTO,NO_EVENT   
     END
     ENDCASE
     
  ELSE:BEGIN
     PRINT,"CWQ_SPECTR_EVENT ACTION: '"+ACTION+"' ??"
     GOTO,NO_EVENT
     ENDCASE
  END
  
MAKE_EVENT:
  
  set = {CWQ_SPECTR_SET,$
         offset : newoffset $
        }
  
  old = {CWQ_SPECTR_SET,$
         offset : oldoffset $
        }
  
  evnt = {CWQ_SPECTR_EVENT,$
          ID:ev.handler,$
          TOP:ev.top,$
          HANDLER:0L,$
          SET : set,$
          OLD : old,$
          EVENT:EV}
  
  ;; Autonomous mode?
  auto = info.int.auto
  
  ;; Keeping it all inside the array
  
NO_EVENT:
  
  ;; Store info structure
  
  WIDGET_CONTROL,STORAGE,set_uvalue=info,/no_copy
  
  ;; Autonomous mode means always acknowledge any event and
  ;; gobble it up.
  IF auto AND datatype(evnt) EQ 'STC' THEN BEGIN
     WIDGET_CONTROL,ev.handler,set_value=evnt.set
     RETURN,0
  END
  
  RETURN,evnt
END


;; This routine is called once, upon realization of the widget
;; At this time, the inner bases (and therefore the cw_plotz 
;; base as well) have been realized. 

PRO cwq_spectr_realize,ID
  
  STORAGE = WIDGET_INFO(ID,/CHILD) 
  WIDGET_CONTROL,STORAGE,get_uvalue = info,/no_copy
  
  IF info.int.fit.type NE 'NONE' THEN BEGIN
     info.int.refit = 1
     cwq_spectr_update,info
  END
  
  ;; Store info structure
  WIDGET_CONTROL,storage,set_uvalue=info,/no_copy
END



;
; COMPOUND WIGET CREATION
;


FUNCTION cwq_spectr,on_base, QLDS, $
                    uvalue=uvalue, no_copy=no_copy, $
                    xsize=xsize,ysize=ysize,$
                    $
                    cw_mouse=cw_mouse, $
                    $
                    xix=xix,yix=yix,tix=tix, windowi=windowi,$
                    offset=offset,$
                    $
                    $;; Miscellaneous
                    $
                    autonomous=autonomous,$
                    smooth=smooth
;+  
;
; KEYWORD DEFAULTS/EXPLANATIONS
;
; All keywords not marked with * may be altered after widget creation
; through the
;  
;   WIDGET_CONTROL,CW_ID,SET_VALUE={<KEYWORD_NAME>:<keyword_value>}
;
; mechanism.
;
  
  default,uvalue,'CWQ_SPECTR'   ;*
  default,no_copy,0             ;* For setting uvalue of this Compound widget
  default,CW_MOUSE,-1L          ;* Compound widget mouse control box. 
  
  default,xsize,200             ;*
  default,ysize,200             ;*
  
  default,xix,0                 ;
  default,yix,0                 ;
  default,tix,0                 ;
  default,windowi,0             ;
  
; Miscellaneous
  default,autonomous,0          ;* Auto-redisplay.
  default,smooth,1              ; Removing fixed patterning
  
;-  
  
;;
;; Once more: Check variable types
;;
  keyword = 0
  scalar = 0
  real = typ(/rea)
  natural = typ(/nat)
  strng = typ(/str)
  lng = typ(/lon)
  
; Widgety things
  
  no_copy = KEYWORD_SET(no_copy)
  parcheck,cw_mouse,     keyword,    lng,    scalar,'CW_MOUSE'
  parcheck,xsize,        keyword,natural,    scalar,'XSIZE'
  parcheck,ysize,        keyword,natural,    scalar,'YSIZE'
  
  
; Miscellaneous
  autonomous = KEYWORD_SET(autonomous)
  
  
  IF since_version('4.0') then sml=1 else sml=0
  small = {xpad:sml,ypad:sml,space:sml}
  
  base = WIDGET_BASE(on_base, _extra=small, $
                     uvalue=uvalue,no_copy=no_copy, $
                     event_func='cwq_spectr_event', $
                     pro_set_value='cwq_spectr_setv', $
                     notify_realize='cwq_spectr_realize')
  
  ;; Internal bases
  ;; 
  storage = WIDGET_BASE(base,map=0,kill_notify='cwq_spectr_clean')
  
  ;; Layout bases
  
  intbase = WIDGET_BASE(base,/column,_extra=small)
  
  val = gt_spectrum(qlds,xix=xix,yix=yix,tix=tix,WINDOW=windowi,lambda=lam)
  IF smooth NE 1 THEN val = smooth(val,smooth) ;; defixpat(val)
  
  ;; Default: point to the middle
  default,offset,N_ELEMENTS(val)/2
  
  ;; SIZES and such
  
  xdsize = xsize-60
  ydsize = ysize-60
  origo = [45,40]
  xticks = -(xdsize/50) < (-2)
  title=qlds.detdesc(windowi).label
  value = [[lam],[val]]
  plotz = cw_plotz(intbase,value = value, uvalue='CW_PLOTZ', $
                   xtitle='Wavelnth',ytitle='Intensity',psym=10,$
                   xwsize=xsize,ywsize=ysize,xticks=xticks,$
                   xdsize=xdsize,ydsize=ydsize,origo=origo,$
                   title=title,focusi=offset,$
                   cw_mouse=cw_mouse)
  
  ;; Line fit section
  fit_method = 'NONE'
  
  labelsep = str_sep(qlds.detdesc(windowi).label,'_')
  IF N_ELEMENTS(labelsep) NE 4 THEN  $
     lab_lambda = qlds.detdesc(windowi).wavelength $
  ELSE $
     lab_lambda = FLOAT(labelsep(2)+"."+labelsep(3))
  
  scry = 14
  
  IF since_version('4.0') THEN BEGIN
     small_label_b = create_struct(small,"scr_ysize",scry)
     align = 1
     leftl = {align_left:align,scr_xsize:2*xsize/3-1,xoffset:0,$
              yoffset:0,scr_ysize:scry}
     rightl = {align_right:align,scr_xsize:xsize/3,xoffset:2*xsize/3,$
               yoffset:0,scr_ysize:scry}
  END ELSE BEGIN
     small_label_b = create_struct(small,"row",1)
  end
  
  aa = string(197b)
  
  rbase = WIDGET_BASE(intbase,_extra=small_label_b)
  dummy = WIDGET_LABEL(rbase,value='Lab wavelength:    ',_extra=leftl)
  labw_id = WIDGET_LABEL(rbase,value=trim(lab_lambda),_extra=rightl)
  
  rbase = WIDGET_BASE(intbase,/row,_extra=small_label_b)
  dummy = WIDGET_LABEL(rbase,value='Peak intensity:    ',_extra=leftl)
  intens_id = WIDGET_LABEL(rbase,value='______',_extra=rightl)
  
  rbase = WIDGET_BASE(intbase,/row,_extra=small_label_b)
  dummy = WIDGET_LABEL(rbase,value='FWHM('+aa+'):           ',_extra=leftl)
  fwhm_id = WIDGET_LABEL(rbase,value='______',_extra=rightl)
  
  rbase = WIDGET_BASE(intbase,/row,_extra=small_label_b)
  dummy = WIDGET_LABEL(rbase,value='Velocity (km/s):   ',_extra=leftl)
  velocity_id = WIDGET_LABEL(rbase,value='______',_extra=rightl)
  
  ;; External values
  ;; These can be set by the user "on the fly"
  
  ext = $
     { $ ;; cwq_spectr_disp,$
       $
       $;; State/action variables
       $
       WINDOWI : windowi, $     ; Setting causes full update
       OFFSET : offset,$        ; Setting causes replot
       XIX : xix, $             ; Setting causes full update
       YIX : yix, $             ; Setting causes full update
       TIX : tix, $             ; Setting causes full update
       FIT : fit_method,$       ; Setting causes refitting+replot
       logarithmic : 0,$        ; Setting causes replot
       smooth : smooth $        ; Use smooth for GIS data?
     }
  
  ;; Internal values
  ;; These are fixed for the lifetime of the compound widget.
  
  labelsep = str_sep(qlds.detdesc(windowi).label,'_')
  IF N_ELEMENTS(labelsep) NE 4 THEN  $
     lab_lambda = qlds.detdesc(windowi).wavelength $
  ELSE $
     lab_lambda = FLOAT(labelsep(2)+"."+labelsep(3))
  
  ;; Line fit information
  ;;
  fit = { type       : fit_method, $
          nlines     : 1, $
          lab_lambda : lab_lambda,$
          a          : dblarr(6), $
          labw_id    : labw_id,$
          intens_id  : intens_id,$
          fwhm_id    : fwhm_id,$
          velocity_id: velocity_id, $ $
          fwhm       : 0.0,$
          vel        : 0.0 $
        }
          
          
  int = $
     { $ ;; cwq_spectr_int,$
       ql_no  : QLDS.QL_NO, $
       cwplotz  : plotz, $
       ratio : FLOAT(ydsize)/FLOAT(xdsize),$
       lastdata : handle_create(),$
       update : 0,$
       replot : 0,$
       refit : 0,$
       fit : fit,$
       auto   : autonomous }    ; Block events?

  
  info = $
     { $ ;;cwq_spectr_info,$
       ext  : ext,$
       int: int}
  
  handle_value,info.int.lastdata,value,/no_copy,/set
  
  WIDGET_CONTROL,storage,set_uvalue=info,/no_copy
  
  RETURN,base
END
