;+
; Project     : SOHO - CDS     
;                   
; Name        : CWQ_WINSEL()
;               
; Purpose     : QL compound widget for selecting a line window
;               
; Explanation : Most Quick Look applications are oriented towards showing data
;               from one specific line extraction window at a time. This
;               compound widget is tailor-made to be used as a selector of the
;               currently displayed window.
;
;               The selector is implemented as a CW_PSELECT pulldown menu with
;               a title that defaults to an empty string.
;
;               It is possible to specify the initial window to be displayed,
;               but windows that haven't been read in by readcdsfits are not
;               selectable. In case the specified initial window is not
;               selectable, the widget finds the first selectable window after
;               the specified window, with possible wrap-around on the list of
;               windows. The INITIAL keyword is thus both an input and an
;               output parameter.
;
;               EVENT HANDLING
;               
;               The compound widget returns event structures of the type
;
;                 {CWQ_WINSEL_EVENT,$
;                    ID :        ID of the compound widget
;                    TOP:        As usual
;                    HANDLER:    As usual
;                    WINDOWI:    The index of the selected window.
;                 }
;
;               The widget appearance is modified automatically when the
;               user selects a window from the pulldown menu.
;
;               SETTING THE CURRENT WINDOW
;
;               This is done by using the WIDGET_CONTROL SET_VALUE mechanism,
;               by setting the value of the compound widget to the index of
;               the desired window. E.g., to select the first window:
;
;                  WIDGET_CONTROL,CWQ_WINSEL_ID,SET_VALUE=0
;
;               where CWQ_WINSEL_ID is the compound widget ID returned by
;               CWQ_WINSEL.
;
; Use         : CW_ID = CWQ_WINSEL(BASE, QLDS [,KEYWORDS])
;    
; Inputs      : BASE : The base to put the compound widget on.
;
;               QLDS : The QLDS that contains the windows to select from.
;
; Opt. Inputs : None.
;               
; Outputs     : Returns the compound widget ID.
;               
; Opt. Outputs: None.
;               
; Keywords    : UVALUE : The user value to be associated with this widget.
;
;               NO_COPY : Whether to use NO_COPY when setting the UVALUE.
;
;               INITIAL : The index of the initially selected window. See the
;                         Explanation section.
;
;               TITLE : The title to appear in front of the window labels on
;                       the root of the pulldown menu.
;
; Calls       : cw_pselect() default detdesc() gt_wlimits() parcheck
;               since_version() trim() typ() widget_base()
;
; Common      : None.
;               
; Restrictions: There must be at least one selectable window.
;               
; Side effects: None.
;               
; Category    : QuickLook, Display
;               
; Prev. Hist. : Based on cw_qltemp
;
; Written     : S.V.H.Haugan, UiO, June 1996
;               
; Modified    : Version 2, SVHH, 16 October 1996
;                       Modified IDL v 4.0(.1) behaviour to save space.
;
; Version     : 2, 16 October 1996
;-            


PRO cwq_winsel_setv,ID,VALUE
  
  IF datatype(value) EQ 'STC' THEN RETURN ;; ONLY primary value
  IF datatype(value) EQ 'STR' THEN RETURN
  
  ;; Get storage/info
  
  store = WIDGET_INFO(id,/child)
  WIDGET_CONTROL,store,get_uvalue=info,/no_copy
  
  info.int.current = FIX(value)
  WIDGET_CONTROL,info.int.psel_id,set_value=trim(value)
  
  ;; Store updated info structure 
  WIDGET_CONTROL,store,set_uvalue=info,/no_copy
END



FUNCTION cwq_winsel_event,ev
  
  ;; Find storage and info
  STORAGE = WIDGET_INFO(ev.handler,/child)
  WIDGET_CONTROL,STORAGE,get_uvalue=info,/no_copy
  
  WIDGET_CONTROL,ev.id,get_uvalue=action
  
  auto = 0
  evnt = 0
  
  number = FIX(action)
  
  evnt = {CWQ_WINSEL_EVENT,$
          ID:ev.handler,$
          TOP:ev.top,$
          HANDLER:0L,$
          windowi:number}
  
  info.int.current = number
  
  ;; Put back updated info
  ;;
  WIDGET_CONTROL,STORAGE,set_uvalue=info,/no_copy
  RETURN,evnt
END


;
; COMPOUND WIGET CREATION
;


FUNCTION cwq_winsel,on_base, QLDS, $
                    uvalue=uvalue, no_copy=no_copy, $
                    initial=initial, $ ;; In/Out
                    title=title

;+  
;
; KEYWORD DEFAULTS/EXPLANATIONS
;
;  The current window may be altered by the
;  
;   WIDGET_CONTROL,CW_ID,SET_VALUE=WINDOW_INDEX
;
; mechanism.
;
  
  default,uvalue,'CWQ_WINSEL'   ;*
  default,no_copy,0             ;* For setting uvalue of this Compound widget
  default,initial,-1            ; Default: Take the first available
  default,title,''
;
;-  
  
;;
;; Once more: Check variable types
;;
  keyword = 0
  scalar = 0
  real = typ(/rea)
  natural = typ(/nat)
  strng = typ(/str)
  lng = typ(/lon)
  
  Nwin = N_ELEMENTS(qlds.detdesc(*))
  
; Widgety things
  
  no_copy = KEYWORD_SET(no_copy)
  
  parcheck,initial,keyword,natural,scalar,'INITIAL',MINVAL=-1,MAXVAL=nwin-1
  parcheck,title,keyword,strng,scalar,'TITLE'
  
  ;;
  ;; What do we have?
  ;;
  names = qlds.detdesc(*).label
  
  ;;
  ;; Which windows are  valid
  ;;
  valid = qlds.detdesc(*).ixstop(0) GE 0
  validix = WHERE(valid,nvalid)
  IF nvalid EQ 0 THEN BEGIN
     MESSAGE,"No selectable windows found"
  END
  
  menu = REPLICATE({PSELECT_S, btext:'', mtext:'', uvalue:'', flags:0}, $
                   Nwin)
  
  wlim = gt_wlimits(qlds,/wave,band=band,/quiet)
  aa = STRING(194B)             ;Angstrom 

  FOR i = 0,Nwin-1 DO BEGIN
     menu(i).btext = names(i)
     menu(i).uvalue = trim(i)
     text = " (Band "+trim(band(i))+")"+ $
        " ["+trim(wlim(0,i))+aa+", "+ trim(wlim(1,i))+aa+"]"
     menu(i).mtext = names(i) + text
  END
  
  IF since_version('4.0') THEN sp = 1 ELSE sp = 0

  base = WIDGET_BASE(on_base, xpad = sp,ypad = sp,space = sp,$
                     uvalue=uvalue,no_copy=no_copy, $
                     event_func='cwq_winsel_event', $
                     pro_set_value='cwq_winsel_setv')
  
  ;;
  ;; Bases for internal use:
  ;;
  
  storage = WIDGET_BASE(base,space=0,xpad=0,ypad=0)
  
  IF initial EQ -1 THEN BEGIN
     initial = validix(0)
  END ELSE BEGIN
     WHILE NOT valid(initial) DO initial = (initial + 1) MOD Nwin
  END
  
  
  PSEL_ID = CW_PSELECT(storage,title,menu,initial=initial)
  
  ;; Desensitize the nonvalid choices
  nonvalix = WHERE(valid - 1b,count) ;; Not valid
  FOR i = 0,count-1 DO BEGIN
     WIDGET_CONTROL,PSEL_ID,set_value = {insensitive:[trim(nonvalix(i))]}
  END
  
  
  ;; External values
  ;; These can be set by the user "on the fly"
  
  ext = $
     { $ ;; cwq_winsel_disp,$
       UPDATE           : 0 $
     }
     
  ;; Internal values
  ;; These are fixed for the lifetime of the compound widget.
  
  int = $
     { $ ;; cwq_winsel_int,$
       ql_no  : QLDS.QL_NO,      $
       PSEL_ID: PSEL_ID,$
       current : initial} 

  
  info = $
     { $ ;;cwq_winsel_info,$
       ext  : ext,$
       int: int}
  
  WIDGET_CONTROL,storage,set_uvalue=info,/no_copy
  
  RETURN,base
END

