;+
; Project     : SOHO - CDS     
;                   
; Name        :  AUXREAD()
;               
; Purpose     : Read block of auxiliary data from a CDS FITS level-1 file.
;               
; Explanation : READCDSCOL is used to read all information
;		about the auxiliary data with 'TTYPEx' == NAME.
;		The data is returned in an IDL structure,
;		with the tags DATA, which contains the actual
;		data, and DESC, which contains the CDS QL Data Format
;		auxiliary data descriptor.
;		If the specified auxiliary data are not found,
;		the tags will have a value of zero.
;
; Use         : AUX = AUXREAD(UNIT,HEADER,NAME)
;    
; Inputs      : UNIT =	the file unit associated with the FITS file.
;
;		HEADER=	the header of the FITS file
;		
;		NAME = 	the type/label name of the desired auxiliary
;			data.
;               
; Opt. Inputs : None.
;               
; Outputs     : Returns structure with auxiliary data and descriptor.
;               
; Opt. Outputs: None.
;               
; Keywords    : STATUS : Set to initialized variable in order to return
;		error message instead of crashing on error. Returns
;		empty string if ok.
;
; Calls       : FXBFIND, READCDSCOL, MKAUXDESC, TRIM
;
; Common      : None.
;               
; Restrictions: The file has to be a CDS FITS level-1 file.
;               
; Side effects: None.
;               
; Category    : Data_Handling, I_O, CDS, FITS, QuickLook
;               
; Prev. Hist. : None.
;
; Written     : Stein Vidar Hagfors Haugan, 27 September 1993
;               
; Modified    : SVHH, Version 2, 8-June-1994
;			Added status=status
;		SVHH, 25 Aug. 1994,
;			Switched READ_CDS-->READCDSCOL
;
; Version     : 2, 8-June-1994
;-            


FUNCTION AuxRead,unit,header,name,status=status
  
  On_Error,2
  IF !DEBUG ne 0 THEN On_Error,0
  
;
; Search for a column with TTYPEx == name
;
  fxbFind,header,'TTYPE',columns,values,ncols
  
  found	= 0
  FOR i=0,ncols-1 DO IF	trim(values(i))	eq name	THEN found=i
  
  IF found eq 0	THEN return,{ Data:0, Desc:0 }
  
  readcdscol,unit,DATA,columns(found),header=status ; Read first
  
  Aux_desc = MkAuxDesc(DATA)			  ; Copy descriptor info
  
  Aux =	 { Data:DATA.Array,		  $	  ; Storage Aux Data
	   Desc:Aux_desc}			  ; Storage Aux Descriptor
  
  return,Aux
END

