;+
; Project     : SOHO - CDS     
;                   
; Name        :  CAL_AUXREAD()
;               
; Purpose     : Read block of auxiliary data from a CDS FITS level-1 file.
;               
; Explanation : READCALCOL is used to read all information
;		about the auxiliary data with 'TTYPEx' == NAME.
;		The data is returned in an IDL structure,
;		with the tags DATA, which contains the actual
;		data, and DESC, which contains the CDS QL Data Format
;		auxiliary data descriptor.
;		If the specified auxiliary data are not found,
;		the tags will have a value of zero.
;
; Use         : AUX = CAL_AUXREAD(UNIT,HEADER,NAME)
;    
; Inputs      : UNIT =	the file unit associated with the FITS file.
;
;		HEADER=	the header of the FITS file
;		
;		NAME = 	the type/label name of the desired auxiliary
;			data.
;               
; Opt. Inputs : None.
;               
; Outputs     : Returns structure with auxiliary data and descriptor.
;               
; Opt. Outputs: None.
;               
; Keywords    : STATUS : Set to initialized variable in order to return
;		error message instead of crashing on error. Returns
;		empty string if ok.
;
; Calls       : FXBFIND, READCDSCOL, CAL_MKAUXDESC(), TRIM
;
; Common      : None.
;               
; Restrictions: The file has to be a CDS FITS level-1 file.
;               
; Side effects: None.
;               
; Category    : Data_Handling, I_O, CDS, FITS, Calibration
;               
; Prev. Hist. : None.
;
; Written     : Stein Vidar Hagfors Haugan, 27 September 1993
;               
; Modified    : SVHH, 24 May 1994, Version 2
;			Added status=status
;		SVHH, 11 June 1994, Version 2.1
;			Shortened name of tags to D and DD, due to
;			EXECUTE string length problem.
;		Version 3, 04-Jun-1998, William Thompson, GSFC
;			Use READCDSCOL instead of READ_CDS
;
; Version     : Version 3, 04-Jun-1998
;-            


FUNCTION Cal_Auxread,unit,header,name,status=status
  
  On_Error,2
  IF !DEBUG ne 0 THEN On_Error,0
  
;
; Search for a column with TTYPEx == name
;
  fxbFind,header,'TTYPE',columns,values,ncols
  
  found	= 0
  FOR i=0,ncols-1 DO IF	trim(values(i))	eq name	THEN found=i
  
  IF found eq 0	THEN return,{ D:0, DD:0	}
  
  readcdscol,unit,DATA,columns(found),header=status ; Read first
  
  Aux_desc = Cal_Mkauxdesc(DATA)		  ; Copy descriptor info
  
  Aux =	 { D:DATA.Array,		  $	  ; Storage Aux Data
	   DD:Aux_desc}				  ; Storage Aux Descriptor
  
  return,Aux
END

