;+
; Project     : SOHO - CDS     
;                   
; Name        : CAL_DETREAD()
;               
; Purpose     : Reads detector data from CDS FITS level-1 file.
;
; Explanation : Based on information from CAL_DETSELECT(), read and store
;		detector data from the file into one contiguous array,
;		QLDS.DetData, with descriptors (indexes & wavelength 
;		information) in QLDS.DetDesc(WndNo)
;               
; Use         : Det = Cal_Detread(UNIT,HEADER)
;    
; Inputs      : UNIT = Device unit associated with the FITS file
;		HEADER = Header (string array) of the FITS file
;		QLDS = The QuickLook data structure to put data into.
;		SELECTION_DATA = Structure from DETSELECT() with
;			information on windows etc.
;               
; Opt. Inputs : None.
;               
; Outputs     : Places the data from the file into the QLDS.
;               
; Opt. Outputs: None.
;               
; Keywords    : STATUS : Set to an initialized, named variable to 
;		return an error message instead of crashing. If
;		everything's ok, a null string will be returned.
;
; Calls       : CAL_MKDETWDESC(), CAL_STORE_WND, FXBDIMEN(), READ_CALFITS
;		TRIM(), WINDOWNO()
;
; Common      : None.
;               
; Restrictions: UNIT/HEADER must refer to a CDS FITS level-1 file.
;		Normally used internally from READCDSFITS.
;               
; Side effects: None.
;               
; Category    : Data_Handling, I_O, CDS, FITS, QuickLook
;               
; Prev. Hist. : None.
;
; Written     : Stein Vidar Hagfors Haugan, 27 September 1993
;               
; Modified    : SVHH, 1 October 1993 -- accomodate possible LONG INTEGER
;					format.
;		SVHH, 26 October 1993
;			Changed FXBPARSETDIM  -> FXBTDIM
;			and     FXBPARSETFORM -> FXBTFORM
;               CDP,  4-Mar-94  Changed reference from TDETY to TDETX in call
;                               to FXBFIND
;		Version 3, William Thompson, GSFC, 4 March 1994
;			Changed to call FXBDIMEN
;		SVHH, 19 May 1994
;			Removed printing of selected window numbers
;		SVHH, 24 May 94 Version 4
;			Added /header, /auto, memlimit=memlimit,
;			status=status.
;			Rewrote the window selection procedures.
;		SVHH, 25 May 94, Version 4.1
;			Cut temp. storage of data array -> less memory used.
;		SVHH, 27 May 94, V 4.2
;			Included /header keyword in call to READ_CALFITS,
;			and ghost DetDesc.ixstart/ixstop values.
;		SVHH, 2 June 94, V 4.3
;			Fixed bug in ghost indexes with F_rows set.
;		SVHH, Version 5, 11-June-1994,
;			Split in two (CAL_DETSELECT + CAL_DETREAD) to minimize
;			memory overhead.
;
; Version     : Version 5, 11 June 1994
;-            

PRO Cal_DetRead,unit,header,QLDS,selection,status=status
  
  On_Error,2
  IF !DEBUG ne 0 THEN On_Error,0
  
  CurrentIx = 0
  
  Det_ColNo = selection.Det_ColNo
  selected = selection.selected
  hdr_only = (selected eq 0)
  Ncols	= N_elements(QLDS.DetDesc)
  Ddims	= N_elements(QLDS.DetDesc(0).ixstart)
  detector = selection.detector
  F_rows = selection.F_rows
  N_rows = selection.N_rows
  
;
; Store all other detectore windows.
;
  FOR WindowNo=0,Ncols-1 DO BEGIN
      read_calfits,unit,Window,Det_ColNo(WindowNo),header=hdr_only(WindowNo) ;,$
						   	; status=status
      IF N_elements(status) gt 0 THEN IF status ne '' THEN return
      QLDS.DetDesc(WindowNo) = Cal_Mkdetwdesc(trim(Detector),Window,Ddims)
      IF selected(WindowNo) THEN cal_store_wnd,WindowNo,CurrentIx,QLDS,Window $
      ELSE BEGIN
	  tdim = fix(fxbDimen(Unit,Det_ColNo(WindowNo)))  ; Mark ghost indexes
	  IF F_rows THEN tdim =	[tdim,N_rows]
	  QLDS.DetDesc(WindowNo).ixstart(*)=0
	  QLDS.DetDesc(WindowNo).ixstop	= tdim - 1
	  QLDS.DetDesc(WindowNo).ixstop(0) = -tdim(0)+1
      END
  END
  
END
