;+
; Project     : SOHO - CDS     
;                   
; Name        : CAL_DETSELECT()
;               
; Purpose     : Select detector data from CDS FITS level-1 file.
;
; Explanation : Figure out how much space will be needed to store all
;		detector data in this file into one contiguous array,
;		Det.DetData, and return information necessary for
;		DETREAD to actually read the data and store them.
;               
; Use         : Det = Cal_Detselect(UNIT,HEADER)
;    
; Inputs      : UNIT = Device unit associated with the FITS file
;		HEADER = Header (string array) of the FITS file
;               
; Opt. Inputs : None.
;               
; Outputs     : Structure with information for DETREAD.
;               
; Opt. Outputs: None.
;               
; Keywords    : SELECT : set to enable selection of some data windows
;		For the windows that are not selected, descriptive 
;		information (DetDesc's) are created, but no detector 
;		data is stored. For windows that are not selected,
;		ixstart will be all zeros, while ixstop will have
;		a negative lambda entry, but by using absolute values,
;		the sizes of the potential data block can be found.
;		Note that, as with selected data, the size of
;		dimension i is now: sz = abs(ixstop(i)) - ixstart(i) + 1
;
;		/HEADER : Set to only read descriptive information
;		about detector data windows -- no data will be loaded
;
;		/AUTO : Set to force automatic, silent truncation of the data
;		by reading just those detector data windows that
;		fit below the memory limit. If the memory limit is exceeded
;		withouth this flag set, the user is prompted for either
;		setting the /HEADER flag or making a manual selection of
;		the data windows.
;
;		MEMLIMIT : Set to the maximum size of memory that
;		the detector data are allowed to occupy.
;
;		STATUS : Set to a initialized, named variable to 
;		return an error message instead of crashing. If
;		everything's ok, a null string will be returned.
;
; Calls       : BELL, CAL_MKDETWDESC(), FXBDIMEN(), FXBFIND, FXBTFORM, FXPAR()
;		READ_CALFITS, TRIM(), YES_NO
;
; Common      : None.
;               
; Restrictions: UNIT/HEADER must refer to a CDS FITS level-1 file.
;		Normally used internally from READCDSFITS.
;               
; Side effects: None.
;               
; Category    : Data_Handling, I_O, CDS, FITS, QuickLook
;               
; Prev. Hist. : Taken out of the old CAL_DETREAD. Split done to minimize
;		memory overhead. Also fixed a bug that was making all data
;		storage LONGs (instead of INTs whenever possible).
;
; Written     : Stein Vidar Hagfors Haugan, 11 June 1994
;               
; Modified    : Never.
;
; Version     : Version 1, 11-June-1994
;-            

FUNCTION Cal_Detselect,unit,header,select=select,auto=auto,header=hdr,$
				 memlimit=memlimit,status=status
  
  On_Error,2
  IF !DEBUG ne 0 THEN On_Error,0
  
  IF N_elements(memlimit) eq 0 THEN memlimit = 15*2L^20	; 15 MB
  
;
; Find all detector data cols.
;
  fxbFind,header,'TDETX',Det_ColNo,values,ncols
  
;
; Return blank if none found.
;
  IF Ncols lt 1	THEN BEGIN
      IF NOT Keyword_SET(auto) THEN BEGIN
	  Bell & Print,'No detector data found!'
      EndIF
      IF N_elements(status) gt 0 THEN status='No detector data found'
      return, 0
  EndIF
  
  
  N_rows = fxpar(header,'NAXIS2')
  F_rows = ( N_rows gt 1 )
  
;
; Get information on all columns in the file
;
  fxbTform,header,tbcol,idltype,format,numval,maxval
  
;
; Find dimensionality
;
  Ndims	= N_elements( fxbDimen(	Unit, Det_ColNo(0) ))
  IF F_rows THEN Ddims = Ndims + 1 ELSE	Ddims=Ndims
  
;
; Do the (possible) selection loop at least once
;
  repflag=1
  
  selected = replicate(1,N_elements(Det_ColNo))	 ; All selected as standard
  
  IF Keyword_SET(hdr) THEN selected(*) = 0	 ; Unless headers specified
  
  WHILE	repflag	DO BEGIN
      ;
      ; Find the size of each selected detector data window, add size in 
      ; wavelength space, find maximum in spatial dimensions.  
      ; Find out whether some data are in long integer format
      ;
      ;
      ; ix(..) stores the size necessary for each dimension
      
      ix = replicate(0,Ddims)		; Basically no space needed
					; but we have Ddims dimensions
      
      longflag = 0
      
      FOR i = 0,Ncols-1	DO IF selected(i) THEN BEGIN
	  IF F_rows THEN ix(Ddims-1)=N_rows
	  tdim = fix( fxbDimen(	Unit, Det_ColNo(i) ))
	  ix(0)	= ix(0)	+ tdim(0)
	  FOR dim=1,Ndims-1 DO BEGIN
	      ix(dim) =	max([ix(dim),tdim(dim)])
	  EndFOR
	  IF idltype(Det_ColNo(i)-1) eq	3 THEN longflag=1
      EndIF
      
      ;
      ; Find total size of data array
      ;
      
      IF longflag THEN datasize	= 4L ELSE datasize=2L
      
      FOR dim=0,Ddims-1	DO BEGIN
	  datasize = datasize*ix(dim)
      END
      
      ; If data > memlimit, set select, and 
      
      repflag =	0
      
      IF NOT Keyword_SET(select) and datasize gt memlimit THEN BEGIN
	  select = 1
	  IF NOT Keyword_SET(auto) THEN	BEGIN
	      Bell
	      Print,"File larger than memory limit"
	      yes_no,"Select detector windows (Y) or headers only (N)",$
			answer,'Y'
	      IF NOT answer THEN BEGIN
		  hdr=1
		  selected(*) =	0
		  select=0
		  repflag=1
	      EndIF
	  EndIF
      EndIF
      
      IF Keyword_SET(select) THEN BEGIN
	  IF Keyword_SET(auto) THEN BEGIN   ; Exclude one window at the time
	      Print,'Excluding one window'
	      selected(0) = 0
	      selected = shift(selected,-1)
	      select = 0
	      repflag =	1
	  END ELSE BEGIN
	      
	      FOR i = 0,Ncols-1	DO BEGIN
		  name = fxPar(header,'TTYPE'+trim(Det_ColNo(i)))
		  yes_no,"Include Detector Window: "+name+'(Y/N)?',answer,'Y'
		  Selected(i) =	answer
	      EndFOR
	      
	      select = 0
	      repflag =	1
	  END
      END
  EndWHILE
  
  
  detector = trim(fxPar(header,'DETECTOR'))
  
;
; Read the first detector window.
;
  hdr_only = intarr(N_elements(selected))
  only = where(selected	eq 0,count)
  IF count gt 0	THEN hdr_only(only) = 1
  
  read_calfits,unit,Window,Det_ColNo(0),/header
  
;
; Make descriptor
;
  DetW_desc = Cal_Mkdetwdesc(trim(detector),Window,Ddims)
  
;
; Return data for CAL_DETREAD.
;
  return, { dimension:ix, $
	    detector:detector,$
	    Det_ColNo:Det_ColNo, $
	    selected:selected, $
	    datasize:datasize, $
	    longflag:longflag, $
	    F_rows:F_rows,$
	    N_rows:N_rows,$
	    DetDesc: replicate(DetW_desc,Ncols)	 }
END
