	PRO OPEN_CALFITS, FILE_NAME, DETECTOR, RASTERHDR, CALHDR,	$
		PATH=DATAPATH
;+
; Project     :	SOHO - CDS
;
; Name        :	OPEN_CALFITS
;
; Purpose     :	Prepares a FITS calibration file for write.
;
; Explanation :	Prepares a FITS binary table file for storing CDS calibration
;		data.  Both a primary and binary table extension FITS header
;		are created, and the structure of the table is defined.  The
;		file is not actually created until the first call to
;		WRITE_CALFITS.  Subsequent calls to WRITE_CALFITS, one for each
;		exposure, write additional data to the file, and CLOSE_CALFITS
;		closes the file.
;
; Use         :	OPEN_CALFITS, FILENAME, DETECTOR, RASTERHDR, CALHDR
;
; Inputs      :	FILENAME  = Name of the file to receive the calibration data.
;			    The default extension is ".fits".
;		DETECTOR  = Either 'NIS' or 'GIS'.
;		RASTERHDR = A structure variable of the form given by either
;			    VDSRASTERDEF or GISRASTERDEF as defined in the
;			    procedure SCI_ANA.
;		CALHDR	  = A special calibration structure containing the
;			    following tags:
;
;				GAS_ID
;				GAS_VOLTAGE
;				GAS_CURRENT
;				GAS_PRESSURE
;
; Opt. Inputs :	None.
;
; Outputs     :	None.
;
; Opt. Outputs:	None.
;
; Keywords    :	PATH	  = Directory to place the output file.  If not passed,
;			    the the file is placed in the user's current
;			    directory.
;
; Calls       :	FXHMAKE, FXADDPAR, DEC2BIN, FXBHMAKE, TRIM, FXBADDCOL, UNSIGN
;
; Common      :	CALFITS_COMMON contains data common to the routines
;		OPEN_CALFITS, WRITE_CALFITS, and CLOSE_CALFITS.  The definition
;		of this common block is in the file "calfits.cmn".
;
; Restrictions:	This routine does not support writing to multiple FITS binary
;		tables simultaneously.
;
;		Binning is not currently handled correctly.
;
; Side effects:	None.
;
; Category    :	Data_handling, I/O, FITS, CDS, Calibration.
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 19 November 1993.
;
; Modified    :	Version 1, William Thompson, GSFC, 19 November 1993.
;		Version 2, William Thompson, GSFC, 14 December 1993.
;			Modified according to comments from Martin Carter as to
;			the meaning and types of the different parameters.
;		Version 3, William Thompson, GSFC, 16 December 1993.
;			Incorporated MCU telemetry formats of slit, OPS, and
;			sun sensor.
;               Version 4, CDP Do a close_calfits if file already open.
;		Version 5, CDP, RAL, 10 February 1994.
;			Changed how exposure count is calculated to reflect
;                       new telmetry format.
;		Version 6, William Thompson, GSFC, 24 February 1994.
;			Removed SLITBITS keyword from header, based on
;			clarification of slit position word in raster header
;			from Martin Carter.  Treatment of slit position word in
;			exposure headers is unchanged.
;               Version 7, CDP, RAL 2-Mar-94
;                       To insert the manually entered Horizontal
;                       and Vertical source positions into the
;                       OPSL/OPSR FITS keywords.
;
; Version     :	Version 7, 2 March 1994.
;-
;
	ON_ERROR, 2
	LOGICAL = ['F','T']
@calfits.cmn
@Posn_Details.com

;
;  Check the number of parameters.
;
	IF N_PARAMS() NE 4 THEN MESSAGE,	$
		'Syntax:  OPEN_CALFITS, FILENAME, DETECTOR, RASTERHDR, CALHDR'
;
;  Store the filename in the common block.
;
	FILENAME = FILE_NAME
;
;  Check to see if an output file is already open.
;
	IF N_ELEMENTS(ALREADY_OPEN) EQ 0 THEN ALREADY_OPEN = 0
;	IF ALREADY_OPEN THEN MESSAGE,	$
;		'Previous output file must be closed before calling OPEN_CALFITS'
	IF ALREADY_OPEN THEN close_calfits
;
;  Store the path for future use.
;
	IF N_ELEMENTS(DATAPATH) EQ 1 THEN PATH = DATAPATH ELSE PATH = ''
;
;  Make a basic FITS primary header, with no primary array, and signaling that
;  there will be an extension to the file.  Add the creation date to the FITS
;  header.
;
	FXHMAKE, HEADER1, /EXTEND, /DATE
;
;  Store information from the raster header structure in the FITS header.
;
	SCIHEAD = RASTERHDR.SCIHEAD
	FXADDPAR, HEADER1, 'FILENAME', FILENAME
	FXADDPAR, HEADER1, 'ORIGIN',   'RAL', 'Rutherford Appleton Laboratory'
	FXADDPAR, HEADER1, 'TELESCOP', 'CALIBRATION',	$
		'Data taken during the CDS calibration'
	FXADDPAR, HEADER1, 'INSTRUME', 'CDS', 'Coronal Diagnostic Spectrometer'
	FXADDPAR, HEADER1, 'DETECTOR', DETECTOR,	$
		'Normal or Grazing Incidence Spectrometer'
	FXADDPAR, HEADER1, 'PROG_ID',  FIX(SCIHEAD.STUDYID),	$
		'Program (Study) ID'
	FXADDPAR, HEADER1, 'PROG_NUM', SCIHEAD.STUDYCOUNTER,	$
		'Program number (Study counter)'
	FXADDPAR, HEADER1, 'SEQ_IND',  SCIHEAD.RASTERCOUNTER, $
		'Sequence (raster) index within program (study)'
	FXADDPAR, HEADER1, 'DATACOMP', FIX(SCIHEAD.DATACOMPSCHEME),	$
		'Telemetry data compression scheme'
	FXADDPAR, HEADER1, 'COMP_OPT', UNSIGN(SCIHEAD.DATACOMPOPTION), $
		'Telemetry data compression option'
	FXADDPAR, HEADER1, 'WNDW_ID',  SCIHEAD.DATAEXTWINDOWID, $
		'Data extraction window list ID'
	FXADDPAR, HEADER1, 'SER_ID',   SCIHEAD.SEQUENCEID,	$
		'On-board sequence load ID'
	FXADDPAR, HEADER1, 'EXPCOUNT', (SCIHEAD.RASTERLENGTHM > 1) * $
                                       (SCIHEAD.RASTERLENGTHS > 1), $
		'Number of exposures in the raster'
	FXADDPAR, HEADER1, 'EXPTIME',  SCIHEAD.EXPOSURETIME/1000., $
		'Exposure time in seconds'
;
;  The following parameters are in MCU telemetry format and have to be decoded
;  before putting into the FITS header.
;  NB the call to ops_decode is retained to keep the translation of the
;  ops bits values but the actual value is replaced by the manually entered
;  horizontal and vertical positions.
;
	OPS_DECODE, SCIHEAD.OPSLCOORDINATE, OPSLBITS, OPSLCOORDINATE
	FXADDPAR, HEADER1, 'OPSLBITS', OPSLBITS,		$
		'OPS L status bits at start of raster'
;	FXADDPAR, HEADER1, 'OPS_L',    OPSLCOORDINATE,		$
;		'OPS L coordinate at start of raster'
	FXADDPAR, HEADER1, 'OPS_L',    cal_hor_posn,		$
		'Source beam Horizontal position'
;
	OPS_DECODE, SCIHEAD.OPSRCOORDINATE, OPSRBITS, OPSRCOORDINATE
	FXADDPAR, HEADER1, 'OPSRBITS', OPSRBITS,		$
		'OPS R status bits at start of raster'
;	FXADDPAR, HEADER1, 'OPS_R',    OPSRCOORDINATE,		$
;		'OPS R coordinate at start of raster'
	FXADDPAR, HEADER1, 'OPS_R',    cal_vert_posn,		$
		'Source beam Vertical position'
;
;  The status bits in the raster header should be zero, so there's no point in
;  writing them to the FITS file.
;
	SLITPOS_DECODE, SCIHEAD.SLITPOSITION, SLITBITS, SLITPOSITION
;	FXADDPAR, HEADER1, 'SLITBITS', SLITBITS,		$
;		'Slit position status bits at start of raster'
	FXADDPAR, HEADER1, 'SLIT_POS', SLITPOSITION,		$
		'Slit position at start of raster'
;
	FXADDPAR, HEADER1, 'SLIT_NUM', FIX(SCIHEAD.SLITNUMBER),	$
		'Slit number'
;
	FXADDPAR, HEADER1, 'MIR_POS',  FIX(SCIHEAD.MIRRORPOSITION), $
		'Mirror position at start of raster'
	FXADDPAR, HEADER1, 'NWINDOWS', FIX(RASTERHDR.NUMDATAWINDOWS),	$
		'Number of data windows'
;
;  Add those keywords derived from the science process status parameter.
;
;  *** Since the format of the science process status word is not known, ***
;  *** this version of the software will simply write out the word as a	 ***
;  *** character string of ones and zeros.				 ***
;
	DEC2BIN, SCIHEAD.SCIPROCSTATUS, SCIPROCSTATUS, /QUIET
;
;	FXADDPAR, HEADER1, 'EV_ENAB',  LOGICAL(SCIPROCSTATUS(0)),	$
;		'Event enabled'
;	FXADDPAR, HEADER1, 'COMP_ERR', LOGICAL(SCIPROCSTATUS(1)),	$
;		'Data compression scheme error'
;	IF DETECTOR EQ 'NIS' THEN BEGIN
;		FXADDPAR, HEADER1, 'VDS_HTR',  LOGICAL(SCIPROCSTATUS(2)), $
;			'VDS high time resolution mode'
;		FXADDPAR, HEADER1, 'VDS_ORI',  LOGICAL(SCIPROCSTATUS(3)), $
;			'VDS telemetry data oriented by columns'
;		FXADDPAR, HEADER1, 'VDS_ACC',  LOGICAL(SCIPROCSTATUS(7)), $
;			'VDS accumulate mode'
;		READOUT = ISHFT(BYTE(SCIHEAD.SCIPROCSTATUS),-4) AND '7'X
;		FXADDPAR, HEADER1, 'VDS_MODE', READOUT, 'VDS camera readout mode'
;	ENDIF
;
	SCIPRST = ''
	FOR I = 0,N_ELEMENTS(SCIPROCSTATUS)-1 DO	$
		SCIPRST = SCIPRST + STRTRIM(FIX(SCIPROCSTATUS(I)),2)
	FXADDPAR, HEADER1, 'SCIPRST',  SCIPRST, 'Science process status word'
;
;  Add the VDS binning factors, if applicable.
;
	IF DETECTOR EQ 'NIS' THEN BEGIN
		BINNING = RASTERHDR.BINFACTORXY
		BINNING = ([ISHFT(BINNING,-4), BINNING] AND 'F'X) + 1
		FXADDPAR, HEADER1, 'VDSBINX', BINNING(0),	$
			'VDS X binning factor'
		FXADDPAR, HEADER1, 'VDSBINY', BINNING(1),	$
			'VDS Y binning factor'
	ENDIF
;
;  Add the calibration-specific data to the header.
;
	FXADDPAR, HEADER1, 'GAS_ID',   CALHDR.GAS_ID,		$
		'Calibration lamp gas ID'
	FXADDPAR, HEADER1, 'GAS_VOLT', CALHDR.GAS_VOLTAGE,	$
		'Calibration lamp gas voltage, in volts'
	FXADDPAR, HEADER1, 'GAS_CURR', CALHDR.GAS_CURRENT,	$
		'Calibration lamp gas current, in amps'
	FXADDPAR, HEADER1, 'GAS_PRES', CALHDR.GAS_PRESSURE,	$
		'Calibration lamp gas pressure, in Torr'
;
;  Use the primary header to create a binary table extension header.  The
;  number of rows is equal to the number of exposures.
;
	HEADER2 = HEADER1
	FXBHMAKE, HEADER2, UNSIGN((SCIHEAD.RASTERLENGTHM > 1) * $
                                  (SCIHEAD.RASTERLENGTHS > 1)), $
                           'CALIBRATION', 'CDS calibration data'
;
;  Create columns for each of the data windows.  Store the column numbers in
;  the array DATAWIN_COL.
;
	N_WINDOWS = RASTERHDR.NUMDATAWINDOWS
	DATAWIN_COL = INTARR(N_WINDOWS)

;
	IF DETECTOR EQ 'NIS' THEN WINDOWS = RASTERHDR.VDSWIN ELSE	$
		WINDOWS = RASTERHDR.GISWIN
;
	FOR I = 0,N_WINDOWS-1 DO BEGIN
		IF DETECTOR EQ 'NIS' THEN BEGIN
			WINDOW = RASTERHDR.VDSWIN(I)
			A = INTARR(WINDOW.LENGTHX,WINDOW.LENGTHY)
			TUNIT = 'ADC'
			TDESC = '(WAVELNTH,SOLAR_Y)
		END ELSE BEGIN
			WINDOW = RASTERHDR.GISWIN(I)
			A = INTARR(WINDOW.LENGTH)
			TUNIT = 'Counts'
			TDESC = 'WAVELNTH'
		ENDELSE
		FXBADDCOL, COL, HEADER2, A, 'WINDOW' + TRIM(I+1),	$
			'Data window', TUNIT=TUNIT, TDESC=TDESC
		IF DETECTOR EQ 'NIS' THEN BEGIN
			FXADDPAR, HEADER2, 'TDETX'+TRIM(COL), WINDOW.STARTX, $
				'Detector start column'
			FXADDPAR, HEADER2, 'TDETY'+TRIM(COL), WINDOW.STARTY, $
				'Detector start row'
		END ELSE BEGIN
			FXADDPAR, HEADER2, 'TDETX'+TRIM(COL), WINDOW.START, $
				'Detector start position'
		ENDELSE
		DATAWIN_COL(I) = COL
	ENDFOR
;
;  Create columns for the data from the exposure headers.
;
	FXBADDCOL, DEL_TIME, HEADER2, 0L, 'DEL_TIME',	$
		'Delta times in milliseconds from start of observation', $
		TUNIT='millisecond'
	FXBADDCOL, OPSL_BITS,HEADER2, 0B, 'OPSLBITS',	$
		'OPS L status bits', TUNIT='code', /BIT
	FXBADDCOL, OPS_L,    HEADER2, 0,  'OPS_L',    'OPS L Coordinate', $
		TUNIT='raw'
	FXBADDCOL, OPSR_BITS,HEADER2, 0B, 'OPSRBITS',	$
		'OPS R status bits', TUNIT='code', /BIT
	FXBADDCOL, OPS_R,    HEADER2, 0,  'OPS_R',    'OPS R Coordinate', $
		TUNIT='raw'
	FXBADDCOL, SLIT_BITS,HEADER2, 0B, 'SLITBITS',	$
		'Slit position status bits', TUNIT='code', /BIT
	FXBADDCOL, SLIT_POS, HEADER2, 0,  'SLIT_POS', 'Slit position',	$
		TUNIT='raw'
	FXBADDCOL, MIR_POS,  HEADER2, 0B, 'MIR_POS',  'Mirror position', $
		TUNIT='raw'
	FXBADDCOL, PITCH_BITS,HEADER2,0B, 'PTCHBITS',	$
		'Sun sensor pitch status bits', TUNIT='code', /BIT
	FXBADDCOL, PITCH,    HEADER2, 0,  'PITCH',    'Sun sensor pitch', $
		TUNIT='raw'
	FXBADDCOL, YAW_BITS, HEADER2, 0B, 'YAW_BITS',	$
		'Sun sensor yaw status bits', TUNIT='code', /BIT
	FXBADDCOL, YAW,      HEADER2, 0,  'YAW',      'Sun sensor yaw', $
		TUNIT='raw'
	FXBADDCOL, EV_LEVEL, HEADER2, 0L, 'EV_LEVEL', 'Event level', $
		TUNIT='code'
	FXBADDCOL, SOLAR_X,  HEADER2, 0,  'SOLAR_X',	$
		'Solar cartesian west (X) pointing', TUNIT='arcsec'
	FXBADDCOL, SOLAR_Y,  HEADER2, 0,  'SOLAR_Y',	$
		'Solar cartesian north (Y) pointing', TUNIT='arcsec'
;
;  Add the columns for the information derived from the event status parameter.
;
	FXBADDCOL, EV_RECOG, HEADER2, 0B, 'EV_RECOG', /LOGICAL,	$
		'Event recognition enabled', TUNIT='logical'
	FXBADDCOL, EV_DETEC, HEADER2, 0B, 'EV_DETEC', /LOGICAL,	$
		'Event detected', TUNIT='logical'
	FXBADDCOL, EV_VALID, HEADER2, 0B, 'EV_VALID', /LOGICAL,	$
		'Event solar coordinates valid', TUNIT='logical'
;
;  Mark the fact that the headers have been defined, and return.
;
	DEFINED = 1
;
	RETURN
	END
