	PRO READ_CALFITS, UNIT, DATA, COLUMN, NANVALUE=NANVALUE,$
		HEADER=HEADER_ONLY, STATUS=STATUS
;+
; Project     :	SOHO - CDS
;
; Name        :	READ_CALFITS
;
; Purpose     :	Reads CDS calibration data from column in FITS binary table.
;
; Explanation :	Reads data stored within a column of a CDS calibration FITS
;		binary table into a structure variable containing the array and
;		associated information.
;
; Use         :	READ_CALFITS, UNIT, DATA, COLUMN
;
;		FXBOPEN, UNIT, 'testcaldata.fits', 1	;Opens the table
;		READ_CALFITS, UNIT, DATA, 'WINDOW1'	;Reads WINDOW1 data
;		FXBCLOSE, UNIT				;Closes the table
;
; Inputs      :	UNIT	= Logical unit number corresponding to the file
;			  containing the binary table.
;
;		COLUMN	= Column in the binary table to read data from, either
;			  as a character string containing a column label
;			  (TTYPE), or as a numerical column index starting from
;			  column one.
;
; Opt. Inputs :	None.
;
; Outputs     :	DATA	= The output data will be read into an anonymous
;			  structure variable with the following tag names:
;
;				ARRAY	= The actual data array.  If /HEADER is
;					  set, then this is simply the number
;					  zero.
;				LABEL	= The column label from TTYPEn.
;				UNITS	= The units of the data.
;				AXES	= The labels for the axes of ARRAY.
;					  The last axis is always labelled
;					  "EXPOSURE".
;
;			  There will also be the following additional tag names
;			  for columns containing detector data windows.
;
;				DETX	= Starting detector column in pixels.
;				DETY	= Starting detector row in pixels (NIS
;					  only).
;
;			  This is a simplified version of the structure used
;			  during operations (see READ_CDS).
;
; Opt. Outputs:	None.
;
; Keywords    :	NANVALUE = Value signalling data dropout.  All points
;			   corresponding to IEEE NaN (not-a-number) are
;			   converted to this number.  Ignored unless DATA is of
;			   type float, double-precision or complex.  The
;			   default is -1.
;
;			   It is anticipated that the CDS data files will
;			   always be stored as integers, so that the use of
;			   this keyword should be unnecessary.
;
;		HEADER	= If set, then only the header is read in.
;
;		STATUS	= If defined and passed, then any error messages will
;			  be returned to the user in this parameter rather than
;			  depending on the MESSAGE routine in IDL.  If no
;			  errors are encountered, then a null string is
;			  returned.  In order to use this feature, STATUS must
;			  be defined first, e.g.
;
;				STATUS = ''
;				READ_CALFITS, STATUS=STATUS, ...
;				IF STATUS NE '' THEN ...
;
; Calls       :	DATATYPE, FXBCOLNUM, FXBHEADER, FXBISOPEN, FXBREAD, FXBTDIM,
;		FXPAR, TRIM
;
; Common      :	None.
;
; Restrictions:	The binary table must have been opened with FXBOPEN.
;
;		This routine is intended solely for CDS calibration data.  It
;		makes certain assumptions about what keywords will be present
;		in the FITS binary table header.
;
; Side effects:	None.
;
; Category    :	Data_Handling, I_O, FITS, CDS, Calibration.
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 15 December 1993.
;
; Modified    :	Version 1, William Thompson, GSFC, 15 December 1993.
;			Based on READ_CDS.
;		Version 2, William Thompson, GSFC, 27 May 1994.
;			Added HEADER keyword.
;		Version 3, William Thompson, GSFC, 21 June 1994
;			Added STATUS keyword.
;               Version 4, C D Pike, RAL, 22 June 1994
;                          Fixed compilation bug on endif (line 170)
;		Version 5, William Thompson, GSFC, 23 June 1994
;			Modified so that STATUS is not touched if not defined.
;
; Version     :	Version 5, 23 June 1994
;-
;
;
	ON_ERROR, 2
;
;  Check the number of parameters.
;
	IF N_PARAMS() LT 3 THEN BEGIN
		MESSAGE = 'Syntax:  READ_CALFITS, UNIT, DATA, COLUMN'
		IF N_ELEMENTS(STATUS) NE 0 THEN BEGIN
			STATUS = MESSAGE
			RETURN
		END ELSE MESSAGE, MESSAGE		
	ENDIF
;
;  Check the logical unit number for the FITS binary table.
;
	IF N_ELEMENTS(UNIT) EQ 0 THEN BEGIN
		MESSAGE = 'UNIT is not defined'
		IF N_ELEMENTS(STATUS) NE 0 THEN BEGIN
			STATUS = MESSAGE
			RETURN
		END ELSE MESSAGE, MESSAGE		
	END ELSE IF NOT FXBISOPEN(UNIT) THEN BEGIN
		MESSAGE = 'Unit ' + TRIM(UNIT) + ' is not open.'
		IF N_ELEMENTS(STATUS) NE 0 THEN BEGIN
			STATUS = MESSAGE
			RETURN
		END ELSE MESSAGE, MESSAGE		
	ENDIF
;
;  Check the column identification, and get the appropriate column number.
;
	IF N_ELEMENTS(COLUMN) EQ 0 THEN BEGIN
		MESSAGE = 'COLUMN is not defined'
		IF N_ELEMENTS(STATUS) NE 0 THEN BEGIN
			STATUS = MESSAGE
			RETURN
		END ELSE MESSAGE, MESSAGE		
	END ELSE IF N_ELEMENTS(COLUMN) GT 1 THEN BEGIN
		MESSAGE = 'COLUMN must be a scalar'
		IF N_ELEMENTS(STATUS) NE 0 THEN BEGIN
			STATUS = MESSAGE
			RETURN
		END ELSE MESSAGE, MESSAGE		
	ENDIF
	IF N_ELEMENTS(STATUS) NE 0 THEN BEGIN
		ICOL = FXBCOLNUM(UNIT,COLUMN,ERRMSG=STATUS)
		IF STATUS EQ '' THEN RETURN
	END ELSE ICOL = FXBCOLNUM(UNIT,COLUMN)
	IF ICOL EQ 0 THEN BEGIN
		MESSAGE = 'COLUMN ' + TRIM(COLUMN) + ' not found.'
		IF N_ELEMENTS(STATUS) NE 0 THEN BEGIN
			STATUS = MESSAGE
			RETURN
		END ELSE MESSAGE, MESSAGE		
	ENDIF
;
;  Read in the data array.
;
	IF N_ELEMENTS(NANVALUE) EQ 1 THEN NAN = NANVALUE ELSE NAN = -1
	IF KEYWORD_SET(HEADER_ONLY) THEN ARRAY = 0 ELSE	BEGIN
		IF N_ELEMENTS(STATUS) NE 0 THEN BEGIN
			FXBREAD, UNIT, ARRAY, ICOL, NANVALUE=NAN, ERRMSG=STATUS
			IF STATUS NE '' THEN RETURN
		END ELSE FXBREAD, UNIT, ARRAY, ICOL, NANVALUE=NAN
	ENDELSE
;
;  Retrieve the binary table header.
;
	HEADER = FXBHEADER(UNIT)
;
;  Extract the associated information for the column in question.
;
	LABEL = TRIM( FXPAR(HEADER, 'TTYPE' + TRIM(ICOL)) )
	UNITS = TRIM( FXPAR(HEADER, 'TUNIT' + TRIM(ICOL)) )
	DESC = FXPAR(HEADER, 'TDESC' + TRIM(ICOL))
	IF !ERR NE -1 THEN DESC = [FXBTDIM(DESC),'EXPOSURE'] ELSE	$
		DESC = 'EXPOSURE'
;
;  Start formatting a command to create the anonymous structure containing the
;  data array and associated data.
;
	COMMAND = 'DATA = {ARRAY: ARRAY, LABEL: LABEL, UNITS: UNITS, ' + $
		'AXES: DESC'
;
;  CDS level-1 data windows are recognized by the presence of a TDETX keyword.
;  If this keyword is present, then include the additional structure elements
;  associated with data windows.
;
	DETX = FXPAR(HEADER, 'TDETX' + TRIM(ICOL), COUNT=COUNT)
	IF COUNT GT 0 THEN BEGIN
		COMMAND = COMMAND + ', DETX: DETX'
		IF TRIM(FXPAR(HEADER,'DETECTOR')) EQ 'NIS' THEN BEGIN
			DETY = FXPAR(HEADER, 'TDETY' + TRIM(ICOL))
			COMMAND = COMMAND + ', DETY: DETY'
		ENDIF
	ENDIF
;
;  Create the anonymous structure and return.
;
	TEST = EXECUTE(COMMAND+'}')
;
	IF N_ELEMENTS(STATUS) NE 0 THEN STATUS = ''
	RETURN
	END
