;+
; Project     : SOHO - CDS     
;                   
; Name        : READCALFITS()
;               
; Purpose     : Read and return the contents of a CDS FITS level-1 file
;               
; Explanation : Reads:
;			- header data (for the whole file), 
;			- fits variables for all detector data windows
;			- detector data 
;               	- fits variables for auxiliary data
;			- auxiliary data
;
;		Returns all data in one IDL structure (the CDS QL
;		data format)
;
; Use         : VAR = READCALFITS( FILENAME )
;    
; Inputs      : FILENAME = the name of the CDS FITS level-1 file.
;               
; Opt. Inputs : None.
;               
; Outputs     : Returns an IDL structure with data from the FITS file
;		(CDS QL data format)
;               
; Opt. Outputs: None.
;               
; Keywords    : /DISPLAY : set this keyword to invoke DSP_INFO
;		to display information about the data in the
;		file. 
;
;		/SELECT : set to prompt for selection of only some
;		of the detector data windows.
;
;		/HEADER : Set to only read descriptive information
;		about detector data windows -- no data will be loaded
;
;		/AUTO : Set to force silent truncation of the data
;		by reading just those detector data windows that
;		fit below the memory limit
;
;		MEMLIMIT : Set to the maximum size of memory that
;		the detector data are allowed to occupy.
;
;		STATUS : Set to a initialized, named variable to 
;		return an error message instead of crashing. If
;		everything's ok, a null string will be returned.
;
;
; Calls       : CAL_AUXREAD(), CAL_HDRREAD(), CAL_DETREAD(), 
;		DATATYPE(), FXBOPEN, FXBCLOSE, QLMGR
;
; Common      : None.
;
; Restrictions: The file MUST be a CDS FITS level-1 file.
;               
; Side effects: Starts QLMGR if not installed.
;               
; Category    : Data_Handling, I_O, FITS, CDS, QuickLook
;               
; Prev. Hist. : None.
;
; Written     : Stein Vidar Hagfors Haugan, 27 September 1993
;               
; Modified    : SVHH, 1 October 1993  - Added keyword inheritance to DSP_INFO.
;		SVHH, 2 October 1993  - QLMGR registration included
;		SVHH, 5 October 1993  - SELECT keyword/functionality included
;               cdp  25-Oct-93  cut out _EXTRA
;		SVHH, 24 May 94 Version 2
;			Added /header, /auto, memlimit=memlimit,
;			status=status
;		SVHH, 28 May 94 V 2.1 -- Implemented ERRMSG in call to FXBOPEN
;		SVHH, 11 June 94, Version 3
;			Split DETREAD into DETSELECT and DETREAD to
;			minimize memory overhead.
;			Also had to shorten structure names due to problems
;			with EXECUTE maximum string length (512 ?).
;               Version 4, SVHH, 28 February 1996
;                       Added HDRTEXT tag to make the QLDS pass
;                       through QLMGR (the standard QLDS has been
;                       somewhat revised since this routine was in use)
;
; Version     : 4, 28 February 1996
;-            


FUNCTION readcalfits,filename,display=display,select=select,status=status,$
				header=hdr,auto=auto,memlimit=memlimit
  
  On_Error,2
  
  IF !DEBUG ne 0 THEN On_Error,0
  
  IF N_elements(memlimit) eq 0 THEN memlimit = 15*2L^20	; 15 MB
  
  fxbOpen,unit,filename,1,header,errmsg=status
  IF N_elements(status)	gt 0 THEN IF status ne '' THEN return,0
  
  selection = cal_DetSelect(unit,header,select=select,header=hdr,$
			  auto=auto,memlimit=memlimit,status=status)
  
  IF N_elements(status)	gt 0 THEN IF status ne '' THEN goto,error_close_exit
  
  IF datatype(selection) ne 'STC' THEN $
	  message,'Failed reading detector data'
  
  DT = cal_AuxRead(unit,header,'DEL_TIME',status=status)
  IF N_elements(status)	gt 0 THEN IF status ne '' THEN goto,error_close_exit
  
  IY = cal_AuxRead(unit,header,'SOLAR_Y',status=status)
  IF N_elements(status)	gt 0 THEN IF status ne '' THEN goto,error_close_exit
  
  IZ = cal_AuxRead(unit,header,'SOLAR_X',status=status)
  IF N_elements(status)	gt 0 THEN IF status ne '' THEN goto,error_close_exit
  
  OX = cal_AuxRead(unit,header,'OPS_L',status=status)
  IF N_elements(status)	gt 0 THEN IF status ne '' THEN goto,error_close_exit
  
  OY = cal_AuxRead(unit,header,'OPS_R',status=status)
  IF N_elements(status)	gt 0 THEN IF status ne '' THEN goto,error_close_exit
  
  MP = cal_AuxRead(unit,header,'MIR_POS',status=status)
  IF N_elements(status)	gt 0 THEN IF status ne '' THEN goto,error_close_exit
  
  SP = cal_AuxRead(unit,header,'SLIT_POS',status=status)
  IF N_elements(status)	gt 0 THEN IF status ne '' THEN goto,error_close_exit
  
  P= cal_auxread(unit,header,'PITCH',status=status)
  IF N_elements(status)	gt 0 THEN IF status ne '' THEN goto,error_close_exit
  
  Y  = cal_auxread(unit,header,'YAW',status=status)
  IF N_elements(status)	gt 0 THEN IF status ne '' THEN goto,error_close_exit
  
  EL=cal_auxread(unit,header,'EV_LEVEL',status=status)
  IF N_elements(status)	gt 0 THEN IF status ne '' THEN goto,error_close_exit
  
  ER=	 cal_auxread(unit,header,'EV_RECOG',status=status)
  IF N_elements(status)	gt 0 THEN IF status ne '' THEN goto,error_close_exit
  
  ED=cal_auxread(unit,header,'EV_DETEC',status=status)
  IF N_elements(status)	gt 0 THEN IF status ne '' THEN goto,error_close_exit
  
  EV=cal_auxread(unit,header,'EV_VALID',status=status)
  IF N_elements(status)	gt 0 THEN IF status ne '' THEN goto,error_close_exit
  
  IF selection.datasize	eq 0 THEN detstring = '0' $
  ELSE IF selection.longflag THEN $
	detstring = "make_array(/long,dim=s.dimension)"	$
  ELSE	detstring = "make_array(/int, dim=s.dimension)"
  
  a = 0
  s = selection
  dummy	= execute( $
      "a={QL_ID:0.0D,QL_NO:0,"+$	    ; Quicklook Manager tags
            "hdrtext:header,"+$        
            "Header:cal_HdrRead(header),"+$
	    "DetData:"+detstring+","+$	    ; Detector data storage
	    "DetDesc:s.DetDesc,"+$	    ; Descriptors of detector windows
	    "DelTdata:DT.D,"+$	    ; Del_Time values
	    "DelTdesc:DT.DD,"+$	    ; Del_Time descriptor
	    "InsYdata:IY.D,"+$	    ; Instr. pointing y-axis
	    "InsYdesc:IY.DD,"+$
	    "InsZdata:IZ.D,"+$	    ; Instr. pointing z-axis
	    "InsZdesc:IZ.DD,"+$
	    "OpsXdata:OX.D,"+$	    ; Offs.Point.Syst. coord X
	    "OpsXdesc:OX.DD,"+$
	    "OpsYdata:OY.D,"+$	    ; Offs.Point.Syst. coord Y
	    "OpsYdesc:OY.DD,"+$
	    "MirPdata:MP.D,"+$	    ; Mirror Position
	    "MirPdesc:MP.DD,"+$
	    "SliPdata:SP.D,"+$	    ; Slit Position
	    "SliPdesc:SP.DD,"+$
	    "PitchData:P.D,"+$
	    "PitchDesc:P.DD,"+$
	    "YawData:Y.D,"+$
	    "YawDesc:Y.DD,"+$
	    "ELevelData:EL.D,"+$
	    "ELevelDesc:EL.DD,"+$
	    "ERecData:ER.D,"+$
	    "ERecDesc:ER.DD,"+$
	    "EDetecData:ED.D,"+$
	    "EDetecDesc:ED.DD,"+$
	    "EValidData:EV.D,"+$
	    "EValidDesc:EV.DD }" )
  
  CAL_DetRead,unit,header,a,selection,status=status	; Read the actual data.
  IF N_elements(status)	gt 0 THEN IF status ne '' THEN goto,error_close_exit
  
  fxbClose,unit
  
  QLMGR,a
  
  IF Keyword_SET(display) THEN dsp_INFO,a; ,_EXTRA=extra
  
  return,a
  
  error_close_exit:
  fxbClose,unit
  return,0
END
