	PRO WRITE_CALFITS, DETECTOR, EXPHDR, EXP_ARRAY
;+
; Project     :	SOHO - CDS
;
; Name        :	WRITE_CALFITS
;
; Purpose     :	Writes data to a FITS calibration file.
;
; Explanation :	Writes an exposure to the FITS calibration file defined by
;		OPEN_CALFITS.  If this is the first exposure, then the file is
;		created and open for write.  Subsequent calls to WRITE_CALFITS,
;		one for each exposure, write additional data to the file, and
;		CLOSE_CALFITS closes the file.
;
; Use         :	WRITE_CALFITS, EXP_NUMBER, DETECTOR, EXPHDR, EXP_ARRAY
;
; Inputs      :	DETECTOR  = Either 'NIS' or 'GIS'.
;		EXPHDR	  = Exposure header structure of the form given by
;			    EXPHDRDEF as defined in the procedure SCI_ANA.
;		EXP_ARRAY = Data array containing the data extraction windows
;			    of the exposure, with the dimensions
;
;				(LENGTHX, LENGTHY, N_WINDOWS)	;NIS
;				(LENGTH, N_WINDOWS)		;GIS
;
; Opt. Inputs :	None.
;
; Outputs     :	None.
;
; Opt. Outputs:	None.
;
; Keywords    :	None.
;
; Calls       :	FXADDPAR, FORM_FILENAME, CONCAT_DIR, FXWRITE, FXBCREATE,
;		FXBWRITE, NINT, DEC2BIN, UNSIGN, OBT2TAI, TAI2UTC
;
; Common      :	CALFITS_COMMON contains data common to the routines
;		OPEN_CALFITS, WRITE_CALFITS, and CLOSE_CALFITS.  The definition
;		of this common block is in the file "calfits.cmn".
;
; Restrictions:	The structure of the output file must already be defined by the
;		routine OPEN_CALFITS.
;
;		Binning is not currently handled correctly.
;
; Side effects:	None.
;
; Category    :	Data_handling, I/O, FITS, CDS, Calibration.
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 19 November 1993.
;
; Modified    :	Version 0.1, William Thompson, GSFC, 19 November 1993.
;		Version 0.2, William Thompson, GSFC, 22 November 1993.
;			Temporarily modified to work with current version of
;			SCI_ANA.
;		Version 0.3, William Thompson, GSFC, 14 December 1993.
;			Modified according to comments from Martin Carter as to
;			the meaning and types of the different parameters.
;		Version 0.4, William Thompson, GSFC, 14 December 1993.
;			Incorporated MCU telemetry formats of slit, OPS, and
;			sun sensor.
;			Removed test code for old telemetry format.
;               Version 0.5, CDP, 17-Jun-95.   Replaced calls to NINT by ROUND
;
; Version     :	Version 0.5, 17-Jun-95.
;-
;
	ON_ERROR, 2
@calfits.cmn
;
;  Check the number of parameters.
;
	IF N_PARAMS() NE 3 THEN MESSAGE,	$
		'Syntax:  WRITE_CALFITS, DETECTOR, EXPHDR, EXP_ARRAY
;
;  Make sure that the structure of the file has been defined.
;
	IF NOT KEYWORD_SET(DEFINED) THEN MESSAGE,	$
		'OPEN_CALFITS must be called before WRITE_CALFITS'
;
;  If the file has not already been opened, then add the start time to the
;  headers.
;
	IF NOT KEYWORD_SET(ALREADY_OPEN) THEN BEGIN
		START_TIME = OBT2TAI(EXPHDR.EXPOSURESTART)
		OBS_DATE = TAI2UTC(START_TIME, /CCSDS)
		FXADDPAR, HEADER1, 'OBS_DATE', OBS_DATE, AFTER='SEQ_IND', $
			'Start date/time of observation'
		FXADDPAR, HEADER2, 'OBS_DATE', OBS_DATE, AFTER='SEQ_IND', $
			'Start date/time of observation'
;
;  Form the name of the output file.
;
		FILE = FORM_FILENAME(FILENAME, '.fits')
		IF PATH NE '' THEN FILE = CONCAT_DIR(PATH,FILE)
;
;  Create the file, write out the primary and binary table headers, and open
;  the binary table for write.
;
		FXWRITE, FILE, HEADER1
		FXBCREATE, UNIT, FILE, HEADER2
		ALREADY_OPEN = 1
	ENDIF
;
;  Extract the exposure number from the exposure header.
;
	I_EXP = EXPHDR.EXPOSURENUMBER + 1
;
;  Write out the data windows.
;
	FOR I = 0,N_ELEMENTS(DATAWIN_COL)-1 DO BEGIN
		IF DETECTOR EQ 'NIS' THEN BEGIN
			N1 = WINDOWS(I).LENGTHX
			N2 = WINDOWS(I).LENGTHY
			A = EXP_ARRAY(0:N1-1,0:N2-1,I)
;
;  The following lines were used during testing to take into account the older
;  SCI_ANA procedure in which the data was reformatted into full detector
;  arrays.  To use these lines, remove the comments, and comment out the above
;  line instead.
;
;			I1 = WINDOWS(I).STARTX
;			I2 = 1024 - WINDOWS(I).STARTY - N2
;			A = EXP_ARRAY(I1:I1+N1-1,I2:I2+N2-1)
;
		END ELSE BEGIN
			N1 = WINDOWS(I).LENGTH
			A = EXP_ARRAY(0:N1-1,I)
;
;  The above comment also applies here.
;
;			I1 = WINDOWS(I).START
;			A = EXP_ARRAY(I1:I1+N1-1)
;
		ENDELSE
		FXBWRITE, UNIT, A, DATAWIN_COL(I), I_EXP
	ENDFOR
;
;  Write out the auxilliary information.
;
	FXBWRITE, UNIT, EXPHDR.SOLARX,			SOLAR_X,  I_EXP
	FXBWRITE, UNIT, EXPHDR.SOLARY,			SOLAR_Y,  I_EXP
	FXBWRITE, UNIT, EXPHDR.MIRRORPOSITION,		MIR_POS,  I_EXP
	FXBWRITE, UNIT, UNSIGN(EXPHDR.EVENTLEVEL),	EV_LEVEL, I_EXP
;
;  Write out the parameters that are in MCU telemetry format.
;
	OPS_DECODE, EXPHDR.OPSLCOORDINATE, BITS, COORDINATE, /MAP
	FXBWRITE, UNIT, BITS,	    OPSL_BITS,	I_EXP
	FXBWRITE, UNIT, COORDINATE, OPS_L,	I_EXP
;
	OPS_DECODE, EXPHDR.OPSRCOORDINATE, BITS, COORDINATE, /MAP
	FXBWRITE, UNIT, BITS,	    OPSR_BITS,	I_EXP
	FXBWRITE, UNIT, COORDINATE, OPS_R,	I_EXP
;
	SLITPOS_DECODE, EXPHDR.SLITPOSITION, BITS, POSITION, /MAP
	FXBWRITE, UNIT, BITS,	    SLIT_BITS,	I_EXP
	FXBWRITE, UNIT, POSITION,   SLIT_POS,	I_EXP
;
	SENSOR_DECODE, EXPHDR.SUNSENSORPITCH, BITS, MAGNITUDE, /MAP
	FXBWRITE, UNIT, BITS,	    PITCH_BITS,	I_EXP
	FXBWRITE, UNIT, MAGNITUDE,  PITCH,	I_EXP
;
	SENSOR_DECODE, EXPHDR.SUNSENSORYAW, BITS, MAGNITUDE, /MAP
	FXBWRITE, UNIT, BITS,	    YAW_BITS,	I_EXP
	FXBWRITE, UNIT, MAGNITUDE,  YAW,	I_EXP
;
;  Write out the delta time.
;
	DELTA_TIME = ROUND(1000*(OBT2TAI(EXPHDR.EXPOSURESTART) - START_TIME))
	FXBWRITE, UNIT, LONG(DELTA_TIME), DEL_TIME, I_EXP
;
;  Write out the information derived from the event status parameter.
;
	DEC2BIN, EXPHDR.EVENTSTATUS, EVENTSTATUS, /QUIET
	FXBWRITE, UNIT, EVENTSTATUS(0), EV_RECOG, I_EXP
	FXBWRITE, UNIT, EVENTSTATUS(1), EV_DETEC, I_EXP
	FXBWRITE, UNIT, EVENTSTATUS(2), EV_VALID, I_EXP
;
	RETURN
	END
