	PRO CDS_IMAGE, FILENAME, DATA, COLUMN, NANVALUE=NANVALUE
;+
; Project     :	SOHO - CDS
;
; Name        :	CDS_IMAGE
;
; Purpose     :	Reads a CDS image from a FITS file (QL or Level-1).
;
; Explanation :	This routine reads in a CDS image from a FITS file.  Two kinds
;		of files are supported: simple FITS files with a single
;		two-dimensional image (quicklook files), or CDS FITS binary
;		table files (level-1).  In the latter case, the routine reads
;		in a data column, and the data is then integrated over the
;		wavelength dimension so that a simple image is returned.
;
;		This routine was deliberately written to "stand alone" as much
;		as possible, so that a minimum of CDS library routines will be
;		needed to support it.
;
; Use         :	CDS_IMAGE, FILENAME, DATA  [, COLUMN ]
;
; Inputs      :	FILENAME = Name of the CDS FITS file.
;
; Opt. Inputs :	COLUMN	 = Column in the binary table to read data from, either
;			   as a character string containing a column label
;			   (TTYPE), or as a numerical column index starting
;			   from column one.  If not passed, the user will be
;			   asked to select from a list.  Ignored if the file is
;			   of basic FITS type with the array in the main part
;			   of the FITS file.
;
; Outputs     :	DATA	 = The output data will be read into an anonymous
;			   structure variable with the following tag names:
;
;				ARRAY	= The actual data array.
;				UNITS	= The units of the data.
;				MISSING	= Value representing missing data.
;				AXES	= The labels for the axes of ARRAY.
;				ORIGIN	= The value of the first point along
;					  each of the axes of ARRAY.
;				SPACING	= The pixel spacing along each axis.
;				ROTATION= The rotation of each axis relative to
;					  the standard coordinate system.  This
;					  is only relevant for spatial
;					  dimensions.
;				WAVELENGTH = Wavelength of observation, in
;					  Angstroms.
;				WAVEMIN	= Minimum wavelength.
;				WAVEMAX	= Maximum wavelength.
;
; Opt. Outputs:	None.
;
; Keywords    :	NANVALUE = Value signalling data dropout.  All points
;			   corresponding to IEEE NaN (not-a-number) are
;			   converted to this number.  Ignored unless DATA is of
;			   type float, double-precision or complex.  If used,
;			   then the MISSING tag in the returned DATA structure
;			   reflects this value.  The default is -1.
;
;			   It is anticipated that the CDS data files will
;			   always be stored as integers, so that the use of
;			   this keyword should be unnecessary.
;
; Calls       :	FXBOPEN, FXBCLOSE, FXBFIND, VALID_NUM, READCDSCOL, REVERSE
;
; Common      :	None.
;
; Restrictions:	This version of the software is restricted to the current
;		(October 93) plan for the CDS FITS binary table files, which
;		are assumed to have the following properties:
;
;			- Either the file is a simple FITS file with a single
;			  data array where the spectral information has already
;			  been removed (quicklook), or
;			- There is only one FITS extension per file, which is
;			  the CDS binary table (level-1), and
;			- There is only one row in the table.
;
; Side effects:	None.
;
; Category    :	Data Handling, I/O, CDS Specific.
;
; Prev. Hist. :	None.  However, part of the logic of this routine was taken
;		from DETREAD by Stein Vidar Hagfors Haugan.
;
; Written     :	William Thompson, GSFC, 26 October 1993.
;
; Modified    :	Version 1, William Thompson, GSFC, 29 October 1993.
;			Added check to make sure column is a data window.
;		Version 2, William Thompson, GSFC, 15 September 1994
;			Modified to call READCDSCOL.
;			Use X-Y coordinate system instead of Y-Z.
;			Added HEADER to the output structure.
;
; Version     :	Version 2, 15 September 1994
;-
;
	ON_ERROR, 2
;
;  Check the number of parameters.
;
	IF N_PARAMS() LT 2 THEN MESSAGE,	$
		'Syntax:  CDS_IMAGE, FILENAME, DATA  [, COLUMN ]'
;
;  Open the file and read the header.  If the number of axes is non-zero, then
;  the data is assumed to be in the main part of the FITS file.  Otherwise, the
;  data will be in a FITS binary table.
;
	OPENR, UNIT, FILENAME, /GET_LUN, /BLOCK
	FXHREAD, UNIT, HEADER
	FREE_LUN, UNIT
	NAXIS = FXPAR(HEADER,'NAXIS')
;
;  If the data is in the main part of the FITS file, then read in the data
;  array, and extract the auxilliary information from the FITS header.
;
	IF NAXIS NE 0 THEN BEGIN
		IF N_ELEMENTS(NANVALUE) EQ 1 THEN NAN = NANVALUE ELSE NAN = -1
		FXREAD, FILENAME, ARRAY, NANVALUE=NAN
;
		LABEL = FXPAR(HEADER,'LABEL')
		UNITS = FXPAR(HEADER,'BUNIT')
		IF FXPAR(HEADER,'BITPIX') LT 0 THEN MISSING = NAN ELSE	$
			MISSING = FXPAR(HEADER,'BLANK')
		AXES     = FXPAR(HEADER,'CTYPE*')
		CRPIX	 = FXPAR(HEADER,'CRPIX*')
		ORIGIN   = FXPAR(HEADER,'CRVAL*')
		SPACING  = FXPAR(HEADER,'CDELT*')
		ORIGIN = ORIGIN + (1 - CRPIX) * SPACING
		ROTATION = FXPAR(HEADER,'CROTA*')
		WAVELENGTH = FXPAR(HEADER,'WAVELNTH')
		WAVEMIN    = FXPAR(HEADER,'WAVEMIN')
		WAVEMAX	   = FXPAR(HEADER,'WAVEMAX')
;
;  Otherwise, open the binary table.  It is assumed that the binary table is
;  the first extension in the file.
;
	END ELSE BEGIN
		FXBOPEN, UNIT, FILENAME, 1, EXTHEADER
;
;  If COLUMN was not passed, then find all data arrays (recognized by having a
;  TDETX keyword) and ask the user which array he/she wishes to read.
;
		IF N_PARAMS() LT 3 THEN BEGIN
			FXBFIND,EXTHEADER,'TDETX',DATA_COL,VALUES,NDATA
			IF NDATA LT 1 THEN MESSAGE, 'No detector data found'
			FXBFIND,EXTHEADER,'TTYPE',COL,TTYPE,NCOL,''
;
;  Print the possibilities with the /MORE option.
;
			OPENW, OUTUNIT, FILEPATH(/TERMINAL), /MORE, /GET_LUN
			FOR I = 0,NDATA-1 DO BEGIN
				PRINTF,OUTUNIT,COL(I),'   ',TTYPE(I)
				IF !ERR EQ 1 THEN GOTO, DONE      ;Entered "Q"
			ENDFOR
DONE:			FREE_LUN, OUTUNIT
;
;  Ask what column to read in.  This can be either a character string or an
;  integer.
;
			COLUMN = ''
			WHILE COLUMN EQ '' DO READ,'Enter column to read: ',COLUMN
			IF VALID_NUM(COLUMN) THEN COLUMN = LONG(COLUMN)
		ENDIF
;
;  Read in the data.  Assume that all the data is in the first row.
;
		READCDSCOL, UNIT, DAT, COLUMN, 1, NANVALUE=NANVALUE
;
;  Make sure that the column is a data window, recognized by having the tag
;  WAVELENGTH.
;
		W = WHERE(TAG_NAMES(DAT) EQ 'WAVELENGTH', COUNT)
		IF COUNT EQ 0 THEN MESSAGE, 'Column ' + STRTRIM(COLUMN,2) + $
			' does not contain a data window'
;
;  Find out which dimension contains the wavelength information.  Assume that
;  all other dimensions are spatial.
;
		I_WAVE = (WHERE(DAT.AXES EQ 'WAVELNTH'))(0)
		SP     =  WHERE(DAT.AXES NE 'WAVELNTH')
;
;  Total the image over the wavelength dimension.
;
		IF I_WAVE NE -1 THEN ARRAY = TOTAL(LONG(DAT.ARRAY), I_WAVE+1)
;
;  Extract the auxilliary data.
;
	        FXBCLOSE, UNIT
		LABEL = DAT.LABEL
		UNITS = DAT.UNITS
		MISSING = DAT.MISSING
		AXES = (DAT.AXES)(SP)
		ORIGIN = (DAT.ORIGIN)(SP)
		SPACING = (DAT.SPACING)(SP)
		ROTATION = (DAT.ROTATION)(SP)
		WAVELENGTH = DAT.WAVELENGTH
		WAVEMIN = DAT.WAVEMIN
		WAVEMAX = DAT.WAVEMAX
	ENDELSE
;
;  If the image needs to be rotated, then rotate it, and update the auxilliary
;  information accordingly.  Only done if the array is two dimensional (which
;  it should be).
;
	SZ = SIZE(ARRAY)
	IF SZ(0) EQ 2 THEN BEGIN
		S1 = SPACING(0)
		S2 = SPACING(1)
		N1 = SZ(1)
		N2 = SZ(2)
		IF AXES(0) EQ 'SOLAR_X' THEN BEGIN
			IF (S1 GE 0) AND (S2 LT 0) THEN BEGIN
				ARRAY = REVERSE(ARRAY,2)
				ORIGIN(1) = ORIGIN(1) + S2*(N2-1)
			END ELSE IF (S1 LT 0) AND (S2 GE 0) THEN BEGIN
				ARRAY = REVERSE(ARRAY,1)
				ORIGIN(0) = ORIGIN(0) + S1*(N1-1)
			END ELSE IF (S1 LT 0) AND (S2 LT 0) THEN BEGIN
				ARRAY = REVERSE(REVERSE(ARRAY,1),2)
				ORIGIN(0) = ORIGIN(0) + S1*(N1-1)
				ORIGIN(1) = ORIGIN(1) + S2*(N2-1)
			ENDIF
			SPACING = ABS(SPACING)
		END ELSE IF AXES(0) EQ 'SOLAR_Y' THEN BEGIN
			IF (S1 GE 0) AND (S2 GE 0) THEN BEGIN
				ARRAY = TRANSPOSE(ARRAY)
			END ELSE IF (S1 LT 0) AND (S2 GE 0) THEN BEGIN
				ARRAY = REVERSE(TRANSPOSE(ARRAY),2)
				ORIGIN(0) = ORIGIN(0) + S1*(N1-1)
			END ELSE IF (S1 GE 0) AND (S2 LT 0) THEN BEGIN
				ARRAY = REVERSE(TRANSPOSE(ARRAY),1)
				ORIGIN(1) = ORIGIN(1) + S2*(N2-1)
			END ELSE IF (S1 LT 0) AND (S2 LT 0) THEN BEGIN
				ARRAY = REVERSE(REVERSE(TRANSPOSE(ARRAY),1),2)
				ORIGIN(0) = ORIGIN(0) + S1*(N1-1)
				ORIGIN(1) = ORIGIN(1) + S2*(N2-1)
			ENDIF
			AXES     = REVERSE(AXES)
			ORIGIN   = REVERSE(ORIGIN)
			SPACING  = REVERSE(ABS(SPACING))
			ROTATION = REVERSE(ROTATION)
		ENDIF
	ENDIF
;
;  Form a new anonymous structure out of the data array and auxilliary
;  information.
;
	COMMAND = 'DATA = {'			+ $
		'ARRAY: ARRAY, '		+ $
		'LABEL: LABEL, '		+ $
		'UNITS: UNITS, '		+ $
		'MISSING: MISSING, '		+ $
		'AXES: AXES, '			+ $
		'ORIGIN: ORIGIN, '		+ $
		'SPACING: SPACING, '		+ $
		'ROTATION: ROTATION, '		+ $
		'WAVELENGTH: WAVELENGTH, '	+ $
		'WAVEMIN: WAVEMIN, '		+ $
		'WAVEMAX: WAVEMAX, '		+ $
		'HEADER: HEADER'
;
;  Create the anonymous structure and return.
;
	TEST = EXECUTE(COMMAND+'}')
;
	RETURN
	END
