;+
; Project     : SOHO - CDS     
;                   
; Name        : CDS_PAD_QLDS
;               
; Purpose     : Make sure all detector data windows have same size.
;               
; Explanation : To ensure that all CDS Quick Look routines may safely
;               assume that all data extraction windows have the same size,
;               this routine forces them to be. The padding is not done with
;               the wavelength dimension, though.
;
;               This routine is normally only called from readcdsfits, as soon
;               as the fits file has been read.
;               
; Use         : CDS_PAD_QLDS,QLDS
;    
; Inputs      : QLDS : A CDS Quick Look Data Structure
;               
; Opt. Inputs : None.
;               
; Outputs     : None. 
;               
; Opt. Outputs: None.
;               
; Keywords    : None.
;
; Calls       : gt_windata(), st_windata.
;
; Common      : None.
;               
; Restrictions: None.
;               
; Side effects: Modifies the contents of the QLDS, pads all data arrays
;               to the size of the largest one, filling with MISSING.
;               
; Category    : Data_handling
;               
; Prev. Hist. : None.
;
; Written     : Stein Vidar H. Haugan, UiO, 18 August 1996
;               
; Modified    : Version 2, SVHH, 23 August 1996
;                       No longer padding the wavelength dimension.
;               Version 3, SVHH, 26 August 1996
;                       Corrected typo sz -> new_sz in calculation of
;                       new ixstop.
;               Version 4, SVHH, 24 September 1996
;                       Added PADDING information.
;               Version 5, SVHH, 4 April 1997
;                       Added test for non-selected windows.
;                       
; Version     : 5, 4 April 1997
;-            


PRO cds_pad_qlds,qlds
  
  ON_ERROR,2
  IF !debug NE 0 THEN ON_ERROR,0
  
  nwin = N_ELEMENTS(qlds.detdesc)
  ndims = N_ELEMENTS(qlds.detdesc(0).ixstop)
  
  ;; Don't touch lambda dimension.
  IF ndims LT 2 THEN RETURN
  
  sizes = abs(qlds.detdesc(*).ixstop)-qlds.detdesc(*).ixstart+1
  
  FOR i = 0,ndims-1 DO sizes(i,0) =  MAX(abs(sizes(i,*)))
  
  sz = REFORM(sizes(1:*,0))
  
  FOR i = 0,nwin-1 DO BEGIN
     this_sz = abs(qlds.detdesc(i).ixstop)-qlds.detdesc(i).ixstart + 1
     this_lambdasz = this_sz(0)
     this_sz = this_sz(1:*)
     IF total(this_sz NE sz) NE 0 AND qlds.detdesc(i).ixstop(0) GE 0 THEN BEGIN
        PRINT,"Window "+trim(i)+" has a different size than others, " + $
           "padding for uniformity"
        IF qlds.detdesc(i).ixstop(0) LT 0 THEN BEGIN
           ;;qlds.detdesc(i).ixstart(0) = 1
           ;;qlds.detdesc(i).ixstop(0) = -sz(0)
           qlds.detdesc(i).ixstop(1:*) = sz(1:*)-1
        END ELSE BEGIN
           missing = qlds.detdesc(i).missing
           new_sz = [this_lambdasz,sz]
           qlds.detdesc(i).padding = new_sz - [this_lambdasz,this_sz]
           array = make_array(/FLOAT,dimension=new_sz,value=missing)
           array(0,0,0,0) = gt_windata(qlds,i,/no_copy)
           qlds.detdesc(i).ixstop = qlds.detdesc(i).ixstart + new_sz - 1
           st_windata,qlds,i,array,/no_copy
        END
     END
  END
END


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; End of 'cds_pad_qlds.pro'.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
