	PRO CDS_SHOW_IMAGE, DATA, WINDOW=WNDW
;+
; Project     :	SOHO - CDS
;
; Name        :	CDS_SHOW_IMAGE
;
; Purpose     :	Displays a CDS image read by CDS_IMAGE.
;
; Explanation :	This routine displays in a window a CDS image read by the
;		routine CDS_IMAGE.  Axes are drawn around the image to signify
;		data coordinates in arcseconds from the center of the Sun.
;
;		Because CDS images are small, the image is shown three times
;		the actual size.
;
;		This routine was deliberately written to "stand alone" as much
;		as possible, so that a minimum of CDS library routines will be
;		needed to support it.
;
; Use         :	CDS_SHOW_IMAGE, DATA
;
; Inputs      :	DATA	= A structure variable containing the CDS image and
;			  auxilliary data.  See CDS_IMAGE for more information.
;
; Opt. Inputs :	None.
;
; Outputs     :	A window is displayed containing the image.
;
; Opt. Outputs:	None.
;
; Keywords    :	WINDOW	= The index of the window used to display the image is
;			  returned in this parameter.
;
; Calls       :	None.
;
; Common      :	None.
;
; Restrictions:	Requires the current graphics device to be a windowing system.
;
; Side effects:	None.
;
; Category    :	Data Handling, I/O, CDS Specific.
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 27 October 1993.
;
; Modified    :	Version 1, William Thompson, GSFC, 29 October 1993.
;			Fixed bug with Y axis origin.
;			Added WINDOW keyword.
;
; Version     :	Version 1, 29 October 1993.
;-
;
	ON_ERROR, 2
;
;  Check the number of parameters.
;
	IF N_PARAMS() NE 1 THEN MESSAGE, 'Syntax:  CDS_SHOW_IMAGE, DATA'
;
;  Check the size of the image.
;
	SZ = SIZE(DATA.ARRAY)
	IF SZ(0) NE 2 THEN MESSAGE, 'Data array is not two dimensional'
;
;  Magnify the data by three.
;
	MAG = 3
	ARRAY = CONGRID(DATA.ARRAY, MAG*SZ(1), MAG*SZ(2))
;
;  Create a window just somewhat bigger than the image.
;
	IF !P.CHARSIZE GT 0 THEN CHARSIZE = !P.CHARSIZE ELSE CHARSIZE = 1
	I1 = !X.MARGIN(0) * CHARSIZE * !D.X_CH_SIZE
	I2 = !X.MARGIN(1) * CHARSIZE * !D.X_CH_SIZE
	J1 = !Y.MARGIN(0) * CHARSIZE * !D.Y_CH_SIZE
	J2 = !Y.MARGIN(1) * CHARSIZE * !D.Y_CH_SIZE
	XSIZE = I1 + MAG*SZ(1) + I2
	YSIZE = J1 + MAG*SZ(2) + J2
	CUR_WINDOW = !D.WINDOW
	WINDOW, /FREE, XSIZE=XSIZE, YSIZE=YSIZE, TITLE=DATA.LABEL
	WNDW = !D.WINDOW
;
;  Display the image at the correct position.
;
	TVSCL, ARRAY, I1, J1
;
;  Put on the axes and labels, but save the current settings of all graphics
;  keywords so that none are disturbed.
;
	P_SAVE = !P
	X_SAVE = !X
	Y_SAVE = !Y
;
	!P.POSITION = [I1/!D.X_SIZE, (I1+MAG*SZ(1)-1)/!D.X_SIZE,	$
		       J1/!D.Y_SIZE, (J1+MAG*SZ(2)-1)/!D.Y_SIZE]
	!X.WINDOW = !P.POSITION(0:1)
	!Y.WINDOW = !P.POSITION(2:3)
	ORIGIN = DATA.ORIGIN
	SX = DATA.SPACING(0)
	SY = DATA.SPACING(1)
	!X.RANGE = [ORIGIN(0) - 0.5*SX, ORIGIN(0)+(SZ(1)-0.5)*SX]
	!Y.RANGE = [ORIGIN(1) - 0.5*SY, ORIGIN(1)+(SZ(2)-0.5)*SY]
	!X.S(1) = MAG / SX  &  !X.S(0) = !P.POSITION(0) - !X.S(1)*!X.RANGE(0)
	!Y.S(1) = MAG / SY  &  !Y.S(0) = !P.POSITION(3) - !Y.S(1)*!Y.RANGE(0)
	AXIS,XAXIS=0,XTITLE=DATA.AXES(0),XSTYLE=1
	AXIS,YAXIS=0,YTITLE=DATA.AXES(1),YSTYLE=1
;
;  Restore the former system parameters, and return to the previous window (if
;  any).
;
	!P = P_SAVE
	!X = X_SAVE
	!Y = Y_SAVE
	IF CUR_WINDOW NE -1 THEN WSET, CUR_WINDOW
;
	RETURN
	END
