	PRO CDS_SIMPLE_FITS, INPUT, CLEAN=CLEAN, BYTE_SCALE=BYTE_SCALE, $
		ERRMSG=ERRMSG
;+
; Project     :	SOHO - CDS
;
; Name        :	CDS_SIMPLE_FITS
;
; Purpose     :	Write out simple 2D FITS files.
;
; Category    :	Class3, FITS
;
; Explanation :	Takes a normal CDS binary table or quicklook structure, and
;		writes out a series of simple two-dimensional FITS image files.
;		This allows the data to be given to users who are unable to use
;		more sophisticated FITS files.
;
; Syntax      :	CDS_SIMPLE_FITS, FILENAME
;		CDS_SIMPLE_FITS, DATA
;
; Examples    :	CDS_SIMPLE_FITS, 's3042r00.fits'
;
;		A = READCDSFITS('s3042r00.fits')
;		CDS_SIMPLE_FITS, A
;
; Inputs      :	FILENAME = Name of CDS FITS file to read in.
;
; Opt. Inputs :	DATA	 = A quicklook data structure as returned by
;			   READCDSFITS.
;
; Outputs     :	The output FITS files are written to the current directory, and
;		have names like "s3042r00_584.fts".
;
; Opt. Outputs:	None.
;
; Keywords    :	CLEAN	   = If set, then CDS_CLEAN_IMAGE is called prior to
;			     writing the FITS files.
;
;		BYTE_SCALE = If set, then the data is scaled into a byte array.
;
;		ERRMSG     = If defined and passed, then any error messages
;			     will be returned to the user in this parameter
;			     rather than depending on the MESSAGE routine in
;			     IDL.  If no errors are encountered, then a null
;			     string is returned.  In order to use this feature,
;			     ERRMSG must be defined first, e.g.
;
;				ERRMSG = ''
;				CDS_SIMPLE_FITS, INPUT, ERRMSG=ERRMSG
;				IF ERRMSG NE '' THEN ...
;
; Calls       :	DATATYPE, READCDSFITS, FXHCLEAN, SXDELPAR, DETDATA, DETDESC,
;		AVERAGE, CDS_CLEAN_IMAGE, BSCALE, FXHMAKE, FXADDPAR,
;		BREAK_FILE, TRIM, FXWRITE
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Prev. Hist. :	None.
;
; History     :	Version 1, 17-Jun-1996, William Thompson, GSFC
;
; Contact     :	WTHOMPSON
;-
;
	ON_ERROR, 2
;
;  Check the number of parameters.
;
	IF N_PARAMS() NE 1 THEN BEGIN
	    MESSAGE = 'Syntax:  CDS_SIMPLE_FITS, FILENAME'
	    GOTO, HANDLE_ERROR
	ENDIF
;
;  If a data structure was passed, then use it.  Otherwise, read the FITS file
;  whose name was passed.
;
	IF DATATYPE(INPUT,1) EQ 'Structure' THEN BEGIN
	    A = INPUT
	END ELSE BEGIN
	    IF DATATYPE(INPUT,1) NE 'String' THEN BEGIN
		MESSAGE = 'INPUT must be either a structure or ' +	$
			'a character string'
		GOTO, HANDLE_ERROR
	    ENDIF
;
	    A = READCDSFITS(INPUT)
	    IF DATATYPE(A,1) NE 'Structure' THEN BEGIN
		MESSAGE ='Unable to read FITS file ' + INPUT
		GOTO, HANDLE_ERROR
	    ENDIF
	ENDELSE
;
;  Form the basic FITS header for the output arrays.
;
	HEADER0 = A.HDRTEXT
	FXHCLEAN, HEADER0
	SXDELPAR, HEADER0, 'EXTNAME'
;
;  Extract the number of windows from the data structure, and remove the
;  keywords associated with each window
;
	N_WINDOWS = N_ELEMENTS(A.DETDESC)
	NAMES = ['TDETX','TDETY','TWAVE','TWMIN','TWMAX','TWBND','TGORD', $
		'TBINX','TBINY']
	FOR I_WINDOW = 1,N_WINDOWS DO SXDELPAR, HEADER0, NAMES + TRIM(I_WINDOW)
;
;  Step through the windows, and extract the window data.
;
	FOR I_WINDOW = 1,N_WINDOWS DO BEGIN
	    AA = DETDATA(A,I_WINDOW)
	    DD = DETDESC(A,I_WINDOW)
;
;  Average over dimensions other than the spatial ones.
;
	    SZ = SIZE(AA)
	    AA = AVERAGE(AA,1,MISSING=DD.MISSING)
	    IF SZ(0) EQ 4 THEN AA = AVERAGE(AA,3,MISSING=DD.MISSING)
;
;  If NIS data, and requested, then remove cosmic rays.
;
	    IF (A.HEADER.DETECTOR EQ 'NIS') AND KEYWORD_SET(CLEAN) $
		    THEN CDS_CLEAN_IMAGE, AA
;
;  If the BYTE_SCALE keyword was set, then scale the image into a byte array.
;
	    IF KEYWORD_SET(BYTE_SCALE) THEN	$
		    BSCALE, AA, MISSING=DD.MISSING, TOP=255
;
;  Form the header for the output array.
;
	    HEADER = HEADER0
	    FXHMAKE, HEADER, AA, /DATE
;
;  Add the parameters CRPIX, CRVAL, etc.
;
	    FXADDPAR, HEADER, 'CTYPE1', 'SOLAR_X',	$
		    'Solar X (cartesian west) axis'
	    FXADDPAR, HEADER, 'CTYPE2', 'SOLAR_Y',	$
		    'Solar Y (cartesian north) axis'
;
	    FXADDPAR, HEADER, 'CUNIT1', 'ARCSEC',	$
		    'Arcseconds from center of sun'
	    FXADDPAR, HEADER, 'CUNIT2', 'ARCSEC',	$
		    'Arcseconds from center of sun'
;
	    FXADDPAR, HEADER, 'CRPIX1', 1, 'Reference pixel along X dimension'
	    FXADDPAR, HEADER, 'CRPIX2', 1, 'Reference pixel along Y dimension'
;
	    FXADDPAR, HEADER, 'CRVAL1', DD.ORIGIN(1),	$
		    'Reference position along X dimension'
	    FXADDPAR, HEADER, 'CRVAL2', DD.ORIGIN(2),	$
		    'Reference position along Y dimension'
;
	    FXADDPAR, HEADER, 'CDELT1', DD.SPACING(1),	$
		    'Increments along X dimension'
	    FXADDPAR, HEADER, 'CDELT2', DD.SPACING(2),	$
		    'Increments along Y dimension'
;
;  Add or modify the wavelength parameters.
;
	    FXADDPAR, HEADER, 'WAVEMIN', DD.WAVEMIN,	$
		    'Minimum wavelength in observation'
	    FXADDPAR, HEADER, 'WAVEMAX', DD.WAVEMAX,	$
		    'Maximum wavelength in observation'
	    FXADDPAR, HEADER, 'WAVELNTH', DD.WAVELENGTH,	$
		    'Principal wavelength in Angstroms', BEFORE='WAVEMIN'
;
;  Form the filename, and change the filename in the header.
;
	    FILENAME = A.HEADER.FILENAME
	    BREAK_FILE, FILENAME, DISK, DIR, NAME, EX
	    WAVE = TRIM(ROUND(DD.WAVELENGTH))
	    FILENAME = NAME + '_' + WAVE
	    FXADDPAR, HEADER, 'FILENAME', FILENAME
;
;  Write out the FITS file.
;
	    FILENAME = FILENAME + '.fts'
	    FXWRITE, FILENAME, HEADER, AA
	ENDFOR
	GOTO, FINISH
;
;  Error handling point.
;
HANDLE_ERROR:
	IF N_ELEMENTS(ERRMSG) NE 0 THEN ERRMSG = 'CDS_SIMPLE_FITS: ' +	$
		MESSAGE ELSE MESSAGE, MESSAGE, /CONTINUE
;
FINISH:
	RETURN
	END
