;+
; Project     :	SOHO - CDS
;
; Name        :	CHECKCDSFITS()
;
; Purpose     :	Check FITS file to insure that it meets the CDS standard.
;
; Explanation :	This procedure makes sure that the FITS file associated with
;		the file FILENAME is a CDS FITS level-1 file with a FITS header
;		that has the proper registered SFDU keywords.
;
; Use         :	RESULT = CHECKCDSFITS( FILENAME [, MESSG] )
;
; Inputs      :	FILENAME:  The FITS file name(s) to be checked (STRING scalar
;			or array).
;
; Opt. Inputs :	None.
;
; Outputs     :	RESULT:	STRING (scalar or array depending on input) is output:
;				'valid' if file meets criterion test,
;				'invalid' if it fails.
;
; Opt. Outputs:	MESSG:	A detailed message describing why the file check 
;			failed (STRING, 1-D array if 1 filename sent, 2-D array
;			(NFile, NError) if array of filenames sent.
;
; Keywords    :	DIR:	The directory where the file(s) in FILENAME can be
;			found.  Note that use of this keyword requires all of
;			the FITS files to be stored in one directory.  Also
;			note that a directory delimiter (i.e., "/" for Unix)
;			must be passed at the end of the directory name.
;
;		VERBOSE: If set, then operational messages will be printed to 
;			the screen.
;
;		ERRMSG:	If defined and passed, then any error messages will 
;			be returned to the user in this parameter rather than
;			being handled by the IDL MESSAGE utility.  If no 
;			errors are encountered, then a null string is 
;			returned.  In order to use this feature, the string 
;			ERRMSG must be defined first, e.g.,
;
;				ERRMSG = ''
;				RES = CHECKCDSFITS( 'data.fits', ERRMSG=ERRMSG)
;				IF ERRMSG(0) NE '' THEN ...
;
; Calls       :	DATATYPE(), FXBCLOSE, FXBFIND, FXBOPEN, FXBSTATE, FXBTDIMEN, 
;		FXBTFORM, VALIDCDSHEAD(), VALIDCDSTYPE
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Category    :	Data Handling, I/O, FITS, SFDU, CDS, Quicklook
;
; Prev. Hist. :	None.
;
; Written     :	Donald G. Luttermoser, GSFC/ARC, 26 October 1995
;
; Modified    :	Version 1, Donald G. Luttermoser, GSFC/ARC, 26 October 1995
;			Initial program.
;
; Version     :	Version 1,  26 October 1995.
;
;-
;
FUNCTION CHECKCDSFITS, FILENAME, MESSG, DIR=DIR, ERRMSG=ERRMSG
;
ON_ERROR, 2 	; Return to caller if error is encountered.
MESSAGE = ''	; Set to non-null string if error is encountered.
NERRMAX = 20	; Maximum number of errors reported.
STIDLTYP = ['Undefined', 'Byte', 'Integer', 'Long Integer', 'Float', $
	'Double', 'Complex', 'String', 'Structure', 'Error in IDLTYPE']
;
; Check the input parameters.
;
RESULT = 'Input error'
IF (N_PARAMS() EQ 0) OR (N_PARAMS() GT 2) THEN BEGIN
	MESSAGE = 'Syntax:  RESULT = CHECKCDSFITS( FILENAME, [MESSG] )'
	GOTO, HANDLE_ERROR
ENDIF
;
IF DATATYPE(FILENAME,1) NE 'String' THEN MESSAGE = 'FILENAME must be a string!'
IF MESSAGE NE '' THEN GOTO, HANDLE_ERROR
;
; Set internal variables.
;
IF N_PARAMS() EQ 2 THEN EXPLAIN = 'Y' ELSE EXPLAIN = 'N'
NF = N_ELEMENTS(FILENAME)
IF KEYWORD_SET(DIR) THEN FILES = DIR + FILENAME ELSE FILES = FILENAME
;
; Definition of column keyword values.
;
VALIDCDSTYPE, TCOLM, AXES, ERRMSG=MESSAGE
TCOLM.TYPE = STRTRIM(TCOLM.TYPE, 2)
TCOLM.UNIT = STRTRIM(TCOLM.UNIT, 2)
;
; Assume the FITS files are valid, we must prove this is not the case. 
;
IF EXPLAIN EQ 'Y' THEN BEGIN
	MESSG = STRARR(NF, NERRMAX)
	MESSG(*, 0) = 'Standard CDS level-1 FITS extended file.'
ENDIF
RESULT = STRARR(NF)
RESULT(*) = 'valid'
;
; Read the CDS FITS files and check the data.
;
FOR I = 0, NF-1 DO BEGIN
	NERR = 0	; Error count.
;
; Check #1:  See if the FITS file exists!
;
	FIL = FINDFILE( FILES(I), COUNT=COUNT )
        IF COUNT EQ 0 THEN BEGIN
		RESULT(I) = 'invalid'
		IF EXPLAIN EQ 'Y' THEN MESSG(I, NERR) = 'File ' + $
			FILES(I) + 'is nonexistent!'
		NERR = (NERR + 1) < (NERRMAX - 1)
		GOTO, NEXT_FILE
	ENDIF
;
	IF KEYWORD_SET(VERBOSE) THEN PRINT, 'Accessing file #' + $
		STRTRIM(STRING(I+1),2) + ':  ' + FILES(I)
;
; Open the binary FITS file.
;
	FXBOPEN, UFITS, FILES(I), 1, EXT_HEAD, ERRMSG=MESSAGE
	IF MESSAGE NE '' THEN BEGIN
		RESULT(I) = 'invalid'
		IF EXPLAIN EQ 'Y' THEN MESSG(I, NERR) = $
			'Error in opening FITS file ' + FILES(I) + $
			' using FXBOPEN.'
		NERR = (NERR + 1) < (NERRMAX - 1)
		GOTO, NEXT_FILE
        ENDIF
;
; Get information from the binary FITS file.
;
	FXBTFORM, EXT_HEAD, TBCOL, IDLTYPE, FORMAT, NUMVAL, MAXVAL, $
		ERRMSG=MESSAGE
	IF MESSAGE NE '' THEN BEGIN
		RESULT(I) = 'invalid'
		IF EXPLAIN EQ 'Y' THEN MESSG(I, NERR) = $
			'Error in retrieving information FITS file ' + $
			FILES(I) + ' using FXBTFORM.'
		NERR = (NERR + 1) < (NERRMAX - 1)
		GOTO, NEXT_FILE
        ENDIF
;
; Check #2:  See if the FITS header keywords are the official registered SFDU
;            keywords.
;
	IF KEYWORD_SET(VERBOSE) THEN PRINT, $
		'  Checking the FITS header keywords ...'
	QHEAD = VALIDCDSHEAD( EXT_HEAD, HEADERR )
	IF QHEAD EQ 'invalid' THEN BEGIN
		TALK = '    -> Error encountered'
		RESULT(I) = 'invalid'
		IF EXPLAIN EQ 'Y' THEN BEGIN
			MESSG(I, NERR) = 'File ' + FILES(I) + $
			' has CDS FITS keyword errors:'
			NHERR = N_ELEMENTS(HEADERR)
			FOR J = 0, NHERR-1 DO BEGIN
				NERR = (NERR + 1) < (NERRMAX - 1)
				MESSG(I, NERR) = '> ' + HEADERR(J)
			ENDFOR
		ENDIF
		NERR = (NERR + 1) < (NERRMAX - 1)
	ENDIF ELSE TALK = '    -> OK'
	IF KEYWORD_SET(VERBOSE) THEN PRINT, TALK
;
; Check #3:  See if the data arrays in the CDS FITS file are the proper size
;            and the proper format.
;
	IF KEYWORD_SET(VERBOSE) THEN PRINT, $
		'  Checking the CDS data dimensions, sizes, and formats ...'
;
; Find various information from the extended column headers.
;
	FXBFIND, UFITS, 'TTYPE', TYPCOLS, TYPVALS, TYPNF
	TYPVALS = STRTRIM(TYPVALS, 2)
	FXBFIND, UFITS, 'TDESC', DESCOLS, DESVALS, DESNF
	DESVALS = STRTRIM(DESVALS, 2)
	FXBFIND, UFITS, 'TUNIT', UNICOLS, UNIVALS, UNINF
	UNIVALS = STRTRIM(UNIVALS, 2)
	NCOL = N_ELEMENTS(IDLTYPE)
;
	FOR J = 0, NCOL-1 DO BEGIN
;
; Get the dimensions of the data in this column.
;
		QDIM = FXBDIMEN(UFITS, J+1)	; Starts at Column #1 (not 0).
		NDIM = N_ELEMENTS(QDIM)
;
; Check for valid TTYPE.
;
		QTYPE = WHERE(TCOLM.TYPE EQ TYPVALS(J))
		QTYPE = QTYPE(0)
; Check for primary window type (TYPE has only partial name) -> stored in the
; first 2 structure indicies.
		IF QTYPE EQ -1 THEN BEGIN
; "WW_" primary spectral data.
			IF STRMID(TCOLM(0).TYPE, 0, 3) EQ $
				STRMID(TYPVALS(J), 0, 3) THEN QTYPE = 0
; "BB_GIS" primary data from the GIS.
			IF STRMID(TCOLM(1).TYPE, 0, 6) EQ $
				STRMID(TYPVALS(J), 0, 6) THEN QTYPE = 0
		ENDIF
; OK, then we couldn't find either one.
		IF QTYPE EQ -1 THEN BEGIN
			RESULT(I) = 'invalid'
			IF EXPLAIN EQ 'Y' THEN MESSG(I, NERR) = TYPVALS(J) + $
				' not a valid column type.'
			NERR = (NERR + 1) < (NERRMAX - 1)
		ENDIF ELSE BEGIN
;
; Check TTYPE vs. IDLTYPE.
;
			IF TCOLM(QTYPE).IDLTYPE NE IDLTYPE(J) THEN BEGIN
				RESULT(I) = 'invalid'
				IF EXPLAIN EQ 'Y' THEN MESSG(I, NERR) = $
				    'Incorrect data type associated with ' + $
				    TYPVALS(J) + ':  ' + $
				    STIDLTYP(IDLTYPE(J) < 9) + '.'
				NERR = (NERR + 1) < (NERRMAX - 1)
			ENDIF
;
; Check TTYPE vs. NDIM.  Note that the dimension of the data array must be 
; within +1 of that stored in TCOLM().NDIM.
;
			CKDIM = NDIM - TCOLM(QTYPE).NDIM
			IF (CKDIM LT 0) OR (CKDIM GT 1) THEN BEGIN
				RESULT(I) = 'invalid'
				IF EXPLAIN EQ 'Y' THEN MESSG(I, NERR) = $
				    'Incorrect number of data dimensions ' + $
				    'associated with ' + TYPVALS(J) + ':  ' + $
				    STRTRIM(STRING(NDIM),2) + '.'
				NERR = (NERR + 1) < (NERRMAX - 1)
			ENDIF
;
; Check TTYPE vs. TUNIT (note that 'Counts' and 'ADC' are identical, only 
; 'Counts' is stored in the TCOLM structure).
;
			IF STRUPCASE(TCOLM(QTYPE).UNIT) NE $
			    STRUPCASE(UNIVALS(J)) THEN BEGIN
			    IF STRUPCASE(UNIVALS(J)) NE 'ADC' THEN BEGIN
				RESULT(I) = 'invalid'
				IF EXPLAIN EQ 'Y' THEN MESSG(I, NERR) = $
				    'Incorrect units associated with ' + $
				    TYPVALS(J) + ':  ' + UNIVALS(J) + '.'
				NERR = (NERR + 1) < (NERRMAX - 1)
			    ENDIF
			ENDIF
;
; Check TTYPE axes units.
;
			CASE NDIM OF
			    1: 
			    2: BEGIN
				IF STRUPCASE(DESVALS(J)) NE AXES(1) THEN BEGIN
				    RESULT(I) = 'invalid'
				    IF EXPLAIN EQ 'Y' THEN MESSG(I, NERR) = $
				    'Incorrect axis labels associated with ' +$
				    TYPVALS(J) + ':  ' + DESVALS(J) + '.'
				    NERR = (NERR + 1) < (NERRMAX - 1)
				ENDIF
			       END
			    3: BEGIN
				IF TCOLM(QTYPE).QWAVE EQ 1 THEN USEAX = $
				    AXES(2) ELSE USEAX = AXES(3)
				IF STRUPCASE(DESVALS(J)) NE USEAX THEN BEGIN
				    RESULT(I) = 'invalid'
				    IF EXPLAIN EQ 'Y' THEN MESSG(I, NERR) = $
				    'Incorrect axis labels associated with ' +$
				    TYPVALS(J) + ':  ' + DESVALS(J) + '.'
				    NERR = (NERR + 1) < (NERRMAX - 1)
				ENDIF
			       END
			    4: BEGIN
				IF TCOLM(QTYPE).QWAVE EQ 1 THEN USEAX = $
				    AXES(4) ELSE USEAX = AXES(5)
				IF STRUPCASE(DESVALS(J)) NE USEAX THEN BEGIN
				    RESULT(I) = 'invalid'
				    IF EXPLAIN EQ 'Y' THEN MESSG(I, NERR) = $
				    'Incorrect axis labels associated with ' +$
				    TYPVALS(J) + ':  ' + DESVALS(J) + '.'
				    NERR = (NERR + 1) < (NERRMAX - 1)
				ENDIF
			       END
			    DEFAULT: BEGIN
				RESULT(I) = 'invalid'
				IF EXPLAIN EQ 'Y' THEN MESSG(I, NERR) = $
				    'Data array dimensions can be no greater'+$
				    ' than 4 for ' + TYPVALS(J)
				    NERR = (NERR + 1) < (NERRMAX - 1)
			       END
			ENDCASE
		ENDELSE
	ENDFOR
;
	IF KEYWORD_SET(VERBOSE) THEN PRINT, ' '
NEXT_FILE:
;
; Close the FITS file.
;
	IF N_ELEMENTS(UFITS) GT 0 THEN BEGIN
		IF FXBSTATE(UFITS) GT 0 THEN FXBCLOSE, UFITS
	ENDIF
ENDFOR
;
; Reformat the MESSG array to eliminate '' entries.
;
IF NERR LT NERRMAX THEN BEGIN
	TMPMESSG = STRARR(NF, NERR)
	FOR I = 0, NF-1 DO TMPMESSG(I, 0:NERR-1) = MESSG(I, 0:NERR-1)
	MESSG = TMPMESSG
ENDIF
;
; Change output arrays to scalars if NF=1.
;
IF NF EQ 1 THEN BEGIN
	RESULT = RESULT(0)
	IF EXPLAIN EQ 'Y' THEN MESSG = REFORM(MESSG)
ENDIF
;
IF MESSAGE NE '' THEN GOTO, HANDLE_ERROR
IF N_ELEMENTS(ERRMSG) GT 0 THEN ERRMSG=MESSAGE
RETURN, RESULT
;
; Error handling portion of the procedure.
;
HANDLE_ERROR:
;
; Change output arrays to scalars if NF=1.
;
IF NF EQ 1 THEN BEGIN
	RESULT = RESULT(0)
	IF EXPLAIN EQ 'Y' THEN MESSG = MESSG(0)
ENDIF
;
IF N_ELEMENTS(ERRMSG) EQ 0 THEN MESSAGE, MESSAGE
ERRMSG = MESSAGE
RETURN, RESULT
;
END
