;+
; Project     : SOHO - CDS     
;                   
; Name        : DETREAD()
;               
; Purpose     : Reads detector data from CDS FITS level-1 file.
;
; Explanation : Based on information from DETSELECT(), read and store
;		detector data from the file into one contiguous array,
;		QLDS.DetData, with descriptors (indexes & wavelength 
;		information) in QLDS.DetDesc(WndNo)
;               
; Use         : DetRead,UNIT,HEADER,QLDS,SELECTION_DATA
;    
; Inputs      : UNIT = Device unit associated with the FITS file
;		HEADER = Header (string array) of the FITS file
;		QLDS = The QuickLook data structure to put data into.
;		SELECTION_DATA = Structure from DETSELECT() with
;			information on windows etc.
;               
; Opt. Inputs : None.
;               
; Outputs     : Places the data from the file into the QLDS.
;               
; Opt. Outputs: None.
;               
; Keywords    : STATUS : Set to an initialized, named variable to 
;		return an error message instead of crashing. If
;		everything's ok, a null string will be returned.
;
; Calls       : FXBDIMEN(), MKDETWDESC(), READCDSCOL, STORE_WND, TRIM()
;		WINDOWNO()
;               
; Common      : None.
;               
; Restrictions: UNIT/HEADER must refer to a CDS FITS level-1 file.
;		Normally used internally from READCDSFITS.
;               
; Side effects: None.
;               
; Category    : Data_Handling, I_O, CDS, FITS, QuickLook
;               
; Prev. Hist. : None.
;
; Written     : Stein Vidar Hagfors Haugan, 27 September 1993
;               
; Modified    : SVHH, 1 October 1993 -- accomodate possible LONG INTEGER
;					format.
;		SVHH, 26 October 1993
;			Changed FXBPARSETDIM  -> FXBTDIM
;			and     FXBPARSETFORM -> FXBTFORM
;               Version 2, William Thompson, GSFC, 4 March 1994
;                       Changed to call FXBDIMEN
;		SVHH, Version 3, 8-June-1994,
;			Included keywords MEMLIMIT etc.. as in CAL_DETREAD()
;		SVHH, Version 4, 11-June-1994,
;			Split in two (DETSELECT + DETREAD) to minimize
;			memory overhead.
;		SVHH, 25 Aug. 1994,
;			Switched READ_CDS --> READCDSCOL
;               Version 5, SVHH, 12 August 1996
;                       Slightly different marker for nonselected windows.
;                       To make ixstop negative even for wavelength-summed
;                       windows, the ixstart/ixstop are shifted.
;                       The formula abs(ixstop(0))-ixstart(0)+1 to get
;                       the size of the windows still works, though.
;
; Version     : Version 5, 12 August 1996
;-            

PRO DetRead,unit,header,QLDS,selection,status=status
  
  
  On_Error,2
  IF !DEBUG ne 0 THEN On_Error,0
  
  CurrentIx = 0
  
  Det_ColNo = selection.Det_ColNo
  selected = selection.selected
  hdr_only = (selected eq 0)
  Ncols	= N_elements(QLDS.DetDesc)
  Ddims	= N_elements(QLDS.DetDesc(0).ixstart)
  detector = selection.detector
  F_rows = selection.F_rows
  N_rows = selection.N_rows
;
; Store all detectore windows.
;
  FOR WindowNo=0,Ncols-1 DO BEGIN
      readcdscol,unit,Window,Det_ColNo(WindowNo),header=hdr_only(WindowNo) ;, $
						  ; status=status
      IF N_elements(status) gt 0 THEN IF status ne '' THEN return
      QLDS.DetDesc(WindowNo) = MkDetWdesc(trim(Detector),Window,Ddims)
      IF selected(WindowNo) THEN Store_Wnd,WindowNo,CurrentIx,QLDS,Window $
      ELSE BEGIN
	  tdim = fix(fxbDimen(Unit,Det_ColNo(WindowNo)))  ; Mark ghost indexes
	  IF F_rows THEN tdim =	[tdim,N_rows]
	  QLDS.DetDesc(WindowNo).ixstart(*)=0
          QLDS.DetDesc(WindowNo).ixstop	= tdim - 1
          QLDS.detdesc(windowno).ixstart(0) = 1
	  QLDS.DetDesc(WindowNo).ixstop(0) = -tdim(0)
      END
  END
  
END


