;+
; Project     : SOHO - CDS     
;                   
; Name        : HDRREAD()
;               
; Purpose     : Read basic header information from CDS FITS level-1 file.
;               
; Explanation : Straightforward. Returns IDL structure with fields for each of
;               the keywords present in the general part of the header in the
;               CDS FITS files.
;               
; Use         : HDR = HDRREAD(HEADER)
;    
; Inputs      : HEADER= The header (array of strings of a CDS FITS
;                       level-1 file.
;               
; Opt. Inputs : None.
;               
; Outputs     : Return value, see above.
;               
; Opt. Outputs: None.
;               
; Keywords    : None.
;
; Calls       : FXPAR, TRIM()
;
; Common      : None.
;               
; Restrictions: See HEADER above.
;               
; Side effects: None.
;               
; Category    : Data_Handling, I_O, CDS, FITS, QuickLook
;               
; Prev. Hist. : None.
;
; Written     : Stein Vidar Hagfors Haugan, 27 September 1993
;               
; Modified    : SVHH, Version 2, 8-June-94  Fixed types for all tags
;
;               V 3 Changed exposure time to real (seconds).  CDP, 11-Jul-95
;
;               SVHH, Version 4, 12-Sept-95. All tag names are identical to
;               the keyword names. Added lots of tags, and sorted them to make
;               comparison with new fits files easy.
;
;               Version 5, SVHH, 15-January-1996. Removed tag LL_TITLE.
;
;               Version 6, SVHH, 22-February-1996. Added tag GSET_ID
;               Version 7, SVHH, 4 August 1997.
;                       Renamed/added tags to match FITS headers
;
; Version     : Version 7, 4 August 1997
;-            


FUNCTION HdrRead,hdr

  return,{HDRX,$
          angle:    double(fxpar(hdr,"ANGLE")), $
	  category: trim(fxpar(hdr,  "CATEGORY")), $
          cmp_name: trim(fxpar(hdr,  "CMP_NAME")), $
          cmp_no:   long(fxpar(hdr,  "CMP_NO")), $
	  comp_err: byte(fxpar(hdr,  "COMP_ERR")), $
	  comp_id:  long(fxpar(hdr,  "COMP_ID")), $
          comp_opt: long(fxpar(hdr,  "COMP_OPT")), $
          datasrc:  trim(fxpar(hdr,  "DATSRC")),$
	  date:     trim(fxpar(hdr,  "DATE")), $
          date_end: trim(fxpar(hdr,  "DATE_END")), $
          date_obs: trim(fxpar(hdr,  "DATE_OBS")), $
          detector: trim(fxpar(hdr,  "DETECTOR")), $
	  dw_id:    long(fxpar(hdr,  "DW_ID")), $
	  ev_enab:  byte(fxpar(hdr,  "EV_ENAB")), $
          expcount: long(fxpar(hdr,  "EXPCOUNT")), $
          exptime:  float(fxpar(hdr, "EXPTIME")), $
          filename: trim(fxpar(hdr,  "FILENAME")), $
          gset_id:  LONG(fxpar(hdr,  "GSET_ID")), $
	  instrume: trim(fxpar(hdr,  "INSTRUME")), $
	  ins_roll: double(fxpar(hdr,"INS_ROLL")), $
	  ins_x0:   double(fxpar(hdr,"INS_X0")), $
	  ins_y0:   double(fxpar(hdr,"INS_Y0")), $
	  ixwidth:  float(fxpar(hdr, "IXWIDTH")), $
	  iywidth:  float(fxpar(hdr, "IYWIDTH")), $
	  ll_id:    long(fxpar(hdr,  "LL_ID")), $
	  mir_pos:  long(fxpar(hdr,  "MIR_POS")), $
	  nwindows: long(fxpar(hdr,  "NWINDOWS")), $
          nx:       long(fxpar(hdr,  "NX")), $
	  ny:       long(fxpar(hdr,  "NY")), $
          object:   trim(fxpar(hdr,  "OBJECT")), $
          obj_id:   trim(fxpar(hdr,  "OBJ_ID")), $
          obs_mode: trim(fxpar(hdr,  "OBS_MODE")),$
          obs_prog: trim(fxpar(hdr,  "OBS_PROG")), $
          obs_seq:  trim(fxpar(hdr,  "OBS_SEQ")),$
	  obt_end:  double(fxpar(hdr,"OBT_END")), $
	  obt_time: double(fxpar(hdr,"OBT_TIME")), $
	  opslbits: trim(fxpar(hdr,  "OPSLBITS")), $
	  opsrbits: trim(fxpar(hdr,  "OPSRBITS")), $
	  ops_l:    long(fxpar(hdr,  "OPS_L")), $
	  ops_r:    long(fxpar(hdr,  "OPS_R")), $
          origin:   trim(fxpar(hdr,  "ORIGIN")), $
	  progname: trim(fxpar(hdr,  "PROGNAME")), $
          prog_id:  long(fxpar(hdr,  "PROG_ID")), $
          prog_ind: long(fxpar(hdr,  "PROG_IND")),$
          prog_num: long(fxpar(hdr,  "PROG_NUM")), $
	  ras_id:   long(fxpar(hdr,  "RAS_ID")), $
          ras_var:  long(fxpar(hdr,  "RAS_VAR")), $
          sc_roll:  double(fxpar(hdr,"SC_ROLL")),$
          sc_x0:    double(fxpar(hdr,"SC_X0")),$
          sc_y0:    double(fxpar(hdr,"SC_Y0")),$
          sci_obj:  trim(fxpar(hdr,  "SCI_OBJ")), $
	  sci_spec: trim(fxpar(hdr,  "SCI_SPEC")), $
          seqvalid: byte(fxpar(hdr,  "SEQVALID")), $
          seq_ind:  long(fxpar(hdr,  "SEQ_IND")), $
          seq_num:  long(fxpar(hdr,  "SEQ_NUM")),$
          ser_id:   long(fxpar(hdr,  "SER_ID")), $
          sfduadid: trim(fxpar(hdr,  "SFDUADID")), $
          slit_num: long(fxpar(hdr,  "SLIT_NUM")), $
          slit_pos: long(fxpar(hdr,  "SLIT_POS")), $
	  studyvar: long(fxpar(hdr,  "STUDYVAR")), $
          study_id: long(fxpar(hdr,  "STUDY_ID")), $
          telescop: trim(fxpar(hdr,  "TELESCOP")), $
          title:    trim(fxpar(hdr,  "TITLE")), $
          title_id: trim(fxpar(hdr,  "TITLE_ID")), $
          tracking: byte(fxpar(hdr,  "TRACKING")), $
          vds_acc:  byte(fxpar(hdr,  "VDS_ACC")), $
          vds_mode: long(fxpar(hdr,  "VDS_MODE")), $
          vds_pmcp: long(fxpar(hdr,  "VDS_PMCP")), $
          vds_ori:  byte(fxpar(hdr,  "VDS_ORI")), $
          wavemax:  double(fxpar(hdr,"WAVEMAX")), $
	  wavemin:  double(fxpar(hdr,"WAVEMIN")), $
	  xcen:     double(fxpar(hdr,"XCEN")), $
          xstep:    double(fxpar(hdr,"XSTEP")), $
          ycen:     double(fxpar(hdr,"YCEN")), $
          ystep:    double(fxpar(hdr,"YSTEP")),$
          zone_id:  long(fxpar(hdr,  "ZONE_ID")) $
         }

END

