	PRO READCDSCOL, UNIT, DATA, COLUMN, ROW, NANVALUE=NANVALUE,	$
		HEADER=HEADER_ONLY, STATUS=STATUS
;+
; Project     :	SOHO - CDS
;
; Name        :	READCDSCOL
;
; Purpose     :	Reads CDS data from a column in a FITS binary table.
;
; Explanation :	Reads data stored within a column of a CDS FITS binary table
;		into a structure variable containing the array and associated
;		information.
;
; Use         :	READCDSCOL, UNIT, DATA, COLUMN  [, ROW ]
;
;		FXBOPEN, UNIT, 'testdata.fits', 1	;Opens the table
;		READCDSCOL, UNIT, HE304, 'he304'	;Reads HE 304 data
;		FXBCLOSE, UNIT				;Closes the table
;
; Inputs      :	UNIT	= Logical unit number corresponding to the file
;			  containing the binary table.
;
;		COLUMN	= Column in the binary table to read data from, either
;			  as a character string containing a column label
;			  (TTYPE), or as a numerical column index starting from
;			  column one.
;
; Opt. Inputs :	ROW	= Either row number in the binary table to read data
;			  from, starting from row one, or a two element array
;			  containing a range of row numbers to read.  If not
;			  passed, then the entire column is read in.
;
;			  Row must be passed for variable length arrays.
;
;			  It is anticipated that CDS binary table files will
;			  consist of a single row, and that there will be no
;			  variable length arrays.  However, the ROW parameter
;			  is retained for possible future use.
;
; Outputs     :	DATA	= The output data will be read into an anonymous
;			  structure variable with the following tag names:
;
;				ARRAY	= The actual data array.  If /HEADER is
;					  set, then this is simply the number
;					  zero.
;				LABEL	= The column label from TTYPEn.
;				UNITS	= The units of the data.
;				MISSING	= Value representing missing data.
;				AXES	= The labels for the axes of ARRAY.
;				ORIGIN	= The value of the first point along
;					  each of the axes of ARRAY.
;				SPACING	= The pixel spacing along each axis.
;				ROTATION= The rotation of each axis relative to
;					  the standard coordinate system.  This
;					  is only relevant for spatial
;					  dimensions.
;
;			  There will also be the following additional tag names
;			  for columns containing detector data windows.
;
;				DETX	= Starting detector column in pixels.
;				DETY	= Starting detector row in pixels (NIS
;					  only).
;				BINX	= Binning factor across columns, in
;					  pixels.
;				BINY	= Binning factor across rows, in pixels
;					  (NIS only).
;				WAVELENGTH = Wavelength of observation, in
;					  Angstroms.
;				WAVEMIN	= Minimum wavelength.
;				WAVEMAX	= Maximum wavelength.
;				WAVEBAND= Wavelength band.
;				GR_ORDER= Grating order.
;
; Opt. Outputs:	None.
;
; Keywords    :	NANVALUE = Value signalling data dropout.  All points
;			   corresponding to IEEE NaN (not-a-number) are
;			   converted to this number.  Ignored unless DATA is of
;			   type float, double-precision or complex.  If used,
;			   then the MISSING tag in the returned DATA structure
;			   reflects this value.  The default is -1.
;
;			   It is anticipated that the CDS data files will
;			   always be stored as integers, so that the use of
;			   this keyword should be unnecessary.
;
;		HEADER	= If set, then only the header is read in.
;
;		STATUS	= If defined and passed, then any error messages will
;			  be returned to the user in this parameter rather than
;			  depending on the MESSAGE routine in IDL.  If no
;			  errors are encountered, then a null string is
;			  returned.  In order to use this feature, STATUS must
;			  be defined first, e.g.
;
;				STATUS = ''
;				READCDSCOL, STATUS=STATUS, ...
;				IF STATUS NE '' THEN ...
;
; Calls       :	DATATYPE, FXBCOLNUM, FXBHEADER, FXBISOPEN, FXBREAD, FXBTDIM,
;		FXPAR, TRIM
;
; Common      :	None.
;
; Restrictions:	The binary table must have been opened with FXBOPEN.
;
;		This routine is intended solely for CDS data.  It makes certain
;		assumptions about what keywords will be present in the FITS
;		binary table header.  The present version of this routine was
;		written to be compatible with the sample file "testdata.fits"
;		generated based on SERTS data.
;
;		Currently, the returned value of DATA.ORIGIN will not be
;		correct if all values of TRPIXn are not equal to 1.  However,
;		it is not anticipated that any other values of TRPIX will be
;		used for the foreseeable future.
;
; Side effects:	If multiple rows are read, then the last entry in DATA.AXES
;		will be "RECORD".  DATA.ORIGIN, DATA.SPACING, and DATA.ROTATION
;		will be similarly updated to reflect this additional dimension.
;
; Category    :	Data_Handling, I_O, FITS, CDS
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 2 July 1993.
;
; Modified    :	Version 1, William Thompson, GSFC, 2 July 1993.
;		Version 2, William Thompson, GSFC, 15 December 1993.
;			Modified so that additional dimension is annotated when
;			multiple rows are read.
;		Version 3, William Thompson, GSFC, 27 May 1994.
;			Added HEADER keyword.
;		Version 4, William Thompson, GSFC, 21 June 1994.
;			Changed from Y-Z to X-Y coordinate system.
;			Added WAVEBAND and GR_ORDER tags.
;			Added STATUS keyword.
;			Renamed from READ_CDS to READCDSCOL
;		Version 5, William Thompson, GSFC, 23 June 1994
;			Modified so that STATUS is not touched if not defined.
;		Version 6, William Thompson, GSFC, 2 August 1995
;			Added tags BINX and BINY
;
; Version     :	Version 6, 2 August 1995
;-
;
;
	ON_ERROR, 2
;
;  Check the number of parameters.
;
	IF N_PARAMS() LT 3 THEN BEGIN
		MESSAGE = 'Syntax:  READCDSCOL, UNIT, DATA, COLUMN  [, ROW ]'
		IF N_ELEMENTS(STATUS) NE 0 THEN BEGIN
			STATUS = MESSAGE
			RETURN
		END ELSE MESSAGE, MESSAGE		
	ENDIF
;
;  Check the logical unit number for the FITS binary table.
;
	IF N_ELEMENTS(UNIT) EQ 0 THEN BEGIN
		MESSAGE = 'UNIT is not defined'
		IF N_ELEMENTS(STATUS) NE 0 THEN BEGIN
			STATUS = MESSAGE
			RETURN
		END ELSE MESSAGE, MESSAGE		
	END ELSE IF NOT FXBISOPEN(UNIT) THEN BEGIN
		MESSAGE = 'Unit ' + TRIM(UNIT) + ' is not open.'
		IF N_ELEMENTS(STATUS) NE 0 THEN BEGIN
			STATUS = MESSAGE
			RETURN
		END ELSE MESSAGE, MESSAGE		
	ENDIF
;
;  Check the column identification, and get the appropriate column number.
;
	IF N_ELEMENTS(COLUMN) EQ 0 THEN BEGIN
		MESSAGE = 'COLUMN is not defined'
		IF N_ELEMENTS(STATUS) NE 0 THEN BEGIN
			STATUS = MESSAGE
			RETURN
		END ELSE MESSAGE, MESSAGE		
	END ELSE IF N_ELEMENTS(COLUMN) GT 1 THEN BEGIN
		MESSAGE = 'COLUMN must be a scalar'
		IF N_ELEMENTS(STATUS) NE 0 THEN BEGIN
			STATUS = MESSAGE
			RETURN
		END ELSE MESSAGE, MESSAGE		
	ENDIF
	IF N_ELEMENTS(STATUS) NE 0 THEN BEGIN
		ICOL = FXBCOLNUM(UNIT,COLUMN,ERRMSG=STATUS)
		IF STATUS EQ '' THEN RETURN
	END ELSE ICOL = FXBCOLNUM(UNIT,COLUMN)
	IF ICOL EQ 0 THEN BEGIN
		MESSAGE = 'COLUMN ' + TRIM(COLUMN) + ' not found.'
		IF N_ELEMENTS(STATUS) NE 0 THEN BEGIN
			STATUS = MESSAGE
			RETURN
		END ELSE MESSAGE, MESSAGE		
	ENDIF
;
;  Read in the data array.
;
	IF N_ELEMENTS(NANVALUE) EQ 1 THEN NAN = NANVALUE ELSE NAN = -1
	IF KEYWORD_SET(HEADER_ONLY) THEN BEGIN
		ARRAY = 0
	END ELSE IF N_PARAMS() EQ 3 THEN BEGIN
		IF N_ELEMENTS(STATUS) NE 0 THEN BEGIN
			FXBREAD, UNIT, ARRAY, ICOL, NANVALUE=NAN, ERRMSG=STATUS
			IF STATUS NE '' THEN RETURN
		END ELSE FXBREAD, UNIT, ARRAY, ICOL, NANVALUE=NAN
	END ELSE BEGIN
		IF N_ELEMENTS(STATUS) NE 0 THEN BEGIN
			FXBREAD, UNIT, ARRAY, ICOL, ROW, NANVALUE=NAN,	$
				ERRMSG=STATUS
			IF STATUS NE '' THEN RETURN
		END ELSE FXBREAD, UNIT, ARRAY, ICOL, ROW, NANVALUE=NAN
	ENDELSE
;
;  Retrieve the binary table header.
;
	HEADER = FXBHEADER(UNIT)
;
;  Extract the associated information for the column in question.
;
	LABEL = TRIM( FXPAR(HEADER, 'TTYPE' + TRIM(ICOL)) )
	UNITS = TRIM( FXPAR(HEADER, 'TUNIT' + TRIM(ICOL)) )
	DESC = FXBTDIM( FXPAR(HEADER, 'TDESC' + TRIM(ICOL)) )
	RPIX = DOUBLE( FXBTDIM( FXPAR(HEADER, 'TRPIX' + TRIM(ICOL)) ))
	RVAL = DOUBLE( FXBTDIM( FXPAR(HEADER, 'TRVAL' + TRIM(ICOL)) ))
	DELT = DOUBLE( FXBTDIM( FXPAR(HEADER, 'TDELT' + TRIM(ICOL)) ))
	ROTA = DOUBLE( FXBTDIM( FXPAR(HEADER, 'TROTA' + TRIM(ICOL)) ))
;
;  If ROTA is scalar, and the other parameters are vectors, then reform ROTA
;  into a vector.  This is a workaround to take into account headers that don't
;  have ROTA keywords.
;
	IF (N_ELEMENTS(ROTA) EQ 1) AND (N_ELEMENTS(DELT) GT 1) THEN	$
		ROTA = REPLICATE(ROTA, N_ELEMENTS(DELT))
;
;  If multiple rows were read, then update the axis notations.
;
	IF FXPAR(HEADER,'NAXIS2') EQ 1 THEN GOTO, CONTINUE
	IF N_ELEMENTS(ROW) EQ 1 THEN GOTO, CONTINUE
	IF N_ELEMENTS(ROW) EQ 2 THEN BEGIN
		IF ROW(0) EQ ROW(1) THEN GOTO, CONTINUE
		FIRST_ROW = ROW(0)
	END ELSE FIRST_ROW = 1
;
	DESC = [DESC, 'RECORD']		;Last axis is record number
	RPIX = [RPIX, 1]		;Reference is first pixel
	RVAL = [RVAL, FIRST_ROW]	;First row number
	DELT = [DELT, 1]		;Pixels are one row apart
	ROTA = [ROTA, 0]		;Rotations are irrelevant
CONTINUE:
;
;  If the reference pixel is not the first pixel in the array, then recalculate
;  the value for the origin.  This is not currently implemented because
;  rotation should also be taken into account.  However, it will probably not
;  be relevant anyways, since RPIX is anticipated to always be 1.
;
;	RVAL = RVAL - DELT * (RPIX - 1)
;
;  Get the value associated with missing data.  For integer data this is stored
;  in the TNULLn keyword.  For floating point data, it's the NaN value used
;  above.  For string arrays it's the null string.
;
	NULL = FXPAR(HEADER, 'TNULL' + TRIM(ICOL))
	CASE DATATYPE(ARRAY,1) OF
		'Byte':		NULL = BYTE(NULL)
		'Integer':	NULL = FIX(NULL)
		'Long':		NULL = LONG(NULL)
		'Float':	NULL = FLOAT(NAN)
		'Double':	NULL = DOUBLE(NAN)
		'Complex':	NULL = NAN * COMPLEX(1,0)
		'String':	NULL = ''
	ENDCASE
;
;  Start formatting a command to create the anonymous structure containing the
;  data array and associated data.
;
	COMMAND = 'DATA = {ARRAY: ARRAY, LABEL: LABEL, UNITS: UNITS, ' + $
		'MISSING: NULL, AXES: DESC, ORIGIN: RVAL, SPACING: DELT, ' + $
		'ROTATION: ROTA'
;
;  CDS level-1 data windows are recognized by the presence of a TDETX keyword.
;  If this keyword is present, then include the additional structure elements
;  associated with data windows.
;
	DETX = FXPAR(HEADER, 'TDETX' + TRIM(ICOL), COUNT=COUNT)
	IF COUNT GT 0 THEN BEGIN
		BINX = FXPAR(HEADER, 'TBINX' + TRIM(ICOL)) > 1
		COMMAND = COMMAND + ', DETX: DETX, BINX: BINX'
		IF TRIM(FXPAR(HEADER,'DETECTOR')) EQ 'NIS' THEN BEGIN
			DETY = FXPAR(HEADER, 'TDETY' + TRIM(ICOL))
			BINY = FXPAR(HEADER, 'TBINY' + TRIM(ICOL)) > 1
			COMMAND = COMMAND + ', DETY: DETY, BINY: BINY'
		ENDIF
		WAVE = FXPAR(HEADER, 'TWAVE' + TRIM(ICOL))
		WMIN = FXPAR(HEADER, 'TWMIN' + TRIM(ICOL))
		WMAX = FXPAR(HEADER, 'TWMAX' + TRIM(ICOL))
		BAND = FXPAR(HEADER, 'TWBND' + TRIM(ICOL))
		GORD = FXPAR(HEADER, 'TGORD' + TRIM(ICOL))
		COMMAND = COMMAND + ', WAVELENGTH: WAVE, WAVEMIN: WMIN, ' + $
			'WAVEMAX: WMAX, WAVEBAND: BAND, GR_ORDER: GORD'
	ENDIF
;
;  Create the anonymous structure and return.
;
	TEST = EXECUTE(COMMAND+'}')
;
	IF N_ELEMENTS(STATUS) NE 0 THEN STATUS = ''
	RETURN
	END
