;+
; Project     : SOHO - CDS     
;                   
; Name        : READCDSFITS()
;               
; Purpose     : Read and return the contents of a CDS FITS level-1 file
;               
; Explanation : Reads:
;			- header data (for the whole file), 
;			- fits variables for all detector data windows
;			- detector data 
;               	- fits variables for auxiliary data
;			- auxiliary data
;
;		Returns all data in one IDL structure (the CDS QL
;		data format)
;
; Use         : VAR = READCDSFITS( FILENAME )
;    
; Inputs      : FILENAME = the name of the CDS FITS level-1 file.
;               
; Opt. Inputs : None.
;               
; Outputs     : Returns an IDL structure with data from the FITS file
;		(CDS QL data format)
;               
; Opt. Outputs: None.
;               
; Keywords    : /DISPLAY : set this keyword to invoke DSP_INFO
;		to display information about the data in the
;		file. 
;
;		/SELECT : set to prompt for selection of only some
;		of the detector data windows.
;
;               PRESELECTED : Array of selected windows to read in. 
;               References the windows by their index (0..Nwindows-1)
;
;		/HEADER : Set to only read descriptive information
;		about detector data windows -- no data will be loaded
;
;		/AUTO : Set to force silent truncation of the data
;		by reading just those detector data windows that
;		fit below the memory limit
;
;               /BATCH : Set to suppress xack window popping up
;               when there are problems with the GSET_ID.
;
;		MEMLIMIT : Set to the maximum size of memory that
;		the detector data are allowed to occupy.
;
;		STATUS : Set to a initialized, named variable to 
;		return an error message instead of crashing. If
;		everything's ok, a null string will be returned.
;
;		In addition, any keywords accepted by UPD_CDS_POINT, such as
;		/NOROLL, are also accepted.
;
;
; Calls       : anytim2utc(), auxread(), average(), break_file, cds_pad_qlds,
;               concat_dir(), datatype(), default, detread, detselect(),
;               dsp_info, find_with_def(), fxbclose, fxbisopen(), fxbopen,
;               fxpar(), get_utc, get_wavecal, hdrread(), load_wavecal(),
;               os_family(), qlmgr, readcalfits(), test_open(), trim(),
;               upd_cds_point, xack
;
; Common      : None.
;
; Restrictions: The file MUST be a CDS FITS level-1 file.
;               
; Side effects: Starts QLMGR if not installed.
;               
; Category    : Data_Handling, I_O, FITS, CDS, QuickLook
;               
; Prev. Hist. : None.
;
; Written     : Stein Vidar Hagfors Haugan, 27 September 1993
;               
; Modified    : SVHH, 1 October 1993  - Added keyword inheritance to DSP_INFO.
;		SVHH, 2 October 1993  - QLMGR registration included
;		SVHH, 5 October 1993  - SELECT keyword/functionality included
;               CDP  25-Oct-93  cut out _EXTRA
;		SVHH, Version 2, 8-June-1994
;			Added /header, /auto, memlimit=memlimit,status=status
;		SVHH, Version 3, 11-June-1994
;			Split DETREAD into DETSELECT + DETREAD, to minimize
;			memory overhead.
;               CDP, Search for input files in CDS_FITS_DATA if not in
;                    curent (or specified) directory and make default 
;                    extension .fits.  1-Jun-95
;		Version 5, William Thompson, GSFC, 2 June 1995
;			Use FIND_WITH_DEF to accomplish changes of version 4
;               Version 6, SVHH, 27-Sept-1995
;                       Added some auxiliary data tags + renamed
;                       other auxiliary data tags to conform to a single
;                       naming convention (Fits TTYPE + DATA/DESC)
;		Version 7, SVHH, 9-November-1995
;                       Added BACKGROUND handling for the NIS.
;               Version 8, CDP, Trap entering array (even array(1)) of
;                               filenames.   16-Nov-95
;               Version 9, SVHH, 15-January-1996, Fixed typo error in
;                       'SLI_POS' --> 'SLIT_POS'.
;               Version 10, SVHH, 22-February-1996. 
;                       Added BKGRCD/BKGPCD/BKMGULCD data. Also added
;                       tag HDRTEXT, containing the header text array.
;                       The DETDATA array is now always a float, so no
;                       duplication of the structure need to be
;                       performed when debiasing or converting to
;                       physical units.
;               Version 11, SVHH, 13 April 1996
;                       Added support for wavelength calibrations 
;                       (wavecal tag).
;               Version 12, SVHH, 14 April 1996
;                       Automatic switch to readcalfits if it's a
;                       calibration fits file.
;                       Using !def_gset_id for default GSET_ID
;               Version 13, SVHH, 18 April 1996
;                       Added BATCH switch for turning off xack dialog box.
;               Version 14, SVHH, 8 May 1996
;                       Added PRESELECTED option.
;               Version 15, William Thompson, 6 August 1996, GSFC 
;                       Added /RESET to FIND_WITH_DEF
;               Version 16, SVHH, 18 August 1996
;                       New storage system doesn't use DETDATA.
;                       Added cds_pad_qlds call to make all windows equal.
;                       It's possible to select the new storage system by
;                       setting OLDSYS=0. If the environment variable
;                       USE_NEW_STORAGE is defined and not an empty string,
;                       then you can turn OFF the new storage system by
;                       setting /OLDSYS.
;               Version 17, SVHH, 23 August 1996
;                       PITCHBITS => PTCHBITS, to reflect actual name of
;                       data column in fits files.
;               Version 18, SVHH, 4 April 1997
;                       Fixed a bug setting STATUS=1 on successful completion,
;                       and added a CATCH,ERROR statement in case of
;                       unforeseen trouble.
;               Version 19, SVHH, 10 May 1997
;                       Added support for compressed files.
;               Version 20, SVHH, 13 May 1997
;                       Decompression into $CDS_FITS_TEMP_W (first choice), or
;                       $CDS_FITS_DATA_W. Also setting umask 002 before
;                       decompressing to share data.
;               Version 21, SVHH, 14 May 1997
;                       Modified how .Z and .gz extensions were tested in case
;                       .fits file extension is already supplied.
;               Version 22, SVHH, 10 June 1997
;                       Made the BACKGROUND tag become a named structure.
;               Version 23, SVHH, 29 June 1997
;                       Added call to UPD_CDS_POINT, and keyword /NOROLL.
;               Version 24, SVHH, 11 July 1997
;                       Fixed a bug in decompression directories & will now
;                       compress to current directory and delete file after
;                       read when $CDS_FITS_TEMP_W/$CDS_FITS_DATA_W are not
;                       set.
;               Version 25, SVHH, 15 July 1997
;                       Renamed STATUS keyword => ERRMSG for consistency with
;                       e.g., FXBOPEN etc.
;		Version 26, 17-Dec-1997, William Thompson, GSFC
;			Allow any UPD_CDS_POINT keyword to be passed through.
;               Version 27, 3-Dec-1999, Zarro (SM&A/GSFC) - added check
;                       for existence of !DEBUG & !DEF_GSET_ID
;               Version 28, 8-Oct-2001, Zarro (EITI/GSFC) - propagated
;                       error from UPD_CDS_POINT
;               Version 29, 12-Aug-2003, Peter Young, RAL
;                       Modified so that it can read compressed fits files
;                       on a Windows machine with gzip installed.
;               Version 30, 10-Oct-2003, William Thompson, GSFC
;                       Fixed FILE_TEST() problem before IDL 5.4
;               Version 31, 22-Jun-2006, William Thompson, GSFC
;                       Don't call FIND_WITH_DEF if full path is given.
;               Version 32, 14-Mar-2008, WTT, support level-1 files.
;
; Version     : Version 32
;-            

FUNCTION ReadCDSfits,filename,display=display,select=select,errmsg=status,$
                     header=hdr,auto=auto,memlimit=memlimit,BATCH=BATCH,$
		     PRESELECTED=PRESELECTED,OLDSYS=OLDSYS,_EXTRA=_EXTRA
  
  IF getenv("USE_NEW_STORAGE") NE '' THEN default,oldsys,0
  default,oldsys,1
  
  On_Error,2

;-- get these checks done early

  defsysv,'!debug',exists=defined
  if not defined then defsysv,'!debug',0

  defsysv,'!def_gset_id',exists=defined
  if not defined then defsysv,'!def_gset_id',0
                                           
  IF !DEBUG ne 0 THEN On_Error,0           
  
  IF N_elements(memlimit) eq 0 THEN memlimit = 15*2L^20	; 15 MB
  
  usestatus = n_elements(status) GT 0
  IF usestatus THEN status = ''
  
  ;; This should catch errors which are not catered for explicitly
  IF usestatus THEN BEGIN
     catch,error_status
     IF error_status NE 0 THEN BEGIN
        catch,/cancel
        status = status+"Caught error: "+!err_string
        GOTO,error_close_exit
     END
  END
  
;
;  Look for file, using $CDS_FITS_DATA as the default directory, and .fits as
;  the default extension.  Open the binary table.
;

;  
;  check it's a scalar
;
  if n_elements(filename) gt 1 then begin
     print,'Input filename must be a scalar.'
     return, -1
  endif
 
;
;  force it to be scalar, ie not array(1)
;
  filename = filename(0)

  if file_exist(filename) then use_file = filename else $
    use_file = find_with_def(filename,'$CDS_FITS_DATA','.fits',/reset)
  
  ;; Compressed files?
  
  if use_file EQ '' then BEGIN
     
     ;; If we have nowhere to write a decompressed file, skip looking for 
     ;; compressed ones.. skip decompression for non-unix systems as well.
     
     write_dir = getenv("CDS_FITS_TEMP_W")
     IF write_dir EQ '' THEN write_dir = getenv("CDS_FITS_DATA_W")
     
     IF !version.os_family NE 'unix' AND !version.os_family NE 'Windows' THEN BEGIN
        txt = "Can't find FITS file"
        IF usestatus THEN status = txt
        print,txt
        return,-1
     END
     
     ;; Check various compressed formats. In case the file name already
     ;; contains the .fits extension, don't add another one.  On the other
     ;; hand, we won't disallow *other* extensions being supplied/used, nor
     ;; will we disallow a dot inside the file name, so we shouldn't
     ;; break_file and then assemble it with .fits...
     
     zips = ['.Z','.gz']
     unzips = ['uncompress -c ','gunzip -c '            ]
    ;
    ; Windows gzip works for both .Z and .gz files
    ;
     IF  !version.os_family EQ 'Windows' THEN BEGIN
       unzips=['gzip -dc ','gzip -dc ']
     ENDIF
     
     IF strpos(strupcase(filename),'.FITS') EQ -1 THEN BEGIN
        zips = ['.fits'+zips, zips]
        unzips = [unzips,unzips]
     END
     
     nzips = n_elements(zips)
     
     FOR zipno = 0,nzips-1 DO BEGIN 
        print,'Cannot find, will check for filename + '+zips(zipno)
        
        use_file = find_with_def(filename,'$CDS_FITS_DATA',zips(zipno))
        IF use_file EQ '' AND zipno EQ nzips-1 THEN BEGIN
           txt = 'No compressed file found either'
           print,txt
           IF usestatus THEN status = txt
           return,-1
        END ELSE IF use_file NE '' THEN BEGIN 
           ;; Found a compressed file - now figure out how to decompress it
        
           break_file,use_file,disk,dir,filnam,ext
           
           new_use_file = concat_dir(write_dir,filnam+".fits")
           unzip = unzips(zipno)+use_file+" > "+new_use_file
           print,unzip
           IF !version.os_family EQ 'Windows' THEN BEGIN
             spawn,unzip
           ENDIF ELSE BEGIN
             spawn,"umask 002 ; "+unzip
           ENDELSE

           if !version.release ge '5.4' then begin
               result1=call_function('file_test',new_use_file,/zero_length)
               result2=call_function('file_test',new_use_file)
           end else begin
               result2=file_exist(new_use_file)
               if result2 then begin
                   openr, testunit, new_use_file, /get_lun
                   result1 = (fstat(testunit)).size eq 0
                   free_lun, testunit
               end else result1 = 0
           endelse

           IF result1 EQ 1 OR result2 EQ 0 THEN BEGIN
              txt = 'Decompression of compressed fits file failed'
              IF usestatus THEN status = txt
              file_delete,new_use_file
              print,txt
              IF !version.os_family EQ 'Windows' AND result1 EQ 1 THEN BEGIN
                print,'  -- Install gzip on your PC. Go to www.gzip.org.'
              ENDIF
              return,-1
           END
           use_file = new_use_file
           PRINT,"Trying to read "+new_use_file
           IF write_dir EQ '' THEN delete_after_read = 1
           GOTO,try_read
        END
     ENDFOR 
     
     txt = "Can't find FITS file"
     IF usestatus THEN status = txt
     print,txt
     return,-1
     
  ENDIF
  
try_read:
 
  fxbOpen,unit,use_file,1,header,errmsg=status

  IF usestatus THEN if status(0) ne '' THEN return,0
  
  ;; We pass it on to readcalfits if it's a calibration fits file.
  ;;
  IF fxpar(header,"TELESCOP") EQ 'CALIBRATION' THEN BEGIN
     FXBCLOSE,UNIT
     RETURN,readcalfits(use_file,display=display,select=select, $
                        status=status,header=hdr,auto=auto,memlimit=memlimit)
  END
  
  selection = DetSelect(unit,header,select=select,header=hdr,$
                        auto=auto,memlimit=memlimit,status=status,$
                        preselected=preselected,$
                        background=background)
  
  IF usestatus THEN if status(0) ne '' THEN goto,error_close_exit
  
  IF datatype(selection) ne 'STC' THEN $
	  message,'Failed reading detector data'
  
  DEL_TIME = AuxRead(unit,header,'DEL_TIME',status=status)
  IF usestatus THEN if status(0) ne '' THEN goto,error_close_exit
  
  Ins_X = AuxRead(unit,header,'INS_X',status=status)
  IF usestatus THEN if status(0) ne '' THEN goto,error_close_exit
  
  INS_Y = AuxRead(unit,header,'INS_Y',status=status)
  IF usestatus THEN if status(0) ne '' THEN goto,error_close_exit
  
  OPSLBITS = AuxRead(unit,header,'OPSLBITS',status=status)
  IF usestatus THEN if status(0) NE '' THEN GOTO,error_close_exit
  
  OPS_L = AuxRead(unit,header,'OPS_L',status=status)
  IF usestatus THEN if status(0) NE '' THEN GOTO,error_close_exit
  
  OPSRBITS = AuxRead(unit,header,'OPSRBITS',status=status)
  IF usestatus THEN if status(0) NE '' THEN GOTO,error_close_exit
  
  OPS_R = AuxRead(unit,header,'OPS_R',status=status)
  IF usestatus THEN if status(0) NE '' THEN GOTO,error_close_exit
  
;  OPS_X = AuxRead(unit,header,'OPS_X',status=status)
;  IF usestatus THEN if status(0) ne '' THEN goto,error_close_exit
;  
;  OPS_Y = AuxRead(unit,header,'OPS_Y',status=status)
;  IF usestatus THEN if status(0) ne '' THEN goto,error_close_exit
  
  SLITBITS = AuxRead(unit,header,'SLITBITS',status=status)
  IF usestatus THEN if status(0) NE '' THEN GOTO,error_close_exit
  
  SLIT_POS = AuxRead(unit,header,'SLIT_POS',status=status)
  IF N_elements(status)	gt 0 THEN if status(0) ne '' THEN goto,error_close_exit
  
  MIR_POS = AuxRead(unit,header,'MIR_POS',status=status)
  IF N_elements(status)	gt 0 THEN if status(0) ne '' THEN goto,error_close_exit
  
  PTCHBITS = AuxRead(unit,header,'PTCHBITS',status=status)
  IF usestatus THEN if status(0) NE '' THEN GOTO,error_close_exit
  
  PITCH = AuxRead(unit,header,'PITCH',status=status)
  IF usestatus THEN if status(0) NE '' THEN GOTO,error_close_exit
  
  YAW_BITS = AuxRead(unit,header,'YAW_BITS',status=status)
  IF usestatus THEN if status(0) NE '' THEN GOTO,error_close_exit
  
  YAW = AuxRead(unit,header,'YAW',status=status)
  IF usestatus THEN if status(0) NE '' THEN GOTO,error_close_exit
  
  EV_LEVEL = AuxRead(unit,header,'EV_LEVEL',status=status)
  IF usestatus THEN if status(0) NE '' THEN GOTO,error_close_exit
  
  ONBOARDX = AuxRead(unit,header,'ONBOARDX',status=status)
  IF usestatus THEN if status(0) NE '' THEN GOTO,error_close_exit
  
  ONBOARDY = AuxRead(unit,header,'ONBOARDY',status=status)
  IF usestatus THEN if status(0) NE '' THEN GOTO,error_close_exit
  
  EV_RECOG = AuxRead(unit,header,'EV_RECOG',status=status)
  IF usestatus THEN if status(0) NE '' THEN GOTO,error_close_exit
  
  EV_DETEC = AuxRead(unit,header,'EV_DETEC',status=status)
  IF usestatus THEN if status(0) NE '' THEN GOTO,error_close_exit
  
  EV_VALID = AuxRead(unit,header,'EV_VALID',status=status)
  IF usestatus THEN if status(0) NE '' THEN GOTO,error_close_exit
  
  ;; GIS data
  BKGRCD = AuxRead(unit,header,'BKGRCD',status=status)
  IF usestatus THEN if status(0) NE '' THEN GOTO,error_close_exit
  
  BKGPCD = AuxRead(unit,header,'BKGPCD',status=status)
  IF usestatus THEN if status(0) NE '' THEN GOTO,error_close_exit
  
  BKGULCD = AuxRead(unit,header,'BKGULCD',status=status)
  IF usestatus THEN if status(0) NE '' THEN GOTO,error_close_exit
  
  ;; Wavelength calibrations.
  
  date = anytim2utc(fxpar(header,"DATE_OBS"))
  detector = fxpar(header,"DETECTOR")
  
  status = ''
  IF STRMID(detector,0,1) EQ 'G' THEN BEGIN
     gset_id = fxpar(header,"GSET_ID")
     ;; gset_id = 29               ; Use to provoke error message
     errmsg = ''
     get_wavecal,'GIS',date,wavecal,gset_id=gset_id,errmsg=status
     IF (status)(0) NE '' THEN BEGIN
        IF NOT KEYWORD_SET(BATCH) THEN $
           xack, $
           ["GET_WAVECAL couldn't find a proper wavelength " + $
            "calibration  entry ", $
            "for fits file "+use_file+".", $
            "",$
            "For now the default GSET_ID="+ $
            trim(!def_gset_id) + " and current date will be used",$
            "You should locate a reasonable wavelength " + $
            "calibration entry for", $
            "the GIS and then do ", $
            "IDL> qlds.wavecal = wavecal"]
        status = ''
        get_utc,utc
        get_wavecal,'GIS',utc,wavecal,gset_id=!def_gset_id,errmsg=status
        IF (status)(0) NE '' THEN MESSAGE,"Unable to find default wavecal"
     END
  END ELSE BEGIN
     get_wavecal,'NIS',date,wavecal,errmsg=status
     IF (status)(0) NE '' THEN BEGIN
        IF NOT KEYWORD_SET(BATCH) THEN $
           xack, $
           ["GET_WAVECAL couldn't find a proper wavelength " + $
            "calibration entry", $
            "for fits file "+use_file+".", $
            "", $
            "The current wavelength calibration for the NIS will be used", $
            "You may locate another wavelength calibration with get_wavecal", $
            "and then do", $
            "IDL> qlds.wavecal = wavecal"]
        get_utc,utc
        get_wavecal,'NIS',utc,wavecal
     END
  END
  
  IF selection.datasize	EQ 0 THEN BEGIN
	selection.dimension(*)=1
        type = 2
  end ELSE IF selection.longflag THEN type=3 $
  else                                type=2
  
  ;; Since we'll convert to physical units anyway...
  ;; 
  type = 4
  
;	detstring = "make_array(/long,dimension=selection.dimension)" $
;  ELSE	detstring = "make_array(/int, dimension=selection.dimension)"

  if N_elements(background) eq 0 then background=0 $
  ELSE background = $
     create_struct(background,name='CDS_BACK_'+$
                   trim(n_elements(background.average(*,0))))
  
  IF NOT KEYWORD_SET(oldsys) THEN BEGIN
     a ={  QL_ID:0.0D, QL_NO:0,$ ; Quicklook Manager tags
           hdrtext : header, $  ; Header text array
           Header: HdrRead(header),$
           $;;	DetData:reform(make_array(type=type,$
           $;;		   dimension=selection.dimension),$
           $;;		   selection.dimension),$  ; Detector data storage
           DetDesc:selection.DetDesc,$ ; Descriptors of detector windows
           background:background,$ ; BACKGROUND levels.
           wavecal:wavecal,$
           Del_timedata:DEL_TIME.data,$ ; Del_Time values
           Del_timedesc:DEL_TIME.desc,$ ; Del_Time descriptor
           INS_XDATA:Ins_X.data,$ ; Instr. pointing x-axis
           INS_XDESC:Ins_X.desc,$
           INS_YDATA:Ins_Y.data,$ ; Instr. pointing y-axis
           INS_YDESC:Ins_Y.desc,$
           OPSLBITSDATA:OPSLBITS.DATA,$
           OPSLBITSDESC:OPSLBITS.DESC,$
           OPS_LDATA:OPS_L.DATA,$
           OPS_LDESC:OPS_L.DESC,$
           OPSRBITSDATA:OPSRBITS.DATA,$
           OPSRBITSDESC:OPSRBITS.DESC,$
           OPS_RDATA:OPS_R.DATA,$
           OPS_RDESC:OPS_R.DESC,$
           SLITBITSDATA:SLITBITS.DATA,$
           SLITBITSDESC:SLITBITS.DESC,$
           SLIT_POSDATA:SLIT_POS.DATA,$
           SLIT_POSDESC:SLIT_POS.DESC,$
           MIR_POSDATA:MIR_POS.DATA,$
           MIR_POSDESC:MIR_POS.DESC,$
           PTCHBITSDATA:PTCHBITS.DATA,$
           PTCHBITSDESC:PTCHBITS.DESC,$
           PITCHDATA:PITCH.DATA,$
           PITCHDESC:PITCH.DESC,$
           YAW_BITSDATA:YAW_BITS.DATA,$
           YAW_BITSDESC:YAW_BITS.DESC,$
           YAWDATA:YAW.DATA,$
           YAWDESC:YAW.DESC,$
           EV_LEVELDATA:EV_LEVEL.DATA,$
           EV_LEVELDESC:EV_LEVEL.DESC,$
           ONBOARDXDATA:ONBOARDX.DATA,$
           ONBOARDXDESC:ONBOARDX.DESC,$
           ONBOARDYDATA:ONBOARDY.DATA,$
           ONBOARDYDESC:ONBOARDY.DESC,$
           EV_RECOGDATA:EV_RECOG.DATA,$
           EV_RECOGDESC:EV_RECOG.DESC,$
           EV_DETECDATA:EV_DETEC.DATA,$
           EV_DETECDESC:EV_DETEC.DESC,$
           EV_VALIDDATA:EV_VALID.DATA,$
           EV_VALIDDESC:EV_VALID.DESC,$
           BKGRCDDATA:BKGRCD.DATA,$
           BKGRCDDESC:BKGRCD.DESC,$
           BKGPCDDATA:BKGPCD.DATA,$
           BKGPCDDESC:BKGPCD.DESC,$
           BKGULCDDATA:BKGULCD.DATA,$
           BKGULCDDESC:BKGULCD.DESC }
  END ELSE BEGIN
     a ={  QL_ID:0.0D, QL_NO:0,$ ; Quicklook Manager tags
           hdrtext : header, $  ; Header text array
           Header: HdrRead(header),$
           DetData:REFORM(make_array(type=type,dimension=selection.dimension),$
                          selection.dimension),$ ; Detector data storage
           DetDesc:selection.DetDesc,$ ; Descriptors of detector windows
           background:background,$ ; BACKGROUND levels.
           wavecal:wavecal,$
           Del_timedata:DEL_TIME.data,$ ; Del_Time values
           Del_timedesc:DEL_TIME.desc,$ ; Del_Time descriptor
           INS_XDATA:Ins_X.data,$ ; Instr. pointing x-axis
           INS_XDESC:Ins_X.desc,$
           INS_YDATA:Ins_Y.data,$ ; Instr. pointing y-axis
           INS_YDESC:Ins_Y.desc,$
           OPSLBITSDATA:OPSLBITS.DATA,$
           OPSLBITSDESC:OPSLBITS.DESC,$
           OPS_LDATA:OPS_L.DATA,$
           OPS_LDESC:OPS_L.DESC,$
           OPSRBITSDATA:OPSRBITS.DATA,$
           OPSRBITSDESC:OPSRBITS.DESC,$
           OPS_RDATA:OPS_R.DATA,$
           OPS_RDESC:OPS_R.DESC,$
           SLITBITSDATA:SLITBITS.DATA,$
           SLITBITSDESC:SLITBITS.DESC,$
           SLIT_POSDATA:SLIT_POS.DATA,$
           SLIT_POSDESC:SLIT_POS.DESC,$
           MIR_POSDATA:MIR_POS.DATA,$
           MIR_POSDESC:MIR_POS.DESC,$
           PTCHBITSDATA:PTCHBITS.DATA,$
           PTCHBITSDESC:PTCHBITS.DESC,$
           PITCHDATA:PITCH.DATA,$
           PITCHDESC:PITCH.DESC,$
           YAW_BITSDATA:YAW_BITS.DATA,$
           YAW_BITSDESC:YAW_BITS.DESC,$
           YAWDATA:YAW.DATA,$
           YAWDESC:YAW.DESC,$
           EV_LEVELDATA:EV_LEVEL.DATA,$
           EV_LEVELDESC:EV_LEVEL.DESC,$
           ONBOARDXDATA:ONBOARDX.DATA,$
           ONBOARDXDESC:ONBOARDX.DESC,$
           ONBOARDYDATA:ONBOARDY.DATA,$
           ONBOARDYDESC:ONBOARDY.DESC,$
           EV_RECOGDATA:EV_RECOG.DATA,$
           EV_RECOGDESC:EV_RECOG.DESC,$
           EV_DETECDATA:EV_DETEC.DATA,$
           EV_DETECDESC:EV_DETEC.DESC,$
           EV_VALIDDATA:EV_VALID.DATA,$
           EV_VALIDDESC:EV_VALID.DESC,$
           BKGRCDDATA:BKGRCD.DATA,$
           BKGRCDDESC:BKGRCD.DESC,$
           BKGPCDDATA:BKGPCD.DATA,$
           BKGPCDDESC:BKGPCD.DESC,$
           BKGULCDDATA:BKGULCD.DATA,$
           BKGULCDDESC:BKGULCD.DESC }
  END
  
  DetRead,unit,header,a,selection,status=status   ; Read the actual data.
  IF N_elements(status)	gt 0 THEN if status(0) ne '' THEN goto,error_close_exit
  
  fxbClose,unit                                        
  
  IF keyword_set(delete_after_read) THEN file_delete,use_file
  
  QLMGR,a
  
  ;; Pad out all windows to same size.
  
  cds_pad_qlds,a
  
  IF Keyword_SET(display) THEN dsp_INFO,a; ,_EXTRA=extra
  
  dummy = load_wavecal(a.header.detector,anytim2utc(a.header.date_obs), $
                       cal_struct = a.wavecal)
  
  upd_cds_point,a,_extra=_extra,errmsg=status

;  Add tags for ROTATED and ALIGNED to support level-1 files.

  if not tag_exist(a, 'ROTATED') then $
    a = add_tag(a, fxpar(header,'ROTATED'), 'ROTATED')
  if not tag_exist(a, 'ALIGNED') then $
    a = ADD_TAG(A, fxpar(header,'ALIGNED'), 'ALIGNED')
  
  return,a
  
  error_close_exit:
  IF fxbisopen(unit) THEN fxbClose,unit
  
  IF keyword_set(delete_after_read) THEN rm_file,use_file
  
  return,0
END
