;+
; Project     : SOHO - CDS     
;                   
; Name        : STORE_WND
;               
; Purpose     : Store data from read_cds into a CDS QLDS
;               
; Explanation : Stores detector data from Data.Array into:
;
; 			Det.DetData(currentix:stopix 
;					[,0:size_dimension_2 
;					[,0:size_dimension_3]] )
;
; 		The number of indexes depends on the dimensionality of the 
;		data.
;
;		- Updates CurrentIx to point to the next free index.
;
; 		- Stores indexes (to Det.DetData) for this data window into
; 		Det.DetDesc(WndNo).ixstart/ixstop
;               
; Use         : STORE_WND,WINDOW_NO,CURRENT_INDEX,DET,DATA
;    
; Inputs      : WINDOW_NO : 
;			The index (into DET.DETDESC) of the
;			detector data window in question.
;               
;		CURRENT_INDEX: 
;			The index (into DET.DETDATA's first dimension) of
;			where a block of detector data may be placed.
;
;		DET :	An IDL structure containing the tags DetDesc and
;			DetData, as described in the CDS QL Data Format.
;
;		DATA:	An IDL structure returned from READ_CDS after
;			a successful read of detector data.
;
;		DDIMS:  The number of dimensions in the data block
;
; Opt. Inputs : None.
;               
; Outputs     : CURRENT_INDEX:
;			is updated to point to the room after
;			the current block of data in DET.DETDATA
;               
;		DET.DETDATA: The data from DATA.ARRAY i stored here under
;		             the old storage system. The data is stored
;		             as handle values (detdesc.handle) under the
;		             new system.
;
;		DET.DETDESC.IXSTART,
;		DET.DETDESC.IXSTOP: 
;			Indexes into DET.DETDATA for the current detector
;			data window. 
;
; Opt. Outputs: None.
;               
; Keywords    : None.
;
; Calls       : datatype(), detdesc(), handle_create(), handle_info()
;               ql_size(), tag_exist()
;
; Common      : None.
;               
; Restrictions: See the data formats.
;               
; Side effects: None.
;               
; Category    : Data_Handling, CDS, QuickLook, Internal
;               
; Prev. Hist. : None.
;
; Written     : Stein Vidar Hagfors Haugan, 27 September 1993
;               
; Modified    : SVHH, 20-Feb-1995: Uing QL_SIZE() instead of 
;			SIZE() to stop trailing singular dimensions
;			from disappearing.
;		Version 2, SVHH, 21-Aug-1995
;			Modified the call to QL_SIZE to save
;			memory.
;               Version 3, SVHH, 12 August 1996
;                       New version compatible with both old and new
;                       storage system.
;               Version 4, SVHH, 1 October 1996
;                       Improved speed in calculation of min/max values.
;
; Version     : 4, 1 October 1996
;-            
;

PRO Store_Wnd,WndNo,CurrentIx,Det,Data
  on_error,2

  if !DEBUG ne 0 then on_error,0
  
  
  IF datatype(data.array) NE 'FLO' THEN array = float(data.array) $
  ELSE array = data.array
  
  ma = max(data.array,min=mi)
  
  ;; If either (but not both) these are affected by MISSING values,
  ;; we must recompute them.
  
  missing = det.detdesc(wndno).missing
  
  IF (mi EQ missing) XOR (ma EQ missing) THEN BEGIN
     missix = where(data.array EQ missing)
     IF ma EQ missing THEN BEGIN
        ;; Max value affected by missing value
        data.array(missix) = mi
        ma = max(data.array) 
     END ELSE BEGIN
        ;; Minimum value affected by missing value
        data.array(missix) = ma
        mi = min(data.array)
     END 
  END
  
  det.detdesc(wndno).max = ma
  det.detdesc(wndno).min = mi
  
  
  IF 1 OR tag_exist(det,"DETDATA") THEN BEGIN
     Dims = N_elements(data.axes)
     SZ = ql_size(SIZE(data.array),dims)
     StopIx = CurrentIx + SZ(1)-1
     ixstart = [CurrentIx] & ixstop=[StopIx]
     IF dims gt 1 THEN BEGIN
        ixstart = [ixstart,intarr(dims-1)]
        ixstop =  [ixstop,sz(2:dims)-1]
     ENDIF
     det.detdesc(wndno).ixstart = ixstart
     det.detdesc(wndno).ixstop = ixstop
     st_windata,det,wndno,array,/no_copy
     CurrentIx=StopIx+1         ; Update current index.
  END ELSE BEGIN
     IF handle_info(det.detdesc(WndNo).handle,/valid_id) EQ 0 THEN $
        det.detdesc(WndNo).handle = handle_create()
     sa = SIZE(data.array)
     
     det.detdesc(WndNo).ixstop = sa(1:sa(0))-1
     handle_value,det.detdesc(WndNo).handle,array,/set,/no_copy
  END
END

