;+
; Project     :	SOHO - CDS
;
; Name        :	VALIDCDSHEAD()
;
; Purpose     :	Check FITS file header for validity in SFDU registration file.
;
; Explanation :	This routine is called by the CHECKCDSFITS procedure and 
;		checks to make sure that all of the FITS header keywords are
;		registered with the NSSDCA through the SFDU registration
;		files.  It also checks to see that the CDS SFDUADID keyword
;		value is correct (='NSSD0229').
;
; Use         :	RESULT = VALIDCDSHEAD( HEADER, [MESSG, KEYREG] )
;
; Inputs      :	HEADER:  The extended header of the FITS file.
;
; Opt. Inputs :	None.
;
; Outputs     :	RESULT:	STRING (scalar) is output:
;				'valid' if file meets criterion test,
;				'invalid' if it fails.
;
; Opt. Outputs:	MESSG:	A detailed message describing why the FITS header is
;			invalid (STRING, can be scalar or array depending on
;			number of errors found).
;
;		KEYREG:	A complete listing of all the registered SFDU FITS 
;			file keywords is returned (STRING).
;
; Keywords    :	ERRMSG:	If defined and passed, then any error messages will 
;			be returned to the user in this parameter rather than
;			being handled by the IDL MESSAGE utility.  If no 
;			errors are encountered, then a null string is 
;			returned.  In order to use this feature, the string 
;			ERRMSG must be defined first, e.g.,
;
;				ERRMSG = ''
;				RES = VALIDCDSHEAD( HEADER, ERRMSG=ERRMSG)
;				IF ERRMSG(0) NE '' THEN ...
;
; Calls       :	DATATYPE(), SFDU_GETREG, SXPAR
;
; Common      :	None.
;
; Restrictions:	Only one FITS file header can be sent at a time.
;
; Side effects:	None.
;
; Category    :	Data Handling, I/O, FITS, SFDU, CDS, Quicklook
;
; Prev. Hist. :	None.
;
; Written     :	Donald G. Luttermoser, GSFC/ARC, 26 October 1995
;
; Modified    :	Version 1, Donald G. Luttermoser, GSFC/ARC, 20 December 1995
;			Initial program.
;
; Version     :	Version 1,  20 December 1995.
;
;-
;
FUNCTION VALIDCDSHEAD, HEADER, MESSG, KEYREG, ERRMSG=ERRMSG
;
ON_ERROR, 2   ; Return to caller if error is encountered.
ON_IOERROR, NOFILE	; Go here if error in reading SFDU file.
MESSAGE = ''  ; Set to non-null string if error is encountered.
;
; Check the input.
;
IF (N_PARAMS() EQ 0) OR (N_PARAMS() GT 3) THEN BEGIN
	MESSAGE = 'Syntax:  VALIDCDSHEAD, HEADER [, MESSG, KEYREG]'
	GOTO, HANDLE_ERROR
ENDIF
;
IF N_PARAMS() GE 2 THEN EXPLAIN = 'Y' ELSE EXPLAIN = 'N'
IF N_PARAMS() EQ 3 THEN OUTKEY = 'Y' ELSE OUTKEY = 'N'
;
IF DATATYPE(HEADER, 1) NE 'String' THEN BEGIN
	MESSAGE = 'HEADER must be a string array containing the FITS header.'
	GOTO, HANDLE_ERROR
ENDIF
;
RESULT = 'valid'
IF EXPLAIN EQ 'Y' THEN MESSG = ''
;
; Get the SFDU registration file information.
;
SFDU_GETREG, ADID_SFDU, ERRMSG=MESSAGE
IF MESSAGE NE '' THEN GOTO, HANDLE_ERROR
;
; Get a listing of all the NSSDC registered FITS/PVL keywords from the
; FITS File Description registration file.
;
IFITS = -1
FOR I = 0, N_ELEMENTS(ADID_SFDU)-1 DO IF ADID_SFDU(I).NAME EQ 'FITS' THEN $
	IFITS = I
ADID = ADID_SFDU(IFITS).CAID + ADID_SFDU(IFITS).DDID
KEYWD  = ''
KEYOUT = ''
DUMMY  = ''
;
OPENR, URD, ADID_SFDU(IFITS).FILE, /GET_LUN  ; Open the SFDU registration file.
;
; This line should always be encountered before getting to the end of the file.
;
WHILE STRMID(DUMMY, 0, 21) NE 'FITS Header Keywords:' DO READF, URD, DUMMY
;
; Found the region with the keyword definitions.
;
WHILE NOT EOF(URD) DO BEGIN
	READF, URD, DUMMY
	DUMMY = STRTRIM(STRCOMPRESS(DUMMY), 2)
	EQPOS = STRPOS(DUMMY, '= /*')
	IF EQPOS NE -1 THEN BEGIN
;
; Modify the column header keywords:  They appear as "TYYYYn" in the SFDU
; registration, but in the actual header, "n" is an integer from 1 through
; some arbitrary value.  Here, we will assume that "n" will get no bigger
; than "99", this may have to be changed to something more sophisticated in
; the future.
;
		KHD = STRTRIM(STRMID(DUMMY, 0, EQPOS-1), 2)
;
; Save keywords for output if wanted.
;
		KEYOUT = [KEYOUT, KHD]
;
; See if the keyword is an extension column keyword.
;
		KLEN = STRLEN(KHD)
		TPOS = STRPOS(KHD, 'T')
		NPOS = STRPOS(KHD, 'n')
		IF (TPOS EQ 0) AND (NPOS EQ KLEN-1) THEN BEGIN
			KEYROOT = STRMID(KHD, 0, KLEN-1)
			KHD = KEYROOT + '1'
			FOR J = 2, 99 DO KHD = [KHD, $
				KEYROOT+STRTRIM(STRING(J),2)]
		ENDIF
;
; Store either the main header keyword scalar or the column header keyword
; array.
;
		KEYWD = [KEYWD, KHD]
	ENDIF
ENDWHILE
;
FREE_LUN, URD		; Close the SFDU registration file.
;
KEYWD  = KEYWD(WHERE(KEYWD NE ''))
KEYOUT = KEYOUT(WHERE(KEYOUT NE ''))
IF OUTKEY EQ 'Y' THEN KEYREG = KEYOUT
;
; Check to make sure that the INSTRUME keyword is set to 'CDS'.
;
CDSERR = ''
INSTRUME = SXPAR(HEADER, 'INSTRUME')
IF !ERR EQ -1 THEN CDSERR = 'INSTRUME keyword not found in FITS header!'
IF DATATYPE(INSTRUME, 1) NE 'String' THEN BEGIN
	CDSERR = 'INSTRUME keyword is not a string in FITS header!'
ENDIF ELSE BEGIN
	IF STRUPCASE(STRTRIM(INSTRUME, 2)) NE 'CDS' THEN $
		CDSERR = "INSTRUME keyword value not set to 'CDS'."
ENDELSE
IF CDSERR NE '' THEN BEGIN
	IF EXPLAIN EQ 'Y' THEN $
		IF MESSG(0) EQ '' THEN MESSG = CDSERR $
		ELSE MESSG = [MESSG, CDSERR]
	RESULT = 'invalid'
ENDIF
;
; Check to make sure that the SFDUADID keyword value is what it should be.
;
SFDUERR = ''
SFDUADID = SXPAR(HEADER, 'SFDUADID')
IF !ERR EQ -1 THEN SFDUERR = 'SFDUADID keyword not found in FITS header!'
IF DATATYPE(SFDUADID, 1) NE 'String' THEN BEGIN
	SFDUERR = 'SFDUADID keyword is not a string in FITS header!'
ENDIF ELSE BEGIN
	IF STRTRIM(SFDUADID, 2) NE ADID THEN $
		SFDUERR = ['SFDUADID keyword value (' + SFDUADID + $
		') is inconsistent with the SFDU registration', $
		'  file value (' + ADID + ').']
ENDELSE
IF SFDUERR(0) NE '' THEN BEGIN
	IF EXPLAIN EQ 'Y' THEN $
		IF MESSG(0) EQ '' THEN MESSG = SFDUERR $
		ELSE MESSG = [MESSG, SFDUERR]
	RESULT = 'invalid'
ENDIF
;
; Now isolate the keywords of the extended header of the FITS file.
;
NHEAD = N_ELEMENTS(HEADER)
;
; Check to make sure that all of the header keywords are in the SFDU 
; registration files (as stored in KEYWD).
;
BADKEY = ''
FOR I = 0, NHEAD-1 DO BEGIN
	EQPOS = STRPOS(HEADER(I), '=')
	IF EQPOS NE -1 THEN BEGIN
		HD = STRTRIM(STRMID(HEADER(I), 0, EQPOS),2)
		ICK = WHERE(KEYWD EQ HD)
		IF ICK(0) EQ -1 THEN BEGIN
			RESULT = 'invalid'
			BADKEY = BADKEY + HD + ', '
		ENDIF
	ENDIF
ENDFOR
;
IF EXPLAIN EQ 'Y' THEN BEGIN
	IF BADKEY NE '' THEN BEGIN
;
; Trim off trailing comma and chop it up if too long:
;
		SLEN = STRLEN(BADKEY)
		BADKEY = STRMID(BADKEY, 0, SLEN-2) + '.'
		BMAXLEN = 60
		IF SLEN GT BMAXLEN THEN BEGIN
			BLKPOS = STRPOS(BADKEY, ' ', BMAXLEN)
			TMPMESS = STRMID(BADKEY, 0, BLKPOS)
			WHILE BLKPOS GT -1 DO BEGIN
				POS0 = BLKPOS + 1
				REMAIN = STRMID(BADKEY, POS0, SLEN-POS0)
				RLEN = STRLEN(REMAIN)
				BLKPOS = STRPOS(BADKEY, ' ', POS0+BMAXLEN)
				IADD = 0
				IF BLKPOS EQ -1 THEN IADD = 1
				IF RLEN LE BMAXLEN THEN IADD = 1
				IF IADD EQ 0 THEN TMPMESS = [TMPMESS, $
					STRMID(BADKEY, POS0, BLKPOS)] $
				ELSE TMPMESS = [TMPMESS, REMAIN]
			ENDWHILE
			BADKEY = TMPMESS
		ENDIF
;
		IF MESSG(0) EQ '' THEN MESSG = BADKEY $
			ELSE MESSG = [MESSG, BADKEY]
	ENDIF
ENDIF
;
IF EXPLAIN EQ 'Y' THEN $
	IF MESSG(0) EQ '' THEN MESSG = 'All FITS file keywords are valid.'
IF MESSAGE NE '' THEN GOTO, HANDLE_ERROR
IF N_ELEMENTS(ERRMSG) GT 0 THEN ERRMSG=MESSAGE
RETURN, RESULT
;
; SFDU file reading error occurred.
;
NOFILE:
FREE_LUN, URD
IF EXPLAIN EQ 'Y' THEN IF MESSG(0) EQ '' THEN MESSG = $
	'Error in reading the SFDU registration file ' + ADID_SFDU(IFITS).FILE
MESSAGE = 'Error in reading the SFDU registration file.'
;
; Error handling portion of the procedure.
;
HANDLE_ERROR:
;
IF EXPLAIN EQ 'Y' THEN $
	IF MESSG(0) EQ '' THEN MESSG = 'Fatal error occurred in VALIDCDSHEAD.'
IF N_ELEMENTS(ERRMSG) EQ 0 THEN MESSAGE, MESSAGE
ERRMSG = MESSAGE
RETURN, RESULT
;
END
