;+
; Project     :	SOHO - CDS
;
; Name        :	VALIDCDSTYPE
;
; Purpose     :	Makes a structure containing valid FITS extension column types.
;
; Explanation :	This procedure creates a structure that contains information
;		regarding the valid TYPES of columns in a CDS binary table
;		FITS file.
;
; Use         :	VALIDCDSTYPE, TCOLM, AXES
;
; Inputs      :	None.
;
; Opt. Inputs :	None.
;
; Outputs     :	TCOLM:	Structure containing valid FITS column information.
;			It is named "TCOLM" and has the following tags:
;			TYPE:	Type of data associated with the binary table
;				column.  Essentially a main FITS header 
;				keyword.
;			UNIT:	The units of the data: 'counts', 'counts/sec',
;				'second', 'arcsec', etc.
;			NDIM:	Minimum number of dimensions allowed for the 
;				data (note that the maximum is 4 dimensions).
;			IDLTYPE:	Type of data values in terms of IDL
;				"type" numbers.
;			QWAVE:	If set to 1, then first dimension of a 3 or 4
;				dimensional array is wavelength (WAVELNTH).
;				If set to 0, then the data in question is 
;				loaded into either a 1-D or 2-D array or the
;				first dimension in a 3 or 4 dimensional array
;				is associated with a detector (DETECTOR).
;
;		AXES:	Units for multi-dimensional data.  The array indicies
;			must conform to the following units:
;			 1-D:	any units,
;			 2-D:	(SOLAR_X,SOLAR_Y),
;			 3-D:	(WAVELNTH,SOLAR_X,SOLAR_Y), or
;				(DETECTOR,SOLAR_X,SOLAR_Y),
;			 4-D:	(WAVELNTH,SOLAR_X,SOLAR_Y,DEL_TIME), or
;				(DETECTOR,SOLAR_X,SOLAR_Y,DEL_TIME).
;
; Opt. Outputs:	None.
;
; Keywords    :	ERRMSG:	If defined and passed, then any error messages will 
;			be returned to the user in this parameter rather than
;			being handled by the IDL MESSAGE utility.  If no 
;			errors are encountered, then a null string is 
;			returned.  In order to use this feature, the string 
;			ERRMSG must be defined first, e.g.,
;
;				ERRMSG = ''
;				VALIDCDSTYPE, TCOLM, AXES, ERRMSG=ERRMSG
;				IF ERRMSG(0) NE '' THEN ...
;
; Calls       :	None.
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Category    :	Data Handling, I/O, FITS, SFDU, CDS, Quicklook
;
; Prev. Hist. :	None.
;
; Written     :	Donald G. Luttermoser, GSFC/ARC, 21 December 1995
;
; Modified    :	Version 1, Donald G. Luttermoser, GSFC/ARC, 21 December 1995
;			Initial program.
;
; Version     :	Version 1,  21 December 1995.
;
;-
;
PRO VALIDCDSTYPE, TCOLM, AXES, ERRMSG=ERRMSG
;
ON_ERROR, 2 	; Return to caller if error is encountered.
MESSAGE = ''	; Set to non-null string if error is encountered.
;
; Check the input parameters.
;
IF N_PARAMS() NE 2 THEN BEGIN
	MESSAGE = 'Syntax:  VALIDCDSTYPE, TCOLM, AXES'
	GOTO, HANDLE_ERROR
ENDIF
;
; Units for a given dimensioned array:
;
AXES = ['anything', '(SOLAR_X,SOLAR_Y)', '(WAVELNTH,SOLAR_X,SOLAR_Y)', $
	'(DETECTOR,SOLAR_X,SOLAR_Y)', '(WAVELNTH,SOLAR_X,SOLAR_Y,DEL_TIME)', $
	'(DETECTOR,SOLAR_X,SOLAR_Y,DEL_TIME)']
;
; Definition of column keyword values.
;
NTCOL = 25	; Number of column types.
;
TCOLM = REPLICATE({TCOLM, TYPE: '', UNIT: '', NDIM: 0, IDLTYPE: 0, QWAVE: 0}, $
	NTCOL)
;
; The first entry ("WW_") represents the primary spectral data and can have 
; more than one entry (i.e., WW_417_5, WW_170_9, WW_721_5, etc.).
; The second entry ("BB_GIS") represents primary data from the GIS.
;
TCOLM(0)  = {TCOLM, 'WW_',      'Counts',     3,  3,  1 }
TCOLM(1)  = {TCOLM, 'BB_GIS',   'Counts',     3,  3,  1 }
TCOLM(2)  = {TCOLM, 'DEL_TIME', 'second',     1,  5,  0 }
TCOLM(3)  = {TCOLM, 'INS_X   ', 'arcsec',     1,  4,  0 }
TCOLM(4)  = {TCOLM, 'INS_Y   ', 'arcsec',     1,  4,  0 }
TCOLM(5)  = {TCOLM, 'OPSLBITS', 'code',       1,  1,  0 }
TCOLM(6)  = {TCOLM, 'OPS_L   ', 'raw',        1,  2,  0 }
TCOLM(7)  = {TCOLM, 'OPSRBITS', 'code',       1,  1,  0 }
TCOLM(8)  = {TCOLM, 'OPS_R   ', 'raw',        1,  2,  0 }
TCOLM(9)  = {TCOLM, 'SLITBITS', 'code',       1,  1,  0 }
TCOLM(10) = {TCOLM, 'SLIT_POS', 'raw',        1,  2,  0 }
TCOLM(11) = {TCOLM, 'MIR_POS ', 'raw',        1,  1,  0 }
TCOLM(12) = {TCOLM, 'PTCHBITS', 'code',       1,  1,  0 }
TCOLM(13) = {TCOLM, 'PITCH   ', 'raw',        1,  2,  0 }
TCOLM(14) = {TCOLM, 'YAW_BITS', 'code',       1,  1,  0 }
TCOLM(15) = {TCOLM, 'YAW     ', 'raw',        1,  2,  0 }
TCOLM(16) = {TCOLM, 'ONBOARDX', 'arcsec',     1,  2,  0 }
TCOLM(17) = {TCOLM, 'ONBOARDY', 'arcsec',     1,  2,  0 }
TCOLM(18) = {TCOLM, 'EV_LEVEL', 'code',       1,  3,  0 }
TCOLM(19) = {TCOLM, 'EV_RECOG', 'logical',    1,  1,  0 }
TCOLM(20) = {TCOLM, 'EV_DETEC', 'logical',    1,  1,  0 }
TCOLM(21) = {TCOLM, 'EV_VALID', 'logical',    1,  1,  0 }
TCOLM(22) = {TCOLM, 'BKGRCD  ', 'counts/sec', 3,  4,  0 }
TCOLM(23) = {TCOLM, 'BKGPCD  ', 'counts/sec', 3,  4,  0 }
TCOLM(24) = {TCOLM, 'BKGULCD ', 'counts/sec', 3,  4,  0 }
;
IF N_ELEMENTS(ERRMSG) GT 0 THEN ERRMSG=MESSAGE
RETURN
;
; Error handling portion of the procedure.
;
HANDLE_ERROR:
;
IF N_ELEMENTS(ERRMSG) EQ 0 THEN MESSAGE, MESSAGE
ERRMSG = MESSAGE
RETURN
;
END
