;+
; Project     : SOHO - CDS     
;                   
; Name        : CAT_DURATION
;               
; Purpose     : Lists the durations of rasters performed on the CDS.
;               
; Explanation : Reads the start and end times from the FITS file header
;               and lists the duration together with the Study ID
;               
; Use         : IDL> cat_duration, study_num
;    
; Inputs      : study_num - the study number of the FITS files to be processed
;                           if not given then all files are processed. 
;               
; Opt. Inputs : None
;               
; Outputs     : data are written to the file study_duration.list
;               
; Opt. Outputs: None
;               
; Keywords    : QUIET - if given then no output to screen
;
; Calls       : HEADFITS, FIND_FILES
;
; Common      : None
;               
; Restrictions: None
;               
; Side effects: None
;               
; Category    : Telemetry, FITS
;               
; Prev. Hist. : None
;
; Written     : C D Pike, RAL, 9-May-96
;               
; Modified    : Use CDSHEADFITS.   CDP, 23-Aug-96
;
; Version     : Version 2, 23-Aug-96
;-            

pro cat_duration, fnum, quiet=quiet

;
;  default = all, otherwise only the file number asked for
;
if n_params() eq 0 then begin
   ff = find_files('*.fits','$CDS_FITS_DATA')
endif else begin
   ff = find_with_def('s'+trim(fnum)+'r00','$CDS_FITS_DATA','.fits')
endelse
if ff(0) eq '' then begin
   print,'No files found'
   return
endif

;
;  check if OK to do large number
;
if n_elements(ff) gt 50 then begin
   bell
   print,trim(n_elements(ff))+' files are involved.'
   yes_no,'Ok to continue?',ok,'Y'
   if not ok then return
endif 
   

;
;  output string array for details
;
output = strarr(n_elements(ff))

;
;  loop through each file
;
for i=0,n_elements(ff)-1 do begin
   h = cdsheadfits(ff(i))
   t0 = utc2tai(anytim2utc(str_pick(h(17),"'","'")))
   t1 = utc2tai(anytim2utc(str_pick(h(18),"'","'")))
   rid = fix(str_pick(h(15),'=','/'))
   rv = fix(str_pick(h(16),'=','/'))
   errmsg = ''
   get_raster,rid,rv,r,errmsg=errmsg
   if r.ras_id eq rid then begin
      theor = trim(long(r.duration))
   endif else begin
      theor = '???'
   endelse
   break_file,ff(i),disk,dir,f,ext
   output(i)  = strpad(f,12,/after) + str_pick(h(28),"'","'") + $
                 '  Actual/Expected ---> '+trim(long(t1-t0)) +' / '+theor+' secs'
endfor

;
;  on screen info?
;
if not keyword_set(quiet) then begin
   print,' '
   print_str,output
endif

;
;  save output
;
print_str,output,/keep,file='study_duration.list',/quiet
end
