;+
; Project     : SOHO - CDS     
;                   
; Name        : EZMGN
;               
; Purpose     : To fit multiple Gaussians to NIS data and compare to CHIANTI
;               
; Explanation : A front end to D Zarro's multiple Gauss fitting routines
;               that also allows a visual comparison with a CHIANTI synthetic
;               spectrum.  (ie Easy Multiple Gauss, NIS) cf EZFIT
;               A maximum of 20 profiles can be fitted simulstaneously.
;               
; Use         : IDL> ezmg, w, sp
;    
; Inputs      : w  - wavelength array (as returned eg by pix2wave) 
;               sp - the spectral data
;               
; Opt. Inputs : chfile - if supplied the CHIANTI spectrum is loaded from
;                        there rather than the default.
;               
; Outputs     : Results listed to screen and file "ezmg.result" if wanted.
;               
; Opt. Outputs: None
;               
; Keywords    : WAVELENGTH - range of wavelengths to plot
;               AR         - use CHIANTI active region data (default = QS)
;               WIDTH      - fix width of line profiles to this value
;               BACKFIX    - fix background to cursor value
;               HARDCOPY   - do hardcopy of the screen plots
;               ILIMIT     - set limit for intensity of identified lines
;                            in CHIANTI spectrum (default = 1.0e10)
;               HELP       - give instructions on use of cursor
;               NO_ID      - do not put line IDs on hardcopy output.
;
; Calls       : FITSPEC
;
; Common      : None
;               
; Restrictions: The cursor interaction is querky as follows 
;
;               (LEFT or MIDDLE buttons only)
;
;               1) when the spectrum is displayed you can either set a fixed
;                  or first guess background level with the first click 
;                  (whether it is a fixed parameter or not is determined by
;                  the /BACKFIX keyword)   .....  or  ....
;
;               2) if the first 2 clicks are below the X-axis they will be 
;                  taken to redefine the range of data to be fitted (if this
;                  option is taken the NEXT click will determine the
;                  background).
;
;               3) thereafter click at the peak (wavelength and intensity
;                  important) to define each line you want fitted.
;
;               4) when done, click to the left of the Y axis to indicate
;                  you are finished.
;
;               (RIGHT button only)
;              
;               5) If pressed at any time within the DATA plot will give a 
;                  possible ID of the line.
;
;               6) if pressed to left of Y-axis will ABORT the procedure (ie
;                  no fitting is attempted)
;
;               NB if you just want a hardcopy of the plots with no fitting
;               then enter ezmgn with the /HARD keyword and immediately click
;               the left button to the left of the Y-axis.  
;               
; Side effects: None
;               
; Category    : Spectral analysis
;               
; Prev. Hist. : None
;
; Written     : C D Pike, RAL, 15-Oct-96
;               
; Modified    : Include errors in hardcopy.  CDP, 4-Feb-97
;               Fix bug in total intensity calculation.  CDP, 28-Feb-97
;               Add help and clean plot at end.  CDP, 5-Mar-97
;               Allow data to be negative and make line identification
;               on hardcopy optional.   CDP, 13-Jun-97
;		Version 6, 08-Jul-1997, William Thompson, GSFC
;			Changed format statements to allow larger dynamic range
;			Ignore sign of width
;               Explicitly trap > 20 profiles.  CDP, 1-Sep-97
;
; Version     : Version 7, 01-Sep-1997
;-            

pro ezmgn, xx, yy, chfile, wavelength=wavelength, width=width, $
                  backfix=backfix,help=help,no_id=no_id,$
                  ar=ar, hardcopy=hardcopy, ilimit=ilimit

common ezmgn_com, cch, band, intensity, info, datamax, nuse, active, line_id

;
;  help with cursor input if not silenced
;
if keyword_set(help) then begin
   print,'The cursor interaction is querky as follows 
;
   print,'         (LEFT or MIDDLE buttons only)
;
   print,'         1) when the spectrum is displayed either set a fixed
   print,'            or first guess background level with the first click 
   print,'            (/BACKFIX determines whether fixed or not'
   print,'            .....  or  ....
;
   print,'         2) if the first 2 clicks are below the X-axis they will be 
   print,'            taken to redefine the range of data to be fitted (if this
   print,'            option is taken the NEXT click will determine the
   print,'            background).
;
   print,'         3) thereafter click at the peak (wavelength and intensity
   print,'            important) to define each line you want fitted.
;
   print,'         4) when done, click to the left of the Y axis to indicate
   print,'            you are finished.
;
   print,'        
   print,'         (RIGHT button only)
   print,'         5) If pressed at any time within the DATA plot will give a 
   print,'            possible ID of the line.
;
   print,'         6) if pressed to left of Y-axis will ABORT the procedure (ie
   print,'            no fitting is attempted)
;
   print,'         NB if you just want a hardcopy of the plots with no fitting
   print,'         then enter ezmgn with the /HARD keyword and immediately click
   print,'         the left button to the left of the Y-axis.  

endif

;
;  was specific wavelength interval given?
;
if not keyword_set(wavelength) then begin
   wavelength = fltarr(2)
   wavelength(0) = min(xx)
   wavelength(1) = max(xx)
endif   


;
;  decide which spectral band is in use
;
if max(xx) lt 500 then nis = 1 else nis = 2

;
;  was intensity limit specified?
;
if n_elements(ilimit) eq 0 then ilimit = 1.0e10

;
;  has anything changed?
;

have_data = 0
if n_elements(cch) gt 0 and n_elements(band) gt 0 then begin
   if (band eq nis) and $
     ( (keyword_set(ar) and (active eq 1)) or $
       (not keyword_set(ar) and (active eq 0))) then have_data  = 1
endif

;
;  if give file name then use it
;
if n_elements(chfile) gt 0 then begin
   print,'Loading file: '+chfile
   cch = gt_chianti(concat_dir('$CDS_ATOMIC',chfile),/br)
   band = nis
   active = keyword_set(ar)
   have_data = 1
   intensity = float(strmid(cch.list_ident,68,10))
   line_id   = strmid(cch.list_ident,0,10)
   info = strarr(n_elements(intensity))
   for i=0,n_elements(intensity)-1 do begin
      info(i) = strpad(trim(cch.list_wvl(i)),10,/after) + ' --> ' + $
                strpad(strmid(cch.list_ident(i),0,10),10,/after) + $
                ' Intensity: ' + ' ' + string(intensity(i),form='(e9.3)')
   endfor
endif

;
;  if not been here before
;
if not have_data then begin
;
;  load appropriate CHIANTI spectrum
;
   if keyword_set(ar) then begin
      if nis eq 1 then begin
         print,'Loading CHIANTI file: '+'nis1_ar_mc_ar'
         ch = gt_chianti(concat_dir('$CDS_ATOMIC','nis1_ar_mc_ar'),/br)
      endif else begin
         print,'Loading CHIANTI file: '+'nis2_ar_mc_ar'
         ch = gt_chianti(concat_dir('$CDS_ATOMIC','nis2_ar_mc_ar'),/br)
      endelse
   endif else begin
      if nis eq 1 then begin
         print,'Loading CHIANTI file: '+'nis1_qs_mc_ar'
         ch = gt_chianti(concat_dir('$CDS_ATOMIC','nis1_qs_mc_ar'),/br)
      endif else begin
         print,'Loading CHIANTI file: '+'nis2_qs_mc_ar'
         ch = gt_chianti(concat_dir('$CDS_ATOMIC','nis2_qs_mc_ar'),/br)
      endelse
   endelse

;
;  various housekeeping bits
;
   intensity = float(strmid(ch.list_ident,68,10))
   line_id   = strmid(ch.list_ident,0,10)
   info = strarr(n_elements(intensity))
   for i=0,n_elements(intensity)-1 do begin
      info(i) = strpad(trim(ch.list_wvl(i)),10,/after) + ' --> ' + $
                strpad(strmid(ch.list_ident(i),0,10),10,/after) + $
                ' Intensity: ' + ' ' + string(intensity(i),form='(e9.3)')
   endfor
 
;
;  store for next time
;
   cch = ch
   band = nis
   active = keyword_set(ar)
endif else begin
   ch = cch
   print,' '
   print,'Using already loaded CHIANTI spectrum...'
   help,ch,/str
   print,' '
endelse

;
;  set up general list of lines
;
nuse = where( (ch.list_wvl gt wavelength(0)) and $
              (ch.list_wvl lt wavelength(1)) and $
               intensity gt ilimit, countnn)
if countnn gt 0 then datamax = max(intensity(nuse)) else datamax = 1.0e10

;
;  was width fixed?
;
if keyword_set(width) then begin 
   fix_wid = 1
endif else begin
   fix_wid=0
   width = xx(5)-xx(0)
endelse

;
;  if no wavelength range given, use all of it
;
if not keyword_set(wavelength) then wavelength = [xx(0),last_item(xx)]
wavelength = float(wavelength)

;
;  max of 20 profiles (+ 3 background parameters)
;
aa = fltarr(63)-999.0

;
;  pick out data to be used from wavelength spec.
;
npix = where(xx ge wavelength(0) and xx le wavelength(1),count)
if count eq 0 then begin
   print,'No data'
   goto, tidyup
endif
x = xx(npix)
y = yy(npix)



;
;  plot the CHIANTI spectrum
;
!p.multi=[0,1,2]
plot,ch.lambda,ch.spectrum,psym=10,xr=wavelength,$
        tit='CHIANTI Synthetic Spectrum',/xst


;
;  plot the data
;
plot,x,y,psym=10,xticklen=-0.05,xr=wavelength,/xst,xtit='Wavelength (A)'


;
;  now the cursor interaction gets fiddly
;

;
;  Ask for background value (if fixed this is the value to use, if not
;  this is the first guess)
;

print,'Set background level (if necessary start by defining window again)'
rpt0:
   cursor,a,b,3,/data 
   if !err eq 4 then begin
      if a lt !x.crange(0) then goto, tidyup
      nw = where( (ch.list_wvl gt a-0.25) and (ch.list_wvl lt a+0.25) and $
                  intensity gt ilimit, count)
      if count gt 0 then begin
         print_str,info(nw),/num
         data = intensity(nw)
         id = line_id(nw)
         lam = float(ch.list_wvl(nw))
         data = (data/datamax)*0.90*!y.crange(1)
         for j=0,n_elements(nw)-1 do begin
            oplot,[lam(j),lam(j)],[0,data(j)],th=2
            xyouts,lam(j),data(j),id(j),chars=1.0
         endfor
         print,' '
      endif
      goto, rpt0
   endif

aa(0) = b
aa(1) = 0.
aa(2) = 0.

;
;  if cursor was to left of Y-axis then jump to end in case needed hardcopy
;
if a lt !x.crange(0) then begin
   goto, jump1
endif

;
;  if cursor was below X-axis then assume it is to redefine wavelength range
;  and need two inputs
;
if b lt !y.crange(0) then begin
   oplot,[a,a],[!y.crange(0),!y.crange(1)],thick=3
   wavelength(0) = a
rpt1:
   cursor,a,b,3,/data 
   if !err eq 4 then begin
      if a lt !x.crange(0) then goto, tidyup
      nw = where( (ch.list_wvl gt a-0.25) and (ch.list_wvl lt a+0.25) and $
                  intensity gt ilimit, count)
      if count gt 0 then begin
         print_str,info(nw),/num
         data = intensity(nw)
         lam = float(ch.list_wvl(nw))
         id = line_id(nw)
         data = (data/datamax)*0.90*!y.crange(1)
         for j=0,n_elements(nw)-1 do begin
            oplot,[lam(j),lam(j)],[0,data(j)],th=2
            xyouts,lam(j),data(j),id(j),chars=1.0
         endfor
         print,' '
      endif
      goto, rpt1
   endif

   oplot,[a,a],[!y.crange(0),!y.crange(1)],thick=3
   wavelength(1) = a
   if wavelength(0) gt wavelength(1) then begin
      temp = wavelength(0)
      wavelength(0) = wavelength(1)
      wavelength(1) = temp
   endif
   print,'OK set background level again'
rpt2:
   cursor,a,b,3,/data 
   if !err eq 4 then begin
      if a lt !x.crange(0) then goto, tidyup
      nw = where( (ch.list_wvl gt a-0.25) and (ch.list_wvl lt a+0.25) and $
                  intensity gt ilimit, count)
      if count gt 0 then begin
         print_str,info(nw),/num
         data = intensity(nw)
         lam = float(ch.list_wvl(nw))
         id = line_id(nw)
         data = (data/datamax)*0.90*!y.crange(1)
         for j=0,n_elements(nw)-1 do begin
            oplot,[lam(j),lam(j)],[0,data(j)],th=2
            xyouts,lam(j),data(j),id(j),chars=1.0
         endfor
         print,' '
      endif
      goto, rpt2
   endif

   aa(0) = b
   aa(1) = 0.
   aa(2) = 0.
   if a lt !x.crange(0) then begin
      goto, jump1
   endif
endif


;
;  check still have some data
;
npix = where(xx ge wavelength(0) and xx le wavelength(1),count)
if count eq 0 then begin
   print,'No data'
   goto, tidyup
endif
x = xx(npix)
y = yy(npix)


;
;  now collect the lines - end with cursor to left of Y-axis
;
a=10000 & b=0 & np=0
while a gt wavelength(0) do begin
   print,'Place cursor on Peak/Centre for line # ',np+1
rpt3:
   cursor,a,b,3,/data 
   if !err eq 4 then begin
      if a lt !x.crange(0) then goto, tidyup
      nw = where( (ch.list_wvl gt a-0.25) and (ch.list_wvl lt a+0.25) and $
                  intensity gt ilimit, count)
      if count gt 0 then begin
         print_str,info(nw),/num
         data = intensity(nw)
         lam = float(ch.list_wvl(nw))
         id = line_id(nw)
         data = (data/datamax)*0.90*!y.crange(1)
         for j=0,n_elements(nw)-1 do begin
            oplot,[lam(j),lam(j)],[0,data(j)],th=2
            xyouts,lam(j),data(j),id(j),chars=1.0
         endfor
         print,' '
      endif
      goto, rpt3
   endif

   aa(np*3+3) = b-aa(0)
   aa(np*3+4) = a
   aa(np*3+5) = width
   np = np+1
   if np gt 19 then begin
      bell
      print,'Maximum number of allowable profiles reached....'
      np = 21
      a = 0
   endif
endwhile

;
;  tidy up unused parameters
;  (unless limit reached last cursor input was not wanted)
;
nnn = where(aa gt -998)
if np lt 21 then begin
   aa = aa(0:n_elements(nnn)-4)
endif else begin
   aa = aa(0:n_elements(nnn)-1)
endelse
;
;  if background  was fixed then fix those parameters
;
if keyword_set(backfix) then begin
   fp = [0,1,2]
endif else begin
   fp = [0]
endelse

;
;  if width was fixed then fix those parameters
;
if fix_wid then begin
   for k=0,np-2 do fp = [fp,k*3+5]
   if not keyword_set(backfix) then fp = fp(1:*) 
endif 

;
;  do the fit
;

if total(fp) eq 0  then begin
   fit_spec,x,y,fx,fy,aa,ep,fit_funct='gauss'
endif else begin
   fit_spec,x,y,fx,fy,aa,ep,fit_funct='gauss',fixp=fp
endelse


;
;  calculate mean bin width
;
ww = x - shift(x,1)
bin_wid = average(ww(1:*))


;
;  calculate a more refined overplot
;
npix = fix((wavelength(1)-wavelength(0))*100.0)
sx = findgen(npix)/100.0 + wavelength(0)
sy = fltarr(npix)
nprof = (n_elements(aa)-3)/3
for i=0,nprof-1 do begin
  sy = sy + aa(i*3+3)*exp(-(sx-aa(i*3+4))*(sx-aa(i*3+4))/(aa(i*3+5)*aa(i*3+5)))
endfor
sy = sy + poly(sx,[aa(0),aa(1),aa(2)]) 
   
;
;  overplot the fit and background
;
oplot,sx,sy,th=2,line=2
oplot,x,poly(x,[aa(0),aa(1),aa(2)]) 

;
;  read in results so far, print to screen and ask if want appended to file
;
text = rd_ascii('ezmg.result')

print,' Line     Ht           Centre          Width       Intensity'
for i=0,np-2 do begin
   print,i+1,aa(i*3+3),aa(i*3+4),abs(aa(i*3+5)),$
             2.51*aa(i*3+3)*abs(aa(i*3+5))/sqrt(2.)/bin_wid,$
          form='(i3,2x,g10.4,2x,g12.6,5x,g10.4,6x,g10.4)'
   print,'+\-  ',ep(i*3+3),ep(i*3+4),ep(i*3+5),$
          form='(a5,g10.4,2x,g12.6,5x,g10.4)'
   print,' '
   t1 = string(i+1,aa(i*3+3),aa(i*3+4),abs(aa(i*3+5)),$
               2.51*aa(i*3+3)*abs(aa(i*3+5))/sqrt(2.)/bin_wid,$
          form='(i3,2x,g10.4,2x,g12.6,5x,g10.4,6x,g10.4)')
   t2 = string('+\-  ',ep(i*3+3),ep(i*3+4),ep(i*3+5),$
          form='(a5,g10.4,2x,g9.3,5x,g10.4)')
   print,' '
   text = [text,t1,t2,' ']
endfor
yes_no,'Save output?',ans,'N'
if ans eq 1 then wrt_ascii,text,'ezmg.result'


;
;  plot the CHIANTI spectrum
;
jump1:
if keyword_set(hardcopy) then begin
   ps
   !p.multi=[0,1,2]
   plot,ch.lambda,ch.spectrum,psym=10,xr=wavelength,$
           tit='CHIANTI Synthetic Spectrum',/xst

   plot,x,y,psym=10,xticklen=-0.05,/xst,xr=wavelength
   data = intensity(nuse)
   lam = float(ch.list_wvl(nuse))
   data = (data/datamax)*0.90*!y.crange(1)
   id = line_id(nuse)
   if not keyword_set(no_id) then begin
      for j=0,n_elements(nuse)-1 do begin
         oplot,[lam(j),lam(j)],[0,data(j)],th=2
         xyouts,lam(j),data(j),id(j),chars=1.0
      endfor
   endif
   if n_elements(sx) gt 0 then begin
      oplot,sx,sy,th=2,line=2
      oplot,x,poly(x,[aa(0),aa(1),aa(2)])
   endif
   psplot
endif

;
;  here to quit, but if file was specified better make sure we start from
;  scratch next time
;
tidyup:
if n_elements(chfile) gt 0 then band = 0

;
;  clean up the plotting
;
cleanplot

end

