;+
; Project     : SOHO - CDS     
;                   
; Name        : GET_AVGSPEC
;               
; Purpose     : Loads averaged example NIS spectrum.
;               
; Explanation : Loads wavelength and data arrays using Bill Thompson's 
;               averaged NIS spectra
;               
; Use         : IDL> get_avgspec, wave, data
;    
; Inputs      : None
;               
; Opt. Inputs : None
;               
; Outputs     : wave - wavelength array
;               data - spectral data array
;               
; Opt. Outputs: None
;               
; Keywords    : VDS   - If set, then returns data calibrated using VDS_CALIB.
;		CALIB - If set, then returns data calibrated using NIS_CALIB.
;
;		The default, if neither /VDS nor /CALIB are used, is to return
;		raw debiased spectrum
;
;               N2  - returns NIS2 spectrum, else NIS1
;
; Calls       : READFITS
;
; Common      : None
;               
; Restrictions: None
;               
; Side effects: None
;               
; Category    : Spectral, data analysis
;               
; Prev. Hist. : None
;
; Written     : C D Pike, RAL.  28-Nov-96 
;               
; Modified    : Name update.  CDP, 4-Dec-96
;		Version 3, 5-Mar-1997, William Thompson, GSFC
;			Added keyword /VDS
;			Use VDS_ROTATE and only return part within slit area.
;               Version 4, CDP, 13-May-97
;                       Scale raw data counts to match reality.  
;               Version 5, CDP Tidy first and last few pixels.  30-Oct-97
;
; Version     : Version 5, 30-Oct-1997
;-            
pro get_avgspec, w, data, calib=calib, vds=vds, n2=n2

on_error, 2
if n_params() ne 2 then message, 'Syntax:  GET_AVGSPEC, W, DATA'

;
;  scale factor for raw
;
scale = 1.0

if keyword_set(vds) then begin
	cal = 'vds_calib'
end else if keyword_set(calib) then begin
	cal = 'calib'
end else begin
	cal = 'debias'
        if keyword_set(n2) then scale = 81.0 else scale = 86.5
endelse

;
if keyword_set(n2) then begin
   nis = 'nis2' 
   w = pix2wave('n2',indgen(1024))
endif else begin
   nis = 'nis1'
   w = pix2wave('n1',indgen(1024))
endelse

data = readfits(concat_dir('$CDS_NIS','avg_qs_'+nis+'_'+cal+'.fts'))
vds_rotate,data,data,1+keyword_set(n2)
sz = size(data)
j1 = -65 + sz(2)/2
j2 = +65 + sz(2)/2
data = average(data(*,j1:j2),2,miss=-100)/scale

;
;  kill edge effects
;
data(0:2) = data(3)
data(1021:1023) = data(1020)

end
 
