;+
; Project     : SOHO - CDS     
;                   
; Name        : GT_DETY
;               
; Purpose     : Return detector Y guess (for NIS) given YIX and waveband.
;               
; Explanation : WARNING! Since there is no one-to-one relationship between
;               waveband and the starting detector Y value (with windows not
;               aligned to start at the same detector Y position), this
;               routine should only be used to get a "guess" at the detector Y
;               position whenever there is no detector window covering the
;               given pixel.
;
;               Actually, this routine was written on the wrong assumption
;               that the windows would be aligned, so don't use this when
;               other methods are available, like finding a window covering
;               the pixel with gt_window and then using information in the
;               QLDS.detdesc structure.
;
; Use         : dety=gt_dety(QLDS,YIX,BAND)
;    
; Inputs      : QLDS: Quick Look Data Structure
;               
;               YIX: Y index (Scalar or array)
;
;               BAND: Wavelength band (1 or 2).
;
; Opt. Inputs : None.
;               
; Outputs     : Returns detector Y.
;               
; Opt. Outputs: None.
;               
; Keywords    : NOCHECK: Skip call to QLMGR
;
;               NOPARCHECK: Skip all parameter checks
;
; Calls       : QLMGR, GT_DETECTOR, PARCHECK, CP_GET_STATE
;
; Common      : None.
;               
; Restrictions: None.
;               
; Side effects: None.
;               
; Category    : GET
;               
; Prev. Hist. : None.
;
; Written     : S.V.H. Haugan, UiO, 20-December-1995
;               
; Modified    : Version 2, SVHH, 5-February-1996
;                       Added hack around lacking state information,
;                       supplying default values + warning.
;               Version 3, SVHH, 16 April 1996
;                       Using GET_VDS_SLITPOS instead of cp_get_state.
;                       This function (gt_dety) doesn't return correct 
;                       values all the time, since dety is a function
;                       of window number as well as yix and band.
;               Version 4, SVHH, 18 November 1996
;                       Updated documentation to reflect the shortfalls of
;                       this routine. Basing the dety estimate on the
;                       *average* position of the slit center instead of just
;                       the short-wavelength version.
;
; Version     : 4, 18 November 1996
;-            

FUNCTION gt_dety,qlds,yix,band,nocheck=nocheck,noparcheck=noparcheck
  
  ON_ERROR,2
  IF !debug GT 0 THEN ON_ERROR,0
  
  IF N_PARAMS() LT 3 THEN MESSAGE,"Use: dety = gt_dety(qlds,yix,band)
  
  IF KEYWORD_SET(noparcheck) EQ 0 THEN nocheck = 1
  
  IF NOT KEYWORD_SET(nocheck) THEN BEGIN
     qlmgr,qlds,valid
     IF NOT valid THEN MESSAGE,"First parameter must be a qlds"
  ENDIF
  
  IF gt_detector(qlds,/nocheck) EQ 'GIS' THEN RETURN,0
  
  IF NOT KEYWORD_SET(noparcheck) THEN begin
     parcheck,yix,2,typ(/nat),[0,1],'Y index'
     parcheck,band,3,typ(/nat),0,'Wavelength band',maxval=2,minval=1
  ENDIF  

  ix = WHERE(qlds.detdesc(*).waveband EQ band,count)
  IF count EQ 0 THEN BEGIN
     errmsg = ''
     get_vds_slitpos,qlds.header.date_obs,state,errmsg = errmsg
     IF errmsg NE '' THEN  $
        MESSAGE,"Cannot calculate the DETY position"
     
     IF band EQ 1 THEN BEGIN
        start = (state.n1_cen_s+state.n1_cen_l - state.nis_ht)/2
        RETURN,yix + start
     END ELSE BEGIN
        start = (state.n2_cen_s+state.n2_cen_l - state.nis_ht)/2
        RETURN,start + yix
     END
  END
  
  RETURN,qlds.detdesc(ix(0)).dety + yix
  
END
