;+
; Project     : SOHO - CDS     
;                   
; Name        : GT_DIMENSION
;               
; Purpose     : Returns information on the dimensionality of a QLDS
;               
; Explanation : Based on the information in the QuickLook Data Stucture,
;               an IDL structure describing the dimensionality of the 
;               detector data windows is returned with the following tags:
;
;               WAVELNTH : TRUE if the spectral dimension is present
;               SOLAR_X:   TRUE if the SOLAR_X dimension is present
;               SOLAR_Y:   TRUE if the SOLAR_Y dimension is present
;               DEL_TIME:  TRUE if the DEL_TIME dimension is present
;               NDIMS:     The number of present dimensions
;
;               Note that a dimension may be singular even if it is present.
;               Whenever the last dimension of an array is singular, it will
;               vanish instantly in IDL.
;               
;               There is also a tag for each of the dimensions that
;               describes the size of the dimension, named SWAVELNTH,
;               SSOLAR_X, SSOLAR_Y, SDEL_TIME.
;
;               For programs displaying two-dimensional "images", i.e.,
;               results of gt_scanx/y or gt_i/m/bimage, there are tags with
;               information on solar x/y and del_time origin and spacing, for
;               use with e.g., PLOT_IMAGE. The tags are called
;               SOLAR_X_ORIGIN/SOLAR_X_SPACING etc. Note that this information
;               is a crude linear approximation of the physical coordinates.
;
;               The tags WAVELNTH_ORIGIN/WAVELNTH_SPACING are arrays with four
;               entries, representing the origin/spacing of the wavelength in
;               each band. For NIS, only two entries will be used, the others
;               will be zero. The spacing values of the wavelength is made by
;               dividing the total wavelength span with the number of pixels.
;               
; Use         : info = GT_DIMENSION(QLDS)
;    
; Inputs      : QLDS: QuickLook Data Structure, see CDS software note #9
;               
; Opt. Inputs : None.
;               
; Outputs     : Returns a structure with information on the dimensionality
;               of the data windows in the QLDS
;               
; Opt. Outputs: None.
;               
; Keywords    : NOCHECK : Set to avoid checking the validity of the QLDS
;
; Calls       : QLMGR, QL_SIZE()
;
; Common      : GT_DIMENSION_CACHE : Cache for speeding up this otherwise
;               quite slow routine.
;               
; Restrictions: QLDS should conform to CDS software not #9
;               
; Side effects: None known.
;               
; Category    : CDS QuickLook
;               
; Prev. Hist. : Taken from QL_DIMENSIONS
;
; Written     : Stein Vidar Hagfors Haugan, 25-Aug-1995
;               
; Modified    : Version 2, SVHH, 17 June 1996
;                       Added memory cache since this routine was very slow
;                       and often used.
;               Version 3, SVHH, 12 August 1996
;                       Now compatible with old and new storage system.
;               Version 4, SVHH, 15 August 1996
;                       No longer referring to the data at all.
;               Version 5, SVHH, 22 August 1996
;                       Added origin/spacing fields.
;               Version 6, SVHH, 27 August 1996
;                       No longer requiring windows to be read in, since the
;                       size should be correct anyway.
;               Version 7, William Thompson, GSFC, 26-Oct-2005
;                       Make sure DEL_TIME_ORIGIN and DEL_TIME_SPACING follow
;                       their historic behavior.
;                       
; Version     : Version 7, 26-Oct-2005
;-            


function GT_DIMENSION,QLDS,NOCHECK=NOCHECK
  COMMON gt_dimension_cache,dimensions,ql_no

  if !debug eq 0 then On_Error,2
  IF N_params()	ne 1 THEN $
	  message,'Use: DIMS = GT_DIMENSION(QL_Data_structure)'
  
  IF NOT Keyword_SET(NOCHECK) THEN BEGIN
      QLMGR,QLDS,valid
      IF NOT valid THEN	message,' Parameter is not a CDS QL data structure'
  EndIF
  
  IF N_ELEMENTS(ql_no) GT 0 THEN BEGIN
     ix = WHERE(ql_no EQ qlds.ql_no,count)
     IF count EQ 1 THEN BEGIN
        RETURN,dimensions(ix(0))
     END
  END
  
  TRUE = (1 EQ 1)

  axes = QLDS.detdesc(0).axes
  NDIMS = N_elements(AXES)
  
  dsz = abs(qlds.detdesc(0).ixstop)-qlds.detdesc(0).ixstart + 1
  
  dsz = [N_ELEMENTS(dsz),dsz,4,0]
  
  SZ = QL_SIZE(dsz,NDIMS) ;; Patches up singular trailing dimensions.

  WAVELNTH = (AXES(0) EQ 'WAVELNTH')
  if WAVELNTH then SWAVELNTH=SZ(1) else SWAVELNTH=0
  
  ;; Get wavelength origin/spacing
  detector = gt_detector(qlds)
  wavelnth_origin = dblarr(4)
  wavelnth_spacing = dblarr(4)
  
  dummy = load_wavecal(detector,qlds.header.date_obs,cal_struct=qlds.wavecal)
  IF detector EQ 'NIS' THEN BEGIN
     lambda1 = pix2wave('N1',[0,1023])
     lambda2 = pix2wave('N2',[0,1023])
     wavelnth_origin(0:1) = [lambda1(0),lambda2(0)]
     wavelnth_spacing(0:1) = ([lambda1(1),lambda2(1)]-wavelnth_origin)/1024.
  END ELSE BEGIN
     lambda1 = pix2wave('G1',[0,2047])
     lambda2 = pix2wave('G2',[0,2047])
     lambda3 = pix2wave('G3',[0,2047])
     lambda4 = pix2wave('G4',[0,2047])
     wavelnth_origin = [lambda1(0),lambda2(0),lambda3(0),lambda4(0)]
     wavelnth_spacing = ([lambda1(1),lambda2(1),lambda3(1),lambda4(1)]$
                         - wavelnth_origin)/2048.
  END
  
  i = where(axes eq 'SOLAR_X',COUNT)
  if count eq 0 then message,$
	"A QLDS should have a SOLAR_X dimension, even when singular"

  SOLAR_X = TRUE
  IF WAVELNTH THEN SSOLAR_X= SZ(1+i(0)) $
  ELSE             SSOLAR_X= SZ(1+i(0))/N_elements(qlds.detdesc)
  SOLAR_X_ORIGIN = qlds.detdesc(0).origin(i)
  SOLAR_X_SPACING = qlds.detdesc(0).spacing(i)

  i = where(axes eq 'SOLAR_Y',COUNT)
  if count eq 0 then message,$
	"A QLDS should have a SOLAR_Y dimension, even when singular"
  SOLAR_Y = TRUE
  SSOLAR_Y = SZ(1+i(0))
  SOLAR_Y_ORIGIN = qlds.detdesc(0).origin(i)
  SOLAR_Y_SPACING = qlds.detdesc(0).spacing(i)
 
  i = where(axes eq 'DEL_TIME',COUNT)
  SDEL_TIME=0
  DEL_TIME = (COUNT GT 0)
  if DEL_TIME then begin
     SDEL_TIME = SZ(1+i(0))
     DEL_TIME = (SDEL_TIME GT 1)
     DEL_TIME_ORIGIN = qlds.detdesc(0).origin(i)
     DEL_TIME_SPACING = qlds.detdesc(0).spacing(i)
  END ELSE BEGIN
     DEL_TIME_ORIGIN = 0.0
     DEL_TIME_SPACING = DOUBLE(qlds.header.exptime)
  END
     
  NDIMS=0
  IF WAVELNTH THEN NDIMS=NDIMS+1
  IF SOLAR_X  THEN NDIMS=NDIMS+1
  IF SOLAR_Y  THEN NDIMS=NDIMS+1
  IF DEL_TIME THEN NDIMS=NDIMS+1

  dimension = {QL_DIMENSION, $
               WAVELNTH:WAVELNTH,$
               SWAVELNTH:SWAVELNTH,$
               SOLAR_X: SOLAR_X,$
               SSOLAR_X: SSOLAR_X,$
               SOLAR_Y: SOLAR_Y,$
                SSOLAR_Y: SSOLAR_Y,$
               DEL_TIME: DEL_TIME,$
               SDEL_TIME:SDEL_TIME,$
               WAVELNTH_ORIGIN:WAVELNTH_ORIGIN,$
               WAVELNTH_SPACING:WAVELNTH_SPACING,$
               SOLAR_X_ORIGIN:SOLAR_X_ORIGIN,$
               SOLAR_X_SPACING:SOLAR_X_SPACING,$
               SOLAR_Y_ORIGIN:SOLAR_Y_ORIGIN,$
               SOLAR_Y_SPACING:SOLAR_Y_SPACING,$
               DEL_TIME_ORIGIN: Float(DEL_TIME_ORIGIN[0]),$
               DEL_TIME_SPACING: Double(DEL_TIME_SPACING[0]),$
               NDIMS: NDIMS}
  
  IF N_ELEMENTS(ql_no) EQ 0 THEN BEGIN
     dimensions = dimension
     ql_no = [qlds.ql_no]
  END ELSE BEGIN
     dimensions = [temporary(dimensions),dimension]
     ql_no = [ql_no,qlds.ql_no]
  END
  
  return,dimension

end

