;+
; Project     : SOHO - CDS     
;                   
; Name        : GT_DURATION()
;               
; Purpose     : Get the duration of a raster from a file or data structure.
;               
; Explanation : Depending on the nature of the input parameter (structure or
;               file name) the routine extracts and returns the duration
;               of the raster.
;               
; Use         : IDL> dur = gt_duration(file_str)
;    
; Inputs      : file_str   -  either:-   the name of a CDS FITS file
;                                 or:-   the name of a CDS data structure
;               
; Opt. Inputs : None
;               
; Outputs     : Function returns the raster duration in a intarr(3) giving
;               the hours, minutes and seconds.
;               
; Opt. Outputs: None
;               
; Keywords    : SECONDS - return duration in seconds otherwise [hr,min,sec]
;
; Calls       : FX... routines
;
; Common      : None
;               
; Restrictions: None
;               
; Side effects: None
;               
; Category    : Data_handling, FITS
;               
; Prev. Hist. : None
;
; Written     : C D Pike, RAL, 15-Jun-94
;               
; Modified    : Allow for exposures to be in reverse order.  CDP, 20-Mar-96
;               Add SECONDS keyword.  CDP, 29-Mar-96
;               Handle missing exposure start times.  CDP, 20-Feb-98
;
; Version     : Version 4, 20_Feb-98
;-            

function gt_duration, input, seconds=seconds

;
;  need at least one parameter
;
if n_params() eq 0 then begin
   print,'USE:   IDL> dur = get_duration(file_or_structure)
   return,0
endif


;
;  get the number of exposures
;
nexp = gt_numexp(input)
if nexp eq 0 then return,0

;
;  get the exposure times
;
t = gt_start(input,/exp)

;
;  check for bad first or last start times
;
t0 = t(0)
t1 = last_item(t)
if anytim2tai(t0) le 0.0 or anytim2tai(t1) le 0.0 then begin
   bell
   print,'First or last exposure start time invalid - duration will be a lower limit.'
endif

;
;  get the exposure time
;
integ = gt_exptime(input)

;
;  first and last (allow for missing values)
;
ttai = anytim2tai(t)
n = where(ttai gt 0.0, count)
if count gt 0 then t = t(n)
st = min(utc2tai(str2utc(t)))
et = max(utc2tai(str2utc(t))) + integ 

;
;  get duration
;
dur = et - st

;
;  to hours min sec
;
hr = fix(dur/3600.)
mins = fix((dur-(hr*3600L))/60.)
sec = fix(round(dur - hr*3600L - mins*60))


if keyword_set(seconds) then return,round(dur) else return,[hr,mins,sec]
end
