;+
; Project     : SOHO - CDS     
;                   
; Name        : GT_FPOINT()
;               
; Purpose     : Gets the pointing from FITS file header.
;               
; Explanation : The routine extracts and returns the pointing
;               used in the rasters of a given study number.
;               
; Use         : IDL> s = gt_fpoint(FITS_number [,/ops])
;    
; Inputs      : FITS_number   -  the number of a CDS FITS file
;               
; Opt. Inputs : None
;               
; Outputs     : Function returns the pointing in either OPS or Solar terms
;               for each raster.
;               
; Opt. Outputs: None
;               
; Keywords    : OPS  -  return pointing for in OPS L/R coordinates, otherwise
;                       in Solar X/Y arcsecs.
;               QUIET - no screen output
;
; Calls       : FX... routines
;
; Common      : None
;               
; Restrictions: None
;               
; Side effects: None
;               
; Category    : Data_handling, FITS
;               
; Prev. Hist. : None
;
; Written     : C D Pike, RAL, 12-May-96
;               
; Modified    : 
;
; Version     : Version 1, 12-May-1996
;-            

function gt_fpoint, input, ops=ops, quiet=quiet

;
;  need at least one parameter
;
if n_params() eq 0 then begin
   print,'USE:   IDL> point = gt_fpoint(file_number)
   return,0
endif

;
;  what was the input?
;
if datatype(input,1) eq 'Integer' then begin
      l = list_fits('s'+trim(input),/qu,/no)
      nl = n_elements(l)
      pos = fltarr(nl,2)
      for i=0,nl-1 do begin
         h = headfits(l(i))
         sx = str_pick(h(37),'=','/') 
         sy = str_pick(h(38),'=','/') 
         sl = str_pick(h(61),'=','/') 
         sr = str_pick(h(63),'=','/') 
         if keyword_set(ops) then begin
            pos(i,0) = sl
            pos(i,1) = sr
         endif else begin
            pos(i,0) = sx
            pos(i,1) = sy
         endelse
         if not keyword_set(quiet) then begin
            print,'Raster '+trim(i),+'  Posn: '+$
                   fmt_vect([pos(i,0),pos(i,1)],form='(f7.1)')
         endif
      endfor
endif else begin   
   print,'Input must be a CDS FITS file number.'
   return,0
endelse
   
;
;  return with requested format
;
return, reform(pos)

end
