;+
; Project     : SOHO - CDS     
;                   
; Name        : GT_IIMAGE
;               
; Purpose     : Return image formed at a given dispersion pixel number
;               
; Explanation : GT_IIMAGE returns a two-dimensional image from
;               a CDS observation raster. The image consists of the
;               intensities at a specific dispersion pixel number, for
;               all the X/Y image positions in the raster.
;
;               Note that the wavelength need not be constant over
;               the whole image, depending on the observational 
;               parameters and the wavelength calibration.
;
;               The dispersion pixel number may be specified with
;               either the wavelength BAND and a detector X value,
;               or a detector window name/number plus an offset
;               that is added to the starting detector x placement
;               of that detector window.
;
;               The returned array has dimensions (X,Y) given by
;               the size of the data contained in the QLDS.
;
;               In addition to the image, values for the physical
;               parameters XSOLAR, YSOLAR, TIME and LAMBDA is
;               returned through output keywords as two-dimensional
;               arrays with the same size as the image.
;
; Use         : im = gt_iimage(QLDS, <keywords>)
;    
; Inputs      : QLDS: CDS QuickLook Data structure
;               
;               In addition, *either* BAND + DETX
;                            *or*     WINDOW + OFFSET keywords
;               must be specified.
;
; Opt. Inputs : BAND: Which wavelength band should be returned.
;
;               DETX: The detector X (dispersion) pixel number.
;                     NIS: 0..1024
;                     GIS: 0..2048
;
;               WINDOW: The number or name of a detector data window
;                       in the QLDS. 
;
;               OFFSET: Added to the starting detector X (dispersion)
;                       value for the selected window.
;               
; Outputs     : Two-dimensional image. Size determined by the content
;               of the QLDS. 
;               
; Opt. Outputs: LAMBDA : wavelength for each pixel (Angstrom) 
;
;               XSOLAR : Solar Cartesian X (arcsecs)
;
;               YSOLAR : Solar Cartesian Y (arcsecs)
;
;               TIME   : Time of the exposures
;               
; Keywords    : NOCHECK: Set to avoid call to QLMGR
;
;               NOPARCHECK: Set to avoid most parameter checking mostly for
;                           internal use.
;
;               QUICK:   Set to avoid calculation of physical parameters
;                        TIME, XSOLAR AND YSOLAR.
;
; Calls       : datatype(), gt_binx(), gt_cds_time(), gt_detector()
;               gt_dimension(), gt_lambda(), gt_solarx(), gt_solary()
;               gt_windata(), gt_window(), gt_wnum(), parcheck, qlmgr
;               st_windata, tag_exist(), trim(), typ()
;
; Common      : None.
;               
; Restrictions: QLDS == QL Data Structure.
;               
; Side effects: None known.
;               
; Category    : GET
;               
; Prev. Hist. : 
;
; Written     : SVH Haugan, UiO, 27-October-1995
;               
; Modified    : Version 1.1, SVH, 2-November-1995
;                           Updated "Calls" documentation field.
;
;               Version 2, SVHH, 16-January-1995
;                          Fixed problem with calculating OFFSET from
;                          detx - detdesc(window).detx **MOD 2048**
;
;               Version 3, SVHH, 6 April 1996
;                          Included keyword QUICK, changed calculation
;                          of wwidth to use ixstart/ixstop.
;               Version 4, SVHH, 12 April 1996
;                          Renamed gt_time -> gt_cds_time
;               Version 5, SVHH, 28 June 1996
;                          Changed upper limit on OFFSET to account for
;                          binning. Also, missing values are not altered when
;                          dividing by the binning factor.
;               Version 6, SVHH, 9 August 1996
;                       Improved the handling of the /QUICK keyword, speedup
;                       about a factor of 10 achieved.
;               Version 7, SVHH, 15 August 1996
;                       Compatible with old and new storage system.
;                          
; Version     : 7, 15 August 1996
;-            


FUNCTION gt_iimage,QLDS, $
                  WINDOW=WINDOW, offset=offset, band=waveband, detx=detx, $
                  NOCHECK=NOCHECK, $
                  noparcheck=noparcheck, lambda=wavelength, $
                  xsolar=solarx, ysolar=SOLARY, time=TIME, $
                  dety=dety, mirpix=mirpix, quick=quick ;; For debugging
  
  ON_ERROR,2
  IF !debug GT 0 THEN ON_ERROR,0
  
; --------------------------------
; Parameter checking...
; --------------------------------
  
  IF NOT KEYWORD_SET(noparcheck) THEN BEGIN
     IF N_params() LT 1 THEN $
        MESSAGE,'Use: gt_iimage(QL_Data,<keywords>)'
     
     IF NOT Keyword_SET(NOCHECK) THEN BEGIN
        QLMGR,QLDS,valid
        IF NOT valid THEN MESSAGE,'First argument must be a QL data structure'
     EndIF
  ENDIF
  
  oldsys = tag_exist(qlds,'DETDATA',/top_level)
  
  det = gt_detector(qlds,/nocheck)
  dims = gt_dimension(qlds,/nocheck)
  binx = gt_binx(qlds,/nocheck)
  
  IF N_elements(waveband) GT 0 THEN BEGIN
     IF N_ELEMENTS(WINDOW) GT 0 OR N_ELEMENTS(offset) GT 0 THEN $
        MESSAGE,"Use *either*  band/detx  *or*  window/offset"
     
     IF det EQ 'NIS' THEN BEGIN
        maxband = 2
        maxdetx = 1023
     END ELSE BEGIN
        maxband = 4
        maxdetx = 2047
     END
     IF NOT KEYWORD_SET(noparcheck) THEN BEGIN
        parcheck,waveband,0,typ(/NAT),0,'(wave)BAND',minval=0,maxval=maxband
        parcheck,detx,0,typ(/nat),0,'DETX',minval=0,maxval=maxdety
     END
     cand = gt_window(qlds,waveband,detx,/loaded)
     IF cand(0) EQ -1 THEN  $
        MESSAGE,("No data windows covering BAND="+trim(waveband)+$
                 " DETX="+trim(detx))
     WINDOW = cand(0)
     offset = detx - (qlds.detdesc(WINDOW).detx MOD 2048)
  ENDIF
  
  IF NOT KEYWORD_SET(noparcheck) THEN BEGIN
     parcheck,WINDOW,0,typ(/NAT,/STR),0,'WINDOW',/noerr,result=r
     IF r(0) NE '' THEN BEGIN
        FOR i=0,N_elements(r)-1 DO MESSAGE,r(i),/informational
        MESSAGE,"Need either band/detx or window/offset"
     ENDIF
     IF datatype(WINDOW) EQ 'STR' THEN WINDOW = gt_wnum(QLDS,wind)
     
     
     catch,error
     IF error NE 0 THEN BEGIN
        catch,/cancel
        IF N_ELEMENTS(value) AND NOT oldsys THEN st_windata,qlds,WINDOW,value
        MESSAGE,!ERR_STRING
     END
     
     value = gt_windata(qlds,WINDOW,/no_copy)
     IF NOT dims.wavelnth THEN wwidth = 1 $
     ELSE                      wwidth = (SIZE(value))(1)
     IF NOT oldsys THEN st_windata,qlds,WINDOW,value,/no_copy
     
     catch,/cancel
     
     maxoffset = (wwidth)*binx-1
     parcheck,offset,0,typ(/nat),0,'OFFSET',minval=0,maxval=maxoffset
  ENDIF
  
  IF datatype(WINDOW) EQ 'STR' THEN WINDOW = gt_wnum(QLDS,wind)
  
  IF dims.ssolar_x LE 1 OR dims.ssolar_y LE 1 THEN  $
     MESSAGE,"Cannot make an image (X or Y dimension singular or missing)"
  
; --------------------------------
; Finished with parameter checking
; --------------------------------
  
;
; Establish some useful entities
;
  
  collapsed = dims.swavelnth EQ 0
  SX = dims.ssolar_x
  sY = dims.ssolar_y
  IF NOT KEYWORD_SET(quick) THEN BEGIN
     idetx = qlds.detdesc(WINDOW).detx + offset
     dety = rebin(INDGEN(1,SY),SX,SY,/sample)
     IF det EQ 'NIS' THEN dety = temporary(dety) + qlds.detdesc(WINDOW).dety
     yix = dety-MIN(dety)
     mirpix = rebin(INDGEN(SX,1),sx,sy,/sample)
  ENDIF
  
  IF N_ELEMENTS(tix) EQ 0 THEN tix = 0
  
  offset = offset/binx
  
  catch,error
  IF error NE 0 THEN BEGIN
     catch,/cancel
     IF N_ELEMENTS(value) AND NOT oldsys THEN st_windata,qlds,WINDOW,value
     MESSAGE,!ERR_STRING
  END
     
  value = gt_windata(qlds,WINDOW,/no_copy)
  IF collapsed THEN $
     image = value(*,*,tix) $
  ELSE  $
     image = value(offset,*,*,tix)
  IF NOT oldsys THEN st_windata,qlds,WINDOW,value,/no_copy
  
  catch,/cancel
  
  image = REFORM(image,/overwrite)
  
  IF binx NE 1 THEN BEGIN
     missix = WHERE(image EQ qlds.detdesc(WINDOW).missing,misscount)
     image = temporary(image)/binx
     IF misscount GT 0 THEN image(missix) = qlds.detdesc(WINDOW).missing
  END
  
  IF NOT KEYWORD_SET(quick) THEN begin
     WAVELENGTH = GT_lambda(QLDS,iDETX,dety,mirpix)
     SOLARX = GT_SOLARX(QLDS,iDETX,dety,mirpix)
     SOLARY = GT_SOLARY(QLDS,iDETX,dety,mirpix)
     TIME = GT_CDS_TIME(QLDS,MIRPIX,YIX,TIX)
  END
  
  RETURN,image
  
END
