;+
; Project     : SOHO - CDS     
;                   
; Name        : GT_LAMBDA
;               
; Purpose     : Returns the wavelength for data points
;               
; Explanation : This function is supposed to return the most correct
;               wavelength for given data points.
;               
;               In order to be able to compensate for possible misalignments
;               etc., data points must be fully specified with detector X
;               values, detector Y (or slit position for GIS), and mirror
;               position.
;
;               Other relevant information is taken from the QLDS.
;
;               All parameters except the QLDS may be either arrays or
;               scalars. The returned array with solar x values will be of the
;               same size as the expression:
;
;               RESULT = c1*DETX + c2*DETY + c3*MIRPIX
;
;               I.e., having DETX = [0,1] and DETY=[0,1,2] does not produce an
;               error, but the result will be a fltarr(2) - NOT a fltarr(2,3)
;               or fltarr(3,2) etc.
;
; Use         : GT_LAMBDA(QLDS,DETX, DETY || SLITPIX ,MIRPIX)
;    
; Inputs      : QLDS: CDS QuickLook Data structure
;
;               DETX: Detector X values. 0..1023 for NIS,
;                     0..8191 for GIS (i.e., DETX also specifies
;                     the waveband).
;
;               DETY: Detector Y values for the NIS detector.
;
;               SLITPIX: Slit positions index for the GIS detector.
;                       Range 0..N(slit positions)
;               
;               MIRPIX: Mirror position index
;                       Range 0..N(mirror positions)
;
; Opt. Inputs : None.
;               
; Outputs     : Returns the wavelength (in Angstrom) for the specified data
;               points. Return value has the same dimension as a linear
;               combination of the input parameters.
;               
; Opt. Outputs: None.
;
; Keywords    : NOCHECK: Set to avoid call to QLMGR
;
; Calls       : ANYTIM2UTC(), GT_DETECTOR(), GT_DIMENSION(), LOAD_WAVECAL(),
;               PARCHECK, PIX2WAVE(), QLMGR, TRIM(), TYP()
;
; Common      : None.
;               
; Restrictions: QLDS == QL Data Structure.
;               Currently, only the DETDESC.origin/spacing fields
;               are used.
;               
; Side effects: None.
;               
; Category    : CDS, QuickLook, Data_Handling.
;               
; Prev. Hist. : 
;
; Written     : S.V.H. Haugan, 18-October-1995
;               
; Modified    : Version 1.1: SVHH, 19-October-1995 
;                       Moved division between NIS band 1/2 to 511
;                       The identity of the two bands were also switched.
;               Version 2.0, SVHH, 1-November-1995
;                       Added the use of !DEBUG flag to "turn on" simulation
;                       of mis-aligned spectrum/detector for testing purposes.
;               Version 3, SVHH, 5-December-1995
;                       Fixed an error for the GIS case (DetX -> iDetX).
;               Version 4, SVHH, 14 April 1996
;                       Using LOAD_WAVECAL()
;               Version 5, SVHH, 23 February 1999
;                       Included calibrated tilt/mirror shift coefficients
;                       in the calculation.
;		Version 6, 13 April 1999, William Thompson, GSFC
;			Call PIX2WAVE with /NOLIMIT
;               Version 7, SVHH, 2 December 1999
;                       Allow floats/fractional input values for DETX/Y
;
; Version     : 7, 2 December 1999
;-            


FUNCTION GT_LAMBDA,Q,DetX,DetY,MirpIx,NOCHECK=NOCHECK,NOPARCHECK=NOPARCHECK
  
  On_Error,2
  IF !DEBUG GT 0 THEN On_Error,0
  
  IF NOT KEYWORD_SET(noparcheck) THEN BEGIN
     IF N_params() LT 4 THEN $
        MESSAGE,"Use: GT_LAMBDA(QLDS,DETX, DETY || SLITPIX ,MIRPIX)
     
     IF NOT Keyword_SET(NOCHECK) THEN BEGIN
        QLMGR,Q,valid
        IF NOT valid THEN MESSAGE,"First argument must be a QLDS"
     EndIF
     
     d = gt_dimension(Q)
     det = gt_detector(Q)
     
     IF det EQ 'NIS' THEN maxdetx=1023 ELSE maxdetx=(4*2048-1)
     IF det EQ 'NIS' THEN maxdety=512+256 ELSE maxdety = (d.ssolar_y-1) >0
     MaxMIR  = (d.ssolar_x-1) >0
     
     parcheck,DetX,2,typ(/REA),INDGEN(5),'DETX',MIN=0,MAX=maxdetx
     parcheck,DetY,3,typ(/REA),INDGEN(5),'DETY/SLITPIX',MIN=0,MAX=maxdety
     parcheck,MirpIx,4,typ(/NAT),INDGEN(5),'MIRPIX',MIN=0,MAX=MaxMIR
  END ELSE BEGIN
     d =  gt_dimension(Q)
     det = gt_detector(q)
  END
  
  dummy = load_wavecal(det,anytim2utc(q.header.date_obs), $
                       cal_struct=q.wavecal)
  
;
;  Blowing up dimensions
;

  idetx =    DetX + 0.0*DetY + 0*MirpIx
  idety =  0*DetX +   DetY + 0*MirpIx
  imirp =  0*DetX + 0*DetY +   MirpIx

  result = 0.0*idetx
  
  IF det EQ 'NIS' THEN BEGIN
     band = (idety LT 512) + 1
     FOR bandi=1,2 DO BEGIN
        ix = WHERE(band EQ bandi,count)
        IF count GT 0 THEN BEGIN
           ;; Get tilt/mirror shift coefficients
           get_tiltcal,bandi,q.header.date_obs,tilt_coeff
           get_mirshift,bandi,q.header.date_obs,mir_coeff
           
           ;; Correct for tilt - tilt is a function of detector X,
           ;; and maxdety/2.0 is used as the zero point for the tilt.
           ;; 
           tilt = poly(idetx(ix),tilt_coeff)
           idetx(ix) = idetx(ix) + (idety(ix)-maxdety/2.)*temporary(tilt)
           
           ;; Correct for mirror shift. First, find mirror position
           ;; Note that this correction is applied with opposite sign
           ;; compared to the tilt - just a convention...
           ;; 
           mirpos = q.mir_posdata(imirp(ix))
           idetx(ix) = idetx(ix) - poly(mirpos-128.0,mir_coeff)
           
           result(ix) = pix2wave('NIS'+trim(bandi),idetx(ix),/nolimit)
        ENDIF
     ENDFOR
  END ELSE BEGIN
     band = (FIX(iDetX) / 2048) + 1
     pix = idetx MOD 2048
     FOR bandi=1,4 DO BEGIN
        ix = WHERE(band EQ bandi,count)
        IF count GT 0 THEN BEGIN
           result(ix) = pix2wave('GIS'+trim(bandi),pix(ix),/nolimit)
        ENDIF
     ENDFOR
  EndELSE
  RETURN,result
END

