;+
; Project     : SOHO - CDS     
;                   
; Name        : GT_SCANY
;               
; Purpose     : Return a "slit spectrogram" with (lambda,X) dimension
;               
; Explanation : This function returns a "slit spectrogram" from a QLDS in the
;               form of an array with 2 dimensions. The first dimension
;               is along the dispersion direction, and the second is along
;               the solar Y/slit (position) direction, thus simulating a 
;               slit spectrogram with the slit placed in the Y direction.
;
;               The QLDS must contain appropriate data, e.g., there must be
;               more than 1 slit position (or positions along the slit) in the
;               raster.  This means that the observations must be represented
;               with an array with dimensions (LAMBDA,SOLARX,SOLARY,DEL_TIME)
;               where SOLARY > 1.
;
;               If the QLDS contains data for several solar X positions, the
;               instrument X position index (XIX) must be supplied.  The solar
;               X position index is analogous to the slit position. If the
;               QLDS represents a time series (DEL_TIME > 1) then the
;               time index TIX (repetition index) must be specified.
;
;               Although GT_SCANY always returns a two-dimensional array,
;               the size of the wavelength dimension may vary, depending
;               on the nature of the request.
;
;               The second (Y) dimension size is determined by the
;               size of the data stored in the QLDS.
;
;               If a specific wavelength band is specified, all the
;               (deselected) data from that wavelength band/detector
;               is returned (i.e., 1024 elements for NIS, 2048
;               elements from GIS), although this can be overridden by
;               using the /NODESELECT keyword.
;
;               If specific wavelength windows are specified, there is
;               a choice between deselecting to a full detector
;               (1024(NIS)/2048(GIS)) or simply packed into a
;               contiguous array.
;
;               In case the sumline compression scheme has been used, the
;               counts will be spread out over the original detector area
;               where the summation has been performed.
;
;               In addition to returning the detector counts,
;               GT_SCANY returns other relevant quantities such as
;               wavelength, solar x/y and time through the use of
;               output keywords. The names of the keywords are
;               constructed in order to make efficient use of
;               abbreviations.
;
;               These auxiliary data are returned either as an array
;               with the same size as the spectrum, or as a scalar (if
;               the value is truly a constant over the whole spectrum,
;               e.g., time).
;
; Use         : data = gt_scany(QLDS, <keywords>)
;    
; Inputs      : QLDS: CDS QuickLook Data structure
;               
;               Some keyword parameters *are* required, though which
;               ones are sufficient may vary.
;
; Opt. Inputs : BAND: Which wavelength band should be returned.
;
;               WINDOW: The number or name of a detector data window
;                       in the QLDS. If it's an array, the data from
;                       the specified windows will be packed in a
;                       contiguous array
;
;               XIX: Solar Cartesian X index (0..)
;
;               TIX: Time index (exposure no.) (0..)1
;               
; Outputs     : Returns a two-dimensional array corresponding to a
;               slit spectrogram with the slit along the solar Y
;               direction.
;               
; Opt. Outputs: LAMBDA : wavelength for each pixel (Angstrom) 
;
;               XSOLAR : Solar Cartesian X (arcsecs)
;
;               YSOLAR : Solar Cartesian Y (arcsecs)
;
;               TIME   : Time of the exposures
;               
; Keywords    : NODESELECT: Set to use the (wavelength) packed format.
;
;               NOCHECK: Set to avoid call to QLMGR
;
;               NOPARCHECK: Set to avoid most parameter checking mostly for
;                           internal use. Automatically sets NOCHECK
;
;               NOEXTRA: Set to skip calculating LAMBDA,X/YSOLAR,TIME
;
;               QUICK: Sets /NOCHECK, /NOPARCHECK, and /NOEXTRA
;
; Calls       : datatype(), default, gt_cds_time(), gt_detector()
;               gt_dety(), gt_dimension(), gt_lambda(), gt_numwin()
;               gt_solarx(), gt_solary(), gt_windata(), gt_wnum(), parcheck
;               qlmgr, st_windata, tag_exist(), trim(), typ()
;
; Common      : None.
;               
; Restrictions: QLDS == QL Data Structure.
;               
; Side effects: None known.
;               
; Category    : GET
;               
; Prev. Hist. :
;
; Written     : SVH Haugan, UiO, 18-October-1995
;               
; Modified    : Version 2, SVHH, 27-October-1995
;                       Slightly rewritten to make all gt_scan_
;                       functions similar.
;
;               Version 3, SVHH, 12-December-1995
;                       Added keywords NOEXTRA + QUICK. Fixed bug
;                       in handling TIX + XIX when NOPARCHECK was set.
;
;               Version 4, SVHH, 5-February-1996
;                       Added the possibility of extracting blank
;                       scans when BAND is set and NODESELECT is NOT set.
;
;               Version 5, SVHH, 8 April 1996
;                       Fixed collapse of lambda (first) dimension 
;                       in REFORM when NOT originally collapsed *but*
;                       singular.
;               Version 6, SVHH, 12 April 1996
;                       Renamed gt_time -> gt_cds_time
;               Version 7, SVHH, 7 August 1996
;                       New storage system with handles.
;               Version 8, SVHH, 15 August 1996
;                       Compatible with both storage systems.
;               Version 9, SVHH, 29 January 1997
;                       Coping (at least) with BINY > 1 cases.
; Version     : 9, 29 January 1997
;-            


FUNCTION gt_scany,QLDS, $
                  WINDOW=WINDOW, xix=xix, tix=tix, band=waveband,$
                  NODESELECT=NODESELECT, NOCHECK=NOCHECK, $
                  noparcheck=noparcheck, lambda=wavelength, $
                  xsolar=solarx, ysolar=SOLARY, time=TIME, $
                  noextra=noextra,quick=quick,$
                  detx=detx, dety=dety, mirpix=mirpix ;; For debugging
  
  ON_ERROR,2
  IF !debug GT 0 THEN ON_ERROR,0
  
; --------------------------------
; Parameter checking...
; --------------------------------
  
  IF KEYWORD_SET(quick) THEN BEGIN
     noparcheck = 1
     noextra = 1
  ENDIF
  
  IF KEYWORD_SET(noparcheck) THEN nocheck = 1
  
  IF NOT KEYWORD_SET(noparcheck) THEN BEGIN
     IF N_params() LT 1 THEN $
        MESSAGE,'Use: gt_scany(QL_Data,<keywords>)'
     
     IF NOT Keyword_SET(NOCHECK) THEN BEGIN
        QLMGR,QLDS,valid
        IF NOT valid THEN MESSAGE,'First argument must be a QL data structure'
     EndIF
  ENDIF
  
  blank = 0
  
  IF N_elements(waveband) GT 0 THEN BEGIN
     IF gt_detector(QLDS) EQ 'NIS' THEN maxband=2
     default,maxband,4
     IF NOT KEYWORD_SET(noparcheck) THEN  $
        parcheck,waveband,0,typ(/NAT),0,'(wave)BAND',minval=1,maxval=maxband
     IF N_elements(WINDOW) GT 0 THEN cand=gt_wnum(QLDS,WINDOW) $
     ELSE cand=INDGEN(N_elements(QLDS.DetDesc))
     cand = WHERE(QLDS.DetDesc(cand).waveband EQ waveband,count)
     IF count EQ 0 THEN BEGIN
        IF N_elements(WINDOW) EQ 0 THEN BEGIN
           IF KEYWORD_SET(nodeselect) THEN $
              MESSAGE,'No data windows in waveband '+trim(waveband) $
           ELSE blank = 1
        END ELSE MESSAGE,'None of those windows in waveband '+trim(waveband)
     EndIF
     WINDOW=cand
     default,nodeselect,0
  ENDIF
  
  IF NOT KEYWORD_SET(noparcheck) THEN BEGIN
        parcheck,WINDOW,0,typ(/NAT,/STR),[0,1],'WINDOW',/noerr,result=r
        IF r(0) NE '' THEN BEGIN
           FOR i=0,N_elements(r)-1 DO Print,r(i)
           MESSAGE,"Need either (wave)BAND or WINDOW"
        ENDIF
  ENDIF
  
  IF datatype(WINDOW) EQ 'STR' THEN WINDOW = gt_wnum(QLDS,wind)
  
  IF NOT KEYWORD_SET(noparcheck) AND NOT blank THEN BEGIN
     parcheck,WINDOW,0,typ(/NAT),[0,1],'WINDOW', $
        minval=0,maxval=gt_numwin(QLDS)-1
  ENDIF
  
  dims = gt_dimension(QLDS)
  
  IF dims.ssolar_y LE 1 THEN BEGIN 
     MESSAGE,"Faking Y scan (Y dimension singular or missing)",/continue
     default,yix,0
  END
  
  IF dims.ssolar_x EQ 1 THEN default,xix,0
  IF dims.sdel_time LE 1 THEN default,tix,0
  
  IF NOT KEYWORD_SET(noparcheck) THEN BEGIN
     
     PARCHECK,XIX,0,typ(/NAT),0,'XIX',$
        minval=0,maxval=dims.ssolar_x-1
     
     PARCHECK,TIX,0,typ(/NAT),0,'TIX',$
        minval=0,maxval=(dims.sdel_time-1)>0
  ENDIF
  
  default,nodeselect,1
  
  IF NOT KEYWORD_SET(noparcheck) THEN  $
     parcheck,nodeselect,0,typ(/NAT),0,'NODESELECT'
  
; --------------------------------
; Finished with parameter checking
; --------------------------------
  
;
; Sort windows according to increasing wavelength
;
  
  IF N_ELEMENTS(WINDOW) GT 1 THEN BEGIN 
     ix =  SORT(QLDS.detdesc(WINDOW).wavelength)
     WINDOW = WINDOW(ix)
  END
  
;
; Establish some useful entities
;
  
  deselect = NOT nodeselect
  detector = gt_detector(QLDS)
  collapsed = dims.swavelnth LE 1
  
  ;; S2 is the size of the spatial dimension.
  S2 = dims.ssolar_y
  
  ;; Correct this for binning in the Y direction:
  IF detector EQ 'NIS' THEN S2 = S2*qlds.detdesc(0).biny
  
;
; Generate data arrays for accumulating/storing output.
;
  
; For a scan in the Y direction, we have:
; SPEC : (lambdasize, S2) -- The extracted data
; DETY : (lambdasize, S2) -- varies as a function of dim 1 and 2 (NIS) 
;                            or only dim. 2 (GIS).
; DETX : (lambdasize, S2) -- varies only as a function of dim. #1
; MIRPIX : Constant
; TIX : Constant
  
  missing = FLOAT(qlds.detdesc(0).missing)
  
  IF NOT KEYWORD_SET(noextra) THEN BEGIN
     IF deselect THEN BEGIN
        ;; No accumulation - spread out in advance
        IF detector EQ 'NIS' THEN BEGIN
           NX = 1024
           spec=REPLICATE(missing,NX,S2)
           detx=rebin(INDGEN(NX,1),NX,S2,/sample)
           IF blank THEN dety0 = gt_dety(qlds,0,1) $
           ELSE          dety0 = QLDS.DetDesc(WINDOW(0)).dety
           dety = rebin(INDGEN(1,S2)+dety0, $
                        NX,S2,/sample)
        END ELSE BEGIN
           NX = 2048
           spec=REPLICATE(missing,NX,S2)
           detx=(rebin(INDGEN(NX,1),NX,S2,/sample) + NX*(waveband-1))
           dety = (rebin(INDGEN(1,S2),NX,S2,/sample))
        END
        MIRPIX = xix
        ;; tix = tix
     END ELSE BEGIN
        ;; Generate starting points for accumulation over Wavelnth dimension
        
        spec = INtarr(1,S2)
        detx = INtarr(1,S2)
        
        mirpix = xix
        ;; mirpix0 = mirpix ; Not used for Y scan
        
        IF detector EQ 'NIS' THEN BEGIN
           ;; May vary for NIS data:
           dety = INDGEN(1,S2)
           dety00 = dety
        END ELSE BEGIN
           ;; May vary for GIS data:
           dety = INDGEN(1,S2)
           dety00 = dety
        END
     END
  END ELSE BEGIN
     IF deselect THEN BEGIN
        IF detector EQ 'NIS' THEN BEGIN
           NX = 1024
           spec=REPLICATE(missing,NX,S2)
        END ELSE BEGIN
           NX = 2048
           spec=REPLICATE(missing,nx,s2)
        END
     END ELSE BEGIN
        spec = INtarr(1,S2)
     END
  END
  
;---------------------------------------------
; Loop over all windows, accumulate/store data.
;---------------------------------------------
  oldsys = tag_exist(qlds,'DETDATA',/top_level)
  
  IF blank THEN maxwin = -1 $
  ELSE maxwin = N_ELEMENTS(WINDOW)-1
  FOR wndix = 0,maxwin DO BEGIN
     wnd = WINDOW(wndix)                       ;; Shorthand
     desc = qlds.detdesc(wnd)
     IF desc.ixstop(0) GE 0 THEN BEGIN
        
        ;; Security 
        catch,error
        IF error NE 0 THEN BEGIN
           catch,/cancel
           IF N_ELEMENTS(value) GT 0 AND NOT oldsys THEN $
              st_windata,qlds,wnd,value
           MESSAGE,"Error caught: "+!ERR_STRING
        END
        
        ;; Form the data block
        value = gt_windata(qlds,wnd,/no_copy)
        IF collapsed THEN BEGIN ; collapsed wavelength dimension
           data = value(xix,*,tix)
        END ELSE BEGIN
           data = value(*,xix,*,tix)
        END
        IF NOT oldsys THEN st_windata,qlds,wnd,value
        
        catch,/cancel

        ;; Collapse any *trailing* extra dimensions
        data = REFORM(data,/overwrite)
        
        ;; Put back the first dimension if it's collapsed
        ;; There *is NOT* a singular first dimension mimicking lambda
        IF collapsed THEN data = REFORM(data,1,(SIZE(data))(1),/overwrite)
        
        ;; Insert collapsed Y dimension if it's not there!!
        IF (SIZE(data))(0) EQ 1 THEN data = REFORM(data,(SIZE(data))(1),1,$
                                                   /overwrite)
        
        ;; Correct for binning in the detector X direction
        IF desc.binx NE 1 THEN $
           data = rebin([data]/FLOAT(desc.binx),$
                        (SIZE(data))(1)*desc.binx,(SIZE(data))(2),/sample)
        
        ;; Correct for binning in detector Y direction
        biny = 1
        IF detector EQ 'NIS' THEN BEGIN 
           biny = desc.biny
           data = rebin([data]/FLOAT(desc.biny),(Size(data))(1),S2,/sample)
        END
        
        missix = where(data EQ desc.missing/biny/desc.binx)
        IF missix(0) NE -1 THEN data(missix) = missing
        
        ;; The data is now guaranteed to be 2-dimensional with correct size.
        
        IF deselect THEN BEGIN
           spec((desc.detx MOD NX),0) = data
           
           ;; For a NIS Y scan, we have to take into account the tilt of the
           ;; spectrum etc, even with a full band readout.
           IF detector EQ 'NIS' AND NOT KEYWORD_SET(noextra) THEN BEGIN
              sz = SIZE(data)
              ;; mirpix is constant for Y scan
              ;; DETX is correct by definition,
              ;; but DETY is tricky
              dety0=(rebin(INDGEN(1,S2),sz(1),S2,/sample) $
                     + qlds.detdesc(wnd).dety)        ;; With offset for NIS
              ;; Put detector y data into position
              dety(desc.detx MOD nx,0) = dety0
            END
        END ELSE BEGIN
           spec = [spec,data]
           IF NOT KEYWORD_SET(noextra) THEN BEGIN
              sz = SIZE(data)
              ;; mirpix is constant for Y scan
              ;; Create block with DETX values, increasing in the 1st dimension
              dtx = rebin(REFORM(desc.detx + INDGEN(sz(1)),sz(1), $
                                 1,/overwrite),$
                          sz(1),S2,/sample)
              detx = [detx,dtx]
              
              ;; Detector Y varies linearly in the 2nd dimension
              IF detector EQ 'NIS' THEN BEGIN
                 dety0=(rebin(INDGEN(1,S2),sz(1),S2,/sample) $
                        + qlds.detdesc(wnd).dety)        ;; With offset for NIS
              END ELSE BEGIN
                 dety0 = rebin(dety00,(SIZE(data))(1),S2,/sample)
              END
              dety = [dety,dety0]
           END
        END
     ENDIF
  EndFOR
  
  IF nodeselect THEN BEGIN
     IF (SIZE(spec))(1) EQ 1 THEN BEGIN
        MESSAGE,"None of the requested data was present in memory",$
           /informational
     END ELSE begin
        spec = spec(1:*,*)
        IF NOT KEYWORD_SET(noextra) THEN BEGIN
           detx = detx(1:*,*)
           dety = dety(1:*,*)
           ;; Mirpix is constant for Y scan
        END
     END
  EndIF
  
  IF NOT KEYWORD_SET(noextra) THEN BEGIN
     S = SIZE(SPEC)
     yix = REBIN(INDGEN(1,S(2)),S(1),S(2),/SAMPLE)
     
     WAVELENGTH = GT_lambda(QLDS,DETX,dety,mirpix)
     SOLARX = GT_SOLARX(QLDS,DETX,dety,mirpix)
     SOLARY = GT_SOLARY(QLDS,DETX,dety,mirpix)
     IF detector EQ 'NIS' THEN biny = qlds.detdesc(0).biny $
     ELSE biny = 1
     TIME = GT_CDS_TIME(QLDS,MIRPIX,YIX/biny,TIX)
  ENDIF
  
  RETURN,spec
  
END
