;+
; Project     : SOHO - CDS     
;                   
; Name        : GT_SLITPOS()
;               
; Purpose     : Retrieve the slit positions from a file or data structure.
;               
; Explanation : Depending on the nature of the input parameter (structure or
;               file name) the routine extracts and returns the slit positions
;               used in the raster.
;               
; Use         : IDL> s = gt_slitpos(file_str)
;    
; Inputs      : file_str   -  either:-   the name of a CDS FITS file
;                                 or:-   the name of a CDS data structure
;               
; Opt. Inputs : None
;               
; Outputs     : Function returns the slit positions used in an integer array.
;               
; Opt. Outputs: None
;               
; Keywords    : None
;
; Calls       : FX... routines
;
; Common      : None
;               
; Restrictions: None
;               
; Side effects: None
;               
; Category    : Data_handling, FITS
;               
; Prev. Hist. : None
;
; Written     : C D Pike, RAL, 24-May-94
;               
; Modified    : To allow standard FITS file directory, CDP, 25-May-94
;               Fix bug in tag name in file case.  CDP, 14-Jun-94
;               Insert cal/ops test.  CDP, 15-Jun-94
;               Fix bug related to previous change.  CDP, 10-Jul-95
;               New tag names in non-calibration QLDS, SVHH, 4-Oct-95
;
; Version     : Version 6, 4-Oct-1995
;-            

function gt_slitpos, input

;
;  need at least one parameter
;
if n_params() eq 0 then begin
   print,'USE:   IDL> s = get_slitpos(file_or_structure)
   return,0
endif

;
;  what was the input?
;
case datatype(input,1) of
   'Structure': begin
                   qlmgr, input, valid
                   if valid then begin
                      t = tag_names(input)
                      IF (WHERE(t EQ "SLIT_POSDATA"))(0) NE -1 THEN BEGIN
                         RETURN,INPUT.SLIT_POSDATA
                      END ELSE BEGIN
                         RETURN,input.slipdata
                      END
                   endif
                   print,'GT_SLITPOS: Structure supplied is non-standard'
                   return,0
                end
;
;  allow user to have own version of FITS files
;
      'String': begin
                   tfile = ''
                   if  file_exist(input) then begin
                      tfile = input
                   endif else begin
                     if file_exist(concat_dir('$CDS_CAL_FITS',input)) then begin
                        tfile = concat_dir('$CDS_CAL_FITS',input)
                     endif
                   endelse
                   
;
;  FITS file doesn't exist in user or std directory
;
                   if tfile eq '' then begin
                      print,'GT_SLITPOS:  Input file does not exist.'
                      return,0
                   endif

                   fxbopen,unit,tfile,1,header
                   if fxpar(header,'TELESCOP') eq 'CALIBRATION' then begin
                      spos = cal_AuxRead(unit,header,'SLIT_POS',status=status)
                   endif else begin
                      spos = AuxRead(unit,header,'SLIT_POS',status=status)
                   endelse
                   if n_elements(status) gt 0 then begin
                      if status ne '' then begin
                         print,'Error reading slit positions in FITS file.'
                         fxbclose,unit
                         return,0
                      endif
                   endif
                   fxbclose,unit
                   if fxpar(header,'TELESCOP') eq 'CALIBRATION' then begin
                      return,spos.d
                   endif else begin
                      return,spos.data
                   endelse
                end
          else: begin
                   print,'Input must be a file name or a CDS data structure.'
                   return,0
                end
endcase

end
