	PRO GT_SOLAR_XY, QLDS, WINDOWI, SOLAR_X, SOLAR_Y, NOCHECK=NOCHECK, $
		NOPADDING=NOPADDING, UPDATE=UPDATE, EXTEND=EXTEND,	$
		ERRMSG=ERRMSG, _EXTRA=_EXTRA
;+
; Project     :	SOHO - CDS
;
; Name        :	GT_SOLAR_XY
;
; Purpose     :	Return solar X and Y values for a CDS data window
;
; Category    :	Class2, CDS, Analysis, Coordinates
;
; Explanation :	This routine returns an array of solar X and Y values for a CDS
;		data window.  The returned arrays have the same dimensions as
;		the window itself.  Thus, if one uses GT_WINDATA to return the
;		window, then GT_SOLAR_XY returns the X and Y positions for
;		every pixel in the window.
;
;		If one uses VDS_ROTATE on a data window, then the same
;		VDS_ROTATE can be used on the X and Y arrays.  (This routine
;		will also recognize if NIS_ROTATE has been applied to the
;		entire QLDS.)
;
;		For GIS data, the first (wavelength) dimension in the X and Y
;		arrays is set to a single pixel--GIS pointing does not depend
;		on the detector pixel.
;
;		If the observation is a "sit-and-stare", i.e. contains a fourth
;		time dimension, then the time dimension will be omitted from
;		the output X and Y arrays, unless the EXTEND keyword is used.
;
;		When used with the 90x240 arcsecond slit, the effect of the
;		slit width on the NIS detector is taken into account when
;		calculating X positions.  However, this feature is only
;		activated when the window width is comparable to the slit
;		width.  A critical assumption is made that the line being
;		observed is in the center of the window.
;
;		For cases where solar feature tracking is turned on, then the
;		returned pointings will be referenced back to the start time of
;		the raster.
;
; Syntax      :	GT_SOLAR_XY, QLDS, WINDOWI, SOLAR_X, SOLAR_Y
;
; Examples    :	A = READCDSFITS('s3300r00.fits')
;		GT_SOLAR_XY, A, 0, X, Y
;
; Inputs      :	QLDS	= A CDS quicklook data structure from READCDSFITS.
;		WINDOWI	= The window index, from 0 to N-1.
;
; Opt. Inputs :	None.
;
; Outputs     :	SOLAR_X	= The array of solar X values.
;		SOLAR_Y = The array of solar Y values.
;
; Opt. Outputs:	None.
;
; Keywords    :	NOCHECK	= If set, then don't check that the QLDS is a proper
;			  quicklook data structure.
;
;		NOPADDING = If set, then the data will be returned without
;			    padding.  Thus, different data windows could be
;			    returned with different sizes.  For example, when
;			    reading in full NIS readout, the window for NIS1
;			    will have a different height than that for NIS2,
;			    because of the different slants in each band.
;
;		UPDATE = If set, then call UPD_CDS_POINT before extracting the
;			 X and Y arrays.  Any other keywords accepted by
;			 UPD_CDS_POINT, such as NOROLL are also accepted.
;
;		EXTEND = The fourth time dimension is normally omitted from the
;			 returned SOLAR_X and SOLAR_Y arrays.  Setting /EXTEND
;			 will put this dimension in when appropriate.
;
;		ERRMSG = If defined and passed, then any error messages will be
;			 returned to the user in this parameter rather than
;			 depending on the MESSAGE routine in IDL.  If no errors
;			 are encountered, then a null string is returned.  In
;			 order to use this feature, ERRMSG must be defined
;			 first, e.g.
;
;				ERRMSG = ''
;				X = GT_SOLAR_XY( ERRMSG=ERRMSG, ... )
;				IF ERRMSG NE '' THEN ...
;
; Calls       :	QLMGR, GT_WINDESC, FIND_WITH_DEF, GET_VDS_SLITPOS, DELVARX,
;		UPD_CDS_POINT
;
; Common      :	None.
;
; Restrictions:	At the moment, subpixel shifts in the Y pointing due to the
;		round-off of the window positions adjusted for slant are not
;		accounted for.  To do this right, second order effects would
;		have to be taken into account.
;
; Side effects:	When the UPDATE keyword is used, then the input data structure
;		is changed by UPD_CDS_POINT.
;
; Prev. Hist. :	None.
;
; History     :	Version 1, 13-May-1997, William Thompson, GSFC
;		Version 2, 17-Dec-1997, William Thompson, GSFC
;			Allow pass-through of any UPD_CDS_POINT keywords
;		Version 3, 28-Feb-2000, William Thompson, GSFC
;			Work with header-only quicklook structures.
;		Version 4, 23-May-2000, William Thompson, GSFC
;			Add offsets between NIS-1 and NIS-2
;		Version 5, 01-Jun-2000, William Thompson, GSFC
;			Detect if NIS_ROTATE has been applied.
;		Version 5, 30-Aug-2000, William Thompson, GSFC
;			Detect if NIS_ROTATE was called with /ALIGN
;
; Contact     :	WTHOMPSON
;-
;
	ON_ERROR, 2
	MESSAGE = ''
;
;  Check the input parameters.
;
	IF N_PARAMS() NE 4 THEN BEGIN
	    MESSAGE = 'Syntax:  GT_SOLAR_XY, QLDS, WINDOWI, SOLAR_X, SOLAR_Y
	    GOTO, HANDLE_ERROR
	ENDIF
;
;  Make sure that the user passed a proper quicklook data structure.
;
	IF NOT KEYWORD_SET(NOCHECK) THEN BEGIN
	    QLMGR, QLDS, VALID
	    IF NOT VALID THEN BEGIN
		MESSAGE = "First parameter must be a " +	$
			"Quick Look Data Structure (QLDS)"
		GOTO, HANDLE_ERROR
	    ENDIF
	ENDIF
;
;  If requested, call UPD_CDS_POINT.
;
	IF KEYWORD_SET(UPDATE) THEN	$
		UPD_CDS_POINT, QLDS, ERRMSG=MESSAGE, _EXTRA=_EXTRA
	IF MESSAGE NE '' THEN GOTO, HANDLE_ERROR
;
;  Get the window description
;
	D = GT_WINDESC( QLDS, WINDOWI, /QUICK, ERRMSG = MESSAGE)
	IF MESSAGE(0) NE '' THEN GOTO, HANDLE_ERROR
;
;  Determine the sizes of the windows.
;
	DIM = ABS(D.IXSTOP - D.IXSTART + 1)
	IF KEYWORD_SET(NOPADDING) THEN DIM = DIM - D.PADDING
	N_DIM = N_ELEMENTS(D.IXSTOP)
;
;  Determine the starting position, the step sizes, and the angle.
;
	XSTART = FLOAT(D.ORIGIN(1))  &  XDELTA = FLOAT(D.SPACING(1))
	YSTART = FLOAT(D.ORIGIN(2))  &  YDELTA = FLOAT(D.SPACING(2))
	ANGLE  = FLOAT(D.ROTATION(1))
;
;  If the window is in NIS-1, then calculate offsets.
;
	ALIGNED = 0
	IF TAG_EXIST(QLDS,'ALIGNED') THEN ALIGNED = QLDS.ALIGNED
	IF (NOT ALIGNED) AND D.WAVEBAND EQ 1 THEN BEGIN
	    GT_NIS_ALIGNMENT, QLDS, XOFFSET, YOFFSET
	    XSTART = XSTART + XOFFSET
	    YSTART = YSTART + YOFFSET
	ENDIF
;
;  Setup default slants.  Normally taken from state database, but in
;  case sites do not have that, then hard code here.
;
	IF FIND_WITH_DEF('cdhsstate.dbf','$ZDBASE') NE '' THEN BEGIN
	    GET_VDS_SLITPOS, QLDS.HEADER.DATE_OBS, DESC
	    DIFF1 = DESC.N1_CEN_L - DESC.N1_CEN_S
	    DIFF2 = DESC.N2_CEN_L - DESC.N2_CEN_S
	END ELSE BEGIN
	    DIFF1 = 17
	    DIFF2 = 10
	ENDELSE
;
;  Determine which spectrograph was used, and define the slant accordingly.
;  Also, set the wavelength dimension to unity for GIS data--the pointing does
;  not depend on detector pixel for GIS data.
;
	IF QLDS.HEADER.DETECTOR EQ 'GIS' THEN BEGIN
	    SLANT = 0
	    DIM(0) = 1
        END ELSE CASE FIX(D.DETY/512) OF
	    1: SLANT = DIFF1 / 1024.
	    0: SLANT = DIFF2 / 1024.
	ENDCASE
;
;  Check to see if NIS_ROTATE has been applied to the array.  If it has, then
;  set the SLANT to zero.
;
	IF TAG_EXIST(QLDS,'ROTATED') THEN IF QLDS.ROTATED THEN SLANT = 0
;
;  Because this routine can be memory intensive, first make sure that memory is
;  not already being taken up by the SOLAR_X and SOLAR_Y arrays.
;
	DELVARX, SOLAR_X
	DELVARX, SOLAR_Y
;
;  Determine the solar X and Y offsets from the first pixel, ignoring rotation.
;  For the X offset, take into account the mirror position.  If slit 6 is used
;  with the NIS detector, and the window is 100 pixels in size or less, then
;  also take into account the horizontal position on the detector.
;
	XX = (LINDGEN(DIM(0)*DIM(1)) / DIM(0)) # REPLICATE(XDELTA, DIM(2))
	IF (QLDS.HEADER.DETECTOR EQ 'NIS') AND (QLDS.HEADER.SLIT_NUM EQ 6) $
		AND (DIM(0)*(D.BINX > 1) LE 100) THEN BEGIN
	    I0 = (DIM(0) - 1.) / 2.
	    DX = 1.68 * (D.BINX > 1)
	    DX = (LINDGEN(DIM(0)) - I0) # REPLICATE(DX, DIM(1)*DIM(2))
	    XX = XX + REFORM(DX, DIM(0)*DIM(1), DIM(2), /OVERWRITE)
	ENDIF
;
;  For the Y offset, take into account the vertical position on the detector,
;  modified by the slant.  For GIS, just take into account the slit position
;  (SLANT=0).
;
	YY = (LINDGEN(DIM(0)*DIM(1)) MOD DIM(0)) # REPLICATE(YDELTA, DIM(2))
	YY = REPLICATE(YDELTA, DIM(0)*DIM(1)) # LINDGEN(DIM(2)) -	$
		SLANT*(TEMPORARY(YY) - 0.5*DIM(0)*YDELTA)
;
;  Reformat into the correct dimensions.
;
	XX = REFORM(XX, DIM(0:2), /OVERWRITE)
	YY = REFORM(YY, DIM(0:2), /OVERWRITE)
;
;  Take into account the angle of rotation, pivoting around the first pixel.
;
	IF ANGLE NE 0 THEN BEGIN
	    COS_A = COS(ANGLE / !RADEG)
	    SIN_A = SIN(ANGLE / !RADEG)
	    SOLAR_X = XX * COS_A  -  YY * SIN_A
	    SOLAR_Y = XX * SIN_A  +  YY * COS_A
	END ELSE BEGIN
	    SOLAR_X = TEMPORARY(XX)
	    SOLAR_Y = TEMPORARY(YY)
	ENDELSE
;
;  Add in the starting point.
;
	SOLAR_X = TEMPORARY(SOLAR_X) + XSTART
	SOLAR_Y = TEMPORARY(SOLAR_Y) + YSTART
;
;  If the data window is four-dimensional, then extend the X and Y arrays.
;
	IF (N_ELEMENTS(DIM) EQ 4) AND KEYWORD_SET(EXTEND) THEN BEGIN
	    SOLAR_X = REFORM(SOLAR_X, N_ELEMENTS(SOLAR_X), /OVERWRITE)
	    SOLAR_Y = REFORM(SOLAR_Y, N_ELEMENTS(SOLAR_Y), /OVERWRITE)
	    SOLAR_X = REFORM(SOLAR_X # REPLICATE(1L, DIM(3)), DIM, /OVERWRITE)
	    SOLAR_Y = REFORM(SOLAR_Y # REPLICATE(1L, DIM(3)), DIM, /OVERWRITE)
	ENDIF
;
	RETURN
;
;  Error handling point.
;
HANDLE_ERROR:
	IF N_ELEMENTS(ERRMSG) NE 0 THEN ERRMSG = 'GT_SOLAR_XY: ' + MESSAGE $
		ELSE BEGIN
	    N_MESSAGE = N_ELEMENTS(MESSAGE)
	    FOR I=0,N_MESSAGE-2 DO MESSAGE, MESSAGE(I), /CONTINUE
	    MESSAGE, MESSAGE(N_MESSAGE-1)
	ENDELSE
;
	RETURN
	END
