;+
; Project     : SOHO - CDS     
;                   
; Name        : GT_SOLARX
;               
; Purpose     : Returns the Solar X coordinates for data points
;               
; Explanation : This function is supposed to return the most correct solar
;               coordinates for given data points.
;               
;               In order to be able to compensate for possible misalignments
;               etc., data points must be fully specified with detector X
;               values, detector Y (or slit position for GIS), and mirror
;               position.
;
;               Other relevant information is taken from the QLDS.
;
;               All parameters except the QLDS may be either arrays or
;               scalars.  The returned array with solar x values will either
;               be a scalar, or of the same size as the expression:
;
;               RESULT = c1*DETX + c2*DETY + c3*MIRPIX
;
;               *or* 
;
;               RESULT = c1*DETY
;
;               I.e., having DETX = [0,1] and DETY=[0,1,2] does not produce an
;               error, but the result will be a fltarr(2) - NOT a fltarr(2,3)
;               or fltarr(3,2) etc.
;
; Use         : GT_SPECTRUM(QLDS,DETX, DETY || SLITPIX ,MIRPIX)
;    
; Inputs      : QLDS: CDS QuickLook Data structure
;
;               DETX: Detector X values. 0..1023 for NIS, 0..8191 for GIS
;                     (i.e., DETX also specifies the waveband).
;
;               DETY: Detector Y values for the NIS detector.
;
;               SLITPIX: Slit positions index for the GIS detector.
;                        Range 0..N(slit positions)-1
;               
;               MIRPIX: Mirror position index
;                       Range 0..N(mirror positions)-1
;
; Opt. Inputs : None.
;               
; Outputs     : Returns the solar x coordinates for the specified data
;               points. Return value has the same dimension as a linear
;               combination of the input parameters.
;               
; Opt. Outputs: None.
;
; Keywords    : NOCHECK: Set to avoid call to QLMGR
;
; Calls       : PARCHECK, QLMGR
;
; Common      : None.
;               
; Restrictions: QLDS == QL Data Structure.
;               Currently, only the DETDESC.origin/spacing fields
;               are used.
;               
; Side effects: None.
;               
; Category    : CDS, QuickLook, Data_Handling.
;               
; Prev. Hist. : 
;
; Written     : S.V.H. Haugan, 18-October-1995
;               
; Modified    : Version 1.1: SVHH, 19-October-1995 
;                            Loosened restrictions on NIS DETY parameter
;                            until more correct procedure is found.
;
; Version     : 1.1, 19-October-1995
;-            


FUNCTION GT_SOLARX,Q,DETX,DETY,MIRPIX,NOCHECK=NOCHECK,noparcheck=noparcheck
  On_Error,2
  IF !DEBUG GT 0 THEN On_Error,0
  
  IF NOT KEYWORD_SET(noparcheck) THEN BEGIN
     
     IF N_params() LT 4 THEN $
        MESSAGE,"Use: GT_SPECTRUM(QLDS,DETX, DETY || SLITPIX ,MIRPIX)
     
     IF NOT Keyword_SET(NOCHECK) THEN BEGIN
        QLMGR,q,valid
        IF NOT valid THEN MESSAGE,"First argument must be a QLDS"
     EndIF
     
     det = gt_detector(q)
     
;  
     D = GT_DIMENSION(Q,/NOCHECK)
     
;
; Parameter/range checking
;
     IF det EQ 'NIS' THEN maxdetx=1023 ELSE maxdetx=(4*2048-1)
     IF det EQ 'NIS' THEN maxdety=1023 ELSE maxdety = (d.ssolar_y-1) >0
     IF det EQ 'NIS' THEN mindety=0 ELSE mindety = 0
     
     MaxMIR  = (d.ssolar_x-1) >0
     
     parcheck,DetX,2,typ(/NAT),INDGEN(5),'DETX',MIN=0,MAX=maxdetx
     parcheck,DetY,3,typ(/NAT),INDGEN(5),'DETY/SLITPIX',MIN=mindety,MAX=maxdety
     parcheck,MirpIx,4,typ(/NAT),INDGEN(5),'MIRPIX',MIN=0,MAX=MaxMIR
     
  END ELSE BEGIN
     det =  gt_detector(q)
     d = GT_dimension(Q,/nocheck)
  END
  
; Blowing up dimensions.. for a full blowup we'd have:
  
;  idetx =   detx + 0*dety + 0*mirpix
;  idety = 0*detx +   dety + 0*mirpix
;  imirp = 0*detx + 0*dety +   mirpix
  
; But if mirpix is scalar, we should return a scalar (since no other
; dependency is currently known)

  IF NOT d.wavelnth THEN $
     X = q.detdesc(0).origin(0) + mirpix*q.detdesc(0).spacing(0) $
  ELSE  X = q.detdesc(0).origin(1) + mirpix*q.detdesc(0).spacing(1)
  RETURN,X
END
