;+
; Project     : SOHO - CDS     
;                   
; Name        : GT_SOLARY
;               
; Purpose     : Returns the Solar Y coordinates for data points
;               
; Explanation : This function is supposed to return the most correct solar
;               coordinates for given data points.
;               
;               In order to be able to compensate for possible misalignments
;               etc., data points must be fully specified with detector X
;               values, detector Y (or slit position for GIS), and mirror
;               position.
;
;               Other relevant information is taken from the QLDS.
;
;               All parameters except the QLDS may be either arrays or
;               scalars. The returned array with solar x values will either
;               be a scalar, or of the same size as the expression:
;
;               RESULT = c1*DETX + c2*DETY + c3*MIRPIX
;
;               *or* 
;
;               RESULT = c1*DETY
;
;               I.e., having DETX = [0,1] and DETY=[0,1,2] does not produce an
;               error, but the result will be a fltarr(2) - NOT a fltarr(2,3)
;               or fltarr(3,2) etc.
;
; Use         : GT_SPECTRUM(QLDS,DETX, DETY || SLITPIX ,MIRPIX)
;    
; Inputs      : QLDS: CDS QuickLook Data structure
;
;               DETX: Detector X values. 0..1023 for NIS, 0..8191 for GIS
;                     (i.e., DETX also specifies the waveband).
;
;               DETY: Detector Y values for the NIS detector.
;
;               SLITPIX: Slit positions index for the GIS detector.
;                        Range 0..N(slit positions)-1
;               
;               MIRPIX: Mirror position index
;                       Range 0..N(mirror positions)-1
;
; Opt. Inputs : None.
;               
; Outputs     : Returns the solar y coordinates for the specified data
;               points. Return value has the same dimension as a linear
;               combination of the input parameters.
;               
; Opt. Outputs: None.
;
; Keywords    : NOCHECK: Set to avoid call to QLMGR
;
; Calls       : PARCHECK, QLMGR
;
; Common      : None.
;               
; Restrictions: QLDS == QL Data Structure.
;               Currently, only the DETDESC.origin/spacing fields
;               are used.
;               
; Side effects: None.
;               
; Category    : CDS, QuickLook, Data_Handling.
;               
; Prev. Hist. : 
;
; Written     : S.V.H. Haugan, 18-October-1995
;               
; Modified    : Version 1.1: SVHH, 19-October-1995 
;                            Loosened restrictions on NIS DETY parameter
;                            until more correct procedure is found.
;               Version 2, SVHH, 20-December-1995
;                            Rewrote some of the NIS DETY handling.
;               Version 3, SVHH, 17 June 1996
;                          Taking the tilt on the NIS detector into account.
;               Version 4, CDP, 20 Feb 2002
;                          Fix for y-binned data on line 162 as suggested by SVHH.
;
; Version     : 4, 20 February 2002
;-            


FUNCTION GT_SOLARY,Q,DETX,DETY,MIRPIX,NOCHECK=NOCHECK,noparcheck=noparcheck
  
  On_Error,2
  IF !DEBUG GT 0 THEN On_Error,0
  
  IF NOT KEYWORD_SET(noparcheck) THEN BEGIN
     
     IF N_params() LT 4 THEN $
        MESSAGE,"Use: GT_SPECTRUM(QLDS,DETX, DETY || SLITPIX ,MIRPIX)
     
     IF NOT Keyword_SET(NOCHECK) THEN BEGIN
        QLMGR,q,valid
        IF NOT valid THEN MESSAGE,"First argument must be a QLDS"
     EndIF

     det = q.header.detector
     
;  
     D = GT_DIMENSION(Q,/NOCHECK)
     
     IF NOT d.solar_y THEN RETURN,q.header.ycen

;
; Parameter/range checking
;
     IF det EQ 'NIS' THEN maxdetx=1023 ELSE maxdetx=(4*2048-1)
     IF det EQ 'NIS' THEN maxdety=1023 ELSE maxdety = (d.ssolar_y-1) >0
     IF det EQ 'NIS' THEN mindety=0 ELSE mindety = 0
     
     MaxMIR  = (d.ssolar_x-1) >0
     
     parcheck,DetX,2,typ(/NAT),INDGEN(5),'DETX',MIN=0,MAX=maxdetx
     parcheck,DetY,3,typ(/NAT),INDGEN(5),'DETY/SLITPIX',MIN=mindety,MAX=maxdety
     parcheck,MirpIx,4,typ(/NAT),INDGEN(5),'MIRPIX',MIN=0,MAX=MaxMIR
     
  END ELSE BEGIN
     det = q.header.detector
     d = gt_dimension(q)
  END

; Blowing up dimensions.. for a full blowup we'd have:
  
  IF det EQ 'NIS' THEN BEGIN
     idetx =   detx + 0*dety + 0*mirpix
     idety = 0*detx +   dety + 0*mirpix
     imirp = 0*detx + 0*dety +   mirpix
  END
  
; But if dety is scalar, we should return a scalar (since no other dependency
; is known)
  
  idety = dety
  ON_ERROR,0
  
  IF det EQ 'NIS' THEN BEGIN
     COMMON GT_SOLARY_VDS_SLITPOS,slitp,date_obs
     IF N_ELEMENTS(date_obs) EQ 0 THEN date_obs = ''
     IF date_obs NE STRMID(q.header.date_obs,0,10) THEN BEGIN
        get_vds_slitpos,q.header.date_obs,slitp ; Get center line coordinates
        date_obs = STRMID(q.header.date_obs,0,10)
        PRINT,"New date_obs:",date_obs
     END
     ix1 = WHERE(idety GT 512)
     ix2 = WHERE(idety LT 512)
     
     ;; Make resultant array
     
     sz = SIZE(idety)
     IF sz(0) NE 0 THEN BEGIN
        sz(sz(0)+1) = 4 ;; Float array
        y = make_array(SIZE=sz,/nozero)
     END ELSE y = [0.0]
     
     ;; If wavelength dimension disappeared for some reason...
     
     IF NOT d.wavelnth THEN yspacing = q.detdesc(0).spacing(1)/q.detdesc(0).biny $
     ELSE                   yspacing = q.detdesc(0).spacing(2)/q.detdesc(0).biny
     
     ;; Calculate pixel offsets from center line.
     IF ix1(0) NE -1 THEN BEGIN
        tilt1 = (slitp.n1_cen_l-slitp.n1_cen_s)/1024.0
        offset1 = idety(ix1) -  (slitp.n1_cen_s + tilt1 * idetx(ix1)) 
        y1 = q.header.ycen + offset1 * yspacing
        y(ix1) = y1
     END
     
     IF ix2(0) NE -1 THEN BEGIN
        tilt2 = (slitp.n2_cen_l-slitp.n2_cen_s)/1024.0
        offset2 = idety(ix2) -  (slitp.n2_cen_s + tilt2 * idetx(ix2)) 
        y2 = q.header.ycen + offset2 * yspacing
        y(ix2) = y2
     END
     
     IF N_ELEMENTS(y) EQ 1 THEN RETURN,y(0)
     RETURN,y
  END
  
  IF NOT d.wavelnth THEN $
     Y = q.detdesc(0).origin(1) + idety*q.detdesc(0).spacing(1) $
  ELSE  Y = q.detdesc(0).origin(2) + idety*q.detdesc(0).spacing(2)
  RETURN, Y
END
