;+
; Project     : SOHO - CDS     
;                   
; Name        : GT_SPECTRUM
;               
; Purpose     : Returns one-dimensional spectrum at specified point
;               
; Explanation : The GT_SPECTRUM function returns a one-dimensional array with
;               a spectrum from the supplied QLDS.
;
;               The extracted spectrum must be uniquely determined among the
;               QLDS data by supplying all the relevant parameters. E.g., with
;               a one-exposure GIS data set, no additional information is
;               required other than e.g., the wavelength band or a window
;               name/number, but a time series with multiple exposures with
;               the NIS would require a solar x index plus a time (exposure
;               no.)  index in order to specify which spectrum should be
;               extracted.
;
;               Failure to supply enough data to uniquely determine which
;               spectrum should be extracted results in an error.
;
;               Although GT_SPECTRUM always returns a one-dimensional
;               spectrum, the number of elements in the returned array may
;               vary depending on the nature of the request.
;               
;               If a specific wavelength band is specified, all the
;               (deselected) data from that wavelength band/detector is
;               returned (i.e., 1024 elements for NIS, 2048 elements from
;               GIS), although this can be overridden by using the /NODESELECT
;               keyword.
;
;               If specific wavelength windows are specified, there is a
;               choice between deselecting to a full detector
;               (1024(NIS)/2048(GIS)) or simply packed into a contiguous
;               array.
;
;               In case the sumline compression scheme has been used, the
;               counts will be spread out over the original detector area
;               where the summation has been performed.
;
;               In addition to returning the detector counts, GT_SPECTRUM
;               returns other relevant quantities such as wavelength, solar
;               x/y and time through the use of output keywords. The names of
;               the keywords are constructed in order to make efficient use of
;               abbreviations.
;
;               These auxiliary data are returned either as an array with the
;               same size as the spectrum, or as a scalar (if the value is
;               truly a constant over the whole spectrum, e.g., time).
;               
; Use         : result = GT_SPECTRUM(QLDS,<keywords>)
;    
; Inputs      : QLDS: CDS QuickLook Data structure
;
;               Some keyword parameters *are* required, though which
;               ones are sufficient may vary.
;
; Opt. Inputs : BAND: Which wavelength band should be returned.
;
;               WINDOW: The number or name of a detector data window
;                       in the QLDS. If it's an array, the data from
;                       the specified windows will be packed in a
;                       contiguous array
;
;               YIX: Solar Cartesian Y index (0..)
;
;               XIX: Solar Cartesian X index (0..)
;
;               TIX: Time index (exposure no.) (0..)
;
;               
; Outputs     : Returns a one-dimensional array with a spectrum at the
;               specified point.
;               
; Opt. Outputs: LAMBDA : wavelength for each pixel (Angstrom) 
;
;               XSOLAR : Solar Cartesian X (arcsecs)
;
;               YSOLAR : Solar Cartesian Y (arcsecs)
;
;               TIME   : Time of the exposure
;
;               
; Keywords    : NOCHECK: Set to avoid call to QLMGR
;
;               NODESELECT: Set to use the (wavelength) packed format.
;
;               NOPARCHECK: Set to avoid most parameter checking mostly for
;                           internal use.
;
;               NOEXTRA : Don't calculate optional outputs LAMBDA, XSOLAR etc.
;
;               QUICK : Sets /NOEXTRA and /NOCHECK and /NOPARCHECK
;               
; Calls       : DEFAULT, GT_LAMBDA(), GT_NUMWIN(), GT_SOLARX(), GT_SOLARY(),
;               GT_WNUM(), PARCHECK, QLMGR, TRIM()
;
; Common      : None.
;               
; Restrictions: QLDS == QL Data Structure.
;               
; Side effects: None.
;               
; Category    : CDS, QuickLook, Data_Handling.
;               
; Prev. Hist. : Based on POINTSLICE
;
; Written     : SVH Haugan, UiO, 18-October-1995
;               
; Modified    : Version 2, SVHH, 8-December-1995
;                                Fixed waveband conversion error for GIS
;               Version 3, SVHH, 12 April 1996
;                       Renamed gt_time -> gt_cds_time
;               Version 4, SVHH, 18 August 1996
;                       Compatible with new and old storage system.
;                       Added NOEXTRA and QUICK keywords, and the possibility
;                       to extract blank scans.
;                       
; Version     : 4, 18 August 1996
;-      

;
;

FUNCTION Gt_Spectrum,QLDS, $
                     WINDOW=WINDOW, xix=xix, yix=yix, tix=tix, band=waveband,$
                     NODESELECT=NODESELECT, NOCHECK=NOCHECK, $
                     noparcheck=noparcheck, lambda=wavelength, $
                     xsolar=solarx, ysolar=SOLARY, time=TIME, $
                     noextra=noextra,quick=quick,$
                     detx=detx, dety=dety, mirpix=mirpix ;; For debugging
  
  ON_ERROR,2
  IF !debug GT 0 THEN ON_ERROR,0
  
;
; Parameter checking...
;
  
  IF KEYWORD_SET(quick) THEN BEGIN
     noparcheck = 1
     noextra = 1
     nocheck = 1
  END
  
  IF NOT KEYWORD_SET(noparcheck) THEN BEGIN
     IF N_params() LT 1 THEN $
        MESSAGE,'Use: gt_spectrum(QL_Data,<keywords>)'
     
     IF NOT Keyword_SET(NOCHECK) THEN BEGIN
        QLMGR,QLDS,valid
        IF NOT valid THEN MESSAGE,'First argument must be a QL data structure'
     EndIF
  ENDIF
  
  blank = 0
  
  IF N_elements(waveband) GT 0 THEN BEGIN
     IF gt_detector(QLDS) EQ 'NIS' THEN maxband=2
     default,maxband,4
     IF NOT KEYWORD_SET(noparcheck) THEN  $
        parcheck,waveband,0,typ(/NAT),0,'(wave)BAND',minval=1,maxval=maxband
     IF N_elements(WINDOW) GT 0 THEN cand=gt_wnum(QLDS,WINDOW) $
     ELSE cand=INDGEN(N_elements(QLDS.DetDesc))
     cand = WHERE(QLDS.DetDesc(cand).waveband EQ waveband,count)
     IF count EQ 0 THEN BEGIN
        IF N_elements(WINDOW) EQ 0 THEN BEGIN
           IF KEYWORD_SET(nodeselect) THEN $
              MESSAGE,'No data windows in waveband '+trim(waveband) $
           ELSE blank = 1
        END ELSE MESSAGE,'None of those windows in waveband '+trim(waveband)
     EndIF
     WINDOW=cand
     default,nodeselect,0
  ENDIF
  
  IF NOT KEYWORD_SET(noparcheck) THEN BEGIN
     parcheck,WINDOW,0,typ(/NAT,/STR),[0,1],'WINDOW',/noerr,result=r
     IF r(0) NE '' THEN BEGIN
        FOR i=0,N_elements(r)-1 DO Print,r(i)
        MESSAGE,"Need either (wave)BAND or WINDOW"
     EndIF
  ENDIF
  
  IF datatype(WINDOW) EQ 'STR' THEN WINDOW = gt_wnum(QLDS,wind)
  
  IF NOT KEYWORD_SET(noparcheck) AND NOT blank THEN BEGIN
     parcheck,WINDOW,0,typ(/NAT),[0,1],'WINDOW', $
        minval=0,maxval=gt_numwin(QLDS)-1
  ENDIF
  
;
; Sort windows according to increasing wavelength
;
  IF N_ELEMENTS(WINDOW) GT 1 THEN BEGIN 
     ix =  SORT(QLDS.detdesc(WINDOW).wavelength)
     WINDOW = WINDOW(ix)
  END
  
  dims = gt_dimension(QLDS)
  
  ;; Creating a spectrum is possible even with single exposures
  ;;
  
  IF NOT KEYWORD_SET(noparcheck) THEN BEGIN
     IF dims.ssolar_x EQ 1 THEN default,xix,0
     PARCHECK,xix,0,typ(/NAT),0,'XIX',$
        minval=0,maxval=dims.ssolar_x-1
     
     IF dims.ssolar_y EQ 1 THEN default,yix,0
     PARCHECK,YIX,0,typ(/NAT),0,'YIX',$
        minval=0,maxval=dims.ssolar_y-1
     
     IF dims.sdel_time LE 1 THEN default,tix,0
     
     PARCHECK,TIX,0,typ(/NAT),0,'TIX',$
        minval=0,maxval=(dims.sdel_time-1)>0
  ENDIF
  
  default,xix,0
  default,yix,0
  default,tix,0
  
  default,nodeselect,1
  
  IF NOT KEYWORD_SET(noparcheck) THEN  $
     parcheck,nodeselect,0,typ(/NAT),0,'NODESELECT'
  
  deselect = NOT nodeselect
  detector = gt_detector(QLDS)
  
  extra = NOT KEYWORD_SET(noextra)
  
  missing = qlds.detdesc(0).missing
  
  spec = [0]
  detx = [0]
  
  IF deselect THEN BEGIN
     IF detector EQ 'NIS' THEN BEGIN
        NX = 1024
        spec=REPLICATE(missing,nx)
        IF extra THEN detx=INDGEN(NX)
     END ELSE BEGIN
        NX = 2048
        spec=REPLICATE(missing,NX)
        IF extra THEN detx=INDGEN(NX) + NX*(waveband-1)
     END
  END
  
;
; Slitpix/DETY and MIRPIX
; For extractions involving more than one wavelength band on the NIS,
; DETY is variable, and has to be accumulated (along the same lines as
; DETX).
; 
  IF blank THEN WINDOW = 0 ;; Dummy
  
  IF extra THEN BEGIN
     IF NOT deselect THEN nx = 1
     IF detector EQ 'NIS' THEN BEGIN
        Dety = REPLICATE(yix + MIN([QLDS.DetDesc(WINDOW).dety]),nx)
        MIRPIX = XIX
     END ELSE BEGIN
        dety = yix
        mirpix = xix
     ENDELSE
  END
  
  oldsys = tag_exist(qlds,'DETDATA',/top_level)
  
  IF blank THEN maxwin = -1 $
  ELSE          maxwin = N_ELEMENTS(WINDOW)-1
  
  FOR wndix = 0,maxwin DO BEGIN
     wnd = WINDOW(wndix)
     syst = systime(1)
     desc = QLDS.DetDesc(wnd)
     IF desc.ixstop(0) GE 0 THEN BEGIN
        
        ;; Security 
        catch,error
        IF error NE 0 THEN BEGIN
           catch,/cancel
           IF N_ELEMENTS(value) GT 0 AND NOT oldsys THEN  $
              st_windata,qlds,wnd,value
           MESSAGE,"Error caught: "+!ERR_STRING
        END
        
        ;; Find data
        value = gt_windata(qlds,wnd,/no_copy)
        IF dims.swavelnth EQ 0 THEN $
           data = value(xix,yix,tix) $
        ELSE  $
           data = value(*,xix,yix,tix)
        IF NOT oldsys THEN st_windata,qlds,wnd,value,/no_copy
        catch,/cancel
        
        ;; Correct for binning in the detector X direction
        IF desc.binx NE 1 THEN $
           data = rebin([data]/FLOAT(desc.binx),$
                        (Size([data]))(1)*desc.binx,/sample)
        
        ;; Correct for binning in detector Y direction
        IF detector EQ 'NIS' THEN BEGIN
           IF desc.biny NE 1 THEN $
              data = data/FLOAT(desc.biny)
           ;; Resizing not necessary for a spectrum
        ENDIF

        IF deselect THEN BEGIN
           spec((desc.detx MOD NX)) = data
           ;; Detector Y may vary between windows in NIS data
           IF extra AND detector EQ 'NIS' THEN BEGIN
              dety0 = yix + qlds.detdesc(wnd).dety + 0*FIX(data)
              dety(desc.detx MOD nx) = dety0
           END
        END ELSE BEGIN
           spec = [spec,data]
           IF extra THEN BEGIN
              sz = SIZE(data)
              ;; Mirpix is constant -- no action
              ;; Create block with DETX values, increasing in the 1st dimension
              dtx = desc.detx + INDGEN(sz(1))
              detx = [detx,dtx]
              
              ;; Detector Y may vary between windows in NIS data
              IF detector EQ 'NIS' THEN BEGIN
                 dety0 = yix + qlds.detdesc(wnd).dety + 0*FIX(data)
                 ;; (But it's constant within each window for a spectrum)
              END ELSE BEGIN
                 ;; Detector Y is constant (thus not accumulated)
              END
              IF detector EQ 'NIS' THEN dety = [dety,dety0] ; Constant for GIS
           END
        END
     ENDIF ;; ixstop(0) ge 0 
  EndFOR
  
  IF nodeselect THEN BEGIN
     IF (SIZE(spec))(1) EQ 1 THEN BEGIN
        MESSAGE,"None of the requested data was present in memory",$
           /informational
     END ELSE BEGIN
        spec = spec(1:*)
        IF extra THEN BEGIN
           detx = detx(1:*)
           IF detector EQ 'NIS' THEN dety = dety(1:*)
        END
     END
  EndIF
  
  ;; yix = dety - MIN(dety) -- Alerady supplied for a point spectrum
  ;; ix = WHERE(yix GT dims.ssolar_y)
  ;; yix(ix) = yix(ix) - MIN(yix(ix))
  
  IF extra THEN BEGIN
     WAVELENGTH = GT_lambda(QLDS,DETX,dety,mirpix)
     SOLARX = GT_SOLARX(QLDS,DETX,dety,mirpix)
     SOLARY = GT_SOLARY(QLDS,DETX,dety,mirpix)
     TIME = GT_CDS_TIME(QLDS,XIX,YIX,TIX)
     ;; Since solary *should* be constant in this case (no misalignment):
     ;;solary = solary(0)
  END
  RETURN,spec
  
END
