;+
; Project     : SOHO - CDS     
;                   
; Name        : GT_START()
;               
; Purpose     : Gets the exposure start times from a file or data structure.
;               
; Explanation : Depending on the nature of the input parameter (structure or
;               file name) the routine extracts and returns the exposure
;               start times used in the raster.
;               
; Use         : IDL> s = gt_start(file_str [,/EXPOSURE, /INT, /EXT, /ECS, $
;           /DATE_ONLY,/TIME_ONLY,/UPPERCASE,/TRUNCATE,/VMS,/STIME,/ERRMSG])
;		e.g.:
;		IDL>print,gt_start(QLDS,/ecs ) --->1996/08/28 11:57:14.426
;		IDL>print,gt_start(QLDS,/vms ) --->28-Aug-1996 11:57:14.426
;		IDL>print,gt_start(QLDS,/ecs,time_only,truncate)-->11:57:14
;
; Inputs      : file_str   -  either:-   the name of a CDS FITS file
;                                 or:-   the name of a CDS data structure
;               
; Opt. Inputs : None
;               
; Outputs     : Function returns the exposure start times.  The default
;               format is standard CCSDS string with other options available
;               via keywords; you can either have the /INTERNAL, or /EXTERNAL,
;		or one or more of the UTC2STR.PRO keywords. 
;		See UTC2STR.PRO  for details about formats!!
;               
; Opt. Outputs: None
;               
; Keywords    : EXPOSURE  -  return start time for all exposures, otherwise
;                            just the raster start.
;               INTERNAL  -  return times in internal CDS format.
;               EXTERNAL  -  return times in external CDS format.
;
;               those of UTC2STR.PRO:              
; 		ECS       -  return times in ECS string format.
;		VMS       = If set, then the output will be in VMS format
;		STIME     = If set, then the output will be in STIME format
;		TRUNCATE  = If set, then the time will be truncated to 
;			    1 second accuracy (rounding down)
;		DATE_ONLY = If set, then only the date is returned.
;		TIME_ONLY = If set, then only the time is returned.
;		UPPERCASE = If set, then the month field in either the VMS or
;                           STIME format is returned as uppercase.
;               ERRMSG    = If defined and passed, then any error messages 
;                           will be returned to the user in this parameter 
;                           rather than being handled by the IDL MESSAGE 
;                           utility.  If no errors are encountered, then a 
;                           null string is returned.		
;
;
; Calls       : FX... routines, INT2UTC,UTC2STR
;
; Common      : None
;               
; Restrictions: Reads OBS_DATE instead of DATE_OBS from calibration files.
;               
; Side effects: None
;               
; Category    : Data_handling, FITS
;               
; Prev. Hist. : None
;
; Written     : C D Pike, RAL, 24-May-94
;               
; Modified    : Fix bug in structure tag name in file case.  CDP, 14-Jun-94
;               Insert cal/ops test.  CDP, 15-Jun-94
;
;               Fix bug caused by different tag names in cal/ops and
;               change .dat --> .mjd  and now that delta times are in
;               seconds not msec.    CDP, 7-Feb-95
;        
;               Handle case of /exp keyword and only one exposure. 
;                                    CDP, 6-Jun-95
;               Substituted DELTDATA --> DEL_TIMEDATA
;               Keep 2-d format if raster was 2-d (eg GIS with slit & mirror
;               movements).          CDP, 13-Jan-96
;               Version 7, SVHH, UiO, 28 February 1996
;                          Fixed spelling mistake (ev(over) => et(over))
;               Check for bad time values.  CDP, 18-Mar-96
;
;		Version 9, GDZ, UCLAN, 15-Oct-1996  added 
;		all the UTC2STR.PRO KEYWORDS 
;
; Version     : Version 9, 15-Oct-1996
;-            

function gt_start,input,EXPOSURE=EXPOSURE, INTERNAL=INTERNAL, $
			EXTERNAL=EXTERNAL,ECS=ECS,DATE_ONLY=DATE_ONLY,$
			TIME_ONLY=TIME_ONLY, UPPERCASE=UPPERCASE,$
			TRUNCATE=TRUNCATE, VMS=VMS,STIME=STIME,ERRMSG=ERRMSG

;
;  need at least one parameter
;
if n_params() eq 0 then begin
   print,'USE:   IDL> start = get_start(file_or_structure)
   return,0
endif

;
;  what was the input?
;
case datatype(input,1) of
;
;  use structure directly
;
   'Structure': begin
                   qlmgr, input, valid
                   if valid then begin
                      rstart = input.header.date_obs
                      dt = input.del_timedata
                   endif else begin
                      print,'GT_START: Structure supplied is non-standard'
                      return,0
                   endelse
                end
;
;  or allow user to have own version of FITS files
;
      'String': begin
                   tfile = ''
                   if  file_exist(input) then begin
                      tfile = input
                   endif else begin
                     if file_exist(concat_dir('$CDS_CAL_FITS',input)) then begin
                        tfile = concat_dir('$CDS_CAL_FITS',input)
                     endif
                   endelse

;
;  FITS file doesn't exist in user or std directory
;
                   if tfile eq '' then begin
                      print,'GT_START:  Input file does not exist.'
                      return,0
                   endif

                   fxbopen,unit,tfile,1,header
                   if fxpar(header,'TELESCOP') eq 'CALIBRATION' then begin
                      rstart = fxpar(header,'OBS_DATE')
                   endif else begin
                      rstart = fxpar(header,'DATE_OBS')
                   endelse
                   if keyword_set(exposure) then begin
                      if fxpar(header,'TELESCOP') eq 'CALIBRATION' then begin
                         dt = cal_AuxRead(unit,header,'DEL_TIME',status=status)
                      endif else begin
                         dt = AuxRead(unit,header,'DEL_TIME',status=status)
                      endelse
                      if n_elements(status) gt 0 then begin
                         if status ne '' then begin
                            print,'Error reading delta times in FITS file.'
                            fxbclose,unit
                            return,0
                         endif
                      endif
                      if fxpar(header,'TELESCOP') eq 'CALIBRATION' then begin
                         dt = dt.d
                      endif else begin
                         dt = dt.data
                      endelse
                   endif
                   fxbclose,unit
                end
          else: begin
                   print,'Input must be a file name or a CDS data structure.'
                   return,0
                end
endcase

;
;  return raster start if exposures not requested
;
if (not keyword_set(exposure)) or (n_elements(dt) eq 1) then begin
   et = str2utc(rstart)
endif else begin
;
;  else continue to calculate exposure start times
;

;
;  get internal format for raster start
;
   utc = str2utc(rstart)

;
;  duplicate structure for number of exposures
;
   s = size(dt)
   if s(0) eq 1 then begin 
      et = replicate(utc,n_elements(dt))
   endif else begin
      et = replicate(utc,s(1),s(2))
   endelse
;
;  assume for now the day does not change
;
   et(*).mjd = utc.mjd

;
;  simply add the millisec offset start time
;
   et(*).time = utc.time + long(reform(dt,n_elements(dt))*1000.)

;
;  check for duff times
;
   nbad = where(dt eq -1.0,count)
   if count gt 0 then begin
      et(nbad).time  = 0L
      et(nbad).mjd   = -1L
   endif

;
;  now check for day overflows and assume cannot span >2 days
;
   over = where(et.time gt 86400000L)
   if over(0) ne -1 then begin
      et(over).mjd = et(over).mjd + 1
      et(over).time = et(over).time - 86400000L
   endif
endelse

;
;  translate back to required format and return
;----------------------------------------------
ERRMSG = ''

IF keyword_set(EXTERNAL) then return,int2utc(et) $
ELSE IF keyword_set(INTERNAL) then return,et $
ELSE return,utc2str(et,ECS=ECS,DATE_ONLY=DATE_ONLY,TIME_ONLY=TIME_ONLY,$
     UPPERCASE=UPPERCASE,TRUNCATE=TRUNCATE, VMS=VMS,STIME=STIME,ERRMSG=ERRMSG)

end


