;+
; Project     : SOHO - CDS     
;                   
; Name        : GT_VALID()
;               
; Purpose     : Determine if keyword values are valid for gt_xxx calls.
;               
; Explanation : This routine may be used to test for the validity of
;               keyword values before calling e.g., GT_SCANX/Y/T/P or
;               GT_IIMAGE etc. Useful when the keyword values are
;               taken from user input.
;               
; Use         : VALID = GT_VALID(QLDS, ...)
;    
; Inputs      : QLDS : Quick Look Data Structure.
;               
; Opt. Inputs : None.
;               
; Outputs     : Returns 1 if no inconsistencies are found, zero otherwise.
;               
; Opt. Outputs: None.
;               
; Keywords    : XIX : X index, scalar or array    ; Of those that are defined,
;               YIX : Y index, scalar or array    ; these must have identical
;               TIX : Time index, scalar or array ; sizes, and be within
;                                                 ; the valid ranges.
;                                                 ; DIMENSION < 2
;
;               BAND : Scalar, causes error if no loaded data in this band.
;               DETX : Scalar. Combination of BAND/DETX tests for loaded data 
;                      covering that pixel. DETX cannot be set unless BAND
;                      is set.
;
;               WINDOW : Scalar or array, string or integer. Causes error
;                        if out of range or referring to an unloaded window.
;                        If BAND and WINDOW is set then all WINDOWs must
;                        be in BAND.
;
;               OFFSET : Scalar. Must be 0 < OFFSET < WINDOW_WIDTH
;                        Cannot be set without also setting window to a
;                        scalar value.
;                        
;               LAMBDA : Scalar or array. Checks to see if there's data
;                        present at that wavelength. If it's an array with
;                        2 elements (representing lower/upper integration
;                        limits for GT_BIMAGE) they should refer to the
;                        same wavelength band.
;
;               NOCHECK: Don't check that the QLDS is a QLDS
;
; Calls       : ANYTIM2UTC(), DATATYPE(), GT_DETECTOR(), GT_DIMENSION(),
;               GT_WAVEBAND() GT_WINDOW(), LOAD_WAVECAL(), PARCHECK, QLMGR,
;               TRIM(), TYP(), WAVE2PIX()
;
; Common      : None.
;               
; Restrictions: None.
;               
; Side effects: None.
;               
; Category    : Get routines
;               
; Prev. Hist. : None.
;
; Written     : Stein Vidar H. Haugan, UiO, 27 February 1996
;               
; Modified    : Version 2, SVHH, 14 April 1996
;                       Using LOAD_WAVECAL().
;
; Version     : 2, 14 April 1996
;-            

FUNCTION GT_VALID,QLDS,XIX=XIX,YIX=YIX,TIX=TIX,LAMBDA=LAMBDA,BAND=waveBAND,$
                  DETX=DETX,WINDOW=WINDOW,OFFSET=OFFSET,NOCHECK=NOCHECK
  
  ON_ERROR,2
  IF !debug GT 0 THEN ON_ERROR,0
  
  IF NOT KEYWORD_SET(nocheck) THEN BEGIN
     qlmgr,qlds,valid
     IF NOT valid THEN BEGIN
        MESSAGE,"First argument must be a QLDS",/informational
        RETURN,0
     END
  ENDIF
  
  detector = gt_detector(qlds)
  IF detector EQ 'NIS' THEN maxdetx = 1023 ELSE maxdetx=2047
  
  dummy = load_wavecal(detector,anytim2utc(qlds.header.date_obs), $
                       cal_struct = qlds.wavecal)
  
  IF N_ELEMENTS(WINDOW) GT 0 THEN BEGIN
     
     ;; We cannot use gt_wnum to convert strings->window numbers.
     IF datatype(WINDOW) EQ 'STR' THEN BEGIN
        wnd = intarr(N_ELEMENTS(WINDOW))
        FOR i = 0,N_ELEMENTS(WINDOW)-1 DO BEGIN
           ix = WHERE(qlds.detdesc(*).label EQ STRUPCASE(WINDOW(i)))
           ;; Error if no window exists with that name
           IF ix(0) EQ -1 THEN RETURN,0
           wnd(i) = ix(0)
        END
        IWINDOW = wnd
     END ELSE IWINDOW = WINDOW
     
     ;; Any window number too large/small?
     ix = WHERE(IWINDOW LT 0 OR IWINDOW GE N_ELEMENTS(qlds.detdesc))
     IF ix(0) NE -1 THEN RETURN,0
     
     ;; Any unloaded windows referred to?
     ix = WHERE(qlds.detdesc(IWINDOW).ixstop(0) LT 0)
     IF ix(0) NE -1 THEN RETURN,0
  END
  
     
  IF N_elements(waveband) GT 0 THEN BEGIN
     
     ;; General max/min check
     IF detector EQ 'NIS' THEN maxband=2 ELSE maxband=4
     parcheck,waveband,0,typ(/NAT),0,'(wave)BAND',minval=1,maxval=maxband,$
        /noerr,result=r
     IF r(0) NE '' THEN RETURN,0
     
     ;; What windows do we have in this band?
     
     cand=INDGEN(N_elements(QLDS.DetDesc))
     cand=WHERE(QLDS.DetDesc(cand).waveband EQ waveband,count)
     IF count EQ 0 THEN RETURN,0
     
     ;; Are all the *specified* windows in the desired waveband?
     IF N_ELEMENTS(IWINDOW) GT 0 THEN BEGIN  ;; Only a subset of the windows
        ix = WHERE(qlds.detdesc(IWINDOW).waveband NE waveband)
        IF ix(0) NE -1 THEN RETURN,0
        cand = IWINDOW
     END
     
     ;; Check that at least one window is loaded.
     ix = WHERE(qlds.detdesc(cand).ixstop(0) GT 0)
     IF ix(0) EQ -1 THEN RETURN,0
  ENDIF
  
  IF N_ELEMENTS(detx) GT 0 THEN BEGIN
     
     ;; Band *must* be specified for detx to have any meaning.
     IF N_ELEMENTS(BAND) EQ 0 THEN RETURN,0
     
     ;; Min/max/dimensionality check
     parcheck,detx,0,typ(/nat),0,'DETX',minval=0,maxval=maxdetx,/noerr,res=r
     IF r(0) NE '' THEN RETURN,0
     
     win = gt_window(qlds,band,detx,/loaded)
     IF win(0) EQ -1 THEN RETURN,0
     
  END
  
  IF N_ELEMENTS(OFFSET) GT 0 THEN BEGIN
     
     ;; Window must be specified (and scalar)
     IF N_ELEMENTS(iwindow) NE 1 THEN RETURN,0
     
     IF offset GT (qlds.detdesc(iwindow).ixstop(0) $
                   -qlds.detdesc(iwindow).ixstart(0)) THEN RETURN,0
     IF offset LT 0 THEN RETURN,0
     
  END
  
  
  IF N_ELEMENTS(lambda) GT 0 THEN BEGIN
     IF N_ELEMENTS(lambda) EQ 1 THEN BEGIN
        ;; Single lambda, find its position
        b = gt_waveband(detector,lambda)
        IF b EQ 0 THEN RETURN,0
        pix = wave2pix(detector+trim(b),lambda)
        IF pix EQ -1 THEN RETURN,0
        
        ;; Any loaded windows covering this pixel?
        w = gt_window(qlds,b,pix,/loaded)
        IF w(0) EQ -1 THEN BEGIN
           IF (gt_window(qlds,b,(pix-1) > 0,/loaded))(0) EQ -1 AND $
              (gt_window(qlds,b,(pix+1) < maxdetx,/loaded))(0) EQ -1  $
              THEN RETURN,0
        END
     END ELSE BEGIN
        ;; General checks
        IF N_ELEMENTS(lambda) NE 2 THEN RETURN,0
        parcheck,lambda,0,typ(/rea),1,'LAMBDA',/noerr,res=r
        IF r(0) NE '' THEN RETURN,0

        ;; What waveband
        b = gt_waveband(detector,lambda(0))
        IF b EQ 0 THEN RETURN,0
        
        ;; Is number two in the same waveband?
        b2 = gt_waveband(detector,lambda(1))
        IF b2 NE b THEN RETURN,0
        
        pix1 = wave2pix(detector+trim(b),lambda(0))
        IF pix1 EQ -1 THEN RETURN,0
        pix2 = wave2pix(detector+trim(b),lambda(0))
        IF pix2 EQ -1 THEN RETURN,0
        
        IF (gt_window(qlds,b,pix1,/loaded))(0) EQ -1 AND $
           (gt_window(qlds,b,pix1+1,/loaded))(0) EQ -1 THEN RETURN,0
        FOR pix = pix1+1,pix2-1 DO BEGIN
           IF (gt_window(qlds,b,pix,/loaded))(0) EQ -1 THEN RETURN,0
        END
     END
  END
  
  dim = gt_dimension(qlds)
  
  IF N_ELEMENTS(xix) GT 0 THEN BEGIN
     parcheck,xix,0,typ(/nat),[0,1,2],'XIX',minval=0, $
        maxval=(dim.ssolar_x-1)>0,/noerr,result=r
     IF r(0) NE '' THEN RETURN,0
  END
  
  IF N_ELEMENTS(yix) GT 0 THEN BEGIN
     parcheck,yix,0,typ(/nat),[0,1,2],'YIX',minval=0, $
        maxval=(dim.ssolar_y-1)>0,/noerr,result=r
     IF r(0) NE '' THEN RETURN,0
  END
  
  IF N_ELEMENTS(tix) GT 0 THEN BEGIN
     parcheck,tix,0,typ(/nat),[0,1,2],'TIX',minval=0, $
        maxval=(dim.sdel_time-1)>0,/noerr,result=r
     IF r(0) NE '' THEN RETURN,0
  END

  RETURN,1
  
END






