;+
; Project     : SOHO - CDS     
;                   
; Name        : GT_WAVEBAND()
;               
; Purpose     : Return the waveband number given detector and wavelengt
;               
; Explanation : Based on the input detector name ("GIS" or "NIS"), the
;               waveband number corresponding to the supplied wavelength is
;               returned.
;               
; Use         : BAND = GT_WAVEBAND(DETECTOR,LAMBDA)
;    
; Inputs      : DETECTOR: Either "NIS" or "GIS"
;               
;               LAMBDA: Wavelength.
;
; Opt. Inputs : None.
;               
; Outputs     : Returns the waveband number, or 0 if outside limits.
;               
; Opt. Outputs: None.
;               
; Keywords    : None.
;
; Calls       : PARCHECK, PIX2WAVE(), TRIM(), TYP()
;
; Common      : None.
;               
; Restrictions: Wavelength should be within specified limits.
;               Only 1st order calculations are made.
;               
; Side effects: None.
;               
; Category    : GET
;               
; Prev. Hist. : None.
;
; Written     : SVH Haugan, UiO, 27-October-1995
;               
; Modified    : Version 2, 8-December-1995, SVHH
;                          Fixed error FOR the GIS case.
;
; Version     : 2, 8-December-1995
;-            

FUNCTION gt_waveband,detector,lambda

  IF N_PARAMS() LT 2 THEN  $
     MESSAGE,"Use: BAND = GT_WAVEBAND(detector,lambda)"
  
  parcheck,detector,1,typ(/str),0,'DETECTOR'
  slen = STRLEN(detector)
  detector = STRUPCASE(detector)
  IF detector EQ STRMID("NIS",0,slen) THEN detector = "NIS" $
  ELSE IF detector EQ STRMID("GIS",0,slen) THEN detector = "GIS" $
  ELSE MESSAGE,"DETECTOR must be either 'NIS' or 'GIS'"
  
  parcheck,lambda,2,typ(/rea),0,'LAMBDA'
  
  IF detector EQ "NIS" THEN BEGIN
     band1 = pix2wave("NIS1",[0,1023])
     IF lambda GE band1(0) AND lambda LE band1(1) THEN RETURN,1
     band2 = pix2wave("NIS2",[0,1023])
     IF lambda GE band2(0) AND lambda LE band2(1) THEN RETURN,2
     MESSAGE,"Wavelength outside NIS range, zero returned",/informational
     RETURN,0
  END ELSE BEGIN
     FOR band = 1,4 DO BEGIN
        lam = pix2wave("GIS"+trim(band),[0,2047])
        IF lambda GE lam(0) AND lambda LE lam(1) THEN RETURN,band
     ENDFOR
     MESSAGE,"Wavelength outside GIS range, zero returned",/informational
  END
  
END

