;+
; Project     : SOHO - CDS     
;                   
; Name        : GT_WINDATA()
;               
; Purpose     : Return block of detector data from one detector window.
;               
; Explanation : This function is meant as a quick and safe way of extracting
;               the full data "cube" corresponding to a data extraction window
;               in a Quick Look Data Structure.
;
;               Example:
;               Let's say you have a GIS raster scanning 15x15 points in a
;               30x30 arcsecond area of the sun, and you want the
;               averaged spectrum from band 1 for the whole area,
;               you first locate which data window corresponds to
;               wavelength band 1 (use e.g., gt_wlimits). Let's say
;               it's window 0. Now you can use:
;
;               data = gt_windata(qlds,0) ;; Returns (2048,15,15) array.
;               data = total(data,2)      ;; Sum to  (2048,   15) array
;               data = total(data,2)      ;; Sum to  (2048).
;               data = data/gt_numexp(qlds) ;; Average. Done!
;               
; Use         : data=gt_windata(QLDS,WINDOWI)
;    
; Inputs      : QLDS : Quick Look Data Structure.
;               
;               WINDOWI : Scalar integer/long with the index of the
;                         desired data window.
; Opt. Inputs : None.
;               
; Outputs     : Returns the block of data for the data extraction window.
;               
; Opt. Outputs: None.
;               
; Keywords    : NOCHECK : Don't check that the QLDS is a QLDS.
;
;               QUICK : Set NOCHECK, and don't check the validity of
;                       WINDOWI.
;
;               NO_COPY : Set to use the NO_COPY keyword when getting the
;                         data from the handle. REMEMBER TO PUT THE DATA BACK
;                         WITH ST_WINDATA IF YOU USE THIS KEYWORD.
;
;		NOPADDING : If set, then the window will be returned without
;			    padding.  Thus, different data windows could be
;			    returned with different sizes.  For example, when
;			    reading in full NIS readout, the window for NIS1
;			    will have a different height than that for NIS2,
;			    because of the different slants in each band.
;
;               INVERT : If set, and if the data were taken when SOHO was
;                        upside-down, then the X and Y axes will be flipped.
;
;               ERRMSG : Set to a defined variable to receive an error
;                        message through this keyword instead of crashing.
;
; Calls       : default, handle_info(), parcheck, qlmgr, tag_exist(), typ()
;
; Common      : None.
;               
; Restrictions: None.
;               
; Side effects: None.
;               
; Category    : Data_handling, Get
;               
; Prev. Hist. : Prompted by request from Eddie Breeveld.
;
; Written     : Stein Vidar H. Haugan, UiO, 8 April 1996
;               
; Modified    : Version 2, SVHH, 12 August 1996
;                       Compatible with old/new storage systems.
;                       Added keyword errmsg.
;               Version 3, SVHH, 18 August 1996
;                       Made the old storage system use both ixstart/ixstop
;                       when extracting data, to make cds_pad_qlds work
;                       better, with correct filling-in of missing values.
;		Version 4, 13-May-1997, William Thompson, GSFC
;			Added keyword NOPADDING
;               Version 5, 03-Aug-2004, William Thompson, GSFC
;                       Added keyword INVERT
;                       
; Version     : Version 4, 03-Aug-2004
;-            

FUNCTION gt_windata,qlds,windowi,nocheck=nocheck,quick=quick,$
                    no_copy=no_copy,nopadding=nopadding,errmsg=errmsg,$
                    invert=invert
  
  ON_ERROR,2
  IF !debug GT 0 THEN ON_ERROR,0
  
  default,no_copy,0
  
  IF NOT KEYWORD_SET(quick) THEN BEGIN
     IF N_PARAMS() NE 2 THEN MESSAGE,"Use: data = gt_windata(qlds,windowi)"
     
     IF NOT KEYWORD_SET(nocheck) THEN BEGIN
        qlmgr,qlds,valid
        IF NOT valid THEN BEGIN
           MESSAGE = "First parameter must be a " +$
              "Quick Look Data Structure (QLDS)"
           GOTO,handle_error
        END
     END
     
     noerror = N_ELEMENTS(errmsg) GT 0 
     parcheck,windowi,2,typ(/nat),0,"WINDOWI", noerror=noerror,$
        minval=0,maxval=N_ELEMENTS(QLDS.DETDESC)-1,result=result
     
     IF KEYWORD_SET(result) THEN BEGIN
        MESSAGE = result
        GOTO,handle_error
     END
  END
  
  IF qlds.detdesc(windowi).ixstop(0) LT 0 THEN BEGIN
     MESSAGE = "Data window "+qlds.detdesc(windowi).label+" is not loaded"
     GOTO,HANDLE_ERROR
  END
  
     
  oldsys = tag_exist(qlds,"DETDATA",/TOP_LEVEL)
     
  IF NOT oldsys THEN BEGIN
     handle = qlds.detdesc(windowi).handle
     h_status = handle_info(handle,/valid_id)
     IF h_status EQ 0 THEN MESSAGE,"Data window not available"
     
     handle_value,handle,value,no_copy=no_copy
     
     IF N_ELEMENTS(value) EQ 0 THEN BEGIN
        MESSAGE = "Data window not available -- lost?"
        GOTO,handle_error
     END

     IF KEYWORD_SET(NOPADDING) THEN BEGIN
	Q = QLDS.DETDESC(WINDOWI)
	E = Q.IXSTOP - Q.IXSTART - Q.PADDING
	CASE N_ELEMENTS(E) OF
	    1: VVAL = VALUE(0:E(0))
	    2: VVAL = VALUE(0:E(0),0:E(1))
	    3: VVAL = VALUE(0:E(0),0:E(1),0:E(2))
	    4: VVAL = VALUE(0:E(0),0:E(1),0:E(2),0:E(3))
	    ELSE: BEGIN
		MESSAGE = "Don't recognize data format"
		GOTO,handle_error
            END
	ENDCASE
     ENDIF ELSE VVAL = VALUE

  END ELSE BEGIN
     b = qlds.detdesc(windowi).ixstart
     e = qlds.detdesc(windowi).ixstop
     if keyword_set(nopadding) then e = e - qlds.detdesc(windowi).padding
     CASE N_ELEMENTS(b) OF 
        1: VVAL = qlds.detdata(b(0):e(0))
        2: VVAL = qlds.detdata(b(0):e(0),b(1):e(1))
        3: VVAL = qlds.detdata(b(0):e(0),b(1):e(1),b(2):e(2))
        4: VVAL = qlds.detdata(b(0):e(0),b(1):e(1),b(2):e(2),b(3):e(3))
        ELSE: BEGIN
           MESSAGE = "Don't recognize data format"
           GOTO,handle_error
        END
    END
  END
  
;
;  If /INVERT was passed, then check whether or not the image needs to be
;  flipped.
;
  IF KEYWORD_SET(INVERT) AND (QLDS.DETDESC(WINDOWI).SPACING(1) LT 0) THEN BEGIN
      SZ = SIZE(VVAL)
      CASE SZ(0) OF
          2: VVAL = REARRANGE(VVAL, [1,-2])
          3: VVAL = REARRANGE(VVAL, [1,-2,-3])
          4: VVAL = REARRANGE(VVAL, [1,-2,-3,4])
          ELSE: VVAL = VVAL
      ENDCASE
  ENDIF
  RETURN, VVAL

HANDLE_ERROR:
  IF N_elements(errmsg) NE 0 THEN BEGIN
     errmsg = MESSAGE
     RETURN,-1
  END ELSE BEGIN
     FOR i=0,N_ELEMENTS(MESSAGE)-1 DO BEGIN
        MESSAGE,MESSAGE(i),/continue
     END
     MESSAGE,"ABORT"
  END
END
