;+
; Project     : SOHO - CDS     
;                   
; Name        : GT_WINDESC()
;               
; Purpose     : Return descriptor structure for one detector window.
;               
; Explanation : This function is meant as a quick and safe way of extracting
;               the description of a data extraction window in a Quick Look
;               Data Structure.
;
; Use         : data=gt_windesc(QLDS,WINDOWI)
;    
; Inputs      : QLDS : Quick Look Data Structure.
;               
;               WINDOWI : Scalar integer/long with the index of the
;                         desired data window.
; Opt. Inputs : None.
;               
; Outputs     : Returns the descriptor structure for the data 
;               extraction window.
;               
; Opt. Outputs: None.
;               
; Keywords    : NOCHECK : Don't check that the QLDS is a QLDS.
;
;               QUICK : Set NOCHECK, and don't check the validity of
;                       WINDOWI.
;
;               ERRMSG : Set to a defined variable to receive an error
;                        message through this keyword instead of crashing.
;
; Calls       : parcheck, qlmgr, tag_exist(), typ()
;
; Common      : None.
;               
; Restrictions: None.
;               
; Side effects: None.
;               
; Category    : Data_handling, Get
;               
; Prev. Hist. : Modified from GT_WINDATA.
;
; Written     : William Thompson, GSFC, 6 January 1997
;               
; Modified    : Version 1, William Thompson, GSFC, 6 January 1997
;		Version 2, William Thompson, GSFC, 28 February 2000
;			Modified to not check if window loaded when /QUICK is
;			used.
;                       
; Version     : Version 2, 28 February 2000
;-            

FUNCTION gt_windesc,qlds,windowi,nocheck=nocheck,quick=quick,errmsg=errmsg
  
  ON_ERROR,2
  IF !debug GT 0 THEN ON_ERROR,0
  
  IF NOT KEYWORD_SET(quick) THEN BEGIN
     IF N_PARAMS() NE 2 THEN MESSAGE,"Use: data = gt_windesc(qlds,windowi)"
     
     IF NOT KEYWORD_SET(nocheck) THEN BEGIN
        qlmgr,qlds,valid
        IF NOT valid THEN BEGIN
           MESSAGE = "First parameter must be a " +$
              "Quick Look Data Structure (QLDS)"
           GOTO,handle_error
        END
     END
     
     noerror = N_ELEMENTS(errmsg) GT 0 
     parcheck,windowi,2,typ(/nat),0,"WINDOWI", noerror=noerror,$
        minval=0,maxval=N_ELEMENTS(QLDS.DETDESC)-1,result=result
     
     IF KEYWORD_SET(result) THEN BEGIN
        MESSAGE = result
        GOTO,handle_error
     END
  
     IF qlds.detdesc(windowi).ixstop(0) LT 0 THEN BEGIN
        MESSAGE = "Data window "+qlds.detdesc(windowi).label+" is not loaded"
        GOTO,HANDLE_ERROR
     END
  END
  
     
  oldsys = tag_exist(qlds,"DETDATA",/TOP_LEVEL)
     
  return, QLDS.DetDesc(windowi)

  
HANDLE_ERROR:
  IF N_elements(errmsg) NE 0 THEN BEGIN
     errmsg = MESSAGE
     RETURN,-1
  END ELSE BEGIN
     FOR i=0,N_ELEMENTS(MESSAGE)-1 DO BEGIN
        MESSAGE,MESSAGE(i),/continue
     END
     MESSAGE,"ABORT"
  END
END
