;+
; Project     : SOHO - CDS     
;                   
; Name        : GT_WINDOW()
;               
; Purpose     : Find a (valid) window from BAND + DETX
;               
; Explanation : Returns the index of those windows covering a specific
;               dispersion pixel. If no windows meet the criteria, -1 is
;               returned.
;               
; Use         : result = GT_WINDOW(QLDS, BAND,DETX)
;    
; Inputs      : QLDS: CDS QuickLook Data structure
;
;               BAND: Detector band number.
;
;               DETX: Detector X pixel.
;               
; Opt. Inputs : None.
;               
; Outputs     : Returns indices (possibly an array) of those windows
;               covering pixel DETX in waveband BAND. Returns -1 if none.
;               
; Opt. Outputs: None.
;               
; Keywords    : LOADED: Set to discard those windows that are not
;                       stored in the qlds.detdata array.
;
; Calls       : GT_BINX(), GT_DETECTOR(), GT_DIMENSION(), GT_NUMWIN(),
;               PARCHECK, QLMGR, TYP()
;
; Common      : None.
;               
; Restrictions: Valid inputs are assumed.
;               
; Side effects: None.
;               
; Category    : GET 
;               
; Prev. Hist. :
;
; Written     : SVH Haugan, UiO, 27-October-1995
;               
; Modified    : Version 1.1, SVHH, 4-December-1995
;                    Detector X values taken MOD 2048 (only affects GIS).
;
;               Version 2, SVHH, 15-January-1996
;                    Now using detx + wwidth MINUS ONE before MOD 2048.
;
;               Version 3, SVHH, 6 April 1996
;                    Calculating wwidth from DETDESC.
;
; Version     : 3, 6 April 1996
;-            

FUNCTION gt_window,qlds,band,detx,loaded=loaded
  
  IF N_PARAMS() LT 3 THEN  $
     MESSAGE,"Use: W = gt_window(qlds,band,detx [,/loaded])"
  
  qlmgr,qlds,valid
  IF NOT valid THEN  $
     MESSAGE,"First argument must be a QL data structure'
  
  det = gt_detector(qlds)
  IF det EQ "NIS" THEN maxband = 2 ELSE maxband = 4
  parcheck,band,2,typ(/nat),0,'BAND'
  
  parcheck,detx,3,typ(/nat),0,'DETX'
  
  dims = gt_dimension(qlds)
  
  cand = WHERE(QLDS.DetDesc(*).waveband EQ band,count)
  IF count EQ 0 THEN RETURN,-1
  ix = WHERE((qlds.detdesc(cand).detx MOD 2048) LE detx,count)
  IF count EQ 0 THEN RETURN,-1
  cand = cand(ix)
  wwidth = abs(qlds.detdesc(cand).ixstop(0))-qlds.detdesc(cand).ixstart(0)
  wwidth = wwidth*gt_binx(qlds)
  ix = WHERE(((qlds.detdesc(cand).detx + wwidth) MOD 2048) GE detx,count)
  IF count EQ 0 THEN RETURN,-1
  cand = cand(ix)
  IF KEYWORD_SET(loaded) THEN BEGIN
     ix = WHERE(qlds.detdesc(cand).ixstart(0) GE 0,count)
     IF count EQ 0 THEN RETURN,-1
     cand = cand(ix)
  ENDIF
   
  RETURN,cand
END

