;+
; Project     : SOHO - CDS     
;                   
; Name        : GT_WLIMITS()
;               
; Purpose     : To retrieve the window limits from a file or data structure.
;               
; Explanation : Depending on the nature of the input parameter (structure or
;               file name) the routine extracts and returns the window 
;               limits used in the raster.
;               
; Use         : IDL> s = gt_wlimits(file_str)
;    
; Inputs      : file_str   -  either:-   the name of a CDS FITS file
;                                 or:-   the name of a CDS data structure
;               
; Opt. Inputs : None
;               
; Outputs     : Function returns the window limits used in a (2xN) array 
;               for GIS data or a (4xN) array for VDS data.
;               
; Opt. Outputs: None
;               
; Keywords    : WAVE  - if specified, the window limits in the dispersion
;                       direction are returned as wavelengths.
;               QUIET - no extra info to screen.
;               BAND  - returns the band ID for each window
;               SHORT - gives a compressed version of the output info
;               WSORT - sorts into wavelength order (in SHORT mode only)
;               HARDCOPY - hardcopy listing - only in /SHORT mode
;
; Calls       : FX... routines
;               ANYTIM2UTC(), PIX2WAVE, LOAD_WAVECAL, READCDSFITS
;
; Common      : None
;               
; Restrictions: None
;               
; Side effects: None
;               
; Category    : Data_handling, FITS
;               
; Prev. Hist. : None
;
; Written     : C D Pike, RAL, 26-May-94
;               
; Modified    : Take account of negative .detdesc.ixstart(0) returned when
;               /header keyword used in FITS reading routines.  CDP, 3/6/94
;               Use new file reading routines.  CDP, 14-Jun-94
;               Insert cal/ops test.  CDP, 15-Jun-94
;
;             : Updated NIS band definition and y max limit "error". Also 
;               now lists a summary of limits with a quiet option to over-
;               ride this.      BJKE, 15-Nov-95.
;
;               Add the band keyword.  CDP, 28-Nov-95
;               Version 7, SVHH, UiO, 29 February 1996 
;                       Added some support for old calibration NIS files.
;               Version 8, SVHH, UiO, 14 April 1996
;                       Added calls to load_wavecal for input == QLDS,
;                       and using readcdsfits (which calls load_wavecal)
;                       when input == filename.
;               Add SHORT AND WSORT keywords.  CDP, 27-Mar-96
;               Add HARDCOPY keyword for SHORT mode.  CDP, 8-Sep-1996
;               Fix typo in last update.  CDP, 24-9-96
;               Tidy quiet version.  CDP, 26-Aug-99
;
; Version     : Version 11, 26-Aug-99
;-            

function gt_wlimits, input, wave=wave, quiet=quiet, band=inband,$
                            wsort=wsort, short=short,hardcopy=hardcopy 

;
;  need at least one parameter
;
if n_params() eq 0 then begin
   print,'USE:   IDL> w = get_wlimits(file_or_structure)
   return,0
endif

;
;  hardcopy?
;
if keyword_set(hardcopy) then hc = 1 else hc = 0

;
;  what was the input?
;
case datatype(input,1) of
   'Structure': begin
                   qlmgr, input, valid
                   if valid then begin
                      w = input.detdesc
                      ins = input.header.detector
                   endif else begin
                      print,'GT_WLIMITS: Structure supplied is non-standard'
                      return,0
                   ENDELSE
                   dummy = load_wavecal(input.header.detector, $
                                        anytim2utc(input.header.date_obs),$
                                        cal_struct=input.wavecal)
                end
;
;  allow user to have own version of FITS files
;
      'String': begin
                   tfile = ''
                   if  file_exist(input) then begin
                      tfile = input
                   endif else begin
                     if file_exist(concat_dir('$CDS_CAL_FITS',input)) then begin
                        tfile = concat_dir('$CDS_CAL_FITS',input)
                     endif
                   endelse

;
;  FITS file doesn't exist in user or std directory
;
                   if tfile eq '' then begin
                      print,'GT_WLIMITS:  Input file does not exist.'
                      return,0
                   endif
                   
                   ;; This automatically loads the wavelength calibrations
                   qlds = readcdsfits(tfile,/header)
                   
                   ins = qlds.header.detector
                   w = qlds.detdesc
                END
          else: begin
                   print,'Input must be a file name or a CDS data structure.'
                   return,0
                end
endcase

;
;  default is to give verbose output. Use the quiet switch to switch off.
;

bequiet = 0
if keyword_set(quiet) then bequiet = 1

;
;  load output array depending on which detector was used
;

nw = n_elements(w)


case ins of
   'NIS':  begin
              yix = (WHERE(w(0).axes EQ 'SOLAR_Y'))(0)
              IF yix EQ -1 THEN MESSAGE,"No solar Y dimension for NIS data?"
              if keyword_set(wave) then out=fltarr(4,nw) else out=intarr(4,nw)
              inband = intarr(nw)
              nl = n_elements(w.label)
              text = strarr(nl)
              for i=0,nw-1 do begin
                 out(0,i) =  w(i).detx
                 out(1,i) = abs(w(i).ixstop(0)) - w(i).ixstart(0) + w(i).detx
                 out(2,i) = w(i).dety
                 out(3,i) = w(i).ixstop(yix) - w(i).ixstart(yix) + w(i).dety
                 if out(2,i) lt 512 then inband(i) = 2 else inband(i) = 1
                 if out(2,i) lt 512 then band = 'NIS2' else band = 'NIS1'
                 if (not bequiet) then begin
                    wmin = pix2wave('NIS'+band,out(0,i),/limit)
                    wmax = pix2wave('NIS'+band,out(1,i),/limit)
                    xmin = fix(out(0,i))
                    xmax = fix(out(1,i))
                    ymin = fix(out(2,i))
                    ymax = fix(out(3,i))
;
                    if not keyword_set(short) then begin
                       print,' '
                       print,'       Window:',i,'             Label: ', w(i).label
                       print,'       Band:         ', band
                       print,'       Det-X:         ', xmin, '        to ', xmax
                       print,'       Det-Y:         ', ymin, '        to ', ymax
                       print,'       Wavelength:   ', wmin, '    to', wmax
                    endif else begin
                       if i eq 0 then begin
                          print,'Window        Label         Band    Xmin   '+$
                                'Xmax    Wmin      Wmax'
                       endif
                       text(i) = string(i,strpad(w(i).label,16,/aft),band,$
                                        xmin,xmax,wmin,wmax,format=$
                                '(i3,7x,a,2x,a4,3x,i4,3x,i4,3x,f7.2,3x,f7.2)')
                    endelse
                 endif
                 if keyword_set(wave) then begin
                    out(0,i) = pix2wave('NIS'+band,out(0,i),/limit)
                    out(1,i) = pix2wave('NIS'+band,out(1,i),/limit)
                 endif
              endfor
              if keyword_set(short) then begin
                 if keyword_set(wsort) then begin
                    wmin = float(strmid(text,49,8))
                    ns = sort(wmin)
                    text = text(ns)
                 endif
                 print_str,text,hardcopy=hc,quiet=bequiet
              endif
           end
   'GIS':  begin
              if keyword_set(wave) then out=fltarr(2,nw) else out=intarr(2,nw)
              inband = intarr(nw)
              nl = n_elements(w.label)
              text = strarr(nl)
              for i=0,nw-1 do begin
                 out(0,i) = w(i).detx
                 out(1,i) = abs(w(i).ixstop(0)) - w(i).ixstart(0) + w(i).detx
                 band = fix ( out ( 0, i ) / 2048 ) + 1
                 inband(i) = band
                 if (not bequiet) then begin
                    band = 'GIS'+string(fix(out(0,i)/2048)+1,form='(i1)')
                    wmin = pix2wave( band, ( out(0,i) mod 2048 ), /limit)
                    wmax = pix2wave( band, ( out(1,i) mod 2048 ), /limit)
                    xmin = fix(out(0,i))
                    xmax = fix(out(1,i))
;
                    if not keyword_set(short) then begin
                       print,' '
                       print,'       Window:',i,'             Label: ', w(i).label
                       print,'       Band:         ', band
                       print,'       Det-X:         ', xmin, '        to ', xmax
                       print,'       Wavelength:   ', wmin, '    to', wmax
                    endif else begin
                       if i eq 0 then begin
                          print,'Window        Label         Band    Xmin   '+$
                                'Xmax    Wmin      Wmax'
                       endif
                       text(i) = string(i,strpad(w(i).label,16,/aft),band,$
                                        xmin,xmax,wmin,wmax,format=$
                                '(i3,7x,a,2x,a4,3x,i4,3x,i4,3x,f7.2,3x,f7.2)')
                    endelse

                 endif
                 if keyword_set(wave) then begin
                    band = 'GIS'+string(fix(out(0,i)/2048)+1,form='(i1)')
                    out(0,i) = pix2wave(band,(out(0,i) mod 2048),/limit)
                    out(1,i) = pix2wave(band,(out(1,i) mod 2048),/limit)
                 endif
              endfor
              if keyword_set(short) then begin
                 if keyword_set(wsort) then begin
                    wmin = float(strmid(text,49,8))
                    ns = sort(wmin)
                    text = text(ns)
                 endif
                 print_str,text,hardcopy=hc,quiet=bequiet
              endif
           end

    else: begin
             print,'GT_WLIMITS: Instrument name not recognised.' 
             out = 0
          end
endcase

return,out

end
