;+
; Project     : SOHO - CDS     
;                   
; Name        : MONO_SPEC()
;               
; Purpose     : Plots a spectrum chosen graphically by user.
;               
; Explanation : A spectrum from the QLDS at a user-supplied location
;               is plotted.  The user graphically selects
;               the spectrum to be displayed - a sort of click and look
;               utility.
;               
; Use         : IDL> spec = mono_spec(qlds [,image=image, window=window, $
;                                            layout=layout, $
;                                            fixedscale=fixedscale])
;    
; Inputs      : qlds  - the standard data structure as given by READCDSFITS
;              
; Opt. Inputs : None
;               
; Outputs     : Function returns the last spectrum chosen
;               
; Opt. Outputs: None
;               
; Keywords    : WINDOW - if supplied it defines the spectral extraction window
;                        from which to plot the spectrum. Note this is not
;                        the "general" window ie NIS1/2 but rather the
;                        data extraction window number (use GT_WLIMITS to see
;                        those available). Defaults to window=0
;
;               IMAGE  - if supplied this can be any 2-d array of the same
;                        dimensions as an image that would be created from the
;                        QLDS supplied.  Eg it could be a velocity map, so
;                        that this routine could be used to inspect the raw
;                        spectrum at any place in the map.  If this parameter
;                        is not supplied then an image is created from the
;                        spectral information in "window".
;
;               LAYOUT - defines the layout for the spectral plots.
;                        Must be a 2-element array giving 
;                        [plots in X, plots in Y]  (cf !p.multi)
;
;               FIXEDSCALE - fix spectral plot maximum to this value.
;
;
; Calls       : ANYTIM2UTC(), GT_WLIMITS,
;
; Common      : None
;               
; Restrictions: None
;               
; Side effects: None
;               
; Category    : Data analysis, spectral
;               
; Prev. Hist. : based on POLY_SPEC
;
; Written     : C D Pike, RAL, 05-Apr-95
;               
; Modified    : Hardcoded data extraction for speed.  CDP, 10-Apr-96
;               Trap missing or bad data.   CDP, 27-Apr-96
;               Add extra info message.     CDP, 17-May-96
;               Convert to use of gt_windata.  SVHH, 31-Jul-96
;               Cut out missing keyword in plot_image and
;               assume it is set via SETFLAG.  CDP, 20-Nov-96
;               Cut out sigrange call in image display.  CDP, 4-Feb-97
;
; Version     : Version 7, 4-Feb-97
;-            

function mono_spec, qlds, window=window, image=image, layout=layout,$
                          fixedscale=fixedscale

;
;  check input
;
if n_params() eq 0 then begin
   print,'Use:  IDL> spec = mono_spec(qlds [,window=window, image=image])'
   return, -1
endif
if datatype(qlds,1) ne 'Structure' then begin
   print,'First parameter must be a QLDS'
   return, -1
endif

;
;  get image size
;
dim = gt_dimension(qlds)
nx = dim.ssolar_x
ny = dim.ssolar_y

;
;  check OK if image given
;
if keyword_set(image) then begin
   si = size(image)
   si = si(0:2)
   if not same_data(si,[2,nx,ny]) then begin
      print,'Image supplied has wrong dimensions for QLDS given.'
      return,-1
   endif
endif


dummy = load_wavecal(qlds.header.detector, $
                     anytim2utc(qlds.header.date_obs), $
                     cal_struct = qlds.wavecal)

;
;  get two fresh windows for the displays
;
window,5
pw2 = 5
window,6
pw1 = 6
wset,pw1
!p.multi=0


;
;  if window was supplied then use the wavelengths from it, otherwise assume
;  window=0
;
  
if n_elements(window) eq 0 then window = 0

w = gt_wlimits(qlds,/q,/wave)
nw = n_elements(w)/4
if window lt 0 or window gt nw then begin
   print,'Invalid window number. Limits are 0-'+strtrim(string(nw),2)
   return,-1
endif
lam1 = w(0,window)
lam2 = w(1,window)
print,'Using window #'+strtrim(string(window),2)+$
           '  Wavelengths: '+fmt_vect([lam1,lam2],delim=' --> ')

;
;  what to do
;
print,'Left mouse button on image to see spectrum.'
print,'Right button, or off image to end'
print,' '

;
;  extract the extraction window location in qlds.detdata
;
p1 = qlds.detdesc(window).ixstart(0)
p2 = qlds.detdesc(window).ixstop(0)
w = gt_wlimits(qlds,/q)
if w(2,window) lt 512 then band = 'N2' else band ='N1'
wavelength = pix2wave(band,indgen(w(1,window)-w(0,window)+1)+w(0,window))


;
;  if not supplied image, make it up
;

dta = gt_windata(qlds,WINDOW)

if not keyword_set(image) then begin
   image = total(dta,1)/(p2-p1+1)
   nbad = where(image le 0,count)
   if count gt 0 then image(nbad) = -1
endif 

;
;  display scales to be used
;
orig = [qlds.detdesc(window).origin(1),qlds.detdesc(window).origin(2)]
scale = [qlds.detdesc(window).spacing(1),qlds.detdesc(window).spacing(2)]

;
;  plot the derived image
;
plot_image,image,origin=orig, scale=scale,$
                 xtit='Solar X (arcsec)',ytit='Solar Y (arcsec)'
s = size(image)

;
;  help on use of cursor
;
print,' '
print,'Use the cursor to select the location'
print,'Select points with LEFT mouse button.'
print,'Exit with RIGHT button (or LEFT button off image)'
print,' '

;
;  note that ix,iy are the image array corrdinates and dx,dy are the
;  coordinates in data units
;
!err = 0
spec=-1

;
;  check plot layout
;
if not keyword_set(layout) then begin
   !p.multi=0 
endif else begin
   if n_elements(layout) eq 2 then begin
      !p.multi=[0,layout]
   endif else begin
      bell
      print,'Bad plot layout'
   endelse
endelse
circle_sym,/fill

;
;  collect the cursor points
;
while !err ne 4 do begin
   cursor,x1,y1,3,/data
   dx = x1
   dy = y1
   oplot,[dx,dx],[dy,dy],psym=8,syms=0.5
   if !err ne 4 then begin
;
;  translate data values back into array coordinates
;
      ix = fix((dx-orig(0))/scale(0))
      iy = fix((dy-orig(1))/scale(1))

;
;  check for out of image as alternative exit
;
      if (ix lt 0) or (ix gt nx) or (iy lt 0) or (iy gt ny) then return,spec


;
;  info 
;
      cx = strtrim(trim(dx,'(f6.1)'),2)
      cy = strtrim(trim(dy,'(f6.1)'),2)
      cxy = fmt_vect([cx,cy])
      print,'Image pixels -->',fmt_vect([ix,iy])
      print,'Solar (X,Y)  --> ',cxy 
      print,' '
      a = ix
      b = iy
      

;
;  collect spectrum
;
      spec = 0
      spec = dta(*,a,b)
      pisave = !p & xisave = !x & yisave = !y 
      wset,pw2
      if n_elements(ppsave) gt 0 then begin
         !p = ppsave & !x = xpsave & !y = ypsave
      endif
      if not keyword_set(fixedscale) then begin
         plot,wavelength,spec,psym=10,xr=[lam1,lam2],xstyle=1,$
              tit = 'Pix: '+fmt_vect([ix,iy])+$
                    '  (X,Y): '+cxy 
      endif else begin
         plot,wavelength,spec,psym=10,xr=[lam1,lam2],xstyle=1,$
              tit = 'Pix: '+fmt_vect([ix,iy])+$
                    '  (X,Y): '+cxy, yr=[0,fixedscale] 
      endelse
      ppsave = !p & xpsave = !x & ypsave = !y 
      !p = pisave & !x = xisave & !y = yisave
      wset,pw1
   endif
endwhile

;
; and return with the spectrum
;

return,spec

end
