	PRO NIS_PIX_ANALYSE, A, XIX, YIX, WIN_NO
;+
; Project     : SOHO - CDS
;
; Name        : NIS_PIX_ANALYSE
;
; Purpose     : Complementary routine to NIS_QUICKLOOK. Allows the spectrum 
;		from an image pixel chosen in NIS_QUICKLOOK to be viewed.
;		Also works as a stand-alone routine.
;
; Category    : Data analysis
;
; Explanation : On passing the data structure, A, and a co-ordinate (X,Y), 
;		the NIS spectrum for that pixel is extracted. A simple menu 
;		allows this spectrum to be viewed in detail.
;
;		The window number from which the original image was extracted 
;		is passed so that, on selecting a pixel in the image, you 
;		are first presented with a view of this portion of the 
;		spectrum.
;
; Syntax      : NIS_PIX_ANALYSE, A, XIX, YIX, WIN_NO
;
; Examples    : 
;
; Inputs      : A	= CDS data structure
;
;		XIX	= Exposure number
;
;		YIX	= Detector Y value
;
;		WIN_NO	= window number
;
; Opt. Inputs : None
;
; Outputs     : None
;
; Opt. Outputs: None
;
; Keywords    : None
;
; Calls       : GT_WINSIZE, GT_NUMWIN, GT_WLABEL, GT_SPECTRUM,
;		PIX2WAVE, GT_LAMBDA
;
; Common      : None.
;
; Restrictions: 
;
; Side effects: None.
;
; Prev. Hist. : None.
;
; History     : Version 1, Peter Young, Cambridge University, 5 September 1996.
;
; Contact     : Peter Young, Cambridge University.
;-

; It's useful to define the following vectors to help in the display of
; windows.
;
detx=a.detdesc.detx & dety=a.detdesc.dety+yix

wsize=gt_winsize(a) & wsize=wsize(0)

; Re-define win_no
;
win_no=win_no-1

; ---------------------------
; Check if there are windows in both NIS1 and NIS2. If there are only
; windows in NIS1, then nis_wind is set to 1, if NIS2 then 2.
;
nis_wind=0
if min(dety) ge 512 then nis_wind=1
if max(dety) lt 512 then nis_wind=2

; ---------------------------
; Define spec and wave arrays
;
spec=fltarr(1024,2) & wave=spec
if (nis_wind eq 0) or (nis_wind eq 1) then begin
  spec(*,0)=gt_spectrum(a,xix=xix,yix=yix,band=1,lamb=l1)
  wave(*,0)=l1
endif

if (nis_wind eq 0) or (nis_wind eq 2) then begin
  spec(*,1)=gt_spectrum(a,xix=xix,yix=yix,band=2,lamb=l2)
  wave(*,1)=l2
endif

if nis_wind eq 1 then begin
  spec(*,1)=fltarr(1024) & wave(*,1)=pix2wave('n2',indgen(1024))
endif

if nis_wind eq 2 then begin
  spec(*,0)=fltarr(1024) & wave(*,0)=pix2wave('n1',indgen(1024))
endif

; The latest version of gt_spectrum sets non-window spec values to -100,
; so will reset these to being 0.
;
ind=where(spec eq -100) & if ind(0) ne -1 then spec(ind)=0.
; ----------------------------

; ----------------------------
; When plotting just windows, use gt_lambda to get the wavelengths, this
; is consistent with the use of gt_spectrum above.
;
winsize=gt_winsize(a) & winsize=winsize(0)
;
xsc=[gt_lambda(a,detx(win_no),dety(win_no),xix) , $
	gt_lambda(a,detx(win_no)+winsize-1,dety(win_no),xix)]
; ----------------------------


; ----------------------------
; band denotes which band of the spectrum is being displayed
band=0 & ysc=[0,0]
if xsc(0) gt 400 then band=1
; ----------------------------

; Plot the requested window
;
plot,wave(*,band),spec(*,band),psym=10,xra=xsc,xsty=1


swtch=0
while (swtch eq 0) do begin
;
  print,'1. Change band.'
  print,'     current is: ',band+1
  print,'2. Choose a window.'
  print,'     current is: ',win_no+1,'.  ',gt_wlabel(a,win_no)
  print,'3. Change x-scale.'
  print,'     current is: ',xsc
  print,'4. Change y-scale.'
  print,'     current is: ',ysc
  print,'5. Print graph'
  print,'0. Exit'
;
  read,menu
;
  case menu of
;
    1: begin
	read,'Type new band (1/2): ',band
        band=band-1
	xsc=[wave(0,band),wave(1023,band)]
	ysc=[0,0]
    end
    2: begin
	swtch2=0
	for i=1,gt_numwin(a) do print,i,'.  ',gt_wlabel(a,i-1)
	print,'
	repeat begin
	  read,'Type in new window: ',win_no
 	  if (win_no le gt_numwin(a)) and (win_no ge 1) then swtch2=1
	endrep until swtch2 eq 1
	win_no=win_no-1
	xsc=[gt_lambda(a,detx(win_no),dety(win_no),xix) , $
	  gt_lambda(a,detx(win_no)+winsize-1,dety(win_no),xix)]
	ysc=[0,0]
    end
    3: read,'Type in new x-scale: ',xsc
    4: read,'Type in new y-scale: ',ysc
    5: ps_mine,'spec.ps'
    0: swtch=1

  endcase

  if xsc(0) gt 400 then band=1
  if xsc(0) lt 400 then band=0

  plot,wave(*,band),spec(*,band),xra=xsc,yra=ysc,psym=10,xsty=1

  if (menu eq 5) then psclose

endwhile

end
