	PRO NIS_QUICKLOOK, A, PIX3=PIX3, GET_PIX=GET_PIX, WIND=WIND, $
		IMAGE=IMAGE, CLEAN=CLEAN, ADD_CONT=ADD_CONT, $
		REM_BACK=REM_BACK, FILL=FILL, NISAT=NISAT, $
                EXPTIME=EXPTIME

;+
; Project     : SOHO - CDS
;
; Name        : NIS_QUICKLOOK
;
; Purpose     : Allows a quick-look at CDS data
;
; Category    : Data analysis
;
; Explanation : Upon running NIS_QUICKLOOK, the set of wavelength windows in
;		the data will be displayed. Choosing one brings up the 
;		spectrum of this window averaged over the whole raster area. 
;		Clicking on one of the wavelength pixels in this window 
;		brings up an image of the observed region at this particular 
;		wavelength.
;
;		If the option GET_PIX is given then one of the spatial pixels 
;		can be chosen and the spectrum at this location can be 
;		analysed - see also the routine NIS_PIX_ANALYSE.
;
;		Will still work with wavelength windows only 1 pixel wide - 
;		the routine goes directly to displaying the image.
;
;		Any drop-outs in the data are assumed to have the value 
;		-100, and will be re-set to the value -5 to ensure the 
;		image is reasonably displayed.
;
;		There are many useful keywords - see below for details.
;
; Syntax      : NIS_QUICKLOOK, A, [, PIX3, GET_PIX, CLEAN, IMAGE=IMAGE
;					WIND=WIND]
;
; Examples    : NIS_QUICKLOOK, A
;
;		   - basic operation
;
;		NIS_QUICKLOOK, A, /CLEAN, /REM_BACK, WIND=7
;
;		   - clean the image of cosmic rays, remove background from
;                    image, and use idl window 7
;
;		NIS_QUICKLOOK, A, IMAGE=IM, /GET
;
;		   - save the displayed image to an array IM, and allow the
;                    spectrum at a chosen image pixel to be viewed
;
; Inputs      : A	= CDS quick-look data structure
;
; Opt. Inputs ;	WIND	= The window that the image/spectra are sent to is by
;			  default window 5. This keyword allows the
;			  specification of a different window.
;
;		ADD_CONT = A contour plot of the same size as the displayed 
;			   image that will be overplotted on the displayed 
;			   image
;
;		EXPTIME = after vds_calib has been run on a data-set, the 
;			  y-scale can be rather small, so this routine 
;			  multiplies the displayed spectra by EXPTIME
;
; Outputs     : None
;
; Opt. Outputs: IMAGE	= The displayed image is sent to the array IMAGE, for
;			  further manipulation in IDL.
;
; Keywords    : CLEAN 	= By default, the raw image is displayed. If CLEAN is
;			  set, then a cosmic ray removal routine is used.
;
;		PIX3	= construct image from the sum of three wavelength
;			  pixels, rather than just one
;
;		FILL	= If set, all missing data pixels in the selected 
;			  data window will be filled using the FILL_MISSING 
;			  routine.
;
;		REM_BACK = If this keyword is set, then after the wavelength
;			   is selected with the mouse, two additional
;			   wavelengths can be chosen which are taken to
;			   correspond to continuum on either side of the
;     			   chosen line. These "continuum images" are averaged
;			   and subtracted from the "line images".
;
;		GET_PIX = once the image is displayed, GET_PIX allows an
;			  image pixel to be chosen. The routine
;			  NIS_PIX_ANALYSE is then called, allowing the
;			  spectrum at that particular pixel to be viewed.
;
;		NISAT	= tells NIS_QUICKLOOK that the data contains complete 
;			  NIS spectra, and so when the averaged spectra are 
;			  displayed a window can be selected with the mouse
;
; Calls       : GT_WINSIZE, GT_NUMWIN, GT_WLABEL, GT_WINDATA, GT_SPECTRUM,
;			PLOT_IMAGE, NIS_PIX_ANALYSE, NEW_SPIKE,
;			FILL_MISSING
;
; Common      : None.
;
; Restrictions: Only works on 3-D arrays, i.e., (lambda,x,y). Won't work with 
;		repeated exposures at the same location.
;
;		When using GET_PIX keyword, be careful using data containing 
;		entire NIS wavebands. The chosen spectrum will not come from 
;		a consistent spatial pixel (because of the slant in the NIS 
;		spectra).
;
; Side effects: None.
;
; Prev. Hist. : None.
;
; History     : Ver.1 5-sep-96 
;		Ver.2 18-jan-97  added REM_BACK keyword.
;		Ver.3 15-oct-97  tidied up the way routine handles missing 
;                                data.
;		Ver.4 18-nov-97  corrected error in the display of the 
;				 averaged spectrum
;		Ver.5 6-jan-98   added FILL keyword
;		Ver.6 20-may-98  added EXPTIME and NISAT keyword
;		Ver.7 2-sep-98	 now calls new_spike to remove cosmic rays
;
; Contact     : Peter Young, Cambridge University (pry10@damtp.cam.ac.uk)
;-
;

IF N_PARAMS() LT 1 THEN BEGIN
  PRINT,'
  PRINT,'Use:  IDL> nis_quicklook, qlds
  PRINT,'
  PRINT,' Keywords:
  PRINT,'   CLEAN - remove cosmic rays from image
  PRINT,'   FILL  - fill missing pixels
  PRINT,'   PIX3  - sum 3 wavelength pixels in making image
  PRINT,'   REM_BACK  - allow selection of two background pixels
  PRINT,'   GET_PIX  - allow viewing of spectrum at a spatial location
  PRINT,'   NISAT - easy wavelength selection for full NIS spectra
  PRINT,'
  PRINT,' Opt. inputs:
  PRINT,'   EXPTIME - multiply image intensities by EXPTIME
  PRINT,'   WIND  - send image to specified IDL window
  PRINT,'   ADD_CONT - add contours of input image to selected image
  PRINT,'
  RETURN
ENDIF

wsize=gt_winsize(a) & wsize=wsize(0)

back1_pix=-1 & back2_pix=-1

if n_elements(wind) eq 0 then wind=5

IF (keyword_set(rem_back) or keyword_set(pix3)) THEN send=0

; -----------------------------------<
; Choose which wavelength window needs to be looked at
;
PRINT,''
PRINT,'Choose a wavelength window to analyse:'
PRINT,''
;
for i=1,gt_numwin(a) do PRINT,i,'.  ',gt_wlabel(a,i-1)
;
read,choice
choice=fix(choice)
; -----------------------------------<

title=gt_wlabel(a,choice-1)
missing=a.detdesc(choice-1).missing

window,wind
!p.multi=0

; -----------------------------------------------------[
; Extract data array from CDS data structure and select pixel(s) from which
; image is to be formed
;
data=gt_windata(a,choice-1)
ind=WHERE(data NE missing)
IF KEYWORD_SET(exptime) THEN data(ind)=data(ind)*a.header.exptime
IF KEYWORD_SET(fill) THEN fill_missing,data,missing,2

;
; Had a problem in that some sequences just used one pixel
;
IF wsize NE 1 THEN BEGIN
  ;
  ; For the chosen window, plot out averaged spectrum. Will be able to choose
  ; a pixel in the spectrum s.t. the image will correspond to that pixel.
  ;
  av_data=average(data,2,missing=missing)
  av_data=average(av_data,2,missing=missing)
  ;
  ; Following help received from Stein Vidar Haugan, I'm now using the
  ; following approximation which is a quick way of getting the wavelengths:
  ;
  wave=a.detdesc(choice-1).origin(0) + $
    a.detdesc(choice-1).spacing(0)*indgen(wsize)
  ;
  ; (a.detdesc gives the detector position of the lower left corner of the
  ; wavelength window.)
  ;
  ; Have put the gt 100 bit in to make sure that data that hasn't been 
  ; de-biased looks OK
  ;
  IF MIN(av_data) GT 100 THEN BEGIN
    PLOT,wave,av_data,PSYM=10,XSTYLE=1,YSTYLE=3
  ENDIF ELSE BEGIN
    PLOT,wave,av_data,PSYM=10,XSTYLE=1
  ENDELSE
  ;
  IF KEYWORD_SET(nisat) THEN BEGIN
    PRINT,''
    PRINT,'Choose a wavelength range with the mouse...'
   ;
    CURSOR,x,y,/DOWN
    minimum=MIN(ABS(wave-x),limit1)
    OPLOT,[wave(limit1),wave(limit1)],[-1e6,1e6],line=2
    WAIT,.5
   ;
    CURSOR,x,y,/DOWN
    minimum=MIN(ABS(wave-x),limit2)
    OPLOT,[wave(limit2),wave(limit2)],[-1e6,1e6],line=2
    WAIT,.5
   ;
    xrange=[wave(limit1),wave(limit2)]
    IF MIN(av_data) GT 100 THEN BEGIN
      PLOT,wave,av_data,PSYM=10,XSTYLE=1,YSTYLE=3,xra=xrange
    ENDIF ELSE BEGIN
      PLOT,wave,av_data,PSYM=10,XSTYLE=1,xra=xrange
    ENDELSE
   ;
  ENDIF
  ;
  PRINT,''
  PRINT,'Choose pixel for which to display image...'
  ;
  CURSOR,x,y,/DOWN
  minimum=MIN(ABS(wave-x),centre_pix)
  PRINT,'
  PRINT,'Chosen wavelength is',string(format='(f7.2)',wave(centre_pix))
  ;
  oplot,[wave(centre_pix),wave(centre_pix)],[-1e4,1e4]
  ;
  wait,.25   ; so you can see which pixel you've chosen
  ;
  ; If rem_back set, then choose two background pixels
  ;
  if N_ELEMENTS(REM_BACK) ne 0 then begin
    PRINT,''
    PRINT,'Choose two background pixels:
    CURSOR,x,y,/DOWN
    minimum=MIN(ABS(wave-x),back1_pix)
    PRINT,STRING(FORMAT='(f7.2)',wave(back1_pix))
    ;
    OPLOT,[wave(back1_pix),wave(back1_pix)],[-1e4,1e4],line=2
    ;
    CURSOR,x,y,/DOWN
    minimum=MIN( ABS(wave-x),back2_pix )
    PRINT,STRING(FORMAT='(f7.2)',wave(back2_pix))
    ;
    OPLOT,[wave(back2_pix),wave(back2_pix)],[-1e4,1e4],line=2
    WAIT,.25
    ;
  endif

ENDIF
;
IF wsize EQ 1 THEN BEGIN
  centre_pix=0
  sub_data=data
ENDIF
; -----------------------------------------------------[

; -------------------+
; Problem if /pix3 has been given, but a pixel at the ends of the window has
; been chosen. Will set centre_pix to be the neighbouring pixel instead.
;
if (wsize gt 2) and (n_elements(pix3) ne 0) then begin
  if centre_pix eq 0 then centre_pix=1
  if centre_pix eq wsize-1 then centre_pix=wsize-2
endif
; -------------------+

; -------------------------------------------<>
; Clean the images with new_spike. If the window size is 10 or less, then the 
; whole array is cleaned, otherwise a sub-set of the data is taken and 
; cleaned based on the values of back1_pix, back2_pix and centre_pix.
;
IF KEYWORD_SET(clean) THEN BEGIN
  IF wsize LE 10 THEN new_spike,data,/neigh,/fill
  IF wsize GT 10 THEN BEGIN
    ;
    IF back1_pix EQ -1 THEN back1_pix=centre_pix-2
    IF back2_pix EQ -1 THEN back2_pix=centre_pix+2
    ;
    i0=MAX([0,MIN([back1_pix-2,back2_pix-2,centre_pix-2])])
    i1=MIN([wsize-1,MAX([back1_pix+2,back2_pix+2,centre_pix+2])])
    data=data(i0:i1,*,*)
    new_spike,data,/neigh,/fill
    ;
    ; need to re-define center_pix, back1_pix and back2_pix
    ;
    centre_pix=centre_pix-i0
    back1_pix=back1_pix-i0
    back2_pix=back2_pix-i0
    ;
  ENDIF
ENDIF
; -------------------------------------------<>

; ------------------X
; Get image
; ---------
IF N_ELEMENTS(pix3) EQ 0 THEN BEGIN
  image=REFORM(data(centre_pix,*,*))
ENDIF ELSE BEGIN
  image=data(centre_pix-1:centre_pix+1,*,*)
  image=average(image,1,missing=missing)
ENDELSE
; ------------------X

;------------------------[
; Get background
; --------------
IF N_ELEMENTS(rem_back) NE 0 THEN BEGIN
  back=[[[REFORM(data(back1_pix,*,*))]], $
        [[REFORM(data(back2_pix,*,*))]]]
  back=average(back,3,missing=missing)
ENDIF
;------------------------[


;------------------------------<
; Image to be plotted. When REM_BACK is set, a missing pixel in *either* 
; image *or* back will trigger the setting of a missing pixel in image2save
; or image2plot. image2save is the image that is output, whereas image2plot
; is that which is plotted. The latter has missing pixels set to -1, whereas
; the former has missing pixels set to missing.
;
IF KEYWORD_SET(rem_back) NE 0 THEN BEGIN
  image2plot=image-back
  image2save=image-back
  swtch=image-image             ; set dimensions of switch
  ind=WHERE(image EQ missing)
  IF ind(0) NE -1 THEN swtch(ind)=1.
  ind=WHERE(back EQ missing)
  IF ind(0) NE -1 THEN swtch(ind)=1.
  ind=WHERE(swtch EQ 1)
  IF ind(0) NE -1 THEN BEGIN
    image2plot(ind)=-1.
    image2save(ind)=missing
  ENDIF
ENDIF ELSE BEGIN
  image2save=image
  ind=WHERE(image EQ missing)
  IF ind(0) NE -1 THEN image(ind)=-1.
  image2plot=image
ENDELSE
;------------------------------<

;-------------------o
; Use the plot_image routine to plot the image
;
orig=[a.detdesc(choice-1).origin(1),a.detdesc(choice-1).origin(2)]
scale=[a.detdesc(choice-1).spacing(1),a.detdesc(choice-1).spacing(2)]
;
plot_image,image2plot,scale=scale,origin=orig,title=title,/_extra, $
	xticklen=-0.01,yticklen=-0.02,xminor=0
;-------------------o

;---------------x
IF N_ELEMENTS(add_cont) NE 0 THEN BEGIN
  CONTV,add_cont,color=0
ENDIF
;---------------x

; ---------------------------+
; Use the following to get the position of a given pixel in the image. The
; keyword /get_pix needs to be set
; ----------------
IF KEYWORD_SET(get_pix) THEN BEGIN
  CURSOR,x1,y1,3,/data
  xix=FIX( (x1-orig(0)) /scale(0) + .5)
  yix=FIX( (y1-orig(1)) /scale(1) + .5)
  PRINT,xix,yix
  nis_pix_analyse,a,xix,yix,choice
ENDIF
; ---------------------------+

; Return the background-removed image
;
image=image2save

END
