;+
; Project     : SOHO - CDS     
;                   
; Name        : PLOT_DELTAT
;               
; Purpose     : Plot the time interval between exposures in a raster.
;               
; Explanation : Extracts the time interval from a CDS data structure (or
;               FITS file) and plot the interval between the start of 
;               exposures in a raster.
;               
; Use         : IDL> plot_deltat, a [, exps, expe, maxdt=maxdt, mindt=mindt]
;    
; Inputs      : a    - a valid CDS data structure or FITS file name.
;               
; Opt. Inputs : exps - the start exposure to plot
;               expe - the last exposure to plot
;               
; Outputs     : Just plots the results.
;               
; Opt. Outputs: None
;               
; Keywords    : MINDT  -  minimum plot value for time difference
;               MAXDT  -  maximum plot value for time difference
;
; Calls       : None
;
; Common      : None
;               
; Restrictions: For reading calibration FITS files. Searches user directory
;               and then CDS_CAL_FITS.
;               
; Side effects: None
;               
; Category    : Util
;               
; Prev. Hist. : None
;
; Written     : C D Pike, RAL, 24-may-94
;               
; Modified    : To use lower level routines, CDP, 25-May-94
;               To include min max plot range, CDP, 8-Jun-94
;               Include cal/ops test and fix tag name bug.  CDP, 15-Jun-94
;               New tag names in non-calibration QLDS, SVHH, 4-Oct-95
;
; Version     : Version 5, 4-Oct-1995
;-            

pro plot_deltat, input, exps, expe, maxdt=maxdt, mindt=mindt


;
;  check form of input
;
if datatype(input,1) eq 'String' then begin
;
;  allow user to have own version of FITS files
;
   tfile = ''
   if  file_exist(input) then begin
      tfile = input
   endif else begin
      if file_exist(concat_dir('$CDS_CAL_FITS',input)) then begin
         tfile = concat_dir('$CDS_CAL_FITS',input)
      endif
   endelse
;
;  FITS file doesn't exist in user or std directory
;
   if tfile eq '' then begin
      print,'PLOT_DELTAT:  Input file does not exist.'
      return
   endif

   fxbopen,unit,tfile,1,header
   if fxpar(header,'TELESCOP') eq 'CALIBRATION' then begin
      t = cal_auxread(unit,header,'DEL_TIME',status=status)
   endif else begin
      t = auxread(unit,header,'DEL_TIME',status=status)
   endelse
   if n_elements(status) gt 0 then begin
      if status ne '' then begin
         print,'Error reading times from FITS file.'
         fxbclose,unit
         return
      endif
   endif
   times = t.d
   mtit = fxpar(header,'FILENAME')
   fxbclose, unit
endif else begin
   qlmgr,input,valid
   if not valid then begin
      print,'Input parameter is not a valid CDS data structure or file name.'
      return
   ENDIF ELSE BEGIN
      t = tag_names(input)
      IF (WHERE(t EQ 'DEL_TIMEDATA'))(0) NE -1 THEN BEGIN
         times = input.del_timedata
      END ELSE BEGIN
         times = input.deltdata
      END
      mtit = input.header.filename
   endelse
endelse

;
;  check if bounds are set
;
if n_elements(exps) eq 0 then exps = 0
if n_elements(expe) eq 0 then expe = n_elements(times) - 1

;
;  check enough data to bother
;
if n_elements(times) le 3 then begin
   print,'Not enough exposures to plot, offset start times were...'
   print,fmt_vect(times/1000.,format='(3f6.1)'),' seconds.'
   return
endif


;
;  extract delta times, convert to seconds
;
t = times(exps:expe)/1000.
tt = t - shift(t,1)
tt = tt(1:*)

;
;  plot with filled circle
;
if n_elements(mindt) eq 0 then mindt = min(tt)
if n_elements(maxdt) eq 0 then maxdt = max(tt)


circle_sym,/fill
x = indgen(n_elements(tt)) + exps
plot,x,tt,/ynoz,psym=-2,xtit='Exposure',ytit='Time between exposures (secs)',$
     title=mtit,charsize=1.3,yrange=[mindt,maxdt]

end
