;+
; Project     : SOHO - CDS     
;                   
; Name        : PLOT_RASTER
;               
; Purpose     : Plot the scan positions in a raster.
;               
; Explanation : Plots the mirror postion array against the slit position
;               array to demonstrate the raster performed.
;               
; Use         : IDL> plot_raster, a 
;    
; Inputs      : a    - a valid CDS data structure.
;               
; Opt. Inputs : None
;               
; Outputs     : Just plots the results.
;               
; Opt. Outputs: None
;               
; Keywords    : None
;
; Calls       : GT_*
;               EXT_RANGE()
;
; Common      : None
;               
; Restrictions: None.
;               
; Side effects: None
;               
; Category    : Util
;               
; Prev. Hist. : None
;
; Written     : C D Pike, RAL, 24-may-94
;               
; Modified    : Use lower level routines to save time.  CDP, 25-may-94
;               Insert cal/ops test, fix tag name bug.  CDP, 15-Jun-94
;               To use ext_range. CDP, 23-Jun-94
;               New tag names in non-calibration QLDS, SVHH, 4-Oct-95
;               Update plot.  CDP, 22-Nov-96
;               Trap bad mirror/slit values.  CDP, 3-Dec-96
;
; Version     : Version 7, 03-Dec-96
;-            

pro plot_raster, input, solar=solar

;
;  validate input
;
qlmgr,input,valid
if not valid then begin
   print,'Input parameter is not a valid CDS data structure.'
   return
endif 

if keyword_set(solar) then begin
   mirr = input.ins_xdata
   slit = input.ins_ydata
endif else begin
   mirr = gt_mirrpos(input)
   slit = gt_slitpos(input)
endelse

;
;  check enough data to bother
;
if n_elements(mirr) le 2 then begin
   print,'Not enough exposures to plot.'
   print,'Mirror positions were...',fmt_vect(mirr)
   print,'Slit   positions were...',fmt_vect(slit)
   return
endif

;
;  get start times
;
t = gt_start(input,/exp)

;
;  sort time
;
tut = utc2tai(str2utc(t))
n = sort(tut)

slit = slit(n)
mirr = mirr(n)

;
;  plot with filled circle
;
circle_sym,/fill


;
;  trap bad uns
;
nok = where(mirr ne 255)
nbad = where(mirr eq 255,cbad)

mirr = mirr(nok)
slit = slit(nok)

;
;  plot data
;
if keyword_set(solar) then del = 2.0 else del = 1.0
if max(slit)-min(slit) le del then mirr_only = 1 else mirr_only = 0

if mirr_only then begin
   mirr = mirr(0:9)
   slit = slit(0:9)
endif

;
;  enlarge ranges to keep 'em off axes
;
mmin = (ext_range(mirr,15))(0)
mmax = (ext_range(mirr,15))(1)
smin = (ext_range(slit,15))(0)
smax = (ext_range(slit,15))(1)

if keyword_set(solar) then begin
   xtit = 'Solar location E/W (arcsec)'
   ytit = 'Solar location S/N (arcsec)'
endif else begin
   xtit = 'Mirror position (West --> East).'
   ytit = 'Slit Position (South --> North)'
endelse

if mirr_only then psym = 8 else psym=-8
plot,mirr,slit,xrange=[mmin,mmax],yrange=[smin,smax],$
      psym=psym,xtit=xtit,$
                ytit=ytit,$
     title=input.header.filename,charsize=1.5


if mirr_only then begin
   for i=0,n_elements(mirr)-2 do begin
      label_symbol,mirr(i),slit(i),8,trim(i),chars=1.3
   endfor
   label_symbol,mirr(i),slit(i),8,'etc.!!',chars=2.0
endif else begin
   for i=0,n_elements(mirr)-1 do begin
      label_symbol,mirr(i),slit(i),8,trim(i),chars=1.3
   endfor
endelse

;
;  inform of bad ones
;
if cbad gt 0 then begin
   if cbad eq 1 then plural = '' else plural = 's'
   xyouts,(mmax-mmin)/10.+mmin,(smax-smin)/10.+smin,$
      '** Warning: raster contains '+trim(cbad)+' bad mirror/slit location'+$
      plural+'.',chars=1.3
endif
          

end
