;+
; Project     : SOHO - CDS     
;                   
; Name        : PLOT_VDS_BIAS
;               
; Purpose     : Plot VDS CCD bias levels 
;               
; Explanation : The VDS CCD is readout by more than one set of electronics.
;               Each set has its own bias level and a knowledge of this
;               value is needed for complete analysis of the data.  Normally,
;               NIS rasters are designed to include bias window data.
;               The values are stored in the QLDS read by READCDSFITS under
;               the tag structure .BACKGROUND.  
;               
; Use         : plot_vds_bias, qlds [,quad=quad, time=time, avg=avg]
;    
; Inputs      : qlds - a valid QL data structure read in by readcdsfits
;               
; Opt. Inputs : None
;               
; Outputs     : None
;               
; Opt. Outputs: None
;               
; Keywords    : QUAD - an array to specify which quadrant backgrounds are to
;                      be plotted.  Default is [1,2,3,4]
;
;               TIME - if specified the values are plotted against time of the
;                      exposure rather than exposure number.
;
;               AVG  - returns average background per quadrant.
;
; Calls       : CDS_UTPLOT
;
; Common      : None
;               
; Restrictions: None
;               
; Side effects: None
;               
; Category    : Data analysis
;               
; Prev. Hist. : None
;
; Written     : C D Pike, RAL, 18-Mar-96
;               
; Modified    : Enlarge axis labelling and add AVG keyword.    CDP, 12-Apr-96
;               Add filename to plots.   CDP, 5-Aug-96
;               Changed name to ..._bias.  CDP, 14-Aug-96
;               Change utplot name.  CDP, 2-Oct-96
;
; Version     : Version 5, 2-Oct-96
;-            

pro plot_vds_bias, qlds, quad=quad, time=time, avg=quad_avg

;
;  check input parameters
;
if n_params() eq 0  then begin
   print,'Use: IDL> plot_vds_bias, qlds'
   return
endif

if datatype(qlds,1) ne 'Structure' then begin
   print,'Use: IDL> plot_vds_bias, qlds'
   return
endif

if qlds.header.detector eq 'GIS' then begin
   print,'GIS study!!'
   return
endif

if datatype(qlds.background,1) ne 'Structure' then begin
   print,'No NIS background data available.'
   return
endif

if n_elements(qlds.background.average) lt 12 then begin
   print, 'Not enough data to plot, average values were...'
   print,qlds.background.average
   return
endif
 

;
;  save  plot setup
;
pm = !p
circle_sym,/fill

;
;  were specific quadrants requested?
;
if not keyword_set(quad) then begin
   uquad = [0,1,2,3]
endif else begin
   uquad = quad-1
endelse
if max(uquad) gt 3 or min(uquad) lt 0 then begin
   print,'Quadrants are from 1:4'
   return
endif

;
;  set up appropriate plot
;
!p.multi = [0,1,n_elements(uquad)]
if !d.name eq 'PS' then bcol=0 else bcol=150
quad_avg = fltarr(n_elements(uquad))

;
;  set up x-axis
;
n = indgen(n_elements(qlds.background.average)/4)
if keyword_set(time) then begin
   t = gt_start(qlds,/exp)
   t = anytim2utc(t)
endif

;
;  get file name
;
break_file,qlds.header.filename,disk,dir,ffn,ext

;
;  plot the values + 1 sigma error bars
;
for i=0,n_elements(uquad)-1 do begin
   ng = where(qlds.background.average(*,uquad(i)) gt 0, count)
   if count gt 0 then begin
      avg = average(qlds.background.average(ng,uquad(i)))
      quad_avg(i) = avg
      pmin = min(qlds.background.average(ng,uquad(i)))
      pmax = max(qlds.background.average(ng,uquad(i)))
      pr = ext_range([pmin,pmax],500)
      tit = ffn+' NIS background values (quadrant '+trim(uquad(i)+1)+')'
      if not keyword_set(time) then begin
         plot_err,n(ng),qlds.background.average(ng,uquad(i)),$
               yerr=qlds.background.stdev(ng,uquad(i)),$
                         psym=8,bcol=bcol,yr=pr, tit=tit,chars=2.0,$
                         xtit='Exposure number',xr=[-1,max(n)+1],xstyle=1
         oplot,[n(0),last_item(n)], [avg,avg],line=2
      endif else begin
         cds_utplot,t(ng),qlds.background.average(ng,uquad(i)),$
                         psym=8,yr=pr,tit=tit, chars=2.0
         cds_outplot,[t(0),last_item(t)], [avg,avg],line=2
      endelse
   endif
endfor

;
;  reset plot setup
;
!p = pm

end
