;+
; Project     : SOHO - CDS     
;                   
; Name        : PLOT_WINDOWS
;               
; Purpose     : Plot the data extraction windows in a raster.
;               
; Explanation : Plots the data extraction windows used in a raster for
;               illustrative purposes.  Can work from a CDS data structure
;               or directly from a FITS file. The window plotted in is sized
;               so that if the image data (VDS) are plotted in a window with
;               xs=1024, the scale of the two plots will correspond approx.
;               
; Use         : IDL> plot_window, file_str 
;    
; Inputs      : file_str  - a valid CDS data structure or FITS file name.
;               
; Opt. Inputs : None
;               
; Outputs     : Just plots the results.
;               
; Opt. Outputs: None
;               
; Keywords    : TITLE - string to use as title to plot.
;
; Calls       : GT_WLIMITS
;
; Common      : None
;               
; Restrictions: For calibration FITS files. Looks in user directory and
;               then CDS_CAL_FITS FOR FITS FILES.
;               
; Side effects: None
;               
; Category    : Util
;               
; Prev. Hist. : None
;
; Written     : C D Pike, RAL, 27-may-94
;               
; Modified    : 
;
; Version     : Version 1, 27-May-94
;-            

pro plot_windows, input, title=title

;
;  retrieve the window limits
;
w = gt_wlimits(input)

;
;  error return
;
if n_elements(w) le 1 then return

;
;  get a title
;
if not keyword_set(title) then begin
   if datatype(input,1) eq 'Structure' then begin
      title = input.header.filename
   endif else begin
      title = input
   endelse
endif

;
;  act according to size of returned array (ie detector)
;
ins  = (size(w))(1)
numw = (size(w))(2)
nwidth = w(1,0) - w(0,0) + 1
blank = strarr(30)  & blank(*) = ' '

xtit = 'Pixels'

case ins of
   2:  begin
          if !d.name eq 'PS' then begin
             ps_long
             device,/landscape
          endif else begin
             window,10,xs=1100,ys=500,title='CDS GIS Raster windows in '+title
          endelse
          !p.multi=[0,2,2]
          for i=0,3 do begin
             xst = i*2048
             xe = xst+2047
             plot,[xst,xe],[0,1],/nodata,xstyle=5,ystyle=5,$
                                 title='GIS detector '+string(i+1,form='(i1)') 
             axis,xaxis=0,xstyle=1,xtit=xtit,chars=1.4
             oplot,[xe,xe,xst,xst],[0,1,1,0]
             if (size(w))(0) eq 1 then begin
                bst = xst > w(0)
                be = xe < w(1)
                bw = be - bst + 1
                if bw gt 1 then begin
                   fill_box,bst+bw/2,1.0,bw,6,/border
                   oplot,[bst,bst],[0,1]
                endif
             endif else begin
                for j=0,numw-1 do begin
                   if w(0,j) ge xst and w(0,j) le xe then begin
                      fill_box,w(0,j)+nwidth/2,1.0,nwidth,6,/border
                   endif
                endfor
             endelse
          endfor
       end
   4:  begin
          if !d.name eq 'PS' then begin
             ps_long
             device,/landscape
          endif else begin
             window,10,xs=1100,ys=350,title='CDS VDS Raster windows'
          endelse
          plot,[0,1023],[-1,1],/nodata,xstyle=5,ystyle=5,title=title 
          axis,xaxis=0,xstyle=1,xtit=xtit,chars=1.4
          oplot,[1023,1023,0,0],[-1,1,1,-1]
          oplot,[0,1023],[0,0],thick=2,line=2
          for i=0,numw-1 do begin
             if w(2,i) gt 511 then ym=-1 else ym = 0
             fill_box,w(0,i)+nwidth/2,ym+1.0,nwidth,6,/border,ymin=ym
          endfor
       end   
 else:
endcase

;
;  return plot set up to standard
;
!p.multi = 0

end
