;+
; Project     : SOHO - CDS     
;                   
; Name        : POLY_SPEC()
;               
; Purpose     : Averages spectra chosen graphically by user.
;               
; Explanation : An image is created from the QLDS at a user-supplied wavelength
;               (or range of wavelengths).  The user graphically selects
;               the spectra to be averaged and the average is returned as the
;               function value.
;               
; Use         : IDL> spec = poly_spec(qlds,wave1 [,wave2, window=window])
;    
; Inputs      : qlds  - the standard data structure as given by readcdsfits
;               wave1 - the wavelength at which the displayed image is
;                       extracted.  
;              
; Opt. Inputs : wave2 - if supplied in addition to wave1, then the image
;                       displayed is an integrated one from wave1-->wave2
;               
; Outputs     : function returns the averaged, deselected spectra
;               
; Opt. Outputs: None
;               
; Keywords    : WINDOW - if supplied it overrides any wavelength information
;                        given and the image is one integrated over the
;                        complete window.  If the window is a wide one (in
;                        pixels terms), this can take a while.
;
;               TEST - for use in until sensible data become available.  This
;                      routine still requires a valid QLDS as input but a
;                      dummy image is substituted for display and cursor
;                      interaction.
;
;               LAMBDA - returns the averaged wavelength array.
;
;               FULL    - if present the full spectral band is returned,
;                        otherwise just the requested window.
;
;               IMAGE  - returns the image used in the display.
;
; Calls       : ANYTIM2UTC(), GT_MIMAGE, GT_BIMAGE, GT_SCANP, GT_WLIMITS,
;               LOAD_WAVECAL()
;
; Common      : None
;               
; Restrictions: None
;               
; Side effects: None
;               
; Category    : Data analysis, spectral
;               
; Prev. Hist. : None
;
; Written     : C D Pike, RAL, 07-Dec-95
;               
; Modified    : Set -9999 as an illegal solar position and added INFO keyword.
;                                                             CDP, 08-Jan-96
;               Added lambda keyword.  CDP, 18-Jan-96
;               Version 4, SVHH, 14 March 1996   Using LOAD_WAVECAL.
;               Called gt_spectrum instead of gt_scanp for single pixel and
;               return image used in keyword variable.    CDP, 26-Mar-96
;           
;               Fixed keyword bug, handled missing data.  CDP, 27-Apr-96
;
; Version     : Version 6, 27-Apr-96
;-            

function poly_spec,qlds,lam1,lam2,window=window,test=test,info=info,$
                   lambda=wavelength, image=image, full=full


dummy = load_wavecal(qlds.header.detector, $
                     anytim2utc(qlds.header.date_obs), $
                     cal_struct = qlds.wavecal)

;
;  if window was supplied then use the wavelengths from it
;
  
  
IF N_ELEMENTS(WINDOW) GT 0  THEN BEGIN
   w = gt_wlimits(qlds,/q,/wave)
   nw = n_elements(w)/4
   if window lt 0 or window gt nw then begin
      print,'Invalid window number. Limits are 0-'+strtrim(string(nw),2)
      return,-1
   endif
   lam1 = w(0,window)
   lam2 = w(1,window)
   print,'Using window #'+strtrim(string(window),2)+$
              '  Wavelengths: '+fmt_vect([lam1,lam2],delim=' --> ')
endif

;
;  were two wavelengths supplied?
;
if n_elements(lam2) eq 0 then begin
   if n_elements(lam1) eq 0 then begin
      print,'Use: IDL> spec = poly_spec(qlds,wave1 [wave2,window=window])'
      return,-1
   endif else begin
      print,'Extracting the image....'
      image = gt_mimage(qlds,lam1,xs=xs,ys=ys)
   endelse
endif else begin
   print,'Extracting the image, please wait....'
   image = gt_bimage(qlds,lam1,lam2,xs=xs,ys=ys)
endelse

;
;  display scales to be used
;
orig = [xs(0,0),ys(0,0)]
xscl = xs(1,0) - xs(0,0)
yscl = ys(0,1) - ys(0,0)
scale = [xscl,yscl]

;
;  handle missing data
;
nbad = where(image le 0,count)
if count gt 0 then image(nbad) = -1

;
;  plot the derived image
;
if keyword_set(test) then image = dist(100,100)
plot_image,sigrange(image,miss=-1),origin=orig, scale=scale,missing = -1, $
                 xtit='Solar X (arcsec)',ytit='Solar Y (arcsec)'
s = size(image)

;
;  help on use of cursor
;
print,' '
print,'Use the cursor to define a point, line or polygon'
print,'Select points with LEFT mouse button.'
print,'Exit and close polygon with RIGHT button'
print,' '

;
;  initialise variables
;  note that ix,iy are the image array corrdinates and dx,dy are the
;  coordinates in data units
;
!err = 0
cnt = 0
ix = intarr(100)
iy = intarr(100)
dx = intarr(100)-9999
dy = intarr(100)-9999
circle_sym,/fill

;
;  collect the cursor points
;
while !err ne 4 do begin
   cursor,x1,y1,3,/data
   dx(cnt) = x1
   dy(cnt) = y1
   if cnt gt 0  and !err ne 4 then begin
      oplot,[dx(cnt-1),dx(cnt)],[dy(cnt-1),dy(cnt)],thick=1
   endif else begin
      if cnt eq 0 then oplot,[dx(cnt),dx(cnt)],[dy(cnt),dy(cnt)],psym=8
   endelse
   if !err eq 4 then begin
      if cnt gt 0 then begin
         oplot,[dx(cnt-1),dx(0)],[dy(cnt-1),dy(0)],thick=1
         dx(cnt) = -9999
         dy(cnt) = -9999
      endif
   endif
   cnt = cnt + 1
endwhile

;
;  only retain the used coordinates
;
dx = dx(where(dx ne -9999)) 
dy = dy(where(dx ne -9999)) 

;
;  translate data values back into array coordinates
;
ix = fix((dx-orig(0))/scale(0))
iy = fix((dy-orig(1))/scale(1))

;
;  if info needed
;
if keyword_set(info) then begin
   print,'Data coords...'
   for i=0,n_elements(dx)-1 do begin
      print,dx(i),dy(i)
   endfor
   print,'Pixels...'
   for i=0,n_elements(ix)-1 do begin
      print,ix(i),iy(i)
   endfor
endif

;
;  handle three cases
;  1) only a single point
;  2) a single line (definition of points liable to be updated)
;  3) a polygon
;
case n_elements(ix) of
    1:  begin
           a = ix
           b = iy
        end
    2:  begin
          xlim1 = min(ix)
          xlim2 = max(ix)
          ylim1 = min(iy)
          ylim2 = max(iy)
          a = intarr((s(1) > s(2))*2) -1
          b = intarr((s(1) > s(2))*2) -1
          kk = 0
          for j=ylim1,ylim2 do begin
             for i=xlim1,xlim2 do begin
                d = pnt_line([i,j],[ix(0),iy(0)],[ix(1),iy(1)])
                if d le 0.5 then begin
                   a(kk) = i
                   b(kk) = j
                   at = nint((a(kk)*scale(0))+orig(0)) + 1
                   bt = nint((b(kk)*scale(1))+orig(1))
                   oplot,[at,at],[bt,bt],psym=8,symsize=0.5
                   kk = kk + 1
                endif
             endfor
          endfor
          n = where(a ge 0)
          a = a(n)
          b = b(n)
        end
 else:  begin

            ix = [ix,ix(0)]
            iy = [iy,iy(0)]

            in = polyfillv(ix,iy,s(1),s(2))
            if in(0) eq -1 then return,-1
            
            polyfill,dx,dy,/line_fill
            
            a = intarr(n_elements(in))
            b = intarr(n_elements(in))
            
            for i=0,n_elements(in)-1 do begin
               a(i) = (get_ij(in(i),s(1)))(0) 
               b(i) = (get_ij(in(i),s(1)))(1) 
               at = a(i)
               bt = b(i)
               oplot,[at,at],[bt,bt],psym=8,symsize=0.5
            endfor
         end
endcase

print,'Number of spectra averaged = ',n_elements(a)

;
;  collect all spectra involved and average them
;
spec = 0
if not keyword_set(test) then begin
   band = gt_waveband(qlds.header.detector,lam1)
   if n_elements(a) gt 1 then begin
      spec = gt_scanp(qlds,xix=a,yix=b,band=band,lambda=wavelength)
   endif else begin
      spec = gt_spectrum(qlds,xix=a(0),yix=b(0),band=band,lambda=wavelength)
   endelse
   s = size(spec)
   if n_elements(a) gt 1 then spec = total(spec,2)/s(2)

;
;  and return with the average intensity and wavelength (if requested)
;
   if n_elements(a) gt 1 then begin
      wavelength = total(wavelength,2)/(size(wavelength))(2)
   endif

endif

;
;  trim return info unless FULL requested
;
if not keyword_set(full) then begin
   w = gt_wlimits(qlds,/wave,/quiet)
   wi = where(w(0,*) le lam1 and w(1,*) ge lam1, count)
   if count ge 0 then begin
      l1 = w(0,wi(0))
      l2 = w(1,wi(0)) 
      n1 = where(wavelength ge l1 and wavelength le l2,count)
      if count gt 0 then begin
         wavelength = wavelength(n1)
         spec = spec(n1)
      endif
   endif
endif

return,spec
 
end
