;+
; Project     : SOHO - CDS     
;                   
; Name        : READSUM2CDS()
;               
; Purpose     : Read and return the contents of a SUMER FITS level-1 file
;               
; Explanation : SUMER version of the CDS routine READCDSFITS.
;
;		Reads:
;			- header data (for the whole file), 
;			- fits variables for all detector data windows
;			- detector data 
;               	- fits variables for auxiliary data
;			- auxiliary data
;
;		Returns all data in one IDL structure (the CDS QL
;		data format)
;
; Use         : VAR = READSUM2CDS( FILENAME )
;    
; Inputs      : FILENAME = the name of the SUMER FITS level-1 file.
;               
; Opt. Inputs : None.
;               
; Outputs     : Returns an IDL structure with data from the FITS file
;		(CDS QL data format)
;               
; Opt. Outputs: None.
;               
; Keywords    : /DISPLAY : set this keyword to invoke DSP_INFO
;		to display information about the data in the
;		file. 
;
;		/SELECT : set to prompt for selection of only some
;		of the detector data windows.
;
;               PRESELECTED : Array of selected windows to read in. 
;               References the windows by their index (0..Nwindows-1)
;
;		/HEADER : Set to only read descriptive information
;		about detector data windows -- no data will be loaded
;
;		/AUTO : Set to force silent truncation of the data
;		by reading just those detector data windows that
;		fit below the memory limit
;
;               /BATCH : Set to suppress xack window popping up
;               when there are problems with the GSET_ID.
;
;		MEMLIMIT : Set to the maximum size of memory that
;		the detector data are allowed to occupy.
;
;		STATUS : Set to a initialized, named variable to 
;		return an error message instead of crashing. If
;		everything's ok, a null string will be returned.
;
;
; Calls       : AUXREAD(), DATATYPE(), SUMDETSELECT(), SUMDETREAD, 
;		DSP_INFO, FXBCLOSE FXBOPEN, HDRREAD(), QLMGR, XACK
;
; Common      : None.
;
; Restrictions: The file MUST be a CDS FITS level-1 file.
;               
; Side effects: Starts QLMGR if not installed.
;               
; Category    : Data_Handling, I_O, FITS, CDS, SUMER, QuickLook
;               
; Prev. Hist. : Based on READCDSFITS by Stein Vidar Hagfors Haugan
;
; Written     : William Thompson, GSFC, 5 July 1996
;               
; Modified    : Version 1, William Thompson, GSFC, 5 July 1996
;
; Version     : Version 1, 5 July 1996
;-            

FUNCTION Readsum2cds,filename,display=display,select=select,status=status,$
                     header=hdr,auto=auto,memlimit=memlimit,$
                     BATCH=BATCH,PRESELECTED=PRESELECTED
  
  On_Error,2

  IF !DEBUG ne 0 THEN On_Error,0
  
  IF N_elements(memlimit) eq 0 THEN memlimit = 15*2L^20	; 15 MB
  
;
;  Look for file, using $SUMER_FITS_DATA as the default directory, and .fits as
;  the default extension.  Open the binary table.
;

;  
;  check it's a scalar
;
  if n_elements(filename) gt 1 then begin
     print,'Input filename must be a scalar.'
     return, -1
  endif
 
;
;  force it to be, ie not array(1)
;
  filename = filename(0)

  use_file = find_with_def(filename,'$SUMER_FITS_DATA','.fits')
  if use_file EQ '' then begin
        print,'Cannot find specified FITS file'
        return,-1
  endif
  fxbOpen,unit,use_file,1,header,errmsg=status

  IF N_elements(status)	gt 0 THEN IF status ne '' THEN return,0
  
  selection = SumDetSelect(unit,header,select=select,header=hdr,$
                        auto=auto,memlimit=memlimit,status=status,$
                        preselected=preselected)
  IF N_elements(status)	gt 0 THEN IF status ne '' THEN goto,error_close_exit
  
  IF datatype(selection) ne 'STC' THEN $
	  message,'Failed reading detector data'
  
  IF selection.datasize	EQ 0 THEN BEGIN
	selection.dimension(*)=1
        type = 2
  end ELSE IF selection.longflag THEN type=3 $
  else                                type=2
  
  ;; Since we'll convert to physical units anyway...
  ;; 
  type = 4
  
;	detstring = "make_array(/long,dimension=selection.dimension)" $
;  ELSE	detstring = "make_array(/int, dimension=selection.dimension)"

  mheader = fxbmheader(unit)
  a ={  QL_ID:0.0D, QL_NO:0,$   ; Quicklook Manager tags
        hdrtext : mheader, $     ; Header text array
	Header: HdrRead(mheader),$
	DetData:reform(make_array(type=type,$
		   dimension=selection.dimension),$
		   selection.dimension),$  ; Detector data storage
	DetDesc:selection.DetDesc}  ; Descriptors of detector windows
  
  SumDetRead,unit,header,a,selection,status=status   ; Read the actual data.
  IF N_elements(status)	gt 0 THEN IF status ne '' THEN goto,error_close_exit
  
  fxbClose,unit                                        
  
  QLMGR,a
  
  IF Keyword_SET(display) THEN dsp_INFO,a; ,_EXTRA=extra
  
  return,a
  
  error_close_exit:
  fxbClose,unit
  return,0
END
