	PRO NIS_AVG_SPECT_DEMO
;+
; Project     :	SOHO - CDS
;
; Name        :	NIS_AVG_SPECT_DEMO
;
; Purpose     :	Demonstrate the NIS average quiet sun spectra.
;
; Category    :	Class2, CDS, Calibration, Spectrum
;
; Explanation :	This routine demonstrates the average spectra which were
;		obtained by combining a large number of NISAT_S runs which were
;		taken over a several month period.  These date are stored in
;		the files:
;
;			avg_qs_nis1_debias.fts
;			avg_qs_nis2_debias.fts
;			avg_qs_nis1_calib.fts
;			avg_qs_nis2_calib.fts
;
;		which should be in the $CDS_NIS directory.  The first two files
;		contain the average of spectra which have only had the
;		VDS_DEBIAS routine applied to them, while the last two have had
;		NIS_CALIB applied.  What is shown in this routine is the
;		average along the slit, but the files contain complete slit
;		images.
;
;		Possible uses of these data are:
;
;		* Serving as a baseline average quiet sun spectrum to compare
;		  other data against.
;
;		* Measuring the tilt, possibly as a function of wavelength,
;		  since any solar effects should be washed out by the large
;		  number of spectra which were averaged together.
;
;		* Measuring any non-uniformities in slit 4, i.e. features in
;		  the velocity or intensity which can be replicated from line
;		  to line.
;
;		One effect that this data demonstrates is that there is some
;		constant background which is not yet accounted for in our
;		calibration.  Possibly this is due to scattered Lyman alpha
;		radiation in the spectrograph.  The dashed lines in the second
;		set of plots shows the effect of the calibration software on a
;		uniform background.
;
; Syntax      :	NIS_AVG_SPECT_DEMO
;
; Examples    :	To send to a PostScript printer, use the following sequence:
;
;			PS
;			NIS_AVG_SPECT_DEMO
;			PSPLOT
;
; Inputs      :	None.
;
; Opt. Inputs :	None.
;
; Outputs     :	None.
;
; Opt. Outputs:	None.
;
; Keywords    :	None.
;
; Calls       :	FIND_WITH_DEF, FXREAD, PIX2WAVE, GET_WAVE_EFF, SETVIEW, YES_NO
;
; Common      :	None.
;
; Restrictions:	The FITS files must exist.  Also, the wave_eff database must be
;		accessible.
;
; Side effects:	None.
;
; Prev. Hist. :	None.
;
; History     :	Version 1,  4-Oct-1996, William Thompson, GSFC.
;
; Contact     :	WTHOMPSON
;-
;
	ON_ERROR, 2
;
;  Read in the FITS files.
;
	FILE = FIND_WITH_DEF('avg_qs_nis1_debias.fts', 'CDS_NIS')
	IF FILE EQ '' THEN MESSAGE, 'Cannot find file avg_qs_nis1_debias.fts'
	FXREAD, FILE, D1
;
	FILE = FIND_WITH_DEF('avg_qs_nis2_debias.fts', 'CDS_NIS')
	IF FILE EQ '' THEN MESSAGE, 'Cannot find file avg_qs_nis2_debias.fts'
	FXREAD, FILE, D2
;
	FILE = FIND_WITH_DEF('avg_qs_nis1_calib.fts', 'CDS_NIS')
	IF FILE EQ '' THEN MESSAGE, 'Cannot find file avg_qs_nis1_calib.fts'
	FXREAD, FILE, C1
;
	FILE = FIND_WITH_DEF('avg_qs_nis2_calib.fts', 'CDS_NIS')
	IF FILE EQ '' THEN MESSAGE, 'Cannot find file avg_qs_nis2_calib.fts'
	FXREAD, FILE, C2
;
;  Get the effect of a constant background on the data.
;
	W1 = PIX2WAVE('N1', INDGEN(1024))
	W2 = PIX2WAVE('N2', INDGEN(1024))
	GET_WAVE_EFF, '1996/10/4', 'N1', A1
	GET_WAVE_EFF, '1996/10/4', 'N2', A2
	S1 = 4 / POLY(W1, A1)
	S2 = 1 / POLY(W2, A2)
;
;  Plot the debiased data.
;
	ERASE
	SETVIEW, 1, 1, 1, 2
	XTITLE = 'Wavelength (A)'
	YTITLE = 'ADC/pixel'
	TITLE = 'Average NIS1 quiet sun spectra -- debiased data'
	PLOT_IO, W1(1:1022), AVERAGE(D1(1:1022,*),2), TITLE=TITLE,	$
		XTITLE=XTITLE, YTITLE=YTITLE
;
	SETVIEW, 1, 1, 2, 2
	TITLE = 'Average NIS2 quiet sun spectra -- debiased data'
	PLOT_IO, W2(1:1022), AVERAGE(D2(1:1022,*),2), TITLE=TITLE,	$
		XTITLE=XTITLE, YTITLE=YTITLE
;
;  Wait until the user is ready to move onto the next set of data.
;
	IF !D.NAME NE 'PS' THEN YES_NO, 'Hit return when ready for next plot'
;
;  Plot the calibrated data, and the estimated background component.
;
	ERASE
	SETVIEW, 1, 1, 1, 2
	YTITLE = 'photons/cm!U2!N/sec/arcsec!U2!N'
	TITLE = 'Average NIS1 quiet sun spectra -- calibrated data'
	PLOT_IO, W1(1:1022), AVERAGE(C1(1:1022,*),2), TITLE=TITLE,	$
		XTITLE=XTITLE, YTITLE=YTITLE
	OPLOT, W1, S1, LINESTYLE=2
;
	SETVIEW, 1, 1, 2, 2
	TITLE = 'Average NIS2 quiet sun spectra -- calibrated data'
	PLOT_IO, W2(1:1022), AVERAGE(C2(1:1022,*),2), TITLE=TITLE,	$
		XTITLE=XTITLE, YTITLE=YTITLE
	OPLOT, W2, S2, LINESTYLE=2
;
;  Return graphics to normal, and return.
;
	SETVIEW
	RETURN
	END
