;+
; Project     : SOHO - CDS     
;                   
; Name        : CAT_RAW.PRO
;               
; Purpose     : To catalogue and/or combine new raw (and pha) files 
;               
; Explanation : STM automatically creates .raw and .pha files when the GIS is
;		in raw data mode.  This routine combines files, if necessary,
;		and optionally adds them to the raw database.
;		The files are renamed when added to the database
;               
; Use         : IDL> cat_gis[, filename='rawfilename']
;		If rawfilename contains a '*', then it will be treated as a
;		filter to select the actual file.
;    
; Inputs      : prompts for rawfilename, if not supplied
;		if rawfilename contains a '*', the name is used as a filter
;		promts for Zone ID (where the file was taken)
;		prompts for a comment to add to the file
;
; Opt. Inputs : rawfilename
;               
; Outputs     : Two files containing all the raw/pha data, with all headers
;		except the last removed.  All trailing zeros also removed.
;               
; Opt. Outputs: None
;               
; Keywords    : filename
;
; Calls       : gis_raw_header, add_raw, round_off, spawn
;
; Common      : None
;               
; Restrictions: Can only be run under UNIX (uses spawn)
;               
; Side effects: !
;               
; Category    : Telemetry
;               
; Prev. Hist. : None
;
; Written     : E Breeveld (erb@mssl.ucl.ac.uk) 6 March 1996
;			From COMBINE_GIS_RAW
;               
; Modified    : ERB,  6 Mar 1996, v1.1, Seperated gis_raw_header function
;		ERB, 16 Apr 1996, v1.2, rewrite original files with new header
;		ERB, 19 Apr 1996, v1.3, return if picked file is empty
;		ERB, 24 Apr 1996, v1.4, add filename filter processing
;		ERB, 15 Jul 1996, v1.5, fixed filament id 'feature'
;		ERB, 24 Jun 1998, v1.6, use getenv in pickfile call
;
; Version     : 1.6
;-            

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

pro cat_raw, filename=filename
;
; check if filename already supplied
;
	rawfile = '' 
	filter = '*.raw'
	if (keyword_set(filename) ne 0) then begin
		if (strpos(filename, '*') eq -1) then rawfile = filename $
			else filter = filename
	endif
;
; Initialise arrays
;
	rawdata = bytarr (2, 256)
	phadata = bytarr (2, 4)
	header = bytarr (512)
	no_files = 0
;
another:
	valid = 0
	while (valid eq 0) do begin
;
		print, ''
;
		if (rawfile eq "") then $
			rawfile = pickfile (/read, path=getenv('CDS_GIS_RAW'), $
				filter=filter)
		if (rawfile eq "") then return
;
		phafile = strmid(rawfile, 0, strlen(rawfile)-4) + '.pha'
		rawfile = strmid(rawfile, 0, strlen(rawfile)-4) + '.raw'
;
		if (not file_exist(phafile)) then begin
			print, 'WARNING - file "', phafile, '" does not exist'
			goto, reread
		endif
;
		if (not file_exist(rawfile)) then begin
			print, 'WARNING - file "', rawfile, '" does not exist'
			goto, reread
		endif
;
		print, 'Using raw file "', rawfile, '"', ' and'
		print, '      pha file "', phafile, '"'
;
		openr, lunraw, /get_lun, rawfile
		openr, lunpha, /get_lun, phafile
;
		valid = 1
reread:
		if (not valid) then begin
			print, 'ERROR - cannot process raw and pha files'
			rawfile = ""
		endif
	endwhile
;
;  save the original filename
;
	no_files = no_files + 1
	if (no_files eq 1) then originalfile = rawfile
;
	readu, lunraw, header		; skip over raw header
	readu, lunpha, header		; read pha header
;
;  translate header and print to screen.  Note head_str is used later
;
	head_str = gis_raw_header (header)
;
	fstruc = fstat (lunraw)
	rawsize = fstruc.size
;
	fstruc = fstat (lunpha)
	phasize = fstruc.size
;
	rawdata = bytarr (rawsize - 512)
	phadata = bytarr (phasize - 512)
;
	readu, lunraw, rawdata
	readu, lunpha, phadata
;
	close, lunraw
	close, lunpha
;
	free_lun, lunraw
	free_lun, lunpha
;
	if (no_files eq 1) then begin
		wholeraw = rawdata
		wholepha = phadata
	endif else begin
		wholeraw = [wholeraw, rawdata]
		wholepha = [wholepha, phadata]
	endelse
;
; check if another file to be added
;
	yesno = ""
	read, 'Add another file? [y/n] default n: ', yesno
	if strupcase(yesno) eq "Y" then begin
;
;  New filter for pickfile - the last 10 characters of the current filename
;
		filter = '*' + strmid(rawfile, strlen(rawfile)-9, 10)
		rawfile = ""
		goto, another
	endif
;
;
;  All files added, Add to the raw files database?
;
	if (string(header(0)) eq 'R') then begin
		print, 'File already catalogued in the database'
		print, string(header(0:79))
	endif
;
ask_raw:
	yesno = ''
	read, 'Do you want to add this file to the raw files database (y/n)? ', yesno
	if (strupcase(yesno) eq 'N') then return
	if (strupcase(yesno) ne 'Y') then goto, ask_raw
	print, ''
;
	zone_id = 0
	print, 'Zone IDs -  1: Coronal;'
	print, '            2: Limb (not used);'
	print, '            3: Coronal Hole;'
	print, '            4: Quiet Sun;'
	print, '            5: Active.'
	read,  'Enter the zone id for the file:    ', zone_id
;
	desc = ''
new_desc:
	read, 'Describe the raw file (40 chars max): ', desc
	if (strlen(desc) gt 40) then goto, new_desc
;
; Work out the start time from the original filename
; Extract time and date
;
	filedate = strmid(originalfile, strlen(originalfile)-22, 12)
	filedate = 	strmid (filedate, 0, 4) + '-' + $		; year
			strmid (filedate, 4, 2) + '-' + $		; month
			strmid (filedate, 6, 2) + 'T' + $		; day
			strmid (filedate, 8, 2) + ':' + $		; hour
			strmid (filedate, 10, 2) + ':' + $		; min
			'00Z'						; sec
;
	errmsg = ''
	tai = utc2tai (filedate, errmsg=errmsg)
	if (errmsg ne '') then begin
		print, 'ERROR in CAT_RAW - ', errmsg
		return
	endif
;
; Produce a new filename
;
;;;	newfile = strmid(originalfile, 0, strlen(originalfile)-4)	; remove extension
;;;	newraw = newfile + '.d_raw'					; add '.d_raw
;;;	newpha = newfile + '.d_pha'					; add '.d_pha
;;; ;
;;;	newraw = strip_dirname (newraw)		; remove the directory
;;;	newpha = strip_dirname (newpha)
;
	newraw = rawfile
	newpha = phafile	
;
;  Try adding it to the database
;
	raw_id = 0			; default for adding a new file
;
;  Approximate the count rate regime
;
	bottom = round_off(head_str.bkgrcd/3, head_str.bkgrcd/10)
	top = round_off(head_str.bkgrcd*3, head_str.bkgrcd)
	cnt_rate = [bottom, top]
;
	if (head_str.bugmcpb gt 0) then ffb = 1 else ffb = -1
;
;  fix filament id 'feature'.  If filament current is zero, then assume
;  science data, i.e., make filament id zero
;
	if (head_str.fcurrent eq 0) then head_str.filament = 0
;
	raw_str = {	raw_id : raw_id,	$
			raw_desc : desc,	$
			start_time : tai,	$
			det_num : head_str.detector, $
			slit_num : head_str.slitn, $
			hvolt : head_str.bkghvl, $
			lld : head_str.bkgldld,	$
			cnt_rate : cnt_rate,	$
			raw_file : strip_dirname(newraw),	$
			ffb : ffb,		$
			fil_id : head_str.filament, $
			fil_cur : head_str.fcurrent, $
			zone_id : zone_id }
;
	errmsg = ''
	!priv = 2
	result = add_raw (raw_str, errmsg = errmsg)
	!priv = 0
	if (errmsg ne '') then begin
		print, 'ERROR - ', errmsg
		return
	endif
;
	print, ' '
	print, 'Success - new raw file ID  =', raw_str.raw_id
	print, 'Suggested count rate range =', cnt_rate(0), '  to', cnt_rate(1)
;
; All files added, now write the output file
;
	header(0) = byte (string ('Raw File ID', raw_str.raw_id, $
			'  Zone id', zone_id, '  ', desc))
;
;  Change the file mode
;
	command1 = 'chmod u+w ' + newraw
	command2 = 'chmod u+w ' + newpha
	spawn, command1
	spawn, command2
;
	openw, lunraw, newraw, /get_lun
	openw, lunpha, newpha, /get_lun
	writeu, lunraw, header
	writeu, lunpha, header
	writeu, lunraw, wholeraw
	writeu, lunpha, wholepha
	close, lunraw
	close, lunpha
	free_lun, lunraw
	free_lun, lunpha
;
;  Change the file mode back
;
	command1 = 'chmod u-w ' + newraw
	command2 = 'chmod u-w ' + newpha
	spawn, command1
	spawn, command2
;;
	print, ' '
	print, 'Written output files: ', newraw, ' and ', newpha
	print, 'New header information: '
;
	head_str = gis_raw_header (header)		; Write out the new header
;
return
end
