;+
; Project     : SOHO - CDS     
;                   
; Name        : GIS_DUMMY
;               
; Purpose     : Creates dummy data array for CDS GIS spectrograph
;
; Explanation : Takes past solar EUV data, folds in CDS performance
;		  and solar parameters to mimic GIS image. Includes -
;			(i)   solar lines in correct location on image
;		  	(ii)  intensities calculated from past data
;		  	      folding in CDS efficiencies
;			(iii) variable slit use
;			(iv)  two window options
;			(v)   background intensities
;			(vi)  statistical noise
;		  Neglects pixel sampling effects (assumed small) and
;		  grating scatter (assumed small w.r.t. slit image
;		  spread).
;
;                 If requested, the output array is written to a disk file
;                 with an unformatted write.  In order to retrieve the data
;                 use the following commands:
;                 IDL> gis = intarr(4,2048)
;                 IDL> openr,lun,'gis.dat',/get_lun
;                 IDL> readu,lun,gis
;
; Use	        : IDL> gis_dummy [,slit=slit,expose=expose etc.]
;                 IDL> gis_dummy, slit=4,length=4,expose=10,wlist=1,image=gis,
;				file='gis.dat'
;
;                 Would return four profiles in variable gis (4,2048)
;		  Also stores in file called gis.dat (4,2048)
;
; Inputs	: Slit width and length, Accumulation time and Window option
;                 are prompted for if not supplied as keyword parameters.
;
; Optional i/p	: None
;
; Outputs	: None
;
; Optional o/p	: See Image keyword
;
; Keywords	: Slit         The slit size (2,4,8 or 90 arcsec).
;                 Length       The slit length (for 2, 4 arcsec slit widths)
;                 Expose       The exposure time in seconds.
;                 Wlist        The line list reference number.
;		  Image	       Name of data array returned
;		  File	       Name of data file created (default image.dat)
;		  No_file      Inhibit saving of data on disk
;                 Zone         Determines source model
;
; Restrictions	: Requires solar data in files gi1.dat, gi2.dat, gi3.dat
;		  and gi4.dat.
;
; Side effects	: None
;
; Category	: Planning, Technical, Model
;
; Written	: R A Harrison, RAL, July 1993
;
; Modified	: C D Pike, 9 Sep 93. Added LENGTH keyword and updated header.
;                                     Changed output array 'image' to integer.
;
;                 CDP, 24-Mar-95 added ZONE keyword.
;                 CDP, 27-Mar-95 update data file format to handle 6 zones.
;                 CDP, 07-Sep-95 include filaments as zone types, make 
;                                default zone = 4 (quiet).
;                 CDP, handle filament variations.  CDP, 17-Oct-95
;                 CDP, Fix array range error.  19-Mar-96
;                 CDP, Bump up line widths to match real data. 28-Apr-96
;
;
; Version	: Version 8, 28-Apr-96
;-
;
pro gis_dummy,slit=slit,length=length,expose=expose,wlist=wlist,file=file,$
   image=image,no_file=no_file,debug=debug,zone=zone

;help,slit,length,expose,wlist,file
;
;  Trap i/o errors
;
on_ioerror,finish
;
;  Contains pixel number, upper spectrum, lower spectrum wavelengths
;
band1=fltarr(2048,2)	;2048 pixels 155-224A band, 0=wavelength, 1=intensity
band2=fltarr(2048,2)	;ditto for 261-346A band
band3=fltarr(2048,2)	;ditto for 395-496A band
band4=fltarr(2048,2)	;ditto for 662-787A band

band1(*,0) = pix2wave('G1',indgen(2048))
band2(*,0) = pix2wave('G2',indgen(2048))
band3(*,0) = pix2wave('G3',indgen(2048))
band4(*,0) = pix2wave('G4',indgen(2048))

;
; Request slit size if not set on command line
;
good_slit = [2,4,8,90]
if not keyword_set(slit) then begin
   read,' Please input slit width choice - 2, 4, 8, 90 arcsec .....>',slit
endif
;
;  Check valid response
;
ok = where(slit eq good_slit)
while ok(0) lt 0 do begin
   read,' Please input slit width choice - 2, 4, 8, 90 arcsec .....>',slit
   ok = where(slit eq good_slit)
endwhile

;
;  get slit length for count rate calculation
;
if slit eq 2 and not keyword_set(length) then begin
   read,' Select slit length - 2, 240 arcsec ....>',length
endif

if slit eq 4 and not keyword_set(length) then begin
   read,' Select slit length - 4, 240 arcsec ....>',length
endif

if slit eq 90 then length=240.
if slit eq 8 then length=50.8

;
;slit width in units of pixels
;
slit=slit/2.0		

if keyword_set(debug) then stop,0


;
; Calculate Gaussian profile: store spectral profile in array 101 which is
; barely enough for slit 90 but overkill for others .
;
;grt1=2.3	; 1 arcsec projected onto detector (see Blue Book)
;grt2=1.9
;grt3=1.6
;grt4=1.3
;
;  Revised values
;
grt1=6.5
grt2=4.0
grt3=5.7
grt4=3.6

grt1=(grt1*slit*2.)/2.
grt2=(grt2*slit*2.)/2.
grt3=(grt3*slit*2.)/2.
grt4=(grt4*slit*2.)/2.

xprofile=fltarr(4,101)	;Gaussian profile from slit
;
;  In the X direction convolve the Gaussian line profile (which already
;  contains the smearing due to the detector) with the shape of the slit.
;  Note: For even slit widths (in pixels), this will shift the centroid of the
;  line 1/2 pixel to the right.
;
xi = indgen(101)-50
xprofile(0,*) = convol( exp(-xi^2/(2.*grt1*grt1)), replicate(1.,slit))
xprofile(1,*) = convol( exp(-xi^2/(2.*grt2*grt2)), replicate(1.,slit))
xprofile(2,*) = convol( exp(-xi^2/(2.*grt3*grt3)), replicate(1.,slit))
xprofile(3,*) = convol( exp(-xi^2/(2.*grt4*grt4)), replicate(1.,slit))
;
;  Normalize intensity
;
xprofile(0,*) = xprofile(0,*) / (sqrt(2.*!pi) * grt1)
xprofile(1,*) = xprofile(1,*) / (sqrt(2.*!pi) * grt2)
xprofile(2,*) = xprofile(2,*) / (sqrt(2.*!pi) * grt3)
xprofile(3,*) = xprofile(3,*) / (sqrt(2.*!pi) * grt4)
if keyword_set(debug) then stop,1

;
;  set up default zone (quiet) if none supplied
;  save filament zones for later
;
if not keyword_set(zone) then zone = 4
if zone ge 7 then begin
  fil_zone = zone
  zone = 7
endif


;
;  setup default 
;
;  Read the line intensities in first spectrum
;
openr,lun_gi1,concat_dir('$CDS_PLAN_TECH','gi1.dat'),/get_lun ;155-224A intensities
count = 0
wave1 = fltarr(500) & inten1 = fltarr(500) & datum=fltarr(9)  

while not eof(lun_gi1) do begin
   readf,lun_gi1,datum
   wave1(count)  = datum(0)
   inten1(count) = datum(zone)
   count = count + 1
endwhile
free_lun,lun_gi1
;
;  Count number found and trim arrays
;
count = where(wave1 gt 0.0)
nline1 = n_elements(count)
wave1  =  wave1(0:nline1-1)
inten1 = inten1(0:nline1-1)
if keyword_set(debug) then stop,2
;
;  ... and the second spectrum
;
openr,lun_gi2,concat_dir('$CDS_PLAN_TECH','gi2.dat'),/get_lun	;261-346A intensities
count = 0
wave2 = fltarr(500) & inten2 = fltarr(500) & datum=fltarr(9)  
while not eof(lun_gi2) do begin
   readf,lun_gi2,datum
   wave2(count)  = datum(0)
   inten2(count) = datum(zone)
   count = count + 1
endwhile
free_lun,lun_gi2
count = where(wave2 gt 0.0)
nline2 = n_elements(count)
wave2  =  wave2(0:nline2-1)
inten2 = inten2(0:nline2-1)
if keyword_set(debug) then stop,3
;
;   ... and the third spectrum
;
openr,lun_gi3,concat_dir('$CDS_PLAN_TECH','gi3.dat'),/get_lun	;395-496A intensities
count = 0
wave3 = fltarr(500) & inten3 = fltarr(500) & datum=fltarr(9)

while not eof(lun_gi3) do begin
   readf,lun_gi3,datum
   wave3(count)  = datum(0)
   inten3(count) = datum(zone)
   count = count + 1
endwhile
free_lun,lun_gi3
count = where(wave3 gt 0.0)
nline3 = n_elements(count)
wave3  =  wave3(0:nline3-1)
inten3 = inten3(0:nline3-1)
if keyword_set(debug) then stop,4
;
; ... and the fourth spectrum
;
openr,lun_gi4,concat_dir('$CDS_PLAN_TECH','gi4.dat'),/get_lun	;662-787A intensities
count = 0
wave4 = fltarr(500) & inten4 = fltarr(500) & datum=fltarr(9)

while not eof(lun_gi4) do begin
   readf,lun_gi4,datum
   wave4(count)  = datum(0)
   inten4(count) = datum(zone)
   count = count + 1
endwhile
free_lun,lun_gi4
count = where(wave4 gt 0.0)
nline4 = n_elements(count)
wave4  =  wave4(0:nline4-1)
inten4 = inten4(0:nline4-1)
if keyword_set(debug) then stop,5
;
;  Find where line falls in wavelength array and convolve the xprofile
;  with the intensity
;
;  First spectrum
;
for i=0,nline1-1 do begin
   n = max(where(band1(*,0) le wave1(i))) 
   if n ge 50 and n lt 1998 then begin
      band1(n-50:n+50,1) = band1(n-50:n+50,1) + xprofile(0,*) * inten1(i)
   endif
endfor
;
;  Second spectrum
;
for i=0,nline2-1 do begin
   n = max(where(band2(*,0) le wave2(i)))
   if n ge 50 and n lt 1998 then begin
      band2(n-50:n+50,1) = band2(n-50:n+50,1) + xprofile(1,*) * inten2(i)
   endif
endfor
;
;  Third spectrum
;
for i=0,nline3-1 do begin
   n = max(where(band3(*,0) le wave3(i)))
   if n ge 50 and n lt 1998 then begin
      band3(n-50:n+50,1) = band3(n-50:n+50,1) + xprofile(2,*) * inten3(i)
   endif
endfor
;
;  Fourth spectrum
;
for i=0,nline4-1 do begin
   n = max(where(band4(*,0) le wave4(i)))
   if n ge 50 and n lt 1998 then begin
      band4(n-50:n+50,1) = band4(n-50:n+50,1) + xprofile(3,*) * inten4(i)
   endif
endfor
if keyword_set(debug) then stop,3.5
;
;  Get the accumulation time if it was not set on the command line
;
if not keyword_set(expose) then begin
   read,' Input Accumulation Time (Sec) .........>',expose
endif
;
; Estimate geometrical factor (i.e. counts are given for 2x2 arcsec area)
;
slitfactor = (slit*2.*length)/(2.*2.)
;
;  Bump up counts accordingly
;
band1(*,1)=band1(*,1)*expose*slitfactor
band2(*,1)=band2(*,1)*expose*slitfactor
band3(*,1)=band3(*,1)*expose*slitfactor
band4(*,1)=band4(*,1)*expose*slitfactor

;
;  create artificial filament type signal
;
if (zone eq 7) then begin
   zone = fil_zone
   if (zone eq 7) or (zone eq 8) then temp = gauss_put(8192,0,500,1024,2048,back=20)
   if (zone eq 9) or (zone eq 10) then temp = gauss_put(8192,0,500,7168,2048,back=20)
   band1(*,1) = temp(0:2047)
   band2(*,1) = temp(2048:4095)
   band3(*,1) = temp(4096:6143)
   band4(*,1) = temp(6144:8191)
endif


;
;  Get line list required if not specified on command line
;
if not keyword_set(wlist) then begin
   print,' Input Window/Line List - Options are:'
   print,'     1   all data'
   print,'     2   three windows - 180-200A, 310-320A, 750-780A'
   read,'      .........>',wlist
endif
good_wlist = [1,2]
;
;  Check valid response
;
ok = where(wlist eq good_wlist)
while ok(0) lt 0 do begin
;  bell
   read,' Please input valid option: ',wlist
   ok = where(wlist eq good_wlist)
endwhile
;
;  Set up mask according to line list option chosen
;
mask1=bytarr(2048)	;first band window mask
mask2=bytarr(2048)	;second band window mask
mask3=bytarr(2048)	;third band window mask
mask4=bytarr(2048)	;fourth band window mask

case wlist of 
   1: begin 
         mask1(*) = 1 
         mask2(*) = 1
	 mask3(*) = 1
	 mask4(*) = 1
      end
   2: begin
	 mask1(721:1291)=1
	 mask2(1163:1397)=1
	 mask4(1415:1894)=1
      end
   else: print,'Invalid line list option:'
endcase
if keyword_set(debug) then stop,5
;
;  Add an assumed background of 1 c/s 
;
band1(*,1) = (band1(*,1)+expose)
band2(*,1) = (band2(*,1)+expose)
band3(*,1) = (band3(*,1)+expose)
band4(*,1) = (band4(*,1)+expose)


;
;  Add noise - random distribution, function of intensity
;
a  = (randomu(seed,2048) - 0.5) * 2.0
band1(*,1)=band1(*,1) + (a * sqrt(band1(*,1)))
a  = (randomu(seed,2048) - 0.5) * 2.0
band2(*,1)=band2(*,1) + (a * sqrt(band2(*,1)))
a  = (randomu(seed,2048) - 0.5) * 2.0
band3(*,1)=band3(*,1) + (a * sqrt(band3(*,1)))
a  = (randomu(seed,2048) - 0.5) * 2.0
band4(*,1)=band4(*,1) + (a * sqrt(band4(*,1)))

;
;  Force to be positive
;
band1 = band1 > 0
band2 = band2 > 0
band3 = band3 > 0
band4 = band4 > 0

if keyword_set(debug) then stop,8

;
; Mask off the data
;
n = where(band1(*,1) gt 65000L)
if n(0) ge 0 then band1(n,1) = -10000L
n = where(band2(*,1) gt 65000L)
if n(0) ge 0 then band2(n,1) = -10000L
n = where(band3(*,1) gt 65000L)
if n(0) ge 0 then band3(n,1) = -10000L
n = where(band4(*,1) gt 65000L)
if n(0) ge 0 then band4(n,1) = -10000L

band1(*,1) = band1(*,1) * mask1 
band2(*,1) = band2(*,1) * mask2 
band3(*,1) = band3(*,1) * mask3
band4(*,1) = band4(*,1) * mask4

;
;  form the image array
;
image=lonarr(4,2048)

image(0,*)=long(band1(*,1))
image(1,*)=long(band2(*,1))
image(2,*)=long(band3(*,1))
image(3,*)=long(band4(*,1))

;
;  Create output file if needed.
;
if not keyword_set(no_file) then begin
   if keyword_set(file) then fname = file else fname = 'image.dat'
   print,'Output file name is '+fname
   openw,lun_out,fname,/get_lun
   writeu,lun_out,image
   free_lun,lun_out
endif

print,' GIS_Dummy - Normal End'
return

;
;  error return
;
finish:
print,' GIS_Dummy - error return'
print,'Probably trouble reading the input data files:'
print,'gi1.dat, gi2.dat, gi3.dat and gi4.dat'
print,'which should be in directory $CDS_PLAN_TECH'
print,' '

end

