;+
; Project     : SOHO - CDS     
;                   
; Name        : LOAD_TP_STRUCT
;               
; Purpose     : Load the raster planning structure onto the screen widgets.
;               
; Explanation : The input parameter is a structure of the type defined in
;               tp_def_struct to specify a raster.  The values within this
;               structure are loaded into their appropriate widgets on the
;               technical planning screen page1.
;               
; Use         : IDL> load_tp_struct, tp_str
;    
; Inputs      : tp_str   -  the raster definition structure.  See tp_def_struct
;                           for its definition.
;               
; Opt. Inputs : None
;               
; Outputs     : None
;               
; Opt. Outputs: None
;               
; Keywords    : None
;
; Calls       : None
;               
; Restrictions: Uses technical planning common blocks for passing information.
;               
; Side effects: None
;               
; Category    : Technical planning
;               
; Prev. Hist. : None
;
; Written     : C D Pike,  RAL,  30-Jun-1993
;               
; Modified    : Fix possible arithmetic bug.  CDP, 5-Jul-94.
;               Updated for new tplan structure.  CDP, 8-Jul-94
;               Trim procedure call names to 15 chars.  CDP, 12-Sep-94
;               Pick up any default extractions.  CDP, 3-Jan-95
;               Handle default NIS extraction height in case
;               of no spectral lines.  CDP, 22-Feb-95
;               Replace calls to NINT by ROUND.  CDP, 17-Jun-95
;               Update zone names.  CDP, 07-Sep-95
;               Handle flag windows mode (flag_state). CDP, 18-Oct-95
;               Fix window heights for full CCD extraction case. CDP, 2-Nov-95
;               Update compression scheme IDs.  CDP, 2-Nov-95
;               Fix zone description interpretation.  CDP, 27-Nov-95
;               Handle interface with mk_study better.  CDP, 08-Aug-96
;               Fix bug in NIS default extraction logic.  CDP, 15-Aug-96
;               Turn on GIS default extraction as default.  CDP, 10-Mar-97
;               Adjust user scheme handling.              CDP, 06-Oct-97
;               Add new LOW++ telemetry rate.            CDP, 12-Dec-97
;               Fix edge effect bug?  CDP, 14-Apr-99
;               Insist on real data as default.  CDP, 04-May-99
; 
; Version     : Version 20, 4-May-99
;-            

pro load_tp_struct,  tp_str


;
;  use normal common blocks for info
;
@tplan_com


;
;  check input parameter
;
if n_params() ne 1 or datatype(tp_str) ne 'STC' then begin
  bell,2
  print,'Invalid parameter input to LOAD_TP_STRUCT.'
  return
endif

;
;  crude check on input structure, if OK load into standard structure and sort
;  to load the number of windows variable
;
if datatype(tp_str) eq 'STC' then begin              

   if n_tags(tp_str) ne n_tags(tp_obs) then begin
      bell,2
      print,'Input structure does not match current tp_obs definition.'
      return
   endif else begin

;
;  make sure VDS mode is defined
;
      err=''
      get_utc,utc
      get_vds_state,utc,sdef,err=err
      if err eq '' then copy_struct,sdef,tp_str,/recur_to
      err=''
      get_vds_slitpos,utc,pdef,err=err
      if err eq '' then copy_struct,pdef,tp_str,/recur_to


;
;  when passed from mk_study the input structure is incomplete so we need to 
;  load the line list and dex details again
;

      tp_obs = tp_str
      if tp_obs.raster_v.ras_id gt 0 and $
            tp_obs.raster_v.ras_var gt 0 then begin
         get_raster,tp_obs.raster_v.ras_id,tp_obs.raster_v.ras_var,raster1
         copy_struct, raster1, tp_obs, /recur_to
         if raster1.ll_id gt 0 then begin
            get_linelist,tp_obs.raster_v.ll_id,db_ll
            tp_db2int, db_ll, raster1
         endif

         if tp_obs.raster_v.dw_id ge 0 then begin
            get_datawin,tp_obs.raster_v.dw_id,dw
            n = where(dw.wins.win_def(2) gt 0)
            if n(0) ge 0 then begin
               tp_obs.data_win.n_windows = n_elements(n)
               tp_obs.data_win2.win_def(*)  = 0
               tp_obs.data_win2.win_name    = ' '
               for i=0,n_elements(n)-1 do begin
                  tp_obs.data_win2(i).win_def(*)  = dw.wins(i).win_def(*)
                  tp_obs.data_win2(i).win_name    = dw.wins(i).win_name
               endfor
            endif
         endif
         stat = tp_load_vdsback(tp_obs)
         tp_str = tp_obs
      endif


      tp_update_pix, tp_obs
      n = where(tp_obs.line_w.flag eq 2, count)
      if count gt 0 then begin
         tp_obs.line_w(n).flag = 1
         flag_state = 0
      endif
      line_window = tp_obs.line_w
      sort_windows,line_window,num_windows
   endelse
endif

;
;  clear any preset buttons first
;
widget_control,top_left,set_button=0
widget_control,top_middle,set_button=0

;
;  check each of the widgets in turn and load the appropriate data onto the
;  screen.
;

;
;  check which dataset was used (if any) those without any accompanying data
;  have their IDs increased by 100 as a flag??!!
;
;
zones = ['','CORONA','LIMB','CORONAL HOLE','QUIET','ACTIVE','FLARE',$
         'GIS_FIL1_NEG','GIS_FIL1_POS','GIS_FIL2_NEG','GIS_FIL2_POS']

zoneid = [4,101,102,103,4,5,106,7,8,9,10]
if tp_obs.raster_v.rv_desc ne '' then begin
   czone = str_pick(tp_obs.raster_v.rv_desc,'<','>')
   zid = where(zones eq czone)
   if zid(0) le 0 then begin 
      current_zone = 4
   endif else begin
      current_zone = zoneid(zid(0))
   endelse
endif



;
;  first the instrument
;
case tp_obs.raster_p.detector of
  'N': begin
;
;  insist on real data as the default
;
           if current_zone eq 4  then current_zone = -4
           if current_zone eq 5  then current_zone = -5
           widget_control,instr_ids(0),set_button=1
           widget_control,image_area,sensitive=1
           tp_coltable
           tp_get_dumdata, tp_obs, VDS_image_data,zone=current_zone
           tv,tp_bscale(VDS_image_data)
           display_nis_win
           tp_vds_axes
           widget_control,vds_backbase(0),map=0
           widget_control,vds_backbase(2),map=0
           widget_control,vds_backbase(1),map=1

           widget_control,gis_dispbase(0),map=0
           widget_control,gis_dispbase(1),map=0
           widget_control,gis_dispbase(2),map=1

           for i=0,4 do widget_control,parmbase(i),map=0
           widget_control,parmbase(1),map=1


           for i=0,2 do widget_control,vdsb_ids(i),set_button=0
           if tp_obs.data_win.vds_back eq 1 then begin
              widget_control,vdsb_ids(0),set_button=1
              vds_extension = 1
           endif else begin
              widget_control,vdsb_ids(1),set_button=1
              vds_extension = 0
           endelse

           if tp_obs.state.vds_read eq 0 then tp_obs.state.vds_read = 1
           if tp_obs.raster_v.vds_map lt 2 then tp_obs.raster_v.vds_map = 2

;
;  and the window height (only relevant for NIS). If have no lines 
;  then go by data extraction windows.
;
           if tp_obs.line_list.n_lines gt 0 then begin
              curr_lheight = tp_str.line_w(0).height
           endif else begin
              if tp_obs.raster_v.dw_id gt 0 then begin
                 get_datawin,tp_obs.raster_v.dw_id,dw
                 curr_lheight = dw.w_height*2
              endif
           endelse
           if (curr_lheight eq 0) or (curr_lheight gt 240) then begin
              curr_lheight = 240
           endif
           if curr_lheight gt 0 then begin
              widget_control,wh_slider_id,set_val=curr_lheight
           endif

;
;  check incoming data windows for full spectral extraction
;
           offset = tp_obs.state.vds_offset 
           nis_def_ext = [0,0,0]
           n = where(tp_obs.data_win2.win_def(2) eq 1024)
           if n(0) ge 0 then begin
              xpos2 = 392 + round((240-curr_lheight)/4.) + offset(0)
              xpos1 = 512 + round((240-curr_lheight)/4.) + offset(1)
              for i=0,n_elements(n)-1 do begin
                 case tp_obs.data_win2(n(i)).win_def(1) of
                  xpos1: nis_def_ext(0) = 1
                  xpos2: nis_def_ext(1) = 1
                      0: nis_def_ext(2) = 1
                   else:
                 endcase
              endfor
           endif
           for i=0,2 do widget_control,ex_ids(i),set_b=nis_def_ext(i)

         end
  'G': begin
           loadct,0
           widget_control,instr_ids(1),set_button=1
           widget_control,image_area,sensitive=1
           tp_get_dumdata, tp_obs, GIS_data,zone=current_zone
           tp_plot_gis,GIS_data,GIS_band
           display_gis_win

           widget_control,vds_backbase(1),map=0
           widget_control,vds_backbase(0),map=0
           widget_control,vds_backbase(2),map=1

           widget_control,gis_dispbase(0),map=0
           widget_control,gis_dispbase(2),map=0
           widget_control,gis_dispbase(1),map=1

           for i=0,4 do widget_control,parmbase(i),map=0
           widget_control,parmbase(3),map=1
           curr_lheight = 1

;
;  approx check on default extraction
;
           gis_def_ext = [0,0,0,0]
           def = where(tp_str.data_win2.win_def(2) eq 2048)
           if def(0) eq -1 then begin
              for i=0,3 do widget_control,gbdef_ids(i),set_button=1
              gis_def_ext = [1,1,1,1]
           endif else begin
              for i=0,n_elements(def)-1 do begin
                 if tp_str.data_win2(def(i)).win_def(0) eq 0 then begin
                    gis_def_ext(0) = 1
                    widget_control,gbdef_ids(0),set_button=1
                 endif
                 if tp_str.data_win2(def(i)).win_def(0) eq 2048 then begin
                    gis_def_ext(1) = 1
                    widget_control,gbdef_ids(1),set_button=1
                 endif
                 if tp_str.data_win2(def(i)).win_def(0) eq 4096 then begin
                    gis_def_ext(2) = 1
                    widget_control,gbdef_ids(2),set_button=1
                 endif
                 if tp_str.data_win2(def(i)).win_def(0) eq 6144 then begin
                    gis_def_ext(3) = 1
                    widget_control,gbdef_ids(3),set_button=1
                 endif
              endfor
           endelse
         end
   else: erase
endcase

;
;  then the slit choice
;
slit = ['(2 x 2)','(4 x 4)','(8 x 51)','(2 x 240)','(4 x 240)','(90 x 240)']
ns = tp_obs.raster_p.slit_num
if ns gt 0 then begin
   widget_control,slit_info,$
              set_val=string('#',ns,' - ',slit(ns-1),form='(a1,i1,a3,a)')
endif

;
;  then the exposure time range buttons
;
eran = 3

;
;  have sensible default
;
if tp_str.raster_v.exptime le 0 then tp_str.raster_v.exptime = 10

if tp_str.raster_v.exptime gt 0 then curr_et_max = 1
if tp_str.raster_v.exptime gt 1. then curr_et_max = 10 
if tp_str.raster_v.exptime gt 10. then curr_et_max = 100 
if tp_str.raster_v.exptime gt 100. then curr_et_max = 1000 
if tp_str.raster_v.exptime gt 1000. then curr_et_max = 10000 


;
;  and the slider value
;
                       
curr_etime = tp_str.raster_v.exptime
widget_control,et_slider_id,set_val=round((curr_etime*1000L/curr_et_max))

;
;  and the text widget giving the value
;
et_form = ['(f5.2)','(f4.1)','(i3)','(i4)','(i5)']
i = fix(alog10(curr_et_max))
val = curr_etime
acc = curr_et_max/100.
val = round_off(val,acc)
stat = execute('char_temp = string(val,form=et_form(i))')
widget_control,et_info,set_value=char_temp
              
;
;  load the line list description
;
curr_llist_name = tp_str.line_list.ll_desc
widget_control,wll_list_1,/sensitive


;
;  these only useful if some windows exist
;
if num_windows gt 0 then begin
;
;  the general line width slider (try and figure it out from the
;  info given)
;
   if tp_obs.raster_p.detector eq 'N' then pmax = 1023 else pmax = 2047
   nok = where(tp_str.line_w.left gt 0 and tp_str.line_w.right lt pmax)
   if nok(0) eq -1 then begin
      if  (tp_str.line_w(0).right-tp_str.line_w(0).left)  gt 1500 and $
                                    tp_str.raster_p.detector eq 'G' then begin
         curr_lwidth = 2048
      endif else begin
         curr_lwidth = 1000
      endelse
   endif else begin
      ok_wid = tp_str.line_w(nok).right - tp_str.line_w(nok).left  
      ok_wid = ok_wid(where(ok_wid gt 0))
      curr_lwidth = fix(round(mode_val(ok_wid+1)))
   endelse
   if curr_lwidth gt 100 then curr_ww_max = 1000 else curr_ww_max = 100
   widget_control,ww_info,set_val=curr_lwidth
   val = fix(curr_lwidth*1000./curr_ww_max) > 3
   widget_control,ww_slider_id,set_val=(val < 1000)

;
;  reset width of edge affected windows but reset 'full extraction' case
;
    

;   n = where(tp_str.line_w.left eq 0 and tp_str.line_w.right gt 0)
;   if n(0) ge 0 then begin
;      tp_str.line_w(n).left  = tp_str.line_w(n).right - curr_lwidth + 1
;      tp_str.line_w(n).x_pos = tp_str.line_w(n).x_pos - (curr_lwidth/4)
;      print,'left ',n,tp_str.line_w(n).left,tp_str.line_w(n).x_pos
;   endif

;   n = where(tp_str.line_w.right eq pmax)
;   if n(0) ge 0 then begin
;      help,tp_str.line_w(n(0)),/str
;      tp_str.line_w(n).right  = tp_str.line_w(n).left + curr_lwidth - 1
;      tp_str.line_w(n).x_pos = tp_str.line_w(n).x_pos + (curr_lwidth/4)
;      help,tp_str.line_w(n(0)),/str
;   endif

   n = where(tp_str.line_w.left eq 0 and tp_str.line_w.right eq pmax)
   if n(0) ge 0 then begin
      tp_str.line_w(n).x_pos = (pmax+1)/2
   endif

;
;  recopy for any updates
;
   tp_obs = tp_str
   line_window = tp_str.line_w

endif 

;
;  set the text windows
;
widget_control,sssx,   set_value=string(tp_str.raster_p.xstep)
widget_control,nsx,    set_value=string(tp_str.raster_p.nx)
if tp_str.raster_p.detector eq 'G' then begin
   widget_control,nsy,  set_value=string(tp_str.raster_p.ny)
   widget_control,sssy, set_value=string(tp_str.raster_p.ystep)
endif else begin
   widget_control,nsy,  set_value=' n/a'
   widget_control,sssy, set_value=' n/a'
endelse

;
;  slit scans not allowed for VDS
;
if tp_str.raster_p.detector eq 'N' then begin
   widget_control, sssy, sensitive=0
   widget_control, nsy, sensitive=0
endif else begin
   widget_control, sssy, sensitive=1
   widget_control, nsy, sensitive=1
endelse
;
;  and the compression scheme
;
if tp_str.raster_v.comp_id gt 0 then begin
   words = ["","Straight copy","Truncate to 12 bits","VBWL Blocksize ",$
            "","","Sum over lines","Sum over windows","Sum "]
   scheme = words(tp_str.raster_v.comp_id)
   if tp_str.raster_v.comp_id eq 3 then begin
      words = ['Default','8','16','32','64','128','256','512']
      scheme = scheme + words(tp_str.raster_v.comp_opt)
   endif

   if tp_str.raster_v.comp_id eq 8 then begin
      words = ['2 rows','4 rows','6 rows','8 rows','10 rows','12 rows','14 rows','16 rows']
      scheme = scheme + words((tp_str.raster_v.comp_opt/2)-1)
   endif

   if scheme eq 'Sum over lines' then begin
      if tp_str.raster_v.vds_orient eq 0 then begin
         scheme = 'Sum over wavelength'
      endif else begin
         scheme = 'Sum over spatial dimension'
      endelse
   endif
   widget_control,ccs_info_2,set_value=scheme
endif else begin
   widget_control,ccs_info_2,set_value=''
endelse

;
;  load telemetry rate
;
if tp_str.raster_v.tel_rate eq 'L' then widget_control,tmr_info_2,set_value='LOW'
if tp_str.raster_v.tel_rate eq 'P' then widget_control,tmr_info_2,set_value='LOW++'
if tp_str.raster_v.tel_rate eq 'M' then widget_control,tmr_info_2,set_value='MEDIUM'
if tp_str.raster_v.tel_rate eq 'H' then widget_control,tmr_info_2,set_value='HIGH'



;
;  and the raster/variation, line list and data window descriptors
;
widget_control,wrp_info_2,set_value=tp_str.raster_p.ras_desc
widget_control,wrv_info_2,set_value=tp_str.raster_v.rv_desc 
widget_control,wll_info_2,set_value=tp_str.line_list.ll_desc
widget_control,wdw_info_2,set_value=tp_str.data_win.dw_desc


return
end
