;+
; Project     : SOHO - CDS
;
; Name        : mk_gset
;
; Purpose     : To load, retreive, modify and add entries to the GSET database
;
; Explanation : The GSET database contains information needed to set up the
;		GIS for use.  It includes HVs, LUTs etc.
;
; Use         : IDL> mk_gset
;
; Inputs      : Prompts user
;
; Opt. Inputs : None
;
; Outputs     : Modifies GSET database as necessary
;
; Opt. Outputs: None
;
; Keywords    : None
;
; Calls       : IDL widget library, GSET database access routines
;
; Restrictions: None
;
; Side effects: None
;
; Category    : GIS, planning
;
; Prev. Hist. : None
;
; Written     : ERB MSSL/UCL 10 November 1995
;
; Modified    : ERB 30 January 1996
;		 New definition of the database - LUTs are byte, 
;		 detector enabled flag added.
;		 FFB  can only be +/- 12V
;		ERB 6 February 1996
;		 Bug fix to display LUT parameters
;		ERB 31 May 1996, v1.4
;		 Allow V4/V3 IDl compatibility
;		ERB 8 July 1996, v2.0
;		 Add automatic wavelength calibration based on LUT start/stop
;		Version 2.5, 18-Apr-1997, William Thompson, GSFC
;			Temporary fix to put launch date of '2-Dec-1995' as
;			effective date.
;		ERB 30 September 1999
;		 printed zone text as well as numbers
;
; Version     : Version 2.6, 30 Sept 1999
;-
; ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
pro mk_gset_event, event
;
; What to do if an event happens
; ------------------------------
;
@gset_widget_ids.common
;
;  Switch on the hourglass
;
	widget_control,/hourglass
;
	case event.id of
;
;  find out what events occured
; 
	    topmenu: begin
;
;  These are the top buttons
;
	  	    case (event.value) of
;
			'Top-up Raw Data'	: gset_modify
;
			'Save New Entry'	: gset_save
;
			'Done'			: widget_control, $
							event.top, /destroy
;
		    else: print, 'ERROR - gset_event - Missing event from topmenu'
;
		    endcase
		end
;
;  These are the list selections
;
	    selectmenu(0): begin
		    widget_control, selectmenu(0), get_uvalue=uvalue

		    selected_id = uvalue(event.index)	; get the GSET ID
		    gset_retrieve, selected_id		; retrieve the record
		end
		
	    selectmenu(1): begin
		    widget_control, selectmenu(1), get_uvalue=uvalue

		    selected_id = uvalue(event.index)
		    gset_retrieve, selected_id
		end
		
	    selectmenu(2): begin
		    widget_control, selectmenu(2), get_uvalue=uvalue

		    selected_id = uvalue(event.index)
		    gset_retrieve, selected_id
		end
		
	else:
	endcase



end ; gset_event

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

pro gset_save

;
;  This gets done if you press "Save Entry"
;  ----------------------------------------
;
@gset_widget_ids.common

;
;   Reset the error flag first
;
	errmsg = ''

;
;  Get all the values from the screen
;
;
	widget_control, slitmenu, get_value=slitnumber
; remember the "+1"
	slitnumber = slitnumber + 1

	description = strarr(4)
	for i=0, 3 do begin
		widget_control, descmenu(i), get_value=value
		description(i) = value
	endfor

	widget_control, ffbmenu, get_value=ffbvalue
; remember the "-1"
	if (ffbvalue eq 0) then ffbvalue = -1
;
	widget_control, zonemenu, get_value=zonenumber
	zonenumber = fix(zonenumber[0])		; ignore any text
	if (zonenumber le 0) or (zonenumber ge 7) then $
		errmsg = 'Illegal zone id - enter a number between 1 and 7'

	hvsetting = intarr(4)
	for i=0, 3 do begin
		widget_control, hvmenu(i), get_value=value
		hvsetting(i) = value
	endfor
	
	lldsetting = intarr(4)
	for i=0, 3 do begin
		widget_control, lldmenu(i), get_value=value
		lldsetting(i) = value
	endfor
	
	checksums = strarr(4)
	for i=0, 3 do begin
		widget_control, checksummenu(i), get_value=value
		checksums(i) = value
	endfor
	
	rawids = intarr(4)
	for i=0, 3 do begin
		widget_control, rawmenu(i), get_value=value
		rawids(i) = value
	endfor

	detused = intarr(4)
	for i=0, 3 do begin
		widget_control, usedmenu(i), get_value=value
		detused(i) = value
	endfor

	widget_control, filsetmenu, get_value=filid

	lutpars = intarr(4,11)
	for i=0,3 do begin
		for j=0,10 do begin
			widget_control, lutmenu(i,j), get_value=value
			lutpars(i,j) = value
		endfor
	endfor

;
;   Check LUT parameters and convert to bytes
;
	if ((min(lutpars) lt 0) or (max(lutpars) gt 255)) $
		then 	errmsg = 'LUT parameters must be byte' $
		else	lutpars = byte(lutpars)

;
;    Check that at least one detector is used!
;    And fill other dummy data as necessary
;
	if (max(detused) eq 0) then $
			errmsg = 'At least one detector must be used!'
;
	for i = 0, 3 do if (detused(i) eq 0) then checksums(i) = '0000'
;
;   Check if all is OK, then fill in a new entry
;
	if (errmsg eq '') then begin
;
;   For new entries, gset_id must be zero
;
		gset_id = 0
;
;   Fill in the structure
;
        	gset_structure = { 	GSET_ID: gset_id,		$
                			HVOLT: hvsetting,  	        $
                			LLD: lldsetting,            	$
                			LUT_CHKSUM: checksums,      	$
                			LUT_PAR: lutpars,		$
                			RAW_ID: rawids,			$
                			SLIT_NUM: slitnumber,		$
                			ZONE_ID: zonenumber,		$
                			FIL_ID: filid,			$
                			FFB: ffbvalue,			$
					DET_USED: detused,		$
                			GSET_DESC: description	}
;
;   Fill the database with the new record
;
		!priv = 2
		result = add_gset(gset_structure, errmsg=errmsg)
		!priv = 0
	endif

	if errmsg eq '' then begin
;
;  No errors, send a message and report the new GSET ID to the idmenu
;
		widget_control, messagemenu, /append,   $
		  set_value='Success - new GSET_ID =' + $
		  string(gset_structure.gset_id)
		widget_control, idmenu, $
		  set_value = 'New GSET ID: ' + 	$
		    strtrim(gset_structure.gset_id, 2), $
	 	  set_uvalue = gset_structure.gset_id
;
;  Now update the wavelength calibration data for the new GSET
;
		gset_wavecal, gset_structure.gset_id, lutpars, errmsg

		if (errmsg ne '') then begin
			widget_control, messagemenu, /append, 		$
			  set_value='ERROR - gset_wavecal - ' + errmsg
			widget_control, messagemenu, /append, 		$
			  set_value = $
	'  Cannot automatically produce a wavelength calibration for this GSET'
		endif
;
;  update all the screen information, including the new lists
;
		gset_retrieve, gset_structure.gset_id
		gset_list
;
	endif else begin
		widget_control, messagemenu, /append, $
				set_value='FAILED - gset_save - '+errmsg
	endelse
;
end ; gset_save

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

pro gset_wavecal, current_id, lutpars, errmsg

;
;  Insert the GSET wavelength calibration into the wavelength database 
;  -------------------------------------------------------------------
;

	start = lutpars(*, 2)
	stop  = lutpars(*, 1)

;
;  work out if starts/stops are nominal
;
	ok = size( $
	  where(((start ne 31) and (start ne 41)) or ((stop - start) ne 42)))

;
	errmsg = ''		; flag to test for calibrated wavelengths

	if (ok(0) ne 0) then begin
		errmsg = 'Non-nominal start/stop values'
		return
	endif

;
;  get coefficients
;
	get_wavecal, 'GIS', '', wavecal, gset_id=22	; start = 31, 41, 31, 31
	coeff22 = wavecal.coeff			

	get_wavecal, 'GIS', '', wavecal, gset_id=23	; start = 31, 41, 41, 31
	coeff23 = wavecal.coeff

	get_wavecal, 'GIS', '', wavecal, gset_id=17	; start = 31, 31, 31, 41
	coeff17 = wavecal.coeff
;
	coeff = dblarr(4, 3, 2)
;
;  Use each detector individually
;
	if (start(0) eq 31) then begin			; detector 1 (index 0)
		coeff(0, *, *) = coeff22(0, *, *)

	endif else begin				; i.e. start eq 41
		errmsg = 'Non-nominal start/stop values'
		return
	endelse
;
	if (start(1) eq 31) then begin			; detector 2 (index 1)
		coeff(1, *, *) = coeff17(1, *, *)
		
	endif else begin				; i.e. start eq 41
		coeff(1, *, *) = coeff22(1, *, *)
		
	endelse
;
	if (start(2) eq 31) then begin			; detector 3 (index 2)
		coeff(2, *, *) = coeff22(2, *, *)
		
	endif else begin				; i.e. start eq 41
		coeff(2, *, *) = coeff23(2, *, *)
		
	endelse
;
	if (start(3) eq 31) then begin			; detector 4 (index 3)
		coeff(3, *, *) = coeff23(3, *, *)
		
	endif else begin				; i.e. start eq 41
		coeff(3, *, *) = coeff17(3, *, *)
		
	endelse

	if (errmsg eq '') then begin

		!priv = 2
		Result = ADD_WAVECAL('2-Dec-1995', 'GIS', COEFF, GSET_ID=current_id, ERRMSG=ERRMSG, $
		  filename='mk_gset  - from calibrations for GSETs 17,22,23' )	
		!priv = 1

	endif


end ; gset_wavecal

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

pro gset_modify

;
;  This gets done if you press "Top-up raw data"
;  ---------------------------------------------
;
@gset_widget_ids.common

;
;  Pick up the current gset structure
;
	widget_control, idmenu, get_uvalue=current_id

;
;  write a warning message
;
	widget_control, messagemenu, /append, 		$
		set_value='WARNING - gset_modify - ' +	$
		  'Only raw IDs and descriptions will be modified'
;
;  get the relevant information from the screen
;
	description = strarr(4)
	raw_ids = intarr(4)
;
	for i=0, 3 do begin
		widget_control, descmenu(i), get_value=value
		description(i) = value
		widget_control, rawmenu(i), get_value=value
		raw_ids(i) = value
	endfor
;
;   Fill the database with the new record
;
	!priv = 2
	errmsg = ''
	result = mod_gset (current_id, RAW_ID=raw_ids, $
			GSET_DESC=description, errmsg=errmsg)
;;;
;;; MOD_GSET: Syntax:  Result = MOD_GSET( GSET_ID, keywords=... )
;;;
	!priv = 0

	if (errmsg ne '') then begin
		widget_control, messagemenu, /append, 		$
			set_value='ERROR - gset_modify - ' + errmsg
	endif else begin
;
;  No errors, send a message
;
		widget_control, messagemenu, /append,   $
		  set_value='Success - modified GSET_ID =' + $
		  string(current_id)
	endelse
;
;  Update screen information
;
	gset_retrieve, current_id
;
end ; gset_modify

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

pro gset_list

;
;  This procedure fills the list of currently defined GSETS and descriptions
;  -------------------------------------------------------------------------
;
@gset_widget_ids.common
;
;  Loop through the three types of GSETS
;
	type = ['Science', 'Filament 1', 'Filament 2']
;
	for i=0,2 do begin
;
		list_gset, i, gsets, nfound		; get the data
		if nfound ne 0 then begin
;
;  Extract the info
;
		   list = 'Zone '+string(format='(I2)', gsets.zone_id) + ', '
;
;  If science data (non-filament) add the slit number
;
		   if i eq 0 then list = list + $
				'Slit ' + strtrim(gsets.slit_num, 2) + ', '
;  Add GSET ID
		   list = list + 'GSET' + string(format='(I4)', gsets.gset_id) + $
				', '
;
;  Add the raw IDs
;
		   list = list + 'Raw IDs'
		   for j=0,3 do list = list +	$
			 string(format='(I4)',gsets.raw_id(j,*))
;
;	reverse sort by the info in the list
;
		   sortorder = reverse(sort([list]))		; force into array
;
;  Fill the list widget, in zone order.  The 'uvalues' contain an array of the
;	GSET ids in the same order as the listed records
;
		   ids = gsets.gset_id				; get the ids
;
		   widget_control, selectmenu(i), $ 		; fill in widget
			set_value=list(sortorder), $
			set_uvalue=ids(sortorder)

		 endif else begin

		   widget_control, messagemenu, /append, $
			set_value='No GSETs found for '+type(i)
		 endelse
	endfor

	widget_control, messagemenu, /input_focus
;
end ; gset_list


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


pro gset_retrieve, new_id
;
;  This gets done if you select a GSET from the lists
;  --------------------------------------------------
;
@gset_widget_ids.common

;
;  Get the record
;
	errmsg = ''
	get_gset, new_id, gset_structure, errmsg=errmsg

	if errmsg eq '' then begin

;
;  fill the fields
;
		widget_control, idmenu, set_value = 'GSET ID: '+ $
			strtrim(gset_structure.gset_id, 2), 		$
			set_uvalue = gset_structure.gset_id
;
; don't forget the "-1"! for the slit
		widget_control, slitmenu, set_value = gset_structure.slit_num - 1

; don't forget the "+1"! for the ffb
		value = gset_structure.ffb
		if (value eq -1) then value = 0
		widget_control, ffbmenu, set_value = value
;
; convert zone id into sensible text
;
		openr, /get_lun, zonelun, $
			'$CDS_PLAN_TECH/zone_definitions.dat'
		zonetext = ""
		for zonenum = 1, gset_structure.zone_id do $
			readf, zonelun, zonetext
		free_lun, zonelun
		zonetext = strmid(zonetext, 1, 2) + strmid(zonetext, 8, 20)
		widget_control, zonemenu, set_value=zonetext
;
		widget_control, filsetmenu, set_value=gset_structure.fil_id

		for i=0,3 do begin
			widget_control, descmenu(i), $
				set_value=gset_structure.gset_desc(i)
			widget_control, hvmenu(i), $
				set_value=gset_structure.hvolt(i)
			widget_control, lldmenu(i), $
				set_value=gset_structure.lld(i)
			widget_control, checksummenu(i), $ 
				set_value=gset_structure.lut_chksum(i)
			widget_control, rawmenu(i), $
					set_value=gset_structure.raw_id(i)
			widget_control, usedmenu(i), $
					set_value=gset_structure.det_used(i)
			for j=0,10 do begin
				widget_control, lutmenu(i,j), $
					set_value=fix(gset_structure.lut_par(i,j))
			endfor
		endfor
;
	endif else begin
		widget_control, messagemenu, /append, $
			set_value='FAILED - gset_retrieve - '+errmsg
	endelse
;;; ;
;;; ;  reset the lists
;;; ;
;;;	gset_list

end ; gset_retrieve

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

pro mk_gset
		
;
;  This is it - set it all up
;  --------------------------
;
@gset_widget_ids.common
;
	base = widget_base (title='GSET database', /column)
;
;  Combined base for top buttons and current GSET ID
;
	topbase = widget_base (base, /row)
;
;   Menu for "Operation" "Done"
;
	opmenu = replicate( {flags:0, name:''}, 3)
	opmenu.flags = [0, 0, 0]
	opmenu.name = [	'Save New Entry',	$
			'Top-up Raw Data', 	$
			'Done']
	topmenu = cw_pdmenu (topbase, opmenu, /return_name)
;
;  menu for list of GSETS
;
	listbase = widget_base (base, /row)
	list1base = widget_base (listbase, /column)
	list2base = widget_base (listbase, /column)
	list3base = widget_base (listbase, /column)
;
	temp = widget_label(list1base, value='Science data')
	temp = widget_label(list2base, value='Filament 1')
	temp = widget_label(list3base, value='Filament 2')
;
	selectmenu = lonarr(3)
	selectmenu(0) = widget_list(list1base, ysize=10, font='fixed', xsize=60)
	selectmenu(1) = widget_list(list2base, ysize=10, font='fixed', xsize=50)
	selectmenu(2) = widget_list(list3base, ysize=10, font='fixed', xsize=50)
;
;  big base for lots of stuff 
;
	bigbase = widget_base (base, /row)
;
;  Combined base for slit, ffb, zone and filament
;
	slitffbbase = widget_base (bigbase, /column)
;
;  GSET ID
;
	idmenu = widget_label (slitffbbase, /frame, 			$
					Value='No Current GSET ID',	$
					Uvalue = 0 )
;
;  Menu for zone id
;
	zonemenu = cw_field(slitffbbase, title='Zone id  ', /string)
;
;  menu for slit selection
;
	slitno = strtrim(indgen(6)+1, 2)
	slitmenu = cw_bgroup(slitffbbase, /exclusive, slitno, /return_index, $
			label_left='Slit Number  ', /row, /frame)
;
;  Menu for front face Bias
;
	ffb = ['+12 Volts', '-12 Volts']
	ffbmenu = cw_bgroup(slitffbbase, /exclusive, ffb, /return_name, $
			label_left='Front Face Bias  ', /row, /frame)
;
;  Menu for filament on/off one/two
;
	filset = ['Science', 'Fil. 1', 'Fil. 2']
	filsetmenu = cw_bgroup(slitffbbase, /exclusive, filset, /return_index, $
			label_left='Science / Filament  ', /row, /frame)
;
;  Base for HV, LLD, Checksums, and raw file IDs
;
	hlcbase = widget_base(bigbase, /frame, column=5)
;
;  Menu for detector used switches
;
	usedmenu = lonarr(4)
	temp=widget_label (hlcbase, value='Detectors used')
	for i=0, 3 do usedmenu(i) = cw_bgroup(hlcbase, /row, $
			/nonexclusive, strtrim(string(i+1),2))
;
;  Menu for HV setting
;
	hvmenu = lonarr(4)
	temp = widget_label (hlcbase, value='HV set')
	for i=0, 3 do hvmenu(i) = cw_field (hlcbase, $
				title=strtrim(i+1, 2), /integer, xsize=5)
;
;  Menu for LLD settings
;
	lldmenu = lonarr(4)
	temp = widget_label (hlcbase, value='LLD set')
	for i=0, 3 do lldmenu(i) = cw_field (hlcbase, $
				title=strtrim(i+1, 2), /integer, xsize=5)
;
;  Menu for Checksum settings
;
	checksummenu = lonarr(4)
	temp = widget_label (hlcbase, value='Checksum')
	for i=0, 3 do checksummenu(i) = cw_field (hlcbase, $
				title=strtrim(i+1, 2), /string, xsize=5)
;
;  Menu for raw file ids
;
	rawmenu=lonarr(4)
	temp = widget_label (hlcbase, value='Raw File')
	for i=0, 3 do rawmenu(i) = cw_field(hlcbase, /integer, $
				title=strtrim(i+1, 2), xsize=5)
;
;  Menus for description
;
	descbase = widget_base (base, row=2, /frame)
	descmenu = lonarr(4)
	for i=0, 3 do descmenu(i) = cw_field(descbase, $
			title='Description '+strtrim(i+1, 2), xsize=40)

;
;  Menu for LUT parameters
;
	lbase = widget_base (base, /frame)
	lutbase = widget_base (lbase, column=11)
	lutmenu = lonarr (4, 11)
;
;  Labels
;
	parlabel = ['LUT    %  ', 'stop', 'start', 'k', 'zpln', 'phi', $
				'bits', 'det', 'x', 'y', 'z']

;
;  4*11 parameters
;
	for j=0,10 do begin
		temp = widget_label (lutbase, value=parlabel(j))
		 
		for i=0,3 do begin
			if (j eq 0) then begin
				lutmenu(i,j) = cw_field(lutbase, /integer, xsize=4, $
				title=strtrim(i+1, 2)+' ')
			endif else begin
				lutmenu(i,j) = cw_field(lutbase, /integer, xsize=4, $
				title=' ')
			endelse
		endfor
	endfor
;
;  message box
;
	messagemenu = widget_text (base, /scroll, value='Messages:', ysize=6) 
	widget_control, messagemenu, /append,	$
		set_value='Select a GSET from the lists, ' +	$
		  'and/or enter new values, then press Save if required'
;
;  Fill in the lists - leave all other info blank
;
	gset_list
;
;  Now lets see if it works!
;
	widget_control, base, /realize
	xmanager, 'mk_gset', base

end ; gset
