;+
; Project     : SOHO - CDS     
;                   
; Name        : MK_RASTER
;               
; Purpose     : Widget selection and definition of a CDS raster observation.
;               
; Explanation : The user selects the required setup using various widgets to
;               define the setup parameters.
;
; Use         : IDL> mk_raster, setup_structure
;
; Inputs      : setup_structure  -  is a structure this is taken to define
;                                   an observation and the details are loaded
;                                   into the widgets on screen.  The values
;                                   can then be adjusted as required.  If the
;                                   parameter is not a structure then it is
;                                   taken to be an output parameter and the
;                                   setup must be defined from scratch on the
;                                   screen.
;               
; Opt. Inputs : None
;               
; Outputs     : setup_structure  - on exit, whatever its input form, this
;                                  will be a structure containing the details
;                                  of the observation according to the latest
;                                  structure definition - see TPLAN_STRUCT
;
; Opt. Outputs: None
;               
; Keywords    : modal        - option for when calling from other programs
;               group_leader - for control from other programs
;               font         - specifies something other than the default font
;               fix_desc     - Not for general use.
;
; Calls       : Many
;               
; Restrictions: None
;               
; Side effects: None
;               
; Category    : Technical planning.
;               
; Prev. Hist. : Based on the original tp_page1.
;
; Written     : C D Pike, RAL, 8-Jul-94
;               
; Modified    : Renamed and updated from TPLAN.  CDP, 7-Dec-94
;               Added scroll bars to main base widget for use on small
;               screens.  CDP, 14-Feb-95
;               Added font keyword, and all widgets use font.  
;                                                            CDP/WT 28/Mar/95
;               Use copy_struct when changing instruments.   CDP, 18-May-95
;               Replace calls to NINT by ROUND.              CDP, 17-Jun-95
;               Fixed error in call to ROUND.                CDP, 28-Jun-95
;               Add GIS filaments to zone types.             CDP, 7-Sep-95
;               Use new state database call (cp_get_state).  CDP, 07-Sep-95
;               Upgrade handling of typed in exposure times. CDP, 26-Sep-95
;               Limit exposure times to 1000 secs, increase size of
;               descriptor widgets and handle GIS filament zone IDs.  
;                                                            CDP, 17-Oct-95
;               Rework messages window.                      CDP, 18-Oct-95
;               Make inclusion of VDS background windows the default.
;                                                            CDP, 23-Oct-95
;               Include new version of 'sum over lines' compression.
;                                                            CDP, 2-Nov-95
;               Include IEF access.                          CDP, 8-Nov-95
;               Add database IDs to display.                 CDP, 9-Nov-95
;               Change security access to databases.         CDP, 10-Nov-95
;               Restore ZDBASE on exit.                      CDP, 13-Nov-95
;               Allow update of db descriptions.             CDP, 27-Nov-95
;               Change command setting ZDBASE on exit        CDP, 18-Jan-96
;               Added call to GET_DFONT to check availability
;               of button fonts.
;               Used STRPAD in place of XSIZE in WIDGET_LABEL
;               to fix version 4 sizing problems.
;               Added calls to XALIVE to recover from crashes
;               when MK_RASTER is called by another program.
;                                                             DMZ, 14-Feb-96
;               Changed code for scrolling message window.    DMZ, 20-Feb-96
;               Changed call to cp_get_state to get_vds_state CDP, 23-Feb-96
;               Cleared a load of redundant code connected with 
;               detector setup                                CDP, 04-Mar-96
;               Gave up on SAVEGEN, back to IDL SAVE.         CDP, 14-Mar-96
;               Use new wavecal system but assume linear.     CDP, 15-Mar-96
;               Add curr_lheight to call to tp_get_desc       CDP, 10-Apr-96
;               Limit number of mirror/slit locations to 255. CDP, 28-Apr-96
;               Trap no write access for save file on exit.   CDP, 07-May-96
;               Trap max exp time on type in.                 CDP, 13-May-96
;               Changed newdata .pdm file 2b independent.     CDP, 19-Jul-96
;               Move .pdm files to software directory,
;                  tidy VDS parameter display, make 12 bit
;                  truncation default NIS compression.        CDP, 29-Jul-96
;               Tidy use of ZDBASE.                           CDP, 08-Aug-96
;               Resize telemetry timing widgets.               CDP, 15-Aug-96
;               Change handling of messages window.             CDP, 10-Oct-96
;               Handle user compression scheme/include TM rate.  CDP, 06-Oct-97
;               Include new LOW++ telemetry rate                  CDP, 12-Dec-97
;               Extended /ALL keyword in text widget to /ALL_TEXT DMZ 1-May-98
;               Add real NIS data to display.                    CDP, 05-May-99
;               Added GROUP_LEADER as UVALUE to MAIN            DMZ, 27-Dec-99
;               Cut out test write for directory on exit.      CDP, 20-Sep-00
; 
; Version     : Version 41, 20-Sep-00
;-            


pro mk_raster_event,ev

;
;  event processor for technical planning procedure (mk_raster)
;

;
;  pick up common definitions
;
@tplan_com


;
;  make sure plotting window is focussed before dealing with events from here
;
widget_control, image_area, get_v=wind
wset, wind


;
;  setup defaults for exposure time display
;
et_title = 'Exposure time (secs).'
et_maxima = [1.,10.,100.,1000.]
et_form = ['(f5.2)','(f4.1)','(i3)','(i4)']

;
;  name arrays for instruments and slits
;
slit_list=['(2 x 2)','(4 x 4)','(8 x 51)','(2 x 240)','(4 x 240)','(90 x 240)']
slit_len = [2,4,51,240,240,240]
slit_wid = [2,4,8,2,4,90]
inst_list = ['N','G']
;
;  get the values, user values and type of widget involved
;
widget_control,ev.id,get_uvalue=uvalue,get_value=value
name = strmid(tag_names(ev,/str),7,1000)
if n_elements(uvalue) eq 0 then uvalue = ''
if n_elements(value)  eq 0 then  value = ''
;print,'name: ',name,' value: ',value,' uvalue: ',uvalue
;help,ev,/str


;
;  Intercept any events generated when in HELP mode (except the one
;  to turn off online help of course!). Note turn off all the 
;  non-exclusive buttons so they can only generate ON events.
;
if n_elements(last_uvalue) eq 0 then last_uvalue = ''
if help_mode and (uvalue ne 'HELP_ONLOFF') then begin
   if uvalue eq last_uvalue then begin
      widget_control, base, /clear_event
      return
   endif
   for i=0,2 do widget_control,ex_ids(i),set_button=0
   for i=0,3 do widget_control,gbdef_ids(i),set_button=0
   widget_control, base, /clear_event
   tp_call_help, uvalue, ev
   last_uvalue = uvalue
   return
endif
            


;
;  act according to type of widget
;
case name of
;
;  Button will be the type of most of the user input from various wwidgets
;
 'BUTTON': begin
            case strmid(uvalue,0,4) of
;
;  Exit from the system (read text widgets first)
;
             'EXIT': begin

;  Save latest TP_OBS in GROUP_LEADER ID (DMZ, Dec'99)

                      widget_control,ev.top,get_uvalue=group_leader
                      if xalive(group_leader) then $
                       widget_control,group_leader,set_uvalue=tp_obs

                      tp_read_textw

                      if db_original eq 'CDS' then $
                          stat = fix_zdbase(/cds) else $
                          stat = fix_zdbase(/user)

;
;  reset wavecal
;

                      if not load_wavecal('n',cal_struct=n_wavecal) then begin
                         print,'Error restoring NIS Wavelength Calibration'
                      endif
                   
                      if not load_wavecal('g',cal_struct=g_wavecal) then begin
                         print,'Error restoring GIS Wavelength Calibration'
                      endif
                   
                      case strmid(uvalue,5,4) of
                       'QUIT': widget_control,/destroy,ev.top
;
;  want to save internal structure in a save file
;
                       'SAVE': begin
                                  id1 = strtrim(string(tp_obs.raster_p.ras_id,$
                                               form='(i3)'),2)
                                  id2 = strtrim(string(tp_obs.raster_v.ras_var,$
                                               form='(i3)'),2)
                                  id3 = strtrim(string(tp_obs.line_list.ll_id,$
                                               form='(i3)'),2)
                                  id4 = strtrim(string(tp_obs.data_win.dw_id,$
                                               form='(i3)'),2)
                                  sfile = 'tp_'+id1+'_'+id2+'_'+id3+'_'+id4
;                                  if not test_open(/write,/quiet) then begin
;                                     xinput,nsfile,['Cannot write to current directory',$
;                                             'Give new save file DIRECTORY'],$
;                                             group=base
;                                     break_file,nsfile,disk,dir,ff,ext
;                                     if dir ne '' then begin
;                                        sfile = concat_dir(dir,sfile)
;                                     endif else begin
;                                        flash_msg,info_line,'Invalid directory name'
;                                        sfile = ''
;                                     endelse
;                                  endif
                                  if sfile ne '' then begin
                                     if not test_open(sfile,/write,/quiet) then begin
                                        flash_msg,info_line,'Cannot write there either'
                                     endif else begin
                                        save,tp_obs,file=sfile
                                        bell
                                        print,' ' & print,' '
                                        print,'MK_RASTER structure saved in file ',$
                                               sfile
                                        print,"Use IDL> restore,file='",$
                                                                       sfile,"'"
                                        print,'to retrieve it into the tp_obs variable.'
                                        widget_control,/destroy,ev.top
                                     endelse
                                  endif else begin
                                     flash_msg,info_line,'Invalid save file name'
                                  endelse
                               end
                         else: flash_msg,info_line,'One or the other please!',$
                               /append
                      endcase
                      loadct,0
                     end

;               
;  HELP will put the help file to the screen
;
             'HELP': begin
                        mode = strmid(uvalue,5,3)
                        case mode of
                          'ONL': begin
                                    if help_mode eq 0 then begin
                                       help_mode = 1
                                       tp_help = tp_read_help('tp_buttons.hlp') 
                                       widget_control, image_area, get_v=wind
                                       wset, wind
                                       tp_draw_message
                                       for i=0,2 do begin
                                          widget_control,ex_ids(i),set_b=0
                                       endfor
                                       for i=0,3 do begin
                                          widget_control,gbdef_ids(i),set_b=0
                                       endfor
;                                       widget_control,nsx,/all_text
;                                       widget_control,nsy,/all_text
;                                       widget_control,sssy,/all_text
;                                       widget_control,sssx,/all_text
;                                       widget_control,wrp_info_2,/all_text
;                                       widget_control,wrv_info_2,/all_text
;                                       widget_control,wdw_info_2,/all_text
;                                       widget_control,ww_info,/all_text
;                                       widget_control,et_info,/all_text
                                       widget_control, sssy, sens=1
                                       widget_control, nsy, sens=1
                                       popup_msg,font=tp_font,group=base,[$
                                       'You are now in online help mode.',$
                                       'When you select any widget with the '+$
                                       'mouse, information on that item will',$
                                       'appear in a widget like this.',$
                                       'The raster set up is not affected in'+$
                                       ' any way.',$
                                       ' ',$
                                       'Beware that the slider widgets can '+$
                                       'generate a lot of events, ',$
                                       'and in some text widgets you will need '+$
                                       'to hit the return key to see the help.'+$
                                       ' ',$
                                       'Dismiss this widget before proceeding.']
                                    endif else begin
                                       help_mode=0
;                                       widget_control,nsx,all_text=0
;                                       widget_control,nsy,all_text=0
;                                       widget_control,sssy,all_text=0
;                                       widget_control,sssx,all_text=0
;                                       widget_control,wrp_info_2,all_text=0  
;                                       widget_control,wrv_info_2,all_text=0
;                                       widget_control,wdw_info_2,all_text=0
;                                       widget_control,ww_info,all_text=0
;                                       widget_control,et_info,all_text=0
                                       load_tp_struct, tp_obs
                                    endelse
                                 end
                          'GEN': widg_help,'tech_plan.hlp',/modal
                          'ALL': begin
                                    tp_dump_setup, tp_obs
                                    widget_control,info_line,/append,set_val=$
                                              'Setup details sent to printer'
                                 end

                          'STR': show_struct, tp_obs

                          'RAS': begin
                                    tp_dump_setup, tp_obs, /raster
                                    widget_control,info_line,set_val=$
                                              'RASTER details sent to printer'
                                 end
                          'LIN': begin
                                    tp_dump_setup, tp_obs, /line
                                    widget_control,info_line,set_val=$
                                              'LINE LIST details sent to printer'
                                 end
                          'DWI': begin
                                    tp_dump_setup, tp_obs, /dwin
                                    widget_control,info_line,set_val=$
                                              'DATA WINDOW details sent to printer'
                                 end
                          else:
                        endcase
                     end

;
;  make message window visible or not
;
             'MESS': begin
                        if see_mess eq 0 then begin
                           widget_control, info_base, /show
                           widget_control, m10, set_val='Hide messages'
                           see_mess = 1
                        endif else begin
                           widget_control, base, /show
                           widget_control, m10, set_val='See messages'
                           see_mess = 0
                        endelse
                     end

;
;  Read details from databases into widget display
;
             'DBAS': begin
                        mode = strmid(uvalue,6,4)
                        if tp_obs.raster_p.detector ne 'N' and $
                           tp_obs.raster_p.detector ne 'G' then begin
                           flash_msg,info_line, 'Choose detector first.'
                        endif else begin

                          case mode of
                           'RASP':  begin
                                     tp_disp_rast, status
                                     if status then begin
                                        widget_control, fr_base, map=1
                                        xmanager, 'tp_disp_rast',fr_base,$
                                              event_handler='tp_d_r_event'
                                     endif
                                    end
                           'LINE':  begin
                                     if tp_obs.raster_p.detector eq 'G' or $
                                         tp_obs.raster_p.detector eq 'N' then begin
                                       tp_disp_llist, status
                                       if status then begin
                                          widget_control, ll_base, map=1
                                          xmanager, 'tp_disp_llist',ll_base,$
                                                    event_handler='tp_d_ll_event'
                                       endif
                                     endif else begin
                                       flash_msg, info_line,$
                                                'Must define instrument first.'
                                     endelse
                                    end
                           'FLAG':  begin
                                      tp_disp_ief, status
                                      if status then begin
                                         widget_control, flag_base, map=1
                                         xmanager, 'tp_disp_ief', flag_base,$
                                            event_handler='tp_d_flg_event'
                                      endif
                                    end
                            else:
                          endcase
                        endelse
                        tp_recalc_dew,' '
                     end

;
;  RESET will do a general (or selective) reset of the setup and will return
;        the user to the initial state.  The observation definition
;        structure tp_obs is cleared. All current windows are deleted. 
;        The default values for exposure time and window width and
;        height are reactivated.  The buttons affecting the window
;        manipulation and the data display mode are reset.
;         
              'RESE': begin
                                     
                       r_mode = strmid(uvalue,6,3)
;
;  reset only LUT
;
                       if r_mode eq 'LUT' or r_mode eq 'ALL' then begin
                          if tp_obs.raster_p.detector eq 'G' then begin
                             loadct,0
                          endif else begin
                             tp_coltable
                             tp_colbar
                          endelse
                          if r_mode eq 'LUT' then goto,tidy_up
                       endif

;
;  reset flag window definitions only
;
                       if r_mode eq 'FLA' or r_mode eq 'ALL' then begin
                          if flag_state eq 1 then begin
                             tp_obs.line_w.flag = 0
                             line_window.flag = 0
                          endif else begin
                             n = where(line_window.flag eq 1,count)
                             if count gt 0 then begin
                                for i=0,n_elements(n)-1 do begin
                                   if tp_obs.raster_p.detector eq 'N' then begin
                                      erase_nis_win,n(i)
                                   endif
                                   del_window,n(i)
                                endfor
                             endif
                          endelse
                          sort_windows, line_window, num_windows
                          if tp_obs.raster_p.detector eq 'N' then begin
                            display_nis_win 
                          endif else begin
                            tp_plot_gis,GIS_data,GIS_band
                            display_gis_win
                          endelse
                          if r_mode eq 'FLA' then goto,tidy_up
                       endif


;
;  reset only detector parameters
;
                       if r_mode eq 'DET' or r_mode eq 'ALL' then begin
                          tp_clear_state
                          if r_mode eq 'DET' then goto,tidy_up
                       endif

;
;  reset line windows only
;
                       if r_mode eq 'LWO' or r_mode eq 'ALL' then begin

                          widget_control,wll_info_2,set_value=''
                          widget_control,wdw_info_2,set_value=''
                          window_mode = 'ALL'

                          if flag_state eq 1 then begin
                             if tp_obs.raster_p.detector eq 'N' then begin
                                erase_nis_win
                                del_window
                             endif else begin
                                del_window
                                tp_plot_gis,GIS_data,GIS_band
                             endelse 
                          endif else begin
                             for i=0,num_windows-1 do begin
                                if line_window(i).flag eq 0 then begin
                                   if tp_obs.raster_p.detector eq 'N' then begin 
                                      erase_nis_win,i
                                   endif
                                   del_window,i
                                endif
                             endfor
                             if tp_obs.raster_p.detector eq 'G' then begin 
                                tp_plot_gis,GIS_data,GIS_band
                                display_gis_win
                             endif
                          endelse
                          sort_windows, line_window, num_windows

                          curr_lwidth = 11
                          curr_ww_max = 100.
                          widget_control,ww_info,set_value=curr_lwidth
                          scale_val = $
                               round((float(curr_lwidth)*1000L/curr_ww_max))
                          widget_control,ww_slider_id,set_val=scale_val

                          curr_lheight = 240
                          widget_control,wh_slider_id,set_val=curr_lheight

                          tp_clear_lines
                          tp_recalc_dew,' '

                          if r_mode eq 'LWO' then begin
                            if (tp_obs.raster_v.comp_id gt 3) and $
                                  (tp_obs.raster_p.detector eq 'N') then begin
                              tp_get_dumdata, tp_obs, VDS_image_data,$
                                              zone=current_zone
                              tp_redisplay
                            endif
                            goto,tidy_up
                          endif
                       endif

;
;  must want all the rest (ie the raster details)
;

                       current_zone = -4
                       tp_clear_raster
                       tp_obs.raster_v.comp_id=1 
                       tp_obs.raster_v.comp_opt=0 
                       load_tp_struct, tp_obs


                      end
;
;  NDAT is intended to lead into the dummy data calculation or it could
;       eventually give a choice of real data files to load.  Note that the 
;       zone ID number is zone+100 if data for that don't exist at present.
;       Note also that the order of the IDs used in mk_raster reflects the
;       order of the columns in the input data for vds/gis_dummy rather than
;       their definition in the data base etc.
;
             'NDAT': begin
                        old_zone = current_zone
                        mode = strmid(uvalue,6,4)
                        case mode of
                          'QNIS': current_zone = -4
                          'ANIS': current_zone = -5
                          'CORO': current_zone = 101
                          'LIMB': current_zone = 102
                          'CHOL': current_zone = 103
                          'QUIE': current_zone = 4
                          'ACTI': current_zone = 5
                          'FLAR': current_zone = 106
                          'N1FI': current_zone = 7
                          'P1FI': current_zone = 8
                          'N2FI': current_zone = 9
                          'P2FI': current_zone = 10
                            else: begin
                                     flash_msg,info_line,/append,$
                                        'Sorry, not yet available.'
                                  end
                        endcase

;
;  check for real data requests
;
                        if current_zone eq -4 then begin
                           if tp_obs.raster_p.detector ne 'N' then begin
                              flash_msg,info_line,/append,'Detector must be NIS.'
                              current_zone = old_zone
                              return
                           endif else begin
                              rfile = concat_dir('$CDS_PLAN_TECH','real_qnis_data')
                              if not file_exist(rfile) then begin
                                 flash_msg,info_line,/append,$
                                   'Data file does not exist. '+rfile
                                 current_zone = old_zone
                                 return
                              endif
                                  
                              tp_get_dumdata,tp_obs,VDS_image_data,$
                                             zone=current_zone

                              tv,tp_bscale(VDS_image_data),0,0
                              tp_colbar
                              display_nis_win
                              tp_vds_axes
                              current_zone = -4
                           endelse
                           return
                        endif
;
                        if current_zone eq -5 then begin
                           if tp_obs.raster_p.detector ne 'N' then begin
                              flash_msg,info_line,/append,'Detector must be NIS.'
                              current_zone = old_zone
                              return
                           endif else begin
                              rfile = concat_dir('$CDS_PLAN_TECH','real_anis_data')
                              if not file_exist(rfile) then begin
                                 flash_msg,info_line,/append,$
                                   'Data file does not exist. '+rfile
                                 current_zone = old_zone
                                 return
                              endif
                                  
                              tp_get_dumdata,tp_obs,VDS_image_data,$
                                             zone=current_zone

                              tv,tp_bscale(VDS_image_data),0,0
                              tp_colbar
                              display_nis_win
                              tp_vds_axes
                              current_zone = -5
                           endelse
                           return
                        endif
;
;
;  check for GIS specials
;

                        if (tp_obs.raster_p.detector ne 'G') and $
                           (current_zone eq 7 or current_zone eq 8 or $
                            current_zone eq 9 or current_zone eq 10) then begin
                           flash_msg,info_line,/append,$
                                    'Those are only for GIS.'
                                     current_zone = old_zone
                        endif
;
;  warn of interesting but not available cases.
;
                        if current_zone gt 100 then begin
                           flash_msg,info_line,/append,$
                             'Sorry, not yet available - using QUIET sun data.'
                        endif

                        if old_zone ne current_zone then begin
                           if tp_obs.raster_p.slit_num gt 0 then begin
                              widget_control,info_line,/append,$
                                      set_value='Calculating new data.....'
                              widget_control,base,/hourglass
                    
                              case tp_obs.raster_p.detector of
                                'N': begin
                                       tp_get_dumdata,tp_obs,VDS_image_data,$
                                                      zone=current_zone
                                       tv,tp_bscale(VDS_image_data),0,0
                                       tp_colbar
                                       display_nis_win
                                       tp_vds_axes
                                     end
                                'G': begin
                                       tp_get_dumdata,tp_obs,GIS_data,$
                                                      zone=current_zone
                                       tp_plot_gis,GIS_data,GIS_band
                                       display_gis_win
                                     end
                               else: flash_msg,info_line,$
                                                    'No instrument defined.'
                              endcase
                           endif else begin
                            widget_control,info_line,/append, $
                                               set_val='No slit selected.'
                           endelse
                        endif
                     end

;
;  SLIT chooses the slit to be used. The selected slit is kept as a string
;       identification from the list in slit_list
;
             'SLIT': begin
                      old_slit = tp_obs.raster_p.slit_num 
                      slit_num = fix(strmid(uvalue,4,1))
                      tp_obs.raster_p.slit_num = slit_num
                      if old_slit ne slit_num then begin
                         widget_control,slit_info,$
                             set_val=string('#',slit_num,' - ',$
                             form='(a1,i1,a3)')+value
                         widget_control,base,/hourglass
                         case tp_obs.raster_p.detector of
                            'N': begin
                                   tp_get_dumdata, tp_obs, VDS_image_data,$
                                                   zone = current_zone
                                   tp_redisplay
                                 end
                            'G': begin
                                   tp_get_dumdata, tp_obs, GIS_data,$
                                                   zone=current_zone
                                   tp_plot_gis,GIS_data,GIS_band
                                   display_gis_win
                                 end
                           else:
                         endcase
                         tp_obs.raster_p.ras_id = 0
                         tp_obs.raster_v.ras_id = 0
                         tp_obs.raster_v.ras_var = 0
                      endif
                     end

;
;  VDS_ sets up the VDS-specific raster details
;
             'VDS_': begin
;
;  now select which VDS parameter it was
;
                       case strmid(uvalue,4,4) of
;
;  Orientation - extract by row or column
;
                          'OR_0': begin
                                   if tp_obs.raster_v.vds_orient ne 0 then begin 
                                      tp_obs.raster_v.vds_orient = 0
                                      tp_obs.raster_v.ras_var = 0
                                      tp_get_dumdata, tp_obs, VDS_image_data,$
                                                      zone=current_zone
                                      tp_redisplay
                                   endif
                                  end

                          'OR_1': begin
                                   if tp_obs.raster_v.vds_orient ne 1 then begin 
                                      tp_obs.raster_v.vds_orient = 1
                                      tp_obs.raster_v.ras_var = 0
                                      tp_get_dumdata, tp_obs, VDS_image_data,$
                                                      zone=current_zone
                                      tp_redisplay
                                   endif
                                  end

;
;  Mapping ie data accumulation (normal=none or accumulate)
;
                          'MAP2': begin
                                    if tp_obs.raster_v.vds_map ne 2 then begin
                                      tp_obs.raster_v.vds_map = 2
                                      tp_obs.raster_v.ras_var = 0
                                    endif
                                  end

                          'MAP3': begin
                                    if tp_obs.raster_v.vds_map ne 3 then begin
                                      tp_obs.raster_v.vds_map = 3
                                      tp_obs.raster_v.ras_var = 0
                                    endif
                                  end

;
;  Plot data from windows in line format
;
                          'PWIN': begin
                                    if not xregistered('tp_plot_vds') then begin
                                       sort_windows,line_window,num_windows
                                       tp_plot_vds
                                       widget_control, plot_vds_base, map=1
                                       xmanager, 'tp_plot_vds',plot_vds_base,$
                                              event_handler='tp_plot_vds_event'
                                       for i=0,npb do begin
                                          widget_control,parmbase(i),map=0
                                       endfor
                                       widget_control,parmbase(1),map=1
                                       tp_draw_vwin, tp_obs
                                    endif
                                  end
;
;  Display data extraction windows
;

                          'DEWS': begin
                                    if not xregistered('tp_display_dew') then begin
                                       tp_display_dew
                                       widget_control, dew_base, map=1
                                       xmanager, 'tp_display_dew',dew_base,$
                                              event_handler='tp_d_dew_event'
                                       for i=0,npb do begin
                                          widget_control,parmbase(i),map=0
                                       endfor
                                       widget_control,parmbase(1),map=1
                                       tp_plot_dew, tp_obs
                                     endif
                                  end

;
;  Look at parameters
;
                          'SEEV': begin
                                    if not xregistered('tp_display_vp') then begin
                                       tp_display_vp
                                       widget_control, vp_base, map=1
                                       xmanager, 'tp_display_vp',vp_base,$
                                              event_handler='tp_d_vp_event'
                                       for i=0,npb do begin
                                          widget_control,parmbase(i),map=0
                                       endfor
                                       widget_control,parmbase(1),map=1
                                     endif
                                  end

                           else:
                       endcase
                       if xregistered('tp_display_vp') then begin
                          words = ['Row','Column']
                          widget_control,vp_widg(0),$
                                       set_v=words(tp_obs.raster_v.vds_orient)
                          words = [' ',' ','Normal','Accumulate']
                          widget_control,vp_widg(1),$
                                       set_v=words(tp_obs.raster_v.vds_map)
                          words = ['No','Yes']
                          s = tp_obs.vds_backwin(3).win_def(0)
                          text = words(tp_obs.data_win.vds_back)
                          if text eq 'Yes' then begin
                             text = text + ' - ' + fmt_vect([s,s])
                          endif
                          widget_control,vp_widg(4),set_v=text
                       endif
                     end

;
;  GIS_ sets up the GIS-specific details
;
             'GIS_': begin
;
;  now select which GIS parameter it was
;
                       case strmid(uvalue,4,4) of
                          'DEWS': begin
                                    tp_display_dew
                                    widget_control, dew_base, map=1
                                    xmanager, 'tp_display_dew',dew_base,$
                                           event_handler='tp_d_dew_event'
                                    for i=0,npb do begin
                                       widget_control,parmbase(i),map=0
                                    endfor
                                    widget_control,parmbase(3),map=1
                                    tp_plot_dew, tp_obs
                                  end
                          else:
                       endcase      
                     end


;
;  NDEFEX chooses VDS default extraction
;
             'NDEF': begin
                        nisband = fix(strmid(uvalue,7,1))
                        nis_def_ext(nisband-1) = ev.select
                        if nis_def_ext(2) and $
                              (nisband lt 3) and $
                                 ev.select then begin
                           nis_def_ext(2) = 0
                           widget_control,ex_ids(2),set_button=0
                        endif
                        if nisband eq 3 and ev.select then begin
                           nis_def_ext = [0,0,1]
                           widget_control,ex_ids(0),set_button=0
                           widget_control,ex_ids(1),set_button=0
                           widget_control,ex_ids(2),set_button=1
;
;  if all CCD then turn off background windows
;
                           vds_extension = 0
                           tp_obs.data_win.vds_back = 0
                           for i=0,2 do widget_control,vdsb_ids(i),set_b=0
                           widget_control,vdsb_ids(1),set_b=1
                         endif
                     end
                        
;
;  EXTEN chooses VDS background window extension either ON or OFF
;
             'EXTE': begin
                        extension = strmid(uvalue,5,3)
                        case extension of
                         'NO': begin
                                 vds_extension = 0 
                                 tp_obs.data_win.vds_back = 0
                               end
                        'YES': begin
                                 vds_extension = 1
                                 tp_obs.data_win.vds_back = 1
                               end
                        'RES': begin
                                 for i=0,2 do begin
                                    widget_control,gis_dispbase(i),map=0
                                 endfor
                                 widget_control, vdsb_slider_id,$
                                       set_val=tp_obs.vds_backwin(3).win_def(0) 
                                 widget_control,gis_dispbase(3),map=1
                               end
                         else:
                        endcase

                        if xregistered('tp_display_dew') then begin
                           tp_recalc_dew,' '
                           tp_plot_dew, tp_obs
                        endif
                        if xregistered('tp_display_vp') then begin
                           words = ['No','Yes']
                           s = tp_obs.vds_backwin(3).win_def(0) 
                           text = words(tp_obs.data_win.vds_back)
                           if text eq 'Yes' then begin
                              text = text + ' - ' + fmt_vect([s,s])
                           endif
                           widget_control,vp_widg(4),set_v=text
                        endif
                        tp_obs.raster_v.ras_var = 0
                        tp_obs.raster_v.dw_id   = 0
                        tp_obs.data_win.dw_id   = 0
                        tp_obs.data_win2.dw_id  = 0
                     end

;
;  GDEFEX chooses default extraction mode for GIS data 
;
             'GDEF': begin
                        gisband = fix(strmid(uvalue,7,1))
                        gis_def_ext(gisband-1) = ev.select
                        widget_control,gbdef_ids(gisband-1),set_but=ev.select
                     end

;
;  INST selects the instrument to be used. This is limited to either the
;       normal or grazing incidence spectrograph. When this is selected 
;       display the appropriate data.
;
;
             'INST': begin
;
;  clear any common bases that actually have instrument specific contents
;
                      if xregistered('tp_disp_llist') then begin
                         widget_control,ll_base,/destroy
                      endif
                      if xregistered('tp_disp_rast') then begin
                         widget_control,fr_base,/destroy
                      endif

                      tp_obs.raster_p.detector = inst_list(fix(strmid(uvalue,4,1))-1)
                      widget_control,ccs_info_2,set_value=''
                      tp_obs.raster_v.comp_id = 1
                      tp_obs.raster_v.comp_opt = 0
                      tp_obs.raster_v.tel_rate = 'M'
                      widget_control,tmr_info_2,set_value='MEDIUM'

;
;  if changing instruments then clear line list and line width/height
;
                      if tp_obs.raster_p.detector ne previous_inst then begin
                         widget_control,wll_info_2,set_value=''
                         widget_control,wdw_info_2,set_value=''
                         widget_control,wrp_info_2,set_value=''
                         widget_control,wrv_info_2,set_value=''
                         del_window
                         curr_lwidth = 11
                         curr_ww_max = 100.
                         widget_control,ww_info,set_value=curr_lwidth
                         scale_val = $
                              round((float(curr_lwidth)*1000L/curr_ww_max))
                         widget_control,ww_slider_id,set_val=scale_val
                   
                         curr_lheight = 240
                         widget_control,wh_slider_id,set_val=curr_lheight

;
;  reset observation structure but reinstate instrument, slit,
;  default exposure time VDS background windows and state.
;               
                         previous_et   = tp_obs.raster_v.exptime
                         previous_inst = tp_obs.raster_p.detector
                         previous_slit = tp_obs.raster_p.slit_num
                         previous_vdsback = tp_obs.vds_backwin.win_def
                         previous_state = tp_obs.state
                         tp_obs = {tp_obs_rec}
                         tp_obs.raster_p.detector = previous_inst
                         tp_obs.raster_p.slit_num = previous_slit
                         tp_obs.raster_v.exptime  = previous_et
                         tp_obs.vds_backwin.win_def  = previous_vdsback 
                         tp_obs = rep_tag_value(tp_obs,previous_state,'STATE')
                         tp_obs.raster_v.comp_id = 1
                         tp_obs.raster_v.comp_opt = 0
                         widget_control,ccs_info_2,set_value='Straight copy'
                         tp_obs.raster_v.tel_rate = 'M'
                         widget_control,tmr_info_2,set_value='MEDIUM'
                         current_zone = -4
                      endif
                      case tp_obs.raster_p.detector of
                          'N': begin
                                 if xregistered('tp_display_gp') then begin
                                    widget_control,gp_base,/destroy
                                 endif
                                 if xregistered('tp_display_dew') then begin
                                    widget_control,dew_base,/destroy
                                 endif
                                 widget_control,image_area,sensitive=1
                                 widget_control, sssy, set_v=' n/a'
                                 widget_control, sssy, sensitive=0
                                 widget_control, nsy, set_v=' n/a'
                                 widget_control, nsy, sensitive=0
                                 if tp_obs.raster_p.slit_num eq 0 then begin
                                    tp_obs.raster_p.slit_num = 4
                                    widget_control,slit_info,$
                                        set_val='#4 - (2 x 240)'
                                 endif
                                 widget_control,base,/hourglass
                                 tp_get_dumdata, tp_obs, VDS_image_data,$
                                                      zone=current_zone
                                 tp_colbar
                                 widget_control,ccs_info_2,$
                                    set_value='Truncate to 12 bits'
                                 tp_obs.raster_v.tel_rate = 'M'
                                 widget_control,tmr_info_2,set_value='MEDIUM'
                                 tp_obs.raster_v.comp_id = 2
                                 tp_obs.raster_v.comp_opt = 0

;
; set sensible defaults for VDS
;
                                 tp_obs.raster_p.nx = 1
                                 tp_obs.raster_p.ny = 1
                                 tp_obs.raster_v.vds_orient = 0
                                 tp_obs.raster_v.vds_map = 2
                                 tp_obs.state.vds_read = 1
                                 tp_obs.data_win.vds_back = 1
                                 vds_extension = 1
                                 for i=0,2 do widget_control,vdsb_ids(i),set_b=0
                                 widget_control,vdsb_ids(1),set_b=1
                                 tp_obs.line_w.x_pos = -1
                                 nis_def_ext = [0,0,0]
                                 for i=0,2 do widget_control,ex_ids(i),set_b=0

                                 for i=0,npb do widget_control,parmbase(i),map=0
                                 widget_control,parmbase(1),map=1     
                               end
                          'G': begin
                                 if xregistered('tp_display_vp') then begin
                                    widget_control,vp_base,/destroy
                                 endif
                                 if xregistered('tp_display_dew') then begin
                                    widget_control,dew_base,/destroy
                                 endif
                                 loadct,0
                                 widget_control,image_area,sensitive=1
                                 widget_control, sssy, $
                                          set_v=string(tp_obs.raster_p.ystep)
                                 widget_control, sssy, sensitive=1
                                 widget_control, nsy, $
                                          set_v=string(tp_obs.raster_p.ny)
                                 widget_control, nsy, sensitive=1
                                 if tp_obs.raster_p.slit_num eq 0 then begin
                                    tp_obs.raster_p.slit_num = 1
                                    flash_msg,info_line,num=2,/append,$
                                      'No slit defined. Using default...',/nobe
                                    widget_control,slit_info,$
                                      set_val='#1 - (2 x 2)'
                                 endif
                                 widget_control,base,/hourglass
                                 current_zone = 4
                                 tp_get_dumdata, tp_obs, GIS_data,$
                                                 zone=current_zone


                                 tp_obs.raster_p.nx = 1
                                 tp_obs.raster_p.ny = 1
                                 tp_obs.line_w.x_pos = -1

                                 for i=0,3 do widget_control,gbdef_ids(i),$
                                              set_button=1
                                 gis_def_ext = [1,1,1,1]


                                 for i=0,npb do widget_control,parmbase(i),map=0
                                 widget_control,parmbase(3),map=1
                               end
                         else: flash_msg,info_line,'Unknown instrument.',/append
                      endcase

;
;  redisplay cleared out, defaulted structure
;
                      load_tp_struct, tp_obs
                      tp_colbar
                      tp_recalc_dew,' '
                     end

;
;   CHECK allows user to check whether a definition exists before they
;   proceed with whatever else they intend to do
;
             'CHEC': begin
                        flash_msg, info_line, /append, $
                                   'Checking not yet implemented.'
                     end

;
;  Calculate raster duration in detail
;
             'RAS_': begin
                        if not xregistered('tp_rasdur') then begin
                           tp_rasdur
                           widget_control, plot_rasdur_base, map=1
                           xmanager, 'tp_rasdur',plot_rasdur_base,$
                                      event_handler='tp_rasdur_event'
                           tp_draw_rasdur
                        endif
                     end


;
;   USER refers to the button to switch between USER and CDS project databases
;        it is a toggle switch
;
             'USER': begin
                       if db_in_use eq 'User' then begin
                          dbenv = find_all_dir('ZDBASE_CDS')
                          errmsg = ''
                          s = fix_zdbase(/cds, errmsg=errmsg)
                          if errmsg ne '' then begin
                             flash_msg, info_line, errmsg, /append
                          endif else begin
                             db_in_use = 'CDS'
                             widget_control, db_switch, $
                                             set_val='Using CDS databases'
                             widget_control, info_line, /append, set_val=$
                                 'Data base directory now: '+dbenv(0)
                             for i=1,n_elements(dbenv)-1 do begin
                                widget_control, info_line, /append, set_val=$
                                 '                         '+dbenv(i)
                             endfor
                          endelse
                       endif else begin
                          dbenv = find_all_dir('ZDBASE_USER')
                          errmsg = ''
                          s = fix_zdbase(/user,errmsg=errmsg)
                          if errmsg ne '' then begin
                             flash_msg, info_line, errmsg, /append
                          endif else begin
                             db_in_use = 'User'
                             widget_control, db_switch, $
                                             set_val='Using USER databases'
                             widget_control, info_line, /append, set_val=$
                                 'Data base directory now: '+dbenv(0)
                             for i=1,n_elements(dbenv)-1 do begin
                                widget_control, info_line, /append, set_val=$
                                 '                         '+dbenv(i)
                             endfor
                          endelse
                       endelse
                     end


;
;   WRIT selects which data the user would like to output for storage in the
;        databases.  Note that if all are to be loaded they must be in the
;        order:  Line list, data windows, IEF windows, fundamental raster, 
;        raster Variation so that due account is taken of the interlocking 
;        ID entries.
;
             'WRIT': begin
                      if strmid(uvalue,6,4) ne 'HELP' then begin
                        if not priv_zdbase(/def) then begin
                           xack,'You do not have write privilege to the databases.',$
                                /modal, xoff=500,yoff=500,/icon
                           key_code_ok = 0
                           !priv=0
                        endif else begin
                           key_code_ok = 1
                           !priv=2
                        endelse
                      endif else begin
                       widg_help, 'tp_wrt_db.hlp', /modal,$
                                  title='Data base access help',group=base
                      endelse
;
;  now select the latter part of the uvalue
;
                      if key_code_ok then begin
                        case strmid(uvalue,6,3) of
                        'ALL': begin
                                 widget_control, wrp_info_2,get_val=text1
                                 widget_control, wrv_info_2,get_val=text2
                                 widget_control, wdw_info_2,get_val=text3
                                 widget_control, wll_info_2,get_val=text4
                                 text1 = text1(0) & text2 = text2(0)
                                 text3 = text3(0) & text4 = text4(0)
                                 if tp_obs.line_list.n_lines eq 0 and $
                                                         text4 eq '' then begin
                                    text4 = 'No spectral lines'
                                    widget_control, wll_info_2,set_val=text4
                                 endif
                                 if text1 eq '' or $
                                       text2 eq '' or $
                                          text3 eq '' or $
                                             text4 eq '' then begin
                                    xack,/modal,/icon,xoff=500,yoff=500,$
                                       'Descriptions incomplete - cannot save.'
                                 endif else begin
;
;  Line list
;
                                    stat1 = tp_wrt_llist(tp_obs, info_line,$
                                                         flag_state, base)
                                    widget_control, wll_info_2,$
                                         set_v=tp_obs.line_list.ll_desc
                                    if stat1 lt 0 then begin
                                       widget_control,info_line,/append,$
                                          set_v='Database entry for LINE LIST not written.'
                                       goto, tidy_up
                                    endif
;
;  Data windows
;
                                    stat2 = tp_wrt_dwin(tp_obs, info_line, flag_state)
                                    if stat2 lt 0 then begin
                                       flash_msg,info_line,'Database entry for'+$
                                               ' DATA WINDOWS not written.',/append
                                       goto, tidy_up
                                    endif
;
;  IEF details
;
                                    stat25 = tp_wrt_ief(tp_obs, info_line)
                                    if stat25 lt 0 then begin
                                       flash_msg,info_line,'IEF entry for'+$
                                               ' FLAG WINDOWS not written.',/append
                                       goto, tidy_up
                                    endif
;
;  Fundamental rasters
;
                                    stat3 = tp_wrt_frast(tp_obs, info_line)
                                    if stat3 lt 0 then begin
                                       flash_msg,info_line,'Database entry for'+$
                                          ' FUNDAMENTAL RASTER not written.',/append
                                       goto, tidy_up
                                    endif
;
;  Raster Variation
;                                   
                                    if tp_obs.raster_v.comp_id gt 0 then begin
                                       tp_get_desc,tp_obs,current_zone,$
                                                   flag_state,curr_lheight
                                       stat4 = tp_wrt_vrast(tp_obs, info_line)
                                       if stat4 lt 0 then begin
                                          flash_msg,info_line,'Database entry for'+$
                                                ' Raster Variation not written.',/append
                                          goto, tidy_up
                                       endif
                                    endif else begin
                                       flash_msg,info_line,'No compression scheme -'+$
                                             ' Raster Variation not written.',/append
                                    endelse
                                 endelse
                               end
                        'RAF': begin
                                 widget_control, wrp_info_2,get_val=text1
                                 if text1(0) eq '' then begin
                                    flash_msg, info_line, /append,$
                                       'No fundamental raster description. '+$
                                        'Cannot store in database.'
                                 endif else begin
                                    stat = tp_wrt_frast(tp_obs, info_line)
                                    if stat lt 0 then begin
                                       flash_msg,info_line,'Database entry for'+$
                                            ' FUNDAMENTAL RASTER not written.',/append
                                    endif
                                 endelse
                               end
                        'RAV': begin
                                 widget_control, wrv_info_2,get_val=text1
                                 if text1(0) eq '' then begin
                                    flash_msg, info_line, /append,$
                                       'No raster variation description. '+$
                                        'Cannot store in database.'
                                 endif else begin
                                    stat = tp_wrt_vrast(tp_obs, info_line)
                                    if stat lt 0 then begin
                                       flash_msg,info_line,'Database entry for'+$
                                          ' Raster Variation not written.',/append
                                    endif
                                 endelse
                               end
                        'LLO': begin
                                 widget_control, wll_info_2,get_val=text1
                                 if text1(0) eq '' then begin
                                    flash_msg, info_line, /append, $
                                       'No line list description. '+$
                                        'Cannot store in database.'
                                 endif else begin
                                    stat = tp_wrt_llist(tp_obs, info_line, $
                                                        flag_state, base)
                                    widget_control, wll_info_2,$
                                         set_v=tp_obs.line_list.ll_desc
                                    if stat lt 0 then begin
                                       widget_control,info_line,/append,$
                                          set_val='Database entry for LINE LIST not written.'
                                    endif
                                 endelse
                               end
                        'DWO': begin
                                 widget_control, wdw_info_2,get_val=text1
                                 if text1(0) eq '' then begin
                                    flash_msg, info_line, /append, $
                                       'No data windows description. '+$
                                        'Cannot store in database.'
                                 endif else begin
                                    stat = tp_wrt_dwin(tp_obs, info_line, flag_state)
                                    if stat lt 0 then begin
                                       flash_msg,info_line,'Database entry for'+$
                                           ' DATA WINDOWS not written.',/append
                                    endif
                                 endelse
                               end
                        'FWO': begin
                                  stat = tp_wrt_ief(tp_obs, info_line, base)
                                  if stat lt 0 then begin
                                     flash_msg,info_line,'IEF entry for'+$
                                           ' FLAG WINDOWS not written.',/append
                                  endif
                               end
                           else:
                        endcase
                      endif
                     end 
;
;   
;COMP selects the type of compression to be applied to the telemetry data.
;        The selection is contained in the file COMPRESSION_SCHEME.PDM 
;        for loading into a pull-down menu.
;
             'COMP': begin
;
;  check special cases
;
                      if fix(strmid(uvalue,4,1)) eq 2 and $
                                   tp_obs.raster_p.detector eq 'G' then begin
                         flash_msg,info_line,/append,$
                              'Truncation scheme not allowed with GIS data.'
                         return
                      endif

                      if fix(strmid(uvalue,4,1)) eq 3 then begin  
                         flash_msg,info_line,/append,$
                           'VBWL requires VDS column extraction. This has been set.'
                         tp_obs.raster_v.vds_orient = 1
                      endif  else begin
                         tp_obs.raster_v.vds_orient = 0
                      endelse
                      if xregistered('tp_display_vp') then begin
                         words = ['Row','Column']
                         widget_control,vp_widg(0),$
                                      set_v=words(tp_obs.raster_v.vds_orient)
                      endif

                      last_comp = tp_obs.raster_v.comp_id 
                      widget_control,ccs_info_2,get_value=temp
                      widget_control,ccs_info_2,set_value=value
                      tp_obs.raster_v.comp_id = fix(strmid(uvalue,4,1))
                      tp_obs.raster_v.comp_opt = fix(strmid(uvalue,5,1))

;
;  set number of rows in User sum over rows mode
;
                      if tp_obs.raster_v.comp_id eq 8 then begin
                         tp_obs.raster_v.comp_opt = (fix(strmid(uvalue,5,1)) +1) * 2
                         tp_obs.raster_v.vds_orient = 0
                      endif

;
;  sort out the sum over wavelength/spatial option
;
                      if fix(strmid(uvalue,4,1)) eq 6 then begin
                        tp_obs.raster_v.comp_opt = 0
                        if fix(strmid(uvalue,5,1)) eq 1 then begin  
                          flash_msg,info_line,/append,$
                            'Requires VDS column extraction. This has been set.'
                          tp_obs.raster_v.vds_orient = 1
                        endif  else begin
                          flash_msg,info_line,/append,$
                            'Requires VDS row extraction. This has been set.'
                          tp_obs.raster_v.vds_orient = 0
                        endelse
                      endif

                      if tp_obs.raster_p.detector eq 'N' then begin
                         tp_get_dumdata, tp_obs, VDS_image_data,$
                                                 zone=current_zone
                         tp_redisplay
                      endif
                     end


;
;   TRATE selects the telemetry rate 
;
             'TRAT': begin
                       case fix(strmid(uvalue,5,1)) of
                         0: begin
                              tp_obs.raster_v.tel_rate = 'P'
                              widget_control,tmr_info_2,set_value='LOW++'
                            end
                         1: begin
                              tp_obs.raster_v.tel_rate = 'L'
                              widget_control,tmr_info_2,set_value='LOW'
                            end
                         2: begin
                              tp_obs.raster_v.tel_rate = 'M'
                              widget_control,tmr_info_2,set_value='MEDIUM'
                            end
                         3: begin
                              tp_obs.raster_v.tel_rate = 'H'
                              widget_control,tmr_info_2,set_value='HIGH'
                            end
                      else: begin
                              tp_obs.raster_v.tel_rate = 'M'
                              widget_control,tmr_info_2,set_value='MEDIUM'
                            end
                        endcase
                     end


;
;   EDIT  uses the table editing compound widget to display and edit the 
;         line window structure. The redisplay is necessary because the edit
;         widget has to turn off all other widgets and this actually erases
;         the draw widget. Maybe there is a better way.
;
             'EDIT': begin
                      widget_control, base,sensitive=0
                      tp_edit_lines
                     end
;
;   GISB  indicates which GIS band to display.  Choice is 1-4 or 5 (All).
;
             'GISB': begin
                      GIS_band = strmid(uvalue,4,1) 
                      if tp_obs.raster_p.detector eq 'G' then begin
                         if window_mode eq 'ALL' then begin
                            tp_plot_gis,GIS_data,GIS_band
                         endif else begin
                            erase
                         endelse
                         tp_redisplay
                      endif
                     end
;
;   ERAN  stands for Exposure RANge and allows the user to select the
;         range of the exposure time slider widget.  The ranges are
;         (0-1), (0-10), (0-100) and (0-1000) seconds. The 
;         accuracy of the readout is adjusted according to the range.
;
;  NB The displayed data are rescaled rather than recalculated - thus 
;     assuming that the data do scale with exposure time.  If this is not
;     true they should be recalculated each time. Made it recalculate GIS data
;     everytime because of 16 bit overflow
; 
             'ERAN': begin
                      range = strmid(uvalue,6,1)
                      if range eq '5' and $
                             tp_obs.raster_p.detector eq 'N' then begin
                         bell
                         widget_control,info_line,/append,set_val=$
                                  'NIS exposures limited to 1000 seconds.'
                      endif else begin
                         last_et_max = curr_et_max
                         curr_et_max = et_maxima(range-1)
                         curr_etime = string(curr_etime*(curr_et_max/last_et_max))
                         i = fix(alog10(curr_et_max))
                         val = float(curr_etime)
                         acc = curr_et_max/100.
                         val = round_off(val,acc)
                         stat = execute('curr_etime = string(val,form=et_form(i))')
                         widget_control,et_info,set_value=curr_etime
                         if (curr_et_max ne last_et_max) then begin
                            widget_control,base,/hourglass
                            eratio = float(curr_etime)/tp_obs.raster_v.exptime
                            tp_obs.raster_v.exptime = float(curr_etime)
                            case tp_obs.raster_p.detector of
                              'N': begin
;                                    VDS_image_data = VDS_image_data * eratio
;for non-linear                     tp_get_dumdata,tp_obs,VDS_image_data
                                    tv,tp_bscale(VDS_image_data),0,0
                                    display_nis_win
                                    tp_colbar
                                    tp_vds_axes
                                   end
                              'G': begin
                                    tp_get_dumdata,tp_obs,GIS_data,$
                                                   zone=current_zone
                                    tp_plot_gis,GIS_data,GIS_band
                                    display_gis_win
                                   end
                              else: 
                            endcase
                            tp_obs.raster_v.ras_var = 0
                         endif 
                      endelse
                     end
;
;   WWRA  stands for Window Width RAnge and allows the user to select the
;         range of the window width slider widget.  The ranges are
;         (0-100) and (0-1000) pixels.  This is applied to all windows unless
;         the window is a flag window AND flag windows are not included in
;         with the dex windows.
; 
             'WWRA': begin
                      range = strmid(uvalue,7,1)
                      last_ww_max = curr_ww_max
                      curr_ww_max = et_maxima(range-1)
                      curr_lwidth = $
                                  fix(curr_lwidth*curr_ww_max/last_ww_max) > 3

                      if tp_obs.raster_p.detector eq 'G' and $
                               curr_lwidth eq 1000 then begin
                         curr_lwidth = 2048
                      endif

                      widget_control,ww_info,set_value=curr_lwidth
                      if curr_ww_max ne last_ww_max then begin

                        for i=0,num_windows-1 do begin
                          if tp_obs.raster_p.detector eq 'N' then erase_nis_win,i 
                          if (flag_state eq 1) or $
                             ((flag_state + line_window(i).flag) eq 0) then begin
                             line_window(i).left  = $
                                      line_window(i).x_pos - (curr_lwidth/2)
                             line_window(i).right = $
                                      line_window(i).left + curr_lwidth - 1
                          endif
                          if tp_obs.raster_p.detector eq 'N' then begin
                             display_nis_win,i 
                          endif 
                        endfor 
                        if tp_obs.raster_p.detector eq 'G' then begin
                         if window_mode eq 'ALL' then begin
                            tp_plot_gis,GIS_data,GIS_band
                         endif
                         display_gis_win
                        endif
                        tp_obs.raster_v.ras_var = 0
                        tp_obs.raster_v.dw_id   = 0
                        tp_obs.data_win.dw_id   = 0
                        tp_obs.data_win2.dw_id  = 0

                      endif
                     end
;
;   LINE  refers to all input dealing with user manipulation of the line
;         windows.  The variable window_state reflects which button in the
;         window manipulation box was chosen and this sets the current mode.  
;         The actual operation on a window only takes place when the cursor 
;         activates the DRAW widget. 
;
             'LINE': window_state = strmid(uvalue,5,3)

;
;   XLOA calls up the general LUT adjusting procedure XLOADCT
;
             'XLOA': begin
                        xloadct
                        tp_colbar
                     end

;
;   DISP changes the display state. At present this can only be ALL to display
;        the whole dataset or WON to display only data within the windows or
;        FON to display data within the Flag windows.
;
             'DISP': begin
                      display_state = strmid(uvalue,5,3)
                      tp_redisplay
                     end
;
;   FLAG changes the flag mode.  Either flags are included in the general 
;   data extraction windows or they are kept separate. 
;
             'FLAG': begin
                        flag_state = fix(strmid(uvalue,5,1))
                        if tp_obs.raster_p.detector eq 'N' then begin
                           display_nis_win 
                        endif else begin
                           tp_plot_gis,GIS_data,GIS_band
                           display_gis_win
                        endelse
                     end
              else :
            endcase
           end
;
;   SLIDER is the generic catch for the slider widgets which affect the 
;          exposure time, the line window width and height and the VDS 
;          background window size (as well as other setup parameters).
;

 'SLIDER': begin
            case strmid(uvalue,0,2) of
;
;          ET controls the exposure time.  Four ranges are set as detailed
;             above. The actual slider value is suppressed and the value
;             converted according to the range set.  The converted value is 
;             shown in a separate widget.
;             
             'ET': begin
                    widget_control,et_slider_id,get_value=value,get_uvalue=uvalue
                    i = fix(alog10(curr_et_max))
                    val = (float(value(0))/1000.0)*curr_et_max
                    acc = curr_et_max/100.
                    val = round_off(val,acc)
                    stat = execute('curr_etime = string(val,form=et_form(i))')
                    widget_control,et_info,set_value=curr_etime
;
;  recalculate data for new exposure time
;
                    if not ev.drag then begin
                       eratio = curr_etime/tp_obs.raster_v.exptime
                       tp_obs.raster_v.exptime = curr_etime
                       widget_control,base,/hourglass
                       case tp_obs.raster_p.detector of
                          'N': begin
;                                 VDS_image_data = VDS_image_data * eratio
;                                tp_get_dumdata, tp_obs, VDS_image_data
                                 tv,tp_bscale(VDS_image_data),0,0
                                 display_nis_win
                                 tp_colbar
                                 tp_vds_axes
                               end
                          'G': begin
                                 tp_get_dumdata, tp_obs, GIS_data,$
                                                 zone=current_zone
                                 tp_plot_gis,GIS_data,GIS_band
                                 display_gis_win
                               end
                         else:
                       endcase
                       tp_obs.raster_v.ras_var = 0
                    endif
                   end
;
;          WW acts upon input from the slider assigned to change the window
;             width.  It sets the global window width.  Note that the windows
;             (or complete spectrum for GIS) are only redrawn when the mouse
;             button is released (ev.drag = 0)
; 
             'WW': begin
                    value = fix(value(0)*curr_ww_max/1000.) > 3
                    widget_control,ww_info,set_value=value
                    if not ev.drag then begin
                      if value ne curr_lwidth then begin
;
;  fiddle?
;
if value eq 1000 and tp_obs.raster_p.detector eq 'G' then begin
   value = 2048
   curr_lwidth = 2048
   widget_control,ww_info,set_value=2048
endif

                        for i=0,num_windows-1 do begin
                          if tp_obs.raster_p.detector eq 'N' then erase_nis_win,i 
                          if (flag_state eq 1) or $
                             ((flag_state + line_window(i).flag) eq 0) then begin
                             line_window(i).left  = line_window(i).x_pos - (value/2)
                             line_window(i).right = line_window(i).left + value - 1
                          endif
                          if tp_obs.raster_p.detector eq 'N' then begin
                             if not ev.drag then display_nis_win,i 
                          endif 
                        endfor 
                        curr_lwidth = value
                        if tp_obs.raster_p.detector eq 'G' and $
                                                       not ev.drag then begin
                         if window_mode eq 'ALL' then tp_plot_gis,GIS_data,GIS_band
                         display_gis_win
                        endif
                        tp_obs.raster_v.ras_var = 0
                        tp_obs.raster_v.dw_id   = 0
                        tp_obs.data_win.dw_id   = 0
                        tp_obs.data_win2.dw_id  = 0
                      endif
                    endif
                   end
;
;          WH allows the user to change the height of the window in VDS data.
;          Action of this slider affects all defined lines since there is no
;          facility for having individual window heights.
;
             'WH': begin
                    if not ev.drag then begin
                      if curr_lheight ne value then begin
                        for i=0,num_windows-1 do begin
                          if (flag_state eq 1) or $
                             ((flag_state + line_window(i).flag) eq 0) then begin
;                            if line_window(i).height eq curr_lheight then begin
                               erase_nis_win,i
                               line_window(i).height = value 
                               display_nis_win,i
;                            endif
                          endif
                        endfor 
                        curr_lheight = value
                        tp_obs.raster_v.ras_var = 0
                        tp_obs.raster_v.dw_id   = 0
                        tp_obs.data_win.dw_id   = 0
                        tp_obs.data_win2.dw_id  = 0
                      endif
                    endif
                   end
             else :
            endcase
           end
;
;   TEXT controls the entry of data to the text windows.
;
 'TEXT_CH'  : begin
            case strmid(uvalue,0,4) of
;
;         SSSX controls the setting of the raster step size in the E-W
;              direction as effected by control of the scan mirror.
;
             'SSSX': begin
                      old_value = tp_obs.raster_p.xstep 
                      if valid_num(value(0)) then begin
                       tp_obs.raster_p.xstep = fix(round(fix(value(0)))) 
                       if (tp_obs.raster_p.xstep mod 2) eq 1 then begin
                          flash_msg,info_line,num=3,/append,/nobeep,$
                                'Mirror steps must be in 2 arcsec units'
                          tp_obs.raster_p.xstep = old_value
                          widget_control,sssx,$
                                       set_value=num2str(tp_obs.raster_p.xstep)
                       endif else begin
                          widget_control,sssx,$
                                       set_value=num2str(tp_obs.raster_p.xstep)
                          if tp_obs.raster_p.xstep ne old_value then begin
                             tp_obs.raster_p.ras_id = 0
                             tp_obs.raster_v.ras_id = 0
                             tp_obs.raster_v.ras_var = 0
                          endif
                          widget_control,sssy,/input_focus
                       endelse
                      endif else begin
                       tp_obs.raster_p.xstep = 0
                       flash_msg,sssx,'Bad value',/nobeep
                      endelse
                     end
;
;         SSSY controls the setting of the raster step size in the N-S
;              direction as effected by control of the slit movement.
;
             'SSSY': begin
                      old_value = tp_obs.raster_p.ystep 
                      if valid_num(value(0)) then begin
                       tp_obs.raster_p.ystep = fix(round(fix(value(0)))) 
                       widget_control,sssy,set_value=num2str(tp_obs.raster_p.ystep)
                       if tp_obs.raster_p.ystep ne old_value then begin
                          tp_obs.raster_p.ras_id = 0
                          tp_obs.raster_v.ras_id = 0
                          tp_obs.raster_v.ras_var = 0
                       endif
                       widget_control,nsx,/input_focus
                      endif else begin
                       tp_obs.raster_p.ystep = 0
                       flash_msg,sssy,'Bad value',/nobeep
                      endelse
                     end
;
;         NSX sets up the number of locations in the x (E-W) direction in the
;             raster
;         
             'NSX':  begin
                      old_value = tp_obs.raster_p.nx 
                      if valid_num(value(0)) then begin
                       tp_obs.raster_p.nx = fix(round(fix(value(0)))) > 1 
                       widget_control,nsx,set_value=num2str(tp_obs.raster_p.nx)
                       if tp_obs.raster_p.nx ne old_value then begin
                          if tp_obs.raster_p.nx gt 255 then begin
                             flash_msg,nsx,'** LIMIT **'
                             widget_control,nsx,set_value='255'
                             tp_obs.raster_p.nx = 255 
                          endif 
                          tp_obs.raster_p.ras_id = 0
                          tp_obs.raster_v.ras_id = 0
                          tp_obs.raster_v.ras_var = 0
                       endif
                      endif else begin
                       flash_msg,nsx,'Bad value',/nobeep
                       tp_obs.raster_p.nx = 1 
                      endelse
                     end
;
;         NSY sets up the number of locations in the y (N-S) direction in the
;             raster
;
             'NSY':  begin
                      old_value = tp_obs.raster_p.ny 
                      if valid_num(value(0)) then begin
                       tp_obs.raster_p.ny = fix(round(fix(value(0)))) > 1 
                       widget_control,nsy,set_value=num2str(tp_obs.raster_p.ny)
                       if tp_obs.raster_p.ny ne old_value then begin
                          if tp_obs.raster_p.ny gt 255 then begin
                             flash_msg,nsy,'** LIMIT **'
                             widget_control,nsy,set_value='255'
                             tp_obs.raster_p.ny = 255 
                          endif 
                          tp_obs.raster_p.ras_id = 0
                          tp_obs.raster_v.ras_id = 0
                          tp_obs.raster_v.ras_var = 0
                       endif
                      endif else begin
                       tp_obs.raster_p.ny = 1 
                       flash_msg,nsy,'Bad value',/nobeep
                      endelse
                     end
             'FUND': begin
                        widget_control,wrp_info_2,get_val=text
                        tp_obs.raster_p.ras_desc = strmid(text(0),0,64)
                     end
             'VARI': begin
                        widget_control,wrv_info_2,get_val=text
                        tp_obs.raster_v.rv_desc = strmid(text(0),0,64)
                     end
             'LINE': begin
                        widget_control,wll_info_2,get_val=text
                        tp_obs.line_list.ll_desc = strmid(text(0),0,64)
                     end
             'DWIN': begin
                        widget_control,wdw_info_2,get_val=text
                        tp_obs.data_win.dw_desc = strmid(text(0),0,64)
                     end
              else : print,'unknown text window'
            endcase
           end
;
;   DRAW traps any events coming from the draw widget ie within the displayed
;        data area.  The action is now dependant upon the previously defined
;        window_state.
;
 'DRAW'  : begin

;
;  if GIS data are involved then we cannot do anything unless all data are
;  displayed for a single band, so check.
;
            if tp_obs.raster_p.detector eq 'G' and $
               (window_mode ne 'ALL' or GIS_band eq 5) then begin
               widget_control,info_line,/append,set_val=$
                  'Display all data for a single channel for cursor to be operative'
               goto,tidy_up
            endif

;    
;  cheat a little.  If the window_state is to define a flag window then tack
;  this onto the end of line identification to save duplicating the line 
;  finding code
;
            if ev.release gt 0 then begin
               if window_state eq 'FLA' then begin
                  save_wstate = window_state
                  window_state = 'IDE'
               endif else begin
                  save_wstate = ''
               endelse
            endif

            case window_state of
;
;        MOV allows the user to move a selected line window in wavelength.
;            When the mouse button is pressed the selected window is outlined
;            (if it exists) and upon release of the button a new window is
;            established at the cursor location.
;
             'MOV': begin

                     if (ev.press gt 0) then begin
                        if tp_obs.raster_p.detector eq 'N' then begin
                          lhit = nis_in_window(line_window,ev.x,ev.y)
                          if lhit ge 0 then display_nis_win,lhit,/dash 
                        endif else begin
                          gx = !x & gy = !y & gp = !p & gd = !d
                          wave = (float(ev.x)/float(gd.x_vsize)-$
                                  float(gx.s(0)))/float(gx.s(1))
                          lhit = gis_in_window(line_window,wave)
                          if lhit ge 0 then display_gis_win,lhit,/solid 
                        endelse
                        if lhit lt 0 then widget_control,info_line,/append,$
                                          set_value='*** No line there ***'
                     endif else begin
                      if lhit ge 0 then begin
                       if tp_obs.raster_p.detector eq 'N' then begin
                         erase_nis_win,lhit 
                         ev.x = 0 > ev.x < 1023
                         del_x = ev.x - line_window(lhit).x_pos
                         line_window(lhit).x_pos = ev.x
                         line_window(lhit).left  = line_window(lhit).left+del_x
                         line_window(lhit).right = line_window(lhit).right+del_x
                         if ev.y gt 127 then sp = 'N1' else sp = 'N2'
                         line_window(lhit).lam = pix2wave(sp,ev.x)
                         display_nis_win,lhit 
                       endif else begin
                         gb = 'G'+string(GIS_band,format='(i1)')
                         tp_plot_gis,GIS_data,GIS_band
                         gx = !x & gy = !y & gp = !p & gd = !d
                         wave = (float(ev.x)/float(gd.x_vsize)-$
                                 float(gx.s(0)))/float(gx.s(1))
                         wmin = pix2wave(gb,0) & wmax = pix2wave(gb,2047)
                         wave = wmin > wave < wmax
                         widget_control,info_line,/append,$
                             set_value=string('Wave = ',wave,format='(a,f8.3)')
                         pixel = wave2pix(gb,wave,/limit)
                         del_x = pixel - line_window(lhit).x_pos
                         line_window(lhit).x_pos = pixel
                         line_window(lhit).left  = line_window(lhit).left+del_x
                         line_window(lhit).right = line_window(lhit).right+del_x
                         line_window(lhit).lam   = wave
                         display_gis_win
                       endelse
;
;  since line has moved, id and order are not maintained
;
                       line_window(lhit).id = ' '
                       line_window(lhit).order = 1
;
;  inform of new position, wavelength
;
                       widget_control, info_line,/append,set_val=$
                          string('New central pixel: ',line_window(lhit).x_pos,$
                                 ' wavelength: ',line_window(lhit).lam,' (A)',$
                                 form='(a,i4,a,f6.2,a)')
                       tp_obs.line_list.ll_id  = 0
                       tp_obs.line_list2.ll_id = 0
                       tp_obs.raster_v.ll_id   = 0
                       tp_obs.data_win.ll_id   = 0
                       tp_obs.data_win.dw_id   = 0
                       tp_obs.data_win2.dw_id  = 0
                       tp_obs.raster_v.dw_id   = 0
                       tp_obs.raster_v.ras_var = 0
           
                      endif
                     endelse
                    end
;
;        IDE gives the identification of the selected line.  The identification
;            is displayed in the information line at the bottom of the page.
;  
             'IDE': begin
                     if ev.release gt 0 then begin
                      if tp_obs.raster_p.detector eq 'N' then begin
                       lhit = nis_in_window(line_window,ev.x,ev.y)
                      endif else begin
                       gx = !x & gy = !y & gp = !p & gd = !d
                       wave = (float(ev.x)/float(gd.x_vsize)-$
                               float(gx.s(0)))/float(gx.s(1))
                       lhit = gis_in_window(line_window,wave)
                      endelse
                      if lhit ge 0 then begin
                       if line_window(lhit).order eq 1 then begin
                          text = string('Window is: ',line_window(lhit).id,' ',$
                                 line_window(lhit).lam,'A',format='(3a,f6.2,a)')
                       endif else begin
                          text = string('Window is: ',line_window(lhit).id,' ',$
                                 line_window(lhit).lam,'A  (2nd order)',$
                                 format='(3a,f6.2,a)')
                       endelse
                       widget_control,info_line,/append,set_value=text
                      endif 
;
;  now position info
;
                      if tp_obs.raster_p.detector eq 'N' then begin
                       if ev.y gt 127 then sp = 'N1' else sp = 'N2'
                       wave = pix2wave(sp,ev.x)
                       if wave ge 0 then begin
                          lid = 'Cursor position is: ' + fmt_vect([ev.x,ev.y])+$
                             ' Wavelength: '+string(wave,format='(f7.3)')+'A.'+$
                             ' Intensity = '+string(VDS_image_data(ev.x,ev.y),$
                                                    format='(i6)') 
                          if wave lt 400 then delwav = 0.1 else delwav = 0.2
                          w = cds_sline(wave-delwav,wave+delwav, /cds)
                          if n_elements(w) gt 1 then begin
                             widget_control,info_line,/append,set_value=' '
                             widget_control,info_line,/append,$
                                    set_value='Possible spectral line identifications'
                             if (size(w))(0) eq 2 then begin
                                n = (size(w))(2)
                             endif else begin
                                n = 1
                             endelse
                             for i=0,n-1 do begin
                                widget_control,info_line,/append,$
                                   set_value=arr2str(w(*,i),' ')
                             endfor
                          endif
                       endif else begin
                          lid = ''
                       endelse
                      endif else begin
                       if GIS_band lt 5 then begin
                        gx = !x & gy = !y & gp = !p & gd = !d
                        wave = (float(ev.x)/float(gd.x_vsize)-$
                                float(gx.s(0)))/float(gx.s(1))
                        ev.y = fix(round((float(ev.y)/float(gd.y_vsize)-$
                                float(gy.s(0)))/float(gy.s(1))))
                        gb = 'G'+string(GIS_band,format='(i1)')
                        ev.x = wave2pix(gb,wave)
                        lid = ' Wavelength: '+string(wave,format='(f7.3)')+'A.'+$
                              ' (pixel:'+string(ev.x,form='(i4)')+')'+$
                              ' Intensity = '+string(ev.y,format='(i6)') 
                       endif else begin
                        bell
                        lid = 'GIS display must be in single band mode.'
                       endelse
                      endelse
                      if save_wstate eq '' then begin
                         widget_control,info_line,set_value=lid,/append
                      endif
;
;  was it to be a flag window?
;
                      if save_wstate ne '' then begin
                         if lhit ge 0 then begin
                           line_window(lhit).flag = 1
                           if total(line_window.flag) gt max_num_fwindow then begin
                              widget_control,info_line,/append,$
                                set_v=string('Too many flag windows. ',$
                                             'Maximum of ',max_num_fwindow,$
                                             form='(a,a,i1)') 
                              line_window(lhit).flag = 0
                           endif else begin
                              widget_control,info_line,/append,$
                                set_v='Window defined as a FLAG window.'
                              if tp_obs.raster_p.detector eq 'N' then begin
                                 erase_nis_win 
                                 display_nis_win 
                              endif else begin
                                 tp_plot_gis,GIS_data,GIS_band
                                 display_gis_win
                              endelse
                           endelse
                         endif
                         save_wstate = ''
                         window_state = 'FLA'
                      endif
                     endif
                    end
;
;        DEL deletes the selected window from the current working list of
;            windows. Valid windows in the variable line_window are deemed
;            to be those with x_pos tags >= zero.  This allows deleted windows
;            to be eliminated and valid ones to be brought to the front.
;            The number of windows defined at any instant is defined as the
;            index of the first non-zero x_pos tag (+1!).
;
             'DEL': begin
                     if ev.release gt 0 then begin
                      if tp_obs.raster_p.detector eq 'N' then begin
                       lhit = nis_in_window(line_window,ev.x,ev.y)
                       if lhit ge 0 then begin
                        widget_control,info_line,/append,$
                               set_value='Line '+line_window(lhit).id+' deleted'
                        xl = line_window(lhit).left
                        xr = line_window(lhit).right
                        y = line_window(lhit).y_pos
                        erase_nis_win,lhit
                        del_window,lhit
                        sort_windows,line_window,num_windows
                       endif else begin
                        widget_control,info_line,/append,$
                               set_value='** No line at that position. **'
                       endelse
                      endif else begin
                       gx = !x & gy = !y & gp = !p & gd = !d
                       wave = (float(ev.x)/float(gd.x_vsize)-$
                               float(gx.s(0)))/float(gx.s(1))
                       lhit = gis_in_window(line_window,wave)
                       if lhit ge 0 then begin
                        display_gis_win,lhit,/solid 
                        wait,1
                        del_window,lhit
                        sort_windows,line_window,num_windows
                        tp_plot_gis,GIS_data,GIS_band
                        display_gis_win
                       endif else begin
                        widget_control,info_line,/append,$
                               set_value='** No line at that position. **'
                       endelse
                      endelse
;
;  if no windows left then adjust descriptors
;
                      if num_windows eq 0 then begin
                         tp_obs.line_list.ll_desc='All detector data'
                         curr_llist = 'All data'
                         curr_llist_name = 'All detector data'
                         widget_control,wll_info_2,set_val='All detector data'
                      endif
                      if lhit ge 0 then begin
                       tp_obs.line_list.ll_id  = 0
                       tp_obs.line_list2.ll_id = 0
                       tp_obs.raster_v.ll_id   = 0
                       tp_obs.data_win.ll_id   = 0
                       tp_obs.data_win.dw_id   = 0
                       tp_obs.data_win2.dw_id  = 0
                       tp_obs.raster_v.dw_id   = 0
                       tp_obs.raster_v.ras_var = 0
                      endif
                     endif
                    end
;
;        ADD allows a user-defined window to be added to the list.  The window
;            is located wherever the mouse button is clicked.  The maximum
;            number of windows allowed is set by the variable max_num_lwindow.
;            This constant is set in tplan_com.
;
             'ADD': begin
                     if ev.release then begin
                      if num_windows lt max_num_lwindow then begin
                       if tp_obs.raster_p.detector eq 'N' then begin
                        y = (ev.y gt 127) * 128  
                        line_window(num_windows).x_pos = ev.x  
                        line_window(num_windows).y_pos = y
                        line_window(num_windows).left  = ev.x-(curr_lwidth/2)  
                        line_window(num_windows).right = $
                               line_window(num_windows).left + curr_lwidth - 1  
                        line_window(num_windows).height = curr_lheight 
                        if ev.y gt 127 then sp = 'N1' else sp = 'N2'
                        line_window(num_windows).lam = pix2wave(sp,ev.x)
                        line_window(num_windows).order = 1
                        line_window(num_windows).id  = 'New VDS line'
                        display_nis_win,num_windows 
                       endif else begin
                        gx = !x & gy = !y & gp = !p & gd = !d
                        wave = (float(ev.x)/float(gd.x_vsize)-$
                                  float(gx.s(0)))/float(gx.s(1))
                        gb = 'G'+string(GIS_band,format='(i1)')
                        line_window(num_windows).lam = wave 
                        line_window(num_windows).order = 1
                        evx = wave2pix(gb,wave)
                        line_window(num_windows).x_pos = wave2pix(gb,wave)  
                        line_window(num_windows).y_pos = GIS_band
                        line_window(num_windows).left  = evx-(curr_lwidth/2)  
                        line_window(num_windows).right = $
                               line_window(num_windows).left + curr_lwidth - 1  
                        line_window(num_windows).height = curr_lheight 
                        line_window(num_windows).id  = 'New GIS line'
                        display_gis_win,num_windows
                       endelse
                       num_windows = num_windows + 1
                       sort_windows,line_window,num_windows
                       widget_control,wll_list_1,/sensitive
;                      
;  give details of new line
;
                       if tp_obs.raster_p.detector eq 'N' then begin
                        if ev.y gt 127 then sp = 'N1' else sp = 'N2'
                        wave = pix2wave(sp,ev.x)
                        lid = 'New line added at: ' + fmt_vect([ev.x,ev.y])+$
                             ' Wavelength: '+string(wave,format='(f7.3)')+'A.'+$
                             ' Intensity = '+string(VDS_image_data(ev.x,ev.y),$
                                                     format='(i6)') 
                       endif else begin
                        if GIS_band lt 5 then begin
                         gx = !x & gy = !y & gp = !p & gd = !d
                         wave = (float(ev.x)/float(gd.x_vsize)-$
                                 float(gx.s(0)))/float(gx.s(1))
                         ev.y = fix(round((float(ev.y)/float(gd.y_vsize)-$
                                 float(gy.s(0)))/float(gy.s(1))))
                         gb = 'G'+string(GIS_band,format='(i1)')
                         ev.x = wave2pix(gb,wave)
                         lid = ' New line added at wavelength: '+$
                                 string(wave,format='(f7.3)')+'A.'+$
                              ' Intensity = '+string(ev.y,format='(i6)') 
                        endif else begin
                         bell
                         lid = 'GIS display must be in single band mode.'
                        endelse
                       endelse
                       widget_control,info_line,set_value=lid,/append
                       tp_obs.line_list.ll_id  = 0
                       tp_obs.line_list2.ll_id = 0
                       tp_obs.raster_v.ll_id   = 0
                       tp_obs.data_win.ll_id   = 0
                       tp_obs.data_win.dw_id   = 0
                       tp_obs.data_win2.dw_id  = 0
                       tp_obs.raster_v.dw_id   = 0
                       tp_obs.raster_v.ras_var = 0
           
                      endif else begin
                       bell
                       flash_msg,info_line,/append,$
                                 'Maximum number of windows reached.'
                      endelse 
                     endif
                    end  
;
;  dummy to cater for when FLAG mode in on and mouse button pressed (release)
;  is catered for under IDE function.
;
             'FLA': begin
                    end
;
;  profile mode for NIS data only
;
             'PRO': begin
                       if tp_obs.raster_p.detector eq 'N' then begin
                         if ev.release eq 1 then begin
                            for i=0,5 do widget_control,wmode_ids(i),set_b=0
                            widget_control,wmode_ids(3),set_b=1
                            window_state='IDE'
                            profiles,VDS_image_data
                         endif
                       endif else begin
                         widget_control,info_line,/append,$
                            set_v='Profile mode not implemented for GIS data.'
                       endelse
                    end

;
;        If no window_state is identified then no window manipulation button
;        has been clicked so no operation is allowed within the display area.
;
             else : begin
                     bell,2
                     widget_control,info_line,/append,$
                        set_value='No current image data manipulation command.'
                    end
            endcase                   
            
;
;  wise to update lines structure to reflect any changes just made
;
            tp_obs.line_w = line_window
            tp_update_lines, tp_obs
           end
                
     else: begin

;
;  was it from a cw_field which are used for exp time and window width 
;  when the user types in the value rather than using the slider and the 
;  base has no name?
;
            widget_control,ev.id,get_value=value,get_uvalue=uvalue
            case uvalue of
           'ETRANGE':  begin
                         last_et = curr_etime
                         val = round_off(float(value(0)),0.001)
                         ett_form='(f5.3)'
                         if val gt 1000.0 then begin
                               val = 1000.0
                               bell
                               widget_control,info_line,$
                                 set_value='Max exposure = 1000.0 seconds'
                         endif
                         if val ge 1.0 then begin
                            val = round_off(val,0.01)
                            ett_form = '(f4.2)'
                         endif
                         if val ge 10.0 then begin
                            val = round_off(val,0.1)
                            ett_form = '(f4.1)'
                         endif
                         if val ge 100. then begin
                            val = round_off(val,1.0)
                            ett_form = '(i4)'
                         endif
                         curr_et_max = 10^(fix(alog10(val)+1))
                         stat = execute('curr_etime = string(val,form=ett_form)')
                         widget_control,et_info,set_value=curr_etime
                         scale_val = $
                               round((float(curr_etime)*1000L/curr_et_max))
                         widget_control,et_slider_id,set_val=scale_val
                         tp_obs.raster_v.exptime = curr_etime
;
;  has exposure time actually changed?
;
                         if (curr_etime ne last_et) then begin
                            widget_control,base,/hourglass
                            eratio = float(curr_etime)/float(last_et)
                            case tp_obs.raster_p.detector of
                              'N': begin
                                    VDS_image_data = VDS_image_data * eratio
;for non-linear                     tp_get_dumdata,tp_obs,VDS_image_data
                                    tv,tp_bscale(VDS_image_data),0,0
                                    display_nis_win
                                    tp_colbar
                                    tp_vds_axes
                                   end
                              'G': begin
                                    tp_get_dumdata,tp_obs,GIS_data,$
                                                   zone=current_zone
                                    tp_plot_gis,GIS_data,GIS_band
                                    display_gis_win
                                   end
                              else: 
                            endcase
                            tp_obs.raster_v.ras_var = 0
                         endif 
                       end
           'WWRANGE':  begin
                         curr_lwidth = value(0) > 3
                         widget_control,ww_info,set_value=curr_lwidth
                         curr_ww_max = 10^(fix(alog10(curr_lwidth)+1))
                         scale_val = $
                               round((float(curr_lwidth)*1000L/curr_ww_max))
                         widget_control,ww_slider_id,set_val=scale_val
;
;  temp? fiddle for GIS
;
if tp_obs.raster_p.detector eq 'G' and curr_lwidth eq 1000 then begin
   curr_lwidth = 2048
   widget_control,ww_info,set_value=curr_lwidth
endif
                         for i=0,num_windows-1 do begin
                           if tp_obs.raster_p.detector eq 'N' then erase_nis_win,i 
                           line_window(i).left = $
                                      line_window(i).x_pos - (curr_lwidth/2)
                           line_window(i).right = $
                                      line_window(i).left + curr_lwidth - 1
                           if tp_obs.raster_p.detector eq 'N' then begin
                              display_nis_win,i 
                           endif 
                         endfor 
                         if tp_obs.raster_p.detector eq 'G' then begin
                           if window_mode eq 'ALL' then begin
                              tp_plot_gis,GIS_data,GIS_band
                           endif
                           display_gis_win
                         endif
                       end
                 else: begin
;                             print,'name: ',name,' value: ',value,' uvalue: ',uvalue
                          end
            endcase
           end
endcase


;
;  destination of GOTOs
;
tidy_up:

;
;  This is the time for checks to be made....
;
if strmid(uvalue,0,4) ne 'EXIT' then begin
   errmsg = ''
   warnmsg = ''
   status = tp_chk_raster(tp_obs, errmsg=errmsg, warnmsg=warnmsg)
   if errmsg ne '' then begin
      bell
      widget_control, info_line, /append, set_val=errmsg
   endif
   if warnmsg ne '' then begin
      widget_control, info_line, set_val=warnmsg, /append
   endif
;
;  in case colour table has changed
;
   if tp_obs.raster_p.detector eq 'N' then tp_vds_axes
endif 

;
;  Many things could have affected the line window definition so
;  recalculate the data extraction windows here so they are constantly
;  available.  Don't do it though if we are just dragging one of the sliders,
;  only when the dragging stops.
;
;
;  if raster duration window is up then keep it in front
;
if tp_obs.raster_p.detector eq 'N' or tp_obs.raster_p.detector eq 'G' then begin
   if tag_exist(ev,'DRAG') then begin
      if not ev.drag then begin
         tp_recalc_dew,' '
         if xregistered('tp_rasdur') then  widget_control,plot_rasdur_base,map=1
      endif
   endif else begin
      tp_recalc_dew, strmid(uvalue,0,4)
      if xregistered('tp_rasdur') then  widget_control,plot_rasdur_base,map=1
   endelse   
endif


end

;
; --------------------------------
;  The main routine
; --------------------------------
;

pro mk_raster, setup_structure, modal=modal, group_leader=group_leader,$
               font=font,fix_desc=fix_desc


;
;  common block source
;
@tplan_com

;
;  define the structures to be used
;
tplan_struct

;
; results will be saved in the observation/raster definition structure
;
tp_obs = {tp_obs_rec}
tp_obs.line_w.x_pos = -1
;
;  initial checks
;
if (!d.flags and 65536) eq 0 then message,'Widgets are NOT available.'

;
;  check not already in use
;  (if not called by a "living" widget then do a reset via XKILL
;

if not xalive(group_leader) then xkill,/all else begin
 if xregistered('mk_raster') then begin
   print,'CDS mk_raster program is already registered.'
   return
 endif
endelse

;
;  set only possible graphics device and load LUT
;
set_plot,'X'
tp_coltable

;
;  the base widget is all-screen and column-orientated
;
if not keyword_set(font) then begin
;   font = get_dfont() & font = font(0)
   font = '-adobe-helvetica-bold-r-normal--0-120-75-75-p-0-iso8859-1'
endif
font=get_dfont(font) & font=font(0)

device, get_screen_size = sz
if (sz(0) ge 1280) and (sz(1) ge 1024) then sz(*) = 0
sz = sz < [1280,1024]
base = widget_base(title='CDS Technical Planning - raster definition',xoff=100,$
                   /column,/frame,x_scroll=sz(0),y_scroll=sz(1))

;
;  the TOP widget is where all the user/numerical/text input takes place
;
top = widget_base(base,/row,space=1)

;
;  the left hand side of the top contains...
;
top_left = widget_base(top,/column,space=5)

;
;  ...the instrument choice
;
inst_base = widget_base(top_left,/row,space=50)
inst_list = ['NIS','GIS']
instr_ids = lonarr(2)
xmenu,inst_list,inst_base,/row,/exclusive,title='Instrument',$
                     buttons=instr_ids,uvalue=['INST1','INST2'],/no_release, $
		     font=font
previous_inst = ''

;
;  ... VDS default extraction switch or GIS default extraction
;
vds_bbase = widget_base(inst_base)
vds_backbase = lonarr(3)

;
;  blank for when no instrument is chosen
;
vds_backbase(0) = widget_base(vds_bbase,map=0)

;
;  
vds_backbase(1) = widget_base(vds_bbase,map=0)
ex_ids = lonarr(3)
xmenu,['NIS1','NIS2','Full CCD'],buttons=ex_ids,vds_backbase(1),/row,$
                   title='VDS default extraction.', /nonexclusive,$
                   uvalue=['NDEFEX_1','NDEFEX_2','NDEFEX_3'], font=font
for i=0,2 do widget_control,ex_ids(i),set_button=0
widget_control,ex_ids(1),set_button=1
nis_def_ext=[0,0,0]


;
;  Default GIS extraction mode (all or nothing)
;
vds_backbase(2) = widget_base(vds_bbase,map=0)
gbdefuv = ['GDEFEX_1','GDEFEX_2','GDEFEX_3','GDEFEX_4']
gb_list = ['1','2','3','4']
xmenu,gb_list,vds_backbase(2),buttons=gbdef_ids,title='GIS default extraction.',$
      uvalue=gbdefuv,/row,/nonexclusive,font=font
for i=0,3 do widget_control,gbdef_ids(i),set_button=1
gis_def_ext = [1,1,1,1]


;
;  ...the slit choices
;
sb = widget_base(top_left,/row)
sb1 = widget_base(sb,/column)
xpdmenu,find_with_def('slit_selection.pdm',!PATH),sb1,font=font
sb2 = widget_base(sb,/column)
slit_info = cw_field(sb2,title='Current slit:',value='    ',$
            /row,xsize=15)



;
;  choose the range for the exposure time slider
;
et = widget_base(top_left,/column)
eb1 = widget_base(et,/row)
xpdmenu,find_with_def('etime_selection.pdm',!PATH),eb1,font=font

;
;  put out the current value separately
;
eb2 = widget_base(eb1,/column)
et_info = cw_field(eb2,title='Exposure time:',value='50',$
             /row,xsize=8,/return,uvalue='ETRANGE')

;
;  ...the exposure time slider
;
et_slider_id = widget_slider(et,title='     Set exposure time in seconds.  ',$
                             value=500,min=10,max=1000,uvalue='ET_SLIDER',$
                             /drag,/suppress,font=font)

;
;  set up initial range and value
;
curr_et_max = 100.
curr_etime  = '50'
tp_obs.raster_v.exptime = 50.0

;
;  choose the range for the window width slider
;
wt = widget_base(top_left,/column)
ww1 = widget_base(wt,/row)
xpdmenu,find_with_def('wwidth_selection.pdm',!PATH),ww1,font=font

;
;  put out the current value separately
;
ww2 = widget_base(ww1,/column)
ww_info = cw_field(ww2,title='Window width:',value='11',$
             /row,xsize=5,/return,uvalue='WWRANGE')
widget_control,ww_info,all_text=0
;
;  ...the window width slider
;
ww_slider_id = widget_slider(wt,title='     Set window width in pixels.  ',$
                             value=110,min=3,max=1000,uvalue='WW_SLIDER',$
                             /drag,/suppress,font=font)

;
;  set up initial range and value
;
curr_ww_max = 100.
curr_lwidth  = '11'


;
; the top middle widget contains...
;
top_middle= widget_base(top,/column,space=10)

;
;  ...the scan step sizes
;
;ssrow = widget_base(top_middle,/row)
ssrow = widget_base(top_left,/row)
ss    = widget_base(ssrow,/column)
wlss  = widget_label(ss,value='M/S step size (arcsec).',font=font)
sss   = widget_base(ss,/row)
sssx  = widget_text(sss,/edit,xsize=10,ysize=1,uvalue='SSSX',value='0',	$
	font=font)
sssy  = widget_text(sss,/edit,xsize=10,ysize=1,uvalue='SSSY',value='0',	$
	font=font)

;
;  ...the number of steps
;
ns   = widget_base(ssrow,/column)
wlns = widget_label(ns,value='Number of M/S scan positions.',font=font)
nss  = widget_base(ns,/row)
nsx  = widget_text(nss,/edit,xsize=10,ysize=1,uvalue='NSX',value='1',font=font)
nsy  = widget_text(nss,/edit,xsize=10,ysize=1,uvalue='NSY',value='1',font=font)
     

;
;  information on  line list
;
wll = widget_base(top_middle,/column)

;
;  and inform of current choice
;
dummy  = widget_label(wll,value='Current descriptions and database IDs',font=font)

fiddle = '' & for i=0,64 do fiddle=fiddle+'a'
wrp_info   = widget_base(wll,/row)
wrp_info_1 = widget_label(wrp_info,value='Raster.....',font=font)
wrp_info_2 = widget_text(wrp_info,value=fiddle,font=font,/editable,/all,$
                         uvalue='FUND',xsize=50)
frid = widget_text(wrp_info,value=' ',font=font,xsize=3)
widget_control, frid,set_val=''
widget_control, wrp_info_2,set_val=''


wrv_info   = widget_base(wll,/row)
wrv_info_1 = widget_label(wrv_info,value='Variation',font=font)
wrv_info_2 = widget_text(wrv_info,value=fiddle,font=font,/editable,$
                         uvalue='VARI',xsize=50)
rvid = widget_text(wrv_info,value=' ',font=font,xsize=3)
widget_control, rvid,set_val=''
widget_control, wrv_info_2,set_val=''

wll_info   = widget_base(wll,/row)
wll_info_1 = widget_label(wll_info,value='Line list...',font=font)
wll_info_2 = widget_text(wll_info,value=fiddle,font=font,/editable,$
                         uvalue='LINE',xsize=50);,/all)
llid = widget_text(wll_info,value=' ',font=font,xsize=3)
widget_control, llid,set_val=''
widget_control, wll_info_2,set_val=''

wdw_info   = widget_base(wll,/row)
wdw_info_1 = widget_label(wdw_info,value='Windows',font=font)
wdw_info_2 = widget_text(wdw_info,value=fiddle,font=font,/editable,$
                         uvalue='DWIN',xsize=50)
dwid = widget_text(wdw_info,value=' ',font=font,xsize=3)
widget_control, dwid,set_val=''
widget_control, wdw_info_2,set_val=''

;
;  set up the line_window structure and clear it out
;  (necessary because values linger from one incarnation to next)
;

line_window = replicate({line_window_rec},max_num_lwindow)
line_window.x_pos = -1

;
;  information/selection of compression scheme
;
cs = widget_base(top_middle,/row)
;
;  ...compression scheme choice
;
xpdmenu,find_with_def('compression_scheme.pdm',!PATH),cs,font=font

;
;  and inform of current choice
;
ccs_info   = widget_base(cs,/column)
ccs_info_1 = widget_label(ccs_info,value='Current compression',font=font)
ccs_info_2 = widget_label(ccs_info,value=strpad(' ',30),/frame,font=font)


;
;  selection of telemetry rate
;
tmr = widget_base(cs,/row)
;
;  ...telemetry rate choice
;
xpdmenu,find_with_def('telemetry_rate.pdm',!PATH),tmr,font=font


;
;  and inform of current choice
;
tmr_info   = widget_base(cs,/column)
tmr_info_1 = widget_label(tmr_info,value='Current TM rate',font=font)
tmr_info_2 = widget_label(tmr_info,value=strpad(' ',30),/frame,font=font)


;
;  set up a layered base so VDS specific widgets can come and go
;  interleaved with GIS selections
;
parm = widget_base(top_middle)

;
;  set number of stacking levels
;
npb = 7
parmbase = lonarr(npb+1)

;
;  first layer is blank
;
parmbase(0) = widget_base(parm,map=0)


;
;  Set up VDS-specific parameters
;
;
;  ...parameter choice
;
parmbase(1) = widget_base(parm,/row,map=0)
xpdmenu,find_with_def('vds_parameters.pdm',!PATH),parmbase(1),font=font

;
;  ...the window height slider
;
wh_slider_id = widget_slider(parmbase(1),$
                             title='VDS window height (arcsec).',$
                             value=240,uvalue='WH_SLIDER',minimum=4,$
                             maximum=240,/drag,xsize=300,font=font)
curr_lheight = '240'

;
;  possible overlay for setting VDS voltage
;
parmbase(2) = widget_base(parm,/row,map=0,/frame)
vv_done = widget_button(parmbase(2),value='Done',uvalue='VDS_VDONE',/no_rel, $
	font=font)
vv_slider_id = widget_slider(parmbase(2),title='Set VDS voltage parameter.',$
                             value=100,uvalue='VV_SLIDER',minimum=0,$
                             maximum=255,/drag,xsize=450,font=font)


;
;  setup of GIS parameters
;
parmbase(3) = widget_base(parm,/row,map=0)
xpdmenu,find_with_def('gis_parameters.pdm',!PATH),parmbase(3),font=font


;
;  set GIS detector voltages
;
parmbase(4) = widget_base(parm,/row,map=0,/frame)
gv_done = widget_button(parmbase(4),value='Done',uvalue='GIS_VDONE',/no_rel, $
	font=font)
gv_slider_id = widget_slider(parmbase(4),title='Set GIS voltage parameter.',$
                             value=100,uvalue='GV_SLIDER',minimum=0,$
                             maximum=255,/drag,xsize=450,font=font)

;
;  parmbase(5) reserved for GIS LUTs when figure out how to handle them
;
parmbase(5) = widget_base(parm,map=0)


;
;  possible overlay for setting VDS 300A range offset
;
parmbase(6) = widget_base(parm,/row,map=0,/frame)
o3_done = widget_button(parmbase(6),value='Done',uvalue='VDS_VDON',/no_rel, $
	font=font)
o3_slider_id = widget_slider(parmbase(6),$
                             title='Set 300A range offset (pixels).',$
                             value=0,uvalue='O3_SLIDER',minimum=0,$
                             maximum=100,/drag,xsize=450,font=font)


;
;  possible overlay for setting VDS 300A range offset
;
parmbase(7) = widget_base(parm,/row,map=0,/frame)
o5_done = widget_button(parmbase(7),value='Done',uvalue='VDS_VDON',/no_rel, $
	font=font)
o5_slider_id = widget_slider(parmbase(7),$
                             title='Set 500A range offset (pixels).',$
                             value=0,uvalue='O5_SLIDER',minimum=0,$
                             maximum=100,/drag,xsize=450,font=font)


;
;  telemetry info space
;
tlm_info = widget_base(top_middle,/row,space=15)
dummy    = widget_label(tlm_info,value='Telemetry time (secs/exposure)', $
	font=font)
tlm_info_2 = widget_label(tlm_info,value=strpad('0',15),frame=5,	$
	font=font,xsize=50)
widget_control,tlm_info_2,set_val='  0.0 '


;
;  In depth analysis of raster duration
;
tlm_info_3 = widget_button(tlm_info,value='Analyse Raster Duration',$
                           uvalue='RAS_DUR',font=font,frame=2)



;
;  MIDDLE "command" line has the options...
;
middle = widget_base(base,/row,space=5,/frame)

;
;  ...Quit without any further ado
;
m4 = widget_base(middle,/column)
xpdmenu,find_with_def('exit_selection.pdm',!PATH),m4,font=font

;
;  ...get help on filling out the screen
;
m5 = widget_base(middle,/column)
cfont = "-adobe-helvetica-bold-r-normal--14-100-100-100-p-82-iso8859-1"
xpdmenu,find_with_def('help_selection.pdm',!PATH),m5,font=cfont


;
;  ...reset a selection of parameters to start again
;
m3 = widget_base(middle,/column)
xpdmenu,find_with_def('reset_selection.pdm',!PATH),m3,font=font


;
;  write output of line windows, flag windows or raster parameters
;
;
;  ...choose type of output
;
m6 = widget_base(middle,/column)
xpdmenu,find_with_def('db_access.pdm',!PATH),m6,font=font



m1 = widget_base(middle)
dum = lonarr(2)
dum(0) = widget_base(m1,map=1)
dum(1) = widget_button(m1,value='Action to the left',$
                             uvalue='',/no_rel,font=font)
widget_control,dum(1),map=0


;
;  ...calculate new data with current instrument settings
;
m2 = widget_base(middle,/column)
xpdmenu,find_with_def('mk_raster_ndata.pdm',!PATH),m2,font=font


;
;  ...allow switch between CDS and USER databases
;
db_original = which_zdbase()
db_in_use = db_original

;
;  check at least one database env. var. exists
;
z1 = getenv('ZDBASE_CDS')
z2 = getenv('ZDBASE_USER')
if z1 eq '' and z2 eq '' then begin
   bell
   print,'Must have a definition for one of ZDBASE_USER or ZDBASE_CDS'
   return
end

m8 = widget_base(middle,/column)
if db_original eq 'CDS' then $
   title = 'Using CDS databases' else $
   title = 'Using USER databases'

db_switch = widget_button(m8,value=title,uvalue='USER_CDS_DB',/no_rel,font=font)

m9 = widget_button(middle,value='Adjust LUT',uvalue='XLOADCT',font=font)



;
;  make message visible/invisible
;
see_mess=0
m10 = widget_button(middle,value='See messages',uvalue='MESS',$
                    font=cfont,frame=2)



;
;  allow editing/listing of current choice
;
m11   = widget_base(middle,/column)
wll_list_1 = widget_button(m11,value='Edit lines',$
                          uvalue='EDIT_LINE',/no_rel,font=font)
widget_control,wll_list_1,sensitive=0
num_windows = 0



;
;  BOTTOM display area
;   
bottom = widget_base(base,/frame,/column)

;
;  ...plotting area
;
image_area = widget_draw(bottom,xsize=1024,ysize=256,/button_events)
have_vert = 0
widget_control,image_area,sensitive=0


;
;  LOWER BOTTOM "command" line has the window manipulating options...
;
low_bottom = widget_base(base,/row,space=5)
window_state = 'N/A'

;
;  choose the window manipulation mode
;
wmode_list = ['Add','Del','Mov','Id','Flag','Prof']
wmode_uv =['LINE_ADD','LINE_DELETE','LINE_MOVE','LINE_IDENT','LINE_FLAG','LINE_PROF']
wmode_rv = widget_base(low_bottom,/column)
wmode_ids = lonarr(6)

xmenu,wmode_list,wmode_rv,buttons=wmode_ids,title='Window interaction mode.',$
      uvalue=wmode_uv,/row,/exclusive,font=font

;
;  choose the display mode
;
;dmode_list = ['All','Win','Flag']
;dmode_uv = ['DISP_ALL','DISP_WONLY','DISP_FONLY']
;dmode_rv = widget_base(low_bottom,/column)
;dmode_ids = lonarr(3)
;
;xmenu,dmode_list,dmode_rv,buttons=dmode_ids,title='Display mode.',$
;      uvalue=dmode_uv,/row,/exclusive,space=1,font=font
;widget_control,dmode_ids(0),set_button=1

dm = widget_base(low_bottom,/row)
dm1 = widget_base(dm,/column)
xpdmenu,find_with_def('display_mode.pdm',!PATH),dm1,font=font

fm = widget_base(low_bottom,/row)
fm1 = widget_base(fm,/column)
xpdmenu,find_with_def('flag_choice.pdm',!PATH),fm1,font=font


;
;  choose which GIS band to display
;
gis_b = widget_base(low_bottom,/row,space=5)
gb_list = ['1','2','3','4','All']
gbuv = ['GISB1','GISB2','GISB3','GISB4','GISB5']
gb_ids = lonarr(5)

;
;  make gb_rv base a stackable one
;
gis_dispbase = lonarr(4)
gb_rv = widget_base(gis_b)

;
;  first layer is blank
;
gis_dispbase(0) = widget_base(gb_rv,map=0)

;
;  put real buttons in next level
;
gis_dispbase(1) = widget_base(gb_rv,map=0)
xmenu,gb_list,gis_dispbase(1),buttons=gb_ids,title='Display GIS band.',$
      uvalue=gbuv,/row,/exclusive,font=font
widget_control,gb_ids(4),set_button=1
GIS_band = 5

;
;  ...adjust VDS display in next layer
;
gis_dispbase(2) = widget_base(gb_rv,map=0,/row)
dummyb = widget_base(gis_dispbase(2),/column)
euv = ['EXTE_YES','EXTE_NO','EXTE_RES']
xmenu,['On','Off','Set'],dummyb,buttons=vdsb_ids,title='VDS Background',$
      uvalue=euv,/row,/exclusive,font=font
widget_control,vdsb_ids(0), set_b=1
vds_extension = 1

;
;  image area for LUT
;
image_area2 = widget_draw(gis_dispbase(2),xsize=225,ysize=100)

;
;  Slider for VDS background window size
;
gis_dispbase(3) = widget_base(gb_rv,map=0,/row)
dummy = widget_button(gis_dispbase(3),value='Done',uvalue='VDS_BDONE',	$
	/no_rel,font=font)
vdsb_slider_id = widget_slider(gis_dispbase(3),xsize=300,$
                               title='VDS background window size',font=font,$
                               min=3,max=100,uvalue='VB_SLIDER',value=50,/drag)


;
;  INFO_LINE line has general user information.
;                   
info_base = widget_base(title='Messages',group_leader=base,yoff=1000,/column)
;                          xsize=800,ysize=300,y_scroll_size=300,$
;                          x_scroll_size=800,/scroll,/column)
info_line = widget_text(info_base,ysize=20,xsize=65,$
                        value='Watch this space for information....',$
                        font=tp_font,/scroll)

;
;  ensure plotting is as needed
;
!p.multi=0

;
;  make the whole thing happen
;
widget_control,base,/realize
widget_control,info_base,/realize
;widget_control, mess_ids(0),set_butt=1 

;
;  find out window id for plotting
;
widget_control,get_value=window,image_area
wset,window
tp_colbar

;
; set default mode of displaying all spectrum
;
window_mode = 'ALL'
display_state = 'ALL'

;
;  set default 'flags included' mode
;
flag_state = 1


;
;  create arrays for the vertical solid and dashed lines
;
vert_line = bytarr(1,128)+255
vert_dash = vert_line
for i=0,15,2 do vert_dash(0,i*8:i*8+7) = 0

;
;  load special LUT
;
tp_coltable
tp_colbar



;
;  load the current state database entry, the official db must be in the 
;  ZDBASE path
;

get_utc,utc
get_vds_state, utc, s
tp_obs.state.date      = s.date
tp_obs.state.vds_read  = fix(s.vds_read)
tp_obs.state.vds_pmcp  = fix(s.vds_pmcp)
tp_obs.state.vds_offset(0)  = 0       ;  s.vds_offset(0)
tp_obs.state.vds_offset(1)  = 0       ;  s.vds_offset(1)
help_mode = 0


;
;  save the current wavecal and load the linear one for GIS and the current NIS
;

n_wavecal = save_wavecal('n')
g_wavecal = save_wavecal('g')

if not load_wavecal('n') then begin
   bell
   widget_control, info_line, $
               set_val='Error loading NIS wavecal.'
endif

if not load_wavecal('g',gset=0) then begin
   bell
   widget_control, info_line, $
               set_val='Error loading GIS wavecal.'
endif


;
;  load input structure if appropriate 
;
if datatype(setup_structure) eq 'STC' then begin
   load_tp_struct, setup_structure 
   tp_colbar
   tp_recalc_dew,' '
   if total(tp_obs.vds_backwin.win_def) eq 0 then begin
      status = tp_load_vdsback(tp_obs)
      if not status then begin
         widget_control, info_line, $
               set_val='Error reading VDS background windows file.'
      endif
   endif
endif else begin
   status = tp_load_vdsback(tp_obs)
   if not status then begin
      widget_control, info_line, $
           set_val='Error reading VDS background windows file.'
   endif
endelse


;
;  help message
;
widget_control,info_line,set_val=' ',/append
widget_control,info_line,set_val=' ',/append
widget_control,info_line,set_val=' ',/append
bl = '                                                     '
widget_control, info_line, /append, $
       set_v='Hit the <Messages> button to hide this window'

;
;  undocumented feature
;
if keyword_set(fix_desc) then begin
   defsysv,'!desc_update',1 
endif else begin
   defsysv,'!desc_update',0 
endelse

;
; if MK_RASTER was called with a GROUP_LEADER, then store it's widget ID
; as a UVALUE of BASE. Later, when MK_RASTER is done, the latest
; TP_OBS stucture will be stored as a UVALUE of GROUP_LEADER and, hence,
; become available to caller (DMZ'99)
;

if xalive(group_leader) then widget_control,base,set_uvalue=group_leader

;
; make the boss do some work

xmanager,'mk_raster',base, modal=modal, group_leader=group_leader

;-- if MK_RASTER crashed for some reason, then the next MK_RASTER call will
;   fall thru the first XMANAGER. The next line will re-activate the event
;   handler.

if not xalive(group_leader) then xmanager

;
; load the final line setup into the input/output structure
;

tp_obs.line_w = line_window

;
;  on EXIT, as an internal indicator that flags were either separate or 
;  common, set the standard flag indicator to '2' if flags are separate.
;
if flag_state eq 0 then begin
   n = where(tp_obs.line_w.flag eq 1,count)
   if count gt 0 then tp_obs.line_w(n).flag = 2
endif

;
;  load the input/output variable with the defined structure
;
setup_structure = tp_obs

end
