;+
; Project     : SOHO - CDS     
;                   
; Name        : TP_DUMP_SETUP
;               
; Purpose     : Produce hardcopy of TPLAN setup structure.
;               
; Explanation : Sets down on paper the details contained within the
;               TPLAN structure.  Can either have shortened versions if
;               requested.
;               
; Use         : tp_dump_setup, tp_obs [,/line, /raster, /dwin]
;    
; Inputs      : tp_obs  -  the standard internal TPLAN structure.
;               
; Opt. Inputs : None
;               
; Outputs     : None
;               
; Opt. Outputs: None
;               
; Keywords    : LINE   - produces a listing of the line list details only
;               RASTER - produces details of the raster definition only
;               DWIN   - produces details of the data windows only
;
; Calls       : None
;
; Common      : None
;               
; Restrictions: None
;               
; Side effects: None
;               
; Category    : Technical, planning
;               
; Prev. Hist. : None
;
; Written     : C D Pike, RAL, 1-Nov-94
;               
; Modified    : Fixed typo in output format.  CDP, 1-Jul-96
;               Rework handling of user compression scheme.  CDP, 06-Oct-97
;
; Version     : Version 3, 06-Oct-97
;-            
pro tp_dump_setup, tp_obs, line=line, raster=raster, dwin=dwin

;
;  check input
;
if n_params() eq 0 then begin
   print,'Use: tp_dump_setup,  structure [,/line, /raster, /dwin]'
   return
endif

;
;  open temporary file
;
openw, lun, 'tplan_setup.list', /get_lun, error=err
if err ne 0 then begin
   print,'Error opening setup list file.'
   return
endif

;
;  check on keywords
;
if not keyword_set(line) and $
      not keyword_set(raster) and $
      not keyword_set(dwin) then want_all = 1 else want_all = 0

if keyword_set(line)   then line   = 1 else line   = 0
if keyword_set(raster) then raster = 1 else raster = 0
if keyword_set(dwin)   then dwin   = 1 else dwin   = 0

;
;  header first
;
printf,lun,'                CDS technical planning definitions for the '+$
             tp_obs.raster_p.detector+'IS'
printf,lun,'                ----------------------------------------------'
printf,lun,' '

;
;  FUNDAMENTAL/VARIATION raster if needed
;  =====================
;
if want_all or raster then begin
   slit = ['(2 x 2)','(4 x 4)','(8 x 51)','(2 x 240)','(4 x 240)','(90 x 240)']
   sid = strtrim(string(tp_obs.raster_p.ras_id,form='(i5)'),2)
   printf,lun,'FUNDAMENTAL raster ID: ',sid,'  (',$
               strtrim(tp_obs.raster_p.ras_desc,2),')' 

   if tp_obs.raster_p.slit_num gt 0 then begin
      printf,lun,string('Slit: #',tp_obs.raster_p.slit_num,' - ',$
                 slit(tp_obs.raster_p.slit_num-1),' arcsecs',form='(a,i1,3a)')
   endif else begin
      printf,lun,'Slit: None defined'
   endelse

   printf,lun,string('Size of mirror steps: ',tp_obs.raster_p.xstep,$
                     ' arcsecs',form='(a,i3,a)')

   if tp_obs.raster_p.detector eq 'G' then begin
      printf,lun,string('Size of slit steps:   ',tp_obs.raster_p.ystep,$
                        ' arcsecs',form='(a,i3,a)')
   endif

   printf,lun,string('Number of mirror locations: ',tp_obs.raster_p.nx,$
                      form='(a,i3)')

   if tp_obs.raster_p.detector eq 'G' then begin
      printf,lun,string('Number of slit locations:   ',tp_obs.raster_p.ny,$
                      form='(a,i3)')
   endif

   printf,lun,'  '
   sid = strtrim(string(tp_obs.raster_v.ras_var,form='(i5)'),2)
   printf,lun,'Raster VARIATION ID: ',sid,'  (',$
               strtrim(tp_obs.raster_v.rv_desc,2),')' 

   et = string(tp_obs.raster_v.exptime,form='(f7.1)')
   printf,lun,'Exposure time:      '+strtrim(et,2)+' secs'


   optnum = ['0','8','16','32','64','126','256','512']
   scheme = ["","Straight copy","Truncate to 12 bits","VBWL Blocksize ",$
            "","","Sum over lines","Sum over windows","Sum "]

 
   if tp_obs.raster_v.comp_id gt 0 then begin
      text = 'Compression scheme: '+scheme(tp_obs.raster_v.comp_id)

      if tp_obs.raster_v.comp_id eq 8 then begin
         text = text + trim(tp_obs.raster_v.comp_opt) + ' rows.'
      endif

      if tp_obs.raster_v.comp_id eq 3 then begin
         if tp_obs.raster_v.comp_opt eq 0 then  begin
            text = text + '  Default option'
         endif else begin
            text = text + '  option: Blocksize ' + optnum(tp_obs.raster_v.comp_opt)
         endelse
      endif
   endif else begin
      text = 'Compression scheme: Not defined'
   endelse
   printf,lun,text

   orient = ['Row','Column']
   map = [' ',' ','Normal','Accumulate']
   if tp_obs.raster_p.detector eq 'N' then begin
      printf,lun,'VDS readout orientation: ',$
                  orient(tp_obs.raster_v.vds_orient)

      printf,lun,'VDS mapping mode: ',map(tp_obs.raster_v.vds_map)
   endif

   tlm = ['Low','Medium','High']
   n = where(strmid(tlm,0,1) eq tp_obs.raster_v.tel_rate)
   if n(0) eq -1 then n = 1 else n=n(0)
   printf,lun,'Telemetry required: ',tlm(n)

   dur = string(tp_obs.raster_v.duration,form='(f9.1)')
   printf,lun,'Raster duration:    ', + strtrim(dur,2) + ' seconds'

   printf,lun,'Raster usable?:     ',tp_obs.raster_v.usable

endif

;
;  LINE LIST details
;  =========
;

if want_all or line then begin
   printf,lun,' '
   printf,lun,' '
   printf,lun,'                       Line list details'
   printf,lun,'                       ================='
   printf,lun,' '
   sid = strtrim(string(tp_obs.line_list.ll_id,form='(i5)'),2)
   printf,lun,'Line list ID: ',sid,'  (',$
               strtrim(tp_obs.line_list.ll_desc,2),')' 

   nlines = tp_obs.line_list.n_lines
   nlc = strtrim(string(nlines,form='(i5)'),2)
   printf,lun,'Number of lines: '+nlc

   if nlines gt 0 then begin
      printf,lun,'    Wavelength          Line name      Order     Band     Pixel'
      for j=0,nlines-1 do begin
         lname = tp_obs.line_list2(j).linename
         if strlen(lname) lt 12 then lname=strpad(lname,12,/after)
         printf,lun,tp_obs.line_list2(j).wavelnth,$
               '           ',lname,' ',$
               tp_obs.line_list2(j).gr_order,$
               tp_obs.line_list2(j).waveband,$
               tp_obs.line_list2(j).cen_pix,$
               format='(6x,f6.2,a,a12,a,5x,i1,9x,i1,6x,i4)'
      endfor
   endif
endif


;
;  DATA WINDOWS details
;  ============
;

if want_all or dwin then begin
   printf,lun,' '
   printf,lun,' '
   printf,lun,'                       Data windows details'
   printf,lun,'                       ===================='
   printf,lun,' '
   sid = strtrim(string(tp_obs.data_win.dw_id,form='(i5)'),2)
   printf,lun,'Data window ID: ',sid,'  (',$
               strtrim(tp_obs.data_win.dw_desc,2),')' 

   if tp_obs.raster_p.detector eq 'N' then begin
      if tp_obs.data_win.vds_back then begin
         printf,lun,'VDS background windows:  Included'
      endif else begin
         printf,lun,'VDS background windows:  NOT included'
      endelse
   endif

   nwin = tp_obs.data_win.n_windows
   printf,lun,string('Total number of data windows: ',nwin,form='(a,i2)')

   printf,lun,' '

   if nwin gt 0 then begin
      printf,lun,'   Window       Name           Xstart    Ystart'+$
                 '   Xlength     Ylength'
      for i=0,nwin-1 do begin
         if tp_obs.data_win2(i).win_name eq ' ' then begin
            text = 'No title'
         endif else begin
            text = strmid(tp_obs.data_win2(i).win_name,0,10)+'....'
         endelse
         if strlen(text) lt 14 then text = strpad(text,14,/after)
         printf,lun,tp_obs.data_win2(i).win_ind,text,$
                    tp_obs.data_win2(i).win_def,$
                    form='(4x,i2,8x,a14,4x,4(i4,6x))'
      endfor
   endif
endif


;
;  close file and print it
;
free_lun, lun
com = 'lpr tplan_setup.list'
spawn, com

end
