;+
; Project     : SOHO - CDS     
;                   
; Name        : TP_PLOT_DEW
;               
; Purpose     : Plot the data extraction windows in a raster.
;               
; Explanation : Plots the data extraction windows used in a raster for
;               illustrative purposes.  Works from the observation 
;               definition structure used in TPLAN and can be run separately
;               from the planning tool TPLAN.
;               
; Use         : IDL> tp_plot_dew, obs_str, /hard, /soft 
;    
; Inputs      : obs_str  - a valid TPLAN data structure.
;                   
; Opt. Inputs : None
;               
; Outputs     : Just plots the results.
;               
; Opt. Outputs: None
;               
; Keywords    : hard - plot only VDS hardware windows
;               soft - plot only the true data extraction windows
;
; Calls       : None
;
; Common      : None
;               
; Restrictions: None
;               
; Side effects: None
;               
; Category    : Planning, technical
;               
; Prev. Hist. : None
;
; Written     : C D Pike, RAL, 13-Jul-94
;               
; Modified    : Fix bug for hardcopy plotting.  CDP, 20-Oct-94
;               Changed display of VDS hardware windows.  CDP, 4-Jan-95
;               Incorporate GIS band reversal in window definition.
;                                                         CDP, 31-May-95
;               Flip VDS display to reflect real orientation of CCD.
;                                                         CDP, 16-Oct-95
;
; Version     : Version 5, 16-Oct-95
;-            

pro tp_plot_dew, input, soft=soft, hard=hard

;
;  common for variables associated with widget mode
;
@tplan_com


;
;  record plot setup, assumed on entry to be that for the main plot
;  area
;
savep = !p  & savex = !x & savey = !y

;
;  retrieve the soft window limits
;
w = input.data_win2.win_def

;
;  and the VDS hard window limits
;
vdsw = input.vds_win.vwin_def


;
;  error returns
;
if n_elements(w) le 1 then return

n = where(w(3,*) gt 0)
if n(0) eq -1 then begin
   return
endif

;
;  number of soft Data Extraction Windows to handle
;
numdew = n_elements(n)

;
;  act according to instrument
;
ins  = input.raster_p.detector

case ins of
 'G':  begin
          if !d.name ne 'PS' then begin
             if not xregistered('tp_display_dew') then begin
                window,xs=1100,ys=500, title='CDS GIS Raster windows'
             endif else begin
                widget_control, dew_image,get_val=wind
                wset,wind
             endelse
          endif
          !p.multi=[0,2,2]
          for i=0,3 do begin
             xst = i*2048
             xe = xst+2047
             plot,[xst,xe],[0,1],/nodata,xstyle=5,ystyle=5,$
                                 title='GIS detector '+string(i+1,form='(i1)') 
             axis,xaxis=0,xstyle=1,xtit='Pixels',chars=1.4
             oplot,[xe,xe,xst,xst],[0,1,1,0]
             for j=0,numdew-1 do begin
                if w(0,j) ge xst and w(0,j) le xe then begin
;
;  if this reversal is not right then kill this line and replace switched_xst
;  2 lines below by w(0,j)
;
                   switched_xst = xe - (w(0,j)+w(2,j)) + 1 +xst
                   nwidth = w(2,j)
                   fill_box,switched_xst+nwidth/2,1.0,nwidth,6,/border
                endif
             endfor
          endfor
       end
 'N':  begin
          n = where(vdsw(3,*) gt 0)
          if n(0) eq -1 then begin
            return
          endif
          numvdsw = n_elements(n)

;
;  which VDS windows wanted? Hard, Soft or both?
;
;  were soft windows required either as default or specifically requested?
;
          if (not keyword_set(hard) and not keyword_set(soft)) or $
              keyword_set(soft) then begin

             if !d.name ne 'PS' then begin
                if not xregistered('tp_display_dew') then begin
                   window,xs=512,ys=512,xpos=0,ypos=512,$
                                        title='CDS VDS Data Extraction Windows'
                endif else begin
                   widget_control, dew_image,get_val=wind
                   wset,wind
                endelse
             endif
         
             plot,[0,1023],[0,1023],/nodata,xstyle=1,ystyle=1,$
                         chars=1.4,yrange=[1023,0],$
                         xtit='CCD columns',ytit='CCD rows'
                     
             oplot,[0,1023],[511,511],line=2
             xyouts,400,675,'NIS1 - 300A band',chars=1.5
             xyouts,400,350,'NIS2 - 500A band',chars=1.5

             for i=0,numdew-1 do begin
                nwidth = w(2,i)
                fill_box,w(0,i)+nwidth/2,w(1,i)+w(3,i),nwidth,6,$
                            /border,ymin=w(1,i)
             endfor
          endif
;
;  were hard windows required either as default or specifically requested?
;
          if (not keyword_set(hard) and not keyword_set(soft)) or $
              keyword_set(hard) then begin


             if keyword_set(hard) then begin
                if !d.name ne 'PS' then begin
                   if not xregistered('tp_display_dew') then begin
                      window,xs=512,ys=512,xpos=0,ypos=512,$
                                title='CDS VDS Hardware Extraction Windows'
                   endif else begin
                      widget_control, dew_image,get_val=wind
                      wset,wind
                   endelse
                endif
                plot,[0,1023],[0,1023],/nodata,xstyle=1,ystyle=1,chars=1.4,$
                          yrange=[1023,0],xtit='CCD columns',ytit='CCD rows'
             endif
             oplot,[511,511],[0,1023],line=1
             oplot,[0,1023],[511,511],line=1
             for i=0,numvdsw-1 do begin
                nwidth = vdsw(2,i)
                fill_box,vdsw(0,i)+nwidth/2,vdsw(1,i)+vdsw(3,i),nwidth,0,$
                         ymin=vdsw(1,i)
             endfor
          endif
       end   
 else:
endcase

;
;  return plot set up to standard
;
if !d.name ne 'PS' then begin
   if xregistered('tp_display_dew') then begin
      widget_control, image_area, get_val=wind
      wset,wind
      !p = savep & !x = savex & !y = savey
   endif
endif 


end
