;+
; Project     : SOHO - CDS     
;                   
; Name        : TP_WRT_DWIN()
;               
; Purpose     : Store the data extraction definition into the database.
;               
; Explanation : Transfers from the mk_raster internal database to the one
;               required by the database access routine and add the
;               definition to the database if it is new. Returns the ID
;               of the data extraction window.
;               
; Use         : id = tp_wrt_dwin(tp_obs, msg_widg)
;    
; Inputs      : tp_obs    - the standard TPLAN structure.
;               msg_widg  - widget ID for messages
;               flag_state - indicates inclusion or not of flag windows 
;               
; Opt. Inputs : None
;               
; Outputs     : Returns ID of data extraction window ( negative if a problem)
;               
; Opt. Outputs: None
;               
; Keywords    : None
;
; Calls       : ADD_DATAWIN()
;
; Common      : None
;               
; Restrictions: Only for internal TPLAN use.
;               
; Side effects: Data extraction window database could be updated.
;               
; Category    : Planning, technical
;               
; Prev. Hist. : None
;
; Written     : C D Pike, RAL, 25-Aug-94
;               
; Modified    : Update to check database contents first.  CDP, 13-Oct-94
;               Allow Line list ID to be zero.  CDP. 22-Feb-95
;               Added win_flag tag to structure. CDP, 26-Jun-95
;               Include flag window handling.    CDP, 10-Nov-95
;               Insert xack warning when entry already exists.  CDP, 17-Oct-96
;
; Version     : Version 6, 17-Oct-96
;-            

function tp_wrt_dwin, tp_obs, msg_widg, flag_state

;
;  how many lines involved
;
num = tp_obs.data_win.n_windows
if num le 0 then return,-1

;
;  real number depends whether flag windows are involved or not
;
if flag_state eq 0 then begin
   n = where(tp_obs.data_win2(0:num-1).win_flag eq 0, count)
   if count gt 0 then use_num = n else return,-1
endif else begin
   use_num = indgen(num)
endelse



;
;  create the structures needed for database access
;

wins = {cds_windows,  $
         win_name: 'string ',$
         win_def:  [0,0,0,0],$
         win_flag: 0}
wins = replicate(wins,n_elements(use_num))


db_dw = {dw_id: 0, $
         detector: ' ',$
         dw_desc:  ' ',$
         w_width:    0,$
         w_height:   0,$
         vds_back:   0,$
         ll_id:      0,$
         ll_desc:  ' ',$
         n_lines:    0,$
         wins:    wins}

;
;  load the details
;
db_dw.detector = tp_obs.data_win.detector
db_dw.dw_desc  = tp_obs.data_win.dw_desc
db_dw.w_width  = tp_obs.data_win2(0).win_def(2)
db_dw.w_height = tp_obs.data_win2(0).win_def(3)
db_dw.vds_back = tp_obs.data_win.vds_back
db_dw.ll_id    = tp_obs.data_win.ll_id
db_dw.ll_desc  = tp_obs.line_list.ll_desc
db_dw.n_lines  = tp_obs.line_list.n_lines


for n=0,n_elements(use_num)-1 do begin
   db_dw.wins(n).win_name  = tp_obs.data_win2(use_num(n)).win_name  
   db_dw.wins(n).win_def   = tp_obs.data_win2(use_num(n)).win_def
   db_dw.wins(n).win_flag  = tp_obs.data_win2(use_num(n)).win_flag
endfor


;
;  check if this definition already exists
;

status = chk_datawin(db_dw)
if status ge 0 then begin
   widget_control, msg_widg, /append,$
                   set_val= string('Data window entry already exists ',$
                                   'in database. ID = ',status, form='(2a,i3)')
   tp_obs.data_win.dw_id  = status
   tp_obs.data_win2.dw_id = status
   tp_obs.raster_v.dw_id  = status

   get_datawin,status,temp
   temp = temp.dw_desc
   if temp ne db_dw.dw_desc then begin
      bell,3
      xack,'Datawin specification already exists with description <'+strtrim(temp,2)+'>',$
            xoff=500,yoff=500,group=base,/modal
      tp_obs.data_win.dw_desc  = temp
   endif


   return, status
endif


;
;  try and write this data extraction list to the database
;
errmsg = ''
help,db_dw,/str
print,db_dw.wins.win_name
print,db_dw.wins.win_def

status = add_datawin(db_dw, errmsg=errmsg)

if errmsg ne '' then begin
   widget_control, msg_widg,set_val=errmsg,/append
   status = -1
endif else begin
   widget_control, msg_widg, /append, $
                   set_val=string('New data windows entry created with ID = ',$
                                   db_dw.dw_id)
   tp_obs.data_win.dw_id   = db_dw.dw_id
   tp_obs.data_win2.dw_id  = db_dw.dw_id
   tp_obs.raster_v.dw_id   = db_dw.dw_id
endelse   

return, status


end   
