;+
; Project     : SOHO - CDS     
;                   
; Name        : TPLAN_STRUCT
;               
; Purpose     : Define the structures used in the technical planning.
;               
; Explanation : Sets the static definition of the structures to be compiled
;               by the technical planning procedure (tplan) for output
;               to and communication with other routines.
;               
; Use         : IDL>  tplan_struct
;    
; Inputs      : None
;               
; Opt. Inputs : None
;               
; Outputs     : None
;               
; Opt. Outputs: None
;               
; Keywords    : None
;
; Calls       : None
;               
; Restrictions: See note re definition differences in DEW and VDS windows.
;               
; Side effects: None
;              
; Category    : Planning, Technical
;               
; Prev. Hist. : An update of original tp_def_struct
;
; Written     : C D Pike, RAL, 8-Jul-94
;               
; Modified    : Update to follow database definitions.   CDP, 19-Aug-94
;               Added VDS background window definitions. CDP, 20-Oct-94
;               Update data_win2 structure to add flags. CDP, 26-Jun-95
;               Update raster variation with ief_id.     CDP, 09-Nov-95
;
; Version     : Version 5, 09-Nov-95
;-                       

pro tplan_struct

;
;  need to know what maximum number of windows is
;
@tplan_com

;
;  the line window structure
;
dummy  = {line_window_rec,$
         x_pos:   -1,     $           ;  line centre in pixels
         lam:    0.0,     $           ;  line wavelength
         order:    0,     $           ;  spectral order
         inten:  0.0,     $           ;  intensity (c/s)
         y_pos:    0,     $           ;  line y position (0 or 128)
         left:     0,     $           ;  left edge of line window in pixels
         right:    0,     $           ;  right edge of line window in pixels
         height:   0,     $           ;  window height in arcsec
         id:      ' ',    $           ;  line identification
         flag:     0}                 ;  for use as a flag?

;
;  make room for the maximum allowed number of line windows 
;
lw = replicate({line_window_rec},max_num_lwindow)
lw.x_pos = -1

;
;  Definition of primary raster details
;
dummy1 = {raster_primary, $
         ras_id:     0,   $   ; Raster ID number
         detector: ' ',   $   ; G or N for GIS or NIS
         ras_desc: ' ',   $   ; Description of purpose of raster
         slit_num:   0,   $   ; Slit number
         xstep:      0,   $   ; Step size in X (mirror) direction (arcsec)
         ystep:      0,   $   ; Step size in Y (slit) direction (arcsec)
         nx:         0,   $   ; Number of X locations
         ny:         0}       ; Number of Y locations

;
;  Definition of raster variation
;
dummy2 = {raster_var,       $
         ras_id:     0,     $   ; Raster ID number
         ras_var:    0,     $   ; Raster variation index number
         rv_desc:  ' ',     $   ; Description of raster variation
         exptime:  0.0,     $   ; Exposure time in seconds to millisec accuracy
         ll_id:      0,     $   ; Line list ID
         comp_id:    1,     $   ; Compression method ID
         comp_opt:   0,     $   ; Compression option parameter.
         dw_id:      0,     $   ; Data extraction window list ID
         ief_id:     0,     $   ; IEF ID
         vds_orient: 0,     $   ; VDS readout orientation (0-row, 1-column)
         vds_map:    0,     $   ; VDS mapping mode
         tel_rate: 'M',     $   ; Telemetry rate - L(ow), M(edium) or H(igh)
         duration: 0.0,     $   ; Estimated duration of raster in seconds
         usable:   ' '}         ; Y or N to signal usability of raster.


dummy3 = {linelist,         $
         ll_id:      0,     $   ; Line list ID number
         detector: ' ',     $   ; G or N for GIS or NIS
         ll_desc:  ' ',     $   ; One line description of line list
         n_lines:    0}         ; Number of lines in line list


dummy4 = {linelist2,        $
         ll_id:      0,     $   ; Line list ID
         linename: ' ',     $   ; Line name eg. "He II 304"
         line_ind:   0,     $   ; Line index number within a line list
         wavelnth: 0.0,     $   ; Wavelength in Angstroms
         gr_order:   0,     $   ; Grating order
         cen_pix:    0,     $   ; Central detector pixel
         waveband:   0}         ; Wavelength band. 1-2 for NIS, 1-4 for GIS.

dummy5 = {datawin,          $
         dw_id:      0,     $   ; Data window list ID number
         detector: ' ',     $   ; Either G or N
         dw_desc:  ' ',     $   ; Description of the data window list
         ll_id:      0,     $   ; Parent line list ID (zero if no line list)
         vds_back:   0,     $   ; Background windows present (0-no, 1-yes)
         n_windows:  0}         ; Number of windows in the data window list

dummy6 = {datawin2,         $
         dw_id:      0,     $   ; Data window list ID
         win_ind:    0,     $   ; Window index within data window list
         win_name: ' ',     $   ; Description of window
         win_flag:   0,     $   ; Is window used for flag generation
         win_def: intarr(4)}    ; Window definition [xst,yst,xlen,ylen]

;-----------------------------------------------------------------------------
;
;  note definition of window parameters for datawin and vdswin.  Here the 4
;  values are taken as (xst,yst,xlen,ylen) but for commanding, the VDS hardware
;  window parameters are required as (xst,yst,xend,yend).
;------------------------------------------------------------------------------
;

dummy7 = {vdswin,           $
         dw_id:      0,     $   ; Data window list ID
         vwin_ind:   0,     $   ; Window index within VDS window list
         vwin_def:intarr(4)}    ; Window definition [xst,yst,xlen,ylen]


llw  = replicate({linelist2},max_num_lwindow)
ddw  = replicate({datawin2}, max_num_lwindow)
vvw  = replicate({vdswin},   max_num_lwindow)

dummy8 = {state_rec,            $
          date:  double(0.0),   $  ; TAI in seconds
          vds_read:  1,         $  ; VDS readout mode 4 Quadrants or Others
          vds_pmcp:  0,         $  ; VDS detector voltage parameter
          vds_offset: intarr(2)}   ; Offsets for 2 VDS spectra wrt CCD centre

;
;  VDS background windows
;
dummy9 = {vdsback,win_def: intarr(4)} ; Window definition [xst,yst,xlen,ylen]
vdsb = replicate({vdsback},4)



;
;  the overall definition of an observation (ie raster)
;
tp_obs = {tp_obs_rec,       $
      raster_p:   dummy1,   $     ;  primary raster definition
      raster_v:   dummy2,   $     ;  raster variation definiton
      line_list:  dummy3,   $     ;  Primary line list details
      line_list2: llw,      $     ;  Secondary line list details
      data_win:   dummy5,   $     ;  Primary data window details
      data_win2:  ddw,      $     ;  Secondary data window details
      vds_win:    vvw,      $     ;  VDS hardware windows
      vds_backwin:vdsb,     $     ;  VDS background windows
      state:      dummy8,   $     ;  Instrument state details
      line_w:     lw}             ;  array of line windows structures


tp_obs.line_w.x_pos = -1
chosen_frast = 0
raster_p_std = 0
raster_v_std = 0
raster_ll_std = 0
key_code_ok = 0
current_zone = 4

end
