	FUNCTION ADD_RES_TYPE, TYPE, ERRMSG=ERRMSG, CHECK_ONLY=CHECK_ONLY
;+
; Project     :	SOHO - CDS
;
; Name        :	ADD_RES_TYPE()
;
; Purpose     :	Adds a resource type name to the database
;
; Explanation :	This procedure adds a resource type name such as "DSN_CONTACT"
;		to the "resource_type" database.  The ID number for the
;		resource type is assigned.
;
; Use         :	Result = ADD_RES_TYPE( TYPE )
;
;		IF ADD_RES_TYPE( TYPE ) EQ 0 THEN ...
;
; Inputs      :	TYPE = A string parameter containing the name of the resource
;		       type.  Can be up to 30 characters long.
;
; Opt. Inputs :	None.
;
; Outputs     :	The result of the function is the ID number assigned to the
;		resource type.  If not successful, then 0 is returned instead.
;
; Opt. Outputs:	None.
;
; Keywords    :	
;       ERRMSG    = If defined and passed, then any error messages will be
;                   returned to the user in this parameter rather than
;                   depending on the MESSAGE routine in IDL.  If no errors are
;                   encountered, then a null string is returned.  In order to
;                   use this feature, ERRMSG must be defined first, e.g.
;
;                       ERRMSG = ''
;                       Result = ADD_RES_TYPE( ERRMSG=ERRMSG, ... )
;                       IF ERRMSG NE '' THEN ...
;
;	CHECK_ONLY= If this keyword is set, then the routine only checks to see
;		    if the proposed entry is valid, but does not attempt to
;		    place it in the database.
;
; Calls       :	DATATYPE, DBOPEN, DB_INFO, DBBUILD, DBCLOSE, DBFIND, DBMATCH
;
; Common      :	None.
;
; Restrictions:	Only this routine can be used to add resource types to the
;		database.  Modifying the database by hand could corrupt its
;		integrity.
;
;		!PRIV must be 2 or greater to use this routine.
;
; Side effects:	Once a reference type has been added to the database, it must
;		never be removed.
;
; Category    :	Planning, Databases.
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 31 March 1995
;
; Modified    :	Version 1, William Thompson, GSFC, 3 April 1995
;
; Version     :	Version 1, 3 April 1995
;-
;
	ON_ERROR, 2
;
;  Check the input parameters
;
        IF N_PARAMS() NE 1 THEN BEGIN
		MESSAGE = 'Syntax:  Result = ADD_RES_TYPE(TYPE)'
		GOTO, HANDLE_ERROR
	ENDIF
;
	IF DATATYPE(TYPE,1) NE 'String' THEN BEGIN
           MESSAGE = 'TYPE must be a character string'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(TYPE) NE 1 THEN BEGIN
           MESSAGE = 'TYPE must be a scalar'
           GOTO, HANDLE_ERROR
	END ELSE IF STRLEN(TYPE) GT 30 THEN BEGIN
           MESSAGE = 'TYPE must be no more than 30 characters long'
           GOTO, HANDLE_ERROR
	ENDIF
;
;  Convert to uppercase, and remove any leading or trailing blanks.
;
	RES_TYPE = STRUPCASE(STRTRIM(TYPE,2))
;
;  Make sure that the user has privilege to write into the database.
;
	IF !PRIV LT 2 THEN BEGIN
           MESSAGE = '!PRIV must be 2 or greater to write into the database'
           GOTO, HANDLE_ERROR
	ENDIF
;
;  Open the database for write access.
;
	DBOPEN, 'resource_type', 1
;
;  Check for an entry with the same description.
;
	ENTRY = DBMATCH('RES_TYPE', RES_TYPE, /FULLSTRING)
	IF ENTRY(0) GT 0 THEN BEGIN
           MESSAGE = 'Resource type with same name already exists'
           GOTO, HANDLE_ERROR
	ENDIF
;
;  Find the largest resource type ID number currently in the database, and add
;  one to it.
;
	N_ENTRIES = DB_INFO('entries','resource_type')
	IF N_ENTRIES EQ 0 THEN TYPE_ID = 1 ELSE BEGIN
		DBEXT, -1, 'type_id', TYPE_IDS
		TYPE_ID = MAX(TYPE_IDS) + 1
	ENDELSE
;
;  If the CHECK_ONLY keyword was set, then return now as if the routine were
;  successful.
;
	IF KEYWORD_SET(CHECK_ONLY) THEN GOTO, FINISH
;
;  Append the data to the opened database.
;
	DBBUILD, TYPE_ID, RES_TYPE, STATUS=STATUS
	IF STATUS EQ 0 THEN BEGIN
           MESSAGE = 'Write to resource type database was not successful'
           GOTO, HANDLE_ERROR
        ENDIF
	GOTO, FINISH
;
;  Handle any errors encountered.
;
HANDLE_ERROR:
	IF N_ELEMENTS(ERRMSG) NE 0 THEN ERRMSG = 'ADD_RES_TYPE: ' + MESSAGE $
		ELSE MESSAGE, MESSAGE 
	TYPE_ID = 0
;
;  Close the database, and return whether the routine was successful or not.
;
FINISH:
	DBCLOSE
;
	RETURN, TYPE_ID
	END
