	PRO GET_SOHO_DET, INSTRUMENT, DATE, OBS, ERRMSG=ERRMSG
;+
; Project     :	SOHO - CDS
;
; Name        :	GET_SOHO_DET
;
; Purpose     :	Extracts a SoHO detailed science plan observation
;
; Explanation :	This routine extracts parameters which describe the SoHO
;		detailed science plan at a particular point in time for a given
;		instrument.
;
; Use         :	GET_SOHO_DET, DATE, OBS
;
; Inputs      :	INSTRUMENT = The name or code value for the instrument to
;			     retrieve the plan entry for.
;		DATE	   = The date/time value that the user wishes the
;			     science plan record for.  This can be in any of
;			     the standard CDS time formats.
;
; Opt. Inputs :	None.
;
; Outputs     :	OBS	= Structure containing the science plan record.  It
;			  contains the following tags:
;
;			STRUCT_TYPE  = The character string 'SOHO-DET-PLAN'.
;			INSTRUME     = Single letter code specifying the
;				       instrument.
;			DATE_OBS     = Date/time of beginning of observation,
;				       in TAI format
;			DATE_END     = Date/time of end of observation, in TAI
;				       format
;			SCI_OBJ      = Science objective
;			SCI_SPEC     = Specific science objective
;			OBS_PROG     = Observing program that will be run
;			PROG_ID	     = Program ID, linking one or more studies
;				       together
;			CMP_NO	     = Campaign number
;			OBJECT	     = Code for object planned to be observed
;			OBJ_ID	     = Object identification
;			XCEN	     = Center(s) of instrument field-of-view
;				       along X axis
;			YCEN	     = Center(s) of instrument field-of-view
;				       along Y axis
;			ANGLE	     = Rotation angle(s) of field-of-view
;				       relative to solar north
;			IXWIDTH	     = Width(s) of field-of-view in instrument
;				       X direction
;			IYWIDTH	     = Width(s) of field-of-view in instrument
;				       Y direction
;			DISTURBANCES = Description of any disturbances
;			JITTER_LIMIT = Jitter limit in 1/10 arcsec units
;
;		If no entry is found, then a simpler structure is returned with
;		INSTRUME set to the null string.
;
; Opt. Outputs:	None.
;
; Keywords    :
;       ERRMSG    = If defined and passed, then any error messages will be
;                   returned to the user in this parameter rather than
;                   depending on the MESSAGE routine in IDL.  If no errors are
;                   encountered, then a null string is returned.  In order to
;                   use this feature, ERRMSG must be defined first, e.g.
;
;                       ERRMSG = ''
;                       GET_SOHO_DET, ERRMSG=ERRMSG, ... 
;                       IF ERRMSG NE '' THEN ...
;
;
; Calls       :	DATATYPE, DBOPEN, DBFIND, DBEXT, DBCLOSE, TRIM, GET_INSTRUMENT
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Category    :	Planning, Databases.
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 12 April 1995
;
; Modified    :	Version 1, William Thompson, GSFC, 12 April 1995
;		Version 2, William Thompson, GSFC, 28 April 1995
;			Returns an error string when no entry is found.
;		Version 3, William Thompson, GSFC, 22 May 1995
;			Changed way DBFIND is called, to speed up.
;			Modified to take into accounted DELETED field.
;
; Version     :	Version 3, 22 May 1995
;-
;
	ON_ERROR, 2
;
;  Initialize the output structure, in case any errors occur.
;
	OBS = {INSTRUME: ''}
;
;  Check the number of parameters.
;
        IF N_PARAMS() NE 3 THEN BEGIN
           MESSAGE = 'Syntax:  GET_SOHO_DET, INSTRUMENT, DATE, OBS'
	   GOTO, HANDLE_ERROR
        ENDIF
;
;  Check the input parameters.
;
	IF DATATYPE(INSTRUMENT,1) NE 'String' THEN BEGIN
           MESSAGE = 'INSTRUMENT must be a character string'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(INSTRUMENT) GT 1 THEN BEGIN
           MESSAGE = 'INSTRUMENT must be a scalar'
           GOTO, HANDLE_ERROR
	ENDIF
;
	IF N_ELEMENTS(DATE) NE 1 THEN BEGIN
           MESSAGE = 'DATE must be a scalar'
           GOTO, HANDLE_ERROR
	ENDIF
;
;  Make sure the instrument parameter is meaningful.
;
	GET_INSTRUMENT, INSTRUMENT, INS_DESC
	IF INS_DESC.CODE EQ '' THEN BEGIN
           MESSAGE = 'Instrument ' + INSTRUMENT + ' not recognized'
           GOTO, HANDLE_ERROR
	ENDIF
;
;  Convert the date to TAI format.
;
	IF DATATYPE(DATE,1) EQ 'Double' THEN TAI = DATE ELSE	$
		TAI = UTC2TAI(DATE)
;
;  Open the database.
;
	DBOPEN, 'soho_details'
;
;  Search for entries with dates spanning that given by the user.
;
	ENTRIES = DBFIND('DATE_END>' + TRIM(TAI,'(F15.3)') + ',DATE_OBS<' + $
		TRIM(TAI,'(F15.3)') + ',INSTRUME=' + INS_DESC.CODE +	$
		',DELETED=N', /SILENT)
;
;  If no entries were found, then return immediately.
;
	IF !ERR EQ 0 THEN BEGIN
		MESSAGE = 'No SOHO details entry found meeting search criteria'
		GOTO, HANDLE_ERROR
	ENDIF
;
;  Extract the entries.
;
	DBEXT, ENTRIES, 'instrume,date_obs,date_end,sci_obj,sci_spec',$
		INSTRUME, DATE_OBS, DATE_END, SCI_OBJ, SCI_SPEC
	DBEXT, ENTRIES, 'obs_prog,prog_id,cmp_no,object,obj_id',	$
		OBS_PROG, PROG_ID, CMP_NO, OBJECT, OBJ_ID
	DBEXT, ENTRIES, 'xcen,ycen,angle,ixwidth,iywidth',		$
		XCEN, YCEN, ANGLE, IXWIDTH, IYWIDTH
	DBEXT, ENTRIES, 'disturbances,jitter_limit',			$
		DISTURBANCES, JITTER_LIMIT
;
;  If more than one entry has been found, then use the one with the closest
;  start time to the requested time.  This avoids problems with times which are
;  right on the boundaries between entries.
;
	DIFF = ABS(DATE_OBS - TAI)
	I = (WHERE(DIFF EQ MIN(DIFF)))(0)
;
;  Define the output structure.
;
	OBS = { STRUCT_TYPE: 'SOHO-DET-PLAN',	$
		INSTRUME: INSTRUME(I),		$
		DATE_OBS: DATE_OBS(I),		$
		DATE_END: DATE_END(I),		$
		SCI_OBJ: SCI_OBJ(I),		$
		SCI_SPEC: SCI_SPEC(I),		$
		OBS_PROG: OBS_PROG(I),		$
		PROG_ID: PROG_ID(I),		$
		CMP_NO: CMP_NO(I),		$
		OBJECT: OBJECT(I),		$
		OBJ_ID: OBJ_ID(I),		$
		XCEN: XCEN(I),			$
		YCEN: YCEN(I),			$
		ANGLE: ANGLE(I),		$
		IXWIDTH: IXWIDTH(I),		$
		IYWIDTH: IYWIDTH(I),		$
		DISTURBANCES: DISTURBANCES(I),	$
		JITTER_LIMIT: JITTER_LIMIT(I)}
	GOTO, FINISH
;
;  Error handling point.
;
HANDLE_ERROR:
	IF N_ELEMENTS(ERRMSG) NE 0 THEN ERRMSG = 'GET_SOHO_DET: ' + MESSAGE $
		ELSE MESSAGE, MESSAGE, /CONTINUE
;
;  Close the database, and return.
;
FINISH:
	DBCLOSE
;
	RETURN
	END
