	PRO LIST_INST_DLYD, START, EEND, RES, N_FOUND, ERRMSG=ERRMSG,	$
		INSTRUMENT=INSTRUMENT
;+
; Project     :	SOHO - CDS
;
; Name        :	LIST_INST_DLYD
;
; Purpose     :	List the instrument delayed command times for a given period
;
; Explanation :	Extracts all the instrument delayed command upload times in the
;		input time range.
;
; Use         :	LIST_INST_DLYD, START, END, RES, N_FOUND
;
; Inputs      :	START, END = The range of date/time values to use in searching
;			     the database.  Can be in any standard CDS time
;			     format.
;
; Opt. Inputs :	None.
;
; Outputs     :	RES	= A structure variable containing the following tags
;			  for each instrument delayed command period:
;
;			INSTRUME   = Single letter code specifying the
;				     instrument.
;			EARLIEST = The start time of the reserved time, in
;				     TAI format.
;			LATEST   = The end time of the reserved time, in TAI
;				     format.
;			NUM_CMDS   = The expected average number of OBDH block
;				     commands per minute between the start time
;				     and end time.
;
;		N_FOUND	= Number of instrument delayed command periods found.
;
; Opt. Outputs:	None.
;
; Keywords    :	
;	INSTRUMENT= If passed, then only the instrument delayed command periods
;		    for the specified instrument will be returned.  Can be
;		    either a code letter or the full name.  (See
;		    GET_INSTRUMENT.)
;
;       ERRMSG    = If defined and passed, then any error messages will be
;                   returned to the user in this parameter rather than
;                   depending on the MESSAGE routine in IDL.  If no errors are
;                   encountered, then a null string is returned.  In order to
;                   use this feature, ERRMSG must be defined first, e.g.
;
;                       ERRMSG = ''
;                       LIST_INST_DLYD, ERRMSG=ERRMSG, ... 
;                       IF ERRMSG NE '' THEN ...
;
;
; Calls       :	DATATYPE, DBOPEN, DBFIND, DBEXT, DBCLOSE, TRIM, GET_INSTRUMENT
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	If the number of planned instrument delayed command periods
;		found is zero, then the output parameter RES is not modified.
;
; Category    :	Planning, Database.
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 6 March 1996
;
; Modified    :	Version 1, William Thompson, GSFC, 6 March 1996
;		Version 2, William Thompson, GSFC, 13 March 1996
;			Changed order of parameters in output structure.
;
; Version     :	Version 2, 13 March 1996
;-
;
	ON_ERROR, 2
;
;  Check the number of parameters.
;
        IF N_PARAMS() LT 4 THEN BEGIN
           MESSAGE = 'Syntax:  LIST_INST_DLYD, START, END, RES, N_FOUND'
	   GOTO, HANDLE_ERROR
        ENDIF
;
;  Check the input parameters.
;
	N_FOUND = 0
	IF N_ELEMENTS(START) NE 1 THEN BEGIN
           MESSAGE = 'START must be a scalar'
           GOTO, HANDLE_ERROR
	END ELSE IF N_ELEMENTS(EEND) NE 1 THEN BEGIN
           MESSAGE = 'END must be a scalar'
           GOTO, HANDLE_ERROR
	ENDIF
;
;  If the INSTRUMENT keyword was passed, then determine which instrument this
;  refers to.
;
	IF N_ELEMENTS(INSTRUMENT) EQ 1 THEN BEGIN
		GET_INSTRUMENT, INSTRUMENT, INS, ERRMSG=ERRMSG
		IF N_ELEMENTS(ERRMSG) NE 0 THEN IF ERRMSG(0) NE '' THEN BEGIN
			MESSAGE = ERRMSG
			GOTO, HANDLE_ERROR
		ENDIF
	ENDIF
;
;  Convert the input dates to TAI format.
;
	IF DATATYPE(START,1) EQ 'Double' THEN TAI_START = START ELSE	$
		TAI_START = UTC2TAI(START)
	IF DATATYPE(EEND,1) EQ 'Double' THEN TAI_END = EEND ELSE	$
		TAI_END = UTC2TAI(EEND)
;
;  Open the instrument delayed command database.
;
	DBOPEN, 'inst_delayed_cmd'
;
;  Find all the entries in the requested time range.
;
	TEST = 'LATEST>' + TRIM(TAI_START,'(F15.3)') + ',EARLIEST<' + $
		TRIM(TAI_END,'(F15.3)')
	IF N_ELEMENTS(INSTRUMENT) EQ 1 THEN TEST = TEST +	$
		',INSTRUME=' + INS.CODE
;
	ENTRIES = DBFIND(TEST, /SILENT)
	IF N_ELEMENTS(ENTRIES) GT 1 THEN ENTRIES = ENTRIES(UNIQ(ENTRIES))
	IF ENTRIES(0) LE 0 THEN BEGIN
		N_FOUND = 0
		GOTO, FINISH
	ENDIF
	N_FOUND = N_ELEMENTS(ENTRIES)
;
;  Extract the requested entries, sorted by start times.
;
	ENTRIES = DBSORT(ENTRIES,'earliest')
	DBEXT, ENTRIES, 'instrume,earliest,latest,num_cmds',	$
		INSTRUME, EARLIEST, LATEST, NUM_CMDS
;
;  Remove any entries that only touch the requested time range.
;
	W = WHERE((LATEST NE TAI_START) AND (EARLIEST NE TAI_END), N_FOUND)
	IF N_FOUND EQ 0 THEN GOTO, FINISH
;
;  Define the structure that the data will be returned in.
;
	RES = { EARLIEST: 0.0D0,	$
		LATEST: 0.0D0,		$
		INSTRUME: 'S',		$
		NUM_CMDS: 0}
;
	IF N_FOUND EQ 1 THEN BEGIN
		RES.INSTRUME	= INSTRUME(W(0))
		RES.EARLIEST	= EARLIEST(W(0))
		RES.LATEST	= LATEST(W(0))
		RES.NUM_CMDS	= NUM_CMDS(W(0))
	END ELSE BEGIN
		RES = REPLICATE(RES, N_FOUND)
		RES.INSTRUME	= INSTRUME(W)
		RES.EARLIEST	= EARLIEST(W)
		RES.LATEST	= LATEST(W)
		RES.NUM_CMDS	= NUM_CMDS(W)
	ENDELSE
;
	GOTO, FINISH
;
;  Handle any errors encountered.
;
HANDLE_ERROR:
	IF N_ELEMENTS(ERRMSG) NE 0 THEN		$
		ERRMSG = 'LIST_INST_DLYD: ' + MESSAGE ELSE MESSAGE, MESSAGE 
;
;  Close the database and return.
;
FINISH:
	DBCLOSE
;
	RETURN
	END
