;---------------------------------------------------------------------------
; Document name: list_other_res.pro
; Created by:    Liyun Wang, GSFC/ARC, August 17, 1995
;
; Last Modified: Thu Aug 17 12:14:13 1995 (lwang@achilles.nascom.nasa.gov)
;---------------------------------------------------------------------------
;
PRO list_other_res, start, eend, resource, n_found, errmsg=errmsg, _extra=extra
;+
; PROJECT:
;       SOHO - CDS/SUMER
;
; NAME:
;       LIST_OTHER_RES
;
; PURPOSE: 
;       List simple resource items that contains only start & stop times
;
; CATEGORY:
;       Planning, Database.
; 
; EXPLANATION:
;       Extracts all the times for any resource items, which contain
;       only START_TIME and END_TIME, in the input time
;       range. Currently the following resource items can be listed
;       through this routine:
;
;          TLM_MDI_M, TLM_MDI_H, TLM_TAPE_DUMP, THROUGHPUT_RCR, 
;          THROUGHPUT_NORCR, SVM_RESERVED, PAYLOAD_RESERVED
;
; SYNTAX: 
;       list_other_res, start, end, resource, n_found, /extra
;
; INPUTS:
;       START, END - The range of date/time values to use in searching
;                    the database.  Can be in any standard CDS time
;                    format.
;
; OPTIONAL INPUTS: 
;       None.
;
; OUTPUTS:
;       RESOURCE - A structure variable containing the following tags
;                  for each planned contact period:
;
;                  START_TIME   = Date/time of beginning of the resource,
;                                 in TAI format
;                  END_TIME     = Date/time of end of the resource,
;                                 in TAI format
;       N_FOUND  - Number of planned contact periods found
;
; OPTIONAL OUTPUTS:
;       None.
;
; KEYWORDS: 
;       EXTRA     - Unambiguous resource keyword (e.g. TAPE_DUMP, MDI_M, COM ...)
;       ERRMSG    - If defined and passed, then any error messages will be
;                   returned to the user in this parameter rather than
;                   depending on the MESSAGE routine in IDL.  If no errors are
;                   encountered, then a null string is returned.  In order to
;                   use this feature, ERRMSG must be defined first, e.g.
;
;                       ERRMSG = ''
;                       LIST_MDI_M, ERRMSG=ERRMSG, ... 
;                       IF ERRMSG NE '' THEN ...
;
; COMMON:
;       None.
;
; RESTRICTIONS: 
;       None.
;
; SIDE EFFECTS:
;       None.
;
; HISTORY:
;       Version 1, August 17, 1995, Liyun Wang, GSFC/ARC. Written
;
; CONTACT:
;       Liyun Wang, GSFC/ARC (Liyun.Wang.1@gsfc.nasa.gov)
;-
;
   ON_ERROR, 2
   
;---------------------------------------------------------------------------
;  Check the number of parameters
;---------------------------------------------------------------------------
   IF N_PARAMS() LT 4 OR (datatype(extra) NE 'STC') THEN BEGIN
      message = 'syntax:  list_other_res, start, end, res_name, n_found, /extra'
      GOTO, handle_error
   END

;---------------------------------------------------------------------------
;  Check the input parameters
;---------------------------------------------------------------------------
   n_found = 0
   IF N_ELEMENTS(start) NE 1 THEN BEGIN
      MESSAGE = 'start must be a scalar'
      GOTO, handle_error
   END ELSE IF N_ELEMENTS(eend) NE 1 THEN BEGIN
      MESSAGE = 'end must be a scalar'
      GOTO, handle_error
   ENDIF

   tags = TAG_NAMES(extra)
   temp = grep('err', tags, index=index)
   IF temp NE '' THEN BEGIN
      ii = WHERE(tags NE temp(0))
      tags = tags(ii)
   ENDIF
   res_name = get_res_name(tags, err=errmsg)
   IF res_name EQ '' THEN BEGIN
      MESSAGE = 'Unsupported resource keyword: '+tags(0)
      GOTO, handle_error
   ENDIF
   
;---------------------------------------------------------------------------
;  Convert the input dates to TAI format
;---------------------------------------------------------------------------
   IF datatype(start, 1) EQ 'Double' THEN tai_start = start ELSE    $
      tai_start = utc2tai(start)
   IF datatype(eend, 1) EQ 'Double' THEN tai_end = eend ELSE        $
      tai_end = utc2tai(eend)
   
;---------------------------------------------------------------------------
;  Open the resource database
;---------------------------------------------------------------------------
   dbopen, 'resource,resource_type'
   
;---------------------------------------------------------------------------
;  Find all the entries in the requested time range
;---------------------------------------------------------------------------
   entries = dbfind('end_time>' + trim(tai_start, '(f15.3)') +	$
                    ',start_time<' + trim(tai_end, '(f15.3)') +	$
                    ',res_type='+res_name, /silent)
   IF !err EQ 0 THEN BEGIN
      n_found = 0
      GOTO, finish
   END ELSE BEGIN
      entries = entries(uniq(entries))
      n_found = N_ELEMENTS(entries)
   ENDELSE
   
;---------------------------------------------------------------------------
;  Extract the requested entries, sorted by tstart times
;---------------------------------------------------------------------------
   entries = dbsort(entries, 'start_time')
   dbext, entries, 'start_time,end_time', start_time, end_time

;---------------------------------------------------------------------------
;  Remove any entries that only touch the requested time range
;---------------------------------------------------------------------------
   w = WHERE((end_time NE tai_start) AND (start_time NE tai_end), n_found)
   IF n_found EQ 0 THEN GOTO, finish
   start_time = start_time(w)
   end_time = end_time(w)

;---------------------------------------------------------------------------
;  Define the structure that the data will be returned in
;---------------------------------------------------------------------------
   resource = { start_time: 0.0d0, end_time: 0.0d0}

   IF n_found EQ 1 THEN BEGIN
      resource.start_time = start_time(0)
      resource.end_time = end_time(0)
   END ELSE BEGIN
      resource = REPLICATE(resource, n_found)
      resource.start_time = start_time
      resource.end_time = end_time
   ENDELSE

   GOTO, finish

;---------------------------------------------------------------------------
;  Handle any errors encountered
;---------------------------------------------------------------------------
handle_error:
   
   IF N_ELEMENTS(errmsg) NE 0 THEN $
      errmsg = 'LIST_OTHER_RES: ' + message $
   ELSE $
      MESSAGE, message
   
;---------------------------------------------------------------------------
;  Close the database and return
;---------------------------------------------------------------------------
finish:
   dbclose
   
   RETURN

END
;---------------------------------------------------------------------------
; End of 'list_other_res.pro'.
;---------------------------------------------------------------------------
