	PRO LIST_SOHO_DET, START, EEND, OBS, N_FOUND, INSTRUMENT=INSTRUMENT, $
               ERRMSG=ERRMSG
;+
; Project     :	SOHO - CDS
;
; Name        :	LIST_SOHO_DET
;
; Purpose     :	List the SoHO detailed plan for a given date range.
;
; Explanation :	Extracts all the SoHO detailed science plan entries which
;		intersect an input time range.
;
; Use         :	LIST_SOHO_DET, START, END, OBS, N_FOUND
;
; Inputs      :	START, END = The range of date/time values to use in searching
;			     the database.  Can be in any standard CDS time
;			     format.
;
; Opt. Inputs :	None.
;
; Outputs     :	OBS	= A structure variable containing the following tags
;			  for each planned observation:
;
;			STRUCT_TYPE  = The character string 'SOHO-DET-LIST'.
;			INSTRUME     = Single letter code specifying the
;				       instrument.
;			DATE_OBS     = Date/time of beginning of observation,
;				       in TAI format
;			DATE_END     = Date/time of end of observation, in TAI
;				       format
;			SCI_OBJ      = Science objective
;			SCI_SPEC     = Specific science objective
;			OBS_PROG     = Observing program that will be run
;			PROG_ID	     = Program ID, linking one or more studies
;				       together
;			CMP_NO	     = Campaign number
;			OBJECT	     = Code for object planned to be observed
;			OBJ_ID	     = Object identification
;			XCEN	     = Center(s) of instrument field-of-view
;				       along X axis
;			YCEN	     = Center(s) of instrument field-of-view
;				       along Y axis
;			ANGLE	     = Rotation angle(s) of field-of-view
;				       relative to solar north
;			IXWIDTH	     = Width(s) of field-of-view in instrument
;				       X direction
;			IYWIDTH	     = Width(s) of field-of-view in instrument
;				       Y direction
;			DISTURBANCES = Description of any disturbances
;			JITTER_LIMIT = Jitter limit in 1/10 arcsec units
;
;		N_FOUND	= Number of planned observations found.
;
; Opt. Outputs:	None.
;
; Keywords    :	
;      INSTRUMENT = If passed, then the name or code value for the
;       	    instrument to retrieve the plan for.  Otherwise,
;		    the plans for all the instruments are retrieved.
;       ERRMSG    = If defined and passed, then any error messages will be
;                   returned to the user in this parameter rather than
;                   depending on the MESSAGE routine in IDL.  If no errors are
;                   encountered, then a null string is returned.  In order to
;                   use this feature, ERRMSG must be defined first, e.g.
;
;                       ERRMSG = ''
;                       LIST_SOHO_DET, ERRMSG=ERRMSG, ... 
;                       IF ERRMSG NE '' THEN ...
;
;
; Calls       :	DATATYPE, DBOPEN, DBFIND, DBEXT, DBCLOSE, TRIM, GET_INSTRUMENT
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	If the number of planned observations found is zero, then the
;		output parameter OBS is not modified.
;
; Category    :	Planning, Database.
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 12 April 1995
;
; Modified    :	Version 1, William Thompson, GSFC, 12 April 1995
;		Version 2, William Thompson, GSFC, 22 May 1995
;			Modified to take into account DELETED field.
;
; Version     :	Version 2, 22 May 1995
;-
;
	ON_ERROR, 2
;
;  Check the number of parameters.
;
        IF N_PARAMS() LT 4 THEN BEGIN
           MESSAGE = 'Syntax:  LIST_SOHO_DET, START, END, OBS, N_FOUND'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN BEGIN
              ERRMSG = MESSAGE
              RETURN
           ENDIF ELSE MESSAGE, MESSAGE
        ENDIF
;
;  Check the input parameters.
;
	N_FOUND = 0
	IF N_ELEMENTS(START) NE 1 THEN BEGIN
           MESSAGE = 'START must be a scalar'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	END ELSE IF N_ELEMENTS(EEND) NE 1 THEN BEGIN
           MESSAGE = 'END must be a scalar'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	ENDIF
;
;  If passed, then interpret the INSTRUMENT keyword parameter.
;
	IF N_ELEMENTS(INSTRUMENT) GT 1 THEN BEGIN
           MESSAGE = 'INSTRUMENT must be a scalar'
           IF N_ELEMENTS(ERRMSG) NE 0 THEN $
              ERRMSG = MESSAGE $
           ELSE MESSAGE, MESSAGE, /CONTINUE 
           GOTO, FINISH
	END ELSE IF N_ELEMENTS(INSTRUMENT) NE 0 THEN BEGIN
		IF DATATYPE(INSTRUMENT,1) NE 'String' THEN BEGIN
                   MESSAGE = 'INSTRUMENT must be a character string'
                   IF N_ELEMENTS(ERRMSG) NE 0 THEN $
                      ERRMSG = MESSAGE $
                   ELSE MESSAGE, MESSAGE, /CONTINUE 
                   GOTO, FINISH
		ENDIF
		GET_INSTRUMENT, INSTRUMENT, INS_DESC
		IF INS_DESC.CODE EQ '' THEN BEGIN
                   MESSAGE = 'Instrument '+INSTRUMENT+' not recognized'
                   IF N_ELEMENTS(ERRMSG) NE 0 THEN $
                      ERRMSG = MESSAGE $
                   ELSE MESSAGE, MESSAGE, /CONTINUE
                   GOTO, FINISH
		ENDIF
	ENDIF
;
;  Convert the input dates to TAI format.
;
	IF DATATYPE(START,1) EQ 'Double' THEN TAI_START = START ELSE	$
		TAI_START = UTC2TAI(START)
	IF DATATYPE(EEND,1) EQ 'Double' THEN TAI_END = EEND ELSE	$
		TAI_END = UTC2TAI(EEND)
;
;  Open the science plan database.
;
	DBOPEN, 'soho_details'
;
;  Find all the entries in the requested time range.
;
	TEST = 'DATE_END>' + TRIM(TAI_START,'(F15.3)') + ',DATE_OBS<' + $
		TRIM(TAI_END,'(F15.3)')
	IF N_ELEMENTS(INSTRUMENT) EQ 1 THEN TEST = TEST + ',INSTRUME=' + $
		INS_DESC.CODE
	TEST = TEST + ',DELETED=N'
	ENTRIES = DBFIND(TEST, /SILENT)
	IF !ERR EQ 0 THEN BEGIN
		N_FOUND = 0
		GOTO, FINISH
	END ELSE BEGIN
		ENTRIES = ENTRIES(UNIQ(ENTRIES))
		N_FOUND = N_ELEMENTS(ENTRIES)
	ENDELSE
;
;  Extract the requested entries, sorted by start times.
;
	ENTRIES = DBSORT(ENTRIES,'date_obs')
	DBEXT, ENTRIES, 'instrume,date_obs,date_end,sci_obj,sci_spec',$
		INSTRUME, DATE_OBS, DATE_END, SCI_OBJ, SCI_SPEC
	DBEXT, ENTRIES, 'obs_prog,prog_id,cmp_no,object,obj_id',	$
		OBS_PROG, PROG_ID, CMP_NO, OBJECT, OBJ_ID
	DBEXT, ENTRIES, 'xcen,ycen,angle,ixwidth,iywidth',		$
		XCEN, YCEN, ANGLE, IXWIDTH, IYWIDTH
	DBEXT, ENTRIES, 'disturbances,jitter_limit',			$
		DISTURBANCES, JITTER_LIMIT
;
;  Remove any entries that only touch the requested time range.
;
	W = WHERE((DATE_END NE TAI_START) AND (DATE_OBS NE TAI_END), N_FOUND)
	IF N_FOUND EQ 0 THEN GOTO, FINISH
;
;  Define the structure that the data will be returned in.
;
	OBS = { STRUCT_TYPE: 'SOHO-DET-LIST',	$
		INSTRUME: 'S',		$
		DATE_OBS: 0.0D0,	$
		DATE_END: 0.0D0,	$
		SCI_OBJ: 'String',	$
		SCI_SPEC: 'String',	$
		OBS_PROG: 'String',	$
		PROG_ID: 0,		$
		CMP_NO: 0,		$
		OBJECT: 'String',	$
		OBJ_ID: 'String',	$
		XCEN: 'String',		$
		YCEN: 'String',		$
		ANGLE: 'String',	$
		IXWIDTH: 'String',	$
		IYWIDTH: 'String',	$
		DISTURBANCES: 'String',	$
		JITTER_LIMIT: 0}
;
	IF N_FOUND EQ 1 THEN BEGIN
		OBS.INSTRUME	= INSTRUME(W(0))
		OBS.DATE_OBS	= DATE_OBS(W(0))
		OBS.DATE_END	= DATE_END(W(0))
		OBS.SCI_OBJ	= SCI_OBJ(W(0))
		OBS.SCI_SPEC	= SCI_SPEC(W(0))
		OBS.OBS_PROG	= OBS_PROG(W(0))
		OBS.PROG_ID	= PROG_ID(W(0))
		OBS.CMP_NO	= CMP_NO(W(0))
		OBS.OBJECT	= OBJECT(W(0))
		OBS.OBJ_ID	= OBJ_ID(W(0))
		OBS.XCEN	= XCEN(W(0))
		OBS.YCEN	= YCEN(W(0))
		OBS.ANGLE	= ANGLE(W(0))
		OBS.IXWIDTH	= IXWIDTH(W(0))
		OBS.IYWIDTH	= IYWIDTH(W(0))
		OBS.DISTURBANCES= DISTURBANCES(W(0))
		OBS.JITTER_LIMIT= JITTER_LIMIT(W(0))
	END ELSE BEGIN
		OBS = REPLICATE(OBS, N_FOUND)
		OBS.INSTRUME	= INSTRUME(W)
		OBS.DATE_OBS	= DATE_OBS(W)
		OBS.DATE_END	= DATE_END(W)
		OBS.SCI_OBJ	= SCI_OBJ(W)
		OBS.SCI_SPEC	= SCI_SPEC(W)
		OBS.OBS_PROG	= OBS_PROG(W)
		OBS.PROG_ID	= PROG_ID(W)
		OBS.CMP_NO	= CMP_NO(W)
		OBS.OBJECT	= OBJECT(W)
		OBS.OBJ_ID	= OBJ_ID(W)
		OBS.XCEN	= XCEN(W)
		OBS.YCEN	= YCEN(W)
		OBS.ANGLE	= ANGLE(W)
		OBS.IXWIDTH	= IXWIDTH(W)
		OBS.IYWIDTH	= IYWIDTH(W)
		OBS.DISTURBANCES= DISTURBANCES(W)
		OBS.JITTER_LIMIT= JITTER_LIMIT(W)
	ENDELSE
;
;  Close the database and return.
;
FINISH:
	DBCLOSE
;
	RETURN
	END
