	PRO CONCAT_PNT, INS_X, INS_Y, WIDTH, HEIGHT, ANGLE, STRING=STRING
;+
; Project     :	SOHO - CDS
;
; Name        :	CONCAT_PNT
;
; Purpose     :	Concatenates instrument pointing to contiguous areas
;
; Explanation :	This routines takes a series of pointing areas, and merges
;		areas that match up to form a larger raster area.
;
; Use         :	CONCAT_DIR, INS_X, INS_Y, WIDTH, HEIGHT  [, ANGLE ]
;
; Inputs      :	INS_X	= Together with INS_Y, an array that gives the pointing
;			  of the center of the boxes.
;		INS_Y	= ...
;		WIDTH	= The widths of the boxes
;		HEIGHT	= The heights of the boxes
;
; Opt. Inputs :	None.
;
; Outputs     :	INS_X, INS_Y, WIDTH, HEIGHT
;
; Opt. Outputs:	ANGLE	= This will be returned as an array of zeros with the
;			  same dimensions as the other arrays.
;
; Keywords    :	STRING	= If set, then the output is returned as one or more
;			  comma delimited strings suitable for use within the
;			  IAP.
;
; Calls       :	TRIM
;
; Common      :	None.
;
; Restrictions:	The input arrays must all have the same dimensions.
;
; Side effects:	None.
;
; Category    :	Planning, science
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 2 May 1995
;
; Modified    :	Version 1, William Thompson, GSFC, 2 May 1995
;		Version 2, William Thompson, GSFC, 5 May 1995
;			Modified so that areas must match up, not just overlay.
;		Version 3, William Thompson, GSFC, 19 July 1996
;			Fixed bug when additional lines added.
;
; Version     :	Version 3, 19 July 1996
;-
;
;
	ON_ERROR, 2
;
	IF N_PARAMS() LT 4 THEN MESSAGE,	$
		'Syntax:  CONCAT_PNT, INS_X, INS_Y, WIDTH, HEIGHT  [, ANGLE ]'
;
;  Convert the window list into the format (Xfirst, Yfirst, Xlast, Ylast).
;
	XSTART = INS_X - WIDTH / 2.  &  YSTART = INS_Y - HEIGHT / 2.
	XSTOP  = INS_X + WIDTH / 2.  &  YSTOP  = INS_Y + HEIGHT / 2.
;
;  Find any matching windows and merge them.  First, step through each window
;  in turn and find any windows that match up in columns.  The second iteration
;  matches up windows in rows.
;
	FOR ITER = 1,2 DO BEGIN
	    I = 0
	    N_WINDOWS = N_ELEMENTS(XSTART)
	    WHILE I LT (N_WINDOWS-1) DO BEGIN
		IF ITER EQ 1 THEN BEGIN
		    W = WHERE((XSTART(I) EQ XSTART(I:*)) AND	$
			      (XSTOP (I) EQ XSTOP (I:*)) AND	$
			      (YSTART(I) LE YSTOP (I:*)) AND	$
			      (YSTOP (I) GE YSTART(I:*)), COUNT) + I
		END ELSE BEGIN
		    W = WHERE((XSTART(I) LE XSTOP (I:*)) AND	$
			      (XSTOP (I) GE XSTART(I:*)) AND	$
			      (YSTART(I) EQ YSTART(I:*)) AND	$
			      (YSTOP (I) EQ YSTOP (I:*)), COUNT) + I
		ENDELSE
;
;  If matching windows were found, then find the window that includes all
;  the matched windows.
;
		IF COUNT GT 1 THEN BEGIN
			XSTART(I) = MIN(XSTART(W))
			YSTART(I) = MIN(YSTART(W))
			XSTOP(I) = MAX(XSTOP(W))
			YSTOP(I) = MAX(YSTOP(W))
;
;  Remove the matching windows from the list, and start checking again from
;  the beginning.
;
			WW = INDGEN(N_WINDOWS)
			WW(W(1:*)) = -1
			W = WHERE(WW GE 0)
			XSTART = XSTART(W)
			YSTART = YSTART(W)
			XSTOP  = XSTOP(W)
			YSTOP  = YSTOP(W)
			N_WINDOWS = N_ELEMENTS(W)
			I = 0
;
;  Otherwise, if no matching windows were found, then try the next window.
;
		END ELSE I = I + 1
	    ENDWHILE
	ENDFOR
;
;  Recreate the input arrays.
;
	INS_X  = (XSTART + XSTOP) / 2.  &  WIDTH  = XSTOP - XSTART
	INS_Y  = (YSTART + YSTOP) / 2.  &  HEIGHT = YSTOP - YSTART
;
;  Create an ANGLE array of all zeroes with the correct number of elements.
;
	ANGLE = FLTARR(N_WINDOWS)
;
;  If the STRING keyword was passed, then convert the arrays to comma delimited
;  strings.  If necessary, split the string up so that each string does not
;  exceed 50 characters.
;
	IF KEYWORD_SET(STRING) THEN BEGIN
;
;  First, initialize the strings.
;
		IS = 0
		S_INS_X  = ''
		S_INS_Y  = ''
		S_WIDTH  = ''
		S_HEIGHT = ''
		S_ANGLE  = ''
;
;  For each element in the arrays, check to see if it would put any of the
;  strings over the 50 character limit.
;
		FOR I = 0, N_WINDOWS-1 DO BEGIN
			SX = TRIM(INS_X(I))
			SY = TRIM(INS_Y(I))
			SW = TRIM(WIDTH(I))
			SH = TRIM(HEIGHT(I))
			SA = TRIM(ANGLE(I))
;
;  You don't have to check if it's the first entry.
;
			IF I EQ 0 THEN BEGIN
				S_INS_X (IS) = SX
				S_INS_Y (IS) = SY
				S_WIDTH (IS) = SW
				S_HEIGHT(IS) = SH
				S_ANGLE (IS) = SA
			END ELSE BEGIN
;
;  If it's over the limit, then start a new set of strings.
;
				LX = STRLEN(S_INS_X (IS)) + STRLEN(SX) + 1
				LY = STRLEN(S_INS_Y (IS)) + STRLEN(SY) + 1
				LW = STRLEN(S_WIDTH (IS)) + STRLEN(SW) + 1
				LH = STRLEN(S_HEIGHT(IS)) + STRLEN(SH) + 1
				LA = STRLEN(S_ANGLE (IS)) + STRLEN(SA) + 1
				IF MAX([LX,LY,LW,LH,LA]) GT 50 THEN BEGIN
					S_INS_X  = [S_INS_X,  SX]
					S_INS_Y  = [S_INS_Y,  SY]
					S_WIDTH  = [S_WIDTH,  SW]
					S_HEIGHT = [S_HEIGHT, SH]
					S_ANGLE  = [S_ANGLE,  SA]
					IS = IS + 1
;
;  Otherwise, append it to the current string with a comma separator.
;
				END ELSE BEGIN
					S_INS_X (IS) = S_INS_X (IS) + ',' + SX
					S_INS_Y (IS) = S_INS_Y (IS) + ',' + SY
					S_WIDTH (IS) = S_WIDTH (IS) + ',' + SW
					S_HEIGHT(IS) = S_HEIGHT(IS) + ',' + SH
					S_ANGLE (IS) = S_ANGLE (IS) + ',' + SA
				ENDELSE
			ENDELSE
		ENDFOR
;
;  Replace the input arrays with the output strings.
;
		INS_X  = S_INS_X
		INS_Y  = S_INS_Y
		WIDTH  = S_WIDTH
		HEIGHT = S_HEIGHT
		ANGLE  = S_ANGLE
	ENDIF
;
	RETURN
	END
