;+
; Project     :	SOHO - CDS
;
; Name        :	GET_PLAN_ITIME()
;
; Purpose     : Get index of time field in plan structure
;
; Category    : Planning
;
; Explanation :	
;
; Syntax      :	INDEX = GET_PLAN_ITIME( PLAN )
;                 or
;               TYPE = 0,1,2,3,4
;
; Examples    :	
;
; Inputs      : PLAN = Plan structure definition 
;
; Opt. Inputs : TYPE = Type code which represents the plan structure type.  If
;		       passed, then the PLAN structure is not passed.  In other
;		       words, only one of PLAN or TYPE is passed.
;
; Outputs     : The returned value is the index of the time field in the
;		planning structure.
;
; Opt. Outputs: None.
;
; Keywords    : None.
;
; Calls       : DATATYPE, TAG_NAMES
;
; Common      : None.
;
; Restrictions: None.
;
; Side effects: If an error occurs, then -1 is returned.
;
; Prev. Hist. : None.
;
; History     :	Version 1, Zarro (ARC/GSFC) 29 January 1995.
;		Version 2, William Thompson, GSFC, 18 January 1996
;			Made more robust.  Does not depend on exact structure
;			type.
;
; Contact     :	WTHOMPSON
;-
;
	FUNCTION GET_PLAN_ITIME, PLAN, ERRMSG=ERRMSG
;
	ON_ERROR, 1
;
;  Define the error message.  This will be updated below.
;
	MESSAGE = ''
;
;  Check the input parameter.
;
	IF NOT EXIST(PLAN) THEN BEGIN
		MESSAGE = 'Syntax:  ITIME = GET_PLAN_ITIME(PLAN)'
		GOTO, HANDLE_ERROR
	ENDIF
;
;  If the input is not a structure, then the type parameter was passed.  Create
;  a structure from the type code.
;
	IF DATATYPE(PLAN) NE 'STC' THEN BEGIN
		TYPE=FIX(PLAN)
                IF TYPE EQ 5 THEN BEGIN
                 RESULT=0l & GOTO,FINISH
                ENDIF
		IF (TYPE LT 0) OR (TYPE GT 4) THEN BEGIN
			MESSAGE = 'Unsupported DB type'
			GOTO, HANDLE_ERROR
		ENDIF
		DEF_INST_PLAN, TPLAN, TYPE=TYPE, INST=WHICH_INST()
	ENDIF ELSE TPLAN=PLAN
;
;  Look for DATE_OBS, START_TIME, RCVR_START in turn.
;
	TAGS=TAG_NAMES(TPLAN)
	ITIME = WHERE(TAGS EQ 'DATE_OBS')
	IF ITIME(0) LT 0 THEN ITIME = WHERE(TAGS EQ 'START_TIME')
	IF ITIME(0) LT 0 THEN ITIME = WHERE(TAGS EQ 'RCVR_START')
	IF ITIME(0) LT 0 THEN ITIME = WHERE(TAGS EQ 'EARLIEST')
	IF ITIME(0) LT 0 THEN BEGIN
		MESSAGE = "Cannot find time tag in structure"
		GOTO, HANDLE_ERROR
	END ELSE BEGIN
		RESULT = ITIME(0)
		GOTO, FINISH
	ENDELSE
;
HANDLE_ERROR:
	IF N_ELEMENTS(ERRMSG) EQ 0 THEN MESSAGE, MESSAGE, /CONTINUE ELSE $
		MESSAGE = 'GET_PLAN_ITIME:' + MESSAGE
	RESULT = -1L
;
FINISH:
	IF N_ELEMENTS(ERRMSG) NE 0 THEN ERRMSG = MESSAGE
	RETURN, RESULT
	END

