	FUNCTION KAP_DATE, FILENAME, MJD=MJD, ERRMSG=ERRMSG
;+
; Project     :	SOHO - CDS
;
; Name        :	KAP_DATE()
;
; Purpose     :	Returns the date of a KAP file
;
; Explanation :	This routine looks at the name of a KAP file, and determines
;		the date it applies to.  It assumes a filename of the form
;		"ECSyyyymmddvvv.KAP".
;
; Use         :	Date = KAP_DATE(FILENAME)
;
; Inputs      :	FILENAME = The name of the KAP file.
;
; Opt. Inputs :	None.
;
; Outputs     :	The result of the function is the date extracted from
;		the filename, in CDS internal format.  If unable to determine a
;		date, then -1 will be returned instead.  (For better error
;		capturing, use the ERRMSG keyword.)
;
; Opt. Outputs:	None.
;
; Keywords    :	MJD	 = If set, then only the Modified Julian Day part of
;			   the date will be returned.
;
;		ERRMSG   = If defined and passed, then any error messages will
;			   be returned to the user in this parameter rather
;			   than being handled by the IDL MESSAGE utility.  If
;			   no errors are encountered, then a null string is
;			   returned.  In order to use this feature, the string
;			   ERRMSG must be defined first, e.g.,
;
;				ERRMSG = ''
;				Result = KAP_DATE( FILENAME, ERRMSG=ERRMSG )
;				IF ERRMSG NE '' THEN ...
;
; Calls       :	DATATYPE, BREAK_FILE, STR2UTC
;
; Common      :	None.
;
; Restrictions:	Only works if the filename has the correct format.
;
; Side effects:	None.
;
; Category    :	Planning, Activity plans
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 24 May 1995
;
; Modified    :	Version 1, William Thompson, GSFC, 24 May 1995
;		Version 2, William Thompson, GSFC, 2 August 1995
;			Made compatible with NFS on VMS computers.
;
; Version     :	Version 2, 2 August 1995
;-
;
	ON_ERROR, 2
;
;  Check the input parameters.
;
	IF N_PARAMS() NE 1 THEN BEGIN
		MESSAGE = 'Syntax:  Date = KAP_DATE(FILENAME)'
	END ELSE IF N_ELEMENTS(FILENAME) NE 1 THEN BEGIN
		MESSAGE = 'FILENAME must be a scalar'
	END ELSE IF DATATYPE(FILENAME,1) NE 'String' THEN BEGIN
		MESSAGE = 'FILENAME must be a character string'
	ENDIF
;
;  Break the filename into its constituent parts.
;
	BREAK_FILE, FILENAME, DISK, DIR, NAME, EXT
;
;  Remove any embedded dollar signs, which may be added by NFS on VMS systems,
;  e.g. Multinet.
;
	REMCHAR, NAME, '$'
;
;  Parse the filename to determine the date, assuming a filename of
;  the form ECSyyyymmddvvv.KAP.
;
	YEAR  = STRMID(NAME, 3, 4)
	MONTH = STRMID(NAME, 7, 2)
	DAY   = STRMID(NAME, 9, 2)
;
	DATE = STR2UTC(YEAR+'-'+MONTH+'-'+DAY, ERRMSG=ERRMSG)
	IF KEYWORD_SET(MJD) THEN DATE = DATE.MJD
	GOTO, FINISH
;
;  Error handling point.
;
HANDLE_ERROR:
	IF N_ELEMENTS(ERRMSG) NE 0 THEN ERRMSG = 'KAP_DATE: ' + MESSAGE	$
		ELSE MESSAGE, MESSAGE
	DATE = -1
;
;  Exit point.
;
FINISH:
	RETURN, DATE
	END
