	FUNCTION LAST_KAP_VERS, DATE, ERRMSG=ERRMSG
;+
; Project     :	SOHO - CDS
;
; Name        :	LAST_KAP_VERS()
;
; Purpose     :	Gets version number of last KAP file read for given date
;
; Explanation :	This routine looks up what the last version number was for the
;		last KAP file for a given date that was read.
;
; Use         :	Version = LAST_KAP_VERS( DATE )
;
; Inputs      :	DATE	= The date that the user wishes to look up the version
;			  for.  This can be in any of the standard CDS time
;			  formats.
;
; Opt. Inputs :	None.
;
; Outputs     :	The result of the function is the version number of the last
;		KAP file read for the date in question.  If no KAP files have
;		been read yet for that date, then -1 is returned.
;
; Opt. Outputs:	None.
;
; Keywords    :
;       ERRMSG    = If defined and passed, then any error messages will be
;                   returned to the user in this parameter rather than
;                   depending on the MESSAGE routine in IDL.  If no errors are
;                   encountered, then a null string is returned.  In order to
;                   use this feature, ERRMSG must be defined first, e.g.
;
;                       ERRMSG = ''
;                       Version = LAST_KAP_VERS(DATE, ERRMSG=ERRMSG)
;                       IF ERRMSG NE '' THEN ...
;
; Calls       :	DATATYPE, DBOPEN, DBFIND, DBEXT, DBCLOSE, TRIM
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	None.
;
; Category    :	Planning, Databases.
;
; Prev. Hist. :	None.
;
; Written     :	William Thompson, GSFC, 24 May 1995
;
; Modified    :	Version 1, William Thompson, GSFC, 24 May 1995
;
; Version     :	Version 1, 24 May 1995
;-
;
	ON_ERROR, 2
;
;  Define a null result.  If the routine is successful, this will be updated
;  later.
;
	VERSION = -1
;
;  Check the number of parameters.
;
        IF N_PARAMS() NE 1 THEN BEGIN
           MESSAGE = 'Syntax:  Version = LAST_KAP_VERS( DATE )'
	   GOTO, HANDLE_ERROR
        ENDIF
;
;  Check the input parameter.
;
	IF N_ELEMENTS(DATE) NE 1 THEN BEGIN
           MESSAGE = 'DATE must be a scalar'
           GOTO, HANDLE_ERROR
	ENDIF
;
;  Convert the date to internal format, and extract the Modified Julian Date.
;
	UTC = ANYTIM2UTC(DATE)
	MJD = UTC.MJD
;
;  Open the database.
;
	DBOPEN, 'kap_version'
;
;  Search for entries with dates equal to that given by the user.
;
	ENTRY = DBFIND('MJD=' + TRIM(MJD), /SILENT)
;
;  If no entries were found, then return immediately.
;
	IF !ERR EQ 0 THEN BEGIN
		VERSION = -1
		GOTO, FINISH
	ENDIF
;
;  Otherwise, extract the version number.
;
	DBEXT, ENTRY(0), 'version', VERSION
	GOTO, FINISH
;
;  Error handling point.
;
HANDLE_ERROR:
	IF N_ELEMENTS(ERRMSG) NE 0 THEN ERRMSG = 'LAST_KAP_VERS: ' + MESSAGE $
		ELSE MESSAGE, MESSAGE, /CONTINUE
;
;  Close the database, and return.
;
FINISH:
	DBCLOSE
;
	RETURN, VERSION(0)
	END
