;---------------------------------------------------------------------------
; Document name: mk_plan_copy.pro
; Created by:    Liyun Wang, GSFC/ARC, May 26, 1995
;
; Last Modified: Tue Aug  6 10:51:18 1996 (lwang@achilles.nascom.nasa.gov)
;---------------------------------------------------------------------------
;
FUNCTION mk_plan_copy, hplan, info=info, new_row=new_row, err=err, $
                       wid=wid
;+
; PROJECT:
;       SOHO - CDS
;
; NAME:	
;       MK_PLAN_COPY()
;
; PURPOSE:
;       Make a new plan entry from a selected (highlighted) plan entry
;
; EXPLANATION:
;       
; CALLING SEQUENCE: 
;       Result = mk_plan_copy(hplan)
;
; INPUTS:
;       HPLAN - A highlighted (selected) plan entry to be copied from
;       INFO  - A structure that contains basic required information for the
;               operation. It should have at least the following tags:
;                  ROW      - Row number in which HPLAN is in
;                  STARTDIS - Start time of the display window
;                  STOPDIS  - End time of the display window
;
; OPTIONAL INPUTS: 
;       WID   - Widget IDs to show date/time of cursor position. If passed, a
;               procedure routine MK_PLAN_COPY_EV must exist to be used to
;               update such information. It must be a 2- or 4-element
;               array: the first two elements are text widget IDs for
;               starting date/time, and the last two elements (which
;               are optional) are text widget IDs for end date/time.
;
;
; OUTPUTS:
;       RESULT - A new plan entry ready to be inserted or appended
;
; OPTIONAL OUTPUTS:
;       None.
;
; KEYWORD PARAMETERS: 
;       ACTION - A string scalar indicating what can be done with the newly
;                copied plan entry. Valid values can be: 'insert', 'append',
;                or 'none'
;
; CALLS:
;       BOX_CURSOR2, GET_PLAN_ITIME, DATATYPE, TAI2UTC, UTC2TAI,
;       MK_PLAN_COPY_EV 
;
; COMMON BLOCKS:
;       None.
;
; RESTRICTIONS: 
;       None.
;
; SIDE EFFECTS:
;       None.
;
; CATEGORY:
;       
; PREVIOUS HISTORY:
;       Written May 26, 1995, Liyun Wang, GSFC/ARC
;
; MODIFICATION HISTORY:
;       Version 1, created, Liyun Wang, GSFC/ARC, May 26, 1995
;       Version 2, Liyun Wang, GSFC/ARC, June 5, 1995
;          Made it possible to update widget contents for date/time
;       Version 3, July 21, 1995, Liyun Wang, GSFC/ARC
;          Made end date/time wdigets updatable
;
; VERSION:
;       Version 3, July 21, 1995
;-
;
   ON_ERROR, 2
   err = ''
   IF datatype(hplan) NE 'STC' OR datatype(info) NE 'STC' THEN BEGIN
      err = 'Syntax: plan = mk_plan_copy(hplan, info=info)'
      MESSAGE, err, /cont
      RETURN, {study_id:-1}
   ENDIF
   
   itime = get_plan_itime(hplan)
   dur = hplan.(itime+1)-hplan.(itime)
   plan = hplan
;---------------------------------------------------------------------------
;  Get the day number of startdis
;---------------------------------------------------------------------------
   day=tai2utc(info.startdis) &  day.time=0
   day=utc2tai(day)

   x1 = FLOAT((hplan.(itime)-day)/3600.d0)
   x2 = FLOAT((hplan.(itime+1)-day)/3600.d0)
   
   y1 = !y.crange(0)+info.row+0.1
   y2 = y1+0.8
   
   temp = CONVERT_COORD([x1,x2],[y1,y2], /data, /to_device)
   xpos = temp(0,*)
   ypos = temp(1,*)
   x0 = xpos(0) & nx = xpos(1)-x0
   y0 = ypos(0) & ny = ypos(1)-y0
   IF N_ELEMENTS(wid) NE 0 THEN BEGIN
      temp = {day:day, wid:wid, dur:dur, x:0, y:0}
      box_cursor2, x0, y0, nx, ny, /init, /fixed, /anywhere, $
         event_pro = 'mk_plan_copy_ev', stc4event = temp
   ENDIF ELSE BEGIN
      box_cursor2, x0, y0, nx, ny, /init, /fixed, /anywhere
   ENDELSE
   temp = CONVERT_COORD(x0, y0, /DEVICE, /to_data)
   
   plan.(itime) = day+temp(0,0)*3600.d0
   plan.(itime+1) = plan.(itime)+dur
   new_row = FIX(temp(1,0))
   
   RETURN, plan
END

;---------------------------------------------------------------------------
; End of 'mk_plan_copy.pro'.
;---------------------------------------------------------------------------
