;---------------------------------------------------------------------------
; Document name: mk_plan_html.pro
; Created by:    Liyun Wang, NASA/GSFC, July 25, 1996
;
; Last Modified: Thu Aug 21 11:31:21 1997 (lwang@achilles.nascom.nasa.gov)
;---------------------------------------------------------------------------
;
;+
; PROJECT:
;       SOHO
;
; NAME:
;       MK_PLAN_HTML
;
; PURPOSE:
;       Make list of SOHO plans of today in HTML table format
;
; CATEGORY:
;       Planning, Utility
;
; SYNTAX:
;       mk_plan_html [, pdate] [, dir=dir] 
;
; INPUTS:
;       None required.
;
; OPTIONAL INPUTS:
;       PDATE - Date of plan schedule, can be in any CDS time format. If
;               passed, it overwrites keyword YESTERDAY or TOMORROW
;
; OUTPUTS:
;       HTML document named 'soho_today.html', 'soho_yesterday.html'
;          (when keyword YESTERDAY is set),  or 'soho_tomorrow.html'
;          (when keyword TOMORROW is set)
;
; OPTIONAL OUTPUTS:
;       None.
;
; KEYWORDS:
;       DIR       - Name of directory in which output file is written
;       YESTERDAY - Make list of plans for yesterday (if PDATE not passed)
;       TOMORROW  - Make list of plans for tomorrow (if PDATE not passed)
;       VERBOSE   - Set this keyword to print output file name on the screen
;       ERR       - Error string
;
; COMMON:
;       PLAN_HTML
;
; RESTRICTIONS:
;       None.
;
; SIDE EFFECTS:
;       None.
;
; HISTORY:
;       Version 1, July 25, 1996, Liyun Wang, NASA/GSFC. Written
;       Version 2, August 13, 1996, Liyun Wang, NASA/GSFC
;          Converted pointing parameters in f15.1 format if they are
;             floating point numbers (e.g., EIT entries)
;       Version 3, August 13, 1996, Liyun Wang, NASA/GSFC
;          Improved appearence for multi-pointing entries
;       Version 4, August 23, 1996, Liyun Wang, NASA/GSFC
;          Added optional input parameter PDATE
;       Version 5, November 26, 1996, Zarro, NASA/GSFC
;          Added output file write protection check and ERR keyword
;       Version 6, August 20, 1997, Liyun Wang, NASA/GSFC
;          Added campaign numbers and their links to campaign database
;             pages on output 
;          Improved appearence of HTML output (via SOHO_HTML_ITEMS)
;
; CONTACT:
;       Liyun Wang, NASA/GSFC (Liyun.Wang.1@gsfc.nasa.gov)
;-
;

PRO mk_plan_html, pdate, dir=dir, tomorrow=tomorrow, yesterday=yesterday, $
                  verbose=verbose,err=err
   COMMON plan_html, first_done
   ON_ERROR, 2

   first_done = 0
   IF N_ELEMENTS(dir) EQ 0 THEN dir = '/tmp'

   get_utc, curr_time
   IF N_ELEMENTS(pdate) EQ 0 THEN BEGIN
      pdate = curr_time
      IF KEYWORD_SET(tomorrow) THEN BEGIN
         pdate.mjd = pdate.mjd+1L
         file = 'soho_tomorrow.html'
      ENDIF ELSE IF KEYWORD_SET(yesterday) THEN BEGIN
         pdate.mjd = pdate.mjd-1L
         file = 'soho_yesterday.html'
      ENDIF ELSE file = 'soho_today.html'
   ENDIF ELSE BEGIN
      pdate = anytim2utc(pdate)
      file = 'soho_plan.html'
   ENDELSE
   ffile = concat_dir(dir, file)
   if not test_open(ffile,/write,err=err) then return

   date_str = anytim2utc(pdate, /ecs, /date)
   tstart = utc2tai(anytim2utc(date_str))
   dur = 86400.d0
   tstop = tstart + dur

;---------------------------------------------------------------------------
;  Set up output file, etc.
;---------------------------------------------------------------------------

   OPENW, unit, ffile, /GET_LUN
   PRINTF, unit, ''
   PRINTF, unit, '<html>'
   PRINTF, unit, '<!--'
   PRINTF, unit, '--------------------------------------------------------'
   PRINTF, unit, 'Document name: '+file
   PRINTF, unit, 'Created by:    mk_plan_html.pro
   PRINTF, unit, '--------------------------------------------------------'
   PRINTF, unit, '-->'
   PRINTF, unit, '<head>'
   PRINTF, unit, '<title>SOHO Operations Schedule For '+date_str+'</title>'
   PRINTF, unit, '</head>'
   PRINTF, unit, '<body link=#23238E vlink=#23238E bgcolor=#a0eed0>'
   PRINTF, unit, '<center><h1>SOHO Operations Schedule For '+date_str+'</h1></center>'

;---------------------------------------------------------------------------
;  Read in detail plan entries
;---------------------------------------------------------------------------
   list_soho_det, tstart, tstop, plan, ndobs

   IF ndobs NE 0 THEN BEGIN
      new = 1
      itime = get_plan_itime(plan, err=err)
      IF itime LT 0 THEN RETURN

      inst = get_soho_inst(/short)
      ninst = N_ELEMENTS(inst)

      FOR j=0, ninst-1 DO BEGIN
         name_inst = get_soho_inst(inst(j))
         ii = WHERE(plan.instrume EQ inst(j), nplans)
         IF nplans GT 0 THEN BEGIN
            plans = plan(ii)
            CASE (name_inst) OF
               'CELIAS': inst_str = 'JOP'
               'SWAN': inst_str = 'SOC'
               'ERNE': inst_str = 'CAMPAIGN'
               ELSE: inst_str = name_inst
            ENDCASE
            iname = inst_str
            soho_items_html, unit, iname, plans, new=new
         ENDIF
      ENDFOR
      html_table_header, unit, /finish
   ENDIF ELSE BEGIN
      PRINTF, unit, '<hr>Plans for '+date_str+' are yet to be made.<p>'
   ENDELSE

   PRINTF, unit, '<hr><address>
   PRINTF, unit, '<B>Author:</B> <br>
   PRINTF, unit, '<a href="http://orpheus.nascom.nasa.gov/~lwang">Liyun Wang</a>, NASA/GSFC, Code 682.3 '
   PRINTF, unit, '(<a href="mailto:Liyun.Wang.1@gsfc.nasa.gov">'+$
      'Liyun.Wang.1@gsfc.nasa.gov</a>)<p>'
   PRINTF, unit, ''
   PRINTF, unit, '<B>Responsible NASA Official:</B><br>'
   PRINTF, unit, 'Arthur I. Poland, NASA/GSFC, Code 682.1'
   PRINTF, unit, '(<a href="mailto:Arthur.I.Poland.1@gsfc.nasa.gov">'+$
      'Arthur.I.Poland.1@gsfc.nasa.gov</a>)<p>'
   PRINTF, unit, ''
   PRINTF, unit, '<B>Document Created:</B> '+$
      anytim2utc(curr_time, /ecs, /trun)+' UT<p>'
   PRINTF, unit, '</address>'
   PRINTF, unit, ''
   PRINTF, unit, '</body>'
   PRINTF, unit, '</html>'
   CLOSE, unit
   FREE_LUN, unit
   IF KEYWORD_SET(verbose) THEN PRINT, 'File '+ffile+' created.'
   RETURN
END

;---------------------------------------------------------------------------
; End of 'mk_plan_html.pro'.
;---------------------------------------------------------------------------
