;+
; Project     :	SOHO - CDS
;
; Name        :	READ_MERGED_DSN
;
; Purpose     :	Read DSN-27 passes from the dsn_merged file.
;
; Explanation : This function tries to find the merged_dsn file.  If available, 
; 		it reads the pass information from the file and extracts the 
;		DSN 27 passes into an array.
;
; Use         :	d27 = READ_MERGED_DSN (tstart, tend, /INCLUSIVE, /TRUNCATE)
;
; Inputs      :	None.
;
; Opt. Inputs : tstart	The earliest time to extract DSN 27 passes from the
;			file in any acceptable time format.  The interpretation
;			of this parameter varies depending on if the /INCLUSIVE
;			keyword is set.  If not included, then all DSN 27 from
;			the file will be returned.
;
;		tend	The latest time to extract DSN 27 passes from the
;			file in any acceptable time format.  The interpretation
;			of this parameter varies depending on if the /INCLUSIVE
;			keyword is set.  May only be included if the parameter
;			tstart is also included.
;
; Outputs     :	None.
;
; Opt. Outputs:	None.
;
; Keywords    : inclusive	When set, any passes intersecting the start or 
;				end times specified by the inputs tstart and 
;				tend will be returned.  Otherwise, only passes
;				which fall completely within the boundries 
;				delineated by tstart and tend will be returned.
;
;		truncate	Trim the times of DSN 27 passes returned by this
;				function so that no pass begins earlier then
;				tstart and no pass ends later then tend.  This
;				only make sense in conjuction with the INCLUSIVE
;				keyword 
;
; Returns     : An array of structures with the following tags:
;
;			START_TIME:	Starting time of pass in TAI format 
;			END_TIME: 	Ending time of pass in TAI format
;			PASS_DURATION:	Pass duration in seconds
;			RES_NAME:	'DSN_27', 
;
;		In the event that no DSN 27 pass are found then, a single
;		structure will be returned with only the RES_NAME tag defined.
;
; Common      :	None.
;
; Restrictions:	The enviroment variable CDS_DSN_MERGE should be defined and point
;		to the location of the file dsn_merged.dat.  If this enviroment
;		variable is not defined, then the fuction will look for this file
;		in its default directory.
;
; Side effects:	None.
;
; Category    :	Planning
;
; Prev. Hist. :	None.
;
; Written     :	Ronald Yurow (EITI)
;
; Modified:   : Ronald Yurow, EITI, October 5, 2001
;                  Initial Release
;		Ronald Yurow, EITI, October 15, 2001
;		   Modified structure tag names to insure compatibility
;		   with other software modules.
;
; Version     :	Version 2, October 15, 2001
;-
	FUNCTION READ_MERGED_DSN, tstart, 			$
	                          tend, 			$
	                          INCLUSIVE = inclusive,	$
	                          TRUNCATE = truncate

	ln = ""
	station = 0
;
; Get the current time.  Good to initialize UTC structure.
;

        GET_UTC, utc

;
; Convert the current time to TAI format
;

	tnow = UTC2TAI (utc)

;
; Check if CDS_DSN_MERGED is defined.  If it is then it will point to
; in which to read the merged dsn file. 
;

	IF getenv ("CDS_DSN_MERGED") eq "" THEN BEGIN
	
;
; Ooops.  The enviroment variable is not defined.  Set the path to point
; to the current directory 
;

	   path = "./"

	ENDIF ELSE BEGIN

;
; OK.  The enviroment variable is defined.  Set the path to point to the
; directory which the enviroment variable is defined to point to.
;

	   path = getenv ("CDS_DSN_MERGED")

	ENDELSE

;
; Get a list of files named dsn_*.dat which exist in the search directory. 
;
	
	lst = file_list (path, "dsn_*.dat", /bydate)

;
; Check if lst is empty.  If it is then we can't read the file and best to
; die with an error message.
;

	IF lst (0) eq "" THEN BEGIN

	   PRINT, "Can't locate merged dsn file."

	   rec = {RES_NAME:'DSN_27'}
           lst = REPLICATE (rec, 1)

	   RETURN, lst

	ENDIF

;
; Set fname to the most current file on the list
;

	fname = lst (0)

;
; Initialize an internal structure to return the search results.
;

        rec = {START_TIME:tnow,				$
	       END_TIME:tnow, 				$
	       PASS_DURATION:0L,			$
	       RES_NAME:'DSN_27'}

        lst = REPLICATE (rec, 1)
        nrec = 0

;
; Open the requested file.
;

	OPENR, lun, FNAME, /GET_LUN

;
; Tell the user we are beginning to read the merged DSN file.
;

;	PRINT, "Beginning to read merged DSN file."

;
; Loop to read the entire merged DSN file.
;

        WHILE NOT EOF (lun) DO BEGIN

           READF, lun, ln

;
; Compress all the spaces into single spaces.
;

	   ln = STRCOMPRESS (ln)


;
; Determine which elements of the string are digits.
;

	   dgt = IS_DIGIT (ln)

;
; Make a copy of ln as a byte array.
;

	   lnb = BYTE (ln)

;
; Check if the string read from the file is long enough.  If it is not the
; proper length then move on to the next string.
;

	   IF N_ELEMENTS (lnb) NE 63 THEN GOTO, LOOP_CONTINUE

;
; Get a list of indexes into lnb which correspond to spaces.  This is useful
; because after the string compression, spaces are now field seperators.
;

;
; First we need to figure what value is represents the the space character.
;

	   sp = (BYTE (' ')) (0)

;
; Now get a list of indexes into the the line where their are spaces.
;

	   sep = where (lnb EQ sp)

;
; Get the date and time of the start of the DSN Pass
;

	   pos = sep (0) + 1

	   time0 = UTC2TAI (ANYTIM2UTC (STRING (lnb (pos:pos+18))))

;
; Get the date and time of the end of the DSN Pass
;

	   pos = sep (2) + 1

	   time1 = UTC2TAI (ANYTIM2UTC (STRING (lnb (pos:pos+18))))


;
; Set pos to point to where we expect the station number to appear in the
; record
;

	   pos = sep (5) + 2

;
; Make sure that the two characters after the D in the station name field are
; actually numbers.  If they are not numbers then we can't use this record.
;

	   IF dgt (pos) EQ 0 OR dgt (pos+1) EQ 0 THEN GOTO, LOOP_CONTINUE

;
; Get the station number for the pass
;


	   READS, STRING (lnb (pos:pos+1)), station

;
; Check if this is station 27.  If it is, add another record into the 
; list of records and write the pass information to it.
;

	   IF station EQ 27 THEN BEGIN

;
; Add another record in to hold the pass information which we just extracted
;

              lst = [lst, rec]

;
; Increment the record counter to point to the new record.
;

	      nrec = nrec + 1

;
; Put the pass information into the newly created record.
;

	      lst (nrec).start_time    = time0
	      lst (nrec).end_time      = time1
              lst (nrec).pass_duration = time1 - time0

	   ENDIF

;
; OK, all done.
;

LOOP_CONTINUE:

	ENDWHILE

;
; Free the file that we read the data from.
;

	FREE_LUN, lun

;
; Get rid of the first record (dummy record) from the pass list
;

	lst = lst (1:*)

;
; Check if both tstart and tend are defined.  If they are, then these two times will mark
; the boundries of the window for which we will return D27 pass information.
;

	IF N_ELEMENTS (tstart) EQ 1 AND  N_ELEMENTS (tend) EQ 1 THEN BEGIN

;
; Convert tstart and tend to TAI format (they may be in some other format) and store in
; t0 and t1
;

	   t0 = UTC2TAI (ANYTIM2UTC (tstart))
	   t1 = UTC2TAI (ANYTIM2UTC (tend))

;
; Find all the entries in the pass list that fall within the requested
; time frame.  First check if the keyword /INCLUSIVE is set.  If it is not 
; then we will only select records that fall entirely within the designated 
; time frame.
;

	   IF NOT KEYWORD_SET (inclusive) THEN BEGIN

	      lindex =  (WHERE (lst.start_time GT t0 AND lst.end_time LT t1))

	   ENDIF ELSE BEGIN 

;
; Otherwise select any record that overlaps the designated time frame
;

	      lindex =  (WHERE (lst.end_time GT t0 AND lst.start_time LT t1))

	   ENDELSE

;
; Check that at least on entry was found in the requested time range.  
; If no entries were found, then we have to handle this as a special case.
;

	   IF lindex (0) NE -1 THEN BEGIN 

;
; Reset lst to consist only of the requested entries.
;

	      lst = lst (lindex)

;
; Reset nrec to reflect the number of records which are now in our pass list
; 

	      nrec = N_ELEMENTS (lst)

;
; Check if the the keyword /TRUNCATE is set.  If it is then we will have to
; adjust the start and ending times of any records that overlap the 
; designated time period so that they begin or end at the edges of the time
; frame.
;

	      IF KEYWORD_SET (truncate) THEN BEGIN

; 
; First, find any records that have start times earlier then the requested 
; start time.  Adjust those records so that their start times are the same as
; the requested start time.  
;

	         ind = WHERE (lst.start_time LT t0)

	         IF ind (0) NE -1 THEN lst (ind).start_time = t0

; 
; Now, find any records that have end times later then the requested 
; end time.  Adjust those records so that their end times are the same as
; the requested end time.  
;

	         ind = WHERE (lst.end_time GT t1)

	         IF ind (0) NE -1 THEN lst (ind).end_time = t1

	      ENDIF 

	   ENDIF ELSE BEGIN

;
; Otherwise, set lst to be just a single record which consists only of the
; resource name (DSN_27).  Also, set nrec to be 0.
;

	      rec = {RES_NAME:'DSN_27'}
              lst = REPLICATE (rec, 1)

	      nrec = 0

	   ENDELSE

	ENDIF

;
; Tell the user we are finished reading the merged DSN file.
;

;	PRINT, "Finished reading merged DSN file."
	PRINT, "Found ", nrec, " passes using DSN-27"

;
; Return the entire array of records.  Were done.
;

	RETURN, lst

	END


