;+
; Project     :	SOHO - CDS
;
; Name        :	ADD_ANOMALY()
;
; Purpose     :	Adds a SOHO anomaly to the database
;
; Explanation :	
;
; Syntax      :	Result = ADD_ANOMALY( DEF )
;
;		IF NOT ADD_ANOMALY( DEF ) THEN ...
;
; Inputs      :	DEF = This is an anonymous structure containing
;		      tags defined by DEF_ANOMALY
;
; Opt. Inputs :	None.
;
; Outputs     :	The result of the function is a logical value representing
;		whether or not the operation was successful, where 1 is
;		successful and 0 is unsuccessful.
;
; Opt. Outputs:	None.
;
; Keywords    :
;       ERRMSG    = If defined and passed, then any error messages will be
;                   returned to the user in this parameter rather than
;                   depending on the MESSAGE routine in IDL.  If no errors are
;                   encountered, then a null string is returned.  In order to
;                   use this feature, ERRMSG must be defined first, e.g.
;
;                       ERRMSG = ''
;                       Result = ADD_ANOMALY( ERRMSG=ERRMSG, ... )
;                       IF ERRMSG NE '' THEN ...
;
;
; Common      :	None.
;
; Restrictions:	!PRIV must be 2 or greater to use this routine.
;
; Side effects:	If input ANOMALY ID is already in DB, then input
;               anomaly will replace current entry, otherwise it
;               will be added with a new ID.
;
; Category    :	Planning, Databases.
;
; Prev. Hist. :	None.
;
; Written     :	Dominic Zarro, ARC/GSFC, 20 January 1996
;
; Version     :	Version 1
;-
;

	function add_anomaly, def, errmsg=errmsg

	on_error, 1
;
;  Initialize result to represent non-success.  If the routine is successful,
;  this value will be updated below.
;
	result = 0 

;
;  Check the input parameters
;
        if datatype(def) ne 'STC' then begin
         message = 'SYNTAX:  RESULT = ADD_ANOMALY(DEF)'
         goto,handle_error
        endif

;
;  Ensure that the user has privilege to write into the database.
;
        !priv=3
	if !priv lt 2 then begin $
         message = '!PRIV must be 2 or greater to write into the Database'
         goto,handle_error
 	endif

        stat=test_open(concat_dir(getenv('SOHO_ANOMALY'),'anomaly.dbf'),/write)
        if not stat then begin
         message='Privilege to update Anomaly DB is denied'
         goto,handle_error
        endif

;
; Check if identical entry already in the database
; If input ID gt 0 and different entry found in DB then replace it
;
        err='' & replacing=0
        get_anomaly,def.id,db_def,err=err
        if err eq '' then begin
         if match_struct(def,db_def) then begin
          message = 'Identical Anomaly entry already in Database'
          goto,handle_error
         endif
         replacing=1
        endif
;
;  Check each of the structure components 
;

        if (def.time le 0.) then begin
         message='Must enter valid time'
         goto, handle_error
        endif
        def.time=double(string(def.time, format='(f15.3)'))
 
        def.system=strmid(strupcase(strtrim(def.system,2)),0,10)
        systems=strtrim(['ECS','FOT',get_soho_inst()],2)
        slook=where(def.system eq systems,cnt)
        if cnt eq 0 then begin
         message='Must enter valid system name'
         goto, handle_error
        endif

        def.origin=strmid(strtrim(def.origin,2),0,20)
        if def.origin eq '' then begin
         message='Must enter Anomaly originator'
         goto, handle_error
        endif

        nx=400
        anomaly=strmid(strtrim(def.anomaly,2),0,nx)
        if anomaly eq '' then begin
         message='Must enter Anomaly description'
         goto,handle_error
        endif
        impact=strmid(strtrim(def.impact,2),0,nx)
        resolution=strmid(strtrim(def.resolution,2),0,nx)
        action=strmid(strtrim(def.action,2),0,nx)

;
;  Open the database for write access.
;
	dbopen, 'anomaly', 1
;
;  Find the largest anomaly ID currently in the database, and add one
;  to it or, if replacing, delete the old entry and give old ID to new entry
;
	n_entries = db_info('entries','anomaly')
	if n_entries eq 0 then anomaly_id = 1L else begin
         if replacing then begin
          anomaly_id=def.id 
          entries = dbfind('id='+strtrim(long(anomaly_id),2)+',deleted=n',/silent)
          if !err ne 0 then begin
           for i=0,n_elements(entries)-1 do dbupdate, entries(i), 'deleted', 'y'
          endif else begin
           message='Could not replace old entry'
           goto,handle_error
          endelse
         endif else begin
          dbext, -1, 'id', anomaly_ids
	  anomaly_id = max(anomaly_ids) + 1L
         endelse
	endelse
;
;  Add the anomaly to the database.
;
	dbbuild,anomaly_id,def.time,def.system,def.origin,$
         anomaly,action,impact,resolution,def.open,'n',status=status
	if status eq 0 then begin
         message = 'Write to Anomaly database was unsuccessful'
         goto, handle_error
	endif
;
;  Update the id number in the structure.  signal success.
;
	def.id = anomaly_id
	result = 1
        goto,finish

;
;  Error handling point.
;

handle_error:
       
	if n_elements(errmsg) ne 0 then errmsg = 'GET_ANOMALY: ' + message $
		else message, message, /continue

;
;  Close the database, and return whether the routine was successful or not.
;

finish:
	dbclose
;
	return, result
	end
