;+
; Project     :	SOHO - CDS
;
; Name        :	DEL_ANOMALY()
;
; Purpose     :	Deletes a SoHO anomaly from the DB
;
; Explanation :
;
; Use         :	S=DEL_ANOMALY(ID)
;
; Inputs      :	ID = ID number to delete
;
; Opt. Inputs :	None.
;
; Outputs     :	The result of the function is a logical value representing
;		whether or not the operation was successful, where 1 is
;		successful and 0 is unsuccessful.
;
; Opt. Outputs:	None.
;
; Keywords    :	
;
;		ERRMSG	   = If defined and passed, then any error messages
;			     will be returned to the user in this parameter
;			     rather than depending on the MESSAGE routine in
;			     IDL.  If no errors are encountered, then a null
;			     string is returned.  In order to use this feature,
;			     ERRMSG must be defined first, e.g.
;
;				ERRMSG = ''
;				Result = DEL_ANOMALY( ERRMSG=ERRMSG, ... )
;				IF ERRMSG NE '' THEN ...
;
;
; Common      :	None.
;
; Restrictions:	None.
;
; Side effects:	Marks records as deleted.  PRG_ANOMALY must be called to really
;		delete the records.
;
; Category    :	Planning, Databases.
;
; Written     :	Dominic Zarro (ARC/GSFC) 21 January 1996
;
; Version     :	Version 1
;-
	function del_anomaly, id, errmsg=errmsg

	on_error, 1
;
;  Initialize RESULT to represent non-success.  If the routine is successful,
;  this value will be updated below.
;
	result = 0
;
;  Check the number of parameters.
;
        if (not exist(id)) then begin
           message = 'SYNTAX:  S=DEL_ANOMALY(ID)'
	   goto, handle_error
        endif
;
;  Make sure that the user has privilege to write into the database.
;
	if !priv lt 2 then begin $
           message = '!PRIV must be 2 or greater to write into the database'
           goto, handle_error
	endif
;
;  Open the database for update.
;
	dbopen, 'anomaly', 1
;
;  Search for the anomaly to delete
;

	entries = dbfind('id='+strtrim(long(id),2)+',deleted=n',/silent)

;
;  If no entries were found, then return immediately.
;
	if !err eq 0 then begin
           message = 'No such entry in the database'
           goto, handle_error
	endif
;
;  Mark the entry as deleted.
;
	for i=0,n_elements(entries)-1 do dbupdate, entries(i), 'deleted', 'y'
;
;  Signal success.
;
	result = 1
	goto, finish
;
;  Error handling point.
;

handle_error:
	if n_elements(errmsg) ne 0 then errmsg = 'DEL_ANOMALY: ' + message	$
		else message, message, /continue

;
;  Close the database, and return.
;
finish:
	dbclose
;
	return, result
	end
