;+
; Project     : SOHO, YOHKOH
;
; Name        : GET_FITS_ROLL
;
; Purpose     : Get image roll
;
; Category    : imaging, FITS
;
; Syntax      : roll=get_fits_roll(stc)
;
; Inputs      : struct - FITS-like structure
;
; Outputs     : ROLL = roll (degrees clockwise positive)
;
; Keywords    : NON_STANDARD_FITS = set to use non-standard roll
;               keywords if FITS keyword-values are zero or missing.

;
; History     : 29 Sept 2008, Zarro (ADNET) - written
;               19 Feb 2009, Zarro (ADNET) - added check for SOLAR_P0 
;               26 March 2009, Zarro (ADNET) - changed to function
;               9 July 2009, Zarro (ADNET) 
;                 - added /NON_STANDARD_FITS to search for cases when 
;                  standard FITS keyword/values are missing or zero.
;
; Contact     : dzarro@solar.stanford.edu
;-

function get_fits_roll,stc,verbose=verbose,$
                  non_standard_fits=non_standard_fits,_extra=extra


if ~is_struct(stc) then return,0.
nimg=n_elements(stc)
roll=0.
standard_fits=~keyword_set(non_standard_fits)

if nimg gt 1 then roll=replicate(roll,nimg)

;-- Next check all the possible CROTx possibilities.
;   If one is non-zero then use it.

choices=['crot','crota','crota1','crota2']
nchoices=n_elements(choices)                                                        
for i=0,nchoices-1 do begin
 if have_tag(stc,choices[i],pindex,/exact) then begin
  chk=where(stc.(pindex) ne 0.,count)
  if (count gt 0) or standard_fits then begin
   if keyword_set(verbose) then message,'using '+choices[i]+' for roll value',/cont
   roll=stc.(pindex)                  
   return,roll
  endif
 endif
endfor

;-- Next check the non-standard ones. If a roll value was not found in
;   the previous CROTx locations, use the first non-zero roll found
;   here.

choices=['sc_roll','p_angle','solar_p0','angle']
nchoices=n_elements(choices)                                                        
for i=0,nchoices-1 do begin
 if have_tag(stc,choices[i],pindex,/exact) then begin
  chk=where(stc.(pindex) ne 0.,count)
  if count gt 0 then begin
   if keyword_set(verbose) then message,'using '+choices[i]+' for roll value',/cont
   roll=-stc.(pindex)                  
   return,roll
  endif
 endif
endfor
                       
return,roll & end
