;+
; Project     : VSO
;
; Name        : VSO_PREP
;
; Purpose     : Wrapper around VSO PREP object
;
; Category    : utility sockets
;
; Example     : IDL> vso_prep,file
;
; Inputs      : FILE = input file to process (with optional URL path)
;
; Keywords    : STATUS= 1/0 for success or failure
;               OUTFILE = prepped output filename
;               INSTRUMENT = optional instrument [in case can't get from FILE]
;               ERR = error messages
;               OPREP = object with prepped data
;               IMAGE_NO = optional image nos to process [for multiple records]
;
; History     : Written 31-March-2009, D.M. Zarro (ADNET)
;               15-Sept-2009, Zarro (ADNET) 
;               - removed INST input (made it optional keyword)
;               - added more error checking and messaging
;               17-Nov-2009, Zarro (ADNET)
;               - added check for write access to output directory
;
; Contact     : DZARRO@SOLAR.STANFORD.EDU
;-

pro vso_prep,file,outfile=outfile,_extra=extra,image_no=image_no,$
            status=status,err=err,instrument=instrument,oprep=oprep

status=0b
error=0
catch,error
if error ne 0 then begin
 catch,/cancel
 err=err_state()
 message,err,/cont
 status=0b
 if obj_valid(vsoprep) then obj_destroy,vsoprep
 if obj_valid(request) then obj_destroy,request
 if obj_valid(otemp) then obj_destroy,otemp
 return
endif

if is_blank(file) and is_blank(instrument) then begin
 pr_syntax,'vso_prep,file [,outfile=outfile]'
 return
endif

;-- determine instrument

if is_string(file) and is_blank(instrument) then begin
 inst=get_fits_det(file,prepped=prepped)
 if is_blank(inst) and is_string(instrument) then inst=strupcase(instrument)

 if is_blank(inst) then begin
  err='Could not determine instrument.'
  message,err,/cont
  return
 endif

 if prepped then begin
  message,file+' already prepped.',/cont
  return
 endif
endif else begin
 if is_blank(inst) and is_string(instrument) then inst=strupcase(instrument)
endelse

vso_prep_list=['EIT','TRACE','EUVI','XRT','XRT','RHESSI']
chk=where(inst eq vso_prep_list,count)
if count eq 0 then begin
 err='Prepping not currently supported for '+inst+' data.'
 message,err,/cont
 return
endif

;-- default output prepped file to current directory

ofile='prepped_temp.fits'
if is_string(file) then begin
 odir=file_dirname(file)
 ofile='prepped_'+file_basename(file)
endif

if is_string(outfile) then begin
 odir=file_dirname(outfile)
 ofile=file_basename(outfile)
endif
if is_blank(odir) or odir eq '.' then odir=curdir()
if ~test_dir(odir,_extra=extra,/verb) then return

ofile=concat_dir(odir,ofile)
outfile=ofile

file_delete,ofile,/quiet

;-- load Java files

vso_startup, status=status
if ~status then begin
 err='Problem loading IDL-Java bridge.'
 message,err,/cont
 return
endif

message,'Prepping '+inst+' data...',/cont

;-- check if preselecting image records

if strupcase(inst) eq 'XRT' then inst='XRT2'
otemp=obj_new(inst)
if ~exist(image_no) then begin
 if have_method(otemp,'preselect') then begin
  otemp->preselect,file,image_no,cancel=cancel
  if cancel then begin
   message,'Prepping cancelled',/cont
   obj_destroy,otemp
   return
  endif
 endif
endif

;-- create new VSO PREP object

status=0b
vsoprep = OBJ_NEW('IDLJavaObject$STATIC$GOV_NASA_GSFC_JIDL_VPS_CLIENT_PREPROCESSORCLIENTREQUEST', 'gov.nasa.gsfc.jidl.vps.client.PreprocessorClientRequest')
if ~obj_valid(vsoprep) then begin
 err='Error creating VSO PREP object.'
 message,err,/cont
 return
endif

;-- create request object

request=vsoprep->newrequest(inst)
if is_string(file) then request->addData,file
if exist(image_no) then begin
 request->addparameter,'image_no',image_no
endif

;-- pass command line prep keywords

if is_struct(extra) then begin
 names=tag_names(extra)
 for i=0,n_elements(names)-1 do request->addparameter,names[i],extra.(i)
endif

;-- send to prep server

request->preprocess

;-- download prepped file

result=request->getData()
if is_string(result) then begin
 sock_copy,result,ofile,_extra=extra
 chk=file_search(ofile,count=count)
endif else count=0

if count gt 0 then begin
 message,'Prepping completed successfully.',/cont 
 message,'Wrote prepped data to - '+ofile,/cont
 status=1b 

;-- return prepped object

 if arg_present(oprep) then begin
  if obj_valid(oprep) then obj_destroy,oprep
  oprep=otemp
  oprep->read,ofile
 endif else obj_destroy,otemp
endif else begin
 err='Prepping failed.'
 message,err,/cont
 obj_destroy,otemp
endelse

;-- cleanup

obj_destroy,vsoprep
obj_destroy,request


return & end
