;+
; Project     : HESSI
;
; Name        : FITS__DEFINE
;
; Purpose     : Define a FITS map object
;
; Category    : imaging maps objects
;               
; Syntax      : This procedure is invoked when a new FITS object is
;               created via:
;
;               IDL> new=obj_new('fits')
;
; History     : Written 19 May 1998, D. Zarro, SM&A/GSFC
;               10 Oct 2009, Zarro (ADNET) 
;                - added capability to read multiple FITS file
;                  extensions
;                - retired mreadfits method 
;
; Contact     : dzarro@solar.stanford.edu
;-

;-- FITS init

function fits::init,_ref_extra=extra

dprint,'% FITS::INIT'

return,self->map::init(_extra=extra)

end

;--------------------------------------------------------------------------

pro fits::cleanup

dprint,'% FITS::CLEANUP'

self->map::cleanup

return & end

;---------------------------------------------------------------------------
;-- FITS reader

pro fits::read,file,data,index=index,err=err,_ref_extra=extra,all=all,$
                      extension=extension,nodata=nodata
err=''

nfiles=n_elements(file)
if is_blank(file) or (nfiles eq 0) then begin
 err='Missing input FITS filename'
 message,err,/cont
 return
endif
file=strtrim(file,2)

;-- avoid making duplicate copies of data if it is not an argument

no_copy=n_params() eq 1
nodata=keyword_set(nodata)

;-- if extension not specified, default to first

do_all=keyword_set(all)
if exist(extension) then begin
 if is_number(extension[0]) then begin
  chk=where(extension ge 0,count)
  if count gt 0 then extension=extension[chk] else extension=0
  extension=get_uniq(extension)
 endif else extension=0
endif else extension=0
n_ext=n_elements(extension)

;-- empty linkedlist

self->empty
k=-1
for i=0,nfiles-1 do begin
 terr='' & k=-1 & m=-1

 if do_all then begin
  n_ext=get_fits_extn(file[i])
  if n_ext eq 0 then begin
   message,'Skipping '+file[i],/cont
   continue
  endif else extension=indgen(n_ext)
  if n_ext gt 1 then message,'Reading '+trim(n_ext)+' extensions.',/cont
 endif

 repeat begin
  k=k+1 
  rext=extension[k]
  self->readfits,file[i],data,index=index,_extra=extra,err=terr,$
                          extension=rext,nodata=nodata
  if ~nodata and (is_string(terr) or (size(data,/n_dim) le 1)) then begin
   if is_blank(terr) then message,'FITS data not simple image.',/cont
   continue
  endif
  tindex=merge_struct(tindex,index)
  if nodata then continue
  nindex=n_elements(index)
  m=m+1
  for j=0,nindex-1 do begin
   if nindex gt 1 then begin
    self->mk_map,index[j],data[*,*,j],m,no_copy=no_copy,err=merr,$
               filename=file[i],_extra=extra,/angles
   endif else begin
    self->mk_map,index[j],data,m,no_copy=no_copy,err=merr,$
               filename=file_basename(file[i]),_extra=extra,/angles
   endelse
  endfor
 endrep until ((k+1) ge n_ext)
endfor
err=terr

if is_struct(tindex) then index=temporary(tindex)

return & end

;--------------------------------------------------------------------------
;-- wrapper around MRDFITS that makes HEADER and EXTENSION into keywords

pro fits::mrdfits,file,data,header=header,extension=extension,$
                    verbose=verbose,_ref_extra=extra

forward_function mrdfits
if ~exist(extension) then extension=0
if keyword_set(verbose) then begin
 message,'Reading file - '+file,/cont
 message,'Reading extensions - '+trim(arr2str(extension)),/cont
endif
data=mrdfits(file,extension,header,_extra=extra,/fscale)

return & end

;-------------------------------------------------------------------------
;-- READFITS method

pro fits::readfits,file,data,header=header,index=index,_ref_extra=extra,err=err,$
                      status=status,nodata=nodata

err=''
status=-1
delvarx,index,data

;-- manually decompress if MRDFITS can't do it

compressed=is_compressed(file,type)
have_mrdfits=have_proc('mrdfits')
dfile=strtrim(file[0],2)

decompress=~have_mrdfits or $
           ((type eq 'Z') and (os_family(/lower) eq 'windows')) or $
           ~since_version('5.3')

if compressed and decompress then begin
 dfile=find_compressed(file,err=err)
 if err ne '' then begin
  message,err,/cont
  return
 endif
endif

;-- just read header

if keyword_set(nodata) then begin
 mrd_head,dfile,header,status=status,/no_check_compress
 if status ge 0 then index=fitshead2struct(header)
 return
endif

;-- call MRDFITS

if have_mrdfits then begin
 self->mrdfits,dfile,data,header=header,status=status,_extra=extra
 if status ge 0 then begin
;  if is_struct(data) then data=data.(0)
  index=fitshead2struct(header)
  sz=size(data)
  if sz[0] eq 3 then index=replicate(index,sz[3])
 endif else begin
  err='End of file during read.'
  message,err,/cont
 endelse
 return
endif else mreadfits,dfile,index,data,header=header,_extra=extra

return & end

;------------------------------------------------------------------------
;-- download remote URL files to local directory

pro fits::get,file,out_dir=out_dir,_ref_extra=extra,local_file=local_file

local_file=''
if is_blank(file) then begin
 message,'No filename entered',/cont
 return
endif

local_file=file

url=stregex(file,'(http[s]?|ftp):\/\/',/fold,/bool)

;-- create temp directory for download

if url[0] then begin
 if is_blank(out_dir) then out_dir=curdir()
 if ~write_dir(out_dir,/quiet) then begin
  out_dir=concat_dir(get_temp_dir(),'fits')
  mk_dir,out_dir
 endif
endif

for i=0,n_elements(url)-1 do begin
 if url[i] then begin
  sock_copy,file[i],out_dir=out_dir,_extra=extra,copy_file=copy_file
  local_file[i]=copy_file
endif
endfor

return & end

;--------------------------------------------------------------------------                  
;-- define FITS object

pro fits__define                 

fits_struct={fits, inherits map}

return & end
