;+
; Project     : HESSI
;
; Name        : SYNOP_SPEX__DEFINE
;
; Purpose     : Define a synop_spex data object.  Allows any data type that can be read
;               in ospex, to be plotted in show_synop.  In show_synop, spex data types are
;               associated with this object.  To plot, the get_plot_obj method is called to return
;               a xyplot or utplot object, which can be passed to plotman or plotted directly.
;
; Category    : Ancillary Synoptic Objects
;
; Syntax      : IDL> c=obj_new('synop_spex')
;
; History     : Written 24-Mar-2009, Kim Tolbert, Dominic Zarro
;               Modified 26-April-2009 (Zarro/ADNET) 
;                - added check for SPEX package in path
;               Modified 1-May-2009 (Zarro/ADNET)
;                - changed SPEX from being inherited to being a helper
;                  object. This allows SYNOP_SPEX to compile if SPEX
;                  package is not installed.
;-
;-----------------------------------------------------------------------------

function synop_spex::init,_ref_extra=extra

if ~self->have_spex_xray() then begin
 message,'SPEX/XRAY packages not installed.',/cont
 return,0
endif

self.spex=obj_new('spex',_extra=extra,/no_gui)
return,obj_valid(self.spex)

end

;-----------------------------------------------------------------------------

pro synop_spex::cleanup
obj_destroy,self.spex
return
end

;-----------------------------------------------------------------------------
;-- check for SPEX path

function synop_spex::have_spex_xray

chk1=have_proc('spex__define')
chk2=have_proc('chianti_kev')
if chk1 and chk2 then return,1b
path1=local_name('$SSW/packages/spex/idl')
if ~is_dir(path1) then return,0b
add_path,path1,/expand,/quiet
path2=local_name('$SSW/packages/xray/idl')
if ~is_dir(path2) then return,0b
add_path,path2,/expand,/quiet

chk1=have_proc('spex__define')
chk2=have_proc('chianti_kev')
return,chk1 and chk2
end

;------------------------------------------------------------------------------

pro synop_spex::read,file,_ref_extra=extra


;-- download if URL

if is_blank(file) then begin
 pr_syntax,'object_name->read,filename'
 return
endif
file=strtrim(file,2)
if is_url(file) then begin
 out_dir=curdir()
 if ~write_dir(out_dir,/quiet) then out_dir=get_temp_dir()
 sock_copy,file,out_dir=out_dir,local_file=ofile,_extra=extra
endif else ofile=file
if is_blank(ofile) then return

data = self.spex->getdata(spex_specfile=ofile,_extra=extra)
return
end

;------------------------------------------------------------------------------
; get_plot_obj function returns a xyplot or utplot object containing all the data to plot and
; options for plotting.  That object can be passed directly to plotman or plotted via obj->plot

function synop_spex::get_plot_obj, spectrum=spectrum,_ref_extra=extra
if keyword_set(spectrum) then begin
  self.spex -> plot_spectrum,/tband,/allint,/no_plotman, /get_plot_obj, obj=obj,_extra=extra
endif else begin
  self.spex -> plot_time,/no_plotman, /get_plot_obj, obj=obj, _extra=extra
endelse
return, obj
end

;-----------------------------------------------------------------------------
; PLOT wrapper

pro synop_spex::plot,_ref_extra=extra
void=self->get_plot_obj(_extra=extra,get_plot_obj=0)

end

;------------------------------------------------------------------------------
;-- wrapper around SPEX GET

function synop_spex::get, filename=filename, _ref_extra=extra
if keyword_set(filename) then return,self.spex->get(/spex_specfile)
return,self.spex->get(_extra=extra)
end

;------------------------------------------------------------------------------
;-- wrapper around SPEX SET

pro synop_spex::set,_ref_extra=extra
self.spex->set,_extra=extra
return
end

;------------------------------------------------------------------------------
; has_data function returns 1 if there is data in the spec object to
; return.

function synop_spex::has_data
return,is_struct(self.spex->getdata())
end

;----------------------------------------------------------------------------
;-- search files (currently only SOXS)

function synop_spex::search,tstart,tend,_ref_extra=extra,soxs=soxs,$
                 hsi=hsi,xrs=xrs

return,soxs_files(tstart,tend,_extra=extra)
end

;------------------------------------------------------------------------------

pro synop_spex__define                 
self={synop_spex, spex:obj_new()}
return & end

